=pod

=head1 NAME

Lingua::ZH::CCDICT::Data - CCDICT data file

=cut

__DATA__
# CCDICT 5.1.1 generated 2006-07-04
# Copyright � www.chineselanguage.org 1994-2006
# This file may be distributed freely only subject to the terms and conditions set forth in the the  Creative Commons Attribution 2.5 License.
# See readme.txt for a description of the content of this file and copyright information
# --------------------------------------------------------------------------------------
U+3400.0	fUTF8	㐀
U+3400.0	fCNS11643	6-222C
U+3400.0	fR/S	 1.4
U+3400.0	fTotalStrokes	 5
U+3400.0	fCangjie	TM
U+3400.0	fFourCorner	11100
U+3400.0	fHakka	  hiu1; kiu1; sieu1
U+3400.0	fCantonese	jau1
U+3400.0	fMandarin	qiu1
U+3400.0	fEnglish	same as &#x4E18;
U+3401.0	fUTF8	㐁
U+3401.0	fCNS11643	4-2224
U+3401.0	fGB	5-3024
U+3401.0	fR/S	 1.5
U+3401.0	fTotalStrokes	 6
U+3401.0	fCangjie	MOW
U+3401.0	fFourCorner	10600
U+3401.0	fHakka	  tiam3; le2
U+3401.0	fCantonese	dou6; tim2
U+3401.0	fMandarin	tian3; tian4
U+3401.0	fEnglish	 lick
U+3402.0	fUTF8	㐂
U+3402.0	fR/S	 1.5
U+3402.0	fTotalStrokes	 6
U+3402.0	fCangjie	PPP
U+3402.0	fCantonese	cat1
U+3402.0	fEnglish	[Japanese character]
U+3403.0	fUTF8	㐃
U+3403.0	fR/S	 2.2
U+3403.0	fTotalStrokes	 3
U+3403.0	fCangjie	OML
U+3403.0	fCantonese	zim1
U+3404.0	fUTF8	㐄
U+3404.0	fCNS11643	6-2130
U+3404.0	fR/S	 2.2
U+3404.0	fTotalStrokes	 3
U+3404.0	fCangjie	JV
U+3404.0	fFourCorner	50000
U+3404.0	fHakka	 ngiu2; ngeu2
U+3404.0	fCantonese	kwaa1; ngau4; au4
U+3404.0	fMandarin	kua3; niu2; kua4
U+3405.0	fUTF8	㐅
U+3405.0	fCNS11643	6-2123
U+3405.0	fR/S	 4.1
U+3405.0	fTotalStrokes	 2
U+3405.0	fCangjie	K
U+3405.0	fFourCorner	40000
U+3405.0	fHakka	 ng3
U+3405.0	fCantonese	ng5
U+3405.0	fMandarin	wu3
U+3405.0	fEnglish	ancient form of &#x4E94;
U+3406.0	fUTF8	㐆
U+3406.0	fCNS11643	F-216C
U+3406.0	fGB	5-3076
U+3406.0	fR/S	 4.5
U+3406.0	fTotalStrokes	 6
U+3406.0	fCangjie	HSMS
U+3406.0	fFourCorner	2642
U+3406.0	fCantonese	ji1; jaan2; zaan2
U+3406.0	fMandarin	yin3
U+3406.0	fEnglish	follow; trust
U+3407.0	fUTF8	㐇
U+3407.0	fR/S	 5.2
U+3407.0	fTotalStrokes	 3
U+3407.0	fCangjie	KNN
U+3407.0	fCantonese	gau2
U+3408.0	fUTF8	㐈
U+3408.0	fR/S	 5.2
U+3408.0	fTotalStrokes	 3
U+3408.0	fCangjie	ON
U+3408.0	fCantonese	jap6
U+3409.0	fUTF8	㐉
U+3409.0	fR/S	 5.2
U+3409.0	fTotalStrokes	 3
U+3409.0	fCangjie	MNN
U+3409.0	fCantonese	ding1
U+340A.0	fUTF8	㐊
U+340A.0	fR/S	 5.3
U+340A.0	fTotalStrokes	 4
U+340A.0	fCangjie	GN
U+340A.0	fCantonese	si6
U+340B.0	fUTF8	㐋
U+340B.0	fR/S	 5.3
U+340B.0	fTotalStrokes	 4
U+340B.0	fCangjie	GN
U+340B.0	fCantonese	tou2
U+340C.0	fUTF8	㐌
U+340C.0	fCNS11643	4-2157
U+340C.0	fGB	3-302B
U+340C.0	fR/S	 5.4
U+340C.0	fTotalStrokes	 5
U+340C.0	fCangjie	OPD
U+340C.0	fFourCorner	80710
U+340C.0	fHakka	 ji2; ri2
U+340C.0	fCantonese	ji4; zyu4
U+340C.0	fMandarin	yi4; si4; yi2
U+340C.0	fEnglish	a savage tribe in Kwangtung
U+340D.0	fUTF8	㐍
U+340D.0	fR/S	 5.4
U+340D.0	fTotalStrokes	 5
U+340D.0	fCangjie	BN
U+340D.0	fCantonese	zaau2
U+340E.0	fUTF8	㐎
U+340E.0	fR/S	 5.4
U+340E.0	fTotalStrokes	 5
U+340E.0	fCangjie	YKN
U+340E.0	fCantonese	man4
U+340F.0	fUTF8	㐏
U+340F.0	fR/S	 5.4
U+340F.0	fTotalStrokes	 5
U+340F.0	fCangjie	MMN
U+340F.0	fCantonese	ng5
U+3410.0	fUTF8	㐐
U+3410.0	fR/S	 5.5
U+3410.0	fTotalStrokes	 6
U+3410.0	fCangjie	VEN
U+3410.0	fCantonese	nou4; lou4 (also)
U+3411.0	fUTF8	㐑
U+3411.0	fR/S	 5.5
U+3411.0	fTotalStrokes	 6
U+3411.0	fCangjie	HEYN
U+3411.0	fCantonese	dung1
U+3412.0	fUTF8	㐒
U+3412.0	fR/S	 5.5
U+3412.0	fTotalStrokes	 6
U+3412.0	fCangjie	SRN
U+3412.0	fCantonese	ziu6
U+3413.0	fUTF8	㐓
U+3413.0	fR/S	 5.5
U+3413.0	fTotalStrokes	 6
U+3413.0	fCangjie	MNRN
U+3413.0	fCantonese	ho2
U+3414.0	fUTF8	㐔
U+3414.0	fR/S	 5.5
U+3414.0	fTotalStrokes	 6
U+3414.0	fCangjie	YRN
U+3414.0	fCantonese	hang1
U+3415.0	fUTF8	㐕
U+3415.0	fR/S	 5.5
U+3415.0	fTotalStrokes	 6
U+3415.0	fCangjie	LWN
U+3415.0	fCantonese	jau4
U+3416.0	fUTF8	㐖
U+3416.0	fCNS11643	4-2336
U+3416.0	fGB	3-3032
U+3416.0	fR/S	 5.6
U+3416.0	fTotalStrokes	 7
U+3416.0	fCangjie	GRU
U+3416.0	fFourCorner	42610
U+3416.0	fCantonese	kit3; zip6
U+3416.0	fMandarin	ye4
U+3416.0	fEnglish	a translated name of India, used in ancient time
U+3417.0	fUTF8	㐗
U+3417.0	fR/S	 5.6
U+3417.0	fTotalStrokes	 7
U+3417.0	fCangjie	JPN
U+3417.0	fCantonese	lou5
U+3418.0	fUTF8	㐘
U+3418.0	fR/S	 5.6
U+3418.0	fTotalStrokes	 7
U+3418.0	fCangjie	FDN
U+3418.0	fCantonese	mai5
U+3419.0	fUTF8	㐙
U+3419.0	fR/S	 5.7
U+3419.0	fTotalStrokes	 8
U+3419.0	fCangjie	MRTN
U+3419.0	fCantonese	dau6
U+341A.0	fUTF8	㐚
U+341A.0	fR/S	 5.7
U+341A.0	fTotalStrokes	 8
U+341A.0	fCangjie	MMRN
U+341A.0	fCantonese	ng4
U+341B.0	fUTF8	㐛
U+341B.0	fR/S	 5.7
U+341B.0	fTotalStrokes	 8
U+341B.0	fCangjie	ORN
U+341B.0	fCantonese	jau6
U+341C.0	fUTF8	㐜
U+341C.0	fCNS11643	4-2835
U+341C.0	fGB	3-3024
U+341C.0	fR/S	 5.8
U+341C.0	fTotalStrokes	 9
U+341C.0	fCangjie	IEKN
U+341C.0	fFourCorner	44110
U+341C.0	fCantonese	sau4; caau4
U+341C.0	fMandarin	qiu2; chou2
U+341C.0	fEnglish	same as &#x4EC7;
U+341D.0	fUTF8	㐝
U+341D.0	fR/S	 5.8
U+341D.0	fTotalStrokes	 9
U+341D.0	fCangjie	QRN
U+341D.0	fCantonese	keoi1
U+341E.0	fUTF8	㐞
U+341E.0	fR/S	 5.8
U+341E.0	fTotalStrokes	 9
U+341E.0	fCangjie	TCN
U+341E.0	fCantonese	kei4
U+341F.0	fUTF8	㐟
U+341F.0	fR/S	 5.8
U+341F.0	fTotalStrokes	 9
U+341F.0	fCangjie	LYN
U+341F.0	fCantonese	fei1
U+3420.0	fUTF8	㐠
U+3420.0	fR/S	 5.8
U+3420.0	fTotalStrokes	 9
U+3420.0	fCangjie	BUN
U+3420.0	fCantonese	jyu5
U+3421.0	fUTF8	㐡
U+3421.0	fCNS11643	3-343B
U+3421.0	fR/S	 5.10
U+3421.0	fTotalStrokes	 11
U+3421.0	fCangjie	KNMBK
U+3421.0	fFourCorner	41010
U+3421.0	fHakka	 no5
U+3421.0	fCantonese	no6; lo6 (also)
U+3421.0	fMandarin	nuo4; nuan4; ru2; ruan3
U+3421.0	fEnglish	same as &#x61E6;
U+3422.0	fUTF8	㐢
U+3422.0	fR/S	 5.10
U+3422.0	fTotalStrokes	 11
U+3422.0	fCangjie	EDN
U+3422.0	fCantonese	fau4
U+3423.0	fUTF8	㐣
U+3423.0	fR/S	 5.10
U+3423.0	fTotalStrokes	 11
U+3423.0	fCangjie	IJJN
U+3423.0	fCantonese	fu3
U+3424.0	fUTF8	㐤
U+3424.0	fCNS11643	3-396D
U+3424.0	fR/S	 5.11
U+3424.0	fTotalStrokes	 12
U+3424.0	fCangjie	KNJBC
U+3424.0	fFourCorner	44010
U+3424.0	fCantonese	daan1; kaau4
U+3424.0	fMandarin	dan1; qiu2
U+3424.0	fEnglish	 implore; beseech; seek after, beg; pray
U+3425.0	fUTF8	㐥
U+3425.0	fR/S	 5.15
U+3425.0	fTotalStrokes	 16
U+3425.0	fCangjie	CSN
U+3425.0	fCantonese	co4
U+3426.0	fUTF8	㐦
U+3426.0	fR/S	 5.18
U+3426.0	fTotalStrokes	 19
U+3426.0	fCangjie	HQN
U+3426.0	fCantonese	geoi2
U+3427.0	fUTF8	㐧
U+3427.0	fR/S	 6.3
U+3427.0	fTotalStrokes	 4
U+3427.0	fCangjie	INNH
U+3427.0	fCantonese	coi4
U+3427.0	fEnglish	non-classical form of &#x7B2C;
U+3428.0	fUTF8	㐨
U+3428.0	fCNS11643	3-2741
U+3428.0	fGB	5-3044
U+3428.0	fR/S	 6.7
U+3428.0	fTotalStrokes	 8
U+3428.0	fCangjie	NNNIN
U+3428.0	fFourCorner	17220
U+3428.0	fCantonese	zeoi6
U+3428.0	fMandarin	xu4
U+3428.0	fEnglish	a kind of fish in a legend (a record in old books)
U+3429.0	fUTF8	㐩
U+3429.0	fCNS11643	3-286C
U+3429.0	fR/S	 7.6; 55.5
U+3429.0	fTotalStrokes	 8
U+3429.0	fCangjie	TTTT
U+3429.0	fFourCorner	55000
U+3429.0	fCantonese	jing4; zing4
U+3429.0	fMandarin	xing2
U+3429.0	fEnglish	wine cups
U+342A.0	fUTF8	㐪
U+342A.0	fR/S	 8.4
U+342A.0	fTotalStrokes	 6
U+342A.0	fCangjie	YOU
U+342A.0	fCantonese	si2
U+342A.0	fEnglish	[Japanese character]
U+342B.0	fUTF8	㐫
U+342B.0	fCNS11643	3-2323
U+342B.0	fR/S	 8.4
U+342B.0	fTotalStrokes	 6
U+342B.0	fCangjie	YUK
U+342B.0	fFourCorner	00770
U+342B.0	fHakka	 hiung1
U+342B.0	fCantonese	hung1; gun3; zung1
U+342B.0	fMandarin	xiong1; xiong3
U+342B.0	fEnglish	same as &#x51F6;
U+342C.0	fUTF8	㐬
U+342C.0	fCNS11643	4-2337
U+342C.0	fGB	5-334D
U+342C.0	fR/S	 8.5; 99.5
U+342C.0	fTotalStrokes	 7
U+342C.0	fCangjie	YIHU
U+342C.0	fFourCorner	00210
U+342C.0	fHakka	 liu2; fong1
U+342C.0	fCantonese	lau4
U+342C.0	fMandarin	liu2
U+342C.0	fEnglish	a cap with pendants
U+342D.0	fUTF8	㐭
U+342D.0	fCNS11643	4-2534
U+342D.0	fR/S	 8.6
U+342D.0	fTotalStrokes	 8; 7
U+342D.0	fCangjie	YWR
U+342D.0	fFourCorner	00600
U+342D.0	fCantonese	lam5
U+342D.0	fMandarin	lin3
U+342D.0	fEnglish	same as &#x5EE9;
U+342E.0	fUTF8	㐮
U+342E.0	fCNS11643	3-4034
U+342E.0	fR/S	 8.11
U+342E.0	fTotalStrokes	 13
U+342E.0	fCangjie	YCTTV
U+342E.0	fFourCorner	00730
U+342E.0	fCantonese	soeng1
U+342E.0	fMandarin	xiang1
U+342E.0	fEnglish	same as &#x8944;
U+342F.0	fUTF8	㐯
U+342F.0	fR/S	 8.15
U+342F.0	fTotalStrokes	 17
U+342F.0	fCangjie	YRRA
U+342F.0	fFourCorner	0060
U+342F.0	fCantonese	jung4
U+342F.0	fMandarin	yong1
U+342F.0	fEnglish	ancient form of &#x5EB8;
U+3430.0	fUTF8	㐰
U+3430.0	fCNS11643	4-2159
U+3430.0	fR/S	 9.3
U+3430.0	fTotalStrokes	 5
U+3430.0	fCangjie	OR
U+3430.0	fFourCorner	26200
U+3430.0	fHakka	 sin5
U+3430.0	fCantonese	seon3; san1
U+3430.0	fMandarin	xin4; shen1
U+3430.0	fEnglish	ancient form of &#x4FE1;
U+3431.0	fUTF8	㐱
U+3431.0	fCNS11643	3-2175
U+3431.0	fGB	5-313D
U+3431.0	fR/S	 9.3
U+3431.0	fTotalStrokes	 5
U+3431.0	fCangjie	OHHH
U+3431.0	fFourCorner	80200
U+3431.0	fHakka	 zhin1; zhin3; zihn1; zihn3
U+3431.0	fCantonese	can2; caa5; caan2; dou3; zaan2
U+3431.0	fMandarin	zhen3
U+3431.0	fEnglish	same as &#x9B12;
U+3432.0	fUTF8	㐲
U+3432.0	fCNS11643	3-216E
U+3432.0	fR/S	 9.3
U+3432.0	fTotalStrokes	 5
U+3432.0	fCangjie	OK
U+3432.0	fFourCorner	24230
U+3432.0	fCantonese	daai6
U+3432.0	fMandarin	da4; dai4
U+3432.0	fEnglish	the name of an island
U+3433.0	fUTF8	㐳
U+3433.0	fCNS11643	3-2171
U+3433.0	fR/S	 9.3
U+3433.0	fTotalStrokes	 5
U+3433.0	fCangjie	OMU
U+3433.0	fFourCorner	21210
U+3433.0	fHakka	  ngut7
U+3433.0	fCantonese	ngat6; at6 (also); ngaat6
U+3433.0	fMandarin	wu4
U+3433.0	fEnglish	uncomfortable
U+3434.0	fUTF8	㐴
U+3434.0	fCNS11643	3-2173
U+3434.0	fR/S	 9.3
U+3434.0	fTotalStrokes	 5
U+3434.0	fCangjie	ONI
U+3434.0	fFourCorner	27220
U+3434.0	fHakka	 ban1
U+3434.0	fCantonese	paan1
U+3434.0	fMandarin	pan1
U+3434.0	fEnglish	non-classical form of &#x6500;
U+3435.0	fUTF8	㐵
U+3435.0	fGB	S-2269
U+3435.0	fR/S	 9.3
U+3435.0	fTotalStrokes	 5
U+3435.0	fCangjie	OMD
U+3435.0	fCantonese	jyu4
U+3436.0	fUTF8	㐶
U+3436.0	fCNS11643	F-2144
U+3436.0	fR/S	 9.3
U+3436.0	fTotalStrokes	 5
U+3436.0	fCangjie	ORU
U+3436.0	fCantonese	ci5
U+3437.0	fUTF8	㐷
U+3437.0	fGB	7-2326
U+3437.0	fR/S	 9.3
U+3437.0	fTotalStrokes	 5
U+3437.0	fCangjie	ONVM
U+3437.0	fCantonese	maa6; maa3 (also)
U+3437.0	fMandarin	ma3; ma4; mian2
U+3437.0	fEnglish	[1] a kind of punishment in Han Dynasty [2] name of chessmen in Chinese chess game [3] simplified form of &#x7F75;
U+3438.0	fUTF8	㐸
U+3438.0	fCNS11643	6-234E
U+3438.0	fGB	7-2321
U+3438.0	fR/S	 9.4
U+3438.0	fTotalStrokes	 6
U+3438.0	fCangjie	ONO
U+3438.0	fFourCorner	27280
U+3438.0	fHakka	 kiam5
U+3438.0	fCantonese	him3; gim1
U+3438.0	fMandarin	qian4
U+3438.0	fEnglish	same as &#x6B20;
U+3439.0	fUTF8	㐹
U+3439.0	fCNS11643	3-2271
U+3439.0	fR/S	 9.4
U+3439.0	fTotalStrokes	 6
U+3439.0	fCangjie	OOMN
U+3439.0	fFourCorner	28210
U+3439.0	fCantonese	ngat6; at6 (also); go1; kwok3; ngaat6
U+3439.0	fMandarin	yi4
U+3439.0	fEnglish	standard form of &#x4EE1;
U+343A.0	fUTF8	㐺
U+343A.0	fCNS11643	4-2231
U+343A.0	fR/S	 9.4
U+343A.0	fTotalStrokes	 6
U+343A.0	fCangjie	OOO
U+343A.0	fFourCorner	28200
U+343A.0	fCantonese	jam4; zung3
U+343A.0	fMandarin	zhong4
U+343A.0	fEnglish	standard form of &#x773E;
U+343B.0	fUTF8	㐻
U+343B.0	fCNS11643	3-2269
U+343B.0	fGB	7-2327
U+343B.0	fR/S	 9.4
U+343B.0	fTotalStrokes	 6
U+343B.0	fCangjie	OOB
U+343B.0	fFourCorner	24220
U+343B.0	fCantonese	noi6; loi6 (also); fu1; mou5
U+343B.0	fMandarin	nei4; n3; ng3
U+343B.0	fEnglish	the name of a person
U+343C.0	fUTF8	㐼
U+343C.0	fCNS11643	3-226A
U+343C.0	fGB	5-3156
U+343C.0	fR/S	 9.4
U+343C.0	fTotalStrokes	 6
U+343C.0	fCangjie	OHT
U+343C.0	fFourCorner	24200
U+343C.0	fCantonese	cing2
U+343C.0	fMandarin	sheng1; cheng4; zheng3
U+343C.0	fEnglish	corrupt form of &#x62EF;
U+343D.0	fUTF8	㐽
U+343D.0	fGB	7-2325
U+343D.0	fR/S	 9.4
U+343D.0	fTotalStrokes	 6
U+343D.0	fCangjie	OHNK
U+343D.0	fCantonese	fung1
U+343D.0	fEnglish	simplified form of &#x5051;
U+343E.0	fUTF8	㐾
U+343E.0	fCNS11643	F-2172
U+343E.0	fGB	7-2328
U+343E.0	fR/S	 9.4
U+343E.0	fTotalStrokes	 6
U+343E.0	fCangjie	OMMU
U+343E.0	fFourCorner	2121
U+343E.0	fCantonese	wun6
U+343F.0	fUTF8	㐿
U+343F.0	fR/S	 9.4
U+343F.0	fTotalStrokes	 6
U+343F.0	fCangjie	OQ
U+343F.0	fCantonese	sau2
U+3440.0	fUTF8	㑀
U+3440.0	fR/S	 9.4
U+3440.0	fTotalStrokes	 6
U+3440.0	fCangjie	OKI
U+3440.0	fFourCorner	2423
U+3440.0	fCantonese	taai3
U+3441.0	fUTF8	㑁
U+3441.0	fCNS11643	5-2334
U+3441.0	fR/S	 9.5
U+3441.0	fTotalStrokes	 7
U+3441.0	fCangjie	OUU
U+3441.0	fFourCorner	22270
U+3441.0	fHakka	 tut7; dut7
U+3441.0	fCantonese	zeot1; zyut3; zyut6
U+3441.0	fMandarin	zhu2; zhuo1
U+3441.0	fEnglish	short; of short stature
U+3442.0	fUTF8	㑂
U+3442.0	fCNS11643	3-244A
U+3442.0	fR/S	 9.5
U+3442.0	fTotalStrokes	 7
U+3442.0	fCangjie	OMOB
U+3442.0	fFourCorner	21220
U+3442.0	fCantonese	fong2; pong4
U+3442.0	fMandarin	fang3; pang2
U+3442.0	fEnglish	same as &#x4EFF;
U+3443.0	fUTF8	㑃
U+3443.0	fCNS11643	3-2447
U+3443.0	fGB	5-316F
U+3443.0	fR/S	 9.5
U+3443.0	fTotalStrokes	 7
U+3443.0	fCangjie	OVIS
U+3443.0	fFourCorner	24220
U+3443.0	fCantonese	aau2; ngaau2 (also); au2; au3; paai1
U+3443.0	fMandarin	ao3
U+3443.0	fEnglish	same as &#x62D7;
U+3444.0	fUTF8	㑄
U+3444.0	fCNS11643	3-244D
U+3444.0	fR/S	 9.5
U+3444.0	fTotalStrokes	 7
U+3444.0	fCangjie	OWYI
U+3444.0	fFourCorner	27250
U+3444.0	fHakka	 wu3; wu1; vu1; vu3
U+3444.0	fCantonese	mou5
U+3444.0	fMandarin	wu3
U+3444.0	fEnglish	ancient form of &#x4FAE;
U+3445.0	fUTF8	㑅
U+3445.0	fCNS11643	6-2571
U+3445.0	fR/S	 9.5
U+3445.0	fTotalStrokes	 7
U+3445.0	fCangjie	OOYM
U+3445.0	fFourCorner	28210
U+3445.0	fHakka	 zok7
U+3445.0	fCantonese	zok3; zok6
U+3445.0	fMandarin	ni3; zuo4; zu3
U+3445.0	fEnglish	same as &#x4F5C;
U+3446.0	fUTF8	㑆
U+3446.0	fGB	S-2268
U+3446.0	fR/S	 9.5
U+3446.0	fTotalStrokes	 7
U+3446.0	fCangjie	ORHU
U+3446.0	fFourCorner	2621
U+3446.0	fCantonese	hing1
U+3447.0	fUTF8	㑇
U+3447.0	fGB	7-223F
U+3447.0	fR/S	 9.5
U+3447.0	fTotalStrokes	 7
U+3447.0	fCangjie	ONSM
U+3447.0	fCantonese	zau3; cau1; zaau3
U+3447.0	fMandarin	zhou4
U+3448.0	fUTF8	㑈
U+3448.0	fGB	7-2323
U+3448.0	fR/S	 9.5
U+3448.0	fTotalStrokes	 7
U+3448.0	fCangjie	OKD
U+3448.0	fCantonese	dung1
U+3448.0	fMandarin	dong4
U+3449.0	fUTF8	㑉
U+3449.0	fCNS11643	3-2746
U+3449.0	fR/S	 9.6
U+3449.0	fTotalStrokes	 8
U+3449.0	fCangjie	OHNN
U+3449.0	fFourCorner	27210
U+3449.0	fHakka	  siuk7; suk7
U+3449.0	fCantonese	suk1; cuk1
U+3449.0	fMandarin	su4
U+3449.0	fEnglish	 be kept in a constrained condition
U+344A.0	fUTF8	㑊
U+344A.0	fCNS11643	5-2525
U+344A.0	fR/S	 9.6
U+344A.0	fTotalStrokes	 8
U+344A.0	fCangjie	OYLC
U+344A.0	fFourCorner	20230
U+344A.0	fHakka	 jit8; rit8; rit7; jit7
U+344A.0	fCantonese	jik6; zik6
U+344A.0	fMandarin	yi4
U+344A.0	fEnglish	a disease
U+344B.0	fUTF8	㑋
U+344B.0	fCNS11643	4-2539
U+344B.0	fGB	3-3122
U+344B.0	fR/S	 9.6
U+344B.0	fTotalStrokes	 8; 7
U+344B.0	fCangjie	OTW
U+344B.0	fFourCorner	25260
U+344B.0	fHakka	 kiung5; kiung3
U+344B.0	fCantonese	hung3; kung4
U+344B.0	fMandarin	qiong4; jiong4; kong3; qiong2
U+344B.0	fEnglish	[1] small; dwarfed [2] bend; crouch
U+344C.0	fUTF8	㑌
U+344C.0	fCNS11643	4-253C
U+344C.0	fGB	5-3170
U+344C.0	fR/S	 9.6
U+344C.0	fTotalStrokes	 8; 7
U+344C.0	fCangjie	OSMG
U+344C.0	fFourCorner	21210
U+344C.0	fHakka	 kiong1; wong1; vong1
U+344C.0	fCantonese	hong1; wong1
U+344C.0	fMandarin	wang1; kuang1
U+344C.0	fEnglish	same as &#x5C2A;
U+344D.0	fUTF8	㑍
U+344D.0	fCNS11643	3-2745
U+344D.0	fR/S	 9.6
U+344D.0	fTotalStrokes	 8
U+344D.0	fCangjie	OQD
U+344D.0	fFourCorner	25290
U+344D.0	fCantonese	loi6; leoi5
U+344D.0	fMandarin	lei4; lei3
U+344D.0	fEnglish	[1] inferior; secondary [2] ugly
U+344E.0	fUTF8	㑎
U+344E.0	fCNS11643	5-252B
U+344E.0	fGB	7-232A
U+344E.0	fR/S	 9.6
U+344E.0	fTotalStrokes	 8
U+344E.0	fCangjie	OVVU
U+344E.0	fFourCorner	22270
U+344E.0	fCantonese	nou5; lou5 (also)
U+344E.0	fMandarin	nao3
U+344E.0	fEnglish	a Chinese family name
U+344F.0	fUTF8	㑏
U+344F.0	fCNS11643	4-253B
U+344F.0	fR/S	 9.6
U+344F.0	fTotalStrokes	 8; 7
U+344F.0	fCangjie	OJDI
U+344F.0	fFourCorner	23240
U+344F.0	fHakka	 du3
U+344F.0	fCantonese	cyu5; zyu2
U+344F.0	fMandarin	zhu4
U+344F.0	fEnglish	corrupted form of &#x4F47;
U+3450.0	fUTF8	㑐
U+3450.0	fCNS11643	4-2538
U+3450.0	fGB	3-307C
U+3450.0	fR/S	 9.6
U+3450.0	fTotalStrokes	 8; 7
U+3450.0	fCangjie	OYMF
U+3450.0	fFourCorner	21290
U+3450.0	fCantonese	suk1
U+3450.0	fMandarin	shu2
U+3450.0	fEnglish	[1] move; start [2] shake [3] the name of a person
U+3451.0	fUTF8	㑑
U+3451.0	fCNS11643	3-2750
U+3451.0	fR/S	 9.6
U+3451.0	fTotalStrokes	 8
U+3451.0	fCangjie	OHBU
U+3451.0	fFourCorner	26200
U+3451.0	fHakka	 zam3
U+3451.0	fCantonese	zat6; za1
U+3451.0	fMandarin	zan3; zan2
U+3452.0	fUTF8	㑒
U+3452.0	fR/S	 9.6
U+3452.0	fTotalStrokes	 8
U+3452.0	fCangjie	OMLO
U+3452.0	fCantonese	cim1
U+3452.0	fEnglish	[Japanese character]
U+3453.0	fUTF8	㑓
U+3453.0	fCNS11643	F-254A
U+3453.0	fR/S	 9.6
U+3453.0	fTotalStrokes	 8
U+3453.0	fCangjie	OIKE
U+3453.0	fCantonese	zeon3
U+3454.0	fUTF8	㑔
U+3454.0	fGB	7-2322
U+3454.0	fR/S	 9.6
U+3454.0	fTotalStrokes	 8
U+3454.0	fCangjie	OMBO
U+3454.0	fCantonese	seoi2
U+3454.0	fMandarin	xu3
U+3454.0	fEnglish	corrupt form of &#x5066;
U+3455.0	fUTF8	㑕
U+3455.0	fGB	7-2324
U+3455.0	fR/S	 9.6
U+3455.0	fTotalStrokes	 8
U+3455.0	fCangjie	OUNI
U+3455.0	fCantonese	seoi3
U+3456.0	fUTF8	㑖
U+3456.0	fR/S	 9.6
U+3456.0	fTotalStrokes	 8
U+3456.0	fCangjie	OOL
U+3456.0	fCantonese	zung6
U+3457.0	fUTF8	㑗
U+3457.0	fCNS11643	3-2B31
U+3457.0	fGB	5-3223
U+3457.0	fR/S	 9.7
U+3457.0	fTotalStrokes	 9
U+3457.0	fCangjie	OHXH
U+3457.0	fFourCorner	22240
U+3457.0	fHakka	 shin1; siin1
U+3457.0	fCantonese	san1; saan1; saan6
U+3457.0	fMandarin	shen1
U+3457.0	fEnglish	[1] a god [2] pregnant
U+3458.0	fUTF8	㑘
U+3458.0	fCNS11643	6-2E5A
U+3458.0	fR/S	 9.7
U+3458.0	fTotalStrokes	 9
U+3458.0	fCangjie	OIT
U+3458.0	fFourCorner	23250
U+3458.0	fHakka	 gai5
U+3458.0	fCantonese	gaai3; kaa3
U+3458.0	fMandarin	jia4; jie4; jie5
U+3458.0	fEnglish	same as &#x4EF7;
U+3459.0	fUTF8	㑙
U+3459.0	fCNS11643	4-2839
U+3459.0	fGB	3-3135
U+3459.0	fR/S	 9.7
U+3459.0	fTotalStrokes	 9
U+3459.0	fCangjie	OSJU
U+3459.0	fFourCorner	21210
U+3459.0	fCantonese	dip3; dip6
U+3459.0	fMandarin	die2; yong3
U+3459.0	fEnglish	frivolous; flippant; capricious; playful
U+345A.0	fUTF8	㑚
U+345A.0	fCNS11643	3-2B30
U+345A.0	fR/S	 9.7
U+345A.0	fTotalStrokes	 9
U+345A.0	fCangjie	OSQL
U+345A.0	fFourCorner	27220
U+345A.0	fHakka	 no2; na3; nai5; ne5
U+345A.0	fCantonese	naa5; laa5 (also); no4; naa4; na5; no1; lo4
U+345A.0	fMandarin	nuo2; na4; ne4; nei2
U+345A.0	fEnglish	[1] used for 'n' in transliteration of Sanscrit words (Buddhist) [2] non-classical form of &#x90A3; [3] you (plural)
U+345B.0	fUTF8	㑛
U+345B.0	fCNS11643	5-2821
U+345B.0	fGB	S-226C
U+345B.0	fR/S	 9.7
U+345B.0	fTotalStrokes	 9
U+345B.0	fCangjie	ODL
U+345B.0	fFourCorner	25290
U+345B.0	fHakka	  suk7
U+345B.0	fCantonese	suk1; cuk1
U+345B.0	fMandarin	su4
U+345B.0	fEnglish	 shake the head in doubt
U+345C.0	fUTF8	㑜
U+345C.0	fCNS11643	4-283A
U+345C.0	fGB	3-313E
U+345C.0	fR/S	 9.7
U+345C.0	fTotalStrokes	 9
U+345C.0	fCangjie	OQHL
U+345C.0	fFourCorner	22220
U+345C.0	fHakka	 zhat7; zet7; zhet7
U+345C.0	fCantonese	jai6; zaai6
U+345C.0	fMandarin	chi4; yi4
U+345C.0	fEnglish	 join (as a seam); engrave
U+345D.0	fUTF8	㑝
U+345D.0	fCNS11643	3-2B2A
U+345D.0	fR/S	 9.7
U+345D.0	fTotalStrokes	 9
U+345D.0	fCangjie	OMGT
U+345D.0	fFourCorner	21240
U+345D.0	fHakka	 lung5; lung6
U+345D.0	fCantonese	lung6
U+345D.0	fMandarin	long4
U+345D.0	fEnglish	stupid
U+345E.0	fUTF8	㑞
U+345E.0	fCNS11643	4-2837
U+345E.0	fGB	5-3226
U+345E.0	fR/S	 9.7
U+345E.0	fTotalStrokes	 9
U+345E.0	fCangjie	OFT
U+345E.0	fFourCorner	29240
U+345E.0	fCantonese	jing6; zing6
U+345E.0	fMandarin	yun4; ying4
U+345E.0	fEnglish	same as &#x5AB5;
U+345F.0	fUTF8	㑟
U+345F.0	fCNS11643	4-283C
U+345F.0	fR/S	 9.7
U+345F.0	fTotalStrokes	 9
U+345F.0	fCangjie	OOMC
U+345F.0	fFourCorner	22280
U+345F.0	fCantonese	bang2; mang1
U+345F.0	fMandarin	bang3; beng3; bo2; peng3
U+345F.0	fEnglish	an insincere and cunning person; a pretentious person
U+3460.0	fUTF8	㑠
U+3460.0	fGB	7-232C
U+3460.0	fR/S	 9.7
U+3460.0	fTotalStrokes	 9
U+3460.0	fCangjie	OLEG
U+3460.0	fCantonese	gin1
U+3461.0	fUTF8	㑡
U+3461.0	fR/S	 9.7
U+3461.0	fTotalStrokes	 9
U+3461.0	fCangjie	OAIL
U+3461.0	fCantonese	zik1
U+3462.0	fUTF8	㑢
U+3462.0	fGB	3-3134
U+3462.0	fR/S	 9.7
U+3462.0	fTotalStrokes	 9
U+3462.0	fCangjie	OGIL
U+3462.0	fCantonese	koek3
U+3463.0	fUTF8	㑣
U+3463.0	fCNS11643	5-2B6C
U+3463.0	fGB	S-226D
U+3463.0	fR/S	 9.8
U+3463.0	fTotalStrokes	 10
U+3463.0	fCangjie	ODD
U+3463.0	fFourCorner	24290
U+3463.0	fCantonese	laam4; lam2; lam4
U+3463.0	fMandarin	lan2
U+3463.0	fEnglish	stupid; loutish; without ability; unable; lacking power
U+3464.0	fUTF8	㑤
U+3464.0	fCNS11643	3-3449
U+3464.0	fR/S	 9.9
U+3464.0	fTotalStrokes	 11
U+3464.0	fCangjie	OTW
U+3464.0	fFourCorner	24260
U+3464.0	fCantonese	maau4; miu4
U+3464.0	fMandarin	miao2
U+3464.0	fEnglish	same as &#x5A8C;
U+3465.0	fUTF8	㑥
U+3465.0	fCNS11643	3-2F52
U+3465.0	fGB	5-3231
U+3465.0	fR/S	 9.8
U+3465.0	fTotalStrokes	 10
U+3465.0	fCangjie	OAPH
U+3465.0	fFourCorner	26220
U+3465.0	fCantonese	ji6; zi6
U+3465.0	fMandarin	yi4
U+3465.0	fEnglish	[1] disrespectful; irreverent; make light of; neglect; careless; rush [2] exchange [3] easy to get along with (said of one's personality)
U+3466.0	fUTF8	㑦
U+3466.0	fCNS11643	4-2B65
U+3466.0	fGB	5-323E
U+3466.0	fR/S	 9.8
U+3466.0	fTotalStrokes	 10
U+3466.0	fCangjie	OISK
U+3466.0	fFourCorner	22230
U+3466.0	fCantonese	lai6; leoi6
U+3466.0	fMandarin	li4
U+3466.0	fEnglish	[1] anger; rage; angry [2] same as &#x623E;
U+3467.0	fUTF8	㑧
U+3467.0	fCNS11643	6-3538
U+3467.0	fR/S	 9.8
U+3467.0	fTotalStrokes	 10; 9
U+3467.0	fCangjie	OHDD
U+3467.0	fFourCorner	22240
U+3467.0	fCantonese	gwai3
U+3467.0	fMandarin	ji4
U+3467.0	fEnglish	corrupt form of &#x60B8;
U+3468.0	fUTF8	㑨
U+3468.0	fR/S	 9.8
U+3468.0	fTotalStrokes	 10
U+3468.0	fCangjie	OROK
U+3468.0	fCantonese	jyu5; zyu5
U+3468.0	fMandarin	yu3
U+3468.0	fEnglish	[1] great; big; tall; vast [2] noble; high in rank [3] very; much
U+3469.0	fUTF8	㑩
U+3469.0	fGB	S-226F
U+3469.0	fR/S	 9.8
U+3469.0	fTotalStrokes	 10
U+3469.0	fCangjie	OWLN
U+3469.0	fCantonese	lo4
U+3469.0	fMandarin	luo2
U+3469.0	fEnglish	smart
U+346A.0	fUTF8	㑪
U+346A.0	fR/S	 9.8
U+346A.0	fTotalStrokes	 10
U+346A.0	fCangjie	OYKL
U+346A.0	fCantonese	caai4; cai4
U+346A.0	fMandarin	chai2
U+346A.0	fEnglish	non-classical form of &#x5115;
U+346B.0	fUTF8	㑫
U+346B.0	fGB	7-232B
U+346B.0	fR/S	 9.8
U+346B.0	fTotalStrokes	 10
U+346B.0	fCangjie	OOIP
U+346B.0	fFourCorner	2823
U+346B.0	fCantonese	nim6; lim6 (also)
U+346C.0	fUTF8	㑬
U+346C.0	fR/S	 9.8
U+346C.0	fTotalStrokes	 10
U+346C.0	fCangjie	OVJR
U+346C.0	fCantonese	gu1; gwu1 (also)
U+346D.0	fUTF8	㑭
U+346D.0	fR/S	 9.8
U+346D.0	fTotalStrokes	 10
U+346D.0	fCangjie	OWML
U+346D.0	fFourCorner	2624
U+346D.0	fCantonese	bei2
U+346E.0	fUTF8	㑮
U+346E.0	fCNS11643	4-3045
U+346E.0	fGB	3-3132
U+346E.0	fR/S	 9.9
U+346E.0	fTotalStrokes	 11
U+346E.0	fCangjie	OBJJ
U+346E.0	fFourCorner	27250
U+346E.0	fHakka	 fun2
U+346E.0	fCantonese	wan4; gwaat1; waan4
U+346E.0	fMandarin	hun2
U+346E.0	fEnglish	[1] a Chinese family name [2] a girl's name
U+346F.0	fUTF8	㑯
U+346F.0	fCNS11643	3-343E
U+346F.0	fR/S	 9.9
U+346F.0	fTotalStrokes	 11
U+346F.0	fCangjie	OMBC
U+346F.0	fFourCorner	21280
U+346F.0	fHakka	 fu5; su3
U+346F.0	fCantonese	seoi2; seoi1
U+346F.0	fMandarin	xu3
U+346F.0	fEnglish	[1] a family name [2] all; together; mutually [3] a low rank officer to take charge of the bandits in ancient time
U+3470.0	fUTF8	㑰
U+3470.0	fCNS11643	3-3448
U+3470.0	fR/S	 9.9
U+3470.0	fTotalStrokes	 11
U+3470.0	fCangjie	OVNO
U+3470.0	fFourCorner	27230
U+3470.0	fCantonese	fui3; fui4
U+3470.0	fMandarin	hui4
U+3470.0	fEnglish	(a corrupted form) distress; very difficulty; anxiety; very tired
U+3471.0	fUTF8	㑱
U+3471.0	fCNS11643	4-304A
U+3471.0	fR/S	 9.9
U+3471.0	fTotalStrokes	 11
U+3471.0	fCangjie	ONHD
U+3471.0	fFourCorner	27290
U+3471.0	fCantonese	jiu5; nau4
U+3471.0	fMandarin	nao2; rao3
U+3471.0	fEnglish	same as &#x27CE8;
U+3472.0	fUTF8	㑲
U+3472.0	fR/S	 9.9
U+3472.0	fTotalStrokes	 11
U+3472.0	fCangjie	OJBJ
U+3472.0	fCantonese	naam4; laam4 (also)
U+3473.0	fUTF8	㑳
U+3473.0	fCNS11643	4-3638
U+3473.0	fGB	3-3070
U+3473.0	fR/S	 9.10
U+3473.0	fTotalStrokes	 12
U+3473.0	fCangjie	OPUU
U+3473.0	fFourCorner	27220
U+3473.0	fHakka	 ziu5
U+3473.0	fCantonese	zau3; cau1; zaau3
U+3473.0	fMandarin	zhou4
U+3473.0	fEnglish	[1] hire oneself out to labour [2] clever
U+3473.1	fUTF8	㑳
U+3473.1	fCNS11643	4-3638
U+3473.1	fGB	3-3070
U+3473.1	fR/S	 9.10
U+3473.1	fTotalStrokes	 12
U+3473.1	fCangjie	OPUU
U+3473.1	fFourCorner	27220
U+3473.2	fUTF8	㑳
U+3473.2	fCNS11643	4-3638
U+3473.2	fGB	3-3070
U+3473.2	fR/S	 9.10
U+3473.2	fTotalStrokes	 12
U+3473.2	fCangjie	OPUU
U+3473.2	fFourCorner	27220
U+3474.0	fUTF8	㑴
U+3474.0	fCNS11643	3-3973
U+3474.0	fGB	5-3260
U+3474.0	fR/S	 9.10
U+3474.0	fTotalStrokes	 12
U+3474.0	fCangjie	OSME
U+3474.0	fFourCorner	27240
U+3474.0	fCantonese	cam1; caam1
U+3474.0	fMandarin	qin1
U+3474.0	fEnglish	standard form of &#x4FB5;
U+3475.0	fUTF8	㑵
U+3475.0	fCNS11643	4-3632
U+3475.0	fGB	5-325C
U+3475.0	fR/S	 9.10
U+3475.0	fTotalStrokes	 12
U+3475.0	fCangjie	OKOK
U+3475.0	fFourCorner	20230
U+3475.0	fCantonese	zat6
U+3475.0	fMandarin	ji2; han4; jie2; zha3; zi2
U+3475.0	fEnglish	same as &#x5AC9;
U+3476.0	fUTF8	㑶
U+3476.0	fCNS11643	4-3637
U+3476.0	fGB	3-3060
U+3476.0	fR/S	 9.10
U+3476.0	fTotalStrokes	 12
U+3476.0	fCangjie	OOND
U+3476.0	fFourCorner	28210
U+3476.0	fCantonese	hei3; haai4
U+3476.0	fMandarin	xi4
U+3476.0	fEnglish	non-classical form of &#x613E;
U+3477.0	fUTF8	㑷
U+3477.0	fCNS11643	6-4655
U+3477.0	fR/S	 9.10
U+3477.0	fTotalStrokes	 12
U+3477.0	fCangjie	OIBP
U+3477.0	fFourCorner	21210
U+3477.0	fHakka	 tai5
U+3477.0	fCantonese	taai3
U+3477.0	fMandarin	tai4; tian4; zhuan4
U+3477.0	fEnglish	same as &#x614B;
U+3478.0	fUTF8	㑸
U+3478.0	fR/S	 9.10
U+3478.0	fTotalStrokes	 12
U+3478.0	fCangjie	OJRR
U+3478.0	fFourCorner	2422
U+3478.0	fCantonese	jiu4; ziu4
U+3478.0	fMandarin	yao2; ai3; you2
U+3478.0	fEnglish	[meaning not known]
U+3479.0	fUTF8	㑹
U+3479.0	fCNS11643	6-497B
U+3479.0	fR/S	 9.10
U+3479.0	fTotalStrokes	 12
U+3479.0	fCangjie	OMWA
U+3479.0	fFourCorner	80600
U+3479.0	fHakka	 fui5; kwai5
U+3479.0	fCantonese	wui6; wui2; gwat3; wui4; mok6
U+3479.0	fMandarin	hui4; gua1; guai4
U+3479.0	fEnglish	non-classical form of &#x6703;
U+347A.0	fUTF8	㑺
U+347A.0	fCNS11643	3-3974
U+347A.0	fR/S	 9.10
U+347A.0	fTotalStrokes	 12
U+347A.0	fCangjie	OOGS
U+347A.0	fFourCorner	20220
U+347A.0	fCantonese	zeon3; zeoi6
U+347A.0	fMandarin	jun4
U+347A.0	fEnglish	variant of &#x4FCA;
U+347B.0	fUTF8	㑻
U+347B.0	fCNS11643	3-4035
U+347B.0	fR/S	 9.11
U+347B.0	fTotalStrokes	 13
U+347B.0	fCangjie	OJPU
U+347B.0	fFourCorner	23270
U+347B.0	fHakka	 mat8
U+347B.0	fCantonese	maat3; maa6; mei5
U+347B.0	fMandarin	ma4
U+347B.0	fEnglish	strong; muscular
U+347C.0	fUTF8	㑼
U+347C.0	fCNS11643	3-4038
U+347C.0	fR/S	 9.11
U+347C.0	fTotalStrokes	 13
U+347C.0	fCangjie	OWHR
U+347C.0	fFourCorner	27260
U+347C.0	fCantonese	loek6
U+347C.0	fMandarin	luue4; le4
U+347C.0	fEnglish	the name of a god
U+347D.0	fUTF8	㑽
U+347D.0	fCNS11643	3-403B
U+347D.0	fGB	S-226A
U+347D.0	fR/S	 9.11
U+347D.0	fTotalStrokes	 13
U+347D.0	fCangjie	OFBG
U+347D.0	fFourCorner	29210
U+347D.0	fCantonese	tong4
U+347D.0	fMandarin	tang2
U+347D.0	fEnglish	same as &#x508F;
U+347E.0	fUTF8	㑾
U+347E.0	fCNS11643	4-3C2C
U+347E.0	fGB	5-3267
U+347E.0	fR/S	 9.11
U+347E.0	fTotalStrokes	 13
U+347E.0	fCangjie	OBYR
U+347E.0	fFourCorner	27260
U+347E.0	fHakka	 jau2
U+347E.0	fCantonese	jiu4; ziu4
U+347E.0	fMandarin	yao2; xiao2
U+347E.0	fEnglish	interchangeable with &#x7E47;
U+347F.0	fUTF8	㑿
U+347F.0	fCNS11643	4-3C2D
U+347F.0	fGB	3-3230
U+347F.0	fR/S	 9.11
U+347F.0	fTotalStrokes	 13
U+347F.0	fCangjie	OVVD
U+347F.0	fFourCorner	22290
U+347F.0	fHakka	  cau3
U+347F.0	fCantonese	zaau6; coek3; huk6; zau3; ziu6
U+347F.0	fMandarin	zhao4; tiao2
U+347F.0	fEnglish	[1] tall (as a man) [2] slender
U+3480.0	fUTF8	㒀
U+3480.0	fCNS11643	4-3C28
U+3480.0	fGB	3-322D
U+3480.0	fR/S	 9.11
U+3480.0	fTotalStrokes	 13
U+3480.0	fCangjie	OYCB
U+3480.0	fFourCorner	20220
U+3480.0	fHakka	 tit7
U+3480.0	fCantonese	zaak6; cim4; cip3; zak6
U+3480.0	fMandarin	zhe2; zha3
U+3480.0	fEnglish	do not fear to
U+3481.0	fUTF8	㒁
U+3481.0	fCNS11643	4-3C2A
U+3481.0	fGB	3-3227
U+3481.0	fR/S	 9.11
U+3481.0	fTotalStrokes	 13
U+3481.0	fCangjie	OWGJ
U+3481.0	fFourCorner	26200
U+3481.0	fCantonese	jyu5; sou3; zyu5
U+3481.0	fMandarin	yu3; yun4
U+3481.0	fEnglish	same as &#x4FC1;
U+3482.0	fUTF8	㒂
U+3482.0	fCNS11643	5-4457
U+3482.0	fGB	7-232D
U+3482.0	fR/S	 9.12
U+3482.0	fTotalStrokes	 14
U+3482.0	fCangjie	OTJA
U+3482.0	fFourCorner	24260
U+3482.0	fHakka	 ch'ok8; cok8
U+3482.0	fCantonese	zoek3; zoek6
U+3482.0	fMandarin	ku4; zhuo2
U+3482.0	fEnglish	[1] bestow; grant [2] act; do [3] quiet; still; peace; tranquility
U+3483.0	fUTF8	㒃
U+3483.0	fCNS11643	4-4237
U+3483.0	fR/S	 9.12
U+3483.0	fTotalStrokes	 14
U+3483.0	fCangjie	OIPC
U+3483.0	fFourCorner	23240
U+3483.0	fCantonese	ji6; zi6
U+3483.0	fMandarin	er4
U+3483.0	fEnglish	same as &#x8CB3;
U+3484.0	fUTF8	㒄
U+3484.0	fCNS11643	4-4233
U+3484.0	fGB	3-3235
U+3484.0	fR/S	 9.12
U+3484.0	fTotalStrokes	 14
U+3484.0	fCangjie	OBKF
U+3484.0	fFourCorner	23230
U+3484.0	fCantonese	jin5; ngam4; zim5
U+3484.0	fMandarin	ran3; nang4; nen4
U+3484.0	fEnglish	[1] brittle and friable [2] fear; dread; fright; scare
U+3485.0	fUTF8	㒅
U+3485.0	fCNS11643	4-422E
U+3485.0	fGB	5-3270
U+3485.0	fR/S	 9.12
U+3485.0	fTotalStrokes	 14
U+3485.0	fCangjie	OYNV
U+3485.0	fFourCorner	20230
U+3485.0	fHakka	  ki3
U+3485.0	fCantonese	kai2
U+3485.0	fMandarin	qi3
U+3485.0	fEnglish	 open the buttons of a coat
U+3486.0	fUTF8	㒆
U+3486.0	fCNS11643	3-4578
U+3486.0	fGB	5-326E
U+3486.0	fR/S	 9.12
U+3486.0	fTotalStrokes	 14
U+3486.0	fCangjie	OORM
U+3486.0	fFourCorner	28220
U+3486.0	fHakka	 hip7
U+3486.0	fCantonese	kap1; jap1
U+3486.0	fMandarin	xi4; chi4; ke4; xiao4; yan4
U+3486.0	fEnglish	same as &#x7FD5;
U+3487.0	fUTF8	㒇
U+3487.0	fCNS11643	6-5A73
U+3487.0	fR/S	 9.12
U+3487.0	fTotalStrokes	 14
U+3487.0	fCangjie	OOTF
U+3487.0	fFourCorner	28230
U+3487.0	fHakka	 vu3
U+3487.0	fCantonese	mou5; ng5
U+3487.0	fMandarin	wu3; mu4; hu1; wu1; xu3
U+3487.0	fEnglish	same as &#x61AE;
U+3488.0	fUTF8	㒈
U+3488.0	fCNS11643	4-422F
U+3488.0	fGB	3-322E
U+3488.0	fR/S	 9.12
U+3488.0	fTotalStrokes	 14
U+3488.0	fCangjie	OMJK
U+3488.0	fFourCorner	28240
U+3488.0	fCantonese	haam3; hon6
U+3488.0	fMandarin	xian4; han4
U+3488.0	fEnglish	dangerous; lofty; steep; high and dangerous
U+3489.0	fUTF8	㒉
U+3489.0	fCNS11643	3-4573
U+3489.0	fR/S	 9.12
U+3489.0	fTotalStrokes	 14
U+3489.0	fCangjie	OFBD
U+3489.0	fFourCorner	29290
U+3489.0	fHakka	 chong3
U+3489.0	fCantonese	cong2; tong2
U+3489.0	fMandarin	chang3; tang3
U+3489.0	fEnglish	same as &#x60DD;
U+348A.0	fUTF8	㒊
U+348A.0	fCNS11643	4-4F59
U+348A.0	fGB	3-3244
U+348A.0	fR/S	 9.14; 9.12
U+348A.0	fTotalStrokes	 16; 14
U+348A.0	fCangjie	OSIM
U+348A.0	fFourCorner	27210
U+348A.0	fHakka	 sep7
U+348A.0	fCantonese	saap3; saap1; sik1
U+348A.0	fMandarin	se4
U+348A.0	fEnglish	unattainable
U+348B.0	fUTF8	㒋
U+348B.0	fCNS11643	3-4572
U+348B.0	fR/S	 9.12
U+348B.0	fTotalStrokes	 14
U+348B.0	fCangjie	OTCL
U+348B.0	fFourCorner	22220
U+348B.0	fHakka	  sii1; se1
U+348B.0	fCantonese	si1
U+348B.0	fMandarin	si1
U+348B.0	fEnglish	[1] servant [2] wood cutter [3] forage [4] same as &#x53AE;
U+348C.0	fUTF8	㒌
U+348C.0	fCNS11643	3-456F
U+348C.0	fR/S	 9.12
U+348C.0	fTotalStrokes	 14
U+348C.0	fCangjie	OFFN
U+348C.0	fFourCorner	29210
U+348C.0	fHakka	 kiun2
U+348C.0	fCantonese	king4
U+348C.0	fMandarin	qiong2
U+348C.0	fEnglish	ancient form of &#x7162;
U+348D.0	fUTF8	㒍
U+348D.0	fCNS11643	3-4577
U+348D.0	fGB	5-3272
U+348D.0	fR/S	 9.12
U+348D.0	fTotalStrokes	 14
U+348D.0	fCangjie	OIIF
U+348D.0	fFourCorner	23290
U+348D.0	fCantonese	leoi4; leoi5
U+348D.0	fMandarin	lei2; lei3
U+348D.0	fEnglish	interchangeable with &#x50AB, &#x513D;
U+348E.0	fUTF8	㒎
U+348E.0	fCNS11643	4-4232
U+348E.0	fGB	3-3231
U+348E.0	fR/S	 9.12
U+348E.0	fTotalStrokes	 14
U+348E.0	fCangjie	OMKY
U+348E.0	fFourCorner	22210
U+348E.0	fCantonese	saap3; sat3
U+348E.0	fMandarin	zhuo2; sa3; sa4; tan4
U+348E.0	fEnglish	evil; wicked; wrong; foul
U+348F.0	fUTF8	㒏
U+348F.0	fR/S	 9.12
U+348F.0	fTotalStrokes	 14
U+348F.0	fCangjie	OJTY
U+348F.0	fCantonese	hon4
U+3490.0	fUTF8	㒐
U+3490.0	fR/S	 9.12
U+3490.0	fTotalStrokes	 14
U+3490.0	fCangjie	OHPA
U+3490.0	fCantonese	seon2
U+3491.0	fUTF8	㒑
U+3491.0	fCNS11643	4-487B
U+3491.0	fGB	3-3072
U+3491.0	fR/S	 9.13
U+3491.0	fTotalStrokes	 15
U+3491.0	fCangjie	OSEG
U+3491.0	fFourCorner	21210
U+3491.0	fCantonese	fui2; hung2; kui2; wak6
U+3491.0	fMandarin	hui4; kui3
U+3491.0	fEnglish	[1] grow up [2] commodity prices
U+3492.0	fUTF8	㒒
U+3492.0	fCNS11643	3-4B26
U+3492.0	fGB	5-3165
U+3492.0	fR/S	 9.13
U+3492.0	fTotalStrokes	 15
U+3492.0	fCangjie	OTCD
U+3492.0	fFourCorner	22290
U+3492.0	fHakka	 puk8
U+3492.0	fCantonese	buk6
U+3492.0	fMandarin	pu2
U+3492.0	fEnglish	same as &#x50D5;
U+3493.0	fUTF8	㒓
U+3493.0	fCNS11643	3-4B25
U+3493.0	fGB	5-3174
U+3493.0	fR/S	 9.13
U+3493.0	fTotalStrokes	 15
U+3493.0	fCangjie	OYGQ
U+3493.0	fFourCorner	24230
U+3493.0	fHakka	 dat7; tat7
U+3493.0	fCantonese	taat3
U+3493.0	fMandarin	ta4
U+3493.0	fEnglish	abscond; escape
U+3494.0	fUTF8	㒔
U+3494.0	fCNS11643	4-487D
U+3494.0	fGB	3-323F
U+3494.0	fR/S	 9.13
U+3494.0	fTotalStrokes	 15
U+3494.0	fCangjie	OWLI
U+3494.0	fFourCorner	26220
U+3494.0	fHakka	  tuk8
U+3494.0	fCantonese	suk6
U+3494.0	fMandarin	du2; shu3
U+3494.0	fEnglish	 shake the head
U+3495.0	fUTF8	㒕
U+3495.0	fCNS11643	3-4B24
U+3495.0	fR/S	 9.13
U+3495.0	fTotalStrokes	 15
U+3495.0	fCangjie	OYVG
U+3495.0	fFourCorner	20210
U+3495.0	fCantonese	ong1; ngong1 (also); zoeng1
U+3495.0	fMandarin	yang1
U+3495.0	fEnglish	unyield
U+3496.0	fUTF8	㒖
U+3496.0	fCNS11643	3-4B28
U+3496.0	fR/S	 9.13
U+3496.0	fTotalStrokes	 15
U+3496.0	fCangjie	OTWB
U+3496.0	fFourCorner	24220
U+3496.0	fHakka	 ngieu3
U+3496.0	fCantonese	ngau5; au5 (also); ngaau5; aau5
U+3496.0	fMandarin	ou3
U+3496.0	fEnglish	same as &#x85D5;
U+3497.0	fUTF8	㒗
U+3497.0	fCNS11643	3-4B2A
U+3497.0	fR/S	 9.13
U+3497.0	fTotalStrokes	 15
U+3497.0	fCangjie	OTBG
U+3497.0	fFourCorner	24210
U+3497.0	fCantonese	toi4
U+3497.0	fMandarin	tai2
U+3497.0	fEnglish	variant of &#x5113;
U+3498.0	fUTF8	㒘
U+3498.0	fGB	3-3162
U+3498.0	fR/S	 9.13
U+3498.0	fTotalStrokes	 15
U+3498.0	fCangjie	OSET
U+3498.0	fCantonese	syu6
U+3499.0	fUTF8	㒙
U+3499.0	fCNS11643	3-5032
U+3499.0	fR/S	 9.14
U+3499.0	fTotalStrokes	 16
U+3499.0	fCangjie	OVFB
U+3499.0	fFourCorner	26220
U+3499.0	fCantonese	min4; min6
U+3499.0	fMandarin	mian2
U+3499.0	fEnglish	[1] hand or bow the head; droop; lower [2] low; beneath
U+349A.0	fUTF8	㒚
U+349A.0	fCNS11643	4-4F5D
U+349A.0	fGB	3-3243
U+349A.0	fR/S	 9.14
U+349A.0	fTotalStrokes	 16
U+349A.0	fCangjie	OBMP
U+349A.0	fFourCorner	22230
U+349A.0	fHakka	 wun3; jun3; run3; vun3
U+349A.0	fCantonese	jan3; waan2
U+349A.0	fMandarin	yin4; wen3
U+349A.0	fEnglish	 rely on
U+349B.0	fUTF8	㒛
U+349B.0	fCNS11643	5-5359
U+349B.0	fR/S	 9.14
U+349B.0	fTotalStrokes	 16
U+349B.0	fCangjie	OSMG
U+349B.0	fFourCorner	27210
U+349B.0	fCantonese	tiu5; deoi2; diu6
U+349B.0	fMandarin	tiao3; diao4
U+349B.0	fEnglish	 stand alone; independent
U+349C.0	fUTF8	㒜
U+349C.0	fCNS11643	4-4F5B
U+349C.0	fGB	5-3148
U+349C.0	fR/S	 9.14
U+349C.0	fTotalStrokes	 16
U+349C.0	fCangjie	OHXC
U+349C.0	fFourCorner	27280
U+349C.0	fCantonese	jyu5; zyu5
U+349C.0	fMandarin	yu3; yu2
U+349C.0	fEnglish	[1] respectful; attentive [2] carefully; cautious; heed; be watchful [3] rely on; trust to; lean towards
U+349D.0	fUTF8	㒝
U+349D.0	fCNS11643	3-543F
U+349D.0	fGB	5-327B
U+349D.0	fR/S	 9.15
U+349D.0	fTotalStrokes	 17
U+349D.0	fCangjie	OTWI
U+349D.0	fFourCorner	24250
U+349D.0	fCantonese	mit6; ziu5
U+349D.0	fMandarin	mie4; wa4
U+349D.0	fEnglish	 deceive; artful; false,
U+349E.0	fUTF8	㒞
U+349E.0	fCNS11643	3-5753
U+349E.0	fR/S	 9.15; 9.16
U+349E.0	fTotalStrokes	 18; 17
U+349E.0	fCangjie	OUOS
U+349E.0	fFourCorner	22220
U+349E.0	fHakka	 zun5
U+349E.0	fCantonese	wai6; zeon3
U+349E.0	fMandarin	jun4; ke3
U+349E.0	fEnglish	[1] valiant; brave [2] eminent
U+349F.0	fUTF8	㒟
U+349F.0	fCNS11643	4-5A67
U+349F.0	fGB	3-3170
U+349F.0	fR/S	 9.16
U+349F.0	fTotalStrokes	 18
U+349F.0	fCangjie	OYSV
U+349F.0	fFourCorner	20230
U+349F.0	fHakka	  ngiau3
U+349F.0	fCantonese	niu5; liu5 (also); seoi1
U+349F.0	fMandarin	niao3
U+349F.0	fEnglish	agile
U+34A0.0	fUTF8	㒠
U+34A0.0	fCNS11643	4-5A68
U+34A0.0	fGB	3-3248
U+34A0.0	fR/S	 9.16
U+34A0.0	fTotalStrokes	 18
U+34A0.0	fCangjie	OYEM
U+34A0.0	fFourCorner	27210
U+34A0.0	fCantonese	haai6
U+34A0.0	fMandarin	xie4
U+34A0.0	fEnglish	[1] narrow; contracted [2] quickly; fast; hasty; soon; promptly
U+34A1.0	fUTF8	㒡
U+34A1.0	fCNS11643	4-5F49
U+34A1.0	fGB	3-3249
U+34A1.0	fR/S	 9.17
U+34A1.0	fTotalStrokes	 19
U+34A1.0	fCangjie	OBUF
U+34A1.0	fFourCorner	22290
U+34A1.0	fCantonese	jau4; zaau4
U+34A1.0	fMandarin	you2
U+34A1.0	fEnglish	the name of a person
U+34A2.0	fUTF8	㒢
U+34A2.0	fR/S	 9.17
U+34A2.0	fTotalStrokes	 19
U+34A2.0	fCangjie	OOMB
U+34A2.0	fCantonese	joek6
U+34A3.0	fUTF8	㒣
U+34A3.0	fR/S	 9.17
U+34A3.0	fTotalStrokes	 19
U+34A3.0	fCangjie	OIOP
U+34A3.0	fCantonese	jing3
U+34A4.0	fUTF8	㒤
U+34A4.0	fCNS11643	4-632F
U+34A4.0	fGB	5-3253
U+34A4.0	fR/S	 9.18
U+34A4.0	fTotalStrokes	 20
U+34A4.0	fCangjie	OSJJ
U+34A4.0	fFourCorner	21240
U+34A4.0	fCantonese	cip3; sip3; zip3
U+34A4.0	fMandarin	she4
U+34A4.0	fEnglish	[1] have one's heart won; submit [2] admire [3] sincerely and willingly [4] interchangeable with &#x61FE;
U+34A5.0	fUTF8	㒥
U+34A5.0	fCNS11643	3-5C33
U+34A5.0	fR/S	 9.18
U+34A5.0	fTotalStrokes	 20
U+34A5.0	fCangjie	OUJT
U+34A5.0	fFourCorner	22210
U+34A5.0	fHakka	  fung1
U+34A5.0	fCantonese	fung1; gaam2
U+34A5.0	fMandarin	feng1
U+34A5.0	fEnglish	an immortal
U+34A6.0	fUTF8	㒦
U+34A6.0	fCNS11643	3-5C31
U+34A6.0	fGB	5-3327
U+34A6.0	fR/S	 9.18
U+34A6.0	fTotalStrokes	 20
U+34A6.0	fCangjie	OWWG
U+34A6.0	fFourCorner	26210
U+34A6.0	fCantonese	leoi5; deoi3
U+34A6.0	fMandarin	lei3
U+34A6.0	fEnglish	same as &#x5121;
U+34A7.0	fUTF8	㒧
U+34A7.0	fCNS11643	7-5274
U+34A7.0	fGB	5-325D
U+34A7.0	fR/S	 9.19
U+34A7.0	fTotalStrokes	 21
U+34A7.0	fCangjie	OYBG
U+34A7.0	fFourCorner	20210
U+34A7.0	fHakka	 li5
U+34A7.0	fCantonese	lai6
U+34A7.0	fMandarin	li2; li4
U+34A7.0	fEnglish	same as &#x5137;
U+34A8.0	fUTF8	㒨
U+34A8.0	fCNS11643	7-574C
U+34A8.0	fGB	5-3329
U+34A8.0	fR/S	 9.19
U+34A8.0	fTotalStrokes	 22; 21
U+34A8.0	fCangjie	OHXU
U+34A8.0	fFourCorner	27210
U+34A8.0	fCantonese	sin1
U+34A8.0	fMandarin	xian1
U+34A8.0	fEnglish	correct form of &#x24662F;
U+34A9.0	fUTF8	㒩
U+34A9.0	fCNS11643	4-6A41
U+34A9.0	fGB	5-332A
U+34A9.0	fR/S	 9.21
U+34A9.0	fTotalStrokes	 23
U+34A9.0	fCangjie	OVOI
U+34A9.0	fFourCorner	27230
U+34A9.0	fCantonese	lo2
U+34A9.0	fMandarin	luo3
U+34A9.0	fEnglish	same as &#x68B4;, &#x88F8;
U+34AA.0	fUTF8	㒪
U+34AA.0	fCNS11643	3-613C
U+34AA.0	fR/S	 9.24
U+34AA.0	fTotalStrokes	 25; 26
U+34AA.0	fCangjie	OSSO
U+34AA.0	fFourCorner	80130
U+34AA.0	fHakka	 siak7; sit7
U+34AA.0	fCantonese	sek3; sek1
U+34AA.0	fMandarin	xi1; ti4; xi2; di4; si4
U+34AA.0	fEnglish	ancient form of &#x932B;
U+34AB.0	fUTF8	㒫
U+34AB.0	fCNS11643	4-2233
U+34AB.0	fR/S	 10.4
U+34AB.0	fTotalStrokes	 6
U+34AB.0	fCangjie	MYMU
U+34AB.0	fFourCorner	71210
U+34AB.0	fCantonese	gei3; zaam1; hei2
U+34AB.0	fMandarin	ji4; zan1
U+34AB.0	fEnglish	ancient form of &#x5142;
U+34AC.0	fUTF8	㒬
U+34AC.0	fCNS11643	F-265A
U+34AC.0	fGB	S-2431
U+34AC.0	fR/S	 10.6
U+34AC.0	fTotalStrokes	 8
U+34AC.0	fCangjie	MUMG
U+34AC.0	fFourCorner	1221
U+34AC.0	fCantonese	wong1
U+34AD.0	fUTF8	㒭
U+34AD.0	fCNS11643	4-2B69
U+34AD.0	fR/S	 10.8
U+34AD.0	fTotalStrokes	 10
U+34AD.0	fCangjie	RURHU
U+34AD.0	fFourCorner	66210
U+34AD.0	fHakka	 kwun1
U+34AD.0	fCantonese	kwan1; kwaan1; gwan1; leoi4
U+34AD.0	fMandarin	kun1; hun2; hun4
U+34AD.0	fEnglish	same as &#x6606;
U+34AE.0	fUTF8	㒮
U+34AE.0	fGB	S-2430
U+34AE.0	fR/S	 10.9
U+34AE.0	fTotalStrokes	 11
U+34AE.0	fCangjie	MUMSO
U+34AE.0	fCantonese	fui1
U+34AF.0	fUTF8	㒯
U+34AF.0	fCNS11643	3-5754
U+34AF.0	fR/S	 10.16; 10.14; 140.14
U+34AF.0	fTotalStrokes	 18
U+34AF.0	fCangjie	FUTMJ
U+34AF.0	fFourCorner	94250
U+34AF.0	fHakka	 jap7
U+34AF.0	fCantonese	jip6
U+34AF.0	fMandarin	ye4; yi4
U+34AF.0	fEnglish	same as &#x71C1;
U+34B0.0	fUTF8	㒰
U+34B0.0	fCNS11643	3-2176
U+34B0.0	fR/S	 11.3
U+34B0.0	fTotalStrokes	 5
U+34B0.0	fCangjie	OM
U+34B0.0	fFourCorner	80100
U+34B0.0	fHakka	 cien2; cion2; tung2
U+34B0.0	fCantonese	cyun4; zan2
U+34B0.0	fMandarin	quan2
U+34B0.0	fEnglish	variant, seal type of &#x5168;
U+34B1.0	fUTF8	㒱
U+34B1.0	fR/S	 11.3
U+34B1.0	fTotalStrokes	 5
U+34B1.0	fCangjie	ONI
U+34B1.0	fCantonese	zik6
U+34B2.0	fUTF8	㒲
U+34B2.0	fCNS11643	3-2279
U+34B2.0	fR/S	 11.4
U+34B2.0	fTotalStrokes	 6
U+34B2.0	fCangjie	OA
U+34B2.0	fFourCorner	80600
U+34B2.0	fHakka	  coi2
U+34B2.0	fCantonese	coi4
U+34B2.0	fMandarin	cai2; zai4
U+34B2.0	fEnglish	ancient form of &#x8CA1;
U+34B3.0	fUTF8	㒳
U+34B3.0	fCNS11643	3-2455
U+34B3.0	fR/S	 11.5
U+34B3.0	fTotalStrokes	 7
U+34B3.0	fCangjie	BLOO
U+34B3.0	fFourCorner	77220
U+34B3.0	fCantonese	loeng5
U+34B3.0	fMandarin	liang3
U+34B3.0	fEnglish	ancient form of &#x5169;
U+34B4.0	fUTF8	㒴
U+34B4.0	fCNS11643	5-2337
U+34B4.0	fR/S	 11.5
U+34B4.0	fTotalStrokes	 7
U+34B4.0	fCangjie	OUU
U+34B4.0	fFourCorner	80770
U+34B4.0	fCantonese	gwat1
U+34B4.0	fMandarin	gu3
U+34B4.0	fEnglish	[1] come [2] to go out
U+34B5.0	fUTF8	㒵
U+34B5.0	fCNS11643	6-257B
U+34B5.0	fR/S	 12.5
U+34B5.0	fTotalStrokes	 7
U+34B5.0	fCangjie	HAC
U+34B5.0	fFourCorner	26800
U+34B5.0	fHakka	 mau5
U+34B5.0	fCantonese	maau6; mau6; mo1
U+34B5.0	fMandarin	mao4
U+34B5.0	fEnglish	corrupted form of &#x7683;
U+34B6.0	fUTF8	㒶
U+34B6.0	fCNS11643	6-257D
U+34B6.0	fR/S	 12.5
U+34B6.0	fTotalStrokes	 7
U+34B6.0	fCangjie	CHA
U+34B6.0	fFourCorner	80600
U+34B6.0	fHakka	 gung1
U+34B6.0	fCantonese	gung1
U+34B6.0	fMandarin	gong1; zhong1
U+34B6.0	fEnglish	ancient form of &#x516C;
U+34B7.0	fUTF8	㒷
U+34B7.0	fCNS11643	3-2456
U+34B7.0	fR/S	 12.5
U+34B7.0	fTotalStrokes	 7
U+34B7.0	fCangjie	BOMC
U+34B7.0	fFourCorner	77800
U+34B7.0	fHakka	 hin1; hin5
U+34B7.0	fCantonese	hing1; hing3; hing2
U+34B7.0	fMandarin	gua3; xing1; xing4; xu3
U+34B7.0	fEnglish	simplified form of &#x8208;
U+34B8.0	fUTF8	㒸
U+34B8.0	fCNS11643	3-2B38
U+34B8.0	fGB	5-313A
U+34B8.0	fR/S	 12.7
U+34B8.0	fTotalStrokes	 9
U+34B8.0	fCangjie	TPHO
U+34B8.0	fFourCorner	80230
U+34B8.0	fHakka	 sui5
U+34B8.0	fCantonese	seoi6
U+34B8.0	fMandarin	sui4; xuan2
U+34B8.0	fEnglish	same as &#x9042;, an ancient form &#x6B72;
U+34B9.0	fUTF8	㒹
U+34B9.0	fCNS11643	7-4D3D
U+34B9.0	fR/S	 12.18
U+34B9.0	fTotalStrokes	 20
U+34B9.0	fCangjie	JCJBC
U+34B9.0	fFourCorner	44880
U+34B9.0	fCantonese	din1
U+34B9.0	fEnglish	non-classical form of &#x985B;
U+34BA.0	fUTF8	㒺
U+34BA.0	fCNS11643	3-2A4C
U+34BA.0	fR/S	 13.6
U+34BA.0	fTotalStrokes	 8
U+34BA.0	fCangjie	BTOV
U+34BA.0	fFourCorner	77220
U+34BA.0	fCantonese	mong5
U+34BA.0	fMandarin	wang3
U+34BB.0	fUTF8	㒻
U+34BB.0	fCNS11643	4-304C
U+34BB.0	fGB	3-483A
U+34BB.0	fR/S	 13.9
U+34BB.0	fTotalStrokes	 11
U+34BB.0	fCangjie	ABUU
U+34BB.0	fFourCorner	60210
U+34BB.0	fHakka	 met8
U+34BB.0	fCantonese	mou6
U+34BB.0	fMandarin	mao4
U+34BB.0	fEnglish	 burst out upon
U+34BC.0	fUTF8	㒼
U+34BC.0	fCNS11643	5-3126
U+34BC.0	fR/S	 13.9
U+34BC.0	fTotalStrokes	 11
U+34BC.0	fCangjie	TLBO
U+34BC.0	fFourCorner	44220
U+34BC.0	fCantonese	mun4; mang4
U+34BC.0	fMandarin	man2
U+34BC.0	fEnglish	[1] average; equivalent; corresponding [2] cover something carefully and tightly without a break
U+34BD.0	fUTF8	㒽
U+34BD.0	fCNS11643	4-363D
U+34BD.0	fGB	3-483C
U+34BD.0	fR/S	 13.10
U+34BD.0	fTotalStrokes	 12
U+34BD.0	fCangjie	AFQU
U+34BD.0	fFourCorner	60710
U+34BD.0	fCantonese	hyun1
U+34BD.0	fMandarin	quan1
U+34BD.0	fEnglish	a kind of cap for children
U+34BE.0	fUTF8	㒾
U+34BE.0	fCNS11643	4-3C32
U+34BE.0	fGB	3-483D
U+34BE.0	fR/S	 13.11; 72.9
U+34BE.0	fTotalStrokes	 13
U+34BE.0	fCangjie	AYSD
U+34BE.0	fFourCorner	60210
U+34BE.0	fCantonese	ci3; si6
U+34BE.0	fMandarin	shi4
U+34BE.0	fEnglish	a kind of wrapper used to cover over the face and head
U+34BF.0	fUTF8	㒿
U+34BF.0	fCNS11643	4-6A42
U+34BF.0	fGB	3-483E
U+34BF.0	fR/S	 13.20; 13.21
U+34BF.0	fTotalStrokes	 23; 22
U+34BF.0	fCangjie	AYBG
U+34BF.0	fFourCorner	60410
U+34BF.0	fHakka	  li2;  li1
U+34BF.0	fCantonese	lei4
U+34BF.0	fMandarin	li2; li4
U+34BF.0	fEnglish	a kind of turban used in ancient time
U+34C0.0	fUTF8	㓀
U+34C0.0	fR/S	 14.2
U+34C0.0	fTotalStrokes	 4
U+34C0.0	fCangjie	BY
U+34C0.0	fCantonese	buk1
U+34C1.0	fUTF8	㓁
U+34C1.0	fR/S	 14.2
U+34C1.0	fTotalStrokes	 4
U+34C1.0	fCangjie	BC
U+34C1.0	fCantonese	mong5
U+34C1.0	fMandarin	wang3
U+34C1.0	fEnglish	a net; net-like, Kangxi radical 122
U+34C2.0	fUTF8	㓂
U+34C2.0	fCNS11643	3-2B3A
U+34C2.0	fR/S	 14.7
U+34C2.0	fTotalStrokes	 9
U+34C2.0	fCangjie	BMUV
U+34C2.0	fFourCorner	37210
U+34C2.0	fHakka	 keu5
U+34C2.0	fCantonese	kau3; kaau3; taau3
U+34C2.0	fMandarin	kou4
U+34C2.0	fEnglish	variant of &#x5BC7;
U+34C3.0	fUTF8	㓃
U+34C3.0	fCNS11643	3-3978
U+34C3.0	fGB	5-3373
U+34C3.0	fR/S	 14.10
U+34C3.0	fTotalStrokes	 12
U+34C3.0	fCangjie	BYRP
U+34C3.0	fFourCorner	37610
U+34C3.0	fCantonese	dou3
U+34C3.0	fMandarin	du4; chui2; zha4
U+34C3.0	fEnglish	standard form of &#x870C;, &#x8A6B;
U+34C4.0	fUTF8	㓄
U+34C4.0	fCNS11643	4-4922
U+34C4.0	fGB	3-3354
U+34C4.0	fR/S	 14.13
U+34C4.0	fTotalStrokes	 15
U+34C4.0	fCangjie	IHLBU
U+34C4.0	fFourCorner	24210
U+34C4.0	fCantonese	zam3; saam2; zaam3
U+34C4.0	fMandarin	zhen4
U+34C4.0	fEnglish	[1] plough; till [2] black color
U+34C5.0	fUTF8	㓅
U+34C5.0	fCNS11643	3-2150
U+34C5.0	fR/S	 15.2
U+34C5.0	fTotalStrokes	 4
U+34C5.0	fCangjie	IMMN
U+34C5.0	fFourCorner	31120
U+34C5.0	fHakka	 tin1; ten1
U+34C5.0	fCantonese	ting1; ding1
U+34C5.0	fMandarin	ting1
U+34C5.0	fEnglish	frozen; cold
U+34C6.0	fUTF8	㓆
U+34C6.0	fGB	S-2239
U+34C6.0	fR/S	 15.4
U+34C6.0	fTotalStrokes	 6
U+34C6.0	fCangjie	IMOP
U+34C6.0	fCantonese	leon4
U+34C7.0	fUTF8	㓇
U+34C7.0	fCNS11643	3-2321
U+34C7.0	fR/S	 15.4
U+34C7.0	fTotalStrokes	 6
U+34C7.0	fCangjie	IMHK
U+34C7.0	fFourCorner	32130
U+34C7.0	fCantonese	juk1
U+34C8.0	fUTF8	㓈
U+34C8.0	fCNS11643	3-2458
U+34C8.0	fR/S	 15.5
U+34C8.0	fTotalStrokes	 7
U+34C8.0	fCangjie	IMMR
U+34C8.0	fFourCorner	31160
U+34C8.0	fCantonese	bing3; bing1
U+34C8.0	fMandarin	bing4; bei4; fei4; ye4
U+34C8.0	fEnglish	the sound of flying or speeding
U+34C9.0	fUTF8	㓉
U+34C9.0	fCNS11643	5-252E
U+34C9.0	fR/S	 15.6
U+34C9.0	fTotalStrokes	 8
U+34C9.0	fCangjie	IMHJR
U+34C9.0	fFourCorner	32160
U+34C9.0	fCantonese	wut6; gwaat3; gwat3; kut3; wun6
U+34C9.0	fMandarin	huo2; huan4; huo4
U+34C9.0	fEnglish	[1] ice; icicles [2] cold; frost [3] crystallized
U+34CA.0	fUTF8	㓊
U+34CA.0	fCNS11643	3-2757
U+34CA.0	fR/S	 15.6
U+34CA.0	fTotalStrokes	 8
U+34CA.0	fCangjie	IMBMR
U+34CA.0	fFourCorner	37120
U+34CA.0	fCantonese	dung6
U+34CA.0	fMandarin	dong4
U+34CA.0	fEnglish	[1] cold [2] indifferent
U+34CB.0	fUTF8	㓋
U+34CB.0	fCNS11643	3-2756
U+34CB.0	fR/S	 15.6
U+34CB.0	fTotalStrokes	 8
U+34CB.0	fCangjie	IMTC
U+34CB.0	fFourCorner	34180
U+34CB.0	fCantonese	gung6
U+34CB.0	fMandarin	gong3; gong4
U+34CB.0	fEnglish	 freeze; congeal; curdle
U+34CC.0	fUTF8	㓌
U+34CC.0	fCNS11643	3-2758
U+34CC.0	fR/S	 15.6
U+34CC.0	fTotalStrokes	 8
U+34CC.0	fCangjie	IMOMG
U+34CC.0	fFourCorner	38110
U+34CC.0	fCantonese	cang1; gwai3; ting4; tong3
U+34CC.0	fMandarin	cheng1
U+34CC.0	fEnglish	bitterly cold
U+34CD.0	fUTF8	㓍
U+34CD.0	fGB	S-2238
U+34CD.0	fR/S	 15.6
U+34CD.0	fTotalStrokes	 8
U+34CD.0	fCangjie	IMYIU
U+34CD.0	fCantonese	cung1
U+34CE.0	fUTF8	㓎
U+34CE.0	fCNS11643	4-2843
U+34CE.0	fGB	3-325E
U+34CE.0	fR/S	 15.7
U+34CE.0	fTotalStrokes	 9
U+34CE.0	fCangjie	IMSME
U+34CE.0	fFourCorner	37140
U+34CE.0	fCantonese	cam1; caam1; saam6
U+34CE.0	fMandarin	qin1; lian2; qin3; qin4
U+34CE.0	fEnglish	[1] cold; bitterly cold [2] cold air
U+34CF.0	fUTF8	㓏
U+34CF.0	fCNS11643	3-2B3C
U+34CF.0	fR/S	 15.7
U+34CF.0	fTotalStrokes	 9
U+34CF.0	fCangjie	IMBCR
U+34CF.0	fFourCorner	37120
U+34CF.0	fCantonese	gwing2
U+34CF.0	fMandarin	jiong3
U+34CF.0	fEnglish	bitterly cold
U+34D0.0	fUTF8	㓐
U+34D0.0	fCNS11643	3-2F5D
U+34D0.0	fR/S	 15.8
U+34D0.0	fTotalStrokes	 10
U+34D0.0	fCangjie	IMGCG
U+34D0.0	fFourCorner	34110
U+34D0.0	fHakka	 liuk7
U+34D0.0	fCantonese	luk6
U+34D0.0	fMandarin	lu4
U+34D0.0	fEnglish	sleet; hail
U+34D1.0	fUTF8	㓑
U+34D1.0	fCNS11643	3-2F5C
U+34D1.0	fR/S	 15.8
U+34D1.0	fTotalStrokes	 10
U+34D1.0	fCangjie	IMGTJ
U+34D1.0	fFourCorner	34140
U+34D1.0	fCantonese	jing6; haang6
U+34D1.0	fMandarin	xing4
U+34D1.0	fEnglish	cold
U+34D2.0	fUTF8	㓒
U+34D2.0	fR/S	 15.8
U+34D2.0	fTotalStrokes	 10
U+34D2.0	fCangjie	IPOP
U+34D2.0	fCantonese	cat1
U+34D3.0	fUTF8	㓓
U+34D3.0	fCNS11643	4-304D
U+34D3.0	fR/S	 15.9
U+34D3.0	fTotalStrokes	 11
U+34D3.0	fCangjie	IMJBJ
U+34D3.0	fFourCorner	34120
U+34D3.0	fCantonese	naam4; laam4 (also); lam5; nam4
U+34D3.0	fMandarin	nan2
U+34D3.0	fEnglish	thin ice
U+34D4.0	fUTF8	㓔
U+34D4.0	fCNS11643	3-397B
U+34D4.0	fR/S	 15.10
U+34D4.0	fTotalStrokes	 12
U+34D4.0	fCangjie	IMHHI
U+34D4.0	fFourCorner	34100
U+34D4.0	fCantonese	ze6
U+34D4.0	fMandarin	xie4
U+34D4.0	fEnglish	same as &#x8B1D;
U+34D5.0	fUTF8	㓕
U+34D5.0	fCNS11643	3-397C
U+34D5.0	fR/S	 15.10
U+34D5.0	fTotalStrokes	 12
U+34D5.0	fCangjie	IMIHF
U+34D5.0	fFourCorner	33150
U+34D5.0	fCantonese	mit6
U+34D5.0	fMandarin	mie4
U+34D6.0	fUTF8	㓖
U+34D6.0	fCNS11643	4-3C34
U+34D6.0	fGB	5-3365
U+34D6.0	fR/S	 15.11
U+34D6.0	fTotalStrokes	 13
U+34D6.0	fCangjie	IMWTJ
U+34D6.0	fFourCorner	36150
U+34D6.0	fHakka	  bit7
U+34D6.0	fCantonese	bat1; bit1
U+34D6.0	fMandarin	bi4; bie2
U+34D6.0	fEnglish	a cold wind
U+34D7.0	fUTF8	㓗
U+34D7.0	fCNS11643	3-4579
U+34D7.0	fR/S	 15.12
U+34D7.0	fTotalStrokes	 14
U+34D7.0	fCangjie	IMQHF
U+34D7.0	fFourCorner	37190
U+34D7.0	fHakka	 giet7
U+34D7.0	fCantonese	git3
U+34D7.0	fMandarin	jie2; qi3
U+34D7.0	fEnglish	non-classical form of &#x6F54;
U+34D8.0	fUTF8	㓘
U+34D8.0	fCNS11643	5-3129
U+34D8.0	fGB	3-3253
U+34D8.0	fR/S	 16.9
U+34D8.0	fTotalStrokes	 11
U+34D8.0	fCangjie	HNMNI
U+34D8.0	fFourCorner	77210
U+34D8.0	fCantonese	suk1
U+34D8.0	fMandarin	su4
U+34D8.0	fEnglish	same as &#x738A;
U+34D9.0	fUTF8	㓙
U+34D9.0	fR/S	 17.4
U+34D9.0	fTotalStrokes	 6
U+34D9.0	fCangjie	UYK
U+34D9.0	fCantonese	hung1; kung4
U+34D9.0	fEnglish	non-classical form of &#x51F6;
U+34DA.0	fUTF8	㓚
U+34DA.0	fCNS11643	4-2161
U+34DA.0	fGB	3-3360
U+34DA.0	fR/S	 18.3
U+34DA.0	fTotalStrokes	 5
U+34DA.0	fCangjie	MLN
U+34DA.0	fFourCorner	12100
U+34DA.0	fCantonese	gung1; geoi6; kung4
U+34DA.0	fMandarin	gong1
U+34DA.0	fEnglish	[1] sickle, reaping-hook [2] gether in the harvest [3] pare; scrape
U+34DB.0	fUTF8	㓛
U+34DB.0	fCNS11643	6-2241
U+34DB.0	fR/S	 18.3
U+34DB.0	fTotalStrokes	 5
U+34DB.0	fCangjie	MSH
U+34DB.0	fFourCorner	17120
U+34DB.0	fHakka	 gung1
U+34DB.0	fCantonese	gung1
U+34DB.0	fMandarin	gong1
U+34DB.0	fEnglish	corrupted form of &#x529F;
U+34DC.0	fUTF8	㓜
U+34DC.0	fCNS11643	3-217E
U+34DC.0	fR/S	 18.3
U+34DC.0	fTotalStrokes	 5
U+34DC.0	fCangjie	VISH
U+34DC.0	fFourCorner	27720
U+34DC.0	fHakka	 ju5
U+34DC.0	fCantonese	jau3; zaau3; ziu3
U+34DC.0	fMandarin	you4; yao4
U+34DC.0	fEnglish	corrupt form of &#x5E7C;
U+34DD.0	fUTF8	㓝
U+34DD.0	fCNS11643	4-223C
U+34DD.0	fGB	5-342F
U+34DD.0	fR/S	 18.4
U+34DD.0	fTotalStrokes	 6
U+34DD.0	fCangjie	TTLN
U+34DD.0	fFourCorner	52000
U+34DD.0	fHakka	 hin2
U+34DD.0	fCantonese	jing4; zing4
U+34DD.0	fMandarin	xing2
U+34DD.0	fEnglish	same as &#x5211;
U+34DE.0	fUTF8	㓞
U+34DE.0	fCNS11643	5-2230
U+34DE.0	fGB	5-337E
U+34DE.0	fR/S	 18.4
U+34DE.0	fTotalStrokes	 6
U+34DE.0	fCangjie	QJSH
U+34DE.0	fFourCorner	57000
U+34DE.0	fHakka	 giet7; k'e5; kat7
U+34DE.0	fCantonese	kit3; kaai3
U+34DE.0	fMandarin	jie2; qi4; qia4; yao2
U+34DE.0	fEnglish	 cut a notch in a stick
U+34DF.0	fUTF8	㓟
U+34DF.0	fCNS11643	4-233E
U+34DF.0	fGB	5-5E65
U+34DF.0	fR/S	 18.5
U+34DF.0	fTotalStrokes	 7
U+34DF.0	fCangjie	DELN
U+34DF.0	fFourCorner	42200
U+34DF.0	fHakka	 pai1;  pi1
U+34DF.0	fCantonese	pai1; ngo4; pei1
U+34DF.0	fMandarin	pi1
U+34DF.0	fEnglish	peel; trim; pare
U+34E0.0	fUTF8	㓠
U+34E0.0	fCNS11643	4-233F
U+34E0.0	fGB	5-3435
U+34E0.0	fR/S	 18.5
U+34E0.0	fTotalStrokes	 7
U+34E0.0	fCangjie	YRLN
U+34E0.0	fFourCorner	22600
U+34E0.0	fHakka	  diam5; diam3
U+34E0.0	fCantonese	dim3
U+34E0.0	fMandarin	dian4
U+34E0.0	fEnglish	 nick in a blade; flaw
U+34E0.1	fUTF8	㓠
U+34E0.1	fCNS11643	4-233F
U+34E0.1	fGB	5-3435
U+34E0.1	fR/S	 18.5
U+34E0.1	fTotalStrokes	 7
U+34E0.1	fCangjie	YRLN
U+34E0.1	fFourCorner	22600
U+34E0.1	fMandarin	zhang1
U+34E1.0	fUTF8	㓡
U+34E1.0	fCNS11643	4-254A
U+34E1.0	fGB	3-6337
U+34E1.0	fR/S	 18.6
U+34E1.0	fTotalStrokes	 8; 7
U+34E1.0	fCangjie	OULN
U+34E1.0	fFourCorner	82700
U+34E1.0	fCantonese	fu2
U+34E1.0	fMandarin	fou3; fu3
U+34E1.0	fEnglish	same as &#x5F23;
U+34E2.0	fUTF8	㓢
U+34E2.0	fCNS11643	4-254C
U+34E2.0	fGB	3-327B
U+34E2.0	fR/S	 18.6
U+34E2.0	fTotalStrokes	 8; 7
U+34E2.0	fCangjie	HRLN
U+34E2.0	fFourCorner	72600
U+34E2.0	fHakka	 lok8
U+34E2.0	fCantonese	lok6; lok3
U+34E2.0	fMandarin	luo2; luo4
U+34E2.0	fEnglish	 trim the knots of a branch
U+34E3.0	fUTF8	㓣
U+34E3.0	fCNS11643	4-2548
U+34E3.0	fGB	3-327A
U+34E3.0	fR/S	 18.6
U+34E3.0	fTotalStrokes	 8; 7
U+34E3.0	fCangjie	ORLN
U+34E3.0	fFourCorner	82600
U+34E3.0	fHakka	 kap7
U+34E3.0	fCantonese	hap1; got3; zak6
U+34E3.0	fMandarin	qia4
U+34E3.0	fEnglish	 pierce; to cut
U+34E4.0	fUTF8	㓤
U+34E4.0	fCNS11643	4-2547
U+34E4.0	fGB	3-3278
U+34E4.0	fR/S	 18.6
U+34E4.0	fTotalStrokes	 8; 7
U+34E4.0	fCangjie	GRLN
U+34E4.0	fFourCorner	42600
U+34E4.0	fHakka	  kat7;  giet7; kiak7; get7
U+34E4.0	fCantonese	gat1; kit3 (also); hot3; kit6
U+34E4.0	fMandarin	jia2; jie2; qia4
U+34E4.0	fEnglish	brand the face of an criminal; strip the skin of the face
U+34E5.0	fUTF8	㓥
U+34E5.0	fGB	S-2264
U+34E5.0	fR/S	 18.6
U+34E5.0	fTotalStrokes	 8
U+34E5.0	fCangjie	FSLN
U+34E5.0	fCantonese	tong1
U+34E6.0	fUTF8	㓦
U+34E6.0	fGB	3-3279
U+34E6.0	fR/S	 18.6
U+34E6.0	fTotalStrokes	 8
U+34E6.0	fCangjie	MALN
U+34E6.0	fCantonese	maak3; baak3 (also); bak3
U+34E6.0	fEnglish	deliberate and plan
U+34E7.0	fUTF8	㓧
U+34E7.0	fCNS11643	3-2B41
U+34E7.0	fR/S	 18.7
U+34E7.0	fTotalStrokes	 9
U+34E7.0	fCangjie	ORLN
U+34E7.0	fFourCorner	82600
U+34E7.0	fCantonese	gam1; deoi1; haang1; zim1; zim2
U+34E7.0	fMandarin	gan1; yan3
U+34E7.0	fEnglish	agricultural implements
U+34E8.0	fUTF8	㓨
U+34E8.0	fCNS11643	3-2B45
U+34E8.0	fGB	5-343D
U+34E8.0	fR/S	 18.7
U+34E8.0	fTotalStrokes	 9
U+34E8.0	fCangjie	KOLN
U+34E8.0	fFourCorner	42200
U+34E8.0	fHakka	 cih5; ciuk7; cu5
U+34E8.0	fCantonese	ci3; cik3 (also)
U+34E8.0	fMandarin	ci4; ci2; ci1; qi4; ji4
U+34E8.0	fEnglish	same as &#x523A;
U+34E9.0	fUTF8	㓩
U+34E9.0	fCNS11643	3-2B42
U+34E9.0	fR/S	 18.7
U+34E9.0	fTotalStrokes	 9
U+34E9.0	fCangjie	NBLN
U+34E9.0	fFourCorner	22200
U+34E9.0	fCantonese	hyun1; fan3; hin2
U+34E9.0	fMandarin	xuan1
U+34E9.0	fEnglish	horned hair
U+34EA.0	fUTF8	㓪
U+34EA.0	fR/S	 18.7
U+34EA.0	fTotalStrokes	 9
U+34EA.0	fCangjie	IVSH
U+34EA.0	fFourCorner	3772
U+34EA.0	fCantonese	long5
U+34EA.0	fMandarin	lang3
U+34EA.0	fEnglish	same as &#x6717;
U+34EB.0	fUTF8	㓫
U+34EB.0	fGB	S-2262
U+34EB.0	fR/S	 18.7
U+34EB.0	fTotalStrokes	 9
U+34EB.0	fCangjie	KTLN
U+34EB.0	fCantonese	fu1
U+34EC.0	fUTF8	㓬
U+34EC.0	fCNS11643	5-2B7C
U+34EC.0	fR/S	 18.8
U+34EC.0	fTotalStrokes	 10
U+34EC.0	fCangjie	FHLN
U+34EC.0	fFourCorner	92200
U+34EC.0	fCantonese	fu1
U+34EC.0	fMandarin	ku1
U+34EC.0	fEnglish	same as &#x5233;
U+34ED.0	fUTF8	㓭
U+34ED.0	fCNS11643	4-3053
U+34ED.0	fR/S	 18.9; 18.8
U+34ED.0	fTotalStrokes	 11; 10
U+34ED.0	fCangjie	AVLN
U+34ED.0	fFourCorner	62700
U+34ED.0	fCantonese	hot3
U+34ED.0	fMandarin	she2; he2
U+34ED.0	fEnglish	same as &#x97A8;
U+34EE.0	fUTF8	㓮
U+34EE.0	fCNS11643	3-2F67
U+34EE.0	fR/S	 18.8
U+34EE.0	fTotalStrokes	 10
U+34EE.0	fCangjie	BRLN
U+34EE.0	fFourCorner	72200
U+34EE.0	fHakka	 diau1;  tiau1
U+34EE.0	fCantonese	diu1
U+34EE.0	fMandarin	diao1
U+34EE.0	fEnglish	same as &#x5F6B;
U+34EF.0	fUTF8	㓯
U+34EF.0	fCNS11643	3-2F69
U+34EF.0	fR/S	 18.8
U+34EF.0	fTotalStrokes	 10
U+34EF.0	fCangjie	VWLN
U+34EF.0	fFourCorner	22600
U+34EF.0	fCantonese	lei4
U+34EF.0	fMandarin	li4; li2
U+34EF.0	fEnglish	same as &#x5299;
U+34F0.0	fUTF8	㓰
U+34F0.0	fR/S	 18.8
U+34F0.0	fTotalStrokes	 10
U+34F0.0	fCangjie	MULN
U+34F0.0	fCantonese	waak6; wak6
U+34F0.0	fMandarin	hua4
U+34F0.0	fEnglish	variant of &#x5283;
U+34F1.0	fUTF8	㓱
U+34F1.0	fCNS11643	4-3057
U+34F1.0	fGB	3-3326
U+34F1.0	fR/S	 18.9
U+34F1.0	fTotalStrokes	 11
U+34F1.0	fCangjie	ONLN
U+34F1.0	fFourCorner	82200
U+34F1.0	fCantonese	tau4; syu1
U+34F1.0	fMandarin	tou2
U+34F1.0	fEnglish	[1] cut; cut out [2] pick out [3] scoop out [4] same as &#x9103;
U+34F2.0	fUTF8	㓲
U+34F2.0	fCNS11643	4-3058
U+34F2.0	fGB	3-3328
U+34F2.0	fR/S	 18.9
U+34F2.0	fTotalStrokes	 11
U+34F2.0	fCangjie	IBLN
U+34F2.0	fFourCorner	32200
U+34F2.0	fHakka	  pien1; pen3
U+34F2.0	fCantonese	pin1; pin3
U+34F2.0	fMandarin	pian1; pian4
U+34F2.0	fEnglish	[1] cut in slices [2] pare
U+34F3.0	fUTF8	㓳
U+34F3.0	fCNS11643	4-3052
U+34F3.0	fGB	3-3324
U+34F3.0	fR/S	 18.9
U+34F3.0	fTotalStrokes	 11
U+34F3.0	fCangjie	AOLN
U+34F3.0	fFourCorner	62800
U+34F3.0	fCantonese	dai1
U+34F3.0	fMandarin	di1
U+34F3.0	fEnglish	use a knife to split something
U+34F4.0	fUTF8	㓴
U+34F4.0	fCNS11643	5-312C
U+34F4.0	fGB	5-3452
U+34F4.0	fR/S	 18.9
U+34F4.0	fTotalStrokes	 11
U+34F4.0	fCangjie	MKLN
U+34F4.0	fFourCorner	12400
U+34F4.0	fCantonese	jyun5; zyun5
U+34F4.0	fMandarin	ruan3; jun4
U+34F4.0	fEnglish	 cut in two; slash
U+34F5.0	fUTF8	㓵
U+34F5.0	fCNS11643	4-3054
U+34F5.0	fGB	5-3453
U+34F5.0	fR/S	 18.9
U+34F5.0	fTotalStrokes	 11
U+34F5.0	fCangjie	RSLN
U+34F5.0	fFourCorner	62200
U+34F5.0	fCantonese	ngok6; ok6 (also)
U+34F5.0	fMandarin	e4
U+34F5.0	fEnglish	same as &#x9354;
U+34F6.0	fUTF8	㓶
U+34F6.0	fCNS11643	4-3059
U+34F6.0	fGB	5-344F
U+34F6.0	fR/S	 18.9
U+34F6.0	fTotalStrokes	 11
U+34F6.0	fCangjie	QKLN
U+34F6.0	fFourCorner	52400
U+34F6.0	fCantonese	kit3
U+34F6.0	fMandarin	qie4
U+34F6.0	fEnglish	same as &#x9365;
U+34F7.0	fUTF8	㓷
U+34F7.0	fCNS11643	3-3A24
U+34F7.0	fGB	5-345C
U+34F7.0	fR/S	 18.10
U+34F7.0	fTotalStrokes	 12
U+34F7.0	fCangjie	HDLN
U+34F7.0	fFourCorner	22900
U+34F7.0	fHakka	  ngi5
U+34F7.0	fCantonese	ji6; nau4; zi6
U+34F7.0	fMandarin	yi4
U+34F7.0	fEnglish	 cut off the nose (a punishment in the Chow dynasty)
U+34F8.0	fUTF8	㓸
U+34F8.0	fCNS11643	6-506C
U+34F8.0	fR/S	 18.10
U+34F8.0	fTotalStrokes	 13; 12
U+34F8.0	fCangjie	RMLN
U+34F8.0	fFourCorner	72100
U+34F8.0	fHakka	 dok8
U+34F8.0	fCantonese	zoek3; coek3; sau2
U+34F8.0	fMandarin	zhuo2; jue2
U+34F8.0	fEnglish	same as &#x65B2;
U+34F9.0	fUTF8	㓹
U+34F9.0	fCNS11643	4-3642
U+34F9.0	fGB	3-332B
U+34F9.0	fR/S	 18.10
U+34F9.0	fTotalStrokes	 12
U+34F9.0	fCangjie	MFLN
U+34F9.0	fFourCorner	72200
U+34F9.0	fCantonese	jeoi6; zeoi6
U+34F9.0	fMandarin	cui4; rui4
U+34F9.0	fEnglish	same as &#x92B3;
U+34FA.0	fUTF8	㓺
U+34FA.0	fCNS11643	4-3645
U+34FA.0	fGB	3-332C
U+34FA.0	fR/S	 18.10
U+34FA.0	fTotalStrokes	 12
U+34FA.0	fCangjie	YKLN
U+34FA.0	fFourCorner	22200
U+34FA.0	fHakka	 kien2; ken2
U+34FA.0	fCantonese	kin4; gin1; gin6
U+34FA.0	fMandarin	jian1; jian4
U+34FA.0	fEnglish	[1] cut [2] castrate an ox
U+34FB.0	fUTF8	㓻
U+34FB.0	fCNS11643	3-3A22
U+34FB.0	fGB	5-345B
U+34FB.0	fR/S	 18.10
U+34FB.0	fTotalStrokes	 12
U+34FB.0	fCangjie	WMLN
U+34FB.0	fFourCorner	62100
U+34FB.0	fHakka	 gong1
U+34FB.0	fCantonese	gong1
U+34FB.0	fMandarin	gang1
U+34FB.0	fEnglish	same as &#x525B;
U+34FC.0	fUTF8	㓼
U+34FC.0	fCNS11643	5-3D70
U+34FC.0	fGB	3-332E
U+34FC.0	fR/S	 18.11
U+34FC.0	fTotalStrokes	 13
U+34FC.0	fCangjie	DELN
U+34FC.0	fFourCorner	42100
U+34FC.0	fCantonese	cat1
U+34FC.0	fMandarin	chi3; chi4; chong4
U+34FC.0	fEnglish	incised wound; cuts
U+34FD.0	fUTF8	㓽
U+34FD.0	fCNS11643	3-4042
U+34FD.0	fGB	5-3461
U+34FD.0	fR/S	 18.11
U+34FD.0	fTotalStrokes	 13
U+34FD.0	fCangjie	UFLN
U+34FD.0	fFourCorner	22900
U+34FD.0	fCantonese	sung4
U+34FD.0	fMandarin	chong2
U+34FD.0	fEnglish	a kind of farm tool; iron spade; spade
U+34FE.0	fUTF8	㓾
U+34FE.0	fCNS11643	4-4241
U+34FE.0	fGB	3-3331
U+34FE.0	fR/S	 18.12
U+34FE.0	fTotalStrokes	 14
U+34FE.0	fCangjie	SQLN
U+34FE.0	fFourCorner	72200
U+34FE.0	fHakka	  chi2; cih2; chii2
U+34FE.0	fCantonese	sai1
U+34FE.0	fMandarin	xi1; chi2
U+34FE.0	fEnglish	[1] slaughter for food [2] cut open [3] dismember
U+34FF.0	fUTF8	㓿
U+34FF.0	fR/S	 18.12
U+34FF.0	fTotalStrokes	 14
U+34FF.0	fCangjie	HELN
U+34FF.0	fCantonese	syu2
U+3500.0	fUTF8	㔀
U+3500.0	fCNS11643	4-4243
U+3500.0	fGB	5-3463
U+3500.0	fR/S	 18.12
U+3500.0	fTotalStrokes	 14
U+3500.0	fCangjie	AFLN
U+3500.0	fFourCorner	62900
U+3500.0	fCantonese	king4
U+3500.0	fMandarin	liang4; le4; qing2
U+3500.0	fEnglish	[1] (non-classical form) brand criminals on the face [2] rob; plunder
U+3501.0	fUTF8	㔁
U+3501.0	fCNS11643	4-4242
U+3501.0	fR/S	 18.12
U+3501.0	fTotalStrokes	 14
U+3501.0	fCangjie	NTLN
U+3501.0	fFourCorner	12100
U+3501.0	fCantonese	dang1; daang1; taang1
U+3501.0	fMandarin	deng1
U+3501.0	fEnglish	a kind of tool; hook; barb; sickle
U+3502.0	fUTF8	㔂
U+3502.0	fCNS11643	4-423C
U+3502.0	fGB	3-3330
U+3502.0	fR/S	 18.12
U+3502.0	fTotalStrokes	 14
U+3502.0	fCangjie	FQLN
U+3502.0	fFourCorner	92200
U+3502.0	fHakka	  lin2
U+3502.0	fCantonese	leon4
U+3502.0	fMandarin	lin2
U+3502.0	fEnglish	 pare or pale fruit
U+3503.0	fUTF8	㔃
U+3503.0	fCNS11643	3-457D
U+3503.0	fGB	5-3426
U+3503.0	fR/S	 18.12
U+3503.0	fTotalStrokes	 14
U+3503.0	fCangjie	VUSH
U+3503.0	fFourCorner	27227
U+3503.0	fCantonese	zyut6
U+3503.0	fMandarin	jue2; pi4
U+3503.0	fEnglish	same as &#x7D55;
U+3504.0	fUTF8	㔄
U+3504.0	fCNS11643	3-457B
U+3504.0	fR/S	 18.12
U+3504.0	fTotalStrokes	 14
U+3504.0	fCangjie	MDLN
U+3504.0	fFourCorner	12900
U+3504.0	fHakka	 siuk8; siuk7
U+3504.0	fCantonese	suk1
U+3504.0	fMandarin	su4
U+3504.0	fEnglish	 mince
U+3505.0	fUTF8	㔅
U+3505.0	fCNS11643	4-423F
U+3505.0	fGB	3-3322
U+3505.0	fR/S	 18.12
U+3505.0	fTotalStrokes	 14
U+3505.0	fCangjie	LXLN
U+3505.0	fFourCorner	52200
U+3505.0	fCantonese	siu3
U+3505.0	fMandarin	xiao4
U+3505.0	fEnglish	[1] cut; hack; reap [2] minced meat mixed up with ginger and cassia
U+3506.0	fUTF8	㔆
U+3506.0	fCNS11643	4-423E
U+3506.0	fGB	3-332F
U+3506.0	fR/S	 18.12
U+3506.0	fTotalStrokes	 14
U+3506.0	fCangjie	MALN
U+3506.0	fFourCorner	72600
U+3506.0	fCantonese	caam5; caam4
U+3506.0	fMandarin	can3; chan2
U+3506.0	fEnglish	[1] cut; hack; reap; trim; clip [2] stab; pierce; brand
U+3507.0	fUTF8	㔇
U+3507.0	fCNS11643	3-457C
U+3507.0	fGB	5-343B
U+3507.0	fR/S	 18.12
U+3507.0	fTotalStrokes	 14
U+3507.0	fCangjie	NELN
U+3507.0	fFourCorner	12200
U+3507.0	fHakka	  pat7
U+3507.0	fCantonese	put3
U+3507.0	fMandarin	po1
U+3507.0	fEnglish	knife to cut grass
U+3508.0	fUTF8	㔈
U+3508.0	fGB	S-2263
U+3508.0	fR/S	 18.12
U+3508.0	fTotalStrokes	 14
U+3508.0	fCangjie	TYLN
U+3508.0	fCantonese	fei6
U+3509.0	fUTF8	㔉
U+3509.0	fGB	S-2267
U+3509.0	fR/S	 18.12
U+3509.0	fTotalStrokes	 14
U+3509.0	fCangjie	SBLN
U+3509.0	fCantonese	zuk1; cung2
U+3509.0	fMandarin	zhu2
U+3509.0	fEnglish	dig; excavate
U+350A.0	fUTF8	㔊
U+350A.0	fCNS11643	4-4923
U+350A.0	fGB	3-3333
U+350A.0	fR/S	 18.13
U+350A.0	fTotalStrokes	 15
U+350A.0	fCangjie	YMLN
U+350A.0	fFourCorner	02100
U+350A.0	fCantonese	zin2; tan2
U+350A.0	fMandarin	zhan3; dan3
U+350A.0	fEnglish	[1] cut; hack; reap [2] strike; attack [3] chastise rebels
U+350B.0	fUTF8	㔋
U+350B.0	fCNS11643	4-4F62
U+350B.0	fGB	3-332D
U+350B.0	fR/S	 18.14
U+350B.0	fTotalStrokes	 16
U+350B.0	fCangjie	STLN
U+350B.0	fFourCorner	12100
U+350B.0	fCantonese	gaam3; ham3; ham5; lam6
U+350B.0	fMandarin	jian4; lan2; lan4
U+350B.0	fEnglish	[1] cut into thin slices; mince [2] sharp
U+350C.0	fUTF8	㔌
U+350C.0	fCNS11643	4-4F61
U+350C.0	fGB	3-3334
U+350C.0	fR/S	 18.14
U+350C.0	fTotalStrokes	 16
U+350C.0	fCangjie	SOLN
U+350C.0	fFourCorner	12820
U+350C.0	fHakka	 ceu3
U+350C.0	fCantonese	zau6; sei3; tam3; zap3; zek3; zik3
U+350C.0	fMandarin	zou4; zhou4
U+350C.0	fEnglish	 mince; cut up small
U+350D.0	fUTF8	㔍
U+350D.0	fCNS11643	4-4F63
U+350D.0	fGB	3-3335
U+350D.0	fR/S	 18.14
U+350D.0	fTotalStrokes	 16
U+350D.0	fCangjie	HTLN
U+350D.0	fFourCorner	82400
U+350D.0	fHakka	 zap7
U+350D.0	fCantonese	caat3; cap3; zaa3; zaai3
U+350D.0	fMandarin	chan3; duo3; zha2
U+350D.0	fEnglish	a lever knife for cutting up fodder etc.
U+350E.0	fUTF8	㔎
U+350E.0	fCNS11643	5-613B
U+350E.0	fR/S	 18.15; 18.17
U+350E.0	fTotalStrokes	 18; 17; 19
U+350E.0	fCangjie	UJSH
U+350E.0	fFourCorner	40227
U+350E.0	fCantonese	sit3
U+350E.0	fMandarin	qi1; xie4; yi4
U+350E.0	fEnglish	 cut apart; divide
U+350F.0	fUTF8	㔏
U+350F.0	fCNS11643	4-556C
U+350F.0	fGB	3-3336
U+350F.0	fR/S	 18.15
U+350F.0	fTotalStrokes	 17; 16
U+350F.0	fCangjie	MRLN
U+350F.0	fFourCorner	12600
U+350F.0	fCantonese	lik6; gak3
U+350F.0	fMandarin	li4
U+350F.0	fEnglish	same as &#x5246;
U+3510.0	fUTF8	㔐
U+3510.0	fCNS11643	7-3E7D
U+3510.0	fR/S	 18.16
U+3510.0	fTotalStrokes	 18
U+3510.0	fCangjie	HMLN
U+3510.0	fFourCorner	82100
U+3510.0	fHakka	 ciam1
U+3510.0	fCantonese	cim1
U+3510.0	fMandarin	qian1
U+3510.0	fEnglish	corrupted form of &#x255360;
U+3511.0	fUTF8	㔑
U+3511.0	fCNS11643	4-5F4A
U+3511.0	fGB	5-3428
U+3511.0	fR/S	 18.17
U+3511.0	fTotalStrokes	 19
U+3511.0	fCangjie	TJSH
U+3511.0	fFourCorner	44220
U+3511.0	fCantonese	cat1
U+3511.0	fMandarin	ce3; chi4; dao4; qi4; shui4
U+3511.0	fEnglish	same as &#x34FC;
U+3512.0	fUTF8	㔒
U+3512.0	fCNS11643	4-6330
U+3512.0	fGB	3-3338
U+3512.0	fR/S	 18.18
U+3512.0	fTotalStrokes	 20
U+3512.0	fCangjie	UBLN
U+3512.0	fFourCorner	22200
U+3512.0	fCantonese	kwai4
U+3512.0	fMandarin	xi2
U+3512.0	fEnglish	[1] cut; cut out [2] pick out [3] scoop out [4] cut short [5] spare
U+3513.0	fUTF8	㔓
U+3513.0	fCNS11643	4-2164
U+3513.0	fGB	3-333B
U+3513.0	fR/S	 19.3
U+3513.0	fTotalStrokes	 5
U+3513.0	fCangjie	HJKS
U+3513.0	fFourCorner	24420
U+3513.0	fCantonese	gin2; zin2
U+3513.0	fMandarin	jian3; xian4
U+3513.0	fEnglish	 stammer
U+3514.0	fUTF8	㔔
U+3514.0	fR/S	 19.4
U+3514.0	fTotalStrokes	 6
U+3514.0	fCangjie	KRR
U+3514.0	fCantonese	gaa1
U+3515.0	fUTF8	㔕
U+3515.0	fCNS11643	4-2240
U+3515.0	fR/S	 19.4
U+3515.0	fTotalStrokes	 6
U+3515.0	fCangjie	ONKS
U+3515.0	fFourCorner	84720
U+3515.0	fCantonese	gat6; gaat1; kaau1
U+3515.0	fMandarin	ji2; yi4
U+3515.0	fEnglish	with purpose; with ambition; with the determination and courage to get ahead
U+3516.0	fUTF8	㔖
U+3516.0	fR/S	 19.4
U+3516.0	fTotalStrokes	 6
U+3516.0	fCangjie	KRS
U+3516.0	fCantonese	gaa1
U+3517.0	fUTF8	㔗
U+3517.0	fCNS11643	4-2344
U+3517.0	fGB	3-333E
U+3517.0	fR/S	 19.5
U+3517.0	fTotalStrokes	 7
U+3517.0	fCangjie	LNKS
U+3517.0	fFourCorner	54020
U+3517.0	fCantonese	fai6; bei6; faai3; fan4; gaat1; ngaat6
U+3517.0	fMandarin	fei4
U+3517.0	fEnglish	[1] strong; healthy [2] clasp under the arm [3] persecute; oppress [4] vigorous
U+3518.0	fUTF8	㔘
U+3518.0	fCNS11643	3-2464
U+3518.0	fR/S	 19.5
U+3518.0	fTotalStrokes	 7
U+3518.0	fCangjie	UUKS
U+3518.0	fFourCorner	24720
U+3518.0	fCantonese	ceot1; zeot1
U+3518.0	fMandarin	chu4
U+3518.0	fEnglish	same as &#x9EDC;
U+3519.0	fUTF8	㔙
U+3519.0	fCNS11643	4-254F
U+3519.0	fGB	3-3341
U+3519.0	fR/S	 19.6
U+3519.0	fTotalStrokes	 8; 7
U+3519.0	fCangjie	TTKS
U+3519.0	fFourCorner	84420
U+3519.0	fCantonese	paang1; ping1
U+3519.0	fMandarin	peng1; bang3; peng2
U+3519.0	fEnglish	great strength; great power
U+351A.0	fUTF8	㔚
U+351A.0	fCNS11643	6-296F
U+351A.0	fR/S	 19.6
U+351A.0	fTotalStrokes	 8
U+351A.0	fCangjie	HRKS
U+351A.0	fFourCorner	24620
U+351A.0	fCantonese	hau2; aau2; haau2
U+351A.0	fMandarin	kou3
U+351A.0	fEnglish	corrupted form of &#x52B6;
U+351B.0	fUTF8	㔛
U+351B.0	fR/S	 19.6
U+351B.0	fTotalStrokes	 8
U+351B.0	fCangjie	GRKS
U+351B.0	fCantonese	kit3
U+351C.0	fUTF8	㔜
U+351C.0	fCNS11643	6-2F27
U+351C.0	fR/S	 19.7
U+351C.0	fTotalStrokes	 9
U+351C.0	fCangjie	BDKS
U+351C.0	fFourCorner	24420
U+351C.0	fHakka	 put8
U+351C.0	fCantonese	but6
U+351C.0	fMandarin	bo2; ba2
U+351C.0	fEnglish	same as &#x52C3;
U+351D.0	fUTF8	㔝
U+351D.0	fCNS11643	4-2B75
U+351D.0	fGB	3-3343
U+351D.0	fR/S	 19.8
U+351D.0	fTotalStrokes	 10
U+351D.0	fCangjie	MBKS
U+351D.0	fFourCorner	14220
U+351D.0	fCantonese	loeng5
U+351D.0	fMandarin	liang3
U+351D.0	fEnglish	[1] strong resistance [2] lazy [3] reluctant
U+351E.0	fUTF8	㔞
U+351E.0	fCNS11643	3-2F6B
U+351E.0	fGB	5-3478
U+351E.0	fR/S	 19.8
U+351E.0	fTotalStrokes	 10
U+351E.0	fCangjie	HPKS
U+351E.0	fFourCorner	24120
U+351E.0	fCantonese	gwaai1; fai3
U+351E.0	fMandarin	kuai1; kuai4; kuang4; wang4
U+351E.0	fEnglish	vigorous; with great strength
U+351F.0	fUTF8	㔟
U+351F.0	fR/S	 19.8
U+351F.0	fTotalStrokes	 10
U+351F.0	fCangjie	HIKS
U+351F.0	fCantonese	sai3
U+351F.0	fEnglish	[Japanese character]
U+3520.0	fUTF8	㔠
U+3520.0	fCNS11643	4-305A
U+3520.0	fGB	3-3346
U+3520.0	fR/S	 19.9
U+3520.0	fTotalStrokes	 11
U+3520.0	fCangjie	AVKS
U+3520.0	fFourCorner	64720
U+3520.0	fCantonese	hat6
U+3520.0	fMandarin	xia1; he2; jia2
U+3520.0	fEnglish	[1] industry [2] sound of making strenuous efforts
U+3521.0	fUTF8	㔡
U+3521.0	fCNS11643	4-364B
U+3521.0	fGB	3-3349
U+3521.0	fR/S	 19.10
U+3521.0	fTotalStrokes	 12
U+3521.0	fCangjie	HPKS
U+3521.0	fFourCorner	24727
U+3521.0	fCantonese	bit3
U+3521.0	fMandarin	bie4
U+3521.0	fEnglish	[1] greatly; deeply [2] with great strength; vigorous
U+3522.0	fUTF8	㔢
U+3522.0	fCNS11643	3-4622
U+3522.0	fGB	5-347C
U+3522.0	fR/S	 19.12
U+3522.0	fTotalStrokes	 14
U+3522.0	fCangjie	VUKS
U+3522.0	fFourCorner	27420
U+3522.0	fHakka	 ciet7
U+3522.0	fCantonese	dyut3; waai4; zyut6
U+3522.0	fMandarin	jue2
U+3522.0	fEnglish	 break; snap
U+3523.0	fUTF8	㔣
U+3523.0	fCNS11643	4-556F
U+3523.0	fGB	3-334E
U+3523.0	fR/S	 19.15
U+3523.0	fTotalStrokes	 17; 16
U+3523.0	fCangjie	WWKS
U+3523.0	fFourCorner	64620
U+3523.0	fCantonese	leoi6; leoi4
U+3523.0	fMandarin	lei4; lei2
U+3523.0	fEnglish	[1] fight against the enemy by pushing down the big rocks from up high (in ancient times) [2] throw over [3] urge; to exert oneself [4] carry in the bosom
U+3524.0	fUTF8	㔤
U+3524.0	fCNS11643	3-5443
U+3524.0	fR/S	 19.15
U+3524.0	fTotalStrokes	 17
U+3524.0	fCangjie	JWKS
U+3524.0	fFourCorner	34620
U+3524.0	fCantonese	san3; saam2
U+3524.0	fMandarin	shen4; shen3
U+3524.0	fEnglish	 exert one's strength
U+3525.0	fUTF8	㔥
U+3525.0	fCNS11643	4-5570
U+3525.0	fGB	3-3348
U+3525.0	fR/S	 19.15
U+3525.0	fTotalStrokes	 17; 16
U+3525.0	fCangjie	WPKS
U+3525.0	fFourCorner	64220
U+3525.0	fCantonese	baa6; baai6; pei4
U+3525.0	fMandarin	ba4; pi2
U+3525.0	fEnglish	non-classical form of &#x75B2;
U+3526.0	fUTF8	㔦
U+3526.0	fCNS11643	5-5363
U+3526.0	fR/S	 19.15; 19.14
U+3526.0	fTotalStrokes	 16; 17
U+3526.0	fCangjie	TVKS
U+3526.0	fFourCorner	84720
U+3526.0	fCantonese	joeng5; zoeng5; zoeng6
U+3526.0	fMandarin	yang3
U+3526.0	fEnglish	 encourage; stimulate; exhort to action; encourage by rewards
U+3527.0	fUTF8	㔧
U+3527.0	fCNS11643	4-5A6E
U+3527.0	fGB	5-3472
U+3527.0	fR/S	 19.16
U+3527.0	fTotalStrokes	 18
U+3527.0	fCangjie	YTKS
U+3527.0	fFourCorner	24220
U+3527.0	fCantonese	leoi6
U+3527.0	fMandarin	lu2; l4; xue4
U+3527.0	fEnglish	[1] incite; urge [2] aid; help; assist [3] approve; acquiesce
U+3528.0	fUTF8	㔨
U+3528.0	fCNS11643	4-305C
U+3528.0	fGB	3-324F
U+3528.0	fR/S	 20.9
U+3528.0	fTotalStrokes	 11
U+3528.0	fCangjie	PLPB
U+3528.0	fFourCorner	27220
U+3528.0	fCantonese	bui3
U+3528.0	fMandarin	bei4
U+3528.0	fEnglish	the name of a person
U+3529.0	fUTF8	㔩
U+3529.0	fCNS11643	4-364D
U+3529.0	fGB	3-3250
U+3529.0	fR/S	 20.10
U+3529.0	fTotalStrokes	 12
U+3529.0	fCangjie	PGIT
U+3529.0	fFourCorner	27120
U+3529.0	fHakka	 rap8; jap8; ap7; jap7
U+3529.0	fCantonese	ap1; ngap1 (also); hap6; aap6
U+3529.0	fMandarin	e4
U+3529.0	fEnglish	part of an ornamental dress; hair ornaments used in old times
U+352A.0	fUTF8	㔪
U+352A.0	fCNS11643	5-3D78
U+352A.0	fGB	5-3332
U+352A.0	fR/S	 20.11
U+352A.0	fTotalStrokes	 13
U+352A.0	fCangjie	PYWI
U+352A.0	fFourCorner	27620
U+352A.0	fCantonese	lou5
U+352A.0	fMandarin	lu3
U+352A.0	fEnglish	 crawl; crawl on hands and knees
U+352B.0	fUTF8	㔫
U+352B.0	fR/S	 21.2
U+352B.0	fTotalStrokes	 4
U+352B.0	fCangjie	KP
U+352B.0	fCantonese	lung4
U+352B.0	fEnglish	[Japanese character]
U+352C.0	fUTF8	㔬
U+352C.0	fCNS11643	6-2972
U+352C.0	fR/S	 21.6
U+352C.0	fTotalStrokes	 8
U+352C.0	fCangjie	PAJ
U+352C.0	fFourCorner	21400
U+352C.0	fHakka	 zok7
U+352C.0	fCantonese	coek3; zoek3; zau3
U+352C.0	fMandarin	zhuo2; zhuo1
U+352C.0	fEnglish	same as &#x5353;
U+352D.0	fUTF8	㔭
U+352D.0	fCNS11643	3-3453
U+352D.0	fR/S	 21.9
U+352D.0	fTotalStrokes	 11
U+352D.0	fCangjie	PAMO
U+352D.0	fFourCorner	26780
U+352D.0	fHakka	 shi2; she2; chii2
U+352D.0	fCantonese	ci4; si4
U+352D.0	fMandarin	chi2; che4; shi1; yi2
U+352D.0	fEnglish	same as &#x5319;
U+352E.0	fUTF8	㔮
U+352E.0	fCNS11643	5-7078
U+352E.0	fGB	3-6B5D
U+352E.0	fR/S	 130.17
U+352E.0	fTotalStrokes	 21
U+352E.0	fCangjie	TOIBP
U+352E.0	fFourCorner	41510
U+352E.0	fCantonese	no4; lo4 (also)
U+352E.0	fMandarin	nuo2
U+352E.0	fEnglish	a kind of animal which looks like a rat
U+352F.0	fUTF8	㔯
U+352F.0	fCNS11643	4-234B
U+352F.0	fGB	3-3048
U+352F.0	fR/S	 22.5
U+352F.0	fTotalStrokes	 7
U+352F.0	fCangjie	SCR
U+352F.0	fFourCorner	71710
U+352F.0	fHakka	  sien2; con3
U+352F.0	fCantonese	syun4; syun3
U+352F.0	fMandarin	xuan2; suan3
U+352F.0	fEnglish	an osier basket for washing rice in
U+3530.0	fUTF8	㔰
U+3530.0	fCNS11643	6-2638
U+3530.0	fR/S	 22.5
U+3530.0	fTotalStrokes	 7
U+3530.0	fCangjie	SNIY
U+3530.0	fFourCorner	71710
U+3530.0	fHakka	 hen2
U+3530.0	fCantonese	hang4
U+3530.0	fMandarin	shu2; heng2; geng4; geng1
U+3530.0	fEnglish	ancient form of &#x6046;
U+3531.0	fUTF8	㔱
U+3531.0	fCNS11643	4-2B77
U+3531.0	fGB	5-3125
U+3531.0	fR/S	 22.9
U+3531.0	fTotalStrokes	 11; 10
U+3531.0	fCangjie	SHXO
U+3531.0	fFourCorner	71710
U+3531.0	fHakka	 ji3; ri3; i3
U+3531.0	fCantonese	jyu4; zyu5
U+3531.0	fMandarin	yu3
U+3531.0	fEnglish	 stack of grain
U+3532.0	fUTF8	㔲
U+3532.0	fCNS11643	3-3A25
U+3532.0	fR/S	 22.11
U+3532.0	fTotalStrokes	 12; 13
U+3532.0	fCangjie	SOIN
U+3532.0	fFourCorner	71710
U+3532.0	fHakka	 kui3; kui1; kwui3; gwi3; k'wui3
U+3532.0	fCantonese	gwai2
U+3532.0	fMandarin	gui3
U+3532.0	fEnglish	ancient form of &#x7C0B;
U+3533.0	fUTF8	㔳
U+3533.0	fCNS11643	6-5076
U+3533.0	fR/S	 22.11
U+3533.0	fTotalStrokes	 13
U+3533.0	fCangjie	SOIN
U+3533.0	fFourCorner	71710
U+3533.0	fCantonese	gwai2; gei2; gwaai2
U+3533.0	fMandarin	gui3; gun4; huan2; jue2
U+3533.0	fEnglish	ancient form of &#x7C0B;
U+3534.0	fUTF8	㔴
U+3534.0	fCNS11643	4-3C3E
U+3534.0	fGB	3-3050
U+3534.0	fR/S	 22.11
U+3534.0	fTotalStrokes	 13
U+3534.0	fCangjie	SWTC
U+3534.0	fFourCorner	71710
U+3534.0	fHakka	 jit8
U+3534.0	fCantonese	jik6; zik6
U+3534.0	fMandarin	yi4
U+3534.0	fEnglish	 incense-burner; caldron
U+3535.0	fUTF8	㔵
U+3535.0	fCNS11643	5-446C
U+3535.0	fR/S	 22.12
U+3535.0	fTotalStrokes	 14
U+3535.0	fCangjie	SRUC
U+3535.0	fFourCorner	71710
U+3535.0	fCantonese	syun2; wan4
U+3535.0	fMandarin	xuan3; xian4
U+3535.0	fEnglish	a kind of vessel
U+3536.0	fUTF8	㔶
U+3536.0	fCNS11643	4-6D53
U+3536.0	fGB	3-3052
U+3536.0	fR/S	 22.24
U+3536.0	fTotalStrokes	 26
U+3536.0	fCangjie	SYJC
U+3536.0	fFourCorner	71710
U+3536.0	fHakka	  gam5;  gung1
U+3536.0	fCantonese	gung3; gaam2; zi1
U+3536.0	fMandarin	gan3; gong4
U+3536.0	fEnglish	[1] small cup [2] box; chest [3] granary [4] cover of a vessel
U+3537.0	fUTF8	㔷
U+3537.0	fCNS11643	6-2636
U+3537.0	fGB	5-307C
U+3537.0	fR/S	 23.5
U+3537.0	fTotalStrokes	 7
U+3537.0	fCangjie	SMOB
U+3537.0	fFourCorner	71710
U+3537.0	fHakka	 leu6; leu5
U+3537.0	fCantonese	lau6
U+3537.0	fMandarin	yue4; lou4
U+3537.0	fEnglish	ancient form of &#x964B;
U+3538.0	fUTF8	㔸
U+3538.0	fCNS11643	4-3650
U+3538.0	fGB	3-304F
U+3538.0	fR/S	 23.10
U+3538.0	fTotalStrokes	 12
U+3538.0	fCangjie	SHYU
U+3538.0	fFourCorner	71710
U+3538.0	fCantonese	tai1; cam2; taai1; zyun1
U+3538.0	fMandarin	ti1
U+3538.0	fEnglish	interchangeable with &#x69B9;
U+3539.0	fUTF8	㔹
U+3539.0	fCNS11643	3-2158
U+3539.0	fGB	5-3045
U+3539.0	fR/S	 24.2
U+3539.0	fTotalStrokes	 4
U+3539.0	fCangjie	JKS
U+3539.0	fFourCorner	44020
U+3539.0	fCantonese	lak6
U+3539.0	fMandarin	le4
U+3539.0	fEnglish	standard form of &#x4EC2;
U+353A.0	fUTF8	㔺
U+353A.0	fCNS11643	6-2371
U+353A.0	fR/S	 24.4
U+353A.0	fTotalStrokes	 6
U+353A.0	fCangjie	JPJ
U+353A.0	fFourCorner	40710
U+353A.0	fHakka	 s'e5; shi5
U+353A.0	fCantonese	sai3; saai3; si6
U+353A.0	fMandarin	shi4
U+353A.0	fEnglish	ancient form of &#x4E16;
U+353B.0	fUTF8	㔻
U+353B.0	fCNS11643	3-232C
U+353B.0	fGB	5-3047
U+353B.0	fR/S	 24.4
U+353B.0	fTotalStrokes	 6
U+353B.0	fCangjie	MFJ
U+353B.0	fFourCorner	10400
U+353B.0	fHakka	 pi1
U+353B.0	fCantonese	pei1
U+353B.0	fMandarin	pei1
U+353B.0	fEnglish	a Chinese family name
U+353C.0	fUTF8	㔼
U+353C.0	fCNS11643	6-507A
U+353C.0	fR/S	 24.11
U+353C.0	fTotalStrokes	 13
U+353C.0	fCangjie	QGJ
U+353C.0	fFourCorner	50400
U+353C.0	fHakka	 sun3; zhun3; zun3
U+353C.0	fCantonese	seon2
U+353C.0	fMandarin	sun3
U+353C.0	fEnglish	non-classical form of &#x69AB;
U+353D.0	fUTF8	㔽
U+353D.0	fCNS11643	3-2764
U+353D.0	fR/S	 25.6
U+353D.0	fTotalStrokes	 8
U+353D.0	fCangjie	YWR
U+353D.0	fFourCorner	21600
U+353D.0	fHakka	 ju3
U+353D.0	fCantonese	jau5; zaau5
U+353D.0	fMandarin	you4; you3; yao4
U+353D.0	fEnglish	same as &#x5363;
U+353E.0	fUTF8	㔾
U+353E.0	fCNS11643	4-212B
U+353E.0	fR/S	 26.1; 26.0
U+353E.0	fTotalStrokes	 2; 3
U+353E.0	fCangjie	SU
U+353E.0	fFourCorner	77710
U+353E.0	fHakka	 ziet7
U+353E.0	fCantonese	cim1; zit3
U+353E.0	fMandarin	jie2
U+353E.0	fEnglish	 officer's seal, kept in two pieces
U+353F.0	fUTF8	㔿
U+353F.0	fCNS11643	6-213B
U+353F.0	fR/S	 26.1
U+353F.0	fTotalStrokes	 3
U+353F.0	fCangjie	MVL
U+353F.0	fFourCorner	10720
U+353F.0	fCantonese	zau3
U+353F.0	fMandarin	zou4
U+3540.0	fUTF8	㕀
U+3540.0	fCNS11643	6-2864
U+3540.0	fR/S	 26.5
U+3540.0	fTotalStrokes	 7
U+3540.0	fCangjie	LWSU
U+3540.0	fFourCorner	50710
U+3540.0	fCantonese	wai3
U+3540.0	fMandarin	chuang2
U+3541.0	fUTF8	㕁
U+3541.0	fCNS11643	4-2556
U+3541.0	fR/S	 26.6
U+3541.0	fTotalStrokes	 8; 7
U+3541.0	fCangjie	KISL
U+3541.0	fFourCorner	47420
U+3541.0	fHakka	 kiok7
U+3541.0	fCantonese	koek3
U+3541.0	fMandarin	que4; jue2
U+3541.0	fEnglish	non-classical form of &#x537B;
U+3542.0	fUTF8	㕂
U+3542.0	fCNS11643	3-2330
U+3542.0	fGB	5-3052
U+3542.0	fR/S	 27.4
U+3542.0	fTotalStrokes	 6
U+3542.0	fCangjie	MOIN
U+3542.0	fFourCorner	71220
U+3542.0	fCantonese	jam4; zaam4; zaam6
U+3542.0	fMandarin	qin2; yin2
U+3542.0	fEnglish	ancient form of &#x541F;
U+3543.0	fUTF8	㕃
U+3543.0	fCNS11643	5-223C
U+3543.0	fR/S	 27.4
U+3543.0	fTotalStrokes	 6
U+3543.0	fCangjie	MMT
U+3543.0	fFourCorner	71240
U+3543.0	fCantonese	kai1; him3; kaai1; lim4; zap3
U+3543.0	fMandarin	xi1
U+3543.0	fEnglish	 fall over; rolling on the ground
U+3544.0	fUTF8	㕄
U+3544.0	fCNS11643	3-232E
U+3544.0	fR/S	 27.4
U+3544.0	fTotalStrokes	 6
U+3544.0	fCangjie	MMVI
U+3544.0	fFourCorner	71230
U+3544.0	fCantonese	zi2; zi3
U+3544.0	fMandarin	zhi3; zhi4
U+3544.0	fEnglish	same as &#x538E;, &#x7825;
U+3545.0	fUTF8	㕅
U+3545.0	fCNS11643	3-246A
U+3545.0	fR/S	 27.5
U+3545.0	fTotalStrokes	 7
U+3545.0	fCangjie	MWL
U+3545.0	fFourCorner	71250
U+3545.0	fCantonese	gaap3
U+3545.0	fMandarin	jia3
U+3545.0	fEnglish	big; extremely; great; vast; extensive; tall
U+3546.0	fUTF8	㕆
U+3546.0	fCNS11643	3-2469
U+3546.0	fGB	5-3053
U+3546.0	fR/S	 27.5
U+3546.0	fTotalStrokes	 7
U+3546.0	fCangjie	MJR
U+3546.0	fFourCorner	71260
U+3546.0	fCantonese	wu6
U+3546.0	fMandarin	hu3; hu4
U+3546.0	fEnglish	fine stone; fine jade
U+3547.0	fUTF8	㕇
U+3547.0	fCNS11643	3-2468
U+3547.0	fGB	5-3056
U+3547.0	fR/S	 27.5
U+3547.0	fTotalStrokes	 7
U+3547.0	fCangjie	MYT
U+3547.0	fFourCorner	71210
U+3547.0	fCantonese	laap6; lap3
U+3547.0	fMandarin	li4; la2
U+3547.0	fEnglish	[1] rocks collapsing (descriptive of sound) [2] big pieces of rocks
U+3548.0	fUTF8	㕈
U+3548.0	fCNS11643	3-2768
U+3548.0	fGB	5-305B
U+3548.0	fR/S	 27.6
U+3548.0	fTotalStrokes	 8
U+3548.0	fCangjie	MYHV
U+3548.0	fFourCorner	71230
U+3548.0	fHakka	 ji3; ri3; i3
U+3548.0	fCantonese	ji2; at3; zi2
U+3548.0	fMandarin	yi3; hou4
U+3548.0	fEnglish	 hide; screen
U+3549.0	fUTF8	㕉
U+3549.0	fCNS11643	6-2A2A
U+3549.0	fGB	5-305A
U+3549.0	fR/S	 27.6
U+3549.0	fTotalStrokes	 8
U+3549.0	fCangjie	MOMR
U+3549.0	fFourCorner	71260
U+3549.0	fCantonese	hap6
U+3549.0	fMandarin	he2; ke4
U+3549.0	fEnglish	dig a hole; tuck in; put the finger into
U+354A.0	fUTF8	㕊
U+354A.0	fCNS11643	3-2B4C
U+354A.0	fGB	5-305C
U+354A.0	fR/S	 27.7
U+354A.0	fTotalStrokes	 9
U+354A.0	fCangjie	MIJB
U+354A.0	fFourCorner	71220
U+354A.0	fCantonese	bou1
U+354A.0	fMandarin	fu1
U+354A.0	fEnglish	[1] slant [2] intricate [3] describe a writer's pen [4] lofty rocks; protruded rocks [5] resolute
U+354B.0	fUTF8	㕋
U+354B.0	fCNS11643	3-2F70
U+354B.0	fGB	5-3061
U+354B.0	fR/S	 27.8
U+354B.0	fTotalStrokes	 10
U+354B.0	fCangjie	MC
U+354B.0	fFourCorner	71210
U+354B.0	fCantonese	kam4
U+354B.0	fMandarin	qin2; jing4
U+354B.0	fEnglish	rock land
U+354C.0	fUTF8	㕌
U+354C.0	fCNS11643	4-2B7A
U+354C.0	fGB	5-305E
U+354C.0	fR/S	 27.8
U+354C.0	fTotalStrokes	 10
U+354C.0	fCangjie	MAMI
U+354C.0	fFourCorner	71240
U+354C.0	fCantonese	ngoi6; oi6 (also); zak6
U+354C.0	fMandarin	yi4; ai4
U+354C.0	fEnglish	[1] raise the curtain [2] a kind of rock
U+354D.0	fUTF8	㕍
U+354D.0	fCNS11643	3-2F71
U+354D.0	fR/S	 27.8
U+354D.0	fTotalStrokes	 10
U+354D.0	fCangjie	MOG
U+354D.0	fFourCorner	71210
U+354D.0	fCantonese	deoi1
U+354D.0	fMandarin	dui1
U+354D.0	fEnglish	same as &#x5806;
U+354E.0	fUTF8	㕎
U+354E.0	fCNS11643	4-3651
U+354E.0	fGB	3-303F
U+354E.0	fR/S	 27.10
U+354E.0	fTotalStrokes	 12
U+354E.0	fCangjie	MGIT
U+354E.0	fFourCorner	71210
U+354E.0	fCantonese	hap6
U+354E.0	fMandarin	ke4; e4; kun3
U+354E.0	fEnglish	caves
U+354F.0	fUTF8	㕏
U+354F.0	fCNS11643	6-5122
U+354F.0	fR/S	 27.11
U+354F.0	fTotalStrokes	 13
U+354F.0	fCangjie	MMTJ
U+354F.0	fFourCorner	71240
U+354F.0	fHakka	 chu2
U+354F.0	fCantonese	cyu4; ceoi4
U+354F.0	fMandarin	chu2
U+354F.0	fEnglish	corrupted form of &#x5EDA;
U+3550.0	fUTF8	㕐
U+3550.0	fCNS11643	5-4C31
U+3550.0	fR/S	 27.12
U+3550.0	fTotalStrokes	 15; 14
U+3550.0	fCangjie	MHXF
U+3550.0	fFourCorner	71220
U+3550.0	fCantonese	se2
U+3550.0	fMandarin	xie3
U+3550.0	fEnglish	[1] inclined; slanting [2] upset; be overthrown [3] collapse; fall flat
U+3551.0	fUTF8	㕑
U+3551.0	fCNS11643	3-4624
U+3551.0	fR/S	 27.12
U+3551.0	fTotalStrokes	 14
U+3551.0	fCangjie	MGTI
U+3551.0	fFourCorner	71240
U+3551.0	fHakka	 chu2
U+3551.0	fCantonese	cyu4; ceoi4 (also)
U+3551.0	fMandarin	chu2
U+3551.0	fEnglish	non-classical form of &#x5831;
U+3552.0	fUTF8	㕒
U+3552.0	fCNS11643	4-4927
U+3552.0	fGB	3-3034
U+3552.0	fR/S	 27.13
U+3552.0	fTotalStrokes	 15
U+3552.0	fCangjie	MTGI
U+3552.0	fFourCorner	71250
U+3552.0	fHakka	 ngui2
U+3552.0	fCantonese	ngai4; ai4 (also); faai1; waai4; waai6
U+3552.0	fMandarin	wei2; wei3
U+3552.0	fEnglish	lofty peak
U+3553.0	fUTF8	㕓
U+3553.0	fR/S	 27.15
U+3553.0	fTotalStrokes	 17
U+3553.0	fCangjie	MWFG
U+3553.0	fFourCorner	7121
U+3553.0	fCantonese	cin4
U+3554.0	fUTF8	㕔
U+3554.0	fCNS11643	3-6070
U+3554.0	fR/S	 27.22
U+3554.0	fTotalStrokes	 24
U+3554.0	fCangjie	MSGP
U+3554.0	fFourCorner	71230
U+3554.0	fCantonese	teng1; ting1 (also)
U+3554.0	fMandarin	ting1
U+3554.0	fEnglish	non-classical form of &#x5EF3;
U+3555.0	fUTF8	㕕
U+3555.0	fCNS11643	3-215D
U+3555.0	fR/S	 28.2
U+3555.0	fTotalStrokes	 4
U+3555.0	fCangjie	II
U+3555.0	fFourCorner	23730
U+3555.0	fHakka	 fam5
U+3555.0	fCantonese	waan6; nap6; wan6
U+3555.0	fMandarin	huan4
U+3555.0	fEnglish	ancient form of &#x5E7B;
U+3556.0	fUTF8	㕖
U+3556.0	fCNS11643	3-2F72
U+3556.0	fR/S	 28.8
U+3556.0	fTotalStrokes	 10
U+3556.0	fCangjie	IIID
U+3556.0	fFourCorner	23900
U+3556.0	fHakka	 sii5
U+3556.0	fCantonese	sou3
U+3556.0	fMandarin	su4
U+3556.0	fEnglish	same as &#x7D20;
U+3557.0	fUTF8	㕗
U+3557.0	fCNS11643	6-3D61
U+3557.0	fR/S	 28.9
U+3557.0	fTotalStrokes	 11
U+3557.0	fCangjie	TOI
U+3557.0	fFourCorner	83800
U+3557.0	fCantonese	jau5; zaau5
U+3557.0	fMandarin	you3; you4
U+3557.0	fEnglish	same as &#x8A98;
U+3558.0	fUTF8	㕘
U+3558.0	fCNS11643	3-3456
U+3558.0	fR/S	 28.9
U+3558.0	fTotalStrokes	 11
U+3558.0	fCangjie	IIIF
U+3558.0	fFourCorner	23900
U+3558.0	fCantonese	saam1; saam3; caam1; sam1; cam1
U+3558.0	fEnglish	variant of &#x53C3;
U+3559.0	fUTF8	㕙
U+3559.0	fCNS11643	4-4929
U+3559.0	fGB	3-3358
U+3559.0	fR/S	 28.13
U+3559.0	fTotalStrokes	 15
U+3559.0	fCangjie	IENAI
U+3559.0	fFourCorner	27410
U+3559.0	fHakka	 cun1;  zun5
U+3559.0	fCantonese	zeon3
U+3559.0	fMandarin	cun1; jun4; rui4
U+3559.0	fEnglish	a wily rabbit; a cunning hare
U+355A.0	fUTF8	㕚
U+355A.0	fCNS11643	5-2141
U+355A.0	fR/S	 29.2
U+355A.0	fTotalStrokes	 4
U+355A.0	fCangjie	EII
U+355A.0	fFourCorner	17340
U+355A.0	fHakka	 zau3
U+355A.0	fCantonese	zaau2
U+355A.0	fMandarin	zhao3
U+355A.0	fEnglish	[1] claws [2] talcons [3] scratch
U+355B.0	fUTF8	㕛
U+355B.0	fCNS11643	3-215F
U+355B.0	fR/S	 29.2
U+355B.0	fTotalStrokes	 4
U+355B.0	fCangjie	EE
U+355B.0	fFourCorner	17400
U+355B.0	fHakka	 ju5
U+355B.0	fCantonese	jau5; jau6; zaau5
U+355B.0	fMandarin	you3; you4; xu4
U+355B.0	fEnglish	ancient form of &#x53F3;
U+355C.0	fUTF8	㕜
U+355C.0	fCNS11643	3-2332
U+355C.0	fR/S	 29.4
U+355C.0	fTotalStrokes	 6
U+355C.0	fCangjie	LE
U+355C.0	fFourCorner	50400
U+355C.0	fCantonese	si2; sei3
U+355C.0	fMandarin	shi3
U+355C.0	fEnglish	ancient form of &#x53F2;
U+355D.0	fUTF8	㕝
U+355D.0	fR/S	 29.5
U+355D.0	fTotalStrokes	 7
U+355D.0	fCangjie	JRE
U+355D.0	fCantonese	si6
U+355D.0	fEnglish	[Japanese character]
U+355E.0	fUTF8	㕞
U+355E.0	fCNS11643	4-255A
U+355E.0	fGB	5-3538
U+355E.0	fR/S	 29.6
U+355E.0	fTotalStrokes	 8; 7
U+355E.0	fCangjie	SBE
U+355E.0	fFourCorner	77240
U+355E.0	fHakka	 sot7
U+355E.0	fCantonese	caat3
U+355E.0	fMandarin	shua1
U+355E.0	fEnglish	 brush
U+355F.0	fUTF8	㕟
U+355F.0	fCNS11643	4-2857
U+355F.0	fGB	3-335A
U+355F.0	fR/S	 29.7
U+355F.0	fTotalStrokes	 9
U+355F.0	fCangjie	YBE
U+355F.0	fFourCorner	27240
U+355F.0	fCantonese	wai2
U+355F.0	fMandarin	kuai4; kui4
U+355F.0	fEnglish	same as &#x559F;
U+3560.0	fUTF8	㕠
U+3560.0	fCNS11643	3-3F78
U+3560.0	fR/S	 29.10
U+3560.0	fTotalStrokes	 12
U+3560.0	fCangjie	IGE
U+3560.0	fFourCorner	30400
U+3560.0	fCantonese	soeng1; sung1 (also)
U+3560.0	fMandarin	shuang1
U+3560.0	fEnglish	same as &#x96D9;
U+3561.0	fUTF8	㕡
U+3561.0	fCNS11643	3-4627
U+3561.0	fR/S	 29.12
U+3561.0	fTotalStrokes	 14
U+3561.0	fCangjie	YRE
U+3561.0	fFourCorner	27640
U+3561.0	fCantonese	kok3; hang1
U+3561.0	fMandarin	huo4; he2; he4
U+3561.0	fEnglish	same as &#x58D1;
U+3562.0	fUTF8	㕢
U+3562.0	fCNS11643	4-4756
U+3562.0	fGB	3-335B
U+3562.0	fR/S	 154.7; 154.12; 29.12
U+3562.0	fTotalStrokes	 14
U+3562.0	fCangjie	YCE
U+3562.0	fFourCorner	27840
U+3562.0	fCantonese	haai6
U+3562.0	fMandarin	gai4; xie4; hai4; ju4; lun3; nou3
U+3562.0	fEnglish	profound significance; just now
U+3563.0	fUTF8	㕣
U+3563.0	fCNS11643	4-216B
U+3563.0	fGB	5-3135
U+3563.0	fR/S	 30.2
U+3563.0	fTotalStrokes	 5
U+3563.0	fCangjie	CR
U+3563.0	fFourCorner	80600
U+3563.0	fHakka	 jen3; ren3
U+3563.0	fCantonese	jin5; zin2
U+3563.0	fMandarin	yan2; yan3; yan4
U+3563.0	fEnglish	[1] a marsh at the foot of the hills [2] interchangeable with &#x6E62;, &#x6C87;
U+3564.0	fUTF8	㕤
U+3564.0	fCNS11643	3-2230
U+3564.0	fGB	5-3764
U+3564.0	fR/S	 30.2
U+3564.0	fTotalStrokes	 5
U+3564.0	fCangjie	RKN
U+3564.0	fFourCorner	64010
U+3564.0	fCantonese	kau4; gwaai2; kaau4
U+3564.0	fMandarin	qiu2
U+3564.0	fEnglish	same as &#x53F4;
U+3565.0	fUTF8	㕥
U+3565.0	fCNS11643	3-222C
U+3565.0	fR/S	 30.2
U+3565.0	fTotalStrokes	 5
U+3565.0	fCangjie	RO
U+3565.0	fFourCorner	68000
U+3565.0	fCantonese	ji5; zyu5
U+3565.0	fMandarin	shen1; yi3
U+3566.0	fUTF8	㕦
U+3566.0	fCNS11643	3-2338
U+3566.0	fGB	5-3769
U+3566.0	fR/S	 30.3
U+3566.0	fTotalStrokes	 6
U+3566.0	fCangjie	RK
U+3566.0	fFourCorner	60430
U+3566.0	fHakka	 wai3; wai2
U+3566.0	fCantonese	waa6; zyu5
U+3566.0	fMandarin	hua1; hua4
U+3566.0	fEnglish	[1] loud; clamour; hubbub [2] big mouth [3] brag
U+3567.0	fUTF8	㕧
U+3567.0	fCNS11643	3-2336
U+3567.0	fGB	5-376E
U+3567.0	fR/S	 30.3
U+3567.0	fTotalStrokes	 6
U+3567.0	fCangjie	RS
U+3567.0	fFourCorner	67070
U+3567.0	fHakka	 shi1; ri2; ji2; sii1
U+3567.0	fCantonese	si1
U+3567.0	fMandarin	xi1
U+3567.0	fEnglish	 mutter; groan
U+3568.0	fUTF8	㕨
U+3568.0	fGB	3-3550
U+3568.0	fR/S	 30.3
U+3568.0	fTotalStrokes	 6
U+3568.0	fCangjie	RHNI
U+3568.0	fFourCorner	6701
U+3568.0	fCantonese	faan6; fan6
U+3568.0	fMandarin	fan4
U+3569.0	fUTF8	㕩
U+3569.0	fCNS11643	3-246C
U+3569.0	fGB	5-3771
U+3569.0	fR/S	 30.4
U+3569.0	fTotalStrokes	 7
U+3569.0	fCangjie	RQJ
U+3569.0	fFourCorner	65000
U+3569.0	fCantonese	pong3; zoeng3
U+3569.0	fMandarin	pang4; zhang4
U+3569.0	fEnglish	[1] (non-classical form) [2] sound of tapping; sound of striking [3] opposing voices
U+356A.0	fUTF8	㕪
U+356A.0	fCNS11643	4-2351
U+356A.0	fGB	3-3562
U+356A.0	fR/S	 30.4
U+356A.0	fTotalStrokes	 7
U+356A.0	fCangjie	RLBU
U+356A.0	fFourCorner	64010
U+356A.0	fCantonese	dam2
U+356A.0	fMandarin	dan3
U+356A.0	fEnglish	[1] birds' chirps [2] gabbling [3] sound in high pitch [4] roar; aloud
U+356B.0	fUTF8	㕫
U+356B.0	fCNS11643	3-246B
U+356B.0	fR/S	 30.4
U+356B.0	fTotalStrokes	 7
U+356B.0	fCangjie	RYHS
U+356B.0	fFourCorner	60020
U+356B.0	fHakka	 fong3
U+356B.0	fCantonese	fung1; fong2
U+356B.0	fMandarin	fang3
U+356B.0	fEnglish	same as &#x4EFF;, &#x65B9;
U+356C.0	fUTF8	㕬
U+356C.0	fCNS11643	3-2471
U+356C.0	fGB	5-377E
U+356C.0	fR/S	 30.4
U+356C.0	fTotalStrokes	 7
U+356C.0	fCangjie	RCI
U+356C.0	fFourCorner	68030
U+356C.0	fCantonese	zung1; zung6
U+356C.0	fMandarin	gong1; song4
U+356C.0	fEnglish	[1] public opinion [2] same as &#x8A1F;
U+356D.0	fUTF8	㕭
U+356D.0	fCNS11643	4-2354
U+356D.0	fGB	3-355A
U+356D.0	fR/S	 30.4
U+356D.0	fTotalStrokes	 7
U+356D.0	fCangjie	RHK
U+356D.0	fFourCorner	62030
U+356D.0	fHakka	 rau1; jeu1; jau1
U+356D.0	fCantonese	aau3; ngaau3 (also); jiu1; jaau1; au3
U+356D.0	fMandarin	ao1; ao4; bi4
U+356D.0	fEnglish	same as &#x54AC;
U+356E.0	fUTF8	㕮
U+356E.0	fCNS11643	4-2359
U+356E.0	fGB	3-355D
U+356E.0	fR/S	 30.4
U+356E.0	fTotalStrokes	 7
U+356E.0	fCangjie	RCK
U+356E.0	fFourCorner	68040
U+356E.0	fHakka	 fu5; fu3
U+356E.0	fCantonese	fu2
U+356E.0	fMandarin	fu4; fu3; tiao4
U+356E.0	fEnglish	 chew; think over
U+356F.0	fUTF8	㕯
U+356F.0	fCNS11643	3-2472
U+356F.0	fGB	5-3134
U+356F.0	fR/S	 30.4; 30.5; 13.5
U+356F.0	fTotalStrokes	 7
U+356F.0	fCangjie	OBR
U+356F.0	fFourCorner	40600
U+356F.0	fCantonese	neot6; leot6 (also); naap6; laap6
U+356F.0	fMandarin	nei4; na4; ne4
U+356F.0	fEnglish	same as &#x5436;
U+3570.0	fUTF8	㕰
U+3570.0	fCNS11643	4-2353
U+3570.0	fGB	5-3778
U+3570.0	fR/S	 30.4
U+3570.0	fTotalStrokes	 7
U+3570.0	fCangjie	RHQU
U+3570.0	fFourCorner	62014
U+3570.0	fCantonese	hok3; huk6; maa3
U+3570.0	fMandarin	xue4; he4
U+3570.0	fEnglish	[1] sound angrily; roaring with rage [2] same as &#x561B;
U+3571.0	fUTF8	㕱
U+3571.0	fCNS11643	3-246F
U+3571.0	fR/S	 30.4
U+3571.0	fTotalStrokes	 7
U+3571.0	fCangjie	RIKU
U+3571.0	fFourCorner	63010
U+3571.0	fCantonese	jau4; zaau4
U+3571.0	fMandarin	you2
U+3571.0	fEnglish	 bark (dog)
U+3572.0	fUTF8	㕲
U+3572.0	fCNS11643	3-246E
U+3572.0	fGB	5-3772
U+3572.0	fR/S	 30.4
U+3572.0	fTotalStrokes	 7
U+3572.0	fCangjie	RD
U+3572.0	fFourCorner	64090
U+3572.0	fCantonese	muk1; zing4
U+3572.0	fMandarin	hua2; ying2
U+3572.0	fEnglish	[1] gag or bribe [2] satisfy one's appetite [3] the name of an old book
U+3573.0	fUTF8	㕳
U+3573.0	fCNS11643	3-247A
U+3573.0	fGB	5-377D
U+3573.0	fR/S	 30.4
U+3573.0	fTotalStrokes	 7
U+3573.0	fCangjie	RUK
U+3573.0	fFourCorner	62070
U+3573.0	fCantonese	hung1
U+3573.0	fMandarin	xiong1
U+3573.0	fEnglish	[1] noisy (of a crowd); [2] brawl; [3] scold
U+3574.0	fUTF8	㕴
U+3574.0	fGB	S-2351
U+3574.0	fR/S	 30.4
U+3574.0	fTotalStrokes	 7
U+3574.0	fCangjie	RBHN
U+3574.0	fFourCorner	6701
U+3574.0	fCantonese	sam2; caam4
U+3574.0	fMandarin	chen2
U+3575.0	fUTF8	㕵
U+3575.0	fCNS11643	F-2349
U+3575.0	fR/S	 30.4
U+3575.0	fTotalStrokes	 7
U+3575.0	fCangjie	RMG
U+3575.0	fFourCorner	6101
U+3575.0	fCantonese	gwok3; gok3 (also)
U+3575.0	fMandarin	guo2; hun2; luo3
U+3576.0	fUTF8	㕶
U+3576.0	fCNS11643	3-2525
U+3576.0	fGB	5-3773
U+3576.0	fR/S	 30.4
U+3576.0	fTotalStrokes	 7
U+3576.0	fCangjie	RMDM
U+3576.0	fFourCorner	61010
U+3576.0	fCantonese	ng4 (also); ng6 (also); m4
U+3576.0	fMandarin	ng4
U+3576.0	fEnglish	oh!
U+3577.0	fUTF8	㕷
U+3577.0	fCNS11643	3-277A
U+3577.0	fGB	5-382B
U+3577.0	fR/S	 30.5
U+3577.0	fTotalStrokes	 8
U+3577.0	fCangjie	RHA
U+3577.0	fFourCorner	66000
U+3577.0	fHakka	 bak8
U+3577.0	fCantonese	waak1; pak1
U+3577.0	fMandarin	huo4; hua4
U+3577.0	fEnglish	corrupted form of &#x549F;
U+3578.0	fUTF8	㕸
U+3578.0	fCNS11643	3-276D
U+3578.0	fGB	5-3830
U+3578.0	fR/S	 30.5
U+3578.0	fTotalStrokes	 8
U+3578.0	fCangjie	RYT
U+3578.0	fFourCorner	60010
U+3578.0	fCantonese	lap1; lai6; lap6
U+3578.0	fMandarin	li4
U+3578.0	fEnglish	the sound of rolling a boat
U+3579.0	fUTF8	㕹
U+3579.0	fCNS11643	4-255D
U+3579.0	fGB	5-3829
U+3579.0	fR/S	 30.5
U+3579.0	fTotalStrokes	 8; 7
U+3579.0	fCangjie	RIKK
U+3579.0	fFourCorner	63040
U+3579.0	fCantonese	fat6
U+3579.0	fMandarin	fa1; fa2
U+3579.0	fEnglish	 buckler; shield
U+357A.0	fUTF8	㕺
U+357A.0	fCNS11643	3-2776
U+357A.0	fR/S	 30.5
U+357A.0	fTotalStrokes	 8
U+357A.0	fCangjie	RRMVS
U+357A.0	fFourCorner	66720
U+357A.0	fCantonese	hiu1
U+357A.0	fMandarin	xiao1; hao2
U+357A.0	fEnglish	corrupted form of &#x56C2;
U+357B.0	fUTF8	㕻
U+357B.0	fCNS11643	5-2542
U+357B.0	fR/S	 30.5
U+357B.0	fTotalStrokes	 8
U+357B.0	fCangjie	YFR
U+357B.0	fFourCorner	00600
U+357B.0	fHakka	 teu5
U+357B.0	fCantonese	tau3; faau2; fu3; taau3
U+357B.0	fMandarin	tou4; pou3
U+357B.0	fEnglish	 spit out
U+357C.0	fUTF8	㕼
U+357C.0	fCNS11643	6-2A39
U+357C.0	fR/S	 30.5
U+357C.0	fTotalStrokes	 8
U+357C.0	fCangjie	RPVK
U+357C.0	fFourCorner	67020
U+357C.0	fCantonese	hung1; dung6
U+357C.0	fMandarin	hong1
U+357C.0	fEnglish	[1] brawl; [2] scold [3] noise from a crowd; hubbub; noisy; loud noise
U+357D.0	fUTF8	㕽
U+357D.0	fR/S	 30.5
U+357D.0	fTotalStrokes	 8
U+357D.0	fCangjie	RVIO
U+357D.0	fFourCorner	6800
U+357D.0	fCantonese	ci5; cek3; ji5
U+357D.0	fMandarin	si4
U+357D.0	fEnglish	a phonetic
U+357E.0	fUTF8	㕾
U+357E.0	fR/S	 30.5
U+357E.0	fTotalStrokes	 8
U+357E.0	fCangjie	FRP
U+357E.0	fCantonese	siu2
U+357F.0	fUTF8	㕿
U+357F.0	fR/S	 30.5
U+357F.0	fTotalStrokes	 8
U+357F.0	fCangjie	HDR
U+357F.0	fCantonese	wo4
U+3580.0	fUTF8	㖀
U+3580.0	fCNS11643	3-2B52
U+3580.0	fGB	5-3844
U+3580.0	fR/S	 30.6
U+3580.0	fTotalStrokes	 9
U+3580.0	fCangjie	RLQ
U+3580.0	fFourCorner	65007
U+3580.0	fHakka	 lut8;  lit8
U+3580.0	fCantonese	leot6; laak6; lak6
U+3580.0	fMandarin	lu4; le4; luo4
U+3580.0	fEnglish	sound, not on a musical instrument
U+3581.0	fUTF8	㖁
U+3581.0	fCNS11643	3-2B60
U+3581.0	fGB	5-383F
U+3581.0	fR/S	 30.6
U+3581.0	fTotalStrokes	 9
U+3581.0	fCangjie	RKKI
U+3581.0	fFourCorner	64030
U+3581.0	fHakka	 nin5; lin5
U+3581.0	fCantonese	leon6
U+3581.0	fMandarin	lin4
U+3581.0	fEnglish	non-classical form of &#x541D;
U+3582.0	fUTF8	㖂
U+3582.0	fCNS11643	3-2B5F
U+3582.0	fGB	5-383C
U+3582.0	fR/S	 30.6
U+3582.0	fTotalStrokes	 9
U+3582.0	fCangjie	RLWP
U+3582.0	fFourCorner	65000
U+3582.0	fCantonese	jai6; zaai6
U+3582.0	fMandarin	yi4
U+3582.0	fEnglish	same as &#x8A4D;
U+3583.0	fUTF8	㖃
U+3583.0	fCNS11643	3-2B62
U+3583.0	fGB	5-383E
U+3583.0	fR/S	 30.6
U+3583.0	fTotalStrokes	 9
U+3583.0	fCangjie	RHMR
U+3583.0	fFourCorner	62060
U+3583.0	fHakka	  keu5;  heu5; heu3
U+3583.0	fCantonese	hau2; haam1; haau3; haau6
U+3583.0	fMandarin	hou4; hou3
U+3583.0	fEnglish	 scold; abuse
U+3584.0	fUTF8	㖄
U+3584.0	fCNS11643	3-2B5A
U+3584.0	fR/S	 30.6
U+3584.0	fTotalStrokes	 9
U+3584.0	fCangjie	RILL
U+3584.0	fFourCorner	62000
U+3584.0	fHakka	 zhiu1; zu1; chuk7
U+3584.0	fCantonese	zau1; zaau1; zaau3
U+3584.0	fMandarin	zhou1; zhu4
U+3584.0	fEnglish	[1] call fowls [2] same as &#x558C;
U+3585.0	fUTF8	㖅
U+3585.0	fCNS11643	4-2859
U+3585.0	fGB	5-3837
U+3585.0	fR/S	 30.6
U+3585.0	fTotalStrokes	 9
U+3585.0	fCangjie	RIHM
U+3585.0	fFourCorner	63050
U+3585.0	fHakka	  sut7; sit7
U+3585.0	fCantonese	seot1
U+3585.0	fMandarin	xu1; xu4
U+3585.0	fEnglish	 whistle
U+3586.0	fUTF8	㖆
U+3586.0	fCNS11643	4-285C
U+3586.0	fGB	3-3626
U+3586.0	fR/S	 30.6
U+3586.0	fTotalStrokes	 9
U+3586.0	fCangjie	RTW
U+3586.0	fFourCorner	65060
U+3586.0	fCantonese	kuk1
U+3586.0	fMandarin	qiong4; qu2; qu3
U+3586.0	fEnglish	[1] condemn; sentence [2] hold a hearing (on a legal case); interrogate; question a prisoner [3] whisper
U+3587.0	fUTF8	㖇
U+3587.0	fCNS11643	3-2B56
U+3587.0	fR/S	 30.6
U+3587.0	fTotalStrokes	 9
U+3587.0	fCangjie	RMBL
U+3587.0	fFourCorner	61020
U+3587.0	fHakka	 ji2; ri2
U+3587.0	fCantonese	ji4; zi4
U+3587.0	fMandarin	er2
U+3587.0	fEnglish	the sides of the mouth
U+3588.0	fUTF8	㖈
U+3588.0	fCNS11643	F-295C
U+3588.0	fR/S	 30.6
U+3588.0	fTotalStrokes	 9
U+3588.0	fCangjie	JPR
U+3588.0	fFourCorner	4460
U+3588.0	fCantonese	lou5
U+3589.0	fUTF8	㖉
U+3589.0	fGB	S-2350
U+3589.0	fR/S	 30.6
U+3589.0	fTotalStrokes	 9
U+3589.0	fCangjie	ROYM
U+3589.0	fCantonese	kei5
U+358A.0	fUTF8	㖊
U+358A.0	fGB	S-2352
U+358A.0	fR/S	 30.6
U+358A.0	fTotalStrokes	 9
U+358A.0	fCangjie	RSMI
U+358A.0	fCantonese	cam4; jing1 (also)
U+358B.0	fUTF8	㖋
U+358B.0	fR/S	 30.6
U+358B.0	fTotalStrokes	 9
U+358B.0	fCangjie	OLLP
U+358B.0	fCantonese	gaai3
U+358C.0	fUTF8	㖌
U+358C.0	fR/S	 30.6
U+358C.0	fTotalStrokes	 9
U+358C.0	fCangjie	ONRP
U+358C.0	fCantonese	sau4
U+358D.0	fUTF8	㖍
U+358D.0	fR/S	 30.6
U+358D.0	fTotalStrokes	 9
U+358D.0	fCangjie	YJRP
U+358D.0	fCantonese	dau2
U+358E.0	fUTF8	㖎
U+358E.0	fR/S	 30.6
U+358E.0	fTotalStrokes	 9
U+358E.0	fCangjie	RPOLL
U+358E.0	fCantonese	gaai3
U+358F.0	fUTF8	㖏
U+358F.0	fCNS11643	4-2C25
U+358F.0	fGB	3-3636
U+358F.0	fR/S	 30.7
U+358F.0	fTotalStrokes	 10
U+358F.0	fCangjie	RAG
U+358F.0	fFourCorner	66010
U+358F.0	fHakka	 ngiap7
U+358F.0	fCantonese	nit6; lit6 (also); nip6
U+358F.0	fMandarin	nie4; nei4
U+358F.0	fEnglish	 scold; abuse
U+3590.0	fUTF8	㖐
U+3590.0	fCNS11643	3-2F7D
U+3590.0	fR/S	 30.7
U+3590.0	fTotalStrokes	 10
U+3590.0	fCangjie	RRSK
U+3590.0	fFourCorner	66500
U+3590.0	fHakka	 wi3; wui3; wui2; wi2; vui3
U+3590.0	fCantonese	wai5; do6; hei3; waai2
U+3590.0	fMandarin	wei3
U+3590.0	fEnglish	a cry to call in ducks
U+3591.0	fUTF8	㖑
U+3591.0	fCNS11643	4-2B7E
U+3591.0	fGB	5-3849
U+3591.0	fR/S	 30.7
U+3591.0	fTotalStrokes	 10
U+3591.0	fCangjie	RIT
U+3591.0	fFourCorner	63050
U+3591.0	fCantonese	haai3; hai6
U+3591.0	fMandarin	xie4
U+3591.0	fEnglish	[1] shout at [2] same as &#x6B38;
U+3592.0	fUTF8	㖒
U+3592.0	fCNS11643	3-2F75
U+3592.0	fR/S	 30.7
U+3592.0	fTotalStrokes	 10
U+3592.0	fCangjie	RCNH
U+3592.0	fFourCorner	68020
U+3592.0	fHakka	 ti2; tai2
U+3592.0	fCantonese	tai4
U+3592.0	fMandarin	ti2
U+3592.0	fEnglish	same as &#x55C1;
U+3593.0	fUTF8	㖓
U+3593.0	fCNS11643	3-2F7C
U+3593.0	fR/S	 30.7
U+3593.0	fTotalStrokes	 10
U+3593.0	fCangjie	RHEJ
U+3593.0	fFourCorner	67050
U+3593.0	fHakka	 fung2; fung1
U+3593.0	fCantonese	hung4
U+3593.0	fMandarin	hong2; hong1
U+3593.0	fEnglish	same as &#x53FF;
U+3594.0	fUTF8	㖔
U+3594.0	fCNS11643	4-2C21
U+3594.0	fGB	3-3630
U+3594.0	fR/S	 30.7
U+3594.0	fTotalStrokes	 10
U+3594.0	fCangjie	RMKR
U+3594.0	fFourCorner	62060
U+3594.0	fCantonese	tan3; taan1
U+3594.0	fMandarin	tun3
U+3594.0	fEnglish	silly; foolish; doting; idiotic
U+3595.0	fUTF8	㖕
U+3595.0	fCNS11643	4-2B7D
U+3595.0	fGB	3-3641
U+3595.0	fR/S	 30.7
U+3595.0	fTotalStrokes	 10
U+3595.0	fCangjie	RYTJ
U+3595.0	fFourCorner	60040
U+3595.0	fCantonese	san1; zip6; zit6
U+3595.0	fMandarin	e4; bo4; nie4
U+3595.0	fEnglish	[1] spit out [2] blame [3] the name of an organic compound
U+3596.0	fUTF8	㖖
U+3596.0	fCNS11643	6-357D
U+3596.0	fR/S	 30.7
U+3596.0	fTotalStrokes	 10; 9
U+3596.0	fCangjie	YTJR
U+3596.0	fFourCorner	00600
U+3596.0	fCantonese	ngot6; ot6 (also); saan1; zip6; zit6
U+3596.0	fMandarin	e4; nie4
U+3596.0	fEnglish	[1] (standard form) [2] spit out [3] blame [4] the name of an organic compound
U+3597.0	fUTF8	㖗
U+3597.0	fCNS11643	3-3024
U+3597.0	fR/S	 30.7
U+3597.0	fTotalStrokes	 10
U+3597.0	fCangjie	RUON
U+3597.0	fFourCorner	62020
U+3597.0	fHakka	 ngim2
U+3597.0	fCantonese	jam4; zaam4
U+3597.0	fMandarin	yin2
U+3597.0	fEnglish	same as &#x5D1F;
U+3598.0	fUTF8	㖘
U+3598.0	fCNS11643	5-2C43
U+3598.0	fGB	S-234B
U+3598.0	fR/S	 30.7
U+3598.0	fTotalStrokes	 10
U+3598.0	fCangjie	RMMV
U+3598.0	fFourCorner	61030
U+3598.0	fCantonese	seon2; san2 (also)
U+3598.0	fMandarin	zhen1
U+3598.0	fEnglish	same as &#x5507;
U+3599.0	fUTF8	㖙
U+3599.0	fR/S	 30.7
U+3599.0	fTotalStrokes	 10
U+3599.0	fCangjie	KRRP
U+3599.0	fCantonese	gaa1
U+359A.0	fUTF8	㖚
U+359A.0	fR/S	 30.7
U+359A.0	fTotalStrokes	 10
U+359A.0	fCangjie	OIRP
U+359A.0	fCantonese	fu6
U+359B.0	fUTF8	㖛
U+359B.0	fR/S	 30.7
U+359B.0	fTotalStrokes	 10
U+359B.0	fCangjie	JRRP
U+359B.0	fCantonese	gu2; gwu2 (also)
U+359C.0	fUTF8	㖜
U+359C.0	fR/S	 30.7
U+359C.0	fTotalStrokes	 10
U+359C.0	fCangjie	YRRP
U+359C.0	fCantonese	jing1
U+359D.0	fUTF8	㖝
U+359D.0	fR/S	 30.7
U+359D.0	fTotalStrokes	 10
U+359D.0	fCangjie	JDRP
U+359D.0	fCantonese	mei6
U+359E.0	fUTF8	㖞
U+359E.0	fGB	7-2250
U+359E.0	fR/S	 30.7
U+359E.0	fTotalStrokes	 10
U+359E.0	fCangjie	RROB
U+359E.0	fCantonese	waa1; wo5; wo3
U+359E.0	fMandarin	wai1
U+359E.0	fEnglish	a wry mouth
U+359F.0	fUTF8	㖟
U+359F.0	fCNS11643	3-345D
U+359F.0	fR/S	 30.8
U+359F.0	fTotalStrokes	 11
U+359F.0	fCangjie	RBBE
U+359F.0	fFourCorner	62040
U+359F.0	fCantonese	sau6
U+359F.0	fMandarin	shou4
U+359F.0	fEnglish	[1] deliver over to personally [2] to communicate orally with one's own mouth
U+35A0.0	fUTF8	㖠
U+35A0.0	fCNS11643	3-3465
U+35A0.0	fR/S	 30.8
U+35A0.0	fTotalStrokes	 11
U+35A0.0	fCangjie	RKMF
U+35A0.0	fFourCorner	64090
U+35A0.0	fHakka	 nai5; nai6; lai3
U+35A0.0	fCantonese	naa5; laa5 (also); nap6; no4
U+35A0.0	fMandarin	na3; ba4; nuo4
U+35A0.0	fEnglish	an interrogative particle; who; which; any
U+35A1.0	fUTF8	㖡
U+35A1.0	fCNS11643	4-306A
U+35A1.0	fGB	3-365A
U+35A1.0	fR/S	 30.8
U+35A1.0	fTotalStrokes	 11
U+35A1.0	fCangjie	RYOK
U+35A1.0	fFourCorner	60040
U+35A1.0	fHakka	 ja5; ra6; ja6
U+35A1.0	fCantonese	je6; nge6 (also); e6; ze6
U+35A1.0	fMandarin	ye4
U+35A1.0	fEnglish	the birds singing during the night
U+35A2.0	fUTF8	㖢
U+35A2.0	fCNS11643	5-3670
U+35A2.0	fR/S	 30.8; 30.9
U+35A2.0	fTotalStrokes	 12; 11
U+35A2.0	fCangjie	RIYP
U+35A2.0	fFourCorner	60010
U+35A2.0	fCantonese	ci1
U+35A2.0	fMandarin	ci1; ji2; qi2
U+35A2.0	fEnglish	same as &#x98FA;
U+35A3.0	fUTF8	㖣
U+35A3.0	fCNS11643	3-3463
U+35A3.0	fGB	5-386A
U+35A3.0	fR/S	 30.8
U+35A3.0	fTotalStrokes	 11
U+35A3.0	fCangjie	RYTR
U+35A3.0	fFourCorner	60060
U+35A3.0	fHakka	 teu5
U+35A3.0	fCantonese	tau3
U+35A3.0	fMandarin	pou3; tou4
U+35A3.0	fEnglish	same as &#x5485;
U+35A4.0	fUTF8	㖤
U+35A4.0	fCNS11643	3-345C
U+35A4.0	fR/S	 30.8
U+35A4.0	fTotalStrokes	 11
U+35A4.0	fCangjie	RUNE
U+35A4.0	fFourCorner	67070
U+35A4.0	fCantonese	haam4
U+35A4.0	fMandarin	han2
U+35A4.0	fEnglish	wrath; anger
U+35A5.0	fUTF8	㖥
U+35A5.0	fCNS11643	4-3070
U+35A5.0	fGB	3-3654
U+35A5.0	fR/S	 30.8
U+35A5.0	fTotalStrokes	 11
U+35A5.0	fCangjie	RWHD
U+35A5.0	fFourCorner	66000
U+35A5.0	fHakka	 kiun3
U+35A5.0	fCantonese	leon1; kwaan3; zeon1
U+35A5.0	fMandarin	jun3; jiong3
U+35A5.0	fEnglish	 spit; vomit
U+35A6.0	fUTF8	㖦
U+35A6.0	fCNS11643	3-345A
U+35A6.0	fR/S	 30.8
U+35A6.0	fTotalStrokes	 11
U+35A6.0	fCangjie	RDW
U+35A6.0	fFourCorner	65090
U+35A6.0	fCantonese	dung2
U+35A6.0	fMandarin	dong3
U+35A6.0	fEnglish	loquacity
U+35A7.0	fUTF8	㖧
U+35A7.0	fCNS11643	4-3073
U+35A7.0	fGB	3-3658
U+35A7.0	fR/S	 30.8
U+35A7.0	fTotalStrokes	 11
U+35A7.0	fCangjie	RHPA
U+35A7.0	fFourCorner	62060
U+35A7.0	fCantonese	fan1; man5; faan1
U+35A7.0	fMandarin	hun1; wen3
U+35A7.0	fEnglish	[1] can not see clear [2] ancient form of &#x543B;
U+35A8.0	fUTF8	㖨
U+35A8.0	fCNS11643	4-3071
U+35A8.0	fGB	3-365F
U+35A8.0	fR/S	 30.8
U+35A8.0	fTotalStrokes	 11
U+35A8.0	fCangjie	RVNE
U+35A8.0	fFourCorner	67030
U+35A8.0	fCantonese	luk6
U+35A8.0	fMandarin	lu4
U+35A8.0	fEnglish	[1] indistinct nasal utterance [2] laugh [3] sound of birds
U+35A9.0	fUTF8	㖩
U+35A9.0	fCNS11643	4-306D
U+35A9.0	fGB	3-3649
U+35A9.0	fR/S	 30.8
U+35A9.0	fTotalStrokes	 11
U+35A9.0	fCangjie	RSJE
U+35A9.0	fFourCorner	67040
U+35A9.0	fHakka	 ci5
U+35A9.0	fCantonese	zeoi1; zaau1
U+35A9.0	fMandarin	zou1; sou3
U+35A9.0	fEnglish	[1] not pure [2] immodest [3] urge [4] same as &#x55FE;
U+35AA.0	fUTF8	㖪
U+35AA.0	fCNS11643	4-306C
U+35AA.0	fGB	3-364B
U+35AA.0	fR/S	 30.8
U+35AA.0	fTotalStrokes	 11
U+35AA.0	fCangjie	RIRM
U+35AA.0	fFourCorner	63050
U+35AA.0	fCantonese	waak6; gwok3
U+35AA.0	fMandarin	yu4; guo2
U+35AA.0	fEnglish	[1] the sound of surprising [2] laugh loudly [3] flow off [4] loquacious [5] sound; voice; tone
U+35AB.0	fUTF8	㖫
U+35AB.0	fCNS11643	F-3343
U+35AB.0	fR/S	 30.8
U+35AB.0	fTotalStrokes	 11
U+35AB.0	fCangjie	RGCE
U+35AB.0	fFourCorner	6404
U+35AB.0	fCantonese	ling4
U+35AB.0	fMandarin	ling2
U+35AB.0	fEnglish	used in names for minority ethnic group in ancient China
U+35AC.0	fUTF8	㖬
U+35AC.0	fGB	S-2354
U+35AC.0	fR/S	 30.8
U+35AC.0	fTotalStrokes	 11
U+35AC.0	fCangjie	RHXE
U+35AC.0	fCantonese	sau1
U+35AD.0	fUTF8	㖭
U+35AD.0	fGB	S-2355
U+35AD.0	fR/S	 30.8
U+35AD.0	fTotalStrokes	 11
U+35AD.0	fCangjie	RHKP
U+35AD.0	fCantonese	tim1; tim2
U+35AD.0	fMandarin	tian3
U+35AD.0	fEnglish	same as &#x8214;
U+35AE.0	fUTF8	㖮
U+35AE.0	fR/S	 30.8
U+35AE.0	fTotalStrokes	 11
U+35AE.0	fCangjie	ROMB
U+35AE.0	fCantonese	leon4
U+35AE.0	fMandarin	lun2
U+35AE.0	fEnglish	an interjection used in poems and songs
U+35AF.0	fUTF8	㖯
U+35AF.0	fR/S	 30.8
U+35AF.0	fTotalStrokes	 11
U+35AF.0	fCangjie	BRRP
U+35AF.0	fCantonese	tung4
U+35B0.0	fUTF8	㖰
U+35B0.0	fR/S	 30.8
U+35B0.0	fTotalStrokes	 11
U+35B0.0	fCangjie	RPBMR
U+35B0.0	fCantonese	tung4
U+35B1.0	fUTF8	㖱
U+35B1.0	fR/S	 30.8
U+35B1.0	fTotalStrokes	 11
U+35B1.0	fCangjie	YOKR
U+35B1.0	fCantonese	je6
U+35B2.0	fUTF8	㖲
U+35B2.0	fR/S	 30.8
U+35B2.0	fTotalStrokes	 11
U+35B2.0	fCangjie	VRRP
U+35B2.0	fCantonese	jyu4
U+35B3.0	fUTF8	㖳
U+35B3.0	fR/S	 30.8
U+35B3.0	fTotalStrokes	 11
U+35B3.0	fCangjie	EVRP
U+35B3.0	fCantonese	jyu5
U+35B4.0	fUTF8	㖴
U+35B4.0	fCNS11643	3-3853
U+35B4.0	fR/S	 30.8; 140.7
U+35B4.0	fTotalStrokes	 11
U+35B4.0	fCangjie	RTPH
U+35B4.0	fFourCorner	44620
U+35B4.0	fCantonese	man5
U+35B4.0	fMandarin	wen3
U+35B5.0	fUTF8	㖵
U+35B5.0	fCNS11643	4-306F
U+35B5.0	fGB	3-3650
U+35B5.0	fR/S	 30.8
U+35B5.0	fTotalStrokes	 11
U+35B5.0	fCangjie	RBMC
U+35B5.0	fFourCorner	67080
U+35B5.0	fCantonese	got3
U+35B6.0	fUTF8	㖶
U+35B6.0	fCNS11643	5-3676
U+35B6.0	fR/S	 30.9
U+35B6.0	fTotalStrokes	 12
U+35B6.0	fCangjie	RMWG
U+35B6.0	fFourCorner	61010
U+35B6.0	fHakka	 jan1
U+35B6.0	fCantonese	jin1; zin1; zin3; zit3
U+35B6.0	fMandarin	ye4
U+35B6.0	fEnglish	[1] same as &#x6357; [2] same as &#x8AF2;
U+35B7.0	fUTF8	㖷
U+35B7.0	fCNS11643	3-3A32
U+35B7.0	fGB	5-3876
U+35B7.0	fR/S	 30.9
U+35B7.0	fTotalStrokes	 12
U+35B7.0	fCangjie	RAMO
U+35B7.0	fFourCorner	66080
U+35B7.0	fCantonese	si4; taai4
U+35B7.0	fMandarin	shi2; ti2
U+35B7.0	fEnglish	[1] a bird singing [2] same as &#x557C;
U+35B8.0	fUTF8	㖸
U+35B8.0	fCNS11643	4-3667
U+35B8.0	fGB	3-3665
U+35B8.0	fR/S	 30.9
U+35B8.0	fTotalStrokes	 12
U+35B8.0	fCangjie	RYPM
U+35B8.0	fFourCorner	61010
U+35B8.0	fHakka	 kiok7; nak7; gi3
U+35B8.0	fCantonese	laa1; koek6; kek3
U+35B8.0	fMandarin	jue1; xue2; jue2
U+35B8.0	fEnglish	[1] same as &#x5671; [2] [3] declarative or imperative expression
U+35B9.0	fUTF8	㖹
U+35B9.0	fCNS11643	3-3A35
U+35B9.0	fGB	5-3922
U+35B9.0	fR/S	 30.9
U+35B9.0	fTotalStrokes	 12
U+35B9.0	fCangjie	RCST
U+35B9.0	fFourCorner	68010
U+35B9.0	fCantonese	pan3; fu2; paan3; pun4
U+35B9.0	fMandarin	pen2; fen4
U+35B9.0	fEnglish	non-classical form of &#x5674;
U+35BA.0	fUTF8	㖺
U+35BA.0	fCNS11643	3-3A2C
U+35BA.0	fGB	5-3870
U+35BA.0	fR/S	 30.9
U+35BA.0	fTotalStrokes	 12
U+35BA.0	fCangjie	RQKA
U+35BA.0	fFourCorner	65060
U+35BA.0	fCantonese	ceon2
U+35BA.0	fMandarin	chun3
U+35BA.0	fEnglish	 blow; breath; puff
U+35BB.0	fUTF8	㖻
U+35BB.0	fCNS11643	4-365E
U+35BB.0	fGB	3-3676
U+35BB.0	fR/S	 30.9
U+35BB.0	fTotalStrokes	 12
U+35BB.0	fCangjie	RNHD
U+35BB.0	fFourCorner	67090
U+35BB.0	fHakka	 ngiu2; ju2; ru2; niu2
U+35BB.0	fCantonese	nau4; lau4 (also); zaau4
U+35BB.0	fMandarin	niu2; rou2
U+35BB.0	fEnglish	the cry of a child
U+35BC.0	fUTF8	㖼
U+35BC.0	fCNS11643	4-3659
U+35BC.0	fGB	5-3871
U+35BC.0	fR/S	 30.9
U+35BC.0	fTotalStrokes	 12
U+35BC.0	fCangjie	RTMD
U+35BC.0	fFourCorner	64090
U+35BC.0	fHakka	 meu2; miau3
U+35BC.0	fCantonese	mau4
U+35BC.0	fMandarin	mou2; duo3; lin2
U+35BC.0	fEnglish	same as &#x8B00;
U+35BD.0	fUTF8	㖽
U+35BD.0	fCNS11643	3-3A39
U+35BD.0	fR/S	 30.9
U+35BD.0	fTotalStrokes	 12
U+35BD.0	fCangjie	RMBC
U+35BD.0	fFourCorner	61080
U+35BD.0	fHakka	 zit7
U+35BD.0	fCantonese	zaak3; zik1
U+35BD.0	fMandarin	ze2; zei2
U+35BD.0	fEnglish	corrupted form of &#x5616;
U+35BE.0	fUTF8	㖾
U+35BE.0	fCNS11643	3-3A3E
U+35BE.0	fR/S	 30.9
U+35BE.0	fTotalStrokes	 12
U+35BE.0	fCangjie	RRTU
U+35BE.0	fFourCorner	66400
U+35BE.0	fHakka	 ngok8
U+35BE.0	fCantonese	ngok6; ok6 (also)
U+35BE.0	fMandarin	e4
U+35BE.0	fEnglish	standard form of &#x54A2;
U+35BF.0	fUTF8	㖿
U+35BF.0	fCNS11643	3-3A2D
U+35BF.0	fGB	5-385A
U+35BF.0	fR/S	 30.9
U+35BF.0	fTotalStrokes	 12
U+35BF.0	fCangjie	RSJL
U+35BF.0	fFourCorner	67027
U+35BF.0	fCantonese	je5; ce4; ze4
U+35BF.0	fMandarin	xie2
U+35BF.0	fEnglish	[1] sound; voice; tone [2] an interjection [3] hesitate; harbour doubts
U+35C0.0	fUTF8	㗀
U+35C0.0	fCNS11643	4-3660
U+35C0.0	fGB	3-366A
U+35C0.0	fR/S	 30.9
U+35C0.0	fTotalStrokes	 12
U+35C0.0	fCangjie	RUVI
U+35C0.0	fFourCorner	62070
U+35C0.0	fHakka	 ju2
U+35C0.0	fCantonese	jau1; zaau1
U+35C0.0	fMandarin	you1
U+35C0.0	fEnglish	same as &#x5466;, &#x5698;
U+35C1.0	fUTF8	㗁
U+35C1.0	fCNS11643	3-3A3D
U+35C1.0	fGB	5-3879
U+35C1.0	fR/S	 30.9
U+35C1.0	fTotalStrokes	 12
U+35C1.0	fCangjie	RRRS
U+35C1.0	fFourCorner	66020
U+35C1.0	fHakka	  ngok8; ngok7
U+35C1.0	fCantonese	ngok6; ok6 (also); zit6
U+35C1.0	fMandarin	e4
U+35C1.0	fEnglish	same as &#x9F76;
U+35C2.0	fUTF8	㗂
U+35C2.0	fCNS11643	4-365F
U+35C2.0	fR/S	 30.9
U+35C2.0	fTotalStrokes	 12
U+35C2.0	fCangjie	RFHU
U+35C2.0	fFourCorner	69060
U+35C2.0	fCantonese	saang2
U+35C2.0	fMandarin	sheng3
U+35C2.0	fEnglish	[1] keep the mouth shut; be careful how one speak [2] non-classical form of &#x7701;
U+35C3.0	fUTF8	㗃
U+35C3.0	fCNS11643	3-3A37
U+35C3.0	fGB	5-392A
U+35C3.0	fR/S	 30.9
U+35C3.0	fTotalStrokes	 12
U+35C3.0	fCangjie	RRPA
U+35C3.0	fFourCorner	67060
U+35C3.0	fHakka	 vut7
U+35C3.0	fCantonese	man5; maan4; maan5
U+35C3.0	fMandarin	wen3; yin4
U+35C3.0	fEnglish	same as &#x543B;
U+35C4.0	fUTF8	㗄
U+35C4.0	fCNS11643	6-4739
U+35C4.0	fGB	3-3668
U+35C4.0	fR/S	 30.9
U+35C4.0	fTotalStrokes	 12
U+35C4.0	fCangjie	RWLS
U+35C4.0	fFourCorner	66020
U+35C4.0	fHakka	 man5
U+35C4.0	fCantonese	ku1; kwu1 (also); maan4; man4; maan6; man6; ming4
U+35C4.0	fMandarin	man2; man4; mian2
U+35C4.0	fEnglish	non-classical form of &#x8B3E;
U+35C5.0	fUTF8	㗅
U+35C5.0	fCNS11643	5-3679
U+35C5.0	fGB	S-2356
U+35C5.0	fR/S	 30.9
U+35C5.0	fTotalStrokes	 12
U+35C5.0	fCangjie	RJRB
U+35C5.0	fFourCorner	67020
U+35C5.0	fCantonese	wu4
U+35C5.0	fMandarin	hu2
U+35C5.0	fEnglish	same as &#x80E1;
U+35C6.0	fUTF8	㗆
U+35C6.0	fCNS11643	5-367A
U+35C6.0	fR/S	 30.9
U+35C6.0	fTotalStrokes	 12
U+35C6.0	fCangjie	RTLJ
U+35C6.0	fFourCorner	64050
U+35C6.0	fCantonese	gaak3
U+35C6.0	fMandarin	ge2; kai4
U+35C6.0	fEnglish	[1] the sound of the ringed pheasant [2] cackling of fowls [3] same as &#x55DD;
U+35C7.0	fUTF8	㗇
U+35C7.0	fCNS11643	3-3A30
U+35C7.0	fR/S	 30.9
U+35C7.0	fTotalStrokes	 12
U+35C7.0	fCangjie	RRSE
U+35C7.0	fFourCorner	67040
U+35C7.0	fHakka	  ha2
U+35C7.0	fCantonese	haa1; gaa5; haa4
U+35C7.0	fMandarin	xia2
U+35C7.0	fEnglish	[1] throat; gullet [2] swallow
U+35C8.0	fUTF8	㗈
U+35C8.0	fCNS11643	4-3658
U+35C8.0	fGB	5-392B
U+35C8.0	fR/S	 30.9
U+35C8.0	fTotalStrokes	 12
U+35C8.0	fCangjie	RNBS
U+35C8.0	fFourCorner	67020
U+35C8.0	fCantonese	maan6
U+35C8.0	fMandarin	man1; man4
U+35C8.0	fEnglish	same as &#x66FC;
U+35C9.0	fUTF8	㗉
U+35C9.0	fCNS11643	4-3665
U+35C9.0	fGB	3-326D
U+35C9.0	fR/S	 30.9
U+35C9.0	fTotalStrokes	 12
U+35C9.0	fCangjie	QHHER
U+35C9.0	fFourCorner	57600
U+35C9.0	fCantonese	loek6
U+35C9.0	fMandarin	luue4; bi4; e4; le4
U+35C9.0	fEnglish	[1] incisive [2] large seal type of &#x7C40;
U+35CA.0	fUTF8	㗊
U+35CA.0	fCNS11643	4-3663
U+35CA.0	fGB	3-3667
U+35CA.0	fR/S	 30.9
U+35CA.0	fTotalStrokes	 12
U+35CA.0	fCangjie	RRRR
U+35CA.0	fFourCorner	66660
U+35CA.0	fCantonese	leoi4; lo4
U+35CA.0	fMandarin	ji2
U+35CA.0	fEnglish	[1] public opinion; clamour; noise [2] ancient form of &#x96F7;
U+35CB.0	fUTF8	㗋
U+35CB.0	fCNS11643	3-3A41
U+35CB.0	fR/S	 30.9
U+35CB.0	fTotalStrokes	 12
U+35CB.0	fCangjie	RNMK
U+35CB.0	fFourCorner	67030
U+35CB.0	fHakka	 heu2
U+35CB.0	fCantonese	hau4
U+35CB.0	fMandarin	hou2
U+35CB.0	fEnglish	standard form of &#x5589;
U+35CC.0	fUTF8	㗌
U+35CC.0	fCNS11643	5-366E
U+35CC.0	fR/S	 30.9
U+35CC.0	fTotalStrokes	 12
U+35CC.0	fCangjie	RJMG
U+35CC.0	fFourCorner	63010
U+35CC.0	fCantonese	zat1; zaat6
U+35CC.0	fMandarin	zhi4
U+35CC.0	fEnglish	[1] (non-classical form) [2] scold with loud voice [3] talk out of control; loquacious
U+35CD.0	fUTF8	㗍
U+35CD.0	fCNS11643	3-3A3F
U+35CD.0	fR/S	 30.9
U+35CD.0	fTotalStrokes	 12
U+35CD.0	fCangjie	RHDA
U+35CD.0	fFourCorner	62060
U+35CD.0	fCantonese	zaai1
U+35CD.0	fMandarin	zhai1
U+35CD.0	fEnglish	to gnaw; to bite; to masticate, sound of biting
U+35CE.0	fUTF8	㗎
U+35CE.0	fGB	S-234D
U+35CE.0	fR/S	 30.9
U+35CE.0	fTotalStrokes	 12
U+35CE.0	fCangjie	RKRD
U+35CE.0	fCantonese	gaa3; gaa4; gaa2
U+35CE.0	fEnglish	 [final particle] a contraction of ge3 a3, indicating emphasis in statements or past tense in
U+35CF.0	fUTF8	㗏
U+35CF.0	fGB	3-3664
U+35CF.0	fR/S	 30.9
U+35CF.0	fTotalStrokes	 12
U+35CF.0	fCangjie	RMFM
U+35CF.0	fCantonese	waai1
U+35D0.0	fUTF8	㗐
U+35D0.0	fCNS11643	3-3A3A
U+35D0.0	fR/S	 30.9
U+35D0.0	fTotalStrokes	 12
U+35D0.0	fCangjie	RKIT
U+35D0.0	fFourCorner	64010
U+35D0.0	fCantonese	haap3
U+35D0.0	fMandarin	ke4
U+35D1.0	fUTF8	㗑
U+35D1.0	fGB	3-366B
U+35D1.0	fR/S	 30.9
U+35D1.0	fTotalStrokes	 12
U+35D1.0	fCangjie	RHQJ
U+35D1.0	fFourCorner	6105
U+35D1.0	fCantonese	baai3; bai6
U+35D1.0	fMandarin	bai4
U+35D2.0	fUTF8	㗒
U+35D2.0	fGB	3-3671
U+35D2.0	fR/S	 30.10
U+35D2.0	fTotalStrokes	 13
U+35D2.0	fCangjie	RYRV
U+35D2.0	fCantonese	oi1; ngoi1 (also); oi2
U+35D2.0	fMandarin	ai4
U+35D2.0	fEnglish	interjection; to express sadness; sorrow and emotional excitement
U+35D3.0	fUTF8	㗓
U+35D3.0	fCNS11643	5-3E39
U+35D3.0	fGB	3-366E
U+35D3.0	fR/S	 30.10
U+35D3.0	fTotalStrokes	 13
U+35D3.0	fCangjie	RYHR
U+35D3.0	fFourCorner	67030
U+35D3.0	fHakka	 zui1; bau1
U+35D3.0	fCantonese	zeoi1; ci3
U+35D3.0	fMandarin	zhui1
U+35D3.0	fEnglish	mouthful
U+35D4.0	fUTF8	㗔
U+35D4.0	fCNS11643	4-3C4A
U+35D4.0	fGB	3-367B
U+35D4.0	fR/S	 30.10
U+35D4.0	fTotalStrokes	 13
U+35D4.0	fCangjie	RYPK
U+35D4.0	fFourCorner	61040
U+35D4.0	fCantonese	hin1
U+35D4.0	fMandarin	qian1
U+35D4.0	fEnglish	 joy; happiness
U+35D5.0	fUTF8	㗕
U+35D5.0	fCNS11643	4-3C46
U+35D5.0	fGB	3-3678
U+35D5.0	fR/S	 30.10
U+35D5.0	fTotalStrokes	 13
U+35D5.0	fCangjie	RTTB
U+35D5.0	fFourCorner	65040
U+35D5.0	fCantonese	gau3; gaau3; ngaau1
U+35D5.0	fMandarin	gou4
U+35D5.0	fEnglish	same as &#x96CA;
U+35D6.0	fUTF8	㗖
U+35D6.0	fCNS11643	3-404F
U+35D6.0	fR/S	 30.10
U+35D6.0	fTotalStrokes	 13
U+35D6.0	fCangjie	RBHX
U+35D6.0	fFourCorner	62070
U+35D6.0	fHakka	 tam3
U+35D6.0	fCantonese	daam6
U+35D6.0	fMandarin	dan4
U+35D6.0	fEnglish	corrupted form of &#x5557;
U+35D7.0	fUTF8	㗗
U+35D7.0	fCNS11643	5-3E3D
U+35D7.0	fGB	5-3938
U+35D7.0	fR/S	 30.10
U+35D7.0	fTotalStrokes	 13
U+35D7.0	fCangjie	RSHWJ
U+35D7.0	fFourCorner	66240
U+35D7.0	fCantonese	bei2; bai2; bei1
U+35D7.0	fMandarin	bai3
U+35D7.0	fEnglish	 separate; part, split; crack, rend; rip open
U+35D8.0	fUTF8	㗘
U+35D8.0	fCNS11643	4-3C47
U+35D8.0	fGB	3-3679
U+35D8.0	fR/S	 30.10
U+35D8.0	fTotalStrokes	 13
U+35D8.0	fCangjie	RIBI
U+35D8.0	fFourCorner	63040
U+35D8.0	fCantonese	bok3; bok6
U+35D8.0	fMandarin	bo2; fu4
U+35D8.0	fEnglish	 bite the hard and solid food
U+35D9.0	fUTF8	㗙
U+35D9.0	fCNS11643	4-3C4C
U+35D9.0	fGB	3-3573
U+35D9.0	fR/S	 30.10
U+35D9.0	fTotalStrokes	 13
U+35D9.0	fCangjie	RPUU
U+35D9.0	fFourCorner	67020
U+35D9.0	fCantonese	co1; zaau1
U+35D9.0	fMandarin	chu2; na4
U+35D9.0	fEnglish	[1] shout or bawl [2] talking manner of a mean person [3] same as &#x8B05;
U+35DA.0	fUTF8	㗚
U+35DA.0	fCNS11643	3-404D
U+35DA.0	fR/S	 30.10
U+35DA.0	fTotalStrokes	 13
U+35DA.0	fCangjie	RMWD
U+35DA.0	fFourCorner	61090
U+35DA.0	fCantonese	leot6; lei6
U+35DA.0	fMandarin	li4
U+35DA.0	fEnglish	vexingly verbose or wordy; prosy; complicated; annoying
U+35DB.0	fUTF8	㗛
U+35DB.0	fCNS11643	3-4050
U+35DB.0	fGB	5-3932
U+35DB.0	fR/S	 30.10
U+35DB.0	fTotalStrokes	 13
U+35DB.0	fCangjie	RHHK
U+35DB.0	fFourCorner	68030
U+35DB.0	fHakka	 siau5
U+35DB.0	fCantonese	siu3
U+35DB.0	fMandarin	xiao4
U+35DB.0	fEnglish	same as &#x7B11;
U+35DC.0	fUTF8	㗜
U+35DC.0	fCNS11643	6-5132
U+35DC.0	fR/S	 30.10
U+35DC.0	fTotalStrokes	 13
U+35DC.0	fCangjie	RYVW
U+35DC.0	fFourCorner	60060
U+35DC.0	fHakka	 chiu5
U+35DC.0	fCantonese	cau3
U+35DC.0	fMandarin	xiu4
U+35DC.0	fEnglish	same as &#x9F45;
U+35DD.0	fUTF8	㗝
U+35DD.0	fGB	S-2348
U+35DD.0	fR/S	 30.10
U+35DD.0	fTotalStrokes	 13
U+35DD.0	fCangjie	RTGF
U+35DD.0	fCantonese	gou1
U+35DE.0	fUTF8	㗞
U+35DE.0	fGB	S-234A
U+35DE.0	fR/S	 30.10
U+35DE.0	fTotalStrokes	 13
U+35DE.0	fCangjie	RMRE
U+35DE.0	fCantonese	po3
U+35DF.0	fUTF8	㗟
U+35DF.0	fR/S	 30.10
U+35DF.0	fTotalStrokes	 13
U+35DF.0	fCangjie	EGRP
U+35DF.0	fCantonese	zyu3
U+35E0.0	fUTF8	㗠
U+35E0.0	fR/S	 30.10
U+35E0.0	fTotalStrokes	 13
U+35E0.0	fCangjie	VRRP
U+35E0.0	fCantonese	ci2
U+35E1.0	fUTF8	㗡
U+35E1.0	fR/S	 30.10
U+35E1.0	fTotalStrokes	 13
U+35E1.0	fCangjie	TOSP
U+35E1.0	fCantonese	jing4
U+35E2.0	fUTF8	㗢
U+35E2.0	fCNS11643	4-4252
U+35E2.0	fGB	3-3579
U+35E2.0	fR/S	 30.11
U+35E2.0	fTotalStrokes	 14
U+35E2.0	fCangjie	RHGS
U+35E2.0	fFourCorner	64020
U+35E2.0	fCantonese	hung4; hung1
U+35E2.0	fMandarin	dong4
U+35E2.0	fEnglish	[1] loquacious [2] the sound of singing [3] sing in a loud voice [4] loud
U+35E3.0	fUTF8	㗣
U+35E3.0	fCNS11643	3-4632
U+35E3.0	fR/S	 30.11
U+35E3.0	fTotalStrokes	 14
U+35E3.0	fCangjie	RKPB
U+35E3.0	fFourCorner	64020
U+35E3.0	fCantonese	tai3; daai3; taai3
U+35E3.0	fMandarin	di4; ti4
U+35E3.0	fEnglish	same as &#x568F;
U+35E4.0	fUTF8	㗤
U+35E4.0	fCNS11643	3-462F
U+35E4.0	fGB	5-393C
U+35E4.0	fR/S	 30.11
U+35E4.0	fTotalStrokes	 14
U+35E4.0	fCangjie	IFR
U+35E4.0	fFourCorner	73600
U+35E4.0	fHakka	 zap7
U+35E4.0	fCantonese	zuk1; cuk1
U+35E4.0	fMandarin	cu4
U+35E4.0	fEnglish	[1] ashamed [2] restless; fidgeting [3] bashful [4] grievous; mournful; sad
U+35E5.0	fUTF8	㗥
U+35E5.0	fCNS11643	3-462D
U+35E5.0	fGB	5-3934
U+35E5.0	fR/S	 30.11
U+35E5.0	fTotalStrokes	 14
U+35E5.0	fCangjie	RYDL
U+35E5.0	fFourCorner	67020
U+35E5.0	fCantonese	kwok3; kok3 (also)
U+35E5.0	fMandarin	kuo4
U+35E5.0	fEnglish	[1] the sound of striking [2] slow expiration of the breath [3] a deep sign
U+35E6.0	fUTF8	㗦
U+35E6.0	fCNS11643	4-424F
U+35E6.0	fGB	3-3724
U+35E6.0	fR/S	 30.11
U+35E6.0	fTotalStrokes	 14
U+35E6.0	fCangjie	RSJL
U+35E6.0	fFourCorner	67020
U+35E6.0	fHakka	 liau2; lau2
U+35E6.0	fCantonese	lou4
U+35E6.0	fMandarin	lao2
U+35E6.0	fEnglish	great noise
U+35E7.0	fUTF8	㗧
U+35E7.0	fCNS11643	4-424A
U+35E7.0	fGB	3-372E
U+35E7.0	fR/S	 30.11
U+35E7.0	fTotalStrokes	 14
U+35E7.0	fCangjie	RJCG
U+35E7.0	fFourCorner	63010
U+35E7.0	fCantonese	zat1; zaat6
U+35E7.0	fMandarin	zhi4
U+35E7.0	fEnglish	[1] scold with loud voice [2] talk out of control; loquacious
U+35E8.0	fUTF8	㗨
U+35E8.0	fCNS11643	4-4255
U+35E8.0	fGB	3-3725
U+35E8.0	fR/S	 30.11
U+35E8.0	fTotalStrokes	 14
U+35E8.0	fCangjie	SER
U+35E8.0	fFourCorner	17600
U+35E8.0	fCantonese	haai1; aai2; hai6
U+35E8.0	fMandarin	ai3
U+35E8.0	fEnglish	[1] laugh; to smile [2] laugh at; ridicule [3] the sound of laughter
U+35E9.0	fUTF8	㗩
U+35E9.0	fCNS11643	3-462B
U+35E9.0	fR/S	 30.11
U+35E9.0	fTotalStrokes	 14
U+35E9.0	fCangjie	RSMA
U+35E9.0	fFourCorner	67060
U+35E9.0	fHakka	  sip8
U+35E9.0	fCantonese	sap1; ci1
U+35E9.0	fMandarin	xi4; xi1
U+35E9.0	fEnglish	the noise of shivering with cold
U+35EA.0	fUTF8	㗪
U+35EA.0	fCNS11643	3-4B3A
U+35EA.0	fR/S	 30.12; 30.11
U+35EA.0	fTotalStrokes	 15; 14
U+35EA.0	fCangjie	RYPF
U+35EA.0	fFourCorner	61090
U+35EA.0	fCantonese	pei1
U+35EA.0	fMandarin	pi1
U+35EA.0	fEnglish	mouthful
U+35EB.0	fUTF8	㗫
U+35EB.0	fCNS11643	4-4253
U+35EB.0	fGB	5-393F
U+35EB.0	fR/S	 30.11
U+35EB.0	fTotalStrokes	 14
U+35EB.0	fCangjie	RBOF
U+35EB.0	fFourCorner	67090
U+35EB.0	fCantonese	cit3; caai3; cam1; saam6
U+35EB.0	fMandarin	qie1; qie4
U+35EB.0	fEnglish	 whisper; talk in a soft and low voice
U+35EC.0	fUTF8	㗬
U+35EC.0	fCNS11643	4-4256
U+35EC.0	fGB	5-3C69
U+35EC.0	fR/S	 30.11
U+35EC.0	fTotalStrokes	 14
U+35EC.0	fCangjie	FBRNN
U+35EC.0	fFourCorner	90200
U+35EC.0	fCantonese	zaa1
U+35EC.0	fMandarin	zha1
U+35EC.0	fEnglish	[1] thick lips [2] talk slowly
U+35ED.0	fUTF8	㗭
U+35ED.0	fCNS11643	4-4251
U+35ED.0	fGB	3-372C
U+35ED.0	fR/S	 30.11
U+35ED.0	fTotalStrokes	 14
U+35ED.0	fCangjie	RHDP
U+35ED.0	fFourCorner	62039
U+35ED.0	fHakka	 sit7
U+35ED.0	fCantonese	sik1; hei1
U+35ED.0	fMandarin	xi1
U+35ED.0	fEnglish	the sound produced by any creature
U+35EE.0	fUTF8	㗮
U+35EE.0	fR/S	 30.11
U+35EE.0	fTotalStrokes	 14
U+35EE.0	fCangjie	NRILO
U+35EE.0	fCantonese	gang1
U+35EF.0	fUTF8	㗯
U+35EF.0	fR/S	 30.11
U+35EF.0	fTotalStrokes	 14
U+35EF.0	fCangjie	JARP
U+35EF.0	fCantonese	ze2
U+35F0.0	fUTF8	㗰
U+35F0.0	fCNS11643	5-4522
U+35F0.0	fGB	3-355C
U+35F0.0	fR/S	 30.11
U+35F0.0	fTotalStrokes	 14
U+35F0.0	fCangjie	RHOO
U+35F0.0	fFourCorner	68080
U+35F0.0	fCantonese	cung4; saai3
U+35F0.0	fMandarin	chu4; cong2
U+35F0.0	fEnglish	[1] character used in transliteration [2] traditional drama of Vietnamese
U+35F1.0	fUTF8	㗱
U+35F1.0	fCNS11643	4-4930
U+35F1.0	fGB	3-3734
U+35F1.0	fR/S	 30.12
U+35F1.0	fTotalStrokes	 15
U+35F1.0	fCangjie	ROGD
U+35F1.0	fFourCorner	60090
U+35F1.0	fCantonese	zaap6; zep6 (also); zap6
U+35F1.0	fMandarin	ji2
U+35F1.0	fEnglish	[1] suck; chew [2] smear the mouth with the blood of a victim when taking an oath
U+35F2.0	fUTF8	㗲
U+35F2.0	fCNS11643	4-492C
U+35F2.0	fGB	3-364C
U+35F2.0	fR/S	 30.12
U+35F2.0	fTotalStrokes	 15
U+35F2.0	fCangjie	RLGM
U+35F2.0	fFourCorner	65010
U+35F2.0	fHakka	 wak8; vak8; wok7
U+35F2.0	fCantonese	waak1; waak6; wok6
U+35F2.0	fMandarin	huo4; xi4
U+35F2.0	fEnglish	 bawl after; shout
U+35F3.0	fUTF8	㗳
U+35F3.0	fCNS11643	3-4B42
U+35F3.0	fR/S	 30.12
U+35F3.0	fTotalStrokes	 15
U+35F3.0	fCangjie	RHOR
U+35F3.0	fFourCorner	68060
U+35F3.0	fHakka	 tap7
U+35F3.0	fCantonese	dap1; taap3; tap3
U+35F3.0	fMandarin	ta3; da5
U+35F3.0	fEnglish	same as &#x5854;
U+35F4.0	fUTF8	㗴
U+35F4.0	fCNS11643	4-492D
U+35F4.0	fGB	5-3856
U+35F4.0	fR/S	 30.12
U+35F4.0	fTotalStrokes	 15
U+35F4.0	fCangjie	RANB
U+35F4.0	fFourCorner	67020
U+35F4.0	fCantonese	ngaan4; aan4 (also); zin4
U+35F4.0	fMandarin	yan2
U+35F4.0	fEnglish	[1] dogs fighting [2] go to law; indictment
U+35F5.0	fUTF8	㗵
U+35F5.0	fCNS11643	3-4B36
U+35F5.0	fGB	5-394F
U+35F5.0	fR/S	 30.12
U+35F5.0	fTotalStrokes	 15
U+35F5.0	fCangjie	RFFO
U+35F5.0	fFourCorner	67080
U+35F5.0	fCantonese	fat1
U+35F5.0	fMandarin	hu1; xu4
U+35F5.0	fEnglish	[1] breath [2] air [3] steam [4] not clear; not quite sure
U+35F6.0	fUTF8	㗶
U+35F6.0	fCNS11643	7-214E
U+35F6.0	fR/S	 30.12
U+35F6.0	fTotalStrokes	 15
U+35F6.0	fCangjie	RNOE
U+35F6.0	fFourCorner	62047
U+35F6.0	fCantonese	put3
U+35F6.0	fMandarin	po1
U+35F6.0	fEnglish	same as &#x20DD1;
U+35F7.0	fUTF8	㗷
U+35F7.0	fGB	S-234F
U+35F7.0	fR/S	 30.12
U+35F7.0	fTotalStrokes	 15
U+35F7.0	fCangjie	RJTO
U+35F7.0	fCantonese	coi3
U+35F7.0	fMandarin	sai3
U+35F7.0	fEnglish	to eat; to bite; to gnaw, a dialect, usually used at the end of a sentence
U+35F8.0	fUTF8	㗸
U+35F8.0	fR/S	 30.12
U+35F8.0	fTotalStrokes	 15
U+35F8.0	fCangjie	RHON
U+35F8.0	fCantonese	haam4
U+35F9.0	fUTF8	㗹
U+35F9.0	fR/S	 30.12
U+35F9.0	fTotalStrokes	 15
U+35F9.0	fCangjie	ROGP
U+35F9.0	fCantonese	wai4
U+35FA.0	fUTF8	㗺
U+35FA.0	fGB	3-3730
U+35FA.0	fR/S	 30.12
U+35FA.0	fTotalStrokes	 15
U+35FA.0	fCangjie	RMKY
U+35FA.0	fCantonese	saap3
U+35FB.0	fUTF8	㗻
U+35FB.0	fCNS11643	4-4F6B
U+35FB.0	fGB	3-3623
U+35FB.0	fR/S	 30.13
U+35FB.0	fTotalStrokes	 16
U+35FB.0	fCangjie	RYBB
U+35FB.0	fFourCorner	67030
U+35FB.0	fHakka	 ko1; wo1; vo1
U+35FB.0	fCantonese	go6; gwo1; gwo3
U+35FB.0	fMandarin	ge1
U+35FB.0	fEnglish	[1] baby's answering [2] eat (baby talk)
U+35FC.0	fUTF8	㗼
U+35FC.0	fCNS11643	4-4F6C
U+35FC.0	fGB	3-356B
U+35FC.0	fR/S	 30.13
U+35FC.0	fTotalStrokes	 16
U+35FC.0	fCangjie	RTCD
U+35FC.0	fFourCorner	62090
U+35FC.0	fHakka	 ngiap8
U+35FC.0	fCantonese	jip6; zip6
U+35FC.0	fMandarin	ye4
U+35FC.0	fEnglish	[1] eating or talking [2] moving of the mouth
U+35FD.0	fUTF8	㗽
U+35FD.0	fCNS11643	3-5041
U+35FD.0	fGB	S-2358
U+35FD.0	fR/S	 30.13; 30.12
U+35FD.0	fTotalStrokes	 16; 15
U+35FD.0	fCangjie	VLR
U+35FD.0	fFourCorner	27600
U+35FD.0	fCantonese	hoeng2; hoeng1
U+35FD.0	fMandarin	xiang3
U+35FD.0	fEnglish	standard form of &#x97FF;
U+35FE.0	fUTF8	㗾
U+35FE.0	fCNS11643	4-4F67
U+35FE.0	fGB	3-373C
U+35FE.0	fR/S	 30.13
U+35FE.0	fTotalStrokes	 16
U+35FE.0	fCangjie	RTJP
U+35FE.0	fFourCorner	61010
U+35FE.0	fCantonese	hoe1; hoe4; heoi1
U+35FE.0	fMandarin	xue1
U+35FE.0	fEnglish	 give vent to the feelings
U+35FF.0	fUTF8	㗿
U+35FF.0	fCNS11643	4-4F6A
U+35FF.0	fGB	3-365B
U+35FF.0	fR/S	 30.13
U+35FF.0	fTotalStrokes	 16
U+35FF.0	fCangjie	RANR
U+35FF.0	fFourCorner	67020
U+35FF.0	fHakka	 ha5; ho3
U+35FF.0	fCantonese	ho1; laa4
U+35FF.0	fMandarin	xia1; xia4
U+35FF.0	fEnglish	noisy talk or laughter
U+3600.0	fUTF8	㘀
U+3600.0	fCNS11643	4-4F6F
U+3600.0	fGB	3-3733
U+3600.0	fR/S	 30.13
U+3600.0	fTotalStrokes	 16
U+3600.0	fCangjie	RHXD
U+3600.0	fFourCorner	67090
U+3600.0	fCantonese	zok3
U+3600.0	fMandarin	zuo2; zuo4
U+3600.0	fEnglish	to sound, a sound, the cry of a bird or animal
U+3601.0	fUTF8	㘁
U+3601.0	fCNS11643	3-503C
U+3601.0	fR/S	 30.13
U+3601.0	fTotalStrokes	 16
U+3601.0	fCangjie	RWLJ
U+3601.0	fFourCorner	66040
U+3601.0	fCantonese	jik6; zik6
U+3601.0	fMandarin	yi4
U+3601.0	fEnglish	[1] call; scream [2] whistle [3] a name of a stream
U+3602.0	fUTF8	㘂
U+3602.0	fCNS11643	3-503A
U+3602.0	fR/S	 30.13
U+3602.0	fTotalStrokes	 16
U+3602.0	fCangjie	RTVP
U+3602.0	fFourCorner	68030
U+3602.0	fCantonese	ci4
U+3602.0	fMandarin	ci2
U+3602.0	fEnglish	ashamed; bashful
U+3603.0	fUTF8	㘃
U+3603.0	fGB	S-2349
U+3603.0	fR/S	 30.13
U+3603.0	fTotalStrokes	 16
U+3603.0	fCangjie	RTKP
U+3603.0	fCantonese	je5
U+3603.0	fEnglish	 here; this
U+3604.0	fUTF8	㘄
U+3604.0	fGB	3-373E
U+3604.0	fR/S	 30.13
U+3604.0	fTotalStrokes	 16
U+3604.0	fCangjie	RDWS
U+3604.0	fCantonese	lang1
U+3605.0	fUTF8	㘅
U+3605.0	fCNS11643	7-3737
U+3605.0	fR/S	 30.14
U+3605.0	fTotalStrokes	 17
U+3605.0	fCangjie	RHON
U+3605.0	fFourCorner	61020
U+3605.0	fHakka	 ham2
U+3605.0	fCantonese	haam4
U+3605.0	fMandarin	xian2
U+3605.0	fEnglish	same as &#x929C;, &#x5563;
U+3606.0	fUTF8	㘆
U+3606.0	fCNS11643	4-5572
U+3606.0	fGB	3-3745
U+3606.0	fR/S	 30.14
U+3606.0	fTotalStrokes	 17; 16
U+3606.0	fCangjie	RGRG
U+3606.0	fFourCorner	64010
U+3606.0	fHakka	 toi2; toi3
U+3606.0	fCantonese	toi2; toi4
U+3606.0	fMandarin	tai3; tai2
U+3606.0	fEnglish	[1] chatter [2] mock
U+3607.0	fUTF8	㘇
U+3607.0	fCNS11643	3-5446
U+3607.0	fR/S	 30.14
U+3607.0	fTotalStrokes	 17
U+3607.0	fCangjie	RFFD
U+3607.0	fFourCorner	69090
U+3607.0	fCantonese	wing4; zing2
U+3607.0	fMandarin	yong2; rong2
U+3608.0	fUTF8	㘈
U+3608.0	fCNS11643	4-5573
U+3608.0	fGB	3-3746
U+3608.0	fR/S	 30.14
U+3608.0	fTotalStrokes	 17; 16
U+3608.0	fCangjie	RPKO
U+3608.0	fFourCorner	67080
U+3608.0	fCantonese	jik6; zi4
U+3608.0	fMandarin	ni3; yi1
U+3608.0	fEnglish	[1] able to take a correct view of things; response correctly (said of small children) [2] sound; echo
U+3609.0	fUTF8	㘉
U+3609.0	fCNS11643	4-4F6D
U+3609.0	fGB	3-3566
U+3609.0	fR/S	 30.15; 30.13
U+3609.0	fTotalStrokes	 16; 18
U+3609.0	fCangjie	RHAL
U+3609.0	fFourCorner	68020
U+3609.0	fCantonese	zik1; zit1; zit3
U+3609.0	fMandarin	zhi4
U+3609.0	fEnglish	 make sound; speak
U+360A.0	fUTF8	㘊
U+360A.0	fCNS11643	5-6140
U+360A.0	fGB	3-3748
U+360A.0	fR/S	 30.15
U+360A.0	fTotalStrokes	 18
U+360A.0	fCangjie	RMBR
U+360A.0	fFourCorner	61060
U+360A.0	fCantonese	kap1; caau3; kaap1; zaau2; zuk1
U+360A.0	fMandarin	yi4
U+360A.0	fEnglish	the abrupt and hasty sound of the crowd
U+360B.0	fUTF8	㘋
U+360B.0	fCNS11643	4-5A70
U+360B.0	fGB	5-3861
U+360B.0	fR/S	 30.15
U+360B.0	fTotalStrokes	 18
U+360B.0	fCangjie	RSEC
U+360B.0	fFourCorner	67080
U+360B.0	fCantonese	jin4; zin4
U+360B.0	fMandarin	xian2
U+360B.0	fEnglish	[1] sound; echo [2] difficult; troublesome
U+360C.0	fUTF8	㘌
U+360C.0	fCNS11643	3-5756
U+360C.0	fGB	5-396C
U+360C.0	fR/S	 30.15
U+360C.0	fTotalStrokes	 18
U+360C.0	fCangjie	RYON
U+360C.0	fFourCorner	62000
U+360C.0	fCantonese	koek6; kek6
U+360C.0	fMandarin	ju4
U+360C.0	fEnglish	non-classical form of &#x5671;
U+360D.0	fUTF8	㘍
U+360D.0	fCNS11643	5-6145
U+360D.0	fGB	5-396D
U+360D.0	fR/S	 30.15
U+360D.0	fTotalStrokes	 18
U+360D.0	fCangjie	RFGI
U+360D.0	fFourCorner	63050
U+360D.0	fHakka	 zit7; zit8
U+360D.0	fCantonese	zat1; zik1
U+360D.0	fMandarin	ji2; qi4
U+360D.0	fEnglish	the noise of mice and of insects
U+360E.0	fUTF8	㘎
U+360E.0	fGB	7-2249
U+360E.0	fR/S	 30.15
U+360E.0	fTotalStrokes	 18
U+360E.0	fCangjie	RLSK
U+360E.0	fCantonese	haam3; haam2 (also); ham3
U+360E.0	fMandarin	han3; han4
U+360E.0	fEnglish	the roars of a tiger
U+360F.0	fUTF8	㘏
U+360F.0	fR/S	 30.15
U+360F.0	fTotalStrokes	 18
U+360F.0	fCangjie	YURP
U+360F.0	fFourCorner	5208
U+360F.0	fCantonese	dou6
U+3610.0	fUTF8	㘐
U+3610.0	fCNS11643	4-5F4C
U+3610.0	fGB	3-3677
U+3610.0	fR/S	 30.16
U+3610.0	fTotalStrokes	 19
U+3610.0	fCangjie	RSFK
U+3610.0	fFourCorner	60040
U+3610.0	fCantonese	paau3; cit3; pau3
U+3610.0	fMandarin	pao4; peng3
U+3610.0	fEnglish	[1] loud [2] roar
U+3611.0	fUTF8	㘑
U+3611.0	fCNS11643	5-6759
U+3611.0	fR/S	 30.16
U+3611.0	fTotalStrokes	 19
U+3611.0	fCangjie	RGFE
U+3611.0	fFourCorner	65030
U+3611.0	fCantonese	lai6
U+3611.0	fMandarin	li4
U+3611.0	fEnglish	[1] curse [2] swear and oath [3] incantations
U+3612.0	fUTF8	㘒
U+3612.0	fR/S	 30.16
U+3612.0	fTotalStrokes	 19
U+3612.0	fCangjie	HGRP
U+3612.0	fCantonese	zung2
U+3613.0	fUTF8	㘓
U+3613.0	fCNS11643	5-6B73
U+3613.0	fGB	3-3738
U+3613.0	fR/S	 30.17
U+3613.0	fTotalStrokes	 20
U+3613.0	fCangjie	RANW
U+3613.0	fFourCorner	67020
U+3613.0	fHakka	 lan2; lan1
U+3613.0	fCantonese	laan4
U+3613.0	fMandarin	lan2
U+3613.0	fEnglish	[1] confused talk, gibberish [2] make a false charge
U+3614.0	fUTF8	㘔
U+3614.0	fCNS11643	3-5C35
U+3614.0	fR/S	 30.17
U+3614.0	fTotalStrokes	 20
U+3614.0	fCangjie	RJTC
U+3614.0	fFourCorner	63080
U+3614.0	fCantonese	coi3
U+3614.0	fMandarin	can3; sai3
U+3614.0	fEnglish	[1] eat; bite; gnaw [2] used at the end of a sentence
U+3615.0	fUTF8	㘕
U+3615.0	fCNS11643	3-5E27
U+3615.0	fR/S	 30.18
U+3615.0	fTotalStrokes	 21
U+3615.0	fCangjie	RTST
U+3615.0	fFourCorner	64010
U+3615.0	fCantonese	haam3; laam6; ham3; lam4
U+3615.0	fMandarin	han3; lan4; lan2
U+3615.0	fEnglish	same as &#x5682;
U+3616.0	fUTF8	㘖
U+3616.0	fCNS11643	4-6632
U+3616.0	fGB	3-374C
U+3616.0	fR/S	 30.18
U+3616.0	fTotalStrokes	 21
U+3616.0	fCangjie	RYHC
U+3616.0	fFourCorner	61080
U+3616.0	fHakka	 ngien2; ngian2
U+3616.0	fCantonese	ngaan4; aan4 (also); ngan4
U+3616.0	fMandarin	yan2
U+3616.0	fEnglish	 wrangle
U+3617.0	fUTF8	㘗
U+3617.0	fGB	3-3750
U+3617.0	fR/S	 30.18
U+3617.0	fTotalStrokes	 21
U+3617.0	fCangjie	RBUG
U+3617.0	fCantonese	keoi1
U+3617.0	fEnglish	[1] whisper [2] whistle
U+3618.0	fUTF8	㘘
U+3618.0	fR/S	 30.19
U+3618.0	fTotalStrokes	 22
U+3618.0	fCangjie	VFR
U+3618.0	fCantonese	lyun2
U+3619.0	fUTF8	㘙
U+3619.0	fCNS11643	3-602E
U+3619.0	fGB	5-384B
U+3619.0	fR/S	 30.20
U+3619.0	fTotalStrokes	 23
U+3619.0	fCangjie	RRRK
U+3619.0	fFourCorner	66040
U+3619.0	fCantonese	ngaam4; aam4 (also); gau1; gau2; hau6; ngam4
U+3619.0	fMandarin	yan2; yan3
U+3619.0	fEnglish	 groan; moan
U+361A.0	fUTF8	㘚
U+361A.0	fCNS11643	3-602D
U+361A.0	fGB	5-3965
U+361A.0	fR/S	 30.20
U+361A.0	fTotalStrokes	 23
U+361A.0	fCangjie	RANK
U+361A.0	fFourCorner	67020
U+361A.0	fCantonese	haam3; haam2 (also); gam1; ham3
U+361A.0	fMandarin	han3
U+361A.0	fEnglish	[1] angry air of look; look furious [2] the roars of a tiger
U+361B.0	fUTF8	㘛
U+361B.0	fCNS11643	4-6C6C
U+361B.0	fGB	3-372A
U+361B.0	fR/S	 30.22
U+361B.0	fTotalStrokes	 25
U+361B.0	fCangjie	RORP
U+361B.0	fFourCorner	68010
U+361B.0	fHakka	 kam5
U+361B.0	fCantonese	ham1; haam1; lim5
U+361B.0	fMandarin	kan1
U+361B.0	fEnglish	non-classical form of &#x5D4C;
U+361C.0	fUTF8	㘜
U+361C.0	fCNS11643	7-622F
U+361C.0	fR/S	 30.23
U+361C.0	fTotalStrokes	 26
U+361C.0	fCangjie	OGR
U+361C.0	fFourCorner	20600
U+361C.0	fHakka	 shiu2; chiu2; su2; cu2
U+361C.0	fCantonese	cau4; caau1; caau4
U+361C.0	fMandarin	chou2; chi3; chu4
U+361C.0	fEnglish	non-classical form of &#x96D4;
U+361D.0	fUTF8	㘝
U+361D.0	fCNS11643	3-2232
U+361D.0	fGB	5-3975
U+361D.0	fR/S	 31.2
U+361D.0	fTotalStrokes	 5
U+361D.0	fCangjie	WE
U+361D.0	fFourCorner	60400
U+361D.0	fHakka	 ngiap8; ngiap7
U+361D.0	fCantonese	naap6; laap6 (also); nim2; nip6; zik1
U+361D.0	fMandarin	ni3; nian3; nie4
U+361D.0	fEnglish	 filch; secrete
U+361E.0	fUTF8	㘞
U+361E.0	fCNS11643	3-2221
U+361E.0	fR/S	 19.3
U+361E.0	fTotalStrokes	 5
U+361E.0	fCangjie	WKS
U+361E.0	fFourCorner	60420
U+361E.0	fCantonese	wo6; deot1
U+361E.0	fMandarin	he4; huo4
U+361E.0	fEnglish	[1] yelling sound of towing a boat [2] call out (Sanskrit)
U+361F.0	fUTF8	㘟
U+361F.0	fCNS11643	3-233D
U+361F.0	fR/S	 31.3
U+361F.0	fTotalStrokes	 6
U+361F.0	fCangjie	WT
U+361F.0	fFourCorner	60440
U+361F.0	fCantonese	coeng1
U+361F.0	fMandarin	chuang1
U+3620.0	fUTF8	㘠
U+3620.0	fCNS11643	5-2553
U+3620.0	fR/S	 31.5
U+3620.0	fTotalStrokes	 8
U+3620.0	fCangjie	WPH
U+3620.0	fFourCorner	60330
U+3620.0	fCantonese	bik1; baai3; dai1
U+3620.0	fMandarin	bi1; bi4; mi4
U+3620.0	fEnglish	 close; stop up; obstruct
U+3621.0	fUTF8	㘡
U+3621.0	fCNS11643	3-277E
U+3621.0	fR/S	 31.5
U+3621.0	fTotalStrokes	 8
U+3621.0	fCangjie	WWL
U+3621.0	fFourCorner	60500
U+3621.0	fHakka	 ap7
U+3621.0	fCantonese	haap6
U+3621.0	fMandarin	xia2
U+3621.0	fEnglish	ancient form of &#x67D9;
U+3622.0	fUTF8	㘢
U+3622.0	fCNS11643	3-2B64
U+3622.0	fR/S	 31.6
U+3622.0	fTotalStrokes	 9
U+3622.0	fCangjie	WFMU
U+3622.0	fFourCorner	60210
U+3622.0	fCantonese	wang2
U+3622.0	fMandarin	weng3
U+3622.0	fEnglish	round cave; round hole
U+3623.0	fUTF8	㘣
U+3623.0	fCNS11643	3-3026
U+3623.0	fGB	5-397B
U+3623.0	fR/S	 31.7
U+3623.0	fTotalStrokes	 10
U+3623.0	fCangjie	WRB
U+3623.0	fFourCorner	60220
U+3623.0	fCantonese	syun4; zyun4
U+3623.0	fMandarin	yuan1; xuan2; yuan2
U+3623.0	fEnglish	[1] compasses [2] same as &#x5713;
U+3624.0	fUTF8	㘤
U+3624.0	fCNS11643	4-4258
U+3624.0	fGB	3-375D
U+3624.0	fR/S	 31.11
U+3624.0	fTotalStrokes	 14
U+3624.0	fCangjie	WELL
U+3624.0	fFourCorner	60120
U+3624.0	fCantonese	waan1
U+3624.0	fMandarin	wan1
U+3624.0	fEnglish	circle of the moving water
U+3625.0	fUTF8	㘥
U+3625.0	fCNS11643	4-6332
U+3625.0	fGB	3-375E
U+3625.0	fR/S	 31.17
U+3625.0	fTotalStrokes	 20
U+3625.0	fCangjie	WBUF
U+3625.0	fFourCorner	60790
U+3625.0	fCantonese	jau4; ngo4 (also); zaau4
U+3625.0	fMandarin	you2
U+3625.0	fEnglish	same as &#x56EE;
U+3626.0	fUTF8	㘦
U+3626.0	fCNS11643	3-2233
U+3626.0	fR/S	 32.2
U+3626.0	fTotalStrokes	 5
U+3626.0	fCangjie	GKS
U+3626.0	fFourCorner	44120
U+3626.0	fCantonese	kan4
U+3626.0	fMandarin	qin2
U+3626.0	fEnglish	[1] wall made of soil [2] used in the name of places [3] inveigle
U+3627.0	fUTF8	㘧
U+3627.0	fCNS11643	3-252E
U+3627.0	fGB	5-3565
U+3627.0	fR/S	 32.4
U+3627.0	fTotalStrokes	 7
U+3627.0	fCangjie	GNIN
U+3627.0	fFourCorner	47120
U+3627.0	fHakka	  si5
U+3627.0	fCantonese	zeoi6
U+3627.0	fMandarin	xu4
U+3627.0	fEnglish	[1] the east and west walls of an enclosure [2] same as &#x5E8F;
U+3628.0	fUTF8	㘨
U+3628.0	fCNS11643	4-235F
U+3628.0	fGB	3-3368
U+3628.0	fR/S	 32.4
U+3628.0	fTotalStrokes	 7
U+3628.0	fCangjie	GOB
U+3628.0	fFourCorner	44120
U+3628.0	fCantonese	nip6; lip6 (also)
U+3628.0	fMandarin	kuai4; nei4
U+3628.0	fEnglish	[1] deep; profound [2] empty; hollow [3] quite; peaceful
U+3629.0	fUTF8	㘩
U+3629.0	fCNS11643	3-2539
U+3629.0	fGB	5-355D
U+3629.0	fR/S	 32.4
U+3629.0	fTotalStrokes	 7
U+3629.0	fCangjie	GPP
U+3629.0	fFourCorner	41110
U+3629.0	fHakka	 pi5
U+3629.0	fCantonese	bei6
U+3629.0	fMandarin	bi4
U+3629.0	fEnglish	 mate; pair
U+362A.0	fUTF8	㘪
U+362A.0	fCNS11643	3-253A
U+362A.0	fGB	5-355E
U+362A.0	fR/S	 32.4
U+362A.0	fTotalStrokes	 7
U+362A.0	fCangjie	GHQU
U+362A.0	fFourCorner	42110
U+362A.0	fHakka	 hau1; hau5
U+362A.0	fCantonese	hou3; kaau3
U+362A.0	fMandarin	hao4; mao4; mao2
U+362A.0	fEnglish	same as &#x8017;
U+362B.0	fUTF8	㘫
U+362B.0	fCNS11643	4-235E
U+362B.0	fGB	5-355B
U+362B.0	fR/S	 32.4
U+362B.0	fTotalStrokes	 7
U+362B.0	fCangjie	GTT
U+362B.0	fFourCorner	45100
U+362B.0	fCantonese	ging2; zing6
U+362B.0	fMandarin	jing4; jing3
U+362B.0	fEnglish	variant of &#x9631;
U+362C.0	fUTF8	㘬
U+362C.0	fCNS11643	3-253D
U+362C.0	fR/S	 32.4
U+362C.0	fTotalStrokes	 7
U+362C.0	fCangjie	GPI
U+362C.0	fFourCorner	47120
U+362C.0	fHakka	 fut8
U+362C.0	fCantonese	aau3; ngaau3 (also); fat1; au1; fat6
U+362C.0	fMandarin	fo2; ao4; fu2; bi4; bo2
U+362C.0	fEnglish	same as &#x4F5B;
U+362D.0	fUTF8	㘭
U+362D.0	fCNS11643	3-253F
U+362D.0	fR/S	 32.4
U+362D.0	fTotalStrokes	 7
U+362D.0	fCangjie	GVIS
U+362D.0	fFourCorner	47120
U+362D.0	fHakka	 au1
U+362D.0	fCantonese	aau3; ngaau3 (also); aau1 (also); ngaau1 (also); au1; au3
U+362D.0	fMandarin	ao1; ao4; you3
U+362D.0	fEnglish	same as &#x5773;
U+362E.0	fUTF8	㘮
U+362E.0	fGB	7-232E
U+362E.0	fR/S	 32.4
U+362E.0	fTotalStrokes	 7
U+362E.0	fCangjie	GSO
U+362E.0	fCantonese	jik6
U+362F.0	fUTF8	㘯
U+362F.0	fCNS11643	3-253B
U+362F.0	fR/S	 32.4
U+362F.0	fTotalStrokes	 7
U+362F.0	fCangjie	GOKS
U+362F.0	fFourCorner	48120
U+362F.0	fCantonese	coeng4
U+362F.0	fMandarin	chang2
U+3630.0	fUTF8	㘰
U+3630.0	fCNS11643	3-2534
U+3630.0	fR/S	 32.4
U+3630.0	fTotalStrokes	 7
U+3630.0	fCangjie	GYJ
U+3630.0	fFourCorner	44100
U+3630.0	fCantonese	zam1
U+3630.0	fMandarin	zhen1
U+3630.0	fEnglish	interchangeable with &#x659F;
U+3631.0	fUTF8	㘱
U+3631.0	fCNS11643	3-2827
U+3631.0	fR/S	 32.5
U+3631.0	fTotalStrokes	 8
U+3631.0	fCangjie	GGB
U+3631.0	fFourCorner	47150
U+3631.0	fHakka	 tam1; tan1
U+3631.0	fCantonese	taan1
U+3631.0	fMandarin	tan1
U+3631.0	fEnglish	ancient form of &#x574D;
U+3632.0	fUTF8	㘲
U+3632.0	fCNS11643	4-256D
U+3632.0	fGB	3-3379
U+3632.0	fR/S	 32.5
U+3632.0	fTotalStrokes	 8
U+3632.0	fCangjie	GSKN
U+3632.0	fFourCorner	47110
U+3632.0	fCantonese	guk1
U+3632.0	fMandarin	ju2
U+3632.0	fEnglish	[1] cove; bay [2] a bend or nook in the hills [3] the outside part of a curved bank; the curve of a bow
U+3633.0	fUTF8	㘳
U+3633.0	fCNS11643	3-282B
U+3633.0	fR/S	 32.5
U+3633.0	fTotalStrokes	 8
U+3633.0	fCangjie	LPG
U+3633.0	fFourCorner	11100
U+3633.0	fHakka	 hiu1; kiu1
U+3633.0	fCantonese	jau1; zaau1
U+3633.0	fMandarin	qiu1
U+3633.0	fEnglish	same as &#x4E18;
U+3634.0	fUTF8	㘴
U+3634.0	fCNS11643	3-282C
U+3634.0	fR/S	 32.5
U+3634.0	fTotalStrokes	 8
U+3634.0	fCangjie	ROG
U+3634.0	fFourCorner	68100
U+3634.0	fHakka	 co1; co5; co3
U+3634.0	fCantonese	zo6; co5
U+3634.0	fMandarin	zuo4
U+3634.0	fEnglish	same as &#x5750;
U+3635.0	fUTF8	㘵
U+3635.0	fGB	3-3370
U+3635.0	fR/S	 32.5
U+3635.0	fTotalStrokes	 8
U+3635.0	fCangjie	GKLB
U+3635.0	fCantonese	bou3
U+3635.0	fMandarin	bu4
U+3635.0	fEnglish	name of a place
U+3636.0	fUTF8	㘶
U+3636.0	fCNS11643	3-2B76
U+3636.0	fR/S	 32.6
U+3636.0	fTotalStrokes	 9
U+3636.0	fCangjie	NQG
U+3636.0	fFourCorner	25100
U+3636.0	fHakka	 ket8
U+3636.0	fCantonese	git6; miu5
U+3636.0	fMandarin	jie2
U+3636.0	fEnglish	same as &#x6840;
U+3637.0	fUTF8	㘷
U+3637.0	fCNS11643	6-2F65
U+3637.0	fGB	S-236C
U+3637.0	fR/S	 32.6
U+3637.0	fTotalStrokes	 9
U+3637.0	fCangjie	GTK
U+3637.0	fFourCorner	44140
U+3637.0	fHakka	 ngioi1; ngiai5; gne5; ngai5; nge5; ngie5
U+3637.0	fCantonese	ngaai6; aai6 (also); ngai6
U+3637.0	fMandarin	ai4; yi4
U+3637.0	fEnglish	same as &#x827E;
U+3638.0	fUTF8	㘸
U+3638.0	fCNS11643	3-2B74
U+3638.0	fR/S	 32.6
U+3638.0	fTotalStrokes	 9
U+3638.0	fCangjie	MNPG
U+3638.0	fFourCorner	10100
U+3638.0	fHakka	 co1; co5; co3
U+3638.0	fCantonese	zong3; co5; zo6
U+3638.0	fMandarin	zuo4; zang4
U+3638.0	fEnglish	ancient form of &#x5750;
U+3639.0	fUTF8	㘹
U+3639.0	fCNS11643	3-2B75
U+3639.0	fR/S	 32.6
U+3639.0	fTotalStrokes	 9
U+3639.0	fCangjie	YPG
U+3639.0	fFourCorner	21100
U+3639.0	fCantonese	ci4
U+3639.0	fMandarin	ci2
U+3639.0	fEnglish	same as &#x5790;
U+363A.0	fUTF8	㘺
U+363A.0	fCNS11643	3-2B73
U+363A.0	fR/S	 32.6
U+363A.0	fTotalStrokes	 9
U+363A.0	fCangjie	GOI
U+363A.0	fFourCorner	43150
U+363A.0	fHakka	 fat8
U+363A.0	fCantonese	fat6
U+363A.0	fMandarin	fa2
U+363A.0	fEnglish	same as &#x57A1;
U+363B.0	fUTF8	㘻
U+363B.0	fCNS11643	3-2B6F
U+363B.0	fR/S	 32.6
U+363B.0	fTotalStrokes	 9
U+363B.0	fCangjie	GWK
U+363B.0	fFourCorner	46100
U+363B.0	fCantonese	jan1; zaan1
U+363B.0	fMandarin	yin1
U+363B.0	fEnglish	[1] stop up; close against; restrain [2] dam a stream and change its direction [3] conceal a mound
U+363C.0	fUTF8	㘼
U+363C.0	fGB	S-2369
U+363C.0	fR/S	 32.6
U+363C.0	fTotalStrokes	 9
U+363C.0	fCangjie	GJKS
U+363C.0	fCantonese	haau2
U+363D.0	fUTF8	㘽
U+363D.0	fR/S	 32.6
U+363D.0	fTotalStrokes	 9
U+363D.0	fCangjie	JIG
U+363D.0	fCantonese	zoi1
U+363E.0	fUTF8	㘾
U+363E.0	fR/S	 32.6
U+363E.0	fTotalStrokes	 9
U+363E.0	fCangjie	GJMD
U+363E.0	fFourCorner	4314
U+363E.0	fCantonese	jyu5
U+363F.0	fUTF8	㘿
U+363F.0	fCNS11643	3-3029
U+363F.0	fGB	5-362C
U+363F.0	fR/S	 32.7
U+363F.0	fTotalStrokes	 10
U+363F.0	fCangjie	GAG
U+363F.0	fFourCorner	46114
U+363F.0	fHakka	 ngiap7
U+363F.0	fCantonese	nit6; lit6 (also); nip6
U+363F.0	fMandarin	nie4
U+363F.0	fEnglish	same as &#x6D85;
U+3640.0	fUTF8	㙀
U+3640.0	fCNS11643	4-2C2F
U+3640.0	fGB	3-3433
U+3640.0	fR/S	 32.7
U+3640.0	fTotalStrokes	 10
U+3640.0	fCangjie	GJCN
U+3640.0	fFourCorner	43110
U+3640.0	fCantonese	lau6
U+3640.0	fMandarin	liu4
U+3640.0	fEnglish	[1] vegetable-plot [2] plough; till lands, among dikes and fields
U+3641.0	fUTF8	㙁
U+3641.0	fCNS11643	4-2C31
U+3641.0	fGB	5-362E
U+3641.0	fR/S	 32.7
U+3641.0	fTotalStrokes	 10
U+3641.0	fCangjie	GOWY
U+3641.0	fFourCorner	48150
U+3641.0	fCantonese	mui4; muk6
U+3641.0	fMandarin	mei2; mang3; mei4; mu4; na4
U+3641.0	fEnglish	[1] dust; dirt; smear [2] same as &#x7267;
U+3642.0	fUTF8	㙂
U+3642.0	fCNS11643	3-302D
U+3642.0	fGB	5-3631
U+3642.0	fR/S	 32.7
U+3642.0	fTotalStrokes	 10
U+3642.0	fCangjie	GCRU
U+3642.0	fFourCorner	48110
U+3642.0	fCantonese	deoi6; deoi3
U+3642.0	fMandarin	dui4
U+3642.0	fEnglish	[1] wall; fence [2] interchangeable with &#x52DF;
U+3643.0	fUTF8	㙃
U+3643.0	fCNS11643	3-302C
U+3643.0	fR/S	 32.7
U+3643.0	fTotalStrokes	 10
U+3643.0	fCangjie	GOMC
U+3643.0	fFourCorner	42180
U+3643.0	fCantonese	bang1; bong1
U+3643.0	fMandarin	bang1
U+3643.0	fEnglish	1] cavity; hole of a mound or an altar of earth
U+3644.0	fUTF8	㙄
U+3644.0	fCNS11643	3-302E
U+3644.0	fR/S	 32.7
U+3644.0	fTotalStrokes	 10
U+3644.0	fCangjie	GPPG
U+3644.0	fFourCorner	41110
U+3644.0	fHakka	 pi3
U+3644.0	fCantonese	bai6
U+3644.0	fMandarin	bi4
U+3644.0	fEnglish	same as &#x965B;
U+3645.0	fUTF8	㙅
U+3645.0	fCNS11643	3-3031
U+3645.0	fR/S	 32.7
U+3645.0	fTotalStrokes	 10
U+3645.0	fCangjie	GRD
U+3645.0	fFourCorner	46190
U+3645.0	fHakka	 bau3
U+3645.0	fCantonese	bou2
U+3645.0	fMandarin	bao3
U+3645.0	fEnglish	same as &#x4FDD;
U+3646.0	fUTF8	㙆
U+3646.0	fR/S	 32.7
U+3646.0	fTotalStrokes	 10
U+3646.0	fCangjie	GJKI
U+3646.0	fCantonese	wang4
U+3647.0	fUTF8	㙇
U+3647.0	fCNS11643	3-346C
U+3647.0	fR/S	 32.8
U+3647.0	fTotalStrokes	 11
U+3647.0	fCangjie	GMSO
U+3647.0	fFourCorner	41130
U+3647.0	fCantonese	cuk1
U+3647.0	fMandarin	chuo1; chu4
U+3647.0	fEnglish	the cattle and horses trampled on the field
U+3648.0	fUTF8	㙈
U+3648.0	fCNS11643	3-346D
U+3648.0	fGB	S-236A
U+3648.0	fR/S	 32.8
U+3648.0	fTotalStrokes	 11
U+3648.0	fCangjie	GYPU
U+3648.0	fFourCorner	41110
U+3648.0	fCantonese	haa3; laa4
U+3648.0	fMandarin	xia4; han2
U+3648.0	fEnglish	the name of a place in Shanxi province
U+3649.0	fUTF8	㙉
U+3649.0	fCNS11643	3-3473
U+3649.0	fR/S	 32.8
U+3649.0	fTotalStrokes	 11
U+3649.0	fCangjie	GTBC
U+3649.0	fFourCorner	45180
U+3649.0	fCantonese	tin2; din6
U+3649.0	fMandarin	tian3
U+3649.0	fEnglish	same as &#x8146;
U+364A.0	fUTF8	㙊
U+364A.0	fCNS11643	3-346A
U+364A.0	fR/S	 32.8
U+364A.0	fTotalStrokes	 11
U+364A.0	fCangjie	GSMV
U+364A.0	fFourCorner	41130
U+364A.0	fHakka	 zhong2
U+364A.0	fCantonese	coeng4
U+364A.0	fMandarin	chang2; zhang4
U+364A.0	fEnglish	same as &#x5834;
U+364B.0	fUTF8	㙋
U+364B.0	fCNS11643	F-3357
U+364B.0	fGB	7-232F
U+364B.0	fR/S	 32.8
U+364B.0	fTotalStrokes	 11
U+364B.0	fCangjie	GTMC
U+364B.0	fFourCorner	4418
U+364B.0	fCantonese	gei1
U+364C.0	fUTF8	㙌
U+364C.0	fGB	7-2331
U+364C.0	fR/S	 32.8
U+364C.0	fTotalStrokes	 11
U+364C.0	fCangjie	GLX
U+364C.0	fCantonese	suk1
U+364D.0	fUTF8	㙍
U+364D.0	fCNS11643	4-3121
U+364D.0	fGB	3-3447
U+364D.0	fR/S	 32.8
U+364D.0	fTotalStrokes	 11
U+364D.0	fCangjie	GEEE
U+364D.0	fFourCorner	47140
U+364D.0	fCantonese	zeoi3; do1
U+364D.0	fEnglish	the name of a place in today's Guangdong province
U+364E.0	fUTF8	㙎
U+364E.0	fCNS11643	4-366B
U+364E.0	fR/S	 32.9
U+364E.0	fTotalStrokes	 12
U+364E.0	fCangjie	GIHV
U+364E.0	fFourCorner	43150
U+364E.0	fCantonese	wai1
U+364E.0	fMandarin	wei1
U+364E.0	fEnglish	[1] breach of a tank; pond [2] cover up; conceal; hide; bury
U+364F.0	fUTF8	㙏
U+364F.0	fCNS11643	3-3A46
U+364F.0	fGB	5-364A
U+364F.0	fR/S	 32.9
U+364F.0	fTotalStrokes	 12
U+364F.0	fCangjie	GOAE
U+364F.0	fFourCorner	48140
U+364F.0	fHakka	  fuk7
U+364F.0	fCantonese	fuk1
U+364F.0	fMandarin	fu4
U+364F.0	fEnglish	 make a cave to live in
U+3650.0	fUTF8	㙐
U+3650.0	fCNS11643	4-366E
U+3650.0	fGB	3-344E
U+3650.0	fR/S	 32.9
U+3650.0	fTotalStrokes	 12
U+3650.0	fCangjie	GUMB
U+3650.0	fFourCorner	42120
U+3650.0	fCantonese	do2; deoi2
U+3650.0	fMandarin	duo3
U+3650.0	fEnglish	[1] move [2] hang down; let fall [3] same as &#x579B;
U+3651.0	fUTF8	㙑
U+3651.0	fCNS11643	3-3A51
U+3651.0	fR/S	 32.9
U+3651.0	fTotalStrokes	 12
U+3651.0	fCangjie	HBG
U+3651.0	fFourCorner	20100
U+3651.0	fCantonese	jyu5; zyu5
U+3651.0	fMandarin	yu3
U+3651.0	fEnglish	same as &#x5B87;
U+3652.0	fUTF8	㙒
U+3652.0	fCNS11643	6-474E
U+3652.0	fR/S	 32.9
U+3652.0	fTotalStrokes	 12
U+3652.0	fCangjie	WNG
U+3652.0	fFourCorner	67100
U+3652.0	fHakka	 ja3; ja1
U+3652.0	fCantonese	je5; ze5; zi2
U+3652.0	fMandarin	ye3; shu4; yu3
U+3652.0	fEnglish	same as &#x91CE;
U+3653.0	fUTF8	㙓
U+3653.0	fCNS11643	3-3A48
U+3653.0	fR/S	 32.9
U+3653.0	fTotalStrokes	 12
U+3653.0	fCangjie	GGGG
U+3653.0	fFourCorner	40100
U+3653.0	fCantonese	kwai4
U+3653.0	fMandarin	kui2
U+3653.0	fEnglish	[1] earth; land; soil; ground; territory [2] opium
U+3654.0	fUTF8	㙔
U+3654.0	fGB	3-3364
U+3654.0	fR/S	 32.9
U+3654.0	fTotalStrokes	 12
U+3654.0	fCangjie	GDMQ
U+3654.0	fFourCorner	4415
U+3654.0	fCantonese	wai5; hon4
U+3654.0	fMandarin	han2
U+3654.0	fEnglish	non-classical form of &#x97D3;
U+3655.0	fUTF8	㙕
U+3655.0	fCNS11643	3-3A4D
U+3655.0	fR/S	 32.9
U+3655.0	fTotalStrokes	 12
U+3655.0	fCangjie	GWB
U+3655.0	fFourCorner	46120
U+3655.0	fHakka	 kwai5
U+3655.0	fCantonese	faai3
U+3655.0	fMandarin	kuai4
U+3655.0	fEnglish	same as &#x584A;
U+3656.0	fUTF8	㙖
U+3656.0	fR/S	 32.9
U+3656.0	fTotalStrokes	 12
U+3656.0	fCangjie	GHLB
U+3656.0	fCantonese	jyu5
U+3657.0	fUTF8	㙗
U+3657.0	fR/S	 32.9
U+3657.0	fTotalStrokes	 12
U+3657.0	fCangjie	GWMV
U+3657.0	fCantonese	wui1; fai3
U+3657.0	fMandarin	kuai4
U+3657.0	fEnglish	same as &#x584A;
U+3658.0	fUTF8	㙘
U+3658.0	fGB	3-344C
U+3658.0	fR/S	 32.9
U+3658.0	fTotalStrokes	 12
U+3658.0	fCangjie	GMWV
U+3658.0	fCantonese	jiu1; ziu1
U+3658.0	fEnglish	used in naming a place
U+3659.0	fUTF8	㙙
U+3659.0	fCNS11643	4-3C59
U+3659.0	fGB	3-3463
U+3659.0	fR/S	 32.10
U+3659.0	fTotalStrokes	 13
U+3659.0	fCangjie	EHG
U+3659.0	fFourCorner	33100
U+3659.0	fCantonese	lung5
U+3659.0	fMandarin	mang2; long3
U+3659.0	fEnglish	[1] smear; daub [2] erase; blot out [3] mud, mire [4] plaster a wall with mud
U+365A.0	fUTF8	㙚
U+365A.0	fCNS11643	4-3C5B
U+365A.0	fGB	3-3462
U+365A.0	fR/S	 32.10
U+365A.0	fTotalStrokes	 13
U+365A.0	fCangjie	GTGQ
U+365A.0	fFourCorner	48150
U+365A.0	fCantonese	sing1
U+365A.0	fMandarin	xing1
U+365A.0	fEnglish	red colored hard and solid mud (soil; clay; earth)
U+365B.0	fUTF8	㙛
U+365B.0	fCNS11643	3-4053
U+365B.0	fR/S	 32.10
U+365B.0	fTotalStrokes	 13
U+365B.0	fCangjie	GIBI
U+365B.0	fFourCorner	43140
U+365B.0	fCantonese	bou2
U+365B.0	fMandarin	bu3
U+365B.0	fEnglish	a person's name in old times
U+365C.0	fUTF8	㙜
U+365C.0	fCNS11643	3-4055
U+365C.0	fGB	5-7964
U+365C.0	fR/S	 32.10
U+365C.0	fTotalStrokes	 13
U+365C.0	fCangjie	YBG
U+365C.0	fFourCorner	00100
U+365C.0	fCantonese	kok3; ci4; toi4
U+365C.0	fMandarin	chi2; tai2
U+365C.0	fEnglish	[1] ancient form of &#x81FA; [2] corrupt form of &#x81FA;
U+365D.0	fUTF8	㙝
U+365D.0	fCNS11643	4-3C56
U+365D.0	fGB	5-365D
U+365D.0	fR/S	 32.10
U+365D.0	fTotalStrokes	 13
U+365D.0	fCangjie	GKSB
U+365D.0	fFourCorner	44120
U+365D.0	fCantonese	hip3
U+365D.0	fMandarin	xie2
U+365D.0	fEnglish	 dike; barrier; embankment
U+365E.0	fUTF8	㙞
U+365E.0	fCNS11643	4-3C58
U+365E.0	fR/S	 32.10
U+365E.0	fTotalStrokes	 13
U+365E.0	fCangjie	GHUD
U+365E.0	fFourCorner	46190
U+365E.0	fCantonese	ngat6; at6 (also); nip6; saan1; zit6
U+365E.0	fMandarin	nie4
U+365E.0	fEnglish	 small mountain; hill
U+365F.0	fUTF8	㙟
U+365F.0	fGB	3-3465
U+365F.0	fR/S	 32.10
U+365F.0	fTotalStrokes	 13
U+365F.0	fCangjie	GIIB
U+365F.0	fCantonese	long5
U+365F.0	fMandarin	lang3
U+365F.0	fEnglish	[1] low-lying ground on the side of a river or lake [2] name of a place in Kwangtung
U+3660.0	fUTF8	㙠
U+3660.0	fCNS11643	4-425E
U+3660.0	fGB	5-3662
U+3660.0	fR/S	 32.11
U+3660.0	fTotalStrokes	 14
U+3660.0	fCangjie	SEG
U+3660.0	fFourCorner	77100
U+3660.0	fHakka	 ji1; ri1; i1
U+3660.0	fCantonese	ai1; ngai1 (also); aai3
U+3660.0	fMandarin	yi4
U+3660.0	fEnglish	dirt; dust
U+3661.0	fUTF8	㙡
U+3661.0	fCNS11643	4-425D
U+3661.0	fGB	3-336A
U+3661.0	fR/S	 32.11
U+3661.0	fTotalStrokes	 14
U+3661.0	fCangjie	GHOO
U+3661.0	fFourCorner	48180
U+3661.0	fHakka	 zung1
U+3661.0	fCantonese	zung1
U+3661.0	fMandarin	zong1
U+3661.0	fEnglish	[1] sheaf of grain [2] skein [3] collect [4] a kind of eatable mushroom
U+3662.0	fUTF8	㙢
U+3662.0	fCNS11643	4-425B
U+3662.0	fGB	5-3653
U+3662.0	fR/S	 32.11
U+3662.0	fTotalStrokes	 14
U+3662.0	fCangjie	GTLB
U+3662.0	fFourCorner	44120
U+3662.0	fHakka	 man2; man1; man5
U+3662.0	fCantonese	mun4; man6; mang6
U+3662.0	fMandarin	man2; men2
U+3662.0	fEnglish	[1] cover over [2] lay down as tiles
U+3663.0	fUTF8	㙣
U+3663.0	fCNS11643	3-463B
U+3663.0	fR/S	 32.11
U+3663.0	fTotalStrokes	 14
U+3663.0	fCangjie	GNSV
U+3663.0	fFourCorner	41130
U+3663.0	fCantonese	zoeng3
U+3663.0	fMandarin	zhang4
U+3663.0	fEnglish	 pile a sand-hills
U+3664.0	fUTF8	㙤
U+3664.0	fCNS11643	4-425F
U+3664.0	fGB	5-3666
U+3664.0	fR/S	 32.11
U+3664.0	fTotalStrokes	 14
U+3664.0	fCangjie	GYPD
U+3664.0	fFourCorner	41140
U+3664.0	fCantonese	laa3
U+3664.0	fMandarin	xia4
U+3664.0	fEnglish	same as &#x7F45;
U+3665.0	fUTF8	㙥
U+3665.0	fCNS11643	3-4639
U+3665.0	fR/S	 32.11
U+3665.0	fTotalStrokes	 14
U+3665.0	fCangjie	GYCV
U+3665.0	fFourCorner	40130
U+3665.0	fHakka	 gun3; gwun3
U+3665.0	fCantonese	gwan3; gwaan2; waan6; wat6
U+3665.0	fMandarin	gun4; gun3
U+3665.0	fEnglish	 splash; make muddy
U+3666.0	fUTF8	㙦
U+3666.0	fGB	3-346C
U+3666.0	fR/S	 32.11
U+3666.0	fTotalStrokes	 14
U+3666.0	fCangjie	OJG
U+3666.0	fCantonese	ce4
U+3667.0	fUTF8	㙧
U+3667.0	fCNS11643	7-216A
U+3667.0	fR/S	 32.12
U+3667.0	fTotalStrokes	 15
U+3667.0	fCangjie	GMLW
U+3667.0	fFourCorner	41160
U+3667.0	fCantonese	lau6
U+3667.0	fEnglish	same as &#x586F;
U+3668.0	fUTF8	㙨
U+3668.0	fCNS11643	3-4B52
U+3668.0	fR/S	 32.12
U+3668.0	fTotalStrokes	 15
U+3668.0	fCangjie	GVII
U+3668.0	fFourCorner	42150
U+3668.0	fCantonese	gei1
U+3668.0	fMandarin	qi2; ji4
U+3668.0	fEnglish	same as &#x757F;
U+3669.0	fUTF8	㙩
U+3669.0	fCNS11643	3-4B48
U+3669.0	fGB	5-366F
U+3669.0	fR/S	 32.12
U+3669.0	fTotalStrokes	 15
U+3669.0	fCangjie	GKCF
U+3669.0	fFourCorner	44190
U+3669.0	fHakka	 liau5; liau2
U+3669.0	fCantonese	liu4; laau6
U+3669.0	fMandarin	liao2
U+3669.0	fEnglish	a surrounding well
U+366A.0	fUTF8	㙪
U+366A.0	fCNS11643	4-4933
U+366A.0	fGB	3-3470
U+366A.0	fR/S	 32.12
U+366A.0	fTotalStrokes	 15
U+366A.0	fCangjie	GGBT
U+366A.0	fFourCorner	44110
U+366A.0	fHakka	 ji5; ri5; rit7; jit7; i5
U+366A.0	fCantonese	ai3; ngai3 (also); zi3
U+366A.0	fMandarin	yi4; ye4
U+366A.0	fEnglish	[1] dust in the air [2] dark; murky
U+366B.0	fUTF8	㙫
U+366B.0	fCNS11643	3-4B4A
U+366B.0	fGB	5-3670
U+366B.0	fR/S	 32.12
U+366B.0	fTotalStrokes	 15
U+366B.0	fCangjie	GOGD
U+366B.0	fFourCorner	40190
U+366B.0	fCantonese	cap1; zap6
U+366B.0	fMandarin	ji2
U+366B.0	fEnglish	a fountain or spring out from the underground
U+366C.0	fUTF8	㙬
U+366C.0	fCNS11643	3-4B4E
U+366C.0	fGB	5-3665
U+366C.0	fR/S	 32.12
U+366C.0	fTotalStrokes	 15
U+366C.0	fCangjie	DKG
U+366C.0	fFourCorner	43100
U+366C.0	fCantonese	ngan4; an4 (also); ngaan4
U+366C.0	fMandarin	yin2
U+366C.0	fEnglish	ancient form of &#x57A0;
U+366D.0	fUTF8	㙭
U+366D.0	fCNS11643	F-4E25
U+366D.0	fR/S	 32.12
U+366D.0	fTotalStrokes	 15
U+366D.0	fCangjie	GMTO
U+366D.0	fCantonese	kyut3
U+366E.0	fUTF8	㙮
U+366E.0	fCNS11643	F-4E2E
U+366E.0	fR/S	 32.12
U+366E.0	fTotalStrokes	 15
U+366E.0	fCangjie	GHOR
U+366E.0	fFourCorner	4816
U+366E.0	fCantonese	daap3; taap3
U+366E.0	fMandarin	da1; da5
U+366E.0	fEnglish	tower; pagoda; spire
U+366F.0	fUTF8	㙯
U+366F.0	fCNS11643	4-492A
U+366F.0	fR/S	 32.12
U+366F.0	fTotalStrokes	 15
U+366F.0	fCangjie	GIMMI
U+366F.0	fFourCorner	45730
U+366F.0	fCantonese	ngai6; ai6 (also); ngaai6
U+366F.0	fMandarin	yi4
U+366F.0	fEnglish	same as &#x85DD;
U+3670.0	fUTF8	㙰
U+3670.0	fCNS11643	3-5044
U+3670.0	fR/S	 32.13
U+3670.0	fTotalStrokes	 16
U+3670.0	fCangjie	NQG
U+3670.0	fFourCorner	27100
U+3670.0	fCantonese	haai5; hai6
U+3670.0	fMandarin	xie4
U+3670.0	fEnglish	the name of a place in ancient times
U+3671.0	fUTF8	㙱
U+3671.0	fCNS11643	4-4F78
U+3671.0	fGB	3-3478
U+3671.0	fR/S	 32.13
U+3671.0	fTotalStrokes	 16
U+3671.0	fCangjie	RUG
U+3671.0	fFourCorner	61100
U+3671.0	fCantonese	gou2; hou6
U+3671.0	fMandarin	hao4
U+3671.0	fEnglish	earthenware used to bake cakes in Northern China
U+3672.0	fUTF8	㙲
U+3672.0	fCNS11643	3-5047
U+3672.0	fGB	5-3678
U+3672.0	fR/S	 32.13
U+3672.0	fTotalStrokes	 16
U+3672.0	fCangjie	GYVG
U+3672.0	fFourCorner	40110
U+3672.0	fHakka	 jung3
U+3672.0	fCantonese	jung1; zung1
U+3672.0	fMandarin	yong1; yong3
U+3672.0	fEnglish	same as &#x58C5;
U+3673.0	fUTF8	㙳
U+3673.0	fCNS11643	3-5042
U+3673.0	fGB	5-3675
U+3673.0	fR/S	 32.13
U+3673.0	fTotalStrokes	 16
U+3673.0	fCangjie	GIHP
U+3673.0	fFourCorner	43135
U+3673.0	fHakka	 kam3
U+3673.0	fCantonese	ham2
U+3673.0	fMandarin	han3; he2; kan3
U+3673.0	fEnglish	hindered; unlucky; always losing
U+3674.0	fUTF8	㙴
U+3674.0	fCNS11643	3-5043
U+3674.0	fR/S	 32.13
U+3674.0	fTotalStrokes	 16
U+3674.0	fCangjie	GNCR
U+3674.0	fFourCorner	47160
U+3674.0	fCantonese	cim3; cim1
U+3674.0	fMandarin	chan4; zhan4
U+3674.0	fEnglish	 conceal; hide, shade; darken
U+3675.0	fUTF8	㙵
U+3675.0	fCNS11643	3-5046
U+3675.0	fR/S	 32.13
U+3675.0	fTotalStrokes	 16
U+3675.0	fCangjie	GYRG
U+3675.0	fFourCorner	40110
U+3675.0	fHakka	 toi2
U+3675.0	fCantonese	toi4
U+3675.0	fMandarin	tai2; hu2
U+3675.0	fEnglish	ancient form of &#x81FA;
U+3676.0	fUTF8	㙶
U+3676.0	fCNS11643	3-5045
U+3676.0	fGB	5-335C
U+3676.0	fR/S	 32.13
U+3676.0	fTotalStrokes	 16
U+3676.0	fCangjie	YRFBG
U+3676.0	fFourCorner	00100
U+3676.0	fHakka	 tong2
U+3676.0	fCantonese	tong4
U+3676.0	fMandarin	tang2; tang3
U+3676.0	fEnglish	variant of &#x5802;
U+3677.0	fUTF8	㙷
U+3677.0	fCNS11643	4-557C
U+3677.0	fGB	3-347C
U+3677.0	fR/S	 32.14
U+3677.0	fTotalStrokes	 17; 16
U+3677.0	fCangjie	GAVF
U+3677.0	fFourCorner	46130
U+3677.0	fHakka	 ship7; siip7; zhih5; zih5; ch'ip7; ts'iip7; chi1; zhit7
U+3677.0	fCantonese	zaap6; zik6
U+3677.0	fMandarin	li4; zhi2
U+3677.0	fEnglish	ground hard and cracked by heat
U+3678.0	fUTF8	㙸
U+3678.0	fCNS11643	3-544D
U+3678.0	fR/S	 32.14
U+3678.0	fTotalStrokes	 17
U+3678.0	fCangjie	GOTO
U+3678.0	fFourCorner	42130
U+3678.0	fHakka	 pok7
U+3678.0	fCantonese	pok3; buk6
U+3678.0	fMandarin	pu1; pu2; bao4; bo2
U+3678.0	fEnglish	same as &#x58A3;
U+3679.0	fUTF8	㙹
U+3679.0	fCNS11643	3-544A
U+3679.0	fR/S	 32.14
U+3679.0	fTotalStrokes	 17
U+3679.0	fCangjie	GTWN
U+3679.0	fFourCorner	44120
U+3679.0	fCantonese	mung6; mung4
U+3679.0	fMandarin	meng2
U+3679.0	fEnglish	[1] damp; marsh [2] interchangeable &#x5922;
U+367A.0	fUTF8	㙺
U+367A.0	fCNS11643	4-557B
U+367A.0	fGB	5-3660
U+367A.0	fR/S	 32.14
U+367A.0	fTotalStrokes	 17; 16
U+367A.0	fCangjie	GSLC
U+367A.0	fFourCorner	41110
U+367A.0	fCantonese	kwai4; gwaai6
U+367A.0	fMandarin	gui4; kui2
U+367A.0	fEnglish	same as &#x9035;, &#x9997;
U+367B.0	fUTF8	㙻
U+367B.0	fCNS11643	7-3F30
U+367B.0	fR/S	 32.15
U+367B.0	fTotalStrokes	 18
U+367B.0	fCangjie	GIWG
U+367B.0	fFourCorner	40110
U+367B.0	fHakka	 chan2; chen2; can2; cen2
U+367B.0	fCantonese	cin4
U+367B.0	fMandarin	chan2; qie4; zan4
U+367B.0	fEnglish	[1] market-place, shops [2] non-classical form of &#x5EDB;
U+367C.0	fUTF8	㙼
U+367C.0	fCNS11643	3-575D
U+367C.0	fR/S	 32.15
U+367C.0	fTotalStrokes	 18
U+367C.0	fCangjie	GWWW
U+367C.0	fFourCorner	46160
U+367C.0	fCantonese	leoi5
U+367C.0	fMandarin	lei3
U+367C.0	fEnglish	same as &#x58D8;
U+367D.0	fUTF8	㙽
U+367D.0	fR/S	 32.15
U+367D.0	fTotalStrokes	 18
U+367D.0	fCangjie	GKMA
U+367D.0	fCantonese	sik1
U+367E.0	fUTF8	㙾
U+367E.0	fCNS11643	7-4644
U+367E.0	fR/S	 32.16
U+367E.0	fTotalStrokes	 19
U+367E.0	fCangjie	GHBD
U+367E.0	fFourCorner	47140
U+367E.0	fCantonese	hok6
U+367E.0	fMandarin	que4; xi4; xue2
U+367E.0	fEnglish	same as &#x5DA8;
U+367F.0	fUTF8	㙿
U+367F.0	fR/S	 32.16
U+367F.0	fTotalStrokes	 19
U+367F.0	fCangjie	GTGS
U+367F.0	fCantonese	hei1
U+3680.0	fUTF8	㚀
U+3680.0	fCNS11643	5-6B76
U+3680.0	fR/S	 32.17
U+3680.0	fTotalStrokes	 20
U+3680.0	fCangjie	GYTI
U+3680.0	fFourCorner	43150
U+3680.0	fCantonese	hei1
U+3680.0	fMandarin	xi1
U+3680.0	fEnglish	[1] lofty [2] dangerous; critical [3] same as &#x8667;
U+3681.0	fUTF8	㚁
U+3681.0	fCNS11643	3-6072
U+3681.0	fGB	5-375A
U+3681.0	fR/S	 32.21
U+3681.0	fTotalStrokes	 24
U+3681.0	fCangjie	GUGGU
U+3681.0	fFourCorner	44210
U+3681.0	fHakka	 kiau5; hiau5
U+3681.0	fCantonese	kiu3; kiu4
U+3681.0	fMandarin	qiao4; qiao2; qu4
U+3681.0	fEnglish	high; elevated; turned up at the ends
U+3682.0	fUTF8	㚂
U+3682.0	fCNS11643	4-6C6E
U+3682.0	fGB	3-347D
U+3682.0	fR/S	 32.22
U+3682.0	fTotalStrokes	 25
U+3682.0	fCangjie	GJBV
U+3682.0	fFourCorner	45130
U+3682.0	fHakka	 nong5;  nang2; lang2
U+3682.0	fCantonese	nong6; long6 (also); zoeng4
U+3682.0	fMandarin	nang4; rang2
U+3682.0	fEnglish	[1] dust; dirt [2] case
U+3683.0	fUTF8	㚃
U+3683.0	fCNS11643	5-373C
U+3683.0	fGB	5-367D
U+3683.0	fR/S	 33.9
U+3683.0	fTotalStrokes	 12
U+3683.0	fCangjie	GBUT
U+3683.0	fFourCorner	40100
U+3683.0	fCantonese	wan1
U+3683.0	fMandarin	yun1; yun2; yun4
U+3683.0	fEnglish	 take a hint; a group of Chinese characters the construction of which suggests the meaning, the fate; good luck or bad are in the pot
U+3684.0	fUTF8	㚄
U+3684.0	fCNS11643	3-4D41
U+3684.0	fR/S	 33.12
U+3684.0	fTotalStrokes	 15
U+3684.0	fCangjie	GBWO
U+3684.0	fFourCorner	40800
U+3684.0	fCantonese	zi3
U+3684.0	fMandarin	zhi4
U+3685.0	fUTF8	㚅
U+3685.0	fCNS11643	3-2B78
U+3685.0	fR/S	 34.6
U+3685.0	fTotalStrokes	 9
U+3685.0	fCangjie	HEMHM
U+3685.0	fFourCorner	27100
U+3685.0	fCantonese	lung4
U+3685.0	fMandarin	long2
U+3685.0	fEnglish	[1] worship of God, praise [2] many; much
U+3686.0	fUTF8	㚆
U+3686.0	fR/S	 35.6
U+3686.0	fTotalStrokes	 9
U+3686.0	fCangjie	YAHE
U+3686.0	fFourCorner	0040
U+3686.0	fCantonese	fuk6
U+3686.0	fMandarin	fu4
U+3686.0	fEnglish	do something in an old way; use the old method; go to the old road
U+3687.0	fUTF8	㚇
U+3687.0	fCNS11643	4-2861
U+3687.0	fGB	3-3A36
U+3687.0	fR/S	 35.6
U+3687.0	fTotalStrokes	 9
U+3687.0	fCangjie	UCHE
U+3687.0	fFourCorner	22400
U+3687.0	fHakka	  zung1
U+3687.0	fCantonese	zung1
U+3687.0	fMandarin	zong1
U+3687.0	fEnglish	[1] ornaments on a horse's head [2] a small feudal state
U+3688.0	fUTF8	㚈
U+3688.0	fR/S	 36.2
U+3688.0	fTotalStrokes	 5
U+3688.0	fCangjie	NIJ
U+3688.0	fCantonese	ngoi6; oi6 (also)
U+3688.0	fEnglish	[Japanese character]
U+3689.0	fUTF8	㚉
U+3689.0	fCNS11643	5-2558
U+3689.0	fR/S	 36.5
U+3689.0	fTotalStrokes	 8
U+3689.0	fCangjie	NSNIN
U+3689.0	fFourCorner	17200
U+3689.0	fCantonese	gu2; gwu2 (also)
U+3689.0	fMandarin	gu3
U+3689.0	fEnglish	[1] (corrupted form) make a profit on sell and buy; the profit in business, [2] interchangeable with &#x5B34;
U+368A.0	fUTF8	㚊
U+368A.0	fCNS11643	4-3677
U+368A.0	fGB	3-3A31
U+368A.0	fR/S	 36.9
U+368A.0	fTotalStrokes	 12
U+368A.0	fCangjie	NNYVO
U+368A.0	fFourCorner	20280
U+368A.0	fHakka	  goi1; gai1
U+368A.0	fCantonese	hoi1
U+368A.0	fMandarin	gai1
U+368A.0	fEnglish	many; numerous
U+368B.0	fUTF8	㚋
U+368B.0	fCNS11643	4-4260
U+368B.0	fGB	3-3A32
U+368B.0	fR/S	 36.11
U+368B.0	fTotalStrokes	 14
U+368B.0	fCangjie	NNBGR
U+368B.0	fFourCorner	27220
U+368B.0	fCantonese	diu1; niu5; seoi4; to5
U+368B.0	fMandarin	diao1
U+368B.0	fEnglish	[1] much; many; numerous [2] great; big; vast
U+368C.0	fUTF8	㚌
U+368C.0	fCNS11643	3-4647
U+368C.0	fR/S	 36.11
U+368C.0	fTotalStrokes	 14
U+368C.0	fCangjie	NNWD
U+368C.0	fFourCorner	26290
U+368C.0	fCantonese	fo2; lo2
U+368C.0	fMandarin	huo3; hua4
U+368C.0	fEnglish	same as &#x5925;
U+368D.0	fUTF8	㚍
U+368D.0	fCNS11643	4-5A7A
U+368D.0	fGB	3-3A33
U+368D.0	fR/S	 36.15
U+368D.0	fTotalStrokes	 18
U+368D.0	fCangjie	NNLMC
U+368D.0	fFourCorner	25280
U+368D.0	fCantonese	fui2; fai3
U+368D.0	fMandarin	kui3; guo2; kui4
U+368D.0	fEnglish	[1] much; many; numerous [2] get more profit [3] estimate more
U+368E.0	fUTF8	㚎
U+368E.0	fCNS11643	3-223A
U+368E.0	fR/S	 37.2
U+368E.0	fTotalStrokes	 5
U+368E.0	fCangjie	KU
U+368E.0	fFourCorner	40770
U+368E.0	fCantonese	heoi3
U+368E.0	fMandarin	qu4
U+368F.0	fUTF8	㚏
U+368F.0	fCNS11643	3-2346
U+368F.0	fR/S	 37.3
U+368F.0	fTotalStrokes	 6
U+368F.0	fCangjie	KT
U+368F.0	fFourCorner	40440
U+368F.0	fHakka	 gau3; hau5
U+368F.0	fCantonese	gou2; zyu5
U+368F.0	fMandarin	gao3
U+368F.0	fEnglish	corrupted form of &#x5930;
U+3690.0	fUTF8	㚐
U+3690.0	fCNS11643	3-2348
U+3690.0	fR/S	 37.3
U+3690.0	fTotalStrokes	 6
U+3690.0	fCangjie	KK
U+3690.0	fFourCorner	40430
U+3690.0	fHakka	 tau5
U+3690.0	fCantonese	tou3; zou6
U+3690.0	fMandarin	tao4; tao3
U+3690.0	fEnglish	same as &#x5957;
U+3691.0	fUTF8	㚑
U+3691.0	fR/S	 37.3
U+3691.0	fTotalStrokes	 6
U+3691.0	fCangjie	SMK
U+3691.0	fCantonese	ling4
U+3691.0	fEnglish	[Japanese character]
U+3692.0	fUTF8	㚒
U+3692.0	fCNS11643	4-2365
U+3692.0	fGB	3-352A
U+3692.0	fR/S	 37.4
U+3692.0	fTotalStrokes	 7
U+3692.0	fCangjie	KOO
U+3692.0	fFourCorner	40030
U+3692.0	fCantonese	sim2; sin2
U+3692.0	fMandarin	shan3
U+3692.0	fEnglish	conceal stolen goods in one's dress
U+3693.0	fUTF8	㚓
U+3693.0	fCNS11643	3-254A
U+3693.0	fR/S	 37.4
U+3693.0	fTotalStrokes	 7
U+3693.0	fCangjie	KD
U+3693.0	fFourCorner	40900
U+3693.0	fCantonese	loi4
U+3693.0	fMandarin	lai3; lai2
U+3693.0	fEnglish	tie-beams of a small boat
U+3694.0	fUTF8	㚔
U+3694.0	fCNS11643	4-2571
U+3694.0	fGB	3-3530
U+3694.0	fR/S	 37.5
U+3694.0	fTotalStrokes	 8
U+3694.0	fCangjie	KTJ
U+3694.0	fFourCorner	40400
U+3694.0	fCantonese	nip6; lip6 (also)
U+3694.0	fMandarin	nie4
U+3694.0	fEnglish	[1] instrument of torture used in ancient times [2] loud [3] an evil doer all the times [4] same as &#x5E78;
U+3695.0	fUTF8	㚕
U+3695.0	fCNS11643	5-2559
U+3695.0	fR/S	 37.5
U+3695.0	fTotalStrokes	 8
U+3695.0	fCangjie	KLLN
U+3695.0	fFourCorner	40520
U+3695.0	fCantonese	bat6; faat1
U+3695.0	fMandarin	bi4; fu2
U+3695.0	fEnglish	[1] great; big; vast [2] same as &#x5F3C; [3] interchangeable with &#x4F5B;
U+3696.0	fUTF8	㚖
U+3696.0	fCNS11643	4-2577
U+3696.0	fGB	3-5C55
U+3696.0	fR/S	 37.5
U+3696.0	fTotalStrokes	 8
U+3696.0	fCangjie	HAK
U+3696.0	fFourCorner	26430
U+3696.0	fCantonese	gou2
U+3696.0	fMandarin	gao3
U+3696.0	fEnglish	[1] come out to the open (said of truth); be known by all [2] ancient form of &#x6FA4;
U+3697.0	fUTF8	㚗
U+3697.0	fCNS11643	4-2573
U+3697.0	fGB	3-352C
U+3697.0	fR/S	 37.5
U+3697.0	fTotalStrokes	 8
U+3697.0	fCangjie	KBM
U+3697.0	fFourCorner	40100
U+3697.0	fCantonese	ce4; wu1
U+3697.0	fMandarin	qie2; xie2
U+3697.0	fEnglish	[1] with a big mouth [2] great; big; vast
U+3698.0	fUTF8	㚘
U+3698.0	fCNS11643	4-2578
U+3698.0	fGB	5-3734
U+3698.0	fR/S	 37.5
U+3698.0	fTotalStrokes	 8
U+3698.0	fCangjie	QOQO
U+3698.0	fFourCorner	55030
U+3698.0	fCantonese	bun6
U+3698.0	fMandarin	ban4
U+3698.0	fEnglish	[1] interchangeable with &#x4F34; [2] same as &#x6276;
U+3699.0	fUTF8	㚙
U+3699.0	fR/S	 37.5
U+3699.0	fTotalStrokes	 8
U+3699.0	fCangjie	KRK
U+3699.0	fFourCorner	4643
U+3699.0	fCantonese	gaa1
U+369A.0	fUTF8	㚚
U+369A.0	fCNS11643	3-2B7D
U+369A.0	fR/S	 37.6
U+369A.0	fTotalStrokes	 9
U+369A.0	fCangjie	KNIR
U+369A.0	fFourCorner	40600
U+369A.0	fCantonese	kwang1
U+369A.0	fMandarin	kong1
U+369A.0	fEnglish	[1] big; great; vast; very [2] liberal [3] bark
U+369B.0	fUTF8	㚛
U+369B.0	fGB	3-3532
U+369B.0	fR/S	 37.6
U+369B.0	fTotalStrokes	 9
U+369B.0	fCangjie	KPA
U+369B.0	fCantonese	hai3
U+369B.0	fMandarin	xi4
U+369B.0	fEnglish	[1] corpulent; abundant [2] flourishing [3] plenteous
U+369C.0	fUTF8	㚜
U+369C.0	fCNS11643	4-3125
U+369C.0	fGB	3-3534
U+369C.0	fR/S	 37.8
U+369C.0	fTotalStrokes	 11
U+369C.0	fCangjie	KIRM
U+369C.0	fFourCorner	40150
U+369C.0	fCantonese	wik6
U+369C.0	fMandarin	yu4; xu4
U+369C.0	fEnglish	[1] with great power; vigorous; very strong; with great force [2] same as &#x57DF;
U+369D.0	fUTF8	㚝
U+369D.0	fCNS11643	6-3E42
U+369D.0	fGB	5-3741
U+369D.0	fR/S	 37.8
U+369D.0	fTotalStrokes	 11
U+369D.0	fCangjie	KOGG
U+369D.0	fFourCorner	40210
U+369D.0	fHakka	 gui1; kui2; kwui1; k'wui2; }
U+369D.0	fCantonese	zeoi1; fui1; teoi4; waai2
U+369D.0	fMandarin	kui2
U+369D.0	fEnglish	same as &#x594E;
U+369E.0	fUTF8	㚞
U+369E.0	fCNS11643	3-347A
U+369E.0	fGB	5-3740
U+369E.0	fR/S	 37.8
U+369E.0	fTotalStrokes	 11
U+369E.0	fCangjie	KDD
U+369E.0	fFourCorner	40990
U+369E.0	fCantonese	sam2
U+369E.0	fMandarin	sen1; meng3; shen3; ying4; yun4
U+369E.0	fEnglish	terror; horror; fear
U+369F.0	fUTF8	㚟
U+369F.0	fCNS11643	3-3A55
U+369F.0	fR/S	 37.9
U+369F.0	fTotalStrokes	 12
U+369F.0	fCangjie	NRPPK
U+369F.0	fFourCorner	27430
U+369F.0	fCantonese	coek3
U+369F.0	fMandarin	chuo4
U+369F.0	fEnglish	a kind of animal
U+36A0.0	fUTF8	㚠
U+36A0.0	fCNS11643	4-3C5C
U+36A0.0	fGB	3-353A
U+36A0.0	fR/S	 37.10
U+36A0.0	fTotalStrokes	 13
U+36A0.0	fCangjie	KTGF
U+36A0.0	fFourCorner	40330
U+36A0.0	fCantonese	hiu1
U+36A0.0	fMandarin	xiao1
U+36A0.0	fEnglish	[1] full grown; fat; plump [2] fertile; rich [3] big and clumsy
U+36A1.0	fUTF8	㚡
U+36A1.0	fCNS11643	3-503E
U+36A1.0	fR/S	 37.13; 30.13
U+36A1.0	fTotalStrokes	 16
U+36A1.0	fCangjie	KRKMR
U+36A1.0	fFourCorner	44620
U+36A1.0	fHakka	 gi3; ji3
U+36A1.0	fCantonese	gei2
U+36A1.0	fMandarin	ji3
U+36A1.0	fEnglish	the kindly behaviour of two friends on meeting
U+36A2.0	fUTF8	㚢
U+36A2.0	fCNS11643	3-2174
U+36A2.0	fGB	5-314C
U+36A2.0	fR/S	 38.2; 9.3
U+36A2.0	fTotalStrokes	 5
U+36A2.0	fCangjie	OV
U+36A2.0	fFourCorner	24240
U+36A2.0	fHakka	 nu2
U+36A2.0	fCantonese	nou4; lou4 (also)
U+36A2.0	fMandarin	nu2
U+36A2.0	fEnglish	[1] ancient form of &#x5974; [2] ancient form of &#x4FAE;
U+36A3.0	fUTF8	㚣
U+36A3.0	fCNS11643	3-234C
U+36A3.0	fR/S	 38.3
U+36A3.0	fTotalStrokes	 6
U+36A3.0	fCangjie	VV
U+36A3.0	fFourCorner	40400
U+36A3.0	fHakka	 gau3; hau2
U+36A3.0	fCantonese	gaau2; gau2; hau4
U+36A3.0	fMandarin	jiao1; can4; xiao2
U+36A3.0	fEnglish	[1] pretty; handsome; artful [2] same as &#x216DD;
U+36A4.0	fUTF8	㚤
U+36A4.0	fCNS11643	3-234A
U+36A4.0	fGB	5-3F3E
U+36A4.0	fR/S	 38.3
U+36A4.0	fTotalStrokes	 6
U+36A4.0	fCangjie	VIP
U+36A4.0	fFourCorner	43440
U+36A4.0	fHakka	 rit7; jit7; jit8
U+36A4.0	fCantonese	jik6; zik6
U+36A4.0	fMandarin	yi4
U+36A4.0	fEnglish	a women who conducted the worship of the goddess of silk-worms
U+36A5.0	fUTF8	㚥
U+36A5.0	fCNS11643	4-2256
U+36A5.0	fR/S	 38.3
U+36A5.0	fTotalStrokes	 6
U+36A5.0	fCangjie	VMD
U+36A5.0	fFourCorner	41440
U+36A5.0	fCantonese	jyu1; zyu1
U+36A5.0	fMandarin	yu2
U+36A5.0	fEnglish	to become associated with each other in good manners and politeness
U+36A6.0	fUTF8	㚦
U+36A6.0	fCNS11643	4-2259
U+36A6.0	fR/S	 38.3
U+36A6.0	fTotalStrokes	 6
U+36A6.0	fCangjie	VML
U+36A6.0	fFourCorner	41420
U+36A6.0	fHakka	 hi1; ki1
U+36A6.0	fCantonese	gei1
U+36A6.0	fMandarin	ji1; yi2
U+36A6.0	fEnglish	ancient form of &#x59EC;
U+36A7.0	fUTF8	㚧
U+36A7.0	fCNS11643	4-2255
U+36A7.0	fGB	5-3F41
U+36A7.0	fR/S	 38.3
U+36A7.0	fTotalStrokes	 6
U+36A7.0	fCangjie	VI
U+36A7.0	fFourCorner	40400
U+36A7.0	fCantonese	cim1; ngam4; zim5
U+36A7.0	fMandarin	yan3
U+36A7.0	fEnglish	same as &#x5B10;
U+36A8.0	fUTF8	㚨
U+36A8.0	fCNS11643	3-2349
U+36A8.0	fR/S	 38.3
U+36A8.0	fTotalStrokes	 6
U+36A8.0	fCangjie	VNJ
U+36A8.0	fFourCorner	47410
U+36A8.0	fCantonese	san1
U+36A8.0	fEnglish	used in girl's name
U+36A9.0	fUTF8	㚩
U+36A9.0	fCNS11643	3-2552
U+36A9.0	fR/S	 38.4
U+36A9.0	fTotalStrokes	 7
U+36A9.0	fCangjie	VBMM
U+36A9.0	fFourCorner	47450
U+36A9.0	fCantonese	jim5; zim5
U+36A9.0	fMandarin	ran3
U+36A9.0	fEnglish	with a charming sprightly carriage (said of a woman)
U+36AA.0	fUTF8	㚪
U+36AA.0	fCNS11643	3-2557
U+36AA.0	fR/S	 38.4
U+36AA.0	fTotalStrokes	 7
U+36AA.0	fCangjie	VHQU
U+36AA.0	fFourCorner	42410
U+36AA.0	fCantonese	hou3
U+36AA.0	fMandarin	hao4
U+36AA.0	fEnglish	same as &#x29C8A;
U+36AB.0	fUTF8	㚫
U+36AB.0	fCNS11643	3-2550
U+36AB.0	fR/S	 38.4
U+36AB.0	fTotalStrokes	 7
U+36AB.0	fCangjie	VNHE
U+36AB.0	fFourCorner	47440
U+36AB.0	fCantonese	saap3
U+36AB.0	fMandarin	sa4; sha4; zha4
U+36AB.0	fEnglish	[1] used in girl's name [2] a woman's countenance; features
U+36AC.0	fUTF8	㚬
U+36AC.0	fCNS11643	3-254F
U+36AC.0	fR/S	 38.4
U+36AC.0	fTotalStrokes	 7
U+36AC.0	fCangjie	VPIM
U+36AC.0	fFourCorner	47420
U+36AC.0	fCantonese	gwan1
U+36AC.0	fMandarin	jun1
U+36AC.0	fEnglish	a lady start to doll up
U+36AD.0	fUTF8	㚭
U+36AD.0	fCNS11643	3-254D
U+36AD.0	fR/S	 38.4
U+36AD.0	fTotalStrokes	 7
U+36AD.0	fCangjie	VIKU
U+36AD.0	fFourCorner	43410
U+36AD.0	fCantonese	jau4; zaau4
U+36AD.0	fMandarin	you2
U+36AD.0	fEnglish	used in girl's name
U+36AE.0	fUTF8	㚮
U+36AE.0	fCNS11643	3-2554
U+36AE.0	fR/S	 38.4
U+36AE.0	fTotalStrokes	 7
U+36AE.0	fCangjie	VBHN
U+36AE.0	fFourCorner	47410
U+36AE.0	fCantonese	daam1
U+36AE.0	fMandarin	dan1
U+36AE.0	fEnglish	same as &#x5989;
U+36AF.0	fUTF8	㚯
U+36AF.0	fGB	7-2335
U+36AF.0	fR/S	 38.4
U+36AF.0	fTotalStrokes	 7
U+36AF.0	fCangjie	VHNK
U+36AF.0	fCantonese	cam4; caam4
U+36AF.0	fMandarin	xin2; xun2
U+36AF.0	fEnglish	a Chinese family name
U+36B0.0	fUTF8	㚰
U+36B0.0	fCNS11643	3-2831
U+36B0.0	fGB	5-3F51
U+36B0.0	fR/S	 38.5
U+36B0.0	fTotalStrokes	 8
U+36B0.0	fCangjie	VMFM
U+36B0.0	fFourCorner	41410
U+36B0.0	fCantonese	pui1
U+36B0.0	fMandarin	bi3
U+36B0.0	fEnglish	same as &#x80DA;
U+36B1.0	fUTF8	㚱
U+36B1.0	fCNS11643	3-283B
U+36B1.0	fR/S	 38.5
U+36B1.0	fTotalStrokes	 8
U+36B1.0	fCangjie	VOM
U+36B1.0	fFourCorner	42410
U+36B1.0	fCantonese	jau1; zaau1
U+36B1.0	fMandarin	qiu1
U+36B1.0	fEnglish	used in girl's name
U+36B2.0	fUTF8	㚲
U+36B2.0	fCNS11643	4-257B
U+36B2.0	fGB	3-3C37
U+36B2.0	fR/S	 38.5
U+36B2.0	fTotalStrokes	 8
U+36B2.0	fCangjie	VYR
U+36B2.0	fFourCorner	41460
U+36B2.0	fCantonese	cim1; sim1
U+36B2.0	fMandarin	chan1; dian3; shan4
U+36B2.0	fEnglish	[1] small and weak [2] used in girl's name [3] a woman's feature; lady's face
U+36B3.0	fUTF8	㚳
U+36B3.0	fCNS11643	3-2842
U+36B3.0	fR/S	 38.5
U+36B3.0	fTotalStrokes	 8
U+36B3.0	fCangjie	VKSR
U+36B3.0	fFourCorner	46400
U+36B3.0	fCantonese	aa3; ngaa3 (also); o1
U+36B3.0	fMandarin	e1
U+36B3.0	fEnglish	same as &#x59BF;
U+36B4.0	fUTF8	㚴
U+36B4.0	fCNS11643	3-2832
U+36B4.0	fR/S	 38.5
U+36B4.0	fTotalStrokes	 8
U+36B4.0	fCangjie	VKLB
U+36B4.0	fFourCorner	44420
U+36B4.0	fCantonese	pou3; bou3
U+36B4.0	fMandarin	pu4; bu4
U+36B4.0	fEnglish	[1] pretty girl; beauty; belle [2] handsome; female beauty
U+36B5.0	fUTF8	㚵
U+36B5.0	fCNS11643	3-283A
U+36B5.0	fR/S	 38.5
U+36B5.0	fTotalStrokes	 8
U+36B5.0	fCangjie	VHEY
U+36B5.0	fFourCorner	47430
U+36B5.0	fCantonese	dung1
U+36B5.0	fMandarin	dong1
U+36B5.0	fEnglish	used in girl's name
U+36B6.0	fUTF8	㚶
U+36B6.0	fCNS11643	3-2843
U+36B6.0	fR/S	 38.5
U+36B6.0	fTotalStrokes	 8
U+36B6.0	fCangjie	VRLR
U+36B6.0	fFourCorner	47470
U+36B6.0	fCantonese	ci5
U+36B6.0	fMandarin	si4
U+36B6.0	fEnglish	[1] same as &#x59D2; [2] same as &#x59EC;
U+36B7.0	fUTF8	㚷
U+36B7.0	fCNS11643	3-2847
U+36B7.0	fR/S	 38.5
U+36B7.0	fTotalStrokes	 8
U+36B7.0	fCangjie	VOF
U+36B7.0	fFourCorner	48490
U+36B7.0	fHakka	 nai1
U+36B7.0	fCantonese	naai5; laai5 (also); nai5
U+36B7.0	fMandarin	nai3; er3; ni3; ni4
U+36B7.0	fEnglish	same as &#x5B2D;
U+36B8.0	fUTF8	㚸
U+36B8.0	fCNS11643	3-2834
U+36B8.0	fR/S	 38.5
U+36B8.0	fTotalStrokes	 8
U+36B8.0	fCangjie	VSMR
U+36B8.0	fFourCorner	47420
U+36B8.0	fCantonese	si1
U+36B8.0	fMandarin	si1
U+36B8.0	fEnglish	same as &#x5AA4;
U+36B9.0	fUTF8	㚹
U+36B9.0	fCNS11643	4-2621
U+36B9.0	fR/S	 38.5
U+36B9.0	fTotalStrokes	 8
U+36B9.0	fCangjie	VHHL
U+36B9.0	fFourCorner	47420
U+36B9.0	fCantonese	maau4; mau5
U+36B9.0	fMandarin	mao3
U+36B9.0	fEnglish	[1] same as &#x5A8C; [2] same as &#x61F0;
U+36BA.0	fUTF8	㚺
U+36BA.0	fCNS11643	3-2844
U+36BA.0	fR/S	 38.5
U+36BA.0	fTotalStrokes	 8
U+36BA.0	fCangjie	VNSD
U+36BA.0	fFourCorner	47440
U+36BA.0	fHakka	 jin3
U+36BA.0	fCantonese	jan6; zaan6
U+36BA.0	fMandarin	yun4
U+36BA.0	fEnglish	non-classical form of &#x5B55;
U+36BB.0	fUTF8	㚻
U+36BB.0	fCNS11643	3-283E
U+36BB.0	fGB	5-5A24
U+36BB.0	fR/S	 38.5
U+36BB.0	fTotalStrokes	 8
U+36BB.0	fCangjie	WV
U+36BB.0	fFourCorner	60400
U+36BB.0	fCantonese	gai1; gei1
U+36BB.0	fMandarin	ji1
U+36BB.0	fEnglish	sodomy; bugger
U+36BC.0	fUTF8	㚼
U+36BC.0	fCNS11643	3-2836
U+36BC.0	fR/S	 38.5
U+36BC.0	fTotalStrokes	 8
U+36BC.0	fCangjie	VW
U+36BC.0	fFourCorner	46400
U+36BC.0	fCantonese	nau2; lau2 (also); naau2
U+36BC.0	fMandarin	hao4
U+36BC.0	fEnglish	a family name
U+36BD.0	fUTF8	㚽
U+36BD.0	fCNS11643	3-283C
U+36BD.0	fR/S	 38.5
U+36BD.0	fTotalStrokes	 8
U+36BD.0	fCangjie	MSV
U+36BD.0	fFourCorner	11400
U+36BD.0	fCantonese	haau2
U+36BD.0	fMandarin	qiao3
U+36BD.0	fEnglish	used in girl's name
U+36BE.0	fUTF8	㚾
U+36BE.0	fCNS11643	5-255D
U+36BE.0	fGB	S-2329
U+36BE.0	fR/S	 38.5
U+36BE.0	fTotalStrokes	 8
U+36BE.0	fCangjie	VRHU
U+36BE.0	fFourCorner	46410
U+36BE.0	fCantonese	hing1
U+36BE.0	fMandarin	xiong1
U+36BE.0	fEnglish	[1] have fun [2] sport; play; frolic
U+36BF.0	fUTF8	㚿
U+36BF.0	fCNS11643	3-2839
U+36BF.0	fGB	7-2332
U+36BF.0	fR/S	 38.5
U+36BF.0	fTotalStrokes	 8
U+36BF.0	fCangjie	VPRU
U+36BF.0	fFourCorner	47410
U+36BF.0	fHakka	 bau1
U+36BF.0	fCantonese	baau1; pau4
U+36BF.0	fMandarin	bao1; pao2
U+36BF.0	fEnglish	same as &#x5305;
U+36C0.0	fUTF8	㛀
U+36C0.0	fGB	7-2334
U+36C0.0	fR/S	 38.5
U+36C0.0	fTotalStrokes	 8
U+36C0.0	fCangjie	VNSM
U+36C0.0	fCantonese	co4
U+36C1.0	fUTF8	㛁
U+36C1.0	fCNS11643	3-2830
U+36C1.0	fR/S	 38.5
U+36C1.0	fTotalStrokes	 8
U+36C1.0	fCangjie	VMFJ
U+36C1.0	fFourCorner	41440
U+36C1.0	fCantonese	paang1; ping1
U+36C1.0	fMandarin	peng1
U+36C1.0	fEnglish	[1] quick; urgent; anxious [2] loyal sincere and respectful
U+36C2.0	fUTF8	㛂
U+36C2.0	fCNS11643	4-286D
U+36C2.0	fGB	5-3F69
U+36C2.0	fR/S	 38.6
U+36C2.0	fTotalStrokes	 9
U+36C2.0	fCangjie	VHAU
U+36C2.0	fFourCorner	42410
U+36C2.0	fHakka	 ngo3
U+36C2.0	fCantonese	no4; lo4 (also)
U+36C2.0	fMandarin	e3; nuo3
U+36C2.0	fEnglish	elegant; delicate
U+36C3.0	fUTF8	㛃
U+36C3.0	fCNS11643	4-2869
U+36C3.0	fGB	3-3C44
U+36C3.0	fR/S	 38.6
U+36C3.0	fTotalStrokes	 9
U+36C3.0	fCangjie	QHV
U+36C3.0	fFourCorner	57400
U+36C3.0	fCantonese	git3
U+36C3.0	fMandarin	jie2
U+36C3.0	fEnglish	[1] clean; pure; clear [2] usually used for names
U+36C4.0	fUTF8	㛄
U+36C4.0	fCNS11643	3-2B7E
U+36C4.0	fGB	5-3F6E
U+36C4.0	fR/S	 38.6
U+36C4.0	fTotalStrokes	 9
U+36C4.0	fCangjie	VYHV
U+36C4.0	fFourCorner	40430
U+36C4.0	fCantonese	ji1; zi1
U+36C4.0	fMandarin	yi1
U+36C4.0	fEnglish	used in girl's name
U+36C5.0	fUTF8	㛅
U+36C5.0	fCNS11643	3-2C22
U+36C5.0	fGB	5-3F5D
U+36C5.0	fR/S	 38.6
U+36C5.0	fTotalStrokes	 9
U+36C5.0	fCangjie	VSJ
U+36C5.0	fFourCorner	41440
U+36C5.0	fCantonese	nei6; lei6 (also)
U+36C5.0	fMandarin	er4
U+36C5.0	fEnglish	used in girl's name
U+36C6.0	fUTF8	㛆
U+36C6.0	fR/S	 38.6
U+36C6.0	fTotalStrokes	 9
U+36C6.0	fCangjie	VNSD
U+36C6.0	fCantonese	do3; do2
U+36C6.0	fMandarin	duo3
U+36C6.0	fEnglish	[1] speculate; conjecture; assume; make an intelligent guess; fathom [2] beautiful [3] used in girl's name
U+36C7.0	fUTF8	㛇
U+36C7.0	fGB	S-232A
U+36C7.0	fR/S	 38.6
U+36C7.0	fTotalStrokes	 9
U+36C7.0	fCangjie	VIG
U+36C7.0	fCantonese	zong1
U+36C8.0	fUTF8	㛈
U+36C8.0	fGB	S-2330
U+36C8.0	fR/S	 38.6
U+36C8.0	fTotalStrokes	 9
U+36C8.0	fCangjie	VJKS
U+36C8.0	fCantonese	haau2
U+36C9.0	fUTF8	㛉
U+36C9.0	fCNS11643	F-2A2A
U+36C9.0	fGB	S-2328
U+36C9.0	fR/S	 38.6
U+36C9.0	fTotalStrokes	 9
U+36C9.0	fCangjie	VMLW
U+36C9.0	fCantonese	jiu3
U+36CA.0	fUTF8	㛊
U+36CA.0	fCNS11643	4-2867
U+36CA.0	fGB	5-3F6A
U+36CA.0	fR/S	 38.6
U+36CA.0	fTotalStrokes	 9
U+36CA.0	fCangjie	VHND
U+36CA.0	fFourCorner	47490
U+36CA.0	fCantonese	do3; do2
U+36CA.0	fMandarin	duo3
U+36CA.0	fEnglish	[1] speculate; conjecture; assume; make an intelligent guess; fathom [2] beautiful (of a woman) [3] used in girl's name
U+36CB.0	fUTF8	㛋
U+36CB.0	fR/S	 38.6
U+36CB.0	fTotalStrokes	 9
U+36CB.0	fCangjie	VOP
U+36CB.0	fCantonese	nei5; lei5 (also)
U+36CC.0	fUTF8	㛌
U+36CC.0	fR/S	 38.6
U+36CC.0	fTotalStrokes	 9
U+36CC.0	fCangjie	VIHQ
U+36CC.0	fCantonese	mau4
U+36CD.0	fUTF8	㛍
U+36CD.0	fCNS11643	3-303F
U+36CD.0	fGB	5-3F62
U+36CD.0	fR/S	 38.7
U+36CD.0	fTotalStrokes	 10
U+36CD.0	fCangjie	VKOO
U+36CD.0	fFourCorner	44430
U+36CD.0	fCantonese	hip3; hap6
U+36CD.0	fMandarin	qie4
U+36CD.0	fEnglish	[1] have one's ambition fulfilled; be successful in one's career [2] same as &#x611C;
U+36CE.0	fUTF8	㛎
U+36CE.0	fCNS11643	3-3042
U+36CE.0	fR/S	 38.7
U+36CE.0	fTotalStrokes	 10
U+36CE.0	fCangjie	VRHR
U+36CE.0	fFourCorner	46460
U+36CE.0	fCantonese	leoi5
U+36CE.0	fMandarin	luu3
U+36CE.0	fEnglish	an ugly woman
U+36CF.0	fUTF8	㛏
U+36CF.0	fCNS11643	3-303D
U+36CF.0	fR/S	 38.7
U+36CF.0	fTotalStrokes	 10
U+36CF.0	fCangjie	VIJE
U+36CF.0	fFourCorner	43430
U+36CF.0	fCantonese	kau4
U+36CF.0	fMandarin	qiu2; ou4
U+36CF.0	fEnglish	used in girl's name
U+36D0.0	fUTF8	㛐
U+36D0.0	fCNS11643	3-3049
U+36D0.0	fR/S	 38.7
U+36D0.0	fTotalStrokes	 10
U+36D0.0	fCangjie	VMLK
U+36D0.0	fFourCorner	41440
U+36D0.0	fCantonese	sou2
U+36D0.0	fMandarin	sao3; sou3; xuan2
U+36D0.0	fEnglish	same as &#x36EE;
U+36D1.0	fUTF8	㛑
U+36D1.0	fCNS11643	3-3047
U+36D1.0	fGB	5-3F77
U+36D1.0	fR/S	 38.7
U+36D1.0	fTotalStrokes	 10
U+36D1.0	fCangjie	YEV
U+36D1.0	fFourCorner	27400
U+36D1.0	fHakka	  can5; con1; can1
U+36D1.0	fCantonese	caan3
U+36D1.0	fMandarin	can4
U+36D1.0	fEnglish	beautiful
U+36D2.0	fUTF8	㛒
U+36D2.0	fCNS11643	4-2C3B
U+36D2.0	fGB	3-3C60
U+36D2.0	fR/S	 38.7
U+36D2.0	fTotalStrokes	 10
U+36D2.0	fCangjie	VMRT
U+36D2.0	fFourCorner	41410
U+36D2.0	fCantonese	dau6; daau2; daau6; zaau6
U+36D2.0	fMandarin	dou4
U+36D2.0	fEnglish	can not speak
U+36D3.0	fUTF8	㛓
U+36D3.0	fCNS11643	3-3043
U+36D3.0	fR/S	 38.7
U+36D3.0	fTotalStrokes	 10
U+36D3.0	fCangjie	VKKB
U+36D3.0	fFourCorner	44420
U+36D3.0	fCantonese	hei1; ci1; dou1
U+36D3.0	fMandarin	chi1
U+36D3.0	fEnglish	used in girl's name
U+36D4.0	fUTF8	㛔
U+36D4.0	fCNS11643	3-3050
U+36D4.0	fR/S	 38.7
U+36D4.0	fTotalStrokes	 10
U+36D4.0	fCangjie	VHEJ
U+36D4.0	fFourCorner	47450
U+36D4.0	fHakka	 fung1
U+36D4.0	fCantonese	fung1
U+36D4.0	fMandarin	feng1; peng2
U+36D4.0	fEnglish	same as &#x59A6;
U+36D5.0	fUTF8	㛕
U+36D5.0	fCNS11643	3-304C
U+36D5.0	fR/S	 38.7
U+36D5.0	fTotalStrokes	 10
U+36D5.0	fCangjie	VRAU
U+36D5.0	fFourCorner	46410
U+36D5.0	fHakka	 ap7; hap8
U+36D5.0	fCantonese	jap1; haap6; zaap1
U+36D5.0	fMandarin	yan3; yi4
U+36D5.0	fEnglish	ancient form of &#x59F6;
U+36D6.0	fUTF8	㛖
U+36D6.0	fCNS11643	3-3044
U+36D6.0	fR/S	 38.7
U+36D6.0	fTotalStrokes	 10
U+36D6.0	fCangjie	VICE
U+36D6.0	fFourCorner	43440
U+36D6.0	fHakka	 so1
U+36D6.0	fCantonese	so1; zo6
U+36D6.0	fMandarin	suo1
U+36D6.0	fEnglish	used in a girl's name
U+36D7.0	fUTF8	㛗
U+36D7.0	fCNS11643	4-2C3F
U+36D7.0	fGB	3-3C67
U+36D7.0	fR/S	 38.7
U+36D7.0	fTotalStrokes	 10
U+36D7.0	fCangjie	VOOG
U+36D7.0	fFourCorner	48410
U+36D7.0	fCantonese	ce1; so1
U+36D7.0	fMandarin	cha1; zuo4
U+36D7.0	fEnglish	[1] frivolous; flippant; disrespectful; irreverent; to insult [2] young [3] used in girl's name
U+36D8.0	fUTF8	㛘
U+36D8.0	fCNS11643	4-2C3A
U+36D8.0	fGB	3-3C5D
U+36D8.0	fR/S	 38.7
U+36D8.0	fTotalStrokes	 10
U+36D8.0	fCangjie	VJBD
U+36D8.0	fFourCorner	44440
U+36D8.0	fCantonese	put3; but6
U+36D8.0	fMandarin	mo4; po4
U+36D8.0	fEnglish	[1] used in girl's name [2] a fat woman
U+36D9.0	fUTF8	㛙
U+36D9.0	fCNS11643	3-3039
U+36D9.0	fR/S	 38.7
U+36D9.0	fTotalStrokes	 10
U+36D9.0	fCangjie	VYTJ
U+36D9.0	fFourCorner	40440
U+36D9.0	fCantonese	san1
U+36D9.0	fMandarin	xin1; qie4; qin2; shen3; shen4
U+36D9.0	fEnglish	[1] used in girl's name [2] same as &#x59BE;
U+36DA.0	fUTF8	㛚
U+36DA.0	fCNS11643	4-2C3C
U+36DA.0	fGB	3-3C6B
U+36DA.0	fR/S	 38.7
U+36DA.0	fTotalStrokes	 10
U+36DA.0	fCangjie	VNIB
U+36DA.0	fFourCorner	47420
U+36DA.0	fCantonese	tung2
U+36DA.0	fMandarin	tong3
U+36DA.0	fEnglish	[1] equal; uniform [2] used in girl's name
U+36DB.0	fUTF8	㛛
U+36DB.0	fCNS11643	3-3051
U+36DB.0	fR/S	 38.7
U+36DB.0	fTotalStrokes	 10
U+36DB.0	fCangjie	VHXH
U+36DB.0	fFourCorner	47440
U+36DB.0	fHakka	 shin2; siin2; shin1; siin1; zhin3; zihn3
U+36DB.0	fCantonese	san1; zan3; saan1; zaan3
U+36DB.0	fMandarin	shen1; zhen4; xin4; chen2
U+36DB.0	fEnglish	[1] pregnant [2] same as &#x5A20;
U+36DC.0	fUTF8	㛜
U+36DC.0	fCNS11643	3-3046
U+36DC.0	fR/S	 38.7
U+36DC.0	fTotalStrokes	 10
U+36DC.0	fCangjie	OKV
U+36DC.0	fFourCorner	28400
U+36DC.0	fCantonese	jau4; zaau4
U+36DC.0	fMandarin	you2
U+36DC.0	fEnglish	used in girl's name
U+36DD.0	fUTF8	㛝
U+36DD.0	fCNS11643	3-3041
U+36DD.0	fR/S	 38.7
U+36DD.0	fTotalStrokes	 10
U+36DD.0	fCangjie	VBUC
U+36DD.0	fFourCorner	46480
U+36DD.0	fCantonese	bui3
U+36DD.0	fMandarin	bei4; beng4
U+36DD.0	fEnglish	used in girl's name
U+36DE.0	fUTF8	㛞
U+36DE.0	fCNS11643	3-303A
U+36DE.0	fR/S	 38.7
U+36DE.0	fTotalStrokes	 10
U+36DE.0	fCangjie	VMGT
U+36DE.0	fFourCorner	41440
U+36DE.0	fCantonese	lung6
U+36DE.0	fMandarin	long4
U+36DE.0	fEnglish	used in girl's name
U+36DF.0	fUTF8	㛟
U+36DF.0	fGB	S-2327
U+36DF.0	fR/S	 38.7
U+36DF.0	fTotalStrokes	 10
U+36DF.0	fCangjie	VNBK
U+36DF.0	fCantonese	wun6
U+36E0.0	fUTF8	㛠
U+36E0.0	fGB	S-2331
U+36E0.0	fR/S	 38.4
U+36E0.0	fTotalStrokes	 7
U+36E0.0	fCangjie	VLSA
U+36E0.0	fCantonese	haan5
U+36E1.0	fUTF8	㛡
U+36E1.0	fCNS11643	F-2E6B
U+36E1.0	fR/S	 38.7
U+36E1.0	fTotalStrokes	 10
U+36E1.0	fCangjie	VJMU
U+36E1.0	fFourCorner	4341
U+36E1.0	fCantonese	wan6
U+36E2.0	fUTF8	㛢
U+36E2.0	fCNS11643	F-2E78
U+36E2.0	fGB	S-232B
U+36E2.0	fR/S	 38.7
U+36E2.0	fTotalStrokes	 10
U+36E2.0	fCangjie	VHDS
U+36E2.0	fFourCorner	4242
U+36E2.0	fCantonese	sau3
U+36E3.0	fUTF8	㛣
U+36E3.0	fGB	7-2336
U+36E3.0	fR/S	 38.7
U+36E3.0	fTotalStrokes	 10
U+36E3.0	fCangjie	VRBO
U+36E3.0	fCantonese	wan4
U+36E4.0	fUTF8	㛤
U+36E4.0	fGB	7-2337
U+36E4.0	fR/S	 38.7
U+36E4.0	fTotalStrokes	 10
U+36E4.0	fCangjie	VMBB
U+36E4.0	fCantonese	lei4
U+36E5.0	fUTF8	㛥
U+36E5.0	fCNS11643	4-3129
U+36E5.0	fGB	3-3D2C
U+36E5.0	fR/S	 38.8
U+36E5.0	fTotalStrokes	 11
U+36E5.0	fCangjie	VEA
U+36E5.0	fFourCorner	42460
U+36E5.0	fCantonese	taap3; dap6
U+36E5.0	fMandarin	ta4
U+36E5.0	fEnglish	[1] prostrate; make obeisance [2] concede or submit willingly
U+36E6.0	fUTF8	㛦
U+36E6.0	fCNS11643	3-3526
U+36E6.0	fR/S	 38.8
U+36E6.0	fTotalStrokes	 11
U+36E6.0	fCangjie	VDD
U+36E6.0	fFourCorner	44490
U+36E6.0	fCantonese	lam5
U+36E6.0	fMandarin	lan3
U+36E6.0	fEnglish	good; nice; fine; excellent
U+36E7.0	fUTF8	㛧
U+36E7.0	fCNS11643	3-352D
U+36E7.0	fR/S	 38.8
U+36E7.0	fTotalStrokes	 11
U+36E7.0	fCangjie	VUMR
U+36E7.0	fFourCorner	42460
U+36E7.0	fCantonese	man5; mun5
U+36E7.0	fMandarin	man3
U+36E7.0	fEnglish	the name of a county in today's Shanxi province
U+36E8.0	fUTF8	㛨
U+36E8.0	fCNS11643	3-3521
U+36E8.0	fR/S	 38.8
U+36E8.0	fTotalStrokes	 11
U+36E8.0	fCangjie	VTGU
U+36E8.0	fFourCorner	48410
U+36E8.0	fCantonese	koeng2; goeng2
U+36E8.0	fMandarin	qiang3
U+36E8.0	fEnglish	(corrupted form) chaos; anarchy; distractions; confusion
U+36E9.0	fUTF8	㛩
U+36E9.0	fCNS11643	3-3536
U+36E9.0	fR/S	 38.8; 38.7
U+36E9.0	fTotalStrokes	 11; 10
U+36E9.0	fCangjie	VYIB
U+36E9.0	fFourCorner	40420
U+36E9.0	fCantonese	zuk6
U+36E9.0	fMandarin	zhou2
U+36E9.0	fEnglish	same as &#x59AF;
U+36EA.0	fUTF8	㛪
U+36EA.0	fCNS11643	4-3128
U+36EA.0	fGB	3-3C79
U+36EA.0	fR/S	 38.8
U+36EA.0	fTotalStrokes	 11
U+36EA.0	fCangjie	VKLU
U+36EA.0	fFourCorner	44410
U+36EA.0	fCantonese	jim3; aam1; gim1; him1; zim1
U+36EA.0	fMandarin	yan4; yuan2
U+36EA.0	fEnglish	[1] slander; libel [2] maidservant [3] used in girl's name [4] womanly
U+36EB.0	fUTF8	㛫
U+36EB.0	fCNS11643	3-352C
U+36EB.0	fGB	5-4025
U+36EB.0	fR/S	 38.8
U+36EB.0	fTotalStrokes	 11
U+36EB.0	fCangjie	VAPH
U+36EB.0	fFourCorner	46420
U+36EB.0	fCantonese	sik1; sek3
U+36EB.0	fMandarin	xi2
U+36EB.0	fEnglish	used in girl's name
U+36EC.0	fUTF8	㛬
U+36EC.0	fCNS11643	3-3523
U+36EC.0	fR/S	 38.8
U+36EC.0	fTotalStrokes	 11
U+36EC.0	fCangjie	VGCG
U+36EC.0	fFourCorner	44410
U+36EC.0	fCantonese	luk6
U+36EC.0	fMandarin	lu4
U+36EC.0	fEnglish	used in girl's name
U+36ED.0	fUTF8	㛭
U+36ED.0	fCNS11643	3-3525
U+36ED.0	fR/S	 38.8
U+36ED.0	fTotalStrokes	 11
U+36ED.0	fCangjie	VTA
U+36ED.0	fFourCorner	44460
U+36ED.0	fCantonese	sik1; sim3
U+36ED.0	fMandarin	xi2
U+36ED.0	fEnglish	used in girl's name
U+36EE.0	fUTF8	㛮
U+36EE.0	fCNS11643	3-3A71
U+36EE.0	fR/S	 38.9
U+36EE.0	fTotalStrokes	 12
U+36EE.0	fCangjie	VJFE
U+36EE.0	fFourCorner	43440
U+36EE.0	fCantonese	sou2
U+36EE.0	fMandarin	sao3
U+36EE.0	fEnglish	same as &#x5AC2;
U+36EF.0	fUTF8	㛯
U+36EF.0	fCNS11643	3-3A67
U+36EF.0	fGB	5-4038
U+36EF.0	fR/S	 38.9
U+36EF.0	fTotalStrokes	 12
U+36EF.0	fCangjie	VNUS
U+36EF.0	fFourCorner	44410
U+36EF.0	fCantonese	faan3; min5
U+36EF.0	fMandarin	zhuan4; mian3
U+36EF.0	fEnglish	same as &#x5B14;
U+36F0.0	fUTF8	㛰
U+36F0.0	fCNS11643	3-3A74
U+36F0.0	fGB	5-4041
U+36F0.0	fR/S	 38.9
U+36F0.0	fTotalStrokes	 12
U+36F0.0	fCangjie	VRPA
U+36F0.0	fFourCorner	47460
U+36F0.0	fHakka	 fun1
U+36F0.0	fCantonese	fan1
U+36F0.0	fMandarin	hun1; yu4
U+36F0.0	fEnglish	same as &#x5A5A;
U+36F1.0	fUTF8	㛱
U+36F1.0	fCNS11643	3-3A64
U+36F1.0	fGB	5-4034
U+36F1.0	fR/S	 38.9
U+36F1.0	fTotalStrokes	 12
U+36F1.0	fCangjie	VWMV
U+36F1.0	fFourCorner	46430
U+36F1.0	fCantonese	wui2
U+36F1.0	fMandarin	wei2; rui4; wei3
U+36F1.0	fEnglish	[1] exquisite; fine [2] used in girl's name
U+36F2.0	fUTF8	㛲
U+36F2.0	fCNS11643	3-3A76
U+36F2.0	fR/S	 38.9
U+36F2.0	fTotalStrokes	 12
U+36F2.0	fCangjie	VMBC
U+36F2.0	fFourCorner	41480
U+36F2.0	fCantonese	faat3
U+36F2.0	fMandarin	yi2; fa3; fa4
U+36F2.0	fEnglish	corrupted form of &#x24416B;
U+36F3.0	fUTF8	㛳
U+36F3.0	fCNS11643	3-3A6C
U+36F3.0	fGB	5-403B
U+36F3.0	fR/S	 38.9
U+36F3.0	fTotalStrokes	 12
U+36F3.0	fCangjie	YBLBV
U+36F3.0	fFourCorner	00400
U+36F3.0	fCantonese	jai6; zaai6; zaam3
U+36F3.0	fMandarin	yi4; cha4
U+36F3.0	fEnglish	an abnomal unborn baby
U+36F4.0	fUTF8	㛴
U+36F4.0	fCNS11643	3-3A6A
U+36F4.0	fGB	5-4042
U+36F4.0	fR/S	 38.9
U+36F4.0	fTotalStrokes	 12
U+36F4.0	fCangjie	VVVW
U+36F4.0	fFourCorner	42460
U+36F4.0	fHakka	 nau1
U+36F4.0	fCantonese	nou5; lou5 (also); zyun5
U+36F4.0	fMandarin	nao3
U+36F4.0	fEnglish	same as &#x60F1;
U+36F5.0	fUTF8	㛵
U+36F5.0	fCNS11643	3-3A65
U+36F5.0	fR/S	 38.9
U+36F5.0	fTotalStrokes	 12
U+36F5.0	fCangjie	VBGB
U+36F5.0	fFourCorner	42440
U+36F5.0	fCantonese	cing1
U+36F5.0	fMandarin	cheng1
U+36F5.0	fEnglish	used in girl's name
U+36F6.0	fUTF8	㛶
U+36F6.0	fCNS11643	5-374E
U+36F6.0	fR/S	 38.9
U+36F6.0	fTotalStrokes	 12
U+36F6.0	fCangjie	VUMF
U+36F6.0	fFourCorner	42480
U+36F6.0	fCantonese	taan3
U+36F6.0	fMandarin	tan4; chou2; tan2
U+36F6.0	fEnglish	do not care about appearance and deportment
U+36F7.0	fUTF8	㛷
U+36F7.0	fCNS11643	3-3A6E
U+36F7.0	fR/S	 38.9
U+36F7.0	fTotalStrokes	 12
U+36F7.0	fCangjie	HUV
U+36F7.0	fFourCorner	23400
U+36F7.0	fCantonese	gai1
U+36F7.0	fMandarin	ji1
U+36F7.0	fEnglish	used in girl's name
U+36F8.0	fUTF8	㛸
U+36F8.0	fCNS11643	5-3746
U+36F8.0	fGB	5-402D
U+36F8.0	fR/S	 38.9
U+36F8.0	fTotalStrokes	 12
U+36F8.0	fCangjie	VGRT
U+36F8.0	fFourCorner	44410
U+36F8.0	fCantonese	syu6
U+36F8.0	fMandarin	shu4
U+36F8.0	fEnglish	(corrupted form) used in girl's name
U+36F9.0	fUTF8	㛹
U+36F9.0	fCNS11643	4-367E
U+36F9.0	fGB	3-3D37
U+36F9.0	fR/S	 38.9
U+36F9.0	fTotalStrokes	 12
U+36F9.0	fCangjie	VOMK
U+36F9.0	fFourCorner	41440
U+36F9.0	fCantonese	pin4
U+36F9.0	fMandarin	pian2
U+36F9.0	fEnglish	pretty; exquisite; fine
U+36FA.0	fUTF8	㛺
U+36FA.0	fCNS11643	3-3A6F
U+36FA.0	fGB	5-403A
U+36FA.0	fR/S	 38.9
U+36FA.0	fTotalStrokes	 12
U+36FA.0	fCangjie	VYTA
U+36FA.0	fFourCorner	40460
U+36FA.0	fHakka	  am1
U+36FA.0	fCantonese	jam1; aam1
U+36FA.0	fMandarin	an1
U+36FA.0	fEnglish	[1] an impure woman [2] dirty
U+36FB.0	fUTF8	㛻
U+36FB.0	fCNS11643	3-3A5F
U+36FB.0	fR/S	 38.9
U+36FB.0	fTotalStrokes	 12
U+36FB.0	fCangjie	VKGG
U+36FB.0	fFourCorner	44410
U+36FB.0	fCantonese	waai1; kwaai2
U+36FB.0	fMandarin	kuai1; kui3
U+36FB.0	fEnglish	womanly; feminine
U+36FC.0	fUTF8	㛼
U+36FC.0	fCNS11643	4-367D
U+36FC.0	fGB	3-3D36
U+36FC.0	fR/S	 38.9
U+36FC.0	fTotalStrokes	 12
U+36FC.0	fCangjie	VHJX
U+36FC.0	fFourCorner	42470
U+36FC.0	fCantonese	caap3; cap3; coi3
U+36FC.0	fMandarin	qia4; sha4
U+36FC.0	fEnglish	[1] interrupt; put in a word [2] lacking in courage; cowardly [3] nervous; socially timid [4] womanly; feminine
U+36FD.0	fUTF8	㛽
U+36FD.0	fCNS11643	3-3A70
U+36FD.0	fR/S	 38.9
U+36FD.0	fTotalStrokes	 12
U+36FD.0	fCangjie	VJFD
U+36FD.0	fFourCorner	43490
U+36FD.0	fCantonese	saan1; san1
U+36FD.0	fMandarin	shan1
U+36FD.0	fEnglish	ancient form of &#x59D7;
U+36FE.0	fUTF8	㛾
U+36FE.0	fCNS11643	3-3A5D
U+36FE.0	fR/S	 38.9
U+36FE.0	fTotalStrokes	 12
U+36FE.0	fCangjie	VIHR
U+36FE.0	fFourCorner	43450
U+36FE.0	fCantonese	haam4
U+36FE.0	fMandarin	xian2
U+36FE.0	fEnglish	not clean; impure (said of a woman)
U+36FF.0	fUTF8	㛿
U+36FF.0	fGB	S-232E
U+36FF.0	fR/S	 38.9
U+36FF.0	fTotalStrokes	 12
U+36FF.0	fCangjie	VTJB
U+36FF.0	fCantonese	daai3; zaai6; zat3
U+36FF.0	fMandarin	zhi4
U+36FF.0	fEnglish	[1] cunning; wicked [2] spoiled; coddled
U+3700.0	fUTF8	㜀
U+3700.0	fGB	S-232F
U+3700.0	fR/S	 38.9
U+3700.0	fTotalStrokes	 12
U+3700.0	fCangjie	VDBU
U+3700.0	fCantonese	soeng1
U+3701.0	fUTF8	㜁
U+3701.0	fCNS11643	F-3A3C
U+3701.0	fGB	S-232D
U+3701.0	fR/S	 38.9
U+3701.0	fTotalStrokes	 12
U+3701.0	fCangjie	VDAM
U+3701.0	fFourCorner	4441
U+3701.0	fCantonese	caa4
U+3702.0	fUTF8	㜂
U+3702.0	fCNS11643	3-3A6D
U+3702.0	fR/S	 38.9
U+3702.0	fTotalStrokes	 12
U+3702.0	fCangjie	GIV
U+3702.0	fFourCorner	44400
U+3702.0	fCantonese	fung1
U+3702.0	fMandarin	feng1
U+3702.0	fEnglish	used in girl's name
U+3703.0	fUTF8	㜃
U+3703.0	fCNS11643	3-3A69
U+3703.0	fR/S	 38.9
U+3703.0	fTotalStrokes	 12
U+3703.0	fCangjie	VOYR
U+3703.0	fFourCorner	40460
U+3703.0	fCantonese	lyun2; lyut6
U+3703.0	fMandarin	lian4; liao2; mang2
U+3703.0	fEnglish	[1] from; by; whence [2] undertake; manage [3] follow
U+3704.0	fUTF8	㜄
U+3704.0	fCNS11643	3-3A68
U+3704.0	fR/S	 38.9
U+3704.0	fTotalStrokes	 12
U+3704.0	fCangjie	VHNI
U+3704.0	fFourCorner	47410
U+3704.0	fCantonese	cam4; caam4; caan2; saan6; zaan2
U+3704.0	fMandarin	xin2; xun2
U+3704.0	fEnglish	a family name (refer to &#x5343;&#x5BB6;&#x59D3;)
U+3705.0	fUTF8	㜅
U+3705.0	fCNS11643	4-3C5F
U+3705.0	fGB	3-3D47
U+3705.0	fR/S	 38.10
U+3705.0	fTotalStrokes	 13
U+3705.0	fCangjie	VYIW
U+3705.0	fFourCorner	40460
U+3705.0	fCantonese	huk1; cuk1
U+3705.0	fMandarin	xu4
U+3705.0	fEnglish	[1] fawn on; flatter [2] love [3] jealous; envy; jealousy
U+3706.0	fUTF8	㜆
U+3706.0	fCNS11643	3-4062
U+3706.0	fR/S	 38.10
U+3706.0	fTotalStrokes	 13
U+3706.0	fCangjie	VYFD
U+3706.0	fFourCorner	49430
U+3706.0	fCantonese	mai6; maai4
U+3706.0	fMandarin	mi4
U+3706.0	fEnglish	commonly known as mother in Wu
U+3707.0	fUTF8	㜇
U+3707.0	fCNS11643	4-3C61
U+3707.0	fGB	3-3D43
U+3707.0	fR/S	 38.10
U+3707.0	fTotalStrokes	 13
U+3707.0	fCangjie	VGGP
U+3707.0	fFourCorner	44430
U+3707.0	fCantonese	wai3; waai6
U+3707.0	fMandarin	wei4; hui4; ye4
U+3707.0	fEnglish	[1] unhappy [2] girlish
U+3708.0	fUTF8	㜈
U+3708.0	fCNS11643	3-406E
U+3708.0	fR/S	 38.10; 38.11
U+3708.0	fTotalStrokes	 13; 14
U+3708.0	fCangjie	NSV
U+3708.0	fFourCorner	18400
U+3708.0	fCantonese	muk6
U+3708.0	fMandarin	mu4
U+3708.0	fEnglish	a beautiful face (of a woman); pretty
U+3709.0	fUTF8	㜉
U+3709.0	fCNS11643	3-406D
U+3709.0	fR/S	 38.10
U+3709.0	fTotalStrokes	 13
U+3709.0	fCangjie	VVVU
U+3709.0	fFourCorner	42410
U+3709.0	fCantonese	jung1; zung1; zung2
U+3709.0	fMandarin	yong1
U+3709.0	fEnglish	[1] womanly [2] used in girl's name
U+370A.0	fUTF8	㜊
U+370A.0	fCNS11643	3-4068
U+370A.0	fR/S	 38.10
U+370A.0	fTotalStrokes	 13
U+370A.0	fCangjie	VSTV
U+370A.0	fFourCorner	47430
U+370A.0	fCantonese	zin2; nan5
U+370A.0	fMandarin	zhan3; pang4
U+370A.0	fEnglish	exquisite; fine
U+370B.0	fUTF8	㜋
U+370B.0	fCNS11643	3-4063
U+370B.0	fR/S	 38.10
U+370B.0	fTotalStrokes	 13
U+370B.0	fCangjie	VTCT
U+370B.0	fFourCorner	48410
U+370B.0	fCantonese	jik1; zi4; zik1
U+370B.0	fMandarin	yi4
U+370B.0	fEnglish	used in girl's name
U+370C.0	fUTF8	㜌
U+370C.0	fCNS11643	4-3C64
U+370C.0	fGB	3-4F2F
U+370C.0	fR/S	 38.10
U+370C.0	fTotalStrokes	 13
U+370C.0	fCangjie	GVHNE
U+370C.0	fFourCorner	47440
U+370C.0	fCantonese	nau5; lau5 (also); gaau3
U+370C.0	fMandarin	nou3; gou4
U+370C.0	fEnglish	[1] a different name for breasts [2] breast-feed; feed a baby with milk [3] stingy; niggardly; miserly
U+370D.0	fUTF8	㜍
U+370D.0	fCNS11643	3-4065
U+370D.0	fGB	5-4052
U+370D.0	fR/S	 38.10
U+370D.0	fTotalStrokes	 13
U+370D.0	fCangjie	VILR
U+370D.0	fFourCorner	40460
U+370D.0	fCantonese	tong4
U+370D.0	fMandarin	tang2
U+370D.0	fEnglish	used in girl's name
U+370E.0	fUTF8	㜎
U+370E.0	fCNS11643	4-3C63
U+370E.0	fGB	5-4050
U+370E.0	fR/S	 38.10
U+370E.0	fTotalStrokes	 13
U+370E.0	fCangjie	VBVK
U+370E.0	fFourCorner	42430
U+370E.0	fHakka	 hi1; hi2; he1
U+370E.0	fCantonese	hai4; haai4; haai6
U+370E.0	fMandarin	xi2; qi2; xi4
U+370E.0	fEnglish	 waiting-maid; waiter
U+370F.0	fUTF8	㜏
U+370F.0	fCNS11643	3-4070
U+370F.0	fGB	5-3F7A
U+370F.0	fR/S	 38.10
U+370F.0	fTotalStrokes	 13
U+370F.0	fCangjie	VRBC
U+370F.0	fFourCorner	46480
U+370F.0	fHakka	 jun2
U+370F.0	fCantonese	wan4
U+370F.0	fMandarin	yun2
U+370F.0	fEnglish	same as &#x5998;
U+3710.0	fUTF8	㜐
U+3710.0	fCNS11643	3-406B
U+3710.0	fR/S	 38.10
U+3710.0	fTotalStrokes	 13
U+3710.0	fCangjie	VUMT
U+3710.0	fFourCorner	42410
U+3710.0	fCantonese	syu6
U+3710.0	fMandarin	shu4
U+3710.0	fEnglish	used in girl's name
U+3711.0	fUTF8	㜑
U+3711.0	fCNS11643	3-406F
U+3711.0	fGB	5-4053
U+3711.0	fR/S	 38.10
U+3711.0	fTotalStrokes	 13
U+3711.0	fCangjie	EBV
U+3711.0	fFourCorner	33400
U+3711.0	fHakka	 po2
U+3711.0	fCantonese	po4; pak3
U+3711.0	fMandarin	po2; fu2
U+3711.0	fEnglish	same as &#x5A46;
U+3712.0	fUTF8	㜒
U+3712.0	fCNS11643	3-406C
U+3712.0	fR/S	 38.10
U+3712.0	fTotalStrokes	 13
U+3712.0	fCangjie	VHXT
U+3712.0	fFourCorner	47440
U+3712.0	fCantonese	ngai6; ai6 (also); ngaai6; zi5
U+3712.0	fMandarin	yi4
U+3712.0	fEnglish	[1] maternal grandmother [2] midwife [3] old woman
U+3713.0	fUTF8	㜓
U+3713.0	fCNS11643	3-4069
U+3713.0	fR/S	 38.10
U+3713.0	fTotalStrokes	 13
U+3713.0	fCangjie	VTOR
U+3713.0	fFourCorner	44460
U+3713.0	fCantonese	daap3
U+3713.0	fMandarin	da4; da2
U+3713.0	fEnglish	womanly
U+3714.0	fUTF8	㜔
U+3714.0	fGB	S-232C
U+3714.0	fR/S	 38.10
U+3714.0	fTotalStrokes	 13
U+3714.0	fCangjie	VYTU
U+3714.0	fCantonese	ging2
U+3715.0	fUTF8	㜕
U+3715.0	fCNS11643	3-464B
U+3715.0	fR/S	 38.11
U+3715.0	fTotalStrokes	 14
U+3715.0	fCangjie	VYJJ
U+3715.0	fFourCorner	45430
U+3715.0	fCantonese	lin4
U+3715.0	fMandarin	lian2
U+3715.0	fEnglish	same as &#x9023;
U+3716.0	fUTF8	㜖
U+3716.0	fCNS11643	4-4267
U+3716.0	fGB	5-4057
U+3716.0	fR/S	 38.11
U+3716.0	fTotalStrokes	 14
U+3716.0	fCangjie	VTWA
U+3716.0	fFourCorner	45460
U+3716.0	fCantonese	cou4
U+3716.0	fMandarin	cao2
U+3716.0	fEnglish	exquisite; fine
U+3717.0	fUTF8	㜗
U+3717.0	fCNS11643	4-426C
U+3717.0	fGB	3-3D2A
U+3717.0	fR/S	 38.11
U+3717.0	fTotalStrokes	 14
U+3717.0	fCangjie	VIIH
U+3717.0	fFourCorner	43420
U+3717.0	fCantonese	caam1; cam2
U+3717.0	fMandarin	can3; chu2; xuan4
U+3717.0	fEnglish	[1] covetousness; greed; cupidity; avarice [2] licentious; obscene; seduce
U+3718.0	fUTF8	㜘
U+3718.0	fCNS11643	4-4268
U+3718.0	fGB	3-3D4D
U+3718.0	fR/S	 38.11
U+3718.0	fTotalStrokes	 14
U+3718.0	fCangjie	VYPM
U+3718.0	fFourCorner	41410
U+3718.0	fCantonese	ze1; geoi6
U+3718.0	fMandarin	ju4
U+3718.0	fEnglish	[1] delicated; lovely; beautiful [2] spoiled [3] jealous; envy; jealousy [4] used in girl's name
U+3719.0	fUTF8	㜙
U+3719.0	fCNS11643	4-4266
U+3719.0	fR/S	 38.11
U+3719.0	fTotalStrokes	 14
U+3719.0	fCangjie	VIXP
U+3719.0	fFourCorner	40410
U+3719.0	fCantonese	luk6
U+3719.0	fMandarin	lu4
U+3719.0	fEnglish	used in girl's name
U+371A.0	fUTF8	㜚
U+371A.0	fCNS11643	3-4649
U+371A.0	fR/S	 38.11
U+371A.0	fTotalStrokes	 14
U+371A.0	fCangjie	VJOA
U+371A.0	fFourCorner	43460
U+371A.0	fCantonese	suk1
U+371A.0	fMandarin	suo1; su4
U+371A.0	fEnglish	a local government post in ancient times
U+371B.0	fUTF8	㜛
U+371B.0	fCNS11643	3-4652
U+371B.0	fR/S	 38.11
U+371B.0	fTotalStrokes	 14
U+371B.0	fCangjie	VJJO
U+371B.0	fFourCorner	47480
U+371B.0	fHakka	 nun5
U+371B.0	fCantonese	jyun5; zyun5
U+371B.0	fMandarin	ruan3; nen4
U+371B.0	fEnglish	non-classical form of &#x5A86;
U+371C.0	fUTF8	㜜
U+371C.0	fCNS11643	3-4651
U+371C.0	fR/S	 38.11
U+371C.0	fTotalStrokes	 14
U+371C.0	fCangjie	VGSK
U+371C.0	fFourCorner	48440
U+371C.0	fHakka	 ngau5
U+371C.0	fCantonese	ngou6; ou6 (also)
U+371C.0	fMandarin	ao4
U+371C.0	fEnglish	same as &#x5AEF;
U+371D.0	fUTF8	㜝
U+371D.0	fCNS11643	4-426B
U+371D.0	fGB	3-3D50
U+371D.0	fR/S	 38.11
U+371D.0	fTotalStrokes	 14
U+371D.0	fCangjie	VOIW
U+371D.0	fFourCorner	48460
U+371D.0	fHakka	 ham3; ngam3; am5
U+371D.0	fCantonese	ngam5; am5 (also); an2
U+371D.0	fMandarin	an3; an4
U+371D.0	fEnglish	[1] restrain one's anger [2] large cheeks [3] bad temper [4] a woman who is partial to one
U+371E.0	fUTF8	㜞
U+371E.0	fCNS11643	4-426D
U+371E.0	fGB	3-3C7A
U+371E.0	fR/S	 38.11
U+371E.0	fTotalStrokes	 14
U+371E.0	fCangjie	JLV
U+371E.0	fFourCorner	52400
U+371E.0	fCantonese	cim3; cam4
U+371E.0	fMandarin	can2; qian4
U+371E.0	fEnglish	[1] beautiful; pretty [2] used in girl's name
U+371F.0	fUTF8	㜟
U+371F.0	fGB	7-2338
U+371F.0	fR/S	 38.11
U+371F.0	fTotalStrokes	 14
U+371F.0	fCangjie	VNJK
U+371F.0	fCantonese	haam3
U+3720.0	fUTF8	㜠
U+3720.0	fCNS11643	3-464D
U+3720.0	fR/S	 38.11
U+3720.0	fTotalStrokes	 14
U+3720.0	fCangjie	VUOG
U+3720.0	fFourCorner	42410
U+3720.0	fCantonese	ceoi1; cyun2
U+3720.0	fMandarin	cui1
U+3720.0	fEnglish	used in girl's name
U+3721.0	fUTF8	㜡
U+3721.0	fCNS11643	3-464E
U+3721.0	fR/S	 38.11
U+3721.0	fTotalStrokes	 14
U+3721.0	fCangjie	VHOO
U+3721.0	fFourCorner	48480
U+3721.0	fCantonese	cung1; sung1
U+3721.0	fMandarin	cong1
U+3721.0	fEnglish	used in girl's name
U+3722.0	fUTF8	㜢
U+3722.0	fCNS11643	4-4269
U+3722.0	fGB	3-3D3D
U+3722.0	fR/S	 38.11
U+3722.0	fTotalStrokes	 14
U+3722.0	fCangjie	VLLV
U+3722.0	fFourCorner	45440
U+3722.0	fCantonese	leoi5
U+3722.0	fEnglish	a bad term by which one addresses a woman
U+3723.0	fUTF8	㜣
U+3723.0	fCNS11643	4-493C
U+3723.0	fGB	3-3D5B
U+3723.0	fR/S	 38.12
U+3723.0	fTotalStrokes	 15
U+3723.0	fCangjie	VBKF
U+3723.0	fFourCorner	43430
U+3723.0	fHakka	 jen2; ren2
U+3723.0	fCantonese	jin4; zin4
U+3723.0	fMandarin	ran2
U+3723.0	fEnglish	a Chinese family name
U+3724.0	fUTF8	㜤
U+3724.0	fCNS11643	4-493A
U+3724.0	fGB	3-3D54
U+3724.0	fR/S	 38.12
U+3724.0	fTotalStrokes	 15
U+3724.0	fCangjie	VMWJ
U+3724.0	fFourCorner	41440
U+3724.0	fCantonese	nim5; lim5 (also); saam2
U+3724.0	fMandarin	nian4; shen3
U+3724.0	fEnglish	[1] fooling around [2] weak [3] slender (said of a woman's figure); [4] thin and long [5] used in girl's name
U+3725.0	fUTF8	㜥
U+3725.0	fCNS11643	3-4B59
U+3725.0	fR/S	 38.12
U+3725.0	fTotalStrokes	 15
U+3725.0	fCangjie	VWLC
U+3725.0	fFourCorner	46480
U+3725.0	fCantonese	maai4
U+3725.0	fMandarin	mai2; mo2
U+3725.0	fEnglish	crafty; artiful
U+3726.0	fUTF8	㜦
U+3726.0	fCNS11643	3-4B57
U+3726.0	fR/S	 38.12
U+3726.0	fTotalStrokes	 15
U+3726.0	fCangjie	VSMI
U+3726.0	fFourCorner	47440
U+3726.0	fCantonese	cam4
U+3726.0	fMandarin	xun2; han4; nie4; si4; xie2; xin2
U+3726.0	fEnglish	used in girl's name
U+3727.0	fUTF8	㜧
U+3727.0	fCNS11643	4-4940
U+3727.0	fGB	3-3D55
U+3727.0	fR/S	 38.12
U+3727.0	fTotalStrokes	 15
U+3727.0	fCangjie	VMTO
U+3727.0	fFourCorner	41480
U+3727.0	fCantonese	jyut3; kyut3; waat1
U+3727.0	fMandarin	yue1; yue4
U+3727.0	fEnglish	same as &#x21855;
U+3728.0	fUTF8	㜨
U+3728.0	fCNS11643	3-4B55
U+3728.0	fR/S	 38.12
U+3728.0	fTotalStrokes	 15
U+3728.0	fCangjie	VSEQ
U+3728.0	fFourCorner	47450
U+3728.0	fCantonese	naai4; laai4 (also)
U+3728.0	fMandarin	er2; nai2
U+3728.0	fEnglish	beautiful; pretty
U+3729.0	fUTF8	㜩
U+3729.0	fCNS11643	3-5052
U+3729.0	fR/S	 38.13
U+3729.0	fTotalStrokes	 16
U+3729.0	fCangjie	VHBK
U+3729.0	fFourCorner	47430
U+3729.0	fCantonese	ou3; ngou3 (also)
U+3729.0	fMandarin	ao4; bie2
U+3729.0	fEnglish	jealous; envy; jealously
U+372A.0	fUTF8	㜪
U+372A.0	fCNS11643	3-5055
U+372A.0	fGB	5-4068
U+372A.0	fR/S	 38.13
U+372A.0	fTotalStrokes	 16
U+372A.0	fCangjie	YLV
U+372A.0	fFourCorner	02400
U+372A.0	fHakka	 sin1
U+372A.0	fCantonese	san1; saan1; sin2
U+372A.0	fMandarin	xin1; xian3
U+372A.0	fEnglish	[1] a Chinese family name [2] same as &#x218CE;
U+372B.0	fUTF8	㜫
U+372B.0	fCNS11643	4-5021
U+372B.0	fGB	S-2334
U+372B.0	fR/S	 38.13
U+372B.0	fTotalStrokes	 16
U+372B.0	fCangjie	VHOK
U+372B.0	fFourCorner	48400
U+372B.0	fCantonese	mei5
U+372B.0	fMandarin	mei3; ma4; mei2
U+372B.0	fEnglish	same as &#x5A84;
U+372C.0	fUTF8	㜬
U+372C.0	fCNS11643	F-542A
U+372C.0	fGB	S-2335
U+372C.0	fR/S	 38.13
U+372C.0	fTotalStrokes	 16
U+372C.0	fCangjie	VNCR
U+372C.0	fFourCorner	4746
U+372C.0	fCantonese	daam1
U+372D.0	fUTF8	㜭
U+372D.0	fR/S	 38.13
U+372D.0	fTotalStrokes	 16
U+372D.0	fCangjie	VFBW
U+372D.0	fFourCorner	4946
U+372D.0	fCantonese	dong1
U+372E.0	fUTF8	㜮
U+372E.0	fCNS11643	4-5622
U+372E.0	fGB	3-3D45
U+372E.0	fR/S	 38.14
U+372E.0	fTotalStrokes	 17; 16
U+372E.0	fCangjie	VSIT
U+372E.0	fFourCorner	48410
U+372E.0	fHakka	 lam1; lam3
U+372E.0	fCantonese	laam6
U+372E.0	fMandarin	lan4
U+372E.0	fEnglish	[1] go beyond normal limit; excessive; out of control [2] covet; greedy [3] to be impolite [4] used in girl's name
U+372F.0	fUTF8	㜯
U+372F.0	fCNS11643	4-5624
U+372F.0	fGB	5-406D
U+372F.0	fR/S	 38.14
U+372F.0	fTotalStrokes	 17; 16
U+372F.0	fCangjie	VSUF
U+372F.0	fFourCorner	47430
U+372F.0	fCantonese	hei1
U+372F.0	fMandarin	xi1
U+372F.0	fEnglish	[1] a word to designate woman [2] a word of revile used to call a woman
U+3730.0	fUTF8	㜰
U+3730.0	fCNS11643	3-575F
U+3730.0	fGB	5-3F58
U+3730.0	fR/S	 38.15
U+3730.0	fTotalStrokes	 18
U+3730.0	fCangjie	VVID
U+3730.0	fFourCorner	42490
U+3730.0	fCantonese	joek6; zoek6
U+3730.0	fMandarin	yue4
U+3730.0	fEnglish	exquisite; fine
U+3731.0	fUTF8	㜱
U+3731.0	fCNS11643	3-5760
U+3731.0	fR/S	 38.15
U+3731.0	fTotalStrokes	 18
U+3731.0	fCangjie	VHLC
U+3731.0	fFourCorner	42480
U+3731.0	fCantonese	zat1
U+3731.0	fMandarin	zhi2; dong4; zhi4
U+3731.0	fEnglish	[1] used in girl's name [2] soft and tender
U+3732.0	fUTF8	㜲
U+3732.0	fCNS11643	4-5F54
U+3732.0	fGB	3-3D67
U+3732.0	fR/S	 38.16
U+3732.0	fTotalStrokes	 19
U+3732.0	fCangjie	VYNV
U+3732.0	fFourCorner	40410
U+3732.0	fCantonese	jing4; zing4
U+3732.0	fMandarin	ying2; weng3
U+3732.0	fEnglish	non-classical form of &#x5B34;
U+3733.0	fUTF8	㜳
U+3733.0	fCNS11643	5-675E
U+3733.0	fR/S	 38.16
U+3733.0	fTotalStrokes	 19
U+3733.0	fCangjie	VYWV
U+3733.0	fFourCorner	40430
U+3733.0	fCantonese	waai4; gwaai1; wai4
U+3733.0	fMandarin	huai2
U+3733.0	fEnglish	peaceful; joyful
U+3734.0	fUTF8	㜴
U+3734.0	fCNS11643	3-5A30
U+3734.0	fR/S	 38.16
U+3734.0	fTotalStrokes	 19
U+3734.0	fCangjie	VTWU
U+3734.0	fFourCorner	44460
U+3734.0	fCantonese	mang6; mung6
U+3734.0	fMandarin	meng4
U+3734.0	fEnglish	non-classical form of &#x77A2;
U+3735.0	fUTF8	㜵
U+3735.0	fCNS11643	3-5A32
U+3735.0	fR/S	 38.16
U+3735.0	fTotalStrokes	 19
U+3735.0	fCangjie	VYSV
U+3735.0	fFourCorner	40430
U+3735.0	fHakka	 ngiau3
U+3735.0	fCantonese	niu5; liu5 (also)
U+3735.0	fMandarin	niao3
U+3735.0	fEnglish	delicate; slender
U+3736.0	fUTF8	㜶
U+3736.0	fCNS11643	3-5C37
U+3736.0	fR/S	 38.17
U+3736.0	fTotalStrokes	 20
U+3736.0	fCangjie	VNRI
U+3736.0	fFourCorner	47410
U+3736.0	fCantonese	min5; man5; man6
U+3736.0	fMandarin	fan4; wan3
U+3736.0	fEnglish	a (married) couple
U+3737.0	fUTF8	㜷
U+3737.0	fCNS11643	4-6335
U+3737.0	fGB	5-402A
U+3737.0	fR/S	 38.17
U+3737.0	fTotalStrokes	 20
U+3737.0	fCangjie	NBV
U+3737.0	fFourCorner	11400
U+3737.0	fHakka	 ni2
U+3737.0	fCantonese	naai5; laai5 (also); laau4; nai5
U+3737.0	fMandarin	mi2; nai3; xian3
U+3737.0	fEnglish	[1] milk [2] the breasts of a woman, [3] suckle [4] word of respect for women [5] mother (for Qi's people) [6] used in girl's name
U+3738.0	fUTF8	㜸
U+3738.0	fCNS11643	3-5C38
U+3738.0	fGB	5-4071
U+3738.0	fR/S	 38.17; 38.16; 140.16
U+3738.0	fTotalStrokes	 20
U+3738.0	fCangjie	TJV
U+3738.0	fFourCorner	44400
U+3738.0	fHakka	 nget7; siet7
U+3738.0	fCantonese	jit6; jip6 (also); sit3; zip6; zit6; zyu1
U+3738.0	fMandarin	xie4; nie4
U+3738.0	fEnglish	non-classical form of &#x5B7D;
U+3739.0	fUTF8	㜹
U+3739.0	fCNS11643	4-6637
U+3739.0	fGB	3-3D6A
U+3739.0	fR/S	 38.18
U+3739.0	fTotalStrokes	 21
U+3739.0	fCangjie	VBUG
U+3739.0	fFourCorner	46410
U+3739.0	fCantonese	keoi4
U+3739.0	fMandarin	qu2
U+3739.0	fEnglish	[1] name of a folk song in ancient times [2] used in girl's name
U+373A.0	fUTF8	㜺
U+373A.0	fCNS11643	4-6865
U+373A.0	fGB	3-3D65
U+373A.0	fR/S	 38.19
U+373A.0	fTotalStrokes	 22; 21
U+373A.0	fCangjie	VHUC
U+373A.0	fFourCorner	44480
U+373A.0	fHakka	 zan5
U+373A.0	fCantonese	zaan2
U+373A.0	fMandarin	zan4
U+373A.0	fEnglish	beautiful (woman)
U+373B.0	fUTF8	㜻
U+373B.0	fCNS11643	3-6162
U+373B.0	fR/S	 38.23
U+373B.0	fTotalStrokes	 26
U+373B.0	fCangjie	VVFP
U+373B.0	fFourCorner	42430
U+373B.0	fHakka	 lan2
U+373B.0	fCantonese	lyun2; lyun5
U+373B.0	fMandarin	luan3; lian4; luan4
U+373B.0	fEnglish	same as &#x5B4C;
U+373C.0	fUTF8	㜼
U+373C.0	fCNS11643	3-6163
U+373C.0	fGB	5-406A
U+373C.0	fR/S	 38.23
U+373C.0	fTotalStrokes	 26
U+373C.0	fCangjie	VWWM
U+373C.0	fFourCorner	46410
U+373C.0	fHakka	 chit8
U+373C.0	fCantonese	zat6; dip6; zaat6; zit1
U+373C.0	fMandarin	zhi2; die2; zhi4
U+373C.0	fEnglish	same as &#x59EA;
U+373D.0	fUTF8	㜽
U+373D.0	fCNS11643	3-234E
U+373D.0	fGB	5-415F
U+373D.0	fR/S	 39.3
U+373D.0	fTotalStrokes	 6
U+373D.0	fCangjie	VVND
U+373D.0	fFourCorner	22400
U+373D.0	fCantonese	zi2; daai2
U+373D.0	fMandarin	zi3
U+373D.0	fEnglish	ancient form of &#x5B50;
U+373E.0	fUTF8	㜾
U+373E.0	fCNS11643	6-2678
U+373E.0	fR/S	 39.4
U+373E.0	fTotalStrokes	 7
U+373E.0	fCangjie	NDOLL
U+373E.0	fFourCorner	18420
U+373E.0	fHakka	 hai2
U+373E.0	fCantonese	haai4
U+373E.0	fMandarin	hai2; xie3
U+373E.0	fEnglish	same as &#x5B69;
U+373F.0	fUTF8	㜿
U+373F.0	fCNS11643	4-236D
U+373F.0	fGB	3-3D6F
U+373F.0	fR/S	 39.4
U+373F.0	fTotalStrokes	 7
U+373F.0	fCangjie	NDNIN
U+373F.0	fFourCorner	17420
U+373F.0	fCantonese	zeoi6; zok6
U+373F.0	fMandarin	xu4
U+373F.0	fEnglish	[1] name of a fish in legend [2] spawn; roe
U+3740.0	fUTF8	㝀
U+3740.0	fCNS11643	3-2849
U+3740.0	fGB	5-545F
U+3740.0	fR/S	 39.5
U+3740.0	fTotalStrokes	 8
U+3740.0	fCangjie	WYND
U+3740.0	fFourCorner	77740
U+3740.0	fHakka	 hau3; hau5
U+3740.0	fCantonese	hou2; hou3
U+3740.0	fMandarin	hao3; hao4
U+3740.0	fEnglish	same as &#x597D;
U+3741.0	fUTF8	㝁
U+3741.0	fCNS11643	4-2870
U+3741.0	fGB	5-4124
U+3741.0	fR/S	 39.6
U+3741.0	fTotalStrokes	 9
U+3741.0	fCangjie	PAND
U+3741.0	fFourCorner	27400
U+3741.0	fCantonese	hyun1; king4
U+3741.0	fMandarin	xuan1; xun2
U+3741.0	fEnglish	same as &#x5B1B;
U+3742.0	fUTF8	㝂
U+3742.0	fCNS11643	4-2C41
U+3742.0	fGB	3-3D70
U+3742.0	fR/S	 39.7
U+3742.0	fTotalStrokes	 10
U+3742.0	fCangjie	NDQHL
U+3742.0	fFourCorner	12420
U+3742.0	fCantonese	zai3; caam4; dap3; gei1; zaai3; zit3
U+3742.0	fMandarin	zhe4; zhi4
U+3742.0	fEnglish	[1] locust [2] larva of Chilo simplex; a kind of moth
U+3743.0	fUTF8	㝃
U+3743.0	fCNS11643	3-3054
U+3743.0	fGB	5-4127
U+3743.0	fR/S	 39.7
U+3743.0	fTotalStrokes	 10
U+3743.0	fCangjie	NDNAU
U+3743.0	fFourCorner	17410
U+3743.0	fHakka	 mien3; men3
U+3743.0	fCantonese	min5; hei1
U+3743.0	fMandarin	wan3; fan4; mian3
U+3743.0	fEnglish	 bear a son
U+3744.0	fUTF8	㝄
U+3744.0	fCNS11643	3-3A7D
U+3744.0	fR/S	 39.9
U+3744.0	fTotalStrokes	 12
U+3744.0	fCangjie	YDPU
U+3744.0	fFourCorner	05410
U+3744.0	fHakka	 shun2
U+3744.0	fCantonese	seon4; zeon2; zeot1
U+3744.0	fMandarin	chun2; dun4; qi4; jun1; run4; quan2; tun2; zhun3; zhun4
U+3744.0	fEnglish	ancient form of &#x7D14;
U+3745.0	fUTF8	㝅
U+3745.0	fCNS11643	4-3C67
U+3745.0	fGB	3-4F30
U+3745.0	fR/S	 39.10
U+3745.0	fTotalStrokes	 13
U+3745.0	fCangjie	GDHNE
U+3745.0	fFourCorner	47440
U+3745.0	fHakka	 kieu5; kieu1; geu5; geu1
U+3745.0	fCantonese	gau3; gaau3; zyu5
U+3745.0	fMandarin	gou4
U+3745.0	fEnglish	[1] milk [2] suckle
U+3746.0	fUTF8	㝆
U+3746.0	fR/S	 39.12
U+3746.0	fTotalStrokes	 15
U+3746.0	fCangjie	NDGRR
U+3746.0	fCantonese	hei2
U+3747.0	fUTF8	㝇
U+3747.0	fCNS11643	7-2D2D
U+3747.0	fR/S	 39.13
U+3747.0	fTotalStrokes	 16
U+3747.0	fCangjie	YDYRD
U+3747.0	fFourCorner	00440
U+3747.0	fHakka	 tun2; shun2
U+3747.0	fCantonese	seon4
U+3747.0	fMandarin	chun2; dun1
U+3747.0	fEnglish	same as &#x9D89;
U+3748.0	fUTF8	㝈
U+3748.0	fCNS11643	3-613D
U+3748.0	fR/S	 39.22
U+3748.0	fTotalStrokes	 25
U+3748.0	fCangjie	VFNDD
U+3748.0	fFourCorner	22440
U+3748.0	fHakka	 lien2; lan2
U+3748.0	fCantonese	lyun4
U+3748.0	fMandarin	xuan4; luan2; shuan4
U+3748.0	fEnglish	same as &#x5B7F;
U+3749.0	fUTF8	㝉
U+3749.0	fGB	3-3B36
U+3749.0	fR/S	 40.1
U+3749.0	fTotalStrokes	 4
U+3749.0	fCangjie	JM
U+3749.0	fCantonese	cyu5
U+3749.0	fMandarin	zhu4
U+3749.0	fEnglish	same as &#x5B81;
U+374A.0	fUTF8	㝊
U+374A.0	fCNS11643	3-223C
U+374A.0	fR/S	 40.2
U+374A.0	fTotalStrokes	 5
U+374A.0	fCangjie	JE
U+374A.0	fFourCorner	30400
U+374A.0	fCantonese	sau2
U+374A.0	fMandarin	shou3
U+374A.0	fEnglish	same as &#x5B88;
U+374B.0	fUTF8	㝋
U+374B.0	fR/S	 40.2
U+374B.0	fTotalStrokes	 5
U+374B.0	fCangjie	JNN
U+374B.0	fFourCorner	3020
U+374B.0	fCantonese	liu4; liu5
U+374B.0	fMandarin	liao2; liao3; liu2
U+374C.0	fUTF8	㝌
U+374C.0	fCNS11643	3-234F
U+374C.0	fGB	5-3D70
U+374C.0	fR/S	 40.3
U+374C.0	fTotalStrokes	 6
U+374C.0	fCangjie	JNO
U+374C.0	fFourCorner	30230
U+374C.0	fHakka	 giu5
U+374C.0	fCantonese	gau3
U+374C.0	fMandarin	jiu4; jie2; zhou4
U+374C.0	fEnglish	same as &#x759A;
U+374D.0	fUTF8	㝍
U+374D.0	fCNS11643	6-2679
U+374D.0	fR/S	 40.3
U+374D.0	fTotalStrokes	 7; 6
U+374D.0	fCangjie	JYSM
U+374D.0	fFourCorner	30400
U+374D.0	fHakka	 sia3
U+374D.0	fCantonese	se2; se3
U+374D.0	fMandarin	xie3; pang2
U+374D.0	fEnglish	simplified form of &#x5BEB;
U+374E.0	fUTF8	㝎
U+374E.0	fCNS11643	3-2559
U+374E.0	fR/S	 40.4; 40.3
U+374E.0	fTotalStrokes	 7; 6
U+374E.0	fCangjie	JINO
U+374E.0	fFourCorner	30307
U+374E.0	fHakka	 tin5
U+374E.0	fCantonese	ding6
U+374E.0	fMandarin	ding4
U+374E.0	fEnglish	same as &#x5B9A;
U+374F.0	fUTF8	㝏
U+374F.0	fCNS11643	4-2370
U+374F.0	fGB	5-3D75
U+374F.0	fR/S	 40.4
U+374F.0	fTotalStrokes	 7
U+374F.0	fCangjie	JOLL
U+374F.0	fFourCorner	30220
U+374F.0	fCantonese	gaai3
U+374F.0	fMandarin	jie4
U+374F.0	fEnglish	same as &#x4ECB;
U+3750.0	fUTF8	㝐
U+3750.0	fCNS11643	3-255B
U+3750.0	fR/S	 40.4
U+3750.0	fTotalStrokes	 7
U+3750.0	fCangjie	JCI
U+3750.0	fFourCorner	30730
U+3750.0	fHakka	 jung2
U+3750.0	fCantonese	jung4; zung2; zung4
U+3750.0	fMandarin	rong2
U+3750.0	fEnglish	ancient form of &#x5BB9;
U+3751.0	fUTF8	㝑
U+3751.0	fCNS11643	6-267A
U+3751.0	fR/S	 40.4
U+3751.0	fTotalStrokes	 7
U+3751.0	fCangjie	JYHS
U+3751.0	fFourCorner	30220
U+3751.0	fHakka	 pong2
U+3751.0	fCantonese	pong4; bong6
U+3751.0	fMandarin	bin1; pang2; mang2; bang1; beng1; jing1; pang1; peng2
U+3751.0	fEnglish	ancient form of &#x65C1;
U+3752.0	fUTF8	㝒
U+3752.0	fCNS11643	3-284F
U+3752.0	fR/S	 40.5
U+3752.0	fTotalStrokes	 8
U+3752.0	fCangjie	JJR
U+3752.0	fFourCorner	30600
U+3752.0	fHakka	 gi1
U+3752.0	fCantonese	geoi1
U+3752.0	fMandarin	ju1; ji1
U+3752.0	fEnglish	ancient form of &#x5C45;
U+3753.0	fUTF8	㝓
U+3753.0	fCNS11643	5-286E
U+3753.0	fR/S	 40.6
U+3753.0	fTotalStrokes	 9
U+3753.0	fCangjie	JOMR
U+3753.0	fFourCorner	30600
U+3753.0	fCantonese	hap1; gaap3; hau1; kaap6
U+3753.0	fMandarin	ke4; ge2
U+3753.0	fEnglish	[1] corresponding; equivalent [2] considerable; to a great extent [3] appropriate [4] same as &#x51F9;
U+3754.0	fUTF8	㝔
U+3754.0	fCNS11643	3-2C30
U+3754.0	fGB	5-3D7D
U+3754.0	fR/S	 40.6
U+3754.0	fTotalStrokes	 9
U+3754.0	fCangjie	JYCK
U+3754.0	fFourCorner	30400
U+3754.0	fCantonese	jiu2; jiu3 (also); ziu3
U+3754.0	fMandarin	yao3; yao4
U+3754.0	fEnglish	deep bottom; the southeast corner of a house
U+3755.0	fUTF8	㝕
U+3755.0	fCNS11643	3-2C31
U+3755.0	fR/S	 40.6
U+3755.0	fTotalStrokes	 9
U+3755.0	fCangjie	JPE
U+3755.0	fFourCorner	30400
U+3755.0	fHakka	 nen2
U+3755.0	fCantonese	ning4; ling4 (also); fai3
U+3755.0	fMandarin	ning2; ning4
U+3755.0	fEnglish	same as &#x5BE7;
U+3756.0	fUTF8	㝖
U+3756.0	fCNS11643	3-2C33
U+3756.0	fR/S	 40.6
U+3756.0	fTotalStrokes	 9
U+3756.0	fCangjie	JNIN
U+3756.0	fFourCorner	30200
U+3756.0	fHakka	 ni2
U+3756.0	fCantonese	ji4; zi4
U+3756.0	fMandarin	yi2; yin2
U+3756.0	fEnglish	same as &#x5B9C;
U+3757.0	fUTF8	㝗
U+3757.0	fCNS11643	4-2C42
U+3757.0	fGB	3-3B3E
U+3757.0	fR/S	 40.7
U+3757.0	fTotalStrokes	 10
U+3757.0	fCangjie	JIAV
U+3757.0	fFourCorner	30730
U+3757.0	fHakka	 long2;  long1
U+3757.0	fCantonese	long4; long6 (also); gong1
U+3757.0	fMandarin	lang2
U+3757.0	fEnglish	the framework of a house
U+3758.0	fUTF8	㝘
U+3758.0	fCNS11643	5-2C7C
U+3758.0	fR/S	 40.7
U+3758.0	fTotalStrokes	 10
U+3758.0	fCangjie	JYMR
U+3758.0	fFourCorner	30600
U+3758.0	fCantonese	jung4; zung4
U+3758.0	fMandarin	weng2; yong2
U+3758.0	fEnglish	[1] (non-classical form) [2] harmonoy; peace [3] pleasant; agreeable; delightful
U+3759.0	fUTF8	㝙
U+3759.0	fCNS11643	6-365D
U+3759.0	fR/S	 40.7
U+3759.0	fTotalStrokes	 10; 9
U+3759.0	fCangjie	JMGC
U+3759.0	fFourCorner	30801
U+3759.0	fHakka	 ji2; jin2
U+3759.0	fCantonese	jan4; zaan4
U+3759.0	fMandarin	yin2; yi2
U+3759.0	fEnglish	ancient form of &#x5BC5;
U+375A.0	fUTF8	㝚
U+375A.0	fR/S	 40.7
U+375A.0	fTotalStrokes	 10
U+375A.0	fCangjie	JNKM
U+375A.0	fFourCorner	3040
U+375A.0	fCantonese	jin4
U+375B.0	fUTF8	㝛
U+375B.0	fCNS11643	3-3542
U+375B.0	fR/S	 40.8
U+375B.0	fTotalStrokes	 11
U+375B.0	fCangjie	JOMW
U+375B.0	fFourCorner	30260
U+375B.0	fHakka	 siuk7
U+375B.0	fCantonese	suk1; sau3
U+375B.0	fMandarin	su4
U+375B.0	fEnglish	standard form of &#x5BBF;
U+375C.0	fUTF8	㝜
U+375C.0	fCNS11643	3-353D
U+375C.0	fR/S	 40.8
U+375C.0	fTotalStrokes	 11
U+375C.0	fCangjie	JTA
U+375C.0	fFourCorner	30600
U+375C.0	fCantonese	sik1
U+375C.0	fMandarin	xi2
U+375C.0	fEnglish	[1] night; dark; darkness [2] of death [3] mourn [4] lose; deprive of
U+375D.0	fUTF8	㝝
U+375D.0	fCNS11643	3-353E
U+375D.0	fR/S	 40.8
U+375D.0	fTotalStrokes	 11
U+375D.0	fCangjie	JDD
U+375D.0	fFourCorner	30990
U+375D.0	fHakka	 lim2
U+375D.0	fCantonese	lam4; lam6
U+375D.0	fMandarin	lin2
U+375D.0	fEnglish	[1] deep; profound [2] far [3] extremely
U+375E.0	fUTF8	㝞
U+375E.0	fCNS11643	4-312E
U+375E.0	fGB	3-3B3A
U+375E.0	fR/S	 40.8
U+375E.0	fTotalStrokes	 11
U+375E.0	fCangjie	JMLM
U+375E.0	fFourCorner	30100
U+375E.0	fHakka	  a3
U+375E.0	fCantonese	aa1; ngaa1 (also)
U+375E.0	fMandarin	ya1; ya4
U+375E.0	fEnglish	crooked
U+375F.0	fUTF8	㝟
U+375F.0	fCNS11643	6-3E5A
U+375F.0	fR/S	 40.8
U+375F.0	fTotalStrokes	 11
U+375F.0	fCangjie	JMBU
U+375F.0	fFourCorner	30210
U+375F.0	fCantonese	mou6; mui6
U+375F.0	fMandarin	mao4; mao2; wu3
U+375F.0	fEnglish	(corrupted form) in contravention of; offend (same as &#x5192;) incautious, risk, put forth
U+3760.0	fUTF8	㝠
U+3760.0	fCNS11643	3-3540
U+3760.0	fR/S	 40.8
U+3760.0	fTotalStrokes	 11
U+3760.0	fCangjie	JAYC
U+3760.0	fFourCorner	30800
U+3760.0	fHakka	 men2
U+3760.0	fCantonese	ming4; ming5 (also)
U+3760.0	fMandarin	ming2; meng3; mi4; mian2; ming4
U+3760.0	fEnglish	same as &#x51A5;
U+3761.0	fUTF8	㝡
U+3761.0	fR/S	 40.8
U+3761.0	fTotalStrokes	 11
U+3761.0	fCangjie	JSJE
U+3761.0	fFourCorner	3014
U+3761.0	fCantonese	zeoi3; zyu4
U+3761.0	fMandarin	zui4
U+3761.0	fEnglish	very
U+3762.0	fUTF8	㝢
U+3762.0	fCNS11643	3-3B21
U+3762.0	fGB	5-3E2E
U+3762.0	fR/S	 40.9
U+3762.0	fTotalStrokes	 12
U+3762.0	fCangjie	JHLB
U+3762.0	fFourCorner	30220
U+3762.0	fHakka	 ji3
U+3762.0	fCantonese	jyu5; zyu5
U+3762.0	fMandarin	yu3
U+3762.0	fEnglish	same as &#x5B87;
U+3763.0	fUTF8	㝣
U+3763.0	fCNS11643	4-3722
U+3763.0	fGB	3-3B41
U+3763.0	fR/S	 40.9
U+3763.0	fTotalStrokes	 12
U+3763.0	fCangjie	JQHK
U+3763.0	fFourCorner	30420
U+3763.0	fCantonese	ai3; ngai3 (also)
U+3763.0	fMandarin	yi4; ye4
U+3763.0	fEnglish	[1] still; calm; silent; quiet [2] peaceful; harmonious; serene
U+3764.0	fUTF8	㝤
U+3764.0	fCNS11643	4-3C6A
U+3764.0	fGB	5-3E31
U+3764.0	fR/S	 40.10
U+3764.0	fTotalStrokes	 13
U+3764.0	fCangjie	JTTB
U+3764.0	fFourCorner	30440
U+3764.0	fCantonese	gau3
U+3764.0	fMandarin	gou4
U+3764.0	fEnglish	[1] night [2] the dark part of a room
U+3765.0	fUTF8	㝥
U+3765.0	fCNS11643	4-3C6C
U+3765.0	fGB	3-3B45
U+3765.0	fR/S	 40.10
U+3765.0	fTotalStrokes	 13
U+3765.0	fCangjie	JVMD
U+3765.0	fFourCorner	30290
U+3765.0	fCantonese	mai4; maai5
U+3765.0	fMandarin	mi3
U+3765.0	fEnglish	[1] have a nighmare [2] sound sleep
U+3766.0	fUTF8	㝦
U+3766.0	fCNS11643	4-3C6F
U+3766.0	fGB	3-3B44
U+3766.0	fR/S	 40.10
U+3766.0	fTotalStrokes	 13
U+3766.0	fCangjie	JOGS
U+3766.0	fFourCorner	30220
U+3766.0	fCantonese	zeon3
U+3766.0	fMandarin	jun4; ya2
U+3766.0	fEnglish	same as &#x5BEF;
U+3767.0	fUTF8	㝧
U+3767.0	fCNS11643	3-4077
U+3767.0	fR/S	 40.10
U+3767.0	fTotalStrokes	 13
U+3767.0	fCangjie	JVOOG
U+3767.0	fFourCorner	30100
U+3767.0	fHakka	 vun3
U+3767.0	fCantonese	wan2
U+3767.0	fMandarin	wen3
U+3767.0	fEnglish	same as &#x7A69;
U+3768.0	fUTF8	㝨
U+3768.0	fR/S	 40.10
U+3768.0	fTotalStrokes	 13
U+3768.0	fCangjie	JGRV
U+3768.0	fCantonese	jyun4
U+3769.0	fUTF8	㝩
U+3769.0	fCNS11643	4-4274
U+3769.0	fGB	3-3B47
U+3769.0	fR/S	 40.11
U+3769.0	fTotalStrokes	 14
U+3769.0	fCangjie	JILE
U+3769.0	fFourCorner	30230
U+3769.0	fHakka	  kong1
U+3769.0	fCantonese	hoi1; hong1 (also)
U+3769.0	fMandarin	kang1
U+3769.0	fEnglish	empty; unoccupied
U+376A.0	fUTF8	㝪
U+376A.0	fCNS11643	4-4275
U+376A.0	fGB	3-3B3C
U+376A.0	fR/S	 40.11
U+376A.0	fTotalStrokes	 14
U+376A.0	fCangjie	JGJI
U+376A.0	fFourCorner	30410
U+376A.0	fHakka	  diam5
U+376A.0	fCantonese	dim3
U+376A.0	fMandarin	dian4; ding3
U+376A.0	fEnglish	[1] slanting house [2] nightmare
U+376B.0	fUTF8	㝫
U+376B.0	fCNS11643	3-4B60
U+376B.0	fR/S	 40.12; 40.11
U+376B.0	fTotalStrokes	 15; 14
U+376B.0	fCangjie	JNLM
U+376B.0	fFourCorner	30210
U+376B.0	fCantonese	lung4
U+376B.0	fMandarin	long2
U+376B.0	fEnglish	shape of the sky
U+376C.0	fUTF8	㝬
U+376C.0	fCNS11643	4-4277
U+376C.0	fGB	3-3B46
U+376C.0	fR/S	 40.11
U+376C.0	fTotalStrokes	 14
U+376C.0	fCangjie	JRQO
U+376C.0	fFourCorner	35630
U+376C.0	fCantonese	wu4
U+376D.0	fUTF8	㝭
U+376D.0	fCNS11643	4-4943
U+376D.0	fGB	5-3E39
U+376D.0	fR/S	 40.12
U+376D.0	fTotalStrokes	 15
U+376D.0	fCangjie	JPAM
U+376D.0	fFourCorner	30910
U+376D.0	fHakka	 siang3
U+376D.0	fCantonese	sing2
U+376D.0	fMandarin	xing3
U+376D.0	fEnglish	[1] (non-classical form) awake (from errors, illusions, etc.); come to one's sense [2] interchangeable with &#x60FA;
U+376E.0	fUTF8	㝮
U+376E.0	fCNS11643	4-4945
U+376E.0	fGB	3-3B49
U+376E.0	fR/S	 40.12
U+376E.0	fTotalStrokes	 15
U+376E.0	fCangjie	JUFE
U+376E.0	fFourCorner	30940
U+376E.0	fCantonese	ceoi3
U+376E.0	fMandarin	cui4
U+376E.0	fEnglish	[1] exile; banish [2] beyond the borders
U+376F.0	fUTF8	㝯
U+376F.0	fCNS11643	4-4946
U+376F.0	fGB	5-3D7B
U+376F.0	fR/S	 40.12
U+376F.0	fTotalStrokes	 15
U+376F.0	fCangjie	JHKB
U+376F.0	fFourCorner	30220
U+376F.0	fCantonese	kiu4
U+376F.0	fMandarin	qiao2
U+376F.0	fEnglish	[1] sojourn [2] sojourner; visitor
U+3770.0	fUTF8	㝰
U+3770.0	fCNS11643	4-5A7D
U+3770.0	fGB	5-3E3C
U+3770.0	fR/S	 40.15
U+3770.0	fTotalStrokes	 18
U+3770.0	fCangjie	JHUS
U+3770.0	fFourCorner	30220
U+3770.0	fCantonese	min4; maan5
U+3770.0	fMandarin	mian2
U+3770.0	fEnglish	[1] unable to meet [2] empty room
U+3771.0	fUTF8	㝱
U+3771.0	fCNS11643	3-5E2B
U+3771.0	fGB	5-3E3F
U+3771.0	fR/S	 40.18
U+3771.0	fTotalStrokes	 21
U+3771.0	fCangjie	JVMN
U+3771.0	fFourCorner	30220
U+3771.0	fCantonese	mung6; mung4
U+3771.0	fMandarin	yu4; meng4
U+3771.0	fEnglish	[1] (standard form) [2] dream [3] visionary [4] stupid
U+3772.0	fUTF8	㝲
U+3772.0	fCNS11643	7-6235
U+3772.0	fR/S	 40.23
U+3772.0	fTotalStrokes	 26
U+3772.0	fCangjie	JVMB
U+3772.0	fFourCorner	30227
U+3772.0	fCantonese	cam2
U+3772.0	fMandarin	qin3
U+3772.0	fEnglish	same as &#x5BE2;
U+3773.0	fUTF8	㝳
U+3773.0	fR/S	 41.2
U+3773.0	fTotalStrokes	 5
U+3773.0	fCangjie	DI
U+3773.0	fCantonese	sam1
U+3774.0	fUTF8	㝴
U+3774.0	fCNS11643	4-2371
U+3774.0	fR/S	 41.4
U+3774.0	fTotalStrokes	 7
U+3774.0	fCangjie	MUDI
U+3774.0	fFourCorner	14210
U+3774.0	fHakka	 ngan2
U+3774.0	fCantonese	jyun4; zyun4
U+3774.0	fMandarin	wan1; wan2
U+3774.0	fEnglish	same as &#x5213;
U+3775.0	fUTF8	㝵
U+3775.0	fCNS11643	4-2626
U+3775.0	fGB	5-4A38
U+3775.0	fR/S	 41.5
U+3775.0	fTotalStrokes	 8
U+3775.0	fCangjie	AMDI
U+3775.0	fFourCorner	60340
U+3775.0	fCantonese	dak1; daak1; ngoi6
U+3775.0	fMandarin	ai4; de2
U+3775.0	fEnglish	[1] ancient form of &#x5F97; [2] same as &#x7919;
U+3776.0	fUTF8	㝶
U+3776.0	fCNS11643	6-3668
U+3776.0	fR/S	 41.7
U+3776.0	fTotalStrokes	 10; 9
U+3776.0	fCangjie	WLHUI
U+3776.0	fFourCorner	60340
U+3776.0	fCantonese	dak1
U+3776.0	fMandarin	ai4; de2
U+3776.0	fEnglish	ancient form of &#x5F97;
U+3777.0	fUTF8	㝷
U+3777.0	fCNS11643	3-3B22
U+3777.0	fR/S	 41.9
U+3777.0	fTotalStrokes	 12
U+3777.0	fCangjie	SMMII
U+3777.0	fFourCorner	17340
U+3777.0	fCantonese	cam4
U+3777.0	fMandarin	xun2
U+3778.0	fUTF8	㝸
U+3778.0	fCNS11643	4-2C4C
U+3778.0	fGB	5-4078
U+3778.0	fR/S	 42.7
U+3778.0	fTotalStrokes	 10
U+3778.0	fCangjie	FHAU
U+3778.0	fFourCorner	90210
U+3778.0	fCantonese	bin6; beng2
U+3778.0	fMandarin	bian4
U+3778.0	fEnglish	same as &#x5F01;
U+3779.0	fUTF8	㝹
U+3779.0	fCNS11643	6-3E64
U+3779.0	fGB	3-3D6D
U+3779.0	fR/S	 42.9; 42.8
U+3779.0	fTotalStrokes	 12; 11
U+3779.0	fCangjie	FHNAU
U+3779.0	fFourCorner	90416
U+3779.0	fHakka	 neu2
U+3779.0	fCantonese	nau4; lau4 (also)
U+3779.0	fMandarin	nou2
U+3779.0	fEnglish	corrupted form of &#x246958;
U+377A.0	fUTF8	㝺
U+377A.0	fCNS11643	4-4278
U+377A.0	fGB	5-407A
U+377A.0	fR/S	 42.11
U+377A.0	fTotalStrokes	 14
U+377A.0	fCangjie	TCFH
U+377A.0	fFourCorner	89220
U+377A.0	fCantonese	lim4
U+377A.0	fMandarin	lian2; lin2
U+377A.0	fEnglish	[1] small; little [2] humble [3] owe [4] deficient; lacking
U+377B.0	fUTF8	㝻
U+377B.0	fCNS11643	5-4C63
U+377B.0	fR/S	 42.12
U+377B.0	fTotalStrokes	 15
U+377B.0	fCangjie	TMFH
U+377B.0	fFourCorner	49120
U+377B.0	fCantonese	gan2
U+377B.0	fMandarin	jin4; jin3
U+377B.0	fEnglish	same as &#x50C5;
U+377C.0	fUTF8	㝼
U+377C.0	fCNS11643	4-225D
U+377C.0	fGB	5-3749
U+377C.0	fR/S	 43.3
U+377C.0	fTotalStrokes	 6
U+377C.0	fCangjie	KUMD
U+377C.0	fFourCorner	41010
U+377C.0	fCantonese	jyu1; zyu1
U+377C.0	fMandarin	yu1
U+377C.0	fEnglish	[1] crooked; winding [2] circle; hover around
U+377D.0	fUTF8	㝽
U+377D.0	fCNS11643	4-2374
U+377D.0	fGB	3-3540
U+377D.0	fR/S	 43.4
U+377D.0	fTotalStrokes	 7
U+377D.0	fCangjie	KUE
U+377D.0	fFourCorner	42010
U+377D.0	fCantonese	seoi2; cai1; ceoi4; coek3
U+377D.0	fMandarin	chui2; shui3; zhui3
U+377D.0	fEnglish	[1] short [2] small; little
U+377E.0	fUTF8	㝾
U+377E.0	fCNS11643	4-2628
U+377E.0	fGB	3-3541
U+377E.0	fR/S	 43.5
U+377E.0	fTotalStrokes	 8
U+377E.0	fCangjie	KUKM
U+377E.0	fFourCorner	44010
U+377E.0	fHakka	  zo3
U+377E.0	fCantonese	zo2
U+377E.0	fMandarin	zuo3
U+377E.0	fEnglish	lame; crippled
U+377F.0	fUTF8	㝿
U+377F.0	fCNS11643	4-2627
U+377F.0	fGB	5-374B
U+377F.0	fR/S	 43.5
U+377F.0	fTotalStrokes	 8
U+377F.0	fCangjie	KUDHE
U+377F.0	fFourCorner	44010
U+377F.0	fCantonese	bai1; bo2 (also); bei3
U+377F.0	fMandarin	bo3; bo2; fu4; qian4
U+377F.0	fEnglish	non-classical form of &#x8DDB;
U+3780.0	fUTF8	㞀
U+3780.0	fCNS11643	4-2874
U+3780.0	fGB	5-374D
U+3780.0	fR/S	 43.6
U+3780.0	fTotalStrokes	 9
U+3780.0	fCangjie	KUKF
U+3780.0	fFourCorner	41010
U+3780.0	fCantonese	fui1
U+3780.0	fMandarin	hui1
U+3780.0	fEnglish	[1] bump [2] strike
U+3781.0	fUTF8	㞁
U+3781.0	fCNS11643	4-2875
U+3781.0	fGB	5-374E
U+3781.0	fR/S	 43.6
U+3781.0	fTotalStrokes	 9
U+3781.0	fCangjie	KUAP
U+3781.0	fFourCorner	46010
U+3781.0	fCantonese	jiu6; ziu6
U+3781.0	fMandarin	yao4
U+3781.0	fEnglish	[1] can not walk normally [2] swollen feet
U+3782.0	fUTF8	㞂
U+3782.0	fCNS11643	4-2C4D
U+3782.0	fGB	3-3544
U+3782.0	fR/S	 43.7
U+3782.0	fTotalStrokes	 10
U+3782.0	fCangjie	KUBV
U+3782.0	fFourCorner	42010
U+3782.0	fHakka	  tui3
U+3782.0	fCantonese	teoi2
U+3782.0	fMandarin	tui3
U+3782.0	fEnglish	lameness
U+3783.0	fUTF8	㞃
U+3783.0	fCNS11643	5-3175
U+3783.0	fR/S	 43.8
U+3783.0	fTotalStrokes	 11
U+3783.0	fCangjie	CRIKU
U+3783.0	fFourCorner	83610
U+3783.0	fCantonese	kek6; gik6
U+3783.0	fMandarin	ji2
U+3783.0	fEnglish	tired; weary
U+3784.0	fUTF8	㞄
U+3784.0	fCNS11643	4-3133
U+3784.0	fGB	3-3545
U+3784.0	fR/S	 43.8
U+3784.0	fTotalStrokes	 11
U+3784.0	fCangjie	KUKLU
U+3784.0	fFourCorner	44010
U+3784.0	fCantonese	am1; ngam1 (also); aam1; an2
U+3784.0	fMandarin	an1
U+3784.0	fEnglish	lame; crippled; feeble
U+3785.0	fUTF8	㞅
U+3785.0	fCNS11643	5-3173
U+3785.0	fR/S	 43.8
U+3785.0	fTotalStrokes	 11
U+3785.0	fCangjie	KUWD
U+3785.0	fFourCorner	46010
U+3785.0	fCantonese	lo6; gwo2; kong3; kwok3
U+3785.0	fMandarin	luo4; guo3
U+3785.0	fEnglish	[1] not straight [2] improper
U+3786.0	fUTF8	㞆
U+3786.0	fCNS11643	4-3132
U+3786.0	fGB	5-374F
U+3786.0	fR/S	 43.8
U+3786.0	fTotalStrokes	 11
U+3786.0	fCangjie	KUKMR
U+3786.0	fFourCorner	44010
U+3786.0	fCantonese	gei2; gei1
U+3786.0	fMandarin	ji3
U+3786.0	fEnglish	same as &#x8E26;
U+3787.0	fUTF8	㞇
U+3787.0	fCNS11643	4-3728
U+3787.0	fGB	3-3546
U+3787.0	fR/S	 43.9
U+3787.0	fTotalStrokes	 12
U+3787.0	fCangjie	KUWMV
U+3787.0	fFourCorner	46010
U+3787.0	fCantonese	wui2; waai2
U+3787.0	fMandarin	wei3; wei4
U+3787.0	fEnglish	same as &#x3FAF;
U+3788.0	fUTF8	㞈
U+3788.0	fCNS11643	4-3729
U+3788.0	fGB	3-3548
U+3788.0	fR/S	 43.9
U+3788.0	fTotalStrokes	 12
U+3788.0	fCangjie	KUISB
U+3788.0	fFourCorner	42010
U+3788.0	fCantonese	but3; bui6; but6
U+3788.0	fMandarin	bo1
U+3788.0	fEnglish	[1] big foot [2] walking
U+3789.0	fUTF8	㞉
U+3789.0	fCNS11643	4-3C71
U+3789.0	fGB	3-3547
U+3789.0	fR/S	 43.10
U+3789.0	fTotalStrokes	 13
U+3789.0	fCangjie	KUTQM
U+3789.0	fFourCorner	48010
U+3789.0	fCantonese	zaat3
U+3789.0	fMandarin	zan3
U+3789.0	fEnglish	big foot
U+378A.0	fUTF8	㞊
U+378A.0	fCNS11643	4-4279
U+378A.0	fGB	3-3549
U+378A.0	fR/S	 43.10; 43.11
U+378A.0	fTotalStrokes	 14; 13
U+378A.0	fCangjie	UFIKU
U+378A.0	fFourCorner	23910
U+378A.0	fCantonese	seot1
U+378A.0	fMandarin	xu4
U+378A.0	fEnglish	unable to walk
U+378B.0	fUTF8	㞋
U+378B.0	fCNS11643	3-2240
U+378B.0	fGB	5-3E4A
U+378B.0	fR/S	 44.2; 44.3; 29.3
U+378B.0	fTotalStrokes	 5
U+378B.0	fCangjie	SE
U+378B.0	fFourCorner	77240
U+378B.0	fCantonese	nin2; lin2 (also)
U+378B.0	fMandarin	nian3
U+378B.0	fEnglish	[1] soft and meek; weak [2] arrange [3] regulate
U+378C.0	fUTF8	㞌
U+378C.0	fCNS11643	3-2353
U+378C.0	fR/S	 44.3
U+378C.0	fTotalStrokes	 6
U+378C.0	fCangjie	SND
U+378C.0	fFourCorner	77240
U+378C.0	fHakka	 jin3
U+378C.0	fCantonese	jan6; lai1; zaan6
U+378C.0	fMandarin	yun4
U+378C.0	fEnglish	same as &#x5B55;
U+378D.0	fUTF8	㞍
U+378D.0	fR/S	 44.3
U+378D.0	fTotalStrokes	 6
U+378D.0	fCangjie	SKNI
U+378D.0	fCantonese	jyun4
U+378D.0	fEnglish	[Japanese character]
U+378E.0	fUTF8	㞎
U+378E.0	fGB	3-3B50
U+378E.0	fR/S	 44.4
U+378E.0	fTotalStrokes	 7
U+378E.0	fCangjie	SAU
U+378E.0	fCantonese	baa2; paa4
U+378E.0	fMandarin	ba3; fu2; pa2
U+378E.0	fEnglish	[1] body waste [2] weight measurement
U+378F.0	fUTF8	㞏
U+378F.0	fCNS11643	4-262A
U+378F.0	fGB	3-3B4F
U+378F.0	fR/S	 44.5
U+378F.0	fTotalStrokes	 8
U+378F.0	fCangjie	SHIO
U+378F.0	fFourCorner	77230
U+378F.0	fCantonese	zaap6; zaat6
U+378F.0	fMandarin	zhi2; zhe2
U+378F.0	fEnglish	[1] small [2] store up or pile in order [3] follow; trace [4] short step
U+3790.0	fUTF8	㞐
U+3790.0	fCNS11643	3-2852
U+3790.0	fGB	5-3E51
U+3790.0	fR/S	 44.5
U+3790.0	fTotalStrokes	 8
U+3790.0	fCangjie	SYT
U+3790.0	fFourCorner	77210
U+3790.0	fHakka	 gi1
U+3790.0	fCantonese	geoi1; gei1
U+3790.0	fMandarin	ju1; ji1
U+3790.0	fEnglish	ancient form of &#x5C45;
U+3791.0	fUTF8	㞑
U+3791.0	fCNS11643	3-2853
U+3791.0	fR/S	 44.5
U+3791.0	fTotalStrokes	 8
U+3791.0	fCangjie	SEU
U+3791.0	fFourCorner	77210
U+3791.0	fHakka	 mui1; mi1; mui3
U+3791.0	fCantonese	mei5
U+3791.0	fMandarin	wei3; yi3
U+3791.0	fEnglish	ancient form of &#x5C3E;
U+3792.0	fUTF8	㞒
U+3792.0	fCNS11643	4-2879
U+3792.0	fGB	3-3B54
U+3792.0	fR/S	 44.6
U+3792.0	fTotalStrokes	 9
U+3792.0	fCangjie	SHBU
U+3792.0	fFourCorner	77260
U+3792.0	fCantonese	ai3; ngai3 (also); hei3 (also); aai3
U+3792.0	fMandarin	xie4; xi4
U+3792.0	fEnglish	[1] (standard form) [2] lusty; strong; gigantic strength [3] heavy sleep with snoring
U+3793.0	fUTF8	㞓
U+3793.0	fCNS11643	4-2878
U+3793.0	fGB	3-3B55
U+3793.0	fR/S	 44.6
U+3793.0	fTotalStrokes	 9
U+3793.0	fCangjie	SPA
U+3793.0	fFourCorner	77260
U+3793.0	fHakka	 shi6
U+3793.0	fCantonese	hei3
U+3793.0	fMandarin	qi4; qi3
U+3793.0	fEnglish	[1] the buttocks; the rump; the sacrum [2] set side ways [3] the male organ
U+3794.0	fUTF8	㞔
U+3794.0	fCNS11643	3-2C39
U+3794.0	fGB	5-3E56
U+3794.0	fR/S	 44.6
U+3794.0	fTotalStrokes	 9
U+3794.0	fCangjie	SNIN
U+3794.0	fFourCorner	77220
U+3794.0	fCantonese	ji4
U+3794.0	fMandarin	shi3; yi2
U+3794.0	fEnglish	(corrupted form) the upright bar for fastening a door
U+3795.0	fUTF8	㞕
U+3795.0	fCNS11643	3-2C38
U+3795.0	fGB	5-3E55
U+3795.0	fR/S	 44.6
U+3795.0	fTotalStrokes	 9
U+3795.0	fCangjie	SCB
U+3795.0	fFourCorner	77220
U+3795.0	fHakka	 siet7
U+3795.0	fCantonese	sit3; ngaat6
U+3795.0	fMandarin	xie4
U+3795.0	fEnglish	same as &#x5C51;
U+3796.0	fUTF8	㞖
U+3796.0	fCNS11643	3-2C36
U+3796.0	fGB	5-3E53
U+3796.0	fR/S	 44.6
U+3796.0	fTotalStrokes	 9
U+3796.0	fCangjie	SDB
U+3796.0	fFourCorner	77290
U+3796.0	fCantonese	ci1; ci3; zi4
U+3796.0	fMandarin	ci4
U+3796.0	fEnglish	[1] this [2] here [3] if so; in this case [4] reluctant to go forward
U+3797.0	fUTF8	㞗
U+3797.0	fCNS11643	4-2C4E
U+3797.0	fGB	3-3B57
U+3797.0	fR/S	 44.7
U+3797.0	fTotalStrokes	 10
U+3797.0	fCangjie	SIJE
U+3797.0	fFourCorner	77230
U+3797.0	fHakka	 kiu2
U+3797.0	fCantonese	gau1; kau4
U+3797.0	fMandarin	qiu2
U+3797.0	fEnglish	the penis
U+3798.0	fUTF8	㞘
U+3798.0	fCNS11643	6-3E6B
U+3798.0	fGB	3-3B59
U+3798.0	fR/S	 44.8
U+3798.0	fTotalStrokes	 11
U+3798.0	fCangjie	SMSO
U+3798.0	fFourCorner	77230
U+3798.0	fCantonese	duk1; tyun4
U+3798.0	fMandarin	du4; tun2
U+3798.0	fEnglish	non-classical form of &#x8C5A;
U+3799.0	fUTF8	㞙
U+3799.0	fCNS11643	3-3545
U+3799.0	fR/S	 44.8
U+3799.0	fTotalStrokes	 11
U+3799.0	fCangjie	SHUE
U+3799.0	fFourCorner	77230
U+3799.0	fHakka	 ngiau5
U+3799.0	fCantonese	niu6; liu6 (also); seoi1
U+3799.0	fMandarin	niao4; sui1
U+3799.0	fEnglish	same as &#x5C3F;
U+379A.0	fUTF8	㞚
U+379A.0	fCNS11643	4-372B
U+379A.0	fGB	3-3B5B
U+379A.0	fR/S	 44.9
U+379A.0	fTotalStrokes	 12
U+379A.0	fCangjie	SHJX
U+379A.0	fFourCorner	77277
U+379A.0	fCantonese	cap1; kaai3
U+379A.0	fMandarin	qi4; zha3
U+379A.0	fEnglish	[1] pile up layer by layer [2] wedge [3] gatepost
U+379B.0	fUTF8	㞛
U+379B.0	fCNS11643	5-3766
U+379B.0	fGB	5-3E61
U+379B.0	fR/S	 44.9
U+379B.0	fTotalStrokes	 12
U+379B.0	fCangjie	SHOA
U+379B.0	fFourCorner	77260
U+379B.0	fCantonese	gei2
U+379B.0	fMandarin	ji3
U+379B.0	fEnglish	[1] red slippers [2] corrupted form of &#x5C65;
U+379C.0	fUTF8	㞜
U+379C.0	fCNS11643	4-427D
U+379C.0	fGB	3-3B5D
U+379C.0	fR/S	 44.11
U+379C.0	fTotalStrokes	 14
U+379C.0	fCangjie	SHOV
U+379C.0	fFourCorner	77240
U+379C.0	fCantonese	teoi1
U+379C.0	fMandarin	tui1
U+379C.0	fEnglish	[1] hempen sandals [2] boots
U+379D.0	fUTF8	㞝
U+379D.0	fR/S	 44.11
U+379D.0	fTotalStrokes	 14
U+379D.0	fCangjie	IFS
U+379D.0	fCantonese	cik1
U+379E.0	fUTF8	㞞
U+379E.0	fGB	3-3B4E
U+379E.0	fR/S	 44.11
U+379E.0	fTotalStrokes	 14
U+379E.0	fCangjie	SHOO
U+379E.0	fFourCorner	7728
U+379E.0	fCantonese	sung4; cung4 (also)
U+379F.0	fUTF8	㞟
U+379F.0	fCNS11643	4-494A
U+379F.0	fGB	3-3B5F
U+379F.0	fR/S	 44.12
U+379F.0	fTotalStrokes	 15
U+379F.0	fCangjie	STWK
U+379F.0	fFourCorner	77234
U+379F.0	fCantonese	din6
U+379F.0	fMandarin	dian4
U+379F.0	fEnglish	[1] savings and (or) reserves [2] interchangeable with &#x5960; [3] interchangeable with &#x588A;
U+37A0.0	fUTF8	㞠
U+37A0.0	fGB	3-3B5E
U+37A0.0	fR/S	 44.12
U+37A0.0	fTotalStrokes	 15
U+37A0.0	fCangjie	SKCF
U+37A0.0	fFourCorner	7729
U+37A0.0	fCantonese	liu4
U+37A0.0	fMandarin	lao2; liao2
U+37A0.0	fEnglish	male organ
U+37A1.0	fUTF8	㞡
U+37A1.0	fCNS11643	3-5A37
U+37A1.0	fR/S	 44.16
U+37A1.0	fTotalStrokes	 19
U+37A1.0	fCangjie	SMMV
U+37A1.0	fFourCorner	77230
U+37A1.0	fHakka	 zhen3
U+37A1.0	fCantonese	zin2
U+37A1.0	fMandarin	zhan3
U+37A1.0	fEnglish	standard form of &#x5C55;
U+37A2.0	fUTF8	㞢
U+37A2.0	fCNS11643	3-2162
U+37A2.0	fGB	5-3F33
U+37A2.0	fR/S	 45.1
U+37A2.0	fTotalStrokes	 4
U+37A2.0	fCangjie	UM
U+37A2.0	fFourCorner	22100
U+37A2.0	fCantonese	zi1
U+37A2.0	fMandarin	zhi1
U+37A2.0	fEnglish	ancient form of &#x4E4B;
U+37A3.0	fUTF8	㞣
U+37A3.0	fCNS11643	4-2377
U+37A3.0	fR/S	 45.4
U+37A3.0	fTotalStrokes	 7
U+37A3.0	fCangjie	UCSH
U+37A3.0	fFourCorner	22220
U+37A3.0	fCantonese	fan1
U+37A3.0	fMandarin	fen1
U+37A3.0	fEnglish	same as &#x82AC;
U+37A4.0	fUTF8	㞤
U+37A4.0	fCNS11643	3-2242
U+37A4.0	fR/S	 46.2
U+37A4.0	fTotalStrokes	 5
U+37A4.0	fCangjie	UO
U+37A4.0	fFourCorner	22800
U+37A4.0	fCantonese	jam4; saam4
U+37A4.0	fMandarin	yin2
U+37A4.0	fEnglish	[1] help; aid; assist [2] ancient form of &#x5C91;
U+37A5.0	fUTF8	㞥
U+37A5.0	fCNS11643	3-2243
U+37A5.0	fGB	5-3A55
U+37A5.0	fR/S	 46.2
U+37A5.0	fTotalStrokes	 5
U+37A5.0	fCangjie	UO
U+37A5.0	fFourCorner	22800
U+37A5.0	fCantonese	sam4; cip3; gip3; saam4; seon2
U+37A5.0	fMandarin	cen2
U+37A5.0	fEnglish	deep in the mountain
U+37A6.0	fUTF8	㞦
U+37A6.0	fCNS11643	3-2241
U+37A6.0	fGB	5-3A56
U+37A6.0	fR/S	 46.2
U+37A6.0	fTotalStrokes	 5
U+37A6.0	fCangjie	UHN
U+37A6.0	fFourCorner	27710
U+37A6.0	fCantonese	gei2
U+37A6.0	fMandarin	ji3
U+37A6.0	fEnglish	the name of a mountain
U+37A7.0	fUTF8	㞧
U+37A7.0	fCNS11643	3-2244
U+37A7.0	fGB	5-3A57
U+37A7.0	fR/S	 46.2
U+37A7.0	fTotalStrokes	 5
U+37A7.0	fCangjie	UNHS
U+37A7.0	fFourCorner	22220
U+37A7.0	fHakka	 fui5; kwai5
U+37A7.0	fCantonese	wui6; wui2; wui4
U+37A7.0	fMandarin	hui4; gua1; guai4
U+37A7.0	fEnglish	ancient form of &#x6703;
U+37A8.0	fUTF8	㞨
U+37A8.0	fCNS11643	3-2356
U+37A8.0	fR/S	 46.3
U+37A8.0	fTotalStrokes	 6
U+37A8.0	fCangjie	UND
U+37A8.0	fFourCorner	27740
U+37A8.0	fCantonese	zi2; zi1
U+37A8.0	fMandarin	zi3; zai3
U+37A8.0	fEnglish	(corrupted form) the name of a mountain (location unknown)
U+37A9.0	fUTF8	㞩
U+37A9.0	fCNS11643	4-2261
U+37A9.0	fR/S	 46.3
U+37A9.0	fTotalStrokes	 6
U+37A9.0	fCangjie	UHNI
U+37A9.0	fFourCorner	22210
U+37A9.0	fHakka	 lam2
U+37A9.0	fCantonese	laam4
U+37A9.0	fMandarin	lan2
U+37A9.0	fEnglish	same as &#x5D50;
U+37AA.0	fUTF8	㞪
U+37AA.0	fCNS11643	5-2251
U+37AA.0	fR/S	 46.3
U+37AA.0	fTotalStrokes	 6
U+37AA.0	fCangjie	NMMU
U+37AA.0	fFourCorner	17770
U+37AA.0	fCantonese	nou4; lou4 (also); mau6; nau4
U+37AA.0	fMandarin	nao2
U+37AA.0	fEnglish	[1] the appearance of the mountain is flat and smooth [2] corrupted form of &#x5CF1;
U+37AB.0	fUTF8	㞫
U+37AB.0	fCNS11643	4-237B
U+37AB.0	fR/S	 46.3
U+37AB.0	fTotalStrokes	 7; 6
U+37AB.0	fCangjie	UINO
U+37AB.0	fFourCorner	22300
U+37AB.0	fHakka	 zeu3
U+37AB.0	fCantonese	zau2; zaau2
U+37AB.0	fMandarin	zou3; ju4; zou4
U+37AB.0	fEnglish	ancient form of &#x8D70;
U+37AC.0	fUTF8	㞬
U+37AC.0	fCNS11643	5-2250
U+37AC.0	fR/S	 46.3
U+37AC.0	fTotalStrokes	 6
U+37AC.0	fCangjie	UM
U+37AC.0	fFourCorner	22100
U+37AC.0	fCantonese	jan2
U+37AC.0	fMandarin	qin4
U+37AC.0	fEnglish	close or near
U+37AD.0	fUTF8	㞭
U+37AD.0	fCNS11643	3-2357
U+37AD.0	fR/S	 46.3
U+37AD.0	fTotalStrokes	 6
U+37AD.0	fCangjie	KU
U+37AD.0	fFourCorner	40770
U+37AD.0	fCantonese	daai6
U+37AD.0	fMandarin	dai4
U+37AD.0	fEnglish	the name of an island
U+37AE.0	fUTF8	㞮
U+37AE.0	fR/S	 46.3
U+37AE.0	fTotalStrokes	 6
U+37AE.0	fCangjie	UNI
U+37AE.0	fCantonese	ngai4; ai4 (also)
U+37AE.0	fEnglish	variant of &#x51FA;
U+37AF.0	fUTF8	㞯
U+37AF.0	fCNS11643	3-2358
U+37AF.0	fR/S	 46.3
U+37AF.0	fTotalStrokes	 6
U+37AF.0	fCangjie	SUU
U+37AF.0	fFourCorner	17770
U+37AF.0	fHakka	 ki3; hi3; gi5
U+37AF.0	fCantonese	hei2; gei2; zit3
U+37AF.0	fMandarin	qi3; jie2
U+37AF.0	fEnglish	same as &#x5C7A;
U+37B0.0	fUTF8	㞰
U+37B0.0	fCNS11643	4-2378
U+37B0.0	fGB	3-3846
U+37B0.0	fR/S	 46.4
U+37B0.0	fTotalStrokes	 7
U+37B0.0	fCangjie	UOJ
U+37B0.0	fFourCorner	28740
U+37B0.0	fCantonese	heoi2
U+37B0.0	fMandarin	xu3
U+37B0.0	fEnglish	the name of a mountain
U+37B1.0	fUTF8	㞱
U+37B1.0	fCNS11643	4-2379
U+37B1.0	fGB	3-3845
U+37B1.0	fR/S	 46.4
U+37B1.0	fTotalStrokes	 7
U+37B1.0	fCangjie	UA
U+37B1.0	fFourCorner	22600
U+37B1.0	fCantonese	sung1; cung1
U+37B1.0	fEnglish	dusk; twilight
U+37B2.0	fUTF8	㞲
U+37B2.0	fCNS11643	3-2560
U+37B2.0	fR/S	 46.4
U+37B2.0	fTotalStrokes	 7
U+37B2.0	fCangjie	UL
U+37B2.0	fFourCorner	25700
U+37B2.0	fCantonese	jung6; zung6
U+37B2.0	fMandarin	yong4
U+37B2.0	fEnglish	[1] the name of a mountain [2] the plain between mountains
U+37B3.0	fUTF8	㞳
U+37B3.0	fCNS11643	5-2370
U+37B3.0	fR/S	 46.4
U+37B3.0	fTotalStrokes	 7
U+37B3.0	fCangjie	UYJ
U+37B3.0	fFourCorner	24700
U+37B3.0	fCantonese	dau2
U+37B3.0	fMandarin	dou3
U+37B3.0	fEnglish	the name of a mountain
U+37B4.0	fUTF8	㞴
U+37B4.0	fCNS11643	5-2374
U+37B4.0	fR/S	 46.4
U+37B4.0	fTotalStrokes	 7
U+37B4.0	fCangjie	UHVP
U+37B4.0	fFourCorner	22740
U+37B4.0	fCantonese	ci4
U+37B4.0	fMandarin	chi2
U+37B4.0	fEnglish	[1] the name of a mountain [2] non-classical form of &#x5CB7;
U+37B5.0	fUTF8	㞵
U+37B5.0	fCNS11643	3-2564
U+37B5.0	fR/S	 46.4
U+37B5.0	fTotalStrokes	 7
U+37B5.0	fCangjie	UYK
U+37B5.0	fFourCorner	22400
U+37B5.0	fCantonese	tou1
U+37B5.0	fMandarin	tao1
U+37B5.0	fEnglish	[1] (corrupted form) [2] frivolous; flippant; capricious; plyful; sportive [3] cunning
U+37B6.0	fUTF8	㞶
U+37B6.0	fCNS11643	3-2565
U+37B6.0	fR/S	 46.4
U+37B6.0	fTotalStrokes	 7
U+37B6.0	fCangjie	UYK
U+37B6.0	fFourCorner	20740
U+37B6.0	fHakka	 min2
U+37B6.0	fCantonese	man4
U+37B6.0	fMandarin	min2; min3
U+37B6.0	fEnglish	same as &#x5CB7;
U+37B7.0	fUTF8	㞷
U+37B7.0	fCNS11643	4-2376
U+37B7.0	fR/S	 45.4
U+37B7.0	fTotalStrokes	 7
U+37B7.0	fCangjie	UMG
U+37B7.0	fFourCorner	22100
U+37B7.0	fCantonese	wong4
U+37B7.0	fMandarin	huang2
U+37B7.0	fEnglish	luxuriant; lush; flourishing
U+37B8.0	fUTF8	㞸
U+37B8.0	fR/S	 46.4
U+37B8.0	fTotalStrokes	 7
U+37B8.0	fCangjie	UMF
U+37B8.0	fCantonese	seoi3
U+37B8.0	fMandarin	sui4
U+37B8.0	fEnglish	[1] same as &#x5461; [2] simplified form of &#x6B72;
U+37B9.0	fUTF8	㞹
U+37B9.0	fCNS11643	3-285B
U+37B9.0	fR/S	 46.5
U+37B9.0	fTotalStrokes	 8
U+37B9.0	fCangjie	UMNR
U+37B9.0	fFourCorner	21720
U+37B9.0	fHakka	 ko3
U+37B9.0	fCantonese	ho2
U+37B9.0	fMandarin	ke3
U+37B9.0	fEnglish	same as &#x5CA2;
U+37BA.0	fUTF8	㞺
U+37BA.0	fCNS11643	6-2A7B
U+37BA.0	fR/S	 46.5
U+37BA.0	fTotalStrokes	 8
U+37BA.0	fCangjie	UOK
U+37BA.0	fFourCorner	22430
U+37BA.0	fHakka	 cuk8
U+37BA.0	fCantonese	zuk6
U+37BA.0	fMandarin	zu2; cou4
U+37BA.0	fEnglish	ancient form of &#x65CF;
U+37BB.0	fUTF8	㞻
U+37BB.0	fCNS11643	4-2631
U+37BB.0	fGB	3-3856
U+37BB.0	fR/S	 46.5
U+37BB.0	fTotalStrokes	 8
U+37BB.0	fCangjie	URMS
U+37BB.0	fFourCorner	22200
U+37BB.0	fCantonese	hou6
U+37BB.0	fMandarin	hao4
U+37BB.0	fEnglish	the name of a mountain
U+37BC.0	fUTF8	㞼
U+37BC.0	fCNS11643	4-2632
U+37BC.0	fR/S	 46.5
U+37BC.0	fTotalStrokes	 8
U+37BC.0	fCangjie	NEU
U+37BC.0	fFourCorner	17770
U+37BC.0	fHakka	 shin2
U+37BC.0	fCantonese	sing4; nip6
U+37BC.0	fMandarin	cheng2; sheng4; zhe2
U+37BC.0	fEnglish	same as &#x4E1E;
U+37BD.0	fUTF8	㞽
U+37BD.0	fCNS11643	4-262E
U+37BD.0	fGB	3-3852
U+37BD.0	fR/S	 46.5
U+37BD.0	fTotalStrokes	 8
U+37BD.0	fCangjie	UIV
U+37BD.0	fFourCorner	23750
U+37BD.0	fCantonese	hyut3
U+37BD.0	fMandarin	yue4; xue4
U+37BD.0	fEnglish	mountain; hill
U+37BE.0	fUTF8	㞾
U+37BE.0	fCNS11643	3-2856
U+37BE.0	fGB	5-3A6D
U+37BE.0	fR/S	 46.5
U+37BE.0	fTotalStrokes	 8
U+37BE.0	fCangjie	USP
U+37BE.0	fFourCorner	27710
U+37BE.0	fHakka	 ni2
U+37BE.0	fCantonese	nei4; lei4 (also)
U+37BE.0	fMandarin	ni2; yi4
U+37BE.0	fEnglish	the name of a mountain in the Lu state
U+37BF.0	fUTF8	㞿
U+37BF.0	fCNS11643	6-2A76
U+37BF.0	fR/S	 46.5
U+37BF.0	fTotalStrokes	 8
U+37BF.0	fCangjie	UGK
U+37BF.0	fFourCorner	22400
U+37BF.0	fHakka	 ki1
U+37BF.0	fCantonese	kei4
U+37BF.0	fMandarin	qi2; chi4
U+37BF.0	fEnglish	same as &#x5C90;
U+37C0.0	fUTF8	㟀
U+37C0.0	fCNS11643	3-2859
U+37C0.0	fR/S	 46.5
U+37C0.0	fTotalStrokes	 8
U+37C0.0	fCangjie	HDU
U+37C0.0	fFourCorner	20770
U+37C0.0	fCantonese	lin4
U+37C0.0	fMandarin	yao3; lian2
U+37C0.0	fEnglish	the name of a mountain
U+37C1.0	fUTF8	㟁
U+37C1.0	fCNS11643	3-285A
U+37C1.0	fR/S	 46.5
U+37C1.0	fTotalStrokes	 8
U+37C1.0	fCangjie	UMMJ
U+37C1.0	fFourCorner	21740
U+37C1.0	fCantonese	ngon6; on6 (also); ngang6
U+37C1.0	fMandarin	an4
U+37C1.0	fEnglish	variant of &#x5CB8;
U+37C2.0	fUTF8	㟂
U+37C2.0	fGB	3-385F
U+37C2.0	fR/S	 46.5
U+37C2.0	fTotalStrokes	 8
U+37C2.0	fCangjie	UWYI
U+37C2.0	fCantonese	mou5
U+37C2.0	fMandarin	chi3; mu3
U+37C2.0	fEnglish	the name of cape in Shandong Province
U+37C3.0	fUTF8	㟃
U+37C3.0	fCNS11643	5-256E
U+37C3.0	fR/S	 46.5
U+37C3.0	fTotalStrokes	 8
U+37C3.0	fCangjie	USMR
U+37C3.0	fFourCorner	27720
U+37C3.0	fHakka	 sii1
U+37C3.0	fCantonese	si1
U+37C3.0	fMandarin	si1; si4
U+37C3.0	fEnglish	corrupt form of &#x53F8;
U+37C4.0	fUTF8	㟄
U+37C4.0	fCNS11643	3-2C3A
U+37C4.0	fR/S	 46.6
U+37C4.0	fTotalStrokes	 9
U+37C4.0	fCangjie	UTQ
U+37C4.0	fFourCorner	28750
U+37C4.0	fCantonese	coeng4
U+37C4.0	fMandarin	xiang2
U+37C4.0	fEnglish	the name of a mountain
U+37C5.0	fUTF8	㟅
U+37C5.0	fCNS11643	4-287B
U+37C5.0	fGB	3-3869
U+37C5.0	fR/S	 46.6
U+37C5.0	fTotalStrokes	 9
U+37C5.0	fCangjie	UUKU
U+37C5.0	fFourCorner	22710
U+37C5.0	fCantonese	ngong4; ong4 (also); ou2; zoeng4; zoeng6
U+37C5.0	fMandarin	yang2
U+37C5.0	fEnglish	lofty; high; steep
U+37C6.0	fUTF8	㟆
U+37C6.0	fGB	S-2122
U+37C6.0	fR/S	 46.6
U+37C6.0	fTotalStrokes	 9
U+37C6.0	fCangjie	UOPJ
U+37C6.0	fCantonese	waa4
U+37C6.0	fMandarin	hua2
U+37C6.0	fEnglish	the name of a mountain
U+37C7.0	fUTF8	㟇
U+37C7.0	fCNS11643	4-2C5B
U+37C7.0	fGB	3-3878
U+37C7.0	fR/S	 46.7
U+37C7.0	fTotalStrokes	 10
U+37C7.0	fCangjie	UOOG
U+37C7.0	fFourCorner	28710
U+37C7.0	fHakka	 zo3; co5
U+37C7.0	fCantonese	zo3; co4
U+37C7.0	fMandarin	zuo4; cuo2; cuo3
U+37C7.0	fEnglish	a hill that appears ready to fall
U+37C8.0	fUTF8	㟈
U+37C8.0	fCNS11643	4-2C53
U+37C8.0	fGB	3-3874
U+37C8.0	fR/S	 46.7
U+37C8.0	fTotalStrokes	 10
U+37C8.0	fCangjie	UIJE
U+37C8.0	fFourCorner	23730
U+37C8.0	fCantonese	kau4
U+37C8.0	fMandarin	qiu2
U+37C8.0	fEnglish	the name of a mountain
U+37C9.0	fUTF8	㟉
U+37C9.0	fCNS11643	4-2C51
U+37C9.0	fGB	3-387E
U+37C9.0	fR/S	 46.7
U+37C9.0	fTotalStrokes	 10
U+37C9.0	fCangjie	UJHQ
U+37C9.0	fFourCorner	23750
U+37C9.0	fCantonese	lou4
U+37C9.0	fMandarin	lao2
U+37C9.0	fEnglish	[1] the name of pavilion (garden) in ancient China [2] the name of a mountain
U+37CA.0	fUTF8	㟊
U+37CA.0	fCNS11643	3-3060
U+37CA.0	fR/S	 46.7
U+37CA.0	fTotalStrokes	 10
U+37CA.0	fCangjie	UBND
U+37CA.0	fFourCorner	22740
U+37CA.0	fCantonese	fau4; fu1
U+37CA.0	fMandarin	fou2; fu2
U+37CA.0	fEnglish	the name of a mountain
U+37CB.0	fUTF8	㟋
U+37CB.0	fCNS11643	4-2C5F
U+37CB.0	fGB	3-387C
U+37CB.0	fR/S	 46.7
U+37CB.0	fTotalStrokes	 10
U+37CB.0	fCangjie	UCRU
U+37CB.0	fFourCorner	28710
U+37CB.0	fCantonese	deoi6; deoi3
U+37CB.0	fMandarin	dui1; dui4
U+37CB.0	fEnglish	deep and level ground gully at the foot of a hill
U+37CC.0	fUTF8	㟌
U+37CC.0	fCNS11643	4-2C55
U+37CC.0	fGB	3-3873
U+37CC.0	fR/S	 46.7
U+37CC.0	fTotalStrokes	 10
U+37CC.0	fCangjie	UIKH
U+37CC.0	fFourCorner	23710
U+37CC.0	fCantonese	mong4
U+37CC.0	fMandarin	mang1; mang2
U+37CC.0	fEnglish	a mountain in today's Sichuan province
U+37CD.0	fUTF8	㟍
U+37CD.0	fCNS11643	3-305C
U+37CD.0	fR/S	 46.7
U+37CD.0	fTotalStrokes	 10
U+37CD.0	fCangjie	UIAV
U+37CD.0	fFourCorner	23730
U+37CD.0	fHakka	  long2
U+37CD.0	fCantonese	long4
U+37CD.0	fMandarin	lang2
U+37CD.0	fEnglish	the hill behind which the sun sets at the winter solstice
U+37CE.0	fUTF8	㟎
U+37CE.0	fCNS11643	4-2C5A
U+37CE.0	fGB	3-3879
U+37CE.0	fR/S	 46.7
U+37CE.0	fTotalStrokes	 10
U+37CE.0	fCangjie	UBV
U+37CE.0	fFourCorner	22740
U+37CE.0	fCantonese	to2; to5
U+37CE.0	fMandarin	tui3; tuo3
U+37CE.0	fEnglish	[1] long mountain ridge [2] steep; lofty mountain
U+37CF.0	fUTF8	㟏
U+37CF.0	fCNS11643	4-2C5E
U+37CF.0	fGB	3-387A
U+37CF.0	fR/S	 46.7
U+37CF.0	fTotalStrokes	 10
U+37CF.0	fCangjie	UOIR
U+37CF.0	fFourCorner	28760
U+37CF.0	fCantonese	ham4
U+37CF.0	fMandarin	han1; han2
U+37CF.0	fEnglish	[1] steep and lofty [2] big valley
U+37D0.0	fUTF8	㟐
U+37D0.0	fCNS11643	4-2C57
U+37D0.0	fGB	3-3861
U+37D0.0	fR/S	 46.7
U+37D0.0	fTotalStrokes	 10
U+37D0.0	fCangjie	UTYV
U+37D0.0	fFourCorner	24710
U+37D0.0	fCantonese	mong5
U+37D0.0	fMandarin	mang3
U+37D0.0	fEnglish	big and high; lofty; steep
U+37D1.0	fUTF8	㟑
U+37D1.0	fCNS11643	3-305D
U+37D1.0	fR/S	 46.7
U+37D1.0	fTotalStrokes	 10
U+37D1.0	fCangjie	UJBD
U+37D1.0	fFourCorner	24740
U+37D1.0	fCantonese	but6
U+37D1.0	fMandarin	bo2
U+37D1.0	fEnglish	mountain; hill
U+37D2.0	fUTF8	㟒
U+37D2.0	fCNS11643	6-367C
U+37D2.0	fR/S	 46.7
U+37D2.0	fTotalStrokes	 10; 9
U+37D2.0	fCangjie	USKR
U+37D2.0	fFourCorner	22600
U+37D2.0	fCantonese	kwan1
U+37D2.0	fMandarin	qun1
U+37D2.0	fEnglish	[1] mountains adjoining; mountains are unbroken connceted [2] the name of a mountain
U+37D3.0	fUTF8	㟓
U+37D3.0	fCNS11643	5-2D2E
U+37D3.0	fR/S	 46.7
U+37D3.0	fTotalStrokes	 10
U+37D3.0	fCangjie	UKKB
U+37D3.0	fFourCorner	24720
U+37D3.0	fCantonese	kei4
U+37D3.0	fMandarin	qi1; qi2
U+37D3.0	fEnglish	[1] lofty and dangerous [2] rocks on the mountain side
U+37D4.0	fUTF8	㟔
U+37D4.0	fCNS11643	3-3065
U+37D4.0	fR/S	 46.7
U+37D4.0	fTotalStrokes	 10
U+37D4.0	fCangjie	UOIR
U+37D4.0	fFourCorner	22600
U+37D4.0	fCantonese	ham4
U+37D4.0	fMandarin	han1; han2
U+37D4.0	fEnglish	same as &#x37CF;
U+37D5.0	fUTF8	㟕
U+37D5.0	fR/S	 46.7
U+37D5.0	fTotalStrokes	 10
U+37D5.0	fCangjie	URYJ
U+37D5.0	fCantonese	giu3
U+37D6.0	fUTF8	㟖
U+37D6.0	fCNS11643	4-2C54
U+37D6.0	fGB	3-3870
U+37D6.0	fR/S	 46.7
U+37D6.0	fTotalStrokes	 10
U+37D6.0	fCangjie	UMGT
U+37D6.0	fFourCorner	22440
U+37D6.0	fCantonese	lung6
U+37D6.0	fMandarin	long4
U+37D6.0	fEnglish	[1] plain; level ground between rocky mountain [2] the name of a county
U+37D7.0	fUTF8	㟗
U+37D7.0	fCNS11643	3-3556
U+37D7.0	fR/S	 46.8
U+37D7.0	fTotalStrokes	 11
U+37D7.0	fCangjie	UCHH
U+37D7.0	fFourCorner	22770
U+37D7.0	fHakka	 bin1
U+37D7.0	fCantonese	ban1
U+37D7.0	fMandarin	bin1
U+37D7.0	fEnglish	same as &#x90A0;
U+37D8.0	fUTF8	㟘
U+37D8.0	fCNS11643	3-3549
U+37D8.0	fR/S	 46.8
U+37D8.0	fTotalStrokes	 11
U+37D8.0	fCangjie	UBGR
U+37D8.0	fFourCorner	27720
U+37D8.0	fCantonese	tiu4
U+37D8.0	fMandarin	tiao2
U+37D8.0	fEnglish	the name of a mountain
U+37D9.0	fUTF8	㟙
U+37D9.0	fCNS11643	4-3137
U+37D9.0	fGB	3-3925
U+37D9.0	fR/S	 46.8
U+37D9.0	fTotalStrokes	 11
U+37D9.0	fCangjie	UTA
U+37D9.0	fFourCorner	24760
U+37D9.0	fCantonese	zaak6
U+37D9.0	fMandarin	ze2; lao3; zhai2
U+37D9.0	fEnglish	[1] shape of the mountain [2] a mountain in today's Shandong province
U+37DA.0	fUTF8	㟚
U+37DA.0	fCNS11643	3-354C
U+37DA.0	fR/S	 46.8
U+37DA.0	fTotalStrokes	 11
U+37DA.0	fCangjie	DEU
U+37DA.0	fFourCorner	44770
U+37DA.0	fHakka	 ki1
U+37DA.0	fCantonese	kei4
U+37DA.0	fMandarin	qi2
U+37DA.0	fEnglish	ancient form of &#x5C90;
U+37DB.0	fUTF8	㟛
U+37DB.0	fCNS11643	4-313D
U+37DB.0	fGB	5-3B3A
U+37DB.0	fR/S	 46.8
U+37DB.0	fTotalStrokes	 11
U+37DB.0	fCangjie	UQTM
U+37DB.0	fFourCorner	22570
U+37DB.0	fCantonese	zaam6; zan2; zan3
U+37DB.0	fMandarin	zan1; zan4
U+37DB.0	fEnglish	high and steep; lofty
U+37DC.0	fUTF8	㟜
U+37DC.0	fCNS11643	6-3E75
U+37DC.0	fR/S	 46.8
U+37DC.0	fTotalStrokes	 11
U+37DC.0	fCangjie	UNNF
U+37DC.0	fFourCorner	22290
U+37DC.0	fCantonese	mei4; nei4
U+37DC.0	fMandarin	mi2
U+37DC.0	fEnglish	same as &#x21FB1;
U+37DD.0	fUTF8	㟝
U+37DD.0	fCNS11643	3-354D
U+37DD.0	fR/S	 46.8
U+37DD.0	fTotalStrokes	 11
U+37DD.0	fCangjie	UYTR
U+37DD.0	fFourCorner	20760
U+37DD.0	fHakka	 pui2
U+37DD.0	fCantonese	pui4; pau2
U+37DD.0	fMandarin	pei2; pou3; bei4; fou2; fu3
U+37DD.0	fEnglish	same as &#x57F9;
U+37DE.0	fUTF8	㟞
U+37DE.0	fCNS11643	4-3139
U+37DE.0	fGB	3-384D
U+37DE.0	fR/S	 46.8
U+37DE.0	fTotalStrokes	 11
U+37DE.0	fCangjie	UII
U+37DE.0	fFourCorner	23750
U+37DE.0	fCantonese	zaan6; zan2; zan6
U+37DE.0	fMandarin	zhan3; zhan4
U+37DE.0	fEnglish	precipitous and lofty; very steep mountains
U+37DF.0	fUTF8	㟟
U+37DF.0	fCNS11643	6-3E78
U+37DF.0	fR/S	 46.8
U+37DF.0	fTotalStrokes	 11
U+37DF.0	fCangjie	UTCU
U+37DF.0	fFourCorner	22710
U+37DF.0	fHakka	 hong5
U+37DF.0	fCantonese	hong6; hong2; hot3
U+37DF.0	fMandarin	xiang4; hang4
U+37DF.0	fEnglish	same as &#x5DF7;
U+37E0.0	fUTF8	㟠
U+37E0.0	fCNS11643	3-3550
U+37E0.0	fR/S	 46.8
U+37E0.0	fTotalStrokes	 11
U+37E0.0	fCangjie	UBTU
U+37E0.0	fFourCorner	27720
U+37E0.0	fHakka	 gong1
U+37E0.0	fCantonese	gong1
U+37E0.0	fMandarin	gang1; gang3
U+37E0.0	fEnglish	same as &#x5CA1;
U+37E1.0	fUTF8	㟡
U+37E1.0	fR/S	 46.8
U+37E1.0	fTotalStrokes	 11
U+37E1.0	fCangjie	UFQU
U+37E1.0	fCantonese	kwai5
U+37E1.0	fEnglish	[Japanese character]
U+37E2.0	fUTF8	㟢
U+37E2.0	fCNS11643	F-343F
U+37E2.0	fR/S	 46.8
U+37E2.0	fTotalStrokes	 11
U+37E2.0	fCangjie	UKMR
U+37E2.0	fCantonese	kei1; kei4
U+37E2.0	fMandarin	qi2
U+37E2.0	fEnglish	the rocks on the hills are of different shapes; rugged and weird, character different from others
U+37E3.0	fUTF8	㟣
U+37E3.0	fR/S	 46.8
U+37E3.0	fTotalStrokes	 11
U+37E3.0	fCangjie	UDCI
U+37E3.0	fCantonese	sung1
U+37E4.0	fUTF8	㟤
U+37E4.0	fGB	3-392F
U+37E4.0	fR/S	 46.8
U+37E4.0	fTotalStrokes	 11
U+37E4.0	fCangjie	UNME
U+37E4.0	fCantonese	luk6
U+37E4.0	fMandarin	lu4
U+37E4.0	fEnglish	[1] plain between mountains [2] Zhuang (a minority ethnic group) in Guangxi [3] said of the level ground
U+37E5.0	fUTF8	㟥
U+37E5.0	fCNS11643	3-3553
U+37E5.0	fR/S	 46.8
U+37E5.0	fTotalStrokes	 11
U+37E5.0	fCangjie	UIKH
U+37E5.0	fFourCorner	22200
U+37E5.0	fCantonese	caam1 (also); cam1
U+37E5.0	fMandarin	cen1
U+37E5.0	fEnglish	simplified form of &#x5D7E;
U+37E6.0	fUTF8	㟦
U+37E6.0	fCNS11643	3-3B2F
U+37E6.0	fGB	5-3B25
U+37E6.0	fR/S	 46.9
U+37E6.0	fTotalStrokes	 12
U+37E6.0	fCangjie	UBJJ
U+37E6.0	fFourCorner	22500
U+37E6.0	fCantonese	wan6
U+37E6.0	fMandarin	yun4
U+37E6.0	fEnglish	same as &#x21F45;
U+37E7.0	fUTF8	㟧
U+37E7.0	fCNS11643	3-3B32
U+37E7.0	fGB	5-3B4B
U+37E7.0	fR/S	 46.9
U+37E7.0	fTotalStrokes	 12
U+37E7.0	fCangjie	URRS
U+37E7.0	fFourCorner	22720
U+37E7.0	fHakka	 ngok8
U+37E7.0	fCantonese	ngok6; ok6 (also); hon6; zim4
U+37E7.0	fMandarin	e4; nie4; xun4
U+37E7.0	fEnglish	same as &#x5D3F;
U+37E8.0	fUTF8	㟨
U+37E8.0	fCNS11643	3-3B29
U+37E8.0	fR/S	 46.9
U+37E8.0	fTotalStrokes	 12
U+37E8.0	fCangjie	UUMB
U+37E8.0	fFourCorner	22720
U+37E8.0	fCantonese	dyun1; cyun4
U+37E8.0	fMandarin	duan1; quan2
U+37E8.0	fEnglish	[1] a fountain in the side of a hill [2] the name of a mountain
U+37E9.0	fUTF8	㟩
U+37E9.0	fCNS11643	4-3732
U+37E9.0	fR/S	 46.9
U+37E9.0	fTotalStrokes	 12
U+37E9.0	fCangjie	RKU
U+37E9.0	fFourCorner	78770
U+37E9.0	fCantonese	man4
U+37E9.0	fMandarin	min2; min3; wen3
U+37E9.0	fEnglish	same as &#x21F10;
U+37EA.0	fUTF8	㟪
U+37EA.0	fCNS11643	4-3734
U+37EA.0	fGB	3-393A
U+37EA.0	fR/S	 46.9
U+37EA.0	fTotalStrokes	 12
U+37EA.0	fCangjie	UWMV
U+37EA.0	fFourCorner	26730
U+37EA.0	fCantonese	wai3; waai1
U+37EA.0	fMandarin	wei3
U+37EA.0	fEnglish	[1] the name of a mountain [2] lofty and steep; high
U+37EB.0	fUTF8	㟫
U+37EB.0	fCNS11643	3-3B35
U+37EB.0	fR/S	 46.9
U+37EB.0	fTotalStrokes	 12
U+37EB.0	fCangjie	UHAE
U+37EB.0	fFourCorner	26730
U+37EB.0	fHakka	 can2
U+37EB.0	fCantonese	cyun4
U+37EB.0	fMandarin	quan2; juan4
U+37EB.0	fEnglish	same as &#x6CC9;
U+37EC.0	fUTF8	㟬
U+37EC.0	fCNS11643	4-3C72
U+37EC.0	fGB	3-393D
U+37EC.0	fR/S	 46.9; 46.10
U+37EC.0	fTotalStrokes	 13; 12
U+37EC.0	fCangjie	UHXE
U+37EC.0	fFourCorner	22400
U+37EC.0	fCantonese	sau2; saau2; sou2
U+37EC.0	fMandarin	sou1; shu3; sou3
U+37EC.0	fEnglish	a mountain in today's Shandong province southwest of Mengyin county
U+37ED.0	fUTF8	㟭
U+37ED.0	fCNS11643	3-3B30
U+37ED.0	fGB	5-3B54
U+37ED.0	fR/S	 46.9
U+37ED.0	fTotalStrokes	 12
U+37ED.0	fCangjie	URPA
U+37ED.0	fFourCorner	27760
U+37ED.0	fHakka	 min2
U+37ED.0	fCantonese	man4
U+37ED.0	fMandarin	min2
U+37ED.0	fEnglish	same as &#x5CB7;
U+37EE.0	fUTF8	㟮
U+37EE.0	fR/S	 46.9
U+37EE.0	fTotalStrokes	 12
U+37EE.0	fCangjie	UJCK
U+37EE.0	fCantonese	dat6; daat6; daau6
U+37EE.0	fEnglish	lofty
U+37EF.0	fUTF8	㟯
U+37EF.0	fCNS11643	3-3B2C
U+37EF.0	fR/S	 46.9
U+37EF.0	fTotalStrokes	 12
U+37EF.0	fCangjie	UJHR
U+37EF.0	fFourCorner	22600
U+37EF.0	fCantonese	ngaak6; aak6 (also)
U+37F0.0	fUTF8	㟰
U+37F0.0	fCNS11643	4-3C76
U+37F0.0	fGB	5-3B64
U+37F0.0	fR/S	 46.10
U+37F0.0	fTotalStrokes	 13
U+37F0.0	fCangjie	UBAC
U+37F0.0	fFourCorner	27780
U+37F0.0	fCantonese	ming5
U+37F0.0	fMandarin	ming3
U+37F0.0	fEnglish	[1] the name of a mountain [2] lofty; steep
U+37F1.0	fUTF8	㟱
U+37F1.0	fCNS11643	4-3C73
U+37F1.0	fGB	5-3B63
U+37F1.0	fR/S	 46.10
U+37F1.0	fTotalStrokes	 13
U+37F1.0	fCangjie	UTGF
U+37F1.0	fFourCorner	28730
U+37F1.0	fCantonese	jiu5; ngau5; ngo4; paau4; zip6
U+37F1.0	fMandarin	yao3
U+37F1.0	fEnglish	shape of a mountain
U+37F2.0	fUTF8	㟲
U+37F2.0	fCNS11643	4-3C7E
U+37F2.0	fGB	3-3949
U+37F2.0	fR/S	 46.10
U+37F2.0	fTotalStrokes	 13
U+37F2.0	fCangjie	UMHF
U+37F2.0	fFourCorner	21790
U+37F2.0	fCantonese	jyun4; zyun4
U+37F2.0	fMandarin	yuan2; jue2
U+37F2.0	fEnglish	mountain peak; mountain top; summit
U+37F3.0	fUTF8	㟳
U+37F3.0	fCNS11643	5-3E6D
U+37F3.0	fR/S	 46.10
U+37F3.0	fTotalStrokes	 13
U+37F3.0	fCangjie	UMWD
U+37F3.0	fFourCorner	21790
U+37F3.0	fCantonese	leot6; gak3; ham2
U+37F3.0	fMandarin	li4
U+37F3.0	fEnglish	the name of a mountain
U+37F4.0	fUTF8	㟴
U+37F4.0	fCNS11643	4-3D21
U+37F4.0	fGB	5-3B4F
U+37F4.0	fR/S	 46.10
U+37F4.0	fTotalStrokes	 13
U+37F4.0	fCangjie	UHUI
U+37F4.0	fFourCorner	26710
U+37F4.0	fCantonese	faai3; fai3; fan4; fan6
U+37F4.0	fMandarin	wei2; kuai4; kui3; wei3
U+37F4.0	fEnglish	[1] lump on the hills [2] rugged; rough; rough [3] lofty; steep [4] the dejection one feels at heart
U+37F5.0	fUTF8	㟵
U+37F5.0	fCNS11643	6-5234
U+37F5.0	fR/S	 46.10
U+37F5.0	fTotalStrokes	 13
U+37F5.0	fCangjie	UWLM
U+37F5.0	fFourCorner	22100
U+37F5.0	fHakka	 gong1
U+37F5.0	fCantonese	gong1
U+37F5.0	fMandarin	gang1; gang3
U+37F5.0	fEnglish	non-classical form of &#x5CA1;
U+37F6.0	fUTF8	㟶
U+37F6.0	fCNS11643	3-407B
U+37F6.0	fGB	5-3B5A
U+37F6.0	fR/S	 46.10
U+37F6.0	fTotalStrokes	 13
U+37F6.0	fCangjie	UMHF
U+37F6.0	fFourCorner	22290
U+37F6.0	fCantonese	jyun4; zyun4
U+37F6.0	fMandarin	yuan2
U+37F6.0	fEnglish	same as &#x37F2;
U+37F7.0	fUTF8	㟷
U+37F7.0	fGB	3-3935
U+37F7.0	fR/S	 46.10
U+37F7.0	fTotalStrokes	 13
U+37F7.0	fCangjie	UTOR
U+37F7.0	fCantonese	daat3; dap3
U+37F7.0	fMandarin	da5
U+37F7.0	fEnglish	wart; pustule; pimple, round lump
U+37F8.0	fUTF8	㟸
U+37F8.0	fCNS11643	4-3C77
U+37F8.0	fGB	3-394B
U+37F8.0	fR/S	 46.10
U+37F8.0	fTotalStrokes	 13
U+37F8.0	fCangjie	UHAJ
U+37F8.0	fFourCorner	26740
U+37F8.0	fCantonese	gou1
U+37F8.0	fMandarin	nie4; gao1
U+37F8.0	fEnglish	standard form of &#x21F17;
U+37F9.0	fUTF8	㟹
U+37F9.0	fCNS11643	4-4322
U+37F9.0	fGB	3-394F
U+37F9.0	fR/S	 46.11
U+37F9.0	fTotalStrokes	 14
U+37F9.0	fCangjie	USJL
U+37F9.0	fFourCorner	27720
U+37F9.0	fCantonese	lou4; liu4
U+37F9.0	fMandarin	liao2; lao2
U+37F9.0	fEnglish	the appearance of a mountain is lofty and steep and profound
U+37FA.0	fUTF8	㟺
U+37FA.0	fCNS11643	6-5C44
U+37FA.0	fR/S	 46.11
U+37FA.0	fTotalStrokes	 14
U+37FA.0	fCangjie	ULLV
U+37FA.0	fFourCorner	22400
U+37FA.0	fHakka	 leu3
U+37FA.0	fCantonese	lau5; laau4
U+37FA.0	fMandarin	luu2; lou3; lou2; lou4
U+37FA.0	fEnglish	same as &#x5D81;
U+37FB.0	fUTF8	㟻
U+37FB.0	fCNS11643	4-432D
U+37FB.0	fGB	5-3B3B
U+37FB.0	fR/S	 46.11
U+37FB.0	fTotalStrokes	 14
U+37FB.0	fCangjie	JLU
U+37FB.0	fFourCorner	52770
U+37FB.0	fHakka	 ciam5
U+37FB.0	fCantonese	cim3; ding1; zam2
U+37FB.0	fMandarin	qian4
U+37FB.0	fEnglish	same as &#x5879;
U+37FC.0	fUTF8	㟼
U+37FC.0	fCNS11643	4-432A
U+37FC.0	fGB	3-3947
U+37FC.0	fR/S	 46.11
U+37FC.0	fTotalStrokes	 14
U+37FC.0	fCangjie	UGSK
U+37FC.0	fFourCorner	22240
U+37FC.0	fCantonese	ngou6; ou6 (also); ngou4
U+37FC.0	fMandarin	ao2
U+37FC.0	fEnglish	same as &#x5D85;
U+37FD.0	fUTF8	㟽
U+37FD.0	fCNS11643	4-4321
U+37FD.0	fGB	3-3952
U+37FD.0	fR/S	 46.11
U+37FD.0	fTotalStrokes	 14
U+37FD.0	fCangjie	UMWF
U+37FD.0	fFourCorner	21790
U+37FD.0	fHakka	 biau3; beu3
U+37FD.0	fCantonese	biu2
U+37FD.0	fMandarin	biao3
U+37FD.0	fEnglish	a high mountain peak
U+37FE.0	fUTF8	㟾
U+37FE.0	fCNS11643	3-4655
U+37FE.0	fR/S	 46.11
U+37FE.0	fTotalStrokes	 14
U+37FE.0	fCangjie	UILB
U+37FE.0	fFourCorner	20720
U+37FE.0	fCantonese	jung4; zung4
U+37FE.0	fMandarin	yong2
U+37FE.0	fEnglish	the name of a mountain in Jian county
U+37FF.0	fUTF8	㟿
U+37FF.0	fCNS11643	4-4324
U+37FF.0	fGB	3-3948
U+37FF.0	fR/S	 46.11
U+37FF.0	fTotalStrokes	 14
U+37FF.0	fCangjie	UTIT
U+37FF.0	fFourCorner	24740
U+37FF.0	fCantonese	mang3; mong5
U+37FF.0	fMandarin	mang3; mang2
U+37FF.0	fEnglish	[1] high and steep [2] the name of a mountain
U+3800.0	fUTF8	㠀
U+3800.0	fCNS11643	3-465B
U+3800.0	fR/S	 46.11
U+3800.0	fTotalStrokes	 14
U+3800.0	fCangjie	HFU
U+3800.0	fFourCorner	27770
U+3800.0	fHakka	 dau3
U+3800.0	fCantonese	dou2
U+3800.0	fMandarin	dao3
U+3800.0	fEnglish	standard form of &#x5CF6;
U+3801.0	fUTF8	㠁
U+3801.0	fCNS11643	3-4658
U+3801.0	fGB	5-3B44
U+3801.0	fR/S	 46.11
U+3801.0	fTotalStrokes	 14
U+3801.0	fCangjie	UIIH
U+3801.0	fFourCorner	23720
U+3801.0	fHakka	 cam1
U+3801.0	fCantonese	caam1 (also); cam1
U+3801.0	fMandarin	cen1
U+3801.0	fEnglish	[1] irregular [2] same as &#x5D7E;
U+3802.0	fUTF8	㠂
U+3802.0	fCNS11643	4-432E
U+3802.0	fGB	3-3946
U+3802.0	fR/S	 46.11
U+3802.0	fTotalStrokes	 14
U+3802.0	fCangjie	UGSK
U+3802.0	fFourCorner	28740
U+3802.0	fCantonese	ngou6; ou6 (also); ngou4
U+3802.0	fMandarin	ao2
U+3802.0	fEnglish	same as &#x5D85;
U+3803.0	fUTF8	㠃
U+3803.0	fR/S	 46.11
U+3803.0	fTotalStrokes	 14
U+3803.0	fCangjie	UYCB
U+3803.0	fCantonese	dik1
U+3804.0	fUTF8	㠄
U+3804.0	fCNS11643	3-4657
U+3804.0	fGB	5-3B68
U+3804.0	fR/S	 46.11
U+3804.0	fTotalStrokes	 14
U+3804.0	fCangjie	USMA
U+3804.0	fFourCorner	22600
U+3804.0	fHakka	 sip8
U+3804.0	fCantonese	zaap6
U+3804.0	fMandarin	xi2
U+3804.0	fEnglish	a mountain in Yunnan
U+3805.0	fUTF8	㠅
U+3805.0	fCNS11643	4-4950
U+3805.0	fGB	3-395F
U+3805.0	fR/S	 46.12
U+3805.0	fTotalStrokes	 15
U+3805.0	fCangjie	UHOE
U+3805.0	fFourCorner	22240
U+3805.0	fCantonese	fuk6; fuk1
U+3805.0	fMandarin	fu4; fu2
U+3805.0	fEnglish	[1] cover; put a cover over something [2] the name of a mountain
U+3806.0	fUTF8	㠆
U+3806.0	fCNS11643	4-494C
U+3806.0	fGB	3-392E
U+3806.0	fR/S	 46.12
U+3806.0	fTotalStrokes	 15
U+3806.0	fCangjie	URRJ
U+3806.0	fFourCorner	26750
U+3806.0	fCantonese	daan1
U+3806.0	fMandarin	dan1
U+3806.0	fEnglish	the name of a mountain
U+3807.0	fUTF8	㠇
U+3807.0	fCNS11643	5-4C6C
U+3807.0	fGB	3-3961
U+3807.0	fR/S	 46.12
U+3807.0	fTotalStrokes	 15
U+3807.0	fCangjie	UYFU
U+3807.0	fFourCorner	22910
U+3807.0	fCantonese	zau6
U+3807.0	fMandarin	jiu4
U+3807.0	fEnglish	the name of a mountain ridge
U+3808.0	fUTF8	㠈
U+3808.0	fCNS11643	3-4B62
U+3808.0	fR/S	 46.12
U+3808.0	fTotalStrokes	 15
U+3808.0	fCangjie	UANG
U+3808.0	fFourCorner	27720
U+3808.0	fCantonese	jeon6; zeon6
U+3808.0	fMandarin	run4
U+3808.0	fEnglish	the name of a place
U+3809.0	fUTF8	㠉
U+3809.0	fCNS11643	5-4C68
U+3809.0	fGB	5-3B72
U+3809.0	fR/S	 46.12
U+3809.0	fTotalStrokes	 15
U+3809.0	fCangjie	UYTG
U+3809.0	fFourCorner	20710
U+3809.0	fCantonese	tung4; hiu1
U+3809.0	fMandarin	tong2
U+3809.0	fEnglish	[1] mountain; hill [2] bald hills [3] valley; ravine
U+380A.0	fUTF8	㠊
U+380A.0	fCNS11643	4-494B
U+380A.0	fGB	5-3B65
U+380A.0	fR/S	 46.12
U+380A.0	fTotalStrokes	 15
U+380A.0	fCangjie	UYPM
U+380A.0	fFourCorner	21710
U+380A.0	fHakka	 hi1;  ki1
U+380A.0	fCantonese	heoi1; keoi1
U+380A.0	fMandarin	qu1
U+380A.0	fEnglish	same as &#x5D87;
U+380B.0	fUTF8	㠋
U+380B.0	fCNS11643	4-4953
U+380B.0	fGB	5-3B6D
U+380B.0	fR/S	 46.12
U+380B.0	fTotalStrokes	 15
U+380B.0	fCangjie	URRS
U+380B.0	fFourCorner	22720
U+380B.0	fCantonese	ngok6; ok6 (also)
U+380B.0	fMandarin	e4
U+380B.0	fEnglish	same as &#x5D3F;
U+380C.0	fUTF8	㠌
U+380C.0	fCNS11643	3-4B64
U+380C.0	fGB	5-3B6A
U+380C.0	fR/S	 46.12
U+380C.0	fTotalStrokes	 15
U+380C.0	fCangjie	UTCO
U+380C.0	fFourCorner	22880
U+380C.0	fCantonese	kai1; hei1
U+380C.0	fMandarin	qi1
U+380C.0	fEnglish	[1] high; lofty; steep [2] corrupted form of &#x5D4C;
U+380D.0	fUTF8	㠍
U+380D.0	fCNS11643	4-494F
U+380D.0	fGB	5-3B6E
U+380D.0	fR/S	 46.12
U+380D.0	fTotalStrokes	 15
U+380D.0	fCangjie	UOGD
U+380D.0	fFourCorner	22900
U+380D.0	fCantonese	cap1; caap1; zap6; zit6
U+380D.0	fMandarin	ji4; ji2; jie2; qi4
U+380D.0	fEnglish	[1] lofty; majestic [2] the name of a mountain
U+380E.0	fUTF8	㠎
U+380E.0	fCNS11643	4-494D
U+380E.0	fGB	3-395D
U+380E.0	fR/S	 46.12
U+380E.0	fTotalStrokes	 15
U+380E.0	fCangjie	UOGD
U+380E.0	fFourCorner	20790
U+380E.0	fCantonese	zaap6
U+380E.0	fMandarin	za2; ji2; qi4
U+380E.0	fEnglish	same as &#x380D;
U+380F.0	fUTF8	㠏
U+380F.0	fCNS11643	3-4B61
U+380F.0	fGB	5-3A7A
U+380F.0	fR/S	 46.12
U+380F.0	fTotalStrokes	 15
U+380F.0	fCangjie	UTMJ
U+380F.0	fFourCorner	24750
U+380F.0	fCantonese	waa4; gwaai3; wat6
U+380F.0	fMandarin	hua2
U+380F.0	fEnglish	the name of a mountain
U+3810.0	fUTF8	㠐
U+3810.0	fCNS11643	4-4954
U+3810.0	fR/S	 46.12
U+3810.0	fTotalStrokes	 15
U+3810.0	fCangjie	UHKB
U+3810.0	fFourCorner	22220
U+3810.0	fHakka	 kiau2; kiau5
U+3810.0	fCantonese	giu6; kiu4; caa4
U+3810.0	fMandarin	jiao4; qiao2
U+3810.0	fEnglish	same as &#x5DA0;
U+3811.0	fUTF8	㠑
U+3811.0	fCNS11643	4-5026
U+3811.0	fGB	3-3969
U+3811.0	fR/S	 46.13
U+3811.0	fTotalStrokes	 16
U+3811.0	fCangjie	UWLY
U+3811.0	fFourCorner	22110
U+3811.0	fHakka	 cui5; cui6
U+3811.0	fCantonese	zeoi6
U+3811.0	fMandarin	zui4
U+3811.0	fEnglish	mountainous; rugged
U+3812.0	fUTF8	㠒
U+3812.0	fCNS11643	4-5027
U+3812.0	fGB	3-3968
U+3812.0	fR/S	 46.13
U+3812.0	fTotalStrokes	 16
U+3812.0	fCangjie	UMFN
U+3812.0	fFourCorner	22700
U+3812.0	fCantonese	biu2
U+3812.0	fMandarin	biao1; biao3
U+3812.0	fEnglish	same as &#x37FD;
U+3813.0	fUTF8	㠓
U+3813.0	fCNS11643	4-5628
U+3813.0	fGB	3-3966
U+3813.0	fR/S	 46.14; 46.13
U+3813.0	fTotalStrokes	 16; 17
U+3813.0	fCangjie	UTBO
U+3813.0	fFourCorner	24730
U+3813.0	fCantonese	mung4
U+3813.0	fMandarin	meng2
U+3813.0	fEnglish	the name of a mountain
U+3814.0	fUTF8	㠔
U+3814.0	fCNS11643	4-5025
U+3814.0	fGB	3-396D
U+3814.0	fR/S	 46.13
U+3814.0	fTotalStrokes	 16
U+3814.0	fCangjie	USRJ
U+3814.0	fFourCorner	22240
U+3814.0	fCantonese	baai3; bai3; be6
U+3814.0	fMandarin	bai4
U+3814.0	fEnglish	[1] shapes of the mountain [2] the strategic and dangious gorge [3] field in between of the valleys
U+3815.0	fUTF8	㠕
U+3815.0	fCNS11643	5-5436
U+3815.0	fR/S	 46.13
U+3815.0	fTotalStrokes	 16
U+3815.0	fCangjie	UNLI
U+3815.0	fFourCorner	22210
U+3815.0	fHakka	 ngui2
U+3815.0	fCantonese	ngai4; ai4 (also); waai5
U+3815.0	fMandarin	wei2; wei3; wei1
U+3815.0	fEnglish	same as &#x5D6C;
U+3816.0	fUTF8	㠖
U+3816.0	fCNS11643	3-5059
U+3816.0	fR/S	 46.13
U+3816.0	fTotalStrokes	 16
U+3816.0	fCangjie	UTGI
U+3816.0	fFourCorner	22550
U+3816.0	fCantonese	ngai5; ai5 (also); zi4; zi6
U+3816.0	fMandarin	yi2; ji4; yi3
U+3816.0	fEnglish	same as &#x5DAC;
U+3817.0	fUTF8	㠗
U+3817.0	fCNS11643	7-2D4C
U+3817.0	fR/S	 46.13
U+3817.0	fTotalStrokes	 16
U+3817.0	fCangjie	UHBK
U+3817.0	fFourCorner	27730
U+3817.0	fCantonese	ou3; ngou3 (also); bei1
U+3817.0	fMandarin	ao4; wo4
U+3817.0	fEnglish	same as &#x5DB4;
U+3818.0	fUTF8	㠘
U+3818.0	fCNS11643	3-545A
U+3818.0	fR/S	 46.14
U+3818.0	fTotalStrokes	 17
U+3818.0	fCangjie	UHXC
U+3818.0	fFourCorner	22800
U+3818.0	fHakka	 sii5; sii6; ri3; ji3
U+3818.0	fCantonese	zeoi6
U+3818.0	fMandarin	yu3
U+3818.0	fEnglish	 island
U+3819.0	fUTF8	㠙
U+3819.0	fCNS11643	3-5456
U+3819.0	fGB	S-2125
U+3819.0	fR/S	 46.14
U+3819.0	fTotalStrokes	 17
U+3819.0	fCangjie	UYRO
U+3819.0	fFourCorner	20730
U+3819.0	fHakka	 ngau2
U+3819.0	fCantonese	hou4; ngaau4
U+3819.0	fMandarin	hao2; yao2; xiao2
U+3819.0	fEnglish	same as &#x5D24;
U+381A.0	fUTF8	㠚
U+381A.0	fCNS11643	5-5B3C
U+381A.0	fGB	3-385E
U+381A.0	fR/S	 46.14
U+381A.0	fTotalStrokes	 17
U+381A.0	fCangjie	UTGI
U+381A.0	fFourCorner	24700
U+381A.0	fCantonese	deoi6; deoi3; zeoi3
U+381A.0	fMandarin	dui4; zhuo2
U+381A.0	fEnglish	high and lofty
U+381B.0	fUTF8	㠛
U+381B.0	fCNS11643	4-5627
U+381B.0	fGB	3-3965
U+381B.0	fR/S	 46.14
U+381B.0	fTotalStrokes	 17; 16
U+381B.0	fCangjie	UTOE
U+381B.0	fFourCorner	24740
U+381B.0	fCantonese	waak6
U+381B.0	fMandarin	wo4
U+381B.0	fEnglish	the name a county
U+381C.0	fUTF8	㠜
U+381C.0	fCNS11643	3-5458
U+381C.0	fR/S	 46.14
U+381C.0	fTotalStrokes	 17
U+381C.0	fCangjie	UPKO
U+381C.0	fFourCorner	27780
U+381C.0	fCantonese	jik6; zik6
U+381C.0	fMandarin	yi2; ni4
U+381C.0	fEnglish	[1] same as &#x5DB7; [2] range of mountains in Hunan province
U+381D.0	fUTF8	㠝
U+381D.0	fCNS11643	7-3F44
U+381D.0	fR/S	 46.15
U+381D.0	fTotalStrokes	 18
U+381D.0	fCangjie	UQOC
U+381D.0	fFourCorner	25780
U+381D.0	fHakka	 zon1
U+381D.0	fCantonese	cyun4; zan1
U+381D.0	fMandarin	cuan2
U+381D.0	fEnglish	non-classical form of &#x5DD1;
U+381E.0	fUTF8	㠞
U+381E.0	fR/S	 46.15
U+381E.0	fTotalStrokes	 18
U+381E.0	fCangjie	HKU
U+381E.0	fCantonese	zing1
U+381F.0	fUTF8	㠟
U+381F.0	fR/S	 46.15
U+381F.0	fTotalStrokes	 18
U+381F.0	fCangjie	UHHE
U+381F.0	fCantonese	lai4; laai4
U+381F.0	fMandarin	li2
U+381F.0	fEnglish	a mountain in ancient times; Lieshan in Guizhou province
U+3820.0	fUTF8	㠠
U+3820.0	fCNS11643	4-5F5A
U+3820.0	fGB	3-3853
U+3820.0	fR/S	 46.16
U+3820.0	fTotalStrokes	 19
U+3820.0	fCangjie	UYPT
U+3820.0	fFourCorner	21710
U+3820.0	fCantonese	leoi4; lou4
U+3820.0	fMandarin	lu2
U+3820.0	fEnglish	the name of a mountain
U+3821.0	fUTF8	㠡
U+3821.0	fCNS11643	5-6763
U+3821.0	fGB	3-3944
U+3821.0	fR/S	 46.16
U+3821.0	fTotalStrokes	 19
U+3821.0	fCangjie	UYSV
U+3821.0	fFourCorner	20730
U+3821.0	fCantonese	niu5; liu5 (also); muk6
U+3821.0	fMandarin	niao3
U+3821.0	fEnglish	high and lofty
U+3822.0	fUTF8	㠢
U+3822.0	fCNS11643	5-6764
U+3822.0	fGB	3-3971
U+3822.0	fR/S	 46.16
U+3822.0	fTotalStrokes	 19
U+3822.0	fCangjie	UHIV
U+3822.0	fFourCorner	22730
U+3822.0	fCantonese	waai4; wai4; wik6
U+3822.0	fMandarin	huai2; hua4; huan4; mo4
U+3822.0	fEnglish	[1] rugged and uneven [2] shrink; recoil; cringe
U+3823.0	fUTF8	㠣
U+3823.0	fGB	3-3844
U+3823.0	fR/S	 46.16
U+3823.0	fTotalStrokes	 19
U+3823.0	fCangjie	UMHM
U+3823.0	fCantonese	lik6
U+3823.0	fMandarin	lai4; li4
U+3823.0	fEnglish	a mountain in Jiangsi province
U+3824.0	fUTF8	㠤
U+3824.0	fGB	S-2121
U+3824.0	fR/S	 46.17
U+3824.0	fTotalStrokes	 20
U+3824.0	fCangjie	UYRV
U+3824.0	fCantonese	soeng1
U+3825.0	fUTF8	㠥
U+3825.0	fCNS11643	4-663A
U+3825.0	fGB	3-3976
U+3825.0	fR/S	 46.18
U+3825.0	fTotalStrokes	 21
U+3825.0	fCangjie	UWWG
U+3825.0	fFourCorner	26710
U+3825.0	fCantonese	leot6; leoi6
U+3825.0	fMandarin	lei3; l4
U+3825.0	fEnglish	[1] rugged and uneven [2] high and lofty [3] roll stones down a hill [4] piles of stones
U+3826.0	fUTF8	㠦
U+3826.0	fCNS11643	3-5E2C
U+3826.0	fR/S	 46.18
U+3826.0	fTotalStrokes	 21
U+3826.0	fCangjie	UUJT
U+3826.0	fFourCorner	25718
U+3826.0	fCantonese	fung1; gaam3; haat6
U+3826.0	fMandarin	feng1
U+3826.0	fEnglish	the name of a mountain
U+3827.0	fUTF8	㠧
U+3827.0	fCNS11643	4-6868
U+3827.0	fGB	3-3977
U+3827.0	fR/S	 46.19
U+3827.0	fTotalStrokes	 22; 21
U+3827.0	fCangjie	UIDY
U+3827.0	fFourCorner	20710
U+3827.0	fCantonese	mei5
U+3827.0	fMandarin	mi3; mi2
U+3827.0	fEnglish	[1] a little slanted; smooth [2] the name of a mountain
U+3828.0	fUTF8	㠨
U+3828.0	fCNS11643	5-7B3D
U+3828.0	fR/S	 46.25
U+3828.0	fTotalStrokes	 28
U+3828.0	fCangjie	UDDI
U+3828.0	fFourCorner	24740
U+3828.0	fCantonese	wat1; gwaat1; waat1
U+3828.0	fMandarin	yu4
U+3828.0	fEnglish	misty mountain
U+3829.0	fUTF8	㠩
U+3829.0	fCNS11643	3-235B
U+3829.0	fR/S	 47.3
U+3829.0	fTotalStrokes	 6
U+3829.0	fCangjie	OVHU
U+3829.0	fFourCorner	28210
U+3829.0	fCantonese	fong1
U+3829.0	fMandarin	huang1
U+3829.0	fEnglish	same as &#x8292;, &#x8352;
U+382A.0	fUTF8	㠪
U+382A.0	fCNS11643	6-216E
U+382A.0	fR/S	 48.1
U+382A.0	fTotalStrokes	 4
U+382A.0	fCangjie	MYM
U+382A.0	fFourCorner	10100
U+382A.0	fHakka	 ki3; ki1; gi3
U+382A.0	fCantonese	geoi6
U+382A.0	fMandarin	ju4
U+382A.0	fEnglish	ancient form of &#x5DE8;
U+382B.0	fUTF8	㠫
U+382B.0	fR/S	 48.7
U+382B.0	fTotalStrokes	 10
U+382B.0	fCangjie	MNQO
U+382B.0	fCantonese	faan4
U+382C.0	fUTF8	㠬
U+382C.0	fR/S	 48.7
U+382C.0	fTotalStrokes	 10
U+382C.0	fCangjie	MRQO
U+382C.0	fCantonese	hau2
U+382D.0	fUTF8	㠭
U+382D.0	fCNS11643	3-3B38
U+382D.0	fGB	5-3551
U+382D.0	fR/S	 48.9
U+382D.0	fTotalStrokes	 12
U+382D.0	fCangjie	MMMM
U+382D.0	fFourCorner	11110
U+382D.0	fCantonese	zin2
U+382D.0	fMandarin	zhan3; zhan4
U+382D.0	fEnglish	[1] open [2] stretch; extend [3] unfold; dilate; prolong
U+382E.0	fUTF8	㠮
U+382E.0	fCNS11643	4-4955
U+382E.0	fR/S	 48.12
U+382E.0	fTotalStrokes	 15
U+382E.0	fCangjie	TAM
U+382E.0	fFourCorner	81610
U+382E.0	fCantonese	pung1; pang1
U+382E.0	fMandarin	peng2
U+382E.0	fEnglish	[1] expand [2] swell [3] non-classical form of &#x809B;
U+382F.0	fUTF8	㠯
U+382F.0	fCNS11643	3-2245
U+382F.0	fR/S	 49.2; 49.1
U+382F.0	fTotalStrokes	 5; 4
U+382F.0	fCangjie	SLR
U+382F.0	fFourCorner	17770
U+382F.0	fHakka	 ji1
U+382F.0	fCantonese	ji5; zyu5; jyu5
U+382F.0	fMandarin	yi3
U+382F.0	fEnglish	standard form of &#x4EE5;
U+3830.0	fUTF8	㠰
U+3830.0	fR/S	 49.5
U+3830.0	fTotalStrokes	 8
U+3830.0	fCangjie	KRSU
U+3830.0	fCantonese	gaa1
U+3831.0	fUTF8	㠱
U+3831.0	fCNS11643	4-3142
U+3831.0	fGB	5-3E72
U+3831.0	fR/S	 49.8
U+3831.0	fTotalStrokes	 11
U+3831.0	fCangjie	SUTMC
U+3831.0	fFourCorner	17800
U+3831.0	fCantonese	gei6
U+3831.0	fMandarin	ji4; qi3
U+3831.0	fEnglish	[1] standard form of &#x8E1E; [2] ancient form of &#x7B95; [3] same as &#x675E;
U+3832.0	fUTF8	㠲
U+3832.0	fCNS11643	3-2246
U+3832.0	fGB	5-3A22
U+3832.0	fR/S	 50.2
U+3832.0	fTotalStrokes	 5
U+3832.0	fCangjie	LBP
U+3832.0	fFourCorner	42210
U+3832.0	fHakka	 bi3
U+3832.0	fCantonese	bei2; baat6; bok3
U+3832.0	fMandarin	bi3
U+3832.0	fEnglish	ripped; split
U+3833.0	fUTF8	㠳
U+3833.0	fR/S	 50.2
U+3833.0	fTotalStrokes	 5
U+3833.0	fCangjie	OLB
U+3833.0	fCantonese	jap6
U+3834.0	fUTF8	㠴
U+3834.0	fCNS11643	4-2265
U+3834.0	fGB	3-3761
U+3834.0	fR/S	 50.3
U+3834.0	fTotalStrokes	 6
U+3834.0	fCangjie	LBSHI
U+3834.0	fFourCorner	47220
U+3834.0	fCantonese	jan6; zaan6
U+3834.0	fMandarin	ren4
U+3834.0	fEnglish	 pillow case
U+3835.0	fUTF8	㠵
U+3835.0	fCNS11643	4-2266
U+3835.0	fGB	3-3760
U+3835.0	fR/S	 50.3
U+3835.0	fTotalStrokes	 6
U+3835.0	fCangjie	YVLB
U+3835.0	fFourCorner	00220
U+3835.0	fCantonese	fong1
U+3835.0	fMandarin	huang1
U+3835.0	fEnglish	[1] turban; headdress; scarf [2] piece of cloth (fabric) used to cover something
U+3836.0	fUTF8	㠶
U+3836.0	fCNS11643	3-235D
U+3836.0	fR/S	 50.3
U+3836.0	fTotalStrokes	 6
U+3836.0	fCangjie	LBHHN
U+3836.0	fFourCorner	43210
U+3836.0	fHakka	  fam2
U+3836.0	fCantonese	faan4
U+3836.0	fMandarin	fan2
U+3836.0	fEnglish	variant of &#x5E06;
U+3837.0	fUTF8	㠷
U+3837.0	fCNS11643	4-2422
U+3837.0	fGB	3-375F
U+3837.0	fR/S	 50.4
U+3837.0	fTotalStrokes	 7
U+3837.0	fCangjie	LBNHE
U+3837.0	fFourCorner	47240
U+3837.0	fCantonese	gap3; gok3; hon6; kaau3
U+3837.0	fMandarin	ge2
U+3837.0	fEnglish	[1] vessel for rice [2] mats on carritage
U+3838.0	fUTF8	㠸
U+3838.0	fCNS11643	4-2424
U+3838.0	fGB	3-3762
U+3838.0	fR/S	 50.4
U+3838.0	fTotalStrokes	 7
U+3838.0	fCangjie	LBQO
U+3838.0	fFourCorner	45230
U+3838.0	fCantonese	fu1; fu3; gu1
U+3838.0	fMandarin	fu1; ku4
U+3838.0	fEnglish	[1] same as &#x886D; [2] &#x8932;
U+3839.0	fUTF8	㠹
U+3839.0	fCNS11643	3-2569
U+3839.0	fGB	5-3A25
U+3839.0	fR/S	 50.4
U+3839.0	fTotalStrokes	 7
U+3839.0	fCangjie	LBOLL
U+3839.0	fFourCorner	48220
U+3839.0	fCantonese	gaai3
U+3839.0	fMandarin	jie4
U+3839.0	fEnglish	 turban; headdress a scarf
U+383A.0	fUTF8	㠺
U+383A.0	fCNS11643	4-2421
U+383A.0	fGB	3-3764
U+383A.0	fR/S	 50.4
U+383A.0	fTotalStrokes	 7
U+383A.0	fCangjie	LBFH
U+383A.0	fFourCorner	49220
U+383A.0	fHakka	 sa1
U+383A.0	fCantonese	saa1
U+383A.0	fMandarin	sha1; miao2
U+383A.0	fEnglish	[1] fine thread [2] a net with small, tiny holes
U+383B.0	fUTF8	㠻
U+383B.0	fCNS11643	3-256A
U+383B.0	fR/S	 50.4
U+383B.0	fTotalStrokes	 7
U+383B.0	fCangjie	JKLB
U+383B.0	fFourCorner	44220
U+383B.0	fHakka	 hi1
U+383B.0	fCantonese	hei1
U+383B.0	fMandarin	xi1
U+383B.0	fEnglish	same as &#x5E0C;
U+383C.0	fUTF8	㠼
U+383C.0	fCNS11643	F-2425
U+383C.0	fR/S	 50.4
U+383C.0	fTotalStrokes	 7
U+383C.0	fCangjie	LBHML
U+383C.0	fFourCorner	4222
U+383C.0	fCantonese	si1
U+383D.0	fUTF8	㠽
U+383D.0	fCNS11643	4-2636
U+383D.0	fGB	3-3769
U+383D.0	fR/S	 50.5
U+383D.0	fTotalStrokes	 8
U+383D.0	fCangjie	LBHEY
U+383D.0	fFourCorner	47230
U+383D.0	fCantonese	tung4
U+383D.0	fMandarin	tong2
U+383D.0	fEnglish	 dish-cloth; mopper; cleaning rag
U+383E.0	fUTF8	㠾
U+383E.0	fCNS11643	4-2637
U+383E.0	fGB	3-376A
U+383E.0	fR/S	 50.5
U+383E.0	fTotalStrokes	 8
U+383E.0	fCangjie	NULB
U+383E.0	fFourCorner	27220
U+383E.0	fCantonese	jyun1; zyun1; zyun3
U+383E.0	fMandarin	yuan1
U+383E.0	fEnglish	[1] dish-cloth; mopper; cleaning rag [2] headdress; scarf; turban
U+383F.0	fUTF8	㠿
U+383F.0	fCNS11643	4-2925
U+383F.0	fGB	3-376D
U+383F.0	fR/S	 50.5; 50.6
U+383F.0	fTotalStrokes	 9; 8
U+383F.0	fCangjie	YPLB
U+383F.0	fFourCorner	22220
U+383F.0	fCantonese	zi1
U+383F.0	fMandarin	zi1; ci3
U+383F.0	fEnglish	[1] a kind of delicate and fine fabric [2] napkin; kerchief or towel [3] a headwear
U+3840.0	fUTF8	㡀
U+3840.0	fCNS11643	4-2425
U+3840.0	fGB	5-407C
U+3840.0	fR/S	 50.5; 50.4
U+3840.0	fTotalStrokes	 7; 8
U+3840.0	fCangjie	FBF
U+3840.0	fFourCorner	90220
U+3840.0	fCantonese	bai6
U+3840.0	fMandarin	bi4
U+3840.0	fEnglish	[1] ragged clothing [2] ragged; old and worn out
U+3841.0	fUTF8	㡁
U+3841.0	fCNS11643	4-2922
U+3841.0	fGB	5-3A2D
U+3841.0	fR/S	 50.6
U+3841.0	fTotalStrokes	 9
U+3841.0	fCangjie	LBKMS
U+3841.0	fFourCorner	44220
U+3841.0	fHakka	 kwa5; kwa3
U+3841.0	fCantonese	kwaa2; kai2; kwaa1; kwaa3; pong4; tap3
U+3841.0	fMandarin	kua3; kai3; kua4
U+3841.0	fEnglish	leggings for riding
U+3842.0	fUTF8	㡂
U+3842.0	fCNS11643	4-2924
U+3842.0	fGB	3-376C
U+3842.0	fR/S	 50.6
U+3842.0	fTotalStrokes	 9
U+3842.0	fCangjie	MNLB
U+3842.0	fFourCorner	12220
U+3842.0	fCantonese	lai6
U+3842.0	fMandarin	li4
U+3842.0	fEnglish	remains; remnants
U+3843.0	fUTF8	㡃
U+3843.0	fCNS11643	6-3063
U+3843.0	fR/S	 50.6
U+3843.0	fTotalStrokes	 9
U+3843.0	fCangjie	LBOVU
U+3843.0	fFourCorner	48210
U+3843.0	fCantonese	fong1
U+3843.0	fMandarin	huang1
U+3843.0	fEnglish	an artisan to soften raw silk by boiling and to dye the dressed silk
U+3844.0	fUTF8	㡄
U+3844.0	fCNS11643	3-2C47
U+3844.0	fGB	5-3A31
U+3844.0	fR/S	 50.6
U+3844.0	fTotalStrokes	 9
U+3844.0	fCangjie	LBPA
U+3844.0	fFourCorner	47220
U+3844.0	fCantonese	seon1; seon2
U+3844.0	fMandarin	xun1; sun3; xun2
U+3844.0	fEnglish	colar
U+3845.0	fUTF8	㡅
U+3845.0	fCNS11643	4-2923
U+3845.0	fGB	3-376F
U+3845.0	fR/S	 50.6
U+3845.0	fTotalStrokes	 9
U+3845.0	fCangjie	LBNIN
U+3845.0	fFourCorner	47220
U+3845.0	fCantonese	no5; lo5 (also)
U+3845.0	fMandarin	nuo3
U+3845.0	fEnglish	storage room
U+3846.0	fUTF8	㡆
U+3846.0	fCNS11643	4-2921
U+3846.0	fGB	5-3A32
U+3846.0	fR/S	 50.6
U+3846.0	fTotalStrokes	 9
U+3846.0	fCangjie	LBYVU
U+3846.0	fFourCorner	40210
U+3846.0	fCantonese	fong1
U+3846.0	fMandarin	huang1
U+3846.0	fEnglish	same as &#x2843; an artisan to soften raw silk by boiling and to dye the dressed silk
U+3847.0	fUTF8	㡇
U+3847.0	fCNS11643	4-2C66
U+3847.0	fGB	3-3774
U+3847.0	fR/S	 50.7
U+3847.0	fTotalStrokes	 10
U+3847.0	fCangjie	LBSJU
U+3847.0	fFourCorner	41210
U+3847.0	fCantonese	zip3
U+3847.0	fMandarin	zhe2; ji2
U+3847.0	fEnglish	[1] colar [2] the front of a Chinese gown [3] lapel of a Chinese dress worn by the literati in former days
U+3848.0	fUTF8	㡈
U+3848.0	fCNS11643	4-2C68
U+3848.0	fGB	5-3A38
U+3848.0	fR/S	 50.7
U+3848.0	fTotalStrokes	 10
U+3848.0	fCangjie	LBNAU
U+3848.0	fFourCorner	47210
U+3848.0	fCantonese	man6
U+3848.0	fMandarin	wen4; men2
U+3848.0	fEnglish	[1] the ropes attached to the bier and held by mourners [2] mourning garments [3] a ceremonial cap for high ministers in old China
U+3849.0	fUTF8	㡉
U+3849.0	fCNS11643	4-3147
U+3849.0	fGB	3-377E
U+3849.0	fR/S	 50.8
U+3849.0	fTotalStrokes	 11
U+3849.0	fCangjie	LBNYI
U+3849.0	fFourCorner	40230
U+3849.0	fCantonese	gin1; zin4
U+3849.0	fMandarin	xian2; yan2
U+3849.0	fEnglish	[1] a kind of fabric [2] a county in today's Shandong province; southwest of Huangxian
U+384A.0	fUTF8	㡊
U+384A.0	fCNS11643	4-3148
U+384A.0	fGB	5-3A3A
U+384A.0	fR/S	 50.8
U+384A.0	fTotalStrokes	 11
U+384A.0	fCangjie	LBNHX
U+384A.0	fFourCorner	47270
U+384A.0	fCantonese	hap1; haap1; hap6
U+384A.0	fMandarin	qia4; qian3
U+384A.0	fEnglish	same as &#x5E22;
U+384B.0	fUTF8	㡋
U+384B.0	fCNS11643	4-3145
U+384B.0	fGB	3-377A
U+384B.0	fR/S	 50.8
U+384B.0	fTotalStrokes	 11
U+384B.0	fCangjie	LBKLU
U+384B.0	fFourCorner	44210
U+384B.0	fCantonese	jip3; zim1
U+384B.0	fMandarin	ye4; e4; ye2
U+384B.0	fEnglish	[1] a kind of headdress for man in old China [2] a bag used to feed the horses
U+384C.0	fUTF8	㡌
U+384C.0	fCNS11643	4-3149
U+384C.0	fR/S	 50.8
U+384C.0	fTotalStrokes	 11
U+384C.0	fCangjie	LBAB
U+384C.0	fFourCorner	46220
U+384C.0	fHakka	 mau5
U+384C.0	fCantonese	mou6; mou2 (also)
U+384C.0	fMandarin	mao4; mei4
U+384C.0	fEnglish	same as &#x5E3D;
U+384D.0	fUTF8	㡍
U+384D.0	fGB	S-223B
U+384D.0	fR/S	 50.8
U+384D.0	fTotalStrokes	 11
U+384D.0	fCangjie	HAKLB
U+384D.0	fCantonese	mok6
U+384E.0	fUTF8	㡎
U+384E.0	fCNS11643	3-355C
U+384E.0	fR/S	 50.8
U+384E.0	fTotalStrokes	 11
U+384E.0	fCangjie	LBIKH
U+384E.0	fFourCorner	43220
U+384E.0	fCantonese	saam1
U+384E.0	fEnglish	[1] simplified form of &#x5E53; [2] same as &#x7E3F; [3] same as &#x8942;
U+384F.0	fUTF8	㡏
U+384F.0	fCNS11643	4-373B
U+384F.0	fGB	5-3A3E
U+384F.0	fR/S	 50.9
U+384F.0	fTotalStrokes	 12
U+384F.0	fCangjie	LBOMN
U+384F.0	fFourCorner	48220
U+384F.0	fCantonese	syu1; so1
U+384F.0	fMandarin	shu1; tou2
U+384F.0	fEnglish	[1] cut a strip of cloth; a breadth of material [2] same as &#x7E7B;
U+3850.0	fUTF8	㡐
U+3850.0	fCNS11643	4-3736
U+3850.0	fGB	3-3829
U+3850.0	fR/S	 50.9
U+3850.0	fTotalStrokes	 12
U+3850.0	fCangjie	LBTBN
U+3850.0	fFourCorner	48220
U+3850.0	fCantonese	zin1
U+3850.0	fMandarin	jian1
U+3850.0	fEnglish	same as &#x220EC;
U+3851.0	fUTF8	㡑
U+3851.0	fCNS11643	4-373D
U+3851.0	fGB	3-3825
U+3851.0	fR/S	 50.9
U+3851.0	fTotalStrokes	 12
U+3851.0	fCangjie	HFLB
U+3851.0	fFourCorner	29220
U+3851.0	fHakka	 ziau3; ciau1; ziu3
U+3851.0	fCantonese	ciu1; hau1; ziu2
U+3851.0	fMandarin	jiao3; qiao3
U+3851.0	fEnglish	a kind of turban worn by women in mourning
U+3852.0	fUTF8	㡒
U+3852.0	fCNS11643	4-373A
U+3852.0	fGB	3-3827
U+3852.0	fR/S	 50.9
U+3852.0	fTotalStrokes	 12
U+3852.0	fCangjie	LBHJU
U+3852.0	fFourCorner	42260
U+3852.0	fCantonese	zeon1
U+3852.0	fMandarin	zhun1
U+3852.0	fEnglish	a calico sack for grain, etc.
U+3853.0	fUTF8	㡓
U+3853.0	fCNS11643	3-3B3C
U+3853.0	fGB	5-3A33
U+3853.0	fR/S	 50.9
U+3853.0	fTotalStrokes	 12
U+3853.0	fCangjie	LBBJJ
U+3853.0	fFourCorner	47250
U+3853.0	fCantonese	gwan1; kwaan1
U+3853.0	fMandarin	gun1
U+3853.0	fEnglish	drawers; trousers or pants
U+3854.0	fUTF8	㡔
U+3854.0	fCNS11643	4-373C
U+3854.0	fGB	3-382A
U+3854.0	fR/S	 50.9
U+3854.0	fTotalStrokes	 12
U+3854.0	fCangjie	NKLB
U+3854.0	fFourCorner	18220
U+3854.0	fCantonese	mou6
U+3854.0	fMandarin	mu4; wu4
U+3854.0	fEnglish	[1] oiled calico [2] varnish-cloth used to cover the cross-bean in front of the shafts of a carriage
U+3855.0	fUTF8	㡕
U+3855.0	fCNS11643	3-3B39
U+3855.0	fR/S	 50.9
U+3855.0	fTotalStrokes	 12
U+3855.0	fCangjie	LBTLK
U+3855.0	fFourCorner	44230
U+3855.0	fHakka	 jong1
U+3855.0	fCantonese	joeng1; zing1
U+3855.0	fMandarin	ying1; yang1
U+3855.0	fEnglish	same as &#x592E;
U+3856.0	fUTF8	㡖
U+3856.0	fCNS11643	4-3739
U+3856.0	fGB	3-3826
U+3856.0	fR/S	 50.9
U+3856.0	fTotalStrokes	 12
U+3856.0	fCangjie	LBHJG
U+3856.0	fFourCorner	42210
U+3856.0	fCantonese	cong4
U+3856.0	fMandarin	zhong4; chuang2
U+3856.0	fEnglish	same as &#x5E62;
U+3857.0	fUTF8	㡗
U+3857.0	fCNS11643	4-3D28
U+3857.0	fGB	3-382D
U+3857.0	fR/S	 50.10
U+3857.0	fTotalStrokes	 13
U+3857.0	fCangjie	LBHYU
U+3857.0	fFourCorner	42210
U+3857.0	fCantonese	tai4
U+3857.0	fMandarin	ti2
U+3857.0	fEnglish	[1] red paper [2] a small piece of thin paper [3] ropes; cords; cables
U+3858.0	fUTF8	㡘
U+3858.0	fCNS11643	4-3D23
U+3858.0	fGB	3-382F
U+3858.0	fR/S	 50.10
U+3858.0	fTotalStrokes	 13
U+3858.0	fCangjie	LBTXC
U+3858.0	fFourCorner	48237
U+3858.0	fCantonese	lim4
U+3858.0	fMandarin	lian2
U+3858.0	fEnglish	[1] screens; cloth partition; door curtain; door flap [2] same as &#x7E11;
U+3859.0	fUTF8	㡙
U+3859.0	fCNS11643	4-3D27
U+3859.0	fGB	3-382C
U+3859.0	fR/S	 50.10
U+3859.0	fTotalStrokes	 13
U+3859.0	fCangjie	LBHWP
U+3859.0	fFourCorner	47210
U+3859.0	fCantonese	bai1; bei1
U+3859.0	fMandarin	bi1; bi4; pi2
U+3859.0	fEnglish	a curtain for a carriage
U+385A.0	fUTF8	㡚
U+385A.0	fCNS11643	4-3D26
U+385A.0	fGB	3-382B
U+385A.0	fR/S	 50.10
U+385A.0	fTotalStrokes	 13
U+385A.0	fCangjie	LBTTB
U+385A.0	fFourCorner	45240
U+385A.0	fCantonese	kau1; haat6; ngaau1
U+385A.0	fMandarin	gou3
U+385A.0	fEnglish	[1] armour [2] unlined garments
U+385B.0	fUTF8	㡛
U+385B.0	fCNS11643	4-3D29
U+385B.0	fGB	3-3823
U+385B.0	fR/S	 50.10
U+385B.0	fTotalStrokes	 13
U+385B.0	fCangjie	LBTYU
U+385B.0	fFourCorner	44210
U+385B.0	fCantonese	mong4; fong1 (also)
U+385B.0	fMandarin	huang1; mang2
U+385B.0	fEnglish	same as &#x3843;
U+385C.0	fUTF8	㡜
U+385C.0	fCNS11643	4-4333
U+385C.0	fGB	3-3835
U+385C.0	fR/S	 50.11
U+385C.0	fTotalStrokes	 14
U+385C.0	fCangjie	LBBOF
U+385C.0	fFourCorner	47290
U+385C.0	fCantonese	sit3; syut3
U+385C.0	fMandarin	xie4; xue3
U+385C.0	fEnglish	[1] ragged clothes [2] left over; remainder; excess; the surplus [3] cut out (of silk fabrics) as a design
U+385D.0	fUTF8	㡝
U+385D.0	fCNS11643	4-4332
U+385D.0	fGB	3-382E
U+385D.0	fR/S	 50.11
U+385D.0	fTotalStrokes	 14
U+385D.0	fCangjie	LBYHJ
U+385D.0	fFourCorner	47230
U+385D.0	fCantonese	fung6
U+385D.0	fMandarin	feng4; fu2
U+385D.0	fEnglish	[1] napkin [2] kerchief [3] headgear; articles for dressing the hair [4] a calligrapher's or painter's signature [5] seal [6] dedicatory notes, etc. on a painting, etc.
U+385E.0	fUTF8	㡞
U+385E.0	fCNS11643	4-4331
U+385E.0	fGB	3-3828
U+385E.0	fR/S	 50.11
U+385E.0	fTotalStrokes	 14
U+385E.0	fCangjie	LBLLV
U+385E.0	fFourCorner	45240
U+385E.0	fCantonese	lau4; laau4; laau5; leoi5
U+385E.0	fMandarin	lou2; lei3; l3
U+385E.0	fEnglish	[1] a sack used to feed the horses [2] an old family name
U+385F.0	fUTF8	㡟
U+385F.0	fCNS11643	4-432F
U+385F.0	fGB	3-3832
U+385F.0	fR/S	 50.11
U+385F.0	fTotalStrokes	 14
U+385F.0	fCangjie	LBTWA
U+385F.0	fFourCorner	45260
U+385F.0	fCantonese	zou1
U+385F.0	fMandarin	zao1
U+385F.0	fEnglish	bed-mats for baby
U+3860.0	fUTF8	㡠
U+3860.0	fCNS11643	3-4B6E
U+3860.0	fGB	5-3A4D
U+3860.0	fR/S	 50.12
U+3860.0	fTotalStrokes	 15
U+3860.0	fCangjie	LBNOT
U+3860.0	fFourCorner	42210
U+3860.0	fCantonese	zing3
U+3860.0	fMandarin	zheng4; men2; zhen1
U+3860.0	fEnglish	same as &#x5E40;
U+3861.0	fUTF8	㡡
U+3861.0	fGB	3-3836
U+3861.0	fR/S	 50.12
U+3861.0	fTotalStrokes	 15
U+3861.0	fCangjie	LBMMI
U+3861.0	fCantonese	cyu4; ceoi4 (also)
U+3861.0	fMandarin	chu2
U+3861.0	fEnglish	a screen to make a temporary kitchen
U+3862.0	fUTF8	㡢
U+3862.0	fCNS11643	3-4B6F
U+3862.0	fR/S	 50.13; 50.12
U+3862.0	fTotalStrokes	 15; 16
U+3862.0	fCangjie	LBAWS
U+3862.0	fFourCorner	46220
U+3862.0	fCantonese	maan6
U+3862.0	fMandarin	man4
U+3862.0	fEnglish	variant of &#x5E54;
U+3863.0	fUTF8	㡣
U+3863.0	fCNS11643	4-5029
U+3863.0	fR/S	 50.13
U+3863.0	fTotalStrokes	 16
U+3863.0	fCangjie	YBYSP
U+3863.0	fFourCorner	01210
U+3863.0	fHakka	 liung2
U+3863.0	fCantonese	lung4; lung5
U+3863.0	fMandarin	long2; mang2
U+3863.0	fEnglish	same as &#x9F8D;
U+3864.0	fUTF8	㡤
U+3864.0	fR/S	 50.13
U+3864.0	fTotalStrokes	 16
U+3864.0	fCangjie	LBTCD
U+3864.0	fCantonese	jip6
U+3865.0	fUTF8	㡥
U+3865.0	fCNS11643	4-562D
U+3865.0	fGB	5-3A50
U+3865.0	fR/S	 50.14
U+3865.0	fTotalStrokes	 17; 16
U+3865.0	fCangjie	LBBMP
U+3865.0	fFourCorner	42230
U+3865.0	fCantonese	jan3; zaan2
U+3865.0	fMandarin	yin1; yin3
U+3865.0	fEnglish	[1] curl up; roll up [2] wrap up; bind
U+3866.0	fUTF8	㡦
U+3866.0	fCNS11643	4-562B
U+3866.0	fGB	5-3A45
U+3866.0	fR/S	 50.14
U+3866.0	fTotalStrokes	 17; 16
U+3866.0	fCangjie	LBJMC
U+3866.0	fFourCorner	43280
U+3866.0	fCantonese	pan1; baan1; faan6
U+3866.0	fMandarin	bin1
U+3866.0	fEnglish	[1] mess; out of order; chaos; confusion [2] tattered (of clothes); in rags
U+3867.0	fUTF8	㡧
U+3867.0	fCNS11643	4-633B
U+3867.0	fGB	3-383C
U+3867.0	fR/S	 50.17
U+3867.0	fTotalStrokes	 20
U+3867.0	fCangjie	LBJCT
U+3867.0	fFourCorner	43210
U+3867.0	fHakka	 cen5; cen6; zen5; zhen5
U+3867.0	fCantonese	zing3; coeng1
U+3867.0	fMandarin	zheng4
U+3867.0	fEnglish	 unroll a scroll
U+3868.0	fUTF8	㡨
U+3868.0	fCNS11643	4-633A
U+3868.0	fGB	5-3A52
U+3868.0	fR/S	 50.17
U+3868.0	fTotalStrokes	 20
U+3868.0	fCangjie	LBOIM
U+3868.0	fFourCorner	43250
U+3868.0	fHakka	  ciam1
U+3868.0	fCantonese	zim1; cin1
U+3868.0	fMandarin	jian1; qian1
U+3868.0	fEnglish	[1] label [2] make a note of [3] record
U+3869.0	fUTF8	㡩
U+3869.0	fCNS11643	3-5F44
U+3869.0	fR/S	 50.19
U+3869.0	fTotalStrokes	 22
U+3869.0	fCangjie	VFLB
U+3869.0	fFourCorner	22220
U+3869.0	fHakka	 lan2
U+3869.0	fCantonese	lyun4
U+3869.0	fMandarin	luan2
U+3869.0	fEnglish	 cord of silk
U+386A.0	fUTF8	㡪
U+386A.0	fCNS11643	4-686B
U+386A.0	fGB	3-383D
U+386A.0	fR/S	 50.19
U+386A.0	fTotalStrokes	 22; 21
U+386A.0	fCangjie	LBBSE
U+386A.0	fFourCorner	42240
U+386A.0	fCantonese	fan1
U+386A.0	fMandarin	nie2
U+386A.0	fEnglish	[1] the varnish on the floor [2] erase; obliterate [3] scribble
U+386B.0	fUTF8	㡫
U+386B.0	fCNS11643	6-4844
U+386B.0	fR/S	 52.9
U+386B.0	fTotalStrokes	 12
U+386B.0	fCangjie	VIAPV
U+386B.0	fFourCorner	26720
U+386B.0	fCantonese	ai3; ngai3 (also); zi3
U+386B.0	fMandarin	yi4
U+386B.0	fEnglish	corrupted form of &#x253E7C;
U+386C.0	fUTF8	㡬
U+386C.0	fR/S	 52.9
U+386C.0	fTotalStrokes	 12
U+386C.0	fCangjie	VIHS
U+386C.0	fCantonese	gei2
U+386C.0	fEnglish	[Japanese character]
U+386D.0	fUTF8	㡭
U+386D.0	fCNS11643	3-465E
U+386D.0	fGB	5-415D
U+386D.0	fR/S	 52.11
U+386D.0	fTotalStrokes	 14
U+386D.0	fCangjie	VVII
U+386D.0	fFourCorner	22710
U+386D.0	fHakka	 gi5
U+386D.0	fCantonese	gai3
U+386D.0	fMandarin	ji4; kui2
U+386D.0	fEnglish	same as &#x7E7C;
U+386E.0	fUTF8	㡮
U+386E.0	fCNS11643	4-502B
U+386E.0	fGB	3-3252
U+386E.0	fR/S	 52.13; 52.12
U+386E.0	fTotalStrokes	 16; 15
U+386E.0	fCangjie	VIOMN
U+386E.0	fFourCorner	28217
U+386E.0	fCantonese	gat6; gaap6; kaap6
U+386E.0	fMandarin	ji2
U+386E.0	fEnglish	[1] danger; precarious [2] fear [3] lofty; high [4] just; honest [5] grind [6] cultivate; polish
U+386F.0	fUTF8	㡯
U+386F.0	fCNS11643	3-2360
U+386F.0	fGB	5-3C7A
U+386F.0	fR/S	 53.3
U+386F.0	fTotalStrokes	 6
U+386F.0	fCangjie	IHP
U+386F.0	fFourCorner	00210
U+386F.0	fCantonese	zaak6; dok6; dou6; saam2; zak6
U+386F.0	fMandarin	du4; zhai2
U+386F.0	fEnglish	[1] ancient form of &#x5B85; [2] ancient form of &#x5EA6;
U+3870.0	fUTF8	㡰
U+3870.0	fCNS11643	3-235F
U+3870.0	fR/S	 53.3
U+3870.0	fTotalStrokes	 6
U+3870.0	fCangjie	IMD
U+3870.0	fFourCorner	00240
U+3870.0	fHakka	 ji3
U+3870.0	fCantonese	jyu5; zyu5
U+3870.0	fMandarin	yu3
U+3870.0	fEnglish	variant of &#x5B87;
U+3871.0	fUTF8	㡱
U+3871.0	fCNS11643	3-235E
U+3871.0	fR/S	 53.3
U+3871.0	fTotalStrokes	 6
U+3871.0	fCangjie	INO
U+3871.0	fFourCorner	00280
U+3871.0	fCantonese	gau3; gaau3; kaau3
U+3871.0	fMandarin	jiu4; jiu3
U+3871.0	fEnglish	same as &#x7078;
U+3872.0	fUTF8	㡲
U+3872.0	fCNS11643	3-256C
U+3872.0	fGB	5-3C7D
U+3872.0	fR/S	 53.4
U+3872.0	fTotalStrokes	 7
U+3872.0	fCangjie	II
U+3872.0	fFourCorner	00250
U+3872.0	fCantonese	waan4
U+3872.0	fMandarin	huan2
U+3872.0	fEnglish	[1] tiles (laid upside down) on a roof [2] ropes [3] hold fast
U+3873.0	fUTF8	㡳
U+3873.0	fCNS11643	6-2747
U+3873.0	fR/S	 53.4
U+3873.0	fTotalStrokes	 7
U+3873.0	fCangjie	IHVP
U+3873.0	fFourCorner	00240
U+3873.0	fHakka	 dai3; gi3; te3
U+3873.0	fCantonese	zi2; dai2
U+3873.0	fMandarin	fan1; di3; de1; zhe2; de5; zhi3
U+3873.0	fEnglish	same as &#x5E95;
U+3874.0	fUTF8	㡴
U+3874.0	fCNS11643	4-2638
U+3874.0	fR/S	 53.5
U+3874.0	fTotalStrokes	 8
U+3874.0	fCangjie	IYT
U+3874.0	fFourCorner	00210
U+3874.0	fCantonese	laap6; kaap1; lai1
U+3874.0	fMandarin	la1
U+3874.0	fEnglish	sound
U+3875.0	fUTF8	㡵
U+3875.0	fCNS11643	4-263D
U+3875.0	fGB	3-3A45
U+3875.0	fR/S	 53.5
U+3875.0	fTotalStrokes	 8
U+3875.0	fCangjie	IOII
U+3875.0	fFourCorner	00230
U+3875.0	fCantonese	ling4; ling5
U+3875.0	fMandarin	ling2
U+3875.0	fEnglish	roof of the house connected
U+3876.0	fUTF8	㡶
U+3876.0	fCNS11643	4-263B
U+3876.0	fGB	3-3A43
U+3876.0	fR/S	 53.5
U+3876.0	fTotalStrokes	 8
U+3876.0	fCangjie	IRC
U+3876.0	fFourCorner	00280
U+3876.0	fCantonese	zi2
U+3876.0	fMandarin	zhi3; ji4
U+3876.0	fEnglish	[1] dwell [2] live in a depraved (crude; vulgar; inferior) place
U+3877.0	fUTF8	㡷
U+3877.0	fCNS11643	5-2626
U+3877.0	fGB	7-2339
U+3877.0	fR/S	 53.5
U+3877.0	fTotalStrokes	 8
U+3877.0	fCangjie	IDM
U+3877.0	fFourCorner	00230
U+3877.0	fCantonese	bun2
U+3877.0	fMandarin	ben3
U+3877.0	fEnglish	a family name
U+3878.0	fUTF8	㡸
U+3878.0	fCNS11643	4-263C
U+3878.0	fGB	3-3A44
U+3878.0	fR/S	 53.5
U+3878.0	fTotalStrokes	 8
U+3878.0	fCangjie	IOS
U+3878.0	fFourCorner	00210
U+3878.0	fHakka	 za5
U+3878.0	fCantonese	zaa6; zaa3
U+3878.0	fMandarin	zha4; zha3
U+3878.0	fEnglish	[1] house; unfinished house [2] uneven; irregular; unsuitable; ill-matched, tenon
U+3879.0	fUTF8	㡹
U+3879.0	fCNS11643	3-2864
U+3879.0	fGB	5-3D23
U+3879.0	fR/S	 53.5
U+3879.0	fTotalStrokes	 8
U+3879.0	fCangjie	IBM
U+3879.0	fFourCorner	00210
U+3879.0	fCantonese	ci3; zeoi1
U+3879.0	fMandarin	ci4
U+3879.0	fEnglish	 rely upon each other
U+387A.0	fUTF8	㡺
U+387A.0	fCNS11643	4-263A
U+387A.0	fGB	3-3A41
U+387A.0	fR/S	 53.5
U+387A.0	fTotalStrokes	 8
U+387A.0	fCangjie	IAM
U+387A.0	fFourCorner	00210
U+387A.0	fCantonese	daan3
U+387A.0	fMandarin	dan4
U+387A.0	fEnglish	[1] cottage; small house [2] small cup
U+387B.0	fUTF8	㡻
U+387B.0	fCNS11643	3-2866
U+387B.0	fGB	5-3D24
U+387B.0	fR/S	 53.5
U+387B.0	fTotalStrokes	 8
U+387B.0	fCangjie	IHHL
U+387B.0	fFourCorner	00220
U+387B.0	fCantonese	liu6
U+387B.0	fMandarin	liao2; liao4
U+387B.0	fEnglish	non-classical form of &#x5ED6;
U+387C.0	fUTF8	㡼
U+387C.0	fCNS11643	F-2F38
U+387C.0	fGB	3-3A4A
U+387C.0	fR/S	 53.6
U+387C.0	fTotalStrokes	 9
U+387C.0	fCangjie	ILWP
U+387C.0	fFourCorner	0025
U+387C.0	fCantonese	ji6; zi6
U+387C.0	fMandarin	yi4
U+387C.0	fEnglish	[1] granary [2] cabin [3] corridor; hallway
U+387D.0	fUTF8	㡽
U+387D.0	fCNS11643	3-2C49
U+387D.0	fR/S	 53.6
U+387D.0	fTotalStrokes	 9
U+387D.0	fCangjie	ILQ
U+387D.0	fFourCorner	00250
U+387D.0	fCantonese	ziu6
U+387D.0	fMandarin	zhao4
U+387D.0	fEnglish	depraved talking
U+387E.0	fUTF8	㡾
U+387E.0	fCNS11643	3-2C4A
U+387E.0	fR/S	 53.6
U+387E.0	fTotalStrokes	 9
U+387E.0	fCangjie	IAV
U+387E.0	fFourCorner	00230
U+387E.0	fHakka	 han5; han6
U+387E.0	fCantonese	haan6
U+387E.0	fMandarin	xian4
U+387E.0	fEnglish	 threshold; movable board at the foot of large doors
U+387F.0	fUTF8	㡿
U+387F.0	fCNS11643	4-292A
U+387F.0	fGB	5-3D2C
U+387F.0	fR/S	 53.6
U+387F.0	fTotalStrokes	 9
U+387F.0	fCangjie	ITU
U+387F.0	fFourCorner	00240
U+387F.0	fHakka	 ch'ak7; cak7; chit7
U+387F.0	fCantonese	cik1
U+387F.0	fMandarin	chi4; chi3
U+387F.0	fEnglish	same as &#x65A5;
U+3880.0	fUTF8	㢀
U+3880.0	fCNS11643	5-2926
U+3880.0	fGB	3-3A48
U+3880.0	fR/S	 53.6
U+3880.0	fTotalStrokes	 9
U+3880.0	fCangjie	IDB
U+3880.0	fFourCorner	00290
U+3880.0	fCantonese	ci3
U+3880.0	fMandarin	ci4; zi4
U+3880.0	fEnglish	side room
U+3881.0	fUTF8	㢁
U+3881.0	fCNS11643	4-2929
U+3881.0	fGB	5-3D2A
U+3881.0	fR/S	 53.6
U+3881.0	fTotalStrokes	 9
U+3881.0	fCangjie	ININ
U+3881.0	fFourCorner	00220
U+3881.0	fCantonese	ci2
U+3881.0	fMandarin	chi3
U+3881.0	fEnglish	same as &#x388B;
U+3882.0	fUTF8	㢂
U+3882.0	fCNS11643	4-2928
U+3882.0	fGB	3-3A4B
U+3882.0	fR/S	 53.6
U+3882.0	fTotalStrokes	 9
U+3882.0	fCangjie	IHRR
U+3882.0	fFourCorner	00270
U+3882.0	fCantonese	jim5; zim5
U+3882.0	fMandarin	yan3
U+3882.0	fEnglish	[1] (terrains) of highly strategic; precipitious (hill, etc. a big mound [2] same as &#x3888;
U+3883.0	fUTF8	㢃
U+3883.0	fCNS11643	4-2C6B
U+3883.0	fGB	3-3A58
U+3883.0	fR/S	 53.7
U+3883.0	fTotalStrokes	 10
U+3883.0	fCangjie	IIAV
U+3883.0	fFourCorner	00230
U+3883.0	fCantonese	long4
U+3883.0	fMandarin	lang2
U+3883.0	fEnglish	[1] high [2] magnanimity
U+3884.0	fUTF8	㢄
U+3884.0	fCNS11643	4-2C6D
U+3884.0	fGB	5-3D2E
U+3884.0	fR/S	 53.7
U+3884.0	fTotalStrokes	 10
U+3884.0	fCangjie	IMRT
U+3884.0	fFourCorner	00210
U+3884.0	fCantonese	dau6; cim1; daau6
U+3884.0	fMandarin	dou4
U+3884.0	fEnglish	[1] a kind of utensil; implement; tool [2] a place of worship; a place where to honor by a service or rite; a place to offer sacrifices [3] kitchen
U+3885.0	fUTF8	㢅
U+3885.0	fCNS11643	4-2C6C
U+3885.0	fGB	3-3A50
U+3885.0	fR/S	 53.7
U+3885.0	fTotalStrokes	 10
U+3885.0	fCangjie	IMGT
U+3885.0	fFourCorner	00240
U+3885.0	fCantonese	lung6
U+3885.0	fMandarin	long4
U+3885.0	fEnglish	[1] shield; screen [2] tall building; edifice [3] same as &#x5F04;
U+3886.0	fUTF8	㢆
U+3886.0	fCNS11643	3-306D
U+3886.0	fR/S	 53.7
U+3886.0	fTotalStrokes	 10
U+3886.0	fCangjie	IWG
U+3886.0	fFourCorner	00210
U+3886.0	fHakka	 chan2; chen2
U+3886.0	fCantonese	cin4
U+3886.0	fMandarin	chan2
U+3886.0	fEnglish	same as &#x5EDB;
U+3887.0	fUTF8	㢇
U+3887.0	fR/S	 53.7
U+3887.0	fTotalStrokes	 10
U+3887.0	fCangjie	IYMR
U+3887.0	fCantonese	jin4
U+3888.0	fUTF8	㢈
U+3888.0	fCNS11643	3-355E
U+3888.0	fGB	5-3D35
U+3888.0	fR/S	 53.8
U+3888.0	fTotalStrokes	 11
U+3888.0	fCangjie	IOG
U+3888.0	fFourCorner	00210
U+3888.0	fCantonese	teoi4
U+3888.0	fMandarin	dui1; tui2
U+3888.0	fEnglish	[1] collapsed house [2] same as &#x5806;
U+3889.0	fUTF8	㢉
U+3889.0	fCNS11643	4-314E
U+3889.0	fGB	3-3A60
U+3889.0	fR/S	 53.8
U+3889.0	fTotalStrokes	 11
U+3889.0	fCangjie	IHDP
U+3889.0	fFourCorner	00210
U+3889.0	fCantonese	caa4
U+3889.0	fMandarin	cha2
U+3889.0	fEnglish	[1] a room with open space [2] an old county near today's Shandong province Yunchengxian
U+388A.0	fUTF8	㢊
U+388A.0	fCNS11643	4-314C
U+388A.0	fGB	3-3A5C
U+388A.0	fR/S	 53.8
U+388A.0	fTotalStrokes	 11
U+388A.0	fCangjie	IKMR
U+388A.0	fFourCorner	00220
U+388A.0	fCantonese	ai2; ngai2 (also); aai2
U+388A.0	fMandarin	ai3
U+388A.0	fEnglish	[1] set side by side [2] hide; conceal [3] store
U+388B.0	fUTF8	㢋
U+388B.0	fCNS11643	4-314F
U+388B.0	fGB	3-3A61
U+388B.0	fR/S	 53.8
U+388B.0	fTotalStrokes	 11
U+388B.0	fCangjie	IONN
U+388B.0	fFourCorner	00227
U+388B.0	fCantonese	ci2
U+388B.0	fMandarin	chi3
U+388B.0	fEnglish	[1] vast [2] open up [3] enlarge; expand
U+388C.0	fUTF8	㢌
U+388C.0	fCNS11643	3-355D
U+388C.0	fR/S	 53.8
U+388C.0	fTotalStrokes	 11
U+388C.0	fCangjie	INLR
U+388C.0	fFourCorner	00220
U+388C.0	fCantonese	aa3; ngaa3 (also)
U+388D.0	fUTF8	㢍
U+388D.0	fCNS11643	5-3829
U+388D.0	fGB	3-3A5B
U+388D.0	fR/S	 53.9
U+388D.0	fTotalStrokes	 12
U+388D.0	fCangjie	ITLK
U+388D.0	fFourCorner	00230
U+388D.0	fCantonese	jing2; zing2; zoeng3; zung2
U+388D.0	fMandarin	ying3; ying2
U+388D.0	fEnglish	 long verandah; long porch
U+388E.0	fUTF8	㢎
U+388E.0	fCNS11643	4-3740
U+388E.0	fGB	3-3A64
U+388E.0	fR/S	 53.9
U+388E.0	fTotalStrokes	 12
U+388E.0	fCangjie	IHJX
U+388E.0	fFourCorner	00270
U+388E.0	fCantonese	zip3; cap3; zaap1; ziu3
U+388E.0	fMandarin	cha1; cha4; ze2; zhai2; zhe2
U+388E.0	fEnglish	[1] hide; conceal [2] a house not so high
U+388F.0	fUTF8	㢏
U+388F.0	fCNS11643	4-3741
U+388F.0	fGB	3-3A67
U+388F.0	fR/S	 53.9
U+388F.0	fTotalStrokes	 12
U+388F.0	fCangjie	IOMN
U+388F.0	fFourCorner	00220
U+388F.0	fCantonese	tau4
U+388F.0	fMandarin	tou2
U+388F.0	fEnglish	[1] cesspool; manger [2] trough; flume; chute [3] same as &#x5EBE; [4] ancient form of &#x903E;
U+3890.0	fUTF8	㢐
U+3890.0	fCNS11643	3-3B41
U+3890.0	fR/S	 53.9
U+3890.0	fTotalStrokes	 12
U+3890.0	fCangjie	IBBR
U+3890.0	fFourCorner	00220
U+3890.0	fCantonese	waa1
U+3891.0	fUTF8	㢑
U+3891.0	fCNS11643	5-3F21
U+3891.0	fR/S	 53.10
U+3891.0	fTotalStrokes	 13
U+3891.0	fCangjie	IOGJ
U+3891.0	fFourCorner	00240
U+3891.0	fCantonese	teoi4
U+3891.0	fMandarin	sui1; tui2
U+3891.0	fEnglish	[1] corrupted form of &#x3888; collapsed house [2] same as &#x5806;
U+3892.0	fUTF8	㢒
U+3892.0	fCNS11643	4-4336
U+3892.0	fGB	3-3A6C
U+3892.0	fR/S	 53.11
U+3892.0	fTotalStrokes	 14
U+3892.0	fCangjie	IYPM
U+3892.0	fFourCorner	00210
U+3892.0	fHakka	  ca2
U+3892.0	fCantonese	caa4
U+3892.0	fMandarin	cha2
U+3892.0	fEnglish	 tumble-down house
U+3893.0	fUTF8	㢓
U+3893.0	fCNS11643	3-465F
U+3893.0	fGB	5-3D43
U+3893.0	fR/S	 53.11
U+3893.0	fTotalStrokes	 14
U+3893.0	fCangjie	IYTJ
U+3893.0	fFourCorner	00240
U+3893.0	fHakka	 zhong1
U+3893.0	fCantonese	zoeng3
U+3893.0	fMandarin	zhang4; yao3
U+3893.0	fEnglish	same as &#x969C;
U+3894.0	fUTF8	㢔
U+3894.0	fCNS11643	4-4339
U+3894.0	fGB	3-3A3F
U+3894.0	fR/S	 53.11
U+3894.0	fTotalStrokes	 14
U+3894.0	fCangjie	IHOO
U+3894.0	fFourCorner	00280
U+3894.0	fCantonese	zung2; cung1; zaai1
U+3894.0	fMandarin	zong3
U+3894.0	fEnglish	[1] many; numerous; crowd; the masses [2] two houses connected each other
U+3895.0	fUTF8	㢕
U+3895.0	fCNS11643	4-433A
U+3895.0	fR/S	 53.11
U+3895.0	fTotalStrokes	 14
U+3895.0	fCangjie	IVHG
U+3895.0	fFourCorner	00210
U+3895.0	fHakka	 jung1
U+3895.0	fCantonese	jung1; zung1
U+3895.0	fMandarin	yong1
U+3895.0	fEnglish	same as &#x5EF1;
U+3896.0	fUTF8	㢖
U+3896.0	fCNS11643	4-495B
U+3896.0	fGB	3-3A70
U+3896.0	fR/S	 53.12
U+3896.0	fTotalStrokes	 15
U+3896.0	fCangjie	IHDW
U+3896.0	fFourCorner	00260
U+3896.0	fCantonese	pun1
U+3896.0	fMandarin	pan1
U+3896.0	fEnglish	[1] store house [2] store
U+3897.0	fUTF8	㢗
U+3897.0	fCNS11643	4-495C
U+3897.0	fR/S	 53.12
U+3897.0	fTotalStrokes	 15
U+3897.0	fCangjie	IHKB
U+3897.0	fFourCorner	00220
U+3897.0	fCantonese	hiu3; hiu1; hou4
U+3897.0	fMandarin	qiao4
U+3897.0	fEnglish	 high house; high building
U+3898.0	fUTF8	㢘
U+3898.0	fCNS11643	3-4B71
U+3898.0	fR/S	 53.12
U+3898.0	fTotalStrokes	 15
U+3898.0	fCangjie	ITXF
U+3898.0	fFourCorner	00230
U+3898.0	fCantonese	lim4; him3
U+3898.0	fMandarin	lian2
U+3898.0	fEnglish	non-classical form of &#x5EC9;
U+3899.0	fUTF8	㢙
U+3899.0	fCNS11643	3-505D
U+3899.0	fGB	5-3D45
U+3899.0	fR/S	 53.13
U+3899.0	fTotalStrokes	 16
U+3899.0	fCangjie	ITMS
U+3899.0	fFourCorner	00220
U+3899.0	fCantonese	kan4; gaan2; kaan4; saap1
U+3899.0	fMandarin	jin3; qin2
U+3899.0	fEnglish	same as &#x5ED1;
U+389A.0	fUTF8	㢚
U+389A.0	fCNS11643	4-502C
U+389A.0	fGB	3-3A5F
U+389A.0	fR/S	 53.13
U+389A.0	fTotalStrokes	 16
U+389A.0	fCangjie	IYPS
U+389A.0	fFourCorner	00220
U+389A.0	fHakka	 lu2
U+389A.0	fCantonese	lou5
U+389A.0	fMandarin	lu3
U+389A.0	fEnglish	[1] corridor; hallway; rooms around the hall (the middle room of a Chinese house) [2] nunnery; convent [3] cottage; hut [4] mansion
U+389B.0	fUTF8	㢛
U+389B.0	fCNS11643	4-502E
U+389B.0	fGB	3-3A57
U+389B.0	fR/S	 53.13
U+389B.0	fTotalStrokes	 16
U+389B.0	fCangjie	IOMO
U+389B.0	fFourCorner	00280
U+389B.0	fCantonese	jim6; zim6
U+389B.0	fMandarin	yan4
U+389B.0	fEnglish	[1] small [2] cabinet; cupboard
U+389C.0	fUTF8	㢜
U+389C.0	fCNS11643	5-5442
U+389C.0	fR/S	 53.13
U+389C.0	fTotalStrokes	 16
U+389C.0	fCangjie	ITWP
U+389C.0	fFourCorner	00230
U+389C.0	fCantonese	kong2
U+389C.0	fMandarin	kang3
U+389C.0	fEnglish	vast
U+389D.0	fUTF8	㢝
U+389D.0	fCNS11643	4-5F5E
U+389D.0	fGB	3-3A71
U+389D.0	fR/S	 53.16
U+389D.0	fTotalStrokes	 19
U+389D.0	fCangjie	INFD
U+389D.0	fFourCorner	00290
U+389D.0	fHakka	 su1; sii1
U+389D.0	fCantonese	sou1
U+389D.0	fMandarin	su1
U+389D.0	fEnglish	 convent; nunnery
U+389E.0	fUTF8	㢞
U+389E.0	fCNS11643	4-633C
U+389E.0	fGB	5-3D4A
U+389E.0	fR/S	 53.17
U+389E.0	fTotalStrokes	 20
U+389E.0	fCangjie	ISMC
U+389E.0	fFourCorner	00280
U+389E.0	fCantonese	jik6; zi6
U+389E.0	fMandarin	yi4
U+389E.0	fEnglish	[1] rooms connected [2] moveable house (a yurt, a portable, tentlike dwelling used by nomadic Mongols)
U+389F.0	fUTF8	㢟
U+389F.0	fCNS11643	4-2429
U+389F.0	fGB	3-335C
U+389F.0	fR/S	 54.3; 54.4
U+389F.0	fTotalStrokes	 7; 6
U+389F.0	fCangjie	NKYLM
U+389F.0	fFourCorner	11400
U+389F.0	fCantonese	cin1; cin2
U+389F.0	fMandarin	chan1; chan3
U+389F.0	fEnglish	 walk slowly; stroll; ramble
U+38A0.0	fUTF8	㢠
U+38A0.0	fCNS11643	3-286A
U+38A0.0	fR/S	 54.5
U+38A0.0	fTotalStrokes	 8
U+38A0.0	fCangjie	NKBR
U+38A0.0	fFourCorner	17400
U+38A0.0	fCantonese	gwing2
U+38A0.0	fMandarin	hui2; jiong3; jun4
U+38A0.0	fEnglish	same as &#x8FE5;
U+38A1.0	fUTF8	㢡
U+38A1.0	fCNS11643	3-4662
U+38A1.0	fR/S	 55.11
U+38A1.0	fTotalStrokes	 14
U+38A1.0	fCangjie	VIT
U+38A1.0	fFourCorner	27440
U+38A1.0	fHakka	 ziong3
U+38A1.0	fCantonese	zoeng2; ceoi1; ceot1
U+38A1.0	fMandarin	jiang3
U+38A1.0	fEnglish	same as &#x596C;
U+38A2.0	fUTF8	㢢
U+38A2.0	fR/S	 55.12
U+38A2.0	fTotalStrokes	 15
U+38A2.0	fCangjie	FKT
U+38A2.0	fCantonese	bai6
U+38A2.0	fEnglish	[Japanese character]
U+38A3.0	fUTF8	㢣
U+38A3.0	fCNS11643	7-2D5B
U+38A3.0	fR/S	 55.13
U+38A3.0	fTotalStrokes	 16
U+38A3.0	fCangjie	TKT
U+38A3.0	fFourCorner	48440
U+38A3.0	fHakka	 kiang2
U+38A3.0	fCantonese	king4
U+38A3.0	fMandarin	qing2; jing4
U+38A3.0	fEnglish	same as &#x64CE;
U+38A4.0	fUTF8	㢤
U+38A4.0	fR/S	 56.4
U+38A4.0	fTotalStrokes	 7
U+38A4.0	fCangjie	JIPS
U+38A4.0	fCantonese	zoi1
U+38A5.0	fUTF8	㢥
U+38A5.0	fCNS11643	4-292B
U+38A5.0	fGB	3-354D
U+38A5.0	fR/S	 56.6
U+38A5.0	fTotalStrokes	 9
U+38A5.0	fCangjie	BRIP
U+38A5.0	fFourCorner	73240
U+38A5.0	fHakka	 tung5; tung6
U+38A5.0	fCantonese	dung6
U+38A5.0	fMandarin	dong4
U+38A5.0	fEnglish	boards used in building a boat
U+38A6.0	fUTF8	㢦
U+38A6.0	fCNS11643	4-3D2D
U+38A6.0	fGB	3-354E
U+38A6.0	fR/S	 56.10
U+38A6.0	fTotalStrokes	 13
U+38A6.0	fCangjie	MRIP
U+38A6.0	fFourCorner	13640
U+38A6.0	fCantonese	go1
U+38A6.0	fMandarin	ge1
U+38A6.0	fEnglish	[1] (corrupted form) [2] wooden post or pile for tethering animals [3] small and sharp pile,
U+38A7.0	fUTF8	㢧
U+38A7.0	fCNS11643	6-2179
U+38A7.0	fR/S	 57.1
U+38A7.0	fTotalStrokes	 4
U+38A7.0	fCangjie	NM
U+38A7.0	fFourCorner	17120
U+38A7.0	fCantonese	gyun2; gyun3; kyun4
U+38A7.0	fMandarin	juan4; gun3
U+38A7.0	fEnglish	same as &#x5377;
U+38A8.0	fUTF8	㢨
U+38A8.0	fCNS11643	4-226A
U+38A8.0	fGB	3-3B62
U+38A8.0	fR/S	 57.3
U+38A8.0	fTotalStrokes	 6
U+38A8.0	fCangjie	NMJ
U+38A8.0	fFourCorner	11240
U+38A8.0	fCantonese	hon6
U+38A8.0	fMandarin	han4
U+38A8.0	fEnglish	[1] defend; ward off; refuse; reject; oppose [2] frontier pass (in Wuxian) in ancient times [3] a county in ancient times
U+38A9.0	fUTF8	㢩
U+38A9.0	fCNS11643	3-2363
U+38A9.0	fGB	5-3E74
U+38A9.0	fR/S	 57.3
U+38A9.0	fTotalStrokes	 6
U+38A9.0	fCangjie	NPI
U+38A9.0	fFourCorner	17220
U+38A9.0	fHakka	 dit7
U+38A9.0	fCantonese	dik1; dim3; tiu4
U+38A9.0	fMandarin	di4
U+38A9.0	fEnglish	[1] shoot; spurt [2] same as &#x7684;
U+38AA.0	fUTF8	㢪
U+38AA.0	fCNS11643	3-2364
U+38AA.0	fR/S	 57.3
U+38AA.0	fTotalStrokes	 6
U+38AA.0	fCangjie	NMMS
U+38AA.0	fFourCorner	11220
U+38AA.0	fHakka	 wu1; vu1
U+38AA.0	fCantonese	wu1
U+38AA.0	fMandarin	wu1
U+38AA.0	fEnglish	standard form of &#x5F19;
U+38AB.0	fUTF8	㢫
U+38AB.0	fGB	7-233D
U+38AB.0	fR/S	 57.3
U+38AB.0	fTotalStrokes	 6
U+38AB.0	fCangjie	NU
U+38AB.0	fCantonese	saan1
U+38AC.0	fUTF8	㢬
U+38AC.0	fCNS11643	3-2570
U+38AC.0	fR/S	 57.4
U+38AC.0	fTotalStrokes	 7
U+38AC.0	fCangjie	NKI
U+38AC.0	fFourCorner	14230
U+38AC.0	fCantonese	wang4
U+38AC.0	fMandarin	hong2
U+38AC.0	fEnglish	same as &#x5F4B;
U+38AD.0	fUTF8	㢭
U+38AD.0	fGB	S-2440
U+38AD.0	fR/S	 57.4
U+38AD.0	fTotalStrokes	 7
U+38AD.0	fCangjie	NYE
U+38AD.0	fCantonese	tou1
U+38AD.0	fEnglish	non-classical form of &#x5F22;
U+38AE.0	fUTF8	㢮
U+38AE.0	fCNS11643	3-286F
U+38AE.0	fR/S	 57.5
U+38AE.0	fTotalStrokes	 8
U+38AE.0	fCangjie	NOPD
U+38AE.0	fFourCorner	18210
U+38AE.0	fHakka	 chi1
U+38AE.0	fCantonese	ci4
U+38AE.0	fMandarin	chi2; shi3
U+38AE.0	fEnglish	same as &#x5F1B;
U+38AF.0	fUTF8	㢯
U+38AF.0	fCNS11643	4-2640
U+38AF.0	fGB	3-3B6A
U+38AF.0	fR/S	 57.5
U+38AF.0	fTotalStrokes	 8
U+38AF.0	fCangjie	NRVP
U+38AF.0	fFourCorner	17240
U+38AF.0	fCantonese	deon1
U+38AF.0	fMandarin	dun1; min2
U+38AF.0	fEnglish	same as &#x5F34;
U+38B0.0	fUTF8	㢰
U+38B0.0	fCNS11643	4-2641
U+38B0.0	fGB	3-3B6D
U+38B0.0	fR/S	 57.5
U+38B0.0	fTotalStrokes	 8
U+38B0.0	fCangjie	NDHE
U+38B0.0	fFourCorner	14240
U+38B0.0	fCantonese	bei6
U+38B0.0	fMandarin	bei4; bi4; huan2
U+38B0.0	fEnglish	[1] stretch a bow [2] the bow decorated with colorful silk and leather [3] bent; curved
U+38B1.0	fUTF8	㢱
U+38B1.0	fR/S	 57.5
U+38B1.0	fTotalStrokes	 8
U+38B1.0	fCangjie	NIF
U+38B1.0	fCantonese	nei4; lei4 (also)
U+38B2.0	fUTF8	㢲
U+38B2.0	fCNS11643	3-2C4E
U+38B2.0	fR/S	 57.6
U+38B2.0	fTotalStrokes	 9
U+38B2.0	fCangjie	NNMC
U+38B2.0	fFourCorner	17800
U+38B2.0	fHakka	 sun5
U+38B2.0	fCantonese	seon3; saan1
U+38B2.0	fMandarin	xun4; sun4; zhuan4
U+38B2.0	fEnglish	same as &#x5DFD;
U+38B3.0	fUTF8	㢳
U+38B3.0	fCNS11643	4-292D
U+38B3.0	fGB	7-233B
U+38B3.0	fR/S	 57.6
U+38B3.0	fTotalStrokes	 9
U+38B3.0	fCangjie	NOHV
U+38B3.0	fFourCorner	18230
U+38B3.0	fCantonese	lou4
U+38B3.0	fMandarin	lu2; lyu4; l4
U+38B3.0	fEnglish	same as &#x7388;
U+38B4.0	fUTF8	㢴
U+38B4.0	fCNS11643	6-307A
U+38B4.0	fR/S	 57.6
U+38B4.0	fTotalStrokes	 9
U+38B4.0	fCangjie	NWNI
U+38B4.0	fFourCorner	17620
U+38B4.0	fHakka	 si1
U+38B4.0	fCantonese	sai1
U+38B4.0	fMandarin	xi1; si1; yue1
U+38B4.0	fEnglish	ancient form of &#x897F;
U+38B5.0	fUTF8	㢵
U+38B5.0	fCNS11643	4-292E
U+38B5.0	fGB	5-3E7A
U+38B5.0	fR/S	 57.6
U+38B5.0	fTotalStrokes	 9
U+38B5.0	fCangjie	NOMR
U+38B5.0	fFourCorner	18260
U+38B5.0	fCantonese	hip3; sip3
U+38B5.0	fMandarin	she4; xie2
U+38B5.0	fEnglish	[1] powerful and strong bow [2] interchangeable with &#x5F3D; &#x97D8;
U+38B6.0	fUTF8	㢶
U+38B6.0	fCNS11643	3-2C4F
U+38B6.0	fR/S	 57.6
U+38B6.0	fTotalStrokes	 9
U+38B6.0	fCangjie	NMA
U+38B6.0	fFourCorner	11260
U+38B6.0	fHakka	 pet7; pit7; bit7
U+38B6.0	fCantonese	bat6
U+38B6.0	fMandarin	bi4
U+38B6.0	fEnglish	same as &#x5F3C;
U+38B7.0	fUTF8	㢷
U+38B7.0	fGB	7-233A
U+38B7.0	fR/S	 57.6
U+38B7.0	fTotalStrokes	 9
U+38B7.0	fCangjie	NUNI
U+38B7.0	fCantonese	seoi3
U+38B8.0	fUTF8	㢸
U+38B8.0	fCNS11643	4-2C71
U+38B8.0	fGB	5-3E7D
U+38B8.0	fR/S	 57.7
U+38B8.0	fTotalStrokes	 10
U+38B8.0	fCangjie	NNOK
U+38B8.0	fFourCorner	18240
U+38B8.0	fCantonese	bat6; baat6; mak3; pong5
U+38B8.0	fMandarin	bi4
U+38B8.0	fEnglish	ancient form of &#x5F3C;
U+38B9.0	fUTF8	㢹
U+38B9.0	fR/S	 57.7
U+38B9.0	fTotalStrokes	 10
U+38B9.0	fCangjie	NYTJ
U+38B9.0	fCantonese	san1
U+38BA.0	fUTF8	㢺
U+38BA.0	fCNS11643	5-3246
U+38BA.0	fGB	7-233C
U+38BA.0	fR/S	 57.8
U+38BA.0	fTotalStrokes	 11
U+38BA.0	fCangjie	NYTV
U+38BA.0	fFourCorner	10240
U+38BA.0	fHakka	 hien2
U+38BA.0	fCantonese	jin4; zin2; zin4
U+38BA.0	fMandarin	xian2
U+38BA.0	fEnglish	[1] bow [2] a Chinese family name
U+38BB.0	fUTF8	㢻
U+38BB.0	fCNS11643	4-3151
U+38BB.0	fGB	3-3B72
U+38BB.0	fR/S	 57.8
U+38BB.0	fTotalStrokes	 11
U+38BB.0	fCangjie	NHDV
U+38BB.0	fFourCorner	12240
U+38BB.0	fCantonese	jeoi6; waai2
U+38BB.0	fMandarin	nei4; wei3
U+38BB.0	fEnglish	[1] draw a bow [2] bent
U+38BC.0	fUTF8	㢼
U+38BC.0	fCNS11643	4-2C72
U+38BC.0	fGB	5-3E7C
U+38BC.0	fR/S	 57.8; 57.7
U+38BC.0	fTotalStrokes	 10; 11
U+38BC.0	fCangjie	NFBF
U+38BC.0	fFourCorner	19220
U+38BC.0	fCantonese	bit3
U+38BC.0	fMandarin	bie4; bi4
U+38BC.0	fEnglish	same as &#x5F46;
U+38BD.0	fUTF8	㢽
U+38BD.0	fCNS11643	3-3B46
U+38BD.0	fR/S	 57.9
U+38BD.0	fTotalStrokes	 12
U+38BD.0	fCangjie	NSJN
U+38BD.0	fFourCorner	17220
U+38BD.0	fHakka	 ngi3
U+38BD.0	fCantonese	nei6; lei6 (also)
U+38BD.0	fMandarin	er3; er4
U+38BD.0	fEnglish	same as &#x990C;
U+38BE.0	fUTF8	㢾
U+38BE.0	fCNS11643	3-3B43
U+38BE.0	fGB	5-3F23
U+38BE.0	fR/S	 57.9
U+38BE.0	fTotalStrokes	 12
U+38BE.0	fCangjie	NMWG
U+38BE.0	fFourCorner	11210
U+38BE.0	fCantonese	gyun3
U+38BE.0	fMandarin	juan4
U+38BE.0	fEnglish	the curved end of a bow
U+38BF.0	fUTF8	㢿
U+38BF.0	fR/S	 57.9
U+38BF.0	fTotalStrokes	 12
U+38BF.0	fCangjie	NONK
U+38BF.0	fCantonese	hau4
U+38C0.0	fUTF8	㣀
U+38C0.0	fCNS11643	5-3F29
U+38C0.0	fR/S	 57.10
U+38C0.0	fTotalStrokes	 13
U+38C0.0	fCangjie	NJBC
U+38C0.0	fFourCorner	14280
U+38C0.0	fCantonese	zan3
U+38C0.0	fMandarin	zhen4
U+38C0.0	fEnglish	[1] rebound [2] impeach [3] play
U+38C1.0	fUTF8	㣁
U+38C1.0	fCNS11643	4-3D2F
U+38C1.0	fGB	3-3B75
U+38C1.0	fR/S	 57.10
U+38C1.0	fTotalStrokes	 13
U+38C1.0	fCangjie	NTHB
U+38C1.0	fFourCorner	14220
U+38C1.0	fCantonese	bei6; baai3
U+38C1.0	fMandarin	bei4
U+38C1.0	fEnglish	 equip a bow with silk and muscular fibre
U+38C2.0	fUTF8	㣂
U+38C2.0	fCNS11643	4-3D2E
U+38C2.0	fGB	3-3B76
U+38C2.0	fR/S	 57.10
U+38C2.0	fTotalStrokes	 13
U+38C2.0	fCangjie	NMRB
U+38C2.0	fFourCorner	11220
U+38C2.0	fCantonese	ngaak6; aak6 (also); zik6
U+38C2.0	fMandarin	heng1; yi4
U+38C2.0	fEnglish	[1] bind [2] pack up the bows and the cover of the bows
U+38C3.0	fUTF8	㣃
U+38C3.0	fCNS11643	6-524D
U+38C3.0	fR/S	 57.11
U+38C3.0	fTotalStrokes	 14; 13
U+38C3.0	fCangjie	NYBN
U+38C3.0	fFourCorner	17220
U+38C3.0	fHakka	 chuk7
U+38C3.0	fCantonese	juk6; zuk6; zuk1
U+38C3.0	fMandarin	li4; yu4; zhu4; yu3; mi2; ju4
U+38C3.0	fEnglish	same as &#x9B3B;
U+38C4.0	fUTF8	㣄
U+38C4.0	fCNS11643	4-495D
U+38C4.0	fGB	3-3B77
U+38C4.0	fR/S	 57.12
U+38C4.0	fTotalStrokes	 15
U+38C4.0	fCangjie	NESD
U+38C4.0	fFourCorner	11290
U+38C4.0	fCantonese	keoi4
U+38C4.0	fMandarin	qu2
U+38C4.0	fEnglish	[1] powerful; strong bows [2] the end of the bows
U+38C5.0	fUTF8	㣅
U+38C5.0	fCNS11643	4-495F
U+38C5.0	fGB	3-3B78
U+38C5.0	fR/S	 57.12
U+38C5.0	fTotalStrokes	 15
U+38C5.0	fCangjie	NMUA
U+38C5.0	fFourCorner	11260
U+38C5.0	fCantonese	zaam6; zam1
U+38C5.0	fMandarin	zan4
U+38C5.0	fEnglish	[1] bow-string; chord [2] straight road [3] draw a bow [4] strong
U+38C6.0	fUTF8	㣆
U+38C6.0	fCNS11643	3-5E2E
U+38C6.0	fGB	5-3F25
U+38C6.0	fR/S	 57.18
U+38C6.0	fTotalStrokes	 21
U+38C6.0	fCangjie	NMBG
U+38C6.0	fFourCorner	11210
U+38C6.0	fCantonese	mei4; nei4
U+38C6.0	fMandarin	xi3; mi2; pei4
U+38C6.0	fEnglish	[1] loose the bow-string [2] a kind of jade [3] same as &#x5F4C;
U+38C7.0	fUTF8	㣇
U+38C7.0	fCNS11643	4-2642
U+38C7.0	fGB	3-3B4A
U+38C7.0	fR/S	 58.5
U+38C7.0	fTotalStrokes	 8
U+38C7.0	fCangjie	VNMB
U+38C7.0	fFourCorner	27220
U+38C7.0	fCantonese	dai6; caa3; zi6
U+38C7.0	fMandarin	ni3; yi4
U+38C7.0	fEnglish	[1] a kind of beast with long hair [2] other name for pig, fox, wild cat, raccoon
U+38C8.0	fUTF8	㣈
U+38C8.0	fCNS11643	4-5031
U+38C8.0	fGB	3-3B4B
U+38C8.0	fR/S	 58.13
U+38C8.0	fTotalStrokes	 16
U+38C8.0	fCangjie	VBVNB
U+38C8.0	fFourCorner	27220
U+38C8.0	fCantonese	si3
U+38C8.0	fMandarin	si4
U+38C8.0	fEnglish	[1] a kind of animal [2] a kind of rat [3] interchangeable with &#x8086;
U+38C9.0	fUTF8	㣉
U+38C9.0	fCNS11643	6-2470
U+38C9.0	fR/S	 59.3
U+38C9.0	fTotalStrokes	 6
U+38C9.0	fCangjie	MHHH
U+38C9.0	fFourCorner	12120
U+38C9.0	fHakka	 gung1
U+38C9.0	fCantonese	gung1
U+38C9.0	fMandarin	gong1
U+38C9.0	fEnglish	ancient form of &#x5DE5;
U+38CA.0	fUTF8	㣊
U+38CA.0	fR/S	 59.4
U+38CA.0	fTotalStrokes	 7
U+38CA.0	fCangjie	HEHHH
U+38CA.0	fCantonese	man4
U+38CB.0	fUTF8	㣋
U+38CB.0	fCNS11643	3-2873
U+38CB.0	fR/S	 59.5
U+38CB.0	fTotalStrokes	 8
U+38CB.0	fCangjie	TTHHH
U+38CB.0	fFourCorner	52020
U+38CB.0	fHakka	 dan1
U+38CB.0	fCantonese	daan1
U+38CB.0	fMandarin	dan1
U+38CB.0	fEnglish	ancient form of &#x4E39;
U+38CC.0	fUTF8	㣌
U+38CC.0	fCNS11643	4-2643
U+38CC.0	fGB	5-3C59
U+38CC.0	fR/S	 59.5
U+38CC.0	fTotalStrokes	 8
U+38CC.0	fCangjie	YRHHH
U+38CC.0	fFourCorner	22620
U+38CC.0	fCantonese	saam3
U+38CC.0	fMandarin	shan4
U+38CC.0	fEnglish	[1] join two things; contiguous; things connected end to end [2] sharp
U+38CD.0	fUTF8	㣍
U+38CD.0	fCNS11643	3-2872
U+38CD.0	fR/S	 59.5
U+38CD.0	fTotalStrokes	 8
U+38CD.0	fCangjie	IRHHH
U+38CD.0	fFourCorner	22620
U+38CD.0	fCantonese	toi4; ciu1; cong2; zaau2
U+38CD.0	fMandarin	tai2
U+38CD.0	fEnglish	name of a constellation
U+38CE.0	fUTF8	㣎
U+38CE.0	fCNS11643	3-3564
U+38CE.0	fGB	5-5B5A
U+38CE.0	fR/S	 59.8
U+38CE.0	fTotalStrokes	 11
U+38CE.0	fCangjie	HAFHH
U+38CE.0	fFourCorner	26200
U+38CE.0	fCantonese	muk6; man6
U+38CE.0	fMandarin	mu4
U+38CE.0	fEnglish	fine (material, etc.) and delicate (workmanship, etc.) decorative design or pattern
U+38CF.0	fUTF8	㣏
U+38CF.0	fCNS11643	3-3B48
U+38CF.0	fR/S	 59.9
U+38CF.0	fTotalStrokes	 12
U+38CF.0	fCangjie	KJHHH
U+38CF.0	fFourCorner	42420
U+38CF.0	fCantonese	zing6
U+38CF.0	fMandarin	jing4
U+38CF.0	fEnglish	ancient form of &#x975C;
U+38D0.0	fUTF8	㣐
U+38D0.0	fCNS11643	3-3B49
U+38D0.0	fR/S	 59.9
U+38D0.0	fTotalStrokes	 12
U+38D0.0	fCangjie	HEHHH
U+38D0.0	fFourCorner	22220
U+38D0.0	fCantonese	bin3
U+38D0.0	fMandarin	bian4
U+38D0.0	fEnglish	same as &#x2249F;
U+38D1.0	fUTF8	㣑
U+38D1.0	fCNS11643	4-433B
U+38D1.0	fGB	3-3A2D
U+38D1.0	fR/S	 59.11
U+38D1.0	fTotalStrokes	 14
U+38D1.0	fCangjie	IBHHH
U+38D1.0	fFourCorner	02220
U+38D1.0	fCantonese	jung4; zung4
U+38D1.0	fMandarin	yong2; rong2
U+38D1.0	fEnglish	[1] double image [2] non-classical form of &#x5BB9;
U+38D2.0	fUTF8	㣒
U+38D2.0	fCNS11643	4-4961
U+38D2.0	fGB	3-3A2E
U+38D2.0	fR/S	 59.12
U+38D2.0	fTotalStrokes	 15
U+38D2.0	fCangjie	CAHHH
U+38D2.0	fFourCorner	82620
U+38D2.0	fCantonese	cing3; saang3; sik1
U+38D2.0	fMandarin	cong1; ceng4
U+38D2.0	fEnglish	[1] hair-raising [2] hairiness
U+38D3.0	fUTF8	㣓
U+38D3.0	fCNS11643	3-505F
U+38D3.0	fR/S	 59.13
U+38D3.0	fTotalStrokes	 16
U+38D3.0	fCangjie	YDHHH
U+38D3.0	fFourCorner	22920
U+38D3.0	fCantonese	caan3
U+38D3.0	fMandarin	can4
U+38D3.0	fEnglish	[1] literary in taste; elegant; ornamental [2] beautiful and gorgeous
U+38D4.0	fUTF8	㣔
U+38D4.0	fCNS11643	3-224B
U+38D4.0	fGB	5-3C25
U+38D4.0	fR/S	 60.2
U+38D4.0	fTotalStrokes	 5
U+38D4.0	fCangjie	HOMN
U+38D4.0	fFourCorner	21220
U+38D4.0	fHakka	 den1
U+38D4.0	fCantonese	ding1
U+38D4.0	fMandarin	ding1
U+38D4.0	fEnglish	to walk alone, to insist on one's ways in doing things
U+38D5.0	fUTF8	㣕
U+38D5.0	fR/S	 60.3
U+38D5.0	fTotalStrokes	 6
U+38D5.0	fCangjie	HOK
U+38D5.0	fCantonese	daai6
U+38D6.0	fUTF8	㣖
U+38D6.0	fR/S	 60.4
U+38D6.0	fTotalStrokes	 7
U+38D6.0	fCangjie	HOKI
U+38D6.0	fCantonese	taai3
U+38D7.0	fUTF8	㣗
U+38D7.0	fR/S	 60.4
U+38D7.0	fTotalStrokes	 7
U+38D7.0	fCangjie	HOHS
U+38D7.0	fCantonese	wu6
U+38D8.0	fUTF8	㣘
U+38D8.0	fCNS11643	6-2B3E
U+38D8.0	fR/S	 60.5
U+38D8.0	fTotalStrokes	 8
U+38D8.0	fCangjie	HOPR
U+38D8.0	fFourCorner	27220
U+38D8.0	fHakka	 gi1
U+38D8.0	fCantonese	keoi1
U+38D8.0	fMandarin	qu2; ju1
U+38D8.0	fEnglish	same as &#x8DD4;
U+38D9.0	fUTF8	㣙
U+38D9.0	fCNS11643	3-2874
U+38D9.0	fGB	5-3C2A
U+38D9.0	fR/S	 60.5
U+38D9.0	fTotalStrokes	 8
U+38D9.0	fCangjie	HOLW
U+38D9.0	fFourCorner	25260
U+38D9.0	fCantonese	dik6; dek6; duk6
U+38D9.0	fMandarin	di2
U+38D9.0	fEnglish	to walk in an easy and leisurely manner, (same as &#x5B99;) time--past, present and future, infinite time; time without beginning or end; eternity
U+38DA.0	fUTF8	㣚
U+38DA.0	fCNS11643	4-2932
U+38DA.0	fGB	3-397B
U+38DA.0	fR/S	 60.6
U+38DA.0	fTotalStrokes	 9
U+38DA.0	fCangjie	HOBMR
U+38DA.0	fFourCorner	27220
U+38DA.0	fCantonese	tung2; tung4
U+38DA.0	fMandarin	tong3; tong2
U+38DA.0	fEnglish	to go straight forward, (same as &#x8855;) a lane; an alley, (same as &#x4F97;) ignorant; rude; rustic; straight-forward; uninterrupted; no obstacle
U+38DB.0	fUTF8	㣛
U+38DB.0	fCNS11643	4-2934
U+38DB.0	fGB	5-3C33
U+38DB.0	fR/S	 60.6
U+38DB.0	fTotalStrokes	 9
U+38DB.0	fCangjie	HOOMR
U+38DB.0	fFourCorner	28260
U+38DB.0	fCantonese	taap3; dap6
U+38DB.0	fMandarin	ta4
U+38DB.0	fEnglish	walking, (ancient form form of &#x6703;) to meet, to gather, to be able, to realize
U+38DC.0	fUTF8	㣜
U+38DC.0	fCNS11643	3-2C52
U+38DC.0	fR/S	 60.6
U+38DC.0	fTotalStrokes	 9
U+38DC.0	fCangjie	HOMTN
U+38DC.0	fFourCorner	22200
U+38DC.0	fCantonese	jing4; haang4
U+38DC.0	fMandarin	xing2
U+38DC.0	fEnglish	to walk; to go
U+38DD.0	fUTF8	㣝
U+38DD.0	fCNS11643	5-292F
U+38DD.0	fGB	7-233E
U+38DD.0	fR/S	 60.6
U+38DD.0	fTotalStrokes	 9
U+38DD.0	fCangjie	HOIJ
U+38DD.0	fFourCorner	23250
U+38DD.0	fCantonese	sung1
U+38DD.0	fMandarin	song1
U+38DD.0	fEnglish	a Chinese family name
U+38DE.0	fUTF8	㣞
U+38DE.0	fCNS11643	5-2933
U+38DE.0	fR/S	 60.6
U+38DE.0	fTotalStrokes	 9
U+38DE.0	fCangjie	HOCIU
U+38DE.0	fFourCorner	28210
U+38DE.0	fCantonese	dyut6
U+38DE.0	fMandarin	duo2; duo4
U+38DE.0	fEnglish	to walki; to go
U+38DF.0	fUTF8	㣟
U+38DF.0	fCNS11643	3-2C53
U+38DF.0	fR/S	 60.6
U+38DF.0	fTotalStrokes	 9
U+38DF.0	fCangjie	HOGR
U+38DF.0	fFourCorner	24260
U+38DF.0	fCantonese	hat1; hei3
U+38DF.0	fMandarin	xi4
U+38DF.0	fEnglish	to go; to walk
U+38E0.0	fUTF8	㣠
U+38E0.0	fCNS11643	6-3125
U+38E0.0	fR/S	 60.6
U+38E0.0	fTotalStrokes	 9
U+38E0.0	fCangjie	HOLHY
U+38E0.0	fFourCorner	28230
U+38E0.0	fHakka	 tung1
U+38E0.0	fCantonese	tou1; tung4; tou4
U+38E0.0	fMandarin	tong2
U+38E0.0	fEnglish	same as &#x4F5F;
U+38E1.0	fUTF8	㣡
U+38E1.0	fR/S	 60.6
U+38E1.0	fTotalStrokes	 9
U+38E1.0	fCangjie	HOOL
U+38E1.0	fCantonese	zung6
U+38E2.0	fUTF8	㣢
U+38E2.0	fCNS11643	5-2D47
U+38E2.0	fR/S	 60.7
U+38E2.0	fTotalStrokes	 10
U+38E2.0	fCangjie	HOCNH
U+38E2.0	fFourCorner	28220
U+38E2.0	fCantonese	tai4; daai1; taai4
U+38E2.0	fMandarin	ti2
U+38E2.0	fEnglish	to rest from work; to take a rest
U+38E3.0	fUTF8	㣣
U+38E3.0	fCNS11643	F-2F3E
U+38E3.0	fR/S	 60.7
U+38E3.0	fTotalStrokes	 10
U+38E3.0	fCangjie	HOKOO
U+38E3.0	fFourCorner	2423
U+38E3.0	fCantonese	sim3; saam2; sim2
U+38E3.0	fMandarin	shan3; shan4
U+38E3.0	fEnglish	[1] walk in an unsteady way; swing to and from [2] doubt; suspect
U+38E4.0	fUTF8	㣤
U+38E4.0	fCNS11643	4-3154
U+38E4.0	fGB	5-3C28
U+38E4.0	fR/S	 60.8
U+38E4.0	fTotalStrokes	 11
U+38E4.0	fCangjie	HOII
U+38E4.0	fFourCorner	23250
U+38E4.0	fCantonese	cin5
U+38E4.0	fMandarin	jian4
U+38E4.0	fEnglish	same as &#x8E10;
U+38E5.0	fUTF8	㣥
U+38E5.0	fCNS11643	3-3566
U+38E5.0	fR/S	 60.8
U+38E5.0	fTotalStrokes	 11
U+38E5.0	fCangjie	HOOGI
U+38E5.0	fFourCorner	24240
U+38E5.0	fHakka	 cih5; cih6; sii5; shi6; shi5
U+38E5.0	fCantonese	zi6
U+38E5.0	fMandarin	zhi4
U+38E5.0	fEnglish	same as &#x506B;
U+38E6.0	fUTF8	㣦
U+38E6.0	fCNS11643	3-3567
U+38E6.0	fR/S	 60.8
U+38E6.0	fTotalStrokes	 11
U+38E6.0	fCangjie	HOHDV
U+38E6.0	fFourCorner	22240
U+38E6.0	fHakka	 vui1
U+38E6.0	fCantonese	wai1
U+38E6.0	fMandarin	wei1
U+38E6.0	fEnglish	same as &#x9036;
U+38E7.0	fUTF8	㣧
U+38E7.0	fR/S	 60.8
U+38E7.0	fTotalStrokes	 11
U+38E7.0	fCangjie	HOVBU
U+38E7.0	fCantonese	jan6; jaan6; zaan6
U+38E7.0	fMandarin	yin4; yong3
U+38E7.0	fEnglish	same as &#x80E4;
U+38E8.0	fUTF8	㣨
U+38E8.0	fR/S	 60.8
U+38E8.0	fTotalStrokes	 11
U+38E8.0	fCangjie	HOVJR
U+38E8.0	fCantonese	gu1; gwu1 (also)
U+38E9.0	fUTF8	㣩
U+38E9.0	fR/S	 60.8
U+38E9.0	fTotalStrokes	 11
U+38E9.0	fCangjie	HODD
U+38E9.0	fCantonese	lam4
U+38EA.0	fUTF8	㣪
U+38EA.0	fR/S	 60.9
U+38EA.0	fTotalStrokes	 12
U+38EA.0	fCangjie	HOBME
U+38EA.0	fFourCorner	2224
U+38EA.0	fCantonese	wun6
U+38EA.0	fMandarin	huan3; kuo4
U+38EA.0	fEnglish	walk with slow steps; proceed slowly
U+38EB.0	fUTF8	㣫
U+38EB.0	fCNS11643	4-3749
U+38EB.0	fGB	5-3C40
U+38EB.0	fR/S	 60.9
U+38EB.0	fTotalStrokes	 12
U+38EB.0	fCangjie	HOHJG
U+38EB.0	fFourCorner	22214
U+38EB.0	fCantonese	zung2; dung2
U+38EB.0	fMandarin	zhong3
U+38EB.0	fEnglish	same as &#x8E35;) the heel, to follow, to call personally at; to go personally to, (ancient form &#x52D5;) to move; to start; to shake, to excite; to rouse; to take action
U+38EC.0	fUTF8	㣬
U+38EC.0	fCNS11643	3-3B4A
U+38EC.0	fR/S	 60.9
U+38EC.0	fTotalStrokes	 12
U+38EC.0	fCangjie	HORSJ
U+38EC.0	fFourCorner	26240
U+38EC.0	fCantonese	cap1
U+38EC.0	fMandarin	qi4
U+38EC.0	fEnglish	to walk
U+38ED.0	fUTF8	㣭
U+38ED.0	fCNS11643	4-3748
U+38ED.0	fGB	5-3C41
U+38ED.0	fR/S	 60.9
U+38ED.0	fTotalStrokes	 12
U+38ED.0	fCangjie	HOUCE
U+38ED.0	fFourCorner	22240
U+38ED.0	fCantonese	zung1
U+38ED.0	fMandarin	zong1
U+38ED.0	fEnglish	number, several; a few, a kind of caldron or kettle, to walk
U+38EE.0	fUTF8	㣮
U+38EE.0	fR/S	 60.9
U+38EE.0	fTotalStrokes	 12
U+38EE.0	fCangjie	HOJBJ
U+38EE.0	fCantonese	naam4; laam4 (also)
U+38EF.0	fUTF8	㣯
U+38EF.0	fCNS11643	4-3D33
U+38EF.0	fGB	3-3A26
U+38EF.0	fR/S	 60.10
U+38EF.0	fTotalStrokes	 13
U+38EF.0	fCangjie	HOSFB
U+38EF.0	fFourCorner	27220
U+38EF.0	fHakka	  siet7
U+38EF.0	fCantonese	sit3
U+38EF.0	fMandarin	xie4
U+38EF.0	fEnglish	[1] wave [2] flutter
U+38F0.0	fUTF8	㣰
U+38F0.0	fCNS11643	5-4569
U+38F0.0	fGB	3-3A28
U+38F0.0	fR/S	 60.11
U+38F0.0	fTotalStrokes	 14
U+38F0.0	fCangjie	HOHDP
U+38F0.0	fFourCorner	22230
U+38F0.0	fCantonese	sat1; sik1
U+38F0.0	fMandarin	xi1; xie4
U+38F0.0	fEnglish	to shake; to rattle
U+38F1.0	fUTF8	㣱
U+38F1.0	fCNS11643	4-433D
U+38F1.0	fGB	3-3A21
U+38F1.0	fR/S	 60.11
U+38F1.0	fTotalStrokes	 14
U+38F1.0	fCangjie	HOQMC
U+38F1.0	fFourCorner	25280
U+38F1.0	fCantonese	zaak6; zak3
U+38F1.0	fMandarin	ze2; zuo4
U+38F1.0	fEnglish	common people; people in ordinary feature or countenance, to walk, to go in a hurry
U+38F2.0	fUTF8	㣲
U+38F2.0	fCNS11643	6-5C67
U+38F2.0	fR/S	 60.11
U+38F2.0	fTotalStrokes	 14
U+38F2.0	fCangjie	HOURK
U+38F2.0	fFourCorner	28240
U+38F2.0	fHakka	 zhin1
U+38F2.0	fCantonese	mei4; zi2; zing1
U+38F2.0	fMandarin	wei2; zheng1; zhi3
U+38F2.0	fEnglish	non-classical form of &#x5FAE;
U+38F3.0	fUTF8	㣳
U+38F3.0	fR/S	 60.12
U+38F3.0	fTotalStrokes	 15
U+38F3.0	fCangjie	HOOTF
U+38F3.0	fCantonese	mou4
U+38F4.0	fUTF8	㣴
U+38F4.0	fR/S	 60.12
U+38F4.0	fTotalStrokes	 15
U+38F4.0	fCangjie	HOTMC
U+38F4.0	fCantonese	wong4
U+38F5.0	fUTF8	㣵
U+38F5.0	fCNS11643	3-5060
U+38F5.0	fGB	5-3C30
U+38F5.0	fR/S	 60.13
U+38F5.0	fTotalStrokes	 16
U+38F5.0	fCangjie	HOYGQ
U+38F5.0	fFourCorner	24230
U+38F5.0	fCantonese	taat3
U+38F5.0	fMandarin	ta4
U+38F5.0	fEnglish	to run away; to flee; to escape; to evade, (interchangeable &#x9054;) to reach; to arrive at, to walk away without meeting each other
U+38F6.0	fUTF8	㣶
U+38F6.0	fCNS11643	4-5034
U+38F6.0	fGB	3-3A2B
U+38F6.0	fR/S	 60.13
U+38F6.0	fTotalStrokes	 16
U+38F6.0	fCangjie	HOYWM
U+38F6.0	fFourCorner	20210
U+38F6.0	fHakka	 zhan1; zen1; zan1; zhen1
U+38F6.0	fCantonese	zaan1; zin1
U+38F6.0	fMandarin	zhan1
U+38F6.0	fEnglish	to walk, to go, to hide; to conceal, very difficult to proceed
U+38F7.0	fUTF8	㣷
U+38F7.0	fCNS11643	4-562F
U+38F7.0	fR/S	 60.14
U+38F7.0	fTotalStrokes	 17; 16
U+38F7.0	fCangjie	HOJPN
U+38F7.0	fFourCorner	23220
U+38F7.0	fCantonese	ning6; ling6 (also)
U+38F7.0	fMandarin	nong4; ning4
U+38F7.0	fEnglish	walking
U+38F8.0	fUTF8	㣸
U+38F8.0	fR/S	 60.16
U+38F8.0	fTotalStrokes	 19
U+38F8.0	fCangjie	HOTOM
U+38F8.0	fCantonese	jeoi6
U+38F9.0	fUTF8	㣹
U+38F9.0	fR/S	 60.17
U+38F9.0	fTotalStrokes	 20
U+38F9.0	fCangjie	HOIOP
U+38F9.0	fCantonese	jing3
U+38FA.0	fUTF8	㣺
U+38FA.0	fCNS11643	6-217E
U+38FA.0	fR/S	 61.1
U+38FA.0	fTotalStrokes	 4; 5
U+38FA.0	fCangjie	PI
U+38FA.0	fFourCorner	93000
U+38FA.0	fHakka	 sim1
U+38FA.0	fCantonese	sam1
U+38FA.0	fMandarin	xin1
U+38FA.0	fEnglish	[1] the heart; the mind; intelligence; the moral nature; the centre [2] affections [3] same as &#x5FC3;
U+38FB.0	fUTF8	㣻
U+38FB.0	fCNS11643	3-2365
U+38FB.0	fGB	5-5269
U+38FB.0	fR/S	 61.2
U+38FB.0	fTotalStrokes	 6
U+38FB.0	fCangjie	KP
U+38FB.0	fFourCorner	40330
U+38FB.0	fCantonese	ngaai6; aai6 (also); ngai6
U+38FB.0	fMandarin	yi4
U+38FB.0	fEnglish	to remedy by punishment; to punish; to reprove; to warn, in great distress; suffering hardships; distress; trouble; worry
U+38FC.0	fUTF8	㣼
U+38FC.0	fCNS11643	3-2368
U+38FC.0	fR/S	 61.3
U+38FC.0	fTotalStrokes	 6
U+38FC.0	fCangjie	PSHI
U+38FC.0	fFourCorner	97020
U+38FC.0	fHakka	 ngiun3; niun1; ngiun1
U+38FC.0	fCantonese	jan2; zaan2; jan5
U+38FC.0	fMandarin	ren3; ren4
U+38FC.0	fEnglish	[1] endure [2] same as &#x5FCD;
U+38FD.0	fUTF8	㣽
U+38FD.0	fCNS11643	4-2431
U+38FD.0	fR/S	 61.3
U+38FD.0	fTotalStrokes	 7
U+38FD.0	fCangjie	VP
U+38FD.0	fFourCorner	40330
U+38FD.0	fHakka	 shu5
U+38FD.0	fCantonese	syu3
U+38FD.0	fMandarin	shu4; nu4
U+38FD.0	fEnglish	ancient form of &#x6055;
U+38FE.0	fUTF8	㣾
U+38FE.0	fCNS11643	5-2267
U+38FE.0	fR/S	 61.3
U+38FE.0	fTotalStrokes	 6
U+38FE.0	fCangjie	PEI
U+38FE.0	fFourCorner	97040
U+38FE.0	fCantonese	caa3
U+38FE.0	fMandarin	cha4
U+38FE.0	fEnglish	do not care; unmindful of
U+38FF.0	fUTF8	㣿
U+38FF.0	fCNS11643	4-2270
U+38FF.0	fGB	3-5133
U+38FF.0	fR/S	 61.3
U+38FF.0	fTotalStrokes	 6
U+38FF.0	fCangjie	PPI
U+38FF.0	fFourCorner	97020
U+38FF.0	fHakka	 diau2; diau5
U+38FF.0	fCantonese	zoek3; coek3; niu5
U+38FF.0	fMandarin	liao3; zhuo2
U+38FF.0	fEnglish	 hang up; suspend
U+3900.0	fUTF8	㤀
U+3900.0	fR/S	 61.3
U+3900.0	fTotalStrokes	 7
U+3900.0	fCangjie	OVP
U+3900.0	fCantonese	mong4
U+3901.0	fUTF8	㤁
U+3901.0	fCNS11643	3-287B
U+3901.0	fR/S	 61.4
U+3901.0	fTotalStrokes	 8
U+3901.0	fCangjie	MKP
U+3901.0	fFourCorner	20330
U+3901.0	fHakka	 tiam3
U+3901.0	fCantonese	tim2
U+3901.0	fMandarin	tian3; mian3
U+3901.0	fEnglish	ancient form of &#x5FDD;
U+3902.0	fUTF8	㤂
U+3902.0	fCNS11643	3-287D
U+3902.0	fR/S	 61.4
U+3902.0	fTotalStrokes	 8
U+3902.0	fCangjie	NHEP
U+3902.0	fFourCorner	17330
U+3902.0	fHakka	 gip7
U+3902.0	fCantonese	gap1
U+3902.0	fMandarin	ji2
U+3902.0	fEnglish	same as &#x6025;
U+3903.0	fUTF8	㤃
U+3903.0	fCNS11643	3-2579
U+3903.0	fGB	5-5351
U+3903.0	fR/S	 61.4
U+3903.0	fTotalStrokes	 7
U+3903.0	fCangjie	PYHS
U+3903.0	fFourCorner	90020
U+3903.0	fCantonese	fong1; fong4
U+3903.0	fMandarin	fang1; fang2
U+3903.0	fEnglish	[1] jealous; envy [2] same as &#x59A8;
U+3904.0	fUTF8	㤄
U+3904.0	fCNS11643	4-242E
U+3904.0	fGB	3-513A
U+3904.0	fR/S	 61.4
U+3904.0	fTotalStrokes	 7
U+3904.0	fCangjie	PJB
U+3904.0	fFourCorner	95020
U+3904.0	fCantonese	pui3
U+3904.0	fMandarin	pei4
U+3904.0	fEnglish	[1] raging animosity or hatred; full of anger and spite [2] unhappy; displeased
U+3905.0	fUTF8	㤅
U+3905.0	fCNS11643	3-287C
U+3905.0	fGB	5-427C
U+3905.0	fR/S	 61.4
U+3905.0	fTotalStrokes	 8
U+3905.0	fCangjie	MUP
U+3905.0	fFourCorner	10330
U+3905.0	fCantonese	oi3; ngoi3 (also)
U+3905.0	fMandarin	ai4
U+3905.0	fEnglish	[1] love; be found of; like; affection [2] breath [3] news [4] stop; end [5] interchangeable &#x5FCC; [6] small seal form of &#x611B;
U+3906.0	fUTF8	㤆
U+3906.0	fCNS11643	4-2430
U+3906.0	fGB	5-534C
U+3906.0	fR/S	 61.4
U+3906.0	fTotalStrokes	 7
U+3906.0	fCangjie	PHE
U+3906.0	fFourCorner	91040
U+3906.0	fHakka	 fan3; fan5
U+3906.0	fCantonese	faan3; faan2
U+3906.0	fMandarin	fan4
U+3906.0	fEnglish	 regret; penitent
U+3907.0	fUTF8	㤇
U+3907.0	fCNS11643	4-2433
U+3907.0	fGB	3-513D
U+3907.0	fR/S	 61.4
U+3907.0	fTotalStrokes	 7
U+3907.0	fCangjie	PHK
U+3907.0	fFourCorner	92030
U+3907.0	fCantonese	ou3; ngou3 (also); ou2
U+3907.0	fMandarin	ao3; fo2; wu4
U+3907.0	fEnglish	[1] terrified; scared; nervous [2] same as &#x61CA;
U+3908.0	fUTF8	㤈
U+3908.0	fCNS11643	3-2573
U+3908.0	fR/S	 61.4
U+3908.0	fTotalStrokes	 7
U+3908.0	fCangjie	PP
U+3908.0	fFourCorner	93000
U+3908.0	fCantonese	cam3; saam3
U+3908.0	fMandarin	qin4
U+3908.0	fEnglish	sad; sorrowful
U+3909.0	fUTF8	㤉
U+3909.0	fCNS11643	4-242F
U+3909.0	fGB	3-513C
U+3909.0	fR/S	 61.4
U+3909.0	fTotalStrokes	 7
U+3909.0	fCangjie	PMVH
U+3909.0	fFourCorner	91040
U+3909.0	fHakka	  nga2;  ka5; ka1
U+3909.0	fCantonese	kaa1; kaa3; kok3; ngaa4
U+3909.0	fMandarin	xian1; qia4; ya2
U+3909.0	fEnglish	 prostrate oneself with fear
U+390A.0	fUTF8	㤊
U+390A.0	fCNS11643	4-2432
U+390A.0	fGB	5-534D
U+390A.0	fR/S	 61.4
U+390A.0	fTotalStrokes	 7
U+390A.0	fCangjie	PKK
U+390A.0	fFourCorner	94040
U+390A.0	fCantonese	haau6
U+390A.0	fMandarin	xiao2; xiao4; yao2
U+390A.0	fEnglish	same as &#x6054;
U+390B.0	fUTF8	㤋
U+390B.0	fCNS11643	3-2624
U+390B.0	fR/S	 61.4
U+390B.0	fTotalStrokes	 7
U+390B.0	fCangjie	PCSH
U+390B.0	fFourCorner	98020
U+390B.0	fCantonese	fan1
U+390B.0	fMandarin	fen1
U+390B.0	fEnglish	confusion; confused or disorderly; chaotic
U+390C.0	fUTF8	㤌
U+390C.0	fCNS11643	3-2879
U+390C.0	fR/S	 61.5
U+390C.0	fTotalStrokes	 8
U+390C.0	fCangjie	PTM
U+390C.0	fFourCorner	94070
U+390C.0	fCantonese	gam1; gaam1; gaam3
U+390C.0	fMandarin	gan1
U+390C.0	fEnglish	same as &#x7518;
U+390D.0	fUTF8	㤍
U+390D.0	fCNS11643	4-2935
U+390D.0	fGB	5-526C
U+390D.0	fR/S	 61.5
U+390D.0	fTotalStrokes	 9
U+390D.0	fCangjie	MSP
U+390D.0	fFourCorner	11330
U+390D.0	fCantonese	haau1; diu6; hau1; hau2; tiu5; ziu4
U+390D.0	fMandarin	qiao1; qiao3
U+390D.0	fEnglish	[1] never reveal one's feeling or emotion [2] same as &#x5DE7;
U+390E.0	fUTF8	㤎
U+390E.0	fCNS11643	4-2937
U+390E.0	fGB	3-5074
U+390E.0	fR/S	 61.5
U+390E.0	fTotalStrokes	 9
U+390E.0	fCangjie	KRP
U+390E.0	fFourCorner	46330
U+390E.0	fCantonese	go1
U+390E.0	fMandarin	ge1
U+390E.0	fEnglish	[1] model; typical example [2] technique of standard handwriting [3] knowledge [4] know; feel; understand
U+390F.0	fUTF8	㤏
U+390F.0	fCNS11643	4-264D
U+390F.0	fGB	3-514A
U+390F.0	fR/S	 61.5
U+390F.0	fTotalStrokes	 8
U+390F.0	fCangjie	PHEY
U+390F.0	fFourCorner	97030
U+390F.0	fCantonese	dung2; tung4
U+390F.0	fMandarin	tong2
U+390F.0	fEnglish	grievous; mournful; melancholy
U+3910.0	fUTF8	㤐
U+3910.0	fCNS11643	3-2C59
U+3910.0	fR/S	 61.5
U+3910.0	fTotalStrokes	 9; 8
U+3910.0	fCangjie	YRP
U+3910.0	fFourCorner	21330
U+3910.0	fHakka	 tiap7
U+3910.0	fCantonese	tip3; zim1
U+3910.0	fMandarin	tie1; chan1
U+3910.0	fEnglish	ancient form of &#x6017;
U+3911.0	fUTF8	㤑
U+3911.0	fCNS11643	3-2924
U+3911.0	fR/S	 61.5
U+3911.0	fTotalStrokes	 8
U+3911.0	fCangjie	PKR
U+3911.0	fFourCorner	94060
U+3911.0	fCantonese	jau1; zaau6
U+3911.0	fMandarin	you4; you1
U+3911.0	fEnglish	non-classical form of &#x6182;
U+3912.0	fUTF8	㤒
U+3912.0	fCNS11643	4-2648
U+3912.0	fR/S	 61.5
U+3912.0	fTotalStrokes	 8
U+3912.0	fCangjie	PKJ
U+3912.0	fFourCorner	94040
U+3912.0	fCantonese	gou1
U+3912.0	fMandarin	gao1
U+3912.0	fEnglish	[1] knowledge; know; be aware of [2] bureau [3] state of affairs
U+3913.0	fUTF8	㤓
U+3913.0	fCNS11643	3-2923
U+3913.0	fGB	5-5359
U+3913.0	fR/S	 61.5
U+3913.0	fTotalStrokes	 8
U+3913.0	fCangjie	PDM
U+3913.0	fFourCorner	95030
U+3913.0	fHakka	 bun5; bun6
U+3913.0	fCantonese	ban6
U+3913.0	fMandarin	ben4
U+3913.0	fEnglish	stupid; doltish; sluggish
U+3914.0	fUTF8	㤔
U+3914.0	fCNS11643	4-264E
U+3914.0	fGB	5-535E
U+3914.0	fR/S	 61.5
U+3914.0	fTotalStrokes	 8
U+3914.0	fCangjie	PODI
U+3914.0	fFourCorner	94000
U+3914.0	fCantonese	fu6
U+3914.0	fMandarin	fu4; fu2
U+3914.0	fEnglish	 concede; submit willingly
U+3915.0	fUTF8	㤕
U+3915.0	fCNS11643	4-264C
U+3915.0	fGB	3-514F
U+3915.0	fR/S	 61.5
U+3915.0	fTotalStrokes	 8
U+3915.0	fCangjie	PUU
U+3915.0	fFourCorner	92070
U+3915.0	fHakka	  tut7; chot7; zot7
U+3915.0	fCantonese	ceot1; zeot1
U+3915.0	fMandarin	duo4; chu4
U+3915.0	fEnglish	grieved; pained
U+3916.0	fUTF8	㤖
U+3916.0	fCNS11643	4-2646
U+3916.0	fGB	5-5362
U+3916.0	fR/S	 61.5
U+3916.0	fTotalStrokes	 8
U+3916.0	fCangjie	PJMN
U+3916.0	fFourCorner	93020
U+3916.0	fCantonese	no6; lo6 (also); cyu5
U+3916.0	fMandarin	zhu4
U+3916.0	fEnglish	 have wisdom; intelligent
U+3917.0	fUTF8	㤗
U+3917.0	fCNS11643	F-2A3E
U+3917.0	fR/S	 61.5
U+3917.0	fTotalStrokes	 9
U+3917.0	fCangjie	QKP
U+3917.0	fCantonese	taai3
U+3918.0	fUTF8	㤘
U+3918.0	fGB	7-2241
U+3918.0	fR/S	 61.5
U+3918.0	fTotalStrokes	 8
U+3918.0	fCangjie	QNSM
U+3918.0	fCantonese	zau3; cau1 (also); zaau3
U+3918.0	fMandarin	chu4; cu4; zhou4
U+3918.0	fEnglish	simplified form of &#x396E;
U+3919.0	fUTF8	㤙
U+3919.0	fCNS11643	3-2C58
U+3919.0	fR/S	 61.5
U+3919.0	fTotalStrokes	 9
U+3919.0	fCangjie	WSP
U+3919.0	fFourCorner	60330
U+3919.0	fCantonese	jan1; zaan1
U+3919.0	fMandarin	en1
U+3919.0	fEnglish	same as &#x6069;
U+391A.0	fUTF8	㤚
U+391A.0	fCNS11643	4-2941
U+391A.0	fR/S	 61.6
U+391A.0	fTotalStrokes	 9
U+391A.0	fCangjie	PHON
U+391A.0	fFourCorner	91020
U+391A.0	fCantonese	hong4; hong2
U+391A.0	fMandarin	hang2
U+391A.0	fEnglish	joy; delight; gratification
U+391B.0	fUTF8	㤛
U+391B.0	fCNS11643	3-2C62
U+391B.0	fR/S	 61.6
U+391B.0	fTotalStrokes	 9
U+391B.0	fCangjie	POHG
U+391B.0	fFourCorner	92010
U+391B.0	fHakka	 ngim5; jim5
U+391B.0	fCantonese	jam6; zaam6
U+391B.0	fMandarin	ren4; nin2; nen4
U+391B.0	fEnglish	same as &#x6041;
U+391C.0	fUTF8	㤜
U+391C.0	fCNS11643	4-293C
U+391C.0	fR/S	 61.6
U+391C.0	fTotalStrokes	 9
U+391C.0	fCangjie	PIHI
U+391C.0	fFourCorner	93050
U+391C.0	fHakka	 tut7
U+391C.0	fCantonese	seot6; kyut3; waat6
U+391C.0	fMandarin	xu4; jue2; yu4
U+391C.0	fEnglish	violent; imperious
U+391D.0	fUTF8	㤝
U+391D.0	fCNS11643	4-2649
U+391D.0	fGB	3-5167
U+391D.0	fR/S	 61.6; 61.5
U+391D.0	fTotalStrokes	 9; 8
U+391D.0	fCangjie	PYIU
U+391D.0	fFourCorner	90010
U+391D.0	fHakka	 chung1; cung1
U+391D.0	fCantonese	cung1
U+391D.0	fMandarin	chong1
U+391D.0	fEnglish	[1] palpitation or fluttering of the heart [2] become interested in something (usually as a result of persuasion) [3] excited; agitated
U+391E.0	fUTF8	㤞
U+391E.0	fCNS11643	4-2939
U+391E.0	fGB	5-5372
U+391E.0	fR/S	 61.6
U+391E.0	fTotalStrokes	 9
U+391E.0	fCangjie	PJHP
U+391E.0	fFourCorner	93010
U+391E.0	fCantonese	caa3; dok6
U+391E.0	fMandarin	duo2; cha4
U+391E.0	fEnglish	[1] disappointed; fail in attaining one's purpose; be disappointed in attaining the ambitions [2] be surprised; be amazed; marvel [3] think; consider; ponder; contemplate [4] expect or hope with eagerness [5] credit
U+391F.0	fUTF8	㤟
U+391F.0	fCNS11643	3-307D
U+391F.0	fR/S	 61.6
U+391F.0	fTotalStrokes	 10
U+391F.0	fCangjie	TWP
U+391F.0	fFourCorner	55330
U+391F.0	fHakka	 kiung3
U+391F.0	fCantonese	hung2; kung2
U+391F.0	fMandarin	kong3; kong4; tou4
U+391F.0	fEnglish	same as &#x6050;
U+3920.0	fUTF8	㤠
U+3920.0	fCNS11643	6-3754
U+3920.0	fR/S	 61.6
U+3920.0	fTotalStrokes	 10; 9
U+3920.0	fCangjie	MNP
U+3920.0	fFourCorner	12330
U+3920.0	fCantonese	lit6; laau4
U+3920.0	fMandarin	lie4; li4
U+3920.0	fEnglish	standard form of &#x3921;
U+3921.0	fUTF8	㤡
U+3921.0	fCNS11643	4-293D
U+3921.0	fGB	3-515A
U+3921.0	fR/S	 61.6
U+3921.0	fTotalStrokes	 9
U+3921.0	fCangjie	PMNN
U+3921.0	fFourCorner	92000
U+3921.0	fCantonese	lai6
U+3921.0	fMandarin	li4
U+3921.0	fEnglish	[1] surprise; amaze [2] afraid; scared; fearful; terrified [3] grieved; distressed [4] same as &#x3920;
U+3922.0	fUTF8	㤢
U+3922.0	fCNS11643	5-2935
U+3922.0	fR/S	 61.6
U+3922.0	fTotalStrokes	 9
U+3922.0	fCangjie	PKB
U+3922.0	fFourCorner	94020
U+3922.0	fCantonese	juk1; zuk1
U+3922.0	fMandarin	yu4; xu4
U+3922.0	fEnglish	 move the mind; agitated; nervous; start thinking
U+3923.0	fUTF8	㤣
U+3923.0	fCNS11643	4-2D28
U+3923.0	fGB	3-5075
U+3923.0	fR/S	 61.6
U+3923.0	fTotalStrokes	 10
U+3923.0	fCangjie	TTP
U+3923.0	fFourCorner	80330
U+3923.0	fCantonese	paang1
U+3923.0	fMandarin	peng1
U+3923.0	fEnglish	same as &#x6072;
U+3924.0	fUTF8	㤤
U+3924.0	fCNS11643	4-2948
U+3924.0	fGB	3-515D
U+3924.0	fR/S	 61.6
U+3924.0	fTotalStrokes	 9
U+3924.0	fCangjie	PLWK
U+3924.0	fFourCorner	95000
U+3924.0	fHakka	 ji2
U+3924.0	fCantonese	jyu4; zyu4
U+3924.0	fMandarin	yu2; yu3
U+3924.0	fEnglish	[1] mournful; melancholy; distressed; grieved; sad [2] fear; dread
U+3925.0	fUTF8	㤥
U+3925.0	fCNS11643	4-293A
U+3925.0	fGB	3-5166
U+3925.0	fR/S	 61.6
U+3925.0	fTotalStrokes	 9
U+3925.0	fCangjie	PYVO
U+3925.0	fFourCorner	90082
U+3925.0	fHakka	 hoi5; hoi6
U+3925.0	fCantonese	hoi6
U+3925.0	fMandarin	hai4
U+3925.0	fEnglish	[1] sorrowful [2] anxious
U+3926.0	fUTF8	㤦
U+3926.0	fCNS11643	4-293B
U+3926.0	fGB	3-5157
U+3926.0	fR/S	 61.6
U+3926.0	fTotalStrokes	 9
U+3926.0	fCangjie	PJLK
U+3926.0	fFourCorner	95000
U+3926.0	fCantonese	lei6
U+3926.0	fMandarin	li4
U+3926.0	fEnglish	mournful; melancholy; distressed; grieved; sad
U+3927.0	fUTF8	㤧
U+3927.0	fCNS11643	4-2940
U+3927.0	fR/S	 61.6
U+3927.0	fTotalStrokes	 9
U+3927.0	fCangjie	PHMR
U+3927.0	fFourCorner	92060
U+3927.0	fCantonese	hau4
U+3927.0	fMandarin	hou2; hou4
U+3927.0	fEnglish	conciliation; come to an amicable understanding; compromise
U+3928.0	fUTF8	㤨
U+3928.0	fCNS11643	3-2C55
U+3928.0	fGB	5-5367
U+3928.0	fR/S	 61.6
U+3928.0	fTotalStrokes	 9
U+3928.0	fCangjie	PTC
U+3928.0	fFourCorner	94080
U+3928.0	fHakka	  giung3
U+3928.0	fCantonese	gung2
U+3928.0	fMandarin	gong3; qiong2
U+3928.0	fEnglish	 be alarmed; afraid
U+3929.0	fUTF8	㤩
U+3929.0	fCNS11643	6-375A
U+3929.0	fR/S	 61.6
U+3929.0	fTotalStrokes	 9; 10
U+3929.0	fCangjie	HERP
U+3929.0	fFourCorner	27330
U+3929.0	fHakka	 sii1
U+3929.0	fCantonese	kok3; si1; soi1
U+3929.0	fMandarin	ke4; si1; sai1; si4
U+3929.0	fEnglish	same as &#x601D;
U+392A.0	fUTF8	㤪
U+392A.0	fCNS11643	3-307C
U+392A.0	fR/S	 61.6
U+392A.0	fTotalStrokes	 10
U+392A.0	fCangjie	MNUP
U+392A.0	fFourCorner	10330
U+392A.0	fHakka	 jen5
U+392A.0	fCantonese	jyun3; wan4; zyun3
U+392A.0	fMandarin	yuan4; yuan1; yun3; yung4
U+392A.0	fEnglish	same as &#x6028;
U+392B.0	fUTF8	㤫
U+392B.0	fR/S	 61.6
U+392B.0	fTotalStrokes	 10
U+392B.0	fCangjie	KBP
U+392B.0	fFourCorner	4033
U+392B.0	fCantonese	dak1; daak1; jaam3; zaam3; jak6; zak6
U+392B.0	fMandarin	de2
U+392C.0	fUTF8	㤬
U+392C.0	fCNS11643	3-2C5E
U+392C.0	fGB	5-5366
U+392C.0	fR/S	 61.6
U+392C.0	fTotalStrokes	 9
U+392C.0	fCangjie	PGG
U+392C.0	fFourCorner	94010
U+392C.0	fHakka	 fui5
U+392C.0	fCantonese	wai6
U+392C.0	fMandarin	hui4; kui4
U+392C.0	fEnglish	same as &#x605A;
U+392D.0	fUTF8	㤭
U+392D.0	fGB	7-233F
U+392D.0	fR/S	 61.6
U+392D.0	fTotalStrokes	 9
U+392D.0	fCangjie	PHKL
U+392D.0	fCantonese	giu1
U+392D.0	fEnglish	untamed
U+392E.0	fUTF8	㤮
U+392E.0	fCNS11643	4-315B
U+392E.0	fGB	3-5077
U+392E.0	fR/S	 61.7
U+392E.0	fTotalStrokes	 11
U+392E.0	fCangjie	KGP
U+392E.0	fFourCorner	41330
U+392E.0	fHakka	 kong2
U+392E.0	fCantonese	gwong3; gong3 (also); gwong2; kwong4
U+392E.0	fMandarin	guang4; kuang2
U+392E.0	fEnglish	[1] error; blunder [2] cheat; swindle; defraud [3] confused; deceit; deceive
U+392F.0	fUTF8	㤯
U+392F.0	fCNS11643	5-2D5A
U+392F.0	fR/S	 61.7
U+392F.0	fTotalStrokes	 10
U+392F.0	fCangjie	PBCR
U+392F.0	fFourCorner	97020
U+392F.0	fCantonese	gwing2
U+392F.0	fMandarin	jiong1; jiong3; jun4
U+392F.0	fEnglish	[1] clever or smart in a small way [2] memory or recollection
U+3930.0	fUTF8	㤰
U+3930.0	fCNS11643	3-3573
U+3930.0	fGB	5-527C
U+3930.0	fR/S	 61.7
U+3930.0	fTotalStrokes	 11
U+3930.0	fCangjie	OSP
U+3930.0	fFourCorner	28330
U+3930.0	fHakka	 cok8
U+3930.0	fCantonese	zok3; zok6 (also)
U+3930.0	fMandarin	zuo4; zan3
U+3930.0	fEnglish	same as &#x600D;
U+3931.0	fUTF8	㤱
U+3931.0	fCNS11643	4-2D22
U+3931.0	fGB	3-5171
U+3931.0	fR/S	 61.7
U+3931.0	fTotalStrokes	 10
U+3931.0	fCangjie	PMRT
U+3931.0	fFourCorner	91010
U+3931.0	fHakka	 teu5
U+3931.0	fCantonese	fu3
U+3931.0	fMandarin	fu4
U+3931.0	fEnglish	[1] anger; passion; rage [2] wait [3] cheat; swindle
U+3932.0	fUTF8	㤲
U+3932.0	fCNS11643	3-3569
U+3932.0	fGB	5-5273
U+3932.0	fR/S	 61.7
U+3932.0	fTotalStrokes	 11
U+3932.0	fCangjie	KOP
U+3932.0	fFourCorner	40330
U+3932.0	fCantonese	hip3; hap6
U+3932.0	fMandarin	qie4; qu4
U+3932.0	fEnglish	[1] think; contemplate; consider [2] pant because of fear
U+3933.0	fUTF8	㤳
U+3933.0	fCNS11643	3-3075
U+3933.0	fR/S	 61.7
U+3933.0	fTotalStrokes	 10
U+3933.0	fCangjie	PMFR
U+3933.0	fFourCorner	91060
U+3933.0	fCantonese	bui2; baak1
U+3933.0	fMandarin	bai3; bei3
U+3933.0	fEnglish	 depend on
U+3934.0	fUTF8	㤴
U+3934.0	fCNS11643	4-2D23
U+3934.0	fGB	3-516F
U+3934.0	fR/S	 61.7
U+3934.0	fTotalStrokes	 10
U+3934.0	fCangjie	PSJU
U+3934.0	fFourCorner	91010
U+3934.0	fCantonese	cip3; cap3
U+3934.0	fMandarin	zhe2; xi2
U+3934.0	fEnglish	[1] move the mind; agitated; nervous; start thinking [2] rest; cease [3] intelligent [4] strange; uncanny [5] light aspirations; ambitions for ...
U+3935.0	fUTF8	㤵
U+3935.0	fCNS11643	3-3570
U+3935.0	fR/S	 61.7
U+3935.0	fTotalStrokes	 11
U+3935.0	fCangjie	NKP
U+3935.0	fFourCorner	18330
U+3935.0	fHakka	 cih2
U+3935.0	fCantonese	ci4
U+3935.0	fMandarin	ci2
U+3935.0	fEnglish	same as &#x6148;
U+3936.0	fUTF8	㤶
U+3936.0	fCNS11643	4-2D24
U+3936.0	fGB	3-5172
U+3936.0	fR/S	 61.7
U+3936.0	fTotalStrokes	 10
U+3936.0	fCangjie	PIUH
U+3936.0	fFourCorner	93010
U+3936.0	fHakka	 mang2
U+3936.0	fCantonese	mong4; pong4
U+3936.0	fMandarin	mang2; pang2
U+3936.0	fEnglish	[1] confuse [2] senile [3] stupid
U+3937.0	fUTF8	㤷
U+3937.0	fCNS11643	3-3077
U+3937.0	fR/S	 61.7
U+3937.0	fTotalStrokes	 10
U+3937.0	fCangjie	POIR
U+3937.0	fFourCorner	98060
U+3937.0	fCantonese	ham1; haam4
U+3937.0	fMandarin	han2
U+3937.0	fEnglish	 debauch; dissipate; act uninhibitively or disrespectfully
U+3938.0	fUTF8	㤸
U+3938.0	fCNS11643	6-375E
U+3938.0	fGB	3-516E
U+3938.0	fR/S	 61.7
U+3938.0	fTotalStrokes	 9; 10
U+3938.0	fCangjie	PGLC
U+3938.0	fFourCorner	94030
U+3938.0	fCantonese	hik1; zik1
U+3938.0	fMandarin	xi4
U+3938.0	fEnglish	uneasy; distrubed; not feeling at peace
U+3939.0	fUTF8	㤹
U+3939.0	fCNS11643	3-3124
U+3939.0	fGB	5-5375
U+3939.0	fR/S	 61.7
U+3939.0	fTotalStrokes	 10
U+3939.0	fCangjie	PIJE
U+3939.0	fFourCorner	93030
U+3939.0	fCantonese	kau4; gaau3; kaau4
U+3939.0	fMandarin	qiu2
U+3939.0	fEnglish	[1] hatred; grudge; enmity; animosity [2] interchangeable with &#x6551;
U+393A.0	fUTF8	㤺
U+393A.0	fCNS11643	4-2D2B
U+393A.0	fGB	5-5371
U+393A.0	fR/S	 61.6
U+393A.0	fTotalStrokes	 10; 9
U+393A.0	fCangjie	PYVU
U+393A.0	fFourCorner	94010
U+393A.0	fCantonese	fong1; fong2; wok6
U+393A.0	fMandarin	huang1; huang3
U+393A.0	fEnglish	corrupt form of &#x614C;
U+393B.0	fUTF8	㤻
U+393B.0	fCNS11643	4-315A
U+393B.0	fGB	3-5078
U+393B.0	fR/S	 61.7
U+393B.0	fTotalStrokes	 11
U+393B.0	fCangjie	HLP
U+393B.0	fFourCorner	77330
U+393B.0	fHakka	 lun5; lun3; lon3
U+393B.0	fCantonese	tung4; nan2
U+393B.0	fEnglish	 act heedlessly
U+393C.0	fUTF8	㤼
U+393C.0	fR/S	 61.7
U+393C.0	fTotalStrokes	 10
U+393C.0	fCangjie	PGIS
U+393C.0	fCantonese	gip3
U+393D.0	fUTF8	㤽
U+393D.0	fGB	7-2343
U+393D.0	fR/S	 61.7
U+393D.0	fTotalStrokes	 10
U+393D.0	fCangjie	PQKI
U+393D.0	fCantonese	cau4; caau4
U+393D.0	fMandarin	chou2
U+393D.0	fEnglish	simplified form of &#x61E4;
U+393E.0	fUTF8	㤾
U+393E.0	fCNS11643	4-315C
U+393E.0	fGB	3-523B
U+393E.0	fR/S	 61.8
U+393E.0	fTotalStrokes	 11
U+393E.0	fCangjie	PBCD
U+393E.0	fFourCorner	97090
U+393E.0	fCantonese	saam3; san3
U+393E.0	fMandarin	san4
U+393E.0	fEnglish	[1] not doing well; disappointed; very discouraged; frustrate [2] same as &#x619B;
U+393F.0	fUTF8	㤿
U+393F.0	fCNS11643	4-3162
U+393F.0	fGB	3-5229
U+393F.0	fR/S	 61.8
U+393F.0	fTotalStrokes	 11
U+393F.0	fCangjie	PKLU
U+393F.0	fFourCorner	94010
U+393F.0	fHakka	 jam1; jam3; ram1; ram3
U+393F.0	fCantonese	jim1
U+393F.0	fMandarin	yan1
U+393F.0	fEnglish	[1] pleased; joyful [2] fussy; arrogant; too particular about somebody; something
U+3940.0	fUTF8	㥀
U+3940.0	fCNS11643	3-356B
U+3940.0	fGB	5-537A
U+3940.0	fR/S	 61.8
U+3940.0	fTotalStrokes	 11
U+3940.0	fCangjie	PJBM
U+3940.0	fFourCorner	94010
U+3940.0	fCantonese	zik6; daak1
U+3940.0	fMandarin	shi2; de2
U+3940.0	fEnglish	[1] exclusive; special [2] same as &#x5FB7;
U+3941.0	fUTF8	㥁
U+3941.0	fCNS11643	6-486F
U+3941.0	fR/S	 61.8
U+3941.0	fTotalStrokes	 12
U+3941.0	fCangjie	JWLP
U+3941.0	fFourCorner	40330
U+3941.0	fCantonese	dak1
U+3941.0	fMandarin	de2; zhi2; zhou4
U+3941.0	fEnglish	same as &#x5FB7;
U+3942.0	fUTF8	㥂
U+3942.0	fCNS11643	4-3163
U+3942.0	fGB	3-522A
U+3942.0	fR/S	 61.8
U+3942.0	fTotalStrokes	 11
U+3942.0	fCangjie	PAMI
U+3942.0	fFourCorner	96040
U+3942.0	fCantonese	tik1; tim5
U+3942.0	fMandarin	te4
U+3942.0	fEnglish	[1] quick; fast [2] sharp, move the mind; agitated; nervous; start thinking [3] fear; dread; be scared of [4] get; obtain
U+3943.0	fUTF8	㥃
U+3943.0	fCNS11643	3-3575
U+3943.0	fR/S	 61.8
U+3943.0	fTotalStrokes	 11
U+3943.0	fCangjie	PAN
U+3943.0	fFourCorner	97020
U+3943.0	fHakka	  mun3; mun6; mun5
U+3943.0	fCantonese	mun6
U+3943.0	fMandarin	men4; men2; men3
U+3943.0	fEnglish	same as &#x60B6;
U+3944.0	fUTF8	㥄
U+3944.0	fCNS11643	4-3160
U+3944.0	fGB	3-5225
U+3944.0	fR/S	 61.8
U+3944.0	fTotalStrokes	 11
U+3944.0	fCangjie	PGCE
U+3944.0	fFourCorner	94040
U+3944.0	fHakka	 lin2
U+3944.0	fCantonese	ling4
U+3944.0	fMandarin	ling2
U+3944.0	fEnglish	[1] pity; commiserate; have sympathy or compassion for [2] scared; afraid; fearful
U+3945.0	fUTF8	㥅
U+3945.0	fCNS11643	5-325B
U+3945.0	fGB	3-5232
U+3945.0	fR/S	 61.8
U+3945.0	fTotalStrokes	 11
U+3945.0	fCangjie	PBBE
U+3945.0	fFourCorner	92040
U+3945.0	fCantonese	sau6
U+3945.0	fMandarin	shou4
U+3945.0	fEnglish	[1] used in a person's name [2] pensive
U+3946.0	fUTF8	㥆
U+3946.0	fCNS11643	4-315D
U+3946.0	fGB	3-523C
U+3946.0	fR/S	 61.8
U+3946.0	fTotalStrokes	 11
U+3946.0	fCangjie	PLE
U+3946.0	fFourCorner	95030
U+3946.0	fCantonese	teoi3
U+3946.0	fMandarin	tu2; dian4; tui4
U+3946.0	fEnglish	[1] indulgent and without restraint [2] abandon oneself tocarnal desire; be dissolute; debauche [3] forget [4] slow; delay
U+3947.0	fUTF8	㥇
U+3947.0	fCNS11643	3-356C
U+3947.0	fR/S	 61.8
U+3947.0	fTotalStrokes	 11
U+3947.0	fCangjie	PII
U+3947.0	fFourCorner	93050
U+3947.0	fCantonese	caan4; can1
U+3947.0	fMandarin	can2; can4
U+3947.0	fEnglish	[1] be perverse; act contrary to [2] jealous; envy; jealousy [3] obstinate; stubborn [4] fierce; truculent
U+3948.0	fUTF8	㥈
U+3948.0	fCNS11643	3-3B4E
U+3948.0	fGB	5-5323
U+3948.0	fR/S	 61.8
U+3948.0	fTotalStrokes	 12
U+3948.0	fCangjie	LRP
U+3948.0	fFourCorner	41330
U+3948.0	fCantonese	dip6
U+3948.0	fMandarin	die2
U+3948.0	fEnglish	interchangeable with &#x60F5;
U+3949.0	fUTF8	㥉
U+3949.0	fCNS11643	4-316C
U+3949.0	fR/S	 61.8
U+3949.0	fTotalStrokes	 11
U+3949.0	fCangjie	PYTR
U+3949.0	fFourCorner	90060
U+3949.0	fCantonese	cai3; pei1
U+3949.0	fMandarin	pou3; che4; chi4
U+3949.0	fEnglish	a little angry
U+394A.0	fUTF8	㥊
U+394A.0	fCNS11643	4-3167
U+394A.0	fGB	5-537E
U+394A.0	fR/S	 61.8
U+394A.0	fTotalStrokes	 11
U+394A.0	fCangjie	PBB
U+394A.0	fFourCorner	97020
U+394A.0	fCantonese	pang4
U+394A.0	fMandarin	peng2
U+394A.0	fEnglish	[1] angry [2] same as &#x6072;
U+394B.0	fUTF8	㥋
U+394B.0	fCNS11643	3-3B4C
U+394B.0	fR/S	 61.8
U+394B.0	fTotalStrokes	 12
U+394B.0	fCangjie	OVP
U+394B.0	fFourCorner	20330
U+394B.0	fCantonese	ji1; zi1
U+394B.0	fMandarin	yi1
U+394B.0	fEnglish	sound of the anguish of sorrow; the pain of grief
U+394C.0	fUTF8	㥌
U+394C.0	fCNS11643	3-356D
U+394C.0	fR/S	 61.8
U+394C.0	fTotalStrokes	 11
U+394C.0	fCangjie	PPFD
U+394C.0	fFourCorner	97020
U+394C.0	fCantonese	guk1
U+394C.0	fMandarin	ju2
U+394C.0	fEnglish	prudent; cautious
U+394D.0	fUTF8	㥍
U+394D.0	fCNS11643	3-3577
U+394D.0	fGB	5-5379
U+394D.0	fR/S	 61.8
U+394D.0	fTotalStrokes	 11
U+394D.0	fCangjie	PTMC
U+394D.0	fFourCorner	94080
U+394D.0	fHakka	 ki5
U+394D.0	fCantonese	gei6
U+394D.0	fMandarin	ji4
U+394D.0	fEnglish	same as &#x60CE;
U+394E.0	fUTF8	㥎
U+394E.0	fCNS11643	3-3B57
U+394E.0	fR/S	 61.8
U+394E.0	fTotalStrokes	 12
U+394E.0	fCangjie	HHP
U+394E.0	fFourCorner	27330
U+394E.0	fCantonese	lei4; laai4
U+394E.0	fMandarin	li2; lai2
U+394E.0	fEnglish	[1] (abbreviated form) [2] resent; hate [3] neglect; negligent [4] joy; delight; gratification
U+394F.0	fUTF8	㥏
U+394F.0	fCNS11643	4-3164
U+394F.0	fGB	3-522E
U+394F.0	fR/S	 61.8
U+394F.0	fTotalStrokes	 11
U+394F.0	fCangjie	PTBC
U+394F.0	fFourCorner	95080
U+394F.0	fHakka	  tien3
U+394F.0	fCantonese	tin2 (also); tin5
U+394F.0	fMandarin	tian3
U+394F.0	fEnglish	bashful; shy; be ashamed
U+3950.0	fUTF8	㥐
U+3950.0	fCNS11643	6-4875
U+3950.0	fR/S	 61.8
U+3950.0	fTotalStrokes	 12
U+3950.0	fCangjie	OMRP
U+3950.0	fFourCorner	80330
U+3950.0	fHakka	 jan2
U+3950.0	fCantonese	jyun3; zyun3
U+3950.0	fMandarin	yuan4
U+3950.0	fEnglish	ancient form of &#x6028;
U+3951.0	fUTF8	㥑
U+3951.0	fCNS11643	3-3B53
U+3951.0	fGB	5-5321
U+3951.0	fR/S	 61.8
U+3951.0	fTotalStrokes	 12
U+3951.0	fCangjie	MBP
U+3951.0	fFourCorner	10330
U+3951.0	fCantonese	jau1; zaau1
U+3951.0	fMandarin	ou1
U+3951.0	fEnglish	corrupt form of &#x6182;
U+3952.0	fUTF8	㥒
U+3952.0	fCNS11643	4-3166
U+3952.0	fGB	3-5231
U+3952.0	fR/S	 61.8
U+3952.0	fTotalStrokes	 11
U+3952.0	fCangjie	PBD
U+3952.0	fFourCorner	92094
U+3952.0	fCantonese	coi2
U+3952.0	fMandarin	cai3
U+3952.0	fEnglish	[1] corrupt; wicked, hate; dislike [2] same as &#x731C;
U+3953.0	fUTF8	㥓
U+3953.0	fCNS11643	4-315E
U+3953.0	fGB	3-5228
U+3953.0	fR/S	 61.8
U+3953.0	fTotalStrokes	 11
U+3953.0	fCangjie	PKMR
U+3953.0	fFourCorner	94020
U+3953.0	fCantonese	kei1; zi2
U+3953.0	fMandarin	qi1; qi3
U+3953.0	fEnglish	[1] happy; contented; pleased [2] fragal, nearly; almost [3] cunning; crafty
U+3954.0	fUTF8	㥔
U+3954.0	fCNS11643	5-2D57
U+3954.0	fGB	S-2129
U+3954.0	fR/S	 61.7; 61.8
U+3954.0	fTotalStrokes	 10; 11
U+3954.0	fCangjie	PYIB
U+3954.0	fFourCorner	90020
U+3954.0	fCantonese	juk6; zuk6
U+3954.0	fMandarin	yu4; yu2
U+3954.0	fEnglish	 move the mind; agitated; nervous; start thinking
U+3955.0	fUTF8	㥕
U+3955.0	fCNS11643	3-3B52
U+3955.0	fR/S	 61.8
U+3955.0	fTotalStrokes	 12
U+3955.0	fCangjie	FFP
U+3955.0	fFourCorner	90330
U+3955.0	fHakka	 lin2; lien2
U+3955.0	fCantonese	lin4
U+3955.0	fMandarin	lian2; lin2
U+3955.0	fEnglish	ancient form of &#x6190;
U+3956.0	fUTF8	㥖
U+3956.0	fCNS11643	5-3842
U+3956.0	fR/S	 61.8
U+3956.0	fTotalStrokes	 12
U+3956.0	fCangjie	BYCIP
U+3956.0	fFourCorner	78430
U+3956.0	fCantonese	cung1
U+3956.0	fMandarin	cong1
U+3956.0	fEnglish	red color
U+3957.0	fUTF8	㥗
U+3957.0	fGB	7-2344
U+3957.0	fR/S	 61.8
U+3957.0	fTotalStrokes	 11
U+3957.0	fCangjie	PAYK
U+3957.0	fCantonese	man5
U+3958.0	fUTF8	㥘
U+3958.0	fR/S	 61.8
U+3958.0	fTotalStrokes	 11
U+3958.0	fCangjie	PGII
U+3958.0	fCantonese	gip3
U+3959.0	fUTF8	㥙
U+3959.0	fR/S	 61.8
U+3959.0	fTotalStrokes	 11
U+3959.0	fCangjie	PLP
U+3959.0	fCantonese	zung1
U+395A.0	fUTF8	㥚
U+395A.0	fCNS11643	4-316A
U+395A.0	fGB	5-537C
U+395A.0	fR/S	 61.9; 61.8
U+395A.0	fTotalStrokes	 12; 11
U+395A.0	fCangjie	PHXO
U+395A.0	fFourCorner	97030
U+395A.0	fHakka	 ji2; ri2; i2
U+395A.0	fCantonese	jyu4; zyu4; zyu5
U+395A.0	fMandarin	yu2
U+395A.0	fEnglish	sorrowfull; grieved
U+395B.0	fUTF8	㥛
U+395B.0	fCNS11643	5-3848
U+395B.0	fGB	3-523E
U+395B.0	fR/S	 61.9
U+395B.0	fTotalStrokes	 12
U+395B.0	fCangjie	PMEM
U+395B.0	fFourCorner	91010
U+395B.0	fCantonese	gik6
U+395B.0	fMandarin	ji2; ke4; su4
U+395B.0	fEnglish	[1] with expedition; urgent; hasty [2] anxious; worried [3] respectrully serious; take careful precautions; do something in a serious manner [4] love; like; be found of; be kind to
U+395C.0	fUTF8	㥜
U+395C.0	fCNS11643	4-3756
U+395C.0	fGB	3-5245
U+395C.0	fR/S	 61.9
U+395C.0	fTotalStrokes	 12
U+395C.0	fCangjie	PWB
U+395C.0	fFourCorner	96020
U+395C.0	fHakka	 wi5; wui6; vui5; wui5
U+395C.0	fCantonese	wai6; syut3; waai6
U+395C.0	fMandarin	wei4
U+395C.0	fEnglish	anxious
U+395D.0	fUTF8	㥝
U+395D.0	fCNS11643	4-3750
U+395D.0	fGB	5-5433
U+395D.0	fR/S	 61.9
U+395D.0	fTotalStrokes	 12
U+395D.0	fCangjie	PNSJ
U+395D.0	fFourCorner	91040
U+395D.0	fCantonese	mei5; maai5
U+395D.0	fMandarin	mian3; mi3
U+395D.0	fEnglish	[1] arouse to action; encourage [2] temper and grind; train or discipline oneself (toward a goal); sharpen (a knife); forge to harden [3] same as &#x5F2D;
U+395E.0	fUTF8	㥞
U+395E.0	fCNS11643	3-3B5C
U+395E.0	fGB	5-5431
U+395E.0	fR/S	 61.9
U+395E.0	fTotalStrokes	 12
U+395E.0	fCangjie	PTPO
U+395E.0	fFourCorner	98030
U+395E.0	fCantonese	seoi3; seoi6
U+395E.0	fMandarin	sui4; cui4; qian4
U+395E.0	fEnglish	[1] think deeply [2] think far ahead [3] profound and abstruse thinking [4] careful deliberations; think and contemplate thoroughly
U+395F.0	fUTF8	㥟
U+395F.0	fCNS11643	4-3759
U+395F.0	fGB	3-5251
U+395F.0	fR/S	 61.9
U+395F.0	fTotalStrokes	 12
U+395F.0	fCangjie	PVNO
U+395F.0	fFourCorner	97030
U+395F.0	fCantonese	haai4; kwaai4
U+395F.0	fMandarin	xie2
U+395F.0	fEnglish	enmity; animus; ill-will
U+3960.0	fUTF8	㥠
U+3960.0	fCNS11643	3-3B5F
U+3960.0	fGB	5-5435
U+3960.0	fR/S	 61.9
U+3960.0	fTotalStrokes	 12
U+3960.0	fCangjie	PNOB
U+3960.0	fFourCorner	97020
U+3960.0	fCantonese	seoi1
U+3960.0	fMandarin	xu1; xu3
U+3960.0	fEnglish	same as &#x8ADD;
U+3961.0	fUTF8	㥡
U+3961.0	fCNS11643	4-3754
U+3961.0	fGB	3-524F
U+3961.0	fR/S	 61.9
U+3961.0	fTotalStrokes	 12
U+3961.0	fCangjie	PSPD
U+3961.0	fFourCorner	97090
U+3961.0	fCantonese	ci3; hei3
U+3961.0	fMandarin	chi4; xi4
U+3961.0	fEnglish	idle; shiftless
U+3962.0	fUTF8	㥢
U+3962.0	fCNS11643	4-374E
U+3962.0	fGB	3-524C
U+3962.0	fR/S	 61.9
U+3962.0	fTotalStrokes	 12
U+3962.0	fCangjie	PTCW
U+3962.0	fFourCorner	98060
U+3962.0	fCantonese	cau4; zaau1; zaau4
U+3962.0	fMandarin	qiu2
U+3962.0	fEnglish	[1] bad; evil; wicked; vice [2] overbearing; haughty [3] melancholy; depressed; dejected; cheerless; despondent [4] considerations; contemplations; think carefully; turn over and over in the mind
U+3963.0	fUTF8	㥣
U+3963.0	fCNS11643	5-3F32
U+3963.0	fR/S	 61.9
U+3963.0	fTotalStrokes	 13
U+3963.0	fCangjie	KGGP
U+3963.0	fFourCorner	40330
U+3963.0	fCantonese	wai6
U+3963.0	fMandarin	hui4
U+3963.0	fEnglish	ancient form of &#x60E0;
U+3964.0	fUTF8	㥤
U+3964.0	fCNS11643	3-4125
U+3964.0	fR/S	 61.9
U+3964.0	fTotalStrokes	 13
U+3964.0	fCangjie	NNP
U+3964.0	fFourCorner	18330
U+3964.0	fHakka	 kim1
U+3964.0	fCantonese	ging1; gwan1; kaan4; ziu2
U+3964.0	fMandarin	qin2; qing2; jin1
U+3964.0	fEnglish	same as &#x77DC;
U+3965.0	fUTF8	㥥
U+3965.0	fCNS11643	4-3755
U+3965.0	fGB	5-542A
U+3965.0	fR/S	 61.9
U+3965.0	fTotalStrokes	 12
U+3965.0	fCangjie	PWLB
U+3965.0	fFourCorner	96020
U+3965.0	fCantonese	jyu4; zyu4
U+3965.0	fMandarin	yu2
U+3965.0	fEnglish	[1] absent-minded; unconscious [2] happy; glad; joyous
U+3966.0	fUTF8	㥦
U+3966.0	fCNS11643	4-3D3E
U+3966.0	fR/S	 61.9
U+3966.0	fTotalStrokes	 13
U+3966.0	fCangjie	SOP
U+3966.0	fFourCorner	71330
U+3966.0	fHakka	 giap7
U+3966.0	fCantonese	hip3; hip6 (also); heoi3; tiu4
U+3966.0	fMandarin	qie4; xia2; xian3
U+3966.0	fEnglish	same as &#x611C;
U+3967.0	fUTF8	㥧
U+3967.0	fCNS11643	3-3B60
U+3967.0	fR/S	 61.9
U+3967.0	fTotalStrokes	 12
U+3967.0	fCangjie	PMBC
U+3967.0	fFourCorner	91080
U+3967.0	fHakka	 shun5
U+3967.0	fCantonese	seon6
U+3967.0	fMandarin	shun4
U+3967.0	fEnglish	ancient form of &#x9806;
U+3968.0	fUTF8	㥨
U+3968.0	fCNS11643	4-3D37
U+3968.0	fGB	3-507E
U+3968.0	fR/S	 61.9
U+3968.0	fTotalStrokes	 13
U+3968.0	fCangjie	VGP
U+3968.0	fFourCorner	44330
U+3968.0	fCantonese	seoi6; ceoi4
U+3968.0	fMandarin	shui3; chui2; shui4; wei3
U+3968.0	fEnglish	[1] unhappy; displeased [2] anger; rage; angry; furious
U+3969.0	fUTF8	㥩
U+3969.0	fCNS11643	5-3258
U+3969.0	fR/S	 61.9; 61.8
U+3969.0	fTotalStrokes	 11; 12
U+3969.0	fCangjie	PJKA
U+3969.0	fFourCorner	94060
U+3969.0	fCantonese	do2; seoi6
U+3969.0	fMandarin	duo3
U+3969.0	fEnglish	idle; lazy; slothful; inactive
U+396A.0	fUTF8	㥪
U+396A.0	fGB	7-2340
U+396A.0	fR/S	 61.9
U+396A.0	fTotalStrokes	 12
U+396A.0	fCangjie	PFDV
U+396A.0	fCantonese	lau4; laau4
U+396A.0	fMandarin	lou2
U+396A.0	fEnglish	simplified form of &#x617A;
U+396B.0	fUTF8	㥫
U+396B.0	fCNS11643	3-3B5B
U+396B.0	fR/S	 61.9
U+396B.0	fTotalStrokes	 12
U+396B.0	fCangjie	PYRA
U+396B.0	fFourCorner	90060
U+396B.0	fCantonese	deon1; seoi3
U+396B.0	fEnglish	variant of &#x60C7;
U+396C.0	fUTF8	㥬
U+396C.0	fCNS11643	3-4126
U+396C.0	fR/S	 61.10
U+396C.0	fTotalStrokes	 13
U+396C.0	fCangjie	PYBS
U+396C.0	fFourCorner	90020
U+396C.0	fHakka	 pong2
U+396C.0	fCantonese	pong4
U+396C.0	fMandarin	pang2
U+396C.0	fEnglish	timid; fearful
U+396D.0	fUTF8	㥭
U+396D.0	fCNS11643	3-412D
U+396D.0	fR/S	 61.10
U+396D.0	fTotalStrokes	 13
U+396D.0	fCangjie	PQKE
U+396D.0	fFourCorner	95030
U+396D.0	fHakka	  tai5
U+396D.0	fCantonese	taai3; tai3; sai6
U+396D.0	fMandarin	tai4; dai4; she2; shi4; tai3
U+396D.0	fEnglish	same as &#x5FD5;
U+396E.0	fUTF8	㥮
U+396E.0	fCNS11643	4-3D3B
U+396E.0	fGB	3-514B
U+396E.0	fR/S	 61.10
U+396E.0	fTotalStrokes	 13
U+396E.0	fCangjie	PPUU
U+396E.0	fFourCorner	97020
U+396E.0	fCantonese	zau3; cau1 (also); zaau3
U+396E.0	fMandarin	chao3; zhou4
U+396E.0	fEnglish	[1] obstinate; stubborn; opinionated; obstinacy; stubornness; intransigent [2] truculent; savage [3] ferocious; fierce
U+396F.0	fUTF8	㥯
U+396F.0	fCNS11643	4-4343
U+396F.0	fGB	3-4C39
U+396F.0	fR/S	 61.10
U+396F.0	fTotalStrokes	 14
U+396F.0	fCangjie	BMSP
U+396F.0	fFourCorner	20337
U+396F.0	fHakka	 jun3; run3
U+396F.0	fCantonese	jan2; zaan2
U+396F.0	fMandarin	yin4; yin3
U+396F.0	fEnglish	 take an interest in; care for
U+3970.0	fUTF8	㥰
U+3970.0	fCNS11643	5-3F42
U+3970.0	fR/S	 61.10
U+3970.0	fTotalStrokes	 13
U+3970.0	fCangjie	PHXE
U+3970.0	fFourCorner	97040
U+3970.0	fCantonese	sou1
U+3970.0	fMandarin	sao1
U+3970.0	fEnglish	mournful; melancholy; distressed; grieved; sad
U+3971.0	fUTF8	㥱
U+3971.0	fCNS11643	6-5269
U+3971.0	fR/S	 61.10
U+3971.0	fTotalStrokes	 13
U+3971.0	fCangjie	PSLY
U+3971.0	fFourCorner	91010
U+3971.0	fHakka	 fui3
U+3971.0	fCantonese	fei2
U+3971.0	fMandarin	fei3
U+3971.0	fEnglish	same as &#x60B1;
U+3972.0	fUTF8	㥲
U+3972.0	fCNS11643	3-4667
U+3972.0	fR/S	 61.10
U+3972.0	fTotalStrokes	 14
U+3972.0	fCangjie	JCP
U+3972.0	fFourCorner	40330
U+3972.0	fCantonese	can1
U+3972.0	fMandarin	chen1; shen4; yin2
U+3972.0	fEnglish	[1] same as &#x8B13; [2] ancient form of &#x614E;
U+3973.0	fUTF8	㥳
U+3973.0	fCNS11643	4-3D3A
U+3973.0	fGB	3-5253
U+3973.0	fR/S	 61.10
U+3973.0	fTotalStrokes	 13
U+3973.0	fCangjie	PMHF
U+3973.0	fFourCorner	91090
U+3973.0	fCantonese	jyun4; zyun4; zyun6
U+3973.0	fMandarin	yuan2
U+3973.0	fEnglish	 survey; measure
U+3974.0	fUTF8	㥴
U+3974.0	fCNS11643	4-3D3C
U+3974.0	fGB	3-5256
U+3974.0	fR/S	 61.10
U+3974.0	fTotalStrokes	 13
U+3974.0	fCangjie	PHYU
U+3974.0	fFourCorner	92010
U+3974.0	fCantonese	ji4; zi4
U+3974.0	fMandarin	yi3; yi2
U+3974.0	fEnglish	[1] do not care about something; disregard [2] be ashamed
U+3975.0	fUTF8	㥵
U+3975.0	fCNS11643	3-4130
U+3975.0	fGB	5-543F
U+3975.0	fR/S	 61.10
U+3975.0	fTotalStrokes	 13
U+3975.0	fCangjie	PWMO
U+3975.0	fFourCorner	96000
U+3975.0	fHakka	 fun5; fun6
U+3975.0	fCantonese	wan6; waan4; waan6
U+3975.0	fMandarin	hun4
U+3975.0	fEnglish	same as &#x6141;
U+3976.0	fUTF8	㥶
U+3976.0	fCNS11643	3-4665
U+3976.0	fGB	5-5335
U+3976.0	fR/S	 61.10
U+3976.0	fTotalStrokes	 14
U+3976.0	fCangjie	JTCP
U+3976.0	fFourCorner	30330
U+3976.0	fCantonese	sak1
U+3976.0	fMandarin	se4
U+3976.0	fEnglish	[1] same as &#x585E; [2] same as &#x6106;
U+3977.0	fUTF8	㥷
U+3977.0	fCNS11643	4-4342
U+3977.0	fGB	3-3A69
U+3977.0	fR/S	 61.10
U+3977.0	fTotalStrokes	 14
U+3977.0	fCangjie	IKOP
U+3977.0	fFourCorner	30230
U+3977.0	fCantonese	ai3; ngai3 (also); aai3
U+3977.0	fMandarin	qie4; ye4; yi4
U+3977.0	fEnglish	same as &#x7631;
U+3978.0	fUTF8	㥸
U+3978.0	fCNS11643	4-4350
U+3978.0	fR/S	 61.10
U+3978.0	fTotalStrokes	 14
U+3978.0	fCangjie	RPWP
U+3978.0	fFourCorner	76730
U+3978.0	fCantonese	man5
U+3978.0	fMandarin	min3
U+3978.0	fEnglish	ancient form of &#x610D;
U+3979.0	fUTF8	㥹
U+3979.0	fCNS11643	3-4666
U+3979.0	fR/S	 61.10
U+3979.0	fTotalStrokes	 14
U+3979.0	fCangjie	FHP
U+3979.0	fFourCorner	98330
U+3979.0	fCantonese	fan2; baan1; faan2
U+3979.0	fMandarin	fen3
U+3979.0	fEnglish	 move
U+397A.0	fUTF8	㥺
U+397A.0	fCNS11643	3-412F
U+397A.0	fR/S	 61.10
U+397A.0	fTotalStrokes	 13
U+397A.0	fCangjie	PGIT
U+397A.0	fFourCorner	94010
U+397A.0	fCantonese	hap6
U+397A.0	fMandarin	he2; he4
U+397A.0	fEnglish	same as &#x7186;
U+397B.0	fUTF8	㥻
U+397B.0	fGB	S-212B
U+397B.0	fR/S	 61.10
U+397B.0	fTotalStrokes	 14
U+397B.0	fCangjie	IDP
U+397B.0	fCantonese	cik1
U+397C.0	fUTF8	㥼
U+397C.0	fCNS11643	4-4348
U+397C.0	fGB	5-5446
U+397C.0	fR/S	 61.11
U+397C.0	fTotalStrokes	 14
U+397C.0	fCangjie	PMYF
U+397C.0	fFourCorner	91020
U+397C.0	fCantonese	jan3; zaan2
U+397C.0	fMandarin	yin4; yin3
U+397C.0	fEnglish	[1] be near to; close to [2] recollect; recall; look back upon
U+397D.0	fUTF8	㥽
U+397D.0	fCNS11643	4-4347
U+397D.0	fGB	3-5223
U+397D.0	fR/S	 61.11
U+397D.0	fTotalStrokes	 14
U+397D.0	fCangjie	PQMC
U+397D.0	fFourCorner	95080
U+397D.0	fCantonese	caak3
U+397D.0	fMandarin	ce4; ze2
U+397D.0	fEnglish	[1] feeling; emotion [2] fact [3] love; affection [4] nature; reason [5] same as &#x8CAC;
U+397E.0	fUTF8	㥾
U+397E.0	fCNS11643	4-4349
U+397E.0	fGB	5-5439
U+397E.0	fR/S	 61.11
U+397E.0	fTotalStrokes	 14
U+397E.0	fCangjie	PSTR
U+397E.0	fFourCorner	91010
U+397E.0	fHakka	  nit8
U+397E.0	fCantonese	nik6; lik6 (also); nik1
U+397E.0	fMandarin	ni4
U+397E.0	fEnglish	[1] ashamed [2] mortified
U+397F.0	fUTF8	㥿
U+397F.0	fCNS11643	3-4B7D
U+397F.0	fR/S	 61.11
U+397F.0	fTotalStrokes	 15
U+397F.0	fCangjie	GKP
U+397F.0	fFourCorner	48334
U+397F.0	fHakka	 ngau5
U+397F.0	fCantonese	ngou6; ou6 (also); tim5
U+397F.0	fMandarin	ao4
U+397F.0	fEnglish	same as &#x6160;
U+3980.0	fUTF8	㦀
U+3980.0	fCNS11643	4-434D
U+3980.0	fGB	5-5440
U+3980.0	fR/S	 61.11
U+3980.0	fTotalStrokes	 14
U+3980.0	fCangjie	PYHJ
U+3980.0	fFourCorner	97030
U+3980.0	fHakka	 pung2
U+3980.0	fCantonese	fung4
U+3980.0	fMandarin	peng2; feng2
U+3980.0	fEnglish	joy; delight; gratification
U+3981.0	fUTF8	㦁
U+3981.0	fCNS11643	4-4965
U+3981.0	fGB	5-527A
U+3981.0	fR/S	 61.11
U+3981.0	fTotalStrokes	 15
U+3981.0	fCangjie	YJP
U+3981.0	fFourCorner	35330
U+3981.0	fCantonese	lin4
U+3981.0	fMandarin	lian2
U+3981.0	fEnglish	[1] sob; weep [2] pay attention, be careful; take care; exercise caution
U+3982.0	fUTF8	㦂
U+3982.0	fCNS11643	3-4C22
U+3982.0	fR/S	 61.11
U+3982.0	fTotalStrokes	 15
U+3982.0	fCangjie	FBRBP
U+3982.0	fFourCorner	90330
U+3982.0	fHakka	 shong2
U+3982.0	fCantonese	soeng4
U+3982.0	fMandarin	chang2; tang4
U+3982.0	fEnglish	ancient form of &#x5E38;
U+3983.0	fUTF8	㦃
U+3983.0	fCNS11643	3-4669
U+3983.0	fGB	5-5370
U+3983.0	fR/S	 61.11
U+3983.0	fTotalStrokes	 14
U+3983.0	fCangjie	PYHM
U+3983.0	fFourCorner	90010
U+3983.0	fHakka	  san3
U+3983.0	fCantonese	caan2; can2; zan3
U+3983.0	fMandarin	chan3
U+3983.0	fEnglish	complete virtue, as shown in a well-spent life
U+3984.0	fUTF8	㦄
U+3984.0	fCNS11643	3-4B7B
U+3984.0	fR/S	 61.11
U+3984.0	fTotalStrokes	 15
U+3984.0	fCangjie	IDP
U+3984.0	fFourCorner	00230
U+3984.0	fHakka	 ma2
U+3984.0	fCantonese	maa4
U+3984.0	fMandarin	ma2; mi4
U+3984.0	fEnglish	same as &#x29536;
U+3985.0	fUTF8	㦅
U+3985.0	fCNS11643	4-434A
U+3985.0	fGB	5-5426
U+3985.0	fR/S	 61.11
U+3985.0	fTotalStrokes	 14
U+3985.0	fCangjie	PKPB
U+3985.0	fFourCorner	94020
U+3985.0	fHakka	 di5; cai5; cai6
U+3985.0	fCantonese	dit6; daai3
U+3985.0	fMandarin	chai4; di4
U+3985.0	fEnglish	same as &#x6178;
U+3986.0	fUTF8	㦆
U+3986.0	fCNS11643	4-434C
U+3986.0	fGB	3-525E
U+3986.0	fR/S	 61.11
U+3986.0	fTotalStrokes	 14
U+3986.0	fCangjie	PYPD
U+3986.0	fFourCorner	91040
U+3986.0	fHakka	 niun5
U+3986.0	fCantonese	fu1
U+3986.0	fMandarin	hu1
U+3986.0	fEnglish	[1] exaggerate; exaggeration [2] arrogant; showing wild imagination; preposterously fantastic [3] without ambition; discourage
U+3987.0	fUTF8	㦇
U+3987.0	fCNS11643	4-4346
U+3987.0	fGB	3-5260
U+3987.0	fR/S	 61.11
U+3987.0	fTotalStrokes	 14
U+3987.0	fCangjie	PIXP
U+3987.0	fFourCorner	90010
U+3987.0	fCantonese	luk6
U+3987.0	fMandarin	lu4
U+3987.0	fEnglish	[1] a peaceful or easy mood; calm at mind [2] move; turn the mind [3] conversion
U+3988.0	fUTF8	㦈
U+3988.0	fGB	7-2341
U+3988.0	fR/S	 61.11
U+3988.0	fTotalStrokes	 14
U+3988.0	fCangjie	PTGT
U+3988.0	fCantonese	oi3; ngoi3 (also)
U+3989.0	fUTF8	㦉
U+3989.0	fCNS11643	3-4C27
U+3989.0	fR/S	 61.12
U+3989.0	fTotalStrokes	 15
U+3989.0	fCangjie	PGBT
U+3989.0	fFourCorner	94010
U+3989.0	fHakka	 i5
U+3989.0	fCantonese	ji3; ci2; zi3
U+3989.0	fMandarin	yi4
U+3989.0	fEnglish	ancient form of &#x39A4;
U+398A.0	fUTF8	㦊
U+398A.0	fCNS11643	3-4B78
U+398A.0	fGB	5-536C
U+398A.0	fR/S	 61.12
U+398A.0	fTotalStrokes	 15
U+398A.0	fCangjie	PTMJ
U+398A.0	fFourCorner	94054
U+398A.0	fCantonese	waa4
U+398A.0	fMandarin	hua2
U+398A.0	fEnglish	[1] lavish; luxurious [2] wasteful
U+398B.0	fUTF8	㦋
U+398B.0	fCNS11643	4-434B
U+398B.0	fGB	3-525D
U+398B.0	fR/S	 61.12; 61.11
U+398B.0	fTotalStrokes	 14; 15
U+398B.0	fCangjie	PKJA
U+398B.0	fFourCorner	94060
U+398B.0	fCantonese	zaa1; caa1
U+398B.0	fMandarin	zha1
U+398B.0	fEnglish	absurd; wild; nonsensical; showing wild imagination; preposterously fantastic
U+398C.0	fUTF8	㦌
U+398C.0	fCNS11643	4-5038
U+398C.0	fGB	3-5129
U+398C.0	fR/S	 61.12
U+398C.0	fTotalStrokes	 16
U+398C.0	fCangjie	HUHUP
U+398C.0	fFourCorner	20331
U+398C.0	fCantonese	fat1
U+398C.0	fMandarin	cuan3; tui4; xu4
U+398C.0	fEnglish	[1] keen or sharp; clever; simple and honest [2] sound sleep or slumber; a deep sleep [3] wake up from sleep
U+398D.0	fUTF8	㦍
U+398D.0	fCNS11643	3-4C2A
U+398D.0	fGB	5-5455
U+398D.0	fR/S	 61.12
U+398D.0	fTotalStrokes	 15
U+398D.0	fCangjie	PRRU
U+398D.0	fFourCorner	96061
U+398D.0	fHakka	  ngok8
U+398D.0	fCantonese	ngok6; ok6 (also)
U+398D.0	fMandarin	e4; wu4
U+398D.0	fEnglish	standard form of &#x6115;
U+398E.0	fUTF8	㦎
U+398E.0	fCNS11643	4-4967
U+398E.0	fGB	3-5227
U+398E.0	fR/S	 61.12
U+398E.0	fTotalStrokes	 15
U+398E.0	fCangjie	PLGM
U+398E.0	fFourCorner	95010
U+398E.0	fCantonese	waak6
U+398E.0	fMandarin	huo4; hua4
U+398E.0	fEnglish	[1] cantankerous; perverse [2] stupid; dull [3] ignorant [4] divide clearly; without ambiguity [5] gracefully quiet
U+398F.0	fUTF8	㦏
U+398F.0	fCNS11643	4-4969
U+398F.0	fGB	3-526E
U+398F.0	fR/S	 61.12
U+398F.0	fTotalStrokes	 15
U+398F.0	fCangjie	PRUC
U+398F.0	fFourCorner	97080
U+398F.0	fCantonese	seon2
U+398F.0	fMandarin	sun3; xuan4
U+398F.0	fEnglish	[1] weak; feeble [2] joy; delight; gratification
U+3990.0	fUTF8	㦐
U+3990.0	fCNS11643	4-4968
U+3990.0	fGB	3-523F
U+3990.0	fR/S	 61.12
U+3990.0	fTotalStrokes	 15
U+3990.0	fCangjie	PIPC
U+3990.0	fFourCorner	93040
U+3990.0	fCantonese	nei6; lei6 (also)
U+3990.0	fMandarin	ni4
U+3990.0	fEnglish	[1] a straightforward character [2] cannot make up one's mind; hesitate
U+3991.0	fUTF8	㦑
U+3991.0	fCNS11643	4-496B
U+3991.0	fGB	3-5262
U+3991.0	fR/S	 61.12
U+3991.0	fTotalStrokes	 15
U+3991.0	fCangjie	PMJK
U+3991.0	fFourCorner	98040
U+3991.0	fCantonese	haam3; haam1
U+3991.0	fMandarin	xian4; lian3
U+3991.0	fEnglish	[1] anger; rage; angry; furious [2] silly; stupid; foolish
U+3992.0	fUTF8	㦒
U+3992.0	fCNS11643	7-227C
U+3992.0	fGB	3-5269
U+3992.0	fR/S	 61.12
U+3992.0	fTotalStrokes	 15
U+3992.0	fCangjie	PWLR
U+3992.0	fFourCorner	96060
U+3992.0	fCantonese	lai2; laai4
U+3992.0	fMandarin	li1; li2
U+3992.0	fEnglish	[1] deceive [2] loquacious
U+3993.0	fUTF8	㦓
U+3993.0	fCNS11643	4-496D
U+3993.0	fGB	3-526B
U+3993.0	fR/S	 61.12
U+3993.0	fTotalStrokes	 15
U+3993.0	fCangjie	PBKF
U+3993.0	fFourCorner	93030
U+3993.0	fCantonese	jin6; zin6
U+3993.0	fMandarin	xian4
U+3993.0	fEnglish	[1] difficult intention [2] difficult idea or sentiment [3] weakened volition; dejected [4] fear; scare; fright
U+3994.0	fUTF8	㦔
U+3994.0	fCNS11643	3-5069
U+3994.0	fR/S	 61.12
U+3994.0	fTotalStrokes	 16
U+3994.0	fCangjie	AKP
U+3994.0	fFourCorner	63330
U+3994.0	fHakka	 jam3
U+3994.0	fCantonese	jim1; zim1
U+3994.0	fMandarin	yan1; yan4
U+3994.0	fEnglish	same as &#x61D5;
U+3995.0	fUTF8	㦕
U+3995.0	fCNS11643	4-496C
U+3995.0	fGB	3-5263
U+3995.0	fR/S	 61.12
U+3995.0	fTotalStrokes	 15
U+3995.0	fCangjie	PNLM
U+3995.0	fFourCorner	97010
U+3995.0	fCantonese	lung4
U+3995.0	fMandarin	long2
U+3995.0	fEnglish	[1] thought; idea; sentiment [2] intention, expectation [3] meaning
U+3996.0	fUTF8	㦖
U+3996.0	fCNS11643	3-4C26
U+3996.0	fGB	5-5378
U+3996.0	fR/S	 61.12
U+3996.0	fTotalStrokes	 15
U+3996.0	fCangjie	PANP
U+3996.0	fFourCorner	97020
U+3996.0	fCantonese	mun6
U+3996.0	fMandarin	men4; man2
U+3996.0	fEnglish	same as &#x6172;
U+3997.0	fUTF8	㦗
U+3997.0	fCNS11643	4-5039
U+3997.0	fGB	3-5272
U+3997.0	fR/S	 61.13
U+3997.0	fTotalStrokes	 16
U+3997.0	fCangjie	PDDF
U+3997.0	fFourCorner	94090
U+3997.0	fHakka	 gim1; gim5
U+3997.0	fCantonese	gam1; gaam1; gaam3; kaam4; naap1
U+3997.0	fMandarin	jin1; jian4; jin4
U+3997.0	fEnglish	determined; resolute
U+3998.0	fUTF8	㦘
U+3998.0	fCNS11643	5-5B4C
U+3998.0	fR/S	 61.13
U+3998.0	fTotalStrokes	 17
U+3998.0	fCangjie	HKP
U+3998.0	fFourCorner	28330
U+3998.0	fCantonese	gik1
U+3998.0	fEnglish	same as &#x61BF;
U+3999.0	fUTF8	㦙
U+3999.0	fGB	S-2127
U+3999.0	fR/S	 61.13
U+3999.0	fTotalStrokes	 16
U+3999.0	fCangjie	PWBP
U+3999.0	fCantonese	jyu4
U+399A.0	fUTF8	㦚
U+399A.0	fCNS11643	4-5B23
U+399A.0	fGB	5-7335
U+399A.0	fR/S	 61.14
U+399A.0	fTotalStrokes	 18
U+399A.0	fCangjie	YJPYJ
U+399A.0	fFourCorner	00440
U+399A.0	fCantonese	bin2
U+399A.0	fMandarin	bian3
U+399A.0	fEnglish	[1] melancholy; grievous; mournful [2] urgent; pressing
U+399B.0	fUTF8	㦛
U+399B.0	fCNS11643	4-5B29
U+399B.0	fR/S	 61.14
U+399B.0	fTotalStrokes	 18
U+399B.0	fCangjie	HCP
U+399B.0	fFourCorner	77330
U+399B.0	fHakka	 ri2; ji2; ji3
U+399B.0	fCantonese	jyu4; zyu5
U+399B.0	fMandarin	yu3; yu2
U+399B.0	fEnglish	 walk fast
U+399C.0	fUTF8	㦜
U+399C.0	fCNS11643	4-5636
U+399C.0	fGB	3-5271
U+399C.0	fR/S	 61.14
U+399C.0	fTotalStrokes	 16; 17
U+399C.0	fCangjie	PTOE
U+399C.0	fFourCorner	94040
U+399C.0	fHakka	 kiok7
U+399C.0	fCantonese	wok6; waa6
U+399C.0	fMandarin	huo4; xue4
U+399C.0	fEnglish	[1] move the mind; agitated; nervous; start thinking [2] melancholoy; grievous; mournful [3] same as &#x9A5A;
U+399D.0	fUTF8	㦝
U+399D.0	fCNS11643	4-5B25
U+399D.0	fGB	3-512C
U+399D.0	fR/S	 61.14
U+399D.0	fTotalStrokes	 18
U+399D.0	fCangjie	BUP
U+399D.0	fFourCorner	26330
U+399D.0	fCantonese	mok6; bok6; miu5
U+399D.0	fMandarin	mao4; miao3
U+399D.0	fEnglish	same as &#x22927;
U+399E.0	fUTF8	㦞
U+399E.0	fCNS11643	3-5767
U+399E.0	fR/S	 61.14
U+399E.0	fTotalStrokes	 18
U+399E.0	fCangjie	GNMP
U+399E.0	fFourCorner	40330
U+399E.0	fHakka	 dau3
U+399E.0	fCantonese	cau4
U+399E.0	fMandarin	chou2; dao3; zhou4
U+399E.0	fEnglish	same as &#x61E4;
U+399F.0	fUTF8	㦟
U+399F.0	fCNS11643	4-5B24
U+399F.0	fGB	3-512B
U+399F.0	fR/S	 61.14
U+399F.0	fTotalStrokes	 18
U+399F.0	fCangjie	BGP
U+399F.0	fFourCorner	26330
U+399F.0	fCantonese	maai4; fat3; mai4
U+399F.0	fMandarin	mai2; hai4
U+399F.0	fEnglish	intelligent; astute; clever
U+39A0.0	fUTF8	㦠
U+39A0.0	fCNS11643	F-5A29
U+39A0.0	fGB	5-5364
U+39A0.0	fR/S	 61.14
U+39A0.0	fTotalStrokes	 17
U+39A0.0	fCangjie	PTGI
U+39A0.0	fFourCorner	9400
U+39A0.0	fCantonese	deoi6
U+39A1.0	fUTF8	㦡
U+39A1.0	fCNS11643	3-5766
U+39A1.0	fR/S	 61.15
U+39A1.0	fTotalStrokes	 18
U+39A1.0	fCangjie	PVID
U+39A1.0	fFourCorner	92090
U+39A1.0	fHakka	 ngok8; lok8; ngau5
U+39A1.0	fCantonese	lok6; lok3; ngok6; ngau6
U+39A1.0	fMandarin	luo4; yue4; le4; liao4; lo4
U+39A1.0	fEnglish	non-classical form of &#x6A02;
U+39A2.0	fUTF8	㦢
U+39A2.0	fCNS11643	4-503A
U+39A2.0	fGB	5-5356
U+39A2.0	fR/S	 61.13; 61.15
U+39A2.0	fTotalStrokes	 16; 18
U+39A2.0	fCangjie	PHAL
U+39A2.0	fFourCorner	98020
U+39A2.0	fCantonese	zit3
U+39A2.0	fMandarin	jie2; qi4
U+39A2.0	fEnglish	[1] pure; virtuous; devotion; dedication [2] have bearings; have manners
U+39A3.0	fUTF8	㦣
U+39A3.0	fCNS11643	5-6778
U+39A3.0	fR/S	 61.16; 61.15
U+39A3.0	fTotalStrokes	 19; 20
U+39A3.0	fCangjie	HNP
U+39A3.0	fFourCorner	21330
U+39A3.0	fHakka	 wi5; wui5; wui6; vui5
U+39A3.0	fCantonese	wai6; at3; waai6; wun2
U+39A3.0	fMandarin	wei4
U+39A3.0	fEnglish	 talk in one's sleep
U+39A4.0	fUTF8	㦤
U+39A4.0	fCNS11643	4-6343
U+39A4.0	fR/S	 61.16
U+39A4.0	fTotalStrokes	 20
U+39A4.0	fCangjie	GTNOP
U+39A4.0	fFourCorner	47130
U+39A4.0	fHakka	 ji5
U+39A4.0	fCantonese	ji3; zi3
U+39A4.0	fMandarin	yi4; yi2
U+39A4.0	fEnglish	same as &#x61FF;
U+39A5.0	fUTF8	㦥
U+39A5.0	fCNS11643	4-5F64
U+39A5.0	fGB	3-527D
U+39A5.0	fR/S	 61.16
U+39A5.0	fTotalStrokes	 19
U+39A5.0	fCangjie	PJQP
U+39A5.0	fFourCorner	93030
U+39A5.0	fHakka	 sien1
U+39A5.0	fCantonese	hyun1
U+39A5.0	fMandarin	xuan1; huan2; xian3
U+39A5.0	fEnglish	same as &#x610B;
U+39A6.0	fUTF8	㦦
U+39A6.0	fCNS11643	5-6773
U+39A6.0	fGB	3-5261
U+39A6.0	fR/S	 61.16
U+39A6.0	fTotalStrokes	 19
U+39A6.0	fCangjie	PANU
U+39A6.0	fFourCorner	97020
U+39A6.0	fCantonese	hik1; hak1
U+39A6.0	fMandarin	he4
U+39A6.0	fEnglish	[1] apprehensive; fearful; afraid, ashamed [2] uneasy in mind; not feeling at peace; disturbed
U+39A7.0	fUTF8	㦧
U+39A7.0	fCNS11643	7-472C
U+39A7.0	fR/S	 61.16
U+39A7.0	fTotalStrokes	 19
U+39A7.0	fCangjie	PHUA
U+39A7.0	fFourCorner	94060
U+39A7.0	fHakka	  cam3
U+39A7.0	fCantonese	caam2
U+39A7.0	fMandarin	can3; qian3
U+39A7.0	fEnglish	same as &#x61AF;
U+39A8.0	fUTF8	㦨
U+39A8.0	fCNS11643	3-5E30
U+39A8.0	fR/S	 61.17
U+39A8.0	fTotalStrokes	 21; 20
U+39A8.0	fCangjie	PANW
U+39A8.0	fFourCorner	97020
U+39A8.0	fCantonese	laan5
U+39A8.0	fMandarin	lan3; lan2; lan4
U+39A8.0	fEnglish	same as &#x5B3E;
U+39A9.0	fUTF8	㦩
U+39A9.0	fCNS11643	4-6342
U+39A9.0	fGB	5-5451
U+39A9.0	fR/S	 61.17
U+39A9.0	fTotalStrokes	 20
U+39A9.0	fCangjie	PNLP
U+39A9.0	fFourCorner	92030
U+39A9.0	fHakka	 jun3
U+39A9.0	fCantonese	jan2; zaan2; zeon6
U+39A9.0	fMandarin	yin3
U+39A9.0	fEnglish	[1] melancholy; grievous; mournful; feel sorrow or grief; grieve; mourn; be sad [2] prudent; cautious
U+39AA.0	fUTF8	㦪
U+39AA.0	fCNS11643	4-6340
U+39AA.0	fGB	3-527E
U+39AA.0	fR/S	 61.17
U+39AA.0	fTotalStrokes	 20
U+39AA.0	fCangjie	PFFE
U+39AA.0	fFourCorner	99040
U+39AA.0	fHakka	 siet7
U+39AA.0	fCantonese	sip3
U+39AA.0	fMandarin	xie4
U+39AA.0	fEnglish	complaint; grudge; unjust
U+39AB.0	fUTF8	㦫
U+39AB.0	fCNS11643	3-5F47
U+39AB.0	fGB	5-545B
U+39AB.0	fR/S	 61.19
U+39AB.0	fTotalStrokes	 22
U+39AB.0	fCangjie	PHUC
U+39AB.0	fFourCorner	94080
U+39AB.0	fCantonese	zaat3
U+39AB.0	fMandarin	zan3
U+39AB.0	fEnglish	lax and crude; idle and remiss
U+39AC.0	fUTF8	㦬
U+39AC.0	fCNS11643	4-686C
U+39AC.0	fGB	3-522F
U+39AC.0	fR/S	 61.19
U+39AC.0	fTotalStrokes	 21; 22
U+39AC.0	fCangjie	PWLG
U+39AC.0	fFourCorner	96010
U+39AC.0	fCantonese	lo5; lo2
U+39AC.0	fMandarin	luo3
U+39AC.0	fEnglish	[1] ashamed; abashed; mortified; humiliated [2] few; little; scarce; rare
U+39AD.0	fUTF8	㦭
U+39AD.0	fCNS11643	3-6178
U+39AD.0	fR/S	 61.24
U+39AD.0	fTotalStrokes	 27
U+39AD.0	fCangjie	PMBO
U+39AD.0	fFourCorner	91010
U+39AD.0	fCantonese	lin4; ling4
U+39AD.0	fMandarin	ling2
U+39AD.0	fEnglish	same as &#x6190;, &#x601C;
U+39AE.0	fUTF8	㦮
U+39AE.0	fCNS11643	3-236A
U+39AE.0	fR/S	 62.3
U+39AE.0	fTotalStrokes	 6; 7
U+39AE.0	fCangjie	IQ
U+39AE.0	fFourCorner	53000
U+39AE.0	fHakka	 cien2
U+39AE.0	fCantonese	zin1; cin4; cin2
U+39AE.0	fMandarin	qian2; jian3; zi1
U+39AE.0	fEnglish	simplified form of &#x9322;
U+39AF.0	fUTF8	㦯
U+39AF.0	fCNS11643	3-2626
U+39AF.0	fR/S	 62.3
U+39AF.0	fTotalStrokes	 7
U+39AF.0	fCangjie	IIM
U+39AF.0	fFourCorner	53100
U+39AF.0	fHakka	 fet8
U+39AF.0	fCantonese	waak6
U+39AF.0	fMandarin	huo4; yu4
U+39AF.0	fEnglish	same as &#x6216;
U+39B0.0	fUTF8	㦰
U+39B0.0	fCNS11643	4-2654
U+39B0.0	fGB	5-4825
U+39B0.0	fR/S	 62.4
U+39B0.0	fTotalStrokes	 8
U+39B0.0	fCangjie	OOI
U+39B0.0	fFourCorner	83050
U+39B0.0	fCantonese	zim1; cim1
U+39B0.0	fMandarin	jian1
U+39B0.0	fEnglish	[1] break off (relations); sever [2] exterminate; annihilate; wipe out [3] pierce; stab; [4] irritate; hurt [5] hold weapons [6] agricultural implements; farm tools [7] sharp will; eager intention; determination
U+39B1.0	fUTF8	㦱
U+39B1.0	fR/S	 62.4
U+39B1.0	fTotalStrokes	 8
U+39B1.0	fCangjie	MIG
U+39B1.0	fFourCorner	1315
U+39B1.0	fCantonese	ngo5
U+39B1.0	fMandarin	wo3
U+39B2.0	fUTF8	㦲
U+39B2.0	fCNS11643	6-2B52
U+39B2.0	fR/S	 62.4
U+39B2.0	fTotalStrokes	 8
U+39B2.0	fCangjie	JIHS
U+39B2.0	fFourCorner	43250
U+39B2.0	fCantonese	zoi1
U+39B2.0	fEnglish	non-classical form of &#x54C9;
U+39B3.0	fUTF8	㦳
U+39B3.0	fR/S	 62.5
U+39B3.0	fTotalStrokes	 9
U+39B3.0	fCangjie	JIG
U+39B3.0	fCantonese	ngo5
U+39B4.0	fUTF8	㦴
U+39B4.0	fCNS11643	4-2D2F
U+39B4.0	fGB	5-482A
U+39B4.0	fR/S	 62.6
U+39B4.0	fTotalStrokes	 10
U+39B4.0	fCangjie	IHER
U+39B4.0	fFourCorner	53600
U+39B4.0	fCantonese	gaak3
U+39B4.0	fMandarin	ge2; qia4
U+39B4.0	fEnglish	[1] arrest; catch; seize [2] brawl; a hand-to-hand fight; struggle for; fight for [3] beat; strike; attack
U+39B5.0	fUTF8	㦵
U+39B5.0	fCNS11643	4-2D30
U+39B5.0	fGB	3-4631
U+39B5.0	fR/S	 62.6
U+39B5.0	fTotalStrokes	 10
U+39B5.0	fCangjie	HDI
U+39B5.0	fFourCorner	23950
U+39B5.0	fCantonese	zyu1
U+39B5.0	fMandarin	zhu1
U+39B5.0	fEnglish	[1] kill; put to death; slaughter; execute [2] punish [3] weed out; exterminate [4] a kind of weapon
U+39B6.0	fUTF8	㦶
U+39B6.0	fCNS11643	5-2D5D
U+39B6.0	fR/S	 62.6
U+39B6.0	fTotalStrokes	 10
U+39B6.0	fCangjie	IMIG
U+39B6.0	fFourCorner	53100
U+39B6.0	fCantonese	dit6; zung5
U+39B6.0	fMandarin	yong3; die2
U+39B6.0	fEnglish	[1] repeatedly; frequently [2] alternate [3] common; ordinary; normal [4] sharp [5] the name of a state in ancient China [6] bold; brave; fierce; violent; severe; strict; stringent
U+39B7.0	fUTF8	㦷
U+39B7.0	fCNS11643	3-357A
U+39B7.0	fR/S	 62.7
U+39B7.0	fTotalStrokes	 11
U+39B7.0	fCangjie	NBI
U+39B7.0	fFourCorner	13250
U+39B7.0	fCantonese	jung5; zung5
U+39B7.0	fMandarin	yong3
U+39B7.0	fEnglish	same as &#x229A8;
U+39B8.0	fUTF8	㦸
U+39B8.0	fCNS11643	3-3B65
U+39B8.0	fGB	5-4831
U+39B8.0	fR/S	 62.8
U+39B8.0	fTotalStrokes	 12
U+39B8.0	fCangjie	YJI
U+39B8.0	fFourCorner	13450
U+39B8.0	fHakka	 git7
U+39B8.0	fCantonese	gik1
U+39B8.0	fMandarin	ji3
U+39B8.0	fEnglish	a lance with two points; a halberd with a crescent
U+39B9.0	fUTF8	㦹
U+39B9.0	fCNS11643	4-3D42
U+39B9.0	fGB	5-4824
U+39B9.0	fR/S	 62.9
U+39B9.0	fTotalStrokes	 13
U+39B9.0	fCangjie	AHI
U+39B9.0	fFourCorner	63250
U+39B9.0	fCantonese	joeng4; maai5; zoeng4; zoeng5
U+39B9.0	fMandarin	yang2; ang4; ying3
U+39B9.0	fEnglish	[1] spear; lance; javelin [2] battle-axe; halberd
U+39BA.0	fUTF8	㦺
U+39BA.0	fCNS11643	4-4354
U+39BA.0	fGB	3-4636
U+39BA.0	fR/S	 62.10
U+39BA.0	fTotalStrokes	 14
U+39BA.0	fCangjie	MII
U+39BA.0	fFourCorner	73350
U+39BA.0	fCantonese	juk6; zuk6
U+39BA.0	fMandarin	ru4; ru3
U+39BA.0	fEnglish	[1] lance with two points [2] halberd with a crescent-shaped blade [3] stimulate; provoke; excite; irritate [4] point with the index finger and the middle finger [5] describe angry or an awe-inspir
U+39BB.0	fUTF8	㦻
U+39BB.0	fCNS11643	5-4D37
U+39BB.0	fR/S	 62.11
U+39BB.0	fTotalStrokes	 15
U+39BB.0	fCangjie	SAI
U+39BB.0	fFourCorner	13650
U+39BB.0	fHakka	 sip8
U+39BB.0	fCantonese	zaap6
U+39BB.0	fMandarin	xi2; zhe2
U+39BB.0	fEnglish	ancient form of &#x8972;
U+39BC.0	fUTF8	㦼
U+39BC.0	fCNS11643	3-4C2B
U+39BC.0	fGB	5-4839
U+39BC.0	fR/S	 62.11
U+39BC.0	fTotalStrokes	 15
U+39BC.0	fCangjie	QXI
U+39BC.0	fFourCorner	53750
U+39BC.0	fCantonese	soeng3
U+39BC.0	fMandarin	shuang4
U+39BC.0	fEnglish	a wooden piles used as a cable to make immovable of a boat
U+39BD.0	fUTF8	㦽
U+39BD.0	fCNS11643	3-5462
U+39BD.0	fR/S	 62.13
U+39BD.0	fTotalStrokes	 17
U+39BD.0	fCangjie	KBIRV
U+39BD.0	fFourCorner	43250
U+39BD.0	fCantonese	juk1; guk1; zuk1
U+39BD.0	fMandarin	yu4; xu4
U+39BD.0	fEnglish	[1] with elegant appearance [2] with beautiful or gorgeous color; refined [3] learned and accomplished
U+39BE.0	fUTF8	㦾
U+39BE.0	fCNS11643	6-276E
U+39BE.0	fR/S	 63.3
U+39BE.0	fTotalStrokes	 7
U+39BE.0	fCangjie	HSPD
U+39BE.0	fFourCorner	30210
U+39BE.0	fHakka	 ji2
U+39BE.0	fCantonese	ji4; zi4
U+39BE.0	fMandarin	yi2; yin3
U+39BE.0	fEnglish	same as &#x6245;
U+39BF.0	fUTF8	㦿
U+39BF.0	fCNS11643	4-2655
U+39BF.0	fGB	3-506C
U+39BF.0	fR/S	 63.4
U+39BF.0	fTotalStrokes	 8
U+39BF.0	fCangjie	ISD
U+39BF.0	fFourCorner	30290
U+39BF.0	fHakka	 ham3; fu5
U+39BF.0	fCantonese	haam2; haang6; him1; wu6
U+39BF.0	fMandarin	hu4
U+39BF.0	fEnglish	the small door inside a house
U+39C0.0	fUTF8	㧀
U+39C0.0	fCNS11643	4-2656
U+39C0.0	fGB	3-506A
U+39C0.0	fR/S	 63.4
U+39C0.0	fTotalStrokes	 8
U+39C0.0	fCangjie	ISNHE
U+39C0.0	fFourCorner	30240
U+39C0.0	fCantonese	gap6
U+39C0.0	fMandarin	ji2
U+39C0.0	fEnglish	 bolt of a door; door latch
U+39C1.0	fUTF8	㧁
U+39C1.0	fCNS11643	4-294D
U+39C1.0	fGB	3-506D
U+39C1.0	fR/S	 63.5
U+39C1.0	fTotalStrokes	 9
U+39C1.0	fCangjie	ISGI
U+39C1.0	fFourCorner	30230
U+39C1.0	fHakka	 kap7;  hap7; kap8; hap8
U+39C1.0	fCantonese	heoi3
U+39C1.0	fMandarin	qu4
U+39C1.0	fEnglish	[1] close [2] shut the door with a bang [3] soar [4] ancient form of &#x95D4;
U+39C2.0	fUTF8	㧂
U+39C2.0	fCNS11643	3-2C64
U+39C2.0	fR/S	 63.5
U+39C2.0	fTotalStrokes	 9
U+39C2.0	fCangjie	HSW
U+39C2.0	fFourCorner	30260
U+39C2.0	fCantonese	tin4
U+39C2.0	fMandarin	tian2; tian1
U+39C2.0	fEnglish	same as &#x5C47;
U+39C3.0	fUTF8	㧃
U+39C3.0	fCNS11643	4-217B
U+39C3.0	fGB	5-4B52
U+39C3.0	fR/S	 64.2
U+39C3.0	fTotalStrokes	 5
U+39C3.0	fCangjie	QVL
U+39C3.0	fFourCorner	52000
U+39C3.0	fHakka	 shiu1; su1
U+39C3.0	fCantonese	sau1
U+39C3.0	fMandarin	jiu1
U+39C3.0	fEnglish	[1] receive; collect; gather in [2] close
U+39C4.0	fUTF8	㧄
U+39C4.0	fCNS11643	4-217A
U+39C4.0	fGB	5-4B51
U+39C4.0	fR/S	 64.2
U+39C4.0	fTotalStrokes	 5
U+39C4.0	fCangjie	QU
U+39C4.0	fFourCorner	52070
U+39C4.0	fHakka	 kam3
U+39C4.0	fCantonese	ham2; kaau4
U+39C4.0	fMandarin	kan3; qian3; qiu2
U+39C4.0	fEnglish	the barking of a little dog to eat and not be satisfied
U+39C5.0	fUTF8	㧅
U+39C5.0	fCNS11643	3-224F
U+39C5.0	fR/S	 64.2
U+39C5.0	fTotalStrokes	 5
U+39C5.0	fCangjie	QSH
U+39C5.0	fFourCorner	57020
U+39C5.0	fCantonese	muk6
U+39C5.0	fMandarin	le4; mu4
U+39C5.0	fEnglish	[1] turn upside down; exert a great deal; pull and drag [2] implicate; involve
U+39C6.0	fUTF8	㧆
U+39C6.0	fCNS11643	4-2274
U+39C6.0	fGB	3-4958
U+39C6.0	fR/S	 64.3
U+39C6.0	fTotalStrokes	 6
U+39C6.0	fCangjie	QLB
U+39C6.0	fFourCorner	54020
U+39C6.0	fCantonese	gan3; gaan1
U+39C6.0	fMandarin	jin4
U+39C6.0	fEnglish	[1] cover something with a piece of cloth [2] a kind of cloth used to cover something
U+39C7.0	fUTF8	㧇
U+39C7.0	fCNS11643	4-2273
U+39C7.0	fR/S	 64.3
U+39C7.0	fTotalStrokes	 6
U+39C7.0	fCangjie	QBM
U+39C7.0	fFourCorner	57020
U+39C7.0	fCantonese	mou6; mou5
U+39C7.0	fMandarin	mao3
U+39C7.0	fEnglish	[1] hold; grasp [2] maintain; support; keep; uphold [3] tie; stalemate
U+39C8.0	fUTF8	㧈
U+39C8.0	fCNS11643	4-2275
U+39C8.0	fGB	5-4B57
U+39C8.0	fR/S	 64.3
U+39C8.0	fTotalStrokes	 6
U+39C8.0	fCangjie	QN
U+39C8.0	fFourCorner	57020
U+39C8.0	fHakka	 jin3
U+39C8.0	fCantonese	jan5; zaan5
U+39C8.0	fMandarin	yin3; yin4
U+39C8.0	fEnglish	ancient form of &#x5F15;
U+39C9.0	fUTF8	㧉
U+39C9.0	fCNS11643	4-243A
U+39C9.0	fGB	3-4966
U+39C9.0	fR/S	 64.4
U+39C9.0	fTotalStrokes	 7
U+39C9.0	fCangjie	QOMN
U+39C9.0	fFourCorner	58010
U+39C9.0	fCantonese	goi3; koi3
U+39C9.0	fMandarin	gai4; kui4
U+39C9.0	fEnglish	[1] rub; grind; polish; wear [2] take; obtain; select [3] beat; strike; attack
U+39CA.0	fUTF8	㧊
U+39CA.0	fCNS11643	4-2436
U+39CA.0	fGB	3-4961
U+39CA.0	fR/S	 64.4
U+39CA.0	fTotalStrokes	 7
U+39CA.0	fCangjie	QJB
U+39CA.0	fFourCorner	55027
U+39CA.0	fCantonese	put3
U+39CA.0	fMandarin	po1; ba2
U+39CA.0	fEnglish	[1] push; excpel [2] shirk; decline [3] row [4] wipe and clean [5] strike; beat [6] be rampant in defiance of authority
U+39CB.0	fUTF8	㧋
U+39CB.0	fCNS11643	3-262C
U+39CB.0	fR/S	 64.4
U+39CB.0	fTotalStrokes	 7
U+39CB.0	fCangjie	QIK
U+39CB.0	fFourCorner	53030
U+39CB.0	fCantonese	hyun2
U+39CB.0	fMandarin	xuan1; xian3; xuan3
U+39CB.0	fEnglish	lengthen and extenuate,as in making noodle; drag out
U+39CC.0	fUTF8	㧌
U+39CC.0	fCNS11643	3-262D
U+39CC.0	fGB	5-4B5E
U+39CC.0	fR/S	 64.4
U+39CC.0	fTotalStrokes	 7
U+39CC.0	fCangjie	QHQU
U+39CC.0	fFourCorner	52014
U+39CC.0	fCantonese	mou6
U+39CC.0	fMandarin	mao4
U+39CC.0	fEnglish	same as &#x6BDB;, &#x82BC;
U+39CD.0	fUTF8	㧍
U+39CD.0	fCNS11643	3-2630
U+39CD.0	fGB	5-4B65
U+39CD.0	fR/S	 64.4
U+39CD.0	fTotalStrokes	 7
U+39CD.0	fCangjie	QYHS
U+39CD.0	fFourCorner	50020
U+39CD.0	fCantonese	fong2
U+39CD.0	fMandarin	fang3
U+39CD.0	fEnglish	[1] same as &#x4EFF; [2] same as &#x6412;
U+39CE.0	fUTF8	㧎
U+39CE.0	fCNS11643	3-262B
U+39CE.0	fGB	5-4B59
U+39CE.0	fR/S	 64.4
U+39CE.0	fTotalStrokes	 7
U+39CE.0	fCangjie	QMVH
U+39CE.0	fFourCorner	51040
U+39CE.0	fCantonese	ngaa4; aa4 (also)
U+39CE.0	fMandarin	ya2
U+39CE.0	fEnglish	[1] forked branches [2] stone roller [3] roll [4] stop; detain; prohibit [5] clutch; grasp
U+39CF.0	fUTF8	㧏
U+39CF.0	fGB	7-2243
U+39CF.0	fR/S	 64.4
U+39CF.0	fTotalStrokes	 7
U+39CF.0	fCangjie	QBK
U+39CF.0	fCantonese	gong1
U+39CF.0	fEnglish	simplified form of &#x6386;
U+39D0.0	fUTF8	㧐
U+39D0.0	fGB	7-224B
U+39D0.0	fR/S	 64.4
U+39D0.0	fTotalStrokes	 7
U+39D0.0	fCangjie	QEE
U+39D0.0	fCantonese	sung2
U+39D0.0	fMandarin	song3
U+39D1.0	fUTF8	㧑
U+39D1.0	fGB	7-2345
U+39D1.0	fR/S	 64.4
U+39D1.0	fTotalStrokes	 7
U+39D1.0	fCangjie	QIKI
U+39D1.0	fCantonese	fai1; faai1
U+39D1.0	fMandarin	wei2; wei3
U+39D1.0	fEnglish	simplified form of &#x649D;
U+39D2.0	fUTF8	㧒
U+39D2.0	fCNS11643	4-2658
U+39D2.0	fGB	3-4A22
U+39D2.0	fR/S	 64.5
U+39D2.0	fTotalStrokes	 8
U+39D2.0	fCangjie	QJC
U+39D2.0	fFourCorner	53080
U+39D2.0	fHakka	 fit7
U+39D2.0	fCantonese	wat6; leot6; waat6
U+39D2.0	fMandarin	xue4; xue2; yu4; yue4
U+39D2.0	fEnglish	[1] beat; strike [2] throw; pitch [3] scoop out; dig out
U+39D3.0	fUTF8	㧓
U+39D3.0	fCNS11643	3-292D
U+39D3.0	fGB	5-4B6F
U+39D3.0	fR/S	 64.5
U+39D3.0	fTotalStrokes	 8
U+39D3.0	fCangjie	QHVO
U+39D3.0	fFourCorner	52030
U+39D3.0	fHakka	  gwa1
U+39D3.0	fCantonese	gwaa1
U+39D3.0	fMandarin	gua1
U+39D3.0	fEnglish	 grab; seize
U+39D4.0	fUTF8	㧔
U+39D4.0	fCNS11643	5-263A
U+39D4.0	fGB	3-4977
U+39D4.0	fR/S	 64.5
U+39D4.0	fTotalStrokes	 8
U+39D4.0	fCangjie	QIH
U+39D4.0	fFourCorner	53050
U+39D4.0	fCantonese	gwaai3; gwai3; tim4
U+39D4.0	fMandarin	guai4
U+39D4.0	fEnglish	[1] disturbe; agitate; harass [2] quarrel; wrangle
U+39D5.0	fUTF8	㧕
U+39D5.0	fCNS11643	3-292C
U+39D5.0	fR/S	 64.5
U+39D5.0	fTotalStrokes	 8
U+39D5.0	fCangjie	QHHL
U+39D5.0	fFourCorner	57020
U+39D5.0	fHakka	  liu3
U+39D5.0	fCantonese	lau5
U+39D5.0	fMandarin	liu3; jiu4; yu2
U+39D5.0	fEnglish	 cover; feel; touch
U+39D6.0	fUTF8	㧖
U+39D6.0	fCNS11643	3-2938
U+39D6.0	fGB	5-4B72
U+39D6.0	fR/S	 64.5
U+39D6.0	fTotalStrokes	 8
U+39D6.0	fCangjie	QISN
U+39D6.0	fFourCorner	53017
U+39D6.0	fHakka	 ak7; jit7; rit7
U+39D6.0	fCantonese	ak1; aak1 (also); ngaak1 (also); ngak1 (also); zim1
U+39D6.0	fMandarin	e4
U+39D6.0	fEnglish	 clutch; grasp
U+39D7.0	fUTF8	㧗
U+39D7.0	fCNS11643	4-2955
U+39D7.0	fGB	3-4A2B
U+39D7.0	fR/S	 64.5; 64.6
U+39D7.0	fTotalStrokes	 9; 8
U+39D7.0	fCangjie	QYMP
U+39D7.0	fFourCorner	51010
U+39D7.0	fHakka	 ze3
U+39D7.0	fCantonese	zi2
U+39D7.0	fMandarin	zi3
U+39D7.0	fEnglish	[1] clatch [2] grasp with the hand [3] pinch [4] kill [5] attach from the sideway [6] strike with the hand
U+39D8.0	fUTF8	㧘
U+39D8.0	fCNS11643	4-2D33
U+39D8.0	fGB	3-494C
U+39D8.0	fR/S	 64.5; 64.6
U+39D8.0	fTotalStrokes	 10; 9
U+39D8.0	fCangjie	YPQ
U+39D8.0	fFourCorner	21500
U+39D8.0	fCantonese	zi6; zi3
U+39D8.0	fMandarin	zi4; cui4; nao3
U+39D8.0	fEnglish	[1] accumulate [2] massage the cheeks
U+39D9.0	fUTF8	㧙
U+39D9.0	fCNS11643	4-2659
U+39D9.0	fGB	3-4A24
U+39D9.0	fR/S	 64.5
U+39D9.0	fTotalStrokes	 8
U+39D9.0	fCangjie	QPH
U+39D9.0	fFourCorner	53000
U+39D9.0	fHakka	 piet8;  bit7; biet8
U+39D9.0	fCantonese	pit3; bit1; bit6
U+39D9.0	fMandarin	bie2; bi4
U+39D9.0	fEnglish	[1] give a playful blow [2] strike against [3] wring; wrench; twist [4] turn for the better
U+39DA.0	fUTF8	㧚
U+39DA.0	fCNS11643	3-292B
U+39DA.0	fR/S	 64.5
U+39DA.0	fTotalStrokes	 8
U+39DA.0	fCangjie	QMVN
U+39DA.0	fFourCorner	51010
U+39DA.0	fHakka	 fong3
U+39DA.0	fCantonese	ngaa5; aa5 (also); ling4
U+39DA.0	fMandarin	fang3; wa3
U+39DA.0	fEnglish	corrupted form of &#x74EC;
U+39DB.0	fUTF8	㧛
U+39DB.0	fGB	S-2240
U+39DB.0	fR/S	 64.5
U+39DB.0	fTotalStrokes	 8
U+39DB.0	fCangjie	LIQ
U+39DB.0	fCantonese	laam5; laam2 (also); hin1
U+39DB.0	fEnglish	simplified form of &#x6394;
U+39DC.0	fUTF8	㧜
U+39DC.0	fGB	S-2242
U+39DC.0	fR/S	 64.5
U+39DC.0	fTotalStrokes	 8
U+39DC.0	fCangjie	QSLB
U+39DC.0	fCantonese	laap6; lip6; laap3; lap6
U+39DC.0	fMandarin	lie4
U+39DC.0	fEnglish	hold; grasp; hold the hair
U+39DD.0	fUTF8	㧝
U+39DD.0	fGB	S-2249
U+39DD.0	fR/S	 64.5
U+39DD.0	fTotalStrokes	 8
U+39DD.0	fCangjie	KRQ
U+39DD.0	fCantonese	ke2; gaa1
U+39DD.0	fEnglish	take
U+39DE.0	fUTF8	㧞
U+39DE.0	fCNS11643	F-272E
U+39DE.0	fR/S	 64.5
U+39DE.0	fTotalStrokes	 8
U+39DE.0	fCangjie	QHKK
U+39DE.0	fCantonese	bat6
U+39DF.0	fUTF8	㧟
U+39DF.0	fGB	7-2244
U+39DF.0	fR/S	 64.5
U+39DF.0	fTotalStrokes	 8
U+39DF.0	fCangjie	QES
U+39DF.0	fCantonese	kwaai5; kwai5
U+39DF.0	fMandarin	kuai3
U+39DF.0	fEnglish	simplified form of &#x64D3;
U+39E0.0	fUTF8	㧠
U+39E0.0	fR/S	 64.5
U+39E0.0	fTotalStrokes	 8
U+39E0.0	fCangjie	QIF
U+39E0.0	fCantonese	mei4
U+39E1.0	fUTF8	㧡
U+39E1.0	fCNS11643	4-294E
U+39E1.0	fGB	3-4A33
U+39E1.0	fR/S	 64.6
U+39E1.0	fTotalStrokes	 9
U+39E1.0	fCangjie	QYVO
U+39E1.0	fFourCorner	50080
U+39E1.0	fHakka	 kai1
U+39E1.0	fCantonese	hoi6
U+39E1.0	fMandarin	hai4
U+39E1.0	fEnglish	[1] move; shake; rock; excite [2] decrease; reduce; diminish; subtract; deduct [3] shoulder; to take upon on oneself
U+39E2.0	fUTF8	㧢
U+39E2.0	fCNS11643	4-2954
U+39E2.0	fGB	5-4C21
U+39E2.0	fR/S	 64.6
U+39E2.0	fTotalStrokes	 9
U+39E2.0	fCangjie	QWK
U+39E2.0	fFourCorner	56000
U+39E2.0	fCantonese	jan1; zaan1; zaan5
U+39E2.0	fMandarin	yin1
U+39E2.0	fEnglish	[1] depend on; lean to; rely on; by means of [2] follow the old or traditional (practices, customs, precedents, etc.) [3] receive and carry on; continue; succeed to
U+39E3.0	fUTF8	㧣
U+39E3.0	fCNS11643	3-2C66
U+39E3.0	fR/S	 64.6
U+39E3.0	fTotalStrokes	 9
U+39E3.0	fCangjie	QHJD
U+39E3.0	fFourCorner	55090
U+39E3.0	fCantonese	zyu1; zyu3
U+39E3.0	fMandarin	zhu1; zhu4
U+39E3.0	fEnglish	[1] stop, detain; stay; end [2] still; calm, only
U+39E4.0	fUTF8	㧤
U+39E4.0	fCNS11643	3-2927
U+39E4.0	fR/S	 64.6; 64.5
U+39E4.0	fTotalStrokes	 8; 9
U+39E4.0	fCangjie	QYIU
U+39E4.0	fFourCorner	50010
U+39E4.0	fCantonese	cung3
U+39E4.0	fMandarin	xian3; chong4
U+39E4.0	fEnglish	[1] (non-classical form) jump; leap; bounce [2] (in general) get angry and leave away [3] bump
U+39E5.0	fUTF8	㧥
U+39E5.0	fCNS11643	3-2C67
U+39E5.0	fR/S	 64.6
U+39E5.0	fTotalStrokes	 9
U+39E5.0	fCangjie	QHGU
U+39E5.0	fFourCorner	54010
U+39E5.0	fHakka	 sien1; sien3
U+39E5.0	fCantonese	sin2
U+39E5.0	fMandarin	xian3
U+39E5.0	fEnglish	 take up in the fingers
U+39E6.0	fUTF8	㧦
U+39E6.0	fCNS11643	3-2C69
U+39E6.0	fGB	5-4C26
U+39E6.0	fR/S	 64.6
U+39E6.0	fTotalStrokes	 9
U+39E6.0	fCangjie	QPA
U+39E6.0	fFourCorner	57020
U+39E6.0	fHakka	  sien5
U+39E6.0	fCantonese	hyun3; gwaang1
U+39E6.0	fMandarin	hong1; xuan4
U+39E6.0	fEnglish	 strike
U+39E7.0	fUTF8	㧧
U+39E7.0	fCNS11643	4-2D3C
U+39E7.0	fR/S	 64.6
U+39E7.0	fTotalStrokes	 10; 9
U+39E7.0	fCangjie	QYIU
U+39E7.0	fFourCorner	54010
U+39E7.0	fHakka	 so1; se1; sii1
U+39E7.0	fCantonese	so1; so3
U+39E7.0	fMandarin	shu1; shu4
U+39E7.0	fEnglish	non-classical form of &#x68B3;
U+39E8.0	fUTF8	㧨
U+39E8.0	fCNS11643	3-2C71
U+39E8.0	fR/S	 64.6
U+39E8.0	fTotalStrokes	 9
U+39E8.0	fCangjie	QHMR
U+39E8.0	fFourCorner	52060
U+39E8.0	fCantonese	kau4
U+39E8.0	fMandarin	qiu2
U+39E8.0	fEnglish	ancient form of &#x624F;
U+39E9.0	fUTF8	㧩
U+39E9.0	fCNS11643	4-2956
U+39E9.0	fGB	3-4A30
U+39E9.0	fR/S	 64.6
U+39E9.0	fTotalStrokes	 9
U+39E9.0	fCangjie	QHNB
U+39E9.0	fFourCorner	57010
U+39E9.0	fCantonese	bui6; pui3
U+39E9.0	fMandarin	pei4
U+39E9.0	fEnglish	[1] dispel [2] move; transfer [3] issue; set aside [4] stir up [5] turning
U+39EA.0	fUTF8	㧪
U+39EA.0	fCNS11643	3-2C68
U+39EA.0	fGB	5-4C25
U+39EA.0	fR/S	 64.6
U+39EA.0	fTotalStrokes	 9
U+39EA.0	fCangjie	QNMU
U+39EA.0	fFourCorner	57010
U+39EA.0	fCantonese	gwai2; gwaai2; ngaai4
U+39EA.0	fMandarin	gui3
U+39EA.0	fEnglish	[1] ruin; destroy; break down [2] hang up; suspend
U+39EB.0	fUTF8	㧫
U+39EB.0	fCNS11643	4-2952
U+39EB.0	fGB	3-4A28
U+39EB.0	fR/S	 64.6
U+39EB.0	fTotalStrokes	 9
U+39EB.0	fCangjie	QMBL
U+39EB.0	fFourCorner	51020
U+39EB.0	fCantonese	ji4; zi4
U+39EB.0	fMandarin	rui2; er2
U+39EB.0	fEnglish	[1] hold; grasp [2] maintain; keep [3] post [4] lead on [5] same as &#x64E9;
U+39EC.0	fUTF8	㧬
U+39EC.0	fCNS11643	4-2D32
U+39EC.0	fGB	3-494B
U+39EC.0	fR/S	 64.6
U+39EC.0	fTotalStrokes	 10
U+39EC.0	fCangjie	MNQ
U+39EC.0	fFourCorner	17500
U+39EC.0	fHakka	 giung3
U+39EC.0	fCantonese	gung2; ngung2; ung2 (also); gung1
U+39EC.0	fMandarin	gong1; gong3
U+39EC.0	fEnglish	[1] embrace [2] push; press
U+39ED.0	fUTF8	㧭
U+39ED.0	fCNS11643	4-2D31
U+39ED.0	fGB	3-494A
U+39ED.0	fR/S	 64.6
U+39ED.0	fTotalStrokes	 10
U+39ED.0	fCangjie	MLQ
U+39ED.0	fFourCorner	17500
U+39ED.0	fCantonese	kung4
U+39ED.0	fMandarin	qing2; qiong2
U+39ED.0	fEnglish	 get something with both raising hands
U+39EE.0	fUTF8	㧮
U+39EE.0	fCNS11643	3-2C6A
U+39EE.0	fR/S	 64.6
U+39EE.0	fTotalStrokes	 9
U+39EE.0	fCangjie	QHX
U+39EE.0	fFourCorner	57070
U+39EE.0	fCantonese	fat1; bak6; bou6; faat1
U+39EE.0	fMandarin	hu1
U+39EE.0	fEnglish	high; tall; lofty
U+39EF.0	fUTF8	㧯
U+39EF.0	fCNS11643	3-2C65
U+39EF.0	fR/S	 64.6
U+39EF.0	fTotalStrokes	 9
U+39EF.0	fCangjie	QJKP
U+39EF.0	fFourCorner	54010
U+39EF.0	fHakka	  lau3;  lau1; lau5
U+39EF.0	fCantonese	lou5
U+39EF.0	fMandarin	lao3
U+39EF.0	fEnglish	[1] a bucket made of willow [2] lift (especially when only a single person is involved)
U+39F0.0	fUTF8	㧰
U+39F0.0	fGB	7-2346
U+39F0.0	fR/S	 64.6
U+39F0.0	fTotalStrokes	 9
U+39F0.0	fCangjie	QHVD
U+39F0.0	fCantonese	lik6; loek6 (also)
U+39F0.0	fMandarin	li4
U+39F0.0	fEnglish	strike; tap, shock
U+39F1.0	fUTF8	㧱
U+39F1.0	fCNS11643	3-3134
U+39F1.0	fR/S	 64.6
U+39F1.0	fTotalStrokes	 10; 9
U+39F1.0	fCangjie	ORQ
U+39F1.0	fFourCorner	82650
U+39F1.0	fCantonese	naa4; laa4 (also); nuk6; zau2; zuk6
U+39F1.0	fMandarin	ba3; chen4; na2; ni4; tian4
U+39F1.0	fEnglish	same as &#x62FF;
U+39F2.0	fUTF8	㧲
U+39F2.0	fCNS11643	3-312A
U+39F2.0	fR/S	 64.7
U+39F2.0	fTotalStrokes	 10
U+39F2.0	fCangjie	QJD
U+39F2.0	fFourCorner	53090
U+39F2.0	fCantonese	sam2
U+39F2.0	fMandarin	san3
U+39F2.0	fEnglish	 shake; rattle
U+39F3.0	fUTF8	㧳
U+39F3.0	fCNS11643	3-357B
U+39F3.0	fGB	5-4B44
U+39F3.0	fR/S	 64.7
U+39F3.0	fTotalStrokes	 11
U+39F3.0	fCangjie	QHQ
U+39F3.0	fFourCorner	58500
U+39F3.0	fHakka	 pai3; pai2; pai5
U+39F3.0	fCantonese	zeoi6; bai1
U+39F3.0	fMandarin	zhai4; bo2; zhuo4
U+39F3.0	fEnglish	[1] pierce [2] separate
U+39F4.0	fUTF8	㧴
U+39F4.0	fCNS11643	3-3131
U+39F4.0	fR/S	 64.7
U+39F4.0	fTotalStrokes	 10
U+39F4.0	fCangjie	QHQI
U+39F4.0	fFourCorner	53050
U+39F4.0	fCantonese	ngo5
U+39F4.0	fMandarin	e3; wo3
U+39F4.0	fEnglish	 rub hands; rub between the hands; twist between the hands
U+39F5.0	fUTF8	㧵
U+39F5.0	fCNS11643	3-313A
U+39F5.0	fR/S	 64.7
U+39F5.0	fTotalStrokes	 10
U+39F5.0	fCangjie	QMFR
U+39F5.0	fFourCorner	51060
U+39F5.0	fHakka	 pui2; piau1; feu2; peu3
U+39F5.0	fCantonese	pau4; bong6; pau2; gong1; paau2; paau4
U+39F5.0	fMandarin	pou3; pou2; bo2; fou2; fu2; fu4; pao2
U+39F5.0	fEnglish	corrupt form of &#x638A;
U+39F6.0	fUTF8	㧶
U+39F6.0	fCNS11643	4-2D3B
U+39F6.0	fGB	5-4C3B
U+39F6.0	fR/S	 64.7
U+39F6.0	fTotalStrokes	 10
U+39F6.0	fCangjie	QHXH
U+39F6.0	fFourCorner	57040
U+39F6.0	fCantonese	hang1
U+39F6.0	fMandarin	keng1
U+39F6.0	fEnglish	[1] the jingling of metals [2] strike [3] lead; guide
U+39F7.0	fUTF8	㧷
U+39F7.0	fCNS11643	3-312D
U+39F7.0	fR/S	 64.7
U+39F7.0	fTotalStrokes	 10
U+39F7.0	fCangjie	QMKR
U+39F7.0	fFourCorner	52060
U+39F7.0	fCantonese	tan3; daau2; taan3; zaam6
U+39F7.0	fMandarin	tun4; duo4
U+39F7.0	fEnglish	[1] to wipe tears [2] bind
U+39F8.0	fUTF8	㧸
U+39F8.0	fCNS11643	3-312B
U+39F8.0	fR/S	 64.7
U+39F8.0	fTotalStrokes	 10
U+39F8.0	fCangjie	QYRN
U+39F8.0	fFourCorner	50020
U+39F8.0	fCantonese	paang1
U+39F8.0	fMandarin	peng1
U+39F8.0	fEnglish	 hit; beat; strike
U+39F9.0	fUTF8	㧹
U+39F9.0	fCNS11643	4-317C
U+39F9.0	fGB	3-4A51
U+39F9.0	fR/S	 64.8
U+39F9.0	fTotalStrokes	 11
U+39F9.0	fCangjie	QAMI
U+39F9.0	fFourCorner	56041
U+39F9.0	fHakka	  dep8;  det7; tit7; diep8; tep8; dip7; dep7
U+39F9.0	fCantonese	tik1
U+39F9.0	fMandarin	te4
U+39F9.0	fEnglish	[1] thump; beat [2] squeeze up close [3] throw
U+39FA.0	fUTF8	㧺
U+39FA.0	fCNS11643	4-3221
U+39FA.0	fGB	3-4A5E
U+39FA.0	fR/S	 64.8
U+39FA.0	fTotalStrokes	 11
U+39FA.0	fCangjie	QEA
U+39FA.0	fFourCorner	52060
U+39FA.0	fHakka	  dap7; tap8; tak7
U+39FA.0	fCantonese	taap3; dap6
U+39FA.0	fMandarin	ta4
U+39FA.0	fEnglish	 thimble
U+39FB.0	fUTF8	㧻
U+39FB.0	fCNS11643	4-317B
U+39FB.0	fGB	5-4C4A
U+39FB.0	fR/S	 64.8
U+39FB.0	fTotalStrokes	 11
U+39FB.0	fCangjie	QMSO
U+39FB.0	fFourCorner	51030
U+39FB.0	fHakka	  dok7; duk8; duk7
U+39FB.0	fCantonese	duk1; doek3
U+39FB.0	fMandarin	zhuo2; zhi3; zu2
U+39FB.0	fEnglish	 rap; beat
U+39FC.0	fUTF8	㧼
U+39FC.0	fCNS11643	3-362C
U+39FC.0	fR/S	 64.8
U+39FC.0	fTotalStrokes	 11
U+39FC.0	fCangjie	QQMV
U+39FC.0	fFourCorner	55030
U+39FC.0	fHakka	 biau3
U+39FC.0	fCantonese	biu2
U+39FC.0	fMandarin	biao4
U+39FC.0	fEnglish	 distribute; scatter; disperse
U+39FD.0	fUTF8	㧽
U+39FD.0	fCNS11643	6-4032
U+39FD.0	fR/S	 64.8
U+39FD.0	fTotalStrokes	 11
U+39FD.0	fCangjie	QWJR
U+39FD.0	fFourCorner	56000
U+39FD.0	fCantonese	gu3; gwu3 (also); guk6
U+39FD.0	fMandarin	gu4; hu2
U+39FD.0	fEnglish	same as &#x7A12;
U+39FE.0	fUTF8	㧾
U+39FE.0	fCNS11643	4-3225
U+39FE.0	fGB	3-4A56
U+39FE.0	fR/S	 64.8
U+39FE.0	fTotalStrokes	 11
U+39FE.0	fCangjie	QPHP
U+39FE.0	fFourCorner	57030
U+39FE.0	fHakka	 fut7
U+39FE.0	fCantonese	fat1
U+39FE.0	fMandarin	ku1
U+39FE.0	fEnglish	[1] to bale out [2] to clean out
U+39FF.0	fUTF8	㧿
U+39FF.0	fCNS11643	3-362F
U+39FF.0	fR/S	 64.8
U+39FF.0	fTotalStrokes	 11
U+39FF.0	fCangjie	QOOO
U+39FF.0	fFourCorner	58080
U+39FF.0	fHakka	 cung1
U+39FF.0	fCantonese	cung1; coeng1
U+39FF.0	fMandarin	chuang1; cong1
U+39FF.0	fEnglish	same as &#x6450;
U+3A00.0	fUTF8	㨀
U+3A00.0	fCNS11643	3-3630
U+3A00.0	fR/S	 64.8
U+3A00.0	fTotalStrokes	 11
U+3A00.0	fCangjie	QHDL
U+3A00.0	fFourCorner	52090
U+3A00.0	fHakka	 bin3
U+3A00.0	fCantonese	bing2; lit6
U+3A00.0	fMandarin	bing3
U+3A00.0	fEnglish	same as &#x62A6;
U+3A01.0	fUTF8	㨁
U+3A01.0	fCNS11643	4-3179
U+3A01.0	fGB	5-4C47
U+3A01.0	fR/S	 64.8
U+3A01.0	fTotalStrokes	 11
U+3A01.0	fCangjie	QJBM
U+3A01.0	fFourCorner	54010
U+3A01.0	fHakka	 chit7; chi5; ts'iit7; cih5
U+3A01.0	fCantonese	zi6; ci5; zik6
U+3A01.0	fMandarin	zhi4; zhi2
U+3A01.0	fEnglish	[1] throw; pitch [2] send [3] stay [4] crutch; staff; stick
U+3A02.0	fUTF8	㨂
U+3A02.0	fCNS11643	3-357E
U+3A02.0	fGB	5-4B6B
U+3A02.0	fR/S	 64.8
U+3A02.0	fTotalStrokes	 11
U+3A02.0	fCangjie	QDW
U+3A02.0	fFourCorner	55096
U+3A02.0	fCantonese	dung2; ce1
U+3A02.0	fMandarin	dong3
U+3A02.0	fEnglish	 beat; strike; attack
U+3A03.0	fUTF8	㨃
U+3A03.0	fCNS11643	4-3171
U+3A03.0	fGB	3-4A57
U+3A03.0	fR/S	 64.8
U+3A03.0	fTotalStrokes	 11
U+3A03.0	fCangjie	QYRD
U+3A03.0	fFourCorner	50040
U+3A03.0	fCantonese	deoi2; cang4
U+3A03.0	fMandarin	dui3; cheng2
U+3A03.0	fEnglish	[1] row; line [2] push; clear out [3] same as &#x673E;
U+3A04.0	fUTF8	㨄
U+3A04.0	fCNS11643	3-3625
U+3A04.0	fGB	5-4C52
U+3A04.0	fR/S	 64.8
U+3A04.0	fTotalStrokes	 11
U+3A04.0	fCangjie	QBGR
U+3A04.0	fFourCorner	57020
U+3A04.0	fCantonese	zau1; ziu6
U+3A04.0	fMandarin	zhao4
U+3A04.0	fEnglish	[1] pierce; stab [2] same as &#x6311; [3] lift something heavy from one side or from the end
U+3A05.0	fUTF8	㨅
U+3A05.0	fCNS11643	4-317E
U+3A05.0	fGB	3-4A38
U+3A05.0	fR/S	 64.8
U+3A05.0	fTotalStrokes	 11
U+3A05.0	fCangjie	QTOB
U+3A05.0	fFourCorner	54020
U+3A05.0	fHakka	 nui5; nui6
U+3A05.0	fCantonese	neoi6; leoi6 (also); noi6
U+3A05.0	fMandarin	rui4; nei4
U+3A05.0	fEnglish	a Chinese family name
U+3A06.0	fUTF8	㨆
U+3A06.0	fCNS11643	4-317A
U+3A06.0	fGB	3-4A4C
U+3A06.0	fR/S	 64.8
U+3A06.0	fTotalStrokes	 11
U+3A06.0	fCangjie	QDD
U+3A06.0	fFourCorner	54090
U+3A06.0	fHakka	 lim2; lim3
U+3A06.0	fCantonese	lam5; lam6 (also)
U+3A06.0	fMandarin	lin3
U+3A06.0	fEnglish	 kill; beat
U+3A07.0	fUTF8	㨇
U+3A07.0	fCNS11643	3-3B6B
U+3A07.0	fGB	5-4B48
U+3A07.0	fR/S	 64.8
U+3A07.0	fTotalStrokes	 12
U+3A07.0	fCangjie	EEQ
U+3A07.0	fFourCorner	34500
U+3A07.0	fCantonese	po4; fuk6; pun4
U+3A07.0	fMandarin	pan2; po2
U+3A07.0	fEnglish	same as &#x642B;
U+3A08.0	fUTF8	㨈
U+3A08.0	fCNS11643	3-362A
U+3A08.0	fR/S	 64.8
U+3A08.0	fTotalStrokes	 11
U+3A08.0	fCangjie	QYKL
U+3A08.0	fFourCorner	50020
U+3A08.0	fCantonese	zai1
U+3A08.0	fMandarin	ji3
U+3A08.0	fEnglish	abbreviated form of &#x64E0;
U+3A09.0	fUTF8	㨉
U+3A09.0	fCNS11643	4-376E
U+3A09.0	fGB	3-4A77
U+3A09.0	fR/S	 64.9
U+3A09.0	fTotalStrokes	 12
U+3A09.0	fCangjie	QRPA
U+3A09.0	fFourCorner	57060
U+3A09.0	fHakka	 min2; mien3; men3
U+3A09.0	fCantonese	man4; maan5
U+3A09.0	fMandarin	min2; min3
U+3A09.0	fEnglish	[1] smooth [2] stroke
U+3A0A.0	fUTF8	㨊
U+3A0A.0	fCNS11643	4-3762
U+3A0A.0	fGB	3-4A65
U+3A0A.0	fR/S	 64.9
U+3A0A.0	fTotalStrokes	 12
U+3A0A.0	fCangjie	QKMB
U+3A0A.0	fFourCorner	54020
U+3A0A.0	fHakka	 to3
U+3A0A.0	fCantonese	wai5; waai4
U+3A0A.0	fMandarin	wei3
U+3A0A.0	fEnglish	[1] discard; reject; abandon [2] feel; stroke [3] sort out the divining stalks [4] drop; lose; fall off [5] weigh; measure weight
U+3A0B.0	fUTF8	㨋
U+3A0B.0	fCNS11643	3-3621
U+3A0B.0	fGB	5-4C45
U+3A0B.0	fR/S	 64.9; 64.8
U+3A0B.0	fTotalStrokes	 11; 12
U+3A0B.0	fCangjie	QJKA
U+3A0B.0	fFourCorner	54060
U+3A0B.0	fCantonese	ce2; ze2
U+3A0B.0	fMandarin	che3; le4; zhen4
U+3A0B.0	fEnglish	 beat; strike; attack
U+3A0C.0	fUTF8	㨌
U+3A0C.0	fCNS11643	4-3D45
U+3A0C.0	fGB	3-4F34
U+3A0C.0	fR/S	 64.9
U+3A0C.0	fTotalStrokes	 13; 12
U+3A0C.0	fCangjie	GQHNE
U+3A0C.0	fFourCorner	47540
U+3A0C.0	fCantonese	gau3
U+3A0C.0	fMandarin	gou4; ru2; ru3
U+3A0C.0	fEnglish	 milk a cow or goat
U+3A0D.0	fUTF8	㨍
U+3A0D.0	fCNS11643	3-4131
U+3A0D.0	fGB	5-4B49
U+3A0D.0	fR/S	 64.9
U+3A0D.0	fTotalStrokes	 13
U+3A0D.0	fCangjie	GIQ
U+3A0D.0	fFourCorner	44500
U+3A0D.0	fHakka	  bong1
U+3A0D.0	fCantonese	bong1
U+3A0D.0	fMandarin	bang1
U+3A0D.0	fEnglish	 oppose; aid by withstanding
U+3A0E.0	fUTF8	㨎
U+3A0E.0	fCNS11643	4-3764
U+3A0E.0	fGB	3-4A67
U+3A0E.0	fR/S	 64.9
U+3A0E.0	fTotalStrokes	 12
U+3A0E.0	fCangjie	QMBK
U+3A0E.0	fFourCorner	51030
U+3A0E.0	fHakka	 ngion1;  ngiet8; ngio1; nio1
U+3A0E.0	fCantonese	jyu5; zyu4
U+3A0E.0	fMandarin	ruan2; ru2
U+3A0E.0	fEnglish	[1] dip; soak [2] push
U+3A0F.0	fUTF8	㨏
U+3A0F.0	fCNS11643	3-3B6A
U+3A0F.0	fR/S	 64.9
U+3A0F.0	fTotalStrokes	 12
U+3A0F.0	fCangjie	QUMF
U+3A0F.0	fFourCorner	52080
U+3A0F.0	fCantonese	taan1
U+3A0F.0	fMandarin	tan1
U+3A0F.0	fEnglish	 beat; strike; attack
U+3A10.0	fUTF8	㨐
U+3A10.0	fCNS11643	4-376B
U+3A10.0	fGB	3-4A6C
U+3A10.0	fR/S	 64.9
U+3A10.0	fTotalStrokes	 12
U+3A10.0	fCangjie	QORD
U+3A10.0	fFourCorner	56090
U+3A10.0	fCantonese	bau2; buk1
U+3A10.0	fMandarin	fou3; bu3; pei2
U+3A10.0	fEnglish	[1] slap lightly on the clothes or coverlet [2] beat; strike; attack
U+3A11.0	fUTF8	㨑
U+3A11.0	fCNS11643	4-3768
U+3A11.0	fGB	3-4A70
U+3A11.0	fR/S	 64.9
U+3A11.0	fTotalStrokes	 12
U+3A11.0	fCangjie	QUCE
U+3A11.0	fFourCorner	52040
U+3A11.0	fCantonese	zung1
U+3A11.0	fMandarin	zong1
U+3A11.0	fEnglish	[1] count; enumerate [2] grasp the head
U+3A12.0	fUTF8	㨒
U+3A12.0	fCNS11643	4-3765
U+3A12.0	fGB	3-4A68
U+3A12.0	fR/S	 64.9
U+3A12.0	fTotalStrokes	 12
U+3A12.0	fCangjie	QKGG
U+3A12.0	fFourCorner	54010
U+3A12.0	fCantonese	fui1
U+3A12.0	fMandarin	kui2; wei3; xie2
U+3A12.0	fEnglish	same as &#x5232;
U+3A13.0	fUTF8	㨓
U+3A13.0	fCNS11643	3-3B6C
U+3A13.0	fR/S	 64.9
U+3A13.0	fTotalStrokes	 12
U+3A13.0	fCangjie	QJCQ
U+3A13.0	fFourCorner	53050
U+3A13.0	fCantonese	lou4
U+3A13.0	fMandarin	lao2; liao2
U+3A13.0	fEnglish	same as &#x22B42;
U+3A14.0	fUTF8	㨔
U+3A14.0	fCNS11643	3-3B69
U+3A14.0	fGB	5-4C5E
U+3A14.0	fR/S	 64.9
U+3A14.0	fTotalStrokes	 12
U+3A14.0	fCangjie	QIHR
U+3A14.0	fFourCorner	53050
U+3A14.0	fHakka	 ham5
U+3A14.0	fCantonese	ham6; haam6; ngaan4
U+3A14.0	fMandarin	han4
U+3A14.0	fEnglish	 shake; rock; jolt; joggle
U+3A15.0	fUTF8	㨕
U+3A15.0	fCNS11643	4-3770
U+3A15.0	fR/S	 64.9
U+3A15.0	fTotalStrokes	 12
U+3A15.0	fCangjie	QNST
U+3A15.0	fFourCorner	57010
U+3A15.0	fCantonese	jing4; zing4
U+3A15.0	fMandarin	ying2
U+3A15.0	fEnglish	same as &#x650D;
U+3A16.0	fUTF8	㨖
U+3A16.0	fCNS11643	4-3761
U+3A16.0	fGB	3-4A7C
U+3A16.0	fR/S	 64.9
U+3A16.0	fTotalStrokes	 12
U+3A16.0	fCangjie	QMGK
U+3A16.0	fFourCorner	58040
U+3A16.0	fCantonese	zi3
U+3A16.0	fMandarin	zhi4
U+3A16.0	fEnglish	[1] stab [2] rob of money; rob of riches [3] go nonstop to [4] cluster together; mass
U+3A17.0	fUTF8	㨗
U+3A17.0	fCNS11643	3-3B6E
U+3A17.0	fR/S	 64.9
U+3A17.0	fTotalStrokes	 12
U+3A17.0	fCangjie	QYSO
U+3A17.0	fFourCorner	51080
U+3A17.0	fHakka	 ciap8
U+3A17.0	fCantonese	zit6; zit3 (also); cip3
U+3A17.0	fMandarin	jie2; cha1; qie4
U+3A17.0	fEnglish	variant of &#x6377;
U+3A18.0	fUTF8	㨘
U+3A18.0	fGB	5-4C5F
U+3A18.0	fR/S	 64.9
U+3A18.0	fTotalStrokes	 12
U+3A18.0	fCangjie	QFHU
U+3A18.0	fFourCorner	6304
U+3A18.0	fCantonese	saang2; sang3; saang3
U+3A18.0	fMandarin	xing3
U+3A18.0	fEnglish	blow the nose with the fingers
U+3A19.0	fUTF8	㨙
U+3A19.0	fCNS11643	4-3D50
U+3A19.0	fGB	3-4A7E
U+3A19.0	fR/S	 64.10
U+3A19.0	fTotalStrokes	 13
U+3A19.0	fCangjie	QBVK
U+3A19.0	fFourCorner	52030
U+3A19.0	fHakka	 kai1
U+3A19.0	fCantonese	haai4
U+3A19.0	fMandarin	xie2
U+3A19.0	fEnglish	[1] clasp or hold under the arm [2] support; prop up; aid [3] exchange; alter [4] lift up or off; raise high
U+3A1A.0	fUTF8	㨚
U+3A1A.0	fCNS11643	4-3D4F
U+3A1A.0	fGB	3-4A63
U+3A1A.0	fR/S	 64.10
U+3A1A.0	fTotalStrokes	 13
U+3A1A.0	fCangjie	QTPA
U+3A1A.0	fFourCorner	54020
U+3A1A.0	fCantonese	seon1
U+3A1A.0	fMandarin	sun3; xun2
U+3A1A.0	fEnglish	[1] select; choose; pick out [2] defend; ward off [3] refuse; reject
U+3A1B.0	fUTF8	㨛
U+3A1B.0	fCNS11643	3-4137
U+3A1B.0	fGB	5-4B70
U+3A1B.0	fR/S	 64.10
U+3A1B.0	fTotalStrokes	 13
U+3A1B.0	fCangjie	QANO
U+3A1B.0	fFourCorner	57020
U+3A1B.0	fCantonese	sim2
U+3A1B.0	fMandarin	shan3
U+3A1B.0	fEnglish	[1] swift; rapid; quick; speedy [2] same as &#x639E;
U+3A1C.0	fUTF8	㨜
U+3A1C.0	fCNS11643	4-3D4C
U+3A1C.0	fGB	5-4D24
U+3A1C.0	fR/S	 64.10
U+3A1C.0	fTotalStrokes	 13
U+3A1C.0	fCangjie	QYPK
U+3A1C.0	fFourCorner	51040
U+3A1C.0	fCantonese	kin4
U+3A1C.0	fMandarin	qian2
U+3A1C.0	fEnglish	[1] help each other [2] shoulder; take upon oneself
U+3A1D.0	fUTF8	㨝
U+3A1D.0	fCNS11643	4-3D4A
U+3A1D.0	fGB	3-4B25
U+3A1D.0	fR/S	 64.10
U+3A1D.0	fTotalStrokes	 13
U+3A1D.0	fCangjie	QSFB
U+3A1D.0	fFourCorner	57020
U+3A1D.0	fHakka	 siet7
U+3A1D.0	fCantonese	sit3
U+3A1D.0	fMandarin	xie4
U+3A1D.0	fEnglish	 push or pull out a stopper
U+3A1E.0	fUTF8	㨞
U+3A1E.0	fCNS11643	3-4135
U+3A1E.0	fGB	5-4C7A
U+3A1E.0	fR/S	 64.10
U+3A1E.0	fTotalStrokes	 13
U+3A1E.0	fCangjie	QQMF
U+3A1E.0	fFourCorner	55090
U+3A1E.0	fCantonese	sou3
U+3A1E.0	fMandarin	su4
U+3A1E.0	fEnglish	 feel about; grope in the dark
U+3A1F.0	fUTF8	㨟
U+3A1F.0	fCNS11643	3-413B
U+3A1F.0	fR/S	 64.10
U+3A1F.0	fTotalStrokes	 13
U+3A1F.0	fCangjie	QUMT
U+3A1F.0	fFourCorner	52018
U+3A1F.0	fCantonese	goi1
U+3A1F.0	fMandarin	guai1; hai2
U+3A1F.0	fEnglish	[1] touch; contact [2] ram; butt
U+3A20.0	fUTF8	㨠
U+3A20.0	fCNS11643	4-3D48
U+3A20.0	fGB	5-4D33
U+3A20.0	fR/S	 64.10
U+3A20.0	fTotalStrokes	 13
U+3A20.0	fCangjie	QBAC
U+3A20.0	fFourCorner	57080
U+3A20.0	fCantonese	mik6
U+3A20.0	fMandarin	mian3; mi4
U+3A20.0	fEnglish	same as &#x5853;
U+3A21.0	fUTF8	㨡
U+3A21.0	fCNS11643	4-3D4E
U+3A21.0	fGB	3-4A7D
U+3A21.0	fR/S	 64.10
U+3A21.0	fTotalStrokes	 13
U+3A21.0	fCangjie	QWMO
U+3A21.0	fFourCorner	56000
U+3A21.0	fHakka	 fun3; gut7; gwut7; fun5; fut7
U+3A21.0	fCantonese	wat6; waan4
U+3A21.0	fMandarin	hu2; hun2
U+3A21.0	fEnglish	[1] touch [2] pick up as with pincers
U+3A22.0	fUTF8	㨢
U+3A22.0	fCNS11643	3-413C
U+3A22.0	fGB	5-4D36
U+3A22.0	fR/S	 64.10
U+3A22.0	fTotalStrokes	 13
U+3A22.0	fCangjie	QIBP
U+3A22.0	fFourCorner	51010
U+3A22.0	fCantonese	pei1
U+3A22.0	fMandarin	pi1
U+3A22.0	fEnglish	 rip open flesh; cut flesh
U+3A23.0	fUTF8	㨣
U+3A23.0	fCNS11643	3-414A
U+3A23.0	fGB	5-4D2A
U+3A23.0	fR/S	 64.10
U+3A23.0	fTotalStrokes	 13
U+3A23.0	fCangjie	QCIM
U+3A23.0	fFourCorner	58020
U+3A23.0	fCantonese	jung2
U+3A23.0	fMandarin	yong3
U+3A24.0	fUTF8	㨤
U+3A24.0	fCNS11643	3-4143
U+3A24.0	fR/S	 64.10
U+3A24.0	fTotalStrokes	 13
U+3A24.0	fCangjie	QSOG
U+3A24.0	fFourCorner	51010
U+3A24.0	fHakka	 kwai5
U+3A24.0	fCantonese	kwaai5; kwai5
U+3A24.0	fMandarin	kuai3; hui4; wai4
U+3A24.0	fEnglish	same as &#x64D3;
U+3A25.0	fUTF8	㨥
U+3A25.0	fCNS11643	3-413F
U+3A25.0	fGB	5-4C42
U+3A25.0	fR/S	 64.10
U+3A25.0	fTotalStrokes	 13
U+3A25.0	fCangjie	QVFB
U+3A25.0	fFourCorner	54020
U+3A25.0	fCantonese	naap6; laap6 (also); nap6
U+3A25.0	fMandarin	na4
U+3A25.0	fEnglish	 hit; strike
U+3A26.0	fUTF8	㨦
U+3A26.0	fCNS11643	3-4147
U+3A26.0	fR/S	 64.10
U+3A26.0	fTotalStrokes	 13
U+3A26.0	fCangjie	QOGE
U+3A26.0	fFourCorner	50040
U+3A26.0	fHakka	 sung3
U+3A26.0	fCantonese	sung2
U+3A26.0	fMandarin	song3
U+3A26.0	fEnglish	[1] (abbreviated form) hold; grasp [2] detain [3] uphold [4] push [5] stand upright
U+3A27.0	fUTF8	㨧
U+3A27.0	fCNS11643	3-4140
U+3A27.0	fR/S	 64.10
U+3A27.0	fTotalStrokes	 13
U+3A27.0	fCangjie	QIKW
U+3A27.0	fFourCorner	53060
U+3A27.0	fCantonese	ban6; bun2
U+3A27.0	fMandarin	ben4
U+3A27.0	fEnglish	wooden stick or bamboo cane (shaped segment of a circle) used to support the covering of a carriage in ancient times
U+3A28.0	fUTF8	㨨
U+3A28.0	fCNS11643	3-4146
U+3A28.0	fGB	5-4D2D
U+3A28.0	fR/S	 64.10
U+3A28.0	fTotalStrokes	 13
U+3A28.0	fCangjie	QHHW
U+3A28.0	fFourCorner	57060
U+3A28.0	fHakka	 liu5; chu1; cu1; chiu1
U+3A28.0	fCantonese	cau1; zau1; caau1; lau6
U+3A28.0	fMandarin	chou1; liu4; chou2
U+3A28.0	fEnglish	draw out
U+3A29.0	fUTF8	㨩
U+3A29.0	fCNS11643	3-4141
U+3A29.0	fR/S	 64.10
U+3A29.0	fTotalStrokes	 13
U+3A29.0	fCangjie	QING
U+3A29.0	fFourCorner	50041
U+3A29.0	fHakka	 ciap8
U+3A29.0	fCantonese	zit6
U+3A29.0	fMandarin	jie2; cha1; qie4
U+3A29.0	fEnglish	same as &#x6377;
U+3A2A.0	fUTF8	㨪
U+3A2A.0	fCNS11643	3-4139
U+3A2A.0	fR/S	 64.10
U+3A2A.0	fTotalStrokes	 13
U+3A2A.0	fCangjie	QAFU
U+3A2A.0	fFourCorner	56010
U+3A2A.0	fHakka	 gong2; fong3
U+3A2A.0	fCantonese	fong2
U+3A2A.0	fMandarin	huang4
U+3A2A.0	fEnglish	[1] shake; be disturbed; oscillating [2] brightness [3] dazzling; glaring
U+3A2B.0	fUTF8	㨫
U+3A2B.0	fGB	S-2244
U+3A2B.0	fR/S	 64.10
U+3A2B.0	fTotalStrokes	 13
U+3A2B.0	fCangjie	QLIT
U+3A2B.0	fCantonese	laam5; lam4; lam5
U+3A2B.0	fMandarin	lan3
U+3A2B.0	fEnglish	be in full possession of
U+3A2C.0	fUTF8	㨬
U+3A2C.0	fGB	7-2347
U+3A2C.0	fR/S	 64.10
U+3A2C.0	fTotalStrokes	 13
U+3A2C.0	fCangjie	QGRV
U+3A2C.0	fCantonese	jyun4
U+3A2D.0	fUTF8	㨭
U+3A2D.0	fCNS11643	4-4358
U+3A2D.0	fGB	3-4B37
U+3A2D.0	fR/S	 64.11
U+3A2D.0	fTotalStrokes	 14
U+3A2D.0	fCangjie	QISU
U+3A2D.0	fFourCorner	53017
U+3A2D.0	fHakka	 fu5
U+3A2D.0	fCantonese	wu6
U+3A2D.0	fMandarin	hu4
U+3A2D.0	fEnglish	 diffuse; distribute
U+3A2E.0	fUTF8	㨮
U+3A2E.0	fCNS11643	4-4361
U+3A2E.0	fGB	3-4B33
U+3A2E.0	fR/S	 64.11
U+3A2E.0	fTotalStrokes	 14
U+3A2E.0	fCangjie	QHVU
U+3A2E.0	fFourCorner	57010
U+3A2E.0	fHakka	  deu1
U+3A2E.0	fCantonese	dau1
U+3A2E.0	fMandarin	dou1
U+3A2E.0	fEnglish	[1] split; open [2] raise up; lift; take hold of [3] lift
U+3A2F.0	fUTF8	㨯
U+3A2F.0	fCNS11643	6-5D32
U+3A2F.0	fR/S	 64.11
U+3A2F.0	fTotalStrokes	 14
U+3A2F.0	fCangjie	QYDL
U+3A2F.0	fFourCorner	57020
U+3A2F.0	fHakka	 kwok7
U+3A2F.0	fCantonese	kwok3; kok3 (also); gwok3; kwong3
U+3A2F.0	fMandarin	kuo4; huo4
U+3A2F.0	fEnglish	same as &#x64F4;
U+3A30.0	fUTF8	㨰
U+3A30.0	fCNS11643	3-4679
U+3A30.0	fGB	5-4D2F
U+3A30.0	fR/S	 64.11
U+3A30.0	fTotalStrokes	 14
U+3A30.0	fCangjie	QYCV
U+3A30.0	fFourCorner	50030
U+3A30.0	fCantonese	gwan3; gwaan2
U+3A30.0	fMandarin	gun4; ge2; gun3; hun4; huo4; jie2
U+3A30.0	fEnglish	 do (Hakka)
U+3A31.0	fUTF8	㨱
U+3A31.0	fCNS11643	3-472E
U+3A31.0	fR/S	 64.11
U+3A31.0	fTotalStrokes	 14
U+3A31.0	fCangjie	QBYR
U+3A31.0	fFourCorner	57060
U+3A31.0	fHakka	 jau2; jeu2
U+3A31.0	fCantonese	jiu4; ziu4
U+3A31.0	fMandarin	yao2; yao4
U+3A31.0	fEnglish	same as &#x6416;
U+3A32.0	fUTF8	㨲
U+3A32.0	fCNS11643	3-472D
U+3A32.0	fR/S	 64.11
U+3A32.0	fTotalStrokes	 14
U+3A32.0	fCangjie	QTJD
U+3A32.0	fFourCorner	54094
U+3A32.0	fCantonese	caak3
U+3A32.0	fMandarin	ce4
U+3A32.0	fEnglish	[1] back up; support [2] take; receive; obtain [3] select; choose
U+3A33.0	fUTF8	㨳
U+3A33.0	fCNS11643	4-4356
U+3A33.0	fGB	5-4D44
U+3A33.0	fR/S	 64.11
U+3A33.0	fTotalStrokes	 14
U+3A33.0	fCangjie	QJKR
U+3A33.0	fFourCorner	53020
U+3A33.0	fHakka	 gi5; ke2
U+3A33.0	fCantonese	gwai2; gwaai2; kyut3; wan4
U+3A33.0	fMandarin	gui1; gui3
U+3A33.0	fEnglish	same as &#x5E8B;
U+3A34.0	fUTF8	㨴
U+3A34.0	fCNS11643	3-467C
U+3A34.0	fR/S	 64.11
U+3A34.0	fTotalStrokes	 14
U+3A34.0	fCangjie	QJJN
U+3A34.0	fFourCorner	58010
U+3A34.0	fHakka	 gen5; gien5
U+3A34.0	fCantonese	kin2; gin3; hin1; zin5
U+3A34.0	fMandarin	jian3; jian4
U+3A34.0	fEnglish	same as &#x5EFA;
U+3A35.0	fUTF8	㨵
U+3A35.0	fCNS11643	3-4727
U+3A35.0	fR/S	 64.11
U+3A35.0	fTotalStrokes	 14
U+3A35.0	fCangjie	QTBH
U+3A35.0	fFourCorner	58020
U+3A35.0	fHakka	 zien1
U+3A35.0	fCantonese	zin2; zin3
U+3A35.0	fMandarin	jian1; jian3
U+3A35.0	fEnglish	non-classical form of &#x63C3;
U+3A36.0	fUTF8	㨶
U+3A36.0	fCNS11643	3-472F
U+3A36.0	fGB	5-4B63
U+3A36.0	fR/S	 64.11
U+3A36.0	fTotalStrokes	 14
U+3A36.0	fCangjie	QHAF
U+3A36.0	fFourCorner	57020
U+3A36.0	fHakka	 dau3
U+3A36.0	fCantonese	dou2
U+3A36.0	fMandarin	dao3; chou2; zhou3; zhou4
U+3A36.0	fEnglish	same as &#x64E3;
U+3A37.0	fUTF8	㨷
U+3A37.0	fCNS11643	4-435C
U+3A37.0	fGB	3-4B29
U+3A37.0	fR/S	 64.11
U+3A37.0	fTotalStrokes	 14
U+3A37.0	fCangjie	QTLM
U+3A37.0	fFourCorner	54010
U+3A37.0	fCantonese	gan3; gaam3
U+3A37.0	fMandarin	jin4
U+3A37.0	fEnglish	[1] wipe; cleanse [2] interchangeable with &#x8271;
U+3A38.0	fUTF8	㨸
U+3A38.0	fCNS11643	3-4677
U+3A38.0	fR/S	 64.11
U+3A38.0	fTotalStrokes	 14
U+3A38.0	fCangjie	QJPU
U+3A38.0	fFourCorner	53070
U+3A38.0	fHakka	  met8; mit8
U+3A38.0	fCantonese	maat3; bei3; maa6
U+3A38.0	fMandarin	ma4
U+3A38.0	fEnglish	 strike
U+3A39.0	fUTF8	㨹
U+3A39.0	fCNS11643	5-4625
U+3A39.0	fGB	3-4B27
U+3A39.0	fR/S	 64.11
U+3A39.0	fTotalStrokes	 14
U+3A39.0	fCangjie	QQJS
U+3A39.0	fFourCorner	55077
U+3A39.0	fHakka	 siet7; wui5; wi5; fui5; fi5; vui5
U+3A39.0	fCantonese	wai6
U+3A39.0	fMandarin	hui4
U+3A39.0	fEnglish	 sweep away
U+3A3A.0	fUTF8	㨺
U+3A3A.0	fCNS11643	3-467D
U+3A3A.0	fGB	5-4C7E
U+3A3A.0	fR/S	 64.11
U+3A3A.0	fTotalStrokes	 14
U+3A3A.0	fCangjie	QTLB
U+3A3A.0	fFourCorner	54020
U+3A3A.0	fHakka	 mun2
U+3A3A.0	fCantonese	min5; mun4
U+3A3A.0	fMandarin	mian3; men2; mi4
U+3A3A.0	fEnglish	[1] ornament; polish; decorate [2] same as &#x636B;
U+3A3B.0	fUTF8	㨻
U+3A3B.0	fCNS11643	4-4975
U+3A3B.0	fGB	5-4B46
U+3A3B.0	fR/S	 64.11
U+3A3B.0	fTotalStrokes	 15
U+3A3B.0	fCangjie	JLQ
U+3A3B.0	fFourCorner	52500
U+3A3B.0	fHakka	 cam1; zam1
U+3A3B.0	fCantonese	caam4; cam3; cam4; cim3; zam6; zim6
U+3A3B.0	fMandarin	can2
U+3A3B.0	fEnglish	[1] strike; raise up [2] cut in two
U+3A3C.0	fUTF8	㨼
U+3A3C.0	fCNS11643	3-4C36
U+3A3C.0	fR/S	 64.11
U+3A3C.0	fTotalStrokes	 15
U+3A3C.0	fCangjie	WRQ
U+3A3C.0	fFourCorner	67500
U+3A3C.0	fHakka	  liok8; liau5
U+3A3C.0	fCantonese	loek6
U+3A3C.0	fMandarin	luue4; le4
U+3A3C.0	fEnglish	 cast aside; throw
U+3A3D.0	fUTF8	㨽
U+3A3D.0	fCNS11643	4-4364
U+3A3D.0	fGB	3-4B2B
U+3A3D.0	fR/S	 64.11
U+3A3D.0	fTotalStrokes	 14
U+3A3D.0	fCangjie	QMWN
U+3A3D.0	fFourCorner	52000
U+3A3D.0	fCantonese	bik1; baat6; fan6
U+3A3D.0	fMandarin	pi4; pi3; qiao3
U+3A3D.0	fEnglish	 cut or rip open
U+3A3E.0	fUTF8	㨾
U+3A3E.0	fCNS11643	5-4623
U+3A3E.0	fR/S	 64.11
U+3A3E.0	fTotalStrokes	 14
U+3A3E.0	fCangjie	QTGE
U+3A3E.0	fFourCorner	58032
U+3A3E.0	fHakka	 jong5; rong6; jong6
U+3A3E.0	fCantonese	joeng6; zoeng6
U+3A3E.0	fMandarin	yang4
U+3A3E.0	fEnglish	 model; pattern
U+3A3F.0	fUTF8	㨿
U+3A3F.0	fCNS11643	3-472B
U+3A3F.0	fR/S	 64.11
U+3A3F.0	fTotalStrokes	 14
U+3A3F.0	fCangjie	QYPN
U+3A3F.0	fFourCorner	51040
U+3A3F.0	fHakka	 gi3
U+3A3F.0	fCantonese	geoi3; gei1; goeng3
U+3A3F.0	fMandarin	ju4; ju1; ji3
U+3A3F.0	fEnglish	variant of &#x64DA;
U+3A40.0	fUTF8	㩀
U+3A40.0	fCNS11643	3-472C
U+3A40.0	fR/S	 64.11
U+3A40.0	fTotalStrokes	 14
U+3A40.0	fCangjie	QYPK
U+3A40.0	fFourCorner	51027
U+3A40.0	fHakka	 gi3
U+3A40.0	fCantonese	geoi3; goeng3
U+3A40.0	fMandarin	ju4; ju1; ji3
U+3A40.0	fEnglish	variant of &#x64DA;
U+3A41.0	fUTF8	㩁
U+3A41.0	fR/S	 64.11
U+3A41.0	fTotalStrokes	 14
U+3A41.0	fCangjie	QJOG
U+3A41.0	fCantonese	kok1; kok3
U+3A41.0	fMandarin	que4
U+3A41.0	fEnglish	[Korean character]
U+3A42.0	fUTF8	㩂
U+3A42.0	fR/S	 64.11
U+3A42.0	fTotalStrokes	 14
U+3A42.0	fCangjie	QNBJ
U+3A42.0	fCantonese	huk6
U+3A42.0	fEnglish	[Korean character]
U+3A43.0	fUTF8	㩃
U+3A43.0	fCNS11643	4-4A24
U+3A43.0	fGB	5-4D56
U+3A43.0	fR/S	 64.12
U+3A43.0	fTotalStrokes	 15
U+3A43.0	fCangjie	QJTY
U+3A43.0	fFourCorner	53030
U+3A43.0	fCantonese	gin2
U+3A43.0	fMandarin	jian3
U+3A43.0	fEnglish	take or capture (a city, etc.); gather; collect
U+3A44.0	fUTF8	㩄
U+3A44.0	fCNS11643	3-4C33
U+3A44.0	fGB	5-4D52
U+3A44.0	fR/S	 64.12
U+3A44.0	fTotalStrokes	 15
U+3A44.0	fCangjie	QUWP
U+3A44.0	fFourCorner	52030
U+3A44.0	fHakka	  sai1;  sii1; se1
U+3A44.0	fCantonese	saai1
U+3A44.0	fMandarin	shai1
U+3A44.0	fEnglish	 beat (as a gong)
U+3A45.0	fUTF8	㩅
U+3A45.0	fCNS11643	7-233A
U+3A45.0	fR/S	 64.12
U+3A45.0	fTotalStrokes	 15
U+3A45.0	fCangjie	QMLW
U+3A45.0	fFourCorner	51060
U+3A45.0	fHakka	 liu5
U+3A45.0	fCantonese	cau1; caau1; lau6
U+3A45.0	fMandarin	chou1
U+3A45.0	fEnglish	ancient form of &#x62BD;
U+3A46.0	fUTF8	㩆
U+3A46.0	fCNS11643	4-4979
U+3A46.0	fGB	3-4B43
U+3A46.0	fR/S	 64.12
U+3A46.0	fTotalStrokes	 15
U+3A46.0	fCangjie	QYFU
U+3A46.0	fFourCorner	53010
U+3A46.0	fCantonese	zau3; zaau6
U+3A46.0	fMandarin	jiu4
U+3A46.0	fEnglish	[1] be in full possession of; grasp [2] beat cruelly; beat brutally; beat severely
U+3A47.0	fUTF8	㩇
U+3A47.0	fCNS11643	4-497A
U+3A47.0	fGB	5-4C49
U+3A47.0	fR/S	 64.12
U+3A47.0	fTotalStrokes	 15
U+3A47.0	fCangjie	QLGM
U+3A47.0	fFourCorner	55010
U+3A47.0	fCantonese	waak6
U+3A47.0	fMandarin	huo4; hua4
U+3A47.0	fEnglish	[1] the thumb [2] split; tear apart [3] sound of ripping or tearing [4] cut open with something sharp
U+3A48.0	fUTF8	㩈
U+3A48.0	fCNS11643	4-4976
U+3A48.0	fGB	3-4B46
U+3A48.0	fR/S	 64.12
U+3A48.0	fTotalStrokes	 15
U+3A48.0	fCangjie	QJCR
U+3A48.0	fFourCorner	53060
U+3A48.0	fCantonese	wan5; kwaan1
U+3A48.0	fMandarin	yun3; xian4
U+3A48.0	fEnglish	 restrain; restrained; timid and awkward; to feel not at home; miserably poor
U+3A49.0	fUTF8	㩉
U+3A49.0	fCNS11643	4-4A28
U+3A49.0	fGB	3-4B40
U+3A49.0	fR/S	 64.12
U+3A49.0	fTotalStrokes	 15
U+3A49.0	fCangjie	QORM
U+3A49.0	fFourCorner	58020
U+3A49.0	fCantonese	daap3; lai1
U+3A49.0	fMandarin	xi1; xie2
U+3A49.0	fEnglish	[1] same as &#x6428; [2] same as &#x62C9; [3] same as &#x62F9;
U+3A4A.0	fUTF8	㩊
U+3A4A.0	fCNS11643	4-4A22
U+3A4A.0	fGB	3-4B41
U+3A4A.0	fR/S	 64.12
U+3A4A.0	fTotalStrokes	 15
U+3A4A.0	fCangjie	QBWI
U+3A4A.0	fFourCorner	52040
U+3A4A.0	fCantonese	syun1
U+3A4A.0	fMandarin	xuan1
U+3A4A.0	fEnglish	[1] pull up the sleeves and show the arms [2] fight with bare hands [3] desire for more than one's rightful share; covet; greedy
U+3A4B.0	fUTF8	㩋
U+3A4B.0	fCNS11643	5-4D3A
U+3A4B.0	fGB	3-4A5D
U+3A4B.0	fR/S	 64.12
U+3A4B.0	fTotalStrokes	 15
U+3A4B.0	fCangjie	QLX
U+3A4B.0	fFourCorner	55020
U+3A4B.0	fHakka	 siuk7; suk7
U+3A4B.0	fCantonese	sok1; siu1; suk1
U+3A4B.0	fMandarin	su4
U+3A4B.0	fEnglish	 strike; pound
U+3A4C.0	fUTF8	㩌
U+3A4C.0	fCNS11643	4-497E
U+3A4C.0	fGB	5-4C76
U+3A4C.0	fR/S	 64.12
U+3A4C.0	fTotalStrokes	 15; 16
U+3A4C.0	fCangjie	QLNC
U+3A4C.0	fFourCorner	55080
U+3A4C.0	fCantonese	fei6; faai3
U+3A4C.0	fMandarin	fei4
U+3A4C.0	fEnglish	[1] strike; fight with hands [2] scratch lightly; brush; shake; dust [3] knock down; floor
U+3A4D.0	fUTF8	㩍
U+3A4D.0	fCNS11643	5-4D3E
U+3A4D.0	fR/S	 64.12
U+3A4D.0	fTotalStrokes	 15
U+3A4D.0	fCangjie	QHDB
U+3A4D.0	fFourCorner	58090
U+3A4D.0	fCantonese	caak3
U+3A4D.0	fMandarin	ce4
U+3A4D.0	fEnglish	[1] back up; support [2] take; receive; fetch; obtain; take hold of
U+3A4E.0	fUTF8	㩎
U+3A4E.0	fCNS11643	3-4C3C
U+3A4E.0	fGB	5-4D4F
U+3A4E.0	fR/S	 64.12
U+3A4E.0	fTotalStrokes	 15
U+3A4E.0	fCangjie	QABK
U+3A4E.0	fFourCorner	53030
U+3A4E.0	fCantonese	jip3; zip3
U+3A4E.0	fMandarin	ye4
U+3A4E.0	fEnglish	same as &#x64EA;
U+3A4F.0	fUTF8	㩏
U+3A4F.0	fGB	7-2348
U+3A4F.0	fR/S	 64.12
U+3A4F.0	fTotalStrokes	 15
U+3A4F.0	fCangjie	QWGF
U+3A4F.0	fCantonese	hak1
U+3A50.0	fUTF8	㩐
U+3A50.0	fCNS11643	4-4A23
U+3A50.0	fGB	5-4D53
U+3A50.0	fR/S	 64.12
U+3A50.0	fTotalStrokes	 15
U+3A50.0	fCangjie	QHGI
U+3A50.0	fFourCorner	58040
U+3A50.0	fCantonese	dan3; dan6 (also)
U+3A51.0	fUTF8	㩑
U+3A51.0	fGB	3-4B39
U+3A51.0	fR/S	 64.12
U+3A51.0	fTotalStrokes	 15
U+3A51.0	fCangjie	QMKY
U+3A51.0	fCantonese	saap3
U+3A52.0	fUTF8	㩒
U+3A52.0	fCNS11643	3-5074
U+3A52.0	fGB	5-4D60
U+3A52.0	fR/S	 64.13
U+3A52.0	fTotalStrokes	 16
U+3A52.0	fCangjie	QDDF
U+3A52.0	fFourCorner	59090
U+3A52.0	fCantonese	gam6; kaam4
U+3A52.0	fMandarin	qin2
U+3A52.0	fEnglish	same as &#x64D2;
U+3A53.0	fUTF8	㩓
U+3A53.0	fCNS11643	4-5639
U+3A53.0	fGB	3-4952
U+3A53.0	fR/S	 64.13
U+3A53.0	fTotalStrokes	 17; 16
U+3A53.0	fCangjie	HEQ
U+3A53.0	fFourCorner	77500
U+3A53.0	fCantonese	wai2
U+3A53.0	fMandarin	hui3
U+3A53.0	fEnglish	 wound; damage; destroy; injure
U+3A54.0	fUTF8	㩔
U+3A54.0	fCNS11643	4-5045
U+3A54.0	fGB	3-4B4C
U+3A54.0	fR/S	 64.13
U+3A54.0	fTotalStrokes	 16
U+3A54.0	fCangjie	QSCE
U+3A54.0	fFourCorner	57040
U+3A54.0	fCantonese	tyun4
U+3A54.0	fMandarin	tun2
U+3A54.0	fEnglish	[1] a tool to make (crumble and to make smooth) a bow [2] beat; strike; attack
U+3A55.0	fUTF8	㩕
U+3A55.0	fCNS11643	3-5073
U+3A55.0	fR/S	 64.13
U+3A55.0	fTotalStrokes	 16
U+3A55.0	fCangjie	QMBI
U+3A55.0	fFourCorner	51030
U+3A55.0	fHakka	 lang2
U+3A55.0	fCantonese	ling1
U+3A55.0	fMandarin	ling1; ling2
U+3A55.0	fEnglish	same as &#x62CE;
U+3A56.0	fUTF8	㩖
U+3A56.0	fCNS11643	3-506E
U+3A56.0	fR/S	 64.13
U+3A56.0	fTotalStrokes	 16
U+3A56.0	fCangjie	QMWM
U+3A56.0	fFourCorner	51010
U+3A56.0	fCantonese	koeng4
U+3A56.0	fMandarin	qiang2; tiao2
U+3A56.0	fEnglish	 back up; support
U+3A57.0	fUTF8	㩗
U+3A57.0	fCNS11643	3-5078
U+3A57.0	fR/S	 64.13
U+3A57.0	fTotalStrokes	 16
U+3A57.0	fCangjie	QUOS
U+3A57.0	fFourCorner	52020
U+3A57.0	fHakka	 hi1
U+3A57.0	fCantonese	kwai4
U+3A57.0	fMandarin	xi1; xie2; xi2
U+3A57.0	fEnglish	non-classical form of &#x651C;
U+3A58.0	fUTF8	㩘
U+3A58.0	fCNS11643	3-5072
U+3A58.0	fR/S	 64.13
U+3A58.0	fTotalStrokes	 16
U+3A58.0	fCangjie	QTGI
U+3A58.0	fFourCorner	58050
U+3A58.0	fHakka	 ngi5; ngi5ngi3
U+3A58.0	fCantonese	ji5; ngai5; hei1; ngaai5
U+3A58.0	fMandarin	yi3; e2; yi2
U+3A58.0	fEnglish	corrupted form of &#x6AA5;
U+3A59.0	fUTF8	㩙
U+3A59.0	fCNS11643	3-5071
U+3A59.0	fGB	5-4D66
U+3A59.0	fR/S	 64.13
U+3A59.0	fTotalStrokes	 16
U+3A59.0	fCangjie	QJTG
U+3A59.0	fFourCorner	53010
U+3A59.0	fHakka	  set7; cet7
U+3A59.0	fCantonese	sak1; nim2; saak1
U+3A59.0	fMandarin	sai1
U+3A59.0	fEnglish	 stop up
U+3A5A.0	fUTF8	㩚
U+3A5A.0	fCNS11643	3-5465
U+3A5A.0	fR/S	 64.14
U+3A5A.0	fTotalStrokes	 17
U+3A5A.0	fCangjie	QTBO
U+3A5A.0	fFourCorner	54030
U+3A5A.0	fCantonese	mung4
U+3A5A.0	fMandarin	meng2
U+3A5A.0	fEnglish	[1] collect (tax, grains) [2] draw together; contract, become less flagrant in behavior
U+3A5B.0	fUTF8	㩛
U+3A5B.0	fCNS11643	3-546A
U+3A5B.0	fGB	5-4C22
U+3A5B.0	fR/S	 64.14
U+3A5B.0	fTotalStrokes	 17
U+3A5B.0	fCangjie	QWJI
U+3A5B.0	fFourCorner	56000
U+3A5B.0	fHakka	 ton2
U+3A5B.0	fCantonese	tyun4; faa1
U+3A5B.0	fMandarin	tuan2; gua1; zhuan1; zhuan4
U+3A5B.0	fEnglish	same as &#x6476;
U+3A5C.0	fUTF8	㩜
U+3A5C.0	fCNS11643	3-5468
U+3A5C.0	fGB	5-4D25
U+3A5C.0	fR/S	 64.14
U+3A5C.0	fTotalStrokes	 17
U+3A5C.0	fCangjie	QSIT
U+3A5C.0	fFourCorner	58010
U+3A5C.0	fHakka	 lam1
U+3A5C.0	fCantonese	laam5; laam2 (also); gaam3
U+3A5C.0	fMandarin	lan3
U+3A5C.0	fEnglish	same as &#x652C;
U+3A5D.0	fUTF8	㩝
U+3A5D.0	fCNS11643	3-5463
U+3A5D.0	fGB	5-4D6C
U+3A5D.0	fR/S	 64.14
U+3A5D.0	fTotalStrokes	 17
U+3A5D.0	fCangjie	QYRO
U+3A5D.0	fFourCorner	50030
U+3A5D.0	fHakka	 hau2; ho2
U+3A5D.0	fCantonese	hou4
U+3A5D.0	fMandarin	hao2
U+3A5D.0	fEnglish	 compare; estimate; pull out
U+3A5E.0	fUTF8	㩞
U+3A5E.0	fCNS11643	3-5466
U+3A5E.0	fR/S	 64.14
U+3A5E.0	fTotalStrokes	 17
U+3A5E.0	fCangjie	QFFD
U+3A5E.0	fFourCorner	59090
U+3A5E.0	fCantonese	cak3; wing1
U+3A5E.0	fMandarin	ce4; ci4
U+3A5E.0	fEnglish	[1] (corrupted form) [2] back up; support [3] take; receive; fetch; obtain; take hold of
U+3A5F.0	fUTF8	㩟
U+3A5F.0	fGB	3-4B52
U+3A5F.0	fR/S	 64.14
U+3A5F.0	fTotalStrokes	 17
U+3A5F.0	fCangjie	QJTD
U+3A5F.0	fCantonese	zaai6; zai6
U+3A5F.0	fMandarin	zhai4
U+3A5F.0	fEnglish	sew some ornaments (button; decorations, etc.) on the clothes
U+3A60.0	fUTF8	㩠
U+3A60.0	fCNS11643	4-5B2D
U+3A60.0	fGB	3-4B55
U+3A60.0	fR/S	 64.15
U+3A60.0	fTotalStrokes	 18
U+3A60.0	fCangjie	QIPF
U+3A60.0	fFourCorner	50031
U+3A60.0	fCantonese	ou1; ngou1 (also)
U+3A60.0	fMandarin	ao2; piao3
U+3A60.0	fEnglish	[1] a fierce or bloody battle [2] untie; unbind [3] same as &#x634A;
U+3A61.0	fUTF8	㩡
U+3A61.0	fCNS11643	3-5769
U+3A61.0	fR/S	 64.15
U+3A61.0	fTotalStrokes	 18
U+3A61.0	fCangjie	QMRR
U+3A61.0	fFourCorner	51060
U+3A61.0	fCantonese	lo2
U+3A61.0	fMandarin	luo3
U+3A61.0	fEnglish	[1] shake; toss [2] wave; sway [3] row
U+3A62.0	fUTF8	㩢
U+3A62.0	fCNS11643	4-5B32
U+3A62.0	fGB	3-4B4E
U+3A62.0	fR/S	 64.15
U+3A62.0	fTotalStrokes	 18
U+3A62.0	fCangjie	QTWI
U+3A62.0	fFourCorner	54050
U+3A62.0	fHakka	  met8
U+3A62.0	fCantonese	mit6; maai4
U+3A62.0	fMandarin	mie4; mi2
U+3A62.0	fEnglish	 beat
U+3A63.0	fUTF8	㩣
U+3A63.0	fCNS11643	4-5B30
U+3A63.0	fGB	5-4D5A
U+3A63.0	fR/S	 64.15
U+3A63.0	fTotalStrokes	 18
U+3A63.0	fCangjie	QSMJ
U+3A63.0	fFourCorner	57050
U+3A63.0	fCantonese	fai1; buk1; faai1
U+3A63.0	fMandarin	hui1
U+3A63.0	fEnglish	same as &#x63EE;
U+3A64.0	fUTF8	㩤
U+3A64.0	fCNS11643	4-5B2E
U+3A64.0	fGB	3-4B53
U+3A64.0	fR/S	 64.15
U+3A64.0	fTotalStrokes	 18
U+3A64.0	fCangjie	QISK
U+3A64.0	fFourCorner	58040
U+3A64.0	fCantonese	fu1
U+3A64.0	fMandarin	fu1
U+3A64.0	fEnglish	[1] open; stretch; extend; display; raise [2] scatter; spread, stir; get excited
U+3A65.0	fUTF8	㩥
U+3A65.0	fCNS11643	3-576F
U+3A65.0	fR/S	 64.15
U+3A65.0	fTotalStrokes	 18
U+3A65.0	fCangjie	QJIM
U+3A65.0	fFourCorner	53050
U+3A65.0	fCantonese	cim1; cam2; sam1
U+3A65.0	fMandarin	san1; jian1; xian1
U+3A65.0	fEnglish	non-classical form of &#x6515;
U+3A66.0	fUTF8	㩦
U+3A66.0	fCNS11643	3-5771
U+3A66.0	fR/S	 64.15
U+3A66.0	fTotalStrokes	 18
U+3A66.0	fCangjie	QOGB
U+3A66.0	fFourCorner	50020
U+3A66.0	fHakka	 hi1
U+3A66.0	fCantonese	kwai4
U+3A66.0	fMandarin	xi1; xie2
U+3A66.0	fEnglish	same as &#x651C;
U+3A67.0	fUTF8	㩧
U+3A67.0	fCNS11643	4-5B31
U+3A67.0	fGB	3-4B54
U+3A67.0	fR/S	 64.15
U+3A67.0	fTotalStrokes	 18
U+3A67.0	fCangjie	QATE
U+3A67.0	fFourCorner	56030
U+3A67.0	fHakka	  pok7
U+3A67.0	fCantonese	bok1; bok6
U+3A67.0	fMandarin	bao3; bo2
U+3A67.0	fEnglish	 strike; beat
U+3A68.0	fUTF8	㩨
U+3A68.0	fCNS11643	3-5768
U+3A68.0	fGB	5-4D70
U+3A68.0	fR/S	 64.15
U+3A68.0	fTotalStrokes	 18
U+3A68.0	fCangjie	QQJP
U+3A68.0	fFourCorner	55030
U+3A68.0	fHakka	 fui5; fi5; wi5; wui5; vui5
U+3A68.0	fCantonese	wai6
U+3A68.0	fMandarin	zui4; hui4
U+3A68.0	fEnglish	 sweep away
U+3A69.0	fUTF8	㩩
U+3A69.0	fCNS11643	4-5B2F
U+3A69.0	fGB	3-4B42
U+3A69.0	fR/S	 64.15
U+3A69.0	fTotalStrokes	 18
U+3A69.0	fCangjie	QPEC
U+3A69.0	fFourCorner	51080
U+3A69.0	fCantonese	king2
U+3A69.0	fMandarin	qing3; qi3
U+3A69.0	fEnglish	 come to an end; terminate
U+3A6A.0	fUTF8	㩪
U+3A6A.0	fCNS11643	3-5772
U+3A6A.0	fR/S	 64.15
U+3A6A.0	fTotalStrokes	 18
U+3A6A.0	fCangjie	QVFC
U+3A6A.0	fFourCorner	51080
U+3A6A.0	fHakka	 get7; giet7
U+3A6A.0	fCantonese	kit3; git3
U+3A6A.0	fMandarin	xie2; jie2; xin4; ye2
U+3A6A.0	fEnglish	same as &#x64F7;
U+3A6B.0	fUTF8	㩫
U+3A6B.0	fCNS11643	F-5E73
U+3A6B.0	fGB	3-4A6E
U+3A6B.0	fR/S	 64.15
U+3A6B.0	fTotalStrokes	 18
U+3A6B.0	fCangjie	QHLC
U+3A6B.0	fCantonese	zat1
U+3A6C.0	fUTF8	㩬
U+3A6C.0	fCNS11643	3-5A3D
U+3A6C.0	fR/S	 64.16
U+3A6C.0	fTotalStrokes	 19
U+3A6C.0	fCangjie	QYMI
U+3A6C.0	fFourCorner	53050
U+3A6C.0	fCantonese	hei1; ngaat6
U+3A6C.0	fMandarin	xi1
U+3A6C.0	fEnglish	corrupted form of &#x256C32;
U+3A6D.0	fUTF8	㩭
U+3A6D.0	fCNS11643	3-5A3E
U+3A6D.0	fGB	5-4C58
U+3A6D.0	fR/S	 64.16
U+3A6D.0	fTotalStrokes	 19
U+3A6D.0	fCangjie	QHBD
U+3A6D.0	fFourCorner	57040
U+3A6D.0	fHakka	 hau5; hau6
U+3A6D.0	fCantonese	gaau2
U+3A6D.0	fMandarin	xiao3; bo2; jiao3; xiao4
U+3A6D.0	fEnglish	 stir about; confuse
U+3A6E.0	fUTF8	㩮
U+3A6E.0	fR/S	 64.17
U+3A6E.0	fTotalStrokes	 20
U+3A6E.0	fCangjie	QHCQ
U+3A6E.0	fCantonese	geoi2; kin4
U+3A6E.0	fMandarin	qian2; xian2
U+3A6E.0	fEnglish	[1] lift up or off; raise high [2] unveil
U+3A6F.0	fUTF8	㩯
U+3A6F.0	fCNS11643	4-6347
U+3A6F.0	fGB	5-4D7B
U+3A6F.0	fR/S	 64.17
U+3A6F.0	fTotalStrokes	 20
U+3A6F.0	fCangjie	QOKF
U+3A6F.0	fFourCorner	58090
U+3A6F.0	fHakka	 pan2
U+3A6F.0	fCantonese	pun4
U+3A6F.0	fMandarin	po2; ban3; pan2
U+3A6F.0	fEnglish	same as &#x642B;
U+3A70.0	fUTF8	㩰
U+3A70.0	fCNS11643	3-5C40
U+3A70.0	fGB	5-4D75
U+3A70.0	fR/S	 64.17
U+3A70.0	fTotalStrokes	 20
U+3A70.0	fCangjie	QTRD
U+3A70.0	fFourCorner	54090
U+3A70.0	fCantonese	ziu2
U+3A70.0	fMandarin	jiao3; qiao2; xiu3
U+3A70.0	fEnglish	[1] stab; cut [2] same as &#x6477;
U+3A71.0	fUTF8	㩱
U+3A71.0	fCNS11643	4-6346
U+3A71.0	fGB	3-4B5A
U+3A71.0	fR/S	 64.18; 64.17
U+3A71.0	fTotalStrokes	 20; 21
U+3A71.0	fCangjie	QBWI
U+3A71.0	fFourCorner	52040
U+3A71.0	fCantonese	zoek3
U+3A71.0	fMandarin	jue2
U+3A71.0	fEnglish	[1] select; choose; pick out [2] lift; carry on the shoulders -- of two or more men [3] wipe out [4] brush over lightly [5] carry; take or bring along at one's convenience [6] cut; par; trim; shave
U+3A72.0	fUTF8	㩲
U+3A72.0	fCNS11643	5-7024
U+3A72.0	fR/S	 64.18
U+3A72.0	fTotalStrokes	 21
U+3A72.0	fCangjie	QTRG
U+3A72.0	fFourCorner	54010
U+3A72.0	fCantonese	kyun4; fun1
U+3A72.0	fMandarin	kun3; quan2
U+3A72.0	fEnglish	[1] fist [2] clasp
U+3A73.0	fUTF8	㩳
U+3A73.0	fCNS11643	3-5E32
U+3A73.0	fGB	7-224C
U+3A73.0	fR/S	 64.18
U+3A73.0	fTotalStrokes	 21
U+3A73.0	fCangjie	QOGE
U+3A73.0	fFourCorner	50040
U+3A73.0	fHakka	  sung3
U+3A73.0	fCantonese	sung2
U+3A73.0	fMandarin	song3
U+3A73.0	fEnglish	[1] push; force with the hand [2] hold
U+3A74.0	fUTF8	㩴
U+3A74.0	fCNS11643	4-6640
U+3A74.0	fGB	3-4B5C
U+3A74.0	fR/S	 64.18
U+3A74.0	fTotalStrokes	 21
U+3A74.0	fCangjie	QBUG
U+3A74.0	fFourCorner	56010
U+3A74.0	fCantonese	guk1; guk6
U+3A74.0	fMandarin	ju1; ju2
U+3A74.0	fEnglish	 seize (as a bird of prey); seize and carry off; grasp
U+3A75.0	fUTF8	㩵
U+3A75.0	fCNS11643	3-6031
U+3A75.0	fGB	5-4D5F
U+3A75.0	fR/S	 64.20
U+3A75.0	fTotalStrokes	 23
U+3A75.0	fCangjie	QYBK
U+3A75.0	fFourCorner	53030
U+3A75.0	fCantonese	ngaat6; aat6 (also); mang1; ngok6
U+3A75.0	fMandarin	nie4; e4
U+3A75.0	fEnglish	 beat; strike; attack
U+3A76.0	fUTF8	㩶
U+3A76.0	fCNS11643	4-6A49
U+3A76.0	fGB	5-4D3E
U+3A76.0	fR/S	 64.20
U+3A76.0	fTotalStrokes	 23
U+3A76.0	fCangjie	QHMB
U+3A76.0	fFourCorner	58020
U+3A76.0	fCantonese	nip6; lip6 (also)
U+3A76.0	fMandarin	nie1; nie4; ning3
U+3A76.0	fEnglish	[1] hold; grasp; seize [2] retain
U+3A77.0	fUTF8	㩷
U+3A77.0	fCNS11643	4-6A48
U+3A77.0	fGB	3-4B4B
U+3A77.0	fR/S	 64.20
U+3A77.0	fTotalStrokes	 23
U+3A77.0	fCangjie	QJTF
U+3A77.0	fFourCorner	53020
U+3A77.0	fCantonese	hin1
U+3A78.0	fUTF8	㩸
U+3A78.0	fR/S	 64.22
U+3A78.0	fTotalStrokes	 25
U+3A78.0	fCangjie	QWWM
U+3A78.0	fFourCorner	5601
U+3A78.0	fCantonese	dip6
U+3A78.0	fMandarin	die2
U+3A78.0	fEnglish	variant of &#xpile on; fold up; draw together; collect;
U+3A79.0	fUTF8	㩹
U+3A79.0	fCNS11643	5-792A
U+3A79.0	fGB	3-4B4D
U+3A79.0	fR/S	 64.22
U+3A79.0	fTotalStrokes	 25
U+3A79.0	fCangjie	QWWM
U+3A79.0	fFourCorner	56010
U+3A79.0	fHakka	  tiap8
U+3A79.0	fCantonese	dip6
U+3A79.0	fMandarin	die2; zha2
U+3A79.0	fEnglish	 fold up
U+3A7A.0	fUTF8	㩺
U+3A7A.0	fCNS11643	4-265B
U+3A7A.0	fGB	3-4639
U+3A7A.0	fR/S	 65.4
U+3A7A.0	fTotalStrokes	 8
U+3A7A.0	fCangjie	PPJE
U+3A7A.0	fFourCorner	14740
U+3A7A.0	fCantonese	pei1
U+3A7A.0	fMandarin	pi1
U+3A7A.0	fEnglish	 a crack on the utensil
U+3A7B.0	fUTF8	㩻
U+3A7B.0	fCNS11643	4-2D40
U+3A7B.0	fGB	5-4640
U+3A7B.0	fR/S	 65.6
U+3A7B.0	fTotalStrokes	 10
U+3A7B.0	fCangjie	NUJE
U+3A7B.0	fFourCorner	24240
U+3A7B.0	fCantonese	kei1
U+3A7B.0	fMandarin	qi1; gui3
U+3A7B.0	fEnglish	[1] rugged mountain path [2] very skinny [3] pile on upon another; accumulation; accumulate
U+3A7C.0	fUTF8	㩼
U+3A7C.0	fCNS11643	4-2D3F
U+3A7C.0	fGB	5-4641
U+3A7C.0	fR/S	 65.6
U+3A7C.0	fTotalStrokes	 10
U+3A7C.0	fCangjie	NNJE
U+3A7C.0	fFourCorner	24240
U+3A7C.0	fHakka	 zhi1; zih1; zhi5
U+3A7C.0	fCantonese	zi1
U+3A7C.0	fMandarin	zhi1
U+3A7C.0	fEnglish	many
U+3A7D.0	fUTF8	㩽
U+3A7D.0	fCNS11643	4-3227
U+3A7D.0	fGB	5-4643
U+3A7D.0	fR/S	 65.7
U+3A7D.0	fTotalStrokes	 11
U+3A7D.0	fCangjie	DLJE
U+3A7D.0	fFourCorner	54940
U+3A7D.0	fCantonese	kei4; cik1; zi1
U+3A7D.0	fMandarin	qi2
U+3A7D.0	fEnglish	[1] branches to be a parasite (on) a tree [2] branches growing horizontally [3] variant of &#x52D1;
U+3A7E.0	fUTF8	㩾
U+3A7E.0	fCNS11643	4-3D54
U+3A7E.0	fGB	3-4369
U+3A7E.0	fR/S	 65.8; 65.9
U+3A7E.0	fTotalStrokes	 13; 12
U+3A7E.0	fCangjie	HMJE
U+3A7E.0	fFourCorner	24140
U+3A7E.0	fCantonese	zeoi1
U+3A7E.0	fMandarin	zhui1; chui2
U+3A7E.0	fEnglish	[1] uneven [2] hang down; let fall
U+3A7F.0	fUTF8	㩿
U+3A7F.0	fCNS11643	4-2440
U+3A7F.0	fGB	3-473B
U+3A7F.0	fR/S	 66.3
U+3A7F.0	fTotalStrokes	 7
U+3A7F.0	fCangjie	ONOK
U+3A7F.0	fFourCorner	88740
U+3A7F.0	fCantonese	fat1; gwaat6
U+3A7F.0	fMandarin	ku1
U+3A7F.0	fEnglish	[1] unstable [2] dull [3] rough; coarse
U+3A80.0	fUTF8	㪀
U+3A80.0	fCNS11643	3-263A
U+3A80.0	fR/S	 66.3
U+3A80.0	fTotalStrokes	 7
U+3A80.0	fCangjie	MDOK
U+3A80.0	fFourCorner	18440
U+3A80.0	fCantonese	jyu1; zyu1
U+3A80.0	fMandarin	yu2
U+3A80.0	fEnglish	[1] enter [2] go ahead; proceed; advance [3] improve [4] offer
U+3A81.0	fUTF8	㪁
U+3A81.0	fCNS11643	4-265E
U+3A81.0	fGB	3-467B
U+3A81.0	fR/S	 66.4
U+3A81.0	fTotalStrokes	 8
U+3A81.0	fCangjie	ONYE
U+3A81.0	fFourCorner	88240
U+3A81.0	fCantonese	kam4
U+3A81.0	fMandarin	qin2
U+3A81.0	fEnglish	[1] hold; grasp [2] irregular; uneven [3] a precipice beneath an overhanging cliff [4] same as &#x62D1;
U+3A82.0	fUTF8	㪂
U+3A82.0	fCNS11643	4-265F
U+3A82.0	fGB	5-4E47
U+3A82.0	fR/S	 66.4
U+3A82.0	fTotalStrokes	 8
U+3A82.0	fCangjie	ONYE
U+3A82.0	fFourCorner	81747
U+3A82.0	fCantonese	fat1; faat1; gwaat6
U+3A82.0	fMandarin	ku1
U+3A82.0	fEnglish	same as &#x3A7F;
U+3A83.0	fUTF8	㪃
U+3A83.0	fGB	3-467C
U+3A83.0	fR/S	 66.5
U+3A83.0	fTotalStrokes	 9
U+3A83.0	fCangjie	MRYE
U+3A83.0	fCantonese	ho2; ho1
U+3A83.0	fMandarin	ke3; ke4
U+3A83.0	fEnglish	thump; beat; strike; attack
U+3A84.0	fUTF8	㪄
U+3A84.0	fCNS11643	4-295D
U+3A84.0	fGB	3-473C
U+3A84.0	fR/S	 66.5
U+3A84.0	fTotalStrokes	 9
U+3A84.0	fCangjie	LNOK
U+3A84.0	fFourCorner	58040
U+3A84.0	fCantonese	fat6; faat1
U+3A84.0	fMandarin	fu2
U+3A84.0	fEnglish	[1] broken; break [2] ruined [3] reason; cause; right [4] principle; theory [5] arrange; repair [6] regulate; operate; govern
U+3A85.0	fUTF8	㪅
U+3A85.0	fCNS11643	3-2C75
U+3A85.0	fR/S	 66.5
U+3A85.0	fTotalStrokes	 9
U+3A85.0	fCangjie	MBYE
U+3A85.0	fFourCorner	10400
U+3A85.0	fHakka	 gang1
U+3A85.0	fCantonese	gang1; gaang1; sing3
U+3A85.0	fMandarin	jing1; geng4
U+3A85.0	fEnglish	same as &#x66F4;
U+3A86.0	fUTF8	㪆
U+3A86.0	fGB	3-467E
U+3A86.0	fR/S	 66.5
U+3A86.0	fTotalStrokes	 9
U+3A86.0	fCangjie	HIYE
U+3A86.0	fCantonese	dai2; daai2
U+3A86.0	fMandarin	di3
U+3A86.0	fEnglish	hidden; mysterious; secret; to conceal
U+3A87.0	fUTF8	㪇
U+3A87.0	fCNS11643	3-313E
U+3A87.0	fR/S	 66.6
U+3A87.0	fTotalStrokes	 10
U+3A87.0	fCangjie	HUOK
U+3A87.0	fFourCorner	28240
U+3A87.0	fCantonese	sin3
U+3A87.0	fMandarin	xian4
U+3A87.0	fEnglish	[1] disseminate; spread [2] the weak sound of a Chinese musical instrument with seven or five strings
U+3A88.0	fUTF8	㪈
U+3A88.0	fCNS11643	4-2D42
U+3A88.0	fGB	3-4721
U+3A88.0	fR/S	 66.6
U+3A88.0	fTotalStrokes	 10
U+3A88.0	fCangjie	GGYE
U+3A88.0	fFourCorner	48140
U+3A88.0	fCantonese	gwai3
U+3A88.0	fMandarin	gui4
U+3A88.0	fEnglish	[1] let go [2] put [3] raid; aggress, encroach up [4] a bad year; a year of famine or disaster
U+3A89.0	fUTF8	㪉
U+3A89.0	fCNS11643	6-382B
U+3A89.0	fR/S	 66.6
U+3A89.0	fTotalStrokes	 10; 9
U+3A89.0	fCangjie	OROK
U+3A89.0	fFourCorner	88640
U+3A89.0	fCantonese	hap6
U+3A89.0	fMandarin	he2; ge2; xia2
U+3A89.0	fEnglish	same as &#x6546;
U+3A8A.0	fUTF8	㪊
U+3A8A.0	fGB	3-4726
U+3A8A.0	fR/S	 66.7
U+3A8A.0	fTotalStrokes	 11
U+3A8A.0	fCangjie	SRYE
U+3A8A.0	fFourCorner	1864
U+3A8A.0	fCantonese	kwan4; kwaan4
U+3A8A.0	fMandarin	qun2
U+3A8A.0	fEnglish	infested with bandits and robbers
U+3A8B.0	fUTF8	㪋
U+3A8B.0	fCNS11643	3-3633
U+3A8B.0	fGB	5-4876
U+3A8B.0	fR/S	 66.7
U+3A8B.0	fTotalStrokes	 11
U+3A8B.0	fCangjie	AJYE
U+3A8B.0	fFourCorner	61440
U+3A8B.0	fCantonese	hon6
U+3A8B.0	fMandarin	han4
U+3A8B.0	fEnglish	[1] same as &#x625E;, &#x634D; [2] same as &#x3A83;
U+3A8C.0	fUTF8	㪌
U+3A8C.0	fCNS11643	4-3228
U+3A8C.0	fGB	5-4878
U+3A8C.0	fR/S	 66.7
U+3A8C.0	fTotalStrokes	 11
U+3A8C.0	fCangjie	NBYE
U+3A8C.0	fFourCorner	18240
U+3A8C.0	fCantonese	tung2; tong2
U+3A8C.0	fMandarin	tong3; yu2; yu3
U+3A8C.0	fEnglish	lead on; advance; strike against; break through
U+3A8D.0	fUTF8	㪍
U+3A8D.0	fCNS11643	3-3635
U+3A8D.0	fGB	5-493B
U+3A8D.0	fR/S	 66.7
U+3A8D.0	fTotalStrokes	 11
U+3A8D.0	fCangjie	JDOK
U+3A8D.0	fFourCorner	48440
U+3A8D.0	fHakka	 put8
U+3A8D.0	fCantonese	but6; bui6
U+3A8D.0	fMandarin	bo1; bo2
U+3A8D.0	fEnglish	[1] same as &#x52C3; [2] interchangeable with &#x6096;
U+3A8E.0	fUTF8	㪎
U+3A8E.0	fCNS11643	4-322C
U+3A8E.0	fGB	3-4722
U+3A8E.0	fR/S	 66.7
U+3A8E.0	fTotalStrokes	 11
U+3A8E.0	fCangjie	KOYE
U+3A8E.0	fFourCorner	48040
U+3A8E.0	fCantonese	sim2
U+3A8E.0	fMandarin	shan3
U+3A8E.0	fEnglish	[1] uncertain; not yet settled [2] irregular [3] raise one's hands [4] to exhaust; to use up; to complete; to finish, all; entirely; totally; completely [5] all; entirely; totally; completely
U+3A8F.0	fUTF8	㪏
U+3A8F.0	fCNS11643	4-3779
U+3A8F.0	fGB	3-472B
U+3A8F.0	fR/S	 66.8
U+3A8F.0	fTotalStrokes	 12
U+3A8F.0	fCangjie	HJYE
U+3A8F.0	fFourCorner	21440
U+3A8F.0	fCantonese	bai2; bei2
U+3A8F.0	fMandarin	pi3; bi3
U+3A8F.0	fEnglish	interchangeable with &#x636D;
U+3A90.0	fUTF8	㪐
U+3A90.0	fGB	3-472D
U+3A90.0	fR/S	 66.8
U+3A90.0	fTotalStrokes	 12
U+3A90.0	fCangjie	IKYE
U+3A90.0	fFourCorner	7824
U+3A90.0	fCantonese	leot6; luk6
U+3A90.0	fMandarin	lu4
U+3A90.0	fEnglish	unstable, dull, rough; coarse
U+3A91.0	fUTF8	㪑
U+3A91.0	fCNS11643	4-3775
U+3A91.0	fGB	3-4728
U+3A91.0	fR/S	 66.8
U+3A91.0	fTotalStrokes	 12
U+3A91.0	fCangjie	KUYE
U+3A91.0	fFourCorner	41740
U+3A91.0	fCantonese	jip3; zip6
U+3A91.0	fMandarin	yan2; ye4
U+3A91.0	fEnglish	 follow; attach; connected
U+3A92.0	fUTF8	㪒
U+3A92.0	fCNS11643	4-3778
U+3A92.0	fGB	3-472A
U+3A92.0	fR/S	 66.8
U+3A92.0	fTotalStrokes	 12
U+3A92.0	fCangjie	HUYE
U+3A92.0	fFourCorner	71240
U+3A92.0	fCantonese	ngai4; ai4 (also); ngaai4
U+3A92.0	fMandarin	yi4; ni2
U+3A92.0	fEnglish	 ruin; destroy; break down
U+3A93.0	fUTF8	㪓
U+3A93.0	fCNS11643	4-3777
U+3A93.0	fGB	5-487D
U+3A93.0	fR/S	 66.8
U+3A93.0	fTotalStrokes	 12
U+3A93.0	fCangjie	HPYE
U+3A93.0	fFourCorner	21140
U+3A93.0	fCantonese	ceoi4; cai4; zak3
U+3A93.0	fMandarin	chai2; chuai2
U+3A93.0	fEnglish	[1] disaster; calamity [2] evil; wicked; mean
U+3A94.0	fUTF8	㪔
U+3A94.0	fGB	5-4879
U+3A94.0	fR/S	 66.8
U+3A94.0	fTotalStrokes	 12
U+3A94.0	fCangjie	DDYE
U+3A94.0	fCantonese	saan3; san3
U+3A94.0	fMandarin	san4; tan2
U+3A94.0	fEnglish	same as &#x6563;
U+3A95.0	fUTF8	㪕
U+3A95.0	fCNS11643	4-3776
U+3A95.0	fGB	3-4729
U+3A95.0	fR/S	 66.8
U+3A95.0	fTotalStrokes	 12
U+3A95.0	fCangjie	YJYE
U+3A95.0	fFourCorner	21440
U+3A95.0	fCantonese	diu6
U+3A95.0	fMandarin	xiao3; diao4
U+3A95.0	fEnglish	[1] beat; pound; strike; rush on; pat; dash; smash [2] take by force; wrest from
U+3A96.0	fUTF8	㪖
U+3A96.0	fCNS11643	5-387C
U+3A96.0	fGB	3-472F
U+3A96.0	fR/S	 66.8
U+3A96.0	fTotalStrokes	 12
U+3A96.0	fCangjie	VEYE
U+3A96.0	fFourCorner	21140
U+3A96.0	fCantonese	luk6
U+3A96.0	fMandarin	lu4
U+3A96.0	fEnglish	[1] sound of beating [2] strike [3] sound of an object dropping into water, etc.
U+3A97.0	fUTF8	㪗
U+3A97.0	fGB	3-472C
U+3A97.0	fR/S	 66.8
U+3A97.0	fTotalStrokes	 12
U+3A97.0	fCangjie	YRYE
U+3A97.0	fCantonese	tau2; taau2
U+3A97.0	fMandarin	tou3
U+3A97.0	fEnglish	[1] open [2] unroll [3] rest; catch one's breath
U+3A98.0	fUTF8	㪘
U+3A98.0	fR/S	 66.8
U+3A98.0	fTotalStrokes	 12
U+3A98.0	fCangjie	OOYE
U+3A98.0	fCantonese	lim5; ling6
U+3A98.0	fMandarin	lian3
U+3A98.0	fEnglish	abbreviated form of &#x6582;
U+3A99.0	fUTF8	㪙
U+3A99.0	fGB	3-4741
U+3A99.0	fR/S	 66.8
U+3A99.0	fTotalStrokes	 12
U+3A99.0	fCangjie	WDOK
U+3A99.0	fCantonese	fo2
U+3A99.0	fMandarin	ke3; ke4; kuo4
U+3A99.0	fEnglish	go to the very source; search into carefully
U+3A9A.0	fUTF8	㪚
U+3A9A.0	fCNS11643	3-3B77
U+3A9A.0	fR/S	 66.8
U+3A9A.0	fTotalStrokes	 12
U+3A9A.0	fCangjie	TAOK
U+3A9A.0	fFourCorner	48640
U+3A9A.0	fCantonese	saan3; saan2; san3
U+3A9A.0	fMandarin	san4
U+3A9A.0	fEnglish	same as &#x6563;
U+3A9B.0	fUTF8	㪛
U+3A9B.0	fGB	3-4730
U+3A9B.0	fR/S	 66.9
U+3A9B.0	fTotalStrokes	 13
U+3A9B.0	fCangjie	TVYE
U+3A9B.0	fCantonese	zam2; zaam1
U+3A9B.0	fMandarin	zhen3
U+3A9B.0	fEnglish	[1] pound rocks to pieces [2] same as &#x628C;
U+3A9C.0	fUTF8	㪜
U+3A9C.0	fCNS11643	4-3D5C
U+3A9C.0	fGB	5-4926
U+3A9C.0	fR/S	 66.9
U+3A9C.0	fTotalStrokes	 13
U+3A9C.0	fCangjie	UBYE
U+3A9C.0	fFourCorner	28240
U+3A9C.0	fCantonese	ceoi2
U+3A9C.0	fMandarin	chuai3
U+3A9C.0	fEnglish	[1] same as &#x63E3; [2] same as &#x6376;
U+3A9D.0	fUTF8	㪝
U+3A9D.0	fCNS11643	4-3D59
U+3A9D.0	fGB	3-4731
U+3A9D.0	fR/S	 66.9
U+3A9D.0	fTotalStrokes	 13
U+3A9D.0	fCangjie	DWYE
U+3A9D.0	fFourCorner	51940
U+3A9D.0	fHakka	  lien5
U+3A9D.0	fCantonese	lin6
U+3A9D.0	fMandarin	lian4
U+3A9D.0	fEnglish	 pound; hammer
U+3A9E.0	fUTF8	㪞
U+3A9E.0	fCNS11643	4-3D5B
U+3A9E.0	fGB	5-4925
U+3A9E.0	fR/S	 66.9
U+3A9E.0	fTotalStrokes	 13
U+3A9E.0	fCangjie	AUYE
U+3A9E.0	fFourCorner	61640
U+3A9E.0	fCantonese	mou6
U+3A9E.0	fMandarin	mao4
U+3A9E.0	fEnglish	[1] be held by the hand [2] resist; oppose
U+3A9F.0	fUTF8	㪟
U+3A9F.0	fCNS11643	3-414C
U+3A9F.0	fR/S	 66.9
U+3A9F.0	fTotalStrokes	 13
U+3A9F.0	fCangjie	YAOK
U+3A9F.0	fFourCorner	08640
U+3A9F.0	fHakka	 dun1
U+3A9F.0	fCantonese	deon1; deoi1; deoi3; tyun4; deon6; deoi6
U+3A9F.0	fMandarin	dun1; chou2; diao1; dui1; dun3; dun4; tuan2; zhun3
U+3A9F.0	fEnglish	same as &#x6566;
U+3AA0.0	fUTF8	㪠
U+3AA0.0	fCNS11643	4-4366
U+3AA0.0	fGB	3-4734
U+3AA0.0	fR/S	 66.10
U+3AA0.0	fTotalStrokes	 14
U+3AA0.0	fCangjie	TCYE
U+3AA0.0	fFourCorner	81340
U+3AA0.0	fHakka	 kiam1; kam1
U+3AA0.0	fCantonese	zaam1; him3
U+3AA0.0	fMandarin	qian4
U+3AA0.0	fEnglish	 suit; things that match
U+3AA1.0	fUTF8	㪡
U+3AA1.0	fCNS11643	5-4630
U+3AA1.0	fGB	3-4735
U+3AA1.0	fR/S	 66.10
U+3AA1.0	fTotalStrokes	 14
U+3AA1.0	fCangjie	JRYE
U+3AA1.0	fFourCorner	38640
U+3AA1.0	fCantonese	koi3; hot3
U+3AA1.0	fMandarin	ke4; ke3
U+3AA1.0	fEnglish	[1] quell; punish by force of arms, enemy; foe [2] oppose; resist
U+3AA2.0	fUTF8	㪢
U+3AA2.0	fCNS11643	5-462E
U+3AA2.0	fGB	3-4733
U+3AA2.0	fR/S	 66.10; 66.9
U+3AA2.0	fTotalStrokes	 14; 13
U+3AA2.0	fCangjie	HEYE
U+3AA2.0	fFourCorner	78440
U+3AA2.0	fCantonese	saau2
U+3AA2.0	fMandarin	shao3
U+3AA2.0	fEnglish	same as &#x22F90;
U+3AA3.0	fUTF8	㪣
U+3AA3.0	fCNS11643	4-4369
U+3AA3.0	fGB	3-7B6F
U+3AA3.0	fR/S	 66.10
U+3AA3.0	fTotalStrokes	 14
U+3AA3.0	fCangjie	YBOK
U+3AA3.0	fFourCorner	08240
U+3AA3.0	fCantonese	hiu1; hau1; kiu3
U+3AA3.0	fMandarin	qiao1; qiao4
U+3AA3.0	fEnglish	[1] a place in Han dynasty [2] same as &#x6572;
U+3AA4.0	fUTF8	㪤
U+3AA4.0	fCNS11643	4-4A2B
U+3AA4.0	fGB	3-473E
U+3AA4.0	fR/S	 66.11
U+3AA4.0	fTotalStrokes	 15
U+3AA4.0	fCangjie	WJOK
U+3AA4.0	fFourCorner	68540
U+3AA4.0	fCantonese	bat1; baat1; but6
U+3AA4.0	fMandarin	bi4
U+3AA4.0	fEnglish	 finish; concluded
U+3AA5.0	fUTF8	㪥
U+3AA5.0	fCNS11643	4-4A2A
U+3AA5.0	fGB	3-4736
U+3AA5.0	fR/S	 66.11
U+3AA5.0	fTotalStrokes	 15
U+3AA5.0	fCangjie	YMYE
U+3AA5.0	fFourCorner	21240
U+3AA5.0	fHakka	 za1
U+3AA5.0	fCantonese	zaa1
U+3AA5.0	fMandarin	zha1
U+3AA5.0	fEnglish	pimples; blotches on the skin
U+3AA6.0	fUTF8	㪦
U+3AA6.0	fCNS11643	4-4A29
U+3AA6.0	fGB	3-4747
U+3AA6.0	fR/S	 66.11
U+3AA6.0	fTotalStrokes	 15
U+3AA6.0	fCangjie	JCHE
U+3AA6.0	fFourCorner	38840
U+3AA6.0	fCantonese	jan6; zaan5; zaan6
U+3AA6.0	fMandarin	yin4
U+3AA6.0	fEnglish	[1] beat; pound; ram down [2] attack
U+3AA7.0	fUTF8	㪧
U+3AA7.0	fR/S	 66.12
U+3AA7.0	fTotalStrokes	 16
U+3AA7.0	fCangjie	OMYE
U+3AA7.0	fCantonese	jap1; kaap1
U+3AA7.0	fEnglish	beat; strike; attack
U+3AA8.0	fUTF8	㪨
U+3AA8.0	fCNS11643	4-5049
U+3AA8.0	fGB	5-492D
U+3AA8.0	fR/S	 66.12
U+3AA8.0	fTotalStrokes	 16
U+3AA8.0	fCangjie	TRYE
U+3AA8.0	fFourCorner	81640
U+3AA8.0	fCantonese	sin6; saam1
U+3AA8.0	fMandarin	shan4
U+3AA8.0	fEnglish	[1] mend; repair [2] copy; write out
U+3AA9.0	fUTF8	㪩
U+3AA9.0	fCNS11643	4-504C
U+3AA9.0	fGB	3-472E
U+3AA9.0	fR/S	 66.12
U+3AA9.0	fTotalStrokes	 16
U+3AA9.0	fCangjie	LXYE
U+3AA9.0	fFourCorner	51240
U+3AA9.0	fCantonese	sok1; duk6; suk1
U+3AA9.0	fMandarin	su4
U+3AA9.0	fEnglish	[1] strike; beat [2] the sound of beating
U+3AAA.0	fUTF8	㪪
U+3AAA.0	fGB	3-4737
U+3AAA.0	fR/S	 66.12
U+3AAA.0	fTotalStrokes	 16
U+3AAA.0	fCangjie	MYYE
U+3AAA.0	fCantonese	saap3; saa1
U+3AAA.0	fMandarin	sa4; xi3
U+3AAA.0	fEnglish	[1] use a wooden pole to support something [2] raise, begin; start
U+3AAB.0	fUTF8	㪫
U+3AAB.0	fCNS11643	3-5773
U+3AAB.0	fR/S	 66.14
U+3AAB.0	fTotalStrokes	 18
U+3AAB.0	fCangjie	YUYE
U+3AAB.0	fFourCorner	28640
U+3AAB.0	fHakka	 jui5
U+3AAB.0	fCantonese	jeoi6; zeoi6
U+3AAB.0	fMandarin	rui4
U+3AAB.0	fEnglish	corrupted form of &#x53E1;
U+3AAC.0	fUTF8	㪬
U+3AAC.0	fCNS11643	4-5B3A
U+3AAC.0	fGB	3-4739
U+3AAC.0	fR/S	 66.14
U+3AAC.0	fTotalStrokes	 18
U+3AAC.0	fCangjie	SGYE
U+3AAC.0	fFourCorner	18240
U+3AAC.0	fHakka	 cok7
U+3AAC.0	fCantonese	coek3; zuk6
U+3AAC.0	fMandarin	zhuo2
U+3AAC.0	fEnglish	[1] pierce [2] dart, as pain
U+3AAD.0	fUTF8	㪭
U+3AAD.0	fCNS11643	4-634C
U+3AAD.0	fGB	3-467D
U+3AAD.0	fR/S	 66.16
U+3AAD.0	fTotalStrokes	 20
U+3AAD.0	fCangjie	YTYE
U+3AAD.0	fFourCorner	21240
U+3AAD.0	fCantonese	lou4; lou6
U+3AAD.0	fMandarin	lu2
U+3AAD.0	fEnglish	[1] draw together; contract to fold [2] collect; gather [3] hide; conceal
U+3AAE.0	fUTF8	㪮
U+3AAE.0	fCNS11643	4-6643
U+3AAE.0	fGB	3-473A
U+3AAE.0	fR/S	 66.17
U+3AAE.0	fTotalStrokes	 21
U+3AAE.0	fCangjie	MRYE
U+3AAE.0	fFourCorner	11640
U+3AAE.0	fCantonese	ling4
U+3AAE.0	fMandarin	ling2
U+3AAE.0	fEnglish	beat; attack; strike
U+3AAF.0	fUTF8	㪯
U+3AAF.0	fCNS11643	3-263B
U+3AAF.0	fR/S	 67.3
U+3AAF.0	fTotalStrokes	 7
U+3AAF.0	fCangjie	YKQ
U+3AAF.0	fFourCorner	00500
U+3AAF.0	fHakka	 gi3
U+3AAF.0	fCantonese	geoi2
U+3AAF.0	fMandarin	ju3; cha2; qu2
U+3AAF.0	fEnglish	non-classical form of &#x8209;
U+3AB0.0	fUTF8	㪰
U+3AB0.0	fCNS11643	6-382C
U+3AB0.0	fR/S	 67.6
U+3AB0.0	fTotalStrokes	 10; 9
U+3AB0.0	fCangjie	YKFD
U+3AB0.0	fFourCorner	00900
U+3AB0.0	fHakka	 zai1
U+3AB0.0	fCantonese	zaai1; zai1
U+3AB0.0	fMandarin	zhai1; zi1
U+3AB0.0	fEnglish	same as &#x9F4B;
U+3AB1.0	fUTF8	㪱
U+3AB1.0	fCNS11643	3-414F
U+3AB1.0	fR/S	 67.9
U+3AB1.0	fTotalStrokes	 13
U+3AB1.0	fCangjie	YKNBK
U+3AB1.0	fFourCorner	07430
U+3AB1.0	fHakka	  fon5; fan3
U+3AB1.0	fCantonese	wun6; kut3
U+3AB1.0	fMandarin	huan4
U+3AB1.0	fEnglish	elegant; variegated
U+3AB2.0	fUTF8	㪲
U+3AB2.0	fR/S	 68.2
U+3AB2.0	fTotalStrokes	 6
U+3AB2.0	fCangjie	YJN
U+3AB2.0	fCantonese	dau2
U+3AB2.0	fEnglish	[Korean character]
U+3AB3.0	fUTF8	㪳
U+3AB3.0	fR/S	 68.2
U+3AB3.0	fTotalStrokes	 6
U+3AB3.0	fCangjie	YJR
U+3AB3.0	fCantonese	dau2
U+3AB3.0	fEnglish	[Korean character]
U+3AB4.0	fUTF8	㪴
U+3AB4.0	fCNS11643	3-293A
U+3AB4.0	fR/S	 68.4
U+3AB4.0	fTotalStrokes	 8
U+3AB4.0	fCangjie	MUYJ
U+3AB4.0	fFourCorner	14200
U+3AB4.0	fCantonese	git3; gat3
U+3AB4.0	fMandarin	jia2
U+3AB4.0	fEnglish	[1] measure [2] estimate
U+3AB5.0	fUTF8	㪵
U+3AB5.0	fCNS11643	4-2961
U+3AB5.0	fGB	3-5063
U+3AB5.0	fR/S	 68.5
U+3AB5.0	fTotalStrokes	 9
U+3AB5.0	fCangjie	FQYJ
U+3AB5.0	fFourCorner	94500
U+3AB5.0	fCantonese	bun3
U+3AB5.0	fMandarin	ban4
U+3AB5.0	fEnglish	half of a unit of measurement used in ancient times
U+3AB6.0	fUTF8	㪶
U+3AB6.0	fCNS11643	3-3141
U+3AB6.0	fR/S	 68.6
U+3AB6.0	fTotalStrokes	 10
U+3AB6.0	fCangjie	MAYJ
U+3AB6.0	fFourCorner	14600
U+3AB6.0	fHakka	 fuk8
U+3AB6.0	fCantonese	huk6
U+3AB6.0	fMandarin	hu2
U+3AB6.0	fEnglish	same as &#x659B;
U+3AB7.0	fUTF8	㪷
U+3AB7.0	fCNS11643	3-392B
U+3AB7.0	fGB	5-6D73
U+3AB7.0	fR/S	 68.7
U+3AB7.0	fTotalStrokes	 11
U+3AB7.0	fCangjie	MTYJ
U+3AB7.0	fFourCorner	14100
U+3AB7.0	fHakka	 deu3
U+3AB7.0	fCantonese	dau2; dau3
U+3AB7.0	fMandarin	dou3; dou4
U+3AB8.0	fUTF8	㪸
U+3AB8.0	fCNS11643	3-3B78
U+3AB8.0	fR/S	 68.8
U+3AB8.0	fTotalStrokes	 12
U+3AB8.0	fCangjie	TCYJ
U+3AB8.0	fFourCorner	44800
U+3AB8.0	fHakka	 zhim1
U+3AB8.0	fCantonese	zam1; caam2; zaam1
U+3AB8.0	fMandarin	zhen1
U+3AB8.0	fEnglish	same as &#x659F;
U+3AB9.0	fUTF8	㪹
U+3AB9.0	fCNS11643	4-4A2E
U+3AB9.0	fGB	3-5066
U+3AB9.0	fR/S	 68.11
U+3AB9.0	fTotalStrokes	 15
U+3AB9.0	fCangjie	LVYJ
U+3AB9.0	fFourCorner	54400
U+3AB9.0	fHakka	 leu3
U+3AB9.0	fCantonese	lau5; laau2
U+3AB9.0	fMandarin	dou3; lou3; lu4
U+3AB9.0	fEnglish	 plunder
U+3ABA.0	fUTF8	㪺
U+3ABA.0	fCNS11643	4-563E
U+3ABA.0	fGB	3-5068
U+3ABA.0	fR/S	 68.13
U+3ABA.0	fTotalStrokes	 17; 16
U+3ABA.0	fCangjie	KUYJ
U+3ABA.0	fFourCorner	44000
U+3ABA.0	fCantonese	keoi1
U+3ABA.0	fMandarin	ju1
U+3ABA.0	fEnglish	[1] bale out [2] decant liquids; to pour out liquor [3] a feast [4] a kind of utensil used to ladle out water
U+3ABB.0	fUTF8	㪻
U+3ABB.0	fCNS11643	4-6A4B
U+3ABB.0	fGB	3-5064
U+3ABB.0	fR/S	 68.19
U+3ABB.0	fTotalStrokes	 23
U+3ABB.0	fCangjie	VFYJ
U+3ABB.0	fFourCorner	22400
U+3ABB.0	fCantonese	gyun3; hyun3
U+3ABB.0	fMandarin	fan4; juan4
U+3ABB.0	fEnglish	[1] a kind of utensil used to ladle out wine [2] a container that gives a measurement of rice
U+3ABC.0	fUTF8	㪼
U+3ABC.0	fCNS11643	4-2962
U+3ABC.0	fGB	3-4C34
U+3ABC.0	fR/S	 69.5
U+3ABC.0	fTotalStrokes	 9
U+3ABC.0	fCangjie	MRHML
U+3ABC.0	fFourCorner	12620
U+3ABC.0	fCantonese	ho2
U+3ABC.0	fMandarin	ke3
U+3ABC.0	fEnglish	[1] pull; dray [2] hinder
U+3ABD.0	fUTF8	㪽
U+3ABD.0	fR/S	 69.5
U+3ABD.0	fTotalStrokes	 9
U+3ABD.0	fCangjie	MAHL
U+3ABD.0	fFourCorner	1062
U+3ABD.0	fCantonese	so2
U+3ABD.0	fMandarin	suo3; suo4
U+3ABE.0	fUTF8	㪾
U+3ABE.0	fCNS11643	4-2D49
U+3ABE.0	fGB	5-4E60
U+3ABE.0	fR/S	 69.6
U+3ABE.0	fTotalStrokes	 10
U+3ABE.0	fCangjie	HRHML
U+3ABE.0	fFourCorner	22620
U+3ABE.0	fCantonese	lok6; lok3
U+3ABE.0	fMandarin	luo4; ge2
U+3ABE.0	fEnglish	[1] cut the flesh from the bone; scrape off [2] pick out [3] get rid of [4] arrest; catch; seize [5] struggle; fight; conflict; vie; compete
U+3ABF.0	fUTF8	㪿
U+3ABF.0	fCNS11643	6-3830
U+3ABF.0	fR/S	 69.6
U+3ABF.0	fTotalStrokes	 10; 9
U+3ABF.0	fCangjie	UUHML
U+3ABF.0	fFourCorner	22420
U+3ABF.0	fHakka	 zhet7
U+3ABF.0	fCantonese	zit3
U+3ABF.0	fMandarin	zhe2; ti2
U+3ABF.0	fEnglish	standard form of &#x6298;
U+3AC0.0	fUTF8	㫀
U+3AC0.0	fCNS11643	6-4961
U+3AC0.0	fR/S	 69.8
U+3AC0.0	fTotalStrokes	 12
U+3AC0.0	fCangjie	AAHML
U+3AC0.0	fFourCorner	62620
U+3AC0.0	fHakka	 din3
U+3AC0.0	fCantonese	ding2
U+3AC0.0	fMandarin	ding3
U+3AC0.0	fEnglish	same as &#x9F0E;
U+3AC1.0	fUTF8	㫁
U+3AC1.0	fCNS11643	6-5D51
U+3AC1.0	fR/S	 69.10
U+3AC1.0	fTotalStrokes	 14
U+3AC1.0	fCangjie	VUHML
U+3AC1.0	fFourCorner	22720
U+3AC1.0	fHakka	 don5; ton1
U+3AC1.0	fCantonese	dyun6; tyun5
U+3AC1.0	fMandarin	duan4
U+3AC1.0	fEnglish	non-classical form of &#x65B7;
U+3AC2.0	fUTF8	㫂
U+3AC2.0	fCNS11643	5-4D4B
U+3AC2.0	fR/S	 69.11
U+3AC2.0	fTotalStrokes	 15
U+3AC2.0	fCangjie	IFHML
U+3AC2.0	fFourCorner	02220
U+3AC2.0	fCantonese	zyu3
U+3AC2.0	fMandarin	zhu4
U+3AC2.0	fEnglish	 chop; cut (wood)
U+3AC3.0	fUTF8	㫃
U+3AC3.0	fCNS11643	3-2371
U+3AC3.0	fGB	5-507C
U+3AC3.0	fR/S	 70.2
U+3AC3.0	fTotalStrokes	 6
U+3AC3.0	fCangjie	YSO
U+3AC3.0	fFourCorner	08200
U+3AC3.0	fCantonese	jin2; zin2
U+3AC3.0	fMandarin	yan3
U+3AC3.0	fEnglish	[1] flags flying [2] long bands or ribbons attached to flags, streamers, etc. [3] ancient form of &#x5043;
U+3AC4.0	fUTF8	㫄
U+3AC4.0	fCNS11643	6-2B70
U+3AC4.0	fR/S	 70.4
U+3AC4.0	fTotalStrokes	 8
U+3AC4.0	fCangjie	MMPS
U+3AC4.0	fFourCorner	10220
U+3AC4.0	fHakka	 pong2
U+3AC4.0	fCantonese	pong4; bong6
U+3AC4.0	fMandarin	pang2; bang1; beng1; jing1; pang1; peng2
U+3AC4.0	fEnglish	ancient form of &#x65C1;
U+3AC5.0	fUTF8	㫅
U+3AC5.0	fCNS11643	6-3834
U+3AC5.0	fR/S	 70.6
U+3AC5.0	fTotalStrokes	 10; 9
U+3AC5.0	fCangjie	YSOML
U+3AC5.0	fFourCorner	08220
U+3AC5.0	fHakka	 ki2
U+3AC5.0	fCantonese	kei4; si4
U+3AC5.0	fMandarin	qi2; cha2; shi2
U+3AC5.0	fEnglish	same as &#x65D7;
U+3AC6.0	fUTF8	㫆
U+3AC6.0	fR/S	 70.5
U+3AC6.0	fTotalStrokes	 9
U+3AC6.0	fCangjie	YSOF
U+3AC6.0	fCantonese	ji5
U+3AC6.0	fEnglish	[Korean character]
U+3AC7.0	fUTF8	㫇
U+3AC7.0	fR/S	 70.6
U+3AC7.0	fTotalStrokes	 10
U+3AC7.0	fCangjie	YYN
U+3AC7.0	fCantonese	jyu1
U+3AC7.0	fEnglish	[Korean character]
U+3AC8.0	fUTF8	㫈
U+3AC8.0	fR/S	 70.6
U+3AC8.0	fTotalStrokes	 10
U+3AC8.0	fCangjie	YYR
U+3AC8.0	fCantonese	jyu1
U+3AC8.0	fEnglish	[Korean character]
U+3AC9.0	fUTF8	㫉
U+3AC9.0	fR/S	 70.6
U+3AC9.0	fTotalStrokes	 10
U+3AC9.0	fCangjie	YSNMU
U+3AC9.0	fCantonese	ngai4; ai4 (also)
U+3AC9.0	fEnglish	[Korean character]
U+3ACA.0	fUTF8	㫊
U+3ACA.0	fCNS11643	3-363B
U+3ACA.0	fGB	5-5125
U+3ACA.0	fR/S	 70.7
U+3ACA.0	fTotalStrokes	 11
U+3ACA.0	fCangjie	YSOMR
U+3ACA.0	fFourCorner	08220
U+3ACA.0	fCantonese	ji2; zi2
U+3ACA.0	fMandarin	yi3
U+3ACA.0	fEnglish	[1] fluttering of the flag [2] the flags
U+3ACB.0	fUTF8	㫋
U+3ACB.0	fCNS11643	3-363E
U+3ACB.0	fGB	S-2424
U+3ACB.0	fR/S	 70.7
U+3ACB.0	fTotalStrokes	 11
U+3ACB.0	fCangjie	YSOGB
U+3ACB.0	fFourCorner	08250
U+3ACB.0	fHakka	 zhen1
U+3ACB.0	fCantonese	zin1
U+3ACB.0	fMandarin	zhan1
U+3ACB.0	fEnglish	non-classical form of &#x65C3;
U+3ACC.0	fUTF8	㫌
U+3ACC.0	fR/S	 70.7
U+3ACC.0	fTotalStrokes	 11
U+3ACC.0	fCangjie	YSOMM
U+3ACC.0	fCantonese	syun4
U+3ACC.0	fEnglish	[Korean character]
U+3ACD.0	fUTF8	㫍
U+3ACD.0	fCNS11643	4-3D65
U+3ACD.0	fGB	3-4F48
U+3ACD.0	fR/S	 70.8; 70.9
U+3ACD.0	fTotalStrokes	 13; 12
U+3ACD.0	fCangjie	YSOOK
U+3ACD.0	fFourCorner	08240
U+3ACD.0	fCantonese	jau4; zaau4
U+3ACD.0	fMandarin	you2
U+3ACD.0	fEnglish	[1] the scallops along the lower edge of a flag; the hanging decorations (ribbons, etc.) of a flag used in ancient times [2] a kind of flag
U+3ACE.0	fUTF8	㫎
U+3ACE.0	fCNS11643	4-4A30
U+3ACE.0	fGB	5-5129
U+3ACE.0	fR/S	 70.11
U+3ACE.0	fTotalStrokes	 15
U+3ACE.0	fCangjie	YSOBJ
U+3ACE.0	fFourCorner	08250
U+3ACE.0	fCantonese	fai1
U+3ACE.0	fMandarin	hui1; gun3; kuai4
U+3ACE.0	fEnglish	same as &#x5FBD;, &#x63EE;
U+3ACF.0	fUTF8	㫏
U+3ACF.0	fCNS11643	4-4A31
U+3ACF.0	fGB	5-512A
U+3ACF.0	fR/S	 70.11
U+3ACF.0	fTotalStrokes	 15
U+3ACF.0	fCangjie	YSOMV
U+3ACF.0	fFourCorner	08240
U+3ACF.0	fCantonese	jiu2; miu5
U+3ACF.0	fMandarin	yao3
U+3ACF.0	fEnglish	[1] a kind of flag [2] flags; streamers
U+3AD0.0	fUTF8	㫐
U+3AD0.0	fCNS11643	4-2278
U+3AD0.0	fGB	5-494B
U+3AD0.0	fR/S	 72.2
U+3AD0.0	fTotalStrokes	 6
U+3AD0.0	fCangjie	AP
U+3AD0.0	fFourCorner	60710
U+3AD0.0	fCantonese	jiu2; miu5
U+3AD0.0	fMandarin	yao3
U+3AD0.0	fEnglish	distant and indistinguishable
U+3AD1.0	fUTF8	㫑
U+3AD1.0	fCNS11643	4-2279
U+3AD1.0	fR/S	 72.2
U+3AD1.0	fTotalStrokes	 6
U+3AD1.0	fCangjie	AKS
U+3AD1.0	fFourCorner	64020
U+3AD1.0	fHakka	 zhi3
U+3AD1.0	fCantonese	zi2; si4
U+3AD1.0	fMandarin	zhi3; shi2
U+3AD1.0	fEnglish	same as &#x65E8;
U+3AD2.0	fUTF8	㫒
U+3AD2.0	fCNS11643	4-2443
U+3AD2.0	fGB	5-4A37
U+3AD2.0	fR/S	 72.3; 72.4; 55.4
U+3AD2.0	fTotalStrokes	 7
U+3AD2.0	fCangjie	AT
U+3AD2.0	fFourCorner	60440
U+3AD2.0	fHakka	 giung3
U+3AD2.0	fCantonese	gung1; gung2
U+3AD2.0	fMandarin	gong3
U+3AD2.0	fEnglish	same as &#x5EFE;
U+3AD3.0	fUTF8	㫓
U+3AD3.0	fCNS11643	3-263C
U+3AD3.0	fR/S	 72.3
U+3AD3.0	fTotalStrokes	 7
U+3AD3.0	fCangjie	AON
U+3AD3.0	fFourCorner	68010
U+3AD3.0	fCantonese	hei3
U+3AD3.0	fMandarin	qi4; qi3
U+3AD3.0	fEnglish	ancient form of &#x6C23;
U+3AD4.0	fUTF8	㫔
U+3AD4.0	fCNS11643	3-263F
U+3AD4.0	fR/S	 72.3
U+3AD4.0	fTotalStrokes	 7
U+3AD4.0	fCangjie	AYM
U+3AD4.0	fFourCorner	60100
U+3AD4.0	fHakka	 gen5
U+3AD4.0	fCantonese	gan3
U+3AD4.0	fMandarin	gen4; yin2
U+3AD4.0	fEnglish	same as &#x826E;
U+3AD5.0	fUTF8	㫕
U+3AD5.0	fCNS11643	3-263E
U+3AD5.0	fR/S	 72.3
U+3AD5.0	fTotalStrokes	 7
U+3AD5.0	fCangjie	AMU
U+3AD5.0	fFourCorner	60210
U+3AD5.0	fHakka	 kwong1
U+3AD5.0	fCantonese	gwong1; gong1 (also)
U+3AD5.0	fMandarin	guang1; guang2; guang4
U+3AD5.0	fEnglish	same as &#x5149;
U+3AD6.0	fUTF8	㫖
U+3AD6.0	fR/S	 72.3
U+3AD6.0	fTotalStrokes	 7
U+3AD6.0	fCangjie	YMA
U+3AD6.0	fCantonese	zi2
U+3AD6.0	fEnglish	[Japanese character]
U+3AD7.0	fUTF8	㫗
U+3AD7.0	fCNS11643	3-2640
U+3AD7.0	fR/S	 72.3; 73.3
U+3AD7.0	fTotalStrokes	 7
U+3AD7.0	fCangjie	AND
U+3AD7.0	fFourCorner	60400
U+3AD7.0	fHakka	 heu5
U+3AD7.0	fCantonese	hau5
U+3AD7.0	fMandarin	hou4
U+3AD7.0	fEnglish	standard form of &#x539A;
U+3AD8.0	fUTF8	㫘
U+3AD8.0	fCNS11643	3-293F
U+3AD8.0	fGB	5-4952
U+3AD8.0	fR/S	 72.4
U+3AD8.0	fTotalStrokes	 8
U+3AD8.0	fCangjie	MFA
U+3AD8.0	fFourCorner	10600
U+3AD8.0	fHakka	 met7
U+3AD8.0	fCantonese	mat6
U+3AD8.0	fMandarin	mi4; mian3
U+3AD8.0	fEnglish	[1] disappeared [2] not supported by [3] avoid [4] unable to see
U+3AD9.0	fUTF8	㫙
U+3AD9.0	fCNS11643	3-293D
U+3AD9.0	fR/S	 72.4
U+3AD9.0	fTotalStrokes	 8
U+3AD9.0	fCangjie	AQO
U+3AD9.0	fFourCorner	65030
U+3AD9.0	fCantonese	fu4
U+3AD9.0	fMandarin	fu2
U+3AD9.0	fEnglish	the sun
U+3ADA.0	fUTF8	㫚
U+3ADA.0	fCNS11643	4-2663
U+3ADA.0	fGB	3-4753
U+3ADA.0	fR/S	 73.4; 72.4
U+3ADA.0	fTotalStrokes	 8
U+3ADA.0	fCangjie	PHA
U+3ADA.0	fFourCorner	27602
U+3ADA.0	fCantonese	fat1
U+3ADA.0	fMandarin	hu1
U+3ADA.0	fEnglish	[1] early morning [2] daylight
U+3ADB.0	fUTF8	㫛
U+3ADB.0	fCNS11643	5-294F
U+3ADB.0	fR/S	 72.5
U+3ADB.0	fTotalStrokes	 9
U+3ADB.0	fCangjie	ARHU
U+3ADB.0	fFourCorner	66010
U+3ADB.0	fCantonese	gwong3; gong3 (also); gwong2
U+3ADB.0	fMandarin	guang4; kuang2; kuang4; mu3
U+3ADB.0	fEnglish	[1] light; bright; brilliant [2] clear [3] intelligent; day dawn
U+3ADC.0	fUTF8	㫜
U+3ADC.0	fCNS11643	4-2963
U+3ADC.0	fR/S	 72.5
U+3ADC.0	fTotalStrokes	 9
U+3ADC.0	fCangjie	AAM
U+3ADC.0	fFourCorner	66010
U+3ADC.0	fCantonese	taan2
U+3ADC.0	fMandarin	tan3; dan4
U+3ADC.0	fEnglish	[1] light; bright; brilliant [2] clear [3] intelligent; day dawn
U+3ADD.0	fUTF8	㫝
U+3ADD.0	fCNS11643	3-2C7B
U+3ADD.0	fR/S	 72.5
U+3ADD.0	fTotalStrokes	 9
U+3ADD.0	fCangjie	AHPM
U+3ADD.0	fFourCorner	62040
U+3ADD.0	fCantonese	dai1
U+3ADD.0	fMandarin	di1
U+3ADD.0	fEnglish	the sun
U+3ADE.0	fUTF8	㫞
U+3ADE.0	fCNS11643	4-2964
U+3ADE.0	fR/S	 72.5
U+3ADE.0	fTotalStrokes	 9
U+3ADE.0	fCangjie	AUE
U+3ADE.0	fFourCorner	62040
U+3ADE.0	fCantonese	tou1; zau2
U+3ADE.0	fMandarin	tao1
U+3ADE.0	fEnglish	[1] of look of the sun [2] the look of the weather
U+3ADF.0	fUTF8	㫟
U+3ADF.0	fCNS11643	4-2965
U+3ADF.0	fGB	3-475D
U+3ADF.0	fR/S	 72.5
U+3ADF.0	fTotalStrokes	 9
U+3ADF.0	fCangjie	AHNR
U+3ADF.0	fFourCorner	62060
U+3ADF.0	fCantonese	jyun4; zyun4
U+3ADF.0	fMandarin	yan2
U+3ADF.0	fEnglish	[1] day after day [2] daily [3] time [4] moving of the sun
U+3AE0.0	fUTF8	㫠
U+3AE0.0	fCNS11643	3-2C78
U+3AE0.0	fR/S	 72.5
U+3AE0.0	fTotalStrokes	 9
U+3AE0.0	fCangjie	AFQ
U+3AE0.0	fFourCorner	69050
U+3AE0.0	fCantonese	bun3
U+3AE0.0	fEnglish	a kind of water
U+3AE1.0	fUTF8	㫡
U+3AE1.0	fCNS11643	3-2D26
U+3AE1.0	fR/S	 72.5
U+3AE1.0	fTotalStrokes	 9
U+3AE1.0	fCangjie	AHEY
U+3AE1.0	fFourCorner	60303
U+3AE1.0	fCantonese	dung1
U+3AE1.0	fMandarin	dong1
U+3AE1.0	fEnglish	same as &#x51AC;
U+3AE2.0	fUTF8	㫢
U+3AE2.0	fR/S	 72.5
U+3AE2.0	fTotalStrokes	 9
U+3AE2.0	fCangjie	AGI
U+3AE2.0	fCantonese	heoi3
U+3AE2.0	fMandarin	qu4
U+3AE3.0	fUTF8	㫣
U+3AE3.0	fGB	S-212D
U+3AE3.0	fR/S	 72.5
U+3AE3.0	fTotalStrokes	 9
U+3AE3.0	fCangjie	ALWU
U+3AE3.0	fCantonese	man5
U+3AE4.0	fUTF8	㫤
U+3AE4.0	fCNS11643	6-3167
U+3AE4.0	fR/S	 72.5
U+3AE4.0	fTotalStrokes	 9
U+3AE4.0	fCangjie	AMNE
U+3AE4.0	fFourCorner	60230
U+3AE4.0	fHakka	 chong3
U+3AE4.0	fCantonese	cong2; zou6; zuk6
U+3AE4.0	fMandarin	chang3; chang4; zhao4
U+3AE4.0	fEnglish	same as &#x6636;
U+3AE5.0	fUTF8	㫥
U+3AE5.0	fCNS11643	4-2D4C
U+3AE5.0	fGB	5-4965
U+3AE5.0	fR/S	 72.6
U+3AE5.0	fTotalStrokes	 10
U+3AE5.0	fCangjie	ANIR
U+3AE5.0	fFourCorner	67060
U+3AE5.0	fCantonese	ming4
U+3AE5.0	fMandarin	ming3
U+3AE5.0	fEnglish	non-classical form of &#x51A5;
U+3AE6.0	fUTF8	㫦
U+3AE6.0	fCNS11643	4-323B
U+3AE6.0	fR/S	 72.6
U+3AE6.0	fTotalStrokes	 11; 10
U+3AE6.0	fCangjie	OLHA
U+3AE6.0	fFourCorner	28264
U+3AE6.0	fCantonese	tou1
U+3AE6.0	fMandarin	tao1
U+3AE6.0	fEnglish	[1] of look of the sun [2] the look of the weather
U+3AE7.0	fUTF8	㫧
U+3AE7.0	fCNS11643	4-2D4B
U+3AE7.0	fR/S	 72.6
U+3AE7.0	fTotalStrokes	 10
U+3AE7.0	fCangjie	AFD
U+3AE7.0	fFourCorner	60900
U+3AE7.0	fHakka	 pau5; pok7
U+3AE7.0	fCantonese	bou6; bok3; buk6; keoi4
U+3AE7.0	fMandarin	bao4; bo1; bo2
U+3AE7.0	fEnglish	same as &#x66B4;
U+3AE8.0	fUTF8	㫨
U+3AE8.0	fR/S	 72.6
U+3AE8.0	fTotalStrokes	 10
U+3AE8.0	fCangjie	AJV
U+3AE8.0	fCantonese	on1; ngon1 (also)
U+3AE9.0	fUTF8	㫩
U+3AE9.0	fCNS11643	3-3147
U+3AE9.0	fR/S	 72.6
U+3AE9.0	fTotalStrokes	 10
U+3AE9.0	fCangjie	JTA
U+3AE9.0	fFourCorner	40600
U+3AE9.0	fHakka	 chun1
U+3AE9.0	fCantonese	ceon1
U+3AE9.0	fMandarin	chun1; chun3
U+3AE9.0	fEnglish	same as &#x6625;
U+3AEA.0	fUTF8	㫪
U+3AEA.0	fCNS11643	6-3837
U+3AEA.0	fR/S	 72.6
U+3AEA.0	fTotalStrokes	 10; 9
U+3AEA.0	fCangjie	QKLA
U+3AEA.0	fFourCorner	50260
U+3AEA.0	fHakka	 chun1
U+3AEA.0	fCantonese	zung1; ceon1
U+3AEA.0	fMandarin	chong1; chun1; chun3
U+3AEA.0	fEnglish	non-classical form of &#x8202;
U+3AEB.0	fUTF8	㫫
U+3AEB.0	fCNS11643	3-314A
U+3AEB.0	fR/S	 72.6
U+3AEB.0	fTotalStrokes	 10
U+3AEB.0	fCangjie	AMTC
U+3AEB.0	fFourCorner	60100
U+3AEB.0	fCantonese	hin2
U+3AEB.0	fMandarin	xian3
U+3AEB.0	fEnglish	non-classical form of &#x986F;
U+3AEC.0	fUTF8	㫬
U+3AEC.0	fCNS11643	F-2F6E
U+3AEC.0	fGB	S-212F
U+3AEC.0	fR/S	 72.6
U+3AEC.0	fTotalStrokes	 10
U+3AEC.0	fCangjie	APA
U+3AEC.0	fFourCorner	6702
U+3AEC.0	fCantonese	ceon4
U+3AED.0	fUTF8	㫭
U+3AED.0	fR/S	 72.6
U+3AED.0	fTotalStrokes	 10
U+3AED.0	fCangjie	AGDI
U+3AED.0	fCantonese	si4
U+3AED.0	fEnglish	[Korean character]
U+3AEE.0	fUTF8	㫮
U+3AEE.0	fCNS11643	4-2D4D
U+3AEE.0	fR/S	 72.6
U+3AEE.0	fTotalStrokes	 10
U+3AEE.0	fCangjie	YPA
U+3AEE.0	fFourCorner	21600
U+3AEE.0	fCantonese	ci2
U+3AEE.0	fMandarin	zi4
U+3AEF.0	fUTF8	㫯
U+3AEF.0	fCNS11643	6-4051
U+3AEF.0	fR/S	 72.7
U+3AEF.0	fTotalStrokes	 11
U+3AEF.0	fCangjie	AAMU
U+3AEF.0	fFourCorner	60210
U+3AEF.0	fCantonese	mou6
U+3AEF.0	fMandarin	mo4; mao4
U+3AEF.0	fEnglish	[1] in contravention of; to offend (a superior, elder, etc.) [2] same as &#x5192;
U+3AF0.0	fUTF8	㫰
U+3AF0.0	fCNS11643	4-3236
U+3AF0.0	fGB	3-476D
U+3AF0.0	fR/S	 72.7
U+3AF0.0	fTotalStrokes	 11
U+3AF0.0	fCangjie	AIAV
U+3AF0.0	fFourCorner	63030
U+3AF0.0	fHakka	 long5; long6; lang2
U+3AF0.0	fCantonese	long6; long5
U+3AF0.0	fMandarin	lang3
U+3AF0.0	fEnglish	[1] expose to the air to dry [2] clear; bright
U+3AF1.0	fUTF8	㫱
U+3AF1.0	fCNS11643	4-323A
U+3AF1.0	fGB	3-4768
U+3AF1.0	fR/S	 72.7
U+3AF1.0	fTotalStrokes	 11
U+3AF1.0	fCangjie	AGLC
U+3AF1.0	fFourCorner	60330
U+3AF1.0	fCantonese	naan5; laan5 (also); nan5
U+3AF1.0	fMandarin	nan3
U+3AF1.0	fEnglish	[1] mild [2] damp and moist [3] red color
U+3AF2.0	fUTF8	㫲
U+3AF2.0	fCNS11643	4-3237
U+3AF2.0	fGB	3-476A
U+3AF2.0	fR/S	 72.7
U+3AF2.0	fTotalStrokes	 11
U+3AF2.0	fCangjie	AJBD
U+3AF2.0	fFourCorner	64040
U+3AF2.0	fCantonese	pui3; but6
U+3AF2.0	fMandarin	pei4
U+3AF2.0	fEnglish	dark; dull
U+3AF3.0	fUTF8	㫳
U+3AF3.0	fCNS11643	3-3643
U+3AF3.0	fR/S	 72.7
U+3AF3.0	fTotalStrokes	 11
U+3AF3.0	fCangjie	MVA
U+3AF3.0	fFourCorner	71600
U+3AF3.0	fHakka	 shin2
U+3AF3.0	fCantonese	san4
U+3AF3.0	fMandarin	chen2; shen2
U+3AF3.0	fEnglish	same as &#x6668;
U+3AF4.0	fUTF8	㫴
U+3AF4.0	fCNS11643	6-404F
U+3AF4.0	fR/S	 72.7
U+3AF4.0	fTotalStrokes	 11
U+3AF4.0	fCangjie	AJKD
U+3AF4.0	fFourCorner	64040
U+3AF4.0	fHakka	 hau1
U+3AF4.0	fCantonese	haau1; hau1; ngau4
U+3AF4.0	fMandarin	xiao1; xiao4; xue1; xiao3
U+3AF4.0	fEnglish	same as &#x54EE;
U+3AF5.0	fUTF8	㫵
U+3AF5.0	fCNS11643	3-3C22
U+3AF5.0	fR/S	 72.8
U+3AF5.0	fTotalStrokes	 12
U+3AF5.0	fCangjie	ALMY
U+3AF5.0	fFourCorner	61010
U+3AF5.0	fCantonese	fei1
U+3AF5.0	fMandarin	fei2
U+3AF5.0	fEnglish	[1] white color [2] corrupted form of &#x8153;
U+3AF6.0	fUTF8	㫶
U+3AF6.0	fCNS11643	4-377E
U+3AF6.0	fGB	5-4974
U+3AF6.0	fR/S	 72.8
U+3AF6.0	fTotalStrokes	 12
U+3AF6.0	fCangjie	ASMB
U+3AF6.0	fFourCorner	67020
U+3AF6.0	fCantonese	zau2
U+3AF6.0	fMandarin	zhou3; cou3
U+3AF6.0	fEnglish	light; bright
U+3AF7.0	fUTF8	㫷
U+3AF7.0	fCNS11643	3-3B7A
U+3AF7.0	fGB	5-496F
U+3AF7.0	fR/S	 72.8
U+3AF7.0	fTotalStrokes	 12
U+3AF7.0	fCangjie	TCA
U+3AF7.0	fFourCorner	44600
U+3AF7.0	fCantonese	gei1
U+3AF7.0	fMandarin	ji1
U+3AF7.0	fEnglish	[1] behind; the back of; afterwards; come after [2] same as &#x671F;
U+3AF8.0	fUTF8	㫸
U+3AF8.0	fCNS11643	3-3B7B
U+3AF8.0	fR/S	 72.8
U+3AF8.0	fTotalStrokes	 12
U+3AF8.0	fCangjie	AJLO
U+3AF8.0	fFourCorner	65080
U+3AF8.0	fCantonese	zip3; cip3
U+3AF8.0	fMandarin	qie4
U+3AF8.0	fEnglish	sunset
U+3AF9.0	fUTF8	㫹
U+3AF9.0	fCNS11643	4-3825
U+3AF9.0	fGB	3-4774
U+3AF9.0	fR/S	 72.8
U+3AF9.0	fTotalStrokes	 12
U+3AF9.0	fCangjie	AHSL
U+3AF9.0	fFourCorner	60220
U+3AF9.0	fCantonese	so3; cuk1
U+3AF9.0	fMandarin	shu4; dai4; yu2
U+3AF9.0	fEnglish	[1] light; bright [2] warm [3] genial
U+3AFA.0	fUTF8	㫺
U+3AFA.0	fCNS11643	3-3C27
U+3AFA.0	fR/S	 72.8
U+3AFA.0	fTotalStrokes	 12
U+3AFA.0	fCangjie	OOA
U+3AFA.0	fFourCorner	88600
U+3AFA.0	fHakka	 sit8
U+3AFA.0	fCantonese	sik1
U+3AFA.0	fMandarin	xi2; xi1; cuo2; cuo4
U+3AFA.0	fEnglish	standard form of &#x6614;
U+3AFB.0	fUTF8	㫻
U+3AFB.0	fCNS11643	4-3822
U+3AFB.0	fR/S	 72.8
U+3AFB.0	fTotalStrokes	 12
U+3AFB.0	fCangjie	AOMB
U+3AFB.0	fFourCorner	68020
U+3AFB.0	fCantonese	gwan3; kwaan3
U+3AFB.0	fMandarin	gun4; kun4
U+3AFB.0	fEnglish	sunlight
U+3AFC.0	fUTF8	㫼
U+3AFC.0	fCNS11643	6-4973
U+3AFC.0	fR/S	 72.8
U+3AFC.0	fTotalStrokes	 12
U+3AFC.0	fCangjie	AHBN
U+3AFC.0	fFourCorner	62000
U+3AFC.0	fCantonese	zit3; zaai3
U+3AFC.0	fMandarin	zhe1; die2; zhe2; zhi4
U+3AFC.0	fEnglish	same as &#x6662;
U+3AFD.0	fUTF8	㫽
U+3AFD.0	fCNS11643	3-3C21
U+3AFD.0	fGB	5-4975
U+3AFD.0	fR/S	 72.8
U+3AFD.0	fTotalStrokes	 12
U+3AFD.0	fCangjie	AVNE
U+3AFD.0	fFourCorner	67030
U+3AFD.0	fCantonese	luk6
U+3AFD.0	fMandarin	lu4
U+3AFD.0	fEnglish	darkness; the sun is dimmed
U+3AFE.0	fUTF8	㫾
U+3AFE.0	fR/S	 72.8
U+3AFE.0	fTotalStrokes	 12
U+3AFE.0	fCangjie	AFBR
U+3AFE.0	fCantonese	soeng6
U+3AFE.0	fEnglish	[Korean character]
U+3AFF.0	fUTF8	㫿
U+3AFF.0	fR/S	 72.8
U+3AFF.0	fTotalStrokes	 12
U+3AFF.0	fCangjie	AOG
U+3AFF.0	fFourCorner	6001
U+3AFF.0	fCantonese	zeoi1
U+3AFF.0	fEnglish	[Korean character]
U+3B00.0	fUTF8	㬀
U+3B00.0	fR/S	 72.8
U+3B00.0	fTotalStrokes	 12
U+3B00.0	fCangjie	AYRD
U+3B00.0	fCantonese	hoeng2
U+3B00.0	fEnglish	[Korean character]
U+3B01.0	fUTF8	㬁
U+3B01.0	fCNS11643	6-533D
U+3B01.0	fR/S	 72.9
U+3B01.0	fTotalStrokes	 13
U+3B01.0	fCangjie	AEOR
U+3B01.0	fFourCorner	61010
U+3B01.0	fCantonese	caap3
U+3B01.0	fMandarin	xuan3; cha1
U+3B01.0	fEnglish	same as &#x23197;
U+3B02.0	fUTF8	㬂
U+3B02.0	fCNS11643	5-3F62
U+3B02.0	fR/S	 72.9
U+3B02.0	fTotalStrokes	 13
U+3B02.0	fCangjie	AWLB
U+3B02.0	fFourCorner	66020
U+3B02.0	fCantonese	jyu4; zyu4
U+3B02.0	fMandarin	yu2
U+3B02.0	fEnglish	used in a person's name
U+3B03.0	fUTF8	㬃
U+3B03.0	fCNS11643	3-4152
U+3B03.0	fR/S	 72.9
U+3B03.0	fTotalStrokes	 13
U+3B03.0	fCangjie	AIRF
U+3B03.0	fFourCorner	60800
U+3B03.0	fCantonese	toi4
U+3B03.0	fMandarin	tai2
U+3B03.0	fEnglish	sunrise
U+3B04.0	fUTF8	㬄
U+3B04.0	fCNS11643	3-4157
U+3B04.0	fR/S	 72.9
U+3B04.0	fTotalStrokes	 13
U+3B04.0	fCangjie	AMGC
U+3B04.0	fFourCorner	60230
U+3B04.0	fCantonese	caan3; maa6
U+3B04.0	fMandarin	chan4; nan3
U+3B04.0	fEnglish	corrupted form of &#x24323A;
U+3B05.0	fUTF8	㬅
U+3B05.0	fCNS11643	3-415A
U+3B05.0	fGB	5-4A3D
U+3B05.0	fR/S	 72.9
U+3B05.0	fTotalStrokes	 13
U+3B05.0	fCangjie	AWLS
U+3B05.0	fFourCorner	60220
U+3B05.0	fHakka	 man5; van5
U+3B05.0	fCantonese	maan6
U+3B05.0	fMandarin	man4; man1; wan4
U+3B05.0	fEnglish	non-classical form of &#x66FC;
U+3B06.0	fUTF8	㬆
U+3B06.0	fCNS11643	3-4155
U+3B06.0	fR/S	 72.9
U+3B06.0	fTotalStrokes	 13
U+3B06.0	fCangjie	ARPA
U+3B06.0	fFourCorner	67060
U+3B06.0	fCantonese	man5
U+3B06.0	fMandarin	min3; mian2; mian4
U+3B06.0	fEnglish	same as &#x6E63;
U+3B07.0	fUTF8	㬇
U+3B07.0	fCNS11643	3-4153
U+3B07.0	fR/S	 72.9
U+3B07.0	fTotalStrokes	 13
U+3B07.0	fCangjie	ANBK
U+3B07.0	fFourCorner	67030
U+3B07.0	fCantonese	wun6
U+3B07.0	fMandarin	huan4
U+3B07.0	fEnglish	[1] the name of a state in ancient China [2] sharp [3] bright-colored; brightness; bright; radiance; light
U+3B08.0	fUTF8	㬈
U+3B08.0	fCNS11643	3-415C
U+3B08.0	fR/S	 72.9
U+3B08.0	fTotalStrokes	 13
U+3B08.0	fCangjie	AABT
U+3B08.0	fFourCorner	66010
U+3B08.0	fCantonese	wan1
U+3B08.0	fMandarin	wen1
U+3B08.0	fEnglish	the sun was shining and it's warm
U+3B09.0	fUTF8	㬉
U+3B09.0	fCNS11643	3-4158
U+3B09.0	fR/S	 72.9
U+3B09.0	fTotalStrokes	 13
U+3B09.0	fCangjie	AMBK
U+3B09.0	fFourCorner	61030
U+3B09.0	fHakka	 non3; nun1
U+3B09.0	fCantonese	nyun5; lyun5 (also); hyun1
U+3B09.0	fMandarin	nuan3; ruo4
U+3B09.0	fEnglish	genial; warm
U+3B0A.0	fUTF8	㬊
U+3B0A.0	fCNS11643	4-3D68
U+3B0A.0	fGB	3-477E
U+3B0A.0	fR/S	 72.9
U+3B0A.0	fTotalStrokes	 13
U+3B0A.0	fCangjie	ABME
U+3B0A.0	fFourCorner	60440
U+3B0A.0	fCantonese	wun6
U+3B0A.0	fMandarin	huan3
U+3B0A.0	fEnglish	[1] light; bright [2] clear [3] intelligent; clever
U+3B0B.0	fUTF8	㬋
U+3B0B.0	fCNS11643	4-3D67
U+3B0B.0	fGB	3-477D
U+3B0B.0	fR/S	 72.9
U+3B0B.0	fTotalStrokes	 13
U+3B0B.0	fCangjie	AONK
U+3B0B.0	fFourCorner	67034
U+3B0B.0	fHakka	 heu2
U+3B0B.0	fCantonese	hau4
U+3B0B.0	fMandarin	hou2
U+3B0B.0	fEnglish	a constellation
U+3B0C.0	fUTF8	㬌
U+3B0C.0	fCNS11643	3-4154
U+3B0C.0	fR/S	 72.9
U+3B0C.0	fTotalStrokes	 13
U+3B0C.0	fCangjie	AYAF
U+3B0C.0	fFourCorner	60900
U+3B0C.0	fCantonese	ging2; kaai3
U+3B0C.0	fMandarin	jing4
U+3B0C.0	fEnglish	sunshine; sunlight
U+3B0D.0	fUTF8	㬍
U+3B0D.0	fCNS11643	3-4738
U+3B0D.0	fGB	5-497E
U+3B0D.0	fR/S	 72.10
U+3B0D.0	fTotalStrokes	 14
U+3B0D.0	fCangjie	AIBI
U+3B0D.0	fFourCorner	63040
U+3B0D.0	fCantonese	bok3
U+3B0D.0	fMandarin	bo2
U+3B0D.0	fEnglish	[1] fierce; cruel [2] sudden [3] a scorching heat
U+3B0E.0	fUTF8	㬎
U+3B0E.0	fCNS11643	4-436E
U+3B0E.0	fGB	5-4A25
U+3B0E.0	fR/S	 72.10
U+3B0E.0	fTotalStrokes	 14
U+3B0E.0	fCangjie	AVIF
U+3B0E.0	fFourCorner	60330
U+3B0E.0	fHakka	 hien3
U+3B0E.0	fCantonese	hin2
U+3B0E.0	fMandarin	xian3
U+3B0E.0	fEnglish	[1] infinitesimal [2] thready; fibrous
U+3B0F.0	fUTF8	㬏
U+3B0F.0	fCNS11643	3-4732
U+3B0F.0	fR/S	 72.10
U+3B0F.0	fTotalStrokes	 14
U+3B0F.0	fCangjie	AMRB
U+3B0F.0	fFourCorner	61020
U+3B0F.0	fCantonese	lik6
U+3B0F.0	fMandarin	li4
U+3B0F.0	fEnglish	[1] light; bright [2] clear [3] intelligent; clever
U+3B10.0	fUTF8	㬐
U+3B10.0	fCNS11643	3-4736
U+3B10.0	fGB	5-497D
U+3B10.0	fR/S	 72.10
U+3B10.0	fTotalStrokes	 14
U+3B10.0	fCangjie	AMIA
U+3B10.0	fFourCorner	61060
U+3B10.0	fCantonese	zeon3
U+3B10.0	fMandarin	jin4; jin3; xing2; ying3
U+3B10.0	fEnglish	same as &#x6649;
U+3B11.0	fUTF8	㬑
U+3B11.0	fGB	S-212E
U+3B11.0	fR/S	 72.11
U+3B11.0	fTotalStrokes	 15
U+3B11.0	fCangjie	AYDL
U+3B11.0	fCantonese	gwok3; gok3 (also)
U+3B12.0	fUTF8	㬒
U+3B12.0	fCNS11643	5-4D53
U+3B12.0	fR/S	 72.11
U+3B12.0	fTotalStrokes	 15
U+3B12.0	fCangjie	ATIT
U+3B12.0	fFourCorner	64040
U+3B12.0	fHakka	 mong3;  mang3
U+3B12.0	fCantonese	mong5
U+3B12.0	fMandarin	mang3; mao4
U+3B12.0	fEnglish	the sun is dimmed; darkness
U+3B13.0	fUTF8	㬓
U+3B13.0	fCNS11643	4-4A32
U+3B13.0	fGB	3-482B
U+3B13.0	fR/S	 72.11
U+3B13.0	fTotalStrokes	 15
U+3B13.0	fCangjie	AMWF
U+3B13.0	fFourCorner	61090
U+3B13.0	fHakka	 piau1; piau3; piau5
U+3B13.0	fCantonese	piu3; pau1; piu5
U+3B13.0	fMandarin	piao4
U+3B13.0	fEnglish	 dry in the sun
U+3B14.0	fUTF8	㬔
U+3B14.0	fCNS11643	3-4C42
U+3B14.0	fR/S	 72.11
U+3B14.0	fTotalStrokes	 15
U+3B14.0	fCangjie	ASMH
U+3B14.0	fFourCorner	67020
U+3B14.0	fCantonese	hou4
U+3B14.0	fMandarin	hao2
U+3B14.0	fEnglish	[1] defy; disobey; disregard [2] contradict [3] perverse; recalcitrant; irregular; abnormal
U+3B15.0	fUTF8	㬕
U+3B15.0	fCNS11643	3-4C45
U+3B15.0	fR/S	 72.11
U+3B15.0	fTotalStrokes	 15
U+3B15.0	fCangjie	AIFQ
U+3B15.0	fFourCorner	68050
U+3B15.0	fCantonese	joeng4; zoeng4
U+3B15.0	fMandarin	yang2
U+3B15.0	fEnglish	[1] (corrupted form) [2] light; bright [3] clear
U+3B16.0	fUTF8	㬖
U+3B16.0	fR/S	 72.11
U+3B16.0	fTotalStrokes	 15
U+3B16.0	fCangjie	AHKR
U+3B16.0	fCantonese	kai2
U+3B16.0	fEnglish	[Korean character]
U+3B17.0	fUTF8	㬗
U+3B17.0	fCNS11643	4-5052
U+3B17.0	fGB	5-4A2D
U+3B17.0	fR/S	 72.12
U+3B17.0	fTotalStrokes	 16
U+3B17.0	fCangjie	ABKF
U+3B17.0	fFourCorner	63030
U+3B17.0	fCantonese	nin6; lin6 (also); zin2
U+3B17.0	fMandarin	nian3; xian4
U+3B17.0	fEnglish	same as &#x665B;
U+3B18.0	fUTF8	㬘
U+3B18.0	fCNS11643	4-5051
U+3B18.0	fGB	3-4778
U+3B18.0	fR/S	 72.12
U+3B18.0	fTotalStrokes	 16
U+3B18.0	fCangjie	ALX
U+3B18.0	fFourCorner	65020
U+3B18.0	fCantonese	suk1
U+3B18.0	fMandarin	su4
U+3B18.0	fEnglish	[1] arid; dry; parched [2] impatient; restless [3] fierce; cruel [4] sudden [5] a scorching heat
U+3B19.0	fUTF8	㬙
U+3B19.0	fCNS11643	3-5121
U+3B19.0	fR/S	 72.12
U+3B19.0	fTotalStrokes	 16
U+3B19.0	fCangjie	ABHF
U+3B19.0	fFourCorner	62020
U+3B19.0	fCantonese	wai5
U+3B19.0	fMandarin	wei3
U+3B19.0	fEnglish	sunshine; light of the sun
U+3B1A.0	fUTF8	㬚
U+3B1A.0	fCNS11643	5-4D50
U+3B1A.0	fGB	3-4832
U+3B1A.0	fR/S	 72.12; 72.11
U+3B1A.0	fTotalStrokes	 16; 15
U+3B1A.0	fCangjie	AYBK
U+3B1A.0	fFourCorner	68040
U+3B1A.0	fCantonese	cit3
U+3B1A.0	fMandarin	che4
U+3B1A.0	fEnglish	[1] light; bright [2] clear
U+3B1B.0	fUTF8	㬛
U+3B1B.0	fCNS11643	3-507B
U+3B1B.0	fR/S	 72.12
U+3B1B.0	fTotalStrokes	 16
U+3B1B.0	fCangjie	AORM
U+3B1B.0	fFourCorner	68020
U+3B1B.0	fCantonese	kap1
U+3B1B.0	fMandarin	xi1
U+3B1B.0	fEnglish	 dry in the sun
U+3B1C.0	fUTF8	㬜
U+3B1C.0	fCNS11643	3-507C
U+3B1C.0	fR/S	 72.12
U+3B1C.0	fTotalStrokes	 16
U+3B1C.0	fCangjie	MGA
U+3B1C.0	fFourCorner	11600
U+3B1C.0	fHakka	 zin3
U+3B1C.0	fCantonese	zeon3
U+3B1C.0	fMandarin	jin4; jian4
U+3B1C.0	fEnglish	standard form of &#x6649;
U+3B1D.0	fUTF8	㬝
U+3B1D.0	fCNS11643	4-5053
U+3B1D.0	fR/S	 72.12
U+3B1D.0	fTotalStrokes	 16
U+3B1D.0	fCangjie	ACWA
U+3B1D.0	fFourCorner	68060
U+3B1D.0	fCantonese	cang4
U+3B1D.0	fMandarin	song1; ceng2
U+3B1D.0	fEnglish	[1] obscure; not bright; poor visibility [2] white; clear; bright; pure; plain
U+3B1E.0	fUTF8	㬞
U+3B1E.0	fCNS11643	7-2E5A
U+3B1E.0	fR/S	 72.12
U+3B1E.0	fTotalStrokes	 16
U+3B1E.0	fCangjie	AUAV
U+3B1E.0	fFourCorner	62020
U+3B1E.0	fHakka	 jet7
U+3B1E.0	fCantonese	hot3
U+3B1E.0	fMandarin	ye4; he4
U+3B1E.0	fEnglish	same as &#x668D;
U+3B1F.0	fUTF8	㬟
U+3B1F.0	fGB	3-3058
U+3B1F.0	fR/S	 72.12
U+3B1F.0	fTotalStrokes	 16
U+3B1F.0	fCangjie	PHCWA
U+3B1F.0	fCantonese	fan1
U+3B1F.0	fEnglish	an adverb, negative expression; never before; to have not happened before
U+3B20.0	fUTF8	㬠
U+3B20.0	fCNS11643	3-5473
U+3B20.0	fGB	5-4A32
U+3B20.0	fR/S	 72.13
U+3B20.0	fTotalStrokes	 17
U+3B20.0	fCangjie	ANKF
U+3B20.0	fFourCorner	68034
U+3B20.0	fCantonese	saai3; saat3; sai3
U+3B20.0	fMandarin	shai4
U+3B20.0	fEnglish	[1] non-classical form of &#x66EC; [2] interchangeable with &#x715E;
U+3B21.0	fUTF8	㬡
U+3B21.0	fCNS11643	3-546E
U+3B21.0	fR/S	 72.13
U+3B21.0	fTotalStrokes	 17
U+3B21.0	fCangjie	AMBI
U+3B21.0	fFourCorner	61030
U+3B21.0	fCantonese	ling4
U+3B21.0	fMandarin	ling2
U+3B21.0	fEnglish	same as &#x2331F;
U+3B22.0	fUTF8	㬢
U+3B22.0	fCNS11643	4-5640
U+3B22.0	fR/S	 72.13
U+3B22.0	fTotalStrokes	 17; 16
U+3B22.0	fCangjie	ATGI
U+3B22.0	fFourCorner	68050
U+3B22.0	fHakka	 hi1
U+3B22.0	fCantonese	hei1; fui1
U+3B22.0	fMandarin	xi1
U+3B22.0	fEnglish	same as &#x66E6;
U+3B23.0	fUTF8	㬣
U+3B23.0	fCNS11643	4-5B40
U+3B23.0	fGB	3-4763
U+3B23.0	fR/S	 72.14
U+3B23.0	fTotalStrokes	 18
U+3B23.0	fCangjie	ATGI
U+3B23.0	fFourCorner	64000
U+3B23.0	fCantonese	deoi6; deoi3; kit3
U+3B23.0	fMandarin	dui4
U+3B23.0	fEnglish	luxuriant; exuberant; lush; flourishing
U+3B24.0	fUTF8	㬤
U+3B24.0	fGB	3-4836
U+3B24.0	fR/S	 72.14
U+3B24.0	fTotalStrokes	 18
U+3B24.0	fCangjie	AAVF
U+3B24.0	fFourCorner	6603
U+3B24.0	fCantonese	jap1; jaap1; zaap1
U+3B24.0	fEnglish	wet clothes not completely dry
U+3B25.0	fUTF8	㬥
U+3B25.0	fCNS11643	4-5B41
U+3B25.0	fGB	5-4A33
U+3B25.0	fR/S	 72.14
U+3B25.0	fTotalStrokes	 18
U+3B25.0	fCangjie	AUUD
U+3B25.0	fFourCorner	60904
U+3B25.0	fCantonese	buk6; bou6
U+3B25.0	fMandarin	bao4; pu4
U+3B25.0	fEnglish	ancient form of &#x66DD;
U+3B26.0	fUTF8	㬦
U+3B26.0	fCNS11643	4-5B3F
U+3B26.0	fGB	5-4A30
U+3B26.0	fR/S	 72.14
U+3B26.0	fTotalStrokes	 18
U+3B26.0	fCangjie	ATOE
U+3B26.0	fFourCorner	64040
U+3B26.0	fCantonese	wok3
U+3B26.0	fMandarin	huo4; yue4
U+3B26.0	fEnglish	brave; courageous; bold; valiant; intrepid; fearless; heroic
U+3B27.0	fUTF8	㬧
U+3B27.0	fCNS11643	7-3F76
U+3B27.0	fR/S	 72.14
U+3B27.0	fTotalStrokes	 18
U+3B27.0	fCangjie	AUUF
U+3B27.0	fFourCorner	60900
U+3B27.0	fCantonese	bou6; bok3
U+3B27.0	fMandarin	bao4; bo2
U+3B27.0	fEnglish	[1] (standard form) [2] collar [3] neck [4] connect collars to the clothes
U+3B28.0	fUTF8	㬨
U+3B28.0	fR/S	 72.14
U+3B28.0	fTotalStrokes	 18
U+3B28.0	fCangjie	AGCC
U+3B28.0	fCantonese	haak1
U+3B28.0	fEnglish	[Korean character]
U+3B29.0	fUTF8	㬩
U+3B29.0	fCNS11643	3-5A40
U+3B29.0	fR/S	 72.15
U+3B29.0	fTotalStrokes	 19
U+3B29.0	fCangjie	AQJP
U+3B29.0	fFourCorner	65030
U+3B29.0	fHakka	 fui5; fi5
U+3B29.0	fCantonese	wai3; waai6
U+3B29.0	fMandarin	hui4
U+3B29.0	fEnglish	same as &#x66B3;
U+3B2A.0	fUTF8	㬪
U+3B2A.0	fCNS11643	3-5A41
U+3B2A.0	fR/S	 72.15
U+3B2A.0	fTotalStrokes	 19
U+3B2A.0	fCangjie	AAAM
U+3B2A.0	fFourCorner	60100
U+3B2A.0	fCantonese	dip6; daap6 (also)
U+3B2A.0	fMandarin	die2; zhi4
U+3B2A.0	fEnglish	variant of &#x758A;
U+3B2B.0	fUTF8	㬫
U+3B2B.0	fCNS11643	3-5C44
U+3B2B.0	fGB	5-4A34
U+3B2B.0	fR/S	 72.16
U+3B2B.0	fTotalStrokes	 20
U+3B2B.0	fCangjie	TLPA
U+3B2B.0	fFourCorner	44600
U+3B2B.0	fCantonese	jin3; zin3
U+3B2B.0	fMandarin	yan4
U+3B2B.0	fEnglish	clear sky; a fine day
U+3B2C.0	fUTF8	㬬
U+3B2C.0	fCNS11643	3-5F4C
U+3B2C.0	fR/S	 72.18
U+3B2C.0	fTotalStrokes	 22
U+3B2C.0	fCangjie	ABUG
U+3B2C.0	fFourCorner	66010
U+3B2C.0	fHakka	 ki3
U+3B2C.0	fCantonese	keoi4; ak1; geoi2; geoi6
U+3B2C.0	fMandarin	ju4
U+3B2C.0	fEnglish	a family name
U+3B2D.0	fUTF8	㬭
U+3B2D.0	fCNS11643	3-5F4D
U+3B2D.0	fGB	5-4A35
U+3B2D.0	fR/S	 72.18; 72.17
U+3B2D.0	fTotalStrokes	 22; 21
U+3B2D.0	fCangjie	ABWI
U+3B2D.0	fFourCorner	62040
U+3B2D.0	fCantonese	zoek3; ziu3 (also); ziu6 (also)
U+3B2D.0	fMandarin	jiao4; shan3; yao3
U+3B2D.0	fEnglish	non-classical form of &#x76AD;
U+3B2E.0	fUTF8	㬮
U+3B2E.0	fCNS11643	5-756E
U+3B2E.0	fGB	3-4828
U+3B2E.0	fR/S	 72.19
U+3B2E.0	fTotalStrokes	 23
U+3B2E.0	fCangjie	ATOG
U+3B2E.0	fFourCorner	60510
U+3B2E.0	fCantonese	naan6; laan6 (also); nan6
U+3B2E.0	fMandarin	nan4; kuai4
U+3B2E.0	fEnglish	[1] gentle; mild; temperate [2] tender; loving; caressing
U+3B2F.0	fUTF8	㬯
U+3B2F.0	fCNS11643	5-756D
U+3B2F.0	fR/S	 72.19; 72.21
U+3B2F.0	fTotalStrokes	 23; 25
U+3B2F.0	fCangjie	AYNQ
U+3B2F.0	fFourCorner	60010
U+3B2F.0	fCantonese	lip6; lit6
U+3B2F.0	fMandarin	lie4
U+3B2F.0	fEnglish	sunset
U+3B30.0	fUTF8	㬰
U+3B30.0	fCNS11643	3-2374
U+3B30.0	fR/S	 73.2
U+3B30.0	fTotalStrokes	 6
U+3B30.0	fCangjie	LWO
U+3B30.0	fFourCorner	50030
U+3B30.0	fHakka	 i2; ji2
U+3B30.0	fCantonese	jyu4; zyu4
U+3B30.0	fMandarin	yu2; yong3; kui4; yu3
U+3B30.0	fEnglish	non-classical form of &#x81FE;
U+3B31.0	fUTF8	㬱
U+3B31.0	fCNS11643	4-5056
U+3B31.0	fR/S	 73.12
U+3B31.0	fTotalStrokes	 16
U+3B31.0	fCangjie	HUA
U+3B31.0	fFourCorner	24600
U+3B31.0	fHakka	 cam3
U+3B31.0	fCantonese	tai3
U+3B31.0	fMandarin	ti4
U+3B31.0	fEnglish	[1] if; supposing [2] nevertheless
U+3B32.0	fUTF8	㬲
U+3B32.0	fCNS11643	3-5474
U+3B32.0	fR/S	 73.13
U+3B32.0	fTotalStrokes	 17
U+3B32.0	fCangjie	MKTCT
U+3B32.0	fFourCorner	18500
U+3B32.0	fCantonese	tim1
U+3B32.0	fMandarin	tian1
U+3B32.0	fEnglish	[1] increase; add to; augment [2] a greater extent; more [3] benefit; profit; advantage
U+3B33.0	fUTF8	㬳
U+3B33.0	fCNS11643	3-2944
U+3B33.0	fR/S	 74.4
U+3B33.0	fTotalStrokes	 8
U+3B33.0	fCangjie	BOJ
U+3B33.0	fFourCorner	78240
U+3B33.0	fCantonese	ng5
U+3B33.0	fMandarin	wu3
U+3B33.0	fEnglish	[1] light; bright [2] clear
U+3B34.0	fUTF8	㬴
U+3B34.0	fCNS11643	3-314E
U+3B34.0	fR/S	 74.6
U+3B34.0	fTotalStrokes	 10
U+3B34.0	fCangjie	BTC
U+3B34.0	fFourCorner	74280
U+3B34.0	fCantonese	hung2; fung2
U+3B34.0	fMandarin	hong3
U+3B34.0	fEnglish	[1] the moon is dimmed; darkness [2] the word used especially by Triad Society ( a secret society during the Ching Dynasty dedicated to the overthrow of the Manchus and the restoration the Manchus and the restoration of the Ming Dynasty
U+3B35.0	fUTF8	㬵
U+3B35.0	fCNS11643	5-2E25
U+3B35.0	fR/S	 74.6
U+3B35.0	fTotalStrokes	 10
U+3B35.0	fCangjie	BYCK
U+3B35.0	fFourCorner	70240
U+3B35.0	fHakka	 gau1
U+3B35.0	fCantonese	gaau1
U+3B35.0	fMandarin	jiao1; xiao2
U+3B35.0	fEnglish	same as &#x4EA4;
U+3B36.0	fUTF8	㬶
U+3B36.0	fCNS11643	3-3649
U+3B36.0	fR/S	 74.7
U+3B36.0	fTotalStrokes	 11
U+3B36.0	fCangjie	BHGR
U+3B36.0	fFourCorner	74260
U+3B36.0	fHakka	 hau3
U+3B36.0	fCantonese	hou6; hou5
U+3B36.0	fMandarin	hao4; gao3; ge2; gao4
U+3B36.0	fEnglish	same as &#x6D69;
U+3B37.0	fUTF8	㬷
U+3B37.0	fGB	S-2146
U+3B37.0	fR/S	 74.7
U+3B37.0	fTotalStrokes	 11
U+3B37.0	fCangjie	BWD
U+3B37.0	fCantonese	kwan3
U+3B38.0	fUTF8	㬸
U+3B38.0	fCNS11643	3-3C28
U+3B38.0	fR/S	 74.8
U+3B38.0	fTotalStrokes	 12
U+3B38.0	fCangjie	BTOP
U+3B38.0	fFourCorner	74210
U+3B38.0	fCantonese	tiu1
U+3B38.0	fMandarin	tiao1
U+3B38.0	fEnglish	a kind of services or ceremonies of offering sacrifices
U+3B39.0	fUTF8	㬹
U+3B39.0	fCNS11643	4-3A43
U+3B39.0	fGB	S-2140
U+3B39.0	fR/S	 130.8
U+3B39.0	fTotalStrokes	 12
U+3B39.0	fCangjie	BBSD
U+3B39.0	fFourCorner	72250
U+3B39.0	fHakka	 zang1
U+3B39.0	fCantonese	zaang1
U+3B39.0	fMandarin	zheng1
U+3B39.0	fEnglish	[1] elbow [2] the heel
U+3B3A.0	fUTF8	㬺
U+3B3A.0	fR/S	 74.10
U+3B3A.0	fTotalStrokes	 14
U+3B3A.0	fCangjie	BFQB
U+3B3A.0	fCantonese	tang4
U+3B3A.0	fEnglish	[Korean character]
U+3B3B.0	fUTF8	㬻
U+3B3B.0	fCNS11643	5-4645
U+3B3B.0	fR/S	 74.10
U+3B3B.0	fTotalStrokes	 14
U+3B3B.0	fCangjie	BTYU
U+3B3B.0	fFourCorner	74210
U+3B3B.0	fCantonese	fong1; baang1
U+3B3B.0	fMandarin	huang3
U+3B3B.0	fEnglish	[1] in between of the flesh [2] a state of a minority ethnic group in Southern China [3] the moon is dimmed
U+3B3C.0	fUTF8	㬼
U+3B3C.0	fCNS11643	5-4D58
U+3B3C.0	fR/S	 74.11
U+3B3C.0	fTotalStrokes	 15
U+3B3C.0	fCangjie	KBOAE
U+3B3C.0	fFourCorner	48240
U+3B3C.0	fCantonese	fuk6; wak6
U+3B3C.0	fMandarin	fu2; fu4
U+3B3C.0	fEnglish	 have; be present; exist; there is
U+3B3D.0	fUTF8	㬽
U+3B3D.0	fGB	S-2142
U+3B3D.0	fR/S	 74.11
U+3B3D.0	fTotalStrokes	 15
U+3B3D.0	fCangjie	BRRS
U+3B3D.0	fCantonese	ngok6; ok6 (also)
U+3B3E.0	fUTF8	㬾
U+3B3E.0	fGB	S-2141
U+3B3E.0	fR/S	 74.11
U+3B3E.0	fTotalStrokes	 15
U+3B3E.0	fCangjie	SEB
U+3B3E.0	fCantonese	tyun4
U+3B3F.0	fUTF8	㬿
U+3B3F.0	fCNS11643	3-5125
U+3B3F.0	fR/S	 74.12
U+3B3F.0	fTotalStrokes	 16
U+3B3F.0	fCangjie	BYDK
U+3B3F.0	fFourCorner	78240
U+3B3F.0	fHakka	 dun1
U+3B3F.0	fCantonese	tan1; daan6
U+3B3F.0	fMandarin	tun1; dun4
U+3B3F.0	fEnglish	moonlight
U+3B40.0	fUTF8	㭀
U+3B40.0	fGB	S-2145
U+3B40.0	fR/S	 74.13
U+3B40.0	fTotalStrokes	 17
U+3B40.0	fCangjie	BHMO
U+3B40.0	fCantonese	sai6
U+3B41.0	fUTF8	㭁
U+3B41.0	fCNS11643	3-2376
U+3B41.0	fGB	5-433B
U+3B41.0	fR/S	 75.2
U+3B41.0	fTotalStrokes	 6
U+3B41.0	fCangjie	DNHS
U+3B41.0	fFourCorner	47920
U+3B41.0	fCantonese	jing4; zing4
U+3B41.0	fMandarin	reng2
U+3B41.0	fEnglish	[1] a kind of tree [2] a kind of lumber used to stop a carriage
U+3B42.0	fUTF8	㭂
U+3B42.0	fCNS11643	3-2643
U+3B42.0	fR/S	 75.3
U+3B42.0	fTotalStrokes	 7
U+3B42.0	fCangjie	DF
U+3B42.0	fFourCorner	49900
U+3B42.0	fCantonese	niu1; liu1 (also); ziu2
U+3B42.0	fMandarin	jiao3
U+3B42.0	fEnglish	[1] height of the end-point [2] slender; thin
U+3B43.0	fUTF8	㭃
U+3B43.0	fCNS11643	4-2448
U+3B43.0	fR/S	 75.3
U+3B43.0	fTotalStrokes	 7
U+3B43.0	fCangjie	DVI
U+3B43.0	fFourCorner	42930
U+3B43.0	fHakka	 gong1
U+3B43.0	fCantonese	gong1
U+3B43.0	fMandarin	gang1
U+3B43.0	fEnglish	ancient form of &#x7DB1;
U+3B44.0	fUTF8	㭄
U+3B44.0	fCNS11643	3-2646
U+3B44.0	fR/S	 75.3
U+3B44.0	fTotalStrokes	 7
U+3B44.0	fCangjie	DNJ
U+3B44.0	fFourCorner	47910
U+3B44.0	fCantonese	seon3
U+3B44.0	fMandarin	chen1; xin4
U+3B44.0	fEnglish	a kind of tree
U+3B45.0	fUTF8	㭅
U+3B45.0	fR/S	 75.3
U+3B45.0	fTotalStrokes	 7
U+3B45.0	fCangjie	DMMM
U+3B45.0	fCantonese	saam1
U+3B45.0	fEnglish	[Korean character]
U+3B46.0	fUTF8	㭆
U+3B46.0	fR/S	 75.3
U+3B46.0	fTotalStrokes	 7
U+3B46.0	fCangjie	NSDJ
U+3B46.0	fCantonese	naai5; laai5 (also)
U+3B46.0	fEnglish	[Korean character]
U+3B47.0	fUTF8	㭇
U+3B47.0	fCNS11643	3-294B
U+3B47.0	fGB	5-434F
U+3B47.0	fR/S	 75.4
U+3B47.0	fTotalStrokes	 8
U+3B47.0	fCangjie	DIHU
U+3B47.0	fFourCorner	43910
U+3B47.0	fCantonese	jyun6; zyun2
U+3B47.0	fMandarin	yuan4
U+3B47.0	fEnglish	[1] a kind of tree [2] corrupted form of &#x676C; [3] non-classical form of &#x6B16;
U+3B48.0	fUTF8	㭈
U+3B48.0	fCNS11643	3-2946
U+3B48.0	fGB	5-434D
U+3B48.0	fR/S	 75.4
U+3B48.0	fTotalStrokes	 8
U+3B48.0	fCangjie	DDK
U+3B48.0	fFourCorner	45930
U+3B48.0	fCantonese	kyut3; lai6
U+3B48.0	fMandarin	jue2; kuai4
U+3B48.0	fEnglish	[1] basin [2] bowl
U+3B49.0	fUTF8	㭉
U+3B49.0	fCNS11643	F-274C
U+3B49.0	fGB	5-6748
U+3B49.0	fR/S	 75.4
U+3B49.0	fTotalStrokes	 8
U+3B49.0	fCangjie	TD
U+3B49.0	fCantonese	waa4
U+3B49.0	fMandarin	hua2
U+3B49.0	fEnglish	farm tools; a spade or shovel
U+3B4A.0	fUTF8	㭊
U+3B4A.0	fCNS11643	3-2950
U+3B4A.0	fGB	5-4346
U+3B4A.0	fR/S	 75.4
U+3B4A.0	fTotalStrokes	 8
U+3B4A.0	fCangjie	DLLL
U+3B4A.0	fFourCorner	42920
U+3B4A.0	fCantonese	sik1; cik1 (also)
U+3B4A.0	fMandarin	xi1
U+3B4A.0	fEnglish	non-classical form of &#x6790;
U+3B4B.0	fUTF8	㭋
U+3B4B.0	fCNS11643	3-294D
U+3B4B.0	fR/S	 75.4
U+3B4B.0	fTotalStrokes	 8
U+3B4B.0	fCangjie	DQJ
U+3B4B.0	fFourCorner	45900
U+3B4B.0	fCantonese	paang5; bang5; faai3
U+3B4B.0	fMandarin	bang4
U+3B4B.0	fEnglish	same as &#x68D2;
U+3B4C.0	fUTF8	㭌
U+3B4C.0	fCNS11643	5-2651
U+3B4C.0	fR/S	 75.4
U+3B4C.0	fTotalStrokes	 8
U+3B4C.0	fCangjie	DHQ
U+3B4C.0	fFourCorner	45900
U+3B4C.0	fCantonese	jyu4; zyu1
U+3B4C.0	fMandarin	mou2; yu2
U+3B4C.0	fEnglish	[1] same as &#x6859; [2] interchangeable with &#x6A21;
U+3B4D.0	fUTF8	㭍
U+3B4D.0	fGB	S-2177
U+3B4D.0	fR/S	 75.4
U+3B4D.0	fTotalStrokes	 8
U+3B4D.0	fCangjie	IPD
U+3B4D.0	fCantonese	cat1; caat1
U+3B4D.0	fEnglish	capital form of seven
U+3B4E.0	fUTF8	㭎
U+3B4E.0	fGB	7-2225
U+3B4E.0	fR/S	 75.4
U+3B4E.0	fTotalStrokes	 8
U+3B4E.0	fCangjie	DBK
U+3B4E.0	fCantonese	gong1; gong3 (also)
U+3B4E.0	fEnglish	[1] simplified form of &#x68E1; [2] interchangeable with &#x625B;
U+3B4F.0	fUTF8	㭏
U+3B4F.0	fGB	7-234A
U+3B4F.0	fR/S	 75.4
U+3B4F.0	fTotalStrokes	 8
U+3B4F.0	fCangjie	DQS
U+3B4F.0	fCantonese	wai5; waai5
U+3B4F.0	fMandarin	wei3
U+3B4F.0	fEnglish	[1] simplified form of &#x6932; [2] same as &#x694E;
U+3B50.0	fUTF8	㭐
U+3B50.0	fR/S	 75.4
U+3B50.0	fTotalStrokes	 8
U+3B50.0	fCangjie	KDJ
U+3B50.0	fCantonese	daai6
U+3B50.0	fEnglish	[Korean character]
U+3B51.0	fUTF8	㭑
U+3B51.0	fCNS11643	3-2D28
U+3B51.0	fR/S	 75.5
U+3B51.0	fTotalStrokes	 9
U+3B51.0	fCangjie	DJD
U+3B51.0	fFourCorner	45990
U+3B51.0	fCantonese	mui6
U+3B51.0	fMandarin	wei4; mei4
U+3B51.0	fEnglish	a kind of tree
U+3B52.0	fUTF8	㭒
U+3B52.0	fCNS11643	3-2D2C
U+3B52.0	fGB	5-4359
U+3B52.0	fR/S	 75.5
U+3B52.0	fTotalStrokes	 9
U+3B52.0	fCangjie	DRLR
U+3B52.0	fFourCorner	47970
U+3B52.0	fCantonese	zi6; zi2
U+3B52.0	fMandarin	si4
U+3B52.0	fEnglish	same as non-classical form &#x801C;
U+3B53.0	fUTF8	㭓
U+3B53.0	fCNS11643	4-296F
U+3B53.0	fR/S	 75.5
U+3B53.0	fTotalStrokes	 9
U+3B53.0	fCangjie	DIT
U+3B53.0	fFourCorner	43940
U+3B53.0	fCantonese	bin6
U+3B53.0	fMandarin	bian4
U+3B53.0	fEnglish	a pillar arch
U+3B54.0	fUTF8	㭔
U+3B54.0	fCNS11643	4-296C
U+3B54.0	fGB	3-4075
U+3B54.0	fR/S	 75.5
U+3B54.0	fTotalStrokes	 9
U+3B54.0	fCangjie	DHFD
U+3B54.0	fFourCorner	42940
U+3B54.0	fCantonese	lou4
U+3B54.0	fMandarin	lu2
U+3B54.0	fEnglish	a kind of tree
U+3B55.0	fUTF8	㭕
U+3B55.0	fCNS11643	3-2D29
U+3B55.0	fGB	5-4351
U+3B55.0	fR/S	 75.5
U+3B55.0	fTotalStrokes	 9
U+3B55.0	fCangjie	DGI
U+3B55.0	fFourCorner	44930
U+3B55.0	fCantonese	keoi1
U+3B55.0	fMandarin	qu1
U+3B55.0	fEnglish	a board (on the back of a donkey) for carrying things; saddle
U+3B56.0	fUTF8	㭖
U+3B56.0	fGB	S-216C
U+3B56.0	fR/S	 75.5
U+3B56.0	fTotalStrokes	 9
U+3B56.0	fCangjie	DOIP
U+3B56.0	fCantonese	doi6
U+3B57.0	fUTF8	㭗
U+3B57.0	fR/S	 75.5
U+3B57.0	fTotalStrokes	 9
U+3B57.0	fCangjie	DBT
U+3B57.0	fCantonese	ming5
U+3B57.0	fEnglish	[Korean character]
U+3B58.0	fUTF8	㭘
U+3B58.0	fCNS11643	4-2D57
U+3B58.0	fGB	5-436D
U+3B58.0	fR/S	 75.6
U+3B58.0	fTotalStrokes	 10
U+3B58.0	fCangjie	DOMR
U+3B58.0	fFourCorner	48960
U+3B58.0	fCantonese	gap3; haap6
U+3B58.0	fMandarin	xia2; he2
U+3B58.0	fEnglish	[1] scabbard; sheath; a case for sword [2] a tree like the acacia
U+3B59.0	fUTF8	㭙
U+3B59.0	fCNS11643	4-2D51
U+3B59.0	fGB	5-4362
U+3B59.0	fR/S	 75.6
U+3B59.0	fTotalStrokes	 10
U+3B59.0	fCangjie	DGDI
U+3B59.0	fFourCorner	44940
U+3B59.0	fCantonese	zaak6
U+3B59.0	fMandarin	zhe2; she2
U+3B59.0	fEnglish	 hammer
U+3B5A.0	fUTF8	㭚
U+3B5A.0	fCNS11643	4-2D54
U+3B5A.0	fGB	3-4132
U+3B5A.0	fR/S	 75.6
U+3B5A.0	fTotalStrokes	 10
U+3B5A.0	fCangjie	DOHV
U+3B5A.0	fFourCorner	48930
U+3B5A.0	fHakka	  li3
U+3B5A.0	fCantonese	leoi5
U+3B5A.0	fMandarin	luu3; l3
U+3B5A.0	fEnglish	a tough kind of wood suitable for arrows
U+3B5B.0	fUTF8	㭛
U+3B5B.0	fCNS11643	4-2D58
U+3B5B.0	fGB	3-4137
U+3B5B.0	fR/S	 75.6
U+3B5B.0	fTotalStrokes	 10
U+3B5B.0	fCangjie	DHHV
U+3B5B.0	fFourCorner	42930
U+3B5B.0	fCantonese	paai3
U+3B5B.0	fMandarin	pai4
U+3B5B.0	fEnglish	[1] a kind of climbing plants; rattan; bark can be used to weave cloth [2] a component part of a loom
U+3B5C.0	fUTF8	㭜
U+3B5C.0	fCNS11643	3-3152
U+3B5C.0	fGB	5-4361
U+3B5C.0	fR/S	 75.6
U+3B5C.0	fTotalStrokes	 10
U+3B5C.0	fCangjie	DIJ
U+3B5C.0	fFourCorner	43950
U+3B5C.0	fCantonese	jung4; zung4
U+3B5C.0	fMandarin	rong2
U+3B5C.0	fEnglish	a kind of tree (like locust tree, similar to the ash, acacia)
U+3B5D.0	fUTF8	㭝
U+3B5D.0	fCNS11643	5-2E2B
U+3B5D.0	fR/S	 75.6
U+3B5D.0	fTotalStrokes	 10
U+3B5D.0	fCangjie	DDKN
U+3B5D.0	fFourCorner	44410
U+3B5D.0	fCantonese	kau4
U+3B5D.0	fMandarin	qiu2
U+3B5D.0	fEnglish	[1] the name of a pavilion in Xinshi [2] the name of a place in Jingling
U+3B5E.0	fUTF8	㭞
U+3B5E.0	fCNS11643	4-2D52
U+3B5E.0	fR/S	 75.6
U+3B5E.0	fTotalStrokes	 10
U+3B5E.0	fCangjie	DFHS
U+3B5E.0	fFourCorner	49920
U+3B5E.0	fCantonese	lyut3
U+3B5E.0	fMandarin	lie4
U+3B5E.0	fEnglish	a kind inferior wood
U+3B5F.0	fUTF8	㭟
U+3B5F.0	fCNS11643	4-2D5B
U+3B5F.0	fR/S	 75.6
U+3B5F.0	fTotalStrokes	 10
U+3B5F.0	fCangjie	TCD
U+3B5F.0	fFourCorner	44900
U+3B5F.0	fCantonese	gung2
U+3B5F.0	fMandarin	gong3
U+3B5F.0	fEnglish	torture used in ancient times
U+3B60.0	fUTF8	㭠
U+3B60.0	fCNS11643	3-3155
U+3B60.0	fR/S	 75.6
U+3B60.0	fTotalStrokes	 10
U+3B60.0	fCangjie	DHGU
U+3B60.0	fFourCorner	44910
U+3B60.0	fCantonese	sin2
U+3B60.0	fMandarin	xian3
U+3B61.0	fUTF8	㭡
U+3B61.0	fCNS11643	4-2D56
U+3B61.0	fGB	3-4136
U+3B61.0	fR/S	 75.6
U+3B61.0	fTotalStrokes	 10
U+3B61.0	fCangjie	DHWK
U+3B61.0	fFourCorner	46900
U+3B61.0	fCantonese	sai3
U+3B61.0	fMandarin	xi4; xin4
U+3B61.0	fEnglish	[1] the name of a mountain [2] a component parts of a loom
U+3B62.0	fUTF8	㭢
U+3B62.0	fCNS11643	3-3151
U+3B62.0	fR/S	 75.6
U+3B62.0	fTotalStrokes	 10
U+3B62.0	fCangjie	DMTN
U+3B62.0	fFourCorner	42900
U+3B62.0	fCantonese	hing1
U+3B62.0	fMandarin	xing1
U+3B62.0	fEnglish	[1] mechanics [2] opportunity [3] urgent
U+3B63.0	fUTF8	㭣
U+3B63.0	fGB	S-2169
U+3B63.0	fR/S	 75.6
U+3B63.0	fTotalStrokes	 10
U+3B63.0	fCangjie	DLSR
U+3B63.0	fCantonese	man6
U+3B64.0	fUTF8	㭤
U+3B64.0	fGB	7-2349
U+3B64.0	fR/S	 75.6
U+3B64.0	fTotalStrokes	 10
U+3B64.0	fCangjie	DPYM
U+3B64.0	fCantonese	niu5; liu5 (also)
U+3B64.0	fMandarin	niao3
U+3B64.0	fEnglish	simplified form of &#x8526;
U+3B65.0	fUTF8	㭥
U+3B65.0	fGB	7-2352
U+3B65.0	fR/S	 75.6
U+3B65.0	fTotalStrokes	 10
U+3B65.0	fCangjie	DNBO
U+3B65.0	fCantonese	fu6
U+3B66.0	fUTF8	㭦
U+3B66.0	fR/S	 75.6
U+3B66.0	fTotalStrokes	 10
U+3B66.0	fCangjie	DJHP
U+3B66.0	fCantonese	zaak6
U+3B66.0	fEnglish	[Korean character]
U+3B67.0	fUTF8	㭧
U+3B67.0	fGB	S-213D
U+3B67.0	fR/S	 75.6
U+3B67.0	fTotalStrokes	 10
U+3B67.0	fCangjie	HYD
U+3B67.0	fCantonese	zau1
U+3B68.0	fUTF8	㭨
U+3B68.0	fCNS11643	3-3659
U+3B68.0	fGB	5-4368
U+3B68.0	fR/S	 75.7
U+3B68.0	fTotalStrokes	 11
U+3B68.0	fCangjie	DMHL
U+3B68.0	fFourCorner	47920
U+3B68.0	fHakka	 ja2; ra2; jai2; ye2
U+3B68.0	fCantonese	je4
U+3B68.0	fMandarin	ye2; xie2
U+3B68.0	fEnglish	the cocca-nut tree
U+3B69.0	fUTF8	㭩
U+3B69.0	fCNS11643	4-3243
U+3B69.0	fGB	3-4160
U+3B69.0	fR/S	 75.7
U+3B69.0	fTotalStrokes	 11
U+3B69.0	fCangjie	DBDI
U+3B69.0	fFourCorner	42940
U+3B69.0	fCantonese	lyut6; lyut3
U+3B69.0	fMandarin	lie4; lei4; ling2
U+3B69.0	fEnglish	a kind of tree can be used as dye-stuff, the mast of a boat
U+3B6A.0	fUTF8	㭪
U+3B6A.0	fCNS11643	3-364A
U+3B6A.0	fGB	7-234F
U+3B6A.0	fR/S	 75.7
U+3B6A.0	fTotalStrokes	 11
U+3B6A.0	fCangjie	DIJB
U+3B6A.0	fFourCorner	43920
U+3B6A.0	fCantonese	fu1; gu1
U+3B6A.0	fMandarin	fu1
U+3B6A.0	fEnglish	[1] a shrub (plant) [2] dense growth of trees
U+3B6B.0	fUTF8	㭫
U+3B6B.0	fCNS11643	3-364F
U+3B6B.0	fGB	5-4423
U+3B6B.0	fR/S	 75.7
U+3B6B.0	fTotalStrokes	 11
U+3B6B.0	fCangjie	DOOG
U+3B6B.0	fFourCorner	48910
U+3B6B.0	fCantonese	co4; cyun4
U+3B6B.0	fMandarin	cuo2; cuan2; zhen4
U+3B6B.0	fEnglish	a fruit tree ( plum), (same as &#x83C6;) hemp stalks, a mat
U+3B6C.0	fUTF8	㭬
U+3B6C.0	fCNS11643	5-3279
U+3B6C.0	fR/S	 75.7
U+3B6C.0	fTotalStrokes	 11
U+3B6C.0	fCangjie	DMSO
U+3B6C.0	fFourCorner	41930
U+3B6C.0	fHakka	 zhu1; zu1
U+3B6C.0	fCantonese	doek3; tou5; zyu1
U+3B6C.0	fMandarin	zhu1; zhuo2
U+3B6C.0	fEnglish	 dead tree
U+3B6D.0	fUTF8	㭭
U+3B6D.0	fCNS11643	4-3242
U+3B6D.0	fGB	3-415A
U+3B6D.0	fR/S	 75.7
U+3B6D.0	fTotalStrokes	 11
U+3B6D.0	fCangjie	DRSN
U+3B6D.0	fFourCorner	42900
U+3B6D.0	fHakka	 bat8; pat8
U+3B6D.0	fCantonese	baat3
U+3B6D.0	fMandarin	ba1; fei4
U+3B6D.0	fEnglish	the name of a common fruit (guave)
U+3B6E.0	fUTF8	㭮
U+3B6E.0	fCNS11643	4-3246
U+3B6E.0	fR/S	 75.7
U+3B6E.0	fTotalStrokes	 11
U+3B6E.0	fCangjie	DVVN
U+3B6E.0	fFourCorner	42920
U+3B6E.0	fHakka	  zat7
U+3B6E.0	fCantonese	zaat3; cyut3; dung6; waai3; zat3
U+3B6E.0	fMandarin	zuo3; zuo4
U+3B6E.0	fEnglish	pieces of wood used to squeeze the fingers of prisoners, to extort evidence from them
U+3B6F.0	fUTF8	㭯
U+3B6F.0	fCNS11643	4-3241
U+3B6F.0	fGB	3-4148
U+3B6F.0	fR/S	 75.7
U+3B6F.0	fTotalStrokes	 11
U+3B6F.0	fCangjie	DSJU
U+3B6F.0	fFourCorner	41910
U+3B6F.0	fCantonese	zip3; cit3
U+3B6F.0	fMandarin	che4; die2; na4; zhe2
U+3B6F.0	fEnglish	[1] tree with small leaves [2] mushroom; fungus [3] mould; mildew
U+3B70.0	fUTF8	㭰
U+3B70.0	fCNS11643	4-3834
U+3B70.0	fGB	5-4867
U+3B70.0	fR/S	 75.7; 75.8
U+3B70.0	fTotalStrokes	 12; 11
U+3B70.0	fCangjie	YPDB
U+3B70.0	fFourCorner	21900
U+3B70.0	fHakka	  zui3
U+3B70.0	fCantonese	zeoi2; zyu1
U+3B70.0	fMandarin	zui3; ji4; jue2
U+3B70.0	fEnglish	stone needles used in accupuncture
U+3B71.0	fUTF8	㭱
U+3B71.0	fCNS11643	3-3658
U+3B71.0	fR/S	 75.7
U+3B71.0	fTotalStrokes	 11
U+3B71.0	fCangjie	DSWL
U+3B71.0	fFourCorner	41910
U+3B71.0	fHakka	 ap7; hap7
U+3B71.0	fCantonese	haap6
U+3B71.0	fMandarin	xia2; he2
U+3B71.0	fEnglish	same as &#x5323;
U+3B72.0	fUTF8	㭲
U+3B72.0	fCNS11643	3-365D
U+3B72.0	fR/S	 75.7
U+3B72.0	fTotalStrokes	 11
U+3B72.0	fCangjie	DCOR
U+3B72.0	fFourCorner	48960
U+3B72.0	fHakka	 kip7
U+3B72.0	fCantonese	gik6; kap1; zin1
U+3B72.0	fMandarin	ji2
U+3B72.0	fEnglish	same as &#x6781;
U+3B73.0	fUTF8	㭳
U+3B73.0	fGB	S-216D
U+3B73.0	fR/S	 75.7
U+3B73.0	fTotalStrokes	 11
U+3B73.0	fCangjie	DJKD
U+3B73.0	fCantonese	haau3
U+3B74.0	fUTF8	㭴
U+3B74.0	fR/S	 75.7
U+3B74.0	fTotalStrokes	 11
U+3B74.0	fCangjie	DLEG
U+3B74.0	fCantonese	gin1
U+3B74.0	fEnglish	the oscines
U+3B75.0	fUTF8	㭵
U+3B75.0	fGB	7-234C
U+3B75.0	fR/S	 75.7
U+3B75.0	fTotalStrokes	 11
U+3B75.0	fCangjie	DPW
U+3B75.0	fCantonese	din6
U+3B76.0	fUTF8	㭶
U+3B76.0	fGB	7-2351
U+3B76.0	fR/S	 75.8
U+3B76.0	fTotalStrokes	 12
U+3B76.0	fCangjie	DTYS
U+3B76.0	fCantonese	fong1
U+3B77.0	fUTF8	㭷
U+3B77.0	fGB	7-2353
U+3B77.0	fR/S	 75.7
U+3B77.0	fTotalStrokes	 11
U+3B77.0	fCangjie	DWKS
U+3B77.0	fCantonese	naam4; laam4 (also)
U+3B78.0	fUTF8	㭸
U+3B78.0	fCNS11643	4-382F
U+3B78.0	fGB	5-4443
U+3B78.0	fR/S	 75.8
U+3B78.0	fTotalStrokes	 12
U+3B78.0	fCangjie	DNAI
U+3B78.0	fFourCorner	47910
U+3B78.0	fCantonese	tou4; tung4; zak6
U+3B78.0	fMandarin	tu2
U+3B78.0	fEnglish	same as &#x688C;
U+3B79.0	fUTF8	㭹
U+3B79.0	fCNS11643	3-3C2E
U+3B79.0	fR/S	 75.8
U+3B79.0	fTotalStrokes	 12
U+3B79.0	fCangjie	DNYI
U+3B79.0	fFourCorner	40930
U+3B79.0	fCantonese	jin4; zin4
U+3B79.0	fMandarin	xian2
U+3B79.0	fEnglish	[1] the name of a county in today's Shandong province [2] a kind of fabric produced in Donglai
U+3B7A.0	fUTF8	㭺
U+3B7A.0	fCNS11643	4-382C
U+3B7A.0	fGB	3-4170
U+3B7A.0	fR/S	 75.8
U+3B7A.0	fTotalStrokes	 12
U+3B7A.0	fCangjie	DKLU
U+3B7A.0	fFourCorner	44910
U+3B7A.0	fCantonese	jim2; zim2
U+3B7A.0	fMandarin	yan3; yan1; an1
U+3B7A.0	fEnglish	[1] the cherry-apple [2] a kind of tree
U+3B7B.0	fUTF8	㭻
U+3B7B.0	fCNS11643	3-3C3F
U+3B7B.0	fGB	5-4439
U+3B7B.0	fR/S	 75.8
U+3B7B.0	fTotalStrokes	 12
U+3B7B.0	fCangjie	DFBR
U+3B7B.0	fFourCorner	49920
U+3B7B.0	fHakka	 tong2
U+3B7B.0	fCantonese	tong4
U+3B7B.0	fMandarin	tang2
U+3B7B.0	fEnglish	same as &#x68E0;
U+3B7C.0	fUTF8	㭼
U+3B7C.0	fCNS11643	3-3C30
U+3B7C.0	fGB	5-444B
U+3B7C.0	fR/S	 75.8
U+3B7C.0	fTotalStrokes	 12
U+3B7C.0	fCangjie	DEA
U+3B7C.0	fFourCorner	42960
U+3B7C.0	fHakka	  tap7
U+3B7C.0	fCantonese	taap3; dap6
U+3B7C.0	fMandarin	ta4
U+3B7C.0	fEnglish	the capital of a column
U+3B7D.0	fUTF8	㭽
U+3B7D.0	fCNS11643	4-382A
U+3B7D.0	fGB	3-417D
U+3B7D.0	fR/S	 75.8
U+3B7D.0	fTotalStrokes	 12
U+3B7D.0	fCangjie	DIHI
U+3B7D.0	fFourCorner	40940
U+3B7D.0	fCantonese	dai2
U+3B7D.0	fMandarin	di3
U+3B7D.0	fEnglish	non-classical form of &#x67E2;
U+3B7E.0	fUTF8	㭾
U+3B7E.0	fCNS11643	5-3933
U+3B7E.0	fR/S	 75.8
U+3B7E.0	fTotalStrokes	 12
U+3B7E.0	fCangjie	DSUU
U+3B7E.0	fFourCorner	47970
U+3B7E.0	fCantonese	gwat6
U+3B7E.0	fMandarin	jue2; yue4
U+3B7E.0	fEnglish	 cut the timber apart; cut a tree
U+3B7F.0	fUTF8	㭿
U+3B7F.0	fCNS11643	4-382D
U+3B7F.0	fGB	3-4175
U+3B7F.0	fR/S	 75.8
U+3B7F.0	fTotalStrokes	 12
U+3B7F.0	fCangjie	DAHL
U+3B7F.0	fFourCorner	46920
U+3B7F.0	fHakka	  ngong1
U+3B7F.0	fCantonese	ngong4; ong4 (also)
U+3B7F.0	fMandarin	ang2
U+3B7F.0	fEnglish	the turned-up eaves of a Chinese roof
U+3B80.0	fUTF8	㮀
U+3B80.0	fCNS11643	3-3C2C
U+3B80.0	fR/S	 75.8
U+3B80.0	fTotalStrokes	 12
U+3B80.0	fCangjie	DUNE
U+3B80.0	fFourCorner	47970
U+3B80.0	fCantonese	haam4
U+3B80.0	fMandarin	han2
U+3B80.0	fEnglish	same as &#x3BAD;
U+3B81.0	fUTF8	㮁
U+3B81.0	fCNS11643	5-3939
U+3B81.0	fR/S	 75.8
U+3B81.0	fTotalStrokes	 12
U+3B81.0	fCangjie	DKKB
U+3B81.0	fFourCorner	44920
U+3B81.0	fCantonese	haau4; ngau4
U+3B81.0	fMandarin	yao2
U+3B81.0	fEnglish	gardenia, a plant of which the nuts produces a yellow dye
U+3B82.0	fUTF8	㮂
U+3B82.0	fCNS11643	4-3835
U+3B82.0	fGB	3-4174
U+3B82.0	fR/S	 75.8
U+3B82.0	fTotalStrokes	 12
U+3B82.0	fCangjie	BUKD
U+3B82.0	fFourCorner	60900
U+3B82.0	fCantonese	guk1
U+3B82.0	fMandarin	ju1; ju2
U+3B82.0	fEnglish	[1] an appliance used to lift food; something like a narrow, long table [2] carry; transport
U+3B83.0	fUTF8	㮃
U+3B83.0	fCNS11643	3-3C35
U+3B83.0	fGB	5-443D
U+3B83.0	fR/S	 75.8
U+3B83.0	fTotalStrokes	 12
U+3B83.0	fCangjie	DHDV
U+3B83.0	fFourCorner	42940
U+3B83.0	fHakka	 wui1; wi1; vui1
U+3B83.0	fCantonese	wai1
U+3B83.0	fMandarin	wei1; rui2
U+3B83.0	fEnglish	the name of a shrub
U+3B84.0	fUTF8	㮄
U+3B84.0	fCNS11643	4-3837
U+3B84.0	fR/S	 75.8
U+3B84.0	fTotalStrokes	 12
U+3B84.0	fCangjie	DMMS
U+3B84.0	fFourCorner	40920
U+3B84.0	fHakka	 bong3
U+3B84.0	fCantonese	bong2; pong3
U+3B84.0	fMandarin	bang3; peng2; bi4; beng4
U+3B84.0	fEnglish	same as &#x699C;
U+3B85.0	fUTF8	㮅
U+3B85.0	fCNS11643	3-3C36
U+3B85.0	fR/S	 75.8
U+3B85.0	fTotalStrokes	 12
U+3B85.0	fCangjie	DOG
U+3B85.0	fFourCorner	40210
U+3B85.0	fCantonese	zeoi1
U+3B85.0	fMandarin	zhui1
U+3B85.0	fEnglish	a kind of tree
U+3B86.0	fUTF8	㮆
U+3B86.0	fCNS11643	3-3C43
U+3B86.0	fR/S	 75.8
U+3B86.0	fTotalStrokes	 12
U+3B86.0	fCangjie	DOTQ
U+3B86.0	fFourCorner	48950
U+3B86.0	fCantonese	jit6; zit6
U+3B86.0	fMandarin	nie4
U+3B86.0	fEnglish	ancient form, same as &#x74FE;
U+3B87.0	fUTF8	㮇
U+3B87.0	fCNS11643	3-3C2B
U+3B87.0	fR/S	 75.8
U+3B87.0	fTotalStrokes	 12
U+3B87.0	fCangjie	DMKP
U+3B87.0	fFourCorner	42930
U+3B87.0	fHakka	 kwat7
U+3B87.0	fCantonese	tim3; kut3
U+3B87.0	fMandarin	tian4; gua1; kuo4; tian3
U+3B87.0	fEnglish	same as &#x681D;
U+3B88.0	fUTF8	㮈
U+3B88.0	fCNS11643	3-3C2D
U+3B88.0	fR/S	 75.8
U+3B88.0	fTotalStrokes	 12
U+3B88.0	fCangjie	DKMF
U+3B88.0	fFourCorner	44990
U+3B88.0	fCantonese	noi6; loi6 (also); jeui6; zeoi6
U+3B88.0	fMandarin	nai4
U+3B88.0	fEnglish	non-classical form of &#x5948;
U+3B89.0	fUTF8	㮉
U+3B89.0	fGB	S-216A
U+3B89.0	fR/S	 75.8
U+3B89.0	fTotalStrokes	 12
U+3B89.0	fCangjie	DOKT
U+3B89.0	fCantonese	hap6
U+3B8A.0	fUTF8	㮊
U+3B8A.0	fR/S	 75.8
U+3B8A.0	fTotalStrokes	 12
U+3B8A.0	fCangjie	DNND
U+3B8A.0	fCantonese	mau6
U+3B8A.0	fEnglish	[Japanese character]
U+3B8B.0	fUTF8	㮋
U+3B8B.0	fCNS11643	4-3D78
U+3B8B.0	fR/S	 75.9
U+3B8B.0	fTotalStrokes	 13
U+3B8B.0	fCangjie	DKBL
U+3B8B.0	fFourCorner	47920
U+3B8B.0	fCantonese	jau5; zaau5
U+3B8B.0	fMandarin	yu4; you3; you4
U+3B8B.0	fEnglish	same as &#x682F;
U+3B8C.0	fUTF8	㮌
U+3B8C.0	fCNS11643	3-4171
U+3B8C.0	fGB	5-4457
U+3B8C.0	fR/S	 75.9
U+3B8C.0	fTotalStrokes	 13
U+3B8C.0	fCangjie	DMWL
U+3B8C.0	fFourCorner	41960
U+3B8C.0	fHakka	 mien2
U+3B8C.0	fCantonese	min4
U+3B8C.0	fMandarin	mian2; min3
U+3B8C.0	fEnglish	same as &#x68C9;
U+3B8D.0	fUTF8	㮍
U+3B8D.0	fCNS11643	3-4166
U+3B8D.0	fR/S	 75.9
U+3B8D.0	fTotalStrokes	 13
U+3B8D.0	fCangjie	TBND
U+3B8D.0	fFourCorner	80900
U+3B8D.0	fCantonese	zin1
U+3B8D.0	fMandarin	zhan4
U+3B8D.0	fEnglish	ancient form of &#x724B;, &#x7B8B;
U+3B8E.0	fUTF8	㮎
U+3B8E.0	fCNS11643	3-4170
U+3B8E.0	fR/S	 75.9
U+3B8E.0	fTotalStrokes	 13
U+3B8E.0	fCangjie	DMFT
U+3B8E.0	fFourCorner	41910
U+3B8E.0	fHakka	 bui1
U+3B8E.0	fCantonese	bui1
U+3B8E.0	fMandarin	bei1
U+3B8E.0	fEnglish	same as &#x676F;
U+3B8F.0	fUTF8	㮏
U+3B8F.0	fCNS11643	4-3D6B
U+3B8F.0	fGB	5-4451
U+3B8F.0	fR/S	 75.9
U+3B8F.0	fTotalStrokes	 13
U+3B8F.0	fCangjie	DDMF
U+3B8F.0	fFourCorner	44990
U+3B8F.0	fCantonese	noi6; loi6 (also)
U+3B8F.0	fMandarin	ni4; nai4
U+3B8F.0	fEnglish	same as &#x67F0;
U+3B90.0	fUTF8	㮐
U+3B90.0	fCNS11643	4-3D6E
U+3B90.0	fGB	3-4239
U+3B90.0	fR/S	 75.9
U+3B90.0	fTotalStrokes	 13
U+3B90.0	fCangjie	DFHU
U+3B90.0	fFourCorner	49960
U+3B90.0	fCantonese	saang2; sing2
U+3B90.0	fMandarin	xing3
U+3B90.0	fEnglish	[1] a rack or a stand with three sticks to cross each other [2] chopping board
U+3B91.0	fUTF8	㮑
U+3B91.0	fCNS11643	4-3D6F
U+3B91.0	fGB	3-4241
U+3B91.0	fR/S	 75.9
U+3B91.0	fTotalStrokes	 13
U+3B91.0	fCangjie	DHJX
U+3B91.0	fFourCorner	42970
U+3B91.0	fCantonese	caap3
U+3B91.0	fMandarin	xi1; qi4
U+3B91.0	fEnglish	[1] the sound of broken sticks; crack of a wooden stick [2] forest; wooded land
U+3B92.0	fUTF8	㮒
U+3B92.0	fCNS11643	3-416C
U+3B92.0	fGB	5-4455
U+3B92.0	fR/S	 75.9
U+3B92.0	fTotalStrokes	 13
U+3B92.0	fCangjie	DMWG
U+3B92.0	fFourCorner	41910
U+3B92.0	fCantonese	jim5; zaan1; zin1
U+3B92.0	fMandarin	ran3
U+3B92.0	fEnglish	[1] wood [2] same as &#x798B;
U+3B93.0	fUTF8	㮓
U+3B93.0	fCNS11643	3-4169
U+3B93.0	fR/S	 75.9
U+3B93.0	fTotalStrokes	 13
U+3B93.0	fCangjie	DPMM
U+3B93.0	fFourCorner	41910
U+3B93.0	fCantonese	gang2
U+3B93.0	fMandarin	geng4; gen4
U+3B93.0	fEnglish	same as &#x4E99;, &#x5825;
U+3B94.0	fUTF8	㮔
U+3B94.0	fCNS11643	3-4164
U+3B94.0	fGB	5-4460
U+3B94.0	fR/S	 75.9
U+3B94.0	fTotalStrokes	 13
U+3B94.0	fCangjie	DHJG
U+3B94.0	fFourCorner	42910
U+3B94.0	fCantonese	cung3; cung1
U+3B94.0	fMandarin	chong4; tong2
U+3B94.0	fEnglish	[1] vegetable [2] non-classical, abbreviated form, same as &#x6A66;
U+3B95.0	fUTF8	㮕
U+3B95.0	fCNS11643	4-3D6D
U+3B95.0	fGB	3-4238
U+3B95.0	fR/S	 75.9
U+3B95.0	fTotalStrokes	 13
U+3B95.0	fCangjie	DMBK
U+3B95.0	fFourCorner	41930
U+3B95.0	fHakka	 ngion1; ji2; ri2
U+3B95.0	fCantonese	jyun5; zyun5
U+3B95.0	fMandarin	ruan3; er2
U+3B95.0	fEnglish	[1] the diospyros lotus, which looks black when dried [2] a small post standing on the beam to support the roof
U+3B96.0	fUTF8	㮖
U+3B96.0	fCNS11643	3-415E
U+3B96.0	fGB	5-444D
U+3B96.0	fR/S	 75.9
U+3B96.0	fTotalStrokes	 13
U+3B96.0	fCangjie	DTLJ
U+3B96.0	fFourCorner	44950
U+3B96.0	fCantonese	git3; gat3
U+3B96.0	fMandarin	jia2
U+3B96.0	fEnglish	[1] lay; put [2] drums; rap [3] quiver [4] stir up
U+3B97.0	fUTF8	㮗
U+3B97.0	fCNS11643	4-3D75
U+3B97.0	fGB	5-4476
U+3B97.0	fR/S	 75.9
U+3B97.0	fTotalStrokes	 13
U+3B97.0	fCangjie	NND
U+3B97.0	fFourCorner	18900
U+3B97.0	fCantonese	ging1; kaan4; sip3
U+3B97.0	fMandarin	qin2
U+3B97.0	fEnglish	same as &#x77DC;
U+3B98.0	fUTF8	㮘
U+3B98.0	fCNS11643	3-4162
U+3B98.0	fR/S	 75.9
U+3B98.0	fTotalStrokes	 13
U+3B98.0	fCangjie	DNHK
U+3B98.0	fFourCorner	48940
U+3B98.0	fCantonese	mou4; maau6
U+3B98.0	fMandarin	mao4
U+3B98.0	fEnglish	a tree; winter peach
U+3B99.0	fUTF8	㮙
U+3B99.0	fCNS11643	3-4163
U+3B99.0	fGB	5-445F
U+3B99.0	fR/S	 75.9
U+3B99.0	fTotalStrokes	 13
U+3B99.0	fCangjie	DRRS
U+3B99.0	fFourCorner	46920
U+3B99.0	fCantonese	ngok6; ok6 (also)
U+3B99.0	fMandarin	e4
U+3B99.0	fEnglish	 pitfall; hole
U+3B9A.0	fUTF8	㮚
U+3B9A.0	fCNS11643	3-417A
U+3B9A.0	fGB	5-445A
U+3B9A.0	fR/S	 75.9
U+3B9A.0	fTotalStrokes	 13
U+3B9A.0	fCangjie	YWD
U+3B9A.0	fFourCorner	21900
U+3B9A.0	fHakka	 lit8
U+3B9A.0	fCantonese	leot6
U+3B9A.0	fMandarin	li4; lie4
U+3B9A.0	fEnglish	ancient form of &#x6817;
U+3B9B.0	fUTF8	㮛
U+3B9B.0	fCNS11643	3-4161
U+3B9B.0	fR/S	 75.9
U+3B9B.0	fTotalStrokes	 13
U+3B9B.0	fCangjie	DAMO
U+3B9B.0	fFourCorner	46980
U+3B9B.0	fHakka	 she2; shi2; chii2
U+3B9B.0	fCantonese	ci4
U+3B9B.0	fMandarin	chi2
U+3B9B.0	fEnglish	[1] same as &#x5319; [2] same as &#x6938;
U+3B9C.0	fUTF8	㮜
U+3B9C.0	fCNS11643	3-415F
U+3B9C.0	fR/S	 75.9
U+3B9C.0	fTotalStrokes	 13
U+3B9C.0	fCangjie	DSVG
U+3B9C.0	fFourCorner	41910
U+3B9C.0	fCantonese	zong1
U+3B9C.0	fMandarin	zang1
U+3B9C.0	fEnglish	a blocks engraved for holding things
U+3B9D.0	fUTF8	㮝
U+3B9D.0	fCNS11643	4-4423
U+3B9D.0	fGB	3-4247
U+3B9D.0	fR/S	 75.9; 75.10
U+3B9D.0	fTotalStrokes	 13; 14
U+3B9D.0	fCangjie	DCRL
U+3B9D.0	fFourCorner	47920
U+3B9D.0	fCantonese	hat6
U+3B9D.0	fMandarin	he2; luo4
U+3B9D.0	fEnglish	[1] angle steel [2] legs of a table
U+3B9E.0	fUTF8	㮞
U+3B9E.0	fCNS11643	4-3D76
U+3B9E.0	fGB	3-4249
U+3B9E.0	fR/S	 75.9
U+3B9E.0	fTotalStrokes	 13
U+3B9E.0	fCangjie	DIOR
U+3B9E.0	fFourCorner	47960
U+3B9E.0	fHakka	 ziet7
U+3B9E.0	fCantonese	zit3
U+3B9E.0	fMandarin	jie2; ni2; ya2
U+3B9E.0	fEnglish	same as &#x6976;
U+3B9F.0	fUTF8	㮟
U+3B9F.0	fCNS11643	3-4160
U+3B9F.0	fR/S	 75.9
U+3B9F.0	fTotalStrokes	 13
U+3B9F.0	fCangjie	DMRD
U+3B9F.0	fFourCorner	44990
U+3B9F.0	fCantonese	nin2; lin2 (also); nin5
U+3B9F.0	fMandarin	nian3; ji2; n4; peng4; rou4
U+3B9F.0	fEnglish	[1] whetstone [2] corner; nook; crack; opening; cleft
U+3BA0.0	fUTF8	㮠
U+3BA0.0	fGB	7-2350
U+3BA0.0	fR/S	 75.9
U+3BA0.0	fTotalStrokes	 13
U+3BA0.0	fCangjie	DTBD
U+3BA0.0	fCantonese	wing4
U+3BA1.0	fUTF8	㮡
U+3BA1.0	fCNS11643	3-425A
U+3BA1.0	fR/S	 86.9
U+3BA1.0	fTotalStrokes	 13
U+3BA1.0	fCangjie	FSD
U+3BA1.0	fFourCorner	91900
U+3BA1.0	fCantonese	gun3; gwun3 (also)
U+3BA1.0	fMandarin	guan4
U+3BA1.0	fEnglish	same as &#x721F;
U+3BA2.0	fUTF8	㮢
U+3BA2.0	fGB	3-4245
U+3BA2.0	fR/S	 75.9
U+3BA2.0	fTotalStrokes	 13
U+3BA2.0	fCangjie	DONK
U+3BA2.0	fCantonese	hau4; haau4
U+3BA2.0	fMandarin	hou2
U+3BA2.0	fEnglish	fruit
U+3BA3.0	fUTF8	㮣
U+3BA3.0	fCNS11643	3-4167
U+3BA3.0	fR/S	 75.9
U+3BA3.0	fTotalStrokes	 13
U+3BA3.0	fCangjie	AUD
U+3BA3.0	fFourCorner	71900
U+3BA3.0	fCantonese	koi3; huk1; koek3
U+3BA3.0	fMandarin	gai4; ze2
U+3BA3.0	fEnglish	[1] variant of &#x6982; [2] interchangeable with &#x6168; [3] interchangeable with &#x6E89;
U+3BA4.0	fUTF8	㮤
U+3BA4.0	fCNS11643	F-4855
U+3BA4.0	fGB	5-4533
U+3BA4.0	fR/S	 75.10
U+3BA4.0	fTotalStrokes	 14
U+3BA4.0	fCangjie	JCORD
U+3BA4.0	fCantonese	cung4
U+3BA4.0	fEnglish	[1] pine; fir [2] same as &#x677E; [3] same as &#x9B06;
U+3BA5.0	fUTF8	㮥
U+3BA5.0	fCNS11643	4-4424
U+3BA5.0	fGB	3-4273
U+3BA5.0	fR/S	 75.10
U+3BA5.0	fTotalStrokes	 14
U+3BA5.0	fCangjie	DIKW
U+3BA5.0	fFourCorner	43960
U+3BA5.0	fCantonese	ban6; faan5
U+3BA5.0	fMandarin	ben1; fen4
U+3BA5.0	fEnglish	[1] sails [2] the mat-covering of small boats [3] interchangeable with &#x8EEC;
U+3BA6.0	fUTF8	㮦
U+3BA6.0	fCNS11643	4-4375
U+3BA6.0	fGB	3-425D
U+3BA6.0	fR/S	 75.10
U+3BA6.0	fTotalStrokes	 14
U+3BA6.0	fCangjie	DJBF
U+3BA6.0	fFourCorner	44990
U+3BA6.0	fCantonese	sok3
U+3BA6.0	fMandarin	suo3; se4
U+3BA6.0	fEnglish	[1] branches growing upwards [2] the tip of a branch or things of similar shape [3] interchangeable with &#x7D22;
U+3BA7.0	fUTF8	㮧
U+3BA7.0	fCNS11643	4-4421
U+3BA7.0	fGB	3-4064
U+3BA7.0	fR/S	 75.10
U+3BA7.0	fTotalStrokes	 14
U+3BA7.0	fCangjie	DHRF
U+3BA7.0	fFourCorner	47920
U+3BA7.0	fHakka	 wu1; vu1
U+3BA7.0	fCantonese	wu1; waat1
U+3BA7.0	fMandarin	wu1
U+3BA7.0	fEnglish	[1] a tree [2] a fruit
U+3BA8.0	fUTF8	㮨
U+3BA8.0	fCNS11643	4-437C
U+3BA8.0	fGB	3-4262
U+3BA8.0	fR/S	 75.10
U+3BA8.0	fTotalStrokes	 14
U+3BA8.0	fCangjie	DWCE
U+3BA8.0	fFourCorner	46940
U+3BA8.0	fCantonese	zik1
U+3BA8.0	fMandarin	ji1; ji2; ji4
U+3BA8.0	fEnglish	Codium macronatum
U+3BA9.0	fUTF8	㮩
U+3BA9.0	fCNS11643	3-473F
U+3BA9.0	fGB	5-4528
U+3BA9.0	fR/S	 75.10
U+3BA9.0	fTotalStrokes	 14
U+3BA9.0	fCangjie	DHUP
U+3BA9.0	fFourCorner	46930
U+3BA9.0	fCantonese	sik1
U+3BA9.0	fMandarin	xi2
U+3BA9.0	fEnglish	a tree
U+3BAA.0	fUTF8	㮪
U+3BAA.0	fCNS11643	4-4378
U+3BAA.0	fGB	3-4255
U+3BAA.0	fR/S	 75.10
U+3BAA.0	fTotalStrokes	 14
U+3BAA.0	fCangjie	DMND
U+3BAA.0	fFourCorner	47990
U+3BAA.0	fCantonese	kung4
U+3BAA.0	fMandarin	qiong2
U+3BAA.0	fEnglish	 small boat
U+3BAB.0	fUTF8	㮫
U+3BAB.0	fCNS11643	4-4373
U+3BAB.0	fGB	3-426F
U+3BAB.0	fR/S	 75.10
U+3BAB.0	fTotalStrokes	 14
U+3BAB.0	fCangjie	DJQR
U+3BAB.0	fFourCorner	43960
U+3BAB.0	fCantonese	hot6; haat6; hot3
U+3BAB.0	fMandarin	he2
U+3BAB.0	fEnglish	[1] a wooden article to be used to adjust a bow [2] a tiger shaped wooden article used to stop the music in ancient times
U+3BAC.0	fUTF8	㮬
U+3BAC.0	fCNS11643	3-4741
U+3BAC.0	fR/S	 75.10
U+3BAC.0	fTotalStrokes	 14
U+3BAC.0	fCangjie	DCIM
U+3BAC.0	fFourCorner	48920
U+3BAC.0	fCantonese	jung1; zung1
U+3BAC.0	fMandarin	weng1
U+3BAC.0	fEnglish	a fruit tree
U+3BAD.0	fUTF8	㮭
U+3BAD.0	fCNS11643	4-4374
U+3BAD.0	fGB	5-4537
U+3BAD.0	fR/S	 75.10
U+3BAD.0	fTotalStrokes	 14
U+3BAD.0	fCangjie	DNIW
U+3BAD.0	fFourCorner	47960
U+3BAD.0	fCantonese	gaam1; ham4
U+3BAD.0	fMandarin	han2; xian2
U+3BAD.0	fEnglish	same as &#x6937;
U+3BAE.0	fUTF8	㮮
U+3BAE.0	fCNS11643	3-474A
U+3BAE.0	fGB	5-4477
U+3BAE.0	fR/S	 75.10
U+3BAE.0	fTotalStrokes	 14
U+3BAE.0	fCangjie	DQHQ
U+3BAE.0	fFourCorner	47950
U+3BAE.0	fCantonese	git3
U+3BAE.0	fMandarin	jie2
U+3BAE.0	fEnglish	a well sweep
U+3BAF.0	fUTF8	㮯
U+3BAF.0	fCNS11643	4-437D
U+3BAF.0	fGB	3-4263
U+3BAF.0	fR/S	 75.10
U+3BAF.0	fTotalStrokes	 14
U+3BAF.0	fCangjie	DWMO
U+3BAF.0	fFourCorner	46900
U+3BAF.0	fCantonese	wan4
U+3BAF.0	fMandarin	hun3; hua2; hun2; kuan3
U+3BAF.0	fEnglish	[1] whole piece of or unbroken firewoods [2] complete; whole; undamaged; general [3] same as &#x68A1;
U+3BB0.0	fUTF8	㮰
U+3BB0.0	fCNS11643	4-437E
U+3BB0.0	fGB	3-4265
U+3BB0.0	fR/S	 75.10
U+3BB0.0	fTotalStrokes	 14
U+3BB0.0	fCangjie	DHWP
U+3BB0.0	fFourCorner	46910
U+3BB0.0	fCantonese	pei4
U+3BB0.0	fMandarin	pi2; bi2
U+3BB0.0	fEnglish	the small beam supporting the rafters at the eaves
U+3BB1.0	fUTF8	㮱
U+3BB1.0	fCNS11643	4-437A
U+3BB1.0	fGB	5-444F
U+3BB1.0	fR/S	 75.10
U+3BB1.0	fTotalStrokes	 14
U+3BB1.0	fCangjie	DTHU
U+3BB1.0	fFourCorner	44910
U+3BB1.0	fCantonese	sing1
U+3BB1.0	fMandarin	sheng1
U+3BB1.0	fEnglish	(said of grass, trees, vegetation and flora) non-classical form of &#x68EE;
U+3BB2.0	fUTF8	㮲
U+3BB2.0	fCNS11643	4-4422
U+3BB2.0	fGB	3-4078
U+3BB2.0	fR/S	 75.10
U+3BB2.0	fTotalStrokes	 14
U+3BB2.0	fCangjie	DPUU
U+3BB2.0	fFourCorner	47920
U+3BB2.0	fHakka	 ceu1; cu1
U+3BB2.0	fCantonese	cau1; caai1
U+3BB2.0	fMandarin	chou1
U+3BB2.0	fEnglish	the stick through a buffalo's nose, by which it is led
U+3BB3.0	fUTF8	㮳
U+3BB3.0	fCNS11643	3-474F
U+3BB3.0	fR/S	 75.10
U+3BB3.0	fTotalStrokes	 14
U+3BB3.0	fCangjie	DBTK
U+3BB3.0	fFourCorner	48930
U+3BB3.0	fCantonese	zan6; zaam6
U+3BB3.0	fMandarin	zhen4
U+3BB3.0	fEnglish	[1] a tree [2] same as &#x681A;
U+3BB4.0	fUTF8	㮴
U+3BB4.0	fCNS11643	3-4740
U+3BB4.0	fR/S	 75.10
U+3BB4.0	fTotalStrokes	 14; 13
U+3BB4.0	fCangjie	DHXE
U+3BB4.0	fFourCorner	47940
U+3BB4.0	fCantonese	sau1
U+3BB4.0	fMandarin	sou1
U+3BB4.0	fEnglish	same as &#x8258;
U+3BB5.0	fUTF8	㮵
U+3BB5.0	fCNS11643	4-4425
U+3BB5.0	fGB	5-452E
U+3BB5.0	fR/S	 75.10
U+3BB5.0	fTotalStrokes	 14
U+3BB5.0	fCangjie	DYSY
U+3BB5.0	fFourCorner	48940
U+3BB5.0	fHakka	 zhen1
U+3BB5.0	fCantonese	zin1
U+3BB5.0	fMandarin	zhan1
U+3BB5.0	fEnglish	same as &#x6834;
U+3BB6.0	fUTF8	㮶
U+3BB6.0	fCNS11643	4-4371
U+3BB6.0	fGB	5-4530
U+3BB6.0	fR/S	 75.10
U+3BB6.0	fTotalStrokes	 14
U+3BB6.0	fCangjie	DTUB
U+3BB6.0	fFourCorner	47920
U+3BB6.0	fCantonese	sok3; sak3
U+3BB6.0	fMandarin	shuo4; shi4
U+3BB6.0	fEnglish	same as &#x69CA;
U+3BB7.0	fUTF8	㮷
U+3BB7.0	fCNS11643	5-464B
U+3BB7.0	fGB	S-2173
U+3BB7.0	fR/S	 75.10
U+3BB7.0	fTotalStrokes	 14
U+3BB7.0	fCangjie	DIUA
U+3BB7.0	fFourCorner	43960
U+3BB7.0	fHakka	 tje1
U+3BB7.0	fCantonese	gai1
U+3BB7.0	fMandarin	ji1
U+3BB7.0	fEnglish	a tree (as maple tree)
U+3BB8.0	fUTF8	㮸
U+3BB8.0	fCNS11643	3-4749
U+3BB8.0	fR/S	 75.10
U+3BB8.0	fTotalStrokes	 14
U+3BB8.0	fCangjie	DYTK
U+3BB8.0	fFourCorner	48930
U+3BB8.0	fCantonese	sung3
U+3BB8.0	fMandarin	song4
U+3BB8.0	fEnglish	corrupted form of &#x9001;
U+3BB9.0	fUTF8	㮹
U+3BB9.0	fCNS11643	4-3D72
U+3BB9.0	fGB	5-4525
U+3BB9.0	fR/S	 75.10; 75.9
U+3BB9.0	fTotalStrokes	 14; 13
U+3BB9.0	fCangjie	DMGK
U+3BB9.0	fFourCorner	48940
U+3BB9.0	fCantonese	zi2
U+3BB9.0	fMandarin	zhi4; zhi3
U+3BB9.0	fEnglish	a tree
U+3BBA.0	fUTF8	㮺
U+3BBA.0	fCNS11643	3-4750
U+3BBA.0	fR/S	 75.10
U+3BBA.0	fTotalStrokes	 14
U+3BBA.0	fCangjie	DMRRR
U+3BBA.0	fFourCorner	50660
U+3BBA.0	fCantonese	bun2; hak1
U+3BBA.0	fMandarin	ben3
U+3BBA.0	fEnglish	ancient form of &#x672C;
U+3BBB.0	fUTF8	㮻
U+3BBB.0	fR/S	 75.10
U+3BBB.0	fTotalStrokes	 14
U+3BBB.0	fCangjie	DEII
U+3BBB.0	fFourCorner	4793
U+3BBB.0	fCantonese	sou1
U+3BBB.0	fEnglish	same as &#x6414;
U+3BBC.0	fUTF8	㮼
U+3BBC.0	fCNS11643	F-4852
U+3BBC.0	fGB	S-2172
U+3BBC.0	fR/S	 75.10
U+3BBC.0	fTotalStrokes	 14
U+3BBC.0	fCangjie	DISM
U+3BBC.0	fFourCorner	4392
U+3BBC.0	fCantonese	sin3
U+3BBD.0	fUTF8	㮽
U+3BBD.0	fR/S	 75.10
U+3BBD.0	fTotalStrokes	 14
U+3BBD.0	fCangjie	DHYE
U+3BBD.0	fCantonese	pun4
U+3BBD.0	fEnglish	[Korean character]
U+3BBE.0	fUTF8	㮾
U+3BBE.0	fGB	3-4270
U+3BBE.0	fR/S	 75.11
U+3BBE.0	fTotalStrokes	 15
U+3BBE.0	fCangjie	IBD
U+3BBE.0	fCantonese	long5
U+3BBE.0	fMandarin	lang3
U+3BBE.0	fEnglish	a place in today's Hunan province
U+3BBF.0	fUTF8	㮿
U+3BBF.0	fCNS11643	3-4C4E
U+3BBF.0	fGB	5-4367
U+3BBF.0	fR/S	 75.11
U+3BBF.0	fTotalStrokes	 15; 14
U+3BBF.0	fCangjie	DWTJ
U+3BBF.0	fFourCorner	46950
U+3BBF.0	fHakka	 bit7
U+3BBF.0	fCantonese	bat1; bit1; bit6
U+3BBF.0	fMandarin	bi4
U+3BBF.0	fEnglish	a light-yellow wood used for carving
U+3BC0.0	fUTF8	㯀
U+3BC0.0	fCNS11643	5-4D6D
U+3BC0.0	fGB	3-4328
U+3BC0.0	fR/S	 75.11
U+3BC0.0	fTotalStrokes	 15
U+3BC0.0	fCangjie	DYSO
U+3BC0.0	fFourCorner	48980
U+3BC0.0	fCantonese	syun6; syun4
U+3BC0.0	fMandarin	xuan4; xian3
U+3BC0.0	fEnglish	bonds and fetters for falcons and dogs used in hunting
U+3BC1.0	fUTF8	㯁
U+3BC1.0	fCNS11643	4-4A3A
U+3BC1.0	fGB	7-234E
U+3BC1.0	fR/S	 75.11
U+3BC1.0	fTotalStrokes	 15
U+3BC1.0	fCangjie	DNLR
U+3BC1.0	fFourCorner	40960
U+3BC1.0	fHakka	 poi5; pi5; pui2
U+3BC1.0	fCantonese	pui4; pang5
U+3BC1.0	fMandarin	pei2; bang4
U+3BC1.0	fEnglish	a Chinese family name
U+3BC2.0	fUTF8	㯂
U+3BC2.0	fCNS11643	3-4C4C
U+3BC2.0	fGB	5-444E
U+3BC2.0	fR/S	 75.11
U+3BC2.0	fTotalStrokes	 15
U+3BC2.0	fCangjie	DKPB
U+3BC2.0	fFourCorner	44920
U+3BC2.0	fCantonese	daai3
U+3BC2.0	fMandarin	dai4
U+3BC2.0	fEnglish	pillar or post of a frame on which silkworms spin
U+3BC3.0	fUTF8	㯃
U+3BC3.0	fCNS11643	5-4D61
U+3BC3.0	fR/S	 75.11
U+3BC3.0	fTotalStrokes	 15
U+3BC3.0	fCangjie	DDOE
U+3BC3.0	fFourCorner	44932
U+3BC3.0	fHakka	  cit7
U+3BC3.0	fCantonese	cat1
U+3BC3.0	fMandarin	qi1
U+3BC3.0	fEnglish	[1] the varnish tree (Rhus vernicifera) [2] lacquer
U+3BC4.0	fUTF8	㯄
U+3BC4.0	fCNS11643	3-4C53
U+3BC4.0	fR/S	 75.11
U+3BC4.0	fTotalStrokes	 15
U+3BC4.0	fCangjie	DDHMU
U+3BC4.0	fFourCorner	44210
U+3BC4.0	fCantonese	zi1
U+3BC4.0	fMandarin	zhi1
U+3BC4.0	fEnglish	luxuriant; exuberant; lush (of trees)
U+3BC5.0	fUTF8	㯅
U+3BC5.0	fCNS11643	4-4A37
U+3BC5.0	fGB	3-4325
U+3BC5.0	fR/S	 75.11
U+3BC5.0	fTotalStrokes	 15
U+3BC5.0	fCangjie	DIHJ
U+3BC5.0	fFourCorner	40946
U+3BC5.0	fCantonese	pei4; bei1
U+3BC5.0	fMandarin	pi2; bi1
U+3BC5.0	fEnglish	the branches growing downwards; small trees; saplings; cuttings of trees for planting
U+3BC6.0	fUTF8	㯆
U+3BC6.0	fCNS11643	4-4A36
U+3BC6.0	fGB	3-413E
U+3BC6.0	fR/S	 75.11
U+3BC6.0	fTotalStrokes	 15
U+3BC6.0	fCangjie	DYHM
U+3BC6.0	fFourCorner	40910
U+3BC6.0	fHakka	 san3
U+3BC6.0	fCantonese	caan2
U+3BC6.0	fMandarin	chan3
U+3BC6.0	fEnglish	a tree (found in Anhui) on which grows a peach-like fruit
U+3BC7.0	fUTF8	㯇
U+3BC7.0	fCNS11643	3-4C5D
U+3BC7.0	fR/S	 75.11
U+3BC7.0	fTotalStrokes	 15
U+3BC7.0	fCangjie	DLYG
U+3BC7.0	fFourCorner	41910
U+3BC7.0	fHakka	 pi5
U+3BC7.0	fCantonese	bai6
U+3BC7.0	fMandarin	bi4
U+3BC7.0	fEnglish	same as &#x6890;
U+3BC8.0	fUTF8	㯈
U+3BC8.0	fCNS11643	3-4C58
U+3BC8.0	fR/S	 75.11
U+3BC8.0	fTotalStrokes	 15
U+3BC8.0	fCangjie	DYDL
U+3BC8.0	fFourCorner	45930
U+3BC8.0	fCantonese	cuk1
U+3BC8.0	fMandarin	su4
U+3BC8.0	fEnglish	(non-classical form) small trees; saplings
U+3BC9.0	fUTF8	㯉
U+3BC9.0	fCNS11643	7-236F
U+3BC9.0	fR/S	 75.11
U+3BC9.0	fTotalStrokes	 15
U+3BC9.0	fCangjie	DYPD
U+3BC9.0	fFourCorner	41940
U+3BC9.0	fCantonese	wok6
U+3BC9.0	fMandarin	hua4; huo4
U+3BC9.0	fEnglish	same as &#x6AB4;
U+3BCA.0	fUTF8	㯊
U+3BCA.0	fCNS11643	4-4A38
U+3BCA.0	fGB	3-4276
U+3BCA.0	fR/S	 75.11
U+3BCA.0	fTotalStrokes	 15
U+3BCA.0	fCangjie	DMYF
U+3BCA.0	fFourCorner	41920
U+3BCA.0	fCantonese	han4
U+3BCA.0	fMandarin	hen2
U+3BCA.0	fEnglish	even measuring board for pecks and bushels
U+3BCB.0	fUTF8	㯋
U+3BCB.0	fCNS11643	4-4A42
U+3BCB.0	fGB	3-6073
U+3BCB.0	fR/S	 75.11
U+3BCB.0	fTotalStrokes	 15
U+3BCB.0	fCangjie	PDMBC
U+3BCB.0	fFourCorner	21980
U+3BCB.0	fCantonese	gwing2; wing6
U+3BCB.0	fMandarin	jiong3; ying3
U+3BCB.0	fEnglish	[1] chest; box [2] footstool; footrest [3] a tree [4] handle of an awl [5] ring of a knife or a sword [6] a round pillow that roused the sleeper when he moved
U+3BCC.0	fUTF8	㯌
U+3BCC.0	fCNS11643	3-4C5A
U+3BCC.0	fR/S	 75.11
U+3BCC.0	fTotalStrokes	 15
U+3BCC.0	fCangjie	DYMO
U+3BCC.0	fFourCorner	41930
U+3BCC.0	fHakka	 chon2; jen2
U+3BCC.0	fCantonese	cyun4
U+3BCC.0	fMandarin	chuan2; zhuan4
U+3BCC.0	fEnglish	corrupted form of &#x693D;
U+3BCD.0	fUTF8	㯍
U+3BCD.0	fCNS11643	3-4C5C
U+3BCD.0	fGB	5-4469
U+3BCD.0	fR/S	 75.11
U+3BCD.0	fTotalStrokes	 15
U+3BCD.0	fCangjie	DVMI
U+3BCD.0	fFourCorner	47940
U+3BCD.0	fCantonese	zoeng2; zoeng3
U+3BCD.0	fMandarin	jiang3
U+3BCD.0	fEnglish	same as &#x69F3;
U+3BCE.0	fUTF8	㯎
U+3BCE.0	fCNS11643	3-4C48
U+3BCE.0	fR/S	 75.11
U+3BCE.0	fTotalStrokes	 15
U+3BCE.0	fCangjie	DYRD
U+3BCE.0	fFourCorner	44990
U+3BCE.0	fCantonese	nyun6; lyun6 (also)
U+3BCE.0	fMandarin	nen4
U+3BCE.0	fEnglish	stupid; dull; foolish; idiotic
U+3BCF.0	fUTF8	㯏
U+3BCF.0	fCNS11643	3-4C55
U+3BCF.0	fGB	5-4539
U+3BCF.0	fR/S	 75.11
U+3BCF.0	fTotalStrokes	 15; 14
U+3BCF.0	fCangjie	GED
U+3BCF.0	fFourCorner	47900
U+3BCF.0	fCantonese	guk1
U+3BCF.0	fMandarin	xin1; gu3
U+3BCF.0	fEnglish	same as &#x7A40;
U+3BD0.0	fUTF8	㯐
U+3BD0.0	fCNS11643	3-5143
U+3BD0.0	fGB	5-454F
U+3BD0.0	fR/S	 75.12
U+3BD0.0	fTotalStrokes	 16
U+3BD0.0	fCangjie	NBD
U+3BD0.0	fFourCorner	74900
U+3BD0.0	fHakka	 to3
U+3BD0.0	fCantonese	to5; teoi2
U+3BD0.0	fMandarin	tuo3; fang3; duo3; tuo2
U+3BD0.0	fEnglish	same as &#x6A62;
U+3BD1.0	fUTF8	㯑
U+3BD1.0	fR/S	 75.11
U+3BD1.0	fTotalStrokes	 15
U+3BD1.0	fCangjie	DNSV
U+3BD1.0	fCantonese	zoeng1
U+3BD1.0	fEnglish	[Korean character]
U+3BD2.0	fUTF8	㯒
U+3BD2.0	fR/S	 75.11
U+3BD2.0	fTotalStrokes	 15
U+3BD2.0	fCangjie	DBBU
U+3BD2.0	fCantonese	mik6
U+3BD2.0	fEnglish	[Korean character]
U+3BD3.0	fUTF8	㯓
U+3BD3.0	fCNS11643	3-5142
U+3BD3.0	fR/S	 75.12
U+3BD3.0	fTotalStrokes	 16
U+3BD3.0	fCangjie	DORM
U+3BD3.0	fFourCorner	48920
U+3BD3.0	fHakka	 tap7
U+3BD3.0	fCantonese	taap3
U+3BD3.0	fMandarin	ta4
U+3BD3.0	fEnglish	same as &#x69BB;
U+3BD4.0	fUTF8	㯔
U+3BD4.0	fCNS11643	4-5060
U+3BD4.0	fGB	3-4335
U+3BD4.0	fR/S	 75.12
U+3BD4.0	fTotalStrokes	 16
U+3BD4.0	fCangjie	HUHUD
U+3BD4.0	fFourCorner	20900
U+3BD4.0	fCantonese	ceoi3
U+3BD4.0	fMandarin	cui4
U+3BD4.0	fEnglish	[1] hull heavily [2] pound heavily
U+3BD5.0	fUTF8	㯕
U+3BD5.0	fCNS11643	4-505A
U+3BD5.0	fGB	3-4330
U+3BD5.0	fR/S	 75.12
U+3BD5.0	fTotalStrokes	 16
U+3BD5.0	fCangjie	DTCL
U+3BD5.0	fFourCorner	42920
U+3BD5.0	fCantonese	sai1
U+3BD5.0	fMandarin	xi1; si1
U+3BD5.0	fEnglish	[1] sticks or frames; a wooden device used to torture the fingers by squeezing [2] the branches growing downwards [3] small trees; saplings; cuttings of trees for planting
U+3BD6.0	fUTF8	㯖
U+3BD6.0	fCNS11643	3-5137
U+3BD6.0	fR/S	 75.12
U+3BD6.0	fTotalStrokes	 16
U+3BD6.0	fCangjie	DJWP
U+3BD6.0	fFourCorner	44930
U+3BD6.0	fCantonese	dak6; daak1
U+3BD6.0	fMandarin	zhe2; de2
U+3BD6.0	fEnglish	same as &#x68CF;
U+3BD7.0	fUTF8	㯗
U+3BD7.0	fCNS11643	3-5130
U+3BD7.0	fR/S	 75.12
U+3BD7.0	fTotalStrokes	 16
U+3BD7.0	fCangjie	DAND
U+3BD7.0	fFourCorner	47920
U+3BD7.0	fCantonese	haan4
U+3BD7.0	fMandarin	xian2; ran3; shun4; xian4
U+3BD7.0	fEnglish	name of a tree
U+3BD8.0	fUTF8	㯘
U+3BD8.0	fCNS11643	4-5059
U+3BD8.0	fGB	3-432F
U+3BD8.0	fR/S	 75.12
U+3BD8.0	fTotalStrokes	 16
U+3BD8.0	fCangjie	DGFO
U+3BD8.0	fFourCorner	47980
U+3BD8.0	fCantonese	fun2
U+3BD8.0	fMandarin	nuan4; kuan3
U+3BD8.0	fEnglish	same as &#x68A1;
U+3BD9.0	fUTF8	㯙
U+3BD9.0	fCNS11643	4-4A39
U+3BD9.0	fGB	3-433A
U+3BD9.0	fR/S	 75.12; 75.11
U+3BD9.0	fTotalStrokes	 16; 15
U+3BD9.0	fCangjie	DYBK
U+3BD9.0	fFourCorner	48940
U+3BD9.0	fCantonese	zit6; cit3; diu6
U+3BD9.0	fMandarin	zhe2; che4
U+3BD9.0	fEnglish	the buckthorn or jujube (tree); dates
U+3BDA.0	fUTF8	㯚
U+3BDA.0	fCNS11643	3-513F
U+3BDA.0	fGB	5-4560
U+3BDA.0	fR/S	 75.12
U+3BDA.0	fTotalStrokes	 16
U+3BDA.0	fCangjie	DHOR
U+3BDA.0	fFourCorner	48960
U+3BDA.0	fCantonese	daap3
U+3BDA.0	fMandarin	ta3; da2; he2
U+3BDA.0	fEnglish	same as &#x6999;
U+3BDB.0	fUTF8	㯛
U+3BDB.0	fCNS11643	3-512A
U+3BDB.0	fR/S	 75.12
U+3BDB.0	fTotalStrokes	 16
U+3BDB.0	fCangjie	DGBM
U+3BDB.0	fFourCorner	44910
U+3BDB.0	fCantonese	wu4
U+3BDB.0	fMandarin	hu2; huo4
U+3BDB.0	fEnglish	a date species
U+3BDC.0	fUTF8	㯜
U+3BDC.0	fCNS11643	3-513D
U+3BDC.0	fR/S	 75.12
U+3BDC.0	fTotalStrokes	 16
U+3BDC.0	fCangjie	DTYJ
U+3BDC.0	fFourCorner	44940
U+3BDC.0	fCantonese	zyut6; seoi6; si1
U+3BDC.0	fMandarin	cui4
U+3BDC.0	fEnglish	same as &#x690A;
U+3BDD.0	fUTF8	㯝
U+3BDD.0	fCNS11643	4-5649
U+3BDD.0	fGB	3-4348
U+3BDD.0	fR/S	 75.13; 75.12
U+3BDD.0	fTotalStrokes	 16; 17
U+3BDD.0	fCangjie	DRMR
U+3BDD.0	fFourCorner	47960
U+3BDD.0	fCantonese	lou6; leot6
U+3BDD.0	fMandarin	lu4
U+3BDD.0	fEnglish	name of a tree
U+3BDE.0	fUTF8	㯞
U+3BDE.0	fCNS11643	3-5133
U+3BDE.0	fR/S	 75.12
U+3BDE.0	fTotalStrokes	 16
U+3BDE.0	fCangjie	DVFB
U+3BDE.0	fFourCorner	46920
U+3BDE.0	fCantonese	gyun3
U+3BDE.0	fMandarin	juan4; yue4
U+3BDE.0	fEnglish	[1] a kind of tree [2] material for making clothes
U+3BDF.0	fUTF8	㯟
U+3BDF.0	fCNS11643	3-5139
U+3BDF.0	fR/S	 75.12
U+3BDF.0	fTotalStrokes	 16
U+3BDF.0	fCangjie	DDVNE
U+3BDF.0	fFourCorner	44130
U+3BDF.0	fCantonese	luk1
U+3BDF.0	fMandarin	lu4
U+3BDF.0	fEnglish	ancient form of &#x3BDF;
U+3BE0.0	fUTF8	㯠
U+3BE0.0	fCNS11643	3-4C4B
U+3BE0.0	fR/S	 75.11
U+3BE0.0	fTotalStrokes	 15
U+3BE0.0	fCangjie	DYVQ
U+3BE0.0	fFourCorner	40950
U+3BE0.0	fHakka	 kien3; ken5
U+3BE0.0	fCantonese	hin3; zin5
U+3BE0.0	fMandarin	qian4; xian4; xun2
U+3BE0.0	fEnglish	[1] a board lying cross-wise [2] the name of a tree
U+3BE1.0	fUTF8	㯡
U+3BE1.0	fCNS11643	4-505E
U+3BE1.0	fGB	3-4336
U+3BE1.0	fR/S	 75.12
U+3BE1.0	fTotalStrokes	 16
U+3BE1.0	fCangjie	DEPRU
U+3BE1.0	fFourCorner	47110
U+3BE1.0	fCantonese	paau3; pau3; pau4
U+3BE1.0	fMandarin	pao4; huan4; pao2
U+3BE1.0	fEnglish	[1] lacquer some kind of ashes on wooden articals or furniture, after drying out, smooth them and then paint [2] black lacquer
U+3BE2.0	fUTF8	㯢
U+3BE2.0	fCNS11643	3-512D
U+3BE2.0	fGB	5-456B
U+3BE2.0	fR/S	 75.12
U+3BE2.0	fTotalStrokes	 16
U+3BE2.0	fCangjie	DRUC
U+3BE2.0	fFourCorner	47980
U+3BE2.0	fCantonese	zam6
U+3BE2.0	fMandarin	zhen4
U+3BE2.0	fEnglish	[1] same as &#x681A; [2] same as &#x3BB3;
U+3BE3.0	fUTF8	㯣
U+3BE3.0	fCNS11643	4-5064
U+3BE3.0	fR/S	 75.12
U+3BE3.0	fTotalStrokes	 16
U+3BE3.0	fCangjie	DTCD
U+3BE3.0	fFourCorner	44990
U+3BE3.0	fHakka	 fun1
U+3BE3.0	fCantonese	fan1
U+3BE3.0	fMandarin	fen2; fen1
U+3BE3.0	fEnglish	same as &#x68FB;
U+3BE4.0	fUTF8	㯤
U+3BE4.0	fCNS11643	3-5135
U+3BE4.0	fR/S	 75.12
U+3BE4.0	fTotalStrokes	 16
U+3BE4.0	fCangjie	DODOO
U+3BE4.0	fFourCorner	44990
U+3BE4.0	fCantonese	gik1; lik6
U+3BE4.0	fMandarin	li4
U+3BE4.0	fEnglish	same as &#x68D8;
U+3BE5.0	fUTF8	㯥
U+3BE5.0	fCNS11643	3-5134
U+3BE5.0	fGB	5-3027
U+3BE5.0	fR/S	 75.12; 75.11
U+3BE5.0	fTotalStrokes	 16; 15
U+3BE5.0	fCangjie	DWDW
U+3BE5.0	fFourCorner	55990
U+3BE5.0	fCantonese	cou4
U+3BE5.0	fMandarin	zao1; cao2; zha2
U+3BE5.0	fEnglish	same as &#x66F9;
U+3BE6.0	fUTF8	㯦
U+3BE6.0	fCNS11643	3-5136
U+3BE6.0	fR/S	 75.12
U+3BE6.0	fTotalStrokes	 16
U+3BE6.0	fCangjie	DTCD
U+3BE6.0	fFourCorner	44990
U+3BE6.0	fHakka	 ki2
U+3BE6.0	fCantonese	kei4
U+3BE6.0	fMandarin	qi2
U+3BE6.0	fEnglish	same as &#x68CA;
U+3BE7.0	fUTF8	㯧
U+3BE7.0	fGB	S-2168
U+3BE7.0	fR/S	 75.12
U+3BE7.0	fTotalStrokes	 16
U+3BE7.0	fCangjie	DGOR
U+3BE7.0	fCantonese	ciu1
U+3BE8.0	fUTF8	㯨
U+3BE8.0	fGB	7-234D
U+3BE8.0	fR/S	 75.12
U+3BE8.0	fTotalStrokes	 16
U+3BE8.0	fCangjie	DMWD
U+3BE8.0	fCantonese	suk1
U+3BE9.0	fUTF8	㯩
U+3BE9.0	fGB	3-4342
U+3BE9.0	fR/S	 75.12
U+3BE9.0	fTotalStrokes	 16
U+3BE9.0	fCangjie	EDKN
U+3BE9.0	fCantonese	tai3; daai3; taai3
U+3BE9.0	fMandarin	ti4
U+3BE9.0	fEnglish	put the growing up silkworms in different frames according to their sizes
U+3BEA.0	fUTF8	㯪
U+3BEA.0	fCNS11643	3-5522
U+3BEA.0	fGB	5-4573
U+3BEA.0	fR/S	 75.13
U+3BEA.0	fTotalStrokes	 17
U+3BEA.0	fCangjie	DMBI
U+3BEA.0	fFourCorner	41930
U+3BEA.0	fHakka	 lin2
U+3BEA.0	fCantonese	ling4
U+3BEA.0	fMandarin	ling2
U+3BEA.0	fEnglish	same as &#x6AFA;
U+3BEB.0	fUTF8	㯫
U+3BEB.0	fCNS11643	4-5648
U+3BEB.0	fGB	3-4347
U+3BEB.0	fR/S	 75.13
U+3BEB.0	fTotalStrokes	 17; 16
U+3BEB.0	fCangjie	DYPO
U+3BEB.0	fFourCorner	41930
U+3BEB.0	fCantonese	keoi4; ngaau1
U+3BEB.0	fMandarin	qu2
U+3BEB.0	fEnglish	a wattle or bamboo fence
U+3BEC.0	fUTF8	㯬
U+3BEC.0	fCNS11643	3-547E
U+3BEC.0	fGB	5-4556
U+3BEC.0	fR/S	 75.13
U+3BEC.0	fTotalStrokes	 17
U+3BEC.0	fCangjie	DDNKQ
U+3BEC.0	fFourCorner	44400
U+3BEC.0	fCantonese	lim5
U+3BEC.0	fMandarin	lian4; lian3
U+3BEC.0	fEnglish	achievements; merit
U+3BED.0	fUTF8	㯭
U+3BED.0	fCNS11643	3-5525
U+3BED.0	fR/S	 75.13
U+3BED.0	fTotalStrokes	 17
U+3BED.0	fCangjie	DYPS
U+3BED.0	fFourCorner	41920
U+3BED.0	fHakka	 lu3; lu1
U+3BED.0	fCantonese	lou5
U+3BED.0	fMandarin	lu3
U+3BED.0	fEnglish	same as &#x6AD3;
U+3BEE.0	fUTF8	㯮
U+3BEE.0	fCNS11643	3-547D
U+3BEE.0	fR/S	 75.13
U+3BEE.0	fTotalStrokes	 17
U+3BEE.0	fCangjie	DWLI
U+3BEE.0	fFourCorner	46920
U+3BEE.0	fCantonese	suk6; zik6
U+3BEE.0	fMandarin	shu2; shu3
U+3BEE.0	fEnglish	name of a tree
U+3BEF.0	fUTF8	㯯
U+3BEF.0	fCNS11643	4-564B
U+3BEF.0	fGB	5-452B
U+3BEF.0	fR/S	 75.13
U+3BEF.0	fTotalStrokes	 17; 16
U+3BEF.0	fCangjie	DHEC
U+3BEF.0	fFourCorner	47980
U+3BEF.0	fCantonese	gung3
U+3BEF.0	fMandarin	gong4; ju4
U+3BEF.0	fEnglish	[1] small cup [2] chest; box [3] a kind of container usually put on one's head to carry things
U+3BF0.0	fUTF8	㯰
U+3BF0.0	fCNS11643	7-3855
U+3BF0.0	fR/S	 75.13
U+3BF0.0	fTotalStrokes	 17
U+3BF0.0	fCangjie	DWLV
U+3BF0.0	fFourCorner	46910
U+3BF0.0	fHakka	 chit8
U+3BF0.0	fCantonese	zik6; cim1; dim1; zaat6
U+3BF0.0	fMandarin	zhi2; zhe2; shi4; zhi4
U+3BF0.0	fEnglish	same as &#x690D;
U+3BF1.0	fUTF8	㯱
U+3BF1.0	fCNS11643	5-5B68
U+3BF1.0	fGB	3-4346
U+3BF1.0	fR/S	 75.13
U+3BF1.0	fTotalStrokes	 17
U+3BF1.0	fCangjie	JBOUD
U+3BF1.0	fFourCorner	50900
U+3BF1.0	fCantonese	pou1; piu4
U+3BF1.0	fMandarin	piao2; biao3; pao1
U+3BF1.0	fEnglish	[1] enlarged bag [2] wrap up [3] parcel; bundle; package
U+3BF2.0	fUTF8	㯲
U+3BF2.0	fCNS11643	3-5477
U+3BF2.0	fGB	5-456E
U+3BF2.0	fR/S	 75.13
U+3BF2.0	fTotalStrokes	 17
U+3BF2.0	fCangjie	DDDF
U+3BF2.0	fFourCorner	54990
U+3BF2.0	fCantonese	gam3
U+3BF2.0	fMandarin	jin4
U+3BF2.0	fEnglish	[1] a wooden (bamboo) framework [2] a wooden pole, used as an obstacle to impede the door [3] frame
U+3BF3.0	fUTF8	㯳
U+3BF3.0	fCNS11643	3-5526
U+3BF3.0	fGB	5-4553
U+3BF3.0	fR/S	 75.13
U+3BF3.0	fTotalStrokes	 17
U+3BF3.0	fCangjie	DTRK
U+3BF3.0	fFourCorner	48940
U+3BF3.0	fHakka	 gin3; kiang2
U+3BF3.0	fCantonese	king4
U+3BF3.0	fMandarin	qing2; jing3; jing4
U+3BF3.0	fEnglish	same as &#x6AA0;
U+3BF4.0	fUTF8	㯴
U+3BF4.0	fCNS11643	F-5F29
U+3BF4.0	fGB	S-216B
U+3BF4.0	fR/S	 75.13
U+3BF4.0	fTotalStrokes	 17
U+3BF4.0	fCangjie	DTJR
U+3BF4.0	fFourCorner	5496
U+3BF4.0	fCantonese	jung4
U+3BF5.0	fUTF8	㯵
U+3BF5.0	fCNS11643	3-5479
U+3BF5.0	fR/S	 75.13
U+3BF5.0	fTotalStrokes	 17
U+3BF5.0	fCangjie	DTHG
U+3BF5.0	fFourCorner	44910
U+3BF5.0	fCantonese	dung2
U+3BF6.0	fUTF8	㯶
U+3BF6.0	fCNS11643	3-5528
U+3BF6.0	fGB	S-2176
U+3BF6.0	fR/S	 75.13
U+3BF6.0	fTotalStrokes	 17
U+3BF6.0	fCangjie	DUCE
U+3BF6.0	fFourCorner	42940
U+3BF6.0	fCantonese	zung1
U+3BF6.0	fMandarin	zong1
U+3BF6.0	fEnglish	the coir palm tree
U+3BF7.0	fUTF8	㯷
U+3BF7.0	fCNS11643	4-5B48
U+3BF7.0	fGB	3-4353
U+3BF7.0	fR/S	 75.14
U+3BF7.0	fTotalStrokes	 18
U+3BF7.0	fCangjie	DOTO
U+3BF7.0	fFourCorner	42930
U+3BF7.0	fCantonese	buk6; lok3; pok3
U+3BF7.0	fMandarin	bu3; pu2
U+3BF7.0	fEnglish	[1] dates [2] a kind of oak; Quercus dentata [3] same as &#x6A38;
U+3BF8.0	fUTF8	㯸
U+3BF8.0	fCNS11643	4-5B44
U+3BF8.0	fGB	5-4374
U+3BF8.0	fR/S	 75.14
U+3BF8.0	fTotalStrokes	 18
U+3BF8.0	fCangjie	DLMT
U+3BF8.0	fFourCorner	45910
U+3BF8.0	fCantonese	zeon2
U+3BF8.0	fMandarin	jin3; jin1
U+3BF8.0	fEnglish	 basin
U+3BF9.0	fUTF8	㯹
U+3BF9.0	fCNS11643	3-577C
U+3BF9.0	fR/S	 75.14
U+3BF9.0	fTotalStrokes	 18
U+3BF9.0	fCangjie	DMFI
U+3BF9.0	fFourCorner	44900
U+3BF9.0	fHakka	 piau3; piau5
U+3BF9.0	fCantonese	biu2; piu5; biu1; piu3
U+3BF9.0	fMandarin	biao1; piao4; biao3; piao1
U+3BF9.0	fEnglish	corrupted form of &#x527D;
U+3BFA.0	fUTF8	㯺
U+3BFA.0	fCNS11643	4-5B4A
U+3BFA.0	fGB	5-4355
U+3BFA.0	fR/S	 75.14
U+3BFA.0	fTotalStrokes	 18
U+3BFA.0	fCangjie	SWD
U+3BFA.0	fFourCorner	78900
U+3BFA.0	fCantonese	haam6; lam6
U+3BFA.0	fMandarin	han3; jian4; xian4
U+3BFA.0	fEnglish	[1] cabinet; wardrobe; cupboard [2] same as &#x58CF;
U+3BFB.0	fUTF8	㯻
U+3BFB.0	fCNS11643	4-5B49
U+3BFB.0	fGB	3-4350
U+3BFB.0	fR/S	 75.14
U+3BFB.0	fTotalStrokes	 18
U+3BFB.0	fCangjie	JBMOD
U+3BFB.0	fFourCorner	50900
U+3BFB.0	fCantonese	gwan2
U+3BFB.0	fMandarin	gun3; hun4; juan4
U+3BFB.0	fEnglish	 bind; tie up; bundle; bondage; restrictions; restraints
U+3BFC.0	fUTF8	㯼
U+3BFC.0	fGB	S-2174
U+3BFC.0	fR/S	 75.14
U+3BFC.0	fTotalStrokes	 18
U+3BFC.0	fCangjie	DTGT
U+3BFC.0	fCantonese	goi3
U+3BFD.0	fUTF8	㯽
U+3BFD.0	fR/S	 75.14
U+3BFD.0	fTotalStrokes	 18
U+3BFD.0	fCangjie	DJAC
U+3BFD.0	fCantonese	ban1; ping1; baan1
U+3BFD.0	fEnglish	[Korean character]
U+3BFE.0	fUTF8	㯾
U+3BFE.0	fCNS11643	4-5F6F
U+3BFE.0	fGB	3-434F
U+3BFE.0	fR/S	 75.15
U+3BFE.0	fTotalStrokes	 19
U+3BFE.0	fCangjie	DYTA
U+3BFE.0	fFourCorner	45930
U+3BFE.0	fCantonese	zou1
U+3BFE.0	fMandarin	zao1
U+3BFE.0	fEnglish	[1] name of a tree [2] same as $
U+3BFF.0	fUTF8	㯿
U+3BFF.0	fCNS11643	4-5F73
U+3BFF.0	fGB	3-435A
U+3BFF.0	fR/S	 75.15
U+3BFF.0	fTotalStrokes	 19
U+3BFF.0	fCangjie	DVVV
U+3BFF.0	fFourCorner	42910
U+3BFF.0	fCantonese	lip6
U+3BFF.0	fMandarin	lie4; ye4; la1
U+3BFF.0	fEnglish	[1] handle of a ladle [2] plant; wistaria, or wisteria, a certain tree in Sichuan, material for candles
U+3C00.0	fUTF8	㰀
U+3C00.0	fCNS11643	3-5A49
U+3C00.0	fR/S	 75.15
U+3C00.0	fTotalStrokes	 19
U+3C00.0	fCangjie	DHHE
U+3C00.0	fFourCorner	47930
U+3C00.0	fCantonese	lai4
U+3C00.0	fMandarin	li2
U+3C00.0	fEnglish	name of a tree
U+3C01.0	fUTF8	㰁
U+3C01.0	fCNS11643	4-5F70
U+3C01.0	fGB	3-4357
U+3C01.0	fR/S	 75.15
U+3C01.0	fTotalStrokes	 19
U+3C01.0	fCangjie	DMRR
U+3C01.0	fFourCorner	41960
U+3C01.0	fCantonese	lo5; lo2
U+3C01.0	fMandarin	luo3; luo4
U+3C01.0	fEnglish	a tree with slanted branches
U+3C02.0	fUTF8	㰂
U+3C02.0	fCNS11643	4-5F6E
U+3C02.0	fGB	3-4229
U+3C02.0	fR/S	 75.15
U+3C02.0	fTotalStrokes	 19
U+3C02.0	fCangjie	DJHW
U+3C02.0	fFourCorner	43960
U+3C02.0	fCantonese	sam2
U+3C02.0	fMandarin	shen3; sun3
U+3C02.0	fEnglish	[1] name of a tree [2] same as $
U+3C03.0	fUTF8	㰃
U+3C03.0	fCNS11643	4-5F72
U+3C03.0	fGB	3-4359
U+3C03.0	fR/S	 75.15
U+3C03.0	fTotalStrokes	 19
U+3C03.0	fCangjie	DHBF
U+3C03.0	fFourCorner	42990
U+3C03.0	fCantonese	min4
U+3C03.0	fMandarin	mian4; mian2
U+3C03.0	fEnglish	same as &#x6AB0;
U+3C04.0	fUTF8	㰄
U+3C04.0	fCNS11643	4-5F75
U+3C04.0	fGB	3-423F
U+3C04.0	fR/S	 75.15
U+3C04.0	fTotalStrokes	 19
U+3C04.0	fCangjie	DBCI
U+3C04.0	fFourCorner	43950
U+3C04.0	fCantonese	zin1; zin3
U+3C04.0	fMandarin	jian1; jian4
U+3C04.0	fEnglish	[1] (non-classical form) [2] a small chestnut tree [3] a kind of fruit; mountain plum
U+3C05.0	fUTF8	㰅
U+3C05.0	fCNS11643	3-5A4C
U+3C05.0	fR/S	 75.15
U+3C05.0	fTotalStrokes	 19
U+3C05.0	fCangjie	DYYB
U+3C05.0	fFourCorner	40930
U+3C05.0	fHakka	 dit7
U+3C05.0	fCantonese	dik1; dek6; zak6
U+3C05.0	fMandarin	di4; di2; zhe2
U+3C05.0	fEnglish	same as &#x6A00;
U+3C06.0	fUTF8	㰆
U+3C06.0	fCNS11643	3-5A4B
U+3C06.0	fR/S	 75.15
U+3C06.0	fTotalStrokes	 19
U+3C06.0	fCangjie	JBLYD
U+3C06.0	fFourCorner	50900
U+3C06.0	fCantonese	bai6; bei6
U+3C06.0	fMandarin	bai4; bei4
U+3C06.0	fEnglish	same as &#x292C2;
U+3C07.0	fUTF8	㰇
U+3C07.0	fCNS11643	F-6253
U+3C07.0	fGB	5-4628
U+3C07.0	fR/S	 75.15
U+3C07.0	fTotalStrokes	 19
U+3C07.0	fCangjie	DJIM
U+3C07.0	fFourCorner	4395
U+3C07.0	fCantonese	cim1; cim3; zim1
U+3C07.0	fEnglish	variant of &#x6AFC;
U+3C08.0	fUTF8	㰈
U+3C08.0	fCNS11643	3-5A4A
U+3C08.0	fR/S	 75.15
U+3C08.0	fTotalStrokes	 19
U+3C08.0	fCangjie	DDYJJ
U+3C08.0	fFourCorner	44309
U+3C08.0	fCantonese	lim5
U+3C08.0	fMandarin	lian3; lian4
U+3C08.0	fEnglish	nice and pretty
U+3C09.0	fUTF8	㰉
U+3C09.0	fCNS11643	3-5C4C
U+3C09.0	fR/S	 75.16
U+3C09.0	fTotalStrokes	 20
U+3C09.0	fCangjie	DMGD
U+3C09.0	fFourCorner	45990
U+3C09.0	fHakka	 cin2; zin1
U+3C09.0	fCantonese	zeon1
U+3C09.0	fMandarin	zhen1; jian1; qin2
U+3C09.0	fEnglish	same as &#x699B;
U+3C0A.0	fUTF8	㰊
U+3C0A.0	fCNS11643	3-5C46
U+3C0A.0	fR/S	 75.16
U+3C0A.0	fTotalStrokes	 20
U+3C0A.0	fCangjie	DGGF
U+3C0A.0	fFourCorner	49980
U+3C0A.0	fCantonese	cim4; caam4
U+3C0A.0	fMandarin	xuan2; xun2
U+3C0A.0	fEnglish	name of a tree
U+3C0B.0	fUTF8	㰋
U+3C0B.0	fCNS11643	4-6351
U+3C0B.0	fGB	5-4574
U+3C0B.0	fR/S	 75.16
U+3C0B.0	fTotalStrokes	 20
U+3C0B.0	fCangjie	DYHC
U+3C0B.0	fFourCorner	41980
U+3C0B.0	fHakka	 pin2
U+3C0B.0	fCantonese	pan4; paan4; pei1
U+3C0B.0	fMandarin	pin2
U+3C0B.0	fEnglish	the apple
U+3C0C.0	fUTF8	㰌
U+3C0C.0	fCNS11643	4-634F
U+3C0C.0	fGB	5-462F
U+3C0C.0	fR/S	 75.16
U+3C0C.0	fTotalStrokes	 20
U+3C0C.0	fCangjie	DMBG
U+3C0C.0	fFourCorner	41910
U+3C0C.0	fCantonese	kok3; gwok3; hok6
U+3C0C.0	fMandarin	que4
U+3C0C.0	fEnglish	name of a tree
U+3C0D.0	fUTF8	㰍
U+3C0D.0	fCNS11643	3-5C48
U+3C0D.0	fGB	5-586D
U+3C0D.0	fR/S	 75.16
U+3C0D.0	fTotalStrokes	 20
U+3C0D.0	fCangjie	YPD
U+3C0D.0	fFourCorner	01900
U+3C0D.0	fHakka	 lung2
U+3C0D.0	fCantonese	lung4
U+3C0D.0	fMandarin	long2
U+3C0D.0	fEnglish	[1] pen; cage [2] bars
U+3C0E.0	fUTF8	㰎
U+3C0E.0	fCNS11643	3-5C4D
U+3C0E.0	fR/S	 75.16
U+3C0E.0	fTotalStrokes	 20
U+3C0E.0	fCangjie	DUOS
U+3C0E.0	fFourCorner	42920
U+3C0E.0	fHakka	 zui1; zui5
U+3C0E.0	fCantonese	zeoi3
U+3C0E.0	fMandarin	cui2; zui4; zui1
U+3C0E.0	fEnglish	same as &#x6A87;
U+3C0F.0	fUTF8	㰏
U+3C0F.0	fCNS11643	4-6354
U+3C0F.0	fGB	5-4631
U+3C0F.0	fR/S	 75.16
U+3C0F.0	fTotalStrokes	 20
U+3C0F.0	fCangjie	DHYB
U+3C0F.0	fFourCorner	48920
U+3C0F.0	fHakka	 gau1; go1
U+3C0F.0	fCantonese	gou1
U+3C0F.0	fMandarin	gao1; gao4
U+3C0F.0	fEnglish	same as &#x7BD9;
U+3C10.0	fUTF8	㰐
U+3C10.0	fCNS11643	3-5C4B
U+3C10.0	fR/S	 75.16
U+3C10.0	fTotalStrokes	 20
U+3C10.0	fCangjie	DNLB
U+3C10.0	fFourCorner	44930
U+3C10.0	fHakka	 to3
U+3C10.0	fCantonese	to5; do6
U+3C10.0	fMandarin	tuo3; jue2; duo3; kui2; tuo2; lei3; tui3
U+3C10.0	fEnglish	same as &#x6A62;
U+3C11.0	fUTF8	㰑
U+3C11.0	fCNS11643	3-5C45
U+3C11.0	fR/S	 75.16
U+3C11.0	fTotalStrokes	 20
U+3C11.0	fCangjie	DPPD
U+3C11.0	fFourCorner	43990
U+3C11.0	fCantonese	saam1; san1
U+3C11.0	fMandarin	shan1
U+3C11.0	fEnglish	fruit tree
U+3C12.0	fUTF8	㰒
U+3C12.0	fCNS11643	F-653E
U+3C12.0	fR/S	 75.16
U+3C12.0	fTotalStrokes	 20
U+3C12.0	fCangjie	DHBD
U+3C12.0	fFourCorner	4794
U+3C12.0	fCantonese	hok6
U+3C12.0	fMandarin	she2; xue2
U+3C13.0	fUTF8	㰓
U+3C13.0	fR/S	 75.16
U+3C13.0	fTotalStrokes	 20
U+3C13.0	fCangjie	DSIC
U+3C13.0	fCantonese	laam5
U+3C13.0	fEnglish	[Japanese character]
U+3C14.0	fUTF8	㰔
U+3C14.0	fCNS11643	5-7027
U+3C14.0	fR/S	 75.17
U+3C14.0	fTotalStrokes	 21
U+3C14.0	fCangjie	DFFE
U+3C14.0	fFourCorner	49940
U+3C14.0	fCantonese	sip3; sit3
U+3C14.0	fMandarin	xie4
U+3C14.0	fEnglish	a kind of edible mushroom
U+3C15.0	fUTF8	㰕
U+3C15.0	fCNS11643	3-5E36
U+3C15.0	fR/S	 75.17; 75.16
U+3C15.0	fTotalStrokes	 21; 20
U+3C15.0	fCangjie	DTGS
U+3C15.0	fFourCorner	48950
U+3C15.0	fCantonese	hei1
U+3C15.0	fMandarin	xi1
U+3C15.0	fEnglish	same as &#x237F5;
U+3C16.0	fUTF8	㰖
U+3C16.0	fCNS11643	3-5C4A
U+3C16.0	fR/S	 75.16
U+3C16.0	fTotalStrokes	 20
U+3C16.0	fCangjie	DSIU
U+3C16.0	fFourCorner	48910
U+3C16.0	fHakka	 lam3
U+3C16.0	fCantonese	laam5; laam2
U+3C16.0	fMandarin	lan3
U+3C16.0	fEnglish	same as &#x6B16;
U+3C17.0	fUTF8	㰗
U+3C17.0	fCNS11643	3-5F50
U+3C17.0	fR/S	 75.18
U+3C17.0	fTotalStrokes	 22
U+3C17.0	fCangjie	DIHO
U+3C17.0	fFourCorner	43980
U+3C17.0	fHakka	 cuk7
U+3C17.0	fCantonese	cik1; cuk1
U+3C17.0	fMandarin	cu4; qi1; zu2; se4; zhou1
U+3C17.0	fEnglish	same as &#x69ED;
U+3C18.0	fUTF8	㰘
U+3C18.0	fCNS11643	4-686E
U+3C18.0	fGB	3-4361
U+3C18.0	fR/S	 75.18
U+3C18.0	fTotalStrokes	 22; 21
U+3C18.0	fCangjie	DNMT
U+3C18.0	fFourCorner	47940
U+3C18.0	fCantonese	ji4; zi4
U+3C18.0	fMandarin	yi2
U+3C18.0	fEnglish	name of a tree
U+3C19.0	fUTF8	㰙
U+3C19.0	fCNS11643	3-6035
U+3C19.0	fR/S	 75.19
U+3C19.0	fTotalStrokes	 23
U+3C19.0	fCangjie	DTOG
U+3C19.0	fFourCorner	40910
U+3C19.0	fCantonese	no4; lo4 (also); no5
U+3C19.0	fMandarin	nuo3; nuo2; luo3
U+3C19.0	fEnglish	same as &#x6A60;
U+3C1A.0	fUTF8	㰚
U+3C1A.0	fCNS11643	3-6033
U+3C1A.0	fR/S	 75.19
U+3C1A.0	fTotalStrokes	 23
U+3C1A.0	fCangjie	DYBG
U+3C1A.0	fFourCorner	40910
U+3C1A.0	fCantonese	lei4; laai4
U+3C1A.0	fMandarin	li2
U+3C1A.0	fEnglish	same as &#x7C6C;
U+3C1B.0	fUTF8	㰛
U+3C1B.0	fCNS11643	3-6034
U+3C1B.0	fR/S	 75.19
U+3C1B.0	fTotalStrokes	 23
U+3C1B.0	fCangjie	DTVD
U+3C1B.0	fFourCorner	44990
U+3C1B.0	fHakka	 lit8
U+3C1B.0	fCantonese	joek6; ngok6
U+3C1B.0	fMandarin	yao4; yue4
U+3C1B.0	fEnglish	name of a tree
U+3C1C.0	fUTF8	㰜
U+3C1C.0	fR/S	 75.19
U+3C1C.0	fTotalStrokes	 23
U+3C1C.0	fCangjie	DPCC
U+3C1C.0	fCantonese	din1
U+3C1C.0	fEnglish	[Korean character]
U+3C1D.0	fUTF8	㰝
U+3C1D.0	fCNS11643	5-2448
U+3C1D.0	fR/S	 76.3
U+3C1D.0	fTotalStrokes	 7
U+3C1D.0	fCangjie	RUNO
U+3C1D.0	fFourCorner	77780
U+3C1D.0	fCantonese	ji5; zyu4; zyu5
U+3C1D.0	fMandarin	yi3
U+3C1D.0	fEnglish	 cough; cough
U+3C1E.0	fUTF8	㰞
U+3C1E.0	fCNS11643	6-2830
U+3C1E.0	fR/S	 76.3
U+3C1E.0	fTotalStrokes	 7
U+3C1E.0	fCangjie	UNO
U+3C1E.0	fFourCorner	27780
U+3C1E.0	fHakka	 chi1; chii1
U+3C1E.0	fCantonese	ci1
U+3C1E.0	fMandarin	ji4; chi1
U+3C1E.0	fEnglish	same as &#x55E4;
U+3C1F.0	fUTF8	㰟
U+3C1F.0	fCNS11643	4-266D
U+3C1F.0	fGB	3-4C22
U+3C1F.0	fR/S	 76.4
U+3C1F.0	fTotalStrokes	 8
U+3C1F.0	fCangjie	ONNO
U+3C1F.0	fFourCorner	87780
U+3C1F.0	fCantonese	gat1; gei3
U+3C1F.0	fMandarin	ji4; qi4
U+3C1F.0	fEnglish	same as &#x5180;
U+3C20.0	fUTF8	㰠
U+3C20.0	fCNS11643	5-2656
U+3C20.0	fR/S	 76.4
U+3C20.0	fTotalStrokes	 8
U+3C20.0	fCangjie	YNNO
U+3C20.0	fFourCorner	07280
U+3C20.0	fCantonese	hong1; kong3
U+3C20.0	fMandarin	hang1; kang4
U+3C20.0	fEnglish	 covet; greedy
U+3C21.0	fUTF8	㰡
U+3C21.0	fCNS11643	4-2670
U+3C21.0	fGB	3-4E31
U+3C21.0	fR/S	 76.4
U+3C21.0	fTotalStrokes	 8
U+3C21.0	fCangjie	OLNO
U+3C21.0	fFourCorner	87280
U+3C21.0	fCantonese	haai3; hai6
U+3C21.0	fMandarin	xie4
U+3C21.0	fEnglish	[1] rash; impatient [2] very angry
U+3C22.0	fUTF8	㰢
U+3C22.0	fCNS11643	4-266C
U+3C22.0	fGB	3-4E30
U+3C22.0	fR/S	 76.4
U+3C22.0	fTotalStrokes	 8
U+3C22.0	fCangjie	MTNO
U+3C22.0	fFourCorner	17480
U+3C22.0	fCantonese	hang1
U+3C22.0	fMandarin	keng1
U+3C22.0	fEnglish	cough
U+3C23.0	fUTF8	㰣
U+3C23.0	fCNS11643	4-2D60
U+3C23.0	fGB	3-4E3F
U+3C23.0	fR/S	 76.5; 76.6
U+3C23.0	fTotalStrokes	 10; 9
U+3C23.0	fCangjie	YMPNO
U+3C23.0	fFourCorner	27180
U+3C23.0	fCantonese	zi1; zi3; zik1
U+3C23.0	fMandarin	zi4
U+3C23.0	fEnglish	[1] vomit; throw up; disgorge [2] hiccough; shortwindedness [3] supposed upward motion of the spirit (in Chinese herb medicine) [4] sigh with grief or regret; lament; deplore
U+3C24.0	fUTF8	㰤
U+3C24.0	fCNS11643	4-2970
U+3C24.0	fGB	3-4E35
U+3C24.0	fR/S	 76.5
U+3C24.0	fTotalStrokes	 9
U+3C24.0	fCangjie	MRNO
U+3C24.0	fFourCorner	17680
U+3C24.0	fHakka	 ho2
U+3C24.0	fCantonese	ho1; kaa3
U+3C24.0	fMandarin	ka4; ke3; he3; qia4; he4; he1
U+3C24.0	fEnglish	[1] laugh hertily; roar with laughter [2] breath
U+3C25.0	fUTF8	㰥
U+3C25.0	fCNS11643	4-2971
U+3C25.0	fGB	3-4E34
U+3C25.0	fR/S	 76.5
U+3C25.0	fTotalStrokes	 9
U+3C25.0	fCangjie	PTNO
U+3C25.0	fFourCorner	47780
U+3C25.0	fCantonese	hai3; waai3
U+3C25.0	fMandarin	xi4; hui4
U+3C25.0	fEnglish	laugh
U+3C26.0	fUTF8	㰦
U+3C26.0	fCNS11643	6-3178
U+3C26.0	fR/S	 76.5
U+3C26.0	fTotalStrokes	 9
U+3C26.0	fCangjie	GINO
U+3C26.0	fFourCorner	47780
U+3C26.0	fCantonese	heoi3
U+3C26.0	fMandarin	qu4; qie1
U+3C26.0	fEnglish	 breath with mouth open
U+3C27.0	fUTF8	㰧
U+3C27.0	fCNS11643	4-2974
U+3C27.0	fGB	3-4E39
U+3C27.0	fR/S	 76.5
U+3C27.0	fTotalStrokes	 9
U+3C27.0	fCangjie	IRNO
U+3C27.0	fFourCorner	27680
U+3C27.0	fCantonese	hoi1; hai6
U+3C27.0	fMandarin	hai1
U+3C27.0	fEnglish	[1] drink [2] swallow
U+3C28.0	fUTF8	㰨
U+3C28.0	fCNS11643	4-2973
U+3C28.0	fGB	3-4E36
U+3C28.0	fR/S	 76.5
U+3C28.0	fTotalStrokes	 9
U+3C28.0	fCangjie	RCNO
U+3C28.0	fFourCorner	67880
U+3C28.0	fCantonese	haai1; hei1
U+3C28.0	fMandarin	xi1
U+3C28.0	fEnglish	[1] take breath or rest (after strenuous exercise) [2] hiccough; shortwindedness
U+3C29.0	fUTF8	㰩
U+3C29.0	fCNS11643	5-2E2E
U+3C29.0	fR/S	 76.6
U+3C29.0	fTotalStrokes	 10
U+3C29.0	fCangjie	LINO
U+3C29.0	fFourCorner	57180
U+3C29.0	fCantonese	hoi1
U+3C29.0	fMandarin	hai1
U+3C29.0	fEnglish	the sound of laughter
U+3C2A.0	fUTF8	㰪
U+3C2A.0	fCNS11643	4-2D5D
U+3C2A.0	fGB	3-4E3A
U+3C2A.0	fR/S	 76.6
U+3C2A.0	fTotalStrokes	 10
U+3C2A.0	fCangjie	GGNO
U+3C2A.0	fFourCorner	47180
U+3C2A.0	fCantonese	gwai1; waa1
U+3C2A.0	fMandarin	wa1; wa2
U+3C2A.0	fEnglish	[1] sound [2] wicked; mean; vicious
U+3C2B.0	fUTF8	㰫
U+3C2B.0	fCNS11643	4-2D61
U+3C2B.0	fGB	3-4E41
U+3C2B.0	fR/S	 76.6
U+3C2B.0	fTotalStrokes	 10
U+3C2B.0	fCangjie	HUNO
U+3C2B.0	fFourCorner	27280
U+3C2B.0	fCantonese	caam1
U+3C2B.0	fMandarin	can1
U+3C2B.0	fEnglish	[1] glad; joyful; happy [2] avaricious; greedy [3] cold
U+3C2C.0	fUTF8	㰬
U+3C2C.0	fCNS11643	4-2D62
U+3C2C.0	fGB	3-4E43
U+3C2C.0	fR/S	 76.6
U+3C2C.0	fTotalStrokes	 10
U+3C2C.0	fCangjie	PANO
U+3C2C.0	fFourCorner	27680
U+3C2C.0	fCantonese	seon1
U+3C2C.0	fMandarin	xun2
U+3C2C.0	fEnglish	[1] shortwindedness; hiccough [2] supposed upward motion of the spirit (in Chinese herb medicine) [3] honesty; good faith; trust
U+3C2D.0	fUTF8	㰭
U+3C2D.0	fCNS11643	4-2D5E
U+3C2D.0	fGB	3-4E3D
U+3C2D.0	fR/S	 76.6
U+3C2D.0	fTotalStrokes	 10
U+3C2D.0	fCangjie	KSNO
U+3C2D.0	fFourCorner	47280
U+3C2D.0	fCantonese	heoi1; haap6
U+3C2D.0	fMandarin	xu1; hu1
U+3C2D.0	fEnglish	pleasure; joy
U+3C2E.0	fUTF8	㰮
U+3C2E.0	fCNS11643	3-3661
U+3C2E.0	fGB	5-6E39
U+3C2E.0	fR/S	 76.7
U+3C2E.0	fTotalStrokes	 11
U+3C2E.0	fCangjie	MVNO
U+3C2E.0	fFourCorner	77280
U+3C2E.0	fCantonese	san6; saan5
U+3C2E.0	fMandarin	shen4
U+3C2E.0	fEnglish	 laugh at
U+3C2F.0	fUTF8	㰯
U+3C2F.0	fCNS11643	4-3248
U+3C2F.0	fGB	3-6E44
U+3C2F.0	fR/S	 76.7
U+3C2F.0	fTotalStrokes	 11
U+3C2F.0	fCangjie	MTNO
U+3C2F.0	fFourCorner	17180
U+3C2F.0	fCantonese	tau3; kaau3
U+3C2F.0	fMandarin	tou4; kou4; hou4; pou3
U+3C2F.0	fEnglish	[1] the way of speaking [2] spit at and not agree about something [3] spit [4] same as &#x5426;
U+3C30.0	fUTF8	㰰
U+3C30.0	fCNS11643	4-3249
U+3C30.0	fGB	3-4E3E
U+3C30.0	fR/S	 76.7
U+3C30.0	fTotalStrokes	 11
U+3C30.0	fCangjie	KONO
U+3C30.0	fFourCorner	47080
U+3C30.0	fCantonese	gaap3; hap6; hot3
U+3C30.0	fMandarin	xia4; qie4
U+3C30.0	fEnglish	[1] asthmatic breathing [2] supposed upward motion of the spirit (in Chinese herb medicine) [3] long for; covet [4] hum over; moan; groan
U+3C31.0	fUTF8	㰱
U+3C31.0	fCNS11643	6-406E
U+3C31.0	fR/S	 76.7
U+3C31.0	fTotalStrokes	 11
U+3C31.0	fCangjie	HWNO
U+3C31.0	fFourCorner	27580
U+3C31.0	fHakka	 cap7
U+3C31.0	fCantonese	saap3; cip3; sap3
U+3C31.0	fMandarin	sha4
U+3C31.0	fEnglish	same as &#x6B43;
U+3C32.0	fUTF8	㰲
U+3C32.0	fCNS11643	4-383B
U+3C32.0	fGB	3-4E48
U+3C32.0	fR/S	 76.8
U+3C32.0	fTotalStrokes	 12
U+3C32.0	fCangjie	IMNO
U+3C32.0	fFourCorner	47180
U+3C32.0	fCantonese	juk1; zuk1
U+3C32.0	fMandarin	yu4; xu4; zhu2
U+3C32.0	fEnglish	[1] blow off [2] blow; puff
U+3C33.0	fUTF8	㰳
U+3C33.0	fCNS11643	4-3839
U+3C33.0	fGB	3-4E3C
U+3C33.0	fR/S	 76.8
U+3C33.0	fTotalStrokes	 12
U+3C33.0	fCangjie	MMNO
U+3C33.0	fFourCorner	17180
U+3C33.0	fHakka	 a5; et8; a3
U+3C33.0	fCantonese	aa3; ngaa3 (also); aa1
U+3C33.0	fMandarin	ya4; ya3; ya1
U+3C33.0	fEnglish	the braying of an ass
U+3C34.0	fUTF8	㰴
U+3C34.0	fCNS11643	3-3C46
U+3C34.0	fR/S	 76.8
U+3C34.0	fTotalStrokes	 12
U+3C34.0	fCangjie	YRNO
U+3C34.0	fFourCorner	07680
U+3C34.0	fCantonese	fau6; faau2
U+3C34.0	fMandarin	pou3; po2; pou4
U+3C34.0	fEnglish	(of statement) grate on the ear; earnest and faithful remonstrance
U+3C35.0	fUTF8	㰵
U+3C35.0	fCNS11643	5-3943
U+3C35.0	fR/S	 76.8
U+3C35.0	fTotalStrokes	 12
U+3C35.0	fCangjie	YJNO
U+3C35.0	fFourCorner	07480
U+3C35.0	fHakka	 zot7
U+3C35.0	fCantonese	zeot1
U+3C35.0	fMandarin	zu2
U+3C35.0	fEnglish	[1] suck [2] drink [3] swallow
U+3C36.0	fUTF8	㰶
U+3C36.0	fCNS11643	4-383C
U+3C36.0	fGB	3-4E4B
U+3C36.0	fR/S	 76.8
U+3C36.0	fTotalStrokes	 12
U+3C36.0	fCangjie	HRNO
U+3C36.0	fFourCorner	27680
U+3C36.0	fCantonese	jau2; zaau2
U+3C36.0	fMandarin	you3
U+3C36.0	fEnglish	[1] wrinkle the nose with sorrowful (when sob or weep) [2] same as &#x5614;
U+3C37.0	fUTF8	㰷
U+3C37.0	fCNS11643	6-3864
U+3C37.0	fGB	3-4E49
U+3C37.0	fR/S	 78.6
U+3C37.0	fTotalStrokes	 10; 9
U+3C37.0	fCangjie	MPNO
U+3C37.0	fFourCorner	17280
U+3C37.0	fCantonese	zi3
U+3C37.0	fMandarin	zi4
U+3C37.0	fEnglish	[1] resurrection; come to life again [2] sick
U+3C38.0	fUTF8	㰸
U+3C38.0	fR/S	 76.8
U+3C38.0	fTotalStrokes	 12
U+3C38.0	fCangjie	OONO
U+3C38.0	fCantonese	lim5; him3; leng5
U+3C38.0	fMandarin	lian3; lian4; luan3
U+3C38.0	fEnglish	[1] abbreviated form of &#x6B5B; [2] abbreviated form of &#x6582;
U+3C39.0	fUTF8	㰹
U+3C39.0	fCNS11643	4-3E21
U+3C39.0	fGB	3-4E50
U+3C39.0	fR/S	 76.9
U+3C39.0	fTotalStrokes	 13
U+3C39.0	fCangjie	IRNO
U+3C39.0	fFourCorner	57280
U+3C39.0	fCantonese	him1; kaam5
U+3C39.0	fMandarin	xian2; jin4; han1; xian1; han3; xian4
U+3C39.0	fEnglish	same as &#x6B26;
U+3C3A.0	fUTF8	㰺
U+3C3A.0	fCNS11643	4-3D7C
U+3C3A.0	fGB	3-4E53
U+3C3A.0	fR/S	 76.9
U+3C3A.0	fTotalStrokes	 13
U+3C3A.0	fCangjie	RSSEO
U+3C3A.0	fFourCorner	77280
U+3C3A.0	fCantonese	haa6; haa4; kwaai4
U+3C3A.0	fMandarin	xia2; xia4
U+3C3A.0	fEnglish	[1] drink [2] swallow [3] a disease of the throat; disease of the larynx
U+3C3B.0	fUTF8	㰻
U+3C3B.0	fCNS11643	4-3D7B
U+3C3B.0	fGB	3-4E4D
U+3C3B.0	fR/S	 76.9
U+3C3B.0	fTotalStrokes	 13
U+3C3B.0	fCangjie	GTNO
U+3C3B.0	fFourCorner	47180
U+3C3B.0	fHakka	 hi3
U+3C3B.0	fCantonese	ji2; hei1; zi5
U+3C3B.0	fMandarin	xi1; yi3; hou4
U+3C3B.0	fEnglish	[1] asses braying [2] be happy suddenly [3] cruel; malignant; coarse [4] rude
U+3C3C.0	fUTF8	㰼
U+3C3C.0	fCNS11643	4-3D7D
U+3C3C.0	fGB	3-4E47
U+3C3C.0	fR/S	 76.9
U+3C3C.0	fTotalStrokes	 13
U+3C3C.0	fCangjie	SONO
U+3C3C.0	fFourCorner	77780
U+3C3C.0	fCantonese	sip3; hap6; hau3; liu4; liu6; sap3
U+3C3C.0	fMandarin	sha4; qie4
U+3C3C.0	fEnglish	[1] greed; avarice [2] blow off
U+3C3D.0	fUTF8	㰽
U+3C3D.0	fCNS11643	4-3D7E
U+3C3D.0	fGB	3-4E4F
U+3C3D.0	fR/S	 76.9
U+3C3D.0	fTotalStrokes	 13
U+3C3D.0	fCangjie	SVNO
U+3C3D.0	fFourCorner	77780
U+3C3D.0	fCantonese	jin3; hin2; zin1; zin2
U+3C3D.0	fMandarin	yan4; mi3; yan3
U+3C3D.0	fEnglish	[1] exert oneself to yell; yell loudly; shout [2] overbearing; arrogant [3] vigorous
U+3C3E.0	fUTF8	㰾
U+3C3E.0	fCNS11643	4-3E22
U+3C3E.0	fGB	3-4E51
U+3C3E.0	fR/S	 76.9
U+3C3E.0	fTotalStrokes	 13
U+3C3E.0	fCangjie	HSNO
U+3C3E.0	fFourCorner	27280
U+3C3E.0	fCantonese	giu3
U+3C3E.0	fMandarin	jiao4
U+3C3E.0	fEnglish	[1] songs of the people of &#x695A; [2] a kind of music instrument in ancient times
U+3C3F.0	fUTF8	㰿
U+3C3F.0	fCNS11643	3-4752
U+3C3F.0	fR/S	 76.10
U+3C3F.0	fTotalStrokes	 14
U+3C3F.0	fCangjie	BKNO
U+3C3F.0	fFourCorner	27480
U+3C3F.0	fCantonese	hei1
U+3C3F.0	fMandarin	xi1
U+3C3F.0	fEnglish	[1] sigh in lamentation; lament [2] exclaim
U+3C40.0	fUTF8	㱀
U+3C40.0	fCNS11643	6-5D77
U+3C40.0	fR/S	 76.10
U+3C40.0	fTotalStrokes	 14
U+3C40.0	fCangjie	PUNO
U+3C40.0	fFourCorner	27480
U+3C40.0	fCantonese	zyut3; ci2
U+3C40.0	fMandarin	chi3; chuai4
U+3C40.0	fEnglish	[1] same as &#x6B3C; [2] same as &#x562C;
U+3C41.0	fUTF8	㱁
U+3C41.0	fCNS11643	4-442A
U+3C41.0	fGB	3-4E55
U+3C41.0	fR/S	 76.10
U+3C41.0	fTotalStrokes	 14
U+3C41.0	fCangjie	UFNO
U+3C41.0	fFourCorner	27980
U+3C41.0	fCantonese	seoi3; si6
U+3C41.0	fMandarin	sui4; shi4; kuan4
U+3C41.0	fEnglish	[1] ask; inquire; investigate [2] same as &#x6B3E;
U+3C42.0	fUTF8	㱂
U+3C42.0	fCNS11643	4-4A4C
U+3C42.0	fGB	5-503A
U+3C42.0	fR/S	 76.11
U+3C42.0	fTotalStrokes	 15
U+3C42.0	fCangjie	IENO
U+3C42.0	fFourCorner	07280
U+3C42.0	fCantonese	hong1; fong1
U+3C42.0	fMandarin	kang1
U+3C42.0	fEnglish	[1] hungry; starving [2] a poor harvest of grains
U+3C43.0	fUTF8	㱃
U+3C43.0	fCNS11643	3-4C68
U+3C43.0	fR/S	 76.11
U+3C43.0	fTotalStrokes	 15
U+3C43.0	fCangjie	OWNO
U+3C43.0	fFourCorner	87680
U+3C43.0	fCantonese	jam2; jam3; zaam2
U+3C43.0	fMandarin	yin3
U+3C43.0	fEnglish	ancient form of &#x98F2;
U+3C44.0	fUTF8	㱄
U+3C44.0	fCNS11643	4-5066
U+3C44.0	fGB	5-7A2F
U+3C44.0	fR/S	 76.12
U+3C44.0	fTotalStrokes	 16
U+3C44.0	fCangjie	WFNO
U+3C44.0	fFourCorner	67380
U+3C44.0	fHakka	 met8
U+3C44.0	fCantonese	hak1; gap3; haak1; maak6
U+3C44.0	fMandarin	hei1; mo4; he4
U+3C44.0	fEnglish	[1] spit [2] cough [3] keep quiet; keep silence; speechless; wordless
U+3C45.0	fUTF8	㱅
U+3C45.0	fCNS11643	4-5067
U+3C45.0	fR/S	 76.12
U+3C45.0	fTotalStrokes	 16
U+3C45.0	fCangjie	GTNO
U+3C45.0	fFourCorner	47180
U+3C45.0	fCantonese	ji3; zi3
U+3C45.0	fMandarin	yi4
U+3C45.0	fEnglish	ancient form of &#x61FF;
U+3C46.0	fUTF8	㱆
U+3C46.0	fCNS11643	4-564F
U+3C46.0	fGB	5-503F
U+3C46.0	fR/S	 76.13
U+3C46.0	fTotalStrokes	 17; 16
U+3C46.0	fCangjie	YTNO
U+3C46.0	fFourCorner	27280
U+3C46.0	fCantonese	hei1
U+3C46.0	fMandarin	xi1
U+3C46.0	fEnglish	 smile at each other
U+3C47.0	fUTF8	㱇
U+3C47.0	fCNS11643	4-564E
U+3C47.0	fGB	3-4E57
U+3C47.0	fR/S	 76.13
U+3C47.0	fTotalStrokes	 17; 16
U+3C47.0	fCangjie	GWNO
U+3C47.0	fFourCorner	47680
U+3C47.0	fCantonese	sik1; hei1; sam1; zip3
U+3C47.0	fMandarin	se4
U+3C47.0	fEnglish	[1] fear; dread; fright; scare [2] sad; sorrowful; mournful
U+3C48.0	fUTF8	㱈
U+3C48.0	fCNS11643	3-552A
U+3C48.0	fR/S	 76.13
U+3C48.0	fTotalStrokes	 17
U+3C48.0	fCangjie	DFNO
U+3C48.0	fFourCorner	47980
U+3C48.0	fCantonese	gam6; cim3; gaam3
U+3C48.0	fMandarin	jin4
U+3C48.0	fEnglish	the name of a person
U+3C49.0	fUTF8	㱉
U+3C49.0	fCNS11643	3-552C
U+3C49.0	fR/S	 76.13
U+3C49.0	fTotalStrokes	 17
U+3C49.0	fCangjie	TDNO
U+3C49.0	fFourCorner	37980
U+3C49.0	fCantonese	jip6; zip6
U+3C49.0	fMandarin	ye4
U+3C49.0	fEnglish	the moving of the mouth
U+3C4A.0	fUTF8	㱊
U+3C4A.0	fCNS11643	4-5F77
U+3C4A.0	fGB	3-4E5A
U+3C4A.0	fR/S	 76.15
U+3C4A.0	fTotalStrokes	 19
U+3C4A.0	fCangjie	MENO
U+3C4A.0	fFourCorner	17280
U+3C4A.0	fCantonese	jau1; zaau1
U+3C4A.0	fMandarin	you1
U+3C4A.0	fEnglish	[1] asthmatic breathing; shortwindedness; hiccough; husky voice [2] same as &#x55C4;
U+3C4B.0	fUTF8	㱋
U+3C4B.0	fCNS11643	7-4E22
U+3C4B.0	fR/S	 76.16
U+3C4B.0	fTotalStrokes	 20
U+3C4B.0	fCangjie	MGNO
U+3C4B.0	fFourCorner	17280
U+3C4B.0	fCantonese	kok3
U+3C4B.0	fMandarin	qiao1; que4
U+3C4B.0	fEnglish	same as &#x5859;, &#x78BA;
U+3C4C.0	fUTF8	㱌
U+3C4C.0	fCNS11643	4-6871
U+3C4C.0	fGB	3-4E54
U+3C4C.0	fR/S	 76.18
U+3C4C.0	fTotalStrokes	 22; 21
U+3C4C.0	fCangjie	SJNO
U+3C4C.0	fFourCorner	17180
U+3C4C.0	fCantonese	jip3; cit3; zim1
U+3C4C.0	fMandarin	xie4; che4; yan3; ye2
U+3C4C.0	fEnglish	[1] take [2] select [3] get angry; lose the temper
U+3C4D.0	fUTF8	㱍
U+3C4D.0	fCNS11643	5-7577
U+3C4D.0	fR/S	 76.19
U+3C4D.0	fTotalStrokes	 23
U+3C4D.0	fCangjie	VFYRO
U+3C4D.0	fFourCorner	27380
U+3C4D.0	fCantonese	lyun4
U+3C4D.0	fMandarin	luan2; juan4
U+3C4D.0	fEnglish	[1] owe [2] deficient [3] confused; puzzled;
U+3C4E.0	fUTF8	㱎
U+3C4E.0	fCNS11643	5-792F
U+3C4E.0	fGB	5-5040
U+3C4E.0	fR/S	 76.21
U+3C4E.0	fTotalStrokes	 25
U+3C4E.0	fCangjie	WFNO
U+3C4E.0	fFourCorner	67380
U+3C4E.0	fCantonese	gwan1; kwaan1
U+3C4E.0	fMandarin	kun1; gun1
U+3C4E.0	fEnglish	cannot be known; agnostic
U+3C4F.0	fUTF8	㱏
U+3C4F.0	fCNS11643	6-2525
U+3C4F.0	fR/S	 77.2
U+3C4F.0	fTotalStrokes	 6
U+3C4F.0	fCangjie	MMYM
U+3C4F.0	fFourCorner	10100
U+3C4F.0	fCantonese	zing3; zing6
U+3C4F.0	fMandarin	ding4; zheng4
U+3C4F.0	fEnglish	ancient form of &#x6B63;
U+3C50.0	fUTF8	㱐
U+3C50.0	fR/S	 77.3
U+3C50.0	fTotalStrokes	 7
U+3C50.0	fCangjie	MIMM
U+3C50.0	fCantonese	zing3
U+3C50.0	fEnglish	[Korean character]
U+3C51.0	fUTF8	㱑
U+3C51.0	fR/S	 77.3
U+3C51.0	fTotalStrokes	 7
U+3C51.0	fCangjie	YMNI
U+3C51.0	fCantonese	seoi3
U+3C51.0	fEnglish	[Korean character]
U+3C52.0	fUTF8	㱒
U+3C52.0	fCNS11643	3-2D3E
U+3C52.0	fR/S	 77.5
U+3C52.0	fTotalStrokes	 9
U+3C52.0	fCangjie	YMMNR
U+3C52.0	fFourCorner	21620
U+3C52.0	fHakka	 o1; ko1
U+3C52.0	fCantonese	ho1; soeng5
U+3C52.0	fMandarin	he1
U+3C52.0	fEnglish	ancient form of &#x8A36;
U+3C53.0	fUTF8	㱓
U+3C53.0	fR/S	 77.5
U+3C53.0	fTotalStrokes	 9
U+3C53.0	fCangjie	YMOII
U+3C53.0	fCantonese	ling4
U+3C53.0	fEnglish	[Korean character]
U+3C54.0	fUTF8	㱔
U+3C54.0	fCNS11643	3-2B7B
U+3C54.0	fR/S	 77.5
U+3C54.0	fTotalStrokes	 9
U+3C54.0	fCangjie	YPNI
U+3C54.0	fFourCorner	21200
U+3C54.0	fCantonese	se1
U+3C54.0	fMandarin	xie1
U+3C54.0	fEnglish	same as &#x4E9B;
U+3C55.0	fUTF8	㱕
U+3C55.0	fCNS11643	3-3C48
U+3C55.0	fGB	5-4868
U+3C55.0	fR/S	 77.8
U+3C55.0	fTotalStrokes	 12
U+3C55.0	fCangjie	YMSMB
U+3C55.0	fFourCorner	27120
U+3C55.0	fHakka	 gwi1
U+3C55.0	fCantonese	gwai1
U+3C55.0	fMandarin	gui1
U+3C55.0	fEnglish	large seal type of &#x6B78;
U+3C56.0	fUTF8	㱖
U+3C56.0	fCNS11643	4-383F
U+3C56.0	fGB	3-467A
U+3C56.0	fR/S	 77.8
U+3C56.0	fTotalStrokes	 12
U+3C56.0	fCangjie	YMYOJ
U+3C56.0	fFourCorner	20140
U+3C56.0	fCantonese	seoi6
U+3C56.0	fMandarin	cui4
U+3C56.0	fEnglish	[1] stay; stop (at a certain stage); desist [2] detain [3] prohibit; end [4] come to; stop at [5] still; calm [6] later
U+3C57.0	fUTF8	㱗
U+3C57.0	fCNS11643	5-465F
U+3C57.0	fR/S	 77.10
U+3C57.0	fTotalStrokes	 14
U+3C57.0	fCangjie	YMHUK
U+3C57.0	fFourCorner	26130
U+3C57.0	fCantonese	jau1; zaau1
U+3C57.0	fMandarin	xiu1
U+3C57.0	fEnglish	[1] a breath [2] news; tidings [3] stop; end
U+3C58.0	fUTF8	㱘
U+3C58.0	fCNS11643	3-5824
U+3C58.0	fR/S	 77.14
U+3C58.0	fTotalStrokes	 18
U+3C58.0	fCangjie	MKYLM
U+3C58.0	fFourCorner	71210
U+3C58.0	fCantonese	jim2; zim1; zim2
U+3C58.0	fMandarin	yan3; an4
U+3C58.0	fEnglish	same as &#x9B58;
U+3C59.0	fUTF8	㱙
U+3C59.0	fCNS11643	4-227C
U+3C59.0	fGB	5-4735
U+3C59.0	fR/S	 78.2
U+3C59.0	fTotalStrokes	 6
U+3C59.0	fCangjie	MNMVS
U+3C59.0	fFourCorner	11220
U+3C59.0	fHakka	  hiu3; hiu1
U+3C59.0	fCantonese	nau2; lau2 (also); jau2 (also); baai6; gwaa2; leoi5; naau2
U+3C59.0	fMandarin	xiu3; xiu4
U+3C59.0	fEnglish	[1] same as &#x673D; [2] same as &#x54BC;, &#x526E;
U+3C5A.0	fUTF8	㱚
U+3C5A.0	fCNS11643	3-2957
U+3C5A.0	fGB	5-473A
U+3C5A.0	fR/S	 78.4
U+3C5A.0	fTotalStrokes	 8
U+3C5A.0	fCangjie	MNB
U+3C5A.0	fFourCorner	17220
U+3C5A.0	fCantonese	caan4; can1
U+3C5A.0	fMandarin	can2; zan4; hai4; yue4; shan4; bo1
U+3C5A.0	fEnglish	same as &#x6B98;
U+3C5B.0	fUTF8	㱛
U+3C5B.0	fCNS11643	5-2662
U+3C5B.0	fR/S	 78.4
U+3C5B.0	fTotalStrokes	 8
U+3C5B.0	fCangjie	MNMNI
U+3C5B.0	fFourCorner	11220
U+3C5B.0	fCantonese	daai2; cyun2
U+3C5B.0	fMandarin	chuan3; bu4
U+3C5B.0	fEnglish	[1] (non-classical form) [2] destroy; injure; damage [3] exhaust [4] complete; finish [5] ancient form of &#x5E03;
U+3C5C.0	fUTF8	㱜
U+3C5C.0	fCNS11643	4-2675
U+3C5C.0	fGB	3-4475
U+3C5C.0	fR/S	 78.4
U+3C5C.0	fTotalStrokes	 8
U+3C5C.0	fCangjie	MNYO
U+3C5C.0	fFourCorner	11280
U+3C5C.0	fCantonese	zaat3
U+3C5C.0	fMandarin	zha2
U+3C5C.0	fEnglish	pestilence
U+3C5D.0	fUTF8	㱝
U+3C5D.0	fR/S	 78.4
U+3C5D.0	fTotalStrokes	 8
U+3C5D.0	fCangjie	MPKS
U+3C5D.0	fCantonese	sei2
U+3C5D.0	fEnglish	[Japanese character]
U+3C5E.0	fUTF8	㱞
U+3C5E.0	fCNS11643	3-2D3F
U+3C5E.0	fGB	5-4742
U+3C5E.0	fR/S	 78.5
U+3C5E.0	fTotalStrokes	 9
U+3C5E.0	fCangjie	MNYT
U+3C5E.0	fFourCorner	10210
U+3C5E.0	fCantonese	ngap6; ap6 (also); kaap1
U+3C5E.0	fMandarin	ji2; la1
U+3C5E.0	fEnglish	[1] danger; precarious; perilous [2] lofty; high [3] decay; break; snap
U+3C5F.0	fUTF8	㱟
U+3C5F.0	fCNS11643	3-2D41
U+3C5F.0	fGB	5-4744
U+3C5F.0	fR/S	 78.5
U+3C5F.0	fTotalStrokes	 9
U+3C5F.0	fCangjie	MNDHE
U+3C5F.0	fFourCorner	14240
U+3C5F.0	fCantonese	pei1
U+3C5F.0	fMandarin	pi3; bo2; pi1; pi2; bei4
U+3C5F.0	fEnglish	[1] rip open the flesh [2] break; snap
U+3C60.0	fUTF8	㱠
U+3C60.0	fCNS11643	3-2D40
U+3C60.0	fGB	5-473D
U+3C60.0	fR/S	 78.5
U+3C60.0	fTotalStrokes	 9
U+3C60.0	fCangjie	MNJR
U+3C60.0	fFourCorner	14260
U+3C60.0	fCantonese	fu1
U+3C60.0	fMandarin	ku1
U+3C60.0	fEnglish	[1] withered; dry [2] calamity; disaster [3] ancient form of &#x8F9C;
U+3C61.0	fUTF8	㱡
U+3C61.0	fCNS11643	3-3169
U+3C61.0	fR/S	 78.6
U+3C61.0	fTotalStrokes	 10
U+3C61.0	fCangjie	MNHGU
U+3C61.0	fFourCorner	14210
U+3C61.0	fCantonese	sing1
U+3C61.0	fMandarin	sheng1; sheng3
U+3C61.0	fEnglish	the soul out of one's body; as good as dead,
U+3C62.0	fUTF8	㱢
U+3C62.0	fCNS11643	3-3663
U+3C62.0	fR/S	 78.7
U+3C62.0	fTotalStrokes	 11
U+3C62.0	fCangjie	MNIAV
U+3C62.0	fFourCorner	13230
U+3C62.0	fCantonese	long4
U+3C62.0	fMandarin	lang2
U+3C62.0	fEnglish	the dead
U+3C63.0	fUTF8	㱣
U+3C63.0	fCNS11643	4-324A
U+3C63.0	fGB	3-447A
U+3C63.0	fR/S	 78.7
U+3C63.0	fTotalStrokes	 11
U+3C63.0	fCangjie	MNBV
U+3C63.0	fFourCorner	12240
U+3C63.0	fCantonese	teoi2
U+3C63.0	fMandarin	tui3
U+3C63.0	fEnglish	[1] uneven [2] weak; feeble
U+3C64.0	fUTF8	㱤
U+3C64.0	fCNS11643	4-3845
U+3C64.0	fGB	3-447D
U+3C64.0	fR/S	 78.8
U+3C64.0	fTotalStrokes	 12
U+3C64.0	fCangjie	MNDHL
U+3C64.0	fFourCorner	11220
U+3C64.0	fCantonese	sik1; sek6; ze3
U+3C64.0	fMandarin	xi2
U+3C64.0	fEnglish	[1] exhaust; extreme; topmost [2] die
U+3C65.0	fUTF8	㱥
U+3C65.0	fCNS11643	4-3844
U+3C65.0	fGB	3-447C
U+3C65.0	fR/S	 78.8
U+3C65.0	fTotalStrokes	 12
U+3C65.0	fCangjie	MNGCE
U+3C65.0	fFourCorner	14240
U+3C65.0	fHakka	 lin2
U+3C65.0	fCantonese	ling4
U+3C65.0	fMandarin	ling2; leng4
U+3C65.0	fEnglish	[1] exhausted [2] dying
U+3C66.0	fUTF8	㱦
U+3C66.0	fCNS11643	4-3846
U+3C66.0	fGB	3-4521
U+3C66.0	fR/S	 78.8
U+3C66.0	fTotalStrokes	 12
U+3C66.0	fCangjie	MNKMR
U+3C66.0	fFourCorner	14220
U+3C66.0	fCantonese	kei1; gei1
U+3C66.0	fMandarin	qi1; ji3; e4; gui4; ji1
U+3C66.0	fEnglish	[1] discard [2] abandon [3] die; death
U+3C67.0	fUTF8	㱧
U+3C67.0	fCNS11643	4-3842
U+3C67.0	fGB	3-4525
U+3C67.0	fR/S	 78.8
U+3C67.0	fTotalStrokes	 12
U+3C67.0	fCangjie	MNJNU
U+3C67.0	fFourCorner	13210
U+3C67.0	fHakka	 wan3
U+3C67.0	fCantonese	waat3; zyun2
U+3C67.0	fMandarin	yuan4; wo4
U+3C67.0	fEnglish	[1] stinking smell [2] expression of a dead person
U+3C68.0	fUTF8	㱨
U+3C68.0	fCNS11643	3-3C4A
U+3C68.0	fGB	5-474C
U+3C68.0	fR/S	 78.8
U+3C68.0	fTotalStrokes	 12
U+3C68.0	fCangjie	MNJMO
U+3C68.0	fFourCorner	13280
U+3C68.0	fCantonese	lim5; lim6 (also)
U+3C68.0	fMandarin	lian4
U+3C68.0	fEnglish	same as &#x6BAE;
U+3C69.0	fUTF8	㱩
U+3C69.0	fGB	S-2376
U+3C69.0	fR/S	 78.8
U+3C69.0	fTotalStrokes	 12
U+3C69.0	fCangjie	MNJNK
U+3C69.0	fCantonese	duk6; caan4
U+3C69.0	fMandarin	du2
U+3C69.0	fEnglish	simplified form of &#x6BB0;
U+3C6A.0	fUTF8	㱪
U+3C6A.0	fCNS11643	3-4223
U+3C6A.0	fGB	5-4751
U+3C6A.0	fR/S	 78.9
U+3C6A.0	fTotalStrokes	 13
U+3C6A.0	fCangjie	MNRPA
U+3C6A.0	fFourCorner	17260
U+3C6A.0	fCantonese	fan1; mun6
U+3C6A.0	fMandarin	hun1; men4
U+3C6A.0	fEnglish	standard form of &#x6B99;
U+3C6B.0	fUTF8	㱫
U+3C6B.0	fCNS11643	4-3E23
U+3C6B.0	fGB	5-474D
U+3C6B.0	fR/S	 78.9
U+3C6B.0	fTotalStrokes	 13
U+3C6B.0	fCangjie	MNDWF
U+3C6B.0	fFourCorner	15290
U+3C6B.0	fCantonese	laan6
U+3C6B.0	fMandarin	lan4
U+3C6B.0	fEnglish	destroyed or ruined; rot; decay; disintegrate; decompose
U+3C6C.0	fUTF8	㱬
U+3C6C.0	fCNS11643	4-3E24
U+3C6C.0	fGB	3-452B
U+3C6C.0	fR/S	 78.9
U+3C6C.0	fTotalStrokes	 13
U+3C6C.0	fCangjie	MNWMV
U+3C6C.0	fFourCorner	16230
U+3C6C.0	fCantonese	wui2; waai2
U+3C6C.0	fMandarin	wei3
U+3C6C.0	fEnglish	cannot recognize people
U+3C6D.0	fUTF8	㱭
U+3C6D.0	fCNS11643	3-4224
U+3C6D.0	fGB	5-474F
U+3C6D.0	fR/S	 78.9
U+3C6D.0	fTotalStrokes	 13
U+3C6D.0	fCangjie	MNHJE
U+3C6D.0	fFourCorner	17240
U+3C6D.0	fCantonese	dyun6
U+3C6D.0	fMandarin	duan4
U+3C6D.0	fEnglish	same as &#x6BC8;
U+3C6E.0	fUTF8	㱮
U+3C6E.0	fGB	7-2221
U+3C6E.0	fR/S	 78.9
U+3C6E.0	fTotalStrokes	 13
U+3C6E.0	fCangjie	MNLMO
U+3C6E.0	fCantonese	kui2; kwui2 (also)
U+3C6E.0	fMandarin	kuai4; kui4
U+3C6E.0	fEnglish	simplified form of &#x6BA8;
U+3C6F.0	fUTF8	㱯
U+3C6F.0	fCNS11643	4-4432
U+3C6F.0	fGB	5-4745
U+3C6F.0	fR/S	 78.10
U+3C6F.0	fTotalStrokes	 14
U+3C6F.0	fCangjie	MNUMT
U+3C6F.0	fFourCorner	12210
U+3C6F.0	fCantonese	ngoi4; oi4 (also); goi1
U+3C6F.0	fMandarin	ai2; gai1; bei4; ji4; jian4; shan3; shen4; yi3
U+3C6F.0	fEnglish	get the unborn baby goat by killng the mother goat
U+3C70.0	fUTF8	㱰
U+3C70.0	fCNS11643	5-4660
U+3C70.0	fR/S	 78.10
U+3C70.0	fTotalStrokes	 14
U+3C70.0	fCangjie	MNJYJ
U+3C70.0	fFourCorner	13240
U+3C70.0	fCantonese	zoi2
U+3C70.0	fMandarin	zai3
U+3C70.0	fEnglish	[1] destroy; ruin [2] put off
U+3C71.0	fUTF8	㱱
U+3C71.0	fCNS11643	4-4433
U+3C71.0	fGB	3-452C
U+3C71.0	fR/S	 78.10
U+3C71.0	fTotalStrokes	 14
U+3C71.0	fCangjie	MNHUI
U+3C71.0	fFourCorner	16210
U+3C71.0	fHakka	 kui5; kui6; wui3; wi3; k'wui5; vui3
U+3C71.0	fCantonese	wui6; wui5
U+3C71.0	fMandarin	hui4; wu4; xi4
U+3C71.0	fEnglish	royal; stony; rocky
U+3C72.0	fUTF8	㱲
U+3C72.0	fCNS11643	4-4430
U+3C72.0	fGB	3-452E
U+3C72.0	fR/S	 78.10
U+3C72.0	fTotalStrokes	 14
U+3C72.0	fCangjie	MNTCT
U+3C72.0	fFourCorner	18210
U+3C72.0	fCantonese	ji3; zik1
U+3C72.0	fMandarin	yi4
U+3C72.0	fEnglish	[1] withered; faded; die [2] a minor illness (of foot and hand)
U+3C73.0	fUTF8	㱳
U+3C73.0	fCNS11643	4-4A50
U+3C73.0	fGB	5-4755
U+3C73.0	fR/S	 78.11
U+3C73.0	fTotalStrokes	 15
U+3C73.0	fCangjie	TAKN
U+3C73.0	fFourCorner	44200
U+3C73.0	fCantonese	mok6
U+3C73.0	fMandarin	mo4
U+3C73.0	fEnglish	lonely; lonesome
U+3C74.0	fUTF8	㱴
U+3C74.0	fCNS11643	4-4A4F
U+3C74.0	fGB	3-447B
U+3C74.0	fR/S	 78.11
U+3C74.0	fTotalStrokes	 15
U+3C74.0	fCangjie	MNQMC
U+3C74.0	fFourCorner	15280
U+3C74.0	fCantonese	zi6; zi3
U+3C74.0	fMandarin	zi4
U+3C74.0	fEnglish	[1] sick [2] dead animal [3] skeleton
U+3C75.0	fUTF8	㱵
U+3C75.0	fCNS11643	4-506B
U+3C75.0	fGB	3-4527
U+3C75.0	fR/S	 78.12
U+3C75.0	fTotalStrokes	 16
U+3C75.0	fCangjie	MNJTC
U+3C75.0	fFourCorner	14280
U+3C75.0	fCantonese	fan3
U+3C75.0	fMandarin	fen4; ben4
U+3C75.0	fEnglish	[1] die [2] open (as an ulcer or sore)
U+3C76.0	fUTF8	㱶
U+3C76.0	fCNS11643	4-506A
U+3C76.0	fGB	3-4531
U+3C76.0	fR/S	 78.12
U+3C76.0	fTotalStrokes	 16
U+3C76.0	fCangjie	MNGTH
U+3C76.0	fFourCorner	12220
U+3C76.0	fCantonese	pang4
U+3C76.0	fMandarin	pang2; peng2; beng4; jiao4; qiao3; ru4
U+3C76.0	fEnglish	[1] swollen corpse [2] swell
U+3C77.0	fUTF8	㱷
U+3C77.0	fR/S	 78.12
U+3C77.0	fTotalStrokes	 16
U+3C77.0	fCangjie	MNOGD
U+3C77.0	fCantonese	zaap6
U+3C77.0	fEnglish	[Korean character]
U+3C78.0	fUTF8	㱸
U+3C78.0	fCNS11643	4-5651
U+3C78.0	fGB	3-4533
U+3C78.0	fR/S	 78.13
U+3C78.0	fTotalStrokes	 17; 16
U+3C78.0	fCangjie	MNSRJ
U+3C78.0	fFourCorner	10240
U+3C78.0	fCantonese	pik1; bik1
U+3C78.0	fMandarin	bi4; bie2
U+3C78.0	fEnglish	[1] exhaust; extreme; highest; farthest [2] die
U+3C79.0	fUTF8	㱹
U+3C79.0	fCNS11643	4-6355
U+3C79.0	fGB	3-4474
U+3C79.0	fR/S	 78.16
U+3C79.0	fTotalStrokes	 20
U+3C79.0	fCangjie	MNMHM
U+3C79.0	fFourCorner	11210
U+3C79.0	fCantonese	lik6
U+3C79.0	fMandarin	li4; suan4; xian4
U+3C79.0	fEnglish	 die from disease
U+3C7A.0	fUTF8	㱺
U+3C7A.0	fCNS11643	3-5C4E
U+3C7A.0	fR/S	 78.16
U+3C7A.0	fTotalStrokes	 20
U+3C7A.0	fCangjie	MNYPT
U+3C7A.0	fFourCorner	11210
U+3C7A.0	fCantonese	leoi4; lou4
U+3C7A.0	fMandarin	lu2
U+3C7A.0	fEnglish	interchangeable with &#x81DA;
U+3C7B.0	fUTF8	㱻
U+3C7B.0	fCNS11643	5-7578
U+3C7B.0	fGB	3-4534
U+3C7B.0	fR/S	 78.19
U+3C7B.0	fTotalStrokes	 23; 24
U+3C7B.0	fCangjie	MNYNQ
U+3C7B.0	fFourCorner	10210
U+3C7B.0	fCantonese	lo6; lo2
U+3C7B.0	fMandarin	luo3; luo4; lei4
U+3C7B.0	fEnglish	[1] diseases of dumb creatures [2] diseases of animals [3] illness; disease; ailment
U+3C7C.0	fUTF8	㱼
U+3C7C.0	fCNS11643	5-244E
U+3C7C.0	fR/S	 79.3
U+3C7C.0	fTotalStrokes	 7
U+3C7C.0	fCangjie	RUHNE
U+3C7C.0	fFourCorner	77740
U+3C7C.0	fCantonese	hoi1
U+3C7C.0	fMandarin	hai1; gai3; ai3
U+3C7C.0	fEnglish	ornaments that people wore in ancient time in order to avoid evil spirits or influences
U+3C7D.0	fUTF8	㱽
U+3C7D.0	fCNS11643	4-2678
U+3C7D.0	fGB	3-4F28
U+3C7D.0	fR/S	 79.4
U+3C7D.0	fTotalStrokes	 8
U+3C7D.0	fCangjie	LUHNE
U+3C7D.0	fFourCorner	47047
U+3C7D.0	fCantonese	dam2
U+3C7D.0	fMandarin	zhen3; qin2; dan4; zhen4; dan3
U+3C7D.0	fEnglish	same as &#x628C;
U+3C7E.0	fUTF8	㱾
U+3C7E.0	fCNS11643	4-2D64
U+3C7E.0	fGB	3-4F2A
U+3C7E.0	fR/S	 79.6
U+3C7E.0	fTotalStrokes	 10
U+3C7E.0	fCangjie	YOHNE
U+3C7E.0	fFourCorner	07840
U+3C7E.0	fCantonese	goi1; hoi1
U+3C7E.0	fMandarin	gai1; hai4; kai1
U+3C7E.0	fEnglish	ornaments that people wore in ancient time in order to avoid evil spirits or influences, sound of laughter
U+3C7F.0	fUTF8	㱿
U+3C7F.0	fCNS11643	3-316B
U+3C7F.0	fGB	5-5063
U+3C7F.0	fR/S	 79.6
U+3C7F.0	fTotalStrokes	 10
U+3C7F.0	fCangjie	GBHNE
U+3C7F.0	fFourCorner	47240
U+3C7F.0	fHakka	 kok7; hok7
U+3C7F.0	fCantonese	kok3
U+3C7F.0	fMandarin	que4
U+3C7F.0	fEnglish	[1] interchangeable with &#x6BBC; [2] interchangeable with &#x6164; [3] same as &#x55C0;
U+3C80.0	fUTF8	㲀
U+3C80.0	fCNS11643	4-324F
U+3C80.0	fGB	3-6F2A
U+3C80.0	fR/S	 79.7
U+3C80.0	fTotalStrokes	 11
U+3C80.0	fCangjie	MVHNE
U+3C80.0	fFourCorner	77240
U+3C80.0	fCantonese	zan1; daai2; saan4; zaan1; zeon1
U+3C80.0	fMandarin	zhen1; chen2
U+3C80.0	fEnglish	[1] beat; strike; attack [2] move with happiness; excited [3] (corrupted form) laugh at
U+3C81.0	fUTF8	㲁
U+3C81.0	fCNS11643	4-3849
U+3C81.0	fGB	3-4F2D
U+3C81.0	fR/S	 79.8
U+3C81.0	fTotalStrokes	 12
U+3C81.0	fCangjie	JMHNE
U+3C81.0	fFourCorner	37140
U+3C81.0	fCantonese	hung1
U+3C81.0	fMandarin	kong1; zhong1
U+3C81.0	fEnglish	[1] beat; strike; attack [2] control [3] charge; sue [4] kill all
U+3C82.0	fUTF8	㲂
U+3C82.0	fCNS11643	4-384A
U+3C82.0	fGB	3-4F2C
U+3C82.0	fR/S	 79.8
U+3C82.0	fTotalStrokes	 12
U+3C82.0	fCangjie	FBHNE
U+3C82.0	fFourCorner	97240
U+3C82.0	fCantonese	cang4; cang2
U+3C82.0	fMandarin	cheng2
U+3C82.0	fEnglish	[1] push [2] unceremoniously; impudently
U+3C83.0	fUTF8	㲃
U+3C83.0	fCNS11643	4-384B
U+3C83.0	fGB	5-506A
U+3C83.0	fR/S	 79.8
U+3C83.0	fTotalStrokes	 12
U+3C83.0	fCangjie	YPHNE
U+3C83.0	fFourCorner	27740
U+3C83.0	fCantonese	gau3; gaau3; kaau3
U+3C83.0	fMandarin	jiu4; qiu1
U+3C83.0	fEnglish	[1] smooth and curved [2] strike or beat heavily
U+3C84.0	fUTF8	㲄
U+3C84.0	fCNS11643	3-4225
U+3C84.0	fR/S	 79.9
U+3C84.0	fTotalStrokes	 13
U+3C84.0	fCangjie	GGHNE
U+3C84.0	fFourCorner	47140
U+3C84.0	fCantonese	huk1
U+3C84.0	fMandarin	kou4
U+3C84.0	fEnglish	unburnt bricks or tiles
U+3C85.0	fUTF8	㲅
U+3C85.0	fCNS11643	4-4435
U+3C85.0	fGB	3-4F36
U+3C85.0	fR/S	 79.10
U+3C85.0	fTotalStrokes	 14
U+3C85.0	fCangjie	JRHNE
U+3C85.0	fFourCorner	37640
U+3C85.0	fCantonese	gai3
U+3C85.0	fMandarin	ji4; kai4
U+3C85.0	fEnglish	[1] bind [2] exhaust; use up; complete; finish [3] slay
U+3C86.0	fUTF8	㲆
U+3C86.0	fCNS11643	3-5147
U+3C86.0	fR/S	 79.12
U+3C86.0	fTotalStrokes	 16
U+3C86.0	fCangjie	GEOII
U+3C86.0	fFourCorner	47300
U+3C86.0	fCantonese	ling4
U+3C86.0	fMandarin	ling2
U+3C86.0	fEnglish	noise
U+3C87.0	fUTF8	㲇
U+3C87.0	fCNS11643	3-5148
U+3C87.0	fR/S	 79.12
U+3C87.0	fTotalStrokes	 16
U+3C87.0	fCangjie	GEHEY
U+3C87.0	fFourCorner	47300
U+3C87.0	fCantonese	dung1; ci1; dung3
U+3C87.0	fMandarin	dong1
U+3C87.0	fEnglish	non-classical form of &#x9F15;
U+3C88.0	fUTF8	㲈
U+3C88.0	fCNS11643	4-506E
U+3C88.0	fGB	5-506F
U+3C88.0	fR/S	 79.12
U+3C88.0	fTotalStrokes	 16
U+3C88.0	fCangjie	GESHR
U+3C88.0	fFourCorner	47600
U+3C88.0	fCantonese	siu4
U+3C88.0	fMandarin	shao2; tao2
U+3C88.0	fEnglish	[1] same as &#x97F6; [2] large seal type of &#x9780;, &#x5A78;
U+3C89.0	fUTF8	㲉
U+3C89.0	fCNS11643	4-5653
U+3C89.0	fGB	3-4F3D
U+3C89.0	fR/S	 79.13
U+3C89.0	fTotalStrokes	 17; 16
U+3C89.0	fCangjie	GLHNE
U+3C89.0	fFourCorner	47740
U+3C89.0	fHakka	 kok7
U+3C89.0	fCantonese	kok3; dik1; hok3
U+3C89.0	fMandarin	que4; kai4; ku1; ke2
U+3C89.0	fEnglish	egg shells
U+3C8A.0	fUTF8	㲊
U+3C8A.0	fCNS11643	3-5825
U+3C8A.0	fR/S	 79.14
U+3C8A.0	fTotalStrokes	 18
U+3C8A.0	fCangjie	YUHNE
U+3C8A.0	fFourCorner	27240
U+3C8A.0	fCantonese	jeoi6; zeoi6
U+3C8A.0	fMandarin	rui4
U+3C8A.0	fEnglish	corrupted form of &#x53E1;
U+3C8B.0	fUTF8	㲋
U+3C8B.0	fCNS11643	4-297B
U+3C8B.0	fGB	3-326E
U+3C8B.0	fR/S	 81.5
U+3C8B.0	fTotalStrokes	 9
U+3C8B.0	fCangjie	NRPP
U+3C8B.0	fFourCorner	27710
U+3C8B.0	fCantonese	coek3
U+3C8B.0	fMandarin	chuo4; zhuo2; zu2
U+3C8B.0	fEnglish	a kind of animal (like a rabbit, it is blue colored and much bigger)
U+3C8C.0	fUTF8	㲌
U+3C8C.0	fCNS11643	3-237B
U+3C8C.0	fR/S	 82.2
U+3C8C.0	fTotalStrokes	 6
U+3C8C.0	fCangjie	HUNHS
U+3C8C.0	fFourCorner	27710
U+3C8C.0	fCantonese	ning6; ling6 (also)
U+3C8C.0	fMandarin	ning4; neng4
U+3C8C.0	fEnglish	dog's hair
U+3C8D.0	fUTF8	㲍
U+3C8D.0	fCNS11643	4-2679
U+3C8D.0	fGB	3-4365
U+3C8D.0	fR/S	 82.4
U+3C8D.0	fTotalStrokes	 8
U+3C8D.0	fCangjie	JEHQU
U+3C8D.0	fFourCorner	42401
U+3C8D.0	fHakka	 zhi1; zih1
U+3C8D.0	fCantonese	zi1
U+3C8D.0	fMandarin	zhi1
U+3C8D.0	fEnglish	down; soft plush
U+3C8E.0	fUTF8	㲎
U+3C8E.0	fCNS11643	3-295B
U+3C8E.0	fGB	5-4E29
U+3C8E.0	fR/S	 82.4
U+3C8E.0	fTotalStrokes	 8
U+3C8E.0	fCangjie	HUHQU
U+3C8E.0	fFourCorner	22710
U+3C8E.0	fCantonese	leoi4; daau1; laau4; syu1; zyu4
U+3C8E.0	fMandarin	luu2; lou2
U+3C8E.0	fEnglish	[1] non-classical form of &#x6C00; [2] non-classical form of &#x6BF9;
U+3C8F.0	fUTF8	㲏
U+3C8F.0	fCNS11643	4-267A
U+3C8F.0	fGB	3-4C2B
U+3C8F.0	fR/S	 82.4
U+3C8F.0	fTotalStrokes	 8
U+3C8F.0	fCangjie	LLHQU
U+3C8F.0	fFourCorner	22010
U+3C8F.0	fCantonese	pou1; piu5
U+3C8F.0	fMandarin	pao1; bao3; pao4; piao3; pin2; ping4
U+3C8F.0	fEnglish	[1] flying feather [2] soft; light
U+3C90.0	fUTF8	㲐
U+3C90.0	fGB	S-2434
U+3C90.0	fR/S	 82.4
U+3C90.0	fTotalStrokes	 8
U+3C90.0	fCangjie	HUOIN
U+3C90.0	fCantonese	gam1
U+3C91.0	fUTF8	㲑
U+3C91.0	fCNS11643	F-275B
U+3C91.0	fGB	S-2433
U+3C91.0	fR/S	 82.4
U+3C91.0	fTotalStrokes	 8
U+3C91.0	fCangjie	HUE
U+3C91.0	fCantonese	seoi2
U+3C92.0	fUTF8	㲒
U+3C92.0	fCNS11643	3-2D43
U+3C92.0	fR/S	 82.5
U+3C92.0	fTotalStrokes	 9
U+3C92.0	fCangjie	PRHQU
U+3C92.0	fFourCorner	22610
U+3C92.0	fHakka	 pau5; pau6
U+3C92.0	fCantonese	bou6
U+3C92.0	fMandarin	bao4
U+3C92.0	fEnglish	 brood; incubate
U+3C93.0	fUTF8	㲓
U+3C93.0	fCNS11643	4-2D66
U+3C93.0	fGB	3-4B60
U+3C93.0	fR/S	 82.6
U+3C93.0	fTotalStrokes	 10
U+3C93.0	fCangjie	HUIHI
U+3C93.0	fFourCorner	23710
U+3C93.0	fCantonese	syu3; zung4
U+3C93.0	fMandarin	shu4; rong2
U+3C93.0	fEnglish	variant of &#x7D68;
U+3C94.0	fUTF8	㲔
U+3C94.0	fCNS11643	4-3252
U+3C94.0	fGB	3-7436
U+3C94.0	fR/S	 82.7
U+3C94.0	fTotalStrokes	 11
U+3C94.0	fCangjie	YJHQU
U+3C94.0	fFourCorner	02410
U+3C94.0	fCantonese	sin1; saan1
U+3C94.0	fMandarin	xian1; shen1
U+3C94.0	fEnglish	woolen textiles; fine cloth
U+3C95.0	fUTF8	㲕
U+3C95.0	fCNS11643	4-3254
U+3C95.0	fGB	3-4B64
U+3C95.0	fR/S	 82.7
U+3C95.0	fTotalStrokes	 11
U+3C95.0	fCangjie	BIHQU
U+3C95.0	fFourCorner	22310
U+3C95.0	fCantonese	laai6; lyut3
U+3C95.0	fMandarin	lei4; lie4
U+3C95.0	fEnglish	variegated; woolen fabric of different colors
U+3C96.0	fUTF8	㲖
U+3C96.0	fCNS11643	4-3253
U+3C96.0	fGB	3-4B63
U+3C96.0	fR/S	 82.7
U+3C96.0	fTotalStrokes	 11
U+3C96.0	fCangjie	FBHQU
U+3C96.0	fFourCorner	92210
U+3C96.0	fCantonese	siu1; siu3
U+3C96.0	fMandarin	xiao1
U+3C96.0	fEnglish	[1] fur [2] feather [3] soft and light
U+3C97.0	fUTF8	㲗
U+3C97.0	fCNS11643	4-3255
U+3C97.0	fGB	3-4B65
U+3C97.0	fR/S	 82.7
U+3C97.0	fTotalStrokes	 11
U+3C97.0	fCangjie	BDHQU
U+3C97.0	fFourCorner	22410
U+3C97.0	fCantonese	fu1; fu3
U+3C97.0	fMandarin	fu1
U+3C97.0	fEnglish	[1] smooth the hair [2] woolen textiles; fine cloth
U+3C98.0	fUTF8	㲘
U+3C98.0	fCNS11643	3-3667
U+3C98.0	fR/S	 82.7
U+3C98.0	fTotalStrokes	 11
U+3C98.0	fCangjie	BCHQU
U+3C98.0	fFourCorner	62810
U+3C98.0	fCantonese	keoi4; kui2
U+3C98.0	fMandarin	qu2
U+3C98.0	fEnglish	corrupted form of &#x6C0D;
U+3C99.0	fUTF8	㲙
U+3C99.0	fCNS11643	4-2D67
U+3C99.0	fGB	3-4B66
U+3C99.0	fR/S	 82.7; 82.6
U+3C99.0	fTotalStrokes	 10; 11
U+3C99.0	fCangjie	HUYIU
U+3C99.0	fFourCorner	20710
U+3C99.0	fCantonese	sau1; saau1; saau2
U+3C99.0	fEnglish	same as &#x6BF9;
U+3C9A.0	fUTF8	㲚
U+3C9A.0	fCNS11643	4-3251
U+3C9A.0	fGB	3-4B67
U+3C9A.0	fR/S	 82.7
U+3C9A.0	fTotalStrokes	 11
U+3C9A.0	fCangjie	EHHQU
U+3C9A.0	fFourCorner	39710
U+3C9A.0	fHakka	  sa1
U+3C9A.0	fCantonese	saa1; seon3; tim2
U+3C9A.0	fMandarin	sha1
U+3C9A.0	fEnglish	the outer robe of a Buddhist priest
U+3C9B.0	fUTF8	㲛
U+3C9B.0	fCNS11643	5-395B
U+3C9B.0	fR/S	 82.8
U+3C9B.0	fTotalStrokes	 12
U+3C9B.0	fCangjie	ORHQU
U+3C9B.0	fFourCorner	86710
U+3C9B.0	fHakka	 tje3
U+3C9B.0	fCantonese	zi2
U+3C9B.0	fMandarin	zhi3
U+3C9B.0	fEnglish	hairy animals
U+3C9C.0	fUTF8	㲜
U+3C9C.0	fCNS11643	3-3C4C
U+3C9C.0	fR/S	 82.8; 86.8
U+3C9C.0	fTotalStrokes	 12
U+3C9C.0	fCangjie	FFHQU
U+3C9C.0	fFourCorner	92810
U+3C9C.0	fCantonese	taan2; dam1; tam2
U+3C9C.0	fMandarin	tan2; tan3
U+3C9C.0	fEnglish	non-classical form of &#x6BEF;
U+3C9D.0	fUTF8	㲝
U+3C9D.0	fCNS11643	4-384F
U+3C9D.0	fGB	5-4E32
U+3C9D.0	fR/S	 82.8
U+3C9D.0	fTotalStrokes	 12
U+3C9D.0	fCangjie	HUOG
U+3C9D.0	fFourCorner	20710
U+3C9D.0	fCantonese	jung5; zung2
U+3C9D.0	fMandarin	rong2; rong3; mao2; mao4
U+3C9D.0	fEnglish	corrupt form of &#x6C04;
U+3C9E.0	fUTF8	㲞
U+3C9E.0	fCNS11643	4-384C
U+3C9E.0	fGB	3-4B69
U+3C9E.0	fR/S	 82.8
U+3C9E.0	fTotalStrokes	 12
U+3C9E.0	fCangjie	YJHQU
U+3C9E.0	fFourCorner	02410
U+3C9E.0	fHakka	 zut7
U+3C9E.0	fCantonese	seot1
U+3C9E.0	fMandarin	su4; zu2
U+3C9E.0	fEnglish	hairy; short hair
U+3C9F.0	fUTF8	㲟
U+3C9F.0	fCNS11643	4-3E29
U+3C9F.0	fGB	3-4B68
U+3C9F.0	fR/S	 82.9
U+3C9F.0	fTotalStrokes	 13
U+3C9F.0	fCangjie	TKHQU
U+3C9F.0	fFourCorner	42510
U+3C9F.0	fCantonese	jing2; zing2
U+3C9F.0	fMandarin	ying3; ying1
U+3C9F.0	fEnglish	hair
U+3CA0.0	fUTF8	㲠
U+3CA0.0	fCNS11643	3-4227
U+3CA0.0	fR/S	 82.9
U+3CA0.0	fTotalStrokes	 13
U+3CA0.0	fCangjie	JKHQU
U+3CA0.0	fFourCorner	58710
U+3CA0.0	fCantonese	mou4; lei4
U+3CA0.0	fMandarin	li2; mao2
U+3CA0.0	fEnglish	[1] non-classical, abbreviated form of &#x6C02; [2] variant of &#x81A7;
U+3CA1.0	fUTF8	㲡
U+3CA1.0	fCNS11643	4-3E28
U+3CA1.0	fGB	3-4B6D
U+3CA1.0	fR/S	 82.9
U+3CA1.0	fTotalStrokes	 13
U+3CA1.0	fCangjie	DFHQU
U+3CA1.0	fFourCorner	42910
U+3CA1.0	fCantonese	noi6; loi6 (also)
U+3CA1.0	fMandarin	nai4; ni4
U+3CA1.0	fEnglish	[1] thick coat of hair (of the animals) [2] hairy
U+3CA2.0	fUTF8	㲢
U+3CA2.0	fCNS11643	4-3E27
U+3CA2.0	fGB	3-4B72
U+3CA2.0	fR/S	 82.9
U+3CA2.0	fTotalStrokes	 13
U+3CA2.0	fCangjie	IBHQU
U+3CA2.0	fFourCorner	32215
U+3CA2.0	fCantonese	bin6; bin1
U+3CA2.0	fMandarin	bian4; bie2
U+3CA2.0	fEnglish	hair knots in a mess
U+3CA3.0	fUTF8	㲣
U+3CA3.0	fCNS11643	4-443F
U+3CA3.0	fGB	3-4B70
U+3CA3.0	fR/S	 82.9
U+3CA3.0	fTotalStrokes	 14; 13
U+3CA3.0	fCangjie	HEHQU
U+3CA3.0	fFourCorner	72410
U+3CA3.0	fCantonese	sau1; saau1; saau2
U+3CA3.0	fEnglish	woolen blanket with decorative design or pattern
U+3CA4.0	fUTF8	㲤
U+3CA4.0	fCNS11643	4-4438
U+3CA4.0	fGB	3-4B75
U+3CA4.0	fR/S	 82.10
U+3CA4.0	fTotalStrokes	 14
U+3CA4.0	fCangjie	YVHQU
U+3CA4.0	fFourCorner	02710
U+3CA4.0	fHakka	 soi1
U+3CA4.0	fCantonese	seoi1
U+3CA4.0	fMandarin	shuai1
U+3CA4.0	fEnglish	[1] long hair [2] a few thin hair hangging down
U+3CA5.0	fUTF8	㲥
U+3CA5.0	fCNS11643	4-4439
U+3CA5.0	fGB	3-4B76
U+3CA5.0	fR/S	 82.10
U+3CA5.0	fTotalStrokes	 14
U+3CA5.0	fCangjie	IRHQU
U+3CA5.0	fFourCorner	02210
U+3CA5.0	fCantonese	tong4
U+3CA5.0	fMandarin	tang2
U+3CA5.0	fEnglish	[1] a kind of woolen textiles with curved pattern [2] a decorative ornament on an official hat
U+3CA6.0	fUTF8	㲦
U+3CA6.0	fCNS11643	4-443A
U+3CA6.0	fGB	5-733D
U+3CA6.0	fR/S	 82.10
U+3CA6.0	fTotalStrokes	 14
U+3CA6.0	fCangjie	JJOHU
U+3CA6.0	fFourCorner	48410
U+3CA6.0	fCantonese	hon6
U+3CA6.0	fMandarin	han4; han2; he3
U+3CA6.0	fEnglish	long hair; mane
U+3CA7.0	fUTF8	㲧
U+3CA7.0	fCNS11643	5-4668
U+3CA7.0	fR/S	 82.10
U+3CA7.0	fTotalStrokes	 14
U+3CA7.0	fCangjie	HUEII
U+3CA7.0	fFourCorner	27710
U+3CA7.0	fCantonese	sou3; cou3
U+3CA7.0	fMandarin	sao4; zao4
U+3CA7.0	fEnglish	hairy
U+3CA8.0	fUTF8	㲨
U+3CA8.0	fCNS11643	4-443C
U+3CA8.0	fGB	3-4B6C
U+3CA8.0	fR/S	 82.10
U+3CA8.0	fTotalStrokes	 14
U+3CA8.0	fCangjie	TJHQU
U+3CA8.0	fFourCorner	42110
U+3CA8.0	fCantonese	jung4; zung4
U+3CA8.0	fMandarin	rong2; rong4
U+3CA8.0	fEnglish	[1] a decorative fine blanket on the top of a saddle [2] petty; trifling
U+3CA9.0	fUTF8	㲩
U+3CA9.0	fR/S	 82.10
U+3CA9.0	fTotalStrokes	 14
U+3CA9.0	fCangjie	HUASM
U+3CA9.0	fCantonese	taap3
U+3CA9.0	fEnglish	[Korean character]
U+3CAA.0	fUTF8	㲪
U+3CAA.0	fCNS11643	4-506F
U+3CAA.0	fGB	3-4B7A
U+3CAA.0	fR/S	 82.12
U+3CAA.0	fTotalStrokes	 16
U+3CAA.0	fCangjie	NTHQU
U+3CAA.0	fFourCorner	12110
U+3CAA.0	fCantonese	dang1
U+3CAA.0	fMandarin	deng1
U+3CAA.0	fEnglish	[1] woolen blanket with decorative design or pattern [2] a kind of animal
U+3CAB.0	fUTF8	㲫
U+3CAB.0	fCNS11643	4-5071
U+3CAB.0	fGB	3-4B77
U+3CAB.0	fR/S	 82.12
U+3CAB.0	fTotalStrokes	 16
U+3CAB.0	fCangjie	TOHQU
U+3CAB.0	fFourCorner	32510
U+3CAB.0	fCantonese	buk6; fu4; pok3
U+3CAB.0	fMandarin	bu2; pu2
U+3CAB.0	fEnglish	hair knots in a mess
U+3CAC.0	fUTF8	㲬
U+3CAC.0	fCNS11643	4-5072
U+3CAC.0	fGB	3-4B78
U+3CAC.0	fR/S	 82.12
U+3CAC.0	fTotalStrokes	 16
U+3CAC.0	fCangjie	OFHQU
U+3CAC.0	fFourCorner	22310
U+3CAC.0	fCantonese	ziu1
U+3CAC.0	fMandarin	jiao1
U+3CAC.0	fEnglish	[1] hair decorations on a helmet used in ancient times [2] weak and weary; weak from exhaustion; tired looking of a bird's feather
U+3CAD.0	fUTF8	㲭
U+3CAD.0	fCNS11643	3-5149
U+3CAD.0	fR/S	 82.12
U+3CAD.0	fTotalStrokes	 16
U+3CAD.0	fCangjie	FFHQU
U+3CAD.0	fFourCorner	92810
U+3CAD.0	fCantonese	taan2; tam2; zaan1
U+3CAD.0	fMandarin	tan3
U+3CAD.0	fEnglish	corrupted form of &#x6BEF;
U+3CAE.0	fUTF8	㲮
U+3CAE.0	fR/S	 82.12
U+3CAE.0	fTotalStrokes	 16
U+3CAE.0	fCangjie	HUHOR
U+3CAE.0	fCantonese	taap3
U+3CAE.0	fEnglish	[Korean character]
U+3CAF.0	fUTF8	㲯
U+3CAF.0	fCNS11643	4-5B50
U+3CAF.0	fGB	3-4B73
U+3CAF.0	fR/S	 82.14
U+3CAF.0	fTotalStrokes	 18
U+3CAF.0	fCangjie	STHQU
U+3CAF.0	fFourCorner	72110
U+3CAF.0	fCantonese	laam4; zim4
U+3CAF.0	fMandarin	ran2; lan2
U+3CAF.0	fEnglish	long hairy
U+3CB0.0	fUTF8	㲰
U+3CB0.0	fCNS11643	4-5B4F
U+3CB0.0	fGB	3-4B5F
U+3CB0.0	fR/S	 82.14
U+3CB0.0	fTotalStrokes	 18
U+3CB0.0	fCangjie	JNHQU
U+3CB0.0	fFourCorner	32210
U+3CB0.0	fCantonese	ning4; ling4 (also); ning6
U+3CB0.0	fMandarin	ning2
U+3CB0.0	fEnglish	a hairy dog
U+3CB1.0	fUTF8	㲱
U+3CB1.0	fCNS11643	3-5A50
U+3CB1.0	fGB	5-4E44
U+3CB1.0	fR/S	 82.15
U+3CB1.0	fTotalStrokes	 19
U+3CB1.0	fCangjie	HUVVV
U+3CB1.0	fFourCorner	22710
U+3CB1.0	fCantonese	lip6
U+3CB1.0	fMandarin	lie4
U+3CB1.0	fEnglish	same as &#x9B23;
U+3CB2.0	fUTF8	㲲
U+3CB2.0	fCNS11643	4-6D57
U+3CB2.0	fGB	3-4B7D
U+3CB2.0	fR/S	 82.22
U+3CB2.0	fTotalStrokes	 26
U+3CB2.0	fCangjie	WMHQU
U+3CB2.0	fFourCorner	62110
U+3CB2.0	fCantonese	dip6; taai1
U+3CB2.0	fMandarin	die2; yi4
U+3CB2.0	fEnglish	fine cotton cloth; fine woolen cloth
U+3CB3.0	fUTF8	㲳
U+3CB3.0	fCNS11643	4-2D6A
U+3CB3.0	fGB	5-5021
U+3CB3.0	fR/S	 83.6
U+3CB3.0	fTotalStrokes	 10
U+3CB3.0	fCangjie	HIHQU
U+3CB3.0	fFourCorner	75730
U+3CB3.0	fCantonese	dit6; zaat6
U+3CB3.0	fMandarin	die2; zhi4
U+3CB3.0	fEnglish	[1] touch; ram [2] pull with hands
U+3CB4.0	fUTF8	㲴
U+3CB4.0	fCNS11643	3-295C
U+3CB4.0	fR/S	 84.4
U+3CB4.0	fTotalStrokes	 8
U+3CB4.0	fCangjie	ONL
U+3CB4.0	fFourCorner	80510
U+3CB4.0	fCantonese	zung3
U+3CB4.0	fMandarin	zhong4; zhou4
U+3CB4.0	fEnglish	mind; disposition; temper
U+3CB5.0	fUTF8	㲵
U+3CB5.0	fCNS11643	3-3668
U+3CB5.0	fR/S	 84.7
U+3CB5.0	fTotalStrokes	 11
U+3CB5.0	fCangjie	ONFB
U+3CB5.0	fFourCorner	80210
U+3CB5.0	fCantonese	siu1
U+3CB5.0	fMandarin	xiao1
U+3CB5.0	fEnglish	same as &#x9704;
U+3CB6.0	fUTF8	㲶
U+3CB6.0	fCNS11643	3-4C69
U+3CB6.0	fR/S	 84.11
U+3CB6.0	fTotalStrokes	 15
U+3CB6.0	fCangjie	ONYWI
U+3CB6.0	fFourCorner	80610
U+3CB6.0	fCantonese	lou5; luk6
U+3CB6.0	fMandarin	luu4; l4
U+3CB6.0	fEnglish	old transcripted form
U+3CB7.0	fUTF8	㲷
U+3CB7.0	fCNS11643	3-514A
U+3CB7.0	fR/S	 84.12
U+3CB7.0	fTotalStrokes	 16
U+3CB7.0	fCangjie	ONRRJ
U+3CB7.0	fFourCorner	80510
U+3CB7.0	fCantonese	daam6
U+3CB7.0	fMandarin	dan4
U+3CB7.0	fEnglish	old transcripted form
U+3CB8.0	fUTF8	㲸
U+3CB8.0	fCNS11643	4-214F
U+3CB8.0	fR/S	 85.1
U+3CB8.0	fTotalStrokes	 4
U+3CB8.0	fCangjie	EN
U+3CB8.0	fFourCorner	37110
U+3CB8.0	fCantonese	kap1
U+3CB8.0	fMandarin	xi1
U+3CB8.0	fEnglish	dried up (of waterway; the channel of a river; a pond, etc.)
U+3CB9.0	fUTF8	㲹
U+3CB9.0	fCNS11643	3-2256
U+3CB9.0	fR/S	 85.2
U+3CB9.0	fTotalStrokes	 5
U+3CB9.0	fCangjie	EHN
U+3CB9.0	fFourCorner	37110
U+3CB9.0	fCantonese	gwai2; gip3; gwaai2; kaau4; zi6
U+3CB9.0	fMandarin	ji3; gui3; qiu2
U+3CB9.0	fEnglish	non-classical form of &#x6C3F;
U+3CBA.0	fUTF8	㲺
U+3CBA.0	fCNS11643	3-2255
U+3CBA.0	fGB	5-546D
U+3CBA.0	fR/S	 85.2
U+3CBA.0	fTotalStrokes	 5
U+3CBA.0	fCangjie	EP
U+3CBA.0	fFourCorner	31110
U+3CBA.0	fHakka	 zit7
U+3CBA.0	fCantonese	cat1; zik1
U+3CBA.0	fMandarin	ji2; ke4; lei2
U+3CBA.0	fEnglish	water trickling out
U+3CBB.0	fUTF8	㲻
U+3CBB.0	fCNS11643	3-2422
U+3CBB.0	fGB	5-315E
U+3CBB.0	fR/S	 85.2; 85.4; 9.4
U+3CBB.0	fTotalStrokes	 6; 5
U+3CBB.0	fCangjie	OE
U+3CBB.0	fFourCorner	22230
U+3CBB.0	fCantonese	nik6; lik6 (also)
U+3CBB.0	fMandarin	ni4
U+3CBB.0	fEnglish	ancient form of &#x6EBA;
U+3CBC.0	fUTF8	㲼
U+3CBC.0	fCNS11643	5-2174
U+3CBC.0	fR/S	 85.2
U+3CBC.0	fTotalStrokes	 5
U+3CBC.0	fCangjie	EK
U+3CBC.0	fFourCorner	34100
U+3CBC.0	fCantonese	ngaai6; aai6 (also); zi6
U+3CBC.0	fMandarin	yi4
U+3CBC.0	fEnglish	[1] the name of a river [2] same as &#x6C4A;
U+3CBD.0	fUTF8	㲽
U+3CBD.0	fCNS11643	4-2322
U+3CBD.0	fGB	3-5338
U+3CBD.0	fR/S	 85.3
U+3CBD.0	fTotalStrokes	 6
U+3CBD.0	fCangjie	ESHI
U+3CBD.0	fFourCorner	37120
U+3CBD.0	fCantonese	nin6; lin6 (also); zin2
U+3CBD.0	fMandarin	nian4; ren3; shang1; xian4
U+3CBD.0	fEnglish	[1] the name of a river in today's southwest of Shanxi province [2] the wet things attach or stick up to each other [3] dirty and muddy
U+3CBE.0	fUTF8	㲾
U+3CBE.0	fCNS11643	3-2654
U+3CBE.0	fR/S	 85.3
U+3CBE.0	fTotalStrokes	 7
U+3CBE.0	fCangjie	JE
U+3CBE.0	fFourCorner	30230
U+3CBE.0	fCantonese	jyu5; zyu6
U+3CBE.0	fMandarin	yu3; yu4
U+3CBE.0	fEnglish	same as &#x96E8;
U+3CBF.0	fUTF8	㲿
U+3CBF.0	fGB	S-2362
U+3CBF.0	fR/S	 85.3
U+3CBF.0	fTotalStrokes	 6
U+3CBF.0	fCangjie	EI
U+3CBF.0	fCantonese	wong2
U+3CBF.0	fMandarin	wang3
U+3CBF.0	fEnglish	simplified form of &#x7007;
U+3CC0.0	fUTF8	㳀
U+3CC0.0	fCNS11643	3-264F
U+3CC0.0	fGB	5-5475
U+3CC0.0	fR/S	 85.4
U+3CC0.0	fTotalStrokes	 7
U+3CC0.0	fCangjie	EI
U+3CC0.0	fFourCorner	33150
U+3CC0.0	fCantonese	gwo3; go3 (also)
U+3CC0.0	fMandarin	guo4; kai3; xi4
U+3CC0.0	fEnglish	a river in ancient times
U+3CC1.0	fUTF8	㳁
U+3CC1.0	fCNS11643	4-244F
U+3CC1.0	fGB	3-5341
U+3CC1.0	fR/S	 85.4
U+3CC1.0	fTotalStrokes	 7
U+3CC1.0	fCangjie	EMO
U+3CC1.0	fFourCorner	31180
U+3CC1.0	fCantonese	zak1
U+3CC1.0	fMandarin	ze4
U+3CC1.0	fEnglish	water flow; water current; momentum of moving water
U+3CC2.0	fUTF8	㳂
U+3CC2.0	fCNS11643	3-265D
U+3CC2.0	fGB	5-547A
U+3CC2.0	fR/S	 85.4
U+3CC2.0	fTotalStrokes	 7
U+3CC2.0	fCangjie	ECI
U+3CC2.0	fFourCorner	38130
U+3CC2.0	fCantonese	jyun4; zyun4
U+3CC2.0	fMandarin	yan2
U+3CC2.0	fEnglish	same as &#x6CBF;
U+3CC3.0	fUTF8	㳃
U+3CC3.0	fCNS11643	6-2849
U+3CC3.0	fR/S	 85.4
U+3CC3.0	fTotalStrokes	 7
U+3CC3.0	fCangjie	EKNJ
U+3CC3.0	fFourCorner	34140
U+3CC3.0	fCantonese	ceoi3; seoi6
U+3CC3.0	fMandarin	cui4
U+3CC3.0	fEnglish	non-classical form of &#x6DEC;
U+3CC4.0	fUTF8	㳄
U+3CC4.0	fCNS11643	3-265A
U+3CC4.0	fR/S	 85.4
U+3CC4.0	fTotalStrokes	 7
U+3CC4.0	fCangjie	ENO
U+3CC4.0	fFourCorner	37180
U+3CC4.0	fHakka	 jen2; ren2; jan2
U+3CC4.0	fCantonese	jin4; zin4
U+3CC4.0	fMandarin	xian2
U+3CC4.0	fEnglish	[1] spittle; saliva [2] water flowing
U+3CC5.0	fUTF8	㳅
U+3CC5.0	fCNS11643	4-2451
U+3CC5.0	fGB	5-5473
U+3CC5.0	fR/S	 85.4
U+3CC5.0	fTotalStrokes	 7
U+3CC5.0	fCangjie	EMF
U+3CC5.0	fFourCorner	31190
U+3CC5.0	fHakka	 liu2
U+3CC5.0	fCantonese	lau4; laau4; laau5
U+3CC5.0	fMandarin	liu2; jiao3
U+3CC5.0	fEnglish	ancient form of &#x6D41;
U+3CC6.0	fUTF8	㳆
U+3CC6.0	fCNS11643	3-264B
U+3CC6.0	fGB	5-547C
U+3CC6.0	fR/S	 85.4
U+3CC6.0	fTotalStrokes	 7
U+3CC6.0	fCangjie	EYJ
U+3CC6.0	fFourCorner	34100
U+3CC6.0	fCantonese	tau2; tung2; waa4; zung3
U+3CC6.0	fMandarin	tong3; shu3; tou3
U+3CC6.0	fEnglish	the name of a place in ancient times
U+3CC7.0	fUTF8	㳇
U+3CC7.0	fCNS11643	6-284A
U+3CC7.0	fGB	3-534F
U+3CC7.0	fR/S	 85.4
U+3CC7.0	fTotalStrokes	 7
U+3CC7.0	fCangjie	ECK
U+3CC7.0	fFourCorner	32141
U+3CC7.0	fCantonese	fu6
U+3CC7.0	fMandarin	tun1; fu4
U+3CC7.0	fEnglish	[1] the name of a place in today's Jiangsu province Yixing county [2] float; waft; swim
U+3CC8.0	fUTF8	㳈
U+3CC8.0	fCNS11643	6-2844
U+3CC8.0	fR/S	 85.4
U+3CC8.0	fTotalStrokes	 7
U+3CC8.0	fCangjie	EJC
U+3CC8.0	fFourCorner	34190
U+3CC8.0	fCantonese	pui3; pui1
U+3CC8.0	fMandarin	xi4; pei4
U+3CC8.0	fEnglish	standard form of &#x6C9B;
U+3CC9.0	fUTF8	㳉
U+3CC9.0	fGB	S-2361
U+3CC9.0	fR/S	 85.4
U+3CC9.0	fTotalStrokes	 7
U+3CC9.0	fCangjie	EB
U+3CC9.0	fFourCorner	3712
U+3CC9.0	fCantonese	jyut6
U+3CCA.0	fUTF8	㳊
U+3CCA.0	fCNS11643	3-2658
U+3CCA.0	fR/S	 85.4
U+3CCA.0	fTotalStrokes	 7
U+3CCA.0	fCangjie	EOK
U+3CCA.0	fFourCorner	38140
U+3CCA.0	fCantonese	jau4; zaau4; zung1
U+3CCA.0	fEnglish	same as &#x6538;
U+3CCB.0	fUTF8	㳋
U+3CCB.0	fCNS11643	3-2961
U+3CCB.0	fR/S	 85.5
U+3CCB.0	fTotalStrokes	 8
U+3CCB.0	fCangjie	EOM
U+3CCB.0	fFourCorner	32110
U+3CCB.0	fCantonese	jau1; zaau1
U+3CCB.0	fMandarin	qiu1
U+3CCB.0	fEnglish	a river in ancient times
U+3CCC.0	fUTF8	㳌
U+3CCC.0	fCNS11643	3-295F
U+3CCC.0	fR/S	 85.5
U+3CCC.0	fTotalStrokes	 8
U+3CCC.0	fCangjie	EWL
U+3CCC.0	fFourCorner	36150
U+3CCC.0	fCantonese	aap3; ngaap3 (also); at3
U+3CCC.0	fMandarin	ya1
U+3CCC.0	fEnglish	[1] marshy; swampy [2] damp; moist
U+3CCD.0	fUTF8	㳍
U+3CCD.0	fCNS11643	3-295E
U+3CCD.0	fR/S	 85.5
U+3CCD.0	fTotalStrokes	 8
U+3CCD.0	fCangjie	EKLB
U+3CCD.0	fFourCorner	34120
U+3CCD.0	fCantonese	bou3
U+3CCD.0	fMandarin	bu4
U+3CCD.0	fEnglish	a place in ancient times
U+3CCE.0	fUTF8	㳎
U+3CCE.0	fCNS11643	3-2962
U+3CCE.0	fR/S	 85.5
U+3CCE.0	fTotalStrokes	 8
U+3CCE.0	fCangjie	EIT
U+3CCE.0	fFourCorner	33140
U+3CCE.0	fCantonese	bin6
U+3CCE.0	fMandarin	bian4; fan4
U+3CCE.0	fEnglish	guide or lead to make the water flowing smoothly
U+3CCF.0	fUTF8	㳏
U+3CCF.0	fCNS11643	3-2960
U+3CCF.0	fGB	5-5526
U+3CCF.0	fR/S	 85.5
U+3CCF.0	fTotalStrokes	 8
U+3CCF.0	fCangjie	ELK
U+3CCF.0	fFourCorner	35100
U+3CCF.0	fCantonese	si3; saai6
U+3CCF.0	fMandarin	shi4; chi3
U+3CCF.0	fEnglish	a river in ancient times, in Henan Province
U+3CD0.0	fUTF8	㳐
U+3CD0.0	fCNS11643	4-267E
U+3CD0.0	fGB	3-535D
U+3CD0.0	fR/S	 85.5
U+3CD0.0	fTotalStrokes	 8
U+3CD0.0	fCangjie	EDU
U+3CD0.0	fFourCorner	32110
U+3CD0.0	fCantonese	zaat6; miu5; zat3
U+3CD0.0	fMandarin	cha2; sa4; zha2; zha3
U+3CD0.0	fEnglish	[1] flowing of the water [2] the sound of the flowing water
U+3CD1.0	fUTF8	㳑
U+3CD1.0	fCNS11643	3-2968
U+3CD1.0	fR/S	 85.5
U+3CD1.0	fTotalStrokes	 8
U+3CD1.0	fCangjie	EBT
U+3CD1.0	fFourCorner	37110
U+3CD1.0	fHakka	 jit7
U+3CD1.0	fCantonese	jat6; zaat6
U+3CD1.0	fMandarin	yi4
U+3CD1.0	fEnglish	abbreviated form of &#x6EA2;
U+3CD2.0	fUTF8	㳒
U+3CD2.0	fCNS11643	3-2966
U+3CD2.0	fR/S	 85.5
U+3CD2.0	fTotalStrokes	 8
U+3CD2.0	fCangjie	EKI
U+3CD2.0	fFourCorner	34130
U+3CD2.0	fCantonese	faat3; baa1; fat3
U+3CD2.0	fMandarin	fa3; bian4
U+3CD2.0	fEnglish	variant of &#x6CD5;
U+3CD3.0	fUTF8	㳓
U+3CD3.0	fCNS11643	F-2766
U+3CD3.0	fR/S	 85.5
U+3CD3.0	fTotalStrokes	 8
U+3CD3.0	fCangjie	EKR
U+3CD3.0	fCantonese	jau6
U+3CD4.0	fUTF8	㳔
U+3CD4.0	fGB	7-235D
U+3CD4.0	fR/S	 85.5
U+3CD4.0	fTotalStrokes	 8
U+3CD4.0	fCangjie	EEDI
U+3CD4.0	fCantonese	deoi6; deui3; deoi3
U+3CD4.0	fMandarin	dui4
U+3CD4.0	fEnglish	soak or be soaked; dip; immerse
U+3CD5.0	fUTF8	㳕
U+3CD5.0	fGB	7-235A
U+3CD5.0	fR/S	 85.5
U+3CD5.0	fTotalStrokes	 8
U+3CD5.0	fCangjie	ETMM
U+3CD5.0	fCantonese	laan4; lan4
U+3CD5.0	fMandarin	lan2
U+3CD5.0	fEnglish	[1] simplified form of &#x7061; [2] interchangeable with &#x703E;
U+3CD6.0	fUTF8	㳖
U+3CD6.0	fCNS11643	3-2D44
U+3CD6.0	fR/S	 85.6
U+3CD6.0	fTotalStrokes	 9
U+3CD6.0	fCangjie	EYHV
U+3CD6.0	fFourCorner	30130
U+3CD6.0	fCantonese	ji1; zi1
U+3CD6.0	fMandarin	yi1
U+3CD6.0	fEnglish	the name of a river
U+3CD7.0	fUTF8	㳗
U+3CD7.0	fCNS11643	5-2A24
U+3CD7.0	fGB	5-554A
U+3CD7.0	fR/S	 85.6
U+3CD7.0	fTotalStrokes	 9
U+3CD7.0	fCangjie	EKIG
U+3CD7.0	fFourCorner	34110
U+3CD7.0	fCantonese	caai3; caa3; cat3; zak3
U+3CD7.0	fMandarin	chai4
U+3CD7.0	fEnglish	[1] bank of a river [2] same as &#x6C4A;
U+3CD8.0	fUTF8	㳘
U+3CD8.0	fCNS11643	4-2722
U+3CD8.0	fGB	3-542B
U+3CD8.0	fR/S	 85.6; 85.5
U+3CD8.0	fTotalStrokes	 9; 8
U+3CD8.0	fCangjie	EYIU
U+3CD8.0	fFourCorner	30110
U+3CD8.0	fHakka	 chung1; cung1
U+3CD8.0	fCantonese	cung1
U+3CD8.0	fMandarin	chong1
U+3CD8.0	fEnglish	murmuring water
U+3CD9.0	fUTF8	㳙
U+3CD9.0	fCNS11643	3-2D54
U+3CD9.0	fR/S	 85.6
U+3CD9.0	fTotalStrokes	 9
U+3CD9.0	fCangjie	EIB
U+3CD9.0	fFourCorner	33120
U+3CD9.0	fHakka	 gian1
U+3CD9.0	fCantonese	gyun1; zyun5
U+3CD9.0	fMandarin	juan1; xuan4
U+3CD9.0	fEnglish	[1] non-classical form of &#x6D93; [2] interchangeable with &#x6CEB;
U+3CDA.0	fUTF8	㳚
U+3CDA.0	fCNS11643	5-297B
U+3CDA.0	fR/S	 85.6
U+3CDA.0	fTotalStrokes	 9
U+3CDA.0	fCangjie	EIHI
U+3CDA.0	fFourCorner	33150
U+3CDA.0	fCantonese	wat1; leot6; waat6
U+3CDA.0	fMandarin	xi4; yu4
U+3CDA.0	fEnglish	flowing of the water
U+3CDB.0	fUTF8	㳛
U+3CDB.0	fCNS11643	5-297C
U+3CDB.0	fGB	3-5462
U+3CDB.0	fR/S	 85.6
U+3CDB.0	fTotalStrokes	 9
U+3CDB.0	fCangjie	EHXO
U+3CDB.0	fFourCorner	35130
U+3CDB.0	fCantonese	jyu4; zyu4
U+3CDB.0	fMandarin	yu2; yiu2
U+3CDB.0	fEnglish	[1] dirty; filthy [2] corrupt [3] soap [4] black [5] unravel silk threads
U+3CDC.0	fUTF8	㳜
U+3CDC.0	fCNS11643	4-297E
U+3CDC.0	fGB	5-553D
U+3CDC.0	fR/S	 85.6
U+3CDC.0	fTotalStrokes	 9
U+3CDC.0	fCangjie	EOD
U+3CDC.0	fFourCorner	34190
U+3CDC.0	fCantonese	jau1; zaau1
U+3CDC.0	fMandarin	xiu1
U+3CDC.0	fEnglish	the flowing water
U+3CDD.0	fUTF8	㳝
U+3CDD.0	fCNS11643	3-2D56
U+3CDD.0	fR/S	 85.6
U+3CDD.0	fTotalStrokes	 9
U+3CDD.0	fCangjie	EOHG
U+3CDD.0	fFourCorner	32110
U+3CDD.0	fCantonese	hong1
U+3CDD.0	fEnglish	corrupted form of &#x6D2D;
U+3CDE.0	fUTF8	㳞
U+3CDE.0	fCNS11643	3-2D4C
U+3CDE.0	fR/S	 85.6
U+3CDE.0	fTotalStrokes	 9
U+3CDE.0	fCangjie	EOL
U+3CDE.0	fFourCorner	35100
U+3CDE.0	fCantonese	cung1
U+3CDE.0	fMandarin	chong1
U+3CDE.0	fEnglish	same as &#x6D7A;
U+3CDF.0	fUTF8	㳟
U+3CDF.0	fCNS11643	F-3049
U+3CDF.0	fR/S	 85.6
U+3CDF.0	fTotalStrokes	 10
U+3CDF.0	fCangjie	TCE
U+3CDF.0	fCantonese	hung4
U+3CE0.0	fUTF8	㳠
U+3CE0.0	fGB	7-2223
U+3CE0.0	fR/S	 85.6
U+3CE0.0	fTotalStrokes	 9
U+3CE0.0	fCangjie	EYK
U+3CE0.0	fCantonese	taat3; tat3
U+3CE0.0	fMandarin	ta4
U+3CE0.0	fEnglish	simplified form of &#x6FBE;
U+3CE1.0	fUTF8	㳡
U+3CE1.0	fGB	7-2354
U+3CE1.0	fR/S	 85.6
U+3CE1.0	fTotalStrokes	 9
U+3CE1.0	fCangjie	EYDI
U+3CE1.0	fCantonese	wo1; gwo1
U+3CE1.0	fEnglish	[1] simplified form of &#x6FC4; [2] same as &#x6E26;
U+3CE2.0	fUTF8	㳢
U+3CE2.0	fGB	7-2355
U+3CE2.0	fR/S	 85.6
U+3CE2.0	fTotalStrokes	 9
U+3CE2.0	fCangjie	EHKL
U+3CE2.0	fCantonese	kiu4
U+3CE3.0	fUTF8	㳣
U+3CE3.0	fR/S	 85.6
U+3CE3.0	fTotalStrokes	 9
U+3CE3.0	fCangjie	EJKP
U+3CE3.0	fFourCorner	3411
U+3CE3.0	fCantonese	lou5
U+3CE3.0	fEnglish	[Korean character]
U+3CE4.0	fUTF8	㳤
U+3CE4.0	fCNS11643	6-3872
U+3CE4.0	fR/S	 85.7
U+3CE4.0	fTotalStrokes	 10; 9
U+3CE4.0	fCangjie	EITI
U+3CE4.0	fFourCorner	34130
U+3CE4.0	fCantonese	suk6
U+3CE4.0	fEnglish	non-classical form of &#x6DD1;
U+3CE5.0	fUTF8	㳥
U+3CE5.0	fCNS11643	3-3170
U+3CE5.0	fGB	5-554B
U+3CE5.0	fR/S	 85.7
U+3CE5.0	fTotalStrokes	 10
U+3CE5.0	fCangjie	EMGT
U+3CE5.0	fFourCorner	31140
U+3CE5.0	fCantonese	lung6
U+3CE5.0	fMandarin	long4; ju4
U+3CE5.0	fEnglish	a river in ancient times
U+3CE6.0	fUTF8	㳦
U+3CE6.0	fCNS11643	3-3171
U+3CE6.0	fR/S	 85.7
U+3CE6.0	fTotalStrokes	 10
U+3CE6.0	fCangjie	EIT
U+3CE6.0	fFourCorner	33150
U+3CE6.0	fCantonese	haai6
U+3CE6.0	fMandarin	xie4
U+3CE6.0	fEnglish	a river
U+3CE7.0	fUTF8	㳧
U+3CE7.0	fCNS11643	4-2D6B
U+3CE7.0	fGB	3-5434
U+3CE7.0	fR/S	 85.7
U+3CE7.0	fTotalStrokes	 10
U+3CE7.0	fCangjie	ESJU
U+3CE7.0	fFourCorner	31110
U+3CE7.0	fCantonese	cip3; zap6
U+3CE7.0	fMandarin	che4; xi2; re4; ce4
U+3CE7.0	fEnglish	[1] the water flowing from ... [2] the current of a stream
U+3CE8.0	fUTF8	㳨
U+3CE8.0	fCNS11643	6-3933
U+3CE8.0	fR/S	 85.7
U+3CE8.0	fTotalStrokes	 10; 9
U+3CE8.0	fCangjie	EVVN
U+3CE8.0	fFourCorner	32120
U+3CE8.0	fHakka	 zat7; zan1; zhap8; zap8; chep8; zep8
U+3CE8.0	fCantonese	zaat3
U+3CE8.0	fMandarin	zan3; jian3; za2
U+3CE8.0	fEnglish	 spatter; besprinkle
U+3CE9.0	fUTF8	㳩
U+3CE9.0	fCNS11643	3-3174
U+3CE9.0	fR/S	 85.7
U+3CE9.0	fTotalStrokes	 10
U+3CE9.0	fCangjie	EMBY
U+3CE9.0	fFourCorner	37140
U+3CE9.0	fCantonese	taam1; tan1
U+3CE9.0	fMandarin	tan1
U+3CE9.0	fEnglish	great billows; large waves; the waves swelling up
U+3CEA.0	fUTF8	㳪
U+3CEA.0	fCNS11643	4-2D6C
U+3CEA.0	fGB	3-5438
U+3CEA.0	fR/S	 85.7
U+3CEA.0	fTotalStrokes	 10
U+3CEA.0	fCangjie	EMFR
U+3CEA.0	fFourCorner	31160
U+3CEA.0	fCantonese	pei3; bei3
U+3CEA.0	fMandarin	pi4; pan4
U+3CEA.0	fEnglish	a river
U+3CEB.0	fUTF8	㳫
U+3CEB.0	fCNS11643	3-3669
U+3CEB.0	fR/S	 85.7
U+3CEB.0	fTotalStrokes	 11
U+3CEB.0	fCangjie	EHX
U+3CEB.0	fFourCorner	12770
U+3CEB.0	fHakka	 tap8
U+3CEB.0	fCantonese	zaam2; dap6; tan2
U+3CEB.0	fMandarin	ta4; zan3
U+3CEB.0	fEnglish	corrupted form of &#x6C93;
U+3CEC.0	fUTF8	㳬
U+3CEC.0	fCNS11643	3-3224
U+3CEC.0	fGB	5-555B
U+3CEC.0	fR/S	 85.7
U+3CEC.0	fTotalStrokes	 10
U+3CEC.0	fCangjie	EONO
U+3CEC.0	fFourCorner	38180
U+3CEC.0	fCantonese	syun4
U+3CEC.0	fMandarin	xuan2
U+3CEC.0	fEnglish	 eddy; whirlpool
U+3CED.0	fUTF8	㳭
U+3CED.0	fCNS11643	3-3223
U+3CED.0	fR/S	 85.7
U+3CED.0	fTotalStrokes	 10
U+3CED.0	fCangjie	ERD
U+3CED.0	fFourCorner	36190
U+3CED.0	fCantonese	jin4; zin4
U+3CED.0	fMandarin	xian2
U+3CED.0	fEnglish	non-classical form of &#x3CC4;, &#x6D8E;
U+3CEE.0	fUTF8	㳮
U+3CEE.0	fCNS11643	3-317E
U+3CEE.0	fGB	5-5565
U+3CEE.0	fR/S	 85.7
U+3CEE.0	fTotalStrokes	 10
U+3CEE.0	fCangjie	ESE
U+3CEE.0	fFourCorner	37130
U+3CEE.0	fCantonese	niu6; liu6 (also); niu4
U+3CEE.0	fMandarin	niao4
U+3CEE.0	fEnglish	interchangeable &#x5C3F;
U+3CEF.0	fUTF8	㳯
U+3CEF.0	fR/S	 85.7
U+3CEF.0	fTotalStrokes	 10
U+3CEF.0	fCangjie	EYTJ
U+3CEF.0	fFourCorner	3014
U+3CEF.0	fCantonese	san1
U+3CEF.0	fEnglish	goddess of an ink-slab
U+3CF0.0	fUTF8	㳰
U+3CF0.0	fGB	7-2357
U+3CF0.0	fR/S	 85.7
U+3CF0.0	fTotalStrokes	 10
U+3CF0.0	fCangjie	EYIT
U+3CF0.0	fCantonese	kau1; gaau1; kaau3
U+3CF0.0	fEnglish	same as &#x6E9D;
U+3CF1.0	fUTF8	㳱
U+3CF1.0	fGB	7-235C
U+3CF1.0	fR/S	 85.7
U+3CF1.0	fTotalStrokes	 10
U+3CF1.0	fCangjie	ENLA
U+3CF1.0	fCantonese	joeng4
U+3CF2.0	fUTF8	㳲
U+3CF2.0	fR/S	 85.7
U+3CF2.0	fTotalStrokes	 10
U+3CF2.0	fCangjie	EEKI
U+3CF2.0	fCantonese	taai3
U+3CF2.0	fEnglish	[Korean character]
U+3CF3.0	fUTF8	㳳
U+3CF3.0	fCNS11643	3-3175
U+3CF3.0	fR/S	 85.7
U+3CF3.0	fTotalStrokes	 10
U+3CF3.0	fCangjie	EJRU
U+3CF3.0	fFourCorner	34110
U+3CF3.0	fCantonese	hak1
U+3CF4.0	fUTF8	㳴
U+3CF4.0	fCNS11643	4-3257
U+3CF4.0	fR/S	 85.8
U+3CF4.0	fTotalStrokes	 11
U+3CF4.0	fCangjie	EPHG
U+3CF4.0	fFourCorner	33110
U+3CF4.0	fCantonese	mat6
U+3CF4.0	fMandarin	mi4
U+3CF4.0	fEnglish	mud; quagmire
U+3CF5.0	fUTF8	㳵
U+3CF5.0	fCNS11643	3-3671
U+3CF5.0	fR/S	 85.8
U+3CF5.0	fTotalStrokes	 11
U+3CF5.0	fCangjie	EHDD
U+3CF5.0	fFourCorner	32140
U+3CF5.0	fCantonese	gwai3
U+3CF5.0	fMandarin	ji4
U+3CF5.0	fEnglish	the name of a river
U+3CF6.0	fUTF8	㳶
U+3CF6.0	fCNS11643	4-3263
U+3CF6.0	fGB	3-5463
U+3CF6.0	fR/S	 85.8
U+3CF6.0	fTotalStrokes	 11
U+3CF6.0	fCangjie	EBDU
U+3CF6.0	fFourCorner	32110
U+3CF6.0	fCantonese	nau5; lau5 (also); gaau3
U+3CF6.0	fMandarin	nou3; gou4
U+3CF6.0	fEnglish	[1] the name of a river [2] give suck to [3] same as &#x91B9; [4] same as &#x4E73;
U+3CF7.0	fUTF8	㳷
U+3CF7.0	fCNS11643	4-3264
U+3CF7.0	fGB	3-546A
U+3CF7.0	fR/S	 85.8
U+3CF7.0	fTotalStrokes	 11
U+3CF7.0	fCangjie	EPHA
U+3CF7.0	fFourCorner	37162
U+3CF7.0	fCantonese	fui3; faat1
U+3CF7.0	fMandarin	hu1; wen3
U+3CF7.0	fEnglish	[1] dark blue color [2] combine; unite [3] close; shut [4] the water is exhausted [5] get the color away by soaking in the water
U+3CF8.0	fUTF8	㳸
U+3CF8.0	fCNS11643	3-366E
U+3CF8.0	fR/S	 85.8
U+3CF8.0	fTotalStrokes	 11
U+3CF8.0	fCangjie	ETOP
U+3CF8.0	fFourCorner	34110
U+3CF8.0	fCantonese	faa1
U+3CF8.0	fMandarin	hua1
U+3CF8.0	fEnglish	the name of a river
U+3CF9.0	fUTF8	㳹
U+3CF9.0	fCNS11643	4-325E
U+3CF9.0	fGB	3-5453
U+3CF9.0	fR/S	 85.8
U+3CF9.0	fTotalStrokes	 11
U+3CF9.0	fCangjie	EDMG
U+3CF9.0	fFourCorner	31110
U+3CF9.0	fCantonese	wong2
U+3CF9.0	fMandarin	wang3
U+3CF9.0	fEnglish	[1] flood; great flood; massive water [2] the name a river [3] the name of a place in today's Shanxi province East of Yin county [4] same as &#x6C6A;
U+3CFA.0	fUTF8	㳺
U+3CFA.0	fCNS11643	3-3722
U+3CFA.0	fR/S	 85.8
U+3CFA.0	fTotalStrokes	 11
U+3CFA.0	fCangjie	EQOD
U+3CFA.0	fFourCorner	38140
U+3CFA.0	fCantonese	jau4; zaau4
U+3CFA.0	fMandarin	you2
U+3CFA.0	fEnglish	non-classical form of &#x6E38;
U+3CFB.0	fUTF8	㳻
U+3CFB.0	fCNS11643	3-366D
U+3CFB.0	fGB	5-5568
U+3CFB.0	fR/S	 85.8
U+3CFB.0	fTotalStrokes	 11
U+3CFB.0	fCangjie	ETA
U+3CFB.0	fFourCorner	34160
U+3CFB.0	fCantonese	zaak3
U+3CFB.0	fMandarin	ze2
U+3CFB.0	fEnglish	[1] a bank of earth or an embankment to block the current of the water [2] stop (or block) up the flowing water
U+3CFC.0	fUTF8	㳼
U+3CFC.0	fCNS11643	3-3C4F
U+3CFC.0	fR/S	 85.8
U+3CFC.0	fTotalStrokes	 12
U+3CFC.0	fCangjie	IME
U+3CFC.0	fFourCorner	53230
U+3CFC.0	fCantonese	bat1
U+3CFC.0	fMandarin	bi4
U+3CFC.0	fEnglish	[1] same as &#x6EED; [2] same as &#x6DE2; [3] interchangeable with &#x6D2B;
U+3CFD.0	fUTF8	㳽
U+3CFD.0	fCNS11643	3-367E
U+3CFD.0	fR/S	 85.8
U+3CFD.0	fTotalStrokes	 11
U+3CFD.0	fCangjie	ENNF
U+3CFD.0	fFourCorner	38190
U+3CFD.0	fHakka	 ni2
U+3CFD.0	fCantonese	nei4; lei4 (also); mei4
U+3CFD.0	fMandarin	mi3
U+3CFD.0	fEnglish	non-classical abbreviated form of &#x7030;
U+3CFE.0	fUTF8	㳾
U+3CFE.0	fCNS11643	4-3258
U+3CFE.0	fGB	7-2358
U+3CFE.0	fR/S	 85.8
U+3CFE.0	fTotalStrokes	 11
U+3CFE.0	fCangjie	ETGU
U+3CFE.0	fFourCorner	38110
U+3CFE.0	fCantonese	goeng1
U+3CFE.0	fMandarin	qiang1
U+3CFE.0	fEnglish	standard form of &#x7F8C;
U+3CFF.0	fUTF8	㳿
U+3CFF.0	fCNS11643	4-3268
U+3CFF.0	fGB	3-546D
U+3CFF.0	fR/S	 85.8; 85.7
U+3CFF.0	fTotalStrokes	 11; 10
U+3CFF.0	fCangjie	EYID
U+3CFF.0	fFourCorner	30190
U+3CFF.0	fCantonese	sit3
U+3CFF.0	fMandarin	xie4
U+3CFF.0	fEnglish	same as &#x6E2B;
U+3D00.0	fUTF8	㴀
U+3D00.0	fCNS11643	5-395E
U+3D00.0	fR/S	 85.8
U+3D00.0	fTotalStrokes	 12; 11
U+3D00.0	fCangjie	ETHO
U+3D00.0	fFourCorner	34130
U+3D00.0	fCantonese	faan1; fan6
U+3D00.0	fMandarin	pan1; fan4
U+3D00.0	fEnglish	deep
U+3D01.0	fUTF8	㴁
U+3D01.0	fCNS11643	4-325A
U+3D01.0	fR/S	 85.8
U+3D01.0	fTotalStrokes	 11
U+3D01.0	fCangjie	EHKT
U+3D01.0	fFourCorner	32150
U+3D01.0	fHakka	 jit7; rit7
U+3D01.0	fCantonese	jik1; zaat6
U+3D01.0	fMandarin	yi4
U+3D01.0	fEnglish	[1] add to [2] saliva
U+3D02.0	fUTF8	㴂
U+3D02.0	fCNS11643	4-325C
U+3D02.0	fGB	3-544D
U+3D02.0	fR/S	 85.8
U+3D02.0	fTotalStrokes	 11
U+3D02.0	fCangjie	EMGB
U+3D02.0	fFourCorner	35140
U+3D02.0	fCantonese	taam1; dam1
U+3D02.0	fMandarin	dan2
U+3D02.0	fEnglish	great billows; great upheavals; boisterous breakers
U+3D03.0	fUTF8	㴃
U+3D03.0	fCNS11643	6-412B
U+3D03.0	fR/S	 85.8
U+3D03.0	fTotalStrokes	 11
U+3D03.0	fCangjie	EISE
U+3D03.0	fFourCorner	33130
U+3D03.0	fCantonese	leoi6
U+3D03.0	fMandarin	lei4; li4
U+3D03.0	fEnglish	same as &#x6DDA;
U+3D04.0	fUTF8	㴄
U+3D04.0	fCNS11643	5-333D
U+3D04.0	fGB	3-543D
U+3D04.0	fR/S	 85.8
U+3D04.0	fTotalStrokes	 11
U+3D04.0	fCangjie	EWCR
U+3D04.0	fFourCorner	36100
U+3D04.0	fCantonese	wang2; zing4
U+3D04.0	fMandarin	weng3; ying2
U+3D04.0	fEnglish	the water flowing in a turned, crooked or winding and revolving way; eddy
U+3D05.0	fUTF8	㴅
U+3D05.0	fCNS11643	3-3C5E
U+3D05.0	fR/S	 85.8
U+3D05.0	fTotalStrokes	 12
U+3D05.0	fCangjie	EWCR
U+3D05.0	fFourCorner	12600
U+3D05.0	fCantonese	siu1
U+3D05.0	fMandarin	xiao1
U+3D05.0	fEnglish	ancient form of &#x6D88;
U+3D06.0	fUTF8	㴆
U+3D06.0	fCNS11643	3-367B
U+3D06.0	fR/S	 85.8
U+3D06.0	fTotalStrokes	 11
U+3D06.0	fCangjie	ESMB
U+3D06.0	fFourCorner	37120
U+3D06.0	fCantonese	zam3
U+3D06.0	fMandarin	jin4; xing4
U+3D06.0	fEnglish	same as &#x6D78;
U+3D07.0	fUTF8	㴇
U+3D07.0	fCNS11643	3-3C5F
U+3D07.0	fR/S	 85.8
U+3D07.0	fTotalStrokes	 12; 11
U+3D07.0	fCangjie	EEE
U+3D07.0	fFourCorner	12230
U+3D07.0	fCantonese	sip3; mong4; sip6
U+3D07.0	fMandarin	she4
U+3D07.0	fEnglish	same as &#x6D89;
U+3D08.0	fUTF8	㴈
U+3D08.0	fCNS11643	3-3673
U+3D08.0	fR/S	 85.8
U+3D08.0	fTotalStrokes	 11
U+3D08.0	fCangjie	EOMU
U+3D08.0	fFourCorner	32170
U+3D08.0	fCantonese	ngan6; an6 (also); zaan6
U+3D08.0	fMandarin	yin4
U+3D08.0	fEnglish	same as &#x57BD;
U+3D09.0	fUTF8	㴉
U+3D09.0	fCNS11643	6-4141
U+3D09.0	fR/S	 85.8
U+3D09.0	fTotalStrokes	 11
U+3D09.0	fCangjie	EIIM
U+3D09.0	fFourCorner	33110
U+3D09.0	fCantonese	zai3; zaai2; zaai3
U+3D09.0	fMandarin	ji4; ji3
U+3D09.0	fEnglish	ancient form of &#x6FDF;
U+3D0A.0	fUTF8	㴊
U+3D0A.0	fCNS11643	3-3678
U+3D0A.0	fR/S	 85.8
U+3D0A.0	fTotalStrokes	 11
U+3D0A.0	fCangjie	ETUN
U+3D0A.0	fFourCorner	32100
U+3D0A.0	fCantonese	jyun1; zyun1; zyun4
U+3D0A.0	fEnglish	same as &#x6E06;, &#x6DF5;
U+3D0B.0	fUTF8	㴋
U+3D0B.0	fGB	7-2356
U+3D0B.0	fR/S	 85.8
U+3D0B.0	fTotalStrokes	 11
U+3D0B.0	fCangjie	ELX
U+3D0B.0	fCantonese	suk1
U+3D0B.0	fMandarin	su4
U+3D0B.0	fEnglish	[1] simplified form of &#x6F5A; [2] same as &#x6EB2; [3] same as &#x701F;
U+3D0C.0	fUTF8	㴌
U+3D0C.0	fR/S	 85.8
U+3D0C.0	fTotalStrokes	 11
U+3D0C.0	fCangjie	EVJR
U+3D0C.0	fCantonese	gu1; gwu1 (also)
U+3D0C.0	fEnglish	[Korean character]
U+3D0D.0	fUTF8	㴍
U+3D0D.0	fR/S	 85.8
U+3D0D.0	fTotalStrokes	 11
U+3D0D.0	fCangjie	ENNO
U+3D0D.0	fCantonese	sing4
U+3D0D.0	fEnglish	[Korean character]
U+3D0E.0	fUTF8	㴎
U+3D0E.0	fCNS11643	F-3633
U+3D0E.0	fGB	3-5456
U+3D0E.0	fR/S	 85.8
U+3D0E.0	fTotalStrokes	 11
U+3D0E.0	fCangjie	EKMF
U+3D0E.0	fCantonese	noi6; loi6 (also)
U+3D0F.0	fUTF8	㴏
U+3D0F.0	fCNS11643	4-3851
U+3D0F.0	fGB	3-5541
U+3D0F.0	fR/S	 85.9
U+3D0F.0	fTotalStrokes	 12
U+3D0F.0	fCangjie	EJCG
U+3D0F.0	fFourCorner	33110
U+3D0F.0	fCantonese	wong2; wong1; wu1
U+3D0F.0	fMandarin	wang3
U+3D0F.0	fEnglish	the pond water is still
U+3D10.0	fUTF8	㴐
U+3D10.0	fCNS11643	4-3856
U+3D10.0	fGB	3-5522
U+3D10.0	fR/S	 85.9
U+3D10.0	fTotalStrokes	 12
U+3D10.0	fCangjie	EBUS
U+3D10.0	fFourCorner	31120
U+3D10.0	fHakka	 mien3; mien5
U+3D10.0	fCantonese	min6; min5
U+3D10.0	fMandarin	mian4
U+3D10.0	fEnglish	expanse of water
U+3D11.0	fUTF8	㴑
U+3D11.0	fCNS11643	3-3C65
U+3D11.0	fR/S	 85.9
U+3D11.0	fTotalStrokes	 12
U+3D11.0	fCangjie	EITU
U+3D11.0	fFourCorner	30140
U+3D11.0	fHakka	 su5; sii5
U+3D11.0	fCantonese	sou3
U+3D11.0	fMandarin	su4
U+3D11.0	fEnglish	go against the stream; trace up to the source
U+3D12.0	fUTF8	㴒
U+3D12.0	fCNS11643	4-3852
U+3D12.0	fGB	3-5535
U+3D12.0	fR/S	 85.9
U+3D12.0	fTotalStrokes	 12
U+3D12.0	fCangjie	EYCT
U+3D12.0	fFourCorner	30140
U+3D12.0	fCantonese	jik6; zik6
U+3D12.0	fMandarin	yi4
U+3D12.0	fEnglish	water current; water flow
U+3D13.0	fUTF8	㴓
U+3D13.0	fCNS11643	3-3C59
U+3D13.0	fR/S	 85.9
U+3D13.0	fTotalStrokes	 12
U+3D13.0	fCangjie	EWP
U+3D13.0	fFourCorner	36130
U+3D13.0	fCantonese	saai1; zaai2
U+3D13.0	fMandarin	shai1; zai3
U+3D13.0	fEnglish	 wash rice
U+3D14.0	fUTF8	㴔
U+3D14.0	fCNS11643	4-3859
U+3D14.0	fGB	3-5533
U+3D14.0	fR/S	 85.9
U+3D14.0	fTotalStrokes	 12
U+3D14.0	fCangjie	ENSP
U+3D14.0	fFourCorner	37130
U+3D14.0	fHakka	 kip7;  hip7
U+3D14.0	fCantonese	kap1; kaap1; zaap1
U+3D14.0	fMandarin	xi1; se4; yi4
U+3D14.0	fEnglish	[1] same as &#x6F5D; [2] same as non-classical form of &#x6F80;
U+3D15.0	fUTF8	㴕
U+3D15.0	fCNS11643	3-3C6A
U+3D15.0	fR/S	 85.9
U+3D15.0	fTotalStrokes	 12
U+3D15.0	fCangjie	ETHD
U+3D15.0	fFourCorner	34190
U+3D15.0	fCantonese	wo4; caap1; zap6
U+3D15.0	fMandarin	ji2
U+3D15.0	fEnglish	[1] boil away; boil over [2] boiling sound [3] the sound of the flowing water, etc., [4] interchangeable with &#x6E2B;
U+3D16.0	fUTF8	㴖
U+3D16.0	fCNS11643	3-3C67
U+3D16.0	fR/S	 85.9
U+3D16.0	fTotalStrokes	 12
U+3D16.0	fCangjie	ETLJ
U+3D16.0	fFourCorner	34150
U+3D16.0	fCantonese	lok6
U+3D16.0	fMandarin	bo2; luo4
U+3D16.0	fEnglish	[1] same as &#x6FFC; [2] same as &#x6CCA;
U+3D17.0	fUTF8	㴗
U+3D17.0	fCNS11643	3-3C55
U+3D17.0	fR/S	 85.9
U+3D17.0	fTotalStrokes	 12
U+3D17.0	fCangjie	EMWV
U+3D17.0	fFourCorner	31140
U+3D17.0	fCantonese	jau1; zaau1
U+3D17.0	fMandarin	you1
U+3D17.0	fEnglish	deep
U+3D18.0	fUTF8	㴘
U+3D18.0	fCNS11643	4-3857
U+3D18.0	fGB	3-5526
U+3D18.0	fR/S	 85.9
U+3D18.0	fTotalStrokes	 12
U+3D18.0	fCangjie	EABU
U+3D18.0	fFourCorner	36160
U+3D18.0	fHakka	 mau5; mo6; mo5; mau6
U+3D18.0	fCantonese	mou6; muk6
U+3D18.0	fMandarin	mao4
U+3D18.0	fEnglish	 leak; ooze out; gush out
U+3D19.0	fUTF8	㴙
U+3D19.0	fCNS11643	4-385A
U+3D19.0	fGB	3-552C
U+3D19.0	fR/S	 85.9
U+3D19.0	fTotalStrokes	 12
U+3D19.0	fCangjie	EHJX
U+3D19.0	fFourCorner	32170
U+3D19.0	fCantonese	zaap3; zaa3
U+3D19.0	fMandarin	zha3; zha2
U+3D19.0	fEnglish	[1] water dripping; drip-tile [2] same as &#x70B8;, &#x7160;, &#x721A;
U+3D1A.0	fUTF8	㴚
U+3D1A.0	fCNS11643	3-3C53
U+3D1A.0	fGB	5-5636
U+3D1A.0	fR/S	 85.9
U+3D1A.0	fTotalStrokes	 12
U+3D1A.0	fCangjie	ETPO
U+3D1A.0	fFourCorner	38132
U+3D1A.0	fCantonese	seoi6
U+3D1A.0	fMandarin	sui4
U+3D1A.0	fEnglish	 small ditch in the field
U+3D1B.0	fUTF8	㴛
U+3D1B.0	fCNS11643	3-3C57
U+3D1B.0	fR/S	 85.9
U+3D1B.0	fTotalStrokes	 12
U+3D1B.0	fCangjie	EMGK
U+3D1B.0	fFourCorner	38140
U+3D1B.0	fCantonese	zi3
U+3D1B.0	fMandarin	zhi4
U+3D1B.0	fEnglish	damp; wet; moist; humid
U+3D1C.0	fUTF8	㴜
U+3D1C.0	fCNS11643	3-3C61
U+3D1C.0	fR/S	 85.9
U+3D1C.0	fTotalStrokes	 12
U+3D1C.0	fCangjie	EISB
U+3D1C.0	fFourCorner	33120
U+3D1C.0	fHakka	 bien5
U+3D1C.0	fCantonese	pin3
U+3D1C.0	fMandarin	bi1; bian4
U+3D1C.0	fEnglish	same as &#x904D;
U+3D1D.0	fUTF8	㴝
U+3D1D.0	fCNS11643	3-422B
U+3D1D.0	fR/S	 85.9
U+3D1D.0	fTotalStrokes	 13; 12
U+3D1D.0	fCangjie	HNOE
U+3D1D.0	fFourCorner	22230
U+3D1D.0	fCantonese	lai4
U+3D1D.0	fMandarin	li2
U+3D1D.0	fEnglish	variant of &#x9ECE;
U+3D1E.0	fUTF8	㴞
U+3D1E.0	fCNS11643	6-4A6D
U+3D1E.0	fR/S	 85.9
U+3D1E.0	fTotalStrokes	 12
U+3D1E.0	fCangjie	EBLA
U+3D1E.0	fFourCorner	32160
U+3D1E.0	fCantonese	tou1; caai3; caau4; dou1; zaau6
U+3D1E.0	fMandarin	tao1
U+3D1E.0	fEnglish	same as &#x6ED4;
U+3D1F.0	fUTF8	㴟
U+3D1F.0	fGB	S-235C
U+3D1F.0	fR/S	 85.9
U+3D1F.0	fTotalStrokes	 12
U+3D1F.0	fCangjie	EMAD
U+3D1F.0	fFourCorner	3114
U+3D1F.0	fCantonese	hau5
U+3D20.0	fUTF8	㴠
U+3D20.0	fCNS11643	F-3C43
U+3D20.0	fR/S	 85.9
U+3D20.0	fTotalStrokes	 12
U+3D20.0	fCangjie	EUNE
U+3D20.0	fFourCorner	3717
U+3D20.0	fHakka	 ham2
U+3D20.0	fCantonese	haam4
U+3D21.0	fUTF8	㴡
U+3D21.0	fCNS11643	F-3C5A
U+3D21.0	fGB	S-2363
U+3D21.0	fR/S	 85.9
U+3D21.0	fTotalStrokes	 12
U+3D21.0	fCangjie	EHDA
U+3D21.0	fCantonese	hoeng1
U+3D22.0	fUTF8	㴢
U+3D22.0	fR/S	 85.9
U+3D22.0	fTotalStrokes	 12
U+3D22.0	fCangjie	EEOL
U+3D22.0	fCantonese	cung1
U+3D22.0	fEnglish	[Korean character]
U+3D23.0	fUTF8	㴣
U+3D23.0	fR/S	 85.9
U+3D23.0	fTotalStrokes	 12
U+3D23.0	fCangjie	EEKN
U+3D23.0	fCantonese	ji4
U+3D23.0	fEnglish	[Korean character]
U+3D24.0	fUTF8	㴤
U+3D24.0	fR/S	 85.9
U+3D24.0	fTotalStrokes	 12
U+3D24.0	fCangjie	EDYG
U+3D24.0	fCantonese	cyu5
U+3D24.0	fEnglish	[Korean character]
U+3D25.0	fUTF8	㴥
U+3D25.0	fCNS11643	4-3E37
U+3D25.0	fGB	3-5553
U+3D25.0	fR/S	 85.10
U+3D25.0	fTotalStrokes	 13
U+3D25.0	fCangjie	EUFB
U+3D25.0	fFourCorner	39120
U+3D25.0	fCantonese	ciu3
U+3D25.0	fMandarin	qiao4
U+3D25.0	fEnglish	great billows; heavy seas rollling in
U+3D26.0	fUTF8	㴦
U+3D26.0	fCNS11643	3-422C
U+3D26.0	fR/S	 85.10
U+3D26.0	fTotalStrokes	 13
U+3D26.0	fCangjie	EJRR
U+3D26.0	fFourCorner	33160
U+3D26.0	fCantonese	gun3; gwun3 (also)
U+3D26.0	fMandarin	gong1; guan4
U+3D26.0	fEnglish	[1] corrupted form of &#x6DAB; [2] interchangeable with &#x76E5;
U+3D27.0	fUTF8	㴧
U+3D27.0	fCNS11643	3-4237
U+3D27.0	fGB	5-564B
U+3D27.0	fR/S	 85.10
U+3D27.0	fTotalStrokes	 13
U+3D27.0	fCangjie	EHUP
U+3D27.0	fFourCorner	36130
U+3D27.0	fCantonese	sik1
U+3D27.0	fMandarin	xi2
U+3D27.0	fEnglish	[1] of flowing water [2] river
U+3D28.0	fUTF8	㴨
U+3D28.0	fCNS11643	3-4235
U+3D28.0	fGB	7-2359
U+3D28.0	fR/S	 85.10
U+3D28.0	fTotalStrokes	 13
U+3D28.0	fCangjie	EBTK
U+3D28.0	fFourCorner	38130
U+3D28.0	fHakka	 ten2
U+3D28.0	fCantonese	zam6
U+3D28.0	fMandarin	zhen4
U+3D28.0	fEnglish	water current; water flow
U+3D29.0	fUTF8	㴩
U+3D29.0	fCNS11643	3-4248
U+3D29.0	fGB	5-5657
U+3D29.0	fR/S	 85.10
U+3D29.0	fTotalStrokes	 13
U+3D29.0	fCangjie	EVVU
U+3D29.0	fFourCorner	32110
U+3D29.0	fCantonese	jung1; zung3
U+3D29.0	fMandarin	yong1
U+3D29.0	fEnglish	same as &#x7049;
U+3D2A.0	fUTF8	㴪
U+3D2A.0	fCNS11643	3-4238
U+3D2A.0	fR/S	 85.10
U+3D2A.0	fTotalStrokes	 13
U+3D2A.0	fCangjie	EHUD
U+3D2A.0	fFourCorner	36190
U+3D2A.0	fCantonese	jit6; zip6; zit6
U+3D2A.0	fMandarin	nie4
U+3D2A.0	fEnglish	the name of a river
U+3D2B.0	fUTF8	㴫
U+3D2B.0	fCNS11643	4-3E30
U+3D2B.0	fGB	3-5542
U+3D2B.0	fR/S	 85.10
U+3D2B.0	fTotalStrokes	 13
U+3D2B.0	fCangjie	ESRL
U+3D2B.0	fFourCorner	37120
U+3D2B.0	fCantonese	gwan3; gwaan6; ngaa4
U+3D2B.0	fMandarin	gun4; jun4
U+3D2B.0	fEnglish	[1] great flowing; big flood [2] the name of a county in ancient times
U+3D2C.0	fUTF8	㴬
U+3D2C.0	fCNS11643	4-3E39
U+3D2C.0	fGB	5-5649
U+3D2C.0	fR/S	 85.10
U+3D2C.0	fTotalStrokes	 13
U+3D2C.0	fCangjie	EHHI
U+3D2C.0	fFourCorner	34100
U+3D2C.0	fCantonese	ze6
U+3D2C.0	fMandarin	xie4
U+3D2C.0	fEnglish	a river in ancient times; head of source from today's Henan province Xinanxian; and flowing south to Luoshui
U+3D2D.0	fUTF8	㴭
U+3D2D.0	fCNS11643	4-3E2D
U+3D2D.0	fGB	3-5560
U+3D2D.0	fR/S	 85.10
U+3D2D.0	fTotalStrokes	 13
U+3D2D.0	fCangjie	EJCU
U+3D2D.0	fFourCorner	33160
U+3D2D.0	fCantonese	jiu2; miu5
U+3D2D.0	fMandarin	yao3
U+3D2D.0	fEnglish	immeasurable depth or profundity; extremely abstruse; unfathomable
U+3D2E.0	fUTF8	㴮
U+3D2E.0	fCNS11643	3-4242
U+3D2E.0	fR/S	 85.10
U+3D2E.0	fTotalStrokes	 13
U+3D2E.0	fCangjie	ESFB
U+3D2E.0	fFourCorner	37120
U+3D2E.0	fCantonese	sit3; dip6
U+3D2E.0	fMandarin	xie4
U+3D2E.0	fEnglish	water current; water flow
U+3D2F.0	fUTF8	㴯
U+3D2F.0	fCNS11643	4-3E38
U+3D2F.0	fGB	3-5558
U+3D2F.0	fR/S	 85.10
U+3D2F.0	fTotalStrokes	 13
U+3D2F.0	fCangjie	EBPA
U+3D2F.0	fFourCorner	31160
U+3D2F.0	fCantonese	zi1
U+3D2F.0	fMandarin	zhi1
U+3D2F.0	fEnglish	[1] water current; water flow [2] the name of a stream
U+3D30.0	fUTF8	㴰
U+3D30.0	fCNS11643	3-4234
U+3D30.0	fR/S	 85.10
U+3D30.0	fTotalStrokes	 13
U+3D30.0	fCangjie	EIBP
U+3D30.0	fFourCorner	31110
U+3D30.0	fCantonese	nang4; lang4 (also); naang4
U+3D30.0	fMandarin	neng2
U+3D30.0	fEnglish	a river in ancient times
U+3D31.0	fUTF8	㴱
U+3D31.0	fCNS11643	3-423C
U+3D31.0	fR/S	 85.10
U+3D31.0	fTotalStrokes	 13
U+3D31.0	fCangjie	EJCC
U+3D31.0	fFourCorner	33130
U+3D31.0	fCantonese	sam1
U+3D31.0	fEnglish	ancient form of &#x6DF1;
U+3D32.0	fUTF8	㴲
U+3D32.0	fCNS11643	4-3E3C
U+3D32.0	fGB	3-5556
U+3D32.0	fR/S	 85.10
U+3D32.0	fTotalStrokes	 13
U+3D32.0	fCangjie	EHYU
U+3D32.0	fFourCorner	32110
U+3D32.0	fCantonese	si1
U+3D32.0	fMandarin	si1
U+3D32.0	fEnglish	[1] a river in ancient times; today's Baiquanhe [2] water's edge; water-front; bank; limit
U+3D33.0	fUTF8	㴳
U+3D33.0	fCNS11643	3-422F
U+3D33.0	fGB	3-5549
U+3D33.0	fR/S	 85.10
U+3D33.0	fTotalStrokes	 13
U+3D33.0	fCangjie	EMIH
U+3D33.0	fFourCorner	32112
U+3D33.0	fCantonese	lung5
U+3D33.0	fMandarin	meng3; long3
U+3D33.0	fEnglish	[1] (corrupted form) [2] smear; spread [3] paint [4] blot out [5] mud; mire [6] ignorant (said of a child); innocent
U+3D34.0	fUTF8	㴴
U+3D34.0	fCNS11643	3-4241
U+3D34.0	fR/S	 85.10
U+3D34.0	fTotalStrokes	 13
U+3D34.0	fCangjie	EVFF
U+3D34.0	fFourCorner	39110
U+3D34.0	fHakka	 tam3;  cam3
U+3D34.0	fCantonese	zaam3
U+3D34.0	fMandarin	zhan4; chen2
U+3D34.0	fEnglish	[1] ancient form of &#x6E5B; [2] interchangeable with &#x6C88;, &#x6C89; [3] interchangeable with &#x6F6D; [4] interchangeable with &#x6D78;
U+3D35.0	fUTF8	㴵
U+3D35.0	fCNS11643	5-4027
U+3D35.0	fR/S	 85.10
U+3D35.0	fTotalStrokes	 13
U+3D35.0	fCangjie	EPHT
U+3D35.0	fFourCorner	33110
U+3D35.0	fCantonese	mat6
U+3D35.0	fMandarin	mi4
U+3D35.0	fEnglish	water current; water flow
U+3D36.0	fUTF8	㴶
U+3D36.0	fCNS11643	4-3E2F
U+3D36.0	fGB	3-5562
U+3D36.0	fR/S	 85.10
U+3D36.0	fTotalStrokes	 13
U+3D36.0	fCangjie	EOBG
U+3D36.0	fFourCorner	34114
U+3D36.0	fCantonese	kok3; huk6
U+3D36.0	fMandarin	que4
U+3D36.0	fEnglish	[1] water; pour water; irrigate [2] wet; moisten; soak in [3] wash
U+3D37.0	fUTF8	㴷
U+3D37.0	fCNS11643	4-3E32
U+3D37.0	fGB	5-5641
U+3D37.0	fR/S	 85.10
U+3D37.0	fTotalStrokes	 13
U+3D37.0	fCangjie	ESJU
U+3D37.0	fFourCorner	34110
U+3D37.0	fCantonese	daam1; dam1; tam4
U+3D37.0	fMandarin	dan1
U+3D37.0	fEnglish	[1] wet; moist; damp [2] incessanty rains [3] flood; great current
U+3D38.0	fUTF8	㴸
U+3D38.0	fCNS11643	3-4232
U+3D38.0	fGB	5-552F
U+3D38.0	fR/S	 85.10
U+3D38.0	fTotalStrokes	 13
U+3D38.0	fCangjie	EANO
U+3D38.0	fFourCorner	37120
U+3D38.0	fHakka	 sham3; sam3
U+3D38.0	fCantonese	sim2
U+3D38.0	fMandarin	shan3; na4; ye4
U+3D38.0	fEnglish	the shimmer on running water
U+3D39.0	fUTF8	㴹
U+3D39.0	fR/S	 85.10
U+3D39.0	fTotalStrokes	 13
U+3D39.0	fCangjie	EYFD
U+3D39.0	fCantonese	mai4
U+3D39.0	fEnglish	[Korean character]
U+3D3A.0	fUTF8	㴺
U+3D3A.0	fR/S	 85.10
U+3D3A.0	fTotalStrokes	 13
U+3D3A.0	fCangjie	EEKO
U+3D3A.0	fCantonese	zip3
U+3D3A.0	fEnglish	[Korean character]
U+3D3B.0	fUTF8	㴻
U+3D3B.0	fCNS11643	3-4243
U+3D3B.0	fR/S	 85.10
U+3D3B.0	fTotalStrokes	 13
U+3D3B.0	fCangjie	EMTI
U+3D3B.0	fFourCorner	34100
U+3D3B.0	fCantonese	syu6
U+3D3C.0	fUTF8	㴼
U+3D3C.0	fCNS11643	3-4755
U+3D3C.0	fR/S	 85.11
U+3D3C.0	fTotalStrokes	 14
U+3D3C.0	fCangjie	EJOA
U+3D3C.0	fFourCorner	33160
U+3D3C.0	fCantonese	suk1
U+3D3C.0	fMandarin	su4
U+3D3C.0	fEnglish	wet; moist; damp
U+3D3D.0	fUTF8	㴽
U+3D3D.0	fCNS11643	4-4449
U+3D3D.0	fGB	3-5575
U+3D3D.0	fR/S	 85.11
U+3D3D.0	fTotalStrokes	 14
U+3D3D.0	fCangjie	EHDP
U+3D3D.0	fFourCorner	32130
U+3D3D.0	fCantonese	sit3; co3; nau4
U+3D3D.0	fMandarin	xie4; yin4
U+3D3D.0	fEnglish	water current; water flow
U+3D3E.0	fUTF8	㴾
U+3D3E.0	fCNS11643	3-4760
U+3D3E.0	fGB	S-2365
U+3D3E.0	fR/S	 85.11
U+3D3E.0	fTotalStrokes	 14
U+3D3E.0	fCangjie	EJDK
U+3D3E.0	fFourCorner	38140
U+3D3E.0	fHakka	 put8
U+3D3E.0	fCantonese	but6
U+3D3E.0	fMandarin	bo2
U+3D3E.0	fEnglish	same as &#x6E24;
U+3D3F.0	fUTF8	㴿
U+3D3F.0	fCNS11643	6-5E3B
U+3D3F.0	fGB	3-5458
U+3D3F.0	fR/S	 85.11
U+3D3F.0	fTotalStrokes	 14
U+3D3F.0	fCangjie	EMNC
U+3D3F.0	fFourCorner	31180
U+3D3F.0	fCantonese	ding2
U+3D3F.0	fMandarin	ding3
U+3D3F.0	fEnglish	same as &#x6FCE;
U+3D40.0	fUTF8	㵀
U+3D40.0	fCNS11643	3-4759
U+3D40.0	fR/S	 85.11
U+3D40.0	fTotalStrokes	 14
U+3D40.0	fCangjie	EYSK
U+3D40.0	fFourCorner	38130
U+3D40.0	fCantonese	zuk6
U+3D40.0	fMandarin	zu2; cuan4
U+3D40.0	fEnglish	water flow; water current
U+3D41.0	fUTF8	㵁
U+3D41.0	fCNS11643	6-5450
U+3D41.0	fR/S	 85.11; 85.10
U+3D41.0	fTotalStrokes	 13; 14
U+3D41.0	fCangjie	ETYU
U+3D41.0	fFourCorner	34110
U+3D41.0	fCantonese	fong1
U+3D41.0	fMandarin	huang1
U+3D41.0	fEnglish	[1] vast and boundless of flowing water [2] a water waste [3] reach
U+3D42.0	fUTF8	㵂
U+3D42.0	fCNS11643	3-4758
U+3D42.0	fR/S	 85.11
U+3D42.0	fTotalStrokes	 14
U+3D42.0	fCangjie	EITF
U+3D42.0	fFourCorner	30130
U+3D42.0	fCantonese	syu3
U+3D42.0	fMandarin	shu4; chuang3
U+3D42.0	fEnglish	the name of a river
U+3D43.0	fUTF8	㵃
U+3D43.0	fCNS11643	4-4446
U+3D43.0	fGB	3-556F
U+3D43.0	fR/S	 85.11
U+3D43.0	fTotalStrokes	 14
U+3D43.0	fCangjie	ELIP
U+3D43.0	fFourCorner	33110
U+3D43.0	fCantonese	se4
U+3D43.0	fMandarin	she2; che4; mang2; tan2; tuo4; zui4
U+3D43.0	fEnglish	the name of a stream
U+3D44.0	fUTF8	㵄
U+3D44.0	fCNS11643	4-444D
U+3D44.0	fR/S	 85.11
U+3D44.0	fTotalStrokes	 14
U+3D44.0	fCangjie	EIRK
U+3D44.0	fFourCorner	33150
U+3D44.0	fCantonese	hon3; wik6
U+3D44.0	fMandarin	han4; qia4; yu4
U+3D44.0	fEnglish	[1] same as &#x6F22; [2] interchangeable with &#x57DF;
U+3D45.0	fUTF8	㵅
U+3D45.0	fCNS11643	4-444A
U+3D45.0	fGB	3-5464
U+3D45.0	fR/S	 85.11
U+3D45.0	fTotalStrokes	 14
U+3D45.0	fCangjie	EOIC
U+3D45.0	fFourCorner	38180
U+3D45.0	fCantonese	taam1; tam1; tam3
U+3D45.0	fMandarin	tan1; dan4; tan4
U+3D45.0	fEnglish	a river in today's northeast of Sichuan Province, between Bazhong and Tongjiang
U+3D46.0	fUTF8	㵆
U+3D46.0	fCNS11643	6-5E40
U+3D46.0	fR/S	 85.11
U+3D46.0	fTotalStrokes	 14
U+3D46.0	fCangjie	EAHR
U+3D46.0	fFourCorner	35160
U+3D46.0	fCantonese	gou2; hou6
U+3D46.0	fMandarin	hao4
U+3D46.0	fEnglish	same as &#x6F94;, &#x6D69;
U+3D47.0	fUTF8	㵇
U+3D47.0	fGB	7-235B
U+3D47.0	fR/S	 85.11
U+3D47.0	fTotalStrokes	 14
U+3D47.0	fCangjie	EHRT
U+3D47.0	fCantonese	tim4
U+3D48.0	fUTF8	㵈
U+3D48.0	fR/S	 85.11
U+3D48.0	fTotalStrokes	 14
U+3D48.0	fCangjie	EEVR
U+3D48.0	fCantonese	gu1; gwu1 (also)
U+3D48.0	fEnglish	[Korean character]
U+3D49.0	fUTF8	㵉
U+3D49.0	fR/S	 85.11
U+3D49.0	fTotalStrokes	 14
U+3D49.0	fCangjie	EEDD
U+3D49.0	fCantonese	lam4
U+3D49.0	fEnglish	[Korean character]
U+3D4A.0	fUTF8	㵊
U+3D4A.0	fCNS11643	3-4C6E
U+3D4A.0	fR/S	 85.12
U+3D4A.0	fTotalStrokes	 15
U+3D4A.0	fCangjie	EASE
U+3D4A.0	fFourCorner	36140
U+3D4A.0	fCantonese	naap6; laap6 (also); naa4; saam2; zip3; zip6
U+3D4A.0	fMandarin	na4; shen3; zhe2
U+3D4A.0	fEnglish	[1] moving image; moving shadow [2] flowing water
U+3D4B.0	fUTF8	㵋
U+3D4B.0	fCNS11643	4-4A5F
U+3D4B.0	fGB	3-5432
U+3D4B.0	fR/S	 85.12
U+3D4B.0	fTotalStrokes	 15
U+3D4B.0	fCangjie	EWLC
U+3D4B.0	fFourCorner	36180
U+3D4B.0	fHakka	 mai3
U+3D4B.0	fCantonese	mik6
U+3D4B.0	fMandarin	mai3; mi4
U+3D4B.0	fEnglish	a river in Kiangsi
U+3D4C.0	fUTF8	㵌
U+3D4C.0	fCNS11643	4-4A61
U+3D4C.0	fGB	3-5636
U+3D4C.0	fR/S	 85.12
U+3D4C.0	fTotalStrokes	 15
U+3D4C.0	fCangjie	EHOU
U+3D4C.0	fFourCorner	32160
U+3D4C.0	fCantonese	ceon4
U+3D4C.0	fMandarin	xun2; chan4; qiong2
U+3D4C.0	fEnglish	 flow; move; stray
U+3D4D.0	fUTF8	㵍
U+3D4D.0	fCNS11643	3-4C6F
U+3D4D.0	fGB	5-5563
U+3D4D.0	fR/S	 85.12
U+3D4D.0	fTotalStrokes	 15
U+3D4D.0	fCangjie	EANP
U+3D4D.0	fFourCorner	37120
U+3D4D.0	fCantonese	mun5; mun6
U+3D4D.0	fMandarin	men4
U+3D4D.0	fEnglish	[1] filled with water; full of water [2] annoyed; vexed; depressed; bored [3] make tea by boiling water and then put the lid on to cover it
U+3D4E.0	fUTF8	㵎
U+3D4E.0	fCNS11643	3-4C7C
U+3D4E.0	fGB	5-5561
U+3D4E.0	fR/S	 85.12
U+3D4E.0	fTotalStrokes	 15
U+3D4E.0	fCangjie	EANB
U+3D4E.0	fFourCorner	37120
U+3D4E.0	fCantonese	gaan3; gaan2
U+3D4E.0	fMandarin	jian4; han3; kan3
U+3D4E.0	fEnglish	[1] mountain stream or torrent [2] a number measuring used in ancient times; a hundred million waterways (ditches) equal to a mountain stream [3] a river in ancient, head source in south of Henan province, flowing east then north to combine with 
U+3D4F.0	fUTF8	㵏
U+3D4F.0	fCNS11643	4-4A68
U+3D4F.0	fR/S	 85.12
U+3D4F.0	fTotalStrokes	 15
U+3D4F.0	fCangjie	ETYJ
U+3D4F.0	fFourCorner	34140
U+3D4F.0	fCantonese	ceoi2; ceoi1
U+3D4F.0	fMandarin	cui3
U+3D4F.0	fEnglish	[1] same as &#x6F3C; [2] interchangeable with &#x6467;
U+3D50.0	fUTF8	㵐
U+3D50.0	fCNS11643	4-4A5D
U+3D50.0	fGB	3-562D
U+3D50.0	fR/S	 85.12
U+3D50.0	fTotalStrokes	 15
U+3D50.0	fCangjie	EMTO
U+3D50.0	fFourCorner	31180
U+3D50.0	fCantonese	kyut3; zyut3
U+3D50.0	fMandarin	que4; jue2
U+3D50.0	fEnglish	[1] the name of a river in Hubei province [2] the name of a state in ancient times
U+3D51.0	fUTF8	㵑
U+3D51.0	fCNS11643	3-4C6D
U+3D51.0	fGB	5-563A
U+3D51.0	fR/S	 85.12
U+3D51.0	fTotalStrokes	 15
U+3D51.0	fCangjie	EKRC
U+3D51.0	fFourCorner	36180
U+3D51.0	fCantonese	ho6
U+3D51.0	fMandarin	he4
U+3D51.0	fEnglish	the name of a river
U+3D52.0	fUTF8	㵒
U+3D52.0	fCNS11643	4-4A59
U+3D52.0	fGB	3-5544
U+3D52.0	fR/S	 85.12
U+3D52.0	fTotalStrokes	 15
U+3D52.0	fCangjie	ELNC
U+3D52.0	fFourCorner	35180
U+3D52.0	fHakka	 bui5; bi5; fui5
U+3D52.0	fCantonese	fai3
U+3D52.0	fMandarin	fei4
U+3D52.0	fEnglish	a district in Shantung
U+3D53.0	fUTF8	㵓
U+3D53.0	fCNS11643	3-4C72
U+3D53.0	fR/S	 85.12
U+3D53.0	fTotalStrokes	 15
U+3D53.0	fCangjie	EJAO
U+3D53.0	fFourCorner	33180
U+3D53.0	fHakka	 chit8
U+3D53.0	fCantonese	zik6; tim1
U+3D53.0	fMandarin	shi2
U+3D53.0	fEnglish	same as &#x6E5C;
U+3D54.0	fUTF8	㵔
U+3D54.0	fCNS11643	5-4678
U+3D54.0	fR/S	 85.12; 85.11
U+3D54.0	fTotalStrokes	 14; 15
U+3D54.0	fCangjie	EKJA
U+3D54.0	fFourCorner	34160
U+3D54.0	fCantonese	ce2; dou2; se2
U+3D54.0	fMandarin	che3; che4
U+3D54.0	fEnglish	not clear; not pure
U+3D55.0	fUTF8	㵕
U+3D55.0	fCNS11643	7-2461
U+3D55.0	fR/S	 85.12
U+3D55.0	fTotalStrokes	 15
U+3D55.0	fCangjie	EIIP
U+3D55.0	fFourCorner	33138
U+3D55.0	fCantonese	sam3
U+3D55.0	fMandarin	zao3; shen4
U+3D55.0	fEnglish	same as &#x6EF2;
U+3D56.0	fUTF8	㵖
U+3D56.0	fCNS11643	3-4C70
U+3D56.0	fR/S	 85.12
U+3D56.0	fTotalStrokes	 15
U+3D56.0	fCangjie	EVRF
U+3D56.0	fFourCorner	36190
U+3D56.0	fCantonese	neoi6; leoi6 (also); neoi3
U+3D56.0	fMandarin	nuu4; n4
U+3D56.0	fEnglish	damp; moist; wet
U+3D57.0	fUTF8	㵗
U+3D57.0	fCNS11643	4-5074
U+3D57.0	fGB	3-5328
U+3D57.0	fR/S	 85.12
U+3D57.0	fTotalStrokes	 16; 15
U+3D57.0	fCangjie	IFE
U+3D57.0	fFourCorner	31230
U+3D57.0	fCantonese	pang4
U+3D57.0	fMandarin	ping2; fu4; pan2
U+3D57.0	fEnglish	turbulent (said of water)
U+3D58.0	fUTF8	㵘
U+3D58.0	fCNS11643	3-514B
U+3D58.0	fR/S	 85.12
U+3D58.0	fTotalStrokes	 16
U+3D58.0	fCangjie	EEEE
U+3D58.0	fFourCorner	12230
U+3D58.0	fCantonese	maan6
U+3D58.0	fMandarin	man4
U+3D58.0	fEnglish	great currents
U+3D59.0	fUTF8	㵙
U+3D59.0	fCNS11643	3-4C76
U+3D59.0	fR/S	 85.12
U+3D59.0	fTotalStrokes	 15
U+3D59.0	fCangjie	EGRR
U+3D59.0	fFourCorner	34160
U+3D59.0	fCantonese	hei2; cing1; zak6
U+3D59.0	fMandarin	qing1
U+3D59.0	fEnglish	corrupted form of &#x6E05;
U+3D5A.0	fUTF8	㵚
U+3D5A.0	fGB	S-2360
U+3D5A.0	fR/S	 85.12
U+3D5A.0	fTotalStrokes	 15
U+3D5A.0	fCangjie	ECKC
U+3D5A.0	fCantonese	fu2
U+3D5B.0	fUTF8	㵛
U+3D5B.0	fR/S	 85.12
U+3D5B.0	fTotalStrokes	 15
U+3D5B.0	fCangjie	ETTR
U+3D5B.0	fCantonese	sin6
U+3D5B.0	fEnglish	[Korean character]
U+3D5C.0	fUTF8	㵜
U+3D5C.0	fR/S	 85.12
U+3D5C.0	fTotalStrokes	 15
U+3D5C.0	fCangjie	EEJB
U+3D5C.0	fCantonese	naam5; laam5 (also)
U+3D5C.0	fEnglish	[Korean character]
U+3D5D.0	fUTF8	㵝
U+3D5D.0	fCNS11643	4-5077
U+3D5D.0	fGB	3-564B
U+3D5D.0	fR/S	 85.13
U+3D5D.0	fTotalStrokes	 16
U+3D5D.0	fCangjie	EYVB
U+3D5D.0	fFourCorner	30120
U+3D5D.0	fHakka	 ri3; ji3; ji5
U+3D5D.0	fCantonese	jeoi6; deoi3; zeoi6
U+3D5D.0	fMandarin	yi4
U+3D5D.0	fEnglish	water roughered by wind
U+3D5E.0	fUTF8	㵞
U+3D5E.0	fCNS11643	4-507D
U+3D5E.0	fGB	3-5647
U+3D5E.0	fR/S	 85.13
U+3D5E.0	fTotalStrokes	 16
U+3D5E.0	fCangjie	EHFP
U+3D5E.0	fFourCorner	39130
U+3D5E.0	fCantonese	sau4
U+3D5E.0	fMandarin	chou2
U+3D5E.0	fEnglish	[1] bellied in water-gas [2] belch; burp
U+3D5F.0	fUTF8	㵟
U+3D5F.0	fCNS11643	5-5536
U+3D5F.0	fGB	S-235F
U+3D5F.0	fR/S	 85.13
U+3D5F.0	fTotalStrokes	 16
U+3D5F.0	fCangjie	EHOK
U+3D5F.0	fFourCorner	38140
U+3D5F.0	fHakka	  mi2
U+3D5F.0	fCantonese	mei4
U+3D5F.0	fMandarin	mei2
U+3D5F.0	fEnglish	fine; thin
U+3D60.0	fUTF8	㵠
U+3D60.0	fCNS11643	4-5076
U+3D60.0	fGB	3-564D
U+3D60.0	fR/S	 85.13
U+3D60.0	fTotalStrokes	 16
U+3D60.0	fCangjie	EJCU
U+3D60.0	fFourCorner	33170
U+3D60.0	fHakka	 kut7; fut7; kwut7
U+3D60.0	fCantonese	fat1; faat1; gwaat6
U+3D60.0	fMandarin	ku1
U+3D60.0	fEnglish	deep water; deep pool
U+3D61.0	fUTF8	㵡
U+3D61.0	fCNS11643	3-514F
U+3D61.0	fR/S	 85.13
U+3D61.0	fTotalStrokes	 16
U+3D61.0	fCangjie	EMBU
U+3D61.0	fFourCorner	31110
U+3D61.0	fCantonese	bok6
U+3D61.0	fMandarin	bo2; bao2
U+3D61.0	fEnglish	swift and torrential (said of flow of water); turbulent flow of water
U+3D62.0	fUTF8	㵢
U+3D62.0	fCNS11643	3-514E
U+3D62.0	fR/S	 85.13
U+3D62.0	fTotalStrokes	 16
U+3D62.0	fCangjie	EMBW
U+3D62.0	fFourCorner	31160
U+3D62.0	fCantonese	leoi4
U+3D62.0	fMandarin	lei2; lei3
U+3D62.0	fEnglish	a marsh in ancient times; in today's Shandong province, betweem Heze and Yun Cheng
U+3D63.0	fUTF8	㵣
U+3D63.0	fCNS11643	4-507A
U+3D63.0	fGB	5-5728
U+3D63.0	fR/S	 85.13
U+3D63.0	fTotalStrokes	 16
U+3D63.0	fCangjie	EAVO
U+3D63.0	fFourCorner	37180
U+3D63.0	fCantonese	hot3; sap3
U+3D63.0	fMandarin	ke3; luo3
U+3D63.0	fEnglish	[1] same as &#x6E34; [2] interchangeable with &#x6112;
U+3D64.0	fUTF8	㵤
U+3D64.0	fCNS11643	5-552D
U+3D64.0	fR/S	 85.13; 85.12
U+3D64.0	fTotalStrokes	 16
U+3D64.0	fCangjie	ESMO
U+3D64.0	fFourCorner	37130
U+3D64.0	fCantonese	saap3
U+3D64.0	fMandarin	sha4; dian4; xia2
U+3D64.0	fEnglish	 overflow; spread
U+3D65.0	fUTF8	㵥
U+3D65.0	fCNS11643	3-5153
U+3D65.0	fR/S	 85.13
U+3D65.0	fTotalStrokes	 16
U+3D65.0	fCangjie	EJPT
U+3D65.0	fFourCorner	33110
U+3D65.0	fCantonese	mat6
U+3D65.0	fMandarin	mi4; bi4
U+3D65.0	fEnglish	(variant) water currents; flow water
U+3D66.0	fUTF8	㵦
U+3D66.0	fCNS11643	3-5157
U+3D66.0	fGB	5-566B
U+3D66.0	fR/S	 85.13
U+3D66.0	fTotalStrokes	 16
U+3D66.0	fCangjie	EYKB
U+3D66.0	fFourCorner	34130
U+3D66.0	fCantonese	seoi5; ceoi4
U+3D66.0	fMandarin	sui3; sui2
U+3D66.0	fEnglish	same as &#x7021;
U+3D67.0	fUTF8	㵧
U+3D67.0	fCNS11643	4-507B
U+3D67.0	fGB	3-562A
U+3D67.0	fR/S	 85.13
U+3D67.0	fTotalStrokes	 16
U+3D67.0	fCangjie	ETAV
U+3D67.0	fFourCorner	34120
U+3D67.0	fHakka	 ai5
U+3D67.0	fCantonese	got3; aai3
U+3D67.0	fMandarin	ge2
U+3D67.0	fEnglish	[1] the name of a river [2] clear [3] pure [4] cover; hide
U+3D68.0	fUTF8	㵨
U+3D68.0	fCNS11643	4-5656
U+3D68.0	fGB	3-532D
U+3D68.0	fR/S	 85.13
U+3D68.0	fTotalStrokes	 17; 16
U+3D68.0	fCangjie	SJE
U+3D68.0	fFourCorner	70230
U+3D68.0	fCantonese	pei3
U+3D68.0	fMandarin	pi4; bi4
U+3D68.0	fEnglish	[1] a dry land in a river; islet [2] tributary stream; affluent stream
U+3D69.0	fUTF8	㵩
U+3D69.0	fCNS11643	3-5151
U+3D69.0	fR/S	 85.13
U+3D69.0	fTotalStrokes	 16
U+3D69.0	fCangjie	ETPD
U+3D69.0	fFourCorner	34190
U+3D69.0	fCantonese	jai6; zaai6; zit6
U+3D69.0	fMandarin	yi4
U+3D69.0	fEnglish	many; flourishing; luxuriant growth (of vegetation)
U+3D6A.0	fUTF8	㵪
U+3D6A.0	fCNS11643	3-514C
U+3D6A.0	fGB	5-5678
U+3D6A.0	fR/S	 85.13
U+3D6A.0	fTotalStrokes	 16
U+3D6A.0	fCangjie	ETGO
U+3D6A.0	fFourCorner	38180
U+3D6A.0	fCantonese	jin4; zin4
U+3D6A.0	fMandarin	xian2
U+3D6A.0	fEnglish	same as &#x6D8E;
U+3D6B.0	fUTF8	㵫
U+3D6B.0	fCNS11643	4-5078
U+3D6B.0	fGB	3-564F
U+3D6B.0	fR/S	 85.13
U+3D6B.0	fTotalStrokes	 16
U+3D6B.0	fCangjie	ENDA
U+3D6B.0	fFourCorner	37160
U+3D6B.0	fCantonese	nap6; lap6 (also); zi5; zik6
U+3D6B.0	fMandarin	yi4; ni3
U+3D6B.0	fEnglish	[1] water gushing out; bubbling up [2] waves sweeping over the flowing water
U+3D6C.0	fUTF8	㵬
U+3D6C.0	fCNS11643	3-5152
U+3D6C.0	fR/S	 85.13
U+3D6C.0	fTotalStrokes	 16
U+3D6C.0	fCangjie	EBNT
U+3D6C.0	fFourCorner	37110
U+3D6C.0	fCantonese	jing4; cap3; dip6; zing4
U+3D6C.0	fMandarin	ying2
U+3D6C.0	fEnglish	used in person's name
U+3D6D.0	fUTF8	㵭
U+3D6D.0	fCNS11643	4-4A62
U+3D6D.0	fGB	5-5667
U+3D6D.0	fR/S	 85.13; 85.12
U+3D6D.0	fTotalStrokes	 15; 16
U+3D6D.0	fCangjie	EJAF
U+3D6D.0	fFourCorner	34130
U+3D6D.0	fCantonese	zyu2; ceoi4; fu6
U+3D6D.0	fMandarin	zhu3
U+3D6D.0	fEnglish	[1] same as &#x6E1A; [2] interchangeable with &#x528C;
U+3D6E.0	fUTF8	㵮
U+3D6E.0	fCNS11643	5-552E
U+3D6E.0	fR/S	 85.13
U+3D6E.0	fTotalStrokes	 16
U+3D6E.0	fCangjie	EKTQ
U+3D6E.0	fFourCorner	34150
U+3D6E.0	fCantonese	seon4
U+3D6E.0	fMandarin	chun2; wen3
U+3D6E.0	fEnglish	clear (fresh) water
U+3D6F.0	fUTF8	㵯
U+3D6F.0	fCNS11643	3-5532
U+3D6F.0	fR/S	 85.14
U+3D6F.0	fTotalStrokes	 17
U+3D6F.0	fCangjie	EHNF
U+3D6F.0	fFourCorner	37110
U+3D6F.0	fCantonese	fung4
U+3D6F.0	fMandarin	feng2
U+3D6F.0	fEnglish	the sound of water flowing; flowing water
U+3D70.0	fUTF8	㵰
U+3D70.0	fCNS11643	3-5538
U+3D70.0	fR/S	 85.14
U+3D70.0	fTotalStrokes	 17
U+3D70.0	fCangjie	EHXC
U+3D70.0	fFourCorner	37180
U+3D70.0	fCantonese	zeoi6
U+3D70.0	fMandarin	xu4
U+3D70.0	fEnglish	same as &#x6F4A;
U+3D71.0	fUTF8	㵱
U+3D71.0	fCNS11643	4-565E
U+3D71.0	fR/S	 85.14
U+3D71.0	fTotalStrokes	 17; 16
U+3D71.0	fCangjie	EMFI
U+3D71.0	fFourCorner	34100
U+3D71.0	fCantonese	piu2; piu3
U+3D71.0	fMandarin	piao1; piao3
U+3D71.0	fEnglish	flowing water
U+3D72.0	fUTF8	㵲
U+3D72.0	fCNS11643	4-5660
U+3D72.0	fGB	3-5651
U+3D72.0	fR/S	 85.14
U+3D72.0	fTotalStrokes	 17; 16
U+3D72.0	fCangjie	EOTQ
U+3D72.0	fFourCorner	38150
U+3D72.0	fHakka	 wu3; vu3
U+3D72.0	fCantonese	mou5
U+3D72.0	fMandarin	wu3
U+3D72.0	fEnglish	a river in Hunan
U+3D73.0	fUTF8	㵳
U+3D73.0	fCNS11643	3-5534
U+3D73.0	fGB	5-5739
U+3D73.0	fR/S	 85.14
U+3D73.0	fTotalStrokes	 17
U+3D73.0	fCangjie	EJSH
U+3D73.0	fFourCorner	33120
U+3D73.0	fCantonese	liu4
U+3D73.0	fMandarin	liao2; liu2
U+3D73.0	fEnglish	same as non-classical form of &#x6F3B;
U+3D74.0	fUTF8	㵴
U+3D74.0	fCNS11643	4-5658
U+3D74.0	fGB	5-5734
U+3D74.0	fR/S	 85.14
U+3D74.0	fTotalStrokes	 17; 16
U+3D74.0	fCangjie	EIMS
U+3D74.0	fFourCorner	33150
U+3D74.0	fCantonese	cong4
U+3D74.0	fMandarin	cang2
U+3D74.0	fEnglish	 sink
U+3D75.0	fUTF8	㵵
U+3D75.0	fCNS11643	3-552E
U+3D75.0	fGB	5-5731
U+3D75.0	fR/S	 85.14
U+3D75.0	fTotalStrokes	 17
U+3D75.0	fCangjie	ESEO
U+3D75.0	fFourCorner	37180
U+3D75.0	fCantonese	zau6; tok3; zaau3; zeoi6
U+3D75.0	fMandarin	zou4; ju4
U+3D75.0	fEnglish	[1] swift currents of the stream [2] sound of water flowing; flowing water
U+3D76.0	fUTF8	㵶
U+3D76.0	fCNS11643	4-5657
U+3D76.0	fGB	5-5730
U+3D76.0	fR/S	 85.14
U+3D76.0	fTotalStrokes	 17; 16
U+3D76.0	fCangjie	EJIG
U+3D76.0	fFourCorner	33150
U+3D76.0	fCantonese	zit3; ceoi3; cyut3; seoi3; zik1
U+3D76.0	fMandarin	zuo3; ji1
U+3D76.0	fEnglish	full; filled up
U+3D77.0	fUTF8	㵷
U+3D77.0	fCNS11643	4-565C
U+3D77.0	fGB	5-7332
U+3D77.0	fR/S	 85.14
U+3D77.0	fTotalStrokes	 17; 16
U+3D77.0	fCangjie	YJEYJ
U+3D77.0	fFourCorner	00440
U+3D77.0	fCantonese	bin6
U+3D77.0	fMandarin	bian4
U+3D77.0	fEnglish	[1] turbulent flowing of the water [2] swift currents
U+3D78.0	fUTF8	㵸
U+3D78.0	fCNS11643	3-553B
U+3D78.0	fR/S	 85.14
U+3D78.0	fTotalStrokes	 17
U+3D78.0	fCangjie	EOMB
U+3D78.0	fFourCorner	38120
U+3D78.0	fCantonese	joek6; zoek6
U+3D78.0	fMandarin	yue4; yao4
U+3D78.0	fEnglish	same as non-classical abbreviated form of &#x7039;
U+3D79.0	fUTF8	㵹
U+3D79.0	fCNS11643	4-5659
U+3D79.0	fGB	3-5644
U+3D79.0	fR/S	 85.14
U+3D79.0	fTotalStrokes	 17; 16
U+3D79.0	fCangjie	EUTK
U+3D79.0	fFourCorner	34130
U+3D79.0	fCantonese	wun4
U+3D79.0	fMandarin	mo4; huan2
U+3D79.0	fEnglish	[1] waves; breakers; billows [2] dense; thick
U+3D7A.0	fUTF8	㵺
U+3D7A.0	fCNS11643	4-565B
U+3D7A.0	fGB	3-5652
U+3D7A.0	fR/S	 85.14
U+3D7A.0	fTotalStrokes	 16; 17
U+3D7A.0	fCangjie	EHHJ
U+3D7A.0	fFourCorner	38140
U+3D7A.0	fCantonese	paai4; hak1; pai4
U+3D7A.0	fMandarin	pai4; pai2; pi4
U+3D7A.0	fEnglish	[1] the name of a stream in ancient times in Danyang [2] plant; sow
U+3D7B.0	fUTF8	㵻
U+3D7B.0	fCNS11643	3-5533
U+3D7B.0	fR/S	 85.14
U+3D7B.0	fTotalStrokes	 17
U+3D7B.0	fCangjie	EBHE
U+3D7B.0	fFourCorner	37140
U+3D7B.0	fCantonese	jau1; saau3; sat3
U+3D7B.0	fMandarin	xiu2; sou4
U+3D7B.0	fEnglish	interchangeable with &#x8184;
U+3D7C.0	fUTF8	㵼
U+3D7C.0	fR/S	 85.14
U+3D7C.0	fTotalStrokes	 17
U+3D7C.0	fCangjie	EBHF
U+3D7C.0	fCantonese	se3
U+3D7C.0	fEnglish	[Japanese character]
U+3D7D.0	fUTF8	㵽
U+3D7D.0	fCNS11643	4-5B56
U+3D7D.0	fGB	3-565A
U+3D7D.0	fR/S	 85.15
U+3D7D.0	fTotalStrokes	 18
U+3D7D.0	fCangjie	EWWW
U+3D7D.0	fFourCorner	36160
U+3D7D.0	fCantonese	leoi5
U+3D7D.0	fMandarin	lei3; dui4
U+3D7D.0	fEnglish	same as abbreviated form of &#x7045;
U+3D7E.0	fUTF8	㵾
U+3D7E.0	fCNS11643	4-5B54
U+3D7E.0	fGB	3-5628
U+3D7E.0	fR/S	 85.15
U+3D7E.0	fTotalStrokes	 18
U+3D7E.0	fCangjie	EQBU
U+3D7E.0	fFourCorner	36110
U+3D7E.0	fCantonese	cing3; cing6
U+3D7E.0	fMandarin	jing4; qing4; cheng3
U+3D7E.0	fEnglish	[1] cold; chilly [2] ancient form of &#x51C8;
U+3D7F.0	fUTF8	㵿
U+3D7F.0	fCNS11643	4-5B5A
U+3D7F.0	fGB	3-565B
U+3D7F.0	fR/S	 85.15
U+3D7F.0	fTotalStrokes	 18
U+3D7F.0	fCangjie	EHAA
U+3D7F.0	fFourCorner	36160
U+3D7F.0	fCantonese	jiu5; haat6; hau6
U+3D7F.0	fMandarin	xiao4; xi2
U+3D7F.0	fEnglish	endlessly long or vast water currents
U+3D80.0	fUTF8	㶀
U+3D80.0	fCNS11643	4-5B57
U+3D80.0	fGB	3-565D
U+3D80.0	fR/S	 85.15
U+3D80.0	fTotalStrokes	 18
U+3D80.0	fCangjie	EBSH
U+3D80.0	fFourCorner	37120
U+3D80.0	fHakka	 gau1
U+3D80.0	fCantonese	gaau1; bak6; gau1; ngaau4; pak3
U+3D80.0	fMandarin	jiao1
U+3D80.0	fEnglish	[1] extensive [2] a name of a river
U+3D81.0	fUTF8	㶁
U+3D81.0	fCNS11643	4-5B60
U+3D81.0	fGB	3-565C
U+3D81.0	fR/S	 85.15
U+3D81.0	fTotalStrokes	 18
U+3D81.0	fCangjie	EBIU
U+3D81.0	fFourCorner	31110
U+3D81.0	fCantonese	gwik1; wak6
U+3D81.0	fMandarin	guo2; huo4
U+3D81.0	fEnglish	[1] after blocking the water flow takes different waterway [2] the sound of flowing water
U+3D82.0	fUTF8	㶂
U+3D82.0	fR/S	 85.15
U+3D82.0	fTotalStrokes	 18
U+3D82.0	fCangjie	EETC
U+3D82.0	fCantonese	wong4
U+3D82.0	fEnglish	[Korean character]
U+3D83.0	fUTF8	㶃
U+3D83.0	fR/S	 85.15
U+3D83.0	fTotalStrokes	 18
U+3D83.0	fCangjie	EEOF
U+3D83.0	fCantonese	mou5
U+3D83.0	fEnglish	[Korean character]
U+3D84.0	fUTF8	㶄
U+3D84.0	fCNS11643	4-5F7B
U+3D84.0	fGB	3-557B
U+3D84.0	fR/S	 85.16
U+3D84.0	fTotalStrokes	 19
U+3D84.0	fCangjie	EANX
U+3D84.0	fFourCorner	37120
U+3D84.0	fCantonese	jim4; teoi4; zim4
U+3D84.0	fMandarin	yan2; xian2; yan3
U+3D84.0	fEnglish	[1] stain [2] dirty; filth [3] water flows forward
U+3D85.0	fUTF8	㶅
U+3D85.0	fCNS11643	4-5F7D
U+3D85.0	fGB	5-557B
U+3D85.0	fR/S	 85.16
U+3D85.0	fTotalStrokes	 19
U+3D85.0	fCangjie	EHBD
U+3D85.0	fFourCorner	37140
U+3D85.0	fCantonese	hok6
U+3D85.0	fMandarin	xue2
U+3D85.0	fEnglish	same as &#x6FA9;
U+3D86.0	fUTF8	㶆
U+3D86.0	fCNS11643	4-5B5B
U+3D86.0	fGB	3-5561
U+3D86.0	fR/S	 85.16; 85.15
U+3D86.0	fTotalStrokes	 18; 19
U+3D86.0	fCangjie	EYRA
U+3D86.0	fFourCorner	34160
U+3D86.0	fHakka	 zhu1; zu1; chu2
U+3D86.0	fCantonese	zyu1
U+3D86.0	fMandarin	zhu1; chu2
U+3D86.0	fEnglish	a small branch of the Pei-ho
U+3D87.0	fUTF8	㶇
U+3D87.0	fCNS11643	4-5F79
U+3D87.0	fGB	5-573B
U+3D87.0	fR/S	 85.16
U+3D87.0	fTotalStrokes	 19
U+3D87.0	fCangjie	EDTC
U+3D87.0	fFourCorner	34180
U+3D87.0	fCantonese	waang4
U+3D87.0	fMandarin	heng4; heng2
U+3D87.0	fEnglish	[1] ferry; across the river in a boat; ferry boat [2] raft
U+3D88.0	fUTF8	㶈
U+3D88.0	fCNS11643	3-5A51
U+3D88.0	fR/S	 85.16
U+3D88.0	fTotalStrokes	 19
U+3D88.0	fCangjie	EFFI
U+3D88.0	fFourCorner	39130
U+3D88.0	fCantonese	jing4; zing4; zing6
U+3D88.0	fMandarin	ying4; ying2
U+3D88.0	fEnglish	same as &#x6FD9;
U+3D89.0	fUTF8	㶉
U+3D89.0	fGB	S-2366
U+3D89.0	fR/S	 85.16
U+3D89.0	fTotalStrokes	 19
U+3D89.0	fCangjie	EBKM
U+3D89.0	fCantonese	kai1
U+3D8A.0	fUTF8	㶊
U+3D8A.0	fR/S	 85.16
U+3D8A.0	fTotalStrokes	 19
U+3D8A.0	fCangjie	ESLC
U+3D8A.0	fCantonese	can2
U+3D8A.0	fEnglish	[Korean character]
U+3D8B.0	fUTF8	㶋
U+3D8B.0	fR/S	 85.16
U+3D8B.0	fTotalStrokes	 19
U+3D8B.0	fCangjie	ETOM
U+3D8B.0	fCantonese	jeoi6
U+3D8B.0	fEnglish	[Korean character]
U+3D8C.0	fUTF8	㶌
U+3D8C.0	fCNS11643	3-5C50
U+3D8C.0	fR/S	 85.17
U+3D8C.0	fTotalStrokes	 20
U+3D8C.0	fCangjie	ESJT
U+3D8C.0	fFourCorner	32170
U+3D8C.0	fCantonese	lin4; lyun4
U+3D8C.0	fMandarin	lian2
U+3D8C.0	fEnglish	the name of a stream, head source from Henan province Jiyuanxian Wangwushan
U+3D8D.0	fUTF8	㶍
U+3D8D.0	fCNS11643	4-635A
U+3D8D.0	fGB	3-5653
U+3D8D.0	fR/S	 85.17
U+3D8D.0	fTotalStrokes	 20
U+3D8D.0	fCangjie	ENFQ
U+3D8D.0	fFourCorner	38150
U+3D8D.0	fCantonese	sin2
U+3D8D.0	fMandarin	xian3
U+3D8D.0	fEnglish	the name of a stream
U+3D8E.0	fUTF8	㶎
U+3D8E.0	fCNS11643	4-635E
U+3D8E.0	fGB	3-5663
U+3D8E.0	fR/S	 85.17
U+3D8E.0	fTotalStrokes	 20
U+3D8E.0	fCangjie	EYWV
U+3D8E.0	fFourCorner	36130
U+3D8E.0	fCantonese	waan4
U+3D8E.0	fMandarin	huan2
U+3D8E.0	fEnglish	same as &#x6FB4;
U+3D8F.0	fUTF8	㶏
U+3D8F.0	fCNS11643	4-635B
U+3D8F.0	fGB	3-5624
U+3D8F.0	fR/S	 85.17
U+3D8F.0	fTotalStrokes	 20
U+3D8F.0	fCangjie	ENLP
U+3D8F.0	fFourCorner	32130
U+3D8F.0	fCantonese	jan1; jan2 (also); zaan1
U+3D8F.0	fMandarin	yin3
U+3D8F.0	fEnglish	an ancient name of a river in Henan province Dengfengxian
U+3D90.0	fUTF8	㶐
U+3D90.0	fR/S	 85.17
U+3D90.0	fTotalStrokes	 20
U+3D90.0	fCangjie	EIOP
U+3D90.0	fCantonese	jing3
U+3D90.0	fEnglish	[Korean character]
U+3D91.0	fUTF8	㶑
U+3D91.0	fCNS11643	3-5C52
U+3D91.0	fR/S	 85.17
U+3D91.0	fTotalStrokes	 20
U+3D91.0	fCangjie	EOOO
U+3D91.0	fFourCorner	38140
U+3D91.0	fCantonese	lim6
U+3D91.0	fMandarin	lian4
U+3D91.0	fEnglish	same as &#x7032;
U+3D92.0	fUTF8	㶒
U+3D92.0	fCNS11643	4-6649
U+3D92.0	fGB	3-542C
U+3D92.0	fR/S	 85.18
U+3D92.0	fTotalStrokes	 21
U+3D92.0	fCangjie	EANF
U+3D92.0	fFourCorner	37120
U+3D92.0	fCantonese	sim2
U+3D92.0	fMandarin	shen3; shan3; tan4
U+3D92.0	fEnglish	[1] the current of a stream [2] brave; courageous; decision with courage; (a person) of determination [3] swift flowing of the water currents [4] float; waft [5] overflow
U+3D93.0	fUTF8	㶓
U+3D93.0	fCNS11643	5-703E
U+3D93.0	fR/S	 85.18
U+3D93.0	fTotalStrokes	 21
U+3D93.0	fCangjie	ETIS
U+3D93.0	fFourCorner	34150
U+3D93.0	fCantonese	cong4
U+3D93.0	fMandarin	cang2
U+3D93.0	fEnglish	 sink; drown; be submerged
U+3D94.0	fUTF8	㶔
U+3D94.0	fCNS11643	4-664A
U+3D94.0	fGB	3-5671
U+3D94.0	fR/S	 85.18
U+3D94.0	fTotalStrokes	 21
U+3D94.0	fCangjie	EWLK
U+3D94.0	fFourCorner	36130
U+3D94.0	fCantonese	baai6; bai6; bei6
U+3D94.0	fMandarin	bei4; bai4
U+3D94.0	fEnglish	[1] swift and torrential (said of flow of water) [2] the name of a stream
U+3D95.0	fUTF8	㶕
U+3D95.0	fCNS11643	4-664C
U+3D95.0	fGB	3-5649
U+3D95.0	fR/S	 85.18
U+3D95.0	fTotalStrokes	 21
U+3D95.0	fCangjie	EHAA
U+3D95.0	fFourCorner	38120
U+3D95.0	fCantonese	gaan2
U+3D95.0	fMandarin	jian3
U+3D95.0	fEnglish	 wash rice
U+3D96.0	fUTF8	㶖
U+3D96.0	fCNS11643	4-6873
U+3D96.0	fGB	3-5672
U+3D96.0	fR/S	 85.18; 85.19
U+3D96.0	fTotalStrokes	 21; 22
U+3D96.0	fCangjie	EOLF
U+3D96.0	fFourCorner	38130
U+3D96.0	fCantonese	suk1
U+3D96.0	fMandarin	shu2; shu4
U+3D96.0	fEnglish	rapid; turbulent of the waves and billows
U+3D97.0	fUTF8	㶗
U+3D97.0	fCNS11643	3-5E39
U+3D97.0	fR/S	 85.18
U+3D97.0	fTotalStrokes	 22; 21
U+3D97.0	fCangjie	OFHAE
U+3D97.0	fFourCorner	82230
U+3D97.0	fCantonese	faan6; fan4; fan6
U+3D97.0	fMandarin	fan4
U+3D97.0	fEnglish	[1] well up; spring out of a fountain or geyser [2] the name of a fountain in today's Henan province
U+3D98.0	fUTF8	㶘
U+3D98.0	fCNS11643	3-5E38
U+3D98.0	fR/S	 85.18
U+3D98.0	fTotalStrokes	 21
U+3D98.0	fCangjie	EHMJ
U+3D98.0	fFourCorner	38140
U+3D98.0	fCantonese	tim5
U+3D98.0	fMandarin	dian4
U+3D98.0	fEnglish	full of water
U+3D99.0	fUTF8	㶙
U+3D99.0	fR/S	 85.19
U+3D99.0	fTotalStrokes	 22
U+3D99.0	fCangjie	EETM
U+3D99.0	fCantonese	jeoi6
U+3D99.0	fEnglish	[Korean character]
U+3D9A.0	fUTF8	㶚
U+3D9A.0	fCNS11643	3-5F51
U+3D9A.0	fR/S	 85.19
U+3D9A.0	fTotalStrokes	 22
U+3D9A.0	fCangjie	EMWB
U+3D9A.0	fFourCorner	31120
U+3D9A.0	fCantonese	baa3
U+3D9A.0	fMandarin	ba4
U+3D9A.0	fEnglish	variant of &#x705E;
U+3D9B.0	fUTF8	㶛
U+3D9B.0	fCNS11643	4-6A4E
U+3D9B.0	fGB	3-553A
U+3D9B.0	fR/S	 85.20
U+3D9B.0	fTotalStrokes	 23
U+3D9B.0	fCangjie	EYSC
U+3D9B.0	fFourCorner	38180
U+3D9B.0	fCantonese	jyu4; zyu4
U+3D9B.0	fMandarin	yu2
U+3D9B.0	fEnglish	[1] the name of a river [2] moving; swinging as in ripples
U+3D9C.0	fUTF8	㶜
U+3D9C.0	fCNS11643	3-6038
U+3D9C.0	fR/S	 85.20
U+3D9C.0	fTotalStrokes	 23
U+3D9C.0	fCangjie	ESTN
U+3D9C.0	fFourCorner	32100
U+3D9C.0	fCantonese	jyun1; zyun1
U+3D9C.0	fEnglish	same as &#x6DF5;
U+3D9D.0	fUTF8	㶝
U+3D9D.0	fR/S	 85.20
U+3D9D.0	fTotalStrokes	 23
U+3D9D.0	fCangjie	EEIP
U+3D9D.0	fCantonese	jing3
U+3D9D.0	fEnglish	[Korean character]
U+3D9E.0	fUTF8	㶞
U+3D9E.0	fCNS11643	4-6B6D
U+3D9E.0	fGB	5-5746
U+3D9E.0	fR/S	 85.21
U+3D9E.0	fTotalStrokes	 24
U+3D9E.0	fCangjie	EAYV
U+3D9E.0	fFourCorner	36130
U+3D9E.0	fCantonese	nong5; long5 (also)
U+3D9E.0	fMandarin	nang3
U+3D9E.0	fEnglish	same as &#x703C;
U+3D9F.0	fUTF8	㶟
U+3D9F.0	fCNS11643	4-6B6E
U+3D9F.0	fGB	3-5675
U+3D9F.0	fR/S	 85.21
U+3D9F.0	fTotalStrokes	 24
U+3D9F.0	fCangjie	EWWF
U+3D9F.0	fFourCorner	36190
U+3D9F.0	fCantonese	leoi5
U+3D9F.0	fMandarin	lei2; lei3
U+3D9F.0	fEnglish	an ancient name of a river in Shanxi and Hebei
U+3DA0.0	fUTF8	㶠
U+3DA0.0	fCNS11643	3-6166
U+3DA0.0	fR/S	 85.23
U+3DA0.0	fTotalStrokes	 26
U+3DA0.0	fCangjie	ESFJ
U+3DA0.0	fFourCorner	36140
U+3DA0.0	fCantonese	jik6; zik6
U+3DA0.0	fMandarin	yi4
U+3DA0.0	fEnglish	water currents; flowing of water
U+3DA1.0	fUTF8	㶡
U+3DA1.0	fCNS11643	3-242A
U+3DA1.0	fGB	5-3349
U+3DA1.0	fR/S	 86.2
U+3DA1.0	fTotalStrokes	 6
U+3DA1.0	fCangjie	HNF
U+3DA1.0	fFourCorner	77210
U+3DA1.0	fCantonese	daai6; fo2
U+3DA1.0	fMandarin	huo3; dai4; zuo2
U+3DA1.0	fEnglish	ancient form of &#x706B;
U+3DA2.0	fUTF8	㶢
U+3DA2.0	fR/S	 86.2
U+3DA2.0	fTotalStrokes	 6
U+3DA2.0	fCangjie	FKN
U+3DA2.0	fCantonese	gau2
U+3DA2.0	fEnglish	[Korean character]
U+3DA3.0	fUTF8	㶣
U+3DA3.0	fCNS11643	4-2452
U+3DA3.0	fGB	3-335D
U+3DA3.0	fR/S	 86.3
U+3DA3.0	fTotalStrokes	 7
U+3DA3.0	fCangjie	MJF
U+3DA3.0	fFourCorner	10800
U+3DA3.0	fHakka	 tam2
U+3DA3.0	fCantonese	cim4; cam4
U+3DA3.0	fMandarin	chan2; tan2; yin2
U+3DA3.0	fEnglish	[1] burn; heat [2] burn over a wider and wider area; glow; shine [3] light; brightness
U+3DA4.0	fUTF8	㶤
U+3DA4.0	fCNS11643	3-2661
U+3DA4.0	fR/S	 86.3
U+3DA4.0	fTotalStrokes	 7
U+3DA4.0	fCangjie	FNI
U+3DA4.0	fFourCorner	97820
U+3DA4.0	fCantonese	caau2
U+3DA4.0	fMandarin	chao3
U+3DA4.0	fEnglish	non-classical form of &#x7092;
U+3DA5.0	fUTF8	㶥
U+3DA5.0	fCNS11643	F-246E
U+3DA5.0	fGB	S-2157
U+3DA5.0	fR/S	 86.3
U+3DA5.0	fTotalStrokes	 7
U+3DA5.0	fCangjie	FMJ
U+3DA5.0	fFourCorner	9184
U+3DA5.0	fCantonese	gon1; soeng2
U+3DA6.0	fUTF8	㶦
U+3DA6.0	fCNS11643	3-296E
U+3DA6.0	fGB	5-5140
U+3DA6.0	fR/S	 86.4
U+3DA6.0	fTotalStrokes	 8
U+3DA6.0	fCangjie	FNIN
U+3DA6.0	fFourCorner	97820
U+3DA6.0	fCantonese	zeon6; zeon4
U+3DA6.0	fMandarin	jin4
U+3DA6.0	fEnglish	[1] ashes; embers [2] remnants of
U+3DA7.0	fUTF8	㶧
U+3DA7.0	fCNS11643	4-2727
U+3DA7.0	fGB	3-4F53
U+3DA7.0	fR/S	 86.4
U+3DA7.0	fTotalStrokes	 8
U+3DA7.0	fCangjie	FOB
U+3DA7.0	fFourCorner	94820
U+3DA7.0	fCantonese	nyun6; lyun6 (also); hei1
U+3DA7.0	fMandarin	nen4; nen3
U+3DA7.0	fEnglish	warm
U+3DA8.0	fUTF8	㶨
U+3DA8.0	fCNS11643	3-2978
U+3DA8.0	fR/S	 86.4
U+3DA8.0	fTotalStrokes	 8
U+3DA8.0	fCangjie	MFF
U+3DA8.0	fFourCorner	10330
U+3DA8.0	fCantonese	pui1; pei1; pei4
U+3DA8.0	fMandarin	pi1
U+3DA8.0	fEnglish	same as &#x708B;
U+3DA9.0	fUTF8	㶩
U+3DA9.0	fCNS11643	F-276E
U+3DA9.0	fGB	S-2153
U+3DA9.0	fR/S	 86.4
U+3DA9.0	fTotalStrokes	 8
U+3DA9.0	fCangjie	FLBU
U+3DA9.0	fFourCorner	9481
U+3DA9.0	fCantonese	jam4
U+3DAA.0	fUTF8	㶪
U+3DAA.0	fCNS11643	3-2973
U+3DAA.0	fR/S	 86.4
U+3DAA.0	fTotalStrokes	 8
U+3DAA.0	fCangjie	MFF
U+3DAA.0	fFourCorner	10800
U+3DAA.0	fCantonese	pui1
U+3DAA.0	fMandarin	pi1
U+3DAB.0	fUTF8	㶫
U+3DAB.0	fCNS11643	3-2D67
U+3DAB.0	fR/S	 86.5
U+3DAB.0	fTotalStrokes	 9
U+3DAB.0	fCangjie	KCF
U+3DAB.0	fFourCorner	40800
U+3DAB.0	fCantonese	liu5; liu4; liu6
U+3DAB.0	fMandarin	liao4; liao3
U+3DAB.0	fEnglish	corrupted form of &#x71CE;
U+3DAC.0	fUTF8	㶬
U+3DAC.0	fCNS11643	3-2D59
U+3DAC.0	fR/S	 86.5
U+3DAC.0	fTotalStrokes	 9
U+3DAC.0	fCangjie	FDJ
U+3DAC.0	fFourCorner	95890
U+3DAC.0	fHakka	 mat8
U+3DAC.0	fCantonese	mut6
U+3DAC.0	fMandarin	mo4; mei2
U+3DAC.0	fEnglish	 dull-burning fire
U+3DAD.0	fUTF8	㶭
U+3DAD.0	fCNS11643	3-2D66
U+3DAD.0	fGB	5-5149
U+3DAD.0	fR/S	 86.5
U+3DAD.0	fTotalStrokes	 9
U+3DAD.0	fCangjie	FVIS
U+3DAD.0	fFourCorner	94820
U+3DAD.0	fCantonese	jau2; aai2; ai1; zaau2
U+3DAD.0	fMandarin	you3; jiu4
U+3DAD.0	fEnglish	almost dry; not completely dried
U+3DAE.0	fUTF8	㶮
U+3DAE.0	fCNS11643	3-2D61
U+3DAE.0	fR/S	 86.5
U+3DAE.0	fTotalStrokes	 9
U+3DAE.0	fCangjie	MBF
U+3DAE.0	fFourCorner	10800
U+3DAE.0	fCantonese	siu1
U+3DAE.0	fMandarin	shao1
U+3DAE.0	fEnglish	same as &#x71D2;
U+3DAF.0	fUTF8	㶯
U+3DAF.0	fCNS11643	3-2D60
U+3DAF.0	fR/S	 86.5
U+3DAF.0	fTotalStrokes	 9
U+3DAF.0	fCangjie	FHHL
U+3DAF.0	fFourCorner	97820
U+3DAF.0	fCantonese	lau6; gaau3; laau6
U+3DAF.0	fMandarin	liu4
U+3DAF.0	fEnglish	fire
U+3DB0.0	fUTF8	㶰
U+3DB0.0	fCNS11643	3-2D5B
U+3DB0.0	fR/S	 86.5
U+3DB0.0	fTotalStrokes	 9
U+3DB0.0	fCangjie	FTM
U+3DB0.0	fFourCorner	94870
U+3DB0.0	fCantonese	ham4
U+3DB0.0	fMandarin	han2
U+3DB0.0	fEnglish	[1] flame; fire [2] light; brightness
U+3DB1.0	fUTF8	㶱
U+3DB1.0	fR/S	 86.5
U+3DB1.0	fTotalStrokes	 9
U+3DB1.0	fCangjie	FDM
U+3DB1.0	fCantonese	bun2
U+3DB1.0	fEnglish	[Korean character]
U+3DB2.0	fUTF8	㶲
U+3DB2.0	fCNS11643	4-2A27
U+3DB2.0	fGB	3-4F64
U+3DB2.0	fR/S	 86.5
U+3DB2.0	fTotalStrokes	 9
U+3DB2.0	fCangjie	FBQ
U+3DB2.0	fFourCorner	97820
U+3DB2.0	fCantonese	jung6; zung6
U+3DB2.0	fMandarin	yong4
U+3DB2.0	fEnglish	measuring unit for thermodynamics
U+3DB3.0	fUTF8	㶳
U+3DB3.0	fCNS11643	3-322E
U+3DB3.0	fGB	5-6735
U+3DB3.0	fR/S	 86.6
U+3DB3.0	fTotalStrokes	 10
U+3DB3.0	fCangjie	LQF
U+3DB3.0	fFourCorner	50800
U+3DB3.0	fCantonese	zeon6; zeon2 (also)
U+3DB3.0	fMandarin	jin4
U+3DB3.0	fEnglish	same as &#x71FC;
U+3DB4.0	fUTF8	㶴
U+3DB4.0	fCNS11643	4-2D73
U+3DB4.0	fGB	3-4F70
U+3DB4.0	fR/S	 86.6
U+3DB4.0	fTotalStrokes	 10
U+3DB4.0	fCangjie	FNIN
U+3DB4.0	fFourCorner	97820
U+3DB4.0	fCantonese	ci2
U+3DB4.0	fMandarin	chi3; shi3
U+3DB4.0	fEnglish	[1] a very great fire; flourishing flames [2] rich; exuberant; grand; prosperous
U+3DB5.0	fUTF8	㶵
U+3DB5.0	fCNS11643	3-3232
U+3DB5.0	fR/S	 86.6
U+3DB5.0	fTotalStrokes	 10
U+3DB5.0	fCangjie	OGF
U+3DB5.0	fFourCorner	22330
U+3DB5.0	fCantonese	jam6; zaam6
U+3DB5.0	fMandarin	ren4
U+3DB5.0	fEnglish	same as &#x9901;, &#x98EA;
U+3DB6.0	fUTF8	㶶
U+3DB6.0	fGB	S-215E
U+3DB6.0	fR/S	 86.6
U+3DB6.0	fTotalStrokes	 10
U+3DB6.0	fCangjie	FHBV
U+3DB6.0	fCantonese	nung1; lung1 (also); nung4
U+3DB6.0	fMandarin	nong2
U+3DB6.0	fEnglish	simplified form of &#x71F6;
U+3DB7.0	fUTF8	㶷
U+3DB7.0	fR/S	 86.6
U+3DB7.0	fTotalStrokes	 10
U+3DB7.0	fCangjie	FPA
U+3DB7.0	fFourCorner	9782
U+3DB7.0	fCantonese	ceon4
U+3DB7.0	fEnglish	[Korean character]
U+3DB8.0	fUTF8	㶸
U+3DB8.0	fR/S	 86.6
U+3DB8.0	fTotalStrokes	 10
U+3DB8.0	fCangjie	FKSS
U+3DB8.0	fCantonese	hip3
U+3DB8.0	fEnglish	[Korean character]
U+3DB9.0	fUTF8	㶹
U+3DB9.0	fCNS11643	3-372B
U+3DB9.0	fR/S	 86.7
U+3DB9.0	fTotalStrokes	 11
U+3DB9.0	fCangjie	FME
U+3DB9.0	fFourCorner	91830
U+3DB9.0	fCantonese	hung6
U+3DB9.0	fMandarin	hong4
U+3DB9.0	fEnglish	fire; flames
U+3DBA.0	fUTF8	㶺
U+3DBA.0	fCNS11643	3-372A
U+3DBA.0	fR/S	 86.7
U+3DBA.0	fTotalStrokes	 11
U+3DBA.0	fCangjie	FMKR
U+3DBA.0	fFourCorner	91863
U+3DBA.0	fCantonese	tim3; tim6
U+3DBA.0	fMandarin	tian4; han2
U+3DBA.0	fEnglish	the light or glow of fire
U+3DBB.0	fUTF8	㶻
U+3DBB.0	fCNS11643	3-3734
U+3DBB.0	fR/S	 86.7
U+3DBB.0	fTotalStrokes	 11
U+3DBB.0	fCangjie	HEQJF
U+3DBB.0	fFourCorner	27330
U+3DBB.0	fCantonese	fung1; bung2; fung3; gik1
U+3DBB.0	fMandarin	feng1
U+3DBB.0	fEnglish	same as &#x70FD;
U+3DBC.0	fUTF8	㶼
U+3DBC.0	fCNS11643	4-3271
U+3DBC.0	fGB	3-5021
U+3DBC.0	fR/S	 86.7
U+3DBC.0	fTotalStrokes	 11
U+3DBC.0	fCangjie	FIOK
U+3DBC.0	fFourCorner	93830
U+3DBC.0	fCantonese	zeon3; oi1
U+3DBC.0	fMandarin	ai1; xi1
U+3DBC.0	fEnglish	[1] very hot [2] burn [3] heat [4] roast [5] dazzling; bright; shining [6] a very great fire; flourishing flames
U+3DBD.0	fUTF8	㶽
U+3DBD.0	fGB	S-215D
U+3DBD.0	fR/S	 86.7
U+3DBD.0	fTotalStrokes	 11
U+3DBD.0	fCangjie	FROB
U+3DBD.0	fCantonese	wo1; gwaa1
U+3DBD.0	fEnglish	simplified form of &#x7171;
U+3DBE.0	fUTF8	㶾
U+3DBE.0	fCNS11643	4-3276
U+3DBE.0	fR/S	 86.7
U+3DBE.0	fTotalStrokes	 11
U+3DBE.0	fCangjie	MWMF
U+3DBE.0	fFourCorner	10800
U+3DBE.0	fCantonese	biu6; biu1
U+3DBE.0	fMandarin	piao1
U+3DBE.0	fEnglish	non-classical abbreviated form of &#x719B;
U+3DBF.0	fUTF8	㶿
U+3DBF.0	fCNS11643	4-3270
U+3DBF.0	fGB	3-4F73
U+3DBF.0	fR/S	 86.7
U+3DBF.0	fTotalStrokes	 11
U+3DBF.0	fCangjie	FJBD
U+3DBF.0	fFourCorner	94847
U+3DBF.0	fCantonese	but6
U+3DBF.0	fMandarin	bo2
U+3DBF.0	fEnglish	the smoke went up; giving forth smoke; smoking; misty
U+3DC0.0	fUTF8	㷀
U+3DC0.0	fCNS11643	4-3277
U+3DC0.0	fR/S	 86.7
U+3DC0.0	fTotalStrokes	 11
U+3DC0.0	fCangjie	FFNJ
U+3DC0.0	fFourCorner	99410
U+3DC0.0	fCantonese	king4
U+3DC0.0	fMandarin	qiong2
U+3DC0.0	fEnglish	same as &#x712D;, &#x7162; &#x60F8;
U+3DC1.0	fUTF8	㷁
U+3DC1.0	fCNS11643	F-3648
U+3DC1.0	fR/S	 86.7
U+3DC1.0	fTotalStrokes	 11
U+3DC1.0	fCangjie	FNIB
U+3DC1.0	fCantonese	jung2
U+3DC2.0	fUTF8	㷂
U+3DC2.0	fCNS11643	3-3C76
U+3DC2.0	fR/S	 86.8
U+3DC2.0	fTotalStrokes	 12
U+3DC2.0	fCangjie	SEF
U+3DC2.0	fFourCorner	77800
U+3DC2.0	fCantonese	seoi5; seoi6; syun4
U+3DC2.0	fMandarin	shu4
U+3DC2.0	fEnglish	bright; light; brilliant
U+3DC3.0	fUTF8	㷃
U+3DC3.0	fCNS11643	4-3865
U+3DC3.0	fGB	3-4F55
U+3DC3.0	fR/S	 86.8
U+3DC3.0	fTotalStrokes	 12
U+3DC3.0	fCangjie	FSMV
U+3DC3.0	fFourCorner	90730
U+3DC3.0	fCantonese	ceoi2; ceoi5
U+3DC3.0	fMandarin	cui1; cui3
U+3DC3.0	fEnglish	fire burns for a long time
U+3DC4.0	fUTF8	㷄
U+3DC4.0	fCNS11643	4-3861
U+3DC4.0	fR/S	 86.8
U+3DC4.0	fTotalStrokes	 12
U+3DC4.0	fCangjie	FWD
U+3DC4.0	fFourCorner	96890
U+3DC4.0	fCantonese	fui2; gun3; kui2
U+3DC4.0	fMandarin	hui3; hui4; guan4
U+3DC4.0	fEnglish	fire; flames
U+3DC5.0	fUTF8	㷅
U+3DC5.0	fCNS11643	3-3C75
U+3DC5.0	fGB	5-5169
U+3DC5.0	fR/S	 86.8
U+3DC5.0	fTotalStrokes	 12
U+3DC5.0	fCangjie	SEF
U+3DC5.0	fFourCorner	17800
U+3DC5.0	fCantonese	caau2
U+3DC5.0	fMandarin	cha2; chao3; miao3
U+3DC5.0	fEnglish	same as &#x7092;
U+3DC6.0	fUTF8	㷆
U+3DC6.0	fCNS11643	4-3862
U+3DC6.0	fR/S	 86.8
U+3DC6.0	fTotalStrokes	 12
U+3DC6.0	fCangjie	FHRJ
U+3DC6.0	fFourCorner	97840
U+3DC6.0	fCantonese	fau6; fu5
U+3DC6.0	fMandarin	fu4; dou4
U+3DC6.0	fEnglish	thriving; flourishing
U+3DC7.0	fUTF8	㷇
U+3DC7.0	fCNS11643	4-3864
U+3DC7.0	fGB	5-3C65
U+3DC7.0	fR/S	 86.8
U+3DC7.0	fTotalStrokes	 12
U+3DC7.0	fCangjie	NNKF
U+3DC7.0	fFourCorner	24280
U+3DC7.0	fCantonese	fui1
U+3DC7.0	fMandarin	kui1; guai4; kui2
U+3DC7.0	fEnglish	same as &#x6062;
U+3DC8.0	fUTF8	㷈
U+3DC8.0	fCNS11643	4-3860
U+3DC8.0	fGB	3-5025
U+3DC8.0	fR/S	 86.8
U+3DC8.0	fTotalStrokes	 12
U+3DC8.0	fCangjie	FKLU
U+3DC8.0	fFourCorner	94810
U+3DC8.0	fHakka	 ap7;  ep7; jam1; ram1
U+3DC8.0	fCantonese	ap1; ngap1 (also); at3
U+3DC8.0	fMandarin	yi4; e4
U+3DC8.0	fEnglish	 half-extinguished fire
U+3DC9.0	fUTF8	㷉
U+3DC9.0	fCNS11643	3-3B23
U+3DC9.0	fR/S	 86.8; 41.9
U+3DC9.0	fTotalStrokes	 12
U+3DC9.0	fCangjie	SFDI
U+3DC9.0	fFourCorner	74200
U+3DC9.0	fHakka	 vui3
U+3DC9.0	fCantonese	wai3; wan6; wat1
U+3DC9.0	fMandarin	wei4; yu4; yun4
U+3DC9.0	fEnglish	same as &#x5C09;
U+3DCA.0	fUTF8	㷊
U+3DCA.0	fCNS11643	3-3C7E
U+3DCA.0	fR/S	 86.8
U+3DCA.0	fTotalStrokes	 12
U+3DCA.0	fCangjie	DDF
U+3DCA.0	fFourCorner	44330
U+3DCA.0	fCantonese	fan4; faan4; faan5
U+3DCA.0	fMandarin	fen2
U+3DCA.0	fEnglish	non-classical form of &#x711A;
U+3DCB.0	fUTF8	㷋
U+3DCB.0	fCNS11643	3-3C70
U+3DCB.0	fR/S	 86.8
U+3DCB.0	fTotalStrokes	 12
U+3DCB.0	fCangjie	FFF
U+3DCB.0	fFourCorner	99880
U+3DCB.0	fCantonese	taam4; dam2; tam4
U+3DCB.0	fMandarin	tan2; tan3
U+3DCB.0	fEnglish	embers
U+3DCC.0	fUTF8	㷌
U+3DCC.0	fGB	S-2158
U+3DCC.0	fR/S	 86.8
U+3DCC.0	fTotalStrokes	 12
U+3DCC.0	fCangjie	FSMB
U+3DCC.0	fCantonese	zau2
U+3DCD.0	fUTF8	㷍
U+3DCD.0	fCNS11643	3-3C78
U+3DCD.0	fR/S	 86.8
U+3DCD.0	fTotalStrokes	 12
U+3DCD.0	fCangjie	FOMB
U+3DCD.0	fFourCorner	98820
U+3DCD.0	fCantonese	leon4
U+3DCD.0	fMandarin	lun2
U+3DCD.0	fEnglish	a chemical term, known formerly as benzene
U+3DCE.0	fUTF8	㷎
U+3DCE.0	fCNS11643	3-4254
U+3DCE.0	fR/S	 86.9
U+3DCE.0	fTotalStrokes	 13
U+3DCE.0	fCangjie	FAPV
U+3DCE.0	fFourCorner	96820
U+3DCE.0	fHakka	 jat7
U+3DCE.0	fCantonese	hot3; kit3
U+3DCE.0	fMandarin	ye4; he4; hong2; xie2
U+3DCE.0	fEnglish	same as &#x668D;
U+3DCF.0	fUTF8	㷏
U+3DCF.0	fCNS11643	3-424D
U+3DCF.0	fGB	5-517A
U+3DCF.0	fR/S	 86.9
U+3DCF.0	fTotalStrokes	 13
U+3DCF.0	fCangjie	HJF
U+3DCF.0	fFourCorner	28800
U+3DCF.0	fCantonese	jung2; zung2
U+3DCF.0	fMandarin	yong3
U+3DCF.0	fEnglish	used in a person's name
U+3DD0.0	fUTF8	㷐
U+3DD0.0	fCNS11643	3-4256
U+3DD0.0	fR/S	 86.9
U+3DD0.0	fTotalStrokes	 13
U+3DD0.0	fCangjie	FHXG
U+3DD0.0	fFourCorner	97810
U+3DD0.0	fCantonese	wai2; kui2; waai2
U+3DD0.0	fMandarin	hui3
U+3DD0.0	fEnglish	same as &#x71EC;
U+3DD1.0	fUTF8	㷑
U+3DD1.0	fCNS11643	4-3E4B
U+3DD1.0	fR/S	 86.9
U+3DD1.0	fTotalStrokes	 13
U+3DD1.0	fCangjie	FMWG
U+3DD1.0	fFourCorner	90100
U+3DD1.0	fCantonese	jin1; nim6; soeng1; zaan1; zin1
U+3DD1.0	fMandarin	yan1
U+3DD1.0	fEnglish	same as &#x7159;, &#x6348;
U+3DD2.0	fUTF8	㷒
U+3DD2.0	fCNS11643	3-424A
U+3DD2.0	fR/S	 86.9
U+3DD2.0	fTotalStrokes	 13
U+3DD2.0	fCangjie	FWLB
U+3DD2.0	fFourCorner	96820
U+3DD2.0	fCantonese	jyu4; zyu4
U+3DD2.0	fMandarin	yu2
U+3DD2.0	fEnglish	 cook meals; cook food; prepare a dinner
U+3DD3.0	fUTF8	㷓
U+3DD3.0	fCNS11643	3-4255
U+3DD3.0	fGB	5-517D
U+3DD3.0	fR/S	 86.9
U+3DD3.0	fTotalStrokes	 13
U+3DD3.0	fCangjie	FPKP
U+3DD3.0	fFourCorner	97830
U+3DD3.0	fCantonese	zung2
U+3DD3.0	fMandarin	zong3
U+3DD3.0	fEnglish	same as &#x719C;
U+3DD4.0	fUTF8	㷔
U+3DD4.0	fCNS11643	4-3E48
U+3DD4.0	fR/S	 86.9
U+3DD4.0	fTotalStrokes	 13
U+3DD4.0	fCangjie	FBLA
U+3DD4.0	fFourCorner	92860
U+3DD4.0	fCantonese	jim6; zim6
U+3DD4.0	fMandarin	yan4
U+3DD4.0	fEnglish	same as non-classical form of &#x7130;
U+3DD5.0	fUTF8	㷕
U+3DD5.0	fCNS11643	4-3E46
U+3DD5.0	fGB	5-517E
U+3DD5.0	fR/S	 86.9
U+3DD5.0	fTotalStrokes	 13
U+3DD5.0	fCangjie	TWF
U+3DD5.0	fFourCorner	89680
U+3DD5.0	fCantonese	cau4; zaau4
U+3DD5.0	fMandarin	qiu2
U+3DD5.0	fEnglish	hearing sound
U+3DD6.0	fUTF8	㷖
U+3DD6.0	fCNS11643	4-3E49
U+3DD6.0	fR/S	 86.9
U+3DD6.0	fTotalStrokes	 13
U+3DD6.0	fCangjie	AFSHR
U+3DD6.0	fFourCorner	67860
U+3DD6.0	fCantonese	ziu3
U+3DD6.0	fMandarin	zhao4
U+3DD6.0	fEnglish	same as &#x7167;
U+3DD7.0	fUTF8	㷗
U+3DD7.0	fCNS11643	6-5454
U+3DD7.0	fR/S	 86.9
U+3DD7.0	fTotalStrokes	 13
U+3DD7.0	fCangjie	LUF
U+3DD7.0	fFourCorner	27800
U+3DD7.0	fCantonese	gwing2
U+3DD7.0	fMandarin	jiong3
U+3DD7.0	fEnglish	(a corrupted form) used in a person's name
U+3DD8.0	fUTF8	㷘
U+3DD8.0	fCNS11643	3-424B
U+3DD8.0	fR/S	 86.9
U+3DD8.0	fTotalStrokes	 13
U+3DD8.0	fCangjie	FIRD
U+3DD8.0	fFourCorner	93890
U+3DD8.0	fCantonese	toi4
U+3DD8.0	fMandarin	tai2
U+3DD8.0	fEnglish	coal; charcoal
U+3DD9.0	fUTF8	㷙
U+3DD9.0	fCNS11643	3-4250
U+3DD9.0	fR/S	 86.9
U+3DD9.0	fTotalStrokes	 13
U+3DD9.0	fCangjie	FTBN
U+3DD9.0	fFourCorner	98820
U+3DD9.0	fCantonese	zin1
U+3DD9.0	fMandarin	jian1
U+3DD9.0	fEnglish	same as &#x714E;
U+3DDA.0	fUTF8	㷚
U+3DDA.0	fCNS11643	F-4278
U+3DDA.0	fGB	S-2154
U+3DDA.0	fR/S	 86.9
U+3DDA.0	fTotalStrokes	 13
U+3DDA.0	fCangjie	FYRN
U+3DDA.0	fCantonese	ting4
U+3DDB.0	fUTF8	㷛
U+3DDB.0	fCNS11643	F-433C
U+3DDB.0	fGB	S-215B
U+3DDB.0	fR/S	 86.9
U+3DDB.0	fTotalStrokes	 13
U+3DDB.0	fCangjie	ODF
U+3DDB.0	fHakka	 bau1; bo1
U+3DDB.0	fCantonese	bou1
U+3DDB.0	fMandarin	bao1; bao4
U+3DDB.0	fEnglish	same as &#x7172;
U+3DDC.0	fUTF8	㷜
U+3DDC.0	fR/S	 86.9
U+3DDC.0	fTotalStrokes	 13
U+3DDC.0	fCangjie	FYCK
U+3DDC.0	fCantonese	jik6
U+3DDC.0	fEnglish	[Korean character]
U+3DDD.0	fUTF8	㷝
U+3DDD.0	fR/S	 86.9
U+3DDD.0	fTotalStrokes	 13
U+3DDD.0	fCangjie	FJCK
U+3DDD.0	fCantonese	dat6
U+3DDD.0	fEnglish	[Korean character]
U+3DDE.0	fUTF8	㷞
U+3DDE.0	fR/S	 86.9
U+3DDE.0	fTotalStrokes	 13
U+3DDE.0	fCangjie	FMAD
U+3DDE.0	fCantonese	hau5
U+3DDE.0	fEnglish	[Korean character]
U+3DDF.0	fUTF8	㷟
U+3DDF.0	fCNS11643	4-4459
U+3DDF.0	fGB	5-517C
U+3DDF.0	fR/S	 86.10
U+3DDF.0	fTotalStrokes	 14
U+3DDF.0	fCangjie	FYHR
U+3DDF.0	fFourCorner	97830
U+3DDF.0	fHakka	 tui1; chui1; zui1; tui5
U+3DDF.0	fCantonese	teoi3
U+3DDF.0	fMandarin	tui1; tui4
U+3DDF.0	fEnglish	same as non-classical form of &#x5B2F;
U+3DE0.0	fUTF8	㷠
U+3DE0.0	fCNS11643	3-4773
U+3DE0.0	fGB	5-5229
U+3DE0.0	fR/S	 86.10
U+3DE0.0	fTotalStrokes	 14
U+3DE0.0	fCangjie	FFNIQ
U+3DE0.0	fFourCorner	90250
U+3DE0.0	fHakka	  lin2; nin2
U+3DE0.0	fCantonese	leon4
U+3DE0.0	fMandarin	lin4; lin2
U+3DE0.0	fEnglish	a will o' the wisp, Ignis fatuus
U+3DE1.0	fUTF8	㷡
U+3DE1.0	fCNS11643	3-4769
U+3DE1.0	fR/S	 86.10
U+3DE1.0	fTotalStrokes	 14
U+3DE1.0	fCangjie	BSF
U+3DE1.0	fFourCorner	61800
U+3DE1.0	fCantonese	gwing2; waan4; wo1
U+3DE1.0	fMandarin	jiong3
U+3DE1.0	fEnglish	 startle; amaze; astound
U+3DE2.0	fUTF8	㷢
U+3DE2.0	fCNS11643	6-5467
U+3DE2.0	fGB	3-5037
U+3DE2.0	fR/S	 86.10
U+3DE2.0	fTotalStrokes	 13; 14
U+3DE2.0	fCangjie	TQMF
U+3DE2.0	fFourCorner	80809
U+3DE2.0	fCantonese	zaa2
U+3DE2.0	fMandarin	ci1; zha3
U+3DE2.0	fEnglish	[1] bundle of charcoal [2] dry in the sun
U+3DE3.0	fUTF8	㷣
U+3DE3.0	fCNS11643	4-4456
U+3DE3.0	fGB	5-5228
U+3DE3.0	fR/S	 86.10
U+3DE3.0	fTotalStrokes	 14
U+3DE3.0	fCangjie	FTGQ
U+3DE3.0	fFourCorner	98850
U+3DE3.0	fCantonese	sing1
U+3DE3.0	fMandarin	xing1
U+3DE3.0	fEnglish	red; the color of fire
U+3DE4.0	fUTF8	㷤
U+3DE4.0	fCNS11643	4-445A
U+3DE4.0	fGB	3-4F35
U+3DE4.0	fR/S	 86.10
U+3DE4.0	fTotalStrokes	 14
U+3DE4.0	fCangjie	GFHNE
U+3DE4.0	fFourCorner	47840
U+3DE4.0	fCantonese	huk1; huk6; kok3
U+3DE4.0	fMandarin	hu4; he4; xue4
U+3DE4.0	fEnglish	[1] rosy clouds of dawn; rosy dawn [2] sounds of a fire
U+3DE5.0	fUTF8	㷥
U+3DE5.0	fCNS11643	4-445C
U+3DE5.0	fGB	5-522A
U+3DE5.0	fR/S	 86.10
U+3DE5.0	fTotalStrokes	 14
U+3DE5.0	fCangjie	FFNEM
U+3DE5.0	fFourCorner	97810
U+3DE5.0	fCantonese	zing1
U+3DE5.0	fMandarin	zheng1
U+3DE5.0	fEnglish	same as &#x70DD;
U+3DE6.0	fUTF8	㷦
U+3DE6.0	fCNS11643	3-4771
U+3DE6.0	fR/S	 86.10
U+3DE6.0	fTotalStrokes	 14
U+3DE6.0	fCangjie	SFF
U+3DE6.0	fFourCorner	19330
U+3DE6.0	fCantonese	gwik1; zik6
U+3DE6.0	fMandarin	xi4; xu4
U+3DE6.0	fEnglish	non-classical form of &#x7781;
U+3DE7.0	fUTF8	㷧
U+3DE7.0	fGB	S-215C
U+3DE7.0	fR/S	 86.10
U+3DE7.0	fTotalStrokes	 14
U+3DE7.0	fCangjie	FMHF
U+3DE7.0	fFourCorner	9189
U+3DE7.0	fCantonese	jyun4
U+3DE8.0	fUTF8	㷨
U+3DE8.0	fCNS11643	F-4958
U+3DE8.0	fGB	S-2155
U+3DE8.0	fR/S	 86.10
U+3DE8.0	fTotalStrokes	 14
U+3DE8.0	fCangjie	FUHJ
U+3DE8.0	fFourCorner	9285
U+3DE8.0	fCantonese	fung1
U+3DE9.0	fUTF8	㷩
U+3DE9.0	fR/S	 86.10
U+3DE9.0	fTotalStrokes	 14
U+3DE9.0	fCangjie	LUF
U+3DE9.0	fFourCorner	2780
U+3DE9.0	fCantonese	hei1
U+3DE9.0	fEnglish	[Korean character]
U+3DEA.0	fUTF8	㷪
U+3DEA.0	fGB	3-5041
U+3DEA.0	fR/S	 86.10
U+3DEA.0	fTotalStrokes	 14
U+3DEA.0	fCangjie	FOGS
U+3DEA.0	fCantonese	zeon3; zyun2
U+3DEA.0	fMandarin	cui4; zuan3
U+3DEA.0	fEnglish	fat; rich, a stew of fish
U+3DEB.0	fUTF8	㷫
U+3DEB.0	fCNS11643	3-4D26
U+3DEB.0	fR/S	 86.11
U+3DEB.0	fTotalStrokes	 15
U+3DEB.0	fCangjie	GEF
U+3DEB.0	fFourCorner	47800
U+3DEB.0	fHakka	 kin3; kin5
U+3DEB.0	fCantonese	hing3; kaai2
U+3DEB.0	fMandarin	qing4; qing3
U+3DEB.0	fEnglish	hot; scorching
U+3DEC.0	fUTF8	㷬
U+3DEC.0	fCNS11643	3-4D23
U+3DEC.0	fR/S	 86.11
U+3DEC.0	fTotalStrokes	 15
U+3DEC.0	fCangjie	FTAK
U+3DEC.0	fFourCorner	94830
U+3DEC.0	fCantonese	mok6; mung4
U+3DEC.0	fMandarin	mo4
U+3DEC.0	fEnglish	fire
U+3DED.0	fUTF8	㷭
U+3DED.0	fCNS11643	3-4D2B
U+3DED.0	fGB	5-5227
U+3DED.0	fR/S	 86.11
U+3DED.0	fTotalStrokes	 15
U+3DED.0	fCangjie	YJF
U+3DED.0	fFourCorner	37800
U+3DED.0	fCantonese	fung1
U+3DED.0	fMandarin	feng1
U+3DED.0	fEnglish	same as &#x70FD;
U+3DEE.0	fUTF8	㷮
U+3DEE.0	fCNS11643	3-4D21
U+3DEE.0	fGB	5-5230
U+3DEE.0	fR/S	 86.11
U+3DEE.0	fTotalStrokes	 15
U+3DEE.0	fCangjie	FTWA
U+3DEE.0	fFourCorner	95860
U+3DEE.0	fCantonese	zou1
U+3DEE.0	fMandarin	zao1
U+3DEE.0	fEnglish	[1] scorched or burned wood [2] burn; heat [3] destroy in fire [4] roast; bake
U+3DEF.0	fUTF8	㷯
U+3DEF.0	fCNS11643	3-4D27
U+3DEF.0	fR/S	 86.11
U+3DEF.0	fTotalStrokes	 15
U+3DEF.0	fCangjie	FGQKQ
U+3DEF.0	fFourCorner	95150
U+3DEF.0	fCantonese	bung6; bang1; bit6; pung2
U+3DEF.0	fMandarin	peng3; beng4
U+3DEF.0	fEnglish	smoke and dust everywhere; air pollution
U+3DF0.0	fUTF8	㷰
U+3DF0.0	fCNS11643	4-4A6A
U+3DF0.0	fGB	3-5043
U+3DF0.0	fR/S	 86.11
U+3DF0.0	fTotalStrokes	 15
U+3DF0.0	fCangjie	FYUB
U+3DF0.0	fFourCorner	90820
U+3DF0.0	fCantonese	ci1
U+3DF0.0	fMandarin	chi1; li2
U+3DF0.0	fEnglish	flames
U+3DF1.0	fUTF8	㷱
U+3DF1.0	fR/S	 86.11
U+3DF1.0	fTotalStrokes	 15
U+3DF1.0	fCangjie	IIF
U+3DF1.0	fCantonese	hung4
U+3DF1.0	fEnglish	[Japanese character]
U+3DF2.0	fUTF8	㷲
U+3DF2.0	fR/S	 86.11
U+3DF2.0	fTotalStrokes	 15
U+3DF2.0	fCangjie	FHGS
U+3DF2.0	fCantonese	dung6
U+3DF2.0	fEnglish	[Korean character]
U+3DF3.0	fUTF8	㷳
U+3DF3.0	fCNS11643	4-5125
U+3DF3.0	fGB	3-3041
U+3DF3.0	fR/S	 86.12
U+3DF3.0	fTotalStrokes	 16
U+3DF3.0	fCangjie	MOGF
U+3DF3.0	fFourCorner	71800
U+3DF3.0	fCantonese	ngon6; on6 (also); ngan6
U+3DF3.0	fMandarin	yan4
U+3DF3.0	fEnglish	color of s flame
U+3DF4.0	fUTF8	㷴
U+3DF4.0	fCNS11643	3-515D
U+3DF4.0	fR/S	 86.12
U+3DF4.0	fTotalStrokes	 16
U+3DF4.0	fCangjie	MHDF
U+3DF4.0	fFourCorner	71280
U+3DF4.0	fCantonese	lik6
U+3DF4.0	fMandarin	li4; ge2
U+3DF4.0	fEnglish	samr as &#x720F;
U+3DF5.0	fUTF8	㷵
U+3DF5.0	fCNS11643	3-515B
U+3DF5.0	fR/S	 86.12
U+3DF5.0	fTotalStrokes	 16
U+3DF5.0	fCangjie	FWGF
U+3DF5.0	fFourCorner	96830
U+3DF5.0	fCantonese	mak6
U+3DF5.0	fMandarin	mo4
U+3DF5.0	fEnglish	same as &#x7205;
U+3DF6.0	fUTF8	㷶
U+3DF6.0	fCNS11643	4-512B
U+3DF6.0	fGB	5-525B
U+3DF6.0	fR/S	 86.12
U+3DF6.0	fTotalStrokes	 16
U+3DF6.0	fCangjie	OBF
U+3DF6.0	fFourCorner	24330
U+3DF6.0	fCantonese	bik1
U+3DF6.0	fMandarin	bi4; bei4
U+3DF6.0	fEnglish	 dry or warm (grains) near a fire
U+3DF7.0	fUTF8	㷷
U+3DF7.0	fCNS11643	3-515F
U+3DF7.0	fR/S	 86.12
U+3DF7.0	fTotalStrokes	 16
U+3DF7.0	fCangjie	FRUC
U+3DF7.0	fFourCorner	97880
U+3DF7.0	fCantonese	zeon3; gyun2
U+3DF7.0	fMandarin	zui1; juan3
U+3DF7.0	fEnglish	same as &#x81C7;
U+3DF8.0	fUTF8	㷸
U+3DF8.0	fCNS11643	3-5161
U+3DF8.0	fR/S	 86.12
U+3DF8.0	fTotalStrokes	 16
U+3DF8.0	fCangjie	FAMJ
U+3DF8.0	fFourCorner	96850
U+3DF8.0	fCantonese	jip6; zaap1; zik1; zip6
U+3DF8.0	fMandarin	ye4; die2
U+3DF8.0	fEnglish	variant of &#x71C1;
U+3DF9.0	fUTF8	㷹
U+3DF9.0	fCNS11643	4-5124
U+3DF9.0	fGB	3-504A
U+3DF9.0	fR/S	 86.12
U+3DF9.0	fTotalStrokes	 16
U+3DF9.0	fCangjie	FTYJ
U+3DF9.0	fFourCorner	94840
U+3DF9.0	fHakka	 cau5; cau6
U+3DF9.0	fCantonese	zaau3; zoek3
U+3DF9.0	fMandarin	zhao4; shao4
U+3DF9.0	fEnglish	[1] fry [2] blazing fire
U+3DFA.0	fUTF8	㷺
U+3DFA.0	fGB	S-2161
U+3DFA.0	fR/S	 86.12
U+3DFA.0	fTotalStrokes	 16
U+3DFA.0	fCangjie	NKF
U+3DFA.0	fCantonese	saat3
U+3DFB.0	fUTF8	㷻
U+3DFB.0	fCNS11643	3-5164
U+3DFB.0	fR/S	 86.12
U+3DFB.0	fTotalStrokes	 16
U+3DFB.0	fCangjie	FOTF
U+3DFB.0	fFourCorner	98830
U+3DFB.0	fHakka	 fu2
U+3DFB.0	fCantonese	mou4
U+3DFB.0	fMandarin	wu2
U+3DFB.0	fEnglish	[1] same as &#x7121; [2] variant of &#x5B2D;
U+3DFC.0	fUTF8	㷼
U+3DFC.0	fCNS11643	3-5165
U+3DFC.0	fR/S	 86.12
U+3DFC.0	fTotalStrokes	 16
U+3DFC.0	fCangjie	TLPF
U+3DFC.0	fFourCorner	44800
U+3DFC.0	fCantonese	jin3; zin3
U+3DFC.0	fMandarin	yan4
U+3DFC.0	fEnglish	[1] standard form of &#x71D5; [2] interchangeable with &#x664F;
U+3DFD.0	fUTF8	㷽
U+3DFD.0	fR/S	 86.12
U+3DFD.0	fTotalStrokes	 16
U+3DFD.0	fCangjie	FTTR
U+3DFD.0	fFourCorner	9060
U+3DFD.0	fCantonese	sin6
U+3DFD.0	fEnglish	[Korean character]
U+3DFE.0	fUTF8	㷾
U+3DFE.0	fCNS11643	3-553D
U+3DFE.0	fR/S	 86.13
U+3DFE.0	fTotalStrokes	 17
U+3DFE.0	fCangjie	FYPO
U+3DFE.0	fFourCorner	91830
U+3DFE.0	fCantonese	koek3; koek6
U+3DFE.0	fMandarin	jue2
U+3DFE.0	fEnglish	blaze of fire; burning vigorously
U+3DFF.0	fUTF8	㷿
U+3DFF.0	fCNS11643	4-5662
U+3DFF.0	fGB	3-4F7B
U+3DFF.0	fR/S	 86.13
U+3DFF.0	fTotalStrokes	 17; 16
U+3DFF.0	fCangjie	FOMO
U+3DFF.0	fFourCorner	98880
U+3DFF.0	fCantonese	him1; hin1
U+3DFF.0	fMandarin	xian1
U+3DFF.0	fEnglish	[1] flame; fire [2] bitter; acrid taste
U+3E00.0	fUTF8	㸀
U+3E00.0	fCNS11643	3-553F
U+3E00.0	fR/S	 86.13
U+3E00.0	fTotalStrokes	 17
U+3E00.0	fCangjie	FYRG
U+3E00.0	fFourCorner	90810
U+3E00.0	fCantonese	toi4; toi6
U+3E00.0	fMandarin	tai2
U+3E00.0	fEnglish	non-classical form of &#x70B1;
U+3E01.0	fUTF8	㸁
U+3E01.0	fCNS11643	3-5544
U+3E01.0	fR/S	 86.13
U+3E01.0	fTotalStrokes	 17
U+3E01.0	fCangjie	FTCD
U+3E01.0	fFourCorner	93890
U+3E01.0	fCantonese	hon3; hon2
U+3E01.0	fMandarin	han4; han3
U+3E01.0	fEnglish	[1] corrupted form of &#x71AF; [2] same as &#x710A;
U+3E02.0	fUTF8	㸂
U+3E02.0	fCNS11643	7-3926
U+3E02.0	fR/S	 86.13
U+3E02.0	fTotalStrokes	 17
U+3E02.0	fCangjie	YOF
U+3E02.0	fFourCorner	38800
U+3E02.0	fCantonese	seoi6
U+3E02.0	fMandarin	sui4
U+3E03.0	fUTF8	㸃
U+3E03.0	fCNS11643	3-5751
U+3E03.0	fR/S	 86.13
U+3E03.0	fTotalStrokes	 17
U+3E03.0	fCangjie	WRF
U+3E03.0	fFourCorner	61330
U+3E03.0	fCantonese	dim2; ding2
U+3E03.0	fMandarin	dian3
U+3E03.0	fEnglish	same as &#x9EDE;
U+3E04.0	fUTF8	㸄
U+3E04.0	fCNS11643	3-582C
U+3E04.0	fR/S	 86.14
U+3E04.0	fTotalStrokes	 18
U+3E04.0	fCangjie	FYX
U+3E04.0	fFourCorner	90820
U+3E04.0	fCantonese	zai6
U+3E04.0	fMandarin	ji4; jie2
U+3E04.0	fEnglish	same as &#x9F4C;
U+3E05.0	fUTF8	㸅
U+3E05.0	fCNS11643	4-5664
U+3E05.0	fGB	5-5142
U+3E05.0	fR/S	 86.14; 86.13
U+3E05.0	fTotalStrokes	 16; 18; 17
U+3E05.0	fCangjie	FHAL
U+3E05.0	fFourCorner	98820
U+3E05.0	fCantonese	zit3
U+3E05.0	fMandarin	jie2
U+3E05.0	fEnglish	ashes; candle end
U+3E06.0	fUTF8	㸆
U+3E06.0	fGB	S-215F
U+3E06.0	fR/S	 86.15
U+3E06.0	fTotalStrokes	 19
U+3E06.0	fCangjie	FHGY
U+3E06.0	fCantonese	kaau3
U+3E06.0	fEnglish	 [kao4]
U+3E07.0	fUTF8	㸇
U+3E07.0	fR/S	 86.15
U+3E07.0	fTotalStrokes	 19
U+3E07.0	fCangjie	FQOC
U+3E07.0	fCantonese	zyun3
U+3E08.0	fUTF8	㸈
U+3E08.0	fCNS11643	3-5C5A
U+3E08.0	fR/S	 86.16
U+3E08.0	fTotalStrokes	 20
U+3E08.0	fCangjie	OGF
U+3E08.0	fFourCorner	20800
U+3E08.0	fCantonese	ziu1
U+3E08.0	fMandarin	jiao1
U+3E08.0	fEnglish	same as &#x7126;
U+3E09.0	fUTF8	㸉
U+3E09.0	fCNS11643	3-5A59
U+3E09.0	fR/S	 86.15
U+3E09.0	fTotalStrokes	 19
U+3E09.0	fCangjie	FFF
U+3E09.0	fFourCorner	99800
U+3E09.0	fCantonese	sit3
U+3E09.0	fMandarin	xie4
U+3E09.0	fEnglish	non-classcial form of &#x71EE;
U+3E0A.0	fUTF8	㸊
U+3E0A.0	fCNS11643	3-5C54
U+3E0A.0	fGB	5-5245
U+3E0A.0	fR/S	 86.16
U+3E0A.0	fTotalStrokes	 20
U+3E0A.0	fCangjie	FDLC
U+3E0A.0	fFourCorner	97880
U+3E0A.0	fCantonese	laai6; lai6; lat6
U+3E0A.0	fMandarin	la4; lai4; lie4
U+3E0A.0	fEnglish	[1] fierce of fire [2] cruel; malicious [3] fire; flame
U+3E0B.0	fUTF8	㸋
U+3E0B.0	fCNS11643	3-5C5B
U+3E0B.0	fR/S	 86.16
U+3E0B.0	fTotalStrokes	 20
U+3E0B.0	fCangjie	BFHDW
U+3E0B.0	fFourCorner	22860
U+3E0B.0	fCantonese	faan4
U+3E0B.0	fMandarin	fan2
U+3E0B.0	fEnglish	interchangeable with &#x81B0;
U+3E0C.0	fUTF8	㸌
U+3E0C.0	fCNS11643	4-635F
U+3E0C.0	fR/S	 86.16
U+3E0C.0	fTotalStrokes	 20
U+3E0C.0	fCangjie	FMBG
U+3E0C.0	fFourCorner	91814
U+3E0C.0	fCantonese	fok3
U+3E0C.0	fMandarin	huo4
U+3E0C.0	fEnglish	 twinkle; bright light in glimpses; flashing
U+3E0D.0	fUTF8	㸍
U+3E0D.0	fCNS11643	4-664E
U+3E0D.0	fGB	3-505F
U+3E0D.0	fR/S	 86.17
U+3E0D.0	fTotalStrokes	 21
U+3E0D.0	fCangjie	FYTI
U+3E0D.0	fFourCorner	93850
U+3E0D.0	fCantonese	hei3
U+3E0D.0	fMandarin	xi4
U+3E0D.0	fEnglish	[1] burn [2] roast [3] boil; to heat
U+3E0E.0	fUTF8	㸎
U+3E0E.0	fCNS11643	5-7357
U+3E0E.0	fGB	5-5224
U+3E0E.0	fR/S	 86.18
U+3E0E.0	fTotalStrokes	 22
U+3E0E.0	fCangjie	FSJJ
U+3E0E.0	fFourCorner	91840
U+3E0E.0	fHakka	 ngiap7
U+3E0E.0	fCantonese	nip6; lip6 (also)
U+3E0E.0	fMandarin	nie4
U+3E0E.0	fEnglish	warm; genial
U+3E0F.0	fUTF8	㸏
U+3E0F.0	fCNS11643	4-6A50
U+3E0F.0	fGB	5-7979
U+3E0F.0	fR/S	 86.19
U+3E0F.0	fTotalStrokes	 23
U+3E0F.0	fCangjie	IDLYF
U+3E0F.0	fFourCorner	00280
U+3E0F.0	fHakka	 mi2
U+3E0F.0	fCantonese	mei4
U+3E0F.0	fMandarin	mi2
U+3E0F.0	fEnglish	[1] same as &#x7222; [2] interchangeable with &#x7CDC;
U+3E10.0	fUTF8	㸐
U+3E10.0	fCNS11643	3-603A
U+3E10.0	fGB	5-525C
U+3E10.0	fR/S	 86.19
U+3E10.0	fTotalStrokes	 23
U+3E10.0	fCangjie	TGF
U+3E10.0	fFourCorner	40330
U+3E10.0	fHakka	 nat7
U+3E10.0	fCantonese	jin4; zin4
U+3E10.0	fMandarin	ran2
U+3E10.0	fEnglish	ancient form of &#x7136;
U+3E11.0	fUTF8	㸑
U+3E11.0	fCNS11643	3-617A
U+3E11.0	fGB	5-5256
U+3E11.0	fR/S	 86.22; 86.23
U+3E11.0	fTotalStrokes	 27; 26
U+3E11.0	fCangjie	HBDDF
U+3E11.0	fFourCorner	77800
U+3E11.0	fHakka	 con5
U+3E11.0	fCantonese	cyun3; seon3
U+3E11.0	fMandarin	cun1; cuan4
U+3E11.0	fEnglish	same as &#x7228;
U+3E12.0	fUTF8	㸒
U+3E12.0	fCNS11643	4-272C
U+3E12.0	fGB	5-4E69
U+3E12.0	fR/S	 87.4
U+3E12.0	fTotalStrokes	 8
U+3E12.0	fCangjie	BHG
U+3E12.0	fFourCorner	20100
U+3E12.0	fCantonese	jam4; zaam4
U+3E12.0	fMandarin	yin2; ren2
U+3E12.0	fEnglish	[1] absurd pursuit or desire; wish wildly; desire to long for (usually more than one's rightful share) [2] knit; weave
U+3E13.0	fUTF8	㸓
U+3E13.0	fCNS11643	3-297A
U+3E13.0	fGB	5-4E68
U+3E13.0	fR/S	 87.4
U+3E13.0	fTotalStrokes	 8
U+3E13.0	fCangjie	BA
U+3E13.0	fFourCorner	20600
U+3E13.0	fCantonese	mat6
U+3E13.0	fMandarin	mi4
U+3E13.0	fEnglish	[1] have disappeared [2] avoid [3] not at all [4] not supported by
U+3E14.0	fUTF8	㸔
U+3E14.0	fCNS11643	3-3758
U+3E14.0	fR/S	 87.7
U+3E14.0	fTotalStrokes	 11
U+3E14.0	fCangjie	HFKU
U+3E14.0	fFourCorner	20600
U+3E14.0	fHakka	 kon5; kan5
U+3E14.0	fCantonese	hon3
U+3E14.0	fMandarin	kan4
U+3E15.0	fUTF8	㸕
U+3E15.0	fCNS11643	3-4774
U+3E15.0	fR/S	 87.10
U+3E15.0	fTotalStrokes	 14
U+3E15.0	fCangjie	HOOGE
U+3E15.0	fFourCorner	70214
U+3E15.0	fCantonese	fok3
U+3E15.0	fMandarin	jue2
U+3E15.0	fEnglish	same as &#x652B;
U+3E16.0	fUTF8	㸖
U+3E16.0	fCNS11643	3-2D6A
U+3E16.0	fR/S	 88.5
U+3E16.0	fTotalStrokes	 9
U+3E16.0	fCangjie	CKBM
U+3E16.0	fFourCorner	87410
U+3E16.0	fCantonese	ceoi1; keoi1
U+3E16.0	fMandarin	qu1
U+3E16.0	fEnglish	 hide or keep in store the vegetable in cold winter days
U+3E17.0	fUTF8	㸗
U+3E17.0	fCNS11643	4-2D76
U+3E17.0	fGB	3-4C3A
U+3E17.0	fR/S	 88.6
U+3E17.0	fTotalStrokes	 10
U+3E17.0	fCangjie	CKBMR
U+3E17.0	fFourCorner	80220
U+3E17.0	fCantonese	tung4
U+3E17.0	fMandarin	tong2
U+3E17.0	fEnglish	not a real father or a nature father (as distinct from foster father)
U+3E18.0	fUTF8	㸘
U+3E18.0	fCNS11643	3-3736
U+3E18.0	fR/S	 88.7
U+3E18.0	fTotalStrokes	 11
U+3E18.0	fCangjie	CKWLN
U+3E18.0	fFourCorner	80200
U+3E18.0	fCantonese	maan6; wan6
U+3E18.0	fMandarin	wan4
U+3E18.0	fEnglish	ancient form of &#x842C;
U+3E19.0	fUTF8	㸙
U+3E19.0	fCNS11643	4-386B
U+3E19.0	fGB	3-4C3B
U+3E19.0	fR/S	 88.9; 88.8
U+3E19.0	fTotalStrokes	 12; 13; 11
U+3E19.0	fCangjie	CKJKA
U+3E19.0	fFourCorner	80600
U+3E19.0	fCantonese	ze1
U+3E19.0	fMandarin	zhe1
U+3E19.0	fEnglish	[1] [2] father [3] husband of a wet nurse (addressed in ancient times)
U+3E1A.0	fUTF8	㸚
U+3E1A.0	fCNS11643	3-297B
U+3E1A.0	fGB	5-3033
U+3E1A.0	fR/S	 89.4
U+3E1A.0	fTotalStrokes	 8
U+3E1A.0	fCangjie	KKKK
U+3E1A.0	fFourCorner	44440
U+3E1A.0	fCantonese	lei5
U+3E1A.0	fMandarin	li3; yi3; li4
U+3E1A.0	fEnglish	[1] scattered or dispersed and clear [2] stop [3] detain [4] conncetion [5] lineage
U+3E1B.0	fUTF8	㸛
U+3E1B.0	fCNS11643	4-2A2C
U+3E1B.0	fGB	3-3A3B
U+3E1B.0	fR/S	 90.5
U+3E1B.0	fTotalStrokes	 9
U+3E1B.0	fCangjie	VMSHR
U+3E1B.0	fFourCorner	27260
U+3E1B.0	fCantonese	siu4
U+3E1B.0	fMandarin	shao2; shuo4
U+3E1C.0	fUTF8	㸜
U+3E1C.0	fCNS11643	4-386C
U+3E1C.0	fGB	3-3A3D
U+3E1C.0	fR/S	 90.8
U+3E1C.0	fTotalStrokes	 12
U+3E1C.0	fCangjie	VMJCM
U+3E1C.0	fFourCorner	23210
U+3E1C.0	fCantonese	hung3; hung1
U+3E1C.0	fMandarin	kong4
U+3E1C.0	fEnglish	 get through the space enclosed by a constellation; go out of
U+3E1D.0	fUTF8	㸝
U+3E1D.0	fCNS11643	3-297D
U+3E1D.0	fGB	5-4E54
U+3E1D.0	fR/S	 91.4
U+3E1D.0	fTotalStrokes	 8
U+3E1D.0	fCangjie	LLNO
U+3E1D.0	fFourCorner	27080
U+3E1D.0	fCantonese	him1; hin1
U+3E1D.0	fMandarin	xian1; kan3
U+3E1D.0	fEnglish	same as &#x6774;
U+3E1E.0	fUTF8	㸞
U+3E1E.0	fCNS11643	3-297E
U+3E1E.0	fR/S	 91.4
U+3E1E.0	fTotalStrokes	 8
U+3E1E.0	fCangjie	LLLLL
U+3E1E.0	fFourCorner	22020
U+3E1E.0	fCantonese	zit3; ban2
U+3E1E.0	fMandarin	zhe2; ban3
U+3E1E.0	fEnglish	[1] blocks for printing [2] supporting boards used in building walls
U+3E1F.0	fUTF8	㸟
U+3E1F.0	fCNS11643	5-2E53
U+3E1F.0	fR/S	 91.6
U+3E1F.0	fTotalStrokes	 10
U+3E1F.0	fCangjie	LLPA
U+3E1F.0	fFourCorner	21060
U+3E1F.0	fCantonese	zi1; taai2
U+3E1F.0	fMandarin	zhi1
U+3E1F.0	fEnglish	 cover; close; shut
U+3E20.0	fUTF8	㸠
U+3E20.0	fCNS11643	6-3940
U+3E20.0	fGB	3-4C2E
U+3E20.0	fR/S	 91.6
U+3E20.0	fTotalStrokes	 10; 9
U+3E20.0	fCangjie	LLLMO
U+3E20.0	fFourCorner	22013
U+3E20.0	fCantonese	tiu5
U+3E20.0	fMandarin	tiao3
U+3E20.0	fEnglish	[1] thick stick; stout carrying pole [2] beam of a bed
U+3E21.0	fUTF8	㸡
U+3E21.0	fCNS11643	4-2D7A
U+3E21.0	fGB	3-4C2D
U+3E21.0	fR/S	 91.6
U+3E21.0	fTotalStrokes	 10
U+3E21.0	fCangjie	LLHJD
U+3E21.0	fFourCorner	25090
U+3E21.0	fCantonese	syu4
U+3E21.0	fMandarin	shu1
U+3E21.0	fEnglish	[1] board to block the water current [2] across a river by a closs-beam
U+3E22.0	fUTF8	㸢
U+3E22.0	fCNS11643	4-445F
U+3E22.0	fGB	3-4C31
U+3E22.0	fR/S	 91.10
U+3E22.0	fTotalStrokes	 14
U+3E22.0	fCangjie	LLTHB
U+3E22.0	fFourCorner	24020
U+3E22.0	fCantonese	bei6
U+3E22.0	fMandarin	bei4
U+3E22.0	fEnglish	[1] cross-beam of a bed [2] window [3] mould; pattern
U+3E23.0	fUTF8	㸣
U+3E23.0	fCNS11643	7-392D
U+3E23.0	fR/S	 91.13
U+3E23.0	fTotalStrokes	 17
U+3E23.0	fCangjie	LLTCD
U+3E23.0	fFourCorner	22090
U+3E23.0	fCantonese	jip6; zip6
U+3E23.0	fMandarin	ye4; yi4
U+3E23.0	fEnglish	same as &#x696D;
U+3E24.0	fUTF8	㸤
U+3E24.0	fCNS11643	4-5B64
U+3E24.0	fGB	5-7333
U+3E24.0	fR/S	 91.14
U+3E24.0	fTotalStrokes	 18
U+3E24.0	fCangjie	YJLLJ
U+3E24.0	fFourCorner	00440
U+3E24.0	fCantonese	pin3; pang4
U+3E24.0	fMandarin	pian4
U+3E24.0	fEnglish	broken leather
U+3E25.0	fUTF8	㸥
U+3E25.0	fCNS11643	4-6650
U+3E25.0	fGB	5-4E5B
U+3E25.0	fR/S	 91.17
U+3E25.0	fTotalStrokes	 21
U+3E25.0	fCangjie	LLNRI
U+3E25.0	fFourCorner	27010
U+3E25.0	fCantonese	zaam6; cam4
U+3E25.0	fMandarin	chan2
U+3E25.0	fEnglish	[1] household registers [2] printing plate [3] supporting boards used in building walls [4] same as &#x6B03;
U+3E26.0	fUTF8	㸦
U+3E26.0	fCNS11643	3-216A
U+3E26.0	fR/S	 92.0
U+3E26.0	fTotalStrokes	 4
U+3E26.0	fCangjie	MVDO
U+3E26.0	fFourCorner	71500
U+3E26.0	fCantonese	wu6; wui1
U+3E26.0	fMandarin	hu4
U+3E26.0	fEnglish	non-classical form same as &#x4E92;
U+3E27.0	fUTF8	㸧
U+3E27.0	fCNS11643	3-3233
U+3E27.0	fR/S	 92.6
U+3E27.0	fTotalStrokes	 10
U+3E27.0	fCangjie	MHAV
U+3E27.0	fFourCorner	17230
U+3E27.0	fCantonese	ngan4; an4 (also); haan2
U+3E27.0	fMandarin	qian3; ken4; yin2
U+3E27.0	fEnglish	[1] interchangeable with &#x5543; [2] standard form of &#x9F66;
U+3E28.0	fUTF8	㸨
U+3E28.0	fCNS11643	4-2326
U+3E28.0	fGB	3-4874
U+3E28.0	fR/S	 93.2
U+3E28.0	fTotalStrokes	 6
U+3E28.0	fCangjie	HQVL
U+3E28.0	fFourCorner	22500
U+3E28.0	fCantonese	kau1; gaau1; kaau1
U+3E28.0	fMandarin	jiu4; jiu1
U+3E28.0	fEnglish	 big bull
U+3E29.0	fUTF8	㸩
U+3E29.0	fCNS11643	3-2665
U+3E29.0	fR/S	 93.3
U+3E29.0	fTotalStrokes	 7
U+3E29.0	fCangjie	HQMJ
U+3E29.0	fFourCorner	21540
U+3E29.0	fCantonese	ngon4; on4 (also); ngon6
U+3E29.0	fMandarin	an2; an4
U+3E29.0	fEnglish	 desist an ox (&#x6B62;&#x725B;)
U+3E2A.0	fUTF8	㸪
U+3E2A.0	fCNS11643	4-2455
U+3E2A.0	fGB	3-4875
U+3E2A.0	fR/S	 93.3
U+3E2A.0	fTotalStrokes	 7
U+3E2A.0	fCangjie	HQLLL
U+3E2A.0	fFourCorner	22500
U+3E2A.0	fCantonese	seon4; maan5
U+3E2A.0	fMandarin	xun4; chun2; xun2
U+3E2A.0	fEnglish	an ox walking slowly
U+3E2B.0	fUTF8	㸫
U+3E2B.0	fCNS11643	6-2C71
U+3E2B.0	fR/S	 93.4
U+3E2B.0	fTotalStrokes	 8
U+3E2B.0	fCangjie	HQHML
U+3E2B.0	fFourCorner	22520
U+3E2B.0	fCantonese	gin1
U+3E2B.0	fMandarin	ke1; qian2
U+3E2B.0	fEnglish	same as &#x728D;
U+3E2C.0	fUTF8	㸬
U+3E2C.0	fCNS11643	4-2733
U+3E2C.0	fGB	3-4878
U+3E2C.0	fR/S	 93.4; 93.5
U+3E2C.0	fTotalStrokes	 8; 9
U+3E2C.0	fCangjie	HQJB
U+3E2C.0	fFourCorner	20520
U+3E2C.0	fHakka	 bui5; bi5
U+3E2C.0	fCantonese	bui3
U+3E2C.0	fMandarin	bei4; fei4; pei4
U+3E2C.0	fEnglish	a two-year old heifer
U+3E2D.0	fUTF8	㸭
U+3E2D.0	fCNS11643	5-2677
U+3E2D.0	fR/S	 93.4
U+3E2D.0	fTotalStrokes	 8
U+3E2D.0	fCangjie	HQAU
U+3E2D.0	fFourCorner	27510
U+3E2D.0	fCantonese	baa1; paa4
U+3E2D.0	fMandarin	ba1
U+3E2D.0	fEnglish	the opposite; contrary of the cattle's horns
U+3E2E.0	fUTF8	㸮
U+3E2E.0	fCNS11643	4-2732
U+3E2E.0	fGB	3-487C
U+3E2E.0	fR/S	 93.4
U+3E2E.0	fTotalStrokes	 8
U+3E2E.0	fCangjie	HQCSH
U+3E2E.0	fFourCorner	28520
U+3E2E.0	fCantonese	fan4
U+3E2E.0	fMandarin	fen2
U+3E2E.0	fEnglish	[1] bull; bulllock; ox (castrated) [2] a four years old ox
U+3E2F.0	fUTF8	㸯
U+3E2F.0	fCNS11643	4-272F
U+3E2F.0	fGB	3-487E
U+3E2F.0	fR/S	 93.4
U+3E2F.0	fTotalStrokes	 8
U+3E2F.0	fCangjie	HQYJ
U+3E2F.0	fFourCorner	24500
U+3E2F.0	fCantonese	fo1; gaau1
U+3E2F.0	fMandarin	ke1; jiu1
U+3E2F.0	fEnglish	[1] a fine breed of cattle (cow, bull or ox) [2] hornless cattle
U+3E30.0	fUTF8	㸰
U+3E30.0	fCNS11643	3-2D6C
U+3E30.0	fGB	5-4B26
U+3E30.0	fR/S	 93.5
U+3E30.0	fTotalStrokes	 9
U+3E30.0	fCangjie	HQJP
U+3E30.0	fFourCorner	23510
U+3E30.0	fHakka	 tu2
U+3E30.0	fCantonese	fo1; to4
U+3E30.0	fMandarin	tuo2
U+3E30.0	fEnglish	same as &#x7290;
U+3E31.0	fUTF8	㸱
U+3E31.0	fCNS11643	3-2D6F
U+3E31.0	fGB	5-4B24
U+3E31.0	fR/S	 93.5
U+3E31.0	fTotalStrokes	 9
U+3E31.0	fCangjie	HQOPD
U+3E31.0	fFourCorner	28510
U+3E31.0	fCantonese	fo1; to4
U+3E31.0	fMandarin	tuo2
U+3E31.0	fEnglish	same as &#x3E30;, &#x7290;
U+3E32.0	fUTF8	㸲
U+3E32.0	fCNS11643	4-2A2E
U+3E32.0	fR/S	 93.5
U+3E32.0	fTotalStrokes	 9
U+3E32.0	fCangjie	HQOS
U+3E32.0	fFourCorner	28510
U+3E32.0	fCantonese	zok6; zok3
U+3E32.0	fMandarin	zuo4; zuo2
U+3E32.0	fEnglish	a kind of mountain cattle, big and heavy
U+3E33.0	fUTF8	㸳
U+3E33.0	fCNS11643	3-2D6E
U+3E33.0	fR/S	 93.5
U+3E33.0	fTotalStrokes	 9
U+3E33.0	fCangjie	HQOII
U+3E33.0	fFourCorner	28530
U+3E33.0	fCantonese	ling4
U+3E33.0	fMandarin	ling2
U+3E33.0	fEnglish	a kind of cattle
U+3E34.0	fUTF8	㸴
U+3E34.0	fR/S	 93.5
U+3E34.0	fTotalStrokes	 9
U+3E34.0	fCangjie	MRHQ
U+3E34.0	fCantonese	sek6
U+3E34.0	fEnglish	[Korean character]
U+3E35.0	fUTF8	㸵
U+3E35.0	fCNS11643	3-3234
U+3E35.0	fGB	5-4B28
U+3E35.0	fR/S	 93.6
U+3E35.0	fTotalStrokes	 10
U+3E35.0	fCangjie	HQGR
U+3E35.0	fFourCorner	24560
U+3E35.0	fCantonese	gwai2
U+3E35.0	fMandarin	gui3; wei3
U+3E35.0	fEnglish	the lowing of an ox
U+3E36.0	fUTF8	㸶
U+3E36.0	fCNS11643	3-3235
U+3E36.0	fR/S	 93.6
U+3E36.0	fTotalStrokes	 10
U+3E36.0	fCangjie	HQWK
U+3E36.0	fFourCorner	26500
U+3E36.0	fCantonese	jin1; zin1; zin4
U+3E36.0	fMandarin	yan1
U+3E36.0	fEnglish	color of the oxtail; a flat yellow
U+3E37.0	fUTF8	㸷
U+3E37.0	fCNS11643	4-2D7D
U+3E37.0	fGB	5-4B27
U+3E37.0	fR/S	 93.6
U+3E37.0	fTotalStrokes	 10
U+3E37.0	fCangjie	QHHQ
U+3E37.0	fFourCorner	57500
U+3E37.0	fCantonese	sai6; si6
U+3E37.0	fMandarin	shi4
U+3E37.0	fEnglish	same as &#x89E2;
U+3E38.0	fUTF8	㸸
U+3E38.0	fCNS11643	4-2D7C
U+3E38.0	fGB	3-4926
U+3E38.0	fR/S	 93.6
U+3E38.0	fTotalStrokes	 10
U+3E38.0	fCangjie	HQHMR
U+3E38.0	fFourCorner	22560
U+3E38.0	fHakka	 k'ieu5; k'ieu3; keu5; keu3
U+3E38.0	fCantonese	hau6; haau3
U+3E38.0	fMandarin	ou3; hou3; kou3
U+3E38.0	fEnglish	[1] calf [2] bull; bullock; ox (castrated) [3] domesticated animals
U+3E39.0	fUTF8	㸹
U+3E39.0	fCNS11643	4-327A
U+3E39.0	fGB	3-492B
U+3E39.0	fR/S	 93.7
U+3E39.0	fTotalStrokes	 11
U+3E39.0	fCangjie	HQBDI
U+3E39.0	fFourCorner	22540
U+3E39.0	fCantonese	lyut6; lyut3
U+3E39.0	fMandarin	lie4; le4
U+3E39.0	fEnglish	[1] cattle with white stripe on the back [2] variegated; parti-colored [3] mixed; impure
U+3E3A.0	fUTF8	㸺
U+3E3A.0	fCNS11643	5-334D
U+3E3A.0	fR/S	 93.7
U+3E3A.0	fTotalStrokes	 11
U+3E3A.0	fCangjie	EHHQ
U+3E3A.0	fFourCorner	39500
U+3E3A.0	fHakka	 sa1
U+3E3A.0	fCantonese	saa1
U+3E3A.0	fMandarin	sha1
U+3E3A.0	fEnglish	a alrge-size buffalo
U+3E3B.0	fUTF8	㸻
U+3E3B.0	fCNS11643	4-3279
U+3E3B.0	fGB	3-492D
U+3E3B.0	fR/S	 93.7
U+3E3B.0	fTotalStrokes	 11
U+3E3B.0	fCangjie	HQIOK
U+3E3B.0	fFourCorner	23530
U+3E3B.0	fCantonese	zi6
U+3E3B.0	fMandarin	si4
U+3E3B.0	fEnglish	[1] a kind of cattle [2] an one year old cattle
U+3E3C.0	fUTF8	㸼
U+3E3C.0	fCNS11643	4-327E
U+3E3C.0	fGB	3-492C
U+3E3C.0	fR/S	 93.7
U+3E3C.0	fTotalStrokes	 11
U+3E3C.0	fCangjie	HQHEJ
U+3E3C.0	fFourCorner	27550
U+3E3C.0	fCantonese	fung1
U+3E3C.0	fMandarin	feng1
U+3E3C.0	fEnglish	same as &#x728E;
U+3E3D.0	fUTF8	㸽
U+3E3D.0	fCNS11643	3-3738
U+3E3D.0	fGB	5-4A7D
U+3E3D.0	fR/S	 93.7
U+3E3D.0	fTotalStrokes	 11
U+3E3D.0	fCangjie	HQBUC
U+3E3D.0	fFourCorner	26580
U+3E3D.0	fHakka	 bui5; boi5; bi5
U+3E3D.0	fCantonese	bui3
U+3E3D.0	fMandarin	bei4
U+3E3D.0	fEnglish	a two-year old heifer
U+3E3E.0	fUTF8	㸾
U+3E3E.0	fCNS11643	6-4166
U+3E3E.0	fR/S	 93.7
U+3E3E.0	fTotalStrokes	 11
U+3E3E.0	fCangjie	HQSIP
U+3E3E.0	fFourCorner	27530
U+3E3E.0	fCantonese	jan6; zaan6
U+3E3E.0	fMandarin	ren4
U+3E3E.0	fEnglish	[1] same as &#x7263; [2] interchangeable &#x97CC;
U+3E3F.0	fUTF8	㸿
U+3E3F.0	fR/S	 93.7
U+3E3F.0	fTotalStrokes	 11
U+3E3F.0	fCangjie	HQGBN
U+3E3F.0	fCantonese	duk6
U+3E3F.0	fMandarin	du2
U+3E3F.0	fEnglish	abbreviated form of &#x72A2;
U+3E40.0	fUTF8	㹀
U+3E40.0	fCNS11643	3-3737
U+3E40.0	fGB	S-2425
U+3E40.0	fR/S	 93.7
U+3E40.0	fTotalStrokes	 11
U+3E40.0	fCangjie	HQJBD
U+3E40.0	fFourCorner	24540
U+3E40.0	fCantonese	bui6; but6
U+3E40.0	fMandarin	zi4; bo2
U+3E40.0	fEnglish	[1] cow [2] female of an animal
U+3E41.0	fUTF8	㹁
U+3E41.0	fCNS11643	4-3870
U+3E41.0	fGB	5-4B33
U+3E41.0	fR/S	 93.8
U+3E41.0	fTotalStrokes	 12
U+3E41.0	fCangjie	HQYRF
U+3E41.0	fFourCorner	20590
U+3E41.0	fCantonese	loeng4
U+3E41.0	fMandarin	liang2; liang4
U+3E41.0	fEnglish	generally called the animals (cattle, sheep etc.) with mixed color of white and black
U+3E42.0	fUTF8	㹂
U+3E42.0	fCNS11643	4-3872
U+3E42.0	fGB	3-4879
U+3E42.0	fR/S	 93.8
U+3E42.0	fTotalStrokes	 12
U+3E42.0	fCangjie	SEHQ
U+3E42.0	fFourCorner	77500
U+3E42.0	fHakka	 kien3; ken3
U+3E42.0	fCantonese	jin4; hin2
U+3E42.0	fMandarin	qian3; ci4
U+3E42.0	fEnglish	an obstinate ox; pig-headed
U+3E43.0	fUTF8	㹃
U+3E43.0	fCNS11643	4-3873
U+3E43.0	fGB	3-745C
U+3E43.0	fR/S	 93.8
U+3E43.0	fTotalStrokes	 12
U+3E43.0	fCangjie	LYHQ
U+3E43.0	fFourCorner	11500
U+3E43.0	fCantonese	he3; fe3; fei2
U+3E43.0	fMandarin	fei4; bi4; fei3
U+3E43.0	fEnglish	[1] two brindled cows to plough face to each other [2] cultivate; plant after the second time of plughing
U+3E44.0	fUTF8	㹄
U+3E44.0	fCNS11643	4-4463
U+3E44.0	fGB	3-4939
U+3E44.0	fR/S	 93.10
U+3E44.0	fTotalStrokes	 14
U+3E44.0	fCangjie	HQWCE
U+3E44.0	fFourCorner	26540
U+3E44.0	fCantonese	zik1
U+3E44.0	fMandarin	ji2; ji4; qi4
U+3E44.0	fEnglish	a kind of cattle; a cattle of short statured
U+3E45.0	fUTF8	㹅
U+3E45.0	fCNS11643	6-5478
U+3E45.0	fR/S	 93.9
U+3E45.0	fTotalStrokes	 13
U+3E45.0	fCangjie	HQPKP
U+3E45.0	fFourCorner	27530
U+3E45.0	fCantonese	zung2
U+3E45.0	fMandarin	zong3
U+3E45.0	fEnglish	same as &#x7E3D;
U+3E46.0	fUTF8	㹆
U+3E46.0	fCNS11643	4-3E4E
U+3E46.0	fGB	3-492A
U+3E46.0	fR/S	 93.9
U+3E46.0	fTotalStrokes	 13
U+3E46.0	fCangjie	HQBJJ
U+3E46.0	fFourCorner	27550
U+3E46.0	fCantonese	fai1
U+3E46.0	fMandarin	hui1
U+3E46.0	fEnglish	[1] the coulter of a plough; head of a brindled cow [2] a kind of cattle
U+3E47.0	fUTF8	㹇
U+3E47.0	fCNS11643	3-425E
U+3E47.0	fGB	5-4B36
U+3E47.0	fR/S	 93.9
U+3E47.0	fTotalStrokes	 13
U+3E47.0	fCangjie	HQAPV
U+3E47.0	fFourCorner	26520
U+3E47.0	fCantonese	hot6; gin1; hok6; hot3
U+3E47.0	fMandarin	jian1; he2
U+3E47.0	fEnglish	same as standard form &#x728D;
U+3E48.0	fUTF8	㹈
U+3E48.0	fGB	S-2438
U+3E48.0	fR/S	 93.9
U+3E48.0	fTotalStrokes	 13
U+3E48.0	fCangjie	JKHQ
U+3E48.0	fCantonese	mou4; lei4
U+3E48.0	fMandarin	li2; mao2
U+3E48.0	fEnglish	[1] same as &#x81A7; [2] same as &#x729B;
U+3E49.0	fUTF8	㹉
U+3E49.0	fCNS11643	4-4462
U+3E49.0	fGB	3-4938
U+3E49.0	fR/S	 93.10
U+3E49.0	fTotalStrokes	 14
U+3E49.0	fCangjie	HQMHF
U+3E49.0	fFourCorner	21590
U+3E49.0	fCantonese	jyun4; zyun4
U+3E49.0	fMandarin	yuan2; wan2
U+3E49.0	fEnglish	[1] a legendary animal (some kind looks like a cow) with three feet [2] a wild ox; the bison
U+3E4A.0	fUTF8	㹊
U+3E4A.0	fCNS11643	4-4460
U+3E4A.0	fGB	3-493C
U+3E4A.0	fR/S	 93.10
U+3E4A.0	fTotalStrokes	 14
U+3E4A.0	fCangjie	HQOBG
U+3E4A.0	fFourCorner	24510
U+3E4A.0	fCantonese	ngok6; ok6 (also)
U+3E4A.0	fMandarin	yue4; lue4; hu2
U+3E4A.0	fEnglish	a cattle of white color
U+3E4B.0	fUTF8	㹋
U+3E4B.0	fCNS11643	4-4464
U+3E4B.0	fGB	3-4935
U+3E4B.0	fR/S	 93.10
U+3E4B.0	fTotalStrokes	 14
U+3E4B.0	fCangjie	HQOLH
U+3E4B.0	fFourCorner	28520
U+3E4B.0	fCantonese	sau1; saau1; saau2
U+3E4B.0	fMandarin	xiu1
U+3E4B.0	fEnglish	[1] cow; ox; bull; cattle [2] a cow with no tail
U+3E4C.0	fUTF8	㹌
U+3E4C.0	fCNS11643	4-4A6F
U+3E4C.0	fGB	3-4928
U+3E4C.0	fR/S	 93.11
U+3E4C.0	fTotalStrokes	 15
U+3E4C.0	fCangjie	HQYHM
U+3E4C.0	fFourCorner	20510
U+3E4C.0	fHakka	 san3
U+3E4C.0	fCantonese	caan2
U+3E4C.0	fMandarin	chan3; sheng4
U+3E4C.0	fEnglish	 breed domestic animals
U+3E4D.0	fUTF8	㹍
U+3E4D.0	fCNS11643	4-4A6E
U+3E4D.0	fGB	5-4B39
U+3E4D.0	fR/S	 93.11
U+3E4D.0	fTotalStrokes	 15
U+3E4D.0	fCangjie	HQYCB
U+3E4D.0	fFourCorner	20520
U+3E4D.0	fCantonese	dik6
U+3E4D.0	fMandarin	di2; du2
U+3E4D.0	fEnglish	bull; bullock; ox (castrated)
U+3E4E.0	fUTF8	㹎
U+3E4E.0	fCNS11643	4-4A72
U+3E4E.0	fGB	3-493E
U+3E4E.0	fR/S	 93.11
U+3E4E.0	fTotalStrokes	 15
U+3E4E.0	fCangjie	HQWVF
U+3E4E.0	fFourCorner	26590
U+3E4E.0	fCantonese	leoi4
U+3E4E.0	fMandarin	lei2
U+3E4E.0	fEnglish	bull; bullock; ox (castrated), &#x6C42;&#x5B50;&#x725B;
U+3E4F.0	fUTF8	㹏
U+3E4F.0	fCNS11643	4-4A71
U+3E4F.0	fGB	3-493D
U+3E4F.0	fR/S	 93.11
U+3E4F.0	fTotalStrokes	 15; 14
U+3E4F.0	fCangjie	HQTLM
U+3E4F.0	fFourCorner	24510
U+3E4F.0	fCantonese	gan2; gaan2; hong4
U+3E4F.0	fMandarin	jin3
U+3E4F.0	fEnglish	[1] good; virtuous [2] a tamed, gental, and yielding cow
U+3E50.0	fUTF8	㹐
U+3E50.0	fCNS11643	4-4A73
U+3E50.0	fGB	5-4B21
U+3E50.0	fR/S	 93.11
U+3E50.0	fTotalStrokes	 15
U+3E50.0	fCangjie	HOHQ
U+3E50.0	fFourCorner	28500
U+3E50.0	fCantonese	sung4; cung4
U+3E50.0	fMandarin	chong2; zhou4
U+3E50.0	fEnglish	 tow a boat against the current in swift stream from the bank; track a boat in the shallow area
U+3E51.0	fUTF8	㹑
U+3E51.0	fCNS11643	4-512E
U+3E51.0	fGB	3-4932
U+3E51.0	fR/S	 93.12
U+3E51.0	fTotalStrokes	 16
U+3E51.0	fCangjie	HQIPC
U+3E51.0	fFourCorner	23540
U+3E51.0	fCantonese	sei3; ci4; gaai1; si3; taai4
U+3E51.0	fMandarin	si4; yi2
U+3E51.0	fEnglish	same as large seal type of &#x726D
U+3E52.0	fUTF8	㹒
U+3E52.0	fCNS11643	4-5130
U+3E52.0	fGB	3-493F
U+3E52.0	fR/S	 93.12
U+3E52.0	fTotalStrokes	 16
U+3E52.0	fCangjie	HQTCO
U+3E52.0	fFourCorner	22530
U+3E52.0	fCantonese	pok3; pak6; pei1
U+3E52.0	fMandarin	pu2; pu3
U+3E52.0	fEnglish	[1] bull; bullock [2] calf
U+3E53.0	fUTF8	㹓
U+3E53.0	fCNS11643	4-512F
U+3E53.0	fGB	3-4925
U+3E53.0	fR/S	 93.12
U+3E53.0	fTotalStrokes	 16
U+3E53.0	fCangjie	HQGGU
U+3E53.0	fFourCorner	24510
U+3E53.0	fCantonese	jiu5; zi6
U+3E53.0	fMandarin	yao4; yao3; yi4
U+3E53.0	fEnglish	[1] a kind of animal [2] cattles and horses to prance; be lively
U+3E54.0	fUTF8	㹔
U+3E54.0	fCNS11643	4-5667
U+3E54.0	fGB	3-4940
U+3E54.0	fR/S	 93.13
U+3E54.0	fTotalStrokes	 17; 16
U+3E54.0	fCangjie	HQMWM
U+3E54.0	fFourCorner	21510
U+3E54.0	fCantonese	goeng1
U+3E54.0	fMandarin	jiang1
U+3E54.0	fEnglish	[1] cattle with long back [2] a white cattle; a cattle having white color on the back
U+3E55.0	fUTF8	㹕
U+3E55.0	fCNS11643	3-5546
U+3E55.0	fR/S	 93.13
U+3E55.0	fTotalStrokes	 17
U+3E55.0	fCangjie	MCHQ
U+3E55.0	fFourCorner	11500
U+3E55.0	fCantonese	waan4; fun1
U+3E55.0	fMandarin	huan1
U+3E55.0	fEnglish	[1] inferior; mean [2] minor
U+3E56.0	fUTF8	㹖
U+3E56.0	fCNS11643	3-5547
U+3E56.0	fGB	5-4B3E
U+3E56.0	fR/S	 93.13
U+3E56.0	fTotalStrokes	 17
U+3E56.0	fCangjie	HQFQO
U+3E56.0	fFourCorner	29530
U+3E56.0	fCantonese	waan6
U+3E56.0	fMandarin	huan4
U+3E56.0	fEnglish	same as &#x8C62;
U+3E57.0	fUTF8	㹗
U+3E57.0	fCNS11643	4-5B66
U+3E57.0	fGB	5-4B2C
U+3E57.0	fR/S	 93.14
U+3E57.0	fTotalStrokes	 18
U+3E57.0	fCangjie	HQGNI
U+3E57.0	fFourCorner	24540
U+3E57.0	fCantonese	tou1
U+3E57.0	fMandarin	chai1; tao2; dai1; chao1; qiu3; tao1
U+3E57.0	fEnglish	cattle and sheep; have no children
U+3E58.0	fUTF8	㹘
U+3E58.0	fCNS11643	4-5B65
U+3E58.0	fGB	3-4941
U+3E58.0	fR/S	 93.14
U+3E58.0	fTotalStrokes	 18
U+3E58.0	fCangjie	HQMBB
U+3E58.0	fFourCorner	21520
U+3E58.0	fCantonese	jyu6; zyu4; zyu6
U+3E58.0	fMandarin	ru4; ru2; rui2
U+3E58.0	fEnglish	[1] shinbone of a cattle [2] a kind of cattle
U+3E59.0	fUTF8	㹙
U+3E59.0	fCNS11643	4-6024
U+3E59.0	fGB	3-4937
U+3E59.0	fR/S	 93.15
U+3E59.0	fTotalStrokes	 19
U+3E59.0	fCangjie	HQFFU
U+3E59.0	fFourCorner	29560
U+3E59.0	fCantonese	wang2; zing4
U+3E59.0	fMandarin	weng3; ying2
U+3E59.0	fEnglish	[1] the sound of calling to a calf [2] calf [3] the lowing of an ox
U+3E5A.0	fUTF8	㹚
U+3E5A.0	fCNS11643	7-536B
U+3E5A.0	fR/S	 93.17
U+3E5A.0	fTotalStrokes	 21
U+3E5A.0	fCangjie	HQFFR
U+3E5A.0	fFourCorner	29560
U+3E5A.0	fCantonese	wang2; zing4
U+3E5A.0	fMandarin	ying3; ying2
U+3E5A.0	fEnglish	same as &#x3E59;
U+3E5B.0	fUTF8	㹛
U+3E5B.0	fCNS11643	4-6A51
U+3E5B.0	fGB	5-4B40
U+3E5B.0	fR/S	 93.19; 93.18
U+3E5B.0	fTotalStrokes	 23; 22
U+3E5B.0	fCangjie	HQMCE
U+3E5B.0	fFourCorner	21547
U+3E5B.0	fCantonese	jiu4; ziu2
U+3E5B.0	fMandarin	rao3; rao2
U+3E5B.0	fEnglish	[1] a tamed, gental, and yielding cow [2] good; virtuous
U+3E5C.0	fUTF8	㹜
U+3E5C.0	fCNS11643	4-2734
U+3E5C.0	fGB	5-4646
U+3E5C.0	fR/S	 94.4
U+3E5C.0	fTotalStrokes	 8
U+3E5C.0	fCangjie	IKIK
U+3E5C.0	fFourCorner	43030
U+3E5C.0	fCantonese	ngan4; an4 (also); ngaan4
U+3E5C.0	fMandarin	yin2
U+3E5C.0	fEnglish	same as &#x72BE;
U+3E5D.0	fUTF8	㹝
U+3E5D.0	fCNS11643	4-2459
U+3E5D.0	fGB	3-4372
U+3E5D.0	fR/S	 94.4
U+3E5D.0	fTotalStrokes	 7
U+3E5D.0	fCangjie	KHHVP
U+3E5D.0	fFourCorner	42240
U+3E5D.0	fCantonese	si6
U+3E5D.0	fMandarin	shi4
U+3E5D.0	fEnglish	[1] a legendary wild animal [2] snake; serpent
U+3E5E.0	fUTF8	㹞
U+3E5E.0	fCNS11643	3-266D
U+3E5E.0	fGB	5-4653
U+3E5E.0	fR/S	 94.4
U+3E5E.0	fTotalStrokes	 7
U+3E5E.0	fCangjie	KHHML
U+3E5E.0	fFourCorner	42220
U+3E5E.0	fCantonese	ngan4; an4 (also); ngaan4
U+3E5E.0	fMandarin	yin2; yin3; ya2
U+3E5E.0	fEnglish	[1] same as &#x72FA; [2] same as &#x9F57; [3] same as &#x5540;
U+3E5F.0	fUTF8	㹟
U+3E5F.0	fCNS11643	4-2456
U+3E5F.0	fGB	3-4376
U+3E5F.0	fR/S	 94.4
U+3E5F.0	fTotalStrokes	 7
U+3E5F.0	fCangjie	KHDK
U+3E5F.0	fFourCorner	45237
U+3E5F.0	fCantonese	kyut3; kui2
U+3E5F.0	fMandarin	jue2; guai4
U+3E5F.0	fEnglish	[1] a kind of animal [2] the animals to run about wildly [3] wild; mad; crazy [4] same as &#x736A;
U+3E60.0	fUTF8	㹠
U+3E60.0	fCNS11643	3-266C
U+3E60.0	fGB	5-4651
U+3E60.0	fR/S	 94.4
U+3E60.0	fTotalStrokes	 7
U+3E60.0	fCangjie	KHPU
U+3E60.0	fFourCorner	45210
U+3E60.0	fHakka	 tun2
U+3E60.0	fCantonese	tyun4; din6
U+3E60.0	fMandarin	tun2
U+3E60.0	fEnglish	same as &#x8C5A;
U+3E61.0	fUTF8	㹡
U+3E61.0	fCNS11643	6-2C75
U+3E61.0	fR/S	 94.5
U+3E61.0	fTotalStrokes	 8
U+3E61.0	fCangjie	KHYVI
U+3E61.0	fFourCorner	40230
U+3E61.0	fCantonese	jyun4; zyun4
U+3E61.0	fMandarin	xuan2; xuan4
U+3E61.0	fEnglish	[1] a rush and impatient disposition [2] non-classical form of &#x72F7;
U+3E62.0	fUTF8	㹢
U+3E62.0	fCNS11643	4-2739
U+3E62.0	fGB	3-4424
U+3E62.0	fR/S	 94.5
U+3E62.0	fTotalStrokes	 8
U+3E62.0	fCangjie	KHKSR
U+3E62.0	fFourCorner	46200
U+3E62.0	fCantonese	gaa1; go1
U+3E62.0	fMandarin	jia1
U+3E62.0	fEnglish	[1] a large ape found in Western China [2] a precious mammal found in Africa
U+3E63.0	fUTF8	㹣
U+3E63.0	fCNS11643	5-272E
U+3E63.0	fR/S	 94.5
U+3E63.0	fTotalStrokes	 8
U+3E63.0	fCangjie	KHHEY
U+3E63.0	fFourCorner	47230
U+3E63.0	fCantonese	zung1
U+3E63.0	fMandarin	zhong1
U+3E63.0	fEnglish	a kind of dog
U+3E64.0	fUTF8	㹤
U+3E64.0	fCNS11643	3-2A29
U+3E64.0	fGB	5-4657
U+3E64.0	fR/S	 94.5
U+3E64.0	fTotalStrokes	 8
U+3E64.0	fCangjie	KHGI
U+3E64.0	fFourCorner	44290
U+3E64.0	fHakka	 kiap7
U+3E64.0	fCantonese	hip3
U+3E64.0	fMandarin	que4; qie4
U+3E64.0	fEnglish	same as &#x602F;
U+3E65.0	fUTF8	㹥
U+3E65.0	fCNS11643	3-2A24
U+3E65.0	fGB	5-465E
U+3E65.0	fR/S	 94.5
U+3E65.0	fTotalStrokes	 8
U+3E65.0	fCangjie	KHYG
U+3E65.0	fFourCorner	40210
U+3E65.0	fCantonese	zyu3
U+3E65.0	fMandarin	zhu4
U+3E65.0	fEnglish	a yellow dog with black head
U+3E66.0	fUTF8	㹦
U+3E66.0	fCNS11643	3-2A25
U+3E66.0	fGB	5-4660
U+3E66.0	fR/S	 94.5
U+3E66.0	fTotalStrokes	 8
U+3E66.0	fCangjie	KHSHR
U+3E66.0	fFourCorner	47260
U+3E66.0	fCantonese	diu1; ciu1
U+3E66.0	fMandarin	diao1
U+3E66.0	fEnglish	[1] dog with a short tail [2] same as non-classical form &#x8C82;
U+3E67.0	fUTF8	㹧
U+3E67.0	fCNS11643	4-273A
U+3E67.0	fGB	3-437C
U+3E67.0	fR/S	 94.5
U+3E67.0	fTotalStrokes	 8
U+3E67.0	fCangjie	KHLBK
U+3E67.0	fFourCorner	45230
U+3E67.0	fCantonese	ong1; ngong1 (also); zoeng1
U+3E67.0	fMandarin	yang1
U+3E67.0	fEnglish	a small animal
U+3E68.0	fUTF8	㹨
U+3E68.0	fR/S	 94.5
U+3E68.0	fTotalStrokes	 8
U+3E68.0	fCangjie	KHLW
U+3E68.0	fFourCorner	4526
U+3E68.0	fCantonese	jau6; jaau6; zaau6
U+3E68.0	fMandarin	you4
U+3E68.0	fEnglish	[1] same as &#x8C81; [2] non-classical form of &#x72D6;
U+3E69.0	fUTF8	㹩
U+3E69.0	fGB	S-225E
U+3E69.0	fR/S	 94.5
U+3E69.0	fTotalStrokes	 8
U+3E69.0	fCangjie	KHEG
U+3E69.0	fCantonese	hing5
U+3E6A.0	fUTF8	㹪
U+3E6A.0	fGB	3-4421
U+3E6A.0	fR/S	 94.5
U+3E6A.0	fTotalStrokes	 8
U+3E6A.0	fCangjie	KHBBM
U+3E6A.0	fCantonese	saan1; san1
U+3E6A.0	fEnglish	same as &#x72E6;
U+3E6B.0	fUTF8	㹫
U+3E6B.0	fCNS11643	4-2A32
U+3E6B.0	fGB	3-4429
U+3E6B.0	fR/S	 94.6
U+3E6B.0	fTotalStrokes	 9
U+3E6B.0	fCangjie	KHKN
U+3E6B.0	fFourCorner	45230
U+3E6B.0	fCantonese	ji4; taai4; zi4
U+3E6B.0	fMandarin	yi2; xi4
U+3E6B.0	fEnglish	[1] a kind of animal [2] same as &#x5937;
U+3E6C.0	fUTF8	㹬
U+3E6C.0	fCNS11643	4-2A34
U+3E6C.0	fGB	3-4428
U+3E6C.0	fR/S	 94.6
U+3E6C.0	fTotalStrokes	 9
U+3E6C.0	fCangjie	KHJLK
U+3E6C.0	fFourCorner	45240
U+3E6C.0	fCantonese	si2
U+3E6C.0	fMandarin	shi3
U+3E6C.0	fEnglish	a kind of animal (of dog tribe)
U+3E6D.0	fUTF8	㹭
U+3E6D.0	fCNS11643	4-2A35
U+3E6D.0	fGB	3-442A
U+3E6D.0	fR/S	 94.6
U+3E6D.0	fTotalStrokes	 9
U+3E6D.0	fCangjie	KHLWP
U+3E6D.0	fFourCorner	45200
U+3E6D.0	fCantonese	jai6; zaai6
U+3E6D.0	fMandarin	yi4
U+3E6D.0	fEnglish	[1] a kind of monkey [2] fox
U+3E6E.0	fUTF8	㹮
U+3E6E.0	fCNS11643	3-2D72
U+3E6E.0	fGB	5-4663
U+3E6E.0	fR/S	 94.6
U+3E6E.0	fTotalStrokes	 9
U+3E6E.0	fCangjie	KHMA
U+3E6E.0	fFourCorner	41260
U+3E6E.0	fCantonese	mak6; mok6; mou6
U+3E6E.0	fMandarin	mo4
U+3E6E.0	fEnglish	[1] a kind of animal (of mule tribe) [2] same as &#x8C98;
U+3E6F.0	fUTF8	㹯
U+3E6F.0	fCNS11643	6-3267
U+3E6F.0	fR/S	 94.6
U+3E6F.0	fTotalStrokes	 9
U+3E6F.0	fCangjie	KHOD
U+3E6F.0	fFourCorner	44290
U+3E6F.0	fCantonese	jau1; zaau1
U+3E6F.0	fMandarin	xiu1
U+3E6F.0	fEnglish	same as &#x8C85;
U+3E70.0	fUTF8	㹰
U+3E70.0	fR/S	 94.6
U+3E70.0	fTotalStrokes	 9
U+3E70.0	fCangjie	KHFMU
U+3E70.0	fCantonese	gwong1; gong1 (also)
U+3E70.0	fEnglish	[Korean character]
U+3E71.0	fUTF8	㹱
U+3E71.0	fCNS11643	3-3237
U+3E71.0	fGB	5-4671
U+3E71.0	fR/S	 94.7
U+3E71.0	fTotalStrokes	 10
U+3E71.0	fCangjie	KHRYO
U+3E71.0	fFourCorner	46280
U+3E71.0	fHakka	 cok7; sit7
U+3E71.0	fCantonese	zoek3; coek3 (also)
U+3E71.0	fMandarin	que4; hu2; qiao3; ran3
U+3E71.0	fEnglish	a celebrated dog
U+3E72.0	fUTF8	㹲
U+3E72.0	fCNS11643	5-2E5C
U+3E72.0	fR/S	 94.7
U+3E72.0	fTotalStrokes	 10
U+3E72.0	fCangjie	KHJKD
U+3E72.0	fFourCorner	44240
U+3E72.0	fCantonese	haau1; hau1; hau3
U+3E72.0	fMandarin	xiao1; xiao4
U+3E72.0	fEnglish	[1] a frightened pig or dog [2] bark [3] a running pig
U+3E73.0	fUTF8	㹳
U+3E73.0	fCNS11643	4-2E21
U+3E73.0	fGB	3-4433
U+3E73.0	fR/S	 94.7
U+3E73.0	fTotalStrokes	 10
U+3E73.0	fCangjie	KHMMR
U+3E73.0	fFourCorner	41260
U+3E73.0	fCantonese	ng4
U+3E73.0	fMandarin	wu2; wu4
U+3E73.0	fEnglish	a kind of ape (gibbon)
U+3E74.0	fUTF8	㹴
U+3E74.0	fCNS11643	4-2D7E
U+3E74.0	fGB	5-466D
U+3E74.0	fR/S	 94.7
U+3E74.0	fTotalStrokes	 10
U+3E74.0	fCangjie	KHMLK
U+3E74.0	fFourCorner	41240
U+3E74.0	fCantonese	gang1
U+3E74.0	fMandarin	geng1
U+3E74.0	fEnglish	[1] a kind of animal [2] fierce dog small and cute
U+3E75.0	fUTF8	㹵
U+3E75.0	fCNS11643	4-2E22
U+3E75.0	fR/S	 94.7
U+3E75.0	fTotalStrokes	 10
U+3E75.0	fCangjie	KHMVM
U+3E75.0	fFourCorner	41210
U+3E75.0	fCantonese	jing5; ging3; hing5
U+3E75.0	fMandarin	jing4
U+3E75.0	fEnglish	[1] hunting-dog [2] hunt in winter [3] imperial tour
U+3E76.0	fUTF8	㹶
U+3E76.0	fCNS11643	4-2E26
U+3E76.0	fGB	3-442C
U+3E76.0	fR/S	 94.7
U+3E76.0	fTotalStrokes	 10
U+3E76.0	fCangjie	KHNKG
U+3E76.0	fFourCorner	42240
U+3E76.0	fCantonese	ting4
U+3E76.0	fMandarin	ting2
U+3E76.0	fEnglish	a kind of wild animal (monkey tribe)
U+3E77.0	fUTF8	㹷
U+3E77.0	fCNS11643	6-416D
U+3E77.0	fR/S	 94.7
U+3E77.0	fTotalStrokes	 11; 10
U+3E77.0	fCangjie	KBIK
U+3E77.0	fFourCorner	43430
U+3E77.0	fCantonese	hei1
U+3E77.0	fMandarin	shi3; xin4
U+3E77.0	fEnglish	same as &#x72F6;, &#x8C68;
U+3E78.0	fUTF8	㹸
U+3E78.0	fCNS11643	4-2E27
U+3E78.0	fR/S	 94.7
U+3E78.0	fTotalStrokes	 10
U+3E78.0	fCangjie	KHHAU
U+3E78.0	fFourCorner	46210
U+3E78.0	fCantonese	maau6; ngaai4; ngaai6
U+3E78.0	fMandarin	mao4; ni2
U+3E78.0	fEnglish	same as &#x730A;
U+3E79.0	fUTF8	㹹
U+3E79.0	fCNS11643	5-3351
U+3E79.0	fR/S	 94.8
U+3E79.0	fTotalStrokes	 11
U+3E79.0	fCangjie	KHILO
U+3E79.0	fFourCorner	40230
U+3E79.0	fCantonese	gang1
U+3E79.0	fMandarin	geng1
U+3E79.0	fEnglish	a kind of dog
U+3E7A.0	fUTF8	㹺
U+3E7A.0	fCNS11643	4-3328
U+3E7A.0	fGB	3-4445
U+3E7A.0	fR/S	 94.8
U+3E7A.0	fTotalStrokes	 11
U+3E7A.0	fCangjie	KHEA
U+3E7A.0	fFourCorner	42260
U+3E7A.0	fCantonese	daat3; dap6
U+3E7A.0	fMandarin	ta4
U+3E7A.0	fEnglish	[1] dog to take food [2] a biter (said of a dog)
U+3E7B.0	fUTF8	㹻
U+3E7B.0	fCNS11643	6-4175
U+3E7B.0	fGB	3-443C
U+3E7B.0	fR/S	 94.8
U+3E7B.0	fTotalStrokes	 11
U+3E7B.0	fCangjie	KHHDV
U+3E7B.0	fFourCorner	42240
U+3E7B.0	fCantonese	wo1; waai1
U+3E7B.0	fMandarin	wo1; wei1
U+3E7B.0	fEnglish	same as &#x7327;
U+3E7C.0	fUTF8	㹼
U+3E7C.0	fCNS11643	4-3329
U+3E7C.0	fGB	3-4440
U+3E7C.0	fR/S	 94.8
U+3E7C.0	fTotalStrokes	 11
U+3E7C.0	fCangjie	KHPFD
U+3E7C.0	fFourCorner	47220
U+3E7C.0	fCantonese	guk1
U+3E7C.0	fMandarin	ju2; chu3; yu4
U+3E7C.0	fEnglish	a kind of animal
U+3E7D.0	fUTF8	㹽
U+3E7D.0	fCNS11643	4-3325
U+3E7D.0	fGB	3-4378
U+3E7D.0	fR/S	 94.8
U+3E7D.0	fTotalStrokes	 11
U+3E7D.0	fCangjie	KHII
U+3E7D.0	fFourCorner	43250
U+3E7D.0	fHakka	 can5; can3
U+3E7D.0	fCantonese	caan2
U+3E7D.0	fMandarin	chan3; zhan4; shan4
U+3E7D.0	fEnglish	a dog crunching its food
U+3E7E.0	fUTF8	㹾
U+3E7E.0	fCNS11643	4-332B
U+3E7E.0	fGB	3-443F
U+3E7E.0	fR/S	 94.8
U+3E7E.0	fTotalStrokes	 11
U+3E7E.0	fCangjie	KHHOR
U+3E7E.0	fFourCorner	48260
U+3E7E.0	fCantonese	piu5; piu2
U+3E7E.0	fMandarin	piao3; jiao4; biao3
U+3E7E.0	fEnglish	[1] cunning; crafty; sly; wily; artful [2] vigorous; strong
U+3E7F.0	fUTF8	㹿
U+3E7F.0	fCNS11643	4-3327
U+3E7F.0	fGB	3-4438
U+3E7F.0	fR/S	 94.8
U+3E7F.0	fTotalStrokes	 11
U+3E7F.0	fCangjie	KHYAJ
U+3E7F.0	fFourCorner	41240
U+3E7F.0	fCantonese	zuk6; coek3
U+3E7F.0	fMandarin	zhuo2; zhao4; diao3
U+3E7F.0	fEnglish	[1] fierce dog [2] bite fiercely (said of dog) [3] hunting in winter [4] a kind of beast
U+3E80.0	fUTF8	㺀
U+3E80.0	fCNS11643	4-332E
U+3E80.0	fGB	5-467B
U+3E80.0	fR/S	 94.8
U+3E80.0	fTotalStrokes	 11
U+3E80.0	fCangjie	KHPHP
U+3E80.0	fFourCorner	47230
U+3E80.0	fCantonese	fat1; nau4
U+3E80.0	fMandarin	nao2
U+3E80.0	fEnglish	same as &#x7331;
U+3E81.0	fUTF8	㺁
U+3E81.0	fCNS11643	4-3879
U+3E81.0	fGB	3-444F
U+3E81.0	fR/S	 94.9
U+3E81.0	fTotalStrokes	 12
U+3E81.0	fCangjie	KHVVW
U+3E81.0	fFourCorner	42260
U+3E81.0	fCantonese	nou5; lou5 (also)
U+3E81.0	fMandarin	nao3
U+3E81.0	fEnglish	a kind of beast
U+3E82.0	fUTF8	㺂
U+3E82.0	fCNS11643	4-3877
U+3E82.0	fGB	3-4448
U+3E82.0	fR/S	 94.9
U+3E82.0	fTotalStrokes	 12
U+3E82.0	fCangjie	KHIHR
U+3E82.0	fFourCorner	43250
U+3E82.0	fCantonese	ngaam4; aam4 (also); gaam2; him1; ngam4
U+3E82.0	fMandarin	gan3; jian4; yan2
U+3E82.0	fEnglish	[1] big and strong sheep [2] ewe; she-goat [3] a kind of dog [4] a dog barking loudly
U+3E83.0	fUTF8	㺃
U+3E83.0	fCNS11643	3-3D2C
U+3E83.0	fGB	5-4677
U+3E83.0	fR/S	 94.9
U+3E83.0	fTotalStrokes	 12
U+3E83.0	fCangjie	KHTPR
U+3E83.0	fFourCorner	44220
U+3E83.0	fCantonese	gau2
U+3E83.0	fMandarin	gou3
U+3E83.0	fEnglish	same as &#x72D7;
U+3E84.0	fUTF8	㺄
U+3E84.0	fCNS11643	4-387B
U+3E84.0	fGB	3-444B
U+3E84.0	fR/S	 94.9
U+3E84.0	fTotalStrokes	 12
U+3E84.0	fCangjie	KHOMN
U+3E84.0	fFourCorner	48220
U+3E84.0	fCantonese	jyu5; zyu5
U+3E84.0	fMandarin	yu2; yu3
U+3E84.0	fEnglish	[1] same as &#x72F3; [2] variant of &#x8C90;
U+3E85.0	fUTF8	㺅
U+3E85.0	fCNS11643	6-4B43
U+3E85.0	fR/S	 94.9
U+3E85.0	fTotalStrokes	 12
U+3E85.0	fCangjie	KHNMK
U+3E85.0	fFourCorner	47230
U+3E85.0	fCantonese	hau4; ngau4
U+3E85.0	fMandarin	hou2
U+3E85.0	fEnglish	same as &#x7334;
U+3E86.0	fUTF8	㺆
U+3E86.0	fGB	S-225B
U+3E86.0	fR/S	 94.9
U+3E86.0	fTotalStrokes	 12
U+3E86.0	fCangjie	KHIKO
U+3E86.0	fCantonese	ban6
U+3E87.0	fUTF8	㺇
U+3E87.0	fCNS11643	4-4465
U+3E87.0	fGB	3-4459
U+3E87.0	fR/S	 94.10
U+3E87.0	fTotalStrokes	 14; 13
U+3E87.0	fCangjie	KHSLK
U+3E87.0	fFourCorner	43230
U+3E87.0	fCantonese	si1
U+3E87.0	fMandarin	si1
U+3E87.0	fEnglish	[1] warden [2] minister of public works (in ancient China) [3] observe; watch
U+3E88.0	fUTF8	㺈
U+3E88.0	fCNS11643	4-3E54
U+3E88.0	fGB	3-4457
U+3E88.0	fR/S	 94.10
U+3E88.0	fTotalStrokes	 13
U+3E88.0	fCangjie	KHUMI
U+3E88.0	fFourCorner	42230
U+3E88.0	fHakka	 chi1; cih1; chii1
U+3E88.0	fCantonese	ci1
U+3E88.0	fMandarin	chi1
U+3E88.0	fEnglish	a kind of dog
U+3E89.0	fUTF8	㺉
U+3E89.0	fCNS11643	4-4466
U+3E89.0	fGB	3-4F32
U+3E89.0	fR/S	 94.10
U+3E89.0	fTotalStrokes	 14; 13
U+3E89.0	fCangjie	GKHNE
U+3E89.0	fFourCorner	47440
U+3E89.0	fCantonese	kau3; huk6
U+3E89.0	fMandarin	hu4
U+3E89.0	fEnglish	a fierce animal of the dog tribe; with dog's head and horse tail; with yellow and black colors
U+3E8A.0	fUTF8	㺊
U+3E8A.0	fCNS11643	4-3E53
U+3E8A.0	fGB	3-4455
U+3E8A.0	fR/S	 94.10
U+3E8A.0	fTotalStrokes	 13
U+3E8A.0	fCangjie	KHTGP
U+3E8A.0	fFourCorner	48230
U+3E8A.0	fHakka	 jong5
U+3E8A.0	fCantonese	joeng6; zoeng6
U+3E8A.0	fMandarin	yang4
U+3E8A.0	fEnglish	a beast (looks like lion)
U+3E8B.0	fUTF8	㺋
U+3E8B.0	fCNS11643	3-4260
U+3E8B.0	fR/S	 94.10
U+3E8B.0	fTotalStrokes	 13
U+3E8B.0	fCangjie	KHCIM
U+3E8B.0	fFourCorner	48220
U+3E8B.0	fCantonese	jung1; zung1
U+3E8B.0	fMandarin	weng1
U+3E8B.0	fEnglish	pig; hog
U+3E8C.0	fUTF8	㺌
U+3E8C.0	fCNS11643	4-3E52
U+3E8C.0	fGB	5-472A
U+3E8C.0	fR/S	 94.10
U+3E8C.0	fTotalStrokes	 13
U+3E8C.0	fCangjie	KHTXC
U+3E8C.0	fFourCorner	48230
U+3E8C.0	fCantonese	laam6
U+3E8C.0	fMandarin	jian4; xian4
U+3E8C.0	fEnglish	[1] endless barking [2] two dogs are fighting
U+3E8D.0	fUTF8	㺍
U+3E8D.0	fGB	S-225F
U+3E8D.0	fR/S	 94.10
U+3E8D.0	fTotalStrokes	 13
U+3E8D.0	fCangjie	KHJOC
U+3E8D.0	fCantonese	ban1; bin1 (also); pun4
U+3E8D.0	fEnglish	simplified form of &#x7371;
U+3E8E.0	fUTF8	㺎
U+3E8E.0	fCNS11643	4-446A
U+3E8E.0	fGB	3-445C
U+3E8E.0	fR/S	 94.11
U+3E8E.0	fTotalStrokes	 14
U+3E8E.0	fCangjie	KHILB
U+3E8E.0	fFourCorner	40220
U+3E8E.0	fCantonese	jung4; zung4
U+3E8E.0	fMandarin	yong2; rong2
U+3E8E.0	fEnglish	fierce beast (zebu, humped-ox?)
U+3E8F.0	fUTF8	㺏
U+3E8F.0	fCNS11643	6-5E6E
U+3E8F.0	fR/S	 94.11
U+3E8F.0	fTotalStrokes	 14
U+3E8F.0	fCangjie	KHLLV
U+3E8F.0	fFourCorner	45240
U+3E8F.0	fCantonese	lau4; lau1 (also); laau4; zyu4
U+3E8F.0	fMandarin	luu2; lou2
U+3E8F.0	fEnglish	same as &#x8C97;
U+3E90.0	fUTF8	㺐
U+3E90.0	fCNS11643	4-4469
U+3E90.0	fGB	3-445F
U+3E90.0	fR/S	 94.11
U+3E90.0	fTotalStrokes	 14
U+3E90.0	fCangjie	KHVVD
U+3E90.0	fFourCorner	42290
U+3E90.0	fCantonese	liu4; zau2
U+3E90.0	fMandarin	zhao3
U+3E90.0	fEnglish	same as &#x7360;
U+3E91.0	fUTF8	㺑
U+3E91.0	fCNS11643	4-4468
U+3E91.0	fGB	3-4444
U+3E91.0	fR/S	 94.11
U+3E91.0	fTotalStrokes	 14
U+3E91.0	fCangjie	KHIIH
U+3E91.0	fFourCorner	43220
U+3E91.0	fCantonese	saam1; cam4
U+3E91.0	fMandarin	shan3; can2
U+3E91.0	fEnglish	[1] dogs wormed through a narrow place [2] oppress cruelly; to injure heartlessly [3] dogs biting [4] dog's hair [5] legendary monster (looks like a dwarf) [6] bark
U+3E92.0	fUTF8	㺒
U+3E92.0	fCNS11643	4-4467
U+3E92.0	fGB	3-445E
U+3E92.0	fR/S	 94.11
U+3E92.0	fTotalStrokes	 14
U+3E92.0	fCangjie	KHSMH
U+3E92.0	fFourCorner	47220
U+3E92.0	fCantonese	haau1
U+3E92.0	fMandarin	xiao1; liao4; yao2
U+3E92.0	fEnglish	[1] barking of a frightened dog [2] confuse; disorder; disturbance [3] cunning; artful; crafty [4] fail and be exposed
U+3E93.0	fUTF8	㺓
U+3E93.0	fCNS11643	3-4776
U+3E93.0	fR/S	 94.11
U+3E93.0	fTotalStrokes	 14
U+3E93.0	fCangjie	KHQMC
U+3E93.0	fFourCorner	45280
U+3E93.0	fCantonese	zaak3; zik1
U+3E93.0	fMandarin	piao4
U+3E93.0	fEnglish	[1] same as &#x8E5F; [2] interchangeable &#x50C4;
U+3E94.0	fUTF8	㺔
U+3E94.0	fCNS11643	4-387A
U+3E94.0	fGB	3-4375
U+3E94.0	fR/S	 94.12; 94.9
U+3E94.0	fTotalStrokes	 12; 15
U+3E94.0	fCangjie	KHIKF
U+3E94.0	fFourCorner	44220
U+3E94.0	fCantonese	hoi3; faai1; hoi6
U+3E94.0	fMandarin	hui1; hai4; wei4
U+3E94.0	fEnglish	[1] a kind of beast [2] a legendary monster
U+3E95.0	fUTF8	㺕
U+3E95.0	fCNS11643	4-4A7B
U+3E95.0	fGB	3-4463
U+3E95.0	fR/S	 94.12
U+3E95.0	fTotalStrokes	 15
U+3E95.0	fCangjie	KHHDW
U+3E95.0	fFourCorner	42260
U+3E95.0	fCantonese	faan4; bin6; fan4
U+3E95.0	fMandarin	fan2
U+3E95.0	fEnglish	[1] the fighting sound of dogs [2] agile; sprightly
U+3E96.0	fUTF8	㺖
U+3E96.0	fCNS11643	4-4A76
U+3E96.0	fGB	3-445D
U+3E96.0	fR/S	 94.12
U+3E96.0	fTotalStrokes	 15
U+3E96.0	fCangjie	KHMJK
U+3E96.0	fFourCorner	48240
U+3E96.0	fHakka	 kam3
U+3E96.0	fCantonese	haam2; ham3
U+3E96.0	fMandarin	xian3; han3
U+3E96.0	fEnglish	barking of a little dogto eat and not be satisfied
U+3E97.0	fUTF8	㺗
U+3E97.0	fCNS11643	4-4A7A
U+3E97.0	fGB	3-4442
U+3E97.0	fR/S	 94.12
U+3E97.0	fTotalStrokes	 15
U+3E97.0	fCangjie	KHRRJ
U+3E97.0	fFourCorner	46250
U+3E97.0	fCantonese	caan1
U+3E97.0	fMandarin	chan1; dan4; yan2
U+3E97.0	fEnglish	 bite; gnaw; dogs bite
U+3E98.0	fUTF8	㺘
U+3E98.0	fCNS11643	4-4A7C
U+3E98.0	fGB	3-4462
U+3E98.0	fR/S	 94.12
U+3E98.0	fTotalStrokes	 15
U+3E98.0	fCangjie	KHOGF
U+3E98.0	fFourCorner	40230
U+3E98.0	fCantonese	zaam3; sou1; tou1; zam3; zin1; zin2
U+3E98.0	fMandarin	zhan4
U+3E98.0	fEnglish	a kind of animal
U+3E99.0	fUTF8	㺙
U+3E99.0	fGB	S-225C
U+3E99.0	fR/S	 94.12
U+3E99.0	fTotalStrokes	 15
U+3E99.0	fCangjie	KHJWO
U+3E99.0	fCantonese	sat6
U+3E9A.0	fUTF8	㺚
U+3E9A.0	fCNS11643	7-2F74
U+3E9A.0	fR/S	 94.13
U+3E9A.0	fTotalStrokes	 16
U+3E9A.0	fCangjie	KHYGQ
U+3E9A.0	fFourCorner	44230
U+3E9A.0	fCantonese	caat3; cat3; sat3
U+3E9A.0	fMandarin	ta4; ta3
U+3E9A.0	fEnglish	same as &#x737A;
U+3E9B.0	fUTF8	㺛
U+3E9B.0	fCNS11643	4-5132
U+3E9B.0	fGB	3-4431
U+3E9B.0	fR/S	 94.13
U+3E9B.0	fTotalStrokes	 16
U+3E9B.0	fCangjie	KHSFT
U+3E9B.0	fFourCorner	41240
U+3E9B.0	fCantonese	zyu3
U+3E9B.0	fMandarin	zhu4
U+3E9B.0	fEnglish	[1] the name of a village in Henan Province [2] the name of a pavilion
U+3E9C.0	fUTF8	㺜
U+3E9C.0	fCNS11643	4-5134
U+3E9C.0	fGB	3-4430
U+3E9C.0	fR/S	 94.13
U+3E9C.0	fTotalStrokes	 16
U+3E9C.0	fCangjie	KHTWV
U+3E9C.0	fFourCorner	45230
U+3E9C.0	fHakka	 nau2; nung2
U+3E9C.0	fCantonese	nung4; lung4 (also)
U+3E9C.0	fMandarin	nong2; nao2; ban3; wan4
U+3E9C.0	fEnglish	a fierce shaggy-haired watch-dog
U+3E9D.0	fUTF8	㺝
U+3E9D.0	fCNS11643	3-5548
U+3E9D.0	fGB	5-4726
U+3E9D.0	fR/S	 94.14
U+3E9D.0	fTotalStrokes	 17
U+3E9D.0	fCangjie	KHSIT
U+3E9D.0	fFourCorner	48210
U+3E9D.0	fCantonese	haam5; ham5; lam6
U+3E9D.0	fMandarin	jian4; xian4
U+3E9D.0	fEnglish	[1] the sound of a tiger [2] a fierce dog barking endlessly [3] fierce dog [4] the sound of a dog's biting
U+3E9E.0	fUTF8	㺞
U+3E9E.0	fCNS11643	4-566A
U+3E9E.0	fGB	3-436B
U+3E9E.0	fR/S	 94.14
U+3E9E.0	fTotalStrokes	 17; 16
U+3E9E.0	fCangjie	KHHXC
U+3E9E.0	fFourCorner	47280
U+3E9E.0	fCantonese	jyu4; zyu4
U+3E9E.0	fMandarin	yu2
U+3E9E.0	fEnglish	[1] sign in lamentation [2] crying of a piggy [3] a kind of beast
U+3E9F.0	fUTF8	㺟
U+3E9F.0	fCNS11643	4-566B
U+3E9F.0	fGB	3-4469
U+3E9F.0	fR/S	 94.14
U+3E9F.0	fTotalStrokes	 17; 16
U+3E9F.0	fCangjie	KHSMG
U+3E9F.0	fFourCorner	47210
U+3E9F.0	fCantonese	zok6; zuk6
U+3E9F.0	fMandarin	zhuo2
U+3E9F.0	fEnglish	[1] a kind of beast (looks like a deer, white-tailed) [2] same as &#x8817;
U+3EA0.0	fUTF8	㺠
U+3EA0.0	fCNS11643	4-5B67
U+3EA0.0	fGB	3-446B
U+3EA0.0	fR/S	 94.15
U+3EA0.0	fTotalStrokes	 18
U+3EA0.0	fCangjie	KHJCO
U+3EA0.0	fFourCorner	43230
U+3EA0.0	fCantonese	jau6; zaau6
U+3EA0.0	fMandarin	you4; yu4
U+3EA0.0	fEnglish	[1] a black ape with long tail [2] same as &#x8C81;
U+3EA1.0	fUTF8	㺡
U+3EA1.0	fCNS11643	5-683B
U+3EA1.0	fGB	3-436E
U+3EA1.0	fR/S	 94.16
U+3EA1.0	fTotalStrokes	 19
U+3EA1.0	fCangjie	KHMHM
U+3EA1.0	fFourCorner	41210
U+3EA1.0	fCantonese	lik6; lim5
U+3EA1.0	fMandarin	li4
U+3EA1.0	fEnglish	a kind of beast
U+3EA2.0	fUTF8	㺢
U+3EA2.0	fGB	3-446C
U+3EA2.0	fR/S	 94.16
U+3EA2.0	fTotalStrokes	 19
U+3EA2.0	fCangjie	KHMBG
U+3EA2.0	fCantonese	fok3; ngo1 (also); kut3
U+3EA2.0	fEnglish	a kind of animal, body is smaller than a giraffe, two short horns on the head, back and legs with black and white stripes spaced in-between
U+3EA3.0	fUTF8	㺣
U+3EA3.0	fCNS11643	4-6365
U+3EA3.0	fGB	3-446E
U+3EA3.0	fR/S	 94.17
U+3EA3.0	fTotalStrokes	 20
U+3EA3.0	fCangjie	KHYTI
U+3EA3.0	fFourCorner	43250
U+3EA3.0	fCantonese	hei1; sik1
U+3EA3.0	fMandarin	xi1
U+3EA3.0	fEnglish	a kind of animal; pig hog
U+3EA4.0	fUTF8	㺤
U+3EA4.0	fCNS11643	4-6366
U+3EA4.0	fGB	3-446F
U+3EA4.0	fR/S	 94.17
U+3EA4.0	fTotalStrokes	 20
U+3EA4.0	fCangjie	KHOIM
U+3EA4.0	fFourCorner	43250
U+3EA4.0	fCantonese	cim1
U+3EA4.0	fMandarin	xian1
U+3EA4.0	fEnglish	a kind of beast
U+3EA5.0	fUTF8	㺥
U+3EA5.0	fCNS11643	4-6367
U+3EA5.0	fGB	3-4470
U+3EA5.0	fR/S	 94.17
U+3EA5.0	fTotalStrokes	 20
U+3EA5.0	fCangjie	KHNRI
U+3EA5.0	fFourCorner	47210
U+3EA5.0	fHakka	 cam2
U+3EA5.0	fCantonese	caam4
U+3EA5.0	fMandarin	chan2; tan2
U+3EA5.0	fEnglish	[1] bark [2] same as &#x6BDA;
U+3EA6.0	fUTF8	㺦
U+3EA6.0	fCNS11643	4-6363
U+3EA6.0	fGB	3-446D
U+3EA6.0	fR/S	 94.17
U+3EA6.0	fTotalStrokes	 20
U+3EA6.0	fCangjie	KHSJT
U+3EA6.0	fFourCorner	42270
U+3EA6.0	fHakka	 lien2
U+3EA6.0	fCantonese	lin4; lyun4
U+3EA6.0	fMandarin	lian2
U+3EA6.0	fEnglish	bounding jumps of animals
U+3EA7.0	fUTF8	㺧
U+3EA7.0	fCNS11643	4-6B6F
U+3EA7.0	fGB	3-4472
U+3EA7.0	fR/S	 94.21
U+3EA7.0	fTotalStrokes	 24
U+3EA7.0	fCangjie	KHRRR
U+3EA7.0	fFourCorner	46260
U+3EA7.0	fCantonese	hiu1; hei3; ho2
U+3EA7.0	fMandarin	xiao1
U+3EA7.0	fEnglish	[1] a white-yellow dog [2] a yellow dog [3] mad dog [4] a group of dogs bark wildly
U+3EA8.0	fUTF8	㺨
U+3EA8.0	fCNS11643	3-242E
U+3EA8.0	fGB	5-416F
U+3EA8.0	fR/S	 96.2
U+3EA8.0	fTotalStrokes	 6
U+3EA8.0	fCangjie	MGI
U+3EA8.0	fFourCorner	12130
U+3EA8.0	fCantonese	si1
U+3EA8.0	fMandarin	si1
U+3EA8.0	fEnglish	fine stone like jade
U+3EA9.0	fUTF8	㺩
U+3EA9.0	fCNS11643	4-2329
U+3EA9.0	fGB	5-416E
U+3EA9.0	fR/S	 96.2
U+3EA9.0	fTotalStrokes	 6
U+3EA9.0	fCangjie	MGVL
U+3EA9.0	fFourCorner	12100
U+3EA9.0	fCantonese	gau6
U+3EA9.0	fMandarin	se4; jiu4
U+3EA9.0	fEnglish	[1] jade articles [2] same as &#x738C;
U+3EAA.0	fUTF8	㺪
U+3EAA.0	fCNS11643	3-2430
U+3EAA.0	fGB	5-416C
U+3EAA.0	fR/S	 96.2
U+3EAA.0	fTotalStrokes	 6
U+3EAA.0	fCangjie	MGY
U+3EAA.0	fFourCorner	13100
U+3EAA.0	fHakka	 puk8
U+3EAA.0	fCantonese	pok3; lung4
U+3EAA.0	fMandarin	pu2
U+3EAA.0	fEnglish	same as &#x749E;
U+3EAB.0	fUTF8	㺫
U+3EAB.0	fCNS11643	3-2431
U+3EAB.0	fR/S	 96.2
U+3EAB.0	fTotalStrokes	 6
U+3EAB.0	fCangjie	MGLN
U+3EAB.0	fFourCorner	12100
U+3EAB.0	fCantonese	kau4; aau3; kaau4
U+3EAB.0	fMandarin	qiu2
U+3EAB.0	fEnglish	same as &#x738C;
U+3EAC.0	fUTF8	㺬
U+3EAC.0	fCNS11643	3-2671
U+3EAC.0	fR/S	 96.3
U+3EAC.0	fTotalStrokes	 7
U+3EAC.0	fCangjie	MGHNI
U+3EAC.0	fFourCorner	17110
U+3EAC.0	fCantonese	gung2
U+3EAC.0	fMandarin	gong3
U+3EAC.0	fEnglish	same as &#x73D9;
U+3EAD.0	fUTF8	㺭
U+3EAD.0	fCNS11643	3-266F
U+3EAD.0	fR/S	 96.3
U+3EAD.0	fTotalStrokes	 7
U+3EAD.0	fCangjie	MGND
U+3EAD.0	fFourCorner	17140
U+3EAD.0	fCantonese	zi2; gei2
U+3EAD.0	fMandarin	zi3
U+3EAD.0	fEnglish	a kind of jade
U+3EAE.0	fUTF8	㺮
U+3EAE.0	fCNS11643	3-2670
U+3EAE.0	fR/S	 96.3
U+3EAE.0	fTotalStrokes	 7
U+3EAE.0	fCangjie	MGMMS
U+3EAE.0	fFourCorner	11120
U+3EAE.0	fCantonese	jyu1; zyu1
U+3EAE.0	fMandarin	yu2
U+3EAE.0	fEnglish	same as &#x7397;
U+3EAF.0	fUTF8	㺯
U+3EAF.0	fR/S	 96.3
U+3EAF.0	fTotalStrokes	 7
U+3EAF.0	fCangjie	MGK
U+3EAF.0	fCantonese	lung6
U+3EAF.0	fEnglish	[Korean character]
U+3EB0.0	fUTF8	㺰
U+3EB0.0	fCNS11643	3-2A2E
U+3EB0.0	fR/S	 96.4
U+3EB0.0	fTotalStrokes	 8
U+3EB0.0	fCangjie	MGMLB
U+3EB0.0	fFourCorner	11120
U+3EB0.0	fCantonese	si1
U+3EB0.0	fMandarin	shi1
U+3EB0.0	fEnglish	same as &#x7461;
U+3EB1.0	fUTF8	㺱
U+3EB1.0	fCNS11643	3-2D76
U+3EB1.0	fR/S	 96.4
U+3EB1.0	fTotalStrokes	 9
U+3EB1.0	fCangjie	OSMGI
U+3EB1.0	fFourCorner	27100
U+3EB1.0	fCantonese	jing4; zing4
U+3EB1.0	fMandarin	reng2
U+3EB1.0	fEnglish	a kind of jade
U+3EB2.0	fUTF8	㺲
U+3EB2.0	fCNS11643	6-2D21
U+3EB2.0	fR/S	 96.4
U+3EB2.0	fTotalStrokes	 8
U+3EB2.0	fCangjie	MGNG
U+3EB2.0	fFourCorner	17110
U+3EB2.0	fHakka	 neu3
U+3EB2.0	fCantonese	nau5; lau5 (also); naau2
U+3EB2.0	fMandarin	niu3
U+3EB2.0	fEnglish	ancient form of &#x9215;
U+3EB3.0	fUTF8	㺳
U+3EB3.0	fCNS11643	3-2A30
U+3EB3.0	fR/S	 96.4
U+3EB3.0	fTotalStrokes	 8
U+3EB3.0	fCangjie	MGYE
U+3EB3.0	fFourCorner	11140
U+3EB3.0	fCantonese	mui4
U+3EB3.0	fMandarin	mei2
U+3EB3.0	fEnglish	same as &#x73AB;
U+3EB4.0	fUTF8	㺴
U+3EB4.0	fCNS11643	4-273B
U+3EB4.0	fGB	3-3F21
U+3EB4.0	fR/S	 96.4
U+3EB4.0	fTotalStrokes	 8
U+3EB4.0	fCangjie	MGIK
U+3EB4.0	fFourCorner	13130
U+3EB4.0	fCantonese	baat3
U+3EB4.0	fMandarin	ba1
U+3EB4.0	fEnglish	[1] the name of a spirit; a god [2] a kind of jade
U+3EB5.0	fUTF8	㺵
U+3EB5.0	fCNS11643	3-2A31
U+3EB5.0	fR/S	 96.4
U+3EB5.0	fTotalStrokes	 8
U+3EB5.0	fCangjie	MGNO
U+3EB5.0	fFourCorner	17180
U+3EB5.0	fCantonese	gau2
U+3EB5.0	fMandarin	jiu3; jiu2
U+3EB5.0	fEnglish	corrupted form of &#x7396;
U+3EB6.0	fUTF8	㺶
U+3EB6.0	fR/S	 96.4
U+3EB6.0	fTotalStrokes	 8
U+3EB6.0	fCangjie	MGYJ
U+3EB6.0	fCantonese	dau2
U+3EB6.0	fEnglish	[Korean character]
U+3EB7.0	fUTF8	㺷
U+3EB7.0	fCNS11643	3-2D7D
U+3EB7.0	fR/S	 96.5
U+3EB7.0	fTotalStrokes	 9
U+3EB7.0	fCangjie	MGIJC
U+3EB7.0	fFourCorner	13190
U+3EB7.0	fCantonese	seot1
U+3EB7.0	fMandarin	xu4
U+3EB7.0	fEnglish	same as &#x73EC;
U+3EB8.0	fUTF8	㺸
U+3EB8.0	fCNS11643	3-323A
U+3EB8.0	fR/S	 96.5
U+3EB8.0	fTotalStrokes	 10
U+3EB8.0	fCangjie	ONMGI
U+3EB8.0	fFourCorner	27100
U+3EB8.0	fCantonese	ping4
U+3EB8.0	fMandarin	ping2
U+3EB8.0	fEnglish	a kind of jade
U+3EB9.0	fUTF8	㺹
U+3EB9.0	fCNS11643	4-2A37
U+3EB9.0	fGB	3-3F30
U+3EB9.0	fR/S	 96.5
U+3EB9.0	fTotalStrokes	 9
U+3EB9.0	fCangjie	MGIT
U+3EB9.0	fFourCorner	13140
U+3EB9.0	fCantonese	bin6
U+3EB9.0	fMandarin	bian4
U+3EB9.0	fEnglish	[1] a kind of jade [2] a conical cap (wore on ceremonious occasions in ancient times) decorated with jade
U+3EBA.0	fUTF8	㺺
U+3EBA.0	fCNS11643	4-2A38
U+3EBA.0	fR/S	 96.5
U+3EBA.0	fTotalStrokes	 9
U+3EBA.0	fCangjie	MGBU
U+3EBA.0	fFourCorner	16100
U+3EBA.0	fCantonese	mou6
U+3EBA.0	fMandarin	mao4
U+3EBA.0	fEnglish	ancient abbreviated form of &#x7441;
U+3EBB.0	fUTF8	㺻
U+3EBB.0	fGB	S-2230
U+3EBB.0	fR/S	 96.5
U+3EBB.0	fTotalStrokes	 9
U+3EBB.0	fCangjie	MGYLB
U+3EBB.0	fFourCorner	1012
U+3EBB.0	fCantonese	fat1
U+3EBC.0	fUTF8	㺼
U+3EBC.0	fGB	S-2231
U+3EBC.0	fR/S	 96.5
U+3EBC.0	fTotalStrokes	 9
U+3EBC.0	fCangjie	MGNYO
U+3EBC.0	fCantonese	pat1
U+3EBD.0	fUTF8	㺽
U+3EBD.0	fCNS11643	F-2C32
U+3EBD.0	fR/S	 96.5
U+3EBD.0	fTotalStrokes	 9
U+3EBD.0	fCangjie	MGMFM
U+3EBD.0	fFourCorner	1111
U+3EBD.0	fCantonese	pei1
U+3EBE.0	fUTF8	㺾
U+3EBE.0	fR/S	 96.5
U+3EBE.0	fTotalStrokes	 9
U+3EBE.0	fCangjie	MGBR
U+3EBE.0	fCantonese	gwing1
U+3EBE.0	fEnglish	[Korean character]
U+3EBF.0	fUTF8	㺿
U+3EBF.0	fCNS11643	4-332F
U+3EBF.0	fGB	5-4235
U+3EBF.0	fR/S	 96.6; 96.7
U+3EBF.0	fTotalStrokes	 11; 10
U+3EBF.0	fCangjie	MGSLL
U+3EBF.0	fFourCorner	11110
U+3EBF.0	fCantonese	ji4; zi4
U+3EBF.0	fMandarin	yi2
U+3EBF.0	fEnglish	[1] a jade-like stone [2] a kind of jade
U+3EC0.0	fUTF8	㻀
U+3EC0.0	fCNS11643	3-3243
U+3EC0.0	fR/S	 96.6
U+3EC0.0	fTotalStrokes	 10
U+3EC0.0	fCangjie	MGLWK
U+3EC0.0	fFourCorner	15100
U+3EC0.0	fCantonese	jyu4; zyu4; zyu5
U+3EC0.0	fMandarin	yu2; you2
U+3EC0.0	fEnglish	fine stone which is little less valuable than jade
U+3EC1.0	fUTF8	㻁
U+3EC1.0	fCNS11643	3-3244
U+3EC1.0	fR/S	 96.6
U+3EC1.0	fTotalStrokes	 10
U+3EC1.0	fCangjie	MGWR
U+3EC1.0	fFourCorner	16100
U+3EC1.0	fCantonese	gwai1
U+3EC1.0	fMandarin	gui1
U+3EC1.0	fEnglish	[1] non-classical form of &#x74CC; [2] variant of &#x7470;
U+3EC2.0	fUTF8	㻂
U+3EC2.0	fCNS11643	3-323B
U+3EC2.0	fGB	5-4233
U+3EC2.0	fR/S	 96.6
U+3EC2.0	fTotalStrokes	 10
U+3EC2.0	fCangjie	MGTT
U+3EC2.0	fFourCorner	18140
U+3EC2.0	fCantonese	ping4
U+3EC2.0	fMandarin	ping2; ping3
U+3EC2.0	fEnglish	[1] a kind of jade [2] non-classical form of &#x73AD;
U+3EC3.0	fUTF8	㻃
U+3EC3.0	fCNS11643	4-3330
U+3EC3.0	fGB	5-4273
U+3EC3.0	fR/S	 96.6
U+3EC3.0	fTotalStrokes	 11; 10
U+3EC3.0	fCangjie	TWMGI
U+3EC3.0	fFourCorner	55100
U+3EC3.0	fCantonese	kuk1
U+3EC3.0	fMandarin	qu3
U+3EC3.0	fEnglish	[1] crooked; bend; twist [2] involved; indirect
U+3EC4.0	fUTF8	㻄
U+3EC4.0	fCNS11643	3-3245
U+3EC4.0	fR/S	 96.6
U+3EC4.0	fTotalStrokes	 10
U+3EC4.0	fCangjie	MGHMO
U+3EC4.0	fFourCorner	12180
U+3EC4.0	fCantonese	bou2
U+3EC4.0	fMandarin	bao3
U+3EC4.0	fEnglish	same as &#x5BF6;
U+3EC5.0	fUTF8	㻅
U+3EC5.0	fGB	7-235E
U+3EC5.0	fR/S	 96.6
U+3EC5.0	fTotalStrokes	 10
U+3EC5.0	fCangjie	MGOMI
U+3EC5.0	fCantonese	wui6
U+3EC5.0	fMandarin	hui4; kuai4
U+3EC5.0	fEnglish	simplified form of &#x74AF;
U+3EC6.0	fUTF8	㻆
U+3EC6.0	fR/S	 96.6
U+3EC6.0	fTotalStrokes	 10
U+3EC6.0	fCangjie	MGIB
U+3EC6.0	fCantonese	jyun5
U+3EC6.0	fEnglish	[Korean character]
U+3EC7.0	fUTF8	㻇
U+3EC7.0	fCNS11643	F-3127
U+3EC7.0	fR/S	 96.6
U+3EC7.0	fTotalStrokes	 10
U+3EC7.0	fCangjie	MGOMG
U+3EC7.0	fFourCorner	1811
U+3EC7.0	fCantonese	cyun4
U+3EC8.0	fUTF8	㻈
U+3EC8.0	fCNS11643	3-3242
U+3EC8.0	fR/S	 96.6
U+3EC8.0	fTotalStrokes	 10
U+3EC8.0	fCangjie	MGHMR
U+3EC8.0	fFourCorner	12160
U+3EC8.0	fCantonese	hau6
U+3EC9.0	fUTF8	㻉
U+3EC9.0	fCNS11643	3-373F
U+3EC9.0	fGB	5-4237
U+3EC9.0	fR/S	 96.7
U+3EC9.0	fTotalStrokes	 11
U+3EC9.0	fCangjie	MGYLH
U+3EC9.0	fFourCorner	11120
U+3EC9.0	fCantonese	bou6
U+3EC9.0	fMandarin	bu4
U+3EC9.0	fEnglish	fine jade
U+3ECA.0	fUTF8	㻊
U+3ECA.0	fCNS11643	3-3742
U+3ECA.0	fGB	5-423E
U+3ECA.0	fR/S	 96.7
U+3ECA.0	fTotalStrokes	 11
U+3ECA.0	fCangjie	MGNAU
U+3ECA.0	fFourCorner	17110
U+3ECA.0	fCantonese	mun4
U+3ECA.0	fMandarin	mang2; men2; meng4
U+3ECA.0	fEnglish	same as non-classical form of &#x748A;
U+3ECB.0	fUTF8	㻋
U+3ECB.0	fCNS11643	3-3741
U+3ECB.0	fR/S	 96.7
U+3ECB.0	fTotalStrokes	 11
U+3ECB.0	fCangjie	MGDL
U+3ECB.0	fFourCorner	15190
U+3ECB.0	fCantonese	laat6
U+3ECB.0	fMandarin	la4; lei4
U+3ECB.0	fEnglish	(abbreviated form) interchangeable with &#x74EC;
U+3ECC.0	fUTF8	㻌
U+3ECC.0	fCNS11643	4-3333
U+3ECC.0	fGB	3-3F4D
U+3ECC.0	fR/S	 96.7
U+3ECC.0	fTotalStrokes	 11
U+3ECC.0	fCangjie	MGOMD
U+3ECC.0	fFourCorner	18190
U+3ECC.0	fCantonese	tou4; caau1; caau4
U+3ECC.0	fMandarin	tu2
U+3ECC.0	fEnglish	fine jade
U+3ECD.0	fUTF8	㻍
U+3ECD.0	fCNS11643	3-373E
U+3ECD.0	fGB	5-4238
U+3ECD.0	fR/S	 96.7
U+3ECD.0	fTotalStrokes	 11
U+3ECD.0	fCangjie	MGRVK
U+3ECD.0	fFourCorner	16130
U+3ECD.0	fCantonese	ng4
U+3ECD.0	fMandarin	wu2
U+3ECD.0	fEnglish	interchangeable with &#x73F8;
U+3ECE.0	fUTF8	㻎
U+3ECE.0	fCNS11643	3-3743
U+3ECE.0	fR/S	 96.7
U+3ECE.0	fTotalStrokes	 11
U+3ECE.0	fCangjie	KMGG
U+3ECE.0	fFourCorner	40030
U+3ECE.0	fCantonese	sat1
U+3ECE.0	fMandarin	se4; li4
U+3ECE.0	fEnglish	ancient form of &#x745F;
U+3ECF.0	fUTF8	㻏
U+3ECF.0	fGB	S-2235
U+3ECF.0	fR/S	 96.7
U+3ECF.0	fTotalStrokes	 11
U+3ECF.0	fCangjie	MGSMF
U+3ECF.0	fCantonese	ling4
U+3ECF.0	fMandarin	ling2; ling3
U+3ECF.0	fEnglish	simplified form of &#x73B2;
U+3ED0.0	fUTF8	㻐
U+3ED0.0	fR/S	 96.7
U+3ED0.0	fTotalStrokes	 11
U+3ED0.0	fCangjie	MGICE
U+3ED0.0	fFourCorner	1314
U+3ED0.0	fCantonese	ceon1
U+3ED0.0	fEnglish	[Korean character]
U+3ED1.0	fUTF8	㻑
U+3ED1.0	fCNS11643	3-3D34
U+3ED1.0	fR/S	 96.8
U+3ED1.0	fTotalStrokes	 12
U+3ED1.0	fCangjie	MGHDD
U+3ED1.0	fFourCorner	12140
U+3ED1.0	fCantonese	gwai3; gwaai3; gwan3
U+3ED1.0	fMandarin	ji4
U+3ED1.0	fEnglish	a kind of jade
U+3ED2.0	fUTF8	㻒
U+3ED2.0	fCNS11643	4-387D
U+3ED2.0	fGB	3-3F58
U+3ED2.0	fR/S	 96.8
U+3ED2.0	fTotalStrokes	 12
U+3ED2.0	fCangjie	MGWHD
U+3ED2.0	fFourCorner	16100
U+3ED2.0	fCantonese	kwan5; kwaan2
U+3ED2.0	fMandarin	yun3; jun4
U+3ED2.0	fEnglish	[1] red colored jade [2] even; regular; equal
U+3ED3.0	fUTF8	㻓
U+3ED3.0	fCNS11643	5-3A2D
U+3ED3.0	fR/S	 96.8
U+3ED3.0	fTotalStrokes	 12
U+3ED3.0	fCangjie	MGSJE
U+3ED3.0	fFourCorner	17140
U+3ED3.0	fCantonese	zau1; lit6; zaau1
U+3ED3.0	fMandarin	zou1
U+3ED3.0	fEnglish	[1] a fine jade [2] the stripes (lines) on a jade
U+3ED4.0	fUTF8	㻔
U+3ED4.0	fCNS11643	3-4263
U+3ED4.0	fR/S	 96.8; 96.9
U+3ED4.0	fTotalStrokes	 13; 12
U+3ED4.0	fCangjie	MGHJM
U+3ED4.0	fFourCorner	12110
U+3ED4.0	fCantonese	do2
U+3ED4.0	fMandarin	duo3; rui4
U+3ED4.0	fEnglish	a kind of jade
U+3ED5.0	fUTF8	㻕
U+3ED5.0	fCNS11643	3-3D31
U+3ED5.0	fR/S	 96.8
U+3ED5.0	fTotalStrokes	 12
U+3ED5.0	fCangjie	MGSUU
U+3ED5.0	fFourCorner	17170
U+3ED5.0	fCantonese	gwat6
U+3ED5.0	fMandarin	jue2
U+3ED5.0	fEnglish	a kind of jade
U+3ED6.0	fUTF8	㻖
U+3ED6.0	fCNS11643	3-3D30
U+3ED6.0	fR/S	 96.8
U+3ED6.0	fTotalStrokes	 12
U+3ED6.0	fCangjie	MGLE
U+3ED6.0	fFourCorner	15130
U+3ED6.0	fCantonese	doi6
U+3ED6.0	fMandarin	dai4
U+3ED6.0	fEnglish	a kind of jade
U+3ED7.0	fUTF8	㻗
U+3ED7.0	fCNS11643	3-426C
U+3ED7.0	fGB	5-737A
U+3ED7.0	fR/S	 96.8
U+3ED7.0	fTotalStrokes	 13; 12
U+3ED7.0	fCangjie	LYMGI
U+3ED7.0	fFourCorner	11100
U+3ED7.0	fCantonese	pui3; faai2; pui5
U+3ED7.0	fMandarin	bei4
U+3ED7.0	fEnglish	same as &#x7432;
U+3ED8.0	fUTF8	㻘
U+3ED8.0	fGB	S-2232
U+3ED8.0	fR/S	 96.8
U+3ED8.0	fTotalStrokes	 12
U+3ED8.0	fCangjie	MGJLL
U+3ED8.0	fCantonese	sam2
U+3ED9.0	fUTF8	㻙
U+3ED9.0	fCNS11643	F-3659
U+3ED9.0	fGB	S-2233
U+3ED9.0	fR/S	 96.8
U+3ED9.0	fTotalStrokes	 12
U+3ED9.0	fCangjie	MGYIB
U+3ED9.0	fFourCorner	1012
U+3ED9.0	fCantonese	juk6
U+3EDA.0	fUTF8	㻚
U+3EDA.0	fCNS11643	F-3D3B
U+3EDA.0	fR/S	 96.8
U+3EDA.0	fTotalStrokes	 12
U+3EDA.0	fCangjie	MGBB
U+3EDA.0	fCantonese	pang4
U+3EDB.0	fUTF8	㻛
U+3EDB.0	fR/S	 96.8
U+3EDB.0	fTotalStrokes	 12
U+3EDB.0	fCangjie	MGAPH
U+3EDB.0	fFourCorner	1612
U+3EDB.0	fCantonese	ji6
U+3EDB.0	fEnglish	[Korean character]
U+3EDC.0	fUTF8	㻜
U+3EDC.0	fR/S	 96.8
U+3EDC.0	fTotalStrokes	 12
U+3EDC.0	fCangjie	MGOOO
U+3EDC.0	fCantonese	cung1
U+3EDC.0	fEnglish	[Korean character]
U+3EDD.0	fUTF8	㻝
U+3EDD.0	fCNS11643	4-3E59
U+3EDD.0	fGB	3-3F5E
U+3EDD.0	fR/S	 96.9; 96.8
U+3EDD.0	fTotalStrokes	 13; 12
U+3EDD.0	fCangjie	MGDLN
U+3EDD.0	fFourCorner	12100
U+3EDD.0	fCantonese	laat6
U+3EDD.0	fMandarin	la4
U+3EDD.0	fEnglish	a kind of jade
U+3EDE.0	fUTF8	㻞
U+3EDE.0	fCNS11643	3-4265
U+3EDE.0	fGB	5-4256
U+3EDE.0	fR/S	 96.9
U+3EDE.0	fTotalStrokes	 13
U+3EDE.0	fCangjie	MGISB
U+3EDE.0	fFourCorner	13120
U+3EDE.0	fHakka	  bin1; bien1
U+3EDE.0	fCantonese	ban1
U+3EDE.0	fMandarin	ban1; bian4; fen4; pin4
U+3EDE.0	fEnglish	inlaid as with jewels
U+3EDF.0	fUTF8	㻟
U+3EDF.0	fCNS11643	3-4269
U+3EDF.0	fR/S	 96.9
U+3EDF.0	fTotalStrokes	 13
U+3EDF.0	fCangjie	MGKMB
U+3EDF.0	fFourCorner	14120
U+3EDF.0	fCantonese	ceoi4
U+3EDF.0	fMandarin	sui2
U+3EDF.0	fEnglish	a kind of jade
U+3EE0.0	fUTF8	㻠
U+3EE0.0	fCNS11643	5-404E
U+3EE0.0	fR/S	 96.9
U+3EE0.0	fTotalStrokes	 13
U+3EE0.0	fCangjie	MGJCK
U+3EE0.0	fFourCorner	13130
U+3EE0.0	fCantonese	dat6; tou4
U+3EE0.0	fMandarin	tu2
U+3EE0.0	fEnglish	jade hanging on the sides of the cap in ancient times; used to plug the ears, to take; to receive
U+3EE1.0	fUTF8	㻡
U+3EE1.0	fCNS11643	3-4272
U+3EE1.0	fR/S	 96.9
U+3EE1.0	fTotalStrokes	 13
U+3EE1.0	fCangjie	MGPTD
U+3EE1.0	fFourCorner	32174
U+3EE1.0	fCantonese	sit3
U+3EE1.0	fMandarin	die2; jue2
U+3EE1.0	fEnglish	[1] interchangeable with &#x97A2; [2] corrupted form of &#x6E2B;
U+3EE2.0	fUTF8	㻢
U+3EE2.0	fCNS11643	F-4349
U+3EE2.0	fGB	S-2236
U+3EE2.0	fR/S	 96.9
U+3EE2.0	fTotalStrokes	 13
U+3EE2.0	fCangjie	MGYSD
U+3EE2.0	fFourCorner	1811
U+3EE2.0	fCantonese	si1
U+3EE3.0	fUTF8	㻣
U+3EE3.0	fCNS11643	F-4356
U+3EE3.0	fR/S	 96.9
U+3EE3.0	fTotalStrokes	 13
U+3EE3.0	fCangjie	MGTMV
U+3EE3.0	fCantonese	sam6
U+3EE4.0	fUTF8	㻤
U+3EE4.0	fR/S	 96.9
U+3EE4.0	fTotalStrokes	 13
U+3EE4.0	fCangjie	MGTPR
U+3EE4.0	fCantonese	gik1
U+3EE4.0	fEnglish	[Korean character]
U+3EE5.0	fUTF8	㻥
U+3EE5.0	fR/S	 96.9
U+3EE5.0	fTotalStrokes	 13
U+3EE5.0	fCangjie	MGTCW
U+3EE5.0	fCantonese	jau4
U+3EE5.0	fEnglish	[Korean character]
U+3EE6.0	fUTF8	㻦
U+3EE6.0	fR/S	 96.9
U+3EE6.0	fTotalStrokes	 13
U+3EE6.0	fCangjie	MGWLB
U+3EE6.0	fCantonese	jyu4
U+3EE6.0	fEnglish	[Korean character]
U+3EE7.0	fUTF8	㻧
U+3EE7.0	fCNS11643	3-4824
U+3EE7.0	fR/S	 96.10
U+3EE7.0	fTotalStrokes	 14
U+3EE7.0	fCangjie	MGNQD
U+3EE7.0	fFourCorner	15190
U+3EE7.0	fCantonese	do6
U+3EE7.0	fMandarin	duo4; he2
U+3EE7.0	fEnglish	a kind of jade
U+3EE8.0	fUTF8	㻨
U+3EE8.0	fCNS11643	7-2537
U+3EE8.0	fR/S	 96.10
U+3EE8.0	fTotalStrokes	 15; 14
U+3EE8.0	fCangjie	SIMGI
U+3EE8.0	fFourCorner	77100
U+3EE8.0	fCantonese	jyun2
U+3EE8.0	fEnglish	ine stone which is little less valuable than jade
U+3EE9.0	fUTF8	㻩
U+3EE9.0	fR/S	 96.10
U+3EE9.0	fTotalStrokes	 14
U+3EE9.0	fCangjie	MGTXC
U+3EE9.0	fCantonese	gim1
U+3EE9.0	fEnglish	[Korean character]
U+3EEA.0	fUTF8	㻪
U+3EEA.0	fGB	3-3F6F
U+3EEA.0	fR/S	 96.10
U+3EEA.0	fTotalStrokes	 14
U+3EEA.0	fCangjie	MGOGS
U+3EEA.0	fCantonese	zeon3; seoi6
U+3EEA.0	fMandarin	sui4
U+3EEA.0	fEnglish	[1] a kind of jade [2] same as &#x74BF; [3] same as &#x7487;
U+3EEB.0	fUTF8	㻫
U+3EEB.0	fCNS11643	4-4B22
U+3EEB.0	fGB	5-422B
U+3EEB.0	fR/S	 96.11
U+3EEB.0	fTotalStrokes	 15
U+3EEB.0	fCangjie	MGWTJ
U+3EEB.0	fFourCorner	16150
U+3EEB.0	fCantonese	bit1; bat1 (also)
U+3EEB.0	fMandarin	bi4
U+3EEB.0	fEnglish	same as ancient form of &#x73CC;
U+3EEC.0	fUTF8	㻬
U+3EEC.0	fCNS11643	4-4B21
U+3EEC.0	fGB	3-3F74
U+3EEC.0	fR/S	 96.11
U+3EEC.0	fTotalStrokes	 15
U+3EEC.0	fCangjie	MGMBS
U+3EEC.0	fFourCorner	11120
U+3EEC.0	fHakka	 tu1; ri2; ji2; shu1
U+3EEC.0	fCantonese	tou1; syu1
U+3EEC.0	fMandarin	shu1; tu2
U+3EEC.0	fEnglish	a kind of jasper
U+3EED.0	fUTF8	㻭
U+3EED.0	fCNS11643	3-4D37
U+3EED.0	fR/S	 96.11
U+3EED.0	fTotalStrokes	 15
U+3EED.0	fCangjie	MGYIJ
U+3EED.0	fFourCorner	10140
U+3EED.0	fCantonese	sat1
U+3EED.0	fMandarin	se4; ze2
U+3EED.0	fEnglish	same as &#x74B1;
U+3EEE.0	fUTF8	㻮
U+3EEE.0	fCNS11643	3-4D3C
U+3EEE.0	fR/S	 96.11
U+3EEE.0	fTotalStrokes	 15
U+3EEE.0	fCangjie	MGBOF
U+3EEE.0	fFourCorner	17190
U+3EEE.0	fCantonese	caan3
U+3EEE.0	fMandarin	can4
U+3EEE.0	fEnglish	same as non-classical form of &#x74A8;
U+3EEF.0	fUTF8	㻯
U+3EEF.0	fCNS11643	3-4D33
U+3EEF.0	fGB	5-425F
U+3EEF.0	fR/S	 96.11
U+3EEF.0	fTotalStrokes	 15
U+3EEF.0	fCangjie	MGYPD
U+3EEF.0	fFourCorner	11140
U+3EEF.0	fCantonese	tou1; tou4; tuk1
U+3EEF.0	fMandarin	tu2
U+3EEF.0	fEnglish	same as non-classical form of &#x3EEC;
U+3EF0.0	fUTF8	㻰
U+3EF0.0	fCNS11643	3-4D38
U+3EF0.0	fGB	5-425D
U+3EF0.0	fR/S	 96.11
U+3EF0.0	fTotalStrokes	 15
U+3EF0.0	fCangjie	MGQJS
U+3EF0.0	fFourCorner	15170
U+3EF0.0	fCantonese	wai6
U+3EF0.0	fMandarin	wei4; mian3; re4; yu4
U+3EF0.0	fEnglish	decorated jade or ornament on a scabbard
U+3EF1.0	fUTF8	㻱
U+3EF1.0	fCNS11643	3-4D34
U+3EF1.0	fR/S	 96.11
U+3EF1.0	fTotalStrokes	 15
U+3EF1.0	fCangjie	MGYHJ
U+3EF1.0	fFourCorner	17130
U+3EF1.0	fCantonese	fung4; zeon1
U+3EF1.0	fEnglish	[1] circle; revolve round; surround; encase [2] same as &#x74A1;
U+3EF2.0	fUTF8	㻲
U+3EF2.0	fCNS11643	3-4D32
U+3EF2.0	fR/S	 96.11
U+3EF2.0	fTotalStrokes	 15
U+3EF2.0	fCangjie	MGLLV
U+3EF2.0	fFourCorner	15140
U+3EF2.0	fCantonese	leoi5
U+3EF2.0	fMandarin	liu2; l3
U+3EF2.0	fEnglish	a rite or service for beginning of Autumn (date marking the beginning of one of the 24 seasonal periods in a year, falling normally on August 8 or 9)
U+3EF3.0	fUTF8	㻳
U+3EF3.0	fGB	S-2237
U+3EF3.0	fR/S	 96.11
U+3EF3.0	fTotalStrokes	 15
U+3EF3.0	fCangjie	MGTHN
U+3EF3.0	fCantonese	lei6
U+3EF4.0	fUTF8	㻴
U+3EF4.0	fR/S	 96.11
U+3EF4.0	fTotalStrokes	 15
U+3EF4.0	fCangjie	MGAWE
U+3EF4.0	fFourCorner	1614
U+3EF4.0	fCantonese	maan6
U+3EF4.0	fEnglish	[Korean character]
U+3EF5.0	fUTF8	㻵
U+3EF5.0	fCNS11643	3-5168
U+3EF5.0	fR/S	 96.12
U+3EF5.0	fTotalStrokes	 16
U+3EF5.0	fCangjie	MGSND
U+3EF5.0	fFourCorner	17140
U+3EF5.0	fCantonese	zaan6
U+3EF5.0	fMandarin	zhan4
U+3EF5.0	fEnglish	a kind of jade
U+3EF6.0	fUTF8	㻶
U+3EF6.0	fCNS11643	5-555D
U+3EF6.0	fR/S	 96.12
U+3EF6.0	fTotalStrokes	 16
U+3EF6.0	fCangjie	MGHLQ
U+3EF6.0	fFourCorner	18150
U+3EF6.0	fCantonese	bat1
U+3EF6.0	fMandarin	bi3; bi4
U+3EF6.0	fEnglish	[1] a jade tube with white and nature colors [2] jade with the colour of nature (green; blue; black) [3] sapphire
U+3EF7.0	fUTF8	㻷
U+3EF7.0	fCNS11643	4-5137
U+3EF7.0	fGB	5-4263
U+3EF7.0	fR/S	 96.12
U+3EF7.0	fTotalStrokes	 16
U+3EF7.0	fCangjie	MGDBB
U+3EF7.0	fFourCorner	14190
U+3EF7.0	fCantonese	gik1; kak3
U+3EF7.0	fMandarin	ji2
U+3EF7.0	fEnglish	[1] the name of a place (be famous for fine jade) [2] a kind of jade
U+3EF8.0	fUTF8	㻸
U+3EF8.0	fCNS11643	3-5169
U+3EF8.0	fGB	5-4264
U+3EF8.0	fR/S	 96.12
U+3EF8.0	fTotalStrokes	 16
U+3EF8.0	fCangjie	MGMUA
U+3EF8.0	fFourCorner	15160
U+3EF8.0	fCantonese	zam1; caam4; saam4
U+3EF8.0	fMandarin	zen1; cen2; jin4; xin2
U+3EF8.0	fEnglish	jade like stone
U+3EF9.0	fUTF8	㻹
U+3EF9.0	fCNS11643	3-554A
U+3EF9.0	fR/S	 96.12
U+3EF9.0	fTotalStrokes	 17
U+3EF9.0	fCangjie	MOMGI
U+3EF9.0	fFourCorner	11100
U+3EF9.0	fCantonese	hyun1
U+3EF9.0	fMandarin	xuan1
U+3EF9.0	fEnglish	[1] crack; crevice [2] spare time; leisure
U+3EFA.0	fUTF8	㻺
U+3EFA.0	fCNS11643	4-566F
U+3EFA.0	fGB	3-3042
U+3EFA.0	fR/S	 96.12
U+3EFA.0	fTotalStrokes	 17; 16
U+3EFA.0	fCangjie	MHDI
U+3EFA.0	fFourCorner	71210
U+3EFA.0	fCantonese	lik6
U+3EFA.0	fMandarin	li4; lie4
U+3EFA.0	fEnglish	a kind of jade
U+3EFB.0	fUTF8	㻻
U+3EFB.0	fCNS11643	F-557A
U+3EFB.0	fR/S	 96.12
U+3EFB.0	fTotalStrokes	 16
U+3EFB.0	fCangjie	MGYDK
U+3EFB.0	fFourCorner	1814
U+3EFB.0	fCantonese	deon1
U+3EFC.0	fUTF8	㻼
U+3EFC.0	fCNS11643	F-5622
U+3EFC.0	fGB	7-235F
U+3EFC.0	fR/S	 96.12
U+3EFC.0	fTotalStrokes	 16
U+3EFC.0	fCangjie	MGMWJ
U+3EFC.0	fCantonese	taam4
U+3EFD.0	fUTF8	㻽
U+3EFD.0	fCNS11643	3-554E
U+3EFD.0	fR/S	 96.13
U+3EFD.0	fTotalStrokes	 17
U+3EFD.0	fCangjie	MGOGS
U+3EFD.0	fFourCorner	10120
U+3EFD.0	fCantonese	zeoi6; seoi6
U+3EFD.0	fMandarin	sui4; xuan2
U+3EFD.0	fEnglish	[1] same as &#x3EEA; [2] same as &#x74BF;, &#x7487;
U+3EFE.0	fUTF8	㻾
U+3EFE.0	fCNS11643	4-5B69
U+3EFE.0	fGB	3-402C
U+3EFE.0	fR/S	 96.13
U+3EFE.0	fTotalStrokes	 18
U+3EFE.0	fCangjie	YVGI
U+3EFE.0	fFourCorner	00100
U+3EFE.0	fCantonese	jung1; zung2
U+3EFE.0	fMandarin	yong1
U+3EFE.0	fEnglish	[1] jade articles [2] fine stone which is little less valuable than jade
U+3EFF.0	fUTF8	㻿
U+3EFF.0	fCNS11643	3-554C
U+3EFF.0	fR/S	 96.13
U+3EFF.0	fTotalStrokes	 17
U+3EFF.0	fCangjie	MGWLI
U+3EFF.0	fFourCorner	16120
U+3EFF.0	fCantonese	suk6
U+3EFF.0	fMandarin	shu3
U+3EFF.0	fEnglish	a kind of jade
U+3F00.0	fUTF8	㼀
U+3F00.0	fR/S	 96.13
U+3F00.0	fTotalStrokes	 17
U+3F00.0	fCangjie	MGYGQ
U+3F00.0	fCantonese	daat6
U+3F00.0	fEnglish	[Korean character]
U+3F01.0	fUTF8	㼁
U+3F01.0	fCNS11643	F-5B2D
U+3F01.0	fR/S	 96.13
U+3F01.0	fTotalStrokes	 17
U+3F01.0	fCangjie	MGTGI
U+3F01.0	fCantonese	ji6
U+3F02.0	fUTF8	㼂
U+3F02.0	fCNS11643	3-5831
U+3F02.0	fR/S	 96.13
U+3F02.0	fTotalStrokes	 18
U+3F02.0	fCangjie	HBMGI
U+3F02.0	fFourCorner	77100
U+3F02.0	fCantonese	man6; waan6
U+3F02.0	fMandarin	wen4; e2; yuan3
U+3F02.0	fEnglish	same as &#x74BA;
U+3F03.0	fUTF8	㼃
U+3F03.0	fR/S	 96.15
U+3F03.0	fTotalStrokes	 19
U+3F03.0	fCangjie	MGVVV
U+3F03.0	fCantonese	laap6
U+3F03.0	fEnglish	[Korean character]
U+3F04.0	fUTF8	㼄
U+3F04.0	fCNS11643	4-602A
U+3F04.0	fR/S	 96.15
U+3F04.0	fTotalStrokes	 19
U+3F04.0	fCangjie	MGVIW
U+3F04.0	fFourCorner	12150
U+3F04.0	fCantonese	gei1; hei3; kaau5; zin5
U+3F04.0	fMandarin	ji1
U+3F04.0	fEnglish	same as &#x74A3;
U+3F05.0	fUTF8	㼅
U+3F05.0	fCNS11643	F-6278
U+3F05.0	fR/S	 96.15
U+3F05.0	fTotalStrokes	 19
U+3F05.0	fCangjie	MGITC
U+3F05.0	fCantonese	gwong2; gong2 (also)
U+3F06.0	fUTF8	㼆
U+3F06.0	fCNS11643	F-627B
U+3F06.0	fGB	3-3F6C
U+3F06.0	fR/S	 96.15
U+3F06.0	fTotalStrokes	 19
U+3F06.0	fCangjie	MGFFI
U+3F06.0	fFourCorner	1911
U+3F06.0	fCantonese	jing4
U+3F07.0	fUTF8	㼇
U+3F07.0	fCNS11643	3-5C61
U+3F07.0	fR/S	 96.16
U+3F07.0	fTotalStrokes	 20
U+3F07.0	fCangjie	MGUOS
U+3F07.0	fFourCorner	12120
U+3F07.0	fCantonese	king4; kaai3
U+3F07.0	fMandarin	qiong2
U+3F07.0	fEnglish	[1] same as non-classical form of &#x74D7; [2] same as &#x74CA;
U+3F08.0	fUTF8	㼈
U+3F08.0	fGB	3-3F57
U+3F08.0	fR/S	 96.19
U+3F08.0	fTotalStrokes	 23
U+3F08.0	fCangjie	MGWLG
U+3F08.0	fCantonese	lo4; leon6
U+3F08.0	fMandarin	luo2
U+3F09.0	fUTF8	㼉
U+3F09.0	fCNS11643	4-2A39
U+3F09.0	fGB	3-5C6A
U+3F09.0	fR/S	 97.4
U+3F09.0	fTotalStrokes	 9
U+3F09.0	fCangjie	HOLBU
U+3F09.0	fFourCorner	74230
U+3F09.0	fCantonese	zam6
U+3F09.0	fMandarin	zhen4; yin4
U+3F09.0	fEnglish	green colored melons
U+3F0A.0	fUTF8	㼊
U+3F0A.0	fCNS11643	4-2A3A
U+3F0A.0	fGB	3-5C69
U+3F0A.0	fR/S	 97.4
U+3F0A.0	fTotalStrokes	 9
U+3F0A.0	fCangjie	PUHVO
U+3F0A.0	fFourCorner	52710
U+3F0A.0	fCantonese	tyun4
U+3F0A.0	fMandarin	tun2
U+3F0A.0	fEnglish	a kind of melon
U+3F0B.0	fUTF8	㼋
U+3F0B.0	fCNS11643	5-2E62
U+3F0B.0	fR/S	 97.5
U+3F0B.0	fTotalStrokes	 10
U+3F0B.0	fCangjie	JRHVO
U+3F0B.0	fFourCorner	42630
U+3F0B.0	fCantonese	gu1; gwu1 (also)
U+3F0B.0	fMandarin	gu1; ru3; gu3; jiu3; mou2
U+3F0B.0	fEnglish	[1] the cucumber [2] dried (preserved) vegetable
U+3F0C.0	fUTF8	㼌
U+3F0C.0	fCNS11643	4-2E2B
U+3F0C.0	fGB	3-5C6B
U+3F0C.0	fR/S	 97.5
U+3F0C.0	fTotalStrokes	 10
U+3F0C.0	fCangjie	HOHVO
U+3F0C.0	fFourCorner	72230
U+3F0C.0	fCantonese	jyu5; zyu5
U+3F0C.0	fMandarin	yu3
U+3F0C.0	fEnglish	[1] lot of melons growing but the root is weak and feeble [2] wearied and worn
U+3F0D.0	fUTF8	㼍
U+3F0D.0	fCNS11643	4-3335
U+3F0D.0	fGB	3-6032
U+3F0D.0	fR/S	 97.6
U+3F0D.0	fTotalStrokes	 11
U+3F0D.0	fCangjie	QDHVO
U+3F0D.0	fFourCorner	52930
U+3F0D.0	fCantonese	leoi5
U+3F0D.0	fMandarin	lei3
U+3F0D.0	fEnglish	[1] pulp of a melon [2] a kind of melon
U+3F0E.0	fUTF8	㼎
U+3F0E.0	fCNS11643	4-3334
U+3F0E.0	fGB	3-5C6D
U+3F0E.0	fR/S	 97.6
U+3F0E.0	fTotalStrokes	 11
U+3F0E.0	fCangjie	HOYCK
U+3F0E.0	fFourCorner	70214
U+3F0E.0	fCantonese	bok6
U+3F0E.0	fMandarin	bo2; bei4; bao2; ke3
U+3F0E.0	fEnglish	a small melon
U+3F0F.0	fUTF8	㼏
U+3F0F.0	fCNS11643	4-387E
U+3F0F.0	fGB	3-5C6E
U+3F0F.0	fR/S	 97.7
U+3F0F.0	fTotalStrokes	 12
U+3F0F.0	fCangjie	BVHVO
U+3F0F.0	fFourCorner	22430
U+3F0F.0	fCantonese	neoi5; leoi5 (also); nei6
U+3F0F.0	fMandarin	nei3
U+3F0F.0	fEnglish	a bad; a wounded; an injured melon
U+3F10.0	fUTF8	㼐
U+3F10.0	fCNS11643	4-4470
U+3F10.0	fGB	3-5C71
U+3F10.0	fR/S	 97.9
U+3F10.0	fTotalStrokes	 14
U+3F10.0	fCangjie	IBHVO
U+3F10.0	fFourCorner	32210
U+3F10.0	fCantonese	pin4
U+3F10.0	fMandarin	pian2
U+3F10.0	fEnglish	a kind of white melon
U+3F11.0	fUTF8	㼑
U+3F11.0	fCNS11643	4-4471
U+3F11.0	fGB	3-5C6F
U+3F11.0	fR/S	 97.9
U+3F11.0	fTotalStrokes	 14
U+3F11.0	fCangjie	DWHVO
U+3F11.0	fFourCorner	52930
U+3F11.0	fCantonese	lin6
U+3F11.0	fMandarin	lian4; luan2
U+3F11.0	fEnglish	pulp of a melon
U+3F12.0	fUTF8	㼒
U+3F12.0	fCNS11643	4-446F
U+3F12.0	fGB	3-5C68
U+3F12.0	fR/S	 97.9
U+3F12.0	fTotalStrokes	 14
U+3F12.0	fCangjie	HOAMH
U+3F12.0	fFourCorner	76230
U+3F12.0	fCantonese	tong2
U+3F12.0	fMandarin	tang3; qiu3
U+3F12.0	fEnglish	[1] a kind of big melon [2] a long melon
U+3F13.0	fUTF8	㼓
U+3F13.0	fCNS11643	4-4B23
U+3F13.0	fGB	3-5C72
U+3F13.0	fR/S	 97.10
U+3F13.0	fTotalStrokes	 15
U+3F13.0	fCangjie	HOTXC
U+3F13.0	fFourCorner	78230
U+3F13.0	fCantonese	lim4
U+3F13.0	fMandarin	lian3; lian2; lian4
U+3F13.0	fEnglish	[1] melon seeds [2] a kind of melon
U+3F14.0	fUTF8	㼔
U+3F14.0	fCNS11643	4-4B24
U+3F14.0	fGB	3-5C70
U+3F14.0	fR/S	 97.10
U+3F14.0	fTotalStrokes	 15
U+3F14.0	fCangjie	WTHVO
U+3F14.0	fFourCorner	62130
U+3F14.0	fCantonese	wan1; waan1; wan4
U+3F14.0	fMandarin	wen1
U+3F14.0	fEnglish	a kind of melon
U+3F15.0	fUTF8	㼕
U+3F15.0	fCNS11643	4-5B6C
U+3F15.0	fGB	3-5C6C
U+3F15.0	fR/S	 97.13
U+3F15.0	fTotalStrokes	 18
U+3F15.0	fCangjie	FWHVO
U+3F15.0	fFourCorner	92630
U+3F15.0	fCantonese	dong1
U+3F15.0	fMandarin	dang1
U+3F15.0	fEnglish	pulp of a melon
U+3F16.0	fUTF8	㼖
U+3F16.0	fCNS11643	4-6D5A
U+3F16.0	fGB	3-5C73
U+3F16.0	fR/S	 97.21
U+3F16.0	fTotalStrokes	 26
U+3F16.0	fCangjie	HOVNI
U+3F16.0	fFourCorner	77230
U+3F16.0	fCantonese	lai6; laai5
U+3F16.0	fMandarin	li4
U+3F16.0	fEnglish	gourd used as a ladle or dipper
U+3F17.0	fUTF8	㼗
U+3F17.0	fCNS11643	4-245E
U+3F17.0	fGB	3-463C
U+3F17.0	fR/S	 98.2
U+3F17.0	fTotalStrokes	 7
U+3F17.0	fCangjie	MNMVN
U+3F17.0	fFourCorner	11210
U+3F17.0	fCantonese	ting4; ting5
U+3F17.0	fMandarin	ting2; ding3
U+3F17.0	fEnglish	 brick
U+3F18.0	fUTF8	㼘
U+3F18.0	fCNS11643	5-2732
U+3F18.0	fR/S	 98.3
U+3F18.0	fTotalStrokes	 8
U+3F18.0	fCangjie	UMVN
U+3F18.0	fFourCorner	21710
U+3F18.0	fCantonese	ngaa5; aa5 (also)
U+3F18.0	fMandarin	wa3
U+3F18.0	fEnglish	high and lofty (of a mountain)
U+3F19.0	fUTF8	㼙
U+3F19.0	fCNS11643	3-2A34
U+3F19.0	fR/S	 98.3
U+3F19.0	fTotalStrokes	 8
U+3F19.0	fCangjie	LBMVN
U+3F19.0	fFourCorner	41210
U+3F19.0	fCantonese	zau3; cuk1; zaau3
U+3F19.0	fMandarin	zhou4
U+3F19.0	fEnglish	same as ancient form of &#x7503;
U+3F1A.0	fUTF8	㼚
U+3F1A.0	fCNS11643	3-2E23
U+3F1A.0	fGB	5-4849
U+3F1A.0	fR/S	 98.4
U+3F1A.0	fTotalStrokes	 9
U+3F1A.0	fCangjie	YNMVN
U+3F1A.0	fFourCorner	01210
U+3F1A.0	fCantonese	gong1
U+3F1A.0	fMandarin	gang1; gang3
U+3F1A.0	fEnglish	[1] earthware; pottery (basin; pot; bowl; cistern; crock) [2] interchangeable with &#x7F38;
U+3F1B.0	fUTF8	㼛
U+3F1B.0	fCNS11643	4-2A3D
U+3F1B.0	fR/S	 98.4
U+3F1B.0	fTotalStrokes	 9
U+3F1B.0	fCangjie	MTMVN
U+3F1B.0	fFourCorner	11410
U+3F1B.0	fCantonese	jing4; zing4
U+3F1B.0	fMandarin	xing2; jing1
U+3F1B.0	fEnglish	same as &#x9203;
U+3F1C.0	fUTF8	㼜
U+3F1C.0	fCNS11643	3-324A
U+3F1C.0	fGB	5-484D
U+3F1C.0	fR/S	 98.5
U+3F1C.0	fTotalStrokes	 10
U+3F1C.0	fCangjie	LKMVN
U+3F1C.0	fFourCorner	50710
U+3F1C.0	fCantonese	ong3; ngong3 (also); ong2 (also); ngong2 (also); on1 (also); ngon1 (also)
U+3F1C.0	fMandarin	ang4; pou2
U+3F1C.0	fEnglish	same as &#x76CE;
U+3F1D.0	fUTF8	㼝
U+3F1D.0	fCNS11643	3-324B
U+3F1D.0	fR/S	 98.5
U+3F1D.0	fTotalStrokes	 10
U+3F1D.0	fCangjie	NUMVN
U+3F1D.0	fFourCorner	27710
U+3F1D.0	fCantonese	wun2
U+3F1D.0	fMandarin	wan3; fan4
U+3F1D.0	fEnglish	[1] same as &#x76CC; [2] variant of &#x7897;
U+3F1E.0	fUTF8	㼞
U+3F1E.0	fCNS11643	4-2E2D
U+3F1E.0	fGB	3-4642
U+3F1E.0	fR/S	 98.5
U+3F1E.0	fTotalStrokes	 10
U+3F1E.0	fCangjie	MJMVN
U+3F1E.0	fFourCorner	11410
U+3F1E.0	fCantonese	pang3; bang1; pang4; pung3
U+3F1E.0	fMandarin	peng4
U+3F1E.0	fEnglish	[1] jar with a small mouth [2] earthen jar [3] a jar for the ashes of the dead
U+3F1F.0	fUTF8	㼟
U+3F1F.0	fCNS11643	3-3249
U+3F1F.0	fGB	5-5B52
U+3F1F.0	fR/S	 98.5
U+3F1F.0	fTotalStrokes	 10
U+3F1F.0	fCangjie	HAMVN
U+3F1F.0	fFourCorner	21610
U+3F1F.0	fCantonese	baak6
U+3F1F.0	fMandarin	bai2; bo2
U+3F1F.0	fEnglish	[1] a tiled house [2] brick wall of a well
U+3F20.0	fUTF8	㼠
U+3F20.0	fCNS11643	4-2E2C
U+3F20.0	fGB	3-4645
U+3F20.0	fR/S	 98.5
U+3F20.0	fTotalStrokes	 10
U+3F20.0	fCangjie	JPMVN
U+3F20.0	fFourCorner	31710
U+3F20.0	fCantonese	to4
U+3F20.0	fMandarin	tuo2
U+3F20.0	fEnglish	[1] a crock with narrow opening [2] an earthen jar
U+3F21.0	fUTF8	㼡
U+3F21.0	fCNS11643	4-333B
U+3F21.0	fGB	3-464D
U+3F21.0	fR/S	 98.6
U+3F21.0	fTotalStrokes	 11
U+3F21.0	fCangjie	HDMVN
U+3F21.0	fFourCorner	21910
U+3F21.0	fCantonese	syu4
U+3F21.0	fMandarin	shu1
U+3F21.0	fEnglish	[1] a small jar with a small mouth [2] bottle; pitcher; jug etc.
U+3F22.0	fUTF8	㼢
U+3F22.0	fCNS11643	4-3921
U+3F22.0	fGB	3-464F
U+3F22.0	fR/S	 98.7; 98.6
U+3F22.0	fTotalStrokes	 12; 11
U+3F22.0	fCangjie	SLMVN
U+3F22.0	fFourCorner	71710
U+3F22.0	fCantonese	ji4; gei1; zi1; zi4
U+3F22.0	fMandarin	yi2; e3
U+3F22.0	fEnglish	[1] bricks [2] same as &#x74F5;
U+3F23.0	fUTF8	㼣
U+3F23.0	fCNS11643	4-333A
U+3F23.0	fGB	3-4649
U+3F23.0	fR/S	 98.6
U+3F23.0	fTotalStrokes	 11
U+3F23.0	fCangjie	MAMVN
U+3F23.0	fFourCorner	11610
U+3F23.0	fCantonese	baak3
U+3F23.0	fMandarin	bo2; bai3
U+3F23.0	fEnglish	[1] bricks of the well; brick wall of a well [2] build a well [3] construct with bricks
U+3F24.0	fUTF8	㼤
U+3F24.0	fCNS11643	4-3338
U+3F24.0	fGB	3-4646
U+3F24.0	fR/S	 98.6
U+3F24.0	fTotalStrokes	 11
U+3F24.0	fCangjie	QHMVN
U+3F24.0	fFourCorner	57710
U+3F24.0	fCantonese	kit3
U+3F24.0	fMandarin	qi4; qie4; ya4
U+3F24.0	fEnglish	earthenware (basin; pot; bowl; crock with a narrow opening)
U+3F25.0	fUTF8	㼥
U+3F25.0	fCNS11643	4-3339
U+3F25.0	fGB	3-4647
U+3F25.0	fR/S	 98.6
U+3F25.0	fTotalStrokes	 11
U+3F25.0	fCangjie	JSMVN
U+3F25.0	fFourCorner	41210
U+3F25.0	fCantonese	tau2
U+3F25.0	fMandarin	kao3; tou3; chu2
U+3F25.0	fEnglish	[1] bottle [2] earthenware
U+3F26.0	fUTF8	㼦
U+3F26.0	fCNS11643	4-3337
U+3F26.0	fGB	3-4648
U+3F26.0	fR/S	 98.6
U+3F26.0	fTotalStrokes	 11
U+3F26.0	fCangjie	MNMVN
U+3F26.0	fFourCorner	17710
U+3F26.0	fCantonese	gung2
U+3F26.0	fMandarin	gong3
U+3F26.0	fEnglish	earthenware; pottery (bottle; pitcher; jug; vase; crock with a narrow opening)
U+3F27.0	fUTF8	㼧
U+3F27.0	fCNS11643	4-3923
U+3F27.0	fGB	3-4653
U+3F27.0	fR/S	 98.7
U+3F27.0	fTotalStrokes	 12
U+3F27.0	fCangjie	NBMVN
U+3F27.0	fFourCorner	11210
U+3F27.0	fHakka	  tung2
U+3F27.0	fCantonese	tung4
U+3F27.0	fMandarin	tong2
U+3F27.0	fEnglish	the upper tiles on a roof
U+3F28.0	fUTF8	㼨
U+3F28.0	fCNS11643	4-3926
U+3F28.0	fGB	3-4652
U+3F28.0	fR/S	 98.7
U+3F28.0	fTotalStrokes	 12
U+3F28.0	fCangjie	ORMVN
U+3F28.0	fFourCorner	81610
U+3F28.0	fHakka	  ham2
U+3F28.0	fCantonese	ham4; haam6
U+3F28.0	fMandarin	han2
U+3F28.0	fEnglish	 concave tile for roofing
U+3F29.0	fUTF8	㼩
U+3F29.0	fCNS11643	4-333C
U+3F29.0	fGB	3-464A
U+3F29.0	fR/S	 98.6; 98.7
U+3F29.0	fTotalStrokes	 11; 12
U+3F29.0	fCangjie	ISMVN
U+3F29.0	fFourCorner	51210
U+3F29.0	fCantonese	sing4
U+3F29.0	fMandarin	cheng2; sheng4
U+3F29.0	fEnglish	[1] concave channels of tiling [2] a long-necked jar [3] utensil; instrument; implement
U+3F2A.0	fUTF8	㼪
U+3F2A.0	fCNS11643	4-3924
U+3F2A.0	fGB	3-464B
U+3F2A.0	fR/S	 98.7
U+3F2A.0	fTotalStrokes	 12
U+3F2A.0	fCangjie	KOMVN
U+3F2A.0	fFourCorner	41010
U+3F2A.0	fCantonese	zip3; gap3; tiu5; zaau1
U+3F2A.0	fMandarin	jie2; jia2
U+3F2A.0	fEnglish	[1] fragments of a tile haven't t been kiln-dried completely [2] tiles overlapping
U+3F2B.0	fUTF8	㼫
U+3F2B.0	fCNS11643	4-3E62
U+3F2B.0	fGB	3-4657
U+3F2B.0	fR/S	 98.8
U+3F2B.0	fTotalStrokes	 13
U+3F2B.0	fCangjie	WDMVN
U+3F2B.0	fFourCorner	61910
U+3F2B.0	fCantonese	wun6; fun1
U+3F2B.0	fMandarin	huan3; huan4; hua4
U+3F2B.0	fEnglish	[1] earthenware or pottery with big opening [2] wine utensil (containers) with big opening
U+3F2C.0	fUTF8	㼬
U+3F2C.0	fCNS11643	4-3E61
U+3F2C.0	fGB	5-4850
U+3F2C.0	fR/S	 98.8
U+3F2C.0	fTotalStrokes	 13
U+3F2C.0	fCangjie	GJMVN
U+3F2C.0	fFourCorner	41410
U+3F2C.0	fCantonese	hang6
U+3F2C.0	fMandarin	xing4
U+3F2C.0	fEnglish	earthenware (a basin; a pot; a bowl; a crock etc.)
U+3F2D.0	fUTF8	㼭
U+3F2D.0	fCNS11643	4-3E5D
U+3F2D.0	fGB	3-4654
U+3F2D.0	fR/S	 98.8
U+3F2D.0	fTotalStrokes	 13
U+3F2D.0	fCangjie	GRMVN
U+3F2D.0	fFourCorner	41710
U+3F2D.0	fHakka	 diam5; tiam5; tiam6
U+3F2D.0	fCantonese	dim6; dim3
U+3F2D.0	fMandarin	dian4; niao3
U+3F2D.0	fEnglish	[1] steady byplacing something underneath [2] prop
U+3F2E.0	fUTF8	㼮
U+3F2E.0	fCNS11643	4-3E60
U+3F2E.0	fGB	3-4655
U+3F2E.0	fR/S	 98.8
U+3F2E.0	fTotalStrokes	 13
U+3F2E.0	fCangjie	JVMVN
U+3F2E.0	fFourCorner	51410
U+3F2E.0	fCantonese	caai1
U+3F2E.0	fMandarin	chai1; mai2; cha1; qi4
U+3F2E.0	fEnglish	[1] rub out the filth with broken tiles [2] brickwork of a well [3] repair a well [4] lay bricks
U+3F2F.0	fUTF8	㼯
U+3F2F.0	fCNS11643	3-4276
U+3F2F.0	fGB	5-484B
U+3F2F.0	fR/S	 98.8
U+3F2F.0	fTotalStrokes	 13
U+3F2F.0	fCangjie	DWMVN
U+3F2F.0	fFourCorner	51910
U+3F2F.0	fCantonese	dung3; dung6
U+3F2F.0	fMandarin	dong4
U+3F2F.0	fEnglish	earthenware (jar with a small mouth)
U+3F30.0	fUTF8	㼰
U+3F30.0	fCNS11643	4-3E63
U+3F30.0	fGB	3-465A
U+3F30.0	fR/S	 98.8
U+3F30.0	fTotalStrokes	 13
U+3F30.0	fCangjie	HJMVN
U+3F30.0	fFourCorner	21410
U+3F30.0	fCantonese	pei4; be1; maa3; paai6; sit3
U+3F30.0	fMandarin	pi2; e2; bi4
U+3F30.0	fEnglish	an earthenware jar with a small mouth and two or four ears, used to store water for fire prevention in ancient times
U+3F31.0	fUTF8	㼱
U+3F31.0	fCNS11643	5-4748
U+3F31.0	fGB	3-465D
U+3F31.0	fR/S	 98.9; 98.8
U+3F31.0	fTotalStrokes	 14; 13
U+3F31.0	fCangjie	LPBCN
U+3F31.0	fFourCorner	11710
U+3F31.0	fCantonese	jyun5; zeon3; zyun5
U+3F31.0	fMandarin	jun4; ruan3
U+3F31.0	fEnglish	[1] manage or regulate the leather material [2] soft [3] hunting clothes (leather trousers)
U+3F32.0	fUTF8	㼲
U+3F32.0	fCNS11643	4-4473
U+3F32.0	fGB	3-465C
U+3F32.0	fR/S	 98.9
U+3F32.0	fTotalStrokes	 14
U+3F32.0	fCangjie	MKMVN
U+3F32.0	fFourCorner	11410
U+3F32.0	fCantonese	lip6; lit6
U+3F32.0	fMandarin	lie4
U+3F32.0	fEnglish	[1] the sound of broken tiles [2] thin tiles
U+3F33.0	fUTF8	㼳
U+3F33.0	fCNS11643	4-4476
U+3F33.0	fGB	3-465E
U+3F33.0	fR/S	 98.9
U+3F33.0	fTotalStrokes	 14
U+3F33.0	fCangjie	FUMVN
U+3F33.0	fFourCorner	91617
U+3F33.0	fCantonese	saang2; sang2; zip3
U+3F33.0	fMandarin	sheng3
U+3F33.0	fEnglish	earthenware (basin; pot; bowl; crock etc.)
U+3F34.0	fUTF8	㼴
U+3F34.0	fCNS11643	4-4474
U+3F34.0	fGB	3-4660
U+3F34.0	fR/S	 98.9
U+3F34.0	fTotalStrokes	 14
U+3F34.0	fCangjie	WBMVN
U+3F34.0	fFourCorner	61417
U+3F34.0	fCantonese	ngau5; au5 (also); ngaau5
U+3F34.0	fMandarin	ou3
U+3F34.0	fEnglish	 earthen vessel; basin; pot; jar
U+3F35.0	fUTF8	㼵
U+3F35.0	fCNS11643	4-4478
U+3F35.0	fGB	3-465F
U+3F35.0	fR/S	 98.9
U+3F35.0	fTotalStrokes	 14
U+3F35.0	fCangjie	AOMVN
U+3F35.0	fFourCorner	61800
U+3F35.0	fCantonese	tai5; daai6; taai4; zit3
U+3F35.0	fMandarin	ti2; di4
U+3F35.0	fEnglish	 small basin; bowl; cup; pot; jar
U+3F36.0	fUTF8	㼶
U+3F36.0	fCNS11643	4-4477
U+3F36.0	fGB	3-4663
U+3F36.0	fR/S	 98.9
U+3F36.0	fTotalStrokes	 14
U+3F36.0	fCangjie	ONMVN
U+3F36.0	fFourCorner	81210
U+3F36.0	fCantonese	jyu4; naau6; zyu4
U+3F36.0	fMandarin	yu2; you2
U+3F36.0	fEnglish	earthenware (basin; pot; bowl; crock etc.)
U+3F37.0	fUTF8	㼷
U+3F37.0	fCNS11643	4-4475
U+3F37.0	fGB	3-4661
U+3F37.0	fR/S	 98.9
U+3F37.0	fTotalStrokes	 14
U+3F37.0	fCangjie	UBMVN
U+3F37.0	fFourCorner	21210
U+3F37.0	fCantonese	zyun1; cyun4
U+3F37.0	fMandarin	chuan2; zhuan1
U+3F37.0	fEnglish	[1] basin [2] same as &#x750E;
U+3F38.0	fUTF8	㼸
U+3F38.0	fCNS11643	4-4B25
U+3F38.0	fGB	3-4669
U+3F38.0	fR/S	 98.10
U+3F38.0	fTotalStrokes	 15
U+3F38.0	fCangjie	JRMVN
U+3F38.0	fFourCorner	31610
U+3F38.0	fCantonese	jung4; zung4
U+3F38.0	fMandarin	rong2; yong2
U+3F38.0	fEnglish	earthenware (basin; pot; bowl; crock etc.)
U+3F39.0	fUTF8	㼹
U+3F39.0	fCNS11643	4-4B27
U+3F39.0	fGB	3-465B
U+3F39.0	fR/S	 98.10
U+3F39.0	fTotalStrokes	 15
U+3F39.0	fCangjie	TUMVN
U+3F39.0	fFourCorner	41210
U+3F39.0	fCantonese	hong1
U+3F39.0	fMandarin	qiang1; kang1; huang1; kuang1
U+3F39.0	fEnglish	[1] eartenware; pottery [2] kind container; vessel
U+3F3A.0	fUTF8	㼺
U+3F3A.0	fCNS11643	4-4B26
U+3F3A.0	fGB	3-4667
U+3F3A.0	fR/S	 98.10
U+3F3A.0	fTotalStrokes	 15
U+3F3A.0	fCangjie	IRMVN
U+3F3A.0	fFourCorner	01210
U+3F3A.0	fCantonese	tong4
U+3F3A.0	fMandarin	tang2
U+3F3A.0	fEnglish	[1] porcelain; porcelain ware; chinaware; china [2] a small bottle with ears
U+3F3B.0	fUTF8	㼻
U+3F3B.0	fCNS11643	4-513F
U+3F3B.0	fGB	3-463F
U+3F3B.0	fR/S	 98.11
U+3F3B.0	fTotalStrokes	 16
U+3F3B.0	fCangjie	HOMVN
U+3F3B.0	fFourCorner	28710
U+3F3B.0	fCantonese	cung4
U+3F3B.0	fMandarin	cong2; chong2
U+3F3B.0	fEnglish	earthenware (jar; a jar with a small mouth and two or four ears; pot; pitcher)
U+3F3C.0	fUTF8	㼼
U+3F3C.0	fCNS11643	3-516F
U+3F3C.0	fR/S	 98.11
U+3F3C.0	fTotalStrokes	 16
U+3F3C.0	fCangjie	MFMVN
U+3F3C.0	fFourCorner	11910
U+3F3C.0	fCantonese	piu4; piu1
U+3F3C.0	fMandarin	piao2
U+3F3C.0	fEnglish	same as &#x74E2;
U+3F3D.0	fUTF8	㼽
U+3F3D.0	fCNS11643	4-5142
U+3F3D.0	fGB	3-466A
U+3F3D.0	fR/S	 98.11
U+3F3D.0	fTotalStrokes	 16
U+3F3D.0	fCangjie	KKMVN
U+3F3D.0	fFourCorner	41010
U+3F3D.0	fCantonese	cong2; song2
U+3F3D.0	fMandarin	shuang3; chuang3
U+3F3D.0	fEnglish	[1] fragments of a tile haven't been kiln-dried completely [2] rub out the filth with broken tiles
U+3F3E.0	fUTF8	㼾
U+3F3E.0	fCNS11643	4-5141
U+3F3E.0	fGB	3-7C2C
U+3F3E.0	fR/S	 98.11
U+3F3E.0	fTotalStrokes	 16
U+3F3E.0	fCangjie	IPMVN
U+3F3E.0	fFourCorner	01217
U+3F3E.0	fCantonese	luk6
U+3F3E.0	fMandarin	lu4
U+3F3E.0	fEnglish	bricks
U+3F3F.0	fUTF8	㼿
U+3F3F.0	fCNS11643	4-5672
U+3F3F.0	fGB	3-466E
U+3F3F.0	fR/S	 98.12
U+3F3F.0	fTotalStrokes	 17; 16
U+3F3F.0	fCangjie	YGMVN
U+3F3F.0	fFourCorner	01110
U+3F3F.0	fCantonese	tung4
U+3F3F.0	fMandarin	tong2; zhong4
U+3F3F.0	fEnglish	[1] the brick wall of a well [2] earthenware (jar; a jar with a small mouth and two or four ears; pot; pitcher)
U+3F40.0	fUTF8	㽀
U+3F40.0	fCNS11643	4-5673
U+3F40.0	fGB	3-4670
U+3F40.0	fR/S	 98.12
U+3F40.0	fTotalStrokes	 17; 16
U+3F40.0	fCangjie	TKMVN
U+3F40.0	fFourCorner	81410
U+3F40.0	fCantonese	zeng6; zing6
U+3F40.0	fMandarin	zheng4
U+3F40.0	fEnglish	earthenware (basin; pot; bowl; crock etc.)
U+3F41.0	fUTF8	㽁
U+3F41.0	fCNS11643	3-5551
U+3F41.0	fGB	5-306E
U+3F41.0	fR/S	 98.12
U+3F41.0	fTotalStrokes	 17
U+3F41.0	fCangjie	MHDN
U+3F41.0	fFourCorner	71210
U+3F41.0	fCantonese	lik6
U+3F41.0	fMandarin	li4
U+3F41.0	fEnglish	same as &#x9B32;
U+3F42.0	fUTF8	㽂
U+3F42.0	fCNS11643	4-5675
U+3F42.0	fGB	3-466D
U+3F42.0	fR/S	 98.12
U+3F42.0	fTotalStrokes	 17; 16
U+3F42.0	fCangjie	OMMVN
U+3F42.0	fFourCorner	81110
U+3F42.0	fCantonese	saap3; sap3; tat3
U+3F42.0	fMandarin	sa4
U+3F42.0	fEnglish	[1] a cracked container [2] a movement of a musical conposition in Tang Dynasty
U+3F43.0	fUTF8	㽃
U+3F43.0	fCNS11643	3-5550
U+3F43.0	fGB	5-4859
U+3F43.0	fR/S	 98.12
U+3F43.0	fTotalStrokes	 17
U+3F43.0	fCangjie	HWMVN
U+3F43.0	fFourCorner	21610
U+3F43.0	fCantonese	pun1; ban1
U+3F43.0	fMandarin	pan1; fan2
U+3F43.0	fEnglish	a big piece of brick
U+3F44.0	fUTF8	㽄
U+3F44.0	fCNS11643	4-5671
U+3F44.0	fGB	3-466C
U+3F44.0	fR/S	 98.12
U+3F44.0	fTotalStrokes	 17; 16
U+3F44.0	fCangjie	TLMVN
U+3F44.0	fFourCorner	42710
U+3F44.0	fCantonese	si1
U+3F44.0	fMandarin	si1; xi1
U+3F44.0	fEnglish	[1] earthen jar [2] crack of earthen container [3] hoarse voice
U+3F45.0	fUTF8	㽅
U+3F45.0	fGB	3-4671
U+3F45.0	fR/S	 98.12
U+3F45.0	fTotalStrokes	 17
U+3F45.0	fCangjie	NTMVN
U+3F45.0	fFourCorner	2111
U+3F45.0	fCantonese	dang1; daang1
U+3F45.0	fEnglish	vessel for ceremony in ancient times
U+3F46.0	fUTF8	㽆
U+3F46.0	fCNS11643	4-5B6F
U+3F46.0	fGB	3-464C
U+3F46.0	fR/S	 98.13
U+3F46.0	fTotalStrokes	 18
U+3F46.0	fCangjie	FWMVN
U+3F46.0	fFourCorner	91610
U+3F46.0	fCantonese	dong1
U+3F46.0	fMandarin	dang1
U+3F46.0	fEnglish	[1] earthenware; bowl; cup [2] tiles with a circular facade at one end used along the edge of palace eaves during the Han Dynasty
U+3F47.0	fUTF8	㽇
U+3F47.0	fCNS11643	4-5B6E
U+3F47.0	fGB	3-4644
U+3F47.0	fR/S	 98.13
U+3F47.0	fTotalStrokes	 18
U+3F47.0	fCangjie	HBMVN
U+3F47.0	fFourCorner	77710
U+3F47.0	fCantonese	huk6; wu4; zyu5
U+3F47.0	fMandarin	hu2; guai4; hui2; meng2; se4
U+3F47.0	fEnglish	[1] unburnt tiles [2] earthenware
U+3F48.0	fUTF8	㽈
U+3F48.0	fCNS11643	4-602C
U+3F48.0	fGB	3-4673
U+3F48.0	fR/S	 98.14
U+3F48.0	fTotalStrokes	 19
U+3F48.0	fCangjie	POMVN
U+3F48.0	fFourCorner	27710
U+3F48.0	fCantonese	ji6; ngaai6
U+3F48.0	fMandarin	yi4
U+3F48.0	fEnglish	a large earthenware jar with a small mouth and two or four ears; a big pot; bottle; pitcher; jug; vase
U+3F49.0	fUTF8	㽉
U+3F49.0	fCNS11643	4-602B
U+3F49.0	fGB	3-4643
U+3F49.0	fR/S	 98.14
U+3F49.0	fTotalStrokes	 19
U+3F49.0	fCangjie	SWMVN
U+3F49.0	fFourCorner	78710
U+3F49.0	fHakka	  gam5; ham5
U+3F49.0	fCantonese	haam6; gam3; ham3
U+3F49.0	fMandarin	han4; xian4; han3; jian4; jian1
U+3F49.0	fEnglish	small covered tub for holding ice
U+3F4A.0	fUTF8	㽊
U+3F4A.0	fCNS11643	4-6875
U+3F4A.0	fGB	3-4675
U+3F4A.0	fR/S	 98.17
U+3F4A.0	fTotalStrokes	 22; 21
U+3F4A.0	fCangjie	FEMVN
U+3F4A.0	fFourCorner	91410
U+3F4A.0	fCantonese	sip3; sik1; sit3
U+3F4A.0	fMandarin	xie4
U+3F4A.0	fEnglish	cracking of tile
U+3F4B.0	fUTF8	㽋
U+3F4B.0	fCNS11643	4-6B70
U+3F4B.0	fGB	3-464E
U+3F4B.0	fR/S	 98.19
U+3F4B.0	fTotalStrokes	 24
U+3F4B.0	fCangjie	VFMVN
U+3F4B.0	fFourCorner	22710
U+3F4B.0	fCantonese	lo4
U+3F4B.0	fMandarin	luo2; luo4
U+3F4B.0	fEnglish	[1] a kind of tile [2] earthenware [3] a cylinder shaped tile used for chimney
U+3F4C.0	fUTF8	㽌
U+3F4C.0	fCNS11643	4-6A53
U+3F4C.0	fGB	3-4676
U+3F4C.0	fR/S	 98.20
U+3F4C.0	fTotalStrokes	 23; 25
U+3F4C.0	fCangjie	MWMVN
U+3F4C.0	fFourCorner	11610
U+3F4C.0	fCantonese	lau6
U+3F4C.0	fMandarin	liu4
U+3F4C.0	fEnglish	the ridge of a roof
U+3F4D.0	fUTF8	㽍
U+3F4D.0	fCNS11643	3-324C
U+3F4D.0	fR/S	 99.5; .5
U+3F4D.0	fTotalStrokes	 10
U+3F4D.0	fCangjie	TMMVN
U+3F4D.0	fFourCorner	41710
U+3F4D.0	fCantonese	ham1
U+3F4D.0	fMandarin	kan1
U+3F4D.0	fEnglish	same as &#x5769;
U+3F4E.0	fUTF8	㽎
U+3F4E.0	fCNS11643	4-3E65
U+3F4E.0	fGB	5-302A
U+3F4E.0	fR/S	 99.8; .8
U+3F4E.0	fTotalStrokes	 13
U+3F4E.0	fCangjie	TVLBU
U+3F4E.0	fFourCorner	44710
U+3F4E.0	fCantonese	taam4
U+3F4E.0	fMandarin	tan2; dan3; dan4; xin1
U+3F4E.0	fEnglish	[1] the profundity of the harem [2] the mysterious and profound of the forbidden palace [3] the flourishing and exuberant of the flame
U+3F4F.0	fUTF8	㽏
U+3F4F.0	fGB	3-5744
U+3F4F.0	fR/S	 99.9
U+3F4F.0	fTotalStrokes	 14
U+3F4F.0	fCangjie	ERTM
U+3F4F.0	fCantonese	gam3
U+3F50.0	fUTF8	㽐
U+3F50.0	fCNS11643	3-4D3F
U+3F50.0	fR/S	 99.10; .10
U+3F50.0	fTotalStrokes	 15
U+3F50.0	fCangjie	TMTXC
U+3F50.0	fFourCorner	48730
U+3F50.0	fCantonese	him1; cim1
U+3F50.0	fMandarin	xian1
U+3F50.0	fEnglish	same as &#x99A6;
U+3F51.0	fUTF8	㽑
U+3F51.0	fCNS11643	4-5678
U+3F51.0	fGB	3-5745
U+3F51.0	fR/S	 99.12; .12
U+3F51.0	fTotalStrokes	 17; 16
U+3F51.0	fCangjie	TMMWJ
U+3F51.0	fFourCorner	41740
U+3F51.0	fCantonese	taam4
U+3F51.0	fMandarin	tan2; dan4
U+3F51.0	fEnglish	[1] delicious; tasty [2] same as &#x91B0;
U+3F52.0	fUTF8	㽒
U+3F52.0	fCNS11643	3-3D40
U+3F52.0	fR/S	 100.7
U+3F52.0	fTotalStrokes	 12
U+3F52.0	fCangjie	WSHQM
U+3F52.0	fFourCorner	65410
U+3F52.0	fCantonese	sang1; saang1
U+3F52.0	fMandarin	sheng1
U+3F52.0	fEnglish	same as &#x7525;
U+3F53.0	fUTF8	㽓
U+3F53.0	fCNS11643	F-504E
U+3F53.0	fGB	5-5A6D
U+3F53.0	fR/S	 100.10
U+3F53.0	fTotalStrokes	 15
U+3F53.0	fCangjie	HMHMM
U+3F53.0	fFourCorner	2511
U+3F53.0	fCantonese	san1
U+3F54.0	fUTF8	㽔
U+3F54.0	fR/S	 100.11
U+3F54.0	fTotalStrokes	 16
U+3F54.0	fCangjie	TOHQM
U+3F54.0	fCantonese	jeoi6
U+3F54.0	fEnglish	[Korean character]
U+3F55.0	fUTF8	㽕
U+3F55.0	fCNS11643	5-2463
U+3F55.0	fR/S	 102.2
U+3F55.0	fTotalStrokes	 7
U+3F55.0	fCangjie	LWNVS
U+3F55.0	fFourCorner	50200
U+3F55.0	fCantonese	jau4; zaau4
U+3F55.0	fMandarin	you2
U+3F55.0	fEnglish	[1] new branches growing (said of trees) [2] hollow; empty
U+3F56.0	fUTF8	㽖
U+3F56.0	fCNS11643	6-2862
U+3F56.0	fR/S	 102.2
U+3F56.0	fTotalStrokes	 7
U+3F56.0	fCangjie	WKS
U+3F56.0	fFourCorner	64020
U+3F56.0	fCantonese	naam4; laam4 (also); nam4
U+3F56.0	fMandarin	liu2; nan2
U+3F56.0	fEnglish	[1] same as &#x7537; [2] non-classical form of &#x7559;
U+3F57.0	fUTF8	㽗
U+3F57.0	fCNS11643	3-2672
U+3F57.0	fR/S	 102.2
U+3F57.0	fTotalStrokes	 7
U+3F57.0	fCangjie	WO
U+3F57.0	fFourCorner	68000
U+3F57.0	fCantonese	din6
U+3F57.0	fMandarin	mu3
U+3F58.0	fUTF8	㽘
U+3F58.0	fCNS11643	4-2A41
U+3F58.0	fGB	3-5A64
U+3F58.0	fR/S	 102.4
U+3F58.0	fTotalStrokes	 9
U+3F58.0	fCangjie	WYHN
U+3F58.0	fFourCorner	60010
U+3F58.0	fCantonese	gong2; gong1
U+3F58.0	fMandarin	gang1; gang3; gang4
U+3F58.0	fEnglish	[1] borderland; the frontier [2] a pit; cave [3] a narrow path in the field [4] administrative unit for rural community of &#x50A3;&#x65CF; ( under the feudal system) [5] pond; marsh [6] saltpond [7] same as &#x4D1A;
U+3F59.0	fUTF8	㽙
U+3F59.0	fCNS11643	3-2E29
U+3F59.0	fR/S	 102.4
U+3F59.0	fTotalStrokes	 9
U+3F59.0	fCangjie	WIHU
U+3F59.0	fFourCorner	63010
U+3F59.0	fCantonese	zeon3; zoeng6
U+3F59.0	fMandarin	yun3; jun4
U+3F59.0	fEnglish	[1] same as &#x756F; [2] ancient form of &#x5141;
U+3F5A.0	fUTF8	㽚
U+3F5A.0	fCNS11643	4-2A43
U+3F5A.0	fGB	3-5A62
U+3F5A.0	fR/S	 102.4
U+3F5A.0	fTotalStrokes	 9
U+3F5A.0	fCangjie	WHQ
U+3F5A.0	fFourCorner	60500
U+3F5A.0	fCantonese	cik1; cik3
U+3F5A.0	fMandarin	chi3; chi4
U+3F5A.0	fEnglish	[1] agricultural implements; farm tools [2] the name of a place
U+3F5B.0	fUTF8	㽛
U+3F5B.0	fCNS11643	4-2E2F
U+3F5B.0	fGB	3-5A67
U+3F5B.0	fR/S	 102.5
U+3F5B.0	fTotalStrokes	 10
U+3F5B.0	fCangjie	WPR
U+3F5B.0	fFourCorner	67020
U+3F5B.0	fHakka	 kieu1; geu1
U+3F5B.0	fCantonese	kau1; ngaau1
U+3F5B.0	fMandarin	gou1; qu2; kou4
U+3F5B.0	fEnglish	 bank; dyke
U+3F5C.0	fUTF8	㽜
U+3F5C.0	fCNS11643	4-2E31
U+3F5C.0	fR/S	 102.5
U+3F5C.0	fTotalStrokes	 10
U+3F5C.0	fCangjie	NUW
U+3F5C.0	fFourCorner	27600
U+3F5C.0	fCantonese	jyun2; zyun2
U+3F5C.0	fMandarin	wan3
U+3F5D.0	fUTF8	㽝
U+3F5D.0	fCNS11643	5-3364
U+3F5D.0	fR/S	 102.6
U+3F5D.0	fTotalStrokes	 11
U+3F5D.0	fCangjie	MNW
U+3F5D.0	fFourCorner	12600
U+3F5D.0	fCantonese	lai6
U+3F5D.0	fMandarin	li4; lie4
U+3F5D.0	fEnglish	[1] sink; fall [2] entrap [3] crush; capture
U+3F5E.0	fUTF8	㽞
U+3F5E.0	fCNS11643	3-3748
U+3F5E.0	fR/S	 102.6
U+3F5E.0	fTotalStrokes	 11
U+3F5E.0	fCangjie	RRW
U+3F5E.0	fFourCorner	66600
U+3F5E.0	fCantonese	lau4
U+3F5E.0	fMandarin	liu2
U+3F5E.0	fEnglish	abbreviated form of &#x7559;
U+3F5F.0	fUTF8	㽟
U+3F5F.0	fCNS11643	3-3D41
U+3F5F.0	fR/S	 102.7
U+3F5F.0	fTotalStrokes	 12
U+3F5F.0	fCangjie	WBDI
U+3F5F.0	fFourCorner	62040
U+3F5F.0	fCantonese	lyut3
U+3F5F.0	fMandarin	lie4
U+3F5F.0	fEnglish	[1] till or to plough the fields [2] same as &#x57D2;
U+3F60.0	fUTF8	㽠
U+3F60.0	fCNS11643	4-3928
U+3F60.0	fGB	3-5A69
U+3F60.0	fR/S	 102.7
U+3F60.0	fTotalStrokes	 12
U+3F60.0	fCangjie	WKOO
U+3F60.0	fFourCorner	64030
U+3F60.0	fCantonese	haap6; haap1; kit3
U+3F60.0	fMandarin	xia2
U+3F60.0	fEnglish	connection of the ditches or waterways
U+3F61.0	fUTF8	㽡
U+3F61.0	fCNS11643	4-3E69
U+3F61.0	fGB	3-5A6D
U+3F61.0	fR/S	 102.8
U+3F61.0	fTotalStrokes	 13
U+3F61.0	fCangjie	WHWJ
U+3F61.0	fFourCorner	66040
U+3F61.0	fCantonese	bei1; baai3
U+3F61.0	fMandarin	bei1
U+3F61.0	fEnglish	[1] field [2] block up the water to irrigate the fields
U+3F62.0	fUTF8	㽢
U+3F62.0	fCNS11643	3-4279
U+3F62.0	fR/S	 102.8
U+3F62.0	fTotalStrokes	 13
U+3F62.0	fCangjie	WKLU
U+3F62.0	fFourCorner	64010
U+3F62.0	fCantonese	am2; ngam2 (also); aam2; au2; au3; zip6
U+3F62.0	fMandarin	an3; ye4
U+3F62.0	fEnglish	 plough the fields
U+3F63.0	fUTF8	㽣
U+3F63.0	fCNS11643	3-427A
U+3F63.0	fR/S	 102.8
U+3F63.0	fTotalStrokes	 13
U+3F63.0	fCangjie	WIRM
U+3F63.0	fFourCorner	63050
U+3F63.0	fHakka	 wet7; vet7
U+3F63.0	fCantonese	wik6
U+3F63.0	fMandarin	yu4
U+3F63.0	fEnglish	[1] limit; frontier [2] country [3] basin
U+3F64.0	fUTF8	㽤
U+3F64.0	fCNS11643	4-3E68
U+3F64.0	fR/S	 102.8
U+3F64.0	fTotalStrokes	 13
U+3F64.0	fCangjie	WPFD
U+3F64.0	fFourCorner	67020
U+3F64.0	fCantonese	guk1
U+3F64.0	fMandarin	ju2
U+3F64.0	fEnglish	a small plot of land
U+3F65.0	fUTF8	㽥
U+3F65.0	fCNS11643	4-4479
U+3F65.0	fGB	3-5A72
U+3F65.0	fR/S	 102.9
U+3F65.0	fTotalStrokes	 14
U+3F65.0	fCangjie	WNHD
U+3F65.0	fFourCorner	67090
U+3F65.0	fCantonese	jau4; zaau4
U+3F65.0	fMandarin	rou2
U+3F65.0	fEnglish	[1] fertile fields; good land (during the Epoch of Spring and Autumn) [2] the name of a place in Zheng Guo (today's Henan province LuShan Xian
U+3F66.0	fUTF8	㽦
U+3F66.0	fCNS11643	3-4D40
U+3F66.0	fGB	5-5A29
U+3F66.0	fR/S	 102.10
U+3F66.0	fTotalStrokes	 15
U+3F66.0	fCangjie	FFBW
U+3F66.0	fFourCorner	99600
U+3F66.0	fCantonese	wan4; ceon4; seon1
U+3F66.0	fMandarin	xun2; xuan4; jun1; dian4
U+3F66.0	fEnglish	reclaimed land; flat and in even level
U+3F67.0	fUTF8	㽧
U+3F67.0	fCNS11643	4-4B2A
U+3F67.0	fGB	3-5A71
U+3F67.0	fR/S	 102.10; 102.9
U+3F67.0	fTotalStrokes	 15; 14
U+3F67.0	fCangjie	WTVI
U+3F67.0	fFourCorner	64032
U+3F67.0	fCantonese	zi1
U+3F67.0	fMandarin	zi1
U+3F67.0	fEnglish	the name of a place
U+3F68.0	fUTF8	㽨
U+3F68.0	fCNS11643	4-4B29
U+3F68.0	fGB	3-5A70
U+3F68.0	fR/S	 102.10
U+3F68.0	fTotalStrokes	 15
U+3F68.0	fCangjie	WTQM
U+3F68.0	fFourCorner	68010
U+3F68.0	fCantonese	co4; co1
U+3F68.0	fMandarin	jie1; cuo2
U+3F68.0	fEnglish	[1] fragmented; uncultivated and desolated fields [2] eliminate; remove [3] clean [4] interchangeable with &#x7625;
U+3F69.0	fUTF8	㽩
U+3F69.0	fCNS11643	4-5144
U+3F69.0	fGB	3-5A6E
U+3F69.0	fR/S	 102.11
U+3F69.0	fTotalStrokes	 16
U+3F69.0	fCangjie	WIIH
U+3F69.0	fFourCorner	63020
U+3F69.0	fCantonese	cam3; can3
U+3F69.0	fMandarin	can4; cao4
U+3F69.0	fEnglish	connected banks of earth or paths in the field
U+3F6A.0	fUTF8	㽪
U+3F6A.0	fCNS11643	5-5C2F
U+3F6A.0	fR/S	 102.12
U+3F6A.0	fTotalStrokes	 17
U+3F6A.0	fCangjie	WCWA
U+3F6A.0	fFourCorner	68060
U+3F6A.0	fCantonese	zang2
U+3F6A.0	fMandarin	zeng3; zha3
U+3F6A.0	fEnglish	paddy field; rice field
U+3F6B.0	fUTF8	㽫
U+3F6B.0	fCNS11643	4-5B72
U+3F6B.0	fGB	3-5A77
U+3F6B.0	fR/S	 102.13
U+3F6B.0	fTotalStrokes	 18
U+3F6B.0	fCangjie	YVGW
U+3F6B.0	fFourCorner	00600
U+3F6B.0	fCantonese	jung1; zung1
U+3F6B.0	fMandarin	yong1; yong3
U+3F6B.0	fEnglish	[1] seeds of the plant allied to the water-lily, used when ground into meal as a coarse food, also as medicine [2] same as &#x58C5;
U+3F6C.0	fUTF8	㽬
U+3F6C.0	fCNS11643	4-5B73
U+3F6C.0	fGB	3-5A76
U+3F6C.0	fR/S	 102.13
U+3F6C.0	fTotalStrokes	 18
U+3F6C.0	fCangjie	MWMRW
U+3F6C.0	fFourCorner	11660
U+3F6C.0	fCantonese	fu3
U+3F6C.0	fMandarin	fu4; bi4
U+3F6C.0	fEnglish	same as &#x526F;
U+3F6D.0	fUTF8	㽭
U+3F6D.0	fCNS11643	4-602E
U+3F6D.0	fGB	3-5A78
U+3F6D.0	fR/S	 102.14
U+3F6D.0	fTotalStrokes	 19
U+3F6D.0	fCangjie	WMBB
U+3F6D.0	fFourCorner	61020
U+3F6D.0	fHakka	 nion1
U+3F6D.0	fCantonese	jyun4; zyun5
U+3F6D.0	fMandarin	ruan3
U+3F6D.0	fEnglish	[1] vacant lot or space [2] soft land; land on the river side
U+3F6E.0	fUTF8	㽮
U+3F6E.0	fCNS11643	4-6369
U+3F6E.0	fR/S	 102.15
U+3F6E.0	fTotalStrokes	 20
U+3F6E.0	fCangjie	WWWM
U+3F6E.0	fFourCorner	60100
U+3F6E.0	fCantonese	sing1
U+3F6E.0	fMandarin	xing1
U+3F6E.0	fEnglish	same as &#x661F;
U+3F6F.0	fUTF8	㽯
U+3F6F.0	fCNS11643	4-6A54
U+3F6F.0	fGB	5-5A3A
U+3F6F.0	fR/S	 102.18
U+3F6F.0	fTotalStrokes	 23
U+3F6F.0	fCangjie	WUOB
U+3F6F.0	fFourCorner	62020
U+3F6F.0	fCantonese	kwai4
U+3F6F.0	fMandarin	qi2; xi2
U+3F70.0	fUTF8	㽰
U+3F70.0	fCNS11643	4-3E6D
U+3F70.0	fGB	3-5F74
U+3F70.0	fR/S	 103.8
U+3F70.0	fTotalStrokes	 13
U+3F70.0	fCangjie	BBNO
U+3F70.0	fFourCorner	77280
U+3F70.0	fCantonese	so1
U+3F70.0	fMandarin	shu1; shu4
U+3F70.0	fEnglish	(corrupted form) carved window frame on a door, the windows with scattered or dispersed frames, a wine filterer
U+3F71.0	fUTF8	㽱
U+3F71.0	fCNS11643	4-2461
U+3F71.0	fGB	5-5C5F
U+3F71.0	fR/S	 104.2
U+3F71.0	fTotalStrokes	 7
U+3F71.0	fCangjie	KVL
U+3F71.0	fFourCorner	00120
U+3F71.0	fHakka	  gau3
U+3F71.0	fCantonese	gaau2
U+3F71.0	fMandarin	jiu1; jiao3; niu2
U+3F71.0	fEnglish	colie with gripes
U+3F72.0	fUTF8	㽲
U+3F72.0	fCNS11643	4-2462
U+3F72.0	fGB	5-5C5E
U+3F72.0	fR/S	 104.2
U+3F72.0	fTotalStrokes	 7
U+3F72.0	fCangjie	KMVS
U+3F72.0	fFourCorner	00120
U+3F72.0	fCantonese	gaau2; caau4; gau2; naau2
U+3F72.0	fMandarin	xiu3; jiao3
U+3F72.0	fEnglish	same as &#x3F71;
U+3F73.0	fUTF8	㽳
U+3F73.0	fGB	5-5C62
U+3F73.0	fR/S	 104.3
U+3F73.0	fTotalStrokes	 8
U+3F73.0	fCangjie	KMD
U+3F73.0	fFourCorner	0012
U+3F73.0	fCantonese	heoi1
U+3F73.0	fMandarin	han4; xie4; xu3; yu2
U+3F73.0	fEnglish	illness; disease
U+3F74.0	fUTF8	㽴
U+3F74.0	fCNS11643	5-2737
U+3F74.0	fR/S	 104.3
U+3F74.0	fTotalStrokes	 8
U+3F74.0	fCangjie	KJK
U+3F74.0	fFourCorner	00140
U+3F74.0	fCantonese	zoeng6
U+3F74.0	fMandarin	zhang4
U+3F74.0	fEnglish	illness; disease
U+3F75.0	fUTF8	㽵
U+3F75.0	fCNS11643	3-2A3C
U+3F75.0	fR/S	 104.3
U+3F75.0	fTotalStrokes	 8
U+3F75.0	fCangjie	KG
U+3F75.0	fFourCorner	00110
U+3F75.0	fCantonese	zong1
U+3F75.0	fEnglish	same as non-classical form of &#x838A;
U+3F76.0	fUTF8	㽶
U+3F76.0	fR/S	 104.3
U+3F76.0	fTotalStrokes	 8
U+3F76.0	fCangjie	KSU
U+3F76.0	fCantonese	gei2
U+3F76.0	fEnglish	[Korean character]
U+3F77.0	fUTF8	㽷
U+3F77.0	fCNS11643	4-2A46
U+3F77.0	fGB	3-5E53
U+3F77.0	fR/S	 104.4
U+3F77.0	fTotalStrokes	 9
U+3F77.0	fCangjie	KE
U+3F77.0	fFourCorner	00130
U+3F77.0	fCantonese	seoi3
U+3F77.0	fMandarin	shui4
U+3F77.0	fEnglish	dropsy; hydrophillic swelling
U+3F78.0	fUTF8	㽸
U+3F78.0	fCNS11643	3-2E2D
U+3F78.0	fR/S	 104.4
U+3F78.0	fTotalStrokes	 9
U+3F78.0	fCangjie	KLBU
U+3F78.0	fFourCorner	00110
U+3F78.0	fCantonese	sam4; caam4
U+3F78.0	fMandarin	shen4; chen2
U+3F78.0	fEnglish	corrupted form &#x760E;
U+3F79.0	fUTF8	㽹
U+3F79.0	fCNS11643	4-2A49
U+3F79.0	fGB	3-5E4B
U+3F79.0	fR/S	 104.4
U+3F79.0	fTotalStrokes	 9
U+3F79.0	fCangjie	KHE
U+3F79.0	fFourCorner	00140
U+3F79.0	fHakka	  fan5; fan3
U+3F79.0	fCantonese	faan2; fan3
U+3F79.0	fMandarin	fan4; wan3
U+3F79.0	fEnglish	[1] vomit [2] faint
U+3F7A.0	fUTF8	㽺
U+3F7A.0	fCNS11643	3-2E2C
U+3F7A.0	fGB	5-5C64
U+3F7A.0	fR/S	 104.4
U+3F7A.0	fTotalStrokes	 9
U+3F7A.0	fCangjie	KNHE
U+3F7A.0	fFourCorner	00140
U+3F7A.0	fCantonese	gap1
U+3F7A.0	fMandarin	ji2; xia4
U+3F7A.0	fEnglish	[1] be weaked by disease; feeble; lean [2] poor in quality
U+3F7B.0	fUTF8	㽻
U+3F7B.0	fCNS11643	3-2E2E
U+3F7B.0	fGB	5-5C66
U+3F7B.0	fR/S	 104.4
U+3F7B.0	fTotalStrokes	 9
U+3F7B.0	fCangjie	KJE
U+3F7B.0	fFourCorner	00140
U+3F7B.0	fCantonese	zi1
U+3F7B.0	fMandarin	zhi1
U+3F7B.0	fEnglish	diseases; illness
U+3F7C.0	fUTF8	㽼
U+3F7C.0	fGB	S-2134
U+3F7C.0	fR/S	 104.4
U+3F7C.0	fTotalStrokes	 9
U+3F7C.0	fCangjie	KNL
U+3F7C.0	fCantonese	jan5
U+3F7D.0	fUTF8	㽽
U+3F7D.0	fCNS11643	3-3256
U+3F7D.0	fGB	5-5C73
U+3F7D.0	fR/S	 104.5
U+3F7D.0	fTotalStrokes	 10
U+3F7D.0	fCangjie	KJR
U+3F7D.0	fFourCorner	00160
U+3F7D.0	fHakka	 gu5
U+3F7D.0	fCantonese	gu3; gwu3 (also)
U+3F7D.0	fMandarin	gu4
U+3F7D.0	fEnglish	same as &#x75FC;
U+3F7E.0	fUTF8	㽾
U+3F7E.0	fCNS11643	4-2E37
U+3F7E.0	fGB	3-5E5F
U+3F7E.0	fR/S	 104.5
U+3F7E.0	fTotalStrokes	 10
U+3F7E.0	fCangjie	KUU
U+3F7E.0	fFourCorner	00170
U+3F7E.0	fCantonese	ngat6; at6 (also); ngaat6; zaat6
U+3F7E.0	fMandarin	wu4
U+3F7E.0	fEnglish	[1] a kind of woman's disease [2] cut apart [3] break
U+3F7F.0	fUTF8	㽿
U+3F7F.0	fCNS11643	4-2E3B
U+3F7F.0	fGB	5-5C77
U+3F7F.0	fR/S	 104.5
U+3F7F.0	fTotalStrokes	 10
U+3F7F.0	fCangjie	KHVO
U+3F7F.0	fFourCorner	00130
U+3F7F.0	fCantonese	gwo1; go1 (also); wo1
U+3F7F.0	fMandarin	ge1
U+3F7F.0	fEnglish	same as &#x7611;
U+3F80.0	fUTF8	㾀
U+3F80.0	fCNS11643	4-2E35
U+3F80.0	fGB	3-5E54
U+3F80.0	fR/S	 104.5
U+3F80.0	fTotalStrokes	 10
U+3F80.0	fCangjie	KGI
U+3F80.0	fFourCorner	00130
U+3F80.0	fHakka	  kiap7
U+3F80.0	fCantonese	hip3
U+3F80.0	fMandarin	qie4
U+3F80.0	fEnglish	[1] weakness [2] lassitude [3] debility
U+3F81.0	fUTF8	㾁
U+3F81.0	fCNS11643	4-2E36
U+3F81.0	fGB	3-5E55
U+3F81.0	fR/S	 104.5
U+3F81.0	fTotalStrokes	 10
U+3F81.0	fCangjie	KIJC
U+3F81.0	fFourCorner	00190
U+3F81.0	fCantonese	fat1; seot6
U+3F81.0	fMandarin	hu1; shu4
U+3F81.0	fEnglish	walking like mad; mad
U+3F82.0	fUTF8	㾂
U+3F82.0	fCNS11643	3-3254
U+3F82.0	fR/S	 104.5
U+3F82.0	fTotalStrokes	 10
U+3F82.0	fCangjie	KIR
U+3F82.0	fFourCorner	00160
U+3F82.0	fCantonese	hoi1
U+3F82.0	fMandarin	hai1
U+3F82.0	fEnglish	[1] illness; disease [2] a chronic disease; never recover after a long illness
U+3F83.0	fUTF8	㾃
U+3F83.0	fCNS11643	4-2E33
U+3F83.0	fGB	3-5E5D
U+3F83.0	fR/S	 104.5
U+3F83.0	fTotalStrokes	 10
U+3F83.0	fCangjie	KJP
U+3F83.0	fFourCorner	00110
U+3F83.0	fHakka	  to2; to3
U+3F83.0	fCantonese	to4
U+3F83.0	fMandarin	tuo2
U+3F83.0	fEnglish	hump-backed
U+3F84.0	fUTF8	㾄
U+3F84.0	fCNS11643	4-2E3F
U+3F84.0	fGB	5-5C76
U+3F84.0	fR/S	 104.5
U+3F84.0	fTotalStrokes	 10
U+3F84.0	fCangjie	KLW
U+3F84.0	fFourCorner	00160
U+3F84.0	fCantonese	duk6
U+3F84.0	fMandarin	du2
U+3F84.0	fEnglish	[1] ancient form of, same as &#x8B9F; [2] same as &#x601E;
U+3F85.0	fUTF8	㾅
U+3F85.0	fCNS11643	4-2E39
U+3F85.0	fGB	3-5E4F
U+3F85.0	fR/S	 104.5
U+3F85.0	fTotalStrokes	 10
U+3F85.0	fCangjie	KLXH
U+3F85.0	fFourCorner	00120
U+3F85.0	fCantonese	zi2; nou5; zi3
U+3F85.0	fMandarin	si4; zhi4; zi3
U+3F85.0	fEnglish	defects; flaws, illness; disease
U+3F86.0	fUTF8	㾆
U+3F86.0	fCNS11643	6-396B
U+3F86.0	fGB	3-5E59
U+3F86.0	fR/S	 104.5
U+3F86.0	fTotalStrokes	 10; 9
U+3F86.0	fCangjie	KGB
U+3F86.0	fFourCorner	00147
U+3F86.0	fCantonese	cim1
U+3F86.0	fMandarin	chan1; ran2; shan3
U+3F86.0	fEnglish	(non-classical form) ecdysis (said of some kinds of reptiles or insects); cast off the skin; molt
U+3F87.0	fUTF8	㾇
U+3F87.0	fCNS11643	3-3253
U+3F87.0	fGB	5-5C75
U+3F87.0	fR/S	 104.5
U+3F87.0	fTotalStrokes	 10
U+3F87.0	fCangjie	KBU
U+3F87.0	fFourCorner	00160
U+3F87.0	fCantonese	muk6; do2; dou1; zyu4
U+3F87.0	fMandarin	mu4
U+3F87.0	fEnglish	illness; disease; ailment
U+3F88.0	fUTF8	㾈
U+3F88.0	fCNS11643	4-2E38
U+3F88.0	fGB	3-5E5B
U+3F88.0	fR/S	 104.5
U+3F88.0	fTotalStrokes	 10
U+3F88.0	fCangjie	KODI
U+3F88.0	fFourCorner	00140
U+3F88.0	fCantonese	fu6
U+3F88.0	fMandarin	fu3; fu4
U+3F88.0	fEnglish	[1] hunchbacked; curved spine [2] short [3] dropsy
U+3F89.0	fUTF8	㾉
U+3F89.0	fCNS11643	4-2E3A
U+3F89.0	fGB	3-5E5C
U+3F89.0	fR/S	 104.5
U+3F89.0	fTotalStrokes	 10
U+3F89.0	fCangjie	KOII
U+3F89.0	fFourCorner	00130
U+3F89.0	fCantonese	ling4
U+3F89.0	fMandarin	ling2
U+3F89.0	fEnglish	thin; lean; slim; enacuated
U+3F8A.0	fUTF8	㾊
U+3F8A.0	fCNS11643	5-336D
U+3F8A.0	fGB	3-5E61
U+3F8A.0	fR/S	 104.6
U+3F8A.0	fTotalStrokes	 11
U+3F8A.0	fCangjie	KDB
U+3F8A.0	fFourCorner	00190
U+3F8A.0	fHakka	 sit7; suk7
U+3F8A.0	fCantonese	zek3; zik6
U+3F8A.0	fMandarin	ji2
U+3F8A.0	fEnglish	ague; chill
U+3F8B.0	fUTF8	㾋
U+3F8B.0	fCNS11643	4-3342
U+3F8B.0	fGB	3-5E67
U+3F8B.0	fR/S	 104.6
U+3F8B.0	fTotalStrokes	 11
U+3F8B.0	fCangjie	KOD
U+3F8B.0	fFourCorner	00190
U+3F8B.0	fCantonese	jau1; zaau1
U+3F8B.0	fMandarin	xiu1; xiu4
U+3F8B.0	fEnglish	[1] dysentery; diarrhea [2] sore; boil; ulcer
U+3F8C.0	fUTF8	㾌
U+3F8C.0	fCNS11643	3-374C
U+3F8C.0	fGB	5-5D25
U+3F8C.0	fR/S	 104.6
U+3F8C.0	fTotalStrokes	 11
U+3F8C.0	fCangjie	KHGU
U+3F8C.0	fFourCorner	00110
U+3F8C.0	fCantonese	sin2; bei3
U+3F8C.0	fMandarin	xian3; xuan3
U+3F8C.0	fEnglish	same as &#x766C;
U+3F8D.0	fUTF8	㾍
U+3F8D.0	fCNS11643	4-3340
U+3F8D.0	fGB	3-5E63
U+3F8D.0	fR/S	 104.6
U+3F8D.0	fTotalStrokes	 11
U+3F8D.0	fCangjie	KMBL
U+3F8D.0	fFourCorner	00120
U+3F8D.0	fHakka	 nai2; nai3
U+3F8D.0	fCantonese	noi4; loi4 (also); nai1
U+3F8D.0	fMandarin	nai2
U+3F8D.0	fEnglish	exhausted; worn-out
U+3F8E.0	fUTF8	㾎
U+3F8E.0	fCNS11643	4-2E3C
U+3F8E.0	fGB	3-5E5A
U+3F8E.0	fR/S	 104.6
U+3F8E.0	fTotalStrokes	 11; 10
U+3F8E.0	fCangjie	KSSU
U+3F8E.0	fFourCorner	00170
U+3F8E.0	fCantonese	aap3; ngaap3 (also); at3; haat6
U+3F8E.0	fMandarin	ya1
U+3F8E.0	fEnglish	[1] [2] sickness; disease [3] favus [4] everything is good except this particular one
U+3F8F.0	fUTF8	㾏
U+3F8F.0	fCNS11643	4-333F
U+3F8F.0	fGB	3-5E60
U+3F8F.0	fR/S	 104.6
U+3F8F.0	fTotalStrokes	 11
U+3F8F.0	fCangjie	KGG
U+3F8F.0	fFourCorner	00110
U+3F8F.0	fHakka	 ngai2
U+3F8F.0	fCantonese	gaai3; hai6
U+3F8F.0	fMandarin	jie4
U+3F8F.0	fEnglish	[1] stupid [2] doltish
U+3F90.0	fUTF8	㾐
U+3F90.0	fCNS11643	4-3345
U+3F90.0	fGB	5-5D21
U+3F90.0	fR/S	 104.6
U+3F90.0	fTotalStrokes	 11
U+3F90.0	fCangjie	KMNN
U+3F90.0	fFourCorner	00120
U+3F90.0	fCantonese	lai6
U+3F90.0	fMandarin	li4
U+3F90.0	fEnglish	[1] ulcer; sore; swelling and sores caused by varnish-poisoning; pestilence; epidemic [2] leprosy [3] interchangeable &#x52F5;
U+3F91.0	fUTF8	㾑
U+3F91.0	fCNS11643	3-374B
U+3F91.0	fGB	5-5D27
U+3F91.0	fR/S	 104.6
U+3F91.0	fTotalStrokes	 11
U+3F91.0	fCangjie	KOMR
U+3F91.0	fFourCorner	00160
U+3F91.0	fHakka	 dap7; hap7
U+3F91.0	fCantonese	daap3
U+3F91.0	fMandarin	da2
U+3F91.0	fEnglish	[1] fat; plump [2] chill [3] cold [4] malaria [5] wart; a pimple [6] a round lump
U+3F92.0	fUTF8	㾒
U+3F92.0	fCNS11643	4-3343
U+3F92.0	fGB	3-5E6C
U+3F92.0	fR/S	 104.6
U+3F92.0	fTotalStrokes	 11
U+3F92.0	fCangjie	KVR
U+3F92.0	fFourCorner	00160
U+3F92.0	fCantonese	jyu4; zyu4
U+3F92.0	fMandarin	ru2; ji4; ru4
U+3F92.0	fEnglish	[1] illness; disease [2] could not have access to ...
U+3F93.0	fUTF8	㾓
U+3F93.0	fCNS11643	4-3932
U+3F93.0	fGB	3-5E73
U+3F93.0	fR/S	 104.7
U+3F93.0	fTotalStrokes	 12
U+3F93.0	fCangjie	KRB
U+3F93.0	fFourCorner	00120
U+3F93.0	fCantonese	jyun1; zyun1
U+3F93.0	fMandarin	yuan1
U+3F93.0	fEnglish	[1] fatigue; exhaustion; weriness [2] ache of the joints (of bones); gout [3] melancholy; depressed [4] anger; fury [5] hatred; bitterness
U+3F94.0	fUTF8	㾔
U+3F94.0	fCNS11643	4-3933
U+3F94.0	fGB	3-5E66
U+3F94.0	fR/S	 104.7
U+3F94.0	fTotalStrokes	 12
U+3F94.0	fCangjie	KRR
U+3F94.0	fFourCorner	00160
U+3F94.0	fCantonese	leoi5
U+3F94.0	fMandarin	luu3; l3
U+3F94.0	fEnglish	[1] long illness [2] ulcer; sore; boil
U+3F95.0	fUTF8	㾕
U+3F95.0	fCNS11643	3-3D46
U+3F95.0	fGB	5-5D34
U+3F95.0	fR/S	 104.7
U+3F95.0	fTotalStrokes	 12
U+3F95.0	fCangjie	KYTJ
U+3F95.0	fFourCorner	00141
U+3F95.0	fHakka	  sin1; sem3; fin1
U+3F95.0	fCantonese	sam2; saan1; syun2
U+3F95.0	fMandarin	xin1; shen3
U+3F95.0	fEnglish	ague
U+3F96.0	fUTF8	㾖
U+3F96.0	fCNS11643	3-3D4B
U+3F96.0	fGB	5-5D30
U+3F96.0	fR/S	 104.7
U+3F96.0	fTotalStrokes	 12
U+3F96.0	fCangjie	KWG
U+3F96.0	fFourCorner	00110
U+3F96.0	fCantonese	lei5
U+3F96.0	fMandarin	li3; luo2
U+3F96.0	fEnglish	grieved; distressed; ill with grief and exhausted
U+3F97.0	fUTF8	㾗
U+3F97.0	fCNS11643	6-4B5A
U+3F97.0	fR/S	 104.7
U+3F97.0	fTotalStrokes	 12
U+3F97.0	fCangjie	KIAV
U+3F97.0	fFourCorner	00130
U+3F97.0	fHakka	 liong5
U+3F97.0	fCantonese	loeng6; gak3; long5
U+3F97.0	fMandarin	liang4; lang3
U+3F97.0	fEnglish	[1] eye disease; strabismus; squint; look askance [2] [3] bright; light; brilliant [4] bright eyes
U+3F98.0	fUTF8	㾘
U+3F98.0	fCNS11643	3-3D49
U+3F98.0	fGB	5-5D2C
U+3F98.0	fR/S	 104.7
U+3F98.0	fTotalStrokes	 12
U+3F98.0	fCangjie	KMLK
U+3F98.0	fFourCorner	00140
U+3F98.0	fHakka	 gien3; gen3
U+3F98.0	fCantonese	gang2
U+3F98.0	fMandarin	geng3
U+3F98.0	fEnglish	 disease; sickness
U+3F99.0	fUTF8	㾙
U+3F99.0	fCNS11643	4-3936
U+3F99.0	fGB	3-5E76
U+3F99.0	fR/S	 104.7
U+3F99.0	fTotalStrokes	 12
U+3F99.0	fCangjie	KKKB
U+3F99.0	fFourCorner	00120
U+3F99.0	fCantonese	jan3; zaan1
U+3F99.0	fMandarin	xin3; yin3
U+3F99.0	fEnglish	same as &#x812A;
U+3F9A.0	fUTF8	㾚
U+3F9A.0	fCNS11643	5-4060
U+3F9A.0	fR/S	 104.7; 104.8
U+3F9A.0	fTotalStrokes	 13; 12
U+3F9A.0	fCangjie	KYPM
U+3F9A.0	fFourCorner	00110
U+3F9A.0	fCantonese	se1
U+3F9A.0	fMandarin	xie1
U+3F9A.0	fEnglish	itchy; ticklish
U+3F9B.0	fUTF8	㾛
U+3F9B.0	fCNS11643	4-392F
U+3F9B.0	fGB	3-5E79
U+3F9B.0	fR/S	 104.7
U+3F9B.0	fTotalStrokes	 12
U+3F9B.0	fCangjie	KSME
U+3F9B.0	fFourCorner	00140
U+3F9B.0	fCantonese	cam2
U+3F9B.0	fMandarin	qin3
U+3F9B.0	fEnglish	ugly; bad-looking
U+3F9C.0	fUTF8	㾜
U+3F9C.0	fCNS11643	4-3930
U+3F9C.0	fGB	3-5E64
U+3F9C.0	fR/S	 104.7
U+3F9C.0	fTotalStrokes	 12
U+3F9C.0	fCangjie	KKOO
U+3F9C.0	fFourCorner	00130
U+3F9C.0	fCantonese	hip3; fan2; keoi1; koek3
U+3F9C.0	fMandarin	qie4
U+3F9C.0	fEnglish	weak breath of a sick man
U+3F9D.0	fUTF8	㾝
U+3F9D.0	fCNS11643	3-3D48
U+3F9D.0	fGB	5-5C67
U+3F9D.0	fR/S	 104.7
U+3F9D.0	fTotalStrokes	 12
U+3F9D.0	fCangjie	KJWJ
U+3F9D.0	fFourCorner	00150
U+3F9D.0	fCantonese	ce3; zi1
U+3F9D.0	fMandarin	che4
U+3F9D.0	fEnglish	[1] (corrupted form) [2] leak out; ejaculate [3] scatter; disperse; vent
U+3F9E.0	fUTF8	㾞
U+3F9E.0	fCNS11643	3-3D4A
U+3F9E.0	fGB	5-5D2D
U+3F9E.0	fR/S	 104.7
U+3F9E.0	fTotalStrokes	 12
U+3F9E.0	fCangjie	KMCW
U+3F9E.0	fFourCorner	00160
U+3F9E.0	fCantonese	jau4; zaau4; zuk1
U+3F9E.0	fMandarin	you2; you3
U+3F9E.0	fEnglish	[1] illness; disease [2] interchangeable with &#x5EAE;
U+3F9F.0	fUTF8	㾟
U+3F9F.0	fCNS11643	5-3A3B
U+3F9F.0	fR/S	 104.7
U+3F9F.0	fTotalStrokes	 12
U+3F9F.0	fCangjie	KYLH
U+3F9F.0	fFourCorner	00120
U+3F9F.0	fCantonese	bou6
U+3F9F.0	fMandarin	bu4
U+3F9F.0	fEnglish	[1] relapse (said of illness) [2] relapse of typhoid fever (typhus)
U+3FA0.0	fUTF8	㾠
U+3FA0.0	fCNS11643	4-3935
U+3FA0.0	fGB	5-5D33
U+3FA0.0	fR/S	 104.7
U+3FA0.0	fTotalStrokes	 12
U+3FA0.0	fCangjie	KKHG
U+3FA0.0	fFourCorner	00110
U+3FA0.0	fCantonese	kong4; kwong4
U+3FA0.0	fMandarin	kuang2; huang2; kui4
U+3FA0.0	fEnglish	a fever
U+3FA1.0	fUTF8	㾡
U+3FA1.0	fCNS11643	4-3938
U+3FA1.0	fGB	3-5E6E
U+3FA1.0	fR/S	 104.7
U+3FA1.0	fTotalStrokes	 12
U+3FA1.0	fCangjie	KGIL
U+3FA1.0	fFourCorner	00120
U+3FA1.0	fCantonese	koek3
U+3FA1.0	fMandarin	que4
U+3FA1.0	fEnglish	ulcer; sore; boil
U+3FA2.0	fUTF8	㾢
U+3FA2.0	fCNS11643	4-3E74
U+3FA2.0	fGB	5-5D2F
U+3FA2.0	fR/S	 104.8
U+3FA2.0	fTotalStrokes	 13
U+3FA2.0	fCangjie	KDOO
U+3FA2.0	fFourCorner	00198
U+3FA2.0	fHakka	 loi3;  lai5
U+3FA2.0	fCantonese	loi6
U+3FA2.0	fMandarin	lai4
U+3FA2.0	fEnglish	itch
U+3FA3.0	fUTF8	㾣
U+3FA3.0	fCNS11643	5-4064
U+3FA3.0	fR/S	 104.8
U+3FA3.0	fTotalStrokes	 13
U+3FA3.0	fCangjie	KC
U+3FA3.0	fFourCorner	00110
U+3FA3.0	fHakka	 kim1
U+3FA3.0	fCantonese	ham1; zaam1
U+3FA3.0	fMandarin	qin1
U+3FA3.0	fEnglish	[1] shiver; shudder; tremble because of a malaria and a very bad cold [2] severe chill; ague
U+3FA4.0	fUTF8	㾤
U+3FA4.0	fCNS11643	4-3E6F
U+3FA4.0	fGB	3-5F2A
U+3FA4.0	fR/S	 104.8
U+3FA4.0	fTotalStrokes	 13
U+3FA4.0	fCangjie	KJCM
U+3FA4.0	fFourCorner	00110
U+3FA4.0	fHakka	 kiong1
U+3FA4.0	fCantonese	hong1
U+3FA4.0	fMandarin	qiang1
U+3FA4.0	fEnglish	disease of the throat
U+3FA5.0	fUTF8	㾥
U+3FA5.0	fCNS11643	4-3E78
U+3FA5.0	fGB	3-5E7E
U+3FA5.0	fR/S	 104.8
U+3FA5.0	fTotalStrokes	 13
U+3FA5.0	fCangjie	KYFE
U+3FA5.0	fFourCorner	00140
U+3FA5.0	fCantonese	cuk1
U+3FA5.0	fMandarin	chu4; xu4
U+3FA5.0	fEnglish	bellyache
U+3FA6.0	fUTF8	㾦
U+3FA6.0	fCNS11643	4-3E6E
U+3FA6.0	fGB	3-5F28
U+3FA6.0	fR/S	 104.8
U+3FA6.0	fTotalStrokes	 13
U+3FA6.0	fCangjie	KYTR
U+3FA6.0	fFourCorner	00160
U+3FA6.0	fCantonese	pui1; pui3
U+3FA6.0	fMandarin	pei4; bang4; pen2
U+3FA6.0	fEnglish	[1] scab over a sore [2] scar of an ulcer [3] weak; feeble
U+3FA7.0	fUTF8	㾧
U+3FA7.0	fCNS11643	4-3E76
U+3FA7.0	fGB	3-5F21
U+3FA7.0	fR/S	 104.8
U+3FA7.0	fTotalStrokes	 13
U+3FA7.0	fCangjie	KWD
U+3FA7.0	fFourCorner	00190
U+3FA7.0	fCantonese	fo3
U+3FA7.0	fMandarin	ke4
U+3FA7.0	fEnglish	[1] favus [2] swelling of the glands [3] lump; scrofulous swellings
U+3FA8.0	fUTF8	㾨
U+3FA8.0	fCNS11643	4-3E73
U+3FA8.0	fGB	3-5E7C
U+3FA8.0	fR/S	 104.8
U+3FA8.0	fTotalStrokes	 13
U+3FA8.0	fCangjie	KKMR
U+3FA8.0	fFourCorner	00120
U+3FA8.0	fCantonese	ji1; zi1
U+3FA8.0	fMandarin	yi1; qi3; yi3
U+3FA8.0	fEnglish	[1] weak; feeble [2] mourn [3] ulcer; cancer; carbuncle [4] short
U+3FA9.0	fUTF8	㾩
U+3FA9.0	fCNS11643	4-3E7A
U+3FA9.0	fGB	5-5D3D
U+3FA9.0	fR/S	 104.8
U+3FA9.0	fTotalStrokes	 13
U+3FA9.0	fCangjie	KHJP
U+3FA9.0	fFourCorner	00110
U+3FA9.0	fCantonese	gwaai1
U+3FA9.0	fMandarin	guai1
U+3FA9.0	fEnglish	malignant boils
U+3FAA.0	fUTF8	㾪
U+3FAA.0	fCNS11643	4-4524
U+3FAA.0	fGB	3-5F31
U+3FAA.0	fR/S	 104.9
U+3FAA.0	fTotalStrokes	 14
U+3FAA.0	fCangjie	KFHU
U+3FAA.0	fFourCorner	00160
U+3FAA.0	fCantonese	saang2
U+3FAA.0	fMandarin	sheng3
U+3FAA.0	fEnglish	lean; thin; slim
U+3FAB.0	fUTF8	㾫
U+3FAB.0	fCNS11643	4-4528
U+3FAB.0	fGB	5-5D50
U+3FAB.0	fR/S	 104.9
U+3FAB.0	fTotalStrokes	 14
U+3FAB.0	fCangjie	KISB
U+3FAB.0	fFourCorner	00120
U+3FAB.0	fHakka	  pien1
U+3FAB.0	fCantonese	pin1
U+3FAB.0	fMandarin	pian1
U+3FAB.0	fEnglish	[1] paralysis [2] palsy
U+3FAC.0	fUTF8	㾬
U+3FAC.0	fCNS11643	6-5F2D
U+3FAC.0	fR/S	 104.9
U+3FAC.0	fTotalStrokes	 14
U+3FAC.0	fCangjie	KPPA
U+3FAC.0	fFourCorner	00160
U+3FAC.0	fCantonese	gaai1
U+3FAC.0	fMandarin	kai1
U+3FAC.0	fEnglish	same as &#x75CE;
U+3FAD.0	fUTF8	㾭
U+3FAD.0	fCNS11643	3-482A
U+3FAD.0	fGB	5-5D4B
U+3FAD.0	fR/S	 104.9
U+3FAD.0	fTotalStrokes	 14
U+3FAD.0	fCangjie	KHDF
U+3FAD.0	fFourCorner	00180
U+3FAD.0	fHakka	 ziu5
U+3FAD.0	fCantonese	zau3
U+3FAD.0	fMandarin	zhou4
U+3FAD.0	fEnglish	[1] heal up [2] go down (as a swelling)
U+3FAE.0	fUTF8	㾮
U+3FAE.0	fCNS11643	3-482C
U+3FAE.0	fGB	5-5D4D
U+3FAE.0	fR/S	 104.9
U+3FAE.0	fTotalStrokes	 14
U+3FAE.0	fCangjie	KHAG
U+3FAE.0	fFourCorner	00110
U+3FAE.0	fHakka	 wong2
U+3FAE.0	fCantonese	wong4; ou3
U+3FAE.0	fMandarin	huang2
U+3FAE.0	fEnglish	same as &#x97D6;
U+3FAF.0	fUTF8	㾯
U+3FAF.0	fCNS11643	4-4523
U+3FAF.0	fGB	3-5F32
U+3FAF.0	fR/S	 104.9
U+3FAF.0	fTotalStrokes	 14
U+3FAF.0	fCangjie	KWMV
U+3FAF.0	fFourCorner	00130
U+3FAF.0	fCantonese	fui1
U+3FAF.0	fMandarin	wei4; tui2; wei3
U+3FAF.0	fEnglish	[1] suffer a stroke of paralysis or apoplexy [2] disease of the private part (of the human body)
U+3FB0.0	fUTF8	㾰
U+3FB0.0	fCNS11643	4-4521
U+3FB0.0	fGB	3-5F30
U+3FB0.0	fR/S	 104.9
U+3FB0.0	fTotalStrokes	 14
U+3FB0.0	fCangjie	KJRB
U+3FB0.0	fFourCorner	00120
U+3FB0.0	fCantonese	wu4
U+3FB0.0	fMandarin	hu2
U+3FB0.0	fEnglish	[1] choke [2] lump in throat; food stuck in the throat
U+3FB1.0	fUTF8	㾱
U+3FB1.0	fR/S	 104.9
U+3FB1.0	fTotalStrokes	 14
U+3FB1.0	fCangjie	KNOP
U+3FB1.0	fCantonese	fai3
U+3FB1.0	fMandarin	bei4; fan4; fei4; fu2
U+3FB1.0	fEnglish	[Korean character]
U+3FB2.0	fUTF8	㾲
U+3FB2.0	fGB	S-2138
U+3FB2.0	fR/S	 104.9
U+3FB2.0	fTotalStrokes	 14
U+3FB2.0	fCangjie	KQSK
U+3FB2.0	fCantonese	ngou4; ou4 (also)
U+3FB3.0	fUTF8	㾳
U+3FB3.0	fR/S	 104.9
U+3FB3.0	fTotalStrokes	 14
U+3FB3.0	fCangjie	KIOV
U+3FB3.0	fCantonese	zi1
U+3FB3.0	fEnglish	[Korean character]
U+3FB4.0	fUTF8	㾴
U+3FB4.0	fR/S	 104.9
U+3FB4.0	fTotalStrokes	 14
U+3FB4.0	fCangjie	KDBM
U+3FB4.0	fCantonese	caa4; zaa1
U+3FB4.0	fEnglish	the erythema of acne rosacea
U+3FB5.0	fUTF8	㾵
U+3FB5.0	fGB	3-5E51
U+3FB5.0	fR/S	 104.9
U+3FB5.0	fTotalStrokes	 14
U+3FB5.0	fCangjie	KYRJ
U+3FB5.0	fCantonese	gai3; gaai3
U+3FB5.0	fMandarin	ji4
U+3FB5.0	fEnglish	dark-coloured birth mark
U+3FB6.0	fUTF8	㾶
U+3FB6.0	fCNS11643	7-2558
U+3FB6.0	fGB	3-5F33
U+3FB6.0	fR/S	 104.10
U+3FB6.0	fTotalStrokes	 15
U+3FB6.0	fCangjie	KBBB
U+3FB6.0	fFourCorner	00120
U+3FB6.0	fCantonese	gwat1
U+3FB6.0	fMandarin	gu3
U+3FB6.0	fEnglish	disease of the knee
U+3FB7.0	fUTF8	㾷
U+3FB7.0	fCNS11643	4-4B38
U+3FB7.0	fGB	3-5F3C
U+3FB7.0	fR/S	 104.10
U+3FB7.0	fTotalStrokes	 15
U+3FB7.0	fCangjie	KHYU
U+3FB7.0	fFourCorner	00110
U+3FB7.0	fCantonese	sai1
U+3FB7.0	fMandarin	xi1
U+3FB7.0	fEnglish	 ache
U+3FB8.0	fUTF8	㾸
U+3FB8.0	fCNS11643	3-4D42
U+3FB8.0	fR/S	 104.10
U+3FB8.0	fTotalStrokes	 15
U+3FB8.0	fCangjie	KYRB
U+3FB8.0	fFourCorner	00120
U+3FB8.0	fHakka	 gau3
U+3FB8.0	fCantonese	gou2
U+3FB8.0	fMandarin	gao3
U+3FB8.0	fEnglish	eczema
U+3FB9.0	fUTF8	㾹
U+3FB9.0	fCNS11643	4-4B37
U+3FB9.0	fGB	3-5F39
U+3FB9.0	fR/S	 104.10
U+3FB9.0	fTotalStrokes	 15
U+3FB9.0	fCangjie	KYPD
U+3FB9.0	fFourCorner	00190
U+3FB9.0	fCantonese	caai4
U+3FB9.0	fMandarin	chai2
U+3FB9.0	fEnglish	[1] lean; thin; slim [2] illness; disease [3] epidemic; pestilence
U+3FBA.0	fUTF8	㾺
U+3FBA.0	fCNS11643	4-4B35
U+3FBA.0	fGB	3-5E49
U+3FBA.0	fR/S	 104.10
U+3FBA.0	fTotalStrokes	 15
U+3FBA.0	fCangjie	KSQF
U+3FBA.0	fFourCorner	00120
U+3FBA.0	fHakka	 ma5
U+3FBA.0	fCantonese	maa6; maa3
U+3FBA.0	fMandarin	ma4; mo4
U+3FBA.0	fEnglish	[1] eye disease [2] with noxious air on [3] disease of an animal [4] decayed sore
U+3FBB.0	fUTF8	㾻
U+3FBB.0	fCNS11643	4-4B33
U+3FBB.0	fGB	3-5F37
U+3FBB.0	fR/S	 104.10
U+3FBB.0	fTotalStrokes	 15
U+3FBB.0	fCangjie	KNLD
U+3FBB.0	fFourCorner	00190
U+3FBB.0	fHakka	 chu2; cu2
U+3FBB.0	fCantonese	zyu6
U+3FBB.0	fMandarin	chu2; zhu4
U+3FBB.0	fEnglish	[1] not any progressive [2] swelling [3] a mark of scar on the skin
U+3FBC.0	fUTF8	㾼
U+3FBC.0	fCNS11643	4-4B2F
U+3FBC.0	fGB	3-5F36
U+3FBC.0	fR/S	 104.10
U+3FBC.0	fTotalStrokes	 15
U+3FBC.0	fCangjie	KYAV
U+3FBC.0	fFourCorner	00130
U+3FBC.0	fCantonese	teoi2; deoi1
U+3FBC.0	fMandarin	tui3
U+3FBC.0	fEnglish	 suffer a stroke of paralysis or apoplexy
U+3FBD.0	fUTF8	㾽
U+3FBD.0	fCNS11643	7-255C
U+3FBD.0	fGB	5-5D4E
U+3FBD.0	fR/S	 104.10
U+3FBD.0	fTotalStrokes	 15
U+3FBD.0	fCangjie	KYHR
U+3FBD.0	fFourCorner	00130
U+3FBD.0	fHakka	 coi2; ch'oi2; ch'ui2; cui2
U+3FBD.0	fCantonese	zeoi6
U+3FBD.0	fMandarin	zhui4; tui2
U+3FBD.0	fEnglish	same as &#x8187;
U+3FBE.0	fUTF8	㾾
U+3FBE.0	fCNS11643	4-4B2E
U+3FBE.0	fGB	3-5F3D
U+3FBE.0	fR/S	 104.10
U+3FBE.0	fTotalStrokes	 15
U+3FBE.0	fCangjie	KTXC
U+3FBE.0	fFourCorner	00130
U+3FBE.0	fHakka	 liam2;  giam1; hiam1; hiam2
U+3FBE.0	fCantonese	him1; cim1
U+3FBE.0	fMandarin	xian1; lian2
U+3FBE.0	fEnglish	 breathe with difficulty; asthma
U+3FBF.0	fUTF8	㾿
U+3FBF.0	fCNS11643	5-4759
U+3FBF.0	fR/S	 104.10; 104.9
U+3FBF.0	fTotalStrokes	 14; 15
U+3FBF.0	fCangjie	KIIL
U+3FBF.0	fFourCorner	00120
U+3FBF.0	fCantonese	long4
U+3FBF.0	fMandarin	lang2; lang3
U+3FBF.0	fEnglish	roar of asthma or expectorate from a dying person
U+3FC0.0	fUTF8	㿀
U+3FC0.0	fGB	S-2135
U+3FC0.0	fR/S	 104.10
U+3FC0.0	fTotalStrokes	 15
U+3FC0.0	fCangjie	KMGG
U+3FC0.0	fHakka	  ban1
U+3FC0.0	fCantonese	baan1; ban1; baang1
U+3FC0.0	fEnglish	[1] blotches on the skin [2] dark-coloured spots
U+3FC1.0	fUTF8	㿁
U+3FC1.0	fCNS11643	4-4B3B
U+3FC1.0	fGB	3-5F3B
U+3FC1.0	fR/S	 104.10; 104.11
U+3FC1.0	fTotalStrokes	 15; 16
U+3FC1.0	fCangjie	KHAJ
U+3FC1.0	fFourCorner	00140
U+3FC1.0	fCantonese	hiu1
U+3FC2.0	fUTF8	㿂
U+3FC2.0	fCNS11643	7-3036
U+3FC2.0	fR/S	 104.11
U+3FC2.0	fTotalStrokes	 16
U+3FC2.0	fCangjie	KUGK
U+3FC2.0	fFourCorner	00140
U+3FC2.0	fCantonese	zing1
U+3FC2.0	fMandarin	zhi3
U+3FC2.0	fEnglish	abbreviated form of &#x7665;
U+3FC3.0	fUTF8	㿃
U+3FC3.0	fCNS11643	4-5148
U+3FC3.0	fGB	3-5F2F
U+3FC3.0	fR/S	 104.11
U+3FC3.0	fTotalStrokes	 16
U+3FC3.0	fCangjie	KKPB
U+3FC3.0	fFourCorner	00120
U+3FC3.0	fHakka	  dai5; che5; ce5
U+3FC3.0	fCantonese	zai6
U+3FC3.0	fMandarin	zhi4; dai4
U+3FC3.0	fEnglish	flux; discharge
U+3FC4.0	fUTF8	㿄
U+3FC4.0	fCNS11643	4-5149
U+3FC4.0	fGB	3-5F3F
U+3FC4.0	fR/S	 104.11
U+3FC4.0	fTotalStrokes	 16
U+3FC4.0	fCangjie	KSKE
U+3FC4.0	fFourCorner	00140
U+3FC4.0	fCantonese	aai3; ngaai3 (also); aam3; ai3
U+3FC4.0	fMandarin	yi1; ai4
U+3FC4.0	fEnglish	the groans or moans from a person in a critical condition
U+3FC5.0	fUTF8	㿅
U+3FC5.0	fCNS11643	3-5175
U+3FC5.0	fGB	5-5D65
U+3FC5.0	fR/S	 104.11
U+3FC5.0	fTotalStrokes	 16
U+3FC5.0	fCangjie	KHOO
U+3FC5.0	fFourCorner	00180
U+3FC5.0	fCantonese	sin2; sin3
U+3FC5.0	fMandarin	xian3; xuan3
U+3FC5.0	fEnglish	same as &#x766C;
U+3FC6.0	fUTF8	㿆
U+3FC6.0	fCNS11643	4-514E
U+3FC6.0	fGB	5-5D64
U+3FC6.0	fR/S	 104.11
U+3FC6.0	fTotalStrokes	 16
U+3FC6.0	fCangjie	KHDV
U+3FC6.0	fFourCorner	00130
U+3FC6.0	fCantonese	gwo1; go1 (also)
U+3FC6.0	fMandarin	ge1
U+3FC6.0	fEnglish	[1] damage by the pests to the rice seedling [2] boil; ulcer; sore
U+3FC7.0	fUTF8	㿇
U+3FC7.0	fCNS11643	4-514A
U+3FC7.0	fGB	3-5F41
U+3FC7.0	fR/S	 104.11
U+3FC7.0	fTotalStrokes	 16
U+3FC7.0	fCangjie	KSMA
U+3FC7.0	fFourCorner	00160
U+3FC7.0	fCantonese	zaap6
U+3FC7.0	fMandarin	xi2; xi4
U+3FC7.0	fEnglish	[1] paralysis [2] rheumatism [3] having no sense of feeling; numb,
U+3FC8.0	fUTF8	㿈
U+3FC8.0	fCNS11643	4-5150
U+3FC8.0	fR/S	 104.11
U+3FC8.0	fTotalStrokes	 16
U+3FC8.0	fCangjie	KVHG
U+3FC8.0	fFourCorner	00110
U+3FC8.0	fCantonese	jung1; zung1
U+3FC8.0	fMandarin	yong1
U+3FC8.0	fEnglish	non-classical form of &#x7670;
U+3FC9.0	fUTF8	㿉
U+3FC9.0	fCNS11643	4-5726
U+3FC9.0	fGB	5-5D49
U+3FC9.0	fR/S	 104.12
U+3FC9.0	fTotalStrokes	 17; 16
U+3FC9.0	fCangjie	KLMC
U+3FC9.0	fFourCorner	00180
U+3FC9.0	fHakka	 tui2
U+3FC9.0	fCantonese	teoi4
U+3FC9.0	fMandarin	tui2
U+3FC9.0	fEnglish	disease of the private part of the human body
U+3FCA.0	fUTF8	㿊
U+3FCA.0	fCNS11643	4-5722
U+3FCA.0	fGB	3-5F42
U+3FCA.0	fR/S	 104.12
U+3FCA.0	fTotalStrokes	 17; 16
U+3FCA.0	fCangjie	KMUA
U+3FCA.0	fFourCorner	00160
U+3FCA.0	fCantonese	caam2; cam2; tam4
U+3FCA.0	fMandarin	qian1; can3
U+3FCA.0	fEnglish	[1] very painful; agonizing [2] same as &#x61AF;
U+3FCB.0	fUTF8	㿋
U+3FCB.0	fCNS11643	4-5B7B
U+3FCB.0	fR/S	 104.13
U+3FCB.0	fTotalStrokes	 18
U+3FCB.0	fCangjie	KRRD
U+3FCB.0	fFourCorner	00190
U+3FCB.0	fCantonese	sou3
U+3FCB.0	fMandarin	sao1; sao4
U+3FCB.0	fEnglish	same as &#x7619;
U+3FCC.0	fUTF8	㿌
U+3FCC.0	fCNS11643	4-5B77
U+3FCC.0	fGB	3-5E77
U+3FCC.0	fR/S	 104.13
U+3FCC.0	fTotalStrokes	 18
U+3FCC.0	fCangjie	KOMO
U+3FCC.0	fFourCorner	00180
U+3FCC.0	fCantonese	him1; cim1; zim3
U+3FCC.0	fMandarin	xian1
U+3FCC.0	fEnglish	interchangeable &#x3FBE;
U+3FCD.0	fUTF8	㿍
U+3FCD.0	fCNS11643	7-406B
U+3FCD.0	fR/S	 104.13
U+3FCD.0	fTotalStrokes	 18
U+3FCD.0	fCangjie	KNBQ
U+3FCD.0	fFourCorner	00150
U+3FCD.0	fCantonese	gaai3
U+3FCD.0	fMandarin	jie4
U+3FCD.0	fEnglish	same as &#x75A5;
U+3FCE.0	fUTF8	㿎
U+3FCE.0	fCNS11643	4-5725
U+3FCE.0	fGB	3-5F2D
U+3FCE.0	fR/S	 104.13; 104.12
U+3FCE.0	fTotalStrokes	 18; 16; 17
U+3FCE.0	fCangjie	KJTC
U+3FCE.0	fFourCorner	00180
U+3FCE.0	fHakka	 fun2
U+3FCE.0	fCantonese	fan6; faan4; faan5; gak3
U+3FCE.0	fMandarin	fen4
U+3FCE.0	fEnglish	pimples caused by fever
U+3FCF.0	fUTF8	㿏
U+3FCF.0	fCNS11643	7-4067
U+3FCF.0	fR/S	 104.13
U+3FCF.0	fTotalStrokes	 18
U+3FCF.0	fCangjie	KSRQ
U+3FCF.0	fFourCorner	00150
U+3FCF.0	fCantonese	kwan4
U+3FCF.0	fMandarin	wan2; qun2
U+3FCF.0	fEnglish	paralysis of the body
U+3FD0.0	fUTF8	㿐
U+3FD0.0	fR/S	 104.13
U+3FD0.0	fTotalStrokes	 18
U+3FD0.0	fCangjie	KGON
U+3FD0.0	fCantonese	ceoi1
U+3FD0.0	fEnglish	[Korean character]
U+3FD1.0	fUTF8	㿑
U+3FD1.0	fCNS11643	4-6032
U+3FD1.0	fGB	3-5F46
U+3FD1.0	fR/S	 104.14
U+3FD1.0	fTotalStrokes	 19
U+3FD1.0	fCangjie	KSMG
U+3FD1.0	fFourCorner	00110
U+3FD1.0	fCantonese	joek6; zoek6
U+3FD1.0	fMandarin	yao4; yue4
U+3FD1.0	fEnglish	illness; disease; ailment
U+3FD2.0	fUTF8	㿒
U+3FD2.0	fCNS11643	4-602F
U+3FD2.0	fGB	3-5E6D
U+3FD2.0	fR/S	 104.14
U+3FD2.0	fTotalStrokes	 19
U+3FD2.0	fCangjie	KGNI
U+3FD2.0	fFourCorner	00140
U+3FD2.0	fCantonese	dou2; tou2
U+3FD2.0	fMandarin	dao3; zhou4; zhou3; chou2
U+3FD2.0	fEnglish	[1] illness; disease; ailment [2] disease of the internal organs [3] disease of the abdomen [4] palpitation of the heart
U+3FD3.0	fUTF8	㿓
U+3FD3.0	fCNS11643	4-6030
U+3FD3.0	fGB	3-5F43
U+3FD3.0	fR/S	 104.14
U+3FD3.0	fTotalStrokes	 19
U+3FD3.0	fCangjie	KKOO
U+3FD3.0	fFourCorner	00180
U+3FD3.0	fCantonese	gaap3
U+3FD3.0	fMandarin	jia2; qia4
U+3FD3.0	fEnglish	disease of the hoof
U+3FD4.0	fUTF8	㿔
U+3FD4.0	fR/S	 104.15
U+3FD4.0	fTotalStrokes	 20
U+3FD4.0	fCangjie	KWWW
U+3FD4.0	fCantonese	leoi5
U+3FD4.0	fMandarin	lei3
U+3FD4.0	fEnglish	wart; pimple; pustule on the skin
U+3FD5.0	fUTF8	㿕
U+3FD5.0	fCNS11643	4-6658
U+3FD5.0	fGB	3-5F40
U+3FD5.0	fR/S	 104.16
U+3FD5.0	fTotalStrokes	 21
U+3FD5.0	fCangjie	KANX
U+3FD5.0	fFourCorner	00120
U+3FD5.0	fCantonese	jim4; zim4
U+3FD5.0	fMandarin	yan2
U+3FD5.0	fEnglish	[1] ulcer; sore; boil [2] wound
U+3FD6.0	fUTF8	㿖
U+3FD6.0	fCNS11643	4-6659
U+3FD6.0	fGB	3-5E57
U+3FD6.0	fR/S	 104.16
U+3FD6.0	fTotalStrokes	 21
U+3FD6.0	fCangjie	KYPT
U+3FD6.0	fFourCorner	00110
U+3FD6.0	fHakka	 lu2
U+3FD6.0	fCantonese	lou4
U+3FD6.0	fMandarin	lu2
U+3FD6.0	fEnglish	kind of ulcer
U+3FD7.0	fUTF8	㿗
U+3FD7.0	fCNS11643	3-5E40
U+3FD7.0	fGB	5-5D6F
U+3FD7.0	fR/S	 104.16
U+3FD7.0	fTotalStrokes	 21
U+3FD7.0	fCangjie	KHUC
U+3FD7.0	fFourCorner	00180
U+3FD7.0	fHakka	 tui1; tui2
U+3FD7.0	fCantonese	teoi4
U+3FD7.0	fMandarin	tui2
U+3FD7.0	fEnglish	a spasm caused by stone
U+3FD8.0	fUTF8	㿘
U+3FD8.0	fCNS11643	5-735C
U+3FD8.0	fGB	3-5F3E
U+3FD8.0	fR/S	 104.17
U+3FD8.0	fTotalStrokes	 22
U+3FD8.0	fCangjie	KFFR
U+3FD8.0	fFourCorner	00160
U+3FD8.0	fCantonese	jing4; zing4
U+3FD8.0	fMandarin	ying2
U+3FD8.0	fEnglish	 get sick; fall ill
U+3FD9.0	fUTF8	㿙
U+3FD9.0	fCNS11643	4-6A55
U+3FD9.0	fGB	3-5F47
U+3FD9.0	fR/S	 104.18
U+3FD9.0	fTotalStrokes	 23
U+3FD9.0	fCangjie	KWLK
U+3FD9.0	fFourCorner	00130
U+3FD9.0	fCantonese	pei3
U+3FD9.0	fMandarin	pi4; bei4
U+3FD9.0	fEnglish	[1] full; filled [2] full of air or gas [3] a heart full of (enthusiasm, sorrow, etc.)
U+3FDA.0	fUTF8	㿚
U+3FDA.0	fCNS11643	4-6B72
U+3FDA.0	fGB	3-5F24
U+3FDA.0	fR/S	 104.19
U+3FDA.0	fTotalStrokes	 24
U+3FDA.0	fCangjie	KWLG
U+3FDA.0	fFourCorner	00110
U+3FDA.0	fCantonese	lo6
U+3FDA.0	fMandarin	luo4
U+3FDA.0	fEnglish	sickness; ailment; disease
U+3FDB.0	fUTF8	㿛
U+3FDB.0	fCNS11643	4-6B71
U+3FDB.0	fGB	3-5E70
U+3FDB.0	fR/S	 104.19
U+3FDB.0	fTotalStrokes	 24
U+3FDB.0	fCangjie	KMMP
U+3FDB.0	fFourCorner	00110
U+3FDB.0	fCantonese	lai6
U+3FDB.0	fMandarin	li4; li2; li3
U+3FDB.0	fEnglish	[1] carbuncle [2] sallow and emaciated [3] scrofulous swellings
U+3FDC.0	fUTF8	㿜
U+3FDC.0	fCNS11643	3-622D
U+3FDC.0	fR/S	 104.23
U+3FDC.0	fTotalStrokes	 28
U+3FDC.0	fCangjie	KHUB
U+3FDC.0	fFourCorner	00120
U+3FDC.0	fCantonese	bit6
U+3FDC.0	fMandarin	bie3
U+3FDC.0	fEnglish	non-classical form of &#x765F;
U+3FDD.0	fUTF8	㿝
U+3FDD.0	fCNS11643	3-2675
U+3FDD.0	fGB	5-5B50
U+3FDD.0	fR/S	 106.2
U+3FDD.0	fTotalStrokes	 7
U+3FDD.0	fCangjie	HAI
U+3FDD.0	fFourCorner	26730
U+3FDD.0	fCantonese	hoeng1
U+3FDD.0	fMandarin	zao4
U+3FDD.0	fEnglish	ancient form of &#x9999;
U+3FDE.0	fUTF8	㿞
U+3FDE.0	fCNS11643	3-2E30
U+3FDE.0	fR/S	 106.4
U+3FDE.0	fTotalStrokes	 9
U+3FDE.0	fCangjie	HAHQU
U+3FDE.0	fFourCorner	22610
U+3FDE.0	fCantonese	mou6
U+3FDE.0	fMandarin	mao4
U+3FDE.0	fEnglish	non-classcial form of &#x8004;
U+3FDF.0	fUTF8	㿟
U+3FDF.0	fCNS11643	3-3258
U+3FDF.0	fGB	5-5B55
U+3FDF.0	fR/S	 106.5
U+3FDF.0	fTotalStrokes	 10
U+3FDF.0	fCangjie	HAHA
U+3FDF.0	fFourCorner	22660
U+3FDF.0	fHakka	 pet7;  gau3
U+3FDF.0	fCantonese	baak6; gau2
U+3FDF.0	fMandarin	bo2; jiao3; bai2; po4; pai1
U+3FDF.0	fEnglish	same as &#x767D;
U+3FE0.0	fUTF8	㿠
U+3FE0.0	fR/S	 106.6
U+3FE0.0	fTotalStrokes	 11
U+3FE0.0	fCangjie	HAFMU
U+3FE0.0	fFourCorner	2961
U+3FE0.0	fCantonese	gwong1; gong1 (also)
U+3FE0.0	fEnglish	[Korean character]
U+3FE1.0	fUTF8	㿡
U+3FE1.0	fCNS11643	3-374E
U+3FE1.0	fR/S	 106.6
U+3FE1.0	fTotalStrokes	 11
U+3FE1.0	fCangjie	HALMO
U+3FE1.0	fFourCorner	26110
U+3FE1.0	fCantonese	dau1
U+3FE1.0	fMandarin	dou1
U+3FE1.0	fEnglish	corrupted form of &#x515C;
U+3FE2.0	fUTF8	㿢
U+3FE2.0	fCNS11643	3-4322
U+3FE2.0	fR/S	 106.8
U+3FE2.0	fTotalStrokes	 13
U+3FE2.0	fCangjie	ABHA
U+3FE2.0	fFourCorner	67600
U+3FE2.0	fCantonese	jiu6; ziu6
U+3FE2.0	fMandarin	yao4; zhui4
U+3FE2.0	fEnglish	same as &#x8000;
U+3FE3.0	fUTF8	㿣
U+3FE3.0	fCNS11643	3-482D
U+3FE3.0	fGB	5-5B62
U+3FE3.0	fR/S	 106.9
U+3FE3.0	fTotalStrokes	 14
U+3FE3.0	fCangjie	HAAPV
U+3FE3.0	fFourCorner	26620
U+3FE3.0	fCantonese	hot3
U+3FE3.0	fMandarin	he1; he2; xia2
U+3FE3.0	fEnglish	white color
U+3FE4.0	fUTF8	㿤
U+3FE4.0	fCNS11643	5-475B
U+3FE4.0	fR/S	 106.9
U+3FE4.0	fTotalStrokes	 14
U+3FE4.0	fCangjie	HAQKA
U+3FE4.0	fFourCorner	25660
U+3FE4.0	fCantonese	ceon2; ceon4
U+3FE4.0	fMandarin	chun3
U+3FE4.0	fEnglish	[1] white [2] clear
U+3FE5.0	fUTF8	㿥
U+3FE5.0	fCNS11643	5-4E59
U+3FE5.0	fGB	3-5C61
U+3FE5.0	fR/S	 106.10
U+3FE5.0	fTotalStrokes	 15
U+3FE5.0	fCangjie	HAOBG
U+3FE5.0	fFourCorner	24614
U+3FE5.0	fCantonese	hok6; huk6; lok6
U+3FE5.0	fMandarin	hu2; luo4; xue2; xiao2; he4
U+3FE5.0	fEnglish	[1] shine; clear and pure white feather [2] same as &#x76AC;
U+3FE6.0	fUTF8	㿦
U+3FE6.0	fCNS11643	4-5156
U+3FE6.0	fGB	3-5C62
U+3FE6.0	fR/S	 106.11
U+3FE6.0	fTotalStrokes	 16
U+3FE6.0	fCangjie	GEHA
U+3FE6.0	fFourCorner	47600
U+3FE6.0	fCantonese	ning6; ling6 (also)
U+3FE6.0	fMandarin	ning4
U+3FE6.0	fEnglish	[1] tell; inform; report [2] accuse
U+3FE7.0	fUTF8	㿧
U+3FE7.0	fCNS11643	4-6037
U+3FE7.0	fGB	5-5B5E
U+3FE7.0	fR/S	 106.14
U+3FE7.0	fTotalStrokes	 19
U+3FE7.0	fCangjie	HAGNI
U+3FE7.0	fFourCorner	24640
U+3FE7.0	fCantonese	cau4; caau4; mai4
U+3FE7.0	fMandarin	chou2
U+3FE7.0	fEnglish	same as &#x7587;
U+3FE8.0	fUTF8	㿨
U+3FE8.0	fCNS11643	4-665B
U+3FE8.0	fGB	5-5B51
U+3FE8.0	fR/S	 106.16
U+3FE8.0	fTotalStrokes	 21
U+3FE8.0	fCangjie	HAMHM
U+3FE8.0	fFourCorner	21610
U+3FE8.0	fCantonese	lik6
U+3FE8.0	fMandarin	li4
U+3FE8.0	fEnglish	same as &#x76AA;
U+3FE9.0	fUTF8	㿩
U+3FE9.0	fCNS11643	4-6C73
U+3FE9.0	fGB	5-5B63
U+3FE9.0	fR/S	 106.20
U+3FE9.0	fTotalStrokes	 25
U+3FE9.0	fCangjie	HAFBF
U+3FE9.0	fFourCorner	29630
U+3FE9.0	fCantonese	tong2
U+3FE9.0	fMandarin	tang3
U+3FE9.0	fEnglish	[1] light; bright [2] white color
U+3FEA.0	fUTF8	㿪
U+3FEA.0	fCNS11643	4-2746
U+3FEA.0	fGB	3-5F76
U+3FEA.0	fR/S	 107.3
U+3FEA.0	fTotalStrokes	 8
U+3FEA.0	fCangjie	DEKNI
U+3FEA.0	fFourCorner	45217
U+3FEA.0	fCantonese	jyun4; wun4; zyun6
U+3FEA.0	fMandarin	huan2
U+3FEA.0	fEnglish	skin disease
U+3FEB.0	fUTF8	㿫
U+3FEB.0	fCNS11643	4-2A4E
U+3FEB.0	fGB	3-463A
U+3FEB.0	fR/S	 107.4
U+3FEB.0	fTotalStrokes	 9
U+3FEB.0	fCangjie	PPDHE
U+3FEB.0	fFourCorner	21240
U+3FEB.0	fHakka	 zi5; pi5
U+3FEB.0	fCantonese	bei3; maa6; maat6
U+3FEB.0	fMandarin	bi4
U+3FEB.0	fEnglish	 split tangled hemp
U+3FEC.0	fUTF8	㿬
U+3FEC.0	fCNS11643	3-2E31
U+3FEC.0	fGB	5-5E67
U+3FEC.0	fR/S	 107.4
U+3FEC.0	fTotalStrokes	 9
U+3FEC.0	fCangjie	DEAU
U+3FEC.0	fFourCorner	47210
U+3FEC.0	fCantonese	baa1; baat6
U+3FEC.0	fMandarin	ba1
U+3FEC.0	fEnglish	[1] nasal disease [2] same as &#x75A4; [3] same as &#x5DF4;
U+3FED.0	fUTF8	㿭
U+3FED.0	fCNS11643	4-2E46
U+3FED.0	fGB	3-5F78
U+3FED.0	fR/S	 107.5
U+3FED.0	fTotalStrokes	 10
U+3FED.0	fCangjie	HYDHE
U+3FED.0	fFourCorner	74240
U+3FED.0	fHakka	 cak7
U+3FED.0	fCantonese	caak3
U+3FED.0	fMandarin	che4
U+3FED.0	fEnglish	the wrinkles in the face
U+3FEE.0	fUTF8	㿮
U+3FEE.0	fCNS11643	4-2E45
U+3FEE.0	fGB	3-5F77
U+3FEE.0	fR/S	 107.5
U+3FEE.0	fTotalStrokes	 10
U+3FEE.0	fCangjie	LKDHE
U+3FEE.0	fFourCorner	54040
U+3FEE.0	fCantonese	joeng3; zing4
U+3FEE.0	fMandarin	ying1; yang4
U+3FEE.0	fEnglish	[1] green; blue; black [2] extravasation of blood [3] look pale or pallid (of the facial complexion)
U+3FEF.0	fUTF8	㿯
U+3FEF.0	fCNS11643	4-3347
U+3FEF.0	fGB	3-5F7A
U+3FEF.0	fR/S	 107.6
U+3FEF.0	fTotalStrokes	 11
U+3FEF.0	fCangjie	ORDHE
U+3FEF.0	fFourCorner	80240
U+3FEF.0	fHakka	 dap7
U+3FEF.0	fCantonese	daap3; dap6
U+3FEF.0	fMandarin	da2
U+3FEF.0	fEnglish	the skin hanging loosely
U+3FF0.0	fUTF8	㿰
U+3FF0.0	fCNS11643	4-3348
U+3FF0.0	fGB	3-5F7B
U+3FF0.0	fR/S	 107.6
U+3FF0.0	fTotalStrokes	 11
U+3FF0.0	fCangjie	YKDHE
U+3FF0.0	fFourCorner	04440
U+3FF0.0	fCantonese	ngaau4; aau4 (also); bik1; ngou4
U+3FF0.0	fMandarin	ao2; bi4
U+3FF0.0	fEnglish	durable and solid leather
U+3FF1.0	fUTF8	㿱
U+3FF1.0	fCNS11643	4-393A
U+3FF1.0	fGB	3-5F7C
U+3FF1.0	fR/S	 107.7
U+3FF1.0	fTotalStrokes	 12
U+3FF1.0	fCangjie	QLDHE
U+3FF1.0	fFourCorner	52240
U+3FF1.0	fCantonese	zyut6; hok6; hon3
U+3FF1.0	fMandarin	xue2
U+3FF1.0	fEnglish	[1] dry and decayed [2] peel off the skin; scrape; pare
U+3FF2.0	fUTF8	㿲
U+3FF2.0	fCNS11643	6-5552
U+3FF2.0	fR/S	 107.8
U+3FF2.0	fTotalStrokes	 13
U+3FF2.0	fCangjie	KRDHE
U+3FF2.0	fFourCorner	44640
U+3FF2.0	fCantonese	ji1; zi1
U+3FF2.0	fMandarin	yi1
U+3FF2.0	fEnglish	[1] same as &#x6B39; [2] interchangeable with &#x9A0E;
U+3FF3.0	fUTF8	㿳
U+3FF3.0	fCNS11643	4-4530
U+3FF3.0	fGB	3-5F7E
U+3FF3.0	fR/S	 107.8; 107.9
U+3FF3.0	fTotalStrokes	 14; 13
U+3FF3.0	fCangjie	DEVVW
U+3FF3.0	fFourCorner	42260
U+3FF3.0	fCantonese	zi1
U+3FF3.0	fMandarin	zi1
U+3FF4.0	fUTF8	㿴
U+3FF4.0	fCNS11643	4-4532
U+3FF4.0	fGB	3-6021
U+3FF4.0	fR/S	 107.9
U+3FF4.0	fTotalStrokes	 14
U+3FF4.0	fCangjie	KJDHE
U+3FF4.0	fFourCorner	44140
U+3FF4.0	fCantonese	daap3
U+3FF4.0	fMandarin	da1
U+3FF4.0	fEnglish	wide piece of leather
U+3FF5.0	fUTF8	㿵
U+3FF5.0	fCNS11643	3-482F
U+3FF5.0	fR/S	 107.9
U+3FF5.0	fTotalStrokes	 14
U+3FF5.0	fCangjie	DEMBK
U+3FF5.0	fFourCorner	41230
U+3FF5.0	fCantonese	lou4; zim5
U+3FF5.0	fMandarin	ran3
U+3FF5.0	fEnglish	soft leather
U+3FF6.0	fUTF8	㿶
U+3FF6.0	fCNS11643	3-4D43
U+3FF6.0	fR/S	 107.10
U+3FF6.0	fTotalStrokes	 15
U+3FF6.0	fCangjie	YSDHE
U+3FF6.0	fFourCorner	04240
U+3FF6.0	fCantonese	bong1
U+3FF6.0	fMandarin	bang1
U+3FF6.0	fEnglish	same as &#x5E6B;
U+3FF7.0	fUTF8	㿷
U+3FF7.0	fCNS11643	4-4B3D
U+3FF7.0	fGB	3-6022
U+3FF7.0	fR/S	 107.10
U+3FF7.0	fTotalStrokes	 15
U+3FF7.0	fCangjie	TMDHE
U+3FF7.0	fFourCorner	84240
U+3FF7.0	fHakka	 cau5; co2
U+3FF7.0	fCantonese	co4; cou3
U+3FF7.0	fMandarin	chui4; zao4; chui3; cuo2; cao4
U+3FF7.0	fEnglish	same as &#x7CD9;
U+3FF8.0	fUTF8	㿸
U+3FF8.0	fCNS11643	4-5157
U+3FF8.0	fGB	3-6023
U+3FF8.0	fR/S	 107.11
U+3FF8.0	fTotalStrokes	 16
U+3FF8.0	fCangjie	DEAWE
U+3FF8.0	fFourCorner	46240
U+3FF8.0	fHakka	 man6; man5
U+3FF8.0	fCantonese	maan5
U+3FF8.0	fMandarin	wan3; wan4
U+3FF8.0	fEnglish	[1] cast off the skin; molt [2] skin
U+3FF9.0	fUTF8	㿹
U+3FF9.0	fCNS11643	4-5B7E
U+3FF9.0	fGB	3-5F79
U+3FF9.0	fR/S	 107.13
U+3FF9.0	fTotalStrokes	 18
U+3FF9.0	fCangjie	DEYGQ
U+3FF9.0	fFourCorner	44230
U+3FF9.0	fCantonese	taat3
U+3FF9.0	fMandarin	ta4
U+3FF9.0	fEnglish	jutting on the epidermis or the cuticle (of plants)
U+3FFA.0	fUTF8	㿺
U+3FFA.0	fCNS11643	4-636C
U+3FFA.0	fGB	5-5E75
U+3FFA.0	fR/S	 107.15
U+3FFA.0	fTotalStrokes	 20
U+3FFA.0	fCangjie	AEDHE
U+3FFA.0	fFourCorner	64140
U+3FFA.0	fHakka	 bok7; pok8; pok7
U+3FFA.0	fCantonese	pok1; mok1
U+3FFA.0	fMandarin	bo1; bao2
U+3FFA.0	fEnglish	 chap (skin) from cold
U+3FFB.0	fUTF8	㿻
U+3FFB.0	fCNS11643	4-2748
U+3FFB.0	fGB	3-335E
U+3FFB.0	fR/S	 108.3
U+3FFB.0	fTotalStrokes	 8
U+3FFB.0	fCangjie	MJBT
U+3FFB.0	fFourCorner	10100
U+3FFB.0	fHakka	 gon1
U+3FFB.0	fCantonese	gon1
U+3FFB.0	fMandarin	yu2; gan1
U+3FFB.0	fEnglish	[1] tray; plate; dish [2] large bowl
U+3FFC.0	fUTF8	㿼
U+3FFC.0	fCNS11643	4-2A51
U+3FFC.0	fGB	3-5B39
U+3FFC.0	fR/S	 108.4
U+3FFC.0	fTotalStrokes	 9
U+3FFC.0	fCangjie	MTBT
U+3FFC.0	fFourCorner	10100
U+3FFC.0	fCantonese	jin4; zin4
U+3FFC.0	fMandarin	yan2
U+3FFC.0	fEnglish	[1] (non-classical form) [2] bowl [3] a small shallow container; a small cup
U+3FFD.0	fUTF8	㿽
U+3FFD.0	fCNS11643	4-2A50
U+3FFD.0	fGB	3-5B3A
U+3FFD.0	fR/S	 108.4
U+3FFD.0	fTotalStrokes	 9
U+3FFD.0	fCangjie	CMST
U+3FFD.0	fFourCorner	80100
U+3FFD.0	fCantonese	hei1; gaai1; haai4
U+3FFD.0	fMandarin	xi1
U+3FFD.0	fEnglish	 small bowl; small basin
U+3FFE.0	fUTF8	㿾
U+3FFE.0	fCNS11643	4-2E47
U+3FFE.0	fGB	5-5A5B
U+3FFE.0	fR/S	 108.5
U+3FFE.0	fTotalStrokes	 10
U+3FFE.0	fCangjie	JMNT
U+3FFE.0	fFourCorner	30100
U+3FFE.0	fCantonese	cyu5; bou3
U+3FFE.0	fMandarin	zhu4
U+3FFE.0	fEnglish	vessel; container
U+3FFF.0	fUTF8	㿿
U+3FFF.0	fCNS11643	4-2E49
U+3FFF.0	fGB	3-5F73
U+3FFF.0	fR/S	 108.5
U+3FFF.0	fTotalStrokes	 10
U+3FFF.0	fCangjie	NOBT
U+3FFF.0	fFourCorner	17100
U+3FFF.0	fCantonese	ngaa5; aa5 (also)
U+3FFF.0	fMandarin	ya3
U+3FFF.0	fEnglish	wine cups
U+4000.0	fUTF8	䀀
U+4000.0	fCNS11643	3-3259
U+4000.0	fGB	5-5A5D
U+4000.0	fR/S	 108.5
U+4000.0	fTotalStrokes	 10
U+4000.0	fCangjie	EUBT
U+4000.0	fFourCorner	37100
U+4000.0	fCantonese	faan3; fan3; fan4; fan6
U+4000.0	fMandarin	fan4; fan2
U+4000.0	fEnglish	cups; small cups
U+4001.0	fUTF8	䀁
U+4001.0	fCNS11643	4-334B
U+4001.0	fGB	3-5B3B
U+4001.0	fR/S	 108.6
U+4001.0	fTotalStrokes	 11
U+4001.0	fCangjie	KBBT
U+4001.0	fFourCorner	40100
U+4001.0	fCantonese	jau6; zaau6
U+4001.0	fMandarin	you3; you4; huai2; he2; an1
U+4001.0	fEnglish	[1] small bowl; small basin [2] a kind of vessel to remove (or to strain out) the water
U+4002.0	fUTF8	䀂
U+4002.0	fCNS11643	3-374F
U+4002.0	fGB	5-5A60
U+4002.0	fR/S	 108.6
U+4002.0	fTotalStrokes	 11
U+4002.0	fCangjie	JVBT
U+4002.0	fFourCorner	30100
U+4002.0	fCantonese	on1; ngon1 (also)
U+4002.0	fMandarin	an1
U+4002.0	fEnglish	 big basin
U+4003.0	fUTF8	䀃
U+4003.0	fCNS11643	3-3D4E
U+4003.0	fGB	5-6E40
U+4003.0	fR/S	 108.7
U+4003.0	fTotalStrokes	 12
U+4003.0	fCangjie	MOBT
U+4003.0	fFourCorner	10100
U+4003.0	fCantonese	teoi4; deoi1; seoi1
U+4003.0	fMandarin	tui2
U+4003.0	fEnglish	a kind of vessel; a container
U+4004.0	fUTF8	䀄
U+4004.0	fCNS11643	3-4323
U+4004.0	fGB	5-5A63
U+4004.0	fR/S	 108.8
U+4004.0	fTotalStrokes	 13
U+4004.0	fCangjie	JPHT
U+4004.0	fFourCorner	30100
U+4004.0	fCantonese	mung4
U+4004.0	fMandarin	meng2
U+4004.0	fEnglish	same as &#x995B;
U+4005.0	fUTF8	䀅
U+4005.0	fCNS11643	5-406F
U+4005.0	fR/S	 108.8; 108.7
U+4005.0	fTotalStrokes	 13; 12
U+4005.0	fCangjie	IGBT
U+4005.0	fFourCorner	34100
U+4005.0	fCantonese	se5; se6
U+4005.0	fMandarin	she4; zhe2
U+4005.0	fEnglish	vessel; container
U+4006.0	fUTF8	䀆
U+4006.0	fCNS11643	3-4832
U+4006.0	fR/S	 108.9
U+4006.0	fTotalStrokes	 14
U+4006.0	fCangjie	LMFT
U+4006.0	fFourCorner	50100
U+4006.0	fCantonese	zeon6
U+4006.0	fMandarin	jin4
U+4006.0	fEnglish	standard form of &#x76E1;
U+4007.0	fUTF8	䀇
U+4007.0	fCNS11643	4-515A
U+4007.0	fGB	3-5B43
U+4007.0	fR/S	 108.11
U+4007.0	fTotalStrokes	 16
U+4007.0	fCangjie	JUBT
U+4007.0	fFourCorner	48107
U+4007.0	fHakka	 gu3
U+4007.0	fCantonese	gu2; gwu2 (also)
U+4007.0	fMandarin	gu3; que4
U+4007.0	fEnglish	a pot for boiling meat and vegetables
U+4008.0	fUTF8	䀈
U+4008.0	fCNS11643	4-4536
U+4008.0	fGB	3-5B42
U+4008.0	fR/S	 108.11; 108.9
U+4008.0	fTotalStrokes	 14; 16
U+4008.0	fCangjie	AUBT
U+4008.0	fFourCorner	71100
U+4008.0	fCantonese	kei3; gei1
U+4008.0	fMandarin	ji4; qi4; xi4
U+4008.0	fEnglish	[1] vessel; container [2] a kind of animal with red hair looks like hedgehog
U+4009.0	fUTF8	䀉
U+4009.0	fCNS11643	4-572C
U+4009.0	fGB	3-5B3C
U+4009.0	fR/S	 108.12
U+4009.0	fTotalStrokes	 17; 16
U+4009.0	fCangjie	HKRBT
U+4009.0	fFourCorner	20100
U+4009.0	fCantonese	kiu4; giu2
U+4009.0	fMandarin	qiao2; sha4
U+4009.0	fEnglish	food containers (bowl, basin, etc.) used in ancient times
U+400A.0	fUTF8	䀊
U+400A.0	fCNS11643	4-6038
U+400A.0	fGB	3-5B47
U+400A.0	fR/S	 108.14; 85.16
U+400A.0	fTotalStrokes	 19
U+400A.0	fCangjie	EHBT
U+400A.0	fFourCorner	37107
U+400A.0	fHakka	  gau3
U+400A.0	fCantonese	gaau2; gau2; gei3; gei6; hau6
U+400A.0	fMandarin	jiao3; xiao4; lu4
U+400A.0	fEnglish	[1] small boiler or kettle [2] stir up [3] mix
U+400B.0	fUTF8	䀋
U+400B.0	fCNS11643	3-5F38
U+400B.0	fR/S	 108.16
U+400B.0	fTotalStrokes	 21
U+400B.0	fCangjie	GWBT
U+400B.0	fFourCorner	48100
U+400B.0	fCantonese	jim4; nam4; zim4
U+400B.0	fMandarin	yan2
U+400B.0	fEnglish	same as &#x9E7D;
U+400C.0	fUTF8	䀌
U+400C.0	fR/S	 108.17
U+400C.0	fTotalStrokes	 22
U+400C.0	fCangjie	LQMAT
U+400C.0	fCantonese	sik1
U+400C.0	fEnglish	[Korean character]
U+400D.0	fUTF8	䀍
U+400D.0	fCNS11643	4-6E48
U+400D.0	fGB	5-5A6C
U+400D.0	fR/S	 108.24
U+400D.0	fTotalStrokes	 29
U+400D.0	fCangjie	YCBT
U+400D.0	fFourCorner	07100
U+400D.0	fCantonese	ham3
U+400D.0	fMandarin	kan4
U+400D.0	fEnglish	[1] vessel; container; box; chest; trunk [2] a small cup
U+400E.0	fUTF8	䀎
U+400E.0	fCNS11643	3-2679
U+400E.0	fR/S	 109.2
U+400E.0	fTotalStrokes	 7
U+400E.0	fCangjie	BUMVS
U+400E.0	fFourCorner	61020
U+400E.0	fCantonese	min5
U+400E.0	fMandarin	mian3
U+400E.0	fEnglish	same as non-classical form of &#x7704;
U+400F.0	fUTF8	䀏
U+400F.0	fCNS11643	4-2465
U+400F.0	fGB	5-332E
U+400F.0	fR/S	 109.2
U+400F.0	fTotalStrokes	 7
U+400F.0	fCangjie	PBU
U+400F.0	fFourCorner	27620
U+400F.0	fHakka	 hien5
U+400F.0	fCantonese	jyun6; zyun6
U+400F.0	fMandarin	xian4; xuan4; xun2
U+400F.0	fEnglish	[1] move one's eyes [2] same as &#x7734; [3] interchangeable with &#x7729;
U+4010.0	fUTF8	䀐
U+4010.0	fCNS11643	4-274E
U+4010.0	fGB	3-5923
U+4010.0	fR/S	 109.3
U+4010.0	fTotalStrokes	 8
U+4010.0	fCangjie	BUHHH
U+4010.0	fFourCorner	62020
U+4010.0	fCantonese	saam1
U+4010.0	fMandarin	shan1; san3; shan4
U+4010.0	fEnglish	[1] regard; look up to [2] look [3] take a glance at
U+4011.0	fUTF8	䀑
U+4011.0	fCNS11643	4-274F
U+4011.0	fGB	3-5927
U+4011.0	fR/S	 109.3
U+4011.0	fTotalStrokes	 8
U+4011.0	fCangjie	BUEI
U+4011.0	fFourCorner	67040
U+4011.0	fCantonese	waat3
U+4011.0	fMandarin	wo4; na4; ni4
U+4011.0	fEnglish	(corrupted form) gouge out an eye or eyes
U+4012.0	fUTF8	䀒
U+4012.0	fCNS11643	4-274D
U+4012.0	fGB	3-5921
U+4012.0	fR/S	 109.3
U+4012.0	fTotalStrokes	 8
U+4012.0	fCangjie	BUHJ
U+4012.0	fFourCorner	62040
U+4012.0	fCantonese	cin1
U+4012.0	fMandarin	qian1
U+4012.0	fEnglish	gloomy; dark; obscure
U+4013.0	fUTF8	䀓
U+4013.0	fCNS11643	4-274C
U+4013.0	fGB	3-5924
U+4013.0	fR/S	 109.3
U+4013.0	fTotalStrokes	 8
U+4013.0	fCangjie	BUKNI
U+4013.0	fFourCorner	64010
U+4013.0	fCantonese	wun6
U+4013.0	fMandarin	huan4
U+4013.0	fEnglish	[1] turn one's eyes to [2] big eyes
U+4014.0	fUTF8	䀔
U+4014.0	fCNS11643	4-274A
U+4014.0	fGB	5-5878
U+4014.0	fR/S	 109.3
U+4014.0	fTotalStrokes	 8
U+4014.0	fCangjie	BUSHI
U+4014.0	fFourCorner	67020
U+4014.0	fCantonese	jan6; zing6
U+4014.0	fMandarin	ren4; niu2
U+4014.0	fEnglish	[1] look [2] confuse; dazzle
U+4015.0	fUTF8	䀕
U+4015.0	fCNS11643	4-2A53
U+4015.0	fGB	3-593A
U+4015.0	fR/S	 109.4
U+4015.0	fTotalStrokes	 9
U+4015.0	fCangjie	BUNL
U+4015.0	fFourCorner	62000
U+4015.0	fCantonese	zan6; zaam6
U+4015.0	fMandarin	zhen4; cheng4
U+4015.0	fEnglish	[1] angry glances; look angrily [2] eyeballs
U+4016.0	fUTF8	䀖
U+4016.0	fCNS11643	4-2A56
U+4016.0	fGB	3-5929
U+4016.0	fR/S	 109.4
U+4016.0	fTotalStrokes	 9
U+4016.0	fCangjie	BUMK
U+4016.0	fFourCorner	61030
U+4016.0	fCantonese	tin1
U+4016.0	fMandarin	tian1
U+4016.0	fEnglish	 look up to; respect
U+4017.0	fUTF8	䀗
U+4017.0	fCNS11643	4-2A57
U+4017.0	fGB	3-5939
U+4017.0	fR/S	 109.4
U+4017.0	fTotalStrokes	 9
U+4017.0	fCangjie	BUDK
U+4017.0	fFourCorner	65030
U+4017.0	fCantonese	kyut3
U+4017.0	fMandarin	jue2
U+4017.0	fEnglish	[1] look askance at [2] dislike [3] a kind of eye disease (tears all the times) [4] pretty eyes [5] same as &#x77B2;
U+4018.0	fUTF8	䀘
U+4018.0	fCNS11643	4-2A54
U+4018.0	fGB	3-5928
U+4018.0	fR/S	 109.4
U+4018.0	fTotalStrokes	 9
U+4018.0	fCangjie	BUMT
U+4018.0	fFourCorner	61040
U+4018.0	fCantonese	kwai4; haai4
U+4018.0	fMandarin	qi1; xi2; xie2
U+4018.0	fEnglish	[1] (a corrupted abbreviated form) [2] cover the line of vision or sight - the straight line between an object and one's eyes [3] look steadily at [4] look at impatiently
U+4019.0	fUTF8	䀙
U+4019.0	fCNS11643	4-2A58
U+4019.0	fGB	3-592E
U+4019.0	fR/S	 109.4
U+4019.0	fTotalStrokes	 9
U+4019.0	fCangjie	BUPSH
U+4019.0	fFourCorner	67020
U+4019.0	fCantonese	cai3
U+4019.0	fMandarin	qi4
U+4019.0	fEnglish	[1] look at [2] examine; observe; survey
U+401A.0	fUTF8	䀚
U+401A.0	fCNS11643	5-2A6D
U+401A.0	fR/S	 109.4
U+401A.0	fTotalStrokes	 9
U+401A.0	fCangjie	BUHVL
U+401A.0	fFourCorner	60720
U+401A.0	fHakka	 nang5
U+401A.0	fCantonese	ngong4; ong4 (also); ngau4
U+401A.0	fMandarin	ang2
U+401A.0	fEnglish	 lift up the eyes and look around
U+401B.0	fUTF8	䀛
U+401B.0	fCNS11643	4-2A5A
U+401B.0	fGB	3-5937
U+401B.0	fR/S	 109.4
U+401B.0	fTotalStrokes	 9
U+401B.0	fCangjie	BUPHH
U+401B.0	fFourCorner	67020
U+401B.0	fHakka	 moi5; moi6; moe5
U+401B.0	fCantonese	maai6; maat6; mui6
U+401B.0	fMandarin	wu4; mei4
U+401B.0	fEnglish	eyes dim from old age; to see dimly
U+401C.0	fUTF8	䀜
U+401C.0	fCNS11643	6-3338
U+401C.0	fR/S	 109.4
U+401C.0	fTotalStrokes	 9
U+401C.0	fCangjie	PHBU
U+401C.0	fFourCorner	27600
U+401C.0	fHakka	 fut7; moe5
U+401C.0	fCantonese	maai6; faat1
U+401C.0	fMandarin	hu1; gu3; mei4; xue2
U+401C.0	fEnglish	 see obscurely, dawn
U+401D.0	fUTF8	䀝
U+401D.0	fGB	S-2131
U+401D.0	fR/S	 109.4
U+401D.0	fTotalStrokes	 9
U+401D.0	fCangjie	BUPP
U+401D.0	fFourCorner	6101
U+401D.0	fCantonese	bei2
U+401E.0	fUTF8	䀞
U+401E.0	fCNS11643	3-2E34
U+401E.0	fR/S	 109.4
U+401E.0	fTotalStrokes	 9
U+401E.0	fCangjie	BUYJ
U+401E.0	fFourCorner	64000
U+401E.0	fCantonese	tou1
U+401E.0	fMandarin	tao1
U+401E.0	fEnglish	with heavy eyelids
U+401F.0	fUTF8	䀟
U+401F.0	fCNS11643	4-2E52
U+401F.0	fGB	3-594D
U+401F.0	fR/S	 109.5
U+401F.0	fTotalStrokes	 10
U+401F.0	fCangjie	BULLN
U+401F.0	fFourCorner	65020
U+401F.0	fCantonese	fai6; faai3; maan5
U+401F.0	fMandarin	fei4; fan2; fen4
U+401F.0	fEnglish	sight blurred; unclear; dim
U+4020.0	fUTF8	䀠
U+4020.0	fCNS11643	3-325D
U+4020.0	fGB	5-5926
U+4020.0	fR/S	 109.5
U+4020.0	fTotalStrokes	 10
U+4020.0	fCangjie	BUBU
U+4020.0	fFourCorner	66000
U+4020.0	fHakka	 gi5; gi3; ki3
U+4020.0	fCantonese	keoi4
U+4020.0	fMandarin	ju4; qu2
U+4020.0	fEnglish	 look right and left
U+4021.0	fUTF8	䀡
U+4021.0	fCNS11643	3-3266
U+4021.0	fGB	5-5923
U+4021.0	fR/S	 109.5
U+4021.0	fTotalStrokes	 10
U+4021.0	fCangjie	BUYR
U+4021.0	fFourCorner	61060
U+4021.0	fHakka	 zham5; cham5; cam5; zham1; zam1
U+4021.0	fCantonese	zim1
U+4021.0	fMandarin	chan4; tan4
U+4021.0	fEnglish	 peep; spy
U+4022.0	fUTF8	䀢
U+4022.0	fCNS11643	3-3260
U+4022.0	fGB	5-5927
U+4022.0	fR/S	 109.5
U+4022.0	fTotalStrokes	 10
U+4022.0	fCangjie	BUOK
U+4022.0	fFourCorner	68035
U+4022.0	fCantonese	seon3
U+4022.0	fMandarin	shun4
U+4022.0	fEnglish	[1] same as &#x77AC; [2] interchangeable with &#x7734;, &#x779A;
U+4023.0	fUTF8	䀣
U+4023.0	fCNS11643	4-2E4D
U+4023.0	fGB	3-594C
U+4023.0	fR/S	 109.5
U+4023.0	fTotalStrokes	 10
U+4023.0	fCangjie	BUPH
U+4023.0	fFourCorner	63000
U+4023.0	fHakka	  bi5
U+4023.0	fCantonese	bei3
U+4023.0	fMandarin	bi4; mi4
U+4023.0	fEnglish	 look straight; gaze at
U+4024.0	fUTF8	䀤
U+4024.0	fCNS11643	3-3261
U+4024.0	fR/S	 109.5
U+4024.0	fTotalStrokes	 10
U+4024.0	fCangjie	NYBU
U+4024.0	fFourCorner	23600
U+4024.0	fCantonese	mau6
U+4024.0	fMandarin	mou4; mao4
U+4024.0	fEnglish	insight (version) of jealousy
U+4025.0	fUTF8	䀥
U+4025.0	fGB	S-2133
U+4025.0	fR/S	 109.5
U+4025.0	fTotalStrokes	 10
U+4025.0	fCangjie	BUHVD
U+4025.0	fCantonese	soek3; lik6
U+4025.0	fMandarin	shuo4
U+4026.0	fUTF8	䀦
U+4026.0	fGB	3-593F
U+4026.0	fR/S	 109.5
U+4026.0	fTotalStrokes	 10
U+4026.0	fCangjie	BUJR
U+4026.0	fCantonese	gu2; gwu2 (also)
U+4026.0	fMandarin	gu3
U+4026.0	fEnglish	stare at
U+4027.0	fUTF8	䀧
U+4027.0	fCNS11643	5-3376
U+4027.0	fR/S	 109.6
U+4027.0	fTotalStrokes	 11
U+4027.0	fCangjie	BUTC
U+4027.0	fFourCorner	64080
U+4027.0	fCantonese	hung2; fung2
U+4027.0	fMandarin	hong3
U+4027.0	fEnglish	sight blurred; obscure and dim; unclear
U+4028.0	fUTF8	䀨
U+4028.0	fCNS11643	4-3350
U+4028.0	fGB	3-5956
U+4028.0	fR/S	 109.6
U+4028.0	fTotalStrokes	 11
U+4028.0	fCangjie	BUHJR
U+4028.0	fFourCorner	62060
U+4028.0	fHakka	  kwat7
U+4028.0	fCantonese	gwaat3; fun3
U+4028.0	fMandarin	huan4
U+4028.0	fEnglish	 look at angrily
U+4029.0	fUTF8	䀩
U+4029.0	fCNS11643	4-3351
U+4029.0	fGB	3-595D
U+4029.0	fR/S	 109.6
U+4029.0	fTotalStrokes	 11
U+4029.0	fCangjie	BUHER
U+4029.0	fFourCorner	67060
U+4029.0	fHakka	 lok7; lok8
U+4029.0	fCantonese	lok6; long5
U+4029.0	fMandarin	luo4
U+4029.0	fEnglish	[1] look at sidelong [2] ogle
U+402A.0	fUTF8	䀪
U+402A.0	fCNS11643	3-3756
U+402A.0	fGB	5-5932
U+402A.0	fR/S	 109.6
U+402A.0	fTotalStrokes	 11
U+402A.0	fCangjie	BUHON
U+402A.0	fFourCorner	61020
U+402A.0	fCantonese	hong4
U+402A.0	fMandarin	hang2
U+402A.0	fEnglish	[1] birds flying up and down [2] look at; regard; inspect
U+402B.0	fUTF8	䀫
U+402B.0	fCNS11643	3-3754
U+402B.0	fGB	5-5934
U+402B.0	fR/S	 109.6
U+402B.0	fTotalStrokes	 11
U+402B.0	fCangjie	BUOMR
U+402B.0	fFourCorner	68060
U+402B.0	fHakka	 giap7
U+402B.0	fCantonese	gaap3
U+402B.0	fMandarin	jia2
U+402B.0	fEnglish	the eyes dim and tired
U+402C.0	fUTF8	䀬
U+402C.0	fCNS11643	4-3352
U+402C.0	fGB	3-595A
U+402C.0	fR/S	 109.6
U+402C.0	fTotalStrokes	 11
U+402C.0	fCangjie	BUOMG
U+402C.0	fFourCorner	68010
U+402C.0	fCantonese	cyun4
U+402C.0	fMandarin	quan2
U+402C.0	fEnglish	[1] glance; wink [2] having one eye smaller than the other [3] one-eyed [4] gazing into distance [5] sight blurred; obscure and dim; unclear
U+402D.0	fUTF8	䀭
U+402D.0	fCNS11643	4-334E
U+402D.0	fGB	3-5960
U+402D.0	fR/S	 109.6
U+402D.0	fTotalStrokes	 11
U+402D.0	fCangjie	BUYVO
U+402D.0	fFourCorner	60080
U+402D.0	fHakka	 goi1
U+402D.0	fCantonese	goi1; hai4
U+402D.0	fMandarin	gai1
U+402D.0	fEnglish	with big eyes
U+402E.0	fUTF8	䀮
U+402E.0	fCNS11643	4-334D
U+402E.0	fGB	3-595F
U+402E.0	fR/S	 109.6
U+402E.0	fTotalStrokes	 11
U+402E.0	fCangjie	BUYVU
U+402E.0	fFourCorner	60010
U+402E.0	fCantonese	fong1
U+402E.0	fMandarin	huang1; mang2
U+402E.0	fEnglish	sight blurred; obscure and dim; obscured
U+402F.0	fUTF8	䀯
U+402F.0	fCNS11643	3-3D50
U+402F.0	fR/S	 109.7
U+402F.0	fTotalStrokes	 12
U+402F.0	fCangjie	BUIJB
U+402F.0	fFourCorner	63020
U+402F.0	fHakka	 pu6; pu5; bu1
U+402F.0	fCantonese	bou2; bok6; bou6; pui4
U+402F.0	fMandarin	bu3
U+402F.0	fEnglish	[1] the evening [2] strictly the time from three to five o'clock in the afternoon
U+4030.0	fUTF8	䀰
U+4030.0	fCNS11643	4-3949
U+4030.0	fGB	3-5973
U+4030.0	fR/S	 109.7
U+4030.0	fTotalStrokes	 12
U+4030.0	fCangjie	BUCOR
U+4030.0	fFourCorner	68060
U+4030.0	fCantonese	guk1
U+4030.0	fMandarin	gu3; ying2
U+4030.0	fEnglish	[1] open eyes [2] big eyes [3] move one's eyes
U+4031.0	fUTF8	䀱
U+4031.0	fCNS11643	3-3D54
U+4031.0	fR/S	 109.7
U+4031.0	fTotalStrokes	 12
U+4031.0	fCangjie	BUHEJ
U+4031.0	fFourCorner	67050
U+4031.0	fHakka	 fung1
U+4031.0	fCantonese	fung1
U+4031.0	fMandarin	feng1
U+4031.0	fEnglish	half-open eyes
U+4032.0	fUTF8	䀲
U+4032.0	fCNS11643	4-394A
U+4032.0	fGB	3-596F
U+4032.0	fR/S	 109.7
U+4032.0	fTotalStrokes	 12
U+4032.0	fCangjie	BUOWY
U+4032.0	fFourCorner	68050
U+4032.0	fCantonese	muk6
U+4032.0	fMandarin	mu4
U+4032.0	fEnglish	diseases in the eyes
U+4033.0	fUTF8	䀳
U+4033.0	fCNS11643	3-3D55
U+4033.0	fR/S	 109.7
U+4033.0	fTotalStrokes	 12
U+4033.0	fCangjie	BUDL
U+4033.0	fFourCorner	65090
U+4033.0	fCantonese	laat6; lat3
U+4033.0	fMandarin	la4; ai4; lai4
U+4033.0	fEnglish	[1] (abbreviated form) [2] not to look straight forward [3] the pupil of the eyes not in the right place
U+4034.0	fUTF8	䀴
U+4034.0	fCNS11643	4-3945
U+4034.0	fGB	3-594F
U+4034.0	fR/S	 109.7
U+4034.0	fTotalStrokes	 12
U+4034.0	fCangjie	BUMVM
U+4034.0	fFourCorner	61010
U+4034.0	fCantonese	jing5; zing2
U+4034.0	fMandarin	ying3
U+4034.0	fEnglish	[1] look straight forward [2] look [3] eyesight blurred; not clear
U+4035.0	fUTF8	䀵
U+4035.0	fCNS11643	4-3948
U+4035.0	fGB	5-593F
U+4035.0	fR/S	 109.7
U+4035.0	fTotalStrokes	 12
U+4035.0	fCangjie	BUIOK
U+4035.0	fFourCorner	63030
U+4035.0	fCantonese	seon3; si1
U+4035.0	fMandarin	shi1; shun4
U+4035.0	fEnglish	same as &#x77AC;
U+4036.0	fUTF8	䀶
U+4036.0	fCNS11643	4-393F
U+4036.0	fGB	3-5979
U+4036.0	fR/S	 109.7
U+4036.0	fTotalStrokes	 12
U+4036.0	fCangjie	BUIAV
U+4036.0	fFourCorner	63030
U+4036.0	fCantonese	loeng6; long4; long5
U+4036.0	fMandarin	liang4; lang3
U+4036.0	fEnglish	[1] strabismus; squint [2] look askance; ogle [3] [4] bright; light; brilliant [5] bright eyes
U+4037.0	fUTF8	䀷
U+4037.0	fCNS11643	4-3941
U+4037.0	fGB	3-5965
U+4037.0	fR/S	 109.7
U+4037.0	fTotalStrokes	 12
U+4037.0	fCangjie	BUGIS
U+4037.0	fFourCorner	64020
U+4037.0	fCantonese	gip3
U+4037.0	fMandarin	jie2
U+4037.0	fEnglish	 have quick glance; look-in; look hastily
U+4038.0	fUTF8	䀸
U+4038.0	fCNS11643	4-394B
U+4038.0	fGB	5-593B
U+4038.0	fR/S	 109.7
U+4038.0	fTotalStrokes	 12
U+4038.0	fCangjie	QLBU
U+4038.0	fFourCorner	52600
U+4038.0	fHakka	 zhi5; zih5
U+4038.0	fCantonese	cai3; daai6; zaai3
U+4038.0	fMandarin	chi4; di4; zhi4
U+4038.0	fEnglish	bright eyes
U+4039.0	fUTF8	䀹
U+4039.0	fCNS11643	4-3946
U+4039.0	fGB	3-5952
U+4039.0	fR/S	 109.7
U+4039.0	fTotalStrokes	 12
U+4039.0	fCangjie	BUKOO
U+4039.0	fFourCorner	64030
U+4039.0	fHakka	 ziap7
U+4039.0	fCantonese	sim2; sip3 (also); zip3; zaap3; gap6
U+4039.0	fMandarin	shan3; jia2; she4
U+4039.0	fEnglish	[1] same as &#x7728; [2] same as &#x776B;
U+403A.0	fUTF8	䀺
U+403A.0	fCNS11643	4-394D
U+403A.0	fGB	3-5970
U+403A.0	fR/S	 109.7
U+403A.0	fTotalStrokes	 12
U+403A.0	fCangjie	OKBU
U+403A.0	fFourCorner	28604
U+403A.0	fCantonese	cau1
U+403A.0	fMandarin	chou1
U+403A.0	fEnglish	[1] squinting eyes; prominent eyes [2] all blind [3] eyesight not clear [4] upper eyelid which has a double fold at the lower edge
U+403B.0	fUTF8	䀻
U+403B.0	fCNS11643	4-3947
U+403B.0	fGB	3-596B
U+403B.0	fR/S	 109.7
U+403B.0	fTotalStrokes	 12
U+403B.0	fCangjie	BULWS
U+403B.0	fFourCorner	65020
U+403B.0	fCantonese	ping3; bing2
U+403B.0	fMandarin	ping4; pin4
U+403B.0	fEnglish	 look; regard; inspect
U+403C.0	fUTF8	䀼
U+403C.0	fCNS11643	4-3943
U+403C.0	fGB	5-5939
U+403C.0	fR/S	 109.7
U+403C.0	fTotalStrokes	 12
U+403C.0	fCangjie	BUMMV
U+403C.0	fFourCorner	61030
U+403C.0	fCantonese	can1; caan1; zing6
U+403C.0	fMandarin	chen1; ren4; zhen3
U+403C.0	fEnglish	non-classical form of &#x778B;
U+403D.0	fUTF8	䀽
U+403D.0	fCNS11643	4-3F22
U+403D.0	fGB	3-5957
U+403D.0	fR/S	 109.8; 109.7
U+403D.0	fTotalStrokes	 13; 12
U+403D.0	fCangjie	BUNKM
U+403D.0	fFourCorner	62040
U+403D.0	fCantonese	jin4; zin2; zin4
U+403D.0	fMandarin	yan2
U+403D.0	fEnglish	 look at each other
U+403E.0	fUTF8	䀾
U+403E.0	fCNS11643	4-394C
U+403E.0	fGB	5-593A
U+403E.0	fR/S	 109.7
U+403E.0	fTotalStrokes	 12
U+403E.0	fCangjie	MEBU
U+403E.0	fFourCorner	17600
U+403E.0	fCantonese	dou2
U+403E.0	fMandarin	du3
U+403E.0	fEnglish	[1] many; much [2] more than; over
U+403F.0	fUTF8	䀿
U+403F.0	fCNS11643	4-3942
U+403F.0	fGB	3-5967
U+403F.0	fR/S	 109.7
U+403F.0	fTotalStrokes	 12
U+403F.0	fCangjie	BUQHL
U+403F.0	fFourCorner	62020
U+403F.0	fCantonese	cai3; daai6
U+403F.0	fMandarin	chi4; di4
U+403F.0	fEnglish	same as &#x4038;
U+4040.0	fUTF8	䁀
U+4040.0	fR/S	 109.7
U+4040.0	fTotalStrokes	 12
U+4040.0	fCangjie	BUQHL
U+4040.0	fCantonese	dai3
U+4040.0	fEnglish	[Japanese character]
U+4041.0	fUTF8	䁁
U+4041.0	fCNS11643	4-3F30
U+4041.0	fR/S	 109.8
U+4041.0	fTotalStrokes	 13
U+4041.0	fCangjie	BUYRF
U+4041.0	fFourCorner	60090
U+4041.0	fCantonese	loeng6
U+4041.0	fMandarin	liang4; lang3
U+4041.0	fEnglish	same as &#x4036;
U+4042.0	fUTF8	䁂
U+4042.0	fCNS11643	4-3F2D
U+4042.0	fGB	3-592F
U+4042.0	fR/S	 109.8
U+4042.0	fTotalStrokes	 13
U+4042.0	fCangjie	SEBU
U+4042.0	fFourCorner	77604
U+4042.0	fCantonese	haan6
U+4042.0	fMandarin	xian4
U+4042.0	fEnglish	big eyes
U+4043.0	fUTF8	䁃
U+4043.0	fCNS11643	3-4324
U+4043.0	fGB	5-5940
U+4043.0	fR/S	 109.8
U+4043.0	fTotalStrokes	 13
U+4043.0	fCangjie	BUQMV
U+4043.0	fFourCorner	65030
U+4043.0	fCantonese	biu1
U+4043.0	fMandarin	biao1
U+4043.0	fEnglish	 look attentively; focus one's look at; gaze at
U+4044.0	fUTF8	䁄
U+4044.0	fCNS11643	4-3F25
U+4044.0	fGB	3-597B
U+4044.0	fR/S	 109.8
U+4044.0	fTotalStrokes	 13
U+4044.0	fCangjie	BUGTJ
U+4044.0	fFourCorner	64040
U+4044.0	fCantonese	jing6; haang6
U+4044.0	fMandarin	xing4
U+4044.0	fEnglish	[1] close the eyes [2] die without regreat or in peace
U+4045.0	fUTF8	䁅
U+4045.0	fCNS11643	4-3F26
U+4045.0	fGB	3-5A2B
U+4045.0	fR/S	 109.8
U+4045.0	fTotalStrokes	 13
U+4045.0	fCangjie	BUNDT
U+4045.0	fFourCorner	67010
U+4045.0	fCantonese	maang5
U+4045.0	fMandarin	meng3; bei4; bi4; meng4
U+4045.0	fEnglish	[1] rage; anger [2] cast an angry look; give the black looks
U+4046.0	fUTF8	䁆
U+4046.0	fCNS11643	4-3F27
U+4046.0	fGB	3-597C
U+4046.0	fR/S	 109.8
U+4046.0	fTotalStrokes	 13
U+4046.0	fCangjie	BUKLU
U+4046.0	fFourCorner	64010
U+4046.0	fCantonese	jip3; zaap1
U+4046.0	fMandarin	an3; yi4
U+4046.0	fEnglish	[1] eyes [2] closed eyes
U+4047.0	fUTF8	䁇
U+4047.0	fCNS11643	4-3F2B
U+4047.0	fGB	3-5A26
U+4047.0	fR/S	 109.8
U+4047.0	fTotalStrokes	 13
U+4047.0	fCangjie	JPHU
U+4047.0	fFourCorner	30600
U+4047.0	fCantonese	mik6; maak6; maat6
U+4047.0	fMandarin	mi4
U+4047.0	fEnglish	[1] look for a short time [2] attentive vision
U+4048.0	fUTF8	䁈
U+4048.0	fCNS11643	4-3F2C
U+4048.0	fGB	3-5A27
U+4048.0	fR/S	 109.8
U+4048.0	fTotalStrokes	 13
U+4048.0	fCangjie	IKBU
U+4048.0	fFourCorner	38604
U+4048.0	fCantonese	kai3; hei2; kaai3
U+4048.0	fMandarin	qi3; qi4
U+4048.0	fEnglish	[1] examine; survey; inspect; visit [2] watch or see in secret; spy; peep
U+4049.0	fUTF8	䁉
U+4049.0	fCNS11643	6-5561
U+4049.0	fR/S	 109.8
U+4049.0	fTotalStrokes	 13
U+4049.0	fCangjie	IIBU
U+4049.0	fFourCorner	33605
U+4049.0	fCantonese	kai3; hei2; kaai3
U+4049.0	fMandarin	qi3; qi4
U+4049.0	fEnglish	same as &#x4048;
U+404A.0	fUTF8	䁊
U+404A.0	fCNS11643	5-4761
U+404A.0	fGB	3-5A2A
U+404A.0	fR/S	 109.9
U+404A.0	fTotalStrokes	 14
U+404A.0	fCangjie	BUEIU
U+404A.0	fFourCorner	67060
U+404A.0	fCantonese	waat3
U+404A.0	fMandarin	wo4
U+404A.0	fEnglish	[1] dark and deep eyed [2] look down [3] block [4] stuff [5] cork; seal
U+404B.0	fUTF8	䁋
U+404B.0	fCNS11643	4-4539
U+404B.0	fGB	3-5A2F
U+404B.0	fR/S	 109.9
U+404B.0	fTotalStrokes	 14
U+404B.0	fCangjie	BUPTD
U+404B.0	fFourCorner	64090
U+404B.0	fCantonese	hip3; sip3; sit3
U+404B.0	fMandarin	she2
U+404B.0	fEnglish	[1] with one eye closed [2] close the eyes [3] same as &#x77B8;
U+404C.0	fUTF8	䁌
U+404C.0	fCNS11643	4-4541
U+404C.0	fGB	3-5A34
U+404C.0	fR/S	 109.9
U+404C.0	fTotalStrokes	 14
U+404C.0	fCangjie	BUWB
U+404C.0	fFourCorner	66020
U+404C.0	fCantonese	wat6; waan3
U+404C.0	fMandarin	yu4
U+404C.0	fEnglish	 see for a short time
U+404D.0	fUTF8	䁍
U+404D.0	fCNS11643	4-453C
U+404D.0	fGB	3-5A30
U+404D.0	fR/S	 109.9
U+404D.0	fTotalStrokes	 14
U+404D.0	fCangjie	BUIHR
U+404D.0	fFourCorner	63050
U+404D.0	fCantonese	hap1
U+404D.0	fMandarin	qia4; jia4; ke4
U+404D.0	fEnglish	[1] blind [2] hollow-eyed [3] look at
U+404E.0	fUTF8	䁎
U+404E.0	fCNS11643	4-4537
U+404E.0	fGB	3-5A3A
U+404E.0	fR/S	 109.9
U+404E.0	fTotalStrokes	 14
U+404E.0	fCangjie	BUYRN
U+404E.0	fFourCorner	60020
U+404E.0	fCantonese	caang4
U+404E.0	fMandarin	cheng2
U+404E.0	fEnglish	[1] make a close inspection [2] secretion of the eye [3] same as &#x77A0;
U+404F.0	fUTF8	䁏
U+404F.0	fCNS11643	3-4837
U+404F.0	fGB	5-594E
U+404F.0	fR/S	 109.9
U+404F.0	fTotalStrokes	 14
U+404F.0	fCangjie	BUMWV
U+404F.0	fFourCorner	61040
U+404F.0	fCantonese	jiu2; miu5
U+404F.0	fMandarin	yao3
U+404F.0	fEnglish	[1] farsightedness (as a physical defect); hypermetropia [2] look from a distance
U+4050.0	fUTF8	䁐
U+4050.0	fCNS11643	4-4543
U+4050.0	fGB	5-5942
U+4050.0	fR/S	 109.9
U+4050.0	fTotalStrokes	 14
U+4050.0	fCangjie	BUTLK
U+4050.0	fFourCorner	64034
U+4050.0	fCantonese	jing1; zing1; zing3
U+4050.0	fMandarin	ying1; ying4
U+4050.0	fEnglish	[1] deep eyed [2] look at; inspect; see
U+4051.0	fUTF8	䁑
U+4051.0	fCNS11643	4-4546
U+4051.0	fGB	3-5926
U+4051.0	fR/S	 109.9
U+4051.0	fTotalStrokes	 14
U+4051.0	fCangjie	BUAMH
U+4051.0	fFourCorner	66020
U+4051.0	fCantonese	joeng4; zoeng4
U+4051.0	fMandarin	yang2
U+4051.0	fEnglish	[1] same as &#x773B; [2] interchangeable with &#x63DA;
U+4052.0	fUTF8	䁒
U+4052.0	fCNS11643	4-453F
U+4052.0	fGB	3-5A32
U+4052.0	fR/S	 109.9
U+4052.0	fTotalStrokes	 14
U+4052.0	fCangjie	BURSJ
U+4052.0	fFourCorner	66040
U+4052.0	fCantonese	cap1; caap1; zap6
U+4052.0	fMandarin	ji2
U+4052.0	fEnglish	[1] wink [2] shed tears
U+4053.0	fUTF8	䁓
U+4053.0	fCNS11643	4-4544
U+4053.0	fGB	3-5A39
U+4053.0	fR/S	 109.9
U+4053.0	fTotalStrokes	 14
U+4053.0	fCangjie	BUUCE
U+4053.0	fFourCorner	62040
U+4053.0	fCantonese	zung1; gai3
U+4053.0	fMandarin	zong1; jie4; zong3
U+4053.0	fEnglish	[1] look at; see [2] peep; look something stealthily [3] angry
U+4054.0	fUTF8	䁔
U+4054.0	fCNS11643	3-483A
U+4054.0	fGB	5-5953
U+4054.0	fR/S	 109.9
U+4054.0	fTotalStrokes	 14
U+4054.0	fCangjie	BUBME
U+4054.0	fFourCorner	62040
U+4054.0	fCantonese	hyun1; hon6; wun6
U+4054.0	fMandarin	xuan1; han4; huan3
U+4054.0	fEnglish	[1] big eyes [2] same as &#x7745;
U+4055.0	fUTF8	䁕
U+4055.0	fCNS11643	4-4538
U+4055.0	fGB	3-5A3D
U+4055.0	fR/S	 109.9
U+4055.0	fTotalStrokes	 14
U+4055.0	fCangjie	BURPA
U+4055.0	fFourCorner	67060
U+4055.0	fCantonese	man4
U+4055.0	fMandarin	min2
U+4055.0	fEnglish	same as &#x76FF;
U+4056.0	fUTF8	䁖
U+4056.0	fGB	7-2226
U+4056.0	fR/S	 109.9
U+4056.0	fTotalStrokes	 14
U+4056.0	fCangjie	BUFDV
U+4056.0	fCantonese	lau1; lau4 (also); laau1
U+4056.0	fMandarin	lou1
U+4056.0	fEnglish	simplified form of &#x779C;
U+4057.0	fUTF8	䁗
U+4057.0	fCNS11643	3-4D45
U+4057.0	fGB	5-5931
U+4057.0	fR/S	 109.10
U+4057.0	fTotalStrokes	 15
U+4057.0	fCangjie	BUUMT
U+4057.0	fFourCorner	62010
U+4057.0	fCantonese	hoi2
U+4057.0	fMandarin	kai3
U+4057.0	fEnglish	[1] light; bright; brilliant [2] clear [3] shine upon; light; illumine
U+4058.0	fUTF8	䁘
U+4058.0	fCNS11643	4-4B45
U+4058.0	fGB	3-5A45
U+4058.0	fR/S	 109.10
U+4058.0	fTotalStrokes	 15
U+4058.0	fCangjie	BUBOU
U+4058.0	fFourCorner	67070
U+4058.0	fCantonese	jiu5; ziu4
U+4058.0	fMandarin	yao2; yao3
U+4058.0	fEnglish	[1] look at; see [2] pretty eyes [3] same as &#x404F;
U+4059.0	fUTF8	䁙
U+4059.0	fCNS11643	4-4B42
U+4059.0	fGB	3-5A44
U+4059.0	fR/S	 109.10
U+4059.0	fTotalStrokes	 15
U+4059.0	fCangjie	BUAJV
U+4059.0	fFourCorner	66040
U+4059.0	fCantonese	aan3; ngaan3 (also); zin2
U+4059.0	fMandarin	yan3; yan4
U+4059.0	fEnglish	[1] eloquent eyes; coverse with eyes; make sheep eyes or passes [2] look at; see [3] look up to; respect
U+405A.0	fUTF8	䁚
U+405A.0	fCNS11643	4-4B44
U+405A.0	fGB	3-596D
U+405A.0	fR/S	 109.10
U+405A.0	fTotalStrokes	 15
U+405A.0	fCangjie	BURBC
U+405A.0	fFourCorner	66080
U+405A.0	fCantonese	syun2; king4
U+405A.0	fMandarin	sun3
U+405A.0	fEnglish	(non-classical form) a kind of eye disease
U+405B.0	fUTF8	䁛
U+405B.0	fCNS11643	4-4B47
U+405B.0	fGB	3-5A36
U+405B.0	fR/S	 109.10
U+405B.0	fTotalStrokes	 15
U+405B.0	fCangjie	BUHUI
U+405B.0	fFourCorner	66010
U+405B.0	fCantonese	gwai3; kwaai5
U+405B.0	fMandarin	gui4; gui3; kui4
U+405B.0	fEnglish	 look; see; inspect
U+405C.0	fUTF8	䁜
U+405C.0	fCNS11643	4-4B43
U+405C.0	fGB	3-5A43
U+405C.0	fR/S	 109.10
U+405C.0	fTotalStrokes	 15
U+405C.0	fCangjie	BUAFU
U+405C.0	fFourCorner	66010
U+405C.0	fCantonese	fong2
U+405C.0	fMandarin	huang3; huang4
U+405C.0	fEnglish	[1] big eyes [2] a kind of eye disease
U+405D.0	fUTF8	䁝
U+405D.0	fCNS11643	4-4B4D
U+405D.0	fGB	3-593E
U+405D.0	fR/S	 109.10
U+405D.0	fTotalStrokes	 15
U+405D.0	fCangjie	FFBBU
U+405D.0	fFourCorner	99600
U+405D.0	fCantonese	jing4; gwing1; zing4
U+405D.0	fMandarin	ying2
U+405D.0	fEnglish	[1] deluding and causing disorder [2] interchangeable &#x7192;
U+405E.0	fUTF8	䁞
U+405E.0	fCNS11643	3-4D47
U+405E.0	fR/S	 109.10
U+405E.0	fTotalStrokes	 15
U+405E.0	fCangjie	OOMBU
U+405E.0	fFourCorner	88601
U+405E.0	fCantonese	saang2
U+405E.0	fMandarin	sheng3
U+405E.0	fEnglish	same as ancient form of &#x7701;
U+405F.0	fUTF8	䁟
U+405F.0	fCNS11643	5-4E60
U+405F.0	fR/S	 109.10
U+405F.0	fTotalStrokes	 15
U+405F.0	fCangjie	BUTQM
U+405F.0	fFourCorner	68010
U+405F.0	fCantonese	caa4
U+405F.0	fMandarin	cha2; duo2
U+405F.0	fEnglish	interchangeable with &#x5DEE;
U+4060.0	fUTF8	䁠
U+4060.0	fCNS11643	4-4B3F
U+4060.0	fR/S	 109.10
U+4060.0	fTotalStrokes	 15
U+4060.0	fCangjie	BUTXC
U+4060.0	fFourCorner	68030
U+4060.0	fCantonese	lim4
U+4060.0	fMandarin	lian2
U+4060.0	fEnglish	 hang down (eyes)
U+4061.0	fUTF8	䁡
U+4061.0	fR/S	 109.10
U+4061.0	fTotalStrokes	 15
U+4061.0	fCangjie	BUANO
U+4061.0	fCantonese	sim2
U+4061.0	fEnglish	[Korean character]
U+4062.0	fUTF8	䁢
U+4062.0	fCNS11643	3-5177
U+4062.0	fGB	5-596D
U+4062.0	fR/S	 109.11
U+4062.0	fTotalStrokes	 16
U+4062.0	fCangjie	BUYSO
U+4062.0	fFourCorner	68080
U+4062.0	fCantonese	syun4
U+4062.0	fMandarin	xuan2
U+4062.0	fEnglish	same as &#x5AD9;
U+4063.0	fUTF8	䁣
U+4063.0	fCNS11643	3-5178
U+4063.0	fR/S	 109.11
U+4063.0	fTotalStrokes	 16
U+4063.0	fCangjie	BUJII
U+4063.0	fFourCorner	65040
U+4063.0	fCantonese	syun4; wan4
U+4063.0	fMandarin	chuan2
U+4063.0	fEnglish	 wink
U+4064.0	fUTF8	䁤
U+4064.0	fCNS11643	4-515C
U+4064.0	fGB	3-5A46
U+4064.0	fR/S	 109.11
U+4064.0	fTotalStrokes	 16
U+4064.0	fCangjie	BUYCB
U+4064.0	fFourCorner	60020
U+4064.0	fCantonese	caak3; cing3
U+4064.0	fMandarin	che4; cheng4
U+4064.0	fEnglish	[1] moistened eyes [2] stare in anger or contempt
U+4065.0	fUTF8	䁥
U+4065.0	fCNS11643	4-5160
U+4065.0	fGB	3-5A3F
U+4065.0	fR/S	 109.11
U+4065.0	fTotalStrokes	 16; 6
U+4065.0	fCangjie	BUSTR
U+4065.0	fFourCorner	61010
U+4065.0	fHakka	  nit8; ne1
U+4065.0	fCantonese	nik1; lik1 (also)
U+4065.0	fMandarin	ni4
U+4065.0	fEnglish	 blink; half shut the eyes
U+4066.0	fUTF8	䁦
U+4066.0	fCNS11643	3-517C
U+4066.0	fR/S	 109.11
U+4066.0	fTotalStrokes	 16
U+4066.0	fCangjie	BUYPM
U+4066.0	fFourCorner	61010
U+4066.0	fCantonese	ceoi3; waan2
U+4066.0	fMandarin	qu4
U+4066.0	fEnglish	same as &#x89B0, &#x89B7;
U+4067.0	fUTF8	䁧
U+4067.0	fCNS11643	3-517A
U+4067.0	fR/S	 109.11
U+4067.0	fTotalStrokes	 16
U+4067.0	fCangjie	BUTLO
U+4067.0	fFourCorner	64030
U+4067.0	fCantonese	miu4
U+4067.0	fMandarin	miao2
U+4067.0	fEnglish	 open wide (eyes)
U+4068.0	fUTF8	䁨
U+4068.0	fCNS11643	4-515D
U+4068.0	fGB	5-5962
U+4068.0	fR/S	 109.11
U+4068.0	fTotalStrokes	 16
U+4068.0	fCangjie	BUYDL
U+4068.0	fFourCorner	67020
U+4068.0	fCantonese	fok3; kok3
U+4068.0	fMandarin	huo4
U+4068.0	fEnglish	[1] open the eyes with astonishment [2] interchangeable with &#x77D0;
U+4069.0	fUTF8	䁩
U+4069.0	fCNS11643	4-5164
U+4069.0	fGB	3-5A22
U+4069.0	fR/S	 109.11
U+4069.0	fTotalStrokes	 16
U+4069.0	fCangjie	BUNWF
U+4069.0	fFourCorner	67030
U+4069.0	fHakka	 ng2
U+4069.0	fCantonese	jyu4; zyu4; zyu6
U+4069.0	fMandarin	yu2
U+4069.0	fEnglish	same as &#x9B5A;
U+406A.0	fUTF8	䁪
U+406A.0	fGB	3-597D
U+406A.0	fR/S	 109.11
U+406A.0	fTotalStrokes	 16
U+406A.0	fCangjie	BUJJL
U+406A.0	fCantonese	zaam2; zam2
U+406A.0	fMandarin	nan3; zhan3
U+406A.0	fEnglish	wink
U+406B.0	fUTF8	䁫
U+406B.0	fCNS11643	4-5732
U+406B.0	fGB	3-5A4D
U+406B.0	fR/S	 109.12
U+406B.0	fTotalStrokes	 17; 16
U+406B.0	fCangjie	BUWGF
U+406B.0	fFourCorner	66030
U+406B.0	fCantonese	wat6; wu4
U+406B.0	fMandarin	hu2
U+406B.0	fEnglish	[1] muddy; dirty [2] eyesight obscured
U+406C.0	fUTF8	䁬
U+406C.0	fCNS11643	4-572E
U+406C.0	fGB	3-5A4E
U+406C.0	fR/S	 109.12
U+406C.0	fTotalStrokes	 17; 16
U+406C.0	fCangjie	BUCWA
U+406C.0	fFourCorner	68060
U+406C.0	fHakka	  cen2; zen1
U+406C.0	fCantonese	cang4
U+406C.0	fMandarin	ceng2
U+406C.0	fEnglish	dimness of sight
U+406D.0	fUTF8	䁭
U+406D.0	fCNS11643	4-5731
U+406D.0	fGB	3-5A4A
U+406D.0	fR/S	 109.12
U+406D.0	fTotalStrokes	 17; 16
U+406D.0	fCangjie	BUIKK
U+406D.0	fFourCorner	63030
U+406D.0	fCantonese	biu1
U+406D.0	fMandarin	biao1
U+406D.0	fEnglish	[1] give an angry look [2] watch; view
U+406E.0	fUTF8	䁮
U+406E.0	fCNS11643	4-572F
U+406E.0	fGB	3-5A4C
U+406E.0	fR/S	 109.12
U+406E.0	fTotalStrokes	 17; 16
U+406E.0	fCangjie	BUMUA
U+406E.0	fFourCorner	61060
U+406E.0	fCantonese	cim4
U+406E.0	fMandarin	qian2
U+406E.0	fEnglish	[1] sad; mournful; grieved [2] worry about [3] think with one's eyes closed [4] close the eyes
U+406F.0	fUTF8	䁯
U+406F.0	fCNS11643	3-555A
U+406F.0	fR/S	 109.12
U+406F.0	fTotalStrokes	 17
U+406F.0	fCangjie	BUORM
U+406F.0	fFourCorner	68020
U+406F.0	fHakka	 him2
U+406F.0	fCantonese	hap1; kaap1
U+406F.0	fMandarin	xi1; she4; xie2
U+406F.0	fEnglish	[1] look at; see; inspect [2] same as &#x77B8; [3] same as &#x404B;
U+4070.0	fUTF8	䁰
U+4070.0	fCNS11643	4-5730
U+4070.0	fGB	3-5A2D
U+4070.0	fR/S	 109.12
U+4070.0	fTotalStrokes	 17; 16
U+4070.0	fCangjie	BUMMC
U+4070.0	fFourCorner	61080
U+4070.0	fCantonese	hong2; gong2
U+4070.0	fMandarin	xiang3; jiang3
U+4070.0	fEnglish	 look in a evil or wicked way; wicked vision
U+4071.0	fUTF8	䁱
U+4071.0	fCNS11643	4-5733
U+4071.0	fGB	5-5930
U+4071.0	fR/S	 109.12
U+4071.0	fTotalStrokes	 17; 16
U+4071.0	fCangjie	BUGGU
U+4071.0	fFourCorner	64010
U+4071.0	fHakka	 eu1; eu2
U+4071.0	fCantonese	kau1; aau1
U+4071.0	fMandarin	ou1; ao4
U+4071.0	fEnglish	deep sunken eye
U+4072.0	fUTF8	䁲
U+4072.0	fCNS11643	4-5734
U+4072.0	fGB	3-5964
U+4072.0	fR/S	 109.12
U+4072.0	fTotalStrokes	 17; 16
U+4072.0	fCangjie	BUWLC
U+4072.0	fFourCorner	66880
U+4072.0	fCantonese	maai4
U+4072.0	fMandarin	mai2
U+4072.0	fEnglish	 peep; look at something stealthily
U+4073.0	fUTF8	䁳
U+4073.0	fCNS11643	5-5622
U+4073.0	fGB	3-5A40
U+4073.0	fR/S	 109.12; 109.11
U+4073.0	fTotalStrokes	 17; 16
U+4073.0	fCangjie	BUTIT
U+4073.0	fFourCorner	64040
U+4073.0	fCantonese	mong5
U+4073.0	fMandarin	mang3
U+4073.0	fEnglish	 lose one eye
U+4074.0	fUTF8	䁴
U+4074.0	fCNS11643	5-6269
U+4074.0	fGB	3-5A53
U+4074.0	fR/S	 109.13
U+4074.0	fTotalStrokes	 18
U+4074.0	fCangjie	BUYWM
U+4074.0	fFourCorner	60010
U+4074.0	fCantonese	zin2
U+4074.0	fMandarin	zhan3
U+4074.0	fEnglish	[1] keep on looking [2] change the facial expression
U+4075.0	fUTF8	䁵
U+4075.0	fCNS11643	4-5C27
U+4075.0	fGB	3-5A51
U+4075.0	fR/S	 109.13
U+4075.0	fTotalStrokes	 18
U+4075.0	fCangjie	BUWLV
U+4075.0	fFourCorner	66030
U+4075.0	fCantonese	bin2; pin2
U+4075.0	fMandarin	bian3
U+4075.0	fEnglish	[1] newborn baby with the eyelids closed [2] big eyes [3] close the eyes
U+4076.0	fUTF8	䁶
U+4076.0	fCNS11643	4-5C29
U+4076.0	fGB	3-5A52
U+4076.0	fR/S	 109.13
U+4076.0	fTotalStrokes	 18
U+4076.0	fCangjie	BUHSK
U+4076.0	fFourCorner	68040
U+4076.0	fCantonese	gik1
U+4076.0	fMandarin	ji1; jiao3
U+4076.0	fEnglish	[1] look at something without winking [2] bright
U+4077.0	fUTF8	䁷
U+4077.0	fCNS11643	4-5C2A
U+4077.0	fGB	3-594B
U+4077.0	fR/S	 109.13
U+4077.0	fTotalStrokes	 18
U+4077.0	fCangjie	HBBU
U+4077.0	fFourCorner	77600
U+4077.0	fCantonese	gok3; uk1; zuk1
U+4077.0	fMandarin	jue2; wo4
U+4077.0	fEnglish	[1] bright eyes [2] angry look; angry eyes [3] ancient form of &#x89BA;
U+4078.0	fUTF8	䁸
U+4078.0	fCNS11643	4-5C25
U+4078.0	fGB	3-5963
U+4078.0	fR/S	 109.13
U+4078.0	fTotalStrokes	 18
U+4078.0	fCangjie	BUTWV
U+4078.0	fFourCorner	65030
U+4078.0	fCantonese	nong4; long4 (also); nung4
U+4078.0	fMandarin	nong2
U+4078.0	fEnglish	[1] eyesight obscured [2] angry glances
U+4079.0	fUTF8	䁹
U+4079.0	fCNS11643	3-5837
U+4079.0	fR/S	 109.13
U+4079.0	fTotalStrokes	 18
U+4079.0	fCangjie	BUSRJ
U+4079.0	fFourCorner	60040
U+4079.0	fCantonese	pai5; paai6; pei5
U+4079.0	fMandarin	bi4
U+4079.0	fEnglish	same as &#x7764;, &#x7765;
U+407A.0	fUTF8	䁺
U+407A.0	fCNS11643	4-5C26
U+407A.0	fGB	3-594E
U+407A.0	fR/S	 109.13
U+407A.0	fTotalStrokes	 18
U+407A.0	fCangjie	BUWLJ
U+407A.0	fFourCorner	66040
U+407A.0	fCantonese	sik1
U+407A.0	fMandarin	shi4
U+407A.0	fEnglish	[1] look at; see; inspect [2] sharp [3] bright-colored; bright eyes [4] same as non-classical form of &#x776A;
U+407B.0	fUTF8	䁻
U+407B.0	fCNS11643	4-6370
U+407B.0	fGB	3-594A
U+407B.0	fR/S	 109.15
U+407B.0	fTotalStrokes	 20
U+407B.0	fCangjie	BUVID
U+407B.0	fFourCorner	62090
U+407B.0	fCantonese	soek3
U+407B.0	fMandarin	shuo4; li4
U+407B.0	fEnglish	[1] pretty eyes [2] have a casual and short glance
U+407C.0	fUTF8	䁼
U+407C.0	fCNS11643	4-636E
U+407C.0	fGB	3-5A58
U+407C.0	fR/S	 109.15
U+407C.0	fTotalStrokes	 20
U+407C.0	fCangjie	BUWFG
U+407C.0	fFourCorner	66010
U+407C.0	fCantonese	mak6
U+407C.0	fMandarin	mo4; mu4
U+407C.0	fEnglish	[1] (literally) [2] look but see nothing -- absent-minded [3] regard as nothing
U+407D.0	fUTF8	䁽
U+407D.0	fCNS11643	4-6371
U+407D.0	fGB	3-5A59
U+407D.0	fR/S	 109.15
U+407D.0	fTotalStrokes	 20
U+407D.0	fCangjie	BUVVV
U+407D.0	fFourCorner	62010
U+407D.0	fCantonese	laap3; lip6
U+407D.0	fMandarin	lie4
U+407D.0	fEnglish	[1] eyesight obscured [2] abnormal vision
U+407E.0	fUTF8	䁾
U+407E.0	fCNS11643	4-636F
U+407E.0	fGB	3-5A54
U+407E.0	fR/S	 109.15
U+407E.0	fTotalStrokes	 20
U+407E.0	fCangjie	BUTWI
U+407E.0	fFourCorner	64050
U+407E.0	fCantonese	mit6; maat6
U+407E.0	fMandarin	mie4
U+407E.0	fEnglish	[1] red and swelling of the eye socket [2] dim-sighted; poor visioned
U+407F.0	fUTF8	䁿
U+407F.0	fCNS11643	4-665D
U+407F.0	fGB	3-5A5B
U+407F.0	fR/S	 109.16
U+407F.0	fTotalStrokes	 21
U+407F.0	fCangjie	WKBU
U+407F.0	fFourCorner	63600
U+407F.0	fCantonese	mak6
U+407F.0	fMandarin	mo4
U+407F.0	fEnglish	[1] sleepy; drowsy [2] startle; surprise; amaze
U+4080.0	fUTF8	䂀
U+4080.0	fCNS11643	3-5E41
U+4080.0	fR/S	 109.16
U+4080.0	fTotalStrokes	 21
U+4080.0	fCangjie	BUTGS
U+4080.0	fFourCorner	68050
U+4080.0	fCantonese	hei1
U+4080.0	fMandarin	xi1
U+4080.0	fEnglish	 wink
U+4081.0	fUTF8	䂁
U+4081.0	fCNS11643	4-6878
U+4081.0	fGB	3-5A5E
U+4081.0	fR/S	 109.17
U+4081.0	fTotalStrokes	 22; 21
U+4081.0	fCangjie	BUNRI
U+4081.0	fFourCorner	67010
U+4081.0	fCantonese	caam4; cam1; cam4; sam1
U+4081.0	fMandarin	chan2
U+4081.0	fEnglish	[1] angry look; look at someone angrily [2] hollow-eyed
U+4082.0	fUTF8	䂂
U+4082.0	fCNS11643	4-6A57
U+4082.0	fGB	3-5A5F
U+4082.0	fR/S	 109.18
U+4082.0	fTotalStrokes	 23
U+4082.0	fCangjie	BUBUG
U+4082.0	fFourCorner	66010
U+4082.0	fCantonese	keoi4
U+4082.0	fMandarin	qu2; ju4
U+4082.0	fEnglish	[1] gaze at in terror [2] a family name
U+4083.0	fUTF8	䂃
U+4083.0	fCNS11643	4-687A
U+4083.0	fGB	3-5A5D
U+4083.0	fR/S	 109.17; 109.18
U+4083.0	fTotalStrokes	 21; 23; 22
U+4083.0	fCangjie	BUBWI
U+4083.0	fFourCorner	62040
U+4083.0	fHakka	 ziau5;  ziok7
U+4083.0	fCantonese	ziu3; tiu4
U+4083.0	fMandarin	jiao4; jue2; jie2; jing1
U+4083.0	fEnglish	 close the eyes in sleep
U+4084.0	fUTF8	䂄
U+4084.0	fCNS11643	4-6C75
U+4084.0	fGB	5-597B
U+4084.0	fR/S	 109.20
U+4084.0	fTotalStrokes	 25
U+4084.0	fCangjie	BUBUE
U+4084.0	fFourCorner	66040
U+4084.0	fCantonese	fok3; fong3
U+4084.0	fMandarin	huo4; kuang4
U+4084.0	fEnglish	 startle; surprise; amaze; look at in surprise
U+4085.0	fUTF8	䂅
U+4085.0	fCNS11643	4-6E4D
U+4085.0	fR/S	 109.25
U+4085.0	fTotalStrokes	 30
U+4085.0	fCangjie	BUVFI
U+4085.0	fFourCorner	62030
U+4085.0	fCantonese	haan1; zin1
U+4085.0	fMandarin	shan1
U+4085.0	fEnglish	[1] have insight or vision of a simple and honest person [2] silly
U+4086.0	fUTF8	䂆
U+4086.0	fCNS11643	4-2751
U+4086.0	fGB	3-6026
U+4086.0	fR/S	 110.3
U+4086.0	fTotalStrokes	 8
U+4086.0	fCangjie	NHPI
U+4086.0	fFourCorner	17220
U+4086.0	fCantonese	gwik1; hek3
U+4086.0	fMandarin	qi4; xu4
U+4086.0	fEnglish	weapons; arms (lance, spear)
U+4087.0	fUTF8	䂇
U+4087.0	fCNS11643	4-2A5F
U+4087.0	fGB	5-5F21
U+4087.0	fR/S	 110.4
U+4087.0	fTotalStrokes	 9
U+4087.0	fCangjie	NHNG
U+4087.0	fFourCorner	17210
U+4087.0	fCantonese	nuk6; luk6 (also); naau2
U+4087.0	fMandarin	niu3; nang2; nuu4; nong3; pang2
U+4087.0	fEnglish	[1] pierce; stab; irritate; hurt [2] thorn
U+4088.0	fUTF8	䂈
U+4088.0	fCNS11643	4-3357
U+4088.0	fGB	3-6027
U+4088.0	fR/S	 110.6; 110.5; 142.5
U+4088.0	fTotalStrokes	 11
U+4088.0	fCangjie	NHLMI
U+4088.0	fFourCorner	15230
U+4088.0	fCantonese	tung4
U+4088.0	fMandarin	cong2; tong2
U+4088.0	fEnglish	[1] pierce; stab; irritate; hurt [2] thorn [3] lance; spear
U+4089.0	fUTF8	䂉
U+4089.0	fCNS11643	4-454A
U+4089.0	fGB	3-602A
U+4089.0	fR/S	 110.9
U+4089.0	fTotalStrokes	 14
U+4089.0	fCangjie	NHONK
U+4089.0	fFourCorner	17230
U+4089.0	fCantonese	hau4; haai6; haau4
U+4089.0	fMandarin	hou2
U+4089.0	fEnglish	weapons; arms (lance, spear)
U+408A.0	fUTF8	䂊
U+408A.0	fCNS11643	3-555D
U+408A.0	fR/S	 110.12
U+408A.0	fTotalStrokes	 17
U+408A.0	fCangjie	NHNAO
U+408A.0	fFourCorner	17230
U+408A.0	fCantonese	jyu6; zyu6
U+408A.0	fMandarin	yu4
U+408A.0	fEnglish	weapons; arms (lance, spear)
U+408B.0	fUTF8	䂋
U+408B.0	fGB	S-243E
U+408B.0	fR/S	 110.12
U+408B.0	fTotalStrokes	 17
U+408B.0	fCangjie	JQNHD
U+408B.0	fCantonese	jau5
U+408C.0	fUTF8	䂌
U+408C.0	fCNS11643	4-5738
U+408C.0	fGB	3-602B
U+408C.0	fR/S	 110.12
U+408C.0	fTotalStrokes	 17; 16
U+408C.0	fCangjie	NHYTG
U+408C.0	fFourCorner	10210
U+408C.0	fCantonese	cung1
U+408C.0	fMandarin	chong1
U+408C.0	fEnglish	a short weapon (daggers; swords; knives)
U+408D.0	fUTF8	䂍
U+408D.0	fCNS11643	4-6374
U+408D.0	fGB	3-602C
U+408D.0	fR/S	 110.15
U+408D.0	fTotalStrokes	 20
U+408D.0	fCangjie	NHATE
U+408D.0	fFourCorner	16230
U+408D.0	fCantonese	bok6
U+408D.0	fMandarin	bao2; bo2
U+408D.0	fEnglish	a kind of weapon carried by the honor guard before the Emperor in ancient times
U+408E.0	fUTF8	䂎
U+408E.0	fCNS11643	4-6B75
U+408E.0	fGB	3-602D
U+408E.0	fR/S	 110.19
U+408E.0	fTotalStrokes	 24
U+408E.0	fCangjie	NHHUC
U+408E.0	fFourCorner	14280
U+408E.0	fCantonese	zyun2
U+408E.0	fMandarin	zuan3; zuan4; cuan4; cuan1
U+408E.0	fEnglish	[1] a short spear; a lance with two points [2] halberd [3] pierce; stab; irritate [4] catch a spear from away
U+408F.0	fUTF8	䂏
U+408F.0	fCNS11643	4-2E57
U+408F.0	fGB	3-5B4A
U+408F.0	fR/S	 111.5
U+408F.0	fTotalStrokes	 10
U+408F.0	fCangjie	OKSHR
U+408F.0	fFourCorner	87460
U+408F.0	fCantonese	diu1
U+408F.0	fMandarin	diao1
U+408F.0	fEnglish	[1] short [2] dog with short tail
U+4090.0	fUTF8	䂐
U+4090.0	fCNS11643	4-2E58
U+4090.0	fGB	3-5B49
U+4090.0	fR/S	 111.5
U+4090.0	fTotalStrokes	 10
U+4090.0	fCangjie	OKUU
U+4090.0	fFourCorner	82470
U+4090.0	fCantonese	dyut3; zyut3; zyut6
U+4090.0	fMandarin	zhuo2; chu4; chuo4
U+4090.0	fEnglish	short ancient form of &#x62D9;
U+4091.0	fUTF8	䂑
U+4091.0	fCNS11643	4-335A
U+4091.0	fGB	3-5B4C
U+4091.0	fR/S	 111.5; 111.6
U+4091.0	fTotalStrokes	 11; 10
U+4091.0	fCangjie	OKYMP
U+4091.0	fFourCorner	81410
U+4091.0	fCantonese	zai1
U+4091.0	fMandarin	zi1; ji1
U+4091.0	fEnglish	short
U+4092.0	fUTF8	䂒
U+4092.0	fCNS11643	4-3359
U+4092.0	fGB	3-5B4B
U+4092.0	fR/S	 111.6
U+4092.0	fTotalStrokes	 11
U+4092.0	fCangjie	OKGR
U+4092.0	fFourCorner	84460
U+4092.0	fCantonese	kit3; haap1
U+4092.0	fMandarin	jie2; qia4; ya4
U+4092.0	fEnglish	[1] short [2] curry favor; to toady; to flatter; to try hard to please
U+4093.0	fUTF8	䂓
U+4093.0	fCNS11643	6-4C2B
U+4093.0	fR/S	 111.7
U+4093.0	fTotalStrokes	 12
U+4093.0	fCangjie	OKBUU
U+4093.0	fFourCorner	86410
U+4093.0	fHakka	 gwi1
U+4093.0	fCantonese	kwai1
U+4093.0	fMandarin	gui1
U+4093.0	fEnglish	standard form of &#x898F;
U+4094.0	fUTF8	䂔
U+4094.0	fCNS11643	4-3F33
U+4094.0	fGB	3-5B4D
U+4094.0	fR/S	 111.8
U+4094.0	fTotalStrokes	 13
U+4094.0	fCangjie	OKGTJ
U+4094.0	fFourCorner	84440
U+4094.0	fCantonese	jing6; haang6
U+4094.0	fMandarin	xing4
U+4094.0	fEnglish	short
U+4095.0	fUTF8	䂕
U+4095.0	fCNS11643	4-454C
U+4095.0	fGB	3-5B4E
U+4095.0	fR/S	 111.9
U+4095.0	fTotalStrokes	 14; 15
U+4095.0	fCangjie	OKVNO
U+4095.0	fFourCorner	87430
U+4095.0	fCantonese	wai3; fui3; ngaat6
U+4095.0	fMandarin	hui4; wei4
U+4095.0	fEnglish	short
U+4096.0	fUTF8	䂖
U+4096.0	fCNS11643	3-2432
U+4096.0	fR/S	 112.1
U+4096.0	fTotalStrokes	 6
U+4096.0	fCangjie	MIR
U+4096.0	fFourCorner	10600
U+4096.0	fCantonese	sek6; zi2
U+4096.0	fMandarin	shi2; si4
U+4096.0	fEnglish	same as &#x77F3;
U+4097.0	fUTF8	䂗
U+4097.0	fCNS11643	3-267A
U+4097.0	fR/S	 112.2
U+4097.0	fTotalStrokes	 7
U+4097.0	fCangjie	MRP
U+4097.0	fFourCorner	11610
U+4097.0	fCantonese	fat1; gwaat6
U+4097.0	fMandarin	ku1
U+4097.0	fEnglish	 be careful; exercise caution; take care; pay attention
U+4098.0	fUTF8	䂘
U+4098.0	fCNS11643	4-2753
U+4098.0	fGB	3-574C
U+4098.0	fR/S	 112.3
U+4098.0	fTotalStrokes	 8
U+4098.0	fCangjie	MREI
U+4098.0	fFourCorner	17640
U+4098.0	fCantonese	caai1; caam1; cai1; cak3; haam1; saam1
U+4098.0	fMandarin	chai1
U+4098.0	fEnglish	pebble; small piece of stone
U+4099.0	fUTF8	䂙
U+4099.0	fCNS11643	F-287A
U+4099.0	fGB	7-2361
U+4099.0	fR/S	 112.3
U+4099.0	fTotalStrokes	 8
U+4099.0	fCangjie	MRNL
U+4099.0	fCantonese	deoi1
U+409A.0	fUTF8	䂚
U+409A.0	fCNS11643	4-2A64
U+409A.0	fGB	3-5754
U+409A.0	fR/S	 112.4
U+409A.0	fTotalStrokes	 9
U+409A.0	fCangjie	MRKK
U+409A.0	fFourCorner	14640
U+409A.0	fCantonese	haau4; ngau4
U+409A.0	fMandarin	yao2; you2
U+409A.0	fEnglish	[1] a kind of stone [2] rugged rocks [3] difficult; hard; difficulty; hardship
U+409B.0	fUTF8	䂛
U+409B.0	fCNS11643	3-2E38
U+409B.0	fGB	5-577C
U+409B.0	fR/S	 112.4
U+409B.0	fTotalStrokes	 9
U+409B.0	fCangjie	MRNIN
U+409B.0	fFourCorner	17620
U+409B.0	fCantonese	jyu4; zyu4
U+409B.0	fMandarin	yu2
U+409B.0	fEnglish	a kind of rock
U+409C.0	fUTF8	䂜
U+409C.0	fCNS11643	4-2A62
U+409C.0	fGB	3-574D
U+409C.0	fR/S	 112.4
U+409C.0	fTotalStrokes	 9
U+409C.0	fCangjie	MRQJ
U+409C.0	fFourCorner	15600
U+409C.0	fCantonese	pong5; baang1; pang5
U+409C.0	fMandarin	bang4; pei2
U+409C.0	fEnglish	[1] rocky [2] same as &#x73A4; [3] interchangeable with &#x5D29;
U+409D.0	fUTF8	䂝
U+409D.0	fCNS11643	3-2E3C
U+409D.0	fR/S	 112.4
U+409D.0	fTotalStrokes	 9
U+409D.0	fCangjie	MRI
U+409D.0	fFourCorner	13650
U+409D.0	fCantonese	zaak3; zak6
U+409D.0	fMandarin	tuo1; jie2; ze2; zhe2
U+409D.0	fEnglish	[1] same as &#x77FA; [2] same as &#x78D4;
U+409E.0	fUTF8	䂞
U+409E.0	fCNS11643	3-2D3B
U+409E.0	fR/S	 112.4
U+409E.0	fTotalStrokes	 9
U+409E.0	fCangjie	MRD
U+409E.0	fFourCorner	10900
U+409E.0	fHakka	 zha5
U+409E.0	fCantonese	ze3
U+409E.0	fMandarin	zhe4
U+409E.0	fEnglish	same as &#x67D8;
U+409F.0	fUTF8	䂟
U+409F.0	fCNS11643	4-2E5F
U+409F.0	fGB	3-5769
U+409F.0	fR/S	 112.5
U+409F.0	fTotalStrokes	 10
U+409F.0	fCangjie	KRMR
U+409F.0	fFourCorner	46600
U+409F.0	fCantonese	gaa1
U+409F.0	fMandarin	jia1
U+409F.0	fEnglish	rocks; stones; minerals, etc.
U+40A0.0	fUTF8	䂠
U+40A0.0	fCNS11643	4-2E5C
U+40A0.0	fGB	3-5761
U+40A0.0	fR/S	 112.5
U+40A0.0	fTotalStrokes	 10
U+40A0.0	fCangjie	MROK
U+40A0.0	fFourCorner	18630
U+40A0.0	fCantonese	si2; ci2
U+40A0.0	fMandarin	shi3; she2
U+40A0.0	fEnglish	[1] the sound of falling rocks [2] the arrowhead made of stone
U+40A1.0	fUTF8	䂡
U+40A1.0	fCNS11643	4-2E62
U+40A1.0	fR/S	 112.5
U+40A1.0	fTotalStrokes	 10
U+40A1.0	fCangjie	MROM
U+40A1.0	fFourCorner	12610
U+40A1.0	fCantonese	dai2
U+40A1.0	fMandarin	di3; zhi3
U+40A1.0	fEnglish	non-classical form of &#x7825;
U+40A2.0	fUTF8	䂢
U+40A2.0	fCNS11643	5-2F34
U+40A2.0	fR/S	 112.5
U+40A2.0	fTotalStrokes	 10
U+40A2.0	fCangjie	MRHEY
U+40A2.0	fFourCorner	17630
U+40A2.0	fCantonese	dung2
U+40A2.0	fMandarin	dong3
U+40A2.0	fEnglish	[1] the sound of falling rocks [2] fallen rocks
U+40A3.0	fUTF8	䂣
U+40A3.0	fCNS11643	3-375B
U+40A3.0	fR/S	 112.5; 112.6
U+40A3.0	fTotalStrokes	 11; 10
U+40A3.0	fCangjie	MRYMP
U+40A3.0	fFourCorner	11610
U+40A3.0	fHakka	 cih1; ze1; ci2
U+40A3.0	fCantonese	ci1
U+40A3.0	fMandarin	ci1; ci2
U+40A3.0	fEnglish	brimstone; block sulphur
U+40A4.0	fUTF8	䂤
U+40A4.0	fCNS11643	3-326A
U+40A4.0	fR/S	 112.5
U+40A4.0	fTotalStrokes	 10
U+40A4.0	fCangjie	MRODI
U+40A4.0	fFourCorner	14600
U+40A4.0	fCantonese	fu6
U+40A4.0	fMandarin	fu4; hai2
U+40A4.0	fEnglish	white stone
U+40A5.0	fUTF8	䂥
U+40A5.0	fCNS11643	3-326B
U+40A5.0	fR/S	 112.5
U+40A5.0	fTotalStrokes	 10
U+40A5.0	fCangjie	MRRVP
U+40A5.0	fFourCorner	17640
U+40A5.0	fHakka	 min2
U+40A5.0	fCantonese	man4
U+40A5.0	fMandarin	min2
U+40A5.0	fEnglish	same as &#x8CE8;
U+40A6.0	fUTF8	䂦
U+40A6.0	fCNS11643	4-2E5E
U+40A6.0	fGB	3-5762
U+40A6.0	fR/S	 112.5
U+40A6.0	fTotalStrokes	 10
U+40A6.0	fCangjie	MROHH
U+40A6.0	fFourCorner	18620
U+40A6.0	fCantonese	zan1
U+40A6.0	fMandarin	zhen3
U+40A6.0	fEnglish	[1] tired; exhausted [2] rugged stone [3] coarse whetstone [4] pile up rocks on the river-bank
U+40A7.0	fUTF8	䂧
U+40A7.0	fCNS11643	4-2E61
U+40A7.0	fGB	3-5765
U+40A7.0	fR/S	 112.5
U+40A7.0	fTotalStrokes	 10
U+40A7.0	fCangjie	MRNF
U+40A7.0	fFourCorner	17690
U+40A7.0	fCantonese	can2; caan2; zaan2
U+40A7.0	fMandarin	zhen3
U+40A7.0	fEnglish	rocks clustered pile up togethers
U+40A8.0	fUTF8	䂨
U+40A8.0	fGB	S-224F
U+40A8.0	fR/S	 112.5
U+40A8.0	fTotalStrokes	 10
U+40A8.0	fCangjie	MRHMY
U+40A8.0	fCantonese	cik1
U+40A9.0	fUTF8	䂩
U+40A9.0	fCNS11643	4-335E
U+40A9.0	fGB	3-5773
U+40A9.0	fR/S	 112.6
U+40A9.0	fTotalStrokes	 11
U+40A9.0	fCangjie	MRWK
U+40A9.0	fFourCorner	16600
U+40A9.0	fCantonese	jin6; gin2; zin6
U+40A9.0	fMandarin	yan4
U+40A9.0	fEnglish	[1] same as &#x786F; [2] same as &#x785C;
U+40AA.0	fUTF8	䂪
U+40AA.0	fCNS11643	4-3361
U+40AA.0	fR/S	 112.6
U+40AA.0	fTotalStrokes	 11
U+40AA.0	fCangjie	MRLMO
U+40AA.0	fFourCorner	12610
U+40AA.0	fCantonese	haau1
U+40AA.0	fMandarin	qiao3; diao4; tiao3
U+40AA.0	fEnglish	[1] same as &#x78FD; [2] interchangeable with &#x929A;
U+40AB.0	fUTF8	䂫
U+40AB.0	fCNS11643	3-375C
U+40AB.0	fGB	5-582B
U+40AB.0	fR/S	 112.6
U+40AB.0	fTotalStrokes	 11
U+40AB.0	fCangjie	MRHEQ
U+40AB.0	fFourCorner	17650
U+40AB.0	fHakka	 fung2
U+40AB.0	fCantonese	haang1; hang1; hung4
U+40AB.0	fMandarin	hong2
U+40AB.0	fEnglish	[1] the noise of stones rolling down [2] to ram the earth for foundations [3] same as &#x592F;
U+40AC.0	fUTF8	䂬
U+40AC.0	fCNS11643	4-335F
U+40AC.0	fGB	3-576F
U+40AC.0	fR/S	 112.6
U+40AC.0	fTotalStrokes	 11
U+40AC.0	fCangjie	MNMR
U+40AC.0	fFourCorner	17600
U+40AC.0	fCantonese	gung2
U+40AC.0	fMandarin	gong3
U+40AC.0	fEnglish	[1] big rocks on the riverside [2] interchangeable wiht &#x6831;
U+40AD.0	fUTF8	䂭
U+40AD.0	fCNS11643	3-375A
U+40AD.0	fR/S	 112.6
U+40AD.0	fTotalStrokes	 11
U+40AD.0	fCangjie	MRYCK
U+40AD.0	fFourCorner	10640
U+40AD.0	fCantonese	haau1; diu6; hau1
U+40AD.0	fMandarin	qiao1
U+40AD.0	fEnglish	the name of a place; today's Jizhou
U+40AE.0	fUTF8	䂮
U+40AE.0	fCNS11643	4-3360
U+40AE.0	fGB	3-576D
U+40AE.0	fR/S	 112.6
U+40AE.0	fTotalStrokes	 11
U+40AE.0	fCangjie	QHMR
U+40AE.0	fFourCorner	54600
U+40AE.0	fHakka	  liok8
U+40AE.0	fCantonese	loek6
U+40AE.0	fMandarin	luue4; le4
U+40AE.0	fEnglish	 grind; sharpen
U+40AF.0	fUTF8	䂯
U+40AF.0	fCNS11643	3-375E
U+40AF.0	fR/S	 112.6
U+40AF.0	fTotalStrokes	 11
U+40AF.0	fCangjie	MRIG
U+40AF.0	fFourCorner	10610
U+40AF.0	fCantonese	gwaai3
U+40AF.0	fMandarin	guai4; guan4
U+40AF.0	fEnglish	a fine stone resembling jade
U+40B0.0	fUTF8	䂰
U+40B0.0	fCNS11643	3-3D5B
U+40B0.0	fR/S	 112.7
U+40B0.0	fTotalStrokes	 12
U+40B0.0	fCangjie	MRHDN
U+40B0.0	fFourCorner	12600
U+40B0.0	fCantonese	laap6
U+40B0.0	fMandarin	la4
U+40B0.0	fEnglish	rocks; stones; minerals, etc.
U+40B1.0	fUTF8	䂱
U+40B1.0	fCNS11643	5-3A59
U+40B1.0	fGB	7-2362
U+40B1.0	fR/S	 112.7
U+40B1.0	fTotalStrokes	 12
U+40B1.0	fCangjie	MRCRU
U+40B1.0	fFourCorner	18610
U+40B1.0	fCantonese	jeoi6; zeoi6
U+40B1.0	fMandarin	rui4; cui4
U+40B1.0	fEnglish	 wear out; rubbed out; die out; wear away; frayed
U+40B2.0	fUTF8	䂲
U+40B2.0	fCNS11643	6-4C33
U+40B2.0	fR/S	 112.7
U+40B2.0	fTotalStrokes	 12
U+40B2.0	fCangjie	GSMR
U+40B2.0	fFourCorner	44600
U+40B2.0	fCantonese	faat3
U+40B2.0	fMandarin	fa2; fa3
U+40B2.0	fEnglish	same as &#x781D;
U+40B3.0	fUTF8	䂳
U+40B3.0	fCNS11643	4-3958
U+40B3.0	fGB	3-5822
U+40B3.0	fR/S	 112.7
U+40B3.0	fTotalStrokes	 12
U+40B3.0	fCangjie	MROOG
U+40B3.0	fFourCorner	18610
U+40B3.0	fHakka	 co3; co5
U+40B3.0	fCantonese	co2
U+40B3.0	fMandarin	cuo3
U+40B3.0	fEnglish	broken stones; rubble
U+40B4.0	fUTF8	䂴
U+40B4.0	fCNS11643	3-3D58
U+40B4.0	fR/S	 112.7
U+40B4.0	fTotalStrokes	 12
U+40B4.0	fCangjie	MRYMR
U+40B4.0	fFourCorner	10660
U+40B4.0	fCantonese	jin4; dan6; zin4
U+40B4.0	fMandarin	yan2
U+40B4.0	fEnglish	same as &#x7814;
U+40B5.0	fUTF8	䂵
U+40B5.0	fGB	7-2363
U+40B5.0	fR/S	 112.7
U+40B5.0	fTotalStrokes	 12
U+40B5.0	fCangjie	MRMBO
U+40B5.0	fCantonese	gung1
U+40B5.0	fEnglish	simplified form of &#x78BD;
U+40B6.0	fUTF8	䂶
U+40B6.0	fCNS11643	4-3953
U+40B6.0	fGB	3-577B
U+40B6.0	fR/S	 112.7
U+40B6.0	fTotalStrokes	 12
U+40B6.0	fCangjie	MRGIS
U+40B6.0	fFourCorner	14620
U+40B6.0	fCantonese	gip3
U+40B6.0	fMandarin	jie2
U+40B6.0	fEnglish	the Crustacea; sea-anemone
U+40B7.0	fUTF8	䂷
U+40B7.0	fCNS11643	4-3F40
U+40B7.0	fGB	3-5834
U+40B7.0	fR/S	 112.8
U+40B7.0	fTotalStrokes	 13
U+40B7.0	fCangjie	MRHJP
U+40B7.0	fFourCorner	12610
U+40B7.0	fCantonese	gwaai1
U+40B7.0	fMandarin	guai1
U+40B7.0	fEnglish	[1] broken; smashed [2] trivial [3] irregular [4] different; varied sizes (of rocks)
U+40B8.0	fUTF8	䂸
U+40B8.0	fCNS11643	4-3F39
U+40B8.0	fGB	3-582A
U+40B8.0	fR/S	 112.8
U+40B8.0	fTotalStrokes	 13
U+40B8.0	fCangjie	MRIRM
U+40B8.0	fFourCorner	13650
U+40B8.0	fCantonese	gwok3; gok3 (also)
U+40B8.0	fMandarin	guo2; xu4
U+40B8.0	fEnglish	[1] smash or to knock to pieces of the stone [2] broken
U+40B9.0	fUTF8	䂹
U+40B9.0	fCNS11643	4-3F3F
U+40B9.0	fGB	3-5837
U+40B9.0	fR/S	 112.8
U+40B9.0	fTotalStrokes	 13
U+40B9.0	fCangjie	MRBF
U+40B9.0	fFourCorner	17680
U+40B9.0	fCantonese	saak3; sok3
U+40B9.0	fMandarin	suo3
U+40B9.0	fEnglish	the sound of the falling pieces of rocks
U+40BA.0	fUTF8	䂺
U+40BA.0	fCNS11643	3-432F
U+40BA.0	fR/S	 112.8
U+40BA.0	fTotalStrokes	 13
U+40BA.0	fCangjie	MRWD
U+40BA.0	fFourCorner	16690
U+40BA.0	fCantonese	wo2
U+40BA.0	fMandarin	wo3; wan3
U+40BA.0	fEnglish	[1] rocks; stones [2] same as &#x9846;
U+40BB.0	fUTF8	䂻
U+40BB.0	fCNS11643	4-3F3C
U+40BB.0	fGB	3-5751
U+40BB.0	fR/S	 112.8
U+40BB.0	fTotalStrokes	 13
U+40BB.0	fCangjie	MRSMV
U+40BB.0	fFourCorner	11630
U+40BB.0	fHakka	 cen5
U+40BB.0	fCantonese	zaang6
U+40BB.0	fMandarin	zheng4
U+40BB.0	fEnglish	[1] burnish [2] stop up
U+40BC.0	fUTF8	䂼
U+40BC.0	fCNS11643	3-4332
U+40BC.0	fR/S	 112.8
U+40BC.0	fTotalStrokes	 13
U+40BC.0	fCangjie	MROIP
U+40BC.0	fFourCorner	18630
U+40BC.0	fCantonese	nit6; lit6 (also); nip6
U+40BC.0	fMandarin	nie4
U+40BC.0	fEnglish	(a kind of) rocks; stones
U+40BD.0	fUTF8	䂽
U+40BD.0	fCNS11643	4-3F3D
U+40BD.0	fR/S	 112.8
U+40BD.0	fTotalStrokes	 13
U+40BD.0	fCangjie	MRYAJ
U+40BD.0	fFourCorner	11640
U+40BD.0	fCantonese	diu6
U+40BD.0	fMandarin	diao4; yi4
U+40BD.0	fEnglish	(a kind of) rocks; stones
U+40BE.0	fUTF8	䂾
U+40BE.0	fCNS11643	3-432C
U+40BE.0	fR/S	 112.8
U+40BE.0	fTotalStrokes	 13
U+40BE.0	fCangjie	MRDOO
U+40BE.0	fFourCorner	14690
U+40BE.0	fCantonese	loi5; lei4; loi2
U+40BE.0	fMandarin	lai3
U+40BE.0	fEnglish	 polish; grind; rub
U+40BF.0	fUTF8	䂿
U+40BF.0	fCNS11643	4-3F3E
U+40BF.0	fGB	3-583C
U+40BF.0	fR/S	 112.8
U+40BF.0	fTotalStrokes	 13
U+40BF.0	fCangjie	MREA
U+40BF.0	fFourCorner	12660
U+40BF.0	fHakka	  tap8
U+40BF.0	fCantonese	daap6; taap3
U+40BF.0	fMandarin	ta4; tie4
U+40BF.0	fEnglish	tread on; trample; pound grain to remove the husk
U+40C0.0	fUTF8	䃀
U+40C0.0	fCNS11643	4-3F38
U+40C0.0	fGB	3-582D
U+40C0.0	fR/S	 112.8
U+40C0.0	fTotalStrokes	 13
U+40C0.0	fCangjie	MRJLV
U+40C0.0	fFourCorner	15640
U+40C0.0	fCantonese	ceoi3
U+40C0.0	fMandarin	cui4; xun4
U+40C0.0	fEnglish	[1] millstone; grindstone [2] rub; grind; polish; wear [3] sharpen (a knife) [4] train; harden; temper
U+40C1.0	fUTF8	䃁
U+40C1.0	fCNS11643	4-3F3A
U+40C1.0	fGB	3-5770
U+40C1.0	fR/S	 112.8
U+40C1.0	fTotalStrokes	 13
U+40C1.0	fCangjie	MRMLM
U+40C1.0	fFourCorner	11610
U+40C1.0	fHakka	 a1; a3
U+40C1.0	fCantonese	aa1; ngaa1 (also)
U+40C1.0	fMandarin	ya1
U+40C1.0	fEnglish	rough; uneven
U+40C2.0	fUTF8	䃂
U+40C2.0	fCNS11643	4-3F3B
U+40C2.0	fGB	3-5830
U+40C2.0	fR/S	 112.8
U+40C2.0	fTotalStrokes	 13
U+40C2.0	fCangjie	MRAPP
U+40C2.0	fFourCorner	16610
U+40C2.0	fCantonese	gwan2
U+40C2.0	fMandarin	gun3; gun4
U+40C2.0	fEnglish	[1] dull sound of the tolling bell [2] the sound of rolling [3] rolling stone [4] a farm tool made of stone (a stone roller for hulling grains, etc.)
U+40C3.0	fUTF8	䃃
U+40C3.0	fR/S	 112.8
U+40C3.0	fTotalStrokes	 13
U+40C3.0	fCangjie	MRBTV
U+40C3.0	fCantonese	mong5
U+40C3.0	fEnglish	[Korean character]
U+40C4.0	fUTF8	䃄
U+40C4.0	fGB	7-2364
U+40C4.0	fR/S	 112.8
U+40C4.0	fTotalStrokes	 13
U+40C4.0	fCangjie	MRYRF
U+40C4.0	fCantonese	ging1
U+40C5.0	fUTF8	䃅
U+40C5.0	fGB	7-2365
U+40C5.0	fR/S	 112.8
U+40C5.0	fTotalStrokes	 13
U+40C5.0	fCangjie	MRCWJ
U+40C5.0	fCantonese	dai1; daai1
U+40C5.0	fEnglish	simplified form of &#x78FE;
U+40C6.0	fUTF8	䃆
U+40C6.0	fR/S	 112.8
U+40C6.0	fTotalStrokes	 13
U+40C6.0	fCangjie	MRTMC
U+40C6.0	fCantonese	kei4
U+40C6.0	fEnglish	[Korean character]
U+40C7.0	fUTF8	䃇
U+40C7.0	fCNS11643	4-3F42
U+40C7.0	fGB	3-5835
U+40C7.0	fR/S	 112.8
U+40C7.0	fTotalStrokes	 13
U+40C7.0	fCangjie	MRHAB
U+40C7.0	fFourCorner	16620
U+40C7.0	fCantonese	min4
U+40C7.0	fMandarin	mian2
U+40C7.0	fEnglish	ink (usually red) for imprinting of seals
U+40C8.0	fUTF8	䃈
U+40C8.0	fCNS11643	4-454E
U+40C8.0	fGB	3-5841
U+40C8.0	fR/S	 112.9
U+40C8.0	fTotalStrokes	 14
U+40C8.0	fCangjie	MRPPA
U+40C8.0	fFourCorner	11660
U+40C8.0	fCantonese	gaai1
U+40C8.0	fMandarin	jie1
U+40C8.0	fEnglish	[1] mountain rocks [2] same as &#x744E;
U+40C9.0	fUTF8	䃉
U+40C9.0	fCNS11643	3-4843
U+40C9.0	fGB	5-584C
U+40C9.0	fR/S	 112.9
U+40C9.0	fTotalStrokes	 14
U+40C9.0	fCangjie	MRRPA
U+40C9.0	fFourCorner	17660
U+40C9.0	fHakka	 min2;  men2
U+40C9.0	fCantonese	man4
U+40C9.0	fMandarin	min2
U+40C9.0	fEnglish	same as &#x8CE8;
U+40CA.0	fUTF8	䃊
U+40CA.0	fCNS11643	4-4551
U+40CA.0	fGB	3-5828
U+40CA.0	fR/S	 112.9
U+40CA.0	fTotalStrokes	 14
U+40CA.0	fCangjie	MRTBM
U+40CA.0	fFourCorner	14610
U+40CA.0	fCantonese	zeoi2; zik1; zik6
U+40CA.0	fMandarin	ju3
U+40CA.0	fEnglish	 rub; grind; polish; wear
U+40CB.0	fUTF8	䃋
U+40CB.0	fCNS11643	3-4841
U+40CB.0	fGB	5-5848
U+40CB.0	fR/S	 112.9
U+40CB.0	fTotalStrokes	 14
U+40CB.0	fCangjie	MROMN
U+40CB.0	fFourCorner	18620
U+40CB.0	fCantonese	jyu4; zyu4
U+40CB.0	fMandarin	yu2
U+40CB.0	fEnglish	a fine stone resembling jade
U+40CC.0	fUTF8	䃌
U+40CC.0	fCNS11643	4-454D
U+40CC.0	fGB	3-583E
U+40CC.0	fR/S	 112.9
U+40CC.0	fTotalStrokes	 14
U+40CC.0	fCangjie	MRMWG
U+40CC.0	fFourCorner	11610
U+40CC.0	fCantonese	zan1
U+40CC.0	fMandarin	zhen1
U+40CC.0	fEnglish	[1] rocky mountain [2] used in the name of a place
U+40CD.0	fUTF8	䃍
U+40CD.0	fCNS11643	4-4555
U+40CD.0	fGB	5-584B
U+40CD.0	fR/S	 112.9
U+40CD.0	fTotalStrokes	 14
U+40CD.0	fCangjie	MRTPO
U+40CD.0	fFourCorner	18630
U+40CD.0	fHakka	 tui3; chui3; ch'ui6; zui3
U+40CD.0	fCantonese	zeoi6
U+40CD.0	fMandarin	dui4; zhao4; zhui4
U+40CD.0	fEnglish	ancient form of &#x589C;
U+40CE.0	fUTF8	䃎
U+40CE.0	fCNS11643	4-454F
U+40CE.0	fGB	3-5840
U+40CE.0	fR/S	 112.9
U+40CE.0	fTotalStrokes	 14
U+40CE.0	fCangjie	MRKNN
U+40CE.0	fFourCorner	14620
U+40CE.0	fCantonese	zaa3; zak3
U+40CE.0	fMandarin	zha3; ze2; zha4
U+40CE.0	fEnglish	[1] the name of a place (usually to be used in naming a place) [2] interchangeable with &#x781F;
U+40CF.0	fUTF8	䃏
U+40CF.0	fCNS11643	5-4776
U+40CF.0	fR/S	 112.9
U+40CF.0	fTotalStrokes	 14
U+40CF.0	fCangjie	MRAHM
U+40CF.0	fFourCorner	16610
U+40CF.0	fCantonese	sing1; saang2
U+40CF.0	fMandarin	xing1
U+40CF.0	fEnglish	a kind of rock
U+40D0.0	fUTF8	䃐
U+40D0.0	fR/S	 112.9
U+40D0.0	fTotalStrokes	 14
U+40D0.0	fCangjie	MRJCK
U+40D0.0	fCantonese	dat6
U+40D0.0	fEnglish	[Korean character]
U+40D1.0	fUTF8	䃑
U+40D1.0	fCNS11643	4-4B5C
U+40D1.0	fGB	5-5850
U+40D1.0	fR/S	 112.10
U+40D1.0	fTotalStrokes	 15
U+40D1.0	fCangjie	MRHYE
U+40D1.0	fFourCorner	17640
U+40D1.0	fCantonese	baan1
U+40D1.0	fMandarin	ban1; pan2
U+40D1.0	fEnglish	[1] same as standard form &#x6591; [2] same as &#x78D0;
U+40D2.0	fUTF8	䃒
U+40D2.0	fCNS11643	4-4B58
U+40D2.0	fGB	3-584F
U+40D2.0	fR/S	 112.10
U+40D2.0	fTotalStrokes	 15
U+40D2.0	fCangjie	MRMRB
U+40D2.0	fFourCorner	11620
U+40D2.0	fCantonese	hat6; gak3; haat6
U+40D2.0	fMandarin	he2
U+40D2.0	fEnglish	poor and barren (of land); field which is not arable
U+40D3.0	fUTF8	䃓
U+40D3.0	fCNS11643	4-4B56
U+40D3.0	fGB	3-584D
U+40D3.0	fR/S	 112.10
U+40D3.0	fTotalStrokes	 15
U+40D3.0	fCangjie	MRTTB
U+40D3.0	fFourCorner	15640
U+40D3.0	fCantonese	gau3; gaau3; kaau4; ngaau1
U+40D3.0	fMandarin	gou4
U+40D3.0	fEnglish	[1] punish; chastise; penalize [2] kill; execute [3] pedal (a footboard) of a cart [4] brick work of a well [5] strong and durable [6] solid; firm
U+40D4.0	fUTF8	䃔
U+40D4.0	fCNS11643	4-4B55
U+40D4.0	fGB	3-5849
U+40D4.0	fR/S	 112.10
U+40D4.0	fTotalStrokes	 15
U+40D4.0	fCangjie	MRJRR
U+40D4.0	fFourCorner	13660
U+40D4.0	fHakka	 fung2; giung1; fung1
U+40D4.0	fCantonese	hung4
U+40D4.0	fMandarin	hong2
U+40D4.0	fEnglish	noise made by a landslip
U+40D5.0	fUTF8	䃕
U+40D5.0	fCNS11643	4-4B5E
U+40D5.0	fGB	3-575C
U+40D5.0	fR/S	 112.10
U+40D5.0	fTotalStrokes	 15
U+40D5.0	fCangjie	FFBMR
U+40D5.0	fFourCorner	99600
U+40D5.0	fCantonese	lou4; lok6
U+40D5.0	fMandarin	lao2; luo4
U+40D5.0	fEnglish	[1] stoneware; stone implement [2] soap-stone [3] the sound of the bumping rocks [4] chord (in music)
U+40D6.0	fUTF8	䃖
U+40D6.0	fCNS11643	7-2624
U+40D6.0	fGB	5-5779
U+40D6.0	fR/S	 112.10
U+40D6.0	fTotalStrokes	 15
U+40D6.0	fCangjie	MRHRF
U+40D6.0	fFourCorner	17620
U+40D6.0	fCantonese	wu2
U+40D6.0	fMandarin	wu3; wu4
U+40D6.0	fEnglish	same as &#x9696;, &#x5862;
U+40D7.0	fUTF8	䃗
U+40D7.0	fCNS11643	4-4B5B
U+40D7.0	fGB	3-5858
U+40D7.0	fR/S	 112.10
U+40D7.0	fTotalStrokes	 15
U+40D7.0	fCangjie	MRVEN
U+40D7.0	fFourCorner	12600
U+40D7.0	fHakka	 bok7
U+40D7.0	fCantonese	bok1; cuk1; mok1
U+40D7.0	fMandarin	bo1; chuo4
U+40D7.0	fEnglish	[1] rocks; stones [2] rocks paved bank [3] big rocks
U+40D8.0	fUTF8	䃘
U+40D8.0	fCNS11643	3-5223
U+40D8.0	fGB	5-5832
U+40D8.0	fR/S	 112.11
U+40D8.0	fTotalStrokes	 16
U+40D8.0	fCangjie	MRSEG
U+40D8.0	fFourCorner	17610
U+40D8.0	fHakka	 gien1; gin1; kin1; gang1
U+40D8.0	fCantonese	hang1
U+40D8.0	fEnglish	the tinkling of gems
U+40D9.0	fUTF8	䃙
U+40D9.0	fCNS11643	3-517D
U+40D9.0	fGB	5-585A
U+40D9.0	fR/S	 112.11
U+40D9.0	fTotalStrokes	 16
U+40D9.0	fCangjie	MRIXP
U+40D9.0	fFourCorner	10610
U+40D9.0	fCantonese	luk6
U+40D9.0	fMandarin	lu4
U+40D9.0	fEnglish	rocks; stones; minerals, etc.
U+40DA.0	fUTF8	䃚
U+40DA.0	fCNS11643	4-516C
U+40DA.0	fGB	5-585B
U+40DA.0	fR/S	 112.11
U+40DA.0	fTotalStrokes	 16
U+40DA.0	fCangjie	MRYSK
U+40DA.0	fFourCorner	18634
U+40DA.0	fCantonese	zuk6; cuk1
U+40DA.0	fMandarin	cu4
U+40DA.0	fEnglish	[1] rugged land of sand and pebble; poor and barren (of land) [2] same as &#x93C3;
U+40DB.0	fUTF8	䃛
U+40DB.0	fCNS11643	3-517E
U+40DB.0	fGB	5-5831
U+40DB.0	fR/S	 112.11
U+40DB.0	fTotalStrokes	 16
U+40DB.0	fCangjie	MRYJJ
U+40DB.0	fFourCorner	15630
U+40DB.0	fCantonese	lin6; lin2
U+40DB.0	fMandarin	lian2; qian4
U+40DB.0	fEnglish	same as &#x93C8;
U+40DC.0	fUTF8	䃜
U+40DC.0	fCNS11643	7-3059
U+40DC.0	fR/S	 112.11
U+40DC.0	fTotalStrokes	 16
U+40DC.0	fCangjie	SEMR
U+40DC.0	fFourCorner	77600
U+40DC.0	fCantonese	ji1; zi1
U+40DC.0	fMandarin	yi1
U+40DC.0	fEnglish	a fine black stone
U+40DD.0	fUTF8	䃝
U+40DD.0	fCNS11643	5-5633
U+40DD.0	fR/S	 112.11
U+40DD.0	fTotalStrokes	 16
U+40DD.0	fCangjie	MRJDK
U+40DD.0	fFourCorner	18640
U+40DD.0	fCantonese	hat6; hiu3; kiu3
U+40DD.0	fMandarin	qiao3; qiao4
U+40DD.0	fEnglish	same as &#x7909;
U+40DE.0	fUTF8	䃞
U+40DE.0	fCNS11643	4-5170
U+40DE.0	fGB	3-585F
U+40DE.0	fR/S	 112.11
U+40DE.0	fTotalStrokes	 16
U+40DE.0	fCangjie	YIMR
U+40DE.0	fFourCorner	04600
U+40DE.0	fCantonese	suk6
U+40DE.0	fMandarin	shu2; yi4
U+40DE.0	fEnglish	[1] a kind of stone [2] the sound of pebbles or stones rubbing or knocking together
U+40DF.0	fUTF8	䃟
U+40DF.0	fGB	S-2251
U+40DF.0	fR/S	 112.12
U+40DF.0	fTotalStrokes	 17
U+40DF.0	fCangjie	MRTBK
U+40DF.0	fCantonese	saan3
U+40DF.0	fEnglish	 used in place name in Hong Kong
U+40E0.0	fUTF8	䃠
U+40E0.0	fCNS11643	4-516D
U+40E0.0	fGB	5-585C
U+40E0.0	fR/S	 112.11
U+40E0.0	fTotalStrokes	 16
U+40E0.0	fCangjie	MRYSO
U+40E0.0	fFourCorner	18621
U+40E0.0	fCantonese	syun4
U+40E1.0	fUTF8	䃡
U+40E1.0	fCNS11643	4-573B
U+40E1.0	fGB	3-5862
U+40E1.0	fR/S	 112.12
U+40E1.0	fTotalStrokes	 17; 16
U+40E1.0	fCangjie	MRMUA
U+40E1.0	fFourCorner	11660
U+40E1.0	fCantonese	zam1
U+40E1.0	fMandarin	jin1; cen2
U+40E1.0	fEnglish	[1] small pebble; small piece of stone or rock [2] stone door
U+40E2.0	fUTF8	䃢
U+40E2.0	fCNS11643	4-5741
U+40E2.0	fGB	3-5844
U+40E2.0	fR/S	 112.12
U+40E2.0	fTotalStrokes	 17; 16
U+40E2.0	fCangjie	MRCNO
U+40E2.0	fFourCorner	17680
U+40E2.0	fCantonese	jam1; zaam1
U+40E2.0	fMandarin	qin1
U+40E2.0	fEnglish	[1] the name of a mountain [2] same as &#x5D94;
U+40E3.0	fUTF8	䃣
U+40E3.0	fCNS11643	3-4842
U+40E3.0	fGB	5-577B
U+40E3.0	fR/S	 112.12
U+40E3.0	fTotalStrokes	 14; 17
U+40E3.0	fCangjie	MRIKF
U+40E3.0	fFourCorner	14620
U+40E3.0	fCantonese	wai2
U+40E3.0	fMandarin	hui3
U+40E3.0	fEnglish	same as &#x6BC0;
U+40E4.0	fUTF8	䃤
U+40E4.0	fCNS11643	4-5739
U+40E4.0	fGB	5-5844
U+40E4.0	fR/S	 112.12
U+40E4.0	fTotalStrokes	 17; 16
U+40E4.0	fCangjie	MRLX
U+40E4.0	fFourCorner	17620
U+40E4.0	fCantonese	suk1
U+40E4.0	fMandarin	su4
U+40E4.0	fEnglish	black colored grindstone or whestone
U+40E5.0	fUTF8	䃥
U+40E5.0	fCNS11643	3-555E
U+40E5.0	fR/S	 112.12
U+40E5.0	fTotalStrokes	 17
U+40E5.0	fCangjie	MRYTG
U+40E5.0	fFourCorner	10610
U+40E5.0	fCantonese	cong4
U+40E5.0	fMandarin	chuang2
U+40E5.0	fEnglish	stones; rocks
U+40E6.0	fUTF8	䃦
U+40E6.0	fCNS11643	3-555F
U+40E6.0	fR/S	 112.12
U+40E6.0	fTotalStrokes	 17
U+40E6.0	fCangjie	YKMR
U+40E6.0	fFourCorner	08600
U+40E6.0	fCantonese	deon1
U+40E6.0	fMandarin	dun1
U+40E6.0	fEnglish	a flat and great rock people squat or crouch on it
U+40E7.0	fUTF8	䃧
U+40E7.0	fCNS11643	4-573F
U+40E7.0	fGB	3-5860
U+40E7.0	fR/S	 112.12
U+40E7.0	fTotalStrokes	 16; 17
U+40E7.0	fCangjie	NMMR
U+40E7.0	fFourCorner	17600
U+40E7.0	fCantonese	lung4
U+40E7.0	fMandarin	long2
U+40E7.0	fEnglish	the sound of falling rocks
U+40E8.0	fUTF8	䃨
U+40E8.0	fR/S	 112.12
U+40E8.0	fTotalStrokes	 17
U+40E8.0	fCangjie	MRNRI
U+40E8.0	fCantonese	koeng4
U+40E8.0	fEnglish	[Korean character]
U+40E9.0	fUTF8	䃩
U+40E9.0	fCNS11643	3-5841
U+40E9.0	fR/S	 112.13
U+40E9.0	fTotalStrokes	 18
U+40E9.0	fCangjie	MRTWV
U+40E9.0	fFourCorner	15630
U+40E9.0	fCantonese	naau4; laau4 (also); bit3; nau4
U+40E9.0	fMandarin	nao2
U+40E9.0	fEnglish	same as &#x7E2D;
U+40EA.0	fUTF8	䃪
U+40EA.0	fCNS11643	4-5C2D
U+40EA.0	fR/S	 112.13
U+40EA.0	fTotalStrokes	 18
U+40EA.0	fCangjie	MRYWM
U+40EA.0	fFourCorner	10610
U+40EA.0	fCantonese	taan4
U+40EA.0	fMandarin	tan2
U+40EA.0	fEnglish	same as &#x58C7;
U+40EB.0	fUTF8	䃫
U+40EB.0	fCNS11643	3-583D
U+40EB.0	fR/S	 112.13
U+40EB.0	fTotalStrokes	 18
U+40EB.0	fCangjie	MRNCR
U+40EB.0	fFourCorner	17660
U+40EB.0	fHakka	 dam3
U+40EB.0	fCantonese	daam2
U+40EB.0	fMandarin	dan3
U+40EB.0	fEnglish	a mineral described as having a juice like gall
U+40EC.0	fUTF8	䃬
U+40EC.0	fCNS11643	4-5C31
U+40EC.0	fGB	3-5863
U+40EC.0	fR/S	 112.13
U+40EC.0	fTotalStrokes	 18
U+40EC.0	fCangjie	MRUHI
U+40EC.0	fFourCorner	12610
U+40EC.0	fCantonese	ngai5; ai5 (also); gwaai1; ngaai5
U+40EC.0	fMandarin	wei3; kui3
U+40EC.0	fEnglish	[1] rocky; plenty of stones and rocks [2] same as &#x78C8;
U+40ED.0	fUTF8	䃭
U+40ED.0	fCNS11643	4-5C2F
U+40ED.0	fGB	3-5869
U+40ED.0	fR/S	 112.13
U+40ED.0	fTotalStrokes	 18
U+40ED.0	fCangjie	MRIRP
U+40ED.0	fFourCorner	13650
U+40ED.0	fCantonese	gam2
U+40ED.0	fMandarin	gan3; gan1
U+40ED.0	fEnglish	[1] stone box; stone case [2] cover with a piece of stone [3] interchangeable with &#x64BC;
U+40EE.0	fUTF8	䃮
U+40EE.0	fCNS11643	4-5C2E
U+40EE.0	fGB	3-5771
U+40EE.0	fR/S	 112.13
U+40EE.0	fTotalStrokes	 18
U+40EE.0	fCangjie	MRYGQ
U+40EE.0	fFourCorner	14630
U+40EE.0	fCantonese	daat6
U+40EE.0	fMandarin	da2
U+40EE.0	fEnglish	[1] a stone build hydraulic measures (to store and to vent the water) in ancient times [2] rocks in the mountain stream
U+40EF.0	fUTF8	䃯
U+40EF.0	fR/S	 112.13
U+40EF.0	fTotalStrokes	 18
U+40EF.0	fCangjie	MRIOD
U+40EF.0	fCantonese	lik1
U+40EF.0	fMandarin	li4
U+40EF.0	fEnglish	abbreviated form of &#x792B;
U+40F0.0	fUTF8	䃰
U+40F0.0	fCNS11643	4-603F
U+40F0.0	fGB	3-586E
U+40F0.0	fR/S	 112.14
U+40F0.0	fTotalStrokes	 19
U+40F0.0	fCangjie	MRJBF
U+40F0.0	fFourCorner	13690
U+40F0.0	fCantonese	caat3
U+40F0.0	fMandarin	ca1
U+40F0.0	fEnglish	brick or stone steps
U+40F1.0	fUTF8	䃱
U+40F1.0	fCNS11643	7-4E5C
U+40F1.0	fR/S	 112.15
U+40F1.0	fTotalStrokes	 20
U+40F1.0	fCangjie	MRJIM
U+40F1.0	fFourCorner	13650
U+40F1.0	fHakka	 sien3
U+40F1.0	fCantonese	cim3; cim1
U+40F1.0	fMandarin	xian4
U+40F1.0	fEnglish	non-classical form of &#x40F8;
U+40F2.0	fUTF8	䃲
U+40F2.0	fCNS11643	3-5C63
U+40F2.0	fR/S	 112.15
U+40F2.0	fTotalStrokes	 20
U+40F2.0	fCangjie	MRHET
U+40F2.0	fFourCorner	17610
U+40F2.0	fCantonese	pun4; bun2; po4; tam4
U+40F2.0	fMandarin	pan2; pan4
U+40F2.0	fEnglish	same as &#x78D0;
U+40F3.0	fUTF8	䃳
U+40F3.0	fCNS11643	4-6376
U+40F3.0	fGB	3-5871
U+40F3.0	fR/S	 112.15
U+40F3.0	fTotalStrokes	 20
U+40F3.0	fCangjie	MRVVV
U+40F3.0	fFourCorner	12610
U+40F3.0	fCantonese	laap6
U+40F3.0	fMandarin	la4; lie4
U+40F3.0	fEnglish	[1] falling stone [2] mineral; ore (with the element of bronze; copper) [3] appearance of the connected mountains; mountain range; chain
U+40F4.0	fUTF8	䃴
U+40F4.0	fCNS11643	4-6375
U+40F4.0	fGB	5-5853
U+40F4.0	fR/S	 112.16; 112.15
U+40F4.0	fTotalStrokes	 20; 21
U+40F4.0	fCangjie	MRYRA
U+40F4.0	fFourCorner	14660
U+40F4.0	fCantonese	zyu1
U+40F4.0	fMandarin	zhu1
U+40F4.0	fEnglish	a whetstone used to grind precious stone (jade, a gem)
U+40F5.0	fUTF8	䃵
U+40F5.0	fCNS11643	4-665F
U+40F5.0	fGB	3-5848
U+40F5.0	fR/S	 112.16
U+40F5.0	fTotalStrokes	 21
U+40F5.0	fCangjie	MRYSV
U+40F5.0	fFourCorner	10630
U+40F5.0	fCantonese	niu5; liu5 (also)
U+40F5.0	fMandarin	niao3
U+40F5.0	fEnglish	[1] a kind of stone [2] bends; curves; turns and twists of the mountains
U+40F6.0	fUTF8	䃶
U+40F6.0	fCNS11643	4-6660
U+40F6.0	fGB	3-5872
U+40F6.0	fR/S	 112.16
U+40F6.0	fTotalStrokes	 21
U+40F6.0	fCangjie	MRYWV
U+40F6.0	fFourCorner	10630
U+40F6.0	fCantonese	waai4
U+40F6.0	fMandarin	huai2; gui1; guai4
U+40F6.0	fEnglish	[1] rugged and uneven of the rocks [2] small piece of stone; pebble [3] a fine stone resembling jade
U+40F7.0	fUTF8	䃷
U+40F7.0	fCNS11643	4-6661
U+40F7.0	fGB	3-5874
U+40F7.0	fR/S	 112.16
U+40F7.0	fTotalStrokes	 21
U+40F7.0	fCangjie	MRYNV
U+40F7.0	fFourCorner	10610
U+40F7.0	fCantonese	jing4; zing4
U+40F7.0	fMandarin	ying2
U+40F7.0	fEnglish	[1] a kind of stone [2] research and examine
U+40F8.0	fUTF8	䃸
U+40F8.0	fCNS11643	4-687C
U+40F8.0	fGB	3-5875
U+40F8.0	fR/S	 112.17
U+40F8.0	fTotalStrokes	 22; 21
U+40F8.0	fCangjie	MROIM
U+40F8.0	fFourCorner	13650
U+40F8.0	fHakka	 sien3
U+40F8.0	fCantonese	cim3; cim1; zaam1
U+40F8.0	fMandarin	xian4; can4
U+40F8.0	fEnglish	[1] electric light; flash of lightning [2] wedge [3] preface; foreword
U+40F9.0	fUTF8	䃹
U+40F9.0	fCNS11643	4-687B
U+40F9.0	fGB	3-5867
U+40F9.0	fR/S	 112.17
U+40F9.0	fTotalStrokes	 22; 21
U+40F9.0	fCangjie	MRANW
U+40F9.0	fFourCorner	17620
U+40F9.0	fCantonese	laan6
U+40F9.0	fMandarin	lan4; lang3
U+40F9.0	fEnglish	[1] jade and stone [2] fine jade
U+40FA.0	fUTF8	䃺
U+40FA.0	fCNS11643	3-6075
U+40FA.0	fGB	5-586B
U+40FA.0	fR/S	 112.19
U+40FA.0	fTotalStrokes	 24
U+40FA.0	fCangjie	MRIDY
U+40FA.0	fFourCorner	10610
U+40FA.0	fHakka	  mo2;  mo5
U+40FA.0	fCantonese	mo4; mo6
U+40FA.0	fMandarin	mo2
U+40FA.0	fEnglish	standard form of &#x78E8;
U+40FB.0	fUTF8	䃻
U+40FB.0	fCNS11643	3-6167
U+40FB.0	fR/S	 112.21
U+40FB.0	fTotalStrokes	 26
U+40FB.0	fCangjie	MRMBB
U+40FB.0	fFourCorner	11620
U+40FB.0	fHakka	 ba5
U+40FB.0	fCantonese	baa3
U+40FB.0	fMandarin	ba4; pai2
U+40FB.0	fEnglish	same as &#x58E9;
U+40FC.0	fUTF8	䃼
U+40FC.0	fR/S	 113.2
U+40FC.0	fTotalStrokes	 7
U+40FC.0	fCangjie	IFY
U+40FC.0	fCantonese	buk1
U+40FC.0	fEnglish	[Korean character]
U+40FD.0	fUTF8	䃽
U+40FD.0	fCNS11643	3-2E3F
U+40FD.0	fGB	5-574D
U+40FD.0	fR/S	 113.4
U+40FD.0	fTotalStrokes	 9; 8
U+40FD.0	fCangjie	IFJE
U+40FD.0	fFourCorner	34240
U+40FD.0	fHakka	 gui3; kwui3
U+40FD.0	fCantonese	gwai2
U+40FD.0	fMandarin	gui3; fu2; zhi1; si4
U+40FD.0	fEnglish	 sacrifice at the five mountains
U+40FE.0	fUTF8	䃾
U+40FE.0	fCNS11643	4-2A67
U+40FE.0	fGB	3-5721
U+40FE.0	fR/S	 113.4
U+40FE.0	fTotalStrokes	 9
U+40FE.0	fCangjie	IFPP
U+40FE.0	fFourCorner	31210
U+40FE.0	fCantonese	bei2
U+40FE.0	fMandarin	bi3; bi4
U+40FE.0	fEnglish	 offer small pig as sacrifice when warship god of life
U+40FF.0	fUTF8	䃿
U+40FF.0	fCNS11643	4-2A66
U+40FF.0	fGB	3-567D
U+40FF.0	fR/S	 113.4
U+40FF.0	fTotalStrokes	 9; 8
U+40FF.0	fCangjie	IFQO
U+40FF.0	fFourCorner	35230
U+40FF.0	fCantonese	fu1; haa1
U+40FF.0	fMandarin	fu1; zhi4
U+40FF.0	fEnglish	[1] rite; service [2] message offered to the gods in worship
U+4100.0	fUTF8	䄀
U+4100.0	fCNS11643	6-3348
U+4100.0	fR/S	 113.4
U+4100.0	fTotalStrokes	 9
U+4100.0	fCangjie	IFI
U+4100.0	fFourCorner	33250
U+4100.0	fCantonese	wo6
U+4100.0	fMandarin	huo4
U+4100.0	fEnglish	same as &#x798D;
U+4101.0	fUTF8	䄁
U+4101.0	fCNS11643	3-326E
U+4101.0	fR/S	 113.5
U+4101.0	fTotalStrokes	 10; 9
U+4101.0	fCangjie	IFPT
U+4101.0	fFourCorner	34210
U+4101.0	fCantonese	jai6; zi6
U+4101.0	fMandarin	yi4
U+4101.0	fEnglish	 worship; honor by a service or rite; offer sacrifices to
U+4102.0	fUTF8	䄂
U+4102.0	fCNS11643	4-2E64
U+4102.0	fR/S	 113.5
U+4102.0	fTotalStrokes	 10
U+4102.0	fCangjie	IFLW
U+4102.0	fFourCorner	35260
U+4102.0	fCantonese	lau6
U+4102.0	fMandarin	liu4
U+4102.0	fEnglish	(ancient form) pray and to curse (the ways of treating a patient in ancient times)
U+4103.0	fUTF8	䄃
U+4103.0	fCNS11643	3-3271
U+4103.0	fR/S	 113.5
U+4103.0	fTotalStrokes	 10; 9
U+4103.0	fCangjie	IFLBK
U+4103.0	fFourCorner	35230
U+4103.0	fHakka	 jong1; rong1
U+4103.0	fCantonese	joeng1; zoeng1
U+4103.0	fMandarin	yang1
U+4103.0	fEnglish	[1] calamity; misfortune [2] retribution
U+4104.0	fUTF8	䄄
U+4104.0	fCNS11643	3-3764
U+4104.0	fGB	5-5755
U+4104.0	fR/S	 113.6
U+4104.0	fTotalStrokes	 11; 10
U+4104.0	fCangjie	IFWK
U+4104.0	fFourCorner	36200
U+4104.0	fCantonese	jan1; zaan1
U+4104.0	fMandarin	yin1
U+4104.0	fEnglish	[1] achievement; accomplishment [2] non-classical form of &#x798B;
U+4105.0	fUTF8	䄅
U+4105.0	fCNS11643	4-3363
U+4105.0	fGB	5-574A
U+4105.0	fR/S	 113.6
U+4105.0	fTotalStrokes	 11
U+4105.0	fCangjie	FQMMF
U+4105.0	fFourCorner	90900
U+4105.0	fCantonese	gyun3; aa1
U+4105.0	fMandarin	juan4
U+4105.0	fEnglish	same as &#x798B;
U+4106.0	fUTF8	䄆
U+4106.0	fCNS11643	4-3362
U+4106.0	fGB	3-5728
U+4106.0	fR/S	 113.6
U+4106.0	fTotalStrokes	 11; 10
U+4106.0	fCangjie	IFHJR
U+4106.0	fFourCorner	32260
U+4106.0	fCantonese	wut6
U+4106.0	fMandarin	huo2; hua1; kuo4; gua1; huan4
U+4106.0	fEnglish	[1] worship; honor by a rite or service; offer sacrifices [2] institution [3] law [4] perform rites in honor of gods
U+4107.0	fUTF8	䄇
U+4107.0	fCNS11643	3-3D61
U+4107.0	fR/S	 113.7
U+4107.0	fTotalStrokes	 12; 11
U+4107.0	fCangjie	IFRHG
U+4107.0	fFourCorner	36210
U+4107.0	fCantonese	cing4; zing6
U+4107.0	fMandarin	cheng2
U+4107.0	fEnglish	(corrupted form) a family name
U+4108.0	fUTF8	䄈
U+4108.0	fCNS11643	3-3D60
U+4108.0	fR/S	 113.7
U+4108.0	fTotalStrokes	 12; 11
U+4108.0	fCangjie	IFMRT
U+4108.0	fFourCorner	31210
U+4108.0	fCantonese	dau6; coeng4; daau2; daau6; hiu1
U+4108.0	fMandarin	dou4
U+4108.0	fEnglish	sacrifice to happiness and good luck
U+4109.0	fUTF8	䄉
U+4109.0	fCNS11643	4-3959
U+4109.0	fGB	3-572C
U+4109.0	fR/S	 113.7
U+4109.0	fTotalStrokes	 12; 11
U+4109.0	fCangjie	IFHQI
U+4109.0	fFourCorner	33250
U+4109.0	fCantonese	ngo4
U+4109.0	fMandarin	e2
U+4109.0	fEnglish	[1] a rite or service [2] same as &#x5A25;
U+410A.0	fUTF8	䄊
U+410A.0	fGB	7-2367
U+410A.0	fR/S	 113.7
U+410A.0	fTotalStrokes	 12
U+410A.0	fCangjie	IFGP
U+410A.0	fFourCorner	3423
U+410A.0	fCantonese	zi3
U+410B.0	fUTF8	䄋
U+410B.0	fCNS11643	4-3F45
U+410B.0	fGB	3-5730
U+410B.0	fR/S	 113.8
U+410B.0	fTotalStrokes	 13; 12
U+410B.0	fCangjie	IFKLU
U+410B.0	fFourCorner	34210
U+410B.0	fCantonese	jim2; zim2; zim3
U+410B.0	fMandarin	yan3; yan4
U+410B.0	fEnglish	[1] pray for preventing or forestalling calamities [2] muddy
U+410C.0	fUTF8	䄌
U+410C.0	fCNS11643	4-3F44
U+410C.0	fGB	5-5761
U+410C.0	fR/S	 113.8
U+410C.0	fTotalStrokes	 13; 12
U+410C.0	fCangjie	IFEEE
U+410C.0	fFourCorner	37240
U+410C.0	fCantonese	zeoi3
U+410C.0	fMandarin	zhui4; chuo4
U+410C.0	fEnglish	[1] perform rities in honor of the ancestral temple; the next day's service of the above mentioned rities [2] same as &#x918A;
U+410D.0	fUTF8	䄍
U+410D.0	fCNS11643	3-433A
U+410D.0	fGB	5-575D
U+410D.0	fR/S	 113.8
U+410D.0	fTotalStrokes	 13
U+410D.0	fCangjie	IFTA
U+410D.0	fFourCorner	34260
U+410D.0	fHakka	 za5
U+410D.0	fCantonese	zaa3; caai4
U+410D.0	fMandarin	zha4; du4; duo2
U+410D.0	fEnglish	the imperial thanksgiving to earth at the end of the year
U+410E.0	fUTF8	䄎
U+410E.0	fCNS11643	3-433B
U+410E.0	fR/S	 113.8
U+410E.0	fTotalStrokes	 13; 12
U+410E.0	fCangjie	IFKMR
U+410E.0	fFourCorner	34220
U+410E.0	fCantonese	kei2; hei2
U+410E.0	fMandarin	qi3
U+410E.0	fEnglish	same as &#x5A4D;
U+410F.0	fUTF8	䄏
U+410F.0	fCNS11643	3-433E
U+410F.0	fGB	5-575A
U+410F.0	fR/S	 113.8
U+410F.0	fTotalStrokes	 13
U+410F.0	fCangjie	IFTHK
U+410F.0	fFourCorner	34230
U+410F.0	fCantonese	hin1; ziu1
U+410F.0	fMandarin	yao1; yu2
U+410F.0	fEnglish	[1] same as &#x7945; [2] interchangeable with &#x5996;
U+4110.0	fUTF8	䄐
U+4110.0	fCNS11643	4-3F47
U+4110.0	fGB	5-3E73
U+4110.0	fR/S	 113.8
U+4110.0	fTotalStrokes	 13
U+4110.0	fCangjie	UFRU
U+4110.0	fFourCorner	27210
U+4110.0	fCantonese	hyun3
U+4110.0	fMandarin	quan4
U+4110.0	fEnglish	[1] worship; honor by a service or rite; offer sacrifices [2] happiess; good fortune; good luck; blessing; bliss
U+4111.0	fUTF8	䄑
U+4111.0	fCNS11643	3-433C
U+4111.0	fR/S	 113.8
U+4111.0	fTotalStrokes	 13; 12
U+4111.0	fCangjie	IFHPA
U+4111.0	fFourCorner	32260
U+4111.0	fCantonese	wut6
U+4111.0	fMandarin	huo2; kuo4
U+4111.0	fEnglish	same as standard form &#x4106;
U+4112.0	fUTF8	䄒
U+4112.0	fCNS11643	3-433F
U+4112.0	fR/S	 113.8
U+4112.0	fTotalStrokes	 13
U+4112.0	fCangjie	IFOIP
U+4112.0	fFourCorner	38230
U+4112.0	fCantonese	nam5; lam5 (also); naam5
U+4112.0	fMandarin	nie4; ren3
U+4112.0	fEnglish	same as standard form &#x7A14;
U+4113.0	fUTF8	䄓
U+4113.0	fCNS11643	3-4846
U+4113.0	fR/S	 113.9
U+4113.0	fTotalStrokes	 14; 13
U+4113.0	fCangjie	IFHAG
U+4113.0	fFourCorner	36210
U+4113.0	fCantonese	waang4; wong4
U+4113.0	fMandarin	heng2; huang2
U+4113.0	fEnglish	[1] worship [2] rite; service
U+4114.0	fUTF8	䄔
U+4114.0	fCNS11643	5-4824
U+4114.0	fGB	7-2366
U+4114.0	fR/S	 113.9
U+4114.0	fTotalStrokes	 14; 13
U+4114.0	fCangjie	IFHLB
U+4114.0	fFourCorner	32220
U+4114.0	fHakka	 gi3; ji3; ri3
U+4114.0	fCantonese	geoi2
U+4114.0	fMandarin	ju3
U+4114.0	fEnglish	a family name
U+4115.0	fUTF8	䄕
U+4115.0	fCNS11643	3-4849
U+4115.0	fR/S	 113.9
U+4115.0	fTotalStrokes	 14
U+4115.0	fCangjie	IFEW
U+4115.0	fFourCorner	32260
U+4115.0	fCantonese	se5
U+4115.0	fMandarin	she4; shen4; tian3
U+4115.0	fEnglish	ancient form of &#x793E;
U+4116.0	fUTF8	䄖
U+4116.0	fR/S	 113.9
U+4116.0	fTotalStrokes	 14
U+4116.0	fCangjie	IFOMV
U+4116.0	fCantonese	jyu4
U+4116.0	fEnglish	used in name of a person
U+4117.0	fUTF8	䄗
U+4117.0	fR/S	 113.9
U+4117.0	fTotalStrokes	 14
U+4117.0	fCangjie	IFLXL
U+4117.0	fCantonese	jyun1
U+4117.0	fEnglish	[Korean character]
U+4118.0	fUTF8	䄘
U+4118.0	fCNS11643	5-4E76
U+4118.0	fR/S	 113.10
U+4118.0	fTotalStrokes	 15; 14
U+4118.0	fCangjie	IFYBS
U+4118.0	fFourCorner	11177
U+4118.0	fCantonese	paang4; lok6; pang4; ping1; teoi3; zaai3; zi4
U+4118.0	fMandarin	peng2
U+4118.0	fEnglish	 rite; service; worship
U+4119.0	fUTF8	䄙
U+4119.0	fCNS11643	3-4D4E
U+4119.0	fR/S	 113.10
U+4119.0	fTotalStrokes	 15; 14
U+4119.0	fCangjie	IFBAC
U+4119.0	fFourCorner	37280
U+4119.0	fCantonese	ming4
U+4119.0	fMandarin	ming2
U+4119.0	fEnglish	happiness; good luck; good fortune; blessing; bliss
U+411A.0	fUTF8	䄚
U+411A.0	fCNS11643	4-5176
U+411A.0	fGB	3-573D
U+411A.0	fR/S	 113.11
U+411A.0	fTotalStrokes	 16
U+411A.0	fCangjie	IFTWA
U+411A.0	fFourCorner	35260
U+411A.0	fCantonese	cou4; zou1
U+411A.0	fMandarin	cao2
U+411A.0	fEnglish	[1] worship the god (of the hog, pig) [2] blessed; bliss; divine help [3] protect; defend
U+411B.0	fUTF8	䄛
U+411B.0	fCNS11643	3-5225
U+411B.0	fGB	5-5765
U+411B.0	fR/S	 113.11
U+411B.0	fTotalStrokes	 16; 15
U+411B.0	fCangjie	IFLLV
U+411B.0	fFourCorner	35240
U+411B.0	fCantonese	leoi4; laau4
U+411B.0	fMandarin	lou2
U+411B.0	fEnglish	standard form of &#x81A2;
U+411C.0	fUTF8	䄜
U+411C.0	fCNS11643	4-5174
U+411C.0	fGB	3-573A
U+411C.0	fR/S	 113.11
U+411C.0	fTotalStrokes	 16; 15
U+411C.0	fCangjie	IFYUB
U+411C.0	fFourCorner	30220
U+411C.0	fHakka	 li2
U+411C.0	fCantonese	lei4
U+411C.0	fMandarin	li2
U+411C.0	fEnglish	good omens; lucky auspices
U+411D.0	fUTF8	䄝
U+411D.0	fCNS11643	4-5175
U+411D.0	fGB	3-573C
U+411D.0	fR/S	 113.11
U+411D.0	fTotalStrokes	 16; 15
U+411D.0	fCangjie	IFQKX
U+411D.0	fFourCorner	35270
U+411D.0	fCantonese	cong1; ceon1; ceon2
U+411D.0	fMandarin	chuang1; chun3; zhuang1
U+411D.0	fEnglish	 show no respect to the service of worship
U+411E.0	fUTF8	䄞
U+411E.0	fR/S	 113.11
U+411E.0	fTotalStrokes	 16
U+411E.0	fCangjie	IFBOF
U+411E.0	fCantonese	zai3
U+411E.0	fEnglish	[Korean character]
U+411F.0	fUTF8	䄟
U+411F.0	fCNS11643	4-5742
U+411F.0	fGB	3-5678
U+411F.0	fR/S	 113.12
U+411F.0	fTotalStrokes	 17; 16
U+411F.0	fCangjie	HUHUF
U+411F.0	fFourCorner	20900
U+411F.0	fCantonese	ceoi3; waan2
U+411F.0	fMandarin	chui4; cui4; cuan3
U+411F.0	fEnglish	[1] worship; offer sacrifices couple times [2] thank
U+4120.0	fUTF8	䄠
U+4120.0	fCNS11643	3-5846
U+4120.0	fR/S	 113.13
U+4120.0	fTotalStrokes	 18; 17
U+4120.0	fCangjie	IFYWM
U+4120.0	fFourCorner	30210
U+4120.0	fHakka	 shan5; san5; sam2; sham2; shen5
U+4120.0	fCantonese	sim4; sin6
U+4120.0	fMandarin	shan4
U+4120.0	fEnglish	[1] sacrifice to heaven [2] imperial power
U+4121.0	fUTF8	䄡
U+4121.0	fCNS11643	3-5847
U+4121.0	fR/S	 113.13
U+4121.0	fTotalStrokes	 18
U+4121.0	fCangjie	IFNCR
U+4121.0	fFourCorner	37260
U+4121.0	fCantonese	daam1
U+4121.0	fEnglish	[1] northern minority ethnic group (in ancient times) [2] (corrupted form of &#x895C;
U+4122.0	fUTF8	䄢
U+4122.0	fCNS11643	3-5A64
U+4122.0	fR/S	 113.14
U+4122.0	fTotalStrokes	 19
U+4122.0	fCangjie	IFYX
U+4122.0	fFourCorner	30220
U+4122.0	fCantonese	zaai1
U+4122.0	fMandarin	qi2
U+4122.0	fEnglish	[1] same as &#x9F4B; [2] same as &#x9F4A;
U+4123.0	fUTF8	䄣
U+4123.0	fR/S	 113.15
U+4123.0	fTotalStrokes	 20
U+4123.0	fCangjie	IFGWC
U+4123.0	fCantonese	suk6
U+4123.0	fEnglish	[Korean character]
U+4124.0	fUTF8	䄤
U+4124.0	fCNS11643	4-6663
U+4124.0	fGB	3-5741
U+4124.0	fR/S	 113.16
U+4124.0	fTotalStrokes	 21; 20
U+4124.0	fCangjie	IFDLC
U+4124.0	fFourCorner	37280
U+4124.0	fHakka	 lai5; lai6
U+4124.0	fCantonese	laai6
U+4124.0	fMandarin	lai4; lan3
U+4124.0	fEnglish	 destroy
U+4125.0	fUTF8	䄥
U+4125.0	fCNS11643	3-6234
U+4125.0	fR/S	 113.24
U+4125.0	fTotalStrokes	 29; 28
U+4125.0	fCangjie	IFMBO
U+4125.0	fFourCorner	31210
U+4125.0	fCantonese	ling4
U+4125.0	fMandarin	ling2
U+4125.0	fEnglish	 god; spirit; immortal
U+4126.0	fUTF8	䄦
U+4126.0	fCNS11643	5-246E
U+4126.0	fR/S	 115.2
U+4126.0	fTotalStrokes	 7
U+4126.0	fCangjie	HDNN
U+4126.0	fFourCorner	27920
U+4126.0	fCantonese	liu5
U+4126.0	fMandarin	liao3
U+4126.0	fEnglish	 put forth ears and to blossom of the grains (corns and cereals)
U+4127.0	fUTF8	䄧
U+4127.0	fCNS11643	3-267D
U+4127.0	fR/S	 115.2
U+4127.0	fTotalStrokes	 7
U+4127.0	fCangjie	HDNHS
U+4127.0	fFourCorner	27920
U+4127.0	fCantonese	jing4; zing4
U+4127.0	fMandarin	reng2; rong3
U+4127.0	fEnglish	a kind of grain
U+4128.0	fUTF8	䄨
U+4128.0	fCNS11643	5-2746
U+4128.0	fR/S	 115.3
U+4128.0	fTotalStrokes	 8
U+4128.0	fCangjie	HDMD
U+4128.0	fFourCorner	21940
U+4128.0	fHakka	 i2
U+4128.0	fCantonese	jyu4; zaau4; zeoi4; zyu1
U+4128.0	fMandarin	yu2; yu3
U+4128.0	fEnglish	grains producing neither flower nor fruit
U+4129.0	fUTF8	䄩
U+4129.0	fCNS11643	6-2D49
U+4129.0	fR/S	 115.3
U+4129.0	fTotalStrokes	 8
U+4129.0	fCangjie	HDIP
U+4129.0	fFourCorner	23940
U+4129.0	fCantonese	jik6; zik6
U+4129.0	fMandarin	yi4; nao2
U+4129.0	fEnglish	same as &#x4D2C;
U+412A.0	fUTF8	䄪
U+412A.0	fCNS11643	4-2756
U+412A.0	fGB	5-5A79
U+412A.0	fR/S	 115.3
U+412A.0	fTotalStrokes	 8
U+412A.0	fCangjie	HDPI
U+412A.0	fFourCorner	27920
U+412A.0	fHakka	 diau5
U+412A.0	fCantonese	tiu2; diu2
U+412A.0	fMandarin	diao3; chuo4
U+412A.0	fEnglish	 take; cary away
U+412B.0	fUTF8	䄫
U+412B.0	fCNS11643	3-2A40
U+412B.0	fGB	5-5A7B
U+412B.0	fR/S	 115.3
U+412B.0	fTotalStrokes	 8
U+412B.0	fCangjie	HDSU
U+412B.0	fFourCorner	27910
U+412B.0	fCantonese	hei2
U+412B.0	fMandarin	qi3
U+412B.0	fEnglish	same as &#x8291;
U+412C.0	fUTF8	䄬
U+412C.0	fCNS11643	3-2A41
U+412C.0	fR/S	 115.3
U+412C.0	fTotalStrokes	 8
U+412C.0	fCangjie	HDPD
U+412C.0	fFourCorner	24912
U+412C.0	fCantonese	ji4; zyu4; ho1
U+412C.0	fMandarin	yi2
U+412C.0	fEnglish	same as &#x79FB;
U+412D.0	fUTF8	䄭
U+412D.0	fCNS11643	3-2A43
U+412D.0	fGB	5-5A77
U+412D.0	fR/S	 115.3
U+412D.0	fTotalStrokes	 8
U+412D.0	fCangjie	HDHJ
U+412D.0	fFourCorner	22940
U+412D.0	fCantonese	nin4; lin4 (also); zin2
U+412D.0	fMandarin	nian2
U+412D.0	fEnglish	same as &#x5E74;
U+412E.0	fUTF8	䄮
U+412E.0	fCNS11643	4-2A6A
U+412E.0	fGB	3-5B55
U+412E.0	fR/S	 115.4
U+412E.0	fTotalStrokes	 9
U+412E.0	fCangjie	HDQO
U+412E.0	fFourCorner	25930
U+412E.0	fHakka	 fu1
U+412E.0	fCantonese	fu1
U+412E.0	fMandarin	fu1
U+412E.0	fEnglish	[1] dark rice plant [2] millet grain (in black color) [3] same as &#x7A03;
U+412F.0	fUTF8	䄯
U+412F.0	fCNS11643	4-2A6F
U+412F.0	fGB	3-5B54
U+412F.0	fR/S	 115.4
U+412F.0	fTotalStrokes	 9
U+412F.0	fCangjie	HDMT
U+412F.0	fFourCorner	21940
U+412F.0	fCantonese	gin2; zin2
U+412F.0	fMandarin	jian3; xian4
U+412F.0	fEnglish	ten handle of grains (rice plant)
U+4130.0	fUTF8	䄰
U+4130.0	fCNS11643	4-2A6B
U+4130.0	fGB	3-5B59
U+4130.0	fR/S	 115.4
U+4130.0	fTotalStrokes	 9
U+4130.0	fCangjie	HDMVH
U+4130.0	fFourCorner	21940
U+4130.0	fCantonese	ngaa4; aa4 (also); fong2
U+4130.0	fMandarin	ya2; zha2
U+4130.0	fEnglish	[1] forty bundles of rice plant [2] panicled millet [3] same as &#x82BD;
U+4131.0	fUTF8	䄱
U+4131.0	fCNS11643	4-2A69
U+4131.0	fR/S	 115.4
U+4131.0	fTotalStrokes	 9
U+4131.0	fCangjie	HDYHS
U+4131.0	fFourCorner	20920
U+4131.0	fCantonese	fong1
U+4131.0	fMandarin	fang1
U+4131.0	fEnglish	a kind of grain
U+4132.0	fUTF8	䄲
U+4132.0	fCNS11643	4-2A6C
U+4132.0	fGB	3-5B5A
U+4132.0	fR/S	 115.4
U+4132.0	fTotalStrokes	 9
U+4132.0	fCangjie	HDOB
U+4132.0	fFourCorner	24920
U+4132.0	fCantonese	jeoi6; ceoi4
U+4132.0	fMandarin	rui4; chui2
U+4132.0	fEnglish	[1] inside; within; inner; interior; domestic [2] heartlessness
U+4133.0	fUTF8	䄳
U+4133.0	fCNS11643	F-2C6D
U+4133.0	fGB	S-2254
U+4133.0	fR/S	 115.4
U+4133.0	fTotalStrokes	 9
U+4133.0	fCangjie	HDYLM
U+4133.0	fFourCorner	2191
U+4133.0	fCantonese	sin1; cin1
U+4134.0	fUTF8	䄴
U+4134.0	fGB	S-2255
U+4134.0	fR/S	 115.4
U+4134.0	fTotalStrokes	 9
U+4134.0	fCangjie	HDB
U+4134.0	fCantonese	jyut6
U+4135.0	fUTF8	䄵
U+4135.0	fR/S	 115.4
U+4135.0	fTotalStrokes	 9
U+4135.0	fCangjie	HDQ
U+4135.0	fCantonese	lai4
U+4135.0	fEnglish	[Korean character]
U+4136.0	fUTF8	䄶
U+4136.0	fCNS11643	4-2E65
U+4136.0	fGB	3-5B66
U+4136.0	fR/S	 115.5
U+4136.0	fTotalStrokes	 10
U+4136.0	fCangjie	HDLLN
U+4136.0	fFourCorner	25920
U+4136.0	fCantonese	bat6; baat6; bui6
U+4136.0	fMandarin	bi4; bo2
U+4136.0	fEnglish	[1] rice plant growing the second time [2] the ear of grain (corn, millet etc.) growing upward
U+4137.0	fUTF8	䄷
U+4137.0	fCNS11643	3-3275
U+4137.0	fGB	5-5B24
U+4137.0	fR/S	 115.5
U+4137.0	fTotalStrokes	 10
U+4137.0	fCangjie	HDMR
U+4137.0	fFourCorner	21960
U+4137.0	fCantonese	sek6
U+4137.0	fMandarin	shi2; dan4; diao3
U+4137.0	fEnglish	same as &#x77F3;
U+4138.0	fUTF8	䄸
U+4138.0	fCNS11643	3-327A
U+4138.0	fGB	S-2256
U+4138.0	fR/S	 115.5
U+4138.0	fTotalStrokes	 10
U+4138.0	fCangjie	HDHA
U+4138.0	fFourCorner	26900
U+4138.0	fCantonese	pok3
U+4138.0	fMandarin	po4
U+4138.0	fEnglish	grains producing no fruit
U+4139.0	fUTF8	䄹
U+4139.0	fGB	S-2436
U+4139.0	fR/S	 115.5
U+4139.0	fTotalStrokes	 10
U+4139.0	fCangjie	HDOHJ
U+4139.0	fCantonese	nin4; lin4 (also)
U+4139.0	fMandarin	nian2; tian3
U+4139.0	fEnglish	same as &#x5E74;
U+413A.0	fUTF8	䄺
U+413A.0	fCNS11643	6-4266
U+413A.0	fR/S	 115.6
U+413A.0	fTotalStrokes	 11
U+413A.0	fCangjie	HDKN
U+413A.0	fFourCorner	25930
U+413A.0	fHakka	  ti2
U+413A.0	fCantonese	zi6
U+413A.0	fMandarin	shi2; zhi4
U+413A.0	fEnglish	tares; a kind of panic grass
U+413B.0	fUTF8	䄻
U+413B.0	fCNS11643	4-3367
U+413B.0	fGB	3-5B73
U+413B.0	fR/S	 115.6
U+413B.0	fTotalStrokes	 11
U+413B.0	fCangjie	HDLMO
U+413B.0	fFourCorner	22910
U+413B.0	fCantonese	tou4; zau6
U+413B.0	fMandarin	chao2; tiao1; tao2; tao1; zhao4
U+413B.0	fEnglish	[1] a second time growing of rice plant [2] rice plant [3] sorghum; kaoliang
U+413C.0	fUTF8	䄼
U+413C.0	fCNS11643	4-3366
U+413C.0	fGB	3-5B6C
U+413C.0	fR/S	 115.6
U+413C.0	fTotalStrokes	 11
U+413C.0	fCangjie	HDMOW
U+413C.0	fFourCorner	21960
U+413C.0	fCantonese	tim2
U+413C.0	fMandarin	tian3
U+413C.0	fEnglish	the name of a place in ancient times
U+413D.0	fUTF8	䄽
U+413D.0	fCNS11643	3-376D
U+413D.0	fR/S	 115.6
U+413D.0	fTotalStrokes	 11
U+413D.0	fCangjie	HDMCW
U+413D.0	fFourCorner	21960
U+413D.0	fCantonese	sin1; tim2
U+413D.0	fMandarin	xian1; tian3
U+413D.0	fEnglish	[1] same as &#x79C8;, &#x89B9; [2] corrupted form of &#x413C;
U+413E.0	fUTF8	䄾
U+413E.0	fCNS11643	4-3365
U+413E.0	fGB	3-5B68
U+413E.0	fR/S	 115.6
U+413E.0	fTotalStrokes	 11
U+413E.0	fCangjie	HDIJ
U+413E.0	fFourCorner	23950
U+413E.0	fCantonese	juk6; zuk6
U+413E.0	fMandarin	ru4; rou4; rong3
U+413E.0	fEnglish	[1] thick; height [2] black millet
U+413F.0	fUTF8	䄿
U+413F.0	fCNS11643	5-343B
U+413F.0	fR/S	 115.6
U+413F.0	fTotalStrokes	 11
U+413F.0	fCangjie	HDLWP
U+413F.0	fFourCorner	25900
U+413F.0	fHakka	 ji5; ri5
U+413F.0	fCantonese	jai6; zaai6
U+413F.0	fMandarin	yi4
U+413F.0	fEnglish	an old term for rice
U+4140.0	fUTF8	䅀
U+4140.0	fCNS11643	5-3439
U+4140.0	fGB	3-5B6D
U+4140.0	fR/S	 115.6
U+4140.0	fTotalStrokes	 11
U+4140.0	fCangjie	HDMNN
U+4140.0	fFourCorner	22900
U+4140.0	fCantonese	lit6
U+4140.0	fMandarin	lie4; li4
U+4140.0	fEnglish	[1] the stalk (stem) of grain [2] neat and orderly rows of rice seedling,
U+4141.0	fUTF8	䅁
U+4141.0	fCNS11643	4-336A
U+4141.0	fGB	3-5B78
U+4141.0	fR/S	 115.6
U+4141.0	fTotalStrokes	 11
U+4141.0	fCangjie	JVHD
U+4141.0	fFourCorner	30900
U+4141.0	fCantonese	on3; ngon3 (also)
U+4141.0	fMandarin	an4
U+4141.0	fEnglish	 husk rice; get the grains by oppressing the ears of the rice plant
U+4142.0	fUTF8	䅂
U+4142.0	fCNS11643	4-3369
U+4142.0	fGB	3-5B74
U+4142.0	fR/S	 115.6
U+4142.0	fTotalStrokes	 11
U+4142.0	fCangjie	HDHER
U+4142.0	fFourCorner	27960
U+4142.0	fHakka	 hok7;  kok7
U+4142.0	fCantonese	hok6; kok3
U+4142.0	fMandarin	he4; he2
U+4142.0	fEnglish	a small kind of millet or panic grass
U+4143.0	fUTF8	䅃
U+4143.0	fCNS11643	4-336B
U+4143.0	fGB	3-5B6A
U+4143.0	fR/S	 115.6
U+4143.0	fTotalStrokes	 11
U+4143.0	fCangjie	MNHD
U+4143.0	fFourCorner	17900
U+4143.0	fHakka	  kiung2
U+4143.0	fCantonese	kung4
U+4143.0	fMandarin	qiong2
U+4143.0	fEnglish	 reap ripe grain
U+4144.0	fUTF8	䅄
U+4144.0	fCNS11643	3-3768
U+4144.0	fGB	5-5B2D
U+4144.0	fR/S	 115.6
U+4144.0	fTotalStrokes	 11
U+4144.0	fCangjie	HDKSS
U+4144.0	fFourCorner	24920
U+4144.0	fCantonese	lai6; lei6
U+4144.0	fMandarin	li4
U+4144.0	fEnglish	long grains
U+4145.0	fUTF8	䅅
U+4145.0	fCNS11643	4-336D
U+4145.0	fGB	5-5B27
U+4145.0	fR/S	 115.6
U+4145.0	fTotalStrokes	 11
U+4145.0	fCangjie	HDGG
U+4145.0	fFourCorner	24910
U+4145.0	fCantonese	gwai1
U+4145.0	fMandarin	gui1; wa1
U+4145.0	fEnglish	[1] farm tool; agricultural implements [2] till; plough; cultivate
U+4146.0	fUTF8	䅆
U+4146.0	fCNS11643	5-3438
U+4146.0	fR/S	 115.6
U+4146.0	fTotalStrokes	 11
U+4146.0	fCangjie	HDIMO
U+4146.0	fFourCorner	27980
U+4146.0	fCantonese	zi3; zik1
U+4146.0	fMandarin	zi4
U+4146.0	fEnglish	[1] dense; crowded; closely [2] a piece of wood used to strike off grain inm a measure thus - to level; all; to adjust; overall,
U+4147.0	fUTF8	䅇
U+4147.0	fCNS11643	3-376E
U+4147.0	fR/S	 115.6
U+4147.0	fTotalStrokes	 11
U+4147.0	fCangjie	MWHD
U+4147.0	fFourCorner	10900
U+4147.0	fHakka	 siuk7
U+4147.0	fCantonese	suk1
U+4147.0	fMandarin	su4
U+4147.0	fEnglish	ancient form of &#x7C9F;
U+4148.0	fUTF8	䅈
U+4148.0	fCNS11643	F-373B
U+4148.0	fR/S	 115.6
U+4148.0	fTotalStrokes	 11
U+4148.0	fCangjie	FQHD
U+4148.0	fFourCorner	9090
U+4148.0	fCantonese	jyun2; zyun2
U+4148.0	fMandarin	yuan4
U+4149.0	fUTF8	䅉
U+4149.0	fGB	S-2259
U+4149.0	fR/S	 115.6
U+4149.0	fTotalStrokes	 11
U+4149.0	fCangjie	HDMTC
U+4149.0	fCantonese	aa3; ngaa3 (also); ji2; zi2
U+4149.0	fMandarin	ya4
U+4149.0	fEnglish	simplified form of &#x7A0F;
U+414A.0	fUTF8	䅊
U+414A.0	fCNS11643	4-3364
U+414A.0	fGB	3-5B75
U+414A.0	fR/S	 115.6
U+414A.0	fTotalStrokes	 11
U+414A.0	fCangjie	HDIHP
U+414A.0	fFourCorner	20910
U+414A.0	fCantonese	caa4; dou3
U+414A.0	fMandarin	chong2; du4
U+414A.0	fEnglish	[1] a spacious house [2] corrupted form of &#x79FA;
U+414B.0	fUTF8	䅋
U+414B.0	fCNS11643	5-3A65
U+414B.0	fR/S	 115.7
U+414B.0	fTotalStrokes	 12
U+414B.0	fCangjie	HDNAU
U+414B.0	fFourCorner	27910
U+414B.0	fCantonese	maan5; wan5
U+414B.0	fMandarin	wan3
U+414B.0	fEnglish	a kind of grain; sweet and round rice
U+414C.0	fUTF8	䅌
U+414C.0	fCNS11643	4-395E
U+414C.0	fGB	3-5B7B
U+414C.0	fR/S	 115.7
U+414C.0	fTotalStrokes	 12
U+414C.0	fCangjie	HDRB
U+414C.0	fFourCorner	26920
U+414C.0	fHakka	 gien1; gian1; gen1
U+414C.0	fCantonese	gyun1; haa1; syun6
U+414C.0	fMandarin	juan1
U+414C.0	fEnglish	the stalks of rice or wheat
U+414D.0	fUTF8	䅍
U+414D.0	fCNS11643	4-3962
U+414D.0	fR/S	 115.7
U+414D.0	fTotalStrokes	 12
U+414D.0	fCangjie	HDNKG
U+414D.0	fFourCorner	22940
U+414D.0	fHakka	 tin3
U+414D.0	fCantonese	ting5; zaau3; zim1
U+414D.0	fMandarin	ting3; dong4
U+414D.0	fEnglish	the stalk of grain
U+414E.0	fUTF8	䅎
U+414E.0	fCNS11643	3-3D67
U+414E.0	fR/S	 115.7
U+414E.0	fTotalStrokes	 12
U+414E.0	fCangjie	HDHDS
U+414E.0	fFourCorner	22920
U+414E.0	fCantonese	jau5; ngaa4; zaau5
U+414E.0	fMandarin	you3
U+414E.0	fEnglish	different; distinguish;
U+414F.0	fUTF8	䅏
U+414F.0	fCNS11643	4-3964
U+414F.0	fR/S	 115.7
U+414F.0	fTotalStrokes	 12
U+414F.0	fCangjie	HDSHU
U+414F.0	fFourCorner	27910
U+414F.0	fCantonese	mei6; mei5; min5
U+414F.0	fMandarin	wei3; wei4; hui4
U+414F.0	fEnglish	[1] rice boiled to gruel [2] congee; porridge
U+4150.0	fUTF8	䅐
U+4150.0	fCNS11643	3-3D6A
U+4150.0	fR/S	 115.7
U+4150.0	fTotalStrokes	 12
U+4150.0	fCangjie	HDBUU
U+4150.0	fFourCorner	26910
U+4150.0	fCantonese	gin2; gan2; zin6
U+4150.0	fMandarin	jian3; qian2
U+4150.0	fEnglish	[1] bundle [2] bind; bondage [3] a smal bundle of [4] measure word; 10 bundle of rice plants
U+4151.0	fUTF8	䅑
U+4151.0	fCNS11643	4-3963
U+4151.0	fGB	3-5B7D
U+4151.0	fR/S	 115.7
U+4151.0	fTotalStrokes	 12
U+4151.0	fCangjie	HDBV
U+4151.0	fFourCorner	22940
U+4151.0	fHakka	  sui1
U+4151.0	fCantonese	jeoi4; zeoi4
U+4151.0	fMandarin	rui2; sui2
U+4151.0	fEnglish	four bundle of grains
U+4152.0	fUTF8	䅒
U+4152.0	fCNS11643	3-3D6B
U+4152.0	fR/S	 115.7
U+4152.0	fTotalStrokes	 12
U+4152.0	fCangjie	HDTYV
U+4152.0	fFourCorner	24910
U+4152.0	fCantonese	mong4
U+4152.0	fMandarin	mang2
U+4152.0	fEnglish	interchangeable with &#x8292;
U+4153.0	fUTF8	䅓
U+4153.0	fCNS11643	4-3961
U+4153.0	fGB	3-5B7E
U+4153.0	fR/S	 115.7
U+4153.0	fTotalStrokes	 12
U+4153.0	fCangjie	HDPRE
U+4153.0	fFourCorner	27940
U+4153.0	fCantonese	geoi2
U+4153.0	fMandarin	ju3; qu4
U+4153.0	fEnglish	[1] curved piece of wood [2] bend; crouch [3] the name of a tree [4] the name of a fruit
U+4154.0	fUTF8	䅔
U+4154.0	fCNS11643	4-3F4F
U+4154.0	fGB	3-5C2D
U+4154.0	fR/S	 115.8
U+4154.0	fTotalStrokes	 13
U+4154.0	fCangjie	HDVVW
U+4154.0	fFourCorner	22960
U+4154.0	fCantonese	zi1
U+4154.0	fMandarin	zi1
U+4154.0	fEnglish	[1] dead (withered) crops; dried grains [2] same as &#x43A9;
U+4155.0	fUTF8	䅕
U+4155.0	fCNS11643	3-4340
U+4155.0	fR/S	 115.8
U+4155.0	fTotalStrokes	 13
U+4155.0	fCangjie	HDSJR
U+4155.0	fFourCorner	27960
U+4155.0	fCantonese	geoi1
U+4155.0	fMandarin	ju1
U+4155.0	fEnglish	a variety of millet
U+4156.0	fUTF8	䅖
U+4156.0	fCNS11643	4-3F4D
U+4156.0	fGB	3-5C26
U+4156.0	fR/S	 115.8
U+4156.0	fTotalStrokes	 13
U+4156.0	fCangjie	HDKLU
U+4156.0	fFourCorner	24910
U+4156.0	fCantonese	am1; ngam1 (also); aam1
U+4156.0	fMandarin	an1; an3
U+4156.0	fEnglish	same as &#x99A3;
U+4157.0	fUTF8	䅗
U+4157.0	fCNS11643	3-4345
U+4157.0	fGB	5-5B37
U+4157.0	fR/S	 115.8
U+4157.0	fTotalStrokes	 13
U+4157.0	fCangjie	HDHDV
U+4157.0	fFourCorner	22940
U+4157.0	fHakka	  sui1
U+4157.0	fCantonese	seoi3; seoi6
U+4157.0	fMandarin	sui4
U+4157.0	fEnglish	[1] push; crowd [2] rumples; creased
U+4158.0	fUTF8	䅘
U+4158.0	fCNS11643	4-3F4C
U+4158.0	fGB	5-5B2E
U+4158.0	fR/S	 115.8
U+4158.0	fTotalStrokes	 13
U+4158.0	fCangjie	HDDOO
U+4158.0	fFourCorner	24930
U+4158.0	fHakka	  loi2
U+4158.0	fCantonese	loi4
U+4158.0	fMandarin	lai2
U+4158.0	fEnglish	an old name for wheat
U+4159.0	fUTF8	䅙
U+4159.0	fCNS11643	3-4342
U+4159.0	fR/S	 115.8
U+4159.0	fTotalStrokes	 13
U+4159.0	fCangjie	HDAPP
U+4159.0	fFourCorner	26910
U+4159.0	fCantonese	muk6; waan6
U+4159.0	fMandarin	hun3; hun4
U+4159.0	fEnglish	[1] grass; herb; straw; weed [2] bundle of straw
U+415A.0	fUTF8	䅚
U+415A.0	fCNS11643	4-3F4B
U+415A.0	fGB	3-5C2B
U+415A.0	fR/S	 115.8
U+415A.0	fTotalStrokes	 13
U+415A.0	fCangjie	HDFQU
U+415A.0	fFourCorner	29910
U+415A.0	fCantonese	hyun2; ci1; hyun3; nei5; seon3; sit3
U+415A.0	fMandarin	quan4; qiang3; quan3; ze2
U+415A.0	fEnglish	closely growing of the rice plants
U+415B.0	fUTF8	䅛
U+415B.0	fCNS11643	3-4343
U+415B.0	fGB	5-5B36
U+415B.0	fR/S	 115.8
U+415B.0	fTotalStrokes	 13
U+415B.0	fCangjie	HDAA
U+415B.0	fFourCorner	26960
U+415B.0	fCantonese	coeng1
U+415B.0	fMandarin	chang1
U+415B.0	fEnglish	husks of rice; rice bran or chaff
U+415C.0	fUTF8	䅜
U+415C.0	fCNS11643	4-4557
U+415C.0	fGB	3-5C29
U+415C.0	fR/S	 115.8; 115.9
U+415C.0	fTotalStrokes	 14; 13
U+415C.0	fCangjie	HDHJM
U+415C.0	fFourCorner	22910
U+415C.0	fCantonese	do6
U+415C.0	fMandarin	duo3; duo4
U+415C.0	fEnglish	[1] store up grains [2] weeping rice plants [3] an ear of grain
U+415D.0	fUTF8	䅝
U+415D.0	fCNS11643	4-3F49
U+415D.0	fGB	5-5B3B
U+415D.0	fR/S	 115.8
U+415D.0	fTotalStrokes	 13
U+415D.0	fCangjie	HDJCM
U+415D.0	fFourCorner	23910
U+415D.0	fCantonese	hung1
U+415D.0	fMandarin	kong1
U+415D.0	fEnglish	the stalk of grain; straw
U+415E.0	fUTF8	䅞
U+415E.0	fCNS11643	5-4131
U+415E.0	fR/S	 115.8
U+415E.0	fTotalStrokes	 13
U+415E.0	fCangjie	HDAMI
U+415E.0	fFourCorner	26940
U+415E.0	fCantonese	nak6; lak6 (also); noi6
U+415E.0	fMandarin	ne2; na4; nai4; ne4
U+415E.0	fEnglish	stalks of grain (rice plant)
U+415F.0	fUTF8	䅟
U+415F.0	fGB	7-2222
U+415F.0	fR/S	 115.8
U+415F.0	fTotalStrokes	 13
U+415F.0	fCangjie	HDIKH
U+415F.0	fCantonese	saam1; caam2 (also); sam1
U+415F.0	fMandarin	can3
U+415F.0	fEnglish	simplified form of &#x7A47;
U+4160.0	fUTF8	䅠
U+4160.0	fCNS11643	3-4851
U+4160.0	fR/S	 115.9
U+4160.0	fTotalStrokes	 14
U+4160.0	fCangjie	HDAMO
U+4160.0	fFourCorner	26980
U+4160.0	fCantonese	tai4; taai4; teon1
U+4160.0	fMandarin	ti2
U+4160.0	fEnglish	[1] same as &#x855B; [2] interchangeable with &#x7A0A;
U+4161.0	fUTF8	䅡
U+4161.0	fCNS11643	3-484A
U+4161.0	fR/S	 115.9
U+4161.0	fTotalStrokes	 14
U+4161.0	fCangjie	HDMBC
U+4161.0	fFourCorner	21980
U+4161.0	fCantonese	seoi2
U+4161.0	fMandarin	xu3
U+4161.0	fEnglish	[1] a kind of grass [2] grass growing in between of (among) the grains
U+4162.0	fUTF8	䅢
U+4162.0	fCNS11643	4-455B
U+4162.0	fGB	3-5C33
U+4162.0	fR/S	 115.9
U+4162.0	fTotalStrokes	 14
U+4162.0	fCangjie	HDLWB
U+4162.0	fFourCorner	25920
U+4162.0	fHakka	 ciu5; ciu6
U+4162.0	fCantonese	zau3; zaau6
U+4162.0	fMandarin	jiu4
U+4162.0	fEnglish	rice fuly ripe
U+4163.0	fUTF8	䅣
U+4163.0	fCNS11643	3-4853
U+4163.0	fGB	5-5B3F
U+4163.0	fR/S	 115.9
U+4163.0	fTotalStrokes	 14
U+4163.0	fCangjie	HDHAG
U+4163.0	fFourCorner	26910
U+4163.0	fCantonese	wong4
U+4163.0	fMandarin	huang2
U+4163.0	fEnglish	[1] a variety of panicled millet [2] a small coarse grain resembling sorghum
U+4164.0	fUTF8	䅤
U+4164.0	fCNS11643	3-484E
U+4164.0	fR/S	 115.9
U+4164.0	fTotalStrokes	 14
U+4164.0	fCangjie	HDHJX
U+4164.0	fFourCorner	22970
U+4164.0	fCantonese	cap1; hei3
U+4164.0	fMandarin	cha1; qi4
U+4164.0	fEnglish	 plant; sow; cultivate
U+4165.0	fUTF8	䅥
U+4165.0	fCNS11643	4-455A
U+4165.0	fGB	3-5C31
U+4165.0	fR/S	 115.9
U+4165.0	fTotalStrokes	 14
U+4165.0	fCangjie	HDAPV
U+4165.0	fFourCorner	26920
U+4165.0	fCantonese	git3; kit3
U+4165.0	fMandarin	jie2
U+4165.0	fEnglish	[1] ripening crops [2] long grains [3] husks; chaff; bran (of grain)
U+4166.0	fUTF8	䅦
U+4166.0	fCNS11643	3-484C
U+4166.0	fR/S	 115.9
U+4166.0	fTotalStrokes	 14
U+4166.0	fCangjie	HDTW
U+4166.0	fFourCorner	24960
U+4166.0	fCantonese	maau4; nau4
U+4166.0	fMandarin	mao2
U+4166.0	fEnglish	feeble; weak of the rice plant
U+4167.0	fUTF8	䅧
U+4167.0	fCNS11643	4-4558
U+4167.0	fGB	3-5C35
U+4167.0	fR/S	 115.9
U+4167.0	fTotalStrokes	 14
U+4167.0	fCangjie	HDYTA
U+4167.0	fFourCorner	20960
U+4167.0	fCantonese	jim1; zaam1; zaam3; zim3
U+4167.0	fMandarin	yan1; yan4
U+4167.0	fEnglish	[1] fine rice seedling; growing rice in order [2] same as &#x852D;
U+4168.0	fUTF8	䅨
U+4168.0	fCNS11643	6-5F6B
U+4168.0	fGB	S-2258
U+4168.0	fR/S	 115.9
U+4168.0	fTotalStrokes	 14
U+4168.0	fCangjie	HDHDA
U+4168.0	fFourCorner	22960
U+4168.0	fCantonese	hoeng1; heoi2
U+4168.0	fMandarin	xiang1
U+4168.0	fEnglish	same as &#x9999;
U+4169.0	fUTF8	䅩
U+4169.0	fCNS11643	4-4559
U+4169.0	fGB	5-5B3D
U+4169.0	fR/S	 115.9
U+4169.0	fTotalStrokes	 14
U+4169.0	fCangjie	HDJEC
U+4169.0	fFourCorner	24980
U+4169.0	fCantonese	zi2
U+4169.0	fMandarin	zhi3
U+4169.0	fEnglish	[1] crooked and winding of the grass (trees, vegetation, flora) [2] crooked branches of a tree adjoin each other [3] same as &#x6B67;
U+416A.0	fUTF8	䅪
U+416A.0	fGB	S-2253
U+416A.0	fR/S	 115.9
U+416A.0	fTotalStrokes	 14
U+416A.0	fCangjie	HDLMO
U+416A.0	fCantonese	teoi4
U+416A.0	fMandarin	tui2
U+416A.0	fEnglish	disease of the private part; hernia; colic
U+416B.0	fUTF8	䅫
U+416B.0	fR/S	 115.9
U+416B.0	fTotalStrokes	 14
U+416B.0	fCangjie	HDYAF
U+416B.0	fCantonese	loek6
U+416B.0	fEnglish	[Korean character]
U+416C.0	fUTF8	䅬
U+416C.0	fCNS11643	4-4B65
U+416C.0	fGB	3-5C40
U+416C.0	fR/S	 115.10
U+416C.0	fTotalStrokes	 15
U+416C.0	fCangjie	HDTCT
U+416C.0	fFourCorner	28910
U+416C.0	fCantonese	aai3; ngaai3 (also); ai3
U+416C.0	fMandarin	ai4; yan3; ye4
U+416C.0	fEnglish	[1] small bundle of rice plant [2] seed [3] grain
U+416D.0	fUTF8	䅭
U+416D.0	fCNS11643	4-4B61
U+416D.0	fGB	3-5C3E
U+416D.0	fR/S	 115.10
U+416D.0	fTotalStrokes	 15
U+416D.0	fCangjie	HDYBS
U+416D.0	fFourCorner	20920
U+416D.0	fCantonese	pong4
U+416D.0	fMandarin	pang2
U+416D.0	fEnglish	[1] a variety of panicled millet [2] a small coarse grain resembling sorghum
U+416E.0	fUTF8	䅮
U+416E.0	fCNS11643	3-4D54
U+416E.0	fR/S	 115.10
U+416E.0	fTotalStrokes	 15
U+416E.0	fCangjie	HDOIR
U+416E.0	fFourCorner	28960
U+416E.0	fCantonese	cong3
U+416E.0	fMandarin	cang4
U+416E.0	fEnglish	a full head of grain; bent over by its own weight, two small pieces bract on the base of the ears of a rice plant, grass; weeds; tares
U+416F.0	fUTF8	䅯
U+416F.0	fCNS11643	4-4B62
U+416F.0	fR/S	 115.10
U+416F.0	fTotalStrokes	 15
U+416F.0	fCangjie	HDILR
U+416F.0	fFourCorner	20960
U+416F.0	fCantonese	tong4
U+416F.0	fMandarin	tang2
U+416F.0	fEnglish	a variety of millet
U+4170.0	fUTF8	䅰
U+4170.0	fCNS11643	3-4D52
U+4170.0	fR/S	 115.10
U+4170.0	fTotalStrokes	 15
U+4170.0	fCangjie	HDWKP
U+4170.0	fFourCorner	26930
U+4170.0	fCantonese	an2; ngan2 (also); zaan2
U+4170.0	fMandarin	en3
U+4170.0	fEnglish	a kind of grass
U+4171.0	fUTF8	䅱
U+4171.0	fCNS11643	7-2632
U+4171.0	fR/S	 115.10
U+4171.0	fTotalStrokes	 15
U+4171.0	fCangjie	HDUMT
U+4171.0	fFourCorner	22910
U+4171.0	fCantonese	ngan6; an6 (also); tong3; waan6
U+4171.0	fMandarin	wen4; hun4
U+4171.0	fEnglish	 eat to the full; surfeited
U+4172.0	fUTF8	䅲
U+4172.0	fCNS11643	3-4D58
U+4172.0	fGB	5-5B42
U+4172.0	fR/S	 115.10
U+4172.0	fTotalStrokes	 15
U+4172.0	fCangjie	HDJPA
U+4172.0	fFourCorner	24960
U+4172.0	fCantonese	kei4
U+4172.0	fMandarin	qi2
U+4172.0	fEnglish	 plant or sow seed of the wheat
U+4173.0	fUTF8	䅳
U+4173.0	fCNS11643	4-4B68
U+4173.0	fGB	3-5B64
U+4173.0	fR/S	 115.10
U+4173.0	fTotalStrokes	 15
U+4173.0	fCangjie	HDPUU
U+4173.0	fFourCorner	27920
U+4173.0	fHakka	 cu1; ze1; cih1
U+4173.0	fCantonese	co4; co1
U+4173.0	fMandarin	chu2
U+4173.0	fEnglish	the stalk of millet
U+4174.0	fUTF8	䅴
U+4174.0	fCNS11643	4-4B66
U+4174.0	fGB	3-5C3A
U+4174.0	fR/S	 115.10
U+4174.0	fTotalStrokes	 15
U+4174.0	fCangjie	HDJBF
U+4174.0	fFourCorner	24990
U+4174.0	fCantonese	sok3
U+4174.0	fMandarin	suo3
U+4174.0	fEnglish	[1] grains [2] rice plant [3] an ear of grain
U+4175.0	fUTF8	䅵
U+4175.0	fCNS11643	4-4B64
U+4175.0	fGB	3-5C3F
U+4175.0	fR/S	 115.10
U+4175.0	fTotalStrokes	 15
U+4175.0	fCangjie	HDTGF
U+4175.0	fFourCorner	28930
U+4175.0	fCantonese	zoek3
U+4175.0	fMandarin	zhuo2
U+4175.0	fEnglish	[1] chaff; bran; husks of grain [2] the name of a place in ancient times
U+4176.0	fUTF8	䅶
U+4176.0	fCNS11643	3-4D59
U+4176.0	fR/S	 115.10
U+4176.0	fTotalStrokes	 15
U+4176.0	fCangjie	HDMVI
U+4176.0	fFourCorner	21940
U+4176.0	fCantonese	nau6; lau6 (also); naau6; zi4
U+4176.0	fMandarin	ru4; nou4; wu3
U+4176.0	fEnglish	[1] hoe [2] weed; hoe up weeds
U+4177.0	fUTF8	䅷
U+4177.0	fCNS11643	4-517D
U+4177.0	fGB	3-5C38
U+4177.0	fR/S	 115.11
U+4177.0	fTotalStrokes	 16
U+4177.0	fCangjie	HDTOD
U+4177.0	fFourCorner	24990
U+4177.0	fCantonese	tou4
U+4177.0	fMandarin	tu2
U+4177.0	fEnglish	[1] an ear of grain [2] non-classical form of &#x7A0C; [3] same as &#x85F7;
U+4178.0	fUTF8	䅸
U+4178.0	fCNS11643	4-517C
U+4178.0	fGB	3-5C39
U+4178.0	fR/S	 115.11
U+4178.0	fTotalStrokes	 16
U+4178.0	fCangjie	HDTYJ
U+4178.0	fFourCorner	24940
U+4178.0	fCantonese	san1; saan1; zuk6
U+4178.0	fMandarin	shen1; zu2
U+4178.0	fEnglish	[1] a grain [2] same as &#x8398;
U+4179.0	fUTF8	䅹
U+4179.0	fCNS11643	3-522B
U+4179.0	fR/S	 115.11
U+4179.0	fTotalStrokes	 16
U+4179.0	fCangjie	HDLLV
U+4179.0	fFourCorner	25940
U+4179.0	fCantonese	lau4; laau5
U+4179.0	fMandarin	lou3; lou2
U+4179.0	fEnglish	same as &#x802C;
U+417A.0	fUTF8	䅺
U+417A.0	fCNS11643	4-5179
U+417A.0	fGB	3-5C42
U+417A.0	fR/S	 115.11
U+417A.0	fTotalStrokes	 16
U+417A.0	fCangjie	HDMWF
U+417A.0	fFourCorner	21990
U+417A.0	fCantonese	biu1
U+417A.0	fMandarin	biao1; miao3
U+417A.0	fEnglish	[1] grain in the ear; put forth ears [2] same as &#x79D2; [3] same as &#x8508;
U+417B.0	fUTF8	䅻
U+417B.0	fCNS11643	4-5178
U+417B.0	fGB	3-5C3C
U+417B.0	fR/S	 115.11
U+417B.0	fTotalStrokes	 16
U+417B.0	fCangjie	HDYUB
U+417B.0	fFourCorner	20920
U+417B.0	fCantonese	lei4
U+417B.0	fMandarin	li2
U+417B.0	fEnglish	[1] measurement; two bundle of grains [2] same as &#x96E2;
U+417C.0	fUTF8	䅼
U+417C.0	fCNS11643	4-517A
U+417C.0	fGB	3-5C43
U+417C.0	fR/S	 115.11
U+417C.0	fTotalStrokes	 16
U+417C.0	fCangjie	HDAWE
U+417C.0	fFourCorner	26940
U+417C.0	fCantonese	maan4; man4; man6
U+417C.0	fMandarin	man2
U+417C.0	fEnglish	[1] a kind of grains [2] plant; sow; cultivate
U+417D.0	fUTF8	䅽
U+417D.0	fCNS11643	3-522D
U+417D.0	fGB	5-5B45
U+417D.0	fR/S	 115.11
U+417D.0	fTotalStrokes	 16
U+417D.0	fCangjie	GEHD
U+417D.0	fFourCorner	47900
U+417D.0	fCantonese	hing1
U+417D.0	fMandarin	xin1; gu3
U+417D.0	fEnglish	[1] same as &#x99A8; [2] same as &#x6996;
U+417E.0	fUTF8	䅾
U+417E.0	fCNS11643	4-5745
U+417E.0	fGB	3-5C45
U+417E.0	fR/S	 115.12
U+417E.0	fTotalStrokes	 17; 16
U+417E.0	fCangjie	HDMUA
U+417E.0	fFourCorner	21960
U+417E.0	fCantonese	sam4
U+417E.0	fMandarin	cen2; qin2; qian2; zen4
U+417E.0	fEnglish	[1] starting to put forth ears [2] a kind of grain
U+417F.0	fUTF8	䅿
U+417F.0	fCNS11643	3-5563
U+417F.0	fR/S	 115.12
U+417F.0	fTotalStrokes	 17
U+417F.0	fCangjie	HDTMJ
U+417F.0	fFourCorner	24950
U+417F.0	fCantonese	waa4
U+417F.0	fMandarin	hua2
U+417F.0	fEnglish	[1] (non-classical form) luxuriant or exuberant of the grains [2] interchangeable &#x83EF;
U+4180.0	fUTF8	䆀
U+4180.0	fCNS11643	4-5747
U+4180.0	fGB	3-5C47
U+4180.0	fR/S	 115.12
U+4180.0	fTotalStrokes	 17; 16
U+4180.0	fCangjie	HDWGF
U+4180.0	fFourCorner	26931
U+4180.0	fHakka	 moi5; moi3
U+4180.0	fCantonese	mui5
U+4180.0	fMandarin	mai3; mei3; mei4; mai4
U+4180.0	fEnglish	mildewed grain
U+4181.0	fUTF8	䆁
U+4181.0	fCNS11643	3-5848
U+4181.0	fR/S	 115.13
U+4181.0	fTotalStrokes	 18
U+4181.0	fCangjie	HDWLJ
U+4181.0	fFourCorner	26940
U+4181.0	fCantonese	gou1
U+4181.0	fMandarin	gao1
U+4181.0	fEnglish	syrup; jelly-like sugar made from grains
U+4182.0	fUTF8	䆂
U+4182.0	fCNS11643	7-412E
U+4182.0	fGB	S-2257
U+4182.0	fR/S	 115.13
U+4182.0	fTotalStrokes	 18
U+4182.0	fCangjie	HDITC
U+4182.0	fFourCorner	20930
U+4182.0	fCantonese	lim4; lam4
U+4182.0	fMandarin	lian2; qian4
U+4182.0	fEnglish	[1] a kind of grain [2] same as &#x7A34;
U+4183.0	fUTF8	䆃
U+4183.0	fCNS11643	4-5C37
U+4183.0	fGB	3-5C4A
U+4183.0	fR/S	 115.13
U+4183.0	fTotalStrokes	 18
U+4183.0	fCangjie	YUHD
U+4183.0	fFourCorner	38900
U+4183.0	fHakka	 tau5; to6; to5; tau6
U+4183.0	fCantonese	dou6; dik6; gou3
U+4183.0	fMandarin	dao4; dao3
U+4183.0	fEnglish	 husk grain
U+4184.0	fUTF8	䆄
U+4184.0	fCNS11643	4-5C35
U+4184.0	fGB	3-5C4C
U+4184.0	fR/S	 115.13
U+4184.0	fTotalStrokes	 18
U+4184.0	fCangjie	HDYWM
U+4184.0	fFourCorner	20910
U+4184.0	fHakka	 san5; shan5; sen5; shen5; shen3
U+4184.0	fCantonese	zin2; sim4; sin6; sing6
U+4184.0	fMandarin	zhan3; shan4
U+4184.0	fEnglish	a sheaf or bundle of cut grain
U+4185.0	fUTF8	䆅
U+4185.0	fCNS11643	4-5C36
U+4185.0	fGB	3-5C3D
U+4185.0	fR/S	 115.13
U+4185.0	fTotalStrokes	 18
U+4185.0	fCangjie	HDIOC
U+4185.0	fFourCorner	27980
U+4185.0	fHakka	 zih1; zii1
U+4185.0	fCantonese	ci4; zi1
U+4185.0	fMandarin	zi1; ci2; zi4; ji3
U+4185.0	fEnglish	grain piled up on the threshing-floor
U+4186.0	fUTF8	䆆
U+4186.0	fR/S	 115.13
U+4186.0	fTotalStrokes	 18
U+4186.0	fCangjie	HDRRD
U+4186.0	fCantonese	cou3
U+4186.0	fEnglish	[Korean character]
U+4187.0	fUTF8	䆇
U+4187.0	fR/S	 115.13
U+4187.0	fTotalStrokes	 18
U+4187.0	fCangjie	HDHNF
U+4187.0	fCantonese	fung6
U+4187.0	fEnglish	[Korean character]
U+4188.0	fUTF8	䆈
U+4188.0	fCNS11643	3-5E46
U+4188.0	fR/S	 115.15
U+4188.0	fTotalStrokes	 21; 20
U+4188.0	fCangjie	HDYSQ
U+4188.0	fFourCorner	27930
U+4188.0	fCantonese	zi6
U+4188.0	fMandarin	zhi4
U+4188.0	fEnglish	non-classical form of &#x7A3A;, &#x7A1A;
U+4189.0	fUTF8	䆉
U+4189.0	fCNS11643	4-6379
U+4189.0	fGB	3-5C3B
U+4189.0	fR/S	 115.15
U+4189.0	fTotalStrokes	 20
U+4189.0	fCangjie	HDWLP
U+4189.0	fFourCorner	26910
U+4189.0	fCantonese	baa6; bit3; ping3
U+4189.0	fMandarin	ba4
U+4189.0	fEnglish	[1] the swing of rice plant [2] a kind of paddy
U+418A.0	fUTF8	䆊
U+418A.0	fCNS11643	4-6665
U+418A.0	fGB	3-5C4E
U+418A.0	fR/S	 115.16
U+418A.0	fTotalStrokes	 21
U+418A.0	fCangjie	HDLPC
U+418A.0	fFourCorner	21980
U+418A.0	fCantonese	seoi6
U+418A.0	fMandarin	cui4; mei4
U+418A.0	fEnglish	[1] unhusked glutinous rice [2] sow seeds
U+418B.0	fUTF8	䆋
U+418B.0	fCNS11643	7-584C
U+418B.0	fR/S	 115.16
U+418B.0	fTotalStrokes	 21; 22
U+418B.0	fCangjie	HDNXU
U+418B.0	fFourCorner	27910
U+418B.0	fCantonese	cau1
U+418B.0	fMandarin	qiu1
U+418B.0	fEnglish	ancient form of &#x79CB;
U+418C.0	fUTF8	䆌
U+418C.0	fGB	S-225A
U+418C.0	fR/S	 115.16
U+418C.0	fTotalStrokes	 21
U+418C.0	fCangjie	HDHUR
U+418C.0	fCantonese	kai1
U+418D.0	fUTF8	䆍
U+418D.0	fCNS11643	4-6664
U+418D.0	fGB	3-5B5E
U+418D.0	fR/S	 115.16
U+418D.0	fTotalStrokes	 21
U+418D.0	fCangjie	HDYBP
U+418D.0	fFourCorner	21910
U+418D.0	fHakka	 lung2
U+418D.0	fCantonese	lung4
U+418D.0	fMandarin	long2
U+418D.0	fEnglish	 reap grain
U+418E.0	fUTF8	䆎
U+418E.0	fCNS11643	4-687E
U+418E.0	fGB	3-5C50
U+418E.0	fR/S	 115.17
U+418E.0	fTotalStrokes	 22; 21
U+418E.0	fCangjie	HDOIM
U+418E.0	fFourCorner	23950
U+418E.0	fCantonese	cim1; lim4
U+418E.0	fMandarin	xian1; xian4
U+418E.0	fEnglish	grass and grains producing no fruit
U+418F.0	fUTF8	䆏
U+418F.0	fCNS11643	4-6921
U+418F.0	fGB	3-5C52
U+418F.0	fR/S	 115.17; 115.18
U+418F.0	fTotalStrokes	 22; 21; 23
U+418F.0	fCangjie	HDFDC
U+418F.0	fFourCorner	29980
U+418F.0	fCantonese	fai6; faai3; faan5
U+418F.0	fMandarin	fei4; fen4
U+418F.0	fEnglish	[1] a kind of rice plant (not glutinous and with purple colored stalk) [2] same as &#x7CDE;
U+4190.0	fUTF8	䆐
U+4190.0	fCNS11643	3-6239
U+4190.0	fR/S	 115.25
U+4190.0	fTotalStrokes	 30
U+4190.0	fCangjie	QDQDD
U+4190.0	fFourCorner	50990
U+4190.0	fCantonese	gwok3; gok3 (also)
U+4190.0	fMandarin	guo2
U+4190.0	fEnglish	ancient form of &#x570B;
U+4191.0	fUTF8	䆑
U+4191.0	fCNS11643	4-2469
U+4191.0	fGB	5-5E31
U+4191.0	fR/S	 116.2
U+4191.0	fTotalStrokes	 7
U+4191.0	fCangjie	JCMN
U+4191.0	fFourCorner	30200
U+4191.0	fCantonese	caang4; cang2; cang4
U+4191.0	fMandarin	cheng2
U+4191.0	fEnglish	a little protruded
U+4192.0	fUTF8	䆒
U+4192.0	fCNS11643	6-2D4E
U+4192.0	fR/S	 116.3
U+4192.0	fTotalStrokes	 8
U+4192.0	fCangjie	JCKNI
U+4192.0	fFourCorner	30510
U+4192.0	fCantonese	gau3
U+4192.0	fMandarin	jiu4
U+4192.0	fEnglish	corrupted form of &#x7A76;
U+4193.0	fUTF8	䆓
U+4193.0	fCNS11643	3-2E46
U+4193.0	fR/S	 116.4
U+4193.0	fTotalStrokes	 9
U+4193.0	fCangjie	JCIHU
U+4193.0	fFourCorner	30210
U+4193.0	fCantonese	ngaai6; aai6 (also); zyun5
U+4193.0	fMandarin	ruan3; e4
U+4193.0	fEnglish	[1] hole; cave; pit [2] dig the ground and build underground living quarters [3] make smooth of the leather
U+4194.0	fUTF8	䆔
U+4194.0	fCNS11643	3-2E45
U+4194.0	fR/S	 116.4
U+4194.0	fTotalStrokes	 9
U+4194.0	fCangjie	JCL
U+4194.0	fFourCorner	30500
U+4194.0	fCantonese	cung1; lung1
U+4194.0	fMandarin	chong1
U+4194.0	fEnglish	[1] pierce through; penetrate through; bore through; thread [2] cross
U+4195.0	fUTF8	䆕
U+4195.0	fCNS11643	4-2A71
U+4195.0	fGB	3-5F57
U+4195.0	fR/S	 116.4; 116.6; 40.6
U+4195.0	fTotalStrokes	 9
U+4195.0	fCangjie	JCDK
U+4195.0	fFourCorner	30530
U+4195.0	fCantonese	jyut3; kyut3
U+4195.0	fMandarin	jue2
U+4195.0	fEnglish	[1] pierce through; penetrate through; cross [2] extremely; to the extreme [3] hole; opening; aperture [4] with no door or window
U+4196.0	fUTF8	䆖
U+4196.0	fCNS11643	4-2A72
U+4196.0	fGB	5-5E36
U+4196.0	fR/S	 116.4
U+4196.0	fTotalStrokes	 9
U+4196.0	fCangjie	JCKI
U+4196.0	fFourCorner	30430
U+4196.0	fHakka	  fen2
U+4196.0	fCantonese	wang4
U+4196.0	fMandarin	hong2
U+4196.0	fEnglish	[1] a vast hall [2] a mansion
U+4197.0	fUTF8	䆗
U+4197.0	fCNS11643	3-3321
U+4197.0	fGB	5-5E3E
U+4197.0	fR/S	 116.5
U+4197.0	fTotalStrokes	 10
U+4197.0	fCangjie	JCRVL
U+4197.0	fFourCorner	30620
U+4197.0	fCantonese	jiu2; giu3
U+4197.0	fMandarin	jiao4; yao3
U+4197.0	fEnglish	[1] deep and dark; profound [2] same as &#x7A88;
U+4198.0	fUTF8	䆘
U+4198.0	fCNS11643	3-327E
U+4198.0	fGB	5-5E3D
U+4198.0	fR/S	 116.5
U+4198.0	fTotalStrokes	 10
U+4198.0	fCangjie	JCWL
U+4198.0	fFourCorner	30506
U+4198.0	fCantonese	aap3; ngaap3 (also); at3; cyun4
U+4198.0	fMandarin	ya1; cuan2
U+4198.0	fEnglish	[1] accupuncture [2] narrow and protrude [3] non-classical form of &#x62F6;
U+4199.0	fUTF8	䆙
U+4199.0	fCNS11643	3-327D
U+4199.0	fR/S	 116.5
U+4199.0	fTotalStrokes	 10
U+4199.0	fCangjie	JCMYM
U+4199.0	fFourCorner	30100
U+4199.0	fCantonese	jiu4; ziu4
U+4199.0	fMandarin	yao2
U+4199.0	fEnglish	(non-classical form) same as standard form of &#x7AB0;, &#x7AAF;
U+419A.0	fUTF8	䆚
U+419A.0	fCNS11643	4-336F
U+419A.0	fGB	5-5E49
U+419A.0	fR/S	 116.6
U+419A.0	fTotalStrokes	 11
U+419A.0	fCangjie	JCBMR
U+419A.0	fFourCorner	30220
U+419A.0	fCantonese	tung4; dung6
U+419A.0	fMandarin	tong2; dong4
U+419A.0	fEnglish	[1] go through; penetrate; passed through [2] same as &#x6D1E;
U+419B.0	fUTF8	䆛
U+419B.0	fCNS11643	4-3370
U+419B.0	fGB	3-5F5B
U+419B.0	fR/S	 116.6
U+419B.0	fTotalStrokes	 11
U+419B.0	fCangjie	JCRHP
U+419B.0	fFourCorner	30610
U+419B.0	fCantonese	zaa3
U+419B.0	fMandarin	zha4; cha2; zhe2
U+419B.0	fEnglish	something in the cave
U+419C.0	fUTF8	䆜
U+419C.0	fCNS11643	3-376F
U+419C.0	fR/S	 116.6
U+419C.0	fTotalStrokes	 11
U+419C.0	fCangjie	JCKB
U+419C.0	fFourCorner	30420
U+419C.0	fCantonese	jau6; zaau6
U+419C.0	fMandarin	you4
U+419C.0	fEnglish	[1] empty; hollow [2] unreal [3] high and vast [4] the space
U+419D.0	fUTF8	䆝
U+419D.0	fCNS11643	5-3445
U+419D.0	fR/S	 116.6
U+419D.0	fTotalStrokes	 11
U+419D.0	fCangjie	JCHBT
U+419D.0	fFourCorner	30100
U+419D.0	fCantonese	hyut3; seot6
U+419D.0	fMandarin	zhu1; zhu2; shu4; xue4
U+419D.0	fEnglish	[1] cave; hole [2] bore through or drill a cave as a dwelling
U+419E.0	fUTF8	䆞
U+419E.0	fCNS11643	4-3372
U+419E.0	fGB	5-5E48
U+419E.0	fR/S	 116.6
U+419E.0	fTotalStrokes	 11
U+419E.0	fCangjie	JCAP
U+419E.0	fFourCorner	30710
U+419E.0	fCantonese	jiu2; miu5
U+419E.0	fMandarin	yao3; yao4
U+419E.0	fEnglish	[1] deep and dark; profound [2] far; vast [3] obscure; mysterious
U+419F.0	fUTF8	䆟
U+419F.0	fCNS11643	4-3371
U+419F.0	fGB	3-5F5C
U+419F.0	fR/S	 116.6
U+419F.0	fTotalStrokes	 11
U+419F.0	fCangjie	JCOMR
U+419F.0	fFourCorner	30600
U+419F.0	fCantonese	hap1; gaap3
U+419F.0	fMandarin	ke4; ge2
U+419F.0	fEnglish	[1] coresponding; equivalent [2] considerable [3] appropriate [4] same as &#x51F9;
U+41A0.0	fUTF8	䆠
U+41A0.0	fCNS11643	6-426F
U+41A0.0	fR/S	 116.6
U+41A0.0	fTotalStrokes	 11
U+41A0.0	fCangjie	JCSLL
U+41A0.0	fFourCorner	30710
U+41A0.0	fCantonese	waan6
U+41A0.0	fMandarin	huan4
U+41A0.0	fEnglish	non-classical form of &#x5BA6;
U+41A1.0	fUTF8	䆡
U+41A1.0	fCNS11643	3-3D6C
U+41A1.0	fGB	5-5E50
U+41A1.0	fR/S	 116.7
U+41A1.0	fTotalStrokes	 12
U+41A1.0	fCangjie	JCIAV
U+41A1.0	fFourCorner	30730
U+41A1.0	fCantonese	laang1; long4
U+41A1.0	fMandarin	lang2; lang4
U+41A1.0	fEnglish	 cave; hole
U+41A2.0	fUTF8	䆢
U+41A2.0	fCNS11643	4-3967
U+41A2.0	fGB	3-5F5F
U+41A2.0	fR/S	 116.7
U+41A2.0	fTotalStrokes	 12
U+41A2.0	fCangjie	JCQDK
U+41A2.0	fFourCorner	30530
U+41A2.0	fCantonese	jyut3; kyut3
U+41A2.0	fMandarin	jue2
U+41A2.0	fEnglish	[1] penetrate; pierce through [2] hole; aperture; opening [3] cave
U+41A3.0	fUTF8	䆣
U+41A3.0	fCNS11643	3-3D6D
U+41A3.0	fR/S	 116.7
U+41A3.0	fTotalStrokes	 12
U+41A3.0	fCangjie	JCMMV
U+41A3.0	fFourCorner	30230
U+41A3.0	fCantonese	san4
U+41A3.0	fMandarin	chen2
U+41A3.0	fEnglish	same as &#x5BB8;
U+41A4.0	fUTF8	䆤
U+41A4.0	fCNS11643	6-4C53
U+41A4.0	fR/S	 116.7
U+41A4.0	fTotalStrokes	 12
U+41A4.0	fCangjie	JCHXH
U+41A4.0	fFourCorner	30400
U+41A4.0	fCantonese	cyun1
U+41A4.0	fMandarin	chuan1
U+41A4.0	fEnglish	same as &#x7A7F;
U+41A5.0	fUTF8	䆥
U+41A5.0	fCNS11643	3-3D6E
U+41A5.0	fR/S	 116.7
U+41A5.0	fTotalStrokes	 12
U+41A5.0	fCangjie	JCMSO
U+41A5.0	fFourCorner	30230
U+41A5.0	fCantonese	cyun1
U+41A5.0	fMandarin	chuan1
U+41A5.0	fEnglish	[1] same as &#x41A4; [2] non-classical form of &#x7A7F;
U+41A6.0	fUTF8	䆦
U+41A6.0	fCNS11643	5-4139
U+41A6.0	fR/S	 116.8
U+41A6.0	fTotalStrokes	 13
U+41A6.0	fCangjie	JCFF
U+41A6.0	fFourCorner	30800
U+41A6.0	fCantonese	sam1
U+41A6.0	fMandarin	tan3; shen4
U+41A6.0	fEnglish	corrupt form of &#x6DF1;
U+41A7.0	fUTF8	䆧
U+41A7.0	fCNS11643	4-3F53
U+41A7.0	fGB	5-5E51
U+41A7.0	fR/S	 116.8
U+41A7.0	fTotalStrokes	 13
U+41A7.0	fCangjie	JCHDV
U+41A7.0	fFourCorner	30400
U+41A7.0	fCantonese	wo1; fo2
U+41A7.0	fMandarin	wo1
U+41A7.0	fEnglish	same as &#x7AA9;
U+41A8.0	fUTF8	䆨
U+41A8.0	fCNS11643	4-3F52
U+41A8.0	fGB	3-5F61
U+41A8.0	fR/S	 116.8
U+41A8.0	fTotalStrokes	 13
U+41A8.0	fCangjie	JCAHK
U+41A8.0	fFourCorner	30430
U+41A8.0	fCantonese	ning4; ling4 (also); ming4
U+41A8.0	fMandarin	ning2; ming2
U+41A8.0	fEnglish	same as &#x51A5;
U+41A9.0	fUTF8	䆩
U+41A9.0	fCNS11643	6-5627
U+41A9.0	fR/S	 116.8
U+41A9.0	fTotalStrokes	 13
U+41A9.0	fCangjie	JCBUK
U+41A9.0	fFourCorner	30430
U+41A9.0	fCantonese	ming4
U+41A9.0	fMandarin	ming2
U+41A9.0	fEnglish	same as &#x51A5;
U+41AA.0	fUTF8	䆪
U+41AA.0	fCNS11643	4-4560
U+41AA.0	fGB	5-407D
U+41AA.0	fR/S	 116.9
U+41AA.0	fTotalStrokes	 14
U+41AA.0	fCangjie	FUJCM
U+41AA.0	fFourCorner	93210
U+41AA.0	fCantonese	hung1; hung3
U+41AA.0	fMandarin	hong1
U+41AA.0	fEnglish	color of the light; brilliance or gleaming
U+41AB.0	fUTF8	䆫
U+41AB.0	fCNS11643	6-5F78
U+41AB.0	fR/S	 116.9
U+41AB.0	fTotalStrokes	 14
U+41AB.0	fCangjie	JCPKP
U+41AB.0	fFourCorner	30330
U+41AB.0	fCantonese	coeng1
U+41AB.0	fMandarin	chuang1; cong1
U+41AB.0	fEnglish	same as &#x7A97;
U+41AC.0	fUTF8	䆬
U+41AC.0	fCNS11643	4-4B6C
U+41AC.0	fGB	5-5E4E
U+41AC.0	fR/S	 116.10
U+41AC.0	fTotalStrokes	 15
U+41AC.0	fCangjie	JCRBC
U+41AC.0	fFourCorner	30800
U+41AC.0	fCantonese	wan5
U+41AC.0	fMandarin	yun3
U+41AC.0	fEnglish	[1] (non-classical form) [2] same as standard form &#x9723; [3] interchangeable with &#x9695;
U+41AD.0	fUTF8	䆭
U+41AD.0	fCNS11643	5-4F23
U+41AD.0	fR/S	 116.10
U+41AD.0	fTotalStrokes	 15
U+41AD.0	fCangjie	JCJJJ
U+41AD.0	fFourCorner	30540
U+41AD.0	fCantonese	hin1; han6
U+41AD.0	fMandarin	xuan1
U+41AD.0	fEnglish	cave; inside of a cave
U+41AE.0	fUTF8	䆮
U+41AE.0	fR/S	 116.10
U+41AE.0	fTotalStrokes	 15
U+41AE.0	fCangjie	JCESE
U+41AE.0	fCantonese	zam3; caam1; caan2; zaam3
U+41AE.0	fMandarin	jin4
U+41AE.0	fEnglish	ancient form of &#x6D78;
U+41AF.0	fUTF8	䆯
U+41AF.0	fCNS11643	5-5424
U+41AF.0	fGB	3-5F6C
U+41AF.0	fR/S	 116.11; 38.11
U+41AF.0	fTotalStrokes	 16
U+41AF.0	fCangjie	JCEEV
U+41AF.0	fFourCorner	30400
U+41AF.0	fCantonese	zeot1; zyut3
U+41AF.0	fMandarin	zhui4; chuo4
U+41AF.0	fEnglish	[1] a short shaped face [2] charming; beautiful; delicate
U+41B0.0	fUTF8	䆰
U+41B0.0	fCNS11643	4-5225
U+41B0.0	fGB	3-5F56
U+41B0.0	fR/S	 116.11
U+41B0.0	fTotalStrokes	 16
U+41B0.0	fCangjie	JCSRR
U+41B0.0	fFourCorner	30710
U+41B0.0	fHakka	  ki1;  hi1; ji1
U+41B0.0	fCantonese	jyu1; zyu1
U+41B0.0	fMandarin	yu1
U+41B0.0	fEnglish	cave; hole
U+41B1.0	fUTF8	䆱
U+41B1.0	fCNS11643	4-5222
U+41B1.0	fGB	3-5F6B
U+41B1.0	fR/S	 116.11
U+41B1.0	fTotalStrokes	 16
U+41B1.0	fCangjie	JCEFF
U+41B1.0	fFourCorner	30180
U+41B1.0	fHakka	  tam2
U+41B1.0	fCantonese	taam1; dam6
U+41B1.0	fMandarin	tan1; tan3
U+41B1.0	fEnglish	spread out; flat
U+41B2.0	fUTF8	䆲
U+41B2.0	fCNS11643	4-5224
U+41B2.0	fGB	5-5E56
U+41B2.0	fR/S	 116.11
U+41B2.0	fTotalStrokes	 16
U+41B2.0	fCangjie	JCILE
U+41B2.0	fFourCorner	30230
U+41B2.0	fCantonese	kwaang1; hong1
U+41B2.0	fMandarin	kang1
U+41B2.0	fEnglish	[1] spacious house [2] emptiness
U+41B3.0	fUTF8	䆳
U+41B3.0	fCNS11643	7-306F
U+41B3.0	fR/S	 116.11
U+41B3.0	fTotalStrokes	 16
U+41B3.0	fCangjie	JCYMO
U+41B3.0	fFourCorner	30300
U+41B3.0	fHakka	 sui3; sui5
U+41B3.0	fCantonese	seoi6; syun4
U+41B3.0	fMandarin	sui4; qiong2
U+41B3.0	fEnglish	same as &#x9083;
U+41B4.0	fUTF8	䆴
U+41B4.0	fR/S	 116.11
U+41B4.0	fTotalStrokes	 16
U+41B4.0	fCangjie	JCNWU
U+41B4.0	fCantonese	gwai1
U+41B5.0	fUTF8	䆵
U+41B5.0	fCNS11643	3-5565
U+41B5.0	fGB	5-5E5B
U+41B5.0	fR/S	 116.12
U+41B5.0	fTotalStrokes	 17
U+41B5.0	fCangjie	JCCWA
U+41B5.0	fFourCorner	30600
U+41B5.0	fCantonese	caang4; cang4; zap6; zin3; zip6
U+41B5.0	fMandarin	cheng2
U+41B5.0	fEnglish	[1] echo [2] high and deep [3] large; big; specious house
U+41B6.0	fUTF8	䆶
U+41B6.0	fCNS11643	3-5566
U+41B6.0	fR/S	 116.12
U+41B6.0	fTotalStrokes	 17
U+41B6.0	fCangjie	JCOGF
U+41B6.0	fFourCorner	30330
U+41B6.0	fCantonese	zau1; zaau1; zik1
U+41B6.0	fMandarin	jiu1
U+41B6.0	fEnglish	the noise of a mouse in the cave
U+41B7.0	fUTF8	䆷
U+41B7.0	fCNS11643	4-574C
U+41B7.0	fGB	5-5E5D
U+41B7.0	fR/S	 116.12
U+41B7.0	fTotalStrokes	 17; 16
U+41B7.0	fCangjie	JCNHB
U+41B7.0	fFourCorner	30220
U+41B7.0	fHakka	 git7; giet7
U+41B7.0	fCantonese	hyut3; waat6; zyut6
U+41B7.0	fMandarin	xue4; yu4
U+41B7.0	fEnglish	[1] empty [2] free from guile
U+41B8.0	fUTF8	䆸
U+41B8.0	fCNS11643	4-574D
U+41B8.0	fGB	5-5E5C
U+41B8.0	fR/S	 116.12
U+41B8.0	fTotalStrokes	 17; 16
U+41B8.0	fCangjie	JCNOT
U+41B8.0	fFourCorner	30100
U+41B8.0	fHakka	 den1; zen1
U+41B8.0	fCantonese	zang1; cang4
U+41B8.0	fMandarin	zheng1; cheng2
U+41B8.0	fEnglish	[1] spacious; capacious [2] sound (of the house) [3] picture (on silk) scroll
U+41B9.0	fUTF8	䆹
U+41B9.0	fCNS11643	4-574B
U+41B9.0	fGB	3-5F6F
U+41B9.0	fR/S	 116.12
U+41B9.0	fTotalStrokes	 17; 16
U+41B9.0	fCangjie	JCYTG
U+41B9.0	fFourCorner	30100
U+41B9.0	fCantonese	cung1
U+41B9.0	fMandarin	chong1; tong3
U+41B9.0	fEnglish	[1] empty; hollow [2] dark; obscure
U+41BA.0	fUTF8	䆺
U+41BA.0	fCNS11643	4-574F
U+41BA.0	fGB	5-5E5A
U+41BA.0	fR/S	 116.12
U+41BA.0	fTotalStrokes	 17; 16
U+41BA.0	fCangjie	JCHDW
U+41BA.0	fFourCorner	30600
U+41BA.0	fCantonese	pun1
U+41BA.0	fMandarin	pan1
U+41BA.0	fEnglish	[1] whirling of the flowing water [2] interchangeable &#x6F58;
U+41BB.0	fUTF8	䆻
U+41BB.0	fCNS11643	3-5567
U+41BB.0	fR/S	 116.12
U+41BB.0	fTotalStrokes	 17
U+41BB.0	fCangjie	JCMJK
U+41BB.0	fFourCorner	30440
U+41BB.0	fCantonese	kiu3
U+41BB.0	fMandarin	qiao4
U+41BB.0	fEnglish	corrupted form of &#x7AC5;
U+41BC.0	fUTF8	䆼
U+41BC.0	fCNS11643	4-5C39
U+41BC.0	fGB	5-5E47
U+41BC.0	fR/S	 116.13
U+41BC.0	fTotalStrokes	 18
U+41BC.0	fCangjie	JCYBB
U+41BC.0	fFourCorner	30300
U+41BC.0	fHakka	 ko1
U+41BC.0	fCantonese	wo1; fo1
U+41BC.0	fMandarin	ke1
U+41BC.0	fEnglish	non-classcial form of standard form &#x7AA0;
U+41BD.0	fUTF8	䆽
U+41BD.0	fCNS11643	4-5C3A
U+41BD.0	fGB	3-5F70
U+41BD.0	fR/S	 116.13
U+41BD.0	fTotalStrokes	 18
U+41BD.0	fCangjie	JCYPO
U+41BD.0	fFourCorner	30230
U+41BD.0	fCantonese	keoi4
U+41BD.0	fMandarin	qu2; ju4; qun2
U+41BD.0	fEnglish	[1] hole; opening [2] cave [3] empty; hollow
U+41BE.0	fUTF8	䆾
U+41BE.0	fCNS11643	4-6048
U+41BE.0	fGB	3-5F68
U+41BE.0	fR/S	 116.14
U+41BE.0	fTotalStrokes	 19
U+41BE.0	fCangjie	JCSIT
U+41BE.0	fFourCorner	30100
U+41BE.0	fHakka	 lam2
U+41BE.0	fCantonese	laam4
U+41BE.0	fMandarin	lan2; lan4
U+41BE.0	fEnglish	[1] big; large; thin; light [2] rugged; uneven [3] deep cave
U+41BF.0	fUTF8	䆿
U+41BF.0	fCNS11643	4-6049
U+41BF.0	fR/S	 116.14
U+41BF.0	fTotalStrokes	 19
U+41BF.0	fCangjie	JCVMD
U+41BF.0	fFourCorner	30290
U+41BF.0	fCantonese	ngai6; ai6 (also); gaam3; ngaai6
U+41BF.0	fMandarin	yi4
U+41BF.0	fEnglish	[1] same as &#x5BF1; [2] standard form of &#x56C8;
U+41C0.0	fUTF8	䇀
U+41C0.0	fCNS11643	4-637C
U+41C0.0	fGB	5-5E61
U+41C0.0	fR/S	 116.15
U+41C0.0	fTotalStrokes	 20
U+41C0.0	fCangjie	JCTJJ
U+41C0.0	fFourCorner	30540
U+41C0.0	fCantonese	jung4; zung4
U+41C0.0	fMandarin	rong2; rong3
U+41C0.0	fEnglish	(non-classical form) ornaments of fine feather or fur (for decoration)
U+41C1.0	fUTF8	䇁
U+41C1.0	fCNS11643	4-6922
U+41C1.0	fGB	3-5F71
U+41C1.0	fR/S	 116.17
U+41C1.0	fTotalStrokes	 22; 21
U+41C1.0	fCangjie	JCNFQ
U+41C1.0	fFourCorner	30350
U+41C1.0	fCantonese	si1
U+41C1.0	fMandarin	si1
U+41C1.0	fEnglish	 cave; hole
U+41C2.0	fUTF8	䇂
U+41C2.0	fCNS11643	3-2434
U+41C2.0	fR/S	 117.1
U+41C2.0	fTotalStrokes	 6
U+41C2.0	fCangjie	YTL
U+41C2.0	fFourCorner	00200
U+41C2.0	fCantonese	hin1
U+41C2.0	fMandarin	qian1
U+41C2.0	fEnglish	(ancient form) fault; sin
U+41C3.0	fUTF8	䇃
U+41C3.0	fCNS11643	3-2A46
U+41C3.0	fR/S	 117.3
U+41C3.0	fTotalStrokes	 8
U+41C3.0	fCangjie	YTRU
U+41C3.0	fFourCorner	07110
U+41C3.0	fCantonese	zi6; ci5
U+41C3.0	fMandarin	si4; xiao4
U+41C3.0	fEnglish	[1] same as &#x7AE2; [2] ancient form of &#x4FDF;
U+41C4.0	fUTF8	䇄
U+41C4.0	fGB	S-237B
U+41C4.0	fR/S	 117.3
U+41C4.0	fTotalStrokes	 8
U+41C4.0	fCangjie	YTON
U+41C4.0	fFourCorner	0811
U+41C4.0	fCantonese	hat1
U+41C5.0	fUTF8	䇅
U+41C5.0	fCNS11643	4-2A76
U+41C5.0	fGB	3-5F4A
U+41C5.0	fR/S	 117.4
U+41C5.0	fTotalStrokes	 9
U+41C5.0	fCangjie	YTI
U+41C5.0	fFourCorner	03150
U+41C5.0	fCantonese	fat6
U+41C5.0	fMandarin	fa1; fa2
U+41C5.0	fEnglish	 stand still
U+41C6.0	fUTF8	䇆
U+41C6.0	fCNS11643	3-2E4A
U+41C6.0	fR/S	 117.4
U+41C6.0	fTotalStrokes	 9
U+41C6.0	fCangjie	YTYJ
U+41C6.0	fFourCorner	04100
U+41C6.0	fCantonese	sap6
U+41C6.0	fEnglish	capacity unit, decalitre
U+41C7.0	fUTF8	䇇
U+41C7.0	fCNS11643	3-3325
U+41C7.0	fGB	S-237A
U+41C7.0	fR/S	 117.5
U+41C7.0	fTotalStrokes	 10
U+41C7.0	fCangjie	YTRVP
U+41C7.0	fFourCorner	07140
U+41C7.0	fCantonese	maang4
U+41C7.0	fMandarin	meng2
U+41C7.0	fEnglish	corrupted form of &#x6C13;
U+41C8.0	fUTF8	䇈
U+41C8.0	fCNS11643	4-2E6D
U+41C8.0	fGB	3-5F4E
U+41C8.0	fR/S	 117.5
U+41C8.0	fTotalStrokes	 10
U+41C8.0	fCangjie	YTEG
U+41C8.0	fFourCorner	07110
U+41C8.0	fCantonese	waa6; wak6
U+41C8.0	fMandarin	hua4; gui4
U+41C8.0	fEnglish	[1] put into a cupboard or closet; put into the proper place; put away [2] put away for safekeeping [3] collect and keep
U+41C9.0	fUTF8	䇉
U+41C9.0	fCNS11643	3-3322
U+41C9.0	fR/S	 117.5
U+41C9.0	fTotalStrokes	 10
U+41C9.0	fCangjie	YTMR
U+41C9.0	fFourCorner	01160
U+41C9.0	fCantonese	baak3
U+41C9.0	fEnglish	capacity unit , hectolitre
U+41CA.0	fUTF8	䇊
U+41CA.0	fR/S	 117.5
U+41CA.0	fTotalStrokes	 10
U+41CA.0	fCangjie	YTII
U+41CA.0	fCantonese	gwong2; gong2 (also)
U+41CB.0	fUTF8	䇋
U+41CB.0	fCNS11643	5-3446
U+41CB.0	fR/S	 117.6
U+41CB.0	fTotalStrokes	 11
U+41CB.0	fCangjie	YTNIN
U+41CB.0	fFourCorner	07120
U+41CB.0	fCantonese	hoi6; waai6
U+41CB.0	fMandarin	hai4; ran3; xie4
U+41CB.0	fEnglish	the name of a god
U+41CC.0	fUTF8	䇌
U+41CC.0	fCNS11643	3-3D6F
U+41CC.0	fR/S	 117.7
U+41CC.0	fTotalStrokes	 12
U+41CC.0	fCangjie	YTFB
U+41CC.0	fFourCorner	09120
U+41CC.0	fCantonese	ciu3
U+41CC.0	fMandarin	qiao4
U+41CC.0	fEnglish	[1] stand [2] erect; create; start
U+41CD.0	fUTF8	䇍
U+41CD.0	fCNS11643	4-396A
U+41CD.0	fGB	3-5F4F
U+41CD.0	fR/S	 117.7
U+41CD.0	fTotalStrokes	 12
U+41CD.0	fCangjie	YTRYO
U+41CD.0	fFourCorner	06180
U+41CD.0	fCantonese	cuk1
U+41CD.0	fMandarin	chu4; chuo4
U+41CD.0	fEnglish	[1] wait [2] same as &#x9F6A;
U+41CE.0	fUTF8	䇎
U+41CE.0	fCNS11643	3-4348
U+41CE.0	fGB	5-5E28
U+41CE.0	fR/S	 117.8
U+41CE.0	fTotalStrokes	 13
U+41CE.0	fCangjie	YTTA
U+41CE.0	fFourCorner	04160
U+41CE.0	fCantonese	coek3
U+41CE.0	fMandarin	que4; qiao3; qi4
U+41CE.0	fEnglish	[1] startle; amaze; surprise [2] afraid; scared; fearful [3] respectful; reverent
U+41CF.0	fUTF8	䇏
U+41CF.0	fCNS11643	4-3F54
U+41CF.0	fGB	3-5F50
U+41CF.0	fR/S	 117.8
U+41CF.0	fTotalStrokes	 13
U+41CF.0	fCangjie	YTYRD
U+41CF.0	fFourCorner	00140
U+41CF.0	fCantonese	deoi2; deoi3
U+41CF.0	fMandarin	dui3; dui4
U+41CF.0	fEnglish	[1] pile one upon another [2] trees fruits well and the branches hanging down
U+41D0.0	fUTF8	䇐
U+41D0.0	fCNS11643	4-3F55
U+41D0.0	fGB	3-5F51
U+41D0.0	fR/S	 117.8
U+41D0.0	fTotalStrokes	 13
U+41D0.0	fCangjie	YTLE
U+41D0.0	fFourCorner	05130
U+41D0.0	fCantonese	lei6
U+41D0.0	fMandarin	li4; lei4
U+41D0.0	fEnglish	same as &#x642F;, &#x849E;
U+41D1.0	fUTF8	䇑
U+41D1.0	fCNS11643	4-3F5A
U+41D1.0	fGB	5-5E2A
U+41D1.0	fR/S	 117.8
U+41D1.0	fTotalStrokes	 13
U+41D1.0	fCangjie	YTHWJ
U+41D1.0	fFourCorner	06140
U+41D1.0	fCantonese	baa6
U+41D1.0	fMandarin	ba4; bai3; pi1
U+41D1.0	fEnglish	[1] a standing short person [2] unstable in walking
U+41D2.0	fUTF8	䇒
U+41D2.0	fCNS11643	4-4563
U+41D2.0	fGB	5-5E2D
U+41D2.0	fR/S	 117.11; 117.9
U+41D2.0	fTotalStrokes	 14; 16
U+41D2.0	fCangjie	AUYT
U+41D2.0	fFourCorner	71100
U+41D2.0	fCantonese	gaai3
U+41D2.0	fMandarin	jie4; qin2; xian4
U+41D2.0	fEnglish	ancient form of &#x5C46;
U+41D3.0	fUTF8	䇓
U+41D3.0	fCNS11643	4-5752
U+41D3.0	fGB	3-5F52
U+41D3.0	fR/S	 117.12
U+41D3.0	fTotalStrokes	 17; 16
U+41D3.0	fCangjie	HCYT
U+41D3.0	fFourCorner	21100
U+41D3.0	fCantonese	seoi1
U+41D3.0	fMandarin	xu1
U+41D3.0	fEnglish	[1] wait for; wait [2] same as &#x9808; [3] interchangeable &#x9700;
U+41D4.0	fUTF8	䇔
U+41D4.0	fCNS11643	4-5C3B
U+41D4.0	fGB	3-5F54
U+41D4.0	fR/S	 117.13
U+41D4.0	fTotalStrokes	 18
U+41D4.0	fCangjie	YNYT
U+41D4.0	fFourCorner	00210
U+41D4.0	fCantonese	lo6; leoi6
U+41D4.0	fMandarin	luo4; nuo4
U+41D4.0	fEnglish	[1] atrophy [2] paralysis; impotent [3] stand
U+41D5.0	fUTF8	䇕
U+41D5.0	fCNS11643	3-5A68
U+41D5.0	fR/S	 117.14
U+41D5.0	fTotalStrokes	 19
U+41D5.0	fCangjie	YTMBB
U+41D5.0	fFourCorner	01120
U+41D5.0	fCantonese	seoi1; deoi3; zyu3
U+41D5.0	fMandarin	xu1
U+41D5.0	fEnglish	same as &#x41D3;
U+41D6.0	fUTF8	䇖
U+41D6.0	fCNS11643	5-2B21
U+41D6.0	fR/S	 118.3
U+41D6.0	fTotalStrokes	 9
U+41D6.0	fCangjie	HPH
U+41D6.0	fFourCorner	88220
U+41D6.0	fCantonese	wan5
U+41D6.0	fMandarin	yun3
U+41D6.0	fEnglish	rope made of bamboo strips
U+41D7.0	fUTF8	䇗
U+41D7.0	fCNS11643	4-2E75
U+41D7.0	fGB	3-634E
U+41D7.0	fR/S	 118.4
U+41D7.0	fTotalStrokes	 10
U+41D7.0	fCangjie	HCI
U+41D7.0	fFourCorner	88730
U+41D7.0	fCantonese	zung1
U+41D7.0	fMandarin	zhong1
U+41D7.0	fEnglish	bamboo with long joints
U+41D8.0	fUTF8	䇘
U+41D8.0	fCNS11643	4-2E70
U+41D8.0	fGB	5-6276
U+41D8.0	fR/S	 118.4
U+41D8.0	fTotalStrokes	 10
U+41D8.0	fCangjie	HMVM
U+41D8.0	fFourCorner	88100
U+41D8.0	fHakka	 fu5; fu6
U+41D8.0	fCantonese	wu6
U+41D8.0	fMandarin	hu4
U+41D8.0	fEnglish	 reel to wind silk
U+41D9.0	fUTF8	䇙
U+41D9.0	fCNS11643	3-3328
U+41D9.0	fR/S	 118.4
U+41D9.0	fTotalStrokes	 10
U+41D9.0	fCangjie	HNL
U+41D9.0	fFourCorner	88220
U+41D9.0	fCantonese	jan5; zaan5
U+41D9.0	fMandarin	yin3
U+41D9.0	fEnglish	the name of a variety of bamboo
U+41DA.0	fUTF8	䇚
U+41DA.0	fCNS11643	4-2E77
U+41DA.0	fGB	5-6278
U+41DA.0	fR/S	 118.4
U+41DA.0	fTotalStrokes	 10
U+41DA.0	fCangjie	HOY
U+41DA.0	fFourCorner	88230
U+41DA.0	fCantonese	pok3
U+41DA.0	fMandarin	pu2
U+41DA.0	fEnglish	[1] same as &#x7BDE; [2] interchangeable with &#x64B2;
U+41DB.0	fUTF8	䇛
U+41DB.0	fCNS11643	3-3329
U+41DB.0	fR/S	 118.4
U+41DB.0	fTotalStrokes	 10
U+41DB.0	fCangjie	HYLM
U+41DB.0	fFourCorner	88100
U+41DB.0	fCantonese	zi2; zi3
U+41DB.0	fMandarin	zhi3; zhi4
U+41DB.0	fEnglish	the name of a variety of bamboo
U+41DC.0	fUTF8	䇜
U+41DC.0	fCNS11643	4-2E73
U+41DC.0	fGB	3-634F
U+41DC.0	fR/S	 118.4
U+41DC.0	fTotalStrokes	 10
U+41DC.0	fCangjie	HNO
U+41DC.0	fFourCorner	88800
U+41DC.0	fCantonese	him2; lim5
U+41DC.0	fMandarin	qian3; lian3; qian4
U+41DC.0	fEnglish	[1] small bamboo [2] a kind of bamboo
U+41DD.0	fUTF8	䇝
U+41DD.0	fCNS11643	4-2E72
U+41DD.0	fGB	3-634A
U+41DD.0	fR/S	 118.4
U+41DD.0	fTotalStrokes	 10
U+41DD.0	fCangjie	HI
U+41DD.0	fFourCorner	88500
U+41DD.0	fCantonese	zim1
U+41DD.0	fMandarin	jian1
U+41DD.0	fEnglish	(abbreviated form) same as &#x39B0;
U+41DE.0	fUTF8	䇞
U+41DE.0	fCNS11643	4-3376
U+41DE.0	fGB	3-6355
U+41DE.0	fR/S	 118.5
U+41DE.0	fTotalStrokes	 11
U+41DE.0	fCangjie	HTM
U+41DE.0	fFourCorner	88770
U+41DE.0	fCantonese	gam1
U+41DE.0	fMandarin	gan1; gan3
U+41DE.0	fEnglish	[1] the name of a variety of bamboo; sweet bamboo [2] large bamboo
U+41DF.0	fUTF8	䇟
U+41DF.0	fCNS11643	4-337D
U+41DF.0	fGB	5-6327
U+41DF.0	fR/S	 118.5
U+41DF.0	fTotalStrokes	 11
U+41DF.0	fCangjie	HNIS
U+41DF.0	fFourCorner	88220
U+41DF.0	fCantonese	gin3
U+41DF.0	fMandarin	jian4
U+41DF.0	fEnglish	same as &#x8171;
U+41E0.0	fUTF8	䇠
U+41E0.0	fCNS11643	3-3774
U+41E0.0	fR/S	 118.5
U+41E0.0	fTotalStrokes	 11
U+41E0.0	fCangjie	HYG
U+41E0.0	fFourCorner	88100
U+41E0.0	fHakka	 chu3; cu3
U+41E0.0	fCantonese	cyu5
U+41E0.0	fMandarin	zhu4; zhou4
U+41E0.0	fEnglish	the pegs on which the strings of a lute are fastened
U+41E1.0	fUTF8	䇡
U+41E1.0	fCNS11643	5-3448
U+41E1.0	fR/S	 118.5
U+41E1.0	fTotalStrokes	 11
U+41E1.0	fCangjie	HJMN
U+41E1.0	fFourCorner	88200
U+41E1.0	fCantonese	cyu5
U+41E1.0	fMandarin	zhu3; zhu4
U+41E1.0	fEnglish	same as &#x677C;
U+41E2.0	fUTF8	䇢
U+41E2.0	fCNS11643	4-3377
U+41E2.0	fGB	3-6356
U+41E2.0	fR/S	 118.5
U+41E2.0	fTotalStrokes	 11
U+41E2.0	fCangjie	HJR
U+41E2.0	fFourCorner	88600
U+41E2.0	fHakka	  ku3
U+41E2.0	fCantonese	fu2
U+41E2.0	fMandarin	ku3
U+41E2.0	fEnglish	[1] trap for catching fish [2] a kind of bamboo
U+41E3.0	fUTF8	䇣
U+41E3.0	fCNS11643	4-337B
U+41E3.0	fGB	5-6324
U+41E3.0	fR/S	 118.5
U+41E3.0	fTotalStrokes	 11
U+41E3.0	fCangjie	HOF
U+41E3.0	fFourCorner	88900
U+41E3.0	fCantonese	nip6; lip6 (also)
U+41E3.0	fMandarin	na4; nei4; yi3
U+41E3.0	fEnglish	[1] bamboo with white bark [2] same as &#x7C4B;
U+41E4.0	fUTF8	䇤
U+41E4.0	fCNS11643	3-3775
U+41E4.0	fR/S	 118.5
U+41E4.0	fTotalStrokes	 11
U+41E4.0	fCangjie	HMOB
U+41E4.0	fFourCorner	88220
U+41E4.0	fCantonese	jeoi6; zeoi6
U+41E4.0	fMandarin	rui4; dui4; su4
U+41E4.0	fEnglish	[1] sharp; acute [2] clever [3] vigorous
U+41E5.0	fUTF8	䇥
U+41E5.0	fCNS11643	6-4276
U+41E5.0	fGB	7-2368
U+41E5.0	fR/S	 118.5
U+41E5.0	fTotalStrokes	 11
U+41E5.0	fCangjie	HMYM
U+41E5.0	fFourCorner	88100
U+41E5.0	fCantonese	zaak3
U+41E5.0	fMandarin	ze2; zuo2
U+41E5.0	fEnglish	[1] same as &#x7B2E; [2] same as &#x7B70; [3] interchangeable with &#x947F;
U+41E6.0	fUTF8	䇦
U+41E6.0	fCNS11643	4-3378
U+41E6.0	fGB	3-635B
U+41E6.0	fR/S	 118.5
U+41E6.0	fTotalStrokes	 11
U+41E6.0	fCangjie	HLBK
U+41E6.0	fFourCorner	88530
U+41E6.0	fCantonese	joeng2; zoeng2
U+41E6.0	fMandarin	ang3; yang3
U+41E6.0	fEnglish	[1] a kind of bamboo [2] colorless bamboo [3] bamboo shoot
U+41E7.0	fUTF8	䇧
U+41E7.0	fCNS11643	4-396D
U+41E7.0	fGB	3-6363
U+41E7.0	fR/S	 118.6
U+41E7.0	fTotalStrokes	 12
U+41E7.0	fCangjie	HMOW
U+41E7.0	fFourCorner	88600
U+41E7.0	fCantonese	zai6; zaai3; zaai6
U+41E7.0	fMandarin	zhi4; zhu4
U+41E7.0	fEnglish	[1] fill an opening with bamboo [2] ancient form of &#x7B95;
U+41E8.0	fUTF8	䇨
U+41E8.0	fCNS11643	4-3974
U+41E8.0	fGB	3-6371
U+41E8.0	fR/S	 118.6
U+41E8.0	fTotalStrokes	 12
U+41E8.0	fCangjie	HHEQ
U+41E8.0	fFourCorner	88500
U+41E8.0	fCantonese	gung3
U+41E8.0	fMandarin	gong4; xiang2
U+41E8.0	fEnglish	[1] a bamboo basket for food containers (such as cup, plate, dish, and bowl, etc.) used in ancient times [2] a basket for chopsticks [3] a bamboo basket used to filter or to strain out the wine [4] a sail made of a thin and long strip of bamboo
U+41E9.0	fUTF8	䇩
U+41E9.0	fCNS11643	4-3971
U+41E9.0	fGB	3-6366
U+41E9.0	fR/S	 118.6
U+41E9.0	fTotalStrokes	 12
U+41E9.0	fCangjie	HLWP
U+41E9.0	fFourCorner	88500
U+41E9.0	fCantonese	jai6; zaai6
U+41E9.0	fMandarin	yi4
U+41E9.0	fEnglish	[1] long [2] big raft [3] a kind of equipment made of bamboo used to catch fishes
U+41EA.0	fUTF8	䇪
U+41EA.0	fCNS11643	3-3D74
U+41EA.0	fR/S	 118.6
U+41EA.0	fTotalStrokes	 12
U+41EA.0	fCangjie	HMIG
U+41EA.0	fFourCorner	88100
U+41EA.0	fCantonese	ci1
U+41EA.0	fMandarin	chi1
U+41EA.0	fEnglish	the name of a variety of bamboo
U+41EB.0	fUTF8	䇫
U+41EB.0	fCNS11643	4-3F5B
U+41EB.0	fGB	3-637D
U+41EB.0	fR/S	 118.6; 118.7
U+41EB.0	fTotalStrokes	 13; 12
U+41EB.0	fCangjie	HSLL
U+41EB.0	fFourCorner	88710
U+41EB.0	fCantonese	gei1
U+41EB.0	fMandarin	ji1
U+41EB.0	fEnglish	[1] bamboo-splints; laths [2] plait [3] same as &#x7BE6;
U+41EC.0	fUTF8	䇬
U+41EC.0	fCNS11643	4-3972
U+41EC.0	fGB	3-636A
U+41EC.0	fR/S	 118.6
U+41EC.0	fTotalStrokes	 12
U+41EC.0	fCangjie	HHJD
U+41EC.0	fFourCorner	88900
U+41EC.0	fCantonese	zyu1; zi1
U+41EC.0	fMandarin	zhu1; chuang3
U+41EC.0	fEnglish	[1] thin bamboo laths knitted sail [2] a thin and long strip of bamboo; books in ancient style [3] set sail; depart
U+41ED.0	fUTF8	䇭
U+41ED.0	fCNS11643	3-3D7A
U+41ED.0	fGB	5-632D
U+41ED.0	fR/S	 118.6
U+41ED.0	fTotalStrokes	 12
U+41ED.0	fCangjie	HJKP
U+41ED.0	fFourCorner	88710
U+41ED.0	fHakka	  lau3; lo3
U+41ED.0	fCantonese	lou5; liu4
U+41ED.0	fMandarin	lao3
U+41ED.0	fEnglish	containers made of thin and long strip of bamboo or willow branches
U+41EE.0	fUTF8	䇮
U+41EE.0	fCNS11643	4-3975
U+41EE.0	fGB	5-6334
U+41EE.0	fR/S	 118.6
U+41EE.0	fTotalStrokes	 12
U+41EE.0	fCangjie	HOHG
U+41EE.0	fFourCorner	88210
U+41EE.0	fCantonese	jam6; zaam6
U+41EE.0	fMandarin	ren3; ren4
U+41EE.0	fEnglish	[1] a bamboo mat used in ancient times [2] interchangeable with &#x88B5;, &#x887D;
U+41EF.0	fUTF8	䇯
U+41EF.0	fCNS11643	3-3D73
U+41EF.0	fGB	5-632E
U+41EF.0	fR/S	 118.6
U+41EF.0	fTotalStrokes	 12
U+41EF.0	fCangjie	HSJ
U+41EF.0	fFourCorner	88400
U+41EF.0	fCantonese	jung4; zung2; zung4
U+41EF.0	fMandarin	rong2
U+41EF.0	fEnglish	a kind of bamboo with patterns on the head
U+41F0.0	fUTF8	䇰
U+41F0.0	fCNS11643	4-396F
U+41F0.0	fGB	3-6379
U+41F0.0	fR/S	 118.6
U+41F0.0	fTotalStrokes	 12
U+41F0.0	fCangjie	HNEM
U+41F0.0	fFourCorner	88100
U+41F0.0	fCantonese	zing1
U+41F0.0	fMandarin	zheng1
U+41F0.0	fEnglish	[1] a bamboo binded torch [2] a kind of bamboo with patterns on the bark
U+41F1.0	fUTF8	䇱
U+41F1.0	fCNS11643	6-4C5B
U+41F1.0	fR/S	 118.6
U+41F1.0	fTotalStrokes	 12
U+41F1.0	fCangjie	HWNI
U+41F1.0	fFourCorner	88600
U+41F1.0	fCantonese	naap6; laap6 (also); nap6
U+41F1.0	fMandarin	na4
U+41F1.0	fEnglish	same as &#x7B1D;
U+41F2.0	fUTF8	䇲
U+41F2.0	fCNS11643	6-4C56
U+41F2.0	fGB	S-233A
U+41F2.0	fR/S	 118.6
U+41F2.0	fTotalStrokes	 12
U+41F2.0	fCangjie	HKT
U+41F2.0	fFourCorner	88530
U+41F2.0	fCantonese	caak3; gaap3; gap3; gok3
U+41F2.0	fMandarin	ce4; jia1
U+41F2.0	fEnglish	[1] simplified form of &#x7B74; [2] same as &#x7B56;
U+41F3.0	fUTF8	䇳
U+41F3.0	fR/S	 118.6
U+41F3.0	fTotalStrokes	 12
U+41F3.0	fCangjie	HIQ
U+41F3.0	fCantonese	zin1
U+41F3.0	fEnglish	abbreviated form of &#x7B8B;
U+41F4.0	fUTF8	䇴
U+41F4.0	fGB	7-236A
U+41F4.0	fR/S	 118.6
U+41F4.0	fTotalStrokes	 12
U+41F4.0	fCangjie	HMCW
U+41F4.0	fCantonese	sai1
U+41F5.0	fUTF8	䇵
U+41F5.0	fCNS11643	3-4350
U+41F5.0	fR/S	 118.7
U+41F5.0	fTotalStrokes	 13
U+41F5.0	fCangjie	HEHL
U+41F5.0	fFourCorner	88120
U+41F5.0	fCantonese	ngan4; an4 (also); ngaan4; zi4
U+41F5.0	fMandarin	yi2
U+41F5.0	fEnglish	one of the wind instruments; a bamboo flute with seven holes
U+41F6.0	fUTF8	䇶
U+41F6.0	fCNS11643	4-3F66
U+41F6.0	fGB	3-6428
U+41F6.0	fR/S	 118.7
U+41F6.0	fTotalStrokes	 13
U+41F6.0	fCangjie	HNBG
U+41F6.0	fFourCorner	88220
U+41F6.0	fCantonese	gok3
U+41F6.0	fMandarin	jue2
U+41F6.0	fEnglish	[1] a small rafters made of bamboo which project form the eaves and support the tiles [2] a kind of bamboo
U+41F7.0	fUTF8	䇷
U+41F7.0	fCNS11643	4-3F62
U+41F7.0	fGB	5-633E
U+41F7.0	fR/S	 118.7
U+41F7.0	fTotalStrokes	 13
U+41F7.0	fCangjie	HRSN
U+41F7.0	fFourCorner	88220
U+41F7.0	fCantonese	bit6; baat1; but6
U+41F7.0	fMandarin	bie2; bi3
U+41F7.0	fEnglish	separated; (to rip of, to cut open) the contract or agreement on slips of bamboo and give to both parties as evidence (in ancient times)
U+41F8.0	fUTF8	䇸
U+41F8.0	fCNS11643	4-3F61
U+41F8.0	fGB	3-6423
U+41F8.0	fR/S	 118.7
U+41F8.0	fTotalStrokes	 13
U+41F8.0	fCangjie	HRHG
U+41F8.0	fFourCorner	88100
U+41F8.0	fCantonese	cing4; ting1
U+41F8.0	fMandarin	cheng2; sheng4; zeng4
U+41F8.0	fEnglish	[1] bamboo mat [2] a kind of bamboo [3] bamboo ware [4] seat cushion in a carriage
U+41F9.0	fUTF8	䇹
U+41F9.0	fCNS11643	3-4352
U+41F9.0	fGB	5-6348
U+41F9.0	fR/S	 118.7
U+41F9.0	fTotalStrokes	 13
U+41F9.0	fCangjie	HSKR
U+41F9.0	fFourCorner	88600
U+41F9.0	fCantonese	kwaan2
U+41F9.0	fMandarin	qun1; jun4
U+41F9.0	fEnglish	same as &#x7B98;
U+41FA.0	fUTF8	䇺
U+41FA.0	fCNS11643	3-4353
U+41FA.0	fR/S	 118.7
U+41FA.0	fTotalStrokes	 13
U+41FA.0	fCangjie	HMRT
U+41FA.0	fFourCorner	88100
U+41FA.0	fHakka	 teu5; teu6
U+41FA.0	fCantonese	dau6; daau2; daau6
U+41FA.0	fMandarin	dou4; chou2
U+41FA.0	fEnglish	 sacrificial vessel
U+41FB.0	fUTF8	䇻
U+41FB.0	fCNS11643	4-3F5D
U+41FB.0	fGB	3-642C
U+41FB.0	fR/S	 118.7
U+41FB.0	fTotalStrokes	 13
U+41FB.0	fCangjie	HSHU
U+41FB.0	fFourCorner	88210
U+41FB.0	fCantonese	mei5
U+41FB.0	fMandarin	wei3; hui4; kui4
U+41FB.0	fEnglish	[1] a kind of bamboo [2] broom; besom
U+41FC.0	fUTF8	䇼
U+41FC.0	fCNS11643	3-434E
U+41FC.0	fR/S	 118.7
U+41FC.0	fTotalStrokes	 13
U+41FC.0	fCangjie	HRAU
U+41FC.0	fFourCorner	88710
U+41FC.0	fCantonese	jap1; ci3; zaap1
U+41FC.0	fMandarin	yi4; chi4
U+41FC.0	fEnglish	a bamboo vessel used to catch fishes
U+41FD.0	fUTF8	䇽
U+41FD.0	fCNS11643	4-3F60
U+41FD.0	fGB	3-6421
U+41FD.0	fR/S	 118.7
U+41FD.0	fTotalStrokes	 13
U+41FD.0	fCangjie	HQHL
U+41FD.0	fFourCorner	88520
U+41FD.0	fCantonese	zai3; zit3 (also); zaai3
U+41FD.0	fMandarin	zhe4; zhi4
U+41FD.0	fEnglish	a coarse bamboo mat
U+41FE.0	fUTF8	䇾
U+41FE.0	fCNS11643	3-434B
U+41FE.0	fGB	5-6344
U+41FE.0	fR/S	 118.7
U+41FE.0	fTotalStrokes	 13
U+41FE.0	fCangjie	HYMR
U+41FE.0	fFourCorner	88600
U+41FE.0	fHakka	 ngien2; ngian2
U+41FE.0	fCantonese	jin4; zin4
U+41FE.0	fMandarin	yan2
U+41FE.0	fEnglish	a large kind of flute
U+41FF.0	fUTF8	䇿
U+41FF.0	fR/S	 118.7
U+41FF.0	fTotalStrokes	 13
U+41FF.0	fCangjie	HDL
U+41FF.0	fFourCorner	8890
U+41FF.0	fCantonese	cuk1
U+41FF.0	fEnglish	[Japanese character]
U+4200.0	fUTF8	䈀
U+4200.0	fCNS11643	4-3F63
U+4200.0	fGB	3-6429
U+4200.0	fR/S	 118.7
U+4200.0	fTotalStrokes	 13
U+4200.0	fCangjie	HBTN
U+4200.0	fFourCorner	88420
U+4200.0	fCantonese	saan1
U+4200.0	fMandarin	san1
U+4200.0	fEnglish	bamboo ware (used to hold food or clothes)
U+4201.0	fUTF8	䈁
U+4201.0	fCNS11643	5-4846
U+4201.0	fR/S	 118.8
U+4201.0	fTotalStrokes	 14
U+4201.0	fCangjie	HOMB
U+4201.0	fFourCorner	88220
U+4201.0	fCantonese	leon4
U+4201.0	fMandarin	lun2; luo4
U+4201.0	fEnglish	tools used on a boat
U+4202.0	fUTF8	䈂
U+4202.0	fCNS11643	4-3976
U+4202.0	fGB	3-6375
U+4202.0	fR/S	 118.8; 118.6
U+4202.0	fTotalStrokes	 12; 14
U+4202.0	fCangjie	HTT
U+4202.0	fFourCorner	88440
U+4202.0	fCantonese	ping4; bing3; kiu5
U+4202.0	fMandarin	ji1; bing4; ping2
U+4202.0	fEnglish	a kind of bamboo
U+4203.0	fUTF8	䈃
U+4203.0	fCNS11643	4-4565
U+4203.0	fGB	3-643B
U+4203.0	fR/S	 118.8
U+4203.0	fTotalStrokes	 14
U+4203.0	fCangjie	HESR
U+4203.0	fFourCorner	88160
U+4203.0	fCantonese	ziu2
U+4203.0	fMandarin	zhao3
U+4203.0	fEnglish	[1] a kind of bamboo [2] hem; margin; edge of bamboo
U+4204.0	fUTF8	䈄
U+4204.0	fCNS11643	3-485B
U+4204.0	fR/S	 118.8
U+4204.0	fTotalStrokes	 14
U+4204.0	fCangjie	HUNE
U+4204.0	fFourCorner	88770
U+4204.0	fCantonese	ham4
U+4204.0	fMandarin	han2
U+4204.0	fEnglish	same as &#x7B68;
U+4205.0	fUTF8	䈅
U+4205.0	fCNS11643	4-456A
U+4205.0	fGB	5-634A
U+4205.0	fR/S	 118.8
U+4205.0	fTotalStrokes	 14
U+4205.0	fCangjie	HIRM
U+4205.0	fFourCorner	88150
U+4205.0	fCantonese	wik6
U+4205.0	fMandarin	yu4
U+4205.0	fEnglish	same as &#x7C46;
U+4206.0	fUTF8	䈆
U+4206.0	fCNS11643	4-4569
U+4206.0	fGB	3-643D
U+4206.0	fR/S	 118.8
U+4206.0	fTotalStrokes	 14
U+4206.0	fCangjie	HISK
U+4206.0	fFourCorner	88230
U+4206.0	fCantonese	daai6
U+4206.0	fMandarin	zhai4; dai4
U+4206.0	fEnglish	a shallow and long shaped bamboo basket
U+4207.0	fUTF8	䈇
U+4207.0	fCNS11643	3-485F
U+4207.0	fGB	5-634C
U+4207.0	fR/S	 118.8
U+4207.0	fTotalStrokes	 14
U+4207.0	fCangjie	HYAJ
U+4207.0	fFourCorner	88400
U+4207.0	fHakka	  zau5
U+4207.0	fCantonese	zaau3
U+4207.0	fMandarin	zhao4
U+4207.0	fEnglish	[1] a basket used to catch fish [2] cover; shade
U+4208.0	fUTF8	䈈
U+4208.0	fCNS11643	4-456E
U+4208.0	fGB	3-6434
U+4208.0	fR/S	 118.8
U+4208.0	fTotalStrokes	 14
U+4208.0	fCangjie	HBAU
U+4208.0	fFourCorner	88210
U+4208.0	fCantonese	fei4
U+4208.0	fMandarin	fei2
U+4208.0	fEnglish	[1] a kind of bamboo [2] exuberant and flourishing of bamboo [3] same as &#x7B06;
U+4209.0	fUTF8	䈉
U+4209.0	fCNS11643	4-4568
U+4209.0	fGB	3-6437
U+4209.0	fR/S	 118.8
U+4209.0	fTotalStrokes	 14
U+4209.0	fCangjie	HYTV
U+4209.0	fFourCorner	88400
U+4209.0	fCantonese	saap3
U+4209.0	fMandarin	qie4; sha4
U+4209.0	fEnglish	[1] same as &#x7B91; [2] same as &#x7FE3;
U+420A.0	fUTF8	䈊
U+420A.0	fCNS11643	3-4859
U+420A.0	fR/S	 118.8
U+420A.0	fTotalStrokes	 14
U+420A.0	fCangjie	HGCE
U+420A.0	fFourCorner	88400
U+420A.0	fCantonese	ling4; liu4
U+420A.0	fMandarin	ling2
U+420A.0	fEnglish	the name of a variety of bamboo
U+420B.0	fUTF8	䈋
U+420B.0	fCNS11643	4-456D
U+420B.0	fGB	3-6442
U+420B.0	fR/S	 118.8
U+420B.0	fTotalStrokes	 14
U+420B.0	fCangjie	HEA
U+420B.0	fFourCorner	88600
U+420B.0	fCantonese	taap3; dap6
U+420B.0	fMandarin	ta3; ta4
U+420B.0	fEnglish	[1] the name of a variety of bamboo [2] a cover made of bamboo
U+420C.0	fUTF8	䈌
U+420C.0	fCNS11643	4-4566
U+420C.0	fGB	3-643A
U+420C.0	fR/S	 118.8
U+420C.0	fTotalStrokes	 14
U+420C.0	fCangjie	HEBM
U+420C.0	fFourCorner	88110
U+420C.0	fCantonese	ceoi1; zeoi1
U+420C.0	fMandarin	qu1
U+420C.0	fEnglish	[1] the name of a variety of bamboo [2] submerged; fertilized by water
U+420D.0	fUTF8	䈍
U+420D.0	fCNS11643	5-4836
U+420D.0	fGB	3-6438
U+420D.0	fR/S	 118.8
U+420D.0	fTotalStrokes	 14
U+420D.0	fCangjie	HYVU
U+420D.0	fFourCorner	88600
U+420D.0	fCantonese	mang4
U+420D.0	fMandarin	mang2
U+420D.0	fEnglish	the name of a variety of bamboo
U+420E.0	fUTF8	䈎
U+420E.0	fCNS11643	4-4B7B
U+420E.0	fGB	5-635B
U+420E.0	fR/S	 118.9
U+420E.0	fTotalStrokes	 15
U+420E.0	fCangjie	HPTD
U+420E.0	fFourCorner	88900
U+420E.0	fCantonese	zip6
U+420E.0	fMandarin	ye4
U+420E.0	fEnglish	same as &#x8449;
U+420F.0	fUTF8	䈏
U+420F.0	fCNS11643	4-4B75
U+420F.0	fGB	3-6446
U+420F.0	fR/S	 118.9
U+420F.0	fTotalStrokes	 15
U+420F.0	fCangjie	HMRW
U+420F.0	fFourCorner	88600
U+420F.0	fCantonese	bok6
U+420F.0	fMandarin	bo2; bao2
U+420F.0	fEnglish	[1] the name of a variety of bamboo [2] a woven tool
U+4210.0	fUTF8	䈐
U+4210.0	fCNS11643	4-4B70
U+4210.0	fGB	3-645B
U+4210.0	fR/S	 118.9
U+4210.0	fTotalStrokes	 15
U+4210.0	fCangjie	HPKF
U+4210.0	fFourCorner	88980
U+4210.0	fCantonese	kui2; kwui2 (also)
U+4210.0	fMandarin	kui4
U+4210.0	fEnglish	[1] an arched frame made of bamboo of wood board [2] covering [3] mats woven from bamboo and other leaves top on vehicles [4] same as &#x7C02;, &#x69F6;
U+4211.0	fUTF8	䈑
U+4211.0	fCNS11643	3-4D61
U+4211.0	fR/S	 118.9
U+4211.0	fTotalStrokes	 15
U+4211.0	fCangjie	HBBR
U+4211.0	fFourCorner	88220
U+4211.0	fHakka	 gwa3;  o2; {o2}
U+4211.0	fCantonese	gwaa2
U+4211.0	fMandarin	gua3; jue2
U+4211.0	fEnglish	a large round shallow bamboo tray for sunning flour etc.
U+4212.0	fUTF8	䈒
U+4212.0	fCNS11643	3-4D5E
U+4212.0	fR/S	 118.9
U+4212.0	fTotalStrokes	 15
U+4212.0	fCangjie	HJBJ
U+4212.0	fFourCorner	88220
U+4212.0	fCantonese	naam5; laam5 (also); nam5
U+4212.0	fMandarin	nan3
U+4212.0	fEnglish	[1] fragile; tender bamboo [2] interchangeable with &#x8E50;
U+4213.0	fUTF8	䈓
U+4213.0	fCNS11643	4-4C22
U+4213.0	fGB	3-644E
U+4213.0	fR/S	 118.9
U+4213.0	fTotalStrokes	 15
U+4213.0	fCangjie	HAPV
U+4213.0	fFourCorner	88720
U+4213.0	fCantonese	got3; haap1
U+4213.0	fMandarin	ge2
U+4213.0	fEnglish	[1] a kind of bamboo with a red skin, it is used for fine mats and other purposes [2] an ancient musical instrument which was used to give the signal to cease playing
U+4214.0	fUTF8	䈔
U+4214.0	fCNS11643	3-4D67
U+4214.0	fGB	5-6368
U+4214.0	fR/S	 118.9
U+4214.0	fTotalStrokes	 15
U+4214.0	fCangjie	HRSE
U+4214.0	fFourCorner	88240
U+4214.0	fCantonese	gaa1
U+4214.0	fMandarin	jia1
U+4214.0	fEnglish	same as &#x7B33;
U+4215.0	fUTF8	䈕
U+4215.0	fCNS11643	4-4B7E
U+4215.0	fGB	3-644B
U+4215.0	fR/S	 118.9
U+4215.0	fTotalStrokes	 15
U+4215.0	fCangjie	HAMO
U+4215.0	fFourCorner	88800
U+4215.0	fCantonese	si4
U+4215.0	fMandarin	chi2; shi5; ti2
U+4215.0	fEnglish	[1] the metal tongue in the reed instruments [2] interchangeable with &#x5319; [3] same as &#x7B53; [4] same as &#x6938;
U+4216.0	fUTF8	䈖
U+4216.0	fCNS11643	3-4D62
U+4216.0	fR/S	 118.9
U+4216.0	fTotalStrokes	 15
U+4216.0	fCangjie	HHDJ
U+4216.0	fFourCorner	88940
U+4216.0	fCantonese	fo1; fo3; tiu3
U+4216.0	fMandarin	ke1
U+4216.0	fEnglish	the name of a variety of bamboo
U+4217.0	fUTF8	䈗
U+4217.0	fCNS11643	4-4C2E
U+4217.0	fGB	3-6455
U+4217.0	fR/S	 118.9
U+4217.0	fTotalStrokes	 15
U+4217.0	fCangjie	HOIE
U+4217.0	fFourCorner	88240
U+4217.0	fCantonese	so2
U+4217.0	fMandarin	suo3
U+4217.0	fEnglish	[1] the name of a variety of bamboo [2] bamboo mats [3] interchangeable with &#x8470;
U+4218.0	fUTF8	䈘
U+4218.0	fCNS11643	5-4F3B
U+4218.0	fR/S	 118.9
U+4218.0	fTotalStrokes	 15
U+4218.0	fCangjie	HTVI
U+4218.0	fFourCorner	88730
U+4218.0	fCantonese	ci4
U+4218.0	fMandarin	ci2
U+4218.0	fEnglish	the name of a variety of bamboo
U+4219.0	fUTF8	䈙
U+4219.0	fCNS11643	3-4D64
U+4219.0	fR/S	 118.9
U+4219.0	fTotalStrokes	 15
U+4219.0	fCangjie	HVFI
U+4219.0	fFourCorner	88940
U+4219.0	fCantonese	zau6
U+4219.0	fMandarin	zhou4
U+4219.0	fEnglish	bamboo to wither; dry up and die
U+421A.0	fUTF8	䈚
U+421A.0	fCNS11643	4-4C30
U+421A.0	fGB	3-6462
U+421A.0	fR/S	 118.9
U+421A.0	fTotalStrokes	 15
U+421A.0	fCangjie	HIRP
U+421A.0	fFourCorner	88330
U+421A.0	fCantonese	doi6; toi4
U+421A.0	fMandarin	dai4; tai2
U+421A.0	fEnglish	[1] bamboo shoot [2] skin (bark) of a bamboo shoot [3] same as &#x7B88;, &#x82D4;
U+421B.0	fUTF8	䈛
U+421B.0	fCNS11643	4-4C25
U+421B.0	fGB	3-6448
U+421B.0	fR/S	 118.9
U+421B.0	fTotalStrokes	 15
U+421B.0	fCangjie	HYTE
U+421B.0	fFourCorner	88447
U+421B.0	fCantonese	faai3
U+421B.0	fMandarin	kuai4
U+421B.0	fEnglish	small bamboo whose joints are three feet apart, used for arrows
U+421C.0	fUTF8	䈜
U+421C.0	fCNS11643	4-4C2D
U+421C.0	fGB	3-6454
U+421C.0	fR/S	 118.9
U+421C.0	fTotalStrokes	 15
U+421C.0	fCangjie	HOSE
U+421C.0	fFourCorner	88240
U+421C.0	fHakka	  cim3
U+421C.0	fCantonese	cam3; kaam4; kim4; saam3; zaam1
U+421C.0	fMandarin	qin4
U+421C.0	fEnglish	an implement used in making ink
U+421D.0	fUTF8	䈝
U+421D.0	fCNS11643	4-4B79
U+421D.0	fGB	3-645E
U+421D.0	fR/S	 118.9
U+421D.0	fTotalStrokes	 15
U+421D.0	fCangjie	HNOB
U+421D.0	fFourCorner	88220
U+421D.0	fCantonese	seoi1; seot1
U+421D.0	fMandarin	xu1
U+421D.0	fEnglish	[1] the name of a variety of bamboo [2] a kind of winnowing basket
U+421E.0	fUTF8	䈞
U+421E.0	fCNS11643	5-4F32
U+421E.0	fR/S	 118.9
U+421E.0	fTotalStrokes	 15
U+421E.0	fCangjie	HYRA
U+421E.0	fFourCorner	88600
U+421E.0	fCantonese	zuk1; caau4; duk1
U+421E.0	fMandarin	du3
U+421E.0	fEnglish	[1] same as &#x7AFA; [2] interchangeable &#x7BE4;
U+421F.0	fUTF8	䈟
U+421F.0	fCNS11643	3-4D5F
U+421F.0	fR/S	 118.9
U+421F.0	fTotalStrokes	 15
U+421F.0	fCangjie	HBCN
U+421F.0	fFourCorner	88820
U+421F.0	fHakka	 zet7
U+421F.0	fCantonese	zit3; zaak1
U+421F.0	fMandarin	ce4
U+421F.0	fEnglish	the sound of crashing or splitting
U+4220.0	fUTF8	䈠
U+4220.0	fCNS11643	5-4F45
U+4220.0	fGB	3-6456
U+4220.0	fR/S	 118.9
U+4220.0	fTotalStrokes	 15
U+4220.0	fCangjie	HBME
U+4220.0	fFourCorner	88440
U+4220.0	fCantonese	wun6
U+4220.0	fMandarin	yuan4; huan3
U+4220.0	fEnglish	[1] books and volumes [2] letters and correspondence [3] snap; break; broken bamboo
U+4221.0	fUTF8	䈡
U+4221.0	fCNS11643	4-4C2C
U+4221.0	fGB	3-6458
U+4221.0	fR/S	 118.9
U+4221.0	fTotalStrokes	 15
U+4221.0	fCangjie	HPKP
U+4221.0	fFourCorner	88330
U+4221.0	fCantonese	cung1; gung1
U+4221.0	fMandarin	song1
U+4221.0	fEnglish	[1] the name of a variety of bamboo [2] same as &#x6AA7;
U+4222.0	fUTF8	䈢
U+4222.0	fCNS11643	4-4C27
U+4222.0	fGB	3-6461
U+4222.0	fR/S	 118.9
U+4222.0	fTotalStrokes	 15
U+4222.0	fCangjie	HIRD
U+4222.0	fFourCorner	88900
U+4222.0	fCantonese	soi2; sai2
U+4222.0	fMandarin	sai3
U+4222.0	fEnglish	[1] the name of a variety of bamboo [2] a thin and long strip of bamboo form making basket
U+4223.0	fUTF8	䈣
U+4223.0	fCNS11643	4-4B77
U+4223.0	fGB	3-6444
U+4223.0	fR/S	 118.9
U+4223.0	fTotalStrokes	 15
U+4223.0	fCangjie	HMMK
U+4223.0	fFourCorner	88140
U+4223.0	fCantonese	zing3
U+4223.0	fMandarin	zheng4
U+4223.0	fEnglish	the name of a variety of bamboo
U+4224.0	fUTF8	䈤
U+4224.0	fCNS11643	3-4D66
U+4224.0	fGB	5-635C
U+4224.0	fR/S	 118.9
U+4224.0	fTotalStrokes	 15
U+4224.0	fCangjie	HDTM
U+4224.0	fFourCorner	88970
U+4224.0	fCantonese	kim4
U+4224.0	fMandarin	qian2
U+4224.0	fEnglish	same as &#x9257;
U+4225.0	fUTF8	䈥
U+4225.0	fCNS11643	3-4D68
U+4225.0	fGB	5-6362
U+4225.0	fR/S	 118.9
U+4225.0	fTotalStrokes	 15
U+4225.0	fCangjie	HNBS
U+4225.0	fFourCorner	88220
U+4225.0	fCantonese	gan1
U+4225.0	fMandarin	jin1
U+4225.0	fEnglish	non-classical form of &#x7B4B;
U+4226.0	fUTF8	䈦
U+4226.0	fCNS11643	4-4C28
U+4226.0	fGB	3-6457
U+4226.0	fR/S	 118.9
U+4226.0	fTotalStrokes	 15
U+4226.0	fCangjie	HUCE
U+4226.0	fFourCorner	88400
U+4226.0	fCantonese	zung1
U+4226.0	fMandarin	zong1
U+4226.0	fEnglish	[1] a curved bamboo whip [2] thin branches of a tree
U+4227.0	fUTF8	䈧
U+4227.0	fCNS11643	5-4F36
U+4227.0	fR/S	 118.9
U+4227.0	fTotalStrokes	 15
U+4227.0	fCangjie	HIKF
U+4227.0	fFourCorner	88420
U+4227.0	fCantonese	wai5; waai2
U+4227.0	fMandarin	wei3
U+4227.0	fEnglish	skin of bamboo shoots; bark of bamboo
U+4228.0	fUTF8	䈨
U+4228.0	fGB	S-2336
U+4228.0	fR/S	 118.9
U+4228.0	fTotalStrokes	 15
U+4228.0	fCangjie	HNLV
U+4228.0	fCantonese	haan6
U+4229.0	fUTF8	䈩
U+4229.0	fGB	S-2337
U+4229.0	fR/S	 118.9
U+4229.0	fTotalStrokes	 15
U+4229.0	fCangjie	HOMN
U+4229.0	fCantonese	gim3
U+422A.0	fUTF8	䈪
U+422A.0	fCNS11643	4-5234
U+422A.0	fGB	3-6464
U+422A.0	fR/S	 118.10
U+422A.0	fTotalStrokes	 16
U+422A.0	fCangjie	HMRB
U+422A.0	fFourCorner	88220
U+422A.0	fHakka	  gak7
U+422A.0	fCantonese	hik1; zik1
U+422A.0	fMandarin	xi4
U+422A.0	fEnglish	 basket
U+422B.0	fUTF8	䈫
U+422B.0	fCNS11643	4-523A
U+422B.0	fGB	3-642D
U+422B.0	fR/S	 118.10
U+422B.0	fTotalStrokes	 16
U+422B.0	fCangjie	HVFB
U+422B.0	fFourCorner	88920
U+422B.0	fHakka	 nap8
U+422B.0	fCantonese	naap6; laap6 (also); nap6
U+422B.0	fMandarin	na4
U+422B.0	fEnglish	same as &#x7B1D;
U+422C.0	fUTF8	䈬
U+422C.0	fCNS11643	3-5231
U+422C.0	fR/S	 118.10
U+422C.0	fTotalStrokes	 16
U+422C.0	fCangjie	HEIB
U+422C.0	fFourCorner	88120
U+422C.0	fCantonese	pou4
U+422C.0	fMandarin	pu2
U+422C.0	fEnglish	a small thin strip (of bamboo) net
U+422D.0	fUTF8	䈭
U+422D.0	fCNS11643	4-5237
U+422D.0	fGB	3-6453
U+422D.0	fR/S	 118.10
U+422D.0	fTotalStrokes	 16
U+422D.0	fCangjie	HHUI
U+422D.0	fFourCorner	88210
U+422D.0	fCantonese	sau1; saau1; saau2
U+422D.0	fMandarin	sou1; huai2
U+422D.0	fEnglish	[1] the name of a variety of bamboo [2] bamboo with high joints
U+422E.0	fUTF8	䈮
U+422E.0	fCNS11643	4-522B
U+422E.0	fGB	3-646F
U+422E.0	fR/S	 118.10
U+422E.0	fTotalStrokes	 16
U+422E.0	fCangjie	HYTR
U+422E.0	fFourCorner	88120
U+422E.0	fCantonese	geoi3
U+422E.0	fMandarin	ju4; ju3; wan3
U+422E.0	fEnglish	[1] a woven instrument; a knitting tool [2] the name of a variety of bamboo
U+422F.0	fUTF8	䈯
U+422F.0	fCNS11643	4-5238
U+422F.0	fGB	3-6463
U+422F.0	fR/S	 118.10
U+422F.0	fTotalStrokes	 16
U+422F.0	fCangjie	HJBC
U+422F.0	fFourCorner	88800
U+422F.0	fCantonese	zan1; zaan1; zaan2
U+422F.0	fMandarin	zhen1
U+422F.0	fEnglish	[1] the name of a variety of bamboo [2] utensils; implements
U+4230.0	fUTF8	䈰
U+4230.0	fCNS11643	3-5235
U+4230.0	fR/S	 118.10
U+4230.0	fTotalStrokes	 16
U+4230.0	fCangjie	HQFB
U+4230.0	fFourCorner	88520
U+4230.0	fHakka	 sau1
U+4230.0	fCantonese	saau1; sau2
U+4230.0	fMandarin	shao1
U+4230.0	fEnglish	[1] basket for holding rice when steaming it [2] bamboo tube for holding chopsticks
U+4231.0	fUTF8	䈱
U+4231.0	fCNS11643	4-5239
U+4231.0	fGB	3-646C
U+4231.0	fR/S	 118.10
U+4231.0	fTotalStrokes	 16
U+4231.0	fCangjie	HBHX
U+4231.0	fFourCorner	88770
U+4231.0	fCantonese	tou1
U+4231.0	fMandarin	tao1
U+4231.0	fEnglish	an open bamboo basket used to feed domestic animals (cow, hog, etc.)
U+4232.0	fUTF8	䈲
U+4232.0	fCNS11643	4-523B
U+4232.0	fGB	3-646B
U+4232.0	fR/S	 118.10
U+4232.0	fTotalStrokes	 16
U+4232.0	fCangjie	HHYE
U+4232.0	fFourCorner	88440
U+4232.0	fCantonese	bun1
U+4232.0	fMandarin	ban1; pan2
U+4232.0	fEnglish	[1] a bamboo basket for fishing [2] the name of a variety of bamboo [3] thin and flat slips of bamboo used for weaving purpose
U+4233.0	fUTF8	䈳
U+4233.0	fCNS11643	4-5236
U+4233.0	fGB	3-6466
U+4233.0	fR/S	 118.10
U+4233.0	fTotalStrokes	 16
U+4233.0	fCangjie	HASM
U+4233.0	fFourCorner	88120
U+4233.0	fHakka	  tap7
U+4233.0	fCantonese	daap6
U+4233.0	fMandarin	ta4
U+4233.0	fEnglish	 window
U+4234.0	fUTF8	䈴
U+4234.0	fCNS11643	4-522F
U+4234.0	fR/S	 118.10
U+4234.0	fTotalStrokes	 16
U+4234.0	fCangjie	HTXC
U+4234.0	fFourCorner	88327
U+4234.0	fCantonese	him3; hip3
U+4234.0	fMandarin	qian4; zhan3
U+4234.0	fEnglish	 cage; basket; noose
U+4235.0	fUTF8	䈵
U+4235.0	fCNS11643	4-523D
U+4235.0	fGB	5-6374
U+4235.0	fR/S	 118.10
U+4235.0	fTotalStrokes	 16
U+4235.0	fCangjie	HCIM
U+4235.0	fFourCorner	88127
U+4235.0	fCantonese	jung1; zung1
U+4235.0	fMandarin	weng1
U+4235.0	fEnglish	luxuriant; exuberant of bamboo
U+4236.0	fUTF8	䈶
U+4236.0	fCNS11643	4-5229
U+4236.0	fGB	3-6471
U+4236.0	fR/S	 118.10
U+4236.0	fTotalStrokes	 16
U+4236.0	fCangjie	HJCR
U+4236.0	fFourCorner	88600
U+4236.0	fCantonese	jung4; zung4
U+4236.0	fMandarin	rong2
U+4236.0	fEnglish	[1] arrow; dart [2] asparagus fern
U+4237.0	fUTF8	䈷
U+4237.0	fCNS11643	3-5232
U+4237.0	fR/S	 118.10
U+4237.0	fTotalStrokes	 16
U+4237.0	fCangjie	HDHR
U+4237.0	fFourCorner	88960
U+4237.0	fCantonese	gaak3; lok3; lok6
U+4237.0	fMandarin	luo4
U+4237.0	fEnglish	same as standard form &#x683C;
U+4238.0	fUTF8	䈸
U+4238.0	fCNS11643	4-575B
U+4238.0	fGB	3-6521
U+4238.0	fR/S	 118.11
U+4238.0	fTotalStrokes	 17; 16
U+4238.0	fCangjie	HNBJ
U+4238.0	fFourCorner	88240
U+4238.0	fHakka	 fuk8
U+4238.0	fCantonese	huk6
U+4238.0	fMandarin	hu2
U+4238.0	fEnglish	bushel measure, corn bin
U+4239.0	fUTF8	䈹
U+4239.0	fCNS11643	5-5C6D
U+4239.0	fR/S	 118.11
U+4239.0	fTotalStrokes	 17
U+4239.0	fCangjie	HJOA
U+4239.0	fFourCorner	88260
U+4239.0	fCantonese	sou2; saau2
U+4239.0	fMandarin	sou3
U+4239.0	fEnglish	same as &#x7C54;
U+423A.0	fUTF8	䈺
U+423A.0	fCNS11643	4-575E
U+423A.0	fGB	3-6441
U+423A.0	fR/S	 118.11
U+423A.0	fTotalStrokes	 17; 16
U+423A.0	fCangjie	HVFY
U+423A.0	fFourCorner	88930
U+423A.0	fCantonese	zung1
U+423A.0	fMandarin	zhong1
U+423A.0	fEnglish	[1] bamboo [2] chest; box
U+423B.0	fUTF8	䈻
U+423B.0	fCNS11643	4-575A
U+423B.0	fGB	3-647E
U+423B.0	fR/S	 118.11
U+423B.0	fTotalStrokes	 17; 16
U+423B.0	fCangjie	HBIB
U+423B.0	fFourCorner	88220
U+423B.0	fCantonese	pou4
U+423B.0	fMandarin	pu2
U+423B.0	fEnglish	heavy mats of bamboo used for deep water fishing
U+423C.0	fUTF8	䈼
U+423C.0	fCNS11643	4-5762
U+423C.0	fGB	3-6522
U+423C.0	fR/S	 118.11
U+423C.0	fTotalStrokes	 17; 16
U+423C.0	fCangjie	HJPU
U+423C.0	fFourCorner	88770
U+423C.0	fCantonese	mit6
U+423C.0	fMandarin	mi4; mie4
U+423C.0	fEnglish	same as &#x7BFE;
U+423D.0	fUTF8	䈽
U+423D.0	fCNS11643	5-5C7D
U+423D.0	fR/S	 118.11
U+423D.0	fTotalStrokes	 17
U+423D.0	fCangjie	HTLM
U+423D.0	fFourCorner	88100
U+423D.0	fHakka	 giun3; gin1; giun1
U+423D.0	fCantonese	gan1
U+423D.0	fMandarin	jin1
U+423D.0	fEnglish	name of a large variety of bamboo from Kwei-chow
U+423E.0	fUTF8	䈾
U+423E.0	fCNS11643	4-5765
U+423E.0	fGB	3-6477
U+423E.0	fR/S	 118.11
U+423E.0	fTotalStrokes	 17; 16
U+423E.0	fCangjie	HDFB
U+423E.0	fFourCorner	88920
U+423E.0	fCantonese	saau1; sau1; se3; sok3
U+423E.0	fMandarin	shao1; shuo4
U+423E.0	fEnglish	[1] tail of a helm [2] flutter in the wind
U+423F.0	fUTF8	䈿
U+423F.0	fCNS11643	4-5753
U+423F.0	fGB	3-6472
U+423F.0	fR/S	 118.11; 118.10
U+423F.0	fTotalStrokes	 16; 17
U+423F.0	fCangjie	HBAC
U+423F.0	fFourCorner	88800
U+423F.0	fCantonese	mik6; maak6
U+423F.0	fMandarin	mai4; mi4
U+423F.0	fEnglish	belt of the canvas top on vehicles
U+4240.0	fUTF8	䉀
U+4240.0	fCNS11643	3-556C
U+4240.0	fR/S	 118.11
U+4240.0	fTotalStrokes	 17
U+4240.0	fCangjie	HITF
U+4240.0	fFourCorner	88230
U+4240.0	fCantonese	syu3; syun4
U+4240.0	fMandarin	shu4
U+4240.0	fEnglish	rectangular chest or box woven from bamboo strips (or wicker)
U+4241.0	fUTF8	䉁
U+4241.0	fCNS11643	4-5756
U+4241.0	fGB	3-6475
U+4241.0	fR/S	 118.11
U+4241.0	fTotalStrokes	 17; 16
U+4241.0	fCangjie	HSJI
U+4241.0	fFourCorner	88130
U+4241.0	fCantonese	ling4
U+4241.0	fMandarin	ling2
U+4241.0	fEnglish	the name of a variety of bamboo, bamboo ware
U+4242.0	fUTF8	䉂
U+4242.0	fCNS11643	4-5758
U+4242.0	fGB	3-647A
U+4242.0	fR/S	 118.11
U+4242.0	fTotalStrokes	 17; 16
U+4242.0	fCangjie	HWVF
U+4242.0	fFourCorner	88900
U+4242.0	fHakka	 lui3
U+4242.0	fCantonese	leoi5
U+4242.0	fMandarin	lei3
U+4242.0	fEnglish	[1] a basket for earth or soil [2] regulations; law [3] model
U+4243.0	fUTF8	䉃
U+4243.0	fCNS11643	4-5759
U+4243.0	fGB	3-6459
U+4243.0	fR/S	 118.11
U+4243.0	fTotalStrokes	 17; 16
U+4243.0	fCangjie	HVMI
U+4243.0	fFourCorner	88240
U+4243.0	fHakka	  ziong3
U+4243.0	fCantonese	zoeng2
U+4243.0	fMandarin	jiang3
U+4243.0	fEnglish	[1] split bamboos [2] mat
U+4244.0	fUTF8	䉄
U+4244.0	fCNS11643	5-5C7E
U+4244.0	fGB	7-236D
U+4244.0	fR/S	 118.11
U+4244.0	fTotalStrokes	 17
U+4244.0	fCangjie	HNLE
U+4244.0	fFourCorner	88240
U+4244.0	fCantonese	ling4
U+4244.0	fMandarin	leng2
U+4244.0	fEnglish	the name of a variety of bamboo
U+4245.0	fUTF8	䉅
U+4245.0	fCNS11643	4-5755
U+4245.0	fGB	3-6369
U+4245.0	fR/S	 118.11
U+4245.0	fTotalStrokes	 17; 16
U+4245.0	fCangjie	HGJI
U+4245.0	fFourCorner	88410
U+4245.0	fCantonese	zap1; caap1; zi3
U+4245.0	fMandarin	zhi4
U+4245.0	fEnglish	[1] small bamboo whose joints are three feet apart, used for arrows [2] the name of a variety of bamboo
U+4246.0	fUTF8	䉆
U+4246.0	fCNS11643	5-5D30
U+4246.0	fR/S	 118.11
U+4246.0	fTotalStrokes	 17
U+4246.0	fCangjie	HHAF
U+4246.0	fFourCorner	88320
U+4246.0	fCantonese	niu5; liu5 (also); diu2
U+4246.0	fMandarin	diao3
U+4246.0	fEnglish	the name of a variety of bamboo
U+4247.0	fUTF8	䉇
U+4247.0	fGB	S-2338
U+4247.0	fR/S	 118.11
U+4247.0	fTotalStrokes	 17
U+4247.0	fCangjie	HTVN
U+4247.0	fCantonese	zaam1
U+4248.0	fUTF8	䉈
U+4248.0	fCNS11643	4-5C44
U+4248.0	fGB	3-6524
U+4248.0	fR/S	 118.12
U+4248.0	fTotalStrokes	 18
U+4248.0	fCangjie	HTBK
U+4248.0	fFourCorner	88240
U+4248.0	fCantonese	saan2
U+4248.0	fMandarin	san3
U+4248.0	fEnglish	a kind of bamboo with a red skin, used for fine mats and other purposes
U+4249.0	fUTF8	䉉
U+4249.0	fCNS11643	4-5C4F
U+4249.0	fGB	5-6434
U+4249.0	fR/S	 118.12
U+4249.0	fTotalStrokes	 18
U+4249.0	fCangjie	HNBO
U+4249.0	fFourCorner	88230
U+4249.0	fCantonese	gu1; gwu1 (also); wu4
U+4249.0	fMandarin	hu2
U+4249.0	fEnglish	interchangeable with &#x89DA;
U+424A.0	fUTF8	䉊
U+424A.0	fCNS11643	4-5C4B
U+424A.0	fGB	3-6460
U+424A.0	fR/S	 118.12
U+424A.0	fTotalStrokes	 18
U+424A.0	fCangjie	HIKJ
U+424A.0	fFourCorner	88500
U+424A.0	fCantonese	faan6; fan4; fan6
U+424A.0	fMandarin	fan4; fang2
U+424A.0	fEnglish	[1] bamboo ware [2] awning in front of a cart [3] covering the horse or mule in the shafts; canvas top on vehicles
U+424B.0	fUTF8	䉋
U+424B.0	fCNS11643	4-5C50
U+424B.0	fGB	3-652E
U+424B.0	fR/S	 118.12
U+424B.0	fTotalStrokes	 18
U+424B.0	fCangjie	HVAU
U+424B.0	fFourCorner	88460
U+424B.0	fCantonese	mei6
U+424B.0	fMandarin	mei4
U+424B.0	fEnglish	a variety of bamboo shoots that sprout in water
U+424C.0	fUTF8	䉌
U+424C.0	fCNS11643	4-5C43
U+424C.0	fGB	3-6352
U+424C.0	fR/S	 118.12
U+424C.0	fTotalStrokes	 18
U+424C.0	fCangjie	HNLO
U+424C.0	fFourCorner	88230
U+424C.0	fCantonese	seoi6
U+424C.0	fMandarin	sui4
U+424C.0	fEnglish	[1] crude bamboo mats [2] a vessel for raising silk-worms
U+424D.0	fUTF8	䉍
U+424D.0	fCNS11643	3-584A
U+424D.0	fGB	5-6346
U+424D.0	fR/S	 118.12
U+424D.0	fTotalStrokes	 18
U+424D.0	fCangjie	HANP
U+424D.0	fFourCorner	88220
U+424D.0	fCantonese	gaan2
U+424D.0	fMandarin	jian3
U+424D.0	fEnglish	(non-classical form) observe; watch; examine
U+424E.0	fUTF8	䉎
U+424E.0	fCNS11643	4-5C4A
U+424E.0	fGB	5-642F
U+424E.0	fR/S	 118.12
U+424E.0	fTotalStrokes	 18
U+424E.0	fCangjie	HFBD
U+424E.0	fFourCorner	88900
U+424E.0	fCantonese	tong4
U+424E.0	fMandarin	tang2
U+424E.0	fEnglish	[1] cover [2] shade [3] a basket used to catch fish
U+424F.0	fUTF8	䉏
U+424F.0	fCNS11643	3-5A6C
U+424F.0	fR/S	 118.13
U+424F.0	fTotalStrokes	 19
U+424F.0	fCangjie	HNBQ
U+424F.0	fFourCorner	88250
U+424F.0	fCantonese	haai5; aam2; hai6
U+424F.0	fMandarin	xie4
U+424F.0	fEnglish	the name of a variety of bamboo
U+4250.0	fUTF8	䉐
U+4250.0	fCNS11643	4-5C42
U+4250.0	fGB	3-6447
U+4250.0	fR/S	 118.12
U+4250.0	fTotalStrokes	 18
U+4250.0	fCangjie	HJJR
U+4250.0	fFourCorner	88560
U+4250.0	fCantonese	fu1
U+4250.0	fMandarin	ku1
U+4250.0	fEnglish	[1] a thin and long strip of bamboo for making baskets, etc.; thin (bamboo) laths [2] the name of a variety of bamboo [3] corrupted form of &#x7C35;, &#x7C2C;
U+4251.0	fUTF8	䉑
U+4251.0	fCNS11643	4-5C4C
U+4251.0	fGB	3-6348
U+4251.0	fR/S	 118.12
U+4251.0	fTotalStrokes	 18
U+4251.0	fCangjie	HOTF
U+4251.0	fFourCorner	88330
U+4251.0	fCantonese	mou4; mou5
U+4251.0	fMandarin	wu2; mo2
U+4251.0	fEnglish	black skinned bamboo
U+4252.0	fUTF8	䉒
U+4252.0	fCNS11643	7-413F
U+4252.0	fR/S	 118.12
U+4252.0	fTotalStrokes	 18
U+4252.0	fCangjie	HHDW
U+4252.0	fFourCorner	88600
U+4252.0	fCantonese	faan1; fan4
U+4252.0	fMandarin	fan1; fan2
U+4252.0	fEnglish	[1] same as &#x7C53; [2] same as &#x85E9;
U+4253.0	fUTF8	䉓
U+4253.0	fCNS11643	4-5C46
U+4253.0	fGB	3-6525
U+4253.0	fR/S	 118.12
U+4253.0	fTotalStrokes	 18
U+4253.0	fCangjie	HDWD
U+4253.0	fFourCorner	88990
U+4253.0	fCantonese	lo6; kok3; lok3; lok6
U+4253.0	fMandarin	luo4; lei2
U+4253.0	fEnglish	[1] a mattress made of bamboo strips [2] bed
U+4254.0	fUTF8	䉔
U+4254.0	fCNS11643	4-5C49
U+4254.0	fGB	5-635E
U+4254.0	fR/S	 118.12
U+4254.0	fTotalStrokes	 18
U+4254.0	fCangjie	HMNI
U+4254.0	fFourCorner	88250
U+4254.0	fCantonese	caan1; can1; saang6
U+4254.0	fMandarin	can1
U+4254.0	fEnglish	bamboo slips, crude; coarse bamboo mats
U+4255.0	fUTF8	䉕
U+4255.0	fCNS11643	4-5C40
U+4255.0	fGB	3-652C
U+4255.0	fR/S	 118.12
U+4255.0	fTotalStrokes	 18
U+4255.0	fCangjie	HCWA
U+4255.0	fFourCorner	88600
U+4255.0	fCantonese	cang4
U+4255.0	fMandarin	ceng2
U+4255.0	fEnglish	[1] a bamboo hat [2] a bamboo shade or covering [3] the name of a variety of bamboo
U+4256.0	fUTF8	䉖
U+4256.0	fCNS11643	5-6334
U+4256.0	fR/S	 118.12
U+4256.0	fTotalStrokes	 18
U+4256.0	fCangjie	HJJI
U+4256.0	fFourCorner	88530
U+4256.0	fCantonese	ling4
U+4256.0	fMandarin	ling2
U+4256.0	fEnglish	same as &#x7B2D;
U+4257.0	fUTF8	䉗
U+4257.0	fCNS11643	4-5C48
U+4257.0	fGB	3-6528
U+4257.0	fR/S	 118.12
U+4257.0	fTotalStrokes	 18
U+4257.0	fCangjie	HQRJ
U+4257.0	fFourCorner	88540
U+4257.0	fCantonese	jap1; zaap1
U+4257.0	fMandarin	yi1
U+4257.0	fEnglish	 receptacle or container (as a cup, ladle, spoon, etc.)
U+4258.0	fUTF8	䉘
U+4258.0	fCNS11643	3-584D
U+4258.0	fR/S	 118.12
U+4258.0	fTotalStrokes	 18
U+4258.0	fCangjie	HDJF
U+4258.0	fFourCorner	88990
U+4258.0	fCantonese	soeng3; cung4
U+4258.0	fMandarin	shuang4; cong2
U+4258.0	fEnglish	 pierce; stab with a sharp wooden stick or a piece of bamboo slip
U+4259.0	fUTF8	䉙
U+4259.0	fCNS11643	3-584B
U+4259.0	fR/S	 118.12
U+4259.0	fTotalStrokes	 18
U+4259.0	fCangjie	HMBI
U+4259.0	fFourCorner	88730
U+4259.0	fCantonese	wan4
U+4259.0	fMandarin	yun2
U+4259.0	fEnglish	the name of a variety of bamboo; big bamboo
U+425A.0	fUTF8	䉚
U+425A.0	fCNS11643	5-6341
U+425A.0	fGB	3-647B
U+425A.0	fR/S	 118.12
U+425A.0	fTotalStrokes	 18
U+425A.0	fCangjie	HWLN
U+425A.0	fFourCorner	88710
U+425A.0	fCantonese	mang4; kaau3; maang4; mong4; ngaa5
U+425A.0	fMandarin	meng2
U+425A.0	fEnglish	[1] the name of a variety of bamboo [2] a bamboo shoot [3] a bamboo tube
U+425B.0	fUTF8	䉛
U+425B.0	fCNS11643	4-6056
U+425B.0	fGB	3-652A
U+425B.0	fR/S	 118.13
U+425B.0	fTotalStrokes	 19
U+425B.0	fCangjie	HHBK
U+425B.0	fFourCorner	88432
U+425B.0	fCantonese	juk1; zuk1
U+425B.0	fMandarin	ao3; yu4
U+425B.0	fEnglish	[1] a bamboo basket for washing rice [2] implements used to move the silkworm
U+425C.0	fUTF8	䉜
U+425C.0	fCNS11643	5-686E
U+425C.0	fR/S	 118.13
U+425C.0	fTotalStrokes	 19
U+425C.0	fCangjie	HOKG
U+425C.0	fFourCorner	88410
U+425C.0	fCantonese	zi6
U+425C.0	fMandarin	zhi4
U+425C.0	fEnglish	young and tender bamboo
U+425D.0	fUTF8	䉝
U+425D.0	fCNS11643	3-5A6A
U+425D.0	fR/S	 118.13
U+425D.0	fTotalStrokes	 19
U+425D.0	fCangjie	HTGI
U+425D.0	fFourCorner	88550
U+425D.0	fCantonese	ngai5; ai5 (also); zi2
U+425D.0	fMandarin	yi3; qi3
U+425D.0	fEnglish	bamboo ware
U+425E.0	fUTF8	䉞
U+425E.0	fCNS11643	4-6053
U+425E.0	fGB	3-6531
U+425E.0	fR/S	 118.13
U+425E.0	fTotalStrokes	 19
U+425E.0	fCangjie	HIHP
U+425E.0	fFourCorner	88250
U+425E.0	fCantonese	dam2; cang4; daam2; daau2
U+425E.0	fMandarin	dan3
U+425E.0	fEnglish	[1] box; chest made of bamboo; bamboo ware [2] the name of a variety of bamboo
U+425F.0	fUTF8	䉟
U+425F.0	fCNS11643	5-6866
U+425F.0	fR/S	 118.13
U+425F.0	fTotalStrokes	 19
U+425F.0	fCangjie	HQOE
U+425F.0	fFourCorner	88540
U+425F.0	fCantonese	wok6
U+425F.0	fMandarin	huo4
U+425F.0	fEnglish	a bamboo basket used to catch fishes
U+4260.0	fUTF8	䉠
U+4260.0	fCNS11643	3-5A6D
U+4260.0	fGB	5-6441
U+4260.0	fR/S	 118.13
U+4260.0	fTotalStrokes	 19
U+4260.0	fCangjie	HHOK
U+4260.0	fFourCorner	88240
U+4260.0	fCantonese	mei4
U+4260.0	fMandarin	wei2
U+4260.0	fEnglish	the name of a variety of bamboo, leaves to make covering and the stems make arrows
U+4261.0	fUTF8	䉡
U+4261.0	fCNS11643	3-5A69
U+4261.0	fGB	5-6442
U+4261.0	fR/S	 118.13
U+4261.0	fTotalStrokes	 19
U+4261.0	fCangjie	HYWM
U+4261.0	fFourCorner	88100
U+4261.0	fCantonese	taan4; tam4
U+4261.0	fMandarin	tan2
U+4261.0	fEnglish	[1] the name of a variety of bamboo [2] a towrope used to tow a boat against the current in swift stream from the bank
U+4262.0	fUTF8	䉢
U+4262.0	fCNS11643	3-5A6B
U+4262.0	fR/S	 118.13
U+4262.0	fTotalStrokes	 19
U+4262.0	fCangjie	HGOW
U+4262.0	fFourCorner	88600
U+4262.0	fCantonese	sik1; saap1
U+4262.0	fMandarin	se4
U+4262.0	fEnglish	 sieve; screen; sifter; strainer
U+4263.0	fUTF8	䉣
U+4263.0	fCNS11643	4-5C51
U+4263.0	fGB	3-6529
U+4263.0	fR/S	 118.12; 118.13
U+4263.0	fTotalStrokes	 18; 19
U+4263.0	fCangjie	HHXF
U+4263.0	fFourCorner	88320
U+4263.0	fCantonese	se3
U+4263.0	fMandarin	xie4
U+4263.0	fEnglish	slips of bamboo provided for writing in ancient times
U+4264.0	fUTF8	䉤
U+4264.0	fR/S	 118.13
U+4264.0	fTotalStrokes	 19
U+4264.0	fCangjie	HFVK
U+4264.0	fCantonese	sou2; sau2; saau2
U+4264.0	fMandarin	sou3
U+4264.0	fEnglish	simplified form of &#x85EA;
U+4265.0	fUTF8	䉥
U+4265.0	fCNS11643	4-637E
U+4265.0	fGB	5-6446
U+4265.0	fR/S	 118.14
U+4265.0	fTotalStrokes	 20
U+4265.0	fCangjie	HQHP
U+4265.0	fFourCorner	88530
U+4265.0	fCantonese	sung1; zung1
U+4265.0	fMandarin	song3
U+4265.0	fEnglish	same as &#x6AA7;
U+4266.0	fUTF8	䉦
U+4266.0	fCNS11643	4-6669
U+4266.0	fGB	3-636D
U+4266.0	fR/S	 118.15
U+4266.0	fTotalStrokes	 21
U+4266.0	fCangjie	HYMU
U+4266.0	fFourCorner	88300
U+4266.0	fCantonese	cin1
U+4266.0	fMandarin	qian1
U+4266.0	fEnglish	[1] the name of a variety of bamboo [2] a farm tool used to collect rice plants
U+4267.0	fUTF8	䉧
U+4267.0	fCNS11643	4-666C
U+4267.0	fGB	3-6373
U+4267.0	fR/S	 118.15
U+4267.0	fTotalStrokes	 21
U+4267.0	fCangjie	HHCN
U+4267.0	fFourCorner	88120
U+4267.0	fCantonese	lau4
U+4267.0	fMandarin	liu2; liu3
U+4267.0	fEnglish	[1] the sound of bamboo [2] the name of a variety of bamboo
U+4268.0	fUTF8	䉨
U+4268.0	fCNS11643	4-666E
U+4268.0	fGB	3-6537
U+4268.0	fR/S	 118.15
U+4268.0	fTotalStrokes	 21
U+4268.0	fCangjie	HYOE
U+4268.0	fFourCorner	88240
U+4268.0	fCantonese	ngai6; ai6 (also); ngaai6
U+4268.0	fMandarin	yi4
U+4268.0	fEnglish	[1] bamboo joints [2] the name of a variety of bamboo [3] small bamboo
U+4269.0	fUTF8	䉩
U+4269.0	fCNS11643	3-5E4A
U+4269.0	fR/S	 118.15
U+4269.0	fTotalStrokes	 21
U+4269.0	fCangjie	HDSR
U+4269.0	fFourCorner	88916
U+4269.0	fCantonese	au1; ngau1 (also); aau1
U+4269.0	fMandarin	ou1
U+4269.0	fEnglish	 bamboo ware; cradle
U+426A.0	fUTF8	䉪
U+426A.0	fCNS11643	4-666A
U+426A.0	fR/S	 118.15
U+426A.0	fTotalStrokes	 21
U+426A.0	fCangjie	HWWW
U+426A.0	fFourCorner	88660
U+426A.0	fHakka	 lui2;  lui3
U+426A.0	fCantonese	leoi6
U+426A.0	fMandarin	lei4
U+426A.0	fEnglish	a small basket with narrow mouth for holding small fishes, frogs, etc.
U+426B.0	fUTF8	䉫
U+426B.0	fCNS11643	3-5E4D
U+426B.0	fGB	5-644C
U+426B.0	fR/S	 118.15
U+426B.0	fTotalStrokes	 21
U+426B.0	fCangjie	HHHE
U+426B.0	fFourCorner	88230
U+426B.0	fCantonese	lai4
U+426B.0	fMandarin	li2
U+426B.0	fEnglish	the name of a variety of bamboo, used as a musical instrument
U+426C.0	fUTF8	䉬
U+426C.0	fCNS11643	4-6668
U+426C.0	fGB	3-6435
U+426C.0	fR/S	 118.15
U+426C.0	fTotalStrokes	 21
U+426C.0	fCangjie	HINE
U+426C.0	fFourCorner	88247
U+426C.0	fCantonese	fai3; bit1; faai3
U+426C.0	fMandarin	fei4
U+426C.0	fEnglish	coarse bamboo mats
U+426D.0	fUTF8	䉭
U+426D.0	fCNS11643	4-666D
U+426D.0	fGB	3-6539
U+426D.0	fR/S	 118.15
U+426D.0	fTotalStrokes	 21
U+426D.0	fCangjie	HVVV
U+426D.0	fFourCorner	88710
U+426D.0	fCantonese	lip6; lap6
U+426D.0	fMandarin	lie4
U+426D.0	fEnglish	[1] the name of a variety of bamboo [2] covering made of bamboo strips
U+426E.0	fUTF8	䉮
U+426E.0	fCNS11643	3-5F56
U+426E.0	fGB	5-6427
U+426E.0	fR/S	 118.16
U+426E.0	fTotalStrokes	 22
U+426E.0	fCangjie	HANG
U+426E.0	fFourCorner	88220
U+426E.0	fCantonese	leon6; long5; mong4
U+426E.0	fMandarin	lin4
U+426E.0	fEnglish	[1] plant; set up [2] damage [3] lose [4] weak
U+426F.0	fUTF8	䉯
U+426F.0	fCNS11643	5-7061
U+426F.0	fR/S	 118.16; 118.15
U+426F.0	fTotalStrokes	 21; 22
U+426F.0	fCangjie	HSEC
U+426F.0	fFourCorner	88800
U+426F.0	fCantonese	haan6; gaa3; haang1; zim3; zin6
U+426F.0	fMandarin	xian4
U+426F.0	fEnglish	dried bamboo
U+4270.0	fUTF8	䉰
U+4270.0	fCNS11643	4-6926
U+4270.0	fGB	5-644B
U+4270.0	fR/S	 118.16
U+4270.0	fTotalStrokes	 22; 21
U+4270.0	fCangjie	HBBK
U+4270.0	fFourCorner	88240
U+4270.0	fCantonese	haau6; ngau4
U+4270.0	fMandarin	xiao2; yao2; jiao1; xiao4; jiao3
U+4270.0	fEnglish	[1] bamboo shoots [2] same as &#x7B4A;
U+4271.0	fUTF8	䉱
U+4271.0	fCNS11643	4-6928
U+4271.0	fGB	3-645C
U+4271.0	fR/S	 118.16
U+4271.0	fTotalStrokes	 22; 21
U+4271.0	fCangjie	HLSR
U+4271.0	fFourCorner	88210
U+4271.0	fCantonese	au1; ngau1 (also); aau1
U+4271.0	fMandarin	ou1
U+4271.0	fEnglish	 cradle
U+4272.0	fUTF8	䉲
U+4272.0	fCNS11643	4-6A5D
U+4272.0	fGB	3-643F
U+4272.0	fR/S	 118.17
U+4272.0	fTotalStrokes	 23
U+4272.0	fCangjie	HNMB
U+4272.0	fFourCorner	88220
U+4272.0	fCantonese	nei4; lei4 (also)
U+4272.0	fMandarin	mi2; bie4
U+4272.0	fEnglish	thin and flat slips of bamboo used for weaving purpose
U+4273.0	fUTF8	䉳
U+4273.0	fCNS11643	4-6A61
U+4273.0	fGB	3-6535
U+4273.0	fR/S	 118.17
U+4273.0	fTotalStrokes	 23; 22
U+4273.0	fCangjie	HNFQ
U+4273.0	fFourCorner	88350
U+4273.0	fCantonese	sin1
U+4273.0	fMandarin	xian1; xian3
U+4273.0	fEnglish	[1] the name of a variety of bamboo [2] domicile; record of the population
U+4274.0	fUTF8	䉴
U+4274.0	fCNS11643	4-6A5C
U+4274.0	fGB	3-653F
U+4274.0	fR/S	 118.17
U+4274.0	fTotalStrokes	 23; 22
U+4274.0	fCangjie	HYRV
U+4274.0	fFourCorner	88730
U+4274.0	fCantonese	joeng4; zoeng4
U+4274.0	fMandarin	rang3; rang2
U+4274.0	fEnglish	[1] a bamboo ware to collect and keep something in [2] a kind of bamboo ware used to dripping rice
U+4275.0	fUTF8	䉵
U+4275.0	fCNS11643	3-605F
U+4275.0	fGB	5-6456
U+4275.0	fR/S	 184.14
U+4275.0	fTotalStrokes	 23
U+4275.0	fCangjie	HBUV
U+4275.0	fFourCorner	88730
U+4275.0	fHakka	 con3;  sien3
U+4275.0	fCantonese	zaan6; zaan3 (also); gyun6; zan6; zyun2
U+4275.0	fMandarin	zuan4; zhuan4; chuan3; zhuan3; juan4
U+4275.0	fEnglish	same as &#x994C;
U+4276.0	fUTF8	䉶
U+4276.0	fCNS11643	4-6B76
U+4276.0	fGB	3-6354
U+4276.0	fR/S	 118.18
U+4276.0	fTotalStrokes	 24
U+4276.0	fCangjie	HOGE
U+4276.0	fFourCorner	88400
U+4276.0	fCantonese	soeng1
U+4276.0	fMandarin	shuang1
U+4276.0	fEnglish	[1] sail (of a boat) [2] a untensil made of bamboo used to filter or to strain out wine
U+4277.0	fUTF8	䉷
U+4277.0	fCNS11643	4-6D5C
U+4277.0	fGB	5-633B
U+4277.0	fR/S	 118.20
U+4277.0	fTotalStrokes	 26
U+4277.0	fCangjie	HRRK
U+4277.0	fFourCorner	88240
U+4277.0	fHakka	  ngiam2
U+4277.0	fCantonese	jim4; lam4; zim4
U+4277.0	fMandarin	yan2; dan4; jin4
U+4277.0	fEnglish	fence; pallisade
U+4278.0	fUTF8	䉸
U+4278.0	fCNS11643	4-6D76
U+4278.0	fGB	3-653C
U+4278.0	fR/S	 118.21
U+4278.0	fTotalStrokes	 27
U+4278.0	fCangjie	HYJJ
U+4278.0	fFourCorner	88440
U+4278.0	fCantonese	bin6
U+4278.0	fMandarin	bian4; bian3
U+4278.0	fEnglish	slips of bamboo for writing
U+4279.0	fUTF8	䉹
U+4279.0	fCNS11643	7-655D
U+4279.0	fR/S	 118.24
U+4279.0	fTotalStrokes	 30
U+4279.0	fCangjie	HMBO
U+4279.0	fFourCorner	88100
U+4279.0	fCantonese	ling4
U+4279.0	fMandarin	ling2; liu3
U+4279.0	fEnglish	same as &#x4241;
U+427A.0	fUTF8	䉺
U+427A.0	fCNS11643	4-2A77
U+427A.0	fGB	3-676B
U+427A.0	fR/S	 119.3
U+427A.0	fTotalStrokes	 9
U+427A.0	fCangjie	FDM
U+427A.0	fFourCorner	91910
U+427A.0	fCantonese	hong2 (also); hung4
U+427A.0	fMandarin	hong2
U+427A.0	fEnglish	[1] deteriorated and become red; old rice; decaying rice [2] red rice
U+427B.0	fUTF8	䉻
U+427B.0	fCNS11643	3-332F
U+427B.0	fR/S	 119.4
U+427B.0	fTotalStrokes	 10
U+427B.0	fCangjie	FDHVP
U+427B.0	fFourCorner	92940
U+427B.0	fCantonese	kei4
U+427B.0	fMandarin	qi2
U+427B.0	fEnglish	[1] red rice [2] coarse rice
U+427C.0	fUTF8	䉼
U+427C.0	fCNS11643	3-3333
U+427C.0	fR/S	 119.4
U+427C.0	fTotalStrokes	 10
U+427C.0	fCangjie	FDHML
U+427C.0	fFourCorner	92920
U+427C.0	fCantonese	liu6
U+427C.0	fMandarin	liao4
U+427C.0	fEnglish	non-classical form of &#x6599;
U+427D.0	fUTF8	䉽
U+427D.0	fCNS11643	6-4322
U+427D.0	fGB	5-6661
U+427D.0	fR/S	 119.5
U+427D.0	fTotalStrokes	 11
U+427D.0	fCangjie	FDFQ
U+427D.0	fFourCorner	99950
U+427D.0	fCantonese	bun2; ban2
U+427D.0	fMandarin	ban3
U+427D.0	fEnglish	same as &#x7C84;
U+427E.0	fUTF8	䉾
U+427E.0	fCNS11643	4-3425
U+427E.0	fGB	3-6773
U+427E.0	fR/S	 119.5
U+427E.0	fTotalStrokes	 11
U+427E.0	fCangjie	LPFD
U+427E.0	fFourCorner	11900
U+427E.0	fCantonese	bei3; bui6; pui3
U+427E.0	fMandarin	bi4; mi4; pei4
U+427E.0	fEnglish	bad; poor quality of rice
U+427F.0	fUTF8	䉿
U+427F.0	fCNS11643	5-3459
U+427F.0	fR/S	 119.5
U+427F.0	fTotalStrokes	 11
U+427F.0	fCangjie	FDHFD
U+427F.0	fFourCorner	92940
U+427F.0	fCantonese	wu4
U+427F.0	fMandarin	hu2; luo4
U+427F.0	fEnglish	same as &#x7CCA;
U+4280.0	fUTF8	䊀
U+4280.0	fCNS11643	3-377C
U+4280.0	fR/S	 119.5
U+4280.0	fTotalStrokes	 11
U+4280.0	fCangjie	FDJR
U+4280.0	fFourCorner	94960
U+4280.0	fCantonese	wu4
U+4280.0	fMandarin	hu2
U+4280.0	fEnglish	[1] same as &#x427F; [2] same as &#x7CCA;
U+4281.0	fUTF8	䊁
U+4281.0	fCNS11643	6-4C60
U+4281.0	fR/S	 119.6
U+4281.0	fTotalStrokes	 12
U+4281.0	fCangjie	FDHGU
U+4281.0	fFourCorner	94910
U+4281.0	fHakka	 sin1
U+4281.0	fCantonese	san1; caap1; saan1
U+4281.0	fMandarin	shen1
U+4281.0	fEnglish	[1] same as &#x7C78; [2] interchangeable with &#x4B40;
U+4282.0	fUTF8	䊂
U+4282.0	fCNS11643	4-397B
U+4282.0	fGB	3-6775
U+4282.0	fR/S	 119.6
U+4282.0	fTotalStrokes	 12
U+4282.0	fCangjie	FDDB
U+4282.0	fFourCorner	95990
U+4282.0	fCantonese	caak3
U+4282.0	fMandarin	suo3; ce4; se4
U+4282.0	fEnglish	poor quality of rice; bad rice cakes stick to each other
U+4283.0	fUTF8	䊃
U+4283.0	fCNS11643	4-397C
U+4283.0	fGB	3-6778
U+4283.0	fR/S	 119.6
U+4283.0	fTotalStrokes	 12
U+4283.0	fCangjie	FDHNB
U+4283.0	fFourCorner	97910
U+4283.0	fCantonese	bui6; paau2; pui3
U+4283.0	fMandarin	pei4
U+4283.0	fEnglish	[1] mixing rice with broth [2] a grain of rice
U+4284.0	fUTF8	䊄
U+4284.0	fCNS11643	3-3D7D
U+4284.0	fR/S	 119.6
U+4284.0	fTotalStrokes	 12
U+4284.0	fCangjie	MNFD
U+4284.0	fFourCorner	17900
U+4284.0	fCantonese	kung4
U+4284.0	fMandarin	qiong2
U+4284.0	fEnglish	polished rice
U+4285.0	fUTF8	䊅
U+4285.0	fCNS11643	3-3D7C
U+4285.0	fR/S	 119.6
U+4285.0	fTotalStrokes	 12
U+4285.0	fCangjie	FDNIR
U+4285.0	fFourCorner	97960
U+4285.0	fCantonese	ming4; maat6
U+4285.0	fMandarin	ming2
U+4285.0	fEnglish	 soak rice
U+4286.0	fUTF8	䊆
U+4286.0	fCNS11643	4-397D
U+4286.0	fGB	3-6545
U+4286.0	fR/S	 119.6
U+4286.0	fTotalStrokes	 12
U+4286.0	fCangjie	HXFD
U+4286.0	fFourCorner	77900
U+4286.0	fCantonese	kau5; kaau3
U+4286.0	fMandarin	jiu4; you3; qiu3
U+4286.0	fEnglish	[1] cooked [2] dry rice flour [3] same as &#x7CD7;
U+4287.0	fUTF8	䊇
U+4287.0	fCNS11643	3-435A
U+4287.0	fR/S	 119.7
U+4287.0	fTotalStrokes	 13
U+4287.0	fCangjie	FDIJB
U+4287.0	fFourCorner	93920
U+4287.0	fHakka	 pu2
U+4287.0	fCantonese	bou1; bou6
U+4287.0	fMandarin	bu4
U+4287.0	fEnglish	[1] same as &#x9914; [2] interchangeable with &#x6661;
U+4288.0	fUTF8	䊈
U+4288.0	fCNS11643	3-4356
U+4288.0	fR/S	 119.7
U+4288.0	fTotalStrokes	 13
U+4288.0	fCangjie	FDOWY
U+4288.0	fFourCorner	98950
U+4288.0	fHakka	 moi2; moe2
U+4288.0	fCantonese	mui4
U+4288.0	fMandarin	mei2
U+4288.0	fEnglish	leaven used in fermenting
U+4289.0	fUTF8	䊉
U+4289.0	fCNS11643	3-4357
U+4289.0	fGB	5-666B
U+4289.0	fR/S	 119.7
U+4289.0	fTotalStrokes	 13
U+4289.0	fCangjie	FDJD
U+4289.0	fFourCorner	93990
U+4289.0	fCantonese	sam2
U+4289.0	fMandarin	san3
U+4289.0	fEnglish	preserved fruit, cucumbers, gourds, etc.
U+428A.0	fUTF8	䊊
U+428A.0	fCNS11643	3-4359
U+428A.0	fR/S	 119.7
U+428A.0	fTotalStrokes	 13
U+428A.0	fCangjie	FDSHU
U+428A.0	fFourCorner	97910
U+428A.0	fCantonese	mei6
U+428A.0	fMandarin	wei4; mei4
U+428A.0	fEnglish	well-boiled congee or rice gruel
U+428B.0	fUTF8	䊋
U+428B.0	fCNS11643	3-435B
U+428B.0	fR/S	 119.7
U+428B.0	fTotalStrokes	 13
U+428B.0	fCangjie	FDVMG
U+428B.0	fFourCorner	94910
U+428B.0	fCantonese	zong1
U+428B.0	fMandarin	zhuang1
U+428B.0	fEnglish	same as &#x599D;
U+428C.0	fUTF8	䊌
U+428C.0	fGB	S-243D
U+428C.0	fR/S	 119.7
U+428C.0	fTotalStrokes	 13
U+428C.0	fCangjie	FDIPP
U+428C.0	fCantonese	bei3
U+428D.0	fUTF8	䊍
U+428D.0	fCNS11643	3-4863
U+428D.0	fR/S	 119.8
U+428D.0	fTotalStrokes	 14
U+428D.0	fCangjie	HHFD
U+428D.0	fFourCorner	27900
U+428D.0	fCantonese	lei4; ci1
U+428D.0	fMandarin	li2
U+428D.0	fEnglish	[1] thick congee or porridge; well-boiled congee or gruel [2] same as &#x9ECE;
U+428E.0	fUTF8	䊎
U+428E.0	fCNS11643	4-4572
U+428E.0	fGB	3-677A
U+428E.0	fR/S	 119.8
U+428E.0	fTotalStrokes	 14
U+428E.0	fCangjie	FDFQU
U+428E.0	fFourCorner	99910
U+428E.0	fCantonese	hyun2; hyun3
U+428E.0	fMandarin	quan3; qun3
U+428E.0	fEnglish	[1] powder; flour [2] roll round with the hand [3] thick congee or porridge
U+428F.0	fUTF8	䊏
U+428F.0	fCNS11643	3-4864
U+428F.0	fR/S	 119.8
U+428F.0	fTotalStrokes	 14
U+428F.0	fCangjie	FDFF
U+428F.0	fFourCorner	99980
U+428F.0	fCantonese	sam2
U+428F.0	fMandarin	san3
U+428F.0	fEnglish	non-classical form of &#x7CC2;, &#x7CDD;
U+4290.0	fUTF8	䊐
U+4290.0	fCNS11643	3-4865
U+4290.0	fR/S	 119.8
U+4290.0	fTotalStrokes	 14
U+4290.0	fCangjie	FDAPP
U+4290.0	fFourCorner	96910
U+4290.0	fCantonese	wan4
U+4290.0	fMandarin	hun2; en4; hua2; huan4
U+4290.0	fEnglish	same as &#x991B;
U+4291.0	fUTF8	䊑
U+4291.0	fCNS11643	6-602C
U+4291.0	fR/S	 119.8
U+4291.0	fTotalStrokes	 14
U+4291.0	fCangjie	FDFBR
U+4291.0	fFourCorner	99920
U+4291.0	fCantonese	hoeng2
U+4291.0	fMandarin	xiang3
U+4291.0	fEnglish	same as &#x9909;, &#x995F;
U+4292.0	fUTF8	䊒
U+4292.0	fCNS11643	3-4866
U+4292.0	fR/S	 119.8
U+4292.0	fTotalStrokes	 14
U+4292.0	fCangjie	FDOG
U+4292.0	fFourCorner	90910
U+4292.0	fCantonese	zing1
U+4292.0	fMandarin	jing1
U+4292.0	fEnglish	same as &#x7CBE;
U+4293.0	fUTF8	䊓
U+4293.0	fCNS11643	3-4D6B
U+4293.0	fGB	5-6676
U+4293.0	fR/S	 119.9
U+4293.0	fTotalStrokes	 15
U+4293.0	fCangjie	FDAMO
U+4293.0	fFourCorner	96980
U+4293.0	fCantonese	si6; zi6
U+4293.0	fMandarin	shi4; zhi4
U+4293.0	fEnglish	 paste up; attach to; stickup; glue
U+4294.0	fUTF8	䊔
U+4294.0	fCNS11643	3-4D6C
U+4294.0	fR/S	 119.9
U+4294.0	fTotalStrokes	 15
U+4294.0	fCangjie	FDTLK
U+4294.0	fFourCorner	94930
U+4294.0	fCantonese	jing1; zing1
U+4294.0	fMandarin	ying1
U+4294.0	fEnglish	polished rice; refined rice
U+4295.0	fUTF8	䊕
U+4295.0	fCNS11643	3-4D69
U+4295.0	fGB	5-666F
U+4295.0	fR/S	 119.9
U+4295.0	fTotalStrokes	 15
U+4295.0	fCangjie	FDNKQ
U+4295.0	fFourCorner	95940
U+4295.0	fCantonese	zin1; gin1
U+4295.0	fMandarin	jian1
U+4295.0	fEnglish	same as &#x9958;
U+4296.0	fUTF8	䊖
U+4296.0	fCNS11643	3-4D6A
U+4296.0	fR/S	 119.9
U+4296.0	fTotalStrokes	 15
U+4296.0	fCangjie	FDJBJ
U+4296.0	fFourCorner	94920
U+4296.0	fCantonese	naam5; laam5 (also); faat1; nam5
U+4296.0	fMandarin	nan3; lan3
U+4296.0	fEnglish	 eat rice-gruel mixed with meat
U+4297.0	fUTF8	䊗
U+4297.0	fCNS11643	4-4C37
U+4297.0	fGB	3-677B
U+4297.0	fR/S	 119.9
U+4297.0	fTotalStrokes	 15
U+4297.0	fCangjie	FDHAG
U+4297.0	fFourCorner	96910
U+4297.0	fCantonese	wong4
U+4297.0	fMandarin	huang2; huang3
U+4297.0	fEnglish	[1] a kind of grain; yellow color; not sticky [2] same as &#x992D;
U+4298.0	fUTF8	䊘
U+4298.0	fCNS11643	7-2657
U+4298.0	fR/S	 119.9
U+4298.0	fTotalStrokes	 15
U+4298.0	fCangjie	FDLWB
U+4298.0	fFourCorner	95920
U+4298.0	fHakka	 ciu5; ciu6
U+4298.0	fCantonese	zau3; zaau6
U+4298.0	fMandarin	jiu4
U+4298.0	fEnglish	rice fully ripe
U+4299.0	fUTF8	䊙
U+4299.0	fCNS11643	4-4C35
U+4299.0	fGB	5-6674
U+4299.0	fR/S	 119.9
U+4299.0	fTotalStrokes	 15
U+4299.0	fCangjie	MTFD
U+4299.0	fFourCorner	11900
U+4299.0	fCantonese	jin4; hin2; zin4; zin6
U+4299.0	fMandarin	yan2
U+4299.0	fEnglish	[1] cooked; well-done [2] ripe [3] fine rice
U+429A.0	fUTF8	䊚
U+429A.0	fCNS11643	3-5236
U+429A.0	fGB	5-6678
U+429A.0	fR/S	 119.10
U+429A.0	fTotalStrokes	 16
U+429A.0	fCangjie	FDYHR
U+429A.0	fFourCorner	97930
U+429A.0	fHakka	 chui1; zui1
U+429A.0	fCantonese	deoi1; zeoi1
U+429A.0	fMandarin	dui1; chui2
U+429A.0	fEnglish	[1] dumpling [2] stick
U+429B.0	fUTF8	䊛
U+429B.0	fCNS11643	4-576D
U+429B.0	fGB	5-6667
U+429B.0	fR/S	 119.10
U+429B.0	fTotalStrokes	 16
U+429B.0	fCangjie	FDKDE
U+429B.0	fFourCorner	97940
U+429B.0	fCantonese	saat3
U+429B.0	fMandarin	sa4; sa3
U+429B.0	fEnglish	shreds; splinters (of rice)
U+429C.0	fUTF8	䊜
U+429C.0	fCNS11643	3-5573
U+429C.0	fGB	5-6656
U+429C.0	fR/S	 119.11
U+429C.0	fTotalStrokes	 17
U+429C.0	fCangjie	FDJII
U+429C.0	fFourCorner	95940
U+429C.0	fHakka	 ton2
U+429C.0	fCantonese	tyun4
U+429C.0	fMandarin	tuan2
U+429C.0	fEnglish	dumplings
U+429D.0	fUTF8	䊝
U+429D.0	fCNS11643	3-5572
U+429D.0	fGB	5-6725
U+429D.0	fR/S	 119.11
U+429D.0	fTotalStrokes	 17
U+429D.0	fCangjie	FDHDP
U+429D.0	fFourCorner	92930
U+429D.0	fCantonese	sit3
U+429D.0	fMandarin	xi1; xie4; su4
U+429D.0	fEnglish	 exile; banish
U+429E.0	fUTF8	䊞
U+429E.0	fCNS11643	4-5769
U+429E.0	fGB	3-6823
U+429E.0	fR/S	 119.11
U+429E.0	fTotalStrokes	 17; 16
U+429E.0	fCangjie	FDYCB
U+429E.0	fFourCorner	90920
U+429E.0	fCantonese	zaak6
U+429E.0	fMandarin	zhai1; zhe2; ti4; zhe4; zhai2
U+429E.0	fEnglish	[1] roll round with the hand; paste up; attach to; stick up; glue [2] poor quality of rice
U+429F.0	fUTF8	䊟
U+429F.0	fCNS11643	4-576A
U+429F.0	fGB	3-677E
U+429F.0	fR/S	 119.11
U+429F.0	fTotalStrokes	 17; 16
U+429F.0	fCangjie	FDTLB
U+429F.0	fFourCorner	94920
U+429F.0	fHakka	 man2
U+429F.0	fCantonese	mun4
U+429F.0	fMandarin	man2; men2
U+429F.0	fEnglish	cold congee or rice-gruel
U+42A0.0	fUTF8	䊠
U+42A0.0	fCNS11643	3-4D74
U+42A0.0	fR/S	 119.11; 119.9
U+42A0.0	fTotalStrokes	 15; 17
U+42A0.0	fCangjie	AUFD
U+42A0.0	fFourCorner	71900
U+42A0.0	fCantonese	hei3
U+42A0.0	fMandarin	xi4
U+42A0.0	fEnglish	same as &#x6C23;, &#x993C;
U+42A1.0	fUTF8	䊡
U+42A1.0	fCNS11643	4-576B
U+42A1.0	fGB	5-6724
U+42A1.0	fR/S	 119.11
U+42A1.0	fTotalStrokes	 17; 16
U+42A1.0	fCangjie	FDAWE
U+42A1.0	fFourCorner	96940
U+42A1.0	fCantonese	mun4
U+42A1.0	fMandarin	man2
U+42A1.0	fEnglish	same as non-classical form &#x9945;
U+42A2.0	fUTF8	䊢
U+42A2.0	fR/S	 119.11
U+42A2.0	fTotalStrokes	 17
U+42A2.0	fCangjie	VIFD
U+42A2.0	fHakka	 ziong1
U+42A2.0	fCantonese	zoeng1
U+42A2.0	fEnglish	paste; starch; thick fluid
U+42A3.0	fUTF8	䊣
U+42A3.0	fCNS11643	5-634E
U+42A3.0	fR/S	 119.12
U+42A3.0	fTotalStrokes	 18
U+42A3.0	fCangjie	FDTMC
U+42A3.0	fFourCorner	94980
U+42A3.0	fCantonese	wong4
U+42A3.0	fMandarin	huang2
U+42A3.0	fEnglish	[1] light yellow dust-like fungoid growth on wine, etc., barley, chaff or husks of wheat [2] non-classical form of &#x992D;
U+42A4.0	fUTF8	䊤
U+42A4.0	fCNS11643	4-5C56
U+42A4.0	fGB	3-6825
U+42A4.0	fR/S	 119.12
U+42A4.0	fTotalStrokes	 18
U+42A4.0	fCangjie	FDMWJ
U+42A4.0	fFourCorner	91940
U+42A4.0	fCantonese	taam4
U+42A4.0	fMandarin	tan2; dan4
U+42A4.0	fEnglish	[1] vegetable mixed with thick soup (broth) [2] congee; gruel
U+42A5.0	fUTF8	䊥
U+42A5.0	fCNS11643	3-5852
U+42A5.0	fR/S	 119.12
U+42A5.0	fTotalStrokes	 18
U+42A5.0	fCangjie	FDLX
U+42A5.0	fFourCorner	95920
U+42A5.0	fCantonese	siu3
U+42A5.0	fMandarin	xiao4
U+42A5.0	fEnglish	[1] congee; porridge; rice gruel [2] mashed; rotten
U+42A6.0	fUTF8	䊦
U+42A6.0	fCNS11643	4-5C55
U+42A6.0	fGB	3-6824
U+42A6.0	fR/S	 119.12
U+42A6.0	fTotalStrokes	 18
U+42A6.0	fCangjie	FDGBT
U+42A6.0	fFourCorner	94910
U+42A6.0	fCantonese	jit3; zit3
U+42A6.0	fMandarin	ye1; ye4; ya2
U+42A6.0	fEnglish	food (some food as glutinouss rice tamale - made by wrapping the rice in broad leaves of reeds and boiled for a few hours - usually with other ingredients, as dates, meat, oyster, beams, etc.
U+42A7.0	fUTF8	䊧
U+42A7.0	fCNS11643	4-5C54
U+42A7.0	fGB	5-667B
U+42A7.0	fR/S	 119.12
U+42A7.0	fTotalStrokes	 18
U+42A7.0	fCangjie	FDLNC
U+42A7.0	fFourCorner	95980
U+42A7.0	fCantonese	pei3; baai3
U+42A7.0	fMandarin	fei4; pi4; bi4
U+42A7.0	fEnglish	same as &#x5C41;
U+42A8.0	fUTF8	䊨
U+42A8.0	fCNS11643	5-6354
U+42A8.0	fR/S	 119.12
U+42A8.0	fTotalStrokes	 18
U+42A8.0	fCangjie	YNFD
U+42A8.0	fFourCorner	00210
U+42A8.0	fCantonese	lo4
U+42A8.0	fMandarin	luo2
U+42A8.0	fEnglish	storing grains; to store up food
U+42A9.0	fUTF8	䊩
U+42A9.0	fCNS11643	3-5853
U+42A9.0	fR/S	 119.12
U+42A9.0	fTotalStrokes	 18
U+42A9.0	fCangjie	FDHDW
U+42A9.0	fFourCorner	92960
U+42A9.0	fCantonese	faan4
U+42A9.0	fMandarin	fan2; fan4
U+42A9.0	fEnglish	rice gravy
U+42AA.0	fUTF8	䊪
U+42AA.0	fCNS11643	3-5A70
U+42AA.0	fGB	5-6653
U+42AA.0	fR/S	 119.13
U+42AA.0	fTotalStrokes	 19
U+42AA.0	fCangjie	FDTWB
U+42AA.0	fFourCorner	94920
U+42AA.0	fCantonese	lai6
U+42AA.0	fMandarin	lai4; li4; la4
U+42AA.0	fEnglish	[1] coarse rice - unhulled [2] interchangeable with &#x7CF2;
U+42AB.0	fUTF8	䊫
U+42AB.0	fCNS11643	4-605B
U+42AB.0	fGB	3-6828
U+42AB.0	fR/S	 119.13
U+42AB.0	fTotalStrokes	 19
U+42AB.0	fCangjie	FDWLY
U+42AB.0	fFourCorner	96910
U+42AB.0	fCantonese	ceoi2; ceoi1
U+42AB.0	fMandarin	cui3; mi2
U+42AB.0	fEnglish	[1] coarse rice [2] red rice
U+42AC.0	fUTF8	䊬
U+42AC.0	fCNS11643	4-605D
U+42AC.0	fGB	3-6829
U+42AC.0	fR/S	 119.13
U+42AC.0	fTotalStrokes	 19
U+42AC.0	fCangjie	HDFD
U+42AC.0	fFourCorner	26900
U+42AC.0	fCantonese	caa2; caa3
U+42AC.0	fMandarin	cha3; cha4
U+42AC.0	fEnglish	[1] the name of a variety of grain [2] hollyhock; the mallow; root and flower can be used as medicine to neutralize poison; antiphretic (febrifuge)
U+42AD.0	fUTF8	䊭
U+42AD.0	fCNS11643	4-6423
U+42AD.0	fGB	3-6779
U+42AD.0	fR/S	 119.14
U+42AD.0	fTotalStrokes	 20
U+42AD.0	fCangjie	FDGNI
U+42AD.0	fFourCorner	94940
U+42AD.0	fCantonese	dou6
U+42AD.0	fMandarin	dao4; chou2
U+42AD.0	fEnglish	[1] overturn; pour out [2] respond [3] examine carefully [4] stick [5] thick congee
U+42AE.0	fUTF8	䊮
U+42AE.0	fCNS11643	3-5C68
U+42AE.0	fGB	5-6730
U+42AE.0	fR/S	 119.14
U+42AE.0	fTotalStrokes	 20
U+42AE.0	fCangjie	FDSMG
U+42AE.0	fFourCorner	97910
U+42AE.0	fHakka	 jau5
U+42AE.0	fCantonese	dik6; dek6
U+42AE.0	fMandarin	di2; tiao4; zhe2; zhe4
U+42AE.0	fEnglish	the name of a variety of grain
U+42AF.0	fUTF8	䊯
U+42AF.0	fCNS11643	7-543A
U+42AF.0	fR/S	 119.15
U+42AF.0	fTotalStrokes	 21
U+42AF.0	fCangjie	FDITC
U+42AF.0	fFourCorner	90980
U+42AF.0	fCantonese	kong3; kong4; kwong3
U+42AF.0	fMandarin	gong3; kuang4
U+42AF.0	fEnglish	same as &#x7A6C;
U+42B0.0	fUTF8	䊰
U+42B0.0	fCNS11643	5-7070
U+42B0.0	fR/S	 119.16; 119.15
U+42B0.0	fTotalStrokes	 21; 22
U+42B0.0	fCangjie	FDYRA
U+42B0.0	fFourCorner	94960
U+42B0.0	fCantonese	cyu4; cyu5
U+42B0.0	fMandarin	chu2; chu3
U+42B0.0	fEnglish	foodstuff; provisions; grains for human consumption
U+42B1.0	fUTF8	䊱
U+42B1.0	fCNS11643	4-6A65
U+42B1.0	fGB	3-682C
U+42B1.0	fR/S	 119.17
U+42B1.0	fTotalStrokes	 23
U+42B1.0	fCangjie	FDOIM
U+42B1.0	fFourCorner	93950
U+42B1.0	fCantonese	cim1; sim2; zaam1
U+42B1.0	fMandarin	xian1
U+42B1.0	fEnglish	cakes made of rice-flour
U+42B2.0	fUTF8	䊲
U+42B2.0	fCNS11643	5-7639
U+42B2.0	fGB	3-6061
U+42B2.0	fR/S	 119.17
U+42B2.0	fTotalStrokes	 23
U+42B2.0	fCangjie	MDYKM
U+42B2.0	fFourCorner	10910
U+42B2.0	fCantonese	caan3; can2
U+42B2.0	fMandarin	chan4; chan3
U+42B2.0	fEnglish	[1] hull rice roughly (to make a rush pounding of the rice); coarse rice [2] grind (grains, millet, etc.)
U+42B3.0	fUTF8	䊳
U+42B3.0	fCNS11643	4-6C7A
U+42B3.0	fGB	5-6733
U+42B3.0	fR/S	 119.19
U+42B3.0	fTotalStrokes	 25
U+42B3.0	fCangjie	FDIDY
U+42B3.0	fFourCorner	90910
U+42B3.0	fCantonese	mei4
U+42B3.0	fMandarin	mi2; mo2; mi3
U+42B3.0	fEnglish	[1] break into pieces; smash; completely smashed [2] chips; crumbs [3] refined; polished (rice); unmixed
U+42B4.0	fUTF8	䊴
U+42B4.0	fCNS11643	3-6145
U+42B4.0	fR/S	 119.19
U+42B4.0	fTotalStrokes	 25
U+42B4.0	fCangjie	FDHOO
U+42B4.0	fFourCorner	98980
U+42B4.0	fCantonese	cim3; him3
U+42B4.0	fMandarin	qian4
U+42B4.0	fEnglish	food made of rice-flour
U+42B5.0	fUTF8	䊵
U+42B5.0	fCNS11643	4-275C
U+42B5.0	fGB	3-6C31
U+42B5.0	fR/S	 120.2
U+42B5.0	fTotalStrokes	 8
U+42B5.0	fCangjie	VFKN
U+42B5.0	fFourCorner	24910
U+42B5.0	fCantonese	kau4
U+42B5.0	fMandarin	qiu2
U+42B5.0	fEnglish	[1] young and small [2] same as &#x7D7F;
U+42B6.0	fUTF8	䊶
U+42B6.0	fCNS11643	6-3364
U+42B6.0	fR/S	 120.3
U+42B6.0	fTotalStrokes	 9
U+42B6.0	fCangjie	VFPD
U+42B6.0	fFourCorner	24910
U+42B6.0	fCantonese	zan5; zaan5; zaan6
U+42B6.0	fMandarin	yin3; zhen4
U+42B6.0	fEnglish	same as &#x7D16;
U+42B7.0	fUTF8	䊷
U+42B7.0	fCNS11643	3-2E57
U+42B7.0	fGB	5-6B51
U+42B7.0	fR/S	 120.3
U+42B7.0	fTotalStrokes	 9
U+42B7.0	fCangjie	VFDH
U+42B7.0	fFourCorner	24900
U+42B7.0	fCantonese	zi1
U+42B7.0	fMandarin	zi1
U+42B7.0	fEnglish	[1] same as &#x7DC7; [2] same as &#x7D14;
U+42B8.0	fUTF8	䊸
U+42B8.0	fCNS11643	3-2E55
U+42B8.0	fGB	5-6B50
U+42B8.0	fR/S	 120.3
U+42B8.0	fTotalStrokes	 9
U+42B8.0	fCangjie	VFMMS
U+42B8.0	fFourCorner	21920
U+42B8.0	fCantonese	jyu1; heoi1; zyu1
U+42B8.0	fMandarin	yu1
U+42B8.0	fEnglish	standard form of &#x7D06;
U+42B9.0	fUTF8	䊹
U+42B9.0	fCNS11643	3-2E58
U+42B9.0	fR/S	 120.3
U+42B9.0	fTotalStrokes	 9
U+42B9.0	fCangjie	VFHJ
U+42B9.0	fFourCorner	22940
U+42B9.0	fCantonese	cin1; cim1
U+42B9.0	fMandarin	qian1
U+42B9.0	fEnglish	abbreviated form of &#x7E96;
U+42BA.0	fUTF8	䊺
U+42BA.0	fCNS11643	4-2E78
U+42BA.0	fGB	3-6C38
U+42BA.0	fR/S	 120.4
U+42BA.0	fTotalStrokes	 10
U+42BA.0	fCangjie	VFMVM
U+42BA.0	fFourCorner	21910
U+42BA.0	fCantonese	wu6
U+42BA.0	fMandarin	hu4; gu3
U+42BA.0	fEnglish	an implement to draw or to collect ropes or cords
U+42BB.0	fUTF8	䊻
U+42BB.0	fCNS11643	4-2E7E
U+42BB.0	fGB	3-6C3C
U+42BB.0	fR/S	 120.4
U+42BB.0	fTotalStrokes	 10
U+42BB.0	fCangjie	VFNO
U+42BB.0	fFourCorner	27980
U+42BB.0	fCantonese	gam1; zim1
U+42BB.0	fMandarin	gan1; yan3
U+42BB.0	fEnglish	silky, very fine thread
U+42BC.0	fUTF8	䊼
U+42BC.0	fCNS11643	3-3335
U+42BC.0	fR/S	 120.4
U+42BC.0	fTotalStrokes	 10
U+42BC.0	fCangjie	VFYLM
U+42BC.0	fFourCorner	21910
U+42BC.0	fCantonese	ci2
U+42BC.0	fMandarin	chi3
U+42BC.0	fEnglish	measuring unit; end of spoilted silk
U+42BD.0	fUTF8	䊽
U+42BD.0	fCNS11643	3-3334
U+42BD.0	fGB	5-6B5D
U+42BD.0	fR/S	 120.4
U+42BD.0	fTotalStrokes	 10
U+42BD.0	fCangjie	VFDK
U+42BD.0	fFourCorner	25930
U+42BD.0	fCantonese	gwaai3
U+42BD.0	fMandarin	guai4
U+42BD.0	fEnglish	[1] fine thread [2] linen thread; silk thread; thread; yarn
U+42BE.0	fUTF8	䊾
U+42BE.0	fCNS11643	4-2E7C
U+42BE.0	fGB	5-6B53
U+42BE.0	fR/S	 120.4
U+42BE.0	fTotalStrokes	 10
U+42BE.0	fCangjie	VFD
U+42BE.0	fFourCorner	24990
U+42BE.0	fCantonese	muk6
U+42BE.0	fMandarin	mu4
U+42BE.0	fEnglish	ropes; cords; cables
U+42BF.0	fUTF8	䊿
U+42BF.0	fCNS11643	3-3339
U+42BF.0	fR/S	 120.4
U+42BF.0	fTotalStrokes	 10
U+42BF.0	fCangjie	VFQO
U+42BF.0	fFourCorner	25930
U+42BF.0	fCantonese	fu1
U+42BF.0	fMandarin	fu1; bo2; ku4
U+42BF.0	fEnglish	same as &#x886D;
U+42C0.0	fUTF8	䋀
U+42C0.0	fCNS11643	4-2F21
U+42C0.0	fGB	5-5267
U+42C0.0	fR/S	 120.4
U+42C0.0	fTotalStrokes	 10
U+42C0.0	fCangjie	ISVIF
U+42C0.0	fFourCorner	30293
U+42C0.0	fCantonese	waa5; kwaa3
U+42C0.0	fMandarin	hua4; kua4
U+42C0.0	fEnglish	shoes made of hemp or hair
U+42C1.0	fUTF8	䋁
U+42C1.0	fCNS11643	3-3338
U+42C1.0	fGB	5-6B5B
U+42C1.0	fR/S	 120.4
U+42C1.0	fTotalStrokes	 10
U+42C1.0	fCangjie	VFYHN
U+42C1.0	fFourCorner	20910
U+42C1.0	fCantonese	gang2; au3; gaang2
U+42C1.0	fMandarin	geng3
U+42C1.0	fEnglish	ancient form of &#x7D86;
U+42C2.0	fUTF8	䋂
U+42C2.0	fCNS11643	4-2E7D
U+42C2.0	fGB	5-6B5A
U+42C2.0	fR/S	 120.4
U+42C2.0	fTotalStrokes	 10
U+42C2.0	fCangjie	VFKK
U+42C2.0	fFourCorner	24940
U+42C2.0	fCantonese	haau4; ngau4
U+42C2.0	fMandarin	xiao2; yao2
U+42C2.0	fEnglish	[1] dresses for the bride, green color, yellow color [2] interchangeable with &#x7D5E;
U+42C3.0	fUTF8	䋃
U+42C3.0	fCNS11643	3-3336
U+42C3.0	fR/S	 120.4
U+42C3.0	fTotalStrokes	 10
U+42C3.0	fCangjie	VFHQU
U+42C3.0	fFourCorner	22910
U+42C3.0	fCantonese	mou6
U+42C3.0	fMandarin	mao4
U+42C3.0	fEnglish	stinging; hairy silk fabrics
U+42C4.0	fUTF8	䋄
U+42C4.0	fCNS11643	3-333A
U+42C4.0	fR/S	 120.4
U+42C4.0	fTotalStrokes	 10
U+42C4.0	fCangjie	VFBK
U+42C4.0	fFourCorner	27920
U+42C4.0	fHakka	 miong3
U+42C4.0	fCantonese	mong5; leon4
U+42C4.0	fMandarin	wang3
U+42C4.0	fEnglish	same as &#x7DB2;
U+42C5.0	fUTF8	䋅
U+42C5.0	fR/S	 120.4
U+42C5.0	fTotalStrokes	 10
U+42C5.0	fCangjie	VFHQ
U+42C5.0	fCantonese	ngau4; au4 (also)
U+42C5.0	fEnglish	[Korean character]
U+42C6.0	fUTF8	䋆
U+42C6.0	fR/S	 120.4
U+42C6.0	fTotalStrokes	 10
U+42C6.0	fCangjie	VFIS
U+42C6.0	fCantonese	wu6
U+42C6.0	fEnglish	[Japanese character]
U+42C7.0	fUTF8	䋇
U+42C7.0	fR/S	 120.4
U+42C7.0	fTotalStrokes	 10
U+42C7.0	fCangjie	VFSO
U+42C7.0	fCantonese	jik6
U+42C7.0	fEnglish	[Japanese character]
U+42C8.0	fUTF8	䋈
U+42C8.0	fCNS11643	4-342E
U+42C8.0	fGB	5-6B45
U+42C8.0	fR/S	 120.5
U+42C8.0	fTotalStrokes	 11
U+42C8.0	fCangjie	VEVIF
U+42C8.0	fFourCorner	47900
U+42C8.0	fCantonese	neoi4; leoi4 (also); zyu4
U+42C8.0	fMandarin	nuu2; ru2
U+42C8.0	fEnglish	[1] bind loose hemp, old yarn [2] stuff; fill [3] waste silk or cotton and silk to be laid and attached each other; to padding; cushioning
U+42C9.0	fUTF8	䋉
U+42C9.0	fCNS11643	4-3426
U+42C9.0	fGB	3-6C4E
U+42C9.0	fR/S	 120.5
U+42C9.0	fTotalStrokes	 11
U+42C9.0	fCangjie	VFJC
U+42C9.0	fFourCorner	23980
U+42C9.0	fCantonese	jyut6; kyut3; zyut6
U+42C9.0	fMandarin	xue4; jue2; ke3; xue2
U+42C9.0	fEnglish	interchangeable with &#x42BD;
U+42CA.0	fUTF8	䋊
U+42CA.0	fCNS11643	3-3823
U+42CA.0	fGB	5-6B5F
U+42CA.0	fR/S	 120.5
U+42CA.0	fTotalStrokes	 11
U+42CA.0	fCangjie	VFMYM
U+42CA.0	fFourCorner	21910
U+42CA.0	fCantonese	zing1
U+42CA.0	fMandarin	zheng1
U+42CA.0	fEnglish	decorations of the emperor's carriage; ornamental items on horses
U+42CB.0	fUTF8	䋋
U+42CB.0	fCNS11643	3-382A
U+42CB.0	fGB	5-6B69
U+42CB.0	fR/S	 120.5
U+42CB.0	fTotalStrokes	 11
U+42CB.0	fCangjie	VFRVP
U+42CB.0	fFourCorner	27940
U+42CB.0	fHakka	 min2
U+42CB.0	fCantonese	man4
U+42CB.0	fMandarin	min2
U+42CB.0	fEnglish	a fowler's net; angle
U+42CC.0	fUTF8	䋌
U+42CC.0	fCNS11643	4-3427
U+42CC.0	fGB	3-6C37
U+42CC.0	fR/S	 120.5
U+42CC.0	fTotalStrokes	 11
U+42CC.0	fCangjie	VFSS
U+42CC.0	fFourCorner	21910
U+42CC.0	fCantonese	goeng2; gin1; kin4
U+42CC.0	fMandarin	jian1; jiang3
U+42CC.0	fEnglish	[1] ancient form of &#x5805; [2] same as &#x42D7;
U+42CD.0	fUTF8	䋍
U+42CD.0	fCNS11643	4-3430
U+42CD.0	fGB	5-6B61
U+42CD.0	fR/S	 120.5
U+42CD.0	fTotalStrokes	 11
U+42CD.0	fCangjie	VFMNR
U+42CD.0	fFourCorner	21920
U+42CD.0	fCantonese	ngo1 (also); o1
U+42CD.0	fMandarin	e1
U+42CD.0	fEnglish	abbreviated form of &#x42EA;
U+42CE.0	fUTF8	䋎
U+42CE.0	fCNS11643	3-382C
U+42CE.0	fGB	5-6B62
U+42CE.0	fR/S	 120.5
U+42CE.0	fTotalStrokes	 11
U+42CE.0	fCangjie	VFAM
U+42CE.0	fFourCorner	26910
U+42CE.0	fCantonese	zan2; zan6
U+42CE.0	fMandarin	zhan4
U+42CE.0	fEnglish	[1] mend (clothes, etc.) [2] interchangeable with &#x7DBB;
U+42CF.0	fUTF8	䋏
U+42CF.0	fCNS11643	4-342D
U+42CF.0	fGB	3-6C46
U+42CF.0	fR/S	 120.5
U+42CF.0	fTotalStrokes	 11
U+42CF.0	fCangjie	VFOS
U+42CF.0	fFourCorner	28911
U+42CF.0	fCantonese	zok6
U+42CF.0	fMandarin	zha4; zuo4
U+42CF.0	fEnglish	[1] thick and big ropes or cords; bulky cables [2] spoilt silk
U+42D0.0	fUTF8	䋐
U+42D0.0	fCNS11643	4-342B
U+42D0.0	fGB	3-6C41
U+42D0.0	fR/S	 120.5
U+42D0.0	fTotalStrokes	 11
U+42D0.0	fCangjie	VFIV
U+42D0.0	fFourCorner	23950
U+42D0.0	fCantonese	jyut6; zyut6
U+42D0.0	fMandarin	yue4
U+42D0.0	fEnglish	[1] fabric; textile with patterns used for hem decorations [2] decorations on carriage and horses [3] varicolored silk or fine cloth loose in textture
U+42D1.0	fUTF8	䋑
U+42D1.0	fCNS11643	4-3429
U+42D1.0	fGB	3-6C40
U+42D1.0	fR/S	 120.5
U+42D1.0	fTotalStrokes	 11
U+42D1.0	fCangjie	VFMOB
U+42D1.0	fFourCorner	21920
U+42D1.0	fCantonese	lit6; bing2
U+42D1.0	fMandarin	bing3
U+42D1.0	fEnglish	[1] tie; knot [2] join
U+42D2.0	fUTF8	䋒
U+42D2.0	fCNS11643	3-3828
U+42D2.0	fR/S	 120.5
U+42D2.0	fTotalStrokes	 11
U+42D2.0	fCangjie	VFNIH
U+42D2.0	fFourCorner	27920
U+42D2.0	fCantonese	nou6; sit3; syu1
U+42D2.0	fEnglish	corrupted form of &#x7D13;
U+42D3.0	fUTF8	䋓
U+42D3.0	fCNS11643	3-382D
U+42D3.0	fR/S	 120.5
U+42D3.0	fTotalStrokes	 11
U+42D3.0	fCangjie	VFNSM
U+42D3.0	fFourCorner	27970
U+42D3.0	fCantonese	zau3
U+42D3.0	fMandarin	zhou4
U+42D3.0	fEnglish	abbreviated form of &#x7E10;
U+42D4.0	fUTF8	䋔
U+42D4.0	fCNS11643	3-3825
U+42D4.0	fR/S	 120.5
U+42D4.0	fTotalStrokes	 11
U+42D4.0	fCangjie	VFMFM
U+42D4.0	fFourCorner	21910
U+42D4.0	fCantonese	pei1; bit1
U+42D4.0	fMandarin	bi4
U+42D4.0	fEnglish	 band together; twist; wring and to disperse fiber then to make thin silk, cloth, cotton-yarn or rope
U+42D5.0	fUTF8	䋕
U+42D5.0	fCNS11643	3-3E36
U+42D5.0	fR/S	 120.6
U+42D5.0	fTotalStrokes	 12
U+42D5.0	fCangjie	OGVIF
U+42D5.0	fFourCorner	21900
U+42D5.0	fCantonese	jam6; zaam6; zi1
U+42D5.0	fMandarin	ren2; ren4
U+42D5.0	fEnglish	same as &#x7D4D;, &#x7D1D;
U+42D6.0	fUTF8	䋖
U+42D6.0	fCNS11643	5-3B2C
U+42D6.0	fR/S	 120.6
U+42D6.0	fTotalStrokes	 12
U+42D6.0	fCangjie	VFLQ
U+42D6.0	fFourCorner	25900
U+42D6.0	fCantonese	jyut6; leot6; waat6
U+42D6.0	fMandarin	yu4
U+42D6.0	fEnglish	[1] long [2] gown; a long dress
U+42D7.0	fUTF8	䋗
U+42D7.0	fCNS11643	3-3E2D
U+42D7.0	fR/S	 120.6
U+42D7.0	fTotalStrokes	 12
U+42D7.0	fCangjie	VFSLL
U+42D7.0	fFourCorner	21910
U+42D7.0	fCantonese	gin1; zin2
U+42D7.0	fMandarin	jian1
U+42D7.0	fEnglish	[1] same as &#x42CC; [2] same as &#x5805;
U+42D8.0	fUTF8	䋘
U+42D8.0	fCNS11643	3-3E32
U+42D8.0	fR/S	 120.6
U+42D8.0	fTotalStrokes	 12
U+42D8.0	fCangjie	VFQD
U+42D8.0	fFourCorner	25990
U+42D8.0	fCantonese	zeoi6; teoi4
U+42D8.0	fMandarin	zhui4; chuo4
U+42D8.0	fEnglish	same as &#x7E0B;
U+42D9.0	fUTF8	䋙
U+42D9.0	fCNS11643	3-3E27
U+42D9.0	fR/S	 120.6
U+42D9.0	fTotalStrokes	 12
U+42D9.0	fCangjie	VFSJ
U+42D9.0	fFourCorner	21940
U+42D9.0	fCantonese	ji5; ngaai6; zi5
U+42D9.0	fMandarin	er3
U+42D9.0	fEnglish	reins; bridle
U+42DA.0	fUTF8	䋚
U+42DA.0	fCNS11643	3-3E30
U+42DA.0	fGB	5-6B7E
U+42DA.0	fR/S	 120.6
U+42DA.0	fTotalStrokes	 12
U+42DA.0	fCangjie	VFSMM
U+42DA.0	fFourCorner	27920
U+42DA.0	fCantonese	jik6; zik6
U+42DA.0	fMandarin	yi4
U+42DA.0	fEnglish	same as &#x7FFC;
U+42DB.0	fUTF8	䋛
U+42DB.0	fCNS11643	3-3E25
U+42DB.0	fGB	5-6B7C
U+42DB.0	fR/S	 120.6
U+42DB.0	fTotalStrokes	 12
U+42DB.0	fCangjie	VFFD
U+42DB.0	fFourCorner	29990
U+42DB.0	fCantonese	mai5
U+42DB.0	fMandarin	mi3; mi2
U+42DB.0	fEnglish	the clustered embroidery patterns (as tiny and fine rice)
U+42DC.0	fUTF8	䋜
U+42DC.0	fCNS11643	6-4C71
U+42DC.0	fR/S	 120.6
U+42DC.0	fTotalStrokes	 12
U+42DC.0	fCangjie	HEVIF
U+42DC.0	fFourCorner	37900
U+42DC.0	fCantonese	hing3
U+42DC.0	fMandarin	qi3; qing4
U+42DC.0	fEnglish	same as &#x7DAE;
U+42DD.0	fUTF8	䋝
U+42DD.0	fR/S	 120.6
U+42DD.0	fTotalStrokes	 12
U+42DD.0	fCangjie	VFFV
U+42DD.0	fCantonese	jing1; zing1
U+42DD.0	fEnglish	abbreviated form of &#x7E93;
U+42DE.0	fUTF8	䋞
U+42DE.0	fCNS11643	3-3E31
U+42DE.0	fR/S	 120.6
U+42DE.0	fTotalStrokes	 12
U+42DE.0	fCangjie	VFBKK
U+42DE.0	fFourCorner	27920
U+42DE.0	fCantonese	mong5; maau5; mong4; mou5
U+42DE.0	fMandarin	wang3
U+42DE.0	fEnglish	abbreviated form of &#x7DB2;
U+42DF.0	fUTF8	䋟
U+42DF.0	fCNS11643	4-3F6A
U+42DF.0	fGB	3-6C74
U+42DF.0	fR/S	 120.7
U+42DF.0	fTotalStrokes	 13
U+42DF.0	fCangjie	VFSUP
U+42DF.0	fFourCorner	27930
U+42DF.0	fCantonese	gei6
U+42DF.0	fMandarin	ji4
U+42DF.0	fEnglish	[1] the weight on a steelyard [2] connected stitches
U+42E0.0	fUTF8	䋠
U+42E0.0	fCNS11643	3-4365
U+42E0.0	fR/S	 120.7
U+42E0.0	fTotalStrokes	 13
U+42E0.0	fCangjie	VFIJB
U+42E0.0	fFourCorner	23920
U+42E0.0	fCantonese	bou2
U+42E0.0	fMandarin	bu3
U+42E0.0	fEnglish	interchangeable with &#x88DC;
U+42E1.0	fUTF8	䋡
U+42E1.0	fCNS11643	3-436B
U+42E1.0	fR/S	 120.7
U+42E1.0	fTotalStrokes	 13
U+42E1.0	fCangjie	VFOMD
U+42E1.0	fFourCorner	28990
U+42E1.0	fHakka	 shu1
U+42E1.0	fCantonese	syu1
U+42E1.0	fMandarin	shu1
U+42E1.0	fEnglish	[1] same as &#x7D13; [2] interchangeable with &#x8212;
U+42E2.0	fUTF8	䋢
U+42E2.0	fCNS11643	4-3F6E
U+42E2.0	fGB	3-6C27
U+42E2.0	fR/S	 120.7
U+42E2.0	fTotalStrokes	 13
U+42E2.0	fCangjie	QLVIF
U+42E2.0	fFourCorner	52900
U+42E2.0	fCantonese	pit3; bit3
U+42E2.0	fMandarin	bie4
U+42E2.0	fEnglish	[1] twist ropes [2] a belt or string to hold the bow [3] tie; bind [4] the cart turning back to the left [5] draw or pull (of a cart) [6] mourn
U+42E3.0	fUTF8	䋣
U+42E3.0	fCNS11643	4-3F73
U+42E3.0	fGB	5-6B47
U+42E3.0	fR/S	 120.7
U+42E3.0	fTotalStrokes	 13
U+42E3.0	fCangjie	OYVIF
U+42E3.0	fFourCorner	82590
U+42E3.0	fCantonese	faan4
U+42E3.0	fMandarin	fan2; pan2
U+42E3.0	fEnglish	[1] decorations put on the mane or horsehair [2] standard form of &#x7E41;
U+42E4.0	fUTF8	䋤
U+42E4.0	fCNS11643	4-3F6F
U+42E4.0	fGB	3-6C26
U+42E4.0	fR/S	 120.7
U+42E4.0	fTotalStrokes	 13
U+42E4.0	fCangjie	QFPI
U+42E4.0	fFourCorner	57930
U+42E4.0	fCantonese	joek6; zoek6; zoeng2
U+42E4.0	fMandarin	yue4; yao4
U+42E4.0	fEnglish	[1] plain white silk [2] soften and whiten raw silk by boiling
U+42E5.0	fUTF8	䋥
U+42E5.0	fCNS11643	3-4361
U+42E5.0	fR/S	 120.7
U+42E5.0	fTotalStrokes	 13
U+42E5.0	fCangjie	VFWG
U+42E5.0	fFourCorner	26910
U+42E5.0	fCantonese	lei4; laai6
U+42E5.0	fMandarin	li2
U+42E5.0	fEnglish	lines; stripes; veins
U+42E6.0	fUTF8	䋦
U+42E6.0	fCNS11643	3-435D
U+42E6.0	fR/S	 120.7
U+42E6.0	fTotalStrokes	 13
U+42E6.0	fCangjie	VFOWY
U+42E6.0	fFourCorner	28950
U+42E6.0	fCantonese	faan4
U+42E6.0	fMandarin	fan2
U+42E6.0	fEnglish	ravelled silk
U+42E7.0	fUTF8	䋧
U+42E7.0	fCNS11643	3-486C
U+42E7.0	fR/S	 120.8
U+42E7.0	fTotalStrokes	 14
U+42E7.0	fCangjie	VFSJR
U+42E7.0	fFourCorner	27960
U+42E7.0	fCantonese	keoi4
U+42E7.0	fMandarin	xuan4; qu2
U+42E7.0	fEnglish	same as &#x7D47;
U+42E8.0	fUTF8	䋨
U+42E8.0	fCNS11643	4-4575
U+42E8.0	fGB	5-6C3C
U+42E8.0	fR/S	 120.8
U+42E8.0	fTotalStrokes	 14
U+42E8.0	fCangjie	VFYTR
U+42E8.0	fFourCorner	20960
U+42E8.0	fCantonese	fu2; fu3
U+42E8.0	fMandarin	fu3
U+42E8.0	fEnglish	[1] set in order the old [2] raw, coarse, waste silk or cotton
U+42E9.0	fUTF8	䋩
U+42E9.0	fCNS11643	4-457E
U+42E9.0	fGB	3-6C7E
U+42E9.0	fR/S	 120.8
U+42E9.0	fTotalStrokes	 14
U+42E9.0	fCangjie	VFHXU
U+42E9.0	fFourCorner	27910
U+42E9.0	fCantonese	ji4; zi4
U+42E9.0	fMandarin	er2
U+42E9.0	fEnglish	pretty and fine silk fabrics
U+42EA.0	fUTF8	䋪
U+42EA.0	fCNS11643	4-4579
U+42EA.0	fGB	5-6C2C
U+42EA.0	fR/S	 120.8
U+42EA.0	fTotalStrokes	 14
U+42EA.0	fCangjie	VFNLR
U+42EA.0	fFourCorner	21920
U+42EA.0	fCantonese	ngo1 (also); o1
U+42EA.0	fMandarin	e1
U+42EA.0	fEnglish	interchangeable with &#x42CD;
U+42EB.0	fUTF8	䋫
U+42EB.0	fCNS11643	4-4621
U+42EB.0	fGB	3-6C61
U+42EB.0	fR/S	 120.8; 167.8
U+42EB.0	fTotalStrokes	 14
U+42EB.0	fCangjie	VFNSD
U+42EB.0	fFourCorner	22950
U+42EB.0	fHakka	 zen1
U+42EB.0	fCantonese	zang1
U+42EB.0	fMandarin	zheng1
U+42EB.0	fEnglish	[1] wind round the ropes [2] crooked; winding; bends; turns; curves; turns and twists [3] play a sound of hasty (fast) string (of a musical instrument)
U+42EC.0	fUTF8	䋬
U+42EC.0	fCNS11643	3-4867
U+42EC.0	fGB	5-6C2D
U+42EC.0	fR/S	 120.8
U+42EC.0	fTotalStrokes	 14
U+42EC.0	fCangjie	VFMKP
U+42EC.0	fFourCorner	22930
U+42EC.0	fCantonese	tim1; zim1
U+42EC.0	fMandarin	tian1
U+42EC.0	fEnglish	[1] rug; carpet; blanket [2] woolen textiles; woolen goods; woolen stuff; woolen fabrics
U+42ED.0	fUTF8	䋭
U+42ED.0	fCNS11643	3-435F
U+42ED.0	fGB	5-6C3E
U+42ED.0	fR/S	 120.8; 120.7
U+42ED.0	fTotalStrokes	 13; 14
U+42ED.0	fCangjie	VFYIB
U+42ED.0	fFourCorner	24920
U+42ED.0	fCantonese	juk6; zuk6; zung2
U+42ED.0	fMandarin	huo4; yu4
U+42ED.0	fEnglish	[1] weave silk fabrics with green color for longitude and white for latitude [2] silk fabrics from Yuyang
U+42EE.0	fUTF8	䋮
U+42EE.0	fCNS11643	3-4872
U+42EE.0	fR/S	 120.8
U+42EE.0	fTotalStrokes	 14
U+42EE.0	fCangjie	VFC
U+42EE.0	fFourCorner	28910
U+42EE.0	fCantonese	gam1; gaam1; kim4
U+42EE.0	fMandarin	jin1; jin4; qian2
U+42EE.0	fEnglish	[1] large seal (a type of Chinese calligraphy) [2] same as &#x7D1F;
U+42EF.0	fUTF8	䋯
U+42EF.0	fCNS11643	3-4875
U+42EF.0	fR/S	 120.8
U+42EF.0	fTotalStrokes	 14
U+42EF.0	fCangjie	IIVIF
U+42EF.0	fFourCorner	33900
U+42EF.0	fCantonese	hing3; kaai2
U+42EF.0	fMandarin	qi3; qing4
U+42EF.0	fEnglish	same as &#x7DAE;
U+42F0.0	fUTF8	䋰
U+42F0.0	fCNS11643	4-4623
U+42F0.0	fGB	3-6C28
U+42F0.0	fR/S	 120.8
U+42F0.0	fTotalStrokes	 14
U+42F0.0	fCangjie	BUKVF
U+42F0.0	fFourCorner	60900
U+42F0.0	fCantonese	guk1; guk6
U+42F0.0	fMandarin	ju1; ju2
U+42F0.0	fEnglish	[1] bind or restrain; restraint; restriction [2] timid and awkward [3] wind around; tangle [4] connect; join [5] together with
U+42F1.0	fUTF8	䋱
U+42F1.0	fCNS11643	5-4854
U+42F1.0	fR/S	 120.8
U+42F1.0	fTotalStrokes	 14
U+42F1.0	fCangjie	VFDOO
U+42F1.0	fFourCorner	24990
U+42F1.0	fCantonese	lei4; loi4
U+42F1.0	fMandarin	lai2
U+42F1.0	fEnglish	same as &#x6584;
U+42F2.0	fUTF8	䋲
U+42F2.0	fCNS11643	3-486F
U+42F2.0	fR/S	 120.8
U+42F2.0	fTotalStrokes	 14
U+42F2.0	fCangjie	VFRLU
U+42F2.0	fFourCorner	26910
U+42F2.0	fCantonese	sing4; mang5; zaan6
U+42F2.0	fMandarin	sheng2; che3; xing3; zhe4
U+42F2.0	fEnglish	non-classical form of &#x7E69;
U+42F3.0	fUTF8	䋳
U+42F3.0	fCNS11643	3-4E2A
U+42F3.0	fR/S	 120.9
U+42F3.0	fTotalStrokes	 15
U+42F3.0	fCangjie	VFLPB
U+42F3.0	fFourCorner	21920
U+42F3.0	fHakka	 boi5; poi5; poi6; poe5
U+42F3.0	fCantonese	bui3
U+42F3.0	fMandarin	bei4; mi4
U+42F3.0	fEnglish	fine silk of many colors
U+42F4.0	fUTF8	䋴
U+42F4.0	fCNS11643	5-4F53
U+42F4.0	fR/S	 120.9
U+42F4.0	fTotalStrokes	 15
U+42F4.0	fCangjie	VFNHD
U+42F4.0	fFourCorner	27990
U+42F4.0	fCantonese	nau6; lau6 (also); zaau2
U+42F4.0	fMandarin	niu4; rong3; rou2; ru3
U+42F4.0	fEnglish	silk fabrics with mixed colors
U+42F5.0	fUTF8	䋵
U+42F5.0	fCNS11643	4-4C3E
U+42F5.0	fGB	3-6D2F
U+42F5.0	fR/S	 120.9
U+42F5.0	fTotalStrokes	 15
U+42F5.0	fCangjie	VFAPV
U+42F5.0	fFourCorner	26920
U+42F5.0	fCantonese	ai3; ngai3 (also); zi3; zit3
U+42F5.0	fMandarin	yi4
U+42F5.0	fEnglish	[1] quick [2] urgent [3] anxious [4] not going to succeed [5] poor quality of silk fabrics
U+42F6.0	fUTF8	䋶
U+42F6.0	fCNS11643	4-4C3C
U+42F6.0	fGB	3-6C54
U+42F6.0	fR/S	 120.9
U+42F6.0	fTotalStrokes	 15
U+42F6.0	fCangjie	VFMBC
U+42F6.0	fFourCorner	21980
U+42F6.0	fCantonese	seoi1
U+42F6.0	fMandarin	xu1; xu4
U+42F6.0	fEnglish	[1] stumble; trip the front feet of a beast [2] a kind of brocade from Shu
U+42F7.0	fUTF8	䋷
U+42F7.0	fCNS11643	4-4C40
U+42F7.0	fGB	3-6C29
U+42F7.0	fR/S	 120.9
U+42F7.0	fTotalStrokes	 15
U+42F7.0	fCangjie	NKVIF
U+42F7.0	fFourCorner	18900
U+42F7.0	fHakka	 meu3
U+42F7.0	fCantonese	mau4; laau4; maau4
U+42F7.0	fMandarin	mou2; liu2
U+42F7.0	fEnglish	a kind of thick, loosely-woven raw silk fabric
U+42F8.0	fUTF8	䋸
U+42F8.0	fCNS11643	3-4D7B
U+42F8.0	fR/S	 120.9
U+42F8.0	fTotalStrokes	 15
U+42F8.0	fCangjie	VFHJU
U+42F8.0	fFourCorner	22960
U+42F8.0	fCantonese	ceon4
U+42F8.0	fMandarin	xun2
U+42F8.0	fEnglish	back center suture of the clothes
U+42F9.0	fUTF8	䋹
U+42F9.0	fCNS11643	3-4E21
U+42F9.0	fR/S	 120.9
U+42F9.0	fTotalStrokes	 15
U+42F9.0	fCangjie	VFMRW
U+42F9.0	fFourCorner	21960
U+42F9.0	fHakka	 fuk7
U+42F9.0	fCantonese	fuk1
U+42F9.0	fMandarin	fu2; fu4
U+42F9.0	fEnglish	same as &#x5E45;
U+42FA.0	fUTF8	䋺
U+42FA.0	fCNS11643	3-4E2D
U+42FA.0	fGB	5-6C47
U+42FA.0	fR/S	 120.9
U+42FA.0	fTotalStrokes	 15
U+42FA.0	fCangjie	VFHDF
U+42FA.0	fFourCorner	29980
U+42FA.0	fHakka	 ciu2
U+42FA.0	fCantonese	cau1
U+42FA.0	fMandarin	qiu1
U+42FA.0	fEnglish	[1] same as &#x97A6; [2] same as &#x97A7;, &#x7DE7;
U+42FB.0	fUTF8	䋻
U+42FB.0	fCNS11643	3-4D76
U+42FB.0	fGB	5-6C41
U+42FB.0	fR/S	 120.9
U+42FB.0	fTotalStrokes	 15
U+42FB.0	fCangjie	VFJBJ
U+42FB.0	fFourCorner	24920
U+42FB.0	fCantonese	nam4; lam4 (also); nei5
U+42FB.0	fMandarin	nin2
U+42FB.0	fEnglish	[1] spin and weave [2] equal; uniform [3] interchangeable with &#x7D1D;
U+42FC.0	fUTF8	䋼
U+42FC.0	fCNS11643	3-4D79
U+42FC.0	fGB	5-6C53
U+42FC.0	fR/S	 120.9
U+42FC.0	fTotalStrokes	 15
U+42FC.0	fCangjie	VFNST
U+42FC.0	fFourCorner	27910
U+42FC.0	fCantonese	ting1
U+42FC.0	fMandarin	ying2; ting3
U+42FC.0	fEnglish	[1] slow; leisurely [2] delay; slacken [3] silk ribbons
U+42FD.0	fUTF8	䋽
U+42FD.0	fCNS11643	4-4C3A
U+42FD.0	fGB	3-6D2A
U+42FD.0	fR/S	 120.9
U+42FD.0	fTotalStrokes	 15
U+42FD.0	fCangjie	VFGGI
U+42FD.0	fFourCorner	24900
U+42FD.0	fCantonese	bung2; pung2
U+42FD.0	fMandarin	beng3; peng3
U+42FD.0	fEnglish	[1] hempen sandals [2] leather shoes (for children)
U+42FE.0	fUTF8	䋾
U+42FE.0	fCNS11643	4-4C3D
U+42FE.0	fGB	3-6D2D
U+42FE.0	fR/S	 120.9
U+42FE.0	fTotalStrokes	 15
U+42FE.0	fCangjie	VFKNN
U+42FE.0	fFourCorner	24920
U+42FE.0	fCantonese	zaa2; naa5; zap3
U+42FE.0	fMandarin	zha3
U+42FE.0	fEnglish	cotton and silk to be laid and attached each other
U+42FF.0	fUTF8	䋿
U+42FF.0	fCNS11643	4-4C3F
U+42FF.0	fGB	3-6D30
U+42FF.0	fR/S	 120.9
U+42FF.0	fTotalStrokes	 15
U+42FF.0	fCangjie	VFWMV
U+42FF.0	fFourCorner	26930
U+42FF.0	fCantonese	wui1
U+42FF.0	fMandarin	wei1
U+42FF.0	fEnglish	a decorative knot made of colorful silk (blue, yellow, red, white, and black)
U+4300.0	fUTF8	䌀
U+4300.0	fCNS11643	3-4D7A
U+4300.0	fR/S	 120.9
U+4300.0	fTotalStrokes	 15
U+4300.0	fCangjie	VFHDJ
U+4300.0	fFourCorner	24900
U+4300.0	fCantonese	fo1
U+4300.0	fMandarin	ke1
U+4300.0	fEnglish	[1] arrange; repair silk [2] colored patterns (stripes, lines, streaks, veins)
U+4301.0	fUTF8	䌁
U+4301.0	fCNS11643	4-4C44
U+4301.0	fR/S	 120.9
U+4301.0	fTotalStrokes	 15
U+4301.0	fCangjie	VFMWV
U+4301.0	fFourCorner	21940
U+4301.0	fCantonese	jiu1; zaau4; zaau6; ziu1
U+4301.0	fEnglish	[1] pleat; fold [2] a piece of string; ribbon used to tie clothes [3] rope; line; cord
U+4302.0	fUTF8	䌂
U+4302.0	fCNS11643	5-4F54
U+4302.0	fR/S	 120.9
U+4302.0	fTotalStrokes	 15
U+4302.0	fCangjie	VFSMG
U+4302.0	fFourCorner	27910
U+4302.0	fCantonese	ak1; ngak1 (also); aau3
U+4302.0	fMandarin	ou4
U+4302.0	fEnglish	same as &#x63E1;
U+4303.0	fUTF8	䌃
U+4303.0	fCNS11643	3-4E26
U+4303.0	fGB	5-6C45
U+4303.0	fR/S	 120.9
U+4303.0	fTotalStrokes	 15
U+4303.0	fCangjie	VFFBN
U+4303.0	fFourCorner	22900
U+4303.0	fCantonese	siu1; sok3
U+4303.0	fMandarin	xiao1; shuo4
U+4303.0	fEnglish	same as &#x7D83;
U+4304.0	fUTF8	䌄
U+4304.0	fCNS11643	3-4E23
U+4304.0	fR/S	 120.9
U+4304.0	fTotalStrokes	 15
U+4304.0	fCangjie	VFMBE
U+4304.0	fFourCorner	21940
U+4304.0	fCantonese	gang2
U+4304.0	fMandarin	geng3
U+4304.0	fEnglish	standard form of &#x7D86;
U+4305.0	fUTF8	䌅
U+4305.0	fCNS11643	3-523A
U+4305.0	fR/S	 120.10
U+4305.0	fTotalStrokes	 16
U+4305.0	fCangjie	VFILR
U+4305.0	fFourCorner	20960
U+4305.0	fCantonese	tong4; tou4
U+4305.0	fMandarin	tang2
U+4305.0	fEnglish	thick ropes
U+4306.0	fUTF8	䌆
U+4306.0	fCNS11643	4-5249
U+4306.0	fGB	5-6C4A
U+4306.0	fR/S	 120.10
U+4306.0	fTotalStrokes	 16
U+4306.0	fCangjie	VFHUI
U+4306.0	fFourCorner	26910
U+4306.0	fCantonese	gwai3; gwaai2
U+4306.0	fMandarin	gui4
U+4306.0	fEnglish	silk fabrics
U+4307.0	fUTF8	䌇
U+4307.0	fCNS11643	3-523C
U+4307.0	fR/S	 120.10
U+4307.0	fTotalStrokes	 16
U+4307.0	fCangjie	VFJBF
U+4307.0	fFourCorner	24990
U+4307.0	fHakka	  sok7
U+4307.0	fCantonese	sok3; sak3
U+4307.0	fMandarin	suo3; hui4
U+4307.0	fEnglish	another form of $ (rope)
U+4308.0	fUTF8	䌈
U+4308.0	fCNS11643	4-5247
U+4308.0	fGB	3-6D38
U+4308.0	fR/S	 120.10
U+4308.0	fTotalStrokes	 16
U+4308.0	fCangjie	VFASM
U+4308.0	fFourCorner	26920
U+4308.0	fHakka	 tap7; lap7
U+4308.0	fCantonese	taap3
U+4308.0	fMandarin	ta4
U+4308.0	fEnglish	 get something with a lasso
U+4309.0	fUTF8	䌉
U+4309.0	fCNS11643	3-523D
U+4309.0	fR/S	 120.10
U+4309.0	fTotalStrokes	 16
U+4309.0	fCangjie	VFWLM
U+4309.0	fFourCorner	26910
U+4309.0	fCantonese	gong1
U+4309.0	fMandarin	gang1
U+4309.0	fEnglish	non-classical form of &#x7DB1;
U+430A.0	fUTF8	䌊
U+430A.0	fCNS11643	3-523E
U+430A.0	fR/S	 120.10
U+430A.0	fTotalStrokes	 16
U+430A.0	fCangjie	VFBOU
U+430A.0	fFourCorner	27970
U+430A.0	fCantonese	ziu4
U+430A.0	fMandarin	zhou4; yao2; you2
U+430A.0	fEnglish	corrupted form of &#x7E47;
U+430B.0	fUTF8	䌋
U+430B.0	fR/S	 120.10
U+430B.0	fTotalStrokes	 16
U+430B.0	fCangjie	VFTOR
U+430B.0	fCantonese	daap3; dap3
U+430B.0	fEnglish	a kerchief hung at the waist, a tiny sack; a purse or pouch carried hanging over the girdle
U+430C.0	fUTF8	䌌
U+430C.0	fCNS11643	4-5773
U+430C.0	fGB	3-6D43
U+430C.0	fR/S	 120.11
U+430C.0	fTotalStrokes	 17; 16
U+430C.0	fCangjie	VFSMA
U+430C.0	fFourCorner	27960
U+430C.0	fCantonese	caap1; cip3
U+430C.0	fMandarin	qie4; qi4; qu3
U+430C.0	fEnglish	[1] lace; embroidered hem (of a garment) [2] same as &#x7DC1;
U+430D.0	fUTF8	䌍
U+430D.0	fCNS11643	3-5578
U+430D.0	fR/S	 120.11
U+430D.0	fTotalStrokes	 17
U+430D.0	fCangjie	VFTLM
U+430D.0	fFourCorner	24910
U+430D.0	fCantonese	gan2
U+430D.0	fMandarin	jin3; han4
U+430D.0	fEnglish	fine woven pattern
U+430E.0	fUTF8	䌎
U+430E.0	fCNS11643	5-5D3F
U+430E.0	fGB	3-6C2D
U+430E.0	fR/S	 120.11
U+430E.0	fTotalStrokes	 17
U+430E.0	fCangjie	WRVIF
U+430E.0	fFourCorner	67900
U+430E.0	fCantonese	loek6
U+430E.0	fMandarin	luue4; le4
U+430E.0	fEnglish	[1] baste for sewing [2] connect; sew; stitch; mend
U+430F.0	fUTF8	䌏
U+430F.0	fCNS11643	3-5576
U+430F.0	fR/S	 120.11
U+430F.0	fTotalStrokes	 17
U+430F.0	fCangjie	VFJPU
U+430F.0	fFourCorner	23970
U+430F.0	fCantonese	mat6
U+430F.0	fMandarin	mi4; mian4
U+4310.0	fUTF8	䌐
U+4310.0	fCNS11643	5-5D46
U+4310.0	fGB	3-6D22
U+4310.0	fR/S	 120.11
U+4310.0	fTotalStrokes	 17
U+4310.0	fCangjie	VFBBU
U+4310.0	fFourCorner	22910
U+4310.0	fCantonese	mik6
U+4310.0	fMandarin	mi4
U+4310.0	fEnglish	ropes; cords; cables
U+4311.0	fUTF8	䌑
U+4311.0	fCNS11643	3-5577
U+4311.0	fR/S	 120.11
U+4311.0	fTotalStrokes	 17
U+4311.0	fCangjie	VFSEG
U+4311.0	fFourCorner	27910
U+4311.0	fCantonese	gin1
U+4311.0	fMandarin	jian1
U+4311.0	fEnglish	interchangeable with &#x5805;
U+4312.0	fUTF8	䌒
U+4312.0	fCNS11643	4-576E
U+4312.0	fR/S	 120.11
U+4312.0	fTotalStrokes	 17; 16
U+4312.0	fCangjie	VFIXP
U+4312.0	fFourCorner	20910
U+4312.0	fCantonese	luk6
U+4312.0	fMandarin	lu4
U+4312.0	fEnglish	[1] pure; unallyed [2] honest; simple [3] purely;
U+4313.0	fUTF8	䌓
U+4313.0	fCNS11643	3-557C
U+4313.0	fR/S	 120.11
U+4313.0	fTotalStrokes	 17
U+4313.0	fCangjie	OEVIF
U+4313.0	fFourCorner	87900
U+4313.0	fHakka	 fan2
U+4313.0	fCantonese	bun1; fan4
U+4313.0	fMandarin	fan2
U+4313.0	fEnglish	corrupted form of &#x7E41;
U+4314.0	fUTF8	䌔
U+4314.0	fCNS11643	3-557B
U+4314.0	fGB	5-6B56
U+4314.0	fR/S	 120.11
U+4314.0	fTotalStrokes	 17
U+4314.0	fCangjie	VFSRR
U+4314.0	fFourCorner	21910
U+4314.0	fCantonese	au1; ngau1 (also); aau3
U+4314.0	fMandarin	ou1; ou4
U+4314.0	fEnglish	corrupted form of &#x6F1A;
U+4315.0	fUTF8	䌕
U+4315.0	fCNS11643	7-3A39
U+4315.0	fR/S	 120.11
U+4315.0	fTotalStrokes	 17
U+4315.0	fCangjie	VFIDD
U+4315.0	fFourCorner	20990
U+4315.0	fCantonese	mei4
U+4315.0	fMandarin	mi2; mo2
U+4315.0	fEnglish	same as &#x7E3B;
U+4316.0	fUTF8	䌖
U+4316.0	fCNS11643	4-5C62
U+4316.0	fGB	5-6C67
U+4316.0	fR/S	 120.12
U+4316.0	fTotalStrokes	 18
U+4316.0	fCangjie	VFOGD
U+4316.0	fFourCorner	20990
U+4316.0	fCantonese	cap1; zit6
U+4316.0	fMandarin	jie2
U+4316.0	fEnglish	[1] gather; assemble; come together; blend; mix [2] disorderly; mixed [3] a general term for wealthy goods and textiles (in southern minority group)
U+4317.0	fUTF8	䌗
U+4317.0	fCNS11643	4-5C60
U+4317.0	fGB	3-6C33
U+4317.0	fR/S	 120.12
U+4317.0	fTotalStrokes	 18
U+4317.0	fCangjie	VFOTF
U+4317.0	fFourCorner	28930
U+4317.0	fCantonese	fu2
U+4317.0	fMandarin	fu3
U+4317.0	fEnglish	[1] the left over of drawing silk (from cocoons) [2] silk [3] arrange [4] raw, coarse, waste cotton or silk
U+4318.0	fUTF8	䌘
U+4318.0	fCNS11643	4-5772
U+4318.0	fGB	3-6C2C
U+4318.0	fR/S	 120.12; 120.11
U+4318.0	fTotalStrokes	 16; 18; 17
U+4318.0	fCangjie	FKVIF
U+4318.0	fFourCorner	98900
U+4318.0	fCantonese	bit3; nei4
U+4318.0	fMandarin	bie4; mi2
U+4318.0	fEnglish	[1] knit; twist ropes [2] a belt to hang sword [3] poor qualitied silk wadding
U+4319.0	fUTF8	䌙
U+4319.0	fCNS11643	3-5857
U+4319.0	fR/S	 120.12
U+4319.0	fTotalStrokes	 18
U+4319.0	fCangjie	VFTMC
U+4319.0	fFourCorner	24980
U+4319.0	fHakka	 fong3
U+4319.0	fCantonese	wong6; fong2
U+4319.0	fMandarin	huang4; huang3
U+4319.0	fEnglish	rope; halliards
U+431A.0	fUTF8	䌚
U+431A.0	fCNS11643	3-5856
U+431A.0	fR/S	 120.12
U+431A.0	fTotalStrokes	 18
U+431A.0	fCangjie	VFMWD
U+431A.0	fFourCorner	21990
U+431A.0	fCantonese	suk1; siu1; tuk1
U+431A.0	fMandarin	su4
U+431A.0	fEnglish	wrinkles; creases; folds
U+431B.0	fUTF8	䌛
U+431B.0	fCNS11643	3-5929
U+431B.0	fGB	5-714A
U+431B.0	fR/S	 120.12
U+431B.0	fTotalStrokes	 18
U+431B.0	fCangjie	BRHVF
U+431B.0	fFourCorner	22690
U+431B.0	fCantonese	jau4; zaau4; ziu4
U+431B.0	fMandarin	zhou4; yao2
U+431B.0	fEnglish	same as &#x7E47;, &#x5FAD;, &#x9676;, &#x8B20;, &#x7531;, &#x7336;, &#x60A0;, &#x7C40;
U+431C.0	fUTF8	䌜
U+431C.0	fCNS11643	4-6066
U+431C.0	fGB	3-6C43
U+431C.0	fR/S	 120.13
U+431C.0	fTotalStrokes	 19
U+431C.0	fCangjie	VFTCD
U+431C.0	fFourCorner	22990
U+431C.0	fHakka	  ngiap8
U+431C.0	fCantonese	nip6; lip6 (also)
U+431C.0	fMandarin	ye4; nie4
U+431C.0	fEnglish	 oversew a seam
U+431D.0	fUTF8	䌝
U+431D.0	fCNS11643	4-6063
U+431D.0	fGB	5-6C6E
U+431D.0	fR/S	 120.13
U+431D.0	fTotalStrokes	 19
U+431D.0	fCangjie	VFDDF
U+431D.0	fFourCorner	24990
U+431D.0	fCantonese	gam1
U+431D.0	fMandarin	jin1; jin4
U+431D.0	fEnglish	[1] silk [2] a general name of silk fabrics or goods [3] interchangeable with &#x7D3A;
U+431E.0	fUTF8	䌞
U+431E.0	fCNS11643	4-6068
U+431E.0	fGB	3-6C6D
U+431E.0	fR/S	 120.13
U+431E.0	fTotalStrokes	 19
U+431E.0	fCangjie	VFOMO
U+431E.0	fFourCorner	28980
U+431E.0	fCantonese	lim5
U+431E.0	fMandarin	lian3
U+431E.0	fEnglish	[1] a knot to hang the apparatus made of reed for raising silkworms [2] rope
U+431F.0	fUTF8	䌟
U+431F.0	fCNS11643	4-6062
U+431F.0	fGB	3-6D4E
U+431F.0	fR/S	 120.13
U+431F.0	fTotalStrokes	 19
U+431F.0	fCangjie	VFSRJ
U+431F.0	fFourCorner	20940
U+431F.0	fHakka	  pit7; biak7
U+431F.0	fCantonese	baak6; pek3
U+431F.0	fMandarin	po2; bi4
U+431F.0	fEnglish	to work fibres into thread for weaving
U+4320.0	fUTF8	䌠
U+4320.0	fCNS11643	7-474A
U+4320.0	fGB	3-6C2F
U+4320.0	fR/S	 120.13; 76.13
U+4320.0	fTotalStrokes	 19
U+4320.0	fCangjie	IOVIF
U+4320.0	fFourCorner	57903
U+4320.0	fCantonese	gaam1; zim2
U+4320.0	fMandarin	xian1; qing4; yan3; yin4
U+4320.0	fEnglish	[1] hold on to one's own views [2] keep the mouth shut [3] stingy; miserly; niggardly; parsimonious
U+4321.0	fUTF8	䌡
U+4321.0	fCNS11643	3-5A73
U+4321.0	fR/S	 120.13
U+4321.0	fTotalStrokes	 19
U+4321.0	fCangjie	VFTWT
U+4321.0	fFourCorner	25910
U+4321.0	fCantonese	tai2
U+4321.0	fMandarin	ti3
U+4321.0	fEnglish	[1] wind around; bind; wrap; tangle [2] bother persistently
U+4322.0	fUTF8	䌢
U+4322.0	fCNS11643	3-5A71
U+4322.0	fGB	5-6C6F
U+4322.0	fR/S	 120.13
U+4322.0	fTotalStrokes	 19
U+4322.0	fCangjie	VFMBI
U+4322.0	fFourCorner	21930
U+4322.0	fCantonese	ling4
U+4322.0	fMandarin	ling2
U+4322.0	fEnglish	[1] raw, coarse, old, waste cotton or silk [2] wooly; fluffy
U+4323.0	fUTF8	䌣
U+4323.0	fCNS11643	5-6D22
U+4323.0	fR/S	 120.14
U+4323.0	fTotalStrokes	 20
U+4323.0	fCangjie	VFHJR
U+4323.0	fFourCorner	28970
U+4323.0	fCantonese	zyun2; waai2; zyut6
U+4323.0	fMandarin	zuan3
U+4323.0	fEnglish	 accumulate; store up
U+4324.0	fUTF8	䌤
U+4324.0	fCNS11643	3-5C69
U+4324.0	fGB	5-6B67
U+4324.0	fR/S	 120.14
U+4324.0	fTotalStrokes	 20
U+4324.0	fCangjie	VFMFB
U+4324.0	fFourCorner	21920
U+4324.0	fCantonese	si1
U+4324.0	fMandarin	shi1; zhi3; si1
U+4324.0	fEnglish	[1] a kind of unrefined or unpolished silken textiles; silken goods; silken fabrics [2] sew; patch clothes (of sword)
U+4325.0	fUTF8	䌥
U+4325.0	fCNS11643	4-6424
U+4325.0	fGB	5-6C73
U+4325.0	fR/S	 120.14
U+4325.0	fTotalStrokes	 20
U+4325.0	fCangjie	VFBMP
U+4325.0	fFourCorner	22930
U+4325.0	fHakka	 jin3; rin3; run3; jun3
U+4325.0	fCantonese	jan5; zaan2; zi1
U+4325.0	fMandarin	yin3
U+4325.0	fEnglish	 'run' a seam in sewing
U+4326.0	fUTF8	䌦
U+4326.0	fCNS11643	3-5C6A
U+4326.0	fR/S	 120.14
U+4326.0	fTotalStrokes	 20
U+4326.0	fCangjie	VFSMG
U+4326.0	fFourCorner	27910
U+4326.0	fCantonese	dou6
U+4326.0	fMandarin	dao4; dao3; gao4; di2
U+4326.0	fEnglish	green color; bluish yellow
U+4327.0	fUTF8	䌧
U+4327.0	fCNS11643	3-5C6B
U+4327.0	fGB	5-6C21
U+4327.0	fR/S	 120.14
U+4327.0	fTotalStrokes	 20
U+4327.0	fCangjie	VFGNI
U+4327.0	fFourCorner	24940
U+4327.0	fHakka	 chu2; chiu2
U+4327.0	fCantonese	cau4
U+4327.0	fMandarin	xiu4; chou2
U+4327.0	fEnglish	non-classical form of &#x7D2C;, &#x7DA2;
U+4328.0	fUTF8	䌨
U+4328.0	fCNS11643	3-5E4F
U+4328.0	fGB	5-6C72
U+4328.0	fR/S	 120.15
U+4328.0	fTotalStrokes	 21
U+4328.0	fCangjie	VFTBF
U+4328.0	fFourCorner	24990
U+4328.0	fCantonese	cat3
U+4328.0	fMandarin	cai4; ca1; sa4
U+4328.0	fEnglish	thin silk fabrics, sound of the rubbing of clothes
U+4329.0	fUTF8	䌩
U+4329.0	fCNS11643	3-5E4E
U+4329.0	fGB	5-6C70
U+4329.0	fR/S	 120.15
U+4329.0	fTotalStrokes	 21
U+4329.0	fCangjie	VFTWI
U+4329.0	fFourCorner	24950
U+4329.0	fHakka	 met7
U+4329.0	fCantonese	mit6
U+4329.0	fMandarin	mie4; mi4
U+4329.0	fEnglish	fine, thin stockings or socks
U+432A.0	fUTF8	䌪
U+432A.0	fCNS11643	4-692A
U+432A.0	fGB	3-6D41
U+432A.0	fR/S	 120.16
U+432A.0	fTotalStrokes	 22; 21
U+432A.0	fCangjie	VFANX
U+432A.0	fFourCorner	27920
U+432A.0	fCantonese	jim5; zim4; zim5
U+432A.0	fMandarin	yan2
U+432A.0	fEnglish	 continue; extend; review
U+432B.0	fUTF8	䌫
U+432B.0	fCNS11643	7-5858
U+432B.0	fR/S	 120.16
U+432B.0	fTotalStrokes	 22
U+432B.0	fCangjie	VFSIU
U+432B.0	fFourCorner	28910
U+432B.0	fCantonese	laam6; leoi6; lip6
U+432B.0	fMandarin	lan4; lan3
U+432B.0	fEnglish	non-classical form of &#x7E9C;
U+432C.0	fUTF8	䌬
U+432C.0	fCNS11643	3-5F59
U+432C.0	fR/S	 120.16
U+432C.0	fTotalStrokes	 22
U+432C.0	fCangjie	VFYBP
U+432C.0	fFourCorner	21910
U+432C.0	fCantonese	cung4
U+432C.0	fMandarin	chong2
U+432C.0	fEnglish	[1] straight; right; just [2] straight-forward [3] uninterrupted
U+432D.0	fUTF8	䌭
U+432D.0	fCNS11643	4-692B
U+432D.0	fGB	5-6C76
U+432D.0	fR/S	 120.16
U+432D.0	fTotalStrokes	 22; 21
U+432D.0	fCangjie	VFTOF
U+432D.0	fFourCorner	24930
U+432D.0	fCantonese	ziu1
U+432D.0	fMandarin	jiao1
U+432D.0	fEnglish	[1] raw hemp [2] a kind of fabric
U+432E.0	fUTF8	䌮
U+432E.0	fCNS11643	4-6A66
U+432E.0	fGB	3-6D55
U+432E.0	fR/S	 120.17
U+432E.0	fTotalStrokes	 23
U+432E.0	fCangjie	VFMBU
U+432E.0	fFourCorner	21960
U+432E.0	fCantonese	soeng1
U+432E.0	fMandarin	shuang1
U+432E.0	fEnglish	silk fabrics with the color of new leaves
U+432F.0	fUTF8	䌯
U+432F.0	fCNS11643	4-6B79
U+432F.0	fGB	3-6D54
U+432F.0	fR/S	 120.18
U+432F.0	fTotalStrokes	 24
U+432F.0	fCangjie	VFTRG
U+432F.0	fFourCorner	24910
U+432F.0	fCantonese	hyun1
U+432F.0	fMandarin	quan1; quan2; guan4
U+432F.0	fEnglish	[1] truban; kerchief [2] children's hat [3] girdle [4] belt of the clothes
U+4330.0	fUTF8	䌰
U+4330.0	fCNS11643	4-6B78
U+4330.0	fGB	3-6D35
U+4330.0	fR/S	 120.18
U+4330.0	fTotalStrokes	 24
U+4330.0	fCangjie	VFSJJ
U+4330.0	fFourCorner	21940
U+4330.0	fCantonese	nip6; lip6 (also); sip3
U+4330.0	fMandarin	nie4; she4
U+4330.0	fEnglish	[1] connected silk [2] measurement
U+4331.0	fUTF8	䌱
U+4331.0	fCNS11643	7-5F33
U+4331.0	fR/S	 120.18
U+4331.0	fTotalStrokes	 24
U+4331.0	fCangjie	VFYNT
U+4331.0	fFourCorner	20910
U+4331.0	fCantonese	lo6; bok6; lok6
U+4331.0	fMandarin	luo4
U+4331.0	fEnglish	same as &#x4334;
U+4332.0	fUTF8	䌲
U+4332.0	fCNS11643	3-6078
U+4332.0	fR/S	 120.18
U+4332.0	fTotalStrokes	 24
U+4332.0	fCangjie	VFTHF
U+4332.0	fFourCorner	24930
U+4332.0	fCantonese	fan1
U+4332.0	fMandarin	xun1
U+4332.0	fEnglish	same as &#x7E81;
U+4333.0	fUTF8	䌳
U+4333.0	fCNS11643	4-6C7C
U+4333.0	fGB	3-6D39
U+4333.0	fR/S	 120.19
U+4333.0	fTotalStrokes	 25
U+4333.0	fCangjie	VFMBI
U+4333.0	fFourCorner	21910
U+4333.0	fCantonese	si1
U+4333.0	fMandarin	shi1; si1
U+4333.0	fEnglish	a kind of unrefined or unpolished silken textiles; silken goods; silken fabrics,
U+4334.0	fUTF8	䌴
U+4334.0	fCNS11643	4-6C7B
U+4334.0	fGB	3-6D58
U+4334.0	fR/S	 120.19
U+4334.0	fTotalStrokes	 25
U+4334.0	fCangjie	VFYNQ
U+4334.0	fFourCorner	20910
U+4334.0	fCantonese	lo6; lok6
U+4334.0	fMandarin	luo4
U+4334.0	fEnglish	uneven; silk with knots; unpolished
U+4335.0	fUTF8	䌵
U+4335.0	fCNS11643	4-6D77
U+4335.0	fGB	3-6D48
U+4335.0	fR/S	 120.21
U+4335.0	fTotalStrokes	 27
U+4335.0	fCangjie	VFSEI
U+4335.0	fFourCorner	27920
U+4335.0	fCantonese	zuk1
U+4335.0	fMandarin	zhu2; zhu3
U+4335.0	fEnglish	lapel and belt
U+4336.0	fUTF8	䌶
U+4336.0	fGB	S-233D
U+4336.0	fR/S	 120.3
U+4336.0	fTotalStrokes	 9
U+4336.0	fCangjie	VMDH
U+4336.0	fCantonese	zi1
U+4336.0	fEnglish	[1] simplified form of &#x42B7; [2] same as &#x7DC7; [3] same as &#x7D14;
U+4337.0	fUTF8	䌷
U+4337.0	fGB	7-2237
U+4337.0	fR/S	 120.5
U+4337.0	fTotalStrokes	 11
U+4337.0	fCangjie	VMLW
U+4337.0	fCantonese	cau4; cau1; caau1; saau4
U+4337.0	fMandarin	chou2; chou1
U+4337.0	fEnglish	simplified form of &#x7D2C;
U+4338.0	fUTF8	䌸
U+4338.0	fGB	7-236E
U+4338.0	fR/S	 120.4
U+4338.0	fTotalStrokes	 10
U+4338.0	fCangjie	VMQNI
U+4338.0	fCantonese	gyun3
U+4338.0	fMandarin	juan4
U+4338.0	fEnglish	[1] simplified form of &#x7E1B; [2] interchangeable with &#x5377;
U+4339.0	fUTF8	䌹
U+4339.0	fGB	7-2370
U+4339.0	fR/S	 120.5
U+4339.0	fTotalStrokes	 11
U+4339.0	fCangjie	VMBR
U+4339.0	fCantonese	gwing2; gwing1
U+4339.0	fMandarin	jiong3
U+4339.0	fEnglish	[1] simplified form of &#x7D45; [2] same as &#x8927;
U+433A.0	fUTF8	䌺
U+433A.0	fGB	7-236F
U+433A.0	fR/S	 120.6
U+433A.0	fTotalStrokes	 12
U+433A.0	fCangjie	VMSJ
U+433A.0	fCantonese	ji5; zi5
U+433A.0	fMandarin	er3
U+433B.0	fUTF8	䌻
U+433B.0	fGB	7-2371
U+433B.0	fR/S	 120.6
U+433B.0	fTotalStrokes	 12
U+433B.0	fCangjie	VMSMM
U+433B.0	fCantonese	jik6; zik6
U+433B.0	fMandarin	yi4
U+433B.0	fEnglish	simplified form of &#x7FFC;
U+433C.0	fUTF8	䌼
U+433C.0	fGB	S-233B
U+433C.0	fR/S	 120.7
U+433C.0	fTotalStrokes	 13
U+433C.0	fCangjie	VMCRU
U+433C.0	fCantonese	deoi6; jeoi6; zeoi6
U+433C.0	fMandarin	rui4
U+433C.0	fEnglish	simplified form of &#x7D90;
U+433D.0	fUTF8	䌽
U+433D.0	fGB	S-233F
U+433D.0	fR/S	 120.8
U+433D.0	fTotalStrokes	 14
U+433D.0	fCangjie	VMBD
U+433D.0	fCantonese	coi2
U+433D.0	fMandarin	cai3
U+433D.0	fEnglish	simplified form of &#x7DB5;
U+433E.0	fUTF8	䌾
U+433E.0	fGB	S-233C
U+433E.0	fR/S	 120.9
U+433E.0	fTotalStrokes	 15
U+433E.0	fCangjie	VMJBJ
U+433E.0	fCantonese	nam4; lam4 (also); zaam4; jaam4
U+433E.0	fMandarin	ren2
U+433E.0	fEnglish	[1] simplified form of &#x7EB4; [2] interchangeable with &#x7EB4;
U+433F.0	fUTF8	䌿
U+433F.0	fGB	S-233E
U+433F.0	fR/S	 120.9
U+433F.0	fTotalStrokes	 15
U+433F.0	fCangjie	VMMRW
U+433F.0	fCantonese	fuk1
U+433F.0	fMandarin	fu2
U+433F.0	fEnglish	simplified form of &#x5E45;
U+4340.0	fUTF8	䍀
U+4340.0	fGB	S-2341
U+4340.0	fR/S	 120.10
U+4340.0	fTotalStrokes	 16
U+4340.0	fCangjie	VMLIT
U+4340.0	fCantonese	laam4; lam4
U+4340.0	fMandarin	lan2
U+4340.0	fEnglish	[1] simplified form of &#x7E7F; [2] simplified form of &#x8964;
U+4341.0	fUTF8	䍁
U+4341.0	fGB	S-2340
U+4341.0	fR/S	 120.13
U+4341.0	fTotalStrokes	 19
U+4341.0	fCangjie	VMYTO
U+4341.0	fCantonese	seoi6
U+4341.0	fMandarin	sui4
U+4341.0	fEnglish	simplified form of &#x7E78;
U+4342.0	fUTF8	䍂
U+4342.0	fCNS11643	4-2A79
U+4342.0	fGB	3-6338
U+4342.0	fR/S	 121.3
U+4342.0	fTotalStrokes	 9
U+4342.0	fCangjie	OUMD
U+4342.0	fFourCorner	81740
U+4342.0	fCantonese	jyu4; zyu1
U+4342.0	fMandarin	yu2
U+4342.0	fEnglish	tools to draw water or liquid
U+4343.0	fUTF8	䍃
U+4343.0	fCNS11643	4-2F29
U+4343.0	fGB	3-4C38
U+4343.0	fR/S	 121.4
U+4343.0	fTotalStrokes	 10
U+4343.0	fCangjie	BOJU
U+4343.0	fFourCorner	27770
U+4343.0	fHakka	 jau2; rau2; jeu2; riu2; ju2
U+4343.0	fCantonese	jau4; ziu4
U+4343.0	fMandarin	yao2; you2
U+4343.0	fEnglish	 hum an air
U+4344.0	fUTF8	䍄
U+4344.0	fCNS11643	3-382F
U+4344.0	fGB	5-625E
U+4344.0	fR/S	 121.5
U+4344.0	fTotalStrokes	 11
U+4344.0	fCangjie	OUYR
U+4344.0	fFourCorner	81760
U+4344.0	fCantonese	dim2
U+4344.0	fMandarin	dian3; dian4
U+4344.0	fEnglish	 indenture (of earthenware)
U+4345.0	fUTF8	䍅
U+4345.0	fCNS11643	3-3832
U+4345.0	fR/S	 121.5
U+4345.0	fTotalStrokes	 11
U+4345.0	fCangjie	OUOII
U+4345.0	fFourCorner	88730
U+4345.0	fCantonese	ling4
U+4345.0	fMandarin	ling2
U+4345.0	fEnglish	containers for water or wine used in ancient times
U+4346.0	fUTF8	䍆
U+4346.0	fCNS11643	5-3460
U+4346.0	fR/S	 121.5
U+4346.0	fTotalStrokes	 11
U+4346.0	fCangjie	OUJMN
U+4346.0	fFourCorner	83720
U+4346.0	fCantonese	zyu2; cyu5
U+4346.0	fMandarin	zhu3; zhu4
U+4346.0	fEnglish	container for grains used in ancient times
U+4347.0	fUTF8	䍇
U+4347.0	fCNS11643	4-3434
U+4347.0	fGB	3-6339
U+4347.0	fR/S	 121.5; 121.4
U+4347.0	fTotalStrokes	 11; 10
U+4347.0	fCangjie	OUHIO
U+4347.0	fFourCorner	82730
U+4347.0	fCantonese	daap6
U+4347.0	fMandarin	ta4; pi1
U+4347.0	fEnglish	flat-bottom crock (with narrow opening); jug, pitcher, vase, etc
U+4348.0	fUTF8	䍈
U+4348.0	fCNS11643	3-3830
U+4348.0	fR/S	 121.5
U+4348.0	fTotalStrokes	 11
U+4348.0	fCangjie	OUMFJ
U+4348.0	fFourCorner	81740
U+4348.0	fCantonese	ping4; biu2
U+4348.0	fMandarin	ping2
U+4348.0	fEnglish	same as &#x7F3E;, &#x74F6;
U+4349.0	fUTF8	䍉
U+4349.0	fCNS11643	4-3433
U+4349.0	fGB	3-633C
U+4349.0	fR/S	 121.5
U+4349.0	fTotalStrokes	 11
U+4349.0	fCangjie	OURC
U+4349.0	fFourCorner	86780
U+4349.0	fCantonese	zaak3; cak3; zak3
U+4349.0	fMandarin	zhai3; qian2
U+4349.0	fEnglish	traces or marks of incomplete or damage
U+434A.0	fUTF8	䍊
U+434A.0	fCNS11643	4-3A28
U+434A.0	fGB	3-633E
U+434A.0	fR/S	 121.6
U+434A.0	fTotalStrokes	 12
U+434A.0	fCangjie	OUYCK
U+434A.0	fFourCorner	80740
U+434A.0	fCantonese	gaau1
U+434A.0	fMandarin	jiao1; jue2; ji1; jiao4
U+434A.0	fEnglish	clay-made blowing wind instruments; music instruments made of clay
U+434B.0	fUTF8	䍋
U+434B.0	fCNS11643	4-4625
U+434B.0	fGB	3-633F
U+434B.0	fR/S	 121.8
U+434B.0	fTotalStrokes	 14
U+434B.0	fCangjie	OUHJM
U+434B.0	fFourCorner	22710
U+434B.0	fCantonese	seoi4; ceoi4
U+434B.0	fMandarin	chui2
U+434B.0	fEnglish	 bottle; pitcher; jug; vase; an earthen jar with narrow opening
U+434C.0	fUTF8	䍌
U+434C.0	fCNS11643	3-4876
U+434C.0	fGB	5-6264
U+434C.0	fR/S	 121.8
U+434C.0	fTotalStrokes	 14
U+434C.0	fCangjie	OUYTR
U+434C.0	fFourCorner	80760
U+434C.0	fCantonese	pau2; bou6; faau4
U+434C.0	fMandarin	fou2; fu2; bu4; pou2
U+434C.0	fEnglish	same as &#x74FF;
U+434D.0	fUTF8	䍍
U+434D.0	fCNS11643	4-524E
U+434D.0	fGB	3-4F39
U+434D.0	fR/S	 121.10
U+434D.0	fTotalStrokes	 16
U+434D.0	fCangjie	GUHNE
U+434D.0	fFourCorner	47740
U+434D.0	fCantonese	kau3; gei2; huk1; kaau3
U+434D.0	fMandarin	kou4; fu1; gu3; fou1; gu4; guang4; ku1
U+434D.0	fEnglish	unburnt bricks; tiles; eathenware, etc.
U+434E.0	fUTF8	䍎
U+434E.0	fCNS11643	4-6A6B
U+434E.0	fGB	3-6340
U+434E.0	fR/S	 121.17; 121.18
U+434E.0	fTotalStrokes	 23; 24
U+434E.0	fCangjie	OUTIF
U+434E.0	fFourCorner	84720
U+434E.0	fCantonese	cyun3
U+434E.0	fMandarin	jian4; cun4; qian4; xian4; xian3
U+434E.0	fEnglish	[1] spindle [2] a earthenware used to pinch or knead [3] a small crock
U+434F.0	fUTF8	䍏
U+434F.0	fCNS11643	F-2147
U+434F.0	fR/S	 122.0
U+434F.0	fTotalStrokes	 5
U+434F.0	fCangjie	BT
U+434F.0	fCantonese	jyun4
U+4350.0	fUTF8	䍐
U+4350.0	fCNS11643	3-2A4D
U+4350.0	fGB	5-5A3C
U+4350.0	fR/S	 122.3
U+4350.0	fTotalStrokes	 8
U+4350.0	fCangjie	WLMJ
U+4350.0	fFourCorner	60400
U+4350.0	fHakka	 hon3
U+4350.0	fCantonese	hon2
U+4350.0	fMandarin	han3; han4
U+4350.0	fEnglish	standard form of &#x7F55;
U+4351.0	fUTF8	䍑
U+4351.0	fCNS11643	3-2E5C
U+4351.0	fR/S	 122.3
U+4351.0	fTotalStrokes	 9
U+4351.0	fCangjie	BKMJ
U+4351.0	fFourCorner	77400
U+4351.0	fCantonese	hon2; hon3
U+4351.0	fMandarin	han3
U+4351.0	fEnglish	[1] same as &#x4350; [2] standard form of &#x7F55;
U+4352.0	fUTF8	䍒
U+4352.0	fCNS11643	3-2E5B
U+4352.0	fGB	5-5A3F
U+4352.0	fR/S	 122.4
U+4352.0	fTotalStrokes	 9
U+4352.0	fCangjie	WLD
U+4352.0	fFourCorner	60900
U+4352.0	fCantonese	mau5; maau5; mou4
U+4352.0	fMandarin	fu2; mou3
U+4352.0	fEnglish	(corrupted form) net; web; network
U+4353.0	fUTF8	䍓
U+4353.0	fCNS11643	3-2E59
U+4353.0	fGB	5-5A41
U+4353.0	fR/S	 122.4
U+4353.0	fTotalStrokes	 9
U+4353.0	fCangjie	WLMVH
U+4353.0	fFourCorner	60240
U+4353.0	fCantonese	wu6; ngaa4
U+4353.0	fMandarin	ya2; hu4
U+4353.0	fEnglish	(corrupted form) net for catching rabbit
U+4354.0	fUTF8	䍔
U+4354.0	fCNS11643	4-2A7B
U+4354.0	fGB	3-5A7C
U+4354.0	fR/S	 122.4
U+4354.0	fTotalStrokes	 9
U+4354.0	fCangjie	WLKI
U+4354.0	fFourCorner	60430
U+4354.0	fCantonese	gwang1; gwaang1; waang4
U+4354.0	fMandarin	gong1; hong2
U+4354.0	fEnglish	[1] full net [2] thick rope; cable
U+4355.0	fUTF8	䍕
U+4355.0	fCNS11643	4-2F2A
U+4355.0	fGB	3-5B22
U+4355.0	fR/S	 122.5
U+4355.0	fTotalStrokes	 10
U+4355.0	fCangjie	WLHPM
U+4355.0	fFourCorner	60740
U+4355.0	fCantonese	dai1
U+4355.0	fMandarin	di1; di3
U+4355.0	fEnglish	[1] net; web; network [2] net for catching rabbit
U+4356.0	fUTF8	䍖
U+4356.0	fCNS11643	6-3A6B
U+4356.0	fR/S	 122.5
U+4356.0	fTotalStrokes	 10; 9
U+4356.0	fCangjie	WLPRU
U+4356.0	fFourCorner	60710
U+4356.0	fHakka	  feu2
U+4356.0	fCantonese	fau4; faau4; fu1
U+4356.0	fMandarin	fou2; fu2; fu4; pao1; hai4; xie4; bao1; fu1
U+4356.0	fEnglish	a net for catching birds
U+4357.0	fUTF8	䍗
U+4357.0	fCNS11643	4-2F2D
U+4357.0	fGB	3-5B24
U+4357.0	fR/S	 122.5
U+4357.0	fTotalStrokes	 10
U+4357.0	fCangjie	WLYVI
U+4357.0	fFourCorner	60730
U+4357.0	fCantonese	jyun6; zyun4
U+4357.0	fMandarin	xuan4
U+4357.0	fEnglish	same as &#x7F65;
U+4358.0	fUTF8	䍘
U+4358.0	fCNS11643	4-3435
U+4358.0	fGB	5-5A49
U+4358.0	fR/S	 122.6
U+4358.0	fTotalStrokes	 11
U+4358.0	fCangjie	WLFD
U+4358.0	fFourCorner	60900
U+4358.0	fHakka	 mi2
U+4358.0	fCantonese	bei1; nei4
U+4358.0	fMandarin	mi2
U+4358.0	fEnglish	a general term for nets
U+4359.0	fUTF8	䍙
U+4359.0	fCNS11643	4-3F75
U+4359.0	fGB	3-5B26
U+4359.0	fR/S	 122.7
U+4359.0	fTotalStrokes	 13; 12
U+4359.0	fCangjie	WLOWY
U+4359.0	fFourCorner	77500
U+4359.0	fCantonese	mui4
U+4359.0	fMandarin	mei2
U+4359.0	fEnglish	[1] nets for catching birds [2] net; web; network
U+435A.0	fUTF8	䍚
U+435A.0	fCNS11643	4-3A2A
U+435A.0	fGB	5-5A4C
U+435A.0	fR/S	 122.7
U+435A.0	fTotalStrokes	 12
U+435A.0	fCangjie	WLIAV
U+435A.0	fFourCorner	60730
U+435A.0	fCantonese	long6
U+435A.0	fMandarin	lang4
U+435A.0	fEnglish	vast
U+435B.0	fUTF8	䍛
U+435B.0	fCNS11643	3-436C
U+435B.0	fGB	5-5A4E
U+435B.0	fR/S	 122.8
U+435B.0	fTotalStrokes	 13
U+435B.0	fCangjie	WLWJR
U+435B.0	fFourCorner	60600
U+435B.0	fCantonese	gu3; gwu3 (also)
U+435B.0	fMandarin	gu4
U+435B.0	fEnglish	a kind of tool used to get fish
U+435C.0	fUTF8	䍜
U+435C.0	fCNS11643	3-436D
U+435C.0	fGB	5-5A4F
U+435C.0	fR/S	 122.8
U+435C.0	fTotalStrokes	 13
U+435C.0	fCangjie	WLOG
U+435C.0	fFourCorner	60210
U+435C.0	fCantonese	zaau3
U+435C.0	fMandarin	zhao4
U+435C.0	fEnglish	a basket or a cage used to keep wild and domestic fowls
U+435D.0	fUTF8	䍝
U+435D.0	fCNS11643	5-4165
U+435D.0	fGB	3-5B2B
U+435D.0	fR/S	 122.8
U+435D.0	fTotalStrokes	 13
U+435D.0	fCangjie	WLEA
U+435D.0	fFourCorner	60600
U+435D.0	fCantonese	taap3; dap6
U+435D.0	fMandarin	ta4; zan3
U+435D.0	fEnglish	net; web; network
U+435E.0	fUTF8	䍞
U+435E.0	fCNS11643	3-487A
U+435E.0	fR/S	 122.8
U+435E.0	fTotalStrokes	 14
U+435E.0	fCangjie	BKIRM
U+435E.0	fFourCorner	77150
U+435E.0	fCantonese	wik6; gwik1
U+435E.0	fMandarin	yu4
U+435E.0	fEnglish	same as &#x7F6D;
U+435F.0	fUTF8	䍟
U+435F.0	fCNS11643	3-487B
U+435F.0	fR/S	 122.9
U+435F.0	fTotalStrokes	 14
U+435F.0	fCangjie	WLUCE
U+435F.0	fFourCorner	60400
U+435F.0	fCantonese	zung1; zung3
U+435F.0	fMandarin	zong4
U+435F.0	fEnglish	a kind of fish net with fine meshes
U+4360.0	fUTF8	䍠
U+4360.0	fGB	3-5B2F
U+4360.0	fR/S	 122.11
U+4360.0	fTotalStrokes	 16
U+4360.0	fCangjie	WLYUB
U+4360.0	fCantonese	lei4
U+4360.0	fMandarin	li2
U+4360.0	fEnglish	simplified form of &#x4366;
U+4361.0	fUTF8	䍡
U+4361.0	fCNS11643	4-524F
U+4361.0	fGB	3-5B31
U+4361.0	fR/S	 122.11
U+4361.0	fTotalStrokes	 16
U+4361.0	fCangjie	WLIXP
U+4361.0	fFourCorner	60210
U+4361.0	fHakka	 luk7; luk8
U+4361.0	fCantonese	luk6
U+4361.0	fMandarin	lu4; liao4
U+4361.0	fEnglish	small fishing net with fine meshes
U+4362.0	fUTF8	䍢
U+4362.0	fCNS11643	4-5775
U+4362.0	fGB	3-5A7A
U+4362.0	fR/S	 122.12
U+4362.0	fTotalStrokes	 17; 16
U+4362.0	fCangjie	WLOTF
U+4362.0	fFourCorner	60330
U+4362.0	fCantonese	mou5; mou4
U+4362.0	fMandarin	wu2; wu3
U+4362.0	fEnglish	netted veined window-sill
U+4363.0	fUTF8	䍣
U+4363.0	fCNS11643	4-5C65
U+4363.0	fGB	3-5B37
U+4363.0	fR/S	 122.13
U+4363.0	fTotalStrokes	 18
U+4363.0	fCangjie	WLMBW
U+4363.0	fFourCorner	60600
U+4363.0	fCantonese	leoi4
U+4363.0	fMandarin	lei2
U+4363.0	fEnglish	a kind of fish net
U+4364.0	fUTF8	䍤
U+4364.0	fCNS11643	4-606D
U+4364.0	fGB	5-5A48
U+4364.0	fR/S	 122.14
U+4364.0	fTotalStrokes	 19
U+4364.0	fCangjie	WLYX
U+4364.0	fFourCorner	60220
U+4364.0	fHakka	 zi5; za1
U+4364.0	fCantonese	zai2; zaai1
U+4364.0	fMandarin	ji3; ji4
U+4364.0	fEnglish	 squeeze out with the hand
U+4365.0	fUTF8	䍥
U+4365.0	fCNS11643	4-667B
U+4365.0	fGB	3-5A7B
U+4365.0	fR/S	 122.16
U+4365.0	fTotalStrokes	 21
U+4365.0	fCangjie	WLMHM
U+4365.0	fFourCorner	60210
U+4365.0	fCantonese	lik6
U+4365.0	fMandarin	li4; lei4
U+4365.0	fEnglish	[1] smoky [2] covering; spread [3] cloth for covering food
U+4366.0	fUTF8	䍦
U+4366.0	fCNS11643	4-6B7B
U+4366.0	fR/S	 122.19
U+4366.0	fTotalStrokes	 24
U+4366.0	fCangjie	WLYBG
U+4366.0	fFourCorner	60210
U+4366.0	fHakka	  li2
U+4366.0	fCantonese	lei4
U+4366.0	fMandarin	li2
U+4366.0	fEnglish	white hat adorned with egret feathers
U+4367.0	fUTF8	䍧
U+4367.0	fCNS11643	6-3A74
U+4367.0	fR/S	 123.4
U+4367.0	fTotalStrokes	 10; 9
U+4367.0	fCangjie	HQTQ
U+4367.0	fFourCorner	28550
U+4367.0	fCantonese	zong1
U+4367.0	fMandarin	zang1
U+4367.0	fEnglish	same as &#x7242;
U+4368.0	fUTF8	䍨
U+4368.0	fCNS11643	4-2F2E
U+4368.0	fGB	3-6751
U+4368.0	fR/S	 123.4
U+4368.0	fTotalStrokes	 10
U+4368.0	fCangjie	TQJB
U+4368.0	fFourCorner	85527
U+4368.0	fCantonese	but3; but6
U+4368.0	fMandarin	po4; bo2; fei4
U+4368.0	fEnglish	[1] castrate [2] ram; he-goat [3] a family name
U+4369.0	fUTF8	䍩
U+4369.0	fCNS11643	3-313F
U+4369.0	fR/S	 123.4
U+4369.0	fTotalStrokes	 10
U+4369.0	fCangjie	TQOK
U+4369.0	fFourCorner	88540
U+4369.0	fCantonese	joeng5; zoeng5; zoeng6
U+4369.0	fMandarin	yang3; ang3
U+4369.0	fEnglish	same as &#x22F1D;
U+436A.0	fUTF8	䍪
U+436A.0	fCNS11643	4-3438
U+436A.0	fGB	5-6641
U+436A.0	fR/S	 123.5
U+436A.0	fTotalStrokes	 11
U+436A.0	fCangjie	TQDJ
U+436A.0	fFourCorner	85590
U+436A.0	fCantonese	mat6
U+436A.0	fMandarin	wa4; kui4
U+436A.0	fEnglish	castrated ram
U+436B.0	fUTF8	䍫
U+436B.0	fCNS11643	4-3436
U+436B.0	fGB	3-6758
U+436B.0	fR/S	 123.5
U+436B.0	fTotalStrokes	 11
U+436B.0	fCangjie	TQJP
U+436B.0	fFourCorner	83510
U+436B.0	fHakka	 to2
U+436B.0	fCantonese	to4; fo1
U+436B.0	fMandarin	tuo2
U+436B.0	fEnglish	a fabulous animal like a ram with nine tails and four ears
U+436C.0	fUTF8	䍬
U+436C.0	fCNS11643	4-3437
U+436C.0	fGB	3-6757
U+436C.0	fR/S	 123.5
U+436C.0	fTotalStrokes	 11
U+436C.0	fCangjie	TQMFJ
U+436C.0	fFourCorner	81540
U+436C.0	fCantonese	paang1; ping1
U+436C.0	fMandarin	peng1
U+436C.0	fEnglish	[1] use; made; act [2] a kind of goat with mixed colors
U+436D.0	fUTF8	䍭
U+436D.0	fCNS11643	6-433C
U+436D.0	fR/S	 123.5
U+436D.0	fTotalStrokes	 11
U+436D.0	fCangjie	TQWYI
U+436D.0	fFourCorner	87550
U+436D.0	fCantonese	zong1; cau4; zo6
U+436D.0	fMandarin	zang1
U+436D.0	fEnglish	same as &#x7242;
U+436E.0	fUTF8	䍮
U+436E.0	fCNS11643	4-3A2D
U+436E.0	fGB	3-675B
U+436E.0	fR/S	 123.6
U+436E.0	fTotalStrokes	 12
U+436E.0	fCangjie	TQLMO
U+436E.0	fFourCorner	82510
U+436E.0	fCantonese	siu6
U+436E.0	fMandarin	zhao4; dao4
U+436E.0	fEnglish	[1] young goat (sheep) under one year old [2] castrated ram of hundred catties (Chinese pound)
U+436F.0	fUTF8	䍯
U+436F.0	fCNS11643	4-3A31
U+436F.0	fGB	3-675C
U+436F.0	fR/S	 123.6
U+436F.0	fTotalStrokes	 12
U+436F.0	fCangjie	TQNMU
U+436F.0	fFourCorner	87510
U+436F.0	fCantonese	gwai2
U+436F.0	fMandarin	gui3; ji4
U+436F.0	fEnglish	same as &#x89E4;
U+4370.0	fUTF8	䍰
U+4370.0	fCNS11643	4-3A30
U+4370.0	fGB	3-675A
U+4370.0	fR/S	 123.6
U+4370.0	fTotalStrokes	 12
U+4370.0	fCangjie	TQWK
U+4370.0	fFourCorner	86500
U+4370.0	fCantonese	jin1; zaan1
U+4370.0	fMandarin	yin1
U+4370.0	fEnglish	[1] a plague or a pestilence of goat (sheep) [2] black goat or sheep [3] black
U+4371.0	fUTF8	䍱
U+4371.0	fCNS11643	4-3F76
U+4371.0	fGB	3-675E
U+4371.0	fR/S	 123.7
U+4371.0	fTotalStrokes	 13
U+4371.0	fCangjie	TQOMD
U+4371.0	fFourCorner	88590
U+4371.0	fCantonese	ceoi4; zyu4
U+4371.0	fMandarin	xu2; yu2
U+4371.0	fEnglish	the wild goat or sheep
U+4372.0	fUTF8	䍲
U+4372.0	fCNS11643	3-487E
U+4372.0	fGB	5-664B
U+4372.0	fR/S	 123.8
U+4372.0	fTotalStrokes	 14
U+4372.0	fCangjie	TQHXU
U+4372.0	fFourCorner	87510
U+4372.0	fCantonese	naai4; laai4 (also); nai4; ngaai4
U+4372.0	fMandarin	ni2; jie1; nai2; ni4
U+4372.0	fEnglish	a kind of sheep (goat) with curled hairs
U+4373.0	fUTF8	䍳
U+4373.0	fCNS11643	4-4627
U+4373.0	fGB	3-6760
U+4373.0	fR/S	 123.8
U+4373.0	fTotalStrokes	 14
U+4373.0	fCangjie	TQEEE
U+4373.0	fFourCorner	87540
U+4373.0	fCantonese	kyut3
U+4373.0	fMandarin	que1; jue2; chuo4; zhuo2; que4
U+4373.0	fEnglish	[1] disease; illness [2] goat (or sheep) get sick [3] a throbbing goat (or sheep)
U+4374.0	fUTF8	䍴
U+4374.0	fCNS11643	3-487D
U+4374.0	fGB	5-664A
U+4374.0	fR/S	 123.8
U+4374.0	fTotalStrokes	 14
U+4374.0	fCangjie	TQHDV
U+4374.0	fFourCorner	82540
U+4374.0	fCantonese	wai2
U+4374.0	fMandarin	wei3; wei4; duo4; rui2
U+4374.0	fEnglish	 squeeze and to crowd against each other (of sheep)
U+4375.0	fUTF8	䍵
U+4375.0	fCNS11643	4-4628
U+4375.0	fGB	3-675D
U+4375.0	fR/S	 123.8
U+4375.0	fTotalStrokes	 14
U+4375.0	fCangjie	TQNSD
U+4375.0	fFourCorner	82550
U+4375.0	fHakka	 zen1
U+4375.0	fCantonese	zang1; kaap1; zaang1
U+4375.0	fMandarin	zheng1
U+4375.0	fEnglish	[1] young goat [2] antelope
U+4376.0	fUTF8	䍶
U+4376.0	fCNS11643	3-487C
U+4376.0	fGB	5-6643
U+4376.0	fR/S	 123.8
U+4376.0	fTotalStrokes	 14
U+4376.0	fCangjie	TQDW
U+4376.0	fFourCorner	85590
U+4376.0	fCantonese	dung1
U+4376.0	fMandarin	dong1; dong4; chen2
U+4376.0	fEnglish	unicorn
U+4377.0	fUTF8	䍷
U+4377.0	fCNS11643	4-4C49
U+4377.0	fGB	3-6750
U+4377.0	fR/S	 123.9
U+4377.0	fTotalStrokes	 15
U+4377.0	fCangjie	TQDMQ
U+4377.0	fFourCorner	84550
U+4377.0	fCantonese	wai5
U+4377.0	fMandarin	wei3; gui4; hui4
U+4377.0	fEnglish	 chase each other (of sheep or goats)
U+4378.0	fUTF8	䍸
U+4378.0	fCNS11643	4-5253
U+4378.0	fGB	3-6763
U+4378.0	fR/S	 123.10
U+4378.0	fTotalStrokes	 16
U+4378.0	fCangjie	TQIBI
U+4378.0	fFourCorner	83540
U+4378.0	fCantonese	bok3
U+4378.0	fMandarin	fu4; bo2
U+4378.0	fEnglish	a legendary goat-like animal with 4 ears and 9 tails
U+4379.0	fUTF8	䍹
U+4379.0	fCNS11643	3-5241
U+4379.0	fR/S	 123.10
U+4379.0	fTotalStrokes	 16
U+4379.0	fCangjie	TQHUK
U+4379.0	fFourCorner	86530
U+4379.0	fCantonese	zin1
U+4379.0	fMandarin	shan1
U+4379.0	fEnglish	same as &#x7FB4;, &#x7FB6;
U+437A.0	fUTF8	䍺
U+437A.0	fCNS11643	4-5777
U+437A.0	fGB	3-6765
U+437A.0	fR/S	 123.11
U+437A.0	fTotalStrokes	 17; 16
U+437A.0	fCangjie	TQLLP
U+437A.0	fFourCorner	85530
U+437A.0	fCantonese	waan4
U+437A.0	fMandarin	huan2; huan4
U+437A.0	fEnglish	a legendary goat-like beast
U+437B.0	fUTF8	䍻
U+437B.0	fCNS11643	4-5C6A
U+437B.0	fGB	3-6767
U+437B.0	fR/S	 123.12
U+437B.0	fTotalStrokes	 18
U+437B.0	fCangjie	TQRUC
U+437B.0	fFourCorner	87580
U+437B.0	fCantonese	syun3; syun2
U+437B.0	fMandarin	xuan4; xuan3
U+437B.0	fEnglish	young goat of sheep under one year old
U+437C.0	fUTF8	䍼
U+437C.0	fCNS11643	4-5C6B
U+437C.0	fGB	3-6766
U+437C.0	fR/S	 123.12
U+437C.0	fTotalStrokes	 18
U+437C.0	fCangjie	TQMUA
U+437C.0	fFourCorner	81560
U+437C.0	fCantonese	zaam1; cam4; zam1
U+437C.0	fMandarin	zen1; can2; zan1; za1; qian2
U+437C.0	fEnglish	[1] salted mutton [2] store up the meat under the ground [3] a goat-like beast [4] odor of a sheep or goat,
U+437D.0	fUTF8	䍽
U+437D.0	fCNS11643	4-692C
U+437D.0	fGB	3-6752
U+437D.0	fR/S	 123.16
U+437D.0	fTotalStrokes	 22; 21
U+437D.0	fCangjie	TQMHM
U+437D.0	fFourCorner	81510
U+437D.0	fHakka	 lit8
U+437D.0	fCantonese	lik6
U+437D.0	fMandarin	li4
U+437D.0	fEnglish	a black goat
U+437E.0	fUTF8	䍾
U+437E.0	fCNS11643	3-3340
U+437E.0	fR/S	 124.4
U+437E.0	fTotalStrokes	 10
U+437E.0	fCangjie	MTSMM
U+437E.0	fFourCorner	17420
U+437E.0	fCantonese	jin5; zin2
U+437E.0	fMandarin	yuan3; tui2; yan3
U+437E.0	fEnglish	flying
U+437F.0	fUTF8	䍿
U+437F.0	fCNS11643	3-3341
U+437F.0	fGB	5-6B22
U+437F.0	fR/S	 124.4
U+437F.0	fTotalStrokes	 10
U+437F.0	fCangjie	SMMG
U+437F.0	fFourCorner	17100
U+437F.0	fCantonese	wong4; wang4
U+437F.0	fMandarin	wang2; huang2
U+437F.0	fEnglish	[1] a kind of dance accompanied by music performed in ancient times [2] same as &#x51F0;
U+4380.0	fUTF8	䎀
U+4380.0	fCNS11643	4-343A
U+4380.0	fGB	3-6B6A
U+4380.0	fR/S	 124.5
U+4380.0	fTotalStrokes	 11
U+4380.0	fCangjie	SMIV
U+4380.0	fFourCorner	13150
U+4380.0	fCantonese	hyut3; zyut6
U+4380.0	fMandarin	yue4; xue4
U+4380.0	fEnglish	[1] flying [2] run swiftly; go at express speed
U+4381.0	fUTF8	䎁
U+4381.0	fCNS11643	3-3833
U+4381.0	fGB	5-6B2A
U+4381.0	fR/S	 124.5
U+4381.0	fTotalStrokes	 11
U+4381.0	fCangjie	JRSMM
U+4381.0	fFourCorner	47620
U+4381.0	fCantonese	wu4
U+4381.0	fMandarin	hu2
U+4381.0	fEnglish	used in a person's name
U+4382.0	fUTF8	䎂
U+4382.0	fCNS11643	4-343B
U+4382.0	fGB	3-6B70
U+4382.0	fR/S	 124.5
U+4382.0	fTotalStrokes	 11
U+4382.0	fCangjie	SMPRU
U+4382.0	fFourCorner	17110
U+4382.0	fCantonese	bou2
U+4382.0	fMandarin	bao3
U+4382.0	fEnglish	[1] colorful feather [2] a feather decorated arrow
U+4383.0	fUTF8	䎃
U+4383.0	fCNS11643	4-343E
U+4383.0	fGB	3-6B6C
U+4383.0	fR/S	 124.5
U+4383.0	fTotalStrokes	 11
U+4383.0	fCangjie	GBSMM
U+4383.0	fFourCorner	57420
U+4383.0	fCantonese	jim5; zim5
U+4383.0	fMandarin	ran3
U+4383.0	fEnglish	fine down; floss; wool
U+4384.0	fUTF8	䎄
U+4384.0	fCNS11643	4-343C
U+4384.0	fGB	3-6B72
U+4384.0	fR/S	 124.5
U+4384.0	fTotalStrokes	 11
U+4384.0	fCangjie	SRSMM
U+4384.0	fFourCorner	17620
U+4384.0	fCantonese	hiu1; diu1
U+4384.0	fMandarin	xiao1; tiao2; diao1
U+4384.0	fEnglish	hairy; long tail feather of a bird
U+4385.0	fUTF8	䎅
U+4385.0	fCNS11643	3-3835
U+4385.0	fR/S	 124.5
U+4385.0	fTotalStrokes	 11
U+4385.0	fCangjie	HASMM
U+4385.0	fFourCorner	27620
U+4385.0	fCantonese	pok3; pak3
U+4385.0	fMandarin	po4; fu4; luo4
U+4385.0	fEnglish	flying
U+4386.0	fUTF8	䎆
U+4386.0	fCNS11643	3-3834
U+4386.0	fR/S	 124.5
U+4386.0	fTotalStrokes	 11
U+4386.0	fCangjie	SMOII
U+4386.0	fFourCorner	17300
U+4386.0	fCantonese	liu6; laam5; lo2
U+4386.0	fMandarin	liao4
U+4386.0	fEnglish	flying
U+4387.0	fUTF8	䎇
U+4387.0	fCNS11643	4-3A34
U+4387.0	fGB	3-6562
U+4387.0	fR/S	 124.6
U+4387.0	fTotalStrokes	 12
U+4387.0	fCangjie	HYSMM
U+4387.0	fFourCorner	27420
U+4387.0	fCantonese	zau1; zaau1; zau2
U+4387.0	fMandarin	zhou1
U+4387.0	fEnglish	[1] quick; quickly [2] hasty [3] soft and meek feather
U+4388.0	fUTF8	䎈
U+4388.0	fCNS11643	4-3A35
U+4388.0	fGB	5-6B2F
U+4388.0	fR/S	 124.6
U+4388.0	fTotalStrokes	 12
U+4388.0	fCangjie	SMLWP
U+4388.0	fFourCorner	15100
U+4388.0	fCantonese	jai6; zaai6
U+4388.0	fMandarin	yi4; xie4
U+4388.0	fEnglish	flying
U+4389.0	fUTF8	䎉
U+4389.0	fCNS11643	5-3B3A
U+4389.0	fR/S	 124.6
U+4389.0	fTotalStrokes	 12
U+4389.0	fCangjie	SMIHI
U+4389.0	fFourCorner	13150
U+4389.0	fCantonese	wat1; waat6
U+4389.0	fMandarin	yu4; shu4; xu4
U+4389.0	fEnglish	[1] walk swiftly [2] same as &#x77DE;
U+438A.0	fUTF8	䎊
U+438A.0	fCNS11643	4-3A33
U+438A.0	fGB	3-6B75
U+438A.0	fR/S	 124.6
U+438A.0	fTotalStrokes	 12
U+438A.0	fCangjie	HRSMM
U+438A.0	fFourCorner	27620
U+438A.0	fCantonese	lok6; pak3; pok3
U+438A.0	fMandarin	po4; luo4
U+438A.0	fEnglish	[1] flying [2] flying away
U+438B.0	fUTF8	䎋
U+438B.0	fCNS11643	3-4370
U+438B.0	fR/S	 124.7
U+438B.0	fTotalStrokes	 13
U+438B.0	fCangjie	HRSMM
U+438B.0	fFourCorner	27620
U+438B.0	fCantonese	kaau3
U+438B.0	fMandarin	kao4; he4
U+438B.0	fEnglish	fly
U+438C.0	fUTF8	䎌
U+438C.0	fCNS11643	4-3F78
U+438C.0	fGB	3-6B79
U+438C.0	fR/S	 124.7
U+438C.0	fTotalStrokes	 13
U+438C.0	fCangjie	SMRYO
U+438C.0	fFourCorner	16180
U+438C.0	fCantonese	cuk1
U+438C.0	fMandarin	zu2; chu4; cu4
U+438C.0	fEnglish	[1] feather [2] flying [3] neat feather
U+438D.0	fUTF8	䎍
U+438D.0	fCNS11643	3-4371
U+438D.0	fR/S	 124.7
U+438D.0	fTotalStrokes	 13
U+438D.0	fCangjie	SMIJB
U+438D.0	fFourCorner	13120
U+438D.0	fCantonese	fu1
U+438D.0	fMandarin	fu1
U+438D.0	fEnglish	[1] fine and soft hair [2] feather
U+438E.0	fUTF8	䎎
U+438E.0	fCNS11643	4-3F77
U+438E.0	fGB	3-6B78
U+438E.0	fR/S	 124.7
U+438E.0	fTotalStrokes	 13
U+438E.0	fCangjie	SMSJU
U+438E.0	fFourCorner	11110
U+438E.0	fCantonese	nip6; lip6 (also); naa5
U+438E.0	fMandarin	na3; na4
U+438E.0	fEnglish	[1] glide or to hover up in the air [2] soaring
U+438F.0	fUTF8	䎏
U+438F.0	fCNS11643	3-436F
U+438F.0	fR/S	 124.8; 124.7
U+438F.0	fTotalStrokes	 13; 14
U+438F.0	fCangjie	ORSMM
U+438F.0	fFourCorner	87620
U+438F.0	fCantonese	ham1; haam4
U+438F.0	fMandarin	han2; an4
U+438F.0	fEnglish	young birds flying
U+4390.0	fUTF8	䎐
U+4390.0	fCNS11643	3-4921
U+4390.0	fR/S	 124.8
U+4390.0	fTotalStrokes	 14
U+4390.0	fCangjie	JJSMM
U+4390.0	fFourCorner	47420
U+4390.0	fCantonese	caau2; cau2; ce2; ze4
U+4390.0	fMandarin	chao3
U+4390.0	fEnglish	hairy
U+4391.0	fUTF8	䎑
U+4391.0	fCNS11643	4-462A
U+4391.0	fGB	3-6B7B
U+4391.0	fR/S	 124.8
U+4391.0	fTotalStrokes	 14
U+4391.0	fCangjie	SMVNE
U+4391.0	fFourCorner	17130
U+4391.0	fCantonese	luk6
U+4391.0	fMandarin	lu4
U+4391.0	fEnglish	[1] flying over the water surface [2] flying up; soaring
U+4392.0	fUTF8	䎒
U+4392.0	fCNS11643	4-4629
U+4392.0	fGB	3-6B67
U+4392.0	fR/S	 124.8
U+4392.0	fTotalStrokes	 14
U+4392.0	fCangjie	SMII
U+4392.0	fFourCorner	13150
U+4392.0	fHakka	  zan3
U+4392.0	fCantonese	zaan2
U+4392.0	fMandarin	zhan3; can2; jian3
U+4392.0	fEnglish	 fly swiftly an powerfully
U+4393.0	fUTF8	䎓
U+4393.0	fCNS11643	4-462B
U+4393.0	fGB	3-6B7C
U+4393.0	fR/S	 124.8
U+4393.0	fTotalStrokes	 14
U+4393.0	fCangjie	SMEA
U+4393.0	fFourCorner	12160
U+4393.0	fCantonese	daap6
U+4393.0	fMandarin	ta4
U+4393.0	fEnglish	flying
U+4394.0	fUTF8	䎔
U+4394.0	fCNS11643	4-5255
U+4394.0	fGB	3-6C21
U+4394.0	fR/S	 124.10
U+4394.0	fTotalStrokes	 16
U+4394.0	fCangjie	SMIBI
U+4394.0	fFourCorner	13140
U+4394.0	fCantonese	fu1
U+4394.0	fMandarin	fu1
U+4394.0	fEnglish	[1] fine and soft hair [2] feather
U+4395.0	fUTF8	䎕
U+4395.0	fCNS11643	4-5256
U+4395.0	fGB	3-6B68
U+4395.0	fR/S	 124.10
U+4395.0	fTotalStrokes	 16
U+4395.0	fCangjie	FFBSM
U+4395.0	fFourCorner	99120
U+4395.0	fCantonese	gwang1; gwaang1; waat6
U+4395.0	fMandarin	hong1
U+4395.0	fEnglish	flying birds
U+4396.0	fUTF8	䎖
U+4396.0	fCNS11643	3-585E
U+4396.0	fGB	5-6B3E
U+4396.0	fR/S	 124.12
U+4396.0	fTotalStrokes	 18
U+4396.0	fCangjie	CASMM
U+4396.0	fFourCorner	87620
U+4396.0	fCantonese	zang1
U+4396.0	fMandarin	zeng1
U+4396.0	fEnglish	soaring up in the sky
U+4397.0	fUTF8	䎗
U+4397.0	fCNS11643	3-5860
U+4397.0	fR/S	 124.12
U+4397.0	fTotalStrokes	 18
U+4397.0	fCangjie	HBSMM
U+4397.0	fFourCorner	27220
U+4397.0	fHakka	 kiau2; k'ieu2
U+4397.0	fCantonese	kiu4
U+4397.0	fMandarin	qiao2
U+4397.0	fEnglish	 fly downwards
U+4398.0	fUTF8	䎘
U+4398.0	fCNS11643	3-585F
U+4398.0	fGB	5-6B34
U+4398.0	fR/S	 124.12
U+4398.0	fTotalStrokes	 18
U+4398.0	fCangjie	LXSMM
U+4398.0	fFourCorner	57220
U+4398.0	fHakka	  siuk7; suk7
U+4398.0	fCantonese	suk1
U+4398.0	fMandarin	su4; suo1
U+4398.0	fEnglish	the sound of a flock of birds flying
U+4399.0	fUTF8	䎙
U+4399.0	fCNS11643	3-5C6E
U+4399.0	fGB	5-6B3A
U+4399.0	fR/S	 124.14
U+4399.0	fTotalStrokes	 20
U+4399.0	fCangjie	JCSMM
U+4399.0	fFourCorner	37820
U+4399.0	fCantonese	pan1; baan1
U+4399.0	fMandarin	bin1
U+4399.0	fEnglish	flying
U+439A.0	fUTF8	䎚
U+439A.0	fCNS11643	4-667D
U+439A.0	fGB	3-6C24
U+439A.0	fR/S	 124.15
U+439A.0	fTotalStrokes	 21
U+439A.0	fCangjie	SMQJP
U+439A.0	fFourCorner	15130
U+439A.0	fCantonese	wai6
U+439A.0	fMandarin	hui4; guan4
U+439A.0	fEnglish	[1] the end of bird wings [2] bird wings
U+439B.0	fUTF8	䎛
U+439B.0	fR/S	 125.3
U+439B.0	fTotalStrokes	 9
U+439B.0	fCangjie	JPR
U+439B.0	fCantonese	lou5
U+439B.0	fEnglish	[Korean character]
U+439C.0	fUTF8	䎜
U+439C.0	fCNS11643	3-3E39
U+439C.0	fGB	5-5F45
U+439C.0	fR/S	 125.6
U+439C.0	fTotalStrokes	 12
U+439C.0	fCangjie	JPJKP
U+439C.0	fFourCorner	44710
U+439C.0	fCantonese	fan1
U+439C.0	fMandarin	hun1
U+439C.0	fEnglish	aged; get old; advanced age; great age; old age
U+439D.0	fUTF8	䎝
U+439D.0	fCNS11643	3-3E3A
U+439D.0	fR/S	 125.6
U+439D.0	fTotalStrokes	 12
U+439D.0	fCangjie	JPSMM
U+439D.0	fFourCorner	40120
U+439D.0	fCantonese	zyu3
U+439D.0	fMandarin	chu2; zhu4
U+439D.0	fEnglish	same as &#x7FE5;
U+439E.0	fUTF8	䎞
U+439E.0	fR/S	 125.6
U+439E.0	fTotalStrokes	 12
U+439E.0	fCangjie	JAS
U+439E.0	fCantonese	ze2
U+439E.0	fEnglish	[Korean character]
U+439F.0	fUTF8	䎟
U+439F.0	fCNS11643	4-2B27
U+439F.0	fGB	3-3C48
U+439F.0	fR/S	 126.3
U+439F.0	fTotalStrokes	 9
U+439F.0	fCangjie	VMBL
U+439F.0	fFourCorner	41420
U+439F.0	fCantonese	ji4; hei3; zi4
U+439F.0	fMandarin	er2; nuo4
U+439F.0	fEnglish	[1] fawn on; flatter; please [2] love [3] attractive
U+43A0.0	fUTF8	䎠
U+43A0.0	fCNS11643	4-2B28
U+43A0.0	fGB	3-6063
U+43A0.0	fR/S	 126.3; 126.7; 43.7
U+43A0.0	fTotalStrokes	 9; 10
U+43A0.0	fCangjie	KIMBL
U+43A0.0	fFourCorner	51010
U+43A0.0	fCantonese	ji4; no6; zi4
U+43A0.0	fMandarin	er2; wan2; nuo2
U+43A0.0	fEnglish	[1] twist or to crumple between hands and make something turn [2] same as &#x611E;, &#x61E6;
U+43A1.0	fUTF8	䎡
U+43A1.0	fCNS11643	4-2F38
U+43A1.0	fGB	3-6064
U+43A1.0	fR/S	 126.4
U+43A1.0	fTotalStrokes	 10
U+43A1.0	fCangjie	MBF
U+43A1.0	fFourCorner	10800
U+43A1.0	fCantonese	jyun5; nyun5
U+43A1.0	fMandarin	nuan3; ruan3
U+43A1.0	fEnglish	[1] shrink; recoil; flinch [2] weak [3] soft and meek; gentle [4] corrupted form of &#x800E;
U+43A2.0	fUTF8	䎢
U+43A2.0	fCNS11643	4-2B29
U+43A2.0	fGB	3-602F
U+43A2.0	fR/S	 127.3
U+43A2.0	fTotalStrokes	 9
U+43A2.0	fCangjie	QDON
U+43A2.0	fFourCorner	58910
U+43A2.0	fCantonese	hat1
U+43A2.0	fMandarin	qi3
U+43A2.0	fEnglish	 measure (esp. for grains) by pints and pecks
U+43A3.0	fUTF8	䎣
U+43A3.0	fCNS11643	3-3838
U+43A3.0	fR/S	 127.5
U+43A3.0	fTotalStrokes	 11
U+43A3.0	fCangjie	QDVIO
U+43A3.0	fFourCorner	58900
U+43A3.0	fCantonese	zi6; ci5; piu3
U+43A3.0	fMandarin	si4; xin4
U+43A3.0	fEnglish	same as &#x801C;
U+43A4.0	fUTF8	䎤
U+43A4.0	fCNS11643	4-3F7A
U+43A4.0	fGB	3-6035
U+43A4.0	fR/S	 127.7
U+43A4.0	fTotalStrokes	 13
U+43A4.0	fCangjie	QDSSR
U+43A4.0	fFourCorner	57920
U+43A4.0	fCantonese	guk6
U+43A4.0	fMandarin	ju2; chu2
U+43A4.0	fEnglish	to till; to plough; to cultivate, agricultural implements; farm tools
U+43A5.0	fUTF8	䎥
U+43A5.0	fGB	S-2373
U+43A5.0	fR/S	 127.7
U+43A5.0	fTotalStrokes	 13
U+43A5.0	fCangjie	QDCOR
U+43A5.0	fCantonese	gaak3
U+43A6.0	fUTF8	䎦
U+43A6.0	fCNS11643	4-462E
U+43A6.0	fGB	3-603A
U+43A6.0	fR/S	 127.8
U+43A6.0	fTotalStrokes	 14
U+43A6.0	fCangjie	QDFF
U+43A6.0	fFourCorner	59980
U+43A6.0	fCantonese	jim5; haam2; zim2; zim5
U+43A6.0	fMandarin	yan3
U+43A6.0	fEnglish	to till; to plough, sharp-pointed spade or shovel
U+43A7.0	fUTF8	䎧
U+43A7.0	fCNS11643	4-462D
U+43A7.0	fGB	3-6039
U+43A7.0	fR/S	 127.8
U+43A7.0	fTotalStrokes	 14
U+43A7.0	fCangjie	QDYTR
U+43A7.0	fFourCorner	50960
U+43A7.0	fCantonese	pong5
U+43A7.0	fMandarin	pou2; bang4
U+43A7.0	fEnglish	[1] agricultural implements; farm tools (a plough, a ploughshare) [2] till; plough
U+43A8.0	fUTF8	䎨
U+43A8.0	fCNS11643	4-462F
U+43A8.0	fGB	3-6037
U+43A8.0	fR/S	 127.8
U+43A8.0	fTotalStrokes	 14
U+43A8.0	fCangjie	QDKLU
U+43A8.0	fFourCorner	54910
U+43A8.0	fCantonese	jip3; aam3; zim1
U+43A8.0	fMandarin	an3; ye4; an4
U+43A8.0	fEnglish	[1] plough and sow [2] farm
U+43A9.0	fUTF8	䎩
U+43A9.0	fCNS11643	4-4632
U+43A9.0	fGB	5-5F3A
U+43A9.0	fR/S	 127.8
U+43A9.0	fTotalStrokes	 14
U+43A9.0	fCangjie	QDVVW
U+43A9.0	fFourCorner	52960
U+43A9.0	fHakka	 zih1
U+43A9.0	fCantonese	zi1
U+43A9.0	fMandarin	zi1
U+43A9.0	fEnglish	 plough
U+43AA.0	fUTF8	䎪
U+43AA.0	fCNS11643	4-4630
U+43AA.0	fGB	3-6038
U+43AA.0	fR/S	 127.8
U+43AA.0	fTotalStrokes	 14
U+43AA.0	fCangjie	QDAMI
U+43AA.0	fFourCorner	56940
U+43AA.0	fCantonese	nak6; lak6 (also); neot6
U+43AA.0	fMandarin	ne4
U+43AA.0	fEnglish	[1] the stalk of grain; straw [2] a hammer used to pound grains and corns, etc.
U+43AB.0	fUTF8	䎫
U+43AB.0	fCNS11643	4-4C4F
U+43AB.0	fGB	3-603B
U+43AB.0	fR/S	 127.9
U+43AB.0	fTotalStrokes	 15
U+43AB.0	fCangjie	QDUCE
U+43AB.0	fFourCorner	52940
U+43AB.0	fHakka	 zung1
U+43AB.0	fCantonese	zung1; coeng1
U+43AB.0	fMandarin	chuang1; chuang4; zong3; zong1
U+43AB.0	fEnglish	 sow seed without first ploughing the ground
U+43AC.0	fUTF8	䎬
U+43AC.0	fGB	7-223B
U+43AC.0	fR/S	 127.9
U+43AC.0	fTotalStrokes	 15
U+43AC.0	fCangjie	QDWLI
U+43AC.0	fCantonese	paa4
U+43AC.0	fMandarin	ba4; pa2
U+43AC.0	fEnglish	[1] simplified form of &#x43B1; [2] same as &#x8019;
U+43AD.0	fUTF8	䎭
U+43AD.0	fCNS11643	4-577D
U+43AD.0	fGB	3-603C
U+43AD.0	fR/S	 127.11
U+43AD.0	fTotalStrokes	 17; 16
U+43AD.0	fCangjie	QDYHR
U+43AD.0	fFourCorner	54930
U+43AD.0	fCantonese	cou1; ciu4; cou3
U+43AD.0	fEnglish	a kind of farm tool made of twisted thron strips used to flatten the land
U+43AE.0	fUTF8	䎮
U+43AE.0	fCNS11643	4-577A
U+43AE.0	fGB	3-603E
U+43AE.0	fR/S	 127.11
U+43AE.0	fTotalStrokes	 17; 16
U+43AE.0	fCangjie	QDYCB
U+43AE.0	fFourCorner	50920
U+43AE.0	fCantonese	tai3; taai3; tik1
U+43AE.0	fMandarin	ti4
U+43AE.0	fEnglish	 plant; raise or grow (plants); cultivate
U+43AF.0	fUTF8	䎯
U+43AF.0	fCNS11643	4-577B
U+43AF.0	fGB	3-603D
U+43AF.0	fR/S	 127.11
U+43AF.0	fTotalStrokes	 17; 16
U+43AF.0	fCangjie	QDTLO
U+43AF.0	fFourCorner	54930
U+43AF.0	fCantonese	hon3; hon2; zin5
U+43AF.0	fMandarin	han4; han3
U+43AF.0	fEnglish	[1] winter ploughing [2] plough a dry farmland (upland) [3] till; plough
U+43B0.0	fUTF8	䎰
U+43B0.0	fCNS11643	4-5C70
U+43B0.0	fGB	5-5F40
U+43B0.0	fR/S	 127.12
U+43B0.0	fTotalStrokes	 18
U+43B0.0	fCangjie	QDTAL
U+43B0.0	fFourCorner	52920
U+43B0.0	fHakka	 cok7; sit8; sit7
U+43B0.0	fCantonese	zok6; zok3
U+43B0.0	fMandarin	zuo2
U+43B0.0	fEnglish	a Chinese family name
U+43B1.0	fUTF8	䎱
U+43B1.0	fCNS11643	4-6721
U+43B1.0	fGB	7-223C
U+43B1.0	fR/S	 127.15
U+43B1.0	fTotalStrokes	 21
U+43B1.0	fCangjie	QDWLP
U+43B1.0	fFourCorner	56910
U+43B1.0	fCantonese	paa4
U+43B1.0	fMandarin	bei1; ba4; pa2
U+43B1.0	fEnglish	same as &#x8019;
U+43B2.0	fUTF8	䎲
U+43B2.0	fCNS11643	4-246D
U+43B2.0	fGB	3-6044
U+43B2.0	fR/S	 128.1
U+43B2.0	fTotalStrokes	 7
U+43B2.0	fCangjie	SJN
U+43B2.0	fFourCorner	17110
U+43B2.0	fCantonese	zip3; hei2
U+43B2.0	fMandarin	yi4; zhe2
U+43B2.0	fEnglish	ear lobe; lobule
U+43B3.0	fUTF8	䎳
U+43B3.0	fCNS11643	3-3346
U+43B3.0	fGB	5-5F4C
U+43B3.0	fR/S	 128.4
U+43B3.0	fTotalStrokes	 10
U+43B3.0	fCangjie	SJB
U+43B3.0	fFourCorner	17120
U+43B3.0	fCantonese	waat6; zyut6
U+43B3.0	fMandarin	yue4; wa4; ye4
U+43B3.0	fEnglish	lose of the ear
U+43B4.0	fUTF8	䎴
U+43B4.0	fCNS11643	3-3345
U+43B4.0	fR/S	 128.4
U+43B4.0	fTotalStrokes	 10
U+43B4.0	fCangjie	SJTT
U+43B4.0	fFourCorner	15100
U+43B4.0	fCantonese	gaang1
U+43B4.0	fMandarin	geng1; sheng4
U+43B4.0	fEnglish	[1] a legendary spirit; immortal [2] non-classical form of &#x8056;
U+43B5.0	fUTF8	䎵
U+43B5.0	fCNS11643	3-383A
U+43B5.0	fR/S	 128.5
U+43B5.0	fTotalStrokes	 11
U+43B5.0	fCangjie	SJPH
U+43B5.0	fFourCorner	13100
U+43B5.0	fCantonese	bei3
U+43B5.0	fMandarin	bi4; mi4
U+43B5.0	fEnglish	[1] (corrupted form) [2] look straight forward [3] ashamed [4] look in a mean way
U+43B6.0	fUTF8	䎶
U+43B6.0	fCNS11643	4-3441
U+43B6.0	fGB	5-5F4E
U+43B6.0	fR/S	 128.5
U+43B6.0	fTotalStrokes	 11
U+43B6.0	fCangjie	SJLWL
U+43B6.0	fFourCorner	15100
U+43B6.0	fCantonese	nei6; lei6 (also)
U+43B6.0	fMandarin	er4
U+43B6.0	fEnglish	offering blood to God
U+43B7.0	fUTF8	䎷
U+43B7.0	fCNS11643	4-3A39
U+43B7.0	fGB	3-604F
U+43B7.0	fR/S	 128.6
U+43B7.0	fTotalStrokes	 12
U+43B7.0	fCangjie	SJHJD
U+43B7.0	fFourCorner	15190
U+43B7.0	fCantonese	zyu3
U+43B7.0	fMandarin	zhu4; fu4; shu4
U+43B7.0	fEnglish	[1] an old man with white hair [2] source; head (of a stream) [3] loud cries
U+43B8.0	fUTF8	䎸
U+43B8.0	fCNS11643	5-4170
U+43B8.0	fR/S	 128.7
U+43B8.0	fTotalStrokes	 13
U+43B8.0	fCangjie	SJMMR
U+43B8.0	fFourCorner	11160
U+43B8.0	fCantonese	ng6
U+43B8.0	fMandarin	wu4
U+43B8.0	fEnglish	 hear; listen; obey; understand
U+43B9.0	fUTF8	䎹
U+43B9.0	fCNS11643	3-4374
U+43B9.0	fR/S	 128.7
U+43B9.0	fTotalStrokes	 13
U+43B9.0	fCangjie	HDSJ
U+43B9.0	fFourCorner	20400
U+43B9.0	fCantonese	man4; maan4; maan6
U+43B9.0	fMandarin	wen4; wen2
U+43B9.0	fEnglish	ancient form of &#x805E;
U+43BA.0	fUTF8	䎺
U+43BA.0	fCNS11643	4-4636
U+43BA.0	fGB	3-6053
U+43BA.0	fR/S	 128.8
U+43BA.0	fTotalStrokes	 14
U+43BA.0	fCangjie	SJHBN
U+43BA.0	fFourCorner	12100
U+43BA.0	fCantonese	zai3; zit6
U+43BA.0	fMandarin	zhi4; zhi3
U+43BA.0	fEnglish	[1] learn [2] hear
U+43BB.0	fUTF8	䎻
U+43BB.0	fCNS11643	4-4634
U+43BB.0	fGB	5-5F54
U+43BB.0	fR/S	 128.8
U+43BB.0	fTotalStrokes	 14
U+43BB.0	fCangjie	SJBGR
U+43BB.0	fFourCorner	17120
U+43BB.0	fCantonese	zau3; zaau2
U+43BB.0	fMandarin	zhou3; zhou4
U+43BB.0	fEnglish	[1] hear [2] clear [3] bright [4] ears
U+43BC.0	fUTF8	䎼
U+43BC.0	fCNS11643	4-4635
U+43BC.0	fGB	3-6054
U+43BC.0	fR/S	 128.8
U+43BC.0	fTotalStrokes	 14; 15
U+43BC.0	fCangjie	SJVNE
U+43BC.0	fFourCorner	17130
U+43BC.0	fCantonese	luk6
U+43BC.0	fMandarin	lu4
U+43BC.0	fEnglish	a kind of insect
U+43BD.0	fUTF8	䎽
U+43BD.0	fCNS11643	4-4637
U+43BD.0	fGB	5-5F55
U+43BD.0	fR/S	 128.8
U+43BD.0	fTotalStrokes	 14
U+43BD.0	fCangjie	SJHPA
U+43BD.0	fFourCorner	12160
U+43BD.0	fCantonese	man4
U+43BD.0	fMandarin	sheng1; wen2; wen4
U+43BD.0	fEnglish	ancient form of &#x805E;
U+43BE.0	fUTF8	䎾
U+43BE.0	fCNS11643	3-4924
U+43BE.0	fR/S	 128.8
U+43BE.0	fTotalStrokes	 14
U+43BE.0	fCangjie	SJOMB
U+43BE.0	fFourCorner	18420
U+43BE.0	fCantonese	gwaan2
U+43BE.0	fMandarin	hun3; gun3
U+43BE.0	fEnglish	corrupted form of &#x7754;
U+43BF.0	fUTF8	䎿
U+43BF.0	fCNS11643	3-4E33
U+43BF.0	fR/S	 128.9
U+43BF.0	fTotalStrokes	 15
U+43BF.0	fCangjie	SJHDF
U+43BF.0	fFourCorner	19180
U+43BF.0	fHakka	 ciu1
U+43BF.0	fCantonese	cau4
U+43BF.0	fMandarin	qiu2; qiu1; xiong4
U+43BF.0	fEnglish	a singing in the ears
U+43C0.0	fUTF8	䏀
U+43C0.0	fCNS11643	4-4C52
U+43C0.0	fGB	3-6055
U+43C0.0	fR/S	 128.9
U+43C0.0	fTotalStrokes	 15
U+43C0.0	fCangjie	SJDLN
U+43C0.0	fFourCorner	12400
U+43C0.0	fCantonese	laat6; lat6; lit6
U+43C0.0	fMandarin	ci4; la4
U+43C0.0	fEnglish	inharmonious
U+43C1.0	fUTF8	䏁
U+43C1.0	fCNS11643	4-525A
U+43C1.0	fGB	5-5F5B
U+43C1.0	fR/S	 128.10
U+43C1.0	fTotalStrokes	 16
U+43C1.0	fCangjie	SJJYJ
U+43C1.0	fFourCorner	13140
U+43C1.0	fCantonese	zoi2
U+43C1.0	fMandarin	zai3
U+43C1.0	fEnglish	unable to hear distinctly or clearly
U+43C2.0	fUTF8	䏂
U+43C2.0	fCNS11643	3-5242
U+43C2.0	fGB	5-5F57
U+43C2.0	fR/S	 128.10
U+43C2.0	fTotalStrokes	 16
U+43C2.0	fCangjie	SJHXE
U+43C2.0	fFourCorner	17140
U+43C2.0	fCantonese	sau2
U+43C2.0	fMandarin	sou3
U+43C2.0	fEnglish	[1] clever [2] quick of apprehension [3] with a good faculty of hearing
U+43C3.0	fUTF8	䏃
U+43C3.0	fCNS11643	4-525B
U+43C3.0	fGB	3-6058
U+43C3.0	fR/S	 128.10
U+43C3.0	fTotalStrokes	 16
U+43C3.0	fCangjie	SJBAC
U+43C3.0	fFourCorner	17180
U+43C3.0	fCantonese	min4
U+43C3.0	fMandarin	ming2; mian2; min3
U+43C3.0	fEnglish	 hear; listen carefully
U+43C4.0	fUTF8	䏄
U+43C4.0	fCNS11643	4-5821
U+43C4.0	fGB	3-605C
U+43C4.0	fR/S	 128.11
U+43C4.0	fTotalStrokes	 17; 16
U+43C4.0	fCangjie	SJJCG
U+43C4.0	fFourCorner	13410
U+43C4.0	fHakka	 zhit7; tsiit7
U+43C4.0	fCantonese	dai2; zaat6; zyut3
U+43C4.0	fMandarin	zhi4; di3; duo2
U+43C4.0	fEnglish	[1] not to listen [2] disease of the ears [3] a dull sense of hearing (inefficacious, unresponsive)
U+43C5.0	fUTF8	䏅
U+43C5.0	fCNS11643	4-5824
U+43C5.0	fGB	3-605A
U+43C5.0	fR/S	 128.11
U+43C5.0	fTotalStrokes	 17; 16
U+43C5.0	fCangjie	SJBOF
U+43C5.0	fFourCorner	17190
U+43C5.0	fCantonese	cai3
U+43C5.0	fMandarin	qi4; chi2; qie4
U+43C5.0	fEnglish	[1] hear; listen [2] a keen sense of hearing
U+43C6.0	fUTF8	䏆
U+43C6.0	fCNS11643	4-5823
U+43C6.0	fGB	3-6059
U+43C6.0	fR/S	 128.11
U+43C6.0	fTotalStrokes	 17; 16
U+43C6.0	fCangjie	SJTWA
U+43C6.0	fFourCorner	15160
U+43C6.0	fCantonese	cou4
U+43C6.0	fMandarin	qiao2; cao2; chua4
U+43C6.0	fEnglish	buzzing in the ears; tinnitus aurium
U+43C7.0	fUTF8	䏇
U+43C7.0	fCNS11643	4-5822
U+43C7.0	fGB	5-5F5C
U+43C7.0	fR/S	 128.11
U+43C7.0	fTotalStrokes	 17; 16
U+43C7.0	fCangjie	SJMWF
U+43C7.0	fFourCorner	11190
U+43C7.0	fCantonese	piu3
U+43C7.0	fMandarin	piao4
U+43C7.0	fEnglish	[1] hear; barely enough to hear clearly [2] hearsay; groundless talk
U+43C8.0	fUTF8	䏈
U+43C8.0	fCNS11643	3-5864
U+43C8.0	fR/S	 128.12
U+43C8.0	fTotalStrokes	 18
U+43C8.0	fCangjie	SJVFF
U+43C8.0	fFourCorner	12190
U+43C8.0	fCantonese	lyun4
U+43C8.0	fMandarin	luan2; lian2
U+43C8.0	fEnglish	standard form of &#x806F;
U+43C9.0	fUTF8	䏉
U+43C9.0	fCNS11643	4-6428
U+43C9.0	fGB	5-5F61
U+43C9.0	fR/S	 128.14
U+43C9.0	fTotalStrokes	 20
U+43C9.0	fCangjie	SJAVF
U+43C9.0	fFourCorner	16130
U+43C9.0	fCantonese	sap1; saap1; zaam1
U+43C9.0	fMandarin	shi1
U+43C9.0	fEnglish	 waver; shake ears (said of cattle and horses)
U+43CA.0	fUTF8	䏊
U+43CA.0	fCNS11643	3-5F5C
U+43CA.0	fR/S	 128.16
U+43CA.0	fTotalStrokes	 22
U+43CA.0	fCangjie	SJYBP
U+43CA.0	fFourCorner	11411
U+43CA.0	fCantonese	lung4
U+43CA.0	fMandarin	long2
U+43CA.0	fEnglish	same as &#x807E;
U+43CB.0	fUTF8	䏋
U+43CB.0	fCNS11643	6-566D
U+43CB.0	fR/S	 129.7
U+43CB.0	fTotalStrokes	 13
U+43CB.0	fCangjie	LFPU
U+43CB.0	fFourCorner	50310
U+43CB.0	fCantonese	suk1
U+43CB.0	fMandarin	su4
U+43CB.0	fEnglish	ancient form of &#x8085;
U+43CC.0	fUTF8	䏌
U+43CC.0	fCNS11643	4-232D
U+43CC.0	fGB	5-3136
U+43CC.0	fR/S	 130.2
U+43CC.0	fTotalStrokes	 6
U+43CC.0	fCangjie	CB
U+43CC.0	fFourCorner	80220
U+43CC.0	fCantonese	hat1; ngaat6
U+43CC.0	fMandarin	xi4; qi4; yi4
U+43CC.0	fEnglish	[1] vibrate; vibration [2] same as &#x4F7E;
U+43CD.0	fUTF8	䏍
U+43CD.0	fCNS11643	6-2549
U+43CD.0	fR/S	 130.2
U+43CD.0	fTotalStrokes	 6
U+43CD.0	fCangjie	IB
U+43CD.0	fFourCorner	23220
U+43CD.0	fCantonese	jyun1; zyun3
U+43CD.0	fMandarin	yuan4
U+43CD.0	fEnglish	same as &#x8099;, &#x870E;
U+43CE.0	fUTF8	䏎
U+43CE.0	fCNS11643	4-246F
U+43CE.0	fGB	3-4C3E
U+43CE.0	fR/S	 130.3
U+43CE.0	fTotalStrokes	 7
U+43CE.0	fCangjie	BHNI
U+43CE.0	fFourCorner	77210
U+43CE.0	fCantonese	fung4; hon6
U+43CE.0	fMandarin	han4; feng2; huan4
U+43CE.0	fEnglish	[1] milk; breasts [2] the young of animals, birds
U+43CF.0	fUTF8	䏏
U+43CF.0	fCNS11643	4-246E
U+43CF.0	fGB	3-4C3D
U+43CF.0	fR/S	 130.3
U+43CF.0	fTotalStrokes	 7
U+43CF.0	fCangjie	BMD
U+43CF.0	fFourCorner	71240
U+43CF.0	fCantonese	heoi1
U+43CF.0	fMandarin	xu1
U+43CF.0	fEnglish	same as &#x5401;
U+43D0.0	fUTF8	䏐
U+43D0.0	fCNS11643	3-2A51
U+43D0.0	fGB	5-4E7B
U+43D0.0	fR/S	 130.4
U+43D0.0	fTotalStrokes	 8
U+43D0.0	fCangjie	BDK
U+43D0.0	fFourCorner	75230
U+43D0.0	fCantonese	kyut3
U+43D0.0	fMandarin	jue2; zhuo4
U+43D0.0	fEnglish	[1] the buttocks [2] the anus
U+43D1.0	fUTF8	䏑
U+43D1.0	fCNS11643	6-3B28
U+43D1.0	fR/S	 130.4
U+43D1.0	fTotalStrokes	 10; 9
U+43D1.0	fCangjie	BOBO
U+43D1.0	fFourCorner	27220
U+43D1.0	fCantonese	zek3; ze3
U+43D1.0	fMandarin	zhe4; di4; zhi4
U+43D1.0	fEnglish	same as &#x7099;
U+43D2.0	fUTF8	䏒
U+43D2.0	fCNS11643	3-2A55
U+43D2.0	fGB	5-4E76
U+43D2.0	fR/S	 130.4
U+43D2.0	fTotalStrokes	 8
U+43D2.0	fCangjie	BLLL
U+43D2.0	fFourCorner	72220
U+43D2.0	fCantonese	pin3
U+43D2.0	fMandarin	pian4
U+43D2.0	fEnglish	[1] half of the body [2] non-classical form of &#x80D6;
U+43D3.0	fUTF8	䏓
U+43D3.0	fCNS11643	3-2A50
U+43D3.0	fR/S	 130.4
U+43D3.0	fTotalStrokes	 8
U+43D3.0	fCangjie	BMMU
U+43D3.0	fFourCorner	71210
U+43D3.0	fCantonese	gun2; gwun2 (also)
U+43D3.0	fMandarin	guan3
U+43D3.0	fEnglish	same as &#x8118;
U+43D4.0	fUTF8	䏔
U+43D4.0	fCNS11643	4-2766
U+43D4.0	fGB	3-4C53
U+43D4.0	fR/S	 130.4
U+43D4.0	fTotalStrokes	 8
U+43D4.0	fCangjie	BNG
U+43D4.0	fFourCorner	77210
U+43D4.0	fCantonese	nau5; lau5 (also); naau2
U+43D4.0	fMandarin	zhou3; niu3
U+43D4.0	fEnglish	[1] eat meat [2] same as &#x8098; [3] same as &#x816C; [4] same as &#x8844;
U+43D5.0	fUTF8	䏕
U+43D5.0	fCNS11643	F-2852
U+43D5.0	fGB	3-4C4A
U+43D5.0	fR/S	 130.4
U+43D5.0	fTotalStrokes	 8
U+43D5.0	fCangjie	BMG
U+43D5.0	fFourCorner	7221
U+43D5.0	fCantonese	jam6; nei5; jaam6; zaam6; jaan6; zaan6
U+43D5.0	fMandarin	ren3; run4
U+43D5.0	fEnglish	ancient form of &#x98EA;
U+43D6.0	fUTF8	䏖
U+43D6.0	fCNS11643	4-2762
U+43D6.0	fGB	3-4C52
U+43D6.0	fR/S	 130.4
U+43D6.0	fTotalStrokes	 8
U+43D6.0	fCangjie	BNL
U+43D6.0	fFourCorner	72200
U+43D6.0	fCantonese	zan6; zaan5
U+43D6.0	fMandarin	zhen4
U+43D6.0	fEnglish	[1] wound; bruise; sore [2] scar [3] muscles of the back
U+43D7.0	fUTF8	䏗
U+43D7.0	fCNS11643	4-2767
U+43D7.0	fGB	3-4C49
U+43D7.0	fR/S	 130.4
U+43D7.0	fTotalStrokes	 8
U+43D7.0	fCangjie	BOMN
U+43D7.0	fFourCorner	78210
U+43D7.0	fHakka	 kui3; kui6; kui5; k'wui5
U+43D7.0	fCantonese	gui3; gwui3 (also); koi3; kui2
U+43D7.0	fMandarin	gui4; gai4; kui4
U+43D7.0	fEnglish	a pain in the loins
U+43D8.0	fUTF8	䏘
U+43D8.0	fCNS11643	4-2763
U+43D8.0	fGB	3-4C48
U+43D8.0	fR/S	 130.4
U+43D8.0	fTotalStrokes	 8
U+43D8.0	fCangjie	BSC
U+43D8.0	fFourCorner	71210
U+43D8.0	fCantonese	pat1
U+43D8.0	fMandarin	pi3; pi4
U+43D8.0	fEnglish	[1] a big abdomen [2] a female animal with a fat belly
U+43D9.0	fUTF8	䏙
U+43D9.0	fCNS11643	4-2765
U+43D9.0	fGB	3-4C51
U+43D9.0	fR/S	 130.4
U+43D9.0	fTotalStrokes	 8
U+43D9.0	fCangjie	BLBU
U+43D9.0	fFourCorner	74210
U+43D9.0	fCantonese	dam1; tam2
U+43D9.0	fMandarin	tan3
U+43D9.0	fEnglish	interchangeable with &#x9193;
U+43DA.0	fUTF8	䏚
U+43DA.0	fCNS11643	5-2758
U+43DA.0	fGB	5-4E75
U+43DA.0	fR/S	 130.4
U+43DA.0	fTotalStrokes	 8
U+43DA.0	fCangjie	BFH
U+43DA.0	fFourCorner	79220
U+43DA.0	fCantonese	cau2; miu5
U+43DA.0	fMandarin	miao3; chao3
U+43DA.0	fEnglish	[1] small [2] missing; wanting; lost [3] the floating ribs on the sides of the trunk,
U+43DB.0	fUTF8	䏛
U+43DB.0	fCNS11643	3-2A5A
U+43DB.0	fR/S	 130.4
U+43DB.0	fTotalStrokes	 8
U+43DB.0	fCangjie	BPMM
U+43DB.0	fFourCorner	77220
U+43DB.0	fCantonese	ceon2
U+43DB.0	fMandarin	chun3
U+43DB.0	fEnglish	[1] (a corrupted form) [2] a kind of insect; earthworm [3] strips of meat cut from the flank and dried in the wind
U+43DC.0	fUTF8	䏜
U+43DC.0	fCNS11643	3-2A53
U+43DC.0	fR/S	 130.4
U+43DC.0	fTotalStrokes	 8
U+43DC.0	fCangjie	BNHE
U+43DC.0	fFourCorner	77240
U+43DC.0	fCantonese	hap1; ho1
U+43DC.0	fEnglish	fat; plump; obesity; obese
U+43DD.0	fUTF8	䏝
U+43DD.0	fGB	7-2235
U+43DD.0	fR/S	 130.4
U+43DD.0	fTotalStrokes	 8
U+43DD.0	fCangjie	BQNI
U+43DD.0	fCantonese	zyun1
U+43DD.0	fMandarin	chun2; zhuan3
U+43DD.0	fEnglish	simplified form of &#x819E;
U+43DE.0	fUTF8	䏞
U+43DE.0	fCNS11643	5-2B33
U+43DE.0	fR/S	 130.5
U+43DE.0	fTotalStrokes	 9
U+43DE.0	fCangjie	BJD
U+43DE.0	fFourCorner	75290
U+43DE.0	fCantonese	mut6
U+43DE.0	fMandarin	mo4
U+43DE.0	fEnglish	the belly; the abdomen
U+43DF.0	fUTF8	䏟
U+43DF.0	fCNS11643	4-2B2F
U+43DF.0	fGB	3-4C64
U+43DF.0	fR/S	 130.5
U+43DF.0	fTotalStrokes	 9
U+43DF.0	fCangjie	BPH
U+43DF.0	fFourCorner	73200
U+43DF.0	fCantonese	bit6; bit3
U+43DF.0	fMandarin	bi4; bie4
U+43DF.0	fEnglish	[1] fat meat [2] big; large
U+43E0.0	fUTF8	䏠
U+43E0.0	fCNS11643	4-2B2A
U+43E0.0	fGB	3-4C62
U+43E0.0	fR/S	 130.5
U+43E0.0	fTotalStrokes	 9
U+43E0.0	fCangjie	BYT
U+43E0.0	fFourCorner	70210
U+43E0.0	fCantonese	jap1; zaap1
U+43E0.0	fMandarin	qi4
U+43E0.0	fEnglish	ancient form of &#x6C41;
U+43E1.0	fUTF8	䏡
U+43E1.0	fCNS11643	4-2B2C
U+43E1.0	fGB	3-4C54
U+43E1.0	fR/S	 130.5
U+43E1.0	fTotalStrokes	 9
U+43E1.0	fCangjie	BMMF
U+43E1.0	fFourCorner	71290
U+43E1.0	fCantonese	si6; ce2; fun2; seoi6; to3
U+43E1.0	fMandarin	shi4
U+43E1.0	fEnglish	[1] uncooked or raw meat [2] an organic compound; Proteose
U+43E2.0	fUTF8	䏢
U+43E2.0	fCNS11643	3-2E62
U+43E2.0	fR/S	 130.5
U+43E2.0	fTotalStrokes	 9
U+43E2.0	fCangjie	BDHE
U+43E2.0	fFourCorner	74240
U+43E2.0	fCantonese	bei2
U+43E2.0	fMandarin	bi3
U+43E2.0	fEnglish	[1] meat [2] rough; cracked skin
U+43E3.0	fUTF8	䏣
U+43E3.0	fCNS11643	3-2E5F
U+43E3.0	fR/S	 130.5
U+43E3.0	fTotalStrokes	 9
U+43E3.0	fCangjie	BBM
U+43E3.0	fFourCorner	77210
U+43E3.0	fCantonese	zeoi1; ceoi3
U+43E3.0	fMandarin	qu1; jue2; qu4
U+43E3.0	fEnglish	same as non-classical form &#x86C6;
U+43E4.0	fUTF8	䏤
U+43E4.0	fCNS11643	6-337D
U+43E4.0	fR/S	 130.5
U+43E4.0	fTotalStrokes	 9
U+43E4.0	fCangjie	BSMR
U+43E4.0	fFourCorner	77220
U+43E4.0	fCantonese	seon3; sei3; si3
U+43E4.0	fMandarin	si4
U+43E4.0	fEnglish	same as &#x9856;, &#x56DF;
U+43E5.0	fUTF8	䏥
U+43E5.0	fCNS11643	5-2B39
U+43E5.0	fGB	3-4C5C
U+43E5.0	fR/S	 130.5
U+43E5.0	fTotalStrokes	 9
U+43E5.0	fCangjie	BGB
U+43E5.0	fFourCorner	75250
U+43E5.0	fCantonese	taam1
U+43E5.0	fEnglish	[1] dry [2] dandruff-like scales of dead flesh and skin
U+43E6.0	fUTF8	䏦
U+43E6.0	fCNS11643	3-3351
U+43E6.0	fGB	5-4F2C
U+43E6.0	fR/S	 130.6
U+43E6.0	fTotalStrokes	 10
U+43E6.0	fCangjie	BHJR
U+43E6.0	fFourCorner	72260
U+43E6.0	fCantonese	gwaat3
U+43E6.0	fMandarin	tian2; hua2; wan3
U+43E6.0	fEnglish	[1] fat of animals or plants [2] same as &#x80CB;
U+43E7.0	fUTF8	䏧
U+43E7.0	fCNS11643	4-2F41
U+43E7.0	fGB	3-4C77
U+43E7.0	fR/S	 130.6
U+43E7.0	fTotalStrokes	 10
U+43E7.0	fCangjie	BNIN
U+43E7.0	fFourCorner	77220
U+43E7.0	fCantonese	naa6; laa6 (also); naa4
U+43E7.0	fMandarin	na2
U+43E7.0	fEnglish	[1] fatty; greasy [2] not thick; not tight [3] plump and pretty of the flesh
U+43E8.0	fUTF8	䏨
U+43E8.0	fCNS11643	4-2F46
U+43E8.0	fGB	3-4C74
U+43E8.0	fR/S	 130.6
U+43E8.0	fTotalStrokes	 10
U+43E8.0	fCangjie	BHRR
U+43E8.0	fFourCorner	77270
U+43E8.0	fCantonese	fui2; deoi1; gwaai3; kui2; waa2
U+43E8.0	fMandarin	hui3
U+43E8.0	fEnglish	[1] swell up greatly [2] swell; swelling; boil
U+43E9.0	fUTF8	䏩
U+43E9.0	fCNS11643	4-2F47
U+43E9.0	fR/S	 130.6
U+43E9.0	fTotalStrokes	 10
U+43E9.0	fCangjie	BOMR
U+43E9.0	fFourCorner	78260
U+43E9.0	fCantonese	hip3; kaap1; lai1
U+43E9.0	fMandarin	xi1
U+43E9.0	fEnglish	same as &#x8105;
U+43EA.0	fUTF8	䏪
U+43EA.0	fCNS11643	4-2F3F
U+43EA.0	fGB	3-4C6A
U+43EA.0	fR/S	 130.6
U+43EA.0	fTotalStrokes	 10
U+43EA.0	fCangjie	BSJ
U+43EA.0	fFourCorner	71240
U+43EA.0	fCantonese	nei6; lei6 (also)
U+43EA.0	fMandarin	er4
U+43EA.0	fEnglish	tendon (of meat animals); sinews; muscles
U+43EB.0	fUTF8	䏫
U+43EB.0	fCNS11643	6-3B31
U+43EB.0	fR/S	 130.6
U+43EB.0	fTotalStrokes	 10; 9
U+43EB.0	fCangjie	BOD
U+43EB.0	fFourCorner	74290
U+43EB.0	fCantonese	jau1; caau3; zaau1
U+43EB.0	fMandarin	xiu1
U+43EB.0	fEnglish	in between of the abdomen and the spine; the spinal column
U+43EC.0	fUTF8	䏬
U+43EC.0	fCNS11643	5-2F6D
U+43EC.0	fR/S	 130.6
U+43EC.0	fTotalStrokes	 10
U+43EC.0	fCangjie	BIHQ
U+43EC.0	fFourCorner	73250
U+43EC.0	fCantonese	mau4; maau4; min4; mui4
U+43EC.0	fMandarin	mou2
U+43EC.0	fEnglish	[1] the spine; the spinal column [2] the ridge
U+43ED.0	fUTF8	䏭
U+43ED.0	fR/S	 130.6
U+43ED.0	fTotalStrokes	 10
U+43ED.0	fCangjie	BHJD
U+43ED.0	fHakka	 zhu1; zu1
U+43ED.0	fCantonese	zyu1
U+43ED.0	fEnglish	the cheeks
U+43EE.0	fUTF8	䏮
U+43EE.0	fR/S	 130.6
U+43EE.0	fTotalStrokes	 10
U+43EE.0	fCangjie	BKSO
U+43EE.0	fCantonese	hip3
U+43EE.0	fMandarin	xi2; xie2
U+43EE.0	fEnglish	abbreviated form of &#x8107;
U+43EF.0	fUTF8	䏯
U+43EF.0	fCNS11643	3-3846
U+43EF.0	fR/S	 130.7
U+43EF.0	fTotalStrokes	 11
U+43EF.0	fCangjie	BGP
U+43EF.0	fFourCorner	74230
U+43EF.0	fCantonese	zi3
U+43EF.0	fMandarin	zhi4
U+43EF.0	fEnglish	same as &#x75E3;
U+43F0.0	fUTF8	䏰
U+43F0.0	fCNS11643	4-344C
U+43F0.0	fGB	5-4F3A
U+43F0.0	fR/S	 130.7
U+43F0.0	fTotalStrokes	 11
U+43F0.0	fCangjie	BSIP
U+43F0.0	fFourCorner	77230
U+43F0.0	fHakka	 ngiun3
U+43F0.0	fCantonese	jeon6; zaan2
U+43F0.0	fMandarin	ren3
U+43F0.0	fEnglish	a district under the Han dynasty
U+43F1.0	fUTF8	䏱
U+43F1.0	fCNS11643	4-3448
U+43F1.0	fGB	3-4D2D
U+43F1.0	fR/S	 130.7
U+43F1.0	fTotalStrokes	 11
U+43F1.0	fCangjie	BSSR
U+43F1.0	fFourCorner	77220
U+43F1.0	fCantonese	guk1
U+43F1.0	fMandarin	ju2
U+43F1.0	fEnglish	[1] body [2] fat; plump
U+43F2.0	fUTF8	䏲
U+43F2.0	fCNS11643	3-3844
U+43F2.0	fGB	5-4F39
U+43F2.0	fR/S	 130.7
U+43F2.0	fTotalStrokes	 11
U+43F2.0	fCangjie	BCNH
U+43F2.0	fFourCorner	78220
U+43F2.0	fHakka	 diet7;  ti2; toi1
U+43F2.0	fCantonese	dit6
U+43F2.0	fMandarin	die2
U+43F2.0	fEnglish	protruded bones; swelling; crooked; crooked nose
U+43F3.0	fUTF8	䏳
U+43F3.0	fCNS11643	4-344B
U+43F3.0	fGB	3-4D22
U+43F3.0	fR/S	 130.7
U+43F3.0	fTotalStrokes	 11
U+43F3.0	fCangjie	BQHL
U+43F3.0	fFourCorner	72220
U+43F3.0	fCantonese	zit3
U+43F3.0	fMandarin	zhe4
U+43F3.0	fEnglish	[1] olein (in chemistry) [2] oil and grease; fats (esp. animal) [3] the ribs and skin
U+43F4.0	fUTF8	䏴
U+43F4.0	fCNS11643	4-3452
U+43F4.0	fGB	3-4D24
U+43F4.0	fR/S	 130.7
U+43F4.0	fTotalStrokes	 11
U+43F4.0	fCangjie	BFB
U+43F4.0	fFourCorner	79220
U+43F4.0	fCantonese	saau3; sau2
U+43F4.0	fMandarin	shao4; she4
U+43F4.0	fEnglish	[1] dim; ambiguous; unclear eyesight [2] sharp ends of something
U+43F5.0	fUTF8	䏵
U+43F5.0	fCNS11643	4-3451
U+43F5.0	fGB	3-4C7E
U+43F5.0	fR/S	 130.7
U+43F5.0	fTotalStrokes	 11
U+43F5.0	fCangjie	BIUH
U+43F5.0	fFourCorner	73210
U+43F5.0	fCantonese	mung5; mong5
U+43F5.0	fMandarin	mang3; meng3
U+43F5.0	fEnglish	[1] plump [2] rich and full [3] plentiful [4] swelling [5] a plumpy trunk (body)
U+43F6.0	fUTF8	䏶
U+43F6.0	fCNS11643	4-344A
U+43F6.0	fGB	3-4D23
U+43F6.0	fR/S	 130.7
U+43F6.0	fTotalStrokes	 11
U+43F6.0	fCangjie	BPPG
U+43F6.0	fFourCorner	71210
U+43F6.0	fCantonese	bai3; baai6
U+43F6.0	fMandarin	bi4
U+43F6.0	fEnglish	[1] the stomach; gizzard (of birds and fowls) [2] same as &#x9AC0;
U+43F7.0	fUTF8	䏷
U+43F7.0	fCNS11643	4-3453
U+43F7.0	fR/S	 130.7
U+43F7.0	fTotalStrokes	 11
U+43F7.0	fCangjie	BAMJ
U+43F7.0	fFourCorner	76240
U+43F7.0	fCantonese	hon6; hong2
U+43F7.0	fMandarin	han4
U+43F7.0	fEnglish	medicine for a sore; boil (injured by a sword or a knief)
U+43F8.0	fUTF8	䏸
U+43F8.0	fCNS11643	6-4358
U+43F8.0	fR/S	 130.7
U+43F8.0	fTotalStrokes	 11
U+43F8.0	fCangjie	BMMR
U+43F8.0	fFourCorner	71260
U+43F8.0	fCantonese	jyu4; zyu4
U+43F8.0	fMandarin	yu2
U+43F8.0	fEnglish	[1] fish [2] eyes of the fishes
U+43F9.0	fUTF8	䏹
U+43F9.0	fCNS11643	3-3840
U+43F9.0	fR/S	 130.7
U+43F9.0	fTotalStrokes	 11
U+43F9.0	fCangjie	BBUU
U+43F9.0	fFourCorner	76210
U+43F9.0	fCantonese	jin6; saam1; zin2; zin6
U+43F9.0	fMandarin	xian4
U+43F9.0	fEnglish	 watch stealthily
U+43FA.0	fUTF8	䏺
U+43FA.0	fCNS11643	3-383F
U+43FA.0	fR/S	 130.7
U+43FA.0	fTotalStrokes	 11
U+43FA.0	fCangjie	BHEJ
U+43FA.0	fFourCorner	77250
U+43FA.0	fCantonese	pong1
U+43FA.0	fEnglish	 swell; swelling
U+43FB.0	fUTF8	䏻
U+43FB.0	fCNS11643	3-3843
U+43FB.0	fR/S	 130.7
U+43FB.0	fTotalStrokes	 11
U+43FB.0	fCangjie	IBGI
U+43FB.0	fFourCorner	24230
U+43FB.0	fCantonese	nang4; lang4 (also); naang4
U+43FB.0	fMandarin	neng2
U+43FB.0	fEnglish	same as &#x80FD;
U+43FC.0	fUTF8	䏼
U+43FC.0	fCNS11643	4-3A49
U+43FC.0	fGB	3-4C55
U+43FC.0	fR/S	 130.8
U+43FC.0	fTotalStrokes	 12
U+43FC.0	fCangjie	BII
U+43FC.0	fFourCorner	73250
U+43FC.0	fCantonese	caan4
U+43FC.0	fMandarin	zan4; can2
U+43FC.0	fEnglish	[1] what is left over of the food (of birds and animals) [2] a big belly
U+43FD.0	fUTF8	䏽
U+43FD.0	fCNS11643	4-3A3A
U+43FD.0	fGB	3-4D3C
U+43FD.0	fR/S	 130.8
U+43FD.0	fTotalStrokes	 12
U+43FD.0	fCangjie	BYTR
U+43FD.0	fFourCorner	70260
U+43FD.0	fCantonese	bau6; bou6
U+43FD.0	fMandarin	bu4
U+43FD.0	fEnglish	[1] meat pulp; mashed meat cooked with soybean sauce [2] a family name
U+43FE.0	fUTF8	䏾
U+43FE.0	fCNS11643	3-3E3F
U+43FE.0	fGB	5-4F3B
U+43FE.0	fR/S	 130.8
U+43FE.0	fTotalStrokes	 12
U+43FE.0	fCangjie	BQKQ
U+43FE.0	fFourCorner	75250
U+43FE.0	fCantonese	pong1; pong4
U+43FE.0	fMandarin	pang1
U+43FE.0	fEnglish	non-classical form of &#x43FA;
U+43FF.0	fUTF8	䏿
U+43FF.0	fCNS11643	4-3A45
U+43FF.0	fGB	3-4E2E
U+43FF.0	fR/S	 130.8
U+43FF.0	fTotalStrokes	 12
U+43FF.0	fCangjie	IKB
U+43FF.0	fFourCorner	38227
U+43FF.0	fCantonese	kai2
U+43FF.0	fMandarin	qi3
U+43FF.0	fEnglish	the calf of the legs
U+4400.0	fUTF8	䐀
U+4400.0	fCNS11643	5-3B4D
U+4400.0	fR/S	 130.8
U+4400.0	fTotalStrokes	 12
U+4400.0	fCangjie	BKMR
U+4400.0	fFourCorner	74220
U+4400.0	fCantonese	gei3
U+4400.0	fMandarin	ji4
U+4400.0	fEnglish	 dismember body of the livestock
U+4401.0	fUTF8	䐁
U+4401.0	fCNS11643	4-3A4A
U+4401.0	fGB	5-4F3F
U+4401.0	fR/S	 130.8
U+4401.0	fTotalStrokes	 12
U+4401.0	fCangjie	BMSO
U+4401.0	fFourCorner	71230
U+4401.0	fHakka	 tun2
U+4401.0	fCantonese	duk1; doek3; zaa1
U+4401.0	fMandarin	zhuo2; niao3
U+4401.0	fEnglish	[1] the buttocks; the rump [2] fat; plump [3] one of the seven constellations; name of a star [4] the anus
U+4402.0	fUTF8	䐂
U+4402.0	fCNS11643	4-3A41
U+4402.0	fGB	3-4D40
U+4402.0	fR/S	 130.8
U+4402.0	fTotalStrokes	 12
U+4402.0	fCangjie	BVNE
U+4402.0	fFourCorner	77230
U+4402.0	fCantonese	luk6; lyun4
U+4402.0	fMandarin	lu4
U+4402.0	fEnglish	[1] fat of animals or plants [2] fat; plump; obese
U+4403.0	fUTF8	䐃
U+4403.0	fCNS11643	4-3A40
U+4403.0	fGB	3-4D34
U+4403.0	fR/S	 130.8
U+4403.0	fTotalStrokes	 12
U+4403.0	fCangjie	BWHD
U+4403.0	fFourCorner	76200
U+4403.0	fCantonese	kwan5; kwaan3
U+4403.0	fMandarin	jun3; jiong3
U+4403.0	fEnglish	[1] fat in the abdomen or intestine [2] protuberances of the muscle [3] fetus inside the belly
U+4404.0	fUTF8	䐄
U+4404.0	fCNS11643	4-3A42
U+4404.0	fGB	3-4D3A
U+4404.0	fR/S	 130.8
U+4404.0	fTotalStrokes	 12
U+4404.0	fCangjie	BNHX
U+4404.0	fFourCorner	77270
U+4404.0	fCantonese	haam6
U+4404.0	fMandarin	xian4; han4; lian3
U+4404.0	fEnglish	[1] the unsatisfaction of hunger requires meat [2] cook the meat
U+4405.0	fUTF8	䐅
U+4405.0	fCNS11643	4-3A3D
U+4405.0	fGB	3-4D42
U+4405.0	fR/S	 130.8
U+4405.0	fTotalStrokes	 12
U+4405.0	fCangjie	BSOK
U+4405.0	fFourCorner	77230
U+4405.0	fCantonese	hei1; zi4
U+4405.0	fMandarin	xi1; yi2
U+4405.0	fEnglish	[1] the buttocks; the bottom; the rump [2] groan; moan
U+4406.0	fUTF8	䐆
U+4406.0	fGB	3-4D38
U+4406.0	fR/S	 130.8
U+4406.0	fTotalStrokes	 12
U+4406.0	fCangjie	BBD
U+4406.0	fFourCorner	7229
U+4406.0	fCantonese	coi2; co1
U+4406.0	fMandarin	cai3; cai4
U+4406.0	fEnglish	big belly; swollen belly
U+4407.0	fUTF8	䐇
U+4407.0	fCNS11643	3-3E46
U+4407.0	fGB	5-4F45
U+4407.0	fR/S	 130.8
U+4407.0	fTotalStrokes	 12
U+4407.0	fCangjie	BPHA
U+4407.0	fFourCorner	77260
U+4407.0	fCantonese	man5; seon4
U+4407.0	fMandarin	chun2
U+4407.0	fEnglish	[1] clear; bright [2] same as &#x543B; [3] same as &#x5507;
U+4408.0	fUTF8	䐈
U+4408.0	fCNS11643	4-3A3F
U+4408.0	fGB	3-4D2F
U+4408.0	fR/S	 130.8
U+4408.0	fTotalStrokes	 12
U+4408.0	fCangjie	BJBM
U+4408.0	fFourCorner	74210
U+4408.0	fHakka	 zhit7; tsiit7
U+4408.0	fCantonese	zik1; piu1
U+4408.0	fMandarin	zhi2
U+4408.0	fEnglish	[1] a piece of one foot two inches dried seasoned meat [2] paste up; attach to; stick up; glue [3] same as &#x6B96;
U+4409.0	fUTF8	䐉
U+4409.0	fCNS11643	4-3A44
U+4409.0	fGB	3-4D44
U+4409.0	fR/S	 130.8
U+4409.0	fTotalStrokes	 12
U+4409.0	fCangjie	BCCW
U+4409.0	fFourCorner	72260
U+4409.0	fCantonese	zi3
U+4409.0	fMandarin	zi4
U+4409.0	fEnglish	[1] fat [2] interchangeable with &#x8166;
U+440A.0	fUTF8	䐊
U+440A.0	fCNS11643	4-3A4B
U+440A.0	fGB	5-4F40
U+440A.0	fR/S	 130.8
U+440A.0	fTotalStrokes	 12
U+440A.0	fCangjie	BAPP
U+440A.0	fFourCorner	76210
U+440A.0	fCantonese	gwan1; kwaan1
U+440A.0	fMandarin	hun2; da2; hun4
U+440A.0	fEnglish	[1] same as &#x45B5; [2] same as &#x991B;
U+440B.0	fUTF8	䐋
U+440B.0	fCNS11643	6-497E
U+440B.0	fR/S	 130.8; 74.8
U+440B.0	fTotalStrokes	 12
U+440B.0	fCangjie	BCIP
U+440B.0	fFourCorner	78230
U+440B.0	fCantonese	cung1
U+440B.0	fMandarin	cong1
U+440B.0	fEnglish	[1] sick; illness; disease [2] red
U+440C.0	fUTF8	䐌
U+440C.0	fCNS11643	6-4D54
U+440C.0	fR/S	 130.8
U+440C.0	fTotalStrokes	 12
U+440C.0	fCangjie	TCB
U+440C.0	fFourCorner	55220
U+440C.0	fCantonese	tin2
U+440C.0	fMandarin	tian3; zhou4
U+440C.0	fEnglish	ancient form of &#x8146;
U+440D.0	fUTF8	䐍
U+440D.0	fCNS11643	6-5676
U+440D.0	fR/S	 130.9
U+440D.0	fTotalStrokes	 13
U+440D.0	fCangjie	BGRT
U+440D.0	fFourCorner	74210
U+440D.0	fCantonese	zo3; zaau1
U+440D.0	fMandarin	chu4; zhou4
U+440D.0	fEnglish	same as &#x4422;
U+440E.0	fUTF8	䐎
U+440E.0	fCNS11643	4-4027
U+440E.0	fGB	3-4D4C
U+440E.0	fR/S	 130.9
U+440E.0	fTotalStrokes	 13
U+440E.0	fCangjie	BAMO
U+440E.0	fFourCorner	76280
U+440E.0	fCantonese	dai1; daai1; daai6
U+440E.0	fMandarin	di1
U+440E.0	fEnglish	fat; greasy
U+440F.0	fUTF8	䐏
U+440F.0	fCNS11643	3-4378
U+440F.0	fGB	5-4F49
U+440F.0	fR/S	 130.9
U+440F.0	fTotalStrokes	 13
U+440F.0	fCangjie	BQKA
U+440F.0	fFourCorner	75260
U+440F.0	fHakka	 chun3; cun3
U+440F.0	fCantonese	ceon2
U+440F.0	fMandarin	chun3
U+440F.0	fEnglish	corpulent; fat
U+4410.0	fUTF8	䐐
U+4410.0	fCNS11643	4-402A
U+4410.0	fGB	5-4F50
U+4410.0	fR/S	 130.9
U+4410.0	fTotalStrokes	 13
U+4410.0	fCangjie	BHDF
U+4410.0	fFourCorner	79280
U+4410.0	fCantonese	cau1
U+4410.0	fMandarin	qiu1
U+4410.0	fEnglish	[1] curved part of the knee [2] between the thigh and calf
U+4411.0	fUTF8	䐑
U+4411.0	fCNS11643	4-4023
U+4411.0	fGB	5-4F4B
U+4411.0	fR/S	 130.9
U+4411.0	fTotalStrokes	 13
U+4411.0	fCangjie	BPTD
U+4411.0	fFourCorner	74290
U+4411.0	fHakka	 tiap8
U+4411.0	fCantonese	zip3; bai3; dip6; pung3; sip3; sit3
U+4411.0	fMandarin	zhe2
U+4411.0	fEnglish	[1] slice off meat [2] mince; hash
U+4412.0	fUTF8	䐒
U+4412.0	fCNS11643	4-4026
U+4412.0	fGB	3-4D4B
U+4412.0	fR/S	 130.9
U+4412.0	fTotalStrokes	 13
U+4412.0	fCangjie	BKNN
U+4412.0	fFourCorner	74220
U+4412.0	fHakka	 cha1; ca1
U+4412.0	fCantonese	zaa1
U+4412.0	fMandarin	zha1
U+4412.0	fEnglish	open in grain (meat)
U+4413.0	fUTF8	䐓
U+4413.0	fCNS11643	3-437A
U+4413.0	fGB	5-4F28
U+4413.0	fR/S	 130.9; 74.9
U+4413.0	fTotalStrokes	 13
U+4413.0	fCangjie	BMBC
U+4413.0	fFourCorner	71280
U+4413.0	fCantonese	jau4; zaau4
U+4413.0	fMandarin	rou2; ru4
U+4413.0	fEnglish	[1] soft [2] same as &#x811C;
U+4414.0	fUTF8	䐔
U+4414.0	fCNS11643	4-3F7D
U+4414.0	fGB	3-4D58
U+4414.0	fR/S	 130.9
U+4414.0	fTotalStrokes	 13
U+4414.0	fCangjie	BISB
U+4414.0	fFourCorner	73227
U+4414.0	fHakka	 bien3; pian5
U+4414.0	fCantonese	ban2; baan3
U+4414.0	fMandarin	bin3; bin4
U+4414.0	fEnglish	[1] muscle [2] tendon
U+4415.0	fUTF8	䐕
U+4415.0	fCNS11643	4-4028
U+4415.0	fGB	3-4D4D
U+4415.0	fR/S	 130.9
U+4415.0	fTotalStrokes	 13
U+4415.0	fCangjie	BRSJ
U+4415.0	fFourCorner	76240
U+4415.0	fCantonese	cap1; caap1; tou1
U+4415.0	fMandarin	ji2
U+4415.0	fEnglish	[1] oil and grease; fats [2] the bursting of a wound
U+4416.0	fUTF8	䐖
U+4416.0	fCNS11643	3-4421
U+4416.0	fR/S	 130.9
U+4416.0	fTotalStrokes	 13
U+4416.0	fCangjie	BSFD
U+4416.0	fFourCorner	77290
U+4416.0	fCantonese	si1; ngaai4; zi4
U+4416.0	fMandarin	xi1; yi2
U+4416.0	fEnglish	same as &#x3567;
U+4417.0	fUTF8	䐗
U+4417.0	fCNS11643	4-402C
U+4417.0	fGB	5-4F3E
U+4417.0	fR/S	 130.9; 130.8
U+4417.0	fTotalStrokes	 13; 12
U+4417.0	fCangjie	BJKA
U+4417.0	fFourCorner	74260
U+4417.0	fCantonese	zyu1
U+4417.0	fMandarin	zhu1; du3
U+4417.0	fEnglish	[1] same as &#x8C6C; [2] same as &#x809A;
U+4418.0	fUTF8	䐘
U+4418.0	fCNS11643	3-437D
U+4418.0	fR/S	 130.9
U+4418.0	fTotalStrokes	 13
U+4418.0	fCangjie	BBME
U+4418.0	fFourCorner	72240
U+4418.0	fCantonese	koek6
U+4418.0	fMandarin	jue2
U+4418.0	fEnglish	[1] laugh heartily; roar with laughter [2] tongue of the cow
U+4419.0	fUTF8	䐙
U+4419.0	fCNS11643	3-4379
U+4419.0	fGB	5-4F4A
U+4419.0	fR/S	 130.9
U+4419.0	fTotalStrokes	 13
U+4419.0	fCangjie	BTLJ
U+4419.0	fFourCorner	74250
U+4419.0	fCantonese	gaak3
U+4419.0	fMandarin	ge2; yi4
U+4419.0	fEnglish	same as &#x8188;
U+441A.0	fUTF8	䐚
U+441A.0	fCNS11643	5-346A
U+441A.0	fGB	3-4D2C
U+441A.0	fR/S	 130.9; 130.7
U+441A.0	fTotalStrokes	 11; 13
U+441A.0	fCangjie	BAIL
U+441A.0	fFourCorner	77220
U+441A.0	fCantonese	zik1
U+441A.0	fMandarin	ji2; ji4
U+441A.0	fEnglish	[1] good graces; great favors [2] luster
U+441B.0	fUTF8	䐛
U+441B.0	fCNS11643	4-4025
U+441B.0	fGB	3-4D4A
U+441B.0	fR/S	 130.9
U+441B.0	fTotalStrokes	 13
U+441B.0	fCangjie	BKSJ
U+441B.0	fFourCorner	74240
U+441B.0	fCantonese	dap1; daap1 (also); dap3
U+441B.0	fMandarin	da2
U+441B.0	fEnglish	 loosen; widen skin
U+441C.0	fUTF8	䐜
U+441C.0	fCNS11643	4-4645
U+441C.0	fGB	3-4D5D
U+441C.0	fR/S	 130.10
U+441C.0	fTotalStrokes	 14
U+441C.0	fCangjie	BJBC
U+441C.0	fFourCorner	74280
U+441C.0	fHakka	 zhin1; zihn1
U+441C.0	fCantonese	can1
U+441C.0	fMandarin	chen1
U+441C.0	fEnglish	dropsical swelling
U+441D.0	fUTF8	䐝
U+441D.0	fCNS11643	4-463E
U+441D.0	fGB	3-4D25
U+441D.0	fR/S	 130.10
U+441D.0	fTotalStrokes	 14
U+441D.0	fCangjie	BFBC
U+441D.0	fFourCorner	79280
U+441D.0	fCantonese	so3; so2
U+441D.0	fMandarin	suo3
U+441D.0	fEnglish	thick; heavy fat of animals or plants
U+441E.0	fUTF8	䐞
U+441E.0	fCNS11643	4-463C
U+441E.0	fGB	3-4D62
U+441E.0	fR/S	 130.10
U+441E.0	fTotalStrokes	 14
U+441E.0	fCangjie	BNMM
U+441E.0	fFourCorner	77220
U+441E.0	fCantonese	joek6; zoek6
U+441E.0	fMandarin	ruo4
U+441E.0	fEnglish	[1] the thin membrane between skin and flesh that protects internal organs or tissues in the human body [2] weak; fragile; delicate [3] flesh; meat
U+441F.0	fUTF8	䐟
U+441F.0	fCNS11643	4-463D
U+441F.0	fGB	3-4D5C
U+441F.0	fR/S	 130.10
U+441F.0	fTotalStrokes	 14
U+441F.0	fCangjie	BTTB
U+441F.0	fFourCorner	75240
U+441F.0	fCantonese	hong2; gong2; hong6; ngaau1
U+441F.0	fMandarin	xiang3; xiang4
U+441F.0	fEnglish	[1] fat; plump [2] bent; curved feet
U+4420.0	fUTF8	䐠
U+4420.0	fCNS11643	4-463F
U+4420.0	fGB	3-4D47
U+4420.0	fR/S	 130.10
U+4420.0	fTotalStrokes	 14
U+4420.0	fCangjie	BTYU
U+4420.0	fFourCorner	74210
U+4420.0	fCantonese	fong1
U+4420.0	fMandarin	hang1; huang3
U+4420.0	fEnglish	[1] between the flesh [2] name of state in Southern China (of minority group) [3] dim moon
U+4421.0	fUTF8	䐡
U+4421.0	fCNS11643	3-5248
U+4421.0	fR/S	 130.10
U+4421.0	fTotalStrokes	 16
U+4421.0	fCangjie	YSOB
U+4421.0	fFourCorner	00227
U+4421.0	fCantonese	ci4
U+4421.0	fMandarin	qi2
U+4421.0	fEnglish	same as &#x81CD;
U+4422.0	fUTF8	䐢
U+4422.0	fCNS11643	4-4642
U+4422.0	fGB	3-4C60
U+4422.0	fR/S	 130.10
U+4422.0	fTotalStrokes	 14
U+4422.0	fCangjie	BPUU
U+4422.0	fFourCorner	77220
U+4422.0	fCantonese	zo3
U+4422.0	fMandarin	zhou4; zhu4
U+4422.0	fEnglish	[1] wrinkle; contract [2] beautiful; elegant [3] dried and seasoned meat, excellent food; delicacies
U+4423.0	fUTF8	䐣
U+4423.0	fCNS11643	4-4640
U+4423.0	fGB	3-4D27
U+4423.0	fR/S	 130.10
U+4423.0	fTotalStrokes	 14
U+4423.0	fCangjie	BRBC
U+4423.0	fFourCorner	76280
U+4423.0	fCantonese	seon3; syun2
U+4423.0	fMandarin	sun3; cuo4
U+4423.0	fEnglish	soup made of cutted meat mixed with blood,
U+4424.0	fUTF8	䐤
U+4424.0	fCNS11643	4-463B
U+4424.0	fGB	3-4D55
U+4424.0	fR/S	 130.10
U+4424.0	fTotalStrokes	 14
U+4424.0	fCangjie	BTQM
U+4424.0	fFourCorner	78210
U+4424.0	fHakka	  cai1
U+4424.0	fCantonese	caai1; cai1; zai6
U+4424.0	fMandarin	chai1; chi2; cuo2; qi4; zhan4
U+4424.0	fEnglish	strips of meat dried dor use in winter
U+4425.0	fUTF8	䐥
U+4425.0	fCNS11643	3-4929
U+4425.0	fGB	5-4F5D
U+4425.0	fR/S	 130.10
U+4425.0	fTotalStrokes	 14
U+4425.0	fCangjie	BCIM
U+4425.0	fFourCorner	78220
U+4425.0	fHakka	 wung3; vung3
U+4425.0	fCantonese	ung3; ngung3 (also); ong3 (also); ngong3 (also)
U+4425.0	fMandarin	weng3
U+4425.0	fEnglish	fetid; stinking
U+4426.0	fUTF8	䐦
U+4426.0	fCNS11643	3-4928
U+4426.0	fR/S	 130.10
U+4426.0	fTotalStrokes	 14
U+4426.0	fCangjie	BGIT
U+4426.0	fFourCorner	74210
U+4426.0	fCantonese	hap1; haap6; hau2; hot3; lim5
U+4426.0	fMandarin	ke4
U+4426.0	fEnglish	sleepy
U+4427.0	fUTF8	䐧
U+4427.0	fCNS11643	4-4638
U+4427.0	fGB	5-4F5E
U+4427.0	fR/S	 130.10
U+4427.0	fTotalStrokes	 14
U+4427.0	fCangjie	BYRB
U+4427.0	fFourCorner	70220
U+4427.0	fCantonese	hou3
U+4427.0	fMandarin	kao4
U+4427.0	fEnglish	[1] same as &#x7292; [2] same as &#x81DB;
U+4428.0	fUTF8	䐨
U+4428.0	fCNS11643	4-4647
U+4428.0	fGB	3-4F33
U+4428.0	fR/S	 130.10
U+4428.0	fTotalStrokes	 14
U+4428.0	fCangjie	GBHNE
U+4428.0	fFourCorner	47240
U+4428.0	fHakka	 guk7; kwuk7
U+4428.0	fCantonese	guk1
U+4428.0	fMandarin	gu3
U+4428.0	fEnglish	[1] instep [2] hind feet of cattle [3] plain
U+4429.0	fUTF8	䐩
U+4429.0	fCNS11643	4-4641
U+4429.0	fGB	3-4C70
U+4429.0	fR/S	 130.10
U+4429.0	fTotalStrokes	 14
U+4429.0	fCangjie	BUMT
U+4429.0	fFourCorner	72210
U+4429.0	fCantonese	goi1
U+4429.0	fMandarin	gai1; kai3
U+4429.0	fEnglish	fat; plump, the fetus of livestock, delicious meat
U+442A.0	fUTF8	䐪
U+442A.0	fGB	3-4D5E
U+442A.0	fR/S	 130.10
U+442A.0	fTotalStrokes	 14
U+442A.0	fCangjie	BOGS
U+442A.0	fCantonese	zeon3
U+442A.0	fMandarin	fan4; juan3
U+442A.0	fEnglish	same as &#x81C7;
U+442B.0	fUTF8	䐫
U+442B.0	fCNS11643	4-4C5D
U+442B.0	fGB	3-4C4D
U+442B.0	fR/S	 130.11
U+442B.0	fTotalStrokes	 15
U+442B.0	fCangjie	BHOO
U+442B.0	fFourCorner	78280
U+442B.0	fCantonese	cung1; sung1
U+442B.0	fMandarin	cong1
U+442B.0	fEnglish	obesity; fat; plump
U+442C.0	fUTF8	䐬
U+442C.0	fCNS11643	4-4C58
U+442C.0	fGB	3-4D65
U+442C.0	fR/S	 130.11
U+442C.0	fTotalStrokes	 15
U+442C.0	fCangjie	BTWA
U+442C.0	fFourCorner	75260
U+442C.0	fHakka	 cau2; co2
U+442C.0	fCantonese	cou4
U+442C.0	fMandarin	cao2
U+442C.0	fEnglish	crisp; crumbllng
U+442D.0	fUTF8	䐭
U+442D.0	fCNS11643	4-4C5E
U+442D.0	fGB	5-4F4C
U+442D.0	fR/S	 130.11
U+442D.0	fTotalStrokes	 15
U+442D.0	fCangjie	BKPB
U+442D.0	fFourCorner	74220
U+442D.0	fCantonese	zai6
U+442D.0	fMandarin	di4; zhi4
U+442D.0	fEnglish	[1] a discharge or flux [2] diarrhea and dysentery
U+442E.0	fUTF8	䐮
U+442E.0	fCNS11643	4-4C55
U+442E.0	fGB	3-4C78
U+442E.0	fR/S	 130.11
U+442E.0	fTotalStrokes	 15
U+442E.0	fCangjie	BYHM
U+442E.0	fFourCorner	70210
U+442E.0	fCantonese	caan2; can2; san3; zin1
U+442E.0	fMandarin	chan3
U+442E.0	fEnglish	skin
U+442F.0	fUTF8	䐯
U+442F.0	fCNS11643	4-4C5C
U+442F.0	fGB	3-4D66
U+442F.0	fR/S	 130.11
U+442F.0	fTotalStrokes	 15
U+442F.0	fCangjie	BWVF
U+442F.0	fFourCorner	76290
U+442F.0	fHakka	 lui3; lo2; lui2
U+442F.0	fCantonese	leoi4; leoi5
U+442F.0	fMandarin	lei2
U+442F.0	fEnglish	fat; stout; gross
U+4430.0	fUTF8	䐰
U+4430.0	fCNS11643	6-606F
U+4430.0	fR/S	 130.11; 130.10
U+4430.0	fTotalStrokes	 14; 15
U+4430.0	fCangjie	BOLB
U+4430.0	fFourCorner	77220
U+4430.0	fHakka	 siu1
U+4430.0	fCantonese	sau1; gaau1; saau1
U+4430.0	fMandarin	xiu1
U+4430.0	fEnglish	same as &#x7F9E;
U+4431.0	fUTF8	䐱
U+4431.0	fCNS11643	4-4C54
U+4431.0	fGB	3-4D69
U+4431.0	fR/S	 130.11
U+4431.0	fTotalStrokes	 15
U+4431.0	fCangjie	BYCB
U+4431.0	fFourCorner	70220
U+4431.0	fCantonese	zaai3; zak6
U+4431.0	fMandarin	zha2
U+4431.0	fEnglish	to pick the meat in between of bones
U+4432.0	fUTF8	䐲
U+4432.0	fCNS11643	4-4C5B
U+4432.0	fGB	3-4D6C
U+4432.0	fR/S	 130.11
U+4432.0	fTotalStrokes	 15
U+4432.0	fCangjie	BSMA
U+4432.0	fFourCorner	77260
U+4432.0	fCantonese	zaap6
U+4432.0	fMandarin	zhi2; zhe2
U+4432.0	fEnglish	half-raw meat; not well cooked meat; the light or glow of fire; raw meat; offensve smell, especially of fish or blood; thin sliced meat
U+4433.0	fUTF8	䐳
U+4433.0	fCNS11643	4-4C5F
U+4433.0	fGB	5-4F44
U+4433.0	fR/S	 130.11
U+4433.0	fTotalStrokes	 15
U+4433.0	fCangjie	BNWF
U+4433.0	fFourCorner	77230
U+4433.0	fCantonese	jyu4; zyu4
U+4433.0	fMandarin	yu2
U+4433.0	fEnglish	same as &#x9B5A;
U+4434.0	fUTF8	䐴
U+4434.0	fCNS11643	5-4228
U+4434.0	fGB	5-4F79
U+4434.0	fR/S	 130.11; 130.9
U+4434.0	fTotalStrokes	 13; 15
U+4434.0	fCangjie	AUB
U+4434.0	fFourCorner	71220
U+4434.0	fCantonese	gui3; gwui3 (also)
U+4434.0	fMandarin	gui4
U+4434.0	fEnglish	lumbago
U+4435.0	fUTF8	䐵
U+4435.0	fCNS11643	4-5263
U+4435.0	fGB	3-4D63
U+4435.0	fR/S	 130.12
U+4435.0	fTotalStrokes	 16
U+4435.0	fCangjie	BTMC
U+4435.0	fFourCorner	74280
U+4435.0	fCantonese	gwang1; wong2
U+4435.0	fMandarin	gong1; huang2
U+4435.0	fEnglish	[1] fat; obese [2] swelling
U+4436.0	fUTF8	䐶
U+4436.0	fCNS11643	4-5264
U+4436.0	fGB	3-4D71
U+4436.0	fR/S	 130.12
U+4436.0	fTotalStrokes	 16
U+4436.0	fCangjie	BMUA
U+4436.0	fFourCorner	71260
U+4436.0	fCantonese	zaam1
U+4436.0	fMandarin	zan1; jin3
U+4436.0	fEnglish	[1] to cook; to boil; to decoct, diseases of the lips; hare-lip, pieces of meat [2] to cook; to boil; to decoct, diseases of the lips; hare-lip, pieces of meat
U+4437.0	fUTF8	䐷
U+4437.0	fCNS11643	3-5246
U+4437.0	fGB	5-4F47
U+4437.0	fR/S	 130.12
U+4437.0	fTotalStrokes	 16
U+4437.0	fCangjie	BRRJ
U+4437.0	fFourCorner	76250
U+4437.0	fCantonese	daan1
U+4437.0	fMandarin	dan1
U+4437.0	fEnglish	paunchy; a large belly
U+4438.0	fUTF8	䐸
U+4438.0	fCNS11643	3-5245
U+4438.0	fR/S	 130.12
U+4438.0	fTotalStrokes	 16
U+4438.0	fCangjie	BLGM
U+4438.0	fFourCorner	75210
U+4438.0	fCantonese	waak6; gwok3
U+4438.0	fMandarin	huo4; guo2
U+4438.0	fEnglish	non-classical form of &#x8195;
U+4439.0	fUTF8	䐹
U+4439.0	fCNS11643	4-525E
U+4439.0	fGB	3-4D3F
U+4439.0	fR/S	 130.12
U+4439.0	fTotalStrokes	 16
U+4439.0	fCangjie	BLX
U+4439.0	fFourCorner	75220
U+4439.0	fHakka	 siau1; seu1; seu5
U+4439.0	fCantonese	sau1
U+4439.0	fMandarin	sou1; sao4
U+4439.0	fEnglish	perserved and dried meat, (a dialect) fried; cooked small pieces of meat, meat broth
U+443A.0	fUTF8	䐺
U+443A.0	fCNS11643	4-5261
U+443A.0	fGB	3-4D6F
U+443A.0	fR/S	 130.12
U+443A.0	fTotalStrokes	 16
U+443A.0	fCangjie	BMWJ
U+443A.0	fFourCorner	71240
U+443A.0	fCantonese	taam3; tam4
U+443A.0	fMandarin	tan4
U+443A.0	fEnglish	[1] delicious; tasty food [2] same as &#x91B0;
U+443B.0	fUTF8	䐻
U+443B.0	fCNS11643	4-5262
U+443B.0	fGB	3-4D6E
U+443B.0	fR/S	 130.12
U+443B.0	fTotalStrokes	 16
U+443B.0	fCangjie	BJRJ
U+443B.0	fFourCorner	74240
U+443B.0	fCantonese	gu1; gwu1 (also)
U+443B.0	fMandarin	gu1
U+443B.0	fEnglish	big pieces of dried meat
U+443C.0	fUTF8	䐼
U+443C.0	fCNS11643	4-5266
U+443C.0	fGB	3-4D6D
U+443C.0	fR/S	 130.12
U+443C.0	fTotalStrokes	 16
U+443C.0	fCangjie	BQHF
U+443C.0	fFourCorner	77290
U+443C.0	fCantonese	hai6
U+443C.0	fMandarin	xie4; xi4
U+443C.0	fEnglish	thin membrane of the throat; bullet
U+443D.0	fUTF8	䐽
U+443D.0	fCNS11643	F-5155
U+443D.0	fR/S	 130.12
U+443D.0	fTotalStrokes	 16
U+443D.0	fCangjie	BTLB
U+443D.0	fCantonese	mun4
U+443D.0	fMandarin	man2
U+443E.0	fUTF8	䐾
U+443E.0	fCNS11643	5-5D5B
U+443E.0	fGB	3-4C68
U+443E.0	fR/S	 130.13
U+443E.0	fTotalStrokes	 17
U+443E.0	fCangjie	BWLJ
U+443E.0	fFourCorner	76240
U+443E.0	fCantonese	dok6
U+443E.0	fMandarin	duo2
U+443E.0	fEnglish	fat; plump
U+443F.0	fUTF8	䐿
U+443F.0	fCNS11643	4-582B
U+443F.0	fGB	3-4D74
U+443F.0	fR/S	 130.13
U+443F.0	fTotalStrokes	 16; 17
U+443F.0	fCangjie	BHBK
U+443F.0	fFourCorner	77230
U+443F.0	fHakka	 juk7; ruk7
U+443F.0	fCantonese	ou3; ngou3 (also)
U+443F.0	fMandarin	yu4; ao2; ao4; bi4
U+443F.0	fEnglish	the crop of a bird
U+4440.0	fUTF8	䑀
U+4440.0	fCNS11643	4-5828
U+4440.0	fGB	3-4D7E
U+4440.0	fR/S	 130.13
U+4440.0	fTotalStrokes	 17; 16
U+4440.0	fCangjie	BSRJ
U+4440.0	fFourCorner	70240
U+4440.0	fCantonese	pik1
U+4440.0	fMandarin	pi4
U+4440.0	fEnglish	[1] the navel [2] the bowels; the entrails; the viscera [3] same as &#x7656;
U+4441.0	fUTF8	䑁
U+4441.0	fCNS11643	4-582C
U+4441.0	fGB	3-4E2C
U+4441.0	fR/S	 130.13
U+4441.0	fTotalStrokes	 17; 16
U+4441.0	fCangjie	HBB
U+4441.0	fFourCorner	77220
U+4441.0	fCantonese	uk1; nguk1 (also)
U+4441.0	fMandarin	wu4
U+4441.0	fEnglish	fatty; greasy; ointment
U+4442.0	fUTF8	䑂
U+4442.0	fCNS11643	4-5C74
U+4442.0	fR/S	 130.14
U+4442.0	fTotalStrokes	 18
U+4442.0	fCangjie	BHXC
U+4442.0	fFourCorner	77280
U+4442.0	fCantonese	jaai5
U+4442.0	fMandarin	ai3; yu3; xi4
U+4442.0	fEnglish	fat; plump
U+4443.0	fUTF8	䑃
U+4443.0	fCNS11643	3-5866
U+4443.0	fR/S	 130.14
U+4443.0	fTotalStrokes	 18
U+4443.0	fCangjie	BTBO
U+4443.0	fFourCorner	74230
U+4443.0	fHakka	 mung2
U+4443.0	fCantonese	mung4
U+4443.0	fMandarin	meng2; meng3
U+4443.0	fEnglish	dim; indistinct
U+4444.0	fUTF8	䑄
U+4444.0	fCNS11643	4-5C76
U+4444.0	fGB	3-4E23
U+4444.0	fR/S	 130.14
U+4444.0	fTotalStrokes	 18
U+4444.0	fCangjie	BHUL
U+4444.0	fFourCorner	76220
U+4444.0	fCantonese	bei6
U+4444.0	fMandarin	pi4; yi4; bei4; bi4
U+4444.0	fEnglish	[1] rich; abundant; big and strong; vigorous; husky [2] same as &#x761C;
U+4445.0	fUTF8	䑅
U+4445.0	fCNS11643	3-5867
U+4445.0	fR/S	 130.14
U+4445.0	fTotalStrokes	 18
U+4445.0	fCangjie	BTWN
U+4445.0	fFourCorner	74220
U+4445.0	fCantonese	mang4; mung4
U+4445.0	fMandarin	meng2
U+4445.0	fEnglish	(non-classical form) dizzy and sleepy; slumberous
U+4446.0	fUTF8	䑆
U+4446.0	fCNS11643	3-5A76
U+4446.0	fR/S	 130.15
U+4446.0	fTotalStrokes	 19
U+4446.0	fCangjie	BTOV
U+4446.0	fFourCorner	78230
U+4446.0	fCantonese	joeng5; zoeng5
U+4446.0	fMandarin	yang3
U+4446.0	fEnglish	feel like vomiting
U+4447.0	fUTF8	䑇
U+4447.0	fCNS11643	4-6074
U+4447.0	fGB	3-4D37
U+4447.0	fR/S	 130.15
U+4447.0	fTotalStrokes	 19
U+4447.0	fCangjie	BHLC
U+4447.0	fFourCorner	72280
U+4447.0	fCantonese	zat1
U+4447.0	fMandarin	zhi2; zhi4
U+4447.0	fEnglish	medicine for a sore; a boil (injure by a sword or a knief)
U+4448.0	fUTF8	䑈
U+4448.0	fCNS11643	4-6073
U+4448.0	fGB	3-4C61
U+4448.0	fR/S	 130.15
U+4448.0	fTotalStrokes	 19
U+4448.0	fCangjie	BVID
U+4448.0	fFourCorner	72290
U+4448.0	fCantonese	bok1
U+4448.0	fMandarin	bo1; bo2
U+4448.0	fEnglish	meat to offer for worship
U+4449.0	fUTF8	䑉
U+4449.0	fCNS11643	5-6D2C
U+4449.0	fR/S	 130.16
U+4449.0	fTotalStrokes	 20
U+4449.0	fCangjie	BYNV
U+4449.0	fFourCorner	70210
U+4449.0	fCantonese	jing4; zing4
U+4449.0	fMandarin	ying2
U+4449.0	fEnglish	excrement; night soil
U+444A.0	fUTF8	䑊
U+444A.0	fCNS11643	4-642A
U+444A.0	fGB	3-4D72
U+444A.0	fR/S	 130.16
U+444A.0	fTotalStrokes	 20
U+444A.0	fCangjie	BYLC
U+444A.0	fFourCorner	75230
U+444A.0	fCantonese	wai4
U+444A.0	fMandarin	wei2; wei4
U+444A.0	fEnglish	[1] fat; plump [2] physical disease [3] carnal
U+444B.0	fUTF8	䑋
U+444B.0	fCNS11643	4-6722
U+444B.0	fGB	3-4E2A
U+444B.0	fR/S	 130.17
U+444B.0	fTotalStrokes	 21
U+444B.0	fCangjie	BYRV
U+444B.0	fFourCorner	70230
U+444B.0	fHakka	 jong5; rong5
U+444B.0	fCantonese	joeng5; zoeng5
U+444B.0	fMandarin	rang3; nao2; rang2
U+444B.0	fEnglish	the fat of hogs; pigs lard
U+444C.0	fUTF8	䑌
U+444C.0	fCNS11643	4-6724
U+444C.0	fGB	3-4D76
U+444C.0	fR/S	 130.17
U+444C.0	fTotalStrokes	 21
U+444C.0	fCangjie	BANW
U+444C.0	fFourCorner	77220
U+444C.0	fHakka	 lan5
U+444C.0	fCantonese	laan4
U+444C.0	fMandarin	lan2; lan4
U+444C.0	fEnglish	[1] what is left over of the food [2] same as &#x721B;
U+444D.0	fUTF8	䑍
U+444D.0	fCNS11643	4-6723
U+444D.0	fGB	3-4D68
U+444D.0	fR/S	 130.17
U+444D.0	fTotalStrokes	 21
U+444D.0	fCangjie	BBCV
U+444D.0	fFourCorner	76240
U+444D.0	fCantonese	jin1; zin1; zin3; zing2
U+444D.0	fMandarin	ying3; yan4
U+444D.0	fEnglish	[1] same as &#x54BD; [2] same as &#x766D;
U+444E.0	fUTF8	䑎
U+444E.0	fCNS11643	4-6725
U+444E.0	fGB	3-4E29
U+444E.0	fR/S	 130.17
U+444E.0	fTotalStrokes	 21
U+444E.0	fCangjie	BOIM
U+444E.0	fFourCorner	73250
U+444E.0	fCantonese	caam2
U+444E.0	fMandarin	qian1; chan3; qian4; jian3
U+444E.0	fEnglish	soup of pig's intestines; meat soup; meat broth
U+444F.0	fUTF8	䑏
U+444F.0	fCNS11643	4-6930
U+444F.0	fGB	3-4E28
U+444F.0	fR/S	 130.18
U+444F.0	fTotalStrokes	 22; 21
U+444F.0	fCangjie	BTRG
U+444F.0	fFourCorner	74210
U+444F.0	fCantonese	kyun4
U+444F.0	fMandarin	quan2; huan1
U+444F.0	fEnglish	[1] ugly [2] a legendary animal
U+4450.0	fUTF8	䑐
U+4450.0	fCNS11643	6-4364
U+4450.0	fR/S	 131.5
U+4450.0	fTotalStrokes	 11
U+4450.0	fCangjie	SLNF
U+4450.0	fFourCorner	78790
U+4450.0	fCantonese	can2; maai4
U+4450.0	fMandarin	zhen3
U+4450.0	fEnglish	[1] bright; light [2] clear [3] same as &#x7787;
U+4451.0	fUTF8	䑑
U+4451.0	fCNS11643	7-4224
U+4451.0	fR/S	 131.12
U+4451.0	fTotalStrokes	 18
U+4451.0	fCangjie	SLTCO
U+4451.0	fFourCorner	72730
U+4451.0	fCantonese	buk6
U+4451.0	fMandarin	pu2
U+4451.0	fEnglish	ancient form of &#x50D5;
U+4452.0	fUTF8	䑒
U+4452.0	fCNS11643	3-2B44
U+4452.0	fR/S	 133.3
U+4452.0	fTotalStrokes	 9
U+4452.0	fCangjie	MGSK
U+4452.0	fFourCorner	17140
U+4452.0	fCantonese	dou3
U+4452.0	fMandarin	dao4
U+4452.0	fEnglish	a kind of farm tool used in ancient times
U+4453.0	fUTF8	䑓
U+4453.0	fCNS11643	3-4424
U+4453.0	fR/S	 133.7
U+4453.0	fTotalStrokes	 13
U+4453.0	fCangjie	TBMIG
U+4453.0	fFourCorner	44100
U+4453.0	fHakka	 toi2
U+4453.0	fCantonese	toi4
U+4453.0	fMandarin	tai2; tai3
U+4453.0	fEnglish	same as &#x81FA;
U+4454.0	fUTF8	䑔
U+4454.0	fCNS11643	4-2F4A
U+4454.0	fGB	3-6543
U+4454.0	fR/S	 134.4
U+4454.0	fTotalStrokes	 10
U+4454.0	fCangjie	HXJB
U+4454.0	fFourCorner	75720
U+4454.0	fHakka	  fat8
U+4454.0	fCantonese	fai3
U+4454.0	fMandarin	fa2; fei4
U+4454.0	fEnglish	to pound rice
U+4455.0	fUTF8	䑕
U+4455.0	fCNS11643	4-3458
U+4455.0	fR/S	 134.5
U+4455.0	fTotalStrokes	 11
U+4455.0	fCangjie	HXHNQ
U+4455.0	fFourCorner	77210
U+4455.0	fCantonese	syu2
U+4455.0	fMandarin	shu3
U+4455.0	fEnglish	non-classical abbreviated form of &#x9F20;
U+4456.0	fUTF8	䑖
U+4456.0	fGB	3-6547
U+4456.0	fR/S	 134.8
U+4456.0	fTotalStrokes	 14
U+4456.0	fCangjie	HXJLO
U+4456.0	fCantonese	zit3
U+4457.0	fUTF8	䑗
U+4457.0	fCNS11643	4-4C63
U+4457.0	fGB	5-645A
U+4457.0	fR/S	 134.9
U+4457.0	fTotalStrokes	 15
U+4457.0	fCangjie	HXAMH
U+4457.0	fFourCorner	76720
U+4457.0	fCantonese	tong4
U+4457.0	fMandarin	dang4
U+4457.0	fEnglish	pound (grain) in order to remove the husk
U+4458.0	fUTF8	䑘
U+4458.0	fCNS11643	4-526B
U+4458.0	fGB	3-6546
U+4458.0	fR/S	 134.10
U+4458.0	fTotalStrokes	 16
U+4458.0	fCangjie	HXTQM
U+4458.0	fFourCorner	78710
U+4458.0	fCantonese	co4
U+4458.0	fMandarin	cuo2; cha2; zhi4
U+4458.0	fEnglish	[1] pound; ram down [2] polish [3] grind barley or wheat
U+4459.0	fUTF8	䑙
U+4459.0	fCNS11643	3-3353
U+4459.0	fR/S	 135.4
U+4459.0	fTotalStrokes	 10
U+4459.0	fCangjie	HRBMM
U+4459.0	fFourCorner	27650
U+4459.0	fHakka	 dam1; tam1
U+4459.0	fCantonese	daam1
U+4459.0	fMandarin	tan1; ran2; tan4; tian4; tian3
U+4459.0	fEnglish	 loll out the tongue
U+445A.0	fUTF8	䑚
U+445A.0	fCNS11643	3-3354
U+445A.0	fR/S	 135.4
U+445A.0	fTotalStrokes	 10
U+445A.0	fCangjie	HRB
U+445A.0	fFourCorner	27620
U+445A.0	fCantonese	tim4; min5
U+445A.0	fMandarin	tian2
U+445A.0	fEnglish	non-classical form of &#x751C;
U+445B.0	fUTF8	䑛
U+445B.0	fCNS11643	3-384A
U+445B.0	fR/S	 135.5
U+445B.0	fTotalStrokes	 11
U+445B.0	fCangjie	HRHVI
U+445B.0	fFourCorner	22640
U+445B.0	fCantonese	saai5
U+445B.0	fMandarin	shi4; chi3; yi4
U+445B.0	fEnglish	standard form of &#x8210;
U+445C.0	fUTF8	䑜
U+445C.0	fCNS11643	4-4C64
U+445C.0	fGB	3-6345
U+445C.0	fR/S	 135.9
U+445C.0	fTotalStrokes	 15
U+445C.0	fCangjie	HRPTD
U+445C.0	fFourCorner	24690
U+445C.0	fHakka	 liap8; tiap8; tap7
U+445C.0	fCantonese	taap3
U+445C.0	fMandarin	ta4; tie1
U+445C.0	fEnglish	to eat heavily, cooked rice, a meal, to drink, to swallow, to lick, the uvula, the ligule
U+445D.0	fUTF8	䑝
U+445D.0	fCNS11643	6-6121
U+445D.0	fGB	S-234C
U+445D.0	fR/S	 136.8
U+445D.0	fTotalStrokes	 14
U+445D.0	fCangjie	RRBNQ
U+445D.0	fFourCorner	66250
U+445D.0	fCantonese	gaa2
U+445D.0	fMandarin	jia3
U+445D.0	fEnglish	non-classical form of &#x659D;
U+445E.0	fUTF8	䑞
U+445E.0	fCNS11643	3-524B
U+445E.0	fGB	5-3127
U+445E.0	fR/S	 136.10
U+445E.0	fTotalStrokes	 16
U+445E.0	fCangjie	SFNIQ
U+445E.0	fFourCorner	71250
U+445E.0	fCantonese	seon3
U+445E.0	fMandarin	shun4
U+445E.0	fEnglish	same as &#x821C;
U+445F.0	fUTF8	䑟
U+445F.0	fCNS11643	3-5E54
U+445F.0	fR/S	 136.15
U+445F.0	fTotalStrokes	 21
U+445F.0	fCangjie	BQHAG
U+445F.0	fFourCorner	26210
U+445F.0	fCantonese	wong4
U+445F.0	fMandarin	huang2
U+445F.0	fEnglish	same as &#x845F;
U+4460.0	fUTF8	䑠
U+4460.0	fCNS11643	4-276F
U+4460.0	fGB	3-6551
U+4460.0	fR/S	 137.2
U+4460.0	fTotalStrokes	 8
U+4460.0	fCangjie	HYNN
U+4460.0	fFourCorner	27420
U+4460.0	fCantonese	liu5
U+4460.0	fMandarin	liao3
U+4460.0	fEnglish	[1] long and narrow boat [2] small boat
U+4461.0	fUTF8	䑡
U+4461.0	fCNS11643	4-2B33
U+4461.0	fGB	3-6554
U+4461.0	fR/S	 137.3
U+4461.0	fTotalStrokes	 9
U+4461.0	fCangjie	HYEI
U+4461.0	fFourCorner	27440
U+4461.0	fCantonese	caa1; dok6
U+4461.0	fMandarin	cha1
U+4461.0	fEnglish	same as &#x8256;
U+4462.0	fUTF8	䑢
U+4462.0	fCNS11643	4-2B32
U+4462.0	fGB	3-6552
U+4462.0	fR/S	 137.3
U+4462.0	fTotalStrokes	 9
U+4462.0	fCangjie	HYMU
U+4462.0	fFourCorner	21410
U+4462.0	fHakka	 ngut7
U+4462.0	fCantonese	ngat6; at6 (also); dou1; zaa3
U+4462.0	fMandarin	wu4
U+4462.0	fEnglish	non-classical form) same as &#x8220;
U+4463.0	fUTF8	䑣
U+4463.0	fCNS11643	3-2E67
U+4463.0	fGB	5-6472
U+4463.0	fR/S	 137.3
U+4463.0	fTotalStrokes	 9
U+4463.0	fCangjie	HYHHH
U+4463.0	fFourCorner	22420
U+4463.0	fCantonese	cam1
U+4463.0	fMandarin	chen1; chen4
U+4463.0	fEnglish	sailing boat
U+4464.0	fUTF8	䑤
U+4464.0	fCNS11643	3-3356
U+4464.0	fR/S	 137.4
U+4464.0	fTotalStrokes	 10
U+4464.0	fCangjie	HYOIN
U+4464.0	fFourCorner	28420
U+4464.0	fCantonese	gam6
U+4464.0	fMandarin	jin4; jing4
U+4464.0	fEnglish	boat; ship; vessel
U+4465.0	fUTF8	䑥
U+4465.0	fCNS11643	4-2F50
U+4465.0	fGB	3-6553
U+4465.0	fR/S	 137.4
U+4465.0	fTotalStrokes	 10
U+4465.0	fCangjie	HYNHE
U+4465.0	fFourCorner	27440
U+4465.0	fCantonese	ngap6; ap6 (also)
U+4465.0	fMandarin	e1; e4; sa4
U+4465.0	fEnglish	a moving boat; a sailing boat, tools used on a boat, a speedy boat
U+4466.0	fUTF8	䑦
U+4466.0	fCNS11643	3-384B
U+4466.0	fGB	5-6476
U+4466.0	fR/S	 137.5
U+4466.0	fTotalStrokes	 11
U+4466.0	fCangjie	HYPR
U+4466.0	fFourCorner	27420
U+4466.0	fCantonese	kau1; keoi4; ngaau1
U+4466.0	fMandarin	gou1
U+4466.0	fEnglish	 boat; ship; vessel
U+4467.0	fUTF8	䑧
U+4467.0	fCNS11643	6-4371
U+4467.0	fR/S	 137.5
U+4467.0	fTotalStrokes	 11
U+4467.0	fCangjie	HYODI
U+4467.0	fFourCorner	24440
U+4467.0	fCantonese	fu1
U+4467.0	fMandarin	bu4; fu2
U+4467.0	fEnglish	a short, deep, and small boat
U+4468.0	fUTF8	䑨
U+4468.0	fCNS11643	6-4370
U+4468.0	fR/S	 137.5
U+4468.0	fTotalStrokes	 11
U+4468.0	fCangjie	HYOPD
U+4468.0	fFourCorner	28410
U+4468.0	fHakka	 to5
U+4468.0	fCantonese	to4
U+4468.0	fMandarin	duo4
U+4468.0	fEnglish	same as &#x8235;
U+4469.0	fUTF8	䑩
U+4469.0	fGB	S-2139
U+4469.0	fR/S	 137.5
U+4469.0	fTotalStrokes	 11
U+4469.0	fCangjie	HYJR
U+4469.0	fCantonese	gu2; gwu2 (also)
U+4469.0	fEnglish	a kind of inland rivers warship used in Qing Dynasty
U+446A.0	fUTF8	䑪
U+446A.0	fCNS11643	3-3E4A
U+446A.0	fR/S	 137.6
U+446A.0	fTotalStrokes	 12
U+446A.0	fCangjie	HYOMR
U+446A.0	fFourCorner	28460
U+446A.0	fCantonese	ngap6; ap6 (also)
U+446A.0	fMandarin	ge1; e4
U+446A.0	fEnglish	a sailing; moving boat
U+446B.0	fUTF8	䑫
U+446B.0	fCNS11643	4-3A4E
U+446B.0	fGB	3-6561
U+446B.0	fR/S	 137.6
U+446B.0	fTotalStrokes	 12
U+446B.0	fCangjie	HYTT
U+446B.0	fFourCorner	28440
U+446B.0	fCantonese	bang1
U+446B.0	fMandarin	beng1
U+446B.0	fEnglish	tools for boat, name of a boat
U+446C.0	fUTF8	䑬
U+446C.0	fCNS11643	4-3A4F
U+446C.0	fGB	3-655E
U+446C.0	fR/S	 137.6
U+446C.0	fTotalStrokes	 12
U+446C.0	fCangjie	HYLMO
U+446C.0	fFourCorner	22410
U+446C.0	fCantonese	tou1; tiu1
U+446C.0	fMandarin	tao1; yao4
U+446C.0	fEnglish	[1] boat [2] large boat [3] dispel; remove; move [4] stir up
U+446D.0	fUTF8	䑭
U+446D.0	fCNS11643	6-4D63
U+446D.0	fR/S	 137.6
U+446D.0	fTotalStrokes	 12
U+446D.0	fCangjie	HYEM
U+446D.0	fFourCorner	21410
U+446D.0	fCantonese	dai3
U+446D.0	fMandarin	di3; di4; zhi4
U+446D.0	fEnglish	war vessel; man-of-war; warship
U+446E.0	fUTF8	䑮
U+446E.0	fGB	S-213A
U+446E.0	fR/S	 137.6
U+446E.0	fTotalStrokes	 12
U+446E.0	fCangjie	HYNSD
U+446E.0	fCantonese	do2
U+446F.0	fUTF8	䑯
U+446F.0	fCNS11643	5-422D
U+446F.0	fR/S	 137.7
U+446F.0	fTotalStrokes	 13
U+446F.0	fCangjie	HYCNH
U+446F.0	fFourCorner	28420
U+446F.0	fCantonese	tai5; daai6
U+446F.0	fMandarin	di4; ti4
U+446F.0	fEnglish	 boat; ship; vessel
U+4470.0	fUTF8	䑰
U+4470.0	fCNS11643	4-402F
U+4470.0	fGB	3-6563
U+4470.0	fR/S	 137.7
U+4470.0	fTotalStrokes	 13
U+4470.0	fCangjie	HYYLH
U+4470.0	fFourCorner	21420
U+4470.0	fCantonese	bou6
U+4470.0	fMandarin	bu4
U+4470.0	fEnglish	small boat
U+4471.0	fUTF8	䑱
U+4471.0	fCNS11643	4-464F
U+4471.0	fGB	3-6565
U+4471.0	fR/S	 137.8
U+4471.0	fTotalStrokes	 14
U+4471.0	fCangjie	HYJNU
U+4471.0	fFourCorner	23410
U+4471.0	fCantonese	jyun2; wun2
U+4471.0	fMandarin	wan3; man2
U+4471.0	fEnglish	 boat; ship; vessel
U+4472.0	fUTF8	䑲
U+4472.0	fCNS11643	4-4651
U+4472.0	fGB	5-6524
U+4472.0	fR/S	 137.8
U+4472.0	fTotalStrokes	 14
U+4472.0	fCangjie	HYYAJ
U+4472.0	fFourCorner	21440
U+4472.0	fCantonese	zaau6; zau3
U+4472.0	fMandarin	zhao4; che4; zhai2
U+4472.0	fEnglish	same as &#x68F9;, &#x6AC2;
U+4473.0	fUTF8	䑳
U+4473.0	fCNS11643	4-4650
U+4473.0	fGB	3-6556
U+4473.0	fR/S	 137.8
U+4473.0	fTotalStrokes	 14
U+4473.0	fCangjie	HYOMB
U+4473.0	fFourCorner	28420
U+4473.0	fHakka	  lun2
U+4473.0	fCantonese	leon4
U+4473.0	fMandarin	lun2; juan1
U+4473.0	fEnglish	the prow of a junk
U+4474.0	fUTF8	䑴
U+4474.0	fCNS11643	3-492C
U+4474.0	fR/S	 137.8
U+4474.0	fTotalStrokes	 14
U+4474.0	fCangjie	HYTMC
U+4474.0	fFourCorner	24480
U+4474.0	fCantonese	kei4; zin1
U+4474.0	fMandarin	qi2
U+4474.0	fEnglish	 boat; ship; vessel
U+4475.0	fUTF8	䑵
U+4475.0	fCNS11643	6-6124
U+4475.0	fGB	S-213F
U+4475.0	fR/S	 137.8
U+4475.0	fTotalStrokes	 14
U+4475.0	fCangjie	HYAB
U+4475.0	fFourCorner	27420
U+4475.0	fCantonese	muk6
U+4475.0	fMandarin	mu4
U+4475.0	fEnglish	corrupted form of &#x8252;
U+4476.0	fUTF8	䑶
U+4476.0	fCNS11643	5-492C
U+4476.0	fR/S	 137.8
U+4476.0	fTotalStrokes	 14
U+4476.0	fCangjie	HYQMB
U+4476.0	fFourCorner	25420
U+4476.0	fCantonese	sin3; sin6
U+4476.0	fMandarin	qian4; can2
U+4476.0	fEnglish	sprightly boat; light boat
U+4477.0	fUTF8	䑷
U+4477.0	fGB	S-213C
U+4477.0	fR/S	 137.8
U+4477.0	fTotalStrokes	 14
U+4477.0	fCangjie	HYISB
U+4477.0	fCantonese	gin1
U+4478.0	fUTF8	䑸
U+4478.0	fR/S	 137.8
U+4478.0	fTotalStrokes	 14
U+4478.0	fCangjie	HYJMF
U+4478.0	fCantonese	zung1
U+4478.0	fEnglish	a group of boats, boat; warship
U+4479.0	fUTF8	䑹
U+4479.0	fCNS11643	3-4E3E
U+4479.0	fR/S	 137.9
U+4479.0	fTotalStrokes	 15
U+4479.0	fCangjie	HYJFE
U+4479.0	fFourCorner	23440
U+4479.0	fCantonese	sau1
U+4479.0	fMandarin	sao1
U+4479.0	fEnglish	same as &#x8258;
U+447A.0	fUTF8	䑺
U+447A.0	fCNS11643	4-4C68
U+447A.0	fR/S	 137.9
U+447A.0	fTotalStrokes	 15
U+447A.0	fCangjie	HYHNI
U+447A.0	fFourCorner	27410
U+447A.0	fCantonese	faan4; fan4; po4; pun4
U+447A.0	fMandarin	fan2
U+447A.0	fEnglish	same as &#x5E06;
U+447B.0	fUTF8	䑻
U+447B.0	fCNS11643	4-4C65
U+447B.0	fGB	3-656A
U+447B.0	fR/S	 137.9
U+447B.0	fTotalStrokes	 15
U+447B.0	fCangjie	HYYSD
U+447B.0	fFourCorner	28440
U+447B.0	fCantonese	jau4; zaau4
U+447B.0	fMandarin	you2
U+447B.0	fEnglish	[1] covering; curtain, or screen of a boat [2] sail a boat
U+447C.0	fUTF8	䑼
U+447C.0	fCNS11643	4-526C
U+447C.0	fGB	3-655C
U+447C.0	fR/S	 137.10
U+447C.0	fTotalStrokes	 16
U+447C.0	fCangjie	HYPUU
U+447C.0	fFourCorner	27420
U+447C.0	fCantonese	zau1; saau2; zaau1
U+447C.0	fMandarin	zou1; zhu1
U+447C.0	fEnglish	[1] ocean-going liner; steamer [2] boat; ship; vessel
U+447D.0	fUTF8	䑽
U+447D.0	fCNS11643	4-526D
U+447D.0	fGB	3-656E
U+447D.0	fR/S	 137.10
U+447D.0	fTotalStrokes	 16
U+447D.0	fCangjie	HYASM
U+447D.0	fFourCorner	26420
U+447D.0	fCantonese	taap3; dap6
U+447D.0	fMandarin	ta4; da2
U+447D.0	fEnglish	a large boat
U+447E.0	fUTF8	䑾
U+447E.0	fGB	S-213B
U+447E.0	fR/S	 137.10
U+447E.0	fTotalStrokes	 16
U+447E.0	fCangjie	HYOGE
U+447E.0	fHakka	 sung1
U+447E.0	fCantonese	zek3
U+447F.0	fUTF8	䑿
U+447F.0	fCNS11643	4-5830
U+447F.0	fGB	3-6572
U+447F.0	fR/S	 137.11
U+447F.0	fTotalStrokes	 17; 16
U+447F.0	fCangjie	HYJOA
U+447F.0	fFourCorner	23460
U+447F.0	fCantonese	suk1
U+447F.0	fMandarin	su4
U+447F.0	fEnglish	small boat
U+4480.0	fUTF8	䒀
U+4480.0	fCNS11643	4-5834
U+4480.0	fGB	3-6570
U+4480.0	fR/S	 137.11
U+4480.0	fTotalStrokes	 17; 16
U+4480.0	fCangjie	HYHOI
U+4480.0	fFourCorner	28450
U+4480.0	fCantonese	bou6 (also); fu1
U+4480.0	fMandarin	bu4; fu2; fu1; pu2
U+4480.0	fEnglish	short and deep boat
U+4481.0	fUTF8	䒁
U+4481.0	fCNS11643	4-5833
U+4481.0	fGB	3-6573
U+4481.0	fR/S	 137.11
U+4481.0	fTotalStrokes	 17; 16
U+4481.0	fCangjie	HYSMA
U+4481.0	fFourCorner	27460
U+4481.0	fCantonese	zaap6
U+4481.0	fMandarin	xi2; ji4; xiao4; ya4
U+4481.0	fEnglish	a small boat connected to the back of a big boat; used to load the passengers, cargo or goods, warship, a kind of tools used to capsize
U+4482.0	fUTF8	䒂
U+4482.0	fCNS11643	3-5625
U+4482.0	fR/S	 137.11
U+4482.0	fTotalStrokes	 17
U+4482.0	fCangjie	HYVMI
U+4482.0	fFourCorner	22440
U+4482.0	fHakka	  ziong3
U+4482.0	fCantonese	zoeng2
U+4482.0	fMandarin	jiang3; xiang3
U+4482.0	fEnglish	 oar
U+4483.0	fUTF8	䒃
U+4483.0	fCNS11643	3-5626
U+4483.0	fR/S	 137.11
U+4483.0	fTotalStrokes	 17
U+4483.0	fCangjie	HYYHR
U+4483.0	fFourCorner	24430
U+4483.0	fCantonese	zou6
U+4483.0	fMandarin	zao4; cao4
U+4483.0	fEnglish	same as &#x8241;, &#x9020;
U+4484.0	fUTF8	䒄
U+4484.0	fCNS11643	5-5D67
U+4484.0	fR/S	 137.11
U+4484.0	fTotalStrokes	 17
U+4484.0	fCangjie	HYBMW
U+4484.0	fFourCorner	27460
U+4484.0	fCantonese	fu3
U+4484.0	fMandarin	fu4; bo2
U+4484.0	fEnglish	[1] fully laden of a boat [2] the name of a ship
U+4485.0	fUTF8	䒅
U+4485.0	fCNS11643	5-5C4D
U+4485.0	fGB	3-6571
U+4485.0	fR/S	 137.11
U+4485.0	fTotalStrokes	 17
U+4485.0	fCangjie	HYFQU
U+4485.0	fFourCorner	29460
U+4485.0	fCantonese	tang4
U+4485.0	fMandarin	teng2
U+4485.0	fEnglish	[1] beautiful eyes [2] pair; two
U+4486.0	fUTF8	䒆
U+4486.0	fCNS11643	4-5832
U+4486.0	fGB	3-6574
U+4486.0	fR/S	 137.12; 137.11
U+4486.0	fTotalStrokes	 18; 16; 17
U+4486.0	fCangjie	HYYBK
U+4486.0	fFourCorner	28440
U+4486.0	fCantonese	cit3
U+4486.0	fMandarin	che4; she4
U+4486.0	fEnglish	[1] the name of a ship [2] sailing boat
U+4487.0	fUTF8	䒇
U+4487.0	fGB	3-6575
U+4487.0	fR/S	 137.12
U+4487.0	fTotalStrokes	 18
U+4487.0	fCangjie	HYJMW
U+4487.0	fCantonese	fu3; daau2; daau6
U+4487.0	fMandarin	fu4
U+4487.0	fEnglish	name of a ship, fully load of a boat
U+4488.0	fUTF8	䒈
U+4488.0	fCNS11643	4-5C7C
U+4488.0	fGB	3-6566
U+4488.0	fR/S	 137.12
U+4488.0	fTotalStrokes	 18
U+4488.0	fCangjie	HYJTC
U+4488.0	fFourCorner	24480
U+4488.0	fCantonese	fei6
U+4488.0	fMandarin	fei4; bu3; fe4
U+4488.0	fEnglish	same as &#x6A68;
U+4489.0	fUTF8	䒉
U+4489.0	fCNS11643	4-642D
U+4489.0	fGB	3-6576
U+4489.0	fR/S	 137.14
U+4489.0	fTotalStrokes	 20
U+4489.0	fCangjie	HYOTQ
U+4489.0	fFourCorner	28450
U+4489.0	fCantonese	mou5
U+4489.0	fMandarin	wu3
U+4489.0	fEnglish	a long, narrow boat
U+448A.0	fUTF8	䒊
U+448A.0	fCNS11643	4-2F53
U+448A.0	fGB	3-6578
U+448A.0	fR/S	 139.3; 139.4
U+448A.0	fTotalStrokes	 10; 9
U+448A.0	fCangjie	CSNAU
U+448A.0	fFourCorner	87210
U+448A.0	fCantonese	hei1; haai4
U+448A.0	fMandarin	xi1
U+448A.0	fEnglish	a patient (of jaundice; icterus) facial complexion
U+448B.0	fUTF8	䒋
U+448B.0	fCNS11643	4-345C
U+448B.0	fGB	3-6579
U+448B.0	fR/S	 139.5
U+448B.0	fTotalStrokes	 11
U+448B.0	fCangjie	LKNAU
U+448B.0	fFourCorner	57010
U+448B.0	fCantonese	joeng2
U+448B.0	fMandarin	yang3
U+448B.0	fEnglish	air current; atmospheric current
U+448C.0	fUTF8	䒌
U+448C.0	fCNS11643	4-526F
U+448C.0	fGB	3-657D
U+448C.0	fR/S	 139.10
U+448C.0	fTotalStrokes	 16
U+448C.0	fCangjie	BCNAU
U+448C.0	fFourCorner	37810
U+448C.0	fCantonese	ming6
U+448C.0	fMandarin	ming4
U+448C.0	fEnglish	dark blue colour, without colour, to close the eyes
U+448D.0	fUTF8	䒍
U+448D.0	fCNS11643	4-526E
U+448D.0	fGB	3-657C
U+448D.0	fR/S	 139.10
U+448D.0	fTotalStrokes	 16
U+448D.0	fCangjie	YSNAU
U+448D.0	fFourCorner	07210
U+448D.0	fCantonese	pong2
U+448D.0	fMandarin	pang3
U+448D.0	fEnglish	without colour, clear and lucid
U+448E.0	fUTF8	䒎
U+448E.0	fCNS11643	4-5835
U+448E.0	fGB	3-657B
U+448E.0	fR/S	 139.11; 139.12
U+448E.0	fTotalStrokes	 16; 18; 17
U+448E.0	fCangjie	TTNAU
U+448E.0	fFourCorner	47410
U+448E.0	fCantonese	mong5
U+448E.0	fMandarin	mang3
U+448E.0	fEnglish	[1] without color [2] clear and lucid
U+448F.0	fUTF8	䒏
U+448F.0	fCNS11643	4-5C7D
U+448F.0	fGB	3-657E
U+448F.0	fR/S	 139.12
U+448F.0	fTotalStrokes	 18
U+448F.0	fCangjie	CANAU
U+448F.0	fFourCorner	87610
U+448F.0	fCantonese	zang2; zang1
U+448F.0	fMandarin	seng1; ceng4
U+448F.0	fEnglish	[1] disturbed in mind [2] not in good health [3] a fierce or ferocious appearance
U+4490.0	fUTF8	䒐
U+4490.0	fCNS11643	4-6933
U+4490.0	fGB	3-6621
U+4490.0	fR/S	 139.16
U+4490.0	fTotalStrokes	 22; 21
U+4490.0	fCangjie	TUNAU
U+4490.0	fFourCorner	47610
U+4490.0	fCantonese	mang4; ci2; mung6
U+4490.0	fMandarin	meng2; meng4
U+4490.0	fEnglish	[1] disturbed in mind [2] not in good health [3] a fierce or ferocious appearance, a bad look
U+4491.0	fUTF8	䒑
U+4491.0	fR/S	 140.0
U+4491.0	fTotalStrokes	 6
U+4491.0	fCangjie	CM
U+4491.0	fCantonese	baat3; cou2
U+4491.0	fMandarin	cao3
U+4491.0	fEnglish	non-classical form of &#x8278;
U+4492.0	fUTF8	䒒
U+4492.0	fCNS11643	6-254F
U+4492.0	fR/S	 140.2
U+4492.0	fTotalStrokes	 6; 8
U+4492.0	fCangjie	TSM
U+4492.0	fFourCorner	44120
U+4492.0	fHakka	 tiau2; tiau1
U+4492.0	fCantonese	tiu4; seoi1; waai2; zaau4; ziu5
U+4492.0	fMandarin	tiao2; yao3; you2
U+4492.0	fEnglish	name of a plant
U+4493.0	fUTF8	䒓
U+4493.0	fCNS11643	3-2437
U+4493.0	fGB	5-673A
U+4493.0	fR/S	 140.2
U+4493.0	fTotalStrokes	 6; 8
U+4493.0	fCangjie	TMVS
U+4493.0	fFourCorner	44200
U+4493.0	fCantonese	kaai2
U+4493.0	fMandarin	kai3
U+4493.0	fEnglish	to do violence, perverse; rebellious, calamities; tribulations; miseries; crime; sin
U+4494.0	fUTF8	䒔
U+4494.0	fCNS11643	3-2439
U+4494.0	fGB	5-673B
U+4494.0	fR/S	 140.2
U+4494.0	fTotalStrokes	 6; 8
U+4494.0	fCangjie	TC
U+4494.0	fFourCorner	44800
U+4494.0	fCantonese	baai3
U+4494.0	fMandarin	bai4
U+4494.0	fEnglish	name of a variety of grass
U+4495.0	fUTF8	䒕
U+4495.0	fCNS11643	3-2727
U+4495.0	fGB	5-6746
U+4495.0	fR/S	 140.3
U+4495.0	fTotalStrokes	 7; 9
U+4495.0	fCangjie	TF
U+4495.0	fFourCorner	44900
U+4495.0	fCantonese	siu2
U+4495.0	fMandarin	xiao3
U+4495.0	fEnglish	[1] a kind of herb medicine [2] luxuriant; flourishing of grass and trees
U+4496.0	fUTF8	䒖
U+4496.0	fCNS11643	4-2476
U+4496.0	fGB	3-6836
U+4496.0	fR/S	 140.3
U+4496.0	fTotalStrokes	 7; 9
U+4496.0	fCangjie	TNJ
U+4496.0	fFourCorner	44410
U+4496.0	fCantonese	seon3
U+4496.0	fMandarin	xin4
U+4496.0	fEnglish	a kind of herb medicine (plants of the mugwort or artemisia family)
U+4497.0	fUTF8	䒗
U+4497.0	fCNS11643	4-247D
U+4497.0	fGB	3-6834
U+4497.0	fR/S	 140.3
U+4497.0	fTotalStrokes	 7; 9
U+4497.0	fCangjie	TON
U+4497.0	fFourCorner	44710
U+4497.0	fHakka	 kiet7;  ket7
U+4497.0	fCantonese	hat1; gaat1; hei3
U+4497.0	fMandarin	qi3; qi4
U+4497.0	fEnglish	a fragrant plant or grass, offered at the autumn festival
U+4498.0	fUTF8	䒘
U+4498.0	fCNS11643	3-272C
U+4498.0	fR/S	 140.3
U+4498.0	fTotalStrokes	 7; 9
U+4498.0	fCangjie	THE
U+4498.0	fFourCorner	44407
U+4498.0	fCantonese	seoi1
U+4498.0	fMandarin	sui1
U+4498.0	fEnglish	[1] same as &#x837E; [2] same as &#x8470;
U+4499.0	fUTF8	䒙
U+4499.0	fGB	S-217A
U+4499.0	fR/S	 140.3
U+4499.0	fTotalStrokes	 9
U+4499.0	fCangjie	TYM
U+4499.0	fCantonese	cong4
U+449A.0	fUTF8	䒚
U+449A.0	fCNS11643	3-2A63
U+449A.0	fR/S	 140.4
U+449A.0	fTotalStrokes	 8; 10
U+449A.0	fCangjie	TFH
U+449A.0	fFourCorner	44200
U+449A.0	fCantonese	siu2
U+449A.0	fMandarin	shao3
U+449A.0	fEnglish	the name of a variety of grass
U+449B.0	fUTF8	䒛
U+449B.0	fCNS11643	3-2A67
U+449B.0	fR/S	 140.4
U+449B.0	fTotalStrokes	 8; 10
U+449B.0	fCangjie	TVIS
U+449B.0	fFourCorner	44720
U+449B.0	fCantonese	waan6
U+449B.0	fMandarin	huan4; heng2
U+449B.0	fEnglish	the name of a variety of grass
U+449C.0	fUTF8	䒜
U+449C.0	fCNS11643	3-2A64
U+449C.0	fGB	5-674C
U+449C.0	fR/S	 140.4
U+449C.0	fTotalStrokes	 8; 10
U+449C.0	fCangjie	THQ
U+449C.0	fFourCorner	44500
U+449C.0	fHakka	  ngiu2
U+449C.0	fCantonese	ngau4; au4 (also); ngaau4
U+449C.0	fMandarin	niu2
U+449C.0	fEnglish	a medicinal plant
U+449D.0	fUTF8	䒝
U+449D.0	fCNS11643	3-2A6A
U+449D.0	fR/S	 140.4
U+449D.0	fTotalStrokes	 8; 10
U+449D.0	fCangjie	TKK
U+449D.0	fFourCorner	44400
U+449D.0	fCantonese	haau4
U+449D.0	fMandarin	xiao2
U+449D.0	fEnglish	root of the thatch, straw or couch grass; a herb medicine to quench thirst
U+449E.0	fUTF8	䒞
U+449E.0	fCNS11643	4-2772
U+449E.0	fGB	3-6846
U+449E.0	fR/S	 140.4
U+449E.0	fTotalStrokes	 8; 10
U+449E.0	fCangjie	TLBU
U+449E.0	fFourCorner	44410
U+449E.0	fCantonese	cam4
U+449E.0	fMandarin	yin2; chen2
U+449E.0	fEnglish	the name of a variety of grass, hot
U+449F.0	fUTF8	䒟
U+449F.0	fCNS11643	3-2A62
U+449F.0	fR/S	 140.4
U+449F.0	fTotalStrokes	 8; 10
U+449F.0	fCangjie	TBY
U+449F.0	fFourCorner	44440
U+449F.0	fCantonese	daan1
U+449F.0	fMandarin	dan1
U+449F.0	fEnglish	the name of a variety of grass
U+44A0.0	fUTF8	䒠
U+44A0.0	fCNS11643	4-2821
U+44A0.0	fGB	3-6838
U+44A0.0	fR/S	 140.4
U+44A0.0	fTotalStrokes	 8; 10
U+44A0.0	fCangjie	TQJ
U+44A0.0	fFourCorner	44500
U+44A0.0	fCantonese	fung1
U+44A0.0	fMandarin	feng1; fan3; xia2
U+44A0.0	fEnglish	same as &#x4E30;
U+44A1.0	fUTF8	䒡
U+44A1.0	fCNS11643	4-2773
U+44A1.0	fGB	3-6848
U+44A1.0	fR/S	 140.4
U+44A1.0	fTotalStrokes	 8; 10
U+44A1.0	fCangjie	TNL
U+44A1.0	fFourCorner	44220
U+44A1.0	fCantonese	jan5; zaan5
U+44A1.0	fMandarin	yin3
U+44A1.0	fEnglish	the name of a variety of grass
U+44A2.0	fUTF8	䒢
U+44A2.0	fCNS11643	3-2A65
U+44A2.0	fGB	5-6756
U+44A2.0	fR/S	 140.4
U+44A2.0	fTotalStrokes	 8; 10
U+44A2.0	fCangjie	THVL
U+44A2.0	fFourCorner	44727
U+44A2.0	fCantonese	ngong4; ong4 (also)
U+44A2.0	fMandarin	ang2; ying4
U+44A2.0	fEnglish	sweet flag; the calamus; its leaves are hung on the door lintels on the 5th of the5th lunar month, to avert evil influences
U+44A3.0	fUTF8	䒣
U+44A3.0	fCNS11643	3-2A69
U+44A3.0	fR/S	 140.4
U+44A3.0	fTotalStrokes	 8; 10
U+44A3.0	fCangjie	TBMM
U+44A3.0	fFourCorner	44550
U+44A3.0	fCantonese	jim5; zim5
U+44A3.0	fMandarin	ran3
U+44A3.0	fEnglish	same as &#x82D2;
U+44A4.0	fUTF8	䒤
U+44A4.0	fCNS11643	5-2763
U+44A4.0	fR/S	 140.4
U+44A4.0	fTotalStrokes	 8; 10
U+44A4.0	fCangjie	TA
U+44A4.0	fFourCorner	44600
U+44A4.0	fCantonese	jat6; zaat6
U+44A4.0	fMandarin	ri4
U+44A4.0	fEnglish	the name of a variety of grass
U+44A5.0	fUTF8	䒥
U+44A5.0	fCNS11643	3-2A61
U+44A5.0	fR/S	 140.4
U+44A5.0	fTotalStrokes	 8; 10
U+44A5.0	fCangjie	TLB
U+44A5.0	fFourCorner	44220
U+44A5.0	fCantonese	mun4
U+44A5.0	fMandarin	man2; fa4; liu3
U+44A5.0	fEnglish	corresponding; equivalent, considerable; to a great extent, appropriate
U+44A6.0	fUTF8	䒦
U+44A6.0	fCNS11643	3-2E6D
U+44A6.0	fGB	5-6754
U+44A6.0	fR/S	 140.5
U+44A6.0	fTotalStrokes	 9; 11
U+44A6.0	fCangjie	THIO
U+44A6.0	fFourCorner	44300
U+44A6.0	fHakka	 fam5
U+44A6.0	fCantonese	faan3
U+44A6.0	fMandarin	pan1; fan4
U+44A6.0	fEnglish	plants floating on the water
U+44A7.0	fUTF8	䒧
U+44A7.0	fCNS11643	4-2B36
U+44A7.0	fGB	3-684B
U+44A7.0	fR/S	 140.5
U+44A7.0	fTotalStrokes	 9; 11
U+44A7.0	fCangjie	TGI
U+44A7.0	fFourCorner	44730
U+44A7.0	fCantonese	keoi1; goi1; heoi3
U+44A7.0	fMandarin	qu4
U+44A7.0	fEnglish	[1] a vessel or utensil made of straw [2] grass or weeds [3] a kind of grass
U+44A8.0	fUTF8	䒨
U+44A8.0	fCNS11643	3-2E70
U+44A8.0	fGB	5-675F
U+44A8.0	fR/S	 140.5
U+44A8.0	fTotalStrokes	 9; 11
U+44A8.0	fCangjie	TOK
U+44A8.0	fFourCorner	44430
U+44A8.0	fCantonese	si2; ci2
U+44A8.0	fMandarin	si4; shi3
U+44A8.0	fEnglish	[1] a kind of vegetable [2] plants of the mugwort or artemisia family
U+44A9.0	fUTF8	䒩
U+44A9.0	fCNS11643	3-2E71
U+44A9.0	fGB	5-6760
U+44A9.0	fR/S	 140.5
U+44A9.0	fTotalStrokes	 9; 11
U+44A9.0	fCangjie	THD
U+44A9.0	fFourCorner	44900
U+44A9.0	fCantonese	wo4; wu6
U+44A9.0	fMandarin	he2; xia2
U+44A9.0	fEnglish	the name of a variety of grass, same as &#x83AB;) not
U+44AA.0	fUTF8	䒪
U+44AA.0	fCNS11643	3-2E6C
U+44AA.0	fGB	5-6766
U+44AA.0	fR/S	 140.5
U+44AA.0	fTotalStrokes	 9; 11
U+44AA.0	fCangjie	TIT
U+44AA.0	fFourCorner	44440
U+44AA.0	fCantonese	bin6
U+44AA.0	fMandarin	bian4
U+44AA.0	fEnglish	the name of a variety of grass
U+44AB.0	fUTF8	䒫
U+44AB.0	fCNS11643	3-2E73
U+44AB.0	fR/S	 140.5
U+44AB.0	fTotalStrokes	 9; 11
U+44AB.0	fCangjie	TOIP
U+44AB.0	fFourCorner	44240
U+44AB.0	fCantonese	doi6
U+44AB.0	fMandarin	dai4
U+44AB.0	fEnglish	grassy
U+44AC.0	fUTF8	䒬
U+44AC.0	fCNS11643	3-2E75
U+44AC.0	fR/S	 140.5
U+44AC.0	fTotalStrokes	 9; 11
U+44AC.0	fCangjie	TMMF
U+44AC.0	fFourCorner	44900
U+44AC.0	fHakka	 mok8
U+44AC.0	fCantonese	mok6
U+44AC.0	fMandarin	mo4
U+44AC.0	fEnglish	[1] ancient form of &#x83AB; [2] standard form of &#x66AE;
U+44AD.0	fUTF8	䒭
U+44AD.0	fCNS11643	6-343A
U+44AD.0	fR/S	 140.5
U+44AD.0	fTotalStrokes	 9; 11
U+44AD.0	fCangjie	TNMI
U+44AD.0	fFourCorner	44340
U+44AD.0	fCantonese	dang2
U+44AD.0	fMandarin	deng3
U+44AD.0	fEnglish	same as &#x7B49;
U+44AE.0	fUTF8	䒮
U+44AE.0	fGB	S-2222
U+44AE.0	fR/S	 140.5
U+44AE.0	fTotalStrokes	 11
U+44AE.0	fCangjie	TBHN
U+44AE.0	fCantonese	gong1
U+44AF.0	fUTF8	䒯
U+44AF.0	fGB	S-2223
U+44AF.0	fR/S	 140.5
U+44AF.0	fTotalStrokes	 11
U+44AF.0	fCangjie	TBKI
U+44AF.0	fCantonese	jyun4
U+44B0.0	fUTF8	䒰
U+44B0.0	fCNS11643	4-2F55
U+44B0.0	fGB	5-676A
U+44B0.0	fR/S	 140.6
U+44B0.0	fTotalStrokes	 10; 12
U+44B0.0	fCangjie	TSMG
U+44B0.0	fFourCorner	44710
U+44B0.0	fCantonese	hong1; hong4
U+44B0.0	fMandarin	kuang1
U+44B0.0	fEnglish	[1] follow; trace [2] submit; accord with [3] accompany [4] a kind of grass
U+44B1.0	fUTF8	䒱
U+44B1.0	fCNS11643	3-3361
U+44B1.0	fR/S	 140.6
U+44B1.0	fTotalStrokes	 10; 12
U+44B1.0	fCangjie	TNEM
U+44B1.0	fFourCorner	44100
U+44B1.0	fCantonese	zing1
U+44B1.0	fMandarin	zheng1
U+44B1.0	fEnglish	same as &#x84B8;
U+44B2.0	fUTF8	䒲
U+44B2.0	fCNS11643	4-2F61
U+44B2.0	fGB	3-6879
U+44B2.0	fR/S	 140.6
U+44B2.0	fTotalStrokes	 10; 12
U+44B2.0	fCangjie	TVHP
U+44B2.0	fFourCorner	44410
U+44B2.0	fCantonese	zaa3
U+44B2.0	fMandarin	cha4
U+44B2.0	fEnglish	a medical herb; a second name for Scutellaria baikalensis ( 黃岑)
U+44B3.0	fUTF8	䒳
U+44B3.0	fCNS11643	3-3364
U+44B3.0	fR/S	 140.6
U+44B3.0	fTotalStrokes	 10; 12
U+44B3.0	fCangjie	TNSD
U+44B3.0	fFourCorner	44904
U+44B3.0	fCantonese	do2; deoi2
U+44B3.0	fMandarin	zhua1; duo3
U+44B3.0	fEnglish	same as &#x6736;, &#x6735;
U+44B4.0	fUTF8	䒴
U+44B4.0	fCNS11643	3-335B
U+44B4.0	fGB	5-6771
U+44B4.0	fR/S	 140.6
U+44B4.0	fTotalStrokes	 10; 12
U+44B4.0	fCangjie	TKB
U+44B4.0	fFourCorner	44220
U+44B4.0	fCantonese	jau5; zaau5
U+44B4.0	fMandarin	you3
U+44B4.0	fEnglish	the name of a variety of grass
U+44B5.0	fUTF8	䒵
U+44B5.0	fCNS11643	3-335E
U+44B5.0	fR/S	 140.6
U+44B5.0	fTotalStrokes	 10; 12
U+44B5.0	fCangjie	TVND
U+44B5.0	fFourCorner	44447
U+44B5.0	fCantonese	hou1; hou3
U+44B5.0	fMandarin	hao4
U+44B5.0	fEnglish	the name of a variety of grass
U+44B6.0	fUTF8	䒶
U+44B6.0	fCNS11643	6-3B56
U+44B6.0	fR/S	 140.6
U+44B6.0	fTotalStrokes	 10; 9; 12
U+44B6.0	fCangjie	TLWP
U+44B6.0	fFourCorner	44500
U+44B6.0	fCantonese	tin1
U+44B6.0	fMandarin	tian1
U+44B6.0	fEnglish	ancient form of &#x5929;
U+44B7.0	fUTF8	䒷
U+44B7.0	fCNS11643	3-335D
U+44B7.0	fGB	5-6778
U+44B7.0	fR/S	 140.6
U+44B7.0	fTotalStrokes	 10; 12
U+44B7.0	fCangjie	THJR
U+44B7.0	fFourCorner	44600
U+44B7.0	fHakka	 kwat7
U+44B7.0	fCantonese	kut3
U+44B7.0	fMandarin	gua1
U+44B7.0	fEnglish	a bitter plant used as a remedy for coughs
U+44B8.0	fUTF8	䒸
U+44B8.0	fCNS11643	4-2F5E
U+44B8.0	fGB	5-677B
U+44B8.0	fR/S	 140.6
U+44B8.0	fTotalStrokes	 10; 12
U+44B8.0	fCangjie	THBT
U+44B8.0	fFourCorner	44100
U+44B8.0	fCantonese	hyut3
U+44B8.0	fMandarin	xue4; xian2; yue4
U+44B8.0	fEnglish	grass
U+44B9.0	fUTF8	䒹
U+44B9.0	fCNS11643	3-335C
U+44B9.0	fGB	5-676B
U+44B9.0	fR/S	 140.6
U+44B9.0	fTotalStrokes	 10; 12
U+44B9.0	fCangjie	TQD
U+44B9.0	fFourCorner	44900
U+44B9.0	fCantonese	loi6
U+44B9.0	fMandarin	lei2; lei4
U+44B9.0	fEnglish	grassy, fruitful
U+44BA.0	fUTF8	䒺
U+44BA.0	fCNS11643	3-3365
U+44BA.0	fR/S	 140.6
U+44BA.0	fTotalStrokes	 10; 12
U+44BA.0	fCangjie	TVIF
U+44BA.0	fFourCorner	44900
U+44BA.0	fCantonese	gan2
U+44BA.0	fMandarin	jin3
U+44BA.0	fEnglish	ancient form of &#x5807;
U+44BB.0	fUTF8	䒻
U+44BB.0	fCNS11643	5-3021
U+44BB.0	fR/S	 140.6
U+44BB.0	fTotalStrokes	 10; 12
U+44BB.0	fCangjie	TURU
U+44BB.0	fFourCorner	44710
U+44BB.0	fHakka	 ki3
U+44BB.0	fCantonese	hei2
U+44BB.0	fMandarin	qi3
U+44BB.0	fEnglish	a plant with a bitter taste, used for gunshot wounds
U+44BC.0	fUTF8	䒼
U+44BC.0	fCNS11643	4-2F59
U+44BC.0	fGB	5-6776
U+44BC.0	fR/S	 140.6
U+44BC.0	fTotalStrokes	 10; 12
U+44BC.0	fCangjie	TTW
U+44BC.0	fFourCorner	44600
U+44BC.0	fHakka	 kiuk7
U+44BC.0	fCantonese	kuk1
U+44BC.0	fMandarin	qu1
U+44BC.0	fEnglish	bamboo frame used in rearing silk worms
U+44BD.0	fUTF8	䒽
U+44BD.0	fCNS11643	4-2F64
U+44BD.0	fGB	3-686E
U+44BD.0	fR/S	 140.6
U+44BD.0	fTotalStrokes	 10; 12
U+44BD.0	fCangjie	TBKK
U+44BD.0	fFourCorner	44220
U+44BD.0	fCantonese	mong5
U+44BD.0	fMandarin	wang3; mei2
U+44BD.0	fEnglish	Illicium anisatum, and kind of poisonous shrub
U+44BE.0	fUTF8	䒾
U+44BE.0	fR/S	 140.6
U+44BE.0	fTotalStrokes	 12
U+44BE.0	fCangjie	TYHV
U+44BE.0	fFourCorner	4473
U+44BE.0	fCantonese	ji1; zi1; ji2; zi2
U+44BF.0	fUTF8	䒿
U+44BF.0	fGB	S-2224
U+44BF.0	fR/S	 140.6
U+44BF.0	fTotalStrokes	 12
U+44BF.0	fCangjie	TBB
U+44C0.0	fUTF8	䓀
U+44C0.0	fR/S	 140.6
U+44C0.0	fTotalStrokes	 12
U+44C0.0	fCangjie	TGR
U+44C0.0	fCantonese	gat1
U+44C0.0	fEnglish	[Korean character]
U+44C1.0	fUTF8	䓁
U+44C1.0	fCNS11643	3-3360
U+44C1.0	fR/S	 140.6
U+44C1.0	fTotalStrokes	 10; 12
U+44C1.0	fCangjie	TGDI
U+44C1.0	fFourCorner	44340
U+44C1.0	fHakka	 den3
U+44C1.0	fCantonese	dang2
U+44C1.0	fMandarin	deng3
U+44C2.0	fUTF8	䓂
U+44C2.0	fCNS11643	4-345E
U+44C2.0	fGB	3-6933
U+44C2.0	fR/S	 140.7
U+44C2.0	fTotalStrokes	 11; 13
U+44C2.0	fCangjie	TYMR
U+44C2.0	fFourCorner	44600
U+44C2.0	fCantonese	jin4; zin4
U+44C2.0	fMandarin	yan2
U+44C2.0	fEnglish	[1] the name of a variety of grass [2] a family name
U+44C3.0	fUTF8	䓃
U+44C3.0	fCNS11643	4-346C
U+44C3.0	fGB	3-692A
U+44C3.0	fR/S	 140.7
U+44C3.0	fTotalStrokes	 11; 13
U+44C3.0	fCangjie	TRAU
U+44C3.0	fFourCorner	44710
U+44C3.0	fCantonese	jap1; zaap1
U+44C3.0	fMandarin	yi4
U+44C3.0	fEnglish	[1] melancholy and sad [2] withered (said of grass and trees)
U+44C4.0	fUTF8	䓄
U+44C4.0	fCNS11643	4-3471
U+44C4.0	fGB	3-6930
U+44C4.0	fR/S	 140.7
U+44C4.0	fTotalStrokes	 11; 13
U+44C4.0	fCangjie	TKHL
U+44C4.0	fFourCorner	44220
U+44C4.0	fCantonese	ngan4; an4 (also)
U+44C4.0	fMandarin	yin2
U+44C4.0	fEnglish	grassy, name of a variety of grass
U+44C5.0	fUTF8	䓅
U+44C5.0	fCNS11643	3-385A
U+44C5.0	fR/S	 140.7
U+44C5.0	fTotalStrokes	 11; 13
U+44C5.0	fCangjie	TEHL
U+44C5.0	fFourCorner	44120
U+44C5.0	fCantonese	kan4
U+44C5.0	fMandarin	qi2
U+44C5.0	fEnglish	same as &#x82B9;
U+44C6.0	fUTF8	䓆
U+44C6.0	fCNS11643	4-3469
U+44C6.0	fGB	5-682F
U+44C6.0	fR/S	 140.7
U+44C6.0	fTotalStrokes	 11; 13
U+44C6.0	fCangjie	TQHL
U+44C6.0	fFourCorner	44520
U+44C6.0	fCantonese	zit3; zoek3
U+44C6.0	fMandarin	zhe4; zhe2
U+44C6.0	fEnglish	 cut, sever grass
U+44C7.0	fUTF8	䓇
U+44C7.0	fCNS11643	4-3465
U+44C7.0	fGB	5-6826
U+44C7.0	fR/S	 140.7
U+44C7.0	fTotalStrokes	 11; 13
U+44C7.0	fCangjie	TGLC
U+44C7.0	fFourCorner	44330
U+44C7.0	fCantonese	jik1; hak1
U+44C7.0	fMandarin	xi4
U+44C7.0	fEnglish	[1] luxuriant; lush of grass and trees [2] same as &#x8D6B; [3] same as &#x5687;
U+44C8.0	fUTF8	䓈
U+44C8.0	fCNS11643	3-3856
U+44C8.0	fGB	5-6839
U+44C8.0	fR/S	 140.7
U+44C8.0	fTotalStrokes	 11; 13
U+44C8.0	fCangjie	THOE
U+44C8.0	fFourCorner	44240
U+44C8.0	fCantonese	jik6; zik6
U+44C8.0	fMandarin	yi4
U+44C8.0	fEnglish	a kind of plant; chicken-head; Euryale ferox
U+44C9.0	fUTF8	䓉
U+44C9.0	fCNS11643	4-3466
U+44C9.0	fGB	3-6869
U+44C9.0	fR/S	 140.7
U+44C9.0	fTotalStrokes	 11; 13
U+44C9.0	fCangjie	TMHL
U+44C9.0	fFourCorner	44220
U+44C9.0	fCantonese	je4; ze4
U+44C9.0	fMandarin	xie2; ye2
U+44C9.0	fEnglish	[1] the name of a variety of grass [2] a kind of vegetable [3] same as &#x6930;
U+44CA.0	fUTF8	䓊
U+44CA.0	fCNS11643	4-3462
U+44CA.0	fGB	3-6921
U+44CA.0	fR/S	 140.7
U+44CA.0	fTotalStrokes	 11; 13
U+44CA.0	fCangjie	TMMR
U+44CA.0	fFourCorner	44600
U+44CA.0	fCantonese	ng4
U+44CA.0	fMandarin	wu2; e4; yu2
U+44CA.0	fEnglish	the name of a variety of grass (as moxa) (Perilla ocimoides)
U+44CB.0	fUTF8	䓋
U+44CB.0	fCNS11643	4-345D
U+44CB.0	fR/S	 140.7
U+44CB.0	fTotalStrokes	 11; 13
U+44CB.0	fCangjie	TEHP
U+44CB.0	fFourCorner	44140
U+44CB.0	fCantonese	zi1
U+44CB.0	fMandarin	zhi1
U+44CB.0	fEnglish	elm-seeds (used as food)
U+44CC.0	fUTF8	䓌
U+44CC.0	fCNS11643	4-3463
U+44CC.0	fR/S	 140.7
U+44CC.0	fTotalStrokes	 11; 13
U+44CC.0	fCangjie	TGP
U+44CC.0	fFourCorner	44330
U+44CC.0	fHakka	 zhi5; zih5
U+44CC.0	fCantonese	zi3
U+44CC.0	fMandarin	zhi4
U+44CC.0	fEnglish	a kind of medicine used in fevers
U+44CD.0	fUTF8	䓍
U+44CD.0	fCNS11643	6-442C
U+44CD.0	fR/S	 140.7
U+44CD.0	fTotalStrokes	 11; 13
U+44CD.0	fCangjie	TAMJ
U+44CD.0	fFourCorner	44400
U+44CD.0	fCantonese	hon2
U+44CD.0	fMandarin	han4; han3
U+44CD.0	fEnglish	cruciferous flowers; with acrid seeds; used as medicine
U+44CE.0	fUTF8	䓎
U+44CE.0	fCNS11643	3-384C
U+44CE.0	fR/S	 140.7
U+44CE.0	fTotalStrokes	 11; 13
U+44CE.0	fCangjie	TFPI
U+44CE.0	fFourCorner	44820
U+44CE.0	fCantonese	cuk1
U+44CE.0	fMandarin	chuo4
U+44CE.0	fEnglish	name of a variety of grass
U+44CF.0	fUTF8	䓏
U+44CF.0	fCNS11643	6-4428
U+44CF.0	fR/S	 140.7
U+44CF.0	fTotalStrokes	 11; 13
U+44CF.0	fCangjie	TMFR
U+44CF.0	fFourCorner	44600
U+44CF.0	fHakka	 pi1
U+44CF.0	fCantonese	fu1
U+44CF.0	fMandarin	fu1
U+44CF.0	fEnglish	flourishing; luxuriant; a lush growth of flowers
U+44D0.0	fUTF8	䓐
U+44D0.0	fCNS11643	5-352A
U+44D0.0	fR/S	 140.7
U+44D0.0	fTotalStrokes	 11; 13
U+44D0.0	fCangjie	TKPU
U+44D0.0	fFourCorner	44710
U+44D0.0	fCantonese	seon4
U+44D0.0	fMandarin	chun2
U+44D0.0	fEnglish	the name of a variety of grass
U+44D1.0	fUTF8	䓑
U+44D1.0	fCNS11643	3-3859
U+44D1.0	fR/S	 140.7
U+44D1.0	fTotalStrokes	 11; 13
U+44D1.0	fCangjie	TEMT
U+44D1.0	fFourCorner	44140
U+44D1.0	fCantonese	ping4
U+44D1.0	fMandarin	ping2; bing3
U+44D1.0	fEnglish	[1] same as &#x8353; [2] same as &#x840D;
U+44D2.0	fUTF8	䓒
U+44D2.0	fCNS11643	3-385C
U+44D2.0	fR/S	 140.7
U+44D2.0	fTotalStrokes	 11; 13
U+44D2.0	fCangjie	TPW
U+44D2.0	fFourCorner	44620
U+44D2.0	fCantonese	gwaai2
U+44D2.0	fMandarin	guai4; kuai3
U+44D2.0	fEnglish	same as &#x84AF;
U+44D3.0	fUTF8	䓓
U+44D3.0	fCNS11643	5-3524
U+44D3.0	fR/S	 140.7
U+44D3.0	fTotalStrokes	 11; 13
U+44D3.0	fCangjie	TQKI
U+44D3.0	fFourCorner	44540
U+44D3.0	fCantonese	caau4; zaau6
U+44D3.0	fMandarin	shan4; chou2
U+44D3.0	fEnglish	[1] simplified form of &#x85B5; [2] the name of a variety of grass [3] same as &#x8464;
U+44D4.0	fUTF8	䓔
U+44D4.0	fGB	S-2221
U+44D4.0	fR/S	 140.7
U+44D4.0	fTotalStrokes	 13
U+44D4.0	fCangjie	TJKD
U+44D4.0	fCantonese	haau3
U+44D5.0	fUTF8	䓕
U+44D5.0	fGB	S-2372
U+44D5.0	fR/S	 140.8
U+44D5.0	fTotalStrokes	 14
U+44D5.0	fCangjie	TYMU
U+44D5.0	fCantonese	waai2
U+44D5.0	fMandarin	tuo3; wei3
U+44D5.0	fEnglish	simplified form of &#x85B3;
U+44D6.0	fUTF8	䓖
U+44D6.0	fGB	7-2231
U+44D6.0	fR/S	 140.7
U+44D6.0	fTotalStrokes	 13
U+44D6.0	fCangjie	TJCS
U+44D6.0	fCantonese	kung4; wu4
U+44D6.0	fMandarin	qiong2
U+44D6.0	fEnglish	simplified form of &#x85ED;
U+44D7.0	fUTF8	䓗
U+44D7.0	fCNS11643	4-3A6C
U+44D7.0	fR/S	 140.8
U+44D7.0	fTotalStrokes	 12; 14
U+44D7.0	fCangjie	TCIP
U+44D7.0	fFourCorner	44330
U+44D7.0	fHakka	 cung1
U+44D7.0	fCantonese	cung1; tung2
U+44D7.0	fMandarin	cong1
U+44D7.0	fEnglish	same as &#x68E3;, &#x8525;
U+44D8.0	fUTF8	䓘
U+44D8.0	fCNS11643	4-3A60
U+44D8.0	fGB	3-6957
U+44D8.0	fR/S	 140.8
U+44D8.0	fTotalStrokes	 12; 14
U+44D8.0	fCangjie	THOR
U+44D8.0	fFourCorner	44600
U+44D8.0	fCantonese	gou1; gaau3
U+44D8.0	fMandarin	gao1; jiu4
U+44D8.0	fEnglish	[1] a kind of plant [2] the name of a variety of grass
U+44D9.0	fUTF8	䓙
U+44D9.0	fCNS11643	4-3A5F
U+44D9.0	fGB	3-6951
U+44D9.0	fR/S	 140.8
U+44D9.0	fTotalStrokes	 12; 14
U+44D9.0	fCangjie	THJP
U+44D9.0	fFourCorner	44110
U+44D9.0	fCantonese	kwaa1; gwai1
U+44D9.0	fMandarin	guai1
U+44D9.0	fEnglish	slanting; inclined; sloping
U+44DA.0	fUTF8	䓚
U+44DA.0	fCNS11643	4-3A64
U+44DA.0	fGB	3-6954
U+44DA.0	fR/S	 140.8
U+44DA.0	fTotalStrokes	 12; 14
U+44DA.0	fCangjie	THOM
U+44DA.0	fFourCorner	44210
U+44DA.0	fCantonese	ceoi1; gu1
U+44DA.0	fMandarin	xu2; cu2
U+44DA.0	fEnglish	a kind of vegetable, a water-plant; the bulbs of which are used as food - Sagittaria sagittifolia
U+44DB.0	fUTF8	䓛
U+44DB.0	fCNS11643	4-3A59
U+44DB.0	fGB	3-695F
U+44DB.0	fR/S	 140.8
U+44DB.0	fTotalStrokes	 12; 14
U+44DB.0	fCangjie	TSUU
U+44DB.0	fFourCorner	44270
U+44DB.0	fCantonese	wat1; gwat1; waat1
U+44DB.0	fMandarin	qu1; fu3; gu3
U+44DB.0	fEnglish	[1] molecular formula (C18H12) [2] brush
U+44DC.0	fUTF8	䓜
U+44DC.0	fCNS11643	5-3B56
U+44DC.0	fGB	5-6858
U+44DC.0	fR/S	 140.8
U+44DC.0	fTotalStrokes	 12; 14
U+44DC.0	fCangjie	TEHM
U+44DC.0	fFourCorner	44140
U+44DC.0	fCantonese	zi1
U+44DC.0	fMandarin	zhi1
U+44DC.0	fEnglish	salted or pickled vegetables
U+44DD.0	fUTF8	䓝
U+44DD.0	fCNS11643	4-3A5B
U+44DD.0	fGB	3-6962
U+44DD.0	fR/S	 140.8
U+44DD.0	fTotalStrokes	 12; 14
U+44DD.0	fCangjie	TNDT
U+44DD.0	fFourCorner	44100
U+44DD.0	fCantonese	maang6; laau4; mang6
U+44DD.0	fMandarin	mang4; meng2; meng4
U+44DD.0	fEnglish	Chinese pennisetum) Pennisetum alopecuroides)
U+44DE.0	fUTF8	䓞
U+44DE.0	fCNS11643	4-3A56
U+44DE.0	fGB	3-695C
U+44DE.0	fR/S	 140.8
U+44DE.0	fTotalStrokes	 12; 14
U+44DE.0	fCangjie	TISK
U+44DE.0	fFourCorner	44230
U+44DE.0	fHakka	 lui5; lui6; li6; li5
U+44DE.0	fCantonese	leoi6
U+44DE.0	fMandarin	li4
U+44DE.0	fEnglish	a stiff grass, the leaves of which are used for sandals
U+44DF.0	fUTF8	䓟
U+44DF.0	fCNS11643	3-3E53
U+44DF.0	fGB	5-6854
U+44DF.0	fR/S	 140.8
U+44DF.0	fTotalStrokes	 12; 14
U+44DF.0	fCangjie	TBGR
U+44DF.0	fFourCorner	44220
U+44DF.0	fCantonese	zau1
U+44DF.0	fMandarin	zhou1; lie4
U+44DF.0	fEnglish	[1] the name of a variety of grass [2] same as &#x8322;
U+44E0.0	fUTF8	䓠
U+44E0.0	fCNS11643	4-3A5E
U+44E0.0	fGB	3-6963
U+44E0.0	fR/S	 140.8
U+44E0.0	fTotalStrokes	 12; 14
U+44E0.0	fCangjie	TEA
U+44E0.0	fFourCorner	44600
U+44E0.0	fCantonese	taap3; dap6
U+44E0.0	fMandarin	ta4
U+44E0.0	fEnglish	[1] the water plantain [2] the lotus floating on the water
U+44E1.0	fUTF8	䓡
U+44E1.0	fCNS11643	3-3E54
U+44E1.0	fR/S	 140.8
U+44E1.0	fTotalStrokes	 12; 14
U+44E1.0	fCangjie	TOKR
U+44E1.0	fFourCorner	44460
U+44E1.0	fHakka	 di1; zhi1; zih1
U+44E1.0	fCantonese	zi1
U+44E1.0	fMandarin	zhi1
U+44E1.0	fEnglish	a plant the seeds of which are used as a cooling medicine
U+44E2.0	fUTF8	䓢
U+44E2.0	fCNS11643	3-3E51
U+44E2.0	fGB	5-684D
U+44E2.0	fR/S	 140.8
U+44E2.0	fTotalStrokes	 12; 14
U+44E2.0	fCangjie	TWJR
U+44E2.0	fFourCorner	44600
U+44E2.0	fCantonese	gu3; gwu3 (also)
U+44E2.0	fMandarin	gu4
U+44E2.0	fEnglish	the name of a variety of grass
U+44E3.0	fUTF8	䓣
U+44E3.0	fCNS11643	4-3A5C
U+44E3.0	fGB	3-6922
U+44E3.0	fR/S	 140.8
U+44E3.0	fTotalStrokes	 12; 14
U+44E3.0	fCangjie	TLBO
U+44E3.0	fFourCorner	44220
U+44E3.0	fCantonese	loeng5
U+44E3.0	fMandarin	liang3
U+44E3.0	fEnglish	the name of a variety of grass
U+44E4.0	fUTF8	䓤
U+44E4.0	fCNS11643	4-3A61
U+44E4.0	fR/S	 140.8
U+44E4.0	fTotalStrokes	 12; 14
U+44E4.0	fCangjie	TPHP
U+44E4.0	fFourCorner	44330
U+44E4.0	fCantonese	fat1
U+44E4.0	fMandarin	hu1
U+44E4.0	fEnglish	vegetation
U+44E5.0	fUTF8	䓥
U+44E5.0	fCNS11643	3-3E55
U+44E5.0	fGB	5-3144
U+44E5.0	fR/S	 140.8
U+44E5.0	fTotalStrokes	 12; 14
U+44E5.0	fCangjie	OTAJ
U+44E5.0	fFourCorner	80400
U+44E5.0	fCantonese	laat6
U+44E5.0	fMandarin	la4
U+44E5.0	fEnglish	sound of the wriggled insects on the grass
U+44E6.0	fUTF8	䓦
U+44E6.0	fCNS11643	6-4E27
U+44E6.0	fR/S	 140.8
U+44E6.0	fTotalStrokes	 12; 14
U+44E6.0	fCangjie	TTBC
U+44E6.0	fFourCorner	44800
U+44E6.0	fCantonese	din2
U+44E6.0	fMandarin	dian3
U+44E6.0	fEnglish	same as &#x5178;
U+44E7.0	fUTF8	䓧
U+44E7.0	fCNS11643	3-3E4C
U+44E7.0	fGB	5-6863
U+44E7.0	fR/S	 140.8
U+44E7.0	fTotalStrokes	 12; 14
U+44E7.0	fCangjie	TNEM
U+44E7.0	fFourCorner	44100
U+44E7.0	fCantonese	gik1
U+44E7.0	fMandarin	ji2; ci4
U+44E7.0	fEnglish	same as &#x4E9F;
U+44E8.0	fUTF8	䓨
U+44E8.0	fGB	S-217C
U+44E8.0	fR/S	 140.8
U+44E8.0	fTotalStrokes	 14
U+44E8.0	fCangjie	TBOU
U+44E8.0	fCantonese	aang1; ngaang1 (also); ang1 (also); ngang1 (also)
U+44E8.0	fEnglish	simplified form of &#x7F43;
U+44E9.0	fUTF8	䓩
U+44E9.0	fGB	S-2226
U+44E9.0	fR/S	 140.8
U+44E9.0	fTotalStrokes	 14
U+44E9.0	fCangjie	TDJE
U+44E9.0	fCantonese	zi1
U+44EA.0	fUTF8	䓪
U+44EA.0	fCNS11643	F-3E6B
U+44EA.0	fGB	S-2227
U+44EA.0	fR/S	 140.8
U+44EA.0	fTotalStrokes	 14
U+44EA.0	fCangjie	TAPH
U+44EA.0	fFourCorner	4422
U+44EA.0	fCantonese	ji6
U+44EB.0	fUTF8	䓫
U+44EB.0	fGB	3-6944
U+44EB.0	fR/S	 140.8
U+44EB.0	fTotalStrokes	 14
U+44EB.0	fCangjie	TKMR
U+44EB.0	fFourCorner	4462
U+44EB.0	fCantonese	kei4
U+44EB.0	fMandarin	ji4; qi2
U+44EB.0	fEnglish	name of a mountain in Taiwan
U+44EC.0	fUTF8	䓬
U+44EC.0	fGB	3-694B
U+44EC.0	fR/S	 140.8
U+44EC.0	fTotalStrokes	 14
U+44EC.0	fCangjie	TYAJ
U+44EC.0	fCantonese	coek3
U+44ED.0	fUTF8	䓭
U+44ED.0	fCNS11643	4-4030
U+44ED.0	fGB	3-6956
U+44ED.0	fR/S	 140.8
U+44ED.0	fTotalStrokes	 13; 14; 12
U+44ED.0	fCangjie	TKCN
U+44ED.0	fFourCorner	44920
U+44ED.0	fCantonese	saat3; cat3
U+44ED.0	fMandarin	sha4; cha4
U+44ED.0	fEnglish	weed or grass with narrow thick glades (as bamboo); water-pepper group
U+44EE.0	fUTF8	䓮
U+44EE.0	fCNS11643	4-4032
U+44EE.0	fGB	3-6A2F
U+44EE.0	fR/S	 140.9
U+44EE.0	fTotalStrokes	 13; 15
U+44EE.0	fCangjie	TNHK
U+44EE.0	fFourCorner	44240
U+44EE.0	fHakka	 meu3;  mau3
U+44EE.0	fCantonese	mau6
U+44EE.0	fMandarin	mou4; mao4
U+44EE.0	fEnglish	small bushy plants
U+44EF.0	fUTF8	䓯
U+44EF.0	fCNS11643	4-3A65
U+44EF.0	fGB	3-6964
U+44EF.0	fR/S	 140.9
U+44EF.0	fTotalStrokes	 12; 15; 13
U+44EF.0	fCangjie	TQMY
U+44EF.0	fFourCorner	44500
U+44EF.0	fCantonese	duk6
U+44EF.0	fMandarin	du2
U+44EF.0	fEnglish	water-pepper
U+44F0.0	fUTF8	䓰
U+44F0.0	fCNS11643	3-4437
U+44F0.0	fGB	5-686A
U+44F0.0	fR/S	 140.9
U+44F0.0	fTotalStrokes	 13; 15
U+44F0.0	fCangjie	TMWG
U+44F0.0	fFourCorner	44100
U+44F0.0	fCantonese	jan1; zaan1
U+44F0.0	fMandarin	yin1
U+44F0.0	fEnglish	absinthin, Artemisia capillaris, a kind of medicinal herb
U+44F1.0	fUTF8	䓱
U+44F1.0	fCNS11643	4-4669
U+44F1.0	fR/S	 140.10; 140.9
U+44F1.0	fTotalStrokes	 14; 15
U+44F1.0	fCangjie	TYPD
U+44F1.0	fFourCorner	44900
U+44F1.0	fHakka	 cai2; ciau2; ceu2
U+44F1.0	fCantonese	caai4
U+44F1.0	fMandarin	chai2; zhai4
U+44F1.0	fEnglish	a sudorfic medicine
U+44F2.0	fUTF8	䓲
U+44F2.0	fCNS11643	4-4047
U+44F2.0	fGB	5-687C
U+44F2.0	fR/S	 140.9
U+44F2.0	fTotalStrokes	 13; 15
U+44F2.0	fCangjie	TOCU
U+44F2.0	fFourCorner	44210
U+44F2.0	fCantonese	jeoi6; zeoi6
U+44F2.0	fMandarin	yue4; rui4; sa4
U+44F2.0	fEnglish	fine; silky; new growth of grass
U+44F3.0	fUTF8	䓳
U+44F3.0	fCNS11643	4-4049
U+44F3.0	fGB	3-6977
U+44F3.0	fR/S	 140.9
U+44F3.0	fTotalStrokes	 13; 15
U+44F3.0	fCangjie	THOV
U+44F3.0	fFourCorner	44230
U+44F3.0	fCantonese	han2; ging3; haan2
U+44F3.0	fMandarin	hen3
U+44F3.0	fEnglish	the name of a variety of grass
U+44F4.0	fUTF8	䓴
U+44F4.0	fCNS11643	4-403A
U+44F4.0	fGB	3-696B
U+44F4.0	fR/S	 140.9
U+44F4.0	fTotalStrokes	 13; 15
U+44F4.0	fCangjie	TMBK
U+44F4.0	fFourCorner	44430
U+44F4.0	fCantonese	jyun5; zi4; zyun5
U+44F4.0	fMandarin	ruan3; ruan2
U+44F4.0	fEnglish	edible fungus from trees
U+44F5.0	fUTF8	䓵
U+44F5.0	fCNS11643	3-442B
U+44F5.0	fR/S	 140.9
U+44F5.0	fTotalStrokes	 13; 15
U+44F5.0	fCangjie	TRRR
U+44F5.0	fFourCorner	44660
U+44F5.0	fCantonese	fu1
U+44F5.0	fMandarin	fu1
U+44F5.0	fEnglish	blossom; flower
U+44F6.0	fUTF8	䓶
U+44F6.0	fCNS11643	3-4429
U+44F6.0	fGB	5-6869
U+44F6.0	fR/S	 140.9
U+44F6.0	fTotalStrokes	 13; 15
U+44F6.0	fCangjie	TDLN
U+44F6.0	fFourCorner	44920
U+44F6.0	fCantonese	laai6
U+44F6.0	fMandarin	la4; lai4
U+44F6.0	fEnglish	same as &#x85FE;
U+44F7.0	fUTF8	䓷
U+44F7.0	fCNS11643	3-4434
U+44F7.0	fR/S	 140.9
U+44F7.0	fTotalStrokes	 13; 15
U+44F7.0	fCangjie	TEHN
U+44F7.0	fFourCorner	44120
U+44F7.0	fCantonese	hang6
U+44F7.0	fMandarin	xing4
U+44F7.0	fEnglish	same as &#x8395;, &#x8347;
U+44F8.0	fUTF8	䓸
U+44F8.0	fCNS11643	4-4048
U+44F8.0	fGB	3-6A2C
U+44F8.0	fR/S	 140.9
U+44F8.0	fTotalStrokes	 13; 15
U+44F8.0	fCangjie	TVVJ
U+44F8.0	fFourCorner	44440
U+44F8.0	fCantonese	gaan1; gan1; hek3; zin6
U+44F8.0	fMandarin	jian1
U+44F8.0	fEnglish	weed
U+44F9.0	fUTF8	䓹
U+44F9.0	fCNS11643	3-443D
U+44F9.0	fR/S	 140.9
U+44F9.0	fTotalStrokes	 13; 15
U+44F9.0	fCangjie	TOIV
U+44F9.0	fFourCorner	44730
U+44F9.0	fCantonese	ji3; zi3
U+44F9.0	fMandarin	yi4
U+44F9.0	fEnglish	same as &#x9950;
U+44FA.0	fUTF8	䓺
U+44FA.0	fCNS11643	4-404C
U+44FA.0	fGB	5-692C
U+44FA.0	fR/S	 140.9
U+44FA.0	fTotalStrokes	 13; 15
U+44FA.0	fCangjie	TTGK
U+44FA.0	fFourCorner	44430
U+44FA.0	fCantonese	mei5
U+44FA.0	fMandarin	mi2; mei3; wei4
U+44FA.0	fEnglish	the name of a variety of grass
U+44FB.0	fUTF8	䓻
U+44FB.0	fGB	S-222B
U+44FB.0	fR/S	 140.9
U+44FB.0	fTotalStrokes	 13; 15
U+44FB.0	fCangjie	TBMV
U+44FB.0	fFourCorner	4421
U+44FB.0	fHakka	 k'ieu5; keu5
U+44FB.0	fCantonese	kau3
U+44FB.0	fEnglish	cardamom seeds
U+44FC.0	fUTF8	䓼
U+44FC.0	fCNS11643	4-4665
U+44FC.0	fGB	3-6A4E
U+44FC.0	fR/S	 140.10
U+44FC.0	fTotalStrokes	 14; 16
U+44FC.0	fCangjie	TVIH
U+44FC.0	fFourCorner	44410
U+44FC.0	fCantonese	mong4
U+44FC.0	fMandarin	mang2; he4
U+44FC.0	fEnglish	name of a variety of grass, grunting of hogs
U+44FD.0	fUTF8	䓽
U+44FD.0	fCNS11643	3-492F
U+44FD.0	fR/S	 140.10
U+44FD.0	fTotalStrokes	 14; 16
U+44FD.0	fCangjie	TYRU
U+44FD.0	fFourCorner	44610
U+44FD.0	fCantonese	gei3
U+44FD.0	fMandarin	ji4
U+44FD.0	fEnglish	the name of a variety of grass
U+44FE.0	fUTF8	䓾
U+44FE.0	fCNS11643	4-4657
U+44FE.0	fGB	3-6A48
U+44FE.0	fR/S	 140.10
U+44FE.0	fTotalStrokes	 14; 16
U+44FE.0	fCangjie	TEHV
U+44FE.0	fFourCorner	44400
U+44FE.0	fHakka	  so1
U+44FE.0	fCantonese	so1
U+44FE.0	fMandarin	suo1
U+44FE.0	fEnglish	abundant vegetation
U+44FF.0	fUTF8	䓿
U+44FF.0	fCNS11643	6-6144
U+44FF.0	fR/S	 140.10
U+44FF.0	fTotalStrokes	 14; 16
U+44FF.0	fCangjie	TNIW
U+44FF.0	fFourCorner	44600
U+44FF.0	fCantonese	ham6
U+44FF.0	fMandarin	han4; han3
U+44FF.0	fEnglish	a second name for lotus flower
U+4500.0	fUTF8	䔀
U+4500.0	fCNS11643	4-4664
U+4500.0	fR/S	 140.10
U+4500.0	fTotalStrokes	 14; 16
U+4500.0	fCangjie	TOHV
U+4500.0	fFourCorner	44240
U+4500.0	fCantonese	seoi1; zeon3
U+4500.0	fMandarin	rui2
U+4500.0	fEnglish	same as &#x837D;, &#x8470;
U+4501.0	fUTF8	䔁
U+4501.0	fCNS11643	3-4931
U+4501.0	fR/S	 140.10
U+4501.0	fTotalStrokes	 14; 16
U+4501.0	fCangjie	TMWD
U+4501.0	fFourCorner	44900
U+4501.0	fCantonese	leot6
U+4501.0	fMandarin	li4
U+4501.0	fEnglish	the name of a variety of grass
U+4502.0	fUTF8	䔂
U+4502.0	fCNS11643	4-4653
U+4502.0	fGB	3-6A4A
U+4502.0	fR/S	 140.10
U+4502.0	fTotalStrokes	 14; 16
U+4502.0	fCangjie	TJYJ
U+4502.0	fFourCorner	44400
U+4502.0	fCantonese	zi2; zoi2
U+4502.0	fMandarin	zai3; zi3
U+4502.0	fEnglish	[1] vegetable soup [2] a kind of vegetable
U+4503.0	fUTF8	䔃
U+4503.0	fCNS11643	5-493C
U+4503.0	fR/S	 140.10; 140.9
U+4503.0	fTotalStrokes	 14; 16; 13
U+4503.0	fCangjie	TIFM
U+4503.0	fFourCorner	44210
U+4503.0	fCantonese	zou2
U+4503.0	fMandarin	zu3
U+4503.0	fEnglish	cordate houttuynia (Houttuynia cordata)
U+4504.0	fUTF8	䔄
U+4504.0	fCNS11643	4-4661
U+4504.0	fGB	3-6A3E
U+4504.0	fR/S	 140.10
U+4504.0	fTotalStrokes	 14; 16
U+4504.0	fCangjie	TBOU
U+4504.0	fFourCorner	44770
U+4504.0	fCantonese	jiu4; ziu4
U+4504.0	fMandarin	yao4; yao2
U+4504.0	fEnglish	[1] the name of a variety of grass; rushes [2] dodder
U+4505.0	fUTF8	䔅
U+4505.0	fCNS11643	3-4930
U+4505.0	fR/S	 140.10
U+4505.0	fTotalStrokes	 14; 16
U+4505.0	fCangjie	TMRR
U+4505.0	fFourCorner	44620
U+4505.0	fCantonese	go1; geoi1
U+4505.0	fMandarin	ge1
U+4505.0	fEnglish	[1] the name of a variety of trees [2] the name of a variety of grass [3] fruit of the tree
U+4506.0	fUTF8	䔆
U+4506.0	fCNS11643	4-466D
U+4506.0	fR/S	 140.10
U+4506.0	fTotalStrokes	 14; 16
U+4506.0	fCangjie	TKHG
U+4506.0	fFourCorner	44210
U+4506.0	fCantonese	lei4
U+4506.0	fMandarin	li2
U+4506.0	fEnglish	a kind of bean; edible
U+4507.0	fUTF8	䔇
U+4507.0	fCNS11643	4-4660
U+4507.0	fGB	3-686C
U+4507.0	fR/S	 140.10
U+4507.0	fTotalStrokes	 14; 16
U+4507.0	fCangjie	TUMT
U+4507.0	fFourCorner	44100
U+4507.0	fCantonese	hei2; oi2; ngoi2 (also)
U+4507.0	fMandarin	qi3; yi3
U+4507.0	fEnglish	a kind of plant, the root is used for food
U+4508.0	fUTF8	䔈
U+4508.0	fCNS11643	4-465C
U+4508.0	fR/S	 140.10
U+4508.0	fTotalStrokes	 14; 16
U+4508.0	fCangjie	TMBC
U+4508.0	fFourCorner	44800
U+4508.0	fCantonese	gung3; waa1
U+4508.0	fMandarin	gong4; gan3; nou3
U+4508.0	fEnglish	lush growth of grass, trees, and fruits
U+4509.0	fUTF8	䔉
U+4509.0	fCNS11643	3-4934
U+4509.0	fGB	5-6941
U+4509.0	fR/S	 140.10
U+4509.0	fTotalStrokes	 14; 16
U+4509.0	fCangjie	THDD
U+4509.0	fFourCorner	44990
U+4509.0	fHakka	 son5
U+4509.0	fCantonese	lik6; syun3
U+4509.0	fMandarin	li4
U+4509.0	fEnglish	[1] scattered or dispersed of grass and trees [2] non-classical form of &#x849C;
U+450A.0	fUTF8	䔊
U+450A.0	fCNS11643	5-493B
U+450A.0	fR/S	 140.10
U+450A.0	fTotalStrokes	 14; 16
U+450A.0	fCangjie	TMGT
U+450A.0	fFourCorner	44140
U+450A.0	fCantonese	bang1
U+450A.0	fMandarin	beng1
U+450A.0	fEnglish	dense growth of grass
U+450B.0	fUTF8	䔋
U+450B.0	fCNS11643	6-6158
U+450B.0	fR/S	 140.10
U+450B.0	fTotalStrokes	 14; 16
U+450B.0	fCangjie	TVFH
U+450B.0	fFourCorner	44920
U+450B.0	fCantonese	so1
U+450B.0	fMandarin	sha1
U+450B.0	fEnglish	a kind of tree; very tall and its lieaves like bird's wings; edible
U+450C.0	fUTF8	䔌
U+450C.0	fCNS11643	3-4937
U+450C.0	fGB	5-6942
U+450C.0	fR/S	 140.10
U+450C.0	fTotalStrokes	 14; 16
U+450C.0	fCangjie	THAJ
U+450C.0	fFourCorner	44403
U+450C.0	fCantonese	gou1
U+450C.0	fMandarin	gao1
U+450C.0	fEnglish	name of a variety of grass, (Pueraria thunbergiana); the white birch
U+450D.0	fUTF8	䔍
U+450D.0	fCNS11643	3-4940
U+450D.0	fR/S	 140.10
U+450D.0	fTotalStrokes	 14; 16
U+450D.0	fCangjie	TSQF
U+450D.0	fFourCorner	44320
U+450D.0	fHakka	 duk7
U+450D.0	fMandarin	du3
U+450E.0	fUTF8	䔎
U+450E.0	fCNS11643	3-4E43
U+450E.0	fR/S	 140.11
U+450E.0	fTotalStrokes	 15; 17
U+450E.0	fCangjie	TYDL
U+450E.0	fFourCorner	44300
U+450E.0	fCantonese	suk1; cuk1
U+450E.0	fMandarin	su4
U+450E.0	fEnglish	the name of a variety of grass
U+450F.0	fUTF8	䔏
U+450F.0	fCNS11643	4-4C72
U+450F.0	fGB	3-6A37
U+450F.0	fR/S	 140.11
U+450F.0	fTotalStrokes	 15; 17
U+450F.0	fCangjie	TYHR
U+450F.0	fFourCorner	44300
U+450F.0	fCantonese	cau3; cou3; zou6; zung1
U+450F.0	fMandarin	chou4
U+450F.0	fEnglish	[1] lush, dense growth of weeds [2] mixed; mingled roots of the grass [3] same as &#x7C09;
U+4510.0	fUTF8	䔐
U+4510.0	fCNS11643	3-4E3F
U+4510.0	fR/S	 140.11
U+4510.0	fTotalStrokes	 15; 17
U+4510.0	fCangjie	TEII
U+4510.0	fFourCorner	44150
U+4510.0	fCantonese	zin1
U+4510.0	fMandarin	jian1
U+4510.0	fEnglish	delicate, colorful and beautiful small piece of paper
U+4511.0	fUTF8	䔑
U+4511.0	fCNS11643	4-4C70
U+4511.0	fGB	5-6961
U+4511.0	fR/S	 140.11
U+4511.0	fTotalStrokes	 15; 17
U+4511.0	fCangjie	TODJ
U+4511.0	fFourCorner	44940
U+4511.0	fHakka	  sia2
U+4511.0	fCantonese	ce4
U+4511.0	fMandarin	xie2; ye2
U+4511.0	fEnglish	a plant having leaves transversely veined
U+4512.0	fUTF8	䔒
U+4512.0	fCNS11643	4-4D24
U+4512.0	fGB	5-6953
U+4512.0	fR/S	 140.11
U+4512.0	fTotalStrokes	 15; 17
U+4512.0	fCangjie	TNLR
U+4512.0	fFourCorner	44260
U+4512.0	fCantonese	pui5; bun1; bun6
U+4512.0	fMandarin	bei4
U+4512.0	fEnglish	the name of a variety of grass
U+4513.0	fUTF8	䔓
U+4513.0	fCNS11643	4-4C6B
U+4513.0	fR/S	 140.11
U+4513.0	fTotalStrokes	 15; 17
U+4513.0	fCangjie	TYRJ
U+4513.0	fFourCorner	44640
U+4513.0	fCantonese	heoi2
U+4513.0	fMandarin	xu3
U+4513.0	fEnglish	herb medicine; teasel root
U+4514.0	fUTF8	䔔
U+4514.0	fCNS11643	3-4E41
U+4514.0	fR/S	 140.11
U+4514.0	fTotalStrokes	 15; 17
U+4514.0	fCangjie	TYTU
U+4514.0	fFourCorner	44210
U+4514.0	fCantonese	ging3; ging6
U+4514.0	fMandarin	jing4; qian3; qiu2; ying3
U+4514.0	fEnglish	the name of a variety of grass
U+4515.0	fUTF8	䔕
U+4515.0	fCNS11643	4-4C71
U+4515.0	fGB	3-6A60
U+4515.0	fR/S	 140.11
U+4515.0	fTotalStrokes	 15; 17
U+4515.0	fCangjie	TBIB
U+4515.0	fFourCorner	44220
U+4515.0	fHakka	  pu2
U+4515.0	fCantonese	pou4
U+4515.0	fMandarin	pu2
U+4515.0	fEnglish	the breast of a bird
U+4516.0	fUTF8	䔖
U+4516.0	fCNS11643	3-4E53
U+4516.0	fR/S	 140.11
U+4516.0	fTotalStrokes	 15; 17
U+4516.0	fCangjie	TNLE
U+4516.0	fFourCorner	44240
U+4516.0	fCantonese	ling4
U+4516.0	fMandarin	ling2
U+4516.0	fEnglish	same as &#x83F1;, &#x8506;
U+4517.0	fUTF8	䔗
U+4517.0	fCNS11643	4-4C6D
U+4517.0	fGB	3-6A4B
U+4517.0	fR/S	 140.11; 140.10
U+4517.0	fTotalStrokes	 15; 17; 14
U+4517.0	fCangjie	TIFQ
U+4517.0	fFourCorner	44250
U+4517.0	fCantonese	coeng4
U+4517.0	fMandarin	xiang2
U+4517.0	fEnglish	a kind of vegetable
U+4518.0	fUTF8	䔘
U+4518.0	fCNS11643	4-4C73
U+4518.0	fGB	3-6A5E
U+4518.0	fR/S	 140.11
U+4518.0	fTotalStrokes	 15; 17
U+4518.0	fCangjie	TOUM
U+4518.0	fFourCorner	44710
U+4518.0	fCantonese	zou6
U+4518.0	fMandarin	zuo4
U+4518.0	fEnglish	fish paste; fish cooked in soy sauce
U+4519.0	fUTF8	䔙
U+4519.0	fCNS11643	3-4E47
U+4519.0	fR/S	 140.11
U+4519.0	fTotalStrokes	 15; 17
U+4519.0	fCangjie	TCPI
U+4519.0	fFourCorner	44120
U+4519.0	fCantonese	diu3; coek3
U+4519.0	fMandarin	diao4
U+4519.0	fEnglish	the name of a variety of grass
U+451A.0	fUTF8	䔚
U+451A.0	fCNS11643	3-4E45
U+451A.0	fGB	5-695B
U+451A.0	fR/S	 140.11
U+451A.0	fTotalStrokes	 15; 17
U+451A.0	fCangjie	TMVB
U+451A.0	fFourCorner	44220
U+451A.0	fCantonese	seon4
U+451A.0	fMandarin	chun2
U+451A.0	fEnglish	herb medicine; the water plantain
U+451B.0	fUTF8	䔛
U+451B.0	fCNS11643	3-4E56
U+451B.0	fGB	5-6847
U+451B.0	fR/S	 140.11
U+451B.0	fTotalStrokes	 15; 17
U+451B.0	fCangjie	TPMC
U+451B.0	fFourCorner	44780
U+451B.0	fHakka	 kwen3
U+451B.0	fCantonese	king2; gwing2 (also); loek6
U+451B.0	fMandarin	qiong3; qing3
U+451B.0	fEnglish	a plant from the fibres of which cloth is made
U+451C.0	fUTF8	䔜
U+451C.0	fCNS11643	4-4C7D
U+451C.0	fGB	3-6A52
U+451C.0	fR/S	 140.11
U+451C.0	fTotalStrokes	 15; 17
U+451C.0	fCangjie	TSJB
U+451C.0	fFourCorner	44450
U+451C.0	fCantonese	naam4; laam4 (also)
U+451C.0	fMandarin	tan1; nan2
U+451C.0	fEnglish	herb medicine; hellebore
U+451D.0	fUTF8	䔝
U+451D.0	fCNS11643	3-5251
U+451D.0	fR/S	 140.11
U+451D.0	fTotalStrokes	 16; 17
U+451D.0	fCangjie	TYPF
U+451D.0	fFourCorner	44900
U+451D.0	fCantonese	zaai1
U+451D.0	fEnglish	the name of a variety of grass
U+451E.0	fUTF8	䔞
U+451E.0	fCNS11643	4-4C6C
U+451E.0	fGB	3-6A65
U+451E.0	fR/S	 140.11
U+451E.0	fTotalStrokes	 15; 17
U+451E.0	fCangjie	TYIJ
U+451E.0	fFourCorner	44400
U+451E.0	fHakka	 lut8
U+451E.0	fCantonese	leot6; leoi6
U+451E.0	fMandarin	luu4; l4
U+451E.0	fEnglish	seeds beginning to sprout
U+451F.0	fUTF8	䔟
U+451F.0	fCNS11643	3-4E4A
U+451F.0	fGB	5-6960
U+451F.0	fR/S	 140.11
U+451F.0	fTotalStrokes	 15; 17
U+451F.0	fCangjie	THDN
U+451F.0	fFourCorner	44920
U+451F.0	fCantonese	ji4; zyu4
U+451F.0	fMandarin	yi2; chi2; chi3
U+451F.0	fEnglish	shake and wave of the grass
U+4520.0	fUTF8	䔠
U+4520.0	fCNS11643	4-4C6E
U+4520.0	fGB	3-6A53
U+4520.0	fR/S	 140.11
U+4520.0	fTotalStrokes	 15; 17
U+4520.0	fCangjie	TDFB
U+4520.0	fFourCorner	44920
U+4520.0	fHakka	 sau1; sau3
U+4520.0	fCantonese	saau2; sau1
U+4520.0	fMandarin	shao3
U+4520.0	fEnglish	the small rootlets of the water-lily
U+4521.0	fUTF8	䔡
U+4521.0	fCNS11643	3-4E48
U+4521.0	fR/S	 140.11
U+4521.0	fTotalStrokes	 15; 17
U+4521.0	fCangjie	TNWF
U+4521.0	fFourCorner	44330
U+4521.0	fCantonese	jyu4; zyu4
U+4521.0	fMandarin	yu2
U+4521.0	fEnglish	the name of a variety of grass; Perilla ocimoides, whose seeds are a bird feed
U+4522.0	fUTF8	䔢
U+4522.0	fCNS11643	3-4E55
U+4522.0	fR/S	 140.11
U+4522.0	fTotalStrokes	 15; 17
U+4522.0	fCangjie	TOOS
U+4522.0	fFourCorner	44720
U+4522.0	fHakka	 fa2;  fa1
U+4522.0	fCantonese	waa4
U+4522.0	fMandarin	hua2; hua4
U+4522.0	fEnglish	[1] glory; splendour [2] flowery [3] China
U+4523.0	fUTF8	䔣
U+4523.0	fCNS11643	3-4E49
U+4523.0	fGB	5-695F
U+4523.0	fR/S	 140.11
U+4523.0	fTotalStrokes	 15; 17
U+4523.0	fCangjie	THNQ
U+4523.0	fFourCorner	44500
U+4523.0	fCantonese	lai4
U+4523.0	fMandarin	li2
U+4523.0	fEnglish	a tribe in ancient times
U+4524.0	fUTF8	䔤
U+4524.0	fCNS11643	5-5046
U+4524.0	fR/S	 140.11
U+4524.0	fTotalStrokes	 15; 17
U+4524.0	fCangjie	THAA
U+4524.0	fFourCorner	44660
U+4524.0	fCantonese	paa1
U+4524.0	fMandarin	pa1
U+4524.0	fEnglish	the lush and luxuriant of the grass and flowers
U+4525.0	fUTF8	䔥
U+4525.0	fCNS11643	3-4E50
U+4525.0	fR/S	 140.11
U+4525.0	fTotalStrokes	 15; 17
U+4525.0	fCangjie	TLLL
U+4525.0	fFourCorner	44220
U+4525.0	fCantonese	siu1
U+4525.0	fMandarin	xiao1
U+4525.0	fEnglish	non-classical form of &#x856D;
U+4526.0	fUTF8	䔦
U+4526.0	fCNS11643	F-5165
U+4526.0	fGB	S-2179
U+4526.0	fR/S	 140.11
U+4526.0	fTotalStrokes	 17
U+4526.0	fCangjie	TDOY
U+4526.0	fCantonese	mui4
U+4527.0	fUTF8	䔧
U+4527.0	fCNS11643	F-5173
U+4527.0	fGB	S-2178
U+4527.0	fR/S	 140.11
U+4527.0	fTotalStrokes	 17
U+4527.0	fCangjie	THND
U+4527.0	fFourCorner	4490
U+4527.0	fCantonese	lei4; laai4
U+4527.0	fMandarin	li2; li4
U+4527.0	fEnglish	same as &#x853E;
U+4528.0	fUTF8	䔨
U+4528.0	fCNS11643	F-5176
U+4528.0	fGB	S-222C
U+4528.0	fR/S	 140.11
U+4528.0	fTotalStrokes	 17
U+4528.0	fCangjie	TVHG
U+4528.0	fCantonese	ung3; ngung3 (also)
U+4529.0	fUTF8	䔩
U+4529.0	fCNS11643	4-4D21
U+4529.0	fR/S	 140.11
U+4529.0	fTotalStrokes	 15; 17
U+4529.0	fCangjie	TDLK
U+4529.0	fFourCorner	44940
U+4529.0	fCantonese	cik1
U+452A.0	fUTF8	䔪
U+452A.0	fCNS11643	4-4C6F
U+452A.0	fGB	3-6A56
U+452A.0	fR/S	 140.11
U+452A.0	fTotalStrokes	 15; 17
U+452A.0	fCangjie	TKKK
U+452A.0	fFourCorner	44430
U+452A.0	fCantonese	song2
U+452A.0	fMandarin	dui4; shuang3
U+452A.0	fEnglish	[1] the name of a variety of grass [2] scattered or dispersed of the grass and trees
U+452B.0	fUTF8	䔫
U+452B.0	fCNS11643	3-4E51
U+452B.0	fR/S	 140.11; 140.12
U+452B.0	fTotalStrokes	 15; 17; 16
U+452B.0	fCangjie	TNMB
U+452B.0	fFourCorner	44190
U+452B.0	fCantonese	so1
U+452C.0	fUTF8	䔬
U+452C.0	fCNS11643	4-4C77
U+452C.0	fGB	3-6A59
U+452C.0	fR/S	 140.11
U+452C.0	fTotalStrokes	 15; 17
U+452C.0	fCangjie	TWTC
U+452C.0	fFourCorner	44800
U+452C.0	fCantonese	ji6; zi6
U+452C.0	fMandarin	yi4
U+452C.0	fEnglish	[1] hemp [2] marijuana (female; pistillate) [3] herbage
U+452D.0	fUTF8	䔭
U+452D.0	fCNS11643	4-5277
U+452D.0	fGB	3-6A7E
U+452D.0	fR/S	 140.12
U+452D.0	fTotalStrokes	 16; 18
U+452D.0	fCangjie	TJPB
U+452D.0	fFourCorner	44220
U+452D.0	fCantonese	ning5; ling5 (also); ning6
U+452D.0	fMandarin	ning3; ning4; zhou3
U+452D.0	fEnglish	the name of a variety of grass; poisonous herb
U+452E.0	fUTF8	䔮
U+452E.0	fCNS11643	4-527B
U+452E.0	fGB	3-6A6B
U+452E.0	fR/S	 140.12
U+452E.0	fTotalStrokes	 16; 18
U+452E.0	fCangjie	TTCL
U+452E.0	fFourCorner	44820
U+452E.0	fCantonese	si1
U+452E.0	fMandarin	si1
U+452E.0	fEnglish	the name of a variety of grass, growing in the water with edible flowers
U+452F.0	fUTF8	䔯
U+452F.0	fCNS11643	4-5279
U+452F.0	fGB	3-6A6D
U+452F.0	fR/S	 140.12
U+452F.0	fTotalStrokes	 16; 18
U+452F.0	fCangjie	TMWR
U+452F.0	fFourCorner	44660
U+452F.0	fHakka	 ku5; ku3
U+452F.0	fCantonese	fu3
U+452F.0	fMandarin	ku4; hu2; hua4
U+452F.0	fEnglish	stalk and flower of the onion and garlic
U+4530.0	fUTF8	䔰
U+4530.0	fCNS11643	4-532C
U+4530.0	fGB	5-6A2A
U+4530.0	fR/S	 140.12
U+4530.0	fTotalStrokes	 16; 18
U+4530.0	fCangjie	TJMW
U+4530.0	fFourCorner	44600
U+4530.0	fHakka	 fu5
U+4530.0	fCantonese	fu3; fu1; pei3
U+4530.0	fMandarin	mu1; fu2; fu4
U+4530.0	fEnglish	variant of &#x844D;
U+4531.0	fUTF8	䔱
U+4531.0	fCNS11643	4-5321
U+4531.0	fGB	3-6A70
U+4531.0	fR/S	 140.12
U+4531.0	fTotalStrokes	 16; 18
U+4531.0	fCangjie	TQRJ
U+4531.0	fFourCorner	44540
U+4531.0	fCantonese	jap1; zaap1
U+4531.0	fMandarin	yi1
U+4531.0	fEnglish	the name of a variety of grass
U+4532.0	fUTF8	䔲
U+4532.0	fCNS11643	4-5276
U+4532.0	fGB	3-6B25
U+4532.0	fR/S	 140.12
U+4532.0	fTotalStrokes	 16; 18
U+4532.0	fCangjie	TNOT
U+4532.0	fFourCorner	44100
U+4532.0	fHakka	 den1
U+4532.0	fCantonese	dang1
U+4532.0	fMandarin	deng1; cheng2; zhuo2
U+4532.0	fEnglish	a plant not yet identified
U+4533.0	fUTF8	䔳
U+4533.0	fCNS11643	3-5253
U+4533.0	fGB	S-222D
U+4533.0	fR/S	 140.12
U+4533.0	fTotalStrokes	 16; 18
U+4533.0	fCangjie	TBKF
U+4533.0	fFourCorner	44330
U+4533.0	fCantonese	jin4; gin1; zin4
U+4533.0	fMandarin	ran2; nan3
U+4533.0	fEnglish	chickling
U+4534.0	fUTF8	䔴
U+4534.0	fCNS11643	4-5325
U+4534.0	fGB	3-6B23
U+4534.0	fR/S	 140.12; 140.13
U+4534.0	fTotalStrokes	 16; 18; 17
U+4534.0	fCangjie	TUFE
U+4534.0	fFourCorner	44940
U+4534.0	fCantonese	cak1
U+4534.0	fMandarin	ce4; cui4
U+4534.0	fEnglish	name of a variety of grass, growing grass, to weed (in farming); to mow grass or cut weed
U+4535.0	fUTF8	䔵
U+4535.0	fCNS11643	3-5257
U+4535.0	fR/S	 140.12
U+4535.0	fTotalStrokes	 16; 18
U+4535.0	fCangjie	TAND
U+4535.0	fFourCorner	44220
U+4535.0	fCantonese	gaan1
U+4535.0	fMandarin	jian1
U+4535.0	fEnglish	same as &#x844C;, &#x8551;
U+4536.0	fUTF8	䔶
U+4536.0	fCNS11643	4-527E
U+4536.0	fGB	3-6A6F
U+4536.0	fR/S	 140.12
U+4536.0	fTotalStrokes	 16; 18
U+4536.0	fCangjie	TQAO
U+4536.0	fFourCorner	44580
U+4536.0	fCantonese	tai4
U+4536.0	fMandarin	ti2
U+4536.0	fEnglish	[1] the name of a variety of grass [2] weed (in farming); mow grass or cut weed
U+4537.0	fUTF8	䔷
U+4537.0	fCNS11643	3-524D
U+4537.0	fGB	5-696F
U+4537.0	fR/S	 140.12
U+4537.0	fTotalStrokes	 16; 18
U+4537.0	fCangjie	TMGN
U+4537.0	fFourCorner	44200
U+4537.0	fCantonese	kam4
U+4537.0	fMandarin	qin2
U+4537.0	fEnglish	Cyperus rotundus, a kind of medicinal herb
U+4538.0	fUTF8	䔸
U+4538.0	fCNS11643	4-5323
U+4538.0	fGB	3-6A6E
U+4538.0	fR/S	 140.12
U+4538.0	fTotalStrokes	 16; 18
U+4538.0	fCangjie	TIKK
U+4538.0	fFourCorner	44430
U+4538.0	fCantonese	biu2; biu1
U+4538.0	fMandarin	biao3
U+4538.0	fEnglish	[1] name of a variety of grass [2] fragrant herb; vanilla [3] fragrant; delicious; balm; spice
U+4539.0	fUTF8	䔹
U+4539.0	fCNS11643	4-527A
U+4539.0	fGB	3-6849
U+4539.0	fR/S	 140.12
U+4539.0	fTotalStrokes	 16; 18
U+4539.0	fCangjie	TNLO
U+4539.0	fFourCorner	44230
U+4539.0	fCantonese	seoi6
U+4539.0	fMandarin	sui4
U+4539.0	fEnglish	a fungus which grows on rotten plants and is eaten when fresh
U+453A.0	fUTF8	䔺
U+453A.0	fCNS11643	4-527C
U+453A.0	fGB	3-6A68
U+453A.0	fR/S	 140.12
U+453A.0	fTotalStrokes	 16; 18
U+453A.0	fCangjie	TNLB
U+453A.0	fFourCorner	44220
U+453A.0	fCantonese	wai4
U+453A.0	fMandarin	wei3; wei2
U+453A.0	fEnglish	heading; earing; to flower; to blossom (of the smartweed group), Cyperus rotundus, a kind of medicinal herb, a peduncle or footstalk of a flower or fruit; a stem; a base, new growing leaves
U+453B.0	fUTF8	䔻
U+453B.0	fCNS11643	4-5272
U+453B.0	fGB	5-697E
U+453B.0	fR/S	 140.12
U+453B.0	fTotalStrokes	 16; 18
U+453B.0	fCangjie	TYDK
U+453B.0	fFourCorner	44440
U+453B.0	fCantonese	deon1
U+453B.0	fMandarin	dun1
U+453B.0	fEnglish	[1] the name of a variety of grass [2] luxuriant; lush of the grass
U+453C.0	fUTF8	䔼
U+453C.0	fCNS11643	5-642D
U+453C.0	fGB	S-222F
U+453C.0	fR/S	 140.14; 140.12
U+453C.0	fTotalStrokes	 18
U+453C.0	fCangjie	TSIM
U+453C.0	fFourCorner	44110
U+453C.0	fCantonese	sap1; sap3
U+453C.0	fMandarin	se4
U+453C.0	fEnglish	the sound of the grass; parsley
U+453D.0	fUTF8	䔽
U+453D.0	fCNS11643	3-524C
U+453D.0	fGB	5-6A26
U+453D.0	fR/S	 140.12
U+453D.0	fTotalStrokes	 16; 18
U+453D.0	fCangjie	TEAV
U+453D.0	fFourCorner	44120
U+453D.0	fCantonese	oi3; ngoi3 (also)
U+453D.0	fMandarin	ai4
U+453D.0	fEnglish	to cover; to put a cover over something, small, low, weak, pure; clean; clear
U+453E.0	fUTF8	䔾
U+453E.0	fCNS11643	4-5322
U+453E.0	fGB	3-6A71
U+453E.0	fR/S	 140.12
U+453E.0	fTotalStrokes	 16; 18
U+453E.0	fCangjie	TQAV
U+453E.0	fFourCorner	44520
U+453E.0	fCantonese	hei3
U+453E.0	fMandarin	qi4; e4; ze4
U+453E.0	fEnglish	a kind of fragrant herb, a kind of vegetable (growing in the water; something like the bracken)
U+453F.0	fUTF8	䔿
U+453F.0	fCNS11643	4-5273
U+453F.0	fGB	3-6A7B
U+453F.0	fR/S	 140.12
U+453F.0	fTotalStrokes	 16; 18
U+453F.0	fCangjie	TTWI
U+453F.0	fFourCorner	44340
U+453F.0	fCantonese	zyun2; cyun2; cyun3
U+453F.0	fMandarin	shun3; jie4; zun3
U+453F.0	fEnglish	[1] lush growth; dense growth of trees and grass [2] huddle together; crowd together
U+4540.0	fUTF8	䕀
U+4540.0	fCNS11643	3-524E
U+4540.0	fGB	5-6971
U+4540.0	fR/S	 140.12
U+4540.0	fTotalStrokes	 16; 18
U+4540.0	fCangjie	TGFO
U+4540.0	fFourCorner	44980
U+4540.0	fCantonese	fun2; haat6
U+4540.0	fMandarin	kuan3
U+4540.0	fEnglish	the name of a variety of grass, Compositae; Chrysanthemum family, perennial herbage; to blossom in winter
U+4541.0	fUTF8	䕁
U+4541.0	fCNS11643	4-5327
U+4541.0	fGB	3-6A72
U+4541.0	fR/S	 140.12
U+4541.0	fTotalStrokes	 16; 18
U+4541.0	fCangjie	TLYD
U+4541.0	fFourCorner	44900
U+4541.0	fCantonese	fei2
U+4541.0	fMandarin	fei3
U+4541.0	fEnglish	suddenly, name of a variety of grass
U+4542.0	fUTF8	䕂
U+4542.0	fGB	S-217D
U+4542.0	fR/S	 140.12
U+4542.0	fTotalStrokes	 18
U+4542.0	fCangjie	TYIV
U+4542.0	fCantonese	jyun5
U+4543.0	fUTF8	䕃
U+4543.0	fCNS11643	4-5331
U+4543.0	fR/S	 140.12
U+4543.0	fTotalStrokes	 16; 18
U+4543.0	fCangjie	TNLI
U+4543.0	fFourCorner	44230
U+4543.0	fCantonese	jam1; zaam1; zaam3
U+4543.0	fMandarin	yin4
U+4543.0	fEnglish	same as &#x852D;
U+4544.0	fUTF8	䕄
U+4544.0	fCNS11643	4-4667
U+4544.0	fGB	3-6B26
U+4544.0	fR/S	 140.12; 140.10
U+4544.0	fTotalStrokes	 14; 18
U+4544.0	fCangjie	TNET
U+4544.0	fFourCorner	44100
U+4544.0	fCantonese	zing1
U+4544.0	fMandarin	zheng1
U+4544.0	fEnglish	meat in the form of paste; salted or pickled meat
U+4545.0	fUTF8	䕅
U+4545.0	fCNS11643	4-5844
U+4545.0	fGB	3-6B24
U+4545.0	fR/S	 140.13
U+4545.0	fTotalStrokes	 17; 16; 19
U+4545.0	fCangjie	TVHE
U+4545.0	fFourCorner	44440
U+4545.0	fCantonese	sou2
U+4545.0	fMandarin	sao1; sao3
U+4545.0	fEnglish	the name of a variety of grass
U+4546.0	fUTF8	䕆
U+4546.0	fCNS11643	4-583C
U+4546.0	fGB	3-6B29
U+4546.0	fR/S	 140.13
U+4546.0	fTotalStrokes	 17; 16; 19
U+4546.0	fCangjie	TDON
U+4546.0	fFourCorner	44920
U+4546.0	fCantonese	dau6
U+4546.0	fMandarin	dou4
U+4546.0	fEnglish	round straw quilt; bedding; cushion; mattress; mat for the chair, name of a variety of grass
U+4547.0	fUTF8	䕇
U+4547.0	fCNS11643	4-5838
U+4547.0	fGB	3-6A79
U+4547.0	fR/S	 140.13
U+4547.0	fTotalStrokes	 17; 16; 19
U+4547.0	fCangjie	TIHI
U+4547.0	fFourCorner	44210
U+4547.0	fCantonese	wui6
U+4547.0	fMandarin	hui3; hui4
U+4547.0	fEnglish	name of a variety of grass, a kind of vegetable
U+4548.0	fUTF8	䕈
U+4548.0	fCNS11643	4-5842
U+4548.0	fGB	3-6B2F
U+4548.0	fR/S	 140.13
U+4548.0	fTotalStrokes	 17; 16; 19
U+4548.0	fCangjie	TVMD
U+4548.0	fFourCorner	44290
U+4548.0	fCantonese	sip3; dip6; seot1; sit3; sou3
U+4548.0	fMandarin	die2; xie4
U+4548.0	fEnglish	[1] the name of a variety of grass [2] same as &#x5C5F;, &#x5C67;
U+4549.0	fUTF8	䕉
U+4549.0	fCNS11643	5-5D79
U+4549.0	fR/S	 140.13
U+4549.0	fTotalStrokes	 17; 19
U+4549.0	fCangjie	TWLJ
U+4549.0	fFourCorner	44400
U+4549.0	fCantonese	zaak6; zak3; zak6
U+4549.0	fMandarin	zhe2; ze2
U+4549.0	fEnglish	a kind of plant, Pueraria thunbergiana, a creeping, edible bean whose fibers can be made into linen-like cloth and whose roots are used in herbal medicine
U+454A.0	fUTF8	䕊
U+454A.0	fCNS11643	4-5836
U+454A.0	fGB	3-6B34
U+454A.0	fR/S	 140.13
U+454A.0	fTotalStrokes	 17; 16; 19
U+454A.0	fCangjie	TYWM
U+454A.0	fFourCorner	44100
U+454A.0	fCantonese	taan4; sik1; tan4
U+454A.0	fMandarin	tan2
U+454A.0	fEnglish	[1] the name of a variety of grass [2] grass that creep and spread luxuriantly
U+454B.0	fUTF8	䕋
U+454B.0	fCNS11643	4-583B
U+454B.0	fGB	3-6B28
U+454B.0	fR/S	 140.13
U+454B.0	fTotalStrokes	 17; 16; 19
U+454B.0	fCangjie	TGIR
U+454B.0	fFourCorner	44160
U+454B.0	fCantonese	tong4
U+454B.0	fMandarin	tang2; chang3
U+454B.0	fEnglish	of elm group
U+454C.0	fUTF8	䕌
U+454C.0	fCNS11643	3-562E
U+454C.0	fR/S	 140.13
U+454C.0	fTotalStrokes	 17; 19
U+454C.0	fCangjie	THDG
U+454C.0	fFourCorner	44910
U+454C.0	fHakka	 chi1; cih1
U+454C.0	fCantonese	zi6
U+454C.0	fMandarin	zhi4
U+454C.0	fEnglish	same as &#x7A49;, &#x7A1A;
U+454D.0	fUTF8	䕍
U+454D.0	fCNS11643	3-5627
U+454D.0	fR/S	 140.13
U+454D.0	fTotalStrokes	 17; 19
U+454D.0	fCangjie	TYVB
U+454D.0	fFourCorner	44220
U+454D.0	fCantonese	jai6; zaau5; zeoi6
U+454D.0	fMandarin	yi4
U+454D.0	fEnglish	the name of a variety of grass
U+454E.0	fUTF8	䕎
U+454E.0	fCNS11643	4-583D
U+454E.0	fGB	3-6B2C
U+454E.0	fR/S	 140.13
U+454E.0	fTotalStrokes	 17; 16; 19
U+454E.0	fCangjie	TAMW
U+454E.0	fFourCorner	44660
U+454E.0	fCantonese	fuk1
U+454E.0	fMandarin	fu2
U+454E.0	fEnglish	[1] (non-classical form) name of a variety of grass [2] corrupted form of &#x844D;
U+454F.0	fUTF8	䕏
U+454F.0	fCNS11643	3-562A
U+454F.0	fR/S	 140.13
U+454F.0	fTotalStrokes	 17; 19
U+454F.0	fCangjie	TTGI
U+454F.0	fFourCorner	44550
U+454F.0	fCantonese	ngo4
U+454F.0	fMandarin	e2
U+454F.0	fEnglish	Artemisia, a kind of plant with edible leaves
U+4550.0	fUTF8	䕐
U+4550.0	fGB	S-217B
U+4550.0	fR/S	 140.13
U+4550.0	fTotalStrokes	 19
U+4550.0	fCangjie	TIFW
U+4550.0	fCantonese	fuk1
U+4551.0	fUTF8	䕑
U+4551.0	fCNS11643	3-562F
U+4551.0	fR/S	 140.13
U+4551.0	fTotalStrokes	 17; 19
U+4551.0	fCangjie	TVFV
U+4551.0	fFourCorner	44940
U+4551.0	fCantonese	seoi1
U+4551.0	fMandarin	jun4
U+4551.0	fEnglish	same as &#x8470;
U+4552.0	fUTF8	䕒
U+4552.0	fCNS11643	3-5869
U+4552.0	fR/S	 140.14
U+4552.0	fTotalStrokes	 18; 20
U+4552.0	fCangjie	TGRR
U+4552.0	fFourCorner	44460
U+4552.0	fCantonese	gaa1
U+4552.0	fMandarin	jia1
U+4552.0	fEnglish	the name of a variety of grass
U+4553.0	fUTF8	䕓
U+4553.0	fCNS11643	4-5C7E
U+4553.0	fGB	3-6B40
U+4553.0	fR/S	 140.14
U+4553.0	fTotalStrokes	 18; 20
U+4553.0	fCangjie	TJBF
U+4553.0	fFourCorner	44900
U+4553.0	fCantonese	caat3
U+4553.0	fMandarin	cha2; chui4
U+4553.0	fEnglish	[1] the name of a variety of grass; poisonous; used to kill fishes [2] valueless things; insignificant things
U+4554.0	fUTF8	䕔
U+4554.0	fCNS11643	4-5D2B
U+4554.0	fGB	3-6A5F
U+4554.0	fR/S	 140.14
U+4554.0	fTotalStrokes	 18; 20
U+4554.0	fCangjie	THON
U+4554.0	fFourCorner	44220
U+4554.0	fCantonese	haam4
U+4554.0	fMandarin	xian2
U+4554.0	fEnglish	the name of a variety of grass, couch grass family
U+4555.0	fUTF8	䕕
U+4555.0	fCNS11643	3-586D
U+4555.0	fGB	5-6A51
U+4555.0	fR/S	 140.14
U+4555.0	fTotalStrokes	 18; 20
U+4555.0	fCangjie	TEAE
U+4555.0	fFourCorner	44140
U+4555.0	fCantonese	maan6
U+4555.0	fMandarin	man4
U+4555.0	fEnglish	the name of a variety of grass
U+4556.0	fUTF8	䕖
U+4556.0	fCNS11643	3-5870
U+4556.0	fR/S	 140.14
U+4556.0	fTotalStrokes	 18; 20
U+4556.0	fCangjie	TYNF
U+4556.0	fFourCorner	44309
U+4556.0	fCantonese	syun1
U+4556.0	fMandarin	sun1
U+4556.0	fEnglish	same as &#x84C0;
U+4557.0	fUTF8	䕗
U+4557.0	fCNS11643	3-586A
U+4557.0	fR/S	 140.14
U+4557.0	fTotalStrokes	 18; 20
U+4557.0	fCangjie	THUL
U+4557.0	fFourCorner	44221
U+4557.0	fCantonese	bei6
U+4557.0	fMandarin	bi4; pei4
U+4557.0	fEnglish	name of a variety of grass
U+4558.0	fUTF8	䕘
U+4558.0	fCNS11643	3-5871
U+4558.0	fGB	5-6962
U+4558.0	fR/S	 140.14
U+4558.0	fTotalStrokes	 18; 20
U+4558.0	fCangjie	TOIC
U+4558.0	fFourCorner	44380
U+4558.0	fCantonese	ling4
U+4558.0	fMandarin	ling3; ling2
U+4558.0	fEnglish	same as &#x82D3;
U+4559.0	fUTF8	䕙
U+4559.0	fCNS11643	4-5D23
U+4559.0	fGB	3-6B37
U+4559.0	fR/S	 140.14
U+4559.0	fTotalStrokes	 18; 20
U+4559.0	fCangjie	TJIG
U+4559.0	fFourCorner	44250
U+4559.0	fCantonese	zit6
U+4559.0	fMandarin	jie2
U+4559.0	fEnglish	 set in order; adjust and repair
U+455A.0	fUTF8	䕚
U+455A.0	fCNS11643	7-4238
U+455A.0	fR/S	 140.14
U+455A.0	fTotalStrokes	 18; 20
U+455A.0	fCangjie	TSLC
U+455A.0	fFourCorner	44710
U+455A.0	fCantonese	gwai6
U+455A.0	fMandarin	kui4
U+455A.0	fEnglish	same as &#x8562;
U+455B.0	fUTF8	䕛
U+455B.0	fCNS11643	4-5D21
U+455B.0	fGB	3-695E
U+455B.0	fR/S	 140.14
U+455B.0	fTotalStrokes	 18; 20
U+455B.0	fCangjie	TYRO
U+455B.0	fFourCorner	44630
U+455B.0	fCantonese	gaap3
U+455B.0	fMandarin	jia2
U+455B.0	fEnglish	the name of a variety of grass
U+455C.0	fUTF8	䕜
U+455C.0	fCNS11643	F-604E
U+455C.0	fGB	S-2225
U+455C.0	fR/S	 140.14
U+455C.0	fTotalStrokes	 20
U+455C.0	fCangjie	TSMJ
U+455C.0	fFourCorner	4440
U+455C.0	fCantonese	ceoi3
U+455D.0	fUTF8	䕝
U+455D.0	fCNS11643	7-3B32
U+455D.0	fGB	3-6B3A
U+455D.0	fR/S	 140.14
U+455D.0	fTotalStrokes	 18; 17; 20
U+455D.0	fCangjie	THDB
U+455D.0	fFourCorner	44957
U+455D.0	fCantonese	cing1
U+455D.0	fMandarin	cheng1
U+455D.0	fEnglish	medical herb; a second name for sesame
U+455E.0	fUTF8	䕞
U+455E.0	fCNS11643	4-6077
U+455E.0	fGB	5-6949
U+455E.0	fR/S	 140.15
U+455E.0	fTotalStrokes	 19; 21
U+455E.0	fCangjie	TANV
U+455E.0	fFourCorner	44220
U+455E.0	fCantonese	long6
U+455E.0	fMandarin	lang4; liao2
U+455E.0	fEnglish	Henbane, poisonous, seeds for medical use, a plant which produces a brown dye
U+455F.0	fUTF8	䕟
U+455F.0	fCNS11643	3-5C73
U+455F.0	fR/S	 140.15
U+455F.0	fTotalStrokes	 20; 21; 19
U+455F.0	fCangjie	THXC
U+455F.0	fFourCorner	44800
U+455F.0	fCantonese	hing1; guk1
U+455F.0	fMandarin	xing1
U+455F.0	fEnglish	a kind of vegetable; rape
U+4560.0	fUTF8	䕠
U+4560.0	fCNS11643	3-5B23
U+4560.0	fR/S	 140.15
U+4560.0	fTotalStrokes	 19; 21
U+4560.0	fCangjie	TINE
U+4560.0	fFourCorner	44240
U+4560.0	fCantonese	fai3
U+4560.0	fMandarin	fei4
U+4560.0	fEnglish	non-classical form of &#x855F;
U+4561.0	fUTF8	䕡
U+4561.0	fCNS11643	4-6078
U+4561.0	fGB	3-6A21
U+4561.0	fR/S	 140.15
U+4561.0	fTotalStrokes	 19; 21
U+4561.0	fCangjie	TANR
U+4561.0	fFourCorner	44220
U+4561.0	fHakka	 li1; li2
U+4561.0	fCantonese	leoi4
U+4561.0	fMandarin	luu2; lu3; l2
U+4561.0	fEnglish	a plant of the Artemisia kind
U+4562.0	fUTF8	䕢
U+4562.0	fCNS11643	3-5B21
U+4562.0	fGB	5-6A44
U+4562.0	fR/S	 140.15
U+4562.0	fTotalStrokes	 19; 21
U+4562.0	fCangjie	TMRM
U+4562.0	fFourCorner	44610
U+4562.0	fHakka	 za5; cha5
U+4562.0	fCantonese	zaa2
U+4562.0	fMandarin	zha3
U+4562.0	fEnglish	careless about; in disorder
U+4563.0	fUTF8	䕣
U+4563.0	fGB	5-6A54
U+4563.0	fR/S	 140.15
U+4563.0	fTotalStrokes	 21
U+4563.0	fCangjie	TLAV
U+4563.0	fFourCorner	4422
U+4563.0	fCantonese	hot6; haap6; hot3
U+4563.0	fMandarin	he2; ke3; she2
U+4563.0	fEnglish	water plant
U+4564.0	fUTF8	䕤
U+4564.0	fCNS11643	3-5B24
U+4564.0	fGB	5-694B
U+4564.0	fR/S	 140.15; 140.16; 163.16
U+4564.0	fTotalStrokes	 19; 21
U+4564.0	fCangjie	TRJL
U+4564.0	fFourCorner	44520
U+4564.0	fCantonese	kei4
U+4564.0	fMandarin	qi2
U+4564.0	fEnglish	same as &#x8604;
U+4565.0	fUTF8	䕥
U+4565.0	fCNS11643	7-4953
U+4565.0	fR/S	 140.15
U+4565.0	fTotalStrokes	 19; 21
U+4565.0	fCangjie	TCFB
U+4565.0	fFourCorner	44220
U+4565.0	fCantonese	nei5; lei5 (also)
U+4565.0	fMandarin	ni3; yi2
U+4565.0	fEnglish	ancient form of &#x82E8;
U+4566.0	fUTF8	䕦
U+4566.0	fCNS11643	4-642F
U+4566.0	fR/S	 140.16
U+4566.0	fTotalStrokes	 20; 22
U+4566.0	fCangjie	TYNV
U+4566.0	fFourCorner	44210
U+4566.0	fCantonese	jing4; zing4
U+4566.0	fMandarin	ying2
U+4566.0	fEnglish	flower of the chrysanthemum; chrysanthemum
U+4567.0	fUTF8	䕧
U+4567.0	fCNS11643	4-6434
U+4567.0	fGB	3-6B46
U+4567.0	fR/S	 140.16
U+4567.0	fTotalStrokes	 20; 22
U+4567.0	fCangjie	TBBK
U+4567.0	fFourCorner	44240
U+4567.0	fCantonese	gaau1
U+4567.0	fMandarin	xiao2; xiao4
U+4567.0	fEnglish	same as &#x832D;
U+4568.0	fUTF8	䕨
U+4568.0	fCNS11643	3-5C7D
U+4568.0	fGB	5-6A61
U+4568.0	fR/S	 140.16
U+4568.0	fTotalStrokes	 20; 22
U+4568.0	fCangjie	TBFF
U+4568.0	fFourCorner	44290
U+4568.0	fCantonese	tang4
U+4568.0	fMandarin	teng2
U+4568.0	fEnglish	same as &#x85E4;
U+4569.0	fUTF8	䕩
U+4569.0	fCNS11643	4-6433
U+4569.0	fGB	3-6B50
U+4569.0	fR/S	 140.16
U+4569.0	fTotalStrokes	 20; 22
U+4569.0	fCangjie	TDKF
U+4569.0	fFourCorner	44990
U+4569.0	fCantonese	lou5
U+4569.0	fMandarin	lao3; lao4
U+4569.0	fEnglish	dry fruit (e.g. nuts), dried fruit
U+456A.0	fUTF8	䕪
U+456A.0	fCNS11643	3-5C74
U+456A.0	fGB	5-685B
U+456A.0	fR/S	 140.16
U+456A.0	fTotalStrokes	 20; 22
U+456A.0	fCangjie	TEWJ
U+456A.0	fFourCorner	44140
U+456A.0	fCantonese	zaak6
U+456A.0	fMandarin	zhe4; ze2
U+456A.0	fEnglish	the water plantain
U+456B.0	fUTF8	䕫
U+456B.0	fCNS11643	7-5468
U+456B.0	fR/S	 140.16
U+456B.0	fTotalStrokes	 21; 22
U+456B.0	fCangjie	TCHE
U+456B.0	fFourCorner	44400
U+456B.0	fHakka	 kwi2
U+456B.0	fCantonese	kwai4
U+456B.0	fMandarin	kui2
U+456B.0	fEnglish	corrupted form of &#x5914;
U+456C.0	fUTF8	䕬
U+456C.0	fCNS11643	3-5C79
U+456C.0	fR/S	 140.16
U+456C.0	fTotalStrokes	 20; 22
U+456C.0	fCangjie	TNMM
U+456C.0	fFourCorner	44210
U+456C.0	fHakka	 giong1
U+456C.0	fCantonese	goeng1
U+456C.0	fMandarin	jiang1
U+456C.0	fEnglish	same as &#x8591;
U+456D.0	fUTF8	䕭
U+456D.0	fCNS11643	4-6432
U+456D.0	fGB	3-6B4D
U+456D.0	fR/S	 140.16
U+456D.0	fTotalStrokes	 20; 22
U+456D.0	fCangjie	TGGF
U+456D.0	fFourCorner	44180
U+456D.0	fCantonese	cam4; cim4 (also); zim1
U+456D.0	fMandarin	xian2; qian2
U+456D.0	fEnglish	the name of a variety of grass, a kind of vegetable, a variety of nettle, urtica
U+456E.0	fUTF8	䕮
U+456E.0	fCNS11643	4-6438
U+456E.0	fGB	5-6A5C
U+456E.0	fR/S	 140.16
U+456E.0	fTotalStrokes	 20; 22
U+456E.0	fCangjie	TGJD
U+456E.0	fFourCorner	44420
U+456E.0	fCantonese	guk1
U+456E.0	fMandarin	ju2
U+456E.0	fEnglish	interchangeable with &#x83CA;
U+456F.0	fUTF8	䕯
U+456F.0	fCNS11643	3-5C7A
U+456F.0	fR/S	 140.16
U+456F.0	fTotalStrokes	 20; 22
U+456F.0	fCangjie	TMFO
U+456F.0	fFourCorner	44910
U+456F.0	fCantonese	piu1; piu4
U+456F.0	fMandarin	piao1; piao2
U+456F.0	fEnglish	duckweed
U+4570.0	fUTF8	䕰
U+4570.0	fCNS11643	4-6430
U+4570.0	fGB	3-6B53
U+4570.0	fR/S	 140.16
U+4570.0	fTotalStrokes	 20; 22
U+4570.0	fCangjie	TFHW
U+4570.0	fFourCorner	44860
U+4570.0	fCantonese	faan4
U+4570.0	fMandarin	fan2; ban4; fan4
U+4570.0	fEnglish	a second name for ( 知母) rhizome of wind-weed (Anemarrhena asphodeloides); herb medicine
U+4571.0	fUTF8	䕱
U+4571.0	fCNS11643	3-5C75
U+4571.0	fR/S	 140.16
U+4571.0	fTotalStrokes	 20; 22
U+4571.0	fCangjie	TMTC
U+4571.0	fFourCorner	44180
U+4571.0	fCantonese	tau4; dam1; ding1; taau4
U+4571.0	fMandarin	tou2; dou3; dou4
U+4571.0	fEnglish	the name of a variety of grass
U+4572.0	fUTF8	䕲
U+4572.0	fCNS11643	4-6436
U+4572.0	fGB	3-6B52
U+4572.0	fR/S	 140.16
U+4572.0	fTotalStrokes	 20; 22
U+4572.0	fCangjie	TIYD
U+4572.0	fFourCorner	44290
U+4572.0	fCantonese	lam5
U+4572.0	fMandarin	lin3
U+4572.0	fEnglish	Artemisia, a kind of plant with edible leaves
U+4573.0	fUTF8	䕳
U+4573.0	fCNS11643	4-6727
U+4573.0	fGB	3-6960
U+4573.0	fR/S	 140.17
U+4573.0	fTotalStrokes	 21; 23
U+4573.0	fCangjie	TNMB
U+4573.0	fFourCorner	44220
U+4573.0	fCantonese	mei4; nei4
U+4573.0	fMandarin	mi2
U+4573.0	fEnglish	the name of a variety of grass
U+4574.0	fUTF8	䕴
U+4574.0	fCNS11643	3-5E59
U+4574.0	fR/S	 140.17
U+4574.0	fTotalStrokes	 21; 23
U+4574.0	fCangjie	THDF
U+4574.0	fFourCorner	44930
U+4574.0	fCantonese	zuk1
U+4574.0	fMandarin	zhuo1; zhuo2
U+4574.0	fEnglish	a second for monkshood ( 附子); Chinese herb medicine
U+4575.0	fUTF8	䕵
U+4575.0	fCNS11643	4-6728
U+4575.0	fGB	3-6A51
U+4575.0	fR/S	 140.17
U+4575.0	fTotalStrokes	 21; 23
U+4575.0	fCangjie	TGRF
U+4575.0	fFourCorner	44620
U+4575.0	fCantonese	kit3
U+4575.0	fMandarin	xie2; xie4
U+4575.0	fEnglish	name of a variety of grass; a second name for (葒草) polygonum orientale
U+4576.0	fUTF8	䕶
U+4576.0	fR/S	 140.17
U+4576.0	fTotalStrokes	 23
U+4576.0	fCangjie	TYRE
U+4576.0	fCantonese	wu6
U+4576.0	fMandarin	hu4
U+4576.0	fEnglish	the name of a variety of grass
U+4577.0	fUTF8	䕷
U+4577.0	fCNS11643	F-6839
U+4577.0	fGB	3-6B5B
U+4577.0	fR/S	 140.17
U+4577.0	fTotalStrokes	 23
U+4577.0	fCangjie	TIDF
U+4577.0	fCantonese	mei4
U+4577.0	fMandarin	mi2
U+4577.0	fEnglish	a kind of plant
U+4578.0	fUTF8	䕸
U+4578.0	fCNS11643	3-5F5D
U+4578.0	fGB	5-6A70
U+4578.0	fR/S	 140.18
U+4578.0	fTotalStrokes	 22; 24
U+4578.0	fCangjie	TWFR
U+4578.0	fFourCorner	44360
U+4578.0	fHakka	 giet7; gai1; get7
U+4578.0	fCantonese	gaai1
U+4578.0	fMandarin	xia2
U+4578.0	fEnglish	the stalks of northern hemp of which ropes are made
U+4579.0	fUTF8	䕹
U+4579.0	fCNS11643	4-6937
U+4579.0	fGB	3-6B5F
U+4579.0	fR/S	 140.18
U+4579.0	fTotalStrokes	 22; 21; 24
U+4579.0	fCangjie	TYDG
U+4579.0	fFourCorner	44910
U+4579.0	fHakka	 cap8
U+4579.0	fCantonese	zaap6; cap3; zap6; zip3
U+4579.0	fMandarin	za2
U+4579.0	fEnglish	a grass (with fine leaves) grown in ponds for gold fish
U+457A.0	fUTF8	䕺
U+457A.0	fCNS11643	7-5872
U+457A.0	fR/S	 140.18
U+457A.0	fTotalStrokes	 22; 24
U+457A.0	fCangjie	TTCE
U+457A.0	fFourCorner	44140
U+457A.0	fCantonese	cung4
U+457A.0	fMandarin	cong2
U+457A.0	fEnglish	[1] dense growth; lush growth or the grass [2] tree; a numerary auxiliary for counting trees or similar things [3] roots that grow above ground
U+457B.0	fUTF8	䕻
U+457B.0	fCNS11643	3-6041
U+457B.0	fGB	5-682B
U+457B.0	fR/S	 140.19
U+457B.0	fTotalStrokes	 23; 25
U+457B.0	fCangjie	TMMP
U+457B.0	fFourCorner	44210
U+457B.0	fCantonese	lai6
U+457B.0	fMandarin	li2; ge2; li4
U+457B.0	fEnglish	 adhere to; stick together
U+457C.0	fUTF8	䕼
U+457C.0	fCNS11643	3-6042
U+457C.0	fGB	5-6955
U+457C.0	fR/S	 140.19
U+457C.0	fTotalStrokes	 23; 25
U+457C.0	fCangjie	TTOG
U+457C.0	fFourCorner	44510
U+457C.0	fCantonese	jin4; nan4
U+457C.0	fMandarin	ran2; nan2; nan4
U+457C.0	fEnglish	the name of a variety of grass
U+457D.0	fUTF8	䕽
U+457D.0	fCNS11643	5-7825
U+457D.0	fR/S	 140.19; 140.20
U+457D.0	fTotalStrokes	 24; 25
U+457D.0	fCangjie	TRMI
U+457D.0	fFourCorner	44120
U+457D.0	fHakka	 ch'uk7; cuk7
U+457D.0	fCantonese	zuk6
U+457D.0	fMandarin	zhu2
U+457D.0	fEnglish	a poisonous medicinal plant of the oleander family
U+457E.0	fUTF8	䕾
U+457E.0	fCNS11643	4-6B7D
U+457E.0	fGB	3-687C
U+457E.0	fR/S	 140.20
U+457E.0	fTotalStrokes	 24; 26
U+457E.0	fCangjie	TRRK
U+457E.0	fFourCorner	44240
U+457E.0	fCantonese	jam4; zaam4
U+457E.0	fMandarin	yan2; yin2
U+457E.0	fEnglish	ancient form of &#x8376;
U+457F.0	fUTF8	䕿
U+457F.0	fCNS11643	4-6D5E
U+457F.0	fGB	3-6B4E
U+457F.0	fR/S	 140.22; 140.21
U+457F.0	fTotalStrokes	 26; 27
U+457F.0	fCangjie	TTOF
U+457F.0	fFourCorner	44520
U+457F.0	fCantonese	hon3; ham3; soeng2
U+457F.0	fMandarin	han4
U+457F.0	fEnglish	the name of a variety of grass
U+4580.0	fUTF8	䖀
U+4580.0	fCNS11643	4-6C7D
U+4580.0	fGB	3-6B5E
U+4580.0	fR/S	 140.21
U+4580.0	fTotalStrokes	 25; 27
U+4580.0	fCangjie	TRRR
U+4580.0	fFourCorner	44660
U+4580.0	fCantonese	hiu1; ceoi3
U+4580.0	fMandarin	xiao1
U+4580.0	fEnglish	same as &#x8648;
U+4581.0	fUTF8	䖁
U+4581.0	fCNS11643	3-616A
U+4581.0	fR/S	 140.22
U+4581.0	fTotalStrokes	 26; 28
U+4581.0	fCangjie	TBYTP
U+4581.0	fFourCorner	40430
U+4581.0	fCantonese	jik1; zi3
U+4581.0	fMandarin	yi4
U+4581.0	fEnglish	[1] number [2] count; enumerate
U+4582.0	fUTF8	䖂
U+4582.0	fCNS11643	4-6D79
U+4582.0	fGB	3-6A42
U+4582.0	fR/S	 140.23
U+4582.0	fTotalStrokes	 27; 29
U+4582.0	fCangjie	TVFQ
U+4582.0	fFourCorner	44500
U+4582.0	fCantonese	lyun4
U+4582.0	fMandarin	luan2; luuan2; lian4
U+4582.0	fEnglish	an edible water plant - Brasenia purpurca
U+4583.0	fUTF8	䖃
U+4583.0	fCNS11643	4-6D7B
U+4583.0	fGB	3-6B60
U+4583.0	fR/S	 140.23
U+4583.0	fTotalStrokes	 27; 29
U+4583.0	fCangjie	TRTRR
U+4583.0	fFourCorner	44660
U+4583.0	fCantonese	joek6; laa3
U+4583.0	fMandarin	yue4; yao4
U+4583.0	fEnglish	[1] the water is waving in the breeze [2] old and desolate; rags and rubbish
U+4584.0	fUTF8	䖄
U+4584.0	fCNS11643	F-6C6C
U+4584.0	fGB	5-6A55
U+4584.0	fR/S	 140.23
U+4584.0	fTotalStrokes	 29
U+4584.0	fCangjie	TTGF
U+4584.0	fCantonese	jin4; zin4
U+4584.0	fMandarin	ran2
U+4584.0	fEnglish	[1] ancient form of &#x7136; [2] ancient form of &#x71C3;
U+4585.0	fUTF8	䖅
U+4585.0	fCNS11643	4-6E37
U+4585.0	fR/S	 140.24
U+4585.0	fTotalStrokes	 28; 30
U+4585.0	fCangjie	TMBO
U+4585.0	fFourCorner	44100
U+4585.0	fCantonese	ling4
U+4585.0	fMandarin	ling2
U+4585.0	fEnglish	the name of a variety of grass; Nasturtium
U+4586.0	fUTF8	䖆
U+4586.0	fCNS11643	4-6E36
U+4586.0	fGB	3-6B61
U+4586.0	fR/S	 140.24
U+4586.0	fTotalStrokes	 28; 30; 26
U+4586.0	fCangjie	TMWV
U+4586.0	fFourCorner	44630
U+4586.0	fCantonese	noeng6; loeng6 (also); zoeng6
U+4586.0	fMandarin	rang3; niang4; rang2
U+4586.0	fEnglish	[1] medicinal herb; a kind of vegetable [2] salted vegetables
U+4587.0	fUTF8	䖇
U+4587.0	fCNS11643	5-7C3E
U+4587.0	fR/S	 140.29
U+4587.0	fTotalStrokes	 33; 35
U+4587.0	fCangjie	TDDH
U+4587.0	fFourCorner	44720
U+4587.0	fCantonese	wat1
U+4587.0	fMandarin	yu4
U+4587.0	fEnglish	fragrant herb; fragrant plant; tulip
U+4588.0	fUTF8	䖈
U+4588.0	fCNS11643	3-2A6D
U+4588.0	fGB	5-604F
U+4588.0	fR/S	 141.2
U+4588.0	fTotalStrokes	 8; 6
U+4588.0	fCangjie	YPP
U+4588.0	fFourCorner	21210
U+4588.0	fHakka	 niok7
U+4588.0	fCantonese	joek6; zoek6
U+4588.0	fMandarin	nue4; ne4
U+4588.0	fEnglish	same as &#x8650;
U+4589.0	fUTF8	䖉
U+4589.0	fCNS11643	4-2B3E
U+4589.0	fGB	3-6142
U+4589.0	fR/S	 141.3
U+4589.0	fTotalStrokes	 9; 7
U+4589.0	fCangjie	YPMD
U+4589.0	fFourCorner	21240
U+4589.0	fCantonese	heoi1
U+4589.0	fEnglish	 roar or howl (of tiger)
U+458A.0	fUTF8	䖊
U+458A.0	fCNS11643	4-2F67
U+458A.0	fGB	3-6148
U+458A.0	fR/S	 141.4
U+458A.0	fTotalStrokes	 10; 8
U+458A.0	fCangjie	KYPU
U+458A.0	fFourCorner	41010
U+458A.0	fCantonese	ngaai6; aai6 (also); zi6
U+458A.0	fMandarin	yi4; yi2
U+458A.0	fEnglish	of tiger
U+458B.0	fUTF8	䖋
U+458B.0	fCNS11643	3-3367
U+458B.0	fR/S	 141.4
U+458B.0	fTotalStrokes	 10; 8
U+458B.0	fCangjie	YUO
U+458B.0	fFourCorner	28200
U+458B.0	fCantonese	joek6; zoek6
U+458B.0	fMandarin	nuue4; ne4
U+458B.0	fEnglish	same as &#x8650;
U+458C.0	fUTF8	䖌
U+458C.0	fCNS11643	3-3366
U+458C.0	fR/S	 141.4
U+458C.0	fTotalStrokes	 10; 8
U+458C.0	fCangjie	YULN
U+458C.0	fFourCorner	22200
U+458C.0	fCantonese	ngat6; at6 (also); zi6
U+458C.0	fMandarin	ni4; qin2; ya2; yi4
U+458C.0	fEnglish	 roar; howl (of tiger)
U+458D.0	fUTF8	䖍
U+458D.0	fCNS11643	3-3368
U+458D.0	fR/S	 141.4
U+458D.0	fTotalStrokes	 10; 8
U+458D.0	fCangjie	YPOK
U+458D.0	fFourCorner	21240
U+458D.0	fCantonese	kin4
U+458D.0	fMandarin	qian2
U+458D.0	fEnglish	same as &#x8654;
U+458E.0	fUTF8	䖎
U+458E.0	fCNS11643	5-3535
U+458E.0	fR/S	 141.5
U+458E.0	fTotalStrokes	 11; 9
U+458E.0	fCangjie	NLYPU
U+458E.0	fFourCorner	71210
U+458E.0	fCantonese	haap6
U+458E.0	fMandarin	xia2
U+458E.0	fEnglish	tiger
U+458F.0	fUTF8	䖏
U+458F.0	fCNS11643	3-3862
U+458F.0	fR/S	 141.5
U+458F.0	fTotalStrokes	 11; 9
U+458F.0	fCangjie	YPPKK
U+458F.0	fFourCorner	21220
U+458F.0	fHakka	 chu3; chu5; cu5; cu3
U+458F.0	fCantonese	cyu5; cyu2 (also); cyu3; syu3
U+458F.0	fMandarin	chu4; chu3
U+458F.0	fEnglish	same as &#x8655;
U+4590.0	fUTF8	䖐
U+4590.0	fCNS11643	5-3B70
U+4590.0	fR/S	 141.6
U+4590.0	fTotalStrokes	 12; 10
U+4590.0	fCangjie	HLYPU
U+4590.0	fFourCorner	71210
U+4590.0	fCantonese	ngan4; an4 (also)
U+4590.0	fMandarin	yin2; jin4
U+4590.0	fEnglish	sound of a tiger
U+4591.0	fUTF8	䖑
U+4591.0	fCNS11643	3-3E60
U+4591.0	fGB	5-605B
U+4591.0	fR/S	 141.6
U+4591.0	fTotalStrokes	 12; 10
U+4591.0	fCangjie	YUA
U+4591.0	fFourCorner	26210
U+4591.0	fCantonese	mik6
U+4591.0	fMandarin	mi4
U+4591.0	fEnglish	a white tiger
U+4592.0	fUTF8	䖒
U+4592.0	fCNS11643	3-4441
U+4592.0	fR/S	 141.7
U+4592.0	fTotalStrokes	 13; 11
U+4592.0	fCangjie	YPMRT
U+4592.0	fFourCorner	21210
U+4592.0	fHakka	 hi1
U+4592.0	fCantonese	hei1
U+4592.0	fMandarin	xi2
U+4592.0	fEnglish	 earthenware sacrificial vase
U+4593.0	fUTF8	䖓
U+4593.0	fCNS11643	5-4254
U+4593.0	fR/S	 141.7
U+4593.0	fTotalStrokes	 13; 11
U+4593.0	fCangjie	UUYPU
U+4593.0	fFourCorner	21710
U+4593.0	fCantonese	neot6; leot6 (also)
U+4593.0	fMandarin	nu4; na4
U+4593.0	fEnglish	of a walking tiger
U+4594.0	fUTF8	䖔
U+4594.0	fCNS11643	5-4253
U+4594.0	fGB	3-614C
U+4594.0	fR/S	 141.7
U+4594.0	fTotalStrokes	 13; 11
U+4594.0	fCangjie	GIYPU
U+4594.0	fFourCorner	41710
U+4594.0	fCantonese	ham2
U+4594.0	fMandarin	kan3; han4
U+4594.0	fEnglish	a kind of white tiger, an angry tiger, sound of a tiger
U+4595.0	fUTF8	䖕
U+4595.0	fCNS11643	3-4442
U+4595.0	fGB	5-605E
U+4595.0	fR/S	 141.7
U+4595.0	fTotalStrokes	 13; 11
U+4595.0	fCangjie	YUBM
U+4595.0	fFourCorner	27210
U+4595.0	fCantonese	zou2
U+4595.0	fMandarin	cuo2; zu3
U+4595.0	fEnglish	a tiger
U+4596.0	fUTF8	䖖
U+4596.0	fCNS11643	3-4443
U+4596.0	fR/S	 141.7
U+4596.0	fTotalStrokes	 13; 11
U+4596.0	fCangjie	YUWL
U+4596.0	fFourCorner	26210
U+4596.0	fCantonese	haap6
U+4596.0	fMandarin	xia2
U+4596.0	fEnglish	same as &#x72CE;
U+4597.0	fUTF8	䖗
U+4597.0	fCNS11643	4-4D26
U+4597.0	fGB	3-6146
U+4597.0	fR/S	 141.9
U+4597.0	fTotalStrokes	 15; 13
U+4597.0	fCangjie	YPIHR
U+4597.0	fFourCorner	21250
U+4597.0	fCantonese	ngaam4; aam4 (also)
U+4597.0	fMandarin	yan2; yan4
U+4597.0	fEnglish	strong; great fighters or warriors - the bear and the tiger
U+4598.0	fUTF8	䖘
U+4598.0	fCNS11643	4-5336
U+4598.0	fGB	3-6150
U+4598.0	fR/S	 141.10
U+4598.0	fTotalStrokes	 16; 14
U+4598.0	fCangjie	YUNAI
U+4598.0	fFourCorner	27210
U+4598.0	fHakka	  tu2
U+4598.0	fCantonese	tou4
U+4598.0	fMandarin	tu2
U+4598.0	fEnglish	a tiger was so called in Tshii state
U+4599.0	fUTF8	䖙
U+4599.0	fCNS11643	4-5D32
U+4599.0	fGB	3-605E
U+4599.0	fR/S	 141.11; 141.12
U+4599.0	fTotalStrokes	 18; 15
U+4599.0	fCangjie	SOHYU
U+4599.0	fFourCorner	78210
U+4599.0	fCantonese	tai1
U+4599.0	fMandarin	ti1; ti2
U+4599.0	fEnglish	a resting tiger; a tiger to lie down and take some rest
U+459A.0	fUTF8	䖚
U+459A.0	fCNS11643	5-643B
U+459A.0	fR/S	 141.12
U+459A.0	fTotalStrokes	 18; 16
U+459A.0	fCangjie	YUHRF
U+459A.0	fFourCorner	27220
U+459A.0	fCantonese	wu1
U+459A.0	fMandarin	wu1
U+459A.0	fEnglish	a tiger
U+459B.0	fUTF8	䖛
U+459B.0	fCNS11643	5-643A
U+459B.0	fR/S	 141.12
U+459B.0	fTotalStrokes	 18; 16
U+459B.0	fCangjie	YUJBF
U+459B.0	fFourCorner	24210
U+459B.0	fCantonese	sok3; sak3
U+459B.0	fMandarin	suo3
U+459B.0	fEnglish	of tiger
U+459C.0	fUTF8	䖜
U+459C.0	fCNS11643	4-643A
U+459C.0	fGB	5-4A3E
U+459C.0	fR/S	 141.14; 141.13
U+459C.0	fTotalStrokes	 20; 17
U+459C.0	fCangjie	YUA
U+459C.0	fFourCorner	21600
U+459C.0	fCantonese	wui4
U+459C.0	fMandarin	yin2; yin4
U+459C.0	fEnglish	the fighting sound made by two tigers
U+459D.0	fUTF8	䖝
U+459D.0	fCNS11643	3-272E
U+459D.0	fR/S	 142.1
U+459D.0	fTotalStrokes	 7
U+459D.0	fCangjie	HLMI
U+459D.0	fFourCorner	20130
U+459D.0	fCantonese	cung4
U+459D.0	fMandarin	hui3; chong2
U+459D.0	fEnglish	corrupted form of &#x866B;, &#x87F2;
U+459E.0	fUTF8	䖞
U+459E.0	fCNS11643	3-2E7D
U+459E.0	fR/S	 142.3
U+459E.0	fTotalStrokes	 9
U+459E.0	fCangjie	LIDI
U+459E.0	fFourCorner	54100
U+459E.0	fCantonese	zau2
U+459E.0	fMandarin	zhou3
U+459E.0	fEnglish	a sea-insect; in the shape of a human's elbow
U+459F.0	fUTF8	䖟
U+459F.0	fCNS11643	3-2F21
U+459F.0	fGB	5-6067
U+459F.0	fR/S	 142.3
U+459F.0	fTotalStrokes	 9
U+459F.0	fCangjie	YVLMI
U+459F.0	fFourCorner	00130
U+459F.0	fHakka	 mang2;  mong2; mang1
U+459F.0	fCantonese	mong4
U+459F.0	fMandarin	meng2; mang3
U+459F.0	fEnglish	non-classical form of &#x8771;, &#x867B;
U+45A0.0	fUTF8	䖠
U+45A0.0	fCNS11643	3-2F22
U+45A0.0	fR/S	 142.3
U+45A0.0	fTotalStrokes	 9
U+45A0.0	fCangjie	LIHNI
U+45A0.0	fFourCorner	57110
U+45A0.0	fCantonese	jyun4; zyun4
U+45A0.0	fMandarin	yuan2
U+45A0.0	fEnglish	non-classical form of &#x8788;
U+45A1.0	fUTF8	䖡
U+45A1.0	fCNS11643	4-2F69
U+45A1.0	fGB	5-6078
U+45A1.0	fR/S	 142.4
U+45A1.0	fTotalStrokes	 10
U+45A1.0	fCangjie	LING
U+45A1.0	fFourCorner	57110
U+45A1.0	fCantonese	nuk6; luk6 (also)
U+45A1.0	fMandarin	nuu4; n4
U+45A1.0	fEnglish	millipede
U+45A2.0	fUTF8	䖢
U+45A2.0	fCNS11643	4-2F6B
U+45A2.0	fGB	3-615D
U+45A2.0	fR/S	 142.4
U+45A2.0	fTotalStrokes	 10
U+45A2.0	fCangjie	LIFH
U+45A2.0	fFourCorner	59120
U+45A2.0	fHakka	 miau2; meu2
U+45A2.0	fCantonese	miu4
U+45A2.0	fMandarin	miao2
U+45A2.0	fEnglish	silkworms just hatched
U+45A3.0	fUTF8	䖣
U+45A3.0	fCNS11643	6-3B69
U+45A3.0	fR/S	 142.4
U+45A3.0	fTotalStrokes	 10; 9
U+45A3.0	fCangjie	LIHLO
U+45A3.0	fFourCorner	52130
U+45A3.0	fHakka	  zau3
U+45A3.0	fCantonese	zou2; zau2
U+45A3.0	fMandarin	zao3; sao4; ye4
U+45A3.0	fEnglish	[1] the flea [2] scratch
U+45A4.0	fUTF8	䖤
U+45A4.0	fCNS11643	3-3868
U+45A4.0	fGB	5-6127
U+45A4.0	fR/S	 142.5
U+45A4.0	fTotalStrokes	 11
U+45A4.0	fCangjie	NULMI
U+45A4.0	fFourCorner	27130
U+45A4.0	fCantonese	jyun1; zyun1
U+45A4.0	fMandarin	yuan1; wan3; yuan2
U+45A4.0	fEnglish	same as &#x873F;
U+45A5.0	fUTF8	䖥
U+45A5.0	fCNS11643	3-3867
U+45A5.0	fGB	5-5F25
U+45A5.0	fR/S	 142.5
U+45A5.0	fTotalStrokes	 11
U+45A5.0	fCangjie	NHLMI
U+45A5.0	fFourCorner	17130
U+45A5.0	fCantonese	maau4
U+45A5.0	fMandarin	mao2; li2
U+45A5.0	fEnglish	same as &#x87CA;
U+45A6.0	fUTF8	䖦
U+45A6.0	fCNS11643	4-347C
U+45A6.0	fGB	3-6179
U+45A6.0	fR/S	 142.5
U+45A6.0	fTotalStrokes	 11
U+45A6.0	fCangjie	LIUU
U+45A6.0	fFourCorner	52170
U+45A6.0	fHakka	 kiut7; chot7; zot7
U+45A6.0	fCantonese	wat1
U+45A6.0	fMandarin	qu1
U+45A6.0	fEnglish	a garden spider
U+45A7.0	fUTF8	䖧
U+45A7.0	fCNS11643	3-3865
U+45A7.0	fGB	5-6122
U+45A7.0	fR/S	 142.5
U+45A7.0	fTotalStrokes	 11
U+45A7.0	fCangjie	LIAM
U+45A7.0	fFourCorner	56110
U+45A7.0	fCantonese	naat6; laat6 (also)
U+45A7.0	fMandarin	na4
U+45A7.0	fEnglish	a poisonous insect; scorpion; to sting; prick of a poisonous insect; venomous sting, to ache
U+45A8.0	fUTF8	䖨
U+45A8.0	fCNS11643	4-347B
U+45A8.0	fGB	3-616A
U+45A8.0	fR/S	 142.5
U+45A8.0	fTotalStrokes	 11
U+45A8.0	fCangjie	LIMR
U+45A8.0	fFourCorner	51160
U+45A8.0	fCantonese	sek6
U+45A8.0	fMandarin	shi2
U+45A8.0	fEnglish	[1] mantis [2] same as &#x87C5;
U+45A9.0	fUTF8	䖩
U+45A9.0	fCNS11643	4-3479
U+45A9.0	fGB	3-6178
U+45A9.0	fR/S	 142.5
U+45A9.0	fTotalStrokes	 11
U+45A9.0	fCangjie	LIPH
U+45A9.0	fFourCorner	53100
U+45A9.0	fCantonese	bat6; bit1
U+45A9.0	fMandarin	bi4; pi4
U+45A9.0	fEnglish	a kind of bee; black bee
U+45AA.0	fUTF8	䖪
U+45AA.0	fCNS11643	5-3C26
U+45AA.0	fR/S	 142.5; 142.6
U+45AA.0	fTotalStrokes	 12; 11
U+45AA.0	fCangjie	YPLMI
U+45AA.0	fFourCorner	21130
U+45AA.0	fCantonese	zi1
U+45AA.0	fMandarin	zi1; ci2
U+45AA.0	fEnglish	same as &#x8800;
U+45AB.0	fUTF8	䖫
U+45AB.0	fCNS11643	6-4448
U+45AB.0	fR/S	 142.5
U+45AB.0	fTotalStrokes	 11
U+45AB.0	fCangjie	LICMJ
U+45AB.0	fFourCorner	58140
U+45AB.0	fCantonese	pong5
U+45AB.0	fMandarin	bang4
U+45AB.0	fEnglish	same as &#x868C;
U+45AC.0	fUTF8	䖬
U+45AC.0	fGB	3-616F
U+45AC.0	fR/S	 142.5
U+45AC.0	fTotalStrokes	 11
U+45AC.0	fCangjie	LIWL
U+45AC.0	fCantonese	gaap3
U+45AD.0	fUTF8	䖭
U+45AD.0	fCNS11643	4-3A72
U+45AD.0	fGB	3-622D
U+45AD.0	fR/S	 142.6
U+45AD.0	fTotalStrokes	 12
U+45AD.0	fCangjie	FQLMI
U+45AD.0	fFourCorner	90130
U+45AD.0	fCantonese	gyun3; gyun2
U+45AD.0	fMandarin	juan4
U+45AD.0	fEnglish	[1] grub (a kind of larva) [2] a variety of spider; octopus; a long-legged spider
U+45AE.0	fUTF8	䖮
U+45AE.0	fCNS11643	3-3E67
U+45AE.0	fGB	5-6136
U+45AE.0	fR/S	 142.6
U+45AE.0	fTotalStrokes	 12
U+45AE.0	fCangjie	LIHBR
U+45AE.0	fFourCorner	57120
U+45AE.0	fCantonese	hoeng2
U+45AE.0	fMandarin	xiang3
U+45AE.0	fEnglish	same as &#x8801;
U+45AF.0	fUTF8	䖯
U+45AF.0	fCNS11643	4-3A74
U+45AF.0	fGB	3-6221
U+45AF.0	fR/S	 142.6
U+45AF.0	fTotalStrokes	 12
U+45AF.0	fCangjie	GGLMI
U+45AF.0	fFourCorner	40130
U+45AF.0	fCantonese	kwai1; fui1; kwaai1
U+45AF.0	fMandarin	wa1; gui4; kui2
U+45AF.0	fEnglish	[1] poisonous insect [2] same as &#x594E; [3] same as &#x86D9;
U+45B0.0	fUTF8	䖰
U+45B0.0	fCNS11643	3-3E65
U+45B0.0	fR/S	 142.6
U+45B0.0	fTotalStrokes	 12
U+45B0.0	fCangjie	LIHHV
U+45B0.0	fFourCorner	52130
U+45B0.0	fCantonese	paai3; bei1; pai3
U+45B0.0	fMandarin	pai4
U+45B0.0	fEnglish	a kind of gnat; a small flying insect
U+45B1.0	fUTF8	䖱
U+45B1.0	fCNS11643	5-3B74
U+45B1.0	fR/S	 142.6
U+45B1.0	fTotalStrokes	 12
U+45B1.0	fCangjie	LISMG
U+45B1.0	fFourCorner	51110
U+45B1.0	fCantonese	hong1; hong4
U+45B1.0	fMandarin	kuang1
U+45B1.0	fEnglish	large shrimp in the sea
U+45B2.0	fUTF8	䖲
U+45B2.0	fCNS11643	4-3A70
U+45B2.0	fGB	3-6229
U+45B2.0	fR/S	 142.6
U+45B2.0	fTotalStrokes	 12
U+45B2.0	fCangjie	LIPA
U+45B2.0	fFourCorner	57120
U+45B2.0	fCantonese	ceon4; zung1
U+45B2.0	fMandarin	xun2
U+45B2.0	fEnglish	a kind of insect (looks like cicada)
U+45B3.0	fUTF8	䖳
U+45B3.0	fCNS11643	4-3A6D
U+45B3.0	fGB	3-622E
U+45B3.0	fR/S	 142.6
U+45B3.0	fTotalStrokes	 12
U+45B3.0	fCangjie	LIJHP
U+45B3.0	fFourCorner	53110
U+45B3.0	fHakka	 za5
U+45B3.0	fCantonese	zaa3; caa3 (also); zit3 (also)
U+45B3.0	fMandarin	zha4
U+45B3.0	fEnglish	the large Medusa that floats on the sea; jelly-fish
U+45B4.0	fUTF8	䖴
U+45B4.0	fCNS11643	3-3E64
U+45B4.0	fGB	5-6137
U+45B4.0	fR/S	 142.6
U+45B4.0	fTotalStrokes	 12
U+45B4.0	fCangjie	LILMO
U+45B4.0	fFourCorner	52110
U+45B4.0	fCantonese	jiu4; ziu4
U+45B4.0	fMandarin	yao2
U+45B4.0	fEnglish	[1] a kind of insect [2] non-classical form of &#x73E7;
U+45B5.0	fUTF8	䖵
U+45B5.0	fCNS11643	3-3E66
U+45B5.0	fGB	5-6130
U+45B5.0	fR/S	 142.6
U+45B5.0	fTotalStrokes	 12
U+45B5.0	fCangjie	LILMI
U+45B5.0	fFourCorner	55130
U+45B5.0	fHakka	 kun1; kwun1
U+45B5.0	fCantonese	kwan1
U+45B5.0	fMandarin	kun2
U+45B5.0	fEnglish	insects
U+45B6.0	fUTF8	䖶
U+45B6.0	fCNS11643	4-4056
U+45B6.0	fGB	5-6141
U+45B6.0	fR/S	 142.7
U+45B6.0	fTotalStrokes	 13
U+45B6.0	fCangjie	LIMSO
U+45B6.0	fFourCorner	51130
U+45B6.0	fCantonese	fui1
U+45B6.0	fMandarin	hui1
U+45B6.0	fEnglish	to raise up the nose in order to get the bugs in the earth (said of a hog), hedgehog family
U+45B7.0	fUTF8	䖷
U+45B7.0	fCNS11643	4-4057
U+45B7.0	fGB	3-623B
U+45B7.0	fR/S	 142.7
U+45B7.0	fTotalStrokes	 13
U+45B7.0	fCangjie	LIKKB
U+45B7.0	fFourCorner	54120
U+45B7.0	fCantonese	hei1
U+45B7.0	fMandarin	xi1
U+45B7.0	fEnglish	a kind of insect
U+45B8.0	fUTF8	䖸
U+45B8.0	fCNS11643	3-444B
U+45B8.0	fR/S	 142.7
U+45B8.0	fTotalStrokes	 13
U+45B8.0	fCangjie	HILMI
U+45B8.0	fFourCorner	23130
U+45B8.0	fCantonese	ngo4; o4; ngai5; ai5
U+45B8.0	fMandarin	e2
U+45B8.0	fEnglish	same as &#x86FE;
U+45B9.0	fUTF8	䖹
U+45B9.0	fCNS11643	3-4447
U+45B9.0	fR/S	 142.7
U+45B9.0	fTotalStrokes	 13
U+45B9.0	fCangjie	LITQ
U+45B9.0	fFourCorner	54140
U+45B9.0	fHakka	 mi3; jong3; rong3
U+45B9.0	fCantonese	mei5; zoeng4
U+45B9.0	fMandarin	yang2
U+45B9.0	fEnglish	interchangeable with &#x86D8;
U+45BA.0	fUTF8	䖺
U+45BA.0	fCNS11643	4-405B
U+45BA.0	fGB	3-3179
U+45BA.0	fR/S	 142.7
U+45BA.0	fTotalStrokes	 13
U+45BA.0	fCangjie	OLHI
U+45BA.0	fFourCorner	28230
U+45BA.0	fCantonese	tiu4; siu6
U+45BA.0	fMandarin	tiao2; zhao4
U+45BA.0	fEnglish	a legendary animal in ancient times
U+45BB.0	fUTF8	䖻
U+45BB.0	fCNS11643	4-3A6E
U+45BB.0	fGB	5-614A
U+45BB.0	fR/S	 142.7; 142.6
U+45BB.0	fTotalStrokes	 12; 13
U+45BB.0	fCangjie	LIYIU
U+45BB.0	fFourCorner	50110
U+45BB.0	fCantonese	lau4; zaau4
U+45BB.0	fMandarin	liu2; you2
U+45BB.0	fEnglish	[1] ephemera; ephemerid; May fly [2] poison of the snake
U+45BC.0	fUTF8	䖼
U+45BC.0	fCNS11643	4-405D
U+45BC.0	fGB	3-6233
U+45BC.0	fR/S	 142.7
U+45BC.0	fTotalStrokes	 13
U+45BC.0	fCangjie	LIGIL
U+45BC.0	fFourCorner	57120
U+45BC.0	fCantonese	koek6; goek3
U+45BC.0	fMandarin	jue2; xue4
U+45BC.0	fEnglish	the dung beetle Scarabaeus; the scavenger beetle
U+45BD.0	fUTF8	䖽
U+45BD.0	fCNS11643	3-444A
U+45BD.0	fGB	5-6147
U+45BD.0	fR/S	 142.7
U+45BD.0	fTotalStrokes	 13
U+45BD.0	fCangjie	HNLMI
U+45BD.0	fFourCorner	22130
U+45BD.0	fCantonese	lei4
U+45BD.0	fMandarin	li2
U+45BD.0	fEnglish	same as &#x870A;
U+45BE.0	fUTF8	䖾
U+45BE.0	fGB	S-236F
U+45BE.0	fR/S	 142.7
U+45BE.0	fTotalStrokes	 13
U+45BE.0	fCangjie	LIJMU
U+45BE.0	fCantonese	jyun4
U+45BF.0	fUTF8	䖿
U+45BF.0	fCNS11643	4-4678
U+45BF.0	fGB	3-6240
U+45BF.0	fR/S	 142.8
U+45BF.0	fTotalStrokes	 14
U+45BF.0	fCangjie	HHLMI
U+45BF.0	fFourCorner	27130
U+45BF.0	fCantonese	lei4
U+45BF.0	fMandarin	li2
U+45BF.0	fEnglish	same as &#x870A;
U+45C0.0	fUTF8	䗀
U+45C0.0	fCNS11643	3-4946
U+45C0.0	fR/S	 142.8
U+45C0.0	fTotalStrokes	 14
U+45C0.0	fCangjie	LIGCE
U+45C0.0	fFourCorner	54140
U+45C0.0	fCantonese	cing1
U+45C0.0	fMandarin	cheng1
U+45C0.0	fEnglish	a kind of clam
U+45C1.0	fUTF8	䗁
U+45C1.0	fCNS11643	4-4672
U+45C1.0	fGB	3-623E
U+45C1.0	fR/S	 142.8
U+45C1.0	fTotalStrokes	 14
U+45C1.0	fCangjie	LIKMR
U+45C1.0	fFourCorner	54120
U+45C1.0	fHakka	  ki3; kia2
U+45C1.0	fCantonese	gei6
U+45C1.0	fMandarin	ji4
U+45C1.0	fEnglish	an insect
U+45C2.0	fUTF8	䗂
U+45C2.0	fCNS11643	4-4673
U+45C2.0	fGB	5-6157
U+45C2.0	fR/S	 142.8
U+45C2.0	fTotalStrokes	 14
U+45C2.0	fCangjie	LIYPU
U+45C2.0	fFourCorner	51110
U+45C2.0	fCantonese	fu2
U+45C2.0	fMandarin	hu3
U+45C2.0	fEnglish	Diemyctylus pyrrhogaster, a kind of amphibious reptile, a kind of spider; a fly-eating spider
U+45C3.0	fUTF8	䗃
U+45C3.0	fCNS11643	4-4671
U+45C3.0	fGB	3-6167
U+45C3.0	fR/S	 142.8
U+45C3.0	fTotalStrokes	 14
U+45C3.0	fCangjie	LIII
U+45C3.0	fFourCorner	53150
U+45C3.0	fCantonese	zaan6; zan2; zan6
U+45C3.0	fMandarin	zhan4
U+45C3.0	fEnglish	[1] the millipede [2] a kind of cicada; big and with black color
U+45C4.0	fUTF8	䗄
U+45C4.0	fCNS11643	4-4675
U+45C4.0	fGB	3-6242
U+45C4.0	fR/S	 142.8
U+45C4.0	fTotalStrokes	 14
U+45C4.0	fCangjie	LICKL
U+45C4.0	fFourCorner	58120
U+45C4.0	fCantonese	fu2; fu6; gu2
U+45C4.0	fMandarin	fu3; pi4
U+45C4.0	fEnglish	a kind of insect; golden beetle
U+45C5.0	fUTF8	䗅
U+45C5.0	fCNS11643	3-4947
U+45C5.0	fGB	5-6070
U+45C5.0	fR/S	 142.8
U+45C5.0	fTotalStrokes	 14
U+45C5.0	fCangjie	LISMV
U+45C5.0	fFourCorner	51130
U+45C5.0	fHakka	 chong2; cong2
U+45C5.0	fCantonese	jau4; coeng4
U+45C5.0	fMandarin	chang2
U+45C5.0	fEnglish	a kind of centipede
U+45C6.0	fUTF8	䗆
U+45C6.0	fCNS11643	4-4670
U+45C6.0	fGB	3-624A
U+45C6.0	fR/S	 142.8
U+45C6.0	fTotalStrokes	 14
U+45C6.0	fCangjie	LIJRR
U+45C6.0	fFourCorner	53170
U+45C6.0	fHakka	 gon1
U+45C6.0	fCantonese	gun2; gwun2 (also); gun1; sung1
U+45C6.0	fMandarin	guan3; wei3
U+45C6.0	fEnglish	[1] a kind of insect [2] ant
U+45C7.0	fUTF8	䗇
U+45C7.0	fCNS11643	4-4676
U+45C7.0	fGB	3-6245
U+45C7.0	fR/S	 142.8
U+45C7.0	fTotalStrokes	 14
U+45C7.0	fCangjie	LIPFD
U+45C7.0	fFourCorner	57120
U+45C7.0	fCantonese	guk6; guk1
U+45C7.0	fMandarin	ju2; qu2
U+45C7.0	fEnglish	the toad (especially in reference to the one supposed to live on the moon), earthworm
U+45C8.0	fUTF8	䗈
U+45C8.0	fCNS11643	6-6169
U+45C8.0	fR/S	 142.8
U+45C8.0	fTotalStrokes	 14
U+45C8.0	fCangjie	LIYVU
U+45C8.0	fFourCorner	50160
U+45C8.0	fCantonese	mong4
U+45C8.0	fMandarin	meng2
U+45C8.0	fEnglish	non-classical form of &#x8771;, &#x867B;
U+45C9.0	fUTF8	䗉
U+45C9.0	fCNS11643	3-4948
U+45C9.0	fR/S	 142.8
U+45C9.0	fTotalStrokes	 14
U+45C9.0	fCangjie	LIAA
U+45C9.0	fFourCorner	56160
U+45C9.0	fCantonese	coeng1
U+45C9.0	fMandarin	chang1
U+45C9.0	fEnglish	a small solitary wasp
U+45CA.0	fUTF8	䗊
U+45CA.0	fCNS11643	4-467B
U+45CA.0	fGB	3-6249
U+45CA.0	fR/S	 142.8
U+45CA.0	fTotalStrokes	 14
U+45CA.0	fCangjie	LIFF
U+45CA.0	fFourCorner	59180
U+45CA.0	fCantonese	taam3
U+45CA.0	fMandarin	tan4; cheng2
U+45CA.0	fEnglish	to put out the tongue (said of animal)
U+45CB.0	fUTF8	䗋
U+45CB.0	fCNS11643	3-4E5A
U+45CB.0	fR/S	 142.9
U+45CB.0	fTotalStrokes	 15
U+45CB.0	fCangjie	LITMD
U+45CB.0	fFourCorner	54190
U+45CB.0	fCantonese	mau4
U+45CB.0	fMandarin	mou2
U+45CB.0	fEnglish	a marine crab
U+45CC.0	fUTF8	䗌
U+45CC.0	fCNS11643	3-4E5C
U+45CC.0	fR/S	 142.9
U+45CC.0	fTotalStrokes	 15
U+45CC.0	fCangjie	LIAHM
U+45CC.0	fFourCorner	56110
U+45CC.0	fCantonese	sing1
U+45CC.0	fMandarin	xing1
U+45CC.0	fEnglish	dragonfly
U+45CD.0	fUTF8	䗍
U+45CD.0	fCNS11643	7-2842
U+45CD.0	fR/S	 142.9
U+45CD.0	fTotalStrokes	 15
U+45CD.0	fCangjie	VOLMI
U+45CD.0	fFourCorner	27130
U+45CD.0	fHakka	 li3
U+45CD.0	fCantonese	lai5
U+45CD.0	fMandarin	li3
U+45CD.0	fEnglish	same as &#x8821;
U+45CE.0	fUTF8	䗎
U+45CE.0	fCNS11643	3-4E57
U+45CE.0	fR/S	 142.9
U+45CE.0	fTotalStrokes	 15
U+45CE.0	fCangjie	LIMWG
U+45CE.0	fFourCorner	51110
U+45CE.0	fCantonese	jin1; zaan2; zin1
U+45CE.0	fMandarin	yan1
U+45CE.0	fEnglish	a kind of insect
U+45CF.0	fUTF8	䗏
U+45CF.0	fCNS11643	3-4E58
U+45CF.0	fR/S	 142.9
U+45CF.0	fTotalStrokes	 15
U+45CF.0	fCangjie	LIJFE
U+45CF.0	fFourCorner	53140
U+45CF.0	fHakka	 seu1
U+45CF.0	fCantonese	sau1
U+45CF.0	fMandarin	sou1
U+45CF.0	fEnglish	an old name for the millipede
U+45D0.0	fUTF8	䗐
U+45D0.0	fCNS11643	4-4D30
U+45D0.0	fGB	3-625A
U+45D0.0	fR/S	 142.9
U+45D0.0	fTotalStrokes	 15
U+45D0.0	fCangjie	YDLMI
U+45D0.0	fFourCorner	08130
U+45D0.0	fCantonese	si1
U+45D0.0	fMandarin	shi1
U+45D0.0	fEnglish	a weevil found in rice, etc.
U+45D1.0	fUTF8	䗑
U+45D1.0	fCNS11643	3-4E5B
U+45D1.0	fGB	5-616D
U+45D1.0	fR/S	 142.9
U+45D1.0	fTotalStrokes	 15
U+45D1.0	fCangjie	LIAYT
U+45D1.0	fFourCorner	56110
U+45D1.0	fCantonese	jap6
U+45D1.0	fMandarin	xi2; yi4
U+45D1.0	fEnglish	firefly
U+45D2.0	fUTF8	䗒
U+45D2.0	fCNS11643	4-4679
U+45D2.0	fGB	3-6248
U+45D2.0	fR/S	 142.9; 142.8
U+45D2.0	fTotalStrokes	 14; 15
U+45D2.0	fCangjie	LITTC
U+45D2.0	fFourCorner	58110
U+45D2.0	fCantonese	bing6; bei3
U+45D2.0	fMandarin	bing4
U+45D2.0	fEnglish	a kind of clam; long and narrow
U+45D3.0	fUTF8	䗓
U+45D3.0	fCNS11643	4-4D36
U+45D3.0	fGB	3-6256
U+45D3.0	fR/S	 142.9
U+45D3.0	fTotalStrokes	 15
U+45D3.0	fCangjie	LIPKP
U+45D3.0	fFourCorner	57130
U+45D3.0	fCantonese	cung1; cung2
U+45D3.0	fMandarin	cong1
U+45D3.0	fEnglish	dragonfly
U+45D4.0	fUTF8	䗔
U+45D4.0	fCNS11643	4-4D2F
U+45D4.0	fGB	3-6255
U+45D4.0	fR/S	 142.9
U+45D4.0	fTotalStrokes	 15
U+45D4.0	fCangjie	LIONK
U+45D4.0	fFourCorner	57130
U+45D4.0	fCantonese	hau4
U+45D4.0	fMandarin	hou2
U+45D4.0	fEnglish	[1] house-lizard; gecko [2] a kind of insect; living in the water
U+45D5.0	fUTF8	䗕
U+45D5.0	fCNS11643	3-494D
U+45D5.0	fR/S	 142.8
U+45D5.0	fTotalStrokes	 14
U+45D5.0	fCangjie	JNUI
U+45D5.0	fFourCorner	30130
U+45D5.0	fCantonese	jyun1; zyun1
U+45D5.0	fMandarin	wan1; wan3
U+45D5.0	fEnglish	same as &#x873F;
U+45D6.0	fUTF8	䗖
U+45D6.0	fGB	S-2370
U+45D6.0	fR/S	 142.9
U+45D6.0	fTotalStrokes	 15
U+45D6.0	fCangjie	LITJB
U+45D6.0	fCantonese	daai3; dai6; zit3
U+45D6.0	fMandarin	chi4; di4; ti2; zhe4
U+45D6.0	fEnglish	simplified form of &#x87AE;
U+45D7.0	fUTF8	䗗
U+45D7.0	fCNS11643	4-533E
U+45D7.0	fGB	3-6261
U+45D7.0	fR/S	 142.10
U+45D7.0	fTotalStrokes	 16
U+45D7.0	fCangjie	LIHMT
U+45D7.0	fFourCorner	58140
U+45D7.0	fCantonese	hin1
U+45D7.0	fMandarin	ji1
U+45D7.0	fEnglish	same as &#x8688;
U+45D8.0	fUTF8	䗘
U+45D8.0	fCNS11643	3-525C
U+45D8.0	fGB	5-6179
U+45D8.0	fR/S	 142.10
U+45D8.0	fTotalStrokes	 16
U+45D8.0	fCangjie	LIGIT
U+45D8.0	fFourCorner	44110
U+45D8.0	fCantonese	gap3
U+45D8.0	fMandarin	ge3; ge2; ke4
U+45D8.0	fEnglish	a weevil
U+45D9.0	fUTF8	䗙
U+45D9.0	fCNS11643	7-3273
U+45D9.0	fR/S	 142.10
U+45D9.0	fTotalStrokes	 16
U+45D9.0	fCangjie	JTCI
U+45D9.0	fFourCorner	30130
U+45D9.0	fCantonese	hon4
U+45D9.0	fMandarin	han2
U+45D9.0	fEnglish	earthworm
U+45DA.0	fUTF8	䗚
U+45DA.0	fCNS11643	4-533D
U+45DA.0	fGB	3-6260
U+45DA.0	fR/S	 142.10
U+45DA.0	fTotalStrokes	 16
U+45DA.0	fCangjie	LIIBI
U+45DA.0	fFourCorner	53140
U+45DA.0	fCantonese	bok3
U+45DA.0	fMandarin	bo2
U+45DA.0	fEnglish	eggs of mantis; chrysalis
U+45DB.0	fUTF8	䗛
U+45DB.0	fCNS11643	4-584D
U+45DB.0	fGB	3-6262
U+45DB.0	fR/S	 142.10
U+45DB.0	fTotalStrokes	 16
U+45DB.0	fCangjie	LIOLB
U+45DB.0	fFourCorner	58127
U+45DB.0	fCantonese	sau1; saau1; suk1
U+45DB.0	fMandarin	su4
U+45DB.0	fEnglish	a kind of insect
U+45DC.0	fUTF8	䗜
U+45DC.0	fCNS11643	3-5260
U+45DC.0	fGB	5-6224
U+45DC.0	fR/S	 142.10
U+45DC.0	fTotalStrokes	 16
U+45DC.0	fCangjie	LIHHW
U+45DC.0	fFourCorner	57160
U+45DC.0	fCantonese	lau4; laau4
U+45DC.0	fMandarin	liu2
U+45DC.0	fEnglish	[1] mole-cricket [2] a kind of insect [3] same as &#x45BB;
U+45DD.0	fUTF8	䗝
U+45DD.0	fCNS11643	7-3275
U+45DD.0	fR/S	 142.10
U+45DD.0	fTotalStrokes	 16
U+45DD.0	fCangjie	ILLMI
U+45DD.0	fFourCorner	35130
U+45DD.0	fCantonese	caam4
U+45DD.0	fMandarin	can2
U+45DD.0	fEnglish	non-classical form of &#x8836;
U+45DE.0	fUTF8	䗞
U+45DE.0	fCNS11643	3-5261
U+45DE.0	fR/S	 142.10
U+45DE.0	fTotalStrokes	 16
U+45DE.0	fCangjie	MKLII
U+45DE.0	fFourCorner	20130
U+45DE.0	fCantonese	caam4
U+45DE.0	fMandarin	can2; chen3; shan3; yin3; zan4; zhan4
U+45DE.0	fEnglish	same as &#x8836;
U+45DF.0	fUTF8	䗟
U+45DF.0	fCNS11643	5-5E32
U+45DF.0	fR/S	 142.11
U+45DF.0	fTotalStrokes	 17
U+45DF.0	fCangjie	SELMI
U+45DF.0	fFourCorner	77130
U+45DF.0	fCantonese	ai3; ngai3 (also)
U+45DF.0	fMandarin	yi4
U+45DF.0	fEnglish	a kind of insect
U+45E0.0	fUTF8	䗠
U+45E0.0	fCNS11643	4-5857
U+45E0.0	fGB	3-6275
U+45E0.0	fR/S	 142.11
U+45E0.0	fTotalStrokes	 17; 16
U+45E0.0	fCangjie	YOLMI
U+45E0.0	fFourCorner	08130
U+45E0.0	fCantonese	syun4; syun6
U+45E0.0	fMandarin	xuan2
U+45E0.0	fEnglish	the larva of mosquitoes
U+45E1.0	fUTF8	䗡
U+45E1.0	fCNS11643	4-5852
U+45E1.0	fGB	3-626D
U+45E1.0	fR/S	 142.11
U+45E1.0	fTotalStrokes	 17; 16
U+45E1.0	fCangjie	LIMYF
U+45E1.0	fFourCorner	51120
U+45E1.0	fCantonese	jin4; zin1
U+45E1.0	fMandarin	yan1; yan2
U+45E1.0	fEnglish	[1] a kind of insect [2] the name of a place recorded in ancient books
U+45E2.0	fUTF8	䗢
U+45E2.0	fCNS11643	3-5637
U+45E2.0	fGB	5-6222
U+45E2.0	fR/S	 142.11
U+45E2.0	fTotalStrokes	 17
U+45E2.0	fCangjie	LIYHR
U+45E2.0	fFourCorner	54130
U+45E2.0	fCantonese	zou2
U+45E2.0	fMandarin	zao3; suo3
U+45E2.0	fEnglish	same as &#x86A4;
U+45E3.0	fUTF8	䗣
U+45E3.0	fCNS11643	4-5858
U+45E3.0	fGB	3-5746
U+45E3.0	fR/S	 142.11
U+45E3.0	fTotalStrokes	 17; 16
U+45E3.0	fCangjie	TMLII
U+45E3.0	fFourCorner	44130
U+45E3.0	fCantonese	haam4
U+45E3.0	fMandarin	han4; gao3
U+45E3.0	fEnglish	insect of mulberry, insects that damage to the melons
U+45E4.0	fUTF8	䗤
U+45E4.0	fCNS11643	4-584F
U+45E4.0	fGB	3-6273
U+45E4.0	fR/S	 142.11
U+45E4.0	fTotalStrokes	 17; 16
U+45E4.0	fCangjie	LIILB
U+45E4.0	fFourCorner	50120
U+45E4.0	fCantonese	jung4; ngaau5; zung4
U+45E4.0	fMandarin	yong2
U+45E4.0	fEnglish	a kind of worm produced in water
U+45E5.0	fUTF8	䗥
U+45E5.0	fCNS11643	4-5856
U+45E5.0	fGB	3-6160
U+45E5.0	fR/S	 142.11
U+45E5.0	fTotalStrokes	 17; 16
U+45E5.0	fCangjie	LIHOO
U+45E5.0	fFourCorner	58180
U+45E5.0	fHakka	 ciung2
U+45E5.0	fCantonese	zung1
U+45E5.0	fMandarin	cong1
U+45E5.0	fEnglish	a gadgly which deposits its eggs in the skin of cattle
U+45E6.0	fUTF8	䗦
U+45E6.0	fCNS11643	3-5638
U+45E6.0	fR/S	 142.11
U+45E6.0	fTotalStrokes	 17
U+45E6.0	fCangjie	LIYHJ
U+45E6.0	fFourCorner	57130
U+45E6.0	fHakka	 fung1
U+45E6.0	fCantonese	fung1
U+45E6.0	fMandarin	pang2
U+45E6.0	fEnglish	same as &#x8702;
U+45E7.0	fUTF8	䗧
U+45E7.0	fCNS11643	4-584E
U+45E7.0	fGB	5-622F
U+45E7.0	fR/S	 142.11
U+45E7.0	fTotalStrokes	 17; 16
U+45E7.0	fCangjie	LIILE
U+45E7.0	fFourCorner	50130
U+45E7.0	fCantonese	hong1; wong1
U+45E7.0	fMandarin	kang1
U+45E7.0	fEnglish	Libellulidae, a variety of dragonfly
U+45E8.0	fUTF8	䗨
U+45E8.0	fCNS11643	3-5632
U+45E8.0	fR/S	 142.11
U+45E8.0	fTotalStrokes	 17
U+45E8.0	fCangjie	LINWF
U+45E8.0	fFourCorner	57130
U+45E8.0	fCantonese	jyu4; zyu4
U+45E8.0	fMandarin	yu2
U+45E8.0	fEnglish	Lepsima saccharina, a kind of silvery worm that eats clothes, books, etc.
U+45E9.0	fUTF8	䗩
U+45E9.0	fCNS11643	4-5854
U+45E9.0	fGB	3-626F
U+45E9.0	fR/S	 142.11
U+45E9.0	fTotalStrokes	 17; 16
U+45E9.0	fCangjie	LIIHF
U+45E9.0	fFourCorner	53150
U+45E9.0	fCantonese	cik1
U+45E9.0	fMandarin	qi1
U+45E9.0	fEnglish	[1] toad (especially in reference to the one supposed to live on the moon) [2] molluscs; mollusks generally
U+45EA.0	fUTF8	䗪
U+45EA.0	fCNS11643	4-585F
U+45EA.0	fGB	5-3D47
U+45EA.0	fR/S	 142.11; 142.14; 53.14
U+45EA.0	fTotalStrokes	 16; 17
U+45EA.0	fCangjie	ITFI
U+45EA.0	fFourCorner	00230
U+45EA.0	fHakka	 zha5; za5
U+45EA.0	fCantonese	ze3
U+45EA.0	fMandarin	zhe4
U+45EA.0	fEnglish	a sort of grasshopper, woodlouse
U+45EB.0	fUTF8	䗫
U+45EB.0	fCNS11643	3-5635
U+45EB.0	fR/S	 142.11
U+45EB.0	fTotalStrokes	 17
U+45EB.0	fCangjie	LIIDD
U+45EB.0	fFourCorner	50190
U+45EB.0	fCantonese	maa4; maa3
U+45EB.0	fMandarin	ma2
U+45EB.0	fEnglish	same as &#x87C6;
U+45EC.0	fUTF8	䗬
U+45EC.0	fCNS11643	3-563C
U+45EC.0	fR/S	 142.11
U+45EC.0	fTotalStrokes	 17
U+45EC.0	fCangjie	YJLMI
U+45EC.0	fFourCorner	37130
U+45EC.0	fCantonese	fung1
U+45EC.0	fMandarin	feng1
U+45EC.0	fEnglish	same as &#x882D;, &#x8702;
U+45ED.0	fUTF8	䗭
U+45ED.0	fGB	S-236E
U+45ED.0	fR/S	 142.11
U+45ED.0	fTotalStrokes	 17
U+45ED.0	fCangjie	LILLP
U+45ED.0	fCantonese	waan6
U+45EE.0	fUTF8	䗮
U+45EE.0	fCNS11643	3-5631
U+45EE.0	fR/S	 142.11
U+45EE.0	fTotalStrokes	 17
U+45EE.0	fCangjie	LIKKK
U+45EE.0	fFourCorner	54130
U+45EE.0	fCantonese	song2; cong2
U+45EE.0	fMandarin	shuang3
U+45EE.0	fEnglish	a kind of insect
U+45EF.0	fUTF8	䗯
U+45EF.0	fCNS11643	4-5D3B
U+45EF.0	fGB	3-6232
U+45EF.0	fR/S	 142.12
U+45EF.0	fTotalStrokes	 18
U+45EF.0	fCangjie	LIYOG
U+45EF.0	fFourCorner	50130
U+45EF.0	fHakka	 zin5
U+45EF.0	fCantonese	zeon3
U+45EF.0	fMandarin	jin4
U+45EF.0	fEnglish	a shell
U+45F0.0	fUTF8	䗰
U+45F0.0	fCNS11643	5-5E35
U+45F0.0	fR/S	 142.12
U+45F0.0	fTotalStrokes	 18; 17
U+45F0.0	fCangjie	LIWJC
U+45F0.0	fFourCorner	57180
U+45F0.0	fCantonese	gun3; gwun3 (also)
U+45F0.0	fMandarin	guan4
U+45F0.0	fEnglish	[1] spiral shell; conch [2] spiral
U+45F1.0	fUTF8	䗱
U+45F1.0	fCNS11643	4-5D39
U+45F1.0	fGB	3-6322
U+45F1.0	fR/S	 142.12
U+45F1.0	fTotalStrokes	 18
U+45F1.0	fCangjie	LITCO
U+45F1.0	fFourCorner	52130
U+45F1.0	fCantonese	buk6
U+45F1.0	fMandarin	pu2
U+45F1.0	fEnglish	[1] a kind of insect [2] snail family [3] snake family
U+45F2.0	fUTF8	䗲
U+45F2.0	fCNS11643	3-5872
U+45F2.0	fR/S	 142.12
U+45F2.0	fTotalStrokes	 18
U+45F2.0	fCangjie	LIFDQ
U+45F2.0	fFourCorner	59150
U+45F2.0	fHakka	 lin1; nin5; nin6
U+45F2.0	fCantonese	leon6
U+45F2.0	fMandarin	lin4
U+45F2.0	fEnglish	the firefly
U+45F3.0	fUTF8	䗳
U+45F3.0	fR/S	 142.12
U+45F3.0	fTotalStrokes	 18
U+45F3.0	fCangjie	LINOT
U+45F3.0	fCantonese	dang1
U+45F3.0	fEnglish	[Japanese character]
U+45F4.0	fUTF8	䗴
U+45F4.0	fCNS11643	4-6124
U+45F4.0	fGB	3-6324
U+45F4.0	fR/S	 142.13
U+45F4.0	fTotalStrokes	 19
U+45F4.0	fCangjie	LIHNG
U+45F4.0	fFourCorner	58140
U+45F4.0	fCantonese	ting4
U+45F4.0	fMandarin	ting2
U+45F5.0	fUTF8	䗵
U+45F5.0	fCNS11643	5-6941
U+45F5.0	fR/S	 142.13
U+45F5.0	fTotalStrokes	 19
U+45F5.0	fCangjie	LIMWM
U+45F5.0	fFourCorner	51110
U+45F5.0	fHakka	  giong1
U+45F5.0	fCantonese	goeng1
U+45F5.0	fMandarin	jiang1
U+45F5.0	fEnglish	dead silkworms, used as medicine
U+45F6.0	fUTF8	䗶
U+45F6.0	fCNS11643	3-5B2A
U+45F6.0	fR/S	 142.13
U+45F6.0	fTotalStrokes	 19
U+45F6.0	fCangjie	LITAV
U+45F6.0	fFourCorner	54120
U+45F6.0	fHakka	 lap8
U+45F6.0	fCantonese	laap6
U+45F6.0	fMandarin	la4; li4
U+45F6.0	fEnglish	non-classical form of &#x881F;
U+45F7.0	fUTF8	䗷
U+45F7.0	fCNS11643	4-6121
U+45F7.0	fGB	3-632A
U+45F7.0	fR/S	 142.13
U+45F7.0	fTotalStrokes	 19
U+45F7.0	fCangjie	LIYTP
U+45F7.0	fFourCorner	50130
U+45F7.0	fCantonese	ji3; zi3; zik1
U+45F7.0	fMandarin	yi4
U+45F7.0	fEnglish	a kind of wasp
U+45F8.0	fUTF8	䗸
U+45F8.0	fCNS11643	4-6125
U+45F8.0	fGB	3-632B
U+45F8.0	fR/S	 142.13
U+45F8.0	fTotalStrokes	 19
U+45F8.0	fCangjie	YVGI
U+45F8.0	fFourCorner	00130
U+45F8.0	fCantonese	jung1; zung2
U+45F8.0	fMandarin	yong1
U+45F8.0	fEnglish	a kind of insect; a weevil found in rice, etc.
U+45F9.0	fUTF8	䗹
U+45F9.0	fCNS11643	F-636E
U+45F9.0	fGB	5-6242
U+45F9.0	fR/S	 142.13
U+45F9.0	fTotalStrokes	 19
U+45F9.0	fCangjie	LITVP
U+45F9.0	fFourCorner	5413
U+45F9.0	fCantonese	ci3; zi4
U+45F9.0	fMandarin	ci4
U+45F9.0	fEnglish	same as &#x86D3;
U+45FA.0	fUTF8	䗺
U+45FA.0	fCNS11643	5-6D4B
U+45FA.0	fR/S	 142.14; 142.13
U+45FA.0	fTotalStrokes	 20; 19
U+45FA.0	fCangjie	NMLII
U+45FA.0	fFourCorner	12130
U+45FA.0	fCantonese	jin5; zin4
U+45FA.0	fMandarin	yan3
U+45FA.0	fEnglish	[1] creeping; wriggly [2] same as &#x8711;
U+45FB.0	fUTF8	䗻
U+45FB.0	fGB	3-6168
U+45FB.0	fR/S	 142.13
U+45FB.0	fTotalStrokes	 19
U+45FB.0	fCangjie	LIHAL
U+45FB.0	fCantonese	zit3; zik1
U+45FB.0	fMandarin	jie2
U+45FB.0	fEnglish	a kind of sea crab
U+45FC.0	fUTF8	䗼
U+45FC.0	fCNS11643	4-643B
U+45FC.0	fGB	3-632E
U+45FC.0	fR/S	 142.14
U+45FC.0	fTotalStrokes	 20
U+45FC.0	fCangjie	LIHJF
U+45FC.0	fFourCorner	52130
U+45FC.0	fCantonese	fan1; faan1; waat1
U+45FC.0	fMandarin	xun1
U+45FC.0	fEnglish	a kind of insect
U+45FD.0	fUTF8	䗽
U+45FD.0	fCNS11643	4-643C
U+45FD.0	fGB	3-627A
U+45FD.0	fR/S	 142.14
U+45FD.0	fTotalStrokes	 20
U+45FD.0	fCangjie	VGLMI
U+45FD.0	fFourCorner	20130
U+45FD.0	fCantonese	wai6
U+45FD.0	fMandarin	wei4
U+45FD.0	fEnglish	a small gadfly that suck blood of horse or cow; weevil found in rice, etc.
U+45FE.0	fUTF8	䗾
U+45FE.0	fCNS11643	4-6440
U+45FE.0	fGB	5-6245
U+45FE.0	fR/S	 142.14
U+45FE.0	fTotalStrokes	 20
U+45FE.0	fCangjie	LIAVF
U+45FE.0	fFourCorner	56130
U+45FE.0	fCantonese	hin2
U+45FE.0	fMandarin	xian3
U+45FE.0	fEnglish	same as &#x8706;
U+45FF.0	fUTF8	䗿
U+45FF.0	fCNS11643	4-643E
U+45FF.0	fGB	3-6177
U+45FF.0	fR/S	 142.14
U+45FF.0	fTotalStrokes	 20
U+45FF.0	fCangjie	LIJPN
U+45FF.0	fFourCorner	53120
U+45FF.0	fCantonese	ning4; ling4 (also)
U+45FF.0	fMandarin	ning3; ning2
U+45FF.0	fEnglish	a kind of insect; mole cricket
U+4600.0	fUTF8	䘀
U+4600.0	fCNS11643	4-6441
U+4600.0	fR/S	 142.14
U+4600.0	fTotalStrokes	 20
U+4600.0	fCangjie	HJLII
U+4600.0	fFourCorner	27130
U+4600.0	fHakka	 fu5; feu5; fu3
U+4600.0	fCantonese	fau6; faau6; waai3
U+4600.0	fMandarin	fu4
U+4600.0	fEnglish	grasshopper
U+4601.0	fUTF8	䘁
U+4601.0	fCNS11643	3-5D26
U+4601.0	fGB	5-6243
U+4601.0	fR/S	 142.14
U+4601.0	fTotalStrokes	 20
U+4601.0	fCangjie	JGLMI
U+4601.0	fFourCorner	43130
U+4601.0	fCantonese	zit3; zit6
U+4601.0	fMandarin	jie2; ge2; ke3
U+4601.0	fEnglish	same as &#x883D;
U+4602.0	fUTF8	䘂
U+4602.0	fCNS11643	3-5E5E
U+4602.0	fR/S	 142.15
U+4602.0	fTotalStrokes	 21
U+4602.0	fCangjie	LIJIM
U+4602.0	fFourCorner	53150
U+4602.0	fCantonese	zim1
U+4602.0	fMandarin	jian1
U+4602.0	fEnglish	same as &#x460B;
U+4603.0	fUTF8	䘃
U+4603.0	fCNS11643	4-6730
U+4603.0	fGB	3-6330
U+4603.0	fR/S	 142.15
U+4603.0	fTotalStrokes	 21
U+4603.0	fCangjie	LIWFG
U+4603.0	fFourCorner	56110
U+4603.0	fCantonese	mak6; maak6; mik6; mong4
U+4603.0	fMandarin	mo4
U+4603.0	fEnglish	[1] a kind of crab [2] same as &#x87D4;
U+4604.0	fUTF8	䘄
U+4604.0	fCNS11643	4-643D
U+4604.0	fGB	5-6244
U+4604.0	fR/S	 142.15
U+4604.0	fTotalStrokes	 20; 21
U+4604.0	fCangjie	JALII
U+4604.0	fFourCorner	44130
U+4604.0	fCantonese	zyu3
U+4604.0	fMandarin	fu4; zhu4
U+4604.0	fEnglish	cicada famiy; a poisonous insect
U+4605.0	fUTF8	䘅
U+4605.0	fCNS11643	4-693E
U+4605.0	fGB	5-6251
U+4605.0	fR/S	 142.16
U+4605.0	fTotalStrokes	 22; 21
U+4605.0	fCangjie	IPLII
U+4605.0	fFourCorner	21130
U+4605.0	fCantonese	noi6; loi6 (also); naang4
U+4605.0	fMandarin	neng3; nai2; nai4
U+4605.0	fEnglish	[1] same as &#x879A; [2] non-classical form of &#x80FD;
U+4606.0	fUTF8	䘆
U+4606.0	fCNS11643	3-5F5E
U+4606.0	fR/S	 142.16
U+4606.0	fTotalStrokes	 22
U+4606.0	fCangjie	LIJQP
U+4606.0	fFourCorner	53130
U+4606.0	fHakka	 hien3
U+4606.0	fCantonese	hyun2; hin2 (also)
U+4606.0	fMandarin	xin3; xian3
U+4606.0	fEnglish	 worm; earth-worm
U+4607.0	fUTF8	䘇
U+4607.0	fCNS11643	3-5F60
U+4607.0	fR/S	 142.16
U+4607.0	fTotalStrokes	 22
U+4607.0	fCangjie	YKLII
U+4607.0	fFourCorner	00130
U+4607.0	fCantonese	man1
U+4607.0	fMandarin	wen2; wen4
U+4607.0	fEnglish	same as &#x868A;
U+4608.0	fUTF8	䘈
U+4608.0	fCNS11643	3-6044
U+4608.0	fR/S	 142.17; 142.16
U+4608.0	fTotalStrokes	 23; 22
U+4608.0	fCangjie	LIMBS
U+4608.0	fFourCorner	51120
U+4608.0	fCantonese	lai6; laai6; lit6
U+4608.0	fMandarin	li4
U+4608.0	fEnglish	corrupted form of &#x8823;
U+4609.0	fUTF8	䘉
U+4609.0	fCNS11643	7-5925
U+4609.0	fR/S	 142.16
U+4609.0	fTotalStrokes	 22
U+4609.0	fCangjie	MKLII
U+4609.0	fFourCorner	20130
U+4609.0	fCantonese	caam4
U+4609.0	fMandarin	can2
U+4609.0	fEnglish	standard form of &#x8836;
U+460A.0	fUTF8	䘊
U+460A.0	fCNS11643	7-5C5E
U+460A.0	fR/S	 142.17
U+460A.0	fTotalStrokes	 23
U+460A.0	fCangjie	LIHWI
U+460A.0	fFourCorner	58150
U+460A.0	fCantonese	mit6
U+460A.0	fMandarin	mie4
U+460A.0	fEnglish	flies; small insects produced in damp places; a kind of gnat
U+460B.0	fUTF8	䘋
U+460B.0	fCNS11643	4-6A78
U+460B.0	fGB	5-6254
U+460B.0	fR/S	 142.17
U+460B.0	fTotalStrokes	 23
U+460B.0	fCangjie	LIOIM
U+460B.0	fFourCorner	53150
U+460B.0	fCantonese	zim1
U+460B.0	fMandarin	jian1
U+460B.0	fEnglish	a kind of insect
U+460C.0	fUTF8	䘌
U+460C.0	fCNS11643	7-5C63
U+460C.0	fR/S	 142.17
U+460C.0	fTotalStrokes	 23
U+460C.0	fCangjie	SRLII
U+460C.0	fFourCorner	71130
U+460C.0	fCantonese	nik1; lik1 (also)
U+460C.0	fMandarin	ni4
U+460C.0	fEnglish	gadfly
U+460D.0	fUTF8	䘍
U+460D.0	fCNS11643	3-6148
U+460D.0	fR/S	 142.19
U+460D.0	fTotalStrokes	 25
U+460D.0	fCangjie	TWBI
U+460D.0	fFourCorner	44130
U+460D.0	fCantonese	caai3
U+460D.0	fMandarin	chai4
U+460D.0	fEnglish	same as &#x8806;
U+460E.0	fUTF8	䘎
U+460E.0	fCNS11643	4-6E39
U+460E.0	fGB	3-6257
U+460E.0	fR/S	 142.22
U+460E.0	fTotalStrokes	 28
U+460E.0	fCangjie	LIVFN
U+460E.0	fFourCorner	52120
U+460E.0	fCantonese	waan1; wan1; wan6
U+460E.0	fMandarin	wan1
U+460E.0	fEnglish	the name of an insect
U+460F.0	fUTF8	䘏
U+460F.0	fCNS11643	3-2F25
U+460F.0	fR/S	 143.3
U+460F.0	fTotalStrokes	 9
U+460F.0	fCangjie	HTNL
U+460F.0	fFourCorner	27120
U+460F.0	fCantonese	seot1
U+460F.0	fMandarin	xu4
U+460F.0	fEnglish	same as &#x6064;
U+4610.0	fUTF8	䘐
U+4610.0	fCNS11643	6-3B73
U+4610.0	fR/S	 143.4
U+4610.0	fTotalStrokes	 10; 9
U+4610.0	fCangjie	HTSHC
U+4610.0	fFourCorner	23100
U+4610.0	fCantonese	nuk6; luk6 (also)
U+4610.0	fMandarin	nuu4; n4
U+4610.0	fEnglish	same as standard form &#x8844;
U+4611.0	fUTF8	䘑
U+4611.0	fCNS11643	3-3869
U+4611.0	fR/S	 143.5
U+4611.0	fTotalStrokes	 11
U+4611.0	fCangjie	HTINE
U+4611.0	fFourCorner	23130
U+4611.0	fHakka	 mak7
U+4611.0	fCantonese	mak6
U+4611.0	fMandarin	mo4; mai4
U+4611.0	fEnglish	[1] veins or arteries [2] the pulse [3] circulation of the blood
U+4612.0	fUTF8	䘒
U+4612.0	fCNS11643	3-444C
U+4612.0	fGB	5-6467
U+4612.0	fR/S	 143.7
U+4612.0	fTotalStrokes	 13
U+4612.0	fCangjie	HTICE
U+4612.0	fFourCorner	23140
U+4612.0	fCantonese	zeoi1; zyun1; zeon1 (also); syun1 (also); co4
U+4612.0	fMandarin	zui1; zui3; zuo1
U+4612.0	fEnglish	[1] boy's reproductive organs [2] fleece; cut down; scrape; pare [3] enfeeble
U+4613.0	fUTF8	䘓
U+4613.0	fCNS11643	4-4723
U+4613.0	fGB	3-654E
U+4613.0	fR/S	 143.8
U+4613.0	fTotalStrokes	 14
U+4613.0	fCangjie	HTNHX
U+4613.0	fFourCorner	27170
U+4613.0	fCantonese	ham3
U+4613.0	fMandarin	kan4; dan4; kao4
U+4613.0	fEnglish	thick blood of cattle and goat
U+4614.0	fUTF8	䘔
U+4614.0	fCNS11643	4-4D37
U+4614.0	fGB	5-6469
U+4614.0	fR/S	 143.9
U+4614.0	fTotalStrokes	 15
U+4614.0	fCangjie	HTJHR
U+4614.0	fFourCorner	23160
U+4614.0	fCantonese	kaak3; ho2; kaa1
U+4614.0	fMandarin	ke4
U+4614.0	fEnglish	same as &#x5580;
U+4615.0	fUTF8	䘕
U+4615.0	fCNS11643	4-2F6F
U+4615.0	fGB	3-397E
U+4615.0	fR/S	 144.4
U+4615.0	fTotalStrokes	 10
U+4615.0	fCangjie	HOYNN
U+4615.0	fFourCorner	21210
U+4615.0	fHakka	 hong2
U+4615.0	fCantonese	hong4
U+4615.0	fMandarin	hang2
U+4615.0	fEnglish	[1] prostitute [2] trade; occupation; profession [3] musicians
U+4616.0	fUTF8	䘖
U+4616.0	fCNS11643	4-3A78
U+4616.0	fR/S	 144.6
U+4616.0	fTotalStrokes	 12
U+4616.0	fCangjie	HOOUN
U+4616.0	fFourCorner	21774
U+4616.0	fCantonese	haam4
U+4617.0	fUTF8	䘗
U+4617.0	fCNS11643	3-4E5E
U+4617.0	fR/S	 144.10
U+4617.0	fTotalStrokes	 16; 15
U+4617.0	fCangjie	HOUFN
U+4617.0	fFourCorner	21903
U+4617.0	fCantonese	fai1; faai1; gwaan2; pui3
U+4617.0	fMandarin	hui1
U+4617.0	fEnglish	[1] beautiful [2] exquisite; fine
U+4618.0	fUTF8	䘘
U+4618.0	fCNS11643	4-534A
U+4618.0	fGB	5-3C4F
U+4618.0	fR/S	 144.10
U+4618.0	fTotalStrokes	 16
U+4618.0	fCangjie	HOQFN
U+4618.0	fFourCorner	21903
U+4618.0	fCantonese	jyu6; zyu6
U+4618.0	fMandarin	yu3; yu4; su4
U+4618.0	fEnglish	same as &#x5FA1;
U+4619.0	fUTF8	䘙
U+4619.0	fCNS11643	3-5B2E
U+4619.0	fR/S	 144.14
U+4619.0	fTotalStrokes	 20; 19
U+4619.0	fCangjie	HODBN
U+4619.0	fFourCorner	21220
U+4619.0	fCantonese	wai6
U+4619.0	fMandarin	wei4
U+4619.0	fEnglish	same as &#x885B;
U+461A.0	fUTF8	䘚
U+461A.0	fCNS11643	3-2A6E
U+461A.0	fR/S	 145.2
U+461A.0	fTotalStrokes	 8
U+461A.0	fCangjie	YHVJ
U+461A.0	fFourCorner	00400
U+461A.0	fHakka	 zut7;  cut7
U+461A.0	fCantonese	zeot1
U+461A.0	fMandarin	zu2; zhu2
U+461A.0	fEnglish	[1] underling; lictor; soldier [2] die [3] finally
U+461B.0	fUTF8	䘛
U+461B.0	fGB	S-2275
U+461B.0	fR/S	 145.2
U+461B.0	fTotalStrokes	 8
U+461B.0	fCangjie	LHN
U+461B.0	fCantonese	gei1
U+461C.0	fUTF8	䘜
U+461C.0	fCNS11643	3-2F28
U+461C.0	fGB	5-6541
U+461C.0	fR/S	 145.3
U+461C.0	fTotalStrokes	 9
U+461C.0	fCangjie	LLB
U+461C.0	fFourCorner	34220
U+461C.0	fCantonese	gan1
U+461C.0	fMandarin	jin1
U+461C.0	fEnglish	same as standard form &#x5DFE;
U+461D.0	fUTF8	䘝
U+461D.0	fCNS11643	4-2B46
U+461D.0	fGB	3-6628
U+461D.0	fR/S	 145.3
U+461D.0	fTotalStrokes	 9; 8
U+461D.0	fCangjie	LIP
U+461D.0	fFourCorner	33240
U+461D.0	fCantonese	jik6; zik6
U+461D.0	fMandarin	yi4
U+461D.0	fEnglish	[1] shirt; garment; gown [2] black dress
U+461E.0	fUTF8	䘞
U+461E.0	fGB	S-227A
U+461E.0	fR/S	 145.3
U+461E.0	fTotalStrokes	 9
U+461E.0	fCangjie	LNVM
U+461E.0	fCantonese	maa6
U+461F.0	fUTF8	䘟
U+461F.0	fCNS11643	5-303D
U+461F.0	fGB	3-6638
U+461F.0	fR/S	 145.4
U+461F.0	fTotalStrokes	 10; 9
U+461F.0	fCangjie	LNL
U+461F.0	fFourCorner	37220
U+461F.0	fCantonese	diu1
U+461F.0	fMandarin	diao1
U+461F.0	fEnglish	kind of silk woven with double threads and water proof; lining the coffin, leather lining of clothing
U+4620.0	fUTF8	䘠
U+4620.0	fCNS11643	4-2F72
U+4620.0	fR/S	 145.4
U+4620.0	fTotalStrokes	 10; 9
U+4620.0	fCangjie	LKK
U+4620.0	fFourCorner	34240
U+4620.0	fCantonese	fat1; faat1; kut3
U+4620.0	fMandarin	bo1; fu2; po4
U+4620.0	fEnglish	[1] corrupted form of &#x889A; [2] same as &#x5E17;
U+4621.0	fUTF8	䘡
U+4621.0	fCNS11643	5-3039
U+4621.0	fR/S	 145.4
U+4621.0	fTotalStrokes	 10
U+4621.0	fCangjie	PPYHV
U+4621.0	fFourCorner	21730
U+4621.0	fCantonese	bei6; bei2
U+4621.0	fMandarin	bi3
U+4621.0	fEnglish	 sleeve
U+4622.0	fUTF8	䘢
U+4622.0	fCNS11643	4-352A
U+4622.0	fGB	3-6636
U+4622.0	fR/S	 145.5
U+4622.0	fTotalStrokes	 11; 10
U+4622.0	fCangjie	LJMN
U+4622.0	fFourCorner	33220
U+4622.0	fCantonese	zyu2
U+4622.0	fMandarin	zhu3
U+4622.0	fEnglish	old and shabby (said of clothes), to fill in or to pack clothes with cotton, to remain in a standstill for a long time
U+4623.0	fUTF8	䘣
U+4623.0	fCNS11643	4-3A7A
U+4623.0	fGB	3-6658
U+4623.0	fR/S	 145.5; 145.6
U+4623.0	fTotalStrokes	 12; 11
U+4623.0	fCangjie	LYMP
U+4623.0	fFourCorner	31210
U+4623.0	fCantonese	zi2
U+4623.0	fMandarin	zhi3; zi3
U+4623.0	fEnglish	clothes that have no lining, stitches in the clothes, wrinkled; crinkled clothes, cotton clothes
U+4624.0	fUTF8	䘤
U+4624.0	fCNS11643	4-352D
U+4624.0	fGB	3-663B
U+4624.0	fR/S	 145.5
U+4624.0	fTotalStrokes	 11; 10
U+4624.0	fCangjie	LIJC
U+4624.0	fFourCorner	33290
U+4624.0	fCantonese	seot6
U+4624.0	fMandarin	shu4
U+4624.0	fEnglish	a sword covering to protect the scabbard
U+4625.0	fUTF8	䘥
U+4625.0	fCNS11643	4-352E
U+4625.0	fGB	3-6642
U+4625.0	fR/S	 145.5
U+4625.0	fTotalStrokes	 11; 10
U+4625.0	fCangjie	LWL
U+4625.0	fFourCorner	36250
U+4625.0	fCantonese	haap6
U+4625.0	fMandarin	jia3; xia2
U+4625.0	fEnglish	the front of a Chinese gown, lapel of a Chinese dress-worn by the literati in former days, a short coat; a short top garment, neck-wears for babies, very fine silk fabric
U+4626.0	fUTF8	䘦
U+4626.0	fCNS11643	4-352C
U+4626.0	fGB	3-664D
U+4626.0	fR/S	 145.5
U+4626.0	fTotalStrokes	 11; 10
U+4626.0	fCangjie	LSP
U+4626.0	fFourCorner	37210
U+4626.0	fCantonese	nai4; lai4 (also); fai3; nei4; zyu4
U+4626.0	fMandarin	ni2; ni3
U+4626.0	fEnglish	[1] a kind of head-dress wore in funeral rites in old days [2] fine clothes
U+4627.0	fUTF8	䘧
U+4627.0	fCNS11643	F-3251
U+4627.0	fGB	S-2276
U+4627.0	fR/S	 145.5
U+4627.0	fTotalStrokes	 11
U+4627.0	fCangjie	LLBK
U+4627.0	fCantonese	joeng1
U+4628.0	fUTF8	䘨
U+4628.0	fCNS11643	4-3A79
U+4628.0	fGB	3-665E
U+4628.0	fR/S	 145.6
U+4628.0	fTotalStrokes	 12; 11
U+4628.0	fCangjie	LYCK
U+4628.0	fFourCorner	30240
U+4628.0	fHakka	 gau3; gau5
U+4628.0	fCantonese	giu2; gaau1; gau2
U+4628.0	fMandarin	jiao3
U+4628.0	fEnglish	leathern trousers
U+4629.0	fUTF8	䘩
U+4629.0	fCNS11643	3-3E71
U+4629.0	fGB	5-6553
U+4629.0	fR/S	 145.6
U+4629.0	fTotalStrokes	 12; 11
U+4629.0	fCangjie	LPA
U+4629.0	fFourCorner	37220
U+4629.0	fCantonese	seon1; ceon4; jyun6
U+4629.0	fMandarin	xun2; xuan4
U+4629.0	fEnglish	[1] collar [2] a throat-band to hold the hat [3] tassel; fringe [4] same as &#x88A8;
U+462A.0	fUTF8	䘪
U+462A.0	fCNS11643	4-3530
U+462A.0	fGB	3-665F
U+462A.0	fR/S	 145.6; 145.5
U+462A.0	fTotalStrokes	 11; 12; 10
U+462A.0	fCangjie	LYIU
U+462A.0	fFourCorner	30210
U+462A.0	fCantonese	cung1
U+462A.0	fMandarin	chong1
U+462A.0	fEnglish	[1] clothes without hem; ragged garments; a garment without a lining [2] single
U+462B.0	fUTF8	䘫
U+462B.0	fCNS11643	3-3E6E
U+462B.0	fR/S	 145.6
U+462B.0	fTotalStrokes	 12
U+462B.0	fCangjie	VRYHV
U+462B.0	fFourCorner	46730
U+462B.0	fHakka	 ji2; ri2
U+462B.0	fCantonese	jyu4; zyu4
U+462B.0	fMandarin	ru2; nou4
U+462B.0	fEnglish	rags; old clothes
U+462C.0	fUTF8	䘬
U+462C.0	fCNS11643	3-3E6C
U+462C.0	fR/S	 145.6
U+462C.0	fTotalStrokes	 12; 11
U+462C.0	fCangjie	LIJ
U+462C.0	fFourCorner	33250
U+462C.0	fCantonese	jung4; zung4
U+462C.0	fMandarin	rong2
U+462C.0	fEnglish	[1] armour [2] military dress
U+462D.0	fUTF8	䘭
U+462D.0	fCNS11643	3-3E6B
U+462D.0	fR/S	 145.6
U+462D.0	fTotalStrokes	 12; 11
U+462D.0	fCangjie	LMIG
U+462D.0	fFourCorner	31210
U+462D.0	fCantonese	zi3; sak3; si6; taai4
U+462D.0	fMandarin	zhi4; die2
U+462D.0	fEnglish	the pleates on the lapel of a garment of thin silk
U+462E.0	fUTF8	䘮
U+462E.0	fCNS11643	3-2F4A
U+462E.0	fR/S	 145.6; 9.8
U+462E.0	fTotalStrokes	 10; 12
U+462E.0	fCangjie	GOOV
U+462E.0	fFourCorner	40730
U+462E.0	fCantonese	song1; song3
U+462E.0	fMandarin	sang1; sa4; sang4; xi4
U+462E.0	fEnglish	same as &#x55AA;
U+462F.0	fUTF8	䘯
U+462F.0	fCNS11643	4-4062
U+462F.0	fGB	3-6668
U+462F.0	fR/S	 145.7
U+462F.0	fTotalStrokes	 13; 12
U+462F.0	fCangjie	LFB
U+462F.0	fFourCorner	39220
U+462F.0	fHakka	  sau1
U+462F.0	fCantonese	saau1; sau1; sok3
U+462F.0	fMandarin	shao1
U+462F.0	fEnglish	[1] the lapel of a coat [2] the waistband of trousers
U+4630.0	fUTF8	䘰
U+4630.0	fCNS11643	4-4725
U+4630.0	fGB	3-665B
U+4630.0	fR/S	 145.8; 145.7
U+4630.0	fTotalStrokes	 14; 13
U+4630.0	fCangjie	LNKM
U+4630.0	fFourCorner	32240
U+4630.0	fCantonese	sin1
U+4630.0	fMandarin	shan1
U+4630.0	fEnglish	a hood; some kind of covering used on a carriage, towel; kerchief
U+4631.0	fUTF8	䘱
U+4631.0	fCNS11643	6-582B
U+4631.0	fR/S	 145.7
U+4631.0	fTotalStrokes	 13
U+4631.0	fCangjie	YCORV
U+4631.0	fFourCorner	00730
U+4631.0	fCantonese	jyu6; zyu6
U+4631.0	fMandarin	yu4
U+4631.0	fEnglish	same as &#x88D5;
U+4632.0	fUTF8	䘲
U+4632.0	fGB	S-227C
U+4632.0	fR/S	 145.7
U+4632.0	fTotalStrokes	 13
U+4632.0	fCangjie	LSME
U+4632.0	fCantonese	zam1
U+4633.0	fUTF8	䘳
U+4633.0	fCNS11643	3-4953
U+4633.0	fGB	5-6567
U+4633.0	fR/S	 145.8
U+4633.0	fTotalStrokes	 14; 13
U+4633.0	fCangjie	LC
U+4633.0	fFourCorner	38210
U+4633.0	fCantonese	kam1; gam1 (also); kaam1
U+4633.0	fMandarin	jin1
U+4633.0	fEnglish	interchangeable with &#x895F;
U+4634.0	fUTF8	䘴
U+4634.0	fCNS11643	4-472D
U+4634.0	fGB	5-655F
U+4634.0	fR/S	 145.8
U+4634.0	fTotalStrokes	 14; 13
U+4634.0	fCangjie	LDCI
U+4634.0	fFourCorner	38230
U+4634.0	fCantonese	zung1
U+4634.0	fMandarin	cong3
U+4634.0	fEnglish	same as &#x8873;
U+4635.0	fUTF8	䘵
U+4635.0	fCNS11643	5-496C
U+4635.0	fR/S	 145.8
U+4635.0	fTotalStrokes	 14; 13
U+4635.0	fCangjie	LVNE
U+4635.0	fFourCorner	37230
U+4635.0	fCantonese	luk6
U+4635.0	fMandarin	lu4
U+4635.0	fEnglish	hissing sound of the clothes
U+4636.0	fUTF8	䘶
U+4636.0	fCNS11643	5-496D
U+4636.0	fGB	3-667A
U+4636.0	fR/S	 145.8
U+4636.0	fTotalStrokes	 14; 13
U+4636.0	fCangjie	LUNE
U+4636.0	fFourCorner	37270
U+4636.0	fHakka	 ham2
U+4636.0	fCantonese	haam4
U+4636.0	fMandarin	han2; han4; yan3
U+4636.0	fEnglish	sleeve
U+4637.0	fUTF8	䘷
U+4637.0	fCNS11643	4-4066
U+4637.0	fGB	3-6669
U+4637.0	fR/S	 145.8
U+4637.0	fTotalStrokes	 13; 14
U+4637.0	fCangjie	LFBF
U+4637.0	fFourCorner	39220
U+4637.0	fCantonese	bit3
U+4637.0	fMandarin	bie4
U+4637.0	fEnglish	sleeves, old and meanly clothes
U+4638.0	fUTF8	䘸
U+4638.0	fCNS11643	4-4731
U+4638.0	fGB	3-6671
U+4638.0	fR/S	 145.8
U+4638.0	fTotalStrokes	 14; 13
U+4638.0	fCangjie	LYOK
U+4638.0	fFourCorner	30240
U+4638.0	fHakka	 jit8; rit8
U+4638.0	fCantonese	jik6
U+4638.0	fMandarin	zhi1; yi4
U+4638.0	fEnglish	the part of the dress under the arms
U+4639.0	fUTF8	䘹
U+4639.0	fCNS11643	4-4727
U+4639.0	fGB	3-6672
U+4639.0	fR/S	 145.8
U+4639.0	fTotalStrokes	 14; 13
U+4639.0	fCangjie	LYOJ
U+4639.0	fFourCorner	30240
U+4639.0	fCantonese	zeoi3; ceoi3
U+4639.0	fMandarin	zui4
U+4639.0	fEnglish	clothes that have no lining (usually for summer wear)
U+463A.0	fUTF8	䘺
U+463A.0	fCNS11643	3-4955
U+463A.0	fGB	5-656C
U+463A.0	fR/S	 145.8
U+463A.0	fTotalStrokes	 14; 13
U+463A.0	fCangjie	LJMO
U+463A.0	fFourCorner	33280
U+463A.0	fHakka	 can5; tien6; tien5; tien3
U+463A.0	fCantonese	zaan6
U+463A.0	fMandarin	zhan4
U+463A.0	fEnglish	a seam which has come undone
U+463B.0	fUTF8	䘻
U+463B.0	fCNS11643	4-4060
U+463B.0	fGB	3-6673
U+463B.0	fR/S	 145.8; 145.7
U+463B.0	fTotalStrokes	 13; 14; 12
U+463B.0	fCangjie	LYIB
U+463B.0	fFourCorner	30220
U+463B.0	fCantonese	juk6; zuk6
U+463B.0	fMandarin	yu4; su4
U+463B.0	fEnglish	curtain on a carriage or car
U+463C.0	fUTF8	䘼
U+463C.0	fCNS11643	4-472A
U+463C.0	fGB	3-6677
U+463C.0	fR/S	 145.8
U+463C.0	fTotalStrokes	 14; 13
U+463C.0	fCangjie	LJNU
U+463C.0	fFourCorner	33210
U+463C.0	fCantonese	jyun2; wun2
U+463C.0	fMandarin	wan3
U+463C.0	fEnglish	[1] stockings; socks [2] sleeve
U+463D.0	fUTF8	䘽
U+463D.0	fCNS11643	4-472E
U+463D.0	fGB	3-666F
U+463D.0	fR/S	 145.8
U+463D.0	fTotalStrokes	 14; 13
U+463D.0	fCangjie	LHXU
U+463D.0	fFourCorner	37210
U+463D.0	fCantonese	ngai4; ai4 (also); ngaai4
U+463D.0	fMandarin	ni2; ni3; ni4
U+463D.0	fEnglish	clothing and ornaments; an ornamental item; a decorative artical of a garment
U+463E.0	fUTF8	䘾
U+463E.0	fCNS11643	4-4726
U+463E.0	fGB	3-6676
U+463E.0	fR/S	 145.8
U+463E.0	fTotalStrokes	 14; 13
U+463E.0	fCangjie	LJRR
U+463E.0	fFourCorner	33270
U+463E.0	fCantonese	gun2; gwun2 (also); kong3
U+463E.0	fMandarin	guan3
U+463E.0	fEnglish	[1] legs of the trousers [2] trousers, fur pants
U+463F.0	fUTF8	䘿
U+463F.0	fCNS11643	4-472B
U+463F.0	fGB	3-6678
U+463F.0	fR/S	 145.8
U+463F.0	fTotalStrokes	 14; 13
U+463F.0	fCangjie	LSUU
U+463F.0	fFourCorner	37270
U+463F.0	fCantonese	gwat6; faat1; gwaat6
U+463F.0	fMandarin	jue2
U+463F.0	fEnglish	shirt or a jacket without any decorated hem
U+4640.0	fUTF8	䙀
U+4640.0	fCNS11643	3-495A
U+4640.0	fR/S	 145.8
U+4640.0	fTotalStrokes	 14
U+4640.0	fCangjie	LBB
U+4640.0	fFourCorner	37220
U+4640.0	fCantonese	bang1
U+4640.0	fMandarin	beng1; beng3
U+4640.0	fEnglish	same as &#x7DB3, &#x7E43;
U+4641.0	fUTF8	䙁
U+4641.0	fCNS11643	3-4950
U+4641.0	fGB	5-6548
U+4641.0	fR/S	 145.8
U+4641.0	fTotalStrokes	 14; 13
U+4641.0	fCangjie	LII
U+4641.0	fFourCorner	33250
U+4641.0	fCantonese	zin1; can1
U+4641.0	fMandarin	can2
U+4641.0	fEnglish	[1] children's garments [2] a mat for wrapping garments
U+4642.0	fUTF8	䙂
U+4642.0	fCNS11643	3-495B
U+4642.0	fGB	5-6569
U+4642.0	fR/S	 145.8
U+4642.0	fTotalStrokes	 14; 13
U+4642.0	fCangjie	LCIP
U+4642.0	fFourCorner	38230
U+4642.0	fCantonese	zung1
U+4642.0	fMandarin	chong2
U+4642.0	fEnglish	non-classical form of &#x8873;
U+4643.0	fUTF8	䙃
U+4643.0	fCNS11643	4-4D3C
U+4643.0	fGB	3-667E
U+4643.0	fR/S	 145.9
U+4643.0	fTotalStrokes	 15; 14
U+4643.0	fCangjie	LKMB
U+4643.0	fFourCorner	34220
U+4643.0	fHakka	 to5; to6
U+4643.0	fCantonese	do6
U+4643.0	fMandarin	duo4; kuo4; pan2; ruan2
U+4643.0	fEnglish	 long robe
U+4644.0	fUTF8	䙄
U+4644.0	fCNS11643	4-4D3F
U+4644.0	fGB	3-6725
U+4644.0	fR/S	 145.9
U+4644.0	fTotalStrokes	 15; 14
U+4644.0	fCangjie	LHJX
U+4644.0	fFourCorner	32270
U+4644.0	fHakka	 cap7
U+4644.0	fCantonese	cap1
U+4644.0	fMandarin	qi4; zha3
U+4644.0	fEnglish	[1] a double hem or border ona arobe [2] bind the loins
U+4645.0	fUTF8	䙅
U+4645.0	fCNS11643	4-4D3A
U+4645.0	fGB	3-667D
U+4645.0	fR/S	 145.9
U+4645.0	fTotalStrokes	 14; 15
U+4645.0	fCangjie	LMWV
U+4645.0	fFourCorner	31240
U+4645.0	fHakka	 jau1; rau1
U+4645.0	fCantonese	jiu1; ziu1; ziu2
U+4645.0	fMandarin	yao1; yao4
U+4645.0	fEnglish	 fold; plait
U+4646.0	fUTF8	䙆
U+4646.0	fCNS11643	4-4D39
U+4646.0	fGB	3-672C
U+4646.0	fR/S	 145.9
U+4646.0	fTotalStrokes	 15; 14
U+4646.0	fCangjie	LNOK
U+4646.0	fFourCorner	32230
U+4646.0	fCantonese	wai3
U+4646.0	fMandarin	gui4; kui4
U+4646.0	fEnglish	slits on the lower part of a gown for freedom of movement
U+4647.0	fUTF8	䙇
U+4647.0	fCNS11643	4-4D42
U+4647.0	fGB	3-6721
U+4647.0	fR/S	 145.9
U+4647.0	fTotalStrokes	 15; 14
U+4647.0	fCangjie	LMBK
U+4647.0	fFourCorner	31230
U+4647.0	fHakka	 ngion1
U+4647.0	fCantonese	jyun4
U+4647.0	fMandarin	ruan2; nuan3
U+4647.0	fEnglish	[1] seam; border [2] binding [3] selvedge
U+4648.0	fUTF8	䙈
U+4648.0	fCNS11643	4-4D40
U+4648.0	fGB	5-6576
U+4648.0	fR/S	 145.9
U+4648.0	fTotalStrokes	 15; 14
U+4648.0	fCangjie	LONK
U+4648.0	fFourCorner	37230
U+4648.0	fCantonese	hau4
U+4648.0	fMandarin	hou2
U+4648.0	fEnglish	a loose garment or cloak, shirt; jacket
U+4649.0	fUTF8	䙉
U+4649.0	fCNS11643	4-4D47
U+4649.0	fGB	3-6728
U+4649.0	fR/S	 145.9
U+4649.0	fTotalStrokes	 15
U+4649.0	fCangjie	LHJU
U+4649.0	fFourCorner	32260
U+4649.0	fCantonese	ceon4
U+4649.0	fMandarin	xun2; zan3
U+4649.0	fEnglish	[1] clothes [2] the central suture; seams on the back (said of clothes)
U+464A.0	fUTF8	䙊
U+464A.0	fGB	S-2277
U+464A.0	fR/S	 145.9
U+464A.0	fTotalStrokes	 15
U+464A.0	fCangjie	LTJB
U+464A.0	fCantonese	ze6
U+464A.0	fMandarin	xie4
U+464A.0	fEnglish	arid; dry; parched
U+464B.0	fUTF8	䙋
U+464B.0	fGB	S-2278
U+464B.0	fR/S	 145.9
U+464B.0	fTotalStrokes	 15
U+464B.0	fCangjie	LJMM
U+464B.0	fCantonese	syun1
U+464C.0	fUTF8	䙌
U+464C.0	fGB	7-2251
U+464C.0	fR/S	 145.9
U+464C.0	fTotalStrokes	 15
U+464C.0	fCangjie	LLMO
U+464C.0	fCantonese	gui2; kui2
U+464C.0	fMandarin	hui4; kui4
U+464C.0	fEnglish	[1] (simplified form) a knot; a button, (dialect) to tie up; fastening; to wear; to slip on, to make a knot by a rope; a piece of string or a ribbon [2] same as &#x7E6A;
U+464D.0	fUTF8	䙍
U+464D.0	fR/S	 145.9
U+464D.0	fTotalStrokes	 15
U+464D.0	fCangjie	LNOO
U+464D.0	fCantonese	fei1
U+464D.0	fEnglish	[Korean character]
U+464E.0	fUTF8	䙎
U+464E.0	fCNS11643	4-5352
U+464E.0	fGB	3-6732
U+464E.0	fR/S	 145.10
U+464E.0	fTotalStrokes	 16; 15
U+464E.0	fCangjie	LBVK
U+464E.0	fFourCorner	32230
U+464E.0	fHakka	 hai2; hai1
U+464E.0	fCantonese	haai4
U+464E.0	fMandarin	xie2
U+464E.0	fEnglish	sit at the bottom of a robe to enable the wearer to walk
U+464F.0	fUTF8	䙏
U+464F.0	fCNS11643	4-5355
U+464F.0	fGB	3-672F
U+464F.0	fR/S	 145.10
U+464F.0	fTotalStrokes	 16
U+464F.0	fCangjie	LIBI
U+464F.0	fFourCorner	33240
U+464F.0	fCantonese	bok3; bok6; pok3
U+464F.0	fMandarin	bo2
U+464F.0	fEnglish	[1] short sleeves shirt [2] clothes that have no lining [3] same as &#x8584;
U+4650.0	fUTF8	䙐
U+4650.0	fCNS11643	4-5356
U+4650.0	fGB	3-6730
U+4650.0	fR/S	 145.10
U+4650.0	fTotalStrokes	 16; 15
U+4650.0	fCangjie	LMRB
U+4650.0	fFourCorner	31220
U+4650.0	fCantonese	hak1
U+4650.0	fMandarin	ke4
U+4650.0	fEnglish	furs and linen garments, thin; light; slight, to pound; to beat, to attack, to hull or unhusk
U+4651.0	fUTF8	䙑
U+4651.0	fCNS11643	3-4E63
U+4651.0	fGB	5-6621
U+4651.0	fR/S	 145.10
U+4651.0	fTotalStrokes	 15; 16
U+4651.0	fCangjie	LYWV
U+4651.0	fFourCorner	30230
U+4651.0	fCantonese	ceoi1
U+4651.0	fMandarin	cui1
U+4651.0	fEnglish	same as &#x7E17;, &#x8870;
U+4652.0	fUTF8	䙒
U+4652.0	fCNS11643	4-534F
U+4652.0	fGB	5-6623
U+4652.0	fR/S	 145.10
U+4652.0	fTotalStrokes	 16; 15
U+4652.0	fCangjie	LYVW
U+4652.0	fFourCorner	30260
U+4652.0	fCantonese	cuk1; cuk6; sin6; tung4
U+4652.0	fMandarin	xu4
U+4652.0	fEnglish	[1] store up; save up; hoard [2] strip off; deprive of [3] undress forcibly
U+4653.0	fUTF8	䙓
U+4653.0	fGB	S-227B
U+4653.0	fR/S	 145.10
U+4653.0	fTotalStrokes	 16
U+4653.0	fCangjie	LWLI
U+4653.0	fCantonese	bai2
U+4653.0	fMandarin	bai3; he4
U+4653.0	fEnglish	simplified form of &#x896C;
U+4654.0	fUTF8	䙔
U+4654.0	fCNS11643	4-5863
U+4654.0	fGB	3-662E
U+4654.0	fR/S	 145.11
U+4654.0	fTotalStrokes	 16; 17
U+4654.0	fCangjie	LSRR
U+4654.0	fFourCorner	31210
U+4654.0	fCantonese	au1; ngau1 (also); aau1; zaau1; zat3; zim2
U+4654.0	fMandarin	ou1
U+4654.0	fEnglish	[1] clothes made of poor fabric [2] head-dress for chidlren (or minority group) [3] an infant's bib
U+4655.0	fUTF8	䙕
U+4655.0	fCNS11643	4-5867
U+4655.0	fGB	3-6632
U+4655.0	fR/S	 145.11
U+4655.0	fTotalStrokes	 16; 17
U+4655.0	fCangjie	LHOO
U+4655.0	fFourCorner	38280
U+4655.0	fCantonese	zung2
U+4655.0	fMandarin	song3; chu4; zong3
U+4655.0	fEnglish	clothes that have no lining
U+4656.0	fUTF8	䙖
U+4656.0	fCNS11643	3-5645
U+4656.0	fR/S	 145.11
U+4656.0	fTotalStrokes	 17
U+4656.0	fCangjie	LUBB
U+4656.0	fFourCorner	32220
U+4656.0	fCantonese	bang1
U+4656.0	fMandarin	beng1
U+4656.0	fEnglish	same as &#x7DB3;, &#x7E43;
U+4657.0	fUTF8	䙗
U+4657.0	fCNS11643	3-5643
U+4657.0	fGB	5-6629
U+4657.0	fR/S	 145.11
U+4657.0	fTotalStrokes	 17
U+4657.0	fCangjie	LYCB
U+4657.0	fFourCorner	30220
U+4657.0	fCantonese	tai3
U+4657.0	fMandarin	ti4
U+4657.0	fEnglish	a swaddling cloth for infant; a broad bandage for carrying an infant on the back
U+4658.0	fUTF8	䙘
U+4658.0	fCNS11643	4-5864
U+4658.0	fGB	3-6737
U+4658.0	fR/S	 145.11
U+4658.0	fTotalStrokes	 16; 17
U+4658.0	fCangjie	LIHF
U+4658.0	fFourCorner	33250
U+4658.0	fCantonese	co2
U+4658.0	fMandarin	cu4; chu3; zu2
U+4658.0	fEnglish	exquisite; fine, bright-colored clothes, clear white
U+4659.0	fUTF8	䙙
U+4659.0	fCNS11643	3-563E
U+4659.0	fR/S	 145.11; 145.12
U+4659.0	fTotalStrokes	 17
U+4659.0	fCangjie	LSEQ
U+4659.0	fFourCorner	37250
U+4659.0	fCantonese	ci4
U+4659.0	fMandarin	chi2
U+4659.0	fEnglish	clothes
U+465A.0	fUTF8	䙚
U+465A.0	fCNS11643	7-3B59
U+465A.0	fR/S	 145.11
U+465A.0	fTotalStrokes	 17
U+465A.0	fCangjie	HFYHV
U+465A.0	fFourCorner	27730
U+465A.0	fCantonese	niu5; liu5 (also)
U+465A.0	fMandarin	niao3
U+465A.0	fEnglish	same as &#x88CA;, &#x892D;
U+465B.0	fUTF8	䙛
U+465B.0	fCNS11643	3-5644
U+465B.0	fR/S	 145.11
U+465B.0	fTotalStrokes	 17
U+465B.0	fCangjie	LYCV
U+465B.0	fFourCorner	30230
U+465B.0	fCantonese	gwan2; gwaan2; waan6
U+465B.0	fMandarin	gun3; guan4
U+465B.0	fEnglish	same as &#x889E;, &#x6E3F;
U+465C.0	fUTF8	䙜
U+465C.0	fCNS11643	4-5865
U+465C.0	fGB	5-657E
U+465C.0	fR/S	 145.11
U+465C.0	fTotalStrokes	 16; 17
U+465C.0	fCangjie	LYHJ
U+465C.0	fFourCorner	37230
U+465C.0	fCantonese	fung4
U+465C.0	fMandarin	feng2
U+465C.0	fEnglish	[1] (corrupted form) legendary mountain deity [2] interchangeable with &#x9022;
U+465D.0	fUTF8	䙝
U+465D.0	fCNS11643	3-5641
U+465D.0	fR/S	 145.11
U+465D.0	fTotalStrokes	 17
U+465D.0	fCangjie	YGIV
U+465D.0	fFourCorner	00730
U+465D.0	fCantonese	dip6; sit3
U+465D.0	fMandarin	xie4
U+465D.0	fEnglish	[1] under garments [2] intimate with [3] dirty; ragged [4] undress; thus - to treat with irreverence
U+465E.0	fUTF8	䙞
U+465E.0	fCNS11643	5-6456
U+465E.0	fR/S	 145.12
U+465E.0	fTotalStrokes	 18; 17
U+465E.0	fCangjie	LNOT
U+465E.0	fFourCorner	32210
U+465E.0	fCantonese	dang1
U+465E.0	fMandarin	deng1
U+465E.0	fEnglish	woolen string; ribbon or lace
U+465F.0	fUTF8	䙟
U+465F.0	fCNS11643	4-5D4A
U+465F.0	fGB	3-666A
U+465F.0	fR/S	 145.12
U+465F.0	fTotalStrokes	 18; 17
U+465F.0	fCangjie	LWDQ
U+465F.0	fFourCorner	36200
U+465F.0	fCantonese	wai4
U+465F.0	fMandarin	wei2; duo4; wei3
U+465F.0	fEnglish	[1] heavy clothes (thickness) [2] hang down
U+4660.0	fUTF8	䙠
U+4660.0	fCNS11643	4-5D48
U+4660.0	fGB	3-6740
U+4660.0	fR/S	 145.12
U+4660.0	fTotalStrokes	 18; 17
U+4660.0	fCangjie	LMTO
U+4660.0	fFourCorner	31280
U+4660.0	fCantonese	gyut6; kyut3
U+4660.0	fMandarin	jue2; wo4
U+4660.0	fEnglish	[1] short clothes [2] lift up clothes and across a river
U+4661.0	fUTF8	䙡
U+4661.0	fCNS11643	4-5D49
U+4661.0	fGB	3-6723
U+4661.0	fR/S	 145.12
U+4661.0	fTotalStrokes	 18; 17
U+4661.0	fCangjie	LLMC
U+4661.0	fFourCorner	35280
U+4661.0	fHakka	 kui3; kui6; kui5; k'wui5
U+4661.0	fCantonese	kui2; kwui2 (also)
U+4661.0	fMandarin	kui4; hui4
U+4661.0	fEnglish	[1] knot [2] the loop for fastening a coat
U+4662.0	fUTF8	䙢
U+4662.0	fCNS11643	4-5D44
U+4662.0	fGB	3-6746
U+4662.0	fR/S	 145.12
U+4662.0	fTotalStrokes	 18; 17
U+4662.0	fCangjie	LCWA
U+4662.0	fFourCorner	38260
U+4662.0	fHakka	 zen5
U+4662.0	fCantonese	zang3; zaang6
U+4662.0	fMandarin	zeng4; ceng2
U+4662.0	fEnglish	[1] cloth; [2] handkerchief
U+4663.0	fUTF8	䙣
U+4663.0	fCNS11643	4-5D46
U+4663.0	fGB	3-673C
U+4663.0	fR/S	 145.12
U+4663.0	fTotalStrokes	 18; 17
U+4663.0	fCangjie	LMKY
U+4663.0	fFourCorner	32210
U+4663.0	fCantonese	saap3
U+4663.0	fMandarin	sa4
U+4663.0	fEnglish	ragged garments or clothing
U+4664.0	fUTF8	䙤
U+4664.0	fCNS11643	4-612A
U+4664.0	fGB	3-6657
U+4664.0	fR/S	 145.13
U+4664.0	fTotalStrokes	 19; 18
U+4664.0	fCangjie	LYBB
U+4664.0	fFourCorner	37230
U+4664.0	fCantonese	do2
U+4664.0	fMandarin	du3; duo3; duo4
U+4664.0	fEnglish	good; nice; fine, long and large clothes
U+4665.0	fUTF8	䙥
U+4665.0	fCNS11643	7-4322
U+4665.0	fR/S	 145.13
U+4665.0	fTotalStrokes	 18; 19
U+4665.0	fCangjie	LMBI
U+4665.0	fFourCorner	31230
U+4665.0	fCantonese	ling4
U+4665.0	fMandarin	ling2
U+4665.0	fEnglish	shiny dresses
U+4666.0	fUTF8	䙦
U+4666.0	fCNS11643	4-6443
U+4666.0	fGB	3-6748
U+4666.0	fR/S	 145.14
U+4666.0	fTotalStrokes	 20; 19
U+4666.0	fCangjie	LTWN
U+4666.0	fFourCorner	34220
U+4666.0	fCantonese	mung4
U+4666.0	fMandarin	meng2
U+4666.0	fEnglish	upper garments
U+4667.0	fUTF8	䙧
U+4667.0	fCNS11643	7-5025
U+4667.0	fR/S	 145.14
U+4667.0	fTotalStrokes	 20; 19
U+4667.0	fCangjie	LHJF
U+4667.0	fFourCorner	32230
U+4667.0	fCantonese	fan1
U+4667.0	fMandarin	xun1
U+4667.0	fEnglish	red color; crimson
U+4668.0	fUTF8	䙨
U+4668.0	fCNS11643	3-5D27
U+4668.0	fR/S	 145.14
U+4668.0	fTotalStrokes	 20; 19
U+4668.0	fCangjie	LWDV
U+4668.0	fFourCorner	36230
U+4668.0	fCantonese	gwo2; go2 (also)
U+4668.0	fMandarin	guo3; luo3
U+4668.0	fEnglish	non-classical form of &#x88F9;
U+4669.0	fUTF8	䙩
U+4669.0	fCNS11643	4-6444
U+4669.0	fGB	5-6632
U+4669.0	fR/S	 145.14
U+4669.0	fTotalStrokes	 20; 19
U+4669.0	fCangjie	LTBO
U+4669.0	fFourCorner	34230
U+4669.0	fCantonese	mung4
U+4669.0	fMandarin	meng2
U+4669.0	fEnglish	clothes
U+466A.0	fUTF8	䙪
U+466A.0	fCNS11643	3-5E5F
U+466A.0	fR/S	 145.15
U+466A.0	fTotalStrokes	 21
U+466A.0	fCangjie	DDKYV
U+466A.0	fFourCorner	44730
U+466A.0	fCantonese	paan3
U+466A.0	fMandarin	pan4; long2; mang4
U+466A.0	fEnglish	same as &#x897B;
U+466B.0	fUTF8	䙫
U+466B.0	fGB	S-2279
U+466B.0	fR/S	 145.16
U+466B.0	fTotalStrokes	 22
U+466B.0	fCangjie	LLPC
U+466B.0	fFourCorner	3128
U+466B.0	fCantonese	kei3
U+466C.0	fUTF8	䙬
U+466C.0	fCNS11643	4-6A7B
U+466C.0	fGB	3-6739
U+466C.0	fR/S	 145.17
U+466C.0	fTotalStrokes	 23; 22
U+466C.0	fCangjie	LBCV
U+466C.0	fFourCorner	36240
U+466C.0	fCantonese	ang3; ngang3 (also); zing3
U+466C.0	fMandarin	ying1; ying4
U+466C.0	fEnglish	[1] plaits on the sides of a petticoat [2] pleates of a skirt [3] contrast of the mixed colors
U+466D.0	fUTF8	䙭
U+466D.0	fCNS11643	7-5932
U+466D.0	fR/S	 145.17
U+466D.0	fTotalStrokes	 22; 23
U+466D.0	fCangjie	LJTO
U+466D.0	fFourCorner	33280
U+466D.0	fCantonese	hin1
U+466D.0	fMandarin	qian1; jian3
U+466D.0	fEnglish	same as &#x8930;
U+466E.0	fUTF8	䙮
U+466E.0	fCNS11643	3-607A
U+466E.0	fR/S	 145.18
U+466E.0	fTotalStrokes	 24; 23
U+466E.0	fCangjie	LTRG
U+466E.0	fFourCorner	34210
U+466E.0	fCantonese	gyun6; gun3
U+466E.0	fMandarin	guan4
U+466E.0	fEnglish	strip of the clothes
U+466F.0	fUTF8	䙯
U+466F.0	fCNS11643	4-6C26
U+466F.0	fGB	5-6638
U+466F.0	fR/S	 145.18
U+466F.0	fTotalStrokes	 24; 23
U+466F.0	fCangjie	LIHO
U+466F.0	fFourCorner	33280
U+466F.0	fCantonese	zuk1; cuk1
U+466F.0	fMandarin	cu4; zhuo2
U+466F.0	fEnglish	[1] exquisite; fine [2] sharp; distinct [3] bright-colored
U+4670.0	fUTF8	䙰
U+4670.0	fCNS11643	3-6079
U+4670.0	fR/S	 145.19
U+4670.0	fTotalStrokes	 24; 25
U+4670.0	fCangjie	LYBG
U+4670.0	fFourCorner	30210
U+4670.0	fCantonese	lei4
U+4670.0	fMandarin	li2
U+4670.0	fEnglish	[1] new feather; damp; moist and sticky [2] a thin silk curtain
U+4671.0	fUTF8	䙱
U+4671.0	fCNS11643	4-6D7D
U+4671.0	fGB	5-6630
U+4671.0	fR/S	 145.21
U+4671.0	fTotalStrokes	 27; 26
U+4671.0	fCangjie	LSEI
U+4671.0	fFourCorner	37220
U+4671.0	fHakka	 zhuk7
U+4671.0	fCantonese	suk6
U+4671.0	fMandarin	shu3; du2; zhu2
U+4671.0	fEnglish	same as &#x8961;
U+4672.0	fUTF8	䙲
U+4672.0	fCNS11643	5-2B47
U+4672.0	fR/S	 146.3
U+4672.0	fTotalStrokes	 9
U+4672.0	fCangjie	MWK
U+4672.0	fFourCorner	10430
U+4672.0	fCantonese	cin1; ceng1
U+4672.0	fMandarin	xian1
U+4672.0	fEnglish	same as &#x9077;
U+4673.0	fUTF8	䙳
U+4673.0	fCNS11643	4-2F73
U+4673.0	fR/S	 146.4; 146.6; 86.6
U+4673.0	fTotalStrokes	 10
U+4673.0	fCangjie	MWF
U+4673.0	fFourCorner	10800
U+4673.0	fCantonese	biu1
U+4673.0	fMandarin	e4
U+4673.0	fEnglish	same as &#x719B;
U+4674.0	fUTF8	䙴
U+4674.0	fCNS11643	4-3531
U+4674.0	fGB	5-5F66
U+4674.0	fR/S	 146.5
U+4674.0	fTotalStrokes	 11
U+4674.0	fCangjie	MWKSU
U+4674.0	fFourCorner	10710
U+4674.0	fHakka	  sien1; cien1
U+4674.0	fCantonese	cin1; sin1
U+4674.0	fMandarin	xian1; qian1
U+4674.0	fEnglish	[1] rise high as bird [2] climb a height
U+4675.0	fUTF8	䙵
U+4675.0	fCNS11643	4-3B21
U+4675.0	fGB	5-5F69
U+4675.0	fR/S	 146.6
U+4675.0	fTotalStrokes	 12
U+4675.0	fCangjie	MWGG
U+4675.0	fFourCorner	10100
U+4675.0	fCantonese	kwai4; saai1
U+4675.0	fMandarin	xi1
U+4675.0	fEnglish	[1] rustic; low; mean [2] despise [3] a family name
U+4676.0	fUTF8	䙶
U+4676.0	fGB	S-243C
U+4676.0	fR/S	 146.6
U+4676.0	fTotalStrokes	 12
U+4676.0	fCangjie	MWHBV
U+4676.0	fCantonese	nung4; lung4 (also)
U+4677.0	fUTF8	䙷
U+4677.0	fCNS11643	3-3379
U+4677.0	fGB	5-4A58
U+4677.0	fR/S	 147.3
U+4677.0	fTotalStrokes	 10
U+4677.0	fCangjie	BUDI
U+4677.0	fFourCorner	60340
U+4677.0	fCantonese	daak1; kaam4
U+4677.0	fMandarin	de2; zhe2
U+4677.0	fEnglish	ancient form of &#x5F97;
U+4678.0	fUTF8	䙸
U+4678.0	fCNS11643	3-3378
U+4678.0	fR/S	 147.3
U+4678.0	fTotalStrokes	 10
U+4678.0	fCangjie	BUDI
U+4678.0	fFourCorner	64210
U+4678.0	fCantonese	dak1
U+4678.0	fMandarin	ai4; de2; de5
U+4678.0	fEnglish	same as &#x4677;
U+4679.0	fUTF8	䙹
U+4679.0	fCNS11643	4-3533
U+4679.0	fGB	5-4A5B
U+4679.0	fR/S	 147.4
U+4679.0	fTotalStrokes	 11
U+4679.0	fCangjie	MTBUU
U+4679.0	fFourCorner	16410
U+4679.0	fCantonese	jin6
U+4679.0	fMandarin	er2; jiang3; nao3; xian4; xiang3
U+4679.0	fEnglish	to vomit milk (said of baby), to vomit unwillingly
U+467A.0	fUTF8	䙺
U+467A.0	fCNS11643	4-3535
U+467A.0	fR/S	 147.4
U+467A.0	fTotalStrokes	 11
U+467A.0	fCangjie	FBUU
U+467A.0	fFourCorner	96810
U+467A.0	fCantonese	lim4
U+467A.0	fMandarin	lian2; lian3; qian3
U+467A.0	fEnglish	[1] corrupted form of &#x899D; [2] standard form of &#x5EC9;
U+467B.0	fUTF8	䙻
U+467B.0	fGB	3-485E
U+467B.0	fR/S	 147.4
U+467B.0	fTotalStrokes	 11
U+467B.0	fCangjie	LHBUU
U+467B.0	fFourCorner	5621
U+467B.0	fCantonese	ci1
U+467B.0	fEnglish	to steal a look; to steal a glance
U+467C.0	fUTF8	䙼
U+467C.0	fCNS11643	4-3B23
U+467C.0	fGB	3-4861
U+467C.0	fR/S	 147.5
U+467C.0	fTotalStrokes	 12
U+467C.0	fCangjie	SRBUU
U+467C.0	fFourCorner	16610
U+467C.0	fCantonese	siu6
U+467C.0	fMandarin	shao4
U+467C.0	fEnglish	to see; to observe, to appear, now; present, ready, to summon; to call up, far, deep; profound
U+467D.0	fUTF8	䙽
U+467D.0	fCNS11643	5-3C35
U+467D.0	fR/S	 147.5
U+467D.0	fTotalStrokes	 12
U+467D.0	fCangjie	KCBUU
U+467D.0	fFourCorner	40210
U+467D.0	fCantonese	kit3
U+467D.0	fMandarin	die2; xie2; xi4
U+467D.0	fEnglish	to see; to observe
U+467E.0	fUTF8	䙾
U+467E.0	fCNS11643	4-3B22
U+467E.0	fGB	3-4860
U+467E.0	fR/S	 147.5
U+467E.0	fTotalStrokes	 12
U+467E.0	fCangjie	JPBUU
U+467E.0	fFourCorner	36710
U+467E.0	fCantonese	si1
U+467E.0	fMandarin	shi1; zhui1; wei2; chi2
U+467E.0	fEnglish	[1] wait for; await [2] interchangeable with &#x4F3A; [3] same as &#x65BD;
U+467F.0	fUTF8	䙿
U+467F.0	fCNS11643	3-3E73
U+467F.0	fR/S	 147.5
U+467F.0	fTotalStrokes	 12
U+467F.0	fCangjie	JDBUU
U+467F.0	fFourCorner	56910
U+467F.0	fCantonese	mei6
U+467F.0	fMandarin	wei4
U+467F.0	fEnglish	 see; observe
U+4680.0	fUTF8	䚀
U+4680.0	fCNS11643	3-4450
U+4680.0	fR/S	 147.6
U+4680.0	fTotalStrokes	 13
U+4680.0	fCangjie	HYBUU
U+4680.0	fFourCorner	26410
U+4680.0	fCantonese	cam1
U+4680.0	fMandarin	chen4
U+4680.0	fEnglish	to see stealthily; to peek
U+4681.0	fUTF8	䚁
U+4681.0	fCNS11643	3-4451
U+4681.0	fR/S	 147.6
U+4681.0	fTotalStrokes	 13
U+4681.0	fCangjie	HBBUU
U+4681.0	fFourCorner	26210
U+4681.0	fCantonese	cam1
U+4681.0	fEnglish	same as &#x4680;
U+4682.0	fUTF8	䚂
U+4682.0	fCNS11643	4-4736
U+4682.0	fGB	5-6D6C
U+4682.0	fR/S	 147.7
U+4682.0	fTotalStrokes	 14
U+4682.0	fCangjie	GCBUU
U+4682.0	fFourCorner	46310
U+4682.0	fCantonese	haak3; hak1
U+4682.0	fMandarin	he4; xi4
U+4682.0	fEnglish	 see; observe; examine
U+4683.0	fUTF8	䚃
U+4683.0	fCNS11643	4-4738
U+4683.0	fGB	3-4864
U+4683.0	fR/S	 147.7
U+4683.0	fTotalStrokes	 14
U+4683.0	fCangjie	YWBUU
U+4683.0	fFourCorner	26610
U+4683.0	fCantonese	jau4
U+4683.0	fMandarin	you2
U+4683.0	fEnglish	to realize fully, to make a thorough examination
U+4684.0	fUTF8	䚄
U+4684.0	fCNS11643	4-4D4A
U+4684.0	fGB	3-486B
U+4684.0	fR/S	 147.8
U+4684.0	fTotalStrokes	 15
U+4684.0	fCangjie	VEBUU
U+4684.0	fFourCorner	26110
U+4684.0	fCantonese	luk6; luk1
U+4684.0	fMandarin	lu4
U+4684.0	fEnglish	[1] make fun of [2] having the same idea, feeling, etc.
U+4685.0	fUTF8	䚅
U+4685.0	fCNS11643	4-4D48
U+4685.0	fGB	3-4863
U+4685.0	fR/S	 147.8
U+4685.0	fTotalStrokes	 15
U+4685.0	fCangjie	DOBUU
U+4685.0	fFourCorner	46910
U+4685.0	fCantonese	loi6; loi4
U+4685.0	fMandarin	lai4; lai2
U+4685.0	fEnglish	[1] look after home or domestic affairs [2] search one's own soul [3] see; observe
U+4686.0	fUTF8	䚆
U+4686.0	fCNS11643	4-535A
U+4686.0	fGB	3-4868
U+4686.0	fR/S	 147.9
U+4686.0	fTotalStrokes	 16
U+4686.0	fCangjie	TKBUU
U+4686.0	fFourCorner	46510
U+4686.0	fCantonese	jing2
U+4686.0	fMandarin	ying1; ou3; yao3; ying3
U+4686.0	fEnglish	see; to look at, to examine
U+4687.0	fUTF8	䚇
U+4687.0	fCNS11643	4-535B
U+4687.0	fGB	3-486D
U+4687.0	fR/S	 147.9
U+4687.0	fTotalStrokes	 16
U+4687.0	fCangjie	FUBUU
U+4687.0	fFourCorner	96610
U+4687.0	fCantonese	saang2; sang2; sing2
U+4687.0	fMandarin	sheng3; zhi4
U+4687.0	fEnglish	[1] observe; inspect carefully [2] leaking out [3] a measurement [4] non-classical form of &#x7701;
U+4688.0	fUTF8	䚈
U+4688.0	fCNS11643	5-5832
U+4688.0	fR/S	 147.9
U+4688.0	fTotalStrokes	 16
U+4688.0	fCangjie	MGBUU
U+4688.0	fFourCorner	16110
U+4688.0	fCantonese	gyun3
U+4688.0	fMandarin	juan1; juan4; wu4; zhuan4
U+4688.0	fEnglish	see; to look at, to examine
U+4689.0	fUTF8	䚉
U+4689.0	fCNS11643	4-535C
U+4689.0	fGB	3-486C
U+4689.0	fR/S	 147.9
U+4689.0	fTotalStrokes	 16
U+4689.0	fCangjie	QKBUU
U+4689.0	fFourCorner	56410
U+4689.0	fCantonese	kai3; haam1; kaai3
U+4689.0	fMandarin	qi4; xi4
U+4689.0	fEnglish	[1] see; look at [2] examine [3] spy upon [4] wait upon
U+468A.0	fUTF8	䚊
U+468A.0	fCNS11643	5-5E43
U+468A.0	fR/S	 147.10
U+468A.0	fTotalStrokes	 17
U+468A.0	fCangjie	TTBUU
U+468A.0	fFourCorner	86110
U+468A.0	fCantonese	gyun1
U+468A.0	fMandarin	juan1
U+468A.0	fEnglish	look from a distance
U+468B.0	fUTF8	䚋
U+468B.0	fCNS11643	4-5868
U+468B.0	fGB	3-4865
U+468B.0	fR/S	 147.10
U+468B.0	fTotalStrokes	 17; 16
U+468B.0	fCangjie	RCBUU
U+468B.0	fFourCorner	66810
U+468B.0	fHakka	 jun5; run6
U+468B.0	fCantonese	wan6; waan4
U+468B.0	fMandarin	yun4
U+468B.0	fEnglish	a Chinese family name
U+468C.0	fUTF8	䚌
U+468C.0	fCNS11643	4-5869
U+468C.0	fGB	3-486E
U+468C.0	fR/S	 147.10
U+468C.0	fTotalStrokes	 17; 16
U+468C.0	fCangjie	HJBUU
U+468C.0	fFourCorner	26410
U+468C.0	fCantonese	gou1
U+468D.0	fUTF8	䚍
U+468D.0	fCNS11643	4-5D4E
U+468D.0	fGB	5-4A66
U+468D.0	fR/S	 147.11
U+468D.0	fTotalStrokes	 18
U+468D.0	fCangjie	QCBUU
U+468D.0	fFourCorner	56810
U+468D.0	fHakka	 tuk8
U+468D.0	fCantonese	cik1
U+468D.0	fMandarin	qi4; di2
U+468D.0	fEnglish	look at; to observe or inspect; to see
U+468E.0	fUTF8	䚎
U+468E.0	fCNS11643	4-5D50
U+468E.0	fGB	3-4871
U+468E.0	fR/S	 147.11
U+468E.0	fTotalStrokes	 18
U+468E.0	fCangjie	QXBUU
U+468E.0	fFourCorner	56710
U+468E.0	fHakka	 chong1; cong1; zhung1
U+468E.0	fCantonese	cong1; zong1
U+468E.0	fMandarin	chuang4
U+468E.0	fEnglish	 peer; see indistinctly
U+468F.0	fUTF8	䚏
U+468F.0	fCNS11643	4-612D
U+468F.0	fGB	3-4873
U+468F.0	fR/S	 147.12
U+468F.0	fTotalStrokes	 19
U+468F.0	fCangjie	FQBUU
U+468F.0	fFourCorner	96210
U+468F.0	fCantonese	leon6
U+468F.0	fMandarin	lin4; lin2; leng4; li4
U+468F.0	fEnglish	to love; intimate; near to; dear; parents; relatives, to see; to look at; to observe
U+4690.0	fUTF8	䚐
U+4690.0	fCNS11643	4-612E
U+4690.0	fGB	5-4A74
U+4690.0	fR/S	 147.12
U+4690.0	fTotalStrokes	 19
U+4690.0	fCangjie	OABUU
U+4690.0	fFourCorner	86610
U+4690.0	fCantonese	zik6
U+4690.0	fMandarin	ji2; ti4
U+4690.0	fEnglish	getting red in the eyes, to look from a distance
U+4691.0	fUTF8	䚑
U+4691.0	fCNS11643	3-5B30
U+4691.0	fGB	5-4A63
U+4691.0	fR/S	 147.12
U+4691.0	fTotalStrokes	 19
U+4691.0	fCangjie	WCBUU
U+4691.0	fFourCorner	66810
U+4691.0	fCantonese	maai4
U+4691.0	fMandarin	mai1; mai2
U+4691.0	fEnglish	to steal a glance; to peep; to spy on
U+4692.0	fUTF8	䚒
U+4692.0	fCNS11643	4-612C
U+4692.0	fGB	3-4872
U+4692.0	fR/S	 147.12
U+4692.0	fTotalStrokes	 19
U+4692.0	fCangjie	YGBUU
U+4692.0	fFourCorner	06110
U+4692.0	fCantonese	cong4; so2; zong6
U+4692.0	fMandarin	chuang2; zhuang4
U+4692.0	fEnglish	[1] eyesight blurred; dim of sight [2] look steadily at
U+4693.0	fUTF8	䚓
U+4693.0	fCNS11643	4-6130
U+4693.0	fGB	5-4A73
U+4693.0	fR/S	 147.12
U+4693.0	fTotalStrokes	 19
U+4693.0	fCangjie	MJBUU
U+4693.0	fFourCorner	16410
U+4693.0	fCantonese	sam2
U+4693.0	fMandarin	shen3; nian3
U+4693.0	fEnglish	same as &#x77AB;
U+4694.0	fUTF8	䚔
U+4694.0	fCNS11643	3-5E60
U+4694.0	fGB	5-4A70
U+4694.0	fR/S	 147.14
U+4694.0	fTotalStrokes	 21; 22
U+4694.0	fCangjie	JCBUU
U+4694.0	fFourCorner	06810
U+4694.0	fCantonese	ban1; baan1; bin1; paan5
U+4694.0	fMandarin	bin1
U+4694.0	fEnglish	 look suddenly; look shortly
U+4695.0	fUTF8	䚕
U+4695.0	fCNS11643	4-6E3B
U+4695.0	fGB	3-4862
U+4695.0	fR/S	 147.19
U+4695.0	fTotalStrokes	 28; 26
U+4695.0	fCangjie	MPBUU
U+4695.0	fFourCorner	16210
U+4695.0	fCantonese	lai6; saai2 (also); laai6; lak6
U+4695.0	fMandarin	li4; xi3; luan2; xi4; li2
U+4695.0	fEnglish	[1] search out; examine into [2] give an angry glances at somebody
U+4696.0	fUTF8	䚖
U+4696.0	fCNS11643	3-623C
U+4696.0	fR/S	 147.24
U+4696.0	fTotalStrokes	 31
U+4696.0	fCangjie	MOBUU
U+4696.0	fFourCorner	16110
U+4696.0	fCantonese	ling4
U+4696.0	fMandarin	ling2
U+4696.0	fEnglish	mountain deity; with human face and animal body
U+4697.0	fUTF8	䚗
U+4697.0	fCNS11643	4-353A
U+4697.0	fGB	3-7237
U+4697.0	fR/S	 148.4
U+4697.0	fTotalStrokes	 11
U+4697.0	fCangjie	NBCI
U+4697.0	fFourCorner	28230
U+4697.0	fCantonese	gong1
U+4697.0	fMandarin	jiang1; zhong1
U+4697.0	fEnglish	 lift; raise
U+4698.0	fUTF8	䚘
U+4698.0	fCNS11643	3-4453
U+4698.0	fGB	5-3C45
U+4698.0	fR/S	 148.6
U+4698.0	fTotalStrokes	 13
U+4698.0	fCangjie	HONBN
U+4698.0	fFourCorner	21227
U+4698.0	fCantonese	caang4
U+4698.0	fMandarin	cheng2; chen2; heng2
U+4698.0	fEnglish	the long and upright horns of cattle
U+4699.0	fUTF8	䚙
U+4699.0	fCNS11643	4-4067
U+4699.0	fGB	3-723E
U+4699.0	fR/S	 148.6
U+4699.0	fTotalStrokes	 13
U+4699.0	fCangjie	NBMAM
U+4699.0	fFourCorner	21210
U+4699.0	fCantonese	hyun1
U+4699.0	fMandarin	xuan1; xi1
U+4699.0	fEnglish	a spoon for cooked rice made of horn of an animal, uneven horns of cattle
U+469A.0	fUTF8	䚚
U+469A.0	fCNS11643	3-4452
U+469A.0	fGB	5-7128
U+469A.0	fR/S	 148.6
U+469A.0	fTotalStrokes	 13
U+469A.0	fCangjie	NBHGU
U+469A.0	fFourCorner	24210
U+469A.0	fCantonese	sin2
U+469A.0	fMandarin	xian3
U+469A.0	fEnglish	horn of an animal
U+469B.0	fUTF8	䚛
U+469B.0	fCNS11643	4-473A
U+469B.0	fGB	5-712D
U+469B.0	fR/S	 148.7
U+469B.0	fTotalStrokes	 14
U+469B.0	fCangjie	NBHGR
U+469B.0	fFourCorner	24260
U+469B.0	fCantonese	huk6
U+469B.0	fMandarin	hu2
U+469B.0	fEnglish	regulate; polish ivory to make it more white
U+469C.0	fUTF8	䚜
U+469C.0	fCNS11643	4-4D51
U+469C.0	fGB	3-7247
U+469C.0	fR/S	 148.8
U+469C.0	fTotalStrokes	 15
U+469C.0	fCangjie	NBHWJ
U+469C.0	fFourCorner	26240
U+469C.0	fCantonese	bai1; bei1
U+469C.0	fMandarin	bi1; bei1
U+469C.0	fEnglish	[1] cattle with crosswise horns [2] crosswise horns of a cattle
U+469D.0	fUTF8	䚝
U+469D.0	fCNS11643	4-4D4C
U+469D.0	fGB	3-7248
U+469D.0	fR/S	 148.8
U+469D.0	fTotalStrokes	 15
U+469D.0	fCangjie	NBYOJ
U+469D.0	fFourCorner	20240
U+469D.0	fCantonese	zyut6; zeot1
U+469D.0	fMandarin	zu2
U+469D.0	fEnglish	[1] new-growing horns [2] clasp or hold under the arm [3] bosom; embrace
U+469E.0	fUTF8	䚞
U+469E.0	fCNS11643	4-4D4E
U+469E.0	fGB	3-7241
U+469E.0	fR/S	 148.8
U+469E.0	fTotalStrokes	 15
U+469E.0	fCangjie	NBDOO
U+469E.0	fFourCorner	24290
U+469E.0	fCantonese	daai2
U+469E.0	fMandarin	dai3
U+469E.0	fEnglish	the middle of the horn
U+469F.0	fUTF8	䚟
U+469F.0	fCNS11643	4-4D53
U+469F.0	fGB	5-7133
U+469F.0	fR/S	 148.8
U+469F.0	fTotalStrokes	 15
U+469F.0	fCangjie	NBAMI
U+469F.0	fFourCorner	26240
U+469F.0	fCantonese	daai2
U+469F.0	fMandarin	dai3
U+469F.0	fEnglish	[1] horns [2] middle of the horn [3] same as &#x6B79;
U+46A0.0	fUTF8	䚠
U+46A0.0	fCNS11643	4-4D4F
U+46A0.0	fGB	3-7245
U+46A0.0	fR/S	 148.8
U+46A0.0	fTotalStrokes	 15
U+46A0.0	fCangjie	NBAPP
U+46A0.0	fFourCorner	26210
U+46A0.0	fCantonese	wan6
U+46A0.0	fMandarin	hun3; hun4; hun2
U+46A0.0	fEnglish	[1] (ancient form) [2] unbroken firewood [3] complete; whole; general [4] round horn [5] animal's horns [6] whole horn
U+46A1.0	fUTF8	䚡
U+46A1.0	fCNS11643	4-5364
U+46A1.0	fGB	3-724C
U+46A1.0	fR/S	 148.9; 148.11
U+46A1.0	fTotalStrokes	 16
U+46A1.0	fCangjie	NBWP
U+46A1.0	fFourCorner	26230
U+46A1.0	fHakka	 soi1
U+46A1.0	fCantonese	soi1
U+46A1.0	fMandarin	gan1; sai1; sai4; si1; shai1
U+46A1.0	fEnglish	the marrow or soft part inside horns
U+46A2.0	fUTF8	䚢
U+46A2.0	fCNS11643	4-5361
U+46A2.0	fGB	3-7249
U+46A2.0	fR/S	 148.9
U+46A2.0	fTotalStrokes	 16
U+46A2.0	fCangjie	NBPTD
U+46A2.0	fFourCorner	24290
U+46A2.0	fCantonese	cit3
U+46A2.0	fMandarin	che4; chi4; yi4
U+46A2.0	fEnglish	horn, jewelry; ornaments used for woman in old days, an eyelet for the hook of a leather belt, a second for childhood
U+46A3.0	fUTF8	䚣
U+46A3.0	fCNS11643	4-5362
U+46A3.0	fGB	3-724B
U+46A3.0	fR/S	 148.9
U+46A3.0	fTotalStrokes	 16
U+46A3.0	fCangjie	NBAMO
U+46A3.0	fFourCorner	26280
U+46A3.0	fCantonese	tai4; taai3; taai4
U+46A3.0	fMandarin	ti2; ti4
U+46A3.0	fEnglish	crooked horns of the animal
U+46A4.0	fUTF8	䚤
U+46A4.0	fR/S	 148.9
U+46A4.0	fTotalStrokes	 16
U+46A4.0	fCangjie	NBWLB
U+46A4.0	fCantonese	jyu4
U+46A4.0	fEnglish	[Korean character]
U+46A5.0	fUTF8	䚥
U+46A5.0	fCNS11643	4-586B
U+46A5.0	fGB	3-724D
U+46A5.0	fR/S	 148.10
U+46A5.0	fTotalStrokes	 17; 16; 19
U+46A5.0	fCangjie	NBNMM
U+46A5.0	fFourCorner	27220
U+46A5.0	fCantonese	nok6; lok6 (also); zoek6
U+46A5.0	fMandarin	ruo4; nuo4
U+46A5.0	fEnglish	 adjust a bow
U+46A6.0	fUTF8	䚦
U+46A6.0	fCNS11643	4-586A
U+46A6.0	fGB	5-713A
U+46A6.0	fR/S	 148.10
U+46A6.0	fTotalStrokes	 17; 16
U+46A6.0	fCangjie	NBHYU
U+46A6.0	fFourCorner	22210
U+46A6.0	fCantonese	zi6; zaai6 (also)
U+46A6.0	fMandarin	zhi4; pi3; chi3
U+46A6.0	fEnglish	slanted corners; curved corners, crooked horn
U+46A7.0	fUTF8	䚧
U+46A7.0	fCNS11643	5-6463
U+46A7.0	fGB	3-724E
U+46A7.0	fR/S	 148.11
U+46A7.0	fTotalStrokes	 18
U+46A7.0	fCangjie	NBSMH
U+46A7.0	fFourCorner	27220
U+46A7.0	fCantonese	lau4
U+46A7.0	fMandarin	liu2
U+46A7.0	fEnglish	curved with angles; curved corner
U+46A8.0	fUTF8	䚨
U+46A8.0	fCNS11643	4-6131
U+46A8.0	fGB	3-723B
U+46A8.0	fR/S	 148.12
U+46A8.0	fTotalStrokes	 19; 20
U+46A8.0	fCangjie	NBNOE
U+46A8.0	fFourCorner	22240
U+46A8.0	fCantonese	fai3
U+46A8.0	fMandarin	fei4; ba2; fa3
U+46A8.0	fEnglish	a string fastened to an arrow to draw it back after shooting
U+46A9.0	fUTF8	䚩
U+46A9.0	fCNS11643	4-6134
U+46A9.0	fGB	3-723F
U+46A9.0	fR/S	 148.12
U+46A9.0	fTotalStrokes	 19
U+46A9.0	fCangjie	NBHKB
U+46A9.0	fFourCorner	22220
U+46A9.0	fHakka	  giau3; geu3
U+46A9.0	fCantonese	giu2; faan4; hiu1; kiu4; ziu4
U+46A9.0	fMandarin	jiao3; jiao4; qiao2
U+46A9.0	fEnglish	 raise one horn higher than the other
U+46AA.0	fUTF8	䚪
U+46AA.0	fCNS11643	4-6446
U+46AA.0	fGB	5-7140
U+46AA.0	fR/S	 148.13
U+46AA.0	fTotalStrokes	 20
U+46AA.0	fCangjie	NBWLV
U+46AA.0	fFourCorner	26230
U+46AA.0	fCantonese	gwaan1
U+46AA.0	fMandarin	guan1
U+46AA.0	fEnglish	[1] curved horns [2] non-classical form of &#x9C5E;, &#x9C25;
U+46AB.0	fUTF8	䚫
U+46AB.0	fCNS11643	4-6447
U+46AB.0	fGB	3-724F
U+46AB.0	fR/S	 148.13
U+46AB.0	fTotalStrokes	 20
U+46AB.0	fCangjie	HKNBG
U+46AB.0	fFourCorner	28220
U+46AB.0	fCantonese	hat6
U+46AB.0	fMandarin	xi2; ao2; jue2
U+46AB.0	fEnglish	ornamental articles made of horns used to decorate the top of a stick or horsewhip, to hit; to beat
U+46AC.0	fUTF8	䚬
U+46AC.0	fR/S	 148.12
U+46AC.0	fTotalStrokes	 19
U+46AC.0	fCangjie	NBFDQ
U+46AC.0	fFourCorner	2925
U+46AC.0	fCantonese	leon4
U+46AC.0	fMandarin	lin2
U+46AD.0	fUTF8	䚭
U+46AD.0	fCNS11643	4-6D22
U+46AD.0	fGB	3-7251
U+46AD.0	fR/S	 148.18
U+46AD.0	fTotalStrokes	 25
U+46AD.0	fCangjie	NBTRG
U+46AD.0	fFourCorner	24210
U+46AD.0	fCantonese	hyun1
U+46AD.0	fMandarin	xuan1; sui4
U+46AE.0	fUTF8	䚮
U+46AE.0	fCNS11643	3-2F2B
U+46AE.0	fGB	5-7154
U+46AE.0	fR/S	 149.2
U+46AE.0	fTotalStrokes	 9
U+46AE.0	fCangjie	YRNHS
U+46AE.0	fFourCorner	07620
U+46AE.0	fHakka	 jin2
U+46AE.0	fCantonese	jing4; zing4
U+46AE.0	fMandarin	reng2; er2
U+46AE.0	fEnglish	[1] thicker [2] heavy [3] still; yet
U+46AF.0	fUTF8	䚯
U+46AF.0	fCNS11643	5-2B49
U+46AF.0	fR/S	 149.2
U+46AF.0	fTotalStrokes	 9
U+46AF.0	fCangjie	YRLN
U+46AF.0	fFourCorner	02600
U+46AF.0	fCantonese	tou2
U+46AF.0	fMandarin	xuan1; tao3; zhen4
U+46AF.0	fEnglish	ancient form of &#x8A0E;
U+46B0.0	fUTF8	䚰
U+46B0.0	fCNS11643	3-2F2D
U+46B0.0	fGB	5-7152
U+46B0.0	fR/S	 149.2
U+46B0.0	fTotalStrokes	 9
U+46B0.0	fCangjie	YRP
U+46B0.0	fFourCorner	01610
U+46B0.0	fCantonese	pei2
U+46B0.0	fMandarin	pi3
U+46B0.0	fEnglish	[1] prepare [2] all complete; all ready [3] corrupted and abbreviated form of &#x8A1B;
U+46B1.0	fUTF8	䚱
U+46B1.0	fCNS11643	4-2F7E
U+46B1.0	fGB	5-7158
U+46B1.0	fR/S	 149.3
U+46B1.0	fTotalStrokes	 10
U+46B1.0	fCangjie	YRF
U+46B1.0	fFourCorner	09600
U+46B1.0	fCantonese	seon3; caan1
U+46B1.0	fMandarin	xin4
U+46B1.0	fEnglish	ancient form of &#x4FE1;
U+46B2.0	fUTF8	䚲
U+46B2.0	fCNS11643	3-337D
U+46B2.0	fR/S	 149.3
U+46B2.0	fTotalStrokes	 10
U+46B2.0	fCangjie	YRHHH
U+46B2.0	fFourCorner	02620
U+46B2.0	fHakka	 san5
U+46B2.0	fCantonese	saan3
U+46B2.0	fMandarin	shan4
U+46B2.0	fEnglish	non-classical form of &#x8A15;
U+46B3.0	fUTF8	䚳
U+46B3.0	fCNS11643	4-3541
U+46B3.0	fGB	5-715B
U+46B3.0	fR/S	 149.4
U+46B3.0	fTotalStrokes	 11
U+46B3.0	fCangjie	YRJE
U+46B3.0	fFourCorner	04640
U+46B3.0	fCantonese	zi3
U+46B3.0	fMandarin	zhi4; xie4
U+46B3.0	fEnglish	do not know, to know nothing about, quickly; fast, sharp; keen
U+46B4.0	fUTF8	䚴
U+46B4.0	fCNS11643	3-387C
U+46B4.0	fGB	5-7165
U+46B4.0	fR/S	 149.4
U+46B4.0	fTotalStrokes	 11
U+46B4.0	fCangjie	YRB
U+46B4.0	fFourCorner	07620
U+46B4.0	fCantonese	waat6
U+46B4.0	fMandarin	yue4; wa4
U+46B4.0	fEnglish	anger; rage; furious, to scold in a loud voice; to upbraid or revile
U+46B5.0	fUTF8	䚵
U+46B5.0	fCNS11643	3-3878
U+46B5.0	fR/S	 149.4
U+46B5.0	fTotalStrokes	 11
U+46B5.0	fCangjie	YRYJ
U+46B5.0	fFourCorner	04600
U+46B5.0	fHakka	  teu3
U+46B5.0	fCantonese	tau2
U+46B5.0	fMandarin	tou3; tao3
U+46B5.0	fEnglish	 beguile
U+46B6.0	fUTF8	䚶
U+46B6.0	fCNS11643	5-3557
U+46B6.0	fGB	3-7261
U+46B6.0	fR/S	 149.4
U+46B6.0	fTotalStrokes	 11
U+46B6.0	fCangjie	YRMK
U+46B6.0	fFourCorner	01630
U+46B6.0	fCantonese	tin1
U+46B6.0	fEnglish	 scold in a loud voice
U+46B7.0	fUTF8	䚷
U+46B7.0	fCNS11643	4-3544
U+46B7.0	fGB	3-7269
U+46B7.0	fR/S	 149.4
U+46B7.0	fTotalStrokes	 11
U+46B7.0	fCangjie	YRCMS
U+46B7.0	fFourCorner	08620
U+46B7.0	fCantonese	ji1; zi1
U+46B7.0	fMandarin	yi1; xi4; yi3; yi4
U+46B7.0	fEnglish	[1] treat; detain [2] according to one's wishes [3] good words; honest; sincere words [4] echo [5] joke; witticism; pleasantry; jest; fun
U+46B8.0	fUTF8	䚸
U+46B8.0	fCNS11643	5-355E
U+46B8.0	fR/S	 149.4
U+46B8.0	fTotalStrokes	 11
U+46B8.0	fCangjie	YROLL
U+46B8.0	fFourCorner	08620
U+46B8.0	fCantonese	haai6
U+46B8.0	fMandarin	xie4
U+46B8.0	fEnglish	good; virtuous
U+46B9.0	fUTF8	䚹
U+46B9.0	fCNS11643	3-3924
U+46B9.0	fGB	5-715E
U+46B9.0	fR/S	 149.4
U+46B9.0	fTotalStrokes	 11
U+46B9.0	fCangjie	YRPP
U+46B9.0	fFourCorner	01610
U+46B9.0	fCantonese	pei2
U+46B9.0	fMandarin	pi3
U+46B9.0	fEnglish	same as &#x5E80;
U+46BA.0	fUTF8	䚺
U+46BA.0	fCNS11643	3-3928
U+46BA.0	fR/S	 149.4
U+46BA.0	fTotalStrokes	 11
U+46BA.0	fCangjie	YROK
U+46BA.0	fFourCorner	08640
U+46BA.0	fCantonese	jiu4; miu5; zaau4; ziu4
U+46BA.0	fMandarin	yao2
U+46BA.0	fEnglish	non-classical form of &#x8B20;
U+46BB.0	fUTF8	䚻
U+46BB.0	fCNS11643	3-3921
U+46BB.0	fGB	5-7144
U+46BB.0	fR/S	 149.4
U+46BB.0	fTotalStrokes	 11
U+46BB.0	fCangjie	BYMR
U+46BB.0	fFourCorner	27600
U+46BB.0	fHakka	 jau2; rau2; jeu2
U+46BB.0	fCantonese	jiu4; ziu4
U+46BB.0	fMandarin	yao2
U+46BB.0	fEnglish	ancient form of &#x8B20;
U+46BC.0	fUTF8	䚼
U+46BC.0	fCNS11643	4-353F
U+46BC.0	fR/S	 149.4
U+46BC.0	fTotalStrokes	 11
U+46BC.0	fCangjie	YRNG
U+46BC.0	fFourCorner	07610
U+46BC.0	fCantonese	nuk6; luk6 (also)
U+46BC.0	fMandarin	nuu4; n4
U+46BC.0	fEnglish	ashamed
U+46BD.0	fUTF8	䚽
U+46BD.0	fCNS11643	3-387D
U+46BD.0	fR/S	 149.4
U+46BD.0	fTotalStrokes	 11
U+46BD.0	fCangjie	YRHQU
U+46BD.0	fFourCorner	02610
U+46BD.0	fCantonese	hou3
U+46BD.0	fMandarin	hao4
U+46BD.0	fEnglish	interchangeable with &#x8017;
U+46BE.0	fUTF8	䚾
U+46BE.0	fCNS11643	3-3927
U+46BE.0	fGB	5-7162
U+46BE.0	fR/S	 149.4
U+46BE.0	fTotalStrokes	 11
U+46BE.0	fCangjie	YRHG
U+46BE.0	fFourCorner	02610
U+46BE.0	fHakka	 kong2
U+46BE.0	fCantonese	jam4; nip6; zaam4; zaam6
U+46BE.0	fMandarin	ren2; nin2
U+46BE.0	fEnglish	[1] think of; remember (someone) [2] read out aloud [3] same as &#x46D8;
U+46BF.0	fUTF8	䚿
U+46BF.0	fCNS11643	4-3543
U+46BF.0	fGB	3-726B
U+46BF.0	fR/S	 149.4
U+46BF.0	fTotalStrokes	 11
U+46BF.0	fCangjie	YRNO
U+46BF.0	fFourCorner	07680
U+46BF.0	fCantonese	jam6
U+46BF.0	fMandarin	yin4
U+46BF.0	fEnglish	to shout in rage; angry shouts, angry speech, to cry continuously, to groan; to moan
U+46C0.0	fUTF8	䛀
U+46C0.0	fCNS11643	5-355D
U+46C0.0	fGB	3-7268
U+46C0.0	fR/S	 149.4
U+46C0.0	fTotalStrokes	 11
U+46C0.0	fCangjie	YRHE
U+46C0.0	fFourCorner	01640
U+46C0.0	fCantonese	faan2; fan2
U+46C0.0	fMandarin	fan3
U+46C0.0	fEnglish	[1] reasonable authority statements [2] same as &#x5476;
U+46C1.0	fUTF8	䛁
U+46C1.0	fCNS11643	6-446D
U+46C1.0	fR/S	 149.4
U+46C1.0	fTotalStrokes	 11
U+46C1.0	fCangjie	YRBMM
U+46C1.0	fFourCorner	07650
U+46C1.0	fHakka	 ngiam2; ngam2
U+46C1.0	fCantonese	naam4; laam4 (also); nam4
U+46C1.0	fMandarin	ran2; nan2
U+46C1.0	fEnglish	interchangeable with &#x5583;, &#x8AF5;
U+46C2.0	fUTF8	䛂
U+46C2.0	fCNS11643	3-3926
U+46C2.0	fR/S	 149.4
U+46C2.0	fTotalStrokes	 11
U+46C2.0	fCangjie	YRHT
U+46C2.0	fFourCorner	04600
U+46C2.0	fCantonese	ci4; ci2
U+46C2.0	fMandarin	chi3; chi2
U+46C2.0	fEnglish	[1] same as &#x8A83;, &#x8B3B; [2] same as &#x8A51;
U+46C3.0	fUTF8	䛃
U+46C3.0	fCNS11643	5-3556
U+46C3.0	fR/S	 149.4
U+46C3.0	fTotalStrokes	 11
U+46C3.0	fCangjie	YRMMU
U+46C3.0	fFourCorner	01610
U+46C3.0	fCantonese	wun6
U+46C3.0	fMandarin	wan4; wang4
U+46C3.0	fEnglish	used in person's name
U+46C4.0	fUTF8	䛄
U+46C4.0	fCNS11643	4-3B33
U+46C4.0	fGB	5-7179
U+46C4.0	fR/S	 149.5
U+46C4.0	fTotalStrokes	 12
U+46C4.0	fCangjie	YRNIU
U+46C4.0	fFourCorner	07610
U+46C4.0	fHakka	 wan3
U+46C4.0	fCantonese	jyun3
U+46C4.0	fMandarin	yuan4; yuan3
U+46C4.0	fEnglish	to console; to comfort; to soothe; to show sympathy for, ill-will; enmity; animus
U+46C5.0	fUTF8	䛅
U+46C5.0	fCNS11643	3-3E79
U+46C5.0	fGB	5-716D
U+46C5.0	fR/S	 149.5
U+46C5.0	fTotalStrokes	 12
U+46C5.0	fCangjie	YRWL
U+46C5.0	fFourCorner	06650
U+46C5.0	fCantonese	haap3
U+46C5.0	fMandarin	xia2
U+46C5.0	fEnglish	loquacious
U+46C6.0	fUTF8	䛆
U+46C6.0	fCNS11643	3-3E7A
U+46C6.0	fGB	5-716F
U+46C6.0	fR/S	 149.5
U+46C6.0	fTotalStrokes	 12
U+46C6.0	fCangjie	YRLW
U+46C6.0	fFourCorner	05660
U+46C6.0	fHakka	 ciu5; ciu6
U+46C6.0	fCantonese	zau6; zaau3
U+46C6.0	fMandarin	zhou4
U+46C6.0	fEnglish	 ask blessings for; pray for
U+46C7.0	fUTF8	䛇
U+46C7.0	fCNS11643	4-3B36
U+46C7.0	fGB	3-7277
U+46C7.0	fR/S	 149.5
U+46C7.0	fTotalStrokes	 12
U+46C7.0	fCangjie	YRCR
U+46C7.0	fFourCorner	08660
U+46C7.0	fCantonese	jin5
U+46C7.0	fMandarin	yan3; yuan3
U+46C7.0	fEnglish	to laugh; to smile; to grin; to giggle; to titter; to chuckle, well-intentioned advice
U+46C8.0	fUTF8	䛈
U+46C8.0	fCNS11643	4-3B35
U+46C8.0	fGB	5-7173
U+46C8.0	fR/S	 149.5
U+46C8.0	fTotalStrokes	 12
U+46C8.0	fCangjie	YROK
U+46C8.0	fFourCorner	08630
U+46C8.0	fCantonese	si3; hin1; saai6
U+46C8.0	fMandarin	shi4
U+46C8.0	fEnglish	interchangeable with &#x8A93;
U+46C9.0	fUTF8	䛉
U+46C9.0	fCNS11643	3-3E76
U+46C9.0	fR/S	 149.5
U+46C9.0	fTotalStrokes	 12
U+46C9.0	fCangjie	YRRVP
U+46C9.0	fFourCorner	07640
U+46C9.0	fCantonese	min6
U+46C9.0	fMandarin	mian4; mi4
U+46C9.0	fEnglish	to induce by words
U+46CA.0	fUTF8	䛊
U+46CA.0	fCNS11643	3-3E7B
U+46CA.0	fR/S	 149.5
U+46CA.0	fTotalStrokes	 12
U+46CA.0	fCangjie	YRRC
U+46CA.0	fFourCorner	06680
U+46CA.0	fCantonese	hei1
U+46CA.0	fMandarin	zhi3
U+46CA.0	fEnglish	cheerful all the time; without stopping, to tell; to say, to designated, to think
U+46CB.0	fUTF8	䛋
U+46CB.0	fCNS11643	3-3E77
U+46CB.0	fR/S	 149.5
U+46CB.0	fTotalStrokes	 12
U+46CB.0	fCangjie	YRIH
U+46CB.0	fFourCorner	03650
U+46CB.0	fCantonese	gei6
U+46CB.0	fMandarin	ci4; ge2; ji4
U+46CB.0	fEnglish	to scheme; to plan; to plot, to seek; to try to get
U+46CC.0	fUTF8	䛌
U+46CC.0	fCNS11643	4-3B34
U+46CC.0	fGB	3-7278
U+46CC.0	fR/S	 149.5
U+46CC.0	fTotalStrokes	 12
U+46CC.0	fCangjie	YRPRU
U+46CC.0	fFourCorner	07610
U+46CC.0	fCantonese	tou4
U+46CC.0	fMandarin	pao2; tao2
U+46CC.0	fEnglish	[1] provoke or arouse (ill-will, etc.); cause disputes; sow discord between two parties [2] talk nonsense; talk inconherently or unintelligibly; lies or wild talks
U+46CD.0	fUTF8	䛍
U+46CD.0	fCNS11643	4-3B2A
U+46CD.0	fGB	3-727D
U+46CD.0	fR/S	 149.5
U+46CD.0	fTotalStrokes	 12
U+46CD.0	fCangjie	YRLLN
U+46CD.0	fFourCorner	05620
U+46CD.0	fHakka	 fui5; fi5
U+46CD.0	fCantonese	fai3
U+46CD.0	fMandarin	fei4
U+46CD.0	fEnglish	 talk fast
U+46CE.0	fUTF8	䛎
U+46CE.0	fCNS11643	4-3B29
U+46CE.0	fGB	3-727A
U+46CE.0	fR/S	 149.5
U+46CE.0	fTotalStrokes	 12
U+46CE.0	fCangjie	YRJC
U+46CE.0	fFourCorner	03610
U+46CE.0	fCantonese	hyut3
U+46CE.0	fMandarin	xue4; hu4; yu4
U+46CE.0	fEnglish	to shout in rage; to scold in anger
U+46CF.0	fUTF8	䛏
U+46CF.0	fCNS11643	4-3B2B
U+46CF.0	fGB	3-727C
U+46CF.0	fR/S	 149.5
U+46CF.0	fTotalStrokes	 12
U+46CF.0	fCangjie	YRSP
U+46CF.0	fFourCorner	07610
U+46CF.0	fCantonese	nai4; lai4 (also); nei4
U+46CF.0	fMandarin	ni2
U+46CF.0	fEnglish	[1] call somebody [2] same as &#x6CE5;
U+46D0.0	fUTF8	䛐
U+46D0.0	fCNS11643	3-3F26
U+46D0.0	fR/S	 149.5
U+46D0.0	fTotalStrokes	 12
U+46D0.0	fCangjie	SRYMR
U+46D0.0	fFourCorner	17600
U+46D0.0	fCantonese	ci4
U+46D0.0	fMandarin	ci2
U+46D0.0	fEnglish	same as &#x8A5E;
U+46D1.0	fUTF8	䛑
U+46D1.0	fCNS11643	6-4E6B
U+46D1.0	fR/S	 149.5
U+46D1.0	fTotalStrokes	 12
U+46D1.0	fCangjie	YRPH
U+46D1.0	fFourCorner	03630
U+46D1.0	fCantonese	mat6
U+46D1.0	fMandarin	mi4
U+46D1.0	fEnglish	same as &#x8B10;
U+46D2.0	fUTF8	䛒
U+46D2.0	fCNS11643	3-3F25
U+46D2.0	fGB	5-7145
U+46D2.0	fR/S	 149.5
U+46D2.0	fTotalStrokes	 12
U+46D2.0	fCangjie	MSYMR
U+46D2.0	fFourCorner	11600
U+46D2.0	fCantonese	bin6
U+46D2.0	fMandarin	bian4
U+46D2.0	fEnglish	standard form of &#x8FAF;
U+46D3.0	fUTF8	䛓
U+46D3.0	fGB	S-2165
U+46D3.0	fR/S	 149.5
U+46D3.0	fTotalStrokes	 12
U+46D3.0	fCangjie	LIYMR
U+46D3.0	fCantonese	gaam1; gam1; ham4
U+46D3.0	fEnglish	[1] abbreviated form of &#x8B7C; [2] ancient form of &#x76E3;
U+46D4.0	fUTF8	䛔
U+46D4.0	fCNS11643	3-4457
U+46D4.0	fR/S	 149.6
U+46D4.0	fTotalStrokes	 13
U+46D4.0	fCangjie	YRMBL
U+46D4.0	fFourCorner	01620
U+46D4.0	fCantonese	naa4; laa4 (also)
U+46D4.0	fMandarin	na2
U+46D4.0	fEnglish	[1] hold in hand; grasp; take [2] arrest; use [3] non-classical form of &#x8ABD;
U+46D5.0	fUTF8	䛕
U+46D5.0	fCNS11643	6-5841
U+46D5.0	fR/S	 149.6
U+46D5.0	fTotalStrokes	 13
U+46D5.0	fCangjie	YRLWK
U+46D5.0	fFourCorner	05640
U+46D5.0	fCantonese	jyu4; naa4; zyu4
U+46D5.0	fMandarin	yu2; yu4
U+46D5.0	fEnglish	non-classical form of &#x8ADB;
U+46D6.0	fUTF8	䛖
U+46D6.0	fCNS11643	3-445E
U+46D6.0	fR/S	 149.6
U+46D6.0	fTotalStrokes	 13
U+46D6.0	fCangjie	YRLWP
U+46D6.0	fFourCorner	05650
U+46D6.0	fCantonese	jai6; zaai6
U+46D6.0	fMandarin	yi4; e4
U+46D6.0	fEnglish	same as &#x8A4D;
U+46D7.0	fUTF8	䛗
U+46D7.0	fCNS11643	4-406E
U+46D7.0	fGB	3-605D
U+46D7.0	fR/S	 149.6
U+46D7.0	fTotalStrokes	 13
U+46D7.0	fCangjie	SLYMR
U+46D7.0	fFourCorner	70760
U+46D7.0	fCantonese	zi2
U+46D7.0	fMandarin	zhi3
U+46D7.0	fEnglish	to pry into or expose another's secret; to gossip about a man's private affairs or about his faults, to accuse or charge
U+46D8.0	fUTF8	䛘
U+46D8.0	fCNS11643	4-406D
U+46D8.0	fGB	3-732B
U+46D8.0	fR/S	 149.6
U+46D8.0	fTotalStrokes	 13
U+46D8.0	fCangjie	YROHG
U+46D8.0	fFourCorner	02610
U+46D8.0	fCantonese	jam4; zaam4; zim4
U+46D8.0	fMandarin	ren2; nin2
U+46D8.0	fEnglish	[1] think of; remember (someone) [2] read out aloud [3] loquacious [4] guttural sound [5] noise; uproar; turmoil
U+46D9.0	fUTF8	䛙
U+46D9.0	fCNS11643	4-406C
U+46D9.0	fGB	3-732A
U+46D9.0	fR/S	 149.6
U+46D9.0	fTotalStrokes	 13
U+46D9.0	fCangjie	YROD
U+46D9.0	fFourCorner	04690
U+46D9.0	fHakka	 hiu1
U+46D9.0	fCantonese	juk1; cuk6
U+46D9.0	fMandarin	xu4
U+46D9.0	fEnglish	 smell the fragrance
U+46DA.0	fUTF8	䛚
U+46DA.0	fCNS11643	3-445A
U+46DA.0	fR/S	 149.6
U+46DA.0	fTotalStrokes	 13
U+46DA.0	fCangjie	QHYMR
U+46DA.0	fFourCorner	57600
U+46DA.0	fHakka	 liok8
U+46DA.0	fCantonese	loek6
U+46DA.0	fMandarin	luue4; le4
U+46DA.0	fEnglish	[1] praise [2] extol
U+46DB.0	fUTF8	䛛
U+46DB.0	fCNS11643	3-445F
U+46DB.0	fR/S	 149.6
U+46DB.0	fTotalStrokes	 13
U+46DB.0	fCangjie	YRWR
U+46DB.0	fFourCorner	06600
U+46DB.0	fCantonese	fui3
U+46DB.0	fMandarin	hui4; qi4
U+46DB.0	fEnglish	same as &#x8A6F;
U+46DC.0	fUTF8	䛜
U+46DC.0	fCNS11643	3-4462
U+46DC.0	fGB	5-7230
U+46DC.0	fR/S	 149.6
U+46DC.0	fTotalStrokes	 13
U+46DC.0	fCangjie	YRHWK
U+46DC.0	fFourCorner	06600
U+46DC.0	fCantonese	seon3
U+46DC.0	fMandarin	xun4
U+46DC.0	fEnglish	same as &#x8A0A;
U+46DD.0	fUTF8	䛝
U+46DD.0	fCNS11643	4-473D
U+46DD.0	fGB	3-7348
U+46DD.0	fR/S	 149.7
U+46DD.0	fTotalStrokes	 14
U+46DD.0	fCangjie	YRNGU
U+46DD.0	fFourCorner	07670
U+46DD.0	fCantonese	nou4; lou4 (also)
U+46DD.0	fMandarin	nao2
U+46DD.0	fEnglish	to like, to puzzle
U+46DE.0	fUTF8	䛞
U+46DE.0	fCNS11643	4-4740
U+46DE.0	fGB	3-733E
U+46DE.0	fR/S	 149.7
U+46DE.0	fTotalStrokes	 14
U+46DE.0	fCangjie	YRAMJ
U+46DE.0	fFourCorner	06640
U+46DE.0	fCantonese	hon5
U+46DE.0	fMandarin	han4; han3
U+46DE.0	fEnglish	[1] loquacity [2] boasting; hyperbole [3] talk harshly; shout angrily
U+46DF.0	fUTF8	䛟
U+46DF.0	fCNS11643	4-474A
U+46DF.0	fGB	3-7324
U+46DF.0	fR/S	 149.7
U+46DF.0	fTotalStrokes	 14
U+46DF.0	fCangjie	YRKOO
U+46DF.0	fFourCorner	04630
U+46DF.0	fCantonese	gip3; gap3
U+46DF.0	fMandarin	jia2
U+46DF.0	fEnglish	same as &#x550A;
U+46E0.0	fUTF8	䛠
U+46E0.0	fCNS11643	4-473C
U+46E0.0	fGB	3-733B
U+46E0.0	fR/S	 149.7
U+46E0.0	fTotalStrokes	 14
U+46E0.0	fCangjie	YRMRT
U+46E0.0	fFourCorner	01610
U+46E0.0	fCantonese	dau3; daau2; daau6
U+46E0.0	fMandarin	dou4; xiang2
U+46E0.0	fEnglish	[1] unable to talk [2] incapable; inefficient
U+46E1.0	fUTF8	䛡
U+46E1.0	fCNS11643	3-496A
U+46E1.0	fR/S	 149.7
U+46E1.0	fTotalStrokes	 14
U+46E1.0	fCangjie	YRHPR
U+46E1.0	fFourCorner	02660
U+46E1.0	fCantonese	waa6
U+46E1.0	fMandarin	hua4
U+46E1.0	fEnglish	ancient form of &#x8A71;
U+46E2.0	fUTF8	䛢
U+46E2.0	fCNS11643	4-4746
U+46E2.0	fGB	3-7342
U+46E2.0	fR/S	 149.7
U+46E2.0	fTotalStrokes	 14
U+46E2.0	fCangjie	YRHDN
U+46E2.0	fFourCorner	02610
U+46E2.0	fHakka	  tut7
U+46E2.0	fCantonese	tuk1
U+46E2.0	fMandarin	tu1
U+46E2.0	fEnglish	cunning talk; slander
U+46E3.0	fUTF8	䛣
U+46E3.0	fCNS11643	4-4742
U+46E3.0	fGB	3-733F
U+46E3.0	fR/S	 149.7
U+46E3.0	fTotalStrokes	 14
U+46E3.0	fCangjie	YRLWS
U+46E3.0	fFourCorner	05620
U+46E3.0	fCantonese	ping1
U+46E3.0	fMandarin	ping1
U+46E3.0	fEnglish	speech; words, to say; to talk, negative, hesitate
U+46E4.0	fUTF8	䛤
U+46E4.0	fCNS11643	3-4961
U+46E4.0	fGB	5-723B
U+46E4.0	fR/S	 149.7
U+46E4.0	fTotalStrokes	 14
U+46E4.0	fCangjie	YRRYO
U+46E4.0	fFourCorner	06680
U+46E4.0	fCantonese	cuk1; zok6
U+46E4.0	fMandarin	cu4
U+46E4.0	fEnglish	[1] urgently; hastily; hurriedly [2] hasty speech
U+46E5.0	fUTF8	䛥
U+46E5.0	fCNS11643	4-4743
U+46E5.0	fGB	3-7344
U+46E5.0	fR/S	 149.7
U+46E5.0	fTotalStrokes	 14
U+46E5.0	fCangjie	YRKKB
U+46E5.0	fFourCorner	04620
U+46E5.0	fCantonese	hat1; hei1
U+46E5.0	fMandarin	xi4
U+46E5.0	fEnglish	[1] speech; word [2] speech sound [3] tone (of one's speech) [4] boasting
U+46E6.0	fUTF8	䛦
U+46E6.0	fCNS11643	3-4969
U+46E6.0	fR/S	 149.7
U+46E6.0	fTotalStrokes	 14
U+46E6.0	fCangjie	YRCOR
U+46E6.0	fFourCorner	08660
U+46E6.0	fCantonese	zung6
U+46E6.0	fMandarin	song4
U+46E6.0	fEnglish	ancient form of &#x8A1F;
U+46E7.0	fUTF8	䛧
U+46E7.0	fCNS11643	3-4967
U+46E7.0	fR/S	 149.7
U+46E7.0	fTotalStrokes	 14
U+46E7.0	fCangjie	YRVFD
U+46E7.0	fFourCorner	09610
U+46E7.0	fCantonese	mai4
U+46E7.0	fMandarin	mi2
U+46E7.0	fEnglish	corrupted form of &#x8B0E;
U+46E8.0	fUTF8	䛨
U+46E8.0	fCNS11643	3-495D
U+46E8.0	fR/S	 149.7
U+46E8.0	fTotalStrokes	 14
U+46E8.0	fCangjie	YRYTJ
U+46E8.0	fFourCorner	00640
U+46E8.0	fCantonese	seon3
U+46E8.0	fMandarin	xin4
U+46E9.0	fUTF8	䛩
U+46E9.0	fCNS11643	4-4D59
U+46E9.0	fGB	3-7323
U+46E9.0	fR/S	 149.8
U+46E9.0	fTotalStrokes	 15
U+46E9.0	fCangjie	YRMLM
U+46E9.0	fFourCorner	01610
U+46E9.0	fCantonese	wu2; kui2; ok3; wu3
U+46E9.0	fMandarin	wu3; wu4
U+46E9.0	fEnglish	[1] slander; defame [2] same as &#x60E1; [3] same as &#x555E;
U+46EA.0	fUTF8	䛪
U+46EA.0	fCNS11643	4-4D56
U+46EA.0	fGB	5-724E
U+46EA.0	fR/S	 149.8
U+46EA.0	fTotalStrokes	 15
U+46EA.0	fCangjie	YRJCN
U+46EA.0	fFourCorner	03620
U+46EA.0	fHakka	 giung5
U+46EA.0	fCantonese	hung3; kung4
U+46EA.0	fMandarin	qiong4; qiong2; wei3
U+46EA.0	fEnglish	[1] question [2] prolix [3] wordy
U+46EB.0	fUTF8	䛫
U+46EB.0	fCNS11643	4-4D5C
U+46EB.0	fGB	5-7163
U+46EB.0	fR/S	 149.8
U+46EB.0	fTotalStrokes	 15
U+46EB.0	fCangjie	YRSMV
U+46EB.0	fFourCorner	01630
U+46EB.0	fCantonese	zoeng1
U+46EB.0	fMandarin	zhang1; zheng4
U+46EB.0	fEnglish	[1] deceive; delude [2] careless; lax; heedless
U+46EC.0	fUTF8	䛬
U+46EC.0	fCNS11643	4-4D5E
U+46EC.0	fGB	3-7355
U+46EC.0	fR/S	 149.8
U+46EC.0	fTotalStrokes	 15
U+46EC.0	fCangjie	YRPOU
U+46EC.0	fFourCorner	07620
U+46EC.0	fCantonese	tou4
U+46EC.0	fMandarin	tao2; chou2
U+46EC.0	fEnglish	[1] baby talk [2] pray [3] felicitate
U+46ED.0	fUTF8	䛭
U+46ED.0	fCNS11643	3-4E68
U+46ED.0	fGB	5-7243
U+46ED.0	fR/S	 149.8
U+46ED.0	fTotalStrokes	 15
U+46ED.0	fCangjie	YRGTJ
U+46ED.0	fFourCorner	04640
U+46ED.0	fHakka	 hen5; hen6
U+46ED.0	fCantonese	hang6
U+46ED.0	fMandarin	xing4
U+46ED.0	fEnglish	angry words; reproof
U+46EE.0	fUTF8	䛮
U+46EE.0	fCNS11643	4-4D61
U+46EE.0	fGB	5-724A
U+46EE.0	fR/S	 149.8
U+46EE.0	fTotalStrokes	 15
U+46EE.0	fCangjie	YRHOR
U+46EE.0	fFourCorner	08660
U+46EE.0	fCantonese	kau5; gaau3; saat1
U+46EE.0	fMandarin	jiu4
U+46EE.0	fEnglish	[1] destroy [2] slander; abuse; defame [3] non-classical form of &#x548E;
U+46EF.0	fUTF8	䛯
U+46EF.0	fCNS11643	4-4D58
U+46EF.0	fR/S	 149.8
U+46EF.0	fTotalStrokes	 15
U+46EF.0	fCangjie	YRSJR
U+46EF.0	fFourCorner	07660
U+46EF.0	fCantonese	geoi3
U+46EF.0	fMandarin	ju4
U+46EF.0	fEnglish	the reasonable of speeking
U+46F0.0	fUTF8	䛰
U+46F0.0	fCNS11643	4-4D5B
U+46F0.0	fGB	5-7246
U+46F0.0	fR/S	 149.8
U+46F0.0	fTotalStrokes	 15
U+46F0.0	fCangjie	YRAPP
U+46F0.0	fFourCorner	06610
U+46F0.0	fCantonese	wan6
U+46F0.0	fMandarin	hun4; hun2
U+46F0.0	fEnglish	(non-classical) sketchy; not clear speech
U+46F1.0	fUTF8	䛱
U+46F1.0	fCNS11643	3-4E6D
U+46F1.0	fR/S	 149.8
U+46F1.0	fTotalStrokes	 15
U+46F1.0	fCangjie	YRLP
U+46F1.0	fFourCorner	05630
U+46F1.0	fCantonese	tai4
U+46F1.0	fMandarin	ti2
U+46F1.0	fEnglish	to talk unremittingly
U+46F2.0	fUTF8	䛲
U+46F2.0	fCNS11643	4-4D5F
U+46F2.0	fGB	3-7353
U+46F2.0	fR/S	 149.8
U+46F2.0	fTotalStrokes	 15
U+46F2.0	fCangjie	YRHAB
U+46F2.0	fFourCorner	06620
U+46F2.0	fCantonese	maan4; man4; man6
U+46F2.0	fMandarin	mian2; man2; man4
U+46F2.0	fEnglish	same as &#x8B3E;
U+46F3.0	fUTF8	䛳
U+46F3.0	fCNS11643	4-4D5A
U+46F3.0	fGB	3-734D
U+46F3.0	fR/S	 149.8
U+46F3.0	fTotalStrokes	 15
U+46F3.0	fCangjie	YRKLU
U+46F3.0	fFourCorner	04610
U+46F3.0	fCantonese	jim3; zim1; zim3
U+46F3.0	fMandarin	yan4; jian3
U+46F3.0	fEnglish	[1] malign [2] hide; conceal
U+46F4.0	fUTF8	䛴
U+46F4.0	fCNS11643	3-4E6B
U+46F4.0	fGB	5-7244
U+46F4.0	fR/S	 149.8
U+46F4.0	fTotalStrokes	 15
U+46F4.0	fCangjie	YRKMR
U+46F4.0	fFourCorner	04620
U+46F4.0	fCantonese	gei1
U+46F4.0	fMandarin	ji1; qi3
U+46F4.0	fEnglish	joke; pleasantry, wild talk; a lie
U+46F5.0	fUTF8	䛵
U+46F5.0	fCNS11643	4-4D60
U+46F5.0	fGB	5-7249
U+46F5.0	fR/S	 149.8
U+46F5.0	fTotalStrokes	 15
U+46F5.0	fCangjie	YRBBE
U+46F5.0	fFourCorner	02640
U+46F5.0	fCantonese	sau6
U+46F5.0	fMandarin	shou4
U+46F5.0	fEnglish	same as &#x6388;
U+46F6.0	fUTF8	䛶
U+46F6.0	fCNS11643	7-2879
U+46F6.0	fR/S	 149.8
U+46F6.0	fTotalStrokes	 15
U+46F6.0	fCangjie	YRQJD
U+46F6.0	fFourCorner	05690
U+46F6.0	fCantonese	loi6; lok6
U+46F6.0	fMandarin	lei3
U+46F6.0	fEnglish	same as &#x8A84;
U+46F7.0	fUTF8	䛷
U+46F7.0	fCNS11643	4-4D55
U+46F7.0	fGB	5-724F
U+46F7.0	fR/S	 149.8
U+46F7.0	fTotalStrokes	 15
U+46F7.0	fCangjie	YRJNU
U+46F7.0	fFourCorner	03610
U+46F7.0	fHakka	 wan3
U+46F7.0	fCantonese	jyun2; zyun2
U+46F7.0	fMandarin	wan3
U+46F7.0	fEnglish	same as &#x5A49;
U+46F8.0	fUTF8	䛸
U+46F8.0	fCNS11643	3-4E71
U+46F8.0	fR/S	 149.8
U+46F8.0	fTotalStrokes	 15
U+46F8.0	fCangjie	YRIYR
U+46F8.0	fFourCorner	00660
U+46F8.0	fCantonese	zaam1
U+46F8.0	fMandarin	che4; shan3
U+46F8.0	fEnglish	corrupted form of &#x8A40;
U+46F9.0	fUTF8	䛹
U+46F9.0	fCNS11643	4-536A
U+46F9.0	fGB	3-7368
U+46F9.0	fR/S	 149.9
U+46F9.0	fTotalStrokes	 16
U+46F9.0	fCangjie	YRVNO
U+46F9.0	fFourCorner	07630
U+46F9.0	fCantonese	sin3
U+46F9.0	fMandarin	qian4; can4
U+46F9.0	fEnglish	to accuse; to censure; to charge
U+46FA.0	fUTF8	䛺
U+46FA.0	fCNS11643	3-526C
U+46FA.0	fR/S	 149.9
U+46FA.0	fTotalStrokes	 16
U+46FA.0	fCangjie	YRWOL
U+46FA.0	fFourCorner	06620
U+46FA.0	fCantonese	gaai3
U+46FA.0	fMandarin	jie4
U+46FA.0	fEnglish	same as &#x8AA1;
U+46FB.0	fUTF8	䛻
U+46FB.0	fCNS11643	3-526D
U+46FB.0	fR/S	 149.9
U+46FB.0	fTotalStrokes	 16
U+46FB.0	fCangjie	YRHJU
U+46FB.0	fFourCorner	02660
U+46FB.0	fHakka	 ju3; riu6; ju5; riu3
U+46FB.0	fCantonese	jau5; zaau5
U+46FB.0	fMandarin	you4
U+46FB.0	fEnglish	 lead on; entice; seduce; tempt
U+46FC.0	fUTF8	䛼
U+46FC.0	fCNS11643	7-334D
U+46FC.0	fR/S	 149.9
U+46FC.0	fTotalStrokes	 16
U+46FC.0	fCangjie	YRHXM
U+46FC.0	fFourCorner	07610
U+46FC.0	fHakka	 kui3; fui3; fui5; fi5; fi3; kwi3; k'wui3
U+46FC.0	fCantonese	wai2
U+46FC.0	fMandarin	hui3
U+46FC.0	fEnglish	 slander, vilify
U+46FD.0	fUTF8	䛽
U+46FD.0	fCNS11643	4-536B
U+46FD.0	fGB	3-7362
U+46FD.0	fR/S	 149.9
U+46FD.0	fTotalStrokes	 16
U+46FD.0	fCangjie	YRHJX
U+46FD.0	fFourCorner	02670
U+46FD.0	fHakka	  cap7
U+46FD.0	fCantonese	zaap3; cap3
U+46FD.0	fMandarin	zha3
U+46FD.0	fEnglish	verbose
U+46FE.0	fUTF8	䛾
U+46FE.0	fCNS11643	3-5647
U+46FE.0	fR/S	 149.10
U+46FE.0	fTotalStrokes	 17
U+46FE.0	fCangjie	YRQMF
U+46FE.0	fFourCorner	05690
U+46FE.0	fHakka	 su5; sii5
U+46FE.0	fCantonese	sou3
U+46FE.0	fMandarin	su4
U+46FE.0	fEnglish	 apprehend
U+46FF.0	fUTF8	䛿
U+46FF.0	fCNS11643	4-5870
U+46FF.0	fGB	3-736B
U+46FF.0	fR/S	 149.10
U+46FF.0	fTotalStrokes	 17; 16
U+46FF.0	fCangjie	YRMRB
U+46FF.0	fFourCorner	01620
U+46FF.0	fCantonese	gaak3
U+46FF.0	fMandarin	ge2
U+46FF.0	fEnglish	[1] cunning; craft; sly; wily; artful [2] bright; wise
U+4700.0	fUTF8	䜀
U+4700.0	fCNS11643	3-564B
U+4700.0	fR/S	 149.10
U+4700.0	fTotalStrokes	 17
U+4700.0	fCangjie	YRSIM
U+4700.0	fFourCorner	07610
U+4700.0	fCantonese	nou5; lou5 (also)
U+4700.0	fMandarin	nao3
U+4700.0	fEnglish	 insult with words
U+4701.0	fUTF8	䜁
U+4701.0	fCNS11643	3-564C
U+4701.0	fR/S	 149.10
U+4701.0	fTotalStrokes	 17
U+4701.0	fCangjie	YRVGK
U+4701.0	fFourCorner	04630
U+4701.0	fCantonese	hai6; haai5
U+4701.0	fMandarin	xi4
U+4701.0	fEnglish	same as &#x8B11;
U+4702.0	fUTF8	䜂
U+4702.0	fCNS11643	5-5E55
U+4702.0	fGB	3-736D
U+4702.0	fR/S	 149.10
U+4702.0	fTotalStrokes	 17
U+4702.0	fCangjie	YRHAJ
U+4702.0	fFourCorner	06640
U+4702.0	fCantonese	hou4
U+4703.0	fUTF8	䜃
U+4703.0	fCNS11643	4-5D5A
U+4703.0	fGB	5-365C
U+4703.0	fR/S	 149.11
U+4703.0	fTotalStrokes	 18
U+4703.0	fCangjie	YGG
U+4703.0	fFourCorner	00100
U+4703.0	fCantonese	deoi1
U+4703.0	fMandarin	dui1
U+4703.0	fEnglish	[1] blame; reproach [2] punish [3] throw; cast [4] same as &#x5806;
U+4704.0	fUTF8	䜄
U+4704.0	fCNS11643	7-4A25
U+4704.0	fGB	3-7423
U+4704.0	fR/S	 149.11; 149.12
U+4704.0	fTotalStrokes	 19; 18
U+4704.0	fCangjie	YRSEQ
U+4704.0	fFourCorner	07650
U+4704.0	fCantonese	ci4
U+4704.0	fMandarin	chi2
U+4704.0	fEnglish	same as &#x8B18;
U+4705.0	fUTF8	䜅
U+4705.0	fCNS11643	4-5D57
U+4705.0	fGB	5-726A
U+4705.0	fR/S	 149.11
U+4705.0	fTotalStrokes	 18
U+4705.0	fCangjie	YRUOG
U+4705.0	fFourCorner	02610
U+4705.0	fCantonese	wai4
U+4705.0	fMandarin	wei2
U+4705.0	fEnglish	to follow; to comply with, to refute; to rebuke; to accuse
U+4706.0	fUTF8	䜆
U+4706.0	fCNS11643	4-5D5B
U+4706.0	fGB	3-7C24
U+4706.0	fR/S	 149.11
U+4706.0	fTotalStrokes	 18
U+4706.0	fCangjie	IDYMR
U+4706.0	fFourCorner	00260
U+4706.0	fCantonese	zip3; nip6
U+4706.0	fMandarin	zhe2; mo4; nei4; she2; sui4
U+4706.0	fEnglish	same as &#x8B98;
U+4707.0	fUTF8	䜇
U+4707.0	fCNS11643	3-5922
U+4707.0	fGB	5-7265
U+4707.0	fR/S	 149.11
U+4707.0	fTotalStrokes	 18
U+4707.0	fCangjie	YRYCV
U+4707.0	fFourCorner	00630
U+4707.0	fCantonese	gwan2
U+4707.0	fMandarin	gun3; gun4
U+4707.0	fEnglish	[1] speak not in a clear way [2] same as &#x8B34;
U+4708.0	fUTF8	䜈
U+4708.0	fCNS11643	3-5925
U+4708.0	fGB	5-7275
U+4708.0	fR/S	 149.11
U+4708.0	fTotalStrokes	 18
U+4708.0	fCangjie	YRVVD
U+4708.0	fFourCorner	02690
U+4708.0	fHakka	  cau1; ziau3; cau5
U+4708.0	fCantonese	ziu2; cau1; cau4
U+4708.0	fMandarin	chao1
U+4708.0	fEnglish	 speak on behalf of another
U+4709.0	fUTF8	䜉
U+4709.0	fCNS11643	4-5D54
U+4709.0	fGB	3-7374
U+4709.0	fR/S	 149.11
U+4709.0	fTotalStrokes	 18
U+4709.0	fCangjie	YRDOE
U+4709.0	fFourCorner	04630
U+4709.0	fCantonese	cat1
U+4709.0	fMandarin	qi1
U+4709.0	fEnglish	[1] to jest; to joke; to quip [2] same as &#x8ABA;
U+470A.0	fUTF8	䜊
U+470A.0	fCNS11643	3-5927
U+470A.0	fGB	5-7268
U+470A.0	fR/S	 149.11
U+470A.0	fTotalStrokes	 18
U+470A.0	fCangjie	YRTWA
U+470A.0	fFourCorner	05660
U+470A.0	fHakka	  zau1; cau2; cau5
U+470A.0	fCantonese	zou1
U+470A.0	fMandarin	zao1; zao4; zuo4
U+470A.0	fEnglish	[1] boast; chatter [2] noise
U+470B.0	fUTF8	䜋
U+470B.0	fCNS11643	4-6140
U+470B.0	fGB	3-735F
U+470B.0	fR/S	 149.12
U+470B.0	fTotalStrokes	 19
U+470B.0	fCangjie	YRLMC
U+470B.0	fFourCorner	05680
U+470B.0	fCantonese	wui6
U+470B.0	fMandarin	hui4
U+470B.0	fEnglish	to suspend; to interrupt, to become aware; to realize
U+470C.0	fUTF8	䜌
U+470C.0	fCNS11643	5-6952
U+470C.0	fR/S	 149.12
U+470C.0	fTotalStrokes	 19
U+470C.0	fCangjie	VFYRF
U+470C.0	fFourCorner	22993
U+470C.0	fHakka	  lien2;  lan2
U+470C.0	fCantonese	lyun4
U+470C.0	fMandarin	luan2
U+470C.0	fEnglish	[1] (ancient form) [2] chaos; distraction; confusion; confused [3] continuous; uninterrupted; tie together [4] manage
U+470D.0	fUTF8	䜍
U+470D.0	fCNS11643	3-5B33
U+470D.0	fR/S	 149.12
U+470D.0	fTotalStrokes	 19
U+470D.0	fCangjie	YRKCF
U+470D.0	fFourCorner	04690
U+470D.0	fHakka	 liau2
U+470D.0	fCantonese	liu4
U+470D.0	fMandarin	liao2
U+470D.0	fEnglish	an artful specious talk
U+470E.0	fUTF8	䜎
U+470E.0	fCNS11643	4-613B
U+470E.0	fGB	3-7338
U+470E.0	fR/S	 149.12
U+470E.0	fTotalStrokes	 19
U+470E.0	fCangjie	YRFFS
U+470E.0	fFourCorner	09620
U+470E.0	fCantonese	lou4
U+470E.0	fMandarin	lao2; lao4
U+470E.0	fEnglish	[1] sound [2] noisy; full of confused noises; clamorous
U+470F.0	fUTF8	䜏
U+470F.0	fCNS11643	4-613D
U+470F.0	fGB	5-7274
U+470F.0	fR/S	 149.12
U+470F.0	fTotalStrokes	 19
U+470F.0	fCangjie	YRNLB
U+470F.0	fFourCorner	04620
U+470F.0	fCantonese	to1
U+470F.0	fMandarin	tuo1
U+470F.0	fEnglish	to withdraw; to decline, clever
U+4710.0	fUTF8	䜐
U+4710.0	fCNS11643	4-6145
U+4710.0	fGB	3-7257
U+4710.0	fR/S	 149.12
U+4710.0	fTotalStrokes	 19
U+4710.0	fCangjie	NBYMR
U+4710.0	fFourCorner	74600
U+4710.0	fCantonese	fai1
U+4710.0	fMandarin	sui4
U+4710.0	fEnglish	to libel; to slander, to make escuses; to shirk (responsibility, etc.)
U+4711.0	fUTF8	䜑
U+4711.0	fCNS11643	5-6951
U+4711.0	fR/S	 149.12
U+4711.0	fTotalStrokes	 19
U+4711.0	fCangjie	YRMMP
U+4711.0	fFourCorner	01630
U+4711.0	fCantonese	wu3; mung4
U+4711.0	fMandarin	wu4; qia4
U+4711.0	fEnglish	[1] same as &#x46E9; [2] same as &#x60E1;
U+4712.0	fUTF8	䜒
U+4712.0	fCNS11643	4-644E
U+4712.0	fGB	3-7421
U+4712.0	fR/S	 149.13
U+4712.0	fTotalStrokes	 20
U+4712.0	fCangjie	YRHBK
U+4712.0	fFourCorner	07630
U+4712.0	fCantonese	ou3; ngou3 (also)
U+4712.0	fMandarin	ao4
U+4712.0	fEnglish	[1] expression; sentence [2] speech [3] tell; inform; accuse; report
U+4713.0	fUTF8	䜓
U+4713.0	fCNS11643	3-5D2D
U+4713.0	fR/S	 149.13
U+4713.0	fTotalStrokes	 20
U+4713.0	fCangjie	YRTPD
U+4713.0	fFourCorner	04690
U+4713.0	fCantonese	sip3
U+4713.0	fMandarin	she4; nie4
U+4713.0	fEnglish	a slip of the tongue
U+4714.0	fUTF8	䜔
U+4714.0	fCNS11643	7-5031
U+4714.0	fR/S	 149.13
U+4714.0	fTotalStrokes	 20
U+4714.0	fCangjie	YRYKB
U+4714.0	fFourCorner	04630
U+4714.0	fCantonese	ceoi4; neot6
U+4714.0	fMandarin	tuo1; sui2
U+4714.0	fEnglish	[1] speak on the ideas of someone else; speak without much thought of ones own [2] obey
U+4715.0	fUTF8	䜕
U+4715.0	fCNS11643	4-644C
U+4715.0	fGB	3-725A
U+4715.0	fR/S	 149.13
U+4715.0	fTotalStrokes	 20
U+4715.0	fCangjie	YRTWB
U+4715.0	fFourCorner	04620
U+4715.0	fHakka	 mai5; mai6
U+4715.0	fCantonese	maai6
U+4715.0	fMandarin	mai4
U+4715.0	fEnglish	 brag
U+4716.0	fUTF8	䜖
U+4716.0	fCNS11643	3-5D2F
U+4716.0	fR/S	 149.13
U+4716.0	fTotalStrokes	 20
U+4716.0	fCangjie	YRMWC
U+4716.0	fFourCorner	01680
U+4716.0	fHakka	  tam2
U+4716.0	fCantonese	taam4; dam6; tam4
U+4716.0	fMandarin	tan2; tan4
U+4716.0	fEnglish	 talk much; boast
U+4717.0	fUTF8	䜗
U+4717.0	fCNS11643	4-6448
U+4717.0	fGB	3-7424
U+4717.0	fR/S	 149.13
U+4717.0	fTotalStrokes	 20
U+4717.0	fCangjie	YRIRP
U+4717.0	fFourCorner	03650
U+4717.0	fCantonese	jam3
U+4717.0	fMandarin	xin4
U+4717.0	fEnglish	angry words
U+4718.0	fUTF8	䜘
U+4718.0	fCNS11643	3-5D31
U+4718.0	fR/S	 149.13
U+4718.0	fTotalStrokes	 20
U+4718.0	fCangjie	YRTRK
U+4718.0	fFourCorner	08640
U+4718.0	fHakka	 gin3
U+4718.0	fCantonese	ging2
U+4718.0	fMandarin	jing3
U+4718.0	fEnglish	same as &#x8B66;
U+4719.0	fUTF8	䜙
U+4719.0	fCNS11643	4-673D
U+4719.0	fGB	3-742D
U+4719.0	fR/S	 149.14
U+4719.0	fTotalStrokes	 21
U+4719.0	fCangjie	YRAVF
U+4719.0	fFourCorner	06630
U+4719.0	fHakka	 ngam2
U+4719.0	fCantonese	ngam4; am4 (also)
U+4719.0	fMandarin	an2
U+4719.0	fEnglish	silly; foolish-gibes
U+471A.0	fUTF8	䜚
U+471A.0	fCNS11643	4-673F
U+471A.0	fGB	3-7426
U+471A.0	fR/S	 149.14
U+471A.0	fTotalStrokes	 21
U+471A.0	fCangjie	YRYWF
U+471A.0	fFourCorner	06630
U+471A.0	fCantonese	daap6
U+471A.0	fMandarin	ta4
U+471A.0	fEnglish	[1] speak recklessly or without forethought; wild talk [2] use words to investigate
U+471B.0	fUTF8	䜛
U+471B.0	fCNS11643	7-5532
U+471B.0	fR/S	 149.14
U+471B.0	fTotalStrokes	 21
U+471B.0	fCangjie	YRNUU
U+471B.0	fFourCorner	07610
U+471B.0	fCantonese	caam4; cam4; dan6; sang6; zan2; zan6
U+471B.0	fMandarin	chan2; chan4
U+471B.0	fEnglish	non-classical form of &#x8B92;
U+471C.0	fUTF8	䜜
U+471C.0	fCNS11643	4-673E
U+471C.0	fGB	3-742C
U+471C.0	fR/S	 149.14
U+471C.0	fTotalStrokes	 21
U+471C.0	fCangjie	YRYBU
U+471C.0	fFourCorner	01660
U+471C.0	fCantonese	wai6
U+471C.0	fMandarin	rui4; wei4
U+471C.0	fEnglish	words of anger; words of dislike or hate
U+471D.0	fUTF8	䜝
U+471D.0	fCNS11643	4-6740
U+471D.0	fGB	3-7431
U+471D.0	fR/S	 149.14
U+471D.0	fTotalStrokes	 21
U+471D.0	fCangjie	YRVVI
U+471D.0	fFourCorner	02610
U+471D.0	fCantonese	tyun2
U+471D.0	fMandarin	tui3; tuan3
U+471D.0	fEnglish	uncertain
U+471E.0	fUTF8	䜞
U+471E.0	fCNS11643	4-6739
U+471E.0	fGB	3-7332
U+471E.0	fR/S	 149.14
U+471E.0	fTotalStrokes	 21
U+471E.0	fCangjie	YRYX
U+471E.0	fFourCorner	00620
U+471E.0	fCantonese	zai1; zaai1; zi1
U+471E.0	fMandarin	ji4
U+471E.0	fEnglish	[1] dose [2] prepared medicines or drugs [3] prepare (medicines and drugs)
U+471F.0	fUTF8	䜟
U+471F.0	fCNS11643	7-5938
U+471F.0	fR/S	 149.15
U+471F.0	fTotalStrokes	 22
U+471F.0	fCangjie	YRJIM
U+471F.0	fFourCorner	03650
U+471F.0	fHakka	 ciam5
U+471F.0	fCantonese	cam3; caam3
U+471F.0	fMandarin	chen4; chen2
U+471F.0	fEnglish	non-classical form of &#x8B96;
U+4720.0	fUTF8	䜠
U+4720.0	fCNS11643	3-5F63
U+4720.0	fR/S	 149.15
U+4720.0	fTotalStrokes	 22
U+4720.0	fCangjie	YRHLC
U+4720.0	fFourCorner	02680
U+4720.0	fCantonese	zat1; zi3
U+4720.0	fMandarin	zhi4; che4
U+4720.0	fEnglish	language of the savage or barbarian
U+4721.0	fUTF8	䜡
U+4721.0	fCNS11643	4-6948
U+4721.0	fGB	3-7432
U+4721.0	fR/S	 149.15
U+4721.0	fTotalStrokes	 22; 21
U+4721.0	fCangjie	YRMBE
U+4721.0	fFourCorner	01640
U+4721.0	fCantonese	juk1
U+4721.0	fMandarin	yu4; xu4; xue4
U+4721.0	fEnglish	smell the fragrants
U+4722.0	fUTF8	䜢
U+4722.0	fCNS11643	5-764C
U+4722.0	fR/S	 149.16
U+4722.0	fTotalStrokes	 23
U+4722.0	fCangjie	YRJQP
U+4722.0	fFourCorner	03630
U+4722.0	fCantonese	hin2
U+4722.0	fMandarin	xian3
U+4722.0	fEnglish	cruel and violent
U+4723.0	fUTF8	䜣
U+4723.0	fGB	7-2230
U+4723.0	fR/S	 149.4
U+4723.0	fTotalStrokes	 11
U+4723.0	fCangjie	IVHML
U+4723.0	fCantonese	jan1; hei1; ngan4; an4 (also); aau1; aau2; zaan1; waa6; zin1
U+4723.0	fMandarin	xin1; xi1; yin2
U+4723.0	fEnglish	[1] simplified form of &#x8A22; [2] same as &#x6B23;
U+4724.0	fUTF8	䜤
U+4724.0	fGB	S-2162
U+4724.0	fR/S	 149.6
U+4724.0	fTotalStrokes	 13
U+4724.0	fCangjie	IVTC
U+4724.0	fCantonese	hung6
U+4725.0	fUTF8	䜥
U+4725.0	fGB	S-2163
U+4725.0	fR/S	 149.8
U+4725.0	fTotalStrokes	 15
U+4725.0	fCangjie	IVNKM
U+4725.0	fCantonese	daan3
U+4725.0	fEnglish	birthday; to be born, give birth to
U+4726.0	fUTF8	䜦
U+4726.0	fGB	S-2164
U+4726.0	fR/S	 149.10
U+4726.0	fTotalStrokes	 17
U+4726.0	fCangjie	IVTPA
U+4726.0	fCantonese	seon1
U+4727.0	fUTF8	䜧
U+4727.0	fGB	S-2167
U+4727.0	fR/S	 149.10
U+4727.0	fTotalStrokes	 17
U+4727.0	fCangjie	IVSIM
U+4727.0	fCantonese	nou5; lou5 (also)
U+4727.0	fMandarin	nao3
U+4727.0	fEnglish	simplified form of &#x4700;
U+4728.0	fUTF8	䜨
U+4728.0	fGB	S-2166
U+4728.0	fR/S	 149.11
U+4728.0	fTotalStrokes	 18
U+4728.0	fCangjie	IVLLP
U+4728.0	fCantonese	waan6
U+4729.0	fUTF8	䜩
U+4729.0	fGB	7-2236
U+4729.0	fR/S	 149.16
U+4729.0	fTotalStrokes	 23
U+4729.0	fCangjie	IVTLF
U+4729.0	fCantonese	jin3; zin3
U+4729.0	fMandarin	yan4
U+4729.0	fEnglish	simplified form of &#x8B8C;
U+472A.0	fUTF8	䜪
U+472A.0	fCNS11643	4-2B49
U+472A.0	fGB	5-704F
U+472A.0	fR/S	 150.2
U+472A.0	fTotalStrokes	 9
U+472A.0	fCangjie	CRKN
U+472A.0	fFourCorner	84610
U+472A.0	fCantonese	kau4
U+472A.0	fMandarin	qiu2
U+472A.0	fEnglish	the name of a pavilion in old times
U+472B.0	fUTF8	䜫
U+472B.0	fCNS11643	4-3023
U+472B.0	fGB	5-7050
U+472B.0	fR/S	 150.3
U+472B.0	fTotalStrokes	 10
U+472B.0	fCangjie	CRM
U+472B.0	fFourCorner	81610
U+472B.0	fCantonese	gong1; waang4
U+472B.0	fMandarin	jiang1; hong2
U+472B.0	fEnglish	[1] the name of a valley in today's Hobei Province [2] same as &#x8C3C;
U+472C.0	fUTF8	䜬
U+472C.0	fCNS11643	6-4E7A
U+472C.0	fR/S	 150.5
U+472C.0	fTotalStrokes	 12
U+472C.0	fCangjie	CRLW
U+472C.0	fFourCorner	85660
U+472C.0	fCantonese	zau6; caai4; zaau6
U+472C.0	fMandarin	song3; xiu4
U+472C.0	fEnglish	same as &#x5CAB;
U+472D.0	fUTF8	䜭
U+472D.0	fCNS11643	3-3F27
U+472D.0	fGB	5-705B
U+472D.0	fR/S	 150.5
U+472D.0	fTotalStrokes	 12
U+472D.0	fCangjie	YBMCR
U+472D.0	fFourCorner	21600
U+472D.0	fCantonese	zeon3
U+472D.0	fMandarin	xun4; jun4
U+472D.0	fEnglish	[1] same as &#x6FEC;, &#x6D5A; [2] same as &#x777F;
U+472E.0	fUTF8	䜮
U+472E.0	fCNS11643	4-474C
U+472E.0	fGB	3-7222
U+472E.0	fR/S	 150.7
U+472E.0	fTotalStrokes	 14
U+472E.0	fCangjie	CRJHQ
U+472E.0	fFourCorner	83650
U+472E.0	fCantonese	liu4; lou4; lou6
U+472E.0	fMandarin	lao2; liao2
U+472E.0	fEnglish	deep valley
U+472F.0	fUTF8	䜯
U+472F.0	fCNS11643	3-4E77
U+472F.0	fR/S	 150.8
U+472F.0	fTotalStrokes	 15
U+472F.0	fCangjie	CRPFD
U+472F.0	fFourCorner	87620
U+472F.0	fCantonese	guk6; guk1
U+472F.0	fMandarin	ju2
U+472F.0	fEnglish	the name of a valley in today's Shanxi province
U+4730.0	fUTF8	䜰
U+4730.0	fCNS11643	4-5875
U+4730.0	fGB	3-7223
U+4730.0	fR/S	 150.10
U+4730.0	fTotalStrokes	 17; 16
U+4730.0	fCangjie	CRHAJ
U+4730.0	fFourCorner	86640
U+4730.0	fCantonese	hou1
U+4730.0	fMandarin	qiao1; hao1
U+4730.0	fEnglish	deep valley
U+4731.0	fUTF8	䜱
U+4731.0	fCNS11643	4-5D61
U+4731.0	fGB	3-7224
U+4731.0	fR/S	 150.11
U+4731.0	fTotalStrokes	 18
U+4731.0	fCangjie	CRAWE
U+4731.0	fFourCorner	86640
U+4731.0	fCantonese	mun4
U+4731.0	fMandarin	fan2; man3
U+4731.0	fEnglish	name of a pavilion in today's Shanxi Province
U+4732.0	fUTF8	䜲
U+4732.0	fCNS11643	4-694C
U+4732.0	fGB	3-7226
U+4732.0	fR/S	 150.15
U+4732.0	fTotalStrokes	 22; 21
U+4732.0	fCangjie	CRVVV
U+4732.0	fFourCorner	82610
U+4732.0	fCantonese	lip6; laai6
U+4732.0	fMandarin	lie4
U+4732.0	fEnglish	[1] the name of a village in old China [2] the name of a valley
U+4733.0	fUTF8	䜳
U+4733.0	fR/S	 151.1
U+4733.0	fTotalStrokes	 8
U+4733.0	fCangjie	MRTV
U+4733.0	fCantonese	jyut3
U+4733.0	fEnglish	[Korean character]
U+4734.0	fUTF8	䜴
U+4734.0	fCNS11643	4-354A
U+4734.0	fGB	3-6E42
U+4734.0	fR/S	 151.4
U+4734.0	fTotalStrokes	 11
U+4734.0	fCangjie	MTYE
U+4734.0	fFourCorner	11140
U+4734.0	fCantonese	co3; si6
U+4734.0	fMandarin	chu4; shi4
U+4734.0	fEnglish	[1] brave; courageous; bold [2] for the good of [3] manage; handle, same as &#x8C49;
U+4735.0	fUTF8	䜵
U+4735.0	fCNS11643	3-3F28
U+4735.0	fR/S	 151.5
U+4735.0	fTotalStrokes	 12
U+4735.0	fCangjie	MTDHE
U+4735.0	fFourCorner	14140
U+4735.0	fCantonese	si6; cek3; zik6
U+4735.0	fMandarin	chi3; shi4
U+4735.0	fEnglish	non-classical form of &#x8C49;
U+4736.0	fUTF8	䜶
U+4736.0	fCNS11643	4-4074
U+4736.0	fGB	3-6E47
U+4736.0	fR/S	 151.6
U+4736.0	fTotalStrokes	 13
U+4736.0	fCangjie	MTHEQ
U+4736.0	fFourCorner	17150
U+4736.0	fHakka	 hong2; gong5; gong1
U+4736.0	fCantonese	gong3; hong4
U+4736.0	fMandarin	xiang2
U+4736.0	fEnglish	a small kidney bean common in North China
U+4737.0	fUTF8	䜷
U+4737.0	fCNS11643	4-474E
U+4737.0	fGB	3-6E48
U+4737.0	fR/S	 151.7
U+4737.0	fTotalStrokes	 14
U+4737.0	fCangjie	MTSME
U+4737.0	fFourCorner	17140
U+4737.0	fCantonese	cam1
U+4737.0	fMandarin	qin1
U+4737.0	fEnglish	[1] fermented and seasoned soybeans [2] wild (of beans); undomesticated
U+4738.0	fUTF8	䜸
U+4738.0	fCNS11643	4-474F
U+4738.0	fGB	3-6E49
U+4738.0	fR/S	 151.7
U+4738.0	fTotalStrokes	 14
U+4738.0	fCangjie	MTSHU
U+4738.0	fFourCorner	17110
U+4738.0	fCantonese	mui5
U+4738.0	fMandarin	mei3; mei2
U+4738.0	fEnglish	small pieces of bean-stalks, leaves under the bean-stalks
U+4739.0	fUTF8	䜹
U+4739.0	fCNS11643	3-496C
U+4739.0	fR/S	 151.7
U+4739.0	fTotalStrokes	 14
U+4739.0	fCangjie	MTDL
U+4739.0	fFourCorner	15190
U+4739.0	fCantonese	cuk1
U+4739.0	fMandarin	shu4
U+473A.0	fUTF8	䜺
U+473A.0	fCNS11643	4-4D64
U+473A.0	fGB	3-6E4A
U+473A.0	fR/S	 151.8
U+473A.0	fTotalStrokes	 15
U+473A.0	fCangjie	MTTA
U+473A.0	fFourCorner	14160
U+473A.0	fCantonese	caak3
U+473A.0	fMandarin	ce4
U+473A.0	fEnglish	husking smashed beans for cake, congee, or gruel
U+473B.0	fUTF8	䜻
U+473B.0	fCNS11643	3-5270
U+473B.0	fR/S	 151.9
U+473B.0	fTotalStrokes	 16
U+473B.0	fCangjie	MTAMO
U+473B.0	fFourCorner	16180
U+473B.0	fCantonese	si6
U+473B.0	fMandarin	chi3; shi4
U+473B.0	fEnglish	same as &#x8C49;
U+473C.0	fUTF8	䜼
U+473C.0	fCNS11643	4-5373
U+473C.0	fGB	3-6E4C
U+473C.0	fR/S	 151.9
U+473C.0	fTotalStrokes	 16
U+473C.0	fCangjie	NKMRT
U+473C.0	fFourCorner	18100
U+473C.0	fCantonese	guk1
U+473C.0	fMandarin	mu4; du2; gu2; gu3; mou2
U+473C.0	fEnglish	a kind of bean, bean-stalks
U+473D.0	fUTF8	䜽
U+473D.0	fCNS11643	3-526F
U+473D.0	fR/S	 151.9
U+473D.0	fTotalStrokes	 16
U+473D.0	fCangjie	MTOMN
U+473D.0	fFourCorner	18120
U+473D.0	fCantonese	jyu4; zyu4; zyun4
U+473D.0	fMandarin	yu2
U+473D.0	fEnglish	a kind of bean, beans that changed the colors
U+473E.0	fUTF8	䜾
U+473E.0	fCNS11643	4-5372
U+473E.0	fGB	3-6E4B
U+473E.0	fR/S	 151.9
U+473E.0	fTotalStrokes	 16
U+473E.0	fCangjie	MTYTA
U+473E.0	fFourCorner	10160
U+473E.0	fCantonese	jam1; zaam1
U+473E.0	fMandarin	yin1
U+473E.0	fEnglish	fermented beans; salted beans
U+473F.0	fUTF8	䜿
U+473F.0	fCNS11643	3-5271
U+473F.0	fR/S	 151.9
U+473F.0	fTotalStrokes	 16
U+473F.0	fCangjie	SIMRT
U+473F.0	fFourCorner	78100
U+473F.0	fCantonese	syu6
U+473F.0	fMandarin	shu4
U+4740.0	fUTF8	䝀
U+4740.0	fCNS11643	4-5876
U+4740.0	fGB	3-6E4D
U+4740.0	fR/S	 151.10
U+4740.0	fTotalStrokes	 17; 16
U+4740.0	fCangjie	MTHHW
U+4740.0	fFourCorner	17160
U+4740.0	fCantonese	lau4
U+4740.0	fMandarin	liu2; liao2
U+4740.0	fEnglish	[1] garden peas (Pisum sativum) [2] another name for beans (around Shanxi area) in ancient times
U+4741.0	fUTF8	䝁
U+4741.0	fCNS11643	4-5877
U+4741.0	fGB	5-6D74
U+4741.0	fR/S	 151.10
U+4741.0	fTotalStrokes	 17; 16
U+4741.0	fCangjie	FFBMT
U+4741.0	fFourCorner	99100
U+4741.0	fCantonese	lou4
U+4741.0	fMandarin	lao2
U+4741.0	fEnglish	a kind of wild leguminous plants; wild green lentils
U+4742.0	fUTF8	䝂
U+4742.0	fCNS11643	3-564E
U+4742.0	fR/S	 151.10
U+4742.0	fTotalStrokes	 17
U+4742.0	fCangjie	SEMRT
U+4742.0	fFourCorner	77100
U+4742.0	fCantonese	syu6
U+4742.0	fMandarin	shu4
U+4742.0	fEnglish	same as &#x8C4E;
U+4743.0	fUTF8	䝃
U+4743.0	fCNS11643	4-6744
U+4743.0	fGB	3-6E4F
U+4743.0	fR/S	 151.14
U+4743.0	fTotalStrokes	 21
U+4743.0	fCangjie	MTAVF
U+4743.0	fFourCorner	16130
U+4743.0	fCantonese	zip6
U+4743.0	fMandarin	che4; zhe2
U+4743.0	fEnglish	beans
U+4744.0	fUTF8	䝄
U+4744.0	fCNS11643	4-6D24
U+4744.0	fGB	3-6E45
U+4744.0	fR/S	 151.18
U+4744.0	fTotalStrokes	 25
U+4744.0	fCangjie	MTOGE
U+4744.0	fFourCorner	10140
U+4744.0	fCantonese	soeng1
U+4744.0	fMandarin	shuang1
U+4744.0	fEnglish	cowpea
U+4745.0	fUTF8	䝅
U+4745.0	fCNS11643	4-3025
U+4745.0	fGB	3-6F2C
U+4745.0	fR/S	 152.3
U+4745.0	fTotalStrokes	 10
U+4745.0	fCangjie	MOG
U+4745.0	fFourCorner	14210
U+4745.0	fCantonese	fui1
U+4745.0	fMandarin	hui1
U+4745.0	fEnglish	a pig
U+4746.0	fUTF8	䝆
U+4746.0	fCNS11643	4-354B
U+4746.0	fGB	3-6F2F
U+4746.0	fR/S	 152.4
U+4746.0	fTotalStrokes	 11
U+4746.0	fCangjie	PHMSO
U+4746.0	fFourCorner	27230
U+4746.0	fCantonese	fat1
U+4746.0	fMandarin	hu1
U+4746.0	fEnglish	hog; pig
U+4747.0	fUTF8	䝇
U+4747.0	fCNS11643	6-4522
U+4747.0	fR/S	 152.4
U+4747.0	fTotalStrokes	 11
U+4747.0	fCangjie	CUMSO
U+4747.0	fFourCorner	81210
U+4747.0	fCantonese	fui1; gwai3; kut3
U+4747.0	fMandarin	hui1
U+4747.0	fEnglish	standard form of &#x8C57;
U+4748.0	fUTF8	䝈
U+4748.0	fCNS11643	4-354C
U+4748.0	fGB	3-6F33
U+4748.0	fR/S	 152.5
U+4748.0	fTotalStrokes	 11; 12
U+4748.0	fCangjie	MOHMU
U+4748.0	fFourCorner	11210
U+4748.0	fCantonese	ak1; ngak1 (also); aak1
U+4748.0	fMandarin	e4
U+4748.0	fEnglish	a big and strenuous pig (five feet tall)
U+4749.0	fUTF8	䝉
U+4749.0	fCNS11643	F-4474
U+4749.0	fR/S	 152.6
U+4749.0	fTotalStrokes	 13
U+4749.0	fCangjie	LBMSO
U+4749.0	fCantonese	mung4
U+474A.0	fUTF8	䝊
U+474A.0	fCNS11643	4-4D67
U+474A.0	fGB	3-6F39
U+474A.0	fR/S	 152.8; 152.14
U+474A.0	fTotalStrokes	 15; 21
U+474A.0	fCangjie	MOJLO
U+474A.0	fFourCorner	15281
U+474A.0	fCantonese	saap3
U+474A.0	fMandarin	sha4
U+474A.0	fEnglish	a sow; an aged sow
U+474B.0	fUTF8	䝋
U+474B.0	fCNS11643	4-4D65
U+474B.0	fGB	3-6F3A
U+474B.0	fR/S	 152.8
U+474B.0	fTotalStrokes	 15
U+474B.0	fCangjie	MOJMF
U+474B.0	fFourCorner	13290
U+474B.0	fHakka	  zung5
U+474B.0	fCantonese	zung3; zung1 (also); ceoi4
U+474B.0	fMandarin	zong4
U+474B.0	fEnglish	 young pig; a litter of pigs
U+474C.0	fUTF8	䝌
U+474C.0	fCNS11643	4-4D68
U+474C.0	fGB	3-6F3B
U+474C.0	fR/S	 152.8
U+474C.0	fTotalStrokes	 15
U+474C.0	fCangjie	MOEEE
U+474C.0	fFourCorner	17240
U+474C.0	fCantonese	kyut3
U+474C.0	fMandarin	jue2; jun4
U+474C.0	fEnglish	a pig seek for food through digging on the earth
U+474D.0	fUTF8	䝍
U+474D.0	fCNS11643	4-5375
U+474D.0	fGB	3-6F36
U+474D.0	fR/S	 152.9
U+474D.0	fTotalStrokes	 16
U+474D.0	fCangjie	MOBJJ
U+474D.0	fFourCorner	17250
U+474D.0	fCantonese	gwan3; gwaan1
U+474D.0	fMandarin	jun4
U+474D.0	fEnglish	[1] wild boar [2] hog; pig
U+474E.0	fUTF8	䝎
U+474E.0	fCNS11643	4-5377
U+474E.0	fGB	3-6F3D
U+474E.0	fR/S	 152.9
U+474E.0	fTotalStrokes	 16
U+474E.0	fCangjie	MOUMB
U+474E.0	fFourCorner	12220
U+474E.0	fCantonese	teon1
U+474E.0	fMandarin	tuan1
U+474E.0	fEnglish	a pig; a hog, a kind of fat animal looks like a pig
U+474F.0	fUTF8	䝏
U+474F.0	fCNS11643	4-5D64
U+474F.0	fGB	5-6E4D
U+474F.0	fR/S	 152.11
U+474F.0	fTotalStrokes	 18
U+474F.0	fCangjie	MOLLV
U+474F.0	fFourCorner	15240
U+474F.0	fHakka	  leu2; lu2
U+474F.0	fCantonese	lau4; lau1 (also)
U+474F.0	fMandarin	luu2; lou2; lou3; l2
U+474F.0	fEnglish	sow on heat
U+4750.0	fUTF8	䝐
U+4750.0	fCNS11643	4-6148
U+4750.0	fGB	5-6E50
U+4750.0	fR/S	 152.12
U+4750.0	fTotalStrokes	 19
U+4750.0	fCangjie	MONLB
U+4750.0	fFourCorner	14220
U+4750.0	fCantonese	wai4; coek3; do6; zit3; zuk6
U+4750.0	fMandarin	wei3; wei2
U+4750.0	fEnglish	[1] castrated hog [2] another name for pig
U+4751.0	fUTF8	䝑
U+4751.0	fCNS11643	4-6146
U+4751.0	fGB	3-6F43
U+4751.0	fR/S	 152.12
U+4751.0	fTotalStrokes	 19
U+4751.0	fCangjie	MOYTG
U+4751.0	fFourCorner	10210
U+4751.0	fCantonese	cung1
U+4751.0	fMandarin	chong1
U+4751.0	fEnglish	[1] a local produced pig [2] sucking pig
U+4752.0	fUTF8	䝒
U+4752.0	fCNS11643	4-6745
U+4752.0	fGB	3-6F44
U+4752.0	fR/S	 152.14
U+4752.0	fTotalStrokes	 21
U+4752.0	fCangjie	MOSEO
U+4752.0	fFourCorner	17282
U+4752.0	fCantonese	zo6
U+4752.0	fMandarin	chu2; zhu4
U+4752.0	fEnglish	a sow, a small pig
U+4753.0	fUTF8	䝓
U+4753.0	fCNS11643	3-5F65
U+4753.0	fGB	5-6E53
U+4753.0	fR/S	 152.15
U+4753.0	fTotalStrokes	 22
U+4753.0	fCangjie	MOVVV
U+4753.0	fFourCorner	12210
U+4753.0	fCantonese	lip6
U+4753.0	fMandarin	lie4; la4
U+4753.0	fEnglish	same as &#x9B23;
U+4754.0	fUTF8	䝔
U+4754.0	fCNS11643	3-614C
U+4754.0	fR/S	 152.18
U+4754.0	fTotalStrokes	 25
U+4754.0	fCangjie	MOTRG
U+4754.0	fFourCorner	14210
U+4754.0	fHakka	 fon1
U+4754.0	fCantonese	fun1
U+4754.0	fMandarin	huan1
U+4754.0	fEnglish	the badger
U+4755.0	fUTF8	䝕
U+4755.0	fCNS11643	4-6D25
U+4755.0	fGB	3-6F3F
U+4755.0	fR/S	 152.18
U+4755.0	fTotalStrokes	 25
U+4755.0	fCangjie	MOSJJ
U+4755.0	fFourCorner	11240
U+4755.0	fHakka	 ngiap7
U+4755.0	fCantonese	zip3
U+4755.0	fMandarin	zhe2
U+4755.0	fEnglish	old name for a hog
U+4756.0	fUTF8	䝖
U+4756.0	fCNS11643	4-3550
U+4756.0	fGB	3-722A
U+4756.0	fR/S	 153.4
U+4756.0	fTotalStrokes	 11
U+4756.0	fCangjie	BHHLO
U+4756.0	fFourCorner	22230
U+4756.0	fCantonese	zaau2; coek3; tiu1; zau2; zoek3
U+4756.0	fMandarin	zhao3
U+4756.0	fEnglish	[1] reptiles without feet [2] a fabulous beast
U+4757.0	fUTF8	䝗
U+4757.0	fCNS11643	6-4524
U+4757.0	fR/S	 153.4
U+4757.0	fTotalStrokes	 11
U+4757.0	fCangjie	BHD
U+4757.0	fFourCorner	24290
U+4757.0	fCantonese	jau1; zaau1
U+4757.0	fMandarin	xiu1
U+4757.0	fEnglish	same as &#x8C85;
U+4758.0	fUTF8	䝘
U+4758.0	fCNS11643	3-392F
U+4758.0	fR/S	 153.4
U+4758.0	fTotalStrokes	 11
U+4758.0	fCangjie	BHHNE
U+4758.0	fFourCorner	27240
U+4758.0	fCantonese	ngai6; ai6 (also); ngaai6
U+4758.0	fMandarin	yi4
U+4758.0	fEnglish	same as &#x6BC5;
U+4759.0	fUTF8	䝙
U+4759.0	fGB	S-2427
U+4759.0	fR/S	 153.4
U+4759.0	fTotalStrokes	 11
U+4759.0	fCangjie	BHSK
U+4759.0	fCantonese	ceoi1
U+4759.0	fEnglish	simplified form of &#x8C99;
U+475A.0	fUTF8	䝚
U+475A.0	fCNS11643	4-3B3E
U+475A.0	fGB	3-722C
U+475A.0	fR/S	 153.5
U+475A.0	fTotalStrokes	 12
U+475A.0	fCangjie	BHSP
U+475A.0	fFourCorner	27210
U+475A.0	fCantonese	nei4; lei4 (also)
U+475A.0	fMandarin	ni2
U+475A.0	fEnglish	a kind of animal, fur can be used for clothes
U+475B.0	fUTF8	䝛
U+475B.0	fGB	S-2426
U+475B.0	fR/S	 153.5
U+475B.0	fTotalStrokes	 12
U+475B.0	fCangjie	BHDHE
U+475B.0	fFourCorner	2424
U+475B.0	fCantonese	bo1; fu6
U+475C.0	fUTF8	䝜
U+475C.0	fCNS11643	3-496E
U+475C.0	fR/S	 153.7
U+475C.0	fTotalStrokes	 14
U+475C.0	fCangjie	BHICE
U+475C.0	fFourCorner	23240
U+475C.0	fHakka	  son1
U+475C.0	fCantonese	syun1
U+475C.0	fMandarin	suan1
U+475C.0	fEnglish	same as &#x72FB;
U+475D.0	fUTF8	䝝
U+475D.0	fCNS11643	3-4E79
U+475D.0	fR/S	 153.8
U+475D.0	fTotalStrokes	 15
U+475D.0	fCangjie	BHKMR
U+475D.0	fFourCorner	24220
U+475D.0	fCantonese	ji1; zi1
U+475D.0	fMandarin	yi1; yi3
U+475D.0	fEnglish	same as &#x7317;
U+475E.0	fUTF8	䝞
U+475E.0	fCNS11643	3-4E78
U+475E.0	fR/S	 153.8
U+475E.0	fTotalStrokes	 15
U+475E.0	fCangjie	BHYPU
U+475E.0	fFourCorner	21210
U+475E.0	fCantonese	fu2; hou6
U+475E.0	fMandarin	hao4
U+475E.0	fEnglish	the name of a immortal; with man's face and bird's body; be known as &#x79BA;&#x865F;
U+475F.0	fUTF8	䝟
U+475F.0	fCNS11643	3-5272
U+475F.0	fGB	5-7073
U+475F.0	fR/S	 153.9
U+475F.0	fTotalStrokes	 16
U+475F.0	fCangjie	BHQHK
U+475F.0	fFourCorner	27230
U+475F.0	fCantonese	at3; bit1; faat6; maat6
U+475F.0	fMandarin	ya4; ye4
U+475F.0	fEnglish	a fierce man-eating animal in legend
U+4760.0	fUTF8	䝠
U+4760.0	fCNS11643	3-5650
U+4760.0	fGB	5-7076
U+4760.0	fR/S	 153.10
U+4760.0	fTotalStrokes	 17
U+4760.0	fCangjie	BHMHF
U+4760.0	fFourCorner	21290
U+4760.0	fHakka	 ngien2
U+4760.0	fCantonese	zyun4
U+4760.0	fMandarin	yuan2; huan2
U+4760.0	fEnglish	same as &#x7342;
U+4761.0	fUTF8	䝡
U+4761.0	fCNS11643	3-592C
U+4761.0	fR/S	 153.11
U+4761.0	fTotalStrokes	 18
U+4761.0	fCangjie	BHTLB
U+4761.0	fFourCorner	24220
U+4761.0	fCantonese	maan6
U+4761.0	fMandarin	man2; man4
U+4761.0	fEnglish	same as &#x734C;
U+4762.0	fUTF8	䝢
U+4762.0	fCNS11643	3-592D
U+4762.0	fR/S	 153.11
U+4762.0	fTotalStrokes	 18
U+4762.0	fCangjie	BHAWE
U+4762.0	fFourCorner	26240
U+4762.0	fCantonese	maan6
U+4762.0	fMandarin	man4; meng2
U+4762.0	fEnglish	[1] same as &#x734C; [2] another name for &#x8C99;
U+4763.0	fUTF8	䝣
U+4763.0	fCNS11643	4-614A
U+4763.0	fGB	3-7232
U+4763.0	fR/S	 153.12
U+4763.0	fTotalStrokes	 19
U+4763.0	fCangjie	BHESD
U+4763.0	fFourCorner	21290
U+4763.0	fCantonese	keoi4
U+4763.0	fMandarin	qu2
U+4763.0	fEnglish	a kind of animal
U+4764.0	fUTF8	䝤
U+4764.0	fCNS11643	7-4A41
U+4764.0	fR/S	 153.12
U+4764.0	fTotalStrokes	 19
U+4764.0	fCangjie	BHKCF
U+4764.0	fFourCorner	24290
U+4764.0	fHakka	 liau2; lau3; liau5
U+4764.0	fCantonese	lou5
U+4764.0	fMandarin	lao3
U+4764.0	fEnglish	 hunt at night with torches
U+4765.0	fUTF8	䝥
U+4765.0	fCNS11643	3-5B37
U+4765.0	fR/S	 153.12
U+4765.0	fTotalStrokes	 19
U+4765.0	fCangjie	BHHUJ
U+4765.0	fFourCorner	26240
U+4765.0	fCantonese	hou4
U+4765.0	fMandarin	hao2
U+4765.0	fEnglish	roar or howl (of beasts) frantic barks of dogs or wolves
U+4766.0	fUTF8	䝦
U+4766.0	fCNS11643	4-5D68
U+4766.0	fGB	3-7233
U+4766.0	fR/S	 153.12; 153.11
U+4766.0	fTotalStrokes	 18; 19
U+4766.0	fCangjie	BHWLO
U+4766.0	fFourCorner	26280
U+4766.0	fCantonese	zung1
U+4766.0	fMandarin	zhong1
U+4766.0	fEnglish	a kind of animal looks like leopard; with horns
U+4767.0	fUTF8	䝧
U+4767.0	fCNS11643	4-3551
U+4767.0	fGB	3-4841
U+4767.0	fR/S	 154.4
U+4767.0	fTotalStrokes	 11
U+4767.0	fCangjie	BCPMM
U+4767.0	fFourCorner	67820
U+4767.0	fCantonese	man4
U+4767.0	fMandarin	min2; men2
U+4767.0	fEnglish	capital (in business), to compute taxes
U+4768.0	fUTF8	䝨
U+4768.0	fCNS11643	4-3554
U+4768.0	fR/S	 154.4
U+4768.0	fTotalStrokes	 11
U+4768.0	fCangjie	LEBUC
U+4768.0	fFourCorner	27800
U+4768.0	fCantonese	jin4; zin4
U+4768.0	fMandarin	xian2
U+4768.0	fEnglish	non-classical form of &#x8CE2;
U+4769.0	fUTF8	䝩
U+4769.0	fCNS11643	5-3C4C
U+4769.0	fR/S	 154.5
U+4769.0	fTotalStrokes	 12
U+4769.0	fCangjie	BCOHH
U+4769.0	fFourCorner	68820
U+4769.0	fCantonese	zan3
U+4769.0	fMandarin	zhen3; zhen4
U+4769.0	fEnglish	same as &#x8CD1;
U+476A.0	fUTF8	䝪
U+476A.0	fCNS11643	4-3B3F
U+476A.0	fGB	3-4847
U+476A.0	fR/S	 154.5
U+476A.0	fTotalStrokes	 12
U+476A.0	fCangjie	BCNYO
U+476A.0	fFourCorner	67880
U+476A.0	fCantonese	so2
U+476A.0	fMandarin	shu3; shu2; fu4
U+476A.0	fEnglish	to provide money or gift and to consult fortune-teller; pay for guidance from divination
U+476B.0	fUTF8	䝫
U+476B.0	fCNS11643	4-3B41
U+476B.0	fGB	3-4845
U+476B.0	fR/S	 154.5
U+476B.0	fTotalStrokes	 12
U+476B.0	fCangjie	BCOS
U+476B.0	fFourCorner	68810
U+476B.0	fCantonese	zok6; zok3
U+476B.0	fMandarin	zuo2
U+476B.0	fEnglish	money; currency and finances
U+476C.0	fUTF8	䝬
U+476C.0	fCNS11643	3-3F2E
U+476C.0	fR/S	 154.5
U+476C.0	fTotalStrokes	 12
U+476C.0	fCangjie	BCYG
U+476C.0	fFourCorner	60810
U+476C.0	fCantonese	zyu3
U+476C.0	fMandarin	zhu4
U+476C.0	fEnglish	a wealthy man
U+476D.0	fUTF8	䝭
U+476D.0	fCNS11643	4-3B42
U+476D.0	fGB	3-4846
U+476D.0	fR/S	 154.5
U+476D.0	fTotalStrokes	 12
U+476D.0	fCangjie	BCPR
U+476D.0	fFourCorner	67820
U+476D.0	fCantonese	gau3
U+476D.0	fMandarin	gou4
U+476D.0	fEnglish	[1] wages; salary [2] redeem; ransom
U+476E.0	fUTF8	䝮
U+476E.0	fCNS11643	6-4F29
U+476E.0	fR/S	 154.5
U+476E.0	fTotalStrokes	 12
U+476E.0	fCangjie	BCYVI
U+476E.0	fFourCorner	60830
U+476E.0	fCantonese	jyun6; seon3; zyun6
U+476E.0	fMandarin	xian4; xuan4
U+476E.0	fEnglish	same as &#x8852;
U+476F.0	fUTF8	䝯
U+476F.0	fCNS11643	4-3B43
U+476F.0	fGB	5-4A44
U+476F.0	fR/S	 154.5
U+476F.0	fTotalStrokes	 12
U+476F.0	fCangjie	BCOPD
U+476F.0	fFourCorner	68810
U+476F.0	fCantonese	zi6
U+476F.0	fMandarin	yi2; yi4
U+476F.0	fEnglish	same as &#x8CA4;
U+4770.0	fUTF8	䝰
U+4770.0	fCNS11643	4-4121
U+4770.0	fR/S	 154.6
U+4770.0	fTotalStrokes	 13
U+4770.0	fCangjie	BCGDI
U+4770.0	fFourCorner	64840
U+4770.0	fCantonese	zi6
U+4770.0	fMandarin	zhi4; ti2
U+4770.0	fEnglish	to store or to accumulate riches or valuables
U+4771.0	fUTF8	䝱
U+4771.0	fCNS11643	3-4466
U+4771.0	fR/S	 154.6
U+4771.0	fTotalStrokes	 13
U+4771.0	fCangjie	KSKSC
U+4771.0	fFourCorner	40800
U+4771.0	fCantonese	hip3
U+4771.0	fMandarin	xie2
U+4771.0	fEnglish	to trouble; to worry; to vex; to annoy
U+4772.0	fUTF8	䝲
U+4772.0	fCNS11643	3-4468
U+4772.0	fR/S	 154.6
U+4772.0	fTotalStrokes	 13
U+4772.0	fCangjie	BCSOY
U+4772.0	fFourCorner	67833
U+4772.0	fCantonese	zeon2
U+4772.0	fMandarin	jin4
U+4772.0	fEnglish	abbreviated form of &#x8D10;
U+4773.0	fUTF8	䝳
U+4773.0	fCNS11643	5-4A29
U+4773.0	fR/S	 154.7
U+4773.0	fTotalStrokes	 14
U+4773.0	fCangjie	YEBUC
U+4773.0	fFourCorner	27800
U+4773.0	fCantonese	caan4
U+4773.0	fMandarin	can2
U+4773.0	fEnglish	to kill for money; to commit murder for money, deeply meaningful; deep and profound; deep and thick
U+4774.0	fUTF8	䝴
U+4774.0	fCNS11643	6-626A
U+4774.0	fR/S	 154.7
U+4774.0	fTotalStrokes	 14
U+4774.0	fCangjie	JBBUC
U+4774.0	fFourCorner	50800
U+4774.0	fHakka	 zi1; zu1
U+4774.0	fCantonese	zai1
U+4774.0	fMandarin	ji1
U+4774.0	fEnglish	non-classical form of &#x9F4E;
U+4775.0	fUTF8	䝵
U+4775.0	fCNS11643	3-496F
U+4775.0	fR/S	 154.7
U+4775.0	fTotalStrokes	 14
U+4775.0	fCangjie	BCIJB
U+4775.0	fFourCorner	63820
U+4775.0	fHakka	 pu5; pu6
U+4775.0	fCantonese	bou6
U+4775.0	fMandarin	bu4
U+4775.0	fEnglish	 give a fee or present
U+4776.0	fUTF8	䝶
U+4776.0	fCNS11643	3-4E7B
U+4776.0	fR/S	 154.8
U+4776.0	fTotalStrokes	 15
U+4776.0	fCangjie	BCYRF
U+4776.0	fFourCorner	60890
U+4776.0	fCantonese	loeng4
U+4776.0	fMandarin	liang2; liang4
U+4776.0	fEnglish	levy taxes
U+4777.0	fUTF8	䝷
U+4777.0	fCNS11643	3-4E7C
U+4777.0	fR/S	 154.8
U+4777.0	fTotalStrokes	 15
U+4777.0	fCangjie	ORBUC
U+4777.0	fFourCorner	86800
U+4777.0	fCantonese	zi1; zi3
U+4777.0	fMandarin	zhi1; zhi4
U+4777.0	fEnglish	[1] securities; assign something as security under an arrangement; mortgage [2] relieve or give aid to the distressed; support [3] same as &#x4783;
U+4778.0	fUTF8	䝸
U+4778.0	fCNS11643	4-4D6E
U+4778.0	fGB	3-4849
U+4778.0	fR/S	 154.8
U+4778.0	fTotalStrokes	 15
U+4778.0	fCangjie	BCTJE
U+4778.0	fFourCorner	64840
U+4778.0	fCantonese	gei6
U+4778.0	fMandarin	ji4
U+4778.0	fEnglish	a kind of tool; instrument; apparatus; implement
U+4779.0	fUTF8	䝹
U+4779.0	fCNS11643	4-4D6C
U+4779.0	fGB	3-484F
U+4779.0	fR/S	 154.8
U+4779.0	fTotalStrokes	 15
U+4779.0	fCangjie	BCJNU
U+4779.0	fFourCorner	63810
U+4779.0	fCantonese	wun2; zyun1; zyun2
U+4779.0	fMandarin	wan3; yuan4
U+4779.0	fEnglish	[1] fairly prosperous [2] well-to-do; not very rich
U+477A.0	fUTF8	䝺
U+477A.0	fCNS11643	7-2936
U+477A.0	fR/S	 154.8
U+477A.0	fTotalStrokes	 15
U+477A.0	fCangjie	YKBUC
U+477A.0	fFourCorner	00800
U+477A.0	fCantonese	gun3; gwun3 (also)
U+477A.0	fMandarin	guan4
U+477A.0	fEnglish	non-classical form of &#x8CAB;
U+477B.0	fUTF8	䝻
U+477B.0	fCNS11643	4-4D6D
U+477B.0	fGB	3-4850
U+477B.0	fR/S	 154.8
U+477B.0	fTotalStrokes	 15
U+477B.0	fCangjie	BCSJR
U+477B.0	fFourCorner	67860
U+477B.0	fHakka	  gi1
U+477B.0	fCantonese	geoi1; ceoi3
U+477B.0	fMandarin	ju1
U+477B.0	fEnglish	 store property
U+477C.0	fUTF8	䝼
U+477C.0	fGB	3-484B
U+477C.0	fR/S	 154.8
U+477C.0	fTotalStrokes	 15
U+477C.0	fCangjie	BCQMB
U+477C.0	fFourCorner	6582
U+477C.0	fHakka	 ciang5; ciang6; cin5
U+477C.0	fCantonese	cing4; zing6; cing2 (also); tiu4
U+477C.0	fMandarin	qing2
U+477C.0	fEnglish	 receive; come into possession of
U+477D.0	fUTF8	䝽
U+477D.0	fCNS11643	5-5139
U+477D.0	fR/S	 154.8
U+477D.0	fTotalStrokes	 15
U+477D.0	fCangjie	BCMGG
U+477D.0	fFourCorner	61810
U+477D.0	fCantonese	ngaai6; aai6 (also)
U+477D.0	fMandarin	wai4; ai4
U+477D.0	fEnglish	name of a person in old China
U+477E.0	fUTF8	䝾
U+477E.0	fCNS11643	3-4F21
U+477E.0	fR/S	 154.8
U+477E.0	fTotalStrokes	 15
U+477E.0	fCangjie	MMBUC
U+477E.0	fFourCorner	13800
U+477E.0	fCantonese	fu3; fu2
U+477E.0	fMandarin	fu4
U+477E.0	fEnglish	[1] bestow; give [2] natural endowment or gifts [3] tax; revenue [4] spread; diffuse [5] compose or sing [6] one of the Chinese literary forms akin to poetry
U+477F.0	fUTF8	䝿
U+477F.0	fCNS11643	3-5274
U+477F.0	fGB	5-4A4E
U+477F.0	fR/S	 154.8
U+477F.0	fTotalStrokes	 16; 15
U+477F.0	fCangjie	HOBUC
U+477F.0	fFourCorner	77800
U+477F.0	fCantonese	gwai3
U+477F.0	fMandarin	gui4
U+477F.0	fEnglish	ancient form of &#x8CB4;
U+4780.0	fUTF8	䞀
U+4780.0	fCNS11643	4-537D
U+4780.0	fGB	3-4855
U+4780.0	fR/S	 154.9
U+4780.0	fTotalStrokes	 16
U+4780.0	fCangjie	BCONK
U+4780.0	fFourCorner	67830
U+4780.0	fCantonese	hau6
U+4780.0	fMandarin	hou4; gou4; min3
U+4780.0	fEnglish	 desire for more money than one's rightful share
U+4781.0	fUTF8	䞁
U+4781.0	fCNS11643	4-5379
U+4781.0	fGB	3-4851
U+4781.0	fR/S	 154.9
U+4781.0	fTotalStrokes	 16
U+4781.0	fCangjie	BCSAV
U+4781.0	fFourCorner	61810
U+4781.0	fCantonese	jin3; an3; zan2; zing4
U+4781.0	fMandarin	yan3; yan4; xian4; yang3; yang4
U+4781.0	fEnglish	[1] corresponding; equivalent [2] appropriate [3] compare the length of two articles
U+4782.0	fUTF8	䞂
U+4782.0	fCNS11643	4-537B
U+4782.0	fGB	3-4852
U+4782.0	fR/S	 154.9
U+4782.0	fTotalStrokes	 16
U+4782.0	fCangjie	BCMBK
U+4782.0	fFourCorner	61830
U+4782.0	fCantonese	jyun5; nyun6; zyun5
U+4782.0	fMandarin	ruan3; nuan3
U+4782.0	fEnglish	[1] fairly prosperous [2] well-to-do; not very rich
U+4783.0	fUTF8	䞃
U+4783.0	fCNS11643	4-537A
U+4783.0	fGB	3-4856
U+4783.0	fR/S	 154.9
U+4783.0	fTotalStrokes	 16
U+4783.0	fCangjie	BCMGK
U+4783.0	fFourCorner	68840
U+4783.0	fCantonese	zi3
U+4783.0	fMandarin	zhi4
U+4783.0	fEnglish	[1] relieve or give aid to the distressed; support [2] shells; precious; valuable
U+4784.0	fUTF8	䞄
U+4784.0	fCNS11643	3-5276
U+4784.0	fR/S	 154.9
U+4784.0	fTotalStrokes	 16
U+4784.0	fCangjie	YDBUC
U+4784.0	fFourCorner	08800
U+4784.0	fCantonese	biu3; biu2
U+4784.0	fMandarin	biao3; biao4
U+4784.0	fEnglish	interchangeable with &#x4FF5;
U+4785.0	fUTF8	䞅
U+4785.0	fCNS11643	7-3C33
U+4785.0	fR/S	 154.10
U+4785.0	fTotalStrokes	 17
U+4785.0	fCangjie	BCMVR
U+4785.0	fFourCorner	61860
U+4785.0	fCantonese	ji4; zi4
U+4785.0	fMandarin	yi2
U+4785.0	fEnglish	non-classical form of &#x8CBD;
U+4786.0	fUTF8	䞆
U+4786.0	fCNS11643	4-587A
U+4786.0	fGB	3-484A
U+4786.0	fR/S	 154.10
U+4786.0	fTotalStrokes	 17; 16
U+4786.0	fCangjie	BCFBC
U+4786.0	fFourCorner	69880
U+4786.0	fCantonese	so3; so2
U+4786.0	fMandarin	suo3
U+4786.0	fEnglish	[1] bone [2] grease and oil
U+4787.0	fUTF8	䞇
U+4787.0	fCNS11643	F-607A
U+4787.0	fR/S	 154.11
U+4787.0	fTotalStrokes	 18
U+4787.0	fCangjie	GIBUC
U+4787.0	fCantonese	zi3
U+4787.0	fMandarin	die2; zhi4
U+4787.0	fEnglish	gifts to superiors or friends; offerings of ceremony
U+4788.0	fUTF8	䞈
U+4788.0	fCNS11643	4-537E
U+4788.0	fGB	3-4842
U+4788.0	fR/S	 154.12; 154.9
U+4788.0	fTotalStrokes	 16; 19
U+4788.0	fCangjie	BCIKF
U+4788.0	fFourCorner	64820
U+4788.0	fCantonese	gwai3
U+4788.0	fMandarin	gui4; gui3
U+4788.0	fEnglish	ancient form of &#x8CA8;
U+4789.0	fUTF8	䞉
U+4789.0	fCNS11643	3-5D35
U+4789.0	fR/S	 154.13
U+4789.0	fTotalStrokes	 20
U+4789.0	fCangjie	BCFQC
U+4789.0	fFourCorner	69880
U+4789.0	fCantonese	sing6
U+4789.0	fMandarin	ying4; sheng4
U+4789.0	fEnglish	same as &#x8CF8;
U+478A.0	fUTF8	䞊
U+478A.0	fCNS11643	3-5E64
U+478A.0	fR/S	 154.15; 154.14
U+478A.0	fTotalStrokes	 22; 21
U+478A.0	fCangjie	BCYPU
U+478A.0	fFourCorner	61860
U+478A.0	fCantonese	seon3
U+478A.0	fMandarin	xun4
U+478A.0	fEnglish	to increase; to add to; to augment, in a higher degree; more, benefit; profit; advantage
U+478B.0	fUTF8	䞋
U+478B.0	fCNS11643	4-6B21
U+478B.0	fGB	5-4A52
U+478B.0	fR/S	 154.16
U+478B.0	fTotalStrokes	 23
U+478B.0	fCangjie	BCYDU
U+478B.0	fFourCorner	66810
U+478B.0	fHakka	 cen5;  cang5
U+478B.0	fCantonese	can3
U+478B.0	fMandarin	chen4
U+478B.0	fEnglish	 give money especially for Buddhist services
U+478C.0	fUTF8	䞌
U+478C.0	fGB	S-2323
U+478C.0	fR/S	 154.6
U+478C.0	fTotalStrokes	 13
U+478C.0	fCangjie	BOHJR
U+478C.0	fCantonese	sit6; sit3
U+478C.0	fMandarin	she2
U+478C.0	fEnglish	to lose in trade; to lose in weight or measure
U+478D.0	fUTF8	䞍
U+478D.0	fGB	7-223D
U+478D.0	fR/S	 154.8
U+478D.0	fTotalStrokes	 15
U+478D.0	fCangjie	BOQMB
U+478D.0	fCantonese	cing4; zing6; cing2 (also)
U+478D.0	fMandarin	qing2
U+478D.0	fEnglish	simplified form of &#x477C;
U+478E.0	fUTF8	䞎
U+478E.0	fGB	S-2324
U+478E.0	fR/S	 154.8
U+478E.0	fTotalStrokes	 15
U+478E.0	fCangjie	BOAA
U+478E.0	fCantonese	coeng1
U+478F.0	fUTF8	䞏
U+478F.0	fGB	S-2322
U+478F.0	fR/S	 154.9
U+478F.0	fTotalStrokes	 16
U+478F.0	fCangjie	BOWP
U+478F.0	fCantonese	si1
U+4790.0	fUTF8	䞐
U+4790.0	fGB	S-2325
U+4790.0	fR/S	 154.9
U+4790.0	fTotalStrokes	 16
U+4790.0	fCangjie	BOQKA
U+4790.0	fCantonese	seon2
U+4790.0	fMandarin	chun3; shun3
U+4790.0	fEnglish	simplified form of &#x8CF0;
U+4791.0	fUTF8	䞑
U+4791.0	fCNS11643	5-304B
U+4791.0	fR/S	 155.3
U+4791.0	fTotalStrokes	 10
U+4791.0	fCangjie	GCM
U+4791.0	fFourCorner	41310
U+4791.0	fCantonese	hung4
U+4791.0	fMandarin	hong2
U+4791.0	fEnglish	a red swelling of the skin
U+4792.0	fUTF8	䞒
U+4792.0	fCNS11643	5-4334
U+4792.0	fR/S	 155.6
U+4792.0	fTotalStrokes	 13
U+4792.0	fCangjie	GCBMR
U+4792.0	fFourCorner	47320
U+4792.0	fCantonese	dung6
U+4792.0	fMandarin	dong4
U+4792.0	fEnglish	red color
U+4793.0	fUTF8	䞓
U+4793.0	fCNS11643	4-4759
U+4793.0	fGB	5-6D6F
U+4793.0	fR/S	 155.7
U+4793.0	fTotalStrokes	 14
U+4793.0	fCangjie	GCMVM
U+4793.0	fFourCorner	41311
U+4793.0	fHakka	 shin5; shin3; siin5; siin3
U+4793.0	fCantonese	cing1
U+4793.0	fMandarin	cheng1
U+4793.0	fEnglish	[1] a deep color [2] dye red [3] blush
U+4794.0	fUTF8	䞔
U+4794.0	fCNS11643	4-4758
U+4794.0	fGB	3-6E3D
U+4794.0	fR/S	 155.7
U+4794.0	fTotalStrokes	 14
U+4794.0	fCangjie	GCSHU
U+4794.0	fFourCorner	47310
U+4794.0	fCantonese	mei5
U+4794.0	fMandarin	wei3
U+4794.0	fEnglish	deep red
U+4795.0	fUTF8	䞕
U+4795.0	fCNS11643	4-6748
U+4795.0	fGB	3-6E3F
U+4795.0	fR/S	 155.14
U+4795.0	fTotalStrokes	 21
U+4795.0	fCangjie	GCMBB
U+4795.0	fFourCorner	41320
U+4795.0	fHakka	 ji2; ri2
U+4795.0	fCantonese	jyu4; zyu4
U+4795.0	fMandarin	ru2; yu2; die2; na3; nie4; xie4
U+4795.0	fEnglish	flames of fire
U+4796.0	fUTF8	䞖
U+4796.0	fCNS11643	5-304C
U+4796.0	fR/S	 156.3
U+4796.0	fTotalStrokes	 10
U+4796.0	fCangjie	GOIP
U+4796.0	fFourCorner	43800
U+4796.0	fCantonese	suk6
U+4796.0	fMandarin	shu3
U+4796.0	fEnglish	the name of a person
U+4797.0	fUTF8	䞗
U+4797.0	fCNS11643	4-3027
U+4797.0	fGB	3-6D6F
U+4797.0	fR/S	 156.3
U+4797.0	fTotalStrokes	 10
U+4797.0	fCangjie	GODH
U+4797.0	fFourCorner	44800
U+4797.0	fCantonese	caai1
U+4797.0	fMandarin	cai1; chai1
U+4797.0	fEnglish	[1] suspicion; doubt [2] go up [3] start [4] occur
U+4798.0	fUTF8	䞘
U+4798.0	fCNS11643	4-3028
U+4798.0	fGB	3-6D71
U+4798.0	fR/S	 156.3
U+4798.0	fTotalStrokes	 10
U+4798.0	fCangjie	GOON
U+4798.0	fFourCorner	48800
U+4798.0	fCantonese	gat6; gat1
U+4798.0	fMandarin	qi4; ji2
U+4798.0	fEnglish	[1] go straight forward [2] raise up
U+4799.0	fUTF8	䞙
U+4799.0	fCNS11643	4-3555
U+4799.0	fGB	3-6D72
U+4799.0	fR/S	 156.4
U+4799.0	fTotalStrokes	 11
U+4799.0	fCangjie	GOMLB
U+4799.0	fFourCorner	41800
U+4799.0	fCantonese	zaap3
U+4799.0	fMandarin	za2
U+4799.0	fEnglish	walk in hurriedly
U+479A.0	fUTF8	䞚
U+479A.0	fCNS11643	4-3556
U+479A.0	fGB	3-6D73
U+479A.0	fR/S	 156.4
U+479A.0	fTotalStrokes	 11
U+479A.0	fCangjie	GOJE
U+479A.0	fFourCorner	44800
U+479A.0	fHakka	 kia1
U+479A.0	fCantonese	kei4
U+479A.0	fMandarin	qi2
U+479A.0	fEnglish	to hang on a big tree, to walk ( the way of Alces machlis, a kind of deer)
U+479B.0	fUTF8	䞛
U+479B.0	fCNS11643	3-3934
U+479B.0	fR/S	 156.4
U+479B.0	fTotalStrokes	 11
U+479B.0	fCangjie	GOMLS
U+479B.0	fFourCorner	41800
U+479B.0	fCantonese	jin1
U+479B.0	fEnglish	dancing of a eastern minority group in old China
U+479C.0	fUTF8	䞜
U+479C.0	fCNS11643	3-3936
U+479C.0	fR/S	 156.4
U+479C.0	fTotalStrokes	 11
U+479C.0	fCangjie	GOMF
U+479C.0	fFourCorner	41800
U+479C.0	fCantonese	fu3; fu6
U+479C.0	fMandarin	bu4; fu4
U+479C.0	fEnglish	[1] reach; arrive [2] lie flat [3] stiff
U+479D.0	fUTF8	䞝
U+479D.0	fCNS11643	3-3F31
U+479D.0	fR/S	 156.5
U+479D.0	fTotalStrokes	 12
U+479D.0	fCangjie	GOMGI
U+479D.0	fFourCorner	41800
U+479D.0	fCantonese	juk6; zuk6
U+479D.0	fMandarin	yu4; ao3
U+479D.0	fEnglish	lame; crippled
U+479E.0	fUTF8	䞞
U+479E.0	fCNS11643	4-3B45
U+479E.0	fGB	5-6D3F
U+479E.0	fR/S	 156.5
U+479E.0	fTotalStrokes	 12
U+479E.0	fCangjie	GOLLN
U+479E.0	fFourCorner	45800
U+479E.0	fCantonese	fat6; faat1; maan5
U+479E.0	fMandarin	fu2; fei4
U+479E.0	fEnglish	[1] walk [2] jump; leap; bounce; spring
U+479F.0	fUTF8	䞟
U+479F.0	fCNS11643	3-3F36
U+479F.0	fGB	5-6D3D
U+479F.0	fR/S	 156.5
U+479F.0	fTotalStrokes	 12
U+479F.0	fCangjie	GOHA
U+479F.0	fFourCorner	46800
U+479F.0	fCantonese	paak3; bak1
U+479F.0	fMandarin	po4; pai1
U+479F.0	fEnglish	[1] press; compel [2] draw near [3] excel; surpass; rise above; transcend [4] fly across; jump over
U+47A0.0	fUTF8	䞠
U+47A0.0	fCNS11643	3-3F32
U+47A0.0	fR/S	 156.5
U+47A0.0	fTotalStrokes	 12
U+47A0.0	fCangjie	GOMR
U+47A0.0	fFourCorner	41800
U+47A0.0	fCantonese	zek3
U+47A0.0	fMandarin	zhi1
U+47A0.0	fEnglish	walk
U+47A1.0	fUTF8	䞡
U+47A1.0	fCNS11643	5-3C52
U+47A1.0	fR/S	 156.5
U+47A1.0	fTotalStrokes	 12
U+47A1.0	fCangjie	GOAM
U+47A1.0	fFourCorner	46800
U+47A1.0	fCantonese	taan2; tan2
U+47A1.0	fMandarin	tan3
U+47A1.0	fEnglish	 walk
U+47A2.0	fUTF8	䞢
U+47A2.0	fCNS11643	3-3F35
U+47A2.0	fR/S	 156.5
U+47A2.0	fTotalStrokes	 12
U+47A2.0	fCangjie	GOOS
U+47A2.0	fFourCorner	48800
U+47A2.0	fCantonese	zok3; zaa3; zok6
U+47A2.0	fMandarin	zuo4; zha4; zuo2
U+47A2.0	fEnglish	 walk
U+47A3.0	fUTF8	䞣
U+47A3.0	fCNS11643	4-3B48
U+47A3.0	fGB	3-6D77
U+47A3.0	fR/S	 156.5
U+47A3.0	fTotalStrokes	 12
U+47A3.0	fCangjie	GOHMY
U+47A3.0	fFourCorner	42800
U+47A3.0	fHakka	 cha3; ca3; cha5
U+47A3.0	fCantonese	ce2; cak3
U+47A3.0	fMandarin	che3
U+47A3.0	fEnglish	 stand crooked
U+47A4.0	fUTF8	䞤
U+47A4.0	fCNS11643	4-3B47
U+47A4.0	fGB	3-6D79
U+47A4.0	fR/S	 156.5
U+47A4.0	fTotalStrokes	 12
U+47A4.0	fCangjie	GOPR
U+47A4.0	fFourCorner	47800
U+47A4.0	fCantonese	keoi4
U+47A4.0	fMandarin	qu2; fu3; qu3
U+47A4.0	fEnglish	to walk, to use; to employ, to control
U+47A5.0	fUTF8	䞥
U+47A5.0	fCNS11643	4-4126
U+47A5.0	fGB	3-6D7B
U+47A5.0	fR/S	 156.6
U+47A5.0	fTotalStrokes	 13
U+47A5.0	fCangjie	GOKB
U+47A5.0	fFourCorner	44800
U+47A5.0	fCantonese	jau6
U+47A5.0	fMandarin	you4
U+47A5.0	fEnglish	walk
U+47A6.0	fUTF8	䞦
U+47A6.0	fCNS11643	4-4127
U+47A6.0	fGB	3-6E22
U+47A6.0	fR/S	 156.6
U+47A6.0	fTotalStrokes	 13
U+47A6.0	fCangjie	GOHER
U+47A6.0	fFourCorner	47800
U+47A6.0	fCantonese	haak6
U+47A6.0	fMandarin	he2; jie2
U+47A6.0	fEnglish	to lie flat, to be inactive; stiff; rigid, dead-locked
U+47A7.0	fUTF8	䞧
U+47A7.0	fCNS11643	5-433C
U+47A7.0	fR/S	 156.6
U+47A7.0	fTotalStrokes	 13
U+47A7.0	fCangjie	GOHMR
U+47A7.0	fFourCorner	42800
U+47A7.0	fCantonese	hau6
U+47A7.0	fMandarin	hou4
U+47A7.0	fEnglish	[1] hard to walk [2] lame; crippled
U+47A8.0	fUTF8	䞨
U+47A8.0	fCNS11643	3-446A
U+47A8.0	fGB	5-6D40
U+47A8.0	fR/S	 156.6
U+47A8.0	fTotalStrokes	 13
U+47A8.0	fCangjie	GOGG
U+47A8.0	fFourCorner	44800
U+47A8.0	fCantonese	kwai2
U+47A8.0	fMandarin	kui3; gui3
U+47A8.0	fEnglish	same as &#x8DEC;
U+47A9.0	fUTF8	䞩
U+47A9.0	fCNS11643	4-4128
U+47A9.0	fGB	5-6D45
U+47A9.0	fR/S	 156.6
U+47A9.0	fTotalStrokes	 13
U+47A9.0	fCangjie	GOOMR
U+47A9.0	fFourCorner	48800
U+47A9.0	fCantonese	ap1; ngap1 (also)
U+47A9.0	fMandarin	xia2; e4; rui2
U+47A9.0	fEnglish	lame; crippled
U+47AA.0	fUTF8	䞪
U+47AA.0	fCNS11643	5-4338
U+47AA.0	fR/S	 156.6
U+47AA.0	fTotalStrokes	 13
U+47AA.0	fCangjie	GOSHL
U+47AA.0	fFourCorner	41800
U+47AA.0	fCantonese	zoeng6
U+47AA.0	fMandarin	jiang4; jiang3
U+47AA.0	fEnglish	walking rapidly
U+47AB.0	fUTF8	䞫
U+47AB.0	fCNS11643	4-475A
U+47AB.0	fGB	5-6D4D
U+47AB.0	fR/S	 156.7
U+47AB.0	fTotalStrokes	 14
U+47AB.0	fCangjie	GOSKR
U+47AB.0	fFourCorner	47800
U+47AB.0	fCantonese	wan5
U+47AB.0	fMandarin	yun3
U+47AB.0	fEnglish	walk
U+47AC.0	fUTF8	䞬
U+47AC.0	fCNS11643	6-6275
U+47AC.0	fR/S	 156.7
U+47AC.0	fTotalStrokes	 14
U+47AC.0	fCangjie	GOHDS
U+47AC.0	fFourCorner	42800
U+47AC.0	fCantonese	tau3; taau3; zyu4; zyu5
U+47AC.0	fMandarin	tou4
U+47AC.0	fEnglish	same as &#x900F;
U+47AD.0	fUTF8	䞭
U+47AD.0	fCNS11643	4-475B
U+47AD.0	fGB	5-6D4E
U+47AD.0	fR/S	 156.7
U+47AD.0	fTotalStrokes	 14
U+47AD.0	fCangjie	GOICE
U+47AD.0	fFourCorner	43800
U+47AD.0	fCantonese	ceon1
U+47AD.0	fMandarin	qun1; qiu3; qiu4
U+47AD.0	fEnglish	walking rapidly, to go ahead, to move quickly; to run; to hurry
U+47AE.0	fUTF8	䞮
U+47AE.0	fCNS11643	3-4976
U+47AE.0	fR/S	 156.7
U+47AE.0	fTotalStrokes	 14
U+47AE.0	fCangjie	GOOMD
U+47AE.0	fFourCorner	48800
U+47AE.0	fHakka	 tu2
U+47AE.0	fCantonese	tou1
U+47AE.0	fMandarin	tu1; tu2
U+47AE.0	fEnglish	to crawl; to creep
U+47AF.0	fUTF8	䞯
U+47AF.0	fCNS11643	5-4A34
U+47AF.0	fR/S	 156.7
U+47AF.0	fTotalStrokes	 14
U+47AF.0	fCangjie	GOBND
U+47AF.0	fFourCorner	42800
U+47AF.0	fCantonese	fu3
U+47AF.0	fMandarin	fu4; fu2
U+47AF.0	fEnglish	to walk, to move quickly; to run; to hurry, to reach the time-limit; to mature
U+47B0.0	fUTF8	䞰
U+47B0.0	fCNS11643	3-4977
U+47B0.0	fGB	5-6D4A
U+47B0.0	fR/S	 156.7
U+47B0.0	fTotalStrokes	 14
U+47B0.0	fCangjie	GOGLC
U+47B0.0	fFourCorner	44800
U+47B0.0	fCantonese	zaak6
U+47B0.0	fMandarin	chuo4; zuo2
U+47B0.0	fEnglish	walking rapidly
U+47B1.0	fUTF8	䞱
U+47B1.0	fCNS11643	5-4A32
U+47B1.0	fR/S	 156.7
U+47B1.0	fTotalStrokes	 14
U+47B1.0	fCangjie	GOCOR
U+47B1.0	fFourCorner	48800
U+47B1.0	fCantonese	huk6
U+47B1.0	fMandarin	hu2
U+47B1.0	fEnglish	to fall over; to lie down
U+47B2.0	fUTF8	䞲
U+47B2.0	fGB	S-242F
U+47B2.0	fR/S	 156.7
U+47B2.0	fTotalStrokes	 14
U+47B2.0	fCangjie	GOHQI
U+47B2.0	fCantonese	ngo5
U+47B3.0	fUTF8	䞳
U+47B3.0	fCNS11643	3-4F27
U+47B3.0	fGB	5-6D53
U+47B3.0	fR/S	 156.8
U+47B3.0	fTotalStrokes	 15
U+47B3.0	fCangjie	GOYTR
U+47B3.0	fFourCorner	40800
U+47B3.0	fCantonese	baak6
U+47B3.0	fMandarin	bo2; pou4; fei4; fu4
U+47B3.0	fEnglish	same as &#x8E23;
U+47B4.0	fUTF8	䞴
U+47B4.0	fCNS11643	4-4D73
U+47B4.0	fGB	3-6E28
U+47B4.0	fR/S	 156.8
U+47B4.0	fTotalStrokes	 15
U+47B4.0	fCangjie	GOBGR
U+47B4.0	fFourCorner	47800
U+47B4.0	fCantonese	zaau1
U+47B4.0	fMandarin	zhao1; zhao4
U+47B4.0	fEnglish	to jump; to leap; to hop, limping; walking unsteadily; limping
U+47B5.0	fUTF8	䞵
U+47B5.0	fCNS11643	4-4D71
U+47B5.0	fGB	5-6D54
U+47B5.0	fR/S	 156.8
U+47B5.0	fTotalStrokes	 15
U+47B5.0	fCangjie	GOEEE
U+47B5.0	fFourCorner	47800
U+47B5.0	fCantonese	kyut3
U+47B5.0	fMandarin	jue2; jue3; zhuo2
U+47B5.0	fEnglish	jump
U+47B6.0	fUTF8	䞶
U+47B6.0	fCNS11643	3-4F24
U+47B6.0	fR/S	 156.8
U+47B6.0	fTotalStrokes	 15
U+47B6.0	fCangjie	GOAPH
U+47B6.0	fFourCorner	46800
U+47B6.0	fCantonese	tong1; tik1
U+47B6.0	fMandarin	ti4; di4
U+47B6.0	fEnglish	walking rapidly
U+47B7.0	fUTF8	䞷
U+47B7.0	fCNS11643	4-4D74
U+47B7.0	fGB	3-6E2B
U+47B7.0	fR/S	 156.8
U+47B7.0	fTotalStrokes	 15
U+47B7.0	fCangjie	GOSUU
U+47B7.0	fFourCorner	47800
U+47B7.0	fCantonese	gyut6; aang1; faat1; gwaat6
U+47B7.0	fMandarin	qu1; jue2
U+47B7.0	fEnglish	same as &#x8D89;
U+47B8.0	fUTF8	䞸
U+47B8.0	fCNS11643	7-293D
U+47B8.0	fR/S	 156.8
U+47B8.0	fTotalStrokes	 15
U+47B8.0	fCangjie	GOMMR
U+47B8.0	fFourCorner	41800
U+47B8.0	fCantonese	fu3; fu6
U+47B8.0	fMandarin	bo2; fu4
U+47B8.0	fEnglish	[1] (corrupted and non-classical form) [2] reach; arrive [3] same as &#x4EC6;
U+47B9.0	fUTF8	䞹
U+47B9.0	fCNS11643	4-5426
U+47B9.0	fR/S	 156.9
U+47B9.0	fTotalStrokes	 16
U+47B9.0	fCangjie	GOHAG
U+47B9.0	fFourCorner	46800
U+47B9.0	fCantonese	wong4; wang4
U+47B9.0	fMandarin	huang2
U+47B9.0	fEnglish	 walk
U+47BA.0	fUTF8	䞺
U+47BA.0	fCNS11643	5-585C
U+47BA.0	fR/S	 156.9
U+47BA.0	fTotalStrokes	 16
U+47BA.0	fCangjie	GOHJU
U+47BA.0	fFourCorner	42800
U+47BA.0	fCantonese	ceon1
U+47BA.0	fMandarin	chun1
U+47BA.0	fEnglish	walk
U+47BB.0	fUTF8	䞻
U+47BB.0	fCNS11643	5-5855
U+47BB.0	fR/S	 156.9
U+47BB.0	fTotalStrokes	 16
U+47BB.0	fCangjie	GONBS
U+47BB.0	fFourCorner	47800
U+47BB.0	fCantonese	jung5; zung5
U+47BB.0	fMandarin	yong3
U+47BB.0	fEnglish	 walk
U+47BC.0	fUTF8	䞼
U+47BC.0	fCNS11643	3-5278
U+47BC.0	fR/S	 156.9
U+47BC.0	fTotalStrokes	 16
U+47BC.0	fCangjie	GOVNO
U+47BC.0	fFourCorner	47800
U+47BC.0	fCantonese	cyun2; cyun3; zan3; zat3
U+47BC.0	fMandarin	cuan4; chui3; jian4; mei4
U+47BC.0	fEnglish	same as &#x7AC4;
U+47BD.0	fUTF8	䞽
U+47BD.0	fCNS11643	4-587D
U+47BD.0	fGB	3-6E2D
U+47BD.0	fR/S	 156.10
U+47BD.0	fTotalStrokes	 17; 16
U+47BD.0	fCangjie	GOJBF
U+47BD.0	fFourCorner	44800
U+47BD.0	fCantonese	saak3; sok3
U+47BD.0	fMandarin	suo3
U+47BD.0	fEnglish	[1] lie flat [2] stiff; rigid [3] be inactive [4] dead-locked
U+47BE.0	fUTF8	䞾
U+47BE.0	fCNS11643	4-5925
U+47BE.0	fGB	3-6E2E
U+47BE.0	fR/S	 156.10
U+47BE.0	fTotalStrokes	 17; 16
U+47BE.0	fCangjie	GOHYU
U+47BE.0	fFourCorner	42800
U+47BE.0	fCantonese	ci4; daai6; zaai3
U+47BE.0	fMandarin	chi2
U+47BE.0	fEnglish	[1] frivolous; contemptuous [2] despise; speak ill of
U+47BF.0	fUTF8	䞿
U+47BF.0	fCNS11643	4-587C
U+47BF.0	fGB	5-6D61
U+47BF.0	fR/S	 156.10
U+47BF.0	fTotalStrokes	 17; 16
U+47BF.0	fCangjie	JTCO
U+47BF.0	fFourCorner	30800
U+47BF.0	fCantonese	hin1
U+47BF.0	fMandarin	xian1
U+47BF.0	fEnglish	interchangeable with &#x8E47;
U+47C0.0	fUTF8	䟀
U+47C0.0	fCNS11643	5-5E6D
U+47C0.0	fR/S	 156.10
U+47C0.0	fTotalStrokes	 17
U+47C0.0	fCangjie	GOTQM
U+47C0.0	fFourCorner	48800
U+47C0.0	fCantonese	caai1
U+47C0.0	fMandarin	chi1
U+47C0.0	fEnglish	walk
U+47C1.0	fUTF8	䟁
U+47C1.0	fCNS11643	4-5D6D
U+47C1.0	fGB	3-6E31
U+47C1.0	fR/S	 156.11
U+47C1.0	fTotalStrokes	 18
U+47C1.0	fCangjie	GOVVD
U+47C1.0	fFourCorner	42800
U+47C1.0	fCantonese	haau1
U+47C1.0	fMandarin	chao1
U+47C1.0	fEnglish	a foot-race
U+47C2.0	fUTF8	䟂
U+47C2.0	fCNS11643	3-592F
U+47C2.0	fR/S	 156.11
U+47C2.0	fTotalStrokes	 18
U+47C2.0	fCangjie	GOAWE
U+47C2.0	fFourCorner	46800
U+47C2.0	fCantonese	mun4; man6
U+47C2.0	fMandarin	men2; man2
U+47C2.0	fEnglish	slow; tardy
U+47C3.0	fUTF8	䟃
U+47C3.0	fCNS11643	4-5D6C
U+47C3.0	fGB	3-6E2C
U+47C3.0	fR/S	 156.11
U+47C3.0	fTotalStrokes	 18
U+47C3.0	fCangjie	GOIIH
U+47C3.0	fFourCorner	43800
U+47C3.0	fHakka	  cam1
U+47C3.0	fCantonese	caam1; cam1; caam3; zaam4
U+47C3.0	fMandarin	can1; ca4; zan4
U+47C3.0	fEnglish	 rush after (crowd)
U+47C4.0	fUTF8	䟄
U+47C4.0	fCNS11643	4-5D6A
U+47C4.0	fGB	3-6E25
U+47C4.0	fR/S	 156.11
U+47C4.0	fTotalStrokes	 18
U+47C4.0	fCangjie	GOQMC
U+47C4.0	fFourCorner	45800
U+47C4.0	fCantonese	cik1
U+47C4.0	fMandarin	qi4; chuo4; zuo2; ze2
U+47C4.0	fEnglish	in a hurry, deceitful; cunning; swindling; wily, to walk quickly
U+47C5.0	fUTF8	䟅
U+47C5.0	fCNS11643	4-5D6E
U+47C5.0	fGB	5-6D50
U+47C5.0	fR/S	 156.11
U+47C5.0	fTotalStrokes	 18
U+47C5.0	fCangjie	JLGYO
U+47C5.0	fFourCorner	52800
U+47C5.0	fCantonese	zim6
U+47C5.0	fMandarin	zan4; jian4
U+47C5.0	fEnglish	to go ahead; to proceed; to advance, to jump; to leap swiftly, brief; short period
U+47C6.0	fUTF8	䟆
U+47C6.0	fCNS11643	3-5930
U+47C6.0	fGB	5-6D43
U+47C6.0	fR/S	 156.11
U+47C6.0	fTotalStrokes	 18
U+47C6.0	fCangjie	GOWTJ
U+47C6.0	fFourCorner	46800
U+47C6.0	fCantonese	bat1
U+47C6.0	fMandarin	bi4; bo2
U+47C6.0	fEnglish	[1] keep persons off the route of the emperor, when on tour [2] imperial halting-place [3] worship of the kitchen god (performed on the 23rd day of the 12th moon)
U+47C7.0	fUTF8	䟇
U+47C7.0	fCNS11643	4-614D
U+47C7.0	fGB	3-6D6D
U+47C7.0	fR/S	 156.12
U+47C7.0	fTotalStrokes	 19
U+47C7.0	fCangjie	GOVII
U+47C7.0	fFourCorner	42805
U+47C7.0	fCantonese	gei1
U+47C7.0	fMandarin	ji1; xi4; qi2; xi1
U+47C7.0	fEnglish	 walk
U+47C8.0	fUTF8	䟈
U+47C8.0	fCNS11643	3-5D36
U+47C8.0	fR/S	 156.13
U+47C8.0	fTotalStrokes	 20
U+47C8.0	fCangjie	GOJIG
U+47C8.0	fFourCorner	43800
U+47C8.0	fCantonese	zat6
U+47C8.0	fMandarin	zhi4; zhi2
U+47C8.0	fEnglish	walk
U+47C9.0	fUTF8	䟉
U+47C9.0	fCNS11643	4-6455
U+47C9.0	fGB	3-6E34
U+47C9.0	fR/S	 156.13
U+47C9.0	fTotalStrokes	 20
U+47C9.0	fCangjie	GOWLI
U+47C9.0	fFourCorner	46800
U+47C9.0	fCantonese	zuk1; zuk6
U+47C9.0	fMandarin	zhu2; shu3
U+47C9.0	fEnglish	[1] walk [2] toddling [3] jump; leap
U+47CA.0	fUTF8	䟊
U+47CA.0	fCNS11643	4-6453
U+47CA.0	fGB	3-6E33
U+47CA.0	fR/S	 156.13
U+47CA.0	fTotalStrokes	 20
U+47CA.0	fCangjie	GOYPO
U+47CA.0	fFourCorner	41800
U+47CA.0	fCantonese	keoi4
U+47CA.0	fMandarin	qu2
U+47CA.0	fEnglish	short strides, with rapid strides, swiftly; quickly
U+47CB.0	fUTF8	䟋
U+47CB.0	fCNS11643	4-6456
U+47CB.0	fGB	3-6E35
U+47CB.0	fR/S	 156.13
U+47CB.0	fTotalStrokes	 20
U+47CB.0	fCangjie	GONCR
U+47CB.0	fFourCorner	47800
U+47CB.0	fCantonese	zim2
U+47CB.0	fMandarin	zhan3; zhan4
U+47CB.0	fEnglish	to press forward; to go in haste; to run quickly
U+47CC.0	fUTF8	䟌
U+47CC.0	fCNS11643	4-674A
U+47CC.0	fGB	3-6E36
U+47CC.0	fR/S	 156.14
U+47CC.0	fTotalStrokes	 21
U+47CC.0	fCangjie	GOJIG
U+47CC.0	fFourCorner	43800
U+47CC.0	fCantonese	zit6
U+47CC.0	fMandarin	ji2; jie2
U+47CC.0	fEnglish	diverging, to go, to walk
U+47CD.0	fUTF8	䟍
U+47CD.0	fCNS11643	4-694E
U+47CD.0	fGB	3-6E38
U+47CD.0	fR/S	 156.15
U+47CD.0	fTotalStrokes	 21; 22
U+47CD.0	fCangjie	GOHUS
U+47CD.0	fFourCorner	46800
U+47CD.0	fCantonese	bin1
U+47CD.0	fMandarin	bian1; dian2
U+47CD.0	fEnglish	to walk; to travel, to go
U+47CE.0	fUTF8	䟎
U+47CE.0	fCNS11643	3-5F66
U+47CE.0	fR/S	 156.15
U+47CE.0	fTotalStrokes	 22
U+47CE.0	fCangjie	GOQOC
U+47CE.0	fFourCorner	45800
U+47CE.0	fHakka	 zan3
U+47CE.0	fCantonese	zaan2
U+47CE.0	fMandarin	zan3
U+47CF.0	fUTF8	䟏
U+47CF.0	fCNS11643	4-694F
U+47CF.0	fGB	5-6D3E
U+47CF.0	fR/S	 156.15
U+47CF.0	fTotalStrokes	 22; 21
U+47CF.0	fCangjie	GOVID
U+47CF.0	fFourCorner	42809
U+47CF.0	fCantonese	lik6; lik1
U+47CF.0	fMandarin	li4; yue4; shuo4
U+47CF.0	fEnglish	[1] jump; leap; bounce; spring [2] run over; oppress
U+47D0.0	fUTF8	䟐
U+47D0.0	fCNS11643	4-6B23
U+47D0.0	fGB	3-6D74
U+47D0.0	fR/S	 156.16
U+47D0.0	fTotalStrokes	 23
U+47D0.0	fCangjie	GOMHM
U+47D0.0	fFourCorner	41800
U+47D0.0	fCantonese	lik6; hip3
U+47D0.0	fMandarin	li4
U+47D0.0	fEnglish	[1] walk [2] agile; adroit [3] a method of wielding the brush in writing Chinese characters
U+47D1.0	fUTF8	䟑
U+47D1.0	fCNS11643	4-6C28
U+47D1.0	fGB	5-6D6A
U+47D1.0	fR/S	 156.17
U+47D1.0	fTotalStrokes	 24
U+47D1.0	fCangjie	GOOMB
U+47D1.0	fFourCorner	48800
U+47D1.0	fCantonese	joek6; zoek6
U+47D1.0	fMandarin	yue4; la3
U+47D1.0	fEnglish	[1] walking rapidly [2] ancient form of &#x8E8D;
U+47D2.0	fUTF8	䟒
U+47D2.0	fCNS11643	4-6D27
U+47D2.0	fGB	3-6E39
U+47D2.0	fR/S	 156.18
U+47D2.0	fTotalStrokes	 25
U+47D2.0	fCangjie	GOTRG
U+47D2.0	fFourCorner	44800
U+47D2.0	fCantonese	kyun4; hyut3
U+47D2.0	fMandarin	quan2; guan4
U+47D2.0	fEnglish	coiled; curled; not stretched; contracted
U+47D3.0	fUTF8	䟓
U+47D3.0	fCNS11643	4-2B4C
U+47D3.0	fGB	3-6F4D
U+47D3.0	fR/S	 157.2
U+47D3.0	fTotalStrokes	 9
U+47D3.0	fCangjie	RMMN
U+47D3.0	fFourCorner	61120
U+47D3.0	fHakka	 dang1; dang2
U+47D3.0	fCantonese	caang1; ding1
U+47D3.0	fMandarin	cheng1
U+47D3.0	fEnglish	[1] walk slowly [2] lame; crippled [3] long and thin legs [4] walk alone [5] insist on one's ways in doing things
U+47D4.0	fUTF8	䟔
U+47D4.0	fCNS11643	3-2F2F
U+47D4.0	fGB	5-6E65
U+47D4.0	fR/S	 157.2
U+47D4.0	fTotalStrokes	 9
U+47D4.0	fCangjie	RMY
U+47D4.0	fFourCorner	63100
U+47D4.0	fCantonese	fu6
U+47D4.0	fMandarin	fu4
U+47D4.0	fEnglish	[1] same as &#x8D74; [2] same as &#x8E23;
U+47D5.0	fUTF8	䟕
U+47D5.0	fCNS11643	4-302A
U+47D5.0	fGB	3-6F4F
U+47D5.0	fR/S	 157.3
U+47D5.0	fTotalStrokes	 10
U+47D5.0	fCangjie	RMEI
U+47D5.0	fFourCorner	67140
U+47D5.0	fHakka	 ca5; za5
U+47D5.0	fCantonese	caa1; caa3; zaai3
U+47D5.0	fMandarin	cha4
U+47D5.0	fEnglish	where the road diverges
U+47D6.0	fUTF8	䟖
U+47D6.0	fCNS11643	3-337E
U+47D6.0	fR/S	 157.3
U+47D6.0	fTotalStrokes	 10
U+47D6.0	fCangjie	RMU
U+47D6.0	fFourCorner	62170
U+47D6.0	fCantonese	tong3; cang1
U+47D6.0	fMandarin	zhi3; tang4
U+47D7.0	fUTF8	䟗
U+47D7.0	fCNS11643	4-355E
U+47D7.0	fGB	3-6F58
U+47D7.0	fR/S	 157.4
U+47D7.0	fTotalStrokes	 11
U+47D7.0	fCangjie	RMHVP
U+47D7.0	fFourCorner	62140
U+47D7.0	fCantonese	si6; se4; zai6
U+47D7.0	fMandarin	shi4
U+47D7.0	fEnglish	[1] stand for a long time [2] accumulate; amass; store up
U+47D8.0	fUTF8	䟘
U+47D8.0	fCNS11643	4-3559
U+47D8.0	fGB	3-6F5A
U+47D8.0	fR/S	 157.4
U+47D8.0	fTotalStrokes	 11
U+47D8.0	fCangjie	RMYHN
U+47D8.0	fFourCorner	60110
U+47D8.0	fHakka	 hong5
U+47D8.0	fCantonese	hong6
U+47D8.0	fMandarin	hang4
U+47D8.0	fEnglish	to straighten; to stretch out
U+47D9.0	fUTF8	䟙
U+47D9.0	fCNS11643	5-3572
U+47D9.0	fR/S	 157.4
U+47D9.0	fTotalStrokes	 11
U+47D9.0	fCangjie	RMPSH
U+47D9.0	fFourCorner	67120
U+47D9.0	fCantonese	cit3; caai3
U+47D9.0	fMandarin	qie4
U+47D9.0	fEnglish	 stagger; fall; drop; stumble
U+47DA.0	fUTF8	䟚
U+47DA.0	fCNS11643	3-393B
U+47DA.0	fR/S	 157.4
U+47DA.0	fTotalStrokes	 11
U+47DA.0	fCangjie	RMMML
U+47DA.0	fFourCorner	61120
U+47DA.0	fCantonese	gei1; kei4
U+47DA.0	fMandarin	qi2
U+47DA.0	fEnglish	same as &#x8E11;
U+47DB.0	fUTF8	䟛
U+47DB.0	fCNS11643	4-355B
U+47DB.0	fGB	3-6F50
U+47DB.0	fR/S	 157.4
U+47DB.0	fTotalStrokes	 11
U+47DB.0	fCangjie	RMJB
U+47DB.0	fFourCorner	65120
U+47DB.0	fCantonese	bat6
U+47DB.0	fMandarin	ba2; bo2
U+47DB.0	fEnglish	walking rapidly; in a hurry, to stride over; to step across
U+47DC.0	fUTF8	䟜
U+47DC.0	fCNS11643	4-355C
U+47DC.0	fGB	3-6F52
U+47DC.0	fR/S	 157.4
U+47DC.0	fTotalStrokes	 11
U+47DC.0	fCangjie	RMOB
U+47DC.0	fFourCorner	64120
U+47DC.0	fCantonese	neot6; leot6 (also); nap6
U+47DC.0	fMandarin	na4
U+47DC.0	fEnglish	[1] break; damage [2] wound of the foot [3] walk
U+47DD.0	fUTF8	䟝
U+47DD.0	fCNS11643	4-355D
U+47DD.0	fGB	3-6F59
U+47DD.0	fR/S	 157.4
U+47DD.0	fTotalStrokes	 11
U+47DD.0	fCangjie	RMHNE
U+47DD.0	fFourCorner	67140
U+47DD.0	fCantonese	tau3
U+47DD.0	fMandarin	tou4
U+47DD.0	fEnglish	 walk right into
U+47DE.0	fUTF8	䟞
U+47DE.0	fCNS11643	3-393C
U+47DE.0	fR/S	 157.4
U+47DE.0	fTotalStrokes	 11
U+47DE.0	fCangjie	RMFH
U+47DE.0	fFourCorner	69120
U+47DE.0	fCantonese	cyu4
U+47DE.0	fMandarin	chu2
U+47DE.0	fEnglish	 falter; hesitate
U+47DF.0	fUTF8	䟟
U+47DF.0	fCNS11643	5-3C63
U+47DF.0	fR/S	 157.5
U+47DF.0	fTotalStrokes	 12
U+47DF.0	fCangjie	HYRYO
U+47DF.0	fFourCorner	76280
U+47DF.0	fCantonese	cuk1; cyut3; zung2
U+47DF.0	fMandarin	cu4
U+47DF.0	fEnglish	urgent; pressed for time
U+47E0.0	fUTF8	䟠
U+47E0.0	fCNS11643	4-3B4C
U+47E0.0	fGB	5-6E6C
U+47E0.0	fR/S	 157.5
U+47E0.0	fTotalStrokes	 12
U+47E0.0	fCangjie	RMIV
U+47E0.0	fFourCorner	63150
U+47E0.0	fCantonese	jyut6; zyut6
U+47E0.0	fMandarin	yue4
U+47E0.0	fEnglish	[1] swift; walking rapidly [2] same as &#x8D8A;
U+47E1.0	fUTF8	䟡
U+47E1.0	fCNS11643	4-3B50
U+47E1.0	fGB	3-6F67
U+47E1.0	fR/S	 157.5
U+47E1.0	fTotalStrokes	 12
U+47E1.0	fCangjie	RMHPM
U+47E1.0	fFourCorner	62140
U+47E1.0	fCantonese	dai3; zi1; daai3
U+47E1.0	fMandarin	di4
U+47E1.0	fEnglish	[1] trample; thread down [2] same as &#x80DD;
U+47E2.0	fUTF8	䟢
U+47E2.0	fCNS11643	3-3F3D
U+47E2.0	fR/S	 157.5
U+47E2.0	fTotalStrokes	 12
U+47E2.0	fCangjie	RMNF
U+47E2.0	fFourCorner	68190
U+47E2.0	fCantonese	nin2; lin2 (also); caan3; sung1
U+47E2.0	fMandarin	nian3; chen2; jian4
U+47E2.0	fEnglish	[1] non-classical form of &#x8DC8; [2] same as &#x8E10; [3] same as &#x8D81;
U+47E3.0	fUTF8	䟣
U+47E3.0	fCNS11643	4-3B4D
U+47E3.0	fGB	3-6F60
U+47E3.0	fR/S	 157.5
U+47E3.0	fTotalStrokes	 12
U+47E3.0	fCangjie	RMIJC
U+47E3.0	fFourCorner	63190
U+47E3.0	fCantonese	ceot1
U+47E3.0	fMandarin	chu4
U+47E3.0	fEnglish	an animal in legend, traces or footprints of an animal
U+47E4.0	fUTF8	䟤
U+47E4.0	fCNS11643	4-3B49
U+47E4.0	fGB	3-6F69
U+47E4.0	fR/S	 157.5
U+47E4.0	fTotalStrokes	 12
U+47E4.0	fCangjie	RMPH
U+47E4.0	fFourCorner	63100
U+47E4.0	fHakka	 siet7; pit7; bit7
U+47E4.0	fCantonese	bai6; bit1
U+47E4.0	fMandarin	bi4
U+47E4.0	fEnglish	 kick (as a ball)
U+47E5.0	fUTF8	䟥
U+47E5.0	fCNS11643	6-4F36
U+47E5.0	fR/S	 157.5
U+47E5.0	fTotalStrokes	 12
U+47E5.0	fCangjie	RMNIH
U+47E5.0	fFourCorner	67120
U+47E5.0	fCantonese	mung4
U+47E5.0	fMandarin	meng2; mang2
U+47E5.0	fEnglish	same as &#x9AF3;
U+47E6.0	fUTF8	䟦
U+47E6.0	fCNS11643	6-4F3C
U+47E6.0	fR/S	 157.5
U+47E6.0	fTotalStrokes	 12
U+47E6.0	fCangjie	RMHKE
U+47E6.0	fFourCorner	62140
U+47E6.0	fCantonese	bat6
U+47E6.0	fMandarin	ba2; bo2; yuan2
U+47E6.0	fEnglish	non-classical form of &#x8DCB;
U+47E7.0	fUTF8	䟧
U+47E7.0	fCNS11643	5-3C5C
U+47E7.0	fR/S	 157.5
U+47E7.0	fTotalStrokes	 12
U+47E7.0	fCangjie	RMW
U+47E7.0	fFourCorner	66100
U+47E7.0	fCantonese	tin4; zing4
U+47E7.0	fMandarin	tian2
U+47E7.0	fEnglish	sound of treading or stepping
U+47E8.0	fUTF8	䟨
U+47E8.0	fCNS11643	4-3B4A
U+47E8.0	fGB	3-6F6A
U+47E8.0	fR/S	 157.5
U+47E8.0	fTotalStrokes	 12
U+47E8.0	fCangjie	RMRVP
U+47E8.0	fFourCorner	67140
U+47E8.0	fCantonese	man4
U+47E8.0	fMandarin	min3; min2
U+47E8.0	fEnglish	[1] lame; crippled [2] go across; go beyond [3] hoofs of an animal
U+47E9.0	fUTF8	䟩
U+47E9.0	fCNS11643	4-3B4B
U+47E9.0	fGB	3-6F5E
U+47E9.0	fR/S	 157.5
U+47E9.0	fTotalStrokes	 12
U+47E9.0	fCangjie	RMGI
U+47E9.0	fFourCorner	64130
U+47E9.0	fCantonese	le5; fui1; geoi3; hai6; koek3
U+47E9.0	fMandarin	le3; lie3
U+47E9.0	fEnglish	hard of walk forward
U+47EA.0	fUTF8	䟪
U+47EA.0	fCNS11643	4-3B4F
U+47EA.0	fGB	3-6F56
U+47EA.0	fR/S	 157.5
U+47EA.0	fTotalStrokes	 12
U+47EA.0	fCangjie	RMHIO
U+47EA.0	fFourCorner	62130
U+47EA.0	fCantonese	fung2
U+47EA.0	fMandarin	feng3
U+47EA.0	fEnglish	same as &#x8982;
U+47EB.0	fUTF8	䟫
U+47EB.0	fCNS11643	3-3F3C
U+47EB.0	fGB	5-6E5C
U+47EB.0	fR/S	 157.5
U+47EB.0	fTotalStrokes	 12
U+47EB.0	fCangjie	FBRYO
U+47EB.0	fFourCorner	90800
U+47EB.0	fHakka	 cang1; cang5; cen2; chen2
U+47EB.0	fCantonese	cang1; zing3
U+47EB.0	fMandarin	cheng2
U+47EB.0	fEnglish	[1] perch for fowls [2] roost
U+47EC.0	fUTF8	䟬
U+47EC.0	fCNS11643	5-3C5F
U+47EC.0	fR/S	 157.5
U+47EC.0	fTotalStrokes	 12
U+47EC.0	fCangjie	RMOM
U+47EC.0	fFourCorner	62110
U+47EC.0	fCantonese	jau3
U+47EC.0	fMandarin	qiu4
U+47EC.0	fEnglish	walk
U+47ED.0	fUTF8	䟭
U+47ED.0	fCNS11643	4-3B51
U+47ED.0	fR/S	 157.5
U+47ED.0	fTotalStrokes	 12
U+47ED.0	fCangjie	RMOS
U+47ED.0	fFourCorner	68110
U+47ED.0	fCantonese	tiu3; tou4
U+47ED.0	fMandarin	zuo4; tiao2
U+47ED.0	fEnglish	[1] non-classical form of &#x8DF3; [2] same as &#x963C;
U+47EE.0	fUTF8	䟮
U+47EE.0	fCNS11643	4-4129
U+47EE.0	fGB	3-6F72
U+47EE.0	fR/S	 157.6
U+47EE.0	fTotalStrokes	 13
U+47EE.0	fCangjie	RMOIK
U+47EE.0	fFourCorner	63130
U+47EE.0	fCantonese	fuk6
U+47EE.0	fMandarin	fu2
U+47EE.0	fEnglish	same as &#x5310;
U+47EF.0	fUTF8	䟯
U+47EF.0	fCNS11643	5-434B
U+47EF.0	fR/S	 157.6
U+47EF.0	fTotalStrokes	 13
U+47EF.0	fCangjie	RMHJR
U+47EF.0	fFourCorner	61164
U+47EF.0	fCantonese	kut3; kwut3 (also)
U+47EF.0	fMandarin	kuo4
U+47EF.0	fEnglish	[1] tread on [2] kick
U+47F0.0	fUTF8	䟰
U+47F0.0	fCNS11643	3-446C
U+47F0.0	fR/S	 157.6
U+47F0.0	fTotalStrokes	 13
U+47F0.0	fCangjie	RMHON
U+47F0.0	fFourCorner	61120
U+47F0.0	fCantonese	gin2
U+47F0.0	fMandarin	xian2; jian3
U+47F0.0	fEnglish	walk
U+47F1.0	fUTF8	䟱
U+47F1.0	fCNS11643	3-446E
U+47F1.0	fR/S	 157.6
U+47F1.0	fTotalStrokes	 13
U+47F1.0	fCangjie	RMDB
U+47F1.0	fFourCorner	65190
U+47F1.0	fCantonese	zik1; ci3; cuk1; zik6
U+47F1.0	fMandarin	ji1
U+47F1.0	fEnglish	same as &#x616B;
U+47F2.0	fUTF8	䟲
U+47F2.0	fGB	S-2271
U+47F2.0	fR/S	 157.6
U+47F2.0	fTotalStrokes	 13
U+47F2.0	fCangjie	RMYIU
U+47F2.0	fCantonese	cung1
U+47F3.0	fUTF8	䟳
U+47F3.0	fCNS11643	F-4558
U+47F3.0	fR/S	 157.6
U+47F3.0	fTotalStrokes	 13
U+47F3.0	fCangjie	ROSMM
U+47F3.0	fFourCorner	6712
U+47F3.0	fCantonese	cuk1
U+47F4.0	fUTF8	䟴
U+47F4.0	fCNS11643	5-4A3E
U+47F4.0	fR/S	 157.7
U+47F4.0	fTotalStrokes	 14
U+47F4.0	fCangjie	RMMMV
U+47F4.0	fFourCorner	62130
U+47F4.0	fCantonese	ngan3; an3 (also); zaan1; zaan3
U+47F4.0	fMandarin	zhen4
U+47F4.0	fEnglish	[1] shake; vibrate; move [2] be shocked or shaken
U+47F5.0	fUTF8	䟵
U+47F5.0	fCNS11643	4-475E
U+47F5.0	fGB	3-6F7A
U+47F5.0	fR/S	 157.7
U+47F5.0	fTotalStrokes	 14
U+47F5.0	fCangjie	RMIJE
U+47F5.0	fFourCorner	63130
U+47F5.0	fCantonese	kau4
U+47F5.0	fMandarin	qiu2
U+47F5.0	fEnglish	 thread on
U+47F6.0	fUTF8	䟶
U+47F6.0	fCNS11643	4-4761
U+47F6.0	fGB	3-7021
U+47F6.0	fR/S	 157.7
U+47F6.0	fTotalStrokes	 14
U+47F6.0	fCangjie	RMOOG
U+47F6.0	fFourCorner	68110
U+47F6.0	fHakka	 co5; ts'io5
U+47F6.0	fCantonese	co3
U+47F6.0	fMandarin	cuo4; zuo4
U+47F6.0	fEnglish	 slip; stumble
U+47F7.0	fUTF8	䟷
U+47F7.0	fCNS11643	4-475F
U+47F7.0	fGB	3-6F79
U+47F7.0	fR/S	 157.7
U+47F7.0	fTotalStrokes	 14
U+47F7.0	fCangjie	RMQHL
U+47F7.0	fFourCorner	62120
U+47F7.0	fHakka	 ki3; ji3; ri3
U+47F7.0	fCantonese	cai3
U+47F7.0	fMandarin	chi4
U+47F7.0	fEnglish	to jump; to leap; to bounce; to spring, to pass over; to go beyond; to transgress; to exceed, lame; crippled
U+47F8.0	fUTF8	䟸
U+47F8.0	fCNS11643	4-4762
U+47F8.0	fGB	3-7023
U+47F8.0	fR/S	 157.7
U+47F8.0	fTotalStrokes	 14
U+47F8.0	fCangjie	RMBT
U+47F8.0	fFourCorner	67140
U+47F8.0	fCantonese	kwai4
U+47F8.0	fMandarin	kui2
U+47F8.0	fEnglish	[1] the calf; muscles; tissue of the part of leg between the knee and ankle [2] stumble; slip [3] the instep [4] jump; leap [5] a disease of the feet
U+47F9.0	fUTF8	䟹
U+47F9.0	fCNS11643	4-4765
U+47F9.0	fGB	3-7022
U+47F9.0	fR/S	 157.7
U+47F9.0	fTotalStrokes	 14
U+47F9.0	fCangjie	RMBDI
U+47F9.0	fFourCorner	62140
U+47F9.0	fCantonese	lyut6
U+47F9.0	fMandarin	lie4; lin4
U+47F9.0	fEnglish	to hop about; to fling oneself about in temper, to pass over; to go beyond; to exceed
U+47FA.0	fUTF8	䟺
U+47FA.0	fCNS11643	3-4978
U+47FA.0	fGB	5-6E68
U+47FA.0	fR/S	 157.7
U+47FA.0	fTotalStrokes	 14
U+47FA.0	fCangjie	RMBUC
U+47FA.0	fFourCorner	66180
U+47FA.0	fCantonese	bui3
U+47FA.0	fMandarin	bei4; bang3; pei4
U+47FA.0	fEnglish	[1] walk slowly because of the wrong position of the feet [2] same as &#x72FD;
U+47FB.0	fUTF8	䟻
U+47FB.0	fCNS11643	4-4764
U+47FB.0	fGB	3-6F7E
U+47FB.0	fR/S	 157.7
U+47FB.0	fTotalStrokes	 14
U+47FB.0	fCangjie	RMOMD
U+47FB.0	fFourCorner	68190
U+47FB.0	fCantonese	dou6
U+47FB.0	fMandarin	du4
U+47FB.0	fEnglish	bare feet; bare-footed, to walk in bare feet, hard to go forward
U+47FC.0	fUTF8	䟼
U+47FC.0	fCNS11643	3-4F28
U+47FC.0	fR/S	 157.8
U+47FC.0	fTotalStrokes	 15
U+47FC.0	fCangjie	RMMPM
U+47FC.0	fFourCorner	63140
U+47FC.0	fHakka	 wu5
U+47FC.0	fCantonese	mou5
U+47FC.0	fMandarin	wu3
U+47FC.0	fEnglish	[1] footprints; track [2] whereabout
U+47FD.0	fUTF8	䟽
U+47FD.0	fCNS11643	3-446F
U+47FD.0	fR/S	 157.6; 157.7
U+47FD.0	fTotalStrokes	 13; 14
U+47FD.0	fCangjie	RMYIU
U+47FD.0	fFourCorner	60110
U+47FD.0	fCantonese	so1
U+47FD.0	fEnglish	same as &#x758F;
U+47FE.0	fUTF8	䟾
U+47FE.0	fCNS11643	4-4D77
U+47FE.0	fGB	3-7035
U+47FE.0	fR/S	 157.8
U+47FE.0	fTotalStrokes	 15
U+47FE.0	fCangjie	RMEEE
U+47FE.0	fFourCorner	67140
U+47FE.0	fCantonese	dyut3
U+47FE.0	fMandarin	zhuo2; jue3
U+47FE.0	fEnglish	[1] to jump; to leap; to bounce; to spring [2] same as &#x47B5;
U+47FF.0	fUTF8	䟿
U+47FF.0	fCNS11643	4-4E22
U+47FF.0	fGB	3-7032
U+47FF.0	fR/S	 157.8
U+47FF.0	fTotalStrokes	 15
U+47FF.0	fCangjie	RMVNE
U+47FF.0	fFourCorner	67130
U+47FF.0	fHakka	 luk7
U+47FF.0	fCantonese	luk6; luk1
U+47FF.0	fMandarin	lu4
U+47FF.0	fEnglish	[1] move [2] walk
U+4800.0	fUTF8	䠀
U+4800.0	fCNS11643	3-4F2A
U+4800.0	fGB	S-2273
U+4800.0	fR/S	 157.8
U+4800.0	fTotalStrokes	 15
U+4800.0	fCangjie	RMFBR
U+4800.0	fFourCorner	69120
U+4800.0	fCantonese	cang1; tong3
U+4800.0	fMandarin	chang3
U+4800.0	fEnglish	sit cross-legged like a Buddhist priest
U+4801.0	fUTF8	䠁
U+4801.0	fCNS11643	7-294E
U+4801.0	fR/S	 157.8
U+4801.0	fTotalStrokes	 15
U+4801.0	fCangjie	RMIIM
U+4801.0	fFourCorner	63110
U+4801.0	fCantonese	zai1
U+4801.0	fMandarin	ji1
U+4801.0	fEnglish	same as &#x8E8B;
U+4802.0	fUTF8	䠂
U+4802.0	fCNS11643	3-4F2E
U+4802.0	fR/S	 157.8
U+4802.0	fTotalStrokes	 15
U+4802.0	fCangjie	DDRYO
U+4802.0	fFourCorner	44800
U+4802.0	fCantonese	co2
U+4802.0	fMandarin	chu3; chu2
U+4802.0	fEnglish	same as &#x695A;
U+4803.0	fUTF8	䠃
U+4803.0	fCNS11643	3-4F29
U+4803.0	fR/S	 157.8
U+4803.0	fTotalStrokes	 15
U+4803.0	fCangjie	RMMLB
U+4803.0	fFourCorner	61120
U+4803.0	fCantonese	loeng5
U+4803.0	fMandarin	liang3
U+4803.0	fEnglish	 squat; crouch
U+4804.0	fUTF8	䠄
U+4804.0	fCNS11643	4-4D7E
U+4804.0	fGB	3-702B
U+4804.0	fR/S	 157.8
U+4804.0	fTotalStrokes	 15
U+4804.0	fCangjie	RMTBC
U+4804.0	fFourCorner	65180
U+4804.0	fCantonese	tin2
U+4804.0	fMandarin	tian3
U+4804.0	fEnglish	[1] walk [2] tracks or whereabout [3] disease of the feet
U+4805.0	fUTF8	䠅
U+4805.0	fCNS11643	4-4E21
U+4805.0	fGB	3-702C
U+4805.0	fR/S	 157.8
U+4805.0	fTotalStrokes	 15
U+4805.0	fCangjie	RMWHD
U+4805.0	fFourCorner	66100
U+4805.0	fCantonese	kwan2
U+4805.0	fMandarin	kun3; ta4
U+4805.0	fEnglish	hands and feet chapped from the cold
U+4806.0	fUTF8	䠆
U+4806.0	fCNS11643	4-4D7D
U+4806.0	fGB	3-6F53
U+4806.0	fR/S	 157.8
U+4806.0	fTotalStrokes	 15
U+4806.0	fCangjie	RMSMV
U+4806.0	fFourCorner	61130
U+4806.0	fCantonese	coeng4
U+4806.0	fMandarin	chang2
U+4806.0	fEnglish	 kowtow; kneel and make obeisance
U+4807.0	fUTF8	䠇
U+4807.0	fCNS11643	4-4D79
U+4807.0	fGB	3-7034
U+4807.0	fR/S	 157.8
U+4807.0	fTotalStrokes	 15
U+4807.0	fCangjie	RMSUU
U+4807.0	fFourCorner	67170
U+4807.0	fCantonese	gwat6
U+4807.0	fMandarin	qu1; jue2
U+4807.0	fEnglish	strong; robust; healthy
U+4808.0	fUTF8	䠈
U+4808.0	fCNS11643	4-4D78
U+4808.0	fGB	3-7033
U+4808.0	fR/S	 157.8
U+4808.0	fTotalStrokes	 15
U+4808.0	fCangjie	RMLE
U+4808.0	fFourCorner	65130
U+4808.0	fHakka	 tut7; tut8
U+4808.0	fCantonese	teot1
U+4808.0	fMandarin	tu2
U+4808.0	fEnglish	 be hindered; make no progress
U+4809.0	fUTF8	䠉
U+4809.0	fCNS11643	4-4D76
U+4809.0	fGB	5-6F26
U+4809.0	fR/S	 157.8
U+4809.0	fTotalStrokes	 15
U+4809.0	fCangjie	RMJRR
U+4809.0	fFourCorner	63170
U+4809.0	fHakka	 gwon3
U+4809.0	fCantonese	wun6
U+4809.0	fMandarin	huan4; hua4; hui3
U+4809.0	fEnglish	same as &#x902D;
U+480A.0	fUTF8	䠊
U+480A.0	fCNS11643	3-4F2D
U+480A.0	fGB	5-6E7C
U+480A.0	fR/S	 157.8
U+480A.0	fTotalStrokes	 15
U+480A.0	fCangjie	RMLMY
U+480A.0	fFourCorner	61110
U+480A.0	fHakka	 pi5; pui5
U+480A.0	fCantonese	fei6; fai3 (also); faai4
U+480A.0	fMandarin	dei4; fei4
U+480A.0	fEnglish	 cut off the feet (one of the five punishments in ancient times)
U+480B.0	fUTF8	䠋
U+480B.0	fCNS11643	4-4E23
U+480B.0	fGB	5-6F22
U+480B.0	fR/S	 157.8
U+480B.0	fTotalStrokes	 15
U+480B.0	fCangjie	RMHWJ
U+480B.0	fFourCorner	66140
U+480B.0	fCantonese	pe5; bei2
U+480B.0	fMandarin	bi4; bi3; ma4
U+480B.0	fEnglish	[1] bell shaped with large base [2] ancient form of &#x9AC0;
U+480C.0	fUTF8	䠌
U+480C.0	fGB	S-2272
U+480C.0	fR/S	 157.8
U+480C.0	fTotalStrokes	 15
U+480C.0	fCangjie	RMEB
U+480C.0	fCantonese	daap6
U+480D.0	fUTF8	䠍
U+480D.0	fCNS11643	4-542E
U+480D.0	fGB	3-7041
U+480D.0	fR/S	 157.9
U+480D.0	fTotalStrokes	 16
U+480D.0	fCangjie	RMRSE
U+480D.0	fFourCorner	67140
U+480D.0	fCantonese	haa4
U+480D.0	fMandarin	xia2; qia2
U+480D.0	fEnglish	at the feet; to learn; now, shoes, (dialect) to take a stride, to sit astride on; to straddle; to ride, to stride over (a ditch, etc.)
U+480E.0	fUTF8	䠎
U+480E.0	fCNS11643	3-527D
U+480E.0	fR/S	 157.9
U+480E.0	fTotalStrokes	 16
U+480E.0	fCangjie	RMSMG
U+480E.0	fFourCorner	67110
U+480E.0	fHakka	 wok7; vok7
U+480E.0	fCantonese	ak1; ngak1 (also); aak1
U+480E.0	fMandarin	wo4
U+480E.0	fEnglish	 grasp
U+480F.0	fUTF8	䠏
U+480F.0	fCNS11643	3-527A
U+480F.0	fGB	5-6F2F
U+480F.0	fR/S	 157.9
U+480F.0	fTotalStrokes	 16
U+480F.0	fCangjie	RMNOK
U+480F.0	fFourCorner	62130
U+480F.0	fCantonese	gwai6; kwaai4
U+480F.0	fMandarin	ji4
U+480F.0	fEnglish	[1] feet [2] same as &#x47F8;
U+4810.0	fUTF8	䠐
U+4810.0	fCNS11643	4-542F
U+4810.0	fGB	3-703D
U+4810.0	fR/S	 157.9
U+4810.0	fTotalStrokes	 16
U+4810.0	fCangjie	RMBUK
U+4810.0	fFourCorner	66130
U+4810.0	fCantonese	kwik1
U+4810.0	fMandarin	qiu4; qu4
U+4810.0	fEnglish	to squat; to crouch, to move elastically; to be easily bent and stretched; to be flexible: (firuratively) to submit or rise according to circumstances
U+4811.0	fUTF8	䠑
U+4811.0	fCNS11643	4-542D
U+4811.0	fGB	3-703C
U+4811.0	fR/S	 157.9
U+4811.0	fTotalStrokes	 16
U+4811.0	fCangjie	RMKGG
U+4811.0	fFourCorner	64110
U+4811.0	fCantonese	kwai2
U+4811.0	fMandarin	kui3
U+4811.0	fEnglish	between the buttocks; the stride made by a man
U+4812.0	fUTF8	䠒
U+4812.0	fCNS11643	5-5862
U+4812.0	fR/S	 157.9
U+4812.0	fTotalStrokes	 16
U+4812.0	fCangjie	RMJRB
U+4812.0	fFourCorner	67120
U+4812.0	fCantonese	wu4
U+4812.0	fMandarin	hu2
U+4812.0	fEnglish	[1] courtesy of a minority group in ancient times [2] to fall [3] drop or go down, on one's knees; kneel down
U+4813.0	fUTF8	䠓
U+4813.0	fCNS11643	4-5430
U+4813.0	fGB	3-7040
U+4813.0	fR/S	 157.9
U+4813.0	fTotalStrokes	 16
U+4813.0	fCangjie	RMTCW
U+4813.0	fFourCorner	68160
U+4813.0	fCantonese	cau1
U+4813.0	fMandarin	qiu1; cu4; jiu4; qu4
U+4813.0	fEnglish	same as &#x8DA5;
U+4814.0	fUTF8	䠔
U+4814.0	fCNS11643	3-5279
U+4814.0	fR/S	 157.9
U+4814.0	fTotalStrokes	 16
U+4814.0	fCangjie	RMTPO
U+4814.0	fFourCorner	68130
U+4814.0	fCantonese	seoi6; maai4
U+4814.0	fMandarin	sui4
U+4814.0	fEnglish	far and deep
U+4815.0	fUTF8	䠕
U+4815.0	fCNS11643	5-5E7B
U+4815.0	fR/S	 157.10; 157.9
U+4815.0	fTotalStrokes	 17; 16
U+4815.0	fCangjie	RMYPD
U+4815.0	fFourCorner	61190
U+4815.0	fCantonese	caai1
U+4815.0	fMandarin	cai1
U+4815.0	fEnglish	rapid marching or running; to go quickly, to tread upon; to trample; to step upon
U+4816.0	fUTF8	䠖
U+4816.0	fGB	S-2270
U+4816.0	fR/S	 157.9
U+4816.0	fTotalStrokes	 16
U+4816.0	fCangjie	RMIOR
U+4816.0	fCantonese	zi1
U+4817.0	fUTF8	䠗
U+4817.0	fCNS11643	4-592B
U+4817.0	fGB	3-7042
U+4817.0	fR/S	 157.10
U+4817.0	fTotalStrokes	 16; 17
U+4817.0	fCangjie	RMHUK
U+4817.0	fFourCorner	66130
U+4817.0	fCantonese	jau3; cau5
U+4817.0	fMandarin	qiu4
U+4817.0	fEnglish	lame
U+4818.0	fUTF8	䠘
U+4818.0	fCNS11643	4-592C
U+4818.0	fGB	3-7043
U+4818.0	fR/S	 157.10
U+4818.0	fTotalStrokes	 17; 16
U+4818.0	fCangjie	RMHWP
U+4818.0	fFourCorner	66110
U+4818.0	fCantonese	pei3
U+4818.0	fMandarin	pi4
U+4818.0	fEnglish	one-legged, crippled; halt, a mate; to mate
U+4819.0	fUTF8	䠙
U+4819.0	fCNS11643	4-5926
U+4819.0	fGB	3-7047
U+4819.0	fR/S	 157.10
U+4819.0	fTotalStrokes	 17; 16
U+4819.0	fCangjie	RMYBS
U+4819.0	fFourCorner	60120
U+4819.0	fHakka	 pong2
U+4819.0	fCantonese	pong4; paau4
U+4819.0	fMandarin	pang2; bei4; pao2
U+4819.0	fEnglish	 hurry on; urgent
U+481A.0	fUTF8	䠚
U+481A.0	fCNS11643	F-5C78
U+481A.0	fGB	5-6F35
U+481A.0	fR/S	 157.10
U+481A.0	fTotalStrokes	 17
U+481A.0	fCangjie	RMJCO
U+481A.0	fFourCorner	6313
U+481A.0	fHakka	 wa3; va3
U+481A.0	fCantonese	waa2; maat6
U+481A.0	fMandarin	wa4
U+481A.0	fEnglish	 step; tread
U+481B.0	fUTF8	䠛
U+481B.0	fCNS11643	4-592A
U+481B.0	fGB	3-7044
U+481B.0	fR/S	 157.10
U+481B.0	fTotalStrokes	 17; 16
U+481B.0	fCangjie	RMBOU
U+481B.0	fFourCorner	67170
U+481B.0	fHakka	 jau2; rau2; jeu2
U+481B.0	fCantonese	jiu4; ziu2; ziu4
U+481B.0	fMandarin	yao2; jiao3; xiao4
U+481B.0	fEnglish	 leap; skip
U+481C.0	fUTF8	䠜
U+481C.0	fCNS11643	4-5928
U+481C.0	fGB	3-7038
U+481C.0	fR/S	 157.10
U+481C.0	fTotalStrokes	 17; 16
U+481C.0	fCangjie	RMTSJ
U+481C.0	fFourCorner	64140
U+481C.0	fCantonese	jung4; zung4
U+481C.0	fMandarin	rong2
U+481C.0	fEnglish	 walk
U+481D.0	fUTF8	䠝
U+481D.0	fCNS11643	3-5653
U+481D.0	fR/S	 157.10
U+481D.0	fTotalStrokes	 17
U+481D.0	fCangjie	RMRBC
U+481D.0	fFourCorner	66180
U+481D.0	fCantonese	hyun1
U+481D.0	fMandarin	xuan1
U+481D.0	fEnglish	to stand up, to establish
U+481E.0	fUTF8	䠞
U+481E.0	fCNS11643	3-5933
U+481E.0	fR/S	 157.11
U+481E.0	fTotalStrokes	 18
U+481E.0	fCangjie	RMIHF
U+481E.0	fFourCorner	63150
U+481E.0	fHakka	 cuk7
U+481E.0	fCantonese	cuk1
U+481E.0	fMandarin	cu4; qi1; qi2; zu2
U+481E.0	fEnglish	same as &#x8E59;
U+481F.0	fUTF8	䠟
U+481F.0	fCNS11643	4-5D74
U+481F.0	fGB	3-6F49
U+481F.0	fR/S	 157.11
U+481F.0	fTotalStrokes	 18
U+481F.0	fCangjie	GIRYO
U+481F.0	fFourCorner	44800
U+481F.0	fHakka	 tiap8
U+481F.0	fCantonese	dip6
U+481F.0	fMandarin	die2; she4
U+481F.0	fEnglish	 short step
U+4820.0	fUTF8	䠠
U+4820.0	fCNS11643	4-5D78
U+4820.0	fGB	3-6F4A
U+4820.0	fR/S	 157.11
U+4820.0	fTotalStrokes	 18
U+4820.0	fCangjie	KPBLO
U+4820.0	fFourCorner	44800
U+4820.0	fCantonese	cai3
U+4820.0	fMandarin	dai4; chi4
U+4820.0	fEnglish	[1] to go, to walk in one leg; leap; crippled [2] ancient form of &#x901D; [3] interchangeable with &#x6EEF;
U+4821.0	fUTF8	䠡
U+4821.0	fCNS11643	4-5D72
U+4821.0	fGB	3-7049
U+4821.0	fR/S	 157.11
U+4821.0	fTotalStrokes	 18
U+4821.0	fCangjie	RMYPM
U+4821.0	fFourCorner	61110
U+4821.0	fCantonese	co4
U+4821.0	fMandarin	cuo2
U+4821.0	fEnglish	sound of footsteps; to tread on; to stamp, to stagger; to fall; to drop; to stumble, to stop up; to gag; blocked
U+4822.0	fUTF8	䠢
U+4822.0	fCNS11643	4-5D75
U+4822.0	fGB	3-6B39
U+4822.0	fR/S	 157.11
U+4822.0	fTotalStrokes	 18
U+4822.0	fCangjie	TWLO
U+4822.0	fFourCorner	44800
U+4822.0	fCantonese	mung6
U+4822.0	fMandarin	meng4
U+4822.0	fEnglish	[1] weary; tired; fatigued [2] lime; crippled [3] stumble and fall
U+4823.0	fUTF8	䠣
U+4823.0	fCNS11643	7-4A57
U+4823.0	fR/S	 157.12
U+4823.0	fTotalStrokes	 19
U+4823.0	fCangjie	RMRUC
U+4823.0	fFourCorner	67180
U+4823.0	fCantonese	syun2
U+4823.0	fMandarin	xuan4; xuan3
U+4823.0	fEnglish	net to trap the animals
U+4824.0	fUTF8	䠤
U+4824.0	fCNS11643	4-6150
U+4824.0	fGB	3-7030
U+4824.0	fR/S	 157.12
U+4824.0	fTotalStrokes	 19
U+4824.0	fCangjie	RMRRJ
U+4824.0	fFourCorner	66150
U+4824.0	fCantonese	do2
U+4824.0	fMandarin	duo3; duo4
U+4824.0	fEnglish	toddle, to fall; fallen; prostrate
U+4825.0	fUTF8	䠥
U+4825.0	fCNS11643	3-5934
U+4825.0	fGB	5-6F36
U+4825.0	fR/S	 157.12; 157.11
U+4825.0	fTotalStrokes	 19; 18
U+4825.0	fCangjie	RMFBK
U+4825.0	fFourCorner	68140
U+4825.0	fCantonese	bit6
U+4825.0	fMandarin	pie1; bie2
U+4825.0	fEnglish	same as &#x8E69;
U+4826.0	fUTF8	䠦
U+4826.0	fCNS11643	4-6151
U+4826.0	fGB	5-6F3E
U+4826.0	fR/S	 157.12
U+4826.0	fTotalStrokes	 19
U+4826.0	fCangjie	RMORA
U+4826.0	fFourCorner	66160
U+4826.0	fCantonese	zi3; ci3; si3
U+4826.0	fMandarin	zhi4; zhe4
U+4826.0	fEnglish	non-classical form of &#x8E93;
U+4827.0	fUTF8	䠧
U+4827.0	fCNS11643	4-5D77
U+4827.0	fGB	5-6F3B
U+4827.0	fR/S	 157.12; 157.11
U+4827.0	fTotalStrokes	 18; 19
U+4827.0	fCangjie	RMSJA
U+4827.0	fFourCorner	67160
U+4827.0	fCantonese	caa4
U+4827.0	fMandarin	chu2
U+4827.0	fEnglish	hard to go forward, stop; to delay
U+4828.0	fUTF8	䠨
U+4828.0	fCNS11643	4-645B
U+4828.0	fGB	3-7054
U+4828.0	fR/S	 157.13
U+4828.0	fTotalStrokes	 20
U+4828.0	fCangjie	RMNCR
U+4828.0	fFourCorner	67160
U+4828.0	fHakka	 zham5; zham6; zam5; cham6; cam5
U+4828.0	fCantonese	cim3; zim1
U+4828.0	fMandarin	chan4; ma4
U+4828.0	fEnglish	galloping horse
U+4829.0	fUTF8	䠩
U+4829.0	fCNS11643	4-645A
U+4829.0	fGB	3-6F70
U+4829.0	fR/S	 157.13
U+4829.0	fTotalStrokes	 20
U+4829.0	fCangjie	RMYMH
U+4829.0	fFourCorner	61150
U+4829.0	fCantonese	gwai6; gui6
U+4829.0	fMandarin	gui4
U+4829.0	fEnglish	to urinate; to pass urine; to empty the bladder; to make water; weary, tired, fatigued
U+482A.0	fUTF8	䠪
U+482A.0	fCNS11643	4-6750
U+482A.0	fGB	3-7057
U+482A.0	fR/S	 157.14
U+482A.0	fTotalStrokes	 21
U+482A.0	fCangjie	RMVVI
U+482A.0	fFourCorner	62110
U+482A.0	fCantonese	teon2; dyun6
U+482A.0	fMandarin	duan4
U+482A.0	fEnglish	[1] footprints; track [2] trample [3] walk far and fast
U+482B.0	fUTF8	䠫
U+482B.0	fCNS11643	4-674C
U+482B.0	fGB	3-7056
U+482B.0	fR/S	 157.14
U+482B.0	fTotalStrokes	 21
U+482B.0	fCangjie	RMSEO
U+482B.0	fFourCorner	67180
U+482B.0	fCantonese	zau6
U+482B.0	fMandarin	zou4
U+482B.0	fEnglish	to succumb to the effect of alcohol; to walk unsteadily (said of drunkards)
U+482C.0	fUTF8	䠬
U+482C.0	fCNS11643	5-744B
U+482C.0	fR/S	 157.15
U+482C.0	fTotalStrokes	 22
U+482C.0	fCangjie	RMNTL
U+482C.0	fFourCorner	67120
U+482C.0	fCantonese	dang3
U+482C.0	fMandarin	deng4
U+482C.0	fEnglish	[1] insomnia [2] exhaust [3] highest; farthest
U+482D.0	fUTF8	䠭
U+482D.0	fCNS11643	4-6B24
U+482D.0	fGB	3-7051
U+482D.0	fR/S	 157.16
U+482D.0	fTotalStrokes	 23
U+482D.0	fCangjie	RMDLC
U+482D.0	fFourCorner	67180
U+482D.0	fCantonese	laai6; hon5; lai6; lan5; maan4
U+482D.0	fMandarin	lai4; lai2
U+482D.0	fEnglish	[1] lame; crippled [2] stumble
U+482E.0	fUTF8	䠮
U+482E.0	fCNS11643	7-5C77
U+482E.0	fR/S	 157.16
U+482E.0	fTotalStrokes	 23
U+482E.0	fCangjie	RMFQF
U+482E.0	fFourCorner	69120
U+482E.0	fCantonese	tang4; hau2; taang4
U+482E.0	fMandarin	teng2
U+482E.0	fEnglish	same as &#x9A30;
U+482F.0	fUTF8	䠯
U+482F.0	fCNS11643	4-6C29
U+482F.0	fGB	3-7058
U+482F.0	fR/S	 157.17
U+482F.0	fTotalStrokes	 24
U+482F.0	fCangjie	RMOMB
U+482F.0	fFourCorner	68120
U+482F.0	fCantonese	joek6; zoek6
U+482F.0	fMandarin	yue4
U+482F.0	fEnglish	[1] ascend; climb; rise [2] step on; tread on; walk upon [3] pull out; uproot [4] promote [5] jump; leap
U+4830.0	fUTF8	䠰
U+4830.0	fCNS11643	3-614F
U+4830.0	fGB	5-6F4A
U+4830.0	fR/S	 157.18
U+4830.0	fTotalStrokes	 25
U+4830.0	fCangjie	RMTRG
U+4830.0	fFourCorner	64110
U+4830.0	fHakka	 kian2
U+4830.0	fCantonese	kyun4
U+4830.0	fMandarin	quan2
U+4830.0	fEnglish	same as &#x8E21;
U+4831.0	fUTF8	䠱
U+4831.0	fCNS11643	3-622E
U+4831.0	fGB	5-6F42
U+4831.0	fR/S	 157.21
U+4831.0	fTotalStrokes	 28
U+4831.0	fCangjie	RMSEI
U+4831.0	fFourCorner	67120
U+4831.0	fCantonese	zuk6
U+4831.0	fMandarin	zhu2; shu3
U+4831.0	fEnglish	same as &#x880B;, &#x8E85;
U+4832.0	fUTF8	䠲
U+4832.0	fCNS11643	3-3F42
U+4832.0	fR/S	 158.5
U+4832.0	fTotalStrokes	 12
U+4832.0	fCangjie	HHOII
U+4832.0	fFourCorner	28430
U+4832.0	fCantonese	ling4
U+4832.0	fMandarin	ling2
U+4832.0	fEnglish	non-classical form of &#x8046;
U+4833.0	fUTF8	䠳
U+4833.0	fCNS11643	5-3C67
U+4833.0	fR/S	 158.5
U+4833.0	fTotalStrokes	 12
U+4833.0	fCangjie	HHUU
U+4833.0	fFourCorner	22470
U+4833.0	fCantonese	can1
U+4833.0	fMandarin	chen1
U+4833.0	fEnglish	[1] walk [2] corrupt form of &#x90AA; [3] corrupt form of &#x5C04;
U+4834.0	fUTF8	䠴
U+4834.0	fCNS11643	4-3B52
U+4834.0	fGB	3-714A
U+4834.0	fR/S	 158.5
U+4834.0	fTotalStrokes	 12
U+4834.0	fCangjie	HHYT
U+4834.0	fFourCorner	20410
U+4834.0	fCantonese	zan2
U+4834.0	fMandarin	zhen3; qin3; yin3
U+4834.0	fEnglish	with a symmetric well-formed; upright; respectable; proper carriage
U+4835.0	fUTF8	䠵
U+4835.0	fCNS11643	4-3B53
U+4835.0	fGB	3-7149
U+4835.0	fR/S	 158.5
U+4835.0	fTotalStrokes	 12
U+4835.0	fCangjie	HHODI
U+4835.0	fFourCorner	24400
U+4835.0	fCantonese	fu6; kwaai1; waa1
U+4835.0	fMandarin	fu4
U+4835.0	fEnglish	to wear clothes
U+4836.0	fUTF8	䠶
U+4836.0	fCNS11643	3-3F41
U+4836.0	fR/S	 158.5
U+4836.0	fTotalStrokes	 12
U+4836.0	fCangjie	HHOK
U+4836.0	fFourCorner	28430
U+4836.0	fCantonese	se6; jik6; je6
U+4836.0	fMandarin	she4
U+4836.0	fEnglish	same as &#x5C04;
U+4837.0	fUTF8	䠷
U+4837.0	fCNS11643	3-4471
U+4837.0	fGB	S-2378
U+4837.0	fR/S	 158.6
U+4837.0	fTotalStrokes	 13
U+4837.0	fCangjie	HHLMO
U+4837.0	fFourCorner	22210
U+4837.0	fHakka	 tiau5
U+4837.0	fCantonese	tiu1
U+4837.0	fMandarin	tiao3
U+4837.0	fEnglish	the appearance of height in a person
U+4838.0	fUTF8	䠸
U+4838.0	fCNS11643	3-4470
U+4838.0	fGB	5-6F76
U+4838.0	fR/S	 158.6
U+4838.0	fTotalStrokes	 13
U+4838.0	fCangjie	HHKMS
U+4838.0	fFourCorner	24420
U+4838.0	fCantonese	kwaa1
U+4838.0	fMandarin	kua1
U+4838.0	fEnglish	obsequious; servile; fawning, to obey; obedient; to comply
U+4839.0	fUTF8	䠹
U+4839.0	fCNS11643	3-4472
U+4839.0	fR/S	 158.6
U+4839.0	fTotalStrokes	 13
U+4839.0	fCangjie	HHYVO
U+4839.0	fFourCorner	20480
U+4839.0	fHakka	 hai2
U+4839.0	fCantonese	haai4
U+4839.0	fMandarin	hai2; ai2
U+4839.0	fEnglish	non-classical form of &#x9AB8;
U+483A.0	fUTF8	䠺
U+483A.0	fGB	S-2377
U+483A.0	fR/S	 158.6
U+483A.0	fTotalStrokes	 13
U+483A.0	fCangjie	HHLSR
U+483A.0	fCantonese	man6
U+483B.0	fUTF8	䠻
U+483B.0	fCNS11643	4-4E26
U+483B.0	fGB	5-6F78
U+483B.0	fR/S	 158.8
U+483B.0	fTotalStrokes	 15
U+483B.0	fCangjie	HHJCN
U+483B.0	fFourCorner	23220
U+483B.0	fCantonese	hung3; kung4
U+483B.0	fMandarin	qiong4; qiong2
U+483B.0	fEnglish	[1] bend the body [2] employ as a servant [3] non-classical form of &#x7AC6; [4] same as &#x7AAE;
U+483C.0	fUTF8	䠼
U+483C.0	fCNS11643	4-5432
U+483C.0	fGB	3-7150
U+483C.0	fR/S	 158.9
U+483C.0	fTotalStrokes	 16
U+483C.0	fCangjie	HHOMN
U+483C.0	fFourCorner	28420
U+483C.0	fCantonese	syu3
U+483C.0	fMandarin	shu4; diao4; xue4; zhu2
U+483C.0	fEnglish	to wear; to put on clothes
U+483D.0	fUTF8	䠽
U+483D.0	fCNS11643	4-5930
U+483D.0	fGB	3-714B
U+483D.0	fR/S	 158.10
U+483D.0	fTotalStrokes	 17; 16
U+483D.0	fCangjie	HHUMT
U+483D.0	fFourCorner	22410
U+483D.0	fHakka	 hai2
U+483D.0	fCantonese	haai4
U+483D.0	fMandarin	hai2
U+483D.0	fEnglish	tall and thin
U+483E.0	fUTF8	䠾
U+483E.0	fCNS11643	3-5656
U+483E.0	fR/S	 158.10
U+483E.0	fTotalStrokes	 17
U+483E.0	fCangjie	HHANO
U+483E.0	fFourCorner	27420
U+483E.0	fCantonese	sim2; sim3; sin2
U+483E.0	fMandarin	shan3; shan4
U+483E.0	fEnglish	 dodge; ward off
U+483F.0	fUTF8	䠿
U+483F.0	fCNS11643	3-5B3E
U+483F.0	fR/S	 158.12
U+483F.0	fTotalStrokes	 19
U+483F.0	fCangjie	HHLMC
U+483F.0	fFourCorner	25480
U+483F.0	fCantonese	waai6; kui2
U+483F.0	fMandarin	wai4
U+483F.0	fEnglish	[1] the name of a person [2] non-classical form of &#x8075;
U+4840.0	fUTF8	䡀
U+4840.0	fCNS11643	4-645F
U+4840.0	fGB	3-7153
U+4840.0	fR/S	 158.13
U+4840.0	fTotalStrokes	 20
U+4840.0	fCangjie	HHYWM
U+4840.0	fFourCorner	20410
U+4840.0	fHakka	 zhen5; zen5; shan5; san5; zhan5
U+4840.0	fCantonese	zin2; zin3
U+4840.0	fMandarin	zhan3
U+4840.0	fEnglish	naked; strip
U+4841.0	fUTF8	䡁
U+4841.0	fCNS11643	3-604C
U+4841.0	fR/S	 158.16
U+4841.0	fTotalStrokes	 23
U+4841.0	fCangjie	HHYBP
U+4841.0	fFourCorner	21410
U+4841.0	fCantonese	lung5
U+4841.0	fMandarin	long3
U+4841.0	fEnglish	[1] not well-formed figure [2] incorrect; unrespectable; improper physical build
U+4842.0	fUTF8	䡂
U+4842.0	fCNS11643	4-2B4D
U+4842.0	fGB	3-4535
U+4842.0	fR/S	 159.2
U+4842.0	fTotalStrokes	 9
U+4842.0	fCangjie	JJVL
U+4842.0	fFourCorner	52000
U+4842.0	fCantonese	kau1
U+4842.0	fMandarin	jiu1; jiu4
U+4842.0	fEnglish	the long wooden bumper at the rear of a cart or carriage
U+4843.0	fUTF8	䡃
U+4843.0	fCNS11643	4-2B4E
U+4843.0	fGB	3-4536
U+4843.0	fR/S	 159.2
U+4843.0	fTotalStrokes	 9
U+4843.0	fCangjie	JJKS
U+4843.0	fFourCorner	54020
U+4843.0	fCantonese	leot6; lik6
U+4843.0	fMandarin	li4
U+4843.0	fEnglish	[1] a tool for brushing the thread [2] ornamental [3] brilliant
U+4844.0	fUTF8	䡄
U+4844.0	fCNS11643	F-2D53
U+4844.0	fR/S	 159.2
U+4844.0	fTotalStrokes	 9
U+4844.0	fCangjie	JJHN
U+4844.0	fCantonese	gei1
U+4845.0	fUTF8	䡅
U+4845.0	fCNS11643	3-3422
U+4845.0	fGB	5-475D
U+4845.0	fR/S	 159.3; 162.3
U+4845.0	fTotalStrokes	 10
U+4845.0	fCangjie	JJLLL
U+4845.0	fFourCorner	52000
U+4845.0	fCantonese	ceon1; ceon4
U+4845.0	fMandarin	chun1; min3; xian3; xun2; zhen4; zuan3
U+4845.0	fEnglish	[1] something used to bind the frame work in a cart (compartment) of old times [2] same as &#x8F34; [3] interchangeable with &#x5DE1;
U+4846.0	fUTF8	䡆
U+4846.0	fCNS11643	3-3943
U+4846.0	fR/S	 159.4
U+4846.0	fTotalStrokes	 11
U+4846.0	fCangjie	JJCI
U+4846.0	fFourCorner	58030
U+4846.0	fCantonese	jung4
U+4846.0	fMandarin	yong2; rong2; rong3
U+4846.0	fEnglish	a moving cart
U+4847.0	fUTF8	䡇
U+4847.0	fCNS11643	3-3944
U+4847.0	fGB	5-475E
U+4847.0	fR/S	 159.4
U+4847.0	fTotalStrokes	 11
U+4847.0	fCangjie	JJMMU
U+4847.0	fFourCorner	51010
U+4847.0	fCantonese	jyut6; waat6; zyut6
U+4847.0	fMandarin	yue4
U+4847.0	fEnglish	same as &#x8ECF;
U+4848.0	fUTF8	䡈
U+4848.0	fCNS11643	3-3948
U+4848.0	fGB	5-4761
U+4848.0	fR/S	 159.4
U+4848.0	fTotalStrokes	 11
U+4848.0	fCangjie	JJKK
U+4848.0	fFourCorner	54040
U+4848.0	fHakka	 gau3
U+4848.0	fCantonese	gok3; gau3; hei3
U+4848.0	fMandarin	jiao4; jue2
U+4848.0	fEnglish	[1] the cross-bar at the sides of a carritage [2] same as &#x8F03;
U+4849.0	fUTF8	䡉
U+4849.0	fCNS11643	4-3561
U+4849.0	fGB	3-4543
U+4849.0	fR/S	 159.4
U+4849.0	fTotalStrokes	 11
U+4849.0	fCangjie	JJYHN
U+4849.0	fFourCorner	50010
U+4849.0	fCantonese	hong2
U+4849.0	fMandarin	kang3
U+4849.0	fEnglish	a kind of cart
U+484A.0	fUTF8	䡊
U+484A.0	fCNS11643	4-3562
U+484A.0	fGB	3-4540
U+484A.0	fR/S	 159.4
U+484A.0	fTotalStrokes	 11
U+484A.0	fCangjie	JJHE
U+484A.0	fFourCorner	51047
U+484A.0	fCantonese	faan2
U+484A.0	fMandarin	fan3; fan2; pei4
U+484A.0	fEnglish	the protruded sides of a carriage (used in ancient times to screen off muddy dust)
U+484B.0	fUTF8	䡋
U+484B.0	fCNS11643	3-3945
U+484B.0	fR/S	 159.4
U+484B.0	fTotalStrokes	 11
U+484B.0	fCangjie	JJJE
U+484B.0	fFourCorner	54040
U+484B.0	fCantonese	kei4; gei1; zik1
U+484B.0	fMandarin	qi2
U+484B.0	fEnglish	same as &#x8EDD;
U+484C.0	fUTF8	䡌
U+484C.0	fCNS11643	3-3946
U+484C.0	fR/S	 159.4
U+484C.0	fTotalStrokes	 11
U+484C.0	fCangjie	JJKI
U+484C.0	fFourCorner	54030
U+484C.0	fHakka	 fen2
U+484C.0	fCantonese	wang4
U+484C.0	fMandarin	hong2
U+484C.0	fEnglish	same as &#x9783;
U+484D.0	fUTF8	䡍
U+484D.0	fCNS11643	3-3942
U+484D.0	fR/S	 159.4
U+484D.0	fTotalStrokes	 11
U+484D.0	fCangjie	JJQO
U+484D.0	fFourCorner	55030
U+484D.0	fCantonese	fu4; bak6
U+484D.0	fMandarin	fu2; fu3
U+484D.0	fEnglish	the linch-pin of a sheel
U+484E.0	fUTF8	䡎
U+484E.0	fCNS11643	6-4545
U+484E.0	fR/S	 159.4
U+484E.0	fTotalStrokes	 11
U+484E.0	fCangjie	JJHS
U+484E.0	fFourCorner	53000
U+484E.0	fCantonese	lou4
U+484E.0	fMandarin	lu2
U+484E.0	fEnglish	non-classical form of &#x8F64;
U+484F.0	fUTF8	䡏
U+484F.0	fCNS11643	3-3F48
U+484F.0	fR/S	 159.5
U+484F.0	fTotalStrokes	 12
U+484F.0	fCangjie	JJNI
U+484F.0	fFourCorner	52030
U+484F.0	fHakka	 fen2
U+484F.0	fCantonese	wang4; waang4; wui4; zyun1
U+484F.0	fMandarin	kong1; hong2
U+484F.0	fEnglish	same as &#x9783;
U+4850.0	fUTF8	䡐
U+4850.0	fCNS11643	3-3F43
U+4850.0	fR/S	 159.5
U+4850.0	fTotalStrokes	 12
U+4850.0	fCangjie	JJJP
U+4850.0	fFourCorner	53010
U+4850.0	fCantonese	to4
U+4850.0	fMandarin	tuo2
U+4850.0	fEnglish	 move swiftly; dart; fleet; drive very fast
U+4851.0	fUTF8	䡑
U+4851.0	fCNS11643	4-3B54
U+4851.0	fGB	3-454D
U+4851.0	fR/S	 159.5
U+4851.0	fTotalStrokes	 12
U+4851.0	fCangjie	JJRVP
U+4851.0	fFourCorner	57040
U+4851.0	fCantonese	man4
U+4851.0	fMandarin	min2
U+4851.0	fEnglish	[1] the rim or felloe of a wheel; tire [2] yoke; collar
U+4852.0	fUTF8	䡒
U+4852.0	fCNS11643	3-3F4A
U+4852.0	fR/S	 159.5
U+4852.0	fTotalStrokes	 12
U+4852.0	fCangjie	JJW
U+4852.0	fFourCorner	56000
U+4852.0	fCantonese	tin4
U+4852.0	fMandarin	tian2
U+4852.0	fEnglish	the sound of moving carriages; noise of a number of vehicles
U+4853.0	fUTF8	䡓
U+4853.0	fCNS11643	3-4475
U+4853.0	fR/S	 159.6
U+4853.0	fTotalStrokes	 13
U+4853.0	fCangjie	HOJJN
U+4853.0	fFourCorner	21500
U+4853.0	fCantonese	gyun3; hin1
U+4853.0	fMandarin	juan4
U+4853.0	fEnglish	[1] shake of a cart [2] awning in front of a cart, covering the horse or mule in the shafts [3] same as &#x8ED2;
U+4854.0	fUTF8	䡔
U+4854.0	fCNS11643	4-412A
U+4854.0	fGB	3-4555
U+4854.0	fR/S	 159.6
U+4854.0	fTotalStrokes	 13
U+4854.0	fCangjie	JJNIN
U+4854.0	fFourCorner	57020
U+4854.0	fCantonese	kai2
U+4854.0	fMandarin	qi3
U+4854.0	fEnglish	to hinder; to impede; to interfere with, to obstruct
U+4855.0	fUTF8	䡕
U+4855.0	fCNS11643	3-4474
U+4855.0	fR/S	 159.6
U+4855.0	fTotalStrokes	 13
U+4855.0	fCangjie	JJNEM
U+4855.0	fFourCorner	57010
U+4855.0	fCantonese	sing4; cing2
U+4855.0	fMandarin	zheng3
U+4855.0	fEnglish	the rear platform of a light carriage, a subordinate cart
U+4856.0	fUTF8	䡖
U+4856.0	fCNS11643	6-5924
U+4856.0	fR/S	 159.6
U+4856.0	fTotalStrokes	 13
U+4856.0	fCangjie	JJMCG
U+4856.0	fFourCorner	51010
U+4856.0	fCantonese	hing3
U+4856.0	fMandarin	qing1; jing4
U+4856.0	fEnglish	still, motionless
U+4857.0	fUTF8	䡗
U+4857.0	fCNS11643	4-412C
U+4857.0	fGB	3-4550
U+4857.0	fR/S	 159.6
U+4857.0	fTotalStrokes	 13
U+4857.0	fCangjie	MNJWJ
U+4857.0	fFourCorner	17500
U+4857.0	fCantonese	gung2
U+4857.0	fMandarin	gong3
U+4857.0	fEnglish	[1] rim of a wheel; felly; felloe [2] run over (of a wheel) [3] oppress [4] the shafts of a cart or carriage
U+4858.0	fUTF8	䡘
U+4858.0	fCNS11643	4-4766
U+4858.0	fGB	3-455A
U+4858.0	fR/S	 159.7
U+4858.0	fTotalStrokes	 14
U+4858.0	fCangjie	JJPW
U+4858.0	fFourCorner	57020
U+4858.0	fCantonese	tin4
U+4858.0	fMandarin	tian2
U+4858.0	fEnglish	the sound of moving carriages
U+4859.0	fUTF8	䡙
U+4859.0	fCNS11643	4-4769
U+4859.0	fGB	3-455B
U+4859.0	fR/S	 159.7
U+4859.0	fTotalStrokes	 14
U+4859.0	fCangjie	JJIAV
U+4859.0	fFourCorner	53030
U+4859.0	fCantonese	long4
U+4859.0	fMandarin	liang2; lang2
U+4859.0	fEnglish	 armed carriage; military cart; cart used by the soldiers
U+485A.0	fUTF8	䡚
U+485A.0	fCNS11643	4-4767
U+485A.0	fGB	3-4559
U+485A.0	fR/S	 159.7
U+485A.0	fTotalStrokes	 14
U+485A.0	fCangjie	JJHAU
U+485A.0	fFourCorner	56010
U+485A.0	fCantonese	maau6
U+485A.0	fMandarin	mao4
U+485A.0	fEnglish	implements; equipment to catch the birds and animals, to entice; to tempt, to exchange; to trade
U+485B.0	fUTF8	䡛
U+485B.0	fCNS11643	3-497E
U+485B.0	fR/S	 159.7
U+485B.0	fTotalStrokes	 14
U+485B.0	fCangjie	JJJWJ
U+485B.0	fFourCorner	55000
U+485B.0	fCantonese	jan6
U+485B.0	fMandarin	yi4; yin4
U+485B.0	fEnglish	the name of a carriage
U+485C.0	fUTF8	䡜
U+485C.0	fCNS11643	4-4E28
U+485C.0	fGB	3-455F
U+485C.0	fR/S	 159.8
U+485C.0	fTotalStrokes	 15
U+485C.0	fCangjie	JJGCG
U+485C.0	fFourCorner	54010
U+485C.0	fCantonese	luk6
U+485C.0	fMandarin	lu4
U+485C.0	fEnglish	[1] car (of a train); compartment; the inside of a cart [2] axis; pivot; axle; axletree
U+485D.0	fUTF8	䡝
U+485D.0	fCNS11643	4-4E27
U+485D.0	fGB	3-456C
U+485D.0	fR/S	 159.8
U+485D.0	fTotalStrokes	 15
U+485D.0	fCangjie	JJJNU
U+485D.0	fFourCorner	53010
U+485D.0	fCantonese	jyun1
U+485D.0	fMandarin	yuan1; yun3
U+485D.0	fEnglish	a rear covering of a big carriage, an armed carriage; a military cart; cart used by the soldiers, name of a carriage
U+485E.0	fUTF8	䡞
U+485E.0	fCNS11643	4-4E2A
U+485E.0	fGB	3-4563
U+485E.0	fR/S	 159.8
U+485E.0	fTotalStrokes	 15
U+485E.0	fCangjie	BUKJJ
U+485E.0	fFourCorner	60500
U+485E.0	fCantonese	guk1
U+485E.0	fMandarin	ju2
U+485E.0	fEnglish	[1] leather that wind round on the shafts of a big carriage [2] interchangeable with &#x6336;
U+485F.0	fUTF8	䡟
U+485F.0	fCNS11643	4-4E29
U+485F.0	fGB	3-4566
U+485F.0	fR/S	 159.8
U+485F.0	fTotalStrokes	 15
U+485F.0	fCangjie	JJHWJ
U+485F.0	fFourCorner	56040
U+485F.0	fCantonese	pai3; bei6; fu6; pei3
U+485F.0	fMandarin	pi4
U+485F.0	fEnglish	a ring of the horizontal front bar on a carriage to control the wheel
U+4860.0	fUTF8	䡠
U+4860.0	fCNS11643	3-5323
U+4860.0	fR/S	 159.9
U+4860.0	fTotalStrokes	 16
U+4860.0	fCangjie	JJYBC
U+4860.0	fFourCorner	51080
U+4860.0	fCantonese	hang1
U+4860.0	fMandarin	keng1
U+4860.0	fEnglish	strong and durable
U+4861.0	fUTF8	䡡
U+4861.0	fCNS11643	5-5872
U+4861.0	fR/S	 159.9
U+4861.0	fTotalStrokes	 16
U+4861.0	fCangjie	JJPPA
U+4861.0	fFourCorner	51060
U+4861.0	fCantonese	haai4; gai1; ham4; ham6
U+4861.0	fMandarin	xie2
U+4861.0	fEnglish	 get up into a carriage
U+4862.0	fUTF8	䡢
U+4862.0	fCNS11643	4-5433
U+4862.0	fGB	3-4574
U+4862.0	fR/S	 159.9
U+4862.0	fTotalStrokes	 16
U+4862.0	fCangjie	JJISB
U+4862.0	fFourCorner	53027
U+4862.0	fCantonese	bin6; pin3
U+4862.0	fMandarin	bian3; bian4
U+4862.0	fEnglish	 small carriage
U+4863.0	fUTF8	䡣
U+4863.0	fCNS11643	4-5434
U+4863.0	fGB	3-4558
U+4863.0	fR/S	 159.9
U+4863.0	fTotalStrokes	 16
U+4863.0	fCangjie	JJBJJ
U+4863.0	fFourCorner	57050
U+4863.0	fCantonese	wan4
U+4863.0	fMandarin	hun2; xuan1
U+4863.0	fEnglish	a piece of curved board; a crank -- of a ( plough) machine, etc., shafts (of a cart or carriage), to return, to avoid bumping against each other
U+4864.0	fUTF8	䡤
U+4864.0	fCNS11643	5-5161
U+4864.0	fGB	3-455E
U+4864.0	fR/S	 159.9; 159.8
U+4864.0	fTotalStrokes	 15; 16
U+4864.0	fCangjie	JAJWJ
U+4864.0	fFourCorner	44500
U+4864.0	fCantonese	zyu1; zeoi2
U+4864.0	fMandarin	zhu1
U+4864.0	fEnglish	[1] carriage [2] an evergreen oak
U+4865.0	fUTF8	䡥
U+4865.0	fCNS11643	3-5658
U+4865.0	fR/S	 159.10
U+4865.0	fTotalStrokes	 17
U+4865.0	fCangjie	JJJCR
U+4865.0	fFourCorner	53060
U+4865.0	fCantonese	jung4; zung2; zung4
U+4865.0	fMandarin	yong2; rong2
U+4865.0	fEnglish	same as &#x4846;
U+4866.0	fUTF8	䡦
U+4866.0	fCNS11643	4-5932
U+4866.0	fGB	3-457A
U+4866.0	fR/S	 159.10
U+4866.0	fTotalStrokes	 17; 16
U+4866.0	fCangjie	JJEED
U+4866.0	fFourCorner	57090
U+4866.0	fCantonese	song2
U+4866.0	fMandarin	sang3
U+4866.0	fEnglish	[1] the empty space of a wheel [2] rim of a wheel; felly; felloe
U+4867.0	fUTF8	䡧
U+4867.0	fCNS11643	4-5935
U+4867.0	fGB	3-4542
U+4867.0	fR/S	 159.10
U+4867.0	fTotalStrokes	 17; 16
U+4867.0	fCangjie	JJHRF
U+4867.0	fFourCorner	57020
U+4867.0	fCantonese	wu1
U+4867.0	fMandarin	wu3; wu1
U+4867.0	fEnglish	 carriage; the head of a carriage
U+4868.0	fUTF8	䡨
U+4868.0	fCNS11643	4-5937
U+4868.0	fGB	3-4572
U+4868.0	fR/S	 159.10
U+4868.0	fTotalStrokes	 16; 17
U+4868.0	fCangjie	TMJWJ
U+4868.0	fFourCorner	80500
U+4868.0	fCantonese	caai4
U+4868.0	fMandarin	cha4; chai2; yin2
U+4868.0	fEnglish	connected carts, back and park the cart, to block; to stop up
U+4869.0	fUTF8	䡩
U+4869.0	fCNS11643	3-5657
U+4869.0	fR/S	 159.10
U+4869.0	fTotalStrokes	 17
U+4869.0	fCangjie	JJJBC
U+4869.0	fFourCorner	54080
U+4869.0	fCantonese	hang1; haam1; haang1
U+4869.0	fMandarin	keng1; gu3; hou4; tou3; zhen3
U+4869.0	fEnglish	[1] the sound of carts or stones (big things) knocking together [2] cross-bar in the rear of a carriage
U+486A.0	fUTF8	䡪
U+486A.0	fCNS11643	4-5931
U+486A.0	fGB	3-4579
U+486A.0	fR/S	 159.10
U+486A.0	fTotalStrokes	 17; 16
U+486A.0	fCangjie	JJISM
U+486A.0	fFourCorner	53027
U+486A.0	fCantonese	sin3; ceoi1; zeoi1
U+486A.0	fMandarin	shan4; chan2
U+486A.0	fEnglish	fan of a carriage
U+486B.0	fUTF8	䡫
U+486B.0	fCNS11643	4-5D7C
U+486B.0	fGB	3-4577
U+486B.0	fR/S	 159.11
U+486B.0	fTotalStrokes	 18
U+486B.0	fCangjie	JJYHJ
U+486B.0	fFourCorner	57030
U+486B.0	fCantonese	pung4
U+486B.0	fMandarin	peng2
U+486B.0	fEnglish	sound of moving carriages; noise of a number of vehicles
U+486C.0	fUTF8	䡬
U+486C.0	fCNS11643	4-5D7B
U+486C.0	fGB	3-457D
U+486C.0	fR/S	 159.11
U+486C.0	fTotalStrokes	 18
U+486C.0	fCangjie	JJAWE
U+486C.0	fFourCorner	56040
U+486C.0	fCantonese	maan6
U+486C.0	fMandarin	wan4; man4
U+486C.0	fEnglish	[1] covering of a carriage [2] tank (an armored vehicle) [3] chariot [4] cover or shelter of a chariot
U+486D.0	fUTF8	䡭
U+486D.0	fCNS11643	4-5D7A
U+486D.0	fGB	3-4578
U+486D.0	fR/S	 159.11
U+486D.0	fTotalStrokes	 18
U+486D.0	fCangjie	JJTQG
U+486D.0	fFourCorner	58010
U+486D.0	fCantonese	sau1; saau1; saau2
U+486D.0	fMandarin	xiu1
U+486D.0	fEnglish	 hearse; funeral carriage
U+486E.0	fUTF8	䡮
U+486E.0	fCNS11643	5-6535
U+486E.0	fGB	5-4760
U+486E.0	fR/S	 159.11
U+486E.0	fTotalStrokes	 18
U+486E.0	fCangjie	JJHOO
U+486E.0	fFourCorner	58080
U+486E.0	fCantonese	zung1
U+486E.0	fMandarin	zong1
U+486E.0	fEnglish	interchangeable with &#x8E64;, &#x8DBF;
U+486F.0	fUTF8	䡯
U+486F.0	fCNS11643	4-5D7D
U+486F.0	fGB	3-457E
U+486F.0	fR/S	 159.11
U+486F.0	fTotalStrokes	 18
U+486F.0	fCangjie	JJHWP
U+486F.0	fFourCorner	56030
U+486F.0	fCantonese	cung1; zung2
U+486F.0	fMandarin	cong1; shuang4; zong3
U+486F.0	fEnglish	[1] the sound of moving carriages; noise of a number of vehicles [2] prison cart; police van to transport prisoners [3] carts wheels
U+4870.0	fUTF8	䡰
U+4870.0	fCNS11643	4-5D7E
U+4870.0	fGB	3-457B
U+4870.0	fR/S	 159.11
U+4870.0	fTotalStrokes	 18
U+4870.0	fCangjie	GEJWJ
U+4870.0	fFourCorner	47500
U+4870.0	fCantonese	hang1; fok3; guk1; huk6; keoi4
U+4870.0	fMandarin	keng1; keng3
U+4870.0	fEnglish	[1] strong and durable [2] same as &#x8F42;
U+4871.0	fUTF8	䡱
U+4871.0	fR/S	 159.11
U+4871.0	fTotalStrokes	 18
U+4871.0	fCangjie	JJSRR
U+4871.0	fCantonese	zyun2
U+4871.0	fMandarin	zhuan3
U+4872.0	fUTF8	䡲
U+4872.0	fCNS11643	4-6159
U+4872.0	fGB	3-456A
U+4872.0	fR/S	 159.12
U+4872.0	fTotalStrokes	 19
U+4872.0	fCangjie	JJRRJ
U+4872.0	fFourCorner	56050
U+4872.0	fCantonese	saan4; dan1
U+4872.0	fMandarin	chan2
U+4872.0	fEnglish	[1] rim of a wheel; felly; felloe [2] a kind of carriage
U+4873.0	fUTF8	䡳
U+4873.0	fCNS11643	4-6158
U+4873.0	fGB	3-4622
U+4873.0	fR/S	 159.12
U+4873.0	fTotalStrokes	 19
U+4873.0	fCangjie	JJTCL
U+4873.0	fFourCorner	52020
U+4873.0	fCantonese	si1
U+4873.0	fMandarin	si1
U+4873.0	fEnglish	cart, carriage etc.
U+4874.0	fUTF8	䡴
U+4874.0	fCNS11643	4-6157
U+4874.0	fGB	3-4626
U+4874.0	fR/S	 159.12
U+4874.0	fTotalStrokes	 19
U+4874.0	fCangjie	JJYTG
U+4874.0	fFourCorner	50010
U+4874.0	fHakka	 chung1; cung1
U+4874.0	fCantonese	cong4; cung1
U+4874.0	fMandarin	chong1; chuang2
U+4874.0	fEnglish	war-chariot
U+4875.0	fUTF8	䡵
U+4875.0	fCNS11643	4-6460
U+4875.0	fGB	3-4627
U+4875.0	fR/S	 159.12; 159.13
U+4875.0	fTotalStrokes	 20; 19
U+4875.0	fCangjie	JJYTO
U+4875.0	fFourCorner	58030
U+4875.0	fCantonese	seoi6
U+4875.0	fMandarin	sui4
U+4875.0	fEnglish	decoration on carriage
U+4876.0	fUTF8	䡶
U+4876.0	fCNS11643	4-6461
U+4876.0	fGB	5-477B
U+4876.0	fR/S	 159.13
U+4876.0	fTotalStrokes	 20
U+4876.0	fCangjie	JJSRJ
U+4876.0	fFourCorner	50040
U+4876.0	fCantonese	pik1; pei1; pei3
U+4876.0	fMandarin	pi4; bei4
U+4876.0	fEnglish	[1] vehicles and garments (commonloy used in feudal times to reward vassals) [2] covering on the crossbar at the ends of carriage poles [3] crank axle of a shade
U+4877.0	fUTF8	䡷
U+4877.0	fCNS11643	4-6752
U+4877.0	fGB	3-4629
U+4877.0	fR/S	 159.14
U+4877.0	fTotalStrokes	 21
U+4877.0	fCangjie	JJTGT
U+4877.0	fFourCorner	54010
U+4877.0	fCantonese	hat6
U+4877.0	fMandarin	kai1; kai4; ke1
U+4877.0	fEnglish	same as &#x8F44;
U+4878.0	fUTF8	䡸
U+4878.0	fCNS11643	F-6853
U+4878.0	fGB	S-235B
U+4878.0	fR/S	 159.14
U+4878.0	fTotalStrokes	 21
U+4878.0	fCangjie	JJTWV
U+4878.0	fCantonese	mit6
U+4879.0	fUTF8	䡹
U+4879.0	fCNS11643	3-5F68
U+4879.0	fGB	5-477C
U+4879.0	fR/S	 159.14
U+4879.0	fTotalStrokes	 22; 21
U+4879.0	fCangjie	JJJBO
U+4879.0	fFourCorner	54080
U+4879.0	fCantonese	zaap1; zi3; zik1
U+4879.0	fMandarin	zhi4
U+4879.0	fEnglish	same as &#x8F0A;
U+487A.0	fUTF8	䡺
U+487A.0	fCNS11643	3-5F67
U+487A.0	fR/S	 159.15
U+487A.0	fTotalStrokes	 22
U+487A.0	fCangjie	JJQJP
U+487A.0	fFourCorner	55030
U+487A.0	fCantonese	wai6
U+487A.0	fMandarin	wei4
U+487A.0	fEnglish	same as &#x8F4A;
U+487B.0	fUTF8	䡻
U+487B.0	fGB	3-462E
U+487B.0	fR/S	 159.15
U+487B.0	fTotalStrokes	 22
U+487B.0	fCangjie	JJHUS
U+487B.0	fCantonese	min4; maan4
U+487B.0	fMandarin	min2
U+487B.0	fEnglish	[1] same as &#x4851; [2] same as &#x8ED2;
U+487C.0	fUTF8	䡼
U+487C.0	fCNS11643	3-607D
U+487C.0	fR/S	 159.17
U+487C.0	fTotalStrokes	 24
U+487C.0	fCangjie	JJMBR
U+487C.0	fFourCorner	51060
U+487C.0	fCantonese	ling4
U+487C.0	fMandarin	ling2
U+487C.0	fEnglish	same as &#x8EE8;
U+487D.0	fUTF8	䡽
U+487D.0	fCNS11643	4-6D61
U+487D.0	fGB	3-462F
U+487D.0	fR/S	 159.19
U+487D.0	fTotalStrokes	 26
U+487D.0	fCangjie	JJHUC
U+487D.0	fFourCorner	54080
U+487D.0	fCantonese	zyun1
U+487D.0	fMandarin	zuan1
U+487D.0	fEnglish	the shafts of a vehicle
U+487E.0	fUTF8	䡾
U+487E.0	fCNS11643	4-6E21
U+487E.0	fGB	3-462A
U+487E.0	fR/S	 159.20
U+487E.0	fTotalStrokes	 27
U+487E.0	fCangjie	JJYBK
U+487E.0	fFourCorner	53030
U+487E.0	fCantonese	jit6; ngaai5
U+487E.0	fMandarin	nie4; nei4
U+487E.0	fEnglish	[1] high cartloads [2] same as &#x8F59;
U+487F.0	fUTF8	䡿
U+487F.0	fCNS11643	3-623D
U+487F.0	fR/S	 159.24
U+487F.0	fTotalStrokes	 31
U+487F.0	fCangjie	JJMBO
U+487F.0	fFourCorner	51010
U+487F.0	fCantonese	ling4
U+487F.0	fMandarin	ling2
U+487F.0	fEnglish	same as &#x8EE8;
U+4880.0	fUTF8	䢀
U+4880.0	fGB	7-2372
U+4880.0	fR/S	 159.3
U+4880.0	fTotalStrokes	 10
U+4880.0	fCangjie	KQON
U+4880.0	fCantonese	ngat6; at6 (also); gaat1
U+4880.0	fMandarin	qi4
U+4881.0	fUTF8	䢁
U+4881.0	fGB	7-2373
U+4881.0	fR/S	 159.4
U+4881.0	fTotalStrokes	 11
U+4881.0	fCangjie	KQB
U+4881.0	fCantonese	jyut6; zyut6
U+4881.0	fMandarin	yue4
U+4881.0	fEnglish	tire of a wheel
U+4882.0	fUTF8	䢂
U+4882.0	fGB	S-235A
U+4882.0	fR/S	 159.5
U+4882.0	fTotalStrokes	 9
U+4882.0	fCangjie	KQYT
U+4882.0	fCantonese	lip1
U+4882.0	fEnglish	elevator
U+4883.0	fUTF8	䢃
U+4883.0	fCNS11643	4-4E2E
U+4883.0	fGB	5-7330
U+4883.0	fR/S	 160.8
U+4883.0	fTotalStrokes	 15
U+4883.0	fCangjie	SJK
U+4883.0	fFourCorner	70400
U+4883.0	fCantonese	ngaai6; aai6 (also); ngai6
U+4883.0	fMandarin	yi4
U+4883.0	fEnglish	[1] administer; regulate; manage; govern [2] people to have both talent and virtue [3] quiet; calm; safe; secure; stable
U+4884.0	fUTF8	䢄
U+4884.0	fCNS11643	3-5D39
U+4884.0	fR/S	 160.13
U+4884.0	fTotalStrokes	 20
U+4884.0	fCangjie	IJDD
U+4884.0	fFourCorner	20990
U+4884.0	fCantonese	saai2
U+4884.0	fMandarin	xi3
U+4885.0	fUTF8	䢅
U+4885.0	fCNS11643	4-476B
U+4885.0	fGB	5-645D
U+4885.0	fR/S	 161.6; 161.7
U+4885.0	fTotalStrokes	 14; 13
U+4885.0	fCangjie	HXMMV
U+4885.0	fFourCorner	77230
U+4885.0	fCantonese	san4
U+4885.0	fMandarin	chen2
U+4885.0	fEnglish	interchangeable with &#x6668;
U+4886.0	fUTF8	䢆
U+4886.0	fCNS11643	F-4C67
U+4886.0	fGB	5-3376
U+4886.0	fR/S	 161.7
U+4886.0	fTotalStrokes	 14
U+4886.0	fCangjie	BNMVI
U+4886.0	fCantonese	jung2
U+4887.0	fUTF8	䢇
U+4887.0	fCNS11643	3-4F38
U+4887.0	fR/S	 161.8
U+4887.0	fTotalStrokes	 15
U+4887.0	fCangjie	JUMVI
U+4887.0	fFourCorner	31243
U+4887.0	fCantonese	jung5; zung2
U+4887.0	fMandarin	rong3; rou3
U+4887.0	fEnglish	[1] a son who is not so good as his father; good-for-nothing; a rotten apple [2] inferior; mean [3] busy with many different matters; no leisure [4] mixed; confused; miscellaneous
U+4888.0	fUTF8	䢈
U+4888.0	fCNS11643	4-6463
U+4888.0	fGB	5-6E3B
U+4888.0	fR/S	 161.13
U+4888.0	fTotalStrokes	 20
U+4888.0	fCangjie	MVOMA
U+4888.0	fFourCorner	71600
U+4888.0	fCantonese	wui6; saan4
U+4888.0	fMandarin	hui4; chen2; qin2
U+4889.0	fUTF8	䢉
U+4889.0	fCNS11643	7-5056
U+4889.0	fR/S	 161.13
U+4889.0	fTotalStrokes	 20
U+4889.0	fCangjie	HXMMV
U+4889.0	fFourCorner	77230
U+4889.0	fCantonese	nung4; lung4 (also)
U+4889.0	fMandarin	nong2
U+4889.0	fEnglish	same as &#x8FB2;
U+488A.0	fUTF8	䢊
U+488A.0	fCNS11643	3-2732
U+488A.0	fR/S	 162.3
U+488A.0	fTotalStrokes	 7
U+488A.0	fCangjie	YND
U+488A.0	fFourCorner	37300
U+488A.0	fCantonese	jau4; zaau4
U+488A.0	fMandarin	you2
U+488A.0	fEnglish	ancient form of &#x904A;
U+488B.0	fUTF8	䢋
U+488B.0	fCNS11643	4-2522
U+488B.0	fGB	5-6F7D
U+488B.0	fR/S	 162.3
U+488B.0	fTotalStrokes	 7
U+488B.0	fCangjie	YML
U+488B.0	fFourCorner	31300
U+488B.0	fCantonese	gei3; gau2; gei6
U+488B.0	fMandarin	ji4
U+488B.0	fEnglish	[1] marshal or herald; an offical rank in old times, indicating exclamation [2] same as &#x54C9;
U+488C.0	fUTF8	䢌
U+488C.0	fCNS11643	F-2872
U+488C.0	fGB	5-7023
U+488C.0	fR/S	 162.4
U+488C.0	fTotalStrokes	 8
U+488C.0	fCangjie	YJB
U+488C.0	fFourCorner	3530
U+488C.0	fCantonese	but3
U+488C.0	fMandarin	bo2
U+488C.0	fEnglish	to walk, rapid marching or running
U+488D.0	fUTF8	䢍
U+488D.0	fCNS11643	3-2A6F
U+488D.0	fR/S	 162.4
U+488D.0	fTotalStrokes	 8
U+488D.0	fCangjie	YYHS
U+488D.0	fFourCorner	30300
U+488D.0	fHakka	 fang5; fang3
U+488D.0	fCantonese	fong6; fong2
U+488D.0	fMandarin	fang3; fen4
U+488D.0	fEnglish	 hasten; hurry to
U+488E.0	fUTF8	䢎
U+488E.0	fCNS11643	5-2769
U+488E.0	fR/S	 162.4
U+488E.0	fTotalStrokes	 8
U+488E.0	fCangjie	YMT
U+488E.0	fFourCorner	31300
U+488E.0	fCantonese	gei3; gei1
U+488E.0	fMandarin	ji3
U+488E.0	fEnglish	same as &#x488B;
U+488F.0	fUTF8	䢏
U+488F.0	fR/S	 162.4
U+488F.0	fTotalStrokes	 8
U+488F.0	fCangjie	YYJ
U+488F.0	fCantonese	dau2
U+488F.0	fEnglish	[Korean character]
U+4890.0	fUTF8	䢐
U+4890.0	fCNS11643	4-2B53
U+4890.0	fGB	5-7029
U+4890.0	fR/S	 162.5
U+4890.0	fTotalStrokes	 9
U+4890.0	fCangjie	YBM
U+4890.0	fFourCorner	37300
U+4890.0	fCantonese	cou4; cuk6
U+4890.0	fMandarin	cu2
U+4890.0	fEnglish	same as &#x5F82;
U+4891.0	fUTF8	䢑
U+4891.0	fCNS11643	4-2B50
U+4891.0	fGB	3-715B
U+4891.0	fR/S	 162.5
U+4891.0	fTotalStrokes	 9
U+4891.0	fCangjie	YHVI
U+4891.0	fFourCorner	32300
U+4891.0	fCantonese	dai2
U+4891.0	fMandarin	di3
U+4891.0	fEnglish	angry; furious, frightened; terrified; terror-stricken
U+4892.0	fUTF8	䢒
U+4892.0	fCNS11643	3-3423
U+4892.0	fGB	5-702F
U+4892.0	fR/S	 162.6
U+4892.0	fTotalStrokes	 10
U+4892.0	fCangjie	YYCK
U+4892.0	fFourCorner	30300
U+4892.0	fCantonese	gaau1
U+4892.0	fMandarin	jiao1
U+4892.0	fEnglish	[1] meet; co-operate; interaction [2] have friendly relations [3] interlocking; interchanging
U+4893.0	fUTF8	䢓
U+4893.0	fCNS11643	4-302D
U+4893.0	fGB	3-7161
U+4893.0	fR/S	 162.6
U+4893.0	fTotalStrokes	 10
U+4893.0	fCangjie	YJMD
U+4893.0	fFourCorner	33300
U+4893.0	fCantonese	jyu4; zyu1
U+4893.0	fMandarin	yu2
U+4893.0	fEnglish	[1] bed [2] window
U+4894.0	fUTF8	䢔
U+4894.0	fCNS11643	4-3030
U+4894.0	fGB	3-715F
U+4894.0	fR/S	 162.6
U+4894.0	fTotalStrokes	 10
U+4894.0	fCangjie	YOMR
U+4894.0	fFourCorner	38300
U+4894.0	fCantonese	hap6
U+4894.0	fMandarin	he2; ge2; jia2
U+4894.0	fEnglish	[1] mixed; abundant; assorted [2] repeated
U+4895.0	fUTF8	䢕
U+4895.0	fCNS11643	3-3424
U+4895.0	fR/S	 162.6
U+4895.0	fTotalStrokes	 10
U+4895.0	fCangjie	YIHM
U+4895.0	fFourCorner	33300
U+4895.0	fCantonese	wat1; aau1; zuk1; zung2
U+4895.0	fMandarin	xu4
U+4895.0	fEnglish	a group of people walking together
U+4896.0	fUTF8	䢖
U+4896.0	fCNS11643	4-302E
U+4896.0	fGB	3-7162
U+4896.0	fR/S	 162.6
U+4896.0	fTotalStrokes	 10
U+4896.0	fCangjie	YLQ
U+4896.0	fFourCorner	35300
U+4896.0	fCantonese	jyut6; gwaai3; leot6; waat6
U+4896.0	fMandarin	yu4; l4
U+4896.0	fEnglish	[1] be scattered (over an area); spread [2] give an account of; explain; expound [3] to follow
U+4897.0	fUTF8	䢗
U+4897.0	fR/S	 162.6
U+4897.0	fTotalStrokes	 10
U+4897.0	fCangjie	YTW
U+4897.0	fFourCorner	3530
U+4897.0	fCantonese	kuk1
U+4897.0	fMandarin	he2; qu3
U+4898.0	fUTF8	䢘
U+4898.0	fR/S	 162.6
U+4898.0	fTotalStrokes	 10
U+4898.0	fCangjie	YJDI
U+4898.0	fCantonese	sau2
U+4898.0	fEnglish	[Korean character]
U+4899.0	fUTF8	䢙
U+4899.0	fCNS11643	3-3949
U+4899.0	fGB	5-7024
U+4899.0	fR/S	 162.7
U+4899.0	fTotalStrokes	 11
U+4899.0	fCangjie	YBUC
U+4899.0	fFourCorner	37300
U+4899.0	fHakka	 pai5; pai6
U+4899.0	fCantonese	baai6; bai6; bak3
U+4899.0	fMandarin	bai4
U+4899.0	fEnglish	 be defeated; destroy; ruin
U+489A.0	fUTF8	䢚
U+489A.0	fCNS11643	4-3569
U+489A.0	fGB	3-7163
U+489A.0	fR/S	 162.7
U+489A.0	fTotalStrokes	 11
U+489A.0	fCangjie	YMLK
U+489A.0	fFourCorner	31300
U+489A.0	fCantonese	gang1
U+489A.0	fMandarin	geng1; gong4; hang2
U+489A.0	fEnglish	[1] a narrow path (for rabbit) [2] same as &#x8FD2;
U+489B.0	fUTF8	䢛
U+489B.0	fCNS11643	3-394D
U+489B.0	fR/S	 162.7
U+489B.0	fTotalStrokes	 11
U+489B.0	fCangjie	YBCR
U+489B.0	fFourCorner	37300
U+489B.0	fCantonese	gwing2
U+489B.0	fMandarin	jiong1; jiong3
U+489B.0	fEnglish	non-classical form of &#x8FE5;
U+489C.0	fUTF8	䢜
U+489C.0	fCNS11643	3-3F53
U+489C.0	fGB	5-703B
U+489C.0	fR/S	 162.8
U+489C.0	fTotalStrokes	 12
U+489C.0	fCangjie	YSMB
U+489C.0	fFourCorner	37300
U+489C.0	fCantonese	gwai1
U+489C.0	fMandarin	gui1
U+489C.0	fEnglish	same as &#x6B78;
U+489D.0	fUTF8	䢝
U+489D.0	fCNS11643	4-3B57
U+489D.0	fGB	5-702E
U+489D.0	fR/S	 162.8
U+489D.0	fTotalStrokes	 12
U+489D.0	fCangjie	YMLM
U+489D.0	fFourCorner	31300
U+489D.0	fCantonese	aa3; ngaa3 (also)
U+489D.0	fMandarin	ya4
U+489D.0	fEnglish	interchangeable with &#x4E9E;
U+489E.0	fUTF8	䢞
U+489E.0	fCNS11643	3-3F4E
U+489E.0	fR/S	 162.8
U+489E.0	fTotalStrokes	 12
U+489E.0	fCangjie	YDD
U+489E.0	fFourCorner	34300
U+489E.0	fCantonese	seot6
U+489E.0	fMandarin	shu4; nu4
U+489E.0	fEnglish	corrupt form of &#x48A4;
U+489F.0	fUTF8	䢟
U+489F.0	fCNS11643	6-4F48
U+489F.0	fR/S	 162.8
U+489F.0	fTotalStrokes	 12
U+489F.0	fCangjie	YYSK
U+489F.0	fFourCorner	38300
U+489F.0	fCantonese	jau4; zaau4
U+489F.0	fMandarin	you2
U+489F.0	fEnglish	[1] same as &#x904A; [2] interchangeable with &#x6E38;
U+48A0.0	fUTF8	䢠
U+48A0.0	fCNS11643	3-3F52
U+48A0.0	fR/S	 162.8
U+48A0.0	fTotalStrokes	 12
U+48A0.0	fCangjie	YFT
U+48A0.0	fFourCorner	39300
U+48A0.0	fCantonese	sung3
U+48A0.0	fMandarin	song4
U+48A0.0	fEnglish	standard form of &#x9001;
U+48A1.0	fUTF8	䢡
U+48A1.0	fCNS11643	4-412F
U+48A1.0	fGB	3-716B
U+48A1.0	fR/S	 162.9
U+48A1.0	fTotalStrokes	 13
U+48A1.0	fCangjie	YPTD
U+48A1.0	fFourCorner	34300
U+48A1.0	fCantonese	jip6
U+48A1.0	fMandarin	xie4
U+48A1.0	fEnglish	to stamp the foot forward, to walk, to stop
U+48A2.0	fUTF8	䢢
U+48A2.0	fCNS11643	3-4A25
U+48A2.0	fR/S	 162.10
U+48A2.0	fTotalStrokes	 14
U+48A2.0	fCangjie	YOIR
U+48A2.0	fFourCorner	38300
U+48A2.0	fCantonese	cong3
U+48A2.0	fMandarin	cang4
U+48A2.0	fEnglish	to pass; to pass through or by
U+48A3.0	fUTF8	䢣
U+48A3.0	fR/S	 162.10
U+48A3.0	fTotalStrokes	 14
U+48A3.0	fCangjie	YNIR
U+48A3.0	fCantonese	jiu4; ziu4
U+48A3.0	fMandarin	yao2
U+48A3.0	fEnglish	same as &#x9059;
U+48A4.0	fUTF8	䢤
U+48A4.0	fCNS11643	4-476C
U+48A4.0	fR/S	 162.10
U+48A4.0	fTotalStrokes	 14
U+48A4.0	fCangjie	YHDD
U+48A4.0	fFourCorner	33300
U+48A4.0	fCantonese	seot6; seoi2
U+48A4.0	fMandarin	shu4
U+48A5.0	fUTF8	䢥
U+48A5.0	fCNS11643	3-4A23
U+48A5.0	fR/S	 162.10
U+48A5.0	fTotalStrokes	 14
U+48A5.0	fCangjie	YMMU
U+48A5.0	fFourCorner	31300
U+48A5.0	fCantonese	jin4; zin2; zin4
U+48A5.0	fMandarin	yuan4; yan2; yan4
U+48A5.0	fEnglish	[1] (corrupted form) [2] (to walk) in harmony; in accord [3] look after; care for each other
U+48A6.0	fUTF8	䢦
U+48A6.0	fCNS11643	3-4F3A
U+48A6.0	fGB	5-7045
U+48A6.0	fR/S	 162.11
U+48A6.0	fTotalStrokes	 15
U+48A6.0	fCangjie	YYIJ
U+48A6.0	fFourCorner	30300
U+48A6.0	fCantonese	seot1
U+48A6.0	fMandarin	shuai4
U+48A6.0	fEnglish	interchangeable with &#x7387;, &#x5E25;
U+48A7.0	fUTF8	䢧
U+48A7.0	fCNS11643	4-4E31
U+48A7.0	fGB	3-7175
U+48A7.0	fR/S	 162.11
U+48A7.0	fTotalStrokes	 15
U+48A7.0	fCangjie	YSMH
U+48A7.0	fFourCorner	37300
U+48A7.0	fHakka	 liau5; liau2
U+48A7.0	fCantonese	liu6
U+48A7.0	fMandarin	liao4; lin3; que4
U+48A7.0	fEnglish	 jerk off (as a hen her fully fledged chickens)
U+48A8.0	fUTF8	䢨
U+48A8.0	fCNS11643	4-4E35
U+48A8.0	fGB	5-7025
U+48A8.0	fR/S	 162.11
U+48A8.0	fTotalStrokes	 15
U+48A8.0	fCangjie	YHOO
U+48A8.0	fFourCorner	38300
U+48A8.0	fCantonese	cung1; sung1
U+48A8.0	fMandarin	cong1
U+48A8.0	fEnglish	[1] walk slowly; walk in a leisurely manner [2] same as &#x8E64;
U+48A9.0	fUTF8	䢩
U+48A9.0	fCNS11643	3-4F3D
U+48A9.0	fGB	5-7041
U+48A9.0	fR/S	 162.11
U+48A9.0	fTotalStrokes	 15
U+48A9.0	fCangjie	YMRK
U+48A9.0	fFourCorner	38300
U+48A9.0	fCantonese	jyu6; zyu4; zyu6
U+48A9.0	fMandarin	yu4
U+48A9.0	fEnglish	same as &#x79A6;, &#x5FA1;
U+48AA.0	fUTF8	䢪
U+48AA.0	fCNS11643	3-532A
U+48AA.0	fR/S	 162.12
U+48AA.0	fTotalStrokes	 16
U+48AA.0	fCangjie	YHKB
U+48AA.0	fFourCorner	32300
U+48AA.0	fCantonese	hiu1
U+48AA.0	fMandarin	qiao1; bo2; cui4; jiao3; nu3; qian2
U+48AA.0	fEnglish	non-classical form of &#x6A47;
U+48AB.0	fUTF8	䢫
U+48AB.0	fCNS11643	3-5328
U+48AB.0	fR/S	 162.12
U+48AB.0	fTotalStrokes	 16
U+48AB.0	fCangjie	YNLB
U+48AB.0	fFourCorner	34300
U+48AB.0	fCantonese	ceoi4
U+48AB.0	fMandarin	sui2
U+48AB.0	fEnglish	same as &#x96A8;
U+48AC.0	fUTF8	䢬
U+48AC.0	fCNS11643	3-5329
U+48AC.0	fR/S	 162.12
U+48AC.0	fTotalStrokes	 16
U+48AC.0	fCangjie	YBBQ
U+48AC.0	fFourCorner	32300
U+48AC.0	fCantonese	cin1
U+48AC.0	fEnglish	ancient form of &#x9077;
U+48AD.0	fUTF8	䢭
U+48AD.0	fCNS11643	4-5938
U+48AD.0	fGB	3-7176
U+48AD.0	fR/S	 162.12; 162.13
U+48AD.0	fTotalStrokes	 16; 17
U+48AD.0	fCangjie	YTGO
U+48AD.0	fFourCorner	38300
U+48AD.0	fCantonese	jin6; sin6
U+48AD.0	fMandarin	yan2; yan4
U+48AD.0	fEnglish	[1] cover; screen; shade; conceal; shut off [2] block [3] shift; move [4] forward; convey [5] to walk
U+48AE.0	fUTF8	䢮
U+48AE.0	fCNS11643	4-5939
U+48AE.0	fGB	5-7048
U+48AE.0	fR/S	 162.13
U+48AE.0	fTotalStrokes	 17; 16
U+48AE.0	fCangjie	YMBW
U+48AE.0	fFourCorner	31300
U+48AE.0	fCantonese	leoi5
U+48AE.0	fMandarin	di2; lei4
U+48AE.0	fEnglish	rapid marching or running
U+48AF.0	fUTF8	䢯
U+48AF.0	fCNS11643	7-4377
U+48AF.0	fR/S	 162.14
U+48AF.0	fTotalStrokes	 18
U+48AF.0	fCangjie	YFFQ
U+48AF.0	fFourCorner	39300
U+48AF.0	fCantonese	leon4
U+48AF.0	fMandarin	lin2
U+48AF.0	fEnglish	standard form of &#x9074;
U+48B0.0	fUTF8	䢰
U+48B0.0	fCNS11643	3-5939
U+48B0.0	fR/S	 162.14
U+48B0.0	fTotalStrokes	 18
U+48B0.0	fCangjie	YSMG
U+48B0.0	fFourCorner	37300
U+48B0.0	fHakka	 jok7
U+48B0.0	fCantonese	tik1
U+48B0.0	fMandarin	ti4; tai2
U+48B0.0	fEnglish	same as &#x8DAF;
U+48B1.0	fUTF8	䢱
U+48B1.0	fCNS11643	4-615E
U+48B1.0	fGB	5-7037
U+48B1.0	fR/S	 162.15
U+48B1.0	fTotalStrokes	 19
U+48B1.0	fCangjie	YGWC
U+48B1.0	fFourCorner	34300
U+48B1.0	fCantonese	duk6
U+48B1.0	fMandarin	du2; zha4
U+48B1.0	fEnglish	[1] slight; abuse; insult; blaspheme [2] desecrate
U+48B2.0	fUTF8	䢲
U+48B2.0	fCNS11643	4-6C2C
U+48B2.0	fGB	3-717B
U+48B2.0	fR/S	 162.20
U+48B2.0	fTotalStrokes	 24
U+48B2.0	fCangjie	YBUE
U+48B2.0	fFourCorner	36300
U+48B2.0	fCantonese	wok6
U+48B2.0	fMandarin	yue4
U+48B2.0	fEnglish	to go all the way forward; without stopping
U+48B3.0	fUTF8	䢳
U+48B3.0	fCNS11643	3-225F
U+48B3.0	fR/S	 163.2
U+48B3.0	fTotalStrokes	 5
U+48B3.0	fCangjie	HNNL
U+48B3.0	fFourCorner	77220
U+48B3.0	fCantonese	gei2
U+48B3.0	fMandarin	ji3; ji4
U+48B3.0	fEnglish	the name of a place
U+48B4.0	fUTF8	䢴
U+48B4.0	fCNS11643	4-2332
U+48B4.0	fGB	S-2151
U+48B4.0	fR/S	 163.3
U+48B4.0	fTotalStrokes	 6
U+48B4.0	fCangjie	HJNL
U+48B4.0	fFourCorner	27420
U+48B4.0	fCantonese	cin1
U+48B4.0	fMandarin	qian1
U+48B4.0	fEnglish	interchangeable with &#x9098;
U+48B5.0	fUTF8	䢵
U+48B5.0	fCNS11643	3-2736
U+48B5.0	fGB	5-6F4F
U+48B5.0	fR/S	 163.4
U+48B5.0	fTotalStrokes	 7
U+48B5.0	fCangjie	MINL
U+48B5.0	fFourCorner	17720
U+48B5.0	fCantonese	wan4; fun2; waan4; waat6
U+48B5.0	fMandarin	yun2
U+48B5.0	fEnglish	same as &#x9116;
U+48B6.0	fUTF8	䢶
U+48B6.0	fCNS11643	3-2737
U+48B6.0	fGB	7-234B
U+48B6.0	fR/S	 163.4
U+48B6.0	fTotalStrokes	 7
U+48B6.0	fCangjie	DNL
U+48B6.0	fFourCorner	47920
U+48B6.0	fCantonese	bong1
U+48B6.0	fEnglish	same as &#x90A6;
U+48B7.0	fUTF8	䢷
U+48B7.0	fCNS11643	3-2738
U+48B7.0	fR/S	 163.4
U+48B7.0	fTotalStrokes	 7
U+48B7.0	fCangjie	BYNL
U+48B7.0	fFourCorner	77420
U+48B7.0	fCantonese	daan1
U+48B7.0	fMandarin	na4
U+48B8.0	fUTF8	䢸
U+48B8.0	fCNS11643	3-2A76
U+48B8.0	fR/S	 163.5
U+48B8.0	fTotalStrokes	 8
U+48B8.0	fCangjie	BMNL
U+48B8.0	fFourCorner	77120
U+48B8.0	fCantonese	zeoi1
U+48B8.0	fMandarin	qu1
U+48B8.0	fEnglish	name of a county in today's Shanxi Province
U+48B9.0	fUTF8	䢹
U+48B9.0	fCNS11643	4-2829
U+48B9.0	fGB	3-7060
U+48B9.0	fR/S	 163.5
U+48B9.0	fTotalStrokes	 8
U+48B9.0	fCangjie	SSNL
U+48B9.0	fFourCorner	77720
U+48B9.0	fCantonese	geoi2
U+48B9.0	fMandarin	ju3
U+48B9.0	fEnglish	name of a pavilion in today's Hunan Province
U+48BA.0	fUTF8	䢺
U+48BA.0	fGB	3-7066
U+48BA.0	fR/S	 163.5
U+48BA.0	fTotalStrokes	 8
U+48BA.0	fCangjie	UUNL
U+48BA.0	fCantonese	ceot1
U+48BA.0	fEnglish	name of a place in today's Sichuan Province
U+48BB.0	fUTF8	䢻
U+48BB.0	fCNS11643	3-2F36
U+48BB.0	fR/S	 163.6
U+48BB.0	fTotalStrokes	 9
U+48BB.0	fCangjie	SLNL
U+48BB.0	fFourCorner	17720
U+48BB.0	fCantonese	san4
U+48BB.0	fMandarin	chen2; jin4; tan2
U+48BB.0	fEnglish	the name of a place
U+48BC.0	fUTF8	䢼
U+48BC.0	fCNS11643	4-2B56
U+48BC.0	fGB	3-706A
U+48BC.0	fR/S	 163.6
U+48BC.0	fTotalStrokes	 9
U+48BC.0	fCangjie	TCNL
U+48BC.0	fFourCorner	47820
U+48BC.0	fCantonese	gung1; cung1
U+48BC.0	fMandarin	gong1
U+48BC.0	fEnglish	[1] the name of a state in old times [2] the name of a pavilion
U+48BD.0	fUTF8	䢽
U+48BD.0	fCNS11643	3-4522
U+48BD.0	fGB	5-6F4D
U+48BD.0	fR/S	 163.6
U+48BD.0	fTotalStrokes	 13
U+48BD.0	fCangjie	TCRAU
U+48BD.0	fFourCorner	44710
U+48BD.0	fCantonese	hong6
U+48BD.0	fMandarin	xiang4
U+48BD.0	fEnglish	[1] same as &#x5DF7; [2] interchangeable with &#x8856;
U+48BE.0	fUTF8	䢾
U+48BE.0	fCNS11643	3-2F3D
U+48BE.0	fR/S	 163.6
U+48BE.0	fTotalStrokes	 9
U+48BE.0	fCangjie	HUNL
U+48BE.0	fFourCorner	27220
U+48BE.0	fCantonese	san1
U+48BE.0	fMandarin	xian1; xian3
U+48BE.0	fEnglish	same as &#x59FA;
U+48BF.0	fUTF8	䢿
U+48BF.0	fCNS11643	4-2B54
U+48BF.0	fGB	3-7074
U+48BF.0	fR/S	 163.6
U+48BF.0	fTotalStrokes	 9
U+48BF.0	fCangjie	JVNL
U+48BF.0	fFourCorner	37420
U+48BF.0	fCantonese	on1; ngon1 (also)
U+48BF.0	fMandarin	an1
U+48BF.0	fEnglish	the name of a place in today's Hobei province
U+48C0.0	fUTF8	䣀
U+48C0.0	fCNS11643	4-2B58
U+48C0.0	fGB	3-7071
U+48C0.0	fR/S	 163.6
U+48C0.0	fTotalStrokes	 9
U+48C0.0	fCangjie	NUNL
U+48C0.0	fFourCorner	27220
U+48C0.0	fCantonese	gwai2; ngaai4
U+48C0.0	fMandarin	gui3; wei2
U+48C0.0	fEnglish	[1] the name of a mountain [2] the name of a place [3] the name of a state
U+48C1.0	fUTF8	䣁
U+48C1.0	fCNS11643	3-2F39
U+48C1.0	fR/S	 163.6
U+48C1.0	fTotalStrokes	 9
U+48C1.0	fCangjie	SMNL
U+48C1.0	fFourCorner	17120
U+48C1.0	fCantonese	jyu5; zyu5
U+48C1.0	fMandarin	yu3
U+48C1.0	fEnglish	the name of a pavilion (Han Dynasty) in today's Henan province
U+48C2.0	fUTF8	䣂
U+48C2.0	fCNS11643	3-2F3A
U+48C2.0	fR/S	 163.6
U+48C2.0	fTotalStrokes	 9
U+48C2.0	fCangjie	QDNL
U+48C2.0	fFourCorner	57920
U+48C2.0	fCantonese	leoi5; loi6
U+48C2.0	fMandarin	lei4; lei3
U+48C2.0	fEnglish	the name of a county in today's Hunan province
U+48C3.0	fUTF8	䣃
U+48C3.0	fCNS11643	6-346D
U+48C3.0	fR/S	 163.7
U+48C3.0	fTotalStrokes	 10; 9
U+48C3.0	fCangjie	OMNL
U+48C3.0	fFourCorner	87127
U+48C3.0	fCantonese	se3
U+48C3.0	fMandarin	xie4
U+48C4.0	fUTF8	䣄
U+48C4.0	fCNS11643	4-3038
U+48C4.0	fGB	3-707D
U+48C4.0	fR/S	 163.7
U+48C4.0	fTotalStrokes	 10
U+48C4.0	fCangjie	ODNL
U+48C4.0	fFourCorner	87920
U+48C4.0	fCantonese	tou4
U+48C4.0	fMandarin	tu2
U+48C4.0	fEnglish	the name of a place in today's Shandong province
U+48C5.0	fUTF8	䣅
U+48C5.0	fCNS11643	4-3034
U+48C5.0	fGB	3-6F29
U+48C5.0	fR/S	 163.7
U+48C5.0	fTotalStrokes	 10
U+48C5.0	fCangjie	MVNL
U+48C5.0	fFourCorner	77220
U+48C5.0	fCantonese	san4
U+48C5.0	fMandarin	chen2
U+48C5.0	fEnglish	the name of an ancient state in today's Henan province
U+48C6.0	fUTF8	䣆
U+48C6.0	fCNS11643	3-3428
U+48C6.0	fR/S	 163.7
U+48C6.0	fTotalStrokes	 10
U+48C6.0	fCangjie	MMNL
U+48C6.0	fFourCorner	17120
U+48C6.0	fHakka	 gin1;  hin2
U+48C6.0	fCantonese	jing4
U+48C6.0	fMandarin	xing2
U+48C6.0	fEnglish	a village in Shantung
U+48C7.0	fUTF8	䣇
U+48C7.0	fCNS11643	4-3033
U+48C7.0	fGB	3-707B
U+48C7.0	fR/S	 163.7
U+48C7.0	fTotalStrokes	 10
U+48C7.0	fCangjie	IENL
U+48C7.0	fFourCorner	47120
U+48C7.0	fCantonese	kau4
U+48C7.0	fMandarin	qiu2
U+48C7.0	fEnglish	[1] the name of a place [2] the name of a county in today's Henan province
U+48C8.0	fUTF8	䣈
U+48C8.0	fCNS11643	4-4771
U+48C8.0	fR/S	 163.7
U+48C8.0	fTotalStrokes	 14
U+48C8.0	fCangjie	RURAU
U+48C8.0	fFourCorner	66710
U+48C8.0	fCantonese	hong6; lung6
U+48C8.0	fMandarin	hang4; liao2; xiang4
U+48C8.0	fEnglish	[1] ancient form of &#x5DF7; [2] interchangeable with &#x8856;
U+48C9.0	fUTF8	䣉
U+48C9.0	fR/S	 163.7
U+48C9.0	fTotalStrokes	 14
U+48C9.0	fCangjie	EHRAU
U+48C9.0	fCantonese	saa1
U+48C9.0	fEnglish	[Korean character]
U+48CA.0	fUTF8	䣊
U+48CA.0	fCNS11643	4-356C
U+48CA.0	fGB	5-6F5D
U+48CA.0	fR/S	 163.8
U+48CA.0	fTotalStrokes	 11
U+48CA.0	fCangjie	FBNL
U+48CA.0	fFourCorner	97220
U+48CA.0	fCantonese	dong2
U+48CA.0	fMandarin	dang3
U+48CA.0	fEnglish	[1] the name of a place [2] same as &#x9EE8;
U+48CB.0	fUTF8	䣋
U+48CB.0	fCNS11643	4-356E
U+48CB.0	fGB	3-7126
U+48CB.0	fR/S	 163.8
U+48CB.0	fTotalStrokes	 11
U+48CB.0	fCangjie	BDNL
U+48CB.0	fFourCorner	27920
U+48CB.0	fCantonese	coi2
U+48CB.0	fMandarin	cai3
U+48CB.0	fEnglish	the name of a place
U+48CC.0	fUTF8	䣌
U+48CC.0	fCNS11643	4-3573
U+48CC.0	fR/S	 163.8
U+48CC.0	fTotalStrokes	 11
U+48CC.0	fCangjie	IINL
U+48CC.0	fFourCorner	07227
U+48CC.0	fCantonese	dai2
U+48CC.0	fMandarin	di3
U+48CC.0	fEnglish	non-classical form of &#x90B8;
U+48CD.0	fUTF8	䣍
U+48CD.0	fCNS11643	4-356B
U+48CD.0	fGB	3-7122
U+48CD.0	fR/S	 163.8
U+48CD.0	fTotalStrokes	 11
U+48CD.0	fCangjie	KUNL
U+48CD.0	fFourCorner	47120
U+48CD.0	fCantonese	jim2; zim1; zim2
U+48CD.0	fMandarin	yan3
U+48CD.0	fEnglish	interchangeable with &#x5944;
U+48CE.0	fUTF8	䣎
U+48CE.0	fCNS11643	3-3950
U+48CE.0	fR/S	 163.8
U+48CE.0	fTotalStrokes	 11
U+48CE.0	fCangjie	VWNL
U+48CE.0	fFourCorner	27620
U+48CE.0	fCantonese	zi1; ci2
U+48CE.0	fMandarin	zi1
U+48CE.0	fEnglish	the name of county (a village)
U+48CF.0	fUTF8	䣏
U+48CF.0	fCNS11643	4-3570
U+48CF.0	fGB	3-7124
U+48CF.0	fR/S	 163.8
U+48CF.0	fTotalStrokes	 11
U+48CF.0	fCangjie	OTCL
U+48CF.0	fFourCorner	27220
U+48CF.0	fCantonese	gung1
U+48CF.0	fMandarin	gong1; jing1
U+48CF.0	fEnglish	[1] non-classical form of &#x48BC; meaning not known
U+48D0.0	fUTF8	䣐
U+48D0.0	fCNS11643	4-3B5E
U+48D0.0	fR/S	 163.9
U+48D0.0	fTotalStrokes	 12
U+48D0.0	fCangjie	TKNL
U+48D0.0	fFourCorner	47520
U+48D0.0	fCantonese	jing1; zing1
U+48D0.0	fMandarin	ying1
U+48D0.0	fEnglish	the name of a place
U+48D1.0	fUTF8	䣑
U+48D1.0	fCNS11643	3-3F58
U+48D1.0	fR/S	 163.9
U+48D1.0	fTotalStrokes	 12
U+48D1.0	fCangjie	MGNL
U+48D1.0	fFourCorner	77220
U+48D1.0	fCantonese	cin4
U+48D1.0	fMandarin	he2; chan2
U+48D1.0	fEnglish	same as &#x5EDB;
U+48D2.0	fUTF8	䣒
U+48D2.0	fGB	S-2152
U+48D2.0	fR/S	 163.9
U+48D2.0	fTotalStrokes	 12
U+48D2.0	fCangjie	LONL
U+48D2.0	fCantonese	gwai3
U+48D3.0	fUTF8	䣓
U+48D3.0	fCNS11643	4-4134
U+48D3.0	fGB	3-7B67
U+48D3.0	fR/S	 163.10
U+48D3.0	fTotalStrokes	 13
U+48D3.0	fCangjie	MBNL
U+48D3.0	fFourCorner	17220
U+48D3.0	fCantonese	lai4
U+48D3.0	fMandarin	li2
U+48D3.0	fEnglish	name of a state, name of a county
U+48D4.0	fUTF8	䣔
U+48D4.0	fCNS11643	4-4136
U+48D4.0	fGB	3-707C
U+48D4.0	fR/S	 163.10
U+48D4.0	fTotalStrokes	 13
U+48D4.0	fCangjie	FCNL
U+48D4.0	fFourCorner	97820
U+48D4.0	fCantonese	so2; caau4; lik6; zeon1
U+48D4.0	fMandarin	suo3; suo4
U+48D4.0	fEnglish	the name of a pavilion in today's Henan province
U+48D5.0	fUTF8	䣕
U+48D5.0	fCNS11643	3-4521
U+48D5.0	fR/S	 163.10; 163.3; 187.3
U+48D5.0	fTotalStrokes	 13
U+48D5.0	fCangjie	SFNL
U+48D5.0	fFourCorner	77320
U+48D5.0	fHakka	 ma5; ma6
U+48D5.0	fCantonese	maa5
U+48D5.0	fMandarin	ma3
U+48D5.0	fEnglish	a Chinese family name
U+48D6.0	fUTF8	䣖
U+48D6.0	fCNS11643	4-593C
U+48D6.0	fR/S	 163.10
U+48D6.0	fTotalStrokes	 16; 13
U+48D6.0	fCangjie	SFRAU
U+48D6.0	fFourCorner	76310
U+48D6.0	fCantonese	maa5
U+48D6.0	fMandarin	ma3
U+48D6.0	fEnglish	same as &#x48D5;
U+48D7.0	fUTF8	䣗
U+48D7.0	fCNS11643	4-4138
U+48D7.0	fGB	3-7133
U+48D7.0	fR/S	 163.10
U+48D7.0	fTotalStrokes	 13
U+48D7.0	fCangjie	HJNL
U+48D7.0	fFourCorner	27420
U+48D7.0	fCantonese	gou1
U+48D8.0	fUTF8	䣘
U+48D8.0	fCNS11643	4-476D
U+48D8.0	fR/S	 163.11
U+48D8.0	fTotalStrokes	 14
U+48D8.0	fCangjie	FGNL
U+48D8.0	fFourCorner	97120
U+48D8.0	fCantonese	tong4
U+48D8.0	fMandarin	tang2
U+48D8.0	fEnglish	ancient form of &#x5802;
U+48D9.0	fUTF8	䣙
U+48D9.0	fCNS11643	4-476E
U+48D9.0	fGB	3-7139
U+48D9.0	fR/S	 163.11
U+48D9.0	fTotalStrokes	 14
U+48D9.0	fCangjie	UBNL
U+48D9.0	fFourCorner	27220
U+48D9.0	fCantonese	pui4
U+48D9.0	fMandarin	pei2
U+48D9.0	fEnglish	name of a county in today's Shanxi Province, name of an ancient state in Inner Mongolia name of a place
U+48DA.0	fUTF8	䣚
U+48DA.0	fCNS11643	3-4A29
U+48DA.0	fR/S	 163.11
U+48DA.0	fTotalStrokes	 14
U+48DA.0	fCangjie	LVNL
U+48DA.0	fFourCorner	57420
U+48DA.0	fCantonese	lau4
U+48DA.0	fMandarin	luu2; lou2; l2
U+48DA.0	fEnglish	the name of a village in today's Henan province
U+48DB.0	fUTF8	䣛
U+48DB.0	fCNS11643	3-4A28
U+48DB.0	fR/S	 163.11
U+48DB.0	fTotalStrokes	 14
U+48DB.0	fCangjie	DENL
U+48DB.0	fFourCorner	47120
U+48DB.0	fCantonese	sat1
U+48DB.0	fMandarin	qi1
U+48DB.0	fEnglish	[1] the name of a place in ancient times [2] same as &#x5380;
U+48DC.0	fUTF8	䣜
U+48DC.0	fCNS11643	5-4A60
U+48DC.0	fR/S	 163.11
U+48DC.0	fTotalStrokes	 14
U+48DC.0	fCangjie	YMNL
U+48DC.0	fFourCorner	27220
U+48DC.0	fCantonese	co4
U+48DC.0	fMandarin	cuo2
U+48DC.0	fEnglish	same as &#x9147;
U+48DD.0	fUTF8	䣝
U+48DD.0	fCNS11643	4-4770
U+48DD.0	fGB	3-713B
U+48DD.0	fR/S	 163.12; 163.11
U+48DD.0	fTotalStrokes	 14; 15
U+48DD.0	fCangjie	SANL
U+48DD.0	fFourCorner	77220
U+48DD.0	fCantonese	tou4; cyu5
U+48DD.0	fMandarin	tu2
U+48DD.0	fEnglish	the name of a place in today's Shanxi province
U+48DE.0	fUTF8	䣞
U+48DE.0	fCNS11643	3-4F3E
U+48DE.0	fR/S	 163.12
U+48DE.0	fTotalStrokes	 15
U+48DE.0	fCangjie	RUNL
U+48DE.0	fFourCorner	67420
U+48DE.0	fCantonese	ngok6; ok6 (also)
U+48DE.0	fMandarin	e4
U+48DE.0	fEnglish	standard form of &#x9102;
U+48DF.0	fUTF8	䣟
U+48DF.0	fCNS11643	4-4E3D
U+48DF.0	fGB	3-7140
U+48DF.0	fR/S	 163.12
U+48DF.0	fTotalStrokes	 15
U+48DF.0	fCangjie	MANL
U+48DF.0	fFourCorner	57627
U+48DF.0	fCantonese	caam4
U+48DF.0	fMandarin	zan1; can2
U+48DF.0	fEnglish	name of pavilion in today's Shandong Province, to pierce; to stab; to irritate; to hurt
U+48E0.0	fUTF8	䣠
U+48E0.0	fCNS11643	4-4E3C
U+48E0.0	fGB	3-713F
U+48E0.0	fR/S	 163.12
U+48E0.0	fTotalStrokes	 15
U+48E0.0	fCangjie	QANL
U+48E0.0	fFourCorner	57620
U+48E0.0	fCantonese	ze6; taai3
U+48E0.0	fMandarin	xi2; jie2; ti4
U+48E0.0	fEnglish	[1] the name of a village in today's Sichuan province [2] a corrupted form of &#x48DF;, &#x48E2;
U+48E1.0	fUTF8	䣡
U+48E1.0	fCNS11643	5-587E
U+48E1.0	fR/S	 163.13
U+48E1.0	fTotalStrokes	 16
U+48E1.0	fCangjie	TINL
U+48E1.0	fFourCorner	87520
U+48E1.0	fCantonese	ji4
U+48E1.0	fMandarin	yi2; ti2
U+48E1.0	fEnglish	name of a place in old times; in today's Anhui Province
U+48E2.0	fUTF8	䣢
U+48E2.0	fCNS11643	5-5F3C
U+48E2.0	fGB	3-7144
U+48E2.0	fR/S	 163.14; 163.11; 127.11
U+48E2.0	fTotalStrokes	 17
U+48E2.0	fCangjie	QDTAL
U+48E2.0	fFourCorner	57920
U+48E2.0	fCantonese	zik6
U+48E2.0	fMandarin	ji2
U+48E2.0	fEnglish	name of a place in ancient times, name of a river in ancient times
U+48E3.0	fUTF8	䣣
U+48E3.0	fCNS11643	3-604D
U+48E3.0	fR/S	 163.20
U+48E3.0	fTotalStrokes	 23
U+48E3.0	fCangjie	FFNL
U+48E3.0	fFourCorner	97320
U+48E3.0	fCantonese	dong2
U+48E3.0	fMandarin	dang3; dao4
U+48E3.0	fEnglish	same as &#x48CA;
U+48E4.0	fUTF8	䣤
U+48E4.0	fCNS11643	4-6B26
U+48E4.0	fGB	3-7148
U+48E4.0	fR/S	 163.20
U+48E4.0	fTotalStrokes	 23
U+48E4.0	fCangjie	BENL
U+48E4.0	fFourCorner	67420
U+48E4.0	fCantonese	fok3
U+48E4.0	fMandarin	jue2; jiao4
U+48E4.0	fEnglish	name of village in today's Shanxi Province
U+48E5.0	fUTF8	䣥
U+48E5.0	fCNS11643	3-2F3F
U+48E5.0	fR/S	 164.2
U+48E5.0	fTotalStrokes	 9
U+48E5.0	fCangjie	MWP
U+48E5.0	fFourCorner	11610
U+48E5.0	fCantonese	bei2
U+48E5.0	fMandarin	bi3; mi4
U+48E5.0	fEnglish	a kind of wine
U+48E6.0	fUTF8	䣦
U+48E6.0	fR/S	 164.2
U+48E6.0	fTotalStrokes	 9
U+48E6.0	fCangjie	MWKS
U+48E6.0	fFourCorner	1462
U+48E6.0	fCantonese	leoi6; laai6
U+48E6.0	fMandarin	lei4
U+48E6.0	fEnglish	to cut; to mince; to slice; to carve
U+48E7.0	fUTF8	䣧
U+48E7.0	fCNS11643	3-342B
U+48E7.0	fR/S	 164.3
U+48E7.0	fTotalStrokes	 10
U+48E7.0	fCangjie	MWIP
U+48E7.0	fFourCorner	13640
U+48E7.0	fCantonese	jik6; zik6
U+48E7.0	fMandarin	yi4
U+48E7.0	fEnglish	interchangeable with &#x9ED3;
U+48E8.0	fUTF8	䣨
U+48E8.0	fCNS11643	3-342C
U+48E8.0	fR/S	 164.3
U+48E8.0	fTotalStrokes	 10
U+48E8.0	fCangjie	MWQU
U+48E8.0	fFourCorner	15610
U+48E8.0	fCantonese	seon4
U+48E8.0	fMandarin	chun2
U+48E8.0	fEnglish	[1] same as &#x48E9; [2] interchangeable with &#x7D14; [3] same as &#x9187;
U+48E9.0	fUTF8	䣩
U+48E9.0	fCNS11643	4-3578
U+48E9.0	fGB	5-6D7C
U+48E9.0	fR/S	 164.4
U+48E9.0	fTotalStrokes	 11
U+48E9.0	fCangjie	MWPU
U+48E9.0	fFourCorner	15610
U+48E9.0	fHakka	 shun2; sun2
U+48E9.0	fCantonese	seon4; sung2; zaan3
U+48E9.0	fMandarin	chun2
U+48E9.0	fEnglish	[1] generous [2] rich [3] pure [4] unmixed [5] beautiful
U+48EA.0	fUTF8	䣪
U+48EA.0	fCNS11643	4-3574
U+48EA.0	fGB	3-6E51
U+48EA.0	fR/S	 164.4
U+48EA.0	fTotalStrokes	 11
U+48EA.0	fCangjie	MWJB
U+48EA.0	fFourCorner	15620
U+48EA.0	fCantonese	put3
U+48EA.0	fMandarin	po4
U+48EA.0	fEnglish	color of the wine, smell of alcohol
U+48EB.0	fUTF8	䣫
U+48EB.0	fCNS11643	3-3953
U+48EB.0	fR/S	 164.4
U+48EB.0	fTotalStrokes	 11
U+48EB.0	fCangjie	MWJE
U+48EB.0	fFourCorner	14640
U+48EB.0	fCantonese	ci1
U+48EB.0	fMandarin	li2
U+48EB.0	fEnglish	soured bean-curd; a milky preparation from beans
U+48EC.0	fUTF8	䣬
U+48EC.0	fCNS11643	5-3636
U+48EC.0	fR/S	 164.4
U+48EC.0	fTotalStrokes	 11
U+48EC.0	fCangjie	MWI
U+48EC.0	fFourCorner	13650
U+48EC.0	fCantonese	seon4; gwo1; lo2
U+48EC.0	fMandarin	dai4; zai3
U+48EC.0	fEnglish	[1] the shiny colorful of the wine; color of the wine [2] sweet
U+48ED.0	fUTF8	䣭
U+48ED.0	fCNS11643	4-3575
U+48ED.0	fR/S	 164.4
U+48ED.0	fTotalStrokes	 11
U+48ED.0	fCangjie	MWIK
U+48ED.0	fFourCorner	13630
U+48ED.0	fCantonese	jau5; tai3
U+48ED.0	fMandarin	tai4
U+48ED.0	fEnglish	an organic compound; phtholein
U+48EE.0	fUTF8	䣮
U+48EE.0	fCNS11643	4-3B60
U+48EE.0	fGB	3-6E57
U+48EE.0	fR/S	 164.5
U+48EE.0	fTotalStrokes	 12
U+48EE.0	fCangjie	MWIKK
U+48EE.0	fFourCorner	13640
U+48EE.0	fCantonese	put3
U+48EE.0	fMandarin	ba2; po4
U+48EE.0	fEnglish	non-classical form of &#x48EA;
U+48EF.0	fUTF8	䣯
U+48EF.0	fCNS11643	3-3F5A
U+48EF.0	fR/S	 164.5
U+48EF.0	fTotalStrokes	 12
U+48EF.0	fCangjie	MWBM
U+48EF.0	fFourCorner	17610
U+48EF.0	fCantonese	cou4; tin2
U+48EF.0	fMandarin	cu2; tian3
U+48EF.0	fEnglish	[1] wine or any thick fluid of the best quality [2] corrupted form of &#x9766;
U+48F0.0	fUTF8	䣰
U+48F0.0	fCNS11643	3-3F5B
U+48F0.0	fR/S	 164.5
U+48F0.0	fTotalStrokes	 12
U+48F0.0	fCangjie	MWSS
U+48F0.0	fFourCorner	11610
U+48F0.0	fCantonese	geoi6; hai4
U+48F0.0	fMandarin	qu2; ju4; yuan4
U+48F0.0	fEnglish	same as &#x91B5;
U+48F1.0	fUTF8	䣱
U+48F1.0	fCNS11643	3-3F5C
U+48F1.0	fGB	5-6E22
U+48F1.0	fR/S	 164.5
U+48F1.0	fTotalStrokes	 12; 13
U+48F1.0	fCangjie	MWPR
U+48F1.0	fFourCorner	17620
U+48F1.0	fCantonese	jyu3; heoi3 (also); heoi2 (also); zyu3
U+48F1.0	fMandarin	xu4; yi4
U+48F1.0	fEnglish	same as &#x9157;
U+48F2.0	fUTF8	䣲
U+48F2.0	fCNS11643	4-3B64
U+48F2.0	fGB	3-6E5C
U+48F2.0	fR/S	 164.5
U+48F2.0	fTotalStrokes	 12
U+48F2.0	fCangjie	MWIT
U+48F2.0	fFourCorner	13640
U+48F2.0	fHakka	 fan5
U+48F2.0	fCantonese	faan3
U+48F2.0	fMandarin	fan4
U+48F2.0	fEnglish	the spirits distilled from refuse grain
U+48F3.0	fUTF8	䣳
U+48F3.0	fR/S	 164.5
U+48F3.0	fTotalStrokes	 12
U+48F3.0	fCangjie	MWSMR
U+48F3.0	fFourCorner	1762
U+48F3.0	fCantonese	si1
U+48F3.0	fEnglish	unstrained wine, vinegar
U+48F4.0	fUTF8	䣴
U+48F4.0	fCNS11643	6-594C
U+48F4.0	fR/S	 164.6
U+48F4.0	fTotalStrokes	 13
U+48F4.0	fCangjie	MWUKU
U+48F4.0	fFourCorner	12610
U+48F4.0	fCantonese	jyu3; heoi2
U+48F4.0	fMandarin	xu4
U+48F4.0	fEnglish	non-classical form of &#x9157;
U+48F5.0	fUTF8	䣵
U+48F5.0	fCNS11643	3-4526
U+48F5.0	fR/S	 164.6
U+48F5.0	fTotalStrokes	 13
U+48F5.0	fCangjie	MWSJ
U+48F5.0	fFourCorner	11640
U+48F5.0	fCantonese	nei6; lei6 (also)
U+48F5.0	fMandarin	er4
U+48F5.0	fEnglish	[1] brew the wine a second time [2] get over a drinking bout
U+48F6.0	fUTF8	䣶
U+48F6.0	fCNS11643	4-413D
U+48F6.0	fGB	3-6E5D
U+48F6.0	fR/S	 164.6
U+48F6.0	fTotalStrokes	 13
U+48F6.0	fCangjie	MWHJR
U+48F6.0	fFourCorner	11664
U+48F6.0	fCantonese	wut6; tim4
U+48F6.0	fMandarin	huo2; tian2
U+48F6.0	fEnglish	[1] unstrained wine [2] offer plenty of food; treat kindly and generously [3] same as &#x751C;
U+48F7.0	fUTF8	䣷
U+48F7.0	fCNS11643	4-413C
U+48F7.0	fGB	5-6E26
U+48F7.0	fR/S	 164.6
U+48F7.0	fTotalStrokes	 13
U+48F7.0	fCangjie	MWHJD
U+48F7.0	fFourCorner	15690
U+48F7.0	fCantonese	zyu1
U+48F7.0	fMandarin	zhu1
U+48F7.0	fEnglish	[1] pour (wine) [2] unstrained wine
U+48F8.0	fUTF8	䣸
U+48F8.0	fCNS11643	4-413F
U+48F8.0	fGB	3-6E5F
U+48F8.0	fR/S	 164.6
U+48F8.0	fTotalStrokes	 13
U+48F8.0	fCangjie	OGMCW
U+48F8.0	fFourCorner	22600
U+48F8.0	fCantonese	jim5
U+48F8.0	fMandarin	ran3
U+48F8.0	fEnglish	tasteless, light, soybean sauce; soy, food in the form of paste, to eat to the full; satiated; surfeited, to dislike
U+48F9.0	fUTF8	䣹
U+48F9.0	fCNS11643	4-413E
U+48F9.0	fGB	3-6E5E
U+48F9.0	fR/S	 164.6
U+48F9.0	fTotalStrokes	 13
U+48F9.0	fCangjie	MWOI
U+48F9.0	fFourCorner	13650
U+48F9.0	fCantonese	fat6
U+48F9.0	fMandarin	fa1; fa2
U+48F9.0	fEnglish	 brew 10 percent of the wine
U+48FA.0	fUTF8	䣺
U+48FA.0	fCNS11643	4-4773
U+48FA.0	fGB	3-6E62
U+48FA.0	fR/S	 164.7
U+48FA.0	fTotalStrokes	 14
U+48FA.0	fCangjie	MWRB
U+48FA.0	fFourCorner	16620
U+48FA.0	fCantonese	gyun3; gyun1
U+48FA.0	fMandarin	juan1
U+48FA.0	fEnglish	filtrated the wine
U+48FB.0	fUTF8	䣻
U+48FB.0	fCNS11643	4-4774
U+48FB.0	fGB	3-6E63
U+48FB.0	fR/S	 164.7
U+48FB.0	fTotalStrokes	 14
U+48FB.0	fCangjie	MWOIR
U+48FB.0	fFourCorner	18660
U+48FB.0	fCantonese	ham1
U+48FB.0	fMandarin	han1
U+48FB.0	fEnglish	[1] color of the wine, dark red color of the face [2] non-classical form of &#x9163;
U+48FC.0	fUTF8	䣼
U+48FC.0	fCNS11643	4-4E3F
U+48FC.0	fGB	3-6E6B
U+48FC.0	fR/S	 164.8
U+48FC.0	fTotalStrokes	 15
U+48FC.0	fCangjie	MWYRF
U+48FC.0	fFourCorner	10690
U+48FC.0	fCantonese	loeng4
U+48FC.0	fMandarin	liang4; liang2
U+48FC.0	fEnglish	[1] mixed starch; one of the six drinks in old time [2] interchangeable with &#x6DBC;
U+48FD.0	fUTF8	䣽
U+48FD.0	fCNS11643	4-4E41
U+48FD.0	fGB	3-6E68
U+48FD.0	fR/S	 164.8
U+48FD.0	fTotalStrokes	 15
U+48FD.0	fCangjie	ORMCW
U+48FD.0	fFourCorner	86600
U+48FD.0	fCantonese	zi1
U+48FD.0	fMandarin	zhi1; ti3
U+48FD.0	fEnglish	[1] wine; rich wine [2] same as &#x918D;
U+48FE.0	fUTF8	䣾
U+48FE.0	fCNS11643	4-4E3E
U+48FE.0	fGB	5-6E2B
U+48FE.0	fR/S	 164.8
U+48FE.0	fTotalStrokes	 15
U+48FE.0	fCangjie	MWJBM
U+48FE.0	fFourCorner	13610
U+48FE.0	fCantonese	mik6
U+48FE.0	fMandarin	mi4
U+48FE.0	fEnglish	cheese dregs; lees, cheese, drunken sound
U+48FF.0	fUTF8	䣿
U+48FF.0	fCNS11643	4-4E40
U+48FF.0	fGB	3-6E65
U+48FF.0	fR/S	 164.8
U+48FF.0	fTotalStrokes	 15
U+48FF.0	fCangjie	MWMDT
U+48FF.0	fFourCorner	11610
U+48FF.0	fCantonese	jyu4
U+48FF.0	fMandarin	yu1
U+48FF.0	fEnglish	to entertain; to feast, to drink within limits; to drink appropriately
U+4900.0	fUTF8	䤀
U+4900.0	fGB	3-6E66
U+4900.0	fR/S	 164.9
U+4900.0	fTotalStrokes	 16
U+4900.0	fCangjie	MWTKR
U+4900.0	fCantonese	joek6
U+4901.0	fUTF8	䤁
U+4901.0	fCNS11643	4-543D
U+4901.0	fGB	3-6E70
U+4901.0	fR/S	 164.9
U+4901.0	fTotalStrokes	 16
U+4901.0	fCangjie	MWTMV
U+4901.0	fFourCorner	14610
U+4901.0	fCantonese	cam4; dam1
U+4901.0	fMandarin	chen2; cen2; che4; she4
U+4901.0	fEnglish	[1] cooked or well-done; prepared or processed [2] brew wine and store up for a long time [3] interchangeable with &#x9156;
U+4902.0	fUTF8	䤂
U+4902.0	fCNS11643	4-543C
U+4902.0	fGB	3-6E6F
U+4902.0	fR/S	 164.9
U+4902.0	fTotalStrokes	 16
U+4902.0	fCangjie	MWTMD
U+4902.0	fFourCorner	14690
U+4902.0	fCantonese	mui4; mei4; mui6
U+4902.0	fMandarin	mei2
U+4902.0	fEnglish	[1] another name for vinegar [2] same as &#x7E6A;
U+4903.0	fUTF8	䤃
U+4903.0	fCNS11643	4-543A
U+4903.0	fGB	3-6E73
U+4903.0	fR/S	 164.9
U+4903.0	fTotalStrokes	 16
U+4903.0	fCangjie	MWYTA
U+4903.0	fFourCorner	10660
U+4903.0	fHakka	 jim1; rim1
U+4903.0	fCantonese	jam1; zaam1
U+4903.0	fMandarin	yin1; yin4
U+4903.0	fEnglish	drunk
U+4904.0	fUTF8	䤄
U+4904.0	fCNS11643	3-532E
U+4904.0	fGB	5-6E2E
U+4904.0	fR/S	 164.9
U+4904.0	fTotalStrokes	 16
U+4904.0	fCangjie	MWMWL
U+4904.0	fFourCorner	11660
U+4904.0	fCantonese	min5; mung2; zyun2
U+4904.0	fMandarin	mian3; zhuan4
U+4904.0	fEnglish	flushed with drink; addicted to intoxicants
U+4905.0	fUTF8	䤅
U+4905.0	fCNS11643	4-5440
U+4905.0	fGB	3-6E72
U+4905.0	fR/S	 164.9
U+4905.0	fTotalStrokes	 16
U+4905.0	fCangjie	MWOMN
U+4905.0	fFourCorner	18620
U+4905.0	fCantonese	tou4
U+4905.0	fMandarin	dou4; tu2
U+4905.0	fEnglish	southerwood, bitter wine
U+4906.0	fUTF8	䤆
U+4906.0	fCNS11643	4-543B
U+4906.0	fGB	5-6E31
U+4906.0	fR/S	 164.9
U+4906.0	fTotalStrokes	 16
U+4906.0	fCangjie	MWNOK
U+4906.0	fFourCorner	12630
U+4906.0	fCantonese	kwai4
U+4906.0	fMandarin	gui3; kui2
U+4906.0	fEnglish	[1] a kind of rice wine; sake [2] wine used in ancestral worshipping or in offerings
U+4907.0	fUTF8	䤇
U+4907.0	fCNS11643	7-3454
U+4907.0	fR/S	 164.9
U+4907.0	fTotalStrokes	 16
U+4907.0	fCangjie	MWJCE
U+4907.0	fFourCorner	13640
U+4907.0	fCantonese	sau1
U+4907.0	fMandarin	sou1
U+4907.0	fEnglish	[1] white wine [2] a measure of wine containing about 3 pints
U+4908.0	fUTF8	䤈
U+4908.0	fCNS11643	3-5330
U+4908.0	fGB	5-6E30
U+4908.0	fR/S	 164.9
U+4908.0	fTotalStrokes	 16
U+4908.0	fCangjie	MWCMT
U+4908.0	fFourCorner	18610
U+4908.0	fCantonese	hei1
U+4908.0	fMandarin	xi1
U+4908.0	fEnglish	same as &#x91AF;
U+4909.0	fUTF8	䤉
U+4909.0	fCNS11643	4-593D
U+4909.0	fGB	3-6E77
U+4909.0	fR/S	 164.10
U+4909.0	fTotalStrokes	 17; 16
U+4909.0	fCangjie	MWPHT
U+4909.0	fFourCorner	13610
U+4909.0	fCantonese	mat6
U+4909.0	fMandarin	mi4; ming2; mo4; ru2
U+4909.0	fEnglish	[1] emptied the drink completely [2] bean sauce; salted preparations
U+490A.0	fUTF8	䤊
U+490A.0	fCNS11643	4-5942
U+490A.0	fGB	3-6E71
U+490A.0	fR/S	 164.10
U+490A.0	fTotalStrokes	 17; 16
U+490A.0	fCangjie	MWTSJ
U+490A.0	fFourCorner	14640
U+490A.0	fCantonese	jung4; maau4; zung4
U+490A.0	fMandarin	rong2
U+490A.0	fEnglish	[1] wine [2] brew a second time
U+490B.0	fUTF8	䤋
U+490B.0	fCNS11643	7-3C70
U+490B.0	fR/S	 164.10
U+490B.0	fTotalStrokes	 17
U+490B.0	fCangjie	TWIRM
U+490B.0	fFourCorner	83650
U+490B.0	fCantonese	juk1; gwok3; wik6
U+490B.0	fMandarin	yu4; guo2
U+490B.0	fEnglish	[1] yellow-faced; emaciated look [2] same as &#x805D;
U+490C.0	fUTF8	䤌
U+490C.0	fCNS11643	4-5941
U+490C.0	fGB	3-6E54
U+490C.0	fR/S	 164.10
U+490C.0	fTotalStrokes	 17; 16
U+490C.0	fCangjie	MWOIR
U+490C.0	fFourCorner	18660
U+490C.0	fCantonese	coeng1
U+490C.0	fEnglish	[1] a kind of wine be brewed from grains [2] a kind of daily drinks for a minority group
U+490D.0	fUTF8	䤍
U+490D.0	fCNS11643	5-6545
U+490D.0	fR/S	 164.11
U+490D.0	fTotalStrokes	 18
U+490D.0	fCangjie	MWTLB
U+490D.0	fFourCorner	14620
U+490D.0	fCantonese	mai4
U+490D.0	fMandarin	mi2
U+490D.0	fEnglish	rotten paste or soybean sauce; to grow mildewed
U+490E.0	fUTF8	䤎
U+490E.0	fCNS11643	4-6161
U+490E.0	fGB	3-6E7D
U+490E.0	fR/S	 164.12
U+490E.0	fTotalStrokes	 19
U+490E.0	fCangjie	MWNHB
U+490E.0	fFourCorner	17620
U+490E.0	fCantonese	gwat1
U+490E.0	fMandarin	ju2
U+490E.0	fEnglish	soybean sauce; soy, food in the form of paste, oysters sauce
U+490F.0	fUTF8	䤏
U+490F.0	fCNS11643	4-6164
U+490F.0	fGB	3-6E7C
U+490F.0	fR/S	 164.12
U+490F.0	fTotalStrokes	 19
U+490F.0	fCangjie	MWLYQ
U+490F.0	fFourCorner	11650
U+490F.0	fCantonese	pei5; pei2
U+490F.0	fMandarin	pi3
U+490F.0	fEnglish	same as &#x572E;
U+4910.0	fUTF8	䤐
U+4910.0	fCNS11643	4-6163
U+4910.0	fGB	3-6E7B
U+4910.0	fR/S	 164.12
U+4910.0	fTotalStrokes	 19
U+4910.0	fCangjie	MWMUA
U+4910.0	fFourCorner	11660
U+4910.0	fCantonese	zam2
U+4910.0	fMandarin	jin3
U+4910.0	fEnglish	to drink; refreshment; to have a little drink sweet, delicious; tasty
U+4911.0	fUTF8	䤑
U+4911.0	fCNS11643	4-6162
U+4911.0	fGB	3-6E78
U+4911.0	fR/S	 164.12
U+4911.0	fTotalStrokes	 19
U+4911.0	fCangjie	MWTMC
U+4911.0	fFourCorner	14680
U+4911.0	fCantonese	ong3; ngong3 (also)
U+4911.0	fMandarin	wang4
U+4911.0	fEnglish	wine
U+4912.0	fUTF8	䤒
U+4912.0	fCNS11643	4-6165
U+4912.0	fGB	3-6E50
U+4912.0	fR/S	 164.12
U+4912.0	fTotalStrokes	 19
U+4912.0	fCangjie	MWVII
U+4912.0	fFourCorner	12650
U+4912.0	fCantonese	gei6
U+4912.0	fMandarin	ji4; ji3
U+4912.0	fEnglish	wine made from glutinous rice, a kind wine to drink after bathing, color of the rice wine, a kind of good wine, bubbles of the wine
U+4913.0	fUTF8	䤓
U+4913.0	fCNS11643	4-675A
U+4913.0	fGB	3-6E7E
U+4913.0	fR/S	 164.14
U+4913.0	fTotalStrokes	 21
U+4913.0	fCangjie	MWTBO
U+4913.0	fFourCorner	14630
U+4913.0	fCantonese	mung4
U+4913.0	fMandarin	meng2
U+4914.0	fUTF8	䤔
U+4914.0	fCNS11643	4-675B
U+4914.0	fGB	3-6E79
U+4914.0	fR/S	 164.14
U+4914.0	fTotalStrokes	 21
U+4914.0	fCangjie	ELMCW
U+4914.0	fFourCorner	32600
U+4914.0	fCantonese	zim6; cin2; gap3; sim2; zit6
U+4914.0	fMandarin	jian4; niu2; xiang4
U+4914.0	fEnglish	[1] dregs of wine [2] tasteless; insipid
U+4915.0	fUTF8	䤕
U+4915.0	fCNS11643	4-6954
U+4915.0	fGB	3-6E59
U+4915.0	fR/S	 164.15
U+4915.0	fTotalStrokes	 22; 21
U+4915.0	fCangjie	MWVID
U+4915.0	fFourCorner	12690
U+4915.0	fCantonese	hok3
U+4915.0	fMandarin	he4; xue4
U+4915.0	fEnglish	same as &#x569B;
U+4916.0	fUTF8	䤖
U+4916.0	fCNS11643	4-6953
U+4916.0	fGB	3-6F23
U+4916.0	fR/S	 164.15
U+4916.0	fTotalStrokes	 22; 21
U+4916.0	fCangjie	MWATE
U+4916.0	fFourCorner	16630
U+4916.0	fCantonese	bou6
U+4916.0	fMandarin	bao4
U+4916.0	fEnglish	[1] a kind of wine [2] a kind of wine that brew for one night
U+4917.0	fUTF8	䤗
U+4917.0	fCNS11643	4-6C2E
U+4917.0	fGB	3-6F27
U+4917.0	fR/S	 164.17; 164.18
U+4917.0	fTotalStrokes	 24; 25
U+4917.0	fCangjie	YQMCW
U+4917.0	fFourCorner	07601
U+4917.0	fCantonese	gam2
U+4917.0	fMandarin	gan3
U+4917.0	fEnglish	taste of strong drink, the bitterness of the wine
U+4918.0	fUTF8	䤘
U+4918.0	fCNS11643	4-6C2D
U+4918.0	fGB	5-6E37
U+4918.0	fR/S	 164.17
U+4918.0	fTotalStrokes	 24
U+4918.0	fCangjie	MWOIM
U+4918.0	fFourCorner	13650
U+4918.0	fCantonese	caam2
U+4918.0	fMandarin	chan3; qian3
U+4918.0	fEnglish	vinegar, smell of vinegar
U+4919.0	fUTF8	䤙
U+4919.0	fCNS11643	4-6E3D
U+4919.0	fGB	3-6F28
U+4919.0	fR/S	 164.21
U+4919.0	fTotalStrokes	 28
U+4919.0	fCangjie	MWVOI
U+4919.0	fFourCorner	17630
U+4919.0	fCantonese	lik6
U+4919.0	fMandarin	li4
U+4919.0	fEnglish	dregs; lees of cheese, cheese
U+491A.0	fUTF8	䤚
U+491A.0	fCNS11643	4-357A
U+491A.0	fR/S	 166.4; 166.8; 130.8
U+491A.0	fTotalStrokes	 11; 12
U+491A.0	fCangjie	BWG
U+491A.0	fFourCorner	76210
U+491A.0	fCantonese	lei5; lap6
U+491A.0	fMandarin	li3; lve4; le4
U+491A.0	fEnglish	same as &#x674E;
U+491B.0	fUTF8	䤛
U+491B.0	fCNS11643	6-3C63
U+491B.0	fR/S	 167.2
U+491B.0	fTotalStrokes	 10; 9
U+491B.0	fCangjie	CVL
U+491B.0	fFourCorner	82100
U+491B.0	fCantonese	kau4
U+491B.0	fMandarin	qiu2
U+491B.0	fEnglish	[1] component parts of a cross-bow [2] same as &#x92B6;
U+491C.0	fUTF8	䤜
U+491C.0	fCNS11643	3-395A
U+491C.0	fR/S	 167.3
U+491C.0	fTotalStrokes	 11
U+491C.0	fCangjie	CQU
U+491C.0	fFourCorner	85110
U+491C.0	fCantonese	deon6
U+491C.0	fMandarin	dun4
U+491C.0	fEnglish	same as &#x920D;
U+491D.0	fUTF8	䤝
U+491D.0	fCNS11643	3-3F5F
U+491D.0	fR/S	 167.4
U+491D.0	fTotalStrokes	 12
U+491D.0	fCangjie	CHVL
U+491D.0	fFourCorner	87120
U+491D.0	fCantonese	ngaang6; aang6 (also); ngang6
U+491D.0	fMandarin	qiong2; ying4
U+491D.0	fEnglish	bracelet; armlet
U+491E.0	fUTF8	䤞
U+491E.0	fCNS11643	3-3F64
U+491E.0	fR/S	 167.4
U+491E.0	fTotalStrokes	 12
U+491E.0	fCangjie	CIKU
U+491E.0	fFourCorner	83110
U+491E.0	fCantonese	wan5
U+491E.0	fMandarin	yun3
U+491E.0	fEnglish	[1] corrupted form of &#x9217; [2] same as &#x92B3;
U+491F.0	fUTF8	䤟
U+491F.0	fCNS11643	3-3F61
U+491F.0	fR/S	 167.4
U+491F.0	fTotalStrokes	 12
U+491F.0	fCangjie	CBHN
U+491F.0	fFourCorner	87110
U+491F.0	fCantonese	cam4
U+491F.0	fMandarin	chen2
U+491F.0	fEnglish	standard form of &#x9202;
U+4920.0	fUTF8	䤠
U+4920.0	fCNS11643	3-3F67
U+4920.0	fR/S	 167.4
U+4920.0	fTotalStrokes	 12
U+4920.0	fCangjie	CYLM
U+4920.0	fFourCorner	81110
U+4920.0	fCantonese	zi2
U+4920.0	fMandarin	ri4; ji1; zhi3
U+4920.0	fEnglish	same as &#x937A;
U+4921.0	fUTF8	䤡
U+4921.0	fCNS11643	3-452C
U+4921.0	fR/S	 167.5
U+4921.0	fTotalStrokes	 13
U+4921.0	fCangjie	CGB
U+4921.0	fFourCorner	85140
U+4921.0	fCantonese	jim5; zim5
U+4921.0	fMandarin	ran3
U+4921.0	fEnglish	iron
U+4922.0	fUTF8	䤢
U+4922.0	fCNS11643	5-436D
U+4922.0	fR/S	 167.5
U+4922.0	fTotalStrokes	 13
U+4922.0	fCangjie	CRKS
U+4922.0	fFourCorner	86120
U+4922.0	fCantonese	baat3
U+4922.0	fMandarin	ba1
U+4922.0	fEnglish	same as &#x91DB;
U+4923.0	fUTF8	䤣
U+4923.0	fR/S	 167.5
U+4923.0	fTotalStrokes	 13
U+4923.0	fCangjie	CHFD
U+4923.0	fFourCorner	8214
U+4923.0	fCantonese	jyut3; lyut3
U+4923.0	fMandarin	luue4; le4
U+4923.0	fEnglish	corrupt form of &#x92DD;
U+4924.0	fUTF8	䤤
U+4924.0	fCNS11643	3-4A2E
U+4924.0	fR/S	 167.6
U+4924.0	fTotalStrokes	 14
U+4924.0	fCangjie	CYVO
U+4924.0	fFourCorner	80180
U+4924.0	fHakka	 goi3; koi1
U+4924.0	fCantonese	hoi1
U+4924.0	fMandarin	kai1
U+4924.0	fEnglish	implement; tool; instrument; utensil
U+4925.0	fUTF8	䤥
U+4925.0	fCNS11643	4-4779
U+4925.0	fGB	3-7630
U+4925.0	fR/S	 167.6
U+4925.0	fTotalStrokes	 14
U+4925.0	fCangjie	CNMU
U+4925.0	fFourCorner	87110
U+4925.0	fCantonese	gwai2
U+4925.0	fMandarin	gui3
U+4925.0	fEnglish	agricultural implements; from tools, shining iron, a plectrum from stringed the teeth of a saw
U+4926.0	fUTF8	䤦
U+4926.0	fCNS11643	3-4A36
U+4926.0	fR/S	 167.6
U+4926.0	fTotalStrokes	 14
U+4926.0	fCangjie	COI
U+4926.0	fFourCorner	83150
U+4926.0	fCantonese	jyut6; zyut6
U+4926.0	fMandarin	yue4
U+4926.0	fEnglish	same as &#x925E;
U+4927.0	fUTF8	䤧
U+4927.0	fCNS11643	4-4776
U+4927.0	fR/S	 167.6
U+4927.0	fTotalStrokes	 14
U+4927.0	fCangjie	CWR
U+4927.0	fFourCorner	86100
U+4927.0	fCantonese	fui3; waai6
U+4927.0	fMandarin	hui4
U+4928.0	fUTF8	䤨
U+4928.0	fCNS11643	4-477B
U+4928.0	fGB	3-762E
U+4928.0	fR/S	 167.6
U+4928.0	fTotalStrokes	 14
U+4928.0	fCangjie	CHHV
U+4928.0	fFourCorner	82130
U+4928.0	fCantonese	paan4; pik1; wan4
U+4928.0	fMandarin	pi1; pi4
U+4928.0	fEnglish	[1] split; slash or rip open (by means of a knife) [2] jewels or ornaments on a sword [3] analyze; study; investigate
U+4929.0	fUTF8	䤩
U+4929.0	fR/S	 167.6
U+4929.0	fTotalStrokes	 14
U+4929.0	fCangjie	CJHP
U+4929.0	fFourCorner	8311
U+4929.0	fCantonese	zaak6
U+4929.0	fMandarin	cha2
U+4929.0	fEnglish	an ornament at the end of a belt used in ancient times
U+492A.0	fUTF8	䤪
U+492A.0	fCNS11643	4-4778
U+492A.0	fGB	3-762F
U+492A.0	fR/S	 167.6
U+492A.0	fTotalStrokes	 14
U+492A.0	fCangjie	CHND
U+492A.0	fFourCorner	87190
U+492A.0	fCantonese	do2
U+492A.0	fMandarin	duo3
U+492A.0	fEnglish	[1] short; imcomplete; deficient [2] same as &#x5241;
U+492B.0	fUTF8	䤫
U+492B.0	fCNS11643	3-4F4F
U+492B.0	fR/S	 167.7
U+492B.0	fTotalStrokes	 15
U+492B.0	fCangjie	CUON
U+492B.0	fFourCorner	82120
U+492B.0	fCantonese	caam4
U+492B.0	fMandarin	chan2
U+492B.0	fEnglish	same as &#x9471;
U+492C.0	fUTF8	䤬
U+492C.0	fCNS11643	4-4E43
U+492C.0	fGB	5-755B
U+492C.0	fR/S	 167.7
U+492C.0	fTotalStrokes	 15
U+492C.0	fCangjie	CEFH
U+492C.0	fFourCorner	89120
U+492C.0	fCantonese	so1; saa1 (also)
U+492C.0	fMandarin	suo1
U+492C.0	fEnglish	[1] small gong [2] sieve; sifter; strainer
U+492D.0	fUTF8	䤭
U+492D.0	fCNS11643	3-4F47
U+492D.0	fR/S	 167.7
U+492D.0	fTotalStrokes	 15
U+492D.0	fCangjie	CKKB
U+492D.0	fFourCorner	84120
U+492D.0	fCantonese	sik1
U+492D.0	fMandarin	shi4; kuan4; sui4; yi2
U+492D.0	fEnglish	[1] ornament; decorate [2] push
U+492E.0	fUTF8	䤮
U+492E.0	fCNS11643	3-4F46
U+492E.0	fR/S	 167.7
U+492E.0	fTotalStrokes	 15
U+492E.0	fCangjie	CYLH
U+492E.0	fFourCorner	81120
U+492E.0	fCantonese	sip3
U+492E.0	fMandarin	she4
U+492E.0	fEnglish	iron, golden ornaments
U+492F.0	fUTF8	䤯
U+492F.0	fCNS11643	7-2A38
U+492F.0	fR/S	 167.7
U+492F.0	fTotalStrokes	 15
U+492F.0	fCangjie	CMTH
U+492F.0	fFourCorner	82120
U+492F.0	fCantonese	jing4; zing4; zoeng2
U+492F.0	fMandarin	xing2
U+492F.0	fEnglish	[1] same as &#x9276; [2] interchangeable &#x784E;
U+4930.0	fUTF8	䤰
U+4930.0	fCNS11643	3-4F4A
U+4930.0	fR/S	 167.7
U+4930.0	fTotalStrokes	 15
U+4930.0	fCangjie	GOC
U+4930.0	fFourCorner	47100
U+4930.0	fCantonese	jing4
U+4930.0	fMandarin	ying2; weng3
U+4930.0	fEnglish	to smelt; to fuse metals
U+4931.0	fUTF8	䤱
U+4931.0	fCNS11643	7-2A3E
U+4931.0	fR/S	 167.7
U+4931.0	fTotalStrokes	 15
U+4931.0	fCangjie	CUHL
U+4931.0	fFourCorner	82120
U+4931.0	fCantonese	sai6
U+4931.0	fMandarin	shi4
U+4931.0	fEnglish	ancient form of &#x8A93;
U+4932.0	fUTF8	䤲
U+4932.0	fGB	5-7551
U+4932.0	fR/S	 167.7
U+4932.0	fTotalStrokes	 15
U+4932.0	fCangjie	CGLC
U+4932.0	fCantonese	cek3
U+4932.0	fMandarin	chi4
U+4932.0	fEnglish	chemical element; ( 鍶) old translation; Sr
U+4933.0	fUTF8	䤳
U+4933.0	fCNS11643	3-5333
U+4933.0	fR/S	 167.8
U+4933.0	fTotalStrokes	 16
U+4933.0	fCangjie	CYOK
U+4933.0	fFourCorner	80140
U+4933.0	fCantonese	ze6
U+4933.0	fMandarin	ye4
U+4933.0	fEnglish	[1] mirrow [2] lens; glass
U+4934.0	fUTF8	䤴
U+4934.0	fCNS11643	3-5334
U+4934.0	fGB	5-7566
U+4934.0	fR/S	 167.8
U+4934.0	fTotalStrokes	 16
U+4934.0	fCangjie	CUNE
U+4934.0	fFourCorner	87170
U+4934.0	fHakka	 ham2
U+4934.0	fCantonese	ham4
U+4934.0	fMandarin	han2
U+4934.0	fEnglish	 armour; box; cup
U+4935.0	fUTF8	䤵
U+4935.0	fCNS11643	4-5448
U+4935.0	fGB	3-7644
U+4935.0	fR/S	 167.8
U+4935.0	fTotalStrokes	 16
U+4935.0	fCangjie	CLMY
U+4935.0	fFourCorner	81110
U+4935.0	fCantonese	fei6; fei1
U+4935.0	fMandarin	fei4
U+4935.0	fEnglish	[1] tiny needle; thin pin; one of the nine needles used in Chinese medicine [2] same as &#x6249;
U+4936.0	fUTF8	䤶
U+4936.0	fCNS11643	4-5446
U+4936.0	fGB	3-7641
U+4936.0	fR/S	 167.8
U+4936.0	fTotalStrokes	 16
U+4936.0	fCangjie	CKLU
U+4936.0	fFourCorner	84110
U+4936.0	fCantonese	jip3
U+4936.0	fMandarin	ye4
U+4936.0	fEnglish	a hammer; a mallet; a bludgeon, agricultural tools; farming implements, an iron (for pressing clothes)
U+4937.0	fUTF8	䤷
U+4937.0	fCNS11643	7-3C7B
U+4937.0	fR/S	 167.9
U+4937.0	fTotalStrokes	 17
U+4937.0	fCangjie	CSAV
U+4937.0	fFourCorner	81110
U+4937.0	fCantonese	jin2; aam1; zim2
U+4937.0	fMandarin	yan3; yan2
U+4937.0	fEnglish	[1] a lance with three or two points [2] edge or margin of an utensil
U+4938.0	fUTF8	䤸
U+4938.0	fCNS11643	3-5669
U+4938.0	fR/S	 167.9
U+4938.0	fTotalStrokes	 17
U+4938.0	fCangjie	CVNO
U+4938.0	fFourCorner	87130
U+4938.0	fCantonese	cyun3; zyun3
U+4938.0	fMandarin	cong1; zuan4
U+4938.0	fEnglish	short spear or lance
U+4939.0	fUTF8	䤹
U+4939.0	fCNS11643	7-3C77
U+4939.0	fGB	5-756C
U+4939.0	fR/S	 167.9
U+4939.0	fTotalStrokes	 17
U+4939.0	fCangjie	CJFE
U+4939.0	fFourCorner	83140
U+4939.0	fHakka	 seu1
U+4939.0	fCantonese	sau1; saau1; suk1; zip3
U+4939.0	fMandarin	sou1
U+4939.0	fEnglish	[1] engrave; carve [2] rust
U+493A.0	fUTF8	䤺
U+493A.0	fCNS11643	4-5945
U+493A.0	fR/S	 167.9
U+493A.0	fTotalStrokes	 17; 16
U+493A.0	fCangjie	CMRL
U+493A.0	fFourCorner	82120
U+493A.0	fCantonese	gan1
U+493A.0	fMandarin	jin1; yin3
U+493A.0	fEnglish	same as &#x91FF;, &#x65A4;
U+493B.0	fUTF8	䤻
U+493B.0	fCNS11643	4-5944
U+493B.0	fGB	3-765D
U+493B.0	fR/S	 167.9
U+493B.0	fTotalStrokes	 17; 16
U+493B.0	fCangjie	CKMB
U+493B.0	fFourCorner	84120
U+493B.0	fCantonese	do6
U+493B.0	fMandarin	duo4
U+493B.0	fEnglish	[1] heavy iron part of a plough [2] the linch-pin of a wheel
U+493C.0	fUTF8	䤼
U+493C.0	fCNS11643	F-5D4C
U+493C.0	fGB	3-7662
U+493C.0	fR/S	 167.9
U+493C.0	fTotalStrokes	 17
U+493C.0	fCangjie	CHAE
U+493C.0	fFourCorner	8613
U+493C.0	fCantonese	sin3
U+493C.0	fMandarin	xian4
U+493C.0	fEnglish	metal wire
U+493D.0	fUTF8	䤽
U+493D.0	fCNS11643	3-593F
U+493D.0	fR/S	 167.10
U+493D.0	fTotalStrokes	 18
U+493D.0	fCangjie	CWLE
U+493D.0	fFourCorner	86130
U+493D.0	fCantonese	gwaan1
U+493D.0	fMandarin	guan1
U+493D.0	fEnglish	farm tools; agricultural implements; a tool used to till the lands; spade or shovel
U+493E.0	fUTF8	䤾
U+493E.0	fCNS11643	3-5941
U+493E.0	fR/S	 167.10
U+493E.0	fTotalStrokes	 18
U+493E.0	fCangjie	CBHX
U+493E.0	fFourCorner	82170
U+493E.0	fCantonese	tou1; tou4
U+493E.0	fMandarin	tao1
U+493E.0	fEnglish	[1] contain; envelop [2] case; box
U+493F.0	fUTF8	䤿
U+493F.0	fCNS11643	4-5E2F
U+493F.0	fGB	5-756F
U+493F.0	fR/S	 167.10
U+493F.0	fTotalStrokes	 18
U+493F.0	fCangjie	CQHQ
U+493F.0	fFourCorner	87150
U+493F.0	fCantonese	kit3; hiu3
U+493F.0	fMandarin	qie4
U+493F.0	fEnglish	same as &#x9365;
U+4940.0	fUTF8	䥀
U+4940.0	fCNS11643	4-5E2C
U+4940.0	fGB	3-7674
U+4940.0	fR/S	 167.10
U+4940.0	fTotalStrokes	 18
U+4940.0	fCangjie	CUMI
U+4940.0	fFourCorner	82130
U+4940.0	fHakka	 zhan3; chan3; can3; cen3; zan3; zhen3
U+4940.0	fCantonese	cin2
U+4940.0	fMandarin	chan3
U+4940.0	fEnglish	 pull; persevere
U+4941.0	fUTF8	䥁
U+4941.0	fCNS11643	7-4439
U+4941.0	fR/S	 167.10
U+4941.0	fTotalStrokes	 18
U+4941.0	fCangjie	CNIW
U+4941.0	fFourCorner	87160
U+4941.0	fCantonese	ham4
U+4941.0	fMandarin	han2
U+4941.0	fEnglish	[1] armor [2] standard form of &#x5705;
U+4942.0	fUTF8	䥂
U+4942.0	fCNS11643	4-5E27
U+4942.0	fGB	3-7673
U+4942.0	fR/S	 167.10
U+4942.0	fTotalStrokes	 18
U+4942.0	fCangjie	CBMO
U+4942.0	fFourCorner	87130
U+4942.0	fCantonese	mung6
U+4942.0	fMandarin	meng2; meng4
U+4942.0	fEnglish	the sharp edge of a shovel or a spade
U+4943.0	fUTF8	䥃
U+4943.0	fCNS11643	5-654F
U+4943.0	fR/S	 167.10
U+4943.0	fTotalStrokes	 18
U+4943.0	fCangjie	COBG
U+4943.0	fFourCorner	84110
U+4943.0	fCantonese	ngok6; ok6 (also)
U+4943.0	fMandarin	yue4
U+4943.0	fEnglish	a big hammer
U+4944.0	fUTF8	䥄
U+4944.0	fCNS11643	4-5E2B
U+4944.0	fGB	3-766C
U+4944.0	fR/S	 167.10
U+4944.0	fTotalStrokes	 18
U+4944.0	fCangjie	CMTA
U+4944.0	fFourCorner	81160
U+4944.0	fHakka	 cu5; cih5
U+4944.0	fCantonese	cou3
U+4944.0	fMandarin	cu4; zan3; zan4
U+4944.0	fEnglish	a Chinese family name
U+4945.0	fUTF8	䥅
U+4945.0	fCNS11643	3-5948
U+4945.0	fR/S	 167.10
U+4945.0	fTotalStrokes	 18
U+4945.0	fCangjie	JMGC
U+4945.0	fFourCorner	40100
U+4945.0	fCantonese	kam4
U+4945.0	fMandarin	qin2; qian4
U+4945.0	fEnglish	ancient form of &#x7434;
U+4946.0	fUTF8	䥆
U+4946.0	fCNS11643	3-5947
U+4946.0	fR/S	 167.10
U+4946.0	fTotalStrokes	 18
U+4946.0	fCangjie	YLYC
U+4946.0	fFourCorner	00100
U+4946.0	fCantonese	kam4
U+4946.0	fMandarin	qin2; jin3; rou4; wei4
U+4946.0	fEnglish	ancient form of &#x7434;
U+4947.0	fUTF8	䥇
U+4947.0	fGB	7-2254
U+4947.0	fR/S	 167.10
U+4947.0	fTotalStrokes	 18
U+4947.0	fCangjie	CISM
U+4947.0	fCantonese	sin3
U+4947.0	fMandarin	shan4
U+4947.0	fEnglish	[1] chemical element; Sarmarium (Sm); old translation of ( 鋱) Tb, (same as 釤) a sickle with a long handle, to swing a sickle to cut (grass or wheat) [2] same as &#x91E4;
U+4948.0	fUTF8	䥈
U+4948.0	fCNS11643	3-5B46
U+4948.0	fGB	5-7572
U+4948.0	fR/S	 167.11
U+4948.0	fTotalStrokes	 19
U+4948.0	fCangjie	CTIT
U+4948.0	fFourCorner	84140
U+4948.0	fCantonese	mou5; mong5 (also)
U+4948.0	fMandarin	mang3; mu3
U+4948.0	fEnglish	[1] the symbol for Cobalt [2] iron (for pressing clothes)
U+4949.0	fUTF8	䥉
U+4949.0	fCNS11643	4-6166
U+4949.0	fR/S	 167.11
U+4949.0	fTotalStrokes	 19
U+4949.0	fCangjie	CJNI
U+4949.0	fFourCorner	83110
U+4949.0	fCantonese	jyun1; zyun1
U+4949.0	fMandarin	yuan1
U+4949.0	fEnglish	the curved iron part of a hoe
U+494A.0	fUTF8	䥊
U+494A.0	fCNS11643	3-5B4B
U+494A.0	fR/S	 167.11
U+494A.0	fTotalStrokes	 19
U+494A.0	fCangjie	CQMC
U+494A.0	fFourCorner	85180
U+494A.0	fCantonese	coeng1
U+494A.0	fMandarin	qiang1
U+494A.0	fEnglish	non-classical form of &#x9306;
U+494B.0	fUTF8	䥋
U+494B.0	fCNS11643	3-5B40
U+494B.0	fR/S	 167.11
U+494B.0	fTotalStrokes	 19
U+494B.0	fCangjie	CYRF
U+494B.0	fFourCorner	80130
U+494B.0	fCantonese	paang1; baang1; bong4; pang1
U+494B.0	fMandarin	peng1
U+494B.0	fEnglish	 smelt; refine the gold; alchemy
U+494C.0	fUTF8	䥌
U+494C.0	fCNS11643	3-5B52
U+494C.0	fR/S	 167.11
U+494C.0	fTotalStrokes	 19
U+494C.0	fCangjie	JAAC
U+494C.0	fFourCorner	30100
U+494C.0	fCantonese	zing3; tip3
U+494C.0	fMandarin	zheng4
U+494C.0	fEnglish	ancient form of &#x8B49;
U+494D.0	fUTF8	䥍
U+494D.0	fCNS11643	4-6169
U+494D.0	fGB	3-754F
U+494D.0	fR/S	 167.11
U+494D.0	fTotalStrokes	 19
U+494D.0	fCangjie	GIC
U+494D.0	fFourCorner	44100
U+494D.0	fCantonese	zi3
U+494D.0	fMandarin	zhi4
U+494D.0	fEnglish	[1] (corrupted form) [2] a whip [3] a farm tool used to regulate seedling and to weed
U+494E.0	fUTF8	䥎
U+494E.0	fCNS11643	5-6A33
U+494E.0	fR/S	 167.11
U+494E.0	fTotalStrokes	 19
U+494E.0	fCangjie	CMVB
U+494E.0	fFourCorner	81120
U+494E.0	fCantonese	seon4
U+494E.0	fMandarin	chun2
U+494E.0	fEnglish	instrument; implement; utensil; tool; apparatus
U+494F.0	fUTF8	䥏
U+494F.0	fCNS11643	4-6168
U+494F.0	fGB	3-7726
U+494F.0	fR/S	 167.11
U+494F.0	fTotalStrokes	 19
U+494F.0	fCangjie	CHOL
U+494F.0	fFourCorner	87120
U+494F.0	fCantonese	jyu5; zyu5
U+494F.0	fMandarin	yu3
U+494F.0	fEnglish	[1] unfitted for each other; irregular; unsuitable; not well-matched [2] hoe [3] same as &#x92D9;
U+4950.0	fUTF8	䥐
U+4950.0	fCNS11643	3-5B51
U+4950.0	fR/S	 167.11
U+4950.0	fTotalStrokes	 19
U+4950.0	fCangjie	NSC
U+4950.0	fFourCorner	17100
U+4950.0	fCantonese	mau4
U+4950.0	fMandarin	mou2
U+4950.0	fEnglish	corrupted form of &#x936A;
U+4951.0	fUTF8	䥑
U+4951.0	fCNS11643	3-5B4C
U+4951.0	fR/S	 167.11
U+4951.0	fTotalStrokes	 19
U+4951.0	fCangjie	CJON
U+4951.0	fFourCorner	84140
U+4951.0	fMandarin	wan3; wan4
U+4951.0	fEnglish	corrupt form of &#x933D;
U+4952.0	fUTF8	䥒
U+4952.0	fCNS11643	3-5B49
U+4952.0	fR/S	 167.11
U+4952.0	fTotalStrokes	 19
U+4952.0	fCangjie	NIC
U+4952.0	fFourCorner	13100
U+4952.0	fCantonese	koeng5; goeng6
U+4952.0	fMandarin	qiang2; chou2; jiang4
U+4952.0	fEnglish	lead (a metal)
U+4953.0	fUTF8	䥓
U+4953.0	fCNS11643	F-6434
U+4953.0	fR/S	 167.11
U+4953.0	fTotalStrokes	 19
U+4953.0	fCangjie	CTCG
U+4953.0	fFourCorner	8411
U+4953.0	fCantonese	gei1; kei4
U+4953.0	fEnglish	hoe
U+4954.0	fUTF8	䥔
U+4954.0	fCNS11643	3-5D3B
U+4954.0	fR/S	 167.12
U+4954.0	fTotalStrokes	 20
U+4954.0	fCangjie	CMWD
U+4954.0	fFourCorner	81190
U+4954.0	fCantonese	suk1
U+4954.0	fMandarin	su4
U+4954.0	fEnglish	gold
U+4955.0	fUTF8	䥕
U+4955.0	fCNS11643	4-616C
U+4955.0	fGB	3-7678
U+4955.0	fR/S	 167.12
U+4955.0	fTotalStrokes	 19; 20
U+4955.0	fCangjie	CFBK
U+4955.0	fFourCorner	88140
U+4955.0	fCantonese	pit3
U+4955.0	fMandarin	pie3
U+4955.0	fEnglish	[1] the blade or edge of a spade [2] an open-lided shallow pan to boil salt
U+4956.0	fUTF8	䥖
U+4956.0	fCNS11643	3-5D40
U+4956.0	fR/S	 167.12
U+4956.0	fTotalStrokes	 20
U+4956.0	fCangjie	CTWK
U+4956.0	fFourCorner	88130
U+4956.0	fCantonese	tin4
U+4956.0	fMandarin	dian4; tian2
U+4956.0	fEnglish	same as &#x923F;
U+4957.0	fUTF8	䥗
U+4957.0	fCNS11643	4-6466
U+4957.0	fGB	3-7721
U+4957.0	fR/S	 167.12
U+4957.0	fTotalStrokes	 20
U+4957.0	fCangjie	CGFO
U+4957.0	fFourCorner	87180
U+4957.0	fHakka	 k'wan3; k'wan5
U+4957.0	fCantonese	fun2
U+4957.0	fMandarin	kuan4; kuan3
U+4957.0	fEnglish	[1] branding-iron [2] solder
U+4958.0	fUTF8	䥘
U+4958.0	fCNS11643	4-646C
U+4958.0	fR/S	 167.12
U+4958.0	fTotalStrokes	 20
U+4958.0	fCangjie	COOA
U+4958.0	fFourCorner	88160
U+4958.0	fCantonese	co3; cok3; cou3
U+4958.0	fMandarin	zhu3; cu4; cuo4
U+4958.0	fEnglish	same as &#x932F;
U+4959.0	fUTF8	䥙
U+4959.0	fCNS11643	5-6E35
U+4959.0	fR/S	 167.12
U+4959.0	fTotalStrokes	 20
U+4959.0	fCangjie	CNLO
U+4959.0	fFourCorner	88130
U+4959.0	fCantonese	seoi6
U+4959.0	fMandarin	sui4
U+4959.0	fEnglish	same as &#x9406;
U+495A.0	fUTF8	䥚
U+495A.0	fCNS11643	3-5D48
U+495A.0	fR/S	 167.12
U+495A.0	fTotalStrokes	 20
U+495A.0	fCangjie	HNBUC
U+495A.0	fFourCorner	77210
U+495A.0	fCantonese	co1; zo3
U+495A.0	fMandarin	chu1
U+495A.0	fEnglish	ancient form of &#x521D;
U+495B.0	fUTF8	䥛
U+495B.0	fCNS11643	4-646D
U+495B.0	fGB	3-767E
U+495B.0	fR/S	 167.12
U+495B.0	fTotalStrokes	 20
U+495B.0	fCangjie	CQHF
U+495B.0	fFourCorner	87190
U+495B.0	fCantonese	git3
U+495B.0	fMandarin	jie2
U+495B.0	fEnglish	[1] a kind of sickle [2] chemical element; old translation of (&#x93B3;); Nickel Ni [3] carve
U+495C.0	fUTF8	䥜
U+495C.0	fR/S	 167.12
U+495C.0	fTotalStrokes	 20
U+495C.0	fCangjie	CAND
U+495C.0	fCantonese	gaan2; gan3
U+495C.0	fMandarin	jian4
U+495C.0	fEnglish	[Korean character]
U+495D.0	fUTF8	䥝
U+495D.0	fCNS11643	4-675F
U+495D.0	fGB	3-7730
U+495D.0	fR/S	 167.13
U+495D.0	fTotalStrokes	 21
U+495D.0	fCangjie	CIPP
U+495D.0	fFourCorner	80110
U+495D.0	fCantonese	juk1; ou1; ngou1 (also); ngou4 (also); ou4 (also)
U+495D.0	fMandarin	ao2
U+495D.0	fEnglish	standard form of &#x93D6;
U+495E.0	fUTF8	䥞
U+495E.0	fCNS11643	3-5E6B
U+495E.0	fR/S	 167.13
U+495E.0	fTotalStrokes	 21
U+495E.0	fCangjie	CHSK
U+495E.0	fFourCorner	88140
U+495E.0	fHakka	 gau3; giau3; kieu3
U+495E.0	fCantonese	giu2
U+495E.0	fMandarin	jiao3
U+495E.0	fEnglish	[1] a metal handle or ear of a vessel [2] cut with shears
U+495F.0	fUTF8	䥟
U+495F.0	fCNS11643	4-6469
U+495F.0	fGB	5-7629
U+495F.0	fR/S	 167.12; 167.13
U+495F.0	fTotalStrokes	 20; 21
U+495F.0	fCangjie	CQAV
U+495F.0	fFourCorner	86120
U+495F.0	fCantonese	jit3
U+495F.0	fMandarin	ye4
U+495F.0	fEnglish	non-classical form of &#x696C;
U+4960.0	fUTF8	䥠
U+4960.0	fCNS11643	4-6761
U+4960.0	fGB	5-7627
U+4960.0	fR/S	 167.13
U+4960.0	fTotalStrokes	 21
U+4960.0	fCangjie	CTIR
U+4960.0	fFourCorner	84150
U+4960.0	fCantonese	zam1; saam4; zaam1
U+4960.0	fMandarin	zhen4
U+4960.0	fEnglish	same as &#x937C;, &#x91DD;
U+4961.0	fUTF8	䥡
U+4961.0	fCNS11643	3-5E6C
U+4961.0	fGB	5-762F
U+4961.0	fR/S	 167.13
U+4961.0	fTotalStrokes	 21
U+4961.0	fCangjie	CLLD
U+4961.0	fFourCorner	84190
U+4961.0	fCantonese	jip6; zi3; zip6
U+4961.0	fMandarin	die2; ye4
U+4961.0	fEnglish	non-classical form of &#x9371;
U+4962.0	fUTF8	䥢
U+4962.0	fCNS11643	4-6763
U+4962.0	fR/S	 167.13
U+4962.0	fTotalStrokes	 21
U+4962.0	fCangjie	GEC
U+4962.0	fFourCorner	44100
U+4962.0	fCantonese	lung4
U+4962.0	fMandarin	long2
U+4962.0	fEnglish	sound of drums; loud music, to beat the drum and to keep the watches at night
U+4963.0	fUTF8	䥣
U+4963.0	fCNS11643	7-5568
U+4963.0	fR/S	 167.13
U+4963.0	fTotalStrokes	 21
U+4963.0	fCangjie	HEC
U+4963.0	fFourCorner	77109
U+4963.0	fCantonese	zok6
U+4963.0	fMandarin	zao2
U+4963.0	fEnglish	same as &#x947F;
U+4964.0	fUTF8	䥤
U+4964.0	fCNS11643	3-5E69
U+4964.0	fR/S	 167.13
U+4964.0	fTotalStrokes	 21
U+4964.0	fCangjie	CMBU
U+4964.0	fFourCorner	81110
U+4964.0	fCantonese	bok6; bou3; cit3; fuk1
U+4964.0	fMandarin	bao2; fu2
U+4964.0	fEnglish	the neck of a pestle; the neck part of a baton used to pound clothes in washing
U+4965.0	fUTF8	䥥
U+4965.0	fCNS11643	3-5E6E
U+4965.0	fR/S	 167.13
U+4965.0	fTotalStrokes	 21
U+4965.0	fCangjie	CYHC
U+4965.0	fFourCorner	80130
U+4965.0	fCantonese	lim4
U+4965.0	fMandarin	lian2
U+4965.0	fEnglish	[1] sickle; reaping-hook [2] an angle of the arrowhead [3] medical instrument used in old times
U+4966.0	fUTF8	䥦
U+4966.0	fCNS11643	3-5E6F
U+4966.0	fR/S	 167.13
U+4966.0	fTotalStrokes	 21
U+4966.0	fCangjie	CYKB
U+4966.0	fFourCorner	84130
U+4966.0	fCantonese	ceoi4
U+4967.0	fUTF8	䥧
U+4967.0	fCNS11643	3-5F6F
U+4967.0	fR/S	 167.14
U+4967.0	fTotalStrokes	 22
U+4967.0	fCangjie	CYHU
U+4967.0	fFourCorner	81160
U+4967.0	fCantonese	syun4
U+4967.0	fMandarin	xuan2; huan2
U+4967.0	fEnglish	same as &#x74BF;, &#x7487;
U+4968.0	fUTF8	䥨
U+4968.0	fCNS11643	3-6051
U+4968.0	fR/S	 167.14; 167.15
U+4968.0	fTotalStrokes	 23; 22
U+4968.0	fCangjie	CANR
U+4968.0	fFourCorner	87120
U+4968.0	fHakka	 li5; li6
U+4968.0	fCantonese	leoi6
U+4968.0	fMandarin	luu4; l4
U+4968.0	fEnglish	 file; rasp
U+4969.0	fUTF8	䥩
U+4969.0	fCNS11643	5-7155
U+4969.0	fR/S	 167.13; 167.14
U+4969.0	fTotalStrokes	 21; 22
U+4969.0	fCangjie	CHOK
U+4969.0	fFourCorner	88140
U+4969.0	fCantonese	mei4
U+4969.0	fMandarin	wei2
U+4969.0	fEnglish	hooks to hang; suspend something
U+496A.0	fUTF8	䥪
U+496A.0	fCNS11643	7-5967
U+496A.0	fR/S	 167.14
U+496A.0	fTotalStrokes	 22
U+496A.0	fCangjie	CAVF
U+496A.0	fFourCorner	86130
U+496A.0	fCantonese	hin2
U+496A.0	fMandarin	xian3
U+496A.0	fEnglish	 cup; pare; trim; shave
U+496B.0	fUTF8	䥫
U+496B.0	fCNS11643	3-5F6D
U+496B.0	fR/S	 167.14
U+496B.0	fTotalStrokes	 22
U+496B.0	fCangjie	CJIG
U+496B.0	fFourCorner	83150
U+496B.0	fCantonese	tit3; diu1
U+496B.0	fMandarin	tie3
U+496B.0	fEnglish	ancient form of &#x9435;
U+496C.0	fUTF8	䥬
U+496C.0	fCNS11643	7-5968
U+496C.0	fR/S	 167.14
U+496C.0	fTotalStrokes	 22
U+496C.0	fCangjie	CTII
U+496C.0	fFourCorner	84140
U+496C.0	fCantonese	bok3; baat6; bok6; hot3
U+496C.0	fMandarin	bo2
U+496C.0	fEnglish	interchangeable with &#x939B;
U+496D.0	fUTF8	䥭
U+496D.0	fCNS11643	3-5F70
U+496D.0	fR/S	 167.14
U+496D.0	fTotalStrokes	 22
U+496D.0	fCangjie	IEC
U+496D.0	fFourCorner	43100
U+496D.0	fCantonese	zing3
U+496D.0	fMandarin	zheng4
U+496D.0	fEnglish	same as &#x8B49;
U+496E.0	fUTF8	䥮
U+496E.0	fCNS11643	5-7455
U+496E.0	fR/S	 167.14
U+496E.0	fTotalStrokes	 22
U+496E.0	fCangjie	CMWL
U+496E.0	fFourCorner	81120
U+496E.0	fCantonese	zuk1
U+496E.0	fMandarin	zhu2
U+496E.0	fEnglish	to beat; to strike
U+496F.0	fUTF8	䥯
U+496F.0	fCNS11643	4-6B2A
U+496F.0	fGB	5-7577
U+496F.0	fR/S	 167.15
U+496F.0	fTotalStrokes	 23
U+496F.0	fCangjie	CWLP
U+496F.0	fFourCorner	86110
U+496F.0	fCantonese	bei1; baa6
U+496F.0	fMandarin	bei1; ba4; bai4
U+496F.0	fEnglish	[1] a farm tool to crush the clod of earth into pieces and make the land flat [2] a big iron stick [3] till lands; plough; cultivate; harrow
U+4970.0	fUTF8	䥰
U+4970.0	fCNS11643	5-765F
U+4970.0	fGB	3-7735
U+4970.0	fR/S	 167.15
U+4970.0	fTotalStrokes	 23
U+4970.0	fCangjie	CYVI
U+4970.0	fFourCorner	80130
U+4970.0	fCantonese	mang4; mong4
U+4970.0	fMandarin	meng2; meng4
U+4970.0	fEnglish	[1] melt [2] sell [3] fling a lance or a spear; brandish spears as if they were flying
U+4971.0	fUTF8	䥱
U+4971.0	fCNS11643	4-6B27
U+4971.0	fGB	3-7576
U+4971.0	fR/S	 167.15
U+4971.0	fTotalStrokes	 23
U+4971.0	fCangjie	CJHF
U+4971.0	fFourCorner	83120
U+4971.0	fCantonese	se2
U+4971.0	fMandarin	xie3
U+4971.0	fEnglish	[1] melt or cast (metal) by using a mould [2] non-classical form of &#x5BEB;
U+4972.0	fUTF8	䥲
U+4972.0	fCNS11643	3-604E
U+4972.0	fR/S	 167.15
U+4972.0	fTotalStrokes	 23
U+4972.0	fCangjie	CSRO
U+4972.0	fFourCorner	87180
U+4972.0	fCantonese	au1; ngau1 (also); aau1
U+4972.0	fMandarin	ou1
U+4972.0	fEnglish	a chemical element; old translation of &#x92AA; Eu
U+4973.0	fUTF8	䥳
U+4973.0	fCNS11643	3-6050
U+4973.0	fR/S	 167.15
U+4973.0	fTotalStrokes	 23
U+4973.0	fCangjie	CMBE
U+4973.0	fFourCorner	81140
U+4973.0	fCantonese	jau1; zaau1
U+4973.0	fMandarin	you1
U+4973.0	fEnglish	a chemical element; old translation of ( &#x92AA;) Eu
U+4974.0	fUTF8	䥴
U+4974.0	fCNS11643	3-6123
U+4974.0	fR/S	 167.16; 167.15
U+4974.0	fTotalStrokes	 24; 23
U+4974.0	fCangjie	CUOS
U+4974.0	fFourCorner	82120
U+4974.0	fCantonese	kwai4; teon1; zeon3; zyun1
U+4974.0	fEnglish	same as &#x942B;
U+4975.0	fUTF8	䥵
U+4975.0	fCNS11643	3-6121
U+4975.0	fR/S	 167.16
U+4975.0	fTotalStrokes	 24
U+4975.0	fCangjie	CAGU
U+4975.0	fFourCorner	84110
U+4975.0	fCantonese	hiu2
U+4975.0	fMandarin	xiao3
U+4975.0	fEnglish	lines; stripes; veins on an iron plate
U+4976.0	fUTF8	䥶
U+4976.0	fCNS11643	4-6C30
U+4976.0	fR/S	 167.16
U+4976.0	fTotalStrokes	 24
U+4976.0	fCangjie	CMHM
U+4976.0	fFourCorner	81110
U+4976.0	fCantonese	lik6; loek3
U+4976.0	fMandarin	ge1; li4
U+4976.0	fEnglish	same as &#x9B32;
U+4977.0	fUTF8	䥷
U+4977.0	fCNS11643	3-6124
U+4977.0	fR/S	 167.16
U+4977.0	fTotalStrokes	 24
U+4977.0	fCangjie	CHTN
U+4977.0	fFourCorner	82100
U+4977.0	fHakka	  zap7
U+4977.0	fCantonese	zaap6; lou5; zan2; zat3
U+4977.0	fMandarin	zha2
U+4977.0	fEnglish	a lever knife for cutting up fodder etc.
U+4978.0	fUTF8	䥸
U+4978.0	fCNS11643	3-616E
U+4978.0	fR/S	 167.17
U+4978.0	fTotalStrokes	 25; 26
U+4978.0	fCangjie	NBC
U+4978.0	fFourCorner	11100
U+4978.0	fCantonese	bei1; nei4
U+4978.0	fMandarin	mi2
U+4978.0	fEnglish	a sickle
U+4979.0	fUTF8	䥹
U+4979.0	fR/S	 167.21
U+4979.0	fTotalStrokes	 29
U+4979.0	fCangjie	CQHU
U+4979.0	fCantonese	ngit6; it6 (also)
U+4979.0	fEnglish	[Korean character]
U+497A.0	fUTF8	䥺
U+497A.0	fGB	7-222F
U+497A.0	fR/S	 167.4
U+497A.0	fTotalStrokes	 12
U+497A.0	fCangjie	OPMVH
U+497A.0	fCantonese	je4; ngaa4; aa4 (also); ze4
U+497A.0	fMandarin	ye2
U+497A.0	fEnglish	[1] same as &#x92E3; [2] simplified form of &#x91FE;
U+497B.0	fUTF8	䥻
U+497B.0	fGB	S-2444
U+497B.0	fR/S	 167.4
U+497B.0	fTotalStrokes	 12
U+497B.0	fCangjie	OPMKU
U+497B.0	fCantonese	mou4
U+497C.0	fUTF8	䥼
U+497C.0	fGB	S-2445
U+497C.0	fR/S	 167.4
U+497C.0	fTotalStrokes	 12
U+497C.0	fCangjie	OPPHH
U+497C.0	fCantonese	mat6
U+497D.0	fUTF8	䥽
U+497D.0	fGB	7-2224
U+497D.0	fR/S	 167.5
U+497D.0	fTotalStrokes	 13
U+497D.0	fCangjie	OPIVE
U+497D.0	fCantonese	put3
U+497D.0	fEnglish	simplified form of &#x&#x93FA;
U+497E.0	fUTF8	䥾
U+497E.0	fGB	S-2447
U+497E.0	fR/S	 167.5
U+497E.0	fTotalStrokes	 13
U+497E.0	fCangjie	OPBYM
U+497E.0	fCantonese	se2
U+497E.0	fMandarin	xie3
U+497E.0	fEnglish	[1] simplified form of &#x4971; [2] non-classical form of &#x5BEB;
U+497F.0	fUTF8	䥿
U+497F.0	fGB	S-2448
U+497F.0	fR/S	 167.5
U+497F.0	fTotalStrokes	 13
U+497F.0	fCangjie	OPHMS
U+497F.0	fCantonese	lai6
U+4980.0	fUTF8	䦀
U+4980.0	fGB	S-2449
U+4980.0	fR/S	 167.6
U+4980.0	fTotalStrokes	 14
U+4980.0	fCangjie	OPSMM
U+4980.0	fCantonese	jyu5
U+4981.0	fUTF8	䦁
U+4981.0	fGB	S-244A
U+4981.0	fR/S	 167.7
U+4981.0	fTotalStrokes	 15
U+4981.0	fCangjie	OPQJL
U+4981.0	fCantonese	bong1
U+4982.0	fUTF8	䦂
U+4982.0	fGB	7-2253
U+4982.0	fR/S	 167.10
U+4982.0	fTotalStrokes	 18
U+4982.0	fCangjie	OPISM
U+4982.0	fCantonese	sin3
U+4982.0	fMandarin	shan4
U+4982.0	fEnglish	[1] simplified form of &#x4947; [2] same as &#x91E4;
U+4983.0	fUTF8	䦃
U+4983.0	fGB	7-2238
U+4983.0	fR/S	 167.12
U+4983.0	fTotalStrokes	 20
U+4983.0	fCangjie	OPTJA
U+4983.0	fCantonese	zoek3; zoek6 (also); coeng1; zi1; pai3
U+4983.0	fEnglish	to dig with a hoe, a big hoe
U+4984.0	fUTF8	䦄
U+4984.0	fGB	S-244D
U+4984.0	fR/S	 167.12
U+4984.0	fTotalStrokes	 20
U+4984.0	fCangjie	OPJIV
U+4984.0	fCantonese	wai6
U+4985.0	fUTF8	䦅
U+4985.0	fGB	7-2239
U+4985.0	fR/S	 167.12
U+4985.0	fTotalStrokes	 20
U+4985.0	fCangjie	OPTTR
U+4985.0	fCantonese	sin3; sin6 (also)
U+4985.0	fMandarin	shan4; zhan3
U+4985.0	fEnglish	[1] simplified form of &#x9425; [2] same as &#x91E4;
U+4986.0	fUTF8	䦆
U+4986.0	fGB	7-223A
U+4986.0	fR/S	 167.20
U+4986.0	fTotalStrokes	 28
U+4986.0	fCangjie	OPBUE
U+4986.0	fCantonese	fok3; kyut3 (also); mong4
U+4986.0	fMandarin	jue2
U+4986.0	fEnglish	a big hoe, to eliminate; to uproot; to clear; to dig; to excavate
U+4987.0	fUTF8	䦇
U+4987.0	fCNS11643	6-3C65
U+4987.0	fR/S	 168.3
U+4987.0	fTotalStrokes	 9; 11
U+4987.0	fCangjie	SICVS
U+4987.0	fFourCorner	78720
U+4987.0	fCantonese	gai3
U+4987.0	fMandarin	ji4
U+4987.0	fEnglish	ancient form of &#x9AFB;
U+4988.0	fUTF8	䦈
U+4988.0	fCNS11643	4-3B67
U+4988.0	fGB	3-4C26
U+4988.0	fR/S	 168.5
U+4988.0	fTotalStrokes	 12; 13
U+4988.0	fCangjie	SIKM
U+4988.0	fFourCorner	74710
U+4988.0	fCantonese	ze1
U+4988.0	fMandarin	jie1; fang3; han3; ji2; mou3; za2; zuo3
U+4988.0	fEnglish	ancient form of &#x55DF;
U+4989.0	fUTF8	䦉
U+4989.0	fCNS11643	F-3F76
U+4989.0	fGB	S-237D
U+4989.0	fR/S	 168.5
U+4989.0	fTotalStrokes	 13
U+4989.0	fCangjie	SIWC
U+4989.0	fCantonese	sei3
U+498A.0	fUTF8	䦊
U+498A.0	fCNS11643	4-4149
U+498A.0	fGB	3-4C27
U+498A.0	fR/S	 168.6
U+498A.0	fTotalStrokes	 13; 14
U+498A.0	fCangjie	SIYMF
U+498A.0	fFourCorner	71790
U+498A.0	fCantonese	niu5; liu5 (also)
U+498A.0	fMandarin	niao3
U+498A.0	fEnglish	an appliance for walking on the sands (as boat for river; car for highway)
U+498B.0	fUTF8	䦋
U+498B.0	fCNS11643	4-5E32
U+498B.0	fGB	3-4C28
U+498B.0	fR/S	 168.11
U+498B.0	fTotalStrokes	 18; 19
U+498B.0	fCangjie	SIGSK
U+498B.0	fFourCorner	78740
U+498B.0	fCantonese	ngou4; ou4 (also)
U+498B.0	fMandarin	ao2
U+498B.0	fEnglish	[1] grow; increase; advance [2] become
U+498C.0	fUTF8	䦌
U+498C.0	fCNS11643	3-395B
U+498C.0	fR/S	 169.3
U+498C.0	fTotalStrokes	 11
U+498C.0	fCangjie	ANG
U+498C.0	fFourCorner	77100
U+498C.0	fCantonese	cyu3
U+498C.0	fMandarin	chu4; po4; rui4
U+498C.0	fEnglish	the door is wide open; directly in the front
U+498D.0	fUTF8	䦍
U+498D.0	fCNS11643	4-357C
U+498D.0	fGB	3-3A74
U+498D.0	fR/S	 169.3
U+498D.0	fTotalStrokes	 11
U+498D.0	fCangjie	ANON
U+498D.0	fFourCorner	77710
U+498D.0	fCantonese	ngat6; at6 (also)
U+498D.0	fMandarin	wu4
U+498D.0	fEnglish	to include; to embrace, to search for, intransigent; stubborn; obstinate
U+498E.0	fUTF8	䦎
U+498E.0	fCNS11643	3-3F6A
U+498E.0	fR/S	 169.4
U+498E.0	fTotalStrokes	 12
U+498E.0	fCangjie	ANMMU
U+498E.0	fFourCorner	77210
U+498E.0	fCantonese	gun2; gwun2 (also)
U+498E.0	fMandarin	guan3
U+498E.0	fEnglish	[1] interchangeable with &#x7BA1; [2] non-classical form of &#x958C;
U+498F.0	fUTF8	䦏
U+498F.0	fCNS11643	4-3B6B
U+498F.0	fGB	3-3A77
U+498F.0	fR/S	 169.4
U+498F.0	fTotalStrokes	 12
U+498F.0	fCangjie	ANOLL
U+498F.0	fFourCorner	77220
U+498F.0	fCantonese	haai6
U+498F.0	fMandarin	xie4
U+498F.0	fEnglish	wing of a door
U+4990.0	fUTF8	䦐
U+4990.0	fCNS11643	3-3F6E
U+4990.0	fR/S	 169.4
U+4990.0	fTotalStrokes	 12
U+4990.0	fCangjie	ANQ
U+4990.0	fFourCorner	77500
U+4990.0	fCantonese	ting2
U+4990.0	fMandarin	ting3
U+4990.0	fEnglish	[1] the bolt of a door; door latch [2] non-classical form of &#x958B;
U+4991.0	fUTF8	䦑
U+4991.0	fCNS11643	3-3F6B
U+4991.0	fR/S	 169.4
U+4991.0	fTotalStrokes	 12
U+4991.0	fCangjie	ANDK
U+4991.0	fFourCorner	77530
U+4991.0	fCantonese	hyut3
U+4991.0	fMandarin	xue4; xie4
U+4991.0	fEnglish	without a door; no family
U+4992.0	fUTF8	䦒
U+4992.0	fCNS11643	3-453B
U+4992.0	fR/S	 169.5
U+4992.0	fTotalStrokes	 13
U+4992.0	fCangjie	ANMR
U+4992.0	fFourCorner	77600
U+4992.0	fCantonese	dong6
U+4992.0	fMandarin	dang4
U+4992.0	fEnglish	[1] the door is closed [2] knock at the door; beat or to strike lightly
U+4993.0	fUTF8	䦓
U+4993.0	fCNS11643	4-4150
U+4993.0	fGB	5-3D51
U+4993.0	fR/S	 169.5
U+4993.0	fTotalStrokes	 13
U+4993.0	fCangjie	ANYR
U+4993.0	fFourCorner	77600
U+4993.0	fHakka	 zham1; zam1; zham5
U+4993.0	fCantonese	zaam1; zim1
U+4993.0	fMandarin	chan1
U+4993.0	fEnglish	 open a door so as to peep
U+4994.0	fUTF8	䦔
U+4994.0	fCNS11643	4-414E
U+4994.0	fGB	3-3A7B
U+4994.0	fR/S	 169.5
U+4994.0	fTotalStrokes	 13
U+4994.0	fCangjie	ANAM
U+4994.0	fFourCorner	77100
U+4994.0	fCantonese	taan2
U+4994.0	fMandarin	dan3; tan3
U+4994.0	fEnglish	[1] peg; stake; door stopper [2] the bolt of a door; door latch
U+4995.0	fUTF8	䦕
U+4995.0	fCNS11643	4-4823
U+4995.0	fR/S	 169.6
U+4995.0	fTotalStrokes	 14
U+4995.0	fCangjie	ANTT
U+4995.0	fFourCorner	77440
U+4995.0	fCantonese	ping1
U+4995.0	fMandarin	peng1
U+4995.0	fEnglish	same as &#x959B;
U+4996.0	fUTF8	䦖
U+4996.0	fCNS11643	3-4A3B
U+4996.0	fGB	5-3D58
U+4996.0	fR/S	 169.6
U+4996.0	fTotalStrokes	 14
U+4996.0	fCangjie	ANGR
U+4996.0	fFourCorner	77600
U+4996.0	fCantonese	kit3
U+4996.0	fMandarin	xia2; xie2
U+4996.0	fEnglish	sound of the door, the gate of the Zheng State city wall ( the Epoch of Spring and Autumn)
U+4997.0	fUTF8	䦗
U+4997.0	fCNS11643	3-4A3F
U+4997.0	fR/S	 169.6
U+4997.0	fTotalStrokes	 14
U+4997.0	fCangjie	ANHBT
U+4997.0	fFourCorner	77100
U+4997.0	fCantonese	gwik1; kwik1
U+4997.0	fMandarin	xu4
U+4997.0	fEnglish	same as &#x4F90;
U+4998.0	fUTF8	䦘
U+4998.0	fCNS11643	3-4A3A
U+4998.0	fR/S	 169.6
U+4998.0	fTotalStrokes	 14
U+4998.0	fCangjie	ANAV
U+4998.0	fFourCorner	77730
U+4998.0	fHakka	 han5; han6
U+4998.0	fCantonese	haan6
U+4998.0	fMandarin	xian4; bi4; xian3
U+4998.0	fEnglish	 threshold
U+4999.0	fUTF8	䦙
U+4999.0	fCNS11643	5-4A6E
U+4999.0	fR/S	 169.6
U+4999.0	fTotalStrokes	 14
U+4999.0	fCangjie	ANGDI
U+4999.0	fFourCorner	77340
U+4999.0	fHakka	 sii5; sii6; shi6; shi5
U+4999.0	fCantonese	zi6
U+4999.0	fMandarin	shi4; si4
U+4999.0	fEnglish	 eununch
U+499A.0	fUTF8	䦚
U+499A.0	fCNS11643	3-4A3C
U+499A.0	fR/S	 169.6
U+499A.0	fTotalStrokes	 14
U+499A.0	fCangjie	ANHJR
U+499A.0	fFourCorner	77600
U+499A.0	fCantonese	fut3; kut3
U+499A.0	fMandarin	kuo4; huo4; kua3; kua4
U+499A.0	fEnglish	[1] wide open of the door [2] non-classical form of &#x95CA;
U+499B.0	fUTF8	䦛
U+499B.0	fGB	7-2246
U+499B.0	fR/S	 169.6
U+499B.0	fTotalStrokes	 14
U+499B.0	fCangjie	ANNSD
U+499B.0	fCantonese	zang3; zaang1 (also); zaang3 (also)
U+499B.0	fMandarin	zheng4; zhi4
U+499B.0	fEnglish	struggle
U+499C.0	fUTF8	䦜
U+499C.0	fCNS11643	7-2A46
U+499C.0	fR/S	 169.7
U+499C.0	fTotalStrokes	 15
U+499C.0	fCangjie	ANMMR
U+499C.0	fFourCorner	77600
U+499C.0	fCantonese	ng4
U+499C.0	fMandarin	wu2; wu4
U+499C.0	fEnglish	same as standard form &#x90DA;
U+499D.0	fUTF8	䦝
U+499D.0	fCNS11643	4-4E48
U+499D.0	fGB	5-3D5C
U+499D.0	fR/S	 169.7
U+499D.0	fTotalStrokes	 15
U+499D.0	fCangjie	ANGLC
U+499D.0	fFourCorner	77330
U+499D.0	fCantonese	waak6
U+499D.0	fMandarin	huo4
U+499D.0	fEnglish	sound of the door, to open
U+499E.0	fUTF8	䦞
U+499E.0	fCNS11643	3-4F54
U+499E.0	fR/S	 169.7
U+499E.0	fTotalStrokes	 15
U+499E.0	fCangjie	ANEMG
U+499E.0	fFourCorner	77220
U+499E.0	fCantonese	jeon6; zeon6
U+499E.0	fMandarin	run4
U+499E.0	fEnglish	same as &#x958F;
U+499F.0	fUTF8	䦟
U+499F.0	fCNS11643	3-4F57
U+499F.0	fGB	5-3D5D
U+499F.0	fR/S	 169.7
U+499F.0	fTotalStrokes	 15
U+499F.0	fCangjie	ANOOG
U+499F.0	fFourCorner	77100
U+499F.0	fCantonese	caai2; ceoi3 (also); ceoi2 (also)
U+499F.0	fMandarin	wen3; chuai4
U+499F.0	fEnglish	[1] struggle; struggle; strive [2] firm; stable; secure
U+49A0.0	fUTF8	䦠
U+49A0.0	fCNS11643	4-544F
U+49A0.0	fGB	3-3B25
U+49A0.0	fR/S	 169.8
U+49A0.0	fTotalStrokes	 16
U+49A0.0	fCangjie	ANMSO
U+49A0.0	fFourCorner	77230
U+49A0.0	fCantonese	si2; ci2; zi1; zi3
U+49A0.0	fMandarin	zhuo2; shi3
U+49A0.0	fEnglish	[1] the name of a river [2] door
U+49A1.0	fUTF8	䦡
U+49A1.0	fCNS11643	4-544B
U+49A1.0	fGB	3-3B27
U+49A1.0	fR/S	 169.8
U+49A1.0	fTotalStrokes	 16
U+49A1.0	fCangjie	ANJRR
U+49A1.0	fFourCorner	77770
U+49A1.0	fCantonese	wun4
U+49A1.0	fMandarin	huan2
U+49A1.0	fEnglish	a door-keeper, an entrance of a palace, a secluded dweling
U+49A2.0	fUTF8	䦢
U+49A2.0	fCNS11643	3-533F
U+49A2.0	fR/S	 169.8
U+49A2.0	fTotalStrokes	 16
U+49A2.0	fCangjie	ANEGI
U+49A2.0	fFourCorner	77130
U+49A2.0	fCantonese	fut3
U+49A2.0	fMandarin	kuo4
U+49A2.0	fEnglish	non-classical form of &#x95CA;
U+49A3.0	fUTF8	䦣
U+49A3.0	fCNS11643	4-544C
U+49A3.0	fR/S	 169.8
U+49A3.0	fTotalStrokes	 16
U+49A3.0	fCangjie	ANYTR
U+49A3.0	fFourCorner	77600
U+49A3.0	fCantonese	fu3
U+49A3.0	fMandarin	fu4
U+49A3.0	fEnglish	open the door
U+49A4.0	fUTF8	䦤
U+49A4.0	fCNS11643	3-5341
U+49A4.0	fR/S	 169.8
U+49A4.0	fTotalStrokes	 16
U+49A4.0	fCangjie	ANEEE
U+49A4.0	fFourCorner	77440
U+49A4.0	fCantonese	ceoi3
U+49A4.0	fMandarin	chui4; chuai4; wen3
U+49A4.0	fEnglish	seed of grains
U+49A5.0	fUTF8	䦥
U+49A5.0	fCNS11643	3-5343
U+49A5.0	fGB	5-3D5F
U+49A5.0	fR/S	 169.8
U+49A5.0	fTotalStrokes	 16
U+49A5.0	fCangjie	ANDD
U+49A5.0	fFourCorner	77990
U+49A5.0	fCantonese	haan4; han4; kaa1; kaa2; kok3
U+49A5.0	fMandarin	xian2
U+49A5.0	fEnglish	[1] learn [2] habit; practice [3] ways of doing things [4] law; regulations
U+49A6.0	fUTF8	䦦
U+49A6.0	fCNS11643	3-5346
U+49A6.0	fR/S	 169.8
U+49A6.0	fTotalStrokes	 16
U+49A6.0	fCangjie	ANC
U+49A6.0	fFourCorner	77100
U+49A6.0	fCantonese	kam4; haam2; haam4; kaam4; sik1
U+49A6.0	fMandarin	qin2
U+49A6.0	fEnglish	same as &#x7434;
U+49A7.0	fUTF8	䦧
U+49A7.0	fCNS11643	3-5345
U+49A7.0	fR/S	 169.8
U+49A7.0	fTotalStrokes	 16
U+49A7.0	fCangjie	ANHXU
U+49A7.0	fFourCorner	77210
U+49A7.0	fCantonese	jik1; zik1
U+49A7.0	fMandarin	xi4; qie2; yan3
U+49A7.0	fEnglish	same as &#x9B29;
U+49A8.0	fUTF8	䦨
U+49A8.0	fCNS11643	7-3473
U+49A8.0	fR/S	 169.8
U+49A8.0	fTotalStrokes	 16
U+49A8.0	fCangjie	ANDW
U+49A8.0	fFourCorner	77900
U+49A8.0	fCantonese	laan4
U+49A8.0	fMandarin	lan2
U+49A8.0	fEnglish	corrupted form of &#x95CC;
U+49A9.0	fUTF8	䦩
U+49A9.0	fGB	3-3B26
U+49A9.0	fR/S	 169.9
U+49A9.0	fTotalStrokes	 17
U+49A9.0	fCangjie	ANBUE
U+49A9.0	fCantonese	man4
U+49AA.0	fUTF8	䦪
U+49AA.0	fCNS11643	4-594E
U+49AA.0	fGB	3-3B2C
U+49AA.0	fR/S	 169.9
U+49AA.0	fTotalStrokes	 17; 16
U+49AA.0	fCangjie	ANAPV
U+49AA.0	fFourCorner	77720
U+49AA.0	fCantonese	ngaat6; aat6 (also)
U+49AA.0	fMandarin	he2; ya4
U+49AA.0	fEnglish	sound of the door
U+49AB.0	fUTF8	䦫
U+49AB.0	fCNS11643	4-594F
U+49AB.0	fR/S	 169.9
U+49AB.0	fTotalStrokes	 17; 16
U+49AB.0	fCangjie	ANTLK
U+49AB.0	fFourCorner	77530
U+49AB.0	fCantonese	jing1; zing1
U+49AB.0	fMandarin	ying1
U+49AB.0	fEnglish	inside the door
U+49AC.0	fUTF8	䦬
U+49AC.0	fCNS11643	4-5954
U+49AC.0	fGB	3-3B28
U+49AC.0	fR/S	 169.9
U+49AC.0	fTotalStrokes	 17; 16
U+49AC.0	fCangjie	ANQHK
U+49AC.0	fFourCorner	77430
U+49AC.0	fCantonese	kyut3
U+49AC.0	fMandarin	que4
U+49AC.0	fEnglish	a vacancy; an opening
U+49AD.0	fUTF8	䦭
U+49AD.0	fCNS11643	3-566E
U+49AD.0	fGB	5-3D69
U+49AD.0	fR/S	 169.9
U+49AD.0	fTotalStrokes	 17
U+49AD.0	fCangjie	ANHDA
U+49AD.0	fFourCorner	77600
U+49AD.0	fHakka	 hong1; hiong1
U+49AD.0	fCantonese	hong1
U+49AD.0	fMandarin	hang2
U+49AD.0	fEnglish	[1] open a door [2] fragrant
U+49AE.0	fUTF8	䦮
U+49AE.0	fCNS11643	3-566C
U+49AE.0	fR/S	 169.9
U+49AE.0	fTotalStrokes	 17
U+49AE.0	fCangjie	ANQKA
U+49AE.0	fFourCorner	77600
U+49AE.0	fCantonese	ceon2
U+49AE.0	fMandarin	chun3
U+49AE.0	fEnglish	door of the center
U+49AF.0	fUTF8	䦯
U+49AF.0	fCNS11643	4-594D
U+49AF.0	fGB	3-3B2E
U+49AF.0	fR/S	 169.9
U+49AF.0	fTotalStrokes	 17; 16
U+49AF.0	fCangjie	ANMGK
U+49AF.0	fFourCorner	77140
U+49AF.0	fCantonese	zi3
U+49AF.0	fMandarin	zhi4
U+49AF.0	fEnglish	very dense; no break; cluster together
U+49B0.0	fUTF8	䦰
U+49B0.0	fR/S	 169.11
U+49B0.0	fTotalStrokes	 19
U+49B0.0	fCangjie	ANNWU
U+49B0.0	fCantonese	gau1; gaau1
U+49B0.0	fEnglish	same as &#x9B2E;
U+49B1.0	fUTF8	䦱
U+49B1.0	fCNS11643	4-5952
U+49B1.0	fGB	3-3A78
U+49B1.0	fR/S	 169.12; 169.9
U+49B1.0	fTotalStrokes	 16; 20; 17
U+49B1.0	fCangjie	ANIKF
U+49B1.0	fFourCorner	77420
U+49B1.0	fCantonese	wai2
U+49B1.0	fMandarin	kui3; wei3
U+49B1.0	fEnglish	[1] open the door [2] the door is slanted
U+49B2.0	fUTF8	䦲
U+49B2.0	fCNS11643	4-6766
U+49B2.0	fGB	3-3B35
U+49B2.0	fR/S	 169.13
U+49B2.0	fTotalStrokes	 21
U+49B2.0	fCangjie	ANNCR
U+49B2.0	fFourCorner	77260
U+49B2.0	fCantonese	jim4; zim4
U+49B2.0	fMandarin	yan2; qian4
U+49B2.0	fEnglish	[1] eaves of a ancestral temple (of the ruling family) [2] door of a shrine or a temple [3] small opening (of a door) [4] spy; peep
U+49B3.0	fUTF8	䦳
U+49B3.0	fCNS11643	4-6471
U+49B3.0	fGB	3-3A75
U+49B3.0	fR/S	 169.13; 169.12
U+49B3.0	fTotalStrokes	 20; 21
U+49B3.0	fCangjie	ANVHL
U+49B3.0	fFourCorner	77727
U+49B3.0	fCantonese	hoeng3
U+49B3.0	fMandarin	xiang4; hang4
U+49B3.0	fEnglish	between steps, window
U+49B4.0	fUTF8	䦴
U+49B4.0	fCNS11643	5-715B
U+49B4.0	fR/S	 169.13
U+49B4.0	fTotalStrokes	 21
U+49B4.0	fCangjie	ANWLJ
U+49B4.0	fFourCorner	77400
U+49B4.0	fCantonese	jik6; zik6
U+49B4.0	fMandarin	yi4
U+49B4.0	fEnglish	open the door
U+49B5.0	fUTF8	䦵
U+49B5.0	fCNS11643	4-6955
U+49B5.0	fGB	3-3A7C
U+49B5.0	fR/S	 169.14
U+49B5.0	fTotalStrokes	 22; 21
U+49B5.0	fCangjie	ANMFB
U+49B5.0	fFourCorner	77220
U+49B5.0	fCantonese	nai5; lai5 (also)
U+49B5.0	fMandarin	mi3; ni3
U+49B5.0	fEnglish	incapable, weak; feeble, narrow minded
U+49B6.0	fUTF8	䦶
U+49B6.0	fGB	7-2245
U+49B6.0	fR/S	 169.6
U+49B6.0	fTotalStrokes	 14
U+49B6.0	fCangjie	LSNSD
U+49B6.0	fCantonese	zang3; zaang1 (also); zaang3 (also)
U+49B6.0	fMandarin	zheng4
U+49B6.0	fEnglish	simplified form of &#x499B;
U+49B7.0	fUTF8	䦷
U+49B7.0	fGB	7-2247
U+49B7.0	fR/S	 169.7
U+49B7.0	fTotalStrokes	 15
U+49B7.0	fCangjie	LSOOG
U+49B7.0	fCantonese	caai2; ceoi3 (also); ceoi2 (also)
U+49B7.0	fMandarin	chuai4; wen3
U+49B7.0	fEnglish	simplified form of &#x499F;
U+49B8.0	fUTF8	䦸
U+49B8.0	fGB	S-2321
U+49B8.0	fR/S	 169.9
U+49B8.0	fTotalStrokes	 17
U+49B8.0	fCangjie	LSWLB
U+49B8.0	fCantonese	jyu6
U+49B9.0	fUTF8	䦹
U+49B9.0	fCNS11643	5-217D
U+49B9.0	fR/S	 170.2
U+49B9.0	fTotalStrokes	 5
U+49B9.0	fCangjie	NLJ
U+49B9.0	fFourCorner	74200
U+49B9.0	fCantonese	sap6; saap6; sou1; zak3
U+49B9.0	fMandarin	shi2
U+49B9.0	fEnglish	the name of a county in today's Sichuan province
U+49BA.0	fUTF8	䦺
U+49BA.0	fCNS11643	3-2260
U+49BA.0	fR/S	 170.2
U+49BA.0	fTotalStrokes	 5
U+49BA.0	fCangjie	NLMN
U+49BA.0	fFourCorner	71220
U+49BA.0	fCantonese	ding1; cap3
U+49BA.0	fMandarin	ding1
U+49BA.0	fEnglish	the name of a hill
U+49BB.0	fUTF8	䦻
U+49BB.0	fCNS11643	3-243E
U+49BB.0	fR/S	 170.3
U+49BB.0	fTotalStrokes	 6
U+49BB.0	fCangjie	NLND
U+49BB.0	fFourCorner	77240
U+49BB.0	fCantonese	zi2
U+49BB.0	fMandarin	zi3; ci4
U+49BB.0	fEnglish	the name of a place
U+49BC.0	fUTF8	䦼
U+49BC.0	fCNS11643	5-247B
U+49BC.0	fR/S	 170.4
U+49BC.0	fTotalStrokes	 7
U+49BC.0	fCangjie	NLDK
U+49BC.0	fFourCorner	75230
U+49BC.0	fCantonese	kyut3
U+49BC.0	fMandarin	pi4; jue2
U+49BC.0	fEnglish	[1] dig a passage through a mountain or hill; drill [2] collapsed in ruins; crumble [3] the earth cracking up
U+49BD.0	fUTF8	䦽
U+49BD.0	fCNS11643	6-2935
U+49BD.0	fR/S	 170.4
U+49BD.0	fTotalStrokes	 7
U+49BD.0	fCangjie	NLNIN
U+49BD.0	fFourCorner	77220
U+49BD.0	fCantonese	zeoi6
U+49BD.0	fMandarin	xu4
U+49BD.0	fEnglish	same as &#x5E8F;
U+49BE.0	fUTF8	䦾
U+49BE.0	fR/S	 170.4
U+49BE.0	fTotalStrokes	 7
U+49BE.0	fCangjie	HRIHU
U+49BE.0	fFourCorner	2371
U+49BE.0	fCantonese	jyun4; waan5
U+49BE.0	fMandarin	yun3
U+49BE.0	fEnglish	[1] lofty [2] same as &#x9656;
U+49BF.0	fUTF8	䦿
U+49BF.0	fGB	S-214B
U+49BF.0	fR/S	 170.4
U+49BF.0	fTotalStrokes	 7
U+49BF.0	fCangjie	NLL
U+49BF.0	fCantonese	zung1
U+49C0.0	fUTF8	䧀
U+49C0.0	fGB	S-214E
U+49C0.0	fR/S	 170.4
U+49C0.0	fTotalStrokes	 7
U+49C0.0	fCangjie	NLKI
U+49C0.0	fCantonese	gwang1
U+49C1.0	fUTF8	䧁
U+49C1.0	fCNS11643	4-2831
U+49C1.0	fGB	3-7532
U+49C1.0	fR/S	 170.5
U+49C1.0	fTotalStrokes	 8
U+49C1.0	fCangjie	NLPR
U+49C1.0	fFourCorner	77220
U+49C1.0	fCantonese	heoi2
U+49C1.0	fMandarin	xu3; chu4
U+49C1.0	fEnglish	to leave; to depart; to separate, distant from, a village in ancient times in Anyi
U+49C2.0	fUTF8	䧂
U+49C2.0	fCNS11643	4-2832
U+49C2.0	fGB	3-7533
U+49C2.0	fR/S	 170.5
U+49C2.0	fTotalStrokes	 8
U+49C2.0	fCangjie	NLSHR
U+49C2.0	fFourCorner	77260
U+49C2.0	fCantonese	ziu3; ziu6
U+49C2.0	fMandarin	zhao4; dao4; tiao3
U+49C2.0	fEnglish	[1] turn over the clod of the earth with a shovel or a spade [2] plough the land for rotation of crops [3] a channel for irrigation in farmland
U+49C3.0	fUTF8	䧃
U+49C3.0	fCNS11643	3-2A7A
U+49C3.0	fR/S	 170.5
U+49C3.0	fTotalStrokes	 8
U+49C3.0	fCangjie	NLW
U+49C3.0	fFourCorner	76200
U+49C3.0	fCantonese	tin4
U+49C3.0	fMandarin	tian2; dian4
U+49C3.0	fEnglish	the name of a place
U+49C4.0	fUTF8	䧄
U+49C4.0	fCNS11643	5-2B64
U+49C4.0	fR/S	 170.6
U+49C4.0	fTotalStrokes	 9
U+49C4.0	fCangjie	NLHER
U+49C4.0	fFourCorner	77260
U+49C4.0	fCantonese	gok3; aak1
U+49C4.0	fMandarin	ge4
U+49C4.0	fEnglish	the name of a person
U+49C5.0	fUTF8	䧅
U+49C5.0	fCNS11643	4-2B5A
U+49C5.0	fGB	3-7537
U+49C5.0	fR/S	 170.6
U+49C5.0	fTotalStrokes	 9
U+49C5.0	fCangjie	NLKN
U+49C5.0	fFourCorner	75232
U+49C5.0	fCantonese	ji4; zi4
U+49C5.0	fMandarin	yi2; ti4; ya4
U+49C5.0	fEnglish	[1] dangerous; hazardous; difficult; trouble [2] disaster; distress [3] the name of a place
U+49C6.0	fUTF8	䧆
U+49C6.0	fCNS11643	3-2F41
U+49C6.0	fR/S	 170.6
U+49C6.0	fTotalStrokes	 9
U+49C6.0	fCangjie	NLTC
U+49C6.0	fFourCorner	74280
U+49C6.0	fCantonese	hung4
U+49C6.0	fMandarin	hong2; kou3; qiong2
U+49C6.0	fEnglish	 pit; hole; gully
U+49C7.0	fUTF8	䧇
U+49C7.0	fCNS11643	4-2B59
U+49C7.0	fGB	3-753A
U+49C7.0	fR/S	 170.6
U+49C7.0	fTotalStrokes	 9
U+49C7.0	fCangjie	NLYHV
U+49C7.0	fFourCorner	70230
U+49C7.0	fCantonese	ji1; zi1
U+49C7.0	fMandarin	yi1; ni3; yi3
U+49C7.0	fEnglish	[1] the name of a hillside [2] the name of a village [3] the name of tomb of an emperor [4] the name of a high mound
U+49C8.0	fUTF8	䧈
U+49C8.0	fGB	S-214F
U+49C8.0	fR/S	 170.6
U+49C8.0	fTotalStrokes	 9
U+49C8.0	fCangjie	NLMLW
U+49C8.0	fCantonese	sai1
U+49C9.0	fUTF8	䧉
U+49C9.0	fCNS11643	4-3040
U+49C9.0	fGB	3-753D
U+49C9.0	fR/S	 170.7
U+49C9.0	fTotalStrokes	 10
U+49C9.0	fCangjie	NLWG
U+49C9.0	fFourCorner	76210
U+49C9.0	fCantonese	lei5
U+49C9.0	fMandarin	li3
U+49C9.0	fEnglish	[1] the name of a pavilion in Han Dynasty [2] the name of a county in ancient China
U+49CA.0	fUTF8	䧊
U+49CA.0	fCNS11643	4-303D
U+49CA.0	fGB	3-753F
U+49CA.0	fR/S	 170.7
U+49CA.0	fTotalStrokes	 10
U+49CA.0	fCangjie	NLHGR
U+49CA.0	fFourCorner	75260
U+49CA.0	fCantonese	huk1
U+49CA.0	fMandarin	gu4
U+49CA.0	fEnglish	a big mound, name of a mound in today's Shanxi Province
U+49CB.0	fUTF8	䧋
U+49CB.0	fCNS11643	3-3435
U+49CB.0	fGB	5-7467
U+49CB.0	fR/S	 170.7
U+49CB.0	fTotalStrokes	 10
U+49CB.0	fCangjie	NLBUU
U+49CB.0	fFourCorner	76210
U+49CB.0	fHakka	 han5
U+49CB.0	fCantonese	hin2
U+49CB.0	fMandarin	xian4; xian3
U+49CB.0	fEnglish	[1] the name of a place [2] boundary; a line
U+49CC.0	fUTF8	䧌
U+49CC.0	fCNS11643	4-303E
U+49CC.0	fR/S	 170.7
U+49CC.0	fTotalStrokes	 10
U+49CC.0	fCangjie	NLBV
U+49CC.0	fFourCorner	72240
U+49CC.0	fCantonese	seoi1
U+49CC.0	fMandarin	sui1
U+49CC.0	fEnglish	the name of a place
U+49CD.0	fUTF8	䧍
U+49CD.0	fCNS11643	3-3437
U+49CD.0	fR/S	 170.7
U+49CD.0	fTotalStrokes	 10
U+49CD.0	fCangjie	NLCOR
U+49CD.0	fFourCorner	78260
U+49CD.0	fCantonese	gwik1
U+49CD.0	fMandarin	xi4
U+49CD.0	fEnglish	non-classical form of &#x9699;
U+49CE.0	fUTF8	䧎
U+49CE.0	fCNS11643	4-303C
U+49CE.0	fGB	3-753E
U+49CE.0	fR/S	 170.7
U+49CE.0	fTotalStrokes	 10
U+49CE.0	fCangjie	NLRB
U+49CE.0	fFourCorner	76220
U+49CE.0	fCantonese	jyun5
U+49CE.0	fMandarin	xuan4
U+49CE.0	fEnglish	a pit; a hole; a gully
U+49CF.0	fUTF8	䧏
U+49CF.0	fR/S	 170.7
U+49CF.0	fTotalStrokes	 10
U+49CF.0	fCangjie	NLHEJ
U+49CF.0	fCantonese	fung1
U+49CF.0	fEnglish	[Japanese character]
U+49D0.0	fUTF8	䧐
U+49D0.0	fCNS11643	3-3963
U+49D0.0	fR/S	 170.8
U+49D0.0	fTotalStrokes	 11
U+49D0.0	fCangjie	NLYRD
U+49D0.0	fFourCorner	70240
U+49D0.0	fCantonese	gwok3; gok3 (also); gwik1; wong6
U+49D0.0	fMandarin	guo1
U+49D0.0	fEnglish	[1] a mountain in Shanxi province [2] the name of a county in old times
U+49D1.0	fUTF8	䧑
U+49D1.0	fCNS11643	3-395D
U+49D1.0	fR/S	 170.8
U+49D1.0	fTotalStrokes	 11
U+49D1.0	fCangjie	NLJMO
U+49D1.0	fFourCorner	73280
U+49D1.0	fHakka	 ti2
U+49D1.0	fCantonese	dai1; taai4
U+49D1.0	fEnglish	ancient form of &#x9684;, &#x5824;
U+49D2.0	fUTF8	䧒
U+49D2.0	fCNS11643	4-357D
U+49D2.0	fGB	3-753C
U+49D2.0	fR/S	 170.8
U+49D2.0	fTotalStrokes	 11
U+49D2.0	fCangjie	NLDOO
U+49D2.0	fFourCorner	74290
U+49D2.0	fCantonese	loi4
U+49D2.0	fMandarin	lai2; lan3
U+49D2.0	fEnglish	steps leading up to a building
U+49D3.0	fUTF8	䧓
U+49D3.0	fCNS11643	3-3960
U+49D3.0	fR/S	 170.8
U+49D3.0	fTotalStrokes	 11
U+49D3.0	fCangjie	NLBGR
U+49D3.0	fFourCorner	77220
U+49D3.0	fCantonese	zau1
U+49D3.0	fMandarin	zhou1
U+49D3.0	fEnglish	[1] big mound [2] abundant; fertile
U+49D4.0	fUTF8	䧔
U+49D4.0	fCNS11643	3-3962
U+49D4.0	fR/S	 170.8
U+49D4.0	fTotalStrokes	 11
U+49D4.0	fCangjie	NLOIP
U+49D4.0	fFourCorner	78230
U+49D4.0	fCantonese	nim6; lim6 (also)
U+49D4.0	fMandarin	nian4; mu4
U+49D4.0	fEnglish	 meet at the bank; shore; beach; coast
U+49D5.0	fUTF8	䧕
U+49D5.0	fCNS11643	3-3964
U+49D5.0	fR/S	 170.8
U+49D5.0	fTotalStrokes	 11
U+49D5.0	fCangjie	NLIRM
U+49D5.0	fFourCorner	73250
U+49D5.0	fCantonese	wik6; sing4
U+49D5.0	fMandarin	yu4; cheng2
U+49D5.0	fEnglish	[1] same as &#x57DF; [2] ancient form of &#x57CE;
U+49D6.0	fUTF8	䧖
U+49D6.0	fCNS11643	3-395F
U+49D6.0	fR/S	 170.8
U+49D6.0	fTotalStrokes	 11
U+49D6.0	fCangjie	NLII
U+49D6.0	fFourCorner	73250
U+49D6.0	fCantonese	cin5
U+49D6.0	fMandarin	jian4
U+49D6.0	fEnglish	 small mound
U+49D7.0	fUTF8	䧗
U+49D7.0	fCNS11643	3-3F6F
U+49D7.0	fR/S	 170.9
U+49D7.0	fTotalStrokes	 12
U+49D7.0	fCangjie	NLOAE
U+49D7.0	fFourCorner	78240
U+49D7.0	fCantonese	bik1
U+49D7.0	fMandarin	bi4
U+49D7.0	fEnglish	mountains collapsing; a land-slide
U+49D8.0	fUTF8	䧘
U+49D8.0	fCNS11643	4-3B6D
U+49D8.0	fGB	5-7523
U+49D8.0	fR/S	 170.9
U+49D8.0	fTotalStrokes	 12; 13
U+49D8.0	fCangjie	NLVNO
U+49D8.0	fFourCorner	77230
U+49D8.0	fCantonese	zyun6; saai3; syun6
U+49D8.0	fMandarin	zhuan4; qi2
U+49D8.0	fEnglish	[1] a low wall on the road side [2] wall [3] the space enclosed by a constellation
U+49D9.0	fUTF8	䧙
U+49D9.0	fCNS11643	3-3F74
U+49D9.0	fR/S	 170.9
U+49D9.0	fTotalStrokes	 12
U+49D9.0	fCangjie	NLUOP
U+49D9.0	fFourCorner	72230
U+49D9.0	fCantonese	ling4
U+49D9.0	fMandarin	ling2
U+49D9.0	fEnglish	ancient form of &#x9675;
U+49DA.0	fUTF8	䧚
U+49DA.0	fCNS11643	3-453F
U+49DA.0	fR/S	 170.10
U+49DA.0	fTotalStrokes	 13
U+49DA.0	fCangjie	NLYRB
U+49DA.0	fFourCorner	70220
U+49DA.0	fCantonese	hou6
U+49DA.0	fMandarin	hao4
U+49DA.0	fEnglish	same as &#x9117;
U+49DB.0	fUTF8	䧛
U+49DB.0	fCNS11643	3-453E
U+49DB.0	fR/S	 170.10
U+49DB.0	fTotalStrokes	 13
U+49DB.0	fCangjie	NLYBS
U+49DB.0	fFourCorner	70220
U+49DB.0	fCantonese	bong6
U+49DB.0	fMandarin	bang4
U+49DB.0	fEnglish	same as &#x508D;
U+49DC.0	fUTF8	䧜
U+49DC.0	fCNS11643	4-4155
U+49DC.0	fGB	5-7526
U+49DC.0	fR/S	 170.10
U+49DC.0	fTotalStrokes	 13
U+49DC.0	fCangjie	NLILR
U+49DC.0	fFourCorner	70267
U+49DC.0	fCantonese	tong4; cang4
U+49DC.0	fMandarin	tang2
U+49DC.0	fEnglish	[1] same as &#x5858; [2] interchangeable &#x5510;
U+49DD.0	fUTF8	䧝
U+49DD.0	fCNS11643	4-4154
U+49DD.0	fGB	3-7548
U+49DD.0	fR/S	 170.10
U+49DD.0	fTotalStrokes	 13
U+49DD.0	fCangjie	NLUMI
U+49DD.0	fFourCorner	72230
U+49DD.0	fCantonese	ci1; lei4
U+49DD.0	fMandarin	chi1; di4; yi2; zhi4
U+49DD.0	fEnglish	[1] the name of a place [2] the name of a mountain
U+49DE.0	fUTF8	䧞
U+49DE.0	fCNS11643	3-453D
U+49DE.0	fGB	5-7465
U+49DE.0	fR/S	 170.10
U+49DE.0	fTotalStrokes	 13
U+49DE.0	fCangjie	NLSQF
U+49DE.0	fFourCorner	71220
U+49DE.0	fHakka	 ma5; ma6
U+49DE.0	fCantonese	maa6
U+49DE.0	fMandarin	ma4; fu4
U+49DE.0	fEnglish	 pile up (as bricks)
U+49DF.0	fUTF8	䧟
U+49DF.0	fCNS11643	3-4541
U+49DF.0	fR/S	 170.10
U+49DF.0	fTotalStrokes	 13
U+49DF.0	fCangjie	NLBHX
U+49DF.0	fFourCorner	72270
U+49DF.0	fHakka	 ham5
U+49DF.0	fCantonese	haam6
U+49DF.0	fMandarin	xian4; xuan4
U+49DF.0	fEnglish	same as &#x9677;
U+49E0.0	fUTF8	䧠
U+49E0.0	fCNS11643	3-4A41
U+49E0.0	fR/S	 170.11
U+49E0.0	fTotalStrokes	 14
U+49E0.0	fCangjie	NLJII
U+49E0.0	fFourCorner	75240
U+49E0.0	fCantonese	zyun1; seon4; zyun2
U+49E0.0	fMandarin	shuan4
U+49E0.0	fEnglish	same as &#x911F;
U+49E1.0	fUTF8	䧡
U+49E1.0	fCNS11643	3-4A40
U+49E1.0	fGB	5-752A
U+49E1.0	fR/S	 170.11
U+49E1.0	fTotalStrokes	 14
U+49E1.0	fCangjie	NLILB
U+49E1.0	fFourCorner	70220
U+49E1.0	fHakka	 jung2
U+49E1.0	fCantonese	jung4; zung4
U+49E1.0	fMandarin	yong1
U+49E1.0	fEnglish	same as &#x5889;
U+49E2.0	fUTF8	䧢
U+49E2.0	fCNS11643	3-4A43
U+49E2.0	fGB	5-7466
U+49E2.0	fR/S	 170.11
U+49E2.0	fTotalStrokes	 14
U+49E2.0	fCangjie	NLSRR
U+49E2.0	fFourCorner	71210
U+49E2.0	fCantonese	keoi1
U+49E2.0	fMandarin	qu1
U+49E2.0	fEnglish	[1] rough road [2] rugged; steep mountain [3] uneasy
U+49E3.0	fUTF8	䧣
U+49E3.0	fCNS11643	3-4A42
U+49E3.0	fR/S	 170.11
U+49E3.0	fTotalStrokes	 14
U+49E3.0	fCangjie	NLMWF
U+49E3.0	fFourCorner	71290
U+49E3.0	fCantonese	jan1; zaan1
U+49E3.0	fMandarin	yin1
U+49E3.0	fEnglish	same as &#x9104;, &#x5819;
U+49E4.0	fUTF8	䧤
U+49E4.0	fCNS11643	3-4F59
U+49E4.0	fR/S	 170.12
U+49E4.0	fTotalStrokes	 15
U+49E4.0	fCangjie	NLTCO
U+49E4.0	fFourCorner	72230
U+49E4.0	fCantonese	buk1; buk6
U+49E4.0	fMandarin	bu3; pu2
U+49E4.0	fEnglish	same as &#x6FEE;
U+49E5.0	fUTF8	䧥
U+49E5.0	fCNS11643	3-4F58
U+49E5.0	fR/S	 170.12
U+49E5.0	fTotalStrokes	 15
U+49E5.0	fCangjie	NLJIP
U+49E5.0	fFourCorner	75230
U+49E5.0	fCantonese	wai6
U+49E5.0	fMandarin	hui4
U+49E5.0	fEnglish	border; frontier
U+49E6.0	fUTF8	䧦
U+49E6.0	fCNS11643	7-2A54
U+49E6.0	fR/S	 170.12
U+49E6.0	fTotalStrokes	 15
U+49E6.0	fCangjie	NLBHF
U+49E6.0	fFourCorner	72220
U+49E6.0	fCantonese	wai4
U+49E6.0	fMandarin	wei2
U+49E6.0	fEnglish	the name of a hillside, a slope in today's Henan province
U+49E7.0	fUTF8	䧧
U+49E7.0	fCNS11643	3-5347
U+49E7.0	fR/S	 170.13
U+49E7.0	fTotalStrokes	 16
U+49E7.0	fCangjie	NLTGI
U+49E7.0	fFourCorner	78250
U+49E7.0	fCantonese	ngai5; ai5 (also)
U+49E7.0	fMandarin	yi3
U+49E7.0	fEnglish	same as &#x5DAC;
U+49E8.0	fUTF8	䧨
U+49E8.0	fCNS11643	4-5453
U+49E8.0	fGB	5-746B
U+49E8.0	fR/S	 170.13
U+49E8.0	fTotalStrokes	 16
U+49E8.0	fCangjie	NLTCD
U+49E8.0	fFourCorner	72290
U+49E8.0	fCantonese	jip6; zip6
U+49E8.0	fMandarin	ye4
U+49E8.0	fEnglish	precipitous; steep and lofty of a high mountain
U+49E9.0	fUTF8	䧩
U+49E9.0	fGB	S-214D
U+49E9.0	fR/S	 170.12
U+49E9.0	fTotalStrokes	 15
U+49E9.0	fCangjie	NLNJK
U+49E9.0	fCantonese	gam2
U+49EA.0	fUTF8	䧪
U+49EA.0	fCNS11643	4-5451
U+49EA.0	fGB	3-754A
U+49EA.0	fR/S	 170.13
U+49EA.0	fTotalStrokes	 16
U+49EA.0	fCangjie	NLIKV
U+49EA.0	fFourCorner	70240
U+49EA.0	fCantonese	cip3
U+49EA.0	fMandarin	che4; zhe2
U+49EA.0	fEnglish	sissy; womanish, fawning manner; obsequiousness
U+49EB.0	fUTF8	䧫
U+49EB.0	fGB	5-752E
U+49EB.0	fR/S	 170.14
U+49EB.0	fTotalStrokes	 17
U+49EB.0	fCangjie	NLYRO
U+49EB.0	fFourCorner	7023
U+49EB.0	fHakka	 hau2
U+49EB.0	fCantonese	hou4
U+49EB.0	fMandarin	hao2
U+49EB.0	fEnglish	a trench (in warfare); a ditch, to soar; to fly, to roam
U+49EC.0	fUTF8	䧬
U+49EC.0	fCNS11643	4-5955
U+49EC.0	fR/S	 170.14
U+49EC.0	fTotalStrokes	 17; 16
U+49EC.0	fCangjie	NLJMC
U+49EC.0	fFourCorner	73280
U+49EC.0	fCantonese	ban1; baai1
U+49EC.0	fEnglish	same as &#x7015;, &#x6FF1;
U+49ED.0	fUTF8	䧭
U+49ED.0	fGB	S-2150
U+49ED.0	fR/S	 170.14
U+49ED.0	fTotalStrokes	 17
U+49ED.0	fCangjie	NLHWP
U+49ED.0	fCantonese	si1
U+49EE.0	fUTF8	䧮
U+49EE.0	fCNS11643	3-5B54
U+49EE.0	fR/S	 170.16
U+49EE.0	fTotalStrokes	 19
U+49EE.0	fCangjie	NLJQP
U+49EE.0	fFourCorner	73230
U+49EE.0	fCantonese	hin3
U+49EE.0	fMandarin	xian4; xian3
U+49EE.0	fEnglish	[1] the name of a place [2] same as &#x986F;
U+49EF.0	fUTF8	䧯
U+49EF.0	fCNS11643	4-6473
U+49EF.0	fGB	3-754C
U+49EF.0	fR/S	 170.17
U+49EF.0	fTotalStrokes	 20
U+49EF.0	fCangjie	NLNRI
U+49EF.0	fFourCorner	77210
U+49EF.0	fCantonese	caam4
U+49EF.0	fMandarin	chan2; zhan4
U+49EF.0	fEnglish	[1] the name of a place [2] skin [3] involve; beguile [4] betray
U+49F0.0	fUTF8	䧰
U+49F0.0	fCNS11643	4-6767
U+49F0.0	fGB	3-754B
U+49F0.0	fR/S	 170.18
U+49F0.0	fTotalStrokes	 21
U+49F0.0	fCangjie	NLNFF
U+49F0.0	fFourCorner	72290
U+49F0.0	fCantonese	wan6
U+49F0.0	fMandarin	hun4
U+49F0.0	fEnglish	a mound
U+49F1.0	fUTF8	䧱
U+49F1.0	fCNS11643	3-3438
U+49F1.0	fR/S	 172.2
U+49F1.0	fTotalStrokes	 10
U+49F1.0	fCangjie	KNOG
U+49F1.0	fFourCorner	40010
U+49F1.0	fHakka	 geu1
U+49F1.0	fCantonese	gau1; gaau1; kaau1
U+49F1.0	fMandarin	jiu1
U+49F1.0	fEnglish	same as &#x9CE9;
U+49F2.0	fUTF8	䧲
U+49F2.0	fCNS11643	4-3624
U+49F2.0	fGB	3-335F
U+49F2.0	fR/S	 172.3
U+49F2.0	fTotalStrokes	 11
U+49F2.0	fCangjie	MJOG
U+49F2.0	fFourCorner	10410
U+49F2.0	fCantonese	gon1
U+49F2.0	fMandarin	gan1; han4
U+49F2.0	fEnglish	the magpie, jackdaw; jay and similar birds
U+49F3.0	fUTF8	䧳
U+49F3.0	fCNS11643	3-3F7A
U+49F3.0	fR/S	 172.3
U+49F3.0	fTotalStrokes	 12; 11
U+49F3.0	fCangjie	YMOG
U+49F3.0	fFourCorner	20110
U+49F3.0	fCantonese	ci1; zeoi1
U+49F3.0	fMandarin	ci1; ci2; ci3
U+49F3.0	fEnglish	same as &#x96CC;
U+49F4.0	fUTF8	䧴
U+49F4.0	fCNS11643	3-3F76
U+49F4.0	fGB	5-4644
U+49F4.0	fR/S	 172.4
U+49F4.0	fTotalStrokes	 12
U+49F4.0	fCangjie	JEOG
U+49F4.0	fFourCorner	40410
U+49F4.0	fCantonese	zi1
U+49F4.0	fMandarin	zhi1
U+49F4.0	fEnglish	[1] bird [2] map out or draw up (a plan); a plan or scheme; calculate; count; consider
U+49F5.0	fUTF8	䧵
U+49F5.0	fCNS11643	3-3F79
U+49F5.0	fR/S	 172.4
U+49F5.0	fTotalStrokes	 12
U+49F5.0	fCangjie	LUOG
U+49F5.0	fFourCorner	40010
U+49F5.0	fCantonese	zam6; zaam6; zaan1
U+49F5.0	fMandarin	zhen4; qi2; shen3
U+49F5.0	fEnglish	non-classical form of &#x9D06;
U+49F6.0	fUTF8	䧶
U+49F6.0	fCNS11643	3-3F77
U+49F6.0	fR/S	 172.4
U+49F6.0	fTotalStrokes	 12
U+49F6.0	fCangjie	OGYC
U+49F6.0	fFourCorner	20800
U+49F6.0	fCantonese	kwai4; bak6; but3; git3; kit3
U+49F6.0	fMandarin	kui2
U+49F6.0	fEnglish	[1] care for; concern about affectionately [2] corrupted form of &#x594A;
U+49F7.0	fUTF8	䧷
U+49F7.0	fCNS11643	4-4156
U+49F7.0	fGB	3-7527
U+49F7.0	fR/S	 172.5
U+49F7.0	fTotalStrokes	 13
U+49F7.0	fCangjie	HYOG
U+49F7.0	fFourCorner	20310
U+49F7.0	fHakka	 jung1; rung1
U+49F7.0	fCantonese	jau4
U+49F7.0	fMandarin	rong2; rou2
U+49F7.0	fEnglish	a Chinese family name
U+49F8.0	fUTF8	䧸
U+49F8.0	fCNS11643	3-4544
U+49F8.0	fR/S	 172.5
U+49F8.0	fTotalStrokes	 13
U+49F8.0	fCangjie	JROG
U+49F8.0	fFourCorner	40610
U+49F8.0	fCantonese	gu1; gwu1 (also); zing4
U+49F8.0	fMandarin	gu1
U+49F8.0	fEnglish	same as &#x9D23;
U+49F9.0	fUTF8	䧹
U+49F9.0	fCNS11643	3-4543
U+49F9.0	fGB	5-3D3E
U+49F9.0	fR/S	 172.5
U+49F9.0	fTotalStrokes	 13
U+49F9.0	fCangjie	IOOG
U+49F9.0	fFourCorner	00210
U+49F9.0	fCantonese	jing1; zing1
U+49F9.0	fMandarin	ying1
U+49F9.0	fEnglish	same as &#x9DF9;
U+49FA.0	fUTF8	䧺
U+49FA.0	fCNS11643	3-4545
U+49FA.0	fR/S	 172.5
U+49FA.0	fTotalStrokes	 13
U+49FA.0	fCangjie	KROG
U+49FA.0	fFourCorner	40610
U+49FA.0	fCantonese	hung4
U+49FA.0	fMandarin	xiong2
U+49FA.0	fEnglish	same as &#x96C4;
U+49FB.0	fUTF8	䧻
U+49FB.0	fCNS11643	3-4A48
U+49FB.0	fR/S	 172.6
U+49FB.0	fTotalStrokes	 14
U+49FB.0	fCangjie	OROG
U+49FB.0	fFourCorner	80610
U+49FB.0	fCantonese	gap3
U+49FB.0	fMandarin	ge1
U+49FB.0	fEnglish	same as &#x9D3F;
U+49FC.0	fUTF8	䧼
U+49FC.0	fCNS11643	3-4F5D
U+49FC.0	fR/S	 172.7
U+49FC.0	fTotalStrokes	 15
U+49FC.0	fCangjie	HROG
U+49FC.0	fFourCorner	20610
U+49FC.0	fCantonese	huk6
U+49FC.0	fMandarin	hu2
U+49FC.0	fEnglish	same as &#x9D60;
U+49FD.0	fUTF8	䧽
U+49FD.0	fCNS11643	3-4F5C
U+49FD.0	fR/S	 172.7
U+49FD.0	fTotalStrokes	 15
U+49FD.0	fCangjie	HDOG
U+49FD.0	fFourCorner	20910
U+49FD.0	fCantonese	ceoi2
U+49FD.0	fMandarin	cui3
U+49FD.0	fEnglish	small, thin and slim neck
U+49FE.0	fUTF8	䧾
U+49FE.0	fCNS11643	3-534B
U+49FE.0	fR/S	 172.8
U+49FE.0	fTotalStrokes	 16
U+49FE.0	fCangjie	OROG
U+49FE.0	fFourCorner	80610
U+49FE.0	fCantonese	syu1
U+49FE.0	fMandarin	shu1
U+49FE.0	fEnglish	a kind of bird, like a wild duck
U+49FF.0	fUTF8	䧿
U+49FF.0	fCNS11643	3-534A
U+49FF.0	fGB	5-7454
U+49FF.0	fR/S	 172.8
U+49FF.0	fTotalStrokes	 16
U+49FF.0	fCangjie	TAOG
U+49FF.0	fFourCorner	40610
U+49FF.0	fCantonese	zoek3; coek3; zak3
U+49FF.0	fMandarin	que4
U+49FF.0	fEnglish	same as &#x9D72;
U+4A00.0	fUTF8	䨀
U+4A00.0	fCNS11643	3-5238
U+4A00.0	fR/S	 119.10; 119.8; 172.8
U+4A00.0	fTotalStrokes	 16
U+4A00.0	fCangjie	ODOG
U+4A00.0	fFourCorner	80910
U+4A00.0	fHakka	 tak8; tet8
U+4A00.0	fCantonese	dek6
U+4A00.0	fMandarin	di2; di4
U+4A00.0	fEnglish	same as &#x6B95;, &#x7CF4;
U+4A01.0	fUTF8	䨁
U+4A01.0	fCNS11643	4-5958
U+4A01.0	fGB	5-745B
U+4A01.0	fR/S	 172.9
U+4A01.0	fTotalStrokes	 17; 16
U+4A01.0	fCangjie	NKOG
U+4A01.0	fFourCorner	18210
U+4A01.0	fCantonese	mou6
U+4A01.0	fMandarin	wu4; che4; yu4
U+4A01.0	fEnglish	same as &#x9DA9;
U+4A02.0	fUTF8	䨂
U+4A02.0	fCNS11643	3-5672
U+4A02.0	fGB	5-7459
U+4A02.0	fR/S	 172.9
U+4A02.0	fTotalStrokes	 17
U+4A02.0	fCangjie	HFOG
U+4A02.0	fFourCorner	29210
U+4A02.0	fHakka	 ciu3; ciu1
U+4A02.0	fCantonese	cau1
U+4A02.0	fMandarin	qiu1
U+4A02.0	fEnglish	 chicken
U+4A03.0	fUTF8	䨃
U+4A03.0	fCNS11643	3-594C
U+4A03.0	fR/S	 172.10
U+4A03.0	fTotalStrokes	 18
U+4A03.0	fCangjie	AVOG
U+4A03.0	fFourCorner	60410
U+4A03.0	fCantonese	aan3; ngaan3 (also)
U+4A03.0	fMandarin	yan4
U+4A04.0	fUTF8	䨄
U+4A04.0	fCNS11643	3-5B55
U+4A04.0	fR/S	 172.11
U+4A04.0	fTotalStrokes	 19
U+4A04.0	fCangjie	OWOG
U+4A04.0	fFourCorner	80610
U+4A04.0	fCantonese	am1; ngam1 (also); aam1; an3
U+4A04.0	fMandarin	an1; yan4
U+4A04.0	fEnglish	same as &#x9D6A;
U+4A05.0	fUTF8	䨅
U+4A05.0	fCNS11643	3-5D4A
U+4A05.0	fR/S	 172.12
U+4A05.0	fTotalStrokes	 20
U+4A05.0	fCangjie	KFOG
U+4A05.0	fFourCorner	40910
U+4A05.0	fHakka	 liau5
U+4A05.0	fCantonese	liu4
U+4A05.0	fMandarin	liao2
U+4A05.0	fEnglish	same as &#x9DEF;
U+4A06.0	fUTF8	䨆
U+4A06.0	fCNS11643	3-5B56
U+4A06.0	fR/S	 172.12
U+4A06.0	fTotalStrokes	 20; 19
U+4A06.0	fCangjie	FKOG
U+4A06.0	fFourCorner	98210
U+4A06.0	fCantonese	bai3
U+4A06.0	fMandarin	bi4; bi2; xu4
U+4A06.0	fEnglish	same as &#x9DE9;
U+4A07.0	fUTF8	䨇
U+4A07.0	fCNS11643	3-5D4B
U+4A07.0	fR/S	 172.12
U+4A07.0	fTotalStrokes	 20
U+4A07.0	fCangjie	OGEE
U+4A07.0	fFourCorner	20440
U+4A07.0	fHakka	  sung1
U+4A07.0	fCantonese	soeng1
U+4A07.0	fMandarin	shuang1
U+4A07.0	fEnglish	 pair; double; even
U+4A08.0	fUTF8	䨈
U+4A08.0	fCNS11643	4-6956
U+4A08.0	fGB	3-7529
U+4A08.0	fR/S	 172.14
U+4A08.0	fTotalStrokes	 22; 21
U+4A08.0	fCangjie	JCOG
U+4A08.0	fFourCorner	30810
U+4A08.0	fCantonese	ban1
U+4A08.0	fMandarin	bin1
U+4A08.0	fEnglish	a small bird
U+4A09.0	fUTF8	䨉
U+4A09.0	fCNS11643	3-6151
U+4A09.0	fR/S	 172.17
U+4A09.0	fTotalStrokes	 25
U+4A09.0	fCangjie	BVOG
U+4A09.0	fFourCorner	60410
U+4A09.0	fCantonese	jing1; zing1
U+4A09.0	fMandarin	ying1
U+4A09.0	fEnglish	same as &#x9E1A;
U+4A0A.0	fUTF8	䨊
U+4A0A.0	fCNS11643	3-6240
U+4A0A.0	fR/S	 172.24
U+4A0A.0	fTotalStrokes	 32
U+4A0A.0	fCangjie	OGOGL
U+4A0A.0	fFourCorner	20220
U+4A0A.0	fCantonese	jyun1
U+4A0A.0	fMandarin	yuan1
U+4A0A.0	fEnglish	a flock of birds
U+4A0B.0	fUTF8	䨋
U+4A0B.0	fCNS11643	3-3966
U+4A0B.0	fR/S	 173.3
U+4A0B.0	fTotalStrokes	 11
U+4A0B.0	fCangjie	MBHP
U+4A0B.0	fFourCorner	10710
U+4A0B.0	fCantonese	joek6; zoek6
U+4A0B.0	fMandarin	nuue4; ne4
U+4A0B.0	fEnglish	same as &#x8650;
U+4A0C.0	fUTF8	䨌
U+4A0C.0	fCNS11643	3-3F7D
U+4A0C.0	fR/S	 173.4
U+4A0C.0	fTotalStrokes	 12
U+4A0C.0	fCangjie	MBMMU
U+4A0C.0	fFourCorner	10210
U+4A0C.0	fCantonese	bok6; fu6
U+4A0C.0	fMandarin	bo2; bao2
U+4A0C.0	fEnglish	same as &#x96F9;
U+4A0D.0	fUTF8	䨍
U+4A0D.0	fCNS11643	6-5031
U+4A0D.0	fR/S	 173.4
U+4A0D.0	fTotalStrokes	 12
U+4A0D.0	fCangjie	MBTT
U+4A0D.0	fFourCorner	10550
U+4A0D.0	fCantonese	jing2; zing4
U+4A0D.0	fMandarin	ying3
U+4A0D.0	fEnglish	deep pond; deep pool
U+4A0E.0	fUTF8	䨎
U+4A0E.0	fCNS11643	3-4548
U+4A0E.0	fR/S	 173.5
U+4A0E.0	fTotalStrokes	 13
U+4A0E.0	fCangjie	MBNI
U+4A0E.0	fFourCorner	10230
U+4A0E.0	fCantonese	wang4
U+4A0E.0	fMandarin	weng1; hong2
U+4A0E.0	fEnglish	same as &#x9710;
U+4A0F.0	fUTF8	䨏
U+4A0F.0	fCNS11643	4-482D
U+4A0F.0	fGB	3-743F
U+4A0F.0	fR/S	 173.6
U+4A0F.0	fTotalStrokes	 14
U+4A0F.0	fCangjie	MBIMO
U+4A0F.0	fFourCorner	10180
U+4A0F.0	fCantonese	ci4; cek3; maan4
U+4A0F.0	fMandarin	ci2
U+4A0F.0	fEnglish	[1] the sound of raining [2] a great fall of rain; heavy rain; it rained large drops
U+4A10.0	fUTF8	䨐
U+4A10.0	fCNS11643	4-4831
U+4A10.0	fGB	5-7349
U+4A10.0	fR/S	 173.6
U+4A10.0	fTotalStrokes	 14
U+4A10.0	fCangjie	MBOMR
U+4A10.0	fFourCorner	10600
U+4A10.0	fCantonese	hap6; haap1
U+4A10.0	fMandarin	xia2; qia4
U+4A10.0	fEnglish	same as &#x6D3D;
U+4A11.0	fUTF8	䨑
U+4A11.0	fCNS11643	4-482E
U+4A11.0	fGB	3-743E
U+4A11.0	fR/S	 173.6
U+4A11.0	fTotalStrokes	 14
U+4A11.0	fCangjie	MBKN
U+4A11.0	fFourCorner	10530
U+4A11.0	fHakka	 ti2
U+4A11.0	fCantonese	tai4
U+4A11.0	fMandarin	ti2
U+4A11.0	fEnglish	foul weather changing to fair
U+4A12.0	fUTF8	䨒
U+4A12.0	fCNS11643	3-4A4A
U+4A12.0	fGB	5-734C
U+4A12.0	fR/S	 173.6
U+4A12.0	fTotalStrokes	 14
U+4A12.0	fCangjie	MBSMM
U+4A12.0	fFourCorner	10120
U+4A12.0	fCantonese	jyu6; wu6
U+4A12.0	fMandarin	yu3; yu4
U+4A12.0	fEnglish	[1] the sound of the water drops [2] raining [3] acquired dialects [4] interchangeable with &#x7FBD;
U+4A13.0	fUTF8	䨓
U+4A13.0	fCNS11643	4-4834
U+4A13.0	fR/S	 173.6
U+4A13.0	fTotalStrokes	 14
U+4A13.0	fCangjie	MBWR
U+4A13.0	fFourCorner	10600
U+4A13.0	fCantonese	leoi4
U+4A13.0	fMandarin	lei2; lei4
U+4A13.0	fEnglish	ancient form of &#x9741;, &#x96F7;
U+4A14.0	fUTF8	䨔
U+4A14.0	fCNS11643	3-4A4B
U+4A14.0	fR/S	 173.6
U+4A14.0	fTotalStrokes	 14
U+4A14.0	fCangjie	MBFMU
U+4A14.0	fFourCorner	10210
U+4A14.0	fCantonese	bok6
U+4A14.0	fMandarin	bao2
U+4A14.0	fEnglish	[1] corrupted form of &#x4A0C; [2] same as &#x96F9;
U+4A15.0	fUTF8	䨕
U+4A15.0	fCNS11643	4-4833
U+4A15.0	fGB	3-7440
U+4A15.0	fR/S	 173.6
U+4A15.0	fTotalStrokes	 14
U+4A15.0	fCangjie	MBEMD
U+4A15.0	fFourCorner	10141
U+4A15.0	fCantonese	jyu1; wu1
U+4A15.0	fMandarin	yu1
U+4A15.0	fEnglish	(variant form) pouring with rain
U+4A16.0	fUTF8	䨖
U+4A16.0	fCNS11643	6-637B
U+4A16.0	fR/S	 173.6
U+4A16.0	fTotalStrokes	 14
U+4A16.0	fCangjie	MBKB
U+4A16.0	fFourCorner	10220
U+4A16.0	fCantonese	zai3
U+4A16.0	fMandarin	ji4
U+4A16.0	fEnglish	non-classical form of &#x973D;
U+4A17.0	fUTF8	䨗
U+4A17.0	fCNS11643	3-4F5E
U+4A17.0	fR/S	 173.7
U+4A17.0	fTotalStrokes	 15
U+4A17.0	fCangjie	MBBND
U+4A17.0	fFourCorner	10400
U+4A17.0	fCantonese	fau4; fu1
U+4A17.0	fMandarin	fu2
U+4A17.0	fEnglish	excessive rain and snow
U+4A18.0	fUTF8	䨘
U+4A18.0	fCNS11643	3-4F61
U+4A18.0	fR/S	 173.7
U+4A18.0	fTotalStrokes	 15
U+4A18.0	fCangjie	MBBUU
U+4A18.0	fFourCorner	10210
U+4A18.0	fHakka	 sien5
U+4A18.0	fCantonese	sin3
U+4A18.0	fMandarin	xian4
U+4A18.0	fEnglish	same as &#x4A37;, &#x9730;
U+4A19.0	fUTF8	䨙
U+4A19.0	fCNS11643	3-4F5F
U+4A19.0	fR/S	 173.7
U+4A19.0	fTotalStrokes	 15
U+4A19.0	fCangjie	MBEHG
U+4A19.0	fFourCorner	10110
U+4A19.0	fCantonese	jam4; zaam4
U+4A19.0	fMandarin	yin2; cen2; ya4
U+4A19.0	fEnglish	same as &#x972A;
U+4A1A.0	fUTF8	䨚
U+4A1A.0	fGB	3-7447
U+4A1A.0	fR/S	 173.8
U+4A1A.0	fTotalStrokes	 16
U+4A1A.0	fCangjie	MBPHP
U+4A1A.0	fFourCorner	1033
U+4A1A.0	fCantonese	fat1; faat1
U+4A1A.0	fEnglish	rain
U+4A1B.0	fUTF8	䨛
U+4A1B.0	fCNS11643	4-5459
U+4A1B.0	fGB	3-7445
U+4A1B.0	fR/S	 173.8
U+4A1B.0	fTotalStrokes	 16
U+4A1B.0	fCangjie	MBDHL
U+4A1B.0	fFourCorner	10920
U+4A1B.0	fCantonese	saak3; sik1
U+4A1B.0	fMandarin	suo3; se4
U+4A1B.0	fEnglish	[1] rain [2] snowflakes [3] light rain; drizzle
U+4A1C.0	fUTF8	䨜
U+4A1C.0	fCNS11643	3-534F
U+4A1C.0	fGB	5-7354
U+4A1C.0	fR/S	 173.8
U+4A1C.0	fTotalStrokes	 16
U+4A1C.0	fCangjie	MBBB
U+4A1C.0	fFourCorner	10220
U+4A1C.0	fCantonese	bang1; baang1; wik6
U+4A1C.0	fMandarin	peng2
U+4A1C.0	fEnglish	[1] pouring rain; rain cats and dogs [2] completely [3] with leaving a trace [4] dull; stupid; doltish
U+4A1D.0	fUTF8	䨝
U+4A1D.0	fCNS11643	3-534E
U+4A1D.0	fR/S	 173.8
U+4A1D.0	fTotalStrokes	 16
U+4A1D.0	fCangjie	MBQMB
U+4A1D.0	fFourCorner	10220
U+4A1D.0	fCantonese	cing1
U+4A1D.0	fMandarin	qing1
U+4A1D.0	fEnglish	 pure woman; the name of a goddess of frost and snow
U+4A1E.0	fUTF8	䨞
U+4A1E.0	fCNS11643	4-5960
U+4A1E.0	fGB	3-744A
U+4A1E.0	fR/S	 173.9
U+4A1E.0	fTotalStrokes	 17; 16
U+4A1E.0	fCangjie	MBHLB
U+4A1E.0	fFourCorner	10420
U+4A1E.0	fCantonese	jyu5; zyu3
U+4A1E.0	fMandarin	yu3; yu4
U+4A1E.0	fEnglish	[1] rain [2] stretch; open; relax; unfold [3] slow; unhurried; leisurely
U+4A1F.0	fUTF8	䨟
U+4A1F.0	fCNS11643	3-5673
U+4A1F.0	fR/S	 173.9
U+4A1F.0	fTotalStrokes	 17
U+4A1F.0	fCangjie	MBEGG
U+4A1F.0	fFourCorner	10110
U+4A1F.0	fCantonese	waa1
U+4A1F.0	fMandarin	wa1
U+4A1F.0	fEnglish	[1] puddle (water in the hoof's track) [2] same as &#x7AAA;
U+4A20.0	fUTF8	䨠
U+4A20.0	fCNS11643	3-5677
U+4A20.0	fR/S	 173.9
U+4A20.0	fTotalStrokes	 17
U+4A20.0	fCangjie	MBAPV
U+4A20.0	fFourCorner	10720
U+4A20.0	fCantonese	oi3; ngoi3 (also); oi2; piu5
U+4A20.0	fMandarin	ai4; ai3
U+4A20.0	fEnglish	same as &#x9744;
U+4A21.0	fUTF8	䨡
U+4A21.0	fCNS11643	4-5458
U+4A21.0	fGB	3-7448
U+4A21.0	fR/S	 173.8; 173.9
U+4A21.0	fTotalStrokes	 16; 17
U+4A21.0	fCangjie	MBUNE
U+4A21.0	fFourCorner	10770
U+4A21.0	fCantonese	ham4
U+4A21.0	fMandarin	han2
U+4A21.0	fEnglish	rained for a long time; too much rain
U+4A22.0	fUTF8	䨢
U+4A22.0	fCNS11643	3-5676
U+4A22.0	fGB	5-7357
U+4A22.0	fR/S	 173.9
U+4A22.0	fTotalStrokes	 17
U+4A22.0	fCangjie	MBTMV
U+4A22.0	fFourCorner	10710
U+4A22.0	fCantonese	daam6
U+4A22.0	fMandarin	dan4; di2; gao4; tan2
U+4A22.0	fEnglish	same as &#x972E;
U+4A23.0	fUTF8	䨣
U+4A23.0	fCNS11643	4-595D
U+4A23.0	fGB	3-7449
U+4A23.0	fR/S	 173.9
U+4A23.0	fTotalStrokes	 17; 16
U+4A23.0	fCangjie	MBTLJ
U+4A23.0	fFourCorner	10500
U+4A23.0	fCantonese	gaak3
U+4A23.0	fMandarin	po4; ge2; geng4
U+4A23.0	fEnglish	leather to be moistened by rain
U+4A24.0	fUTF8	䨤
U+4A24.0	fCNS11643	4-595F
U+4A24.0	fGB	3-7446
U+4A24.0	fR/S	 173.9
U+4A24.0	fTotalStrokes	 16; 17
U+4A24.0	fCangjie	MBYLW
U+4A24.0	fFourCorner	10300
U+4A24.0	fCantonese	dik6
U+4A24.0	fMandarin	di2
U+4A24.0	fEnglish	 rain
U+4A25.0	fUTF8	䨥
U+4A25.0	fCNS11643	4-5E35
U+4A25.0	fGB	3-744C
U+4A25.0	fR/S	 173.10
U+4A25.0	fTotalStrokes	 18
U+4A25.0	fCangjie	MBOGE
U+4A25.0	fFourCorner	10400
U+4A25.0	fCantonese	wok6; soeng1
U+4A25.0	fMandarin	huo4; hu4
U+4A25.0	fEnglish	[1] pouring rain; rain hard; rain cats and dogs [2] pair; couple [3] two; both
U+4A26.0	fUTF8	䨦
U+4A26.0	fCNS11643	3-594D
U+4A26.0	fGB	5-7363
U+4A26.0	fR/S	 173.10
U+4A26.0	fTotalStrokes	 18
U+4A26.0	fCangjie	MBYBS
U+4A26.0	fFourCorner	10220
U+4A26.0	fCantonese	pong4
U+4A26.0	fMandarin	pang2
U+4A26.0	fEnglish	same as &#x96F1;
U+4A27.0	fUTF8	䨧
U+4A27.0	fCNS11643	3-594E
U+4A27.0	fR/S	 173.10
U+4A27.0	fTotalStrokes	 18
U+4A27.0	fCangjie	MBYTI
U+4A27.0	fFourCorner	10130
U+4A27.0	fCantonese	jam1; zaam1
U+4A27.0	fMandarin	yin1
U+4A27.0	fEnglish	[1] standard form of &#x9720; [2] corrupted form of &#x9712;
U+4A28.0	fUTF8	䨨
U+4A28.0	fCNS11643	4-5E36
U+4A28.0	fGB	5-735B
U+4A28.0	fR/S	 173.10
U+4A28.0	fTotalStrokes	 18
U+4A28.0	fCangjie	MBYHR
U+4A28.0	fFourCorner	10300
U+4A28.0	fCantonese	zeoi1; zaa1
U+4A28.0	fMandarin	zhui1
U+4A28.0	fEnglish	[1] thunder [2] hidden; concealed; secret [3] same as &#x9824;
U+4A29.0	fUTF8	䨩
U+4A29.0	fCNS11643	7-4458
U+4A29.0	fR/S	 173.10
U+4A29.0	fTotalStrokes	 18
U+4A29.0	fCangjie	MBIFL
U+4A29.0	fFourCorner	10250
U+4A29.0	fCantonese	ling4
U+4A29.0	fMandarin	ling2
U+4A29.0	fEnglish	same as &#x9748;
U+4A2A.0	fUTF8	䨪
U+4A2A.0	fCNS11643	7-4460
U+4A2A.0	fR/S	 173.10
U+4A2A.0	fTotalStrokes	 18
U+4A2A.0	fCangjie	MBGWG
U+4A2A.0	fFourCorner	10110
U+4A2A.0	fCantonese	maai4; mai4; mo1
U+4A2A.0	fMandarin	mai2
U+4A2A.0	fEnglish	same as &#x973E;
U+4A2B.0	fUTF8	䨫
U+4A2B.0	fCNS11643	4-6174
U+4A2B.0	fR/S	 173.11
U+4A2B.0	fTotalStrokes	 19
U+4A2B.0	fCangjie	MBJON
U+4A2B.0	fFourCorner	10407
U+4A2B.0	fCantonese	mak6; maak6; maan6
U+4A2B.0	fMandarin	mai4; man4
U+4A2B.0	fEnglish	interchangeable with &#x9722;
U+4A2C.0	fUTF8	䨬
U+4A2C.0	fCNS11643	4-6173
U+4A2C.0	fR/S	 173.11
U+4A2C.0	fTotalStrokes	 19
U+4A2C.0	fCangjie	MBEDD
U+4A2C.0	fFourCorner	10190
U+4A2C.0	fHakka	  lam2
U+4A2C.0	fCantonese	lim4
U+4A2C.0	fMandarin	lan2; lian2
U+4A2C.0	fEnglish	rain falling
U+4A2D.0	fUTF8	䨭
U+4A2D.0	fCNS11643	3-5B57
U+4A2D.0	fR/S	 173.11
U+4A2D.0	fTotalStrokes	 19
U+4A2D.0	fCangjie	MIFB
U+4A2D.0	fFourCorner	19720
U+4A2D.0	fCantonese	siu1; tou4
U+4A2D.0	fMandarin	xiao1
U+4A2D.0	fEnglish	same as &#x9704;
U+4A2E.0	fUTF8	䨮
U+4A2E.0	fCNS11643	3-5B58
U+4A2E.0	fR/S	 173.11
U+4A2E.0	fTotalStrokes	 19
U+4A2E.0	fCangjie	MBQJS
U+4A2E.0	fFourCorner	10170
U+4A2E.0	fCantonese	syut3
U+4A2E.0	fMandarin	xue3
U+4A2E.0	fEnglish	same as &#x96EA;
U+4A2F.0	fUTF8	䨯
U+4A2F.0	fCNS11643	3-5B59
U+4A2F.0	fR/S	 173.11
U+4A2F.0	fTotalStrokes	 19
U+4A2F.0	fCangjie	MBNLW
U+4A2F.0	fFourCorner	10290
U+4A2F.0	fCantonese	zan6; zaan6; zang1
U+4A2F.0	fMandarin	zhen4
U+4A2F.0	fEnglish	cloudy
U+4A30.0	fUTF8	䨰
U+4A30.0	fCNS11643	4-6475
U+4A30.0	fGB	5-736C
U+4A30.0	fR/S	 173.12
U+4A30.0	fTotalStrokes	 20
U+4A30.0	fCangjie	MBETJ
U+4A30.0	fFourCorner	10150
U+4A30.0	fCantonese	pok3; bun2; fuk6; pak3
U+4A30.0	fMandarin	po4
U+4A30.0	fEnglish	[1] rain [2] same as &#x6FFC;
U+4A31.0	fUTF8	䨱
U+4A31.0	fCNS11643	3-5D4D
U+4A31.0	fR/S	 173.12
U+4A31.0	fTotalStrokes	 20
U+4A31.0	fCangjie	MBHOE
U+4A31.0	fFourCorner	10240
U+4A31.0	fCantonese	fuk1; fuk6
U+4A31.0	fMandarin	fu4
U+4A31.0	fEnglish	[1] spilt water [2] pour out water
U+4A32.0	fUTF8	䨲
U+4A32.0	fCNS11643	4-6958
U+4A32.0	fGB	3-7453
U+4A32.0	fR/S	 173.14
U+4A32.0	fTotalStrokes	 22; 21
U+4A32.0	fCangjie	MBNAI
U+4A32.0	fFourCorner	17210
U+4A32.0	fHakka	 neu2; wan5
U+4A32.0	fCantonese	nau4; lau4 (also)
U+4A32.0	fMandarin	nou2
U+4A32.0	fEnglish	an old name for a rabbit
U+4A33.0	fUTF8	䨳
U+4A33.0	fCNS11643	4-695A
U+4A33.0	fGB	5-7352
U+4A33.0	fR/S	 173.14
U+4A33.0	fTotalStrokes	 22; 21
U+4A33.0	fCangjie	MBBUU
U+4A33.0	fFourCorner	10210
U+4A33.0	fCantonese	hei3; hei1
U+4A33.0	fMandarin	xi4
U+4A33.0	fEnglish	the rain stopped
U+4A34.0	fUTF8	䨴
U+4A34.0	fCNS11643	4-695B
U+4A34.0	fGB	3-743D
U+4A34.0	fR/S	 173.14
U+4A34.0	fTotalStrokes	 22; 21
U+4A34.0	fCangjie	MBTGI
U+4A34.0	fFourCorner	10140
U+4A34.0	fCantonese	deoi6; deoi3 (also); hau1
U+4A34.0	fMandarin	dui4; weng4
U+4A34.0	fEnglish	gathering clouds
U+4A35.0	fUTF8	䨵
U+4A35.0	fCNS11643	3-6058
U+4A35.0	fR/S	 173.15
U+4A35.0	fTotalStrokes	 23
U+4A35.0	fCangjie	MBEMJ
U+4A35.0	fFourCorner	10140
U+4A35.0	fCantonese	daam6; tam4
U+4A35.0	fMandarin	tan4; dan4
U+4A35.0	fEnglish	rained for a long time
U+4A36.0	fUTF8	䨶
U+4A36.0	fCNS11643	3-6153
U+4A36.0	fR/S	 173.15
U+4A36.0	fTotalStrokes	 23; 25
U+4A36.0	fCangjie	MBIBL
U+4A36.0	fFourCorner	10220
U+4A36.0	fCantonese	wan5
U+4A36.0	fMandarin	yun3
U+4A36.0	fEnglish	[1] ancient form of &#x9723; [2] interchangeable with &#x9695;, &#x6B9E;
U+4A37.0	fUTF8	䨷
U+4A37.0	fCNS11643	3-6127
U+4A37.0	fR/S	 173.16
U+4A37.0	fTotalStrokes	 24
U+4A37.0	fCangjie	MBDBE
U+4A37.0	fFourCorner	10240
U+4A37.0	fCantonese	sin3; sin4
U+4A37.0	fMandarin	xian4
U+4A37.0	fEnglish	same as &#x9730;
U+4A38.0	fUTF8	䨸
U+4A38.0	fCNS11643	3-6152
U+4A38.0	fR/S	 173.17
U+4A38.0	fTotalStrokes	 25
U+4A38.0	fCangjie	MBNLP
U+4A38.0	fFourCorner	10230
U+4A38.0	fCantonese	jan2; zaan2; zaang6
U+4A38.0	fMandarin	yin3
U+4A38.0	fEnglish	cloudy
U+4A39.0	fUTF8	䨹
U+4A39.0	fCNS11643	3-6224
U+4A39.0	fGB	5-7376
U+4A39.0	fR/S	 173.19; 173.18
U+4A39.0	fTotalStrokes	 27; 26
U+4A39.0	fCangjie	MBOLF
U+4A39.0	fFourCorner	10230
U+4A39.0	fCantonese	suk1
U+4A39.0	fMandarin	shu2
U+4A39.0	fEnglish	swift; rapid; quick; fast
U+4A3A.0	fUTF8	䨺
U+4A3A.0	fCNS11643	3-6244
U+4A3A.0	fR/S	 173.28
U+4A3A.0	fTotalStrokes	 36
U+4A3A.0	fCangjie	MIMII
U+4A3A.0	fFourCorner	10730
U+4A3A.0	fCantonese	deoi6
U+4A3A.0	fMandarin	dui4; zhui4
U+4A3A.0	fEnglish	cloudy
U+4A3B.0	fUTF8	䨻
U+4A3B.0	fCNS11643	3-6246
U+4A3B.0	fR/S	 173.44
U+4A3B.0	fTotalStrokes	 52
U+4A3B.0	fCangjie	MWMWW
U+4A3B.0	fFourCorner	11660
U+4A3B.0	fCantonese	bang6
U+4A3B.0	fMandarin	beng4
U+4A3B.0	fEnglish	roar of thunders; thunderpeal, very loud sounds
U+4A3C.0	fUTF8	䨼
U+4A3C.0	fCNS11643	4-695C
U+4A3C.0	fGB	3-7439
U+4A3C.0	fR/S	 174.14
U+4A3C.0	fTotalStrokes	 22; 21
U+4A3C.0	fCangjie	QBTOE
U+4A3C.0	fFourCorner	54240
U+4A3C.0	fCantonese	wu6
U+4A3C.0	fMandarin	hu4
U+4A3C.0	fEnglish	dyestuff (of dark green)
U+4A3D.0	fUTF8	䨽
U+4A3D.0	fCNS11643	4-3628
U+4A3D.0	fGB	3-7457
U+4A3D.0	fR/S	 175.3
U+4A3D.0	fTotalStrokes	 11
U+4A3D.0	fCangjie	LYSU
U+4A3D.0	fFourCorner	11710
U+4A3D.0	fCantonese	fei2
U+4A3D.0	fMandarin	fei3
U+4A3D.0	fEnglish	to resolve, to dissolve; to decompose, to analyze, to part, to differ, different, a bird ( like a legendary bird with man's face and one leg)
U+4A3E.0	fUTF8	䨾
U+4A3E.0	fCNS11643	4-362A
U+4A3E.0	fGB	3-3A5E
U+4A3E.0	fR/S	 175.3
U+4A3E.0	fTotalStrokes	 11
U+4A3E.0	fCangjie	ILMY
U+4A3E.0	fFourCorner	00210
U+4A3E.0	fCantonese	fai6; fei2
U+4A3E.0	fMandarin	fei4; fei3
U+4A3E.0	fEnglish	hidden; concealed; secret, narrow and small, ugly, low; mean, inferior
U+4A3F.0	fUTF8	䨿
U+4A3F.0	fCNS11643	4-3B74
U+4A3F.0	fGB	3-7459
U+4A3F.0	fR/S	 175.4
U+4A3F.0	fTotalStrokes	 12
U+4A3F.0	fCangjie	MKLMY
U+4A3F.0	fFourCorner	20111
U+4A3F.0	fHakka	  cap8;  sap7
U+4A3F.0	fCantonese	zaap6
U+4A3F.0	fMandarin	za2; qian2
U+4A3F.0	fEnglish	bad; evil; wicked; vice; wickedness
U+4A40.0	fUTF8	䩀
U+4A40.0	fCNS11643	4-4E4C
U+4A40.0	fR/S	 175.7
U+4A40.0	fTotalStrokes	 15
U+4A40.0	fCangjie	LYBUC
U+4A40.0	fFourCorner	11800
U+4A40.0	fCantonese	bei6; bei3
U+4A40.0	fMandarin	bi4; bei4
U+4A41.0	fUTF8	䩁
U+4A41.0	fCNS11643	4-647B
U+4A41.0	fGB	3-745F
U+4A41.0	fR/S	 175.12
U+4A41.0	fTotalStrokes	 20
U+4A41.0	fCangjie	LYHUU
U+4A41.0	fFourCorner	11710
U+4A41.0	fCantonese	fei1
U+4A41.0	fMandarin	fei1
U+4A41.0	fEnglish	thin hairs; fine hairs
U+4A42.0	fUTF8	䩂
U+4A42.0	fCNS11643	4-415A
U+4A42.0	fGB	3-7960
U+4A42.0	fR/S	 176.4
U+4A42.0	fTotalStrokes	 13
U+4A42.0	fCangjie	MWOIN
U+4A42.0	fFourCorner	18620
U+4A42.0	fCantonese	haam1
U+4A42.0	fMandarin	xian1
U+4A42.0	fEnglish	to come to the front; to take the lead; to bear responsibility, to make good or to succeed (said of people of humble origin)
U+4A43.0	fUTF8	䩃
U+4A43.0	fCNS11643	3-454A
U+4A43.0	fR/S	 176.4
U+4A43.0	fTotalStrokes	 13
U+4A43.0	fCangjie	MWPP
U+4A43.0	fFourCorner	11610
U+4A43.0	fCantonese	si6
U+4A43.0	fMandarin	shi4
U+4A43.0	fEnglish	appearance; a person's features
U+4A44.0	fUTF8	䩄
U+4A44.0	fGB	S-237E
U+4A44.0	fR/S	 176.4
U+4A44.0	fTotalStrokes	 13
U+4A44.0	fCangjie	MWBHU
U+4A44.0	fCantonese	tin2; min5
U+4A44.0	fMandarin	tian3
U+4A44.0	fEnglish	simplified form of &#x9766;
U+4A45.0	fUTF8	䩅
U+4A45.0	fCNS11643	3-4A4D
U+4A45.0	fR/S	 176.5; 176.9; 107.9
U+4A45.0	fTotalStrokes	 14
U+4A45.0	fCangjie	MWDHE
U+4A45.0	fFourCorner	14640
U+4A45.0	fCantonese	zin2
U+4A45.0	fMandarin	zhan3
U+4A45.0	fEnglish	[1] broad; wide; spacious; vast [2] turn red from shame or embarrassment; to blush, to be ashamed
U+4A46.0	fUTF8	䩆
U+4A46.0	fCNS11643	4-4838
U+4A46.0	fGB	3-7962
U+4A46.0	fR/S	 176.5
U+4A46.0	fTotalStrokes	 14
U+4A46.0	fCangjie	MWOS
U+4A46.0	fFourCorner	18610
U+4A46.0	fCantonese	zaan2
U+4A46.0	fMandarin	zha4; jian3; zhan3
U+4A46.0	fEnglish	old, wrinkles; creases, ashamed, shamed-faced look
U+4A47.0	fUTF8	䩇
U+4A47.0	fCNS11643	4-4837
U+4A47.0	fGB	3-7961
U+4A47.0	fR/S	 176.5
U+4A47.0	fTotalStrokes	 14
U+4A47.0	fCangjie	MWYR
U+4A47.0	fFourCorner	11660
U+4A47.0	fCantonese	zaam1
U+4A47.0	fMandarin	zhan1
U+4A47.0	fEnglish	a small head, to come to the front; to take the lead; to bear responsibility, to make good or to succeed (said of people of humble origin), an ugly face
U+4A48.0	fUTF8	䩈
U+4A48.0	fCNS11643	4-545D
U+4A48.0	fGB	3-7963
U+4A48.0	fR/S	 176.7
U+4A48.0	fTotalStrokes	 16
U+4A48.0	fCangjie	MWOWY
U+4A48.0	fFourCorner	18650
U+4A48.0	fHakka	 fi5; fui5
U+4A48.0	fCantonese	fui3; bik1
U+4A48.0	fMandarin	hui4; wei4; xue3
U+4A48.0	fEnglish	a plumpy; fat face
U+4A49.0	fUTF8	䩉
U+4A49.0	fCNS11643	4-545C
U+4A49.0	fGB	5-7765
U+4A49.0	fR/S	 176.7
U+4A49.0	fTotalStrokes	 16
U+4A49.0	fCangjie	MWIJB
U+4A49.0	fFourCorner	13620
U+4A49.0	fCantonese	fu6; fu2
U+4A49.0	fMandarin	fu3
U+4A49.0	fEnglish	[1] the face [2] the cheeks; the jaw
U+4A4A.0	fUTF8	䩊
U+4A4A.0	fCNS11643	4-5962
U+4A4A.0	fGB	3-7964
U+4A4A.0	fR/S	 176.8
U+4A4A.0	fTotalStrokes	 17; 16
U+4A4A.0	fCangjie	MWJNU
U+4A4A.0	fFourCorner	13610
U+4A4A.0	fCantonese	jyun2; zyun2
U+4A4A.0	fMandarin	wan3; wo4
U+4A4A.0	fEnglish	interchangeable with &#x5A49;
U+4A4B.0	fUTF8	䩋
U+4A4B.0	fCNS11643	4-647C
U+4A4B.0	fGB	3-7966
U+4A4B.0	fR/S	 176.11
U+4A4B.0	fTotalStrokes	 20
U+4A4B.0	fCangjie	MWIDD
U+4A4B.0	fFourCorner	10690
U+4A4B.0	fCantonese	mo5; mo1
U+4A4B.0	fMandarin	mo3
U+4A4B.0	fEnglish	[1] a green face; look aghast [2] interchangeable U+61E1) ashamed
U+4A4C.0	fUTF8	䩌
U+4A4C.0	fCNS11643	4-676E
U+4A4C.0	fGB	3-7967
U+4A4C.0	fR/S	 176.12
U+4A4C.0	fTotalStrokes	 21
U+4A4C.0	fCangjie	MWOGF
U+4A4C.0	fFourCorner	10630
U+4A4C.0	fHakka	 ziau1; zeu1
U+4A4C.0	fCantonese	ciu4
U+4A4C.0	fMandarin	qiao2
U+4A4C.0	fEnglish	 shrivelled-up face
U+4A4D.0	fUTF8	䩍
U+4A4D.0	fCNS11643	5-7163
U+4A4D.0	fR/S	 176.12
U+4A4D.0	fTotalStrokes	 21
U+4A4D.0	fCangjie	MWKCF
U+4A4D.0	fFourCorner	14690
U+4A4D.0	fCantonese	liu5; liu4
U+4A4D.0	fMandarin	liao3
U+4A4D.0	fEnglish	 look pale or pallid
U+4A4E.0	fUTF8	䩎
U+4A4E.0	fGB	S-2421
U+4A4E.0	fR/S	 176.13
U+4A4E.0	fTotalStrokes	 22
U+4A4E.0	fCangjie	MWOMO
U+4A4E.0	fCantonese	cim1
U+4A4F.0	fUTF8	䩏
U+4A4F.0	fCNS11643	4-6C33
U+4A4F.0	fGB	3-7968
U+4A4F.0	fR/S	 176.15
U+4A4F.0	fTotalStrokes	 24
U+4A4F.0	fCangjie	MWTWI
U+4A4F.0	fFourCorner	14650
U+4A4F.0	fCantonese	mit6
U+4A4F.0	fMandarin	mie4
U+4A4F.0	fEnglish	[1] small face [2] small [3] young [4] junior
U+4A50.0	fUTF8	䩐
U+4A50.0	fCNS11643	4-3B79
U+4A50.0	fGB	5-7724
U+4A50.0	fR/S	 177.3
U+4A50.0	fTotalStrokes	 12
U+4A50.0	fCangjie	TJON
U+4A50.0	fFourCorner	48510
U+4A50.0	fCantonese	fat1; faat1; gaap1
U+4A50.0	fMandarin	hu1; ge2; ji2
U+4A50.0	fEnglish	[1] tie it tightly [2] lead an ox [3] wart; pustule; pimple [4] round lump
U+4A51.0	fUTF8	䩑
U+4A51.0	fCNS11643	3-4023
U+4A51.0	fR/S	 177.3
U+4A51.0	fTotalStrokes	 12
U+4A51.0	fCangjie	TJN
U+4A51.0	fFourCorner	47520
U+4A51.0	fCantonese	wang4
U+4A51.0	fMandarin	hong2
U+4A51.0	fEnglish	same as &#x9783;
U+4A52.0	fUTF8	䩒
U+4A52.0	fCNS11643	4-3B76
U+4A52.0	fGB	3-787C
U+4A52.0	fR/S	 177.3
U+4A52.0	fTotalStrokes	 12
U+4A52.0	fCangjie	TJMD
U+4A52.0	fFourCorner	41520
U+4A52.0	fCantonese	jyu4; zyu1
U+4A52.0	fMandarin	yu2
U+4A52.0	fEnglish	dressed leather; large belt
U+4A53.0	fUTF8	䩓
U+4A53.0	fCNS11643	3-454D
U+4A53.0	fR/S	 177.4
U+4A53.0	fTotalStrokes	 13
U+4A53.0	fCangjie	TJPI
U+4A53.0	fFourCorner	47520
U+4A53.0	fCantonese	kei4
U+4A53.0	fMandarin	qi2
U+4A53.0	fEnglish	[1] (non-classical form) [2] boots [3] the leather belts that connect a cart with the horse, etc. [4] the leather decorations on the ends of the hub (of a wheel)
U+4A54.0	fUTF8	䩔
U+4A54.0	fCNS11643	4-415D
U+4A54.0	fGB	3-7928
U+4A54.0	fR/S	 177.4
U+4A54.0	fTotalStrokes	 13
U+4A54.0	fCangjie	TJHNE
U+4A54.0	fFourCorner	47540
U+4A54.0	fCantonese	do6; bong2; bung2; pun4
U+4A54.0	fMandarin	duo4
U+4A54.0	fEnglish	[1] the hem; margin of the heel of a shoe [2] weeping sash or cord on a saddle [3] leather rope [4] fringes of pearls on crowns [5] pennant [6] same as &#x97B6;
U+4A55.0	fUTF8	䩕
U+4A55.0	fCNS11643	4-415F
U+4A55.0	fGB	3-7927
U+4A55.0	fR/S	 177.4
U+4A55.0	fTotalStrokes	 13
U+4A55.0	fCangjie	TJHVL
U+4A55.0	fFourCorner	47520
U+4A55.0	fCantonese	ngong4; ong4 (also)
U+4A55.0	fMandarin	yang3; ang2
U+4A55.0	fEnglish	[1] clogs; pattens [2] same as &#x786C;
U+4A56.0	fUTF8	䩖
U+4A56.0	fCNS11643	3-454C
U+4A56.0	fGB	5-7727
U+4A56.0	fR/S	 177.4
U+4A56.0	fTotalStrokes	 13
U+4A56.0	fCangjie	TJFH
U+4A56.0	fFourCorner	49520
U+4A56.0	fCantonese	saa1
U+4A56.0	fMandarin	sha1
U+4A56.0	fEnglish	shoes; boots
U+4A57.0	fUTF8	䩗
U+4A57.0	fCNS11643	F-4632
U+4A57.0	fR/S	 177.4
U+4A57.0	fTotalStrokes	 13
U+4A57.0	fCangjie	TJB
U+4A57.0	fFourCorner	4752
U+4A57.0	fCantonese	baa3
U+4A57.0	fMandarin	ba4
U+4A57.0	fEnglish	same as &#x9738;
U+4A58.0	fUTF8	䩘
U+4A58.0	fCNS11643	6-5A28
U+4A58.0	fR/S	 177.4
U+4A58.0	fTotalStrokes	 13
U+4A58.0	fCangjie	TJMVI
U+4A58.0	fFourCorner	41530
U+4A58.0	fCantonese	dai3; daai1
U+4A58.0	fMandarin	di4
U+4A58.0	fEnglish	[1] non-classical form of &#x4A5A; [2] interchangeable &#x976A;) [3] same as &#x97AE;
U+4A59.0	fUTF8	䩙
U+4A59.0	fCNS11643	4-483A
U+4A59.0	fGB	5-772F
U+4A59.0	fR/S	 177.5
U+4A59.0	fTotalStrokes	 14
U+4A59.0	fCangjie	TJYVI
U+4A59.0	fFourCorner	40530
U+4A59.0	fCantonese	jyun5; zyun4
U+4A59.0	fMandarin	xuan4
U+4A59.0	fEnglish	[1] scabbard [2] reins or traces
U+4A5A.0	fUTF8	䩚
U+4A5A.0	fCNS11643	4-4840
U+4A5A.0	fGB	5-772C
U+4A5A.0	fR/S	 177.5
U+4A5A.0	fTotalStrokes	 14
U+4A5A.0	fCangjie	TJHVI
U+4A5A.0	fFourCorner	42540
U+4A5A.0	fHakka	 ti2
U+4A5A.0	fCantonese	dai3; daai1; daai2
U+4A5A.0	fMandarin	di4; dian4
U+4A5A.0	fEnglish	[1] interchangeable with &#x976A; [2] same as &#x97AE;
U+4A5B.0	fUTF8	䩛
U+4A5B.0	fCNS11643	4-483B
U+4A5B.0	fGB	3-792F
U+4A5B.0	fR/S	 177.5
U+4A5B.0	fTotalStrokes	 14
U+4A5B.0	fCangjie	TJPH
U+4A5B.0	fFourCorner	43530
U+4A5B.0	fCantonese	bat6; bit1
U+4A5B.0	fMandarin	bi4
U+4A5B.0	fEnglish	[1] leather belt used to bind or restrain a cart [2] same as &#x8F61;
U+4A5C.0	fUTF8	䩜
U+4A5C.0	fCNS11643	3-4A4F
U+4A5C.0	fGB	5-772B
U+4A5C.0	fR/S	 177.5
U+4A5C.0	fTotalStrokes	 14
U+4A5C.0	fCangjie	TJLW
U+4A5C.0	fFourCorner	45560
U+4A5C.0	fHakka	 ciu5
U+4A5C.0	fCantonese	zau6
U+4A5C.0	fMandarin	zhou4
U+4A5C.0	fEnglish	same as &#x5191;
U+4A5D.0	fUTF8	䩝
U+4A5D.0	fCNS11643	3-4A50
U+4A5D.0	fR/S	 177.5
U+4A5D.0	fTotalStrokes	 14
U+4A5D.0	fCangjie	TJHVO
U+4A5D.0	fFourCorner	42530
U+4A5D.0	fCantonese	pok3; pau4
U+4A5D.0	fMandarin	pao2
U+4A5D.0	fEnglish	[1] non-classical form of &#x9784; [2] same as &#x530F; [3] interchangeable with &#x67B9; [4] interchangeable with &#x5305;
U+4A5E.0	fUTF8	䩞
U+4A5E.0	fCNS11643	4-483D
U+4A5E.0	fGB	3-792B
U+4A5E.0	fR/S	 177.5
U+4A5E.0	fTotalStrokes	 14
U+4A5E.0	fCangjie	TJYR
U+4A5E.0	fFourCorner	41560
U+4A5E.0	fCantonese	tip3
U+4A5E.0	fMandarin	tie4; nian2; tian3; tie2; wei3
U+4A5E.0	fEnglish	a saddle-flap, trappings
U+4A5F.0	fUTF8	䩟
U+4A5F.0	fCNS11643	4-4E4F
U+4A5F.0	fGB	3-7934
U+4A5F.0	fR/S	 177.6
U+4A5F.0	fTotalStrokes	 15
U+4A5F.0	fCangjie	TJKN
U+4A5F.0	fFourCorner	45530
U+4A5F.0	fCantonese	ji4; zi4
U+4A5F.0	fMandarin	yi2
U+4A5F.0	fEnglish	[1] leather; processed hides [2] trappings, known together as saddle; a saddle-like terrain or thing and rein, etc.
U+4A60.0	fUTF8	䩠
U+4A60.0	fCNS11643	4-5463
U+4A60.0	fGB	5-7735
U+4A60.0	fR/S	 177.7
U+4A60.0	fTotalStrokes	 16
U+4A60.0	fCangjie	TJNKG
U+4A60.0	fFourCorner	42540
U+4A60.0	fHakka	  tin2
U+4A60.0	fCantonese	ting1
U+4A60.0	fMandarin	ting1
U+4A60.0	fEnglish	straps of leather round boxes
U+4A61.0	fUTF8	䩡
U+4A61.0	fCNS11643	4-5461
U+4A61.0	fGB	3-7933
U+4A61.0	fR/S	 177.7
U+4A61.0	fTotalStrokes	 16
U+4A61.0	fCangjie	TJKOO
U+4A61.0	fFourCorner	44530
U+4A61.0	fCantonese	gaap3
U+4A61.0	fMandarin	jia2
U+4A61.0	fEnglish	[1] a kind of boots in old times [2] heel of a shoe [3] same as &#x9788;
U+4A62.0	fUTF8	䩢
U+4A62.0	fCNS11643	4-5460
U+4A62.0	fGB	3-7938
U+4A62.0	fR/S	 177.7
U+4A62.0	fTotalStrokes	 16
U+4A62.0	fCangjie	TJQHL
U+4A62.0	fFourCorner	42520
U+4A62.0	fHakka	 zih5; zhi5; zhat7; zat7; zhet7
U+4A62.0	fCantonese	zai3; dat3; dik6
U+4A62.0	fMandarin	zhe2; da2; zhi4
U+4A62.0	fEnglish	scabbard
U+4A63.0	fUTF8	䩣
U+4A63.0	fCNS11643	4-5464
U+4A63.0	fGB	3-793C
U+4A63.0	fR/S	 177.7
U+4A63.0	fTotalStrokes	 16
U+4A63.0	fCangjie	TJOMD
U+4A63.0	fFourCorner	48590
U+4A63.0	fCantonese	tou4; pou4
U+4A63.0	fMandarin	tu2; duo3; tui4
U+4A63.0	fEnglish	shoe-cushions
U+4A64.0	fUTF8	䩤
U+4A64.0	fCNS11643	4-5462
U+4A64.0	fGB	3-7925
U+4A64.0	fR/S	 177.7
U+4A64.0	fTotalStrokes	 16
U+4A64.0	fCangjie	TJBUU
U+4A64.0	fFourCorner	46510
U+4A64.0	fCantonese	kit3; git3
U+4A64.0	fMandarin	xie2; xi4
U+4A64.0	fEnglish	[1] lead an ox [2] tie it tightly [3] quick, urgent; hurried; hasty
U+4A65.0	fUTF8	䩥
U+4A65.0	fCNS11643	4-5963
U+4A65.0	fGB	3-7936
U+4A65.0	fR/S	 177.8; 177.7
U+4A65.0	fTotalStrokes	 16; 17
U+4A65.0	fCangjie	TJNKM
U+4A65.0	fFourCorner	42540
U+4A65.0	fCantonese	daan6
U+4A65.0	fMandarin	dan4
U+4A65.0	fEnglish	a belt for horse, horses without saddle or rein, leather shoes
U+4A66.0	fUTF8	䩦
U+4A66.0	fCNS11643	4-5468
U+4A66.0	fGB	3-793B
U+4A66.0	fR/S	 177.7
U+4A66.0	fTotalStrokes	 16
U+4A66.0	fCangjie	OKTLJ
U+4A66.0	fFourCorner	28500
U+4A66.0	fCantonese	tiu4
U+4A66.0	fMandarin	tiao2; zuo4
U+4A66.0	fEnglish	[1] the reins [2] a small bronze ring used to connect the reins
U+4A67.0	fUTF8	䩧
U+4A67.0	fCNS11643	3-5352
U+4A67.0	fR/S	 177.7
U+4A67.0	fTotalStrokes	 16
U+4A67.0	fCangjie	TJYMR
U+4A67.0	fFourCorner	40560
U+4A67.0	fCantonese	kit3; hyut3
U+4A67.0	fMandarin	xue4; xie4
U+4A67.0	fEnglish	 tie up hurriedly
U+4A68.0	fUTF8	䩨
U+4A68.0	fCNS11643	3-567C
U+4A68.0	fR/S	 177.8
U+4A68.0	fTotalStrokes	 17
U+4A68.0	fCangjie	TJSMV
U+4A68.0	fFourCorner	41530
U+4A68.0	fHakka	 chong3
U+4A68.0	fCantonese	coeng3
U+4A68.0	fMandarin	chang4
U+4A68.0	fEnglish	same as &#x97D4;
U+4A69.0	fUTF8	䩩
U+4A69.0	fCNS11643	4-5968
U+4A69.0	fGB	3-7943
U+4A69.0	fR/S	 177.8
U+4A69.0	fTotalStrokes	 17; 16
U+4A69.0	fCangjie	TJJNU
U+4A69.0	fFourCorner	43510
U+4A69.0	fCantonese	jyun2
U+4A69.0	fMandarin	wan3; yuan3
U+4A69.0	fEnglish	a measure of capacity, a tool used to take out the mud from a well, back covering or shelter of a large carriage, military vehicles
U+4A6A.0	fUTF8	䩪
U+4A6A.0	fCNS11643	4-5964
U+4A6A.0	fGB	3-7941
U+4A6A.0	fR/S	 177.8
U+4A6A.0	fTotalStrokes	 17; 16
U+4A6A.0	fCangjie	TJJRR
U+4A6A.0	fFourCorner	43570
U+4A6A.0	fCantonese	gun2; gwun2 (also)
U+4A6A.0	fMandarin	guan3
U+4A6A.0	fEnglish	known together as saddle; rein and bridle, etc.
U+4A6B.0	fUTF8	䩫
U+4A6B.0	fCNS11643	7-3D56
U+4A6B.0	fR/S	 177.8
U+4A6B.0	fTotalStrokes	 17
U+4A6B.0	fCangjie	TJMLB
U+4A6B.0	fFourCorner	41520
U+4A6B.0	fCantonese	loeng5; long4
U+4A6B.0	fMandarin	liang4; liang3
U+4A6B.0	fEnglish	same as &#x7DC9;
U+4A6C.0	fUTF8	䩬
U+4A6C.0	fCNS11643	3-5679
U+4A6C.0	fGB	5-7740
U+4A6C.0	fR/S	 177.8
U+4A6C.0	fTotalStrokes	 17
U+4A6C.0	fCangjie	TJQKQ
U+4A6C.0	fFourCorner	45550
U+4A6C.0	fCantonese	bung2
U+4A6C.0	fMandarin	beng3; feng3
U+4A6C.0	fEnglish	[1] an ornamental item on a scabbard [2] leather shoes [3] weapons; arms [4] interchangeable with &#x979B;
U+4A6D.0	fUTF8	䩭
U+4A6D.0	fCNS11643	3-567D
U+4A6D.0	fGB	5-7741
U+4A6D.0	fR/S	 177.8
U+4A6D.0	fTotalStrokes	 17
U+4A6D.0	fCangjie	TJKMR
U+4A6D.0	fFourCorner	44520
U+4A6D.0	fCantonese	gei1
U+4A6D.0	fEnglish	same as &#x7F88;
U+4A6E.0	fUTF8	䩮
U+4A6E.0	fCNS11643	3-567E
U+4A6E.0	fGB	5-7746
U+4A6E.0	fR/S	 177.8
U+4A6E.0	fTotalStrokes	 17
U+4A6E.0	fCangjie	TJVNE
U+4A6E.0	fFourCorner	47530
U+4A6E.0	fCantonese	luk6
U+4A6E.0	fMandarin	lu4
U+4A6E.0	fEnglish	 quiver
U+4A6F.0	fUTF8	䩯
U+4A6F.0	fCNS11643	4-5E3E
U+4A6F.0	fGB	3-7945
U+4A6F.0	fR/S	 177.9
U+4A6F.0	fTotalStrokes	 18
U+4A6F.0	fCangjie	TJMEM
U+4A6F.0	fFourCorner	41510
U+4A6F.0	fCantonese	gik1; gik6
U+4A6F.0	fMandarin	ji2
U+4A6F.0	fEnglish	interchangeable with &#x9769;
U+4A70.0	fUTF8	䩰
U+4A70.0	fCNS11643	5-657E
U+4A70.0	fR/S	 177.9
U+4A70.0	fTotalStrokes	 18
U+4A70.0	fCangjie	TJRSJ
U+4A70.0	fFourCorner	46540
U+4A70.0	fCantonese	jyun5; gyun1
U+4A70.0	fMandarin	ji2; xuan4
U+4A70.0	fEnglish	corrupted form of &#x9799;
U+4A71.0	fUTF8	䩱
U+4A71.0	fCNS11643	4-5E42
U+4A71.0	fGB	3-794C
U+4A71.0	fR/S	 177.9
U+4A71.0	fTotalStrokes	 18
U+4A71.0	fCangjie	TJOMN
U+4A71.0	fFourCorner	48521
U+4A71.0	fCantonese	syu3; zyu4
U+4A71.0	fMandarin	yu2; shu4
U+4A71.0	fEnglish	[1] scabbard; sheath [2] remaining; overplus; surplus [3] sin; evil [4] the front of a cutting strip of cloth
U+4A72.0	fUTF8	䩲
U+4A72.0	fCNS11643	4-5E3C
U+4A72.0	fGB	3-794E
U+4A72.0	fR/S	 177.9
U+4A72.0	fTotalStrokes	 18
U+4A72.0	fCangjie	TJITE
U+4A72.0	fFourCorner	40540
U+4A72.0	fCantonese	dou1
U+4A72.0	fMandarin	du1
U+4A72.0	fEnglish	[1] a tool used to cut leather [2] an ox to tow, drag or pull a boat
U+4A73.0	fUTF8	䩳
U+4A73.0	fCNS11643	5-6573
U+4A73.0	fR/S	 177.9
U+4A73.0	fTotalStrokes	 18
U+4A73.0	fCangjie	TJJFE
U+4A73.0	fFourCorner	43540
U+4A73.0	fCantonese	sau1; cuk1
U+4A73.0	fMandarin	sou1; shu3; su4
U+4A73.0	fEnglish	[1] tanning [2] soft leather
U+4A74.0	fUTF8	䩴
U+4A74.0	fCNS11643	5-657B
U+4A74.0	fR/S	 177.9
U+4A74.0	fTotalStrokes	 18
U+4A74.0	fCangjie	TJJRB
U+4A74.0	fFourCorner	47520
U+4A74.0	fHakka	 fu2
U+4A74.0	fCantonese	wu4
U+4A74.0	fMandarin	hu2
U+4A74.0	fEnglish	quiver
U+4A75.0	fUTF8	䩵
U+4A75.0	fCNS11643	4-5E3B
U+4A75.0	fGB	5-773A
U+4A75.0	fR/S	 177.9
U+4A75.0	fTotalStrokes	 18
U+4A75.0	fCangjie	TJBJJ
U+4A75.0	fFourCorner	47550
U+4A75.0	fHakka	 jun5; run6
U+4A75.0	fCantonese	wan6
U+4A75.0	fMandarin	yun4
U+4A75.0	fEnglish	 worker in leather
U+4A76.0	fUTF8	䩶
U+4A76.0	fCNS11643	4-5E41
U+4A76.0	fGB	3-7951
U+4A76.0	fR/S	 177.10
U+4A76.0	fTotalStrokes	 18; 19
U+4A76.0	fCangjie	TJUMI
U+4A76.0	fFourCorner	42530
U+4A76.0	fCantonese	cin2
U+4A76.0	fMandarin	chan3
U+4A76.0	fEnglish	saddle and rein etc. for the imperial carriage; or for the two outside horses of a team of four abreast, equipments used to saddle a horse
U+4A77.0	fUTF8	䩷
U+4A77.0	fCNS11643	3-5B5C
U+4A77.0	fR/S	 177.10
U+4A77.0	fTotalStrokes	 19
U+4A77.0	fCangjie	TJYBS
U+4A77.0	fFourCorner	40520
U+4A77.0	fCantonese	bong1; pang4
U+4A77.0	fMandarin	bang1
U+4A77.0	fEnglish	same as &#x5E6B;
U+4A78.0	fUTF8	䩸
U+4A78.0	fCNS11643	3-5B5B
U+4A78.0	fGB	5-7749
U+4A78.0	fR/S	 177.10
U+4A78.0	fTotalStrokes	 19
U+4A78.0	fCangjie	TJTSJ
U+4A78.0	fFourCorner	44540
U+4A78.0	fCantonese	jung4; zung4
U+4A78.0	fMandarin	rong2; rong3
U+4A78.0	fEnglish	[1] fine woolen blanket on top of a saddle; used as some decoration [2] same as &#x6C04;
U+4A79.0	fUTF8	䩹
U+4A79.0	fCNS11643	4-6177
U+4A79.0	fGB	3-7952
U+4A79.0	fR/S	 177.10
U+4A79.0	fTotalStrokes	 19
U+4A79.0	fCangjie	TJMRB
U+4A79.0	fFourCorner	41520
U+4A79.0	fCantonese	ngaak6; aak6 (also); ngak6
U+4A79.0	fMandarin	he2; e2
U+4A79.0	fEnglish	[1] repair shoes [2] the front part of a shoe [3] same as &#x97B9;
U+4A7A.0	fUTF8	䩺
U+4A7A.0	fCNS11643	4-617B
U+4A7A.0	fGB	3-7953
U+4A7A.0	fR/S	 177.10
U+4A7A.0	fTotalStrokes	 19
U+4A7A.0	fCangjie	TJCIM
U+4A7A.0	fFourCorner	48520
U+4A7A.0	fHakka	 wung1; vung1
U+4A7A.0	fCantonese	jung1; zung1
U+4A7A.0	fMandarin	weng1
U+4A7A.0	fEnglish	the "uppers" of a boot
U+4A7B.0	fUTF8	䩻
U+4A7B.0	fCNS11643	3-5B5A
U+4A7B.0	fR/S	 177.10
U+4A7B.0	fTotalStrokes	 19
U+4A7B.0	fCangjie	TJSQF
U+4A7B.0	fFourCorner	41520
U+4A7B.0	fCantonese	baa3
U+4A7B.0	fMandarin	ba4
U+4A7B.0	fEnglish	same as &#x4A57;, &#x9738;
U+4A7C.0	fUTF8	䩼
U+4A7C.0	fCNS11643	4-6523
U+4A7C.0	fGB	3-7954
U+4A7C.0	fR/S	 177.11
U+4A7C.0	fTotalStrokes	 20
U+4A7C.0	fCangjie	TJYHJ
U+4A7C.0	fFourCorner	47530
U+4A7C.0	fCantonese	fung4
U+4A7C.0	fMandarin	feng2
U+4A7C.0	fEnglish	[1] stitches on a coverlet [2] drum [3] drum beat
U+4A7D.0	fUTF8	䩽
U+4A7D.0	fCNS11643	4-647E
U+4A7D.0	fGB	3-7924
U+4A7D.0	fR/S	 177.11
U+4A7D.0	fTotalStrokes	 20
U+4A7D.0	fCangjie	TJSRR
U+4A7D.0	fFourCorner	41510
U+4A7D.0	fCantonese	jyu1; zyu1
U+4A7D.0	fMandarin	ou4
U+4A7D.0	fEnglish	[1] quiver (for bows and arrows) one a horse [2] store; keep and collect [3] lance; spear; trident
U+4A7E.0	fUTF8	䩾
U+4A7E.0	fCNS11643	5-6E49
U+4A7E.0	fR/S	 177.11
U+4A7E.0	fTotalStrokes	 20
U+4A7E.0	fCangjie	TJITF
U+4A7E.0	fFourCorner	40530
U+4A7E.0	fCantonese	ze3
U+4A7E.0	fMandarin	zhe4
U+4A7E.0	fEnglish	herb medicine; dendrobium
U+4A7F.0	fUTF8	䩿
U+4A7F.0	fCNS11643	7-5627
U+4A7F.0	fR/S	 177.12
U+4A7F.0	fTotalStrokes	 21
U+4A7F.0	fCangjie	TJJTC
U+4A7F.0	fFourCorner	44580
U+4A7F.0	fCantonese	fan4; baan6; faan4
U+4A7F.0	fMandarin	fen2
U+4A7F.0	fEnglish	same as &#x9F16;
U+4A80.0	fUTF8	䪀
U+4A80.0	fCNS11643	7-5625
U+4A80.0	fR/S	 177.12
U+4A80.0	fTotalStrokes	 21
U+4A80.0	fCangjie	TJJRP
U+4A80.0	fFourCorner	43530
U+4A80.0	fCantonese	gun2; gwun2 (also); faat1; wun2; zyun2
U+4A80.0	fMandarin	guan3; ruan3
U+4A80.0	fEnglish	same as &#x4A6A;
U+4A81.0	fUTF8	䪁
U+4A81.0	fCNS11643	4-6772
U+4A81.0	fGB	3-7957
U+4A81.0	fR/S	 177.12
U+4A81.0	fTotalStrokes	 21
U+4A81.0	fCangjie	TJTCO
U+4A81.0	fFourCorner	43530
U+4A81.0	fCantonese	buk6; pun4
U+4A81.0	fMandarin	bo2; bu3
U+4A81.0	fEnglish	 halter
U+4A82.0	fUTF8	䪂
U+4A82.0	fCNS11643	4-6770
U+4A82.0	fGB	3-7956
U+4A82.0	fR/S	 177.12
U+4A82.0	fTotalStrokes	 21
U+4A82.0	fCangjie	TJDBB
U+4A82.0	fFourCorner	45590
U+4A82.0	fCantonese	gaak3
U+4A82.0	fMandarin	ji2; ge2
U+4A82.0	fEnglish	[1] reins; a halter [2] interchangeable &#x9769;
U+4A83.0	fUTF8	䪃
U+4A83.0	fCNS11643	5-7169
U+4A83.0	fR/S	 177.12
U+4A83.0	fTotalStrokes	 21
U+4A83.0	fCangjie	TJYDK
U+4A83.0	fFourCorner	48540
U+4A83.0	fCantonese	deon1; maan4
U+4A83.0	fMandarin	dun1
U+4A83.0	fEnglish	wine container used by minority tribe in ancient times
U+4A84.0	fUTF8	䪄
U+4A84.0	fCNS11643	7-5626
U+4A84.0	fR/S	 177.12
U+4A84.0	fTotalStrokes	 21
U+4A84.0	fCangjie	TJTMC
U+4A84.0	fFourCorner	44580
U+4A84.0	fCantonese	wong4
U+4A84.0	fMandarin	huang2
U+4A84.0	fEnglish	ancient form of &#x714C;
U+4A85.0	fUTF8	䪅
U+4A85.0	fCNS11643	3-5F74
U+4A85.0	fR/S	 177.13
U+4A85.0	fTotalStrokes	 22
U+4A85.0	fCangjie	TJWLI
U+4A85.0	fFourCorner	46520
U+4A85.0	fCantonese	duk6
U+4A85.0	fMandarin	shu3; du2
U+4A85.0	fEnglish	same as &#x97E3;
U+4A86.0	fUTF8	䪆
U+4A86.0	fCNS11643	5-7462
U+4A86.0	fR/S	 177.13
U+4A86.0	fTotalStrokes	 22
U+4A86.0	fCangjie	TJTWT
U+4A86.0	fFourCorner	45510
U+4A86.0	fCantonese	tai2
U+4A86.0	fMandarin	ti3
U+4A86.0	fEnglish	[1] soft; tender [2] weak; feeble [3] gentle; mild
U+4A87.0	fUTF8	䪇
U+4A87.0	fCNS11643	4-6B2C
U+4A87.0	fGB	3-795C
U+4A87.0	fR/S	 177.14
U+4A87.0	fTotalStrokes	 23
U+4A87.0	fCangjie	TJTII
U+4A87.0	fFourCorner	44540
U+4A87.0	fCantonese	bok6
U+4A87.0	fMandarin	bo2
U+4A87.0	fEnglish	[1] cushioned seat on a vehicle [2] a leather wrapped collar or yoke of a carriage
U+4A88.0	fUTF8	䪈
U+4A88.0	fCNS11643	3-605A
U+4A88.0	fGB	5-775C
U+4A88.0	fR/S	 177.14
U+4A88.0	fTotalStrokes	 23
U+4A88.0	fCangjie	TJYLR
U+4A88.0	fFourCorner	45530
U+4A88.0	fHakka	 kien5; ken5
U+4A88.0	fCantonese	hin3; hin2
U+4A88.0	fMandarin	qian4; qian3
U+4A88.0	fEnglish	 leathern girdle; belt
U+4A89.0	fUTF8	䪉
U+4A89.0	fCNS11643	4-6C34
U+4A89.0	fGB	3-795B
U+4A89.0	fR/S	 177.15
U+4A89.0	fTotalStrokes	 24
U+4A89.0	fCangjie	TJVVV
U+4A89.0	fFourCorner	42510
U+4A89.0	fCantonese	lip6
U+4A89.0	fMandarin	lie4; la4
U+4A89.0	fEnglish	metal decoration on a halter
U+4A8A.0	fUTF8	䪊
U+4A8A.0	fCNS11643	4-6D2D
U+4A8A.0	fGB	5-772A
U+4A8A.0	fR/S	 177.16
U+4A8A.0	fTotalStrokes	 25
U+4A8A.0	fCangjie	TJYBP
U+4A8A.0	fFourCorner	41511
U+4A8A.0	fHakka	  lung2
U+4A8A.0	fCantonese	lung4
U+4A8A.0	fMandarin	long2
U+4A8A.0	fEnglish	[1] halter [2] head-stall
U+4A8B.0	fUTF8	䪋
U+4A8B.0	fCNS11643	3-6154
U+4A8B.0	fR/S	 177.16
U+4A8B.0	fTotalStrokes	 25
U+4A8B.0	fCangjie	TJYLC
U+4A8B.0	fFourCorner	45530
U+4A8B.0	fCantonese	wai6
U+4A8B.0	fMandarin	wei4
U+4A8B.0	fEnglish	silk cordage; twine; rope
U+4A8C.0	fUTF8	䪌
U+4A8C.0	fCNS11643	4-6D65
U+4A8C.0	fGB	3-795D
U+4A8C.0	fR/S	 177.17
U+4A8C.0	fTotalStrokes	 26
U+4A8C.0	fCangjie	TJNRI
U+4A8C.0	fFourCorner	47510
U+4A8C.0	fHakka	 cam2; can5
U+4A8C.0	fCantonese	zaam6
U+4A8C.0	fMandarin	chan2; zhan4
U+4A8C.0	fEnglish	[1] saddle flap [2] trappings
U+4A8D.0	fUTF8	䪍
U+4A8D.0	fCNS11643	3-6170
U+4A8D.0	fR/S	 177.17
U+4A8D.0	fTotalStrokes	 26
U+4A8D.0	fCangjie	TJANW
U+4A8D.0	fFourCorner	47520
U+4A8D.0	fCantonese	laan4
U+4A8D.0	fMandarin	lan2; lan4
U+4A8D.0	fEnglish	a container for bows and arrows
U+4A8E.0	fUTF8	䪎
U+4A8E.0	fCNS11643	4-6E22
U+4A8E.0	fGB	3-795E
U+4A8E.0	fR/S	 177.18
U+4A8E.0	fTotalStrokes	 27
U+4A8E.0	fCangjie	TJUOB
U+4A8E.0	fFourCorner	42520
U+4A8E.0	fCantonese	seoi1
U+4A8E.0	fMandarin	sui1
U+4A8E.0	fEnglish	[1] strings of a cap [2] decorations for saddle [3] sash; band; cord [4] flag
U+4A8F.0	fUTF8	䪏
U+4A8F.0	fCNS11643	4-4161
U+4A8F.0	fGB	3-402F
U+4A8F.0	fR/S	 178.4
U+4A8F.0	fTotalStrokes	 13
U+4A8F.0	fCangjie	DQOB
U+4A8F.0	fFourCorner	44520
U+4A8F.0	fCantonese	naap6; laap6 (also)
U+4A8F.0	fMandarin	na4; neng2
U+4A8F.0	fEnglish	weak; feeble; tender; delicate; soft, (dialect) to hang down the head and sleepy
U+4A90.0	fUTF8	䪐
U+4A90.0	fCNS11643	4-4843
U+4A90.0	fGB	3-4036
U+4A90.0	fR/S	 178.5
U+4A90.0	fTotalStrokes	 14
U+4A90.0	fCangjie	DQPH
U+4A90.0	fFourCorner	43500
U+4A90.0	fHakka	 bi5
U+4A90.0	fCantonese	bei3; faai6; maat6; pit3
U+4A90.0	fMandarin	bi4; pi4
U+4A90.0	fEnglish	a frame for a bow
U+4A91.0	fUTF8	䪑
U+4A91.0	fCNS11643	4-4841
U+4A91.0	fGB	3-4035
U+4A91.0	fR/S	 178.5
U+4A91.0	fTotalStrokes	 14
U+4A91.0	fCangjie	DQJP
U+4A91.0	fFourCorner	43510
U+4A91.0	fCantonese	to4
U+4A91.0	fMandarin	tuo2
U+4A91.0	fEnglish	 stick; to paste leather to shoes
U+4A92.0	fUTF8	䪒
U+4A92.0	fCNS11643	4-4842
U+4A92.0	fGB	3-4034
U+4A92.0	fR/S	 178.5
U+4A92.0	fTotalStrokes	 14
U+4A92.0	fCangjie	DQYG
U+4A92.0	fFourCorner	40510
U+4A92.0	fHakka	 zhu3; zu3; zhu5
U+4A92.0	fCantonese	zyu3
U+4A92.0	fMandarin	zhu4; jiao4; zhi4
U+4A92.0	fEnglish	leathern gaiters
U+4A93.0	fUTF8	䪓
U+4A93.0	fCNS11643	4-4844
U+4A93.0	fGB	3-4033
U+4A93.0	fR/S	 178.5
U+4A93.0	fTotalStrokes	 14
U+4A93.0	fCangjie	DQYR
U+4A93.0	fFourCorner	41560
U+4A93.0	fCantonese	dip3
U+4A93.0	fMandarin	die2
U+4A93.0	fEnglish	a metal ornament on some kind of belt (clothing of the Northern barbarians)
U+4A94.0	fUTF8	䪔
U+4A94.0	fCNS11643	3-5354
U+4A94.0	fGB	5-4329
U+4A94.0	fR/S	 178.7
U+4A94.0	fTotalStrokes	 16
U+4A94.0	fCangjie	DQIJB
U+4A94.0	fFourCorner	43520
U+4A94.0	fCantonese	bou2
U+4A94.0	fMandarin	fu3; bu3
U+4A94.0	fEnglish	[1] cross-bar of wood under the carriage [2] undergarments
U+4A95.0	fUTF8	䪕
U+4A95.0	fCNS11643	4-596B
U+4A95.0	fGB	3-403B
U+4A95.0	fR/S	 178.8
U+4A95.0	fTotalStrokes	 17; 16
U+4A95.0	fCangjie	DQPFD
U+4A95.0	fFourCorner	47520
U+4A95.0	fCantonese	guk6; guk1
U+4A95.0	fMandarin	ju2
U+4A95.0	fEnglish	[1] wrap or band [2] same as &#x97A0;
U+4A96.0	fUTF8	䪖
U+4A96.0	fCNS11643	7-4477
U+4A96.0	fR/S	 178.9
U+4A96.0	fTotalStrokes	 18
U+4A96.0	fCangjie	DQOAE
U+4A96.0	fFourCorner	48540
U+4A96.0	fCantonese	pok3; pak3
U+4A96.0	fMandarin	fu4; po4
U+4A96.0	fEnglish	[1] same as &#x4A99; [2] same as &#x4A94;
U+4A97.0	fUTF8	䪗
U+4A97.0	fCNS11643	4-5E45
U+4A97.0	fGB	3-403D
U+4A97.0	fR/S	 178.9
U+4A97.0	fTotalStrokes	 18
U+4A97.0	fCangjie	DQRSE
U+4A97.0	fFourCorner	47540
U+4A97.0	fHakka	 ton5; ton6
U+4A97.0	fCantonese	haa4
U+4A97.0	fMandarin	xia2
U+4A97.0	fEnglish	the stiffening put in at the heel of a shoe
U+4A98.0	fUTF8	䪘
U+4A98.0	fCNS11643	3-5952
U+4A98.0	fR/S	 178.9
U+4A98.0	fTotalStrokes	 18
U+4A98.0	fCangjie	DQAMO
U+4A98.0	fFourCorner	46580
U+4A98.0	fCantonese	wai5; daai1; waai5
U+4A98.0	fMandarin	wei3
U+4A98.0	fEnglish	same as &#x97D9;
U+4A99.0	fUTF8	䪙
U+4A99.0	fCNS11643	4-617D
U+4A99.0	fGB	5-4332
U+4A99.0	fR/S	 178.10
U+4A99.0	fTotalStrokes	 19
U+4A99.0	fCangjie	DQIBI
U+4A99.0	fFourCorner	43540
U+4A99.0	fCantonese	pok3
U+4A99.0	fMandarin	fu4; fu2; po4
U+4A99.0	fEnglish	[1] leather wrapped collar for a draft animal of a carriage [2] bags used on a carriage [3] same as &#x4A94;
U+4A9A.0	fUTF8	䪚
U+4A9A.0	fCNS11643	4-617E
U+4A9A.0	fGB	3-4040
U+4A9A.0	fR/S	 178.10
U+4A9A.0	fTotalStrokes	 19
U+4A9A.0	fCangjie	DQASM
U+4A9A.0	fFourCorner	46520
U+4A9A.0	fCantonese	daap3; tap3
U+4A9A.0	fMandarin	da1; he4; ta4
U+4A9A.0	fEnglish	[1] hot; burning [2] fur clothing [3] weapons [4] drum
U+4A9B.0	fUTF8	䪛
U+4A9B.0	fCNS11643	4-6773
U+4A9B.0	fGB	3-4042
U+4A9B.0	fR/S	 178.12
U+4A9B.0	fTotalStrokes	 21
U+4A9B.0	fCangjie	DQHDW
U+4A9B.0	fFourCorner	42560
U+4A9B.0	fCantonese	faan1
U+4A9B.0	fMandarin	fan1; fan2
U+4A9B.0	fEnglish	[1] group; crowd; swarm; flock [2] a square of leather [3] leather wrapped
U+4A9C.0	fUTF8	䪜
U+4A9C.0	fCNS11643	4-6960
U+4A9C.0	fGB	5-4337
U+4A9C.0	fR/S	 178.13
U+4A9C.0	fTotalStrokes	 22; 21
U+4A9C.0	fCangjie	DQNCR
U+4A9C.0	fFourCorner	47560
U+4A9C.0	fHakka	 zham1
U+4A9C.0	fCantonese	cim1; sim4; zim4
U+4A9C.0	fMandarin	yan2; chan4
U+4A9C.0	fEnglish	[1] shield; screen [2] same as &#x895C;
U+4A9D.0	fUTF8	䪝
U+4A9D.0	fCNS11643	3-605B
U+4A9D.0	fGB	5-4336
U+4A9D.0	fR/S	 178.14
U+4A9D.0	fTotalStrokes	 23
U+4A9D.0	fCangjie	DQTOE
U+4A9D.0	fFourCorner	44540
U+4A9D.0	fCantonese	waak6; wok6; wu6
U+4A9D.0	fMandarin	wo4; hu4
U+4A9D.0	fEnglish	leather cord that winding on the handle of a knife
U+4A9E.0	fUTF8	䪞
U+4A9E.0	fCNS11643	3-454E
U+4A9E.0	fGB	5-7768
U+4A9E.0	fR/S	 179.4
U+4A9E.0	fTotalStrokes	 13
U+4A9E.0	fCangjie	MKLSM
U+4A9E.0	fFourCorner	20100
U+4A9E.0	fHakka	 cap7; cap8
U+4A9E.0	fCantonese	zaap6
U+4A9E.0	fMandarin	za2
U+4A9E.0	fEnglish	vile; depraved
U+4A9F.0	fUTF8	䪟
U+4A9F.0	fR/S	 179.6
U+4A9F.0	fTotalStrokes	 15
U+4A9F.0	fCangjie	YBELM
U+4A9F.0	fCantonese	zai1
U+4A9F.0	fEnglish	[Korean character]
U+4AA0.0	fUTF8	䪠
U+4AA0.0	fCNS11643	3-5721
U+4AA0.0	fR/S	 179.8
U+4AA0.0	fTotalStrokes	 17
U+4AA0.0	fCangjie	LOLSM
U+4AA0.0	fFourCorner	57100
U+4AA0.0	fCantonese	zai1; saai1; zaai1
U+4AA0.0	fMandarin	ji1
U+4AA0.0	fEnglish	same as &#x97F2;, &#x9F4F;
U+4AA1.0	fUTF8	䪡
U+4AA1.0	fCNS11643	3-5B63
U+4AA1.0	fGB	5-7769
U+4AA1.0	fR/S	 179.10; 179.11
U+4AA1.0	fTotalStrokes	 19; 20
U+4AA1.0	fCangjie	LOLSM
U+4AA1.0	fFourCorner	57100
U+4AA1.0	fCantonese	zai1
U+4AA1.0	fMandarin	ji1
U+4AA1.0	fEnglish	same as &#x9F4F;
U+4AA2.0	fUTF8	䪢
U+4AA2.0	fCNS11643	3-5B62
U+4AA2.0	fR/S	 179.10
U+4AA2.0	fTotalStrokes	 19
U+4AA2.0	fCangjie	LHIOM
U+4AA2.0	fFourCorner	57210
U+4AA2.0	fCantonese	zai1
U+4AA2.0	fMandarin	ji1
U+4AA2.0	fEnglish	smashed; pickled vegetable
U+4AA3.0	fUTF8	䪣
U+4AA3.0	fCNS11643	3-5D52
U+4AA3.0	fR/S	 179.11
U+4AA3.0	fTotalStrokes	 20
U+4AA3.0	fCangjie	JHIOM
U+4AA3.0	fFourCorner	57210
U+4AA3.0	fCantonese	zai1
U+4AA3.0	fEnglish	same as &#x4AA2;
U+4AA4.0	fUTF8	䪤
U+4AA4.0	fCNS11643	4-6775
U+4AA4.0	fGB	3-796A
U+4AA4.0	fR/S	 179.12
U+4AA4.0	fTotalStrokes	 21
U+4AA4.0	fCangjie	LMHDW
U+4AA4.0	fFourCorner	12160
U+4AA4.0	fCantonese	faan4
U+4AA4.0	fMandarin	fan2
U+4AA4.0	fEnglish	garlic
U+4AA5.0	fUTF8	䪥
U+4AA5.0	fCNS11643	4-6B2D
U+4AA5.0	fGB	3-485B
U+4AA5.0	fR/S	 179.14
U+4AA5.0	fTotalStrokes	 23
U+4AA5.0	fCangjie	YCELM
U+4AA5.0	fFourCorner	27810
U+4AA5.0	fCantonese	haai6
U+4AA5.0	fMandarin	xie4; die2
U+4AA5.0	fEnglish	[1] shallots or scallions [2] small (piece of land) [3] narrow (minded)
U+4AA6.0	fUTF8	䪦
U+4AA6.0	fCNS11643	3-4025
U+4AA6.0	fR/S	 180.3
U+4AA6.0	fTotalStrokes	 12
U+4AA6.0	fCangjie	YAM
U+4AA6.0	fFourCorner	01610
U+4AA6.0	fHakka	  fung2
U+4AA6.0	fCantonese	hung4
U+4AA6.0	fMandarin	hong2
U+4AA6.0	fEnglish	 loud voice
U+4AA7.0	fUTF8	䪧
U+4AA7.0	fCNS11643	3-4026
U+4AA7.0	fR/S	 180.3
U+4AA7.0	fTotalStrokes	 12
U+4AA7.0	fCangjie	YAPD
U+4AA7.0	fFourCorner	04610
U+4AA7.0	fCantonese	ci4
U+4AA7.0	fMandarin	chi2
U+4AA7.0	fEnglish	ancient music; Chinese classical music
U+4AA8.0	fUTF8	䪨
U+4AA8.0	fCNS11643	3-4027
U+4AA8.0	fR/S	 180.3
U+4AA8.0	fTotalStrokes	 12
U+4AA8.0	fCangjie	YAPI
U+4AA8.0	fFourCorner	07620
U+4AA8.0	fCantonese	bok3; bok6
U+4AA8.0	fMandarin	bo2; bao2; qu2
U+4AA9.0	fUTF8	䪩
U+4AA9.0	fCNS11643	3-4551
U+4AA9.0	fR/S	 180.4
U+4AA9.0	fTotalStrokes	 13
U+4AA9.0	fCangjie	YAOIN
U+4AA9.0	fFourCorner	08620
U+4AA9.0	fHakka	 jim2; ngim2
U+4AA9.0	fCantonese	jam4; zaam4
U+4AA9.0	fMandarin	yin2
U+4AA9.0	fEnglish	same as &#x541F;
U+4AAA.0	fUTF8	䪪
U+4AAA.0	fR/S	 180.5
U+4AAA.0	fTotalStrokes	 14
U+4AAA.0	fCangjie	KRYTA
U+4AAA.0	fCantonese	gaa1
U+4AAA.0	fEnglish	[Korean character]
U+4AAB.0	fUTF8	䪫
U+4AAB.0	fCNS11643	4-546A
U+4AAB.0	fGB	3-7B30
U+4AAB.0	fR/S	 180.7
U+4AAB.0	fTotalStrokes	 16
U+4AAB.0	fCangjie	YAMVM
U+4AAB.0	fFourCorner	01610
U+4AAB.0	fCantonese	hang4; haang1
U+4AAB.0	fMandarin	jing1
U+4AAB.0	fEnglish	ancient music; Chinese classical music
U+4AAC.0	fUTF8	䪬
U+4AAC.0	fCNS11643	4-5469
U+4AAC.0	fGB	3-7B31
U+4AAC.0	fR/S	 180.7
U+4AAC.0	fTotalStrokes	 16
U+4AAC.0	fCangjie	YAJBD
U+4AAC.0	fFourCorner	04640
U+4AAC.0	fCantonese	put3; but6
U+4AAC.0	fMandarin	bo2; pu2
U+4AAC.0	fEnglish	the sound of pressing down something
U+4AAD.0	fUTF8	䪭
U+4AAD.0	fCNS11643	3-5953
U+4AAD.0	fR/S	 180.9
U+4AAD.0	fTotalStrokes	 18
U+4AAD.0	fCangjie	YAYTA
U+4AAD.0	fFourCorner	00660
U+4AAD.0	fCantonese	jyun2; zyun2
U+4AAD.0	fMandarin	ruan3
U+4AAD.0	fEnglish	a kind of musical instrument
U+4AAE.0	fUTF8	䪮
U+4AAE.0	fCNS11643	4-6222
U+4AAE.0	fGB	3-7B2F
U+4AAE.0	fR/S	 180.10
U+4AAE.0	fTotalStrokes	 19
U+4AAE.0	fCangjie	YAPUU
U+4AAE.0	fFourCorner	07620
U+4AAE.0	fCantonese	cau2; zaau1
U+4AAE.0	fMandarin	chou3
U+4AAE.0	fEnglish	[1] beautiful sound, appealing to the ear; fascinating to listen to (said of music) [2] noise or uproar of the crowd
U+4AAF.0	fUTF8	䪯
U+4AAF.0	fCNS11643	4-6223
U+4AAF.0	fGB	5-787A
U+4AAF.0	fR/S	 180.10
U+4AAF.0	fTotalStrokes	 19
U+4AAF.0	fCangjie	FFBYA
U+4AAF.0	fFourCorner	99600
U+4AAF.0	fCantonese	jing4; ang1
U+4AAF.0	fMandarin	ying2
U+4AAF.0	fEnglish	[1] sound [2] whisper [3] groan; moan
U+4AB0.0	fUTF8	䪰
U+4AB0.0	fCNS11643	3-5F75
U+4AB0.0	fR/S	 180.13
U+4AB0.0	fTotalStrokes	 22
U+4AB0.0	fCangjie	YAYTP
U+4AB0.0	fFourCorner	00630
U+4AB0.0	fCantonese	ji1; zi1
U+4AB0.0	fMandarin	yi1
U+4AB0.0	fEnglish	same as &#x566B;
U+4AB1.0	fUTF8	䪱
U+4AB1.0	fCNS11643	4-3B7B
U+4AB1.0	fGB	3-3B60
U+4AB1.0	fR/S	 181.3
U+4AB1.0	fTotalStrokes	 12
U+4AB1.0	fCangjie	SUMBC
U+4AB1.0	fFourCorner	11780
U+4AB1.0	fCantonese	goi2
U+4AB1.0	fMandarin	gai3
U+4AB1.0	fEnglish	same as &#x80F2;
U+4AB2.0	fUTF8	䪲
U+4AB2.0	fCNS11643	F-402D
U+4AB2.0	fR/S	 181.3
U+4AB2.0	fTotalStrokes	 12
U+4AB2.0	fCangjie	MUMBC
U+4AB2.0	fFourCorner	1128
U+4AB2.0	fCantonese	ngat6; at6 (also); kwaan1; waan4; waan6
U+4AB2.0	fEnglish	[1] interchangeable with &#x9811; [2] interchangeable with &#x9AE1;
U+4AB3.0	fUTF8	䪳
U+4AB3.0	fCNS11643	4-4165
U+4AB3.0	fGB	3-6066
U+4AB3.0	fR/S	 181.4
U+4AB3.0	fTotalStrokes	 13
U+4AB3.0	fCangjie	SKMBC
U+4AB3.0	fFourCorner	11580
U+4AB3.0	fCantonese	wan5
U+4AB3.0	fMandarin	yi1; yun3
U+4AB3.0	fEnglish	slanted face causing by the paralyzed of the facial nerve
U+4AB4.0	fUTF8	䪴
U+4AB4.0	fCNS11643	4-4164
U+4AB4.0	fGB	3-6065
U+4AB4.0	fR/S	 181.4
U+4AB4.0	fTotalStrokes	 13
U+4AB4.0	fCangjie	LUMBC
U+4AB4.0	fFourCorner	41080
U+4AB4.0	fHakka	 zhim3; dam1; zihm3
U+4AB4.0	fCantonese	zam2; daam2
U+4AB4.0	fMandarin	zhen3; dan3; dan4
U+4AB4.0	fEnglish	 occiput; bone of thehead on which one rests when speeling
U+4AB5.0	fUTF8	䪵
U+4AB5.0	fCNS11643	5-4434
U+4AB5.0	fR/S	 181.4
U+4AB5.0	fTotalStrokes	 13
U+4AB5.0	fCangjie	MHMBC
U+4AB5.0	fFourCorner	71280
U+4AB5.0	fCantonese	ngaa5; aa5 (also)
U+4AB5.0	fMandarin	ya3
U+4AB6.0	fUTF8	䪶
U+4AB6.0	fCNS11643	5-4B3E
U+4AB6.0	fR/S	 181.5
U+4AB6.0	fTotalStrokes	 14
U+4AB6.0	fCangjie	BMMBC
U+4AB6.0	fFourCorner	71180
U+4AB6.0	fCantonese	zeoi1
U+4AB6.0	fMandarin	ju1
U+4AB6.0	fEnglish	the chin; the jaws
U+4AB7.0	fUTF8	䪷
U+4AB7.0	fCNS11643	4-484D
U+4AB7.0	fGB	3-606B
U+4AB7.0	fR/S	 181.5
U+4AB7.0	fTotalStrokes	 14
U+4AB7.0	fCangjie	PRMBC
U+4AB7.0	fFourCorner	21680
U+4AB7.0	fCantonese	kau3; gaau3
U+4AB7.0	fMandarin	hou4; gou4
U+4AB7.0	fEnglish	 toil or labor sedulously
U+4AB8.0	fUTF8	䪸
U+4AB8.0	fCNS11643	3-4A51
U+4AB8.0	fGB	5-5F79
U+4AB8.0	fR/S	 181.5
U+4AB8.0	fTotalStrokes	 14
U+4AB8.0	fCangjie	RPMBC
U+4AB8.0	fFourCorner	71780
U+4AB8.0	fCantonese	man4
U+4AB8.0	fMandarin	min2
U+4AB8.0	fEnglish	[1] strong; powerful; vigorous [2] same as &#x9822;
U+4AB9.0	fUTF8	䪹
U+4AB9.0	fCNS11643	4-484E
U+4AB9.0	fGB	3-6067
U+4AB9.0	fR/S	 181.5
U+4AB9.0	fTotalStrokes	 14
U+4AB9.0	fCangjie	MMMBC
U+4AB9.0	fFourCorner	11180
U+4AB9.0	fCantonese	pei1; pai4; pui4
U+4AB9.0	fMandarin	pei2; pi1; pai2
U+4AB9.0	fEnglish	 large face
U+4ABA.0	fUTF8	䪺
U+4ABA.0	fCNS11643	3-4A53
U+4ABA.0	fR/S	 181.5
U+4ABA.0	fTotalStrokes	 14
U+4ABA.0	fCangjie	GIMBC
U+4ABA.0	fFourCorner	41780
U+4ABA.0	fCantonese	gap3; gok3
U+4ABA.0	fMandarin	ge2
U+4ABA.0	fEnglish	[1] (non-classical form) [2] teethridge; gum; the jaw bone [3] the skull [4] the top of the skull
U+4ABB.0	fUTF8	䪻
U+4ABB.0	fCNS11643	4-484C
U+4ABB.0	fGB	3-606D
U+4ABB.0	fR/S	 181.5
U+4ABB.0	fTotalStrokes	 14
U+4ABB.0	fCangjie	ITMBC
U+4ABB.0	fFourCorner	21480
U+4ABB.0	fCantonese	bin6
U+4ABB.0	fMandarin	bian4; fan4
U+4ABB.0	fEnglish	[1] cap [2] slanting cap [3] face [4] bald head
U+4ABC.0	fUTF8	䪼
U+4ABC.0	fCNS11643	4-484B
U+4ABC.0	fGB	3-606C
U+4ABC.0	fR/S	 181.5
U+4ABC.0	fTotalStrokes	 14
U+4ABC.0	fCangjie	UUMBC
U+4ABC.0	fFourCorner	21780
U+4ABC.0	fHakka	 chot7; zot7
U+4ABC.0	fCantonese	dyut3; zyut3
U+4ABC.0	fMandarin	zhuo2; gu3
U+4ABC.0	fEnglish	 cheek bones; physiognomy
U+4ABD.0	fUTF8	䪽
U+4ABD.0	fCNS11643	5-4B3F
U+4ABD.0	fR/S	 181.5
U+4ABD.0	fTotalStrokes	 14
U+4ABD.0	fCangjie	RSMBC
U+4ABD.0	fFourCorner	61280
U+4ABD.0	fCantonese	hou6
U+4ABD.0	fMandarin	hao4
U+4ABD.0	fEnglish	hoary-haired person
U+4ABE.0	fUTF8	䪾
U+4ABE.0	fCNS11643	4-4847
U+4ABE.0	fGB	3-606A
U+4ABE.0	fR/S	 181.5
U+4ABE.0	fTotalStrokes	 14
U+4ABE.0	fCangjie	MCOHH
U+4ABE.0	fFourCorner	18820
U+4ABE.0	fCantonese	can2; caan2; zaan2
U+4ABE.0	fMandarin	zhen3; mi2; zhen4
U+4ABE.0	fEnglish	[1] deal with affairs cautiously [2] head with less hair [3] ashamed; humiliated
U+4ABF.0	fUTF8	䪿
U+4ABF.0	fCNS11643	3-4F68
U+4ABF.0	fR/S	 181.6
U+4ABF.0	fTotalStrokes	 15
U+4ABF.0	fCangjie	HWMBC
U+4ABF.0	fFourCorner	21680
U+4ABF.0	fHakka	  sim5;  sin5
U+4ABF.0	fCantonese	seon3
U+4ABF.0	fMandarin	xin4; sheng3
U+4ABF.0	fEnglish	same as &#x56DF;
U+4AC0.0	fUTF8	䫀
U+4AC0.0	fCNS11643	4-4E59
U+4AC0.0	fGB	3-682E
U+4AC0.0	fR/S	 181.6
U+4AC0.0	fTotalStrokes	 15
U+4AC0.0	fCangjie	AIMBC
U+4AC0.0	fFourCorner	71780
U+4AC0.0	fCantonese	gan2
U+4AC0.0	fMandarin	gen3; dian3; jian3
U+4AC0.0	fEnglish	the lower end of the jaws, high cheek-bone, to bend (or lower) one's head
U+4AC1.0	fUTF8	䫁
U+4AC1.0	fCNS11643	4-4E5C
U+4AC1.0	fGB	3-6549
U+4AC1.0	fR/S	 181.6
U+4AC1.0	fTotalStrokes	 15
U+4AC1.0	fCangjie	HUMBC
U+4AC1.0	fFourCorner	21680
U+4AC1.0	fCantonese	bei6
U+4AC1.0	fMandarin	bi4
U+4AC1.0	fEnglish	[1] the first son [2] the dog give birth the first baby
U+4AC2.0	fUTF8	䫂
U+4AC2.0	fCNS11643	5-5260
U+4AC2.0	fR/S	 181.6
U+4AC2.0	fTotalStrokes	 15
U+4AC2.0	fCangjie	NNMBC
U+4AC2.0	fFourCorner	21280
U+4AC2.0	fCantonese	do2; do1
U+4AC2.0	fMandarin	duo3
U+4AC2.0	fEnglish	ugly
U+4AC3.0	fUTF8	䫃
U+4AC3.0	fCNS11643	4-546D
U+4AC3.0	fGB	5-6E3A
U+4AC3.0	fR/S	 181.7
U+4AC3.0	fTotalStrokes	 16
U+4AC3.0	fCangjie	MVMBC
U+4AC3.0	fFourCorner	71280
U+4AC3.0	fCantonese	seon4
U+4AC3.0	fMandarin	zhen4; chen2; chun2
U+4AC3.0	fEnglish	ancient form of &#x8123;, &#x5507;
U+4AC4.0	fUTF8	䫄
U+4AC4.0	fCNS11643	4-546F
U+4AC4.0	fGB	3-607C
U+4AC4.0	fR/S	 181.7
U+4AC4.0	fTotalStrokes	 16
U+4AC4.0	fCangjie	CUMBC
U+4AC4.0	fFourCorner	81280
U+4AC4.0	fCantonese	caat1
U+4AC4.0	fMandarin	chua4
U+4AC4.0	fEnglish	better than, small head, short face
U+4AC5.0	fUTF8	䫅
U+4AC5.0	fCNS11643	3-5356
U+4AC5.0	fR/S	 181.7
U+4AC5.0	fTotalStrokes	 16
U+4AC5.0	fCangjie	JDMBC
U+4AC5.0	fFourCorner	31980
U+4AC5.0	fCantonese	saam3; san3; tam4
U+4AC5.0	fMandarin	san4
U+4AC5.0	fEnglish	 shake one's head
U+4AC6.0	fUTF8	䫆
U+4AC6.0	fCNS11643	4-4E5D
U+4AC6.0	fGB	3-606E
U+4AC6.0	fR/S	 181.6; 181.7
U+4AC6.0	fTotalStrokes	 15; 16
U+4AC6.0	fCangjie	ISMBC
U+4AC6.0	fFourCorner	51280
U+4AC6.0	fCantonese	sing4; dip6; sing6
U+4AC6.0	fMandarin	cheng2; zheng4
U+4AC6.0	fEnglish	the front and back of the neck
U+4AC7.0	fUTF8	䫇
U+4AC7.0	fCNS11643	3-5360
U+4AC7.0	fR/S	 181.7
U+4AC7.0	fTotalStrokes	 16
U+4AC7.0	fCangjie	HMMBC
U+4AC7.0	fFourCorner	21580
U+4AC7.0	fCantonese	zim4
U+4AC7.0	fMandarin	ran2
U+4AC7.0	fEnglish	same as &#x9AEF;
U+4AC8.0	fUTF8	䫈
U+4AC8.0	fCNS11643	3-5357
U+4AC8.0	fGB	5-6024
U+4AC8.0	fR/S	 181.7
U+4AC8.0	fTotalStrokes	 16
U+4AC8.0	fCangjie	UNMBC
U+4AC8.0	fFourCorner	21280
U+4AC8.0	fHakka	 cem3; shim3; siim3; kim3
U+4AC8.0	fCantonese	zam6; cim4; saam4; zaam1
U+4AC8.0	fMandarin	zen4
U+4AC8.0	fEnglish	ugly; vulgar; coarse
U+4AC9.0	fUTF8	䫉
U+4AC9.0	fCNS11643	3-535E
U+4AC9.0	fGB	5-706B
U+4AC9.0	fR/S	 181.7
U+4AC9.0	fTotalStrokes	 16
U+4AC9.0	fCangjie	BHMBC
U+4AC9.0	fFourCorner	21280
U+4AC9.0	fCantonese	maau6
U+4AC9.0	fMandarin	mao4
U+4AC9.0	fEnglish	ancient form of &#x7683;, &#x8C8C;
U+4ACA.0	fUTF8	䫊
U+4ACA.0	fCNS11643	7-3551
U+4ACA.0	fR/S	 181.7
U+4ACA.0	fTotalStrokes	 16
U+4ACA.0	fCangjie	MRMBC
U+4ACA.0	fFourCorner	11680
U+4ACA.0	fCantonese	pui4
U+4ACA.0	fMandarin	pei2; bo2
U+4ACA.0	fEnglish	same as &#x4AB9;
U+4ACB.0	fUTF8	䫋
U+4ACB.0	fCNS11643	3-5725
U+4ACB.0	fGB	5-602E
U+4ACB.0	fR/S	 181.8
U+4ACB.0	fTotalStrokes	 17
U+4ACB.0	fCangjie	HVMBC
U+4ACB.0	fFourCorner	21480
U+4ACB.0	fCantonese	wai1; teoi4
U+4ACB.0	fMandarin	wei1; tui2
U+4ACB.0	fEnglish	[1] female attendants or servants [2] same as &#x96A4;
U+4ACC.0	fUTF8	䫌
U+4ACC.0	fCNS11643	4-5973
U+4ACC.0	fGB	3-607E
U+4ACC.0	fR/S	 181.8
U+4ACC.0	fTotalStrokes	 17; 16; 18
U+4ACC.0	fCangjie	HJMBC
U+4ACC.0	fFourCorner	21486
U+4ACC.0	fCantonese	pai2; pei2; pei3
U+4ACC.0	fMandarin	pi3; pi4
U+4ACC.0	fEnglish	bowed one's head
U+4ACD.0	fUTF8	䫍
U+4ACD.0	fCNS11643	7-3D69
U+4ACD.0	fR/S	 181.8
U+4ACD.0	fTotalStrokes	 17
U+4ACD.0	fCangjie	LYMBC
U+4ACD.0	fFourCorner	11180
U+4ACD.0	fCantonese	fu2
U+4ACD.0	fMandarin	fu3
U+4ACD.0	fEnglish	[1] corrupted form of &#x982B; [2] interchangeable &#x4FEF;
U+4ACE.0	fUTF8	䫎
U+4ACE.0	fCNS11643	3-5722
U+4ACE.0	fR/S	 181.8
U+4ACE.0	fTotalStrokes	 17
U+4ACE.0	fCangjie	EEEEC
U+4ACE.0	fFourCorner	71480
U+4ACE.0	fCantonese	dyut3; zyut3
U+4ACE.0	fMandarin	zhuo2
U+4ACE.0	fEnglish	short
U+4ACF.0	fUTF8	䫏
U+4ACF.0	fCNS11643	4-5977
U+4ACF.0	fGB	5-602A
U+4ACF.0	fR/S	 181.8
U+4ACF.0	fTotalStrokes	 17; 16
U+4ACF.0	fCangjie	TCMBC
U+4ACF.0	fFourCorner	41880
U+4ACF.0	fCantonese	hei1
U+4ACF.0	fMandarin	qi2
U+4ACF.0	fEnglish	standard form of &#x9B4C;
U+4AD0.0	fUTF8	䫐
U+4AD0.0	fCNS11643	3-5723
U+4AD0.0	fGB	5-602B
U+4AD0.0	fR/S	 181.8
U+4AD0.0	fTotalStrokes	 17
U+4AD0.0	fCangjie	DDMC
U+4AD0.0	fFourCorner	41980
U+4AD0.0	fCantonese	lam4; laam4; laau5
U+4AD0.0	fMandarin	lin2; lin4
U+4AD0.0	fEnglish	 bend (or lower) one's head; bow (usually refer to submission or admission of a wrong doing)
U+4AD1.0	fUTF8	䫑
U+4AD1.0	fCNS11643	3-5724
U+4AD1.0	fGB	5-602D
U+4AD1.0	fR/S	 181.8
U+4AD1.0	fTotalStrokes	 17
U+4AD1.0	fCangjie	KRMBC
U+4AD1.0	fFourCorner	41680
U+4AD1.0	fCantonese	ji1; hei1; zi1
U+4AD1.0	fMandarin	yi1
U+4AD1.0	fEnglish	[1] good; nice; fine [2] same as &#x4ACF; [3] standard form of &#x9B4C;
U+4AD2.0	fUTF8	䫒
U+4AD2.0	fCNS11643	3-5727
U+4AD2.0	fGB	5-6032
U+4AD2.0	fR/S	 181.8
U+4AD2.0	fTotalStrokes	 17
U+4AD2.0	fCangjie	HAMBC
U+4AD2.0	fFourCorner	71680
U+4AD2.0	fCantonese	mun4; mung4
U+4AD2.0	fMandarin	men2
U+4AD2.0	fEnglish	[1] disease [2] in a coma; apoplexy; unconscious; stupor
U+4AD3.0	fUTF8	䫓
U+4AD3.0	fCNS11643	3-5729
U+4AD3.0	fR/S	 181.8
U+4AD3.0	fTotalStrokes	 17
U+4AD3.0	fCangjie	YRMBC
U+4AD3.0	fFourCorner	01680
U+4AD3.0	fCantonese	ng4; ngaa4
U+4AD3.0	fMandarin	wu2
U+4AD3.0	fEnglish	(corrupted form) large head
U+4AD4.0	fUTF8	䫔
U+4AD4.0	fCNS11643	4-5E4E
U+4AD4.0	fGB	3-6123
U+4AD4.0	fR/S	 181.9
U+4AD4.0	fTotalStrokes	 18
U+4AD4.0	fCangjie	QKMBC
U+4AD4.0	fFourCorner	51480
U+4AD4.0	fCantonese	kai3; haai6
U+4AD4.0	fMandarin	qie4; qi4; xi4
U+4AD4.0	fEnglish	[1] servant [2] fear [3] short; a short person [4] angry [5] small headed
U+4AD5.0	fUTF8	䫕
U+4AD5.0	fCNS11643	3-5954
U+4AD5.0	fR/S	 181.9
U+4AD5.0	fTotalStrokes	 18
U+4AD5.0	fCangjie	YBMBC
U+4AD5.0	fFourCorner	01280
U+4AD5.0	fCantonese	dit3; zaat6
U+4AD5.0	fMandarin	die2; zhi4
U+4AD5.0	fEnglish	small headed
U+4AD6.0	fUTF8	䫖
U+4AD6.0	fCNS11643	4-5E49
U+4AD6.0	fGB	3-6124
U+4AD6.0	fR/S	 181.9
U+4AD6.0	fTotalStrokes	 18
U+4AD6.0	fCangjie	TVMBC
U+4AD6.0	fFourCorner	41780
U+4AD6.0	fCantonese	cam2
U+4AD6.0	fMandarin	chen3; shen4; hui3; nou4; ting2; ya4; zhu4
U+4AD6.0	fEnglish	timid; weak and dulll
U+4AD7.0	fUTF8	䫗
U+4AD7.0	fCNS11643	4-5E47
U+4AD7.0	fGB	3-612B
U+4AD7.0	fR/S	 181.9
U+4AD7.0	fTotalStrokes	 18
U+4AD7.0	fCangjie	RSSEC
U+4AD7.0	fFourCorner	71280
U+4AD7.0	fCantonese	haa4
U+4AD7.0	fMandarin	xia2; xia4
U+4AD7.0	fEnglish	[1] reckless; unrestrained; without limit (said of speech; words, etc.) [2] hard to tell
U+4AD8.0	fUTF8	䫘
U+4AD8.0	fCNS11643	4-5E4B
U+4AD8.0	fGB	3-6127
U+4AD8.0	fR/S	 181.9
U+4AD8.0	fTotalStrokes	 18
U+4AD8.0	fCangjie	AVMBC
U+4AD8.0	fFourCorner	61780
U+4AD8.0	fCantonese	hot6; hot3
U+4AD8.0	fMandarin	he2; he1; ke3
U+4AD8.0	fEnglish	[1] healthy; strong [2] hair-less
U+4AD9.0	fUTF8	䫙
U+4AD9.0	fCNS11643	3-5957
U+4AD9.0	fR/S	 181.9
U+4AD9.0	fTotalStrokes	 18
U+4AD9.0	fCangjie	JDMBC
U+4AD9.0	fFourCorner	41980
U+4AD9.0	fCantonese	song2
U+4AD9.0	fMandarin	sang3
U+4AD9.0	fEnglish	non-classical form of &#x9859;
U+4ADA.0	fUTF8	䫚
U+4ADA.0	fCNS11643	3-5955
U+4ADA.0	fR/S	 181.9
U+4ADA.0	fTotalStrokes	 18
U+4ADA.0	fCangjie	BBMBC
U+4ADA.0	fFourCorner	71280
U+4ADA.0	fCantonese	gwaa1
U+4ADA.0	fMandarin	gua1
U+4ADA.0	fEnglish	short headed
U+4ADB.0	fUTF8	䫛
U+4ADB.0	fCNS11643	4-5E4D
U+4ADB.0	fGB	3-612A
U+4ADB.0	fR/S	 181.9
U+4ADB.0	fTotalStrokes	 18
U+4ADB.0	fCangjie	ONKC
U+4ADB.0	fFourCorner	21280
U+4ADB.0	fCantonese	hau4
U+4ADB.0	fMandarin	hou2
U+4ADB.0	fEnglish	reckless; unrestrained; without limit (said of speech; words, etc.)
U+4ADC.0	fUTF8	䫜
U+4ADC.0	fCNS11643	4-5E4C
U+4ADC.0	fGB	3-6129
U+4ADC.0	fR/S	 181.9
U+4ADC.0	fTotalStrokes	 18
U+4ADC.0	fCangjie	UIMBC
U+4ADC.0	fFourCorner	21780
U+4ADC.0	fCantonese	au1; ngau1 (also); au3
U+4ADC.0	fMandarin	ao1
U+4ADC.0	fEnglish	[1] sunken eyes [2] hollow
U+4ADD.0	fUTF8	䫝
U+4ADD.0	fCNS11643	3-595A
U+4ADD.0	fR/S	 181.9
U+4ADD.0	fTotalStrokes	 18
U+4ADD.0	fCangjie	HAMBC
U+4ADD.0	fFourCorner	21680
U+4ADD.0	fCantonese	fuk1
U+4ADD.0	fMandarin	fu4; fu3
U+4ADD.0	fEnglish	same as &#x99A5;
U+4ADE.0	fUTF8	䫞
U+4ADE.0	fCNS11643	4-6227
U+4ADE.0	fGB	3-6132
U+4ADE.0	fR/S	 181.10
U+4ADE.0	fTotalStrokes	 19
U+4ADE.0	fCangjie	TFMBC
U+4ADE.0	fFourCorner	81380
U+4ADE.0	fCantonese	hiu1; kiu5
U+4ADE.0	fMandarin	xiao1; rao2; qiao1; xiao4; fen2
U+4ADE.0	fEnglish	[1] large head [2] a wide forehead [3] numerous; crowd [4] interchangeable with &#x9812;
U+4ADF.0	fUTF8	䫟
U+4ADF.0	fCNS11643	4-6229
U+4ADF.0	fGB	3-6079
U+4ADF.0	fR/S	 181.10
U+4ADF.0	fTotalStrokes	 19
U+4ADF.0	fCangjie	RCMBC
U+4ADF.0	fFourCorner	61880
U+4ADF.0	fCantonese	wan6
U+4ADF.0	fMandarin	hun2; hun4; yun3; wen3
U+4ADF.0	fEnglish	[1] facial complexion (yellowish because of worrying) [2] a round skull and face
U+4AE0.0	fUTF8	䫠
U+4AE0.0	fCNS11643	4-622D
U+4AE0.0	fGB	3-6131
U+4AE0.0	fR/S	 181.10
U+4AE0.0	fTotalStrokes	 19
U+4AE0.0	fCangjie	HRMBC
U+4AE0.0	fFourCorner	21686
U+4AE0.0	fCantonese	pei1
U+4AE0.0	fMandarin	pei2; pi1
U+4AE0.0	fEnglish	short hair and beard
U+4AE1.0	fUTF8	䫡
U+4AE1.0	fCNS11643	4-6226
U+4AE1.0	fGB	3-6133
U+4AE1.0	fR/S	 181.10
U+4AE1.0	fTotalStrokes	 19
U+4AE1.0	fCangjie	TCMBC
U+4AE1.0	fFourCorner	81386
U+4AE1.0	fCantonese	ngaam4; aam4 (also); him3; ngam4
U+4AE1.0	fMandarin	yan2; qian4; jian4; qian3
U+4AE1.0	fEnglish	[1] a long shape of head and face [2] the cheeks; the jaw
U+4AE2.0	fUTF8	䫢
U+4AE2.0	fCNS11643	4-622C
U+4AE2.0	fGB	3-6130
U+4AE2.0	fR/S	 181.10
U+4AE2.0	fTotalStrokes	 19
U+4AE2.0	fCangjie	HUMBC
U+4AE2.0	fFourCorner	71280
U+4AE2.0	fCantonese	si1
U+4AE2.0	fMandarin	si1
U+4AE2.0	fEnglish	[1] good; fine; excellent [2] pleasing [3] wry neck
U+4AE3.0	fUTF8	䫣
U+4AE3.0	fCNS11643	3-5B64
U+4AE3.0	fR/S	 181.10
U+4AE3.0	fTotalStrokes	 19
U+4AE3.0	fCangjie	BKMBC
U+4AE3.0	fFourCorner	21480
U+4AE3.0	fCantonese	hai4; haai4; kaai3; kit3
U+4AE3.0	fMandarin	xi1; xi2
U+4AE3.0	fEnglish	 wry neck
U+4AE4.0	fUTF8	䫤
U+4AE4.0	fCNS11643	4-6225
U+4AE4.0	fGB	5-603E
U+4AE4.0	fR/S	 181.10
U+4AE4.0	fTotalStrokes	 19
U+4AE4.0	fCangjie	BCMBC
U+4AE4.0	fFourCorner	31880
U+4AE4.0	fCantonese	mang1; ming4
U+4AE4.0	fMandarin	ming2
U+4AE4.0	fEnglish	between the eyebrows and eyes; general facial appearance
U+4AE5.0	fUTF8	䫥
U+4AE5.0	fCNS11643	4-622F
U+4AE5.0	fGB	3-7A39
U+4AE5.0	fR/S	 181.10
U+4AE5.0	fTotalStrokes	 19
U+4AE5.0	fCangjie	HIMBC
U+4AE5.0	fFourCorner	21286
U+4AE5.0	fCantonese	fui2; ngaai5
U+4AE5.0	fMandarin	wei3; kui3; qi1
U+4AE5.0	fEnglish	[1] wry neck [2] large head
U+4AE6.0	fUTF8	䫦
U+4AE6.0	fCNS11643	4-6228
U+4AE6.0	fGB	3-612E
U+4AE6.0	fR/S	 181.10
U+4AE6.0	fTotalStrokes	 19
U+4AE6.0	fCangjie	GTMBC
U+4AE6.0	fFourCorner	41180
U+4AE6.0	fCantonese	gap3; gok3; haap3; koi3
U+4AE6.0	fMandarin	ge2; kai4
U+4AE6.0	fEnglish	[1] the lower jawbone [2] the cranium
U+4AE7.0	fUTF8	䫧
U+4AE7.0	fCNS11643	4-622B
U+4AE7.0	fGB	3-612F
U+4AE7.0	fR/S	 181.10
U+4AE7.0	fTotalStrokes	 19
U+4AE7.0	fCangjie	HJMBC
U+4AE7.0	fFourCorner	21480
U+4AE7.0	fCantonese	hou6
U+4AE8.0	fUTF8	䫨
U+4AE8.0	fCNS11643	4-652F
U+4AE8.0	fGB	3-612C
U+4AE8.0	fR/S	 181.11
U+4AE8.0	fTotalStrokes	 20
U+4AE8.0	fCangjie	GSOKC
U+4AE8.0	fFourCorner	41280
U+4AE8.0	fCantonese	zeoi6; ngou6
U+4AE8.0	fMandarin	ao2; ao4
U+4AE8.0	fEnglish	tall and big; colossal
U+4AE9.0	fUTF8	䫩
U+4AE9.0	fCNS11643	4-6531
U+4AE9.0	fGB	3-6122
U+4AE9.0	fR/S	 181.11
U+4AE9.0	fTotalStrokes	 20
U+4AE9.0	fCangjie	IHMBC
U+4AE9.0	fFourCorner	21280
U+4AE9.0	fCantonese	sam2; san2
U+4AE9.0	fMandarin	san3; can3
U+4AE9.0	fEnglish	 shake one's head
U+4AEA.0	fUTF8	䫪
U+4AEA.0	fCNS11643	4-6530
U+4AEA.0	fGB	3-6136
U+4AEA.0	fR/S	 181.11
U+4AEA.0	fTotalStrokes	 20
U+4AEA.0	fCangjie	KKMBC
U+4AEA.0	fFourCorner	41080
U+4AEA.0	fCantonese	song2; sin1
U+4AEA.0	fMandarin	shuang3
U+4AEA.0	fEnglish	ugly; repulsive
U+4AEB.0	fUTF8	䫫
U+4AEB.0	fCNS11643	3-5D54
U+4AEB.0	fR/S	 181.11
U+4AEB.0	fTotalStrokes	 20
U+4AEB.0	fCangjie	LVMBC
U+4AEB.0	fFourCorner	51480
U+4AEB.0	fCantonese	lau4
U+4AEB.0	fMandarin	lou2; lou4
U+4AEB.0	fEnglish	[1] same as &#x9ACF; [2] interchangeable &#x93E4;
U+4AEC.0	fUTF8	䫬
U+4AEC.0	fCNS11643	4-6777
U+4AEC.0	fGB	3-6139
U+4AEC.0	fR/S	 181.12
U+4AEC.0	fTotalStrokes	 21
U+4AEC.0	fCangjie	MAMBC
U+4AEC.0	fFourCorner	71680
U+4AEC.0	fHakka	 cim3; zhim3; zihm3
U+4AEC.0	fCantonese	zam2
U+4AEC.0	fMandarin	zhen3; qin3; zen4
U+4AEC.0	fEnglish	 hang the head, as when sleepy
U+4AED.0	fUTF8	䫭
U+4AED.0	fCNS11643	4-6779
U+4AED.0	fGB	3-6128
U+4AED.0	fR/S	 181.12
U+4AED.0	fTotalStrokes	 21
U+4AED.0	fCangjie	LCMBC
U+4AED.0	fFourCorner	51880
U+4AED.0	fCantonese	wui6; kui2
U+4AED.0	fMandarin	hui4
U+4AED.0	fEnglish	bald-headed
U+4AEE.0	fUTF8	䫮
U+4AEE.0	fCNS11643	4-6776
U+4AEE.0	fGB	3-6078
U+4AEE.0	fR/S	 181.12
U+4AEE.0	fTotalStrokes	 21
U+4AEE.0	fCangjie	SIMBC
U+4AEE.0	fFourCorner	11380
U+4AEE.0	fCantonese	saam4; cam4
U+4AEE.0	fMandarin	chan2; can2; can3; ti4
U+4AEE.0	fEnglish	[1] bend or lower of one's head; bow (usually refer to submission or admission of a wrong doing) [2] take [3] choose [4] dwell [5] stop
U+4AEF.0	fUTF8	䫯
U+4AEF.0	fCNS11643	4-677C
U+4AEF.0	fGB	3-613A
U+4AEF.0	fR/S	 181.12
U+4AEF.0	fTotalStrokes	 21
U+4AEF.0	fCangjie	YSKRC
U+4AEF.0	fFourCorner	01280
U+4AEF.0	fCantonese	ji1; ci4; kei1; saai1; zi1
U+4AEF.0	fMandarin	yi1
U+4AEF.0	fEnglish	[1] (abbreviated form) [2] take a casual look at [3] look sideways [4] female beauty; handsome
U+4AF0.0	fUTF8	䫰
U+4AF0.0	fCNS11643	4-677B
U+4AF0.0	fGB	3-613B
U+4AF0.0	fR/S	 181.12
U+4AF0.0	fTotalStrokes	 21
U+4AF0.0	fCangjie	MCFDQ
U+4AF0.0	fFourCorner	19850
U+4AF0.0	fCantonese	leon6
U+4AF0.0	fMandarin	lin3; lin4
U+4AF0.0	fEnglish	sparse hair
U+4AF1.0	fUTF8	䫱
U+4AF1.0	fCNS11643	7-5632
U+4AF1.0	fR/S	 181.12
U+4AF1.0	fTotalStrokes	 21
U+4AF1.0	fCangjie	MBMBC
U+4AF1.0	fFourCorner	11280
U+4AF1.0	fCantonese	jyu4; zyu4
U+4AF1.0	fMandarin	ru2; na2
U+4AF1.0	fEnglish	same as &#x986C;
U+4AF2.0	fUTF8	䫲
U+4AF2.0	fCNS11643	4-6963
U+4AF2.0	fGB	3-613E
U+4AF2.0	fR/S	 181.13
U+4AF2.0	fTotalStrokes	 22; 21
U+4AF2.0	fCangjie	IPMBC
U+4AF2.0	fFourCorner	51386
U+4AF2.0	fCantonese	ham3; haam2; zim2
U+4AF2.0	fMandarin	kan3; han4; han3
U+4AF2.0	fEnglish	[1] thin and sickly in appearance; emaciated look [2] move one's head
U+4AF3.0	fUTF8	䫳
U+4AF3.0	fCNS11643	3-5F76
U+4AF3.0	fR/S	 181.13
U+4AF3.0	fTotalStrokes	 22
U+4AF3.0	fCangjie	WIMBC
U+4AF3.0	fFourCorner	61180
U+4AF3.0	fCantonese	duk6
U+4AF3.0	fMandarin	du2
U+4AF3.0	fEnglish	same as &#x9AD1;
U+4AF4.0	fUTF8	䫴
U+4AF4.0	fCNS11643	4-6962
U+4AF4.0	fGB	3-613D
U+4AF4.0	fR/S	 181.13
U+4AF4.0	fTotalStrokes	 22; 21
U+4AF4.0	fCangjie	DFMBC
U+4AF4.0	fFourCorner	41980
U+4AF4.0	fHakka	 gim5; gim3
U+4AF4.0	fCantonese	gam6; gaam3; ging6
U+4AF4.0	fMandarin	jin4; jing4; yin4
U+4AF4.0	fEnglish	[1] gnash the teeth with rage [2] exhausted; without energy
U+4AF5.0	fUTF8	䫵
U+4AF5.0	fCNS11643	4-6C39
U+4AF5.0	fGB	3-613F
U+4AF5.0	fR/S	 181.15
U+4AF5.0	fTotalStrokes	 24
U+4AF5.0	fCangjie	HSMBC
U+4AF5.0	fFourCorner	21280
U+4AF5.0	fCantonese	min4
U+4AF5.0	fMandarin	mian2
U+4AF5.0	fEnglish	twins
U+4AF6.0	fUTF8	䫶
U+4AF6.0	fCNS11643	5-7848
U+4AF6.0	fR/S	 181.15
U+4AF6.0	fTotalStrokes	 24
U+4AF6.0	fCangjie	DKMBC
U+4AF6.0	fFourCorner	41480
U+4AF6.0	fCantonese	fan4
U+4AF6.0	fMandarin	fan2; fen2; fan3
U+4AF6.0	fEnglish	ugly
U+4AF7.0	fUTF8	䫷
U+4AF7.0	fCNS11643	3-6155
U+4AF7.0	fGB	5-6049
U+4AF7.0	fR/S	 181.16
U+4AF7.0	fTotalStrokes	 25
U+4AF7.0	fCangjie	MRMBC
U+4AF7.0	fFourCorner	11180
U+4AF7.0	fCantonese	ngok6; ok6 (also)
U+4AF7.0	fMandarin	e4
U+4AF7.0	fEnglish	same as &#x984E;
U+4AF8.0	fUTF8	䫸
U+4AF8.0	fCNS11643	4-362B
U+4AF8.0	fGB	3-4E5C
U+4AF8.0	fR/S	 182.2
U+4AF8.0	fTotalStrokes	 11
U+4AF8.0	fCangjie	HNSH
U+4AF8.0	fFourCorner	77210
U+4AF8.0	fCantonese	caau1; siu1
U+4AF8.0	fMandarin	chao1; nao2
U+4AF8.0	fEnglish	[1] blow [2] hot wind; wind of heat
U+4AF9.0	fUTF8	䫹
U+4AF9.0	fCNS11643	4-3B7E
U+4AF9.0	fGB	3-4E5D
U+4AF9.0	fR/S	 182.3
U+4AF9.0	fTotalStrokes	 12
U+4AF9.0	fCangjie	HNM
U+4AF9.0	fFourCorner	71210
U+4AF9.0	fCantonese	hung4; kyun4
U+4AF9.0	fMandarin	hong2
U+4AF9.0	fEnglish	[1] (non-classical form) [2] the sound of the wind [3] typhoon; great gale of wind
U+4AFA.0	fUTF8	䫺
U+4AFA.0	fCNS11643	3-4553
U+4AFA.0	fR/S	 182.4
U+4AFA.0	fTotalStrokes	 13
U+4AFA.0	fCangjie	KIHNI
U+4AFA.0	fFourCorner	47010
U+4AFA.0	fCantonese	wang4
U+4AFA.0	fMandarin	hong2; hou4
U+4AFA.0	fEnglish	 typhoon; great gale of wind
U+4AFB.0	fUTF8	䫻
U+4AFB.0	fCNS11643	4-416D
U+4AFB.0	fGB	3-4E62
U+4AFB.0	fR/S	 182.4
U+4AFB.0	fTotalStrokes	 13
U+4AFB.0	fCangjie	HNA
U+4AFB.0	fFourCorner	76210
U+4AFB.0	fHakka	 lut8
U+4AFB.0	fCantonese	jat6; waat6
U+4AFB.0	fMandarin	yu4; xue2
U+4AFB.0	fEnglish	gale of wind
U+4AFC.0	fUTF8	䫼
U+4AFC.0	fCNS11643	4-4172
U+4AFC.0	fGB	3-4E64
U+4AFC.0	fR/S	 182.4
U+4AFC.0	fTotalStrokes	 13
U+4AFC.0	fCangjie	HNDK
U+4AFC.0	fFourCorner	75210
U+4AFC.0	fCantonese	hyut3
U+4AFC.0	fMandarin	xue4
U+4AFC.0	fEnglish	[1] breeze; light wind [2] the sound of the wind
U+4AFD.0	fUTF8	䫽
U+4AFD.0	fCNS11643	4-416E
U+4AFD.0	fGB	3-4E63
U+4AFD.0	fR/S	 182.4
U+4AFD.0	fTotalStrokes	 13
U+4AFD.0	fCangjie	HNHQU
U+4AFD.0	fFourCorner	72210
U+4AFD.0	fCantonese	pou1; pau1
U+4AFD.0	fMandarin	pao1
U+4AFD.0	fEnglish	breeze; light wind
U+4AFE.0	fUTF8	䫾
U+4AFE.0	fCNS11643	4-4851
U+4AFE.0	fGB	3-4E68
U+4AFE.0	fR/S	 182.5
U+4AFE.0	fTotalStrokes	 14
U+4AFE.0	fCangjie	HNPH
U+4AFE.0	fFourCorner	73210
U+4AFE.0	fCantonese	bik1
U+4AFE.0	fMandarin	pie1; bi4; bi1; bie2
U+4AFE.0	fEnglish	[1] breeze; light wind [2] cold or flu
U+4AFF.0	fUTF8	䫿
U+4AFF.0	fCNS11643	4-4856
U+4AFF.0	fGB	3-4E69
U+4AFF.0	fR/S	 182.5
U+4AFF.0	fTotalStrokes	 14
U+4AFF.0	fCangjie	HNSHR
U+4AFF.0	fFourCorner	77210
U+4AFF.0	fCantonese	ciu1; coi2; coi3
U+4AFF.0	fMandarin	chao1
U+4AFF.0	fEnglish	 cool breeze; cold wind from the north; cold wind from the south-west
U+4B00.0	fUTF8	䬀
U+4B00.0	fCNS11643	4-4859
U+4B00.0	fGB	3-4E6A
U+4B00.0	fR/S	 182.5
U+4B00.0	fTotalStrokes	 14
U+4B00.0	fCangjie	HNVIS
U+4B00.0	fFourCorner	74210
U+4B00.0	fCantonese	jau5; zaau1
U+4B00.0	fMandarin	you3; you1
U+4B00.0	fEnglish	the sound of the wind
U+4B01.0	fUTF8	䬁
U+4B01.0	fCNS11643	4-4850
U+4B01.0	fGB	3-4E67
U+4B01.0	fR/S	 182.5
U+4B01.0	fTotalStrokes	 14
U+4B01.0	fCangjie	HNJP
U+4B01.0	fFourCorner	73210
U+4B01.0	fCantonese	ji4; naa5; zyu4; zyu6
U+4B01.0	fMandarin	yi2
U+4B01.0	fEnglish	 light cyclone; whirlwind
U+4B02.0	fUTF8	䬂
U+4B02.0	fCNS11643	4-4854
U+4B02.0	fGB	3-4E65
U+4B02.0	fR/S	 182.5
U+4B02.0	fTotalStrokes	 14
U+4B02.0	fCangjie	HNIV
U+4B02.0	fFourCorner	73210
U+4B02.0	fCantonese	hyut3; zyut6
U+4B02.0	fMandarin	yue4; xue4
U+4B02.0	fEnglish	breeze; light wind
U+4B03.0	fUTF8	䬃
U+4B03.0	fCNS11643	3-4A57
U+4B03.0	fR/S	 182.5
U+4B03.0	fTotalStrokes	 14
U+4B03.0	fCangjie	HNYT
U+4B03.0	fFourCorner	70210
U+4B03.0	fHakka	 sap7
U+4B03.0	fCantonese	saap3; sap3; seon3; sou3
U+4B03.0	fMandarin	sa4
U+4B03.0	fEnglish	same as &#x98AF;
U+4B04.0	fUTF8	䬄
U+4B04.0	fCNS11643	4-4E60
U+4B04.0	fGB	3-4E6B
U+4B04.0	fR/S	 182.6
U+4B04.0	fTotalStrokes	 15
U+4B04.0	fCangjie	HNIHI
U+4B04.0	fFourCorner	73210
U+4B04.0	fCantonese	hyut3; syu3; waat6; zyut6
U+4B04.0	fMandarin	xi2; yu4
U+4B04.0	fEnglish	breeze; light wind
U+4B05.0	fUTF8	䬅
U+4B05.0	fCNS11643	4-4E61
U+4B05.0	fGB	3-4E70
U+4B05.0	fR/S	 182.6
U+4B05.0	fTotalStrokes	 15
U+4B05.0	fCangjie	HNKSS
U+4B05.0	fFourCorner	74210
U+4B05.0	fCantonese	lai6; lik6
U+4B05.0	fMandarin	xie2; li4; lie4
U+4B05.0	fEnglish	[1] gale; gust of swift wind [2] timely wind
U+4B06.0	fUTF8	䬆
U+4B06.0	fCNS11643	4-5476
U+4B06.0	fGB	3-4E74
U+4B06.0	fR/S	 182.7
U+4B06.0	fTotalStrokes	 16
U+4B06.0	fCangjie	HNHDN
U+4B06.0	fFourCorner	72210
U+4B06.0	fHakka	 lit8; li5; li6
U+4B06.0	fCantonese	leot6
U+4B06.0	fMandarin	li4
U+4B06.0	fEnglish	driving wind and rain
U+4B07.0	fUTF8	䬇
U+4B07.0	fCNS11643	4-5477
U+4B07.0	fGB	5-504F
U+4B07.0	fR/S	 182.7
U+4B07.0	fTotalStrokes	 16
U+4B07.0	fCangjie	CUHNI
U+4B07.0	fFourCorner	87210
U+4B07.0	fCantonese	jyun6; waat6; zyun6
U+4B07.0	fMandarin	yuan4
U+4B07.0	fEnglish	breeze; light wind
U+4B08.0	fUTF8	䬈
U+4B08.0	fCNS11643	4-5473
U+4B08.0	fGB	3-4E75
U+4B08.0	fR/S	 182.7
U+4B08.0	fTotalStrokes	 16
U+4B08.0	fCangjie	HNCRU
U+4B08.0	fFourCorner	78210
U+4B08.0	fCantonese	deoi6; deoi3
U+4B08.0	fMandarin	dui4
U+4B08.0	fEnglish	breeze; light wind
U+4B09.0	fUTF8	䬉
U+4B09.0	fCNS11643	4-5474
U+4B09.0	fGB	3-4E72
U+4B09.0	fR/S	 182.7
U+4B09.0	fTotalStrokes	 16
U+4B09.0	fCangjie	HNGLC
U+4B09.0	fFourCorner	74210
U+4B09.0	fCantonese	waak6
U+4B09.0	fMandarin	huo4; hao4
U+4B09.0	fEnglish	hot wind; wind of heat
U+4B0A.0	fUTF8	䬊
U+4B0A.0	fCNS11643	4-5475
U+4B0A.0	fGB	3-4E6D
U+4B0A.0	fR/S	 182.7
U+4B0A.0	fTotalStrokes	 16
U+4B0A.0	fCangjie	HNKOO
U+4B0A.0	fFourCorner	74210
U+4B0A.0	fCantonese	saap3
U+4B0A.0	fMandarin	sha4; qie4
U+4B0A.0	fEnglish	 gale; gust of swift wind
U+4B0B.0	fUTF8	䬋
U+4B0B.0	fCNS11643	4-597D
U+4B0B.0	fGB	3-4E76
U+4B0B.0	fR/S	 182.8
U+4B0B.0	fTotalStrokes	 17; 16
U+4B0B.0	fCangjie	HNGCE
U+4B0B.0	fFourCorner	74210
U+4B0B.0	fCantonese	ling4
U+4B0B.0	fMandarin	leng2
U+4B0B.0	fEnglish	violent wind; strong wind
U+4B0C.0	fUTF8	䬌
U+4B0C.0	fCNS11643	4-5A22
U+4B0C.0	fGB	3-614A
U+4B0C.0	fR/S	 182.8
U+4B0C.0	fTotalStrokes	 17; 16
U+4B0C.0	fCangjie	YUHNI
U+4B0C.0	fFourCorner	27210
U+4B0C.0	fCantonese	piu1; paau1
U+4B0C.0	fMandarin	pou1
U+4B0C.0	fEnglish	[1] the wind blows [2] blown by the wind
U+4B0D.0	fUTF8	䬍
U+4B0D.0	fCNS11643	4-5A21
U+4B0D.0	fGB	3-4E79
U+4B0D.0	fR/S	 182.8
U+4B0D.0	fTotalStrokes	 17; 16
U+4B0D.0	fCangjie	HNPHP
U+4B0D.0	fFourCorner	77210
U+4B0D.0	fCantonese	fat1
U+4B0D.0	fMandarin	hu1; xu4
U+4B0D.0	fEnglish	[1] the sound of wind [2] gale; typhoon
U+4B0E.0	fUTF8	䬎
U+4B0E.0	fCNS11643	4-597C
U+4B0E.0	fGB	3-4E77
U+4B0E.0	fR/S	 182.8
U+4B0E.0	fTotalStrokes	 17; 16
U+4B0E.0	fCangjie	HNIRM
U+4B0E.0	fFourCorner	73210
U+4B0E.0	fCantonese	gwok3; gok3 (also); kwong4
U+4B0E.0	fMandarin	guo2; xu4
U+4B0E.0	fEnglish	[1] hot air; hot vapor [2] wind of the heat
U+4B0F.0	fUTF8	䬏
U+4B0F.0	fCNS11643	4-597A
U+4B0F.0	fGB	3-4E7B
U+4B0F.0	fR/S	 182.8
U+4B0F.0	fTotalStrokes	 17; 16
U+4B0F.0	fCangjie	HNYTR
U+4B0F.0	fFourCorner	70210
U+4B0F.0	fCantonese	bau6; faau2
U+4B0F.0	fMandarin	fou3; bu4
U+4B0F.0	fEnglish	wind; light breezes
U+4B10.0	fUTF8	䬐
U+4B10.0	fCNS11643	4-597E
U+4B10.0	fGB	3-4E78
U+4B10.0	fR/S	 182.8
U+4B10.0	fTotalStrokes	 17; 16
U+4B10.0	fCangjie	HNHDV
U+4B10.0	fFourCorner	72210
U+4B10.0	fCantonese	jeoi4; waai2
U+4B10.0	fMandarin	wei1; wei3; rui2
U+4B10.0	fEnglish	breeze
U+4B11.0	fUTF8	䬑
U+4B11.0	fCNS11643	4-5E51
U+4B11.0	fGB	3-4E7C
U+4B11.0	fR/S	 182.9
U+4B11.0	fTotalStrokes	 18
U+4B11.0	fCangjie	HNWB
U+4B11.0	fFourCorner	76210
U+4B11.0	fCantonese	wai6; waai6; waan3
U+4B11.0	fMandarin	wei4; yu4
U+4B11.0	fEnglish	[1] gale; typhoon [2] the sound of wind
U+4B12.0	fUTF8	䬒
U+4B12.0	fCNS11643	7-447E
U+4B12.0	fR/S	 182.9
U+4B12.0	fTotalStrokes	 18
U+4B12.0	fCangjie	HNJFE
U+4B12.0	fFourCorner	73210
U+4B12.0	fHakka	  seu1
U+4B12.0	fCantonese	sau1; saau1; siu1; wai4
U+4B12.0	fMandarin	sou1; sou3; xiao1
U+4B12.0	fEnglish	[1] blow chilly, as the wind [2] a whistling sound
U+4B13.0	fUTF8	䬓
U+4B13.0	fCNS11643	4-5E50
U+4B13.0	fGB	3-4F21
U+4B13.0	fR/S	 182.9
U+4B13.0	fTotalStrokes	 18
U+4B13.0	fCangjie	HNYTA
U+4B13.0	fFourCorner	70210
U+4B13.0	fHakka	 am5
U+4B13.0	fCantonese	am3; ngam3 (also); aam3; kaau1
U+4B13.0	fMandarin	an4; ang3
U+4B13.0	fEnglish	 squall; hurricane
U+4B14.0	fUTF8	䬔
U+4B14.0	fCNS11643	4-5E52
U+4B14.0	fGB	3-4E7E
U+4B14.0	fR/S	 182.9
U+4B14.0	fTotalStrokes	 18
U+4B14.0	fCangjie	HNOMN
U+4B14.0	fFourCorner	78210
U+4B14.0	fHakka	 ji2; ri2
U+4B14.0	fCantonese	jyu4; zyu4
U+4B14.0	fMandarin	xu4; yu2
U+4B14.0	fEnglish	 hurricane
U+4B15.0	fUTF8	䬕
U+4B15.0	fCNS11643	3-595D
U+4B15.0	fR/S	 182.9
U+4B15.0	fTotalStrokes	 18
U+4B15.0	fCangjie	HNHNI
U+4B15.0	fFourCorner	77210
U+4B15.0	fCantonese	hong1; hoeng1
U+4B15.0	fMandarin	qiang3; shang3; qiang4
U+4B15.0	fEnglish	[1] the sound of the wind [2] degenerate social customs and practices
U+4B16.0	fUTF8	䬖
U+4B16.0	fCNS11643	3-595E
U+4B16.0	fR/S	 182.9
U+4B16.0	fTotalStrokes	 18
U+4B16.0	fCangjie	HNHAG
U+4B16.0	fFourCorner	76210
U+4B16.0	fCantonese	waang4
U+4B16.0	fMandarin	heng2
U+4B16.0	fEnglish	same as &#x4B1D;
U+4B17.0	fUTF8	䬗
U+4B17.0	fCNS11643	3-595F
U+4B17.0	fR/S	 182.9
U+4B17.0	fTotalStrokes	 18
U+4B17.0	fCangjie	AHHNI
U+4B17.0	fFourCorner	67210
U+4B17.0	fCantonese	joeng4; zoeng4
U+4B17.0	fMandarin	yang2
U+4B17.0	fEnglish	non-classical form of &#x98BA;
U+4B18.0	fUTF8	䬘
U+4B18.0	fCNS11643	4-6232
U+4B18.0	fGB	3-4F24
U+4B18.0	fR/S	 182.10
U+4B18.0	fTotalStrokes	 19
U+4B18.0	fCangjie	HNYRB
U+4B18.0	fFourCorner	70210
U+4B18.0	fCantonese	haau1; gwik1; hau1; hou3; man5; zing4
U+4B18.0	fMandarin	xiao1
U+4B18.0	fEnglish	[1] the wind blows [2] blown by the wind
U+4B19.0	fUTF8	䬙
U+4B19.0	fCNS11643	3-5B67
U+4B19.0	fR/S	 182.10
U+4B19.0	fTotalStrokes	 19
U+4B19.0	fCangjie	HNBOU
U+4B19.0	fFourCorner	77217
U+4B19.0	fCantonese	jiu4; ziu4
U+4B19.0	fMandarin	yao2
U+4B19.0	fEnglish	same as &#x98BB;
U+4B1A.0	fUTF8	䬚
U+4B1A.0	fCNS11643	3-5B66
U+4B1A.0	fR/S	 182.10
U+4B1A.0	fTotalStrokes	 19
U+4B1A.0	fCangjie	HNSQF
U+4B1A.0	fFourCorner	71212
U+4B1A.0	fCantonese	faan4; fan3; fan4; kaau1
U+4B1A.0	fMandarin	fan2
U+4B1A.0	fEnglish	[1] same as &#x98BF; [2] same as &#x5E06;
U+4B1B.0	fUTF8	䬛
U+4B1B.0	fCNS11643	4-6536
U+4B1B.0	fGB	3-4E6E
U+4B1B.0	fR/S	 182.11
U+4B1B.0	fTotalStrokes	 20
U+4B1B.0	fCangjie	HNWTJ
U+4B1B.0	fFourCorner	76210
U+4B1B.0	fCantonese	bat1; bit1
U+4B1B.0	fMandarin	bi4; yu4
U+4B1B.0	fEnglish	the roaring of the wind, cold wind
U+4B1C.0	fUTF8	䬜
U+4B1C.0	fCNS11643	3-5D55
U+4B1C.0	fR/S	 182.11
U+4B1C.0	fTotalStrokes	 20
U+4B1C.0	fCangjie	HNYUB
U+4B1C.0	fFourCorner	70210
U+4B1C.0	fCantonese	ci1
U+4B1C.0	fMandarin	chi1
U+4B1C.0	fEnglish	same as standard form &#x9B51;
U+4B1D.0	fUTF8	䬝
U+4B1D.0	fCNS11643	4-677E
U+4B1D.0	fGB	3-4F25
U+4B1D.0	fR/S	 182.12
U+4B1D.0	fTotalStrokes	 21
U+4B1D.0	fCangjie	HNTMC
U+4B1D.0	fFourCorner	74210
U+4B1D.0	fCantonese	waang4
U+4B1D.0	fMandarin	heng2; hong1; hong4
U+4B1D.0	fEnglish	interchangeable &#x4B16;
U+4B1E.0	fUTF8	䬞
U+4B1E.0	fCNS11643	4-6B30
U+4B1E.0	fGB	3-4E71
U+4B1E.0	fR/S	 182.14
U+4B1E.0	fTotalStrokes	 23
U+4B1E.0	fCangjie	HNGNI
U+4B1E.0	fFourCorner	74210
U+4B1E.0	fCantonese	tou4
U+4B1E.0	fMandarin	chou2; tao2
U+4B1E.0	fEnglish	[1] wind; strom [2] the sound of the wind
U+4B1F.0	fUTF8	䬟
U+4B1F.0	fCNS11643	4-6C3A
U+4B1F.0	fGB	3-4E6F
U+4B1F.0	fR/S	 182.15
U+4B1F.0	fTotalStrokes	 24
U+4B1F.0	fCangjie	HNHCN
U+4B1F.0	fFourCorner	72210
U+4B1F.0	fCantonese	lau4
U+4B1F.0	fMandarin	liu2; liu3
U+4B1F.0	fEnglish	the sound of wind
U+4B20.0	fUTF8	䬠
U+4B20.0	fCNS11643	3-572B
U+4B20.0	fR/S	 183.8; 183.9; 173.9
U+4B20.0	fTotalStrokes	 17
U+4B20.0	fCangjie	MBNOO
U+4B20.0	fFourCorner	10410
U+4B20.0	fCantonese	fei1
U+4B20.0	fEnglish	ancient form of &#x970F;
U+4B21.0	fUTF8	䬡
U+4B21.0	fCNS11643	3-5960
U+4B21.0	fR/S	 183.9; 183.8
U+4B21.0	fTotalStrokes	 18; 17
U+4B21.0	fCangjie	JANOO
U+4B21.0	fFourCorner	44410
U+4B21.0	fHakka	 zhu5; zu5
U+4B21.0	fCantonese	zyu3
U+4B21.0	fMandarin	zhu4
U+4B21.0	fEnglish	interchangeable with &#x7FE5;
U+4B22.0	fUTF8	䬢
U+4B22.0	fCNS11643	3-343A
U+4B22.0	fR/S	 184.2
U+4B22.0	fTotalStrokes	 10; 11
U+4B22.0	fCangjie	OISH
U+4B22.0	fFourCorner	87720
U+4B22.0	fHakka	 hau2; tau1
U+4B22.0	fCantonese	tou1
U+4B22.0	fMandarin	tao1
U+4B22.0	fEnglish	same as &#x9955;
U+4B23.0	fUTF8	䬣
U+4B23.0	fCNS11643	4-362C
U+4B23.0	fGB	3-7A4B
U+4B23.0	fR/S	 184.3
U+4B23.0	fTotalStrokes	 11; 12
U+4B23.0	fCangjie	OION
U+4B23.0	fFourCorner	88710
U+4B23.0	fCantonese	hat1; hei3
U+4B23.0	fMandarin	qi4; xi4
U+4B23.0	fEnglish	same as &#x993C;
U+4B24.0	fUTF8	䬤
U+4B24.0	fCNS11643	3-4029
U+4B24.0	fR/S	 184.3; 184.9; 44.9
U+4B24.0	fTotalStrokes	 12
U+4B24.0	fCangjie	SOIV
U+4B24.0	fFourCorner	77230
U+4B24.0	fCantonese	zaan3; zan3; zin1
U+4B24.0	fMandarin	zan4; chao2; zhan1; zuo4
U+4B24.0	fEnglish	[1] ancient form of &#x9961; [2] same as &#x9958;
U+4B25.0	fUTF8	䬥
U+4B25.0	fCNS11643	3-402A
U+4B25.0	fR/S	 184.3
U+4B25.0	fTotalStrokes	 12
U+4B25.0	fCangjie	IPOIV
U+4B25.0	fFourCorner	43730
U+4B25.0	fCantonese	ji3; zi3; zik1
U+4B25.0	fMandarin	yi4
U+4B25.0	fEnglish	ancient form of &#x9950;
U+4B26.0	fUTF8	䬦
U+4B26.0	fCNS11643	4-3C24
U+4B26.0	fGB	3-7A4F
U+4B26.0	fR/S	 184.4
U+4B26.0	fTotalStrokes	 12; 13
U+4B26.0	fCangjie	OIHNE
U+4B26.0	fFourCorner	87747
U+4B26.0	fCantonese	dau3
U+4B26.0	fMandarin	dou4; she4
U+4B26.0	fEnglish	food items merely arranged for display purpose only; not to ber eaten; to set out fruit, etc., in plates arranged only for show
U+4B27.0	fUTF8	䬧
U+4B27.0	fCNS11643	3-4028
U+4B27.0	fR/S	 184.4
U+4B27.0	fTotalStrokes	 12; 13
U+4B27.0	fCangjie	OIMMU
U+4B27.0	fFourCorner	81710
U+4B27.0	fCantonese	jyun4; zyun4
U+4B27.0	fMandarin	yuan2
U+4B27.0	fEnglish	round shaped cakes
U+4B28.0	fUTF8	䬨
U+4B28.0	fCNS11643	4-3C25
U+4B28.0	fGB	3-7A4E
U+4B28.0	fR/S	 184.4
U+4B28.0	fTotalStrokes	 12; 13
U+4B28.0	fCangjie	OIPKN
U+4B28.0	fFourCorner	87720
U+4B28.0	fCantonese	gau3; gaau3; haau3
U+4B28.0	fMandarin	jiu4; cu4; zu2
U+4B28.0	fEnglish	 eat to the full; surfeited
U+4B29.0	fUTF8	䬩
U+4B29.0	fCNS11643	3-455D
U+4B29.0	fR/S	 184.4
U+4B29.0	fTotalStrokes	 13
U+4B29.0	fCangjie	MFOIV
U+4B29.0	fFourCorner	10730
U+4B29.0	fCantonese	zaai1; zai1; zok6
U+4B29.0	fEnglish	same as &#x9F4B;
U+4B2A.0	fUTF8	䬪
U+4B2A.0	fCNS11643	3-402C
U+4B2A.0	fGB	5-784B
U+4B2A.0	fR/S	 184.4
U+4B2A.0	fTotalStrokes	 12; 13
U+4B2A.0	fCangjie	OIMF
U+4B2A.0	fFourCorner	81790
U+4B2A.0	fCantonese	bok3
U+4B2A.0	fMandarin	bo2; fu4
U+4B2A.0	fEnglish	same as &#x993A;
U+4B2B.0	fUTF8	䬫
U+4B2B.0	fCNS11643	4-4177
U+4B2B.0	fGB	3-7A54
U+4B2B.0	fR/S	 184.5
U+4B2B.0	fTotalStrokes	 13; 14
U+4B2B.0	fCangjie	OIHPI
U+4B2B.0	fFourCorner	82740
U+4B2B.0	fCantonese	tai4
U+4B2B.0	fMandarin	di1; ti2; can3
U+4B2B.0	fEnglish	[1] clarfied butter (regarded as symbol of Buddhist wisdom or truth) [2] cream cheese [3] sponge on others; board
U+4B2C.0	fUTF8	䬬
U+4B2C.0	fCNS11643	4-4174
U+4B2C.0	fGB	3-7A52
U+4B2C.0	fR/S	 184.5
U+4B2C.0	fTotalStrokes	 13; 14
U+4B2C.0	fCangjie	OILBK
U+4B2C.0	fFourCorner	85730
U+4B2C.0	fCantonese	joeng3; zoeng1
U+4B2C.0	fMandarin	yang3; ying4; yang4
U+4B2C.0	fEnglish	[1] eat to the full; surfeited [2] well-stacked (figure, etc.); plump
U+4B2D.0	fUTF8	䬭
U+4B2D.0	fCNS11643	3-4A5C
U+4B2D.0	fR/S	 184.5
U+4B2D.0	fTotalStrokes	 14
U+4B2D.0	fCangjie	RHOIV
U+4B2D.0	fFourCorner	67730
U+4B2D.0	fCantonese	tou1
U+4B2D.0	fMandarin	tao1
U+4B2D.0	fEnglish	non-classical form of &#x9955;
U+4B2E.0	fUTF8	䬮
U+4B2E.0	fCNS11643	3-455E
U+4B2E.0	fR/S	 184.5
U+4B2E.0	fTotalStrokes	 13; 14
U+4B2E.0	fCangjie	OIVIO
U+4B2E.0	fFourCorner	88700
U+4B2E.0	fCantonese	ji4; zi4; zi6
U+4B2E.0	fMandarin	yi2
U+4B2E.0	fEnglish	ancient form of &#x98F4;
U+4B2F.0	fUTF8	䬯
U+4B2F.0	fCNS11643	4-4175
U+4B2F.0	fGB	3-7A50
U+4B2F.0	fR/S	 184.5
U+4B2F.0	fTotalStrokes	 13; 14
U+4B2F.0	fCangjie	OIYR
U+4B2F.0	fFourCorner	81760
U+4B2F.0	fCantonese	nim4; lim4 (also); nim1
U+4B2F.0	fMandarin	nian2; ran2
U+4B2F.0	fEnglish	[1] have congee (wheat gruel) with friends in old times [2] interchageable &#x9902;
U+4B30.0	fUTF8	䬰
U+4B30.0	fCNS11643	3-4556
U+4B30.0	fR/S	 184.5
U+4B30.0	fTotalStrokes	 13; 14
U+4B30.0	fCangjie	OISHR
U+4B30.0	fFourCorner	87760
U+4B30.0	fCantonese	siu6; dou6
U+4B30.0	fMandarin	shao4
U+4B30.0	fEnglish	[1] light refreshment [2] animal feeds; fodder; forage made of wild vegetables [3] rice bran and water from washing rice
U+4B31.0	fUTF8	䬱
U+4B31.0	fCNS11643	3-4555
U+4B31.0	fR/S	 184.5
U+4B31.0	fTotalStrokes	 13; 14
U+4B31.0	fCangjie	OIDM
U+4B31.0	fFourCorner	85730
U+4B31.0	fCantonese	ban6; baan6; gwik1; mang1
U+4B31.0	fMandarin	ben4
U+4B31.0	fEnglish	coarse grains such as corn, millet, etc.; simple food
U+4B32.0	fUTF8	䬲
U+4B32.0	fCNS11643	3-455A
U+4B32.0	fR/S	 184.5
U+4B32.0	fTotalStrokes	 13; 14
U+4B32.0	fCangjie	OIPR
U+4B32.0	fFourCorner	87720
U+4B32.0	fCantonese	kau1; ngaau1
U+4B32.0	fMandarin	gou1
U+4B32.0	fEnglish	[1] eat to the half full [2] corrupted and abbreviated form of &#x9947;
U+4B33.0	fUTF8	䬳
U+4B33.0	fCNS11643	4-4179
U+4B33.0	fGB	3-7A56
U+4B33.0	fR/S	 184.5
U+4B33.0	fTotalStrokes	 13; 14
U+4B33.0	fCangjie	OIFQ
U+4B33.0	fFourCorner	89750
U+4B33.0	fCantonese	bun2; baan2 (also); ban2
U+4B33.0	fMandarin	ban3
U+4B33.0	fEnglish	cakes made of rice flour
U+4B34.0	fUTF8	䬴
U+4B34.0	fCNS11643	4-417A
U+4B34.0	fGB	5-784F
U+4B34.0	fR/S	 184.5
U+4B34.0	fTotalStrokes	 13; 14
U+4B34.0	fCangjie	OIDJ
U+4B34.0	fFourCorner	85790
U+4B34.0	fCantonese	mut3
U+4B34.0	fMandarin	mo4
U+4B34.0	fEnglish	same as &#x79E3;
U+4B35.0	fUTF8	䬵
U+4B35.0	fCNS11643	3-4A58
U+4B35.0	fGB	5-7857
U+4B35.0	fR/S	 184.6
U+4B35.0	fTotalStrokes	 14; 15
U+4B35.0	fCangjie	OIYVO
U+4B35.0	fFourCorner	80780
U+4B35.0	fCantonese	goi1; at3
U+4B35.0	fMandarin	gai1; ai4
U+4B35.0	fEnglish	[1] syrup; jelly-like sugar made from grains [2] hiccough; hiccup [3] same as &#x9932;
U+4B36.0	fUTF8	䬶
U+4B36.0	fCNS11643	3-4A59
U+4B36.0	fR/S	 184.6
U+4B36.0	fTotalStrokes	 14; 15
U+4B36.0	fCangjie	OIAV
U+4B36.0	fFourCorner	87730
U+4B36.0	fCantonese	ngan6; an6 (also); on3
U+4B36.0	fMandarin	en4
U+4B36.0	fEnglish	[1] strave; hungry [2] famished [3] feeble [4] decay of decomposition of fish
U+4B37.0	fUTF8	䬷
U+4B37.0	fCNS11643	4-485B
U+4B37.0	fGB	3-7A5F
U+4B37.0	fR/S	 184.6
U+4B37.0	fTotalStrokes	 14; 15
U+4B37.0	fCangjie	OININ
U+4B37.0	fFourCorner	87720
U+4B37.0	fCantonese	se2
U+4B37.0	fMandarin	she3; na3
U+4B37.0	fEnglish	[1] well-fed [2] glut; eat to one's heart's content
U+4B38.0	fUTF8	䬸
U+4B38.0	fCNS11643	7-2B38
U+4B38.0	fR/S	 184.6
U+4B38.0	fTotalStrokes	 15
U+4B38.0	fCangjie	MEOIV
U+4B38.0	fFourCorner	17730
U+4B38.0	fCantonese	caan1
U+4B38.0	fMandarin	can1
U+4B38.0	fEnglish	corrupted form of &#x9910;
U+4B39.0	fUTF8	䬹
U+4B39.0	fCNS11643	3-4A5A
U+4B39.0	fGB	5-7854
U+4B39.0	fR/S	 184.6
U+4B39.0	fTotalStrokes	 14; 15
U+4B39.0	fCangjie	OIMIG
U+4B39.0	fFourCorner	81710
U+4B39.0	fCantonese	zat1; zaat6
U+4B39.0	fMandarin	zhi4
U+4B39.0	fEnglish	same as &#x928D;
U+4B3A.0	fUTF8	䬺
U+4B3A.0	fCNS11643	4-485C
U+4B3A.0	fGB	3-7A61
U+4B3A.0	fR/S	 184.6
U+4B3A.0	fTotalStrokes	 14; 15
U+4B3A.0	fCangjie	OITQ
U+4B3A.0	fFourCorner	88750
U+4B3A.0	fCantonese	joeng6; seoi1; zoeng6
U+4B3A.0	fMandarin	yang4
U+4B3A.0	fEnglish	same as &#x4B50;
U+4B3B.0	fUTF8	䬻
U+4B3B.0	fCNS11643	6-6440
U+4B3B.0	fR/S	 184.6
U+4B3B.0	fTotalStrokes	 14; 15
U+4B3B.0	fCangjie	OIIQ
U+4B3B.0	fFourCorner	83750
U+4B3B.0	fCantonese	zin3; zaan1
U+4B3B.0	fMandarin	jian4; kan3
U+4B3B.0	fEnglish	non-classical form of &#x991E;
U+4B3C.0	fUTF8	䬼
U+4B3C.0	fCNS11643	3-4F6A
U+4B3C.0	fGB	5-785C
U+4B3C.0	fR/S	 184.7
U+4B3C.0	fTotalStrokes	 15; 16
U+4B3C.0	fCangjie	OIRB
U+4B3C.0	fFourCorner	86720
U+4B3C.0	fCantonese	jyun3; zyun1
U+4B3C.0	fMandarin	yuan4; yuan3; yuan1
U+4B3C.0	fEnglish	satiated; dislike
U+4B3D.0	fUTF8	䬽
U+4B3D.0	fCNS11643	4-4E65
U+4B3D.0	fGB	3-7A67
U+4B3D.0	fR/S	 184.7
U+4B3D.0	fTotalStrokes	 15; 16
U+4B3D.0	fCangjie	OICRU
U+4B3D.0	fFourCorner	88710
U+4B3D.0	fCantonese	seoi3; deoi3
U+4B3D.0	fMandarin	shui4; dui4; rui2; hui1; lei4
U+4B3D.0	fEnglish	[1] wine poured in a libation [2] a little drink or a few drinks [3] cakes; pastry
U+4B3E.0	fUTF8	䬾
U+4B3E.0	fCNS11643	4-4E64
U+4B3E.0	fGB	3-7A68
U+4B3E.0	fR/S	 184.7
U+4B3E.0	fTotalStrokes	 15; 16
U+4B3E.0	fCangjie	OICNH
U+4B3E.0	fFourCorner	88720
U+4B3E.0	fCantonese	tai4; daai3; taai4
U+4B3E.0	fMandarin	ti2
U+4B3E.0	fEnglish	cakes and biscuits
U+4B3F.0	fUTF8	䬿
U+4B3F.0	fCNS11643	4-4E67
U+4B3F.0	fGB	3-7A69
U+4B3F.0	fR/S	 184.7
U+4B3F.0	fTotalStrokes	 15; 16
U+4B3F.0	fCangjie	OISHU
U+4B3F.0	fFourCorner	87710
U+4B3F.0	fCantonese	mei5; mei6
U+4B3F.0	fMandarin	wei3; wei4; mei4; mo4
U+4B3F.0	fEnglish	[1] small; little [2] congee; rice gruel
U+4B40.0	fUTF8	䭀
U+4B40.0	fCNS11643	4-4E66
U+4B40.0	fGB	5-7858
U+4B40.0	fR/S	 184.7
U+4B40.0	fTotalStrokes	 15; 16
U+4B40.0	fCangjie	OIYNJ
U+4B40.0	fFourCorner	87730
U+4B40.0	fCantonese	seon3
U+4B40.0	fMandarin	xun4
U+4B40.0	fEnglish	food (of the Taoist)
U+4B41.0	fUTF8	䭁
U+4B41.0	fCNS11643	4-5478
U+4B41.0	fGB	3-7A46
U+4B41.0	fR/S	 184.7
U+4B41.0	fTotalStrokes	 16
U+4B41.0	fCangjie	QLOIV
U+4B41.0	fFourCorner	52730
U+4B41.0	fHakka	 zhi5; zih5
U+4B41.0	fCantonese	zai3
U+4B41.0	fMandarin	zhe2; zhi4
U+4B41.0	fEnglish	very bad smell
U+4B42.0	fUTF8	䭂
U+4B42.0	fCNS11643	4-4E69
U+4B42.0	fGB	3-7A66
U+4B42.0	fR/S	 184.7
U+4B42.0	fTotalStrokes	 15; 16
U+4B42.0	fCangjie	OIRAU
U+4B42.0	fFourCorner	86710
U+4B42.0	fCantonese	jap1; zaap1
U+4B42.0	fMandarin	yi4; yi3; ye4
U+4B42.0	fEnglish	[1] damp; humid [2] eat to the full
U+4B43.0	fUTF8	䭃
U+4B43.0	fCNS11643	4-547C
U+4B43.0	fGB	3-7A72
U+4B43.0	fR/S	 184.8
U+4B43.0	fTotalStrokes	 16; 17
U+4B43.0	fCangjie	OIOIP
U+4B43.0	fFourCorner	88732
U+4B43.0	fCantonese	jam5; zaam5
U+4B43.0	fMandarin	nie1; ren3
U+4B43.0	fEnglish	[1] eat to the fool [2] same as &#x98EA;
U+4B44.0	fUTF8	䭄
U+4B44.0	fCNS11643	4-547D
U+4B44.0	fGB	3-7A6B
U+4B44.0	fR/S	 184.8
U+4B44.0	fTotalStrokes	 16; 17
U+4B44.0	fCangjie	OIJLN
U+4B44.0	fFourCorner	85700
U+4B44.0	fCantonese	si6
U+4B44.0	fMandarin	shi4
U+4B44.0	fEnglish	gluttonous; to eat piggishly
U+4B45.0	fUTF8	䭅
U+4B45.0	fCNS11643	4-547A
U+4B45.0	fGB	3-7A6F
U+4B45.0	fR/S	 184.8
U+4B45.0	fTotalStrokes	 16; 17
U+4B45.0	fCangjie	OIWJR
U+4B45.0	fFourCorner	86700
U+4B45.0	fCantonese	wu4
U+4B45.0	fMandarin	gu4; hu2
U+4B45.0	fEnglish	[1] cakes [2] same as &#x992C;
U+4B46.0	fUTF8	䭆
U+4B46.0	fCNS11643	4-5A27
U+4B46.0	fGB	3-7A47
U+4B46.0	fR/S	 184.8
U+4B46.0	fTotalStrokes	 16; 17
U+4B46.0	fCangjie	SYOIV
U+4B46.0	fFourCorner	78730
U+4B46.0	fCantonese	nak6; lak6 (also); neot6
U+4B46.0	fMandarin	ne4
U+4B46.0	fEnglish	[1] small children eat less [2] tired of eating [3] baked cakes
U+4B47.0	fUTF8	䭇
U+4B47.0	fCNS11643	4-5522
U+4B47.0	fGB	5-785F
U+4B47.0	fR/S	 184.8
U+4B47.0	fTotalStrokes	 16; 17
U+4B47.0	fCangjie	OIPGR
U+4B47.0	fFourCorner	87720
U+4B47.0	fCantonese	jit3; zit3
U+4B47.0	fMandarin	ye4; yi4
U+4B47.0	fEnglish	[1] same as &#x564E; [2] ancient form of &#x9950;
U+4B48.0	fUTF8	䭈
U+4B48.0	fCNS11643	4-5A28
U+4B48.0	fGB	3-7A74
U+4B48.0	fR/S	 184.9
U+4B48.0	fTotalStrokes	 17; 16; 18
U+4B48.0	fCangjie	OINKQ
U+4B48.0	fFourCorner	85740
U+4B48.0	fCantonese	gin1; zin1
U+4B48.0	fMandarin	jian1; jian4; zhan1
U+4B48.0	fEnglish	[1] same as &#x9930; [2] same as &#x9958;
U+4B49.0	fUTF8	䭉
U+4B49.0	fCNS11643	4-5A2A
U+4B49.0	fGB	3-7A75
U+4B49.0	fR/S	 184.9
U+4B49.0	fTotalStrokes	 17; 16; 18
U+4B49.0	fCangjie	OIKMB
U+4B49.0	fFourCorner	84720
U+4B49.0	fHakka	 sui3; cui3
U+4B49.0	fCantonese	seoi2; seoi5; saau1
U+4B49.0	fMandarin	sui3; wei3
U+4B49.0	fEnglish	cakes; biscuits
U+4B4A.0	fUTF8	䭊
U+4B4A.0	fCNS11643	5-6031
U+4B4A.0	fR/S	 184.9
U+4B4A.0	fTotalStrokes	 17; 18
U+4B4A.0	fCangjie	OITLK
U+4B4A.0	fFourCorner	84730
U+4B4A.0	fCantonese	jing2; zing2
U+4B4A.0	fMandarin	ying3; yang3
U+4B4A.0	fEnglish	same as &#x4B58;
U+4B4B.0	fUTF8	䭋
U+4B4B.0	fCNS11643	3-572D
U+4B4B.0	fR/S	 184.9
U+4B4B.0	fTotalStrokes	 17; 18
U+4B4B.0	fCangjie	OIORD
U+4B4B.0	fFourCorner	86790
U+4B4B.0	fCantonese	baau2
U+4B4B.0	fMandarin	bao3
U+4B4B.0	fEnglish	same as &#x98FD;
U+4B4C.0	fUTF8	䭌
U+4B4C.0	fCNS11643	3-5964
U+4B4C.0	fR/S	 184.9
U+4B4C.0	fTotalStrokes	 18
U+4B4C.0	fCangjie	JBOIV
U+4B4C.0	fFourCorner	47732
U+4B4C.0	fCantonese	wu4
U+4B4C.0	fMandarin	tie4; hu2
U+4B4C.0	fEnglish	same as &#x992C;, &#x7CCA;
U+4B4D.0	fUTF8	䭍
U+4B4D.0	fCNS11643	5-602C
U+4B4D.0	fR/S	 184.9
U+4B4D.0	fTotalStrokes	 17; 18
U+4B4D.0	fCangjie	OIQMT
U+4B4D.0	fFourCorner	85710
U+4B4D.0	fCantonese	wu4; fu4
U+4B4D.0	fMandarin	hu2
U+4B4D.0	fEnglish	non-classical form of &#x992C;
U+4B4E.0	fUTF8	䭎
U+4B4E.0	fCNS11643	3-572C
U+4B4E.0	fR/S	 184.9
U+4B4E.0	fTotalStrokes	 17; 18
U+4B4E.0	fCangjie	OIPTD
U+4B4E.0	fFourCorner	84790
U+4B4E.0	fCantonese	jip6; zip6
U+4B4E.0	fMandarin	xie2; ye4
U+4B4E.0	fEnglish	cakes
U+4B4F.0	fUTF8	䭏
U+4B4F.0	fR/S	 184.9
U+4B4F.0	fTotalStrokes	 18
U+4B4F.0	fCangjie	OIHSB
U+4B4F.0	fCantonese	bin2
U+4B4F.0	fEnglish	[Korean character]
U+4B50.0	fUTF8	䭐
U+4B50.0	fCNS11643	4-5E58
U+4B50.0	fGB	5-786E
U+4B50.0	fR/S	 184.10
U+4B50.0	fTotalStrokes	 18; 19
U+4B50.0	fCangjie	OITGP
U+4B50.0	fFourCorner	88730
U+4B50.0	fCantonese	joeng6; seoi1; zoeng6
U+4B50.0	fMandarin	yang4
U+4B50.0	fEnglish	[1] cakes; pastry; biscuits [2] a bait; food
U+4B51.0	fUTF8	䭑
U+4B51.0	fCNS11643	4-5E57
U+4B51.0	fGB	3-7B21
U+4B51.0	fR/S	 184.10
U+4B51.0	fTotalStrokes	 18; 19
U+4B51.0	fCangjie	OITXC
U+4B51.0	fFourCorner	88730
U+4B51.0	fHakka	 liam3
U+4B51.0	fCantonese	lim5; lim4
U+4B51.0	fMandarin	lian2; lian3; xian4; qian4
U+4B51.0	fEnglish	[1] eat insufficiently [2] unsatisfied
U+4B52.0	fUTF8	䭒
U+4B52.0	fCNS11643	4-5E5B
U+4B52.0	fGB	3-7A7E
U+4B52.0	fR/S	 184.10
U+4B52.0	fTotalStrokes	 18; 19
U+4B52.0	fCangjie	OIHUP
U+4B52.0	fFourCorner	86730
U+4B52.0	fHakka	 sit7
U+4B52.0	fCantonese	sik1
U+4B52.0	fMandarin	xi2
U+4B52.0	fEnglish	[1] draw breath [2] eat
U+4B53.0	fUTF8	䭓
U+4B53.0	fCNS11643	4-5E59
U+4B53.0	fGB	3-7A5B
U+4B53.0	fR/S	 184.10
U+4B53.0	fTotalStrokes	 18; 19
U+4B53.0	fCangjie	OIUMT
U+4B53.0	fFourCorner	82710
U+4B53.0	fCantonese	ngan6; an6 (also); neot6
U+4B53.0	fMandarin	wen4; en4
U+4B53.0	fEnglish	[1] food (wheat gruel, etc.) to welcome the guest in old times [2] eat to the full; surfeited
U+4B54.0	fUTF8	䭔
U+4B54.0	fCNS11643	4-5E5A
U+4B54.0	fGB	3-7A7A
U+4B54.0	fR/S	 184.10
U+4B54.0	fTotalStrokes	 18; 19
U+4B54.0	fCangjie	OIYHR
U+4B54.0	fFourCorner	87737
U+4B54.0	fHakka	  doi1;  dui1
U+4B54.0	fCantonese	deoi1
U+4B54.0	fMandarin	dui1
U+4B54.0	fEnglish	steamed dumplings
U+4B55.0	fUTF8	䭕
U+4B55.0	fCNS11643	4-6539
U+4B55.0	fGB	3-7A48
U+4B55.0	fR/S	 184.11
U+4B55.0	fTotalStrokes	 20
U+4B55.0	fCangjie	JLOIV
U+4B55.0	fFourCorner	52730
U+4B55.0	fCantonese	zaam2; cim3; zan2; zit6
U+4B55.0	fMandarin	zan3; jian4; zan4; jian3
U+4B55.0	fEnglish	[1] tasteless; without enough salt; insipid; dull; not interesting [2] taste food before royalty partook
U+4B56.0	fUTF8	䭖
U+4B56.0	fCNS11643	4-6236
U+4B56.0	fGB	3-7B22
U+4B56.0	fR/S	 184.11
U+4B56.0	fTotalStrokes	 19; 20
U+4B56.0	fCangjie	OIITF
U+4B56.0	fFourCorner	80730
U+4B56.0	fCantonese	zyu3
U+4B56.0	fMandarin	zhu4
U+4B56.0	fEnglish	animal feeds; fodder; forage
U+4B57.0	fUTF8	䭗
U+4B57.0	fCNS11643	3-5B68
U+4B57.0	fR/S	 184.11
U+4B57.0	fTotalStrokes	 19; 20
U+4B57.0	fCangjie	OIYTU
U+4B57.0	fFourCorner	80710
U+4B57.0	fCantonese	jing2; zing2
U+4B57.0	fMandarin	ying3
U+4B57.0	fEnglish	same as &#x4B58;
U+4B58.0	fUTF8	䭘
U+4B58.0	fCNS11643	4-653C
U+4B58.0	fGB	3-7B26
U+4B58.0	fR/S	 184.12
U+4B58.0	fTotalStrokes	 20; 21
U+4B58.0	fCangjie	OIAYF
U+4B58.0	fFourCorner	86790
U+4B58.0	fCantonese	jing2; zing2
U+4B58.0	fMandarin	ying3; yan4; ying4
U+4B58.0	fEnglish	[1] well-stacked (figure, etc.); full; plump [2] food [3] cakes
U+4B59.0	fUTF8	䭙
U+4B59.0	fCNS11643	4-653B
U+4B59.0	fGB	3-7B25
U+4B59.0	fR/S	 184.12
U+4B59.0	fTotalStrokes	 20; 21
U+4B59.0	fCangjie	OIMUA
U+4B59.0	fFourCorner	81760
U+4B59.0	fCantonese	zam2; gaan2
U+4B59.0	fMandarin	jian3; jin3
U+4B59.0	fEnglish	[1] exquisite; fine [2] sweet; taste a little sweet [3] taste [4] diseases (of the lips)
U+4B5A.0	fUTF8	䭚
U+4B5A.0	fCNS11643	3-5D57
U+4B5A.0	fGB	5-7871
U+4B5A.0	fR/S	 184.12
U+4B5A.0	fTotalStrokes	 20; 21
U+4B5A.0	fCangjie	OIYTG
U+4B5A.0	fFourCorner	80710
U+4B5A.0	fHakka	 cong2; tung2
U+4B5A.0	fCantonese	cong4
U+4B5A.0	fMandarin	chuang2; ne4
U+4B5A.0	fEnglish	 eat immoderately; gorge
U+4B5B.0	fUTF8	䭛
U+4B5B.0	fCNS11643	3-5D59
U+4B5B.0	fR/S	 184.12
U+4B5B.0	fTotalStrokes	 20; 21
U+4B5B.0	fCangjie	OIMJK
U+4B5B.0	fFourCorner	88740
U+4B5B.0	fCantonese	daam6; dam6; wui6
U+4B5B.0	fMandarin	gan3; dan4
U+4B5B.0	fEnglish	[1] same as &#x994F; [2] non-classical of standard form &#x6F89;
U+4B5C.0	fUTF8	䭜
U+4B5C.0	fR/S	 184.12
U+4B5C.0	fTotalStrokes	 21
U+4B5C.0	fCangjie	OIKCF
U+4B5C.0	fCantonese	liu4
U+4B5C.0	fEnglish	[Korean character]
U+4B5D.0	fUTF8	䭝
U+4B5D.0	fCNS11643	4-6824
U+4B5D.0	fGB	5-7856
U+4B5D.0	fR/S	 184.13
U+4B5D.0	fTotalStrokes	 21; 22
U+4B5D.0	fCangjie	OIOMA
U+4B5D.0	fFourCorner	88760
U+4B5D.0	fCantonese	kui2; kwui2 (also)
U+4B5D.0	fMandarin	guai4; kuai4
U+4B5D.0	fEnglish	[1] eat [2] non-classical form of &#x81BE;
U+4B5E.0	fUTF8	䭞
U+4B5E.0	fCNS11643	4-6823
U+4B5E.0	fGB	5-7853
U+4B5E.0	fR/S	 184.13
U+4B5E.0	fTotalStrokes	 21; 22
U+4B5E.0	fCangjie	OIWLJ
U+4B5E.0	fFourCorner	86740
U+4B5E.0	fCantonese	jik6; zik6
U+4B5E.0	fMandarin	yi4
U+4B5E.0	fEnglish	same as &#x7E79;
U+4B5F.0	fUTF8	䭟
U+4B5F.0	fCNS11643	3-5E7D
U+4B5F.0	fR/S	 184.13
U+4B5F.0	fTotalStrokes	 21; 22
U+4B5F.0	fCangjie	OITPD
U+4B5F.0	fFourCorner	84790
U+4B5F.0	fCantonese	jip6; zip6
U+4B5F.0	fMandarin	ye4
U+4B5F.0	fEnglish	same as &#x4B4E;
U+4B60.0	fUTF8	䭠
U+4B60.0	fCNS11643	4-6822
U+4B60.0	fGB	5-7874
U+4B60.0	fR/S	 184.13
U+4B60.0	fTotalStrokes	 21; 22
U+4B60.0	fCangjie	OIITC
U+4B60.0	fFourCorner	80730
U+4B60.0	fCantonese	gim2; haam2; him3
U+4B60.0	fMandarin	qian3; jian3; qian4
U+4B60.0	fEnglish	[1] snack [2] supplementary dishes [3] corrupted form of &#x4B51 [4] same as &#x6B49; [5] same as &#x9921;
U+4B61.0	fUTF8	䭡
U+4B61.0	fCNS11643	4-6967
U+4B61.0	fGB	3-7B2A
U+4B61.0	fR/S	 184.14
U+4B61.0	fTotalStrokes	 22; 21; 23
U+4B61.0	fCangjie	OIBMP
U+4B61.0	fFourCorner	82730
U+4B61.0	fCantonese	an3; ngan3 (also); neoi6; neot6
U+4B61.0	fMandarin	wen4; yuan4; en4
U+4B61.0	fEnglish	[1] food (wheat gruel, etc.) to welcome the guest in old times [2] eat to the full; surfeited
U+4B62.0	fUTF8	䭢
U+4B62.0	fCNS11643	4-6965
U+4B62.0	fGB	3-7A57
U+4B62.0	fR/S	 184.14
U+4B62.0	fTotalStrokes	 22; 21; 23
U+4B62.0	fCangjie	OIJPN
U+4B62.0	fFourCorner	83720
U+4B62.0	fHakka	 nen2
U+4B62.0	fCantonese	ning4; ling4 (also)
U+4B62.0	fMandarin	ning2
U+4B62.0	fEnglish	 eat to repletation
U+4B63.0	fUTF8	䭣
U+4B63.0	fCNS11643	3-5F77
U+4B63.0	fR/S	 184.14
U+4B63.0	fTotalStrokes	 22; 23
U+4B63.0	fCangjie	OIYX
U+4B63.0	fFourCorner	80720
U+4B63.0	fCantonese	ci4
U+4B63.0	fMandarin	ci2
U+4B63.0	fEnglish	same as &#x9908;, &#x7CA2;, &#x8E84;
U+4B64.0	fUTF8	䭤
U+4B64.0	fCNS11643	4-6968
U+4B64.0	fGB	3-7B28
U+4B64.0	fR/S	 184.14
U+4B64.0	fTotalStrokes	 22; 21; 23
U+4B64.0	fCangjie	OIYLR
U+4B64.0	fFourCorner	85730
U+4B64.0	fCantonese	hin3; hin2
U+4B64.0	fMandarin	qian3
U+4B64.0	fEnglish	[1] chew; eat [2] roll round with the hand [3] cakes; biscuits
U+4B65.0	fUTF8	䭥
U+4B65.0	fCNS11643	4-6B31
U+4B65.0	fR/S	 184.15
U+4B65.0	fTotalStrokes	 23; 24
U+4B65.0	fCangjie	OITOV
U+4B65.0	fFourCorner	88730
U+4B65.0	fCantonese	joeng6; zoeng6
U+4B65.0	fMandarin	xue4; yang4; yao4; zhou4
U+4B65.0	fEnglish	[1] (non-classical form) bait [2] food [3] cakes and biscuits
U+4B66.0	fUTF8	䭦
U+4B66.0	fCNS11643	3-6156
U+4B66.0	fR/S	 184.17
U+4B66.0	fTotalStrokes	 25; 26
U+4B66.0	fCangjie	OITEI
U+4B66.0	fFourCorner	84740
U+4B66.0	fCantonese	bok6; bok3
U+4B66.0	fMandarin	bo2
U+4B66.0	fEnglish	[1] cakes; biscuits; pastry [2] same as &#x993A;
U+4B67.0	fUTF8	䭧
U+4B67.0	fCNS11643	4-6D30
U+4B67.0	fGB	3-7B2C
U+4B67.0	fR/S	 184.17
U+4B67.0	fTotalStrokes	 25; 26
U+4B67.0	fCangjie	OIIDF
U+4B67.0	fFourCorner	80790
U+4B67.0	fHakka	 mi3; mi2
U+4B67.0	fCantonese	mei5
U+4B67.0	fMandarin	mi3
U+4B67.0	fEnglish	 feed a child with gruel
U+4B68.0	fUTF8	䭨
U+4B68.0	fCNS11643	4-6D67
U+4B68.0	fGB	3-7B2D
U+4B68.0	fR/S	 184.18
U+4B68.0	fTotalStrokes	 26; 27
U+4B68.0	fCangjie	OIUOB
U+4B68.0	fFourCorner	82720
U+4B68.0	fCantonese	seoi3
U+4B68.0	fMandarin	sui1; shui4; sui4; rui4; hui1; xi1
U+4B68.0	fEnglish	same as &#x4B3D;
U+4B69.0	fUTF8	䭩
U+4B69.0	fCNS11643	4-6E25
U+4B69.0	fGB	3-7B2E
U+4B69.0	fR/S	 184.19
U+4B69.0	fTotalStrokes	 27; 28
U+4B69.0	fCangjie	OIIDY
U+4B69.0	fFourCorner	80710
U+4B69.0	fCantonese	mo4; mo1
U+4B69.0	fMandarin	mo2; mi4
U+4B69.0	fEnglish	[1] feed a baby [2] eat [3] food
U+4B6A.0	fUTF8	䭪
U+4B6A.0	fGB	S-2149
U+4B6A.0	fR/S	 184.12
U+4B6A.0	fTotalStrokes	 21
U+4B6A.0	fCangjie	NVAMG
U+4B6A.0	fCantonese	loeng4
U+4B6A.0	fMandarin	liang2
U+4B6A.0	fEnglish	simplified form of &#x7CE7;
U+4B6B.0	fUTF8	䭫
U+4B6B.0	fCNS11643	3-4F6C
U+4B6B.0	fGB	5-7924
U+4B6B.0	fR/S	 185.6
U+4B6B.0	fTotalStrokes	 15
U+4B6B.0	fCangjie	PATHU
U+4B6B.0	fFourCorner	28660
U+4B6B.0	fCantonese	kai2
U+4B6B.0	fMandarin	ji1; qi3
U+4B6B.0	fEnglish	interchangeable with &#x7A3D;
U+4B6C.0	fUTF8	䭬
U+4B6C.0	fCNS11643	3-5367
U+4B6C.0	fR/S	 185.6; 185.13; 47.13
U+4B6C.0	fTotalStrokes	 16; 15
U+4B6C.0	fCangjie	PAVVU
U+4B6C.0	fFourCorner	22661
U+4B6C.0	fCantonese	kai2
U+4B6C.0	fMandarin	qi3
U+4B6C.0	fEnglish	[1] same as &#x4B6B; [2] interchangeable with &#x7A3D;
U+4B6D.0	fUTF8	䭭
U+4B6D.0	fCNS11643	4-5E5F
U+4B6D.0	fGB	3-7B33
U+4B6D.0	fR/S	 185.9
U+4B6D.0	fTotalStrokes	 18
U+4B6D.0	fCangjie	TUMBC
U+4B6D.0	fFourCorner	81680
U+4B6D.0	fCantonese	sau2; saau2; sou2
U+4B6D.0	fMandarin	shou3; shu2; shu4; tu4
U+4B6D.0	fEnglish	[1] the eldest son [2] give birth to the first child
U+4B6E.0	fUTF8	䭮
U+4B6E.0	fCNS11643	4-623C
U+4B6E.0	fGB	3-7B58
U+4B6E.0	fR/S	 185.10
U+4B6E.0	fTotalStrokes	 19
U+4B6E.0	fCangjie	SHTHU
U+4B6E.0	fFourCorner	72600
U+4B6E.0	fCantonese	fat1; bau1; faat1; heoi6; o1; teoi4
U+4B6E.0	fMandarin	fu2; bi4
U+4B6E.0	fEnglish	jewelry; ornaments; trinkets (for the forehead of a woman)
U+4B6F.0	fUTF8	䭯
U+4B6F.0	fCNS11643	4-485D
U+4B6F.0	fR/S	 186.5
U+4B6F.0	fTotalStrokes	 14
U+4B6F.0	fCangjie	HAUU
U+4B6F.0	fFourCorner	22670
U+4B6F.0	fCantonese	but6
U+4B6F.0	fMandarin	bo2
U+4B6F.0	fEnglish	same as &#x999E;
U+4B70.0	fUTF8	䭰
U+4B70.0	fCNS11643	3-572F
U+4B70.0	fR/S	 186.8
U+4B70.0	fTotalStrokes	 17
U+4B70.0	fCangjie	HAQKQ
U+4B70.0	fFourCorner	25650
U+4B70.0	fHakka	 fung3
U+4B70.0	fCantonese	bung6; bung2; faai3; zaai6
U+4B70.0	fMandarin	beng3; beng4
U+4B70.0	fEnglish	strong sweet smell; strong fragrance
U+4B71.0	fUTF8	䭱
U+4B71.0	fCNS11643	4-5526
U+4B71.0	fGB	3-7A2D
U+4B71.0	fR/S	 186.8
U+4B71.0	fTotalStrokes	 17; 16
U+4B71.0	fCangjie	HAFBF
U+4B71.0	fFourCorner	29620
U+4B71.0	fCantonese	bit6; miu5; mok1
U+4B71.0	fMandarin	bie2; pie1
U+4B71.0	fEnglish	a little; slightly sweet-smelling
U+4B72.0	fUTF8	䭲
U+4B72.0	fCNS11643	3-572E
U+4B72.0	fR/S	 186.8
U+4B72.0	fTotalStrokes	 17
U+4B72.0	fCangjie	HAKMR
U+4B72.0	fFourCorner	24620
U+4B72.0	fHakka	 ji3; ri3
U+4B72.0	fCantonese	ji2; zi2
U+4B72.0	fMandarin	yi3; ni3
U+4B72.0	fEnglish	fragrant
U+4B73.0	fUTF8	䭳
U+4B73.0	fCNS11643	3-6171
U+4B73.0	fR/S	 186.18
U+4B73.0	fTotalStrokes	 27; 26
U+4B73.0	fCangjie	HIHDA
U+4B73.0	fFourCorner	26600
U+4B73.0	fHakka	 ngui5; ngui6
U+4B73.0	fCantonese	ngai6; ai6 (also)
U+4B73.0	fMandarin	wei4
U+4B73.0	fEnglish	Asafoetida
U+4B74.0	fUTF8	䭴
U+4B74.0	fCNS11643	3-402D
U+4B74.0	fGB	5-4129
U+4B74.0	fR/S	 187.2
U+4B74.0	fTotalStrokes	 12
U+4B74.0	fCangjie	SFJ
U+4B74.0	fFourCorner	71400
U+4B74.0	fHakka	 fan2
U+4B74.0	fCantonese	waan4; man5; wan4
U+4B74.0	fMandarin	huan2; yuan4
U+4B74.0	fEnglish	a year-old colt
U+4B75.0	fUTF8	䭵
U+4B75.0	fCNS11643	5-444A
U+4B75.0	fR/S	 187.3
U+4B75.0	fTotalStrokes	 13
U+4B75.0	fCangjie	SFHNI
U+4B75.0	fFourCorner	77310
U+4B75.0	fCantonese	faan4
U+4B75.0	fMandarin	fan2
U+4B75.0	fEnglish	same as standard form &#x98BF;
U+4B76.0	fUTF8	䭶
U+4B76.0	fCNS11643	3-4560
U+4B76.0	fR/S	 187.3
U+4B76.0	fTotalStrokes	 13
U+4B76.0	fCangjie	SFML
U+4B76.0	fFourCorner	71320
U+4B76.0	fCantonese	kei4
U+4B76.0	fMandarin	qi2
U+4B76.0	fEnglish	ancient form of &#x9A0F;
U+4B77.0	fUTF8	䭷
U+4B77.0	fCNS11643	4-485E
U+4B77.0	fGB	3-3D78
U+4B77.0	fR/S	 187.4
U+4B77.0	fTotalStrokes	 14
U+4B77.0	fCangjie	SFHQU
U+4B77.0	fFourCorner	72310
U+4B77.0	fHakka	  mau2
U+4B77.0	fCantonese	mou4; wu4
U+4B77.0	fMandarin	mao2; liu2; lie4
U+4B77.0	fEnglish	a screen fixed at the side of an ancient chariot
U+4B78.0	fUTF8	䭸
U+4B78.0	fCNS11643	3-4A65
U+4B78.0	fR/S	 187.4
U+4B78.0	fTotalStrokes	 14
U+4B78.0	fCangjie	SFCK
U+4B78.0	fFourCorner	78340
U+4B78.0	fCantonese	fu6
U+4B78.0	fMandarin	fu4
U+4B78.0	fEnglish	 stallion; male horse
U+4B79.0	fUTF8	䭹
U+4B79.0	fCNS11643	4-485F
U+4B79.0	fGB	3-3D7B
U+4B79.0	fR/S	 187.4
U+4B79.0	fTotalStrokes	 14
U+4B79.0	fCangjie	SFHVL
U+4B79.0	fFourCorner	77320
U+4B79.0	fHakka	 ngong1
U+4B79.0	fCantonese	ngong4; ong4 (also)
U+4B79.0	fMandarin	ang2; ang4; ang3
U+4B79.0	fEnglish	[1] an angry or startled horse [2] a horse with enormous speed and staying power [3] a horse with white abdominal region
U+4B7A.0	fUTF8	䭺
U+4B7A.0	fCNS11643	3-4A5F
U+4B7A.0	fR/S	 187.4
U+4B7A.0	fTotalStrokes	 14
U+4B7A.0	fCangjie	SFYHN
U+4B7A.0	fFourCorner	70310
U+4B7A.0	fCantonese	ngong5; ong5 (also); ngong4
U+4B7A.0	fMandarin	ang3; ang2
U+4B7A.0	fEnglish	[1] a horse to raise its head high [2] same as &#x44B79;
U+4B7B.0	fUTF8	䭻
U+4B7B.0	fCNS11643	3-4A67
U+4B7B.0	fR/S	 187.4
U+4B7B.0	fTotalStrokes	 14
U+4B7B.0	fCangjie	SFCSH
U+4B7B.0	fFourCorner	78320
U+4B7B.0	fCantonese	fan1
U+4B7B.0	fMandarin	fen1
U+4B7B.0	fEnglish	 swifty horse
U+4B7C.0	fUTF8	䭼
U+4B7C.0	fCNS11643	3-4A68
U+4B7C.0	fR/S	 187.4
U+4B7C.0	fTotalStrokes	 14
U+4B7C.0	fCangjie	SFMML
U+4B7C.0	fFourCorner	71320
U+4B7C.0	fCantonese	kei4
U+4B7C.0	fMandarin	qi2
U+4B7C.0	fEnglish	ancient form of &#x9A0F;
U+4B7D.0	fUTF8	䭽
U+4B7D.0	fCNS11643	3-4A6B
U+4B7D.0	fR/S	 187.4
U+4B7D.0	fTotalStrokes	 14
U+4B7D.0	fCangjie	SFHQ
U+4B7D.0	fFourCorner	75300
U+4B7D.0	fCantonese	kwan4; gwok3; kwaan4
U+4B7D.0	fMandarin	qun2
U+4B7D.0	fEnglish	same as &#x7FA3;, &#x7FA4;
U+4B7E.0	fUTF8	䭾
U+4B7E.0	fCNS11643	3-4A69
U+4B7E.0	fR/S	 187.4
U+4B7E.0	fTotalStrokes	 14
U+4B7E.0	fCangjie	SFIK
U+4B7E.0	fFourCorner	73330
U+4B7E.0	fHakka	 to2
U+4B7E.0	fCantonese	to4; do6
U+4B7E.0	fMandarin	tuo2
U+4B7E.0	fEnglish	same as &#x99B1;
U+4B7F.0	fUTF8	䭿
U+4B7F.0	fCNS11643	4-4E6F
U+4B7F.0	fGB	5-412D
U+4B7F.0	fR/S	 187.5
U+4B7F.0	fTotalStrokes	 15
U+4B7F.0	fCangjie	SFHQO
U+4B7F.0	fFourCorner	75330
U+4B7F.0	fCantonese	jat6; zaat6
U+4B7F.0	fMandarin	yi4
U+4B7F.0	fEnglish	[1] swift horse [2] summon; ask; solicit; request; seek for
U+4B80.0	fUTF8	䮀
U+4B80.0	fCNS11643	3-4F71
U+4B80.0	fR/S	 187.5
U+4B80.0	fTotalStrokes	 15
U+4B80.0	fCangjie	SFPRU
U+4B80.0	fFourCorner	77310
U+4B80.0	fCantonese	but6
U+4B80.0	fMandarin	bo2
U+4B80.0	fEnglish	a kind of animal, with a horse shape, the tail of an ox and with only one horn
U+4B81.0	fUTF8	䮁
U+4B81.0	fCNS11643	3-4F70
U+4B81.0	fR/S	 187.5
U+4B81.0	fTotalStrokes	 15
U+4B81.0	fCangjie	SFIT
U+4B81.0	fFourCorner	73340
U+4B81.0	fCantonese	pin4
U+4B81.0	fMandarin	pian2
U+4B81.0	fEnglish	same as &#x99E2;
U+4B82.0	fUTF8	䮂
U+4B82.0	fCNS11643	4-4E6E
U+4B82.0	fGB	3-3E22
U+4B82.0	fR/S	 187.5
U+4B82.0	fTotalStrokes	 15
U+4B82.0	fCangjie	SFIKK
U+4B82.0	fFourCorner	73340
U+4B82.0	fCantonese	but6; bat6 (also)
U+4B82.0	fMandarin	ba2; bo2
U+4B82.0	fEnglish	a horse with long hair
U+4B83.0	fUTF8	䮃
U+4B83.0	fCNS11643	3-4F6F
U+4B83.0	fR/S	 187.5
U+4B83.0	fTotalStrokes	 15
U+4B83.0	fCangjie	SFGI
U+4B83.0	fFourCorner	74330
U+4B83.0	fHakka	 ki1
U+4B83.0	fCantonese	keoi1
U+4B83.0	fMandarin	qu1
U+4B83.0	fEnglish	same as &#x9A45;
U+4B84.0	fUTF8	䮄
U+4B84.0	fCNS11643	4-4E6D
U+4B84.0	fGB	3-3E2A
U+4B84.0	fR/S	 187.5
U+4B84.0	fTotalStrokes	 15
U+4B84.0	fCangjie	SFYVI
U+4B84.0	fFourCorner	70330
U+4B84.0	fCantonese	jyun4; zyun1; zyun4
U+4B84.0	fMandarin	xuan2
U+4B84.0	fEnglish	[1] an one-year-old horse [2] black horse
U+4B85.0	fUTF8	䮅
U+4B85.0	fCNS11643	4-4E70
U+4B85.0	fGB	3-3E24
U+4B85.0	fR/S	 187.5
U+4B85.0	fTotalStrokes	 15
U+4B85.0	fCangjie	SFIV
U+4B85.0	fFourCorner	73350
U+4B85.0	fCantonese	jyut6
U+4B86.0	fUTF8	䮆
U+4B86.0	fCNS11643	3-536F
U+4B86.0	fR/S	 187.6
U+4B86.0	fTotalStrokes	 16
U+4B86.0	fCangjie	SFMFJ
U+4B86.0	fFourCorner	71340
U+4B86.0	fCantonese	pei1; baai6; pei3
U+4B86.0	fMandarin	pi2
U+4B86.0	fEnglish	same as &#x99D3;
U+4B87.0	fUTF8	䮇
U+4B87.0	fCNS11643	3-536E
U+4B87.0	fR/S	 187.6
U+4B87.0	fTotalStrokes	 16
U+4B87.0	fCangjie	SFLQ
U+4B87.0	fFourCorner	75300
U+4B87.0	fCantonese	wat6
U+4B87.0	fMandarin	yu4
U+4B87.0	fEnglish	same as &#x9A48;
U+4B88.0	fUTF8	䮈
U+4B88.0	fCNS11643	3-536D
U+4B88.0	fR/S	 187.6
U+4B88.0	fTotalStrokes	 16
U+4B88.0	fCangjie	SFNIN
U+4B88.0	fFourCorner	77320
U+4B88.0	fCantonese	ci4
U+4B88.0	fMandarin	chi2
U+4B88.0	fEnglish	same as &#x99B3;
U+4B89.0	fUTF8	䮉
U+4B89.0	fCNS11643	5-5A2C
U+4B89.0	fR/S	 187.6
U+4B89.0	fTotalStrokes	 16
U+4B89.0	fCangjie	SFOHV
U+4B89.0	fFourCorner	78330
U+4B89.0	fHakka	  li3;  lu2
U+4B89.0	fCantonese	leoi6; leoi4
U+4B89.0	fMandarin	luu3; lu2; luu4; l4
U+4B89.0	fEnglish	post-house
U+4B8A.0	fUTF8	䮊
U+4B8A.0	fCNS11643	3-5369
U+4B8A.0	fR/S	 187.6
U+4B8A.0	fTotalStrokes	 16
U+4B8A.0	fCangjie	SFKN
U+4B8A.0	fFourCorner	75330
U+4B8A.0	fCantonese	ji4; zi4
U+4B8A.0	fMandarin	yi2
U+4B8A.0	fEnglish	the name of a horse
U+4B8B.0	fUTF8	䮋
U+4B8B.0	fCNS11643	4-552D
U+4B8B.0	fGB	3-3E2F
U+4B8B.0	fR/S	 187.6
U+4B8B.0	fTotalStrokes	 16
U+4B8B.0	fCangjie	SFMNN
U+4B8B.0	fFourCorner	72300
U+4B8B.0	fCantonese	lit6
U+4B8B.0	fMandarin	lie4; li4
U+4B8B.0	fEnglish	[1] the name of a horse [2] horses in an ordered line [3] travel quickly; move fast; hasten [4] tame [5] well-bred [6] put under control
U+4B8C.0	fUTF8	䮌
U+4B8C.0	fCNS11643	5-5A2E
U+4B8C.0	fR/S	 187.6
U+4B8C.0	fTotalStrokes	 16
U+4B8C.0	fCangjie	SFOD
U+4B8C.0	fFourCorner	74390
U+4B8C.0	fCantonese	jau1; zaau1
U+4B8C.0	fEnglish	[1] fine horse [2] swift horse [3] swift; speedy
U+4B8D.0	fUTF8	䮍
U+4B8D.0	fCNS11643	7-3628
U+4B8D.0	fR/S	 187.6
U+4B8D.0	fTotalStrokes	 16
U+4B8D.0	fCangjie	SFYHV
U+4B8D.0	fFourCorner	71730
U+4B8D.0	fCantonese	niu5; liu5 (also)
U+4B8D.0	fMandarin	niao3; xing4
U+4B8D.0	fEnglish	same as &#x892D;, &#x88CA;
U+4B8E.0	fUTF8	䮎
U+4B8E.0	fCNS11643	4-5A2E
U+4B8E.0	fR/S	 187.7
U+4B8E.0	fTotalStrokes	 17; 16
U+4B8E.0	fCangjie	SFKKB
U+4B8E.0	fFourCorner	74320
U+4B8E.0	fCantonese	hei3; hei1
U+4B8E.0	fMandarin	xi1; xi4
U+4B8E.0	fEnglish	 running horse
U+4B8F.0	fUTF8	䮏
U+4B8F.0	fCNS11643	3-5732
U+4B8F.0	fR/S	 187.7
U+4B8F.0	fTotalStrokes	 17
U+4B8F.0	fCangjie	SFMMR
U+4B8F.0	fFourCorner	71360
U+4B8F.0	fCantonese	ng4; ng6
U+4B8F.0	fMandarin	wu2
U+4B8F.0	fEnglish	the name of a horse
U+4B90.0	fUTF8	䮐
U+4B90.0	fCNS11643	3-5735
U+4B90.0	fR/S	 187.7
U+4B90.0	fTotalStrokes	 17
U+4B90.0	fCangjie	SFBCR
U+4B90.0	fFourCorner	77320
U+4B90.0	fCantonese	gwing1
U+4B90.0	fMandarin	jiong1
U+4B90.0	fEnglish	non-classical form of &#x99C9;
U+4B91.0	fUTF8	䮑
U+4B91.0	fCNS11643	4-5A2D
U+4B91.0	fGB	3-3E42
U+4B91.0	fR/S	 187.7
U+4B91.0	fTotalStrokes	 17; 16
U+4B91.0	fCangjie	SFBDI
U+4B91.0	fFourCorner	72340
U+4B91.0	fCantonese	leoi6; jyut3
U+4B91.0	fMandarin	lei4
U+4B91.0	fEnglish	[1] gray horse [2] a kind of grain
U+4B92.0	fUTF8	䮒
U+4B92.0	fCNS11643	3-5731
U+4B92.0	fR/S	 187.7
U+4B92.0	fTotalStrokes	 17
U+4B92.0	fCangjie	SFIJB
U+4B92.0	fFourCorner	73320
U+4B92.0	fCantonese	pou1; pei1
U+4B92.0	fMandarin	pu1
U+4B92.0	fEnglish	the name of a horse
U+4B93.0	fUTF8	䮓
U+4B93.0	fCNS11643	4-5E67
U+4B93.0	fR/S	 187.8
U+4B93.0	fTotalStrokes	 18
U+4B93.0	fCangjie	SFYAJ
U+4B93.0	fFourCorner	71340
U+4B93.0	fCantonese	zoek3; coek3; zeon6
U+4B93.0	fMandarin	zhuo2; zhao4
U+4B93.0	fEnglish	[1] [`] walking horse [2] a horse not moving forward [3] swift horse
U+4B94.0	fUTF8	䮔
U+4B94.0	fCNS11643	5-6A6E
U+4B94.0	fR/S	 187.8; 187.9
U+4B94.0	fTotalStrokes	 19; 18
U+4B94.0	fCangjie	SFHJM
U+4B94.0	fFourCorner	72310
U+4B94.0	fCantonese	zeoi1; zeoi2; zyut3
U+4B94.0	fMandarin	zui1; zui3
U+4B94.0	fEnglish	[1] a young and fleet-footed horse; foal [2] a horse with a heavy load and hard to move forward
U+4B95.0	fUTF8	䮕
U+4B95.0	fCNS11643	4-5E62
U+4B95.0	fGB	3-3E51
U+4B95.0	fR/S	 187.8
U+4B95.0	fTotalStrokes	 18
U+4B95.0	fCangjie	SFEEE
U+4B95.0	fFourCorner	77340
U+4B95.0	fCantonese	zyut3; zyut6
U+4B95.0	fMandarin	chuo4
U+4B95.0	fEnglish	[1] horse with white forehead [2] ( &#x7684;&#x76E7;)
U+4B96.0	fUTF8	䮖
U+4B96.0	fCNS11643	3-5968
U+4B96.0	fR/S	 187.8
U+4B96.0	fTotalStrokes	 18
U+4B96.0	fCangjie	SFAA
U+4B96.0	fFourCorner	76360
U+4B96.0	fCantonese	coeng1
U+4B96.0	fMandarin	chang1
U+4B96.0	fEnglish	the name of a horse
U+4B97.0	fUTF8	䮗
U+4B97.0	fCNS11643	4-5E68
U+4B97.0	fGB	3-3E4B
U+4B97.0	fR/S	 187.8
U+4B97.0	fTotalStrokes	 18
U+4B97.0	fCangjie	SFUMJ
U+4B97.0	fFourCorner	72340
U+4B97.0	fCantonese	ngon6; on6 (also); ngan6
U+4B97.0	fMandarin	an4; yan4; niu4
U+4B97.0	fEnglish	[1] white from forehead to lips of a horse [2] reddish of a horse head [3] horse head
U+4B98.0	fUTF8	䮘
U+4B98.0	fCNS11643	3-5969
U+4B98.0	fR/S	 187.8
U+4B98.0	fTotalStrokes	 18
U+4B98.0	fCangjie	SFHXU
U+4B98.0	fFourCorner	77310
U+4B98.0	fHakka	 ngi2; ji2; ri2
U+4B98.0	fCantonese	ji4; zi4
U+4B98.0	fMandarin	ni2; er2; pai4; po2
U+4B98.0	fEnglish	 small horse
U+4B99.0	fUTF8	䮙
U+4B99.0	fCNS11643	4-5E63
U+4B99.0	fGB	3-3E48
U+4B99.0	fR/S	 187.8
U+4B99.0	fTotalStrokes	 18
U+4B99.0	fCangjie	SFIRM
U+4B99.0	fFourCorner	73350
U+4B99.0	fCantonese	wik6; gwik1
U+4B99.0	fMandarin	yu4
U+4B99.0	fEnglish	[1] the name of a horse [2] traveling horse
U+4B9A.0	fUTF8	䮚
U+4B9A.0	fCNS11643	4-5E65
U+4B9A.0	fGB	3-3E47
U+4B9A.0	fR/S	 187.8
U+4B9A.0	fTotalStrokes	 18
U+4B9A.0	fCangjie	SFGCE
U+4B9A.0	fFourCorner	74340
U+4B9A.0	fCantonese	lang6; ling6
U+4B9A.0	fMandarin	leng4
U+4B9A.0	fEnglish	[1] diseases of horse [2] horse eat grains
U+4B9B.0	fUTF8	䮛
U+4B9B.0	fCNS11643	3-596C
U+4B9B.0	fR/S	 187.8
U+4B9B.0	fTotalStrokes	 18
U+4B9B.0	fCangjie	SFNLI
U+4B9B.0	fFourCorner	74300
U+4B9B.0	fCantonese	fu6
U+4B9B.0	fMandarin	fu4
U+4B9B.0	fEnglish	same as &#x99D9;
U+4B9C.0	fUTF8	䮜
U+4B9C.0	fCNS11643	4-6241
U+4B9C.0	fGB	3-3E53
U+4B9C.0	fR/S	 187.9
U+4B9C.0	fTotalStrokes	 19
U+4B9C.0	fCangjie	SFPTD
U+4B9C.0	fFourCorner	74390
U+4B9C.0	fCantonese	zaap6; cap3; saap3
U+4B9C.0	fMandarin	sha4; ye4; zha2
U+4B9C.0	fEnglish	same as &#x4BA2;
U+4B9D.0	fUTF8	䮝
U+4B9D.0	fCNS11643	4-623F
U+4B9D.0	fGB	3-3E36
U+4B9D.0	fR/S	 187.9
U+4B9D.0	fTotalStrokes	 19
U+4B9D.0	fCangjie	SFBJJ
U+4B9D.0	fFourCorner	77350
U+4B9D.0	fCantonese	wan4
U+4B9D.0	fMandarin	hun2; huan2; hui1; huan3
U+4B9D.0	fEnglish	[1] a legendary animal [2] wild horse; mustang
U+4B9E.0	fUTF8	䮞
U+4B9E.0	fCNS11643	3-5B6A
U+4B9E.0	fGB	5-4143
U+4B9E.0	fR/S	 187.9
U+4B9E.0	fTotalStrokes	 19
U+4B9E.0	fCangjie	SFQKA
U+4B9E.0	fFourCorner	75360
U+4B9E.0	fCantonese	ceon2
U+4B9E.0	fMandarin	chun3; chu4
U+4B9E.0	fEnglish	[1] a piebald horse [2] particoloured [3] miscellaneous [4] confused and disorderly [5] stupid horse
U+4B9F.0	fUTF8	䮟
U+4B9F.0	fCNS11643	4-6246
U+4B9F.0	fR/S	 187.9
U+4B9F.0	fTotalStrokes	 19
U+4B9F.0	fCangjie	SFJCE
U+4B9F.0	fFourCorner	73340
U+4B9F.0	fCantonese	sau1
U+4B9F.0	fMandarin	sou1; sou3
U+4B9F.0	fEnglish	[1] big and tall (said of horse) [2] same as &#x641C; [3] same as &#x99F7;
U+4BA0.0	fUTF8	䮠
U+4BA0.0	fCNS11643	4-6240
U+4BA0.0	fGB	3-3E55
U+4BA0.0	fR/S	 187.9
U+4BA0.0	fTotalStrokes	 19
U+4BA0.0	fCangjie	SFMRW
U+4BA0.0	fFourCorner	71360
U+4BA0.0	fCantonese	bik1
U+4BA0.0	fMandarin	bi1
U+4BA0.0	fEnglish	a kind of animal
U+4BA1.0	fUTF8	䮡
U+4BA1.0	fCNS11643	4-6244
U+4BA1.0	fGB	3-3E58
U+4BA1.0	fR/S	 187.9
U+4BA1.0	fTotalStrokes	 19
U+4BA1.0	fCangjie	SFOAE
U+4BA1.0	fFourCorner	78340
U+4BA1.0	fCantonese	bat6; bei3
U+4BA1.0	fMandarin	bi4; bo1
U+4BA1.0	fEnglish	[1] a supernatural horse [2] same as &#x99DC; [3] same as &#x99C1;
U+4BA2.0	fUTF8	䮢
U+4BA2.0	fCNS11643	4-6245
U+4BA2.0	fGB	3-3E57
U+4BA2.0	fR/S	 187.9
U+4BA2.0	fTotalStrokes	 19
U+4BA2.0	fCangjie	SFHJX
U+4BA2.0	fFourCorner	72370
U+4BA2.0	fCantonese	zaap6; dip6
U+4BA2.0	fMandarin	sha4; die2
U+4BA2.0	fEnglish	[1] walking horse [2] gallop [3] swift
U+4BA3.0	fUTF8	䮣
U+4BA3.0	fCNS11643	4-6541
U+4BA3.0	fGB	3-3E67
U+4BA3.0	fR/S	 187.10
U+4BA3.0	fTotalStrokes	 20
U+4BA3.0	fCangjie	SFEED
U+4BA3.0	fFourCorner	77390
U+4BA3.0	fCantonese	song1
U+4BA3.0	fMandarin	sang1; sang3
U+4BA3.0	fEnglish	a yellow with a white tail
U+4BA4.0	fUTF8	䮤
U+4BA4.0	fCNS11643	4-6540
U+4BA4.0	fGB	3-3E66
U+4BA4.0	fR/S	 187.10
U+4BA4.0	fTotalStrokes	 20
U+4BA4.0	fCangjie	SFOBG
U+4BA4.0	fFourCorner	74310
U+4BA4.0	fCantonese	hok6
U+4BA4.0	fMandarin	he4; di2; yue4; jue2; huo4
U+4BA4.0	fEnglish	[1] one of the 36 gardens in Han Dynasty [2] same as &#x99B0;
U+4BA5.0	fUTF8	䮥
U+4BA5.0	fCNS11643	3-5D5D
U+4BA5.0	fR/S	 187.10
U+4BA5.0	fTotalStrokes	 20
U+4BA5.0	fCangjie	SFMRB
U+4BA5.0	fFourCorner	71320
U+4BA5.0	fCantonese	lik6
U+4BA5.0	fMandarin	li4
U+4BA5.0	fEnglish	color of the horse
U+4BA6.0	fUTF8	䮦
U+4BA6.0	fCNS11643	3-5D5B
U+4BA6.0	fR/S	 187.10
U+4BA6.0	fTotalStrokes	 20
U+4BA6.0	fCangjie	SFYRB
U+4BA6.0	fFourCorner	70320
U+4BA6.0	fHakka	 giau1
U+4BA6.0	fCantonese	giu1
U+4BA6.0	fMandarin	jiao1
U+4BA6.0	fEnglish	non-classical form of &#x9A55;
U+4BA7.0	fUTF8	䮧
U+4BA7.0	fCNS11643	4-6547
U+4BA7.0	fGB	3-743A
U+4BA7.0	fR/S	 187.10
U+4BA7.0	fTotalStrokes	 20
U+4BA7.0	fCangjie	JJOSF
U+4BA7.0	fFourCorner	48420
U+4BA7.0	fCantonese	hon6; hon4
U+4BA7.0	fMandarin	han4; han2
U+4BA7.0	fEnglish	long-hair horse
U+4BA8.0	fUTF8	䮨
U+4BA8.0	fCNS11643	5-6E66
U+4BA8.0	fR/S	 187.10
U+4BA8.0	fTotalStrokes	 20
U+4BA8.0	fCangjie	SFJYJ
U+4BA8.0	fFourCorner	73340
U+4BA8.0	fCantonese	zoi2
U+4BA8.0	fMandarin	zai3
U+4BA8.0	fEnglish	[1] a piebald horse [2] particoloured
U+4BA9.0	fUTF8	䮩
U+4BA9.0	fCNS11643	5-6E6D
U+4BA9.0	fR/S	 187.10
U+4BA9.0	fTotalStrokes	 20
U+4BA9.0	fCangjie	SFBBB
U+4BA9.0	fFourCorner	77320
U+4BA9.0	fCantonese	gwat1; gwaat1; wut6
U+4BA9.0	fMandarin	gu2; gu3
U+4BA9.0	fEnglish	a kind of animal (from Beihai)
U+4BAA.0	fUTF8	䮪
U+4BAA.0	fCNS11643	3-5D5C
U+4BAA.0	fR/S	 187.11; 187.10
U+4BAA.0	fTotalStrokes	 20; 21
U+4BAA.0	fCangjie	SFNQD
U+4BAA.0	fFourCorner	75390
U+4BAA.0	fCantonese	sing4; zaang6
U+4BAA.0	fMandarin	cheng2
U+4BAA.0	fEnglish	(corrupted form) same as &#x9A2C;
U+4BAB.0	fUTF8	䮫
U+4BAB.0	fCNS11643	4-6829
U+4BAB.0	fGB	5-4148
U+4BAB.0	fR/S	 187.11
U+4BAB.0	fTotalStrokes	 21
U+4BAB.0	fCangjie	SFLLV
U+4BAB.0	fFourCorner	75340
U+4BAB.0	fHakka	  leu2
U+4BAB.0	fCantonese	lau4; laau4; lou4
U+4BAB.0	fMandarin	lou2; luu2; l2
U+4BAB.0	fEnglish	[1] large horse [2] jackass
U+4BAC.0	fUTF8	䮬
U+4BAC.0	fCNS11643	4-682A
U+4BAC.0	fGB	3-3E61
U+4BAC.0	fR/S	 187.11
U+4BAC.0	fTotalStrokes	 21
U+4BAC.0	fCangjie	SFTAK
U+4BAC.0	fFourCorner	74330
U+4BAC.0	fCantonese	mok6; maak6
U+4BAC.0	fMandarin	mo4
U+4BAC.0	fEnglish	[1] the name of a horse [2] same as &#x9A40;
U+4BAD.0	fUTF8	䮭
U+4BAD.0	fCNS11643	4-682B
U+4BAD.0	fGB	3-3E4D
U+4BAD.0	fR/S	 187.11
U+4BAD.0	fTotalStrokes	 21
U+4BAD.0	fCangjie	SFBBU
U+4BAD.0	fFourCorner	72310
U+4BAD.0	fCantonese	mik6
U+4BAD.0	fMandarin	mi4
U+4BAD.0	fEnglish	[1] look at in surprise [2] gnaw; bite (said of a horse)
U+4BAE.0	fUTF8	䮮
U+4BAE.0	fCNS11643	4-682D
U+4BAE.0	fGB	3-3E38
U+4BAE.0	fR/S	 187.11
U+4BAE.0	fTotalStrokes	 21
U+4BAE.0	fCangjie	SFJON
U+4BAE.0	fFourCorner	74347
U+4BAE.0	fCantonese	mak6
U+4BAE.0	fMandarin	mo4; mai4
U+4BAE.0	fEnglish	a kind of mule; a kind of beast
U+4BAF.0	fUTF8	䮯
U+4BAF.0	fCNS11643	3-5F23
U+4BAF.0	fR/S	 187.11
U+4BAF.0	fTotalStrokes	 21
U+4BAF.0	fCangjie	SFGSK
U+4BAF.0	fFourCorner	78340
U+4BAF.0	fCantonese	ngou4; ou4 (also); ngou6
U+4BAF.0	fMandarin	ao2; ao4
U+4BAF.0	fEnglish	same as &#x9A41;
U+4BB0.0	fUTF8	䮰
U+4BB0.0	fCNS11643	5-722D
U+4BB0.0	fR/S	 187.11
U+4BB0.0	fTotalStrokes	 21
U+4BB0.0	fCangjie	SFYCB
U+4BB0.0	fFourCorner	70320
U+4BB0.0	fCantonese	zaak6
U+4BB0.0	fMandarin	zhe2; dan3; dan4
U+4BB0.0	fEnglish	[1] a kind of beast [2] a kind of mule
U+4BB1.0	fUTF8	䮱
U+4BB1.0	fCNS11643	4-6828
U+4BB1.0	fGB	3-3E63
U+4BB1.0	fR/S	 187.11
U+4BB1.0	fTotalStrokes	 21
U+4BB1.0	fCangjie	SFYMO
U+4BB1.0	fFourCorner	71330
U+4BB1.0	fCantonese	zuk6
U+4BB1.0	fMandarin	zhu2
U+4BB1.0	fEnglish	[1] a kind of beast [2] a flock of horse to chase each other
U+4BB2.0	fUTF8	䮲
U+4BB2.0	fCNS11643	7-5A3E
U+4BB2.0	fR/S	 187.12
U+4BB2.0	fTotalStrokes	 22
U+4BB2.0	fCangjie	SFTMC
U+4BB2.0	fFourCorner	74380
U+4BB2.0	fCantonese	wong4
U+4BB2.0	fMandarin	huang2
U+4BB2.0	fEnglish	same as &#x9A1C;
U+4BB3.0	fUTF8	䮳
U+4BB3.0	fCNS11643	5-7476
U+4BB3.0	fR/S	 187.12
U+4BB3.0	fTotalStrokes	 22
U+4BB3.0	fCangjie	SFHDW
U+4BB3.0	fFourCorner	72360
U+4BB3.0	fCantonese	faan4
U+4BB3.0	fMandarin	fan2
U+4BB3.0	fEnglish	[1] give birth to [2] bring and to rear [3] interchangeable with &#x8543;
U+4BB4.0	fUTF8	䮴
U+4BB4.0	fCNS11643	4-696B
U+4BB4.0	fGB	3-3E6D
U+4BB4.0	fR/S	 187.12
U+4BB4.0	fTotalStrokes	 22; 21
U+4BB4.0	fCangjie	SFNOT
U+4BB4.0	fFourCorner	72310
U+4BB4.0	fCantonese	dang6
U+4BB4.0	fMandarin	deng4; teng1
U+4BB4.0	fEnglish	[1] unstable walking [2] fall [3] disease of the horse
U+4BB5.0	fUTF8	䮵
U+4BB5.0	fCNS11643	4-6969
U+4BB5.0	fGB	5-4151
U+4BB5.0	fR/S	 187.12
U+4BB5.0	fTotalStrokes	 22; 21
U+4BB5.0	fCangjie	SFYTG
U+4BB5.0	fFourCorner	70310
U+4BB5.0	fCantonese	tung4
U+4BB5.0	fMandarin	tong2; yong3
U+4BB5.0	fEnglish	[1] the name of a horse [2] young horse
U+4BB6.0	fUTF8	䮶
U+4BB6.0	fCNS11643	3-5F7A
U+4BB6.0	fR/S	 187.12
U+4BB6.0	fTotalStrokes	 22
U+4BB6.0	fCangjie	SFOGD
U+4BB6.0	fFourCorner	70390
U+4BB6.0	fCantonese	biu1
U+4BB6.0	fMandarin	ji2
U+4BB6.0	fEnglish	same as &#x9A6B;
U+4BB7.0	fUTF8	䮷
U+4BB7.0	fCNS11643	4-6B33
U+4BB7.0	fGB	3-3E70
U+4BB7.0	fR/S	 187.13
U+4BB7.0	fTotalStrokes	 23
U+4BB7.0	fCangjie	SFWLI
U+4BB7.0	fFourCorner	76320
U+4BB7.0	fCantonese	duk6
U+4BB7.0	fMandarin	du2
U+4BB7.0	fEnglish	[1] movement of a horse [2] traveling horse
U+4BB8.0	fUTF8	䮸
U+4BB8.0	fCNS11643	4-6B34
U+4BB8.0	fGB	3-3E2B
U+4BB8.0	fR/S	 187.13
U+4BB8.0	fTotalStrokes	 23
U+4BB8.0	fCangjie	HBSQF
U+4BB8.0	fFourCorner	77320
U+4BB8.0	fCantonese	ak1; ngak1 (also); aak1
U+4BB8.0	fMandarin	jue2; wo4; hu2; wu4; mu2
U+4BB8.0	fEnglish	[1] [2] move slowly and then swiftly (said of a horse) [3] swift horse [4] to neigh
U+4BB9.0	fUTF8	䮹
U+4BB9.0	fCNS11643	4-6B32
U+4BB9.0	fGB	3-3E33
U+4BB9.0	fR/S	 187.13
U+4BB9.0	fTotalStrokes	 23
U+4BB9.0	fCangjie	SFYMH
U+4BB9.0	fFourCorner	71350
U+4BB9.0	fCantonese	wai3; kyut3; waai3
U+4BB9.0	fMandarin	hui4; wei4; jue2
U+4BB9.0	fEnglish	[1] angry horse [2] hot-tempered horse [3] an evil horse
U+4BBA.0	fUTF8	䮺
U+4BBA.0	fCNS11643	3-612B
U+4BBA.0	fR/S	 187.14
U+4BBA.0	fTotalStrokes	 24
U+4BBA.0	fCangjie	SFYX
U+4BBA.0	fFourCorner	70320
U+4BBA.0	fCantonese	zai1
U+4BBA.0	fMandarin	qi2; ji4
U+4BBA.0	fEnglish	[1] luxuriant of the grass; the grass growing densely [2] horses traveling in line order and swiftly
U+4BBB.0	fUTF8	䮻
U+4BBB.0	fCNS11643	7-602E
U+4BBB.0	fR/S	 187.14
U+4BBB.0	fTotalStrokes	 24
U+4BBB.0	fCangjie	SFGNI
U+4BBB.0	fFourCorner	74340
U+4BBB.0	fCantonese	dou2; tou2
U+4BBB.0	fMandarin	dao3; chi4; dao4; dei3
U+4BBB.0	fEnglish	same as &#x7982;
U+4BBC.0	fUTF8	䮼
U+4BBC.0	fCNS11643	7-602D
U+4BBC.0	fR/S	 187.14
U+4BBC.0	fTotalStrokes	 24
U+4BBC.0	fCangjie	SFFFQ
U+4BBC.0	fFourCorner	79350
U+4BBC.0	fCantonese	leon4
U+4BBC.0	fMandarin	lin2
U+4BBC.0	fEnglish	standard form of &#x9A4E;
U+4BBD.0	fUTF8	䮽
U+4BBD.0	fCNS11643	3-6157
U+4BBD.0	fR/S	 187.15
U+4BBD.0	fTotalStrokes	 25
U+4BBD.0	fCangjie	SFIPF
U+4BBD.0	fFourCorner	70330
U+4BBD.0	fCantonese	biu1
U+4BBD.0	fMandarin	biao1
U+4BBD.0	fEnglish	same as &#x9463;
U+4BBE.0	fUTF8	䮾
U+4BBE.0	fCNS11643	4-6D68
U+4BBE.0	fGB	3-3E21
U+4BBE.0	fR/S	 187.16
U+4BBE.0	fTotalStrokes	 26
U+4BBE.0	fCangjie	SFYBP
U+4BBE.0	fFourCorner	71310
U+4BBE.0	fCantonese	pung4; pong4
U+4BBE.0	fMandarin	peng2; pang2; long2
U+4BBE.0	fEnglish	[1] same as &#x9F90; [2] same as &#x9F8D;
U+4BBF.0	fUTF8	䮿
U+4BBF.0	fCNS11643	3-6227
U+4BBF.0	fR/S	 187.17
U+4BBF.0	fTotalStrokes	 27
U+4BBF.0	fCangjie	SFJTO
U+4BBF.0	fFourCorner	73380
U+4BBF.0	fHakka	 kien5; ken3
U+4BBF.0	fCantonese	gin2
U+4BBF.0	fMandarin	qian1; jian3
U+4BBF.0	fEnglish	the lame mule or ass
U+4BC0.0	fUTF8	䯀
U+4BC0.0	fCNS11643	4-6E3F
U+4BC0.0	fGB	3-3E60
U+4BC0.0	fR/S	 187.18
U+4BC0.0	fTotalStrokes	 28
U+4BC0.0	fCangjie	SFSJJ
U+4BC0.0	fFourCorner	71340
U+4BC0.0	fHakka	 ngiap7
U+4BC0.0	fCantonese	nip6; lip6 (also); seoi1; waai2
U+4BC0.0	fMandarin	nie4
U+4BC0.0	fEnglish	fleet horse
U+4BC1.0	fUTF8	䯁
U+4BC1.0	fCNS11643	3-6237
U+4BC1.0	fR/S	 187.19
U+4BC1.0	fTotalStrokes	 29
U+4BC1.0	fCangjie	SFYNQ
U+4BC1.0	fFourCorner	70310
U+4BC1.0	fCantonese	leoi4
U+4BC1.0	fMandarin	luo2
U+4BC1.0	fEnglish	same as abbreviated form &#x9A58;, &#x9A3E;
U+4BC2.0	fUTF8	䯂
U+4BC2.0	fCNS11643	4-6E57
U+4BC2.0	fGB	3-4251
U+4BC2.0	fR/S	 187.24
U+4BC2.0	fTotalStrokes	 34
U+4BC2.0	fCangjie	SFSFD
U+4BC2.0	fFourCorner	71900
U+4BC2.0	fCantonese	san1
U+4BC2.0	fMandarin	ji2
U+4BC2.0	fEnglish	[1] numerous; very many [2] large group of horses traveling in a line
U+4BC3.0	fUTF8	䯃
U+4BC3.0	fGB	S-2345
U+4BC3.0	fR/S	 187.6
U+4BC3.0	fTotalStrokes	 16
U+4BC3.0	fCangjie	NMJV
U+4BC3.0	fCantonese	on1
U+4BC3.0	fEnglish	simplified form of &#x978D;
U+4BC4.0	fUTF8	䯄
U+4BC4.0	fGB	S-2347
U+4BC4.0	fR/S	 187.7
U+4BC4.0	fTotalStrokes	 17
U+4BC4.0	fCangjie	NMROB
U+4BC4.0	fCantonese	waa1
U+4BC4.0	fEnglish	simplified form of &#x9A27;
U+4BC5.0	fUTF8	䯅
U+4BC5.0	fGB	S-2342
U+4BC5.0	fR/S	 187.10
U+4BC5.0	fTotalStrokes	 20
U+4BC5.0	fCangjie	NMSJE
U+4BC5.0	fCantonese	nip6
U+4BC5.0	fMandarin	nie4
U+4BC5.0	fEnglish	simplified form of &#x4BC0;
U+4BC6.0	fUTF8	䯆
U+4BC6.0	fCNS11643	3-3969
U+4BC6.0	fR/S	 188.1
U+4BC6.0	fTotalStrokes	 11
U+4BC6.0	fCangjie	BBU
U+4BC6.0	fFourCorner	72210
U+4BC6.0	fCantonese	jik6; zik6
U+4BC6.0	fMandarin	yi4
U+4BC6.0	fEnglish	same as &#x9AAE;
U+4BC7.0	fUTF8	䯇
U+4BC7.0	fCNS11643	3-402E
U+4BC7.0	fGB	5-776B
U+4BC7.0	fR/S	 188.2
U+4BC7.0	fTotalStrokes	 12
U+4BC7.0	fCangjie	BBKS
U+4BC7.0	fFourCorner	74220
U+4BC7.0	fCantonese	fat1; gwaat6; kyut3
U+4BC7.0	fMandarin	ku1; kuo4
U+4BC7.0	fEnglish	 exert oneself; make an effort; diligent; sedulous
U+4BC8.0	fUTF8	䯈
U+4BC8.0	fCNS11643	4-4862
U+4BC8.0	fGB	3-796E
U+4BC8.0	fR/S	 188.4
U+4BC8.0	fTotalStrokes	 14
U+4BC8.0	fCangjie	BBMMU
U+4BC8.0	fFourCorner	71210
U+4BC8.0	fCantonese	jyun4; hin2; zyun4
U+4BC8.0	fMandarin	wan2
U+4BC8.0	fEnglish	same as &#x4BCA;
U+4BC9.0	fUTF8	䯉
U+4BC9.0	fCNS11643	3-4A6D
U+4BC9.0	fR/S	 188.4
U+4BC9.0	fTotalStrokes	 14
U+4BC9.0	fCangjie	BBNO
U+4BC9.0	fFourCorner	77280
U+4BC9.0	fCantonese	waat3; at3
U+4BC9.0	fMandarin	wa4; ya4
U+4BC9.0	fEnglish	 choke; the throat chocked and unable to breath smoothly
U+4BCA.0	fUTF8	䯊
U+4BCA.0	fCNS11643	3-4F75
U+4BCA.0	fGB	5-7770
U+4BCA.0	fR/S	 188.5
U+4BCA.0	fTotalStrokes	 15
U+4BCA.0	fCangjie	BBMNR
U+4BCA.0	fFourCorner	71220
U+4BCA.0	fCantonese	kaa3; o1
U+4BCA.0	fMandarin	ke1; qia4
U+4BCA.0	fEnglish	[1] the pelvis [2] bone stuck in the throat [3] the kneecap; patella; the five lowest pieces of bone of the spinal column
U+4BCB.0	fUTF8	䯋
U+4BCB.0	fCNS11643	4-4E75
U+4BCB.0	fGB	3-796F
U+4BCB.0	fR/S	 188.5
U+4BCB.0	fTotalStrokes	 15
U+4BCB.0	fCangjie	BBIKK
U+4BCB.0	fFourCorner	73240
U+4BCB.0	fHakka	 pat8
U+4BCB.0	fCantonese	bat6; bok6
U+4BCB.0	fMandarin	ba2; bo2; ke1; fei4
U+4BCB.0	fEnglish	shoulder-blade
U+4BCC.0	fUTF8	䯌
U+4BCC.0	fCNS11643	4-4E73
U+4BCC.0	fGB	3-7972
U+4BCC.0	fR/S	 188.5
U+4BCC.0	fTotalStrokes	 15
U+4BCC.0	fCangjie	BBSKN
U+4BCC.0	fFourCorner	77210
U+4BCC.0	fHakka	 kau1; gau1
U+4BCC.0	fCantonese	hou1; hau1
U+4BCC.0	fMandarin	kao1
U+4BCC.0	fEnglish	[1] end of the spine: the rump [2] buttocks
U+4BCD.0	fUTF8	䯍
U+4BCD.0	fCNS11643	4-4E76
U+4BCD.0	fGB	3-7970
U+4BCD.0	fR/S	 188.5
U+4BCD.0	fTotalStrokes	 15
U+4BCD.0	fCangjie	BBOII
U+4BCD.0	fFourCorner	78230
U+4BCD.0	fCantonese	ling4
U+4BCD.0	fMandarin	ling2
U+4BCD.0	fEnglish	[1] bones [2] the pelvis; the five lowest pieces of bone of the spinal column
U+4BCE.0	fUTF8	䯎
U+4BCE.0	fCNS11643	3-4F78
U+4BCE.0	fR/S	 188.5
U+4BCE.0	fTotalStrokes	 15
U+4BCE.0	fCangjie	BBCMJ
U+4BCE.0	fFourCorner	78240
U+4BCE.0	fCantonese	gon3
U+4BCE.0	fMandarin	gan4
U+4BCE.0	fEnglish	same as &#x9AAD;
U+4BCF.0	fUTF8	䯏
U+4BCF.0	fCNS11643	3-5372
U+4BCF.0	fGB	5-7777
U+4BCF.0	fR/S	 188.6
U+4BCF.0	fTotalStrokes	 16
U+4BCF.0	fCangjie	BBHJR
U+4BCF.0	fFourCorner	72260
U+4BCF.0	fCantonese	kut3; kwut3 (also); gwaat3
U+4BCF.0	fMandarin	gua1; huo2; hua2
U+4BCF.0	fEnglish	the shoulder blade; scapula
U+4BD0.0	fUTF8	䯐
U+4BD0.0	fCNS11643	3-5373
U+4BD0.0	fR/S	 188.6
U+4BD0.0	fTotalStrokes	 16
U+4BD0.0	fCangjie	BBOBO
U+4BD0.0	fFourCorner	74220
U+4BD0.0	fHakka	 hai2
U+4BD0.0	fCantonese	haai4; hai4; ngoi4
U+4BD0.0	fMandarin	xie2; hai2
U+4BD0.0	fEnglish	non-classical form of &#x9AB8;
U+4BD1.0	fUTF8	䯑
U+4BD1.0	fCNS11643	4-5530
U+4BD1.0	fGB	3-7979
U+4BD1.0	fR/S	 188.6
U+4BD1.0	fTotalStrokes	 16
U+4BD1.0	fCangjie	BBFMU
U+4BD1.0	fFourCorner	79210
U+4BD1.0	fCantonese	kwong1; kong1 (also); hong4
U+4BD1.0	fMandarin	guang2
U+4BD1.0	fEnglish	[1] the pelvis [2] bone stuck in the throat [3] the kneecap; patella; the five lowest pieces of bone of the spinal column
U+4BD2.0	fUTF8	䯒
U+4BD2.0	fCNS11643	4-5532
U+4BD2.0	fGB	3-797A
U+4BD2.0	fR/S	 188.6
U+4BD2.0	fTotalStrokes	 16
U+4BD2.0	fCangjie	BBHON
U+4BD2.0	fFourCorner	71220
U+4BD2.0	fHakka	 hang2
U+4BD2.0	fCantonese	hang4
U+4BD2.0	fMandarin	heng2
U+4BD2.0	fEnglish	tail-end of the backbone of an ox
U+4BD3.0	fUTF8	䯓
U+4BD3.0	fCNS11643	4-552F
U+4BD3.0	fGB	3-7974
U+4BD3.0	fR/S	 188.6
U+4BD3.0	fTotalStrokes	 16
U+4BD3.0	fCangjie	BBGG
U+4BD3.0	fFourCorner	74210
U+4BD3.0	fCantonese	kwai1; kwaai4
U+4BD3.0	fMandarin	kui2
U+4BD3.0	fEnglish	[1] the head bone of livestock [2] the shoulder blade; scapula
U+4BD4.0	fUTF8	䯔
U+4BD4.0	fCNS11643	4-552E
U+4BD4.0	fGB	3-797D
U+4BD4.0	fR/S	 188.6; 187.10
U+4BD4.0	fTotalStrokes	 16; 20
U+4BD4.0	fCangjie	BBJHP
U+4BD4.0	fFourCorner	73214
U+4BD4.0	fCantonese	zaak3
U+4BD4.0	fMandarin	sang1; cen2; ze2
U+4BD4.0	fEnglish	 incline; turn on one side
U+4BD5.0	fUTF8	䯕
U+4BD5.0	fCNS11643	3-5739
U+4BD5.0	fR/S	 188.7
U+4BD5.0	fTotalStrokes	 17
U+4BD5.0	fCangjie	BBNKG
U+4BD5.0	fFourCorner	72241
U+4BD5.0	fHakka	 tin2
U+4BD5.0	fCantonese	ting1
U+4BD5.0	fMandarin	ting2
U+4BD5.0	fEnglish	bony; skinny
U+4BD6.0	fUTF8	䯖
U+4BD6.0	fCNS11643	4-5A31
U+4BD6.0	fGB	3-7A21
U+4BD6.0	fR/S	 188.7
U+4BD6.0	fTotalStrokes	 17; 16
U+4BD6.0	fCangjie	BBIAV
U+4BD6.0	fFourCorner	73230
U+4BD6.0	fCantonese	long4; lou6
U+4BD6.0	fMandarin	liang2; lang2
U+4BD6.0	fEnglish	[1] the muscles of the thigh [2] the hips [3] the haunches
U+4BD7.0	fUTF8	䯗
U+4BD7.0	fCNS11643	4-5A35
U+4BD7.0	fGB	5-777A
U+4BD7.0	fR/S	 188.7
U+4BD7.0	fTotalStrokes	 17; 16
U+4BD7.0	fCangjie	BBPPG
U+4BD7.0	fFourCorner	71210
U+4BD7.0	fCantonese	bei2; bei1; bei3
U+4BD7.0	fMandarin	bi3; bi4
U+4BD7.0	fEnglish	same as &#x9AC0;
U+4BD8.0	fUTF8	䯘
U+4BD8.0	fCNS11643	4-5A30
U+4BD8.0	fGB	5-777B
U+4BD8.0	fR/S	 188.7
U+4BD8.0	fTotalStrokes	 17; 16
U+4BD8.0	fCangjie	BBJMU
U+4BD8.0	fFourCorner	73210
U+4BD8.0	fHakka	 fan5; fon5
U+4BD8.0	fCantonese	jyun6; wun6
U+4BD8.0	fMandarin	huan4
U+4BD8.0	fEnglish	 knee-joint
U+4BD9.0	fUTF8	䯙
U+4BD9.0	fCNS11643	3-5738
U+4BD9.0	fR/S	 188.7
U+4BD9.0	fTotalStrokes	 17
U+4BD9.0	fCangjie	BBIJB
U+4BD9.0	fFourCorner	73220
U+4BD9.0	fCantonese	bok3; pak3; pang4
U+4BD9.0	fMandarin	fu3; po4
U+4BD9.0	fEnglish	same as &#x9AC6;
U+4BDA.0	fUTF8	䯚
U+4BDA.0	fCNS11643	4-5A33
U+4BDA.0	fGB	5-777C
U+4BDA.0	fR/S	 188.7
U+4BDA.0	fTotalStrokes	 17; 16
U+4BDA.0	fCangjie	BBVIB
U+4BDA.0	fFourCorner	72220
U+4BDA.0	fCantonese	jiu5; zing1; ziu5; zoek3
U+4BDA.0	fMandarin	yao3; ou3
U+4BDA.0	fEnglish	the ribs
U+4BDB.0	fUTF8	䯛
U+4BDB.0	fCNS11643	5-6651
U+4BDB.0	fR/S	 188.8
U+4BDB.0	fTotalStrokes	 18
U+4BDB.0	fCangjie	BBJNU
U+4BDB.0	fFourCorner	73210
U+4BDB.0	fHakka	 won3; van3; von3
U+4BDB.0	fCantonese	wun3; wun2
U+4BDB.0	fMandarin	wan3; jian3; wan4
U+4BDB.0	fEnglish	the knee-joint
U+4BDC.0	fUTF8	䯜
U+4BDC.0	fCNS11643	4-5E6E
U+4BDC.0	fGB	3-7A22
U+4BDC.0	fR/S	 188.8
U+4BDC.0	fTotalStrokes	 18
U+4BDC.0	fCangjie	BBAPH
U+4BDC.0	fFourCorner	76220
U+4BDC.0	fCantonese	tik1
U+4BDC.0	fMandarin	xi2; ti4; chi4
U+4BDC.0	fEnglish	marrow; fluid of yellow color between the waist bones
U+4BDD.0	fUTF8	䯝
U+4BDD.0	fCNS11643	3-5B70
U+4BDD.0	fR/S	 188.9
U+4BDD.0	fTotalStrokes	 19
U+4BDD.0	fCangjie	BBKMB
U+4BDD.0	fFourCorner	74220
U+4BDD.0	fHakka	 sui3; soi3; sioi3
U+4BDD.0	fCantonese	seoi5
U+4BDD.0	fMandarin	sui3
U+4BDD.0	fEnglish	same as &#x9AD3;
U+4BDE.0	fUTF8	䯞
U+4BDE.0	fCNS11643	4-6248
U+4BDE.0	fGB	3-797E
U+4BDE.0	fR/S	 188.9
U+4BDE.0	fTotalStrokes	 19
U+4BDE.0	fCangjie	BBBBR
U+4BDE.0	fFourCorner	77220
U+4BDE.0	fCantonese	kwaa1
U+4BDE.0	fMandarin	kua1
U+4BDE.0	fEnglish	bone (the lowest pieces of bone of the spinal column and sides of the abdominal region)
U+4BDF.0	fUTF8	䯟
U+4BDF.0	fCNS11643	4-6247
U+4BDF.0	fGB	3-7A24
U+4BDF.0	fR/S	 188.9
U+4BDF.0	fTotalStrokes	 19
U+4BDF.0	fCangjie	BBTPO
U+4BDF.0	fFourCorner	78230
U+4BDF.0	fCantonese	deoi6
U+4BDF.0	fMandarin	dui4; xia2
U+4BDF.0	fEnglish	[1] bone [2] stupid; dull
U+4BE0.0	fUTF8	䯠
U+4BE0.0	fCNS11643	4-6549
U+4BE0.0	fGB	5-7821
U+4BE0.0	fR/S	 188.10; 188.9
U+4BE0.0	fTotalStrokes	 20; 19
U+4BE0.0	fCangjie	BBABT
U+4BE0.0	fFourCorner	76210
U+4BE0.0	fCantonese	ou2; ngou2 (also); faau2; pei3; zuk1
U+4BE0.0	fMandarin	ao3; ao4
U+4BE0.0	fEnglish	the five lowest pieces of bone of the spinal column
U+4BE1.0	fUTF8	䯡
U+4BE1.0	fCNS11643	3-5D5F
U+4BE1.0	fR/S	 188.10
U+4BE1.0	fTotalStrokes	 20
U+4BE1.0	fCangjie	BBTXC
U+4BE1.0	fFourCorner	78230
U+4BE1.0	fCantonese	gim3
U+4BE1.0	fMandarin	jian4; jian3; qian4
U+4BE2.0	fUTF8	䯢
U+4BE2.0	fCNS11643	4-6830
U+4BE2.0	fGB	5-7976
U+4BE2.0	fR/S	 188.11
U+4BE2.0	fTotalStrokes	 21
U+4BE2.0	fCangjie	IDBBB
U+4BE2.0	fFourCorner	00220
U+4BE2.0	fCantonese	mo4; mo1
U+4BE2.0	fMandarin	mo2; mo3
U+4BE2.0	fEnglish	[1] partial paralysis; hemiplegia - paralysis of half of one's body [2] very small; diminutive; minute
U+4BE3.0	fUTF8	䯣
U+4BE3.0	fCNS11643	4-696E
U+4BE3.0	fGB	3-7A23
U+4BE3.0	fR/S	 188.12
U+4BE3.0	fTotalStrokes	 22; 21
U+4BE3.0	fCangjie	BBLMC
U+4BE3.0	fFourCorner	75280
U+4BE3.0	fCantonese	kwai3; waai2
U+4BE3.0	fMandarin	kui4; gui4
U+4BE3.0	fEnglish	[1] the kneecap; patella [2] the cranium
U+4BE4.0	fUTF8	䯤
U+4BE4.0	fCNS11643	4-6B35
U+4BE4.0	fGB	5-7778
U+4BE4.0	fR/S	 188.13
U+4BE4.0	fTotalStrokes	 23
U+4BE4.0	fCangjie	BBOMA
U+4BE4.0	fFourCorner	78260
U+4BE4.0	fCantonese	kui2; kwui2 (also)
U+4BE4.0	fMandarin	kuai4
U+4BE4.0	fEnglish	[1] boneware used to comb the hair [2] bind the hair [3] same as &#x9B20;
U+4BE5.0	fUTF8	䯥
U+4BE5.0	fCNS11643	4-6C3E
U+4BE5.0	fGB	3-7A28
U+4BE5.0	fR/S	 188.14
U+4BE5.0	fTotalStrokes	 24
U+4BE5.0	fCangjie	BBAVF
U+4BE5.0	fFourCorner	76230
U+4BE5.0	fCantonese	ngam6; am6 (also); zaap1
U+4BE5.0	fMandarin	an4; qi4
U+4BE5.0	fEnglish	[1] with a high skull [2] dried meat strips
U+4BE6.0	fUTF8	䯦
U+4BE6.0	fCNS11643	4-6D33
U+4BE6.0	fGB	3-7A27
U+4BE6.0	fR/S	 188.15
U+4BE6.0	fTotalStrokes	 25
U+4BE6.0	fCangjie	BBTWI
U+4BE6.0	fFourCorner	74250
U+4BE6.0	fCantonese	maat3; maa3; mut6
U+4BE6.0	fMandarin	mo4; ma4
U+4BE6.0	fEnglish	tiny bone
U+4BE7.0	fUTF8	䯧
U+4BE7.0	fCNS11643	3-4030
U+4BE7.0	fGB	5-3355
U+4BE7.0	fR/S	 189.2
U+4BE7.0	fTotalStrokes	 12
U+4BE7.0	fCangjie	YRBB
U+4BE7.0	fFourCorner	00220
U+4BE7.0	fCantonese	king2; at3; got3; king3
U+4BE7.0	fMandarin	qing3
U+4BE7.0	fEnglish	same as &#x5ECE;
U+4BE8.0	fUTF8	䯨
U+4BE8.0	fCNS11643	4-4222
U+4BE8.0	fGB	5-7965
U+4BE8.0	fR/S	 189.3
U+4BE8.0	fTotalStrokes	 13
U+4BE8.0	fCangjie	YBK
U+4BE8.0	fFourCorner	00430
U+4BE8.0	fCantonese	haau1; hau1; kok3
U+4BE8.0	fMandarin	huo4; fen2; he4
U+4BE8.0	fEnglish	ancient form of &#x6572;
U+4BE9.0	fUTF8	䯩
U+4BE9.0	fR/S	 189.4
U+4BE9.0	fTotalStrokes	 14
U+4BE9.0	fCangjie	YBAU
U+4BE9.0	fCantonese	hou6
U+4BE9.0	fEnglish	[Korean character]
U+4BEA.0	fUTF8	䯪
U+4BEA.0	fCNS11643	4-624A
U+4BEA.0	fGB	3-7B71
U+4BEA.0	fR/S	 189.9
U+4BEA.0	fTotalStrokes	 19
U+4BEA.0	fCangjie	YBMBC
U+4BEA.0	fFourCorner	01280
U+4BEA.0	fCantonese	haau2
U+4BEA.0	fMandarin	kao3; kao4
U+4BEA.0	fEnglish	big head
U+4BEB.0	fUTF8	䯫
U+4BEB.0	fCNS11643	5-747A
U+4BEB.0	fR/S	 189.12
U+4BEB.0	fTotalStrokes	 22
U+4BEB.0	fCangjie	AFYRB
U+4BEB.0	fFourCorner	60920
U+4BEB.0	fCantonese	hou6
U+4BEB.0	fMandarin	hao4; sha4
U+4BEB.0	fEnglish	same as &#x9865;
U+4BEC.0	fUTF8	䯬
U+4BEC.0	fCNS11643	7-6475
U+4BEC.0	fR/S	 189.18; 189.15
U+4BEC.0	fTotalStrokes	 28; 25
U+4BEC.0	fCangjie	YJRRJ
U+4BEC.0	fFourCorner	06450
U+4BEC.0	fCantonese	do2
U+4BEC.0	fMandarin	duo3
U+4BEC.0	fEnglish	same as &#x56B2;
U+4BED.0	fUTF8	䯭
U+4BED.0	fCNS11643	3-4031
U+4BED.0	fR/S	 190.2
U+4BED.0	fTotalStrokes	 12
U+4BED.0	fCangjie	SHE
U+4BED.0	fFourCorner	72400
U+4BED.0	fCantonese	cim1
U+4BED.0	fMandarin	xian1
U+4BED.0	fEnglish	of nice and good hair
U+4BEE.0	fUTF8	䯮
U+4BEE.0	fCNS11643	5-3D52
U+4BEE.0	fR/S	 190.2
U+4BEE.0	fTotalStrokes	 12
U+4BEE.0	fCangjie	SHNHS
U+4BEE.0	fFourCorner	72220
U+4BEE.0	fCantonese	noi4; loi4 (also); naang4; nai1
U+4BEE.0	fMandarin	nai2; nai3; neng2
U+4BEE.0	fEnglish	dishevelled hair
U+4BEF.0	fUTF8	䯯
U+4BEF.0	fCNS11643	6-645E
U+4BEF.0	fR/S	 190.4
U+4BEF.0	fTotalStrokes	 14
U+4BEF.0	fCangjie	SHFH
U+4BEF.0	fFourCorner	72200
U+4BEF.0	fCantonese	so1; seoi1; zyu4
U+4BEF.0	fMandarin	sha1
U+4BEF.0	fEnglish	same as &#x9AFF;
U+4BF0.0	fUTF8	䯰
U+4BF0.0	fCNS11643	4-486A
U+4BF0.0	fGB	3-7B37
U+4BF0.0	fR/S	 190.4
U+4BF0.0	fTotalStrokes	 14
U+4BF0.0	fCangjie	SHOLL
U+4BF0.0	fFourCorner	72220
U+4BF0.0	fCantonese	gaai3
U+4BF0.0	fMandarin	jie4
U+4BF0.0	fEnglish	use a hair-pin to set and dress the hair
U+4BF1.0	fUTF8	䯱
U+4BF1.0	fCNS11643	4-4869
U+4BF1.0	fGB	3-7B36
U+4BF1.0	fR/S	 190.4
U+4BF1.0	fTotalStrokes	 14
U+4BF1.0	fCangjie	SHMF
U+4BF1.0	fFourCorner	72901
U+4BF1.0	fHakka	 pi2;  fu5; pui1; pi1
U+4BF1.0	fCantonese	pei1; pei4
U+4BF1.0	fMandarin	pei3; fu4; pei1; fu3
U+4BF1.0	fEnglish	dishevelled hair
U+4BF2.0	fUTF8	䯲
U+4BF2.0	fCNS11643	4-4867
U+4BF2.0	fGB	3-7B39
U+4BF2.0	fR/S	 190.4
U+4BF2.0	fTotalStrokes	 14
U+4BF2.0	fCangjie	SHAU
U+4BF2.0	fFourCorner	72710
U+4BF2.0	fHakka	 ba1
U+4BF2.0	fCantonese	paa1; baa1
U+4BF2.0	fMandarin	pa1; pa2
U+4BF2.0	fEnglish	[1] coiffure with a topknot [2] dishevelled hair
U+4BF3.0	fUTF8	䯳
U+4BF3.0	fCNS11643	3-4A74
U+4BF3.0	fR/S	 190.4
U+4BF3.0	fTotalStrokes	 14
U+4BF3.0	fCangjie	SHCI
U+4BF3.0	fFourCorner	72730
U+4BF3.0	fHakka	 sung1
U+4BF3.0	fCantonese	sung1; tou4
U+4BF3.0	fMandarin	song1
U+4BF3.0	fEnglish	same as &#x9B06;
U+4BF4.0	fUTF8	䯴
U+4BF4.0	fCNS11643	5-4B59
U+4BF4.0	fR/S	 190.4
U+4BF4.0	fTotalStrokes	 14
U+4BF4.0	fCangjie	SHHNE
U+4BF4.0	fFourCorner	72400
U+4BF4.0	fCantonese	coeng4
U+4BF4.0	fMandarin	chang2
U+4BF4.0	fEnglish	a coiffure with a topknot
U+4BF5.0	fUTF8	䯵
U+4BF5.0	fCNS11643	4-4E7B
U+4BF5.0	fGB	3-7B3A
U+4BF5.0	fR/S	 190.5
U+4BF5.0	fTotalStrokes	 15
U+4BF5.0	fCangjie	SHJT
U+4BF5.0	fFourCorner	72440
U+4BF5.0	fCantonese	ngot6; ot6 (also); zit6
U+4BF5.0	fMandarin	nie4
U+4BF5.0	fEnglish	hair style for little children
U+4BF6.0	fUTF8	䯶
U+4BF6.0	fCNS11643	3-4F7A
U+4BF6.0	fR/S	 190.5
U+4BF6.0	fTotalStrokes	 15
U+4BF6.0	fCangjie	SHBM
U+4BF6.0	fFourCorner	72100
U+4BF6.0	fCantonese	maan4; man4; mun4
U+4BF6.0	fMandarin	man2
U+4BF6.0	fEnglish	non-classical form of &#x9B18;
U+4BF7.0	fUTF8	䯷
U+4BF7.0	fCNS11643	4-5535
U+4BF7.0	fGB	3-7B40
U+4BF7.0	fR/S	 190.6
U+4BF7.0	fTotalStrokes	 16
U+4BF7.0	fCangjie	SHIJ
U+4BF7.0	fFourCorner	72450
U+4BF7.0	fHakka	 sung1; siung1
U+4BF7.0	fCantonese	sung1
U+4BF7.0	fMandarin	song1
U+4BF7.0	fEnglish	fine hair
U+4BF8.0	fUTF8	䯸
U+4BF8.0	fCNS11643	4-5536
U+4BF8.0	fGB	3-7B45
U+4BF8.0	fR/S	 190.6
U+4BF8.0	fTotalStrokes	 16
U+4BF8.0	fCangjie	SHIMO
U+4BF8.0	fFourCorner	72180
U+4BF8.0	fCantonese	ci3
U+4BF8.0	fMandarin	ci4
U+4BF8.0	fEnglish	 wig
U+4BF9.0	fUTF8	䯹
U+4BF9.0	fCNS11643	4-5A38
U+4BF9.0	fGB	5-7940
U+4BF9.0	fR/S	 190.7
U+4BF9.0	fTotalStrokes	 17; 16
U+4BF9.0	fCangjie	SHSME
U+4BF9.0	fFourCorner	72400
U+4BF9.0	fCantonese	cim1; zim4
U+4BF9.0	fMandarin	xian1
U+4BF9.0	fEnglish	hair
U+4BFA.0	fUTF8	䯺
U+4BFA.0	fCNS11643	3-573D
U+4BFA.0	fGB	5-793E
U+4BFA.0	fR/S	 190.7
U+4BFA.0	fTotalStrokes	 17
U+4BFA.0	fCangjie	SHHPR
U+4BFA.0	fFourCorner	72600
U+4BFA.0	fCantonese	kut3; kwut3 (also)
U+4BFA.0	fMandarin	kuo4
U+4BFA.0	fEnglish	standard form of &#x9AFA;
U+4BFB.0	fUTF8	䯻
U+4BFB.0	fR/S	 190.7
U+4BFB.0	fTotalStrokes	 17
U+4BFB.0	fCangjie	SHHGR
U+4BFB.0	fHakka	 gi5
U+4BFB.0	fCantonese	gou3; gai3
U+4BFB.0	fEnglish	[Korean character]
U+4BFC.0	fUTF8	䯼
U+4BFC.0	fCNS11643	4-5A3B
U+4BFC.0	fGB	3-7B48
U+4BFC.0	fR/S	 190.7
U+4BFC.0	fTotalStrokes	 16; 17
U+4BFC.0	fCangjie	SHKHF
U+4BFC.0	fFourCorner	72280
U+4BFC.0	fCantonese	dik6
U+4BFC.0	fMandarin	di2
U+4BFC.0	fEnglish	 coiffure with a topknot; wig
U+4BFD.0	fUTF8	䯽
U+4BFD.0	fCNS11643	4-5E70
U+4BFD.0	fGB	3-7B4F
U+4BFD.0	fR/S	 190.8
U+4BFD.0	fTotalStrokes	 18
U+4BFD.0	fCangjie	SHYTR
U+4BFD.0	fFourCorner	72600
U+4BFD.0	fCantonese	pau4; fu1; paau4
U+4BFD.0	fMandarin	pou3; fu3; fu1; pou2; bao3
U+4BFD.0	fEnglish	[1] fine hair; beautiful hair [2] short hair [3] a coiffure with a topknot
U+4BFE.0	fUTF8	䯾
U+4BFE.0	fCNS11643	4-5E73
U+4BFE.0	fGB	3-7B4E
U+4BFE.0	fR/S	 190.8
U+4BFE.0	fTotalStrokes	 18
U+4BFE.0	fCangjie	SHBGR
U+4BFE.0	fFourCorner	72220
U+4BFE.0	fCantonese	cau4; tiu4
U+4BFE.0	fMandarin	tiao2; zhu4; chou2; diao1
U+4BFE.0	fEnglish	[1] hairy; with lots of hair [2] let the hair grow, as children do
U+4BFF.0	fUTF8	䯿
U+4BFF.0	fCNS11643	3-596E
U+4BFF.0	fGB	5-7943
U+4BFF.0	fR/S	 190.8
U+4BFF.0	fTotalStrokes	 18
U+4BFF.0	fCangjie	SHYOJ
U+4BFF.0	fFourCorner	72400
U+4BFF.0	fCantonese	zeot1; cyut3
U+4BFF.0	fMandarin	zu2; zuo2
U+4BFF.0	fEnglish	[1] coiffure with a topknot [2] hairy [3] dishevelled hair
U+4C00.0	fUTF8	䰀
U+4C00.0	fCNS11643	4-5E74
U+4C00.0	fGB	3-7B4C
U+4C00.0	fR/S	 190.8
U+4C00.0	fTotalStrokes	 18
U+4C00.0	fCangjie	SHHDV
U+4C00.0	fFourCorner	72400
U+4C00.0	fCantonese	wo2; wo1 (also)
U+4C00.0	fMandarin	wei1; wo1; wo3
U+4C00.0	fEnglish	dressed hair of a Chinese woman
U+4C01.0	fUTF8	䰁
U+4C01.0	fCNS11643	4-5E76
U+4C01.0	fGB	3-7B51
U+4C01.0	fR/S	 190.8
U+4C01.0	fTotalStrokes	 18
U+4C01.0	fCangjie	SHVNE
U+4C01.0	fFourCorner	72130
U+4C01.0	fCantonese	fai3
U+4C01.0	fMandarin	fei4; pu2
U+4C01.0	fEnglish	 meet; run into suddenly; unexpectedly
U+4C02.0	fUTF8	䰂
U+4C02.0	fCNS11643	4-5E75
U+4C02.0	fGB	3-7B4D
U+4C02.0	fR/S	 190.8
U+4C02.0	fTotalStrokes	 18
U+4C02.0	fCangjie	SHBD
U+4C02.0	fFourCorner	72904
U+4C02.0	fHakka	  cai3;  coi3
U+4C02.0	fCantonese	coi3
U+4C02.0	fMandarin	cai3; cai4
U+4C02.0	fEnglish	[1] coiffure with a topknot [2] turban; kerchief
U+4C03.0	fUTF8	䰃
U+4C03.0	fCNS11643	4-5E77
U+4C03.0	fGB	3-7B50
U+4C03.0	fR/S	 190.8
U+4C03.0	fTotalStrokes	 18
U+4C03.0	fCangjie	SHTTC
U+4C03.0	fFourCorner	72100
U+4C03.0	fCantonese	paang4; bong2; pang4
U+4C03.0	fMandarin	peng2; bang3; bang4
U+4C03.0	fEnglish	[1] dishevelled hair [2] same as &#x78B0;
U+4C04.0	fUTF8	䰄
U+4C04.0	fCNS11643	4-624E
U+4C04.0	fGB	3-7B55
U+4C04.0	fR/S	 190.9
U+4C04.0	fTotalStrokes	 19
U+4C04.0	fCangjie	SHWP
U+4C04.0	fFourCorner	72330
U+4C04.0	fHakka	  soi1; sii1
U+4C04.0	fCantonese	soi1
U+4C04.0	fMandarin	si1; sai1; shi4
U+4C04.0	fEnglish	short hair
U+4C05.0	fUTF8	䰅
U+4C05.0	fCNS11643	3-5B72
U+4C05.0	fGB	5-7931
U+4C05.0	fR/S	 190.9
U+4C05.0	fTotalStrokes	 19
U+4C05.0	fCangjie	SHMBC
U+4C05.0	fFourCorner	72800
U+4C05.0	fCantonese	sou1
U+4C05.0	fMandarin	xu1
U+4C05.0	fEnglish	same as &#x9808; &#x9B1A;) beard; whiskers
U+4C06.0	fUTF8	䰆
U+4C06.0	fCNS11643	4-624C
U+4C06.0	fGB	3-7B59
U+4C06.0	fR/S	 190.9
U+4C06.0	fTotalStrokes	 19
U+4C06.0	fCangjie	SHNHD
U+4C06.0	fFourCorner	72900
U+4C06.0	fCantonese	jau4; zaau4
U+4C06.0	fMandarin	rou2
U+4C06.0	fEnglish	[1] beautiful and hairy mane [2] yellow hair; the hoary hair of the aged
U+4C07.0	fUTF8	䰇
U+4C07.0	fR/S	 190.9
U+4C07.0	fTotalStrokes	 19
U+4C07.0	fCangjie	SHJKA
U+4C07.0	fCantonese	kei4
U+4C07.0	fMandarin	qi2
U+4C07.0	fEnglish	same as &#x9B10;
U+4C08.0	fUTF8	䰈
U+4C08.0	fCNS11643	4-654D
U+4C08.0	fGB	3-7B56
U+4C08.0	fR/S	 190.10
U+4C08.0	fTotalStrokes	 20
U+4C08.0	fCangjie	SHTQM
U+4C08.0	fFourCorner	72210
U+4C08.0	fCantonese	co4; co1; gwaan1; zaau1
U+4C08.0	fMandarin	cuo3; cha3; cuo2
U+4C08.0	fEnglish	[1] fine hair [2] hairy
U+4C09.0	fUTF8	䰉
U+4C09.0	fCNS11643	4-654F
U+4C09.0	fGB	3-7B5F
U+4C09.0	fR/S	 190.10
U+4C09.0	fTotalStrokes	 20
U+4C09.0	fCangjie	SHHYE
U+4C09.0	fFourCorner	72447
U+4C09.0	fHakka	  pan2
U+4C09.0	fCantonese	pun4; baan1 (also); ban1; bun2; bun6
U+4C09.0	fMandarin	ban1; pan2; pan4
U+4C09.0	fEnglish	grey hair
U+4C0A.0	fUTF8	䰊
U+4C0A.0	fCNS11643	4-654E
U+4C0A.0	fGB	3-7B5D
U+4C0A.0	fR/S	 190.10
U+4C0A.0	fTotalStrokes	 20
U+4C0A.0	fCangjie	SHIBI
U+4C0A.0	fFourCorner	72340
U+4C0A.0	fCantonese	bok3
U+4C0A.0	fMandarin	bo2
U+4C0A.0	fEnglish	[1] hair [2] sparse hair
U+4C0B.0	fUTF8	䰋
U+4C0B.0	fCNS11643	7-5179
U+4C0B.0	fR/S	 190.10
U+4C0B.0	fTotalStrokes	 20
U+4C0B.0	fCangjie	SHAJV
U+4C0B.0	fFourCorner	72400
U+4C0B.0	fCantonese	faat3; man4
U+4C0B.0	fMandarin	man2
U+4C0B.0	fEnglish	same as &#x9AEE;, &#x9B18;
U+4C0C.0	fUTF8	䰌
U+4C0C.0	fCNS11643	5-723F
U+4C0C.0	fR/S	 190.11
U+4C0C.0	fTotalStrokes	 21
U+4C0C.0	fCangjie	SHHOO
U+4C0C.0	fFourCorner	72280
U+4C0C.0	fCantonese	zung2
U+4C0C.0	fMandarin	song1; zong3
U+4C0C.0	fEnglish	[1] bind the hair in a knot on the top of the head [2] mane [3] dishevelled hair
U+4C0D.0	fUTF8	䰍
U+4C0D.0	fCNS11643	3-5F26
U+4C0D.0	fR/S	 190.11
U+4C0D.0	fTotalStrokes	 21
U+4C0D.0	fCangjie	SHDOE
U+4C0D.0	fFourCorner	72130
U+4C0D.0	fCantonese	jau1; zaau1
U+4C0D.0	fMandarin	xiu1; ci4; qi1
U+4C0D.0	fEnglish	same as &#x9AE4;, &#x9AF9;
U+4C0E.0	fUTF8	䰎
U+4C0E.0	fCNS11643	4-6972
U+4C0E.0	fGB	5-7947
U+4C0E.0	fR/S	 190.12
U+4C0E.0	fTotalStrokes	 22; 21
U+4C0E.0	fCangjie	SHLMC
U+4C0E.0	fFourCorner	72800
U+4C0E.0	fHakka	 kui3; wui3; wi3; kui5; kui6; k'wui5; vui5
U+4C0E.0	fCantonese	kwai3; waai2
U+4C0E.0	fMandarin	kui4; gui4; hui3
U+4C0E.0	fEnglish	a ribbon for tying the hair in a knot
U+4C0F.0	fUTF8	䰏
U+4C0F.0	fCNS11643	4-6C43
U+4C0F.0	fGB	3-7B63
U+4C0F.0	fR/S	 190.14
U+4C0F.0	fTotalStrokes	 24
U+4C0F.0	fCangjie	SHJIG
U+4C0F.0	fFourCorner	72250
U+4C0F.0	fCantonese	zit3; zaai3
U+4C0F.0	fMandarin	ji4
U+4C0F.0	fEnglish	[1] sparse hair on a knot on the top of the head [2] hair dressed without ornaments
U+4C10.0	fUTF8	䰐
U+4C10.0	fCNS11643	4-6C41
U+4C10.0	fGB	3-7B5E
U+4C10.0	fR/S	 190.14
U+4C10.0	fTotalStrokes	 24
U+4C10.0	fCangjie	SHSIT
U+4C10.0	fFourCorner	72100
U+4C10.0	fHakka	  lam2
U+4C10.0	fCantonese	laam4
U+4C10.0	fMandarin	lan2; lan4
U+4C10.0	fEnglish	long unkempt hair
U+4C11.0	fUTF8	䰑
U+4C11.0	fCNS11643	3-612D
U+4C11.0	fR/S	 190.14
U+4C11.0	fTotalStrokes	 24
U+4C11.0	fCangjie	SHMBB
U+4C11.0	fFourCorner	72220
U+4C11.0	fCantonese	sou1; siu1
U+4C11.0	fMandarin	xu1
U+4C11.0	fEnglish	[1] non-classical form of &#x4C05; [2] same as &#x9808; &#x9B1A;
U+4C12.0	fUTF8	䰒
U+4C12.0	fCNS11643	5-7852
U+4C12.0	fR/S	 190.14
U+4C12.0	fTotalStrokes	 24
U+4C12.0	fCangjie	SHTBO
U+4C12.0	fFourCorner	72230
U+4C12.0	fHakka	 mung2
U+4C12.0	fCantonese	mung4
U+4C12.0	fMandarin	meng2
U+4C12.0	fEnglish	long flowing mane
U+4C13.0	fUTF8	䰓
U+4C13.0	fCNS11643	4-6D35
U+4C13.0	fGB	3-7B64
U+4C13.0	fR/S	 190.15
U+4C13.0	fTotalStrokes	 25
U+4C13.0	fCangjie	SHHUS
U+4C13.0	fFourCorner	72220
U+4C13.0	fCantonese	min4
U+4C13.0	fMandarin	mian2
U+4C13.0	fEnglish	[1] hair [2] burn a certain stick and then to blacken the eyebrows (in old times)
U+4C14.0	fUTF8	䰔
U+4C14.0	fCNS11643	7-616A
U+4C14.0	fR/S	 190.15
U+4C14.0	fTotalStrokes	 25
U+4C14.0	fCangjie	SHHET
U+4C14.0	fFourCorner	72100
U+4C14.0	fCantonese	pun4
U+4C14.0	fMandarin	ban1; pan2
U+4C14.0	fEnglish	non-classical form of &#x4C09;
U+4C15.0	fUTF8	䰕
U+4C15.0	fCNS11643	4-6D69
U+4C15.0	fGB	3-7B3C
U+4C15.0	fR/S	 190.16
U+4C15.0	fTotalStrokes	 26
U+4C15.0	fCangjie	SHYPT
U+4C15.0	fFourCorner	72210
U+4C15.0	fCantonese	lou4
U+4C15.0	fMandarin	lu2; luu2
U+4C15.0	fEnglish	mane
U+4C16.0	fUTF8	䰖
U+4C16.0	fCNS11643	4-6E49
U+4C16.0	fGB	3-7B65
U+4C16.0	fR/S	 190.19
U+4C16.0	fTotalStrokes	 29
U+4C16.0	fCangjie	SHHUC
U+4C16.0	fFourCorner	72800
U+4C16.0	fHakka	 zan5
U+4C16.0	fCantonese	zaan3; cyun4
U+4C16.0	fMandarin	zan3; cuan2
U+4C16.0	fEnglish	[1] fine hair; luster of hair; smooth and glossy of the hair [2] the woman's hair in a knot on the top of the head [3] hairy
U+4C17.0	fUTF8	䰗
U+4C17.0	fCNS11643	7-517D
U+4C17.0	fR/S	 191.10; .10
U+4C17.0	fTotalStrokes	 20
U+4C17.0	fCangjie	LNNRU
U+4C17.0	fFourCorner	77710
U+4C17.0	fHakka	 k'ieu1; kieu1; geu1; keu1
U+4C17.0	fCantonese	gau1
U+4C17.0	fMandarin	jiu1
U+4C17.0	fEnglish	[1] (non-classical form) [2] lot; ticket [3] draw lots
U+4C18.0	fUTF8	䰘
U+4C18.0	fCNS11643	5-7241
U+4C18.0	fGB	3-3B32
U+4C18.0	fR/S	 191.11; .11
U+4C18.0	fTotalStrokes	 21
U+4C18.0	fCangjie	LNSMH
U+4C18.0	fFourCorner	77200
U+4C18.0	fCantonese	lau4; gau2; hip3; laau4
U+4C18.0	fMandarin	liu2; li4; liao3
U+4C18.0	fEnglish	[1] kill by hanging or to strangulation [2] kill [3] lower the grade of mourning garments, when wearing the sackcloth with the edges evenly bound
U+4C19.0	fUTF8	䰙
U+4C19.0	fCNS11643	3-4A77
U+4C19.0	fR/S	 193.4
U+4C19.0	fTotalStrokes	 14
U+4C19.0	fCangjie	MBJE
U+4C19.0	fFourCorner	14240
U+4C19.0	fCantonese	ngai5; ai5 (also); gaan1; gam3; gei6; ngaai5
U+4C19.0	fMandarin	yi3
U+4C19.0	fEnglish	[1] an ancient unit of capacity with three feet and a big opening [2] a container used to wash rice
U+4C1A.0	fUTF8	䰚
U+4C1A.0	fCNS11643	3-4A76
U+4C1A.0	fR/S	 193.4
U+4C1A.0	fTotalStrokes	 14
U+4C1A.0	fCangjie	MBYK
U+4C1A.0	fFourCorner	10240
U+4C1A.0	fCantonese	man4
U+4C1A.0	fMandarin	wen2
U+4C1A.0	fEnglish	[1] the top part of the rice gruel [2] chafe; scour; rub; wipe with hand
U+4C1B.0	fUTF8	䰛
U+4C1B.0	fCNS11643	3-4D3E
U+4C1B.0	fGB	5-7957
U+4C1B.0	fR/S	 193.5; 98.5
U+4C1B.0	fTotalStrokes	 15
U+4C1B.0	fCangjie	MBMVN
U+4C1B.0	fFourCorner	11210
U+4C1B.0	fCantonese	lik6
U+4C1B.0	fMandarin	li4
U+4C1B.0	fEnglish	same as &#x9B32;
U+4C1C.0	fUTF8	䰜
U+4C1C.0	fCNS11643	3-5376
U+4C1C.0	fGB	5-3F2D
U+4C1C.0	fR/S	 193.6
U+4C1C.0	fTotalStrokes	 16
U+4C1C.0	fCangjie	NMBN
U+4C1C.0	fFourCorner	11220
U+4C1C.0	fCantonese	lik6
U+4C1C.0	fMandarin	li4
U+4C1C.0	fEnglish	ancient form of &#x9B32;
U+4C1D.0	fUTF8	䰝
U+4C1D.0	fCNS11643	7-5A4B
U+4C1D.0	fGB	3-7B6D
U+4C1D.0	fR/S	 193.12
U+4C1D.0	fTotalStrokes	 22
U+4C1D.0	fCangjie	MBCWA
U+4C1D.0	fFourCorner	18260
U+4C1D.0	fHakka	 zen5
U+4C1D.0	fCantonese	zang6
U+4C1D.0	fMandarin	zeng4
U+4C1D.0	fEnglish	[1] a large earthen pot for cooking [2] a large iron cauldron used to cook in old times
U+4C1E.0	fUTF8	䰞
U+4C1E.0	fCNS11643	3-6158
U+4C1E.0	fGB	5-7960
U+4C1E.0	fR/S	 193.15; 193.14
U+4C1E.0	fTotalStrokes	 25; 24
U+4C1E.0	fCangjie	NNMRB
U+4C1E.0	fFourCorner	17220
U+4C1E.0	fCantonese	zyu2
U+4C1E.0	fMandarin	zhu3
U+4C1E.0	fEnglish	same as &#x7151;, &#x716E;
U+4C1F.0	fUTF8	䰟
U+4C1F.0	fCNS11643	3-4A79
U+4C1F.0	fR/S	 194.4
U+4C1F.0	fTotalStrokes	 14
U+4C1F.0	fCangjie	MMII
U+4C1F.0	fFourCorner	10210
U+4C1F.0	fCantonese	wan4; waan4; waat1
U+4C1F.0	fMandarin	hun2
U+4C1F.0	fEnglish	same as &#x9B42;
U+4C20.0	fUTF8	䰠
U+4C20.0	fCNS11643	4-4F24
U+4C20.0	fGB	3-7A38
U+4C20.0	fR/S	 194.5; 207.5
U+4C20.0	fTotalStrokes	 15
U+4C20.0	fCangjie	HILWL
U+4C20.0	fFourCorner	25210
U+4C20.0	fCantonese	san1; saan1; saan4
U+4C20.0	fMandarin	shen1; shen2
U+4C20.0	fEnglish	[1] god; spirit; immortal [2] mountain deity
U+4C21.0	fUTF8	䰡
U+4C21.0	fCNS11643	4-4F25
U+4C21.0	fGB	5-7839
U+4C21.0	fR/S	 194.5
U+4C21.0	fTotalStrokes	 15
U+4C21.0	fCangjie	HIHQO
U+4C21.0	fFourCorner	25210
U+4C21.0	fCantonese	ci3
U+4C21.0	fMandarin	chi4; chi1
U+4C21.0	fEnglish	 fierce ghost; malicious spirit; evil spirits
U+4C22.0	fUTF8	䰢
U+4C22.0	fCNS11643	4-553F
U+4C22.0	fGB	3-7A3A
U+4C22.0	fR/S	 194.6
U+4C22.0	fTotalStrokes	 16
U+4C22.0	fCangjie	HIHON
U+4C22.0	fFourCorner	21210
U+4C22.0	fCantonese	hang4; haang3
U+4C22.0	fMandarin	xing4
U+4C22.0	fEnglish	[1] ghost [2] star
U+4C23.0	fUTF8	䰣
U+4C23.0	fCNS11643	3-5377
U+4C23.0	fR/S	 194.6
U+4C23.0	fTotalStrokes	 16
U+4C23.0	fCangjie	HIBKK
U+4C23.0	fFourCorner	27212
U+4C23.0	fCantonese	mong5; wong5
U+4C23.0	fMandarin	wang3
U+4C23.0	fEnglish	same as &#x9B4D;
U+4C24.0	fUTF8	䰤
U+4C24.0	fCNS11643	4-5E7E
U+4C24.0	fGB	3-7A36
U+4C24.0	fR/S	 194.8
U+4C24.0	fTotalStrokes	 18
U+4C24.0	fCangjie	DWHUI
U+4C24.0	fFourCorner	56910
U+4C24.0	fCantonese	dung1
U+4C24.0	fMandarin	dong1
U+4C24.0	fEnglish	[1] ugly [2] monster; ghost
U+4C25.0	fUTF8	䰥
U+4C25.0	fCNS11643	3-5970
U+4C25.0	fGB	5-7840
U+4C25.0	fR/S	 194.8
U+4C25.0	fTotalStrokes	 18
U+4C25.0	fCangjie	IMHUI
U+4C25.0	fFourCorner	53210
U+4C25.0	fHakka	 fet8; wet7
U+4C25.0	fCantonese	waak6
U+4C25.0	fMandarin	yu4; huo4; ji4; she4
U+4C25.0	fEnglish	the ghost of a child
U+4C26.0	fUTF8	䰦
U+4C26.0	fCNS11643	4-5E7D
U+4C26.0	fGB	3-7A41
U+4C26.0	fR/S	 194.8
U+4C26.0	fTotalStrokes	 18
U+4C26.0	fCangjie	HIHWJ
U+4C26.0	fFourCorner	26210
U+4C26.0	fCantonese	pai2; paai2; pei2
U+4C26.0	fMandarin	pi3; pi4
U+4C26.0	fEnglish	[1] ugly [2] ghost; evil spirits; bogy; goblin; demons and monsters
U+4C27.0	fUTF8	䰧
U+4C27.0	fCNS11643	4-5E7C
U+4C27.0	fGB	3-7A3F
U+4C27.0	fR/S	 194.8
U+4C27.0	fTotalStrokes	 18
U+4C27.0	fCangjie	HIYPU
U+4C27.0	fFourCorner	21210
U+4C27.0	fCantonese	fu1; bou6
U+4C27.0	fMandarin	hu1
U+4C27.0	fEnglish	[1] sinister [2] evil and shrewd
U+4C28.0	fUTF8	䰨
U+4C28.0	fCNS11643	3-5B73
U+4C28.0	fR/S	 194.9
U+4C28.0	fTotalStrokes	 19
U+4C28.0	fCangjie	HIAHU
U+4C28.0	fFourCorner	27210
U+4C28.0	fCantonese	mei6
U+4C28.0	fMandarin	mei4
U+4C28.0	fEnglish	same as &#x9B3D;, &#x9B45;
U+4C29.0	fUTF8	䰩
U+4C29.0	fCNS11643	5-6668
U+4C29.0	fGB	3-7A3C
U+4C29.0	fR/S	 194.9; 194.8
U+4C29.0	fTotalStrokes	 18; 19
U+4C29.0	fCangjie	JAHUI
U+4C29.0	fFourCorner	46610
U+4C29.0	fCantonese	ce2; ci2
U+4C29.0	fMandarin	che3; chi3
U+4C29.0	fEnglish	[1] ugly; repulsive [2] mountain ghosts
U+4C2A.0	fUTF8	䰪
U+4C2A.0	fCNS11643	3-5D65
U+4C2A.0	fR/S	 194.10
U+4C2A.0	fTotalStrokes	 20
U+4C2A.0	fCangjie	FBHUI
U+4C2A.0	fFourCorner	16210
U+4C2A.0	fCantonese	mei6
U+4C2A.0	fMandarin	mei4
U+4C2A.0	fEnglish	corrupted form of &#x9B45;
U+4C2B.0	fUTF8	䰫
U+4C2B.0	fCNS11643	4-6977
U+4C2B.0	fGB	5-783A
U+4C2B.0	fR/S	 194.12
U+4C2B.0	fTotalStrokes	 22; 21
U+4C2B.0	fCangjie	HIGGU
U+4C2B.0	fFourCorner	24210
U+4C2B.0	fCantonese	caau1
U+4C2B.0	fMandarin	chao1; chao2
U+4C2B.0	fEnglish	[1] nimble and active ghost [2] move swiftly [3] strong [4] crafty; cunning; wily; smart; clever; shrewd
U+4C2C.0	fUTF8	䰬
U+4C2C.0	fCNS11643	4-6976
U+4C2C.0	fGB	3-7A43
U+4C2C.0	fR/S	 194.12
U+4C2C.0	fTotalStrokes	 22; 21
U+4C2C.0	fCangjie	HINHB
U+4C2C.0	fFourCorner	27210
U+4C2C.0	fCantonese	gwat6; gwaat1
U+4C2C.0	fMandarin	ju2
U+4C2C.0	fEnglish	wild ghost; ghost without head; a demon
U+4C2D.0	fUTF8	䰭
U+4C2D.0	fCNS11643	7-5A4F
U+4C2D.0	fR/S	 194.12
U+4C2D.0	fTotalStrokes	 22
U+4C2D.0	fCangjie	MBHUI
U+4C2D.0	fFourCorner	16210
U+4C2D.0	fCantonese	jyu4; zyu4
U+4C2D.0	fMandarin	ru2; nou4
U+4C2D.0	fEnglish	same as &#x4C30;
U+4C2E.0	fUTF8	䰮
U+4C2E.0	fR/S	 194.13
U+4C2E.0	fTotalStrokes	 23
U+4C2E.0	fCangjie	HITGI
U+4C2E.0	fCantonese	ji6
U+4C2E.0	fEnglish	[Korean character]
U+4C2F.0	fUTF8	䰯
U+4C2F.0	fCNS11643	4-6C48
U+4C2F.0	fGB	3-7A45
U+4C2F.0	fR/S	 194.14
U+4C2F.0	fTotalStrokes	 24
U+4C2F.0	fCangjie	HIPKO
U+4C2F.0	fFourCorner	27210
U+4C2F.0	fCantonese	ji6; zi5
U+4C2F.0	fMandarin	ni3; yi4; ran2
U+4C2F.0	fEnglish	 fear; dread; be afraid of
U+4C30.0	fUTF8	䰰
U+4C30.0	fCNS11643	4-6C49
U+4C30.0	fGB	3-7A44
U+4C30.0	fR/S	 194.14
U+4C30.0	fTotalStrokes	 24
U+4C30.0	fCangjie	MBHUI
U+4C30.0	fFourCorner	16210
U+4C30.0	fCantonese	jyu4; naau6; zyu4
U+4C30.0	fMandarin	ru2
U+4C30.0	fEnglish	same as &#x4C2D;
U+4C31.0	fUTF8	䰱
U+4C31.0	fCNS11643	4-6E58
U+4C31.0	fGB	3-7A3B
U+4C31.0	fR/S	 194.24
U+4C31.0	fTotalStrokes	 34
U+4C31.0	fCangjie	MOHUI
U+4C31.0	fFourCorner	16110
U+4C31.0	fCantonese	ling4
U+4C31.0	fMandarin	ling2
U+4C31.0	fEnglish	the name of a god, a legendary spirits having the face of a man and the body of a beast
U+4C32.0	fUTF8	䰲
U+4C32.0	fCNS11643	5-3D54
U+4C32.0	fGB	3-7739
U+4C32.0	fR/S	 195.1
U+4C32.0	fTotalStrokes	 12
U+4C32.0	fCangjie	NFU
U+4C32.0	fFourCorner	22310
U+4C32.0	fHakka	 at7; jet7; ret7
U+4C32.0	fCantonese	aat3; ngaat3 (also); at3; zat3
U+4C32.0	fMandarin	ya4
U+4C32.0	fEnglish	the name of a fish (sheat)
U+4C33.0	fUTF8	䰳
U+4C33.0	fR/S	 195.2
U+4C33.0	fTotalStrokes	 13
U+4C33.0	fCangjie	NFMN
U+4C33.0	fCantonese	ding1
U+4C33.0	fEnglish	[Korean character]
U+4C34.0	fUTF8	䰴
U+4C34.0	fCNS11643	4-486D
U+4C34.0	fGB	3-773F
U+4C34.0	fR/S	 195.3
U+4C34.0	fTotalStrokes	 14
U+4C34.0	fCangjie	NFON
U+4C34.0	fFourCorner	28310
U+4C34.0	fCantonese	gat1
U+4C34.0	fMandarin	qi4; zhi4
U+4C34.0	fEnglish	[1] wafting fish [2] fish [3] cut a fish
U+4C35.0	fUTF8	䰵
U+4C35.0	fCNS11643	5-4B64
U+4C35.0	fR/S	 195.3
U+4C35.0	fTotalStrokes	 14
U+4C35.0	fCangjie	NFND
U+4C35.0	fFourCorner	27340
U+4C35.0	fCantonese	zi2; zi1
U+4C35.0	fEnglish	another name for &#x9BD4;
U+4C36.0	fUTF8	䰶
U+4C36.0	fR/S	 195.3
U+4C36.0	fTotalStrokes	 14
U+4C36.0	fCangjie	NFYV
U+4C36.0	fCantonese	mong1
U+4C36.0	fEnglish	[Korean character]
U+4C37.0	fUTF8	䰷
U+4C37.0	fCNS11643	4-4F28
U+4C37.0	fGB	3-7741
U+4C37.0	fR/S	 195.4
U+4C37.0	fTotalStrokes	 15
U+4C37.0	fCangjie	NFQJ
U+4C37.0	fFourCorner	25300
U+4C37.0	fCantonese	pong5; pang5; pau4
U+4C37.0	fMandarin	bang4; bo2
U+4C37.0	fEnglish	[1] fish [2] same as &#x868C;
U+4C38.0	fUTF8	䰸
U+4C38.0	fCNS11643	4-4F34
U+4C38.0	fGB	5-7647
U+4C38.0	fR/S	 195.4
U+4C38.0	fTotalStrokes	 15
U+4C38.0	fCangjie	NFCI
U+4C38.0	fFourCorner	28330
U+4C38.0	fHakka	 gung1
U+4C38.0	fCantonese	gung1
U+4C38.0	fMandarin	gong1
U+4C38.0	fEnglish	[1] interchangeable with &#x9B5F; [2] same as &#x9C29;
U+4C39.0	fUTF8	䰹
U+4C39.0	fCNS11643	3-5021
U+4C39.0	fR/S	 195.4
U+4C39.0	fTotalStrokes	 15
U+4C39.0	fCangjie	NFI
U+4C39.0	fFourCorner	23350
U+4C39.0	fCantonese	caak6
U+4C39.0	fMandarin	ji4; ze2
U+4C39.0	fEnglish	[1] non-classical and abbreviated form of &#x9C61 [2] same as &#x9C02;
U+4C3A.0	fUTF8	䰺
U+4C3A.0	fCNS11643	7-2B6A
U+4C3A.0	fR/S	 195.4
U+4C3A.0	fTotalStrokes	 15
U+4C3A.0	fCangjie	NFHML
U+4C3A.0	fFourCorner	22320
U+4C3A.0	fCantonese	gaai3
U+4C3A.0	fMandarin	jie4
U+4C3A.0	fEnglish	corrupted form of &#x9B6A;
U+4C3B.0	fUTF8	䰻
U+4C3B.0	fCNS11643	4-4F39
U+4C3B.0	fGB	3-7748
U+4C3B.0	fR/S	 195.4
U+4C3B.0	fTotalStrokes	 15
U+4C3B.0	fCangjie	NFOK
U+4C3B.0	fFourCorner	28340
U+4C3B.0	fHakka	  ng2
U+4C3B.0	fCantonese	zyu4
U+4C3B.0	fMandarin	yu2
U+4C3B.0	fEnglish	 fish
U+4C3C.0	fUTF8	䰼
U+4C3C.0	fCNS11643	4-4F32
U+4C3C.0	fGB	3-774C
U+4C3C.0	fR/S	 195.4
U+4C3C.0	fTotalStrokes	 15
U+4C3C.0	fCangjie	NFOIN
U+4C3C.0	fFourCorner	28320
U+4C3C.0	fCantonese	cam4
U+4C3C.0	fMandarin	jin1; cen2; xin2; shen3; yin2
U+4C3C.0	fEnglish	[1] a condiment made from minced fish salted [2] preserved fish; salted fish; fish cured in distiller's grains [3] a fish
U+4C3D.0	fUTF8	䰽
U+4C3D.0	fCNS11643	4-4F2B
U+4C3D.0	fGB	3-7743
U+4C3D.0	fR/S	 195.4
U+4C3D.0	fTotalStrokes	 15
U+4C3D.0	fCangjie	NFJB
U+4C3D.0	fFourCorner	20320
U+4C3D.0	fCantonese	pui3; bui3
U+4C3D.0	fMandarin	bei4
U+4C3D.0	fEnglish	the globefish; flowfish; puffer
U+4C3E.0	fUTF8	䰾
U+4C3E.0	fCNS11643	4-4F29
U+4C3E.0	fGB	3-7750
U+4C3E.0	fR/S	 195.4
U+4C3E.0	fTotalStrokes	 15
U+4C3E.0	fCangjie	NFAU
U+4C3E.0	fFourCorner	27310
U+4C3E.0	fCantonese	baa1; bat6
U+4C3E.0	fMandarin	ba4
U+4C3E.0	fEnglish	[1] a kind of fish; a bonito [2] same as &#x9B81;
U+4C3F.0	fUTF8	䰿
U+4C3F.0	fCNS11643	3-5427
U+4C3F.0	fR/S	 195.5
U+4C3F.0	fTotalStrokes	 16
U+4C3F.0	fCangjie	NFOPD
U+4C3F.0	fFourCorner	28310
U+4C3F.0	fCantonese	to4
U+4C3F.0	fMandarin	tuo2
U+4C3F.0	fEnglish	non-classical form of &#x9B80;
U+4C40.0	fUTF8	䱀
U+4C40.0	fCNS11643	4-5544
U+4C40.0	fGB	3-775A
U+4C40.0	fR/S	 195.5
U+4C40.0	fTotalStrokes	 16
U+4C40.0	fCangjie	NFLBK
U+4C40.0	fFourCorner	25330
U+4C40.0	fHakka	 jong1; rong1
U+4C40.0	fCantonese	joeng1; ong3; zoeng1
U+4C40.0	fMandarin	yang1
U+4C40.0	fEnglish	a kind of small fish
U+4C41.0	fUTF8	䱁
U+4C41.0	fCNS11643	4-5542
U+4C41.0	fGB	3-7755
U+4C41.0	fR/S	 195.5
U+4C41.0	fTotalStrokes	 16
U+4C41.0	fCangjie	NFDM
U+4C41.0	fFourCorner	24340
U+4C41.0	fCantonese	giu4; kiu4
U+4C41.0	fMandarin	qiao2
U+4C41.0	fEnglish	a kind of medium sized fish grown in the sea, long and flat, with a big mouth
U+4C42.0	fUTF8	䱂
U+4C42.0	fCNS11643	4-5548
U+4C42.0	fGB	3-7767
U+4C42.0	fR/S	 195.5
U+4C42.0	fTotalStrokes	 16
U+4C42.0	fCangjie	NFVIS
U+4C42.0	fFourCorner	24320
U+4C42.0	fHakka	 ju3; riu6; ju5; riu3
U+4C42.0	fCantonese	jau1; zaau2
U+4C42.0	fMandarin	you4; you3; you1; yao1
U+4C42.0	fEnglish	a small yellowish-black fish
U+4C43.0	fUTF8	䱃
U+4C43.0	fCNS11643	3-537E
U+4C43.0	fR/S	 195.5
U+4C43.0	fTotalStrokes	 16
U+4C43.0	fCangjie	NFHQO
U+4C43.0	fFourCorner	25330
U+4C43.0	fCantonese	zat6; dit6; zaat6
U+4C43.0	fMandarin	zhi4; di3
U+4C43.0	fEnglish	a fish
U+4C44.0	fUTF8	䱄
U+4C44.0	fCNS11643	3-5428
U+4C44.0	fR/S	 195.5
U+4C44.0	fTotalStrokes	 16
U+4C44.0	fCangjie	NFHVO
U+4C44.0	fFourCorner	22310
U+4C44.0	fHakka	 gu1
U+4C44.0	fCantonese	gaai3
U+4C44.0	fMandarin	jie4
U+4C44.0	fEnglish	non-classical and corrupted form of &#x9B6A;
U+4C45.0	fUTF8	䱅
U+4C45.0	fCNS11643	4-5541
U+4C45.0	fGB	3-7753
U+4C45.0	fR/S	 195.5
U+4C45.0	fTotalStrokes	 16
U+4C45.0	fCangjie	NFDJ
U+4C45.0	fFourCorner	25390
U+4C45.0	fCantonese	mut6
U+4C45.0	fMandarin	mo4; mei4
U+4C45.0	fEnglish	same as &#x9C74;, &#x9B86;
U+4C46.0	fUTF8	䱆
U+4C46.0	fCNS11643	3-5426
U+4C46.0	fR/S	 195.5
U+4C46.0	fTotalStrokes	 16
U+4C46.0	fCangjie	NFNSD
U+4C46.0	fFourCorner	27340
U+4C46.0	fCantonese	jing6; sing4
U+4C46.0	fMandarin	sheng2
U+4C46.0	fEnglish	same as &#x9C66;
U+4C47.0	fUTF8	䱇
U+4C47.0	fCNS11643	3-5424
U+4C47.0	fGB	5-764E
U+4C47.0	fR/S	 195.5
U+4C47.0	fTotalStrokes	 16
U+4C47.0	fCangjie	NFAM
U+4C47.0	fFourCorner	26310
U+4C47.0	fHakka	 shan5; san5; sen5; shen5
U+4C47.0	fCantonese	sin5
U+4C47.0	fMandarin	shan4; tao2
U+4C47.0	fEnglish	an eel
U+4C48.0	fUTF8	䱈
U+4C48.0	fCNS11643	3-5422
U+4C48.0	fR/S	 195.5
U+4C48.0	fTotalStrokes	 16
U+4C48.0	fCangjie	NFMMF
U+4C48.0	fFourCorner	21390
U+4C48.0	fCantonese	ngai6; ai6 (also); kei4; ngaai6
U+4C48.0	fMandarin	qi2; yi4
U+4C48.0	fEnglish	[1] ancient form of &#x9BA8; [2] same as &#x9C2D;
U+4C49.0	fUTF8	䱉
U+4C49.0	fCNS11643	3-537B
U+4C49.0	fR/S	 195.5
U+4C49.0	fTotalStrokes	 16
U+4C49.0	fCangjie	NFBM
U+4C49.0	fFourCorner	27310
U+4C49.0	fCantonese	sin5
U+4C49.0	fMandarin	zu1; shan4
U+4C49.0	fEnglish	[1] corrupted form of &#x4C47 [2] same as &#x9C53;, &#x9C54;
U+4C4A.0	fUTF8	䱊
U+4C4A.0	fCNS11643	4-5A40
U+4C4A.0	fGB	3-7822
U+4C4A.0	fR/S	 195.6
U+4C4A.0	fTotalStrokes	 17; 16
U+4C4A.0	fCangjie	NFFD
U+4C4A.0	fFourCorner	29390
U+4C4A.0	fHakka	 mi3
U+4C4A.0	fCantonese	mai5
U+4C4A.0	fMandarin	mi3
U+4C4A.0	fEnglish	fish-spawn
U+4C4B.0	fUTF8	䱋
U+4C4B.0	fCNS11643	3-5741
U+4C4B.0	fGB	5-7651
U+4C4B.0	fR/S	 195.6
U+4C4B.0	fTotalStrokes	 17
U+4C4B.0	fCangjie	NFTC
U+4C4B.0	fFourCorner	24380
U+4C4B.0	fCantonese	gung2; daam2
U+4C4B.0	fMandarin	gong3; dan3
U+4C4B.0	fEnglish	[1] spawn; roe [2] a kind of legendary fish said to be thousands of miles long [3] interchangeable with &#x9B5F;
U+4C4C.0	fUTF8	䱌
U+4C4C.0	fCNS11643	4-5F21
U+4C4C.0	fGB	3-7825
U+4C4C.0	fR/S	 195.6; 195.7
U+4C4C.0	fTotalStrokes	 18; 17
U+4C4C.0	fCangjie	NFSLL
U+4C4C.0	fFourCorner	21310
U+4C4C.0	fHakka	 hi1; ri2; ji2
U+4C4C.0	fCantonese	ji4; zi4
U+4C4C.0	fMandarin	yi2
U+4C4C.0	fEnglish	the fresh-water porpoise
U+4C4D.0	fUTF8	䱍
U+4C4D.0	fCNS11643	7-3E49
U+4C4D.0	fR/S	 195.6
U+4C4D.0	fTotalStrokes	 17
U+4C4D.0	fCangjie	NFMBM
U+4C4D.0	fFourCorner	21310
U+4C4D.0	fCantonese	gang2
U+4C4D.0	fMandarin	geng4
U+4C4D.0	fEnglish	tuna, something like sturgeon
U+4C4E.0	fUTF8	䱎
U+4C4E.0	fCNS11643	4-5A42
U+4C4E.0	fGB	3-7769
U+4C4E.0	fR/S	 195.6
U+4C4E.0	fTotalStrokes	 16; 17
U+4C4E.0	fCangjie	NFMAM
U+4C4E.0	fFourCorner	21310
U+4C4E.0	fCantonese	gang2
U+4C4E.0	fMandarin	geng4
U+4C4E.0	fEnglish	same as &#x4C4D;
U+4C4F.0	fUTF8	䱏
U+4C4F.0	fCNS11643	3-5972
U+4C4F.0	fR/S	 195.7
U+4C4F.0	fTotalStrokes	 18
U+4C4F.0	fCangjie	NFMRT
U+4C4F.0	fFourCorner	21310
U+4C4F.0	fCantonese	tau2; daau6; taau2
U+4C4F.0	fMandarin	dou4; tou3
U+4C4F.0	fEnglish	a fish
U+4C50.0	fUTF8	䱐
U+4C50.0	fCNS11643	4-5F23
U+4C50.0	fGB	3-782C
U+4C50.0	fR/S	 195.7
U+4C50.0	fTotalStrokes	 18
U+4C50.0	fCangjie	NFBND
U+4C50.0	fFourCorner	22340
U+4C50.0	fCantonese	fu1; tou1
U+4C50.0	fMandarin	fu2; pou2; fou2
U+4C50.0	fEnglish	[1] a kind of black fish [2] another name for the globefish; blowfish; puffer
U+4C51.0	fUTF8	䱑
U+4C51.0	fCNS11643	5-666E
U+4C51.0	fR/S	 195.7
U+4C51.0	fTotalStrokes	 18
U+4C51.0	fCangjie	NFQHL
U+4C51.0	fFourCorner	22320
U+4C51.0	fCantonese	zyut6; haap6; sit3
U+4C51.0	fMandarin	xie4; xue2
U+4C51.0	fEnglish	a fish, something like crab grown in the sea
U+4C52.0	fUTF8	䱒
U+4C52.0	fCNS11643	4-5F27
U+4C52.0	fGB	5-765C
U+4C52.0	fR/S	 195.7
U+4C52.0	fTotalStrokes	 18
U+4C52.0	fCangjie	NFRAU
U+4C52.0	fFourCorner	26310
U+4C52.0	fHakka	 jip7; rip7
U+4C52.0	fCantonese	jip3; zaap1
U+4C52.0	fMandarin	yi4
U+4C52.0	fEnglish	[1] the name of a fish [2] salt
U+4C53.0	fUTF8	䱓
U+4C53.0	fCNS11643	4-5F25
U+4C53.0	fGB	3-7775
U+4C53.0	fR/S	 195.7
U+4C53.0	fTotalStrokes	 18
U+4C53.0	fCangjie	NFNKG
U+4C53.0	fFourCorner	22340
U+4C53.0	fCantonese	ting4
U+4C53.0	fMandarin	ting2; ting3
U+4C53.0	fEnglish	a kind of fish, something like the sheat-fish
U+4C54.0	fUTF8	䱔
U+4C54.0	fCNS11643	3-5978
U+4C54.0	fGB	5-765E
U+4C54.0	fR/S	 195.7
U+4C54.0	fTotalStrokes	 18
U+4C54.0	fCangjie	NFOLK
U+4C54.0	fFourCorner	28340
U+4C54.0	fCantonese	tiu4
U+4C54.0	fMandarin	tiao2
U+4C54.0	fEnglish	same as &#x9C37;
U+4C55.0	fUTF8	䱕
U+4C55.0	fCNS11643	4-5F24
U+4C55.0	fGB	3-782A
U+4C55.0	fR/S	 195.7
U+4C55.0	fTotalStrokes	 18
U+4C55.0	fCangjie	NFOWY
U+4C55.0	fFourCorner	28350
U+4C55.0	fCantonese	mau4
U+4C55.0	fMandarin	mou2; mei2
U+4C55.0	fEnglish	[1] labrus [2] moving fish
U+4C56.0	fUTF8	䱖
U+4C56.0	fCNS11643	3-5977
U+4C56.0	fR/S	 195.7
U+4C56.0	fTotalStrokes	 18
U+4C56.0	fCangjie	NFMLS
U+4C56.0	fFourCorner	21370
U+4C56.0	fCantonese	lau4; laau4; laau6; liu1
U+4C56.0	fMandarin	liu2; liu3
U+4C56.0	fEnglish	same as &#x9C21;
U+4C57.0	fUTF8	䱗
U+4C57.0	fCNS11643	4-5F29
U+4C57.0	fGB	3-7827
U+4C57.0	fR/S	 195.7
U+4C57.0	fTotalStrokes	 18
U+4C57.0	fCangjie	YENWF
U+4C57.0	fFourCorner	27336
U+4C57.0	fHakka	 con1
U+4C57.0	fCantonese	caan1
U+4C57.0	fMandarin	can1
U+4C57.0	fEnglish	a long thin fish (Trichurus armatus)
U+4C58.0	fUTF8	䱘
U+4C58.0	fCNS11643	4-5F2B
U+4C58.0	fGB	3-7829
U+4C58.0	fR/S	 195.7
U+4C58.0	fTotalStrokes	 18
U+4C58.0	fCangjie	HNNWF
U+4C58.0	fFourCorner	22330
U+4C58.0	fCantonese	lei4
U+4C58.0	fMandarin	li2
U+4C59.0	fUTF8	䱙
U+4C59.0	fCNS11643	4-625D
U+4C59.0	fGB	3-7837
U+4C59.0	fR/S	 195.8
U+4C59.0	fTotalStrokes	 19
U+4C59.0	fCangjie	NFYFE
U+4C59.0	fFourCorner	27340
U+4C59.0	fCantonese	suk1
U+4C59.0	fEnglish	same as &#x9B9B;
U+4C5A.0	fUTF8	䱚
U+4C5A.0	fCNS11643	5-6B35
U+4C5A.0	fGB	3-783F
U+4C5A.0	fR/S	 195.8
U+4C5A.0	fTotalStrokes	 19
U+4C5A.0	fCangjie	NFVNE
U+4C5A.0	fFourCorner	27330
U+4C5A.0	fCantonese	luk6
U+4C5A.0	fMandarin	lu4
U+4C5A.0	fEnglish	a kind of fish
U+4C5B.0	fUTF8	䱛
U+4C5B.0	fCNS11643	4-6267
U+4C5B.0	fGB	3-7835
U+4C5B.0	fR/S	 195.8
U+4C5B.0	fTotalStrokes	 19
U+4C5B.0	fCangjie	NFIRM
U+4C5B.0	fFourCorner	23350
U+4C5B.0	fHakka	 fet8; wet7
U+4C5B.0	fCantonese	kwik1; gwik1 (also)
U+4C5B.0	fMandarin	xu4
U+4C5B.0	fEnglish	a kind of lizard which frequents the bamboo
U+4C5C.0	fUTF8	䱜
U+4C5C.0	fCNS11643	4-6257
U+4C5C.0	fGB	3-7834
U+4C5C.0	fR/S	 195.8
U+4C5C.0	fTotalStrokes	 19
U+4C5C.0	fCangjie	NFTA
U+4C5C.0	fFourCorner	24360
U+4C5C.0	fHakka	 siak7;  cok7
U+4C5C.0	fCantonese	cok3
U+4C5C.0	fMandarin	qi4; cuo4; que4
U+4C5C.0	fEnglish	a species of shark allied to the saw-fish
U+4C5D.0	fUTF8	䱝
U+4C5D.0	fCNS11643	4-6261
U+4C5D.0	fGB	3-783A
U+4C5D.0	fR/S	 195.8
U+4C5D.0	fTotalStrokes	 19
U+4C5D.0	fCangjie	NFHWJ
U+4C5D.0	fFourCorner	26340
U+4C5D.0	fCantonese	paai4
U+4C5D.0	fMandarin	pai2; ba4
U+4C5D.0	fEnglish	black carp
U+4C5E.0	fUTF8	䱞
U+4C5E.0	fCNS11643	4-6259
U+4C5E.0	fGB	3-7836
U+4C5E.0	fR/S	 195.8
U+4C5E.0	fTotalStrokes	 19
U+4C5E.0	fCangjie	NFKMF
U+4C5E.0	fFourCorner	24390
U+4C5E.0	fCantonese	noi6; loi6 (also)
U+4C5E.0	fMandarin	nai4; liu2; wei3
U+4C5E.0	fEnglish	a kind of fish
U+4C5F.0	fUTF8	䱟
U+4C5F.0	fCNS11643	4-6254
U+4C5F.0	fGB	3-7840
U+4C5F.0	fR/S	 195.8
U+4C5F.0	fTotalStrokes	 19
U+4C5F.0	fCangjie	NFSJR
U+4C5F.0	fFourCorner	27360
U+4C5F.0	fCantonese	geoi3; goeng3
U+4C5F.0	fMandarin	ju4; ju1
U+4C5F.0	fEnglish	[1] (an ancient name) [2] kind of silver fish [3] a kind of insect
U+4C60.0	fUTF8	䱠
U+4C60.0	fCNS11643	3-5B76
U+4C60.0	fR/S	 195.8
U+4C60.0	fTotalStrokes	 19
U+4C60.0	fCangjie	NFII
U+4C60.0	fFourCorner	23350
U+4C60.0	fCantonese	zaan6
U+4C60.0	fMandarin	chan3; zhan4
U+4C60.0	fEnglish	a kind of fish
U+4C61.0	fUTF8	䱡
U+4C61.0	fCNS11643	5-6B31
U+4C61.0	fR/S	 195.8
U+4C61.0	fTotalStrokes	 19
U+4C61.0	fCangjie	NFPFD
U+4C61.0	fFourCorner	27320
U+4C61.0	fCantonese	guk1
U+4C61.0	fMandarin	ju2; qu1
U+4C61.0	fEnglish	the globefish; blowfish; puffer
U+4C62.0	fUTF8	䱢
U+4C62.0	fCNS11643	4-6262
U+4C62.0	fGB	3-777E
U+4C62.0	fR/S	 195.8
U+4C62.0	fTotalStrokes	 19
U+4C62.0	fCangjie	NFNSD
U+4C62.0	fFourCorner	22350
U+4C62.0	fCantonese	zang1
U+4C62.0	fMandarin	zheng1
U+4C62.0	fEnglish	a kind of fish
U+4C63.0	fUTF8	䱣
U+4C63.0	fCNS11643	5-6B2B
U+4C63.0	fR/S	 195.8
U+4C63.0	fTotalStrokes	 19
U+4C63.0	fCangjie	NFYOJ
U+4C63.0	fFourCorner	20340
U+4C63.0	fCantonese	zeot1; cyut3
U+4C63.0	fMandarin	zu2
U+4C63.0	fEnglish	[1] a long narrow fish [2] another name for tuna
U+4C64.0	fUTF8	䱤
U+4C64.0	fCNS11643	4-6260
U+4C64.0	fGB	3-783B
U+4C64.0	fR/S	 195.8
U+4C64.0	fTotalStrokes	 19
U+4C64.0	fCangjie	NFNHX
U+4C64.0	fFourCorner	27370
U+4C64.0	fCantonese	haam6
U+4C64.0	fMandarin	xian4; jian1
U+4C64.0	fEnglish	another name for &#x9C64;
U+4C65.0	fUTF8	䱥
U+4C65.0	fCNS11643	4-6264
U+4C65.0	fGB	3-7838
U+4C65.0	fR/S	 195.8
U+4C65.0	fTotalStrokes	 19
U+4C65.0	fCangjie	HNNWF
U+4C65.0	fFourCorner	22330
U+4C65.0	fHakka	 zhi5; zih5
U+4C65.0	fCantonese	zai3; dai3; zaai3
U+4C65.0	fMandarin	zhi4; zhi2
U+4C65.0	fEnglish	a name of a fish
U+4C66.0	fUTF8	䱦
U+4C66.0	fCNS11643	3-5B78
U+4C66.0	fR/S	 195.8
U+4C66.0	fTotalStrokes	 19
U+4C66.0	fCangjie	NFOG
U+4C66.0	fFourCorner	20310
U+4C66.0	fCantonese	zeoi1
U+4C66.0	fEnglish	the globefish; blowfish; puffer
U+4C67.0	fUTF8	䱧
U+4C67.0	fCNS11643	3-5B7B
U+4C67.0	fR/S	 195.8
U+4C67.0	fTotalStrokes	 19
U+4C67.0	fCangjie	NFFQU
U+4C67.0	fFourCorner	29310
U+4C67.0	fCantonese	gyun2
U+4C67.0	fMandarin	shan4
U+4C67.0	fEnglish	[1] a kind of fish [2] same as &#x9C54;
U+4C68.0	fUTF8	䱨
U+4C68.0	fCNS11643	3-5B7C
U+4C68.0	fR/S	 195.8
U+4C68.0	fTotalStrokes	 19
U+4C68.0	fCangjie	NFDBN
U+4C68.0	fFourCorner	22300
U+4C68.0	fCantonese	ci3; cit3; zaai3
U+4C68.0	fMandarin	zhi4
U+4C68.0	fEnglish	same as &#x4C65;
U+4C69.0	fUTF8	䱩
U+4C69.0	fCNS11643	F-6472
U+4C69.0	fR/S	 195.8
U+4C69.0	fTotalStrokes	 19
U+4C69.0	fCangjie	NFBTV
U+4C69.0	fCantonese	mong5
U+4C6A.0	fUTF8	䱪
U+4C6A.0	fR/S	 195.8
U+4C6A.0	fTotalStrokes	 19
U+4C6A.0	fCangjie	NFQMV
U+4C6A.0	fCantonese	biu2
U+4C6A.0	fEnglish	[Korean character]
U+4C6B.0	fUTF8	䱫
U+4C6B.0	fCNS11643	3-5D77
U+4C6B.0	fR/S	 195.9
U+4C6B.0	fTotalStrokes	 20
U+4C6B.0	fCangjie	DNNWF
U+4C6B.0	fFourCorner	52330
U+4C6B.0	fCantonese	laat6
U+4C6B.0	fMandarin	la4
U+4C6B.0	fEnglish	same as &#x9BFB;
U+4C6C.0	fUTF8	䱬
U+4C6C.0	fCNS11643	4-6555
U+4C6C.0	fGB	3-7858
U+4C6C.0	fR/S	 195.9
U+4C6C.0	fTotalStrokes	 20
U+4C6C.0	fCangjie	NFNOB
U+4C6C.0	fFourCorner	27320
U+4C6C.0	fCantonese	seoi1
U+4C6C.0	fMandarin	xu4
U+4C6C.0	fEnglish	a kind of fish
U+4C6D.0	fUTF8	䱭
U+4C6D.0	fCNS11643	7-5231
U+4C6D.0	fR/S	 195.9
U+4C6D.0	fTotalStrokes	 20
U+4C6D.0	fCangjie	NFPMM
U+4C6D.0	fFourCorner	21310
U+4C6D.0	fCantonese	gang2
U+4C6D.0	fMandarin	geng1; geng4
U+4C6D.0	fEnglish	tuna
U+4C6E.0	fUTF8	䱮
U+4C6E.0	fCNS11643	4-655B
U+4C6E.0	fGB	3-7842
U+4C6E.0	fR/S	 195.9
U+4C6E.0	fTotalStrokes	 20
U+4C6E.0	fCangjie	NFQHK
U+4C6E.0	fFourCorner	27330
U+4C6E.0	fCantonese	ngaak6; aak6 (also); ngak6
U+4C6E.0	fMandarin	e4; e2
U+4C6E.0	fEnglish	a kind of fish, the skin for making arms or weapons
U+4C6F.0	fUTF8	䱯
U+4C6F.0	fCNS11643	3-5D6C
U+4C6F.0	fR/S	 195.9
U+4C6F.0	fTotalStrokes	 20
U+4C6F.0	fCangjie	NKNWF
U+4C6F.0	fFourCorner	18330
U+4C6F.0	fCantonese	mou4
U+4C6F.0	fMandarin	mu2
U+4C6F.0	fEnglish	(corrupted form) a kind of fish
U+4C70.0	fUTF8	䱰
U+4C70.0	fCNS11643	7-5233
U+4C70.0	fR/S	 195.9
U+4C70.0	fTotalStrokes	 20
U+4C70.0	fCangjie	NFHJG
U+4C70.0	fFourCorner	22310
U+4C70.0	fCantonese	zung6; ceoi4; cung5
U+4C70.0	fMandarin	tong2; zhong4
U+4C70.0	fEnglish	same as &#x9BA6;, &#x9BC9;, &#x9C67;
U+4C71.0	fUTF8	䱱
U+4C71.0	fCNS11643	4-6553
U+4C71.0	fGB	3-7853
U+4C71.0	fR/S	 195.9
U+4C71.0	fTotalStrokes	 20
U+4C71.0	fCangjie	NFYBB
U+4C71.0	fFourCorner	20320
U+4C71.0	fHakka	 ti2
U+4C71.0	fCantonese	tai4
U+4C71.0	fMandarin	di4; ti2
U+4C71.0	fEnglish	 newt; eft
U+4C72.0	fUTF8	䱲
U+4C72.0	fCNS11643	3-5D6B
U+4C72.0	fR/S	 195.9
U+4C72.0	fTotalStrokes	 20
U+4C72.0	fCangjie	NFVNO
U+4C72.0	fFourCorner	27330
U+4C72.0	fCantonese	jyun4; zyun4
U+4C72.0	fMandarin	yan2
U+4C72.0	fEnglish	a kind of fish
U+4C73.0	fUTF8	䱳
U+4C73.0	fCNS11643	3-5D6D
U+4C73.0	fR/S	 195.9
U+4C73.0	fTotalStrokes	 20
U+4C73.0	fCangjie	NFMWG
U+4C73.0	fFourCorner	21310
U+4C73.0	fCantonese	zin1
U+4C73.0	fMandarin	zhan1
U+4C73.0	fEnglish	[1] same as &#x9C63; [2] interchangeable &#x9C54;
U+4C74.0	fUTF8	䱴
U+4C74.0	fR/S	 195.9
U+4C74.0	fTotalStrokes	 20
U+4C74.0	fCangjie	NFPMM
U+4C74.0	fCantonese	gang2; gaang2; gaang3
U+4C74.0	fMandarin	geng4
U+4C74.0	fEnglish	same as &#x4C6D;
U+4C75.0	fUTF8	䱵
U+4C75.0	fCNS11643	4-683D
U+4C75.0	fGB	3-785C
U+4C75.0	fR/S	 195.10
U+4C75.0	fTotalStrokes	 21
U+4C75.0	fCangjie	NFCIM
U+4C75.0	fFourCorner	28320
U+4C75.0	fCantonese	jung1; zung1; zung2
U+4C75.0	fMandarin	weng1
U+4C75.0	fEnglish	a kind of fish, living in the ocean
U+4C76.0	fUTF8	䱶
U+4C76.0	fCNS11643	3-5F2D
U+4C76.0	fR/S	 195.10
U+4C76.0	fTotalStrokes	 21
U+4C76.0	fCangjie	NFIVL
U+4C76.0	fFourCorner	27320
U+4C76.0	fCantonese	long4
U+4C76.0	fMandarin	lang2
U+4C76.0	fEnglish	[1] male crab [2] cod oil
U+4C77.0	fUTF8	䱷
U+4C77.0	fCNS11643	3-5F30
U+4C77.0	fGB	5-4948
U+4C77.0	fR/S	 195.10
U+4C77.0	fTotalStrokes	 21
U+4C77.0	fCangjie	YFOK
U+4C77.0	fFourCorner	28240
U+4C77.0	fHakka	 ng2
U+4C77.0	fCantonese	jyu4; zyu4
U+4C77.0	fMandarin	yu2
U+4C77.0	fEnglish	 fish; seize on
U+4C78.0	fUTF8	䱸
U+4C78.0	fCNS11643	4-683C
U+4C78.0	fGB	3-784F
U+4C78.0	fR/S	 195.10
U+4C78.0	fTotalStrokes	 21
U+4C78.0	fCangjie	NFHXE
U+4C78.0	fFourCorner	27340
U+4C78.0	fCantonese	sau1; caau1; saau1; saau2
U+4C78.0	fMandarin	sou1
U+4C78.0	fEnglish	[1] a family name [2] same as &#x7DE7;
U+4C79.0	fUTF8	䱹
U+4C79.0	fCNS11643	3-5F2E
U+4C79.0	fGB	5-7671
U+4C79.0	fR/S	 195.10
U+4C79.0	fTotalStrokes	 21
U+4C79.0	fCangjie	NFTQM
U+4C79.0	fFourCorner	28310
U+4C79.0	fCantonese	zaa2; caa4
U+4C79.0	fMandarin	zha3; na4
U+4C79.0	fEnglish	(standard form) same as &#x9B93;
U+4C7A.0	fUTF8	䱺
U+4C7A.0	fCNS11643	3-5F2B
U+4C7A.0	fR/S	 195.10
U+4C7A.0	fTotalStrokes	 21
U+4C7A.0	fCangjie	NFUMT
U+4C7A.0	fFourCorner	22310
U+4C7A.0	fCantonese	haai4
U+4C7A.0	fMandarin	hai2
U+4C7A.0	fEnglish	 male crab
U+4C7B.0	fUTF8	䱻
U+4C7B.0	fCNS11643	4-683A
U+4C7B.0	fGB	3-784D
U+4C7B.0	fR/S	 195.10
U+4C7B.0	fTotalStrokes	 21
U+4C7B.0	fCangjie	NFBBB
U+4C7B.0	fFourCorner	27320
U+4C7B.0	fHakka	 wat8; gut7; vat8
U+4C7B.0	fCantonese	waat6; gwaat1; waa4; wat6
U+4C7B.0	fMandarin	gu3; hua2
U+4C7B.0	fEnglish	 winged snake
U+4C7C.0	fUTF8	䱼
U+4C7C.0	fCNS11643	3-5F29
U+4C7C.0	fR/S	 195.10
U+4C7C.0	fTotalStrokes	 21
U+4C7C.0	fCangjie	NFSTV
U+4C7C.0	fFourCorner	27330
U+4C7C.0	fCantonese	zin2
U+4C7C.0	fMandarin	zhan3
U+4C7C.0	fEnglish	a kind of fish, seven inchs long, good for soup
U+4C7D.0	fUTF8	䱽
U+4C7D.0	fCNS11643	3-5F2F
U+4C7D.0	fR/S	 195.10
U+4C7D.0	fTotalStrokes	 21
U+4C7D.0	fCangjie	NFOIR
U+4C7D.0	fFourCorner	28360
U+4C7D.0	fHakka	  ciong1
U+4C7D.0	fCantonese	cong1; coeng1
U+4C7D.0	fEnglish	the pomfret
U+4C7E.0	fUTF8	䱾
U+4C7E.0	fCNS11643	4-697D
U+4C7E.0	fGB	3-7854
U+4C7E.0	fR/S	 195.11
U+4C7E.0	fTotalStrokes	 22; 21
U+4C7E.0	fCangjie	NFLLV
U+4C7E.0	fFourCorner	25340
U+4C7E.0	fCantonese	lau4; laau1; laau4
U+4C7E.0	fMandarin	lou2; yu2
U+4C7E.0	fEnglish	a carp-like savory fish
U+4C7F.0	fUTF8	䱿
U+4C7F.0	fCNS11643	3-5F7D
U+4C7F.0	fR/S	 195.11
U+4C7F.0	fTotalStrokes	 22
U+4C7F.0	fCangjie	NFJJL
U+4C7F.0	fFourCorner	22320
U+4C7F.0	fCantonese	caam3
U+4C7F.0	fMandarin	chan4
U+4C7F.0	fEnglish	a kind of fish
U+4C80.0	fUTF8	䲀
U+4C80.0	fCNS11643	4-6A21
U+4C80.0	fGB	3-776C
U+4C80.0	fR/S	 195.11
U+4C80.0	fTotalStrokes	 22; 21
U+4C80.0	fCangjie	GINWF
U+4C80.0	fFourCorner	44330
U+4C80.0	fCantonese	zi3
U+4C80.0	fMandarin	zhi4; die2; sui4; zou4
U+4C80.0	fEnglish	a kind of fish
U+4C81.0	fUTF8	䲁
U+4C81.0	fCNS11643	4-697C
U+4C81.0	fGB	3-7866
U+4C81.0	fR/S	 195.11
U+4C81.0	fTotalStrokes	 22; 21
U+4C81.0	fCangjie	NFSFI
U+4C81.0	fFourCorner	24300
U+4C81.0	fCantonese	wai3; to5; waai3
U+4C81.0	fMandarin	wei4
U+4C81.0	fEnglish	general term of a kind of fish, a snake-like fish
U+4C82.0	fUTF8	䲂
U+4C82.0	fCNS11643	3-5F7C
U+4C82.0	fR/S	 195.11
U+4C82.0	fTotalStrokes	 22
U+4C82.0	fCangjie	NFYSO
U+4C82.0	fFourCorner	28380
U+4C82.0	fCantonese	syun4
U+4C82.0	fMandarin	xuan2
U+4C82.0	fEnglish	a kind of fish
U+4C83.0	fUTF8	䲃
U+4C83.0	fCNS11643	4-697E
U+4C83.0	fGB	3-7869
U+4C83.0	fR/S	 195.11
U+4C83.0	fTotalStrokes	 22; 21
U+4C83.0	fCangjie	NFVVD
U+4C83.0	fFourCorner	22390
U+4C83.0	fCantonese	zou2
U+4C83.0	fMandarin	chao2; zao3
U+4C83.0	fEnglish	[1] a carp-like fish with chicken feet [2] spawn or roe
U+4C84.0	fUTF8	䲄
U+4C84.0	fCNS11643	3-6024
U+4C84.0	fR/S	 195.11
U+4C84.0	fTotalStrokes	 22
U+4C84.0	fCangjie	NFOYK
U+4C84.0	fFourCorner	28340
U+4C84.0	fCantonese	man5; maan5; mang4
U+4C84.0	fMandarin	min3; min2
U+4C84.0	fEnglish	[1] same as &#x9BB8; [2] same as &#x9C35;
U+4C85.0	fUTF8	䲅
U+4C85.0	fCNS11643	4-697B
U+4C85.0	fGB	3-7831
U+4C85.0	fR/S	 195.11
U+4C85.0	fTotalStrokes	 22; 21
U+4C85.0	fCangjie	NFQOU
U+4C85.0	fFourCorner	26310
U+4C85.0	fHakka	 gui1; kwui1
U+4C85.0	fCantonese	kwai1
U+4C85.0	fMandarin	gui1
U+4C85.0	fEnglish	a fresh-water porpoise found in the Yang-tsze
U+4C86.0	fUTF8	䲆
U+4C86.0	fCNS11643	3-6021
U+4C86.0	fR/S	 195.11
U+4C86.0	fTotalStrokes	 22
U+4C86.0	fCangjie	NFNWF
U+4C86.0	fFourCorner	27330
U+4C86.0	fCantonese	jyu4; sou1
U+4C86.0	fMandarin	xian1
U+4C87.0	fUTF8	䲇
U+4C87.0	fR/S	 195.11
U+4C87.0	fTotalStrokes	 22
U+4C87.0	fCangjie	NFYDL
U+4C87.0	fCantonese	cuk1
U+4C87.0	fEnglish	[Korean character]
U+4C88.0	fUTF8	䲈
U+4C88.0	fCNS11643	F-6A3F
U+4C88.0	fR/S	 195.11
U+4C88.0	fTotalStrokes	 22
U+4C88.0	fCangjie	NFIDD
U+4C88.0	fCantonese	maa4
U+4C89.0	fUTF8	䲉
U+4C89.0	fCNS11643	4-6B3C
U+4C89.0	fGB	3-786D
U+4C89.0	fR/S	 195.12
U+4C89.0	fTotalStrokes	 23
U+4C89.0	fCangjie	NFTCL
U+4C89.0	fFourCorner	22320
U+4C89.0	fCantonese	si1
U+4C89.0	fMandarin	si1
U+4C89.0	fEnglish	[1] a kind of fish [2] another name for tuna
U+4C8A.0	fUTF8	䲊
U+4C8A.0	fCNS11643	4-6B3D
U+4C8A.0	fGB	3-7867
U+4C8A.0	fR/S	 195.12
U+4C8A.0	fTotalStrokes	 23
U+4C8A.0	fCangjie	NFNLB
U+4C8A.0	fFourCorner	24320
U+4C8A.0	fCantonese	do6; to5
U+4C8A.0	fMandarin	duo4; tuo3
U+4C8A.0	fEnglish	[1] fry (of fish) [2] crab [3] get rid of the scales of a fish
U+4C8B.0	fUTF8	䲋
U+4C8B.0	fCNS11643	4-6B40
U+4C8B.0	fGB	3-786A
U+4C8B.0	fR/S	 195.12
U+4C8B.0	fTotalStrokes	 23
U+4C8B.0	fCangjie	NFQOA
U+4C8B.0	fFourCorner	25360
U+4C8B.0	fCantonese	sam4
U+4C8B.0	fMandarin	cen2
U+4C8B.0	fEnglish	(non-classical form) a kind of fish
U+4C8C.0	fUTF8	䲌
U+4C8C.0	fCNS11643	4-6B3B
U+4C8C.0	fGB	3-786B
U+4C8C.0	fR/S	 195.12
U+4C8C.0	fTotalStrokes	 23
U+4C8C.0	fCangjie	NFGFO
U+4C8C.0	fFourCorner	27380
U+4C8C.0	fCantonese	fun2
U+4C8C.0	fMandarin	kuan3
U+4C8C.0	fEnglish	[1] a fish [2] the sound of touching the fish net
U+4C8D.0	fUTF8	䲍
U+4C8D.0	fCNS11643	7-5E23
U+4C8D.0	fR/S	 195.12
U+4C8D.0	fTotalStrokes	 23
U+4C8D.0	fCangjie	HYFQF
U+4C8D.0	fFourCorner	29430
U+4C8D.0	fCantonese	tang4
U+4C8D.0	fMandarin	teng2
U+4C8D.0	fEnglish	standard form of &#x9C27;
U+4C8E.0	fUTF8	䲎
U+4C8E.0	fCNS11643	3-6067
U+4C8E.0	fR/S	 195.12
U+4C8E.0	fTotalStrokes	 23
U+4C8E.0	fCangjie	NFMJK
U+4C8E.0	fFourCorner	28340
U+4C8E.0	fMandarin	nei3
U+4C8E.0	fEnglish	[1] corrupted form of &$=#x9BD8; [2] same as &#x9BBE;
U+4C8F.0	fUTF8	䲏
U+4C8F.0	fCNS11643	3-6064
U+4C8F.0	fR/S	 195.12
U+4C8F.0	fTotalStrokes	 23
U+4C8F.0	fCangjie	NFFFS
U+4C8F.0	fFourCorner	29320
U+4C8F.0	fCantonese	lou4; lou6
U+4C8F.0	fMandarin	lao2
U+4C8F.0	fEnglish	a kind of fish
U+4C90.0	fUTF8	䲐
U+4C90.0	fCNS11643	5-785B
U+4C90.0	fR/S	 195.13
U+4C90.0	fTotalStrokes	 24
U+4C90.0	fCangjie	NFYPS
U+4C90.0	fFourCorner	21320
U+4C90.0	fCantonese	lou5
U+4C90.0	fMandarin	lu3
U+4C90.0	fEnglish	a kind of fish
U+4C91.0	fUTF8	䲑
U+4C91.0	fCNS11643	5-7856
U+4C91.0	fR/S	 195.13
U+4C91.0	fTotalStrokes	 24
U+4C91.0	fCangjie	NFTGI
U+4C91.0	fFourCorner	28350
U+4C91.0	fCantonese	ji4; zeoi4; zi4
U+4C91.0	fMandarin	yi2
U+4C91.0	fEnglish	[1] spawn, roe [2] a kind of fish
U+4C92.0	fUTF8	䲒
U+4C92.0	fCNS11643	3-6130
U+4C92.0	fR/S	 195.13
U+4C92.0	fTotalStrokes	 24
U+4C92.0	fCangjie	NFNBQ
U+4C92.0	fFourCorner	27350
U+4C92.0	fCantonese	haai5; hai5; ngai4
U+4C92.0	fMandarin	xie4; xie3
U+4C92.0	fEnglish	same as &#x87F9;
U+4C93.0	fUTF8	䲓
U+4C93.0	fCNS11643	4-6C4D
U+4C93.0	fGB	3-782B
U+4C93.0	fR/S	 195.13
U+4C93.0	fTotalStrokes	 24
U+4C93.0	fCangjie	NFOMO
U+4C93.0	fFourCorner	28380
U+4C93.0	fCantonese	jim2; zim5
U+4C93.0	fMandarin	yan3
U+4C93.0	fEnglish	[1] a kind of fish [2] the mouth of a fish at surface of the water; gasping for breath
U+4C94.0	fUTF8	䲔
U+4C94.0	fCNS11643	4-6C4E
U+4C94.0	fGB	5-767C
U+4C94.0	fR/S	 195.13
U+4C94.0	fTotalStrokes	 24
U+4C94.0	fCangjie	NFMWM
U+4C94.0	fFourCorner	21310
U+4C94.0	fCantonese	goeng1; king4
U+4C94.0	fMandarin	jiang1; qing2; qing4
U+4C94.0	fEnglish	[1] whale [2] huge
U+4C95.0	fUTF8	䲕
U+4C95.0	fCNS11643	5-7724
U+4C95.0	fR/S	 195.12; 195.13
U+4C95.0	fTotalStrokes	 23; 24
U+4C95.0	fCangjie	NFTCA
U+4C95.0	fFourCorner	28360
U+4C95.0	fCantonese	pou1; pou2
U+4C95.0	fMandarin	pu1; pu3; pu4
U+4C95.0	fEnglish	the Yangtze porpoise
U+4C96.0	fUTF8	䲖
U+4C96.0	fCNS11643	3-615A
U+4C96.0	fR/S	 195.14
U+4C96.0	fTotalStrokes	 25
U+4C96.0	fCangjie	NFGNI
U+4C96.0	fFourCorner	24340
U+4C96.0	fCantonese	cau4; caa1; caau1; caau4
U+4C96.0	fMandarin	chou2
U+4C96.0	fEnglish	big fish
U+4C97.0	fUTF8	䲗
U+4C97.0	fCNS11643	4-6D39
U+4C97.0	fGB	3-7863
U+4C97.0	fR/S	 195.14
U+4C97.0	fTotalStrokes	 25
U+4C97.0	fCangjie	NFHON
U+4C97.0	fFourCorner	21320
U+4C97.0	fCantonese	haam4
U+4C97.0	fMandarin	han2; xian2
U+4C97.0	fEnglish	a kind of fish, with long and flat body, small mouth and pointed lips
U+4C98.0	fUTF8	䲘
U+4C98.0	fGB	3-7876
U+4C98.0	fR/S	 195.14
U+4C98.0	fTotalStrokes	 25
U+4C98.0	fCangjie	NFHJR
U+4C98.0	fCantonese	gun2; gwun2 (also); sung1
U+4C98.0	fMandarin	guan3; kang4; wei2
U+4C98.0	fEnglish	a kind of fish; with a long cylinder body
U+4C99.0	fUTF8	䲙
U+4C99.0	fCNS11643	3-6131
U+4C99.0	fR/S	 195.15; 195.13
U+4C99.0	fTotalStrokes	 24; 26
U+4C99.0	fCangjie	NFHAL
U+4C99.0	fFourCorner	28320
U+4C99.0	fCantonese	zit3
U+4C99.0	fMandarin	jie2
U+4C99.0	fEnglish	a kind of fish, grown in fresh water, with a flat body, orange color or blue colored pattern
U+4C9A.0	fUTF8	䲚
U+4C9A.0	fCNS11643	4-6E2C
U+4C9A.0	fGB	3-7871
U+4C9A.0	fR/S	 195.16
U+4C9A.0	fTotalStrokes	 27
U+4C9A.0	fCangjie	NFDLC
U+4C9A.0	fFourCorner	27380
U+4C9A.0	fHakka	 lai5; lai6
U+4C9A.0	fCantonese	laai6; lai6; lam6
U+4C9A.0	fMandarin	ta4; lai4; lan4
U+4C9A.0	fEnglish	a small fish
U+4C9B.0	fUTF8	䲛
U+4C9B.0	fCNS11643	4-6E2D
U+4C9B.0	fGB	3-7877
U+4C9B.0	fR/S	 195.16
U+4C9B.0	fTotalStrokes	 27
U+4C9B.0	fCangjie	NFTWU
U+4C9B.0	fFourCorner	24360
U+4C9B.0	fCantonese	mung5; mung4
U+4C9B.0	fMandarin	meng1; meng2
U+4C9B.0	fEnglish	[1] an ancient name for tuna [2] a kind of sturgeon
U+4C9C.0	fUTF8	䲜
U+4C9C.0	fCNS11643	4-6E5B
U+4C9C.0	fR/S	 195.33
U+4C9C.0	fTotalStrokes	 44
U+4C9C.0	fCangjie	NFNFF
U+4C9C.0	fFourCorner	27330
U+4C9C.0	fCantonese	jip6; zip6
U+4C9C.0	fMandarin	ye4
U+4C9C.0	fEnglish	lots of fish
U+4C9D.0	fUTF8	䲝
U+4C9D.0	fGB	S-224C
U+4C9D.0	fR/S	 195.4
U+4C9D.0	fTotalStrokes	 15
U+4C9D.0	fCangjie	NMOSU
U+4C9D.0	fCantonese	ceoi2; coeng1; seoi6
U+4C9D.0	fEnglish	simplified form of &#x9BE7;
U+4C9E.0	fUTF8	䲞
U+4C9E.0	fGB	S-224E
U+4C9E.0	fR/S	 195.5
U+4C9E.0	fTotalStrokes	 16
U+4C9E.0	fCangjie	NMYT
U+4C9E.0	fCantonese	lap6
U+4C9E.0	fMandarin	li4; luo3
U+4C9E.0	fEnglish	a second general name for perch, etc.
U+4C9F.0	fUTF8	䲟
U+4C9F.0	fGB	7-2232
U+4C9F.0	fR/S	 195.5
U+4C9F.0	fTotalStrokes	 16
U+4C9F.0	fCangjie	NMHPL
U+4C9F.0	fCantonese	jaan3; zaan3
U+4C9F.0	fMandarin	yin4
U+4C9F.0	fEnglish	simplified form of &#x9BA3) a kind of fish, live in the ocean, dark brown color with two white vertical marks; flat shaped head;
U+4CA0.0	fUTF8	䲠
U+4CA0.0	fGB	7-2233
U+4CA0.0	fR/S	 195.9
U+4CA0.0	fTotalStrokes	 20
U+4CA0.0	fCangjie	NMQKA
U+4CA0.0	fCantonese	ceon1
U+4CA0.0	fEnglish	simplified form of &#x9C06;
U+4CA1.0	fUTF8	䲡
U+4CA1.0	fGB	7-224D
U+4CA1.0	fR/S	 195.9
U+4CA1.0	fTotalStrokes	 20
U+4CA1.0	fCangjie	NMTCW
U+4CA1.0	fCantonese	caau1; cau1
U+4CA1.0	fEnglish	simplified form of &#x9C0C;
U+4CA2.0	fUTF8	䲢
U+4CA2.0	fGB	7-2234
U+4CA2.0	fR/S	 195.10
U+4CA2.0	fTotalStrokes	 21
U+4CA2.0	fCangjie	BFQM
U+4CA2.0	fCantonese	taang4; tang4
U+4CA2.0	fMandarin	teng2
U+4CA2.0	fEnglish	simplified form of &#x9C27;
U+4CA3.0	fUTF8	䲣
U+4CA3.0	fGB	7-224E
U+4CA3.0	fR/S	 195.10
U+4CA3.0	fTotalStrokes	 21
U+4CA3.0	fCangjie	YMOK
U+4CA3.0	fCantonese	jyu4; zyu4
U+4CA3.0	fMandarin	yu2
U+4CA3.0	fEnglish	to fish; to seize
U+4CA4.0	fUTF8	䲤
U+4CA4.0	fGB	S-224D
U+4CA4.0	fR/S	 195.10
U+4CA4.0	fTotalStrokes	 21
U+4CA4.0	fCangjie	NFEMW
U+4CA4.0	fCantonese	zau2
U+4CA5.0	fUTF8	䲥
U+4CA5.0	fCNS11643	3-4565
U+4CA5.0	fGB	5-5B74
U+4CA5.0	fR/S	 196.2
U+4CA5.0	fTotalStrokes	 13
U+4CA5.0	fCangjie	KNHAF
U+4CA5.0	fFourCorner	40320
U+4CA5.0	fCantonese	gau1
U+4CA5.0	fMandarin	jiu1
U+4CA5.0	fEnglish	same as &#x9CE9;
U+4CA6.0	fUTF8	䲦
U+4CA6.0	fCNS11643	3-4A7A
U+4CA6.0	fR/S	 196.3
U+4CA6.0	fTotalStrokes	 14
U+4CA6.0	fCangjie	HFK
U+4CA6.0	fFourCorner	24330
U+4CA6.0	fCantonese	daai6
U+4CA6.0	fMandarin	di4; cha2; dai4; tuo3
U+4CA6.0	fEnglish	a legendary bird
U+4CA7.0	fUTF8	䲧
U+4CA7.0	fCNS11643	3-4A7B
U+4CA7.0	fR/S	 196.3; 196.11; 32.11
U+4CA7.0	fTotalStrokes	 14
U+4CA7.0	fCangjie	GHAF
U+4CA7.0	fFourCorner	47120
U+4CA7.0	fHakka	 tu5; tu6
U+4CA7.0	fCantonese	dou6; caau4; dung3
U+4CA7.0	fMandarin	du4; shui4
U+4CA7.0	fEnglish	the goatsucker
U+4CA8.0	fUTF8	䲨
U+4CA8.0	fCNS11643	3-4A7C
U+4CA8.0	fGB	5-354D
U+4CA8.0	fR/S	 196.3
U+4CA8.0	fTotalStrokes	 14
U+4CA8.0	fCangjie	MHAF
U+4CA8.0	fFourCorner	17120
U+4CA8.0	fCantonese	hung4
U+4CA8.0	fMandarin	hong2
U+4CA8.0	fEnglish	same as &#x9D3B;
U+4CA9.0	fUTF8	䲩
U+4CA9.0	fCNS11643	3-4A7E
U+4CA9.0	fR/S	 196.3
U+4CA9.0	fTotalStrokes	 14
U+4CA9.0	fCangjie	SHAF
U+4CA9.0	fFourCorner	77220
U+4CA9.0	fCantonese	si1
U+4CA9.0	fMandarin	shi1
U+4CA9.0	fEnglish	same as &#x9CF2;
U+4CAA.0	fUTF8	䲪
U+4CAA.0	fCNS11643	5-4B6B
U+4CAA.0	fGB	3-352F
U+4CAA.0	fR/S	 196.3
U+4CAA.0	fTotalStrokes	 14
U+4CAA.0	fCangjie	KHAF
U+4CAA.0	fFourCorner	47020
U+4CAA.0	fCantonese	ji6; hei3
U+4CAA.0	fMandarin	xi4
U+4CAA.0	fEnglish	a kind of bird
U+4CAB.0	fUTF8	䲫
U+4CAB.0	fCNS11643	3-4A7D
U+4CAB.0	fR/S	 196.3
U+4CAB.0	fTotalStrokes	 14
U+4CAB.0	fCangjie	MUHAF
U+4CAB.0	fFourCorner	17212
U+4CAB.0	fCantonese	gaau1; kaau1
U+4CAB.0	fEnglish	corrupted form of &#x9CE9;
U+4CAC.0	fUTF8	䲬
U+4CAC.0	fCNS11643	5-5338
U+4CAC.0	fGB	3-5D26
U+4CAC.0	fR/S	 196.4
U+4CAC.0	fTotalStrokes	 15
U+4CAC.0	fCangjie	HFHVP
U+4CAC.0	fFourCorner	22340
U+4CAC.0	fCantonese	kei4; gan1; zi1
U+4CAC.0	fMandarin	zhi1; qi2
U+4CAC.0	fEnglish	[1] chicken [2] another name for a pheasant [3] the wild goose
U+4CAD.0	fUTF8	䲭
U+4CAD.0	fCNS11643	4-4F3F
U+4CAD.0	fGB	5-4F7E
U+4CAD.0	fR/S	 196.4
U+4CAD.0	fTotalStrokes	 15
U+4CAD.0	fCangjie	HPHAF
U+4CAD.0	fFourCorner	77720
U+4CAD.0	fCantonese	ci1
U+4CAD.0	fMandarin	chi2
U+4CAD.0	fEnglish	[1] kite [2] owl [3] wine cups
U+4CAE.0	fUTF8	䲮
U+4CAE.0	fCNS11643	3-5023
U+4CAE.0	fR/S	 196.4
U+4CAE.0	fTotalStrokes	 15
U+4CAE.0	fCangjie	MUHAF
U+4CAE.0	fFourCorner	17220
U+4CAE.0	fCantonese	jyun4; zyun4
U+4CAE.0	fMandarin	yuan2
U+4CAE.0	fEnglish	a kind of bird
U+4CAF.0	fUTF8	䲯
U+4CAF.0	fCNS11643	4-4F44
U+4CAF.0	fGB	3-5C76
U+4CAF.0	fR/S	 196.4
U+4CAF.0	fTotalStrokes	 15
U+4CAF.0	fCangjie	NEHAF
U+4CAF.0	fFourCorner	17220
U+4CAF.0	fCantonese	kap6; gaap6; kaap6
U+4CAF.0	fMandarin	ji2
U+4CAF.0	fEnglish	a kind of black small bird
U+4CB0.0	fUTF8	䲰
U+4CB0.0	fCNS11643	4-4F3B
U+4CB0.0	fGB	5-5B79
U+4CB0.0	fR/S	 196.4
U+4CB0.0	fTotalStrokes	 15
U+4CB0.0	fCangjie	MIHAF
U+4CB0.0	fFourCorner	17720
U+4CB0.0	fHakka	 jun5; run6
U+4CB0.0	fCantonese	wan6
U+4CB0.0	fMandarin	yun4
U+4CB0.0	fEnglish	a bird that is said to eat snakes
U+4CB1.0	fUTF8	䲱
U+4CB1.0	fCNS11643	7-2B6D
U+4CB1.0	fR/S	 196.4
U+4CB1.0	fTotalStrokes	 15
U+4CB1.0	fCangjie	HFYHS
U+4CB1.0	fFourCorner	20320
U+4CB1.0	fCantonese	fong2; fong4
U+4CB1.0	fMandarin	fang3
U+4CB1.0	fEnglish	same as &#x9DAD;
U+4CB2.0	fUTF8	䲲
U+4CB2.0	fCNS11643	4-4F40
U+4CB2.0	fGB	3-5D23
U+4CB2.0	fR/S	 196.4
U+4CB2.0	fTotalStrokes	 15
U+4CB2.0	fCangjie	CIHAF
U+4CB2.0	fFourCorner	87720
U+4CB2.0	fCantonese	gung1
U+4CB2.0	fMandarin	gong1
U+4CB2.0	fEnglish	a hawk-like bird, much smaller
U+4CB3.0	fUTF8	䲳
U+4CB3.0	fCNS11643	3-5028
U+4CB3.0	fR/S	 196.4
U+4CB3.0	fTotalStrokes	 15
U+4CB3.0	fCangjie	YNHAF
U+4CB3.0	fFourCorner	07220
U+4CB3.0	fCantonese	hong4
U+4CB3.0	fMandarin	hang2
U+4CB3.0	fEnglish	birds flying up and down
U+4CB4.0	fUTF8	䲴
U+4CB4.0	fCNS11643	3-5029
U+4CB4.0	fR/S	 196.4; 196.11; 86.11
U+4CB4.0	fTotalStrokes	 15
U+4CB4.0	fCangjie	FHAF
U+4CB4.0	fFourCorner	97820
U+4CB4.0	fCantonese	zam6
U+4CB4.0	fMandarin	zhen4
U+4CB4.0	fEnglish	same as &#x9D06;
U+4CB5.0	fUTF8	䲵
U+4CB5.0	fCNS11643	3-5026
U+4CB5.0	fR/S	 196.4
U+4CB5.0	fTotalStrokes	 15
U+4CB5.0	fCangjie	FHHAF
U+4CB5.0	fFourCorner	90320
U+4CB5.0	fCantonese	zoek3
U+4CB5.0	fMandarin	que4; hu4
U+4CB5.0	fEnglish	same as &#x96C0;
U+4CB6.0	fUTF8	䲶
U+4CB6.0	fR/S	 196.4
U+4CB6.0	fTotalStrokes	 15
U+4CB6.0	fCangjie	MUHAF
U+4CB6.0	fCantonese	jyun4
U+4CB6.0	fEnglish	[Korean character]
U+4CB7.0	fUTF8	䲷
U+4CB7.0	fCNS11643	3-4C63
U+4CB7.0	fR/S	 196.4
U+4CB7.0	fTotalStrokes	 15
U+4CB7.0	fCangjie	HFD
U+4CB7.0	fFourCorner	27900
U+4CB7.0	fCantonese	niu5; liu5 (also)
U+4CB7.0	fMandarin	xiao1
U+4CB8.0	fUTF8	䲸
U+4CB8.0	fCNS11643	4-4F41
U+4CB8.0	fGB	3-5C7D
U+4CB8.0	fR/S	 196.4
U+4CB8.0	fTotalStrokes	 15
U+4CB8.0	fCangjie	OLHAF
U+4CB8.0	fFourCorner	87220
U+4CB8.0	fCantonese	gaai3
U+4CB8.0	fMandarin	jie4
U+4CB8.0	fEnglish	a kind of bird
U+4CB9.0	fUTF8	䲹
U+4CB9.0	fCNS11643	4-554E
U+4CB9.0	fGB	3-5D2C
U+4CB9.0	fR/S	 196.5
U+4CB9.0	fTotalStrokes	 16
U+4CB9.0	fCangjie	MMHAF
U+4CB9.0	fFourCorner	17120
U+4CB9.0	fCantonese	pei4
U+4CB9.0	fMandarin	pei2
U+4CB9.0	fEnglish	ospery; water hawk
U+4CBA.0	fUTF8	䲺
U+4CBA.0	fCNS11643	3-542F
U+4CBA.0	fR/S	 196.5
U+4CBA.0	fTotalStrokes	 16
U+4CBA.0	fCangjie	TMHAF
U+4CBA.0	fFourCorner	47727
U+4CBA.0	fCantonese	gam3
U+4CBA.0	fMandarin	gan4
U+4CBA.0	fEnglish	birds singing; chirps
U+4CBB.0	fUTF8	䲻
U+4CBB.0	fCNS11643	4-554D
U+4CBB.0	fGB	5-5C27
U+4CBB.0	fR/S	 196.5
U+4CBB.0	fTotalStrokes	 16
U+4CBB.0	fCangjie	YIHAF
U+4CBB.0	fFourCorner	07720
U+4CBB.0	fCantonese	jyun4; zyun4
U+4CBB.0	fMandarin	xuan1; xuan2; yuan2
U+4CBB.0	fEnglish	[1] the swallow [2] same as &#x9CF6;
U+4CBC.0	fUTF8	䲼
U+4CBC.0	fCNS11643	3-5430
U+4CBC.0	fR/S	 196.5
U+4CBC.0	fTotalStrokes	 16
U+4CBC.0	fCangjie	HMHAF
U+4CBC.0	fFourCorner	27120
U+4CBC.0	fCantonese	sang1; saang1
U+4CBC.0	fMandarin	sheng1
U+4CBC.0	fEnglish	[1] a kind of bird [2] same as &#x9F2A;
U+4CBD.0	fUTF8	䲽
U+4CBD.0	fCNS11643	4-554F
U+4CBD.0	fGB	3-5764
U+4CBD.0	fR/S	 196.5
U+4CBD.0	fTotalStrokes	 16
U+4CBD.0	fCangjie	MRHAF
U+4CBD.0	fFourCorner	17620
U+4CBD.0	fCantonese	sek6
U+4CBD.0	fMandarin	shi2; dao3
U+4CBD.0	fEnglish	a kind of bird; suspended rocks
U+4CBE.0	fUTF8	䲾
U+4CBE.0	fCNS11643	3-542B
U+4CBE.0	fR/S	 196.5
U+4CBE.0	fTotalStrokes	 16
U+4CBE.0	fCangjie	MSHAF
U+4CBE.0	fFourCorner	11320
U+4CBE.0	fHakka	 kau3
U+4CBE.0	fCantonese	haau2
U+4CBE.0	fMandarin	qiao3
U+4CBE.0	fEnglish	the tailor-bird
U+4CBF.0	fUTF8	䲿
U+4CBF.0	fCNS11643	4-5554
U+4CBF.0	fR/S	 196.5
U+4CBF.0	fTotalStrokes	 16
U+4CBF.0	fCangjie	SPHAF
U+4CBF.0	fFourCorner	77220
U+4CBF.0	fCantonese	ci4
U+4CBF.0	fMandarin	ci2
U+4CBF.0	fEnglish	non-classical form of &#x9DBF;, &#x9DC0;
U+4CC0.0	fUTF8	䳀
U+4CC0.0	fCNS11643	7-364A
U+4CC0.0	fGB	3-5D31
U+4CC0.0	fR/S	 196.5
U+4CC0.0	fTotalStrokes	 16
U+4CC0.0	fCangjie	HFHQO
U+4CC0.0	fFourCorner	25330
U+4CC0.0	fCantonese	dit6
U+4CC0.0	fMandarin	yi4; die2
U+4CC0.0	fEnglish	[1] the cuckoo [2] a kind of pigeon [3] a kind of water bird
U+4CC1.0	fUTF8	䳁
U+4CC1.0	fCNS11643	4-554A
U+4CC1.0	fGB	3-5D2D
U+4CC1.0	fR/S	 196.5
U+4CC1.0	fTotalStrokes	 16
U+4CC1.0	fCangjie	HFIKK
U+4CC1.0	fFourCorner	23340
U+4CC1.0	fCantonese	bat6
U+4CC1.0	fMandarin	ba2; bin3; bo2; yuan2
U+4CC1.0	fEnglish	[1] a wild duck-like water bird [2] a pheasant like bird
U+4CC2.0	fUTF8	䳂
U+4CC2.0	fCNS11643	3-542E
U+4CC2.0	fGB	5-5C29
U+4CC2.0	fR/S	 196.5
U+4CC2.0	fTotalStrokes	 16
U+4CC2.0	fCangjie	SRHAF
U+4CC2.0	fFourCorner	17620
U+4CC2.0	fCantonese	diu1
U+4CC2.0	fMandarin	diao1; tiao2
U+4CC2.0	fEnglish	[1] a kind of small bird [2] sound of birds [3] the feathered; birds
U+4CC3.0	fUTF8	䳃
U+4CC3.0	fCNS11643	4-5557
U+4CC3.0	fGB	3-5D34
U+4CC3.0	fR/S	 196.5
U+4CC3.0	fTotalStrokes	 16
U+4CC3.0	fCangjie	HFNIU
U+4CC3.0	fFourCorner	27210
U+4CC3.0	fCantonese	jyun2; zyun1; zyun2
U+4CC3.0	fMandarin	wan3
U+4CC3.0	fEnglish	a kind of phoenix-like bird
U+4CC4.0	fUTF8	䳄
U+4CC4.0	fCNS11643	3-574B
U+4CC4.0	fGB	5-5C2E
U+4CC4.0	fR/S	 196.5; 196.6
U+4CC4.0	fTotalStrokes	 17; 16
U+4CC4.0	fCangjie	YMPHF
U+4CC4.0	fFourCorner	27120
U+4CC4.0	fCantonese	ci1; ci2
U+4CC4.0	fMandarin	ci1; ci2
U+4CC4.0	fEnglish	[1] same as &#x9D1C; [2] same as &#x96CC;
U+4CC5.0	fUTF8	䳅
U+4CC5.0	fCNS11643	4-554B
U+4CC5.0	fGB	3-5D30
U+4CC5.0	fR/S	 196.5
U+4CC5.0	fTotalStrokes	 16
U+4CC5.0	fCangjie	HFRC
U+4CC5.0	fFourCorner	26380
U+4CC5.0	fCantonese	zi2
U+4CC5.0	fMandarin	zhi3
U+4CC5.0	fEnglish	[1] a kind of bird which shares its nest with rats [2] the sound of birds
U+4CC6.0	fUTF8	䳆
U+4CC6.0	fCNS11643	3-5431
U+4CC6.0	fR/S	 196.5
U+4CC6.0	fTotalStrokes	 16
U+4CC6.0	fCangjie	HAHAF
U+4CC6.0	fFourCorner	27620
U+4CC6.0	fCantonese	baak6
U+4CC6.0	fMandarin	bo2; bai2
U+4CC6.0	fEnglish	a kind of species of myna (or mynah); a grackle
U+4CC7.0	fUTF8	䳇
U+4CC7.0	fCNS11643	4-5555
U+4CC7.0	fGB	3-5326
U+4CC7.0	fR/S	 196.5
U+4CC7.0	fTotalStrokes	 16
U+4CC7.0	fCangjie	WYHAF
U+4CC7.0	fFourCorner	77520
U+4CC7.0	fHakka	 wu3; vu3
U+4CC7.0	fCantonese	mou5; mou2
U+4CC7.0	fMandarin	mu3; wu3
U+4CC7.0	fEnglish	the cockatoo
U+4CC8.0	fUTF8	䳈
U+4CC8.0	fCNS11643	3-5434
U+4CC8.0	fR/S	 196.5
U+4CC8.0	fTotalStrokes	 16
U+4CC8.0	fCangjie	HFPRU
U+4CC8.0	fFourCorner	27310
U+4CC8.0	fCantonese	bou2
U+4CC8.0	fMandarin	bao3
U+4CC8.0	fEnglish	same as &#x9D07;
U+4CC9.0	fUTF8	䳉
U+4CC9.0	fCNS11643	4-5550
U+4CC9.0	fGB	3-5D33
U+4CC9.0	fR/S	 196.5
U+4CC9.0	fTotalStrokes	 16
U+4CC9.0	fCangjie	HYHAF
U+4CC9.0	fFourCorner	27320
U+4CC9.0	fCantonese	dung1; dam1; dung6
U+4CC9.0	fMandarin	dong1; dan4
U+4CC9.0	fEnglish	[1] a kind of water bird [2] same as &#x9D06;
U+4CCA.0	fUTF8	䳊
U+4CCA.0	fCNS11643	7-3651
U+4CCA.0	fR/S	 196.5
U+4CCA.0	fTotalStrokes	 16
U+4CCA.0	fCangjie	IKHAF
U+4CCA.0	fFourCorner	47020
U+4CCA.0	fCantonese	bat6; baat6; but6
U+4CCA.0	fMandarin	ba2
U+4CCA.0	fEnglish	same as &#x4CC1;
U+4CCB.0	fUTF8	䳋
U+4CCB.0	fCNS11643	3-5633
U+4CCB.0	fGB	5-6126
U+4CCB.0	fR/S	 196.6; 142.11
U+4CCB.0	fTotalStrokes	 17
U+4CCB.0	fCangjie	LIHAF
U+4CCB.0	fFourCorner	57120
U+4CCB.0	fCantonese	tung4
U+4CCB.0	fMandarin	jiao1; tong2; zhong4
U+4CCB.0	fEnglish	[1] a pheasant-like bird, with black and body and red feet [2] a kind of bird [3] same as &#x87C2;
U+4CCC.0	fUTF8	䳌
U+4CCC.0	fCNS11643	3-574C
U+4CCC.0	fR/S	 196.6
U+4CCC.0	fTotalStrokes	 17
U+4CCC.0	fCangjie	IBHAF
U+4CCC.0	fFourCorner	27220
U+4CCC.0	fCantonese	gyun1
U+4CCC.0	fMandarin	juan1
U+4CCC.0	fEnglish	non-classical form of &#x9D51;
U+4CCD.0	fUTF8	䳍
U+4CCD.0	fCNS11643	4-5A51
U+4CCD.0	fGB	3-5D3D
U+4CCD.0	fR/S	 196.6
U+4CCD.0	fTotalStrokes	 17; 16
U+4CCD.0	fCangjie	TCHAF
U+4CCD.0	fFourCorner	47820
U+4CCD.0	fCantonese	gung6; gung1 (also)
U+4CCD.0	fMandarin	gong1
U+4CCD.0	fEnglish	a kind of pheasant-like bird
U+4CCE.0	fUTF8	䳎
U+4CCE.0	fCNS11643	3-5747
U+4CCE.0	fR/S	 196.6
U+4CCE.0	fTotalStrokes	 17
U+4CCE.0	fCangjie	HXHAF
U+4CCE.0	fFourCorner	77720
U+4CCE.0	fCantonese	kau5; haau6; hin1; kaau3; kaau5
U+4CCE.0	fMandarin	jiu4
U+4CCE.0	fEnglish	a pigeon-like bird, with a comb or crest
U+4CCF.0	fUTF8	䳏
U+4CCF.0	fCNS11643	3-5749
U+4CCF.0	fGB	5-5C2C
U+4CCF.0	fR/S	 196.6
U+4CCF.0	fTotalStrokes	 17
U+4CCF.0	fCangjie	GGHAF
U+4CCF.0	fFourCorner	47120
U+4CCF.0	fCantonese	gwai3; kyut3 (also); gwaai3
U+4CCF.0	fMandarin	gui4
U+4CCF.0	fEnglish	[1] cuckoo [2] pelican [3] same as &#x9D03;
U+4CD0.0	fUTF8	䳐
U+4CD0.0	fCNS11643	3-5746
U+4CD0.0	fR/S	 196.6
U+4CD0.0	fTotalStrokes	 17
U+4CD0.0	fCangjie	IOHAF
U+4CD0.0	fFourCorner	37320
U+4CD0.0	fCantonese	ci3
U+4CD0.0	fMandarin	ci4
U+4CD0.0	fEnglish	an owl-like bird
U+4CD1.0	fUTF8	䳑
U+4CD1.0	fCNS11643	4-5A52
U+4CD1.0	fGB	3-5D3E
U+4CD1.0	fR/S	 196.6
U+4CD1.0	fTotalStrokes	 17; 16
U+4CD1.0	fCangjie	KBHAF
U+4CD1.0	fFourCorner	47220
U+4CD1.0	fCantonese	jau5; zuk1
U+4CD1.0	fMandarin	you3; yu4
U+4CD1.0	fEnglish	a pheasant-like bird
U+4CD2.0	fUTF8	䳒
U+4CD2.0	fCNS11643	3-574E
U+4CD2.0	fR/S	 196.6
U+4CD2.0	fTotalStrokes	 17
U+4CD2.0	fCangjie	JIHAF
U+4CD2.0	fFourCorner	43350
U+4CD2.0	fHakka	 ren1; jen1; jen2
U+4CD2.0	fCantonese	zyun4
U+4CD2.0	fMandarin	yuan2
U+4CD2.0	fEnglish	 kite
U+4CD3.0	fUTF8	䳓
U+4CD3.0	fCNS11643	4-5A50
U+4CD3.0	fGB	3-6041
U+4CD3.0	fR/S	 196.6
U+4CD3.0	fTotalStrokes	 17; 16
U+4CD3.0	fCangjie	JPHAF
U+4CD3.0	fFourCorner	47720
U+4CD3.0	fCantonese	lou5
U+4CD3.0	fMandarin	lao3
U+4CD3.0	fEnglish	[1] a kind of water bird [2] a vulture; a condor
U+4CD4.0	fUTF8	䳔
U+4CD4.0	fCNS11643	4-5F2F
U+4CD4.0	fGB	3-6544
U+4CD4.0	fR/S	 196.6; 196.7
U+4CD4.0	fTotalStrokes	 18; 17
U+4CD4.0	fCangjie	HXHAF
U+4CD4.0	fFourCorner	77320
U+4CD4.0	fCantonese	guk1
U+4CD4.0	fMandarin	ju2; jiu4
U+4CD4.0	fEnglish	[1] a pigeon-like bird; with a comb or crest [2] a shrike; butcherbird
U+4CD5.0	fUTF8	䳕
U+4CD5.0	fCNS11643	4-5F35
U+4CD5.0	fGB	3-5D58
U+4CD5.0	fR/S	 196.7
U+4CD5.0	fTotalStrokes	 18
U+4CD5.0	fCangjie	BDHAF
U+4CD5.0	fFourCorner	27420
U+4CD5.0	fCantonese	fau4; faau4; fu1; man5; wui6
U+4CD5.0	fMandarin	fu2; fou2
U+4CD5.0	fEnglish	[1] the pigeon [2] the turtle dove
U+4CD6.0	fUTF8	䳖
U+4CD6.0	fCNS11643	4-5F2C
U+4CD6.0	fGB	3-5D49
U+4CD6.0	fR/S	 196.7
U+4CD6.0	fTotalStrokes	 18
U+4CD6.0	fCangjie	HFSJU
U+4CD6.0	fFourCorner	21310
U+4CD6.0	fCantonese	nip6; lip6 (also)
U+4CD6.0	fMandarin	nie4; nei4; ye4
U+4CD6.0	fEnglish	[1] flying birds [2] a kind of bird
U+4CD7.0	fUTF8	䳗
U+4CD7.0	fCNS11643	3-5A21
U+4CD7.0	fR/S	 196.7
U+4CD7.0	fTotalStrokes	 18
U+4CD7.0	fCangjie	HFHQI
U+4CD7.0	fFourCorner	27550
U+4CD7.0	fCantonese	ngo4
U+4CD7.0	fMandarin	e2
U+4CD7.0	fEnglish	ancient form of &#x9D5D;
U+4CD8.0	fUTF8	䳘
U+4CD8.0	fCNS11643	3-597C
U+4CD8.0	fR/S	 196.7
U+4CD8.0	fTotalStrokes	 18
U+4CD8.0	fCangjie	HFHQI
U+4CD8.0	fFourCorner	23350
U+4CD8.0	fCantonese	ngo4
U+4CD8.0	fMandarin	e2
U+4CD8.0	fEnglish	same as &#x4CD7;, &#x9D5D;
U+4CD9.0	fUTF8	䳙
U+4CD9.0	fCNS11643	4-5F32
U+4CD9.0	fGB	3-5D50
U+4CD9.0	fR/S	 196.7
U+4CD9.0	fTotalStrokes	 18
U+4CD9.0	fCangjie	LSHAF
U+4CD9.0	fFourCorner	57220
U+4CD9.0	fCantonese	hing2; sing2
U+4CD9.0	fMandarin	xing3
U+4CD9.0	fEnglish	[1] an egret-like bird [2] a kind of water bird
U+4CDA.0	fUTF8	䳚
U+4CDA.0	fCNS11643	4-5F31
U+4CDA.0	fGB	3-5D4F
U+4CDA.0	fR/S	 196.7
U+4CDA.0	fTotalStrokes	 18
U+4CDA.0	fCangjie	AJHAF
U+4CDA.0	fFourCorner	67420
U+4CDA.0	fHakka	 kan3; kon3
U+4CDA.0	fCantonese	hon2; hot3 (also)
U+4CDA.0	fMandarin	kan3; he2
U+4CDA.0	fEnglish	a bird which is said to sing night and day
U+4CDB.0	fUTF8	䳛
U+4CDB.0	fCNS11643	5-6726
U+4CDB.0	fR/S	 196.7
U+4CDB.0	fTotalStrokes	 18
U+4CDB.0	fCangjie	AVHAF
U+4CDB.0	fFourCorner	67420
U+4CDB.0	fCantonese	aan3; ngaan3 (also); an3
U+4CDB.0	fMandarin	yan4
U+4CDB.0	fEnglish	same as &#x9DC3;
U+4CDC.0	fUTF8	䳜
U+4CDC.0	fCNS11643	4-5F38
U+4CDC.0	fGB	3-5D55
U+4CDC.0	fR/S	 196.7
U+4CDC.0	fTotalStrokes	 18
U+4CDC.0	fCangjie	ODHAF
U+4CDC.0	fFourCorner	87920
U+4CDC.0	fCantonese	tou4
U+4CDC.0	fMandarin	yu2; tu2
U+4CDC.0	fEnglish	same as &#x9D4C;
U+4CDD.0	fUTF8	䳝
U+4CDD.0	fCNS11643	4-626A
U+4CDD.0	fGB	3-5D69
U+4CDD.0	fR/S	 196.8
U+4CDD.0	fTotalStrokes	 19
U+4CDD.0	fCangjie	YRHAF
U+4CDD.0	fFourCorner	07620
U+4CDD.0	fCantonese	pau2; faau2; fu2
U+4CDD.0	fMandarin	pou3; bu4; diao4; fu3
U+4CDD.0	fEnglish	[1] a kind of small birds [2] a kind of bird
U+4CDE.0	fUTF8	䳞
U+4CDE.0	fCNS11643	3-5C21
U+4CDE.0	fR/S	 196.8
U+4CDE.0	fTotalStrokes	 19
U+4CDE.0	fCangjie	QQHAF
U+4CDE.0	fFourCorner	57520
U+4CDE.0	fCantonese	bung2; bang1
U+4CDE.0	fMandarin	beng3
U+4CDE.0	fEnglish	birds flying all directions
U+4CDF.0	fUTF8	䳟
U+4CDF.0	fCNS11643	4-626C
U+4CDF.0	fGB	5-5C3D
U+4CDF.0	fR/S	 196.8
U+4CDF.0	fTotalStrokes	 19
U+4CDF.0	fCangjie	ABHF
U+4CDF.0	fFourCorner	67020
U+4CDF.0	fHakka	 min2
U+4CDF.0	fCantonese	ming4
U+4CDF.0	fMandarin	ming2; kou4; mo3
U+4CDF.0	fEnglish	a kind of pheasant
U+4CE0.0	fUTF8	䳠
U+4CE0.0	fCNS11643	4-6561
U+4CE0.0	fGB	3-5D62
U+4CE0.0	fR/S	 196.8; 196.9
U+4CE0.0	fTotalStrokes	 20; 19
U+4CE0.0	fCangjie	HMHAF
U+4CE0.0	fFourCorner	27120
U+4CE0.0	fCantonese	seoi6; seoi4
U+4CE0.0	fMandarin	chui2; rui4; zhu4
U+4CE0.0	fEnglish	[1] owl [2] kite [3] hawk; sparrow hawk [4] stop flying (said of bird)
U+4CE1.0	fUTF8	䳡
U+4CE1.0	fCNS11643	3-5C28
U+4CE1.0	fR/S	 196.8
U+4CE1.0	fTotalStrokes	 19
U+4CE1.0	fCangjie	OGHAF
U+4CE1.0	fFourCorner	27120
U+4CE1.0	fHakka	 zhui1
U+4CE1.0	fCantonese	aan3; ngaan3 (also); zeoi1
U+4CE1.0	fMandarin	zhui1
U+4CE1.0	fEnglish	same as &#x9D7B;
U+4CE2.0	fUTF8	䳢
U+4CE2.0	fCNS11643	4-626F
U+4CE2.0	fGB	3-5D5C
U+4CE2.0	fR/S	 196.8
U+4CE2.0	fTotalStrokes	 19
U+4CE2.0	fCangjie	TCHAF
U+4CE2.0	fFourCorner	47820
U+4CE2.0	fCantonese	kei4; hei1
U+4CE2.0	fMandarin	qi2
U+4CE2.0	fEnglish	same as &#x9D80;
U+4CE3.0	fUTF8	䳣
U+4CE3.0	fCNS11643	5-6B3C
U+4CE3.0	fR/S	 196.8
U+4CE3.0	fTotalStrokes	 19
U+4CE3.0	fCangjie	JSHAF
U+4CE3.0	fFourCorner	43320
U+4CE3.0	fCantonese	jyun1; zyun1
U+4CE3.0	fMandarin	yuan2
U+4CE3.0	fEnglish	non-classical form of &#x9CF6;
U+4CE4.0	fUTF8	䳤
U+4CE4.0	fCNS11643	4-5F36
U+4CE4.0	fGB	3-5D4E
U+4CE4.0	fR/S	 196.8
U+4CE4.0	fTotalStrokes	 19; 18
U+4CE4.0	fCangjie	FBHAF
U+4CE4.0	fFourCorner	97220
U+4CE4.0	fCantonese	bit3
U+4CE4.0	fEnglish	[1] a kind of strange, weird, legendary bird [2] another name for an owl
U+4CE5.0	fUTF8	䳥
U+4CE5.0	fCNS11643	F-6475
U+4CE5.0	fGB	3-5D52
U+4CE5.0	fR/S	 196.8
U+4CE5.0	fTotalStrokes	 19
U+4CE5.0	fCangjie	UIHAF
U+4CE5.0	fCantonese	ling4
U+4CE6.0	fUTF8	䳦
U+4CE6.0	fCNS11643	3-5D78
U+4CE6.0	fR/S	 196.9
U+4CE6.0	fTotalStrokes	 20
U+4CE6.0	fCangjie	JMHAF
U+4CE6.0	fFourCorner	37120
U+4CE6.0	fCantonese	syun1; hyun1; kwaan2
U+4CE6.0	fMandarin	xuan1
U+4CE6.0	fEnglish	[1] wren; the tailorbird [2] a kind of small bird
U+4CE7.0	fUTF8	䳧
U+4CE7.0	fCNS11643	4-656C
U+4CE7.0	fGB	3-5D7B
U+4CE7.0	fR/S	 196.9
U+4CE7.0	fTotalStrokes	 20
U+4CE7.0	fCangjie	HFONK
U+4CE7.0	fFourCorner	27330
U+4CE7.0	fCantonese	hau1; haau4
U+4CE7.0	fMandarin	hou2
U+4CE7.0	fEnglish	an eagle; a hawk
U+4CE8.0	fUTF8	䳨
U+4CE8.0	fCNS11643	3-5D7D
U+4CE8.0	fR/S	 196.9
U+4CE8.0	fTotalStrokes	 20
U+4CE8.0	fCangjie	HGHAF
U+4CE8.0	fFourCorner	27120
U+4CE8.0	fCantonese	wong4
U+4CE8.0	fMandarin	huang2
U+4CE8.0	fEnglish	[1] same as &#x51F0; [2] standard form of &#x7687;
U+4CE9.0	fUTF8	䳩
U+4CE9.0	fCNS11643	3-5D7B
U+4CE9.0	fR/S	 196.9
U+4CE9.0	fTotalStrokes	 20
U+4CE9.0	fCangjie	MVHAF
U+4CE9.0	fFourCorner	17420
U+4CE9.0	fCantonese	jiu1; ziu1
U+4CE9.0	fMandarin	yao1
U+4CE9.0	fEnglish	[1] a kind of bird [2] a pheasant-like bird, red body and with a long tail
U+4CEA.0	fUTF8	䳪
U+4CEA.0	fCNS11643	4-656A
U+4CEA.0	fGB	3-5D78
U+4CEA.0	fR/S	 196.9
U+4CEA.0	fTotalStrokes	 20
U+4CEA.0	fCangjie	UBHAF
U+4CEA.0	fFourCorner	27220
U+4CEA.0	fCantonese	tyun4; teon1; zyun1
U+4CEA.0	fMandarin	zhuan1; juan4; tuan2
U+4CEA.0	fEnglish	non-classical form of &#x9DD2;
U+4CEB.0	fUTF8	䳫
U+4CEB.0	fCNS11643	4-6564
U+4CEB.0	fGB	3-5E22
U+4CEB.0	fR/S	 196.9
U+4CEB.0	fTotalStrokes	 20
U+4CEB.0	fCangjie	NKHAF
U+4CEB.0	fFourCorner	27420
U+4CEB.0	fCantonese	kwai4
U+4CEB.0	fMandarin	kui2
U+4CEB.0	fEnglish	(standard form) a small pigeon
U+4CEC.0	fUTF8	䳬
U+4CEC.0	fCNS11643	3-5D7C
U+4CEC.0	fR/S	 196.9
U+4CEC.0	fTotalStrokes	 20
U+4CEC.0	fCangjie	TJHAF
U+4CEC.0	fFourCorner	47520
U+4CEC.0	fCantonese	jik6; aa3; bit3; fong2; gei3; tou4; zik6; zim2
U+4CEC.0	fMandarin	yi4; e4; yao3
U+4CEC.0	fEnglish	[1] same as &#x9DCA; [2] same as &#x9D83;
U+4CED.0	fUTF8	䳭
U+4CED.0	fCNS11643	7-5242
U+4CED.0	fGB	5-5C39
U+4CED.0	fR/S	 196.9
U+4CED.0	fTotalStrokes	 20
U+4CED.0	fCangjie	HPSLF
U+4CED.0	fFourCorner	27720
U+4CED.0	fCantonese	zik3; zik1; zit1
U+4CED.0	fMandarin	ji2
U+4CED.0	fEnglish	wagtail
U+4CEE.0	fUTF8	䳮
U+4CEE.0	fCNS11643	4-6562
U+4CEE.0	fGB	3-5D7C
U+4CEE.0	fR/S	 196.9
U+4CEE.0	fTotalStrokes	 20
U+4CEE.0	fCangjie	BEHAF
U+4CEE.0	fFourCorner	73320
U+4CEE.0	fCantonese	mak6; maak6; maat6
U+4CEE.0	fMandarin	mo4
U+4CEE.0	fEnglish	 look at in surprise (said of bird)
U+4CEF.0	fUTF8	䳯
U+4CEF.0	fCNS11643	4-656B
U+4CEF.0	fGB	3-5D7A
U+4CEF.0	fR/S	 196.9
U+4CEF.0	fTotalStrokes	 20
U+4CEF.0	fCangjie	HGHAF
U+4CEF.0	fFourCorner	27120
U+4CEF.0	fCantonese	cung4
U+4CEF.0	fMandarin	chong2; chong3
U+4CEF.0	fEnglish	[1] a kind of bird [2] small bird flying [3] a general name for small birds
U+4CF0.0	fUTF8	䳰
U+4CF0.0	fCNS11643	3-5D7E
U+4CF0.0	fR/S	 196.9; 196.18; 9.18
U+4CF0.0	fTotalStrokes	 20
U+4CF0.0	fCangjie	ORDF
U+4CF0.0	fFourCorner	27220
U+4CF0.0	fCantonese	bou2
U+4CF0.0	fMandarin	bao3
U+4CF0.0	fEnglish	same as &#x9D07;
U+4CF1.0	fUTF8	䳱
U+4CF1.0	fCNS11643	3-5F34
U+4CF1.0	fR/S	 196.10
U+4CF1.0	fTotalStrokes	 21
U+4CF1.0	fCangjie	NSHAF
U+4CF1.0	fFourCorner	17320
U+4CF1.0	fCantonese	mou6
U+4CF1.0	fMandarin	mu4; wu4
U+4CF1.0	fEnglish	non-classical form of &#x9DA9;
U+4CF2.0	fUTF8	䳲
U+4CF2.0	fCNS11643	4-6840
U+4CF2.0	fGB	5-5C47
U+4CF2.0	fR/S	 196.10
U+4CF2.0	fTotalStrokes	 21
U+4CF2.0	fCangjie	QVHAF
U+4CF2.0	fFourCorner	51320
U+4CF2.0	fCantonese	zan3
U+4CF2.0	fMandarin	zhen1; zhen4
U+4CF2.0	fEnglish	a swarm of flying birds; birds soar in groups
U+4CF3.0	fUTF8	䳳
U+4CF3.0	fCNS11643	4-6842
U+4CF3.0	fGB	3-5E2F
U+4CF3.0	fR/S	 196.10
U+4CF3.0	fTotalStrokes	 21
U+4CF3.0	fCangjie	UFHAF
U+4CF3.0	fFourCorner	27920
U+4CF3.0	fCantonese	seot1; hyut3
U+4CF3.0	fMandarin	xu4
U+4CF3.0	fEnglish	a kind of small bird, strange, weird bird
U+4CF4.0	fUTF8	䳴
U+4CF4.0	fCNS11643	4-6841
U+4CF4.0	fGB	3-5E26
U+4CF4.0	fR/S	 196.10
U+4CF4.0	fTotalStrokes	 21
U+4CF4.0	fCangjie	WEHAF
U+4CF4.0	fFourCorner	67120
U+4CF4.0	fCantonese	taap3; dap6
U+4CF4.0	fMandarin	ta4; da2
U+4CF4.0	fEnglish	[1] a kind of bird [2] a flying young bird
U+4CF5.0	fUTF8	䳵
U+4CF5.0	fCNS11643	5-7263
U+4CF5.0	fR/S	 196.10
U+4CF5.0	fTotalStrokes	 21
U+4CF5.0	fCangjie	DOKSF
U+4CF5.0	fFourCorner	47920
U+4CF5.0	fCantonese	cik1
U+4CF5.0	fMandarin	chi4
U+4CF5.0	fEnglish	a kind of water bird with colorful feather
U+4CF6.0	fUTF8	䳶
U+4CF6.0	fCNS11643	3-5F33
U+4CF6.0	fR/S	 196.10
U+4CF6.0	fTotalStrokes	 21
U+4CF6.0	fCangjie	HFBVK
U+4CF6.0	fFourCorner	22330
U+4CF6.0	fCantonese	kai1; gaai1; haai4
U+4CF6.0	fMandarin	qi1
U+4CF6.0	fEnglish	[1] same as &#x3D89; [2] same as &#x96DE;
U+4CF7.0	fUTF8	䳷
U+4CF7.0	fCNS11643	4-6A27
U+4CF7.0	fGB	3-5C7E
U+4CF7.0	fR/S	 196.11
U+4CF7.0	fTotalStrokes	 22; 21
U+4CF7.0	fCangjie	HOHAF
U+4CF7.0	fFourCorner	28320
U+4CF7.0	fCantonese	cung4
U+4CF7.0	fMandarin	cong2
U+4CF7.0	fEnglish	another name for chicken
U+4CF8.0	fUTF8	䳸
U+4CF8.0	fCNS11643	4-6A2C
U+4CF8.0	fGB	3-7C23
U+4CF8.0	fR/S	 196.11
U+4CF8.0	fTotalStrokes	 21; 22
U+4CF8.0	fCangjie	IDHAF
U+4CF8.0	fFourCorner	00220
U+4CF8.0	fHakka	 ma2; mai2
U+4CF8.0	fCantonese	maa4; mai4; min5
U+4CF8.0	fMandarin	ma2; mai2
U+4CF8.0	fEnglish	a kind of wild goose
U+4CF9.0	fUTF8	䳹
U+4CF9.0	fGB	3-5E34
U+4CF9.0	fR/S	 196.11
U+4CF9.0	fTotalStrokes	 22
U+4CF9.0	fCangjie	HFJME
U+4CF9.0	fFourCorner	2331
U+4CF9.0	fCantonese	kau3; kaau3; kaau5
U+4CF9.0	fMandarin	kou4
U+4CF9.0	fEnglish	[1] a kind dove-like bird [2] same as &#x9D7D; [3] same as &#x9CE9;
U+4CFA.0	fUTF8	䳺
U+4CFA.0	fCNS11643	3-6028
U+4CFA.0	fGB	5-5C51
U+4CFA.0	fR/S	 196.11
U+4CFA.0	fTotalStrokes	 22
U+4CFA.0	fCangjie	OWHAF
U+4CFA.0	fFourCorner	87620
U+4CFA.0	fCantonese	am1; ngam1 (also); aan3; ngaan3 (also); aam1; an3
U+4CFA.0	fMandarin	an1; yan4
U+4CFA.0	fEnglish	same as &#x9D6A;
U+4CFB.0	fUTF8	䳻
U+4CFB.0	fCNS11643	4-6A26
U+4CFB.0	fGB	3-5D5F
U+4CFB.0	fR/S	 196.11
U+4CFB.0	fTotalStrokes	 22; 21
U+4CFB.0	fCangjie	JLHAF
U+4CFB.0	fFourCorner	52320
U+4CFB.0	fCantonese	caam4
U+4CFB.0	fMandarin	can2; chan2; die2; zhan4
U+4CFB.0	fEnglish	[1] eagle; hawk [2] another name for osprey or water hawk [3] a kind of eagle-like bird with white pattern
U+4CFC.0	fUTF8	䳼
U+4CFC.0	fCNS11643	3-6026
U+4CFC.0	fGB	5-5B7A
U+4CFC.0	fR/S	 196.11
U+4CFC.0	fTotalStrokes	 22
U+4CFC.0	fCangjie	HFSRR
U+4CFC.0	fFourCorner	21310
U+4CFC.0	fCantonese	au1; ngau1 (also); aau1
U+4CFC.0	fMandarin	you1
U+4CFC.0	fEnglish	standard form of &#x9DD7;
U+4CFD.0	fUTF8	䳽
U+4CFD.0	fCNS11643	3-6027
U+4CFD.0	fR/S	 196.11
U+4CFD.0	fTotalStrokes	 22
U+4CFD.0	fCangjie	UGHAF
U+4CFD.0	fFourCorner	27220
U+4CFD.0	fCantonese	hok6
U+4CFD.0	fMandarin	he4
U+4CFD.0	fEnglish	[1] non-classical form of &#x9E16; [2] same as &#x9DB4;
U+4CFE.0	fUTF8	䳾
U+4CFE.0	fCNS11643	4-6B45
U+4CFE.0	fGB	3-5E39
U+4CFE.0	fR/S	 196.12
U+4CFE.0	fTotalStrokes	 23
U+4CFE.0	fCangjie	NTHAF
U+4CFE.0	fFourCorner	17120
U+4CFE.0	fHakka	 den1; zen1
U+4CFE.0	fCantonese	dang1
U+4CFE.0	fMandarin	deng1
U+4CFE.0	fEnglish	a long-legged marsh fowl, with a red crest
U+4CFF.0	fUTF8	䳿
U+4CFF.0	fCNS11643	3-606B
U+4CFF.0	fR/S	 196.12
U+4CFF.0	fTotalStrokes	 23
U+4CFF.0	fCangjie	BFHAF
U+4CFF.0	fFourCorner	27320
U+4CFF.0	fCantonese	jin4; zin4
U+4CFF.0	fMandarin	ran2; lan2
U+4CFF.0	fEnglish	a kind of bird
U+4D00.0	fUTF8	䴀
U+4D00.0	fCNS11643	5-772C
U+4D00.0	fR/S	 196.12
U+4D00.0	fTotalStrokes	 23
U+4D00.0	fCangjie	YGHAF
U+4D00.0	fFourCorner	07120
U+4D00.0	fHakka	 tung2
U+4D00.0	fCantonese	tung4; zung1
U+4D00.0	fMandarin	tong2
U+4D00.0	fEnglish	the hornbill
U+4D01.0	fUTF8	䴁
U+4D01.0	fCNS11643	4-6B44
U+4D01.0	fGB	3-5E38
U+4D01.0	fR/S	 196.12
U+4D01.0	fTotalStrokes	 23
U+4D01.0	fCangjie	JBHAF
U+4D01.0	fFourCorner	37420
U+4D01.0	fCantonese	jyu6; zyu6
U+4D01.0	fMandarin	yu4
U+4D01.0	fEnglish	[1] a kind of rat-like bird [2] a vulture or a condor-like bird
U+4D02.0	fUTF8	䴂
U+4D02.0	fCNS11643	5-7734
U+4D02.0	fR/S	 196.12
U+4D02.0	fTotalStrokes	 23
U+4D02.0	fCangjie	NOHAF
U+4D02.0	fFourCorner	27220
U+4D02.0	fHakka	 siong5; siong6
U+4D02.0	fCantonese	zoeng6
U+4D02.0	fMandarin	xiang4; hang4
U+4D02.0	fEnglish	the hornbill (Buceros) of Siam
U+4D03.0	fUTF8	䴃
U+4D03.0	fCNS11643	4-6B47
U+4D03.0	fGB	3-5D41
U+4D03.0	fR/S	 196.12
U+4D03.0	fTotalStrokes	 23
U+4D03.0	fCangjie	GUHAF
U+4D03.0	fFourCorner	47220
U+4D03.0	fCantonese	naau4; laau4 (also); nau4
U+4D03.0	fMandarin	nao2
U+4D03.0	fEnglish	[1] wren; the tailorbird [2] the golden oriole
U+4D04.0	fUTF8	䴄
U+4D04.0	fCNS11643	3-606A
U+4D04.0	fR/S	 196.12
U+4D04.0	fTotalStrokes	 23
U+4D04.0	fCangjie	BQHAF
U+4D04.0	fFourCorner	27220
U+4D04.0	fCantonese	seon3
U+4D04.0	fMandarin	shun4; li4
U+4D04.0	fEnglish	a kind of bird
U+4D05.0	fUTF8	䴅
U+4D05.0	fCNS11643	3-6069
U+4D05.0	fR/S	 196.12
U+4D05.0	fTotalStrokes	 23
U+4D05.0	fCangjie	JCHAF
U+4D05.0	fFourCorner	47827
U+4D05.0	fCantonese	fan4
U+4D05.0	fMandarin	ben1; fen2
U+4D05.0	fEnglish	a legendary bird, like magpie, with white body, red tail and six feet
U+4D06.0	fUTF8	䴆
U+4D06.0	fCNS11643	4-6B48
U+4D06.0	fGB	3-5E35
U+4D06.0	fR/S	 196.12
U+4D06.0	fTotalStrokes	 23
U+4D06.0	fCangjie	TOHAF
U+4D06.0	fFourCorner	37520
U+4D06.0	fCantonese	buk6
U+4D06.0	fMandarin	pao3; pu2
U+4D06.0	fEnglish	[1] a kind of bird [2] a kind of bird good at casting lots or divining
U+4D07.0	fUTF8	䴇
U+4D07.0	fCNS11643	4-6C50
U+4D07.0	fGB	3-5E3B
U+4D07.0	fR/S	 196.13
U+4D07.0	fTotalStrokes	 24
U+4D07.0	fCangjie	MIHAF
U+4D07.0	fFourCorner	17320
U+4D07.0	fCantonese	ling4
U+4D07.0	fMandarin	ling2
U+4D07.0	fEnglish	[1] [`] another name for crane [2] a small swan [3] non-classical form of &#x9D12;
U+4D08.0	fUTF8	䴈
U+4D08.0	fCNS11643	4-6C52
U+4D08.0	fGB	3-5E36
U+4D08.0	fR/S	 196.13
U+4D08.0	fTotalStrokes	 24
U+4D08.0	fCangjie	HKHAF
U+4D08.0	fFourCorner	27420
U+4D08.0	fCantonese	ou2; ngou2 (also); ou3 (also); ngou3 (also)
U+4D08.0	fMandarin	ao3
U+4D08.0	fEnglish	(non-classical form) a kind of water bird
U+4D09.0	fUTF8	䴉
U+4D09.0	fCNS11643	4-6C51
U+4D09.0	fGB	3-5E3E
U+4D09.0	fR/S	 196.13
U+4D09.0	fTotalStrokes	 24
U+4D09.0	fCangjie	WVHAF
U+4D09.0	fFourCorner	67720
U+4D09.0	fCantonese	syun4; waan4 (also)
U+4D09.0	fMandarin	xuan2
U+4D09.0	fEnglish	[1] general term for a group of bird (looks like egret) [2] fly in a circle
U+4D0A.0	fUTF8	䴊
U+4D0A.0	fCNS11643	3-6136
U+4D0A.0	fGB	5-5B78
U+4D0A.0	fR/S	 196.13
U+4D0A.0	fTotalStrokes	 24
U+4D0A.0	fCangjie	TIHAF
U+4D0A.0	fFourCorner	87520
U+4D0A.0	fCantonese	ji4; faan3; gon1; zi4; zi5
U+4D0A.0	fMandarin	yi2
U+4D0A.0	fEnglish	same as &#x9E03;
U+4D0B.0	fUTF8	䴋
U+4D0B.0	fCNS11643	4-6C54
U+4D0B.0	fGB	3-5E3D
U+4D0B.0	fR/S	 196.13
U+4D0B.0	fTotalStrokes	 24
U+4D0B.0	fCangjie	HFWLV
U+4D0B.0	fFourCorner	26330
U+4D0B.0	fCantonese	waan4; syun4
U+4D0B.0	fMandarin	xuan2
U+4D0B.0	fEnglish	[1] a kind of water bird, deep eyed [2] general term for a group of bird
U+4D0C.0	fUTF8	䴌
U+4D0C.0	fCNS11643	4-6D3D
U+4D0C.0	fGB	5-5C59
U+4D0C.0	fR/S	 196.14
U+4D0C.0	fTotalStrokes	 25
U+4D0C.0	fCangjie	HFTBO
U+4D0C.0	fFourCorner	24330
U+4D0C.0	fCantonese	mung4
U+4D0C.0	fMandarin	meng2
U+4D0C.0	fEnglish	a kind of water bird, a big gray-white colored body with hairy tail
U+4D0D.0	fUTF8	䴍
U+4D0D.0	fCNS11643	3-615C
U+4D0D.0	fR/S	 196.14
U+4D0D.0	fTotalStrokes	 25
U+4D0D.0	fCangjie	BCBCF
U+4D0D.0	fFourCorner	67820
U+4D0D.0	fCantonese	ang1; ngang1 (also)
U+4D0D.0	fMandarin	ying1
U+4D0D.0	fEnglish	a kind of strange and weird bird
U+4D0E.0	fUTF8	䴎
U+4D0E.0	fCNS11643	7-6337
U+4D0E.0	fR/S	 196.15
U+4D0E.0	fTotalStrokes	 26
U+4D0E.0	fCangjie	WWWF
U+4D0E.0	fFourCorner	60320
U+4D0E.0	fCantonese	leoi5
U+4D0E.0	fMandarin	lei3
U+4D0E.0	fEnglish	same as &#x9E13;
U+4D0F.0	fUTF8	䴏
U+4D0F.0	fCNS11643	3-622A
U+4D0F.0	fR/S	 196.16
U+4D0F.0	fTotalStrokes	 27
U+4D0F.0	fCangjie	TFHAF
U+4D0F.0	fFourCorner	47320
U+4D0F.0	fCantonese	jin3; zin3
U+4D0F.0	fMandarin	yan4
U+4D0F.0	fEnglish	same as &#x71D5;
U+4D10.0	fUTF8	䴐
U+4D10.0	fCNS11643	3-623E
U+4D10.0	fR/S	 196.20
U+4D10.0	fTotalStrokes	 31
U+4D10.0	fCangjie	JCHAF
U+4D10.0	fFourCorner	37820
U+4D10.0	fCantonese	bou2
U+4D10.0	fMandarin	bao3
U+4D10.0	fEnglish	[1] a kind of bird [2] same as &#x9D07;
U+4D11.0	fUTF8	䴑
U+4D11.0	fCNS11643	4-6E59
U+4D11.0	fGB	3-5E44
U+4D11.0	fR/S	 196.23
U+4D11.0	fTotalStrokes	 34
U+4D11.0	fCangjie	WMHAF
U+4D11.0	fFourCorner	67120
U+4D11.0	fCantonese	dip6
U+4D11.0	fMandarin	die2
U+4D11.0	fEnglish	[1] a legendary weird bird, resembling the magpie, dark red color, with two heads and four feet [2] corrupted form of &#x9E13;
U+4D12.0	fUTF8	䴒
U+4D12.0	fCNS11643	3-6243
U+4D12.0	fR/S	 196.24
U+4D12.0	fTotalStrokes	 35
U+4D12.0	fCangjie	MOHAF
U+4D12.0	fFourCorner	17120
U+4D12.0	fCantonese	ling4
U+4D12.0	fMandarin	ling2
U+4D12.0	fEnglish	same as &#x9D12;
U+4D13.0	fUTF8	䴓
U+4D13.0	fGB	7-2228
U+4D13.0	fR/S	 196.4
U+4D13.0	fTotalStrokes	 15
U+4D13.0	fCangjie	MBPYM
U+4D13.0	fCantonese	si1
U+4D13.0	fMandarin	shi1
U+4D13.0	fEnglish	simplified form of &#x9CFE;
U+4D14.0	fUTF8	䴔
U+4D14.0	fGB	7-2229
U+4D14.0	fR/S	 196.6
U+4D14.0	fTotalStrokes	 17
U+4D14.0	fCangjie	YKPYM
U+4D14.0	fCantonese	gaau1; gau1
U+4D14.0	fMandarin	jiao1
U+4D14.0	fEnglish	simplified form of &#x9D41;
U+4D15.0	fUTF8	䴕
U+4D15.0	fGB	7-222A
U+4D15.0	fR/S	 196.6
U+4D15.0	fTotalStrokes	 17
U+4D15.0	fCangjie	MNPYM
U+4D15.0	fCantonese	lit6
U+4D15.0	fMandarin	lie4
U+4D15.0	fEnglish	simplified form of &#x9D37;
U+4D16.0	fUTF8	䴖
U+4D16.0	fGB	7-222B
U+4D16.0	fR/S	 196.8
U+4D16.0	fTotalStrokes	 19
U+4D16.0	fCangjie	QBPYM
U+4D16.0	fCantonese	zing1
U+4D16.0	fMandarin	jing1
U+4D16.0	fEnglish	simplified form of &#x9D84 bird;
U+4D17.0	fUTF8	䴗
U+4D17.0	fGB	7-222C
U+4D17.0	fR/S	 196.9
U+4D17.0	fTotalStrokes	 20
U+4D17.0	fCangjie	BKPYM
U+4D17.0	fCantonese	gwik1; gwing2
U+4D17.0	fMandarin	ju2
U+4D17.0	fEnglish	simplified form of &#x4D17;
U+4D18.0	fUTF8	䴘
U+4D18.0	fGB	7-222D
U+4D18.0	fR/S	 196.10
U+4D18.0	fTotalStrokes	 21
U+4D18.0	fCangjie	HNPYM
U+4D18.0	fCantonese	tai1; taai1; taai2; taai3
U+4D18.0	fMandarin	ti1
U+4D18.0	fEnglish	simplified form of &#x9DC9;
U+4D19.0	fUTF8	䴙
U+4D19.0	fGB	7-222E
U+4D19.0	fR/S	 196.13
U+4D19.0	fTotalStrokes	 24
U+4D19.0	fCangjie	SRYJM
U+4D19.0	fCantonese	pik1
U+4D19.0	fMandarin	pi4
U+4D19.0	fEnglish	simplified form of &#x9E0A;
U+4D1A.0	fUTF8	䴚
U+4D1A.0	fCNS11643	4-4F47
U+4D1A.0	fGB	3-6F46
U+4D1A.0	fR/S	 197.4
U+4D1A.0	fTotalStrokes	 15
U+4D1A.0	fCangjie	YWYHN
U+4D1A.0	fFourCorner	20610
U+4D1A.0	fCantonese	gong2; gong1
U+4D1A.0	fMandarin	kang3; gang3
U+4D1A.0	fEnglish	saltpond; salty marshes
U+4D1B.0	fUTF8	䴛
U+4D1B.0	fCNS11643	4-5F39
U+4D1B.0	fGB	3-6F47
U+4D1B.0	fR/S	 197.7
U+4D1B.0	fTotalStrokes	 18
U+4D1B.0	fCangjie	YWFB
U+4D1B.0	fFourCorner	29620
U+4D1B.0	fCantonese	siu1
U+4D1B.0	fMandarin	xiao1; xiao4; jiao3; tu2; xi4; yin2
U+4D1B.0	fEnglish	 make decoction of salt
U+4D1C.0	fUTF8	䴜
U+4D1C.0	fCNS11643	7-5729
U+4D1C.0	fR/S	 197.10
U+4D1C.0	fTotalStrokes	 21
U+4D1C.0	fCangjie	YWHUI
U+4D1C.0	fFourCorner	26610
U+4D1C.0	fCantonese	waai1; gwaat3; nei6; wai4; wun6
U+4D1C.0	fMandarin	huai2
U+4D1C.0	fEnglish	salt; a certain minority group in ancient times used as another name for salt
U+4D1D.0	fUTF8	䴝
U+4D1D.0	fCNS11643	4-6B4D
U+4D1D.0	fGB	3-6F48
U+4D1D.0	fR/S	 197.12
U+4D1D.0	fTotalStrokes	 23
U+4D1D.0	fCangjie	YWASE
U+4D1D.0	fFourCorner	26640
U+4D1D.0	fCantonese	ceoi3; cai3; zyut3
U+4D1D.0	fMandarin	chuai4; bu4
U+4D1D.0	fEnglish	[1] salt [2] soybean sauce; soy
U+4D1E.0	fUTF8	䴞
U+4D1E.0	fCNS11643	5-7973
U+4D1E.0	fR/S	 197.14
U+4D1E.0	fTotalStrokes	 25
U+4D1E.0	fCangjie	YWSMG
U+4D1E.0	fFourCorner	27610
U+4D1E.0	fCantonese	dik6
U+4D1E.0	fMandarin	di2
U+4D1E.0	fEnglish	salty; briny; salted
U+4D1F.0	fUTF8	䴟
U+4D1F.0	fCNS11643	5-4B6D
U+4D1F.0	fGB	5-797B
U+4D1F.0	fR/S	 198.3
U+4D1F.0	fTotalStrokes	 14
U+4D1F.0	fCangjie	IPHNI
U+4D1F.0	fFourCorner	00210
U+4D1F.0	fCantonese	wun4
U+4D1F.0	fMandarin	huan2; huan4
U+4D1F.0	fEnglish	[1] an one-year old deer [2] a three-year old deer
U+4D20.0	fUTF8	䴠
U+4D20.0	fCNS11643	4-4F49
U+4D20.0	fGB	3-7C2D
U+4D20.0	fR/S	 198.4
U+4D20.0	fTotalStrokes	 15
U+4D20.0	fCangjie	IPHK
U+4D20.0	fFourCorner	00430
U+4D20.0	fCantonese	ou2; ngou2 (also); jiu2 (also); ziu1
U+4D20.0	fMandarin	ao3; yao3
U+4D20.0	fEnglish	a young tailed deer
U+4D21.0	fUTF8	䴡
U+4D21.0	fCNS11643	4-4F4A
U+4D21.0	fR/S	 198.4
U+4D21.0	fTotalStrokes	 15
U+4D21.0	fCangjie	MTCP
U+4D21.0	fFourCorner	10210
U+4D21.0	fCantonese	lai6
U+4D21.0	fMandarin	li4
U+4D21.0	fEnglish	same as &#x9E97;
U+4D22.0	fUTF8	䴢
U+4D22.0	fCNS11643	3-502D
U+4D22.0	fR/S	 198.4
U+4D22.0	fTotalStrokes	 15
U+4D22.0	fCangjie	IPD
U+4D22.0	fFourCorner	00290
U+4D22.0	fCantonese	mei4
U+4D22.0	fMandarin	jun1; mi2
U+4D22.0	fEnglish	same as &#x9E8B;
U+4D23.0	fUTF8	䴣
U+4D23.0	fCNS11643	3-5438
U+4D23.0	fR/S	 198.5
U+4D23.0	fTotalStrokes	 16
U+4D23.0	fCangjie	IPHFD
U+4D23.0	fFourCorner	02240
U+4D23.0	fCantonese	fu1
U+4D23.0	fMandarin	hu1
U+4D23.0	fEnglish	a kind of animal
U+4D24.0	fUTF8	䴤
U+4D24.0	fCNS11643	4-5558
U+4D24.0	fGB	3-7C30
U+4D24.0	fR/S	 198.5
U+4D24.0	fTotalStrokes	 16
U+4D24.0	fCangjie	IPHQM
U+4D24.0	fFourCorner	00214
U+4D24.0	fCantonese	sang1; saang1
U+4D24.0	fMandarin	sheng1
U+4D24.0	fEnglish	[1] a kind of animal, as big as a rabbit [2] a two-year old deer
U+4D25.0	fUTF8	䴥
U+4D25.0	fCNS11643	7-365C
U+4D25.0	fGB	3-7C32
U+4D25.0	fR/S	 198.5
U+4D25.0	fTotalStrokes	 16
U+4D25.0	fCangjie	IPKSR
U+4D25.0	fFourCorner	00260
U+4D25.0	fHakka	 ga1
U+4D25.0	fCantonese	gaa1
U+4D25.0	fMandarin	jia1
U+4D25.0	fEnglish	[1] buck [2] stag
U+4D26.0	fUTF8	䴦
U+4D26.0	fCNS11643	3-5A23
U+4D26.0	fR/S	 198.7
U+4D26.0	fTotalStrokes	 18
U+4D26.0	fCangjie	IPYMR
U+4D26.0	fFourCorner	00260
U+4D26.0	fCantonese	ngan4; an4 (also); ngaan4; ngai4
U+4D26.0	fMandarin	yin2; ren2
U+4D26.0	fEnglish	a legendary animal; a fox-like animal and with a man's eyes
U+4D27.0	fUTF8	䴧
U+4D27.0	fCNS11643	4-6272
U+4D27.0	fGB	3-7C34
U+4D27.0	fR/S	 198.8
U+4D27.0	fTotalStrokes	 19
U+4D27.0	fCangjie	IPHDV
U+4D27.0	fFourCorner	02240
U+4D27.0	fHakka	 wui1; wi1; vui1
U+4D27.0	fCantonese	wai1
U+4D27.0	fMandarin	wei2
U+4D27.0	fEnglish	the best cut of venison
U+4D28.0	fUTF8	䴨
U+4D28.0	fR/S	 198.10
U+4D28.0	fTotalStrokes	 21
U+4D28.0	fCangjie	IPMHF
U+4D28.0	fCantonese	jyun4
U+4D28.0	fEnglish	[Korean character]
U+4D29.0	fUTF8	䴩
U+4D29.0	fCNS11643	3-5F39
U+4D29.0	fR/S	 198.11
U+4D29.0	fTotalStrokes	 22; 21
U+4D29.0	fCangjie	IPMWF
U+4D29.0	fFourCorner	00290
U+4D29.0	fCantonese	piu4
U+4D29.0	fMandarin	piao2
U+4D29.0	fEnglish	a kind of deer
U+4D2A.0	fUTF8	䴪
U+4D2A.0	fCNS11643	3-6137
U+4D2A.0	fR/S	 198.13; 198.12
U+4D2A.0	fTotalStrokes	 24; 23
U+4D2A.0	fCangjie	IPIFE
U+4D2A.0	fFourCorner	00230
U+4D2A.0	fCantonese	luk6
U+4D2A.0	fMandarin	lu4
U+4D2A.0	fEnglish	a fabulous creature, something like a deer, with a single horn
U+4D2B.0	fUTF8	䴫
U+4D2B.0	fCNS11643	3-6138
U+4D2B.0	fGB	5-7A28
U+4D2B.0	fR/S	 198.13
U+4D2B.0	fTotalStrokes	 24
U+4D2B.0	fCangjie	IPMBI
U+4D2B.0	fFourCorner	01230
U+4D2B.0	fCantonese	ling4
U+4D2B.0	fMandarin	ling2
U+4D2B.0	fEnglish	same as &#x9EA2;, &#x7F9A;
U+4D2C.0	fUTF8	䴬
U+4D2C.0	fCNS11643	4-4873
U+4D2C.0	fGB	3-6D5A
U+4D2C.0	fR/S	 199.3
U+4D2C.0	fTotalStrokes	 14
U+4D2C.0	fCangjie	JEIP
U+4D2C.0	fFourCorner	43404
U+4D2C.0	fHakka	 jit8; rit8
U+4D2C.0	fCantonese	jik6; zik6; zim2
U+4D2C.0	fMandarin	yi4
U+4D2C.0	fEnglish	wheat which has been cleaned from chaff
U+4D2D.0	fUTF8	䴭
U+4D2D.0	fCNS11643	4-4874
U+4D2D.0	fGB	3-6D59
U+4D2D.0	fR/S	 199.3
U+4D2D.0	fTotalStrokes	 14
U+4D2D.0	fCangjie	JEDH
U+4D2D.0	fFourCorner	44240
U+4D2D.0	fCantonese	coi4; co3
U+4D2D.0	fMandarin	cai2
U+4D2D.0	fEnglish	distiller's grains or yeast; ferment for brewing
U+4D2E.0	fUTF8	䴮
U+4D2E.0	fCNS11643	3-4B21
U+4D2E.0	fR/S	 199.3
U+4D2E.0	fTotalStrokes	 14
U+4D2E.0	fCangjie	JEU
U+4D2E.0	fFourCorner	42407
U+4D2E.0	fCantonese	saan3
U+4D2E.0	fMandarin	shan4
U+4D2E.0	fEnglish	[1] cakes of leaven; yeast [2] ferment for brewing [3] crumbs of barley
U+4D2F.0	fUTF8	䴯
U+4D2F.0	fCNS11643	4-4F4D
U+4D2F.0	fGB	3-6D5D
U+4D2F.0	fR/S	 199.4
U+4D2F.0	fTotalStrokes	 15
U+4D2F.0	fCangjie	JEPHH
U+4D2F.0	fFourCorner	47400
U+4D2F.0	fCantonese	fat1
U+4D2F.0	fMandarin	hu1
U+4D2F.0	fEnglish	a kind of cake
U+4D30.0	fUTF8	䴰
U+4D30.0	fCNS11643	4-4F4B
U+4D30.0	fGB	3-6D5C
U+4D30.0	fR/S	 199.4
U+4D30.0	fTotalStrokes	 15
U+4D30.0	fCangjie	JEI
U+4D30.0	fFourCorner	43405
U+4D30.0	fCantonese	suk6
U+4D30.0	fMandarin	shu1; shu2
U+4D30.0	fEnglish	[1] a family name [2] same as &#x4D2C;
U+4D31.0	fUTF8	䴱
U+4D31.0	fCNS11643	4-5559
U+4D31.0	fGB	3-6D63
U+4D31.0	fR/S	 199.5
U+4D31.0	fTotalStrokes	 16
U+4D31.0	fCangjie	JEJP
U+4D31.0	fFourCorner	43401
U+4D31.0	fHakka	 to2
U+4D31.0	fCantonese	to1; do2; fo1; to4
U+4D31.0	fMandarin	tuo1; tuo2
U+4D31.0	fEnglish	cakes made of bean flour
U+4D32.0	fUTF8	䴲
U+4D32.0	fCNS11643	4-555C
U+4D32.0	fGB	3-6D5E
U+4D32.0	fR/S	 199.5
U+4D32.0	fTotalStrokes	 16
U+4D32.0	fCangjie	JEDJ
U+4D32.0	fFourCorner	45400
U+4D32.0	fHakka	 mat8
U+4D32.0	fCantonese	mut6
U+4D32.0	fMandarin	mo4
U+4D32.0	fEnglish	broken grain
U+4D33.0	fUTF8	䴳
U+4D33.0	fCNS11643	4-555A
U+4D33.0	fGB	3-6D62
U+4D33.0	fR/S	 199.5
U+4D33.0	fTotalStrokes	 16
U+4D33.0	fCangjie	JEJC
U+4D33.0	fFourCorner	43240
U+4D33.0	fCantonese	waat6; wat6; wok6
U+4D33.0	fMandarin	hua2; he4
U+4D33.0	fEnglish	distiller's grains or yeast
U+4D34.0	fUTF8	䴴
U+4D34.0	fCNS11643	4-555D
U+4D34.0	fGB	3-6D61
U+4D34.0	fR/S	 199.5
U+4D34.0	fTotalStrokes	 16
U+4D34.0	fCangjie	JEYR
U+4D34.0	fFourCorner	41260
U+4D34.0	fCantonese	tip3
U+4D34.0	fMandarin	tie4
U+4D34.0	fEnglish	[1] a kind of cake [2] a kind of herb medicine [3] same as &#x9ECF;
U+4D35.0	fUTF8	䴵
U+4D35.0	fCNS11643	3-574F
U+4D35.0	fGB	5-6D25
U+4D35.0	fR/S	 199.6
U+4D35.0	fTotalStrokes	 17
U+4D35.0	fCangjie	JETT
U+4D35.0	fFourCorner	48240
U+4D35.0	fHakka	 biang3
U+4D35.0	fCantonese	bing2
U+4D35.0	fMandarin	bing3; zhuo2
U+4D35.0	fEnglish	same as &#x9905;
U+4D36.0	fUTF8	䴶
U+4D36.0	fCNS11643	4-5F3C
U+4D36.0	fGB	5-6D27
U+4D36.0	fR/S	 199.7
U+4D36.0	fTotalStrokes	 18
U+4D36.0	fCangjie	JEHEJ
U+4D36.0	fFourCorner	47400
U+4D36.0	fCantonese	pung4; leot6; miu4
U+4D36.0	fMandarin	peng2
U+4D36.0	fEnglish	same as &#x9EB7;
U+4D37.0	fUTF8	䴷
U+4D37.0	fCNS11643	4-5F3B
U+4D37.0	fGB	5-6D28
U+4D37.0	fR/S	 199.7
U+4D37.0	fTotalStrokes	 18
U+4D37.0	fCangjie	JEJMU
U+4D37.0	fFourCorner	43401
U+4D37.0	fCantonese	wan4
U+4D37.0	fMandarin	hun2; hun3; huan2; huan3
U+4D37.0	fEnglish	use the whole piece of barley to ferment for brewing
U+4D38.0	fUTF8	䴸
U+4D38.0	fCNS11643	3-5A27
U+4D38.0	fR/S	 199.7
U+4D38.0	fTotalStrokes	 18
U+4D38.0	fCangjie	JEBND
U+4D38.0	fFourCorner	42240
U+4D38.0	fCantonese	fu1
U+4D38.0	fMandarin	fu1
U+4D38.0	fEnglish	non-classical form of &#x9EA9;
U+4D39.0	fUTF8	䴹
U+4D39.0	fCNS11643	4-6279
U+4D39.0	fGB	3-6D67
U+4D39.0	fR/S	 199.8
U+4D39.0	fTotalStrokes	 19
U+4D39.0	fCangjie	JEWD
U+4D39.0	fFourCorner	46409
U+4D39.0	fHakka	 go3; kwo3
U+4D39.0	fCantonese	gwo2; go2 (also)
U+4D39.0	fMandarin	guo3; hua4; luo3; hun2
U+4D39.0	fEnglish	cakes; pastry; biscuits
U+4D3A.0	fUTF8	䴺
U+4D3A.0	fCNS11643	4-6276
U+4D3A.0	fGB	3-6D6A
U+4D3A.0	fR/S	 199.8
U+4D3A.0	fTotalStrokes	 19
U+4D3A.0	fCangjie	JEYTR
U+4D3A.0	fFourCorner	40406
U+4D3A.0	fCantonese	bau6; bou6
U+4D3A.0	fMandarin	bu4; cai3; chan4
U+4D3A.0	fEnglish	cakes; cookies; biscuits; pastries; dumplings
U+4D3B.0	fUTF8	䴻
U+4D3B.0	fCNS11643	4-627D
U+4D3B.0	fGB	3-6D68
U+4D3B.0	fR/S	 199.8
U+4D3B.0	fTotalStrokes	 19
U+4D3B.0	fCangjie	HHJON
U+4D3B.0	fFourCorner	27407
U+4D3B.0	fCantonese	lei4
U+4D3B.0	fMandarin	li2
U+4D3B.0	fEnglish	dry food
U+4D3C.0	fUTF8	䴼
U+4D3C.0	fCNS11643	4-6277
U+4D3C.0	fGB	3-6D5F
U+4D3C.0	fR/S	 199.8
U+4D3C.0	fTotalStrokes	 19
U+4D3C.0	fCangjie	JEII
U+4D3C.0	fFourCorner	43405
U+4D3C.0	fCantonese	caan3; can2
U+4D3C.0	fMandarin	chan3; chan4
U+4D3C.0	fEnglish	 extirpate; level off; pare barley
U+4D3D.0	fUTF8	䴽
U+4D3D.0	fCNS11643	4-627C
U+4D3D.0	fGB	3-6D69
U+4D3D.0	fR/S	 199.8
U+4D3D.0	fTotalStrokes	 19
U+4D3D.0	fCangjie	JEHWJ
U+4D3D.0	fFourCorner	46404
U+4D3D.0	fCantonese	pei4; bei1; bik1
U+4D3D.0	fMandarin	pi2; bai4
U+4D3D.0	fEnglish	a small round piece of distiller's grains or yeast
U+4D3E.0	fUTF8	䴾
U+4D3E.0	fCNS11643	4-6849
U+4D3E.0	fGB	3-6D6B
U+4D3E.0	fR/S	 199.10
U+4D3E.0	fTotalStrokes	 21
U+4D3E.0	fCangjie	JETQM
U+4D3E.0	fFourCorner	48401
U+4D3E.0	fCantonese	co4; zi2
U+4D3E.0	fMandarin	cuo2; cuo4; ye4; zhen3; zi3
U+4D3E.0	fEnglish	to grind barley
U+4D3F.0	fUTF8	䴿
U+4D3F.0	fCNS11643	4-6848
U+4D3F.0	fGB	5-6D2E
U+4D3F.0	fR/S	 199.10
U+4D3F.0	fTotalStrokes	 21
U+4D3F.0	fCangjie	JEBMO
U+4D3F.0	fFourCorner	47403
U+4D3F.0	fCantonese	mung4
U+4D3F.0	fMandarin	meng2
U+4D3F.0	fEnglish	grains from the distillery, crumbs of barley, crumbs of rice
U+4D40.0	fUTF8	䵀
U+4D40.0	fCNS11643	4-684A
U+4D40.0	fGB	3-6D66
U+4D40.0	fR/S	 199.10
U+4D40.0	fTotalStrokes	 21
U+4D40.0	fCangjie	JEFBC
U+4D40.0	fFourCorner	49408
U+4D40.0	fCantonese	so2
U+4D40.0	fMandarin	suo3
U+4D40.0	fEnglish	[1] coarse crumbs of barley [2] unrefined or unpolished wheat
U+4D41.0	fUTF8	䵁
U+4D41.0	fCNS11643	5-754D
U+4D41.0	fR/S	 199.11
U+4D41.0	fTotalStrokes	 22
U+4D41.0	fCangjie	JEVMI
U+4D41.0	fFourCorner	47400
U+4D41.0	fHakka	 ziong1; cong5
U+4D41.0	fCantonese	coeng3; coeng1
U+4D41.0	fMandarin	qiang4
U+4D41.0	fEnglish	paste; starch
U+4D42.0	fUTF8	䵂
U+4D42.0	fCNS11643	5-7548
U+4D42.0	fR/S	 199.11
U+4D42.0	fTotalStrokes	 22
U+4D42.0	fCangjie	JEYCB
U+4D42.0	fFourCorner	40402
U+4D42.0	fCantonese	zik6; zak6
U+4D42.0	fMandarin	zhi2; di2; zhe2
U+4D42.0	fEnglish	to grind wheat to powder; with bran and flour mixed together
U+4D43.0	fUTF8	䵃
U+4D43.0	fCNS11643	4-6B4E
U+4D43.0	fGB	5-6D2F
U+4D43.0	fR/S	 199.12
U+4D43.0	fTotalStrokes	 23
U+4D43.0	fCangjie	JETMC
U+4D43.0	fFourCorner	44400
U+4D43.0	fCantonese	kwong3; kong3 (also); wong4
U+4D43.0	fMandarin	kuang4; huang2
U+4D43.0	fEnglish	a kind of barley, dust of ferment for brewing; small crumbs of grains from the distillery
U+4D44.0	fUTF8	䵄
U+4D44.0	fCNS11643	3-6139
U+4D44.0	fR/S	 199.13
U+4D44.0	fTotalStrokes	 24
U+4D44.0	fCangjie	JETWT
U+4D44.0	fFourCorner	45401
U+4D44.0	fCantonese	fung1
U+4D44.0	fMandarin	feng1; bi2; bo2; feng4; peng3
U+4D44.0	fEnglish	[1] same as &#x9EB7; [2] interchangeable with &#x8C50;
U+4D45.0	fUTF8	䵅
U+4D45.0	fCNS11643	7-5A7D
U+4D45.0	fR/S	 199.11
U+4D45.0	fTotalStrokes	 22
U+4D45.0	fCangjie	QKJON
U+4D45.0	fFourCorner	48403
U+4D45.0	fCantonese	ngaau4; aau4 (also); ngou4
U+4D45.0	fMandarin	ao2
U+4D45.0	fEnglish	same as &#x71AC;
U+4D46.0	fUTF8	䵆
U+4D46.0	fCNS11643	4-6D41
U+4D46.0	fGB	3-6D6C
U+4D46.0	fR/S	 199.14
U+4D46.0	fTotalStrokes	 25
U+4D46.0	fCangjie	JETBO
U+4D46.0	fFourCorner	44230
U+4D46.0	fCantonese	mung4
U+4D46.0	fMandarin	meng2
U+4D46.0	fEnglish	same as &#x4D3F;
U+4D47.0	fUTF8	䵇
U+4D47.0	fCNS11643	7-2C31
U+4D47.0	fR/S	 200.4
U+4D47.0	fTotalStrokes	 15
U+4D47.0	fCangjie	IDOK
U+4D47.0	fFourCorner	08240
U+4D47.0	fCantonese	sin3; sam1
U+4D47.0	fMandarin	san1; xian4
U+4D47.0	fEnglish	[1] dislike; reject; hate [2] house; building
U+4D48.0	fUTF8	䵈
U+4D48.0	fCNS11643	4-6571
U+4D48.0	fGB	3-7C28
U+4D48.0	fR/S	 200.9
U+4D48.0	fTotalStrokes	 20
U+4D48.0	fCangjie	HEIDD
U+4D48.0	fFourCorner	22290
U+4D48.0	fCantonese	huk1; guk1
U+4D48.0	fMandarin	ku1
U+4D48.0	fEnglish	[1] unsmelted hemp and silk [2] unspinned hemp [3] raw silk
U+4D49.0	fUTF8	䵉
U+4D49.0	fCNS11643	4-6572
U+4D49.0	fGB	3-7C29
U+4D49.0	fR/S	 200.9
U+4D49.0	fTotalStrokes	 20
U+4D49.0	fCangjie	IDOMN
U+4D49.0	fFourCorner	00220
U+4D49.0	fCantonese	tau4
U+4D49.0	fMandarin	tou2
U+4D49.0	fEnglish	Abutilon, grassy plant with fabres from which clothe is made
U+4D4A.0	fUTF8	䵊
U+4D4A.0	fCNS11643	7-3665
U+4D4A.0	fR/S	 201.4
U+4D4A.0	fTotalStrokes	 16
U+4D4A.0	fCangjie	BYTMC
U+4D4A.0	fFourCorner	74480
U+4D4A.0	fCantonese	tan1; teon1; zyun1
U+4D4A.0	fMandarin	tuan1
U+4D4A.0	fEnglish	same as &#x4D4D;
U+4D4B.0	fUTF8	䵋
U+4D4B.0	fCNS11643	4-5F3D
U+4D4B.0	fGB	3-7B78
U+4D4B.0	fR/S	 201.6
U+4D4B.0	fTotalStrokes	 18
U+4D4B.0	fCangjie	TCKB
U+4D4B.0	fFourCorner	44820
U+4D4B.0	fCantonese	wai5; fui2; kui2
U+4D4B.0	fMandarin	hui3; wei3; hui4
U+4D4B.0	fEnglish	[1] yellow [2] greenish yellow
U+4D4C.0	fUTF8	䵌
U+4D4C.0	fCNS11643	4-6321
U+4D4C.0	fGB	3-7B79
U+4D4C.0	fR/S	 201.7
U+4D4C.0	fTotalStrokes	 19
U+4D4C.0	fCangjie	TCKOO
U+4D4C.0	fFourCorner	44830
U+4D4C.0	fCantonese	him1; cim1
U+4D4C.0	fMandarin	xian1
U+4D4C.0	fEnglish	reddish yellow
U+4D4D.0	fUTF8	䵍
U+4D4D.0	fCNS11643	4-6576
U+4D4D.0	fGB	3-7B7B
U+4D4D.0	fR/S	 201.8
U+4D4D.0	fTotalStrokes	 20
U+4D4D.0	fCangjie	TCYRD
U+4D4D.0	fFourCorner	40840
U+4D4D.0	fCantonese	tan1
U+4D4D.0	fMandarin	sun1
U+4D4D.0	fEnglish	[1] yellow [2] used in person's name
U+4D4E.0	fUTF8	䵎
U+4D4E.0	fCNS11643	4-684B
U+4D4E.0	fGB	3-7B7D
U+4D4E.0	fR/S	 201.9
U+4D4E.0	fTotalStrokes	 21
U+4D4E.0	fCangjie	TCUMB
U+4D4E.0	fFourCorner	42820
U+4D4E.0	fCantonese	teon1; caau3; doi6; sou1; toi5; zyun1
U+4D4E.0	fMandarin	tuan1
U+4D4E.0	fEnglish	[1] dark yellow [2] yellow [3] bright; well-illuminated
U+4D4F.0	fUTF8	䵏
U+4D4F.0	fCNS11643	4-6B52
U+4D4F.0	fGB	3-7B7E
U+4D4F.0	fR/S	 201.11
U+4D4F.0	fTotalStrokes	 23
U+4D4F.0	fCangjie	TCSMH
U+4D4F.0	fFourCorner	47820
U+4D4F.0	fCantonese	lou6; lou5
U+4D4F.0	fMandarin	lao3; lao4
U+4D4F.0	fEnglish	yellow; straw colord
U+4D50.0	fUTF8	䵐
U+4D50.0	fCNS11643	4-6C5A
U+4D50.0	fGB	3-7B7C
U+4D50.0	fR/S	 201.12
U+4D50.0	fTotalStrokes	 24
U+4D50.0	fCangjie	TCRRJ
U+4D50.0	fFourCorner	46850
U+4D50.0	fCantonese	cin2; zin2
U+4D50.0	fMandarin	chan3; chan4
U+4D50.0	fEnglish	[1] yellow [2] yellowish black; dark yellow
U+4D51.0	fUTF8	䵑
U+4D51.0	fCNS11643	7-2C33
U+4D51.0	fR/S	 202.3
U+4D51.0	fTotalStrokes	 15
U+4D51.0	fCangjie	HESHI
U+4D51.0	fFourCorner	27120
U+4D51.0	fHakka	 nit8
U+4D51.0	fCantonese	nik1; lik1 (also)
U+4D51.0	fMandarin	ni4
U+4D51.0	fEnglish	glue; cement
U+4D52.0	fUTF8	䵒
U+4D52.0	fCNS11643	4-555E
U+4D52.0	fGB	3-7C53
U+4D52.0	fR/S	 202.4
U+4D52.0	fTotalStrokes	 16
U+4D52.0	fCangjie	HEA
U+4D52.0	fFourCorner	26100
U+4D52.0	fHakka	 nit8
U+4D52.0	fCantonese	nik1; lik1 (also)
U+4D52.0	fMandarin	ni4; ri4
U+4D52.0	fEnglish	glue; cement
U+4D53.0	fUTF8	䵓
U+4D53.0	fCNS11643	7-3667
U+4D53.0	fR/S	 202.4
U+4D53.0	fTotalStrokes	 16
U+4D53.0	fCangjie	HEHQ
U+4D53.0	fFourCorner	25100
U+4D53.0	fCantonese	syu2; laai4
U+4D53.0	fMandarin	li2
U+4D53.0	fEnglish	the mole
U+4D54.0	fUTF8	䵔
U+4D54.0	fCNS11643	5-6F52
U+4D54.0	fGB	3-7C54
U+4D54.0	fR/S	 202.8
U+4D54.0	fTotalStrokes	 20
U+4D54.0	fCangjie	HEDW
U+4D54.0	fFourCorner	25190
U+4D54.0	fCantonese	dung2
U+4D54.0	fMandarin	dong3
U+4D54.0	fEnglish	[1] turn on the heel [2] not moving forward
U+4D55.0	fUTF8	䵕
U+4D55.0	fCNS11643	4-6579
U+4D55.0	fGB	3-7C57
U+4D55.0	fR/S	 202.8
U+4D55.0	fTotalStrokes	 20
U+4D55.0	fCangjie	HESJR
U+4D55.0	fFourCorner	27160
U+4D55.0	fCantonese	geoi3
U+4D55.0	fMandarin	ju4
U+4D55.0	fEnglish	[1] a variety of millet [2] stick [3] sticky; glutinous
U+4D56.0	fUTF8	䵖
U+4D56.0	fCNS11643	4-684F
U+4D56.0	fGB	5-7A50
U+4D56.0	fR/S	 202.8
U+4D56.0	fTotalStrokes	 20; 21
U+4D56.0	fCangjie	SEHDE
U+4D56.0	fFourCorner	77130
U+4D56.0	fCantonese	hin3; gin3
U+4D56.0	fMandarin	jian4; qian1; qian4; xian4
U+4D56.0	fEnglish	[1] panicled millet [2] congee; porridge; rice gruel [3] ripening of paddy or rice [4] harvest
U+4D57.0	fUTF8	䵗
U+4D57.0	fCNS11643	4-684C
U+4D57.0	fGB	3-7C58
U+4D57.0	fR/S	 202.9
U+4D57.0	fTotalStrokes	 21
U+4D57.0	fCangjie	HEMRW
U+4D57.0	fFourCorner	21160
U+4D57.0	fCantonese	baak6; fuk6
U+4D57.0	fMandarin	fu1; fu2; bi1
U+4D57.0	fEnglish	[1] clean the withered; dried and decayed leaves of the bottom part of millets [2] grains [3] beans or peas [4] a variety of millet [5] beans and peas collectively [6] paste up; attach to; stick up; glue
U+4D58.0	fUTF8	䵘
U+4D58.0	fCNS11643	4-684E
U+4D58.0	fGB	3-7C52
U+4D58.0	fR/S	 202.9
U+4D58.0	fTotalStrokes	 21
U+4D58.0	fCangjie	HEAMH
U+4D58.0	fFourCorner	26120
U+4D58.0	fCantonese	saai3; sai3; sam2
U+4D58.0	fMandarin	shai4; sha4
U+4D58.0	fEnglish	[1] doesn't t stick to each other; hard to adhesive [2] same as &#x66EC;
U+4D59.0	fUTF8	䵙
U+4D59.0	fCNS11643	4-684D
U+4D59.0	fGB	3-7C59
U+4D59.0	fR/S	 202.9
U+4D59.0	fTotalStrokes	 21
U+4D59.0	fCangjie	HEKNN
U+4D59.0	fFourCorner	24120
U+4D59.0	fCantonese	zaa1
U+4D59.0	fMandarin	zha4; zha3
U+4D59.0	fEnglish	[1] sticky; append; padding; cushioning, [2] paste on something
U+4D5A.0	fUTF8	䵚
U+4D5A.0	fCNS11643	4-6A31
U+4D5A.0	fGB	5-7A54
U+4D5A.0	fR/S	 202.10
U+4D5A.0	fTotalStrokes	 22; 21
U+4D5A.0	fCangjie	HEBHX
U+4D5A.0	fFourCorner	22170
U+4D5A.0	fCantonese	tou2; ceoi4; dou3; saam2
U+4D5A.0	fMandarin	tao3
U+4D5A.0	fEnglish	kaoliang; sorghum
U+4D5B.0	fUTF8	䵛
U+4D5B.0	fCNS11643	4-6B55
U+4D5B.0	fGB	3-7C56
U+4D5B.0	fR/S	 202.11
U+4D5B.0	fTotalStrokes	 23
U+4D5B.0	fCangjie	HESEG
U+4D5B.0	fFourCorner	27110
U+4D5B.0	fCantonese	hin3; gin3; hin1
U+4D5B.0	fMandarin	jian4; xian4
U+4D5B.0	fEnglish	non-classical form of &#x4D56;
U+4D5C.0	fUTF8	䵜
U+4D5C.0	fCNS11643	4-6D42
U+4D5C.0	fGB	3-7C55
U+4D5C.0	fR/S	 202.13
U+4D5C.0	fTotalStrokes	 25
U+4D5C.0	fCangjie	HETWV
U+4D5C.0	fFourCorner	25130
U+4D5C.0	fCantonese	nung5; lung5 (also); nung2
U+4D5C.0	fMandarin	nong3
U+4D5C.0	fEnglish	[1] plough and sow [2] a kind of fruit
U+4D5D.0	fUTF8	䵝
U+4D5D.0	fCNS11643	4-422C
U+4D5D.0	fGB	5-7A29
U+4D5D.0	fR/S	 203.1
U+4D5D.0	fTotalStrokes	 13
U+4D5D.0	fCangjie	WFU
U+4D5D.0	fFourCorner	62310
U+4D5D.0	fCantonese	ji3; zi3
U+4D5D.0	fMandarin	yi4; ya4; ai1; zha2; yin1
U+4D5D.0	fEnglish	[1] dark black [2] black
U+4D5E.0	fUTF8	䵞
U+4D5E.0	fCNS11643	3-4B23
U+4D5E.0	fR/S	 203.2
U+4D5E.0	fTotalStrokes	 14
U+4D5E.0	fCangjie	WFLN
U+4D5E.0	fFourCorner	62300
U+4D5E.0	fCantonese	king4
U+4D5E.0	fMandarin	jing1; jing4; qing2
U+4D5E.0	fEnglish	ancient form of &#x9EE5;
U+4D5F.0	fUTF8	䵟
U+4D5F.0	fCNS11643	4-4F52
U+4D5F.0	fGB	5-7A2A
U+4D5F.0	fR/S	 203.3
U+4D5F.0	fTotalStrokes	 15
U+4D5F.0	fCangjie	WFMJ
U+4D5F.0	fFourCorner	61340
U+4D5F.0	fHakka	 gon5
U+4D5F.0	fCantonese	gon2
U+4D5F.0	fMandarin	gan3
U+4D5F.0	fEnglish	black spots on the face as from age or underfed
U+4D60.0	fUTF8	䵠
U+4D60.0	fCNS11643	4-4F53
U+4D60.0	fGB	3-7C3F
U+4D60.0	fR/S	 203.3
U+4D60.0	fTotalStrokes	 15; 17
U+4D60.0	fCangjie	WFPI
U+4D60.0	fFourCorner	67320
U+4D60.0	fCantonese	dik1; coek3; zoek3
U+4D60.0	fMandarin	di2; zhuo2
U+4D60.0	fEnglish	[1] rushes used form making mats [2] face decorations (for women) [3] ornament on the face of a woman
U+4D61.0	fUTF8	䵡
U+4D61.0	fCNS11643	4-555F
U+4D61.0	fGB	5-7A2E
U+4D61.0	fR/S	 203.4
U+4D61.0	fTotalStrokes	 16
U+4D61.0	fCangjie	MKWGF
U+4D61.0	fFourCorner	20330
U+4D61.0	fCantonese	jin6; gan2
U+4D61.0	fMandarin	jian3
U+4D61.0	fEnglish	non-classical form of &#x4D64;
U+4D62.0	fUTF8	䵢
U+4D62.0	fCNS11643	4-5A5F
U+4D62.0	fGB	5-7A31
U+4D62.0	fR/S	 203.5
U+4D62.0	fTotalStrokes	 17; 16
U+4D62.0	fCangjie	WFJD
U+4D62.0	fFourCorner	65390
U+4D62.0	fCantonese	mui6; maa3; maa6; mei6
U+4D62.0	fMandarin	mei4; wei4
U+4D62.0	fEnglish	[1] light black [2] dark black
U+4D63.0	fUTF8	䵣
U+4D63.0	fCNS11643	5-6123
U+4D63.0	fGB	3-7C45
U+4D63.0	fR/S	 203.5
U+4D63.0	fTotalStrokes	 17
U+4D63.0	fCangjie	WFAM
U+4D63.0	fFourCorner	66310
U+4D63.0	fCantonese	daat3
U+4D63.0	fMandarin	da2; zhan3
U+4D63.0	fEnglish	[1] white with black; black and beautiful [2] the name of a county in ancient times
U+4D64.0	fUTF8	䵤
U+4D64.0	fCNS11643	5-6740
U+4D64.0	fGB	5-7A2D
U+4D64.0	fR/S	 203.6
U+4D64.0	fTotalStrokes	 18
U+4D64.0	fCangjie	MTWGF
U+4D64.0	fFourCorner	11330
U+4D64.0	fCantonese	gin2; gan2
U+4D64.0	fMandarin	jian3; xian4; dian3
U+4D64.0	fEnglish	non-classical form of &#x4D64;
U+4D65.0	fUTF8	䵥
U+4D65.0	fCNS11643	5-673D
U+4D65.0	fR/S	 203.6
U+4D65.0	fTotalStrokes	 18
U+4D65.0	fCangjie	WFNAU
U+4D65.0	fFourCorner	67310
U+4D65.0	fCantonese	wat1
U+4D65.0	fMandarin	yu4; she4; wan2; ying4
U+4D65.0	fEnglish	[1] black [2] dark; deep
U+4D66.0	fUTF8	䵦
U+4D66.0	fCNS11643	4-5F3F
U+4D66.0	fGB	5-7A36
U+4D66.0	fR/S	 203.6
U+4D66.0	fTotalStrokes	 18
U+4D66.0	fCangjie	WFEMD
U+4D66.0	fFourCorner	61340
U+4D66.0	fCantonese	ze6; ce5
U+4D66.0	fMandarin	xie4
U+4D66.0	fEnglish	non-classical form of &#x6C59;, &#x6C61;
U+4D67.0	fUTF8	䵧
U+4D67.0	fCNS11643	5-6741
U+4D67.0	fR/S	 203.6
U+4D67.0	fTotalStrokes	 18
U+4D67.0	fCangjie	JIWGF
U+4D67.0	fFourCorner	43350
U+4D67.0	fCantonese	zoi3
U+4D67.0	fMandarin	zai4
U+4D67.0	fEnglish	 dye
U+4D68.0	fUTF8	䵨
U+4D68.0	fCNS11643	4-6323
U+4D68.0	fGB	3-7C46
U+4D68.0	fR/S	 203.7
U+4D68.0	fTotalStrokes	 19
U+4D68.0	fCangjie	WFIKH
U+4D68.0	fFourCorner	63310
U+4D68.0	fHakka	 mang2
U+4D68.0	fCantonese	mong4
U+4D68.0	fMandarin	mang2
U+4D68.0	fEnglish	[1] dark; dim; obscure [2] personal secrets
U+4D69.0	fUTF8	䵩
U+4D69.0	fCNS11643	3-5C2B
U+4D69.0	fR/S	 203.7
U+4D69.0	fTotalStrokes	 19
U+4D69.0	fCangjie	HNWGF
U+4D69.0	fFourCorner	22330
U+4D69.0	fCantonese	lai4
U+4D69.0	fMandarin	li2
U+4D69.0	fEnglish	non-classical form of &#x9EE7;
U+4D6A.0	fUTF8	䵪
U+4D6A.0	fCNS11643	4-657C
U+4D6A.0	fGB	5-7A3B
U+4D6A.0	fR/S	 203.8
U+4D6A.0	fTotalStrokes	 20
U+4D6A.0	fCangjie	WFAPP
U+4D6A.0	fFourCorner	66310
U+4D6A.0	fCantonese	gwan3
U+4D6A.0	fMandarin	gun4
U+4D6A.0	fEnglish	[1] pure black [2] omit [3] forget [4] wonder about doing nothing
U+4D6B.0	fUTF8	䵫
U+4D6B.0	fCNS11643	4-6621
U+4D6B.0	fGB	5-7A3D
U+4D6B.0	fR/S	 203.8
U+4D6B.0	fTotalStrokes	 20
U+4D6B.0	fCangjie	JNUF
U+4D6B.0	fFourCorner	30330
U+4D6B.0	fCantonese	fan1
U+4D6B.0	fMandarin	xun1; yu4
U+4D6B.0	fEnglish	[1] same as &#x7E81; [2] same as &#x9EE6;
U+4D6C.0	fUTF8	䵬
U+4D6C.0	fCNS11643	4-657D
U+4D6C.0	fGB	3-7C4A
U+4D6C.0	fR/S	 203.8
U+4D6C.0	fTotalStrokes	 20
U+4D6C.0	fCangjie	WFEA
U+4D6C.0	fFourCorner	62360
U+4D6C.0	fCantonese	daap6
U+4D6C.0	fMandarin	ta4
U+4D6C.0	fEnglish	[1] black [2] corrupted form of &#x6FCC;
U+4D6D.0	fUTF8	䵭
U+4D6D.0	fCNS11643	5-6F58
U+4D6D.0	fR/S	 203.9; 203.8
U+4D6D.0	fTotalStrokes	 20; 21
U+4D6D.0	fCangjie	WFJKA
U+4D6D.0	fFourCorner	64360
U+4D6D.0	fCantonese	ze3; kei4; syu1
U+4D6D.0	fMandarin	zhe4
U+4D6D.0	fEnglish	black
U+4D6E.0	fUTF8	䵮
U+4D6E.0	fCNS11643	4-6852
U+4D6E.0	fGB	3-7C40
U+4D6E.0	fR/S	 203.9
U+4D6E.0	fTotalStrokes	 21
U+4D6E.0	fCangjie	WFAMH
U+4D6E.0	fFourCorner	66320
U+4D6E.0	fCantonese	joeng4; soeng1; zoeng6
U+4D6E.0	fMandarin	yang4; shang1
U+4D6E.0	fEnglish	[1] red-and-black color [2] light green; light blue
U+4D6F.0	fUTF8	䵯
U+4D6F.0	fCNS11643	3-5F3B
U+4D6F.0	fR/S	 203.9
U+4D6F.0	fTotalStrokes	 21
U+4D6F.0	fCangjie	WFHJG
U+4D6F.0	fFourCorner	62310
U+4D6F.0	fHakka	 ton3; tan3
U+4D6F.0	fCantonese	teon2; teoi3
U+4D6F.0	fMandarin	tuan3
U+4D6F.0	fEnglish	the land outside a city; country
U+4D70.0	fUTF8	䵰
U+4D70.0	fCNS11643	4-6B56
U+4D70.0	fGB	5-7A33
U+4D70.0	fR/S	 203.11
U+4D70.0	fTotalStrokes	 23
U+4D70.0	fCangjie	WFOAH
U+4D70.0	fFourCorner	68320
U+4D70.0	fCantonese	soeng1
U+4D70.0	fMandarin	shang1
U+4D70.0	fEnglish	[1] black color [2] red-and-black color
U+4D71.0	fUTF8	䵱
U+4D71.0	fCNS11643	4-6C5B
U+4D71.0	fGB	3-7C4F
U+4D71.0	fR/S	 203.12
U+4D71.0	fTotalStrokes	 24
U+4D71.0	fCangjie	WFGRR
U+4D71.0	fFourCorner	64361
U+4D71.0	fCantonese	hik1; sik1
U+4D71.0	fMandarin	xi1; he4; xi4
U+4D71.0	fEnglish	[1] red-and-black color [2] blue-and-black color [3] black color
U+4D72.0	fUTF8	䵲
U+4D72.0	fCNS11643	4-6D44
U+4D72.0	fGB	3-7C50
U+4D72.0	fR/S	 203.13
U+4D72.0	fTotalStrokes	 25
U+4D72.0	fCangjie	WFRRD
U+4D72.0	fFourCorner	66390
U+4D72.0	fHakka	 ciau1
U+4D72.0	fCantonese	ciu1
U+4D72.0	fMandarin	qiao1; diao4
U+4D72.0	fEnglish	hemp turning black with the damp
U+4D73.0	fUTF8	䵳
U+4D73.0	fCNS11643	3-615E
U+4D73.0	fGB	5-7A35
U+4D73.0	fR/S	 203.13
U+4D73.0	fTotalStrokes	 25
U+4D73.0	fCangjie	WFOMA
U+4D73.0	fFourCorner	68360
U+4D73.0	fCantonese	wai3; waai3; wui6
U+4D73.0	fMandarin	hui4; wei4
U+4D73.0	fEnglish	light black
U+4D74.0	fUTF8	䵴
U+4D74.0	fCNS11643	4-6D43
U+4D74.0	fGB	3-7C48
U+4D74.0	fR/S	 203.13
U+4D74.0	fTotalStrokes	 25
U+4D74.0	fCangjie	WFRXU
U+4D74.0	fFourCorner	77317
U+4D74.0	fCantonese	wan6; zaan6
U+4D74.0	fMandarin	yun4; zeng4
U+4D74.0	fEnglish	[1] black color [2] black specks; mottles on the face [3] black face [4] same as &#x6688;
U+4D75.0	fUTF8	䵵
U+4D75.0	fCNS11643	4-6D6F
U+4D75.0	fGB	3-6541
U+4D75.0	fR/S	 203.14
U+4D75.0	fTotalStrokes	 26
U+4D75.0	fCangjie	HBUF
U+4D75.0	fFourCorner	88330
U+4D75.0	fCantonese	caat3; zat3; zyut3
U+4D75.0	fMandarin	chua4; zha2; zhuo2
U+4D75.0	fEnglish	[1] yellowish black and white [2] short and black [3] over-fatigued; weary; tired
U+4D76.0	fUTF8	䵶
U+4D76.0	fCNS11643	4-5F40
U+4D76.0	fGB	3-7523
U+4D76.0	fR/S	 205.5
U+4D76.0	fTotalStrokes	 18
U+4D76.0	fCangjie	PRRXU
U+4D76.0	fFourCorner	27610
U+4D76.0	fCantonese	keoi4
U+4D76.0	fMandarin	qu2; gou1
U+4D76.0	fEnglish	a kind of frog; a kind of sea turtle; with two horns; and pattern on the tortoise shell
U+4D77.0	fUTF8	䵷
U+4D77.0	fCNS11643	3-5C2C
U+4D77.0	fGB	5-744A
U+4D77.0	fR/S	 205.6
U+4D77.0	fTotalStrokes	 19
U+4D77.0	fCangjie	GGRXU
U+4D77.0	fFourCorner	47110
U+4D77.0	fCantonese	waa1
U+4D77.0	fMandarin	wa1; hua2
U+4D77.0	fEnglish	[1] same as &#x9F03;, &#x86D9; [2] interchangeable with &#x54C7;
U+4D78.0	fUTF8	䵸
U+4D78.0	fCNS11643	4-6A37
U+4D78.0	fGB	3-7524
U+4D78.0	fR/S	 205.9
U+4D78.0	fTotalStrokes	 22; 21
U+4D78.0	fCangjie	HFRXU
U+4D78.0	fFourCorner	29710
U+4D78.0	fCantonese	cuk1; caau1
U+4D78.0	fMandarin	qiu1
U+4D78.0	fEnglish	same as &#x9F00;
U+4D79.0	fUTF8	䵹
U+4D79.0	fCNS11643	3-613A
U+4D79.0	fR/S	 205.11
U+4D79.0	fTotalStrokes	 24
U+4D79.0	fCangjie	ODRXU
U+4D79.0	fFourCorner	81710
U+4D79.0	fCantonese	zi1
U+4D79.0	fMandarin	zhi1
U+4D79.0	fEnglish	standard form of &#x9F05;
U+4D7A.0	fUTF8	䵺
U+4D7A.0	fCNS11643	4-5561
U+4D7A.0	fGB	3-3472
U+4D7A.0	fR/S	 206.3; 206.13; 32.13
U+4D7A.0	fTotalStrokes	 16
U+4D7A.0	fCangjie	GBUL
U+4D7A.0	fFourCorner	42127
U+4D7A.0	fCantonese	ting5; ding2
U+4D7A.0	fMandarin	ting3
U+4D7A.0	fEnglish	same as &#x5722;, &#x753A;
U+4D7B.0	fUTF8	䵻
U+4D7B.0	fCNS11643	4-6C61
U+4D7B.0	fGB	3-7C3D
U+4D7B.0	fR/S	 206.11
U+4D7B.0	fTotalStrokes	 24
U+4D7B.0	fCangjie	BLQJS
U+4D7B.0	fFourCorner	25277
U+4D7B.0	fCantonese	seoi6; waai6
U+4D7B.0	fMandarin	hui4; gu3; hu2
U+4D7B.0	fEnglish	[1] a small tripod of bronze with two ears [2] standard form of &#x93CF;
U+4D7C.0	fUTF8	䵼
U+4D7C.0	fCNS11643	4-6C62
U+4D7C.0	fGB	3-7C3C
U+4D7C.0	fR/S	 206.11
U+4D7C.0	fTotalStrokes	 24
U+4D7C.0	fCangjie	VIBUL
U+4D7C.0	fFourCorner	27220
U+4D7C.0	fCantonese	soeng1
U+4D7C.0	fMandarin	shang1
U+4D7C.0	fEnglish	 boil; cook; stew
U+4D7D.0	fUTF8	䵽
U+4D7D.0	fCNS11643	4-5F43
U+4D7D.0	fGB	3-7C5C
U+4D7D.0	fR/S	 207.5
U+4D7D.0	fTotalStrokes	 18
U+4D7D.0	fCangjie	GEGI
U+4D7D.0	fFourCorner	44730
U+4D7D.0	fCantonese	caap3; cat6
U+4D7D.0	fMandarin	ca4
U+4D7D.0	fEnglish	[1] the sound of drums [2] beat the side of a drum
U+4D7E.0	fUTF8	䵾
U+4D7E.0	fCNS11643	4-5F42
U+4D7E.0	fGB	5-7A57
U+4D7E.0	fR/S	 207.5
U+4D7E.0	fTotalStrokes	 18
U+4D7E.0	fCangjie	OIGTE
U+4D7E.0	fFourCorner	24140
U+4D7E.0	fHakka	 fu2
U+4D7E.0	fCantonese	fu4
U+4D7E.0	fMandarin	fu2; fu3
U+4D7E.0	fEnglish	noisy; hilarity and mirth of soldiers
U+4D7F.0	fUTF8	䵿
U+4D7F.0	fCNS11643	4-5F44
U+4D7F.0	fGB	3-7C5D
U+4D7F.0	fR/S	 207.5
U+4D7F.0	fTotalStrokes	 18
U+4D7F.0	fCangjie	GEYR
U+4D7F.0	fFourCorner	44600
U+4D7F.0	fCantonese	tip3; nim1
U+4D7F.0	fMandarin	tie1; tie4
U+4D7F.0	fEnglish	[1] the drum make no sound [2] spacious
U+4D80.0	fUTF8	䶀
U+4D80.0	fCNS11643	4-6326
U+4D80.0	fGB	3-7C5E
U+4D80.0	fR/S	 207.6
U+4D80.0	fTotalStrokes	 19
U+4D80.0	fCangjie	GEOMR
U+4D80.0	fFourCorner	44600
U+4D80.0	fHakka	 tap7
U+4D80.0	fCantonese	taap3; daap6 (also); dap6
U+4D80.0	fMandarin	ta4
U+4D80.0	fEnglish	the sound of tambourines or drums
U+4D81.0	fUTF8	䶁
U+4D81.0	fCNS11643	4-6859
U+4D81.0	fGB	3-7C60
U+4D81.0	fR/S	 207.8
U+4D81.0	fTotalStrokes	 21
U+4D81.0	fCangjie	GEEA
U+4D81.0	fFourCorner	44600
U+4D81.0	fCantonese	taap3; dap6
U+4D81.0	fMandarin	ta4
U+4D81.0	fEnglish	noises; disorderly or confused sound of the drums
U+4D82.0	fUTF8	䶂
U+4D82.0	fCNS11643	4-5562
U+4D82.0	fGB	3-7C64
U+4D82.0	fR/S	 208.3
U+4D82.0	fTotalStrokes	 16
U+4D82.0	fCangjie	HVPI
U+4D82.0	fFourCorner	77712
U+4D82.0	fHakka	 chok7; zok7
U+4D82.0	fCantonese	zoek3
U+4D82.0	fMandarin	zuo2; zhuo2
U+4D82.0	fEnglish	squirrel
U+4D83.0	fUTF8	䶃
U+4D83.0	fCNS11643	4-5A63
U+4D83.0	fGB	3-7C65
U+4D83.0	fR/S	 208.4
U+4D83.0	fTotalStrokes	 17; 16
U+4D83.0	fCangjie	HVOIN
U+4D83.0	fFourCorner	78712
U+4D83.0	fCantonese	ham4
U+4D83.0	fMandarin	han2; gan1
U+4D83.0	fEnglish	[1] a kind of rat [2] lizard
U+4D84.0	fUTF8	䶄
U+4D84.0	fCNS11643	4-5F45
U+4D84.0	fGB	3-7C69
U+4D84.0	fR/S	 208.5
U+4D84.0	fTotalStrokes	 18
U+4D84.0	fCangjie	HVMFJ
U+4D84.0	fFourCorner	71714
U+4D84.0	fCantonese	ping4
U+4D84.0	fMandarin	ping2
U+4D84.0	fEnglish	striped mottled rat; mountain rat; short and red-brown colored with spots
U+4D85.0	fUTF8	䶅
U+4D85.0	fCNS11643	4-6327
U+4D85.0	fGB	3-7C6F
U+4D85.0	fR/S	 208.6
U+4D85.0	fTotalStrokes	 19
U+4D85.0	fCangjie	HVHER
U+4D85.0	fFourCorner	77716
U+4D85.0	fCantonese	hok6; kok3; lok3; lok6
U+4D85.0	fMandarin	he4; he2; luo4
U+4D85.0	fEnglish	a marmot-like rat with a big head
U+4D86.0	fUTF8	䶆
U+4D86.0	fCNS11643	4-685A
U+4D86.0	fGB	3-7C72
U+4D86.0	fR/S	 208.8
U+4D86.0	fTotalStrokes	 21
U+4D86.0	fCangjie	HVOG
U+4D86.0	fFourCorner	70710
U+4D86.0	fHakka	 chui1; zui1
U+4D86.0	fCantonese	zeoi1; ceoi4; kei4
U+4D86.0	fMandarin	zui1
U+4D86.0	fEnglish	an old name for a rat
U+4D87.0	fUTF8	䶇
U+4D87.0	fCNS11643	3-602A
U+4D87.0	fR/S	 208.9
U+4D87.0	fTotalStrokes	 22
U+4D87.0	fCangjie	HVLWB
U+4D87.0	fFourCorner	75712
U+4D87.0	fCantonese	zau6
U+4D87.0	fMandarin	zhou4
U+4D87.0	fEnglish	 rat
U+4D88.0	fUTF8	䶈
U+4D88.0	fCNS11643	4-6B58
U+4D88.0	fGB	3-7C75
U+4D88.0	fR/S	 208.10
U+4D88.0	fTotalStrokes	 23
U+4D88.0	fCangjie	HVIBI
U+4D88.0	fFourCorner	73714
U+4D88.0	fCantonese	bok3
U+4D88.0	fMandarin	bo2
U+4D88.0	fEnglish	a kind of rat, known together as the kangaroo
U+4D89.0	fUTF8	䶉
U+4D89.0	fCNS11643	4-6B5B
U+4D89.0	fGB	3-7C77
U+4D89.0	fR/S	 208.10
U+4D89.0	fTotalStrokes	 23
U+4D89.0	fCangjie	HVHHW
U+4D89.0	fFourCorner	77760
U+4D89.0	fHakka	  liu2
U+4D89.0	fCantonese	lau4
U+4D89.0	fMandarin	liu2
U+4D89.0	fEnglish	the name of a small animal about the size of a rabbit, Rhyzomys Sinensis
U+4D8A.0	fUTF8	䶊
U+4D8A.0	fCNS11643	3-5A2C
U+4D8A.0	fR/S	 209.4
U+4D8A.0	fTotalStrokes	 18
U+4D8A.0	fCangjie	HLNG
U+4D8A.0	fFourCorner	27214
U+4D8A.0	fHakka	 nuk8
U+4D8A.0	fCantonese	nuk6; luk6 (also)
U+4D8A.0	fMandarin	niu4; nuu4; n4
U+4D8A.0	fEnglish	(corrupted form) interchangeable with &#x8844;
U+4D8B.0	fUTF8	䶋
U+4D8B.0	fCNS11643	4-5F46
U+4D8B.0	fGB	3-7C79
U+4D8B.0	fR/S	 209.4
U+4D8B.0	fTotalStrokes	 18
U+4D8B.0	fCangjie	HLNHE
U+4D8B.0	fFourCorner	27247
U+4D8B.0	fHakka	 kip7
U+4D8B.0	fCantonese	hap1; kaap1
U+4D8B.0	fMandarin	xi1
U+4D8B.0	fEnglish	stertorous breathing
U+4D8C.0	fUTF8	䶌
U+4D8C.0	fCNS11643	3-5E23
U+4D8C.0	fGB	5-7A68
U+4D8C.0	fR/S	 209.5
U+4D8C.0	fTotalStrokes	 20; 19
U+4D8C.0	fCangjie	HLPRU
U+4D8C.0	fFourCorner	27212
U+4D8C.0	fCantonese	paau3
U+4D8C.0	fMandarin	bao4; pao4
U+4D8C.0	fEnglish	same as &#x76B0;
U+4D8D.0	fUTF8	䶍
U+4D8D.0	fCNS11643	7-5267
U+4D8D.0	fR/S	 209.6
U+4D8D.0	fTotalStrokes	 20
U+4D8D.0	fCangjie	MGHUL
U+4D8D.0	fFourCorner	16121
U+4D8D.0	fCantonese	tai3
U+4D8D.0	fMandarin	di4; ti4
U+4D8D.0	fEnglish	same as &#x568F;
U+4D8E.0	fUTF8	䶎
U+4D8E.0	fCNS11643	5-6F61
U+4D8E.0	fR/S	 209.6
U+4D8E.0	fTotalStrokes	 20
U+4D8E.0	fCangjie	HLOMR
U+4D8E.0	fFourCorner	28261
U+4D8E.0	fCantonese	hap1; hot3
U+4D8E.0	fMandarin	he2; sha4; ca1
U+4D8E.0	fEnglish	the sound of breath; snoring; snorting
U+4D8F.0	fUTF8	䶏
U+4D8F.0	fCNS11643	4-685C
U+4D8F.0	fGB	5-7A6B
U+4D8F.0	fR/S	 209.7
U+4D8F.0	fTotalStrokes	 21
U+4D8F.0	fCangjie	CHHUL
U+4D8F.0	fFourCorner	86221
U+4D8F.0	fHakka	 ti5; ti6; pi6; pi5
U+4D8F.0	fCantonese	ji4; taai1
U+4D8F.0	fMandarin	ti4; ti3; xi3
U+4D8F.0	fEnglish	 to snivel (as when weeping bitterly)
U+4D90.0	fUTF8	䶐
U+4D90.0	fCNS11643	4-6E31
U+4D90.0	fGB	5-7A69
U+4D90.0	fR/S	 209.13
U+4D90.0	fTotalStrokes	 27
U+4D90.0	fCangjie	HLOMA
U+4D90.0	fFourCorner	28266
U+4D90.0	fCantonese	wui3; fai3
U+4D90.0	fMandarin	wai4; kuai4; huai4
U+4D90.0	fEnglish	 take breath; snoring; snorting
U+4D91.0	fUTF8	䶑
U+4D91.0	fCNS11643	5-7B7A
U+4D91.0	fR/S	 209.15; 209.16
U+4D91.0	fTotalStrokes	 29; 30
U+4D91.0	fCangjie	HLJBO
U+4D91.0	fFourCorner	24281
U+4D91.0	fHakka	  ti5; di5; cih5; chi5
U+4D91.0	fCantonese	tai3
U+4D91.0	fMandarin	di4; ti4
U+4D91.0	fEnglish	 sneeze
U+4D92.0	fUTF8	䶒
U+4D92.0	fCNS11643	4-5A66
U+4D92.0	fGB	3-6622
U+4D92.0	fR/S	 210.3
U+4D92.0	fTotalStrokes	 17; 16
U+4D92.0	fCangjie	YXV
U+4D92.0	fFourCorner	00220
U+4D92.0	fCantonese	cai4
U+4D92.0	fMandarin	qi2
U+4D92.0	fEnglish	[1] outstanding ability [2] exquisite; fine
U+4D93.0	fUTF8	䶓
U+4D93.0	fCNS11643	5-612D
U+4D93.0	fR/S	 210.3
U+4D93.0	fTotalStrokes	 17
U+4D93.0	fCangjie	LBYX
U+4D93.0	fFourCorner	40220
U+4D93.0	fCantonese	zai3; caap1; zaai3
U+4D93.0	fMandarin	ji4; qi4
U+4D93.0	fEnglish	[1] ramie; linen sackcloth [2] twist and join (cords); continue [3] order the arrest of; capture
U+4D94.0	fUTF8	䶔
U+4D94.0	fCNS11643	4-5F48
U+4D94.0	fGB	5-7421
U+4D94.0	fR/S	 211.3
U+4D94.0	fTotalStrokes	 18
U+4D94.0	fCangjie	YUPD
U+4D94.0	fFourCorner	24710
U+4D94.0	fCantonese	ci4
U+4D94.0	fMandarin	chi2; min2
U+4D94.0	fEnglish	prominent gums (of the teeth)
U+4D95.0	fUTF8	䶕
U+4D95.0	fCNS11643	4-6329
U+4D95.0	fGB	3-7466
U+4D95.0	fR/S	 211.4
U+4D95.0	fTotalStrokes	 19
U+4D95.0	fCangjie	YUAU
U+4D95.0	fFourCorner	27710
U+4D95.0	fCantonese	baa6; baa1; paa4
U+4D95.0	fMandarin	ba1; pa2
U+4D95.0	fEnglish	irregular and prominent teeth
U+4D96.0	fUTF8	䶖
U+4D96.0	fCNS11643	4-632B
U+4D96.0	fGB	3-7465
U+4D96.0	fR/S	 211.4
U+4D96.0	fTotalStrokes	 19
U+4D96.0	fCangjie	YUOIN
U+4D96.0	fFourCorner	28720
U+4D96.0	fCantonese	gam6; gaam3
U+4D96.0	fMandarin	jin4; qin2
U+4D96.0	fEnglish	[1] disease of tongue (said of an ox) [2] keep the mouth shut
U+4D97.0	fUTF8	䶗
U+4D97.0	fCNS11643	4-6627
U+4D97.0	fGB	3-7467
U+4D97.0	fR/S	 211.5
U+4D97.0	fTotalStrokes	 20
U+4D97.0	fCangjie	YUMNR
U+4D97.0	fFourCorner	21720
U+4D97.0	fHakka	  ka1; k'ie5; kat7
U+4D97.0	fCantonese	kaa1; haap3; hap3
U+4D97.0	fMandarin	ke1; ke4; qia3; qie4
U+4D97.0	fEnglish	serrated, to crunch
U+4D98.0	fUTF8	䶘
U+4D98.0	fCNS11643	4-6625
U+4D98.0	fGB	3-746B
U+4D98.0	fR/S	 211.5
U+4D98.0	fTotalStrokes	 20
U+4D98.0	fCangjie	YUYT
U+4D98.0	fFourCorner	20710
U+4D98.0	fCantonese	laap6
U+4D98.0	fMandarin	li4; la4
U+4D98.0	fEnglish	[1] the sound of chewing something dry and hard [2] sound of gnaw or bite
U+4D99.0	fUTF8	䶙
U+4D99.0	fCNS11643	4-6626
U+4D99.0	fGB	3-7461
U+4D99.0	fR/S	 211.5
U+4D99.0	fTotalStrokes	 20
U+4D99.0	fCangjie	YUSS
U+4D99.0	fFourCorner	21710
U+4D99.0	fCantonese	geoi6
U+4D99.0	fMandarin	ju4
U+4D99.0	fEnglish	swelling of the gums (of the teeth)
U+4D9A.0	fUTF8	䶚
U+4D9A.0	fCNS11643	3-5F3D
U+4D9A.0	fR/S	 211.6
U+4D9A.0	fTotalStrokes	 21
U+4D9A.0	fCangjie	YULMI
U+4D9A.0	fFourCorner	25730
U+4D9A.0	fCantonese	geoi2
U+4D9A.0	fMandarin	qu3
U+4D9A.0	fEnglish	non-classical form of &#x9F72;
U+4D9B.0	fUTF8	䶛
U+4D9B.0	fCNS11643	4-6861
U+4D9B.0	fGB	3-746D
U+4D9B.0	fR/S	 211.6
U+4D9B.0	fTotalStrokes	 21
U+4D9B.0	fCangjie	MNYMU
U+4D9B.0	fFourCorner	12770
U+4D9B.0	fCantonese	laat6
U+4D9B.0	fMandarin	lie4; la4
U+4D9B.0	fEnglish	[1] separate the meat from the bones with teeth [2] the sound of using teeth to separate the meat from the bones
U+4D9C.0	fUTF8	䶜
U+4D9C.0	fCNS11643	4-6A3D
U+4D9C.0	fGB	3-7474
U+4D9C.0	fR/S	 211.7
U+4D9C.0	fTotalStrokes	 22; 21
U+4D9C.0	fCangjie	YUHGR
U+4D9C.0	fFourCorner	24760
U+4D9C.0	fCantonese	guk1
U+4D9C.0	fMandarin	gu4; guo4; hu4
U+4D9C.0	fEnglish	[1] the process to whiten the ivory or elephant tusk [2] sound of gnaw
U+4D9D.0	fUTF8	䶝
U+4D9D.0	fCNS11643	4-6A3C
U+4D9D.0	fGB	3-746E
U+4D9D.0	fR/S	 211.7
U+4D9D.0	fTotalStrokes	 22; 21
U+4D9D.0	fCangjie	YUKOO
U+4D9D.0	fFourCorner	24730
U+4D9D.0	fCantonese	haap6; haap1
U+4D9D.0	fMandarin	qia4; xia2
U+4D9D.0	fEnglish	[1] growing irregular teeth; crooked teeth [2] cleft tooth, sound of gnaw
U+4D9E.0	fUTF8	䶞
U+4D9E.0	fCNS11643	3-606E
U+4D9E.0	fR/S	 211.8
U+4D9E.0	fTotalStrokes	 23
U+4D9E.0	fCangjie	YUTMC
U+4D9E.0	fFourCorner	24780
U+4D9E.0	fCantonese	kei4
U+4D9E.0	fMandarin	ji1; qi2
U+4D9E.0	fEnglish	 gnaw; bite
U+4D9F.0	fUTF8	䶟
U+4D9F.0	fCNS11643	5-774D
U+4D9F.0	fR/S	 211.8
U+4D9F.0	fTotalStrokes	 23
U+4D9F.0	fCangjie	YUNHX
U+4D9F.0	fFourCorner	27770
U+4D9F.0	fCantonese	haam6
U+4D9F.0	fMandarin	xian4
U+4D9F.0	fEnglish	 gnash the teeth (in anger)
U+4DA0.0	fUTF8	䶠
U+4DA0.0	fCNS11643	4-6C69
U+4DA0.0	fGB	5-743B
U+4DA0.0	fR/S	 211.9
U+4DA0.0	fTotalStrokes	 24
U+4DA0.0	fCangjie	IRYMU
U+4DA0.0	fFourCorner	53770
U+4DA0.0	fCantonese	gaam1
U+4DA0.0	fMandarin	jian1; xian2; jian3; yan2; jia2
U+4DA0.0	fEnglish	same as &#x4DA2;
U+4DA1.0	fUTF8	䶡
U+4DA1.0	fCNS11643	4-6C66
U+4DA1.0	fGB	5-742D
U+4DA1.0	fR/S	 211.9
U+4DA1.0	fTotalStrokes	 24
U+4DA1.0	fCangjie	YUBCN
U+4DA1.0	fFourCorner	22700
U+4DA1.0	fCantonese	zit6; saat6
U+4DA1.0	fMandarin	shi4; shi2; ze2; zhi4
U+4DA1.0	fEnglish	[1] the sound of biting [2] same as &#x9F70;
U+4DA2.0	fUTF8	䶢
U+4DA2.0	fCNS11643	4-6C6A
U+4DA2.0	fGB	3-7478
U+4DA2.0	fR/S	 211.9
U+4DA2.0	fTotalStrokes	 24
U+4DA2.0	fCangjie	YUIHR
U+4DA2.0	fFourCorner	23750
U+4DA2.0	fCantonese	gaam1
U+4DA2.0	fMandarin	xian2
U+4DA2.0	fEnglish	[1] gnaw; bite [2] hold in the mouth
U+4DA3.0	fUTF8	䶣
U+4DA3.0	fCNS11643	4-6D4C
U+4DA3.0	fGB	3-746F
U+4DA3.0	fR/S	 211.10
U+4DA3.0	fTotalStrokes	 25
U+4DA3.0	fCangjie	YUUMT
U+4DA3.0	fFourCorner	22710
U+4DA3.0	fCantonese	ngoi4; oi4 (also); goi1
U+4DA3.0	fMandarin	ai2
U+4DA3.0	fEnglish	[1] grind the teeth [2] teeth
U+4DA4.0	fUTF8	䶤
U+4DA4.0	fCNS11643	4-6D4B
U+4DA4.0	fGB	3-747A
U+4DA4.0	fR/S	 211.10
U+4DA4.0	fTotalStrokes	 25
U+4DA4.0	fCangjie	YUBBB
U+4DA4.0	fFourCorner	27720
U+4DA4.0	fCantonese	hat6; waa4
U+4DA4.0	fMandarin	hua2
U+4DA4.0	fEnglish	the sound of chewing bones
U+4DA5.0	fUTF8	䶥
U+4DA5.0	fCNS11643	4-6D70
U+4DA5.0	fGB	5-7443
U+4DA5.0	fR/S	 211.11
U+4DA5.0	fTotalStrokes	 26
U+4DA5.0	fCangjie	YUYPM
U+4DA5.0	fFourCorner	21717
U+4DA5.0	fHakka	  za1
U+4DA5.0	fCantonese	zaa1; zeoi2
U+4DA5.0	fMandarin	zha1; ju3; cha2; chu3
U+4DA5.0	fEnglish	[1] open out [2] brag; make a display
U+4DA6.0	fUTF8	䶦
U+4DA6.0	fCNS11643	3-6176
U+4DA6.0	fGB	5-7432
U+4DA6.0	fR/S	 211.11
U+4DA6.0	fTotalStrokes	 26
U+4DA6.0	fCangjie	YUQMC
U+4DA6.0	fFourCorner	25780
U+4DA6.0	fCantonese	caak3; zak3
U+4DA6.0	fMandarin	yi2; ze2
U+4DA6.0	fEnglish	[1] upper and lower teeth in order and corresponding each other [2] gnaw; bite
U+4DA7.0	fUTF8	䶧
U+4DA7.0	fCNS11643	3-622B
U+4DA7.0	fGB	5-742B
U+4DA7.0	fR/S	 211.12
U+4DA7.0	fTotalStrokes	 27
U+4DA7.0	fCangjie	YUGGU
U+4DA7.0	fFourCorner	24710
U+4DA7.0	fCantonese	ngaau5; aau5 (also); ngau5
U+4DA7.0	fMandarin	yao3
U+4DA7.0	fEnglish	same as &#x9F69;, &#x54AC;
U+4DA8.0	fUTF8	䶨
U+4DA8.0	fCNS11643	3-6232
U+4DA8.0	fR/S	 211.13
U+4DA8.0	fTotalStrokes	 28
U+4DA8.0	fCangjie	YUOMO
U+4DA8.0	fFourCorner	28780
U+4DA8.0	fCantonese	zaam1; tam4; zaam3; zam1
U+4DA8.0	fMandarin	qian1
U+4DA8.0	fEnglish	in the manner a toothless person chewing food
U+4DA9.0	fUTF8	䶩
U+4DA9.0	fCNS11643	4-6E4B
U+4DA9.0	fGB	3-7470
U+4DA9.0	fR/S	 211.14
U+4DA9.0	fTotalStrokes	 29
U+4DA9.0	fCangjie	YUYX
U+4DA9.0	fFourCorner	20720
U+4DA9.0	fHakka	 zi5
U+4DA9.0	fCantonese	zai6; caai1; dam3; zaai6
U+4DA9.0	fMandarin	qi4; ji4
U+4DA9.0	fEnglish	 bite
U+4DAA.0	fUTF8	䶪
U+4DAA.0	fCNS11643	4-6E4C
U+4DAA.0	fGB	5-7445
U+4DAA.0	fR/S	 211.14
U+4DAA.0	fTotalStrokes	 29
U+4DAA.0	fCangjie	YUJIG
U+4DAA.0	fFourCorner	23750
U+4DAA.0	fCantonese	caat3
U+4DAA.0	fMandarin	cha2
U+4DAA.0	fEnglish	[1] sharp teeth [2] break something hard like a knife, sand (in food) [3] ugly
U+4DAB.0	fUTF8	䶫
U+4DAB.0	fCNS11643	4-6E5A
U+4DAB.0	fGB	3-7473
U+4DAB.0	fR/S	 211.20
U+4DAB.0	fTotalStrokes	 35
U+4DAB.0	fCangjie	YURRK
U+4DAB.0	fFourCorner	26740
U+4DAB.0	fCantonese	jim6; zim6
U+4DAB.0	fMandarin	yan4; yan2; kan3
U+4DAB.0	fEnglish	[1] fine teeth [2] irregular teeth
U+4DAC.0	fUTF8	䶬
U+4DAC.0	fCNS11643	5-7561
U+4DAC.0	fR/S	 212.6
U+4DAC.0	fTotalStrokes	 22; 20
U+4DAC.0	fCangjie	MJYBP
U+4DAC.0	fFourCorner	11210
U+4DAC.0	fCantonese	gin1
U+4DAC.0	fMandarin	jian1
U+4DAC.0	fEnglish	hard fins on the back of a dragon
U+4DAD.0	fUTF8	䶭
U+4DAD.0	fGB	S-242E
U+4DAD.0	fR/S	 212.4
U+4DAD.0	fTotalStrokes	 20
U+4DAD.0	fCangjie	IPMG
U+4DAD.0	fCantonese	wong1
U+4DAE.0	fUTF8	䶮
U+4DAE.0	fGB	7-2227
U+4DAE.0	fR/S	 212.4
U+4DAE.0	fTotalStrokes	 20
U+4DAE.0	fCangjie	IPMK
U+4DAE.0	fCantonese	jim5; jim2 (also); zim5
U+4DAE.0	fMandarin	yan2; yan3
U+4DAE.0	fEnglish	clever; wise; superior, a person's name ( in Earlier Five Dynasties, South Han, meaning flying dragon over the sky)
U+4DAF.0	fUTF8	䶯
U+4DAF.0	fCNS11643	5-6B69
U+4DAF.0	fGB	3-3C69
U+4DAF.0	fR/S	 213.3
U+4DAF.0	fTotalStrokes	 19
U+4DAF.0	fCangjie	VNXU
U+4DAF.0	fFourCorner	47410
U+4DAF.0	fCantonese	kwai1; gwaai1; gwaai2; kwaai1
U+4DAF.0	fMandarin	kui1
U+4DAF.0	fEnglish	used in a girl's name
U+4DB0.0	fUTF8	䶰
U+4DB0.0	fCNS11643	5-6C44
U+4DB0.0	fR/S	 213.4; 82.16
U+4DB0.0	fTotalStrokes	 20
U+4DB0.0	fCangjie	NUHQU
U+4DB0.0	fFourCorner	22710
U+4DB0.0	fCantonese	ziu1
U+4DB0.0	fEnglish	feather ornaments
U+4DB1.0	fUTF8	䶱
U+4DB1.0	fCNS11643	4-6862
U+4DB1.0	fGB	3-7235
U+4DB1.0	fR/S	 213.5
U+4DB1.0	fTotalStrokes	 21
U+4DB1.0	fCangjie	HYNXU
U+4DB1.0	fFourCorner	27310
U+4DB1.0	fCantonese	tung4
U+4DB1.0	fMandarin	tong2; zhong1
U+4DB1.0	fEnglish	a kind of turtle
U+4DB2.0	fUTF8	䶲
U+4DB2.0	fGB	3-7234
U+4DB2.0	fR/S	 213.5
U+4DB2.0	fTotalStrokes	 21
U+4DB2.0	fCangjie	GBNXU
U+4DB2.0	fCantonese	jim4; zim4
U+4DB2.0	fMandarin	nan2; nan4; ran2
U+4DB2.0	fEnglish	the edge; the hem or the margin of the tortoise shell
U+4DB3.0	fUTF8	䶳
U+4DB3.0	fCNS11643	4-6863
U+4DB3.0	fGB	3-7C7D
U+4DB3.0	fR/S	 214.4
U+4DB3.0	fTotalStrokes	 21
U+4DB3.0	fCangjie	OBYHN
U+4DB3.0	fFourCorner	80210
U+4DB3.0	fCantonese	joek6; zoek6
U+4DB3.0	fMandarin	yue4
U+4DB3.0	fEnglish	[1] lookup [2] adore [3] to rely upon
U+4DB4.0	fUTF8	䶴
U+4DB4.0	fCNS11643	4-6D51
U+4DB4.0	fGB	5-7A72
U+4DB4.0	fR/S	 214.8
U+4DB4.0	fTotalStrokes	 25
U+4DB4.0	fCangjie	FOOMB
U+4DB4.0	fFourCorner	97220
U+4DB4.0	fCantonese	ceoi1; ceoi2
U+4DB4.0	fMandarin	chui1; chui4
U+4DB4.0	fEnglish	same as &#x5439;
U+4DB5.0	fUTF8	䶵
U+4DB5.0	fCNS11643	3-622C
U+4DB5.0	fR/S	 214.10
U+4DB5.0	fTotalStrokes	 27
U+4DB5.0	fCangjie	OBHYU
U+4DB5.0	fFourCorner	82210
U+4DB5.0	fCantonese	ci4; si4
U+4DB5.0	fMandarin	chi2; shi3
U+4DB5.0	fEnglish	same as &#x7B8E;
U+4DB6.0	fUTF8	䶶
U+4DB6.0	fR/S	
U+4DB7.0	fUTF8	䶷
U+4DB7.0	fR/S	
U+4DB8.0	fUTF8	䶸
U+4DB8.0	fR/S	
U+4DB9.0	fUTF8	䶹
U+4DB9.0	fR/S	
U+4DBA.0	fUTF8	䶺
U+4DBA.0	fR/S	
U+4DBB.0	fUTF8	䶻
U+4DBB.0	fR/S	
U+4DBC.0	fUTF8	䶼
U+4DBC.0	fR/S	
U+4DBD.0	fUTF8	䶽
U+4DBD.0	fR/S	
U+4DBE.0	fUTF8	䶾
U+4DBE.0	fR/S	
U+4DBF.0	fUTF8	䶿
U+4DBF.0	fR/S	
U+4E00.0	fUTF8	一
U+4E00.0	fCNS11643	1-4421
U+4E00.0	fBig5	A440
U+4E00.0	fGB	0-523B
U+4E00.0	fR/S	 1.0
U+4E00.0	fTotalStrokes	 1
U+4E00.0	fCangjie	M
U+4E00.0	fFourCorner	10000
U+4E00.0	fHakka	  jit7; rit7
U+4E00.0	fCantonese	jat1
U+4E00.0	fMandarin	yi1
U+4E00.0	fEnglish	[1] one; unit [2] whole; all; throughout; complete [3] one principle [4] once; as soon as [5] single; alone [6] slightly; a little [7] a; an; the [8] each; per; every time [9] unify; unite [10] same; together [11] union; uniformity; uniform [12] so; such; to such extent [13] another [14] once [15] a little [16] a Chinese family name
U+4E01.0	fUTF8	丁
U+4E01.0	fCNS11643	1-4423
U+4E01.0	fBig5	A442
U+4E01.0	fGB	0-3621
U+4E01.0	fR/S	 1.1
U+4E01.0	fTotalStrokes	 2
U+4E01.0	fCangjie	MN
U+4E01.0	fFourCorner	10200
U+4E01.0	fHakka	  den1; dien1
U+4E01.0	fCantonese	ding1
U+4E01.0	fMandarin	ding1
U+4E01.0	fEnglish	[1] fourth celestial stem [2] fourth; grade D [3] population [4] male adult [5] attendants [6] T-shaped [7] small cubes of meat or vegetable [8] a Chinese family name
U+4E01.1	fUTF8	丁
U+4E01.1	fCNS11643	1-4423
U+4E01.1	fBig5	A442
U+4E01.1	fGB	0-3621
U+4E01.1	fR/S	 1.1
U+4E01.1	fTotalStrokes	 2
U+4E01.1	fCangjie	MN
U+4E01.1	fFourCorner	10200
U+4E01.1	fCantonese	zaang1; zang1 (丁zaang1的異讀字,also)
U+4E01.1	fMandarin	zheng1
U+4E01.1	fEnglish	descriptive of cling, clang, tinkling sound
U+4E02.0	fUTF8	丂
U+4E02.0	fCNS11643	4-2126
U+4E02.0	fGB	5-3021
U+4E02.0	fR/S	 1.1
U+4E02.0	fTotalStrokes	 2
U+4E02.0	fCangjie	MVS
U+4E02.0	fFourCorner	10200
U+4E02.0	fHakka	  kau3
U+4E02.0	fCantonese	haau2
U+4E02.0	fMandarin	kao3; qiao3
U+4E02.0	fEnglish	obstruction of breath (qi) as it seeks release
U+4E02.1	fUTF8	丂
U+4E02.1	fCNS11643	4-2126
U+4E02.1	fGB	5-3021
U+4E02.1	fR/S	 1.1
U+4E02.1	fTotalStrokes	 2
U+4E02.1	fCangjie	MVS
U+4E02.1	fFourCorner	10200
U+4E02.1	fMandarin	yu2
U+4E03.0	fUTF8	七
U+4E03.0	fCNS11643	1-4424
U+4E03.0	fBig5	A443
U+4E03.0	fGB	0-465F
U+4E03.0	fR/S	 1.1
U+4E03.0	fTotalStrokes	 2
U+4E03.0	fCangjie	JU
U+4E03.0	fFourCorner	40710
U+4E03.0	fHakka	  cit7
U+4E03.0	fCantonese	cat1
U+4E03.0	fMandarin	qi1
U+4E03.0	fEnglish	[1] seven [2] a seventh day after death, when sacrifice is offered until seven times seven days (forty-ninth) is over [3] a Chinese family name
U+4E04.0	fUTF8	丄
U+4E04.0	fCNS11643	3-2126
U+4E04.0	fGB	E-2121
U+4E04.0	fR/S	 1.1
U+4E04.0	fTotalStrokes	 2
U+4E04.0	fCangjie	LM
U+4E04.0	fFourCorner	20100
U+4E04.0	fHakka	  song3;  shong3; shong6; shong5; hong6; hong5  song1; shong1 song5
U+4E04.0	fCantonese	soeng6 soeng5
U+4E04.0	fMandarin	shang4 shang3 shang5
U+4E04.0	fEnglish	same as &#x4E0A;
U+4E05.0	fUTF8	丅
U+4E05.0	fCNS11643	3-2125
U+4E05.0	fGB	E-2122
U+4E05.0	fR/S	 1.1
U+4E05.0	fTotalStrokes	 2
U+4E05.0	fCangjie	ML
U+4E05.0	fFourCorner	10200
U+4E05.0	fHakka	 ha5;  ha3; ha6  ha1; ka1
U+4E05.0	fCantonese	haa6; haa1 (also); haa2 (also) haa5
U+4E05.0	fMandarin	xia4 xia5 jia4
U+4E05.0	fEnglish	same as &#x4E0B;
U+4E06.0	fUTF8	丆
U+4E06.0	fGB	1-7D3D
U+4E06.0	fR/S	 1.1
U+4E06.0	fTotalStrokes	 2
U+4E06.0	fCangjie	XMH
U+4E06.0	fCantonese	daan1
U+4E06.0	fEnglish	
U+4E07.0	fUTF8	万
U+4E07.0	fCNS11643	2-2126
U+4E07.0	fBig5	C945
U+4E07.0	fGB	0-4D72
U+4E07.0	fR/S	 1.2
U+4E07.0	fTotalStrokes	 3
U+4E07.0	fCangjie	MS
U+4E07.0	fFourCorner	10227
U+4E07.0	fHakka	wan6; wan5; van5
U+4E07.0	fCantonese	maan6
U+4E07.0	fMandarin	wan4
U+4E07.0	fEnglish	[1] ten thousand; innumerable [2] ancient form of &#x842C; [3] a Chinese family name [4] simplified form of &#x842C;
U+4E07.1	fUTF8	万
U+4E07.1	fCNS11643	2-2126
U+4E07.1	fBig5	C945
U+4E07.1	fGB	0-4D72
U+4E07.1	fR/S	 1.2
U+4E07.1	fTotalStrokes	 3
U+4E07.1	fCangjie	MS
U+4E07.1	fFourCorner	10227
U+4E07.1	fCantonese	mak6; maak6 (万的異讀字)
U+4E07.1	fMandarin	mo4
U+4E07.1	fEnglish	萬俟, a Chinese family name
U+4E08.0	fUTF8	丈
U+4E08.0	fCNS11643	1-4437
U+4E08.0	fBig5	A456
U+4E08.0	fGB	0-5549
U+4E08.0	fR/S	 1.2
U+4E08.0	fTotalStrokes	 3
U+4E08.0	fCangjie	JK
U+4E08.0	fFourCorner	50000
U+4E08.0	fHakka	 chong6; cong3; cong5; chong5
U+4E08.0	fCantonese	zoeng6; zoeng2 (also)
U+4E08.0	fMandarin	zhang4
U+4E08.0	fEnglish	[1] unit of length equal to 3.3 meters (10 feet) [2] elder; senior [3] measure; survey (land) [4] gentleman; man; husband
U+4E08.1	fUTF8	丈
U+4E08.1	fCNS11643	1-4437
U+4E08.1	fBig5	A456
U+4E08.1	fGB	0-5549
U+4E08.1	fR/S	 1.2
U+4E08.1	fTotalStrokes	 3
U+4E08.1	fCangjie	JK
U+4E08.1	fFourCorner	50000
U+4E08.1	fHakka	cong1
U+4E09.0	fUTF8	三
U+4E09.0	fCNS11643	1-4435
U+4E09.0	fBig5	A454
U+4E09.0	fGB	0-487D
U+4E09.0	fR/S	 1.2
U+4E09.0	fTotalStrokes	 3
U+4E09.0	fCangjie	MMM
U+4E09.0	fFourCorner	10101
U+4E09.0	fHakka	  sam1
U+4E09.0	fCantonese	saam1
U+4E09.0	fMandarin	san1
U+4E09.0	fEnglish	[1] three; three times [2] a Chinese family name
U+4E09.1	fUTF8	三
U+4E09.1	fCNS11643	1-4435
U+4E09.1	fBig5	A454
U+4E09.1	fGB	0-487D
U+4E09.1	fR/S	 1.2
U+4E09.1	fTotalStrokes	 3
U+4E09.1	fCangjie	MMM
U+4E09.1	fFourCorner	10101
U+4E09.1	fHakka	 sam3; sam5
U+4E09.1	fCantonese	saam3
U+4E09.1	fMandarin	san4
U+4E0A.0	fUTF8	上
U+4E0A.0	fCNS11643	1-4438
U+4E0A.0	fBig5	A457
U+4E0A.0	fGB	0-494F
U+4E0A.0	fR/S	 1.2
U+4E0A.0	fTotalStrokes	 3
U+4E0A.0	fCangjie	YM
U+4E0A.0	fFourCorner	21100
U+4E0A.0	fHakka	  song3;  shong3; shong6; shong5; hong6; hong5
U+4E0A.0	fCantonese	soeng6
U+4E0A.0	fMandarin	shang4
U+4E0A.0	fEnglish	[1] above [2] go up; upward; up [3] go to [4] appear on scene or enter category [5] present [6] affix; apply; place in position [7] wind up [8] upper; first; best; better; superior [9] previous; before; above [10] top; summit; on [11] ascend; mount; board [12] go to court
U+4E0A.1	fUTF8	上
U+4E0A.1	fCNS11643	1-4438
U+4E0A.1	fBig5	A457
U+4E0A.1	fGB	0-494F
U+4E0A.1	fR/S	 1.2
U+4E0A.1	fTotalStrokes	 3
U+4E0A.1	fCangjie	YM
U+4E0A.1	fFourCorner	21100
U+4E0A.1	fHakka	 song1; shong1
U+4E0A.1	fCantonese	soeng5
U+4E0A.1	fEnglish	 go up
U+4E0A.2	fUTF8	上
U+4E0A.2	fCNS11643	1-4438
U+4E0A.2	fBig5	A457
U+4E0A.2	fGB	0-494F
U+4E0A.2	fR/S	 1.2
U+4E0A.2	fTotalStrokes	 3
U+4E0A.2	fCangjie	YM
U+4E0A.2	fFourCorner	21100
U+4E0A.2	fHakka	 song5
U+4E0A.2	fMandarin	shang3
U+4E0A.2	fEnglish	falling-rising tone, one of the four tones in classical Chinese and the third tone in modern Mandarin Chinese
U+4E0A.3	fUTF8	上
U+4E0A.3	fCNS11643	1-4438
U+4E0A.3	fBig5	A457
U+4E0A.3	fGB	0-494F
U+4E0A.3	fR/S	 1.2
U+4E0A.3	fTotalStrokes	 3
U+4E0A.3	fCangjie	YM
U+4E0A.3	fFourCorner	21100
U+4E0A.3	fMandarin	shang5
U+4E0B.0	fUTF8	下
U+4E0B.0	fCNS11643	1-4436
U+4E0B.0	fBig5	A455
U+4E0B.0	fGB	0-4F42
U+4E0B.0	fR/S	 1.2
U+4E0B.0	fTotalStrokes	 3
U+4E0B.0	fCangjie	MY
U+4E0B.0	fFourCorner	10230
U+4E0B.0	fHakka	 ha5;  ha3; ha6
U+4E0B.0	fCantonese	haa6; haa1 (also); haa2 (also)
U+4E0B.0	fEnglish	[1] put down [2] lay [3] fall [4] descend [5] begin [6] below; under [7] inferior; lower [8] next
U+4E0B.1	fUTF8	下
U+4E0B.1	fCNS11643	1-4436
U+4E0B.1	fBig5	A455
U+4E0B.1	fGB	0-4F42
U+4E0B.1	fR/S	 1.2
U+4E0B.1	fTotalStrokes	 3
U+4E0B.1	fCangjie	MY
U+4E0B.1	fFourCorner	10230
U+4E0B.1	fHakka	  ha1; ka1
U+4E0B.1	fCantonese	haa5
U+4E0B.1	fMandarin	xia4
U+4E0B.1	fEnglish	 go down once
U+4E0B.2	fUTF8	下
U+4E0B.2	fCNS11643	1-4436
U+4E0B.2	fBig5	A455
U+4E0B.2	fGB	0-4F42
U+4E0B.2	fR/S	 1.2
U+4E0B.2	fTotalStrokes	 3
U+4E0B.2	fCangjie	MY
U+4E0B.2	fFourCorner	10230
U+4E0B.2	fMandarin	xia5
U+4E0C.0	fUTF8	丌
U+4E0C.0	fCNS11643	2-2127
U+4E0C.0	fBig5	C946
U+4E0C.0	fGB	0-5822
U+4E0C.0	fR/S	 1.2
U+4E0C.0	fTotalStrokes	 3
U+4E0C.0	fCangjie	ML
U+4E0C.0	fFourCorner	10220
U+4E0C.0	fHakka	 hi1; gi1
U+4E0C.0	fCantonese	gei1
U+4E0C.0	fMandarin	ji1
U+4E0C.0	fEnglish	table
U+4E0C.1	fUTF8	丌
U+4E0C.1	fCNS11643	2-2127
U+4E0C.1	fBig5	C946
U+4E0C.1	fGB	0-5822
U+4E0C.1	fR/S	 1.2
U+4E0C.1	fTotalStrokes	 3
U+4E0C.1	fCangjie	ML
U+4E0C.1	fFourCorner	10220
U+4E0C.1	fCantonese	kei4
U+4E0C.1	fMandarin	qi2
U+4E0C.1	fEnglish	ancient form of &#x5176;
U+4E0D.0	fUTF8	不
U+4E0D.0	fCNS11643	1-4462
U+4E0D.0	fBig5	A4A3
U+4E0D.0	fGB	0-323B
U+4E0D.0	fR/S	 1.3
U+4E0D.0	fTotalStrokes	 4
U+4E0D.0	fCangjie	MF
U+4E0D.0	fFourCorner	10900
U+4E0D.0	fHakka	  but7
U+4E0D.0	fCantonese	bat1
U+4E0D.0	fMandarin	bu4
U+4E0D.0	fEnglish	no, not; un-; negative prefix
U+4E0D.1	fUTF8	不
U+4E0D.1	fCNS11643	1-4462
U+4E0D.1	fBig5	A4A3
U+4E0D.1	fGB	0-323B
U+4E0D.1	fR/S	 1.3
U+4E0D.1	fTotalStrokes	 4
U+4E0D.1	fCangjie	MF
U+4E0D.1	fFourCorner	10900
U+4E0D.1	fCantonese	fau2
U+4E0D.1	fMandarin	fou3
U+4E0D.2	fUTF8	不
U+4E0D.2	fCNS11643	1-4462
U+4E0D.2	fBig5	A4A3
U+4E0D.2	fGB	0-323B
U+4E0D.2	fR/S	 1.3
U+4E0D.2	fTotalStrokes	 4
U+4E0D.2	fCangjie	MF
U+4E0D.2	fFourCorner	10900
U+4E0D.2	fMandarin	fou1
U+4E0D.3	fUTF8	不
U+4E0D.3	fCNS11643	1-4462
U+4E0D.3	fBig5	A4A3
U+4E0D.3	fGB	0-323B
U+4E0D.3	fR/S	 1.3
U+4E0D.3	fTotalStrokes	 4
U+4E0D.3	fCangjie	MF
U+4E0D.3	fFourCorner	10900
U+4E0D.3	fMandarin	fu1
U+4E0D.4	fUTF8	不
U+4E0D.4	fCNS11643	1-4462
U+4E0D.4	fBig5	A4A3
U+4E0D.4	fGB	0-323B
U+4E0D.4	fR/S	 1.3
U+4E0D.4	fTotalStrokes	 4
U+4E0D.4	fCangjie	MF
U+4E0D.4	fFourCorner	10900
U+4E0D.4	fMandarin	bu5
U+4E0E.0	fUTF8	与
U+4E0E.0	fCNS11643	2-212F
U+4E0E.0	fBig5	C94F
U+4E0E.0	fGB	0-536B
U+4E0E.0	fR/S	 1.3
U+4E0E.0	fTotalStrokes	 4
U+4E0E.0	fCangjie	YSM
U+4E0E.0	fFourCorner	21407
U+4E0E.0	fHakka	 ji2; ri2; ji1
U+4E0E.0	fCantonese	jyu5; jyu4; jyu6
U+4E0E.0	fMandarin	yu3
U+4E0E.0	fEnglish	[1] same as &#x8207; [2] simplified form of &#x8207;
U+4E0E.1	fUTF8	与
U+4E0E.1	fCNS11643	2-212F
U+4E0E.1	fBig5	C94F
U+4E0E.1	fGB	0-536B
U+4E0E.1	fR/S	 1.3
U+4E0E.1	fTotalStrokes	 4
U+4E0E.1	fCangjie	YSM
U+4E0E.1	fFourCorner	21407
U+4E0E.1	fMandarin	yu4
U+4E0F.0	fUTF8	丏
U+4E0F.0	fCNS11643	2-212D
U+4E0F.0	fBig5	C94D
U+4E0F.0	fGB	3-3021
U+4E0F.0	fR/S	 1.3
U+4E0F.0	fTotalStrokes	 4
U+4E0F.0	fCangjie	MLVS
U+4E0F.0	fFourCorner	10727
U+4E0F.0	fHakka	 mien3; men3; mien5
U+4E0F.0	fCantonese	min5
U+4E0F.0	fMandarin	mian3
U+4E0F.0	fEnglish	[1] curtain to ward off arrows; parapet [2] hidden; invisible
U+4E0F.1	fUTF8	丏
U+4E0F.1	fCNS11643	2-212D
U+4E0F.1	fBig5	C94D
U+4E0F.1	fGB	3-3021
U+4E0F.1	fR/S	 1.3
U+4E0F.1	fTotalStrokes	 4
U+4E0F.1	fCangjie	MLVS
U+4E0F.1	fFourCorner	10727
U+4E0F.1	fMandarin	gai4
U+4E0F.1	fEnglish	same as &#x4E10;
U+4E10.0	fUTF8	丐
U+4E10.0	fCNS11643	1-4461
U+4E10.0	fBig5	A4A2
U+4E10.0	fGB	0-5824
U+4E10.0	fR/S	 1.3
U+4E10.0	fTotalStrokes	 4
U+4E10.0	fCangjie	MYVS
U+4E10.0	fFourCorner	10207
U+4E10.0	fHakka	  goi5; gai5
U+4E10.0	fCantonese	koi3; goi3
U+4E10.0	fMandarin	gai4
U+4E10.0	fEnglish	[1] beggar; [2] beg for alms [3] give
U+4E11.0	fUTF8	丑
U+4E11.0	fCNS11643	1-4460
U+4E11.0	fBig5	A4A1
U+4E11.0	fGB	0-3373
U+4E11.0	fR/S	 1.3
U+4E11.0	fTotalStrokes	 4
U+4E11.0	fCangjie	NG
U+4E11.0	fFourCorner	17105
U+4E11.0	fHakka	  chiu3;  ciu3; cu3; zhu3
U+4E11.0	fCantonese	cau2
U+4E11.0	fMandarin	chou3
U+4E11.0	fEnglish	[1] clown; comedian [2] the period of the day from 1 to 3 a.m. [3] second of terrestrial branches (ox) [4] a Chinese family name [5] simplified form of &#x919C;
U+4E12.0	fUTF8	丒
U+4E12.0	fCNS11643	4-2139
U+4E12.0	fGB	E-2123
U+4E12.0	fR/S	 1.3
U+4E12.0	fTotalStrokes	 4
U+4E12.0	fCangjie	SKM
U+4E12.0	fFourCorner	17420
U+4E12.0	fCantonese	ng5
U+4E12.0	fMandarin	chou3
U+4E12.0	fEnglish	the period from 1 to 3 a.m.; 2nd character of the 'branches'
U+4E13.0	fUTF8	专
U+4E13.0	fGB	0-5728
U+4E13.0	fR/S	 1.3
U+4E13.0	fTotalStrokes	 4
U+4E13.0	fCangjie	QNI
U+4E13.0	fHakka	 zhon1
U+4E13.0	fCantonese	zyun1
U+4E13.0	fMandarin	zhuan1
U+4E13.0	fEnglish	simplified form of &#x5C08;
U+4E13.1	fUTF8	专
U+4E13.1	fGB	0-5728
U+4E13.1	fR/S	 1.3
U+4E13.1	fTotalStrokes	 4
U+4E13.1	fCangjie	QNI
U+4E14.0	fUTF8	且
U+4E14.0	fCNS11643	1-4562
U+4E14.0	fBig5	A542
U+4E14.0	fGB	0-4752
U+4E14.0	fR/S	 1.4
U+4E14.0	fTotalStrokes	 5
U+4E14.0	fCangjie	BM
U+4E14.0	fFourCorner	77100
U+4E14.0	fHakka	  cia3
U+4E14.0	fCantonese	ce2
U+4E14.0	fMandarin	qie3
U+4E14.0	fEnglish	[1] moreover; still; further [2] just; for the time being [3] expressing impending event [4] even [5] at random [6] now [7] furthermore; besides [8] both ... and ...; while
U+4E14.1	fUTF8	且
U+4E14.1	fCNS11643	1-4562
U+4E14.1	fBig5	A542
U+4E14.1	fGB	0-4752
U+4E14.1	fR/S	 1.4
U+4E14.1	fTotalStrokes	 5
U+4E14.1	fCangjie	BM
U+4E14.1	fFourCorner	77100
U+4E14.1	fCantonese	zeoi1; ceoi1; ceoi2
U+4E14.1	fMandarin	ju1
U+4E14.1	fEnglish	 final particle; similar to &#x8005;
U+4E14.2	fUTF8	且
U+4E14.2	fCNS11643	1-4562
U+4E14.2	fBig5	A542
U+4E14.2	fGB	0-4752
U+4E14.2	fR/S	 1.4
U+4E14.2	fTotalStrokes	 5
U+4E14.2	fCangjie	BM
U+4E14.2	fFourCorner	77100
U+4E14.2	fMandarin	zu3
U+4E14.2	fEnglish	ancient form of &#x7956;
U+4E14.3	fUTF8	且
U+4E14.3	fCNS11643	1-4562
U+4E14.3	fBig5	A542
U+4E14.3	fGB	0-4752
U+4E14.3	fR/S	 1.4
U+4E14.3	fTotalStrokes	 5
U+4E14.3	fCangjie	BM
U+4E14.3	fFourCorner	77100
U+4E14.3	fCantonese	cou4
U+4E14.3	fMandarin	cu2
U+4E14.3	fEnglish	[1] interchangeable with &#x5F82; [2] interchangeable with &#x7C97;
U+4E15.0	fUTF8	丕
U+4E15.0	fCNS11643	1-4561
U+4E15.0	fBig5	A541
U+4E15.0	fGB	0-5827
U+4E15.0	fR/S	 1.4
U+4E15.0	fTotalStrokes	 5
U+4E15.0	fCangjie	MFM
U+4E15.0	fFourCorner	10109
U+4E15.0	fHakka	 pi1
U+4E15.0	fCantonese	pei1
U+4E15.0	fMandarin	pi1
U+4E15.0	fEnglish	[1] great; distinguished; grand; glorious [2] in observance of (a ruling, etc.) [3] receive [4] a Chinese family name
U+4E15.1	fUTF8	丕
U+4E15.1	fCNS11643	1-4561
U+4E15.1	fBig5	A541
U+4E15.1	fGB	0-5827
U+4E15.1	fR/S	 1.4
U+4E15.1	fTotalStrokes	 5
U+4E15.1	fCangjie	MFM
U+4E15.1	fFourCorner	10109
U+4E15.1	fMandarin	bu4
U+4E15.1	fEnglish	interchangeable with &#x4E0D;
U+4E16.0	fUTF8	世
U+4E16.0	fCNS11643	1-4560
U+4E16.0	fBig5	A540
U+4E16.0	fGB	0-4A40
U+4E16.0	fR/S	 1.4
U+4E16.0	fTotalStrokes	 5
U+4E16.0	fCangjie	PT
U+4E16.0	fFourCorner	44717
U+4E16.0	fHakka	  si5;  se5;  s'e5; shi5; she5
U+4E16.0	fCantonese	sai3; sai2 (also)
U+4E16.0	fMandarin	shi4
U+4E16.0	fEnglish	[1] generation [2] person's life span; incarnation; one life [3] epoch; era; age [4] the world [5] experience of human society [6] 30 years
U+4E17.0	fUTF8	丗
U+4E17.0	fCNS11643	4-2155
U+4E17.0	fGB	E-2124
U+4E17.0	fR/S	 1.3
U+4E17.0	fTotalStrokes	 5; 4
U+4E17.0	fCangjie	XTJ
U+4E17.0	fFourCorner	44100
U+4E17.0	fCantonese	sai3
U+4E17.0	fMandarin	shi4
U+4E17.0	fEnglish	same as &#x2000D;
U+4E18.0	fUTF8	丘
U+4E18.0	fCNS11643	1-4563
U+4E18.0	fBig5	A543
U+4E18.0	fGB	0-4770
U+4E18.0	fR/S	 1.4
U+4E18.0	fTotalStrokes	 5
U+4E18.0	fCangjie	OM
U+4E18.0	fFourCorner	72101
U+4E18.0	fHakka	  hiu1; kiu1; sieu1
U+4E18.0	fCantonese	jau1
U+4E18.0	fMandarin	qiu1
U+4E18.0	fEnglish	[1] hillock; mound [2] generally used to denote the open country, or wild country [3] big; elder [4] empty [5] the first name of Confucius [6] a Chinese family name
U+4E19.0	fUTF8	丙
U+4E19.0	fCNS11643	1-455F
U+4E19.0	fBig5	A4FE
U+4E19.0	fGB	0-317B
U+4E19.0	fR/S	 1.4
U+4E19.0	fTotalStrokes	 5
U+4E19.0	fCangjie	MOB
U+4E19.0	fFourCorner	10227
U+4E19.0	fHakka	  biang3
U+4E19.0	fCantonese	bing2
U+4E19.0	fMandarin	bing3
U+4E19.0	fEnglish	[1] third [2] third of the ten celestial stems [3] another name for fire [4] tail of a fish [5] a Chinese family name
U+4E19.1	fUTF8	丙
U+4E19.1	fCNS11643	1-455F
U+4E19.1	fBig5	A4FE
U+4E19.1	fGB	0-317B
U+4E19.1	fR/S	 1.4
U+4E19.1	fTotalStrokes	 5
U+4E19.1	fCangjie	MOB
U+4E19.1	fFourCorner	10227
U+4E19.1	fMandarin	bing4
U+4E1A.0	fUTF8	业
U+4E1A.0	fGB	0-5235
U+4E1A.0	fR/S	 1.4
U+4E1A.0	fTotalStrokes	 5
U+4E1A.0	fCangjie	TC
U+4E1A.0	fHakka	 ngiap8
U+4E1A.0	fCantonese	jip6
U+4E1A.0	fMandarin	ye4
U+4E1A.0	fEnglish	simplified form of &#x696D;
U+4E1B.0	fUTF8	丛
U+4E1B.0	fGB	0-3454
U+4E1B.0	fR/S	 1.4
U+4E1B.0	fTotalStrokes	 5
U+4E1B.0	fCangjie	OOM
U+4E1B.0	fHakka	  cung2; ciung3
U+4E1B.0	fCantonese	cung4
U+4E1B.0	fMandarin	cong2; cong1
U+4E1B.0	fEnglish	simplified form of &#x53E2;
U+4E1C.0	fUTF8	东
U+4E1C.0	fGB	0-362B
U+4E1C.0	fR/S	 1.4
U+4E1C.0	fTotalStrokes	 5
U+4E1C.0	fCangjie	KD
U+4E1C.0	fHakka	  dung1
U+4E1C.0	fCantonese	dung1
U+4E1C.0	fMandarin	dong1
U+4E1C.0	fEnglish	simplified form of &#x6771;
U+4E1D.0	fUTF8	丝
U+4E1D.0	fGB	0-4B3F
U+4E1D.0	fR/S	 1.4
U+4E1D.0	fTotalStrokes	 5
U+4E1D.0	fCangjie	VVM
U+4E1D.0	fHakka	  sii1;  su1; si1; se1
U+4E1D.0	fCantonese	si1
U+4E1D.0	fMandarin	si1
U+4E1D.0	fEnglish	simplified form of &#x7D72;
U+4E1E.0	fUTF8	丞
U+4E1E.0	fCNS11643	1-4722
U+4E1E.0	fBig5	A5E0
U+4E1E.0	fGB	0-5829
U+4E1E.0	fR/S	 1.5
U+4E1E.0	fTotalStrokes	 6
U+4E1E.0	fCangjie	NEM
U+4E1E.0	fFourCorner	17103
U+4E1E.0	fHakka	 shin2; sin2; sen2; shin3
U+4E1E.0	fCantonese	sing4; cing2
U+4E1E.0	fMandarin	cheng2
U+4E1E.0	fEnglish	[1] assist; aid; rescue [2] deputy to an official [3] magistrate [4] premier; minister
U+4E1E.1	fUTF8	丞
U+4E1E.1	fCNS11643	1-4722
U+4E1E.1	fBig5	A5E0
U+4E1E.1	fGB	0-5829
U+4E1E.1	fR/S	 1.5
U+4E1E.1	fTotalStrokes	 6
U+4E1E.1	fCangjie	NEM
U+4E1E.1	fFourCorner	17103
U+4E1E.1	fMandarin	sheng4
U+4E1E.2	fUTF8	丞
U+4E1E.2	fCNS11643	1-4722
U+4E1E.2	fBig5	A5E0
U+4E1E.2	fGB	0-5829
U+4E1E.2	fR/S	 1.5
U+4E1E.2	fTotalStrokes	 6
U+4E1E.2	fCangjie	NEM
U+4E1E.2	fFourCorner	17103
U+4E1E.2	fMandarin	zheng1
U+4E1E.3	fUTF8	丞
U+4E1E.3	fCNS11643	1-4722
U+4E1E.3	fBig5	A5E0
U+4E1E.3	fGB	0-5829
U+4E1E.3	fR/S	 1.5
U+4E1E.3	fTotalStrokes	 6
U+4E1E.3	fCangjie	NEM
U+4E1E.3	fFourCorner	17103
U+4E1E.3	fMandarin	zheng3
U+4E1F.0	fUTF8	丟
U+4E1F.0	fCNS11643	1-4723
U+4E1F.0	fBig5	A5E1
U+4E1F.0	fGB	E-2125
U+4E1F.0	fR/S	 1.5
U+4E1F.0	fTotalStrokes	 6
U+4E1F.0	fCangjie	XXMGI
U+4E1F.0	fFourCorner	10731
U+4E1F.0	fHakka	 diu1
U+4E1F.0	fCantonese	diu1
U+4E1F.0	fMandarin	diu1
U+4E1F.0	fEnglish	[1] throw [2] lose; discard [3] put aside; lay aside
U+4E20.0	fUTF8	丠
U+4E20.0	fCNS11643	3-2262
U+4E20.0	fGB	5-3023
U+4E20.0	fR/S	 1.5
U+4E20.0	fTotalStrokes	 6
U+4E20.0	fCangjie	LPM
U+4E20.0	fFourCorner	11100
U+4E20.0	fHakka	  hiu1; kiu1; sieu1
U+4E20.0	fCantonese	jau1
U+4E20.0	fMandarin	qiu1
U+4E20.0	fEnglish	same as &#x4E18;
U+4E21.0	fUTF8	両
U+4E21.0	fCNS11643	3-2261
U+4E21.0	fGB	E-2126
U+4E21.0	fR/S	 1.5
U+4E21.0	fTotalStrokes	 6
U+4E21.0	fCangjie	MUB
U+4E21.0	fFourCorner	10220
U+4E21.0	fHakka	  liong3;  liong1
U+4E21.0	fCantonese	loeng2; loeng5; loeng6
U+4E21.0	fMandarin	liang3; liang4
U+4E21.0	fEnglish	same as &#x5169;
U+4E22.0	fUTF8	丢
U+4E22.0	fCNS11643	3-2263
U+4E22.0	fGB	0-362A
U+4E22.0	fR/S	 1.5
U+4E22.0	fTotalStrokes	 6
U+4E22.0	fCangjie	HGI
U+4E22.0	fFourCorner	20730
U+4E22.0	fHakka	 diu1
U+4E22.0	fCantonese	diu1
U+4E22.0	fMandarin	diu1
U+4E22.0	fEnglish	same as &#x4E1F;
U+4E23.0	fUTF8	丣
U+4E23.0	fCNS11643	4-2335
U+4E23.0	fGB	E-2127
U+4E23.0	fR/S	 1.6
U+4E23.0	fTotalStrokes	 7
U+4E23.0	fCangjie	MLLS
U+4E23.0	fFourCorner	10770
U+4E23.0	fHakka	 jiu1;  ju1;  ju3; riu1; riu3; jiu3; ju2
U+4E23.0	fCantonese	jau5
U+4E23.0	fMandarin	you3
U+4E23.0	fEnglish	same as &#x9149;
U+4E24.0	fUTF8	两
U+4E24.0	fCNS11643	3-243F
U+4E24.0	fGB	0-413D
U+4E24.0	fR/S	 1.6
U+4E24.0	fTotalStrokes	 7
U+4E24.0	fCangjie	MOOB
U+4E24.0	fFourCorner	10220
U+4E24.0	fHakka	  liong3;  liong1
U+4E24.0	fCantonese	loeng2; loeng5; loeng6
U+4E24.0	fMandarin	liang3; liang4
U+4E24.0	fEnglish	simplified form of &#x5169;
U+4E25.0	fUTF8	严
U+4E25.0	fGB	0-514F
U+4E25.0	fR/S	 1.6
U+4E25.0	fTotalStrokes	 7
U+4E25.0	fCangjie	MTCH
U+4E25.0	fHakka	  ngiam2; niam2; gniam3
U+4E25.0	fCantonese	jim4
U+4E25.0	fMandarin	yan2; yan3; yan4
U+4E25.0	fEnglish	simplified form of &#x56B4;
U+4E26.0	fUTF8	並
U+4E26.0	fCNS11643	1-4B64
U+4E26.0	fBig5	A8C3
U+4E26.0	fGB	8-2C76
U+4E26.0	fR/S	 1.7
U+4E26.0	fTotalStrokes	 8
U+4E26.0	fCangjie	TTC
U+4E26.0	fFourCorner	80102
U+4E26.0	fHakka	  bin5; pin3
U+4E26.0	fCantonese	bing3; bing6 (old)
U+4E26.0	fMandarin	bing4
U+4E26.0	fEnglish	[1] and; also; furthernore; at the same time [2] on the same level with; even; equal; side by side [3] together; parallel with [4] really (to reinforce a negative) [5] entirely; completely
U+4E27.0	fUTF8	丧
U+4E27.0	fGB	0-4925
U+4E27.0	fR/S	 24.6; 1.7
U+4E27.0	fTotalStrokes	 8
U+4E27.0	fCangjie	GCV
U+4E27.0	fHakka	  song3;  song1; song6 song5
U+4E27.0	fCantonese	song1; song3
U+4E27.0	fMandarin	sang1 sang4
U+4E27.0	fEnglish	simplified form of &#x55AA;
U+4E28.0	fUTF8	丨
U+4E28.0	fCNS11643	3-2121
U+4E28.0	fGB	0-582D
U+4E28.0	fR/S	 2.0
U+4E28.0	fTotalStrokes	 1
U+4E28.0	fCangjie	L
U+4E28.0	fFourCorner	20000
U+4E28.0	fHakka	 gun5; kun3; gwun5
U+4E28.0	fCantonese	gwan2; kwan2; jat1
U+4E28.0	fMandarin	gun3; shu4
U+4E28.0	fEnglish	[1] vertical line [2] Kangxi radical 2
U+4E29.0	fUTF8	丩
U+4E29.0	fCNS11643	4-2127
U+4E29.0	fGB	5-303C
U+4E29.0	fR/S	 2.1
U+4E29.0	fTotalStrokes	 2
U+4E29.0	fCangjie	VL
U+4E29.0	fFourCorner	22700
U+4E29.0	fHakka	 giu5;  kiu2; kiau2
U+4E29.0	fCantonese	gau2
U+4E29.0	fMandarin	jiu1
U+4E29.0	fEnglish	to join or connect the vine
U+4E2A.0	fUTF8	个
U+4E2A.0	fCNS11643	3-212C
U+4E2A.0	fGB	0-3876
U+4E2A.0	fR/S	 2.2
U+4E2A.0	fTotalStrokes	 3
U+4E2A.0	fCangjie	OL
U+4E2A.0	fFourCorner	80200
U+4E2A.0	fHakka	 gai5; go5; ge5
U+4E2A.0	fCantonese	go3
U+4E2A.0	fMandarin	ge4
U+4E2A.0	fEnglish	simplified form of &#x500B;
U+4E2A.1	fUTF8	个
U+4E2A.1	fCNS11643	3-212C
U+4E2A.1	fGB	0-3876
U+4E2A.1	fR/S	 2.2
U+4E2A.1	fTotalStrokes	 3
U+4E2A.1	fCangjie	OL
U+4E2A.1	fFourCorner	80200
U+4E2A.1	fMandarin	ge5
U+4E2A.2	fUTF8	个
U+4E2A.2	fCNS11643	3-212C
U+4E2A.2	fGB	0-3876
U+4E2A.2	fR/S	 2.2
U+4E2A.2	fTotalStrokes	 3
U+4E2A.2	fCangjie	OL
U+4E2A.2	fFourCorner	80200
U+4E2A.2	fMandarin	ge3
U+4E2B.0	fUTF8	丫
U+4E2B.0	fCNS11643	1-4439
U+4E2B.0	fBig5	A458
U+4E2B.0	fGB	0-513E
U+4E2B.0	fR/S	 2.2
U+4E2B.0	fTotalStrokes	 3
U+4E2B.0	fCangjie	CL
U+4E2B.0	fFourCorner	80200
U+4E2B.0	fHakka	  a1
U+4E2B.0	fCantonese	aa1; ngaa1 (also)
U+4E2B.0	fMandarin	ya1
U+4E2B.0	fEnglish	something that branches or forks upward; bifurcation; forked branch
U+4E2C.0	fUTF8	丬
U+4E2C.0	fGB	0-635C
U+4E2C.0	fR/S	 90.0
U+4E2C.0	fTotalStrokes	 4
U+4E2C.0	fCangjie	XLIM
U+4E2C.0	fCantonese	baan6
U+4E2C.0	fMandarin	qiang2; pan2
U+4E2C.0	fEnglish	Kangxi radical 90 variant
U+4E2D.0	fUTF8	中
U+4E2D.0	fCNS11643	1-4463
U+4E2D.0	fBig5	A4A4
U+4E2D.0	fGB	0-5650
U+4E2D.0	fR/S	 2.3
U+4E2D.0	fTotalStrokes	 4
U+4E2D.0	fCangjie	L
U+4E2D.0	fFourCorner	50006
U+4E2D.0	fHakka	  zung1; zhung1; zhung5;  dung1
U+4E2D.0	fCantonese	zung1; zung3
U+4E2D.0	fMandarin	zhong1
U+4E2D.0	fEnglish	[1] middle; central; center; in the midst of; among; within; between [2] middle of time, place, period, area (used as postposition, like within, during) [3] be touched by (weather, poison, etc.) [4] pass examination [5] fit taste; liking; be agreeable to [6] verb complement like in hit on, hook on) [7] medium-grade; mediocre [8] in the middle; halfway [9] China; Chinese; Sino-
U+4E2D.1	fUTF8	中
U+4E2D.1	fCNS11643	1-4463
U+4E2D.1	fBig5	A4A4
U+4E2D.1	fGB	0-5650
U+4E2D.1	fR/S	 2.3
U+4E2D.1	fTotalStrokes	 4
U+4E2D.1	fCangjie	L
U+4E2D.1	fFourCorner	50006
U+4E2D.1	fHakka	 zung5
U+4E2D.1	fMandarin	zhong4
U+4E2D.1	fEnglish	[1] hit (target); attain (a goal) [2] be hit by; to be affected by
U+4E2E.0	fUTF8	丮
U+4E2E.0	fCNS11643	2-2130
U+4E2E.0	fBig5	C950
U+4E2E.0	fGB	E-2128
U+4E2E.0	fR/S	 2.3
U+4E2E.0	fTotalStrokes	 4
U+4E2E.0	fCangjie	NQ
U+4E2E.0	fFourCorner	17510
U+4E2E.0	fCantonese	gik1
U+4E2E.0	fMandarin	ji3
U+4E2E.0	fEnglish	catch
U+4E2F.0	fUTF8	丯
U+4E2F.0	fCNS11643	4-213A
U+4E2F.0	fGB	E-2129
U+4E2F.0	fR/S	  2.3
U+4E2F.0	fTotalStrokes	 4
U+4E2F.0	fCangjie	QJ
U+4E2F.0	fFourCorner	50000
U+4E2F.0	fCantonese	gaai3
U+4E2F.0	fMandarin	jie4
U+4E30.0	fUTF8	丰
U+4E30.0	fCNS11643	1-4464
U+4E30.0	fBig5	A4A5
U+4E30.0	fGB	0-3761
U+4E30.0	fR/S	 2.3
U+4E30.0	fTotalStrokes	 4
U+4E30.0	fCangjie	QJ
U+4E30.0	fFourCorner	50000
U+4E30.0	fHakka	 fung1; gai5
U+4E30.0	fCantonese	fung1
U+4E30.0	fMandarin	feng1
U+4E30.0	fEnglish	[1] good-looking; buxom [2] appearance and carriage of a person [3] simplified form of &#x8C50;
U+4E31.0	fUTF8	丱
U+4E31.0	fCNS11643	2-2143
U+4E31.0	fBig5	C963
U+4E31.0	fGB	3-3031
U+4E31.0	fR/S	 2.4
U+4E31.0	fTotalStrokes	 5
U+4E31.0	fCangjie	VLLLM
U+4E31.0	fFourCorner	22770
U+4E31.0	fHakka	 kwan1;  kwan5; gwan5
U+4E31.0	fCantonese	kong3; kwong3 (also); gwaan3
U+4E31.0	fMandarin	guan4
U+4E31.0	fEnglish	[1] child's hairstyle bound in two tufts [2] one's infancy period [3] juvenile; youngster
U+4E31.1	fUTF8	丱
U+4E31.1	fCNS11643	2-2143
U+4E31.1	fBig5	C963
U+4E31.1	fGB	3-3031
U+4E31.1	fR/S	 2.4
U+4E31.1	fTotalStrokes	 5
U+4E31.1	fCangjie	VLLLM
U+4E31.1	fFourCorner	22770
U+4E32.0	fUTF8	串
U+4E32.0	fCNS11643	1-486B
U+4E32.0	fBig5	A6EA
U+4E32.0	fGB	0-342E
U+4E32.0	fR/S	 2.6
U+4E32.0	fTotalStrokes	 7
U+4E32.0	fCangjie	LL
U+4E32.0	fFourCorner	50006
U+4E32.0	fHakka	 con5; chon5
U+4E32.0	fCantonese	cyun3; cyun2 (also)
U+4E32.0	fMandarin	chuan4
U+4E32.0	fEnglish	[1] string together [2] pass out and in irregularly or freely [3] take part in play [4] string (of coins, etc.) [5] pour into [6] conspire; league up to swindle
U+4E32.1	fUTF8	串
U+4E32.1	fCNS11643	1-486B
U+4E32.1	fBig5	A6EA
U+4E32.1	fGB	0-342E
U+4E32.1	fR/S	 2.6
U+4E32.1	fTotalStrokes	 7
U+4E32.1	fCangjie	LL
U+4E32.1	fFourCorner	50006
U+4E32.1	fCantonese	gwaan3
U+4E32.1	fMandarin	guan4
U+4E33.0	fUTF8	丳
U+4E33.0	fCNS11643	2-2531
U+4E33.0	fBig5	CBB1
U+4E33.0	fGB	3-3028
U+4E33.0	fR/S	 2.7
U+4E33.0	fTotalStrokes	 8
U+4E33.0	fCangjie	LLWW
U+4E33.0	fFourCorner	55006
U+4E33.0	fHakka	 con3; can3
U+4E33.0	fCantonese	caan2
U+4E33.0	fMandarin	chan3
U+4E33.0	fEnglish	spit; grill
U+4E34.0	fUTF8	临
U+4E34.0	fGB	0-4159
U+4E34.0	fR/S	 2.8
U+4E34.0	fTotalStrokes	 9
U+4E34.0	fCangjie	LLOA
U+4E34.0	fHakka	  lim2; lim1; lim3
U+4E34.0	fCantonese	lam4; lam1 (also); lam6
U+4E34.0	fMandarin	lin2; lin4
U+4E34.0	fEnglish	simplified form of &#x81E8;
U+4E35.0	fUTF8	丵
U+4E35.0	fCNS11643	4-2B5E
U+4E35.0	fGB	E-212A
U+4E35.0	fR/S	 2.9
U+4E35.0	fTotalStrokes	 10
U+4E35.0	fCangjie	TCTJ
U+4E35.0	fFourCorner	32400
U+4E35.0	fCantonese	zok6
U+4E35.0	fMandarin	zhuo2; zhuo3
U+4E35.0	fEnglish	thick (grass)
U+4E36.0	fUTF8	丶
U+4E36.0	fCNS11643	3-2122
U+4E36.0	fGB	0-583C
U+4E36.0	fR/S	 3.0
U+4E36.0	fTotalStrokes	 1
U+4E36.0	fCangjie	I
U+4E36.0	fFourCorner	30000
U+4E36.0	fHakka	 zhu3; zu3
U+4E36.0	fCantonese	zyu2
U+4E36.0	fMandarin	zhu3; dian3
U+4E36.0	fEnglish	[1] interpunction [2] comma [3] Kangxi radical 3
U+4E37.0	fUTF8	丷
U+4E37.0	fGB	1-7D65
U+4E37.0	fR/S	 3.1
U+4E37.0	fTotalStrokes	 2
U+4E37.0	fCangjie	XIH
U+4E37.0	fCantonese	baat3
U+4E37.0	fEnglish	
U+4E38.0	fUTF8	丸
U+4E38.0	fCNS11643	1-443A
U+4E38.0	fBig5	A459
U+4E38.0	fGB	0-4D68
U+4E38.0	fR/S	 3.2
U+4E38.0	fTotalStrokes	 3
U+4E38.0	fCangjie	KNI
U+4E38.0	fFourCorner	50017
U+4E38.0	fHakka	 jen2; ren2; jan2
U+4E38.0	fCantonese	jyun4; jyun2 (also)
U+4E38.0	fMandarin	wan2
U+4E38.0	fEnglish	[1] small round object; pellet; pill; small ball [2] make pills [3] egg [4] indicating completion or completeness
U+4E39.0	fUTF8	丹
U+4E39.0	fCNS11643	1-4465
U+4E39.0	fBig5	A4A6
U+4E39.0	fGB	0-3524
U+4E39.0	fR/S	 3.3
U+4E39.0	fTotalStrokes	 4
U+4E39.0	fCangjie	BY
U+4E39.0	fFourCorner	77440
U+4E39.0	fHakka	  dan1
U+4E39.0	fCantonese	daan1
U+4E39.0	fMandarin	dan1
U+4E39.0	fEnglish	[1] cinnabar (native HgS); vermilion (artificial HgS used as pigment) [2] red; scarlet [3] sophisticated decoction [4] medical pill; ointment and powder (esp. the pill of immortality sought by alchemists)
U+4E39.1	fUTF8	丹
U+4E39.1	fCNS11643	1-4465
U+4E39.1	fBig5	A4A6
U+4E39.1	fGB	0-3524
U+4E39.1	fR/S	 3.3
U+4E39.1	fTotalStrokes	 4
U+4E39.1	fCangjie	BY
U+4E39.1	fFourCorner	77440
U+4E39.1	fMandarin	dan5
U+4E3A.0	fUTF8	为
U+4E3A.0	fGB	0-4E2A
U+4E3A.0	fR/S	 3.3
U+4E3A.0	fTotalStrokes	 4
U+4E3A.0	fCangjie	IKSI
U+4E3A.0	fHakka	 wui2; wui3; wi2; wi5; wui5
U+4E3A.0	fCantonese	wai4; wai6 (also); wai2 (also)
U+4E3A.0	fMandarin	wei2 wei4
U+4E3A.0	fEnglish	simplified form of &#x70BA;
U+4E3A.1	fUTF8	为
U+4E3A.1	fGB	0-4E2A
U+4E3A.1	fR/S	 3.3
U+4E3A.1	fTotalStrokes	 4
U+4E3A.1	fCangjie	IKSI
U+4E3A.1	fHakka	 wui2; wui3; wi2; wi5; wui5
U+4E3A.1	fCantonese	wai4; wai6 (also); wai2 (also)
U+4E3A.1	fMandarin	wei2 wei4
U+4E3B.0	fUTF8	主
U+4E3B.0	fCNS11643	1-4564
U+4E3B.0	fBig5	A544
U+4E3B.0	fGB	0-5677
U+4E3B.0	fR/S	 3.4
U+4E3B.0	fTotalStrokes	 5
U+4E3B.0	fCangjie	YG
U+4E3B.0	fFourCorner	00104
U+4E3B.0	fHakka	  zu3; zhu3
U+4E3B.0	fCantonese	zyu2
U+4E3B.0	fMandarin	zhu3; zhu4
U+4E3B.0	fEnglish	[1] master, chief; leader; owner [2] host [3] main aim; goal [4] Jesus Christ; God; Lord [5] officiate at; preside over; take charge of; direct; head [6] control; determine [7] main; chief; primary; principal
U+4E3C.0	fUTF8	丼
U+4E3C.0	fCNS11643	2-2144
U+4E3C.0	fBig5	C964
U+4E3C.0	fGB	E-212B
U+4E3C.0	fR/S	 3.4
U+4E3C.0	fTotalStrokes	 5
U+4E3C.0	fCangjie	TTI
U+4E3C.0	fFourCorner	55000
U+4E3C.0	fHakka	 ziang3
U+4E3C.0	fCantonese	zeng2; zing2; dam2
U+4E3C.0	fMandarin	jing3; dan3
U+4E3C.0	fEnglish	 bowl of food well
U+4E3C.1	fUTF8	丼
U+4E3C.1	fCNS11643	2-2144
U+4E3C.1	fBig5	C964
U+4E3C.1	fGB	E-212B
U+4E3C.1	fR/S	 3.4
U+4E3C.1	fTotalStrokes	 5
U+4E3C.1	fCangjie	TTI
U+4E3C.1	fFourCorner	55000
U+4E3D.0	fUTF8	丽
U+4E3D.0	fCNS11643	3-2740
U+4E3D.0	fGB	0-4076
U+4E3D.0	fR/S	 3.7; 1.6
U+4E3D.0	fTotalStrokes	 7; 8
U+4E3D.0	fCangjie	MBIB
U+4E3D.0	fFourCorner	11220
U+4E3D.0	fHakka	 li5
U+4E3D.0	fCantonese	lai6; lai4; lei4
U+4E3D.0	fMandarin	li4; li2; li3; si1
U+4E3D.0	fEnglish	simplified form of &#x9E97;
U+4E3E.0	fUTF8	举
U+4E3E.0	fGB	0-3E59
U+4E3E.0	fR/S	 3.8
U+4E3E.0	fTotalStrokes	 9
U+4E3E.0	fCangjie	FCQ
U+4E3E.0	fHakka	 gi3
U+4E3E.0	fCantonese	geoi2
U+4E3E.0	fMandarin	ju3
U+4E3E.0	fEnglish	simplified form of &#x8209;
U+4E3E.1	fUTF8	举
U+4E3E.1	fGB	0-3E59
U+4E3E.1	fR/S	 3.8
U+4E3E.1	fTotalStrokes	 9
U+4E3E.1	fCangjie	FCQ
U+4E3E.1	fMandarin	ju5
U+4E3F.0	fUTF8	丿
U+4E3F.0	fCNS11643	3-2123
U+4E3F.0	fGB	0-582F
U+4E3F.0	fR/S	 4.0
U+4E3F.0	fTotalStrokes	 1
U+4E3F.0	fCangjie	H
U+4E3F.0	fFourCorner	20000
U+4E3F.0	fHakka	 piet7; p'iat8
U+4E3F.0	fCantonese	pit3
U+4E3F.0	fMandarin	pie3
U+4E3F.0	fEnglish	[1] left stroke in Chinese writing [2] Kangxi radical 4
U+4E40.0	fUTF8	乀
U+4E40.0	fCNS11643	4-2122
U+4E40.0	fGB	E-212C
U+4E40.0	fR/S	 4.0; 4.1
U+4E40.0	fTotalStrokes	 1; 2
U+4E40.0	fCangjie	MO
U+4E40.0	fFourCorner	17030
U+4E40.0	fCantonese	fat1
U+4E40.0	fMandarin	fu2
U+4E40.0	fEnglish	stretch
U+4E41.0	fUTF8	乁
U+4E41.0	fCNS11643	4-2123
U+4E41.0	fGB	1-7D7B
U+4E41.0	fR/S	 4.0; 4.1
U+4E41.0	fTotalStrokes	 1; 2
U+4E41.0	fCangjie	MO
U+4E41.0	fFourCorner	17030
U+4E41.0	fCantonese	ji4
U+4E41.0	fMandarin	yi2
U+4E41.0	fEnglish	right stroke in Chinese writing
U+4E42.0	fUTF8	乂
U+4E42.0	fCNS11643	2-2121
U+4E42.0	fBig5	C940
U+4E42.0	fGB	3-302A
U+4E42.0	fR/S	 4.1
U+4E42.0	fTotalStrokes	 2
U+4E42.0	fCangjie	K
U+4E42.0	fFourCorner	40000
U+4E42.0	fHakka	  ngai5;  ngi5; ngi6; ngi3; ngioi5; ngiai5
U+4E42.0	fCantonese	ngaai6; aai6 (also); ai6; ngoi6; oi6
U+4E42.0	fMandarin	yi4; ai4; wu3
U+4E42.0	fEnglish	[1] interchangeable with &#x827E; [2] ancient form of &#x5208;
U+4E43.0	fUTF8	乃
U+4E43.0	fCNS11643	1-4425
U+4E43.0	fBig5	A444
U+4E43.0	fGB	0-444B
U+4E43.0	fR/S	 4.1
U+4E43.0	fTotalStrokes	 2
U+4E43.0	fCangjie	NHS
U+4E43.0	fFourCorner	17227
U+4E43.0	fHakka	  nai1;  nai3; lai1; lai3; ne3
U+4E43.0	fCantonese	naai5; laai5 (also)
U+4E43.0	fMandarin	nai3
U+4E43.0	fEnglish	[1] you; your; his [2] your, followed by a noun, has the force of a reflexive pronoun referring to the person speaking, consequently [3] then; only then; after that; afterwards [4] be (in definite assertion); be really; be indeed [5] even [6] so; such [7] actually; really; such; formerly; as it turned out; after all; namely [8] then
U+4E43.1	fUTF8	乃
U+4E43.1	fCNS11643	1-4425
U+4E43.1	fBig5	A444
U+4E43.1	fGB	0-444B
U+4E43.1	fR/S	 4.1
U+4E43.1	fTotalStrokes	 2
U+4E43.1	fCangjie	NHS
U+4E43.1	fFourCorner	17227
U+4E43.1	fCantonese	oi2; ngoi2 (also)
U+4E44.0	fUTF8	乄
U+4E44.0	fGB	E-212D
U+4E44.0	fR/S	 4.1
U+4E44.0	fTotalStrokes	 2
U+4E44.0	fCangjie	XXKU
U+4E44.0	fMandarin	wu3
U+4E44.0	fEnglish	 find a place to dwell; sum; a measurement of paper do
U+4E45.0	fUTF8	久
U+4E45.0	fCNS11643	1-443C
U+4E45.0	fBig5	A45B
U+4E45.0	fGB	0-3E43
U+4E45.0	fR/S	 4.2
U+4E45.0	fTotalStrokes	 3
U+4E45.0	fCangjie	NO
U+4E45.0	fFourCorner	27800
U+4E45.0	fHakka	  giu3; kieu3
U+4E45.0	fCantonese	gau2
U+4E45.0	fMandarin	jiu3
U+4E45.0	fEnglish	[1] long [2] for a long time (ago) [3] detain someone long [4] old
U+4E46.0	fUTF8	乆
U+4E46.0	fCNS11643	3-2130
U+4E46.0	fGB	E-212E
U+4E46.0	fR/S	 4.2
U+4E46.0	fTotalStrokes	 3
U+4E46.0	fCangjie	VO
U+4E46.0	fFourCorner	28000
U+4E46.0	fHakka	 giu3
U+4E46.0	fCantonese	gau2
U+4E46.0	fMandarin	jiu3
U+4E46.0	fEnglish	long time (ago); time passage, grow late
U+4E47.0	fUTF8	乇
U+4E47.0	fCNS11643	2-2128
U+4E47.0	fBig5	C947
U+4E47.0	fGB	0-5831
U+4E47.0	fR/S	 4.2
U+4E47.0	fTotalStrokes	 3
U+4E47.0	fCangjie	HP
U+4E47.0	fFourCorner	20714
U+4E47.0	fHakka	 tok7; kiet7
U+4E47.0	fCantonese	zaak6
U+4E47.0	fMandarin	zhe2; ne4; zhe4
U+4E47.0	fEnglish	 depend on; entrust with
U+4E47.1	fUTF8	乇
U+4E47.1	fCNS11643	2-2128
U+4E47.1	fBig5	C947
U+4E47.1	fGB	0-5831
U+4E47.1	fR/S	 4.2
U+4E47.1	fTotalStrokes	 3
U+4E47.1	fCangjie	HP
U+4E47.1	fFourCorner	20714
U+4E47.1	fCantonese	tok3
U+4E47.1	fMandarin	tuo1
U+4E48.0	fUTF8	么
U+4E48.0	fCNS11643	1-443D
U+4E48.0	fBig5	A45C
U+4E48.0	fGB	0-4334
U+4E48.0	fR/S	 4.2
U+4E48.0	fTotalStrokes	 3
U+4E48.0	fCangjie	HI
U+4E48.0	fFourCorner	20732
U+4E48.0	fHakka	 jau1; mo2; rau1; jeu1; mo1; ma3; me3
U+4E48.0	fCantonese	maa1; jiu1; mo1
U+4E48.0	fMandarin	me5; yao1; mo2; mo5; ma5; mo3
U+4E48.0	fEnglish	simplified form of &#x9EBD;
U+4E48.1	fUTF8	么
U+4E48.1	fCNS11643	1-443D
U+4E48.1	fBig5	A45C
U+4E48.1	fGB	0-4334
U+4E48.1	fR/S	 4.2
U+4E48.1	fTotalStrokes	 3
U+4E48.1	fCangjie	HI
U+4E48.1	fFourCorner	20732
U+4E49.0	fUTF8	义
U+4E49.0	fCNS11643	3-212E
U+4E49.0	fGB	0-5265
U+4E49.0	fR/S	 4.2; 3.2
U+4E49.0	fTotalStrokes	 3
U+4E49.0	fCangjie	IK
U+4E49.0	fFourCorner	40000
U+4E49.0	fHakka	 ngi5
U+4E49.0	fCantonese	ji6
U+4E49.0	fMandarin	yi4; yi2
U+4E49.0	fEnglish	simplified form of &#x7FA9;
U+4E4A.0	fUTF8	乊
U+4E4A.0	fGB	1-7D6B
U+4E4A.0	fR/S	 4.2
U+4E4A.0	fTotalStrokes	 3
U+4E4A.0	fCangjie	HC
U+4E4A.0	fEnglish	
U+4E4B.0	fUTF8	之
U+4E4B.0	fCNS11643	1-4466
U+4E4B.0	fBig5	A4A7
U+4E4B.0	fGB	0-562E
U+4E4B.0	fR/S	 4.3
U+4E4B.0	fTotalStrokes	 4
U+4E4B.0	fCangjie	INO
U+4E4B.0	fFourCorner	30307
U+4E4B.0	fHakka	  zih1;  zu1; zhi1; zi1; tje1
U+4E4B.0	fCantonese	zi1
U+4E4B.0	fMandarin	zhi1
U+4E4B.0	fEnglish	[1] go to; leave for; arrive at [2] zigzag; winding; S-shaped road [3] expletive [4] it; him her; them (third person objective case) [5] this; that; these; those [6] possessive particle; of [7] until [8] to
U+4E4C.0	fUTF8	乌
U+4E4C.0	fGB	0-4E5A
U+4E4C.0	fR/S	 4.3
U+4E4C.0	fTotalStrokes	 4
U+4E4C.0	fCangjie	PVSM
U+4E4C.0	fHakka	 vu1
U+4E4C.0	fCantonese	wu1
U+4E4C.0	fMandarin	wu1; ya1; wu4; yan1
U+4E4C.0	fEnglish	simplified form of &#x70CF;
U+4E4D.0	fUTF8	乍
U+4E4D.0	fCNS11643	1-4565
U+4E4D.0	fBig5	A545
U+4E4D.0	fGB	0-5527
U+4E4D.0	fR/S	 4.4
U+4E4D.0	fTotalStrokes	 5
U+4E4D.0	fCangjie	OS
U+4E4D.0	fFourCorner	80211
U+4E4D.0	fHakka	  za5; za3
U+4E4D.0	fCantonese	zaa3; zaa6 (also); zok3
U+4E4D.0	fMandarin	zha4; zuo4
U+4E4D.0	fEnglish	[1] at first; for the first time; newly [2] suddenly; unexpectedly; abruptly; inadvertently
U+4E4E.0	fUTF8	乎
U+4E4E.0	fCNS11643	1-4567
U+4E4E.0	fBig5	A547
U+4E4E.0	fGB	0-3A75
U+4E4E.0	fR/S	 4.4
U+4E4E.0	fTotalStrokes	 5
U+4E4E.0	fCangjie	HFD
U+4E4E.0	fFourCorner	20409
U+4E4E.0	fHakka	  fu2; fu3
U+4E4E.0	fCantonese	fu4; fu1; wu4
U+4E4E.0	fMandarin	hu1
U+4E4E.0	fEnglish	[1] to; at; in; from; than [2] interrogative particle [3] exclamatory final particle
U+4E4E.1	fUTF8	乎
U+4E4E.1	fCNS11643	1-4567
U+4E4E.1	fBig5	A547
U+4E4E.1	fGB	0-3A75
U+4E4E.1	fR/S	 4.4
U+4E4E.1	fTotalStrokes	 5
U+4E4E.1	fCangjie	HFD
U+4E4E.1	fFourCorner	20409
U+4E4E.1	fMandarin	hu2; hu3
U+4E4E.2	fUTF8	乎
U+4E4E.2	fCNS11643	1-4567
U+4E4E.2	fBig5	A547
U+4E4E.2	fGB	0-3A75
U+4E4E.2	fR/S	 4.4
U+4E4E.2	fTotalStrokes	 5
U+4E4E.2	fCangjie	HFD
U+4E4E.2	fFourCorner	20409
U+4E4E.2	fMandarin	hu5
U+4E4F.0	fUTF8	乏
U+4E4F.0	fCNS11643	1-4566
U+4E4F.0	fBig5	A546
U+4E4F.0	fGB	0-3726
U+4E4F.0	fR/S	 4.4
U+4E4F.0	fTotalStrokes	 5
U+4E4F.0	fCangjie	HINO
U+4E4F.0	fFourCorner	20307
U+4E4F.0	fHakka	  fat8; fap8
U+4E4F.0	fCantonese	fat6
U+4E4F.0	fMandarin	fa2
U+4E4F.0	fEnglish	[1] in want of; deficient; lack [2] exhausted; tired [3] poor; poverty-stricken [4] useless [5] weak and without strength
U+4E50.0	fUTF8	乐
U+4E50.0	fGB	0-4056
U+4E50.0	fR/S	 4.4
U+4E50.0	fTotalStrokes	 5
U+4E50.0	fCangjie	HVD
U+4E50.0	fHakka	 ngok8; lok8; ngau5
U+4E50.0	fCantonese	lok6; ngok6; ok6 (also); ngaau6; aau6 (also); ngaau4 (also); aau4 (also); lok3; ngau6
U+4E50.0	fMandarin	le4; yao4; luo4; liao2
U+4E50.0	fEnglish	simplified form of &#x6A02;
U+4E50.1	fUTF8	乐
U+4E50.1	fGB	0-4056
U+4E50.1	fR/S	 4.4
U+4E50.1	fTotalStrokes	 5
U+4E50.1	fCangjie	HVD
U+4E50.1	fMandarin	yue4; liao4; lo4
U+4E51.0	fUTF8	乑
U+4E51.0	fCNS11643	4-2227
U+4E51.0	fGB	E-212F
U+4E51.0	fR/S	 4.5
U+4E51.0	fTotalStrokes	 6
U+4E51.0	fCangjie	OHHO
U+4E51.0	fFourCorner	20230
U+4E51.0	fCantonese	jam4
U+4E51.0	fMandarin	zhong4
U+4E51.0	fEnglish	stand side by side
U+4E52.0	fUTF8	乒
U+4E52.0	fCNS11643	1-4724
U+4E52.0	fBig5	A5E2
U+4E52.0	fGB	0-4639
U+4E52.0	fR/S	 4.5
U+4E52.0	fTotalStrokes	 6
U+4E52.0	fCangjie	OMH
U+4E52.0	fFourCorner	72201
U+4E52.0	fHakka	 pin1; bin1; bit7; bit8
U+4E52.0	fCantonese	bing1; ping1
U+4E52.0	fMandarin	ping1
U+4E52.0	fEnglish	[1] used with 'pong' for sound 'ping pong' [2] used with 'pong' for table tennis
U+4E53.0	fUTF8	乓
U+4E53.0	fCNS11643	1-4725
U+4E53.0	fBig5	A5E3
U+4E53.0	fGB	0-4552
U+4E53.0	fR/S	 4.5
U+4E53.0	fTotalStrokes	 6
U+4E53.0	fCangjie	OMI
U+4E53.0	fFourCorner	72031
U+4E53.0	fHakka	 pong1; bang1; bak8
U+4E53.0	fCantonese	bam1; pong1
U+4E53.0	fMandarin	pang1; pang5
U+4E53.0	fEnglish	[1] used with 'ping' for table tennis [2] used with 'ping' for sound 'ping pong'
U+4E54.0	fUTF8	乔
U+4E54.0	fCNS11643	3-2345
U+4E54.0	fGB	0-4747
U+4E54.0	fR/S	 37.3; 4.5
U+4E54.0	fTotalStrokes	 6
U+4E54.0	fCangjie	HKLL
U+4E54.0	fFourCorner	20220
U+4E54.0	fHakka	 kiau2
U+4E54.0	fCantonese	kiu4
U+4E54.0	fMandarin	gui4; qiao2; jiao1
U+4E54.0	fEnglish	simplified form of &#x55AC;
U+4E55.0	fUTF8	乕
U+4E55.0	fGB	E-2130
U+4E55.0	fR/S	  4.6
U+4E55.0	fTotalStrokes	 7
U+4E55.0	fCangjie	HQB
U+4E55.0	fCantonese	fu2
U+4E55.0	fMandarin	hu3; hu4
U+4E55.0	fEnglish	 tiger brave; fierce a Chinese family name
U+4E56.0	fUTF8	乖
U+4E56.0	fCNS11643	1-4B65
U+4E56.0	fBig5	A8C4
U+4E56.0	fGB	0-3954
U+4E56.0	fR/S	 4.7
U+4E56.0	fTotalStrokes	 8
U+4E56.0	fCangjie	HJLP
U+4E56.0	fFourCorner	20111
U+4E56.0	fHakka	  gwai1;  gai1; kuai1
U+4E56.0	fCantonese	gwaai1
U+4E56.0	fMandarin	guai1
U+4E56.0	fEnglish	[1] obedient; well-behaved; lovely [2] cunning; artful; crafty; wily; shrewd [3] oppose; contradict; be at variance [4] perverse; obstinate; untoward; sulky
U+4E57.0	fUTF8	乗
U+4E57.0	fCNS11643	3-2B22
U+4E57.0	fGB	E-2131
U+4E57.0	fR/S	 4.8
U+4E57.0	fTotalStrokes	 9
U+4E57.0	fCangjie	HDT
U+4E57.0	fFourCorner	20900
U+4E57.0	fHakka	  shin5; sin2; sin5; shin1; shin2; sen1; sin1; sen2; sen5; sin3
U+4E57.0	fCantonese	sing4 sing6
U+4E57.0	fMandarin	cheng2 sheng4
U+4E57.0	fEnglish	same as &#x4E58;
U+4E58.0	fUTF8	乘
U+4E58.0	fCNS11643	1-537D
U+4E58.0	fBig5	ADBC
U+4E58.0	fGB	0-334B
U+4E58.0	fR/S	 4.9
U+4E58.0	fTotalStrokes	 10
U+4E58.0	fCangjie	HDLP
U+4E58.0	fFourCorner	20901
U+4E58.0	fHakka	  shin5; sin2; sin5; shin1; shin2; sen1; sin1; sen2; sen5; sin3
U+4E58.0	fCantonese	sing4
U+4E58.0	fMandarin	cheng2
U+4E58.0	fEnglish	[1] ride; mount; go aboard (horse, carriage, sedan chair, ship, airplane) [2] avail oneself of; take advantage of (chance, wind, current) [3] multiply; multiplication
U+4E58.1	fUTF8	乘
U+4E58.1	fCNS11643	1-537D
U+4E58.1	fBig5	ADBC
U+4E58.1	fGB	0-334B
U+4E58.1	fR/S	 4.9
U+4E58.1	fTotalStrokes	 10
U+4E58.1	fCangjie	HDLP
U+4E58.1	fFourCorner	20901
U+4E58.1	fCantonese	sing6
U+4E58.1	fMandarin	sheng4
U+4E58.1	fEnglish	[1] historical records [2] ancient carriage; number of carriages [3] Buddhist teaching; a main division of Buddhist schools [4] team of four horses
U+4E59.0	fUTF8	乙
U+4E59.0	fCNS11643	1-4422
U+4E59.0	fBig5	A441
U+4E59.0	fGB	0-5252
U+4E59.0	fR/S	 5.0
U+4E59.0	fTotalStrokes	 1
U+4E59.0	fCangjie	NU
U+4E59.0	fFourCorner	17710
U+4E59.0	fHakka	 jet7; jat7; rit7; jit7; ret7
U+4E59.0	fCantonese	jyut3; jyut6
U+4E59.0	fMandarin	yi3
U+4E59.0	fEnglish	[1] second [2] the second celestial stems [3] one [4] someone; a suppositional person [5] an ancient Chinese family name
U+4E5A.0	fUTF8	乚
U+4E5A.0	fCNS11643	4-2124
U+4E5A.0	fGB	E-2132
U+4E5A.0	fR/S	 5.0
U+4E5A.0	fTotalStrokes	 1
U+4E5A.0	fCangjie	LU
U+4E5A.0	fFourCorner	20710
U+4E5A.0	fHakka	 jun3; run3
U+4E5A.0	fCantonese	jyut3; jan2
U+4E5A.0	fMandarin	yin3
U+4E5A.0	fEnglish	 hidden; mysterious; secret conceal small minute
U+4E5B.0	fUTF8	乛
U+4E5B.0	fGB	1-7D70
U+4E5B.0	fR/S	 5.0
U+4E5B.0	fTotalStrokes	 1
U+4E5B.0	fCangjie	XXMH
U+4E5B.0	fEnglish	 kwukyel
U+4E5C.0	fUTF8	乜
U+4E5C.0	fCNS11643	2-2122
U+4E5C.0	fBig5	C941
U+4E5C.0	fGB	0-583F
U+4E5C.0	fR/S	 5.1
U+4E5C.0	fTotalStrokes	 2
U+4E5C.0	fCangjie	PN
U+4E5C.0	fFourCorner	47712
U+4E5C.0	fHakka	  mak7; mat7; ma6; ma3; me1; ja3
U+4E5C.0	fCantonese	me1; me5; le6 (also); me6 (also); me2; nip2; mi1
U+4E5C.0	fMandarin	mie1
U+4E5C.0	fEnglish	[1] squint [2] what? Why huh? [3] what?
U+4E5C.1	fUTF8	乜
U+4E5C.1	fCNS11643	2-2122
U+4E5C.1	fBig5	C941
U+4E5C.1	fGB	0-583F
U+4E5C.1	fR/S	 5.1
U+4E5C.1	fTotalStrokes	 2
U+4E5C.1	fCangjie	PN
U+4E5C.1	fFourCorner	47712
U+4E5C.1	fCantonese	mat1
U+4E5C.1	fMandarin	nie4
U+4E5C.1	fEnglish	a rare Chinese family name
U+4E5C.2	fUTF8	乜
U+4E5C.2	fCNS11643	2-2122
U+4E5C.2	fBig5	C941
U+4E5C.2	fGB	0-583F
U+4E5C.2	fR/S	 5.1
U+4E5C.2	fTotalStrokes	 2
U+4E5C.2	fCangjie	PN
U+4E5C.2	fFourCorner	47712
U+4E5C.2	fCantonese	ne6
U+4E5D.0	fUTF8	九
U+4E5D.0	fCNS11643	1-4426
U+4E5D.0	fBig5	A445
U+4E5D.0	fGB	0-3E45
U+4E5D.0	fR/S	 5.1
U+4E5D.0	fTotalStrokes	 2
U+4E5D.0	fCangjie	KN
U+4E5D.0	fFourCorner	40017
U+4E5D.0	fHakka	  giu3; kieu3
U+4E5D.0	fCantonese	gau2
U+4E5D.0	fMandarin	jiu3; jiu1; qiu2
U+4E5D.0	fEnglish	[1] nine [2] unit for distance
U+4E5E.0	fUTF8	乞
U+4E5E.0	fCNS11643	1-443F
U+4E5E.0	fBig5	A45E
U+4E5E.0	fGB	0-4672
U+4E5E.0	fR/S	 5.2
U+4E5E.0	fTotalStrokes	 3
U+4E5E.0	fCangjie	ON
U+4E5E.0	fFourCorner	80717
U+4E5E.0	fHakka	 ket7;  kiet7; k'ie3; ngiek7; ngiet7
U+4E5E.0	fCantonese	hat1; hei3
U+4E5E.0	fMandarin	qi3; qi4; you4
U+4E5E.0	fEnglish	 ask for alms; beg; request; entreat; pray humbly; beggar
U+4E5F.0	fUTF8	也
U+4E5F.0	fCNS11643	1-443E
U+4E5F.0	fBig5	A45D
U+4E5F.0	fGB	0-5232
U+4E5F.0	fR/S	 5.2
U+4E5F.0	fTotalStrokes	 3
U+4E5F.0	fCangjie	PD
U+4E5F.0	fFourCorner	44712
U+4E5F.0	fHakka	  ja1; ja3; ra1; ra3; ja6; ra6; ja5
U+4E5F.0	fCantonese	jaa5
U+4E5F.0	fMandarin	ye3
U+4E5F.0	fEnglish	[1] and; also; besides; either; too [2] used in sentence beginning, like English might as well [3] still [4] even [5] an expletive in Chinese writing [6] indicating what s.t. is [7] indicating a caesura or break in mid-sentence
U+4E60.0	fUTF8	习
U+4E60.0	fGB	0-4F30
U+4E60.0	fR/S	 15.1; 5.2
U+4E60.0	fTotalStrokes	 3
U+4E60.0	fCangjie	SIM
U+4E60.0	fHakka	 sip8
U+4E60.0	fCantonese	zaap6
U+4E60.0	fMandarin	xi2
U+4E60.0	fEnglish	simplified form of &#x7FD2;
U+4E61.0	fUTF8	乡
U+4E61.0	fGB	0-4F67
U+4E61.0	fR/S	 52.0
U+4E61.0	fTotalStrokes	 3
U+4E61.0	fCangjie	VVH
U+4E61.0	fCantonese	hoeng1; hoeng3
U+4E61.0	fMandarin	xiang1; xiang3; xiang4
U+4E61.0	fEnglish	simplified form of &#x9109;
U+4E62.0	fUTF8	乢
U+4E62.0	fCNS11643	3-2146
U+4E62.0	fGB	E-2133
U+4E62.0	fR/S	 46.1; 5.3
U+4E62.0	fTotalStrokes	 4
U+4E62.0	fCangjie	UU
U+4E62.0	fFourCorner	22710
U+4E62.0	fHakka	 jat7
U+4E62.0	fCantonese	goi3
U+4E62.0	fMandarin	gai4
U+4E62.0	fEnglish	 lid; cover hide cover umbrella
U+4E63.0	fUTF8	乣
U+4E63.0	fCNS11643	3-2145
U+4E63.0	fGB	3-3E73
U+4E63.0	fR/S	 5.3
U+4E63.0	fTotalStrokes	 4
U+4E63.0	fCangjie	VIU
U+4E63.0	fFourCorner	22710
U+4E63.0	fCantonese	gau2; kau2
U+4E63.0	fMandarin	diu1; jiu3
U+4E64.0	fUTF8	乤
U+4E64.0	fGB	1-7D6A
U+4E64.0	fR/S	 5.3
U+4E64.0	fTotalStrokes	 4
U+4E64.0	fCangjie	MYN
U+4E64.0	fCantonese	haa6
U+4E64.0	fEnglish	 first character of Korean place name "Halpho"
U+4E65.0	fUTF8	乥
U+4E65.0	fGB	1-7D6C
U+4E65.0	fR/S	 5.3
U+4E65.0	fTotalStrokes	 4
U+4E65.0	fCangjie	HCN
U+4E65.0	fCantonese	fu4
U+4E65.0	fEnglish	 kwukyel
U+4E66.0	fUTF8	书
U+4E66.0	fGB	0-4A69
U+4E66.0	fR/S	 5.3; 2.3
U+4E66.0	fTotalStrokes	 4
U+4E66.0	fCangjie	IDS
U+4E66.0	fHakka	 shu1
U+4E66.0	fCantonese	syu1
U+4E66.0	fMandarin	shu1
U+4E66.0	fEnglish	simplified form of &#x66F8;
U+4E67.0	fUTF8	乧
U+4E67.0	fGB	E-2134
U+4E67.0	fR/S	 5.4
U+4E67.0	fTotalStrokes	 5
U+4E67.0	fCangjie	YJN
U+4E67.0	fCantonese	dau2
U+4E67.0	fMandarin	dou3
U+4E67.0	fEnglish	
U+4E68.0	fUTF8	乨
U+4E68.0	fCNS11643	3-2264
U+4E68.0	fGB	5-303F
U+4E68.0	fR/S	 5.5
U+4E68.0	fTotalStrokes	 6
U+4E68.0	fCangjie	IRU
U+4E68.0	fFourCorner	22610
U+4E68.0	fCantonese	ci2
U+4E68.0	fMandarin	shi3
U+4E68.0	fEnglish	 the beginning start; begin; be the first
U+4E69.0	fUTF8	乩
U+4E69.0	fCNS11643	1-4726
U+4E69.0	fBig5	A5E4
U+4E69.0	fGB	0-5840
U+4E69.0	fR/S	 5.5
U+4E69.0	fTotalStrokes	 6
U+4E69.0	fCangjie	YRU
U+4E69.0	fFourCorner	22610
U+4E69.0	fHakka	  gi1; ki1
U+4E69.0	fCantonese	gei1
U+4E69.0	fMandarin	ji1
U+4E69.0	fEnglish	 divine; resolve doubts by an application to spiritual beings
U+4E6A.0	fUTF8	乪
U+4E6A.0	fCNS11643	4-2228
U+4E6A.0	fGB	1-7C5A
U+4E6A.0	fR/S	  5.5
U+4E6A.0	fTotalStrokes	 6
U+4E6A.0	fCangjie	NW
U+4E6A.0	fFourCorner	16710
U+4E6A.0	fCantonese	kek6
U+4E6A.0	fMandarin	nang1; nang2
U+4E6A.0	fEnglish	a bend in a river
U+4E6B.0	fUTF8	乫
U+4E6B.0	fGB	1-7D21
U+4E6B.0	fR/S	 5.5
U+4E6B.0	fTotalStrokes	 6
U+4E6B.0	fCangjie	KRN
U+4E6B.0	fCantonese	gaa1
U+4E6B.0	fMandarin	jia1
U+4E6B.0	fEnglish	 used in place names
U+4E6C.0	fUTF8	乬
U+4E6C.0	fGB	1-7D24
U+4E6C.0	fR/S	 5.5
U+4E6C.0	fTotalStrokes	 6
U+4E6C.0	fCangjie	SSN
U+4E6C.0	fCantonese	geoi6
U+4E6C.0	fEnglish	 hang
U+4E6D.0	fUTF8	乭
U+4E6D.0	fGB	1-7D32
U+4E6D.0	fR/S	 5.5
U+4E6D.0	fTotalStrokes	 6
U+4E6D.0	fCangjie	MRN
U+4E6D.0	fCantonese	sek6
U+4E6D.0	fMandarin	shi2
U+4E6D.0	fEnglish	 rock used in Korean names
U+4E6E.0	fUTF8	乮
U+4E6E.0	fGB	1-7D3F
U+4E6E.0	fR/S	 5.5
U+4E6E.0	fTotalStrokes	 6
U+4E6E.0	fCangjie	HLN
U+4E6E.0	fCantonese	maau5
U+4E6E.0	fEnglish	
U+4E6F.0	fUTF8	乯
U+4E6F.0	fGB	1-7D7A
U+4E6F.0	fR/S	 5.5
U+4E6F.0	fTotalStrokes	 6
U+4E6F.0	fCangjie	HFDN
U+4E6F.0	fCantonese	fu4
U+4E6F.0	fEnglish	
U+4E70.0	fUTF8	买
U+4E70.0	fGB	0-4272
U+4E70.0	fR/S	 5.5; 37.3
U+4E70.0	fTotalStrokes	 6
U+4E70.0	fCangjie	NYK
U+4E70.0	fHakka	 mai1; mai3
U+4E70.0	fCantonese	maai5
U+4E70.0	fMandarin	mai3
U+4E70.0	fEnglish	simplified form of &#x8CB7;
U+4E71.0	fUTF8	乱
U+4E71.0	fCNS11643	3-2440
U+4E71.0	fGB	0-4252
U+4E71.0	fR/S	 5.6
U+4E71.0	fTotalStrokes	 7
U+4E71.0	fCangjie	HRU
U+4E71.0	fFourCorner	22610
U+4E71.0	fHakka	 lon5; lon6
U+4E71.0	fCantonese	lyun6; lyun2 (also)
U+4E71.0	fMandarin	luan4
U+4E71.0	fEnglish	simplified form of &#x4E82;
U+4E72.0	fUTF8	乲
U+4E72.0	fGB	1-7D54
U+4E72.0	fR/S	 5.6
U+4E72.0	fTotalStrokes	 7
U+4E72.0	fCangjie	ION
U+4E72.0	fCantonese	ci3
U+4E72.0	fEnglish	 Korean character
U+4E73.0	fUTF8	乳
U+4E73.0	fCNS11643	1-4B66
U+4E73.0	fBig5	A8C5
U+4E73.0	fGB	0-4869
U+4E73.0	fR/S	 5.7
U+4E73.0	fTotalStrokes	 8
U+4E73.0	fCangjie	BDU
U+4E73.0	fFourCorner	22410
U+4E73.0	fHakka	  nen5; jui1; jui3; lien5; nen3; rui3; len1; len5
U+4E73.0	fCantonese	jyu5
U+4E73.0	fMandarin	ru3
U+4E73.0	fEnglish	[1] breasts; nipple [2] milk [3] any milk-like liquid [4] the young of animals, birds, etc.; young; newly born [5] give birth [6] suckle (babies) [7] triturate
U+4E74.0	fUTF8	乴
U+4E74.0	fCNS11643	4-2531
U+4E74.0	fGB	E-2135
U+4E74.0	fR/S	 5.7
U+4E74.0	fTotalStrokes	 8; 7
U+4E74.0	fCangjie	QLN
U+4E74.0	fFourCorner	52710
U+4E74.0	fCantonese	zyut6
U+4E74.0	fMandarin	xue2
U+4E74.0	fEnglish	[1] grasp hold [2] pick up
U+4E75.0	fUTF8	乵
U+4E75.0	fCNS11643	4-2530
U+4E75.0	fGB	E-2136
U+4E75.0	fR/S	  5.7
U+4E75.0	fTotalStrokes	 8; 7
U+4E75.0	fCangjie	YJU
U+4E75.0	fFourCorner	02410
U+4E75.0	fCantonese	jim5
U+4E75.0	fMandarin	yan3
U+4E75.0	fEnglish	 enter make progress; advance urge forward
U+4E76.0	fUTF8	乶
U+4E76.0	fGB	1-7D61
U+4E76.0	fR/S	 5.7
U+4E76.0	fTotalStrokes	 8
U+4E76.0	fCangjie	IBN
U+4E76.0	fCantonese	fu2
U+4E76.0	fMandarin	fu3
U+4E76.0	fEnglish	 used in place name Pholha
U+4E77.0	fUTF8	乷
U+4E77.0	fGB	E-2137
U+4E77.0	fR/S	 5.7
U+4E77.0	fTotalStrokes	 8
U+4E77.0	fCangjie	EHN
U+4E77.0	fCantonese	saa1
U+4E77.0	fMandarin	sha1
U+4E77.0	fEnglish	
U+4E78.0	fUTF8	乸
U+4E78.0	fGB	1-7C40
U+4E78.0	fR/S	 5.7; 80.3
U+4E78.0	fTotalStrokes	 8
U+4E78.0	fCangjie	PDWYI
U+4E78.0	fHakka	  ma2; zia3; ma3
U+4E78.0	fCantonese	naa2; laa2 (also)
U+4E78.0	fMandarin	na3
U+4E78.0	fEnglish	[1] feminine suffix [2] mother
U+4E79.0	fUTF8	乹
U+4E79.0	fCNS11643	3-2B23
U+4E79.0	fGB	E-2138
U+4E79.0	fR/S	 5.8
U+4E79.0	fTotalStrokes	 9
U+4E79.0	fCangjie	JJU
U+4E79.0	fFourCorner	42410
U+4E79.0	fHakka	 gon1; ken2; kien2
U+4E79.0	fCantonese	gon1
U+4E79.0	fMandarin	gan1; qian2
U+4E79.0	fEnglish	to clean; dried up
U+4E7A.0	fUTF8	乺
U+4E7A.0	fGB	1-7D4F
U+4E7A.0	fR/S	 5.8
U+4E7A.0	fTotalStrokes	 9
U+4E7A.0	fCangjie	XHLN
U+4E7A.0	fCantonese	so2
U+4E7A.0	fEnglish	 painting tool place name
U+4E7B.0	fUTF8	乻
U+4E7B.0	fGB	1-7D74
U+4E7B.0	fR/S	 5.8
U+4E7B.0	fTotalStrokes	 9
U+4E7B.0	fCangjie	YYN
U+4E7B.0	fCantonese	jyu1
U+4E7B.0	fEnglish	
U+4E7C.0	fUTF8	乼
U+4E7C.0	fGB	1-7D5B
U+4E7C.0	fR/S	 5.8
U+4E7C.0	fTotalStrokes	 9
U+4E7C.0	fCangjie	EGN
U+4E7C.0	fCantonese	zyu3
U+4E7C.0	fEnglish	 rope
U+4E7D.0	fUTF8	乽
U+4E7D.0	fGB	1-7D55
U+4E7D.0	fR/S	 5.9
U+4E7D.0	fTotalStrokes	 10
U+4E7D.0	fCangjie	JAN
U+4E7D.0	fCantonese	ze2
U+4E7D.0	fEnglish	
U+4E7E.0	fUTF8	乾
U+4E7E.0	fCNS11643	1-5871
U+4E7E.0	fBig5	B0AE
U+4E7E.0	fGB	0-472C
U+4E7E.0	fR/S	 5.10
U+4E7E.0	fTotalStrokes	 11
U+4E7E.0	fCangjie	JJON
U+4E7E.0	fFourCorner	48417
U+4E7E.0	fHakka	  gon1; kien2;  ken2; k'ian3
U+4E7E.0	fCantonese	kin4; gon1
U+4E7E.0	fMandarin	gan1
U+4E7E.0	fEnglish	[1] clean [2] dry; dried (up) [3] exhausted, finished; cleaned out [4] dry, preserved meat, fruit [5] scold [6] adopted (relation) not related by blood [7] cash in place of gift [8] in vain [9] without tears or other liquids [10] only; with nothing else
U+4E7E.1	fUTF8	乾
U+4E7E.1	fCNS11643	1-5871
U+4E7E.1	fBig5	B0AE
U+4E7E.1	fGB	0-472C
U+4E7E.1	fR/S	 5.10
U+4E7E.1	fTotalStrokes	 11
U+4E7E.1	fCangjie	JJON
U+4E7E.1	fFourCorner	48417
U+4E7E.1	fHakka	 kian2
U+4E7E.1	fMandarin	qian2
U+4E7E.1	fEnglish	[1] the first of the Eight Diagrams [2] heaven; male; father; sovereign [3] a Chinese family name
U+4E7F.0	fUTF8	乿
U+4E7F.0	fCNS11643	2-334E
U+4E7F.0	fBig5	D44B
U+4E7F.0	fGB	5-3040
U+4E7F.0	fR/S	 5.10
U+4E7F.0	fTotalStrokes	 11
U+4E7F.0	fCangjie	BFU
U+4E7F.0	fFourCorner	22910
U+4E7F.0	fCantonese	zi6
U+4E7F.0	fMandarin	chi4; zhi4; luan4
U+4E7F.0	fEnglish	 cure; heal
U+4E80.0	fUTF8	亀
U+4E80.0	fCNS11643	3-396C
U+4E80.0	fGB	E-2139
U+4E80.0	fR/S	 5.10; 213.0
U+4E80.0	fTotalStrokes	 11
U+4E80.0	fCangjie	NWLU
U+4E80.0	fFourCorner	27710
U+4E80.0	fHakka	 kwui1
U+4E80.0	fCantonese	gwai1; gwan1; gau1
U+4E80.0	fMandarin	gui1; jiu1
U+4E80.0	fEnglish	turtle or tortoise; cuckold
U+4E81.0	fUTF8	亁
U+4E81.0	fCNS11643	3-396E
U+4E81.0	fGB	E-213A
U+4E81.0	fR/S	 5.11
U+4E81.0	fTotalStrokes	 12
U+4E81.0	fCangjie	JJON
U+4E81.0	fFourCorner	48410
U+4E81.0	fMandarin	gan1
U+4E81.0	fEnglish	 dry dried; as opposed to fresh
U+4E82.0	fUTF8	亂
U+4E82.0	fCNS11643	1-632A
U+4E82.0	fBig5	B6C3
U+4E82.0	fGB	1-4252
U+4E82.0	fR/S	 5.12
U+4E82.0	fTotalStrokes	 13
U+4E82.0	fCangjie	BBU
U+4E82.0	fFourCorner	22210
U+4E82.0	fHakka	  lon5; lon6; lon3
U+4E82.0	fCantonese	lyun6; lyun2 (also)
U+4E82.0	fMandarin	luan4
U+4E82.0	fEnglish	[1] chaos; anarchy; distraction; confusion [2] rebellion; revolt; insurrection [3] confused; perplexed; agitated; disarranged; raveled [4] out of order; out of sorts; throw into disorder; confuse; confound
U+4E83.0	fUTF8	亃
U+4E83.0	fCNS11643	2-4156
U+4E83.0	fBig5	DCB1
U+4E83.0	fGB	5-3042
U+4E83.0	fR/S	 5.12
U+4E83.0	fTotalStrokes	 13
U+4E83.0	fCangjie	FQU
U+4E83.0	fFourCorner	92210
U+4E83.0	fCantonese	leon5; leon6
U+4E83.0	fMandarin	lin2; lin3
U+4E83.0	fEnglish	name of a kind of animal
U+4E84.0	fUTF8	亄
U+4E84.0	fCNS11643	2-4157
U+4E84.0	fBig5	DCB2
U+4E84.0	fGB	5-3041
U+4E84.0	fR/S	 5.12
U+4E84.0	fTotalStrokes	 13
U+4E84.0	fCangjie	GTU
U+4E84.0	fFourCorner	42110
U+4E84.0	fCantonese	ji3; ji4
U+4E84.0	fMandarin	yi4
U+4E84.0	fEnglish	greedy; stingy
U+4E85.0	fUTF8	亅
U+4E85.0	fCNS11643	3-2124
U+4E85.0	fGB	8-2F7B
U+4E85.0	fR/S	 6.0
U+4E85.0	fTotalStrokes	 1
U+4E85.0	fCangjie	LI
U+4E85.0	fFourCorner	20000
U+4E85.0	fHakka	  kiet7; giet7; get7
U+4E85.0	fCantonese	kyut3
U+4E85.0	fMandarin	jue2
U+4E85.0	fEnglish	[1] perpendicular hook; one of the strokes in Chinese writing [2] Kangxi radical 6
U+4E86.0	fUTF8	了
U+4E86.0	fCNS11643	1-4427
U+4E86.0	fBig5	A446
U+4E86.0	fGB	0-414B
U+4E86.0	fR/S	 6.1
U+4E86.0	fTotalStrokes	 2
U+4E86.0	fCangjie	NN
U+4E86.0	fFourCorner	17207
U+4E86.0	fHakka	  liau3; le3
U+4E86.0	fCantonese	liu5; laa1; liu4
U+4E86.0	fMandarin	liao3
U+4E86.0	fEnglish	an expletive in Mandarin language
U+4E86.1	fUTF8	了
U+4E86.1	fCNS11643	1-4427
U+4E86.1	fBig5	A446
U+4E86.1	fGB	0-414B
U+4E86.1	fR/S	 6.1
U+4E86.1	fTotalStrokes	 2
U+4E86.1	fCangjie	NN
U+4E86.1	fFourCorner	17207
U+4E86.1	fMandarin	le5
U+4E86.1	fEnglish	[1] finish; end; settle; complete; particle of completed action [2] intelligent; remarkable [3] entirely; wholly [4] understand [5] put in; take away
U+4E86.2	fUTF8	了
U+4E86.2	fCNS11643	1-4427
U+4E86.2	fBig5	A446
U+4E86.2	fGB	0-414B
U+4E86.2	fR/S	 6.1
U+4E86.2	fTotalStrokes	 2
U+4E86.2	fCangjie	NN
U+4E86.2	fFourCorner	17207
U+4E86.2	fMandarin	liao4
U+4E87.0	fUTF8	亇
U+4E87.0	fCNS11643	3-212D
U+4E87.0	fGB	1-7D3B
U+4E87.0	fR/S	 6.2; 4.2
U+4E87.0	fTotalStrokes	 3
U+4E87.0	fCangjie	NN
U+4E87.0	fFourCorner	21200
U+4E87.0	fCantonese	ding1
U+4E87.0	fMandarin	ge4
U+4E87.0	fEnglish	 hammer
U+4E88.0	fUTF8	予
U+4E88.0	fCNS11643	1-4468
U+4E88.0	fBig5	A4A9
U+4E88.0	fGB	0-5368
U+4E88.0	fR/S	 6.3
U+4E88.0	fTotalStrokes	 4
U+4E88.0	fCangjie	NINN
U+4E88.0	fFourCorner	17202
U+4E88.0	fHakka	 ji1; ri1; i1; i5; ji2
U+4E88.0	fCantonese	jyu4; jyu5
U+4E88.0	fMandarin	yu2
U+4E88.0	fEnglish	 I; me
U+4E88.1	fUTF8	予
U+4E88.1	fCNS11643	1-4468
U+4E88.1	fBig5	A4A9
U+4E88.1	fGB	0-5368
U+4E88.1	fR/S	 6.3
U+4E88.1	fTotalStrokes	 4
U+4E88.1	fCangjie	NINN
U+4E88.1	fFourCorner	17202
U+4E88.1	fMandarin	yu3
U+4E88.1	fEnglish	 give
U+4E89.0	fUTF8	争
U+4E89.0	fCNS11643	3-2265
U+4E89.0	fGB	0-5579
U+4E89.0	fR/S	 6.5
U+4E89.0	fTotalStrokes	 6
U+4E89.0	fCangjie	NSD
U+4E89.0	fFourCorner	27500
U+4E89.0	fHakka	  zang1; zen1
U+4E89.0	fCantonese	zang1; zaang1 (also); zang3; zaang3 (also); caang1
U+4E89.0	fMandarin	zheng1; zheng4
U+4E89.0	fEnglish	simplified form of &#x722D;
U+4E8A.0	fUTF8	亊
U+4E8A.0	fGB	E-213B
U+4E8A.0	fR/S	 6.6
U+4E8A.0	fTotalStrokes	 7
U+4E8A.0	fCangjie	JFLN
U+4E8A.0	fCantonese	si6
U+4E8A.0	fMandarin	shi4
U+4E8A.0	fEnglish	 affair; matter; business serve accident; incident
U+4E8B.0	fUTF8	事
U+4E8B.0	fCNS11643	1-4B67
U+4E8B.0	fBig5	A8C6
U+4E8B.0	fGB	0-4A42
U+4E8B.0	fR/S	 6.7
U+4E8B.0	fTotalStrokes	 8
U+4E8B.0	fCangjie	JLLN
U+4E8B.0	fFourCorner	50007
U+4E8B.0	fHakka	 su3; sii6; sii5; se5; su5; she5; s'e6; se3 (colloquial)
U+4E8B.0	fCantonese	si6; si2 (also)
U+4E8B.0	fMandarin	shi4; dian4; zhi4
U+4E8B.0	fEnglish	[1] affair; matter; business [2] fact; event; happening [3] job; occupation; task [4] duties; functions [5] trouble; accident; undesirable event [6] matter in general; matters [7] often part of a title [8] service [9] subject [10] attend; do [11] serve [12] manage a business
U+4E8B.1	fUTF8	事
U+4E8B.1	fCNS11643	1-4B67
U+4E8B.1	fBig5	A8C6
U+4E8B.1	fGB	0-4A42
U+4E8B.1	fR/S	 6.7
U+4E8B.1	fTotalStrokes	 8
U+4E8B.1	fCangjie	JLLN
U+4E8B.1	fFourCorner	50007
U+4E8B.1	fMandarin	shi5
U+4E8C.0	fUTF8	二
U+4E8C.0	fCNS11643	1-4428
U+4E8C.0	fBig5	A447
U+4E8C.0	fGB	0-367E
U+4E8C.0	fR/S	 7.0
U+4E8C.0	fTotalStrokes	 2
U+4E8C.0	fCangjie	MM
U+4E8C.0	fFourCorner	10100
U+4E8C.0	fHakka	 ngi5; gni5; ngi6; gni6; ng5
U+4E8C.0	fCantonese	ji6; ji2 (also)
U+4E8C.0	fMandarin	er4
U+4E8C.0	fEnglish	[1] two [2] second; twice
U+4E8D.0	fUTF8	亍
U+4E8D.0	fCNS11643	2-2129
U+4E8D.0	fBig5	C948
U+4E8D.0	fGB	0-5821
U+4E8D.0	fR/S	 7.1
U+4E8D.0	fTotalStrokes	 3
U+4E8D.0	fCangjie	MMN
U+4E8D.0	fFourCorner	10201
U+4E8D.0	fHakka	 cok7; cuk7; ch'ok7; ch'uk7; chok7; chuk7
U+4E8D.0	fCantonese	cuk1
U+4E8D.0	fMandarin	chu4
U+4E8D.0	fEnglish	[1] walk with small steps [2] a Chinese family name [3] a Korean place name
U+4E8E.0	fUTF8	于
U+4E8E.0	fCNS11643	1-4440
U+4E8E.0	fBig5	A45F
U+4E8E.0	fGB	0-535A
U+4E8E.0	fR/S	 7.1
U+4E8E.0	fTotalStrokes	 3
U+4E8E.0	fCangjie	MD
U+4E8E.0	fFourCorner	10400
U+4E8E.0	fHakka	 ji1; wu1; ri1; ji3; i1
U+4E8E.0	fCantonese	jyu1
U+4E8E.0	fMandarin	yu2
U+4E8E.0	fEnglish	[1] in; at; by; to [2] go or proceed; take [3] a Chinese family name
U+4E8E.1	fUTF8	于
U+4E8E.1	fCNS11643	1-4440
U+4E8E.1	fBig5	A45F
U+4E8E.1	fGB	0-535A
U+4E8E.1	fR/S	 7.1
U+4E8E.1	fTotalStrokes	 3
U+4E8E.1	fCangjie	MD
U+4E8E.1	fFourCorner	10400
U+4E8E.1	fMandarin	xu1
U+4E8E.2	fUTF8	于
U+4E8E.2	fCNS11643	1-4440
U+4E8E.2	fBig5	A45F
U+4E8E.2	fGB	0-535A
U+4E8E.2	fR/S	 7.1
U+4E8E.2	fTotalStrokes	 3
U+4E8E.2	fCangjie	MD
U+4E8E.2	fFourCorner	10400
U+4E8E.2	fCantonese	jyu4
U+4E8E.2	fMandarin	yu3
U+4E8F.0	fUTF8	亏
U+4E8F.0	fCNS11643	3-2131
U+4E8F.0	fGB	0-3F77
U+4E8F.0	fR/S	 7.1
U+4E8F.0	fTotalStrokes	 3
U+4E8F.0	fCangjie	MMVS
U+4E8F.0	fFourCorner	10200
U+4E8F.0	fHakka	 wu1; ri1; ji1; k'wui1
U+4E8F.0	fCantonese	kwai1
U+4E8F.0	fMandarin	yu2; kui1
U+4E8F.0	fEnglish	simplified form of &#x8667;
U+4E90.0	fUTF8	亐
U+4E90.0	fGB	E-213C
U+4E90.0	fR/S	 1.2; 7.1
U+4E90.0	fTotalStrokes	 3
U+4E90.0	fCangjie	XXMKS
U+4E90.0	fCantonese	jyu1
U+4E90.0	fMandarin	yu2
U+4E91.0	fUTF8	云
U+4E91.0	fCNS11643	1-4469
U+4E91.0	fBig5	A4AA
U+4E91.0	fGB	0-5446
U+4E91.0	fR/S	 7.2
U+4E91.0	fTotalStrokes	 4
U+4E91.0	fCangjie	MMI
U+4E91.0	fFourCorner	10731
U+4E91.0	fHakka	  jun2; jun3; run2
U+4E91.0	fCantonese	wan4; wan2 (also)
U+4E91.0	fMandarin	yun2
U+4E91.0	fEnglish	[1] say; speak [2] mean [3] simplified form of &#x96F2;
U+4E92.0	fUTF8	互
U+4E92.0	fCNS11643	1-446B
U+4E92.0	fBig5	A4AC
U+4E92.0	fGB	0-3B25
U+4E92.0	fR/S	 7.2
U+4E92.0	fTotalStrokes	 4
U+4E92.0	fCangjie	MVNM
U+4E92.0	fFourCorner	10107
U+4E92.0	fHakka	  fu1;  fu5
U+4E92.0	fCantonese	wu6
U+4E92.0	fMandarin	hu4
U+4E92.0	fEnglish	each other; mutually, reciprocally
U+4E93.0	fUTF8	亓
U+4E93.0	fCNS11643	2-2131
U+4E93.0	fBig5	C951
U+4E93.0	fGB	0-5841
U+4E93.0	fR/S	 7.2
U+4E93.0	fTotalStrokes	 4
U+4E93.0	fCangjie	MML
U+4E93.0	fFourCorner	10221
U+4E93.0	fHakka	 ki2
U+4E93.0	fCantonese	kei4; gei1; gei3
U+4E93.0	fMandarin	qi2
U+4E93.0	fEnglish	a Chinese family name
U+4E94.0	fUTF8	五
U+4E94.0	fCNS11643	1-446C
U+4E94.0	fBig5	A4AD
U+4E94.0	fGB	0-4E65
U+4E94.0	fR/S	 7.2
U+4E94.0	fTotalStrokes	 4
U+4E94.0	fCangjie	MDM
U+4E94.0	fFourCorner	10107
U+4E94.0	fHakka	  ng3
U+4E94.0	fCantonese	ng5
U+4E94.0	fMandarin	wu3
U+4E94.0	fEnglish	[1] five; fifth [2] a Chinese family name
U+4E95.0	fUTF8	井
U+4E95.0	fCNS11643	1-446A
U+4E95.0	fBig5	A4AB
U+4E95.0	fGB	0-3E2E
U+4E95.0	fR/S	 7.2
U+4E95.0	fTotalStrokes	 4
U+4E95.0	fCangjie	TT
U+4E95.0	fFourCorner	55000
U+4E95.0	fHakka	  ziang3
U+4E95.0	fCantonese	zeng2; zing2 (also)
U+4E95.0	fMandarin	jing3
U+4E95.0	fEnglish	[1] well [2] an ancient system of farming and land distribution [3] a Chinese family name
U+4E96.0	fUTF8	亖
U+4E96.0	fCNS11643	4-213B
U+4E96.0	fGB	E-213D
U+4E96.0	fR/S	 7.2
U+4E96.0	fTotalStrokes	 4
U+4E96.0	fCangjie	MMMM
U+4E96.0	fFourCorner	10100
U+4E96.0	fHakka	  si5
U+4E96.0	fCantonese	sei3; si3 (also)
U+4E96.0	fMandarin	si4
U+4E96.0	fEnglish	same as &#x56DB;
U+4E97.0	fUTF8	亗
U+4E97.0	fCNS11643	3-216C
U+4E97.0	fGB	E-213E
U+4E97.0	fR/S	 7.3
U+4E97.0	fTotalStrokes	 5
U+4E97.0	fCangjie	UMM
U+4E97.0	fFourCorner	22100
U+4E97.0	fHakka	 soi5;  sui5;  se5
U+4E97.0	fCantonese	seoi3
U+4E97.0	fMandarin	sui4
U+4E97.0	fEnglish	 harvest year age
U+4E98.0	fUTF8	亘
U+4E98.0	fCNS11643	3-2266
U+4E98.0	fGB	0-5828
U+4E98.0	fR/S	 7.4
U+4E98.0	fTotalStrokes	 6
U+4E98.0	fCangjie	MAM
U+4E98.0	fFourCorner	10100
U+4E98.0	fHakka	  sien1; sien2; sen1; gen3
U+4E98.0	fCantonese	gang2; syun1; wu6
U+4E98.0	fMandarin	xuan1; gen4; geng4; hu4
U+4E98.0	fEnglish	eddy; whirlpool
U+4E98.1	fUTF8	亘
U+4E98.1	fCNS11643	3-2266
U+4E98.1	fGB	0-5828
U+4E98.1	fR/S	 7.4
U+4E98.1	fTotalStrokes	 6
U+4E98.1	fCangjie	MAM
U+4E98.1	fFourCorner	10100
U+4E98.2	fUTF8	亘
U+4E98.2	fCNS11643	3-2266
U+4E98.2	fGB	0-5828
U+4E98.2	fR/S	 7.4
U+4E98.2	fTotalStrokes	 6
U+4E98.2	fCangjie	MAM
U+4E98.2	fFourCorner	10100
U+4E99.0	fUTF8	亙
U+4E99.0	fCNS11643	1-4727
U+4E99.0	fBig5	A5E5
U+4E99.0	fGB	E-213F
U+4E99.0	fR/S	 7.4
U+4E99.0	fTotalStrokes	 6
U+4E99.0	fCangjie	MBM
U+4E99.0	fFourCorner	10106
U+4E99.0	fHakka	 kwen3; gen3; gwen3
U+4E99.0	fCantonese	gang2
U+4E99.0	fMandarin	geng4; gen4
U+4E99.0	fEnglish	[1] extend (over space or time); from one end to another [2] a Chinese family name
U+4E9A.0	fUTF8	亚
U+4E9A.0	fGB	0-5147
U+4E9A.0	fR/S	 1.5; 7.4
U+4E9A.0	fTotalStrokes	 6
U+4E9A.0	fCangjie	MTC
U+4E9A.0	fHakka	 a1; a3; a5
U+4E9A.0	fCantonese	aa3; ngaa3 (also); aa1; ngaa1 (also)
U+4E9A.0	fMandarin	ya4; ya3; ya1
U+4E9A.0	fEnglish	simplified form of &#x4E9E;
U+4E9B.0	fUTF8	些
U+4E9B.0	fCNS11643	1-4B68
U+4E9B.0	fBig5	A8C7
U+4E9B.0	fGB	0-5029
U+4E9B.0	fR/S	 7.5
U+4E9B.0	fTotalStrokes	 7
U+4E9B.0	fCangjie	YPMM
U+4E9B.0	fFourCorner	21101
U+4E9B.0	fHakka	  sia1; siam1
U+4E9B.0	fCantonese	se1; so3; saa1 (also); sai3
U+4E9B.0	fMandarin	xie1; suo1; xi1; sa1; xie5
U+4E9B.0	fEnglish	[1] a small quantity or number; little; few; some [2] used after &#x9019;
U+4E9B.1	fUTF8	些
U+4E9B.1	fCNS11643	1-4B68
U+4E9B.1	fBig5	A8C7
U+4E9B.1	fGB	0-5029
U+4E9B.1	fR/S	 7.5
U+4E9B.1	fTotalStrokes	 7
U+4E9B.1	fCangjie	YPMM
U+4E9B.1	fFourCorner	21101
U+4E9B.1	fCantonese	so1
U+4E9B.1	fMandarin	suo4
U+4E9B.1	fEnglish	 particle expressing regret (only in
U+4E9C.0	fUTF8	亜
U+4E9C.0	fCNS11643	3-2442
U+4E9C.0	fGB	E-2140
U+4E9C.0	fR/S	 7.5
U+4E9C.0	fTotalStrokes	 7
U+4E9C.0	fCangjie	MLLM
U+4E9C.0	fFourCorner	10100
U+4E9C.0	fCantonese	aa3; ngaa3 (also)
U+4E9C.0	fMandarin	ya4; ya3
U+4E9C.0	fEnglish	 Asia; Asian second
U+4E9D.0	fUTF8	亝
U+4E9D.0	fCNS11643	3-2742
U+4E9D.0	fGB	E-2141
U+4E9D.0	fR/S	 7.6
U+4E9D.0	fTotalStrokes	 8
U+4E9D.0	fCangjie	IIIM
U+4E9D.0	fFourCorner	23100
U+4E9D.0	fHakka	 zai1
U+4E9D.0	fCantonese	cai4; zaai1
U+4E9D.0	fMandarin	qi2; zhai1; zi1
U+4E9D.0	fEnglish	 even; regular; uniform all alike arrange
U+4E9E.0	fUTF8	亞
U+4E9E.0	fCNS11643	1-4B69
U+4E9E.0	fBig5	A8C8
U+4E9E.0	fGB	1-5147
U+4E9E.0	fR/S	 7.6
U+4E9E.0	fTotalStrokes	 8
U+4E9E.0	fCangjie	MLLM
U+4E9E.0	fFourCorner	10107
U+4E9E.0	fHakka	  a1; a3; a6; a5
U+4E9E.0	fCantonese	aa3; ngaa3 (also); aa1; ngaa1 (also)
U+4E9E.0	fMandarin	ya4; ya3; ya1
U+4E9E.0	fEnglish	[1] second (in excellence) [2] Asia
U+4E9F.0	fUTF8	亟
U+4E9F.0	fCNS11643	1-4F67
U+4E9F.0	fBig5	AB45
U+4E9F.0	fGB	0-583D
U+4E9F.0	fR/S	 7.6
U+4E9F.0	fTotalStrokes	 8
U+4E9F.0	fCangjie	MEM
U+4E9F.0	fFourCorner	10104
U+4E9F.0	fHakka	 git7;  kit8
U+4E9F.0	fCantonese	gik1
U+4E9F.0	fMandarin	ji2; qi4
U+4E9F.0	fEnglish	urgently; pressingly; anxiously
U+4E9F.1	fUTF8	亟
U+4E9F.1	fCNS11643	1-4F67
U+4E9F.1	fBig5	AB45
U+4E9F.1	fGB	0-583D
U+4E9F.1	fR/S	 7.6
U+4E9F.1	fTotalStrokes	 8
U+4E9F.1	fCangjie	MEM
U+4E9F.1	fFourCorner	10104
U+4E9F.1	fCantonese	kei3
U+4E9F.1	fEnglish	repeatedly; again and again
U+4EA0.0	fUTF8	亠
U+4EA0.0	fCNS11643	1-2728
U+4EA0.0	fGB	0-596F
U+4EA0.0	fR/S	 8.0
U+4EA0.0	fTotalStrokes	 2
U+4EA0.0	fCangjie	XIM
U+4EA0.0	fFourCorner	0000
U+4EA0.0	fHakka	 teu2; teu3; tiu2
U+4EA0.0	fCantonese	tau4
U+4EA0.0	fMandarin	tou2
U+4EA0.0	fEnglish	Kangxi radical 8
U+4EA1.0	fUTF8	亡
U+4EA1.0	fCNS11643	1-4441
U+4EA1.0	fBig5	A460
U+4EA1.0	fGB	0-4D76
U+4EA1.0	fR/S	 8.1
U+4EA1.0	fTotalStrokes	 3
U+4EA1.0	fCangjie	YV
U+4EA1.0	fFourCorner	00710
U+4EA1.0	fHakka	  mong2; mong3
U+4EA1.0	fCantonese	mong4; mong1
U+4EA1.0	fMandarin	wang2
U+4EA1.0	fEnglish	[1] perish; loose [2] die [3] flee; escape [4] lost; death [5] the late (applicable only to deceased blood relatives or friends)
U+4EA1.1	fUTF8	亡
U+4EA1.1	fCNS11643	1-4441
U+4EA1.1	fBig5	A460
U+4EA1.1	fGB	0-4D76
U+4EA1.1	fR/S	 8.1
U+4EA1.1	fTotalStrokes	 3
U+4EA1.1	fCangjie	YV
U+4EA1.1	fFourCorner	00710
U+4EA1.1	fCantonese	mou4
U+4EA1.1	fMandarin	wu2
U+4EA1.1	fEnglish	no; not a
U+4EA2.0	fUTF8	亢
U+4EA2.0	fCNS11643	1-446D
U+4EA2.0	fBig5	A4AE
U+4EA2.0	fGB	0-3F3A
U+4EA2.0	fR/S	 8.2
U+4EA2.0	fTotalStrokes	 4
U+4EA2.0	fCangjie	YHN
U+4EA2.0	fFourCorner	00217
U+4EA2.0	fHakka	  kong5;  gong1; kong1; hong2
U+4EA2.0	fCantonese	kong3
U+4EA2.0	fMandarin	kang4; geng1; hang2
U+4EA2.0	fEnglish	[1] proud [2] indomitable [3] excessive; extreme [4] the name of a constellation
U+4EA2.1	fUTF8	亢
U+4EA2.1	fCNS11643	1-446D
U+4EA2.1	fBig5	A4AE
U+4EA2.1	fGB	0-3F3A
U+4EA2.1	fR/S	 8.2
U+4EA2.1	fTotalStrokes	 4
U+4EA2.1	fCangjie	YHN
U+4EA2.1	fFourCorner	00217
U+4EA2.1	fCantonese	gong1
U+4EA2.1	fMandarin	gang1
U+4EA2.1	fEnglish	 a person's neck
U+4EA3.0	fUTF8	亣
U+4EA3.0	fCNS11643	3-2147
U+4EA3.0	fGB	E-2142
U+4EA3.0	fR/S	 8.2; 99.2
U+4EA3.0	fTotalStrokes	 4
U+4EA3.0	fCangjie	YLL
U+4EA3.0	fFourCorner	00220
U+4EA3.0	fHakka	 tai5
U+4EA3.0	fCantonese	daai6; tai3
U+4EA3.0	fMandarin	ta4; da4; duo4; tai3; tai4
U+4EA4.0	fUTF8	交
U+4EA4.0	fCNS11643	1-4728
U+4EA4.0	fBig5	A5E6
U+4EA4.0	fGB	0-3D3B
U+4EA4.0	fR/S	 8.4
U+4EA4.0	fTotalStrokes	 6
U+4EA4.0	fCangjie	YCK
U+4EA4.0	fFourCorner	00408
U+4EA4.0	fHakka	  gau1; gau2; gau5; kau1
U+4EA4.0	fCantonese	gaau1
U+4EA4.0	fMandarin	jiao1
U+4EA4.0	fEnglish	[1] submit; hand in or over; transfer [2] be on good terms with; come into contact with; meet [3] exchange [4] intersect; cross [5] friend; friendship [6] come close to; join; alternate; engage in [7] unite in sexual intercourse [8] simultaneous(ly) [9] each other
U+4EA5.0	fUTF8	亥
U+4EA5.0	fCNS11643	1-472A
U+4EA5.0	fBig5	A5E8
U+4EA5.0	fGB	0-3A25
U+4EA5.0	fR/S	 8.4
U+4EA5.0	fTotalStrokes	 6
U+4EA5.0	fCangjie	YVHO
U+4EA5.0	fFourCorner	00802
U+4EA5.0	fHakka	 hoi6; hoi3;  hoi5
U+4EA5.0	fCantonese	hoi6
U+4EA5.0	fMandarin	hai4
U+4EA5.0	fEnglish	[1] the last of 12 terrestrial branches [2] the hours between 9 and 11 p.m. [3] a Chinese family name
U+4EA6.0	fUTF8	亦
U+4EA6.0	fCNS11643	1-4729
U+4EA6.0	fBig5	A5E7
U+4EA6.0	fGB	0-5260
U+4EA6.0	fR/S	 8.4
U+4EA6.0	fTotalStrokes	 6
U+4EA6.0	fCangjie	YLNC
U+4EA6.0	fFourCorner	00330
U+4EA6.0	fHakka	  jit8; rit8
U+4EA6.0	fCantonese	jik6
U+4EA6.0	fMandarin	yi4
U+4EA6.0	fEnglish	also, too; likewise
U+4EA7.0	fUTF8	产
U+4EA7.0	fGB	0-327A
U+4EA7.0	fR/S	 117.1; 8.4
U+4EA7.0	fTotalStrokes	 6
U+4EA7.0	fCangjie	YTH
U+4EA7.0	fCantonese	caan2
U+4EA7.0	fMandarin	chan3
U+4EA7.0	fEnglish	simplified form of &#x7523;
U+4EA8.0	fUTF8	亨
U+4EA8.0	fCNS11643	1-486C
U+4EA8.0	fBig5	A6EB
U+4EA8.0	fGB	0-3A60
U+4EA8.0	fR/S	 8.5
U+4EA8.0	fTotalStrokes	 7
U+4EA8.0	fCangjie	YRNN
U+4EA8.0	fFourCorner	00207
U+4EA8.0	fHakka	 hen1; kien1; hang1; ken1
U+4EA8.0	fCantonese	hang1; hoeng2
U+4EA8.0	fMandarin	heng1; xiang3
U+4EA8.0	fEnglish	 go through smoothly; prosperous
U+4EA8.1	fUTF8	亨
U+4EA8.1	fCNS11643	1-486C
U+4EA8.1	fBig5	A6EB
U+4EA8.1	fGB	0-3A60
U+4EA8.1	fR/S	 8.5
U+4EA8.1	fTotalStrokes	 7
U+4EA8.1	fCangjie	YRNN
U+4EA8.1	fFourCorner	00207
U+4EA8.1	fCantonese	paang1
U+4EA8.1	fMandarin	peng1
U+4EA9.0	fUTF8	亩
U+4EA9.0	fGB	0-4436
U+4EA9.0	fR/S	 8.5
U+4EA9.0	fTotalStrokes	 7
U+4EA9.0	fCangjie	YW
U+4EA9.0	fHakka	 meu1; miau1
U+4EA9.0	fCantonese	mau5
U+4EA9.0	fMandarin	mou3; mu3
U+4EA9.0	fEnglish	simplified form of &#x755D;
U+4EAA.0	fUTF8	亪
U+4EAA.0	fGB	1-7D7D
U+4EAA.0	fR/S	 8.5
U+4EAA.0	fTotalStrokes	 7
U+4EAA.0	fCangjie	YCMO
U+4EAA.0	fCantonese	bin3
U+4EAA.0	fEnglish	 kwukyel
U+4EAB.0	fUTF8	享
U+4EAB.0	fCNS11643	1-4B6A
U+4EAB.0	fBig5	A8C9
U+4EAB.0	fGB	0-4F6D
U+4EAB.0	fR/S	 8.6
U+4EAB.0	fTotalStrokes	 8
U+4EAB.0	fCangjie	YRND
U+4EAB.0	fFourCorner	00407
U+4EAB.0	fHakka	  hiong3; ken1
U+4EAB.0	fCantonese	hoeng2
U+4EAB.0	fMandarin	xiang3; peng1; heng1
U+4EAB.0	fEnglish	[1] enjoy; receive (peace, happiness, long life, good health) [2] offer; enjoy or taste of the sacrifices (gods, spirits) [3] entertain
U+4EAC.0	fUTF8	京
U+4EAC.0	fCNS11643	1-4B6B
U+4EAC.0	fBig5	A8CA
U+4EAC.0	fGB	0-3E29
U+4EAC.0	fR/S	 8.6
U+4EAC.0	fTotalStrokes	 8
U+4EAC.0	fCangjie	YRF
U+4EAC.0	fFourCorner	00906
U+4EAC.0	fHakka	  gin1; liong2
U+4EAC.0	fCantonese	ging1; jyun4
U+4EAC.0	fMandarin	jing1; jing3; qiang2; yuan2
U+4EAC.0	fEnglish	[1] capital city; metropolis [2] ten million [3] great; greatness [4] a Chinese family name
U+4EAD.0	fUTF8	亭
U+4EAD.0	fCNS11643	1-4F68
U+4EAD.0	fBig5	AB46
U+4EAD.0	fGB	0-4D24
U+4EAD.0	fR/S	 8.7
U+4EAD.0	fTotalStrokes	 9
U+4EAD.0	fCangjie	YRBN
U+4EAD.0	fFourCorner	00201
U+4EAD.0	fHakka	  tin2; tin3
U+4EAD.0	fCantonese	ting4; ting2 (also)
U+4EAD.0	fMandarin	ting2
U+4EAD.0	fEnglish	[1] booth; pavilion; garden house or rest house [2] slim and erect (ladies' figure) [3] exactly during
U+4EAE.0	fUTF8	亮
U+4EAE.0	fCNS11643	1-4F69
U+4EAE.0	fBig5	AB47
U+4EAE.0	fGB	0-4141
U+4EAE.0	fR/S	 8.7
U+4EAE.0	fTotalStrokes	 9
U+4EAE.0	fCangjie	YRBU
U+4EAE.0	fFourCorner	00217
U+4EAE.0	fHakka	 liong5; liong6
U+4EAE.0	fCantonese	loeng6
U+4EAE.0	fMandarin	liang4; liang2
U+4EAE.0	fEnglish	[1] bright; lustrous; brilliant; luminous; radiant; clear [2] transparent; not opaque [3] display; show; give a quick show just for a glance
U+4EAE.1	fUTF8	亮
U+4EAE.1	fCNS11643	1-4F69
U+4EAE.1	fBig5	AB47
U+4EAE.1	fGB	0-4141
U+4EAE.1	fR/S	 8.7
U+4EAE.1	fTotalStrokes	 9
U+4EAE.1	fCangjie	YRBU
U+4EAE.1	fFourCorner	00217
U+4EAE.1	fMandarin	liang5
U+4EAF.0	fUTF8	亯
U+4EAF.0	fCNS11643	3-2B26
U+4EAF.0	fGB	E-2143
U+4EAF.0	fR/S	 8.7
U+4EAF.0	fTotalStrokes	 9
U+4EAF.0	fCangjie	YRA
U+4EAF.0	fFourCorner	00600
U+4EAF.0	fCantonese	hang1; paang1; hoeng2
U+4EAF.0	fMandarin	xiang3; peng1
U+4EAF.0	fEnglish	 receive enjoy
U+4EB0.0	fUTF8	亰
U+4EB0.0	fCNS11643	3-2B25
U+4EB0.0	fGB	E-2144
U+4EB0.0	fR/S	 8.7
U+4EB0.0	fTotalStrokes	 9
U+4EB0.0	fCangjie	YAF
U+4EB0.0	fFourCorner	00900
U+4EB0.0	fCantonese	ging1
U+4EB0.0	fMandarin	jing1
U+4EB0.0	fEnglish	capital city
U+4EB1.0	fUTF8	亱
U+4EB1.0	fCNS11643	3-2B27
U+4EB1.0	fGB	E-2145
U+4EB1.0	fR/S	 8.7
U+4EB1.0	fTotalStrokes	 9
U+4EB1.0	fCangjie	YOAM
U+4EB1.0	fFourCorner	00210
U+4EB1.0	fHakka	 ja5; ra6; ja3
U+4EB1.0	fCantonese	je6
U+4EB1.0	fMandarin	ye4
U+4EB1.0	fEnglish	 night; dark in night by night
U+4EB2.0	fUTF8	亲
U+4EB2.0	fCNS11643	3-2B24
U+4EB2.0	fGB	0-4757
U+4EB2.0	fR/S	 117.4; 8.7
U+4EB2.0	fTotalStrokes	 9
U+4EB2.0	fCangjie	YTD
U+4EB2.0	fFourCorner	00900
U+4EB2.0	fHakka	 cin1
U+4EB2.0	fCantonese	can1; can3 (also)
U+4EB2.0	fMandarin	qin1; qin4; xin1; qing4
U+4EB2.0	fEnglish	simplified form of &#x89AA;
U+4EB2.1	fUTF8	亲
U+4EB2.1	fCNS11643	3-2B24
U+4EB2.1	fGB	0-4757
U+4EB2.1	fR/S	 117.4; 8.7
U+4EB2.1	fTotalStrokes	 9
U+4EB2.1	fCangjie	YTD
U+4EB2.1	fFourCorner	00900
U+4EB2.1	fMandarin	qin5
U+4EB3.0	fUTF8	亳
U+4EB3.0	fCNS11643	1-537E
U+4EB3.0	fBig5	ADBD
U+4EB3.0	fGB	0-5971
U+4EB3.0	fR/S	 8.8
U+4EB3.0	fTotalStrokes	 10
U+4EB3.0	fCangjie	YRBP
U+4EB3.0	fFourCorner	00714
U+4EB3.0	fHakka	  bok7;  pok8
U+4EB3.0	fCantonese	bok3; bok6
U+4EB3.0	fMandarin	bo2; bo4
U+4EB3.0	fEnglish	the seat of government during the Shang Dynasty, located in today's Shangchiu County, Honan Province
U+4EB4.0	fUTF8	亴
U+4EB4.0	fCNS11643	4-362E
U+4EB4.0	fGB	E-2146
U+4EB4.0	fR/S	 8.10; 99.10
U+4EB4.0	fTotalStrokes	 12
U+4EB4.0	fCangjie	YRBN
U+4EB4.0	fFourCorner	00410
U+4EB4.0	fCantonese	jau6
U+4EB4.0	fMandarin	you4
U+4EB5.0	fUTF8	亵
U+4EB5.0	fGB	0-5974
U+4EB5.0	fR/S	 8.10
U+4EB5.0	fTotalStrokes	 12
U+4EB5.0	fCangjie	YQIV
U+4EB5.0	fHakka	 siet7
U+4EB5.0	fCantonese	sit3
U+4EB5.0	fMandarin	xie4
U+4EB5.0	fEnglish	simplified form of &#x893B;
U+4EB6.0	fUTF8	亶
U+4EB6.0	fCNS11643	2-4158
U+4EB6.0	fBig5	DCB3
U+4EB6.0	fGB	3-3257
U+4EB6.0	fR/S	 8.11
U+4EB6.0	fTotalStrokes	 13
U+4EB6.0	fCangjie	YWRM
U+4EB6.0	fFourCorner	00106
U+4EB6.0	fHakka	  tan3
U+4EB6.0	fCantonese	taan2; daan6; sin4; sin6
U+4EB6.0	fMandarin	dan3; dan4; chan4; shan4; zhan1
U+4EB6.0	fEnglish	[1] [..] [2] sincere; honest; real [3] faith; loyalty [4] greatness; vastness; flatness [5] fly [6] a Chinese family name
U+4EB6.1	fUTF8	亶
U+4EB6.1	fCNS11643	2-4158
U+4EB6.1	fBig5	DCB3
U+4EB6.1	fGB	3-3257
U+4EB6.1	fR/S	 8.11
U+4EB6.1	fTotalStrokes	 13
U+4EB6.1	fCangjie	YWRM
U+4EB6.1	fFourCorner	00106
U+4EB6.2	fUTF8	亶
U+4EB6.2	fCNS11643	2-4158
U+4EB6.2	fBig5	DCB3
U+4EB6.2	fGB	3-3257
U+4EB6.2	fR/S	 8.11
U+4EB6.2	fTotalStrokes	 13
U+4EB6.2	fCangjie	YWRM
U+4EB6.2	fFourCorner	00106
U+4EB6.2	fMandarin	chan2
U+4EB6.2	fEnglish	used for a mountain name in a legend
U+4EB6.3	fUTF8	亶
U+4EB6.3	fCNS11643	2-4158
U+4EB6.3	fBig5	DCB3
U+4EB6.3	fGB	3-3257
U+4EB6.3	fR/S	 8.11
U+4EB6.3	fTotalStrokes	 13
U+4EB6.3	fCangjie	YWRM
U+4EB6.3	fFourCorner	00106
U+4EB7.0	fUTF8	亷
U+4EB7.0	fCNS11643	3-4033
U+4EB7.0	fGB	E-2147
U+4EB7.0	fR/S	 8.11
U+4EB7.0	fTotalStrokes	 13
U+4EB7.0	fCangjie	YHXC
U+4EB7.0	fFourCorner	00230
U+4EB7.0	fCantonese	lim4
U+4EB7.0	fMandarin	lian2
U+4EB7.0	fEnglish	upright; honorable; honest
U+4EB8.0	fUTF8	亸
U+4EB8.0	fGB	8-2C22
U+4EB8.0	fR/S	 8.14; 30.13
U+4EB8.0	fTotalStrokes	 16
U+4EB8.0	fCangjie	YDCWJ
U+4EB8.0	fHakka	 do3; lo3
U+4EB8.0	fCantonese	do2
U+4EB8.0	fMandarin	duo3
U+4EB8.0	fEnglish	read aloud, recite, chant; droop
U+4EB9.0	fUTF8	亹
U+4EB9.0	fCNS11643	2-6D44
U+4EB9.0	fBig5	F6D6
U+4EB9.0	fGB	3-325A
U+4EB9.0	fR/S	 8.19
U+4EB9.0	fTotalStrokes	 21
U+4EB9.0	fCangjie	YHBM
U+4EB9.0	fFourCorner	00107
U+4EB9.0	fHakka	 mui3; mi3; mui1; tau3
U+4EB9.0	fCantonese	mei5
U+4EB9.0	fMandarin	wei3; wei4
U+4EB9.0	fEnglish	 busy, untiring, progressing
U+4EB9.1	fUTF8	亹
U+4EB9.1	fCNS11643	2-6D44
U+4EB9.1	fBig5	F6D6
U+4EB9.1	fGB	3-325A
U+4EB9.1	fR/S	 8.19
U+4EB9.1	fTotalStrokes	 21
U+4EB9.1	fCangjie	YHBM
U+4EB9.1	fFourCorner	00107
U+4EB9.1	fCantonese	mun4
U+4EB9.1	fMandarin	men2
U+4EBA.0	fUTF8	人
U+4EBA.0	fCNS11643	1-4429
U+4EBA.0	fBig5	A448
U+4EBA.0	fGB	0-484B
U+4EBA.0	fR/S	 9.0
U+4EBA.0	fTotalStrokes	 2
U+4EBA.0	fCangjie	O
U+4EBA.0	fFourCorner	80000
U+4EBA.0	fHakka	  ngin2; gnin2; gnin3
U+4EBA.0	fCantonese	jan4; jan1 (also); jan2 (also)
U+4EBA.0	fMandarin	ren2
U+4EBA.0	fEnglish	[1] human being; person [2] people; mankind [3] everybody [4] grown-up [5] a person of certain character
U+4EBA.1	fUTF8	人
U+4EBA.1	fCNS11643	1-4429
U+4EBA.1	fBig5	A448
U+4EBA.1	fGB	0-484B
U+4EBA.1	fR/S	 9.0
U+4EBA.1	fTotalStrokes	 2
U+4EBA.1	fCangjie	O
U+4EBA.1	fFourCorner	80000
U+4EBA.1	fMandarin	ren5
U+4EBB.0	fUTF8	亻
U+4EBB.0	fGB	0-5869
U+4EBB.0	fR/S	 9.0
U+4EBB.0	fTotalStrokes	 2
U+4EBB.0	fCangjie	HL
U+4EBB.0	fFourCorner	2020
U+4EBB.0	fHakka	 ngin2
U+4EBB.0	fCantonese	jan4
U+4EBB.0	fMandarin	dan1ren2pang2; ren2
U+4EBB.0	fEnglish	Kangxi radical 9
U+4EBC.0	fUTF8	亼
U+4EBC.0	fCNS11643	3-2132
U+4EBC.0	fGB	E-2148
U+4EBC.0	fR/S	 9.1
U+4EBC.0	fTotalStrokes	 3
U+4EBC.0	fCangjie	OM
U+4EBC.0	fFourCorner	80100
U+4EBC.0	fHakka	  sip8; ch'ip8; cip8
U+4EBC.0	fCantonese	zaap6; zap6 (also)
U+4EBC.0	fMandarin	ji2
U+4EBC.0	fEnglish	 assemble; gather together
U+4EBD.0	fUTF8	亽
U+4EBD.0	fGB	1-7D37
U+4EBD.0	fR/S	 9.1
U+4EBD.0	fTotalStrokes	 3
U+4EBD.0	fCangjie	XXOI
U+4EBD.0	fCantonese	zaap6
U+4EBD.0	fEnglish	 kwukyel
U+4EBE.0	fUTF8	亾
U+4EBE.0	fCNS11643	3-2133
U+4EBE.0	fGB	E-2149
U+4EBE.0	fR/S	 9.1
U+4EBE.0	fTotalStrokes	 3
U+4EBE.0	fCangjie	OV
U+4EBE.0	fFourCorner	28710
U+4EBE.0	fHakka	 mong2
U+4EBE.0	fCantonese	mou4; mong4
U+4EBE.0	fMandarin	wang2; wu2
U+4EBE.0	fEnglish	death, destroyed; lose, perish
U+4EBF.0	fUTF8	亿
U+4EBF.0	fGB	0-525A
U+4EBF.0	fR/S	 9.1
U+4EBF.0	fTotalStrokes	 3
U+4EBF.0	fCangjie	ON
U+4EBF.0	fHakka	 ji5; jit7
U+4EBF.0	fCantonese	jik1
U+4EBF.0	fMandarin	yi4
U+4EBF.0	fEnglish	simplified form of &#x5104;
U+4EC0.0	fUTF8	什
U+4EC0.0	fCNS11643	1-446F
U+4EC0.0	fBig5	A4B0
U+4EC0.0	fGB	0-4A32
U+4EC0.0	fR/S	 9.2
U+4EC0.0	fTotalStrokes	 4
U+4EC0.0	fCangjie	OJ
U+4EC0.0	fFourCorner	24200
U+4EC0.0	fHakka	  ship8; sip8; cap8; siip8
U+4EC0.0	fCantonese	sap6; sam6
U+4EC0.0	fMandarin	shi2
U+4EC0.0	fEnglish	[1] sundry; miscellaneous [2] ten [3] squad [4] file of ten soldiers (in former times)
U+4EC0.1	fUTF8	什
U+4EC0.1	fCNS11643	1-446F
U+4EC0.1	fBig5	A4B0
U+4EC0.1	fGB	0-4A32
U+4EC0.1	fR/S	 9.2
U+4EC0.1	fTotalStrokes	 4
U+4EC0.1	fCangjie	OJ
U+4EC0.1	fFourCorner	24200
U+4EC0.1	fCantonese	zaap6
U+4EC0.1	fMandarin	shen2
U+4EC0.1	fEnglish	[1] what [2] miscellaneous
U+4EC1.0	fUTF8	仁
U+4EC1.0	fCNS11643	1-446E
U+4EC1.0	fBig5	A4AF
U+4EC1.0	fGB	0-484A
U+4EC1.0	fR/S	 9.2
U+4EC1.0	fTotalStrokes	 4
U+4EC1.0	fCangjie	OMM
U+4EC1.0	fFourCorner	21210
U+4EC1.0	fHakka	 jin2; rin2; jin3; ngin2
U+4EC1.0	fCantonese	jan4
U+4EC1.0	fMandarin	ren2
U+4EC1.0	fEnglish	[1] benevolence; humanity; mercy; kindness; charity [2] kernel; core [3] a person of great virtue [4] respectable; esteemed [5] sensitive to external stimuli [6] humane; compassionate; humanitarian (policy, government); merciful (heart, character)
U+4EC2.0	fUTF8	仂
U+4EC2.0	fCNS11643	2-2132
U+4EC2.0	fBig5	C952
U+4EC2.0	fGB	0-586C
U+4EC2.0	fR/S	 9.2
U+4EC2.0	fTotalStrokes	 4
U+4EC2.0	fCangjie	OKS
U+4EC2.0	fFourCorner	24227
U+4EC2.0	fHakka	 lit8;  let8
U+4EC2.0	fCantonese	lak6; lik6
U+4EC2.0	fMandarin	le4; li4
U+4EC2.0	fEnglish	[1] surplus; excess [2] fraction (of number)
U+4EC3.0	fUTF8	仃
U+4EC3.0	fCNS11643	1-4470
U+4EC3.0	fBig5	A4B1
U+4EC3.0	fGB	0-586A
U+4EC3.0	fR/S	 9.2
U+4EC3.0	fTotalStrokes	 4
U+4EC3.0	fCangjie	OMN
U+4EC3.0	fFourCorner	21220
U+4EC3.0	fHakka	  den1;  dang1; din3
U+4EC3.0	fCantonese	ding1
U+4EC3.0	fMandarin	ding1
U+4EC3.0	fEnglish	lonely; solitary; without relatives
U+4EC4.0	fUTF8	仄
U+4EC4.0	fCNS11643	1-4476
U+4EC4.0	fBig5	A4B7
U+4EC4.0	fGB	0-5846
U+4EC4.0	fR/S	 9.2
U+4EC4.0	fTotalStrokes	 4
U+4EC4.0	fCangjie	MO
U+4EC4.0	fFourCorner	71280
U+4EC4.0	fHakka	 zet7
U+4EC4.0	fCantonese	zak1
U+4EC4.0	fMandarin	ze4
U+4EC4.0	fEnglish	[1] oblique; slanting; [2] oblique tones; said of the three tones other than the even tone (in ancient Chinese phonology) [3] narrow [4] uneasy
U+4EC5.0	fUTF8	仅
U+4EC5.0	fCNS11643	3-2149
U+4EC5.0	fGB	0-3D76
U+4EC5.0	fR/S	 9.2
U+4EC5.0	fTotalStrokes	 4
U+4EC5.0	fCangjie	OE
U+4EC5.0	fFourCorner	27240
U+4EC5.0	fHakka	 giun3
U+4EC5.0	fCantonese	gan2; gan6
U+4EC5.0	fMandarin	jin3; jin4
U+4EC5.0	fEnglish	simplified form of &#x50C5;
U+4EC6.0	fUTF8	仆
U+4EC6.0	fCNS11643	1-4471
U+4EC6.0	fBig5	A4B2
U+4EC6.0	fGB	0-464D
U+4EC6.0	fR/S	 9.2
U+4EC6.0	fTotalStrokes	 4
U+4EC6.0	fCangjie	OY
U+4EC6.0	fFourCorner	23200
U+4EC6.0	fHakka	 puk7;  fu5; puk8
U+4EC6.0	fCantonese	fu6; puk1 (also); puk6 (also); buk6
U+4EC6.0	fMandarin	pu1; fu4; pu2; pu3
U+4EC6.0	fEnglish	[1] prostrate [2] fall to the ground; drop dead [3] simplified form of &#x50D5;
U+4EC7.0	fUTF8	仇
U+4EC7.0	fCNS11643	1-4472
U+4EC7.0	fBig5	A4B3
U+4EC7.0	fGB	0-3370
U+4EC7.0	fR/S	 9.2
U+4EC7.0	fTotalStrokes	 4
U+4EC7.0	fCangjie	OKN
U+4EC7.0	fFourCorner	24217
U+4EC7.0	fHakka	 siu2;  shiu2;  chiu2; shu3; su2; ciu2; shu2; kiu2
U+4EC7.0	fCantonese	sau4; cau4 (also)
U+4EC7.0	fMandarin	chou2
U+4EC7.0	fEnglish	[1] foe; enemy; rival; adversary [2] hatred; enmity; antagonism; hostility; feud [3] hate
U+4EC7.1	fUTF8	仇
U+4EC7.1	fCNS11643	1-4472
U+4EC7.1	fBig5	A4B3
U+4EC7.1	fGB	0-3370
U+4EC7.1	fR/S	 9.2
U+4EC7.1	fTotalStrokes	 4
U+4EC7.1	fCangjie	OKN
U+4EC7.1	fFourCorner	24217
U+4EC7.1	fCantonese	kau4
U+4EC7.1	fMandarin	qiu2
U+4EC7.1	fEnglish	[1] spouse [2] mate [3] a Chinese family name
U+4EC8.0	fUTF8	仈
U+4EC8.0	fCNS11643	2-2134
U+4EC8.0	fBig5	C954
U+4EC8.0	fGB	3-305C
U+4EC8.0	fR/S	 9.2
U+4EC8.0	fTotalStrokes	 4
U+4EC8.0	fCangjie	OC
U+4EC8.0	fFourCorner	28200
U+4EC8.0	fCantonese	baat3
U+4EC8.0	fMandarin	ba1
U+4EC9.0	fUTF8	仉
U+4EC9.0	fCNS11643	2-2133
U+4EC9.0	fBig5	C953
U+4EC9.0	fGB	0-586B
U+4EC9.0	fR/S	 9.2
U+4EC9.0	fTotalStrokes	 4
U+4EC9.0	fCangjie	OHN
U+4EC9.0	fFourCorner	27210
U+4EC9.0	fHakka	 zhong3; zong3
U+4EC9.0	fCantonese	zoeng2
U+4EC9.0	fMandarin	zhang3
U+4EC9.0	fEnglish	surname of the mother of Mencius
U+4ECA.0	fUTF8	今
U+4ECA.0	fCNS11643	1-4474
U+4ECA.0	fBig5	A4B5
U+4ECA.0	fGB	0-3D71
U+4ECA.0	fR/S	 9.2
U+4ECA.0	fTotalStrokes	 4
U+4ECA.0	fCangjie	OIN
U+4ECA.0	fFourCorner	80207
U+4ECA.0	fHakka	  gim1; gin1 (coll.)
U+4ECA.0	fCantonese	gam1
U+4ECA.0	fMandarin	jin1
U+4ECA.0	fEnglish	[1] present; recent; modern [2] now; currently; presently; nowadays [3] immediately; right away
U+4ECB.0	fUTF8	介
U+4ECB.0	fCNS11643	1-4475
U+4ECB.0	fBig5	A4B6
U+4ECB.0	fGB	0-3D69
U+4ECB.0	fR/S	 9.2
U+4ECB.0	fTotalStrokes	 4
U+4ECB.0	fCangjie	OLL
U+4ECB.0	fFourCorner	80220
U+4ECB.0	fHakka	  gai5; kie5; kiai5; giai5
U+4ECB.0	fCantonese	gaai3; gaa3
U+4ECB.0	fMandarin	jie4
U+4ECB.0	fEnglish	[1] shelled aquatic animal; sea shell [2] lie between; interpose [3] upright (said of one's character); straightforward; conscientious [4] great and honorable; huge; big [5] aid; benefit [6] tiny [7] armor; mail [8] interchangeable with &#x82A5; [9] stage direction for certain movements [10] used for &#x4EF7; [11] pray for blessing [12] keep in mind; take seriously [13] simple; plain [14] alone [15] a Chinese family name
U+4ECB.1	fUTF8	介
U+4ECB.1	fCNS11643	1-4475
U+4ECB.1	fBig5	A4B6
U+4ECB.1	fGB	0-3D69
U+4ECB.1	fR/S	 9.2
U+4ECB.1	fTotalStrokes	 4
U+4ECB.1	fCangjie	OLL
U+4ECB.1	fFourCorner	80220
U+4ECB.1	fMandarin	ga4
U+4ECB.1	fEnglish	 such a
U+4ECC.0	fUTF8	仌
U+4ECC.0	fCNS11643	3-214C
U+4ECC.0	fGB	5-313C
U+4ECC.0	fR/S	  9.2
U+4ECC.0	fTotalStrokes	 4
U+4ECC.0	fCangjie	OO
U+4ECC.0	fFourCorner	80800
U+4ECC.0	fCantonese	bing1
U+4ECC.0	fMandarin	bing1
U+4ECD.0	fUTF8	仍
U+4ECD.0	fCNS11643	1-4473
U+4ECD.0	fBig5	A4B4
U+4ECD.0	fGB	0-4854
U+4ECD.0	fR/S	 9.2
U+4ECD.0	fTotalStrokes	 4
U+4ECD.0	fCangjie	ONHS
U+4ECD.0	fFourCorner	27227
U+4ECD.0	fHakka	  jin2; rin2; jin3
U+4ECD.0	fCantonese	jing4
U+4ECD.0	fMandarin	reng2
U+4ECD.0	fEnglish	[1] yet; still; as ever [2] keeping; continuing; again and again; over and over
U+4ECE.0	fUTF8	从
U+4ECE.0	fCNS11643	3-214B
U+4ECE.0	fGB	0-3453
U+4ECE.0	fR/S	 9.2
U+4ECE.0	fTotalStrokes	 4
U+4ECE.0	fCangjie	OO
U+4ECE.0	fFourCorner	88000
U+4ECE.0	fHakka	 ciung2; ch'iung2; siung6; ziung6; ziung3; siung5
U+4ECE.0	fCantonese	cung4
U+4ECE.0	fMandarin	cong2; chuang2; zong4; zong1
U+4ECE.0	fEnglish	simplified form of &#x5F9E;
U+4ECE.1	fUTF8	从
U+4ECE.1	fCNS11643	3-214B
U+4ECE.1	fGB	0-3453
U+4ECE.1	fR/S	 9.2
U+4ECE.1	fTotalStrokes	 4
U+4ECE.1	fCangjie	OO
U+4ECE.1	fFourCorner	88000
U+4ECE.1	fMandarin	cong1
U+4ECF.0	fUTF8	仏
U+4ECF.0	fCNS11643	3-214A
U+4ECF.0	fGB	E-214A
U+4ECF.0	fR/S	  9.2
U+4ECF.0	fTotalStrokes	 4
U+4ECF.0	fCangjie	OI
U+4ECF.0	fFourCorner	22230
U+4ECF.0	fCantonese	fat6
U+4ECF.0	fMandarin	fo2; fu2; bo2
U+4ECF.0	fEnglish	buddha
U+4ED0.0	fUTF8	仐
U+4ED0.0	fCNS11643	4-213C
U+4ED0.0	fGB	E-214B
U+4ED0.0	fR/S	 9.2
U+4ED0.0	fTotalStrokes	 4
U+4ED0.0	fCangjie	OJ
U+4ED0.0	fFourCorner	80400
U+4ED0.0	fCantonese	tou1
U+4ED0.0	fMandarin	san3
U+4ED0.0	fEnglish	non-classical form of &#x5098;
U+4ED1.0	fUTF8	仑
U+4ED1.0	fGB	0-4258
U+4ED1.0	fR/S	 9.2
U+4ED1.0	fTotalStrokes	 4
U+4ED1.0	fCangjie	XXOP
U+4ED1.0	fHakka	 lun2
U+4ED1.0	fCantonese	leon4; leon2 (also)
U+4ED1.0	fMandarin	lun2
U+4ED1.0	fEnglish	simplified form of &#x4F96;
U+4ED2.0	fUTF8	仒
U+4ED2.0	fGB	1-7D73
U+4ED2.0	fR/S	 9.2
U+4ED2.0	fTotalStrokes	 4
U+4ED2.0	fCangjie	XOY
U+4ED2.0	fCantonese	bing1
U+4ED2.0	fEnglish	 kwukyel
U+4ED3.0	fUTF8	仓
U+4ED3.0	fGB	0-3256
U+4ED3.0	fR/S	 9.2
U+4ED3.0	fTotalStrokes	 4
U+4ED3.0	fCangjie	OSU
U+4ED3.0	fHakka	 cong1
U+4ED3.0	fCantonese	cong1
U+4ED3.0	fMandarin	cang1; chuan4; chuang4
U+4ED3.0	fEnglish	simplified form of &#x5009;
U+4ED4.0	fUTF8	仔
U+4ED4.0	fCNS11643	1-456A
U+4ED4.0	fBig5	A54A
U+4ED4.0	fGB	0-5750
U+4ED4.0	fR/S	 9.3
U+4ED4.0	fTotalStrokes	 5
U+4ED4.0	fCangjie	OND
U+4ED4.0	fFourCorner	27247
U+4ED4.0	fHakka	 zai3;  zih3; ze3; zu3; e3; e2; zii3
U+4ED4.0	fCantonese	zai2; zi1; zai4 (also)
U+4ED4.0	fMandarin	zai3
U+4ED4.0	fEnglish	[1] young animals [2] child [3] small thing [4] one who tends cattle
U+4ED4.1	fUTF8	仔
U+4ED4.1	fCNS11643	1-456A
U+4ED4.1	fBig5	A54A
U+4ED4.1	fGB	0-5750
U+4ED4.1	fR/S	 9.3
U+4ED4.1	fTotalStrokes	 5
U+4ED4.1	fCangjie	OND
U+4ED4.1	fFourCorner	27247
U+4ED4.1	fMandarin	zi3; zi1
U+4ED4.1	fEnglish	careful
U+4ED4.2	fUTF8	仔
U+4ED4.2	fCNS11643	1-456A
U+4ED4.2	fBig5	A54A
U+4ED4.2	fGB	0-5750
U+4ED4.2	fR/S	 9.3
U+4ED4.2	fTotalStrokes	 5
U+4ED4.2	fCangjie	OND
U+4ED4.2	fFourCorner	27247
U+4ED4.2	fCantonese	zi2
U+4ED5.0	fUTF8	仕
U+4ED5.0	fCNS11643	1-456B
U+4ED5.0	fBig5	A54B
U+4ED5.0	fGB	0-4A4B
U+4ED5.0	fR/S	 9.3
U+4ED5.0	fTotalStrokes	 5
U+4ED5.0	fCangjie	OG
U+4ED5.0	fFourCorner	24210
U+4ED5.0	fHakka	 sii5; su3; sii6; se5
U+4ED5.0	fCantonese	si6
U+4ED5.0	fMandarin	shi4
U+4ED5.0	fEnglish	[1] official [2] enter government service; serve government
U+4ED6.0	fUTF8	他
U+4ED6.0	fCNS11643	1-456C
U+4ED6.0	fBig5	A54C
U+4ED6.0	fGB	0-4B7B
U+4ED6.0	fR/S	 9.3
U+4ED6.0	fTotalStrokes	 5
U+4ED6.0	fCangjie	OPD
U+4ED6.0	fFourCorner	24212
U+4ED6.0	fHakka	  ta1
U+4ED6.0	fCantonese	taa1; to1
U+4ED6.0	fMandarin	ta1; duo4; tuo1
U+4ED6.0	fEnglish	[1] he; him [2] other; another [3] different [4] elsewhere [5] future
U+4ED7.0	fUTF8	仗
U+4ED7.0	fCNS11643	1-456D
U+4ED7.0	fBig5	A54D
U+4ED7.0	fGB	0-554C
U+4ED7.0	fR/S	 9.3
U+4ED7.0	fTotalStrokes	 5
U+4ED7.0	fCangjie	OJK
U+4ED7.0	fFourCorner	25200
U+4ED7.0	fHakka	 zong5; zhong5; cong5; cong3; chong3; chong6
U+4ED7.0	fCantonese	zoeng3; zoeng2 (also)
U+4ED7.0	fMandarin	zhang4; zhang3
U+4ED7.0	fEnglish	[1] weaponry [2] lean upon; rely upon; depend on [3] battle; fight; war [4] weaponry on parade [5] take up [6] stand up in defense
U+4ED7.1	fUTF8	仗
U+4ED7.1	fCNS11643	1-456D
U+4ED7.1	fBig5	A54D
U+4ED7.1	fGB	0-554C
U+4ED7.1	fR/S	 9.3
U+4ED7.1	fTotalStrokes	 5
U+4ED7.1	fCangjie	OJK
U+4ED7.1	fFourCorner	25200
U+4ED7.1	fCantonese	zoeng6
U+4ED8.0	fUTF8	付
U+4ED8.0	fCNS11643	1-4569
U+4ED8.0	fBig5	A549
U+4ED8.0	fGB	0-3836
U+4ED8.0	fR/S	 9.3
U+4ED8.0	fTotalStrokes	 5
U+4ED8.0	fCangjie	ODI
U+4ED8.0	fFourCorner	24200
U+4ED8.0	fHakka	  fu5
U+4ED8.0	fCantonese	fu6
U+4ED8.0	fMandarin	fu4
U+4ED8.0	fEnglish	[1] pay (money) [2] deliver (goods); hand over; consign [3] dismiss off the mind [4] entrust
U+4ED8.1	fUTF8	付
U+4ED8.1	fCNS11643	1-4569
U+4ED8.1	fBig5	A549
U+4ED8.1	fGB	0-3836
U+4ED8.1	fR/S	 9.3
U+4ED8.1	fTotalStrokes	 5
U+4ED8.1	fCangjie	ODI
U+4ED8.1	fFourCorner	24200
U+4ED8.1	fMandarin	fu5
U+4ED9.0	fUTF8	仙
U+4ED9.0	fCNS11643	1-4570
U+4ED9.0	fBig5	A550
U+4ED9.0	fGB	0-4F49
U+4ED9.0	fR/S	 9.3
U+4ED9.0	fTotalStrokes	 5
U+4ED9.0	fCangjie	OU
U+4ED9.0	fFourCorner	22270
U+4ED9.0	fHakka	 sien1; sen1; sian1
U+4ED9.0	fCantonese	sin1
U+4ED9.0	fMandarin	xian1
U+4ED9.0	fEnglish	[1] god; immortal; fairy [2] divine [3] good [4] immortal; celestial talent or heavenly beauty (often used as modifier, in praise of) [5] noun modifier, having to do with Taoist, esoteric magic
U+4ED9.1	fUTF8	仙
U+4ED9.1	fCNS11643	1-4570
U+4ED9.1	fBig5	A550
U+4ED9.1	fGB	0-4F49
U+4ED9.1	fR/S	 9.3
U+4ED9.1	fTotalStrokes	 5
U+4ED9.1	fCangjie	OU
U+4ED9.1	fFourCorner	22270
U+4ED9.1	fMandarin	xian5
U+4EDA.0	fUTF8	仚
U+4EDA.0	fCNS11643	2-214A
U+4EDA.0	fBig5	C96A
U+4EDA.0	fGB	3-3059
U+4EDA.0	fR/S	 9.3
U+4EDA.0	fTotalStrokes	 5
U+4EDA.0	fCangjie	OU
U+4EDA.0	fFourCorner	80770
U+4EDA.0	fCantonese	hin1
U+4EDA.0	fMandarin	xian1
U+4EDA.0	fEnglish	used in place name
U+4EDB.0	fUTF8	仛
U+4EDB.0	fCNS11643	4-215B
U+4EDB.0	fGB	5-3149
U+4EDB.0	fR/S	  9.3
U+4EDB.0	fTotalStrokes	 5
U+4EDB.0	fCangjie	OHP
U+4EDB.0	fFourCorner	22210
U+4EDB.0	fHakka	 za5
U+4EDB.0	fCantonese	tok3
U+4EDB.0	fMandarin	duo2; tuo1
U+4EDB.0	fEnglish	 he
U+4EDB.1	fUTF8	仛
U+4EDB.1	fCNS11643	4-215B
U+4EDB.1	fGB	5-3149
U+4EDB.1	fR/S	  9.3
U+4EDB.1	fTotalStrokes	 5
U+4EDB.1	fCangjie	OHP
U+4EDB.1	fFourCorner	22210
U+4EDB.2	fUTF8	仛
U+4EDB.2	fCNS11643	4-215B
U+4EDB.2	fGB	5-3149
U+4EDB.2	fR/S	  9.3
U+4EDB.2	fTotalStrokes	 5
U+4EDB.2	fCangjie	OHP
U+4EDB.2	fFourCorner	22210
U+4EDB.2	fMandarin	cha4
U+4EDB.2	fEnglish	[1] extravagant and dissipated
U+4EDB.3	fUTF8	仛
U+4EDB.3	fCNS11643	4-215B
U+4EDB.3	fGB	5-3149
U+4EDB.3	fR/S	  9.3
U+4EDB.3	fTotalStrokes	 5
U+4EDB.3	fCangjie	OHP
U+4EDB.3	fFourCorner	22210
U+4EDB.3	fMandarin	zhe2
U+4EDB.3	fEnglish	[..]
U+4EDC.0	fUTF8	仜
U+4EDC.0	fCNS11643	2-2146
U+4EDC.0	fBig5	C966
U+4EDC.0	fGB	5-3147
U+4EDC.0	fR/S	 9.3
U+4EDC.0	fTotalStrokes	 5
U+4EDC.0	fCangjie	OM
U+4EDC.0	fFourCorner	21210
U+4EDC.0	fCantonese	hung4
U+4EDC.0	fMandarin	hong2
U+4EDD.0	fUTF8	仝
U+4EDD.0	fCNS11643	2-2149
U+4EDD.0	fBig5	C969
U+4EDD.0	fGB	0-595A
U+4EDD.0	fR/S	 9.3
U+4EDD.0	fTotalStrokes	 5
U+4EDD.0	fCangjie	OM
U+4EDD.0	fFourCorner	80101
U+4EDD.0	fHakka	  tung2; cien2
U+4EDD.0	fCantonese	tung4
U+4EDD.0	fMandarin	tong2; tong4
U+4EDD.0	fEnglish	[1] a Chinese family name [2] variant of &#x540C;
U+4EDE.0	fUTF8	仞
U+4EDE.0	fCNS11643	1-4571
U+4EDE.0	fBig5	A551
U+4EDE.0	fGB	0-5870
U+4EDE.0	fR/S	 9.3
U+4EDE.0	fTotalStrokes	 5
U+4EDE.0	fCangjie	OSHI
U+4EDE.0	fFourCorner	27220
U+4EDE.0	fHakka	 ngiun3; niun5
U+4EDE.0	fCantonese	jan6
U+4EDE.0	fMandarin	ren4
U+4EDE.0	fEnglish	[1] ancient unit of length (8 feet) [2] measure depth; fathom
U+4EDF.0	fUTF8	仟
U+4EDF.0	fCNS11643	1-4623
U+4EDF.0	fBig5	A561
U+4EDF.0	fGB	0-472A
U+4EDF.0	fR/S	 9.3
U+4EDF.0	fTotalStrokes	 5
U+4EDF.0	fCangjie	OHJ
U+4EDF.0	fFourCorner	22240
U+4EDF.0	fHakka	 cien1; ch'ien1; cen1; ts'ian1
U+4EDF.0	fCantonese	cin1
U+4EDF.0	fMandarin	qian1
U+4EDF.0	fEnglish	[1] leader of one thousand men [2] thousand [3] variant of &#x9621;
U+4EE0.0	fUTF8	仠
U+4EE0.0	fCNS11643	3-216D
U+4EE0.0	fGB	5-3146
U+4EE0.0	fR/S	  9.3
U+4EE0.0	fTotalStrokes	 5
U+4EE0.0	fCangjie	OMJ
U+4EE0.0	fFourCorner	21240
U+4EE0.0	fCantonese	gon2
U+4EE0.0	fMandarin	gan2; gan3; han4
U+4EE1.0	fUTF8	仡
U+4EE1.0	fCNS11643	2-2148
U+4EE1.0	fBig5	C968
U+4EE1.0	fGB	0-586E
U+4EE1.0	fR/S	 9.3
U+4EE1.0	fTotalStrokes	 5
U+4EE1.0	fCangjie	OON
U+4EE1.0	fFourCorner	28217
U+4EE1.0	fHakka	 ngiet7; nget7; jat7
U+4EE1.0	fCantonese	ngat6; go1; gat1 (also); at6 (also); hat1; gaak3
U+4EE1.0	fMandarin	yi4; wu4
U+4EE1.0	fEnglish	[1] gallant; valiant [2] stately [3] upright
U+4EE1.1	fUTF8	仡
U+4EE1.1	fCNS11643	2-2148
U+4EE1.1	fBig5	C968
U+4EE1.1	fGB	0-586E
U+4EE1.1	fR/S	 9.3
U+4EE1.1	fTotalStrokes	 5
U+4EE1.1	fCangjie	OON
U+4EE1.1	fFourCorner	28217
U+4EE1.1	fCantonese	ko1
U+4EE1.1	fMandarin	ge1
U+4EE2.0	fUTF8	仢
U+4EE2.0	fCNS11643	4-215A
U+4EE2.0	fGB	5-314A
U+4EE2.0	fR/S	  9.3
U+4EE2.0	fTotalStrokes	 5
U+4EE2.0	fCangjie	OPI
U+4EE2.0	fFourCorner	27220
U+4EE2.0	fCantonese	bok6
U+4EE2.0	fMandarin	di2
U+4EE3.0	fUTF8	代
U+4EE3.0	fCNS11643	1-456E
U+4EE3.0	fBig5	A54E
U+4EE3.0	fGB	0-347A
U+4EE3.0	fR/S	 9.3
U+4EE3.0	fTotalStrokes	 5
U+4EE3.0	fCangjie	OIP
U+4EE3.0	fFourCorner	23240
U+4EE3.0	fHakka	  toi5; toi6; toi3
U+4EE3.0	fCantonese	doi6
U+4EE3.0	fMandarin	dai4
U+4EE3.0	fEnglish	[1] generation [2] dynasty [3] era; period [4] be a substitute or an equivalent; take place of; replace; replacement (of person or generation) [5] ancient kingdom [6] a Chinese family name
U+4EE4.0	fUTF8	令
U+4EE4.0	fCNS11643	1-456F
U+4EE4.0	fBig5	A54F
U+4EE4.0	fGB	0-416E
U+4EE4.0	fR/S	 9.3
U+4EE4.0	fTotalStrokes	 5
U+4EE4.0	fCangjie	OINI
U+4EE4.0	fFourCorner	80307
U+4EE4.0	fHakka	  lin5;  lin2; lin6; lin3
U+4EE4.0	fCantonese	ling6; ling4; lim1 (also); ling2 (also); ling5
U+4EE4.0	fMandarin	ling4; ling3
U+4EE4.0	fEnglish	[1] directive; order [2] order [3] cause; make [4] nice; good; excellent; esteemed [5] the law [6] season [7] your [8] a Chinese family name
U+4EE4.1	fUTF8	令
U+4EE4.1	fCNS11643	1-456F
U+4EE4.1	fBig5	A54F
U+4EE4.1	fGB	0-416E
U+4EE4.1	fR/S	 9.3
U+4EE4.1	fTotalStrokes	 5
U+4EE4.1	fCangjie	OINI
U+4EE4.1	fFourCorner	80307
U+4EE4.1	fCantonese	ling1
U+4EE4.1	fMandarin	ling2
U+4EE4.1	fEnglish	a ream (of paper)
U+4EE5.0	fUTF8	以
U+4EE5.0	fCNS11643	1-4568
U+4EE5.0	fBig5	A548
U+4EE5.0	fGB	0-5254
U+4EE5.0	fR/S	 9.3
U+4EE5.0	fTotalStrokes	 5
U+4EE5.0	fCangjie	VIO
U+4EE5.0	fFourCorner	28100
U+4EE5.0	fHakka	 ji1; ri1; ji3; ri5; ji5; i1
U+4EE5.0	fCantonese	ji5; jyu5
U+4EE5.0	fMandarin	yi3
U+4EE5.0	fEnglish	[1] by means of; because of; therefore [2] consider as [3] in order to [4] with; indicating implement; instrumentality [5] giving cause
U+4EE6.0	fUTF8	仦
U+4EE6.0	fCNS11643	3-2172
U+4EE6.0	fGB	E-214C
U+4EE6.0	fR/S	  9.3
U+4EE6.0	fTotalStrokes	 5
U+4EE6.0	fCangjie	OF
U+4EE6.0	fFourCorner	29200
U+4EE6.0	fCantonese	caau3
U+4EE6.0	fMandarin	chao4
U+4EE7.0	fUTF8	仧
U+4EE7.0	fCNS11643	3-2170
U+4EE7.0	fGB	E-214D
U+4EE7.0	fR/S	 9.3
U+4EE7.0	fTotalStrokes	 5
U+4EE7.0	fCangjie	YMO
U+4EE7.0	fFourCorner	21800
U+4EE7.0	fCantonese	coeng4
U+4EE7.0	fMandarin	chang2; zhang3
U+4EE8.0	fUTF8	仨
U+4EE8.0	fCNS11643	2-2145
U+4EE8.0	fBig5	C965
U+4EE8.0	fGB	0-586D
U+4EE8.0	fR/S	 9.3
U+4EE8.0	fTotalStrokes	 5
U+4EE8.0	fCangjie	OMMM
U+4EE8.0	fFourCorner	21211
U+4EE8.0	fCantonese	saam1; saa1 (also)
U+4EE8.0	fMandarin	sa1
U+4EE8.0	fEnglish	three (Peking colloquialism); cannot be followed by measure word
U+4EE9.0	fUTF8	仩
U+4EE9.0	fCNS11643	2-2147
U+4EE9.0	fBig5	C967
U+4EE9.0	fGB	E-214E
U+4EE9.0	fR/S	 9.3
U+4EE9.0	fTotalStrokes	 5
U+4EE9.0	fCangjie	OYM
U+4EE9.0	fFourCorner	21210
U+4EE9.0	fCantonese	coeng4
U+4EE9.0	fMandarin	shang4
U+4EEA.0	fUTF8	仪
U+4EEA.0	fGB	0-5247
U+4EEA.0	fR/S	 9.3
U+4EEA.0	fTotalStrokes	 5
U+4EEA.0	fCangjie	OIK
U+4EEA.0	fHakka	 ngi2
U+4EEA.0	fCantonese	ji4
U+4EEA.0	fMandarin	yi2
U+4EEA.0	fEnglish	simplified form of &#x5100;
U+4EEB.0	fUTF8	仫
U+4EEB.0	fGB	0-586F
U+4EEB.0	fR/S	 9.3
U+4EEB.0	fTotalStrokes	 5
U+4EEB.0	fCangjie	OHI
U+4EEB.0	fCantonese	muk6; mo1 (also)
U+4EEB.0	fMandarin	mu4
U+4EEB.0	fEnglish	tribe
U+4EEC.0	fUTF8	们
U+4EEC.0	fGB	0-4347
U+4EEC.0	fR/S	 9.3
U+4EEC.0	fTotalStrokes	 5
U+4EEC.0	fCangjie	OLS
U+4EEC.0	fHakka	 mun2
U+4EEC.0	fCantonese	mun4
U+4EEC.0	fMandarin	men5; men2
U+4EEC.0	fEnglish	simplified form of &#x5011;
U+4EED.0	fUTF8	仭
U+4EED.0	fGB	E-214F
U+4EED.0	fR/S	 9.3
U+4EED.0	fTotalStrokes	 5
U+4EED.0	fCangjie	XXOSK
U+4EED.0	fHakka	 ngiun3
U+4EED.0	fCantonese	jan6
U+4EED.0	fMandarin	ren4
U+4EED.0	fEnglish	 an ancient unit of measure (8 feet) "fathom"
U+4EEE.0	fUTF8	仮
U+4EEE.0	fCNS11643	3-226F
U+4EEE.0	fGB	5-3158
U+4EEE.0	fR/S	  9.4
U+4EEE.0	fTotalStrokes	 6
U+4EEE.0	fCangjie	OHE
U+4EEE.0	fFourCorner	21240
U+4EEE.0	fCantonese	faan2
U+4EEE.0	fMandarin	fan3; jia3; jia4; jia1; jia2
U+4EEE.0	fEnglish	falsehood, deception; vacation
U+4EEF.0	fUTF8	仯
U+4EEF.0	fCNS11643	3-2268
U+4EEF.0	fGB	5-3155
U+4EEF.0	fR/S	  9.4
U+4EEF.0	fTotalStrokes	 6
U+4EEF.0	fCangjie	OFH
U+4EEF.0	fFourCorner	29220
U+4EEF.0	fHakka	  miau3; meu3
U+4EEF.0	fCantonese	caau3
U+4EEF.0	fMandarin	chao4
U+4EEF.0	fEnglish	small
U+4EF0.0	fUTF8	仰
U+4EF0.0	fCNS11643	1-4737
U+4EF0.0	fBig5	A5F5
U+4EF0.0	fGB	0-5176
U+4EF0.0	fR/S	 9.4
U+4EF0.0	fTotalStrokes	 6
U+4EF0.0	fCangjie	OHVL
U+4EF0.0	fFourCorner	27220
U+4EF0.0	fHakka	  ngiong3;  ngiong1; gniong3; niong1; ngok8
U+4EF0.0	fCantonese	joeng5; ngong5 (also); ong5 (also)
U+4EF0.0	fMandarin	yang3; ang2; yang4
U+4EF0.0	fEnglish	[1] raise the head to look; look up to; up; upwards [2] adore; admire; revere [3] lean; rely upon [4] hope [5] swallow
U+4EF1.0	fUTF8	仱
U+4EF1.0	fCNS11643	2-216F
U+4EF1.0	fBig5	C9B0
U+4EF1.0	fGB	3-3062
U+4EF1.0	fR/S	 9.4
U+4EF1.0	fTotalStrokes	 6
U+4EF1.0	fCangjie	OOIN
U+4EF1.0	fFourCorner	28227
U+4EF1.0	fCantonese	kim4; ging1
U+4EF1.0	fMandarin	qian2; jing1
U+4EF2.0	fUTF8	仲
U+4EF2.0	fCNS11643	1-4734
U+4EF2.0	fBig5	A5F2
U+4EF2.0	fGB	0-5659
U+4EF2.0	fR/S	 9.4
U+4EF2.0	fTotalStrokes	 6
U+4EF2.0	fCangjie	OL
U+4EF2.0	fFourCorner	25206
U+4EF2.0	fHakka	 chung5; cung5; chung6; cung3; zung3
U+4EF2.0	fCantonese	zung6
U+4EF2.0	fMandarin	zhong4
U+4EF2.0	fEnglish	[1] in the middle; between two entities [2] second (in order of birth) [3] brothers [4] a Chinese family name
U+4EF3.0	fUTF8	仳
U+4EF3.0	fCNS11643	1-4738
U+4EF3.0	fBig5	A5F6
U+4EF3.0	fGB	0-5872
U+4EF3.0	fR/S	 9.4
U+4EF3.0	fTotalStrokes	 6
U+4EF3.0	fCangjie	OPP
U+4EF3.0	fFourCorner	21210
U+4EF3.0	fHakka	 pi2;  pi1; pi3
U+4EF3.0	fCantonese	pei2; pei4
U+4EF3.0	fMandarin	pi3; pi2
U+4EF3.0	fEnglish	 part company; separate (of lovers, husband and wife, relatives)
U+4EF4.0	fUTF8	仴
U+4EF4.0	fCNS11643	2-2179
U+4EF4.0	fBig5	C9BA
U+4EF4.0	fGB	5-3159
U+4EF4.0	fR/S	 9.4
U+4EF4.0	fTotalStrokes	 6
U+4EF4.0	fCangjie	OB
U+4EF4.0	fFourCorner	27220
U+4EF4.0	fCantonese	wut6; jyut6
U+4EF4.0	fMandarin	wan4
U+4EF5.0	fUTF8	仵
U+4EF5.0	fCNS11643	2-216D
U+4EF5.0	fBig5	C9AE
U+4EF5.0	fGB	0-5875
U+4EF5.0	fR/S	 9.4
U+4EF5.0	fTotalStrokes	 6
U+4EF5.0	fCangjie	OOJ
U+4EF5.0	fFourCorner	28240
U+4EF5.0	fHakka	 ng3
U+4EF5.0	fCantonese	ng5
U+4EF5.0	fMandarin	wu3
U+4EF5.0	fEnglish	a Chinese family name
U+4EF6.0	fUTF8	件
U+4EF6.0	fCNS11643	1-4735
U+4EF6.0	fBig5	A5F3
U+4EF6.0	fGB	0-3C7E
U+4EF6.0	fR/S	 9.4
U+4EF6.0	fTotalStrokes	 6
U+4EF6.0	fCangjie	OHQ
U+4EF6.0	fFourCorner	25200
U+4EF6.0	fHakka	 kien5; ken3; kien6; k'ian6; ken5; kian5
U+4EF6.0	fCantonese	gin6; gin2 (also)
U+4EF6.0	fMandarin	jian4
U+4EF6.0	fEnglish	auxiliary noun applied to articles, things, clothes, etc.
U+4EF7.0	fUTF8	价
U+4EF7.0	fCNS11643	2-2171
U+4EF7.0	fBig5	C9B2
U+4EF7.0	fGB	0-3C5B
U+4EF7.0	fR/S	 9.4
U+4EF7.0	fTotalStrokes	 6
U+4EF7.0	fCangjie	OOLL
U+4EF7.0	fFourCorner	28220
U+4EF7.0	fHakka	 gai5; kie5; kiai5; ga5; giai5
U+4EF7.0	fCantonese	gaai3; gaa3; gai3
U+4EF7.0	fMandarin	jie4; jie5
U+4EF7.0	fEnglish	[1] good [2] male servant
U+4EF7.1	fUTF8	价
U+4EF7.1	fCNS11643	2-2171
U+4EF7.1	fBig5	C9B2
U+4EF7.1	fGB	0-3C5B
U+4EF7.1	fR/S	 9.4
U+4EF7.1	fTotalStrokes	 6
U+4EF7.1	fCangjie	OOLL
U+4EF7.1	fFourCorner	28220
U+4EF7.1	fMandarin	ga1
U+4EF7.1	fEnglish	[1] here [2] there
U+4EF7.2	fUTF8	价
U+4EF7.2	fCNS11643	2-2171
U+4EF7.2	fBig5	C9B2
U+4EF7.2	fGB	0-3C5B
U+4EF7.2	fR/S	 9.4
U+4EF7.2	fTotalStrokes	 6
U+4EF7.2	fCangjie	OOLL
U+4EF7.2	fFourCorner	28220
U+4EF7.2	fMandarin	jia4
U+4EF8.0	fUTF8	仸
U+4EF8.0	fCNS11643	3-226B
U+4EF8.0	fGB	5-3157
U+4EF8.0	fR/S	 9.4
U+4EF8.0	fTotalStrokes	 6
U+4EF8.0	fCangjie	OHK
U+4EF8.0	fFourCorner	22230
U+4EF8.0	fCantonese	jiu2
U+4EF8.0	fMandarin	fo2; yao3
U+4EF8.0	fEnglish	 bent; distorted; crooked feeble
U+4EF9.0	fUTF8	仹
U+4EF9.0	fCNS11643	3-2267
U+4EF9.0	fGB	5-314E
U+4EF9.0	fR/S	 9.4
U+4EF9.0	fTotalStrokes	 6
U+4EF9.0	fCangjie	OQJ
U+4EF9.0	fFourCorner	25200
U+4EF9.0	fCantonese	fung1
U+4EF9.0	fMandarin	feng1
U+4EFA.0	fUTF8	仺
U+4EFA.0	fCNS11643	3-2276
U+4EFA.0	fGB	E-2150
U+4EFA.0	fR/S	 9.4
U+4EFA.0	fTotalStrokes	 6
U+4EFA.0	fCangjie	OSM
U+4EFA.0	fFourCorner	80100
U+4EFA.0	fCantonese	cong1
U+4EFA.0	fMandarin	cang1
U+4EFB.0	fUTF8	任
U+4EFB.0	fCNS11643	1-4736
U+4EFB.0	fBig5	A5F4
U+4EFB.0	fGB	0-484E
U+4EFB.0	fR/S	 9.4
U+4EFB.0	fTotalStrokes	 6
U+4EFB.0	fCangjie	OHG
U+4EFB.0	fFourCorner	22214
U+4EFB.0	fHakka	 ngim3;  jim5;  ngim5; gnim5; gnim6; rim6; ngim6; niung5
U+4EFB.0	fCantonese	jam6; jam4
U+4EFB.0	fMandarin	ren4
U+4EFB.0	fEnglish	[1] duty [2] let (one act at will) [3] employ (one for a job) [4] bear (a burden) [5] official post; office [6] be responsible for [7] let; allow [8] trust [9] any; no matter who, what, how
U+4EFB.1	fUTF8	任
U+4EFB.1	fCNS11643	1-4736
U+4EFB.1	fBig5	A5F4
U+4EFB.1	fGB	0-484E
U+4EFB.1	fR/S	 9.4
U+4EFB.1	fTotalStrokes	 6
U+4EFB.1	fCangjie	OHG
U+4EFB.1	fFourCorner	22214
U+4EFB.1	fMandarin	ren2
U+4EFB.1	fEnglish	[1] a Chinese family name [2] used for &#x598A;
U+4EFC.0	fUTF8	仼
U+4EFC.0	fCNS11643	3-226E
U+4EFC.0	fGB	E-2151
U+4EFC.0	fR/S	  9.4
U+4EFC.0	fTotalStrokes	 6
U+4EFC.0	fCangjie	OMG
U+4EFC.0	fFourCorner	21210
U+4EFC.0	fCantonese	wong4
U+4EFC.0	fMandarin	wang2
U+4EFD.0	fUTF8	份
U+4EFD.0	fCNS11643	1-4739
U+4EFD.0	fBig5	A5F7
U+4EFD.0	fGB	0-375D
U+4EFD.0	fR/S	 9.4
U+4EFD.0	fTotalStrokes	 6
U+4EFD.0	fCangjie	OCSH
U+4EFD.0	fFourCorner	28227
U+4EFD.0	fHakka	 fun5;  fun1; fun6; bin1; fun3; bun1
U+4EFD.0	fCantonese	fan6; ban1; fan2 (also)
U+4EFD.0	fMandarin	fen4
U+4EFD.0	fEnglish	[1] portion; part [2] duty [3] family
U+4EFD.1	fUTF8	份
U+4EFD.1	fCNS11643	1-4739
U+4EFD.1	fBig5	A5F7
U+4EFD.1	fGB	0-375D
U+4EFD.1	fR/S	 9.4
U+4EFD.1	fTotalStrokes	 6
U+4EFD.1	fCangjie	OCSH
U+4EFD.1	fFourCorner	28227
U+4EFD.1	fMandarin	fen5
U+4EFE.0	fUTF8	仾
U+4EFE.0	fCNS11643	4-222D
U+4EFE.0	fGB	E-2152
U+4EFE.0	fR/S	 9.4
U+4EFE.0	fTotalStrokes	 6
U+4EFE.0	fCangjie	OMVM
U+4EFE.0	fFourCorner	21210
U+4EFE.0	fHakka	  dai1; di1
U+4EFE.0	fCantonese	dai1
U+4EFE.0	fMandarin	di1
U+4EFE.0	fEnglish	to hang or bow the head, to droop, to lower; low, beneath
U+4EFF.0	fUTF8	仿
U+4EFF.0	fCNS11643	1-472B
U+4EFF.0	fBig5	A5E9
U+4EFF.0	fGB	0-3742
U+4EFF.0	fR/S	 9.4
U+4EFF.0	fTotalStrokes	 6
U+4EFF.0	fCangjie	OYHS
U+4EFF.0	fFourCorner	20227
U+4EFF.0	fHakka	  fong3; fang5
U+4EFF.0	fCantonese	fong2
U+4EFF.0	fMandarin	fang3; fang2; pang2
U+4EFF.0	fEnglish	 imitate, copy
U+4EFF.1	fUTF8	仿
U+4EFF.1	fCNS11643	1-472B
U+4EFF.1	fBig5	A5E9
U+4EFF.1	fGB	0-3742
U+4EFF.1	fR/S	 9.4
U+4EFF.1	fTotalStrokes	 6
U+4EFF.1	fCangjie	OYHS
U+4EFF.1	fFourCorner	20227
U+4EFF.1	fCantonese	pong4
U+4F00.0	fUTF8	伀
U+4F00.0	fCNS11643	2-2170
U+4F00.0	fBig5	C9B1
U+4F00.0	fGB	3-3063
U+4F00.0	fR/S	 9.4
U+4F00.0	fTotalStrokes	 6
U+4F00.0	fCangjie	OCI
U+4F00.0	fFourCorner	28232
U+4F00.0	fHakka	 zhung1; zung1
U+4F00.0	fCantonese	zung1
U+4F00.0	fMandarin	zhong1
U+4F00.0	fEnglish	excited
U+4F01.0	fUTF8	企
U+4F01.0	fCNS11643	1-473A
U+4F01.0	fBig5	A5F8
U+4F01.0	fGB	0-4673
U+4F01.0	fR/S	 9.4
U+4F01.0	fTotalStrokes	 6
U+4F01.0	fCangjie	OYLM
U+4F01.0	fFourCorner	80101
U+4F01.0	fHakka	  ki1; ki3
U+4F01.0	fCantonese	kei5; kei2 (also)
U+4F01.0	fMandarin	qi3
U+4F01.0	fEnglish	[1] stand on tiptoe [2] hope; long; expect
U+4F02.0	fUTF8	伂
U+4F02.0	fCNS11643	2-2174
U+4F02.0	fBig5	C9B5
U+4F02.0	fGB	5-3151
U+4F02.0	fR/S	 9.4
U+4F02.0	fTotalStrokes	 6
U+4F02.0	fCangjie	OJB
U+4F02.0	fFourCorner	25227
U+4F02.0	fCantonese	pui3
U+4F02.0	fMandarin	pei4
U+4F03.0	fUTF8	伃
U+4F03.0	fCNS11643	3-226D
U+4F03.0	fGB	3-3068
U+4F03.0	fR/S	 9.4
U+4F03.0	fTotalStrokes	 6
U+4F03.0	fCangjie	ONIN
U+4F03.0	fFourCorner	27220
U+4F03.0	fHakka	 ji2; ri2; i2
U+4F03.0	fCantonese	jyu4
U+4F03.0	fMandarin	yu3; yu2
U+4F03.0	fEnglish	handsome
U+4F04.0	fUTF8	伄
U+4F04.0	fCNS11643	2-2178
U+4F04.0	fBig5	C9B9
U+4F04.0	fGB	3-3067
U+4F04.0	fR/S	 9.4
U+4F04.0	fTotalStrokes	 6
U+4F04.0	fCangjie	ONL
U+4F04.0	fFourCorner	27227
U+4F04.0	fHakka	 diau5
U+4F04.0	fCantonese	diu3
U+4F04.0	fMandarin	diao4
U+4F04.0	fEnglish	ripe ears of grain hanging down
U+4F05.0	fUTF8	伅
U+4F05.0	fCNS11643	2-2175
U+4F05.0	fBig5	C9B6
U+4F05.0	fGB	5-3154
U+4F05.0	fR/S	 9.4
U+4F05.0	fTotalStrokes	 6
U+4F05.0	fCangjie	OPU
U+4F05.0	fFourCorner	25217
U+4F05.0	fCantonese	dan6; deon6
U+4F05.0	fMandarin	dun4
U+4F06.0	fUTF8	伆
U+4F06.0	fCNS11643	3-226C
U+4F06.0	fGB	3-3064
U+4F06.0	fR/S	  9.4
U+4F06.0	fTotalStrokes	 6
U+4F06.0	fCangjie	OPHH
U+4F06.0	fFourCorner	27220
U+4F06.0	fHakka	 wut7; vut8
U+4F06.0	fCantonese	mat6
U+4F06.0	fMandarin	wen2; wu4; wen4
U+4F06.0	fEnglish	 cut asunder
U+4F07.0	fUTF8	伇
U+4F07.0	fCNS11643	3-2274
U+4F07.0	fGB	5-315B
U+4F07.0	fR/S	  9.4
U+4F07.0	fTotalStrokes	 6
U+4F07.0	fCangjie	OHNE
U+4F07.0	fFourCorner	27240
U+4F07.0	fCantonese	jik6
U+4F07.0	fMandarin	yi4
U+4F08.0	fUTF8	伈
U+4F08.0	fCNS11643	2-2172
U+4F08.0	fBig5	C9B3
U+4F08.0	fGB	3-3066
U+4F08.0	fR/S	 9.4
U+4F08.0	fTotalStrokes	 6
U+4F08.0	fCangjie	OP
U+4F08.0	fFourCorner	23200
U+4F08.0	fHakka	  sim3; cim5
U+4F08.0	fCantonese	sam2
U+4F08.0	fMandarin	xin3; qin4
U+4F08.0	fEnglish	nervous; fearful
U+4F09.0	fUTF8	伉
U+4F09.0	fCNS11643	1-472C
U+4F09.0	fBig5	A5EA
U+4F09.0	fGB	0-5878
U+4F09.0	fR/S	 9.4
U+4F09.0	fTotalStrokes	 6
U+4F09.0	fCangjie	OYHN
U+4F09.0	fFourCorner	20217
U+4F09.0	fHakka	 kong5
U+4F09.0	fCantonese	kong3
U+4F09.0	fMandarin	kang4; gang1
U+4F09.0	fEnglish	[1] spouse; matrimonial match [2] strong; virile
U+4F0A.0	fUTF8	伊
U+4F0A.0	fCNS11643	1-472E
U+4F0A.0	fBig5	A5EC
U+4F0A.0	fGB	0-5241
U+4F0A.0	fR/S	 9.4
U+4F0A.0	fTotalStrokes	 6
U+4F0A.0	fCangjie	OSK
U+4F0A.0	fFourCorner	27257
U+4F0A.0	fHakka	  ji1; ji2; ri2; i1
U+4F0A.0	fCantonese	ji1
U+4F0A.0	fMandarin	yi1
U+4F0A.0	fEnglish	[1] he; she [2] a Chinese family name [3] he; she; it [4] that [5] what; who
U+4F0B.0	fUTF8	伋
U+4F0B.0	fCNS11643	1-473B
U+4F0B.0	fBig5	A5F9
U+4F0B.0	fGB	3-305D
U+4F0B.0	fR/S	 9.4
U+4F0B.0	fTotalStrokes	 6
U+4F0B.0	fCangjie	ONHE
U+4F0B.0	fFourCorner	27247
U+4F0B.0	fHakka	 kip7; kip8; gip7
U+4F0B.0	fCantonese	kap1; gap1
U+4F0B.0	fMandarin	ji2
U+4F0B.0	fEnglish	[1] deceptive [2] name of Tzu-szu, grandson of Confucius
U+4F0C.0	fUTF8	伌
U+4F0C.0	fCNS11643	4-222A
U+4F0C.0	fGB	E-2153
U+4F0C.0	fR/S	  9.4
U+4F0C.0	fTotalStrokes	 6
U+4F0C.0	fCangjie	OMSU
U+4F0C.0	fFourCorner	21210
U+4F0C.0	fCantonese	aai3; ngaai3 (also)
U+4F0C.0	fMandarin	ai4
U+4F0D.0	fUTF8	伍
U+4F0D.0	fCNS11643	1-4730
U+4F0D.0	fBig5	A5EE
U+4F0D.0	fGB	0-4E69
U+4F0D.0	fR/S	 9.4
U+4F0D.0	fTotalStrokes	 6
U+4F0D.0	fCangjie	OMDM
U+4F0D.0	fFourCorner	21217
U+4F0D.0	fHakka	  ng3
U+4F0D.0	fCantonese	ng5
U+4F0D.0	fMandarin	wu3
U+4F0D.0	fEnglish	[1] five [2] military unit of five soldiers [3] army [4] associate (with a person) [5] a Chinese family name
U+4F0E.0	fUTF8	伎
U+4F0E.0	fCNS11643	2-216A
U+4F0E.0	fBig5	C9AB
U+4F0E.0	fGB	0-3C3F
U+4F0E.0	fR/S	 9.4
U+4F0E.0	fTotalStrokes	 6
U+4F0E.0	fCangjie	OJE
U+4F0E.0	fFourCorner	24247
U+4F0E.0	fHakka	 ki3;  ki1; gi3
U+4F0E.0	fCantonese	gei6; kei4
U+4F0E.0	fMandarin	ji4; qi2
U+4F0E.0	fEnglish	[1] interchangeable with &#x6280; [2] interchangeable with &#x5993;
U+4F0F.0	fUTF8	伏
U+4F0F.0	fCNS11643	1-4733
U+4F0F.0	fBig5	A5F1
U+4F0F.0	fGB	0-377C
U+4F0F.0	fR/S	 9.4
U+4F0F.0	fTotalStrokes	 6
U+4F0F.0	fCangjie	OIK
U+4F0F.0	fFourCorner	23234
U+4F0F.0	fHakka	  fuk8; buk8; puk8; fuk7
U+4F0F.0	fCantonese	fuk6; buk6 (also)
U+4F0F.0	fMandarin	fu2; fou4; fu3; fu4
U+4F0F.0	fEnglish	[1] prostate; yield [2] hide; lie in ambush [3] name of a season [4] a Chinese family name [5] lie in concealment [6] admit defeat [7] submit; cause to submit [8] sit over (eggs) [9] I (courtesy in correspondence)
U+4F10.0	fUTF8	伐
U+4F10.0	fCNS11643	1-4731
U+4F10.0	fBig5	A5EF
U+4F10.0	fGB	0-3725
U+4F10.0	fR/S	 9.4
U+4F10.0	fTotalStrokes	 6
U+4F10.0	fCangjie	OI
U+4F10.0	fFourCorner	23250
U+4F10.0	fHakka	  fat8
U+4F10.0	fCantonese	fat6
U+4F10.0	fMandarin	fa2; fa1; pei4
U+4F10.0	fEnglish	[1] cut; chop (wood) [2] attack; smite [3] boast
U+4F11.0	fUTF8	休
U+4F11.0	fCNS11643	1-4732
U+4F11.0	fBig5	A5F0
U+4F11.0	fGB	0-505D
U+4F11.0	fR/S	 9.4
U+4F11.0	fTotalStrokes	 6
U+4F11.0	fCangjie	OD
U+4F11.0	fFourCorner	24290
U+4F11.0	fHakka	  hiu1; hieu1
U+4F11.0	fCantonese	jau1; heoi2; nau2; jyu3
U+4F11.0	fMandarin	xiu1; xu4
U+4F11.0	fEnglish	[1] rest; rest [2] stop; cease; give up; retire [3] happiness; joy; weal [4] do not (followed by verb) [5] off
U+4F12.0	fUTF8	伒
U+4F12.0	fCNS11643	2-217A
U+4F12.0	fBig5	C9BB
U+4F12.0	fGB	E-2154
U+4F12.0	fR/S	 9.4
U+4F12.0	fTotalStrokes	 6
U+4F12.0	fCangjie	OHML
U+4F12.0	fFourCorner	22221
U+4F12.0	fCantonese	gan3
U+4F12.0	fMandarin	jin4
U+4F13.0	fUTF8	伓
U+4F13.0	fCNS11643	2-2177
U+4F13.0	fBig5	C9B8
U+4F13.0	fGB	5-3152
U+4F13.0	fR/S	 9.4
U+4F13.0	fTotalStrokes	 6
U+4F13.0	fCangjie	OMF
U+4F13.0	fFourCorner	21290
U+4F13.0	fCantonese	pei1
U+4F13.0	fMandarin	bei1
U+4F14.0	fUTF8	伔
U+4F14.0	fCNS11643	2-216E
U+4F14.0	fBig5	C9AF
U+4F14.0	fGB	E-2155
U+4F14.0	fR/S	 9.4
U+4F14.0	fTotalStrokes	 6
U+4F14.0	fCangjie	OBHU
U+4F14.0	fFourCorner	27210
U+4F14.0	fCantonese	dam2; daam6
U+4F14.0	fMandarin	dan3; chen2; shen3
U+4F15.0	fUTF8	伕
U+4F15.0	fCNS11643	1-472F
U+4F15.0	fBig5	A5ED
U+4F15.0	fGB	5-314F
U+4F15.0	fR/S	 9.4
U+4F15.0	fTotalStrokes	 6
U+4F15.0	fCangjie	OQO
U+4F15.0	fFourCorner	25230
U+4F15.0	fHakka	 fen1; fu1
U+4F15.0	fCantonese	fu1
U+4F15.0	fMandarin	fu1; fu2
U+4F15.0	fEnglish	common laborer
U+4F16.0	fUTF8	伖
U+4F16.0	fCNS11643	3-2270
U+4F16.0	fGB	E-2156
U+4F16.0	fR/S	 9.4
U+4F16.0	fTotalStrokes	 6
U+4F16.0	fCangjie	OKE
U+4F16.0	fFourCorner	24240
U+4F16.0	fCantonese	tong2
U+4F16.0	fMandarin	tang3
U+4F17.0	fUTF8	众
U+4F17.0	fCNS11643	3-2277
U+4F17.0	fGB	0-565A
U+4F17.0	fR/S	  9.4
U+4F17.0	fTotalStrokes	 6
U+4F17.0	fCangjie	OOO
U+4F17.0	fFourCorner	80880
U+4F17.0	fHakka	 zhung5
U+4F17.0	fCantonese	zung3
U+4F17.0	fMandarin	zhong4; yin2
U+4F17.0	fEnglish	simplified form of &#x8846;
U+4F18.0	fUTF8	优
U+4F18.0	fCNS11643	2-216B
U+4F18.0	fBig5	C9AC
U+4F18.0	fGB	0-5345
U+4F18.0	fR/S	 9.4
U+4F18.0	fTotalStrokes	 6
U+4F18.0	fCangjie	OIKU
U+4F18.0	fFourCorner	23214
U+4F18.0	fHakka	 ju2
U+4F18.0	fCantonese	jau1
U+4F18.0	fMandarin	you1
U+4F18.0	fEnglish	simplified form of &#x512A;
U+4F19.0	fUTF8	伙
U+4F19.0	fCNS11643	1-472D
U+4F19.0	fBig5	A5EB
U+4F19.0	fGB	0-3B6F
U+4F19.0	fR/S	 9.4
U+4F19.0	fTotalStrokes	 6
U+4F19.0	fCangjie	OF
U+4F19.0	fFourCorner	29280
U+4F19.0	fHakka	  fo3
U+4F19.0	fCantonese	fo2
U+4F19.0	fMandarin	huo3
U+4F19.0	fEnglish	[1] companion; colleague [2] furniture; household goods [3] a squad of ten men
U+4F19.1	fUTF8	伙
U+4F19.1	fCNS11643	1-472D
U+4F19.1	fBig5	A5EB
U+4F19.1	fGB	0-3B6F
U+4F19.1	fR/S	 9.4
U+4F19.1	fTotalStrokes	 6
U+4F19.1	fCangjie	OF
U+4F19.1	fFourCorner	29280
U+4F19.1	fMandarin	huo5
U+4F1A.0	fUTF8	会
U+4F1A.0	fCNS11643	3-2275
U+4F1A.0	fGB	0-3B61
U+4F1A.0	fR/S	 9.4
U+4F1A.0	fTotalStrokes	 6
U+4F1A.0	fCangjie	OMMI
U+4F1A.0	fFourCorner	80730
U+4F1A.0	fHakka	 fui5; kwai5
U+4F1A.0	fCantonese	wui6; wui2 (also); wui5; kui2; kwui2 (also); wui4
U+4F1A.0	fMandarin	hui4; gua1; gui4; guai4
U+4F1A.0	fEnglish	simplified form of &#x6703;
U+4F1A.1	fUTF8	会
U+4F1A.1	fCNS11643	3-2275
U+4F1A.1	fGB	0-3B61
U+4F1A.1	fR/S	 9.4
U+4F1A.1	fTotalStrokes	 6
U+4F1A.1	fCangjie	OMMI
U+4F1A.1	fFourCorner	80730
U+4F1A.1	fMandarin	kuai4
U+4F1B.0	fUTF8	伛
U+4F1B.0	fCNS11643	4-222E
U+4F1B.0	fGB	0-5871
U+4F1B.0	fR/S	 9.4
U+4F1B.0	fTotalStrokes	 6
U+4F1B.0	fCangjie	OSK
U+4F1B.0	fFourCorner	21210
U+4F1B.0	fHakka	 ki1
U+4F1B.0	fCantonese	jyu2
U+4F1B.0	fMandarin	yu3
U+4F1B.0	fEnglish	simplified form of &#x50B4;
U+4F1C.0	fUTF8	伜
U+4F1C.0	fCNS11643	3-2273
U+4F1C.0	fGB	E-2157
U+4F1C.0	fR/S	 9.4
U+4F1C.0	fTotalStrokes	 6
U+4F1C.0	fCangjie	OKNJ
U+4F1C.0	fFourCorner	27240
U+4F1C.0	fCantonese	ceoi3
U+4F1C.0	fMandarin	cui4; zu2
U+4F1C.0	fEnglish	deputy; vice-
U+4F1D.0	fUTF8	伝
U+4F1D.0	fCNS11643	2-2173
U+4F1D.0	fBig5	C9B4
U+4F1D.0	fGB	5-3150
U+4F1D.0	fR/S	 9.4
U+4F1D.0	fTotalStrokes	 6
U+4F1D.0	fCangjie	OMMI
U+4F1D.0	fFourCorner	21231
U+4F1D.0	fCantonese	cyun4; wan4
U+4F1D.0	fMandarin	chuan2; yun2; zhuan4
U+4F1D.0	fEnglish	circulate; move constantly
U+4F1E.0	fUTF8	伞
U+4F1E.0	fGB	0-4921
U+4F1E.0	fR/S	 9.4
U+4F1E.0	fTotalStrokes	 6
U+4F1E.0	fCangjie	OFJ
U+4F1E.0	fHakka	 san5
U+4F1E.0	fCantonese	saan3
U+4F1E.0	fMandarin	san3
U+4F1E.0	fEnglish	simplified form of &#x5098;
U+4F1F.0	fUTF8	伟
U+4F1F.0	fGB	0-4E30
U+4F1F.0	fR/S	 9.4
U+4F1F.0	fTotalStrokes	 6
U+4F1F.0	fCangjie	OQS
U+4F1F.0	fHakka	 vui3
U+4F1F.0	fCantonese	wai5
U+4F1F.0	fMandarin	wei3
U+4F1F.0	fEnglish	simplified form of &#x5049;
U+4F20.0	fUTF8	传
U+4F20.0	fGB	0-342B
U+4F20.0	fR/S	 9.4
U+4F20.0	fTotalStrokes	 6
U+4F20.0	fCangjie	OQNI
U+4F20.0	fHakka	 chon2; chon5
U+4F20.0	fCantonese	cyun4; zyun6; zyun2 (also)
U+4F20.0	fMandarin	chuan2
U+4F20.0	fEnglish	simplified form of &#x50B3;
U+4F20.1	fUTF8	传
U+4F20.1	fGB	0-342B
U+4F20.1	fR/S	 9.4
U+4F20.1	fTotalStrokes	 6
U+4F20.1	fCangjie	OQNI
U+4F20.1	fMandarin	zhuan4
U+4F21.0	fUTF8	伡
U+4F21.0	fGB	8-2E24
U+4F21.0	fR/S	 9.4
U+4F21.0	fTotalStrokes	 6
U+4F21.0	fCangjie	OKQ
U+4F21.0	fCantonese	ce1
U+4F21.0	fMandarin	che1
U+4F22.0	fUTF8	伢
U+4F22.0	fCNS11643	2-2176
U+4F22.0	fBig5	C9B7
U+4F22.0	fGB	0-5873
U+4F22.0	fR/S	 9.4
U+4F22.0	fTotalStrokes	 6
U+4F22.0	fCangjie	OMVH
U+4F22.0	fFourCorner	27240
U+4F22.0	fHakka	 nga2
U+4F22.0	fCantonese	ngaa4; aa4 (also)
U+4F22.0	fMandarin	ya2
U+4F22.0	fEnglish	[1] my [2] I [3] we
U+4F23.0	fUTF8	伣
U+4F23.0	fGB	8-7A2D
U+4F23.0	fR/S	 9.4
U+4F23.0	fTotalStrokes	 6
U+4F23.0	fCangjie	XOBHU
U+4F23.0	fCantonese	jin5
U+4F23.0	fMandarin	xian4
U+4F23.0	fEnglish	like
U+4F24.0	fUTF8	伤
U+4F24.0	fGB	0-494B
U+4F24.0	fR/S	 9.4
U+4F24.0	fTotalStrokes	 6
U+4F24.0	fCangjie	OOKS
U+4F24.0	fHakka	 shong1
U+4F24.0	fCantonese	soeng1
U+4F24.0	fMandarin	shang1
U+4F24.0	fEnglish	simplified form of &#x50B7;
U+4F25.0	fUTF8	伥
U+4F25.0	fGB	0-5876
U+4F25.0	fR/S	 9.5; 9.4
U+4F25.0	fTotalStrokes	 6
U+4F25.0	fCangjie	OPO
U+4F25.0	fHakka	 chong2
U+4F25.0	fCantonese	coeng1; caang1 (also); caang4; zaang1
U+4F25.0	fMandarin	chang1; zheng4
U+4F25.0	fEnglish	simplified form of &#x5000;
U+4F26.0	fUTF8	伦
U+4F26.0	fGB	0-4257
U+4F26.0	fR/S	 9.4
U+4F26.0	fTotalStrokes	 6
U+4F26.0	fCangjie	OOP
U+4F26.0	fHakka	 lun2
U+4F26.0	fCantonese	leon4
U+4F26.0	fMandarin	lun2
U+4F26.0	fEnglish	simplified form of &#x502B;
U+4F27.0	fUTF8	伧
U+4F27.0	fGB	0-5877
U+4F27.0	fR/S	 9.4
U+4F27.0	fTotalStrokes	 6
U+4F27.0	fCangjie	OOSU
U+4F27.0	fHakka	 cong1
U+4F27.0	fCantonese	cong1; caang4
U+4F27.0	fMandarin	cang1; cheng2; cang4
U+4F27.0	fEnglish	simplified form of &#x5096;
U+4F28.0	fUTF8	伨
U+4F28.0	fCNS11643	3-2272
U+4F28.0	fGB	5-315A
U+4F28.0	fR/S	 9.4
U+4F28.0	fTotalStrokes	 6
U+4F28.0	fCangjie	OPIM
U+4F28.0	fFourCorner	27220
U+4F28.0	fCantonese	seon1
U+4F28.0	fMandarin	xun4
U+4F29.0	fUTF8	伩
U+4F29.0	fCNS11643	F-2171
U+4F29.0	fGB	5-315C
U+4F29.0	fR/S	 9.4
U+4F29.0	fTotalStrokes	 6
U+4F29.0	fCangjie	OYK
U+4F29.0	fFourCorner	2024
U+4F29.0	fCantonese	seon3; man1
U+4F29.0	fMandarin	xin4
U+4F29.0	fEnglish	[1] trust, believe; letter [2] small
U+4F2A.0	fUTF8	伪
U+4F2A.0	fGB	0-4E31
U+4F2A.0	fR/S	 9.4
U+4F2A.0	fTotalStrokes	 6
U+4F2A.0	fCangjie	OIKS
U+4F2A.0	fHakka	 ngui3
U+4F2A.0	fCantonese	ngai6; ai6 (also)
U+4F2A.0	fMandarin	wei4; wei3; e2
U+4F2A.0	fEnglish	simplified form of &#x50DE;
U+4F2B.0	fUTF8	伫
U+4F2B.0	fGB	0-5879
U+4F2B.0	fR/S	 9.4
U+4F2B.0	fTotalStrokes	 6
U+4F2B.0	fCangjie	OJM
U+4F2B.0	fHakka	 du3
U+4F2B.0	fCantonese	cyu5
U+4F2B.0	fMandarin	zhu4
U+4F2B.0	fEnglish	simplified form of &#x4F47;
U+4F2C.0	fUTF8	伬
U+4F2C.0	fCNS11643	2-216C
U+4F2C.0	fBig5	C9AD
U+4F2C.0	fGB	E-2158
U+4F2C.0	fR/S	 9.4
U+4F2C.0	fTotalStrokes	 6
U+4F2C.0	fCangjie	OSO
U+4F2C.0	fFourCorner	26237
U+4F2C.0	fCantonese	cek3; ce2
U+4F2C.0	fMandarin	chi3
U+4F2D.0	fUTF8	伭
U+4F2D.0	fCNS11643	2-2328
U+4F2D.0	fBig5	CA66
U+4F2D.0	fGB	3-3071
U+4F2D.0	fR/S	 9.5
U+4F2D.0	fTotalStrokes	 7
U+4F2D.0	fCangjie	OYVI
U+4F2D.0	fFourCorner	20232
U+4F2D.0	fHakka	 hien2; hen2
U+4F2D.0	fCantonese	jin4; jyun4
U+4F2D.0	fMandarin	xuan2
U+4F2D.0	fEnglish	dark; sombre; profound (used by modern Taoists)
U+4F2E.0	fUTF8	伮
U+4F2E.0	fCNS11643	4-2339
U+4F2E.0	fGB	E-2159
U+4F2E.0	fR/S	  9.5
U+4F2E.0	fTotalStrokes	 7
U+4F2E.0	fCangjie	OVE
U+4F2E.0	fFourCorner	27240
U+4F2E.0	fHakka	  nu2
U+4F2E.0	fCantonese	nou5; lou5 (also); nou4
U+4F2E.0	fMandarin	nu2; nu3
U+4F2F.0	fUTF8	伯
U+4F2F.0	fCNS11643	1-4924
U+4F2F.0	fBig5	A742
U+4F2F.0	fGB	0-322E
U+4F2F.0	fR/S	 9.5
U+4F2F.0	fTotalStrokes	 7
U+4F2F.0	fCangjie	OHA
U+4F2F.0	fFourCorner	26200
U+4F2F.0	fHakka	  bak7
U+4F2F.0	fCantonese	baak3; mak6
U+4F2F.0	fMandarin	bo2; bai3; mo4
U+4F2F.0	fEnglish	[1] uncle; father's elder brother [2] first of brothers [3] term of address for husband�s elder brother (the mother following children's address) [4] feudal rank of the nobility (count) [5] term of honor [6] a Chinese family name
U+4F2F.1	fUTF8	伯
U+4F2F.1	fCNS11643	1-4924
U+4F2F.1	fBig5	A742
U+4F2F.1	fGB	0-322E
U+4F2F.1	fR/S	 9.5
U+4F2F.1	fTotalStrokes	 7
U+4F2F.1	fCangjie	OHA
U+4F2F.1	fFourCorner	26200
U+4F2F.1	fMandarin	bo5
U+4F2F.2	fUTF8	伯
U+4F2F.2	fCNS11643	1-4924
U+4F2F.2	fBig5	A742
U+4F2F.2	fGB	0-322E
U+4F2F.2	fR/S	 9.5
U+4F2F.2	fTotalStrokes	 7
U+4F2F.2	fCangjie	OHA
U+4F2F.2	fFourCorner	26200
U+4F2F.2	fCantonese	baa3
U+4F2F.2	fMandarin	ba4
U+4F2F.2	fEnglish	 leader of states
U+4F30.0	fUTF8	估
U+4F30.0	fCNS11643	1-4875
U+4F30.0	fBig5	A6F4
U+4F30.0	fGB	0-3940
U+4F30.0	fR/S	 9.5
U+4F30.0	fTotalStrokes	 7
U+4F30.0	fCangjie	OJR
U+4F30.0	fFourCorner	24260
U+4F30.0	fHakka	  gu3; gu1; gu5
U+4F30.0	fCantonese	gu2; gwu2 (also); gwu3 (also)
U+4F30.0	fMandarin	gu1; gu3
U+4F30.0	fEnglish	 estimate; calculate; evaluate
U+4F30.1	fUTF8	估
U+4F30.1	fCNS11643	1-4875
U+4F30.1	fBig5	A6F4
U+4F30.1	fGB	0-3940
U+4F30.1	fR/S	 9.5
U+4F30.1	fTotalStrokes	 7
U+4F30.1	fCangjie	OJR
U+4F30.1	fFourCorner	24260
U+4F30.1	fCantonese	gu3
U+4F30.1	fMandarin	gu4
U+4F30.1	fEnglish	 sell (used clothing)
U+4F31.0	fUTF8	伱
U+4F31.0	fCNS11643	3-2450
U+4F31.0	fGB	E-215A
U+4F31.0	fR/S	  9.5
U+4F31.0	fTotalStrokes	 7
U+4F31.0	fCangjie	OOF
U+4F31.0	fFourCorner	28290
U+4F31.0	fHakka	  ni3;  ni1; ngi2
U+4F31.0	fCantonese	nei5; lei5 (also)
U+4F31.0	fMandarin	ni3
U+4F31.0	fEnglish	thou ; you
U+4F32.0	fUTF8	伲
U+4F32.0	fCNS11643	3-2448
U+4F32.0	fGB	0-5923
U+4F32.0	fR/S	 9.5
U+4F32.0	fTotalStrokes	 7
U+4F32.0	fCangjie	OSP
U+4F32.0	fFourCorner	27210
U+4F32.0	fHakka	  ni3;  ni1
U+4F32.0	fCantonese	nei4; lei4 (also)
U+4F32.0	fMandarin	ni3; ni4
U+4F32.0	fEnglish	[1] we [2] we [3] I
U+4F33.0	fUTF8	伳
U+4F33.0	fCNS11643	2-2329
U+4F33.0	fBig5	CA67
U+4F33.0	fGB	5-3162
U+4F33.0	fR/S	 9.5
U+4F33.0	fTotalStrokes	 7
U+4F33.0	fCangjie	OPT
U+4F33.0	fFourCorner	24217
U+4F33.0	fHakka	  siet7
U+4F33.0	fCantonese	sit3
U+4F33.0	fMandarin	xie4
U+4F33.0	fEnglish	extravagant
U+4F34.0	fUTF8	伴
U+4F34.0	fCNS11643	1-4872
U+4F34.0	fBig5	A6F1
U+4F34.0	fGB	0-3069
U+4F34.0	fR/S	 9.5
U+4F34.0	fTotalStrokes	 7
U+4F34.0	fCangjie	OFQ
U+4F34.0	fFourCorner	29250
U+4F34.0	fHakka	 pan1; pan3; pan5; ban5; pan6
U+4F34.0	fCantonese	bun6; pun5 (also); pun2 (also); pun3
U+4F34.0	fMandarin	ban4; pan4; pai4
U+4F34.0	fEnglish	[1] companion; comrade; partner [2] accompany
U+4F35.0	fUTF8	伵
U+4F35.0	fCNS11643	3-244F
U+4F35.0	fGB	5-316A
U+4F35.0	fR/S	 9.5
U+4F35.0	fTotalStrokes	 7
U+4F35.0	fCangjie	OWC
U+4F35.0	fFourCorner	26200
U+4F35.0	fCantonese	gwik1
U+4F35.0	fMandarin	xu4
U+4F36.0	fUTF8	伶
U+4F36.0	fCNS11643	1-4926
U+4F36.0	fBig5	A744
U+4F36.0	fGB	0-4166
U+4F36.0	fR/S	 9.5
U+4F36.0	fTotalStrokes	 7
U+4F36.0	fCangjie	OOII
U+4F36.0	fFourCorner	28237
U+4F36.0	fHakka	  lang2;  lin2; lang3
U+4F36.0	fCantonese	ling4; ling1 (also)
U+4F36.0	fMandarin	ling2; ling4
U+4F36.0	fEnglish	[1] drama performer; theatrical performer; actor; actress [2] solitary; lonely
U+4F37.0	fUTF8	伷
U+4F37.0	fCNS11643	3-2446
U+4F37.0	fGB	5-3168
U+4F37.0	fR/S	  9.5
U+4F37.0	fTotalStrokes	 7
U+4F37.0	fCangjie	OLW
U+4F37.0	fFourCorner	25260
U+4F37.0	fHakka	 ciu5
U+4F37.0	fCantonese	zau6
U+4F37.0	fMandarin	zhou4
U+4F37.0	fEnglish	descendent
U+4F38.0	fUTF8	伸
U+4F38.0	fCNS11643	1-487A
U+4F38.0	fBig5	A6F9
U+4F38.0	fGB	0-496C
U+4F38.0	fR/S	 9.5
U+4F38.0	fTotalStrokes	 7
U+4F38.0	fCangjie	OLWL
U+4F38.0	fFourCorner	25206
U+4F38.0	fHakka	  shin1;  sin1; cun1; chun1; siin1
U+4F38.0	fCantonese	san1
U+4F38.0	fMandarin	shen1; chen1
U+4F38.0	fEnglish	[1] stretch; extend; straighten [2] report
U+4F39.0	fUTF8	伹
U+4F39.0	fCNS11643	3-2445
U+4F39.0	fGB	5-3167
U+4F39.0	fR/S	 9.5
U+4F39.0	fTotalStrokes	 7
U+4F39.0	fCangjie	OBM
U+4F39.0	fFourCorner	27210
U+4F39.0	fCantonese	ceoi1
U+4F39.0	fMandarin	qu1
U+4F39.0	fEnglish	unskillful; clumsy; clumsy person
U+4F3A.0	fUTF8	伺
U+4F3A.0	fCNS11643	1-4879
U+4F3A.0	fBig5	A6F8
U+4F3A.0	fGB	0-4B45
U+4F3A.0	fR/S	 9.5
U+4F3A.0	fTotalStrokes	 7
U+4F3A.0	fCangjie	OSMR
U+4F3A.0	fFourCorner	27220
U+4F3A.0	fHakka	 sii6; sii5; su5; su3; se5
U+4F3A.0	fCantonese	zi6; si6
U+4F3A.0	fMandarin	si4
U+4F3A.0	fEnglish	[1] spy; reconnoiter; watch; investigate [2] wait for chance
U+4F3A.1	fUTF8	伺
U+4F3A.1	fCNS11643	1-4879
U+4F3A.1	fBig5	A6F8
U+4F3A.1	fGB	0-4B45
U+4F3A.1	fR/S	 9.5
U+4F3A.1	fTotalStrokes	 7
U+4F3A.1	fCangjie	OSMR
U+4F3A.1	fFourCorner	27220
U+4F3A.1	fMandarin	ci4
U+4F3A.1	fEnglish	 serve; wait upon (a person)
U+4F3B.0	fUTF8	伻
U+4F3B.0	fCNS11643	2-227B
U+4F3B.0	fBig5	CA5B
U+4F3B.0	fGB	5-3164
U+4F3B.0	fR/S	 9.5
U+4F3B.0	fTotalStrokes	 7
U+4F3B.0	fCangjie	OMFJ
U+4F3B.0	fFourCorner	21249
U+4F3B.0	fHakka	  pin2
U+4F3B.0	fCantonese	ping1; paang1; ping4
U+4F3B.0	fMandarin	beng1; rang1
U+4F3B.0	fEnglish	[1] send emissary [2] messenger; emissary [3] servant
U+4F3C.0	fUTF8	似
U+4F3C.0	fCNS11643	1-487D
U+4F3C.0	fBig5	A6FC
U+4F3C.0	fGB	0-4B46
U+4F3C.0	fR/S	 9.5
U+4F3C.0	fTotalStrokes	 7
U+4F3C.0	fCangjie	OVIO
U+4F3C.0	fFourCorner	28200
U+4F3C.0	fHakka	 sii5; su5; sii6; ci5; se5
U+4F3C.0	fCantonese	ci5
U+4F3C.0	fMandarin	shi4
U+4F3C.0	fEnglish	[1] resemble; to seem; similar to [2] like; as if
U+4F3C.1	fUTF8	似
U+4F3C.1	fCNS11643	1-487D
U+4F3C.1	fBig5	A6FC
U+4F3C.1	fGB	0-4B46
U+4F3C.1	fR/S	 9.5
U+4F3C.1	fTotalStrokes	 7
U+4F3C.1	fCangjie	OVIO
U+4F3C.1	fFourCorner	28200
U+4F3C.1	fMandarin	si4
U+4F3D.0	fUTF8	伽
U+4F3D.0	fCNS11643	1-4878
U+4F3D.0	fBig5	A6F7
U+4F3D.0	fGB	0-5924
U+4F3D.0	fR/S	 9.5
U+4F3D.0	fTotalStrokes	 7
U+4F3D.0	fCangjie	OKSR
U+4F3D.0	fFourCorner	26200
U+4F3D.0	fHakka	 ga1
U+4F3D.0	fCantonese	gaa1
U+4F3D.0	fMandarin	jia1; ga1
U+4F3D.0	fEnglish	transcription of Greek gamma or names
U+4F3D.1	fUTF8	伽
U+4F3D.1	fCNS11643	1-4878
U+4F3D.1	fBig5	A6F7
U+4F3D.1	fGB	0-5924
U+4F3D.1	fR/S	 9.5
U+4F3D.1	fTotalStrokes	 7
U+4F3D.1	fCangjie	OKSR
U+4F3D.1	fFourCorner	26200
U+4F3D.1	fCantonese	ke4
U+4F3D.1	fMandarin	qie2
U+4F3D.1	fEnglish	[1] transcription of Sanskrit gha in Buddhist texts [2] a Buddhist temple [3] a Buddhist deity (Sangharama or Sanghagara)
U+4F3E.0	fUTF8	伾
U+4F3E.0	fCNS11643	2-2322
U+4F3E.0	fBig5	CA60
U+4F3E.0	fGB	3-306C
U+4F3E.0	fR/S	 9.5
U+4F3E.0	fTotalStrokes	 7
U+4F3E.0	fCangjie	OMFM
U+4F3E.0	fFourCorner	21219
U+4F3E.0	fHakka	 pi1
U+4F3E.0	fCantonese	pei1
U+4F3E.0	fMandarin	pi1; bi4; pei1
U+4F3E.0	fEnglish	mighty
U+4F3F.0	fUTF8	伿
U+4F3F.0	fCNS11643	2-232A
U+4F3F.0	fBig5	CA68
U+4F3F.0	fGB	5-3169
U+4F3F.0	fR/S	 9.5
U+4F3F.0	fTotalStrokes	 7
U+4F3F.0	fCangjie	ORC
U+4F3F.0	fFourCorner	26280
U+4F3F.0	fCantonese	ji6; zi3
U+4F3F.0	fMandarin	yi4; yang1; yang3; zhi4
U+4F40.0	fUTF8	佀
U+4F40.0	fCNS11643	3-244E
U+4F40.0	fGB	E-215B
U+4F40.0	fR/S	 9.5
U+4F40.0	fTotalStrokes	 7
U+4F40.0	fCangjie	ORLR
U+4F40.0	fFourCorner	27270
U+4F40.0	fHakka	 se5
U+4F40.0	fCantonese	ci5
U+4F40.0	fMandarin	si4; shi4
U+4F40.0	fEnglish	resemble
U+4F41.0	fUTF8	佁
U+4F41.0	fCNS11643	2-2326
U+4F41.0	fBig5	CA64
U+4F41.0	fGB	3-3078
U+4F41.0	fR/S	 9.5
U+4F41.0	fTotalStrokes	 7
U+4F41.0	fCangjie	OIR
U+4F41.0	fFourCorner	23260
U+4F41.0	fCantonese	ji5; ci3; ji4
U+4F41.0	fMandarin	ai3; si4; yi3; tai3; chi4
U+4F42.0	fUTF8	佂
U+4F42.0	fCNS11643	3-2449
U+4F42.0	fGB	5-3160
U+4F42.0	fR/S	 9.5
U+4F42.0	fTotalStrokes	 7
U+4F42.0	fCangjie	OMYM
U+4F42.0	fFourCorner	21210
U+4F42.0	fHakka	 zhin1
U+4F42.0	fCantonese	zing1
U+4F42.0	fMandarin	zheng1; zheng4
U+4F43.0	fUTF8	佃
U+4F43.0	fCNS11643	1-487B
U+4F43.0	fBig5	A6FA
U+4F43.0	fGB	0-3568
U+4F43.0	fR/S	 9.5
U+4F43.0	fTotalStrokes	 7
U+4F43.0	fCangjie	OW
U+4F43.0	fFourCorner	26200
U+4F43.0	fHakka	 tien5; ten3; tien6
U+4F43.0	fCantonese	din6
U+4F43.0	fMandarin	dian4; tian2; tian1
U+4F43.0	fEnglish	[1] tenant farmer [2] tenant a farm
U+4F43.1	fUTF8	佃
U+4F43.1	fCNS11643	1-487B
U+4F43.1	fBig5	A6FA
U+4F43.1	fGB	0-3568
U+4F43.1	fR/S	 9.5
U+4F43.1	fTotalStrokes	 7
U+4F43.1	fCangjie	OW
U+4F43.1	fFourCorner	26200
U+4F43.1	fCantonese	tin4
U+4F43.1	fEnglish	 hunt
U+4F44.0	fUTF8	佄
U+4F44.0	fCNS11643	3-244B
U+4F44.0	fGB	5-3161
U+4F44.0	fR/S	  9.5
U+4F44.0	fTotalStrokes	 7
U+4F44.0	fCangjie	OTM
U+4F44.0	fFourCorner	24270
U+4F44.0	fCantonese	ham4
U+4F44.0	fMandarin	han2
U+4F45.0	fUTF8	佅
U+4F45.0	fCNS11643	3-2443
U+4F45.0	fGB	3-3069
U+4F45.0	fR/S	 9.5
U+4F45.0	fTotalStrokes	 7
U+4F45.0	fCangjie	OJD
U+4F45.0	fFourCorner	25290
U+4F45.0	fCantonese	mui6; maai6 (also)
U+4F45.0	fMandarin	mai4
U+4F46.0	fUTF8	但
U+4F46.0	fCNS11643	1-487E
U+4F46.0	fBig5	A6FD
U+4F46.0	fGB	0-352B
U+4F46.0	fR/S	 9.5
U+4F46.0	fTotalStrokes	 7
U+4F46.0	fCangjie	OAM
U+4F46.0	fFourCorner	26210
U+4F46.0	fHakka	  tan3;  dan5; tan5; tan6
U+4F46.0	fCantonese	daan6
U+4F46.0	fMandarin	dan4
U+4F46.0	fEnglish	but; however; yet
U+4F47.0	fUTF8	佇
U+4F47.0	fCNS11643	1-486F
U+4F47.0	fBig5	A6EE
U+4F47.0	fGB	1-5879
U+4F47.0	fR/S	 9.5
U+4F47.0	fTotalStrokes	 7
U+4F47.0	fCangjie	OJMN
U+4F47.0	fFourCorner	23221
U+4F47.0	fHakka	 du3; du5
U+4F47.0	fCantonese	cyu5; ning4; ling4 (also)
U+4F47.0	fMandarin	zhu4
U+4F47.0	fEnglish	[1] stand (for a long time); wait [2] hope; expect; look towards
U+4F48.0	fUTF8	佈
U+4F48.0	fCNS11643	1-4929
U+4F48.0	fBig5	A747
U+4F48.0	fGB	E-215C
U+4F48.0	fR/S	 9.5
U+4F48.0	fTotalStrokes	 7
U+4F48.0	fCangjie	OKLB
U+4F48.0	fFourCorner	24227
U+4F48.0	fHakka	  bu5
U+4F48.0	fCantonese	bou3
U+4F48.0	fMandarin	bu4
U+4F48.0	fEnglish	[1] announce; publicise; declare [2] arrange
U+4F49.0	fUTF8	佉
U+4F49.0	fCNS11643	2-227D
U+4F49.0	fBig5	CA5D
U+4F49.0	fGB	3-306B
U+4F49.0	fR/S	 9.5
U+4F49.0	fTotalStrokes	 7
U+4F49.0	fCangjie	OGI
U+4F49.0	fFourCorner	24231
U+4F49.0	fHakka	  ki1
U+4F49.0	fCantonese	keoi1
U+4F49.0	fMandarin	qu1; qia1
U+4F49.0	fEnglish	[1] transliteration of Sanskrit "kh" [2] name of a divine being [3] used in combination with &#x5E36; for waist belt
U+4F4A.0	fUTF8	佊
U+4F4A.0	fCNS11643	3-2444
U+4F4A.0	fGB	3-3076
U+4F4A.0	fR/S	  9.5
U+4F4A.0	fTotalStrokes	 7
U+4F4A.0	fCangjie	ODHE
U+4F4A.0	fFourCorner	24240
U+4F4A.0	fCantonese	bei2
U+4F4A.0	fMandarin	bi3
U+4F4B.0	fUTF8	佋
U+4F4B.0	fCNS11643	3-244C
U+4F4B.0	fGB	5-316C
U+4F4B.0	fR/S	 9.5
U+4F4B.0	fTotalStrokes	 7
U+4F4B.0	fCangjie	OSHR
U+4F4B.0	fFourCorner	27260
U+4F4B.0	fHakka	 shau5; sau5
U+4F4B.0	fCantonese	siu6
U+4F4B.0	fMandarin	shao1; shao4; zhao1
U+4F4B.1	fUTF8	佋
U+4F4B.1	fCNS11643	3-244C
U+4F4B.1	fGB	5-316C
U+4F4B.1	fR/S	 9.5
U+4F4B.1	fTotalStrokes	 7
U+4F4B.1	fCangjie	OSHR
U+4F4B.1	fFourCorner	27260
U+4F4C.0	fUTF8	佌
U+4F4C.0	fCNS11643	2-253D
U+4F4C.0	fBig5	CBBD
U+4F4C.0	fGB	3-307D
U+4F4C.0	fR/S	 9.5
U+4F4C.0	fTotalStrokes	 7
U+4F4C.0	fCangjie	OYMP
U+4F4C.0	fFourCorner	21210
U+4F4C.0	fHakka	 cih3; ze3
U+4F4C.0	fCantonese	ci2
U+4F4C.0	fMandarin	ci3
U+4F4C.0	fEnglish	small
U+4F4D.0	fUTF8	位
U+4F4D.0	fCNS11643	1-486D
U+4F4D.0	fBig5	A6EC
U+4F4D.0	fGB	0-4E3B
U+4F4D.0	fR/S	 9.5
U+4F4D.0	fTotalStrokes	 7
U+4F4D.0	fCangjie	OYT
U+4F4D.0	fFourCorner	20218
U+4F4D.0	fHakka	 wui5; wui3; wui6; wi5; vui5
U+4F4D.0	fCantonese	wai6; wai2 (also)
U+4F4D.0	fMandarin	wei4
U+4F4D.0	fEnglish	[1] position; rank; status [2] location; seat [3] numerical adjunct [4] seat of authority [5] person [6] take its place
U+4F4E.0	fUTF8	低
U+4F4E.0	fCNS11643	1-4925
U+4F4E.0	fBig5	A743
U+4F4E.0	fGB	0-354D
U+4F4E.0	fR/S	 9.5
U+4F4E.0	fTotalStrokes	 7
U+4F4E.0	fCangjie	OHPM
U+4F4E.0	fFourCorner	22240
U+4F4E.0	fHakka	  dai1; di1; te1
U+4F4E.0	fCantonese	dai1
U+4F4E.0	fMandarin	di1
U+4F4E.0	fEnglish	[1] low [2] lower; bend; bow
U+4F4F.0	fUTF8	住
U+4F4F.0	fCNS11643	1-486E
U+4F4F.0	fBig5	A6ED
U+4F4F.0	fGB	0-5721
U+4F4F.0	fR/S	 9.5
U+4F4F.0	fTotalStrokes	 7
U+4F4F.0	fCangjie	OYG
U+4F4F.0	fFourCorner	20214
U+4F4F.0	fHakka	 chu5; cu5; cu3; dai5; chu6
U+4F4F.0	fCantonese	zyu6; zyu2 (also)
U+4F4F.0	fMandarin	zhu4
U+4F4F.0	fEnglish	[1] dwell; inhabit; live (at); reside [2] stop (action) [3] lodge; stop for the night [4] remain [5] used after a verb to complement its meaning
U+4F50.0	fUTF8	佐
U+4F50.0	fCNS11643	1-4876
U+4F50.0	fBig5	A6F5
U+4F50.0	fGB	0-5774
U+4F50.0	fR/S	 9.5
U+4F50.0	fTotalStrokes	 7
U+4F50.0	fCangjie	OKM
U+4F50.0	fFourCorner	24211
U+4F50.0	fHakka	  zo3
U+4F50.0	fCantonese	zo3
U+4F50.0	fMandarin	zuo3
U+4F50.0	fEnglish	[1] assist; aid; second [2] assistant
U+4F51.0	fUTF8	佑
U+4F51.0	fCNS11643	1-4877
U+4F51.0	fBig5	A6F6
U+4F51.0	fGB	0-5353
U+4F51.0	fR/S	 9.5
U+4F51.0	fTotalStrokes	 7
U+4F51.0	fCangjie	OKR
U+4F51.0	fFourCorner	24260
U+4F51.0	fHakka	 ju5; jeu3; riu6; jiu6; ju3; jiu3
U+4F51.0	fCantonese	jau6
U+4F51.0	fMandarin	you4; you3
U+4F51.0	fEnglish	 help; protect; aid; bless
U+4F52.0	fUTF8	佒
U+4F52.0	fCNS11643	2-2324
U+4F52.0	fBig5	CA62
U+4F52.0	fGB	3-306E
U+4F52.0	fR/S	 9.5
U+4F52.0	fTotalStrokes	 7
U+4F52.0	fCangjie	OLBK
U+4F52.0	fFourCorner	25230
U+4F52.0	fHakka	 jong1
U+4F52.0	fCantonese	joeng5; ngong5 (also); ong5 (also); joeng2
U+4F52.0	fMandarin	yang1; yang3
U+4F53.0	fUTF8	体
U+4F53.0	fCNS11643	2-227E
U+4F53.0	fBig5	CA5E
U+4F53.0	fGB	0-4C65
U+4F53.0	fR/S	 9.5
U+4F53.0	fTotalStrokes	 7
U+4F53.0	fCangjie	ODM
U+4F53.0	fFourCorner	25230
U+4F53.0	fHakka	  ti3; pun3
U+4F53.0	fCantonese	ban6
U+4F53.0	fMandarin	ti3; ben4; ti1
U+4F53.0	fEnglish	simplified form of &#x9AD4;
U+4F53.1	fUTF8	体
U+4F53.1	fCNS11643	2-227E
U+4F53.1	fBig5	CA5E
U+4F53.1	fGB	0-4C65
U+4F53.1	fR/S	 9.5
U+4F53.1	fTotalStrokes	 7
U+4F53.1	fCangjie	ODM
U+4F53.1	fFourCorner	25230
U+4F53.1	fCantonese	tai2
U+4F54.0	fUTF8	佔
U+4F54.0	fCNS11643	1-487C
U+4F54.0	fBig5	A6FB
U+4F54.0	fGB	E-215D
U+4F54.0	fR/S	 9.5
U+4F54.0	fTotalStrokes	 7
U+4F54.0	fCangjie	OYR
U+4F54.0	fFourCorner	21260
U+4F54.0	fHakka	  zham5;  zam5; ziam5
U+4F54.0	fCantonese	zim3; cim1; zim1
U+4F54.0	fMandarin	zhan4
U+4F54.0	fEnglish	 occupy by force; usurp possess
U+4F55.0	fUTF8	何
U+4F55.0	fCNS11643	1-4874
U+4F55.0	fBig5	A6F3
U+4F55.0	fGB	0-3A4E
U+4F55.0	fR/S	 9.5
U+4F55.0	fTotalStrokes	 7
U+4F55.0	fCangjie	OMNR
U+4F55.0	fFourCorner	21220
U+4F55.0	fHakka	  ho2; ho3
U+4F55.0	fCantonese	ho4; ho6; ho2 (also)
U+4F55.0	fMandarin	he2; he4
U+4F55.0	fEnglish	[1] what; why; where; which; how [2] how (in exclamation) [3] equivalent to a negative [4] a Chinese family name
U+4F56.0	fUTF8	佖
U+4F56.0	fCNS11643	2-227A
U+4F56.0	fBig5	CA5A
U+4F56.0	fGB	3-3075
U+4F56.0	fR/S	 9.5
U+4F56.0	fTotalStrokes	 7
U+4F56.0	fCangjie	OPH
U+4F56.0	fFourCorner	23290
U+4F56.0	fHakka	  bit7
U+4F56.0	fCantonese	bit1
U+4F56.0	fMandarin	bi4
U+4F56.0	fEnglish	 be rude
U+4F57.0	fUTF8	佗
U+4F57.0	fCNS11643	1-4870
U+4F57.0	fBig5	A6EF
U+4F57.0	fGB	0-5922
U+4F57.0	fR/S	 9.5
U+4F57.0	fTotalStrokes	 7
U+4F57.0	fCangjie	OJP
U+4F57.0	fFourCorner	23211
U+4F57.0	fHakka	 to2; ta1
U+4F57.0	fCantonese	to4; taa1; taa4
U+4F57.0	fMandarin	tuo2; tuo4; duo3; yi2
U+4F57.0	fEnglish	 load
U+4F57.1	fUTF8	佗
U+4F57.1	fCNS11643	1-4870
U+4F57.1	fBig5	A6EF
U+4F57.1	fGB	0-5922
U+4F57.1	fR/S	 9.5
U+4F57.1	fTotalStrokes	 7
U+4F57.1	fCangjie	OJP
U+4F57.1	fFourCorner	23211
U+4F57.1	fCantonese	to1
U+4F57.1	fMandarin	tuo1
U+4F57.1	fEnglish	[1] he [2] a Chinese family name
U+4F58.0	fUTF8	佘
U+4F58.0	fCNS11643	2-2327
U+4F58.0	fBig5	CA65
U+4F58.0	fGB	0-595C
U+4F58.0	fR/S	 9.5
U+4F58.0	fTotalStrokes	 7
U+4F58.0	fCangjie	OMMF
U+4F58.0	fFourCorner	80901
U+4F58.0	fHakka	 sa2;  sia2;  sha2; sia3
U+4F58.0	fCantonese	se4
U+4F58.0	fMandarin	she2
U+4F58.0	fEnglish	a Chinese family name
U+4F59.0	fUTF8	余
U+4F59.0	fCNS11643	1-4927
U+4F59.0	fBig5	A745
U+4F59.0	fGB	0-5360
U+4F59.0	fR/S	 9.5
U+4F59.0	fTotalStrokes	 7
U+4F59.0	fCangjie	OMD
U+4F59.0	fFourCorner	80904
U+4F59.0	fHakka	  ji2; ji3; ri2; i2
U+4F59.0	fCantonese	jyu4; ceoi4
U+4F59.0	fMandarin	yu2; ye2; xu2
U+4F59.0	fEnglish	[1] I, me (in formal speech) [2] a Chinese family name [3] simplified form of &#x9918;
U+4F5A.0	fUTF8	佚
U+4F5A.0	fCNS11643	1-492A
U+4F5A.0	fBig5	A748
U+4F5A.0	fGB	0-587D
U+4F5A.0	fR/S	 9.5
U+4F5A.0	fTotalStrokes	 7
U+4F5A.0	fCangjie	OHQO
U+4F5A.0	fFourCorner	25230
U+4F5A.0	fHakka	  jit8; rit8
U+4F5A.0	fCantonese	jat6
U+4F5A.0	fMandarin	yi4
U+4F5A.0	fEnglish	idleness; comfort
U+4F5A.1	fUTF8	佚
U+4F5A.1	fCNS11643	1-492A
U+4F5A.1	fBig5	A748
U+4F5A.1	fGB	0-587D
U+4F5A.1	fR/S	 9.5
U+4F5A.1	fTotalStrokes	 7
U+4F5A.1	fCangjie	OHQO
U+4F5A.1	fFourCorner	25230
U+4F5A.1	fCantonese	dit6
U+4F5A.1	fMandarin	die2
U+4F5B.0	fUTF8	佛
U+4F5B.0	fCNS11643	1-4873
U+4F5B.0	fBig5	A6F2
U+4F5B.0	fGB	0-3770
U+4F5B.0	fR/S	 9.5
U+4F5B.0	fTotalStrokes	 7
U+4F5B.0	fCangjie	OLLN
U+4F5B.0	fFourCorner	25227
U+4F5B.0	fHakka	  fut8
U+4F5B.0	fCantonese	fat6
U+4F5B.0	fMandarin	bo2; fu2
U+4F5B.0	fEnglish	[1] Buddha [2] of Buddhism
U+4F5B.1	fUTF8	佛
U+4F5B.1	fCNS11643	1-4873
U+4F5B.1	fBig5	A6F2
U+4F5B.1	fGB	0-3770
U+4F5B.1	fR/S	 9.5
U+4F5B.1	fTotalStrokes	 7
U+4F5B.1	fCangjie	OLLN
U+4F5B.1	fFourCorner	25227
U+4F5B.1	fCantonese	bat6
U+4F5B.1	fMandarin	bi4
U+4F5B.2	fUTF8	佛
U+4F5B.2	fCNS11643	1-4873
U+4F5B.2	fBig5	A6F2
U+4F5B.2	fGB	0-3770
U+4F5B.2	fR/S	 9.5
U+4F5B.2	fTotalStrokes	 7
U+4F5B.2	fCangjie	OLLN
U+4F5B.2	fFourCorner	25227
U+4F5B.2	fCantonese	fat1
U+4F5B.2	fMandarin	fo2
U+4F5C.0	fUTF8	作
U+4F5C.0	fCNS11643	1-4922
U+4F5C.0	fBig5	A740
U+4F5C.0	fGB	0-5777
U+4F5C.0	fR/S	 9.5
U+4F5C.0	fTotalStrokes	 7
U+4F5C.0	fCangjie	OOS
U+4F5C.0	fFourCorner	28211
U+4F5C.0	fHakka	  zok7; zo5
U+4F5C.0	fCantonese	zok3; zok2 (also)
U+4F5C.0	fMandarin	zuo4; zu3
U+4F5C.0	fEnglish	[1] do; make [2] works (of a writer, etc.) [3] rise; stand up [4] pretend; affect [5] regard ... as; take ... for [6] compose; write [7] profession
U+4F5C.1	fUTF8	作
U+4F5C.1	fCNS11643	1-4922
U+4F5C.1	fBig5	A740
U+4F5C.1	fGB	0-5777
U+4F5C.1	fR/S	 9.5
U+4F5C.1	fTotalStrokes	 7
U+4F5C.1	fCangjie	OOS
U+4F5C.1	fFourCorner	28211
U+4F5C.1	fMandarin	zuo1
U+4F5C.1	fEnglish	[1] small workshop [2] a Chinese gesture in greeting, with the greeter holding his hands together in an up-and-down motion [3] get trapped by oneself
U+4F5C.2	fUTF8	作
U+4F5C.2	fCNS11643	1-4922
U+4F5C.2	fBig5	A740
U+4F5C.2	fGB	0-5777
U+4F5C.2	fR/S	 9.5
U+4F5C.2	fTotalStrokes	 7
U+4F5C.2	fCangjie	OOS
U+4F5C.2	fFourCorner	28211
U+4F5C.2	fMandarin	zuo2
U+4F5C.2	fEnglish	[1] ingredients of a recipe; dressing; seasoning; materials [2] abuse; waste; ill-use; trample; treat harshly [3] allowable [4] be in good spirits [5] in vogue [6] hold in high regard [7] perhaps; likely
U+4F5D.0	fUTF8	佝
U+4F5D.0	fCNS11643	1-4928
U+4F5D.0	fBig5	A746
U+4F5D.0	fGB	0-587E
U+4F5D.0	fR/S	 9.5
U+4F5D.0	fTotalStrokes	 7
U+4F5D.0	fCangjie	OPR
U+4F5D.0	fFourCorner	27220
U+4F5D.0	fHakka	 kiu5
U+4F5D.0	fCantonese	keoi1; gau1 (also); kau3; keoi1kau3; kau1
U+4F5D.0	fMandarin	kou4; gou1; hou4; ju1
U+4F5D.0	fEnglish	rickets
U+4F5E.0	fUTF8	佞
U+4F5E.0	fCNS11643	1-4871
U+4F5E.0	fBig5	A6F0
U+4F5E.0	fGB	0-587A
U+4F5E.0	fR/S	 9.5
U+4F5E.0	fTotalStrokes	 7
U+4F5E.0	fCangjie	OMMV
U+4F5E.0	fFourCorner	21244
U+4F5E.0	fHakka	  nin5; nin6; len2; lin5
U+4F5E.0	fCantonese	ning6; ling6 (also)
U+4F5E.0	fMandarin	ning4
U+4F5E.0	fEnglish	[1] eloquent; persuasive; gifted with a glib tongue [2] obsequious; fawning [3] believe (superstition) [4] flatter [5] one given to flattery
U+4F5F.0	fUTF8	佟
U+4F5F.0	fCNS11643	2-2325
U+4F5F.0	fBig5	CA63
U+4F5F.0	fGB	0-5921
U+4F5F.0	fR/S	 9.5
U+4F5F.0	fTotalStrokes	 7
U+4F5F.0	fCangjie	OHEY
U+4F5F.0	fFourCorner	27233
U+4F5F.0	fHakka	  tung1; tung2
U+4F5F.0	fCantonese	tung4
U+4F5F.0	fMandarin	tong2
U+4F5F.0	fEnglish	a Chinese family name
U+4F60.0	fUTF8	你
U+4F60.0	fCNS11643	1-4923
U+4F60.0	fBig5	A741
U+4F60.0	fGB	0-4463
U+4F60.0	fR/S	 9.5
U+4F60.0	fTotalStrokes	 7
U+4F60.0	fCangjie	ONF
U+4F60.0	fFourCorner	27292
U+4F60.0	fHakka	  ni3;  ni1;  ngi2; li1; li3; ngia1; ng2
U+4F60.0	fCantonese	nei5; lei5 (also)
U+4F60.0	fMandarin	ni3
U+4F60.0	fEnglish	you
U+4F61.0	fUTF8	佡
U+4F61.0	fCNS11643	2-232B
U+4F61.0	fBig5	CA69
U+4F61.0	fGB	3-306F
U+4F61.0	fR/S	 9.5
U+4F61.0	fTotalStrokes	 7
U+4F61.0	fCangjie	OOU
U+4F61.0	fFourCorner	28270
U+4F61.0	fCantonese	hin1
U+4F61.0	fMandarin	xuan1; san3
U+4F61.0	fEnglish	make effort; endeavor
U+4F62.0	fUTF8	佢
U+4F62.0	fCNS11643	2-227C
U+4F62.0	fBig5	CA5C
U+4F62.0	fGB	3-305E
U+4F62.0	fR/S	 9.5
U+4F62.0	fTotalStrokes	 7
U+4F62.0	fCangjie	OSS
U+4F62.0	fFourCorner	21217
U+4F62.0	fHakka	  gi2;  ki2; ki3; gia1
U+4F62.0	fCantonese	keoi5; heoi5 (also)
U+4F62.0	fMandarin	qu2; ju4; ju3
U+4F62.0	fEnglish	he (Cantonese, Hakka)
U+4F63.0	fUTF8	佣
U+4F63.0	fCNS11643	1-4921
U+4F63.0	fBig5	A6FE
U+4F63.0	fGB	0-5336
U+4F63.0	fR/S	 9.5
U+4F63.0	fTotalStrokes	 7
U+4F63.0	fCangjie	OBQ
U+4F63.0	fFourCorner	27220
U+4F63.0	fHakka	 rung6; jung5; jung3; jung2
U+4F63.0	fCantonese	jung2; jung4; jung6
U+4F63.0	fMandarin	yong1; yong4; yong2; chong1; zhong3
U+4F63.0	fEnglish	simplified form of &#x50AD;
U+4F64.0	fUTF8	佤
U+4F64.0	fCNS11643	2-2321
U+4F64.0	fBig5	CA5F
U+4F64.0	fGB	0-5874
U+4F64.0	fR/S	 9.4; 9.5
U+4F64.0	fTotalStrokes	 6
U+4F64.0	fCangjie	OMVN
U+4F64.0	fFourCorner	21217
U+4F64.0	fCantonese	ngaa5; aa5 (also)
U+4F64.0	fMandarin	wa3
U+4F65.0	fUTF8	佥
U+4F65.0	fGB	0-595D
U+4F65.0	fR/S	 9.5
U+4F65.0	fTotalStrokes	 7
U+4F65.0	fCangjie	OMFM
U+4F65.0	fHakka	 ciam1
U+4F65.0	fCantonese	cim1
U+4F65.0	fMandarin	qian1
U+4F65.0	fEnglish	simplified form of &#x50C9;
U+4F66.0	fUTF8	佦
U+4F66.0	fCNS11643	3-6448
U+4F66.0	fGB	E-215E
U+4F66.0	fR/S	 9.5
U+4F66.0	fTotalStrokes	 7
U+4F66.0	fCangjie	XOMR
U+4F66.0	fFourCorner	2126
U+4F66.0	fCantonese	jau6
U+4F67.0	fUTF8	佧
U+4F67.0	fCNS11643	2-2323
U+4F67.0	fBig5	CA61
U+4F67.0	fGB	0-587B
U+4F67.0	fR/S	 9.5
U+4F67.0	fTotalStrokes	 7
U+4F67.0	fCangjie	OYMY
U+4F67.0	fFourCorner	21231
U+4F67.0	fCantonese	kaa1; kaa2
U+4F67.0	fMandarin	ka3
U+4F68.0	fUTF8	佨
U+4F68.0	fCNS11643	3-6449
U+4F68.0	fGB	E-215F
U+4F68.0	fR/S	 9.5
U+4F68.0	fTotalStrokes	 7
U+4F68.0	fCangjie	OPRU
U+4F68.0	fFourCorner	2721
U+4F68.0	fCantonese	baau1
U+4F69.0	fUTF8	佩
U+4F69.0	fCNS11643	1-4B79
U+4F69.0	fBig5	A8D8
U+4F69.0	fGB	0-4565
U+4F69.0	fR/S	 9.6
U+4F69.0	fTotalStrokes	 8
U+4F69.0	fCangjie	OHNB
U+4F69.0	fFourCorner	27210
U+4F69.0	fHakka	  pui5; pi5; poi5; bi5
U+4F69.0	fCantonese	pui3
U+4F69.0	fMandarin	pei4
U+4F69.0	fEnglish	[1] wear; carry to admire; adore [2] be grateful [3] something worn on a girdle or clothing; pendant
U+4F6A.0	fUTF8	佪
U+4F6A.0	fCNS11643	2-253F
U+4F6A.0	fBig5	CBBF
U+4F6A.0	fGB	5-3178
U+4F6A.0	fR/S	 9.6
U+4F6A.0	fTotalStrokes	 8
U+4F6A.0	fCangjie	OWR
U+4F6A.0	fFourCorner	26200
U+4F6A.0	fHakka	 fui2; fi2
U+4F6A.0	fCantonese	wui4
U+4F6A.0	fMandarin	huai2; hui2
U+4F6A.0	fEnglish	hesitate
U+4F6B.0	fUTF8	佫
U+4F6B.0	fCNS11643	2-254B
U+4F6B.0	fBig5	CBCB
U+4F6B.0	fGB	3-312D
U+4F6B.0	fR/S	 9.6
U+4F6B.0	fTotalStrokes	 8
U+4F6B.0	fCangjie	OHER
U+4F6B.0	fFourCorner	27264
U+4F6B.0	fHakka	 hok8; hok7
U+4F6B.0	fCantonese	hok6; gaak3; hok3
U+4F6B.0	fMandarin	he4
U+4F6B.0	fEnglish	a Chinese family name
U+4F6C.0	fUTF8	佬
U+4F6C.0	fCNS11643	1-4B71
U+4F6C.0	fBig5	A8D0
U+4F6C.0	fGB	0-4050
U+4F6C.0	fR/S	 9.6
U+4F6C.0	fTotalStrokes	 8
U+4F6C.0	fCangjie	OJKP
U+4F6C.0	fFourCorner	24211
U+4F6C.0	fHakka	  lau3; lo3
U+4F6C.0	fCantonese	lou2; lou5; liu4
U+4F6C.0	fMandarin	lao3; liao2; lu4
U+4F6C.0	fEnglish	[1] fellow; man; person; guy [2] vulgar person; hillbilly
U+4F6D.0	fUTF8	佭
U+4F6D.0	fCNS11643	3-2747
U+4F6D.0	fGB	5-3179
U+4F6D.0	fR/S	 9.6
U+4F6D.0	fTotalStrokes	 8; 9
U+4F6D.0	fCangjie	OHEQ
U+4F6D.0	fFourCorner	27250
U+4F6D.0	fHakka	  gong5; hong2
U+4F6D.0	fCantonese	gong3
U+4F6D.0	fMandarin	xiang2
U+4F6D.0	fEnglish	 unsubmissive obstreperous
U+4F6E.0	fUTF8	佮
U+4F6E.0	fCNS11643	2-254C
U+4F6E.0	fBig5	CBCC
U+4F6E.0	fGB	3-3129
U+4F6E.0	fR/S	 9.6
U+4F6E.0	fTotalStrokes	 8
U+4F6E.0	fCangjie	OOMR
U+4F6E.0	fFourCorner	28261
U+4F6E.0	fHakka	  hap8;  kap7; gap7; am1
U+4F6E.0	fCantonese	gap3; daap6; haap6; tap1
U+4F6E.0	fMandarin	ge2; an1; ta4
U+4F6E.0	fEnglish	 bring together as a company or friends
U+4F6F.0	fUTF8	佯
U+4F6F.0	fCNS11643	1-4B6C
U+4F6F.0	fBig5	A8CB
U+4F6F.0	fGB	0-5170
U+4F6F.0	fR/S	 9.6
U+4F6F.0	fTotalStrokes	 8
U+4F6F.0	fCangjie	OTQ
U+4F6F.0	fFourCorner	28251
U+4F6F.0	fHakka	 jong2; rong2; jong5
U+4F6F.0	fCantonese	joeng4; jeoi4
U+4F6F.0	fMandarin	yang2; qiang3
U+4F6F.0	fEnglish	[1] pretend; feign; sham [2] false; deceitful; feigning
U+4F70.0	fUTF8	佰
U+4F70.0	fCNS11643	1-4B76
U+4F70.0	fBig5	A8D5
U+4F70.0	fGB	0-305B
U+4F70.0	fR/S	 9.6
U+4F70.0	fTotalStrokes	 8
U+4F70.0	fCangjie	OMA
U+4F70.0	fFourCorner	21260
U+4F70.0	fHakka	 bak7
U+4F70.0	fCantonese	baak3; mak6
U+4F70.0	fMandarin	bai3; bo2; lai2; mo4
U+4F70.0	fEnglish	hundred
U+4F71.0	fUTF8	佱
U+4F71.0	fCNS11643	4-253E
U+4F71.0	fGB	E-2160
U+4F71.0	fR/S	 9.6
U+4F71.0	fTotalStrokes	 8; 7
U+4F71.0	fCangjie	OMMM
U+4F71.0	fFourCorner	80100
U+4F71.0	fCantonese	faat3
U+4F71.0	fMandarin	fa3
U+4F72.0	fUTF8	佲
U+4F72.0	fCNS11643	3-274F
U+4F72.0	fGB	5-317A
U+4F72.0	fR/S	 9.6
U+4F72.0	fTotalStrokes	 8
U+4F72.0	fCangjie	ONIR
U+4F72.0	fFourCorner	27260
U+4F72.0	fHakka	 min2
U+4F72.0	fCantonese	ming5
U+4F72.0	fMandarin	ming2
U+4F73.0	fUTF8	佳
U+4F73.0	fCNS11643	1-4B6F
U+4F73.0	fBig5	A8CE
U+4F73.0	fGB	0-3C51
U+4F73.0	fR/S	 9.6
U+4F73.0	fTotalStrokes	 8
U+4F73.0	fCangjie	OGG
U+4F73.0	fFourCorner	24214
U+4F73.0	fHakka	  ga1; gai1
U+4F73.0	fCantonese	gaai1
U+4F73.0	fMandarin	jia1; jia5
U+4F73.0	fEnglish	[1] good; beautiful; fine [2] auspicious [3] distinguished
U+4F74.0	fUTF8	佴
U+4F74.0	fCNS11643	2-2539
U+4F74.0	fBig5	CBB9
U+4F74.0	fGB	0-5926
U+4F74.0	fR/S	 9.6
U+4F74.0	fTotalStrokes	 8
U+4F74.0	fCangjie	OSJ
U+4F74.0	fFourCorner	21240
U+4F74.0	fHakka	 ngi1; ngi5; ngong1
U+4F74.0	fCantonese	ji6; ji5 (also); noi6; loi6 (also); naai6
U+4F74.0	fMandarin	er4; nai4; mi3
U+4F74.0	fEnglish	a second; an assistant
U+4F75.0	fUTF8	併
U+4F75.0	fCNS11643	1-4B77
U+4F75.0	fBig5	A8D6
U+4F75.0	fGB	E-2161
U+4F75.0	fR/S	 9.6; 9.8
U+4F75.0	fTotalStrokes	 8
U+4F75.0	fCangjie	OTT
U+4F75.0	fFourCorner	28241
U+4F75.0	fHakka	  bin5; biang1; bin3; pin1; pin5
U+4F75.0	fCantonese	bing3; ping3 (also); bing6 (also)
U+4F75.0	fMandarin	bing4
U+4F75.0	fEnglish	combine; annex
U+4F76.0	fUTF8	佶
U+4F76.0	fCNS11643	2-2538
U+4F76.0	fBig5	CBB8
U+4F76.0	fGB	0-5925
U+4F76.0	fR/S	 9.6
U+4F76.0	fTotalStrokes	 8
U+4F76.0	fCangjie	OGR
U+4F76.0	fFourCorner	24261
U+4F76.0	fHakka	  git7;  giet7
U+4F76.0	fCantonese	gat1; git6
U+4F76.0	fMandarin	ji2
U+4F76.0	fEnglish	 strong; robust exact; correct
U+4F76.1	fUTF8	佶
U+4F76.1	fCNS11643	2-2538
U+4F76.1	fBig5	CBB8
U+4F76.1	fGB	0-5925
U+4F76.1	fR/S	 9.6
U+4F76.1	fTotalStrokes	 8
U+4F76.1	fCangjie	OGR
U+4F76.1	fFourCorner	24261
U+4F76.1	fCantonese	git3
U+4F77.0	fUTF8	佷
U+4F77.0	fCNS11643	2-253C
U+4F77.0	fBig5	CBBC
U+4F77.0	fGB	5-317C
U+4F77.0	fR/S	 9.6
U+4F77.0	fTotalStrokes	 8
U+4F77.0	fCangjie	OAV
U+4F77.0	fFourCorner	27232
U+4F77.0	fCantonese	han2
U+4F77.0	fMandarin	heng2; hen3
U+4F77.0	fEnglish	 act contrary to
U+4F78.0	fUTF8	佸
U+4F78.0	fCNS11643	2-2543
U+4F78.0	fBig5	CBC3
U+4F78.0	fGB	3-3125
U+4F78.0	fR/S	 9.6
U+4F78.0	fTotalStrokes	 8
U+4F78.0	fCangjie	OHJR
U+4F78.0	fFourCorner	22264
U+4F78.0	fHakka	 kwat7; gwat7
U+4F78.0	fCantonese	wut6 (also); kut3; kwut3 (also)
U+4F78.0	fMandarin	huo2
U+4F78.0	fEnglish	meet
U+4F79.0	fUTF8	佹
U+4F79.0	fCNS11643	2-2541
U+4F79.0	fBig5	CBC1
U+4F79.0	fGB	3-312A
U+4F79.0	fR/S	 9.6
U+4F79.0	fTotalStrokes	 8
U+4F79.0	fCangjie	ONMU
U+4F79.0	fFourCorner	27212
U+4F79.0	fHakka	 gui3; gwi3; kwui3
U+4F79.0	fCantonese	gwai2
U+4F79.0	fMandarin	gui3; gui1
U+4F79.0	fEnglish	 depend upon support go against variant deceive
U+4F7A.0	fUTF8	佺
U+4F7A.0	fCNS11643	1-4C21
U+4F7A.0	fBig5	A8DE
U+4F7A.0	fGB	3-3128
U+4F7A.0	fR/S	 9.6
U+4F7A.0	fTotalStrokes	 8
U+4F7A.0	fCangjie	OOMG
U+4F7A.0	fFourCorner	28214
U+4F7A.0	fHakka	  cien2; cion2
U+4F7A.0	fCantonese	cyun4
U+4F7A.0	fMandarin	quan2
U+4F7A.0	fEnglish	fairy
U+4F7B.0	fUTF8	佻
U+4F7B.0	fCNS11643	1-4B7A
U+4F7B.0	fBig5	A8D9
U+4F7B.0	fGB	0-592C
U+4F7B.0	fR/S	 9.6
U+4F7B.0	fTotalStrokes	 8
U+4F7B.0	fCangjie	OLMO
U+4F7B.0	fFourCorner	22213
U+4F7B.0	fHakka	 tiau1; chau5; cau5
U+4F7B.0	fCantonese	tiu1; tiu4 (also); tiu3 (also)
U+4F7B.0	fMandarin	tiao1; tiao2; diao4; tiao3; yao2
U+4F7B.0	fEnglish	[1] frivolous; imprudent [2] steal; stealthily; act on a furtive manner [3] delay; dilatory; slow [4] provoke
U+4F7C.0	fUTF8	佼
U+4F7C.0	fCNS11643	2-2533
U+4F7C.0	fBig5	CBB3
U+4F7C.0	fGB	0-592E
U+4F7C.0	fR/S	 9.6
U+4F7C.0	fTotalStrokes	 8
U+4F7C.0	fCangjie	OYCK
U+4F7C.0	fFourCorner	20248
U+4F7C.0	fHakka	 gau3; giau1; gau1
U+4F7C.0	fCantonese	gaau2; gaau1
U+4F7C.0	fMandarin	jiao3; jiao1; xiao4
U+4F7C.0	fEnglish	[1] beautiful; handsome; attractive; charming [2] outstanding
U+4F7D.0	fUTF8	佽
U+4F7D.0	fCNS11643	2-2535
U+4F7D.0	fBig5	CBB5
U+4F7D.0	fGB	3-312F
U+4F7D.0	fR/S	 9.6
U+4F7D.0	fTotalStrokes	 8
U+4F7D.0	fCangjie	OIMO
U+4F7D.0	fFourCorner	77282
U+4F7D.0	fHakka	 cih5; ze5
U+4F7D.0	fCantonese	ci3
U+4F7D.0	fMandarin	ci4
U+4F7D.0	fEnglish	 aid help
U+4F7E.0	fUTF8	佾
U+4F7E.0	fCNS11643	1-4B7C
U+4F7E.0	fBig5	A8DB
U+4F7E.0	fGB	0-592B
U+4F7E.0	fR/S	 9.6
U+4F7E.0	fTotalStrokes	 8
U+4F7E.0	fCangjie	OCB
U+4F7E.0	fFourCorner	28227
U+4F7E.0	fHakka	 jit8; rit8
U+4F7E.0	fCantonese	jat6
U+4F7E.0	fMandarin	yi4
U+4F7E.0	fEnglish	a row or file of dancers, especially referring to those in ancient dances at sacrifices or other rites; dance squad
U+4F7F.0	fUTF8	使
U+4F7F.0	fCNS11643	1-4B70
U+4F7F.0	fBig5	A8CF
U+4F7F.0	fGB	0-4A39
U+4F7F.0	fR/S	 9.6
U+4F7F.0	fTotalStrokes	 8
U+4F7F.0	fCangjie	OJLK
U+4F7F.0	fFourCorner	25206
U+4F7F.0	fHakka	  sii3;  su3; sii6; sii5; se3; se5
U+4F7F.0	fCantonese	si2; sai2 (also); si3
U+4F7F.0	fEnglish	[1] use; employ; apply [2] make; act [3] indulge in [4] send as diplomatic personnel; diplomatic envoys
U+4F7F.1	fUTF8	使
U+4F7F.1	fCNS11643	1-4B70
U+4F7F.1	fBig5	A8CF
U+4F7F.1	fGB	0-4A39
U+4F7F.1	fR/S	 9.6
U+4F7F.1	fTotalStrokes	 8
U+4F7F.1	fCangjie	OJLK
U+4F7F.1	fFourCorner	25206
U+4F7F.1	fMandarin	shi4; shi3
U+4F7F.1	fEnglish	[1] be appointed as a diplomatic envoy; be an ambassador to [2] envoy; emissary; minister
U+4F80.0	fUTF8	侀
U+4F80.0	fCNS11643	2-2536
U+4F80.0	fBig5	CBB6
U+4F80.0	fGB	5-3171
U+4F80.0	fR/S	 9.6
U+4F80.0	fTotalStrokes	 8
U+4F80.0	fCangjie	OMTN
U+4F80.0	fFourCorner	22200
U+4F80.0	fHakka	 hin2
U+4F80.0	fCantonese	jing4
U+4F80.0	fMandarin	xing2
U+4F80.0	fEnglish	form
U+4F81.0	fUTF8	侁
U+4F81.0	fCNS11643	2-2542
U+4F81.0	fBig5	CBC2
U+4F81.0	fGB	3-3123
U+4F81.0	fR/S	 9.6
U+4F81.0	fTotalStrokes	 8
U+4F81.0	fCangjie	OHGU
U+4F81.0	fFourCorner	24211
U+4F81.0	fHakka	  sin1;  sien1
U+4F81.0	fCantonese	san1
U+4F81.0	fMandarin	shen1
U+4F81.0	fEnglish	crowd
U+4F82.0	fUTF8	侂
U+4F82.0	fCNS11643	2-2549
U+4F82.0	fBig5	CBC9
U+4F82.0	fGB	3-312E
U+4F82.0	fR/S	 9.6
U+4F82.0	fTotalStrokes	 8
U+4F82.0	fCangjie	OIHP
U+4F82.0	fFourCorner	20214
U+4F82.0	fHakka	 za5;  tok7
U+4F82.0	fCantonese	tok3
U+4F82.0	fMandarin	tuo1
U+4F82.0	fEnglish	 commission; entrust to; depute request; ask
U+4F83.0	fUTF8	侃
U+4F83.0	fCNS11643	1-4B75
U+4F83.0	fBig5	A8D4
U+4F83.0	fGB	0-5929
U+4F83.0	fR/S	 9.6
U+4F83.0	fTotalStrokes	 8
U+4F83.0	fCangjie	ORHU
U+4F83.0	fFourCorner	26210
U+4F83.0	fHakka	 kan3; kon3
U+4F83.0	fCantonese	hon2; hon3
U+4F83.0	fMandarin	kan3
U+4F83.0	fEnglish	[1] straightforward; frank; bold; open [2] amiable; pleasant [3] with confidence and composure
U+4F84.0	fUTF8	侄
U+4F84.0	fCNS11643	2-253B
U+4F84.0	fBig5	CBBB
U+4F84.0	fGB	0-5636
U+4F84.0	fR/S	 9.6
U+4F84.0	fTotalStrokes	 8
U+4F84.0	fCangjie	OMIG
U+4F84.0	fFourCorner	21214
U+4F84.0	fHakka	 cit8; chit8
U+4F84.0	fCantonese	zat6; zat1
U+4F84.0	fMandarin	zhi2
U+4F84.0	fEnglish	nephew
U+4F85.0	fUTF8	侅
U+4F85.0	fCNS11643	2-2534
U+4F85.0	fBig5	CBB4
U+4F85.0	fGB	3-3130
U+4F85.0	fR/S	 9.6
U+4F85.0	fTotalStrokes	 8
U+4F85.0	fCangjie	OYVO
U+4F85.0	fFourCorner	20282
U+4F85.0	fHakka	  goi1;  goi5
U+4F85.0	fCantonese	goi1
U+4F85.0	fMandarin	gai1; kai1; hai4
U+4F85.0	fEnglish	 give prepared for included in embraced in
U+4F86.0	fUTF8	來
U+4F86.0	fCNS11643	1-4B74
U+4F86.0	fBig5	A8D3
U+4F86.0	fGB	1-4034
U+4F86.0	fR/S	 9.6
U+4F86.0	fTotalStrokes	 8
U+4F86.0	fCangjie	DOO
U+4F86.0	fFourCorner	40908
U+4F86.0	fHakka	  loi2; loi3
U+4F86.0	fCantonese	loi4; lai4; loi2 (also); lai2 (also)
U+4F86.0	fMandarin	lai2; lai4
U+4F86.0	fEnglish	[1] come; coming; arrive [2] used in place of a verb [3] return; to come back; returning
U+4F86.1	fUTF8	來
U+4F86.1	fCNS11643	1-4B74
U+4F86.1	fBig5	A8D3
U+4F86.1	fGB	1-4034
U+4F86.1	fR/S	 9.6
U+4F86.1	fTotalStrokes	 8
U+4F86.1	fCangjie	DOO
U+4F86.1	fFourCorner	40908
U+4F86.1	fCantonese	loi6
U+4F87.0	fUTF8	侇
U+4F87.0	fCNS11643	2-2537
U+4F87.0	fBig5	CBB7
U+4F87.0	fGB	3-307B
U+4F87.0	fR/S	 9.6
U+4F87.0	fTotalStrokes	 8
U+4F87.0	fCangjie	OKN
U+4F87.0	fFourCorner	25232
U+4F87.0	fHakka	 ji2; ri2
U+4F87.0	fCantonese	ji4
U+4F87.0	fMandarin	yi2
U+4F87.0	fEnglish	a class; a category a corpse
U+4F88.0	fUTF8	侈
U+4F88.0	fCNS11643	1-4B78
U+4F88.0	fBig5	A8D7
U+4F88.0	fGB	0-335E
U+4F88.0	fR/S	 9.6
U+4F88.0	fTotalStrokes	 8
U+4F88.0	fCangjie	ONIN
U+4F88.0	fFourCorner	27227
U+4F88.0	fHakka	 ci3;  chi3; chi1; ci1; chii3
U+4F88.0	fCantonese	ci2
U+4F88.0	fMandarin	chi3; yi2
U+4F88.0	fEnglish	[1] wasteful; luxurious; lavish; extravagant [2] exaggerate; bragging [3] evildoing [4] excessive
U+4F89.0	fUTF8	侉
U+4F89.0	fCNS11643	2-253A
U+4F89.0	fBig5	CBBA
U+4F89.0	fGB	0-5928
U+4F89.0	fR/S	 9.6
U+4F89.0	fTotalStrokes	 8
U+4F89.0	fCangjie	OKMS
U+4F89.0	fFourCorner	24227
U+4F89.0	fHakka	 kwa1; ka1; kwa5
U+4F89.0	fCantonese	kwaa2; kwaa1
U+4F89.0	fMandarin	kua1; kua3; e4
U+4F89.0	fEnglish	[1] presumptuous [2] boastful
U+4F8A.0	fUTF8	侊
U+4F8A.0	fCNS11643	3-2744
U+4F8A.0	fGB	3-307E
U+4F8A.0	fR/S	 9.6
U+4F8A.0	fTotalStrokes	 8
U+4F8A.0	fCangjie	OFMU
U+4F8A.0	fFourCorner	29210
U+4F8A.0	fCantonese	gwong1; gong1 (also)
U+4F8A.0	fMandarin	guang1
U+4F8A.0	fEnglish	big
U+4F8B.0	fUTF8	例
U+4F8B.0	fCNS11643	1-4B73
U+4F8B.0	fBig5	A8D2
U+4F8B.0	fGB	0-407D
U+4F8B.0	fR/S	 9.6
U+4F8B.0	fTotalStrokes	 8
U+4F8B.0	fCangjie	OMNN
U+4F8B.0	fFourCorner	22200
U+4F8B.0	fHakka	  li5; li6; li3
U+4F8B.0	fCantonese	lai6
U+4F8B.0	fMandarin	li4; lie4
U+4F8B.0	fEnglish	[1] regulation; rule; something with which to compare; practice [2] precedent [3] example; instance [4] regular; routine
U+4F8C.0	fUTF8	侌
U+4F8C.0	fCNS11643	3-2751
U+4F8C.0	fGB	E-2162
U+4F8C.0	fR/S	 9.6
U+4F8C.0	fTotalStrokes	 8
U+4F8C.0	fCangjie	XOINI
U+4F8C.0	fFourCorner	80730
U+4F8C.0	fCantonese	jam1
U+4F8C.0	fMandarin	yin1
U+4F8D.0	fUTF8	侍
U+4F8D.0	fCNS11643	1-4B6E
U+4F8D.0	fBig5	A8CD
U+4F8D.0	fGB	0-4A4C
U+4F8D.0	fR/S	 9.6
U+4F8D.0	fTotalStrokes	 8
U+4F8D.0	fCangjie	OGDI
U+4F8D.0	fFourCorner	24241
U+4F8D.0	fHakka	 sii5; shi6; sii6; si5; su3; she5; shi5
U+4F8D.0	fCantonese	si6; si2 (also)
U+4F8D.0	fMandarin	shi4
U+4F8D.0	fEnglish	[1] serve; wait upon [2] accompany one's elder or superior [3] designation for oneself when addressing an elder or superior [4] attendant
U+4F8E.0	fUTF8	侎
U+4F8E.0	fCNS11643	3-2748
U+4F8E.0	fGB	E-2163
U+4F8E.0	fR/S	  9.6
U+4F8E.0	fTotalStrokes	 8
U+4F8E.0	fCangjie	OFD
U+4F8E.0	fFourCorner	29290
U+4F8E.0	fHakka	 mi3; mia1
U+4F8E.0	fCantonese	mei5
U+4F8E.0	fMandarin	mi3; mei3
U+4F8E.0	fEnglish	 soothe; pacify settle; establish
U+4F8F.0	fUTF8	侏
U+4F8F.0	fCNS11643	1-4B7D
U+4F8F.0	fBig5	A8DC
U+4F8F.0	fGB	0-592A
U+4F8F.0	fR/S	 9.6
U+4F8F.0	fTotalStrokes	 8
U+4F8F.0	fCangjie	OHJD
U+4F8F.0	fFourCorner	25290
U+4F8F.0	fHakka	 zu1;  zhu1
U+4F8F.0	fCantonese	zyu1
U+4F8F.0	fMandarin	zhu1; zhou1
U+4F8F.0	fEnglish	[1] short [2] pigmy; dwarf
U+4F90.0	fUTF8	侐
U+4F90.0	fCNS11643	2-2544
U+4F90.0	fBig5	CBC4
U+4F90.0	fGB	3-3126
U+4F90.0	fR/S	 9.6
U+4F90.0	fTotalStrokes	 8
U+4F90.0	fCangjie	OHBT
U+4F90.0	fFourCorner	27210
U+4F90.0	fHakka	  sit7
U+4F90.0	fCantonese	gwik1; kwik1
U+4F90.0	fMandarin	xu4; xi4
U+4F90.0	fEnglish	quiet
U+4F91.0	fUTF8	侑
U+4F91.0	fCNS11643	1-4B7E
U+4F91.0	fBig5	A8DD
U+4F91.0	fGB	0-5927
U+4F91.0	fR/S	 9.6
U+4F91.0	fTotalStrokes	 8
U+4F91.0	fCangjie	OKB
U+4F91.0	fFourCorner	24227
U+4F91.0	fHakka	 ju3; ju5; riu3; jiu3
U+4F91.0	fCantonese	jau6
U+4F91.0	fMandarin	you4
U+4F91.0	fEnglish	[1] help [2] urge (or press) somebody to eat or drink [3] repay other's kindness
U+4F92.0	fUTF8	侒
U+4F92.0	fCNS11643	2-2548
U+4F92.0	fBig5	CBC8
U+4F92.0	fGB	5-317B
U+4F92.0	fR/S	 9.6
U+4F92.0	fTotalStrokes	 8
U+4F92.0	fCangjie	OJV
U+4F92.0	fFourCorner	23244
U+4F92.0	fHakka	 on1
U+4F92.0	fCantonese	on1; ngon1 (also)
U+4F92.0	fMandarin	an1
U+4F93.0	fUTF8	侓
U+4F93.0	fCNS11643	3-2743
U+4F93.0	fGB	E-2164
U+4F93.0	fR/S	  9.6
U+4F93.0	fTotalStrokes	 8
U+4F93.0	fCangjie	OLQ
U+4F93.0	fFourCorner	25200
U+4F93.0	fHakka	 lit8; lut8
U+4F93.0	fCantonese	leot6
U+4F93.0	fMandarin	lu4
U+4F94.0	fUTF8	侔
U+4F94.0	fCNS11643	2-2546
U+4F94.0	fBig5	CBC6
U+4F94.0	fGB	0-5930
U+4F94.0	fR/S	 9.6
U+4F94.0	fTotalStrokes	 8
U+4F94.0	fCangjie	OIHQ
U+4F94.0	fFourCorner	23250
U+4F94.0	fHakka	 meu2; miu2
U+4F94.0	fCantonese	mau4
U+4F94.0	fMandarin	mou2
U+4F94.0	fEnglish	equal
U+4F95.0	fUTF8	侕
U+4F95.0	fCNS11643	2-254A
U+4F95.0	fBig5	CBCA
U+4F95.0	fGB	E-2165
U+4F95.0	fR/S	 9.6
U+4F95.0	fTotalStrokes	 8
U+4F95.0	fCangjie	OMBL
U+4F95.0	fFourCorner	21227
U+4F95.0	fHakka	 ji2
U+4F95.0	fCantonese	ji4
U+4F95.0	fMandarin	er2
U+4F95.0	fEnglish	a small post standing on the beam, to support the roof
U+4F96.0	fUTF8	侖
U+4F96.0	fCNS11643	1-4B7B
U+4F96.0	fBig5	A8DA
U+4F96.0	fGB	1-4258
U+4F96.0	fR/S	 9.6
U+4F96.0	fTotalStrokes	 8
U+4F96.0	fCangjie	OMBT
U+4F96.0	fFourCorner	80227
U+4F96.0	fHakka	 lun2; lun3
U+4F96.0	fCantonese	leon4; leon2 (also)
U+4F96.0	fMandarin	lun2
U+4F96.0	fEnglish	logical reasons; logical order
U+4F97.0	fUTF8	侗
U+4F97.0	fCNS11643	2-253E
U+4F97.0	fBig5	CBBE
U+4F97.0	fGB	0-3631
U+4F97.0	fR/S	 9.6
U+4F97.0	fTotalStrokes	 8
U+4F97.0	fCangjie	OBMR
U+4F97.0	fFourCorner	27220
U+4F97.0	fHakka	  tung2; tung1; tung3
U+4F97.0	fCantonese	tung4; tung2
U+4F97.0	fMandarin	tong1; tong2; tong3; tong4
U+4F97.0	fEnglish	 big ignorant rude; rustic
U+4F97.1	fUTF8	侗
U+4F97.1	fCNS11643	2-253E
U+4F97.1	fBig5	CBBE
U+4F97.1	fGB	0-3631
U+4F97.1	fR/S	 9.6
U+4F97.1	fTotalStrokes	 8
U+4F97.1	fCangjie	OBMR
U+4F97.1	fFourCorner	27220
U+4F97.1	fCantonese	dung6
U+4F97.1	fMandarin	dong4
U+4F98.0	fUTF8	侘
U+4F98.0	fCNS11643	2-2532
U+4F98.0	fBig5	CBB2
U+4F98.0	fGB	3-3131
U+4F98.0	fR/S	 9.6
U+4F98.0	fTotalStrokes	 8
U+4F98.0	fCangjie	OJHP
U+4F98.0	fFourCorner	23214
U+4F98.0	fHakka	  za5;  tok7
U+4F98.0	fCantonese	caa3
U+4F98.0	fMandarin	cha4; cha3
U+4F98.0	fEnglish	[1] boast [2] disappointed
U+4F99.0	fUTF8	侙
U+4F99.0	fCNS11643	4-2536
U+4F99.0	fGB	3-3079
U+4F99.0	fR/S	 9.6
U+4F99.0	fTotalStrokes	 8; 7
U+4F99.0	fCangjie	OIPM
U+4F99.0	fFourCorner	23210
U+4F99.0	fHakka	 chit7
U+4F99.0	fCantonese	cik1
U+4F99.0	fMandarin	chi4
U+4F9A.0	fUTF8	侚
U+4F9A.0	fCNS11643	2-2540
U+4F9A.0	fBig5	CBC0
U+4F9A.0	fGB	3-312B
U+4F9A.0	fR/S	 9.6
U+4F9A.0	fTotalStrokes	 8
U+4F9A.0	fCangjie	OPA
U+4F9A.0	fFourCorner	27220
U+4F9A.0	fHakka	 sun2
U+4F9A.0	fCantonese	seon1; seon6; ceon4
U+4F9A.0	fMandarin	xun4; xun2
U+4F9A.0	fEnglish	fast
U+4F9B.0	fUTF8	供
U+4F9B.0	fCNS11643	1-4B72
U+4F9B.0	fBig5	A8D1
U+4F9B.0	fGB	0-3929
U+4F9B.0	fR/S	 9.6
U+4F9B.0	fTotalStrokes	 8
U+4F9B.0	fCangjie	OTC
U+4F9B.0	fFourCorner	24281
U+4F9B.0	fHakka	  giung1; kiung1; giung5
U+4F9B.0	fCantonese	gung1; gung3
U+4F9B.0	fMandarin	gong1
U+4F9B.0	fEnglish	 supply; contribute to
U+4F9B.1	fUTF8	供
U+4F9B.1	fCNS11643	1-4B72
U+4F9B.1	fBig5	A8D1
U+4F9B.1	fGB	0-3929
U+4F9B.1	fR/S	 9.6
U+4F9B.1	fTotalStrokes	 8
U+4F9B.1	fCangjie	OTC
U+4F9B.1	fFourCorner	24281
U+4F9B.1	fHakka	 giung5
U+4F9B.1	fMandarin	gong4
U+4F9B.1	fEnglish	[1] give a statement or an account of a criminal act; give evidence; confess [2] offer in worship [3] articles of sacrifice; offerings [4] testimony of a prisoner; confession [5] supplies
U+4F9C.0	fUTF8	侜
U+4F9C.0	fCNS11643	2-2545
U+4F9C.0	fBig5	CBC5
U+4F9C.0	fGB	3-3127
U+4F9C.0	fR/S	 9.6
U+4F9C.0	fTotalStrokes	 8
U+4F9C.0	fCangjie	OHBY
U+4F9C.0	fFourCorner	27240
U+4F9C.0	fHakka	 zhiu1; zu1; zhu1
U+4F9C.0	fCantonese	zau1
U+4F9C.0	fMandarin	zhou1
U+4F9C.0	fEnglish	lies
U+4F9D.0	fUTF8	依
U+4F9D.0	fCNS11643	1-4B6D
U+4F9D.0	fBig5	A8CC
U+4F9D.0	fGB	0-5240
U+4F9D.0	fR/S	 9.6
U+4F9D.0	fTotalStrokes	 8
U+4F9D.0	fCangjie	OYHV
U+4F9D.0	fFourCorner	20232
U+4F9D.0	fHakka	 ji1;  ji3; ri3; ri1; i3; i1
U+4F9D.0	fCantonese	ji1; ji2
U+4F9D.0	fMandarin	yi1; yi3; yi2
U+4F9D.0	fEnglish	[1] depend on; lean to [2] follow; comply with; consent; yield up [3] be tolerant to; forgive [4] according to; in the light of; judging by
U+4F9E.0	fUTF8	侞
U+4F9E.0	fCNS11643	2-2547
U+4F9E.0	fBig5	CBC7
U+4F9E.0	fGB	3-3133
U+4F9E.0	fR/S	 9.6
U+4F9E.0	fTotalStrokes	 8
U+4F9E.0	fCangjie	OVR
U+4F9E.0	fFourCorner	26200
U+4F9E.0	fCantonese	jyu4
U+4F9E.0	fMandarin	ru3; ru2
U+4F9F.0	fUTF8	侟
U+4F9F.0	fCNS11643	3-274C
U+4F9F.0	fGB	5-3173
U+4F9F.0	fR/S	  9.6
U+4F9F.0	fTotalStrokes	 8
U+4F9F.0	fCangjie	OKLD
U+4F9F.0	fFourCorner	24240
U+4F9F.0	fCantonese	cyun4
U+4F9F.0	fMandarin	jian4
U+4FA0.0	fUTF8	侠
U+4FA0.0	fCNS11643	3-2749
U+4FA0.0	fGB	0-4F40
U+4FA0.0	fR/S	 9.6
U+4FA0.0	fTotalStrokes	 8
U+4FA0.0	fCangjie	OKT
U+4FA0.0	fFourCorner	25230
U+4FA0.0	fHakka	 hiap8
U+4FA0.0	fCantonese	hap6; haap6 (also)
U+4FA0.0	fMandarin	xia2
U+4FA0.0	fEnglish	simplified form of &#x4FE0;
U+4FA1.0	fUTF8	価
U+4FA1.0	fCNS11643	3-274B
U+4FA1.0	fGB	E-2166
U+4FA1.0	fR/S	 9.6
U+4FA1.0	fTotalStrokes	 8
U+4FA1.0	fCangjie	OMLW
U+4FA1.0	fFourCorner	21260
U+4FA1.0	fCantonese	ci5
U+4FA1.0	fMandarin	jia4; jie4
U+4FA1.0	fEnglish	price; value
U+4FA2.0	fUTF8	侢
U+4FA2.0	fCNS11643	3-274A
U+4FA2.0	fGB	5-3172
U+4FA2.0	fR/S	 9.6
U+4FA2.0	fTotalStrokes	 8
U+4FA2.0	fCangjie	OMGB
U+4FA2.0	fFourCorner	21250
U+4FA2.0	fCantonese	daai3
U+4FA2.0	fMandarin	zai4
U+4FA2.0	fEnglish	load
U+4FA3.0	fUTF8	侣
U+4FA3.0	fCNS11643	3-274D
U+4FA3.0	fGB	0-4242
U+4FA3.0	fR/S	 9.6
U+4FA3.0	fTotalStrokes	 8
U+4FA3.0	fCangjie	ORR
U+4FA3.0	fFourCorner	26260
U+4FA3.0	fCantonese	leoi5
U+4FA3.0	fMandarin	luu3; l3
U+4FA3.0	fEnglish	 companion associate with
U+4FA4.0	fUTF8	侤
U+4FA4.0	fGB	1-7D2E
U+4FA4.0	fR/S	 9.6
U+4FA4.0	fTotalStrokes	 8
U+4FA4.0	fCangjie	OJKS
U+4FA4.0	fCantonese	haau2
U+4FA5.0	fUTF8	侥
U+4FA5.0	fGB	0-3D44
U+4FA5.0	fR/S	 9.6
U+4FA5.0	fTotalStrokes	 8
U+4FA5.0	fCangjie	OJPU
U+4FA5.0	fHakka	 hiau1; hiau3
U+4FA5.0	fCantonese	hiu1; jiu4; giu1
U+4FA5.0	fMandarin	jiao3; yao2; jiao1
U+4FA5.0	fEnglish	simplified form of &#x50E5;
U+4FA6.0	fUTF8	侦
U+4FA6.0	fGB	0-556C
U+4FA6.0	fR/S	 9.6
U+4FA6.0	fTotalStrokes	 8
U+4FA6.0	fCangjie	OYBO
U+4FA6.0	fHakka	 zhin1
U+4FA6.0	fCantonese	zing1
U+4FA6.0	fMandarin	zhen1; zheng1
U+4FA6.0	fEnglish	simplified form of &#x5075;
U+4FA7.0	fUTF8	侧
U+4FA7.0	fGB	0-3260
U+4FA7.0	fR/S	 9.6
U+4FA7.0	fTotalStrokes	 8
U+4FA7.0	fCangjie	OBON
U+4FA7.0	fHakka	 zet7
U+4FA7.0	fCantonese	zak1
U+4FA7.0	fMandarin	ce4; ze4
U+4FA7.0	fEnglish	simplified form of &#x5074;
U+4FA8.0	fUTF8	侨
U+4FA8.0	fGB	0-4748
U+4FA8.0	fR/S	 9.6
U+4FA8.0	fTotalStrokes	 8
U+4FA8.0	fCangjie	OHKL
U+4FA8.0	fHakka	 kiau2
U+4FA8.0	fCantonese	kiu4
U+4FA8.0	fMandarin	qiao2
U+4FA8.0	fEnglish	simplified form of &#x50D1;
U+4FA9.0	fUTF8	侩
U+4FA9.0	fGB	0-3F6B
U+4FA9.0	fR/S	 9.6
U+4FA9.0	fTotalStrokes	 8
U+4FA9.0	fCangjie	OOMI
U+4FA9.0	fHakka	 kwai5
U+4FA9.0	fCantonese	kui2; kwui2 (also); kui3 (also); kwui3 (also)
U+4FA9.0	fMandarin	kuai4
U+4FA9.0	fEnglish	simplified form of &#x5108;
U+4FAA.0	fUTF8	侪
U+4FAA.0	fGB	0-592D
U+4FAA.0	fR/S	 9.6
U+4FAA.0	fTotalStrokes	 8
U+4FAA.0	fCangjie	OYKL
U+4FAA.0	fHakka	 sa2; ce2
U+4FAA.0	fCantonese	caai4
U+4FAA.0	fMandarin	chai2
U+4FAA.0	fEnglish	simplified form of &#x5115;
U+4FAB.0	fUTF8	侫
U+4FAB.0	fCNS11643	F-2544
U+4FAB.0	fGB	E-2167
U+4FAB.0	fR/S	 9.6
U+4FAB.0	fTotalStrokes	 8
U+4FAB.0	fCangjie	OYVV
U+4FAB.0	fFourCorner	2024
U+4FAB.0	fHakka	 nin5
U+4FAB.0	fCantonese	ning6; ling6 (also)
U+4FAB.0	fMandarin	ning4
U+4FAB.0	fEnglish	 flattery glib
U+4FAC.0	fUTF8	侬
U+4FAC.0	fGB	0-592F
U+4FAC.0	fR/S	 9.6
U+4FAC.0	fTotalStrokes	 8
U+4FAC.0	fCangjie	OHBV
U+4FAC.0	fHakka	 nung2
U+4FAC.0	fCantonese	nung4; lung4 (also)
U+4FAC.0	fMandarin	nong2
U+4FAC.0	fEnglish	simplified form of &#x5102;
U+4FAD.0	fUTF8	侭
U+4FAD.0	fGB	E-2168
U+4FAD.0	fR/S	 9.6
U+4FAD.0	fTotalStrokes	 8
U+4FAD.0	fCangjie	OSOY
U+4FAD.0	fCantonese	zeon2
U+4FAD.0	fMandarin	jin3; jin4
U+4FAD.0	fEnglish	complete; utmost
U+4FAE.0	fUTF8	侮
U+4FAE.0	fCNS11643	1-4F78
U+4FAE.0	fBig5	AB56
U+4FAE.0	fGB	0-4E6A
U+4FAE.0	fR/S	 9.7; 9.6
U+4FAE.0	fTotalStrokes	 9
U+4FAE.0	fCangjie	OOWY
U+4FAE.0	fFourCorner	28257
U+4FAE.0	fHakka	 wu1; wu3; fu5; wu5; vu1; vu3
U+4FAE.0	fCantonese	mou5
U+4FAE.0	fMandarin	wu3
U+4FAE.0	fEnglish	[1] bully [2] insult; disgrace; humiliate [3] insult; bully
U+4FAF.0	fUTF8	侯
U+4FAF.0	fCNS11643	1-4F6C
U+4FAF.0	fBig5	AB4A
U+4FAF.0	fGB	0-3A6E
U+4FAF.0	fR/S	 9.7
U+4FAF.0	fTotalStrokes	 9
U+4FAF.0	fCangjie	ONMK
U+4FAF.0	fFourCorner	27234
U+4FAF.0	fHakka	 heu2; heu3; hiu2
U+4FAF.0	fCantonese	hau4; hau6
U+4FAF.0	fMandarin	hou2; hou4
U+4FAF.0	fEnglish	[1] the second of the five grades of the nobility [2] the target in archery [3] marquis; nobleman; high official [4] a Chinese family name
U+4FB0.0	fUTF8	侰
U+4FB0.0	fCNS11643	3-2B2D
U+4FB0.0	fGB	5-3229
U+4FB0.0	fR/S	 9.7
U+4FB0.0	fTotalStrokes	 9
U+4FB0.0	fCangjie	OSKR
U+4FB0.0	fFourCorner	27260
U+4FB0.0	fHakka	 kiun2; kiung2
U+4FB0.0	fCantonese	kwan3
U+4FB0.0	fMandarin	jiong3
U+4FB1.0	fUTF8	侱
U+4FB1.0	fCNS11643	3-2B2E
U+4FB1.0	fGB	3-313F
U+4FB1.0	fR/S	  9.7
U+4FB1.0	fTotalStrokes	 9
U+4FB1.0	fCangjie	ORHG
U+4FB1.0	fFourCorner	26210
U+4FB1.0	fHakka	 chin3
U+4FB1.0	fCantonese	cing2
U+4FB1.0	fMandarin	cheng3
U+4FB2.0	fUTF8	侲
U+4FB2.0	fCNS11643	2-2922
U+4FB2.0	fBig5	CDE0
U+4FB2.0	fGB	3-313B
U+4FB2.0	fR/S	 9.7
U+4FB2.0	fTotalStrokes	 9
U+4FB2.0	fCangjie	OMMV
U+4FB2.0	fFourCorner	21232
U+4FB2.0	fHakka	 zhin3; zin3; shin2
U+4FB2.0	fCantonese	zan3
U+4FB2.0	fMandarin	zhen4; zhen1
U+4FB2.0	fEnglish	lad
U+4FB3.0	fUTF8	侳
U+4FB3.0	fCNS11643	2-292A
U+4FB3.0	fBig5	CDE8
U+4FB3.0	fGB	3-3142
U+4FB3.0	fR/S	 9.7
U+4FB3.0	fTotalStrokes	 9
U+4FB3.0	fCangjie	OOOG
U+4FB3.0	fFourCorner	28214
U+4FB3.0	fCantonese	zo6; co3
U+4FB3.0	fMandarin	cuo4; zuo4; cuo1
U+4FB4.0	fUTF8	侴
U+4FB4.0	fCNS11643	3-2B36
U+4FB4.0	fGB	3-327D
U+4FB4.0	fR/S	  9.7
U+4FB4.0	fTotalStrokes	 9
U+4FB4.0	fCangjie	OMNN
U+4FB4.0	fFourCorner	80120
U+4FB4.0	fHakka	 chiu3; cu3
U+4FB4.0	fCantonese	cau2
U+4FB4.0	fMandarin	chou3
U+4FB4.0	fEnglish	a Chinese family name
U+4FB5.0	fUTF8	侵
U+4FB5.0	fCNS11643	1-4F6B
U+4FB5.0	fBig5	AB49
U+4FB5.0	fGB	0-4756
U+4FB5.0	fR/S	 9.7
U+4FB5.0	fTotalStrokes	 9
U+4FB5.0	fCangjie	OSME
U+4FB5.0	fFourCorner	27247
U+4FB5.0	fHakka	  cim1; cim5
U+4FB5.0	fCantonese	cam1; cam2
U+4FB5.0	fMandarin	qin1; qin3; qiu3
U+4FB5.0	fEnglish	[1] raid; aggress; invade [2] encroach upon; use force stealthily [3] proceed gradually [4] bad year; a year of famine or disaster
U+4FB6.0	fUTF8	侶
U+4FB6.0	fCNS11643	1-4F73
U+4FB6.0	fBig5	AB51
U+4FB6.0	fGB	E-2169
U+4FB6.0	fR/S	 9.7
U+4FB6.0	fTotalStrokes	 9
U+4FB6.0	fCangjie	ORHR
U+4FB6.0	fFourCorner	26260
U+4FB6.0	fHakka	  li3;  li1
U+4FB6.0	fCantonese	leoi5
U+4FB6.0	fMandarin	l3; luu3; lu3
U+4FB6.0	fEnglish	[1] companion; mate [2] associate with
U+4FB7.0	fUTF8	侷
U+4FB7.0	fCNS11643	1-5021
U+4FB7.0	fBig5	AB5D
U+4FB7.0	fGB	E-216A
U+4FB7.0	fR/S	 9.7
U+4FB7.0	fTotalStrokes	 9
U+4FB7.0	fCangjie	OSSR
U+4FB7.0	fFourCorner	27227
U+4FB7.0	fHakka	 kiuk8
U+4FB7.0	fCantonese	guk6
U+4FB7.0	fMandarin	ju2
U+4FB7.0	fEnglish	narrow; cramped; confined
U+4FB8.0	fUTF8	侸
U+4FB8.0	fCNS11643	3-2B2C
U+4FB8.0	fGB	3-313A
U+4FB8.0	fR/S	  9.7
U+4FB8.0	fTotalStrokes	 9
U+4FB8.0	fCangjie	OMRT
U+4FB8.0	fFourCorner	21210
U+4FB8.0	fHakka	 deu1; shu6; su5; shu5
U+4FB8.0	fCantonese	syu6
U+4FB8.0	fMandarin	shu4
U+4FB8.0	fEnglish	 stand erect
U+4FB9.0	fUTF8	侹
U+4FB9.0	fCNS11643	2-2930
U+4FB9.0	fBig5	CDEE
U+4FB9.0	fGB	3-3124
U+4FB9.0	fR/S	 9.7
U+4FB9.0	fTotalStrokes	 9
U+4FB9.0	fCangjie	ONKG
U+4FB9.0	fFourCorner	22241
U+4FB9.0	fHakka	 tin3
U+4FB9.0	fCantonese	ting5; ting2
U+4FB9.0	fMandarin	ting3; ding4; ting4
U+4FBA.0	fUTF8	侺
U+4FBA.0	fCNS11643	2-292E
U+4FBA.0	fBig5	CDEC
U+4FBA.0	fGB	5-3221
U+4FBA.0	fR/S	 9.7
U+4FBA.0	fTotalStrokes	 9
U+4FBA.0	fCangjie	OUON
U+4FBA.0	fFourCorner	22227
U+4FBA.0	fHakka	 shim6; siim5; shim5
U+4FBA.0	fCantonese	sam6; sam4
U+4FBA.0	fMandarin	shen4
U+4FBA.0	fEnglish	bent; stooping
U+4FBB.0	fUTF8	侻
U+4FBB.0	fCNS11643	2-2929
U+4FBB.0	fBig5	CDE7
U+4FBB.0	fGB	3-3145
U+4FBB.0	fR/S	 9.7
U+4FBB.0	fTotalStrokes	 9
U+4FBB.0	fCangjie	OCRU
U+4FBB.0	fFourCorner	28216
U+4FBB.0	fHakka	  tot7
U+4FBB.0	fCantonese	teoi3; tyut3
U+4FBB.0	fMandarin	tuo1; tui4
U+4FBB.0	fEnglish	cunning; artful
U+4FBC.0	fUTF8	侼
U+4FBC.0	fCNS11643	4-2838
U+4FBC.0	fGB	3-3138
U+4FBC.0	fR/S	  9.7
U+4FBC.0	fTotalStrokes	 9
U+4FBC.0	fCangjie	OJBD
U+4FBC.0	fFourCorner	24240
U+4FBC.0	fCantonese	but6
U+4FBC.0	fMandarin	bo2
U+4FBD.0	fUTF8	侽
U+4FBD.0	fCNS11643	4-283E
U+4FBD.0	fGB	E-216B
U+4FBD.0	fR/S	 9.7
U+4FBD.0	fTotalStrokes	 9
U+4FBD.0	fCangjie	OWKS
U+4FBD.0	fFourCorner	26220
U+4FBD.0	fHakka	 nam2
U+4FBD.0	fCantonese	naam4; laam4 (also)
U+4FBD.0	fMandarin	nan2
U+4FBE.0	fUTF8	侾
U+4FBE.0	fCNS11643	3-2B2B
U+4FBE.0	fGB	E-216C
U+4FBE.0	fR/S	  9.7
U+4FBE.0	fTotalStrokes	 9
U+4FBE.0	fCangjie	OJKD
U+4FBE.0	fFourCorner	24240
U+4FBE.0	fCantonese	haau1
U+4FBE.0	fMandarin	hao1
U+4FBF.0	fUTF8	便
U+4FBF.0	fCNS11643	1-4F6D
U+4FBF.0	fBig5	AB4B
U+4FBF.0	fGB	0-3163
U+4FBF.0	fR/S	 9.7
U+4FBF.0	fTotalStrokes	 9
U+4FBF.0	fCangjie	OMLK
U+4FBF.0	fFourCorner	21246
U+4FBF.0	fHakka	  pien5; pen2; pien1; pien6; p'ian5; pen3; pen5; p'ian6; p'ian3
U+4FBF.0	fCantonese	bin6; bin2 (also)
U+4FBF.0	fMandarin	bian4; pian4
U+4FBF.0	fEnglish	[1] expedient; convenient; handy [2] fitting; appropriate [3] in that case; even if [4] then [5] advantageous [6] excrement and urine; to relieve oneself [7] informal; at ease; plain; ordinary
U+4FBF.1	fUTF8	便
U+4FBF.1	fCNS11643	1-4F6D
U+4FBF.1	fBig5	AB4B
U+4FBF.1	fGB	0-3163
U+4FBF.1	fR/S	 9.7
U+4FBF.1	fTotalStrokes	 9
U+4FBF.1	fCangjie	OMLK
U+4FBF.1	fFourCorner	21246
U+4FBF.1	fHakka	 pien2
U+4FBF.1	fCantonese	pin4
U+4FBF.1	fMandarin	pian2
U+4FBF.1	fEnglish	[1] cheap; inexpensive [2] a Chinese family name
U+4FC0.0	fUTF8	俀
U+4FC0.0	fCNS11643	2-292F
U+4FC0.0	fBig5	CDED
U+4FC0.0	fGB	5-3225
U+4FC0.0	fR/S	 9.7
U+4FC0.0	fTotalStrokes	 9
U+4FC0.0	fCangjie	OBV
U+4FC0.0	fFourCorner	22243
U+4FC0.0	fCantonese	teoi2
U+4FC0.0	fMandarin	tui3; tui2
U+4FC1.0	fUTF8	俁
U+4FC1.0	fCNS11643	2-2925
U+4FC1.0	fBig5	CDE3
U+4FC1.0	fGB	E-216D
U+4FC1.0	fR/S	 9.7
U+4FC1.0	fTotalStrokes	 9
U+4FC1.0	fCangjie	ORVK
U+4FC1.0	fFourCorner	26234
U+4FC1.0	fHakka	 ng2;  ngu3; ngi3
U+4FC1.0	fCantonese	jyu5
U+4FC1.0	fMandarin	yu3
U+4FC1.0	fEnglish	big
U+4FC2.0	fUTF8	係
U+4FC2.0	fCNS11643	1-4F7B
U+4FC2.0	fBig5	AB59
U+4FC2.0	fGB	1-7872
U+4FC2.0	fR/S	 9.7
U+4FC2.0	fTotalStrokes	 9
U+4FC2.0	fCangjie	OHVF
U+4FC2.0	fFourCorner	22293
U+4FC2.0	fHakka	  he5; he6
U+4FC2.0	fCantonese	hai6; hai2 (also)
U+4FC2.0	fMandarin	xi4
U+4FC2.0	fEnglish	[1] bind; belong to; attach to; connect with [2] relation; relationship; consequence [3] be [4] a Chinese family name
U+4FC3.0	fUTF8	促
U+4FC3.0	fCNS11643	1-4F72
U+4FC3.0	fBig5	AB50
U+4FC3.0	fGB	0-3459
U+4FC3.0	fR/S	 9.7
U+4FC3.0	fTotalStrokes	 9
U+4FC3.0	fCangjie	ORYO
U+4FC3.0	fFourCorner	26281
U+4FC3.0	fHakka	  cuk7
U+4FC3.0	fCantonese	cuk1
U+4FC3.0	fMandarin	cu4; chuo4
U+4FC3.0	fEnglish	[1] close; crowded; near [2] urge; press; hurry; promote [3] hurried; urgent
U+4FC4.0	fUTF8	俄
U+4FC4.0	fCNS11643	1-4F7A
U+4FC4.0	fBig5	AB58
U+4FC4.0	fGB	0-366D
U+4FC4.0	fR/S	 9.7
U+4FC4.0	fTotalStrokes	 9
U+4FC4.0	fCangjie	OHQI
U+4FC4.0	fFourCorner	23250
U+4FC4.0	fHakka	  ngo2; ngo3
U+4FC4.0	fCantonese	ngo4; ngo5; o4
U+4FC4.0	fMandarin	e2
U+4FC4.0	fEnglish	sudden; suddenly; momentarily
U+4FC4.1	fUTF8	俄
U+4FC4.1	fCNS11643	1-4F7A
U+4FC4.1	fBig5	AB58
U+4FC4.1	fGB	0-366D
U+4FC4.1	fR/S	 9.7
U+4FC4.1	fTotalStrokes	 9
U+4FC4.1	fCangjie	OHQI
U+4FC4.1	fFourCorner	23250
U+4FC4.1	fMandarin	e4
U+4FC4.1	fEnglish	Russian
U+4FC5.0	fUTF8	俅
U+4FC5.0	fCNS11643	2-287E
U+4FC5.0	fBig5	CDDE
U+4FC5.0	fGB	0-5934
U+4FC5.0	fR/S	 9.7
U+4FC5.0	fTotalStrokes	 9
U+4FC5.0	fCangjie	OIJE
U+4FC5.0	fFourCorner	23232
U+4FC5.0	fHakka	 kiu2
U+4FC5.0	fCantonese	kau4
U+4FC5.0	fMandarin	qiu2; di4
U+4FC5.0	fEnglish	ornamental cap
U+4FC6.0	fUTF8	俆
U+4FC6.0	fCNS11643	3-2B32
U+4FC6.0	fGB	5-3224
U+4FC6.0	fR/S	 9.7
U+4FC6.0	fTotalStrokes	 9
U+4FC6.0	fCangjie	OOMD
U+4FC6.0	fFourCorner	28290
U+4FC6.0	fCantonese	ceoi4; syu1
U+4FC6.0	fMandarin	xu3; xu2; shu1
U+4FC7.0	fUTF8	俇
U+4FC7.0	fCNS11643	2-292C
U+4FC7.0	fBig5	CDEA
U+4FC7.0	fGB	3-3143
U+4FC7.0	fR/S	 9.7
U+4FC7.0	fTotalStrokes	 9
U+4FC7.0	fCangjie	OKHG
U+4FC7.0	fFourCorner	21214
U+4FC7.0	fHakka	 kong3; kong2; k'wong3
U+4FC7.0	fCantonese	gwaang6; gwong6; gwang6
U+4FC7.0	fMandarin	kuang3; guang3; kuang1; kuang4; guang4
U+4FC7.0	fEnglish	[1] abrupt; quick [2] go far off
U+4FC7.1	fUTF8	俇
U+4FC7.1	fCNS11643	2-292C
U+4FC7.1	fBig5	CDEA
U+4FC7.1	fGB	3-3143
U+4FC7.1	fR/S	 9.7
U+4FC7.1	fTotalStrokes	 9
U+4FC7.1	fCangjie	OKHG
U+4FC7.1	fFourCorner	21214
U+4FC7.1	fCantonese	hong1
U+4FC8.0	fUTF8	俈
U+4FC8.0	fCNS11643	3-2B2F
U+4FC8.0	fGB	5-3222
U+4FC8.0	fR/S	 9.7
U+4FC8.0	fTotalStrokes	 9
U+4FC8.0	fCangjie	OHGR
U+4FC8.0	fFourCorner	24260
U+4FC8.0	fHakka	 guk7; kuk7; kwuk7
U+4FC8.0	fCantonese	guk1
U+4FC8.0	fMandarin	ku4
U+4FC8.0	fEnglish	 inform quickly urgent communication
U+4FC9.0	fUTF8	俉
U+4FC9.0	fCNS11643	2-2923
U+4FC9.0	fBig5	CDE1
U+4FC9.0	fGB	3-3139
U+4FC9.0	fR/S	 9.7
U+4FC9.0	fTotalStrokes	 9
U+4FC9.0	fCangjie	OMMR
U+4FC9.0	fFourCorner	21261
U+4FC9.0	fHakka	 ngu3
U+4FC9.0	fCantonese	ng6
U+4FC9.0	fMandarin	wu4; wu2
U+4FCA.0	fUTF8	俊
U+4FCA.0	fCNS11643	1-4F76
U+4FCA.0	fBig5	AB54
U+4FCA.0	fGB	0-3F21
U+4FCA.0	fR/S	 9.7
U+4FCA.0	fTotalStrokes	 9
U+4FCA.0	fCangjie	OICE
U+4FCA.0	fFourCorner	23247
U+4FCA.0	fHakka	  zun5; zun3
U+4FCA.0	fCantonese	zeon3
U+4FCA.0	fMandarin	jun4; zun4
U+4FCA.0	fEnglish	[1] talented; capable; superior; refined; smart; bright [2] handsome; good-looking [3] big; huge
U+4FCB.0	fUTF8	俋
U+4FCB.0	fCNS11643	2-2924
U+4FCB.0	fBig5	CDE2
U+4FCB.0	fGB	3-3140
U+4FCB.0	fR/S	 9.7
U+4FCB.0	fTotalStrokes	 9
U+4FCB.0	fCangjie	ORAU
U+4FCB.0	fFourCorner	26217
U+4FCB.0	fHakka	 jip7; rip7
U+4FCB.0	fCantonese	jap1
U+4FCB.0	fMandarin	yi4; zhi2
U+4FCB.0	fEnglish	strong
U+4FCC.0	fUTF8	俌
U+4FCC.0	fCNS11643	3-2B33
U+4FCC.0	fGB	5-317D
U+4FCC.0	fR/S	 9.7
U+4FCC.0	fTotalStrokes	 9
U+4FCC.0	fCangjie	OIJB
U+4FCC.0	fFourCorner	23220
U+4FCC.0	fCantonese	fu6
U+4FCC.0	fMandarin	fu3
U+4FCD.0	fUTF8	俍
U+4FCD.0	fCNS11643	2-287D
U+4FCD.0	fBig5	CDDD
U+4FCD.0	fGB	3-3148
U+4FCD.0	fR/S	 9.7
U+4FCD.0	fTotalStrokes	 9
U+4FCD.0	fCangjie	OIAV
U+4FCD.0	fFourCorner	23232
U+4FCD.0	fHakka	 liong2
U+4FCD.0	fCantonese	loeng4
U+4FCD.0	fMandarin	lang2; liang2
U+4FCE.0	fUTF8	俎
U+4FCE.0	fCNS11643	1-4F7D
U+4FCE.0	fBig5	AB5B
U+4FCE.0	fGB	0-595E
U+4FCE.0	fR/S	 9.7
U+4FCE.0	fTotalStrokes	 9
U+4FCE.0	fCangjie	OOBM
U+4FCE.0	fFourCorner	87810
U+4FCE.0	fHakka	 zu3; cu3; zih3; cih1
U+4FCE.0	fCantonese	zo2
U+4FCE.0	fMandarin	zu3
U+4FCE.0	fEnglish	[1] painted wooden stand used in offering rites [2] chopping board or block [3] small table or stand [4] a Chinese family name
U+4FCF.0	fUTF8	俏
U+4FCF.0	fCNS11643	1-4F70
U+4FCF.0	fBig5	AB4E
U+4FCF.0	fGB	0-474E
U+4FCF.0	fR/S	 9.7
U+4FCF.0	fTotalStrokes	 9
U+4FCF.0	fCangjie	OFB
U+4FCF.0	fFourCorner	29227
U+4FCF.0	fHakka	  ciau5; siau5
U+4FCF.0	fCantonese	ciu3
U+4FCF.0	fMandarin	qiao4; xiao1
U+4FCF.0	fEnglish	[1] like; similar; resemble; be like [2] pretty and cute; good-looking; smart; handsome
U+4FD0.0	fUTF8	俐
U+4FD0.0	fCNS11643	1-4F79
U+4FD0.0	fBig5	AB57
U+4FD0.0	fGB	0-407E
U+4FD0.0	fR/S	 9.7
U+4FD0.0	fTotalStrokes	 9
U+4FD0.0	fCangjie	OHDN
U+4FD0.0	fFourCorner	22200
U+4FD0.0	fHakka	 li5; li6; li3
U+4FD0.0	fCantonese	lei6
U+4FD0.0	fMandarin	li4
U+4FD0.0	fEnglish	[1] facile; easy and quick; smooth [2] sharp; clever [3] in good order; tidy; neat
U+4FD1.0	fUTF8	俑
U+4FD1.0	fCNS11643	1-4F6F
U+4FD1.0	fBig5	AB4D
U+4FD1.0	fGB	0-5938
U+4FD1.0	fR/S	 9.7
U+4FD1.0	fTotalStrokes	 9
U+4FD1.0	fCangjie	ONIB
U+4FD1.0	fFourCorner	27227
U+4FD1.0	fHakka	  jung3; rung3
U+4FD1.0	fCantonese	jung2; jung5
U+4FD1.0	fMandarin	yong3; tong1
U+4FD1.0	fEnglish	wooden or earthen figures of men and women buried with the dead; tomb figure; figurine
U+4FD2.0	fUTF8	俒
U+4FD2.0	fCNS11643	3-2B28
U+4FD2.0	fGB	5-3228
U+4FD2.0	fR/S	  9.7
U+4FD2.0	fTotalStrokes	 9
U+4FD2.0	fCangjie	OJMU
U+4FD2.0	fFourCorner	23210
U+4FD2.0	fHakka	 wan2; fan5; fan6; fun5
U+4FD2.0	fCantonese	wan6
U+4FD2.0	fMandarin	hun4
U+4FD2.0	fEnglish	 finish
U+4FD3.0	fUTF8	俓
U+4FD3.0	fCNS11643	2-2921
U+4FD3.0	fBig5	CDDF
U+4FD3.0	fGB	3-3077
U+4FD3.0	fR/S	 9.7
U+4FD3.0	fTotalStrokes	 9
U+4FD3.0	fCangjie	OMVM
U+4FD3.0	fFourCorner	21211
U+4FD3.0	fHakka	 gang5
U+4FD3.0	fCantonese	ging3
U+4FD3.0	fMandarin	jing4; ying2
U+4FD3.0	fEnglish	 straight pass
U+4FD4.0	fUTF8	俔
U+4FD4.0	fCNS11643	2-2926
U+4FD4.0	fBig5	CDE4
U+4FD4.0	fGB	3-305F
U+4FD4.0	fR/S	 9.7
U+4FD4.0	fTotalStrokes	 9
U+4FD4.0	fCangjie	OBUU
U+4FD4.0	fFourCorner	26210
U+4FD4.0	fHakka	 hien5
U+4FD4.0	fCantonese	jin5; jin6
U+4FD4.0	fMandarin	xian4; qian4
U+4FD4.0	fEnglish	like
U+4FD5.0	fUTF8	俕
U+4FD5.0	fCNS11643	3-2B29
U+4FD5.0	fGB	3-3147
U+4FD5.0	fR/S	  9.7
U+4FD5.0	fTotalStrokes	 9
U+4FD5.0	fCangjie	OJD
U+4FD5.0	fFourCorner	23290
U+4FD5.0	fCantonese	saam3
U+4FD5.0	fMandarin	san4
U+4FD6.0	fUTF8	俖
U+4FD6.0	fCNS11643	2-292D
U+4FD6.0	fBig5	CDEB
U+4FD6.0	fGB	5-317E
U+4FD6.0	fR/S	 9.7
U+4FD6.0	fTotalStrokes	 9
U+4FD6.0	fCangjie	OMFR
U+4FD6.0	fFourCorner	21269
U+4FD6.0	fCantonese	pui2; poi2; pui5
U+4FD6.0	fMandarin	pai3; pei3
U+4FD7.0	fUTF8	俗
U+4FD7.0	fCNS11643	1-4F77
U+4FD7.0	fBig5	AB55
U+4FD7.0	fGB	0-4B57
U+4FD7.0	fR/S	 9.7
U+4FD7.0	fTotalStrokes	 9
U+4FD7.0	fCangjie	OCOR
U+4FD7.0	fFourCorner	28268
U+4FD7.0	fHakka	  siuk8; shuk8; suk8
U+4FD7.0	fCantonese	zuk6
U+4FD7.0	fMandarin	su2
U+4FD7.0	fEnglish	[1] customs or customary [2] vulgar; unrefined [3] common; popular [4] lay (as distinguished from clerical); worldly; secular [5] tasteless; trite
U+4FD8.0	fUTF8	俘
U+4FD8.0	fCNS11643	1-4F74
U+4FD8.0	fBig5	AB52
U+4FD8.0	fGB	0-377D
U+4FD8.0	fR/S	 9.7
U+4FD8.0	fTotalStrokes	 9
U+4FD8.0	fCangjie	OBND
U+4FD8.0	fFourCorner	22247
U+4FD8.0	fHakka	  fu1
U+4FD8.0	fCantonese	fu1
U+4FD8.0	fMandarin	fu2
U+4FD8.0	fEnglish	[1] prisoner of war [2] capture; take prisoner
U+4FD9.0	fUTF8	俙
U+4FD9.0	fCNS11643	2-2928
U+4FD9.0	fBig5	CDE6
U+4FD9.0	fGB	3-3141
U+4FD9.0	fR/S	 9.7
U+4FD9.0	fTotalStrokes	 9
U+4FD9.0	fCangjie	OKKB
U+4FD9.0	fFourCorner	24227
U+4FD9.0	fHakka	  hi1
U+4FD9.0	fCantonese	hei1
U+4FD9.0	fMandarin	xi1; xi3
U+4FD9.0	fEnglish	 pretend; counterfeit
U+4FDA.0	fUTF8	俚
U+4FDA.0	fCNS11643	1-4F7C
U+4FDA.0	fBig5	AB5A
U+4FDA.0	fGB	0-5935
U+4FDA.0	fR/S	 9.7
U+4FDA.0	fTotalStrokes	 9
U+4FDA.0	fCangjie	OWG
U+4FDA.0	fFourCorner	26214
U+4FDA.0	fHakka	 li3;  li1
U+4FDA.0	fCantonese	lei5
U+4FDA.0	fMandarin	li3
U+4FDA.0	fEnglish	[1] rustic; vulgar; unpolished [2] small town or village; tribe (aborigines, etc.) [3] he; she
U+4FDB.0	fUTF8	俛
U+4FDB.0	fCNS11643	2-292B
U+4FDB.0	fBig5	CDE9
U+4FDB.0	fGB	E-216E
U+4FDB.0	fR/S	 9.7
U+4FDB.0	fTotalStrokes	 9
U+4FDB.0	fCangjie	ONAU
U+4FDB.0	fFourCorner	27216
U+4FDB.0	fHakka	 fu3;  mien3; men3; men1
U+4FDB.0	fCantonese	min5; fu2
U+4FDB.0	fMandarin	mian3; fu3; fu2
U+4FDB.0	fEnglish	 make effort; endeavor lower the head
U+4FDB.1	fUTF8	俛
U+4FDB.1	fCNS11643	2-292B
U+4FDB.1	fBig5	CDE9
U+4FDB.1	fGB	E-216E
U+4FDB.1	fR/S	 9.7
U+4FDB.1	fTotalStrokes	 9
U+4FDB.1	fCangjie	ONAU
U+4FDB.1	fFourCorner	27216
U+4FDC.0	fUTF8	俜
U+4FDC.0	fCNS11643	2-2927
U+4FDC.0	fBig5	CDE5
U+4FDC.0	fGB	0-5937
U+4FDC.0	fR/S	 9.7
U+4FDC.0	fTotalStrokes	 9
U+4FDC.0	fCangjie	OLWS
U+4FDC.0	fFourCorner	25227
U+4FDC.0	fHakka	 pin2; pin5
U+4FDC.0	fCantonese	ping1
U+4FDC.0	fMandarin	ping1; ping2
U+4FDC.0	fEnglish	 trust to send a message
U+4FDD.0	fUTF8	保
U+4FDD.0	fCNS11643	1-4F71
U+4FDD.0	fBig5	AB4F
U+4FDD.0	fGB	0-3123
U+4FDD.0	fR/S	 9.7
U+4FDD.0	fTotalStrokes	 9
U+4FDD.0	fCangjie	ORD
U+4FDD.0	fFourCorner	26294
U+4FDD.0	fHakka	  bau3;  bo3
U+4FDD.0	fCantonese	bou2
U+4FDD.0	fMandarin	bao3
U+4FDD.0	fEnglish	[1] protect; guard; safeguard; shelter; defend [2] be responsible; guarantee; insure [3] waiter; tender [4] care for [5] keep; maintain [6] stand guarantor or surety for someone [7] an official post in ancient China
U+4FDE.0	fUTF8	俞
U+4FDE.0	fCNS11643	1-4F7E
U+4FDE.0	fBig5	AB5C
U+4FDE.0	fGB	0-5361
U+4FDE.0	fR/S	 9.7
U+4FDE.0	fTotalStrokes	 9
U+4FDE.0	fCangjie	OMBN
U+4FDE.0	fFourCorner	80221
U+4FDE.0	fHakka	 ji3; ji2; i2
U+4FDE.0	fCantonese	jyu4; jyu6; syu3
U+4FDE.0	fMandarin	yu2; shu4; yu4
U+4FDE.0	fEnglish	 a Chinese family name consent; approve
U+4FDF.0	fUTF8	俟
U+4FDF.0	fCNS11643	1-4F75
U+4FDF.0	fBig5	AB53
U+4FDF.0	fGB	0-5939
U+4FDF.0	fR/S	 9.7
U+4FDF.0	fTotalStrokes	 9
U+4FDF.0	fCangjie	OIOK
U+4FDF.0	fFourCorner	23234
U+4FDF.0	fHakka	 sii5; su5; sii6; se5
U+4FDF.0	fCantonese	zi6
U+4FDF.0	fMandarin	si4
U+4FDF.0	fEnglish	[1] wait for; wait until; await [2] as soon as; until
U+4FDF.1	fUTF8	俟
U+4FDF.1	fCNS11643	1-4F75
U+4FDF.1	fBig5	AB53
U+4FDF.1	fGB	0-5939
U+4FDF.1	fR/S	 9.7
U+4FDF.1	fTotalStrokes	 9
U+4FDF.1	fCangjie	OIOK
U+4FDF.1	fFourCorner	23234
U+4FDF.1	fCantonese	kei4
U+4FDF.1	fMandarin	qi2
U+4FE0.0	fUTF8	俠
U+4FE0.0	fCNS11643	1-4F6E
U+4FE0.0	fBig5	AB4C
U+4FE0.0	fGB	1-4F40
U+4FE0.0	fR/S	 9.7
U+4FE0.0	fTotalStrokes	 9
U+4FE0.0	fCangjie	OKOO
U+4FE0.0	fFourCorner	24238
U+4FE0.0	fHakka	  hiap8; kiap7
U+4FE0.0	fCantonese	hap6; haap6 (also)
U+4FE0.0	fMandarin	xia2
U+4FE0.0	fEnglish	[1] person adept in martial arts and dedicated to helping the poor and weak; one who fights rather than submit to injustice; chivalrous person [2] chivalry [3] a Chinese family name
U+4FE1.0	fUTF8	信
U+4FE1.0	fCNS11643	1-4F6A
U+4FE1.0	fBig5	AB48
U+4FE1.0	fGB	0-5045
U+4FE1.0	fR/S	 9.7
U+4FE1.0	fTotalStrokes	 9
U+4FE1.0	fCangjie	OYMR
U+4FE1.0	fFourCorner	20261
U+4FE1.0	fHakka	  sin5
U+4FE1.0	fCantonese	seon3; seon2 (also)
U+4FE1.0	fMandarin	xin4
U+4FE1.0	fEnglish	[1] honest; truthfulness; faith [2] believing; true [3] believe; trust [4] envoy; emissary; messenger [5] news; information; word [6] letter; mail [7] credentials; evidence; pledge; sign [8] let; free; aimless; at will; at random; without a plan [9] regular periodical appearance
U+4FE1.1	fUTF8	信
U+4FE1.1	fCNS11643	1-4F6A
U+4FE1.1	fBig5	AB48
U+4FE1.1	fGB	0-5045
U+4FE1.1	fR/S	 9.7
U+4FE1.1	fTotalStrokes	 9
U+4FE1.1	fCangjie	OYMR
U+4FE1.1	fFourCorner	20261
U+4FE1.1	fCantonese	san1
U+4FE1.1	fMandarin	shen1
U+4FE2.0	fUTF8	俢
U+4FE2.0	fCNS11643	4-283F
U+4FE2.0	fGB	E-216F
U+4FE2.0	fR/S	 9.7
U+4FE2.0	fTotalStrokes	 9
U+4FE2.0	fCangjie	OHEH
U+4FE2.0	fFourCorner	28222
U+4FE2.0	fCantonese	sau1
U+4FE2.0	fMandarin	xiu1
U+4FE3.0	fUTF8	俣
U+4FE3.0	fCNS11643	3-2B35
U+4FE3.0	fGB	0-5936
U+4FE3.0	fR/S	 9.7
U+4FE3.0	fTotalStrokes	 9
U+4FE3.0	fCangjie	ORMK
U+4FE3.0	fFourCorner	26230
U+4FE3.0	fCantonese	jyu5
U+4FE3.0	fMandarin	yu3
U+4FE3.0	fEnglish	big
U+4FE4.0	fUTF8	俤
U+4FE4.0	fCNS11643	F-292D
U+4FE4.0	fGB	3-3146
U+4FE4.0	fR/S	 9.7
U+4FE4.0	fTotalStrokes	 9
U+4FE4.0	fCangjie	OCNH
U+4FE4.0	fFourCorner	2822
U+4FE4.0	fCantonese	dai6
U+4FE4.0	fMandarin	ti4
U+4FE4.0	fEnglish	same as 面影 U+9762 U+5F71, visage, face; trace, shadow; memory
U+4FE5.0	fUTF8	俥
U+4FE5.0	fCNS11643	3-2B34
U+4FE5.0	fGB	5-3153
U+4FE5.0	fR/S	 9.7
U+4FE5.0	fTotalStrokes	 9
U+4FE5.0	fCangjie	OJWJ
U+4FE5.0	fFourCorner	25200
U+4FE5.0	fCantonese	ce1
U+4FE5.0	fMandarin	ju1; che1
U+4FE5.0	fEnglish	 rickshaw; cart; vehicle the name of piece in Chinese chess
U+4FE6.0	fUTF8	俦
U+4FE6.0	fGB	0-5931
U+4FE6.0	fR/S	 9.7
U+4FE6.0	fTotalStrokes	 9
U+4FE6.0	fCangjie	OQKI
U+4FE6.0	fHakka	 chiu2
U+4FE6.0	fCantonese	cau4
U+4FE6.0	fMandarin	chou2
U+4FE6.0	fEnglish	simplified form of &#x5114;
U+4FE7.0	fUTF8	俧
U+4FE7.0	fCNS11643	3-644A
U+4FE7.0	fGB	E-2170
U+4FE7.0	fR/S	 9.7
U+4FE7.0	fTotalStrokes	 9
U+4FE7.0	fCangjie	XOGP
U+4FE7.0	fFourCorner	2423
U+4FE7.0	fCantonese	zi3
U+4FE8.0	fUTF8	俨
U+4FE8.0	fGB	0-5932
U+4FE8.0	fR/S	 9.7
U+4FE8.0	fTotalStrokes	 9
U+4FE8.0	fCangjie	OMTH
U+4FE8.0	fHakka	 ngiam3
U+4FE8.0	fCantonese	jim5
U+4FE8.0	fMandarin	yan3
U+4FE8.0	fEnglish	simplified form of &#x513C;
U+4FE9.0	fUTF8	俩
U+4FE9.0	fCNS11643	4-283D
U+4FE9.0	fGB	0-4129
U+4FE9.0	fR/S	 9.7
U+4FE9.0	fTotalStrokes	 9
U+4FE9.0	fCangjie	OMOB
U+4FE9.0	fFourCorner	21220
U+4FE9.0	fHakka	 liong3
U+4FE9.0	fCantonese	loeng5
U+4FE9.0	fMandarin	liang3; lia3
U+4FE9.0	fEnglish	simplified form of &#x5006;
U+4FEA.0	fUTF8	俪
U+4FEA.0	fGB	0-5933
U+4FEA.0	fR/S	 9.7
U+4FEA.0	fTotalStrokes	 9
U+4FEA.0	fCangjie	XOMBB
U+4FEA.0	fHakka	 li5
U+4FEA.0	fCantonese	lai6
U+4FEA.0	fMandarin	li4; li2
U+4FEA.0	fEnglish	simplified form of &#x5137;
U+4FEB.0	fUTF8	俫
U+4FEB.0	fGB	8-7A32
U+4FEB.0	fR/S	 9.7
U+4FEB.0	fTotalStrokes	 9
U+4FEB.0	fCangjie	ODT
U+4FEB.0	fCantonese	loi4; loi6
U+4FEB.0	fMandarin	lai2; lai4
U+4FEB.0	fEnglish	to induce to come; to encourage
U+4FEC.0	fUTF8	俬
U+4FEC.0	fCNS11643	2-2931
U+4FEC.0	fBig5	CDEF
U+4FEC.0	fGB	E-2171
U+4FEC.0	fR/S	 9.7
U+4FEC.0	fTotalStrokes	 9
U+4FEC.0	fCangjie	OHDI
U+4FEC.0	fFourCorner	22230
U+4FEC.0	fHakka	 su1
U+4FEC.0	fCantonese	si1
U+4FEC.0	fMandarin	si1
U+4FEC.0	fEnglish	傢俬, furniture
U+4FED.0	fUTF8	俭
U+4FED.0	fGB	0-3C73
U+4FED.0	fR/S	 9.7
U+4FED.0	fTotalStrokes	 9
U+4FED.0	fCangjie	OOMM
U+4FED.0	fHakka	 kiam5
U+4FED.0	fCantonese	gim6
U+4FED.0	fMandarin	jian3; jian4
U+4FED.0	fEnglish	simplified form of &#x5109;
U+4FEE.0	fUTF8	修
U+4FEE.0	fCNS11643	1-543A
U+4FEE.0	fBig5	ADD7
U+4FEE.0	fGB	0-505E
U+4FEE.0	fR/S	 9.8
U+4FEE.0	fTotalStrokes	 10
U+4FEE.0	fCangjie	OLOH
U+4FEE.0	fFourCorner	28222
U+4FEE.0	fHakka	  siu1; sieu1; siu3
U+4FEE.0	fCantonese	sau1
U+4FEE.0	fMandarin	xiu1
U+4FEE.0	fEnglish	[1] repair; mend [2] adorn; decorate [3] construct; build [4] long; slender [5] prune; cut; sharpen; trim [6] study; cultivate [7] write; compile; edit
U+4FEF.0	fUTF8	俯
U+4FEF.0	fCNS11643	1-5424
U+4FEF.0	fBig5	ADC1
U+4FEF.0	fGB	0-3829
U+4FEF.0	fR/S	 9.8
U+4FEF.0	fTotalStrokes	 10
U+4FEF.0	fCangjie	OIOI
U+4FEF.0	fFourCorner	20240
U+4FEF.0	fHakka	  fu3; fu1
U+4FEF.0	fCantonese	fu2
U+4FEF.0	fMandarin	fu3
U+4FEF.0	fEnglish	[1] bow down; face down; look down; stoop [2] condescend; deign
U+4FF0.0	fUTF8	俰
U+4FF0.0	fCNS11643	3-2F48
U+4FF0.0	fGB	5-3236
U+4FF0.0	fR/S	  9.8
U+4FF0.0	fTotalStrokes	 10
U+4FF0.0	fCangjie	OHDR
U+4FF0.0	fFourCorner	26200
U+4FF0.0	fCantonese	wo6
U+4FF0.0	fMandarin	he4; huo4
U+4FF1.0	fUTF8	俱
U+4FF1.0	fCNS11643	1-5434
U+4FF1.0	fBig5	ADD1
U+4FF1.0	fGB	0-3E63
U+4FF1.0	fR/S	 9.8
U+4FF1.0	fTotalStrokes	 10
U+4FF1.0	fCangjie	OBMC
U+4FF1.0	fFourCorner	26281
U+4FF1.0	fHakka	 gi1;  ki3;  ki1; gi3
U+4FF1.0	fCantonese	keoi1; geoi1
U+4FF1.0	fMandarin	ju4; ju1
U+4FF1.0	fEnglish	[1] all; together; altogether [2] accompany [3] a Chinese family name
U+4FF2.0	fUTF8	俲
U+4FF2.0	fCNS11643	3-2F4D
U+4FF2.0	fGB	E-2172
U+4FF2.0	fR/S	 9.8
U+4FF2.0	fTotalStrokes	 10
U+4FF2.0	fCangjie	OYKS
U+4FF2.0	fFourCorner	24220
U+4FF2.0	fCantonese	haau6
U+4FF2.0	fMandarin	xiao4
U+4FF2.0	fEnglish	same as &#x509A;
U+4FF3.0	fUTF8	俳
U+4FF3.0	fCNS11643	1-5439
U+4FF3.0	fBig5	ADD6
U+4FF3.0	fGB	0-593D
U+4FF3.0	fR/S	 9.8
U+4FF3.0	fTotalStrokes	 10
U+4FF3.0	fCangjie	OLMY
U+4FF3.0	fFourCorner	21211
U+4FF3.0	fHakka	 pai2; fui1; pai3
U+4FF3.0	fCantonese	paai4; pui4
U+4FF3.0	fMandarin	pai2
U+4FF3.0	fEnglish	[1] variety show; vaudeville [2] insincere; not serious [3] walk to and fro
U+4FF4.0	fUTF8	俴
U+4FF4.0	fCNS11643	2-2D71
U+4FF4.0	fBig5	D0D0
U+4FF4.0	fGB	3-306A
U+4FF4.0	fR/S	 9.8
U+4FF4.0	fTotalStrokes	 10
U+4FF4.0	fCangjie	OII
U+4FF4.0	fFourCorner	23253
U+4FF4.0	fHakka	  cien3
U+4FF4.0	fCantonese	cin5; zin1; zin5
U+4FF4.0	fMandarin	jian4; jian1
U+4FF4.0	fEnglish	thin
U+4FF5.0	fUTF8	俵
U+4FF5.0	fCNS11643	2-2D70
U+4FF5.0	fBig5	D0CF
U+4FF5.0	fGB	3-3149
U+4FF5.0	fR/S	 9.8
U+4FF5.0	fTotalStrokes	 10
U+4FF5.0	fCangjie	OQMV
U+4FF5.0	fFourCorner	25232
U+4FF5.0	fHakka	 biau3; beu3
U+4FF5.0	fCantonese	biu2; biu3
U+4FF5.0	fMandarin	biao4; biao3
U+4FF5.0	fEnglish	divide; distribute
U+4FF6.0	fUTF8	俶
U+4FF6.0	fCNS11643	2-2D75
U+4FF6.0	fBig5	D0D4
U+4FF6.0	fGB	3-3151
U+4FF6.0	fR/S	 9.8
U+4FF6.0	fTotalStrokes	 10
U+4FF6.0	fCangjie	OYFE
U+4FF6.0	fFourCorner	27240
U+4FF6.0	fHakka	 shuk7; suk7
U+4FF6.0	fCantonese	cuk1; tik1; suk6
U+4FF6.0	fMandarin	chu4
U+4FF6.0	fEnglish	[1] start; begin; beginning [2] arrange the baggage for travel [3] build
U+4FF6.1	fUTF8	俶
U+4FF6.1	fCNS11643	2-2D75
U+4FF6.1	fBig5	D0D4
U+4FF6.1	fGB	3-3151
U+4FF6.1	fR/S	 9.8
U+4FF6.1	fTotalStrokes	 10
U+4FF6.1	fCangjie	OYFE
U+4FF6.1	fFourCorner	27240
U+4FF6.1	fMandarin	ti4
U+4FF6.1	fEnglish	not bound by conventions
U+4FF7.0	fUTF8	俷
U+4FF7.0	fCNS11643	2-2D76
U+4FF7.0	fBig5	D0D5
U+4FF7.0	fGB	5-323A
U+4FF7.0	fR/S	 9.8
U+4FF7.0	fTotalStrokes	 10
U+4FF7.0	fCangjie	OBAU
U+4FF7.0	fFourCorner	27217
U+4FF7.0	fCantonese	fai3; fei4
U+4FF7.0	fMandarin	fei4
U+4FF8.0	fUTF8	俸
U+4FF8.0	fCNS11643	1-5427
U+4FF8.0	fBig5	ADC4
U+4FF8.0	fGB	0-593A
U+4FF8.0	fR/S	 9.8
U+4FF8.0	fTotalStrokes	 10
U+4FF8.0	fCangjie	OQKQ
U+4FF8.0	fFourCorner	25253
U+4FF8.0	fHakka	  fung3;  bung3;  fung5; fung6
U+4FF8.0	fCantonese	fung6; fung2 (also)
U+4FF8.0	fMandarin	feng4; beng3
U+4FF8.0	fEnglish	wages; government salary; emoluments
U+4FF9.0	fUTF8	俹
U+4FF9.0	fCNS11643	3-2F44
U+4FF9.0	fGB	3-307A
U+4FF9.0	fR/S	 9.8
U+4FF9.0	fTotalStrokes	 10
U+4FF9.0	fCangjie	OMLM
U+4FF9.0	fFourCorner	21210
U+4FF9.0	fHakka	 a1; a5
U+4FF9.0	fCantonese	aa3; ngaa3 (also)
U+4FF9.0	fMandarin	ya4
U+4FFA.0	fUTF8	俺
U+4FFA.0	fCNS11643	1-5430
U+4FFA.0	fBig5	ADCD
U+4FFA.0	fGB	0-3033
U+4FFA.0	fR/S	 9.8
U+4FFA.0	fTotalStrokes	 10
U+4FFA.0	fCangjie	OKLU
U+4FFA.0	fFourCorner	24216
U+4FFA.0	fHakka	  jam3; en1; an1; ram3
U+4FFA.0	fCantonese	aan2; jim3 (also); ngaan2 (also); an2
U+4FFA.0	fMandarin	an3; yan4
U+4FFA.0	fEnglish	the personal pronoun I, used especially in Shantung province
U+4FFB.0	fUTF8	俻
U+4FFB.0	fCNS11643	3-2F58
U+4FFB.0	fGB	E-2173
U+4FFB.0	fR/S	 9.8
U+4FFB.0	fTotalStrokes	 10
U+4FFB.0	fCangjie	OHEW
U+4FFB.0	fFourCorner	27260
U+4FFB.0	fCantonese	bei6
U+4FFB.0	fMandarin	bei4
U+4FFB.0	fEnglish	 prepare provide put in order complete perfection completeness
U+4FFC.0	fUTF8	俼
U+4FFC.0	fCNS11643	3-2F4C
U+4FFC.0	fGB	E-2174
U+4FFC.0	fR/S	 9.8
U+4FFC.0	fTotalStrokes	 10
U+4FFC.0	fCangjie	OYIB
U+4FFC.0	fFourCorner	20227
U+4FFC.0	fCantonese	juk6
U+4FFC.0	fMandarin	yu4; zhou1
U+4FFD.0	fUTF8	俽
U+4FFD.0	fCNS11643	3-2F56
U+4FFD.0	fGB	5-3237
U+4FFD.0	fR/S	 9.8
U+4FFD.0	fTotalStrokes	 10
U+4FFD.0	fCangjie	OHLO
U+4FFD.0	fFourCorner	27280
U+4FFD.0	fCantonese	jan1
U+4FFD.0	fMandarin	xin1
U+4FFE.0	fUTF8	俾
U+4FFE.0	fCNS11643	1-543D
U+4FFE.0	fBig5	ADDA
U+4FFE.0	fGB	0-5942
U+4FFE.0	fR/S	 9.8
U+4FFE.0	fTotalStrokes	 10
U+4FFE.0	fCangjie	OHWJ
U+4FFE.0	fFourCorner	26240
U+4FFE.0	fHakka	  bi1
U+4FFE.0	fCantonese	bei2; pai3
U+4FFE.0	fMandarin	bi3; bei1; pi4
U+4FFE.0	fEnglish	[1] cause; enable [2] that; so that; in order that [3] give; pay (salary, wages) [4] serve with (sugar, salt, etc.) [5] allow; permit [6] to (in the sense of giving)
U+4FFF.0	fUTF8	俿
U+4FFF.0	fCNS11643	3-2F46
U+4FFF.0	fGB	5-322F
U+4FFF.0	fR/S	 9.8
U+4FFF.0	fTotalStrokes	 10
U+4FFF.0	fCangjie	OYPU
U+4FFF.0	fFourCorner	21210
U+4FFF.0	fCantonese	fu2
U+4FFF.0	fMandarin	jian4; hu3; chi2
U+5000.0	fUTF8	倀
U+5000.0	fCNS11643	1-5431
U+5000.0	fBig5	ADCE
U+5000.0	fGB	1-5876
U+5000.0	fR/S	 9.8
U+5000.0	fTotalStrokes	 10
U+5000.0	fCangjie	OSMV
U+5000.0	fFourCorner	21232
U+5000.0	fHakka	 chong2; cong2; zhong5; zong5
U+5000.0	fCantonese	coeng1; caang1 (also); caang4; zan3; cang1; zaang1
U+5000.0	fMandarin	chang1
U+5000.0	fEnglish	[1] a ghost of controlled by the tiger [2] wild; rash
U+5000.1	fUTF8	倀
U+5000.1	fCNS11643	1-5431
U+5000.1	fBig5	ADCE
U+5000.1	fGB	1-5876
U+5000.1	fR/S	 9.8
U+5000.1	fTotalStrokes	 10
U+5000.1	fCangjie	OSMV
U+5000.1	fFourCorner	21232
U+5000.1	fCantonese	cang4
U+5000.1	fMandarin	cheng2
U+5000.2	fUTF8	倀
U+5000.2	fCNS11643	1-5431
U+5000.2	fBig5	ADCE
U+5000.2	fGB	1-5876
U+5000.2	fR/S	 9.8
U+5000.2	fTotalStrokes	 10
U+5000.2	fCangjie	OSMV
U+5000.2	fFourCorner	21232
U+5000.2	fMandarin	zheng4
U+5001.0	fUTF8	倁
U+5001.0	fCNS11643	4-2B62
U+5001.0	fGB	E-2175
U+5001.0	fR/S	 9.8
U+5001.0	fTotalStrokes	 10
U+5001.0	fCangjie	OOKR
U+5001.0	fFourCorner	26200
U+5001.0	fCantonese	zi1
U+5001.0	fMandarin	chi2; zhi1
U+5002.0	fUTF8	倂
U+5002.0	fCNS11643	3-2F4B
U+5002.0	fGB	E-2176
U+5002.0	fR/S	 9.8
U+5002.0	fTotalStrokes	 10
U+5002.0	fCangjie	XOYJJ
U+5002.0	fFourCorner	23240
U+5002.0	fHakka	 bin5
U+5002.0	fCantonese	bing3; ping3 (also)
U+5002.0	fMandarin	bing4
U+5002.0	fEnglish	[1] on a level with; even; equal; go side by side [2] all; entire [3] together [4] combine; annex
U+5003.0	fUTF8	倃
U+5003.0	fCNS11643	3-2F49
U+5003.0	fGB	E-2177
U+5003.0	fR/S	 9.8
U+5003.0	fTotalStrokes	 10
U+5003.0	fCangjie	OHOR
U+5003.0	fFourCorner	28264
U+5003.0	fHakka	  giu5
U+5003.0	fCantonese	gau6
U+5003.0	fMandarin	jiu4; zan2; za2
U+5003.0	fEnglish	damage; destroy
U+5004.0	fUTF8	倄
U+5004.0	fCNS11643	3-2F47
U+5004.0	fGB	3-3153
U+5004.0	fR/S	  9.8
U+5004.0	fTotalStrokes	 10
U+5004.0	fCangjie	OKKB
U+5004.0	fFourCorner	24220
U+5004.0	fCantonese	haau4
U+5004.0	fMandarin	yao2
U+5005.0	fUTF8	倅
U+5005.0	fCNS11643	2-2D6A
U+5005.0	fBig5	D0C9
U+5005.0	fGB	3-3157
U+5005.0	fR/S	 9.8
U+5005.0	fTotalStrokes	 10
U+5005.0	fCangjie	OYOJ
U+5005.0	fFourCorner	20248
U+5005.0	fHakka	  cui5;  zut7
U+5005.0	fCantonese	ceoi3; zeot1 (also); cyut3
U+5005.0	fMandarin	cui4; zu2
U+5005.0	fEnglish	deputy; vice-
U+5006.0	fUTF8	倆
U+5006.0	fCNS11643	1-542A
U+5006.0	fBig5	ADC7
U+5006.0	fGB	1-4129
U+5006.0	fR/S	 9.8
U+5006.0	fTotalStrokes	 10
U+5006.0	fCangjie	OMLB
U+5006.0	fFourCorner	21227
U+5006.0	fHakka	 liong3
U+5006.0	fCantonese	loeng5
U+5006.0	fMandarin	liang3
U+5006.0	fEnglish	craft; ability
U+5006.1	fUTF8	倆
U+5006.1	fCNS11643	1-542A
U+5006.1	fBig5	ADC7
U+5006.1	fGB	1-4129
U+5006.1	fR/S	 9.8
U+5006.1	fTotalStrokes	 10
U+5006.1	fCangjie	OMLB
U+5006.1	fFourCorner	21227
U+5006.1	fMandarin	lia3
U+5006.1	fEnglish	two; pair; couple
U+5007.0	fUTF8	倇
U+5007.0	fCNS11643	2-2D6B
U+5007.0	fBig5	D0CA
U+5007.0	fGB	3-315B
U+5007.0	fR/S	 9.8
U+5007.0	fTotalStrokes	 10
U+5007.0	fCangjie	OJNU
U+5007.0	fFourCorner	23212
U+5007.0	fCantonese	jyun2
U+5007.0	fMandarin	wan3
U+5008.0	fUTF8	倈
U+5008.0	fCNS11643	3-2F4F
U+5008.0	fGB	3-313C
U+5008.0	fR/S	  9.8
U+5008.0	fTotalStrokes	 10
U+5008.0	fCangjie	ODOO
U+5008.0	fFourCorner	24290
U+5008.0	fHakka	 loi3; loi5
U+5008.0	fCantonese	loi4; loi6
U+5008.0	fMandarin	lai2; lai4
U+5008.0	fEnglish	 induce to come encourage
U+5009.0	fUTF8	倉
U+5009.0	fCNS11643	1-543F
U+5009.0	fBig5	ADDC
U+5009.0	fGB	1-3256
U+5009.0	fR/S	 9.8
U+5009.0	fTotalStrokes	 10
U+5009.0	fCangjie	OIAR
U+5009.0	fFourCorner	80607
U+5009.0	fHakka	  cong1
U+5009.0	fCantonese	cong1
U+5009.0	fMandarin	cang1; chuan4; chuang4
U+5009.0	fEnglish	[1] granary; storehouse; warehouse [2] cabin, as in the ship [3] green [4] a Chinese family name
U+500A.0	fUTF8	倊
U+500A.0	fCNS11643	4-2B64
U+500A.0	fGB	E-2178
U+500A.0	fR/S	  9.8
U+500A.0	fTotalStrokes	 10
U+500A.0	fCangjie	OCIP
U+500A.0	fFourCorner	28230
U+500A.0	fCantonese	zung3
U+500A.0	fMandarin	zong4
U+500B.0	fUTF8	個
U+500B.0	fCNS11643	1-5436
U+500B.0	fBig5	ADD3
U+500B.0	fGB	1-3876
U+500B.0	fR/S	 9.8
U+500B.0	fTotalStrokes	 10
U+500B.0	fCangjie	OWJR
U+500B.0	fFourCorner	26200
U+500B.0	fHakka	 gai5;  go5; ge5; ga5; ke5
U+500B.0	fCantonese	go3; go2 (also)
U+500B.0	fMandarin	ge4
U+500B.0	fEnglish	[1] a numerary adjunct [2] piece [3] single [4] roughly [5] an adjunct to an indefinite pronoun, as this, that
U+500B.1	fUTF8	個
U+500B.1	fCNS11643	1-5436
U+500B.1	fBig5	ADD3
U+500B.1	fGB	1-3876
U+500B.1	fR/S	 9.8
U+500B.1	fTotalStrokes	 10
U+500B.1	fCangjie	OWJR
U+500B.1	fFourCorner	26200
U+500B.1	fMandarin	ge3
U+500B.2	fUTF8	個
U+500B.2	fCNS11643	1-5436
U+500B.2	fBig5	ADD3
U+500B.2	fGB	1-3876
U+500B.2	fR/S	 9.8
U+500B.2	fTotalStrokes	 10
U+500B.2	fCangjie	OWJR
U+500B.2	fFourCorner	26200
U+500B.2	fMandarin	ge5
U+500C.0	fUTF8	倌
U+500C.0	fCNS11643	1-5421
U+500C.0	fBig5	ADBE
U+500C.0	fGB	0-5944
U+500C.0	fR/S	 9.8
U+500C.0	fTotalStrokes	 10
U+500C.0	fCangjie	OJRR
U+500C.0	fFourCorner	23277
U+500C.0	fHakka	 gon1; gon3; gwon1; kwan1
U+500C.0	fCantonese	gun1; gwun1 (also)
U+500C.0	fMandarin	guan1; guan4
U+500C.0	fEnglish	[1] boy or an assistant in the employ of a teahouse, tavern or restaurant [2] an euphemism for a prostitute [3] groom
U+500D.0	fUTF8	倍
U+500D.0	fCNS11643	1-5422
U+500D.0	fBig5	ADBF
U+500D.0	fGB	0-3136
U+500D.0	fR/S	 9.8
U+500D.0	fTotalStrokes	 10
U+500D.0	fCangjie	OYTR
U+500D.0	fFourCorner	20261
U+500D.0	fHakka	  pui5;  pi5; poi6; pui3; pui6; poi5
U+500D.0	fCantonese	pui5; bui3
U+500D.0	fMandarin	bei4; pei2
U+500D.0	fEnglish	[1] double; double [2] times; fold [3] rebel; insubordinate
U+500D.1	fUTF8	倍
U+500D.1	fCNS11643	1-5422
U+500D.1	fBig5	ADBF
U+500D.1	fGB	0-3136
U+500D.1	fR/S	 9.8
U+500D.1	fTotalStrokes	 10
U+500D.1	fCangjie	OYTR
U+500D.1	fFourCorner	20261
U+500E.0	fUTF8	倎
U+500E.0	fCNS11643	2-2D7E
U+500E.0	fBig5	D0DD
U+500E.0	fGB	5-3232
U+500E.0	fR/S	 9.8
U+500E.0	fTotalStrokes	 10
U+500E.0	fCangjie	OTBC
U+500E.0	fFourCorner	25281
U+500E.0	fCantonese	tin2
U+500E.0	fMandarin	tian1
U+500F.0	fUTF8	倏
U+500F.0	fCNS11643	1-5924
U+500F.0	fBig5	B0BF
U+500F.0	fGB	0-593F
U+500F.0	fR/S	 9.8
U+500F.0	fTotalStrokes	 10
U+500F.0	fCangjie	OLOK
U+500F.0	fFourCorner	27234
U+500F.0	fHakka	 suk7;  shuk7; shap7; sap7
U+500F.0	fCantonese	suk1
U+500F.0	fMandarin	shu1; shu4
U+500F.0	fEnglish	hastily; suddenly
U+5010.0	fUTF8	倐
U+5010.0	fCNS11643	3-2F53
U+5010.0	fGB	E-2179
U+5010.0	fR/S	  9.8
U+5010.0	fTotalStrokes	 10
U+5010.0	fCangjie	OLHF
U+5010.0	fFourCorner	28280
U+5010.0	fHakka	 shuk7; shap7; sap7; suk7
U+5010.0	fCantonese	suk1
U+5010.0	fMandarin	shu1; tiao2
U+5010.0	fEnglish	 hastily suddenly
U+5011.0	fUTF8	們
U+5011.0	fCNS11643	1-542F
U+5011.0	fBig5	ADCC
U+5011.0	fGB	1-4347
U+5011.0	fR/S	 9.8
U+5011.0	fTotalStrokes	 10
U+5011.0	fCangjie	OAN
U+5011.0	fFourCorner	27220
U+5011.0	fHakka	 mun2; mun3
U+5011.0	fCantonese	mun4
U+5011.0	fMandarin	men5; men2
U+5011.0	fEnglish	an adjunct to a pronoun or noun to indicate plurality
U+5012.0	fUTF8	倒
U+5012.0	fCNS11643	1-542E
U+5012.0	fBig5	ADCB
U+5012.0	fGB	0-3539
U+5012.0	fR/S	 9.8
U+5012.0	fTotalStrokes	 10
U+5012.0	fCangjie	OMGN
U+5012.0	fFourCorner	22200
U+5012.0	fHakka	 do3;  do5;  dau5; dau3
U+5012.0	fCantonese	dou3
U+5012.0	fMandarin	dao4
U+5012.0	fEnglish	[1] inverse; place upside down; in reverse order or the wrong direction [2] pour out; empty [3] on the contrary [4] after all [5] but; and yet
U+5012.1	fUTF8	倒
U+5012.1	fCNS11643	1-542E
U+5012.1	fBig5	ADCB
U+5012.1	fGB	0-3539
U+5012.1	fR/S	 9.8
U+5012.1	fTotalStrokes	 10
U+5012.1	fCangjie	OMGN
U+5012.1	fFourCorner	22200
U+5012.1	fHakka	 dau3
U+5012.1	fCantonese	dou2
U+5012.1	fMandarin	dao3
U+5012.1	fEnglish	 fall down; lie down
U+5013.0	fUTF8	倓
U+5013.0	fCNS11643	2-2D6C
U+5013.0	fBig5	D0CB
U+5013.0	fGB	3-3159
U+5013.0	fR/S	 9.8
U+5013.0	fTotalStrokes	 10
U+5013.0	fCangjie	OFF
U+5013.0	fFourCorner	29289
U+5013.0	fHakka	  tam2; ram2; jam2
U+5013.0	fCantonese	taam4; daam6
U+5013.0	fMandarin	tan2
U+5013.0	fEnglish	quiet; peaceful
U+5013.1	fUTF8	倓
U+5013.1	fCNS11643	2-2D6C
U+5013.1	fBig5	D0CB
U+5013.1	fGB	3-3159
U+5013.1	fR/S	 9.8
U+5013.1	fTotalStrokes	 10
U+5013.1	fCangjie	OFF
U+5013.1	fFourCorner	29289
U+5014.0	fUTF8	倔
U+5014.0	fCNS11643	1-5432
U+5014.0	fBig5	ADCF
U+5014.0	fGB	0-3E73
U+5014.0	fR/S	 9.8
U+5014.0	fTotalStrokes	 10
U+5014.0	fCangjie	OSUU
U+5014.0	fFourCorner	27272
U+5014.0	fHakka	  kut7; kiut7
U+5014.0	fCantonese	gwat6
U+5014.0	fMandarin	jue2
U+5014.0	fEnglish	hard; stubborn; obstinate; intransigent
U+5014.1	fUTF8	倔
U+5014.1	fCNS11643	1-5432
U+5014.1	fBig5	ADCF
U+5014.1	fGB	0-3E73
U+5014.1	fR/S	 9.8
U+5014.1	fTotalStrokes	 10
U+5014.1	fCangjie	OSUU
U+5014.1	fFourCorner	27272
U+5014.1	fMandarin	jue4
U+5014.1	fEnglish	gruff; surly; rude in manner or speech
U+5015.0	fUTF8	倕
U+5015.0	fCNS11643	2-335E
U+5015.0	fBig5	D45B
U+5015.0	fGB	5-3235
U+5015.0	fR/S	 9.8
U+5015.0	fTotalStrokes	 10
U+5015.0	fCangjie	OHJM
U+5015.0	fFourCorner	22214
U+5015.0	fHakka	 shui2; sui2
U+5015.0	fCantonese	seoi4
U+5015.0	fMandarin	chui2
U+5015.0	fEnglish	name of an expert craftsman in ancient China
U+5016.0	fUTF8	倖
U+5016.0	fCNS11643	1-5429
U+5016.0	fBig5	ADC6
U+5016.0	fGB	E-217A
U+5016.0	fR/S	 9.8
U+5016.0	fTotalStrokes	 10
U+5016.0	fCangjie	OGTJ
U+5016.0	fFourCorner	24241
U+5016.0	fHakka	 hen5; hen6; hen3
U+5016.0	fCantonese	hang6
U+5016.0	fMandarin	xing4
U+5016.0	fEnglish	[1] good luck; lucky; by luck or chance; fortunate [2] dote on; spoil
U+5017.0	fUTF8	倗
U+5017.0	fCNS11643	2-2D77
U+5017.0	fBig5	D0D6
U+5017.0	fGB	3-3155
U+5017.0	fR/S	 9.8
U+5017.0	fTotalStrokes	 10
U+5017.0	fCangjie	OBB
U+5017.0	fFourCorner	27220
U+5017.0	fHakka	  pen2
U+5017.0	fCantonese	pang4; paang4
U+5017.0	fMandarin	peng2
U+5017.0	fEnglish	[1] associate with [2] help
U+5018.0	fUTF8	倘
U+5018.0	fCNS11643	1-5438
U+5018.0	fBig5	ADD5
U+5018.0	fGB	0-4C48
U+5018.0	fR/S	 9.8
U+5018.0	fTotalStrokes	 10
U+5018.0	fCangjie	OFBR
U+5018.0	fFourCorner	29227
U+5018.0	fHakka	  tong3; song3
U+5018.0	fCantonese	tong2
U+5018.0	fMandarin	tang3; chang2
U+5018.0	fEnglish	if; supposing; in the event of
U+5018.1	fUTF8	倘
U+5018.1	fCNS11643	1-5438
U+5018.1	fBig5	ADD5
U+5018.1	fGB	0-4C48
U+5018.1	fR/S	 9.8
U+5018.1	fTotalStrokes	 10
U+5018.1	fCangjie	OFBR
U+5018.1	fFourCorner	29227
U+5018.1	fCantonese	soeng4
U+5019.0	fUTF8	候
U+5019.0	fCNS11643	1-5437
U+5019.0	fBig5	ADD4
U+5019.0	fGB	0-3A72
U+5019.0	fR/S	 9.8
U+5019.0	fTotalStrokes	 10
U+5019.0	fCangjie	OLNK
U+5019.0	fFourCorner	27234
U+5019.0	fHakka	  heu5; heu6; heu3; hiu3
U+5019.0	fCantonese	hau6; hau1 (also); hau2 (also)
U+5019.0	fMandarin	hou4
U+5019.0	fEnglish	[1] wait; expect [2] period; time; season [3] situation; condition [4] pay (bills); take care of the bill [5] greet; inquire after
U+5019.1	fUTF8	候
U+5019.1	fCNS11643	1-5437
U+5019.1	fBig5	ADD4
U+5019.1	fGB	0-3A72
U+5019.1	fR/S	 9.8
U+5019.1	fTotalStrokes	 10
U+5019.1	fCangjie	OLNK
U+5019.1	fFourCorner	27234
U+501A.0	fUTF8	倚
U+501A.0	fCNS11643	1-542D
U+501A.0	fBig5	ADCA
U+501A.0	fGB	0-5250
U+501A.0	fR/S	 9.8
U+501A.0	fTotalStrokes	 10
U+501A.0	fCangjie	OKMR
U+501A.0	fFourCorner	24221
U+501A.0	fHakka	  ji3; ri3; i3
U+501A.0	fCantonese	ji2; kei1
U+501A.0	fMandarin	ji1
U+501A.0	fEnglish	[1] rely on; depend on [2] lean toward; rest on [3] biased; partial
U+501A.1	fUTF8	倚
U+501A.1	fCNS11643	1-542D
U+501A.1	fBig5	ADCA
U+501A.1	fGB	0-5250
U+501A.1	fR/S	 9.8
U+501A.1	fTotalStrokes	 10
U+501A.1	fCangjie	OKMR
U+501A.1	fFourCorner	24221
U+501A.1	fMandarin	yi3; yi4
U+501B.0	fUTF8	倛
U+501B.0	fCNS11643	2-2D6F
U+501B.0	fBig5	D0CE
U+501B.0	fGB	5-322B
U+501B.0	fR/S	 9.8
U+501B.0	fTotalStrokes	 10
U+501B.0	fCangjie	OTMC
U+501B.0	fFourCorner	24281
U+501B.0	fCantonese	hei1; gei1
U+501B.0	fMandarin	qi1
U+501C.0	fUTF8	倜
U+501C.0	fCNS11643	2-2D78
U+501C.0	fBig5	D0D7
U+501C.0	fGB	0-5943
U+501C.0	fR/S	 9.8
U+501C.0	fTotalStrokes	 10
U+501C.0	fCangjie	OBGR
U+501C.0	fFourCorner	27220
U+501C.0	fHakka	  tit7
U+501C.0	fCantonese	tik1; zau1
U+501C.0	fMandarin	ti4
U+501C.0	fEnglish	[1] raise high [2] unrestrained; unoccupied
U+501D.0	fUTF8	倝
U+501D.0	fCNS11643	3-2F45
U+501D.0	fGB	5-733B
U+501D.0	fR/S	  9.8
U+501D.0	fTotalStrokes	 10
U+501D.0	fCangjie	JJO
U+501D.0	fFourCorner	48400
U+501D.0	fHakka	  gon5
U+501D.0	fCantonese	gon3
U+501D.0	fMandarin	gan4
U+501D.0	fEnglish	 sunrise dawn
U+501E.0	fUTF8	倞
U+501E.0	fCNS11643	2-2D69
U+501E.0	fBig5	D0C8
U+501E.0	fGB	3-3156
U+501E.0	fR/S	 9.8
U+501E.0	fTotalStrokes	 10
U+501E.0	fCangjie	OYRF
U+501E.0	fFourCorner	20296
U+501E.0	fHakka	 gin5;  liong5; liong6
U+501E.0	fCantonese	ging6
U+501E.0	fMandarin	jing4
U+501E.0	fEnglish	strong
U+501E.1	fUTF8	倞
U+501E.1	fCNS11643	2-2D69
U+501E.1	fBig5	D0C8
U+501E.1	fGB	3-3156
U+501E.1	fR/S	 9.8
U+501E.1	fTotalStrokes	 10
U+501E.1	fCangjie	OYRF
U+501E.1	fFourCorner	20296
U+501E.1	fCantonese	loeng6
U+501E.1	fMandarin	liang4
U+501E.1	fEnglish	[1] far [2] request
U+501F.0	fUTF8	借
U+501F.0	fCNS11643	1-542C
U+501F.0	fBig5	ADC9
U+501F.0	fGB	0-3D68
U+501F.0	fR/S	 9.8
U+501F.0	fTotalStrokes	 10
U+501F.0	fCangjie	OTA
U+501F.0	fFourCorner	24261
U+501F.0	fHakka	  zia5
U+501F.0	fCantonese	ze3; ze2 (also)
U+501F.0	fMandarin	jie4
U+501F.0	fEnglish	[1] borrow [2] lend [3] avail oneself; make use of; resort to [4] make pretext of [5] if; supposing
U+5020.0	fUTF8	倠
U+5020.0	fCNS11643	2-2D79
U+5020.0	fBig5	D0D8
U+5020.0	fGB	3-3152
U+5020.0	fR/S	 9.8
U+5020.0	fTotalStrokes	 10
U+5020.0	fCangjie	OOG
U+5020.0	fFourCorner	20214
U+5020.0	fHakka	 zih1; zui1; zii1; fui1
U+5020.0	fCantonese	seoi1; fei1; zi1
U+5020.0	fMandarin	xu1; sui1
U+5020.0	fEnglish	 ugly face
U+5021.0	fUTF8	倡
U+5021.0	fCNS11643	1-5435
U+5021.0	fBig5	ADD2
U+5021.0	fGB	0-332B
U+5021.0	fR/S	 9.8
U+5021.0	fTotalStrokes	 10
U+5021.0	fCangjie	OAA
U+5021.0	fFourCorner	26260
U+5021.0	fHakka	  cong1;  cong5;  chong1; chong5
U+5021.0	fCantonese	coeng1; coeng3
U+5021.0	fMandarin	chang1
U+5021.0	fEnglish	[1] prostitute [2] wild and unrestrained
U+5021.1	fUTF8	倡
U+5021.1	fCNS11643	1-5435
U+5021.1	fBig5	ADD2
U+5021.1	fGB	0-332B
U+5021.1	fR/S	 9.8
U+5021.1	fTotalStrokes	 10
U+5021.1	fCangjie	OAA
U+5021.1	fFourCorner	26260
U+5021.1	fMandarin	chang4
U+5021.1	fEnglish	 guide; lead; introduce; initiate; advocate
U+5022.0	fUTF8	倢
U+5022.0	fCNS11643	2-2D6D
U+5022.0	fBig5	D0CC
U+5022.0	fGB	3-3150
U+5022.0	fR/S	 9.8
U+5022.0	fTotalStrokes	 10
U+5022.0	fCangjie	OJLO
U+5022.0	fFourCorner	15281
U+5022.0	fHakka	 ciap8;  ziap7
U+5022.0	fCantonese	zit3; zit6 (also); zip3
U+5022.0	fMandarin	jie2
U+5022.0	fEnglish	 beautiful; handsome woman
U+5023.0	fUTF8	倣
U+5023.0	fCNS11643	1-5423
U+5023.0	fBig5	ADC0
U+5023.0	fGB	E-217B
U+5023.0	fR/S	 9.8
U+5023.0	fTotalStrokes	 10
U+5023.0	fCangjie	OYSK
U+5023.0	fFourCorner	28240
U+5023.0	fHakka	 fong3
U+5023.0	fCantonese	fong2
U+5023.0	fMandarin	fang3; pang2
U+5023.0	fEnglish	imitate
U+5024.0	fUTF8	値
U+5024.0	fCNS11643	3-2F4E
U+5024.0	fGB	E-217C
U+5024.0	fR/S	 9.8
U+5024.0	fTotalStrokes	 10
U+5024.0	fCangjie	OJBV
U+5024.0	fFourCorner	24210
U+5024.0	fHakka	 chit8; cit8; dat8; ts'iit8
U+5024.0	fCantonese	zik6
U+5024.0	fMandarin	zhi2
U+5024.0	fEnglish	[1] prices of commodities; value; cost; cost; be worth [2] at the time of ... [3] meet; happen
U+5025.0	fUTF8	倥
U+5025.0	fCNS11643	1-5426
U+5025.0	fBig5	ADC3
U+5025.0	fGB	0-5945
U+5025.0	fR/S	 9.8
U+5025.0	fTotalStrokes	 10
U+5025.0	fCangjie	OJCM
U+5025.0	fFourCorner	23211
U+5025.0	fHakka	 kung1
U+5025.0	fCantonese	hung1; hung3
U+5025.0	fMandarin	kong1
U+5025.0	fEnglish	boorish; ignorant
U+5025.1	fUTF8	倥
U+5025.1	fCNS11643	1-5426
U+5025.1	fBig5	ADC3
U+5025.1	fGB	0-5945
U+5025.1	fR/S	 9.8
U+5025.1	fTotalStrokes	 10
U+5025.1	fCangjie	OJCM
U+5025.1	fFourCorner	23211
U+5025.1	fCantonese	hung2
U+5025.1	fMandarin	kong3
U+5025.1	fEnglish	urgent; press
U+5025.2	fUTF8	倥
U+5025.2	fCNS11643	1-5426
U+5025.2	fBig5	ADC3
U+5025.2	fGB	0-5945
U+5025.2	fR/S	 9.8
U+5025.2	fTotalStrokes	 10
U+5025.2	fCangjie	OJCM
U+5025.2	fFourCorner	23211
U+5025.2	fMandarin	kong4
U+5026.0	fUTF8	倦
U+5026.0	fCNS11643	1-5425
U+5026.0	fBig5	ADC2
U+5026.0	fGB	0-3E6B
U+5026.0	fR/S	 9.8
U+5026.0	fTotalStrokes	 10
U+5026.0	fCangjie	OFQU
U+5026.0	fFourCorner	29212
U+5026.0	fHakka	 kien5; k'ioi5; gien3; ken3; k'ian6; ken5; kian3
U+5026.0	fCantonese	gyun6
U+5026.0	fMandarin	juan4
U+5026.0	fEnglish	tired; weary
U+5027.0	fUTF8	倧
U+5027.0	fCNS11643	2-2D7A
U+5027.0	fBig5	D0D9
U+5027.0	fGB	3-315A
U+5027.0	fR/S	 9.8
U+5027.0	fTotalStrokes	 10
U+5027.0	fCangjie	OJMF
U+5027.0	fFourCorner	13291
U+5027.0	fCantonese	zung1
U+5027.0	fMandarin	zong1
U+5028.0	fUTF8	倨
U+5028.0	fCNS11643	1-5433
U+5028.0	fBig5	ADD0
U+5028.0	fGB	0-5946
U+5028.0	fR/S	 9.8
U+5028.0	fTotalStrokes	 10
U+5028.0	fCangjie	OSJR
U+5028.0	fFourCorner	27264
U+5028.0	fHakka	  gi3; gi1; gi5
U+5028.0	fCantonese	geoi3
U+5028.0	fMandarin	ju4
U+5028.0	fEnglish	[1] arrogant, haughty, rude [2] slightly bent
U+5029.0	fUTF8	倩
U+5029.0	fCNS11643	1-5428
U+5029.0	fBig5	ADC5
U+5029.0	fGB	0-593B
U+5029.0	fR/S	 9.8
U+5029.0	fTotalStrokes	 10
U+5029.0	fCangjie	OQMB
U+5029.0	fFourCorner	25227
U+5029.0	fHakka	  cien5; cin1; ziang1; cen3; cen5; ciang3; cin3
U+5029.0	fCantonese	sin6; sin3; cing3
U+5029.0	fMandarin	qian4; qing4
U+5029.0	fEnglish	[1] pretty dimples of a smiling woman [2] handsome; beautiful; lovely [3] son-in-law [4] ask somebody to do something for oneself; solicit the service of a ghost writer, etc.
U+502A.0	fUTF8	倪
U+502A.0	fCNS11643	1-543C
U+502A.0	fBig5	ADD9
U+502A.0	fGB	0-445F
U+502A.0	fR/S	 9.8
U+502A.0	fTotalStrokes	 10
U+502A.0	fCangjie	OHXU
U+502A.0	fFourCorner	27217
U+502A.0	fHakka	  ngi2;  ji2; ri2; gni3
U+502A.0	fCantonese	ngai4; ngai6; ai4 (also); ai6 (also)
U+502A.0	fMandarin	ni2; ya2; ni4; yai2
U+502A.0	fEnglish	[1] young and weak [2] division [3] beginning [4] limit; bounds [5] a Chinese family name [6] we [7] I
U+502B.0	fUTF8	倫
U+502B.0	fCNS11643	1-543E
U+502B.0	fBig5	ADDB
U+502B.0	fGB	1-4257
U+502B.0	fR/S	 9.8
U+502B.0	fTotalStrokes	 10
U+502B.0	fCangjie	OOMB
U+502B.0	fFourCorner	28227
U+502B.0	fHakka	  lun2; lun3
U+502B.0	fCantonese	leon4
U+502B.0	fMandarin	lun2
U+502B.0	fEnglish	[1] normal human relationships among people [2] comparison; peer; match [3] classification [4] order; logic [5] regular; ordinary [6] choose; choice
U+502C.0	fUTF8	倬
U+502C.0	fCNS11643	2-2D74
U+502C.0	fBig5	D0D3
U+502C.0	fGB	0-593E
U+502C.0	fR/S	 9.8
U+502C.0	fTotalStrokes	 10
U+502C.0	fCangjie	OYAJ
U+502C.0	fFourCorner	21246
U+502C.0	fHakka	  zok7
U+502C.0	fCantonese	coek3; zoek3
U+502C.0	fMandarin	zhuo1; zhuo2
U+502C.0	fEnglish	 noticeable; large clear; distin
U+502D.0	fUTF8	倭
U+502D.0	fCNS11643	1-543B
U+502D.0	fBig5	ADD8
U+502D.0	fGB	0-5941
U+502D.0	fR/S	 9.8
U+502D.0	fTotalStrokes	 10
U+502D.0	fCangjie	OHDV
U+502D.0	fFourCorner	22244
U+502D.0	fHakka	 wo1;  wui1; wi1; vui1; vo1
U+502D.0	fCantonese	wo1; wo2; wai2
U+502D.0	fMandarin	wo1; wo3
U+502D.0	fEnglish	name of a human race; an old name for Japan
U+502D.1	fUTF8	倭
U+502D.1	fCNS11643	1-543B
U+502D.1	fBig5	ADD8
U+502D.1	fGB	0-5941
U+502D.1	fR/S	 9.8
U+502D.1	fTotalStrokes	 10
U+502D.1	fCangjie	OHDV
U+502D.1	fFourCorner	22244
U+502D.1	fCantonese	wai1
U+502D.1	fMandarin	wei1
U+502D.1	fEnglish	[1] winding; circuitous; meandering
U+502E.0	fUTF8	倮
U+502E.0	fCNS11643	3-2F51
U+502E.0	fGB	0-5940
U+502E.0	fR/S	 9.8
U+502E.0	fTotalStrokes	 10
U+502E.0	fCangjie	OWD
U+502E.0	fFourCorner	26290
U+502E.0	fHakka	 go3; ko5; kwo3; ko3; ko1
U+502E.0	fCantonese	lo2
U+502E.0	fMandarin	luo3
U+502E.0	fEnglish	bare; naked; uncovered
U+502F.0	fUTF8	倯
U+502F.0	fCNS11643	2-2D7C
U+502F.0	fBig5	D0DB
U+502F.0	fGB	5-322D
U+502F.0	fR/S	 9.8
U+502F.0	fTotalStrokes	 10
U+502F.0	fCangjie	ODCI
U+502F.0	fFourCorner	28232
U+502F.0	fHakka	 siung2
U+502F.0	fCantonese	sung1
U+502F.0	fMandarin	song1
U+5030.0	fUTF8	倰
U+5030.0	fCNS11643	2-2D6E
U+5030.0	fBig5	D0CD
U+5030.0	fGB	3-314A
U+5030.0	fR/S	 9.8
U+5030.0	fTotalStrokes	 10
U+5030.0	fCangjie	OGCE
U+5030.0	fFourCorner	24247
U+5030.0	fHakka	 lin2; lin5; lin6
U+5030.0	fCantonese	ling4; ling6
U+5030.0	fMandarin	leng2; leng4; ling2
U+5030.0	fEnglish	tired; fatigued
U+5031.0	fUTF8	倱
U+5031.0	fCNS11643	2-2D7D
U+5031.0	fBig5	D0DC
U+5031.0	fGB	5-3230
U+5031.0	fR/S	 9.8
U+5031.0	fTotalStrokes	 10
U+5031.0	fCangjie	OAPP
U+5031.0	fFourCorner	26211
U+5031.0	fCantonese	wan6
U+5031.0	fMandarin	hun4
U+5032.0	fUTF8	倲
U+5032.0	fCNS11643	3-2F43
U+5032.0	fGB	3-306D
U+5032.0	fR/S	  9.8
U+5032.0	fTotalStrokes	 10
U+5032.0	fCangjie	ODW
U+5032.0	fFourCorner	25290
U+5032.0	fHakka	  dung1; dung5; dong5
U+5032.0	fCantonese	dung1
U+5032.0	fMandarin	dong1
U+5032.0	fEnglish	inferior; humble; rustic
U+5033.0	fUTF8	倳
U+5033.0	fCNS11643	2-2D72
U+5033.0	fBig5	D0D1
U+5033.0	fGB	3-314C
U+5033.0	fR/S	 9.8
U+5033.0	fTotalStrokes	 10
U+5033.0	fCangjie	OJLN
U+5033.0	fFourCorner	25207
U+5033.0	fHakka	 cih5; zii5; zih5
U+5033.0	fCantonese	zi3; zi6; si6
U+5033.0	fMandarin	zi4
U+5033.0	fEnglish	[1] stab [2] erect; establish
U+5034.0	fUTF8	倴
U+5034.0	fCNS11643	4-2B61
U+5034.0	fGB	3-314F
U+5034.0	fR/S	 9.8
U+5034.0	fTotalStrokes	 10
U+5034.0	fCangjie	OKJT
U+5034.0	fFourCorner	24240
U+5034.0	fCantonese	ban6; ban3 (also)
U+5034.0	fMandarin	ben1; ben4
U+5035.0	fUTF8	倵
U+5035.0	fCNS11643	2-2D7B
U+5035.0	fBig5	D0DA
U+5035.0	fGB	5-322A
U+5035.0	fR/S	 9.8
U+5035.0	fTotalStrokes	 10
U+5035.0	fCangjie	OMPM
U+5035.0	fFourCorner	23240
U+5035.0	fCantonese	mou5
U+5035.0	fMandarin	wu3
U+5036.0	fUTF8	倶
U+5036.0	fCNS11643	3-2F50
U+5036.0	fGB	E-217D
U+5036.0	fR/S	 9.8
U+5036.0	fTotalStrokes	 10
U+5036.0	fCangjie	OBUC
U+5036.0	fFourCorner	26280
U+5036.0	fHakka	 ki3
U+5036.0	fCantonese	keoi1; geoi1
U+5036.0	fMandarin	ju4; ju1
U+5036.0	fEnglish	all
U+5037.0	fUTF8	倷
U+5037.0	fCNS11643	2-2D73
U+5037.0	fBig5	D0D2
U+5037.0	fGB	3-314E
U+5037.0	fR/S	 9.8
U+5037.0	fTotalStrokes	 10
U+5037.0	fCangjie	OKMF
U+5037.0	fFourCorner	24291
U+5037.0	fHakka	 lai3
U+5037.0	fCantonese	noi6; loi6 (also); naai5 (also); laai5 (also)
U+5037.0	fMandarin	nai4
U+5038.0	fUTF8	倸
U+5038.0	fCNS11643	3-2F54
U+5038.0	fGB	E-217E
U+5038.0	fR/S	  9.8
U+5038.0	fTotalStrokes	 10
U+5038.0	fCangjie	OBD
U+5038.0	fFourCorner	22290
U+5038.0	fHakka	 coi3
U+5038.0	fCantonese	coi2
U+5038.0	fMandarin	cai3
U+5039.0	fUTF8	倹
U+5039.0	fCNS11643	3-2F55
U+5039.0	fGB	E-2221
U+5039.0	fR/S	 9.8
U+5039.0	fTotalStrokes	 10
U+5039.0	fCangjie	OOMO
U+5039.0	fFourCorner	28230
U+5039.0	fCantonese	gim6
U+5039.0	fMandarin	jian3
U+5039.0	fEnglish	temperate; frugal; economical
U+503A.0	fUTF8	债
U+503A.0	fGB	0-552E
U+503A.0	fR/S	 9.8
U+503A.0	fTotalStrokes	 10
U+503A.0	fCangjie	OQMO
U+503A.0	fHakka	 zai5
U+503A.0	fCantonese	zaai3
U+503A.0	fMandarin	zhai4
U+503A.0	fEnglish	simplified form of &#x50B5;
U+503B.0	fUTF8	倻
U+503B.0	fGB	3-314B
U+503B.0	fR/S	 9.8; 9.9
U+503B.0	fTotalStrokes	 10
U+503B.0	fCangjie	OSJL
U+503B.0	fCantonese	je4
U+503B.0	fMandarin	ye1
U+503B.0	fEnglish	 phonetic used in Korean place names
U+503C.0	fUTF8	值
U+503C.0	fCNS11643	1-542B
U+503C.0	fBig5	ADC8
U+503C.0	fGB	0-5635
U+503C.0	fR/S	 9.8
U+503C.0	fTotalStrokes	 10
U+503C.0	fCangjie	OJBM
U+503C.0	fFourCorner	24216
U+503C.0	fHakka	 cit8; ts'ik8; cet8
U+503C.0	fCantonese	zik6
U+503C.0	fMandarin	zhi2; zhi4
U+503C.0	fEnglish	price
U+503D.0	fUTF8	倽
U+503D.0	fGB	5-3239
U+503D.0	fR/S	 9.8
U+503D.0	fTotalStrokes	 10
U+503D.0	fCangjie	XOOMR
U+503D.0	fFourCorner	2826
U+503D.0	fCantonese	saa2
U+503D.0	fMandarin	sha4
U+503D.0	fEnglish	 what which who why
U+503E.0	fUTF8	倾
U+503E.0	fGB	0-4763
U+503E.0	fR/S	 9.8
U+503E.0	fTotalStrokes	 10
U+503E.0	fCangjie	OPMO
U+503E.0	fHakka	 kin1; ken1
U+503E.0	fCantonese	king1
U+503E.0	fMandarin	qing1; qing2
U+503E.0	fEnglish	simplified form of &#x50BE;
U+503F.0	fUTF8	倿
U+503F.0	fCNS11643	3-644B
U+503F.0	fGB	E-2222
U+503F.0	fR/S	 9.8
U+503F.0	fTotalStrokes	 10
U+503F.0	fCangjie	OYTV
U+503F.0	fFourCorner	2024
U+503F.0	fCantonese	ning6; ling6 (also)
U+5040.0	fUTF8	偀
U+5040.0	fCNS11643	2-3366
U+5040.0	fBig5	D463
U+5040.0	fGB	5-322C
U+5040.0	fR/S	 9.9
U+5040.0	fTotalStrokes	 11
U+5040.0	fCangjie	OTLK
U+5040.0	fFourCorner	24235
U+5040.0	fCantonese	jing1
U+5040.0	fMandarin	ying1
U+5041.0	fUTF8	偁
U+5041.0	fCNS11643	2-335A
U+5041.0	fBig5	D457
U+5041.0	fGB	5-3249
U+5041.0	fR/S	 9.9
U+5041.0	fTotalStrokes	 11
U+5041.0	fCangjie	OBGB
U+5041.0	fFourCorner	22247
U+5041.0	fCantonese	cing1; can3; cing3
U+5041.0	fMandarin	cheng1; cheng4
U+5041.0	fEnglish	state
U+5042.0	fUTF8	偂
U+5042.0	fCNS11643	3-343C
U+5042.0	fGB	E-2223
U+5042.0	fR/S	 9.9
U+5042.0	fTotalStrokes	 11
U+5042.0	fCangjie	OTBN
U+5042.0	fFourCorner	28220
U+5042.0	fCantonese	cin4
U+5042.0	fMandarin	jian1; qian2; jian3
U+5043.0	fUTF8	偃
U+5043.0	fCNS11643	1-5876
U+5043.0	fBig5	B0B3
U+5043.0	fGB	0-5948
U+5043.0	fR/S	 9.9
U+5043.0	fTotalStrokes	 11
U+5043.0	fCangjie	OSAV
U+5043.0	fFourCorner	21214
U+5043.0	fHakka	 jen3; ren3; jan3
U+5043.0	fCantonese	jin2; jim2 (also)
U+5043.0	fMandarin	yan3
U+5043.0	fEnglish	[1] cease; be at rest; stop; suppress; lay off [2] lay on one's back [3] embankment; earthen bank or dyke [4] a Chinese family name
U+5044.0	fUTF8	偄
U+5044.0	fCNS11643	3-343F
U+5044.0	fGB	3-3161
U+5044.0	fR/S	  9.9
U+5044.0	fTotalStrokes	 11
U+5044.0	fCangjie	OMBK
U+5044.0	fFourCorner	21230
U+5044.0	fHakka	 ngion3; nion1; non3
U+5044.0	fCantonese	jyun5
U+5044.0	fMandarin	ruan3; nuan4; ru2
U+5044.0	fEnglish	weak
U+5045.0	fUTF8	偅
U+5045.0	fCNS11643	2-335F
U+5045.0	fBig5	D45C
U+5045.0	fGB	5-3245
U+5045.0	fR/S	 9.9
U+5045.0	fTotalStrokes	 11
U+5045.0	fCangjie	OHJG
U+5045.0	fFourCorner	22214
U+5045.0	fHakka	 zhung3; zung3
U+5045.0	fCantonese	zung3; zung2
U+5045.0	fMandarin	zhong4
U+5046.0	fUTF8	偆
U+5046.0	fCNS11643	2-3365
U+5046.0	fBig5	D462
U+5046.0	fGB	3-315D
U+5046.0	fR/S	 9.9
U+5046.0	fTotalStrokes	 11
U+5046.0	fCangjie	OQKA
U+5046.0	fFourCorner	25263
U+5046.0	fHakka	 chun3; cun3
U+5046.0	fCantonese	ceon2
U+5046.0	fMandarin	chun3
U+5046.0	fEnglish	rich; well-to-do
U+5047.0	fUTF8	假
U+5047.0	fCNS11643	1-5875
U+5047.0	fBig5	B0B2
U+5047.0	fGB	0-3C59
U+5047.0	fR/S	 9.9
U+5047.0	fTotalStrokes	 11
U+5047.0	fCangjie	ORSE
U+5047.0	fFourCorner	27247
U+5047.0	fHakka	 ga5
U+5047.0	fCantonese	gaa2; gaa1 (also); haa4; kaa4
U+5047.0	fMandarin	jia3; xia2; ge2; jia1; jia2
U+5047.0	fEnglish	false; not real; phony; artificial; fake; bogus; sham; falsehood, deception
U+5047.1	fUTF8	假
U+5047.1	fCNS11643	1-5875
U+5047.1	fBig5	B0B2
U+5047.1	fGB	0-3C59
U+5047.1	fR/S	 9.9
U+5047.1	fTotalStrokes	 11
U+5047.1	fCangjie	ORSE
U+5047.1	fFourCorner	27247
U+5047.1	fHakka	 ga3
U+5047.1	fCantonese	gaa3
U+5047.1	fMandarin	jia4
U+5047.1	fEnglish	 holiday; vacation
U+5048.0	fUTF8	偈
U+5048.0	fCNS11643	2-3358
U+5048.0	fBig5	D455
U+5048.0	fGB	0-594A
U+5048.0	fR/S	 9.9
U+5048.0	fTotalStrokes	 11
U+5048.0	fCangjie	OAPV
U+5048.0	fFourCorner	26227
U+5048.0	fHakka	  ki5; giet7; gai1; ket7; ket8; get; kiet8
U+5048.0	fCantonese	git6; gai6; gai2 (also)
U+5048.0	fMandarin	jie2; ji4; qi4
U+5048.0	fEnglish	[1] brave; martial [2] hasty; scudding
U+5049.0	fUTF8	偉
U+5049.0	fCNS11643	1-5879
U+5049.0	fBig5	B0B6
U+5049.0	fGB	1-4E30
U+5049.0	fR/S	 9.9
U+5049.0	fTotalStrokes	 11
U+5049.0	fCangjie	ODMQ
U+5049.0	fFourCorner	24256
U+5049.0	fHakka	 wui3; wi3; vui3
U+5049.0	fCantonese	wai5
U+5049.0	fMandarin	wei3
U+5049.0	fEnglish	great; big; gigantic; extraordinary
U+504A.0	fUTF8	偊
U+504A.0	fCNS11643	2-335C
U+504A.0	fBig5	D459
U+504A.0	fGB	3-316B
U+504A.0	fR/S	 9.9
U+504A.0	fTotalStrokes	 11
U+504A.0	fCangjie	OHLB
U+504A.0	fFourCorner	22227
U+504A.0	fHakka	 gi3; ji3; ri3; i3
U+504A.0	fCantonese	jyu2; geoi2; jyu5 (also); 83
U+504A.0	fMandarin	yu3; ju3
U+504A.0	fEnglish	 walk alone self-reliant
U+504A.1	fUTF8	偊
U+504A.1	fCNS11643	2-335C
U+504A.1	fBig5	D459
U+504A.1	fGB	3-316B
U+504A.1	fR/S	 9.9
U+504A.1	fTotalStrokes	 11
U+504A.1	fCangjie	OHLB
U+504A.1	fFourCorner	22227
U+504B.0	fUTF8	偋
U+504B.0	fCNS11643	2-3355
U+504B.0	fBig5	D452
U+504B.0	fGB	5-324F
U+504B.0	fR/S	 9.9
U+504B.0	fTotalStrokes	 11
U+504B.0	fCangjie	OSTT
U+504B.0	fFourCorner	27241
U+504B.0	fHakka	 bin3
U+504B.0	fCantonese	bing2; ping2
U+504B.0	fMandarin	bing3; bing4
U+504C.0	fUTF8	偌
U+504C.0	fCNS11643	1-5877
U+504C.0	fBig5	B0B4
U+504C.0	fGB	0-593C
U+504C.0	fR/S	 9.9
U+504C.0	fTotalStrokes	 11
U+504C.0	fCangjie	OTKR
U+504C.0	fFourCorner	24264
U+504C.0	fHakka	  ngia3; ngia1; ngia5
U+504C.0	fCantonese	je6; joek6 (also)
U+504C.0	fMandarin	ruo4
U+504C.0	fEnglish	thus; so
U+504D.0	fUTF8	偍
U+504D.0	fCNS11643	2-3359
U+504D.0	fBig5	D456
U+504D.0	fGB	3-3163
U+504D.0	fR/S	 9.9
U+504D.0	fTotalStrokes	 11
U+504D.0	fCangjie	OAMO
U+504D.0	fFourCorner	26281
U+504D.0	fCantonese	tai4
U+504D.0	fMandarin	ti2
U+504E.0	fUTF8	偎
U+504E.0	fCNS11643	1-587C
U+504E.0	fBig5	B0B9
U+504E.0	fGB	0-594B
U+504E.0	fR/S	 9.9
U+504E.0	fTotalStrokes	 11
U+504E.0	fCangjie	OWMV
U+504E.0	fFourCorner	26232
U+504E.0	fHakka	 woi1; wa3; wui1; voi1; voe1
U+504E.0	fCantonese	wui1
U+504E.0	fMandarin	wei1
U+504E.0	fEnglish	[1] cuddle; embrace [2] intimate; very dear to [3] lean on
U+504F.0	fUTF8	偏
U+504F.0	fCNS11643	1-5923
U+504F.0	fBig5	B0BE
U+504F.0	fGB	0-462B
U+504F.0	fR/S	 9.9
U+504F.0	fTotalStrokes	 11
U+504F.0	fCangjie	OHSB
U+504F.0	fFourCorner	23227
U+504F.0	fHakka	 pien1; pen1; p'ian3; p'ian1
U+504F.0	fCantonese	pin1
U+504F.0	fMandarin	pian1
U+504F.0	fEnglish	[1] biased; not fair; prejudiced; partial [2] leaning; inclined to one side; slanting [3] an auxiliary verb indicating a sense of contrariness or determination
U+5050.0	fUTF8	偐
U+5050.0	fCNS11643	3-3444
U+5050.0	fGB	5-324C
U+5050.0	fR/S	  9.9
U+5050.0	fTotalStrokes	 11
U+5050.0	fCangjie	OYHH
U+5050.0	fFourCorner	20220
U+5050.0	fHakka	 ngan5; ngien5; ngan6; ngien6
U+5050.0	fCantonese	ngaan6; aan6 (also)
U+5050.0	fMandarin	yan4
U+5050.0	fEnglish	false; counterfeit; spurious
U+5051.0	fUTF8	偑
U+5051.0	fCNS11643	2-336A
U+5051.0	fBig5	D467
U+5051.0	fGB	3-3065
U+5051.0	fR/S	 9.9
U+5051.0	fTotalStrokes	 11
U+5051.0	fCangjie	OHNI
U+5051.0	fFourCorner	27210
U+5051.0	fCantonese	fung1
U+5051.0	fMandarin	feng1
U+5052.0	fUTF8	偒
U+5052.0	fCNS11643	3-3441
U+5052.0	fGB	5-314D
U+5052.0	fR/S	  9.9
U+5052.0	fTotalStrokes	 11
U+5052.0	fCangjie	OAMH
U+5052.0	fFourCorner	26220
U+5052.0	fCantonese	tong2
U+5052.0	fMandarin	tang3; dang4
U+5053.0	fUTF8	偓
U+5053.0	fCNS11643	2-3354
U+5053.0	fBig5	D451
U+5053.0	fGB	3-3172
U+5053.0	fR/S	 9.9
U+5053.0	fTotalStrokes	 11
U+5053.0	fCangjie	OSMG
U+5053.0	fFourCorner	27214
U+5053.0	fHakka	 wok7; vok7
U+5053.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+5053.0	fMandarin	wo4
U+5053.0	fEnglish	restrained; kept in order
U+5054.0	fUTF8	偔
U+5054.0	fCNS11643	3-3442
U+5054.0	fGB	3-3167
U+5054.0	fR/S	  9.9
U+5054.0	fTotalStrokes	 11
U+5054.0	fCangjie	ORRS
U+5054.0	fFourCorner	26220
U+5054.0	fCantonese	ngok6; ok6 (also)
U+5054.0	fMandarin	e4
U+5055.0	fUTF8	偕
U+5055.0	fCNS11643	1-587D
U+5055.0	fBig5	B0BA
U+5055.0	fGB	0-5949
U+5055.0	fR/S	 9.9
U+5055.0	fTotalStrokes	 11
U+5055.0	fCangjie	OPPA
U+5055.0	fFourCorner	21261
U+5055.0	fHakka	 hai2; hai1; gai1
U+5055.0	fCantonese	gaai1; haai4 (also)
U+5055.0	fMandarin	xie2; jie1
U+5055.0	fEnglish	[1] accompany [2] together
U+5056.0	fUTF8	偖
U+5056.0	fCNS11643	3-2F57
U+5056.0	fGB	E-2224
U+5056.0	fR/S	 9.9; 9.8
U+5056.0	fTotalStrokes	 11; 10
U+5056.0	fCangjie	OJKA
U+5056.0	fFourCorner	24260
U+5056.0	fCantonese	ce2
U+5056.0	fMandarin	che3
U+5056.0	fEnglish	 rip up; tear down raise haul
U+5057.0	fUTF8	偗
U+5057.0	fCNS11643	2-3369
U+5057.0	fBig5	D466
U+5057.0	fGB	E-2225
U+5057.0	fR/S	 9.9
U+5057.0	fTotalStrokes	 11
U+5057.0	fCangjie	OFHU
U+5057.0	fFourCorner	29262
U+5057.0	fCantonese	saang2
U+5057.0	fMandarin	sheng3
U+5058.0	fUTF8	偘
U+5058.0	fCNS11643	3-3447
U+5058.0	fGB	E-2226
U+5058.0	fR/S	 9.9
U+5058.0	fTotalStrokes	 11
U+5058.0	fCangjie	ORRR
U+5058.0	fFourCorner	26260
U+5058.0	fCantonese	hon2
U+5058.0	fMandarin	kan3
U+5058.0	fEnglish	ancient family name
U+5059.0	fUTF8	偙
U+5059.0	fCNS11643	3-343D
U+5059.0	fGB	3-316F
U+5059.0	fR/S	  9.9
U+5059.0	fTotalStrokes	 11
U+5059.0	fCangjie	OYBB
U+5059.0	fFourCorner	20220
U+5059.0	fCantonese	dai3
U+5059.0	fMandarin	di4
U+505A.0	fUTF8	做
U+505A.0	fCNS11643	1-5878
U+505A.0	fBig5	B0B5
U+505A.0	fGB	0-5776
U+505A.0	fR/S	 9.9
U+505A.0	fTotalStrokes	 11
U+505A.0	fCangjie	OJRK
U+505A.0	fFourCorner	28240
U+505A.0	fHakka	  zo5; zii5
U+505A.0	fCantonese	zou6
U+505A.0	fMandarin	zuo4
U+505A.0	fEnglish	[1] work; make; do [2] act as [3] pretend to be [4] give (a party, reception, etc.) [5] enter (a profession) [6] become
U+505B.0	fUTF8	偛
U+505B.0	fCNS11643	2-335B
U+505B.0	fBig5	D458
U+505B.0	fGB	3-3169
U+505B.0	fR/S	 9.9
U+505B.0	fTotalStrokes	 11
U+505B.0	fCangjie	OHJX
U+505B.0	fFourCorner	21277
U+505B.0	fCantonese	zaap3
U+505B.0	fMandarin	cha1
U+505C.0	fUTF8	停
U+505C.0	fCNS11643	1-5874
U+505C.0	fBig5	B0B1
U+505C.0	fGB	0-4D23
U+505C.0	fR/S	 9.9
U+505C.0	fTotalStrokes	 11
U+505C.0	fCangjie	OYRN
U+505C.0	fFourCorner	20221
U+505C.0	fHakka	  tin2; tin3
U+505C.0	fCantonese	ting4; ting2 (also)
U+505C.0	fMandarin	ting2
U+505C.0	fEnglish	[1] stop; pause; halt; stay; park [2] suspend; delay [3] percentage
U+505D.0	fUTF8	偝
U+505D.0	fCNS11643	2-3356
U+505D.0	fBig5	D453
U+505D.0	fGB	5-3244
U+505D.0	fR/S	 9.9
U+505D.0	fTotalStrokes	 11
U+505D.0	fCangjie	OLPB
U+505D.0	fFourCorner	21227
U+505D.0	fHakka	 boi5; poi5; poi6; pi5
U+505D.0	fCantonese	bui3; bui6
U+505D.0	fMandarin	bei4; bei1
U+505D.0	fEnglish	 not facing desert
U+505E.0	fUTF8	偞
U+505E.0	fCNS11643	2-3352
U+505E.0	fBig5	D44F
U+505E.0	fGB	3-315F
U+505E.0	fR/S	 9.9
U+505E.0	fTotalStrokes	 11
U+505E.0	fCangjie	OPTD
U+505E.0	fFourCorner	24294
U+505E.0	fHakka	 tiap8; jap8; rap8
U+505E.0	fCantonese	jip6
U+505E.0	fMandarin	ye4; xie4
U+505E.0	fEnglish	gray; lightsome
U+505F.0	fUTF8	偟
U+505F.0	fCNS11643	2-3360
U+505F.0	fBig5	D45D
U+505F.0	fGB	5-3247
U+505F.0	fR/S	 9.9
U+505F.0	fTotalStrokes	 11
U+505F.0	fCangjie	OHAG
U+505F.0	fFourCorner	26214
U+505F.0	fHakka	  fong2
U+505F.0	fCantonese	wong4
U+505F.0	fMandarin	huang2
U+505F.0	fEnglish	 agitated alarmed
U+5060.0	fUTF8	偠
U+5060.0	fCNS11643	2-3353
U+5060.0	fBig5	D450
U+5060.0	fGB	5-3242
U+5060.0	fR/S	 9.9
U+5060.0	fTotalStrokes	 11
U+5060.0	fCangjie	OMWV
U+5060.0	fFourCorner	21244
U+5060.0	fHakka	 rau5; jeu5; jau3
U+5060.0	fCantonese	jiu5 (also); jiu2
U+5060.0	fMandarin	yao3
U+5060.0	fEnglish	small-waisted; agile
U+5061.0	fUTF8	偡
U+5061.0	fCNS11643	2-3351
U+5061.0	fBig5	D44E
U+5061.0	fGB	3-315E
U+5061.0	fR/S	 9.9
U+5061.0	fTotalStrokes	 11
U+5061.0	fCangjie	OTMV
U+5061.0	fFourCorner	24211
U+5061.0	fCantonese	zaam6; zaam3
U+5061.0	fMandarin	zhan4
U+5062.0	fUTF8	偢
U+5062.0	fCNS11643	2-335D
U+5062.0	fBig5	D45A
U+5062.0	fGB	3-316A
U+5062.0	fR/S	 9.9
U+5062.0	fTotalStrokes	 11
U+5062.0	fCangjie	OHDF
U+5062.0	fFourCorner	29280
U+5062.0	fHakka	 ciau5; ciu1
U+5062.0	fCantonese	ciu2 (also); cau2 (also); cau1
U+5062.0	fMandarin	chou3; qiu1
U+5062.0	fEnglish	 stare at
U+5063.0	fUTF8	偣
U+5063.0	fCNS11643	2-3363
U+5063.0	fBig5	D460
U+5063.0	fGB	5-324B
U+5063.0	fR/S	 9.9
U+5063.0	fTotalStrokes	 11
U+5063.0	fCangjie	OYTA
U+5063.0	fFourCorner	20261
U+5063.0	fCantonese	jim1
U+5063.0	fMandarin	yan1
U+5064.0	fUTF8	偤
U+5064.0	fCNS11643	2-3364
U+5064.0	fBig5	D461
U+5064.0	fGB	5-324D
U+5064.0	fR/S	 9.9
U+5064.0	fTotalStrokes	 11
U+5064.0	fCangjie	OTCW
U+5064.0	fFourCorner	28261
U+5064.0	fCantonese	jau4
U+5064.0	fMandarin	you3; you2
U+5065.0	fUTF8	健
U+5065.0	fCNS11643	1-587A
U+5065.0	fBig5	B0B7
U+5065.0	fGB	0-3D21
U+5065.0	fR/S	 9.9
U+5065.0	fTotalStrokes	 11
U+5065.0	fCangjie	ONKQ
U+5065.0	fFourCorner	25240
U+5065.0	fHakka	 ken3; kian5; kien5; gien5; kien6; ken5
U+5065.0	fCantonese	gin6
U+5065.0	fMandarin	jian4
U+5065.0	fEnglish	[1] strong; healthy [2] vigorous; capable [3] fond of; inclined to; liable to [4] strengthen; toughen
U+5066.0	fUTF8	偦
U+5066.0	fCNS11643	3-3440
U+5066.0	fGB	5-3250
U+5066.0	fR/S	  9.9
U+5066.0	fTotalStrokes	 11
U+5066.0	fCangjie	ONOB
U+5066.0	fFourCorner	27220
U+5066.0	fCantonese	seoi2
U+5066.0	fMandarin	xu1; xu3
U+5067.0	fUTF8	偧
U+5067.0	fCNS11643	4-3047
U+5067.0	fGB	5-3243
U+5067.0	fR/S	 9.9
U+5067.0	fTotalStrokes	 11
U+5067.0	fCangjie	OKNN
U+5067.0	fFourCorner	24220
U+5067.0	fHakka	 cha1
U+5067.0	fCantonese	zaa3
U+5067.0	fMandarin	zha1
U+5068.0	fUTF8	偨
U+5068.0	fCNS11643	2-3A3F
U+5068.0	fBig5	D85B
U+5068.0	fGB	3-3177
U+5068.0	fR/S	 9.9; 9.10
U+5068.0	fTotalStrokes	 11
U+5068.0	fCangjie	OYPD
U+5068.0	fFourCorner	21294
U+5068.0	fCantonese	ci1; zi1 (also)
U+5068.0	fMandarin	chai1; ci1
U+5068.0	fEnglish	uneven
U+5069.0	fUTF8	偩
U+5069.0	fCNS11643	2-3361
U+5069.0	fBig5	D45E
U+5069.0	fGB	3-312C
U+5069.0	fR/S	 9.9
U+5069.0	fTotalStrokes	 11
U+5069.0	fCangjie	ONBC
U+5069.0	fFourCorner	27286
U+5069.0	fHakka	  fu5
U+5069.0	fCantonese	fu6
U+5069.0	fMandarin	fu4
U+5069.0	fEnglish	follow
U+506A.0	fUTF8	偪
U+506A.0	fCNS11643	2-3350
U+506A.0	fBig5	D44D
U+506A.0	fGB	E-2227
U+506A.0	fR/S	 9.9
U+506A.0	fTotalStrokes	 11
U+506A.0	fCangjie	OMRW
U+506A.0	fFourCorner	21266
U+506A.0	fHakka	  bit7; bet7
U+506A.0	fCantonese	bik1; 903
U+506A.0	fMandarin	bi1; fu3
U+506A.0	fEnglish	 compel; pressure; force bother
U+506B.0	fUTF8	偫
U+506B.0	fCNS11643	2-3362
U+506B.0	fBig5	D45F
U+506B.0	fGB	3-316C
U+506B.0	fR/S	 9.9
U+506B.0	fTotalStrokes	 11
U+506B.0	fCangjie	OHOI
U+506B.0	fFourCorner	24241
U+506B.0	fHakka	 sii5; cih5; shih6; cih6; shi5
U+506B.0	fCantonese	zi6; ci5; doi6
U+506B.0	fMandarin	zhi4
U+506B.0	fEnglish	 wait for lay in
U+506C.0	fUTF8	偬
U+506C.0	fCNS11643	3-344B
U+506C.0	fGB	0-594C
U+506C.0	fR/S	 9.9
U+506C.0	fTotalStrokes	 11
U+506C.0	fCangjie	OPKP
U+506C.0	fFourCorner	27230
U+506C.0	fHakka	 cung1; zung3
U+506C.0	fCantonese	zung2
U+506C.0	fMandarin	zong3
U+506C.0	fEnglish	[1] urgent; having no leisure [2] be in (financial, etc.) straits
U+506D.0	fUTF8	偭
U+506D.0	fCNS11643	1-5926
U+506D.0	fBig5	B0C1
U+506D.0	fGB	3-3160
U+506D.0	fR/S	 9.9
U+506D.0	fTotalStrokes	 11
U+506D.0	fCangjie	OMWL
U+506D.0	fFourCorner	21260
U+506D.0	fHakka	  mien5; men5
U+506D.0	fCantonese	min5; min6
U+506D.0	fMandarin	mian3; mian4
U+506D.0	fEnglish	transgress
U+506E.0	fUTF8	偮
U+506E.0	fCNS11643	2-3367
U+506E.0	fBig5	D464
U+506E.0	fGB	3-3164
U+506E.0	fR/S	 9.9
U+506E.0	fTotalStrokes	 11
U+506E.0	fCangjie	ORSJ
U+506E.0	fFourCorner	26241
U+506E.0	fHakka	 sip8; cip8
U+506E.0	fCantonese	cap1; zap1
U+506E.0	fMandarin	ji2
U+506F.0	fUTF8	偯
U+506F.0	fCNS11643	1-5925
U+506F.0	fBig5	B0C0
U+506F.0	fGB	3-316D
U+506F.0	fR/S	 9.9
U+506F.0	fTotalStrokes	 11
U+506F.0	fCangjie	OYRV
U+506F.0	fFourCorner	20232
U+506F.0	fHakka	 oi1; ji3; ri3; i3
U+506F.0	fCantonese	ji2
U+506F.0	fMandarin	yi3
U+506F.0	fEnglish	sob
U+5070.0	fUTF8	偰
U+5070.0	fCNS11643	2-334F
U+5070.0	fBig5	D44C
U+5070.0	fGB	5-323F
U+5070.0	fR/S	 9.9
U+5070.0	fTotalStrokes	 11
U+5070.0	fCangjie	OQHK
U+5070.0	fFourCorner	27234
U+5070.0	fHakka	 siet7
U+5070.0	fCantonese	sit3
U+5070.0	fMandarin	xie4
U+5070.0	fEnglish	a Chinese family name
U+5071.0	fUTF8	偱
U+5071.0	fCNS11643	3-3443
U+5071.0	fGB	E-2228
U+5071.0	fR/S	 9.9
U+5071.0	fTotalStrokes	 11
U+5071.0	fCangjie	OHJU
U+5071.0	fFourCorner	22260
U+5071.0	fHakka	 sun2
U+5071.0	fCantonese	seon1
U+5071.0	fMandarin	xun2
U+5072.0	fUTF8	偲
U+5072.0	fCNS11643	2-3357
U+5072.0	fBig5	D454
U+5072.0	fGB	3-3166
U+5072.0	fR/S	 9.9
U+5072.0	fTotalStrokes	 11
U+5072.0	fCangjie	OWP
U+5072.0	fFourCorner	26230
U+5072.0	fHakka	  sii1; se1
U+5072.0	fCantonese	si1; caai1; soi1 (also)
U+5072.0	fMandarin	si1
U+5072.0	fEnglish	[1] strong [2] talented
U+5072.1	fUTF8	偲
U+5072.1	fCNS11643	2-3357
U+5072.1	fBig5	D454
U+5072.1	fGB	3-3166
U+5072.1	fR/S	 9.9
U+5072.1	fTotalStrokes	 11
U+5072.1	fCangjie	OWP
U+5072.1	fFourCorner	26230
U+5072.1	fCantonese	cai1
U+5072.1	fMandarin	cai1
U+5073.0	fUTF8	偳
U+5073.0	fCNS11643	2-3368
U+5073.0	fBig5	D465
U+5073.0	fGB	3-3168
U+5073.0	fR/S	 9.9
U+5073.0	fTotalStrokes	 11
U+5073.0	fCangjie	OUMB
U+5073.0	fFourCorner	22227
U+5073.0	fCantonese	dyun1; zeoi3
U+5073.0	fMandarin	duan1
U+5074.0	fUTF8	側
U+5074.0	fCNS11643	1-5921
U+5074.0	fBig5	B0BC
U+5074.0	fGB	1-3260
U+5074.0	fR/S	 9.9
U+5074.0	fTotalStrokes	 11
U+5074.0	fCangjie	OBCN
U+5074.0	fFourCorner	22200
U+5074.0	fHakka	  zet7; cet7; tsek7
U+5074.0	fCantonese	zak1
U+5074.0	fMandarin	ce4; ze4
U+5074.0	fEnglish	[1] side; sideways [2] slant; incline towards [3] low and narrow-minded; prejudiced
U+5075.0	fUTF8	偵
U+5075.0	fCNS11643	1-587E
U+5075.0	fBig5	B0BB
U+5075.0	fGB	1-556C
U+5075.0	fR/S	 9.9
U+5075.0	fTotalStrokes	 11
U+5075.0	fCangjie	OYBC
U+5075.0	fFourCorner	21286
U+5075.0	fHakka	 zin1;  zhin1
U+5075.0	fCantonese	zing1
U+5075.0	fMandarin	zhen1; zheng1
U+5075.0	fEnglish	[1] detect; spy; scout [2] scout; spy; detective
U+5076.0	fUTF8	偶
U+5076.0	fCNS11643	1-587B
U+5076.0	fBig5	B0B8
U+5076.0	fGB	0-453C
U+5076.0	fR/S	 9.9
U+5076.0	fTotalStrokes	 11
U+5076.0	fCangjie	OWLB
U+5076.0	fFourCorner	26227
U+5076.0	fHakka	 ngeu3;  ngieu3; ngiu3; ngiau3
U+5076.0	fCantonese	ngau5; au5 (also)
U+5076.0	fMandarin	ou3
U+5076.0	fEnglish	[1] idol; image [2] coincidentally; accidentally [3] once in a while [4] not to be taken for granted [5] even number [6] counterpart [7] mate [8] one's compan; fellows; buddies
U+5077.0	fUTF8	偷
U+5077.0	fCNS11643	1-5922
U+5077.0	fBig5	B0BD
U+5077.0	fGB	0-4D35
U+5077.0	fR/S	 9.9
U+5077.0	fTotalStrokes	 11
U+5077.0	fCangjie	OOMN
U+5077.0	fFourCorner	28221
U+5077.0	fHakka	  teu1; tiu1
U+5077.0	fCantonese	tau1
U+5077.0	fMandarin	tou1
U+5077.0	fEnglish	[1] steal; filch; burglarize; pilfer [2] do something without others' knowledge; stealthily; surreptitiously [3] while away without purpose [4] have an ex-marital activity
U+5078.0	fUTF8	偸
U+5078.0	fCNS11643	3-344C
U+5078.0	fGB	E-2229
U+5078.0	fR/S	 9.9
U+5078.0	fTotalStrokes	 11
U+5078.0	fCangjie	OOMV
U+5078.0	fFourCorner	28220
U+5078.0	fCantonese	tau1; cau1
U+5078.0	fMandarin	tou1
U+5078.0	fEnglish	to steal
U+5079.0	fUTF8	偹
U+5079.0	fCNS11643	3-344A
U+5079.0	fGB	E-222A
U+5079.0	fR/S	 9.9
U+5079.0	fTotalStrokes	 11
U+5079.0	fCangjie	OLHW
U+5079.0	fFourCorner	27260
U+5079.0	fHakka	 pi5; pi6
U+5079.0	fCantonese	bei6
U+5079.0	fMandarin	bei4
U+5079.0	fEnglish	prepare; ready; perfect
U+507A.0	fUTF8	偺
U+507A.0	fCNS11643	1-5872
U+507A.0	fBig5	B0AF
U+507A.0	fGB	E-222B
U+507A.0	fR/S	 9.9
U+507A.0	fTotalStrokes	 11
U+507A.0	fCangjie	OHOA
U+507A.0	fFourCorner	28264
U+507A.0	fHakka	 za5
U+507A.0	fCantonese	zaa1
U+507A.0	fMandarin	za2; zan2; zan5
U+507A.0	fEnglish	us
U+507B.0	fUTF8	偻
U+507B.0	fCNS11643	3-3445
U+507B.0	fGB	0-594D
U+507B.0	fR/S	 9.9
U+507B.0	fTotalStrokes	 11
U+507B.0	fCangjie	OFDV
U+507B.0	fFourCorner	29240
U+507B.0	fHakka	 leu2
U+507B.0	fCantonese	lau4; leoi5
U+507B.0	fMandarin	lou2; luu3; lou4; qiu2
U+507B.0	fEnglish	simplified form of &#x50C2;
U+507C.0	fUTF8	偼
U+507C.0	fCNS11643	3-3446
U+507C.0	fGB	E-222C
U+507C.0	fR/S	  9.9
U+507C.0	fTotalStrokes	 11
U+507C.0	fCangjie	OYSO
U+507C.0	fFourCorner	21280
U+507C.0	fCantonese	zit3
U+507C.0	fMandarin	jie2
U+507D.0	fUTF8	偽
U+507D.0	fCNS11643	1-5873
U+507D.0	fBig5	B0B0
U+507D.0	fGB	E-222D
U+507D.0	fR/S	 9.9
U+507D.0	fTotalStrokes	 11
U+507D.0	fCangjie	OIKF
U+507D.0	fFourCorner	22227
U+507D.0	fHakka	 ngui3
U+507D.0	fCantonese	ngai6; ai6 (also)
U+507D.0	fMandarin	wei3; e2; wei4
U+507D.0	fEnglish	false; counterfeit; bogus
U+507E.0	fUTF8	偾
U+507E.0	fGB	0-5947
U+507E.0	fR/S	 9.9
U+507E.0	fTotalStrokes	 11
U+507E.0	fCangjie	OJTO
U+507E.0	fHakka	 fun3
U+507E.0	fCantonese	fan5
U+507E.0	fMandarin	fen4; ben1
U+507E.0	fEnglish	simplified form of &#x50E8;
U+507F.0	fUTF8	偿
U+507F.0	fGB	0-3325
U+507F.0	fR/S	 9.9
U+507F.0	fTotalStrokes	 11
U+507F.0	fCangjie	OFBI
U+507F.0	fHakka	 shong2
U+507F.0	fCantonese	soeng4
U+507F.0	fMandarin	chang2; shang3
U+507F.0	fEnglish	simplified form of &#x511F;
U+5080.0	fUTF8	傀
U+5080.0	fCNS11643	1-5E2E
U+5080.0	fBig5	B3C8
U+5080.0	fGB	0-3F7E
U+5080.0	fR/S	 9.10
U+5080.0	fTotalStrokes	 12
U+5080.0	fCangjie	OHUI
U+5080.0	fFourCorner	26213
U+5080.0	fHakka	 kui3;  gui3; kui5; ngui3; k'wui3; kwi2; kwui3
U+5080.0	fCantonese	faai3; gwai1
U+5080.0	fMandarin	kui3; hui1
U+5080.0	fEnglish	 puppet
U+5080.1	fUTF8	傀
U+5080.1	fCNS11643	1-5E2E
U+5080.1	fBig5	B3C8
U+5080.1	fGB	0-3F7E
U+5080.1	fR/S	 9.10
U+5080.1	fTotalStrokes	 12
U+5080.1	fCangjie	OHUI
U+5080.1	fFourCorner	26213
U+5080.1	fCantonese	kwai1
U+5080.1	fMandarin	gui1
U+5080.1	fEnglish	great; gigantic; wonderful
U+5081.0	fUTF8	傁
U+5081.0	fCNS11643	3-3975
U+5081.0	fGB	5-3246
U+5081.0	fR/S	 9.10; 116.10
U+5081.0	fTotalStrokes	 12
U+5081.0	fCangjie	OHXE
U+5081.0	fFourCorner	27240
U+5081.0	fHakka	 seu3
U+5081.0	fCantonese	sau2
U+5081.0	fMandarin	sou3
U+5081.0	fEnglish	old man
U+5082.0	fUTF8	傂
U+5082.0	fCNS11643	2-3A42
U+5082.0	fBig5	D85E
U+5082.0	fGB	5-3258
U+5082.0	fR/S	 9.10
U+5082.0	fTotalStrokes	 12
U+5082.0	fCangjie	OHYU
U+5082.0	fFourCorner	21211
U+5082.0	fCantonese	zi6 (also); ci4
U+5082.0	fMandarin	chi3; zhi4
U+5083.0	fUTF8	傃
U+5083.0	fCNS11643	2-3A3B
U+5083.0	fBig5	D857
U+5083.0	fGB	3-3173
U+5083.0	fR/S	 9.10
U+5083.0	fTotalStrokes	 12
U+5083.0	fCangjie	OQMF
U+5083.0	fFourCorner	25293
U+5083.0	fHakka	 su5; se5; sii5
U+5083.0	fCantonese	sou3
U+5083.0	fMandarin	su4
U+5083.0	fEnglish	facing; turned towards
U+5084.0	fUTF8	傄
U+5084.0	fCNS11643	4-3634
U+5084.0	fGB	3-3176
U+5084.0	fR/S	 9.10
U+5084.0	fTotalStrokes	 12
U+5084.0	fCangjie	OKCU
U+5084.0	fFourCorner	24260
U+5084.0	fHakka	 hat7
U+5084.0	fCantonese	gwaat3
U+5084.0	fMandarin	xia1
U+5084.0	fEnglish	stout; vigorous; brave
U+5085.0	fUTF8	傅
U+5085.0	fCNS11643	1-5E2B
U+5085.0	fBig5	B3C5
U+5085.0	fGB	0-3835
U+5085.0	fR/S	 9.10
U+5085.0	fTotalStrokes	 12
U+5085.0	fCangjie	OIBI
U+5085.0	fFourCorner	23242
U+5085.0	fHakka	  fu5; fu6; fu3
U+5085.0	fCantonese	fu6; fu2 (also); fu1
U+5085.0	fMandarin	fu4; fu1
U+5085.0	fEnglish	[1] tutor; teacher; teach [2] go together with; add to; be attached to [3] a Chinese family name
U+5085.1	fUTF8	傅
U+5085.1	fCNS11643	1-5E2B
U+5085.1	fBig5	B3C5
U+5085.1	fGB	0-3835
U+5085.1	fR/S	 9.10
U+5085.1	fTotalStrokes	 12
U+5085.1	fCangjie	OIBI
U+5085.1	fFourCorner	23242
U+5086.0	fUTF8	傆
U+5086.0	fCNS11643	4-3633
U+5086.0	fGB	5-3256
U+5086.0	fR/S	  9.10
U+5086.0	fTotalStrokes	 12
U+5086.0	fCangjie	OMHF
U+5086.0	fFourCorner	21290
U+5086.0	fHakka	 ngien5; ngien6
U+5086.0	fCantonese	jyun6
U+5086.0	fMandarin	yuan4
U+5086.0	fEnglish	an unprincipled and cunning man
U+5087.0	fUTF8	傇
U+5087.0	fCNS11643	2-3A43
U+5087.0	fBig5	D85F
U+5087.0	fGB	5-3240
U+5087.0	fR/S	 9.10
U+5087.0	fTotalStrokes	 12
U+5087.0	fCangjie	OTSJ
U+5087.0	fFourCorner	24241
U+5087.0	fCantonese	jung5
U+5087.0	fMandarin	rong3; jia1
U+5088.0	fUTF8	傈
U+5088.0	fCNS11643	3-3971
U+5088.0	fGB	0-407C
U+5088.0	fR/S	  9.10
U+5088.0	fTotalStrokes	 12
U+5088.0	fCangjie	OMWD
U+5088.0	fFourCorner	21290
U+5088.0	fHakka	  lit8
U+5088.0	fCantonese	leot6
U+5088.0	fMandarin	li4
U+5088.0	fEnglish	tribe
U+5089.0	fUTF8	傉
U+5089.0	fCNS11643	3-3972
U+5089.0	fGB	3-3175
U+5089.0	fR/S	  9.10
U+5089.0	fTotalStrokes	 12
U+5089.0	fCangjie	OMVI
U+5089.0	fFourCorner	21240
U+5089.0	fCantonese	nuk6; luk6 (also)
U+5089.0	fMandarin	ru4
U+508A.0	fUTF8	傊
U+508A.0	fCNS11643	4-3636
U+508A.0	fGB	E-222E
U+508A.0	fR/S	  9.10
U+508A.0	fTotalStrokes	 12
U+508A.0	fCangjie	ORBC
U+508A.0	fFourCorner	26280
U+508A.0	fCantonese	wan6
U+508A.0	fMandarin	yun3
U+508B.0	fUTF8	傋
U+508B.0	fCNS11643	2-3A39
U+508B.0	fBig5	D855
U+508B.0	fGB	3-3174
U+508B.0	fR/S	 9.10
U+508B.0	fTotalStrokes	 12
U+508B.0	fCangjie	OTTB
U+508B.0	fFourCorner	25247
U+508B.0	fCantonese	gong2; gau3
U+508B.0	fMandarin	gou4; gang3
U+508C.0	fUTF8	傌
U+508C.0	fCNS11643	2-3A3C
U+508C.0	fBig5	D858
U+508C.0	fGB	E-222F
U+508C.0	fR/S	 9.10
U+508C.0	fTotalStrokes	 12
U+508C.0	fCangjie	OSQF
U+508C.0	fFourCorner	21227
U+508C.0	fHakka	  ma1;  ma5; ma6
U+508C.0	fCantonese	maa6; maa3 (also)
U+508C.0	fMandarin	ma4
U+508C.0	fEnglish	 curse; revile; abuse scold
U+508D.0	fUTF8	傍
U+508D.0	fCNS11643	1-5E2A
U+508D.0	fBig5	B3C4
U+508D.0	fGB	0-3078
U+508D.0	fR/S	 9.10
U+508D.0	fTotalStrokes	 12
U+508D.0	fCangjie	OYBS
U+508D.0	fFourCorner	20227
U+508D.0	fHakka	 pong2;  pong3; bong3; pong5; bong5; pong6
U+508D.0	fCantonese	bong6; pong4; bong4 (also); pong6
U+508D.0	fMandarin	pang2
U+508D.0	fEnglish	by the side of; beside
U+508D.1	fUTF8	傍
U+508D.1	fCNS11643	1-5E2A
U+508D.1	fBig5	B3C4
U+508D.1	fGB	0-3078
U+508D.1	fR/S	 9.10
U+508D.1	fTotalStrokes	 12
U+508D.1	fCangjie	OYBS
U+508D.1	fFourCorner	20227
U+508D.1	fMandarin	bang1; beng1
U+508D.1	fEnglish	near; approaching
U+508D.2	fUTF8	傍
U+508D.2	fCNS11643	1-5E2A
U+508D.2	fBig5	B3C4
U+508D.2	fGB	0-3078
U+508D.2	fR/S	 9.10
U+508D.2	fTotalStrokes	 12
U+508D.2	fCangjie	OYBS
U+508D.2	fFourCorner	20227
U+508D.2	fMandarin	bang4
U+508D.2	fEnglish	[1] depend on [2] draw near; be close to
U+508E.0	fUTF8	傎
U+508E.0	fCNS11643	2-3A3D
U+508E.0	fBig5	D859
U+508E.0	fGB	5-3255
U+508E.0	fR/S	 9.10
U+508E.0	fTotalStrokes	 12
U+508E.0	fCangjie	OJBC
U+508E.0	fFourCorner	24281
U+508E.0	fHakka	 dien1
U+508E.0	fCantonese	din1
U+508E.0	fMandarin	dian1
U+508E.0	fEnglish	topple
U+508F.0	fUTF8	傏
U+508F.0	fCNS11643	3-396F
U+508F.0	fGB	5-325E
U+508F.0	fR/S	  9.10
U+508F.0	fTotalStrokes	 12
U+508F.0	fCangjie	OILR
U+508F.0	fFourCorner	20260
U+508F.0	fHakka	  tong2
U+508F.0	fCantonese	tong4
U+508F.0	fMandarin	tang2
U+508F.0	fEnglish	 ward off parry keep out (wind, rain, or cold)
U+5090.0	fUTF8	傐
U+5090.0	fCNS11643	4-3631
U+5090.0	fGB	E-2230
U+5090.0	fR/S	  9.10
U+5090.0	fTotalStrokes	 12
U+5090.0	fCangjie	OYRB
U+5090.0	fFourCorner	20220
U+5090.0	fCantonese	hou6
U+5090.0	fMandarin	hao4
U+5091.0	fUTF8	傑
U+5091.0	fCNS11643	1-5E2D
U+5091.0	fBig5	B3C7
U+5091.0	fGB	8-2C79
U+5091.0	fR/S	 9.10
U+5091.0	fTotalStrokes	 12
U+5091.0	fCangjie	ONQD
U+5091.0	fFourCorner	25294
U+5091.0	fHakka	  kiet8; ket8; k'iat8
U+5091.0	fCantonese	git6
U+5091.0	fMandarin	jie2
U+5091.0	fEnglish	[1] hero [2] outstanding; remarkable; extraordinarily
U+5092.0	fUTF8	傒
U+5092.0	fCNS11643	2-3A41
U+5092.0	fBig5	D85D
U+5092.0	fGB	5-325A
U+5092.0	fR/S	 9.10
U+5092.0	fTotalStrokes	 12
U+5092.0	fCangjie	OBVK
U+5092.0	fFourCorner	22234
U+5092.0	fHakka	 hi1; hi2; he1
U+5092.0	fCantonese	hai4; hai6; hoi4
U+5092.0	fMandarin	xi1; xi2
U+5092.0	fEnglish	 a personal name servant
U+5092.1	fUTF8	傒
U+5092.1	fCNS11643	2-3A41
U+5092.1	fBig5	D85D
U+5092.1	fGB	5-325A
U+5092.1	fR/S	 9.10
U+5092.1	fTotalStrokes	 12
U+5092.1	fCangjie	OBVK
U+5092.1	fFourCorner	22234
U+5092.1	fMandarin	xi4
U+5093.0	fUTF8	傓
U+5093.0	fCNS11643	4-363C
U+5093.0	fGB	5-325F
U+5093.0	fR/S	  9.10
U+5093.0	fTotalStrokes	 12
U+5093.0	fCangjie	OISM
U+5093.0	fFourCorner	23220
U+5093.0	fHakka	 shen5; sen5; san5; shan5
U+5093.0	fCantonese	sin3
U+5093.0	fMandarin	shan4
U+5093.0	fEnglish	 excite; inflame
U+5094.0	fUTF8	傔
U+5094.0	fCNS11643	2-3A37
U+5094.0	fBig5	D853
U+5094.0	fGB	3-317E
U+5094.0	fR/S	 9.10
U+5094.0	fTotalStrokes	 12
U+5094.0	fCangjie	OTXC
U+5094.0	fFourCorner	28237
U+5094.0	fHakka	  kiam5
U+5094.0	fCantonese	him3
U+5094.0	fMandarin	qian4
U+5094.0	fEnglish	servant
U+5095.0	fUTF8	傕
U+5095.0	fCNS11643	2-3A36
U+5095.0	fBig5	D852
U+5095.0	fGB	3-3222
U+5095.0	fR/S	 9.10
U+5095.0	fTotalStrokes	 12
U+5095.0	fCangjie	OOBG
U+5095.0	fFourCorner	24214
U+5095.0	fHakka	 kok7; gok7
U+5095.0	fCantonese	gok3
U+5095.0	fMandarin	jue2
U+5095.0	fEnglish	a Chinese family name
U+5096.0	fUTF8	傖
U+5096.0	fCNS11643	1-5E2F
U+5096.0	fBig5	B3C9
U+5096.0	fGB	1-5877
U+5096.0	fR/S	 9.10
U+5096.0	fTotalStrokes	 12
U+5096.0	fCangjie	OOIR
U+5096.0	fFourCorner	28267
U+5096.0	fHakka	  cong1
U+5096.0	fCantonese	cong1; caang1; cam2; caang4
U+5096.0	fMandarin	cang1; cheng2; cang4
U+5096.0	fEnglish	[1] vulgar person; cheap; lowly [2] confused; disorderly
U+5097.0	fUTF8	傗
U+5097.0	fCNS11643	3-3970
U+5097.0	fGB	3-317D
U+5097.0	fR/S	  9.10
U+5097.0	fTotalStrokes	 12
U+5097.0	fCangjie	OYVW
U+5097.0	fFourCorner	20260
U+5097.0	fHakka	  hiuk7
U+5097.0	fCantonese	cuk1
U+5097.0	fMandarin	chu4
U+5097.0	fEnglish	bent-down
U+5098.0	fUTF8	傘
U+5098.0	fCNS11643	1-5E30
U+5098.0	fBig5	B3CA
U+5098.0	fGB	1-4921
U+5098.0	fR/S	 9.10
U+5098.0	fTotalStrokes	 12
U+5098.0	fCangjie	OOOJ
U+5098.0	fFourCorner	80408
U+5098.0	fHakka	  san5; zha1; san3; shan3
U+5098.0	fCantonese	saan3
U+5098.0	fMandarin	san3
U+5098.0	fEnglish	[1] umbrella; parasol [2] parachute
U+5099.0	fUTF8	備
U+5099.0	fCNS11643	1-5E2C
U+5099.0	fBig5	B3C6
U+5099.0	fGB	1-3138
U+5099.0	fR/S	 9.10
U+5099.0	fTotalStrokes	 12
U+5099.0	fCangjie	OTHB
U+5099.0	fFourCorner	24227
U+5099.0	fHakka	 pi5; pi6; pi3
U+5099.0	fCantonese	bei6
U+5099.0	fMandarin	bei4
U+5099.0	fEnglish	[1] sense of completeness; perfection [2] be equipped with [3] get ready [4] prepare against [5] fully; in every possible way
U+509A.0	fUTF8	傚
U+509A.0	fCNS11643	1-5E31
U+509A.0	fBig5	B3CB
U+509A.0	fGB	E-2231
U+509A.0	fR/S	 9.10
U+509A.0	fTotalStrokes	 12
U+509A.0	fCangjie	OYKK
U+509A.0	fFourCorner	28240
U+509A.0	fHakka	  hau5; hau3
U+509A.0	fCantonese	haau6
U+509A.0	fMandarin	xiao4
U+509A.0	fEnglish	to model after; imitate; mimic; emulate; copy
U+509B.0	fUTF8	傛
U+509B.0	fCNS11643	2-3A35
U+509B.0	fBig5	D851
U+509B.0	fGB	3-3221
U+509B.0	fR/S	 9.10
U+509B.0	fTotalStrokes	 12
U+509B.0	fCangjie	OJCR
U+509B.0	fFourCorner	23268
U+509B.0	fHakka	 jung2; rung2
U+509B.0	fCantonese	jung6 (also); jung5
U+509B.0	fMandarin	yong3; yong2; rong2
U+509B.0	fEnglish	uneasy; not at rest
U+509B.1	fUTF8	傛
U+509B.1	fCNS11643	2-3A35
U+509B.1	fBig5	D851
U+509B.1	fGB	3-3221
U+509B.1	fR/S	 9.10
U+509B.1	fTotalStrokes	 12
U+509B.1	fCangjie	OJCR
U+509B.1	fFourCorner	23268
U+509B.1	fCantonese	jung4
U+509C.0	fUTF8	傜
U+509C.0	fCNS11643	2-3A40
U+509C.0	fBig5	D85C
U+509C.0	fGB	5-3259
U+509C.0	fR/S	 9.10
U+509C.0	fTotalStrokes	 12
U+509C.0	fCangjie	OBOU
U+509C.0	fFourCorner	27272
U+509C.0	fHakka	  jau2; rau2; jeu2
U+509C.0	fCantonese	jiu4
U+509C.0	fMandarin	yao2
U+509C.0	fEnglish	a minority tribe
U+509D.0	fUTF8	傝
U+509D.0	fCNS11643	2-3A3E
U+509D.0	fBig5	D85A
U+509D.0	fGB	3-317B
U+509D.0	fR/S	 9.10
U+509D.0	fTotalStrokes	 12
U+509D.0	fCangjie	OASM
U+509D.0	fFourCorner	26227
U+509D.0	fHakka	  tap7
U+509D.0	fCantonese	taam3; taap3; daap3
U+509D.0	fMandarin	ta4; tan4
U+509D.0	fEnglish	[1] distressed in mind [2] low; sordid
U+509E.0	fUTF8	傞
U+509E.0	fCNS11643	2-3A38
U+509E.0	fBig5	D854
U+509E.0	fGB	3-3171
U+509E.0	fR/S	 9.10
U+509E.0	fTotalStrokes	 12
U+509E.0	fCangjie	OTQM
U+509E.0	fFourCorner	28211
U+509E.0	fHakka	 so1; co1
U+509E.0	fCantonese	co1; so1
U+509E.0	fMandarin	suo1
U+509E.0	fEnglish	unsteady
U+509F.0	fUTF8	傟
U+509F.0	fCNS11643	4-3639
U+509F.0	fGB	5-325B
U+509F.0	fR/S	  9.10
U+509F.0	fTotalStrokes	 12
U+509F.0	fCangjie	OCIM
U+509F.0	fFourCorner	28220
U+509F.0	fCantonese	ong2; ngong2 (also)
U+509F.0	fMandarin	yang3
U+50A0.0	fUTF8	傠
U+50A0.0	fCNS11643	4-3630
U+50A0.0	fGB	3-3074
U+50A0.0	fR/S	  9.10
U+50A0.0	fTotalStrokes	 12
U+50A0.0	fCangjie	OYRI
U+50A0.0	fFourCorner	24200
U+50A0.0	fCantonese	fat6
U+50A0.0	fMandarin	fa1; fa2
U+50A1.0	fUTF8	傡
U+50A1.0	fCNS11643	4-363A
U+50A1.0	fGB	E-2232
U+50A1.0	fR/S	 9.10
U+50A1.0	fTotalStrokes	 12
U+50A1.0	fCangjie	OYTT
U+50A1.0	fFourCorner	20210
U+50A1.0	fCantonese	bing6
U+50A1.0	fMandarin	bing4; bing1
U+50A2.0	fUTF8	傢
U+50A2.0	fCNS11643	1-5E29
U+50A2.0	fBig5	B3C3
U+50A2.0	fGB	1-7841
U+50A2.0	fR/S	 9.10
U+50A2.0	fTotalStrokes	 12
U+50A2.0	fCangjie	OJMO
U+50A2.0	fFourCorner	23232
U+50A2.0	fHakka	  ga1
U+50A2.0	fCantonese	gaa1
U+50A2.0	fMandarin	jia1; jia5; jie5
U+50A2.0	fEnglish	[1] furniture [2] tool or tools
U+50A3.0	fUTF8	傣
U+50A3.0	fCNS11643	2-3A3A
U+50A3.0	fBig5	D856
U+50A3.0	fGB	0-3476
U+50A3.0	fR/S	 9.10
U+50A3.0	fTotalStrokes	 12
U+50A3.0	fCangjie	OQKE
U+50A3.0	fFourCorner	25299
U+50A3.0	fCantonese	daai2; taai3 (also)
U+50A3.0	fMandarin	dai3; tai4
U+50A3.0	fEnglish	the Dai minority living in South China
U+50A4.0	fUTF8	傤
U+50A4.0	fGB	8-2E27
U+50A4.0	fR/S	 9.10; 9.13
U+50A4.0	fTotalStrokes	 12
U+50A4.0	fCangjie	OJIQ
U+50A4.0	fCantonese	zoi3
U+50A4.0	fMandarin	zai4
U+50A5.0	fUTF8	傥
U+50A5.0	fGB	0-594E
U+50A5.0	fR/S	 9.10
U+50A5.0	fTotalStrokes	 12
U+50A5.0	fCangjie	OFBU
U+50A5.0	fHakka	 tong3
U+50A5.0	fCantonese	tong2
U+50A5.0	fMandarin	tang3
U+50A5.0	fEnglish	simplified form of &#x513B;
U+50A6.0	fUTF8	傦
U+50A6.0	fCNS11643	3-644D
U+50A6.0	fGB	E-2233
U+50A6.0	fR/S	 9.10
U+50A6.0	fTotalStrokes	 12
U+50A6.0	fCangjie	OBBB
U+50A6.0	fFourCorner	2722
U+50A6.0	fCantonese	gwat1
U+50A7.0	fUTF8	傧
U+50A7.0	fGB	0-594F
U+50A7.0	fR/S	 9.10
U+50A7.0	fTotalStrokes	 12
U+50A7.0	fCangjie	OJOC
U+50A7.0	fHakka	 bin5
U+50A7.0	fCantonese	ban3; ban1 (also)
U+50A7.0	fMandarin	bin4; bin1
U+50A7.0	fEnglish	simplified form of &#x5110;
U+50A8.0	fUTF8	储
U+50A8.0	fGB	0-3422
U+50A8.0	fR/S	 9.10
U+50A8.0	fTotalStrokes	 12
U+50A8.0	fCangjie	OIVA
U+50A8.0	fHakka	 shu2
U+50A8.0	fCantonese	cyu5
U+50A8.0	fMandarin	chu2; chu3
U+50A8.0	fEnglish	simplified form of &#x5132;
U+50A9.0	fUTF8	傩
U+50A9.0	fGB	0-5950
U+50A9.0	fR/S	 9.10
U+50A9.0	fTotalStrokes	 12
U+50A9.0	fCangjie	OEOG
U+50A9.0	fHakka	 no2
U+50A9.0	fCantonese	no4; lo4 (also)
U+50A9.0	fMandarin	nuo2
U+50A9.0	fEnglish	simplified form of &#x513A;
U+50AA.0	fUTF8	傪
U+50AA.0	fCNS11643	3-4039
U+50AA.0	fGB	3-315C
U+50AA.0	fR/S	  9.11
U+50AA.0	fTotalStrokes	 13
U+50AA.0	fCangjie	OIIH
U+50AA.0	fFourCorner	23220
U+50AA.0	fHakka	  cam1
U+50AA.0	fCantonese	caam1
U+50AA.0	fMandarin	can1; san3
U+50AA.0	fEnglish	fine-looking; handsome
U+50AB.0	fUTF8	傫
U+50AB.0	fCNS11643	3-403C
U+50AB.0	fGB	3-3228
U+50AB.0	fR/S	 9.11
U+50AB.0	fTotalStrokes	 13
U+50AB.0	fCangjie	OWVF
U+50AB.0	fFourCorner	26290
U+50AB.0	fCantonese	leoi5
U+50AB.0	fMandarin	lei3; lei4
U+50AB.0	fEnglish	 tired; exhausted a Chinese family name
U+50AC.0	fUTF8	催
U+50AC.0	fCNS11643	1-6331
U+50AC.0	fBig5	B6CA
U+50AC.0	fGB	0-345F
U+50AC.0	fR/S	 9.11
U+50AC.0	fTotalStrokes	 13
U+50AC.0	fCangjie	OUOG
U+50AC.0	fFourCorner	22214
U+50AC.0	fHakka	  cui1
U+50AC.0	fCantonese	ceoi1
U+50AC.0	fMandarin	cui1
U+50AC.0	fEnglish	 hasten; press; urge; hurry
U+50AD.0	fUTF8	傭
U+50AD.0	fCNS11643	1-632B
U+50AD.0	fBig5	B6C4
U+50AD.0	fGB	1-5336
U+50AD.0	fR/S	 9.11
U+50AD.0	fTotalStrokes	 13
U+50AD.0	fCangjie	OILB
U+50AD.0	fFourCorner	20227
U+50AD.0	fHakka	  jung2; rung2; rin2; jin2
U+50AD.0	fCantonese	jung4; cung1
U+50AD.0	fMandarin	yong1; yong2; chong1; yong4; zhong3
U+50AD.0	fEnglish	[1] hire; employ; charter [2] servant; domestic help
U+50AD.1	fUTF8	傭
U+50AD.1	fCNS11643	1-632B
U+50AD.1	fBig5	B6C4
U+50AD.1	fGB	1-5336
U+50AD.1	fR/S	 9.11
U+50AD.1	fTotalStrokes	 13
U+50AD.1	fCangjie	OILB
U+50AD.1	fFourCorner	20227
U+50AD.1	fCantonese	jung2
U+50AE.0	fUTF8	傮
U+50AE.0	fCNS11643	2-415C
U+50AE.0	fBig5	DCB7
U+50AE.0	fGB	3-3224
U+50AE.0	fR/S	 9.11
U+50AE.0	fTotalStrokes	 13
U+50AE.0	fCangjie	OTWA
U+50AE.0	fFourCorner	24266
U+50AE.0	fHakka	  zau1
U+50AE.0	fCantonese	zou1; cou4
U+50AE.0	fMandarin	zao1; cao2
U+50AE.0	fEnglish	finish; to go around
U+50AE.1	fUTF8	傮
U+50AE.1	fCNS11643	2-415C
U+50AE.1	fBig5	DCB7
U+50AE.1	fGB	3-3224
U+50AE.1	fR/S	 9.11
U+50AE.1	fTotalStrokes	 13
U+50AE.1	fCangjie	OTWA
U+50AE.1	fFourCorner	24266
U+50AF.0	fUTF8	傯
U+50AF.0	fCNS11643	1-6334
U+50AF.0	fBig5	B6CD
U+50AF.0	fGB	E-2234
U+50AF.0	fR/S	 9.11
U+50AF.0	fTotalStrokes	 13
U+50AF.0	fCangjie	OHWP
U+50AF.0	fFourCorner	27232
U+50AF.0	fHakka	  zung3
U+50AF.0	fCantonese	zung2; zung3
U+50AF.0	fMandarin	zong3; cong1; zong4
U+50AF.0	fEnglish	urgent
U+50B0.0	fUTF8	傰
U+50B0.0	fCNS11643	2-4162
U+50B0.0	fBig5	DCBD
U+50B0.0	fGB	3-3229
U+50B0.0	fR/S	 9.11
U+50B0.0	fTotalStrokes	 13
U+50B0.0	fCangjie	OUBB
U+50B0.0	fFourCorner	22127
U+50B0.0	fHakka	 pen1; pung2
U+50B0.0	fCantonese	pang4; bang1
U+50B0.0	fMandarin	peng2; beng4
U+50B1.0	fUTF8	傱
U+50B1.0	fCNS11643	2-4165
U+50B1.0	fBig5	DCC0
U+50B1.0	fGB	3-3061
U+50B1.0	fR/S	 9.11
U+50B1.0	fTotalStrokes	 13
U+50B1.0	fCangjie	OHOO
U+50B1.0	fFourCorner	28281
U+50B1.0	fCantonese	sung2
U+50B1.0	fMandarin	song3
U+50B2.0	fUTF8	傲
U+50B2.0	fCNS11643	1-632D
U+50B2.0	fBig5	B6C6
U+50B2.0	fGB	0-3041
U+50B2.0	fR/S	 9.11
U+50B2.0	fTotalStrokes	 13
U+50B2.0	fCangjie	OGSK
U+50B2.0	fFourCorner	28240
U+50B2.0	fHakka	 ngau5; ngau3; ngau6
U+50B2.0	fCantonese	ngou6; ou6 (also)
U+50B2.0	fMandarin	ao4
U+50B2.0	fEnglish	[1] proud; haughty; overbearing [2] disdain; despise; look down upon [3] rash and impatient
U+50B3.0	fUTF8	傳
U+50B3.0	fCNS11643	1-632E
U+50B3.0	fBig5	B6C7
U+50B3.0	fGB	1-342B
U+50B3.0	fR/S	 9.11
U+50B3.0	fTotalStrokes	 13
U+50B3.0	fCangjie	OJII
U+50B3.0	fFourCorner	25243
U+50B3.0	fHakka	 chon2; con3; chon3; zon3; chon6; chon5; con5
U+50B3.0	fCantonese	cyun4; zyun2 (also); zyun3
U+50B3.0	fMandarin	chuan2
U+50B3.0	fEnglish	[1] pass (a bal, an order, learning, etc.) [2] summon [3] propagate; transmit; disseminate [4] preach
U+50B3.1	fUTF8	傳
U+50B3.1	fCNS11643	1-632E
U+50B3.1	fBig5	B6C7
U+50B3.1	fGB	1-342B
U+50B3.1	fR/S	 9.11
U+50B3.1	fTotalStrokes	 13
U+50B3.1	fCangjie	OJII
U+50B3.1	fFourCorner	25243
U+50B3.1	fHakka	 con2
U+50B3.1	fCantonese	zyun6
U+50B3.1	fMandarin	zhuan4
U+50B3.1	fEnglish	 biography
U+50B4.0	fUTF8	傴
U+50B4.0	fCNS11643	2-415F
U+50B4.0	fBig5	DCBA
U+50B4.0	fGB	1-5871
U+50B4.0	fR/S	 9.11
U+50B4.0	fTotalStrokes	 13
U+50B4.0	fCangjie	OSRR
U+50B4.0	fFourCorner	21216
U+50B4.0	fHakka	 eu1; ku1; ki1; ri1; ji3; ji1; eu3
U+50B4.0	fCantonese	jyu2
U+50B4.0	fMandarin	yu3
U+50B4.0	fEnglish	hunchbacked; humpback; stoop
U+50B5.0	fUTF8	債
U+50B5.0	fCNS11643	1-632C
U+50B5.0	fBig5	B6C5
U+50B5.0	fGB	1-552E
U+50B5.0	fR/S	 9.11
U+50B5.0	fTotalStrokes	 13
U+50B5.0	fCangjie	OQMC
U+50B5.0	fFourCorner	25286
U+50B5.0	fHakka	  zai5; zit7
U+50B5.0	fCantonese	zaai3
U+50B5.0	fMandarin	zhai4
U+50B5.0	fEnglish	 debt; obligation
U+50B6.0	fUTF8	傶
U+50B6.0	fCNS11643	2-4168
U+50B6.0	fBig5	DCC3
U+50B6.0	fGB	5-3263
U+50B6.0	fR/S	 9.11
U+50B6.0	fTotalStrokes	 13
U+50B6.0	fCangjie	OIHF
U+50B6.0	fFourCorner	23250
U+50B6.0	fCantonese	cik1; cau3
U+50B6.0	fMandarin	cou4; zu2; cuo4; xi4
U+50B7.0	fUTF8	傷
U+50B7.0	fCNS11643	1-6332
U+50B7.0	fBig5	B6CB
U+50B7.0	fGB	1-494B
U+50B7.0	fR/S	 9.11
U+50B7.0	fTotalStrokes	 13
U+50B7.0	fCangjie	OOAH
U+50B7.0	fFourCorner	28227
U+50B7.0	fHakka	  song1; shong1
U+50B7.0	fCantonese	soeng1
U+50B7.0	fMandarin	shang1
U+50B7.0	fEnglish	[1] cut; wound; injury [2] cut; injure [3] grief; grieve; distressed [4] impede; impediment [5] hurt (feelings) [6] make sick
U+50B8.0	fUTF8	傸
U+50B8.0	fCNS11643	2-4169
U+50B8.0	fBig5	DCC4
U+50B8.0	fGB	5-3265
U+50B8.0	fR/S	 9.11
U+50B8.0	fTotalStrokes	 13
U+50B8.0	fCangjie	OKKK
U+50B8.0	fFourCorner	24234
U+50B8.0	fHakka	 song3
U+50B8.0	fCantonese	cong2; song2
U+50B8.0	fMandarin	qiang3
U+50B8.0	fEnglish	evil; wicked ; perverse
U+50B9.0	fUTF8	傹
U+50B9.0	fCNS11643	3-403A
U+50B9.0	fGB	3-322C
U+50B9.0	fR/S	  9.11
U+50B9.0	fTotalStrokes	 13
U+50B9.0	fCangjie	OYTU
U+50B9.0	fFourCorner	20210
U+50B9.0	fCantonese	goeng1
U+50B9.0	fMandarin	jing4; jiang1
U+50BA.0	fUTF8	傺
U+50BA.0	fCNS11643	2-4164
U+50BA.0	fBig5	DCBF
U+50BA.0	fGB	0-5951
U+50BA.0	fR/S	 9.11
U+50BA.0	fTotalStrokes	 13
U+50BA.0	fCangjie	OBOF
U+50BA.0	fFourCorner	22291
U+50BA.0	fHakka	 zi5; che5; ce5
U+50BA.0	fCantonese	cai3
U+50BA.0	fMandarin	chi4
U+50BA.0	fEnglish	[1] hinder [2] be disappointed
U+50BB.0	fUTF8	傻
U+50BB.0	fCNS11643	1-6333
U+50BB.0	fBig5	B6CC
U+50BB.0	fGB	0-4935
U+50BB.0	fR/S	 9.11
U+50BB.0	fTotalStrokes	 13
U+50BB.0	fCangjie	OHCE
U+50BB.0	fFourCorner	26247
U+50BB.0	fHakka	 so2; sa3
U+50BB.0	fCantonese	so4; so2 (also)
U+50BB.0	fMandarin	sha3
U+50BB.0	fEnglish	 foolish; silly; stupid imbecile
U+50BC.0	fUTF8	傼
U+50BC.0	fCNS11643	3-4037
U+50BC.0	fGB	E-2235
U+50BC.0	fR/S	 9.11; 9.12
U+50BC.0	fTotalStrokes	 13; 14
U+50BC.0	fCangjie	OTLO
U+50BC.0	fFourCorner	24230
U+50BC.0	fHakka	 hon3; han3; han1
U+50BC.0	fCantonese	hon3
U+50BC.0	fMandarin	han4
U+50BC.0	fEnglish	surname
U+50BD.0	fUTF8	傽
U+50BD.0	fCNS11643	2-4159
U+50BD.0	fBig5	DCB4
U+50BD.0	fGB	3-322B
U+50BD.0	fR/S	 9.11
U+50BD.0	fTotalStrokes	 13
U+50BD.0	fCangjie	OYTJ
U+50BD.0	fFourCorner	20246
U+50BD.0	fHakka	 zhong1; zong1
U+50BD.0	fCantonese	zoeng1
U+50BD.0	fMandarin	zhang1
U+50BD.0	fEnglish	terror-stricken
U+50BD.1	fUTF8	傽
U+50BD.1	fCNS11643	2-4159
U+50BD.1	fBig5	DCB4
U+50BD.1	fGB	3-322B
U+50BD.1	fR/S	 9.11
U+50BD.1	fTotalStrokes	 13
U+50BD.1	fCangjie	OYTJ
U+50BD.1	fFourCorner	20246
U+50BD.2	fUTF8	傽
U+50BD.2	fCNS11643	2-4159
U+50BD.2	fBig5	DCB4
U+50BD.2	fGB	3-322B
U+50BD.2	fR/S	 9.11
U+50BD.2	fTotalStrokes	 13
U+50BD.2	fCangjie	OYTJ
U+50BD.2	fFourCorner	20246
U+50BE.0	fUTF8	傾
U+50BE.0	fCNS11643	1-6330
U+50BE.0	fBig5	B6C9
U+50BE.0	fGB	1-4763
U+50BE.0	fR/S	 9.11
U+50BE.0	fTotalStrokes	 13
U+50BE.0	fCangjie	OPMC
U+50BE.0	fFourCorner	21286
U+50BE.0	fHakka	 kin1; ken1; kin3
U+50BE.0	fCantonese	king1
U+50BE.0	fMandarin	qing1; qing2
U+50BE.0	fEnglish	[1] slant; bend [2] collapse; fall flat; upset; subvert [3] pour out [4] exhaust (one's wealth, etc.); exert oneself to do (something) [5] admire; be fascinated; be intrigued
U+50BE.1	fUTF8	傾
U+50BE.1	fCNS11643	1-6330
U+50BE.1	fBig5	B6C9
U+50BE.1	fGB	1-4763
U+50BE.1	fR/S	 9.11
U+50BE.1	fTotalStrokes	 13
U+50BE.1	fCangjie	OPMC
U+50BE.1	fFourCorner	21286
U+50BE.1	fMandarin	keng1
U+50BE.1	fEnglish	 frame or implicate a person
U+50BF.0	fUTF8	傿
U+50BF.0	fCNS11643	2-415A
U+50BF.0	fBig5	DCB5
U+50BF.0	fGB	3-3223
U+50BF.0	fR/S	 9.11
U+50BF.0	fTotalStrokes	 13
U+50BF.0	fCangjie	OMYF
U+50BF.0	fFourCorner	21227
U+50BF.0	fHakka	 jen2; ren2; jen1
U+50BF.0	fCantonese	jin3; jin1; jin4
U+50BF.0	fMandarin	yan4; yan1
U+50BF.0	fEnglish	 act as a broker
U+50C0.0	fUTF8	僀
U+50C0.0	fCNS11643	4-3C29
U+50C0.0	fGB	5-3241
U+50C0.0	fR/S	  9.11
U+50C0.0	fTotalStrokes	 13
U+50C0.0	fCangjie	OKPB
U+50C0.0	fFourCorner	24220
U+50C0.0	fCantonese	dai3
U+50C0.0	fMandarin	di4
U+50C1.0	fUTF8	僁
U+50C1.0	fCNS11643	2-4163
U+50C1.0	fBig5	DCBE
U+50C1.0	fGB	3-322A
U+50C1.0	fR/S	 9.11
U+50C1.0	fTotalStrokes	 13
U+50C1.0	fCangjie	OHDP
U+50C1.0	fFourCorner	22239
U+50C1.0	fHakka	  siet7; sit7
U+50C1.0	fCantonese	sit3; sat1; seot1; sik1
U+50C1.0	fMandarin	xi1; xie4
U+50C1.0	fEnglish	 speak in a whisper
U+50C2.0	fUTF8	僂
U+50C2.0	fCNS11643	2-4161
U+50C2.0	fBig5	DCBC
U+50C2.0	fGB	1-594D
U+50C2.0	fR/S	 9.11
U+50C2.0	fTotalStrokes	 13
U+50C2.0	fCangjie	OLLV
U+50C2.0	fFourCorner	25244
U+50C2.0	fHakka	  leu2; li1; liu2
U+50C2.0	fCantonese	lau4; leoi5
U+50C2.0	fMandarin	lou2; luu3; l3; lou4; qiu2
U+50C2.0	fEnglish	[1] hunchbacked; deformed; humpback [2] bent [3] a Chinese family name
U+50C3.0	fUTF8	僃
U+50C3.0	fCNS11643	3-403D
U+50C3.0	fGB	E-2236
U+50C3.0	fR/S	 9.11
U+50C3.0	fTotalStrokes	 13
U+50C3.0	fCangjie	OTPB
U+50C3.0	fFourCorner	21220
U+50C3.0	fCantonese	bei6
U+50C3.0	fMandarin	bei4
U+50C3.0	fEnglish	prepared
U+50C4.0	fUTF8	僄
U+50C4.0	fCNS11643	2-415D
U+50C4.0	fBig5	DCB8
U+50C4.0	fGB	3-3225
U+50C4.0	fR/S	 9.11
U+50C4.0	fTotalStrokes	 13
U+50C4.0	fCangjie	OMWF
U+50C4.0	fFourCorner	21291
U+50C4.0	fHakka	 piau5
U+50C4.0	fCantonese	piu3 (also); piu1; piu5
U+50C4.0	fMandarin	piao4; biao1; piao1
U+50C4.0	fEnglish	 light airy
U+50C5.0	fUTF8	僅
U+50C5.0	fCNS11643	1-632F
U+50C5.0	fBig5	B6C8
U+50C5.0	fGB	1-3D76
U+50C5.0	fR/S	 9.11
U+50C5.0	fTotalStrokes	 13
U+50C5.0	fCangjie	OTLM
U+50C5.0	fFourCorner	24214
U+50C5.0	fHakka	  giun3
U+50C5.0	fCantonese	gan2
U+50C5.0	fMandarin	jin3
U+50C5.0	fEnglish	[1] only [2] barely; scarcely; almost; merely; solely; just
U+50C5.1	fUTF8	僅
U+50C5.1	fCNS11643	1-632F
U+50C5.1	fBig5	B6C8
U+50C5.1	fGB	1-3D76
U+50C5.1	fR/S	 9.11
U+50C5.1	fTotalStrokes	 13
U+50C5.1	fCangjie	OTLM
U+50C5.1	fFourCorner	24214
U+50C5.1	fCantonese	gan6
U+50C5.1	fMandarin	jin4
U+50C6.0	fUTF8	僆
U+50C6.0	fCNS11643	2-415B
U+50C6.0	fBig5	DCB6
U+50C6.0	fGB	3-313D
U+50C6.0	fR/S	 9.11
U+50C6.0	fTotalStrokes	 13
U+50C6.0	fCangjie	OYJJ
U+50C6.0	fFourCorner	25230
U+50C6.0	fHakka	 lien3; lon5; lien2; lon6; len3
U+50C6.0	fCantonese	lin5; leng1; lin4
U+50C6.0	fMandarin	lian2; lian3; lian4
U+50C6.0	fEnglish	a chicken
U+50C7.0	fUTF8	僇
U+50C7.0	fCNS11643	1-6335
U+50C7.0	fBig5	B6CE
U+50C7.0	fGB	3-322F
U+50C7.0	fR/S	 9.11
U+50C7.0	fTotalStrokes	 13
U+50C7.0	fCangjie	OSMH
U+50C7.0	fFourCorner	27222
U+50C7.0	fHakka	 luk8;  liuk8
U+50C7.0	fCantonese	luk6; kuk6
U+50C7.0	fMandarin	lu4; liu4
U+50C7.0	fEnglish	[1] kill; massacre [2] disgrace; shame; humiliate [3] collaborate; pool strength together
U+50C8.0	fUTF8	僈
U+50C8.0	fCNS11643	2-4160
U+50C8.0	fBig5	DCBB
U+50C8.0	fGB	3-3226
U+50C8.0	fR/S	 9.11
U+50C8.0	fTotalStrokes	 13
U+50C8.0	fCangjie	OAWE
U+50C8.0	fFourCorner	26247
U+50C8.0	fHakka	 man5; man6; man3
U+50C8.0	fCantonese	maan6; man6
U+50C8.0	fMandarin	man4
U+50C8.0	fEnglish	[1] negligent [2] remiss
U+50C9.0	fUTF8	僉
U+50C9.0	fCNS11643	2-4167
U+50C9.0	fBig5	DCC2
U+50C9.0	fGB	1-595D
U+50C9.0	fR/S	 9.11
U+50C9.0	fTotalStrokes	 13
U+50C9.0	fCangjie	OMRO
U+50C9.0	fFourCorner	80886
U+50C9.0	fHakka	 ciam1
U+50C9.0	fCantonese	cim1
U+50C9.0	fMandarin	qian1
U+50C9.0	fEnglish	all; the whole; together; unanimous
U+50CA.0	fUTF8	僊
U+50CA.0	fCNS11643	2-415E
U+50CA.0	fBig5	DCB9
U+50CA.0	fGB	E-2237
U+50CA.0	fR/S	 9.11
U+50CA.0	fTotalStrokes	 13
U+50CA.0	fCangjie	OMWU
U+50CA.0	fFourCorner	21212
U+50CA.0	fHakka	  sien1
U+50CA.0	fCantonese	sin1
U+50CA.0	fMandarin	xian1
U+50CA.0	fEnglish	Taoist super-being; transcendent; immortal
U+50CB.0	fUTF8	僋
U+50CB.0	fCNS11643	2-4166
U+50CB.0	fBig5	DCC1
U+50CB.0	fGB	3-3154
U+50CB.0	fR/S	 9.11
U+50CB.0	fTotalStrokes	 13
U+50CB.0	fCangjie	OOIC
U+50CB.0	fFourCorner	28286
U+50CB.0	fHakka	  tam5
U+50CB.0	fCantonese	taam3; taam2; taam5
U+50CB.0	fMandarin	tan4; qiu2; lan4
U+50CB.0	fEnglish	silly; foolish
U+50CC.0	fUTF8	僌
U+50CC.0	fCNS11643	3-4036
U+50CC.0	fGB	5-322E
U+50CC.0	fR/S	  9.11
U+50CC.0	fTotalStrokes	 13
U+50CC.0	fCangjie	OJJK
U+50CC.0	fFourCorner	28240
U+50CC.0	fCantonese	jing4
U+50CC.0	fMandarin	ying2
U+50CD.0	fUTF8	働
U+50CD.0	fCNS11643	3-403E
U+50CD.0	fGB	8-2C7A
U+50CD.0	fR/S	 9.11
U+50CD.0	fTotalStrokes	 13
U+50CD.0	fCangjie	OHGS
U+50CD.0	fFourCorner	24220
U+50CD.0	fCantonese	dung6
U+50CD.0	fMandarin	dong4
U+50CD.0	fEnglish	 labor work
U+50CE.0	fUTF8	僎
U+50CE.0	fCNS11643	1-677C
U+50CE.0	fBig5	B9B6
U+50CE.0	fGB	3-323A
U+50CE.0	fR/S	 9.12
U+50CE.0	fTotalStrokes	 14
U+50CE.0	fCangjie	ORUC
U+50CE.0	fFourCorner	27281
U+50CE.0	fHakka	 sien3;  con3
U+50CE.0	fCantonese	zaan6; zeon1; zyun1; syun2
U+50CE.0	fMandarin	zhuan4; zun1
U+50CE.0	fEnglish	collect
U+50CF.0	fUTF8	像
U+50CF.0	fCNS11643	1-6779
U+50CF.0	fBig5	B9B3
U+50CF.0	fGB	0-4F71
U+50CF.0	fR/S	 9.12
U+50CF.0	fTotalStrokes	 14
U+50CF.0	fCangjie	ONAO
U+50CF.0	fFourCorner	27232
U+50CF.0	fHakka	  siong5;  ciong5;  cam5; ziong5
U+50CF.0	fCantonese	zoeng6
U+50CF.0	fMandarin	xiang4; yang4
U+50CF.0	fEnglish	[1] image; portrait; picture [2] resemble; resemblance [3] like; as
U+50D0.0	fUTF8	僐
U+50D0.0	fCNS11643	3-456B
U+50D0.0	fGB	5-3271
U+50D0.0	fR/S	 9.12
U+50D0.0	fTotalStrokes	 14
U+50D0.0	fCangjie	OTGR
U+50D0.0	fFourCorner	28260
U+50D0.0	fHakka	 shen5; san5; sen5
U+50D0.0	fCantonese	sin6
U+50D0.0	fMandarin	shan4
U+50D0.0	fEnglish	elegant; refined
U+50D1.0	fUTF8	僑
U+50D1.0	fCNS11643	1-677A
U+50D1.0	fBig5	B9B4
U+50D1.0	fGB	1-4748
U+50D1.0	fR/S	 9.12
U+50D1.0	fTotalStrokes	 14
U+50D1.0	fCangjie	OHKB
U+50D1.0	fFourCorner	22227
U+50D1.0	fHakka	 kiau2
U+50D1.0	fCantonese	kiu4
U+50D1.0	fMandarin	qiao2
U+50D1.0	fEnglish	 sojourn
U+50D2.0	fUTF8	僒
U+50D2.0	fCNS11643	4-422D
U+50D2.0	fGB	3-3238
U+50D2.0	fR/S	 9.12
U+50D2.0	fTotalStrokes	 14
U+50D2.0	fCangjie	OJCR
U+50D2.0	fFourCorner	23260
U+50D2.0	fHakka	 kiun2; kiung2
U+50D2.0	fCantonese	kwan3
U+50D2.0	fMandarin	jiong3; jun3
U+50D3.0	fUTF8	僓
U+50D3.0	fCNS11643	2-4921
U+50D3.0	fBig5	E0F9
U+50D3.0	fGB	3-3165
U+50D3.0	fR/S	 9.12
U+50D3.0	fTotalStrokes	 14
U+50D3.0	fCangjie	OLMC
U+50D3.0	fFourCorner	25286
U+50D3.0	fCantonese	teoi4
U+50D3.0	fMandarin	tui2; tui3; hui4
U+50D4.0	fUTF8	僔
U+50D4.0	fCNS11643	2-4877
U+50D4.0	fBig5	E0F1
U+50D4.0	fGB	3-3237
U+50D4.0	fR/S	 9.12
U+50D4.0	fTotalStrokes	 14
U+50D4.0	fCangjie	OTWI
U+50D4.0	fFourCorner	28246
U+50D4.0	fHakka	  zun3
U+50D4.0	fCantonese	zyun2
U+50D4.0	fMandarin	zun3
U+50D4.0	fEnglish	[1] assemble [2] respect
U+50D5.0	fUTF8	僕
U+50D5.0	fCNS11643	1-6778
U+50D5.0	fBig5	B9B2
U+50D5.0	fGB	1-464D
U+50D5.0	fR/S	 9.12
U+50D5.0	fTotalStrokes	 14
U+50D5.0	fCangjie	OTCO
U+50D5.0	fFourCorner	22234
U+50D5.0	fHakka	 puk8; pok7
U+50D5.0	fCantonese	buk6
U+50D5.0	fMandarin	pu2; pu1; pu3
U+50D5.0	fEnglish	[1] slave; servant [2] a modest term referring to oneself [3] act as a driver; drive (formerly)
U+50D6.0	fUTF8	僖
U+50D6.0	fCNS11643	1-6775
U+50D6.0	fBig5	B9AF
U+50D6.0	fGB	0-5952
U+50D6.0	fR/S	 9.12
U+50D6.0	fTotalStrokes	 14
U+50D6.0	fCangjie	OGRR
U+50D6.0	fFourCorner	24261
U+50D6.0	fHakka	  hi3;  hi1
U+50D6.0	fCantonese	hei1
U+50D6.0	fMandarin	xi1
U+50D6.0	fEnglish	[1] joy; gladness; delight; joyful [2] a Chinese family name
U+50D7.0	fUTF8	僗
U+50D7.0	fCNS11643	2-4878
U+50D7.0	fBig5	E0F2
U+50D7.0	fGB	3-3137
U+50D7.0	fR/S	 9.12
U+50D7.0	fTotalStrokes	 14
U+50D7.0	fCangjie	OFFS
U+50D7.0	fFourCorner	29227
U+50D7.0	fCantonese	lou4
U+50D7.0	fMandarin	lao2; lao4
U+50D8.0	fUTF8	僘
U+50D8.0	fCNS11643	3-456E
U+50D8.0	fGB	E-2238
U+50D8.0	fR/S	  9.12
U+50D8.0	fTotalStrokes	 14
U+50D8.0	fCangjie	OFBK
U+50D8.0	fFourCorner	28240
U+50D8.0	fHakka	 chong3
U+50D8.0	fCantonese	cong2
U+50D8.0	fMandarin	chang3
U+50D9.0	fUTF8	僙
U+50D9.0	fCNS11643	3-456C
U+50D9.0	fGB	5-3261
U+50D9.0	fR/S	 9.12
U+50D9.0	fTotalStrokes	 14
U+50D9.0	fCangjie	OTLC
U+50D9.0	fFourCorner	24280
U+50D9.0	fHakka	 gwong1
U+50D9.0	fCantonese	gwong1; gong1 (also)
U+50D9.0	fMandarin	guang1
U+50DA.0	fUTF8	僚
U+50DA.0	fCNS11643	1-6777
U+50DA.0	fBig5	B9B1
U+50DA.0	fGB	0-4145
U+50DA.0	fR/S	 9.12
U+50DA.0	fTotalStrokes	 14
U+50DA.0	fCangjie	OKCF
U+50DA.0	fFourCorner	24296
U+50DA.0	fHakka	 liau2; liau3
U+50DA.0	fCantonese	liu4; liu5
U+50DA.0	fMandarin	liao2
U+50DA.0	fEnglish	[1] companion; friend [2] colleague; subordinates [3] officials
U+50DA.1	fUTF8	僚
U+50DA.1	fCNS11643	1-6777
U+50DA.1	fBig5	B9B1
U+50DA.1	fGB	0-4145
U+50DA.1	fR/S	 9.12
U+50DA.1	fTotalStrokes	 14
U+50DA.1	fCangjie	OKCF
U+50DA.1	fFourCorner	24296
U+50DA.1	fCantonese	lou5
U+50DA.1	fMandarin	liao3
U+50DB.0	fUTF8	僛
U+50DB.0	fCNS11643	2-487B
U+50DB.0	fBig5	E0F5
U+50DB.0	fGB	3-3232
U+50DB.0	fR/S	 9.12
U+50DB.0	fTotalStrokes	 14
U+50DB.0	fCangjie	OTCO
U+50DB.0	fFourCorner	27282
U+50DB.0	fHakka	 ki1
U+50DB.0	fCantonese	hei1
U+50DB.0	fMandarin	qi1
U+50DB.0	fEnglish	 act like a drunken man
U+50DC.0	fUTF8	僜
U+50DC.0	fCNS11643	3-456D
U+50DC.0	fGB	3-323B
U+50DC.0	fR/S	  9.12
U+50DC.0	fTotalStrokes	 14
U+50DC.0	fCangjie	ONOT
U+50DC.0	fFourCorner	22218
U+50DC.0	fHakka	 den5; dang2
U+50DC.0	fCantonese	cing1
U+50DC.0	fMandarin	deng4
U+50DC.0	fEnglish	exhausted; wearied
U+50DD.0	fUTF8	僝
U+50DD.0	fCNS11643	2-487D
U+50DD.0	fBig5	E0F7
U+50DD.0	fGB	3-3239
U+50DD.0	fR/S	 9.12
U+50DD.0	fTotalStrokes	 14
U+50DD.0	fCangjie	OSND
U+50DD.0	fFourCorner	27247
U+50DD.0	fHakka	  zan1; can2
U+50DD.0	fCantonese	saan4; caan4 (also); zaan6
U+50DD.0	fMandarin	chan2; zhan3; zhuan4
U+50DD.0	fEnglish	 revile abuse
U+50DE.0	fUTF8	僞
U+50DE.0	fCNS11643	3-4576
U+50DE.0	fGB	1-4E31
U+50DE.0	fR/S	 9.12
U+50DE.0	fTotalStrokes	 14
U+50DE.0	fCangjie	OBHF
U+50DE.0	fFourCorner	22220
U+50DE.0	fHakka	 ngui3; wui6
U+50DE.0	fCantonese	ngai6; ai6 (also)
U+50DE.0	fMandarin	wei4; wei3; e2
U+50DE.0	fEnglish	[1] false; counterfeit [2] simulated; artificial [3] illegal; not legally constituted
U+50DF.0	fUTF8	僟
U+50DF.0	fCNS11643	4-4235
U+50DF.0	fGB	E-2239
U+50DF.0	fR/S	 9.12
U+50DF.0	fTotalStrokes	 14
U+50DF.0	fCangjie	OVII
U+50DF.0	fFourCorner	22250
U+50DF.0	fCantonese	gei1
U+50DF.0	fMandarin	ji1
U+50E0.0	fUTF8	僠
U+50E0.0	fCNS11643	2-4926
U+50E0.0	fBig5	E0FE
U+50E0.0	fGB	E-223A
U+50E0.0	fR/S	 9.12
U+50E0.0	fTotalStrokes	 14
U+50E0.0	fCangjie	OHDW
U+50E0.0	fFourCorner	22269
U+50E0.0	fCantonese	faan1; bo1
U+50E0.0	fMandarin	fan1
U+50E1.0	fUTF8	僡
U+50E1.0	fCNS11643	3-4570
U+50E1.0	fGB	5-3269
U+50E1.0	fR/S	  9.12
U+50E1.0	fTotalStrokes	 14
U+50E1.0	fCangjie	OJIP
U+50E1.0	fFourCorner	25230
U+50E1.0	fCantonese	wai6
U+50E1.0	fMandarin	hui4
U+50E2.0	fUTF8	僢
U+50E2.0	fCNS11643	3-4575
U+50E2.0	fGB	5-326F
U+50E2.0	fR/S	  9.12
U+50E2.0	fTotalStrokes	 14
U+50E2.0	fCangjie	OBBQ
U+50E2.0	fFourCorner	22250
U+50E2.0	fHakka	 chon3; can3; con3
U+50E2.0	fCantonese	cyun2
U+50E2.0	fMandarin	chuan3
U+50E2.0	fEnglish	 flow in opposite directions
U+50E3.0	fUTF8	僣
U+50E3.0	fCNS11643	2-4925
U+50E3.0	fBig5	E0FD
U+50E3.0	fGB	E-223B
U+50E3.0	fR/S	 9.12
U+50E3.0	fTotalStrokes	 14
U+50E3.0	fCangjie	OQOA
U+50E3.0	fFourCorner	25263
U+50E3.0	fHakka	  tiet7;  ziam5
U+50E3.0	fCantonese	tit3; zim3; zim2
U+50E3.0	fMandarin	jian4; tie3; zen4
U+50E3.0	fEnglish	assume; usurp
U+50E4.0	fUTF8	僤
U+50E4.0	fCNS11643	2-487E
U+50E4.0	fBig5	E0F8
U+50E4.0	fGB	3-3158
U+50E4.0	fR/S	 9.12
U+50E4.0	fTotalStrokes	 14
U+50E4.0	fCangjie	ORRJ
U+50E4.0	fFourCorner	26256
U+50E4.0	fHakka	  tan3; tan5
U+50E4.0	fCantonese	daan6; daan1
U+50E4.0	fMandarin	dan4; chan3; tan2
U+50E4.0	fEnglish	sincere
U+50E5.0	fUTF8	僥
U+50E5.0	fCNS11643	1-6774
U+50E5.0	fBig5	B9AE
U+50E5.0	fGB	1-3D44
U+50E5.0	fR/S	 9.12
U+50E5.0	fTotalStrokes	 14
U+50E5.0	fCangjie	OGGU
U+50E5.0	fFourCorner	24211
U+50E5.0	fHakka	 hiau1; hiau3; hieu1
U+50E5.0	fCantonese	hiu1; giu1; giu2
U+50E5.0	fMandarin	jiao3
U+50E5.0	fEnglish	luck; lucky; by chance
U+50E6.0	fUTF8	僦
U+50E6.0	fCNS11643	2-4876
U+50E6.0	fBig5	E0F0
U+50E6.0	fGB	0-5956
U+50E6.0	fR/S	 9.12
U+50E6.0	fTotalStrokes	 14
U+50E6.0	fCangjie	OYFU
U+50E6.0	fFourCorner	23214
U+50E6.0	fHakka	 ciu5; ciu6; ziu6; ziu5
U+50E6.0	fCantonese	zau6; zau3
U+50E6.0	fMandarin	jiu4
U+50E6.0	fEnglish	 hire; rent
U+50E7.0	fUTF8	僧
U+50E7.0	fCNS11643	1-6772
U+50E7.0	fBig5	B9AC
U+50E7.0	fGB	0-492E
U+50E7.0	fR/S	 9.12; 9.11
U+50E7.0	fTotalStrokes	 14
U+50E7.0	fCangjie	OCWA
U+50E7.0	fFourCorner	28266
U+50E7.0	fHakka	 sen1; zen1; sien1; cen1; zien1
U+50E7.0	fCantonese	zang1; sang1 (also)
U+50E7.0	fMandarin	seng1
U+50E7.0	fEnglish	[1] Buddhist; Buddhist priest; monk [2] a Chinese family name
U+50E8.0	fUTF8	僨
U+50E8.0	fCNS11643	2-4879
U+50E8.0	fBig5	E0F3
U+50E8.0	fGB	1-5947
U+50E8.0	fR/S	 9.12
U+50E8.0	fTotalStrokes	 14
U+50E8.0	fCangjie	OJTC
U+50E8.0	fFourCorner	24286
U+50E8.0	fHakka	 fun3
U+50E8.0	fCantonese	fan5; fan3
U+50E8.0	fMandarin	fen4; ben1
U+50E8.0	fEnglish	 ruin; cause fail overthrown
U+50E9.0	fUTF8	僩
U+50E9.0	fCNS11643	1-677D
U+50E9.0	fBig5	B9B7
U+50E9.0	fGB	3-3144
U+50E9.0	fR/S	 9.12
U+50E9.0	fTotalStrokes	 14
U+50E9.0	fCangjie	OANB
U+50E9.0	fFourCorner	27220
U+50E9.0	fHakka	 han2; han3
U+50E9.0	fCantonese	haan5
U+50E9.0	fMandarin	xian4; jian3
U+50E9.0	fEnglish	 courageous martial dignified
U+50EA.0	fUTF8	僪
U+50EA.0	fCNS11643	2-487C
U+50EA.0	fBig5	E0F6
U+50EA.0	fGB	3-323C
U+50EA.0	fR/S	 9.12
U+50EA.0	fTotalStrokes	 14
U+50EA.0	fCangjie	ONHB
U+50EA.0	fFourCorner	27227
U+50EA.0	fCantonese	gwat6; jyut6; kyut3
U+50EA.0	fMandarin	jue2
U+50EB.0	fUTF8	僫
U+50EB.0	fCNS11643	3-4571
U+50EB.0	fGB	5-3254
U+50EB.0	fR/S	  9.12
U+50EB.0	fTotalStrokes	 14
U+50EB.0	fCangjie	OMMP
U+50EB.0	fFourCorner	21230
U+50EB.0	fCantonese	ok3; ngok3 (also)
U+50EB.0	fMandarin	e4
U+50EC.0	fUTF8	僬
U+50EC.0	fCNS11643	2-4922
U+50EC.0	fBig5	E0FA
U+50EC.0	fGB	0-5955
U+50EC.0	fR/S	 9.12
U+50EC.0	fTotalStrokes	 14
U+50EC.0	fCangjie	OOGF
U+50EC.0	fFourCorner	20231
U+50EC.0	fHakka	  ziau1
U+50EC.0	fCantonese	ziu1; ziu3 (also)
U+50EC.0	fMandarin	jiao1; jiao4; jiao3; qiao3
U+50EC.0	fEnglish	 clever alert in mind pigmies
U+50ED.0	fUTF8	僭
U+50ED.0	fCNS11643	1-6776
U+50ED.0	fBig5	B9B0
U+50ED.0	fGB	0-5954
U+50ED.0	fR/S	 9.12
U+50ED.0	fTotalStrokes	 14
U+50ED.0	fCangjie	OMUA
U+50ED.0	fFourCorner	25261
U+50ED.0	fHakka	 ziam5; tiet8
U+50ED.0	fCantonese	cim3; zim3 (also); cim5 (also)
U+50ED.0	fMandarin	jian4; jin1; zen4
U+50ED.0	fEnglish	 assume; usurp; overstep one's authority
U+50EE.0	fUTF8	僮
U+50EE.0	fCNS11643	1-6773
U+50EE.0	fBig5	B9AD
U+50EE.0	fGB	0-5957
U+50EE.0	fR/S	 9.12
U+50EE.0	fTotalStrokes	 14
U+50EE.0	fCangjie	OYTG
U+50EE.0	fFourCorner	20214
U+50EE.0	fHakka	 tung2; tung3
U+50EE.0	fCantonese	tung4; zong3; zung3
U+50EE.0	fMandarin	tong2; bian4
U+50EE.0	fEnglish	[1] servant [2] boy [3] a Chinese family name
U+50EE.1	fUTF8	僮
U+50EE.1	fCNS11643	1-6773
U+50EE.1	fBig5	B9AD
U+50EE.1	fGB	0-5957
U+50EE.1	fR/S	 9.12
U+50EE.1	fTotalStrokes	 14
U+50EE.1	fCangjie	OYTG
U+50EE.1	fFourCorner	20214
U+50EE.1	fCantonese	zong6
U+50EE.1	fMandarin	zhuang4
U+50EE.1	fEnglish	the name of a small tribe in southwestern China
U+50EF.0	fUTF8	僯
U+50EF.0	fCNS11643	2-4924
U+50EF.0	fBig5	E0FC
U+50EF.0	fGB	3-3236
U+50EF.0	fR/S	 9.12
U+50EF.0	fTotalStrokes	 14
U+50EF.0	fCangjie	OFDQ
U+50EF.0	fFourCorner	29259
U+50EF.0	fHakka	  lin2; lin3
U+50EF.0	fCantonese	leon4; leon5; leon6; lin5
U+50EF.0	fMandarin	lin3; lin2
U+50EF.0	fEnglish	ashamed
U+50F0.0	fUTF8	僰
U+50F0.0	fCNS11643	2-4923
U+50F0.0	fBig5	E0FB
U+50F0.0	fGB	3-305A
U+50F0.0	fR/S	 9.12
U+50F0.0	fTotalStrokes	 14
U+50F0.0	fCangjie	DBO
U+50F0.0	fFourCorner	44402
U+50F0.0	fHakka	 pet7
U+50F0.0	fCantonese	baak6; bok3 (also); bat6; pat6
U+50F0.0	fMandarin	bo2
U+50F0.0	fEnglish	ancient aboriginal tribe
U+50F1.0	fUTF8	僱
U+50F1.0	fCNS11643	1-677B
U+50F1.0	fBig5	B9B5
U+50F1.0	fGB	E-223C
U+50F1.0	fR/S	 9.12
U+50F1.0	fTotalStrokes	 14
U+50F1.0	fCangjie	OHSG
U+50F1.0	fFourCorner	23214
U+50F1.0	fHakka	 gu5
U+50F1.0	fCantonese	gu3; gwu3 (also)
U+50F1.0	fMandarin	gu4
U+50F1.0	fEnglish	 employ; hire; engage
U+50F2.0	fUTF8	僲
U+50F2.0	fGB	E-223D
U+50F2.0	fR/S	 9.13
U+50F2.0	fTotalStrokes	 15
U+50F2.0	fCangjie	OMWQ
U+50F2.0	fCantonese	sin1
U+50F2.0	fMandarin	xian1
U+50F2.0	fEnglish	 [meaning and reading not studied]
U+50F3.0	fUTF8	僳
U+50F3.0	fCNS11643	2-487A
U+50F3.0	fBig5	E0F4
U+50F3.0	fGB	0-4B5B
U+50F3.0	fR/S	 9.12
U+50F3.0	fTotalStrokes	 14
U+50F3.0	fCangjie	OMWD
U+50F3.0	fFourCorner	21299
U+50F3.0	fCantonese	suk1
U+50F3.0	fMandarin	su4
U+50F3.0	fEnglish	minority name
U+50F4.0	fUTF8	僴
U+50F4.0	fCNS11643	3-4574
U+50F4.0	fGB	E-223E
U+50F4.0	fR/S	 9.12
U+50F4.0	fTotalStrokes	 14
U+50F4.0	fCangjie	OANA
U+50F4.0	fFourCorner	27220
U+50F4.0	fCantonese	haan5
U+50F4.0	fMandarin	xian4
U+50F5.0	fUTF8	僵
U+50F5.0	fCNS11643	1-6C23
U+50F5.0	fBig5	BBF8
U+50F5.0	fGB	0-3D29
U+50F5.0	fR/S	 9.13
U+50F5.0	fTotalStrokes	 15
U+50F5.0	fCangjie	OMWM
U+50F5.0	fFourCorner	21216
U+50F5.0	fHakka	  giong1
U+50F5.0	fCantonese	goeng1
U+50F5.0	fMandarin	jiang1
U+50F5.0	fEnglish	[1] lie flat [2] be inactive; stiff; rigid [3] be at a stale mate; deadlocked
U+50F6.0	fUTF8	僶
U+50F6.0	fCNS11643	2-4F54
U+50F6.0	fBig5	E4EC
U+50F6.0	fGB	5-3233
U+50F6.0	fR/S	 9.13
U+50F6.0	fTotalStrokes	 15
U+50F6.0	fCangjie	ORXU
U+50F6.0	fFourCorner	27217
U+50F6.0	fHakka	 mien3; men3
U+50F6.0	fCantonese	man5
U+50F6.0	fMandarin	min3
U+50F6.0	fEnglish	 make an effort
U+50F7.0	fUTF8	僷
U+50F7.0	fCNS11643	3-4B29
U+50F7.0	fGB	E-223F
U+50F7.0	fR/S	 9.13; 9.12
U+50F7.0	fTotalStrokes	 15; 14
U+50F7.0	fCangjie	OTPD
U+50F7.0	fFourCorner	24290
U+50F7.0	fHakka	 tiap8;  jap8; rap8
U+50F7.0	fCantonese	jip6
U+50F7.0	fMandarin	ye4
U+50F7.0	fEnglish	handsome; light-hearted
U+50F8.0	fUTF8	僸
U+50F8.0	fCNS11643	2-4F51
U+50F8.0	fBig5	E4E9
U+50F8.0	fGB	3-323E
U+50F8.0	fR/S	 9.13
U+50F8.0	fTotalStrokes	 15
U+50F8.0	fCangjie	ODDF
U+50F8.0	fFourCorner	24291
U+50F8.0	fHakka	 gim5; kim1
U+50F8.0	fCantonese	gam3
U+50F8.0	fMandarin	jin4; yin4
U+50F8.0	fEnglish	[1] a musical instrument [2] loop up at and follow
U+50F9.0	fUTF8	價
U+50F9.0	fCNS11643	1-6C24
U+50F9.0	fBig5	BBF9
U+50F9.0	fGB	1-3C5B
U+50F9.0	fR/S	 9.13
U+50F9.0	fTotalStrokes	 15
U+50F9.0	fCangjie	OMWC
U+50F9.0	fFourCorner	21286
U+50F9.0	fHakka	  ga5
U+50F9.0	fCantonese	gaa3
U+50F9.0	fMandarin	jia4; jie5
U+50F9.0	fEnglish	[1] prices; cost; value [2] valence (chemistry)
U+50FA.0	fUTF8	僺
U+50FA.0	fCNS11643	4-4879
U+50FA.0	fGB	E-2240
U+50FA.0	fR/S	 9.13
U+50FA.0	fTotalStrokes	 15
U+50FA.0	fCangjie	ORRD
U+50FA.0	fFourCorner	26290
U+50FA.0	fCantonese	ciu3
U+50FA.0	fMandarin	qiao4
U+50FB.0	fUTF8	僻
U+50FB.0	fCNS11643	1-6C22
U+50FB.0	fBig5	BBF7
U+50FB.0	fGB	0-4627
U+50FB.0	fR/S	 9.13
U+50FB.0	fTotalStrokes	 15
U+50FB.0	fCangjie	OSRJ
U+50FB.0	fFourCorner	20241
U+50FB.0	fHakka	 pit7
U+50FB.0	fCantonese	pik1
U+50FB.0	fMandarin	pi4
U+50FB.0	fEnglish	[1] biased [2] not easily accessible; out-of-the-way; secluded [3] not common; unorthodox; not ordinarily; unusual
U+50FB.1	fUTF8	僻
U+50FB.1	fCNS11643	1-6C22
U+50FB.1	fBig5	BBF7
U+50FB.1	fGB	0-4627
U+50FB.1	fR/S	 9.13
U+50FB.1	fTotalStrokes	 15
U+50FB.1	fCangjie	OSRJ
U+50FB.1	fFourCorner	20241
U+50FB.1	fCantonese	bui6
U+50FB.1	fMandarin	bei4
U+50FC.0	fUTF8	僼
U+50FC.0	fCNS11643	3-4B2B
U+50FC.0	fGB	5-3276
U+50FC.0	fR/S	 9.13
U+50FC.0	fTotalStrokes	 15
U+50FC.0	fCangjie	OTWT
U+50FC.0	fFourCorner	25210
U+50FC.0	fCantonese	fung1
U+50FC.0	fMandarin	feng1
U+50FD.0	fUTF8	僽
U+50FD.0	fCNS11643	2-4F58
U+50FD.0	fBig5	E4F0
U+50FD.0	fGB	3-3240
U+50FD.0	fR/S	 9.13
U+50FD.0	fTotalStrokes	 15
U+50FD.0	fCangjie	OHFP
U+50FD.0	fFourCorner	29238
U+50FD.0	fHakka	  ceu3; zhiu5; zu5
U+50FD.0	fCantonese	zau3; zau6 (also)
U+50FD.0	fMandarin	zhou4
U+50FD.0	fEnglish	 scolf; sad
U+50FE.0	fUTF8	僾
U+50FE.0	fCNS11643	2-4F55
U+50FE.0	fBig5	E4ED
U+50FE.0	fGB	3-317C
U+50FE.0	fR/S	 9.13
U+50FE.0	fTotalStrokes	 15
U+50FE.0	fCangjie	OBBE
U+50FE.0	fFourCorner	22247
U+50FE.0	fHakka	  oi5
U+50FE.0	fCantonese	oi3; ngoi3 (also); oi2
U+50FE.0	fMandarin	ai4; yi3
U+50FE.0	fEnglish	[1] like; similar [2] pant; out of breath
U+50FF.0	fUTF8	僿
U+50FF.0	fCNS11643	2-4F4E
U+50FF.0	fBig5	E4E6
U+50FF.0	fGB	3-3241
U+50FF.0	fR/S	 9.13
U+50FF.0	fTotalStrokes	 15
U+50FF.0	fCangjie	OJTG
U+50FF.0	fFourCorner	23214
U+50FF.0	fHakka	 sii1; chi5; cih5; se5
U+50FF.0	fCantonese	si1; sai3; zi6; coi3
U+50FF.0	fMandarin	sai4; si4
U+50FF.0	fEnglish	 small; minute lacking sincerity
U+5100.0	fUTF8	儀
U+5100.0	fCNS11643	1-6C21
U+5100.0	fBig5	BBF6
U+5100.0	fGB	1-5247
U+5100.0	fR/S	 9.13
U+5100.0	fTotalStrokes	 15
U+5100.0	fCangjie	OTGI
U+5100.0	fFourCorner	28253
U+5100.0	fHakka	 ngi2; gni2; ji2; gni3
U+5100.0	fCantonese	ji4
U+5100.0	fMandarin	yi2
U+5100.0	fEnglish	[1] appearance; deportment [2] ceremonies; rites; rule; regulation; form; standard [3] customs [4] instruments; apparatus [5] a Chinese family name
U+5101.0	fUTF8	儁
U+5101.0	fCNS11643	3-4B2D
U+5101.0	fGB	E-2241
U+5101.0	fR/S	  9.13
U+5101.0	fTotalStrokes	 15
U+5101.0	fCangjie	OOGS
U+5101.0	fFourCorner	20220
U+5101.0	fHakka	 zun5
U+5101.0	fCantonese	zeon3
U+5101.0	fMandarin	jun4; juan4
U+5101.0	fEnglish	 outstanding; fine superior talented; capable handsome
U+5102.0	fUTF8	儂
U+5102.0	fCNS11643	1-6C25
U+5102.0	fBig5	BBFA
U+5102.0	fGB	1-592F
U+5102.0	fR/S	 9.13
U+5102.0	fTotalStrokes	 15
U+5102.0	fCangjie	OTWV
U+5102.0	fFourCorner	25232
U+5102.0	fHakka	  nung2; lung2
U+5102.0	fCantonese	nung4; lung4 (also)
U+5102.0	fMandarin	nong2
U+5102.0	fEnglish	[1] I, me (in old usage) [2] you [3] he; she [4] a Chinese family name
U+5103.0	fUTF8	儃
U+5103.0	fCNS11643	2-4F4F
U+5103.0	fBig5	E4E7
U+5103.0	fGB	5-3278
U+5103.0	fR/S	 9.13
U+5103.0	fTotalStrokes	 15
U+5103.0	fCangjie	OYWM
U+5103.0	fFourCorner	20216
U+5103.0	fHakka	 tan3; sham2; sam2
U+5103.0	fCantonese	sin4; taan2; zin1; cin4
U+5103.0	fMandarin	chan2; tan3; shan4; dan4; tan2
U+5103.0	fEnglish	manner; mien; air
U+5104.0	fUTF8	億
U+5104.0	fCNS11643	1-6B7E
U+5104.0	fBig5	BBF5
U+5104.0	fGB	1-525A
U+5104.0	fR/S	 9.13
U+5104.0	fTotalStrokes	 15
U+5104.0	fCangjie	OYTP
U+5104.0	fFourCorner	20236
U+5104.0	fHakka	  jit8; ji5; ri5; rit8; jit7
U+5104.0	fCantonese	jik1
U+5104.0	fMandarin	yi4
U+5104.0	fEnglish	[1] a hundred million; many [2] tranquility; repose [3] (according to) estimates
U+5105.0	fUTF8	儅
U+5105.0	fCNS11643	1-6C28
U+5105.0	fBig5	BBFD
U+5105.0	fGB	3-3121
U+5105.0	fR/S	 9.13
U+5105.0	fTotalStrokes	 15
U+5105.0	fCangjie	OFBW
U+5105.0	fFourCorner	29266
U+5105.0	fCantonese	dong1; dong3
U+5105.0	fMandarin	dang1; dang4; tang4
U+5105.0	fEnglish	stop
U+5106.0	fUTF8	儆
U+5106.0	fCNS11643	2-4F52
U+5106.0	fBig5	E4EA
U+5106.0	fGB	0-5953
U+5106.0	fR/S	 9.13
U+5106.0	fTotalStrokes	 15
U+5106.0	fCangjie	OTRK
U+5106.0	fFourCorner	28240
U+5106.0	fHakka	 gin3
U+5106.0	fCantonese	ging2
U+5106.0	fMandarin	jing3; jing4
U+5106.0	fEnglish	[1] be on guard; get ready (for an attack, etc.) [2] warn; caution; warning
U+5107.0	fUTF8	儇
U+5107.0	fCNS11643	2-4F53
U+5107.0	fBig5	E4EB
U+5107.0	fGB	0-5958
U+5107.0	fR/S	 9.13
U+5107.0	fTotalStrokes	 15
U+5107.0	fCangjie	OWLV
U+5107.0	fFourCorner	26232
U+5107.0	fHakka	 fan1; fan2
U+5107.0	fCantonese	hyun1
U+5107.0	fMandarin	xuan1
U+5107.0	fEnglish	[1] active [2] nimble [3] expert
U+5108.0	fUTF8	儈
U+5108.0	fCNS11643	1-6C26
U+5108.0	fBig5	BBFB
U+5108.0	fGB	1-3F6B
U+5108.0	fR/S	 9.13
U+5108.0	fTotalStrokes	 15
U+5108.0	fCangjie	OOMA
U+5108.0	fFourCorner	28266
U+5108.0	fHakka	  kwai5; kai5
U+5108.0	fCantonese	kui2; kui3 (also); kwui2 (also); kwui3 (also)
U+5108.0	fMandarin	kuai4
U+5108.0	fEnglish	 middleman; go-between; broker
U+5109.0	fUTF8	儉
U+5109.0	fCNS11643	1-6C27
U+5109.0	fBig5	BBFC
U+5109.0	fGB	1-3C73
U+5109.0	fR/S	 9.13
U+5109.0	fTotalStrokes	 15
U+5109.0	fCangjie	OOMO
U+5109.0	fFourCorner	28286
U+5109.0	fHakka	 kiam5; kiang3; kiam3
U+5109.0	fCantonese	gim6
U+5109.0	fMandarin	jian3; jian4
U+5109.0	fEnglish	[1] frugal; economical; thrift [2] meager [3] poor harvest
U+510A.0	fUTF8	儊
U+510A.0	fCNS11643	2-4F59
U+510A.0	fBig5	E4F1
U+510A.0	fGB	E-2242
U+510A.0	fR/S	 9.13
U+510A.0	fTotalStrokes	 15
U+510A.0	fCangjie	ODDO
U+510A.0	fFourCorner	24281
U+510A.0	fHakka	 cu3; cih3; ze5; cih5
U+510A.0	fCantonese	co3; co2
U+510A.0	fMandarin	chu4
U+510A.0	fEnglish	rough and rugged
U+510B.0	fUTF8	儋
U+510B.0	fCNS11643	2-4F56
U+510B.0	fBig5	E4EE
U+510B.0	fGB	0-5959
U+510B.0	fR/S	 9.13
U+510B.0	fTotalStrokes	 15
U+510B.0	fCangjie	ONCR
U+510B.0	fFourCorner	27261
U+510B.0	fHakka	 dam5; zham1; zam1
U+510B.0	fCantonese	daam1; daam3
U+510B.0	fMandarin	dan1; dan4
U+510B.0	fEnglish	[1] shoulder or bear a burden [2] load of two piculs (especially of rice or grains)
U+510C.0	fUTF8	儌
U+510C.0	fCNS11643	2-4F57
U+510C.0	fBig5	E4EF
U+510C.0	fGB	E-2243
U+510C.0	fR/S	 9.13
U+510C.0	fTotalStrokes	 15
U+510C.0	fCangjie	OHSK
U+510C.0	fFourCorner	28240
U+510C.0	fHakka	 hiau1; hieu1
U+510C.0	fCantonese	giu2; giu1 (also); hiu1
U+510C.0	fMandarin	jiao3; yao2; jiao1
U+510C.0	fEnglish	lucky; fortunate
U+510D.0	fUTF8	儍
U+510D.0	fCNS11643	3-4B2C
U+510D.0	fGB	5-3277
U+510D.0	fR/S	 9.13
U+510D.0	fTotalStrokes	 15
U+510D.0	fCangjie	OUCE
U+510D.0	fFourCorner	22247
U+510D.0	fHakka	 so2
U+510D.0	fCantonese	so4
U+510D.0	fMandarin	sha3
U+510D.0	fEnglish	[1] stupid; foolish; silly; imbecile; dumb [2] naive [3] stunned; stupefied; terrified [4] think or work mechanically
U+510E.0	fUTF8	儎
U+510E.0	fCNS11643	3-4B27
U+510E.0	fGB	5-3252
U+510E.0	fR/S	  9.13
U+510E.0	fTotalStrokes	 15
U+510E.0	fCangjie	OJIJ
U+510E.0	fFourCorner	23250
U+510E.0	fHakka	  zai5;  zoi5
U+510E.0	fCantonese	zoi3
U+510E.0	fMandarin	zai4; zai3
U+510E.0	fEnglish	carry; contain; load
U+510F.0	fUTF8	儏
U+510F.0	fCNS11643	3-644E
U+510F.0	fGB	E-2244
U+510F.0	fR/S	 9.13
U+510F.0	fTotalStrokes	 15
U+510F.0	fCangjie	OYED
U+510F.0	fFourCorner	2729
U+510F.0	fCantonese	caan3
U+5110.0	fUTF8	儐
U+5110.0	fCNS11643	1-7033
U+5110.0	fBig5	BEAA
U+5110.0	fGB	1-594F
U+5110.0	fR/S	 9.14
U+5110.0	fTotalStrokes	 16
U+5110.0	fCangjie	OJMC
U+5110.0	fFourCorner	23286
U+5110.0	fHakka	 bin5; bin1
U+5110.0	fCantonese	ban3; ban1 (also)
U+5110.0	fMandarin	bin4; bin1
U+5110.0	fEnglish	[1] entertain guests [2] arrange; set in order [3] guide
U+5111.0	fUTF8	儑
U+5111.0	fCNS11643	2-5643
U+5111.0	fBig5	E8F8
U+5111.0	fGB	3-3242
U+5111.0	fR/S	 9.14
U+5111.0	fTotalStrokes	 16
U+5111.0	fCangjie	OAVF
U+5111.0	fFourCorner	26233
U+5111.0	fHakka	 ngam2
U+5111.0	fCantonese	ngaam4; aam4 (also)
U+5111.0	fMandarin	an4; e4
U+5111.0	fEnglish	[1] devoid of intelligence; not at ease [2] foolish gibes; jokes; raillery
U+5112.0	fUTF8	儒
U+5112.0	fCNS11643	1-7030
U+5112.0	fBig5	BEA7
U+5112.0	fGB	0-4865
U+5112.0	fR/S	 9.14
U+5112.0	fTotalStrokes	 16
U+5112.0	fCangjie	OMBB
U+5112.0	fFourCorner	21227
U+5112.0	fHakka	  ji2; ji3; ri2; i2
U+5112.0	fCantonese	jyu4
U+5112.0	fMandarin	ru2
U+5112.0	fEnglish	[1] the learned; scholars collectively [2] Confucian; Confucianism [3] weak; shrinking from hardship
U+5113.0	fUTF8	儓
U+5113.0	fCNS11643	2-5640
U+5113.0	fBig5	E8F5
U+5113.0	fGB	5-327A
U+5113.0	fR/S	 9.14
U+5113.0	fTotalStrokes	 16
U+5113.0	fCangjie	OGRG
U+5113.0	fFourCorner	24214
U+5113.0	fHakka	  toi2; toi3
U+5113.0	fCantonese	toi4
U+5113.0	fMandarin	tai2; dai4
U+5113.0	fEnglish	servant
U+5114.0	fUTF8	儔
U+5114.0	fCNS11643	1-7032
U+5114.0	fBig5	BEA9
U+5114.0	fGB	1-5931
U+5114.0	fR/S	 9.14
U+5114.0	fTotalStrokes	 16
U+5114.0	fCangjie	OGNI
U+5114.0	fFourCorner	24241
U+5114.0	fHakka	 cu2;  chiu2; ciu2; chu2
U+5114.0	fCantonese	cau4
U+5114.0	fMandarin	chou2
U+5114.0	fEnglish	[1] companion; companions [2] class
U+5115.0	fUTF8	儕
U+5115.0	fCNS11643	1-7034
U+5115.0	fBig5	BEAB
U+5115.0	fGB	1-592D
U+5115.0	fR/S	 9.14
U+5115.0	fTotalStrokes	 16
U+5115.0	fCangjie	OYX
U+5115.0	fFourCorner	20223
U+5115.0	fHakka	 sa2; sa3; ha2; ce2
U+5115.0	fCantonese	caai4
U+5115.0	fMandarin	chai2
U+5115.0	fEnglish	[1] class; company [2] and adjunct to show plurality [3] match (as man and wife) [4] people; persons [5] all
U+5116.0	fUTF8	儖
U+5116.0	fCNS11643	3-5031
U+5116.0	fGB	E-2245
U+5116.0	fR/S	  9.14
U+5116.0	fTotalStrokes	 16
U+5116.0	fCangjie	OSIT
U+5116.0	fFourCorner	28210
U+5116.0	fCantonese	laam4
U+5116.0	fMandarin	lan2
U+5117.0	fUTF8	儗
U+5117.0	fCNS11643	2-5641
U+5117.0	fBig5	E8F6
U+5117.0	fGB	E-2246
U+5117.0	fR/S	 9.14
U+5117.0	fTotalStrokes	 16
U+5117.0	fCangjie	OPKO
U+5117.0	fFourCorner	27281
U+5117.0	fHakka	  ngi3
U+5117.0	fCantonese	ji5; ji4 (also); ji6; hoi3
U+5117.0	fMandarin	ni3; hai4; ni4
U+5117.0	fEnglish	compare with; draw analogy with
U+5118.0	fUTF8	儘
U+5118.0	fCNS11643	1-7031
U+5118.0	fBig5	BEA8
U+5118.0	fGB	1-7845
U+5118.0	fR/S	 9.14
U+5118.0	fTotalStrokes	 16
U+5118.0	fCangjie	OLMT
U+5118.0	fFourCorner	25217
U+5118.0	fHakka	 zin3; cin3; cin5; cin6
U+5118.0	fCantonese	zeon2
U+5118.0	fMandarin	jin3; jin4
U+5118.0	fEnglish	[1] utmost; extreme [2] let (someone do it)
U+5119.0	fUTF8	儙
U+5119.0	fCNS11643	4-4F5A
U+5119.0	fGB	E-2247
U+5119.0	fR/S	 9.14
U+5119.0	fTotalStrokes	 16
U+5119.0	fCangjie	OYLR
U+5119.0	fFourCorner	25230
U+5119.0	fCantonese	hin3
U+5119.0	fMandarin	qian4
U+511A.0	fUTF8	儚
U+511A.0	fCNS11643	2-5642
U+511A.0	fBig5	E8F7
U+511A.0	fGB	3-323D
U+511A.0	fR/S	 9.14
U+511A.0	fTotalStrokes	 16
U+511A.0	fCangjie	OTWN
U+511A.0	fFourCorner	25227
U+511A.0	fCantonese	mung4; mung5
U+511A.0	fMandarin	meng2
U+511A.0	fEnglish	equivalent to 果敢 U+679C U+6562, fleeting, momentary, ephemeral; vain, empty; fickle
U+511B.0	fUTF8	儛
U+511B.0	fCNS11643	3-5035
U+511B.0	fGB	5-327D
U+511B.0	fR/S	  9.14
U+511B.0	fTotalStrokes	 16
U+511B.0	fCangjie	OOTQ
U+511B.0	fFourCorner	28250
U+511B.0	fHakka	 wu3; vu3
U+511B.0	fCantonese	mou5
U+511B.0	fMandarin	wu3
U+511B.0	fEnglish	 skip about; dance for joy
U+511C.0	fUTF8	儜
U+511C.0	fCNS11643	2-563F
U+511C.0	fBig5	E8F4
U+511C.0	fGB	3-3073
U+511C.0	fR/S	 9.14
U+511C.0	fTotalStrokes	 16
U+511C.0	fCangjie	OJPN
U+511C.0	fFourCorner	23221
U+511C.0	fHakka	  nen2
U+511C.0	fCantonese	ning4; ling4 (also); nang4
U+511C.0	fMandarin	neng2; ning2
U+511C.0	fEnglish	 weak wearied in distress
U+511D.0	fUTF8	儝
U+511D.0	fCNS11643	3-5030
U+511D.0	fGB	E-2248
U+511D.0	fR/S	 9.14
U+511D.0	fTotalStrokes	 16
U+511D.0	fCangjie	OFFD
U+511D.0	fFourCorner	29290
U+511D.0	fCantonese	king4
U+511D.0	fMandarin	qiong2
U+511E.0	fUTF8	儞
U+511E.0	fCNS11643	3-5034
U+511E.0	fGB	E-2249
U+511E.0	fR/S	  9.14
U+511E.0	fTotalStrokes	 16
U+511E.0	fCangjie	OMFB
U+511E.0	fFourCorner	21220
U+511E.0	fCantonese	nei5; lei5 (also)
U+511E.0	fMandarin	er3; ni3; nai3
U+511F.0	fUTF8	償
U+511F.0	fCNS11643	1-7341
U+511F.0	fBig5	C076
U+511F.0	fGB	1-3325
U+511F.0	fR/S	 9.15
U+511F.0	fTotalStrokes	 17
U+511F.0	fCangjie	OFBC
U+511F.0	fFourCorner	29286
U+511F.0	fHakka	 song2;  shong2; song3
U+511F.0	fCantonese	soeng4; coeng4
U+511F.0	fMandarin	chang2; shang3
U+511F.0	fEnglish	[1] repay [2] compensate; make restitution [3] fulfill (a wish)
U+5120.0	fUTF8	儠
U+5120.0	fCNS11643	2-5C49
U+5120.0	fBig5	ECBD
U+5120.0	fGB	3-3247
U+5120.0	fR/S	 9.15
U+5120.0	fTotalStrokes	 17
U+5120.0	fCangjie	OVVV
U+5120.0	fFourCorner	22216
U+5120.0	fHakka	 liap8; lap8
U+5120.0	fCantonese	lip6
U+5120.0	fMandarin	lie4
U+5120.0	fEnglish	[1] robust [2] vigorous
U+5121.0	fUTF8	儡
U+5121.0	fCNS11643	1-7342
U+5121.0	fBig5	C077
U+5121.0	fGB	0-405C
U+5121.0	fR/S	 9.15
U+5121.0	fTotalStrokes	 17
U+5121.0	fCangjie	OWWW
U+5121.0	fFourCorner	26260
U+5121.0	fHakka	 lui3
U+5121.0	fCantonese	leoi5
U+5121.0	fMandarin	lei3; lei2
U+5121.0	fEnglish	[1] puppet; dummy [2] sickly and thin [3] dilapidated
U+5122.0	fUTF8	儢
U+5122.0	fCNS11643	2-5C47
U+5122.0	fBig5	ECBB
U+5122.0	fGB	3-3178
U+5122.0	fR/S	 9.15
U+5122.0	fTotalStrokes	 17
U+5122.0	fCangjie	OYPP
U+5122.0	fFourCorner	21236
U+5122.0	fHakka	 li5; li3; li6
U+5122.0	fCantonese	leoi5
U+5122.0	fMandarin	luu3; lu3
U+5122.0	fEnglish	careless; negligent
U+5123.0	fUTF8	儣
U+5123.0	fCNS11643	4-5565
U+5123.0	fGB	5-314B
U+5123.0	fR/S	  9.15
U+5123.0	fTotalStrokes	 16; 17
U+5123.0	fCangjie	OITC
U+5123.0	fFourCorner	20280
U+5123.0	fCantonese	kwong2; kong2 (also)
U+5123.0	fMandarin	kuang4; kuang3
U+5124.0	fUTF8	儤
U+5124.0	fCNS11643	2-5C48
U+5124.0	fBig5	ECBC
U+5124.0	fGB	3-3245
U+5124.0	fR/S	 9.15
U+5124.0	fTotalStrokes	 17
U+5124.0	fCangjie	OATE
U+5124.0	fFourCorner	26032
U+5124.0	fHakka	 bau5; pau5
U+5124.0	fCantonese	bou3
U+5124.0	fMandarin	bao4
U+5124.0	fEnglish	on duty
U+5125.0	fUTF8	儥
U+5125.0	fCNS11643	2-5C46
U+5125.0	fBig5	ECBA
U+5125.0	fGB	3-314D
U+5125.0	fR/S	 9.15
U+5125.0	fTotalStrokes	 17
U+5125.0	fCangjie	OGWC
U+5125.0	fFourCorner	24286
U+5125.0	fHakka	 juk7; ruk7
U+5125.0	fCantonese	juk6; dik6; duk6
U+5125.0	fMandarin	du2; di2; yu4
U+5125.0	fEnglish	[1] sell [2] hawk about for sale
U+5126.0	fUTF8	儦
U+5126.0	fCNS11643	2-5C45
U+5126.0	fBig5	ECB9
U+5126.0	fGB	3-3246
U+5126.0	fR/S	 9.15
U+5126.0	fTotalStrokes	 17
U+5126.0	fCangjie	OIPF
U+5126.0	fFourCorner	20231
U+5126.0	fHakka	 piau1; biau1
U+5126.0	fCantonese	biu1
U+5126.0	fMandarin	biao1
U+5126.0	fEnglish	milling
U+5127.0	fUTF8	儧
U+5127.0	fCNS11643	3-5441
U+5127.0	fGB	5-3268
U+5127.0	fR/S	  9.15
U+5127.0	fTotalStrokes	 17
U+5127.0	fCangjie	OQOC
U+5127.0	fFourCorner	25280
U+5127.0	fHakka	 zan3
U+5127.0	fCantonese	zaan2
U+5127.0	fMandarin	zan3
U+5128.0	fUTF8	儨
U+5128.0	fCNS11643	3-5440
U+5128.0	fGB	E-224A
U+5128.0	fR/S	 9.15
U+5128.0	fTotalStrokes	 17
U+5128.0	fCangjie	OHLC
U+5128.0	fFourCorner	22280
U+5128.0	fCantonese	zat1
U+5128.0	fMandarin	zhi2
U+5129.0	fUTF8	儩
U+5129.0	fCNS11643	2-5C4A
U+5129.0	fBig5	ECBE
U+5129.0	fGB	3-3234
U+5129.0	fR/S	 9.15
U+5129.0	fTotalStrokes	 17
U+5129.0	fCangjie	OBCH
U+5129.0	fFourCorner	26227
U+5129.0	fCantonese	ci3; si3
U+5129.0	fMandarin	si4
U+5129.0	fEnglish	complete
U+512A.0	fUTF8	優
U+512A.0	fCNS11643	1-7340
U+512A.0	fBig5	C075
U+512A.0	fGB	1-5345
U+512A.0	fR/S	 9.15
U+512A.0	fTotalStrokes	 17
U+512A.0	fCangjie	OMBE
U+512A.0	fFourCorner	21247
U+512A.0	fHakka	  ju2; riu2; jiu3; jiu1; jiu2
U+512A.0	fCantonese	jau1
U+512A.0	fMandarin	you1
U+512A.0	fEnglish	[1] good; superior; excellent [2] abundant; plenty [3] players (as in an opera); actor [4] victory; winning [5] soft [6] a Chinese family name
U+512B.0	fUTF8	儫
U+512B.0	fCNS11643	3-5033
U+512B.0	fGB	5-327E
U+512B.0	fR/S	 9.14; 9.15
U+512B.0	fTotalStrokes	 16
U+512B.0	fCangjie	OYRO
U+512B.0	fFourCorner	20230
U+512B.0	fHakka	 hau2; ho2
U+512B.0	fCantonese	hou4
U+512B.0	fMandarin	hao2
U+512C.0	fUTF8	儬
U+512C.0	fCNS11643	4-5569
U+512C.0	fGB	E-224B
U+512C.0	fR/S	 9.15
U+512C.0	fTotalStrokes	 17; 16
U+512C.0	fCangjie	OQBU
U+512C.0	fFourCorner	26210
U+512C.0	fCantonese	cing3
U+512C.0	fMandarin	chen4; qin4
U+512D.0	fUTF8	儭
U+512D.0	fCNS11643	2-6144
U+512D.0	fBig5	EFB8
U+512D.0	fGB	3-316E
U+512D.0	fR/S	 9.16
U+512D.0	fTotalStrokes	 18
U+512D.0	fCangjie	OYDU
U+512D.0	fFourCorner	26210
U+512D.0	fHakka	  cang5; cin1
U+512D.0	fCantonese	can3
U+512D.0	fMandarin	chen4; qin1
U+512D.0	fEnglish	 assist give alms
U+512E.0	fUTF8	儮
U+512E.0	fCNS11643	2-6145
U+512E.0	fBig5	EFB9
U+512E.0	fGB	E-224C
U+512E.0	fR/S	 9.16
U+512E.0	fTotalStrokes	 18
U+512E.0	fCangjie	OMHM
U+512E.0	fFourCorner	21211
U+512E.0	fCantonese	lik6
U+512E.0	fMandarin	li4
U+512F.0	fUTF8	儯
U+512F.0	fCNS11643	3-5752
U+512F.0	fGB	E-224D
U+512F.0	fR/S	  9.16
U+512F.0	fTotalStrokes	 18
U+512F.0	fCangjie	OFQF
U+512F.0	fFourCorner	29220
U+512F.0	fHakka	 tin2; ten2
U+512F.0	fCantonese	tang4
U+512F.0	fMandarin	teng2
U+512F.0	fEnglish	[1] tall; lanky [2] idle; careless
U+5130.0	fUTF8	儰
U+5130.0	fCNS11643	2-4F50
U+5130.0	fBig5	E4E8
U+5130.0	fGB	3-3136
U+5130.0	fR/S	 9.16
U+5130.0	fTotalStrokes	 14
U+5130.0	fCangjie	OTIF
U+5130.0	fFourCorner	20227
U+5130.0	fCantonese	wai5
U+5130.0	fMandarin	wei3
U+5131.0	fUTF8	儱
U+5131.0	fCNS11643	2-6143
U+5131.0	fBig5	EFB7
U+5131.0	fGB	5-3163
U+5131.0	fR/S	 9.16
U+5131.0	fTotalStrokes	 18
U+5131.0	fCangjie	OYBP
U+5131.0	fFourCorner	21211
U+5131.0	fHakka	  lung3
U+5131.0	fCantonese	lung5
U+5131.0	fMandarin	long3; long2; long4
U+5131.0	fEnglish	 rude barbarous
U+5132.0	fUTF8	儲
U+5132.0	fCNS11643	1-7343
U+5132.0	fBig5	C078
U+5132.0	fGB	1-3422
U+5132.0	fR/S	 9.16; 9.15
U+5132.0	fTotalStrokes	 18
U+5132.0	fCangjie	OYRA
U+5132.0	fFourCorner	24260
U+5132.0	fHakka	 su2; cu2; shu2; chu2; chiu2
U+5132.0	fCantonese	cyu5; cyu1; cyu4
U+5132.0	fMandarin	chu3; chu2
U+5132.0	fEnglish	[1] save; store; reserve; saving [2] deputy; alternate [3] a Chinese family name
U+5133.0	fUTF8	儳
U+5133.0	fCNS11643	1-782B
U+5133.0	fBig5	C35F
U+5133.0	fGB	3-324A
U+5133.0	fR/S	 9.17
U+5133.0	fTotalStrokes	 19
U+5133.0	fCangjie	ONRI
U+5133.0	fFourCorner	27213
U+5133.0	fHakka	  cam2; cam5
U+5133.0	fCantonese	caam3; caam4; zaam6
U+5133.0	fMandarin	chan4; chan2
U+5133.0	fEnglish	 obstinate stupid uneven mix
U+5134.0	fUTF8	儴
U+5134.0	fCNS11643	2-6537
U+5134.0	fBig5	F1EB
U+5134.0	fGB	5-3326
U+5134.0	fR/S	 9.17
U+5134.0	fTotalStrokes	 19
U+5134.0	fCangjie	OYRV
U+5134.0	fFourCorner	20232
U+5134.0	fHakka	 jong2; rong1; jong1; rong2
U+5134.0	fCantonese	joeng4; soeng1
U+5134.0	fMandarin	rang2
U+5134.0	fEnglish	an ancient form of $
U+5135.0	fUTF8	儵
U+5135.0	fCNS11643	2-6538
U+5135.0	fBig5	F1EC
U+5135.0	fGB	E-224E
U+5135.0	fR/S	 9.17
U+5135.0	fTotalStrokes	 19
U+5135.0	fCangjie	OLOF
U+5135.0	fFourCorner	27231
U+5135.0	fHakka	 sok7; shap7; shuk7; sap7
U+5135.0	fCantonese	suk1; suk6
U+5135.0	fMandarin	shu1; shu4; chou2
U+5135.0	fEnglish	hastily; hurriedly
U+5136.0	fUTF8	儶
U+5136.0	fCNS11643	3-5C32
U+5136.0	fGB	3-324B
U+5136.0	fR/S	 9.18
U+5136.0	fTotalStrokes	 20
U+5136.0	fCangjie	OUOB
U+5136.0	fFourCorner	22220
U+5136.0	fHakka	 zun5
U+5136.0	fCantonese	wai6
U+5136.0	fMandarin	hui4
U+5136.0	fEnglish	 valiant; brave eminent
U+5137.0	fUTF8	儷
U+5137.0	fCNS11643	1-7A62
U+5137.0	fBig5	C4D7
U+5137.0	fGB	1-5933
U+5137.0	fR/S	 9.19
U+5137.0	fTotalStrokes	 21
U+5137.0	fCangjie	OMMP
U+5137.0	fFourCorner	21211
U+5137.0	fHakka	  li5; li6
U+5137.0	fCantonese	lai6
U+5137.0	fMandarin	li4; li2
U+5137.0	fEnglish	[1] couple; pair [2] husband and wife; married couple
U+5138.0	fUTF8	儸
U+5138.0	fCNS11643	1-7A63
U+5138.0	fBig5	C4D8
U+5138.0	fGB	5-3234
U+5138.0	fR/S	 9.19
U+5138.0	fTotalStrokes	 21
U+5138.0	fCangjie	OWLG
U+5138.0	fFourCorner	26214
U+5138.0	fHakka	  lo2
U+5138.0	fCantonese	lo4
U+5138.0	fMandarin	luo2
U+5138.0	fEnglish	bandit; daredevil
U+5139.0	fUTF8	儹
U+5139.0	fCNS11643	2-6B4F
U+5139.0	fBig5	F5C1
U+5139.0	fGB	5-3324
U+5139.0	fR/S	 9.19
U+5139.0	fTotalStrokes	 21
U+5139.0	fCangjie	OHUC
U+5139.0	fFourCorner	24286
U+5139.0	fHakka	 zan5; zan3
U+5139.0	fCantonese	zaan2; zyun2 (also)
U+5139.0	fMandarin	zan3; zuan3
U+5139.0	fEnglish	 accumulate; hoard; store up
U+513A.0	fUTF8	儺
U+513A.0	fCNS11643	2-6B4E
U+513A.0	fBig5	F5C0
U+513A.0	fGB	1-5950
U+513A.0	fR/S	 9.19
U+513A.0	fTotalStrokes	 21
U+513A.0	fCangjie	OTOG
U+513A.0	fFourCorner	20214
U+513A.0	fHakka	  no2
U+513A.0	fCantonese	no4; lo4 (also)
U+513A.0	fMandarin	nuo2
U+513A.0	fEnglish	rich
U+513B.0	fUTF8	儻
U+513B.0	fCNS11643	1-7B58
U+513B.0	fBig5	C56C
U+513B.0	fGB	1-594E
U+513B.0	fR/S	 9.20
U+513B.0	fTotalStrokes	 22
U+513B.0	fCangjie	OFBF
U+513B.0	fFourCorner	29231
U+513B.0	fHakka	  tong3; dong3
U+513B.0	fCantonese	tong2
U+513B.0	fMandarin	tang3
U+513B.0	fEnglish	if
U+513C.0	fUTF8	儼
U+513C.0	fCNS11643	1-7B57
U+513C.0	fBig5	C56B
U+513C.0	fGB	1-5932
U+513C.0	fR/S	 9.20
U+513C.0	fTotalStrokes	 22
U+513C.0	fCangjie	ORRK
U+513C.0	fFourCorner	26248
U+513C.0	fHakka	 ngiam3; ngiam2
U+513C.0	fCantonese	jim5
U+513C.0	fMandarin	yan3
U+513C.0	fEnglish	[1] respectable; respectful; majestic [2] (to act, talk, appear) as if; like
U+513D.0	fUTF8	儽
U+513D.0	fCNS11643	2-6E7D
U+513D.0	fBig5	F7D0
U+513D.0	fGB	E-224F
U+513D.0	fR/S	 9.21
U+513D.0	fTotalStrokes	 23
U+513D.0	fCangjie	OWWF
U+513D.0	fFourCorner	26293
U+513D.0	fHakka	  lui5
U+513D.0	fCantonese	leoi5; leoi4 (also)
U+513D.0	fMandarin	lei4; lei3
U+513D.0	fEnglish	 lazy tired out; worn fatigued
U+513E.0	fUTF8	儾
U+513E.0	fCNS11643	3-606F
U+513E.0	fGB	3-324C
U+513E.0	fR/S	  9.22
U+513E.0	fTotalStrokes	 24
U+513E.0	fCangjie	OJBV
U+513E.0	fFourCorner	25230
U+513E.0	fCantonese	nong6; long6 (also)
U+513E.0	fMandarin	nang4
U+513E.0	fEnglish	 slow; dull irresolute
U+513F.0	fUTF8	儿
U+513F.0	fCNS11643	1-442A
U+513F.0	fBig5	A449
U+513F.0	fGB	0-3679
U+513F.0	fR/S	 10.0
U+513F.0	fTotalStrokes	 2
U+513F.0	fCangjie	LU
U+513F.0	fFourCorner	22010
U+513F.0	fHakka	 ngin2; jin2; rin2; ji2
U+513F.0	fCantonese	jan4; ji4; ji1 (also); ngai4; jan5
U+513F.0	fMandarin	er2; er1; ni2
U+513F.0	fEnglish	simplified form of &#x5152;
U+513F.1	fUTF8	儿
U+513F.1	fCNS11643	1-442A
U+513F.1	fBig5	A449
U+513F.1	fGB	0-3679
U+513F.1	fR/S	 10.0
U+513F.1	fTotalStrokes	 2
U+513F.1	fCangjie	LU
U+513F.1	fFourCorner	22010
U+513F.1	fMandarin	er5; r5
U+5140.0	fUTF8	兀
U+5140.0	fCNS11643	1-4442
U+5140.0	fBig5	A461
U+5140.0	fGB	0-5823
U+5140.0	fR/S	 10.1
U+5140.0	fTotalStrokes	 3
U+5140.0	fCangjie	MU
U+5140.0	fFourCorner	10210
U+5140.0	fHakka	 ngut7
U+5140.0	fCantonese	ngat6; at6 (also)
U+5140.0	fMandarin	wu4; wu1
U+5140.0	fEnglish	[1] cut off the feet [2] high and flat on the top [3] this [4] ignorant-looking
U+5141.0	fUTF8	允
U+5141.0	fCNS11643	1-4478
U+5141.0	fBig5	A4B9
U+5141.0	fGB	0-544A
U+5141.0	fR/S	 10.2
U+5141.0	fTotalStrokes	 4
U+5141.0	fCangjie	IHU
U+5141.0	fFourCorner	23210
U+5141.0	fHakka	  jun1;  jun3; run1; jiun1
U+5141.0	fCantonese	wan5; jyun4
U+5141.0	fMandarin	yun3; yan2
U+5141.0	fEnglish	 grant; allow; consent
U+5142.0	fUTF8	兂
U+5142.0	fCNS11643	4-213D
U+5142.0	fGB	E-2250
U+5142.0	fR/S	 10.2
U+5142.0	fTotalStrokes	 4
U+5142.0	fCangjie	KVU
U+5142.0	fFourCorner	40210
U+5142.0	fCantonese	zaam1
U+5142.0	fMandarin	zan1; zan3
U+5142.0	fEnglish	 hairpin; clasp wear in hair
U+5143.0	fUTF8	元
U+5143.0	fCNS11643	1-4477
U+5143.0	fBig5	A4B8
U+5143.0	fGB	0-542A
U+5143.0	fR/S	 10.2
U+5143.0	fTotalStrokes	 4
U+5143.0	fCangjie	MMU
U+5143.0	fFourCorner	10211
U+5143.0	fHakka	  ngien2;  jen2; nien2; gnian3; ngian2
U+5143.0	fCantonese	jyun4
U+5143.0	fMandarin	yuan2
U+5143.0	fEnglish	[1] beginning; first; original [2] dollar [3] head [4] eldest; chief; big [5] 60 years (Chinese astrology) [6] the Yuan Dynasty (1279-1367) [7] a Chinese family name
U+5144.0	fUTF8	兄
U+5144.0	fCNS11643	1-4573
U+5144.0	fBig5	A553
U+5144.0	fGB	0-5056
U+5144.0	fR/S	 10.3
U+5144.0	fTotalStrokes	 5
U+5144.0	fCangjie	RHU
U+5144.0	fFourCorner	60210
U+5144.0	fHakka	  hiung1
U+5144.0	fCantonese	hing1; fong3
U+5144.0	fMandarin	xiong1
U+5144.0	fEnglish	[1] one's elder brother [2] a term used in addressing a senior of the same generation to show respect
U+5144.1	fUTF8	兄
U+5144.1	fCNS11643	1-4573
U+5144.1	fBig5	A553
U+5144.1	fGB	0-5056
U+5144.1	fR/S	 10.3
U+5144.1	fTotalStrokes	 5
U+5144.1	fCangjie	RHU
U+5144.1	fFourCorner	60210
U+5144.1	fMandarin	xiong5
U+5145.0	fUTF8	充
U+5145.0	fCNS11643	1-4572
U+5145.0	fBig5	A552
U+5145.0	fGB	0-3364
U+5145.0	fR/S	 10.4; 10.3
U+5145.0	fTotalStrokes	 5
U+5145.0	fCangjie	YIHU
U+5145.0	fFourCorner	00213
U+5145.0	fHakka	  cung1;  chung1
U+5145.0	fCantonese	cung1
U+5145.0	fMandarin	chong1
U+5145.0	fEnglish	[1] fill [2] full; sufficient [3] take; cheat; pretend [4] a Chinese family name
U+5146.0	fUTF8	兆
U+5146.0	fCNS11643	1-473E
U+5146.0	fBig5	A5FC
U+5146.0	fGB	0-5557
U+5146.0	fR/S	 10.4
U+5146.0	fTotalStrokes	 6
U+5146.0	fCangjie	LMUO
U+5146.0	fFourCorner	32113
U+5146.0	fHakka	 shau5; cau3; cau5;  chau5; sau5; s'eu5; seu5; ch'eu5
U+5146.0	fCantonese	siu6; ziu6
U+5146.0	fMandarin	zhao4
U+5146.0	fEnglish	[1] sign (in fortune-telling); [2] portend; foretell [3] omen [4] million (China); trillion; billion (Taiwan) [5] begin; beginning [6] a Chinese family name
U+5147.0	fUTF8	兇
U+5147.0	fCNS11643	1-473D
U+5147.0	fBig5	A5FB
U+5147.0	fGB	E-2251
U+5147.0	fR/S	 10.4
U+5147.0	fTotalStrokes	 6
U+5147.0	fCangjie	UKHU
U+5147.0	fFourCorner	22217
U+5147.0	fHakka	  hiung1
U+5147.0	fCantonese	hung1
U+5147.0	fMandarin	xiong1; xiong3
U+5147.0	fEnglish	[1] fierce; violent; cruel; ferocious [2] truculent; inhuman
U+5148.0	fUTF8	先
U+5148.0	fCNS11643	1-473F
U+5148.0	fBig5	A5FD
U+5148.0	fGB	0-4F48
U+5148.0	fR/S	 10.4
U+5148.0	fTotalStrokes	 6
U+5148.0	fCangjie	HGHU
U+5148.0	fFourCorner	24211
U+5148.0	fHakka	 sien1;  sen1; sian1
U+5148.0	fCantonese	sin1; sin3
U+5148.0	fMandarin	xian1; xian3; xian4
U+5148.0	fEnglish	[1] first; foremost [2] before; earlier; in advance [3] the late ...; the deceased ... [4] one's forebears [5] the abbreviation for Mister or Sir [6] a Chinese family name
U+5149.0	fUTF8	光
U+5149.0	fCNS11643	1-473C
U+5149.0	fBig5	A5FA
U+5149.0	fGB	0-3962
U+5149.0	fR/S	 10.4
U+5149.0	fTotalStrokes	 6
U+5149.0	fCangjie	FMU
U+5149.0	fFourCorner	90211
U+5149.0	fHakka	  gong1; gwong1; kwong1
U+5149.0	fCantonese	gwong1; gong1 (also)
U+5149.0	fMandarin	guang1; guang2; guang4
U+5149.0	fEnglish	[1] light; brightness; brilliant; shine; light rays [2] glossy; smooth [3] glory; glorious; honor [4] exhaust; use up [5] alone; only [6] bare; bare; naked [7] a Chinese family name
U+514A.0	fUTF8	兊
U+514A.0	fCNS11643	3-2278
U+514A.0	fGB	E-2252
U+514A.0	fR/S	 10.4
U+514A.0	fTotalStrokes	 6
U+514A.0	fCangjie	CIHU
U+514A.0	fFourCorner	80210
U+514A.0	fCantonese	deoi3
U+514A.0	fMandarin	dui4
U+514B.0	fUTF8	克
U+514B.0	fCNS11643	1-492C
U+514B.0	fBig5	A74A
U+514B.0	fGB	0-3F4B
U+514B.0	fR/S	 10.5
U+514B.0	fTotalStrokes	 7
U+514B.0	fCangjie	JRHU
U+514B.0	fFourCorner	40216
U+514B.0	fHakka	 ket7;  kiet7
U+514B.0	fCantonese	hak1; haak1 (also)
U+514B.0	fMandarin	ke4
U+514B.0	fEnglish	[1] be able to [2] win; overcome; conquer [3] love of superiority [4] gram [5] limit
U+514C.0	fUTF8	兌
U+514C.0	fCNS11643	1-492B
U+514C.0	fBig5	A749
U+514C.0	fGB	E-2253
U+514C.0	fR/S	 10.5
U+514C.0	fTotalStrokes	 7
U+514C.0	fCangjie	CRHU
U+514C.0	fFourCorner	80216
U+514C.0	fHakka	  tui3;  dui5; tui5
U+514C.0	fCantonese	deoi3; deoi6 (also); jeoi6
U+514C.0	fMandarin	dui4; rui4; yue4
U+514C.0	fEnglish	[1] exchange; barter [2] water; weaken by adding water (said of wine, etc.)
U+514D.0	fUTF8	免
U+514D.0	fCNS11643	1-492D
U+514D.0	fBig5	A74B
U+514D.0	fGB	0-4362
U+514D.0	fR/S	 10.5
U+514D.0	fTotalStrokes	 8
U+514D.0	fCangjie	NAHU
U+514D.0	fFourCorner	27416
U+514D.0	fHakka	  men1;  mien1;  mien3; mian1; men3
U+514D.0	fCantonese	min5; man6
U+514D.0	fMandarin	mian3; wen4
U+514D.0	fEnglish	[1] avoid; escape; evade [2] forego; spare; excuse; exempt [3] dismiss (from office)
U+514E.0	fUTF8	兎
U+514E.0	fCNS11643	3-2454
U+514E.0	fGB	E-2254
U+514E.0	fR/S	 10.5
U+514E.0	fTotalStrokes	 7
U+514E.0	fCangjie	HLAI
U+514E.0	fFourCorner	20410
U+514E.0	fHakka	 tu5
U+514E.0	fCantonese	tou3
U+514E.0	fMandarin	tu4
U+514E.0	fEnglish	rabbit; hare
U+514F.0	fUTF8	兏
U+514F.0	fCNS11643	4-233B
U+514F.0	fGB	E-2255
U+514F.0	fR/S	 10.5
U+514F.0	fTotalStrokes	 7
U+514F.0	fCangjie	MYMU
U+514F.0	fFourCorner	72210
U+514F.0	fCantonese	coeng4
U+514F.0	fMandarin	chang2; zhang3
U+5150.0	fUTF8	児
U+5150.0	fCNS11643	3-2453
U+5150.0	fGB	E-2256
U+5150.0	fR/S	 10.5
U+5150.0	fTotalStrokes	 7
U+5150.0	fCangjie	LAHU
U+5150.0	fFourCorner	26210
U+5150.0	fCantonese	ji4
U+5150.0	fMandarin	er2; ni2
U+5150.0	fEnglish	 son; child; oneself final part
U+5151.0	fUTF8	兑
U+5151.0	fCNS11643	3-2451
U+5151.0	fGB	0-3652
U+5151.0	fR/S	 10.5
U+5151.0	fTotalStrokes	 7
U+5151.0	fCangjie	CRHU
U+5151.0	fFourCorner	80210
U+5151.0	fCantonese	deoi3; deoi6 (also); teoi3
U+5151.0	fMandarin	dui4; yue4; rui4
U+5151.0	fEnglish	 cash exchange
U+5152.0	fUTF8	兒
U+5152.0	fCNS11643	1-4C23
U+5152.0	fBig5	A8E0
U+5152.0	fGB	1-3679
U+5152.0	fR/S	 10.6
U+5152.0	fTotalStrokes	 8
U+5152.0	fCangjie	HXHU
U+5152.0	fFourCorner	77217
U+5152.0	fHakka	 ji2; ri2; ji3; i2; ngi2
U+5152.0	fCantonese	ngai4; ai4 (also); ji4; ji1 (also)
U+5152.0	fMandarin	er2; er1; ni2
U+5152.0	fEnglish	[1] baby; child [2] son [3] oneself; referring to oneself when addressing parents [4] final particle after noun, pronoun, adjective, adverb, and verb
U+5152.1	fUTF8	兒
U+5152.1	fCNS11643	1-4C23
U+5152.1	fBig5	A8E0
U+5152.1	fGB	1-3679
U+5152.1	fR/S	 10.6
U+5152.1	fTotalStrokes	 8
U+5152.1	fCangjie	HXHU
U+5152.1	fFourCorner	77217
U+5152.1	fMandarin	er5; r5
U+5153.0	fUTF8	兓
U+5153.0	fCNS11643	4-253F
U+5153.0	fGB	5-427A
U+5153.0	fR/S	  10.6
U+5153.0	fTotalStrokes	 8; 7
U+5153.0	fCangjie	MUMVU
U+5153.0	fFourCorner	55010
U+5153.0	fCantonese	zam1
U+5153.0	fMandarin	xin1; shen1
U+5154.0	fUTF8	兔
U+5154.0	fCNS11643	1-4C22
U+5154.0	fBig5	A8DF
U+5154.0	fGB	0-4D43
U+5154.0	fR/S	 10.6
U+5154.0	fTotalStrokes	 8
U+5154.0	fCangjie	NAHI
U+5154.0	fFourCorner	17413
U+5154.0	fHakka	  tu5
U+5154.0	fCantonese	tou3
U+5154.0	fMandarin	tu4
U+5154.0	fEnglish	[1] rabbit; hare [2] young boy kept for sexual perversion
U+5155.0	fUTF8	兕
U+5155.0	fCNS11643	1-4C24
U+5155.0	fBig5	A8E1
U+5155.0	fGB	0-596E
U+5155.0	fR/S	 10.6
U+5155.0	fTotalStrokes	 8
U+5155.0	fCangjie	SSUU
U+5155.0	fFourCorner	77217
U+5155.0	fHakka	  si1; si5; si6
U+5155.0	fCantonese	zi6; ci5; zi3
U+5155.0	fMandarin	si4
U+5155.0	fEnglish	 female rhinoceros
U+5156.0	fUTF8	兖
U+5156.0	fCNS11643	3-2752
U+5156.0	fGB	0-5970
U+5156.0	fR/S	 10.6
U+5156.0	fTotalStrokes	 8
U+5156.0	fCangjie	YCIHU
U+5156.0	fFourCorner	00210
U+5156.0	fCantonese	jin5; jin2
U+5156.0	fMandarin	yan3
U+5156.0	fEnglish	 establish one of nine empire divisions
U+5157.0	fUTF8	兗
U+5157.0	fCNS11643	1-5022
U+5157.0	fBig5	AB5E
U+5157.0	fGB	E-2257
U+5157.0	fR/S	 10.7
U+5157.0	fTotalStrokes	 9
U+5157.0	fCangjie	YCRHU
U+5157.0	fFourCorner	00216
U+5157.0	fHakka	 jen3; zan5; jan3
U+5157.0	fCantonese	jin2; jin5
U+5157.0	fMandarin	yan3
U+5157.0	fEnglish	one of nine empire divisions of the Empire under Yu the Great including parts of Shantung and Hopeh provinces
U+5158.0	fUTF8	兘
U+5158.0	fCNS11643	4-2840
U+5158.0	fGB	E-2258
U+5158.0	fR/S	 10.7
U+5158.0	fTotalStrokes	 9
U+5158.0	fCangjie	MUIR
U+5158.0	fFourCorner	13210
U+5158.0	fCantonese	ci2
U+5158.0	fMandarin	shi3
U+5159.0	fUTF8	兙
U+5159.0	fCNS11643	1-2279
U+5159.0	fBig5	A259
U+5159.0	fGB	E-2259
U+5159.0	fR/S	 10.7
U+5159.0	fTotalStrokes	 9
U+5159.0	fCangjie	JUJ
U+5159.0	fFourCorner	44210
U+5159.0	fCantonese	sap6
U+5159.0	fMandarin	shi2ke4
U+5159.0	fEnglish	decagram
U+515A.0	fUTF8	党
U+515A.0	fCNS11643	2-2E21
U+515A.0	fBig5	D0DE
U+515A.0	fGB	0-3533
U+515A.0	fR/S	 10.8
U+515A.0	fTotalStrokes	 10
U+515A.0	fCangjie	FBRHU
U+515A.0	fFourCorner	90216
U+515A.0	fHakka	 dong3
U+515A.0	fCantonese	dong2
U+515A.0	fMandarin	dang3; zhang3
U+515A.0	fEnglish	simplified form of &#x9EE8;
U+515B.0	fUTF8	兛
U+515B.0	fCNS11643	1-227A
U+515B.0	fBig5	A25A
U+515B.0	fGB	E-225A
U+515B.0	fR/S	 10.8
U+515B.0	fTotalStrokes	 10
U+515B.0	fCangjie	JUHJ
U+515B.0	fFourCorner	4221
U+515B.0	fCantonese	cin1
U+515B.0	fMandarin	qian1ke4; qian1
U+515B.0	fEnglish	kilogram
U+515C.0	fUTF8	兜
U+515C.0	fCNS11643	1-5927
U+515C.0	fBig5	B0C2
U+515C.0	fGB	0-3635
U+515C.0	fR/S	 10.9
U+515C.0	fTotalStrokes	 11
U+515C.0	fCangjie	HVHU
U+515C.0	fFourCorner	77217
U+515C.0	fHakka	  deu1; diu1
U+515C.0	fCantonese	dau1
U+515C.0	fMandarin	dou1
U+515C.0	fEnglish	[1] head-covering; helmet [2] overalls [3] solicit [4] go for a drive around; move around [5] surround; wrap up [6] small pocket on clothes; pouch
U+515D.0	fUTF8	兝
U+515D.0	fCNS11643	1-227C
U+515D.0	fBig5	A25C
U+515D.0	fGB	E-225B
U+515D.0	fR/S	 10.9
U+515D.0	fTotalStrokes	 11
U+515D.0	fCangjie	JUCSH
U+515D.0	fFourCorner	4821
U+515D.0	fCantonese	fan1
U+515D.0	fMandarin	fen1
U+515D.0	fEnglish	dg
U+515E.0	fUTF8	兞
U+515E.0	fCNS11643	1-227B
U+515E.0	fBig5	A25B
U+515E.0	fGB	E-225C
U+515E.0	fR/S	 10.9
U+515E.0	fTotalStrokes	 11
U+515E.0	fCangjie	JUHQU
U+515E.0	fFourCorner	4221
U+515E.0	fCantonese	hou4
U+515E.0	fMandarin	mao2
U+515E.0	fEnglish	mg
U+515F.0	fUTF8	兟
U+515F.0	fCNS11643	2-3A44
U+515F.0	fBig5	D860
U+515F.0	fGB	3-4948
U+515F.0	fR/S	 10.10
U+515F.0	fTotalStrokes	 12
U+515F.0	fCangjie	HUHGU
U+515F.0	fFourCorner	24211
U+515F.0	fHakka	 sin1
U+515F.0	fCantonese	san1; saan1
U+515F.0	fMandarin	xin1; shen1
U+515F.0	fEnglish	 advance
U+5160.0	fUTF8	兠
U+5160.0	fCNS11643	3-3976
U+5160.0	fGB	E-225D
U+5160.0	fR/S	 10.10; 16.10
U+5160.0	fTotalStrokes	 12
U+5160.0	fCangjie	LMMU
U+5160.0	fFourCorner	11210
U+5160.0	fHakka	 deu1
U+5160.0	fCantonese	dau1
U+5160.0	fMandarin	dou1
U+5161.0	fUTF8	兡
U+5161.0	fCNS11643	1-227D
U+5161.0	fBig5	A25D
U+5161.0	fGB	E-225E
U+5161.0	fR/S	 10.11
U+5161.0	fTotalStrokes	 13
U+5161.0	fCangjie	JUMA
U+5161.0	fFourCorner	4121
U+5161.0	fCantonese	baak3
U+5161.0	fMandarin	bai3ke4
U+5161.0	fEnglish	hg
U+5162.0	fUTF8	兢
U+5162.0	fCNS11643	1-677E
U+5162.0	fBig5	B9B8
U+5162.0	fGB	0-3E24
U+5162.0	fR/S	 10.12
U+5162.0	fTotalStrokes	 14
U+5162.0	fCangjie	JUJRU
U+5162.0	fFourCorner	44216
U+5162.0	fHakka	 kin1; gin5; kin3; gin1
U+5162.0	fCantonese	ging1
U+5162.0	fMandarin	jing1
U+5162.0	fEnglish	 fear; freed; fearful; apprehensive; cautious
U+5163.0	fUTF8	兣
U+5163.0	fCNS11643	1-227E
U+5163.0	fBig5	A25E
U+5163.0	fGB	E-225F
U+5163.0	fR/S	 10.14
U+5163.0	fTotalStrokes	 16
U+5163.0	fCangjie	JUMWG
U+5163.0	fFourCorner	4121
U+5163.0	fCantonese	lei4
U+5163.0	fMandarin	li3; ke4
U+5163.0	fEnglish	 decigram cg
U+5164.0	fUTF8	兤
U+5164.0	fCNS11643	3-5E25
U+5164.0	fGB	E-2260
U+5164.0	fR/S	  10.19
U+5164.0	fTotalStrokes	 21
U+5164.0	fCangjie	FUITC
U+5164.0	fFourCorner	90210
U+5164.0	fCantonese	fong2
U+5164.0	fMandarin	huang3; huang2
U+5165.0	fUTF8	入
U+5165.0	fCNS11643	1-442B
U+5165.0	fBig5	A44A
U+5165.0	fGB	0-486B
U+5165.0	fR/S	 11.0
U+5165.0	fTotalStrokes	 2
U+5165.0	fCangjie	OH
U+5165.0	fFourCorner	80000
U+5165.0	fHakka	  ngip8; gnip8
U+5165.0	fCantonese	jap6
U+5165.0	fMandarin	ru4
U+5165.0	fEnglish	[1] enter; come in(to) [2] join; come into the company of [3] arrive at; reach [4] put in [5] receipts; income [6] get out of sight; disappear [7] get (inside, picked, elected, etc.) [8] agree with; conform to [9] one of the four tones of a character
U+5166.0	fUTF8	兦
U+5166.0	fCNS11643	3-2134
U+5166.0	fGB	E-2261
U+5166.0	fR/S	 11.1
U+5166.0	fTotalStrokes	 3
U+5166.0	fCangjie	XOV
U+5166.0	fFourCorner	28710
U+5166.0	fHakka	 mong2
U+5166.0	fCantonese	mong4; mou4
U+5166.0	fMandarin	wang2; wu2
U+5166.0	fEnglish	destruction
U+5167.0	fUTF8	內
U+5167.0	fCNS11643	1-4479
U+5167.0	fBig5	A4BA
U+5167.0	fGB	E-2262
U+5167.0	fR/S	 11.2
U+5167.0	fTotalStrokes	 4
U+5167.0	fCangjie	OB
U+5167.0	fFourCorner	40227
U+5167.0	fHakka	 nui5; nui6; lui3
U+5167.0	fCantonese	noi6; naap6; loi6 (also); laap6 (also)
U+5167.0	fMandarin	nei4; na4
U+5167.0	fEnglish	[1] inside; within; inner; interior [2] wife [3] palace of an emperor
U+5168.0	fUTF8	全
U+5168.0	fCNS11643	1-4740
U+5168.0	fBig5	A5FE
U+5168.0	fGB	0-482B
U+5168.0	fR/S	 11.4; 9.4
U+5168.0	fTotalStrokes	 6
U+5168.0	fCangjie	OMG
U+5168.0	fFourCorner	80104
U+5168.0	fHakka	 cien2; cion2; cen2; ts'ian3
U+5168.0	fCantonese	cyun4; cyun2 (also)
U+5168.0	fMandarin	quan2
U+5168.0	fEnglish	[1] perfect [2] complete; whole; total; intact; all; entire; absolute [3] to keep whole or intact [4] a Chinese family name
U+5169.0	fUTF8	兩
U+5169.0	fCNS11643	1-4C25
U+5169.0	fBig5	A8E2
U+5169.0	fGB	1-413D
U+5169.0	fR/S	 11.6
U+5169.0	fTotalStrokes	 8
U+5169.0	fCangjie	MLBO
U+5169.0	fFourCorner	10227
U+5169.0	fHakka	  liong3;  liong1
U+5169.0	fCantonese	loeng2; loeng5; loeng6
U+5169.0	fMandarin	liang3; liang4
U+5169.0	fEnglish	[1] two; pair; couple [2] both; either [3] tael; ounce [4] piece of cloth, etc. of about 44 feet (in Ancient China) [5] a group of 25 soldiers (in Ancient China)
U+516A.0	fUTF8	兪
U+516A.0	fCNS11643	3-2B37
U+516A.0	fGB	E-2263
U+516A.0	fR/S	 11.7
U+516A.0	fTotalStrokes	 9
U+516A.0	fCangjie	OMBV
U+516A.0	fFourCorner	80230
U+516A.0	fHakka	 ji2; ji3; ri2; i2
U+516A.0	fCantonese	jyu4; jyu6; syu3
U+516A.0	fMandarin	yu2; shu4; yu4
U+516A.0	fEnglish	[1] a Chinese family name [2] to answer in the affirmative; consent [3] make a boat by hollowing the log
U+516B.0	fUTF8	八
U+516B.0	fCNS11643	1-442C
U+516B.0	fBig5	A44B
U+516B.0	fGB	0-304B
U+516B.0	fR/S	 12.0
U+516B.0	fTotalStrokes	 2
U+516B.0	fCangjie	HO
U+516B.0	fFourCorner	80000
U+516B.0	fHakka	  bat7
U+516B.0	fCantonese	baat3; baat2 (also)
U+516B.0	fMandarin	ba1; ba2
U+516B.0	fEnglish	eight
U+516C.0	fUTF8	公
U+516C.0	fCNS11643	1-447C
U+516C.0	fBig5	A4BD
U+516C.0	fGB	0-392B
U+516C.0	fR/S	 12.2
U+516C.0	fTotalStrokes	 4
U+516C.0	fCangjie	CI
U+516C.0	fFourCorner	80732
U+516C.0	fHakka	  gung1
U+516C.0	fCantonese	gung1; gung4 (also)
U+516C.0	fMandarin	gong1; zhong1
U+516C.0	fEnglish	[1] unselfish; unbiased; fair [2] make public; open to all [3] public [4] the first of old China's five grades of the nobility; duke [5] the father of one's husband; one's father-in-law [6] grandfather [7] a respectful salutation [8] the male (animals) [9] office; official duties [10] a Chinese family name
U+516D.0	fUTF8	六
U+516D.0	fCNS11643	1-447A
U+516D.0	fBig5	A4BB
U+516D.0	fGB	0-4179
U+516D.0	fR/S	 12.2
U+516D.0	fTotalStrokes	 4
U+516D.0	fCangjie	YC
U+516D.0	fFourCorner	00800
U+516D.0	fHakka	 liuk7;  luk7
U+516D.0	fCantonese	luk6; luk2 (also); liu1
U+516D.0	fMandarin	liu4; lu4; liu2
U+516D.0	fEnglish	six
U+516E.0	fUTF8	兮
U+516E.0	fCNS11643	1-447B
U+516E.0	fBig5	A4BC
U+516E.0	fGB	0-5962
U+516E.0	fR/S	 12.2
U+516E.0	fTotalStrokes	 4
U+516E.0	fCangjie	CMVS
U+516E.0	fFourCorner	80207
U+516E.0	fHakka	  hi1; hi2
U+516E.0	fCantonese	hai4
U+516E.0	fMandarin	xi1
U+516E.0	fEnglish	[1] an adjunct with no independent meaning, roughly equivalent to "oh" or "ah" in English; exclamatory particle [2] a particle of pause used in ancient poetry and still used in eulogies
U+516F.0	fUTF8	兯
U+516F.0	fGB	1-7D67
U+516F.0	fR/S	 12.2; 26.2
U+516F.0	fTotalStrokes	 4
U+516F.0	fCangjie	CSL
U+516F.0	fCantonese	zit3
U+516F.0	fEnglish	 kwukyel
U+5170.0	fUTF8	兰
U+5170.0	fGB	0-403C
U+5170.0	fR/S	 12.3
U+5170.0	fTotalStrokes	 5
U+5170.0	fCangjie	TMM
U+5170.0	fHakka	 lan2
U+5170.0	fCantonese	laan4; laan1 (also); laan2 (also)
U+5170.0	fMandarin	lan2
U+5170.0	fEnglish	simplified form of &#x862D;
U+5171.0	fUTF8	共
U+5171.0	fCNS11643	1-4741
U+5171.0	fBig5	A640
U+5171.0	fGB	0-3932
U+5171.0	fR/S	 12.4
U+5171.0	fTotalStrokes	 6
U+5171.0	fCangjie	TC
U+5171.0	fFourCorner	44801
U+5171.0	fHakka	  kiung5;  giung3; kiung6; kiung3; giung1
U+5171.0	fCantonese	gung6; gung1; gung2
U+5171.0	fMandarin	gong4; gong1; gong3; hong2; ju4
U+5171.0	fEnglish	[1] common; same [2] together [3] all; total collectively [4] share; work together
U+5172.0	fUTF8	兲
U+5172.0	fCNS11643	3-227A
U+5172.0	fGB	E-2264
U+5172.0	fR/S	 12.4
U+5172.0	fTotalStrokes	 6
U+5172.0	fCangjie	MGC
U+5172.0	fFourCorner	10800
U+5172.0	fCantonese	tin1
U+5172.0	fMandarin	tian1
U+5172.0	fEnglish	 heaven sky
U+5173.0	fUTF8	关
U+5173.0	fCNS11643	4-2234
U+5173.0	fGB	0-3958
U+5173.0	fR/S	 12.4
U+5173.0	fTotalStrokes	 6
U+5173.0	fCangjie	TK
U+5173.0	fFourCorner	80430
U+5173.0	fHakka	 kwan1
U+5173.0	fCantonese	gwaan1
U+5173.0	fMandarin	xiao4; guan1; wan1; chuan3
U+5173.0	fEnglish	simplified form of &#x95DC;
U+5174.0	fUTF8	兴
U+5174.0	fGB	0-504B
U+5174.0	fR/S	 12.4
U+5174.0	fTotalStrokes	 6
U+5174.0	fCangjie	FMC
U+5174.0	fHakka	 hin1; hin5
U+5174.0	fCantonese	hing1; hing3
U+5174.0	fMandarin	xing1; xing4
U+5174.0	fEnglish	simplified form of &#x8208;
U+5174.1	fUTF8	兴
U+5174.1	fGB	0-504B
U+5174.1	fR/S	 12.4
U+5174.1	fTotalStrokes	 6
U+5174.1	fCangjie	FMC
U+5175.0	fUTF8	兵
U+5175.0	fCNS11643	1-492E
U+5175.0	fBig5	A74C
U+5175.0	fGB	0-3178
U+5175.0	fR/S	 12.5
U+5175.0	fTotalStrokes	 7
U+5175.0	fCangjie	OMC
U+5175.0	fFourCorner	72801
U+5175.0	fHakka	  bin1
U+5175.0	fCantonese	bing1
U+5175.0	fMandarin	bing1
U+5175.0	fEnglish	[1] arms; weapons [2] soldier; serviceman; the military; troops [3] strike; attack [4] a piece in Chinese Chess; pawn
U+5176.0	fUTF8	其
U+5176.0	fCNS11643	1-4C27
U+5176.0	fBig5	A8E4
U+5176.0	fGB	0-4664
U+5176.0	fR/S	 12.6
U+5176.0	fTotalStrokes	 8
U+5176.0	fCangjie	TMMC
U+5176.0	fFourCorner	44801
U+5176.0	fHakka	  ki2; ki3; gia1
U+5176.0	fCantonese	kei4; gei1; gei6; gei3
U+5176.0	fMandarin	qi2; ji1
U+5176.0	fEnglish	[1] he; she; it; they; his; her; its; their [2] this; that; the [3] an interrogative used to introduce a question
U+5177.0	fUTF8	具
U+5177.0	fCNS11643	1-4C26
U+5177.0	fBig5	A8E3
U+5177.0	fGB	0-3E5F
U+5177.0	fR/S	 12.6
U+5177.0	fTotalStrokes	 8
U+5177.0	fCangjie	BMMC
U+5177.0	fFourCorner	60801
U+5177.0	fHakka	  ki3; ki1; gi5
U+5177.0	fCantonese	geoi6
U+5177.0	fMandarin	ju4
U+5177.0	fEnglish	[1] appliance; implement; utensil; tools, etc. [2] talent; capability [3] prepare; equip [4] complete; all; the whole [5] a Chinese family name
U+5177.1	fUTF8	具
U+5177.1	fCNS11643	1-4C26
U+5177.1	fBig5	A8E3
U+5177.1	fGB	0-3E5F
U+5177.1	fR/S	 12.6
U+5177.1	fTotalStrokes	 8
U+5177.1	fCangjie	BMMC
U+5177.1	fFourCorner	60801
U+5177.1	fMandarin	ju5
U+5178.0	fUTF8	典
U+5178.0	fCNS11643	1-4C28
U+5178.0	fBig5	A8E5
U+5178.0	fGB	0-3564
U+5178.0	fR/S	 12.6
U+5178.0	fTotalStrokes	 8
U+5178.0	fCangjie	TBC
U+5178.0	fFourCorner	55801
U+5178.0	fHakka	 dien3; den3; tian3
U+5178.0	fCantonese	din2
U+5178.0	fMandarin	dian3; tian3
U+5178.0	fEnglish	[1] rule; statute; law; canon [2] tale or story from the classics; an allusion [3] pawn; mortgage [4] take charge of
U+5179.0	fUTF8	兹
U+5179.0	fCNS11643	3-2E7B
U+5179.0	fGB	0-5748
U+5179.0	fR/S	 140.6; 12.7
U+5179.0	fTotalStrokes	 9
U+5179.0	fCangjie	TVII
U+5179.0	fFourCorner	80730
U+5179.0	fCantonese	zi1; ci4
U+5179.0	fMandarin	zi1; ci2
U+5179.0	fEnglish	 now; here this time; year
U+517A.0	fUTF8	兺
U+517A.0	fGB	1-7D45
U+517A.0	fR/S	 12.8
U+517A.0	fTotalStrokes	 10
U+517A.0	fCangjie	XXCSH
U+517A.0	fCantonese	fan1
U+517B.0	fUTF8	养
U+517B.0	fGB	0-5178
U+517B.0	fR/S	 12.7
U+517B.0	fTotalStrokes	 9
U+517B.0	fCangjie	TOLL
U+517B.0	fHakka	 jong3
U+517B.0	fCantonese	joeng5; joeng6
U+517B.0	fMandarin	yang3; yang4
U+517B.0	fEnglish	simplified form of &#x990A;
U+517C.0	fUTF8	兼
U+517C.0	fCNS11643	1-5440
U+517C.0	fBig5	ADDD
U+517C.0	fGB	0-3C66
U+517C.0	fR/S	 12.8
U+517C.0	fTotalStrokes	 10
U+517C.0	fCangjie	TXC
U+517C.0	fFourCorner	80237
U+517C.0	fHakka	  giam1
U+517C.0	fCantonese	gim1
U+517C.0	fMandarin	jian1
U+517C.0	fEnglish	[1] unite (in one); combine; connect; annex [2] and; also; together with; both; equally; concurrently
U+517D.0	fUTF8	兽
U+517D.0	fGB	0-4A5E
U+517D.0	fR/S	 12.9
U+517D.0	fTotalStrokes	 11
U+517D.0	fCangjie	CWMR
U+517D.0	fHakka	 chiu5
U+517D.0	fCantonese	sau3
U+517D.0	fMandarin	shou4
U+517D.0	fEnglish	simplified form of &#x7378;
U+517E.0	fUTF8	兾
U+517E.0	fCNS11643	3-403F
U+517E.0	fGB	E-2265
U+517E.0	fR/S	 12.11
U+517E.0	fTotalStrokes	 13
U+517E.0	fCangjie	CWTC
U+517E.0	fFourCorner	80800
U+517E.0	fCantonese	kei3
U+517E.0	fMandarin	ji4
U+517E.0	fEnglish	same as &#x5180;
U+517F.0	fUTF8	兿
U+517F.0	fCNS11643	3-6453
U+517F.0	fGB	E-2266
U+517F.0	fR/S	 12.11; 28.11
U+517F.0	fTotalStrokes	 13
U+517F.0	fCangjie	TQII
U+517F.0	fCantonese	ngai6; ai6 (also)
U+517F.0	fMandarin	yi4
U+5180.0	fUTF8	冀
U+5180.0	fCNS11643	1-7035
U+5180.0	fBig5	BEAC
U+5180.0	fGB	0-3C3D
U+5180.0	fR/S	 12.14
U+5180.0	fTotalStrokes	 16
U+5180.0	fCangjie	LPWTC
U+5180.0	fFourCorner	11801
U+5180.0	fHakka	 gi3; gi1; gi5; ki3
U+5180.0	fCantonese	kei3; gei3
U+5180.0	fMandarin	ji4
U+5180.0	fEnglish	[1] hope (for); wish [2] Hebei (Hopeh) province [3] a Chinese family name
U+5181.0	fUTF8	冁
U+5181.0	fGB	0-5966
U+5181.0	fR/S	 12.16
U+5181.0	fTotalStrokes	 18
U+5181.0	fCangjie	CJSTV
U+5181.0	fHakka	 zhen3
U+5181.0	fCantonese	cin2; zin2 (also)
U+5181.0	fMandarin	chan3; che4
U+5181.0	fEnglish	simplified form of &#x56C5;
U+5182.0	fUTF8	冂
U+5182.0	fCNS11643	3-2127
U+5182.0	fGB	0-5867
U+5182.0	fR/S	 13.0
U+5182.0	fTotalStrokes	 2
U+5182.0	fCangjie	LS
U+5182.0	fFourCorner	77220
U+5182.0	fHakka	 kwen3; kwen1; gen1; gen3; gwen3
U+5182.0	fCantonese	gwing1; gwing2
U+5182.0	fMandarin	jiong1
U+5182.0	fEnglish	 wide Kangxi radical number 13
U+5183.0	fUTF8	冃
U+5183.0	fCNS11643	4-213E
U+5183.0	fGB	E-2267
U+5183.0	fR/S	  13.2
U+5183.0	fTotalStrokes	 4
U+5183.0	fCangjie	BMM
U+5183.0	fFourCorner	77220
U+5183.0	fCantonese	mou6
U+5183.0	fMandarin	mao4
U+5184.0	fUTF8	冄
U+5184.0	fCNS11643	3-214D
U+5184.0	fGB	E-2268
U+5184.0	fR/S	  13.2
U+5184.0	fTotalStrokes	 4
U+5184.0	fCangjie	BMM
U+5184.0	fFourCorner	77550
U+5184.0	fHakka	  ngiam3
U+5184.0	fCantonese	jim5
U+5184.0	fMandarin	ran3
U+5184.0	fEnglish	 tender weak gradually alterrrrating
U+5185.0	fUTF8	内
U+5185.0	fCNS11643	3-2148
U+5185.0	fGB	0-445A
U+5185.0	fR/S	 13.2; 11.2
U+5185.0	fTotalStrokes	 4
U+5185.0	fCangjie	OB
U+5185.0	fFourCorner	40220
U+5185.0	fHakka	 lui5; nui6; lui3
U+5185.0	fCantonese	noi6; loi6 (also); naap6; laap6 (also)
U+5185.0	fMandarin	nei4; na4
U+5185.0	fEnglish	inside
U+5186.0	fUTF8	円
U+5186.0	fCNS11643	3-214E
U+5186.0	fGB	E-2269
U+5186.0	fR/S	 13.2
U+5186.0	fTotalStrokes	 4
U+5186.0	fCangjie	BLM
U+5186.0	fFourCorner	77220
U+5186.0	fCantonese	jyun4
U+5186.0	fMandarin	yuan2
U+5186.0	fEnglish	yen
U+5187.0	fUTF8	冇
U+5187.0	fCNS11643	2-212E
U+5187.0	fBig5	C94E
U+5187.0	fGB	3-3056
U+5187.0	fR/S	 1.3; 13.2
U+5187.0	fTotalStrokes	 4
U+5187.0	fCangjie	KB
U+5187.0	fFourCorner	40227
U+5187.0	fHakka	  mau2; pang5; bang5; mo2
U+5187.0	fCantonese	mou5; mo5
U+5187.0	fMandarin	mao3; mou3
U+5187.0	fEnglish	 have not; did not
U+5187.1	fUTF8	冇
U+5187.1	fCNS11643	2-212E
U+5187.1	fBig5	C94E
U+5187.1	fGB	3-3056
U+5187.1	fR/S	 1.3; 13.2
U+5187.1	fTotalStrokes	 4
U+5187.1	fCangjie	KB
U+5187.1	fFourCorner	40227
U+5187.1	fMandarin	peng4
U+5187.1	fEnglish	[1] empty; hollow [2] decayed; rotten
U+5188.0	fUTF8	冈
U+5188.0	fCNS11643	4-2152
U+5188.0	fGB	0-3854
U+5188.0	fR/S	 13.2
U+5188.0	fTotalStrokes	 4
U+5188.0	fCangjie	BK
U+5188.0	fFourCorner	77220
U+5188.0	fHakka	 gong1
U+5188.0	fCantonese	gong1
U+5188.0	fMandarin	gang1; gang3
U+5188.0	fEnglish	simplified form of &#x5CA1;
U+5189.0	fUTF8	冉
U+5189.0	fCNS11643	1-4574
U+5189.0	fBig5	A554
U+5189.0	fGB	0-483D
U+5189.0	fR/S	 13.3
U+5189.0	fTotalStrokes	 5
U+5189.0	fCangjie	GB
U+5189.0	fFourCorner	50557
U+5189.0	fHakka	 ngiam3;  ngiam1; jan3
U+5189.0	fCantonese	jim5; jim2 (also)
U+5189.0	fMandarin	ran3; zhu3
U+5189.0	fEnglish	[1] tender; weak [2] gradually [3] outer edge of a turtle's shell [4] a Chinese family name
U+518A.0	fUTF8	冊
U+518A.0	fCNS11643	1-4575
U+518A.0	fBig5	A555
U+518A.0	fGB	E-226A
U+518A.0	fR/S	 13.3
U+518A.0	fTotalStrokes	 5
U+518A.0	fCangjie	BT
U+518A.0	fFourCorner	77440
U+518A.0	fHakka	 cak7
U+518A.0	fCantonese	caak3
U+518A.0	fMandarin	ce4; chai3
U+518A.0	fEnglish	book; volume
U+518B.0	fUTF8	冋
U+518B.0	fCNS11643	3-2178
U+518B.0	fGB	E-226B
U+518B.0	fR/S	 13.3
U+518B.0	fTotalStrokes	 5
U+518B.0	fCangjie	BR
U+518B.0	fFourCorner	77220
U+518B.0	fHakka	 kwen3
U+518B.0	fCantonese	gwing1; gwing2
U+518B.0	fMandarin	jiong1
U+518B.0	fEnglish	a desert a border prairie the 1st form is Radical 13
U+518C.0	fUTF8	册
U+518C.0	fCNS11643	3-2179
U+518C.0	fGB	0-3261
U+518C.0	fR/S	 13.3
U+518C.0	fTotalStrokes	 5
U+518C.0	fCangjie	BBM
U+518C.0	fFourCorner	77440
U+518C.0	fHakka	  cak7
U+518C.0	fCantonese	caak3
U+518C.0	fMandarin	ce4
U+518C.0	fEnglish	[1] register; book or books in general; volumes [2] list; statistical tables; to record; records [3] an order to confer nobility titles
U+518D.0	fUTF8	再
U+518D.0	fCNS11643	1-4742
U+518D.0	fBig5	A641
U+518D.0	fGB	0-5459
U+518D.0	fR/S	 13.4
U+518D.0	fTotalStrokes	 6
U+518D.0	fCangjie	MGB
U+518D.0	fFourCorner	10447
U+518D.0	fHakka	  zai5
U+518D.0	fCantonese	zoi3
U+518D.0	fMandarin	zai4; ji4
U+518D.0	fEnglish	[1] again; repeated; re- [2] still; further; then
U+518E.0	fUTF8	冎
U+518E.0	fCNS11643	4-2236
U+518E.0	fGB	5-3133
U+518E.0	fR/S	  13.4
U+518E.0	fTotalStrokes	 6
U+518E.0	fCangjie	BNB
U+518E.0	fFourCorner	77220
U+518E.0	fCantonese	gwaa2
U+518E.0	fMandarin	gua3
U+518F.0	fUTF8	冏
U+518F.0	fCNS11643	2-232C
U+518F.0	fBig5	CA6A
U+518F.0	fGB	3-3057
U+518F.0	fR/S	 13.5
U+518F.0	fTotalStrokes	 7
U+518F.0	fCangjie	BCR
U+518F.0	fFourCorner	77220
U+518F.0	fHakka	 kwen3
U+518F.0	fCantonese	gwing2
U+518F.0	fMandarin	jiong3
U+518F.0	fEnglish	bright light
U+5190.0	fUTF8	冐
U+5190.0	fCNS11643	3-2755
U+5190.0	fGB	E-226C
U+5190.0	fR/S	 13.6
U+5190.0	fTotalStrokes	 8
U+5190.0	fCangjie	AB
U+5190.0	fFourCorner	60220
U+5190.0	fCantonese	mou6; mak6
U+5190.0	fMandarin	mao4; mo4
U+5190.0	fEnglish	[1] incautious; imprudent; rash [2] risk; brave; be exposed to (hardships) [3] put forth; issue forth; go up (as fire, smoke, etc.) [4] a Chinese family name
U+5191.0	fUTF8	冑
U+5191.0	fCNS11643	1-5024
U+5191.0	fBig5	AB60
U+5191.0	fGB	E-226D
U+5191.0	fR/S	 13.7
U+5191.0	fTotalStrokes	 9
U+5191.0	fCangjie	LWB
U+5191.0	fFourCorner	50227
U+5191.0	fHakka	 ciu3; ciu5; ciu6; diu3
U+5191.0	fCantonese	zau6
U+5191.0	fMandarin	zhou4
U+5191.0	fEnglish	helmet
U+5192.0	fUTF8	冒
U+5192.0	fCNS11643	1-5023
U+5192.0	fBig5	AB5F
U+5192.0	fGB	0-4330
U+5192.0	fR/S	 13.7
U+5192.0	fTotalStrokes	 9
U+5192.0	fCangjie	ABU
U+5192.0	fFourCorner	60600
U+5192.0	fHakka	  mau5; mo6; mau3; mo5; mau6; met8
U+5192.0	fCantonese	mou6; mak6
U+5192.0	fMandarin	mao4; mo4; mou4; mei4
U+5192.0	fEnglish	risk; brave; dare
U+5193.0	fUTF8	冓
U+5193.0	fCNS11643	2-2E23
U+5193.0	fBig5	D0E0
U+5193.0	fGB	3-3025
U+5193.0	fR/S	 13.8
U+5193.0	fTotalStrokes	 10
U+5193.0	fCangjie	TTGB
U+5193.0	fFourCorner	55557
U+5193.0	fHakka	 kieu1; kieu5; geu5
U+5193.0	fCantonese	gau3 (also); kau3; kau1; gau1 (also)
U+5193.0	fMandarin	gou4; gou1
U+5193.0	fEnglish	 secluded place in a palace; secret cabinet
U+5194.0	fUTF8	冔
U+5194.0	fCNS11643	2-2E22
U+5194.0	fBig5	D0DF
U+5194.0	fGB	3-483B
U+5194.0	fR/S	 13.8
U+5194.0	fTotalStrokes	 10
U+5194.0	fCangjie	ARMD
U+5194.0	fFourCorner	60641
U+5194.0	fHakka	 hi1;  hi3
U+5194.0	fCantonese	heoi2; hei1
U+5194.0	fMandarin	xu3; xu1
U+5194.0	fEnglish	cap worn during the Yin dynasty
U+5195.0	fUTF8	冕
U+5195.0	fCNS11643	1-5928
U+5195.0	fBig5	B0C3
U+5195.0	fGB	0-4361
U+5195.0	fR/S	 13.9
U+5195.0	fTotalStrokes	 11
U+5195.0	fCangjie	ANAU
U+5195.0	fFourCorner	60416
U+5195.0	fHakka	 mien3; mien1;  men1; men3
U+5195.0	fCantonese	min5
U+5195.0	fMandarin	mian3; man1; wan2
U+5195.0	fEnglish	[1] crown [2] ceremonial cap for high ministers in ancient China
U+5196.0	fUTF8	冖
U+5196.0	fCNS11643	3-2128
U+5196.0	fGB	0-5A22
U+5196.0	fR/S	 14.0
U+5196.0	fTotalStrokes	 2
U+5196.0	fCangjie	IN
U+5196.0	fFourCorner	37000
U+5196.0	fHakka	 met8; met7; mit8
U+5196.0	fCantonese	mik6
U+5196.0	fMandarin	mi4
U+5196.0	fEnglish	 cover Kangxi radical number 14
U+5197.0	fUTF8	冗
U+5197.0	fCNS11643	1-447D
U+5197.0	fBig5	A4BE
U+5197.0	fGB	0-485F
U+5197.0	fR/S	 14.2
U+5197.0	fTotalStrokes	 4
U+5197.0	fCangjie	BHN
U+5197.0	fFourCorner	37217
U+5197.0	fHakka	  jung3; rung3; cin3
U+5197.0	fCantonese	jung2; jung5
U+5197.0	fMandarin	rong3
U+5197.0	fEnglish	 excessive superfluous
U+5198.0	fUTF8	冘
U+5198.0	fCNS11643	2-2135
U+5198.0	fBig5	C955
U+5198.0	fGB	3-3265
U+5198.0	fR/S	 14.2
U+5198.0	fTotalStrokes	 4
U+5198.0	fCangjie	LBU
U+5198.0	fFourCorner	40212
U+5198.0	fHakka	 ju2; riu2; jiu3; rim2; jim2
U+5198.0	fCantonese	jam4; jau4
U+5198.0	fMandarin	yin2; you2
U+5198.0	fEnglish	 move on coubtful
U+5199.0	fUTF8	写
U+5199.0	fCNS11643	3-217A
U+5199.0	fGB	0-5034
U+5199.0	fR/S	 14.3
U+5199.0	fTotalStrokes	 5
U+5199.0	fCangjie	BYSM
U+5199.0	fFourCorner	37400
U+5199.0	fHakka	 sia3
U+5199.0	fCantonese	se2; se3
U+5199.0	fMandarin	xie3
U+5199.0	fEnglish	simplified form of &#x5BEB;
U+519A.0	fUTF8	冚
U+519A.0	fGB	1-7C22
U+519A.0	fR/S	 14.3
U+519A.0	fTotalStrokes	 5
U+519A.0	fCangjie	BU
U+519A.0	fHakka	 ham3
U+519A.0	fCantonese	kam2; ham6; gam2
U+519A.0	fMandarin	kan3
U+519A.0	fEnglish	[1] cover; lid [2] standard form of &#x205AB;
U+519B.0	fUTF8	军
U+519B.0	fGB	0-3E7C
U+519B.0	fR/S	 159.2; 14.4
U+519B.0	fTotalStrokes	 6
U+519B.0	fCangjie	BKQ
U+519B.0	fHakka	 giun1
U+519B.0	fCantonese	gwan1
U+519B.0	fMandarin	jun1
U+519B.0	fEnglish	simplified form of &#x8ECD;
U+519C.0	fUTF8	农
U+519C.0	fGB	0-4529
U+519C.0	fR/S	 14.4
U+519C.0	fTotalStrokes	 6
U+519C.0	fCangjie	HBV
U+519C.0	fHakka	 nung2
U+519C.0	fCantonese	nung4; lung4 (also)
U+519C.0	fMandarin	nong2
U+519C.0	fEnglish	simplified form of &#x8FB2;
U+519D.0	fUTF8	冝
U+519D.0	fCNS11643	3-2457
U+519D.0	fGB	E-226E
U+519D.0	fR/S	 14.5
U+519D.0	fTotalStrokes	 7
U+519D.0	fCangjie	BBM
U+519D.0	fFourCorner	37100
U+519D.0	fCantonese	ji4
U+519D.0	fMandarin	yi2
U+519D.0	fEnglish	suitable; right; fitting; proper
U+519E.0	fUTF8	冞
U+519E.0	fCNS11643	2-254D
U+519E.0	fBig5	CBCD
U+519E.0	fGB	3-3266
U+519E.0	fR/S	 14.6
U+519E.0	fTotalStrokes	 8
U+519E.0	fCangjie	BFD
U+519E.0	fFourCorner	77904
U+519E.0	fHakka	  mi2; ni2; ni3
U+519E.0	fCantonese	mei4
U+519E.0	fMandarin	mi2
U+519E.0	fEnglish	far
U+519F.0	fUTF8	冟
U+519F.0	fCNS11643	3-2B39
U+519F.0	fGB	3-3267
U+519F.0	fR/S	 14.7
U+519F.0	fTotalStrokes	 9
U+519F.0	fCangjie	BHAP
U+519F.0	fFourCorner	37711
U+519F.0	fCantonese	sik1
U+519F.0	fMandarin	shi4
U+51A0.0	fUTF8	冠
U+51A0.0	fCNS11643	1-5025
U+51A0.0	fBig5	AB61
U+51A0.0	fGB	0-395A
U+51A0.0	fR/S	 14.7
U+51A0.0	fTotalStrokes	 9
U+51A0.0	fCangjie	BMUI
U+51A0.0	fFourCorner	37214
U+51A0.0	fHakka	  gon5; gon1; gwon1; kwan5; kwan1
U+51A0.0	fCantonese	gun1; gun3; gwun1 (also); gwun3 (also)
U+51A0.0	fMandarin	guan1
U+51A0.0	fEnglish	[1] cap [2] comb or crest of a bird [3] a Chinese family name
U+51A0.1	fUTF8	冠
U+51A0.1	fCNS11643	1-5025
U+51A0.1	fBig5	AB61
U+51A0.1	fGB	0-395A
U+51A0.1	fR/S	 14.7
U+51A0.1	fTotalStrokes	 9
U+51A0.1	fCangjie	BMUI
U+51A0.1	fFourCorner	37214
U+51A0.1	fHakka	 gwon5
U+51A0.1	fMandarin	guan4
U+51A0.1	fEnglish	[1] at 20 when a young man is capped [2] first-rate [3] wear a cap
U+51A1.0	fUTF8	冡
U+51A1.0	fCNS11643	3-2F5B
U+51A1.0	fGB	5-3374
U+51A1.0	fR/S	  14.8
U+51A1.0	fTotalStrokes	 10
U+51A1.0	fCangjie	BMMO
U+51A1.0	fFourCorner	37220
U+51A1.0	fCantonese	mung4
U+51A1.0	fMandarin	meng2
U+51A2.0	fUTF8	冢
U+51A2.0	fCNS11643	1-5443
U+51A2.0	fBig5	ADE0
U+51A2.0	fGB	0-5A23
U+51A2.0	fR/S	 14.8
U+51A2.0	fTotalStrokes	 10
U+51A2.0	fCangjie	XBMSO
U+51A2.0	fFourCorner	37232
U+51A2.0	fHakka	 cung3;  chung3
U+51A2.0	fCantonese	cung2
U+51A2.0	fMandarin	zhong3
U+51A2.0	fEnglish	[1] high grave [2] peak; summit [3] eldest [4] great; supreme; prime (all referring to official ranks)
U+51A3.0	fUTF8	冣
U+51A3.0	fCNS11643	3-2F59
U+51A3.0	fGB	E-226F
U+51A3.0	fR/S	 14.8
U+51A3.0	fTotalStrokes	 10
U+51A3.0	fCangjie	BSJE
U+51A3.0	fFourCorner	37140
U+51A3.0	fHakka	  ci5
U+51A3.0	fCantonese	zeoi3
U+51A3.0	fMandarin	ju4
U+51A3.0	fEnglish	 collect; assemble
U+51A4.0	fUTF8	冤
U+51A4.0	fCNS11643	1-5441
U+51A4.0	fBig5	ADDE
U+51A4.0	fGB	0-5429
U+51A4.0	fR/S	 14.8
U+51A4.0	fTotalStrokes	 10
U+51A4.0	fCangjie	BNAI
U+51A4.0	fFourCorner	37413
U+51A4.0	fHakka	 jen1; zan1; jan1
U+51A4.0	fCantonese	jyun1
U+51A4.0	fMandarin	yuan1
U+51A4.0	fEnglish	[1] oppression; injustice; grievance; wrong [2] feud; animosity; enmity [3] cheat; lie [4] spend money recklessly; waste money [5] make false accusations
U+51A5.0	fUTF8	冥
U+51A5.0	fCNS11643	1-5442
U+51A5.0	fBig5	ADDF
U+51A5.0	fGB	0-5A24
U+51A5.0	fR/S	 14.8
U+51A5.0	fTotalStrokes	 10
U+51A5.0	fCangjie	BAYC
U+51A5.0	fFourCorner	37800
U+51A5.0	fHakka	 men2; men3
U+51A5.0	fCantonese	ming4; ming5 (also)
U+51A5.0	fMandarin	ming2; meng3; mi4; mian2; ming4
U+51A5.0	fEnglish	[1] dark; gloomy; obscure; dim; dusk [2] stupidity; stupid [3] far and high [4] deep; profound [5] unseen world [6] night [7] a Chinese family name
U+51A6.0	fUTF8	冦
U+51A6.0	fCNS11643	3-2F5A
U+51A6.0	fGB	E-2270
U+51A6.0	fR/S	 14.8
U+51A6.0	fTotalStrokes	 10
U+51A6.0	fCangjie	BMUE
U+51A6.0	fFourCorner	37210
U+51A6.0	fCantonese	kau3
U+51A6.0	fMandarin	kou4
U+51A6.0	fEnglish	 bandits; thieves enemy invade
U+51A7.0	fUTF8	冧
U+51A7.0	fGB	1-7C23
U+51A7.0	fR/S	 14.8
U+51A7.0	fTotalStrokes	 10
U+51A7.0	fCangjie	BDD
U+51A7.0	fCantonese	lam1; lam3; nam1
U+51A7.0	fEnglish	[1] phonetic "num" as in "number" [2] collapse [3] pile up; repeat
U+51A8.0	fUTF8	冨
U+51A8.0	fCNS11643	3-344D
U+51A8.0	fGB	5-3375
U+51A8.0	fR/S	 14.9; 14.6; 102.6
U+51A8.0	fTotalStrokes	 11
U+51A8.0	fCangjie	BMRW
U+51A8.0	fFourCorner	77600
U+51A8.0	fCantonese	fu3
U+51A8.0	fMandarin	fu4
U+51A8.0	fEnglish	 abundant; ample rich; wealthy
U+51A9.0	fUTF8	冩
U+51A9.0	fCNS11643	F-465C
U+51A9.0	fGB	E-2271
U+51A9.0	fR/S	 14.12
U+51A9.0	fTotalStrokes	 14
U+51A9.0	fCangjie	BHXF
U+51A9.0	fHakka	 sia3
U+51A9.0	fCantonese	se2
U+51A9.0	fMandarin	xie3
U+51A9.0	fEnglish	 write draw; sketch compose
U+51AA.0	fUTF8	冪
U+51AA.0	fCNS11643	1-7036
U+51AA.0	fBig5	BEAD
U+51AA.0	fGB	E-2272
U+51AA.0	fR/S	 14.14
U+51AA.0	fTotalStrokes	 16
U+51AA.0	fCangjie	BTAB
U+51AA.0	fFourCorner	37227
U+51AA.0	fHakka	 met7;  mit8
U+51AA.0	fCantonese	mik6
U+51AA.0	fMandarin	mi4
U+51AA.0	fEnglish	[1] cover-cloth; cover with cloth [2] cloth cover; veil [3] power (mathematics)
U+51AB.0	fUTF8	冫
U+51AB.0	fCNS11643	1-272F
U+51AB.0	fGB	0-597B
U+51AB.0	fR/S	 15.0
U+51AB.0	fTotalStrokes	 2
U+51AB.0	fCangjie	IM
U+51AB.0	fFourCorner	3010
U+51AB.0	fHakka	  ben1
U+51AB.0	fCantonese	bing1
U+51AB.0	fMandarin	bing1
U+51AB.0	fEnglish	 ice Kangxi radical number 15
U+51AC.0	fUTF8	冬
U+51AC.0	fCNS11643	1-4576
U+51AC.0	fBig5	A556
U+51AC.0	fGB	0-362C
U+51AC.0	fR/S	 15.3
U+51AC.0	fTotalStrokes	 5
U+51AC.0	fCangjie	HEY
U+51AC.0	fFourCorner	27303
U+51AC.0	fHakka	  dung1
U+51AC.0	fCantonese	dung1
U+51AC.0	fMandarin	dong1
U+51AC.0	fEnglish	[1] winter [2] 11th lunar month; period from the 10th to the 12th month
U+51AD.0	fUTF8	冭
U+51AD.0	fCNS11643	4-215E
U+51AD.0	fGB	E-2273
U+51AD.0	fR/S	  15.3
U+51AD.0	fTotalStrokes	 5
U+51AD.0	fCangjie	KY
U+51AD.0	fFourCorner	40100
U+51AD.0	fCantonese	taai3
U+51AD.0	fMandarin	tai2; tai4
U+51AE.0	fUTF8	冮
U+51AE.0	fGB	3-325B
U+51AE.0	fR/S	 15.3
U+51AE.0	fTotalStrokes	 5
U+51AE.0	fCangjie	IMM
U+51AE.0	fFourCorner	3111
U+51AE.0	fCantonese	gong1
U+51AE.0	fMandarin	gang1
U+51AF.0	fUTF8	冯
U+51AF.0	fGB	0-376B
U+51AF.0	fR/S	 15.3; 187.2
U+51AF.0	fTotalStrokes	 5
U+51AF.0	fCangjie	IMNVM
U+51AF.0	fHakka	 pung2
U+51AF.0	fCantonese	fung4; pang4
U+51AF.0	fMandarin	feng2; peng1; ping2; bing4; fen4
U+51AF.0	fEnglish	simplified form of &#x99AE;
U+51B0.0	fUTF8	冰
U+51B0.0	fCNS11643	1-4743
U+51B0.0	fBig5	A642
U+51B0.0	fGB	0-3179
U+51B0.0	fR/S	 15.4
U+51B0.0	fTotalStrokes	 6
U+51B0.0	fCangjie	IME
U+51B0.0	fFourCorner	32130
U+51B0.0	fHakka	  ben1; bien1
U+51B0.0	fCantonese	bing1; jing4
U+51B0.0	fMandarin	bing1; ning2
U+51B0.0	fEnglish	[1] ice; icicles [2] cold; frost
U+51B1.0	fUTF8	冱
U+51B1.0	fCNS11643	2-217B
U+51B1.0	fBig5	C9BC
U+51B1.0	fGB	0-597C
U+51B1.0	fR/S	 15.4
U+51B1.0	fTotalStrokes	 6
U+51B1.0	fCangjie	IMMVM
U+51B1.0	fFourCorner	31117
U+51B1.0	fHakka	 fu3; fu5
U+51B1.0	fCantonese	wu6
U+51B1.0	fMandarin	hu4
U+51B1.0	fEnglish	 freezing stopped up; closed off
U+51B2.0	fUTF8	冲
U+51B2.0	fCNS11643	3-227E
U+51B2.0	fGB	0-3365
U+51B2.0	fR/S	  15.4
U+51B2.0	fTotalStrokes	 6
U+51B2.0	fCangjie	IML
U+51B2.0	fFourCorner	35100
U+51B2.0	fHakka	 cung1;  chung1; zhung1; cung2
U+51B2.0	fCantonese	cung1; cung3
U+51B2.0	fMandarin	chong1; chong3
U+51B2.0	fEnglish	simplified form of &#x885D;
U+51B2.1	fUTF8	冲
U+51B2.1	fCNS11643	3-227E
U+51B2.1	fGB	0-3365
U+51B2.1	fR/S	  15.4
U+51B2.1	fTotalStrokes	 6
U+51B2.1	fCangjie	IML
U+51B2.1	fFourCorner	35100
U+51B2.1	fMandarin	chong4
U+51B3.0	fUTF8	决
U+51B3.0	fCNS11643	3-227D
U+51B3.0	fGB	0-3E76
U+51B3.0	fR/S	  15.4
U+51B3.0	fTotalStrokes	 6
U+51B3.0	fCangjie	IMDK
U+51B3.0	fFourCorner	35130
U+51B3.0	fHakka	  giet7; get7; ket7
U+51B3.0	fCantonese	kyut3; hyut3
U+51B3.0	fMandarin	jue2
U+51B3.0	fEnglish	decide; determine; judge
U+51B4.0	fUTF8	冴
U+51B4.0	fCNS11643	3-227C
U+51B4.0	fGB	E-2274
U+51B4.0	fR/S	 15.4
U+51B4.0	fTotalStrokes	 6
U+51B4.0	fCangjie	IMMVH
U+51B4.0	fFourCorner	31140
U+51B4.0	fCantonese	wu6
U+51B4.0	fMandarin	hu4
U+51B4.0	fEnglish	 freezing stopped up; closed off
U+51B5.0	fUTF8	况
U+51B5.0	fCNS11643	3-2459
U+51B5.0	fGB	0-3F76
U+51B5.0	fR/S	  15.5
U+51B5.0	fTotalStrokes	 7
U+51B5.0	fCangjie	IMRHU
U+51B5.0	fFourCorner	36110
U+51B5.0	fHakka	  kong3;  kong5; kwong3; k'wong3
U+51B5.0	fCantonese	fong3
U+51B5.0	fMandarin	kuang4
U+51B5.0	fEnglish	 condition; situation furthermore
U+51B6.0	fUTF8	冶
U+51B6.0	fCNS11643	1-492F
U+51B6.0	fBig5	A74D
U+51B6.0	fGB	0-5231
U+51B6.0	fR/S	 15.5
U+51B6.0	fTotalStrokes	 7
U+51B6.0	fCangjie	IMIR
U+51B6.0	fFourCorner	33160
U+51B6.0	fHakka	  ja3; ja1; ra3; ra1
U+51B6.0	fCantonese	je5
U+51B6.0	fMandarin	ye3
U+51B6.0	fEnglish	[1] smelt; fuse metals [2] seductive; fascinating
U+51B7.0	fUTF8	冷
U+51B7.0	fCNS11643	1-4930
U+51B7.0	fBig5	A74E
U+51B7.0	fGB	0-4064
U+51B7.0	fR/S	 15.5
U+51B7.0	fTotalStrokes	 7
U+51B7.0	fCangjie	IMOII
U+51B7.0	fFourCorner	38127
U+51B7.0	fHakka	  lang1; lang3; len1
U+51B7.0	fCantonese	laang5; laang1 (also)
U+51B7.0	fMandarin	leng3; ling2
U+51B7.0	fEnglish	[1] cold; cool [2] off-season (said of business, farming, etc.) [3] a Chinese family name
U+51B8.0	fUTF8	冸
U+51B8.0	fCNS11643	3-245A
U+51B8.0	fGB	5-3362
U+51B8.0	fR/S	  15.5
U+51B8.0	fTotalStrokes	 7
U+51B8.0	fCangjie	IMFQ
U+51B8.0	fFourCorner	39150
U+51B8.0	fHakka	 pan3; pan5
U+51B8.0	fCantonese	pun3
U+51B8.0	fMandarin	pan4
U+51B8.0	fEnglish	 melt
U+51B9.0	fUTF8	冹
U+51B9.0	fCNS11643	2-232D
U+51B9.0	fBig5	CA6B
U+51B9.0	fGB	5-3360
U+51B9.0	fR/S	 15.5
U+51B9.0	fTotalStrokes	 7
U+51B9.0	fCangjie	IMIKK
U+51B9.0	fFourCorner	33147
U+51B9.0	fHakka	 fut7
U+51B9.0	fCantonese	fat1; bat1
U+51B9.0	fMandarin	fu2; fu3; fa3
U+51BA.0	fUTF8	冺
U+51BA.0	fCNS11643	4-233D
U+51BA.0	fGB	E-2275
U+51BA.0	fR/S	 15.5
U+51BA.0	fTotalStrokes	 7
U+51BA.0	fCangjie	IMRVP
U+51BA.0	fFourCorner	37140
U+51BA.0	fCantonese	man5
U+51BA.0	fMandarin	min3
U+51BB.0	fUTF8	冻
U+51BB.0	fGB	0-3633
U+51BB.0	fR/S	 15.5
U+51BB.0	fTotalStrokes	 7
U+51BB.0	fCangjie	IMKD
U+51BB.0	fHakka	 dung5
U+51BB.0	fCantonese	dung3; dung2 (also); dung1
U+51BB.0	fMandarin	dong4; dong1
U+51BB.0	fEnglish	simplified form of &#x51CD;
U+51BC.0	fUTF8	冼
U+51BC.0	fCNS11643	2-254E
U+51BC.0	fBig5	CBCE
U+51BC.0	fGB	0-597E
U+51BC.0	fR/S	 15.6
U+51BC.0	fTotalStrokes	 8
U+51BC.0	fCangjie	IMHGU
U+51BC.0	fFourCorner	34111
U+51BC.0	fHakka	 sen3
U+51BC.0	fCantonese	sin2
U+51BC.0	fMandarin	xian3; sheng3
U+51BD.0	fUTF8	冽
U+51BD.0	fCNS11643	1-4C29
U+51BD.0	fBig5	A8E6
U+51BD.0	fGB	0-597D
U+51BD.0	fR/S	 15.6
U+51BD.0	fTotalStrokes	 8
U+51BD.0	fCangjie	IMMNN
U+51BD.0	fFourCorner	32100
U+51BD.0	fHakka	  liet8; liat8; let8
U+51BD.0	fCantonese	lit6
U+51BD.0	fMandarin	lie4; li4
U+51BD.0	fEnglish	pure; clear; crystal-clear (water or wine)
U+51BE.0	fUTF8	冾
U+51BE.0	fCNS11643	2-254F
U+51BE.0	fBig5	CBCF
U+51BE.0	fGB	5-3368
U+51BE.0	fR/S	 15.6
U+51BE.0	fTotalStrokes	 8
U+51BE.0	fCangjie	IMOMR
U+51BE.0	fFourCorner	38161
U+51BE.0	fCantonese	hap1; haap6
U+51BE.0	fMandarin	xia2
U+51BF.0	fUTF8	冿
U+51BF.0	fCNS11643	4-2540
U+51BF.0	fGB	5-336A
U+51BF.0	fR/S	  15.6
U+51BF.0	fTotalStrokes	 8; 7
U+51BF.0	fCangjie	IMLQ
U+51BF.0	fFourCorner	35100
U+51BF.0	fCantonese	zin1
U+51BF.0	fMandarin	jian1
U+51C0.0	fUTF8	净
U+51C0.0	fCNS11643	4-2543
U+51C0.0	fGB	0-3E3B
U+51C0.0	fR/S	 15.6
U+51C0.0	fTotalStrokes	 8; 7
U+51C0.0	fCangjie	IMNSD
U+51C0.0	fFourCorner	37150
U+51C0.0	fHakka	 ciang5
U+51C0.0	fCantonese	zing6; zeng6 (also)
U+51C0.0	fMandarin	jing4
U+51C0.0	fEnglish	simplified form of &#x51C8;
U+51C1.0	fUTF8	凁
U+51C1.0	fCNS11643	3-2B3B
U+51C1.0	fGB	3-325D
U+51C1.0	fR/S	  15.7
U+51C1.0	fTotalStrokes	 9
U+51C1.0	fCangjie	IMDL
U+51C1.0	fFourCorner	35190
U+51C1.0	fCantonese	sau1
U+51C1.0	fMandarin	shu4
U+51C2.0	fUTF8	凂
U+51C2.0	fCNS11643	3-2B3D
U+51C2.0	fGB	E-2276
U+51C2.0	fR/S	 15.7; 15.8
U+51C2.0	fTotalStrokes	 9
U+51C2.0	fCangjie	IMNAU
U+51C2.0	fFourCorner	37110
U+51C2.0	fHakka	 mui3
U+51C2.0	fCantonese	mui5
U+51C2.0	fMandarin	mei3
U+51C2.0	fEnglish	 request ask a favour of
U+51C3.0	fUTF8	凃
U+51C3.0	fCNS11643	3-2B3E
U+51C3.0	fGB	5-336B
U+51C3.0	fR/S	 15.7
U+51C3.0	fTotalStrokes	 9
U+51C3.0	fCangjie	IMOMD
U+51C3.0	fFourCorner	38190
U+51C3.0	fCantonese	tou4
U+51C3.0	fMandarin	tu2
U+51C3.0	fEnglish	 mud a Chinese family name
U+51C4.0	fUTF8	凄
U+51C4.0	fCNS11643	2-2E25
U+51C4.0	fBig5	D0E2
U+51C4.0	fGB	0-4660
U+51C4.0	fR/S	 15.8
U+51C4.0	fTotalStrokes	 10
U+51C4.0	fCangjie	IMJLV
U+51C4.0	fFourCorner	35144
U+51C4.0	fHakka	 ci1; zi1
U+51C4.0	fCantonese	cai1
U+51C4.0	fMandarin	qi1
U+51C4.0	fEnglish	bitter cold; miserable; dreary
U+51C5.0	fUTF8	凅
U+51C5.0	fCNS11643	2-2E26
U+51C5.0	fBig5	D0E3
U+51C5.0	fGB	3-3260
U+51C5.0	fR/S	 15.8
U+51C5.0	fTotalStrokes	 10
U+51C5.0	fCangjie	IMWJR
U+51C5.0	fFourCorner	36100
U+51C5.0	fHakka	 gu5
U+51C5.0	fCantonese	gu3; gwu3 (also)
U+51C5.0	fMandarin	gu4; he2
U+51C5.0	fEnglish	 dried up exhausted; tired dry
U+51C6.0	fUTF8	准
U+51C6.0	fCNS11643	1-5446
U+51C6.0	fBig5	ADE3
U+51C6.0	fGB	0-573C
U+51C6.0	fR/S	 15.8
U+51C6.0	fTotalStrokes	 10
U+51C6.0	fCangjie	IMOG
U+51C6.0	fFourCorner	30114
U+51C6.0	fHakka	  zun3; zhun3
U+51C6.0	fCantonese	zeon2
U+51C6.0	fMandarin	zhun3; shui3; zhuo1; zhuo2
U+51C6.0	fEnglish	[1] approve; allow; permit; grant; authorise [2] in accordance with; accordingly [3] equivalent; equal [4] simplified form of &#x6E96;
U+51C7.0	fUTF8	凇
U+51C7.0	fCNS11643	3-2F5E
U+51C7.0	fGB	0-5A21
U+51C7.0	fR/S	  15.8
U+51C7.0	fTotalStrokes	 10
U+51C7.0	fCangjie	IMDCI
U+51C7.0	fFourCorner	38130
U+51C7.0	fHakka	 sung1; siung1; ciung2
U+51C7.0	fCantonese	sung1
U+51C7.0	fMandarin	song1; song4
U+51C7.0	fEnglish	 dewdrop icicle
U+51C8.0	fUTF8	凈
U+51C8.0	fCNS11643	2-2E27
U+51C8.0	fBig5	D0E4
U+51C8.0	fGB	1-3E3B
U+51C8.0	fR/S	 15.8
U+51C8.0	fTotalStrokes	 10
U+51C8.0	fCangjie	IMBSD
U+51C8.0	fFourCorner	32157
U+51C8.0	fHakka	 ciang5; ciang6; ciang3
U+51C8.0	fCantonese	zing6; zeng6 (also)
U+51C8.0	fMandarin	jing4
U+51C8.0	fEnglish	 clean; pure cleanse
U+51C9.0	fUTF8	凉
U+51C9.0	fCNS11643	3-2F5F
U+51C9.0	fGB	0-4139
U+51C9.0	fR/S	  15.8
U+51C9.0	fTotalStrokes	 10
U+51C9.0	fCangjie	IMYRF
U+51C9.0	fFourCorner	30190
U+51C9.0	fHakka	  liong2; liong3
U+51C9.0	fCantonese	loeng4; loeng6
U+51C9.0	fMandarin	liang2; liang4
U+51C9.0	fEnglish	 cool; cold disheartened
U+51CA.0	fUTF8	凊
U+51CA.0	fCNS11643	2-2E24
U+51CA.0	fBig5	D0E1
U+51CA.0	fGB	3-325F
U+51CA.0	fR/S	 15.8
U+51CA.0	fTotalStrokes	 10
U+51CA.0	fCangjie	IMQMB
U+51CA.0	fFourCorner	35127
U+51CA.0	fHakka	  cin5; cin6
U+51CA.0	fCantonese	zing6; cing3
U+51CA.0	fMandarin	qing4; jing4
U+51CA.0	fEnglish	cold; cool; chilly
U+51CB.0	fUTF8	凋
U+51CB.0	fCNS11643	1-5447
U+51CB.0	fBig5	ADE4
U+51CB.0	fGB	0-3572
U+51CB.0	fR/S	 15.8
U+51CB.0	fTotalStrokes	 10
U+51CB.0	fCangjie	IMBGR
U+51CB.0	fFourCorner	37120
U+51CB.0	fHakka	  diau1
U+51CB.0	fCantonese	diu1
U+51CB.0	fMandarin	diao1
U+51CB.0	fEnglish	[1] withered; faded [2] exhausted; emaciated
U+51CC.0	fUTF8	凌
U+51CC.0	fCNS11643	1-5445
U+51CC.0	fBig5	ADE2
U+51CC.0	fGB	0-4168
U+51CC.0	fR/S	 15.8
U+51CC.0	fTotalStrokes	 10
U+51CC.0	fCangjie	IMGCE
U+51CC.0	fFourCorner	34147
U+51CC.0	fHakka	  lin2; lin3
U+51CC.0	fCantonese	ling4
U+51CC.0	fMandarin	ling2; ling4
U+51CC.0	fEnglish	[1] accumulated ice [2] insult; maltreat; through one's weight around [3] rise; ride; soar [4] traverse [5] a Chinese family name
U+51CD.0	fUTF8	凍
U+51CD.0	fCNS11643	1-5444
U+51CD.0	fBig5	ADE1
U+51CD.0	fGB	1-3633
U+51CD.0	fR/S	 15.8
U+51CD.0	fTotalStrokes	 10
U+51CD.0	fCangjie	IMDW
U+51CD.0	fFourCorner	35196
U+51CD.0	fHakka	  dung5
U+51CD.0	fCantonese	dung3; dung2 (also); dung1
U+51CD.0	fMandarin	dong4; dong1
U+51CD.0	fEnglish	[1] freeze [2] cold; icy [3] a Chinese family name
U+51CE.0	fUTF8	凎
U+51CE.0	fCNS11643	2-2E28
U+51CE.0	fBig5	D0E5
U+51CE.0	fGB	3-3261
U+51CE.0	fR/S	 15.8
U+51CE.0	fTotalStrokes	 10
U+51CE.0	fCangjie	IMC
U+51CE.0	fFourCorner	38119
U+51CE.0	fCantonese	gam3
U+51CE.0	fMandarin	gan4
U+51CF.0	fUTF8	减
U+51CF.0	fCNS11643	3-344F
U+51CF.0	fGB	0-3C75
U+51CF.0	fR/S	  15.9
U+51CF.0	fTotalStrokes	 11
U+51CF.0	fCangjie	IMIHR
U+51CF.0	fFourCorner	33150
U+51CF.0	fHakka	  gam3
U+51CF.0	fCantonese	gaam2; kam2
U+51CF.0	fMandarin	jian3
U+51CF.0	fEnglish	decrease; subtract; diminish
U+51D0.0	fUTF8	凐
U+51D0.0	fCNS11643	2-336B
U+51D0.0	fBig5	D468
U+51D0.0	fGB	5-336F
U+51D0.0	fR/S	 15.9
U+51D0.0	fTotalStrokes	 11
U+51D0.0	fCangjie	IMMWG
U+51D0.0	fFourCorner	31114
U+51D0.0	fCantonese	jan1
U+51D0.0	fMandarin	yin1; yin2
U+51D1.0	fUTF8	凑
U+51D1.0	fCNS11643	3-344E
U+51D1.0	fGB	0-3455
U+51D1.0	fR/S	  15.9
U+51D1.0	fTotalStrokes	 11
U+51D1.0	fCangjie	IMQKK
U+51D1.0	fFourCorner	35130
U+51D1.0	fHakka	 ceu5; ciu5
U+51D1.0	fCantonese	cau3
U+51D1.0	fMandarin	cou4
U+51D1.0	fEnglish	piece together; assemble
U+51D2.0	fUTF8	凒
U+51D2.0	fCNS11643	3-397D
U+51D2.0	fGB	5-3367
U+51D2.0	fR/S	  15.10
U+51D2.0	fTotalStrokes	 12
U+51D2.0	fCangjie	IMUMT
U+51D2.0	fFourCorner	32110
U+51D2.0	fHakka	  ki2
U+51D2.0	fCantonese	ngoi2; oi2 (also)
U+51D2.0	fMandarin	yi2; ai2
U+51D2.0	fEnglish	the appearance of frost and snow
U+51D3.0	fUTF8	凓
U+51D3.0	fCNS11643	3-397A
U+51D3.0	fGB	E-2277
U+51D3.0	fR/S	  15.10
U+51D3.0	fTotalStrokes	 12
U+51D3.0	fCangjie	IMMWD
U+51D3.0	fFourCorner	31190
U+51D3.0	fHakka	  lit8
U+51D3.0	fCantonese	leot6
U+51D3.0	fMandarin	li4
U+51D3.0	fEnglish	 shiver severe cold
U+51D4.0	fUTF8	凔
U+51D4.0	fCNS11643	2-3A45
U+51D4.0	fBig5	D861
U+51D4.0	fGB	3-325C
U+51D4.0	fR/S	 15.10
U+51D4.0	fTotalStrokes	 12
U+51D4.0	fCangjie	IMOIR
U+51D4.0	fFourCorner	38167
U+51D4.0	fHakka	 cong1
U+51D4.0	fCantonese	cong1; cong3
U+51D4.0	fMandarin	cang1; cheng1
U+51D4.0	fEnglish	cold
U+51D5.0	fUTF8	凕
U+51D5.0	fCNS11643	4-363F
U+51D5.0	fGB	3-3262
U+51D5.0	fR/S	  15.10
U+51D5.0	fTotalStrokes	 12
U+51D5.0	fCangjie	IMBAC
U+51D5.0	fFourCorner	37180
U+51D5.0	fCantonese	ming5
U+51D5.0	fMandarin	ming3
U+51D6.0	fUTF8	凖
U+51D6.0	fCNS11643	3-397E
U+51D6.0	fGB	E-2278
U+51D6.0	fR/S	 15.10
U+51D6.0	fTotalStrokes	 12
U+51D6.0	fCangjie	IGJ
U+51D6.0	fFourCorner	30400
U+51D6.0	fHakka	 zhun3
U+51D6.0	fCantonese	zeon2
U+51D6.0	fMandarin	zhun3; zhuo1
U+51D6.0	fEnglish	rule; guideline; standard
U+51D7.0	fUTF8	凗
U+51D7.0	fCNS11643	2-416A
U+51D7.0	fBig5	DCC5
U+51D7.0	fGB	5-3371
U+51D7.0	fR/S	 15.11
U+51D7.0	fTotalStrokes	 13
U+51D7.0	fCangjie	IMUOG
U+51D7.0	fFourCorner	32114
U+51D7.0	fHakka	  cui1
U+51D7.0	fCantonese	ceoi4; ceoi1
U+51D7.0	fMandarin	cui2; cui1
U+51D7.0	fEnglish	deep, clear water
U+51D8.0	fUTF8	凘
U+51D8.0	fCNS11643	2-4927
U+51D8.0	fBig5	E140
U+51D8.0	fGB	3-3263
U+51D8.0	fR/S	 15.12
U+51D8.0	fTotalStrokes	 14
U+51D8.0	fCangjie	IMTCL
U+51D8.0	fFourCorner	32121
U+51D8.0	fHakka	 sii1; se1
U+51D8.0	fCantonese	si1
U+51D8.0	fMandarin	si1; si4
U+51D8.0	fEnglish	 thaw; melt
U+51D9.0	fUTF8	凙
U+51D9.0	fCNS11643	3-4B2F
U+51D9.0	fGB	E-2279
U+51D9.0	fR/S	  15.13
U+51D9.0	fTotalStrokes	 15
U+51D9.0	fCangjie	IMWLJ
U+51D9.0	fFourCorner	36140
U+51D9.0	fHakka	  tok8
U+51D9.0	fCantonese	dok6
U+51D9.0	fMandarin	duo2
U+51D9.0	fEnglish	 icicle
U+51DA.0	fUTF8	凚
U+51DA.0	fCNS11643	3-4B2E
U+51DA.0	fGB	3-3264
U+51DA.0	fR/S	  15.13
U+51DA.0	fTotalStrokes	 15
U+51DA.0	fCangjie	IMDDF
U+51DA.0	fFourCorner	34191
U+51DA.0	fHakka	  gim5
U+51DA.0	fCantonese	gam6
U+51DA.0	fMandarin	jin4
U+51DA.0	fEnglish	 cold chilly
U+51DB.0	fUTF8	凛
U+51DB.0	fCNS11643	3-4B30
U+51DB.0	fGB	0-415D
U+51DB.0	fR/S	 15.13
U+51DB.0	fTotalStrokes	 15
U+51DB.0	fCangjie	IMYWF
U+51DB.0	fFourCorner	30190
U+51DB.0	fHakka	 lim3
U+51DB.0	fCantonese	lam5
U+51DB.0	fMandarin	lin3
U+51DB.0	fEnglish	to shiver with cold or fear
U+51DC.0	fUTF8	凜
U+51DC.0	fCNS11643	1-6C29
U+51DC.0	fBig5	BBFE
U+51DC.0	fGB	E-227A
U+51DC.0	fR/S	 15.13
U+51DC.0	fTotalStrokes	 15
U+51DC.0	fCangjie	IMYWD
U+51DC.0	fFourCorner	30191
U+51DC.0	fHakka	 lim3
U+51DC.0	fCantonese	lam5
U+51DC.0	fMandarin	lin3
U+51DC.0	fEnglish	[1] cold; bleak [2] be apprehensive; shiver with cold or fear [3] imposing; awe-inspiring
U+51DD.0	fUTF8	凝
U+51DD.0	fCNS11643	1-7037
U+51DD.0	fBig5	BEAE
U+51DD.0	fGB	0-447D
U+51DD.0	fR/S	 15.14
U+51DD.0	fTotalStrokes	 16
U+51DD.0	fCangjie	IMPKO
U+51DD.0	fFourCorner	37181
U+51DD.0	fHakka	  ngien2; kien2; ken2; gnian3; ngi2; gen5; ngen2; ken5; ngian2
U+51DD.0	fCantonese	jing4; king4 (also)
U+51DD.0	fMandarin	ning2; ning4
U+51DD.0	fEnglish	[1] coagulate; congeal [2] freeze [3] form; take shape [4] concentrate; cohere
U+51DE.0	fUTF8	凞
U+51DE.0	fCNS11643	2-5644
U+51DE.0	fBig5	E8F9
U+51DE.0	fGB	E-227B
U+51DE.0	fR/S	 15.14
U+51DE.0	fTotalStrokes	 16
U+51DE.0	fCangjie	IMLSF
U+51DE.0	fFourCorner	37331
U+51DE.0	fCantonese	hei1
U+51DE.0	fMandarin	xi1
U+51DE.0	fEnglish	bright; splendid; glorious
U+51DF.0	fUTF8	凟
U+51DF.0	fCNS11643	3-5442
U+51DF.0	fGB	5-336C
U+51DF.0	fR/S	  15.15
U+51DF.0	fTotalStrokes	 17
U+51DF.0	fCangjie	IMGWC
U+51DF.0	fFourCorner	34180
U+51DF.0	fHakka	  tuk8
U+51DF.0	fCantonese	duk6
U+51DF.0	fMandarin	du2
U+51DF.0	fEnglish	 be rude to; annoy; profane trouble; harass
U+51E0.0	fUTF8	几
U+51E0.0	fCNS11643	1-442D
U+51E0.0	fBig5	A44C
U+51E0.0	fGB	0-3C38
U+51E0.0	fR/S	 16.0
U+51E0.0	fTotalStrokes	 2
U+51E0.0	fCangjie	HN
U+51E0.0	fFourCorner	77210
U+51E0.0	fHakka	 gi1; gi3; ri3; ji3
U+51E0.0	fCantonese	gei1; gei2
U+51E0.0	fMandarin	ji3; ji4; qi2
U+51E0.0	fEnglish	[1] small table [2] simplified form of &#x5E7E;
U+51E0.1	fUTF8	几
U+51E0.1	fCNS11643	1-442D
U+51E0.1	fBig5	A44C
U+51E0.1	fGB	0-3C38
U+51E0.1	fR/S	 16.0
U+51E0.1	fTotalStrokes	 2
U+51E0.1	fCangjie	HN
U+51E0.1	fFourCorner	77210
U+51E0.1	fMandarin	ji1
U+51E1.0	fUTF8	凡
U+51E1.0	fCNS11643	1-443B
U+51E1.0	fBig5	A45A
U+51E1.0	fGB	0-3732
U+51E1.0	fR/S	 16.1; 3.2
U+51E1.0	fTotalStrokes	 3
U+51E1.0	fCangjie	HNI
U+51E1.0	fFourCorner	77210
U+51E1.0	fHakka	  fam2; fam1; fam3
U+51E1.0	fCantonese	faan4
U+51E1.0	fMandarin	fan2
U+51E1.0	fEnglish	[1] ordinary; common; dull [2] wordily; mortal; earthly [3] generally; every; whenever; wherever [4] altogether
U+51E2.0	fUTF8	凢
U+51E2.0	fCNS11643	3-212F
U+51E2.0	fGB	E-227C
U+51E2.0	fR/S	 16.1; 4.2
U+51E2.0	fTotalStrokes	 3
U+51E2.0	fCangjie	XHHN
U+51E2.0	fFourCorner	30210
U+51E2.0	fCantonese	faan4
U+51E2.0	fMandarin	fan2
U+51E2.0	fEnglish	all; any; every ordinary; common
U+51E3.0	fUTF8	凣
U+51E3.0	fCNS11643	3-2135
U+51E3.0	fGB	E-227D
U+51E3.0	fR/S	 16.1
U+51E3.0	fTotalStrokes	 3
U+51E3.0	fCangjie	LHN
U+51E3.0	fFourCorner	20210
U+51E3.0	fHakka	  fam2
U+51E3.0	fCantonese	faan4
U+51E3.0	fMandarin	fan2
U+51E3.0	fEnglish	 all common; general every whenever generally
U+51E4.0	fUTF8	凤
U+51E4.0	fCNS11643	3-2151
U+51E4.0	fGB	0-376F
U+51E4.0	fR/S	 16.2
U+51E4.0	fTotalStrokes	 4
U+51E4.0	fCangjie	HNE
U+51E4.0	fFourCorner	77210
U+51E4.0	fHakka	 fung5
U+51E4.0	fCantonese	fung6; fung2 (also)
U+51E4.0	fMandarin	feng4
U+51E4.0	fEnglish	simplified form of &#x9CF3;
U+51E5.0	fUTF8	凥
U+51E5.0	fCNS11643	3-217B
U+51E5.0	fGB	5-3E49
U+51E5.0	fR/S	 16.3
U+51E5.0	fTotalStrokes	 5
U+51E5.0	fCangjie	SHN
U+51E5.0	fFourCorner	77210
U+51E5.0	fCantonese	geoi1
U+51E5.0	fMandarin	ju1
U+51E5.0	fEnglish	 dwell
U+51E6.0	fUTF8	処
U+51E6.0	fGB	E-227E
U+51E6.0	fR/S	 16.3; 34.2
U+51E6.0	fTotalStrokes	 5
U+51E6.0	fCangjie	HEHN
U+51E6.0	fFourCorner	2740
U+51E6.0	fCantonese	cyu5; cyu2 (also); cyu3; syu3
U+51E6.0	fMandarin	chu3; chu4; ju4
U+51E6.0	fEnglish	 place; locale department
U+51E7.0	fUTF8	凧
U+51E7.0	fGB	E-2321
U+51E7.0	fR/S	 16.3
U+51E7.0	fTotalStrokes	 5
U+51E7.0	fCangjie	HNLB
U+51E7.0	fFourCorner	7721
U+51E7.0	fCantonese	kei4
U+51E7.0	fEnglish	kite
U+51E8.0	fUTF8	凨
U+51E8.0	fCNS11643	3-2322
U+51E8.0	fGB	5-3348
U+51E8.0	fR/S	 16.4
U+51E8.0	fTotalStrokes	 6
U+51E8.0	fCangjie	HNMMI
U+51E8.0	fFourCorner	77210
U+51E8.0	fCantonese	fung1
U+51E8.0	fMandarin	feng1; feng3; feng4
U+51E8.0	fEnglish	 wind air manners atmosphere
U+51E9.0	fUTF8	凩
U+51E9.0	fGB	E-2322
U+51E9.0	fR/S	 16.4
U+51E9.0	fTotalStrokes	 6
U+51E9.0	fCangjie	XXHND
U+51E9.0	fCantonese	do2
U+51E9.0	fEnglish	 wintry wind
U+51EA.0	fUTF8	凪
U+51EA.0	fCNS11643	F-2259
U+51EA.0	fGB	E-2323
U+51EA.0	fR/S	 16.4
U+51EA.0	fTotalStrokes	 6
U+51EA.0	fCangjie	HNYLM
U+51EA.0	fCantonese	jau1
U+51EA.0	fEnglish	 calm, lull
U+51EB.0	fUTF8	凫
U+51EB.0	fGB	0-596C
U+51EB.0	fR/S	 16.4
U+51EB.0	fTotalStrokes	 6
U+51EB.0	fCangjie	PYSN
U+51EB.0	fCantonese	fu4
U+51EB.0	fMandarin	fu2
U+51EB.0	fEnglish	simplified form of &#x9CEC;
U+51EC.0	fUTF8	凬
U+51EC.0	fCNS11643	3-245B
U+51EC.0	fGB	5-334A
U+51EC.0	fR/S	 182.0; 16.5
U+51EC.0	fTotalStrokes	 7
U+51EC.0	fCangjie	HNMA
U+51EC.0	fFourCorner	77210
U+51EC.0	fCantonese	fung1
U+51EC.0	fMandarin	feng1
U+51ED.0	fUTF8	凭
U+51ED.0	fCNS11643	3-2759
U+51ED.0	fGB	0-463E
U+51ED.0	fR/S	  16.6
U+51ED.0	fTotalStrokes	 8
U+51ED.0	fCangjie	OGHN
U+51ED.0	fFourCorner	22210
U+51ED.0	fHakka	  pin2; ben5; pung2; pen2; pen6; pen5
U+51ED.0	fCantonese	pang4; peng1 (also); bang6 (also)
U+51ED.0	fMandarin	ping2
U+51ED.0	fEnglish	simplified form of &#x6191;
U+51EE.0	fUTF8	凮
U+51EE.0	fCNS11643	4-2545
U+51EE.0	fGB	E-2324
U+51EE.0	fR/S	 16.6
U+51EE.0	fTotalStrokes	 8; 7
U+51EE.0	fCangjie	HNMA
U+51EE.0	fFourCorner	77210
U+51EE.0	fCantonese	fung1
U+51EE.0	fMandarin	feng1
U+51EE.0	fEnglish	wind
U+51EF.0	fUTF8	凯
U+51EF.0	fGB	0-3F2D
U+51EF.0	fR/S	 16.6
U+51EF.0	fTotalStrokes	 8
U+51EF.0	fCangjie	UUHN
U+51EF.0	fHakka	 kai3; koi3
U+51EF.0	fCantonese	hoi2; ngoi2 (also); oi2 (also)
U+51EF.0	fMandarin	kai3
U+51EF.0	fEnglish	simplified form of &#x51F1;
U+51F0.0	fUTF8	凰
U+51F0.0	fCNS11643	1-5929
U+51F0.0	fBig5	B0C4
U+51F0.0	fGB	0-3B4B
U+51F0.0	fR/S	 16.9
U+51F0.0	fTotalStrokes	 11
U+51F0.0	fCangjie	HNHAG
U+51F0.0	fFourCorner	77210
U+51F0.0	fHakka	 fong2; fong3
U+51F0.0	fCantonese	wong4; wong2 (also)
U+51F0.0	fMandarin	huang2
U+51F0.0	fEnglish	 female phoenix; a legendary bird in Chinese mythology
U+51F1.0	fUTF8	凱
U+51F1.0	fCNS11643	1-5E33
U+51F1.0	fBig5	B3CD
U+51F1.0	fGB	1-3F2D
U+51F1.0	fR/S	 16.10
U+51F1.0	fTotalStrokes	 12
U+51F1.0	fCangjie	UTHN
U+51F1.0	fFourCorner	27110
U+51F1.0	fHakka	 kai3; koi3
U+51F1.0	fCantonese	hoi2; ngoi2 (also); oi2 (also)
U+51F1.0	fMandarin	kai3
U+51F1.0	fEnglish	 triumphant triumph; victory
U+51F2.0	fUTF8	凲
U+51F2.0	fCNS11643	6-4665
U+51F2.0	fGB	E-2325
U+51F2.0	fR/S	 16.10
U+51F2.0	fTotalStrokes	 12
U+51F2.0	fCangjie	HNTXC
U+51F2.0	fFourCorner	77210
U+51F2.0	fCantonese	gaam1
U+51F2.0	fMandarin	gan1
U+51F3.0	fUTF8	凳
U+51F3.0	fCNS11643	1-6821
U+51F3.0	fBig5	B9B9
U+51F3.0	fGB	0-354A
U+51F3.0	fR/S	 16.12
U+51F3.0	fTotalStrokes	 14
U+51F3.0	fCangjie	NTHN
U+51F3.0	fFourCorner	12217
U+51F3.0	fHakka	  den5; dien5
U+51F3.0	fCantonese	dang3; dang2 (also)
U+51F3.0	fMandarin	deng4
U+51F3.0	fEnglish	 bench; stool
U+51F4.0	fUTF8	凴
U+51F4.0	fCNS11643	3-457A
U+51F4.0	fGB	E-2326
U+51F4.0	fR/S	  16.12
U+51F4.0	fTotalStrokes	 14
U+51F4.0	fCangjie	IFHN
U+51F4.0	fFourCorner	31210
U+51F4.0	fHakka	 pung2; pin2; ben5
U+51F4.0	fCantonese	pang4; bang6 (also)
U+51F4.0	fMandarin	ping2
U+51F4.0	fEnglish	 rely on; depend on evidence; proof
U+51F5.0	fUTF8	凵
U+51F5.0	fCNS11643	2-2123
U+51F5.0	fBig5	C942
U+51F5.0	fGB	0-5B49
U+51F5.0	fR/S	 17.0
U+51F5.0	fTotalStrokes	 2
U+51F5.0	fCangjie	VL
U+51F5.0	fFourCorner	22770
U+51F5.0	fHakka	 kam5; kam3
U+51F5.0	fCantonese	ham3; ham2; jyu1; ham1; hom1
U+51F5.0	fMandarin	qu1; kan3; kan4
U+51F5.0	fEnglish	[1] receptacle [2] open mouth [3] cough [4] Kangxi radical 17
U+51F6.0	fUTF8	凶
U+51F6.0	fCNS11643	1-447E
U+51F6.0	fBig5	A4BF
U+51F6.0	fGB	0-5057
U+51F6.0	fR/S	 17.2
U+51F6.0	fTotalStrokes	 4
U+51F6.0	fCangjie	UK
U+51F6.0	fFourCorner	22770
U+51F6.0	fHakka	  hiung1
U+51F6.0	fCantonese	hung1
U+51F6.0	fMandarin	xiong1; xiong3
U+51F6.0	fEnglish	[1] evil; bad [2] famine [3] unlucky; unfortunate [4] fear; fearsome [5] very; excessive; excess
U+51F7.0	fUTF8	凷
U+51F7.0	fCNS11643	4-2160
U+51F7.0	fGB	5-3377
U+51F7.0	fR/S	  17.3
U+51F7.0	fTotalStrokes	 5
U+51F7.0	fCangjie	UG
U+51F7.0	fFourCorner	22770
U+51F7.0	fCantonese	faai3
U+51F7.0	fMandarin	kuai4
U+51F7.0	fEnglish	 dirt clod piece
U+51F8.0	fUTF8	凸
U+51F8.0	fCNS11643	1-4579
U+51F8.0	fBig5	A559
U+51F8.0	fGB	0-4D39
U+51F8.0	fR/S	 17.3
U+51F8.0	fTotalStrokes	 5
U+51F8.0	fCangjie	BSS
U+51F8.0	fFourCorner	77777
U+51F8.0	fHakka	 tut7; tiet8; tut8; dung3
U+51F8.0	fCantonese	dat6; gu2
U+51F8.0	fMandarin	tu1; die2; gu3
U+51F8.0	fEnglish	[1] protrude; bulge out; convex [2] protrude; jut
U+51F9.0	fUTF8	凹
U+51F9.0	fCNS11643	1-4577
U+51F9.0	fBig5	A557
U+51F9.0	fGB	0-303C
U+51F9.0	fR/S	 17.3
U+51F9.0	fTotalStrokes	 5
U+51F9.0	fCangjie	SSU
U+51F9.0	fFourCorner	77770
U+51F9.0	fHakka	  au5;  au1; ngiap7; jap7; rap7
U+51F9.0	fCantonese	aau3; aau1 (also); ngaau1 (also); nap1; ngaau3 (also); waa1; lap1 (also)
U+51F9.0	fMandarin	ao1; wa1; wa4; yao1
U+51F9.0	fEnglish	[1] concave; hollow [2] indented; depressed; indention
U+51FA.0	fUTF8	出
U+51FA.0	fCNS11643	1-4578
U+51FA.0	fBig5	A558
U+51FA.0	fGB	0-3376
U+51FA.0	fR/S	 17.3
U+51FA.0	fTotalStrokes	 5
U+51FA.0	fCangjie	UU
U+51FA.0	fFourCorner	22772
U+51FA.0	fHakka	  cut7;  ch'ut7; chut7
U+51FA.0	fCantonese	ceot1
U+51FA.0	fMandarin	chu1; chui4
U+51FA.0	fEnglish	[1] go out; come out [2] produce; reproduce
U+51FB.0	fUTF8	击
U+51FB.0	fGB	0-3B77
U+51FB.0	fR/S	 17.3
U+51FB.0	fTotalStrokes	 5
U+51FB.0	fCangjie	QU
U+51FB.0	fHakka	 git7
U+51FB.0	fCantonese	gik1
U+51FB.0	fMandarin	ji2; ji1; ji4
U+51FB.0	fEnglish	simplified form of &#x64CA;
U+51FC.0	fUTF8	凼
U+51FC.0	fGB	0-5B4A
U+51FC.0	fR/S	 17.4; 85.2
U+51FC.0	fTotalStrokes	 6
U+51FC.0	fCangjie	UE
U+51FC.0	fCantonese	tam5
U+51FC.0	fMandarin	dang4
U+51FC.0	fEnglish	 ditch pool
U+51FD.0	fUTF8	函
U+51FD.0	fCNS11643	1-4C2A
U+51FD.0	fBig5	A8E7
U+51FD.0	fGB	0-3A2F
U+51FD.0	fR/S	 17.6
U+51FD.0	fTotalStrokes	 8
U+51FD.0	fCangjie	UNE
U+51FD.0	fFourCorner	10772
U+51FD.0	fHakka	  ham2; ham3
U+51FD.0	fCantonese	haam4
U+51FD.0	fMandarin	han2; han1; xian2
U+51FD.0	fEnglish	[1] letter; correspondence [2] armor [3] contain; envelop [4] sheath, as for a sword or knife [5] envelope; case; box
U+51FE.0	fUTF8	凾
U+51FE.0	fCNS11643	3-275A
U+51FE.0	fGB	E-2327
U+51FE.0	fR/S	 17.7
U+51FE.0	fTotalStrokes	 9; 8
U+51FE.0	fCangjie	UNRE
U+51FE.0	fFourCorner	17770
U+51FE.0	fHakka	  ham2
U+51FE.0	fCantonese	haam4
U+51FE.0	fMandarin	han2
U+51FE.0	fEnglish	 correspondence case box
U+51FF.0	fUTF8	凿
U+51FF.0	fGB	0-5464
U+51FF.0	fR/S	 17.10
U+51FF.0	fTotalStrokes	 12
U+51FF.0	fCangjie	TCTU
U+51FF.0	fHakka	 zok8; cok8
U+51FF.0	fCantonese	zok6; zok2 (also)
U+51FF.0	fMandarin	zuo4; zao2; cao4; jue2; zao4; zu2
U+51FF.0	fEnglish	simplified form of &#x947F;
U+5200.0	fUTF8	刀
U+5200.0	fCNS11643	1-442E
U+5200.0	fBig5	A44D
U+5200.0	fGB	0-3536
U+5200.0	fR/S	 18.0
U+5200.0	fTotalStrokes	 2
U+5200.0	fCangjie	SH
U+5200.0	fFourCorner	17220
U+5200.0	fHakka	  dau1; do1
U+5200.0	fCantonese	dou1
U+5200.0	fMandarin	dao1; diao1
U+5200.0	fEnglish	[1] knife; blade; sword [2] knife-shaped coin
U+5201.0	fUTF8	刁
U+5201.0	fCNS11643	1-442F
U+5201.0	fBig5	A44E
U+5201.0	fGB	0-3573
U+5201.0	fR/S	 18.0
U+5201.0	fTotalStrokes	 2
U+5201.0	fCangjie	SM
U+5201.0	fFourCorner	17120
U+5201.0	fHakka	  diau1
U+5201.0	fCantonese	diu1
U+5201.0	fMandarin	diao1
U+5201.0	fEnglish	[1] low and cunning; tricky; sly; crafty; cunning; wicked; artful; knavish [2] a Chinese family name
U+5202.0	fUTF8	刂
U+5202.0	fCNS11643	4-212A
U+5202.0	fGB	0-5856
U+5202.0	fR/S	 18.0
U+5202.0	fTotalStrokes	 2
U+5202.0	fCangjie	LN
U+5202.0	fFourCorner	22000
U+5202.0	fCantonese	dou1
U+5202.0	fMandarin	dao1
U+5202.0	fEnglish	 knife Kangxi radical number 18
U+5203.0	fUTF8	刃
U+5203.0	fCNS11643	1-4443
U+5203.0	fBig5	A462
U+5203.0	fGB	0-4850
U+5203.0	fR/S	 18.1
U+5203.0	fTotalStrokes	 3
U+5203.0	fCangjie	SHI
U+5203.0	fFourCorner	17420
U+5203.0	fHakka	 ngiun3; ngiun1; niun5
U+5203.0	fCantonese	jan6
U+5203.0	fMandarin	ren4
U+5203.0	fEnglish	[1] blade or edge of a knife [2] kill
U+5204.0	fUTF8	刄
U+5204.0	fCNS11643	3-2136
U+5204.0	fGB	E-2328
U+5204.0	fR/S	 18.1
U+5204.0	fTotalStrokes	 3
U+5204.0	fCangjie	SK
U+5204.0	fFourCorner	17450
U+5204.0	fHakka	 ngiun3; niun5; cong3
U+5204.0	fCantonese	jan6
U+5204.0	fMandarin	ren4
U+5204.0	fEnglish	edged tool; cutlery; knife edge
U+5205.0	fUTF8	刅
U+5205.0	fCNS11643	3-2152
U+5205.0	fGB	E-2329
U+5205.0	fR/S	  18.2
U+5205.0	fTotalStrokes	 4
U+5205.0	fCangjie	SHC
U+5205.0	fFourCorner	17330
U+5205.0	fHakka	  cong1
U+5205.0	fCantonese	cong1
U+5205.0	fMandarin	chuang1
U+5205.0	fEnglish	 create; make; invent begin
U+5206.0	fUTF8	分
U+5206.0	fCNS11643	1-4521
U+5206.0	fBig5	A4C0
U+5206.0	fGB	0-3756
U+5206.0	fR/S	 18.2
U+5206.0	fTotalStrokes	 4
U+5206.0	fCangjie	CSH
U+5206.0	fFourCorner	80227
U+5206.0	fHakka	  fun1; fun5; bun1; fun6; fun3; bin1
U+5206.0	fCantonese	fan1; fan6
U+5206.0	fMandarin	fen1; fen3
U+5206.0	fEnglish	[1] divide [2] part [3] share [4] distribute [5] distinguish [6] one minute [7] one cent [8] one hundredth of a tael [9] centimeter [10] located separately; branch
U+5206.1	fUTF8	分
U+5206.1	fCNS11643	1-4521
U+5206.1	fBig5	A4C0
U+5206.1	fGB	0-3756
U+5206.1	fR/S	 18.2
U+5206.1	fTotalStrokes	 4
U+5206.1	fCangjie	CSH
U+5206.1	fFourCorner	80227
U+5206.1	fMandarin	fen4
U+5206.1	fEnglish	[1] role or part [2] part or portion
U+5206.2	fUTF8	分
U+5206.2	fCNS11643	1-4521
U+5206.2	fBig5	A4C0
U+5206.2	fGB	0-3756
U+5206.2	fR/S	 18.2
U+5206.2	fTotalStrokes	 4
U+5206.2	fCangjie	CSH
U+5206.2	fFourCorner	80227
U+5206.2	fMandarin	fen5
U+5207.0	fUTF8	切
U+5207.0	fCNS11643	1-4522
U+5207.0	fBig5	A4C1
U+5207.0	fGB	0-4750
U+5207.0	fR/S	 18.2
U+5207.0	fTotalStrokes	 4
U+5207.0	fCangjie	PSH
U+5207.0	fFourCorner	47720
U+5207.0	fHakka	  ciet7; cet7; ts'iat7; ciet8
U+5207.0	fCantonese	cit3; cai3
U+5207.0	fMandarin	qie1; qi4
U+5207.0	fEnglish	 cut; mince; slice; carve
U+5207.1	fUTF8	切
U+5207.1	fCNS11643	1-4522
U+5207.1	fBig5	A4C1
U+5207.1	fGB	0-4750
U+5207.1	fR/S	 18.2
U+5207.1	fTotalStrokes	 4
U+5207.1	fCangjie	PSH
U+5207.1	fFourCorner	47720
U+5207.1	fMandarin	qie4
U+5207.1	fEnglish	 be close to
U+5208.0	fUTF8	刈
U+5208.0	fCNS11643	1-4523
U+5208.0	fBig5	A4C2
U+5208.0	fGB	0-5857
U+5208.0	fR/S	 18.2
U+5208.0	fTotalStrokes	 4
U+5208.0	fCangjie	KLN
U+5208.0	fFourCorner	42000
U+5208.0	fHakka	  ngai5;  ngi5; ngi6; ngi3; ngiai5
U+5208.0	fCantonese	ngaai6; aai6 (also)
U+5208.0	fMandarin	yi4
U+5208.0	fEnglish	 reap; mow
U+5209.0	fUTF8	刉
U+5209.0	fCNS11643	2-214C
U+5209.0	fBig5	C9BE
U+5209.0	fGB	3-3271
U+5209.0	fR/S	 18.3
U+5209.0	fTotalStrokes	 5
U+5209.0	fCangjie	ONLN
U+5209.0	fFourCorner	82700
U+5209.0	fCantonese	gei1
U+5209.0	fMandarin	ji4; ji1
U+520A.0	fUTF8	刊
U+520A.0	fCNS11643	1-457A
U+520A.0	fBig5	A55A
U+520A.0	fGB	0-3F2F
U+520A.0	fR/S	 18.3
U+520A.0	fTotalStrokes	 5
U+520A.0	fCangjie	MJLN
U+520A.0	fFourCorner	12400
U+520A.0	fHakka	  kan1
U+520A.0	fCantonese	hon1; hon2 (also)
U+520A.0	fMandarin	kan1
U+520A.0	fEnglish	[1] hew; cut [2] engrave [3] publication; periodical [4] publish
U+520B.0	fUTF8	刋
U+520B.0	fCNS11643	3-217D
U+520B.0	fGB	E-232A
U+520B.0	fR/S	  18.3
U+520B.0	fTotalStrokes	 5
U+520B.0	fCangjie	HJLN
U+520B.0	fFourCorner	22400
U+520B.0	fHakka	 kan1;  kien5
U+520B.0	fCantonese	cin3; hon2
U+520B.0	fMandarin	qian4; kan1
U+520B.0	fEnglish	 publication; periodical publish
U+520C.0	fUTF8	刌
U+520C.0	fCNS11643	2-214B
U+520C.0	fBig5	C96B
U+520C.0	fGB	3-354A
U+520C.0	fR/S	 18.3
U+520C.0	fTotalStrokes	 5
U+520C.0	fCangjie	DILN
U+520C.0	fFourCorner	42300
U+520C.0	fHakka	 cun3; cun5
U+520C.0	fCantonese	cyun2
U+520C.0	fMandarin	cun3
U+520C.0	fEnglish	 cut up small
U+520D.0	fUTF8	刍
U+520D.0	fGB	0-5B3B
U+520D.0	fR/S	 58.2; 18.3
U+520D.0	fTotalStrokes	 5
U+520D.0	fCangjie	NSM
U+520D.0	fHakka	 cih1
U+520D.0	fCantonese	co1
U+520D.0	fMandarin	chu2; zou1
U+520D.0	fEnglish	simplified form of &#x82BB;
U+520E.0	fUTF8	刎
U+520E.0	fCNS11643	1-4747
U+520E.0	fBig5	A646
U+520E.0	fGB	0-5858
U+520E.0	fR/S	 18.4
U+520E.0	fTotalStrokes	 6
U+520E.0	fCangjie	PHLN
U+520E.0	fFourCorner	22200
U+520E.0	fHakka	  wut7; wun3; vut7
U+520E.0	fCantonese	man5
U+520E.0	fMandarin	wen3
U+520E.0	fEnglish	 cut the throat
U+520F.0	fUTF8	刏
U+520F.0	fCNS11643	3-2325
U+520F.0	fGB	3-4B7E
U+520F.0	fR/S	  18.4
U+520F.0	fTotalStrokes	 6
U+520F.0	fCangjie	XONLN
U+520F.0	fFourCorner	82700
U+520F.0	fCantonese	gei1
U+520F.0	fMandarin	ji1; ji4
U+5210.0	fUTF8	刐
U+5210.0	fCNS11643	2-217D
U+5210.0	fBig5	C9BF
U+5210.0	fGB	5-3432
U+5210.0	fR/S	 18.4
U+5210.0	fTotalStrokes	 6
U+5210.0	fCangjie	BYLN
U+5210.0	fFourCorner	72400
U+5210.0	fHakka	 dan3; tan3
U+5210.0	fCantonese	daan2; daan3; taan2
U+5210.0	fMandarin	dan3
U+5210.0	fEnglish	 cut; trim
U+5211.0	fUTF8	刑
U+5211.0	fCNS11643	1-4745
U+5211.0	fBig5	A644
U+5211.0	fGB	0-504C
U+5211.0	fR/S	 18.4
U+5211.0	fTotalStrokes	 6
U+5211.0	fCangjie	MTLN
U+5211.0	fFourCorner	12400
U+5211.0	fHakka	  hin2; hin3
U+5211.0	fCantonese	jing4
U+5211.0	fMandarin	xing2
U+5211.0	fEnglish	punishment; penalty
U+5212.0	fUTF8	划
U+5212.0	fCNS11643	1-4746
U+5212.0	fBig5	A645
U+5212.0	fGB	0-3B2E
U+5212.0	fR/S	 18.4
U+5212.0	fTotalStrokes	 6
U+5212.0	fCangjie	ILN
U+5212.0	fFourCorner	52000
U+5212.0	fHakka	  fa2;  pa2; go5; wa1; fa1; wak8; vak8
U+5212.0	fCantonese	waa1; waa4 (also); waak6; faa3
U+5212.0	fMandarin	hua4; huai5; jiao3
U+5212.0	fEnglish	[1] row or paddle boat; oar [2] finger guess [3] simplified form of &#x5283;
U+5212.1	fUTF8	划
U+5212.1	fCNS11643	1-4746
U+5212.1	fBig5	A645
U+5212.1	fGB	0-3B2E
U+5212.1	fR/S	 18.4
U+5212.1	fTotalStrokes	 6
U+5212.1	fCangjie	ILN
U+5212.1	fFourCorner	52000
U+5212.1	fMandarin	hua2
U+5213.0	fUTF8	刓
U+5213.0	fCNS11643	2-217C
U+5213.0	fBig5	C9BD
U+5213.0	fGB	3-3272
U+5213.0	fR/S	 18.4
U+5213.0	fTotalStrokes	 6
U+5213.0	fCangjie	MULN
U+5213.0	fFourCorner	12200
U+5213.0	fHakka	 ngan2; ngien2
U+5213.0	fCantonese	jyun4
U+5213.0	fMandarin	wan2; wan1
U+5213.0	fEnglish	 round of; trim
U+5214.0	fUTF8	刔
U+5214.0	fCNS11643	3-2324
U+5214.0	fGB	E-232B
U+5214.0	fR/S	  18.4
U+5214.0	fTotalStrokes	 6
U+5214.0	fCangjie	DKLN
U+5214.0	fFourCorner	52000
U+5214.0	fCantonese	kyut3
U+5214.0	fMandarin	jue2
U+5214.0	fEnglish	 scoop out gouge
U+5215.0	fUTF8	刕
U+5215.0	fCNS11643	3-2326
U+5215.0	fGB	5-3422
U+5215.0	fR/S	  18.4
U+5215.0	fTotalStrokes	 6
U+5215.0	fCangjie	SHSHH
U+5215.0	fFourCorner	17220
U+5215.0	fHakka	 li2
U+5215.0	fCantonese	lei4
U+5215.0	fMandarin	li2
U+5216.0	fUTF8	刖
U+5216.0	fCNS11643	1-4748
U+5216.0	fBig5	A647
U+5216.0	fGB	0-6B3E
U+5216.0	fR/S	 18.4
U+5216.0	fTotalStrokes	 6
U+5216.0	fCangjie	BLN
U+5216.0	fFourCorner	72200
U+5216.0	fHakka	  ngiet8; nget8
U+5216.0	fCantonese	jyut6
U+5216.0	fMandarin	yue4; hua4; wa4; wo4; wu4
U+5216.0	fEnglish	 cut off feet as a punishment
U+5217.0	fUTF8	列
U+5217.0	fCNS11643	1-4744
U+5217.0	fBig5	A643
U+5217.0	fGB	0-4150
U+5217.0	fR/S	 18.4
U+5217.0	fTotalStrokes	 6
U+5217.0	fCangjie	MNLN
U+5217.0	fFourCorner	12200
U+5217.0	fHakka	 liet8; let8; liat8
U+5217.0	fCantonese	lit6; laat6 (also)
U+5217.0	fMandarin	lie4; li4
U+5217.0	fEnglish	[1] arrange in line; line up [2] enumerate [3] display [4] line; series
U+5218.0	fUTF8	刘
U+5218.0	fCNS11643	3-2327
U+5218.0	fGB	0-4175
U+5218.0	fR/S	 18.4
U+5218.0	fTotalStrokes	 6
U+5218.0	fCangjie	YKLN
U+5218.0	fFourCorner	02400
U+5218.0	fHakka	 liu2
U+5218.0	fCantonese	lau4; lau2 (also)
U+5218.0	fMandarin	liu2
U+5218.0	fEnglish	simplified form of &#x5289;
U+5219.0	fUTF8	则
U+5219.0	fGB	0-5472
U+5219.0	fR/S	 18.4
U+5219.0	fTotalStrokes	 6
U+5219.0	fCangjie	BOLN
U+5219.0	fHakka	 zet7
U+5219.0	fCantonese	zak1
U+5219.0	fMandarin	ze2
U+5219.0	fEnglish	simplified form of &#x5247;
U+521A.0	fUTF8	刚
U+521A.0	fGB	0-3855
U+521A.0	fR/S	 18.4
U+521A.0	fTotalStrokes	 6
U+521A.0	fCangjie	BKLN
U+521A.0	fHakka	 gong1
U+521A.0	fCantonese	gong1
U+521A.0	fMandarin	gang1
U+521A.0	fEnglish	simplified form of &#x525B;
U+521B.0	fUTF8	创
U+521B.0	fGB	0-3434
U+521B.0	fR/S	 18.4
U+521B.0	fTotalStrokes	 6
U+521B.0	fCangjie	OULN
U+521B.0	fHakka	 cong3; cong1
U+521B.0	fCantonese	cong3; cong1
U+521B.0	fMandarin	chuang4; chuan4; qiang1; tiao3
U+521B.0	fEnglish	simplified form of &#x5275;
U+521B.1	fUTF8	创
U+521B.1	fGB	0-3434
U+521B.1	fR/S	 18.4
U+521B.1	fTotalStrokes	 6
U+521B.1	fCangjie	OULN
U+521B.1	fMandarin	chuang1
U+521C.0	fUTF8	刜
U+521C.0	fCNS11643	2-232E
U+521C.0	fBig5	CA6C
U+521C.0	fGB	3-3276
U+521C.0	fR/S	 18.5
U+521C.0	fTotalStrokes	 7
U+521C.0	fCangjie	LNLN
U+521C.0	fFourCorner	52000
U+521C.0	fHakka	  fut7
U+521C.0	fCantonese	fat1; fat6 (also)
U+521C.0	fMandarin	fu2; mo2
U+521C.0	fEnglish	chop
U+521D.0	fUTF8	初
U+521D.0	fCNS11643	1-4F4F
U+521D.0	fBig5	AAEC
U+521D.0	fGB	0-3375
U+521D.0	fR/S	 18.5
U+521D.0	fTotalStrokes	 7
U+521D.0	fCangjie	LSH
U+521D.0	fFourCorner	37220
U+521D.0	fHakka	  co1; cu1; ze1; cih1
U+521D.0	fCantonese	co1
U+521D.0	fMandarin	chu1
U+521D.0	fEnglish	[1] first [2] original [3] junior [4] beginning; initial; primary; early
U+521E.0	fUTF8	刞
U+521E.0	fCNS11643	2-232F
U+521E.0	fBig5	CA6D
U+521E.0	fGB	5-3436
U+521E.0	fR/S	 18.5
U+521E.0	fTotalStrokes	 7
U+521E.0	fCangjie	BMLN
U+521E.0	fFourCorner	72100
U+521E.0	fCantonese	ceoi3
U+521E.0	fMandarin	qu4
U+521F.0	fUTF8	刟
U+521F.0	fCNS11643	3-245F
U+521F.0	fGB	E-232C
U+521F.0	fR/S	 18.5
U+521F.0	fTotalStrokes	 7
U+521F.0	fCangjie	SRLN
U+521F.0	fFourCorner	12600
U+521F.0	fCantonese	diu1
U+521F.0	fMandarin	ju1
U+5220.0	fUTF8	删
U+5220.0	fCNS11643	3-2461
U+5220.0	fGB	0-493E
U+5220.0	fR/S	 18.5
U+5220.0	fTotalStrokes	 7
U+5220.0	fCangjie	BMLN
U+5220.0	fFourCorner	72400
U+5220.0	fHakka	  san1; san3
U+5220.0	fCantonese	saan1
U+5220.0	fMandarin	shan1
U+5220.0	fEnglish	 cut; delete; take out; erase
U+5221.0	fUTF8	刡
U+5221.0	fCNS11643	2-2330
U+5221.0	fBig5	CA6E
U+5221.0	fGB	5-343A
U+5221.0	fR/S	 18.5
U+5221.0	fTotalStrokes	 7
U+5221.0	fCangjie	RPLN
U+5221.0	fFourCorner	72700
U+5221.0	fHakka	 men3
U+5221.0	fCantonese	man5
U+5221.0	fMandarin	min3
U+5221.0	fEnglish	 scrape pare
U+5222.0	fUTF8	刢
U+5222.0	fCNS11643	4-2340
U+5222.0	fGB	5-3437
U+5222.0	fR/S	  18.5
U+5222.0	fTotalStrokes	 7
U+5222.0	fCangjie	OILN
U+5222.0	fFourCorner	82300
U+5222.0	fCantonese	ling4
U+5222.0	fMandarin	ling2
U+5223.0	fUTF8	刣
U+5223.0	fCNS11643	3-245C
U+5223.0	fGB	E-232D
U+5223.0	fR/S	 18.5
U+5223.0	fTotalStrokes	 7
U+5223.0	fCangjie	IRLN
U+5223.0	fFourCorner	22600
U+5223.0	fHakka	 tai2; zhung1; zung1
U+5223.0	fCantonese	zung1
U+5223.0	fMandarin	zhong1
U+5223.0	fEnglish	[1] scrape [2] oppose
U+5224.0	fUTF8	判
U+5224.0	fCNS11643	1-4932
U+5224.0	fBig5	A750
U+5224.0	fGB	0-4550
U+5224.0	fR/S	 18.5
U+5224.0	fTotalStrokes	 7
U+5224.0	fCangjie	FQLN
U+5224.0	fFourCorner	92500
U+5224.0	fHakka	  pan5;  pan3; pan6
U+5224.0	fCantonese	pun3
U+5224.0	fMandarin	pan4
U+5224.0	fEnglish	judge; discriminate; conclude
U+5225.0	fUTF8	別
U+5225.0	fCNS11643	1-4931
U+5225.0	fBig5	A74F
U+5225.0	fGB	E-232E
U+5225.0	fR/S	 18.5
U+5225.0	fTotalStrokes	 7
U+5225.0	fCangjie	RSLN
U+5225.0	fFourCorner	62200
U+5225.0	fHakka	 pet8; piet8
U+5225.0	fCantonese	bit6; bit3
U+5225.0	fMandarin	bie2
U+5225.0	fEnglish	[1] part [2] distinguish; differentiate; separate [3] other; another; different [4] do not (in imperative expressions)
U+5225.1	fUTF8	別
U+5225.1	fCNS11643	1-4931
U+5225.1	fBig5	A74F
U+5225.1	fGB	E-232E
U+5225.1	fR/S	 18.5
U+5225.1	fTotalStrokes	 7
U+5225.1	fCangjie	RSLN
U+5225.1	fFourCorner	62200
U+5225.1	fMandarin	bie4
U+5226.0	fUTF8	刦
U+5226.0	fCNS11643	3-245E
U+5226.0	fGB	E-232F
U+5226.0	fR/S	  18.5
U+5226.0	fTotalStrokes	 7
U+5226.0	fCangjie	GILN
U+5226.0	fFourCorner	42700
U+5226.0	fHakka	 giap7
U+5226.0	fCantonese	gip3
U+5226.0	fMandarin	jie2
U+5226.0	fEnglish	 rob; plunder
U+5227.0	fUTF8	刧
U+5227.0	fCNS11643	3-245D
U+5227.0	fGB	E-2330
U+5227.0	fR/S	 18.5
U+5227.0	fTotalStrokes	 7
U+5227.0	fCangjie	GISH
U+5227.0	fFourCorner	47720
U+5227.0	fHakka	  giap7
U+5227.0	fCantonese	gip3
U+5227.0	fMandarin	jie2
U+5227.0	fEnglish	disaster; misfortune; coerce
U+5228.0	fUTF8	刨
U+5228.0	fCNS11643	1-4935
U+5228.0	fBig5	A753
U+5228.0	fGB	0-4559
U+5228.0	fR/S	 18.5
U+5228.0	fTotalStrokes	 7
U+5228.0	fCangjie	PULN
U+5228.0	fFourCorner	22700
U+5228.0	fHakka	  pau2
U+5228.0	fCantonese	paau4; paau2 (also)
U+5228.0	fMandarin	pao2; bao4
U+5228.0	fEnglish	[1] carpenter's plane; plane, level [2] dig; excavate
U+5229.0	fUTF8	利
U+5229.0	fCNS11643	1-4933
U+5229.0	fBig5	A751
U+5229.0	fGB	0-407B
U+5229.0	fR/S	 18.5
U+5229.0	fTotalStrokes	 7
U+5229.0	fCangjie	HDLN
U+5229.0	fFourCorner	22900
U+5229.0	fHakka	  li5; li6; li3
U+5229.0	fCantonese	lei6; lei2 (also)
U+5229.0	fMandarin	li4
U+5229.0	fEnglish	[1] profit; gains; gain; advantage; benefit; merit [2] sharp [3] benefit; serve
U+5229.1	fUTF8	利
U+5229.1	fCNS11643	1-4933
U+5229.1	fBig5	A751
U+5229.1	fGB	0-407B
U+5229.1	fR/S	 18.5
U+5229.1	fTotalStrokes	 7
U+5229.1	fCangjie	HDLN
U+5229.1	fFourCorner	22900
U+5229.1	fMandarin	li5
U+522A.0	fUTF8	刪
U+522A.0	fCNS11643	1-4934
U+522A.0	fBig5	A752
U+522A.0	fGB	E-2331
U+522A.0	fR/S	 18.5
U+522A.0	fTotalStrokes	 7
U+522A.0	fCangjie	BTLN
U+522A.0	fFourCorner	72400
U+522A.0	fHakka	 san1
U+522A.0	fCantonese	saan1
U+522A.0	fMandarin	shan1
U+522A.0	fEnglish	cut; delete; erase; geld
U+522B.0	fUTF8	别
U+522B.0	fCNS11643	3-2460
U+522B.0	fGB	0-3170
U+522B.0	fR/S	 18.5
U+522B.0	fTotalStrokes	 7
U+522B.0	fCangjie	RSLN
U+522B.0	fFourCorner	62400
U+522B.0	fHakka	 pet8; piet8; p'iat8
U+522B.0	fCantonese	bit6; bit3
U+522B.0	fMandarin	bie2
U+522B.0	fEnglish	separate; other; do not
U+522B.1	fUTF8	别
U+522B.1	fCNS11643	3-2460
U+522B.1	fGB	0-3170
U+522B.1	fR/S	 18.5
U+522B.1	fTotalStrokes	 7
U+522B.1	fCangjie	RSLN
U+522B.1	fFourCorner	62400
U+522B.1	fMandarin	bie4
U+522C.0	fUTF8	刬
U+522C.0	fGB	8-7A3C
U+522C.0	fR/S	 18.5
U+522C.0	fTotalStrokes	 7
U+522C.0	fCangjie	IJLN
U+522C.0	fCantonese	caan2
U+522C.0	fMandarin	chan3
U+522C.0	fEnglish	to level off; to trim; to pare down
U+522D.0	fUTF8	刭
U+522D.0	fGB	0-5859
U+522D.0	fR/S	 18.5
U+522D.0	fTotalStrokes	 7
U+522D.0	fCangjie	NMLN
U+522D.0	fHakka	 gin5; gin3
U+522D.0	fCantonese	ging2
U+522D.0	fMandarin	jing3; jing1; jing4
U+522D.0	fEnglish	simplified form of &#x5244;
U+522E.0	fUTF8	刮
U+522E.0	fCNS11643	1-4C30
U+522E.0	fBig5	A8ED
U+522E.0	fGB	0-394E
U+522E.0	fR/S	 18.6
U+522E.0	fTotalStrokes	 8
U+522E.0	fCangjie	HRLN
U+522E.0	fFourCorner	22600
U+522E.0	fHakka	 kwat7; gat7; kuat7; gwat8; gwat7
U+522E.0	fCantonese	gwaat3; gwaat2 (also)
U+522E.0	fMandarin	gua1
U+522E.0	fEnglish	 shave; pare; scrape
U+522F.0	fUTF8	刯
U+522F.0	fCNS11643	3-275B
U+522F.0	fGB	E-2332
U+522F.0	fR/S	  18.6
U+522F.0	fTotalStrokes	 8
U+522F.0	fCangjie	MMLN
U+522F.0	fFourCorner	12100
U+522F.0	fCantonese	gang1
U+522F.0	fMandarin	gen1
U+5230.0	fUTF8	到
U+5230.0	fCNS11643	1-4C2F
U+5230.0	fBig5	A8EC
U+5230.0	fGB	0-353D
U+5230.0	fR/S	 18.6
U+5230.0	fTotalStrokes	 8
U+5230.0	fCangjie	MGLN
U+5230.0	fFourCorner	12100
U+5230.0	fHakka	  dau5;  do5; do3
U+5230.0	fCantonese	dou3; dou2 (also)
U+5230.0	fMandarin	dao4
U+5230.0	fEnglish	[1] reach; arrive; go to; been to [2] a Chinese family name
U+5230.1	fUTF8	到
U+5230.1	fCNS11643	1-4C2F
U+5230.1	fBig5	A8EC
U+5230.1	fGB	0-353D
U+5230.1	fR/S	 18.6
U+5230.1	fTotalStrokes	 8
U+5230.1	fCangjie	MGLN
U+5230.1	fFourCorner	12100
U+5230.1	fMandarin	dao5
U+5231.0	fUTF8	刱
U+5231.0	fCNS11643	2-2554
U+5231.0	fBig5	CBD4
U+5231.0	fGB	5-3423
U+5231.0	fR/S	 18.6
U+5231.0	fTotalStrokes	 8
U+5231.0	fCangjie	TTSHI
U+5231.0	fFourCorner	87450
U+5231.0	fHakka	 cong3; cong1
U+5231.0	fCantonese	cong3; cong1
U+5231.0	fMandarin	chuang1; chuang4
U+5231.0	fEnglish	 establish; create knife cut
U+5232.0	fUTF8	刲
U+5232.0	fCNS11643	2-2551
U+5232.0	fBig5	CBD1
U+5232.0	fGB	3-3277
U+5232.0	fR/S	 18.6
U+5232.0	fTotalStrokes	 8
U+5232.0	fCangjie	GGLN
U+5232.0	fFourCorner	42100
U+5232.0	fHakka	 kui2; gui1; kwi2; k'wui2
U+5232.0	fCantonese	gwai1; kwai1
U+5232.0	fMandarin	kui1; gui4
U+5232.0	fEnglish	cut; slice off
U+5233.0	fUTF8	刳
U+5233.0	fCNS11643	2-2552
U+5233.0	fBig5	CBD2
U+5233.0	fGB	0-585A
U+5233.0	fR/S	 18.6
U+5233.0	fTotalStrokes	 8
U+5233.0	fCangjie	KSLN
U+5233.0	fFourCorner	42200
U+5233.0	fHakka	 ku1;  kwa1
U+5233.0	fCantonese	fu1
U+5233.0	fMandarin	ku1; ku2
U+5233.0	fEnglish	[1] cut apart; carve apart [2] gouge; scoop; hollow
U+5234.0	fUTF8	刴
U+5234.0	fCNS11643	3-275E
U+5234.0	fGB	E-2333
U+5234.0	fR/S	 18.6
U+5234.0	fTotalStrokes	 8
U+5234.0	fCangjie	NDLN
U+5234.0	fFourCorner	12900
U+5234.0	fHakka	 do3; do5; dok8; dok7
U+5234.0	fCantonese	do2 (also); doek3
U+5234.0	fMandarin	duo4
U+5234.0	fEnglish	 chop by pounding; mince; hash
U+5235.0	fUTF8	刵
U+5235.0	fCNS11643	2-2550
U+5235.0	fBig5	CBD0
U+5235.0	fGB	3-6045
U+5235.0	fR/S	 18.6
U+5235.0	fTotalStrokes	 8
U+5235.0	fCangjie	SJLN
U+5235.0	fFourCorner	12400
U+5235.0	fHakka	 ngi3; ngi5
U+5235.0	fCantonese	ji6; ji5; nei6
U+5235.0	fMandarin	er4
U+5235.0	fEnglish	punishment by cutting off ears
U+5236.0	fUTF8	制
U+5236.0	fCNS11643	1-4C31
U+5236.0	fBig5	A8EE
U+5236.0	fGB	0-5646
U+5236.0	fR/S	 18.6
U+5236.0	fTotalStrokes	 8
U+5236.0	fCangjie	HBLN
U+5236.0	fFourCorner	22200
U+5236.0	fHakka	 zhi5; ze5; zi5; zih5; tje5
U+5236.0	fCantonese	zai3
U+5236.0	fMandarin	zhi4
U+5236.0	fEnglish	[1] establish; institute; set up [2] prevail; overpower; control [3] system [4] used before the signature in letter writing to indicate the writer is in mourning [5] a Chinese family name
U+5237.0	fUTF8	刷
U+5237.0	fCNS11643	1-4C2D
U+5237.0	fBig5	A8EA
U+5237.0	fGB	0-4B22
U+5237.0	fR/S	 18.6
U+5237.0	fTotalStrokes	 8
U+5237.0	fCangjie	SBLN
U+5237.0	fFourCorner	72200
U+5237.0	fHakka	  sot7;  cat7; so5
U+5237.0	fCantonese	caat3; saat3 (also); caat2 (also)
U+5237.0	fMandarin	shua1
U+5237.0	fEnglish	[1] brush; clean; daub; clean with brush, scrub [2] brush [3] eliminate
U+5237.1	fUTF8	刷
U+5237.1	fCNS11643	1-4C2D
U+5237.1	fBig5	A8EA
U+5237.1	fGB	0-4B22
U+5237.1	fR/S	 18.6
U+5237.1	fTotalStrokes	 8
U+5237.1	fCangjie	SBLN
U+5237.1	fFourCorner	72200
U+5237.1	fMandarin	shua4
U+5237.1	fEnglish	[1] pale (face) [2] choose
U+5238.0	fUTF8	券
U+5238.0	fCNS11643	1-4C2C
U+5238.0	fBig5	A8E9
U+5238.0	fGB	0-482F
U+5238.0	fR/S	 18.6
U+5238.0	fTotalStrokes	 8
U+5238.0	fCangjie	FQSH
U+5238.0	fFourCorner	90227
U+5238.0	fHakka	 gien5; kien5; ken5; gen1
U+5238.0	fCantonese	hyun3; gyun3 (also)
U+5238.0	fMandarin	quan4; xuan4
U+5238.0	fEnglish	[1] ticket [2] certificate [3] bond
U+5239.0	fUTF8	刹
U+5239.0	fCNS11643	3-275F
U+5239.0	fGB	0-4932
U+5239.0	fR/S	 18.6
U+5239.0	fTotalStrokes	 8
U+5239.0	fCangjie	KDLN
U+5239.0	fFourCorner	42900
U+5239.0	fHakka	 sat7
U+5239.0	fCantonese	caat3; saat3
U+5239.0	fMandarin	cha4
U+5239.0	fEnglish	 (Buddhist) temple, shrine, monastery or abbey
U+5239.1	fUTF8	刹
U+5239.1	fCNS11643	3-275F
U+5239.1	fGB	0-4932
U+5239.1	fR/S	 18.6
U+5239.1	fTotalStrokes	 8
U+5239.1	fCangjie	KDLN
U+5239.1	fFourCorner	42900
U+5239.1	fMandarin	sha1
U+523A.0	fUTF8	刺
U+523A.0	fCNS11643	1-4C2E
U+523A.0	fBig5	A8EB
U+523A.0	fGB	0-344C
U+523A.0	fR/S	 18.6
U+523A.0	fTotalStrokes	 8
U+523A.0	fCangjie	DBLN
U+523A.0	fFourCorner	52900
U+523A.0	fHakka	  ciuk7;  cih5; ci5; ze5; lak7; cu5
U+523A.0	fCantonese	ci3; cik3 (also); sik3 (also); cik1; cek3
U+523A.0	fMandarin	ci4; qi4; ci1; ji4
U+523A.0	fEnglish	[1] pierce; stab; prick [2] irritate; stimulate [3] thorn; splinter; small fishbones; sting [4] assassinate [5] name card
U+523B.0	fUTF8	刻
U+523B.0	fCNS11643	1-4C2B
U+523B.0	fBig5	A8E8
U+523B.0	fGB	0-3F4C
U+523B.0	fR/S	 18.6
U+523B.0	fTotalStrokes	 8
U+523B.0	fCangjie	YOLN
U+523B.0	fFourCorner	02800
U+523B.0	fHakka	 ket7;  kat7; kiet7; kwat7
U+523B.0	fCantonese	hak1; haak1 (also)
U+523B.0	fMandarin	ke4
U+523B.0	fEnglish	[1] carve; engrave; cut [2] quarter (of an hour) [3] cruel; heartless; unfeeling; cutting; harshly; acrimonious; biting [4] moment
U+523B.1	fUTF8	刻
U+523B.1	fCNS11643	1-4C2B
U+523B.1	fBig5	A8E8
U+523B.1	fGB	0-3F4C
U+523B.1	fR/S	 18.6
U+523B.1	fTotalStrokes	 8
U+523B.1	fCangjie	YOLN
U+523B.1	fFourCorner	02800
U+523B.1	fMandarin	ke1
U+523B.1	fEnglish	 carve; engrave
U+523C.0	fUTF8	刼
U+523C.0	fCNS11643	3-275D
U+523C.0	fGB	E-2334
U+523C.0	fR/S	 18.6
U+523C.0	fTotalStrokes	 8
U+523C.0	fCangjie	GISK
U+523C.0	fFourCorner	47750
U+523C.0	fCantonese	gip3
U+523C.0	fMandarin	jie2
U+523C.0	fEnglish	 plunder rob openly
U+523D.0	fUTF8	刽
U+523D.0	fCNS11643	4-254E
U+523D.0	fGB	0-3974
U+523D.0	fR/S	 18.6
U+523D.0	fTotalStrokes	 8; 7
U+523D.0	fCangjie	OILN
U+523D.0	fFourCorner	82700
U+523D.0	fHakka	 kwai5
U+523D.0	fCantonese	kui2; kwui2 (also)
U+523D.0	fMandarin	kuai4; gui4; gua1
U+523D.0	fEnglish	simplified form of &#x528A;
U+523E.0	fUTF8	刾
U+523E.0	fCNS11643	3-644F
U+523E.0	fGB	E-2335
U+523E.0	fR/S	 18.6
U+523E.0	fTotalStrokes	 8
U+523E.0	fCangjie	KTLN
U+523E.0	fCantonese	ci3; cik3 (also)
U+523E.0	fMandarin	ci4; qi4
U+523E.0	fEnglish	 stab prick; irritate prod
U+523F.0	fUTF8	刿
U+523F.0	fGB	0-585B
U+523F.0	fR/S	 18.6
U+523F.0	fTotalStrokes	 8
U+523F.0	fCangjie	UNLN
U+523F.0	fHakka	 kwui5; k'wui3
U+523F.0	fCantonese	gwai3
U+523F.0	fMandarin	gui4
U+523F.0	fEnglish	simplified form of &#x528C;
U+5240.0	fUTF8	剀
U+5240.0	fGB	0-585C
U+5240.0	fR/S	 18.6
U+5240.0	fTotalStrokes	 8
U+5240.0	fCangjie	UULN
U+5240.0	fHakka	 koi3
U+5240.0	fCantonese	hoi2
U+5240.0	fMandarin	kai3; sheng4
U+5240.0	fEnglish	simplified form of &#x5274;
U+5241.0	fUTF8	剁
U+5241.0	fCNS11643	1-4C32
U+5241.0	fBig5	A8EF
U+5241.0	fGB	0-3667
U+5241.0	fR/S	 18.6
U+5241.0	fTotalStrokes	 8
U+5241.0	fCangjie	HDLN
U+5241.0	fFourCorner	72900
U+5241.0	fHakka	 dok8;  do3;  do5; dio5
U+5241.0	fCantonese	doek3; do2 (also); do3
U+5241.0	fMandarin	duo4
U+5241.0	fEnglish	chop by pounding; mince; hash
U+5242.0	fUTF8	剂
U+5242.0	fGB	0-3C41
U+5242.0	fR/S	 18.6
U+5242.0	fTotalStrokes	 8
U+5242.0	fCangjie	YLLN
U+5242.0	fHakka	 zi5
U+5242.0	fCantonese	zai1
U+5242.0	fMandarin	ji4; zui1
U+5242.0	fEnglish	simplified form of &#x5291;
U+5243.0	fUTF8	剃
U+5243.0	fCNS11643	1-5027
U+5243.0	fBig5	AB63
U+5243.0	fGB	0-4C6A
U+5243.0	fR/S	 18.7
U+5243.0	fTotalStrokes	 9
U+5243.0	fCangjie	CHLN
U+5243.0	fFourCorner	82200
U+5243.0	fHakka	  tai5;  ti5; t'e5
U+5243.0	fCantonese	tai3
U+5243.0	fMandarin	ti4
U+5243.0	fEnglish	 shave
U+5244.0	fUTF8	剄
U+5244.0	fCNS11643	2-2932
U+5244.0	fBig5	CDF0
U+5244.0	fGB	1-5859
U+5244.0	fR/S	 18.7
U+5244.0	fTotalStrokes	 9
U+5244.0	fCangjie	MMLN
U+5244.0	fFourCorner	12100
U+5244.0	fHakka	  gin5;  gang5; gin3; giang3
U+5244.0	fCantonese	ging2
U+5244.0	fMandarin	jing3; jing1; jing4
U+5244.0	fEnglish	 cut the throat
U+5245.0	fUTF8	剅
U+5245.0	fCNS11643	3-2B3F
U+5245.0	fGB	3-6E40
U+5245.0	fR/S	  18.7
U+5245.0	fTotalStrokes	 9
U+5245.0	fCangjie	MTLN
U+5245.0	fFourCorner	12100
U+5245.0	fCantonese	lau4
U+5245.0	fMandarin	lou2
U+5246.0	fUTF8	剆
U+5246.0	fCNS11643	2-2553
U+5246.0	fBig5	CBD3
U+5246.0	fGB	E-2336
U+5246.0	fR/S	 18.7
U+5246.0	fTotalStrokes	 9
U+5246.0	fCangjie	IILN
U+5246.0	fFourCorner	32700
U+5246.0	fHakka	 lot7
U+5246.0	fCantonese	lo5; lo4
U+5246.0	fMandarin	gen1; luo2; luo3
U+5247.0	fUTF8	則
U+5247.0	fCNS11643	1-502C
U+5247.0	fBig5	AB68
U+5247.0	fGB	1-5472
U+5247.0	fR/S	 18.7
U+5247.0	fTotalStrokes	 9
U+5247.0	fCangjie	BCLN
U+5247.0	fFourCorner	62800
U+5247.0	fHakka	  zet7; ziet7
U+5247.0	fCantonese	zak1
U+5247.0	fMandarin	ze2
U+5247.0	fEnglish	[1] law; rule; regulation; standard; norm; criterion [2] a particle indicating consequence (usually used after a supposition) or a reason [3] a numerary particle used before news reports, etc. [4] but; however [5] imitate; follow
U+5248.0	fUTF8	剈
U+5248.0	fCNS11643	3-2B40
U+5248.0	fGB	3-327C
U+5248.0	fR/S	  18.7
U+5248.0	fTotalStrokes	 9
U+5248.0	fCangjie	RBLN
U+5248.0	fFourCorner	62200
U+5248.0	fCantonese	jyun1
U+5248.0	fMandarin	yuan1
U+5249.0	fUTF8	剉
U+5249.0	fCNS11643	2-2933
U+5249.0	fBig5	CDF1
U+5249.0	fGB	E-2337
U+5249.0	fR/S	 18.7
U+5249.0	fTotalStrokes	 9
U+5249.0	fCangjie	OGLN
U+5249.0	fFourCorner	82100
U+5249.0	fHakka	  co5
U+5249.0	fCantonese	co3
U+5249.0	fMandarin	cuo4
U+5249.0	fEnglish	[1] file; smooth [2] steel file
U+524A.0	fUTF8	削
U+524A.0	fCNS11643	1-5028
U+524A.0	fBig5	AB64
U+524A.0	fGB	0-4F77
U+524A.0	fR/S	 18.7
U+524A.0	fTotalStrokes	 9
U+524A.0	fCangjie	FBLN
U+524A.0	fFourCorner	92200
U+524A.0	fHakka	  siok7
U+524A.0	fCantonese	soek3; ciu3
U+524A.0	fMandarin	xue1; qiao4; shao4; xiao4
U+524A.0	fEnglish	[1] scrape off; pare; cut; shave; trim [2] deprive
U+524A.1	fUTF8	削
U+524A.1	fCNS11643	1-5028
U+524A.1	fBig5	AB64
U+524A.1	fGB	0-4F77
U+524A.1	fR/S	 18.7
U+524A.1	fTotalStrokes	 9
U+524A.1	fCangjie	FBLN
U+524A.1	fFourCorner	92200
U+524A.1	fMandarin	xiao1
U+524B.0	fUTF8	剋
U+524B.0	fCNS11643	1-502B
U+524B.0	fBig5	AB67
U+524B.0	fGB	1-7848
U+524B.0	fR/S	 18.7
U+524B.0	fTotalStrokes	 9
U+524B.0	fCangjie	JULN
U+524B.0	fFourCorner	42210
U+524B.0	fHakka	 kiet7;  ket7
U+524B.0	fCantonese	hak1; haak1 (also)
U+524B.0	fMandarin	ke4; kei1
U+524B.0	fEnglish	[1] overcome [2] cut down [3] limit [4] engrave; imprint [5] can; be able to
U+524C.0	fUTF8	剌
U+524C.0	fCNS11643	1-502A
U+524C.0	fBig5	AB66
U+524C.0	fGB	0-585D
U+524C.0	fR/S	 18.7
U+524C.0	fTotalStrokes	 9
U+524C.0	fCangjie	DLLN
U+524C.0	fFourCorner	52900
U+524C.0	fHakka	  lat8; la1
U+524C.0	fCantonese	laat6; laa1; laai1 (also); lai1
U+524C.0	fMandarin	la4
U+524C.0	fEnglish	 go against; contradict; perverse; disagreeable; rebellious
U+524C.1	fUTF8	剌
U+524C.1	fCNS11643	1-502A
U+524C.1	fBig5	AB66
U+524C.1	fGB	0-585D
U+524C.1	fR/S	 18.7
U+524C.1	fTotalStrokes	 9
U+524C.1	fCangjie	DLLN
U+524C.1	fFourCorner	52900
U+524C.1	fMandarin	la2
U+524C.1	fEnglish	 slash open
U+524D.0	fUTF8	前
U+524D.0	fCNS11643	1-5029
U+524D.0	fBig5	AB65
U+524D.0	fGB	0-4730
U+524D.0	fR/S	 18.7
U+524D.0	fTotalStrokes	 9
U+524D.0	fCangjie	TBLN
U+524D.0	fFourCorner	80221
U+524D.0	fHakka	 cien2; cen2; ts'ian3
U+524D.0	fCantonese	cin4
U+524D.0	fMandarin	qian2; jian3
U+524D.0	fEnglish	[1] front; in front; forward; before [2] previous; former; preceding; past; of earlier times [3] future [4] advance; proceed; progress; precede
U+524E.0	fUTF8	剎
U+524E.0	fCNS11643	1-5026
U+524E.0	fBig5	AB62
U+524E.0	fGB	E-2338
U+524E.0	fR/S	 18.7
U+524E.0	fTotalStrokes	 9
U+524E.0	fCangjie	KCLN
U+524E.0	fFourCorner	42900
U+524E.0	fHakka	 sat7
U+524E.0	fCantonese	caat3; saat3
U+524E.0	fMandarin	cha4
U+524E.0	fEnglish	temple
U+524E.1	fUTF8	剎
U+524E.1	fCNS11643	1-5026
U+524E.1	fBig5	AB62
U+524E.1	fGB	E-2338
U+524E.1	fR/S	 18.7
U+524E.1	fTotalStrokes	 9
U+524E.1	fCangjie	KCLN
U+524E.1	fFourCorner	42900
U+524E.1	fMandarin	sha1
U+524F.0	fUTF8	剏
U+524F.0	fCNS11643	3-2B43
U+524F.0	fGB	E-2339
U+524F.0	fR/S	 18.7
U+524F.0	fTotalStrokes	 9
U+524F.0	fCangjie	TTSK
U+524F.0	fFourCorner	87450
U+524F.0	fCantonese	cong3; cong1
U+524F.0	fMandarin	chuang4; chuang1; chuan4
U+524F.0	fEnglish	 establish; create knife cut
U+5250.0	fUTF8	剐
U+5250.0	fGB	0-3950
U+5250.0	fR/S	 18.7
U+5250.0	fTotalStrokes	 9
U+5250.0	fCangjie	RBLN
U+5250.0	fHakka	 gwa3
U+5250.0	fCantonese	waa2 (also); gwaa2
U+5250.0	fMandarin	gua3
U+5250.0	fEnglish	simplified form of &#x526E;
U+5251.0	fUTF8	剑
U+5251.0	fGB	0-3D23
U+5251.0	fR/S	 18.7
U+5251.0	fTotalStrokes	 9
U+5251.0	fCangjie	OMLN
U+5251.0	fHakka	 giam5
U+5251.0	fCantonese	gim3
U+5251.0	fMandarin	jian4
U+5251.0	fEnglish	simplified form of &#x528D;
U+5252.0	fUTF8	剒
U+5252.0	fCNS11643	2-2E2B
U+5252.0	fBig5	D0E8
U+5252.0	fGB	5-3445
U+5252.0	fR/S	 18.8
U+5252.0	fTotalStrokes	 10
U+5252.0	fCangjie	TALN
U+5252.0	fFourCorner	42600
U+5252.0	fHakka	  cok7
U+5252.0	fCantonese	cok3; co3; co4
U+5252.0	fMandarin	cuo4
U+5252.0	fEnglish	 make rhinoceros-horn into cups
U+5253.0	fUTF8	剓
U+5253.0	fCNS11643	3-2F68
U+5253.0	fGB	5-3425
U+5253.0	fR/S	  18.8
U+5253.0	fTotalStrokes	 10
U+5253.0	fCangjie	HHSH
U+5253.0	fFourCorner	27220
U+5253.0	fCantonese	lei4
U+5253.0	fMandarin	li2
U+5254.0	fUTF8	剔
U+5254.0	fCNS11643	1-544A
U+5254.0	fBig5	ADE7
U+5254.0	fGB	0-4C5E
U+5254.0	fR/S	 18.8
U+5254.0	fTotalStrokes	 10
U+5254.0	fCangjie	AHLN
U+5254.0	fFourCorner	62200
U+5254.0	fHakka	  tit7; tit8
U+5254.0	fCantonese	tik1; tai3 (also)
U+5254.0	fMandarin	ti1; shi4; ti4
U+5254.0	fEnglish	[1] separate bones from meat; scrape meat off bones [2] pick out inferior materials; scrape off [3] a rising stroke in Chinese characters
U+5255.0	fUTF8	剕
U+5255.0	fCNS11643	2-2E2E
U+5255.0	fBig5	D0EB
U+5255.0	fGB	3-7455
U+5255.0	fR/S	 18.8
U+5255.0	fTotalStrokes	 10
U+5255.0	fCangjie	LMSYN
U+5255.0	fFourCorner	12100
U+5255.0	fHakka	 pui5; pi5; fui1; fi1; fui5
U+5255.0	fCantonese	fei6; fai3 (also); fai6
U+5255.0	fMandarin	fei4
U+5255.0	fEnglish	the punishment of cutting off the feet or removing the knee-pan
U+5256.0	fUTF8	剖
U+5256.0	fCNS11643	1-5448
U+5256.0	fBig5	ADE5
U+5256.0	fGB	0-464A
U+5256.0	fR/S	 18.8
U+5256.0	fTotalStrokes	 10
U+5256.0	fCangjie	YRLN
U+5256.0	fFourCorner	02600
U+5256.0	fHakka	 piau3;  peu3; po5; feu3; piu3; bok7; peu1
U+5256.0	fCantonese	fau2; pau2 (also)
U+5256.0	fMandarin	pou1; po3; fu3
U+5256.0	fEnglish	[1] cut, rip or tear open [2] explain; analyse; dissect
U+5257.0	fUTF8	剗
U+5257.0	fCNS11643	3-2F65
U+5257.0	fGB	3-3275
U+5257.0	fR/S	  18.8
U+5257.0	fTotalStrokes	 10
U+5257.0	fCangjie	IILN
U+5257.0	fFourCorner	52500
U+5257.0	fHakka	 can3; can1
U+5257.0	fCantonese	caan2; can2
U+5257.0	fMandarin	chan3
U+5257.0	fEnglish	 level off; trim; pare down
U+5258.0	fUTF8	剘
U+5258.0	fCNS11643	4-2B6E
U+5258.0	fGB	5-3444
U+5258.0	fR/S	 18.8
U+5258.0	fTotalStrokes	 10
U+5258.0	fCangjie	TCLN
U+5258.0	fFourCorner	42800
U+5258.0	fCantonese	kei4
U+5258.0	fMandarin	qi2
U+5259.0	fUTF8	剙
U+5259.0	fCNS11643	3-2F62
U+5259.0	fGB	E-233A
U+5259.0	fR/S	 18.8
U+5259.0	fTotalStrokes	 10
U+5259.0	fCangjie	TTSHI
U+5259.0	fFourCorner	83400
U+5259.0	fCantonese	cong3; cong1
U+5259.0	fMandarin	chuang4; chuang1
U+5259.0	fEnglish	 establish; create knife cut
U+525A.0	fUTF8	剚
U+525A.0	fCNS11643	2-2E2A
U+525A.0	fBig5	D0E7
U+525A.0	fGB	3-327E
U+525A.0	fR/S	 18.8
U+525A.0	fTotalStrokes	 10
U+525A.0	fCangjie	JNLN
U+525A.0	fFourCorner	52000
U+525A.0	fHakka	 zih5; sii5; sii6; zau3; zii5; zih3; ci5
U+525A.0	fCantonese	zi3; zi6; si6
U+525A.0	fMandarin	zi4
U+525A.0	fEnglish	 plant on the ground; stab
U+525B.0	fUTF8	剛
U+525B.0	fCNS11643	1-544B
U+525B.0	fBig5	ADE8
U+525B.0	fGB	1-3855
U+525B.0	fR/S	 18.8
U+525B.0	fTotalStrokes	 10
U+525B.0	fCangjie	BULN
U+525B.0	fFourCorner	72200
U+525B.0	fHakka	  gong1; giong1
U+525B.0	fCantonese	gong1
U+525B.0	fMandarin	gang1
U+525B.0	fEnglish	[1] hard; tough; unyielding; inflexible; firm; rigid; strong; indomitable [2] just now [3] just; exactly [4] barely; only
U+525C.0	fUTF8	剜
U+525C.0	fCNS11643	1-5449
U+525C.0	fBig5	ADE6
U+525C.0	fGB	0-5860
U+525C.0	fR/S	 18.8
U+525C.0	fTotalStrokes	 10
U+525C.0	fCangjie	JULN
U+525C.0	fFourCorner	32200
U+525C.0	fHakka	 wan1; wat7; won3; wan3; van1
U+525C.0	fCantonese	wun1; wun2 (also)
U+525C.0	fMandarin	wan1
U+525C.0	fEnglish	to cut out; carve out; scoop out; gouge out
U+525D.0	fUTF8	剝
U+525D.0	fCNS11643	1-544C
U+525D.0	fBig5	ADE9
U+525D.0	fGB	E-233B
U+525D.0	fR/S	 18.8
U+525D.0	fTotalStrokes	 10
U+525D.0	fCangjie	VELN
U+525D.0	fFourCorner	22100
U+525D.0	fHakka	  bok7
U+525D.0	fCantonese	mok1; pok3; bok1
U+525D.0	fMandarin	bo1; pu2
U+525D.0	fEnglish	 peel; peel off; strip; skin; make bare; shell
U+525D.1	fUTF8	剝
U+525D.1	fCNS11643	1-544C
U+525D.1	fBig5	ADE9
U+525D.1	fGB	E-233B
U+525D.1	fR/S	 18.8
U+525D.1	fTotalStrokes	 10
U+525D.1	fCangjie	VELN
U+525D.1	fFourCorner	22100
U+525D.1	fMandarin	bao1
U+525D.1	fEnglish	 strip
U+525E.0	fUTF8	剞
U+525E.0	fCNS11643	2-2E2C
U+525E.0	fBig5	D0E9
U+525E.0	fGB	0-585E
U+525E.0	fR/S	 18.8
U+525E.0	fTotalStrokes	 10
U+525E.0	fCangjie	KRLN
U+525E.0	fFourCorner	42600
U+525E.0	fHakka	 gi3; ji3; ri3
U+525E.0	fCantonese	gei1; gei2 (also)
U+525E.0	fMandarin	ji1; ji3; yi3
U+525E.0	fEnglish	 carving or engraving knife grave
U+525F.0	fUTF8	剟
U+525F.0	fCNS11643	2-2E2D
U+525F.0	fBig5	D0EA
U+525F.0	fGB	3-3323
U+525F.0	fR/S	 18.8
U+525F.0	fTotalStrokes	 10
U+525F.0	fCangjie	EEEEN
U+525F.0	fFourCorner	82400
U+525F.0	fHakka	  dot7
U+525F.0	fCantonese	zyut3; zyut1
U+525F.0	fMandarin	duo1; duo2; chuo1
U+525F.0	fEnglish	 prick cut blocks; engrave
U+5260.0	fUTF8	剠
U+5260.0	fCNS11643	3-2F60
U+5260.0	fGB	3-3321
U+5260.0	fR/S	 18.8
U+5260.0	fTotalStrokes	 10
U+5260.0	fCangjie	YFLN
U+5260.0	fFourCorner	02900
U+5260.0	fHakka	  liong5;  gin1
U+5260.0	fCantonese	loek6; king4
U+5260.0	fMandarin	qing2; le4
U+5260.0	fEnglish	brand
U+5261.0	fUTF8	剡
U+5261.0	fCNS11643	2-2E29
U+5261.0	fBig5	D0E6
U+5261.0	fGB	0-585F
U+5261.0	fR/S	 18.8
U+5261.0	fTotalStrokes	 10
U+5261.0	fCangjie	FFLN
U+5261.0	fFourCorner	92800
U+5261.0	fHakka	  jam3; ram3
U+5261.0	fCantonese	jim5; sim6
U+5261.0	fMandarin	yan3; shan4
U+5261.0	fEnglish	 sharp; sharp-pointed sharpen
U+5262.0	fUTF8	剢
U+5262.0	fCNS11643	2-2E2F
U+5262.0	fBig5	D0EC
U+5262.0	fGB	5-3448
U+5262.0	fR/S	 18.8
U+5262.0	fTotalStrokes	 10
U+5262.0	fCangjie	XMOLN
U+5262.0	fFourCorner	12200
U+5262.0	fCantonese	duk1; doek3
U+5262.0	fMandarin	zhuo2
U+5263.0	fUTF8	剣
U+5263.0	fCNS11643	3-2F66
U+5263.0	fGB	E-233C
U+5263.0	fR/S	 18.8
U+5263.0	fTotalStrokes	 10
U+5263.0	fCangjie	XOOLN
U+5263.0	fFourCorner	82400
U+5263.0	fCantonese	gim3
U+5263.0	fMandarin	jian4
U+5263.0	fEnglish	sword; dagger; saber
U+5264.0	fUTF8	剤
U+5264.0	fCNS11643	3-2F61
U+5264.0	fGB	E-233D
U+5264.0	fR/S	 18.8
U+5264.0	fTotalStrokes	 10
U+5264.0	fCangjie	YLLN
U+5264.0	fFourCorner	02200
U+5264.0	fCantonese	zai1
U+5264.0	fMandarin	ji4; zui1
U+5264.0	fEnglish	medicinal preparation
U+5265.0	fUTF8	剥
U+5265.0	fCNS11643	3-2F63
U+5265.0	fGB	0-307E
U+5265.0	fR/S	 18.8
U+5265.0	fTotalStrokes	 10
U+5265.0	fCangjie	NELN
U+5265.0	fFourCorner	22100
U+5265.0	fHakka	 bok7
U+5265.0	fCantonese	mok1; pok3; bok1
U+5265.0	fMandarin	bo1; bao1; pu1
U+5265.0	fEnglish	peel
U+5266.0	fUTF8	剦
U+5266.0	fCNS11643	F-2E2D
U+5266.0	fGB	E-233E
U+5266.0	fR/S	 18.8
U+5266.0	fTotalStrokes	 10
U+5266.0	fCangjie	KULN
U+5266.0	fHakka	  jam1; ram1
U+5266.0	fCantonese	jim2; jim1
U+5266.0	fMandarin	yan1
U+5266.0	fEnglish	 castrate
U+5267.0	fUTF8	剧
U+5267.0	fCNS11643	3-2F64
U+5267.0	fGB	0-3E67
U+5267.0	fR/S	 18.8
U+5267.0	fTotalStrokes	 10
U+5267.0	fCangjie	SRLN
U+5267.0	fFourCorner	72200
U+5267.0	fHakka	 kiak8
U+5267.0	fCantonese	kek6
U+5267.0	fMandarin	ju4; ji2
U+5267.0	fEnglish	simplified form of &#x5287;
U+5268.0	fUTF8	剨
U+5268.0	fCNS11643	3-3450
U+5268.0	fGB	5-344E
U+5268.0	fR/S	  18.9
U+5268.0	fTotalStrokes	 11
U+5268.0	fCangjie	QRLN
U+5268.0	fFourCorner	52600
U+5268.0	fHakka	 wak8
U+5268.0	fCantonese	waak6
U+5268.0	fMandarin	huo4
U+5268.0	fEnglish	sound of something being smashed
U+5269.0	fUTF8	剩
U+5269.0	fCNS11643	1-5E37
U+5269.0	fBig5	B3D1
U+5269.0	fGB	0-4A23
U+5269.0	fR/S	 18.10
U+5269.0	fTotalStrokes	 12
U+5269.0	fCangjie	HPLN
U+5269.0	fFourCorner	22900
U+5269.0	fHakka	 sin3;  shin5; chin5; sin5; sen5; cen5; jin3; chun1; cun1; jin5
U+5269.0	fCantonese	sing6; zing6 (also)
U+5269.0	fMandarin	sheng4
U+5269.0	fEnglish	 remain; be left over; in excess; leftovers; residue(s); remains; remainder; surplus
U+526A.0	fUTF8	剪
U+526A.0	fCNS11643	1-592A
U+526A.0	fBig5	B0C5
U+526A.0	fGB	0-3C74
U+526A.0	fR/S	 18.9
U+526A.0	fTotalStrokes	 11
U+526A.0	fCangjie	TBNH
U+526A.0	fFourCorner	80227
U+526A.0	fHakka	 zien3; zen3; tsian3
U+526A.0	fCantonese	zin2
U+526A.0	fMandarin	jian3
U+526A.0	fEnglish	[1] cut or clip with scissors; cut; shear; trim [2] annihilate; destroy completely
U+526B.0	fUTF8	剫
U+526B.0	fCNS11643	2-336C
U+526B.0	fBig5	D469
U+526B.0	fGB	3-3327
U+526B.0	fR/S	 18.9
U+526B.0	fTotalStrokes	 11
U+526B.0	fCangjie	IELN
U+526B.0	fFourCorner	02200
U+526B.0	fHakka	  tok8
U+526B.0	fCantonese	dok6
U+526B.0	fMandarin	duo2; du4
U+526B.0	fEnglish	[1] cut; hew [2] work in wood
U+526C.0	fUTF8	剬
U+526C.0	fCNS11643	2-336E
U+526C.0	fBig5	D46B
U+526C.0	fGB	3-3325
U+526C.0	fR/S	 18.9
U+526C.0	fTotalStrokes	 11
U+526C.0	fCangjie	UBLN
U+526C.0	fFourCorner	22200
U+526C.0	fHakka	  don1
U+526C.0	fCantonese	zai3; tyun4; dyun1
U+526C.0	fMandarin	duan1; zhi4; tuan2; zhuan3
U+526C.0	fEnglish	[1] cut; trim [2] order
U+526D.0	fUTF8	剭
U+526D.0	fCNS11643	2-336D
U+526D.0	fBig5	D46A
U+526D.0	fGB	3-3329
U+526D.0	fR/S	 18.9
U+526D.0	fTotalStrokes	 11
U+526D.0	fCangjie	SGLN
U+526D.0	fFourCorner	72200
U+526D.0	fHakka	 wuk7; vuk7
U+526D.0	fCantonese	uk1; nguk1 (also)
U+526D.0	fMandarin	wu1
U+526D.0	fEnglish	 put to death
U+526E.0	fUTF8	剮
U+526E.0	fCNS11643	2-336F
U+526E.0	fBig5	D46C
U+526E.0	fGB	1-3950
U+526E.0	fR/S	 18.9
U+526E.0	fTotalStrokes	 11
U+526E.0	fCangjie	BBLN
U+526E.0	fFourCorner	72200
U+526E.0	fHakka	  gwa3; wa1; ga3; kua3
U+526E.0	fCantonese	gwaa2; waa2 (also)
U+526E.0	fMandarin	gua3
U+526E.0	fEnglish	cut; cut flesh from bones
U+526F.0	fUTF8	副
U+526F.0	fCNS11643	1-592B
U+526F.0	fBig5	B0C6
U+526F.0	fGB	0-3831
U+526F.0	fR/S	 18.9
U+526F.0	fTotalStrokes	 11
U+526F.0	fCangjie	MWLN
U+526F.0	fFourCorner	12600
U+526F.0	fHakka	  fu5
U+526F.0	fCantonese	fu3; pik1
U+526F.0	fMandarin	fu4; bi4; fou4
U+526F.0	fEnglish	[1] assist [2] secondary; auxiliary; subsidiary; supplement [3] deputy; assistant [4] set
U+5270.0	fUTF8	剰
U+5270.0	fCNS11643	3-3A23
U+5270.0	fGB	E-233F
U+5270.0	fR/S	 18.9
U+5270.0	fTotalStrokes	 12; 11
U+5270.0	fCangjie	HTLN
U+5270.0	fFourCorner	22900
U+5270.0	fCantonese	sing6
U+5270.0	fMandarin	sheng4
U+5270.0	fEnglish	leftovers; residue; remains
U+5271.0	fUTF8	剱
U+5271.0	fGB	E-2340
U+5271.0	fR/S	 18.9
U+5271.0	fTotalStrokes	 11
U+5271.0	fCangjie	XOOSK
U+5271.0	fCantonese	gim3
U+5271.0	fMandarin	jian4
U+5271.0	fEnglish	sword; dagger; saber
U+5272.0	fUTF8	割
U+5272.0	fCNS11643	1-5E34
U+5272.0	fBig5	B3CE
U+5272.0	fGB	0-386E
U+5272.0	fR/S	 18.10
U+5272.0	fTotalStrokes	 12
U+5272.0	fCangjie	JRLN
U+5272.0	fFourCorner	32600
U+5272.0	fHakka	  got7; gok7
U+5272.0	fCantonese	got3
U+5272.0	fMandarin	ge1
U+5272.0	fEnglish	 cut; divide; partition; sever
U+5273.0	fUTF8	剳
U+5273.0	fCNS11643	3-3A21
U+5273.0	fGB	E-2341
U+5273.0	fR/S	 18.10
U+5273.0	fTotalStrokes	 12
U+5273.0	fCangjie	TRLN
U+5273.0	fFourCorner	42600
U+5273.0	fHakka	  dap7
U+5273.0	fCantonese	zaat3; zaap3 (also); zap3
U+5273.0	fMandarin	da2; zha1; zha2
U+5273.0	fEnglish	 brief note official communique
U+5274.0	fUTF8	剴
U+5274.0	fCNS11643	1-5E35
U+5274.0	fBig5	B3CF
U+5274.0	fGB	1-585C
U+5274.0	fR/S	 18.10
U+5274.0	fTotalStrokes	 12
U+5274.0	fCangjie	UTLN
U+5274.0	fFourCorner	22100
U+5274.0	fHakka	  koi3;  kai3
U+5274.0	fCantonese	hoi2; goi1
U+5274.0	fMandarin	kai3; sheng4
U+5274.0	fEnglish	[1] sickle [2] thoroughly; clearly
U+5275.0	fUTF8	創
U+5275.0	fCNS11643	1-5E36
U+5275.0	fBig5	B3D0
U+5275.0	fGB	1-3434
U+5275.0	fR/S	 18.10
U+5275.0	fTotalStrokes	 12
U+5275.0	fCangjie	ORLN
U+5275.0	fFourCorner	82600
U+5275.0	fHakka	  cong1;  cong3; cong6; cong5
U+5275.0	fCantonese	cong3; cong1
U+5275.0	fMandarin	chuang4; chuan4; qiang1; tiao3
U+5275.0	fEnglish	[1] start; establish; begin; initiate; found; create [2] original; unprecedented
U+5275.1	fUTF8	創
U+5275.1	fCNS11643	1-5E36
U+5275.1	fBig5	B3D0
U+5275.1	fGB	1-3434
U+5275.1	fR/S	 18.10
U+5275.1	fTotalStrokes	 12
U+5275.1	fCangjie	ORLN
U+5275.1	fFourCorner	82600
U+5275.1	fMandarin	chuang1; chuan1
U+5275.1	fEnglish	[1] wound [2] sore; boil; ulcer
U+5276.0	fUTF8	剶
U+5276.0	fCNS11643	3-3451
U+5276.0	fGB	3-332A
U+5276.0	fR/S	  18.9
U+5276.0	fTotalStrokes	 11
U+5276.0	fCangjie	VOLN
U+5276.0	fFourCorner	22200
U+5276.0	fHakka	 chon1; con1
U+5276.0	fCantonese	cyun1
U+5276.0	fMandarin	juan1
U+5276.0	fEnglish	 prune a tree
U+5277.0	fUTF8	剷
U+5277.0	fCNS11643	1-6337
U+5277.0	fBig5	B6D0
U+5277.0	fGB	E-2342
U+5277.0	fR/S	 18.11
U+5277.0	fTotalStrokes	 13
U+5277.0	fCangjie	YMLN
U+5277.0	fFourCorner	02200
U+5277.0	fHakka	 can3
U+5277.0	fCantonese	caan2
U+5277.0	fMandarin	chan3; chan4
U+5277.0	fEnglish	[1] shovel [2] shovel; level off; raze to the ground
U+5278.0	fUTF8	剸
U+5278.0	fCNS11643	2-416C
U+5278.0	fBig5	DCC7
U+5278.0	fGB	3-3273
U+5278.0	fR/S	 18.11
U+5278.0	fTotalStrokes	 13
U+5278.0	fCangjie	JILN
U+5278.0	fFourCorner	52300
U+5278.0	fHakka	  ton2; zhon1; zhon3; zon3; chon3
U+5278.0	fCantonese	tyun4; zyun1
U+5278.0	fMandarin	tuan2; zhuan1; zhuan3; zhuan4
U+5278.0	fEnglish	cut
U+5279.0	fUTF8	剹
U+5279.0	fCNS11643	3-4041
U+5279.0	fGB	E-2343
U+5279.0	fR/S	  18.11
U+5279.0	fTotalStrokes	 13
U+5279.0	fCangjie	SHLN
U+5279.0	fFourCorner	12200
U+5279.0	fHakka	 liuk8
U+5279.0	fCantonese	luk6
U+5279.0	fMandarin	lu4
U+527A.0	fUTF8	剺
U+527A.0	fCNS11643	2-416B
U+527A.0	fBig5	DCC6
U+527A.0	fGB	3-4744
U+527A.0	fR/S	 18.11
U+527A.0	fTotalStrokes	 13
U+527A.0	fCangjie	JKMSH
U+527A.0	fFourCorner	58227
U+527A.0	fCantonese	lei4; lai5 (also)
U+527A.0	fMandarin	li2
U+527A.0	fEnglish	mark
U+527B.0	fUTF8	剻
U+527B.0	fCNS11643	2-416D
U+527B.0	fBig5	DCC8
U+527B.0	fGB	5-3460
U+527B.0	fR/S	 18.11
U+527B.0	fTotalStrokes	 13
U+527B.0	fCangjie	UBLN
U+527B.0	fFourCorner	22200
U+527B.0	fCantonese	pang2; fau4
U+527B.0	fMandarin	fou2; fou1
U+527C.0	fUTF8	剼
U+527C.0	fCNS11643	2-416E
U+527C.0	fBig5	DCC9
U+527C.0	fGB	5-344D
U+527C.0	fR/S	 18.11
U+527C.0	fTotalStrokes	 13
U+527C.0	fCangjie	IHLN
U+527C.0	fFourCorner	22200
U+527C.0	fCantonese	saam1
U+527C.0	fMandarin	shan1
U+527D.0	fUTF8	剽
U+527D.0	fCNS11643	1-6338
U+527D.0	fBig5	B6D1
U+527D.0	fGB	0-5862
U+527D.0	fR/S	 18.11
U+527D.0	fTotalStrokes	 13
U+527D.0	fCangjie	MFLN
U+527D.0	fFourCorner	12900
U+527D.0	fHakka	 piau1;  piau3;  piau5
U+527D.0	fCantonese	piu3; piu5 (also); piu4 (also)
U+527D.0	fMandarin	piao1; piao4; piao2; biao3; biao1
U+527D.0	fEnglish	[1] rob; plunder; steal [2] agile; fast
U+527E.0	fUTF8	剾
U+527E.0	fCNS11643	3-4040
U+527E.0	fGB	3-3274
U+527E.0	fR/S	  18.11
U+527E.0	fTotalStrokes	 13
U+527E.0	fCangjie	SRLN
U+527E.0	fFourCorner	72700
U+527E.0	fHakka	 ki1;  eu1
U+527E.0	fCantonese	kau1
U+527E.0	fMandarin	kou1
U+527E.0	fEnglish	 pick up with knife
U+527F.0	fUTF8	剿
U+527F.0	fCNS11643	1-6336
U+527F.0	fBig5	B6CF
U+527F.0	fGB	0-3D4B
U+527F.0	fR/S	 18.11; 18.14
U+527F.0	fTotalStrokes	 13
U+527F.0	fCangjie	VDLN
U+527F.0	fFourCorner	22900
U+527F.0	fHakka	 ziau3; giau3; ziau1
U+527F.0	fCantonese	ziu2; caau1
U+527F.0	fMandarin	jiao3; chao1; chao4
U+527F.0	fEnglish	 destroy; exterminate; annihilate; stamp out; put down
U+5280.0	fUTF8	劀
U+5280.0	fCNS11643	2-4928
U+5280.0	fBig5	E141
U+5280.0	fGB	5-3469
U+5280.0	fR/S	 18.12
U+5280.0	fTotalStrokes	 14
U+5280.0	fCangjie	NBLN
U+5280.0	fFourCorner	12200
U+5280.0	fHakka	 kwat7; git7
U+5280.0	fCantonese	gwaat3; wat6
U+5280.0	fMandarin	gua1
U+5280.0	fEnglish	 cut away the proud flesh from an ulcer
U+5281.0	fUTF8	劁
U+5281.0	fCNS11643	2-4929
U+5281.0	fBig5	E142
U+5281.0	fGB	0-5864
U+5281.0	fR/S	 18.12
U+5281.0	fTotalStrokes	 14
U+5281.0	fCangjie	OFLN
U+5281.0	fFourCorner	22300
U+5281.0	fHakka	  ziau1;  ziau5
U+5281.0	fCantonese	ciu4; ciu1
U+5281.0	fMandarin	qiao1; qiao2; qiao4
U+5281.0	fEnglish	[1] cut or bite in two [2] reap
U+5282.0	fUTF8	劂
U+5282.0	fCNS11643	1-6823
U+5282.0	fBig5	B9BB
U+5282.0	fGB	0-5863
U+5282.0	fR/S	 18.12
U+5282.0	fTotalStrokes	 14
U+5282.0	fCangjie	MOLN
U+5282.0	fFourCorner	72200
U+5282.0	fHakka	 kiet7; ket7
U+5282.0	fCantonese	kyut3; gyut3 (also)
U+5282.0	fMandarin	jue2; jue4
U+5282.0	fEnglish	 chisel for engraving engrave
U+5283.0	fUTF8	劃
U+5283.0	fCNS11643	1-6822
U+5283.0	fBig5	B9BA
U+5283.0	fGB	1-3B2E
U+5283.0	fR/S	 18.12
U+5283.0	fTotalStrokes	 14
U+5283.0	fCangjie	LMLN
U+5283.0	fFourCorner	52100
U+5283.0	fHakka	 wak8; wat8; vak8; wak7
U+5283.0	fCantonese	waak6
U+5283.0	fMandarin	hua4; huai5; jiao3
U+5283.0	fEnglish	[1] unify [2] lay boundaries [3] draw a line; mark; delineate [4] plan; design [5] set aside; divide [6] stroke (of a Chinese character)
U+5283.1	fUTF8	劃
U+5283.1	fCNS11643	1-6822
U+5283.1	fBig5	B9BA
U+5283.1	fGB	1-3B2E
U+5283.1	fR/S	 18.12
U+5283.1	fTotalStrokes	 14
U+5283.1	fCangjie	LMLN
U+5283.1	fFourCorner	52100
U+5283.1	fMandarin	hua2
U+5283.1	fEnglish	 cut
U+5284.0	fUTF8	劄
U+5284.0	fCNS11643	2-4C61
U+5284.0	fBig5	E35A
U+5284.0	fGB	E-2344
U+5284.0	fR/S	 118.8; 18.12
U+5284.0	fTotalStrokes	 14
U+5284.0	fCangjie	HRLN
U+5284.0	fFourCorner	88623
U+5284.0	fHakka	 zap7; zhap7
U+5284.0	fCantonese	zaap3 (also); zaat3
U+5284.0	fMandarin	zha2; zha1; da2
U+5284.0	fEnglish	 brief note official communique
U+5285.0	fUTF8	劅
U+5285.0	fCNS11643	4-4925
U+5285.0	fGB	E-2345
U+5285.0	fR/S	 18.13
U+5285.0	fTotalStrokes	 15
U+5285.0	fCangjie	WILN
U+5285.0	fFourCorner	62100
U+5285.0	fCantonese	zoek3
U+5285.0	fMandarin	zhuo4; zhuo2; zhu2
U+5286.0	fUTF8	劆
U+5286.0	fCNS11643	3-4B31
U+5286.0	fGB	E-2346
U+5286.0	fR/S	 18.13
U+5286.0	fTotalStrokes	 15
U+5286.0	fCangjie	ICLN
U+5286.0	fFourCorner	02200
U+5286.0	fCantonese	lim4
U+5286.0	fMandarin	lian2
U+5286.0	fEnglish	sickle
U+5287.0	fUTF8	劇
U+5287.0	fCNS11643	1-6C2A
U+5287.0	fBig5	BC40
U+5287.0	fGB	1-3E67
U+5287.0	fR/S	 18.13
U+5287.0	fTotalStrokes	 15
U+5287.0	fCangjie	YOLN
U+5287.0	fFourCorner	22200
U+5287.0	fHakka	 kiak8;  kiak7; gi3; hiak7
U+5287.0	fCantonese	kek6
U+5287.0	fMandarin	ju4; ji2
U+5287.0	fEnglish	[1] drama; theatrical play; theatrical work; play; opera [2] intense; strenuous; acute; severe [3] play [4] a Chinese family name
U+5288.0	fUTF8	劈
U+5288.0	fCNS11643	1-6C2B
U+5288.0	fBig5	BC41
U+5288.0	fGB	0-457C
U+5288.0	fR/S	 18.13
U+5288.0	fTotalStrokes	 15
U+5288.0	fCangjie	SJSH
U+5288.0	fFourCorner	70227
U+5288.0	fHakka	  piak7; pit7; bit7; tiap8
U+5288.0	fCantonese	pik1; pek3 (also); pek6 (also)
U+5288.0	fMandarin	pi1
U+5288.0	fEnglish	[1] cleave; cut apart; split; rive; rend [2] wedge
U+5288.1	fUTF8	劈
U+5288.1	fCNS11643	1-6C2B
U+5288.1	fBig5	BC41
U+5288.1	fGB	0-457C
U+5288.1	fR/S	 18.13
U+5288.1	fTotalStrokes	 15
U+5288.1	fCangjie	SJSH
U+5288.1	fFourCorner	70227
U+5288.1	fMandarin	pi3
U+5288.1	fEnglish	 split; chop
U+5289.0	fUTF8	劉
U+5289.0	fCNS11643	1-6C2C
U+5289.0	fBig5	BC42
U+5289.0	fGB	1-4175
U+5289.0	fR/S	 18.13
U+5289.0	fTotalStrokes	 15
U+5289.0	fCangjie	HCLN
U+5289.0	fFourCorner	72100
U+5289.0	fHakka	  liu2; lieu2; liu3
U+5289.0	fCantonese	lau4; lau2 (also)
U+5289.0	fMandarin	liu2
U+5289.0	fEnglish	[1] kill; destroy [2] a Chinese family name
U+528A.0	fUTF8	劊
U+528A.0	fCNS11643	1-6C2E
U+528A.0	fBig5	BC44
U+528A.0	fGB	1-3974
U+528A.0	fR/S	 18.13
U+528A.0	fTotalStrokes	 15
U+528A.0	fCangjie	OALN
U+528A.0	fFourCorner	82600
U+528A.0	fHakka	  kwai5; kai5; kwai2; k'uai5
U+528A.0	fCantonese	kui2; kwui2 (also); gui3
U+528A.0	fMandarin	gui4; kuai4; gua1
U+528A.0	fEnglish	amputate; cut off
U+528B.0	fUTF8	劋
U+528B.0	fCNS11643	2-4F5A
U+528B.0	fBig5	E4F2
U+528B.0	fGB	3-3332
U+528B.0	fR/S	 18.13
U+528B.0	fTotalStrokes	 15
U+528B.0	fCangjie	RDLN
U+528B.0	fFourCorner	62900
U+528B.0	fHakka	 ziau3
U+528B.0	fCantonese	ziu2; caau1
U+528B.0	fMandarin	jiao3; zao3
U+528C.0	fUTF8	劌
U+528C.0	fCNS11643	2-4F5B
U+528C.0	fBig5	E4F3
U+528C.0	fGB	1-585B
U+528C.0	fR/S	 18.13
U+528C.0	fTotalStrokes	 15
U+528C.0	fCangjie	YHLN
U+528C.0	fFourCorner	12200
U+528C.0	fHakka	 gui5; kui5; kui3; kwui5; k'wui3
U+528C.0	fCantonese	gwai3
U+528C.0	fMandarin	gui4
U+528C.0	fEnglish	 cut; injure; stab; stick on
U+528D.0	fUTF8	劍
U+528D.0	fCNS11643	1-6C2D
U+528D.0	fBig5	BC43
U+528D.0	fGB	1-3D23
U+528D.0	fR/S	 18.13
U+528D.0	fTotalStrokes	 15
U+528D.0	fCangjie	OOLN
U+528D.0	fFourCorner	82800
U+528D.0	fHakka	  giam5
U+528D.0	fCantonese	gim3
U+528D.0	fMandarin	jian4
U+528D.0	fEnglish	 sword; dagger; saber
U+528E.0	fUTF8	劎
U+528E.0	fCNS11643	3-4B32
U+528E.0	fGB	E-2347
U+528E.0	fR/S	 18.13
U+528E.0	fTotalStrokes	 15
U+528E.0	fCangjie	OOSH
U+528E.0	fFourCorner	87820
U+528E.0	fCantonese	gim3
U+528E.0	fMandarin	jian4
U+528E.0	fEnglish	sword; dagger
U+528F.0	fUTF8	劏
U+528F.0	fGB	1-7C24
U+528F.0	fR/S	 18.13
U+528F.0	fTotalStrokes	 15
U+528F.0	fCangjie	FWLN
U+528F.0	fHakka	 tong1
U+528F.0	fCantonese	tong1
U+528F.0	fMandarin	tang1
U+528F.0	fEnglish	 butcher
U+5290.0	fUTF8	劐
U+5290.0	fCNS11643	3-5036
U+5290.0	fGB	0-5865
U+5290.0	fR/S	  18.14
U+5290.0	fTotalStrokes	 16
U+5290.0	fCangjie	TELN
U+5290.0	fFourCorner	42400
U+5290.0	fHakka	 wok7; vok7
U+5290.0	fCantonese	fok3; wok6 (also); wok3 (also); hap6
U+5290.0	fMandarin	huo1
U+5290.0	fEnglish	destroy
U+5291.0	fUTF8	劑
U+5291.0	fCNS11643	1-7038
U+5291.0	fBig5	BEAF
U+5291.0	fGB	1-3C41
U+5291.0	fR/S	 18.14
U+5291.0	fTotalStrokes	 16
U+5291.0	fCangjie	YXLN
U+5291.0	fFourCorner	02200
U+5291.0	fHakka	  zi5
U+5291.0	fCantonese	zai1; zai6
U+5291.0	fMandarin	ji4; zui1
U+5291.0	fEnglish	[1] dose (of medicine) [2] prepared medicines or drugs; medicinal preparation [3] prepare (medicines and drugs)
U+5292.0	fUTF8	劒
U+5292.0	fCNS11643	4-4F64
U+5292.0	fGB	E-2348
U+5292.0	fR/S	  18.14
U+5292.0	fTotalStrokes	 16
U+5292.0	fCangjie	OOSHI
U+5292.0	fFourCorner	87820
U+5292.0	fHakka	 giam5
U+5292.0	fCantonese	gim3
U+5292.0	fMandarin	jian4
U+5292.0	fEnglish	sword, dagger, saber
U+5293.0	fUTF8	劓
U+5293.0	fCNS11643	1-7039
U+5293.0	fBig5	BEB0
U+5293.0	fGB	0-5866
U+5293.0	fR/S	 18.14
U+5293.0	fTotalStrokes	 16
U+5293.0	fCangjie	HLLN
U+5293.0	fFourCorner	22200
U+5293.0	fHakka	 ngi5; ngi6
U+5293.0	fCantonese	ji6
U+5293.0	fMandarin	yi4
U+5293.0	fEnglish	 cut off nose (as a form of punishment in ancient China)
U+5294.0	fUTF8	劔
U+5294.0	fCNS11643	3-5037
U+5294.0	fGB	E-2349
U+5294.0	fR/S	 18.14
U+5294.0	fTotalStrokes	 16
U+5294.0	fCangjie	OOSK
U+5294.0	fFourCorner	87850
U+5294.0	fCantonese	gim3
U+5294.0	fMandarin	jian4
U+5294.0	fEnglish	sword; dagger; saber
U+5295.0	fUTF8	劕
U+5295.0	fCNS11643	4-556D
U+5295.0	fGB	5-344B
U+5295.0	fR/S	 18.15
U+5295.0	fTotalStrokes	 17; 16
U+5295.0	fCangjie	HCLN
U+5295.0	fFourCorner	72800
U+5295.0	fHakka	 zhit7; tsiit7
U+5295.0	fCantonese	zat1
U+5295.0	fMandarin	zhi2; zhi4
U+5295.0	fEnglish	[1] laminous [2] splendid
U+5296.0	fUTF8	劖
U+5296.0	fCNS11643	2-6539
U+5296.0	fBig5	F1ED
U+5296.0	fGB	3-3337
U+5296.0	fR/S	 18.17
U+5296.0	fTotalStrokes	 19
U+5296.0	fCangjie	NILN
U+5296.0	fFourCorner	22200
U+5296.0	fHakka	  cam2; cam1
U+5296.0	fCantonese	caam4; caam5
U+5296.0	fMandarin	chan2
U+5296.0	fEnglish	 make into mince cut
U+5297.0	fUTF8	劗
U+5297.0	fCNS11643	2-6B51
U+5297.0	fBig5	F5C3
U+5297.0	fGB	5-346E
U+5297.0	fR/S	 18.19
U+5297.0	fTotalStrokes	 21
U+5297.0	fCangjie	HCLN
U+5297.0	fFourCorner	22800
U+5297.0	fCantonese	zin2; zyun1
U+5297.0	fMandarin	cuan2; jian3
U+5298.0	fUTF8	劘
U+5298.0	fCNS11643	2-6B50
U+5298.0	fBig5	F5C2
U+5298.0	fGB	3-3339
U+5298.0	fR/S	 18.19
U+5298.0	fTotalStrokes	 21
U+5298.0	fCangjie	IYLN
U+5298.0	fFourCorner	02200
U+5298.0	fHakka	 mo2;  mi2
U+5298.0	fCantonese	mo4
U+5298.0	fMandarin	mo2; mi2
U+5298.0	fEnglish	make into mince
U+5299.0	fUTF8	劙
U+5299.0	fCNS11643	2-6E7E
U+5299.0	fBig5	F7D1
U+5299.0	fGB	3-333A
U+5299.0	fR/S	 18.21
U+5299.0	fTotalStrokes	 23
U+5299.0	fCangjie	VILN
U+5299.0	fFourCorner	22100
U+5299.0	fCantonese	lai5; lei4 (also); lai6; lei5
U+5299.0	fMandarin	li2; li4
U+5299.0	fEnglish	 divide; partition
U+529A.0	fUTF8	劚
U+529A.0	fCNS11643	3-602C
U+529A.0	fGB	5-3467
U+529A.0	fR/S	 18.21
U+529A.0	fTotalStrokes	 23
U+529A.0	fCangjie	SILN
U+529A.0	fFourCorner	72200
U+529A.0	fCantonese	zuk1
U+529A.0	fMandarin	zhu2; zhu3
U+529A.0	fEnglish	cut
U+529B.0	fUTF8	力
U+529B.0	fCNS11643	1-4430
U+529B.0	fBig5	A44F
U+529B.0	fGB	0-4126
U+529B.0	fR/S	 19.0
U+529B.0	fTotalStrokes	 2
U+529B.0	fCangjie	KS
U+529B.0	fFourCorner	40027
U+529B.0	fHakka	  lit8
U+529B.0	fCantonese	lik6; lik1 (also)
U+529B.0	fMandarin	li4
U+529B.0	fEnglish	[1] strength [2] force; power [3] ability; capability [4] vigor [5] vigorously; earnestly [6] do one's best
U+529B.1	fUTF8	力
U+529B.1	fCNS11643	1-4430
U+529B.1	fBig5	A44F
U+529B.1	fGB	0-4126
U+529B.1	fR/S	 19.0
U+529B.1	fTotalStrokes	 2
U+529B.1	fCangjie	KS
U+529B.1	fFourCorner	40027
U+529B.1	fMandarin	li5
U+529C.0	fUTF8	劜
U+529C.0	fCNS11643	3-2137
U+529C.0	fGB	5-346F
U+529C.0	fR/S	  19.1
U+529C.0	fTotalStrokes	 3
U+529C.0	fCangjie	KSU
U+529C.0	fFourCorner	42010
U+529C.0	fCantonese	aat3; ngaat3 (also)
U+529C.0	fMandarin	ya1; ya4
U+529D.0	fUTF8	劝
U+529D.0	fCNS11643	3-2154
U+529D.0	fGB	0-4830
U+529D.0	fR/S	 19.2
U+529D.0	fTotalStrokes	 4
U+529D.0	fCangjie	EKS
U+529D.0	fFourCorner	74420
U+529D.0	fHakka	 ken5; kian5; kien5
U+529D.0	fCantonese	hyun3
U+529D.0	fMandarin	quan4
U+529D.0	fEnglish	simplified form of &#x52F8;
U+529E.0	fUTF8	办
U+529E.0	fCNS11643	3-2153
U+529E.0	fGB	0-306C
U+529E.0	fR/S	 19.2
U+529E.0	fTotalStrokes	 4
U+529E.0	fCangjie	KSC
U+529E.0	fFourCorner	40330
U+529E.0	fHakka	 pan5
U+529E.0	fCantonese	baan6; baan2 (also)
U+529E.0	fMandarin	ban4
U+529E.0	fEnglish	simplified form of &#x8FA6;
U+529F.0	fUTF8	功
U+529F.0	fCNS11643	1-457C
U+529F.0	fBig5	A55C
U+529F.0	fGB	0-3926
U+529F.0	fR/S	 19.3
U+529F.0	fTotalStrokes	 5
U+529F.0	fCangjie	MKS
U+529F.0	fFourCorner	14127
U+529F.0	fHakka	  gung1
U+529F.0	fCantonese	gung1
U+529F.0	fMandarin	gong1
U+529F.0	fEnglish	[1] achievement; merit; accomplishment; good result; exploit [2] usefulness; effectiveness [3] function [4] work (physics)
U+52A0.0	fUTF8	加
U+52A0.0	fCNS11643	1-457B
U+52A0.0	fBig5	A55B
U+52A0.0	fGB	0-3C53
U+52A0.0	fR/S	 19.3
U+52A0.0	fTotalStrokes	 5
U+52A0.0	fCangjie	KSR
U+52A0.0	fFourCorner	46000
U+52A0.0	fHakka	  ga1
U+52A0.0	fCantonese	gaa1; gaa3 (also)
U+52A0.0	fMandarin	jia1
U+52A0.0	fEnglish	[1] plus; add (to) [2] increase; augment [3] append
U+52A1.0	fUTF8	务
U+52A1.0	fGB	0-4E71
U+52A1.0	fR/S	 34.2; 19.3
U+52A1.0	fTotalStrokes	 5
U+52A1.0	fCangjie	HEKS
U+52A1.0	fHakka	 vu5; mu5
U+52A1.0	fCantonese	mou6
U+52A1.0	fMandarin	wu4; mao1; mao4; wu3
U+52A1.0	fEnglish	simplified form of &#x52D9;
U+52A1.1	fUTF8	务
U+52A1.1	fGB	0-4E71
U+52A1.1	fR/S	 34.2; 19.3
U+52A1.1	fTotalStrokes	 5
U+52A1.1	fCangjie	HEKS
U+52A1.1	fMandarin	wu5
U+52A2.0	fUTF8	劢
U+52A2.0	fGB	0-5B3D
U+52A2.0	fR/S	 19.3
U+52A2.0	fTotalStrokes	 5
U+52A2.0	fCangjie	XMSKS
U+52A2.0	fHakka	 mai5; mai3
U+52A2.0	fCantonese	maai6
U+52A2.0	fMandarin	mai4
U+52A2.0	fEnglish	simplified form of &#x52F1;
U+52A3.0	fUTF8	劣
U+52A3.0	fCNS11643	1-4749
U+52A3.0	fBig5	A648
U+52A3.0	fGB	0-4153
U+52A3.0	fR/S	 19.4
U+52A3.0	fTotalStrokes	 6
U+52A3.0	fCangjie	FHKS
U+52A3.0	fFourCorner	90427
U+52A3.0	fHakka	  lot7
U+52A3.0	fCantonese	lyut3; lyut6 (also)
U+52A3.0	fMandarin	lie4
U+52A3.0	fEnglish	bad; inferior; mean; of low quality
U+52A4.0	fUTF8	劤
U+52A4.0	fCNS11643	4-2242
U+52A4.0	fGB	3-4C33
U+52A4.0	fR/S	 19.4
U+52A4.0	fTotalStrokes	 6
U+52A4.0	fCangjie	HLKS
U+52A4.0	fFourCorner	24220
U+52A4.0	fHakka	  gin5
U+52A4.0	fCantonese	ging6
U+52A4.0	fMandarin	jin4; jing4
U+52A4.0	fEnglish	strong
U+52A5.0	fUTF8	劥
U+52A5.0	fCNS11643	4-223F
U+52A5.0	fGB	E-234A
U+52A5.0	fR/S	  19.4
U+52A5.0	fTotalStrokes	 6
U+52A5.0	fCangjie	YNKS
U+52A5.0	fFourCorner	04220
U+52A5.0	fCantonese	haang1
U+52A5.0	fMandarin	keng1
U+52A6.0	fUTF8	劦
U+52A6.0	fCNS11643	2-217E
U+52A6.0	fBig5	C9C0
U+52A6.0	fGB	3-333C
U+52A6.0	fR/S	 19.4
U+52A6.0	fTotalStrokes	 6
U+52A6.0	fCangjie	KSKSS
U+52A6.0	fFourCorner	47427
U+52A6.0	fCantonese	hip6 (also); hip3
U+52A6.0	fMandarin	xie2
U+52A7.0	fUTF8	劧
U+52A7.0	fCNS11643	4-2241
U+52A7.0	fGB	E-234B
U+52A7.0	fR/S	  19.4
U+52A7.0	fTotalStrokes	 6
U+52A7.0	fCangjie	QKS
U+52A7.0	fFourCorner	24520
U+52A7.0	fCantonese	zi2
U+52A7.0	fMandarin	zhi3
U+52A8.0	fUTF8	动
U+52A8.0	fCNS11643	3-2328
U+52A8.0	fGB	0-362F
U+52A8.0	fR/S	 19.4
U+52A8.0	fTotalStrokes	 6
U+52A8.0	fCangjie	MIKS
U+52A8.0	fFourCorner	14720
U+52A8.0	fHakka	 tung1; tung5
U+52A8.0	fCantonese	dung6
U+52A8.0	fMandarin	dong4
U+52A8.0	fEnglish	simplified form of &#x52D5;
U+52A9.0	fUTF8	助
U+52A9.0	fCNS11643	1-4937
U+52A9.0	fBig5	A755
U+52A9.0	fGB	0-567A
U+52A9.0	fR/S	 19.5
U+52A9.0	fTotalStrokes	 7
U+52A9.0	fCangjie	BMKS
U+52A9.0	fFourCorner	74127
U+52A9.0	fHakka	  cu5; cih5; cu6; ze5; co5
U+52A9.0	fCantonese	zo6
U+52A9.0	fMandarin	zhu4
U+52A9.0	fEnglish	 help; aid; assist; help; assistance
U+52AA.0	fUTF8	努
U+52AA.0	fCNS11643	1-4938
U+52AA.0	fBig5	A756
U+52AA.0	fGB	0-452C
U+52AA.0	fR/S	 19.5
U+52AA.0	fTotalStrokes	 7
U+52AA.0	fCangjie	VEKS
U+52AA.0	fFourCorner	47427
U+52AA.0	fHakka	  nu3; lu3; lu2; nu5
U+52AA.0	fCantonese	nou5; lou5 (also)
U+52AA.0	fMandarin	nu3
U+52AA.0	fEnglish	[1] exert oneself; strive; make effort [2] protrude
U+52AB.0	fUTF8	劫
U+52AB.0	fCNS11643	1-4936
U+52AB.0	fBig5	A754
U+52AB.0	fGB	0-3D59
U+52AB.0	fR/S	 19.5
U+52AB.0	fTotalStrokes	 7
U+52AB.0	fCangjie	GIKS
U+52AB.0	fFourCorner	47720
U+52AB.0	fHakka	  giap7
U+52AB.0	fCantonese	gip3
U+52AB.0	fMandarin	jie2
U+52AB.0	fEnglish	[1] rob; plunder; take by force [2] sufferings; disaster; misfortunes
U+52AC.0	fUTF8	劬
U+52AC.0	fCNS11643	1-4939
U+52AC.0	fBig5	A757
U+52AC.0	fGB	0-5B3E
U+52AC.0	fR/S	 19.5
U+52AC.0	fTotalStrokes	 7
U+52AC.0	fCangjie	PRKS
U+52AC.0	fFourCorner	24627
U+52AC.0	fHakka	 ki1;  gi1; ki2
U+52AC.0	fCantonese	keoi4
U+52AC.0	fMandarin	qu2
U+52AC.0	fEnglish	labor; diligent; toil; labor incessantly
U+52AD.0	fUTF8	劭
U+52AD.0	fCNS11643	2-2331
U+52AD.0	fBig5	CA6F
U+52AD.0	fGB	0-5B3F
U+52AD.0	fR/S	 19.5
U+52AD.0	fTotalStrokes	 7
U+52AD.0	fCangjie	SRKS
U+52AD.0	fFourCorner	14627
U+52AD.0	fHakka	 chau5; sau2; shau5; cau5; shau2; sau5; seu2
U+52AD.0	fCantonese	siu6
U+52AD.0	fMandarin	shao4; qiao2; shao2
U+52AD.0	fEnglish	[1] encourage; urge [2] graceful; excellent; admirable; respectable
U+52AE.0	fUTF8	劮
U+52AE.0	fCNS11643	2-2332
U+52AE.0	fBig5	CA70
U+52AE.0	fGB	3-333D
U+52AE.0	fR/S	 19.5
U+52AE.0	fTotalStrokes	 7
U+52AE.0	fCangjie	HOKS
U+52AE.0	fFourCorner	24527
U+52AE.0	fCantonese	jat6
U+52AE.0	fMandarin	yi4
U+52AF.0	fUTF8	劯
U+52AF.0	fCNS11643	4-2346
U+52AF.0	fGB	E-234C
U+52AF.0	fR/S	 19.5
U+52AF.0	fTotalStrokes	 7
U+52AF.0	fCangjie	MRKS
U+52AF.0	fFourCorner	14620
U+52AF.0	fCantonese	zyu2
U+52AF.0	fMandarin	zhu1
U+52B0.0	fUTF8	劰
U+52B0.0	fCNS11643	4-2347
U+52B0.0	fGB	E-234D
U+52B0.0	fR/S	  19.5
U+52B0.0	fTotalStrokes	 7
U+52B0.0	fCangjie	HAKS
U+52B0.0	fFourCorner	24620
U+52B0.0	fCantonese	mok6
U+52B0.0	fMandarin	miao3; mo4
U+52B1.0	fUTF8	励
U+52B1.0	fCNS11643	4-2348
U+52B1.0	fGB	0-4078
U+52B1.0	fR/S	 19.5
U+52B1.0	fTotalStrokes	 7
U+52B1.0	fCangjie	MSKS
U+52B1.0	fFourCorner	74220
U+52B1.0	fHakka	 li5
U+52B1.0	fCantonese	lai6
U+52B1.0	fMandarin	li4
U+52B1.0	fEnglish	simplified form of &#x52F5;
U+52B2.0	fUTF8	劲
U+52B2.0	fGB	0-3E22
U+52B2.0	fR/S	 19.5
U+52B2.0	fTotalStrokes	 7
U+52B2.0	fCangjie	NMKS
U+52B2.0	fHakka	 gin5; gin1
U+52B2.0	fCantonese	ging6; ging3
U+52B2.0	fMandarin	jing4; jin4
U+52B2.0	fEnglish	simplified form of &#x52C1;
U+52B3.0	fUTF8	劳
U+52B3.0	fCNS11643	3-2463
U+52B3.0	fGB	0-404D
U+52B3.0	fR/S	 19.5
U+52B3.0	fTotalStrokes	 7
U+52B3.0	fCangjie	TBKS
U+52B3.0	fFourCorner	44420
U+52B3.0	fHakka	  lau2; lau5; lau3
U+52B3.0	fCantonese	lou4; lou2 (also); lou6
U+52B3.0	fMandarin	lao2; lao4
U+52B3.0	fEnglish	simplified form of &#x52DE;
U+52B4.0	fUTF8	労
U+52B4.0	fCNS11643	3-2462
U+52B4.0	fGB	E-234E
U+52B4.0	fR/S	 19.5
U+52B4.0	fTotalStrokes	 7
U+52B4.0	fCangjie	FBKS
U+52B4.0	fFourCorner	90420
U+52B4.0	fCantonese	lou4
U+52B4.0	fMandarin	lao2; lao4
U+52B4.0	fEnglish	labor; toil; do manual work
U+52B5.0	fUTF8	劵
U+52B5.0	fCNS11643	3-2761
U+52B5.0	fGB	E-234F
U+52B5.0	fR/S	  19.6
U+52B5.0	fTotalStrokes	 8
U+52B5.0	fCangjie	FQKS
U+52B5.0	fFourCorner	90420
U+52B5.0	fHakka	 kian3
U+52B5.0	fCantonese	hyun3; gyun3 (also)
U+52B5.0	fMandarin	juan4; quan4
U+52B5.0	fEnglish	 certificate; ticket title deeds
U+52B6.0	fUTF8	劶
U+52B6.0	fCNS11643	5-253A
U+52B6.0	fGB	E-2350
U+52B6.0	fR/S	  19.6
U+52B6.0	fTotalStrokes	 8
U+52B6.0	fCangjie	HRKS
U+52B6.0	fFourCorner	74220
U+52B6.0	fCantonese	hau2
U+52B6.0	fMandarin	kou3
U+52B7.0	fUTF8	劷
U+52B7.0	fCNS11643	5-2537
U+52B7.0	fGB	E-2351
U+52B7.0	fR/S	  19.6
U+52B7.0	fTotalStrokes	 8
U+52B7.0	fCangjie	TQKS
U+52B7.0	fFourCorner	84520
U+52B7.0	fCantonese	lik6
U+52B7.0	fMandarin	yang2
U+52B8.0	fUTF8	劸
U+52B8.0	fCNS11643	5-2538
U+52B8.0	fGB	E-2352
U+52B8.0	fR/S	  19.6
U+52B8.0	fTotalStrokes	 8
U+52B8.0	fCangjie	GGKS
U+52B8.0	fFourCorner	44120
U+52B8.0	fCantonese	aai1; ngaai1 (also)
U+52B8.0	fMandarin	wa1
U+52B9.0	fUTF8	効
U+52B9.0	fCNS11643	3-2760
U+52B9.0	fGB	8-2C7B
U+52B9.0	fR/S	  19.6
U+52B9.0	fTotalStrokes	 8
U+52B9.0	fCangjie	YKKS
U+52B9.0	fFourCorner	04420
U+52B9.0	fHakka	  hau5;  hau3; hau6
U+52B9.0	fCantonese	haau6
U+52B9.0	fMandarin	xiao4
U+52B9.0	fEnglish	 efficacious; effectiveness toil; serve in the army
U+52BA.0	fUTF8	劺
U+52BA.0	fCNS11643	5-253B
U+52BA.0	fGB	3-3342
U+52BA.0	fR/S	  19.6
U+52BA.0	fTotalStrokes	 8
U+52BA.0	fCangjie	IQKS
U+52BA.0	fFourCorner	24520
U+52BA.0	fCantonese	mau4
U+52BA.0	fMandarin	mou2
U+52BB.0	fUTF8	劻
U+52BB.0	fCNS11643	1-4C34
U+52BB.0	fBig5	A8F1
U+52BB.0	fGB	3-333F
U+52BB.0	fR/S	 19.6
U+52BB.0	fTotalStrokes	 8
U+52BB.0	fCangjie	SGKS
U+52BB.0	fFourCorner	74727
U+52BB.0	fHakka	 kiong1
U+52BB.0	fCantonese	hong1
U+52BB.0	fMandarin	kuang1
U+52BB.0	fEnglish	flurried; be in flurry; zealous
U+52BC.0	fUTF8	劼
U+52BC.0	fCNS11643	2-2555
U+52BC.0	fBig5	CBD5
U+52BC.0	fGB	3-3340
U+52BC.0	fR/S	 19.6
U+52BC.0	fTotalStrokes	 8
U+52BC.0	fCangjie	GRKS
U+52BC.0	fFourCorner	44627
U+52BC.0	fHakka	 kat7;  giet7; get7
U+52BC.0	fCantonese	kit3
U+52BC.0	fMandarin	jie2; qia2
U+52BC.0	fEnglish	be discreet; prudent; cautious
U+52BD.0	fUTF8	劽
U+52BD.0	fCNS11643	4-2550
U+52BD.0	fGB	E-2353
U+52BD.0	fR/S	  19.6
U+52BD.0	fTotalStrokes	 8; 7
U+52BD.0	fCangjie	MNKS
U+52BD.0	fFourCorner	12420
U+52BD.0	fCantonese	lit6
U+52BD.0	fMandarin	lie4
U+52BE.0	fUTF8	劾
U+52BE.0	fCNS11643	1-4C33
U+52BE.0	fBig5	A8F0
U+52BE.0	fGB	0-5B40
U+52BE.0	fR/S	 19.6
U+52BE.0	fTotalStrokes	 8
U+52BE.0	fCangjie	YOKS
U+52BE.0	fFourCorner	04827
U+52BE.0	fHakka	  het8; kai3; het7
U+52BE.0	fCantonese	hat6
U+52BE.0	fMandarin	he2; kai1; xie3; xie4
U+52BE.0	fEnglish	 accuse; impeach; charge
U+52BF.0	fUTF8	势
U+52BF.0	fCNS11643	3-2762
U+52BF.0	fGB	0-4A46
U+52BF.0	fR/S	 19.6
U+52BF.0	fTotalStrokes	 8
U+52BF.0	fCangjie	QIKS
U+52BF.0	fFourCorner	55420
U+52BF.0	fHakka	 s'e5; shi5
U+52BF.0	fCantonese	sai3
U+52BF.0	fMandarin	shi4
U+52BF.0	fEnglish	simplified form of &#x52E2;
U+52BF.1	fUTF8	势
U+52BF.1	fCNS11643	3-2762
U+52BF.1	fGB	0-4A46
U+52BF.1	fR/S	 19.6
U+52BF.1	fTotalStrokes	 8
U+52BF.1	fCangjie	QIKS
U+52BF.1	fFourCorner	55420
U+52BF.1	fMandarin	shi5
U+52C0.0	fUTF8	勀
U+52C0.0	fCNS11643	2-2934
U+52C0.0	fBig5	CDF2
U+52C0.0	fGB	5-3473
U+52C0.0	fR/S	 19.7
U+52C0.0	fTotalStrokes	 9
U+52C0.0	fCangjie	JUKS
U+52C0.0	fFourCorner	44227
U+52C0.0	fHakka	 ket7
U+52C0.0	fCantonese	hak1
U+52C0.0	fMandarin	ke4
U+52C1.0	fUTF8	勁
U+52C1.0	fCNS11643	1-5030
U+52C1.0	fBig5	AB6C
U+52C1.0	fGB	1-3E22
U+52C1.0	fR/S	 19.7
U+52C1.0	fTotalStrokes	 9
U+52C1.0	fCangjie	MMKS
U+52C1.0	fFourCorner	14127
U+52C1.0	fHakka	 gin5; gang5; gin6; ngang5; ngang6; gin1
U+52C1.0	fCantonese	ging6; ging3
U+52C1.0	fMandarin	jing4
U+52C1.0	fEnglish	strong; tough; powerful; sturdy
U+52C1.1	fUTF8	勁
U+52C1.1	fCNS11643	1-5030
U+52C1.1	fBig5	AB6C
U+52C1.1	fGB	1-3E22
U+52C1.1	fR/S	 19.7
U+52C1.1	fTotalStrokes	 9
U+52C1.1	fCangjie	MMKS
U+52C1.1	fFourCorner	14127
U+52C1.1	fMandarin	jin4
U+52C1.1	fEnglish	[1] vigor; energy; strength [2] spirit [3] an air; manner
U+52C2.0	fUTF8	勂
U+52C2.0	fCNS11643	2-2935
U+52C2.0	fBig5	CDF3
U+52C2.0	fGB	E-2354
U+52C2.0	fR/S	 19.7
U+52C2.0	fTotalStrokes	 9
U+52C2.0	fCangjie	HRKS
U+52C2.0	fFourCorner	24627
U+52C2.0	fCantonese	hou4
U+52C2.0	fMandarin	hao2
U+52C3.0	fUTF8	勃
U+52C3.0	fCNS11643	1-502F
U+52C3.0	fBig5	AB6B
U+52C3.0	fGB	0-322A
U+52C3.0	fR/S	 19.7
U+52C3.0	fTotalStrokes	 9
U+52C3.0	fCangjie	JDKS
U+52C3.0	fFourCorner	44427
U+52C3.0	fHakka	  put8
U+52C3.0	fCantonese	but6
U+52C3.0	fMandarin	bo2
U+52C3.0	fEnglish	sudden(ly); sudden, quick(ly)
U+52C4.0	fUTF8	勄
U+52C4.0	fCNS11643	3-2B48
U+52C4.0	fGB	E-2355
U+52C4.0	fR/S	 19.7
U+52C4.0	fTotalStrokes	 9
U+52C4.0	fCangjie	OYKS
U+52C4.0	fFourCorner	84520
U+52C4.0	fHakka	 mien3; meu1; men3
U+52C4.0	fCantonese	man5
U+52C4.0	fMandarin	min3
U+52C4.0	fEnglish	 active clever prompt witty smart earnest diligent hasten
U+52C5.0	fUTF8	勅
U+52C5.0	fCNS11643	3-2B46
U+52C5.0	fGB	E-2356
U+52C5.0	fR/S	  19.7
U+52C5.0	fTotalStrokes	 9
U+52C5.0	fCangjie	DLKS
U+52C5.0	fFourCorner	54920
U+52C5.0	fHakka	  chit7; ts'iit7; cit7
U+52C5.0	fCantonese	cik1
U+52C5.0	fMandarin	chi4
U+52C5.0	fEnglish	[1] imperial decree [2] orders given to demons and spirits by Taoist priests when they exercise magic powers
U+52C6.0	fUTF8	勆
U+52C6.0	fCNS11643	4-2849
U+52C6.0	fGB	E-2357
U+52C6.0	fR/S	  19.7
U+52C6.0	fTotalStrokes	 9
U+52C6.0	fCangjie	IVKS
U+52C6.0	fFourCorner	34720
U+52C6.0	fCantonese	long4
U+52C6.0	fMandarin	lang2
U+52C7.0	fUTF8	勇
U+52C7.0	fCNS11643	1-502D
U+52C7.0	fBig5	AB69
U+52C7.0	fGB	0-5342
U+52C7.0	fR/S	 19.7
U+52C7.0	fTotalStrokes	 9
U+52C7.0	fCangjie	NBKS
U+52C7.0	fFourCorner	17427
U+52C7.0	fHakka	  jung3; rung3
U+52C7.0	fCantonese	jung5; jung6
U+52C7.0	fMandarin	yong3
U+52C7.0	fEnglish	[1] brave; courageous; fierce; bold; valiant; intrepid; fearless [2] soldier; conscript [3] bravery; courage
U+52C8.0	fUTF8	勈
U+52C8.0	fCNS11643	4-284B
U+52C8.0	fGB	E-2358
U+52C8.0	fR/S	 19.7
U+52C8.0	fTotalStrokes	 9
U+52C8.0	fCangjie	NBKS
U+52C8.0	fFourCorner	14220
U+52C8.0	fCantonese	jung5
U+52C8.0	fMandarin	yong3
U+52C9.0	fUTF8	勉
U+52C9.0	fCNS11643	1-502E
U+52C9.0	fBig5	AB6A
U+52C9.0	fGB	0-4363
U+52C9.0	fR/S	 19.7
U+52C9.0	fTotalStrokes	 9
U+52C9.0	fCangjie	NUKS
U+52C9.0	fFourCorner	24212
U+52C9.0	fHakka	  men1;  mien1;  mien3; men3; mian1
U+52C9.0	fCantonese	min5
U+52C9.0	fMandarin	mian3
U+52C9.0	fEnglish	[1] urge; encourage [2] strive; make efforts; exert oneself to
U+52CA.0	fUTF8	勊
U+52CA.0	fCNS11643	3-2B47
U+52CA.0	fGB	5-3474
U+52CA.0	fR/S	 19.7
U+52CA.0	fTotalStrokes	 9
U+52CA.0	fCangjie	JUKS
U+52CA.0	fFourCorner	44210
U+52CA.0	fCantonese	hak1
U+52CA.0	fMandarin	ke4
U+52CB.0	fUTF8	勋
U+52CB.0	fGB	0-512B
U+52CB.0	fR/S	 19.7
U+52CB.0	fTotalStrokes	 9
U+52CB.0	fCangjie	ROKS
U+52CB.0	fHakka	 hiun1
U+52CB.0	fCantonese	fan1
U+52CB.0	fMandarin	xun1
U+52CB.0	fEnglish	simplified form of &#x52DB;
U+52CC.0	fUTF8	勌
U+52CC.0	fCNS11643	3-2F6F
U+52CC.0	fGB	E-2359
U+52CC.0	fR/S	 19.8
U+52CC.0	fTotalStrokes	 10
U+52CC.0	fCangjie	FUKS
U+52CC.0	fFourCorner	94727
U+52CC.0	fCantonese	gyun6
U+52CC.0	fMandarin	quan4; juan4; juan1
U+52CC.0	fEnglish	 labor tired
U+52CD.0	fUTF8	勍
U+52CD.0	fCNS11643	2-2E30
U+52CD.0	fBig5	D0ED
U+52CD.0	fGB	3-3344
U+52CD.0	fR/S	 19.8
U+52CD.0	fTotalStrokes	 10
U+52CD.0	fCangjie	YFKS
U+52CD.0	fFourCorner	04927
U+52CD.0	fHakka	  gin1; kiang2
U+52CD.0	fCantonese	king4
U+52CD.0	fMandarin	qing2
U+52CD.0	fEnglish	 strong; powerful; mighty violent
U+52CE.0	fUTF8	勎
U+52CE.0	fCNS11643	3-2F6C
U+52CE.0	fGB	5-3476
U+52CE.0	fR/S	  19.8
U+52CE.0	fTotalStrokes	 10
U+52CE.0	fCangjie	GGKS
U+52CE.0	fFourCorner	44120
U+52CE.0	fCantonese	luk6
U+52CE.0	fMandarin	lu4
U+52CF.0	fUTF8	勏
U+52CF.0	fCNS11643	3-2F6A
U+52CF.0	fGB	E-235A
U+52CF.0	fR/S	  19.8
U+52CF.0	fTotalStrokes	 10
U+52CF.0	fCangjie	YRKS
U+52CF.0	fFourCorner	04620
U+52CF.0	fCantonese	bau6
U+52CF.0	fMandarin	pou3; bu4
U+52D0.0	fUTF8	勐
U+52D0.0	fCNS11643	3-2F6D
U+52D0.0	fGB	0-5B42
U+52D0.0	fR/S	  19.8
U+52D0.0	fTotalStrokes	 10
U+52D0.0	fCangjie	NTKS
U+52D0.0	fFourCorner	14120
U+52D0.0	fCantonese	maang5
U+52D0.0	fMandarin	meng3
U+52D0.0	fEnglish	 imperial degree Taoist magic
U+52D1.0	fUTF8	勑
U+52D1.0	fCNS11643	3-2F6E
U+52D1.0	fGB	E-235B
U+52D1.0	fR/S	 19.8
U+52D1.0	fTotalStrokes	 10
U+52D1.0	fCangjie	DOKS
U+52D1.0	fFourCorner	44920
U+52D1.0	fHakka	 loi5; loi6; chit7; ts'iit7
U+52D1.0	fCantonese	loi6; cik1; loi4
U+52D1.0	fMandarin	chi4; lai4
U+52D1.0	fEnglish	reward
U+52D2.0	fUTF8	勒
U+52D2.0	fCNS11643	1-592C
U+52D2.0	fBig5	B0C7
U+52D2.0	fGB	0-4055
U+52D2.0	fR/S	 19.9
U+52D2.0	fTotalStrokes	 11
U+52D2.0	fCangjie	TJKS
U+52D2.0	fFourCorner	44527
U+52D2.0	fHakka	  let8; lek8; lit8
U+52D2.0	fCantonese	lak6; laak6 (also)
U+52D2.0	fMandarin	le4; le1; ye4
U+52D2.0	fEnglish	[1] force; compel [2] reign; rule; control; command [3] engrave; carve [4] bridle [5] horizontal stroke (calligraphy)
U+52D2.1	fUTF8	勒
U+52D2.1	fCNS11643	1-592C
U+52D2.1	fBig5	B0C7
U+52D2.1	fGB	0-4055
U+52D2.1	fR/S	 19.9
U+52D2.1	fTotalStrokes	 11
U+52D2.1	fCangjie	TJKS
U+52D2.1	fFourCorner	44527
U+52D2.1	fMandarin	lei1
U+52D2.1	fEnglish	 strangle; tighten
U+52D3.0	fUTF8	勓
U+52D3.0	fCNS11643	2-3371
U+52D3.0	fBig5	D46E
U+52D3.0	fGB	5-347A
U+52D3.0	fR/S	 19.9
U+52D3.0	fTotalStrokes	 11
U+52D3.0	fCangjie	PAKS
U+52D3.0	fFourCorner	24627
U+52D3.0	fCantonese	hat6; faai3
U+52D3.0	fMandarin	kai4; biao1
U+52D4.0	fUTF8	勔
U+52D4.0	fCNS11643	3-3452
U+52D4.0	fGB	5-775F
U+52D4.0	fR/S	  19.9
U+52D4.0	fTotalStrokes	 11
U+52D4.0	fCangjie	MWKS
U+52D4.0	fFourCorner	14627
U+52D4.0	fHakka	  mien3
U+52D4.0	fCantonese	min5
U+52D4.0	fMandarin	mian3
U+52D4.0	fEnglish	 endeavor; make effort urge
U+52D5.0	fUTF8	動
U+52D5.0	fCNS11643	1-592F
U+52D5.0	fBig5	B0CA
U+52D5.0	fGB	1-362F
U+52D5.0	fR/S	 19.9
U+52D5.0	fTotalStrokes	 11
U+52D5.0	fCangjie	HGKS
U+52D5.0	fFourCorner	24127
U+52D5.0	fHakka	  tung1; tung5; tung3; tung6
U+52D5.0	fCantonese	dung6
U+52D5.0	fMandarin	dong4
U+52D5.0	fEnglish	[1] move; stir [2] change; alter [3] act; do [4] touch (one's heart); arouse; excite; move [5] take up [6] use [7] eat or drink [8] movement; action; motion
U+52D6.0	fUTF8	勖
U+52D6.0	fCNS11643	2-3370
U+52D6.0	fBig5	D46D
U+52D6.0	fGB	0-5B43
U+52D6.0	fR/S	 19.9
U+52D6.0	fTotalStrokes	 11
U+52D6.0	fCangjie	AUKS
U+52D6.0	fFourCorner	64627
U+52D6.0	fHakka	 hiuk7
U+52D6.0	fCantonese	juk1; huk1
U+52D6.0	fMandarin	xu4; mao4
U+52D6.0	fEnglish	enjoin; advise; preach to
U+52D7.0	fUTF8	勗
U+52D7.0	fCNS11643	1-5B2B
U+52D7.0	fBig5	B1E5
U+52D7.0	fGB	5-4A3C
U+52D7.0	fR/S	 19.9; 72.7
U+52D7.0	fTotalStrokes	 11
U+52D7.0	fCangjie	ABMS
U+52D7.0	fFourCorner	60127
U+52D7.0	fHakka	  hiuk7
U+52D7.0	fCantonese	juk1; huk1
U+52D7.0	fMandarin	xu4
U+52D7.0	fEnglish	 encourage; excite; stimulate
U+52D8.0	fUTF8	勘
U+52D8.0	fCNS11643	1-592E
U+52D8.0	fBig5	B0C9
U+52D8.0	fGB	0-3F31
U+52D8.0	fR/S	 19.9
U+52D8.0	fTotalStrokes	 11
U+52D8.0	fCangjie	TVKS
U+52D8.0	fFourCorner	44727
U+52D8.0	fHakka	  kam5;  kam1
U+52D8.0	fCantonese	ham3; ham1
U+52D8.0	fMandarin	kan1; kan4
U+52D8.0	fEnglish	[1] investigate; explore; examine; check [2] compare critically; collate
U+52D9.0	fUTF8	務
U+52D9.0	fCNS11643	1-592D
U+52D9.0	fBig5	B0C8
U+52D9.0	fGB	1-4E71
U+52D9.0	fR/S	 19.9
U+52D9.0	fTotalStrokes	 11
U+52D9.0	fCangjie	NHOKS
U+52D9.0	fFourCorner	18227
U+52D9.0	fHakka	  mu5;  wu5; vu5
U+52D9.0	fCantonese	mou6
U+52D9.0	fMandarin	wu4; mao1; mao4; wu3
U+52D9.0	fEnglish	[1] attend to; strive after to be engaged in; endeavor; work; strive; make effort [2] duty; affairs; business [3] must; should; necessary [4] a Chinese family name
U+52D9.1	fUTF8	務
U+52D9.1	fCNS11643	1-592D
U+52D9.1	fBig5	B0C8
U+52D9.1	fGB	1-4E71
U+52D9.1	fR/S	 19.9
U+52D9.1	fTotalStrokes	 11
U+52D9.1	fCangjie	NHOKS
U+52D9.1	fFourCorner	18227
U+52DA.0	fUTF8	勚
U+52DA.0	fGB	8-7A43
U+52DA.0	fR/S	 19.9; 19.12
U+52DA.0	fTotalStrokes	 11
U+52DA.0	fCangjie	POKS
U+52DA.0	fCantonese	ji6; jai6 (also)
U+52DA.0	fMandarin	yi4
U+52DA.0	fEnglish	work hard; belabored; toil
U+52DB.0	fUTF8	勛
U+52DB.0	fCNS11643	1-5E3A
U+52DB.0	fBig5	B3D4
U+52DB.0	fGB	1-512B
U+52DB.0	fR/S	 19.10
U+52DB.0	fTotalStrokes	 12
U+52DB.0	fCangjie	RCKS
U+52DB.0	fFourCorner	64827
U+52DB.0	fHakka	 hiun1; hun1
U+52DB.0	fCantonese	fan1
U+52DB.0	fMandarin	xun1
U+52DB.0	fEnglish	 meritorious deed merits rank
U+52DC.0	fUTF8	勜
U+52DC.0	fCNS11643	5-366A
U+52DC.0	fGB	5-347E
U+52DC.0	fR/S	 19.10
U+52DC.0	fTotalStrokes	 12
U+52DC.0	fCangjie	CMKS
U+52DC.0	fFourCorner	84120
U+52DC.0	fCantonese	jung2
U+52DC.0	fMandarin	weng3
U+52DD.0	fUTF8	勝
U+52DD.0	fCNS11643	1-5E39
U+52DD.0	fBig5	B3D3
U+52DD.0	fGB	1-4A24
U+52DD.0	fR/S	 19.10
U+52DD.0	fTotalStrokes	 12
U+52DD.0	fCangjie	BFQS
U+52DD.0	fFourCorner	79227
U+52DD.0	fHakka	 sin5; shin5; sen5; shin1
U+52DD.0	fCantonese	sing3; sing1
U+52DD.0	fMandarin	sheng4
U+52DD.0	fEnglish	[1] win; excel; triumph; surpass; get the better of [2] victory; success; superiority [3] win (sports) [4] scenic view; place of natural beauty [5] excellent; distinctive; wonderful; outstanding; superior; surpassing; superb
U+52DD.1	fUTF8	勝
U+52DD.1	fCNS11643	1-5E39
U+52DD.1	fBig5	B3D3
U+52DD.1	fGB	1-4A24
U+52DD.1	fR/S	 19.10
U+52DD.1	fTotalStrokes	 12
U+52DD.1	fCangjie	BFQS
U+52DD.1	fFourCorner	79227
U+52DD.1	fMandarin	sheng1
U+52DD.1	fEnglish	 be competent enough (for a task)
U+52DE.0	fUTF8	勞
U+52DE.0	fCNS11643	1-5E38
U+52DE.0	fBig5	B3D2
U+52DE.0	fGB	1-404D
U+52DE.0	fR/S	 19.10
U+52DE.0	fTotalStrokes	 12
U+52DE.0	fCangjie	FFBKS
U+52DE.0	fFourCorner	99427
U+52DE.0	fHakka	  lau2; lo2; lo3; lau5; lau3
U+52DE.0	fCantonese	lou4; lou6; lou2 (also)
U+52DE.0	fMandarin	lao2
U+52DE.0	fEnglish	[1] labor; take the toil; do (manual) work [2] trouble; worry; bother [3] meritorious deeds; services [4] a Chinese family name
U+52DE.1	fUTF8	勞
U+52DE.1	fCNS11643	1-5E38
U+52DE.1	fBig5	B3D2
U+52DE.1	fGB	1-404D
U+52DE.1	fR/S	 19.10
U+52DE.1	fTotalStrokes	 12
U+52DE.1	fCangjie	FFBKS
U+52DE.1	fFourCorner	99427
U+52DE.1	fMandarin	lao4
U+52DE.1	fEnglish	 comfort or entertain (be tired)
U+52DF.0	fUTF8	募
U+52DF.0	fCNS11643	1-6339
U+52DF.0	fBig5	B6D2
U+52DF.0	fGB	0-443C
U+52DF.0	fR/S	 19.11
U+52DF.0	fTotalStrokes	 13
U+52DF.0	fCangjie	TAKS
U+52DF.0	fFourCorner	44427
U+52DF.0	fHakka	  mu5;  mok7
U+52DF.0	fCantonese	mou6
U+52DF.0	fMandarin	mu4
U+52DF.0	fEnglish	 levy; raise summon recruit
U+52E0.0	fUTF8	勠
U+52E0.0	fCNS11643	3-4045
U+52E0.0	fGB	E-235C
U+52E0.0	fR/S	  19.11
U+52E0.0	fTotalStrokes	 13
U+52E0.0	fCangjie	SHKS
U+52E0.0	fFourCorner	14220
U+52E0.0	fHakka	  liuk8; luk7
U+52E0.0	fCantonese	luk6
U+52E0.0	fMandarin	lu4
U+52E0.0	fEnglish	join forces; unite
U+52E1.0	fUTF8	勡
U+52E1.0	fCNS11643	3-4044
U+52E1.0	fGB	5-3521
U+52E1.0	fR/S	 19.11
U+52E1.0	fTotalStrokes	 13
U+52E1.0	fCangjie	MFKS
U+52E1.0	fFourCorner	14920
U+52E1.0	fHakka	 piau5
U+52E1.0	fCantonese	piu3
U+52E1.0	fMandarin	piao4
U+52E1.0	fEnglish	 seize by force; plunder
U+52E2.0	fUTF8	勢
U+52E2.0	fCNS11643	1-633C
U+52E2.0	fBig5	B6D5
U+52E2.0	fGB	1-4A46
U+52E2.0	fR/S	 19.11
U+52E2.0	fTotalStrokes	 13
U+52E2.0	fCangjie	GIKS
U+52E2.0	fFourCorner	45427
U+52E2.0	fHakka	  si5;  s'e5;  shi5; se5; she5
U+52E2.0	fCantonese	sai3
U+52E2.0	fMandarin	shi4
U+52E2.0	fEnglish	[1] power; force; influence; strength [2] tendency; trend [3] the natural features [4] situation; circumstances; aspect; conditions [5] signs; gestures [6] male genitals
U+52E2.1	fUTF8	勢
U+52E2.1	fCNS11643	1-633C
U+52E2.1	fBig5	B6D5
U+52E2.1	fGB	1-4A46
U+52E2.1	fR/S	 19.11
U+52E2.1	fTotalStrokes	 13
U+52E2.1	fCangjie	GIKS
U+52E2.1	fFourCorner	45427
U+52E2.1	fMandarin	shi5
U+52E3.0	fUTF8	勣
U+52E3.0	fCNS11643	1-633D
U+52E3.0	fBig5	B6D6
U+52E3.0	fGB	E-235D
U+52E3.0	fR/S	 19.11
U+52E3.0	fTotalStrokes	 13
U+52E3.0	fCangjie	QCKS
U+52E3.0	fFourCorner	54827
U+52E3.0	fHakka	  zit7
U+52E3.0	fCantonese	zik1
U+52E3.0	fMandarin	ji1
U+52E3.0	fEnglish	achievements; conduct deserving
U+52E4.0	fUTF8	勤
U+52E4.0	fCNS11643	1-633B
U+52E4.0	fBig5	B6D4
U+52E4.0	fGB	0-475A
U+52E4.0	fR/S	 19.11
U+52E4.0	fTotalStrokes	 13
U+52E4.0	fCangjie	TMKS
U+52E4.0	fFourCorner	44127
U+52E4.0	fHakka	  kiun2;  kin2; kiun3
U+52E4.0	fCantonese	kan4
U+52E4.0	fMandarin	qin2
U+52E4.0	fEnglish	[1] industrious; diligent; sedulous; hardworking; endeavor; strive [2] frequently; regularly
U+52E5.0	fUTF8	勥
U+52E5.0	fCNS11643	4-3C3C
U+52E5.0	fGB	5-3524
U+52E5.0	fR/S	 19.11
U+52E5.0	fTotalStrokes	 13
U+52E5.0	fCangjie	NIKS
U+52E5.0	fFourCorner	13420
U+52E5.0	fHakka	 kiong3; giong3
U+52E5.0	fCantonese	koeng5
U+52E5.0	fMandarin	jiang4; qiang3; jiang3
U+52E5.0	fEnglish	 exert one's strength; resist; pursue
U+52E6.0	fUTF8	勦
U+52E6.0	fCNS11643	1-633A
U+52E6.0	fBig5	B6D3
U+52E6.0	fGB	E-235E
U+52E6.0	fR/S	 19.11
U+52E6.0	fTotalStrokes	 13
U+52E6.0	fCangjie	VDKS
U+52E6.0	fFourCorner	24927
U+52E6.0	fHakka	 ziau3
U+52E6.0	fCantonese	ziu2; caau1; caau4; cau2; ziu1
U+52E6.0	fMandarin	jiao3; chao1; chao4
U+52E6.0	fEnglish	destroy; exterminate; annihilate
U+52E7.0	fUTF8	勧
U+52E7.0	fCNS11643	3-4046
U+52E7.0	fGB	E-235F
U+52E7.0	fR/S	 19.11
U+52E7.0	fTotalStrokes	 13
U+52E7.0	fCangjie	OGKS
U+52E7.0	fFourCorner	84220
U+52E7.0	fCantonese	hyun3
U+52E7.0	fMandarin	quan4
U+52E7.0	fEnglish	recommend; advise; urge
U+52E8.0	fUTF8	勨
U+52E8.0	fCNS11643	5-4467
U+52E8.0	fGB	3-334A
U+52E8.0	fR/S	 19.12
U+52E8.0	fTotalStrokes	 14
U+52E8.0	fCangjie	NOKS
U+52E8.0	fFourCorner	24220
U+52E8.0	fCantonese	zoeng6
U+52E8.0	fMandarin	yang3; xiang4
U+52E9.0	fUTF8	勩
U+52E9.0	fCNS11643	2-492A
U+52E9.0	fBig5	E143
U+52E9.0	fGB	3-3345
U+52E9.0	fR/S	 19.12
U+52E9.0	fTotalStrokes	 14
U+52E9.0	fCangjie	PCKS
U+52E9.0	fFourCorner	44827
U+52E9.0	fHakka	 sii5; ji5; ri6; i5
U+52E9.0	fCantonese	ji6; jai6 (also)
U+52E9.0	fMandarin	yi4; shi4
U+52E9.0	fEnglish	 work hard belabored toil
U+52EA.0	fUTF8	勪
U+52EA.0	fCNS11643	3-4621
U+52EA.0	fGB	E-2360
U+52EA.0	fR/S	 19.12
U+52EA.0	fTotalStrokes	 14
U+52EA.0	fCangjie	HBKS
U+52EA.0	fFourCorner	24220
U+52EA.0	fCantonese	kiu5
U+52EA.0	fMandarin	jue2; qiao1; jiao3
U+52EB.0	fUTF8	勫
U+52EB.0	fCNS11643	2-492B
U+52EB.0	fBig5	E144
U+52EB.0	fGB	5-3523
U+52EB.0	fR/S	 19.12
U+52EB.0	fTotalStrokes	 14
U+52EB.0	fCangjie	HWKS
U+52EB.0	fFourCorner	24627
U+52EB.0	fCantonese	faan1; faan4
U+52EB.0	fMandarin	fan2
U+52EC.0	fUTF8	勬
U+52EC.0	fCNS11643	5-4463
U+52EC.0	fGB	3-334C
U+52EC.0	fR/S	  19.12
U+52EC.0	fTotalStrokes	 14
U+52EC.0	fCangjie	FFKS
U+52EC.0	fFourCorner	94920
U+52EC.0	fCantonese	gyun1
U+52EC.0	fMandarin	juan4; juan1
U+52ED.0	fUTF8	勭
U+52ED.0	fCNS11643	3-457E
U+52ED.0	fGB	3-334B
U+52ED.0	fR/S	  19.12
U+52ED.0	fTotalStrokes	 14
U+52ED.0	fCangjie	YGKS
U+52ED.0	fFourCorner	04120
U+52ED.0	fCantonese	tung4
U+52ED.0	fMandarin	tong2
U+52EE.0	fUTF8	勮
U+52EE.0	fCNS11643	3-4B33
U+52EE.0	fGB	3-334D
U+52EE.0	fR/S	 19.13
U+52EE.0	fTotalStrokes	 15
U+52EE.0	fCangjie	YOKS
U+52EE.0	fFourCorner	24220
U+52EE.0	fCantonese	geoi6; geoi3 (also); keoi3
U+52EE.0	fMandarin	ju4
U+52EF.0	fUTF8	勯
U+52EF.0	fCNS11643	2-4F5D
U+52EF.0	fBig5	E4F5
U+52EF.0	fGB	5-3526
U+52EF.0	fR/S	 19.13
U+52EF.0	fTotalStrokes	 15
U+52EF.0	fCangjie	YMKS
U+52EF.0	fFourCorner	04127
U+52EF.0	fCantonese	daan1
U+52EF.0	fMandarin	dan1
U+52EF.0	fEnglish	exhausted
U+52F0.0	fUTF8	勰
U+52F0.0	fCNS11643	1-6C2F
U+52F0.0	fBig5	BC45
U+52F0.0	fGB	0-5B44
U+52F0.0	fR/S	 19.13
U+52F0.0	fTotalStrokes	 15
U+52F0.0	fCangjie	KSWP
U+52F0.0	fFourCorner	46430
U+52F0.0	fHakka	  hiap8
U+52F0.0	fCantonese	hip3; hip6
U+52F0.0	fMandarin	xie2
U+52F0.0	fEnglish	harmonious
U+52F1.0	fUTF8	勱
U+52F1.0	fCNS11643	2-4F5C
U+52F1.0	fBig5	E4F4
U+52F1.0	fGB	1-5B3D
U+52F1.0	fR/S	 19.13
U+52F1.0	fTotalStrokes	 15
U+52F1.0	fCangjie	TBKS
U+52F1.0	fFourCorner	44427
U+52F1.0	fHakka	 mai3; mai5; mai6
U+52F1.0	fCantonese	maai6
U+52F1.0	fMandarin	mai4
U+52F1.0	fEnglish	put forth effort; strive forward
U+52F2.0	fUTF8	勲
U+52F2.0	fCNS11643	3-4D2C
U+52F2.0	fGB	E-2361
U+52F2.0	fR/S	 86.11; 19.13
U+52F2.0	fTotalStrokes	 15
U+52F2.0	fCangjie	HSF
U+52F2.0	fFourCorner	24330
U+52F2.0	fCantonese	fan1
U+52F2.0	fMandarin	xun1
U+52F2.0	fEnglish	 meritorious deed merits rank
U+52F3.0	fUTF8	勳
U+52F3.0	fCNS11643	1-703A
U+52F3.0	fBig5	BEB1
U+52F3.0	fGB	8-2D53
U+52F3.0	fR/S	 19.14
U+52F3.0	fTotalStrokes	 16
U+52F3.0	fCangjie	HFKS
U+52F3.0	fFourCorner	24327
U+52F3.0	fHakka	 hiun1
U+52F3.0	fCantonese	fan1
U+52F3.0	fMandarin	xun1; xun4
U+52F3.0	fEnglish	meritorious deed; meritorious services; merits; rank; honors; achievements; services
U+52F4.0	fUTF8	勴
U+52F4.0	fCNS11643	2-5C4B
U+52F4.0	fBig5	ECBF
U+52F4.0	fGB	3-3347
U+52F4.0	fR/S	 19.15
U+52F4.0	fTotalStrokes	 17
U+52F4.0	fCangjie	YPKS
U+52F4.0	fFourCorner	24227
U+52F4.0	fHakka	 li5; li6
U+52F4.0	fCantonese	leoi6
U+52F4.0	fMandarin	l4; luu4; lu4
U+52F4.0	fEnglish	 help; aid
U+52F5.0	fUTF8	勵
U+52F5.0	fCNS11643	1-7344
U+52F5.0	fBig5	C079
U+52F5.0	fGB	1-4078
U+52F5.0	fR/S	 19.15
U+52F5.0	fTotalStrokes	 17
U+52F5.0	fCangjie	MBKS
U+52F5.0	fFourCorner	74227
U+52F5.0	fHakka	 li5; li6; li3
U+52F5.0	fCantonese	lai6
U+52F5.0	fMandarin	li4
U+52F5.0	fEnglish	[1] incite; encourage; rouse (to action) [2] exert oneself
U+52F6.0	fUTF8	勶
U+52F6.0	fCNS11643	5-5362
U+52F6.0	fGB	5-3527
U+52F6.0	fR/S	 19.15; 19.14
U+52F6.0	fTotalStrokes	 17; 16
U+52F6.0	fCangjie	HKKS
U+52F6.0	fFourCorner	28420
U+52F6.0	fCantonese	cit3
U+52F6.0	fMandarin	che4
U+52F7.0	fUTF8	勷
U+52F7.0	fCNS11643	2-653A
U+52F7.0	fBig5	F1EE
U+52F7.0	fGB	3-334F
U+52F7.0	fR/S	 19.17
U+52F7.0	fTotalStrokes	 19
U+52F7.0	fCangjie	YVKS
U+52F7.0	fFourCorner	04727
U+52F7.0	fHakka	 siong1; rong2; jong2; jong5
U+52F7.0	fCantonese	joeng4; soeng1
U+52F7.0	fMandarin	rang2; xiang1
U+52F7.0	fEnglish	 urgent in haste; hurriedly help
U+52F8.0	fUTF8	勸
U+52F8.0	fCNS11643	1-7961
U+52F8.0	fBig5	C455
U+52F8.0	fGB	1-4830
U+52F8.0	fR/S	 19.17; 19.18
U+52F8.0	fTotalStrokes	 19
U+52F8.0	fCangjie	TGKS
U+52F8.0	fFourCorner	44227
U+52F8.0	fHakka	  kien5;  ken5; k'ian5; kian5
U+52F8.0	fCantonese	hyun3
U+52F8.0	fMandarin	quan4
U+52F8.0	fEnglish	[1] exhort; urge; persuade [2] recommend; advise
U+52F9.0	fUTF8	勹
U+52F9.0	fCNS11643	1-2734
U+52F9.0	fGB	0-5968
U+52F9.0	fR/S	 20.0
U+52F9.0	fTotalStrokes	 2
U+52F9.0	fCangjie	XHS
U+52F9.0	fFourCorner	2702
U+52F9.0	fHakka	 bau1
U+52F9.0	fCantonese	baau1
U+52F9.0	fMandarin	bao1
U+52F9.0	fEnglish	 wrap Kangxi radical number 20
U+52FA.0	fUTF8	勺
U+52FA.0	fCNS11643	1-4444
U+52FA.0	fBig5	A463
U+52FA.0	fGB	0-4957
U+52FA.0	fR/S	 20.1
U+52FA.0	fTotalStrokes	 3
U+52FA.0	fCangjie	PI
U+52FA.0	fFourCorner	27320
U+52FA.0	fHakka	 sok8;  shok7; chok7; shok8; zok7; sok7
U+52FA.0	fCantonese	zoek3; soek3 (also); coek3 (also)
U+52FA.0	fMandarin	shao2; zhuo2; shuo4
U+52FA.0	fEnglish	 spoon; ladle; scoop
U+52FB.0	fUTF8	勻
U+52FB.0	fCNS11643	1-4524
U+52FB.0	fBig5	A4C3
U+52FB.0	fGB	E-2362
U+52FB.0	fR/S	 20.2
U+52FB.0	fTotalStrokes	 4
U+52FB.0	fCangjie	PMM
U+52FB.0	fFourCorner	27120
U+52FB.0	fHakka	  jun2; run2; giun1; jun3
U+52FB.0	fCantonese	wan4
U+52FB.0	fMandarin	yun2
U+52FB.0	fEnglish	equal; impartially; uniform; even
U+52FC.0	fUTF8	勼
U+52FC.0	fCNS11643	2-2136
U+52FC.0	fBig5	C956
U+52FC.0	fGB	5-332C
U+52FC.0	fR/S	 20.2
U+52FC.0	fTotalStrokes	 4
U+52FC.0	fCangjie	PKN
U+52FC.0	fFourCorner	27420
U+52FC.0	fHakka	 kieu1; giu1; geu1
U+52FC.0	fCantonese	gau1; kau1
U+52FC.0	fMandarin	jiu1
U+52FD.0	fUTF8	勽
U+52FD.0	fCNS11643	3-2155
U+52FD.0	fGB	5-332B
U+52FD.0	fR/S	 20.2
U+52FD.0	fTotalStrokes	 4
U+52FD.0	fCangjie	PO
U+52FD.0	fFourCorner	27820
U+52FD.0	fHakka	 wut7; pau5; pau6
U+52FD.0	fCantonese	bou6
U+52FD.0	fMandarin	bao4
U+52FD.0	fEnglish	 incubate brood hatch
U+52FE.0	fUTF8	勾
U+52FE.0	fCNS11643	1-4525
U+52FE.0	fBig5	A4C4
U+52FE.0	fGB	0-3934
U+52FE.0	fR/S	 20.2
U+52FE.0	fTotalStrokes	 4
U+52FE.0	fCangjie	PI
U+52FE.0	fFourCorner	27720
U+52FE.0	fHakka	  geu1; kieu1; giu1; keu1; geu2; gio1; gio3
U+52FE.0	fCantonese	ngau1; au1 (also); gau1 (also)
U+52FE.0	fMandarin	gou1
U+52FE.0	fEnglish	[1] mark; put a check; mark on [2] cancel; cross out (or off) [3] hook [4] join; connect [5] evoke [6] entice; seduce [7] hook
U+52FE.1	fUTF8	勾
U+52FE.1	fCNS11643	1-4525
U+52FE.1	fBig5	A4C4
U+52FE.1	fGB	0-3934
U+52FE.1	fR/S	 20.2
U+52FE.1	fTotalStrokes	 4
U+52FE.1	fCangjie	PI
U+52FE.1	fFourCorner	27720
U+52FE.1	fMandarin	gou4
U+52FE.1	fEnglish	[1] manage [2] business; affairs
U+52FF.0	fUTF8	勿
U+52FF.0	fCNS11643	1-4526
U+52FF.0	fBig5	A4C5
U+52FF.0	fGB	0-4E70
U+52FF.0	fR/S	 20.2
U+52FF.0	fTotalStrokes	 4
U+52FF.0	fCangjie	PHH
U+52FF.0	fFourCorner	27220
U+52FF.0	fHakka	  wut8; vut8
U+52FF.0	fCantonese	mat6
U+52FF.0	fMandarin	wu4; mei2
U+52FF.0	fEnglish	must not; do not; without; never; a negative word used in formal speech
U+5300.0	fUTF8	匀
U+5300.0	fCNS11643	3-2156
U+5300.0	fGB	0-5448
U+5300.0	fR/S	 20.2
U+5300.0	fTotalStrokes	 4
U+5300.0	fCangjie	PIM
U+5300.0	fFourCorner	27120
U+5300.0	fCantonese	wan4
U+5300.0	fMandarin	yun2
U+5300.0	fEnglish	equal
U+5301.0	fUTF8	匁
U+5301.0	fCNS11643	F-2130
U+5301.0	fGB	E-2363
U+5301.0	fR/S	 20.2
U+5301.0	fTotalStrokes	 4
U+5301.0	fCangjie	PK
U+5301.0	fFourCorner	2740
U+5301.0	fCantonese	jan6
U+5301.0	fEnglish	Japanese unit of weight (1/1000 of a kan)
U+5302.0	fUTF8	匂
U+5302.0	fGB	E-2364
U+5302.0	fR/S	 20.2
U+5302.0	fTotalStrokes	 4
U+5302.0	fCangjie	XPP
U+5302.0	fCantonese	hung1
U+5302.0	fEnglish	 fragrance, smell
U+5303.0	fUTF8	匃
U+5303.0	fCNS11643	3-2223
U+5303.0	fGB	E-2365
U+5303.0	fR/S	  20.3
U+5303.0	fTotalStrokes	 5
U+5303.0	fCangjie	PVO
U+5303.0	fFourCorner	27720
U+5303.0	fHakka	 got7; goi5; goi1
U+5303.0	fCantonese	koi3
U+5303.0	fMandarin	gai4
U+5303.0	fEnglish	 beggar beg give
U+5304.0	fUTF8	匄
U+5304.0	fCNS11643	3-2222
U+5304.0	fGB	E-2366
U+5304.0	fR/S	  20.3
U+5304.0	fTotalStrokes	 5
U+5304.0	fCangjie	PYV
U+5304.0	fFourCorner	27720
U+5304.0	fCantonese	koi3
U+5304.0	fMandarin	gai4; ge2
U+5304.0	fEnglish	 beg for alms beggar
U+5305.0	fUTF8	包
U+5305.0	fCNS11643	1-457D
U+5305.0	fBig5	A55D
U+5305.0	fGB	0-307C
U+5305.0	fR/S	 20.3
U+5305.0	fTotalStrokes	 5
U+5305.0	fCangjie	PRU
U+5305.0	fFourCorner	27712
U+5305.0	fHakka	  bau1
U+5305.0	fCantonese	baau1
U+5305.0	fMandarin	bao1
U+5305.0	fEnglish	[1] wrap; pack; bundle [2] include [3] surround [4] guarantee [5] parcel; package; bundle
U+5306.0	fUTF8	匆
U+5306.0	fCNS11643	1-457E
U+5306.0	fBig5	A55E
U+5306.0	fGB	0-3452
U+5306.0	fR/S	 20.3
U+5306.0	fTotalStrokes	 5
U+5306.0	fCangjie	PKK
U+5306.0	fFourCorner	27450
U+5306.0	fHakka	 cung1
U+5306.0	fCantonese	cung1
U+5306.0	fMandarin	cong1
U+5306.0	fEnglish	hastily; in haste; hurriedly
U+5307.0	fUTF8	匇
U+5307.0	fCNS11643	3-2224
U+5307.0	fGB	E-2367
U+5307.0	fR/S	  20.3
U+5307.0	fTotalStrokes	 5
U+5307.0	fCangjie	PNI
U+5307.0	fFourCorner	27220
U+5307.0	fCantonese	cung1
U+5308.0	fUTF8	匈
U+5308.0	fCNS11643	1-474A
U+5308.0	fBig5	A649
U+5308.0	fGB	0-5059
U+5308.0	fR/S	 20.4
U+5308.0	fTotalStrokes	 6
U+5308.0	fCangjie	PUK
U+5308.0	fFourCorner	27720
U+5308.0	fHakka	 hiung1
U+5308.0	fCantonese	hung1
U+5308.0	fMandarin	xiong1; xiong4
U+5308.0	fEnglish	[1] breast; bosom; chest; thorax [2] clamor [3] the Hsiung Nu "Huns"
U+5309.0	fUTF8	匉
U+5309.0	fCNS11643	2-2333
U+5309.0	fBig5	CA71
U+5309.0	fGB	5-332D
U+5309.0	fR/S	 20.5
U+5309.0	fTotalStrokes	 7
U+5309.0	fCangjie	PMFJ
U+5309.0	fFourCorner	27420
U+5309.0	fCantonese	ping1; paang1 (also); pang1
U+5309.0	fMandarin	peng1
U+5309.0	fEnglish	noise of waters
U+530A.0	fUTF8	匊
U+530A.0	fCNS11643	2-2556
U+530A.0	fBig5	CBD6
U+530A.0	fGB	5-3331
U+530A.0	fR/S	 20.6
U+530A.0	fTotalStrokes	 8
U+530A.0	fCangjie	PFD
U+530A.0	fFourCorner	27920
U+530A.0	fHakka	  kiuk7
U+530A.0	fCantonese	guk1
U+530A.0	fMandarin	ju2
U+530A.0	fEnglish	ha5dful
U+530B.0	fUTF8	匋
U+530B.0	fCNS11643	2-2557
U+530B.0	fBig5	CBD7
U+530B.0	fGB	5-332F
U+530B.0	fR/S	 20.6
U+530B.0	fTotalStrokes	 8
U+530B.0	fCangjie	POJU
U+530B.0	fFourCorner	27720
U+530B.0	fHakka	  tau2; to2; jau2; rau2
U+530B.0	fCantonese	tou4
U+530B.0	fMandarin	tao2; yao2
U+530B.0	fEnglish	pottery
U+530C.0	fUTF8	匌
U+530C.0	fCNS11643	4-2551
U+530C.0	fGB	3-324D
U+530C.0	fR/S	  20.6
U+530C.0	fTotalStrokes	 8; 7
U+530C.0	fCangjie	POMR
U+530C.0	fFourCorner	27620
U+530C.0	fHakka	 kap7; hap8
U+530C.0	fCantonese	hap6 (also); gap3
U+530C.0	fMandarin	ge2; ke4
U+530C.0	fEnglish	 environ
U+530D.0	fUTF8	匍
U+530D.0	fCNS11643	1-5031
U+530D.0	fBig5	AB6D
U+530D.0	fGB	0-5969
U+530D.0	fR/S	 20.7
U+530D.0	fTotalStrokes	 9
U+530D.0	fCangjie	PIJB
U+530D.0	fFourCorner	27220
U+530D.0	fHakka	  pu2; pu3
U+530D.0	fCantonese	pou4
U+530D.0	fMandarin	pu2; fu3
U+530D.0	fEnglish	[1] crawl; creep [2] lie prostrate; prostrate
U+530E.0	fUTF8	匎
U+530E.0	fCNS11643	2-2E31
U+530E.0	fBig5	D0EE
U+530E.0	fGB	E-2368
U+530E.0	fR/S	 20.8
U+530E.0	fTotalStrokes	 10
U+530E.0	fCangjie	PKLU
U+530E.0	fFourCorner	27720
U+530E.0	fCantonese	ap1; ngap1 (also); am1
U+530E.0	fMandarin	an4; fu2
U+530F.0	fUTF8	匏
U+530F.0	fCNS11643	1-5931
U+530F.0	fBig5	B0CC
U+530F.0	fGB	0-5E4B
U+530F.0	fR/S	 20.9
U+530F.0	fTotalStrokes	 11
U+530F.0	fCangjie	KSPRU
U+530F.0	fFourCorner	47212
U+530F.0	fHakka	 pau2; pau3
U+530F.0	fCantonese	paau4
U+530F.0	fMandarin	pao2
U+530F.0	fEnglish	[1] gourd; bottle gourd; calabash [2] a kind of musical instrument originally made of a gourd
U+5310.0	fUTF8	匐
U+5310.0	fCNS11643	1-5930
U+5310.0	fBig5	B0CB
U+5310.0	fGB	0-596B
U+5310.0	fR/S	 20.9
U+5310.0	fTotalStrokes	 11
U+5310.0	fCangjie	PMRW
U+5310.0	fFourCorner	27620
U+5310.0	fHakka	  pet8; piet8; fuk8
U+5310.0	fCantonese	baak6 (also); fuk6
U+5310.0	fMandarin	fu2; pu2; pu3
U+5310.0	fEnglish	[1] lie prostrate; prostrate [2] crawl; creep
U+5311.0	fUTF8	匑
U+5311.0	fCNS11643	2-3A47
U+5311.0	fBig5	D863
U+5311.0	fGB	5-3338
U+5311.0	fR/S	 20.10
U+5311.0	fTotalStrokes	 12
U+5311.0	fCangjie	PHHN
U+5311.0	fFourCorner	27420
U+5311.0	fCantonese	gung1; kung4
U+5311.0	fMandarin	gong1; qiong1
U+5312.0	fUTF8	匒
U+5312.0	fCNS11643	2-3A46
U+5312.0	fBig5	D862
U+5312.0	fGB	3-324E
U+5312.0	fR/S	 20.10
U+5312.0	fTotalStrokes	 12
U+5312.0	fCangjie	PTOR
U+5312.0	fFourCorner	27620
U+5312.0	fHakka	  dap7; tap7
U+5312.0	fCantonese	daap3
U+5312.0	fMandarin	da2; da1
U+5312.0	fEnglish	piled one on another
U+5313.0	fUTF8	匓
U+5313.0	fCNS11643	6-5074
U+5313.0	fGB	E-2369
U+5313.0	fR/S	 20.11
U+5313.0	fTotalStrokes	 13
U+5313.0	fCangjie	PHPE
U+5313.0	fFourCorner	27720
U+5313.0	fCantonese	gau3
U+5313.0	fMandarin	jiu4
U+5314.0	fUTF8	匔
U+5314.0	fCNS11643	3-5038
U+5314.0	fGB	3-3251
U+5314.0	fR/S	 20.13; 20.14
U+5314.0	fTotalStrokes	 16; 15
U+5314.0	fCangjie	PHHR
U+5314.0	fFourCorner	27220
U+5314.0	fCantonese	kung4; kung1
U+5314.0	fMandarin	qiong1; qiong2
U+5315.0	fUTF8	匕
U+5315.0	fCNS11643	1-4431
U+5315.0	fBig5	A450
U+5315.0	fGB	0-5830
U+5315.0	fR/S	 21.0
U+5315.0	fTotalStrokes	 2
U+5315.0	fCangjie	UH
U+5315.0	fFourCorner	40710
U+5315.0	fHakka	 bi3
U+5315.0	fCantonese	bei2 (also); bei6 (also); bei3
U+5315.0	fMandarin	bi3
U+5315.0	fEnglish	[1] spoon; ladle [2] arrowhead [3] dagger; knife
U+5316.0	fUTF8	化
U+5316.0	fCNS11643	1-4527
U+5316.0	fBig5	A4C6
U+5316.0	fGB	0-3B2F
U+5316.0	fR/S	 21.2
U+5316.0	fTotalStrokes	 4
U+5316.0	fCangjie	OP
U+5316.0	fFourCorner	24210
U+5316.0	fHakka	  fa5
U+5316.0	fCantonese	faa3; faa1; faa2 (also)
U+5316.0	fMandarin	hua4; hua1
U+5316.0	fEnglish	[1] change; convert; transform; reform; influence [2] short for chemistry
U+5317.0	fUTF8	北
U+5317.0	fCNS11643	1-4621
U+5317.0	fBig5	A55F
U+5317.0	fGB	0-3131
U+5317.0	fR/S	 21.3
U+5317.0	fTotalStrokes	 5
U+5317.0	fCangjie	LMP
U+5317.0	fFourCorner	11110
U+5317.0	fHakka	  bet7; biet7
U+5317.0	fCantonese	bak1; bui3; bei3 (also); baak1
U+5317.0	fMandarin	bei3; bei4; bo4
U+5317.0	fEnglish	[1] north; northern; northerly [2] northward [3] defeated
U+5318.0	fUTF8	匘
U+5318.0	fCNS11643	4-305E
U+5318.0	fGB	5-333E
U+5318.0	fR/S	  21.9
U+5318.0	fTotalStrokes	 11
U+5318.0	fCangjie	PVVW
U+5318.0	fFourCorner	22760
U+5318.0	fCantonese	nou5; lou5 (also)
U+5318.0	fMandarin	nao3
U+5318.0	fEnglish	the brain
U+5319.0	fUTF8	匙
U+5319.0	fCNS11643	1-5932
U+5319.0	fBig5	B0CD
U+5319.0	fGB	0-3357
U+5319.0	fR/S	 21.9
U+5319.0	fTotalStrokes	 11
U+5319.0	fCangjie	AOP
U+5319.0	fFourCorner	61801
U+5319.0	fHakka	 ci2;  shi2; si2; sii2; shi3; she2; chii2; chi2
U+5319.0	fCantonese	ci4; si4; ci2 (also)
U+5319.0	fMandarin	chi2; shi1
U+5319.0	fEnglish	 spoon
U+5319.1	fUTF8	匙
U+5319.1	fCNS11643	1-5932
U+5319.1	fBig5	B0CD
U+5319.1	fGB	0-3357
U+5319.1	fR/S	 21.9
U+5319.1	fTotalStrokes	 11
U+5319.1	fCangjie	AOP
U+5319.1	fFourCorner	61801
U+5319.1	fMandarin	shi5
U+5319.1	fEnglish	 key
U+531A.0	fUTF8	匚
U+531A.0	fCNS11643	2-2124
U+531A.0	fBig5	C943
U+531A.0	fGB	0-584E
U+531A.0	fR/S	 22.0
U+531A.0	fTotalStrokes	 2
U+531A.0	fCangjie	MV
U+531A.0	fFourCorner	71710
U+531A.0	fHakka	 fong1
U+531A.0	fCantonese	fong1; fo1
U+531A.0	fMandarin	fang1; xi3
U+531A.0	fEnglish	[1] box [2] Kangxi radical 22
U+531B.0	fUTF8	匛
U+531B.0	fCNS11643	4-2165
U+531B.0	fGB	E-236A
U+531B.0	fR/S	 22.3
U+531B.0	fTotalStrokes	 5
U+531B.0	fCangjie	SNO
U+531B.0	fFourCorner	71710
U+531B.0	fCantonese	gau6
U+531B.0	fMandarin	jiu4
U+531C.0	fUTF8	匜
U+531C.0	fCNS11643	2-214D
U+531C.0	fBig5	C96C
U+531C.0	fGB	3-3047
U+531C.0	fR/S	 22.3
U+531C.0	fTotalStrokes	 5
U+531C.0	fCangjie	SPD
U+531C.0	fFourCorner	71711
U+531C.0	fHakka	 ji2; ri2; i2
U+531C.0	fCantonese	ji4
U+531C.0	fMandarin	yi2
U+531C.0	fEnglish	 wash basin
U+531D.0	fUTF8	匝
U+531D.0	fCNS11643	1-4622
U+531D.0	fBig5	A560
U+531D.0	fGB	0-5451
U+531D.0	fR/S	 22.3
U+531D.0	fTotalStrokes	 5
U+531D.0	fCangjie	SLB
U+531D.0	fFourCorner	71712
U+531D.0	fHakka	 zap7
U+531D.0	fCantonese	zaap3
U+531D.0	fMandarin	za1
U+531D.0	fEnglish	 full circle encircle
U+531E.0	fUTF8	匞
U+531E.0	fCNS11643	3-2226
U+531E.0	fGB	E-236B
U+531E.0	fR/S	 22.3
U+531E.0	fTotalStrokes	 5
U+531E.0	fCangjie	SM
U+531E.0	fFourCorner	71710
U+531E.0	fCantonese	zoeng6
U+531E.0	fMandarin	jiang4
U+531F.0	fUTF8	匟
U+531F.0	fCNS11643	2-2222
U+531F.0	fBig5	C9C2
U+531F.0	fGB	E-236C
U+531F.0	fR/S	 22.4
U+531F.0	fTotalStrokes	 6
U+531F.0	fCangjie	SYHN
U+531F.0	fFourCorner	71711
U+531F.0	fHakka	 kong5
U+531F.0	fCantonese	kong3
U+531F.0	fMandarin	kang4
U+531F.0	fEnglish	divan for two persons
U+5320.0	fUTF8	匠
U+5320.0	fCNS11643	1-474C
U+5320.0	fBig5	A64B
U+5320.0	fGB	0-3D33
U+5320.0	fR/S	 22.4
U+5320.0	fTotalStrokes	 6
U+5320.0	fCangjie	SHML
U+5320.0	fFourCorner	71712
U+5320.0	fHakka	  siong3; siong5; siong6
U+5320.0	fCantonese	zoeng6; zoeng2 (also)
U+5320.0	fMandarin	jiang4
U+5320.0	fEnglish	 craftsman; artisan; skilled workman
U+5320.1	fUTF8	匠
U+5320.1	fCNS11643	1-474C
U+5320.1	fBig5	A64B
U+5320.1	fGB	0-3D33
U+5320.1	fR/S	 22.4
U+5320.1	fTotalStrokes	 6
U+5320.1	fCangjie	SHML
U+5320.1	fFourCorner	71712
U+5320.1	fMandarin	jiang5
U+5321.0	fUTF8	匡
U+5321.0	fCNS11643	1-474B
U+5321.0	fBig5	A64A
U+5321.0	fGB	0-3F6F
U+5321.0	fR/S	 22.4
U+5321.0	fTotalStrokes	 6
U+5321.0	fCangjie	SMG
U+5321.0	fFourCorner	71711
U+5321.0	fHakka	  kiong1; siong5
U+5321.0	fCantonese	hong1
U+5321.0	fMandarin	kuang1; wang1
U+5321.0	fEnglish	[1] rectify; correct; restore; revise [2] deliver from
U+5322.0	fUTF8	匢
U+5322.0	fCNS11643	2-2221
U+5322.0	fBig5	C9C1
U+5322.0	fGB	E-236D
U+5322.0	fR/S	 22.4
U+5322.0	fTotalStrokes	 6
U+5322.0	fCangjie	SPHH
U+5322.0	fFourCorner	71712
U+5322.0	fCantonese	fat1
U+5322.0	fMandarin	hu1
U+5323.0	fUTF8	匣
U+5323.0	fCNS11643	1-493A
U+5323.0	fBig5	A758
U+5323.0	fGB	0-4F3B
U+5323.0	fR/S	 22.5
U+5323.0	fTotalStrokes	 7
U+5323.0	fCangjie	SWL
U+5323.0	fFourCorner	71715
U+5323.0	fHakka	  hap8;  ap7; gap7; hap7
U+5323.0	fCantonese	haap6; hap6 (also)
U+5323.0	fMandarin	xia2
U+5323.0	fEnglish	[1] small box; case [2] cage [3] [4] also [5] all
U+5324.0	fUTF8	匤
U+5324.0	fCNS11643	3-6450
U+5324.0	fGB	E-236E
U+5324.0	fR/S	 22.5
U+5324.0	fTotalStrokes	 7
U+5324.0	fCangjie	SMGI
U+5324.0	fFourCorner	7171
U+5324.0	fCantonese	kuk1
U+5324.0	fMandarin	qu1
U+5325.0	fUTF8	匥
U+5325.0	fCNS11643	3-2465
U+5325.0	fGB	3-3049
U+5325.0	fR/S	  22.5
U+5325.0	fTotalStrokes	 7
U+5325.0	fCangjie	SIT
U+5325.0	fFourCorner	71710
U+5325.0	fCantonese	faan4
U+5325.0	fMandarin	bian4; fan2
U+5326.0	fUTF8	匦
U+5326.0	fGB	0-5850
U+5326.0	fR/S	 22.6
U+5326.0	fTotalStrokes	 8
U+5326.0	fCangjie	SKQN
U+5326.0	fHakka	 kwui3
U+5326.0	fCantonese	gwai2
U+5326.0	fMandarin	gui3
U+5326.0	fEnglish	simplified form of &#x532D;
U+5327.0	fUTF8	匧
U+5327.0	fCNS11643	3-2B49
U+5327.0	fGB	5-3121
U+5327.0	fR/S	  22.7
U+5327.0	fTotalStrokes	 9
U+5327.0	fCangjie	SKOO
U+5327.0	fFourCorner	71710
U+5327.0	fHakka	 giap7; hiap7
U+5327.0	fCantonese	haap6
U+5327.0	fMandarin	qie4
U+5327.0	fEnglish	 trunk portfolio
U+5328.0	fUTF8	匨
U+5328.0	fCNS11643	4-284E
U+5328.0	fGB	E-236F
U+5328.0	fR/S	  22.7
U+5328.0	fTotalStrokes	 9
U+5328.0	fCangjie	SVMG
U+5328.0	fFourCorner	71710
U+5328.0	fCantonese	zong1
U+5328.0	fMandarin	zang1; cang2
U+5329.0	fUTF8	匩
U+5329.0	fCNS11643	4-2850
U+5329.0	fGB	E-2370
U+5329.0	fR/S	 22.7
U+5329.0	fTotalStrokes	 9
U+5329.0	fCangjie	SUMG
U+5329.0	fFourCorner	71711
U+5329.0	fCantonese	hong1
U+5329.0	fMandarin	kuang1
U+532A.0	fUTF8	匪
U+532A.0	fCNS11643	1-544D
U+532A.0	fBig5	ADEA
U+532A.0	fGB	0-374B
U+532A.0	fR/S	 22.8
U+532A.0	fTotalStrokes	 10
U+532A.0	fCangjie	SLMY
U+532A.0	fFourCorner	71711
U+532A.0	fHakka	 fui1;  fui3; fi1; fi3
U+532A.0	fCantonese	fei2; fei1
U+532A.0	fMandarin	fei3; fei1; fen1
U+532A.0	fEnglish	bandits; rebels; robbers; gangsters; insurgents
U+532B.0	fUTF8	匫
U+532B.0	fCNS11643	4-2B78
U+532B.0	fGB	3-304C
U+532B.0	fR/S	  22.8
U+532B.0	fTotalStrokes	 10
U+532B.0	fCangjie	SPHA
U+532B.0	fFourCorner	71710
U+532B.0	fCantonese	fat1
U+532B.0	fMandarin	hu1
U+532C.0	fUTF8	匬
U+532C.0	fCNS11643	4-305F
U+532C.0	fGB	3-304E
U+532C.0	fR/S	 22.9
U+532C.0	fTotalStrokes	 11
U+532C.0	fCangjie	SOMN
U+532C.0	fFourCorner	71710
U+532C.0	fHakka	 teu2; ji3; ri3
U+532C.0	fCantonese	jyu6
U+532C.0	fMandarin	tou2; yu3
U+532C.0	fEnglish	 stack of grain
U+532D.0	fUTF8	匭
U+532D.0	fCNS11643	2-3372
U+532D.0	fBig5	D46F
U+532D.0	fGB	1-5850
U+532D.0	fR/S	 22.9
U+532D.0	fTotalStrokes	 11
U+532D.0	fCangjie	SJJN
U+532D.0	fFourCorner	71711
U+532D.0	fHakka	 gui3; kui3; kwui3
U+532D.0	fCantonese	gwai2
U+532D.0	fMandarin	gui3
U+532D.0	fEnglish	 small box; chest; casket
U+532E.0	fUTF8	匮
U+532E.0	fGB	0-5851
U+532E.0	fR/S	 22.9
U+532E.0	fTotalStrokes	 11
U+532E.0	fCangjie	SLMO
U+532E.0	fHakka	 k'wui5
U+532E.0	fCantonese	gwai6
U+532E.0	fMandarin	kui4; gui4
U+532E.0	fEnglish	simplified form of &#x5331;
U+532F.0	fUTF8	匯
U+532F.0	fCNS11643	1-633E
U+532F.0	fBig5	B6D7
U+532F.0	fGB	1-3B63
U+532F.0	fR/S	 22.11
U+532F.0	fTotalStrokes	 13
U+532F.0	fCangjie	SEOG
U+532F.0	fFourCorner	71711
U+532F.0	fHakka	 fui5; fi5; fui3
U+532F.0	fCantonese	wui6
U+532F.0	fMandarin	hui4; hui3; kuai1; ma1
U+532F.0	fEnglish	[1] remit money [2] converge [3] flow into
U+5330.0	fUTF8	匰
U+5330.0	fCNS11643	2-492C
U+5330.0	fBig5	E145
U+5330.0	fGB	3-304D
U+5330.0	fR/S	 22.12
U+5330.0	fTotalStrokes	 14
U+5330.0	fCangjie	SRRJ
U+5330.0	fFourCorner	71714
U+5330.0	fHakka	  dan1
U+5330.0	fCantonese	daan1 (also); dan1
U+5330.0	fMandarin	dan1
U+5330.0	fEnglish	the shrine for tablets in the ancestral hall
U+5331.0	fUTF8	匱
U+5331.0	fCNS11643	1-6824
U+5331.0	fBig5	B9BC
U+5331.0	fGB	1-5851
U+5331.0	fR/S	 22.12
U+5331.0	fTotalStrokes	 14
U+5331.0	fCangjie	SLMC
U+5331.0	fFourCorner	71718
U+5331.0	fHakka	 kui3; kui6; kui5; k'wui5; kwi5
U+5331.0	fCantonese	gwai6
U+5331.0	fMandarin	kui4; gui4
U+5331.0	fEnglish	[1] lack; deficient [2] chest; cabinet [3] exhausted
U+5332.0	fUTF8	匲
U+5332.0	fCNS11643	3-4623
U+5332.0	fGB	E-2371
U+5332.0	fR/S	 22.12
U+5332.0	fTotalStrokes	 14
U+5332.0	fCangjie	SKRR
U+5332.0	fFourCorner	71710
U+5332.0	fHakka	 liam2
U+5332.0	fCantonese	lim4
U+5332.0	fMandarin	lian2
U+5333.0	fUTF8	匳
U+5333.0	fCNS11643	3-4B34
U+5333.0	fGB	E-2372
U+5333.0	fR/S	  22.13
U+5333.0	fTotalStrokes	 15
U+5333.0	fCangjie	SOMO
U+5333.0	fFourCorner	71710
U+5333.0	fHakka	 liam2
U+5333.0	fCantonese	lim4
U+5333.0	fMandarin	lian2
U+5333.0	fEnglish	 ladies toilet case with mirror; dressing case
U+5334.0	fUTF8	匴
U+5334.0	fCNS11643	2-5645
U+5334.0	fBig5	E8FA
U+5334.0	fGB	3-3051
U+5334.0	fR/S	 22.14
U+5334.0	fTotalStrokes	 16
U+5334.0	fCangjie	SHBT
U+5334.0	fFourCorner	71714
U+5334.0	fHakka	  son5
U+5334.0	fCantonese	syun3; syun2
U+5334.0	fMandarin	suan3; suan4
U+5334.0	fEnglish	 box; basket
U+5335.0	fUTF8	匵
U+5335.0	fCNS11643	3-5444
U+5335.0	fGB	5-3124
U+5335.0	fR/S	  22.15
U+5335.0	fTotalStrokes	 17
U+5335.0	fCangjie	SGWC
U+5335.0	fFourCorner	71710
U+5335.0	fHakka	  tuk8
U+5335.0	fCantonese	duk6
U+5335.0	fMandarin	du2
U+5335.0	fEnglish	 case casket
U+5336.0	fUTF8	匶
U+5336.0	fCNS11643	3-5C34
U+5336.0	fGB	E-2373
U+5336.0	fR/S	 22.17; 22.18
U+5336.0	fTotalStrokes	 20; 19
U+5336.0	fCangjie	STOX
U+5336.0	fFourCorner	71710
U+5336.0	fCantonese	gau6 (also); gau3
U+5336.0	fMandarin	jiu4
U+5337.0	fUTF8	匷
U+5337.0	fCNS11643	2-686A
U+5337.0	fBig5	F3FD
U+5337.0	fGB	E-2374
U+5337.0	fR/S	 22.18
U+5337.0	fTotalStrokes	 20
U+5337.0	fCangjie	SBUG
U+5337.0	fFourCorner	71711
U+5337.0	fCantonese	goek3; keoi4
U+5337.0	fMandarin	qu2
U+5338.0	fUTF8	匸
U+5338.0	fCNS11643	3-2129
U+5338.0	fGB	8-287A
U+5338.0	fR/S	 23.0
U+5338.0	fTotalStrokes	 2
U+5338.0	fCangjie	MV
U+5338.0	fFourCorner	71710
U+5338.0	fHakka	 hi3; hi1
U+5338.0	fCantonese	hai5; hai2
U+5338.0	fMandarin	xi3; xi4
U+5338.0	fEnglish	 box Kangxi radical number 23
U+5339.0	fUTF8	匹
U+5339.0	fCNS11643	1-4528
U+5339.0	fBig5	A4C7
U+5339.0	fGB	0-4625
U+5339.0	fR/S	 23.2
U+5339.0	fTotalStrokes	 4
U+5339.0	fCangjie	SC
U+5339.0	fFourCorner	71711
U+5339.0	fHakka	  pit7
U+5339.0	fCantonese	pat1
U+5339.0	fMandarin	pi1
U+5339.0	fEnglish	[1] bolt (of cloth) [2] match [3] equal
U+5339.1	fUTF8	匹
U+5339.1	fCNS11643	1-4528
U+5339.1	fBig5	A4C7
U+5339.1	fGB	0-4625
U+5339.1	fR/S	 23.2
U+5339.1	fTotalStrokes	 4
U+5339.1	fCangjie	SC
U+5339.1	fFourCorner	71711
U+5339.1	fMandarin	pi3
U+5339.1	fEnglish	a noun classifier for horses
U+533A.0	fUTF8	区
U+533A.0	fCNS11643	3-2157
U+533A.0	fGB	0-4778
U+533A.0	fR/S	 23.2
U+533A.0	fTotalStrokes	 4
U+533A.0	fCangjie	SK
U+533A.0	fFourCorner	71710
U+533A.0	fHakka	 ki1
U+533A.0	fCantonese	keoi1; au1; ngau1 (also)
U+533A.0	fMandarin	qu1; ou1; gou1; qiu1
U+533A.0	fEnglish	simplified form of &#x5340;
U+533B.0	fUTF8	医
U+533B.0	fCNS11643	3-2466
U+533B.0	fGB	0-523D
U+533B.0	fR/S	 23.5
U+533B.0	fTotalStrokes	 7
U+533B.0	fCangjie	SOK
U+533B.0	fFourCorner	71710
U+533B.0	fHakka	 ji1; ri1; ri5; ji5; i5
U+533B.0	fCantonese	ai3; ngai3 (also); ji1
U+533B.0	fMandarin	yi4; yi1; yi3
U+533B.0	fEnglish	simplified form of &#x91AB;
U+533C.0	fUTF8	匼
U+533C.0	fCNS11643	2-2558
U+533C.0	fBig5	CBD8
U+533C.0	fGB	3-304A
U+533C.0	fR/S	 23.6
U+533C.0	fTotalStrokes	 8
U+533C.0	fCangjie	SOMR
U+533C.0	fFourCorner	71716
U+533C.0	fCantonese	hap1; am2; ngam2 (also); ngo1 (also); o1; am3
U+533C.0	fMandarin	qia4; an4; ke4; an3; ke1
U+533D.0	fUTF8	匽
U+533D.0	fCNS11643	2-2936
U+533D.0	fBig5	CDF4
U+533D.0	fGB	3-304B
U+533D.0	fR/S	 23.7
U+533D.0	fTotalStrokes	 9
U+533D.0	fCangjie	SAV
U+533D.0	fFourCorner	71714
U+533D.0	fHakka	 jen3; ren3; jan3
U+533D.0	fCantonese	jin2
U+533D.0	fMandarin	yan3; yan4
U+533D.0	fEnglish	 hide; secrete; repress bend
U+533E.0	fUTF8	匾
U+533E.0	fCNS11643	1-5935
U+533E.0	fBig5	B0D0
U+533E.0	fGB	0-5852
U+533E.0	fR/S	 23.9
U+533E.0	fTotalStrokes	 11
U+533E.0	fCangjie	SHSB
U+533E.0	fFourCorner	71712
U+533E.0	fHakka	  bien3; ben3; pian3
U+533E.0	fCantonese	bin2
U+533E.0	fMandarin	bian3
U+533E.0	fEnglish	 wooden tablet
U+533F.0	fUTF8	匿
U+533F.0	fCNS11643	1-5933
U+533F.0	fBig5	B0CE
U+533F.0	fGB	0-4464
U+533F.0	fR/S	 23.9; 22.8
U+533F.0	fTotalStrokes	 11
U+533F.0	fCangjie	STKR
U+533F.0	fFourCorner	71716
U+533F.0	fHakka	 nit8; net8; lit8; lik8; lit7
U+533F.0	fCantonese	nik1; lik1 (also); tik1; nik6
U+533F.0	fMandarin	ni4; te4
U+533F.0	fEnglish	 hide; go into hiding; conceal
U+5340.0	fUTF8	區
U+5340.0	fCNS11643	1-5934
U+5340.0	fBig5	B0CF
U+5340.0	fGB	1-4778
U+5340.0	fR/S	 23.9
U+5340.0	fTotalStrokes	 11
U+5340.0	fCangjie	SRRR
U+5340.0	fFourCorner	71716
U+5340.0	fHakka	  ki1; kia1; iu1
U+5340.0	fCantonese	keoi1; au1; ngau1 (also)
U+5340.0	fMandarin	qu1; ou1; gou1; qiu1
U+5340.0	fEnglish	[1] distinguish; discriminate [2] area; district; region; zone [3] border [4] little; few
U+5341.0	fUTF8	十
U+5341.0	fCNS11643	1-4432
U+5341.0	fBig5	A451
U+5341.0	fGB	0-4A2E
U+5341.0	fR/S	 24.0
U+5341.0	fTotalStrokes	 2
U+5341.0	fCangjie	J
U+5341.0	fFourCorner	40000
U+5341.0	fHakka	  ship8;  sip8; siip8
U+5341.0	fCantonese	sap6
U+5341.0	fMandarin	shi2
U+5341.0	fEnglish	[1] ten; tenth [2] complete; completely; perfect; extremely; perfectly
U+5342.0	fUTF8	卂
U+5342.0	fCNS11643	4-2130
U+5342.0	fGB	5-303A
U+5342.0	fR/S	  24.1
U+5342.0	fTotalStrokes	 3
U+5342.0	fCangjie	NJ
U+5342.0	fFourCorner	17410
U+5342.0	fCantonese	seon3
U+5342.0	fMandarin	xin4
U+5342.0	fEnglish	 fly rapidly
U+5343.0	fUTF8	千
U+5343.0	fCNS11643	1-4445
U+5343.0	fBig5	A464
U+5343.0	fGB	0-4727
U+5343.0	fR/S	 24.1
U+5343.0	fTotalStrokes	 3
U+5343.0	fCangjie	HJ
U+5343.0	fFourCorner	20400
U+5343.0	fHakka	 cien1; cen1; ts'ian1
U+5343.0	fCantonese	cin1
U+5343.0	fMandarin	qian1
U+5343.0	fEnglish	[1] thousand [2] many; numerous; very [3] a Chinese family name [4] [high clipped tone] card cheat; swindler cheat at cards; swindle
U+5344.0	fUTF8	卄
U+5344.0	fCNS11643	3-2138
U+5344.0	fBig5	A2CD
U+5344.0	fGB	5-3022
U+5344.0	fR/S	 24.1; 1.2
U+5344.0	fTotalStrokes	 3
U+5344.0	fCangjie	JJ
U+5344.0	fFourCorner	44000
U+5344.0	fHakka	  ngip8
U+5344.0	fCantonese	nim6; jaa6 (also); lim6 (also); je6 (also)
U+5344.0	fMandarin	nian4; ru4
U+5344.0	fEnglish	twenty; twentieth
U+5345.0	fUTF8	卅
U+5345.0	fCNS11643	1-452B
U+5345.0	fBig5	A4CA
U+5345.0	fGB	0-5826
U+5345.0	fR/S	 24.2
U+5345.0	fTotalStrokes	 4
U+5345.0	fCangjie	TJ
U+5345.0	fFourCorner	44000
U+5345.0	fHakka	  sap7;  sa1
U+5345.0	fCantonese	saa1; saa4
U+5345.0	fMandarin	sa4
U+5345.0	fEnglish	thirty; thirtieth
U+5346.0	fUTF8	卆
U+5346.0	fCNS11643	3-2159
U+5346.0	fGB	E-2375
U+5346.0	fR/S	 24.2
U+5346.0	fTotalStrokes	 4
U+5346.0	fCangjie	KNJ
U+5346.0	fFourCorner	47400
U+5346.0	fCantonese	zeot1
U+5346.0	fMandarin	zu2; cu4
U+5346.0	fEnglish	 soldier servant at last; finally
U+5347.0	fUTF8	升
U+5347.0	fCNS11643	1-452A
U+5347.0	fBig5	A4C9
U+5347.0	fGB	0-497D
U+5347.0	fR/S	 24.2
U+5347.0	fTotalStrokes	 4
U+5347.0	fCangjie	HT
U+5347.0	fFourCorner	24400
U+5347.0	fHakka	  shin1;  sin1; siin1
U+5347.0	fCantonese	sing1
U+5347.0	fMandarin	sheng1
U+5347.0	fEnglish	[1] rise; arise; raise; go up; ascend [2] advance; promote [3] a unit of volume measurement (especially for grain) [4] a Chinese family name
U+5348.0	fUTF8	午
U+5348.0	fCNS11643	1-4529
U+5348.0	fBig5	A4C8
U+5348.0	fGB	0-4E67
U+5348.0	fR/S	 24.2
U+5348.0	fTotalStrokes	 4
U+5348.0	fCangjie	OJ
U+5348.0	fFourCorner	80400
U+5348.0	fHakka	  ng3
U+5348.0	fCantonese	ng5; ng2 (also)
U+5348.0	fMandarin	wu3
U+5348.0	fEnglish	[1] noon; high noon [2] 11 am to 1 PM (in Old Chinese time measurement [3] seventh terrestrial branch [4] a Chinese family name
U+5348.1	fUTF8	午
U+5348.1	fCNS11643	1-4529
U+5348.1	fBig5	A4C8
U+5348.1	fGB	0-4E67
U+5348.1	fR/S	 24.2
U+5348.1	fTotalStrokes	 4
U+5348.1	fCangjie	OJ
U+5348.1	fFourCorner	80400
U+5348.1	fMandarin	wu5
U+5349.0	fUTF8	卉
U+5349.0	fCNS11643	1-4625
U+5349.0	fBig5	A563
U+5349.0	fGB	0-3B5C
U+5349.0	fR/S	 24.3
U+5349.0	fTotalStrokes	 5
U+5349.0	fCangjie	JT
U+5349.0	fFourCorner	40440
U+5349.0	fHakka	 fui3; wui3; fi3; wi3; vui3
U+5349.0	fCantonese	wai2; wai5 (also)
U+5349.0	fMandarin	hui4; hui3
U+5349.0	fEnglish	 general term for plants myriads
U+534A.0	fUTF8	半
U+534A.0	fCNS11643	1-4624
U+534A.0	fBig5	A562
U+534A.0	fGB	0-306B
U+534A.0	fR/S	 24.3
U+534A.0	fTotalStrokes	 5
U+534A.0	fCangjie	FQ
U+534A.0	fFourCorner	90500
U+534A.0	fHakka	  ban5
U+534A.0	fCantonese	bun3
U+534A.0	fMandarin	ban4; pan4
U+534A.0	fEnglish	[1] half [2] very little [3] in the middle [4] partly; about half
U+534B.0	fUTF8	卋
U+534B.0	fCNS11643	3-232A
U+534B.0	fGB	E-2376
U+534B.0	fR/S	 1.5; 24.4
U+534B.0	fTotalStrokes	 6
U+534B.0	fCangjie	JT
U+534B.0	fFourCorner	40770
U+534B.0	fCantonese	sai3
U+534B.0	fMandarin	shi4
U+534B.0	fEnglish	same as &#x4E16;
U+534C.0	fUTF8	卌
U+534C.0	fCNS11643	2-214E
U+534C.0	fBig5	C96D
U+534C.0	fGB	3-3022
U+534C.0	fR/S	 24.4; 24.3
U+534C.0	fTotalStrokes	 5
U+534C.0	fCangjie	TT
U+534C.0	fFourCorner	44000
U+534C.0	fCantonese	sei3; se3
U+534C.0	fMandarin	xi4
U+534C.0	fEnglish	forty; 40
U+534D.0	fUTF8	卍
U+534D.0	fCNS11643	2-2223
U+534D.0	fBig5	C9C3
U+534D.0	fGB	5-3037
U+534D.0	fR/S	 24.4
U+534D.0	fTotalStrokes	 6
U+534D.0	fCangjie	NX
U+534D.0	fFourCorner	12217
U+534D.0	fHakka	 wan6; wan5; wan3; van5
U+534D.0	fCantonese	maan6
U+534D.0	fMandarin	wan4
U+534D.0	fEnglish	swastika - fourth of auspicious
U+534E.0	fUTF8	华
U+534E.0	fCNS11643	F-222C
U+534E.0	fGB	0-3B2A
U+534E.0	fR/S	 24.4
U+534E.0	fTotalStrokes	 6
U+534E.0	fCangjie	OPJ
U+534E.0	fHakka	 fa2; fa1
U+534E.0	fCantonese	waa4; waa2 (also); faa1; waa6
U+534E.0	fMandarin	hua2; hua4; kua1; hua1
U+534E.0	fEnglish	simplified form of &#x83EF;
U+534F.0	fUTF8	协
U+534F.0	fCNS11643	3-232B
U+534F.0	fGB	0-502D
U+534F.0	fR/S	 24.4
U+534F.0	fTotalStrokes	 6
U+534F.0	fCangjie	JKSC
U+534F.0	fFourCorner	47030
U+534F.0	fHakka	 hiap8
U+534F.0	fCantonese	hip3; hip6
U+534F.0	fMandarin	xie2
U+534F.0	fEnglish	simplified form of &#x5354;
U+5350.0	fUTF8	卐
U+5350.0	fCNS11643	3-232D
U+5350.0	fGB	5-303E
U+5350.0	fR/S	 24.4
U+5350.0	fTotalStrokes	 6
U+5350.0	fCangjie	VX
U+5350.0	fFourCorner	21120
U+5350.0	fCantonese	maan6
U+5350.0	fMandarin	wan4
U+5350.0	fEnglish	swastika - fourth of auspicious
U+5351.0	fUTF8	卑
U+5351.0	fCNS11643	1-4C38
U+5351.0	fBig5	A8F5
U+5351.0	fGB	0-3130
U+5351.0	fR/S	 24.6
U+5351.0	fTotalStrokes	 8
U+5351.0	fCangjie	HWHJ
U+5351.0	fFourCorner	26400
U+5351.0	fHakka	  bi1; bi3
U+5351.0	fCantonese	bei1
U+5351.0	fMandarin	bei1; ban1; bi4; pei1
U+5351.0	fEnglish	[1] humble; low [2] debased; depraved; vile [3] inferior [4] a modest expression referring to oneself
U+5352.0	fUTF8	卒
U+5352.0	fCNS11643	1-4C35
U+5352.0	fBig5	A8F2
U+5352.0	fGB	0-5764
U+5352.0	fR/S	 24.6
U+5352.0	fTotalStrokes	 8
U+5352.0	fCangjie	YOOJ
U+5352.0	fFourCorner	00408
U+5352.0	fHakka	  zut7; cut7
U+5352.0	fCantonese	zeot1; cyut3
U+5352.0	fMandarin	zu2
U+5352.0	fEnglish	[1] soldier [2] servant; underling; lackey [3] unit of one hundred soldiers [4] at last; finally; after all; at long last
U+5352.1	fUTF8	卒
U+5352.1	fCNS11643	1-4C35
U+5352.1	fBig5	A8F2
U+5352.1	fGB	0-5764
U+5352.1	fR/S	 24.6
U+5352.1	fTotalStrokes	 8
U+5352.1	fCangjie	YOOJ
U+5352.1	fFourCorner	00408
U+5352.1	fMandarin	cu4; cui4
U+5352.1	fEnglish	suddenly; abruptly; hurriedly
U+5353.0	fUTF8	卓
U+5353.0	fCNS11643	1-4C37
U+5353.0	fBig5	A8F4
U+5353.0	fGB	0-573F
U+5353.0	fR/S	 24.6
U+5353.0	fTotalStrokes	 8
U+5353.0	fCangjie	YAJ
U+5353.0	fFourCorner	21406
U+5353.0	fHakka	  zok7
U+5353.0	fCantonese	coek3; zoek3 (also)
U+5353.0	fMandarin	zhuo1; zhuo2
U+5353.0	fEnglish	[1] profound; brilliant; eminent [2] lofty; high [3] (to stand) upright; erect [4] a Chinese family name
U+5354.0	fUTF8	協
U+5354.0	fCNS11643	1-4C36
U+5354.0	fBig5	A8F3
U+5354.0	fGB	1-502D
U+5354.0	fR/S	 24.6
U+5354.0	fTotalStrokes	 8
U+5354.0	fCangjie	JKSS
U+5354.0	fFourCorner	44027
U+5354.0	fHakka	  hiap8
U+5354.0	fCantonese	hip3; hip6
U+5354.0	fMandarin	xie2
U+5354.0	fEnglish	[1] agree; an agreement [2] be united; cooperate; bring into harmony; coordinate [3] assist; aid; help
U+5355.0	fUTF8	单
U+5355.0	fCNS11643	3-2763
U+5355.0	fGB	0-3525
U+5355.0	fR/S	 12.6; 24.6
U+5355.0	fTotalStrokes	 8
U+5355.0	fCangjie	CWJ
U+5355.0	fFourCorner	80500
U+5355.0	fHakka	 dan1
U+5355.0	fCantonese	daan1; sin6; sim4
U+5355.0	fMandarin	dan1; chan1; shan4; tan2; chan2; yue4; zhan4
U+5355.0	fEnglish	simplified form of &#x55AE;
U+5356.0	fUTF8	卖
U+5356.0	fGB	0-4274
U+5356.0	fR/S	 24.6
U+5356.0	fTotalStrokes	 8
U+5356.0	fCangjie	JNYK
U+5356.0	fHakka	 mai5; mai3
U+5356.0	fCantonese	maai6; maai2 (also)
U+5356.0	fMandarin	mai4; yu4
U+5356.0	fEnglish	simplified form of &#x8CE3;
U+5356.1	fUTF8	卖
U+5356.1	fGB	0-4274
U+5356.1	fR/S	 24.6
U+5356.1	fTotalStrokes	 8
U+5356.1	fCangjie	JNYK
U+5356.1	fMandarin	mai5
U+5357.0	fUTF8	南
U+5357.0	fCNS11643	1-5032
U+5357.0	fBig5	AB6E
U+5357.0	fGB	0-444F
U+5357.0	fR/S	 24.7
U+5357.0	fTotalStrokes	 9
U+5357.0	fCangjie	JBTJ
U+5357.0	fFourCorner	40227
U+5357.0	fHakka	  nam2;  lam2; nam3
U+5357.0	fCantonese	naam4; laam4 (also); naam2 (also); naa1 (also); laam2 (also); naa4; laa4 (also); laa1 (also); lam4
U+5357.0	fMandarin	nan2
U+5357.0	fEnglish	[1] south; southern part; southward [2] a type of ancient music played in the south of China [3] a Chinese family name
U+5357.1	fUTF8	南
U+5357.1	fCNS11643	1-5032
U+5357.1	fBig5	AB6E
U+5357.1	fGB	0-444F
U+5357.1	fR/S	 24.7
U+5357.1	fTotalStrokes	 9
U+5357.1	fCangjie	JBTJ
U+5357.1	fFourCorner	40227
U+5357.1	fMandarin	na2; na1
U+5357.1	fEnglish	used for transliteration of Sanskrit
U+5358.0	fUTF8	単
U+5358.0	fCNS11643	3-2B4A
U+5358.0	fGB	E-2377
U+5358.0	fR/S	 24.7; 42.6
U+5358.0	fTotalStrokes	 9
U+5358.0	fCangjie	FWJ
U+5358.0	fFourCorner	23506
U+5358.0	fCantonese	daan1
U+5358.0	fMandarin	dan1; chan2; shan4
U+5358.0	fEnglish	same as &#x55AE;
U+5359.0	fUTF8	卙
U+5359.0	fCNS11643	4-3060
U+5359.0	fGB	5-3049
U+5359.0	fR/S	  24.9
U+5359.0	fTotalStrokes	 11
U+5359.0	fCangjie	TVJ
U+5359.0	fFourCorner	44700
U+5359.0	fHakka	  sip8; zhim1; zihm1
U+5359.0	fCantonese	zap1
U+5359.0	fMandarin	ji2
U+5359.0	fEnglish	[1] full; abundant [2] collect; assemble
U+535A.0	fUTF8	博
U+535A.0	fCNS11643	1-5E3B
U+535A.0	fBig5	B3D5
U+535A.0	fGB	0-3229
U+535A.0	fR/S	 24.10
U+535A.0	fTotalStrokes	 12
U+535A.0	fCangjie	JIBI
U+535A.0	fFourCorner	43046
U+535A.0	fHakka	  bok7
U+535A.0	fCantonese	bok3
U+535A.0	fMandarin	bo2
U+535A.0	fEnglish	[1] wide; extensive [2] abundant; ample; rich [3] broadly knowledgeable; well-read; learned; eruditegamble [4] barter for; to exchange [5] play games; gamble [6] win; gain [7] a Chinese family name
U+535B.0	fUTF8	卛
U+535B.0	fCNS11643	4-6630
U+535B.0	fGB	E-2378
U+535B.0	fR/S	 24.19
U+535B.0	fTotalStrokes	 21
U+535B.0	fCangjie	VFJ
U+535B.0	fFourCorner	22400
U+535B.0	fCantonese	seot1
U+535B.0	fMandarin	shuai4; luu4
U+535C.0	fUTF8	卜
U+535C.0	fCNS11643	1-4433
U+535C.0	fBig5	A452
U+535C.0	fGB	0-3237
U+535C.0	fR/S	 25.0
U+535C.0	fTotalStrokes	 2
U+535C.0	fCangjie	Y
U+535C.0	fFourCorner	23000
U+535C.0	fHakka	  buk7; pet8
U+535C.0	fCantonese	buk1; baak6; pak6
U+535C.0	fMandarin	bu3; bo5
U+535C.0	fEnglish	[1] divine; consult the oracle [2] foretell; predict [3] choose; select [4] a Chinese family name
U+535D.0	fUTF8	卝
U+535D.0	fCNS11643	3-215A
U+535D.0	fGB	5-312C
U+535D.0	fR/S	  25.2
U+535D.0	fTotalStrokes	 4
U+535D.0	fCangjie	LMY
U+535D.0	fFourCorner	22000
U+535D.0	fCantonese	gwaan3; kong3; kwong3 (also)
U+535D.0	fMandarin	kuang4
U+535D.0	fEnglish	 hair style ore
U+535E.0	fUTF8	卞
U+535E.0	fCNS11643	1-452C
U+535E.0	fBig5	A4CB
U+535E.0	fGB	0-3165
U+535E.0	fR/S	 25.2
U+535E.0	fTotalStrokes	 4
U+535E.0	fCangjie	YY
U+535E.0	fFourCorner	00230
U+535E.0	fHakka	 pien5; pien6; pen3; pien3
U+535E.0	fCantonese	bin6
U+535E.0	fMandarin	bian4
U+535E.0	fEnglish	[1] be impatient; be in hurry; rash; impetuous [2] excitable [3] a Chinese family name
U+535F.0	fUTF8	卟
U+535F.0	fCNS11643	3-2227
U+535F.0	fGB	0-5F32
U+535F.0	fR/S	 30.2; 25.3; 25.2
U+535F.0	fTotalStrokes	 5
U+535F.0	fCangjie	RY
U+535F.0	fFourCorner	63000
U+535F.0	fHakka	  gi1
U+535F.0	fCantonese	buk1
U+535F.0	fMandarin	bu3
U+535F.0	fEnglish	 a chemical compound divination consider
U+5360.0	fUTF8	占
U+5360.0	fCNS11643	1-4627
U+5360.0	fBig5	A565
U+5360.0	fGB	0-553C
U+5360.0	fR/S	 25.3
U+5360.0	fTotalStrokes	 5
U+5360.0	fCangjie	YR
U+5360.0	fFourCorner	21600
U+5360.0	fHakka	  zam5; zham1;  zam1;  zham5
U+5360.0	fCantonese	zim1; zim3
U+5360.0	fMandarin	zhan1
U+5360.0	fEnglish	[1] divine [2] observe [3] a Chinese family name
U+5360.1	fUTF8	占
U+5360.1	fCNS11643	1-4627
U+5360.1	fBig5	A565
U+5360.1	fGB	0-553C
U+5360.1	fR/S	 25.3
U+5360.1	fTotalStrokes	 5
U+5360.1	fCangjie	YR
U+5360.1	fFourCorner	21600
U+5360.1	fMandarin	zhan4
U+5360.1	fEnglish	 occupy
U+5361.0	fUTF8	卡
U+5361.0	fCNS11643	1-4626
U+5361.0	fBig5	A564
U+5361.0	fGB	0-3F28
U+5361.0	fR/S	 25.3
U+5361.0	fTotalStrokes	 5
U+5361.0	fCangjie	YMY
U+5361.0	fFourCorner	21231
U+5361.0	fHakka	  ka2;  ka5; ka3; cap8; ka6; ka1; cap7
U+5361.0	fCantonese	kaa1; kaa2 (also); kaat1; zaap6
U+5361.0	fMandarin	ka3; cha4
U+5361.0	fEnglish	[1] card; punch card; cardboard [2] abbreviated form for calorie [3] guardhouse [4] customs barrier; roadblock; checkpoint [5] block; check
U+5361.1	fUTF8	卡
U+5361.1	fCNS11643	1-4626
U+5361.1	fBig5	A564
U+5361.1	fGB	0-3F28
U+5361.1	fR/S	 25.3
U+5361.1	fTotalStrokes	 5
U+5361.1	fCangjie	YMY
U+5361.1	fFourCorner	21231
U+5361.1	fMandarin	qia2
U+5361.1	fEnglish	 be squeezed in between; be sandwiched
U+5361.2	fUTF8	卡
U+5361.2	fCNS11643	1-4626
U+5361.2	fBig5	A564
U+5361.2	fGB	0-3F28
U+5361.2	fR/S	 25.3
U+5361.2	fTotalStrokes	 5
U+5361.2	fCangjie	YMY
U+5361.2	fFourCorner	21231
U+5361.2	fMandarin	qia3
U+5361.2	fEnglish	 choke; be choked
U+5362.0	fUTF8	卢
U+5362.0	fGB	0-422C
U+5362.0	fR/S	 25.3
U+5362.0	fTotalStrokes	 5
U+5362.0	fCangjie	YS
U+5362.0	fHakka	 lu2
U+5362.0	fCantonese	lou4; lou2 (also)
U+5362.0	fMandarin	lu2; luu2; lei2
U+5362.0	fEnglish	simplified form of &#x76E7;
U+5363.0	fUTF8	卣
U+5363.0	fCNS11643	2-2334
U+5363.0	fBig5	CA72
U+5363.0	fGB	0-5855
U+5363.0	fR/S	 25.5
U+5363.0	fTotalStrokes	 7
U+5363.0	fCangjie	YWS
U+5363.0	fFourCorner	21600
U+5363.0	fHakka	 ju3; riu3
U+5363.0	fCantonese	jau5
U+5363.0	fMandarin	you3; you4
U+5363.0	fEnglish	 container or vessel for wine; Chinese chalice (in ancient China)
U+5364.0	fUTF8	卤
U+5364.0	fCNS11643	4-234D
U+5364.0	fGB	0-4231
U+5364.0	fR/S	 25.5
U+5364.0	fTotalStrokes	 7; 9
U+5364.0	fCangjie	YWK
U+5364.0	fFourCorner	21600
U+5364.0	fHakka	 lu1; lu3
U+5364.0	fCantonese	lou5
U+5364.0	fMandarin	lu3; chi3; di2
U+5364.0	fEnglish	simplified form of &#x6EF7;
U+5365.0	fUTF8	卥
U+5365.0	fCNS11643	3-6451
U+5365.0	fGB	E-2379
U+5365.0	fR/S	 25.6
U+5365.0	fTotalStrokes	 8
U+5365.0	fCangjie	YWKK
U+5365.0	fFourCorner	2160
U+5365.0	fCantonese	sai1
U+5365.0	fMandarin	xi1
U+5366.0	fUTF8	卦
U+5366.0	fCNS11643	1-4C39
U+5366.0	fBig5	A8F6
U+5366.0	fGB	0-5854
U+5366.0	fR/S	 25.6
U+5366.0	fTotalStrokes	 8
U+5366.0	fCangjie	GGY
U+5366.0	fFourCorner	43100
U+5366.0	fHakka	  gwa5; ga5; kua5
U+5366.0	fCantonese	gwaa3
U+5366.0	fMandarin	gua4
U+5366.0	fEnglish	one of the eight diagrams of the Book of Changes
U+5367.0	fUTF8	卧
U+5367.0	fCNS11643	3-2A5C
U+5367.0	fGB	0-4E54
U+5367.0	fR/S	 131.2; 25.6
U+5367.0	fTotalStrokes	 8
U+5367.0	fCangjie	SLY
U+5367.0	fFourCorner	73700
U+5367.0	fHakka	 ngo3; ngo5
U+5367.0	fCantonese	ngo6; o6
U+5367.0	fMandarin	wo4
U+5367.0	fEnglish	 lie down crouch
U+5368.0	fUTF8	卨
U+5368.0	fCNS11643	4-3064
U+5368.0	fGB	3-3055
U+5368.0	fR/S	 25.9
U+5368.0	fTotalStrokes	 11
U+5368.0	fCangjie	YBBR
U+5368.0	fFourCorner	21220
U+5368.0	fCantonese	sit3; jyu5
U+5368.0	fMandarin	xie4
U+5369.0	fUTF8	卩
U+5369.0	fCNS11643	3-212A
U+5369.0	fGB	0-5A60
U+5369.0	fR/S	 26.0
U+5369.0	fTotalStrokes	 2
U+5369.0	fCangjie	SL
U+5369.0	fFourCorner	77220
U+5369.0	fHakka	 ziet7; zet7
U+5369.0	fCantonese	zit3
U+5369.0	fMandarin	jie2; bu4
U+5369.0	fEnglish	 seal Kangxi radical number 26
U+536A.0	fUTF8	卪
U+536A.0	fCNS11643	4-2131
U+536A.0	fGB	1-7D39
U+536A.0	fR/S	  26.1
U+536A.0	fTotalStrokes	 3
U+536A.0	fCangjie	SLI
U+536A.0	fFourCorner	77220
U+536A.0	fCantonese	zit3
U+536A.0	fMandarin	jie2
U+536A.0	fEnglish	 kwukyel
U+536B.0	fUTF8	卫
U+536B.0	fGB	0-4E40
U+536B.0	fR/S	 5.2; 26.1
U+536B.0	fTotalStrokes	 3
U+536B.0	fCangjie	SLM
U+536B.0	fHakka	 vui5
U+536B.0	fCantonese	wai6
U+536B.0	fMandarin	wei4
U+536B.0	fEnglish	simplified form of &#x885B;
U+536C.0	fUTF8	卬
U+536C.0	fCNS11643	2-2137
U+536C.0	fBig5	C957
U+536C.0	fGB	3-326A
U+536C.0	fR/S	 26.2
U+536C.0	fTotalStrokes	 4
U+536C.0	fCangjie	HVSL
U+536C.0	fFourCorner	27720
U+536C.0	fHakka	  ngong1
U+536C.0	fCantonese	ngong4; joeng5 (also)
U+536C.0	fMandarin	yang3; ang2
U+536C.0	fEnglish	[1] high; high-priced [2] raise the head; self-possessed [3] used as the pronoun I
U+536D.0	fUTF8	卭
U+536D.0	fCNS11643	3-2228
U+536D.0	fGB	5-354A
U+536D.0	fR/S	  26.3
U+536D.0	fTotalStrokes	 5
U+536D.0	fCangjie	MSL
U+536D.0	fFourCorner	17120
U+536D.0	fHakka	 ngong1
U+536D.0	fCantonese	kung4
U+536D.0	fMandarin	qiong2
U+536D.0	fEnglish	high; raise
U+536E.0	fUTF8	卮
U+536E.0	fCNS11643	1-4629
U+536E.0	fBig5	A567
U+536E.0	fGB	0-5834
U+536E.0	fR/S	 26.3
U+536E.0	fTotalStrokes	 5
U+536E.0	fCangjie	HMSU
U+536E.0	fFourCorner	72217
U+536E.0	fHakka	 zi1
U+536E.0	fCantonese	zi1; zi6
U+536E.0	fMandarin	zhi1
U+536E.0	fEnglish	 container for holding wine; goblet with handles
U+536F.0	fUTF8	卯
U+536F.0	fCNS11643	1-4628
U+536F.0	fBig5	A566
U+536F.0	fGB	0-432E
U+536F.0	fR/S	 26.3
U+536F.0	fTotalStrokes	 5
U+536F.0	fCangjie	HHSL
U+536F.0	fFourCorner	77720
U+536F.0	fHakka	  mau3; mau1
U+536F.0	fCantonese	maau5; mau3
U+536F.0	fMandarin	mao3
U+536F.0	fEnglish	[1] the 4th of the Terrestrial Branches [2] the period from 5 to 7 am [3] roll call [4] a Chinese family name
U+5370.0	fUTF8	印
U+5370.0	fCNS11643	1-474D
U+5370.0	fBig5	A64C
U+5370.0	fGB	0-5321
U+5370.0	fR/S	 26.4
U+5370.0	fTotalStrokes	 6
U+5370.0	fCangjie	HPSL
U+5370.0	fFourCorner	77720
U+5370.0	fHakka	  jin5; rin5; rang5; jang5
U+5370.0	fCantonese	jan3; ngan3 (also); an3 (also)
U+5370.0	fMandarin	yin4
U+5370.0	fEnglish	[1] print; stamp; imprint [2] seal; stamp; chop [3] imprint; mark [4] a Chinese family name
U+5371.0	fUTF8	危
U+5371.0	fCNS11643	1-474E
U+5371.0	fBig5	A64D
U+5371.0	fGB	0-4E23
U+5371.0	fR/S	 26.4
U+5371.0	fTotalStrokes	 6
U+5371.0	fCangjie	NMSU
U+5371.0	fFourCorner	27212
U+5371.0	fHakka	  ngui2; ngui3
U+5371.0	fCantonese	ngai4; ai4 (also)
U+5371.0	fMandarin	wei1; wei2
U+5371.0	fEnglish	[1] dangerous; danger; precarious; perilous [2] restless [3] fear; be upset; be afraid [4] lofty; high [5] just; honest; straightforward [6] a Chinese family name
U+5372.0	fUTF8	卲
U+5372.0	fCNS11643	2-2335
U+5372.0	fBig5	CA73
U+5372.0	fGB	3-326B
U+5372.0	fR/S	 26.5
U+5372.0	fTotalStrokes	 7
U+5372.0	fCangjie	SRSL
U+5372.0	fFourCorner	17620
U+5372.0	fHakka	 shau5; seu5
U+5372.0	fCantonese	siu6
U+5372.0	fMandarin	shao4; shao1
U+5372.0	fEnglish	 eminent; lofty beautiful a Chinese family name
U+5373.0	fUTF8	即
U+5373.0	fCNS11643	1-493B
U+5373.0	fBig5	A759
U+5373.0	fGB	0-3C34
U+5373.0	fR/S	 26.5
U+5373.0	fTotalStrokes	 7
U+5373.0	fCangjie	AISL
U+5373.0	fFourCorner	77720
U+5373.0	fHakka	  zit7
U+5373.0	fCantonese	zik1
U+5373.0	fMandarin	ji2
U+5373.0	fEnglish	promptly; quickly; immediately
U+5374.0	fUTF8	却
U+5374.0	fCNS11643	3-2467
U+5374.0	fGB	0-4834
U+5374.0	fR/S	 26.5
U+5374.0	fTotalStrokes	 7
U+5374.0	fCangjie	GISL
U+5374.0	fFourCorner	47720
U+5374.0	fHakka	  kiok7
U+5374.0	fCantonese	koek3
U+5374.0	fMandarin	que4
U+5374.0	fEnglish	 still; but decline retreat
U+5375.0	fUTF8	卵
U+5375.0	fCNS11643	1-493C
U+5375.0	fBig5	A75A
U+5375.0	fGB	0-4251
U+5375.0	fR/S	 26.5
U+5375.0	fTotalStrokes	 7
U+5375.0	fCangjie	HHSLI
U+5375.0	fFourCorner	77720
U+5375.0	fHakka	  lon3
U+5375.0	fCantonese	leon2; leon5 (also); lo5
U+5375.0	fMandarin	luan3; gun1; kun1; lan3; luo3
U+5375.0	fEnglish	[1] egg; ovum [2] fish roe; spawn [3] testicles
U+5376.0	fUTF8	卶
U+5376.0	fCNS11643	3-2765
U+5376.0	fGB	5-3379
U+5376.0	fR/S	 26.6
U+5376.0	fTotalStrokes	 8
U+5376.0	fCangjie	NNSL
U+5376.0	fFourCorner	27220
U+5376.0	fCantonese	ci2
U+5376.0	fMandarin	shi4
U+5377.0	fUTF8	卷
U+5377.0	fCNS11643	1-4C3A
U+5377.0	fBig5	A8F7
U+5377.0	fGB	0-3E6D
U+5377.0	fR/S	 26.6
U+5377.0	fTotalStrokes	 8
U+5377.0	fCangjie	FQSU
U+5377.0	fFourCorner	90712
U+5377.0	fHakka	 gen3; gien3; k'ian5; kian3
U+5377.0	fCantonese	gyun2; gyun3 (also); kyun4
U+5377.0	fMandarin	juan3; gun3
U+5377.0	fEnglish	 roll up
U+5377.1	fUTF8	卷
U+5377.1	fCNS11643	1-4C3A
U+5377.1	fBig5	A8F7
U+5377.1	fGB	0-3E6D
U+5377.1	fR/S	 26.6
U+5377.1	fTotalStrokes	 8
U+5377.1	fCangjie	FQSU
U+5377.1	fFourCorner	90712
U+5377.1	fMandarin	juan4
U+5377.1	fEnglish	[1] scroll; painting which can be easily folded or rolled up [2] book [3] division of a book; volume [4] test paper [5] filed documents; files
U+5378.0	fUTF8	卸
U+5378.0	fCNS11643	1-4C3B
U+5378.0	fBig5	A8F8
U+5378.0	fGB	0-5036
U+5378.0	fR/S	 26.6; 26.7
U+5378.0	fTotalStrokes	 8
U+5378.0	fCangjie	OMSL
U+5378.0	fFourCorner	87120
U+5378.0	fHakka	  sia5
U+5378.0	fCantonese	se3
U+5378.0	fMandarin	xie4
U+5378.0	fEnglish	[1] get rid of; remove [2] unload (cargoes, etc.) [3] lay down; retire from office; resign
U+5379.0	fUTF8	卹
U+5379.0	fCNS11643	1-4C3C
U+5379.0	fBig5	A8F9
U+5379.0	fGB	E-237A
U+5379.0	fR/S	 26.6
U+5379.0	fTotalStrokes	 8
U+5379.0	fCangjie	HTSL
U+5379.0	fFourCorner	27120
U+5379.0	fHakka	 sit7;  sut7; sut8; sot8
U+5379.0	fCantonese	seot1
U+5379.0	fMandarin	xu4; su4
U+5379.0	fEnglish	[1] sympathy; pity; sympathise [2] comfort; give relieve
U+537A.0	fUTF8	卺
U+537A.0	fCNS11643	3-2766
U+537A.0	fGB	0-5A61
U+537A.0	fR/S	  26.6
U+537A.0	fTotalStrokes	 8
U+537A.0	fCangjie	NEMSU
U+537A.0	fFourCorner	17710
U+537A.0	fHakka	 giun3
U+537A.0	fCantonese	gan2; kan2
U+537A.0	fMandarin	jin3
U+537A.0	fEnglish	(nuptial) winecups (made from a gourd split in two)
U+537B.0	fUTF8	卻
U+537B.0	fCNS11643	1-5033
U+537B.0	fBig5	AB6F
U+537B.0	fGB	E-237B
U+537B.0	fR/S	 26.7
U+537B.0	fTotalStrokes	 9
U+537B.0	fCangjie	CRSL
U+537B.0	fFourCorner	87620
U+537B.0	fHakka	 kiok7; hiok7; hiak7
U+537B.0	fCantonese	koek3
U+537B.0	fMandarin	que4; jue2
U+537B.0	fEnglish	[1] still; but; yet [2] refuse; accept; decline [3] retreat; withdraw
U+537C.0	fUTF8	卼
U+537C.0	fCNS11643	2-2937
U+537C.0	fBig5	CDF5
U+537C.0	fGB	5-3759
U+537C.0	fR/S	 26.7
U+537C.0	fTotalStrokes	 9
U+537C.0	fCangjie	MUNMU
U+537C.0	fFourCorner	17212
U+537C.0	fHakka	 ngut7
U+537C.0	fCantonese	ngat6; at6 (also)
U+537C.0	fMandarin	wu4
U+537C.0	fEnglish	 unsteady dangerous
U+537D.0	fUTF8	卽
U+537D.0	fCNS11643	3-2B4B
U+537D.0	fGB	E-237C
U+537D.0	fR/S	 26.7
U+537D.0	fTotalStrokes	 9
U+537D.0	fCangjie	HPSL
U+537D.0	fFourCorner	27720
U+537D.0	fHakka	 zit7
U+537D.0	fCantonese	zik1
U+537D.0	fMandarin	ji2
U+537D.0	fEnglish	[1] promptly; immediately; now [2] then; accordingly [3] even if-indicating supposition or sequence
U+537E.0	fUTF8	卾
U+537E.0	fCNS11643	4-3065
U+537E.0	fGB	E-237D
U+537E.0	fR/S	  26.9
U+537E.0	fTotalStrokes	 11
U+537E.0	fCangjie	RSSL
U+537E.0	fFourCorner	67220
U+537E.0	fHakka	 ngok8; ngong1
U+537E.0	fCantonese	ngok6; ok6 (also)
U+537E.0	fMandarin	e4
U+537F.0	fUTF8	卿
U+537F.0	fCNS11643	1-544E
U+537F.0	fBig5	ADEB
U+537F.0	fGB	0-4764
U+537F.0	fR/S	 26.9; 26.7
U+537F.0	fTotalStrokes	 11
U+537F.0	fCangjie	HHAIL
U+537F.0	fFourCorner	77720
U+537F.0	fHakka	  kin1
U+537F.0	fCantonese	hing1
U+537F.0	fMandarin	qing1
U+537F.0	fEnglish	[1] salutation of an emperor to his ministers [2] used in addressing one's wife; honey; darling; dear [3] nobleman; high officer; high official rank
U+5380.0	fUTF8	厀
U+5380.0	fCNS11643	3-4047
U+5380.0	fGB	E-237E
U+5380.0	fR/S	 26.11
U+5380.0	fTotalStrokes	 13
U+5380.0	fCangjie	DESL
U+5380.0	fFourCorner	47120
U+5380.0	fCantonese	sat1
U+5380.0	fMandarin	xi1
U+5381.0	fUTF8	厁
U+5381.0	fGB	1-7D48
U+5381.0	fR/S	 26.11
U+5381.0	fTotalStrokes	 13
U+5381.0	fCangjie	OJSL
U+5381.0	fCantonese	ce4
U+5382.0	fUTF8	厂
U+5382.0	fCNS11643	2-2125
U+5382.0	fBig5	C944
U+5382.0	fGB	0-3327
U+5382.0	fR/S	 27.0
U+5382.0	fTotalStrokes	 2
U+5382.0	fCangjie	MH
U+5382.0	fFourCorner	71200
U+5382.0	fHakka	 hon5
U+5382.0	fCantonese	hon3; hon2
U+5382.0	fMandarin	han3
U+5382.1	fUTF8	厂
U+5382.1	fCNS11643	2-2125
U+5382.1	fBig5	C944
U+5382.1	fGB	0-3327
U+5382.1	fR/S	 27.0
U+5382.1	fTotalStrokes	 2
U+5382.1	fCangjie	MH
U+5382.1	fFourCorner	71200
U+5382.1	fHakka	ngiam2
U+5382.1	fCantonese	am1; ngam1
U+5382.1	fMandarin	an1
U+5382.2	fUTF8	厂
U+5382.2	fCNS11643	2-2125
U+5382.2	fBig5	C944
U+5382.2	fGB	0-3327
U+5382.2	fR/S	 27.0
U+5382.2	fTotalStrokes	 2
U+5382.2	fCangjie	MH
U+5382.2	fFourCorner	71200
U+5382.2	fHakka	chong3
U+5382.2	fCantonese	cong2
U+5382.2	fMandarin	chang3
U+5382.2	fEnglish	simplified form of &#x5EE0;
U+5382.3	fUTF8	厂
U+5382.3	fCNS11643	2-2125
U+5382.3	fBig5	C944
U+5382.3	fGB	0-3327
U+5382.3	fR/S	 27.0
U+5382.3	fTotalStrokes	 2
U+5382.3	fCangjie	MH
U+5382.3	fFourCorner	71200
U+5382.3	fCantonese	ngon6 (also); on6 (also)
U+5382.4	fUTF8	厂
U+5382.4	fCNS11643	2-2125
U+5382.4	fBig5	C944
U+5382.4	fGB	0-3327
U+5382.4	fR/S	 27.0
U+5382.4	fTotalStrokes	 2
U+5382.4	fCangjie	MH
U+5382.4	fFourCorner	71200
U+5382.4	fCantonese	ho1
U+5383.0	fUTF8	厃
U+5383.0	fCNS11643	4-2144
U+5383.0	fGB	5-3429
U+5383.0	fR/S	  27.2
U+5383.0	fTotalStrokes	 4
U+5383.0	fCangjie	NM
U+5383.0	fFourCorner	27200
U+5383.0	fCantonese	ngai5; ai5 (also)
U+5383.0	fMandarin	zhan1; wei3; yan2
U+5383.0	fEnglish	 look upward
U+5384.0	fUTF8	厄
U+5384.0	fCNS11643	1-452D
U+5384.0	fBig5	A4CC
U+5384.0	fGB	0-3672
U+5384.0	fR/S	 27.2
U+5384.0	fTotalStrokes	 4
U+5384.0	fCangjie	MSU
U+5384.0	fFourCorner	71212
U+5384.0	fHakka	  ak7
U+5384.0	fCantonese	ak1; aak1 (also); ngak1 (also); ngaak1 (also)
U+5384.0	fMandarin	e4; e3
U+5384.0	fEnglish	[1] adversity; difficulty; distress; hardship [2] impeded; cramped
U+5385.0	fUTF8	厅
U+5385.0	fGB	0-4C7C
U+5385.0	fR/S	 27.2
U+5385.0	fTotalStrokes	 4
U+5385.0	fCangjie	MMN
U+5385.0	fHakka	 ten1; tang1; tin1
U+5385.0	fCantonese	teng1; ting1 (also)
U+5385.0	fMandarin	ting1
U+5385.0	fEnglish	simplified form of &#x5EF3;
U+5386.0	fUTF8	历
U+5386.0	fCNS11643	3-215B
U+5386.0	fGB	0-407A
U+5386.0	fR/S	 27.2
U+5386.0	fTotalStrokes	 4
U+5386.0	fCangjie	MKS
U+5386.0	fFourCorner	71220
U+5386.0	fHakka	 let8; lit8; lak8
U+5386.0	fCantonese	lik6
U+5386.0	fMandarin	li4
U+5386.0	fEnglish	simplified form of &#x6B77;
U+5387.0	fUTF8	厇
U+5387.0	fCNS11643	4-2167
U+5387.0	fGB	5-304F
U+5387.0	fR/S	 27.3
U+5387.0	fTotalStrokes	 5
U+5387.0	fCangjie	MHP
U+5387.0	fFourCorner	71210
U+5387.0	fCantonese	zaak6
U+5387.0	fMandarin	zhe2
U+5388.0	fUTF8	厈
U+5388.0	fCNS11643	4-2166
U+5388.0	fGB	E-2421
U+5388.0	fR/S	 27.3
U+5388.0	fTotalStrokes	 5
U+5388.0	fCangjie	MMJ
U+5388.0	fFourCorner	71240
U+5388.0	fCantonese	hon3
U+5388.0	fMandarin	han3; an1; chang3; an4
U+5389.0	fUTF8	厉
U+5389.0	fCNS11643	3-2229
U+5389.0	fGB	0-4077
U+5389.0	fR/S	 27.3
U+5389.0	fTotalStrokes	 5
U+5389.0	fCangjie	MMS
U+5389.0	fFourCorner	71220
U+5389.0	fHakka	 li5
U+5389.0	fCantonese	lai6; lai3
U+5389.0	fMandarin	li4; lai4; lie4
U+5389.0	fEnglish	simplified form of &#x53B2;
U+538A.0	fUTF8	厊
U+538A.0	fCNS11643	2-2224
U+538A.0	fBig5	C9C4
U+538A.0	fGB	5-3050
U+538A.0	fR/S	 27.4
U+538A.0	fTotalStrokes	 6
U+538A.0	fCangjie	MMVH
U+538A.0	fFourCorner	71241
U+538A.0	fHakka	  nga3; nga1
U+538A.0	fCantonese	ngaa5; aa5 (also); aa1; aa2
U+538A.0	fMandarin	ya3; ya4
U+538A.0	fEnglish	not fitting one into another
U+538B.0	fUTF8	压
U+538B.0	fCNS11643	3-232F
U+538B.0	fGB	0-5139
U+538B.0	fR/S	 27.4
U+538B.0	fTotalStrokes	 6
U+538B.0	fCangjie	MGI
U+538B.0	fFourCorner	71210
U+538B.0	fHakka	 ap7
U+538B.0	fCantonese	aat3; ngaat3 (also)
U+538B.0	fMandarin	ya1; ya4; yan4
U+538B.0	fEnglish	simplified form of &#x58D3;
U+538C.0	fUTF8	厌
U+538C.0	fGB	0-5161
U+538C.0	fR/S	 27.4
U+538C.0	fTotalStrokes	 6
U+538C.0	fCangjie	MIK
U+538C.0	fHakka	 jam5
U+538C.0	fCantonese	jim3; aat3; ngaat3 (also); jim1; jim2
U+538C.0	fMandarin	yan4; yan1; ya1; yan3; ye4
U+538C.0	fEnglish	simplified form of &#x53AD;
U+538D.0	fUTF8	厍
U+538D.0	fGB	0-5847
U+538D.0	fR/S	 27.4
U+538D.0	fTotalStrokes	 6
U+538D.0	fCangjie	XMKQ
U+538D.0	fHakka	 sha5
U+538D.0	fCantonese	se3
U+538D.0	fMandarin	she4
U+538D.0	fEnglish	simplified form of &#x5399;
U+538E.0	fUTF8	厎
U+538E.0	fCNS11643	2-2336
U+538E.0	fBig5	CA74
U+538E.0	fGB	3-3035
U+538E.0	fR/S	 27.5
U+538E.0	fTotalStrokes	 7
U+538E.0	fCangjie	MHPM
U+538E.0	fFourCorner	71242
U+538E.0	fHakka	 di1; zih3; zhi3; zhi1
U+538E.0	fCantonese	zi2; dai2
U+538E.0	fMandarin	zhi3; di1; zhi4
U+538E.0	fEnglish	settle
U+538F.0	fUTF8	厏
U+538F.0	fCNS11643	2-2337
U+538F.0	fBig5	CA75
U+538F.0	fGB	5-3055
U+538F.0	fR/S	 27.5
U+538F.0	fTotalStrokes	 7
U+538F.0	fCangjie	MOS
U+538F.0	fFourCorner	71211
U+538F.0	fHakka	  za5
U+538F.0	fCantonese	zaa3
U+538F.0	fMandarin	zha3; cha3; zhe2
U+538F.0	fEnglish	disagreeing; not fitting
U+5390.0	fUTF8	厐
U+5390.0	fGB	8-2E21
U+5390.0	fR/S	 27.5; 212.2
U+5390.0	fTotalStrokes	 7
U+5390.0	fCangjie	MIKP
U+5390.0	fCantonese	pong4
U+5390.0	fMandarin	pang2
U+5390.0	fEnglish	disorderly, messy; huge, big
U+5391.0	fUTF8	厑
U+5391.0	fGB	1-7D7C
U+5391.0	fR/S	 27.5
U+5391.0	fTotalStrokes	 7
U+5391.0	fCangjie	MRP
U+5391.0	fCantonese	ngaai4; aai4 (also)
U+5392.0	fUTF8	厒
U+5392.0	fCNS11643	2-2559
U+5392.0	fBig5	CBD9
U+5392.0	fGB	E-2422
U+5392.0	fR/S	 27.6
U+5392.0	fTotalStrokes	 8
U+5392.0	fCangjie	MOJU
U+5392.0	fFourCorner	71274
U+5392.0	fCantonese	hip3
U+5392.0	fMandarin	he2; ke4; que4
U+5393.0	fUTF8	厓
U+5393.0	fCNS11643	3-2767
U+5393.0	fGB	5-3057
U+5393.0	fR/S	  27.6
U+5393.0	fTotalStrokes	 8
U+5393.0	fCangjie	MGG
U+5393.0	fFourCorner	71210
U+5393.0	fHakka	  ngai2; ngai3
U+5393.0	fCantonese	ngaai4; aai4 (also); ngai4; ai4
U+5393.0	fMandarin	ai2; ya2; yai2
U+5393.0	fEnglish	[1] precipice; shore; bank [2] a Chinese family name [3] I see &#x2028E;
U+5394.0	fUTF8	厔
U+5394.0	fCNS11643	2-255A
U+5394.0	fBig5	CBDA
U+5394.0	fGB	3-3036
U+5394.0	fR/S	 27.6
U+5394.0	fTotalStrokes	 8
U+5394.0	fCangjie	MMIG
U+5394.0	fFourCorner	71214
U+5394.0	fHakka	 zhit7; tsiit7; zi5
U+5394.0	fCantonese	zat6; zat1
U+5394.0	fMandarin	zhi4
U+5394.0	fEnglish	 stop; hinder place near water
U+5395.0	fUTF8	厕
U+5395.0	fGB	0-325E
U+5395.0	fR/S	 27.6
U+5395.0	fTotalStrokes	 8
U+5395.0	fCangjie	MBON
U+5395.0	fHakka	 ca5; cih5; sa3
U+5395.0	fCantonese	ci3; cak1
U+5395.0	fMandarin	ce4; ci4
U+5395.0	fEnglish	simplified form of &#x53A0;
U+5396.0	fUTF8	厖
U+5396.0	fCNS11643	2-2939
U+5396.0	fBig5	CDF7
U+5396.0	fGB	3-3038
U+5396.0	fR/S	 27.7
U+5396.0	fTotalStrokes	 9
U+5396.0	fCangjie	MIUH
U+5396.0	fFourCorner	71212
U+5396.0	fHakka	 mang2
U+5396.0	fCantonese	pong4; mong4
U+5396.0	fMandarin	pang2; mang2; mang3
U+5396.0	fEnglish	[1] bulky; huge; thick and large [2] confused; disorderly
U+5397.0	fUTF8	厗
U+5397.0	fCNS11643	2-2938
U+5397.0	fBig5	CDF6
U+5397.0	fGB	3-303A
U+5397.0	fR/S	 27.7
U+5397.0	fTotalStrokes	 9
U+5397.0	fCangjie	MYTJ
U+5397.0	fFourCorner	71241
U+5397.0	fCantonese	tai4
U+5397.0	fMandarin	ti2
U+5398.0	fUTF8	厘
U+5398.0	fCNS11643	2-293B
U+5398.0	fBig5	CDF9
U+5398.0	fGB	0-4065
U+5398.0	fR/S	 27.7
U+5398.0	fTotalStrokes	 9
U+5398.0	fCangjie	MWG
U+5398.0	fFourCorner	71214
U+5398.0	fHakka	  li2; chan2; can2
U+5398.0	fCantonese	lei4; lei2 (also); cin4
U+5398.0	fMandarin	li2; chan2
U+5398.0	fEnglish	thousandth part of tael
U+5399.0	fUTF8	厙
U+5399.0	fCNS11643	2-293A
U+5399.0	fBig5	CDF8
U+5399.0	fGB	1-5847
U+5399.0	fR/S	 27.7
U+5399.0	fTotalStrokes	 9
U+5399.0	fCangjie	MJWJ
U+5399.0	fFourCorner	71250
U+5399.0	fHakka	 sha5; sa5
U+5399.0	fCantonese	se3
U+5399.0	fMandarin	she4
U+5399.0	fEnglish	a Chinese family name
U+539A.0	fUTF8	厚
U+539A.0	fCNS11643	1-5034
U+539A.0	fBig5	AB70
U+539A.0	fGB	0-3A71
U+539A.0	fR/S	 27.7
U+539A.0	fTotalStrokes	 9
U+539A.0	fCangjie	MAND
U+539A.0	fFourCorner	71247
U+539A.0	fHakka	  heu5; heu3; pun1; hiu3; heu6
U+539A.0	fCantonese	hau5
U+539A.0	fMandarin	hou4
U+539A.0	fEnglish	[1] thick; thickness [2] deep friendship [3] treat kindly; generous [4] substantial; greatly [5] kind; considerate; virtuous
U+539B.0	fUTF8	厛
U+539B.0	fCNS11643	3-6452
U+539B.0	fGB	E-2423
U+539B.0	fR/S	 27.7
U+539B.0	fTotalStrokes	 9
U+539B.0	fCangjie	MRHL
U+539B.0	fCantonese	teng1
U+539B.0	fMandarin	ting1
U+539C.0	fUTF8	厜
U+539C.0	fCNS11643	2-3373
U+539C.0	fBig5	D470
U+539C.0	fGB	3-303E
U+539C.0	fR/S	 27.8
U+539C.0	fTotalStrokes	 10
U+539C.0	fCangjie	MHJM
U+539C.0	fFourCorner	71214
U+539C.0	fHakka	  zui1; shui2; sui2
U+539C.0	fCantonese	seoi4; ceoi1
U+539C.0	fMandarin	zui1; chui2
U+539C.0	fEnglish	 mountain peak
U+539D.0	fUTF8	厝
U+539D.0	fCNS11643	1-5450
U+539D.0	fBig5	ADED
U+539D.0	fGB	0-5848
U+539D.0	fR/S	 27.8
U+539D.0	fTotalStrokes	 10
U+539D.0	fCangjie	MTA
U+539D.0	fFourCorner	71261
U+539D.0	fHakka	 co5;  cih5; cu5
U+539D.0	fCantonese	cou3
U+539D.0	fMandarin	cuo4
U+539D.0	fEnglish	 cut or engrave grave or gravest
U+539E.0	fUTF8	厞
U+539E.0	fCNS11643	2-2E32
U+539E.0	fBig5	D0EF
U+539E.0	fGB	5-305D
U+539E.0	fR/S	 27.8
U+539E.0	fTotalStrokes	 10
U+539E.0	fCangjie	MLMY
U+539E.0	fFourCorner	71211
U+539E.0	fHakka	 fui1p'ui5; fi1p'i5; pui5; pi5; fui1; fui5
U+539E.0	fCantonese	fei6
U+539E.0	fMandarin	fei4; fei1
U+539E.0	fEnglish	hidden; low; base
U+539F.0	fUTF8	原
U+539F.0	fCNS11643	1-544F
U+539F.0	fBig5	ADEC
U+539F.0	fGB	0-542D
U+539F.0	fR/S	 27.8
U+539F.0	fTotalStrokes	 10
U+539F.0	fCangjie	MHAF
U+539F.0	fFourCorner	71296
U+539F.0	fHakka	  ngien2; nien2; gnian3; ngen2; ngian2
U+539F.0	fCantonese	jyun4; jyun6; jyun5
U+539F.0	fMandarin	yuan2; yuan4
U+539F.0	fEnglish	[1] source; origin; beginning [2] original; primary [3] steppe; vast plain; field [4] graveyard [5] excuse; pardon [6] a Chinese family name
U+53A0.0	fUTF8	厠
U+53A0.0	fCNS11643	3-3454
U+53A0.0	fGB	1-325E
U+53A0.0	fR/S	  27.9
U+53A0.0	fTotalStrokes	 11
U+53A0.0	fCangjie	MBCN
U+53A0.0	fFourCorner	71220
U+53A0.0	fHakka	 ca5;  ci5;  cih5;  sa3; zet7; ze5
U+53A0.0	fCantonese	ci3; cak1
U+53A0.0	fMandarin	ci4; ce4
U+53A0.0	fEnglish	 mingle with toilet; lavatory
U+53A1.0	fUTF8	厡
U+53A1.0	fCNS11643	4-3068
U+53A1.0	fGB	E-2424
U+53A1.0	fR/S	  27.9
U+53A1.0	fTotalStrokes	 11
U+53A1.0	fCangjie	MHAE
U+53A1.0	fFourCorner	71230
U+53A1.0	fHakka	 ngian2
U+53A1.0	fCantonese	jyun4
U+53A1.0	fMandarin	yuan2
U+53A2.0	fUTF8	厢
U+53A2.0	fGB	0-4F61
U+53A2.0	fR/S	 27.9
U+53A2.0	fTotalStrokes	 11
U+53A2.0	fCangjie	MDBU
U+53A2.0	fHakka	 siong1
U+53A2.0	fCantonese	soeng1
U+53A2.0	fMandarin	xiang1
U+53A2.0	fEnglish	 side-room; wing theatre box
U+53A3.0	fUTF8	厣
U+53A3.0	fGB	0-5849
U+53A3.0	fR/S	 27.9
U+53A3.0	fTotalStrokes	 11
U+53A3.0	fCangjie	MIKL
U+53A3.0	fHakka	 jam3
U+53A3.0	fCantonese	jim2; jim3
U+53A3.0	fMandarin	yan3
U+53A3.0	fEnglish	simplified form of &#x53B4;
U+53A4.0	fUTF8	厤
U+53A4.0	fCNS11643	2-3A48
U+53A4.0	fBig5	D864
U+53A4.0	fGB	E-2425
U+53A4.0	fR/S	 27.10
U+53A4.0	fTotalStrokes	 12
U+53A4.0	fCangjie	MHDD
U+53A4.0	fFourCorner	71294
U+53A4.0	fCantonese	lik6
U+53A4.0	fMandarin	li4
U+53A4.0	fEnglish	 calculate the calendar
U+53A5.0	fUTF8	厥
U+53A5.0	fCNS11643	1-5E3C
U+53A5.0	fBig5	B3D6
U+53A5.0	fGB	0-584A
U+53A5.0	fR/S	 27.10
U+53A5.0	fTotalStrokes	 12
U+53A5.0	fCangjie	MTUO
U+53A5.0	fFourCorner	71282
U+53A5.0	fHakka	 kiet7; ket7; ket8
U+53A5.0	fCantonese	kyut3
U+53A5.0	fMandarin	jue2
U+53A5.0	fEnglish	[1] personal and possessive pronoun [2] dig [3] faint
U+53A6.0	fUTF8	厦
U+53A6.0	fCNS11643	3-3A28
U+53A6.0	fGB	0-4F43
U+53A6.0	fR/S	  27.10
U+53A6.0	fTotalStrokes	 12
U+53A6.0	fCangjie	MMUE
U+53A6.0	fFourCorner	71240
U+53A6.0	fHakka	  ha5; ha6; ha3
U+53A6.0	fCantonese	haa6
U+53A6.0	fMandarin	sha4; xia4
U+53A6.0	fEnglish	big building; mansion
U+53A7.0	fUTF8	厧
U+53A7.0	fCNS11643	2-3A49
U+53A7.0	fBig5	D865
U+53A7.0	fGB	5-3067
U+53A7.0	fR/S	 27.10
U+53A7.0	fTotalStrokes	 12
U+53A7.0	fCangjie	MJBC
U+53A7.0	fFourCorner	71281
U+53A7.0	fCantonese	din1
U+53A7.0	fMandarin	dian1
U+53A8.0	fUTF8	厨
U+53A8.0	fCNS11643	3-3A27
U+53A8.0	fGB	0-3378
U+53A8.0	fR/S	 27.10
U+53A8.0	fTotalStrokes	 12
U+53A8.0	fCangjie	MMTI
U+53A8.0	fFourCorner	71240
U+53A8.0	fHakka	 cu2;  chu2
U+53A8.0	fCantonese	cyu4; ceoi4 (also)
U+53A8.0	fMandarin	chu2
U+53A8.0	fEnglish	 kitchen closet cupboard
U+53A9.0	fUTF8	厩
U+53A9.0	fCNS11643	3-404A
U+53A9.0	fGB	0-3E47
U+53A9.0	fR/S	 27.9
U+53A9.0	fTotalStrokes	 11; 13
U+53A9.0	fCangjie	MAIU
U+53A9.0	fFourCorner	71210
U+53A9.0	fCantonese	gau3
U+53A9.0	fMandarin	jiu4
U+53A9.0	fEnglish	 stable barnyard
U+53AA.0	fUTF8	厪
U+53AA.0	fCNS11643	3-4049
U+53AA.0	fGB	5-3069
U+53AA.0	fR/S	 27.11
U+53AA.0	fTotalStrokes	 13
U+53AA.0	fCangjie	MTLM
U+53AA.0	fFourCorner	71210
U+53AA.0	fHakka	 giun3
U+53AA.0	fCantonese	kan4; gan2
U+53AA.0	fMandarin	jin4; jin3; qin2
U+53AA.0	fEnglish	 hut careful
U+53AB.0	fUTF8	厫
U+53AB.0	fCNS11643	3-4048
U+53AB.0	fGB	5-3066
U+53AB.0	fR/S	  27.11
U+53AB.0	fTotalStrokes	 13
U+53AB.0	fCangjie	MGSK
U+53AB.0	fFourCorner	71240
U+53AB.0	fCantonese	ngou4; ou4 (also)
U+53AB.0	fMandarin	ao2
U+53AB.0	fEnglish	granary
U+53AC.0	fUTF8	厬
U+53AC.0	fCNS11643	2-492D
U+53AC.0	fBig5	E146
U+53AC.0	fGB	3-3040
U+53AC.0	fR/S	 27.12
U+53AC.0	fTotalStrokes	 14
U+53AC.0	fCangjie	MAHR
U+53AC.0	fFourCorner	71263
U+53AC.0	fHakka	 gui3; gwi3; kwui3
U+53AC.0	fCantonese	gwai2
U+53AC.0	fMandarin	gui3
U+53AC.0	fEnglish	dried up; exhausted
U+53AD.0	fUTF8	厭
U+53AD.0	fCNS11643	1-6825
U+53AD.0	fBig5	B9BD
U+53AD.0	fGB	1-5161
U+53AD.0	fR/S	 27.12
U+53AD.0	fTotalStrokes	 14
U+53AD.0	fCangjie	MABK
U+53AD.0	fFourCorner	71234
U+53AD.0	fHakka	  jam5; ram5; ram1; jam3; jam1
U+53AD.0	fCantonese	jim3; jim1; aat3; ngaat3 (also); jim2
U+53AD.0	fMandarin	yan4; yan1; ya1; yan3; ye4
U+53AD.0	fEnglish	[1] dislike; detest; hate; reject [2] get tired of [3] satiated; surfeited
U+53AE.0	fUTF8	厮
U+53AE.0	fCNS11643	3-4625
U+53AE.0	fGB	0-584B
U+53AE.0	fR/S	  27.12
U+53AE.0	fTotalStrokes	 14
U+53AE.0	fCangjie	MTCL
U+53AE.0	fFourCorner	71220
U+53AE.0	fHakka	 sii1; se1
U+53AE.0	fCantonese	si1
U+53AE.0	fMandarin	si1
U+53AE.0	fEnglish	 servant make a disturbance
U+53AF.0	fUTF8	厯
U+53AF.0	fCNS11643	4-3C40
U+53AF.0	fGB	E-2426
U+53AF.0	fR/S	 27.12; 61.10; 27.11
U+53AF.0	fTotalStrokes	 13; 14
U+53AF.0	fCangjie	MDDI
U+53AF.0	fFourCorner	71230
U+53AF.0	fCantonese	lik6
U+53AF.0	fMandarin	li4
U+53B0.0	fUTF8	厰
U+53B0.0	fCNS11643	3-4626
U+53B0.0	fGB	E-2427
U+53B0.0	fR/S	 27.12
U+53B0.0	fTotalStrokes	 14
U+53B0.0	fCangjie	MFBK
U+53B0.0	fFourCorner	71240
U+53B0.0	fHakka	  cong3; chong3; chong5
U+53B0.0	fCantonese	cong2
U+53B0.0	fMandarin	chang3; an1
U+53B0.0	fEnglish	factory; plant; workshop; mill
U+53B1.0	fUTF8	厱
U+53B1.0	fCNS11643	3-4B35
U+53B1.0	fGB	3-3039
U+53B1.0	fR/S	  27.13
U+53B1.0	fTotalStrokes	 15
U+53B1.0	fCangjie	MOMO
U+53B1.0	fFourCorner	71280
U+53B1.0	fCantonese	laam4; cim1
U+53B1.0	fMandarin	lan2; qian1
U+53B2.0	fUTF8	厲
U+53B2.0	fCNS11643	1-6C30
U+53B2.0	fBig5	BC46
U+53B2.0	fGB	1-4077
U+53B2.0	fR/S	 27.13
U+53B2.0	fTotalStrokes	 15
U+53B2.0	fCangjie	MTWB
U+53B2.0	fFourCorner	71227
U+53B2.0	fHakka	  li5; li6; li3
U+53B2.0	fCantonese	lai6; laai3; laai6; lai3
U+53B2.0	fMandarin	li4; lai4; lie4
U+53B2.0	fEnglish	[1] coarse whetstone [2] harsh; violent; severe; stern; serious [3] persuade; urge; encourage [4] bad; evil [5] epidemic [6] oppress; oppressive; cruel [7] a Chinese family name
U+53B3.0	fUTF8	厳
U+53B3.0	fCNS11643	3-5445
U+53B3.0	fGB	E-2428
U+53B3.0	fR/S	 27.15
U+53B3.0	fTotalStrokes	 17
U+53B3.0	fCangjie	FMNJK
U+53B3.0	fFourCorner	23248
U+53B3.0	fCantonese	jim4
U+53B3.0	fMandarin	yan2
U+53B3.0	fEnglish	 strict; rigorous; rigid stern
U+53B4.0	fUTF8	厴
U+53B4.0	fCNS11643	2-653B
U+53B4.0	fBig5	F1EF
U+53B4.0	fGB	1-5849
U+53B4.0	fR/S	 27.17
U+53B4.0	fTotalStrokes	 19
U+53B4.0	fCangjie	MKWL
U+53B4.0	fFourCorner	71506
U+53B4.0	fHakka	 jam3; ram3
U+53B4.0	fCantonese	jim2; jim3
U+53B4.0	fMandarin	yan3
U+53B4.0	fEnglish	shell
U+53B5.0	fUTF8	厵
U+53B5.0	fCNS11643	F-6D2E
U+53B5.0	fGB	5-3075
U+53B5.0	fR/S	 27.28
U+53B5.0	fTotalStrokes	 30
U+53B5.0	fCangjie	MFMFF
U+53B5.0	fFourCorner	7129
U+53B5.0	fHakka	 ngien2
U+53B5.0	fCantonese	jyun4
U+53B5.0	fMandarin	yuan2
U+53B5.0	fEnglish	 spring source
U+53B6.0	fUTF8	厶
U+53B6.0	fCNS11643	3-212B
U+53B6.0	fGB	0-5B4C
U+53B6.0	fR/S	 28.0
U+53B6.0	fTotalStrokes	 2
U+53B6.0	fCangjie	VI
U+53B6.0	fFourCorner	20730
U+53B6.0	fHakka	 sii1; meu1; su1; se1
U+53B6.0	fCantonese	si1; mau5
U+53B6.0	fMandarin	si1; mou3
U+53B6.0	fEnglish	 private; secret Kangxi radical number 28
U+53B7.0	fUTF8	厷
U+53B7.0	fCNS11643	3-215C
U+53B7.0	fGB	5-3529
U+53B7.0	fR/S	  28.2
U+53B7.0	fTotalStrokes	 4
U+53B7.0	fCangjie	KI
U+53B7.0	fFourCorner	40030
U+53B7.0	fHakka	 kwen1
U+53B7.0	fCantonese	gwang1
U+53B7.0	fMandarin	gong1; hong2
U+53B7.0	fEnglish	 the fore-arm (most Chinese-English dictionaries give the upper-arm) round
U+53B8.0	fUTF8	厸
U+53B8.0	fCNS11643	4-2145
U+53B8.0	fGB	E-2429
U+53B8.0	fR/S	  28.2
U+53B8.0	fTotalStrokes	 4
U+53B8.0	fCangjie	II
U+53B8.0	fFourCorner	22730
U+53B8.0	fCantonese	leon4
U+53B8.0	fMandarin	lin2; mian3
U+53B9.0	fUTF8	厹
U+53B9.0	fCNS11643	2-2138
U+53B9.0	fBig5	C958
U+53B9.0	fGB	3-3350
U+53B9.0	fR/S	 28.2
U+53B9.0	fTotalStrokes	 4
U+53B9.0	fCangjie	KNI
U+53B9.0	fFourCorner	47731
U+53B9.0	fHakka	  kiu2
U+53B9.0	fCantonese	kau4; jau4; kau2; nau2
U+53B9.0	fMandarin	qiu2; rou2
U+53B9.0	fEnglish	 spear tribe
U+53BA.0	fUTF8	厺
U+53BA.0	fCNS11643	3-222A
U+53BA.0	fGB	E-242A
U+53BA.0	fR/S	 28.3
U+53BA.0	fTotalStrokes	 5
U+53BA.0	fCangjie	KI
U+53BA.0	fFourCorner	40730
U+53BA.0	fCantonese	heoi3
U+53BA.0	fMandarin	qu4
U+53BA.0	fEnglish	 go depart
U+53BB.0	fUTF8	去
U+53BB.0	fCNS11643	1-462A
U+53BB.0	fBig5	A568
U+53BB.0	fGB	0-4825
U+53BB.0	fR/S	 28.3
U+53BB.0	fTotalStrokes	 5
U+53BB.0	fCangjie	GI
U+53BB.0	fFourCorner	40731
U+53BB.0	fHakka	  hi5;  ki5; ki3
U+53BB.0	fCantonese	heoi3; heoi2
U+53BB.0	fMandarin	qu4
U+53BB.0	fEnglish	[1] go away; leave; depart [2] get rid of; remove [3] be ... apart [4] past; gone [5] an auxiliary verb [6] the fourth of the four tones in Chinese phonetics [7] play the part of (Peking opera)
U+53BC.0	fUTF8	厼
U+53BC.0	fGB	1-7D29
U+53BC.0	fR/S	 28.3; 42.2
U+53BC.0	fTotalStrokes	 5
U+53BC.0	fCangjie	IF
U+53BC.0	fCantonese	ji5
U+53BC.0	fEnglish	 kwukyel
U+53BD.0	fUTF8	厽
U+53BD.0	fCNS11643	4-2248
U+53BD.0	fGB	5-352B
U+53BD.0	fR/S	  28.4
U+53BD.0	fTotalStrokes	 6
U+53BD.0	fCangjie	III
U+53BD.0	fFourCorner	23730
U+53BD.0	fCantonese	leoi5
U+53BD.0	fMandarin	lei3
U+53BE.0	fUTF8	厾
U+53BE.0	fCNS11643	3-2331
U+53BE.0	fGB	3-3030
U+53BE.0	fR/S	 28.4; 5.5
U+53BE.0	fTotalStrokes	 6
U+53BE.0	fCangjie	NGI
U+53BE.0	fFourCorner	23730
U+53BE.0	fCantonese	duk1; tuk1
U+53BE.0	fMandarin	diu1; du1; du3
U+53BE.0	fEnglish	 lightly tap through away drop sentence final particle
U+53BF.0	fUTF8	县
U+53BF.0	fGB	0-4F58
U+53BF.0	fR/S	 28.5
U+53BF.0	fTotalStrokes	 7
U+53BF.0	fCangjie	BMI
U+53BF.0	fHakka	 hien5; jen5
U+53BF.0	fCantonese	jyun6; jyun2 (also); jyun4
U+53BF.0	fMandarin	xian4; xuan2
U+53BF.0	fEnglish	simplified form of &#x7E23;
U+53C0.0	fUTF8	叀
U+53C0.0	fCNS11643	4-2559
U+53C0.0	fGB	E-242B
U+53C0.0	fR/S	 28.6
U+53C0.0	fTotalStrokes	 8; 7
U+53C0.0	fCangjie	JWI
U+53C0.0	fFourCorner	50730
U+53C0.0	fCantonese	zyun1
U+53C0.0	fMandarin	zhuan1; hui4
U+53C1.0	fUTF8	叁
U+53C1.0	fCNS11643	3-2769
U+53C1.0	fGB	0-487E
U+53C1.0	fR/S	  28.6
U+53C1.0	fTotalStrokes	 8
U+53C1.0	fCangjie	IKMMM
U+53C1.0	fFourCorner	23100
U+53C1.0	fHakka	 sam1
U+53C1.0	fCantonese	saam1; saam3; sam1
U+53C1.0	fMandarin	san1; can1
U+53C1.0	fEnglish	bank form of numeral "three"
U+53C2.0	fUTF8	参
U+53C2.0	fCNS11643	3-276A
U+53C2.0	fGB	0-324E
U+53C2.0	fR/S	 28.6
U+53C2.0	fTotalStrokes	 8
U+53C2.0	fCangjie	IKHHH
U+53C2.0	fFourCorner	23200
U+53C2.0	fHakka	 cam1; sem1
U+53C2.0	fCantonese	caam1; saam1; sam1; saam3; cam1
U+53C2.0	fMandarin	can1; san1; den1; cen1; san3
U+53C2.0	fEnglish	simplified form of &#x53C3;
U+53C2.1	fUTF8	参
U+53C2.1	fCNS11643	3-276A
U+53C2.1	fGB	0-324E
U+53C2.1	fR/S	 28.6
U+53C2.1	fTotalStrokes	 8
U+53C2.1	fCangjie	IKHHH
U+53C2.1	fFourCorner	23200
U+53C2.1	fMandarin	shen1
U+53C3.0	fUTF8	參
U+53C3.0	fCNS11643	1-5936
U+53C3.0	fBig5	B0D1
U+53C3.0	fGB	1-324E
U+53C3.0	fR/S	 28.9
U+53C3.0	fTotalStrokes	 11
U+53C3.0	fCangjie	IIIH
U+53C3.0	fFourCorner	23202
U+53C3.0	fHakka	  cam1; sem3; sem5
U+53C3.0	fCantonese	caam1; saam1; sam1; cam1; saam3
U+53C3.0	fMandarin	can1; den1
U+53C3.0	fEnglish	[1] take part in; get involved in [2] visit; interview; call on [3] impeach; censure [4] recommend [5] counsel; consult together [6] consider; collate; compare
U+53C3.1	fUTF8	參
U+53C3.1	fCNS11643	1-5936
U+53C3.1	fBig5	B0D1
U+53C3.1	fGB	1-324E
U+53C3.1	fR/S	 28.9
U+53C3.1	fTotalStrokes	 11
U+53C3.1	fCangjie	IIIH
U+53C3.1	fFourCorner	23202
U+53C3.1	fHakka	 sem1
U+53C3.1	fMandarin	shen1
U+53C3.1	fEnglish	[1] name of a star [2] ginseng
U+53C4.0	fUTF8	叄
U+53C4.0	fCNS11643	3-3455
U+53C4.0	fGB	E-242C
U+53C4.0	fR/S	 28.9; 28.10
U+53C4.0	fTotalStrokes	 11
U+53C4.0	fCangjie	IIIM
U+53C4.0	fFourCorner	23100
U+53C4.0	fCantonese	caam1; sam1
U+53C4.0	fMandarin	can1; cen1; can4; shen1; san1; san3
U+53C4.0	fEnglish	take part in; intervene
U+53C5.0	fUTF8	叅
U+53C5.0	fCNS11643	3-3A29
U+53C5.0	fGB	E-242D
U+53C5.0	fR/S	  28.10
U+53C5.0	fTotalStrokes	 12
U+53C5.0	fCangjie	IIIP
U+53C5.0	fFourCorner	23330
U+53C5.0	fHakka	  cam1
U+53C5.0	fCantonese	caam1
U+53C5.0	fMandarin	can1; cen1; can4
U+53C5.0	fEnglish	 counel; consult together take part in intervene
U+53C6.0	fUTF8	叆
U+53C6.0	fGB	8-7A45
U+53C6.0	fR/S	 28.12
U+53C6.0	fTotalStrokes	 14
U+53C6.0	fCangjie	XMIBB
U+53C6.0	fCantonese	oi2; ngoi2 (also)
U+53C6.0	fMandarin	ai4
U+53C6.0	fEnglish	 cloudy sky dark; obscure
U+53C7.0	fUTF8	叇
U+53C7.0	fGB	8-7A46
U+53C7.0	fR/S	 28.13
U+53C7.0	fTotalStrokes	 15
U+53C7.0	fCangjie	XMIYL
U+53C7.0	fCantonese	doi6
U+53C7.0	fMandarin	dai4
U+53C8.0	fUTF8	又
U+53C8.0	fCNS11643	1-4434
U+53C8.0	fBig5	A453
U+53C8.0	fGB	0-5356
U+53C8.0	fR/S	 29.0
U+53C8.0	fTotalStrokes	 2
U+53C8.0	fCangjie	NK
U+53C8.0	fFourCorner	17400
U+53C8.0	fHakka	 ju5; jiu3; ru6; jeu3; jiu6; ju3; ru3
U+53C8.0	fCantonese	jau6
U+53C8.0	fMandarin	you4
U+53C8.0	fEnglish	[1] and; also; again; in addition to [2] moreover; furthermore [3] and (used in a mixed fraction such as one and three fourths)
U+53C9.0	fUTF8	叉
U+53C9.0	fCNS11643	1-4446
U+53C9.0	fBig5	A465
U+53C9.0	fGB	0-3266
U+53C9.0	fR/S	 29.1
U+53C9.0	fTotalStrokes	 3
U+53C9.0	fCangjie	EI
U+53C9.0	fFourCorner	17400
U+53C9.0	fHakka	  ca1; za5
U+53C9.0	fCantonese	caa1; caa5
U+53C9.0	fMandarin	cha1; cha2; chai1; cha4; cha5
U+53C9.0	fEnglish	[1] interlace fingers; cross arms [2] thrust; pierce; stab [3] fork; prong [4] push another's neck with one's hand
U+53C9.1	fUTF8	叉
U+53C9.1	fCNS11643	1-4446
U+53C9.1	fBig5	A465
U+53C9.1	fGB	0-3266
U+53C9.1	fR/S	 29.1
U+53C9.1	fTotalStrokes	 3
U+53C9.1	fCangjie	EI
U+53C9.1	fFourCorner	17400
U+53C9.1	fMandarin	cha3
U+53C9.1	fEnglish	divergent (said of the road)
U+53CA.0	fUTF8	及
U+53CA.0	fCNS11643	1-452F
U+53CA.0	fBig5	A4CE
U+53CA.0	fGB	0-3C30
U+53CA.0	fR/S	 29.2
U+53CA.0	fTotalStrokes	 4
U+53CA.0	fCangjie	NHE
U+53CA.0	fFourCorner	17247
U+53CA.0	fHakka	  kip8
U+53CA.0	fCantonese	kap6; gap6 (also)
U+53CA.0	fMandarin	ji2
U+53CA.0	fEnglish	[1] reach; come up to; attain [2] and, as well as; with [3] just at the moment; timely; when [4] as long as; up to; until [5] continue; proceed [6] extend
U+53CB.0	fUTF8	友
U+53CB.0	fCNS11643	1-452E
U+53CB.0	fBig5	A4CD
U+53CB.0	fGB	0-5351
U+53CB.0	fR/S	 29.2
U+53CB.0	fTotalStrokes	 4
U+53CB.0	fCangjie	KE
U+53CB.0	fFourCorner	40047
U+53CB.0	fHakka	  ju1;  jiu1; ju3; jeu1; riu1; riu3; jiu3
U+53CB.0	fCantonese	jau5; jau2 (also)
U+53CB.0	fMandarin	you3
U+53CB.0	fEnglish	[1] friend; companion; friendly; friendship [2] fraternity; fraternal love [3] befriend
U+53CB.1	fUTF8	友
U+53CB.1	fCNS11643	1-452E
U+53CB.1	fBig5	A4CD
U+53CB.1	fGB	0-5351
U+53CB.1	fR/S	 29.2
U+53CB.1	fTotalStrokes	 4
U+53CB.1	fCangjie	KE
U+53CB.1	fFourCorner	40047
U+53CC.0	fUTF8	双
U+53CC.0	fCNS11643	3-215E
U+53CC.0	fGB	0-4B2B
U+53CC.0	fR/S	 29.2
U+53CC.0	fTotalStrokes	 4
U+53CC.0	fCangjie	EE
U+53CC.0	fFourCorner	17440
U+53CC.0	fHakka	 sung1
U+53CC.0	fCantonese	soeng1; sung1 (also)
U+53CC.0	fMandarin	shuang1; shuan1; shuang4
U+53CC.0	fEnglish	simplified form of &#x96D9;
U+53CD.0	fUTF8	反
U+53CD.0	fCNS11643	1-4530
U+53CD.0	fBig5	A4CF
U+53CD.0	fGB	0-3734
U+53CD.0	fR/S	 29.2
U+53CD.0	fTotalStrokes	 4
U+53CD.0	fCangjie	HE
U+53CD.0	fFourCorner	71247
U+53CD.0	fHakka	  fan3; p'on1
U+53CD.0	fCantonese	faan2; faan1; faan3; ban2
U+53CD.0	fMandarin	fan3; fan1; fan4
U+53CD.0	fEnglish	[1] reverse; opposite; contrary [2] return (something); turn back; retreat [3] introspect; retrospect [4] rebel; rebellion; revolt [5] infer
U+53CE.0	fUTF8	収
U+53CE.0	fCNS11643	3-2160
U+53CE.0	fGB	E-242E
U+53CE.0	fR/S	  29.2
U+53CE.0	fTotalStrokes	 4
U+53CE.0	fCangjie	VLE
U+53CE.0	fFourCorner	27740
U+53CE.0	fHakka	 shu1; shiu1
U+53CE.0	fCantonese	sau1; p65360
U+53CE.0	fMandarin	shou1
U+53CE.0	fEnglish	 gather together; collect harvest
U+53CF.0	fUTF8	叏
U+53CF.0	fCNS11643	4-2169
U+53CF.0	fGB	E-242F
U+53CF.0	fR/S	  29.3
U+53CF.0	fTotalStrokes	 5
U+53CF.0	fCangjie	DJE
U+53CF.0	fFourCorner	50400
U+53CF.0	fCantonese	gwaai3
U+53CF.0	fMandarin	guai4
U+53D0.0	fUTF8	叐
U+53D0.0	fCNS11643	3-222B
U+53D0.0	fGB	E-2430
U+53D0.0	fR/S	 29.3
U+53D0.0	fTotalStrokes	 5
U+53D0.0	fCangjie	HKE
U+53D0.0	fFourCorner	20440
U+53D0.0	fHakka	 pat8
U+53D0.0	fCantonese	bat6
U+53D0.0	fMandarin	ba2
U+53D0.0	fEnglish	 prick a dog to make him go
U+53D1.0	fUTF8	发
U+53D1.0	fGB	0-3722
U+53D1.0	fR/S	 29.3
U+53D1.0	fTotalStrokes	 5
U+53D1.0	fCangjie	IVKE
U+53D1.0	fHakka	 fat7
U+53D1.0	fCantonese	faat3
U+53D1.0	fMandarin	fa1; bo1; fa3
U+53D1.0	fEnglish	simplified form of &#x767C;
U+53D1.1	fUTF8	发
U+53D1.1	fGB	0-3722
U+53D1.1	fR/S	 29.3
U+53D1.1	fTotalStrokes	 5
U+53D1.1	fCangjie	IVKE
U+53D1.1	fMandarin	fa4
U+53D1.2	fUTF8	发
U+53D1.2	fGB	0-3722
U+53D1.2	fR/S	 29.3
U+53D1.2	fTotalStrokes	 5
U+53D1.2	fCangjie	IVKE
U+53D1.2	fMandarin	fa5
U+53D2.0	fUTF8	叒
U+53D2.0	fCNS11643	3-2333
U+53D2.0	fGB	5-3536
U+53D2.0	fR/S	 29.4
U+53D2.0	fTotalStrokes	 6
U+53D2.0	fCangjie	EEE
U+53D2.0	fFourCorner	17440
U+53D2.0	fHakka	 jok7; rok7; joi3
U+53D2.0	fCantonese	joek6
U+53D2.0	fMandarin	ruo4
U+53D2.0	fEnglish	 obedient united
U+53D3.0	fUTF8	叓
U+53D3.0	fCNS11643	4-234F
U+53D3.0	fGB	E-2431
U+53D3.0	fR/S	 29.5
U+53D3.0	fTotalStrokes	 7
U+53D3.0	fCangjie	JLE
U+53D3.0	fFourCorner	50400
U+53D3.0	fCantonese	si6
U+53D3.0	fMandarin	shi4; li4
U+53D3.0	fEnglish	 affair; matter work
U+53D4.0	fUTF8	叔
U+53D4.0	fCNS11643	1-4C3E
U+53D4.0	fBig5	A8FB
U+53D4.0	fGB	0-4A65
U+53D4.0	fR/S	 29.6
U+53D4.0	fTotalStrokes	 8
U+53D4.0	fCangjie	YFE
U+53D4.0	fFourCorner	27940
U+53D4.0	fHakka	  suk7; shuk7
U+53D4.0	fCantonese	suk1
U+53D4.0	fMandarin	shu1
U+53D4.0	fEnglish	[1] father's younger brother; paternal uncles [2] younger brother of one's husband [3] a general designation for members of one's father generation who are younger than one's father [4] declining [5] a Chinese family name
U+53D4.1	fUTF8	叔
U+53D4.1	fCNS11643	1-4C3E
U+53D4.1	fBig5	A8FB
U+53D4.1	fGB	0-4A65
U+53D4.1	fR/S	 29.6
U+53D4.1	fTotalStrokes	 8
U+53D4.1	fCangjie	YFE
U+53D4.1	fFourCorner	27940
U+53D4.1	fMandarin	shu5
U+53D5.0	fUTF8	叕
U+53D5.0	fCNS11643	3-276B
U+53D5.0	fGB	5-3539
U+53D5.0	fR/S	 29.6
U+53D5.0	fTotalStrokes	 8
U+53D5.0	fCangjie	EEEE
U+53D5.0	fFourCorner	17440
U+53D5.0	fCantonese	zyut3
U+53D5.0	fMandarin	chuo4; zhuo2; yi3
U+53D5.0	fEnglish	connect
U+53D6.0	fUTF8	取
U+53D6.0	fCNS11643	1-4C3D
U+53D6.0	fBig5	A8FA
U+53D6.0	fGB	0-4821
U+53D6.0	fR/S	 29.6
U+53D6.0	fTotalStrokes	 8
U+53D6.0	fCangjie	SJE
U+53D6.0	fFourCorner	17140
U+53D6.0	fHakka	  ci3
U+53D6.0	fCantonese	ceoi2
U+53D6.0	fMandarin	qu3; qiu1; qu1; qu4
U+53D6.0	fEnglish	[1] take; receive; obtain; fetch; take hold of [2] select; choose [3] summon; recall [4] marry; take a wife
U+53D7.0	fUTF8	受
U+53D7.0	fCNS11643	1-4C3F
U+53D7.0	fBig5	A8FC
U+53D7.0	fGB	0-4A5C
U+53D7.0	fR/S	 29.6
U+53D7.0	fTotalStrokes	 8
U+53D7.0	fCangjie	BBE
U+53D7.0	fFourCorner	20407
U+53D7.0	fHakka	 su5; shiu6; su3; shiu5; siu3; shu5
U+53D7.0	fCantonese	sau6
U+53D7.0	fMandarin	shou4
U+53D7.0	fEnglish	[1] receive; accept; get; [2] take; bear; stand; suffer; tolerate; endure [3] be pleasant to (the ears, etc) [4] preceding a verb to form a passive voice
U+53D8.0	fUTF8	变
U+53D8.0	fGB	0-3164
U+53D8.0	fR/S	 29.6
U+53D8.0	fTotalStrokes	 8
U+53D8.0	fCangjie	YCE
U+53D8.0	fHakka	 bien5
U+53D8.0	fCantonese	bin3
U+53D8.0	fMandarin	bian4
U+53D8.0	fEnglish	simplified form of &#x8B8A;
U+53D9.0	fUTF8	叙
U+53D9.0	fCNS11643	3-2B50
U+53D9.0	fGB	0-5070
U+53D9.0	fR/S	  29.7
U+53D9.0	fTotalStrokes	 9
U+53D9.0	fCangjie	ODE
U+53D9.0	fFourCorner	87940
U+53D9.0	fHakka	  si5
U+53D9.0	fCantonese	zeoi6
U+53D9.0	fMandarin	xu4
U+53D9.0	fEnglish	express; state; relate; narrate
U+53DA.0	fUTF8	叚
U+53DA.0	fCNS11643	3-2B4F
U+53DA.0	fGB	E-2432
U+53DA.0	fR/S	 29.7
U+53DA.0	fTotalStrokes	 9
U+53DA.0	fCangjie	RSSE
U+53DA.0	fFourCorner	77240
U+53DA.0	fHakka	 ha2; ga3
U+53DA.0	fCantonese	gaa2
U+53DA.0	fMandarin	jia3; jia4; xia2
U+53DA.0	fEnglish	false
U+53DB.0	fUTF8	叛
U+53DB.0	fCNS11643	1-5035
U+53DB.0	fBig5	AB71
U+53DB.0	fGB	0-4551
U+53DB.0	fR/S	 29.7
U+53DB.0	fTotalStrokes	 9
U+53DB.0	fCangjie	FQHE
U+53DB.0	fFourCorner	91547
U+53DB.0	fHakka	 ban3; pan3; ban6; ban5
U+53DB.0	fCantonese	bun6
U+53DB.0	fMandarin	pan4; ban4
U+53DB.0	fEnglish	 rebel; revolt; rebellion; rebellious
U+53DC.0	fUTF8	叜
U+53DC.0	fCNS11643	3-2B4E
U+53DC.0	fGB	5-353A
U+53DC.0	fR/S	 29.7
U+53DC.0	fTotalStrokes	 9
U+53DC.0	fCangjie	JFE
U+53DC.0	fFourCorner	30400
U+53DC.0	fCantonese	sau2
U+53DC.0	fMandarin	sao1; sou3; sou1
U+53DC.0	fEnglish	old man
U+53DD.0	fUTF8	叝
U+53DD.0	fCNS11643	3-2B4D
U+53DD.0	fGB	E-2433
U+53DD.0	fR/S	  29.7
U+53DD.0	fTotalStrokes	 9
U+53DD.0	fCangjie	GISLE
U+53DD.0	fFourCorner	47740
U+53DD.0	fCantonese	gap1
U+53DD.0	fMandarin	gao4; ji2
U+53DD.0	fEnglish	 respectfully inform
U+53DE.0	fUTF8	叞
U+53DE.0	fCNS11643	6-3578
U+53DE.0	fGB	5-353B
U+53DE.0	fR/S	  29.8
U+53DE.0	fTotalStrokes	 10; 9
U+53DE.0	fCangjie	SFE
U+53DE.0	fFourCorner	77240
U+53DE.0	fHakka	 wui3; wi3; wui6; wui5; vui3
U+53DE.0	fCantonese	wai3; wat1; wan6
U+53DE.0	fMandarin	wei4 yu4
U+53DE.0	fEnglish	same as &#x5C09;
U+53DF.0	fUTF8	叟
U+53DF.0	fCNS11643	1-5451
U+53DF.0	fBig5	ADEE
U+53DF.0	fGB	0-5B45
U+53DF.0	fR/S	 29.8
U+53DF.0	fTotalStrokes	 10
U+53DF.0	fCangjie	HXLE
U+53DF.0	fFourCorner	77407
U+53DF.0	fHakka	  seu3; siu3; seu5
U+53DF.0	fCantonese	sau2; sau1
U+53DF.0	fMandarin	sou3; sou1; sao1; shen1
U+53DF.0	fEnglish	 old man; elder; senior; the old; venerable sir
U+53E0.0	fUTF8	叠
U+53E0.0	fCNS11643	3-404B
U+53E0.0	fGB	0-357E
U+53E0.0	fR/S	  29.11
U+53E0.0	fTotalStrokes	 13
U+53E0.0	fCangjie	EEEM
U+53E0.0	fFourCorner	77100
U+53E0.0	fHakka	  tiap8
U+53E0.0	fCantonese	dip6; daap6 (also)
U+53E0.0	fMandarin	die2
U+53E0.0	fEnglish	simplified form of &#x758A;
U+53E1.0	fUTF8	叡
U+53E1.0	fCNS11643	2-5646
U+53E1.0	fBig5	E8FB
U+53E1.0	fGB	E-2434
U+53E1.0	fR/S	 29.14
U+53E1.0	fTotalStrokes	 16
U+53E1.0	fCangjie	YUE
U+53E1.0	fFourCorner	27640
U+53E1.0	fHakka	 jui5; jui3; rui6; rui3
U+53E1.0	fCantonese	jeoi6; haai6
U+53E1.0	fMandarin	rui4
U+53E1.0	fEnglish	astute; profound; shrewd
U+53E2.0	fUTF8	叢
U+53E2.0	fCNS11643	1-763B
U+53E2.0	fBig5	C24F
U+53E2.0	fGB	1-3454
U+53E2.0	fR/S	 29.16
U+53E2.0	fTotalStrokes	 18
U+53E2.0	fCangjie	TCTE
U+53E2.0	fFourCorner	32147
U+53E2.0	fHakka	  cung2; ciung3
U+53E2.0	fCantonese	cung4
U+53E2.0	fMandarin	cong2; cong1
U+53E2.0	fEnglish	[1] shrub; thicket [2] crowd together; meet in large numbers [3] hideout; den (for robbers) [4] a Chinese family name
U+53E3.0	fUTF8	口
U+53E3.0	fCNS11643	1-4447
U+53E3.0	fBig5	A466
U+53E3.0	fGB	0-3F5A
U+53E3.0	fR/S	 30.0
U+53E3.0	fTotalStrokes	 3
U+53E3.0	fCangjie	R
U+53E3.0	fFourCorner	60000
U+53E3.0	fHakka	  heu3;  k'ieu3; keu3; kiu3; hiu3; heu5
U+53E3.0	fCantonese	hau2
U+53E3.0	fMandarin	kou3
U+53E3.0	fEnglish	[1] mouth [2] open end; opening [3] entrance; gate (especially in the Great Wall or city walls) [4] person [5] a certain article (as a cistern, a big jar, etc) [6] edge or blade of a knife [7] crack
U+53E3.1	fUTF8	口
U+53E3.1	fCNS11643	1-4447
U+53E3.1	fBig5	A466
U+53E3.1	fGB	0-3F5A
U+53E3.1	fR/S	 30.0
U+53E3.1	fTotalStrokes	 3
U+53E3.1	fCangjie	R
U+53E3.1	fFourCorner	60000
U+53E3.1	fMandarin	kou5
U+53E4.0	fUTF8	古
U+53E4.0	fCNS11643	1-462C
U+53E4.0	fBig5	A56A
U+53E4.0	fGB	0-3945
U+53E4.0	fR/S	 30.2
U+53E4.0	fTotalStrokes	 5
U+53E4.0	fCangjie	JR
U+53E4.0	fFourCorner	40600
U+53E4.0	fHakka	  gu3
U+53E4.0	fCantonese	gu2; gwu2 (also); gu1 (also); gwu1 (also)
U+53E4.0	fMandarin	gu3
U+53E4.0	fEnglish	[1] old; classic; ancient; antiquated; antiquity [2] not following current customs or practices [3] a Chinese family name
U+53E5.0	fUTF8	句
U+53E5.0	fCNS11643	1-463B
U+53E5.0	fBig5	A579
U+53E5.0	fGB	0-3E64
U+53E5.0	fR/S	 30.2
U+53E5.0	fTotalStrokes	 5
U+53E5.0	fCangjie	PR
U+53E5.0	fFourCorner	27620
U+53E5.0	fHakka	  gi5
U+53E5.0	fCantonese	geoi3; ngau1; gau1 (also); geoi2 (also); au1 (also); gau3
U+53E5.0	fMandarin	ju4; gou4; qu2
U+53E5.0	fEnglish	 sentence
U+53E5.1	fUTF8	句
U+53E5.1	fCNS11643	1-463B
U+53E5.1	fBig5	A579
U+53E5.1	fGB	0-3E64
U+53E5.1	fR/S	 30.2
U+53E5.1	fTotalStrokes	 5
U+53E5.1	fCangjie	PR
U+53E5.1	fFourCorner	27620
U+53E5.1	fMandarin	gou1
U+53E5.1	fEnglish	a Chinese family name
U+53E6.0	fUTF8	另
U+53E6.0	fCNS11643	1-4636
U+53E6.0	fBig5	A574
U+53E6.0	fGB	0-416D
U+53E6.0	fR/S	 30.2
U+53E6.0	fTotalStrokes	 5
U+53E6.0	fCangjie	RKS
U+53E6.0	fFourCorner	60427
U+53E6.0	fHakka	  lang3; lang5; lang6; lin5; nang6; nang5; len3
U+53E6.0	fCantonese	ling6
U+53E6.0	fMandarin	ling4
U+53E6.0	fEnglish	[1] another; extra; in addition; besides; other [2] separate; separation (as a couple); divide
U+53E7.0	fUTF8	叧
U+53E7.0	fCNS11643	4-216C
U+53E7.0	fGB	E-2435
U+53E7.0	fR/S	 30.2
U+53E7.0	fTotalStrokes	 5
U+53E7.0	fCangjie	RSH
U+53E7.0	fFourCorner	60220
U+53E7.0	fCantonese	waa1
U+53E7.0	fMandarin	gua3; kuai1
U+53E7.0	fEnglish	 cut meat off away from bones ancient torture
U+53E8.0	fUTF8	叨
U+53E8.0	fCNS11643	1-4631
U+53E8.0	fBig5	A56F
U+53E8.0	fGB	0-5F36
U+53E8.0	fR/S	 30.2
U+53E8.0	fTotalStrokes	 5
U+53E8.0	fCangjie	RSH
U+53E8.0	fFourCorner	67020
U+53E8.0	fHakka	 tau1; to1; dau1
U+53E8.0	fCantonese	tou1; dou1
U+53E8.0	fMandarin	dao1; dao2
U+53E8.0	fEnglish	talkative; quarrulous; garrulous; fond of talking
U+53E8.1	fUTF8	叨
U+53E8.1	fCNS11643	1-4631
U+53E8.1	fBig5	A56F
U+53E8.1	fGB	0-5F36
U+53E8.1	fR/S	 30.2
U+53E8.1	fTotalStrokes	 5
U+53E8.1	fCangjie	RSH
U+53E8.1	fFourCorner	67020
U+53E8.1	fMandarin	tao1
U+53E8.1	fEnglish	 be favored; get the benefit of
U+53E9.0	fUTF8	叩
U+53E9.0	fCNS11643	1-4630
U+53E9.0	fBig5	A56E
U+53E9.0	fGB	0-5F35
U+53E9.0	fR/S	 30.2
U+53E9.0	fTotalStrokes	 5
U+53E9.0	fCangjie	RSL
U+53E9.0	fFourCorner	67020
U+53E9.0	fHakka	  keu5;  k'ieu5; kiu5
U+53E9.0	fCantonese	kau3
U+53E9.0	fMandarin	kou4; kou3
U+53E9.0	fEnglish	[1] knock; hit [2] ask [3] kowtow; bow
U+53EA.0	fUTF8	只
U+53EA.0	fCNS11643	1-4637
U+53EA.0	fBig5	A575
U+53EA.0	fGB	0-563B
U+53EA.0	fR/S	 30.2
U+53EA.0	fTotalStrokes	 5
U+53EA.0	fCangjie	RC
U+53EA.0	fFourCorner	60800
U+53EA.0	fHakka	  zhit7;  zit7; zi3; zhi3; zih3; tsiit7
U+53EA.0	fCantonese	zi2; zek3; zi1; kei4
U+53EA.0	fMandarin	zhi3; qi2; ti3
U+53EA.0	fEnglish	[1] only; just; simply [2] but; yet [3] simplified form of &#x8879;
U+53EA.1	fUTF8	只
U+53EA.1	fCNS11643	1-4637
U+53EA.1	fBig5	A575
U+53EA.1	fGB	0-563B
U+53EA.1	fR/S	 30.2
U+53EA.1	fTotalStrokes	 5
U+53EA.1	fCangjie	RC
U+53EA.1	fFourCorner	60800
U+53EA.1	fMandarin	zhi1
U+53EB.0	fUTF8	叫
U+53EB.0	fCNS11643	1-4635
U+53EB.0	fBig5	A573
U+53EB.0	fGB	0-3D50
U+53EB.0	fR/S	 30.2
U+53EB.0	fTotalStrokes	 5
U+53EB.0	fCangjie	RVL
U+53EB.0	fFourCorner	64000
U+53EB.0	fHakka	  giau5; kieu5
U+53EB.0	fCantonese	giu3
U+53EB.0	fMandarin	jiao4
U+53EB.0	fEnglish	[1] cry; shout; hail; scream [2] be called; known as [3] call; summon [4] cause
U+53EC.0	fUTF8	召
U+53EC.0	fCNS11643	1-462E
U+53EC.0	fBig5	A56C
U+53EC.0	fGB	0-5559
U+53EC.0	fR/S	 30.2
U+53EC.0	fTotalStrokes	 5
U+53EC.0	fCangjie	SHR
U+53EC.0	fFourCorner	17602
U+53EC.0	fHakka	 zau1; seu5; shau5; ch'eu5; ceu5; zhau5; chau5
U+53EC.0	fCantonese	ziu6; siu6; ziu1
U+53EC.0	fMandarin	zhao4; shao4; zhao1
U+53EC.0	fEnglish	[1] imperial decree; summon; call up [2] cause; invite
U+53ED.0	fUTF8	叭
U+53ED.0	fCNS11643	1-463C
U+53ED.0	fBig5	A57A
U+53ED.0	fGB	0-3048
U+53ED.0	fR/S	 30.2
U+53ED.0	fTotalStrokes	 5
U+53ED.0	fCangjie	RC
U+53ED.0	fFourCorner	68000
U+53ED.0	fHakka	  ba5; bak7; pat7; bat8; ba1; pat8
U+53ED.0	fCantonese	baa1
U+53ED.0	fMandarin	ba1; ba5
U+53ED.0	fEnglish	 trumpet
U+53EE.0	fUTF8	叮
U+53EE.0	fCNS11643	1-462F
U+53EE.0	fBig5	A56D
U+53EE.0	fGB	0-3623
U+53EE.0	fR/S	 30.2
U+53EE.0	fTotalStrokes	 5
U+53EE.0	fCangjie	RMN
U+53EE.0	fFourCorner	61020
U+53EE.0	fHakka	 den1; dang5; din1
U+53EE.0	fCantonese	ding1; deng1 (also)
U+53EE.0	fMandarin	ding1
U+53EE.0	fEnglish	[1] chimes of a bell [2] exhort or enjoin repeatedly [3] sting (as a mosquito, etc.)
U+53EF.0	fUTF8	可
U+53EF.0	fCNS11643	1-462B
U+53EF.0	fBig5	A569
U+53EF.0	fGB	0-3F49
U+53EF.0	fR/S	 30.2
U+53EF.0	fTotalStrokes	 5
U+53EF.0	fCangjie	MNR
U+53EF.0	fFourCorner	10620
U+53EF.0	fHakka	  ko3; ho3
U+53EF.0	fCantonese	ho2; hak1
U+53EF.0	fMandarin	ke3
U+53EF.0	fEnglish	[1] may; can; be able to; possibly [2] around; estimated at [3] an auxiliary [4] but; however [5] a Chinese family name
U+53EF.1	fUTF8	可
U+53EF.1	fCNS11643	1-462B
U+53EF.1	fBig5	A569
U+53EF.1	fGB	0-3F49
U+53EF.1	fR/S	 30.2
U+53EF.1	fTotalStrokes	 5
U+53EF.1	fCangjie	MNR
U+53EF.1	fFourCorner	10620
U+53EF.1	fMandarin	ke4
U+53EF.1	fEnglish	used in combination for khan
U+53F0.0	fUTF8	台
U+53F0.0	fCNS11643	1-463A
U+53F0.0	fBig5	A578
U+53F0.0	fGB	0-4C28
U+53F0.0	fR/S	 30.2
U+53F0.0	fTotalStrokes	 5
U+53F0.0	fCangjie	IR
U+53F0.0	fFourCorner	23600
U+53F0.0	fHakka	  toi2; toi3; ri2; ji2
U+53F0.0	fCantonese	toi4; ji4; toi2 (also)
U+53F0.0	fMandarin	tai2; hu2; yi2; tai1
U+53F0.0	fEnglish	simplified form of &#x81FA;
U+53F1.0	fUTF8	叱
U+53F1.0	fCNS11643	1-4639
U+53F1.0	fBig5	A577
U+53F1.0	fGB	0-5F33
U+53F1.0	fR/S	 30.2
U+53F1.0	fTotalStrokes	 5
U+53F1.0	fCangjie	RP
U+53F1.0	fFourCorner	64010
U+53F1.0	fHakka	  fa5
U+53F1.0	fCantonese	cik1
U+53F1.0	fMandarin	chi4
U+53F1.0	fEnglish	 scold; shout at; bawl out; revile
U+53F2.0	fUTF8	史
U+53F2.0	fCNS11643	1-4638
U+53F2.0	fBig5	A576
U+53F2.0	fGB	0-4A37
U+53F2.0	fR/S	 30.2
U+53F2.0	fTotalStrokes	 5
U+53F2.0	fCangjie	LK
U+53F2.0	fFourCorner	50006
U+53F2.0	fHakka	 sii3; su3; su5; sii6; se3
U+53F2.0	fCantonese	si2
U+53F2.0	fMandarin	shi3
U+53F2.0	fEnglish	[1] history; chronicle; annals [2] a Chinese family name
U+53F3.0	fUTF8	右
U+53F3.0	fCNS11643	1-462D
U+53F3.0	fBig5	A56B
U+53F3.0	fGB	0-5352
U+53F3.0	fR/S	 30.2
U+53F3.0	fTotalStrokes	 5
U+53F3.0	fCangjie	KR
U+53F3.0	fFourCorner	40600
U+53F3.0	fHakka	 ju5; jiu3; jeu3; riu6; ju3; jiu6
U+53F3.0	fCantonese	jau6; jau2 (also)
U+53F3.0	fMandarin	you4; you3
U+53F3.0	fEnglish	[1] right [2] west [3] assist; aid [4] emphasize
U+53F4.0	fUTF8	叴
U+53F4.0	fCNS11643	3-2231
U+53F4.0	fGB	5-3765
U+53F4.0	fR/S	 30.2
U+53F4.0	fTotalStrokes	 5
U+53F4.0	fCangjie	KNR
U+53F4.0	fFourCorner	40600
U+53F4.0	fHakka	  kiu2
U+53F4.0	fCantonese	kau4
U+53F4.0	fMandarin	qiu2
U+53F4.0	fEnglish	a spear the head of which has three edges
U+53F5.0	fUTF8	叵
U+53F5.0	fCNS11643	1-4634
U+53F5.0	fBig5	A572
U+53F5.0	fGB	0-584F
U+53F5.0	fR/S	 30.2
U+53F5.0	fTotalStrokes	 5
U+53F5.0	fCangjie	SR
U+53F5.0	fFourCorner	71716
U+53F5.0	fHakka	 po3
U+53F5.0	fCantonese	po2
U+53F5.0	fMandarin	po3
U+53F5.0	fEnglish	cannot; be unable do; improbable
U+53F6.0	fUTF8	叶
U+53F6.0	fCNS11643	3-222D
U+53F6.0	fGB	0-5236
U+53F6.0	fR/S	  30.2
U+53F6.0	fTotalStrokes	 5
U+53F6.0	fCangjie	RJ
U+53F6.0	fFourCorner	64000
U+53F6.0	fHakka	 hiap8; shap8; hiap7; sap8; jap8
U+53F6.0	fCantonese	hip3; hip6 (also); jip6; jip2 (also); sip3
U+53F6.0	fMandarin	xie2; ye4; die2; she4
U+53F6.0	fEnglish	simplified form of &#x8449;
U+53F7.0	fUTF8	号
U+53F7.0	fCNS11643	3-222E
U+53F7.0	fGB	0-3A45
U+53F7.0	fR/S	 30.2
U+53F7.0	fTotalStrokes	 5
U+53F7.0	fCangjie	RMVS
U+53F7.0	fFourCorner	60200
U+53F7.0	fHakka	 hau5; ho5; ho6; hau6; hau1
U+53F7.0	fCantonese	hou6; hou2 (also); hou4
U+53F7.0	fMandarin	hao4
U+53F7.0	fEnglish	simplified form of &#x865F;
U+53F7.1	fUTF8	号
U+53F7.1	fCNS11643	3-222E
U+53F7.1	fGB	0-3A45
U+53F7.1	fR/S	 30.2
U+53F7.1	fTotalStrokes	 5
U+53F7.1	fCangjie	RMVS
U+53F7.1	fFourCorner	60200
U+53F7.1	fMandarin	hao2
U+53F7.2	fUTF8	号
U+53F7.2	fCNS11643	3-222E
U+53F7.2	fGB	0-3A45
U+53F7.2	fR/S	 30.2
U+53F7.2	fTotalStrokes	 5
U+53F7.2	fCangjie	RMVS
U+53F7.2	fFourCorner	60200
U+53F7.2	fMandarin	hao5
U+53F8.0	fUTF8	司
U+53F8.0	fCNS11643	1-4633
U+53F8.0	fBig5	A571
U+53F8.0	fGB	0-4B3E
U+53F8.0	fR/S	 30.2
U+53F8.0	fTotalStrokes	 5
U+53F8.0	fCangjie	SMR
U+53F8.0	fFourCorner	17620
U+53F8.0	fHakka	  sii1;  su1; se1
U+53F8.0	fCantonese	si1
U+53F8.0	fMandarin	si1; si4
U+53F8.0	fEnglish	[1] have charge of; control; manage; preside over [2] (government) department
U+53F9.0	fUTF8	叹
U+53F9.0	fCNS11643	3-222F
U+53F9.0	fGB	0-4C3E
U+53F9.0	fR/S	  30.2
U+53F9.0	fTotalStrokes	 5
U+53F9.0	fCangjie	RE
U+53F9.0	fFourCorner	67040
U+53F9.0	fHakka	 tan5
U+53F9.0	fCantonese	taan3
U+53F9.0	fMandarin	yi3; tan4; tan1
U+53F9.0	fEnglish	simplified form of &#x5606;
U+53FA.0	fUTF8	叺
U+53FA.0	fCNS11643	F-2150
U+53FA.0	fGB	E-2436
U+53FA.0	fR/S	 30.2
U+53FA.0	fTotalStrokes	 5
U+53FA.0	fCangjie	RO
U+53FA.0	fFourCorner	6800
U+53FA.0	fCantonese	cek3
U+53FA.0	fMandarin	chi3
U+53FA.0	fEnglish	a straw bag; a tobacco pouch
U+53FB.0	fUTF8	叻
U+53FB.0	fCNS11643	1-463D
U+53FB.0	fBig5	A57B
U+53FB.0	fGB	0-5F37
U+53FB.0	fR/S	 30.2
U+53FB.0	fTotalStrokes	 5
U+53FB.0	fCangjie	RKS
U+53FB.0	fFourCorner	64027
U+53FB.0	fHakka	 let8; lit8; liak8
U+53FB.0	fCantonese	lak6; lik1 (also); lek1; laak6 (also)
U+53FB.0	fMandarin	le4; li4
U+53FB.0	fEnglish	[1] place names [2] clever; smart; sharp; brilliant
U+53FC.0	fUTF8	叼
U+53FC.0	fCNS11643	1-4632
U+53FC.0	fBig5	A570
U+53FC.0	fGB	0-3570
U+53FC.0	fR/S	 30.2
U+53FC.0	fTotalStrokes	 5
U+53FC.0	fCangjie	RSM
U+53FC.0	fFourCorner	67020
U+53FC.0	fHakka	 diau1
U+53FC.0	fCantonese	diu1
U+53FC.0	fMandarin	diao1
U+53FC.0	fEnglish	holding in mouth
U+53FD.0	fUTF8	叽
U+53FD.0	fCNS11643	4-216A
U+53FD.0	fGB	0-5F34
U+53FD.0	fR/S	  30.2
U+53FD.0	fTotalStrokes	 5
U+53FD.0	fCangjie	RHN
U+53FD.0	fFourCorner	62010
U+53FD.0	fCantonese	gei1
U+53FD.0	fMandarin	jiao1; ji1; ji4
U+53FD.0	fEnglish	simplified form of &#x5630;
U+53FE.0	fUTF8	叾
U+53FE.0	fGB	1-7C57
U+53FE.0	fR/S	 30.2
U+53FE.0	fTotalStrokes	 5
U+53FE.0	fCangjie	NNR
U+53FE.0	fCantonese	duk1
U+53FE.0	fEnglish	西貢叾, a place in Hong Kong
U+53FF.0	fUTF8	叿
U+53FF.0	fCNS11643	3-2335
U+53FF.0	fGB	5-3767
U+53FF.0	fR/S	  30.3
U+53FF.0	fTotalStrokes	 6
U+53FF.0	fCangjie	RM
U+53FF.0	fFourCorner	61010
U+53FF.0	fHakka	  fung2; fung1
U+53FF.0	fCantonese	hung1
U+53FF.0	fMandarin	hong1
U+53FF.0	fEnglish	din of a market place
U+5400.0	fUTF8	吀
U+5400.0	fCNS11643	3-2337
U+5400.0	fGB	E-2437
U+5400.0	fR/S	  30.3
U+5400.0	fTotalStrokes	 6
U+5400.0	fCangjie	RHJ
U+5400.0	fFourCorner	62040
U+5400.0	fCantonese	mei5
U+5400.0	fMandarin	mie1
U+5401.0	fUTF8	吁
U+5401.0	fCNS11643	1-4754
U+5401.0	fBig5	A653
U+5401.0	fGB	0-5375
U+5401.0	fR/S	 30.3
U+5401.0	fTotalStrokes	 6
U+5401.0	fCangjie	RMD
U+5401.0	fFourCorner	61040
U+5401.0	fHakka	 hi1; wu1; ji1
U+5401.0	fCantonese	heoi1; jyu6; joek6 (also); jyu1
U+5401.0	fMandarin	xu1
U+5401.0	fEnglish	 sigh
U+5401.1	fUTF8	吁
U+5401.1	fCNS11643	1-4754
U+5401.1	fBig5	A653
U+5401.1	fGB	0-5375
U+5401.1	fR/S	 30.3
U+5401.1	fTotalStrokes	 6
U+5401.1	fCangjie	RMD
U+5401.1	fFourCorner	61040
U+5401.1	fMandarin	yu4
U+5402.0	fUTF8	吂
U+5402.0	fCNS11643	4-224C
U+5402.0	fGB	3-3551
U+5402.0	fR/S	  30.3
U+5402.0	fTotalStrokes	 6
U+5402.0	fCangjie	YVR
U+5402.0	fFourCorner	00600
U+5402.0	fHakka	 mang2
U+5402.0	fCantonese	mong4
U+5402.0	fMandarin	mang2
U+5402.1	fUTF8	吂
U+5402.1	fCNS11643	4-224C
U+5402.1	fGB	3-3551
U+5402.1	fR/S	  30.3
U+5402.1	fTotalStrokes	 6
U+5402.1	fCangjie	YVR
U+5402.1	fFourCorner	00600
U+5402.1	fMandarin	mang4
U+5403.0	fUTF8	吃
U+5403.0	fCNS11643	1-475A
U+5403.0	fBig5	A659
U+5403.0	fGB	0-3354
U+5403.0	fR/S	 30.3
U+5403.0	fTotalStrokes	 6
U+5403.0	fCangjie	RON
U+5403.0	fFourCorner	68017
U+5403.0	fHakka	  ngiet7; kiet7;  ket7; kit8;  ngat7; nget7
U+5403.0	fCantonese	hek3; jaak3 (also); hat1 (also); gat1; kat1
U+5403.0	fMandarin	chi1; ji1
U+5403.0	fEnglish	[1] eat [2] sustain; suffer; endure; bear [3] inhale; drink
U+5403.1	fUTF8	吃
U+5403.1	fCNS11643	1-475A
U+5403.1	fBig5	A659
U+5403.1	fGB	0-3354
U+5403.1	fR/S	 30.3
U+5403.1	fTotalStrokes	 6
U+5403.1	fCangjie	RON
U+5403.1	fFourCorner	68017
U+5403.1	fMandarin	ji2
U+5403.1	fEnglish	 stammer; stutter
U+5404.0	fUTF8	各
U+5404.0	fCNS11643	1-4756
U+5404.0	fBig5	A655
U+5404.0	fGB	0-3877
U+5404.0	fR/S	 30.3
U+5404.0	fTotalStrokes	 6
U+5404.0	fCangjie	HER
U+5404.0	fFourCorner	27604
U+5404.0	fHakka	  gok7
U+5404.0	fCantonese	gok3; go3
U+5404.0	fMandarin	ge4; ge2; ge3
U+5404.0	fEnglish	[1] each; individually; every [2] all
U+5405.0	fUTF8	吅
U+5405.0	fCNS11643	3-2339
U+5405.0	fGB	5-376C
U+5405.0	fR/S	 30.3
U+5405.0	fTotalStrokes	 6
U+5405.0	fCangjie	RR
U+5405.0	fFourCorner	66000
U+5405.0	fCantonese	hyun1
U+5405.0	fMandarin	xuan1
U+5406.0	fUTF8	吆
U+5406.0	fCNS11643	1-475C
U+5406.0	fBig5	A65B
U+5406.0	fGB	0-5F3A
U+5406.0	fR/S	 30.3
U+5406.0	fTotalStrokes	 6
U+5406.0	fCangjie	RVI
U+5406.0	fFourCorner	62032
U+5406.0	fHakka	  jau1; rau1; jeu1
U+5406.0	fCantonese	jiu1
U+5406.0	fMandarin	yao1
U+5406.0	fEnglish	 bawl; yell; shout; cry out
U+5407.0	fUTF8	吇
U+5407.0	fCNS11643	2-2225
U+5407.0	fBig5	C9C5
U+5407.0	fGB	5-3770
U+5407.0	fR/S	 30.3
U+5407.0	fTotalStrokes	 6
U+5407.0	fCangjie	RND
U+5407.0	fFourCorner	67047
U+5407.0	fCantonese	zi2
U+5407.0	fMandarin	zi3
U+5408.0	fUTF8	合
U+5408.0	fCNS11643	1-4759
U+5408.0	fBig5	A658
U+5408.0	fGB	0-3A4F
U+5408.0	fR/S	 30.3
U+5408.0	fTotalStrokes	 6
U+5408.0	fCangjie	OMR
U+5408.0	fFourCorner	80601
U+5408.0	fHakka	  hap8;  gap7; kap7;  gak7; lap7
U+5408.0	fCantonese	hap6; gap3; gaap3 (also); ho4; kap3
U+5408.0	fMandarin	he2; ge3; xia2
U+5408.0	fEnglish	[1] combine; unite; join; gather; collect [2] close; shut [3] suit
U+5409.0	fUTF8	吉
U+5409.0	fCNS11643	1-474F
U+5409.0	fBig5	A64E
U+5409.0	fGB	0-3C2A
U+5409.0	fR/S	 30.3
U+5409.0	fTotalStrokes	 6
U+5409.0	fCangjie	GR
U+5409.0	fFourCorner	40601
U+5409.0	fHakka	  git7
U+5409.0	fCantonese	gat1
U+5409.0	fMandarin	ji2
U+5409.0	fEnglish	lucky; propitious; good; auspicious; fortunate; favorable
U+540A.0	fUTF8	吊
U+540A.0	fCNS11643	1-4752
U+540A.0	fBig5	A651
U+540A.0	fGB	0-3575
U+540A.0	fR/S	 30.3
U+540A.0	fTotalStrokes	 6
U+540A.0	fCangjie	RLB
U+540A.0	fFourCorner	60227
U+540A.0	fHakka	  diau5
U+540A.0	fCantonese	diu3; diu2 (also); diu4 (also)
U+540A.0	fMandarin	diao4
U+540A.0	fEnglish	 condole; mourn; pity hang
U+540B.0	fUTF8	吋
U+540B.0	fCNS11643	1-4755
U+540B.0	fBig5	A654
U+540B.0	fGB	8-2873
U+540B.0	fR/S	 30.3
U+540B.0	fTotalStrokes	 6
U+540B.0	fCangjie	RDI
U+540B.0	fFourCorner	64000
U+540B.0	fHakka	 cun5
U+540B.0	fCantonese	cyun3; jing1 (also)
U+540B.0	fMandarin	cun4; dou3; dou4
U+540B.0	fEnglish	 inch
U+540C.0	fUTF8	同
U+540C.0	fCNS11643	1-4751
U+540C.0	fBig5	A650
U+540C.0	fGB	0-4D2C
U+540C.0	fR/S	 30.3
U+540C.0	fTotalStrokes	 6
U+540C.0	fCangjie	BMR
U+540C.0	fFourCorner	77220
U+540C.0	fHakka	  tung2; tung3
U+540C.0	fCantonese	tung4
U+540C.0	fMandarin	tong2
U+540C.0	fEnglish	[1] same; similar; equal; identical; common [2] share; agree [3] together
U+540C.1	fUTF8	同
U+540C.1	fCNS11643	1-4751
U+540C.1	fBig5	A650
U+540C.1	fGB	0-4D2C
U+540C.1	fR/S	 30.3
U+540C.1	fTotalStrokes	 6
U+540C.1	fCangjie	BMR
U+540C.1	fFourCorner	77220
U+540C.1	fMandarin	tong4
U+540C.2	fUTF8	同
U+540C.2	fCNS11643	1-4751
U+540C.2	fBig5	A650
U+540C.2	fGB	0-4D2C
U+540C.2	fR/S	 30.3
U+540C.2	fTotalStrokes	 6
U+540C.2	fCangjie	BMR
U+540C.2	fFourCorner	77220
U+540D.0	fUTF8	名
U+540D.0	fCNS11643	1-4758
U+540D.0	fBig5	A657
U+540D.0	fGB	0-437B
U+540D.0	fR/S	 30.3
U+540D.0	fTotalStrokes	 6
U+540D.0	fCangjie	NIR
U+540D.0	fFourCorner	27600
U+540D.0	fHakka	  miang2; miang3
U+540D.0	fCantonese	ming4; meng4 (also); meng2 (also)
U+540D.0	fMandarin	ming2
U+540D.0	fEnglish	[1] name; rank; title; designation [2] position; honor; fame; renown; reputation [3] famous; noted; distinguished; renowned; valuable; precious; noble; rare; great [4] name; describe
U+540E.0	fUTF8	后
U+540E.0	fCNS11643	1-475B
U+540E.0	fBig5	A65A
U+540E.0	fGB	0-3A73
U+540E.0	fR/S	 30.3
U+540E.0	fTotalStrokes	 6
U+540E.0	fCangjie	HMR
U+540E.0	fFourCorner	72261
U+540E.0	fHakka	  heu5; heu6; heu3; hiu3
U+540E.0	fCantonese	hau6; hau2 (also)
U+540E.0	fMandarin	hou4; hou3
U+540E.0	fEnglish	[1] queen; empress; sovereign [2] the god of the earth [3] after; behind [4] simplified form of &#x5F8C;
U+540F.0	fUTF8	吏
U+540F.0	fCNS11643	1-4750
U+540F.0	fBig5	A64F
U+540F.0	fGB	0-4074
U+540F.0	fR/S	 30.3
U+540F.0	fTotalStrokes	 6
U+540F.0	fCangjie	JLK
U+540F.0	fFourCorner	50006
U+540F.0	fHakka	  li5; li6; li3
U+540F.0	fCantonese	lei6
U+540F.0	fMandarin	li4
U+540F.0	fEnglish	 civil officer; government official; magistrate
U+5410.0	fUTF8	吐
U+5410.0	fCNS11643	1-4753
U+5410.0	fBig5	A652
U+5410.0	fGB	0-4D42
U+5410.0	fR/S	 30.3
U+5410.0	fTotalStrokes	 6
U+5410.0	fCangjie	RG
U+5410.0	fFourCorner	64010
U+5410.0	fHakka	 tu5
U+5410.0	fCantonese	dat6
U+5410.0	fMandarin	tu3
U+5410.0	fEnglish	 spit; utter
U+5410.1	fUTF8	吐
U+5410.1	fCNS11643	1-4753
U+5410.1	fBig5	A652
U+5410.1	fGB	0-4D42
U+5410.1	fR/S	 30.3
U+5410.1	fTotalStrokes	 6
U+5410.1	fCangjie	RG
U+5410.1	fFourCorner	64010
U+5410.1	fHakka	  tu5
U+5410.1	fCantonese	tou3; tou2
U+5410.1	fMandarin	tu4
U+5410.1	fEnglish	 vomit; spew (out); cough up; throw up
U+5411.0	fUTF8	向
U+5411.0	fCNS11643	1-4757
U+5411.0	fBig5	A656
U+5411.0	fGB	0-4F72
U+5411.0	fR/S	 30.3
U+5411.0	fTotalStrokes	 6
U+5411.0	fCangjie	HBR
U+5411.0	fFourCorner	27220
U+5411.0	fHakka	  hiong5
U+5411.0	fCantonese	hoeng3; hoeng2 (also)
U+5411.0	fMandarin	xiang4; xiang3
U+5411.0	fEnglish	[1] turn; face [2] toward; direction; trend [3] until now [4] a Chinese family name
U+5412.0	fUTF8	吒
U+5412.0	fCNS11643	1-475D
U+5412.0	fBig5	A65C
U+5412.0	fGB	0-5F38
U+5412.0	fR/S	 30.3
U+5412.0	fTotalStrokes	 6
U+5412.0	fCangjie	RHP
U+5412.0	fFourCorner	62014
U+5412.0	fHakka	 za5; za1
U+5412.0	fCantonese	zaa1; zaa3; caak1 (also)
U+5412.0	fMandarin	zha4; zha1; cha4
U+5412.0	fEnglish	[1] shout with anger; roar [2] smack on eating
U+5413.0	fUTF8	吓
U+5413.0	fCNS11643	3-233A
U+5413.0	fGB	0-4F45
U+5413.0	fR/S	  30.3
U+5413.0	fTotalStrokes	 6
U+5413.0	fCangjie	RMY
U+5413.0	fFourCorner	61030
U+5413.0	fHakka	 ha3; hak7
U+5413.0	fCantonese	haa2; haak3; haa5; haak1; haa1 (also)
U+5413.0	fMandarin	xia4; he4
U+5413.0	fEnglish	[1] simplified form of &#x5687; [2] simplified form of &#x5687;
U+5414.0	fUTF8	吔
U+5414.0	fGB	3-3552
U+5414.0	fR/S	 30.3
U+5414.0	fTotalStrokes	 6
U+5414.0	fCangjie	RPD
U+5414.0	fFourCorner	6401
U+5414.0	fHakka	 e3; i5
U+5414.0	fCantonese	jaa1; jaa2; jaa4; jaak3; je5
U+5414.0	fMandarin	ye3
U+5414.0	fEnglish	 cry of pain used for -e-
U+5415.0	fUTF8	吕
U+5415.0	fCNS11643	3-233B
U+5415.0	fGB	0-4240
U+5415.0	fR/S	 30.3
U+5415.0	fTotalStrokes	 6
U+5415.0	fCangjie	RR
U+5415.0	fFourCorner	60600
U+5415.0	fCantonese	leoi5
U+5415.0	fMandarin	luu3; l3
U+5415.0	fEnglish	 a Chinese family name a musical note
U+5416.0	fUTF8	吖
U+5416.0	fCNS11643	3-2334
U+5416.0	fGB	0-5F39
U+5416.0	fR/S	 30.3
U+5416.0	fTotalStrokes	 6
U+5416.0	fCangjie	RCL
U+5416.0	fFourCorner	62020
U+5416.0	fHakka	 a1
U+5416.0	fCantonese	aa1; ngaa1 (also); aa3 (also); ngaa3 (also)
U+5416.0	fMandarin	a1
U+5416.0	fEnglish	[1] used in transliteration [2] [final particle] expresses idea of disagreeing or contradicting
U+5417.0	fUTF8	吗
U+5417.0	fGB	0-4270
U+5417.0	fR/S	 30.3
U+5417.0	fTotalStrokes	 6
U+5417.0	fCangjie	RNVM
U+5417.0	fHakka	 ma1; ma3
U+5417.0	fCantonese	maa1; maa3 (also); maa5
U+5417.0	fMandarin	ma2; ma3; ma5; ma1; ma4
U+5417.0	fEnglish	simplified form of &#x55CE;
U+5417.1	fUTF8	吗
U+5417.1	fGB	0-4270
U+5417.1	fR/S	 30.3
U+5417.1	fTotalStrokes	 6
U+5417.1	fCangjie	RNVM
U+5418.0	fUTF8	吘
U+5418.0	fCNS11643	2-2340
U+5418.0	fBig5	CA7E
U+5418.0	fGB	5-3777
U+5418.0	fR/S	 30.4
U+5418.0	fTotalStrokes	 7
U+5418.0	fCangjie	ROJ
U+5418.0	fFourCorner	68040
U+5418.0	fCantonese	ngau5; au5 (also)
U+5418.0	fMandarin	ou3
U+5419.0	fUTF8	吙
U+5419.0	fCNS11643	2-233D
U+5419.0	fBig5	CA7B
U+5419.0	fGB	5-3821
U+5419.0	fR/S	 30.4
U+5419.0	fTotalStrokes	 7
U+5419.0	fCangjie	RF
U+5419.0	fFourCorner	69080
U+5419.0	fCantonese	hoe1; fo2
U+5419.0	fMandarin	xue1
U+541A.0	fUTF8	吚
U+541A.0	fCNS11643	3-2476
U+541A.0	fGB	E-2438
U+541A.0	fR/S	 30.4
U+541A.0	fTotalStrokes	 7
U+541A.0	fCangjie	RSK
U+541A.0	fFourCorner	67050
U+541A.0	fHakka	 ji2; ri2; i1
U+541A.0	fCantonese	ji1
U+541A.0	fMandarin	yi1
U+541A.0	fEnglish	used to represent sound
U+541B.0	fUTF8	君
U+541B.0	fCNS11643	1-4949
U+541B.0	fBig5	A767
U+541B.0	fGB	0-3E7D
U+541B.0	fR/S	 30.4
U+541B.0	fTotalStrokes	 7
U+541B.0	fCangjie	SKR
U+541B.0	fFourCorner	17607
U+541B.0	fHakka	  giun1
U+541B.0	fCantonese	gwan1
U+541B.0	fMandarin	jun1
U+541B.0	fEnglish	[1] sovereign; monarch; ruler; king; lord [2] you (used in addressing a male in formal speech)
U+541C.0	fUTF8	吜
U+541C.0	fCNS11643	2-233E
U+541C.0	fBig5	CA7C
U+541C.0	fGB	5-3823
U+541C.0	fR/S	 30.4
U+541C.0	fTotalStrokes	 7
U+541C.0	fCangjie	RNG
U+541C.0	fFourCorner	67015
U+541C.0	fCantonese	cau2
U+541C.0	fMandarin	chou3
U+541D.0	fUTF8	吝
U+541D.0	fCNS11643	1-493D
U+541D.0	fBig5	A75B
U+541D.0	fGB	0-415F
U+541D.0	fR/S	 30.4
U+541D.0	fTotalStrokes	 7
U+541D.0	fCangjie	YKR
U+541D.0	fFourCorner	00604
U+541D.0	fHakka	 lin5;  nin5; lin3; lin6; nin6
U+541D.0	fCantonese	leon6
U+541D.0	fMandarin	lin4
U+541D.0	fEnglish	stingy; parsimonious; niggardly
U+541E.0	fUTF8	吞
U+541E.0	fCNS11643	1-493F
U+541E.0	fBig5	A75D
U+541E.0	fGB	0-4D4C
U+541E.0	fR/S	 30.4
U+541E.0	fTotalStrokes	 7
U+541E.0	fCangjie	MKR
U+541E.0	fFourCorner	20603
U+541E.0	fHakka	 tun1
U+541E.0	fCantonese	tan1
U+541E.0	fMandarin	tun1
U+541E.0	fEnglish	 swallow; absorb; annex; engulf; gulp
U+541F.0	fUTF8	吟
U+541F.0	fCNS11643	1-4957
U+541F.0	fBig5	A775
U+541F.0	fGB	0-5277
U+541F.0	fR/S	 30.4
U+541F.0	fTotalStrokes	 7
U+541F.0	fCangjie	ROIN
U+541F.0	fFourCorner	68027
U+541F.0	fHakka	  ngim2; gnim3; jim2
U+541F.0	fCantonese	jam4; ngam4 (also); am4 (also)
U+541F.0	fMandarin	yin2
U+541F.0	fEnglish	[1] sing; hum; recite; intone; chant [2] moan; sigh
U+5420.0	fUTF8	吠
U+5420.0	fCNS11643	1-4952
U+5420.0	fBig5	A770
U+5420.0	fGB	0-374D
U+5420.0	fR/S	 30.4
U+5420.0	fTotalStrokes	 7
U+5420.0	fCangjie	RIK
U+5420.0	fFourCorner	63034
U+5420.0	fHakka	 poi5; poi6; poi3; pui6; pi5; pui5
U+5420.0	fCantonese	fai6
U+5420.0	fMandarin	fei4
U+5420.0	fEnglish	 bark (said of a dog)
U+5421.0	fUTF8	吡
U+5421.0	fCNS11643	3-2479
U+5421.0	fGB	0-5F41
U+5421.0	fR/S	  30.4
U+5421.0	fTotalStrokes	 7
U+5421.0	fCangjie	RPP
U+5421.0	fFourCorner	61010
U+5421.0	fCantonese	pei2; bei2; bei1
U+5421.0	fMandarin	bi4; bi3; pi3
U+5421.0	fEnglish	blame
U+5422.0	fUTF8	吢
U+5422.0	fCNS11643	3-247E
U+5422.0	fGB	5-526A
U+5422.0	fR/S	 30.4
U+5422.0	fTotalStrokes	 7
U+5422.0	fCangjie	PR
U+5422.0	fFourCorner	33600
U+5422.0	fCantonese	cam3
U+5422.0	fMandarin	qin4
U+5422.0	fEnglish	vomiting of a dog
U+5423.0	fUTF8	吣
U+5423.0	fCNS11643	3-2475
U+5423.0	fGB	0-5F44
U+5423.0	fR/S	  30.4
U+5423.0	fTotalStrokes	 7
U+5423.0	fCangjie	RP
U+5423.0	fFourCorner	63000
U+5423.0	fHakka	 sim3
U+5423.0	fCantonese	cam3
U+5423.0	fMandarin	qin4
U+5423.0	fEnglish	vomit
U+5424.0	fUTF8	吤
U+5424.0	fCNS11643	2-2345
U+5424.0	fBig5	CAA5
U+5424.0	fGB	5-377C
U+5424.0	fR/S	 30.4
U+5424.0	fTotalStrokes	 7
U+5424.0	fCangjie	ROLL
U+5424.0	fFourCorner	68020
U+5424.0	fCantonese	gaai3; haai6; go3
U+5424.0	fMandarin	jie4; ge1
U+5425.0	fUTF8	吥
U+5425.0	fCNS11643	2-233F
U+5425.0	fBig5	CA7D
U+5425.0	fGB	3-3554
U+5425.0	fR/S	 30.4
U+5425.0	fTotalStrokes	 7
U+5425.0	fCangjie	RMF
U+5425.0	fFourCorner	61090
U+5425.0	fHakka	 pau2; bau1; piau1; feu2
U+5425.0	fCantonese	bat1; pat1; fau4; bau6; pau3
U+5425.0	fMandarin	bu4; pou1; pou5
U+5425.0	fEnglish	 draw in the breath
U+5426.0	fUTF8	否
U+5426.0	fCNS11643	1-4941
U+5426.0	fBig5	A75F
U+5426.0	fGB	0-3771
U+5426.0	fR/S	 30.4
U+5426.0	fTotalStrokes	 7
U+5426.0	fCangjie	MFR
U+5426.0	fFourCorner	10609
U+5426.0	fHakka	  feu3; pi3; fiu3
U+5426.0	fCantonese	fau2; pei2; pei5
U+5426.0	fMandarin	fou3
U+5426.0	fEnglish	[1] not; no [2] negative
U+5426.1	fUTF8	否
U+5426.1	fCNS11643	1-4941
U+5426.1	fBig5	A75F
U+5426.1	fGB	0-3771
U+5426.1	fR/S	 30.4
U+5426.1	fTotalStrokes	 7
U+5426.1	fCangjie	MFR
U+5426.1	fFourCorner	10609
U+5426.1	fMandarin	pi3
U+5426.1	fEnglish	evil; bad
U+5427.0	fUTF8	吧
U+5427.0	fCNS11643	1-4943
U+5427.0	fBig5	A761
U+5427.0	fGB	0-3049
U+5427.0	fR/S	 30.4
U+5427.0	fTotalStrokes	 7
U+5427.0	fCangjie	RAU
U+5427.0	fFourCorner	67017
U+5427.0	fHakka	  ba1; pa3
U+5427.0	fCantonese	baa6; baa1
U+5427.0	fMandarin	ba5; pa1
U+5427.0	fEnglish	a particle used after an imperative sentence
U+5427.1	fUTF8	吧
U+5427.1	fCNS11643	1-4943
U+5427.1	fBig5	A761
U+5427.1	fGB	0-3049
U+5427.1	fR/S	 30.4
U+5427.1	fTotalStrokes	 7
U+5427.1	fCangjie	RAU
U+5427.1	fFourCorner	67017
U+5427.1	fMandarin	ba1
U+5427.1	fEnglish	loquacious
U+5428.0	fUTF8	吨
U+5428.0	fCNS11643	2-2344
U+5428.0	fBig5	CAA4
U+5428.0	fGB	0-3656
U+5428.0	fR/S	 30.4
U+5428.0	fTotalStrokes	 7
U+5428.0	fCangjie	RPU
U+5428.0	fFourCorner	65017
U+5428.0	fHakka	  tun2
U+5428.0	fCantonese	deon1; zeon1
U+5428.0	fMandarin	dun1; dun4
U+5428.0	fEnglish	simplified form of &#x5678;
U+5429.0	fUTF8	吩
U+5429.0	fCNS11643	1-494A
U+5429.0	fBig5	A768
U+5429.0	fGB	0-3754
U+5429.0	fR/S	 30.4
U+5429.0	fTotalStrokes	 7
U+5429.0	fCangjie	RCSH
U+5429.0	fFourCorner	68027
U+5429.0	fHakka	  fun1; pun5
U+5429.0	fCantonese	fan1
U+5429.0	fMandarin	fen1; pen1
U+5429.0	fEnglish	 order; command; instruct; direct
U+542A.0	fUTF8	吪
U+542A.0	fCNS11643	2-233A
U+542A.0	fBig5	CA78
U+542A.0	fGB	3-355B
U+542A.0	fR/S	 30.4
U+542A.0	fTotalStrokes	 7
U+542A.0	fCangjie	ROP
U+542A.0	fFourCorner	61010
U+542A.0	fHakka	  ngo2
U+542A.0	fCantonese	ngo4; faa4 (also); o4
U+542A.0	fMandarin	e2; hua1
U+542A.0	fEnglish	move
U+542B.0	fUTF8	含
U+542B.0	fCNS11643	1-4956
U+542B.0	fBig5	A774
U+542B.0	fGB	0-3A2C
U+542B.0	fR/S	 30.4
U+542B.0	fTotalStrokes	 7
U+542B.0	fCangjie	OINR
U+542B.0	fFourCorner	80602
U+542B.0	fHakka	  ham2; ham3
U+542B.0	fCantonese	ham4; ham3
U+542B.0	fMandarin	han2; han4
U+542B.0	fEnglish	[1] hold in mouth [2] contain; include [3] bear
U+542C.0	fUTF8	听
U+542C.0	fCNS11643	1-4958
U+542C.0	fBig5	A776
U+542C.0	fGB	0-4C7D
U+542C.0	fR/S	 30.4
U+542C.0	fTotalStrokes	 7
U+542C.0	fCangjie	RHML
U+542C.0	fFourCorner	62021
U+542C.0	fHakka	  tang1;  tin1;  tin5; tang5; ten1; niun2
U+542C.0	fCantonese	jan5; ting1; teng1 (also); ting3
U+542C.0	fMandarin	ting1; ting4; yin2
U+542C.0	fEnglish	simplified form of &#x807D;
U+542C.1	fUTF8	听
U+542C.1	fCNS11643	1-4958
U+542C.1	fBig5	A776
U+542C.1	fGB	0-4C7D
U+542C.1	fR/S	 30.4
U+542C.1	fTotalStrokes	 7
U+542C.1	fCangjie	RHML
U+542C.1	fFourCorner	62021
U+542C.1	fMandarin	ting5
U+542D.0	fUTF8	吭
U+542D.0	fCNS11643	1-493E
U+542D.0	fBig5	A75C
U+542D.0	fGB	0-3F54
U+542D.0	fR/S	 30.4
U+542D.0	fTotalStrokes	 7
U+542D.0	fCangjie	RYHN
U+542D.0	fFourCorner	60017
U+542D.0	fHakka	 hong2;  kong5
U+542D.0	fCantonese	hong4; hang1 (also); haang1
U+542D.0	fMandarin	hang2; keng1; hang3; hang4
U+542D.0	fEnglish	throat
U+542E.0	fUTF8	吮
U+542E.0	fCNS11643	1-494F
U+542E.0	fBig5	A76D
U+542E.0	fGB	0-4B31
U+542E.0	fR/S	 30.4
U+542E.0	fTotalStrokes	 7
U+542E.0	fCangjie	RIHU
U+542E.0	fFourCorner	63010
U+542E.0	fHakka	 cion1;  cion3; zun3; con1; zion1
U+542E.0	fCantonese	syun5; seon5; cyun5
U+542E.0	fMandarin	shun3; juan4
U+542E.0	fEnglish	 suck; sip; lick
U+542F.0	fUTF8	启
U+542F.0	fCNS11643	3-2524
U+542F.0	fGB	0-4674
U+542F.0	fR/S	  30.4
U+542F.0	fTotalStrokes	 7
U+542F.0	fCangjie	ISR
U+542F.0	fFourCorner	30260
U+542F.0	fHakka	 ki3
U+542F.0	fCantonese	kai2
U+542F.0	fMandarin	qi3
U+542F.0	fEnglish	simplified form of &#x5553;
U+5430.0	fUTF8	吰
U+5430.0	fCNS11643	2-2338
U+5430.0	fBig5	CA76
U+5430.0	fGB	3-3555
U+5430.0	fR/S	 30.4
U+5430.0	fTotalStrokes	 7
U+5430.0	fCangjie	RKI
U+5430.0	fFourCorner	64032
U+5430.0	fHakka	 fen2; fong2; fung2
U+5430.0	fCantonese	wang4
U+5430.0	fMandarin	hu1; hong2
U+5430.0	fEnglish	clang
U+5431.0	fUTF8	吱
U+5431.0	fCNS11643	1-4955
U+5431.0	fBig5	A773
U+5431.0	fGB	0-5628
U+5431.0	fR/S	 30.4
U+5431.0	fTotalStrokes	 7
U+5431.0	fCangjie	RJE
U+5431.0	fFourCorner	64047
U+5431.0	fHakka	 zi1; zi5; zhi1; zih1
U+5431.0	fCantonese	zi1; gei1
U+5431.0	fMandarin	zhi1; zi1; qi3; qi4
U+5431.0	fEnglish	chirping; squeaking; hissing
U+5432.0	fUTF8	吲
U+5432.0	fCNS11643	3-2477
U+5432.0	fGB	0-5F45
U+5432.0	fR/S	 30.4
U+5432.0	fTotalStrokes	 7
U+5432.0	fCangjie	RNL
U+5432.0	fFourCorner	62000
U+5432.0	fCantonese	jan5; can2
U+5432.0	fMandarin	shen3; yin3
U+5432.0	fEnglish	 smile at sneer at
U+5433.0	fUTF8	吳
U+5433.0	fCNS11643	1-4946
U+5433.0	fBig5	A764
U+5433.0	fGB	E-2439
U+5433.0	fR/S	 30.4
U+5433.0	fTotalStrokes	 7
U+5433.0	fCangjie	RVNK
U+5433.0	fFourCorner	26801
U+5433.0	fHakka	  ng2; ng3
U+5433.0	fCantonese	ng4
U+5433.0	fMandarin	wu2
U+5433.0	fEnglish	[1] the name of a state in the Warring states period [2] the name of a state in the Epoch of the Three Kingdoms
U+5434.0	fUTF8	吴
U+5434.0	fCNS11643	3-247B
U+5434.0	fGB	0-4E62
U+5434.0	fR/S	  30.4
U+5434.0	fTotalStrokes	 7
U+5434.0	fCangjie	RMK
U+5434.0	fFourCorner	60430
U+5434.0	fCantonese	ng4
U+5434.0	fMandarin	wu2
U+5434.0	fEnglish	 name of warring state a Chinese family name
U+5435.0	fUTF8	吵
U+5435.0	fCNS11643	1-4950
U+5435.0	fBig5	A76E
U+5435.0	fGB	0-3333
U+5435.0	fR/S	 30.4
U+5435.0	fTotalStrokes	 7
U+5435.0	fCangjie	RFH
U+5435.0	fFourCorner	69020
U+5435.0	fHakka	 cau3; cau2; zau2
U+5435.0	fCantonese	caau2
U+5435.0	fMandarin	chao3; chao1
U+5435.0	fEnglish	[1] argue; dispute; quarrel; wrangle; row [2] disturb; annoy
U+5436.0	fUTF8	吶
U+5436.0	fCNS11643	1-4951
U+5436.0	fBig5	A76F
U+5436.0	fGB	E-243A
U+5436.0	fR/S	 30.4
U+5436.0	fTotalStrokes	 7
U+5436.0	fCangjie	ROB
U+5436.0	fFourCorner	64027
U+5436.0	fHakka	 nut8; lap8
U+5436.0	fCantonese	naap6; neot6 (also); laap6 (also); nat6; leot6 (also); lat6 (also); naa3; laa3 (also); ne1; le1 (also)
U+5436.0	fMandarin	ne4; na4; ne5
U+5436.0	fEnglish	[1] raise voice; yell out loud; shout [2] speak hesitatingly; stammer
U+5436.1	fUTF8	吶
U+5436.1	fCNS11643	1-4951
U+5436.1	fBig5	A76F
U+5436.1	fGB	E-243A
U+5436.1	fR/S	 30.4
U+5436.1	fTotalStrokes	 7
U+5436.1	fCangjie	ROB
U+5436.1	fFourCorner	64027
U+5436.1	fMandarin	na5
U+5437.0	fUTF8	吷
U+5437.0	fCNS11643	2-2339
U+5437.0	fBig5	CA77
U+5437.0	fGB	3-3563
U+5437.0	fR/S	 30.4
U+5437.0	fTotalStrokes	 7
U+5437.0	fCangjie	RDK
U+5437.0	fFourCorner	65030
U+5437.0	fHakka	 hiet7; giet7
U+5437.0	fCantonese	hyut3; kyut3
U+5437.0	fMandarin	xue4; chuo4; jue2
U+5437.0	fEnglish	[1] sip [2] suck up [3] wheeze
U+5438.0	fUTF8	吸
U+5438.0	fCNS11643	1-494E
U+5438.0	fBig5	A76C
U+5438.0	fGB	0-4E7C
U+5438.0	fR/S	 30.4
U+5438.0	fTotalStrokes	 7
U+5438.0	fCangjie	RNHE
U+5438.0	fFourCorner	67047
U+5438.0	fHakka	  kip7; sip7; gip7
U+5438.0	fCantonese	kap1; ngap1
U+5438.0	fMandarin	xi1; fan2
U+5438.0	fEnglish	 inhale; suck in; absorb; attract; imbibe; draw
U+5439.0	fUTF8	吹
U+5439.0	fCNS11643	1-494C
U+5439.0	fBig5	A76A
U+5439.0	fGB	0-3435
U+5439.0	fR/S	 30.4
U+5439.0	fTotalStrokes	 7
U+5439.0	fCangjie	RNO
U+5439.0	fFourCorner	67082
U+5439.0	fHakka	  cui1;  ch'oi1; ch'ui1; coi1; choi1; chui1
U+5439.0	fCantonese	ceoi1; ceoi3
U+5439.0	fMandarin	chui1; chui4
U+5439.0	fEnglish	[1] blow; puff [2] brag; boast; praise in exaggerated words [3] break up
U+543A.0	fUTF8	吺
U+543A.0	fCNS11643	3-2470
U+543A.0	fGB	3-355F
U+543A.0	fR/S	  30.4
U+543A.0	fTotalStrokes	 7
U+543A.0	fCangjie	RHNE
U+543A.0	fFourCorner	67040
U+543A.0	fHakka	 deu1{te5}; deu1
U+543A.0	fCantonese	dau1
U+543A.0	fMandarin	dou1
U+543A.0	fEnglish	talkative
U+543B.0	fUTF8	吻
U+543B.0	fCNS11643	1-494D
U+543B.0	fBig5	A76B
U+543B.0	fGB	0-4E47
U+543B.0	fR/S	 30.4
U+543B.0	fTotalStrokes	 7
U+543B.0	fCangjie	RPHH
U+543B.0	fFourCorner	67020
U+543B.0	fHakka	  wut7;  wun3; vut7
U+543B.0	fCantonese	man5
U+543B.0	fMandarin	wen3
U+543B.0	fEnglish	[1] kiss [2] lip [3] tone of one's speech
U+543C.0	fUTF8	吼
U+543C.0	fCNS11643	1-4953
U+543C.0	fBig5	A771
U+543C.0	fGB	0-3A70
U+543C.0	fR/S	 30.4
U+543C.0	fTotalStrokes	 7
U+543C.0	fCangjie	RNDU
U+543C.0	fFourCorner	62010
U+543C.0	fHakka	  hau1; k'ieu5; ho5; hau3; keu5; keu3
U+543C.0	fCantonese	hau3; haau1 (also); hau1; hau2
U+543C.0	fMandarin	hou3; hou4
U+543C.0	fEnglish	[1] roar; howl [2] [3] anger [4] [5] promise
U+543D.0	fUTF8	吽
U+543D.0	fCNS11643	2-2341
U+543D.0	fBig5	CAA1
U+543D.0	fGB	3-3559
U+543D.0	fR/S	 30.4
U+543D.0	fTotalStrokes	 7
U+543D.0	fCangjie	RHQ
U+543D.0	fFourCorner	65000
U+543D.0	fHakka	 k'ieu5; jim1; rim1
U+543D.0	fCantonese	hau1 (also); ngau4; au4 (also); hung1; hau2 (also); ngau6; hau3; au6 (also)
U+543D.0	fMandarin	hong1; hou3; ou2; ou1; yin1
U+543D.0	fEnglish	[1] oenomatopoia [2] bellow [3] sick; dull
U+543E.0	fUTF8	吾
U+543E.0	fCNS11643	1-4940
U+543E.0	fBig5	A75E
U+543E.0	fGB	0-4E61
U+543E.0	fR/S	 30.4
U+543E.0	fTotalStrokes	 7
U+543E.0	fCangjie	MMR
U+543E.0	fFourCorner	10601
U+543E.0	fHakka	  ng2; nga2; ng3
U+543E.0	fCantonese	ng4; ngaa4; jyu4
U+543E.0	fMandarin	wu2
U+543E.0	fEnglish	[1] I; me; we; us [2] my; our [3] a family name [4] I [5] I [6] my
U+543E.1	fUTF8	吾
U+543E.1	fCNS11643	1-4940
U+543E.1	fBig5	A75E
U+543E.1	fGB	0-4E61
U+543E.1	fR/S	 30.4
U+543E.1	fTotalStrokes	 7
U+543E.1	fCangjie	MMR
U+543E.1	fFourCorner	10601
U+543E.1	fMandarin	yu2
U+543E.1	fEnglish	[1] interchangeable with &#x865E; [2] interchangeable with &#x9B5A;
U+543E.2	fUTF8	吾
U+543E.2	fCNS11643	1-4940
U+543E.2	fBig5	A75E
U+543E.2	fGB	0-4E61
U+543E.2	fR/S	 30.4
U+543E.2	fTotalStrokes	 7
U+543E.2	fCangjie	MMR
U+543E.2	fFourCorner	10601
U+543E.2	fMandarin	ya2
U+543E.2	fEnglish	used in a place name
U+543F.0	fUTF8	吿
U+543F.0	fCNS11643	3-2521
U+543F.0	fGB	E-243B
U+543F.0	fR/S	 30.4
U+543F.0	fTotalStrokes	 7
U+543F.0	fCangjie	HQR
U+543F.0	fFourCorner	25600
U+543F.0	fCantonese	gou3; guk1
U+543F.0	fMandarin	gao4; gu4
U+543F.0	fEnglish	[1] tell; inform; report; form [2] accuse
U+5440.0	fUTF8	呀
U+5440.0	fCNS11643	1-4954
U+5440.0	fBig5	A772
U+5440.0	fGB	0-513D
U+5440.0	fR/S	 30.4
U+5440.0	fTotalStrokes	 7
U+5440.0	fCangjie	RMVH
U+5440.0	fFourCorner	61040
U+5440.0	fHakka	  nga1;  nga2; a1; ja1; ua1
U+5440.0	fCantonese	aa1; aa3; ngaa1 (also); ngaa3 (also); haa1; aa6
U+5440.0	fMandarin	ya1; xia1; ya2
U+5440.0	fEnglish	 creaking sound
U+5440.1	fUTF8	呀
U+5440.1	fCNS11643	1-4954
U+5440.1	fBig5	A772
U+5440.1	fGB	0-513D
U+5440.1	fR/S	 30.4
U+5440.1	fTotalStrokes	 7
U+5440.1	fCangjie	RMVH
U+5440.1	fFourCorner	61040
U+5440.1	fMandarin	ya5
U+5440.1	fEnglish	[1] a final particle used to express surprise or emphasis [2] gape (as in surprise)
U+5441.0	fUTF8	呁
U+5441.0	fCNS11643	2-2343
U+5441.0	fBig5	CAA3
U+5441.0	fGB	3-355E
U+5441.0	fR/S	 30.4
U+5441.0	fTotalStrokes	 7
U+5441.0	fCangjie	RPMM
U+5441.0	fFourCorner	67020
U+5441.0	fHakka	 giun1; giun3
U+5441.0	fCantonese	gwan3; gan3; gwan2
U+5441.0	fMandarin	jun4
U+5441.0	fEnglish	 vomit
U+5442.0	fUTF8	呂
U+5442.0	fCNS11643	1-4948
U+5442.0	fBig5	A766
U+5442.0	fGB	E-243C
U+5442.0	fR/S	 30.4
U+5442.0	fTotalStrokes	 7
U+5442.0	fCangjie	RHR
U+5442.0	fFourCorner	60600
U+5442.0	fHakka	  li1; li3
U+5442.0	fCantonese	leoi5
U+5442.0	fMandarin	l3; luu3; lu3
U+5442.0	fEnglish	a Chinese family name
U+5443.0	fUTF8	呃
U+5443.0	fCNS11643	1-4945
U+5443.0	fBig5	A763
U+5443.0	fGB	0-5F40
U+5443.0	fR/S	 30.4
U+5443.0	fTotalStrokes	 7
U+5443.0	fCangjie	RMSU
U+5443.0	fFourCorner	61012
U+5443.0	fHakka	 ak7;  et8; ai5; ngak8; et7
U+5443.0	fCantonese	ak1; ngak1 (also); aak1; ngaak1 (also); aai3; ngaai3 (also)
U+5443.0	fMandarin	e4; e5
U+5443.0	fEnglish	[1] belch; hiccup [2] a stuttering tone of voice [3] chirp; cackle [4] cheat; deceive; tease [5] [final particle] expresses idea that a suggestion, offer or invitation is acceptable, or that an accusation is denied
U+5444.0	fUTF8	呄
U+5444.0	fCNS11643	3-2473
U+5444.0	fGB	E-243D
U+5444.0	fR/S	  30.4
U+5444.0	fTotalStrokes	 7
U+5444.0	fCangjie	NSJR
U+5444.0	fFourCorner	17600
U+5444.0	fCantonese	gak1
U+5444.0	fMandarin	ge2
U+5445.0	fUTF8	呅
U+5445.0	fCNS11643	2-233C
U+5445.0	fBig5	CA7A
U+5445.0	fGB	3-3560
U+5445.0	fR/S	 30.4
U+5445.0	fTotalStrokes	 7
U+5445.0	fCangjie	RYK
U+5445.0	fFourCorner	60040
U+5445.0	fCantonese	mui4; man5; man1
U+5445.0	fMandarin	mei2
U+5445.0	fEnglish	a dollar
U+5446.0	fUTF8	呆
U+5446.0	fCNS11643	1-4944
U+5446.0	fBig5	A762
U+5446.0	fGB	0-3474
U+5446.0	fR/S	 30.4
U+5446.0	fTotalStrokes	 7
U+5446.0	fCangjie	RD
U+5446.0	fFourCorner	60904
U+5446.0	fHakka	  ngoi2; ngo5; nga3; nga5; ngoe1; ngoe2
U+5446.0	fCantonese	ngoi4; daai1; oi4 (also)
U+5446.0	fMandarin	dai1; ai2; bao3
U+5446.0	fEnglish	[1] dull; dull-witted; stupid; unintelligent [2] blank; wooden
U+5447.0	fUTF8	呇
U+5447.0	fCNS11643	2-2346
U+5447.0	fBig5	CAA6
U+5447.0	fGB	E-243E
U+5447.0	fR/S	 30.4
U+5447.0	fTotalStrokes	 7
U+5447.0	fCangjie	ER
U+5447.0	fFourCorner	12603
U+5447.0	fCantonese	kai2; hei2
U+5447.0	fMandarin	qi3
U+5447.0	fEnglish	star
U+5448.0	fUTF8	呈
U+5448.0	fCNS11643	1-4947
U+5448.0	fBig5	A765
U+5448.0	fGB	0-334A
U+5448.0	fR/S	 30.4
U+5448.0	fTotalStrokes	 7
U+5448.0	fCangjie	RHG
U+5448.0	fFourCorner	60104
U+5448.0	fHakka	 cin2;  chin2; ch'ang2; cang2; chin3; cihn2
U+5448.0	fCantonese	cing4
U+5448.0	fMandarin	cheng2
U+5448.0	fEnglish	[1] submit; present; hand in (to a superior) [2] show manifest; expose; show; appear; disclose; exhibit [3] petition; appeal
U+5449.0	fUTF8	呉
U+5449.0	fCNS11643	3-247C
U+5449.0	fGB	E-243F
U+5449.0	fR/S	 30.4
U+5449.0	fTotalStrokes	 7
U+5449.0	fCangjie	RVNC
U+5449.0	fFourCorner	60800
U+5449.0	fCantonese	ng4
U+5449.0	fMandarin	wu2
U+5449.0	fEnglish	 one of warring states a Chinese family name
U+544A.0	fUTF8	告
U+544A.0	fCNS11643	1-494B
U+544A.0	fBig5	A769
U+544A.0	fGB	0-3866
U+544A.0	fR/S	 30.4
U+544A.0	fTotalStrokes	 7
U+544A.0	fCangjie	HGR
U+544A.0	fFourCorner	24601
U+544A.0	fHakka	  gau5;  go5; guk7; kwuk7
U+544A.0	fCantonese	gou3; guk1
U+544A.0	fMandarin	gao4; gu4; gu3; ju2
U+544A.0	fEnglish	 tell; announce; inform accuse
U+544B.0	fUTF8	呋
U+544B.0	fCNS11643	3-2478
U+544B.0	fGB	0-5F3B
U+544B.0	fR/S	 30.4
U+544B.0	fTotalStrokes	 7
U+544B.0	fCangjie	RQO
U+544B.0	fFourCorner	35030
U+544B.0	fCantonese	fu1
U+544B.0	fMandarin	fu1
U+544C.0	fUTF8	呌
U+544C.0	fCNS11643	3-2474
U+544C.0	fGB	E-2440
U+544C.0	fR/S	  30.4
U+544C.0	fTotalStrokes	 7
U+544C.0	fCangjie	RYJ
U+544C.0	fFourCorner	64000
U+544C.0	fHakka	 giau5; kieu5
U+544C.0	fCantonese	giu3; kau3
U+544C.0	fMandarin	jiao4
U+544C.0	fEnglish	 call summon
U+544D.0	fUTF8	呍
U+544D.0	fCNS11643	3-246D
U+544D.0	fGB	E-2441
U+544D.0	fR/S	 30.4
U+544D.0	fTotalStrokes	 7
U+544D.0	fCangjie	RMMI
U+544D.0	fFourCorner	61030
U+544D.0	fCantonese	ngau4; au4 (also); wan4
U+544D.0	fMandarin	hong1
U+544D.0	fEnglish	phonetic
U+544E.0	fUTF8	呎
U+544E.0	fCNS11643	1-4942
U+544E.0	fBig5	A760
U+544E.0	fGB	8-2874
U+544E.0	fR/S	 30.4
U+544E.0	fTotalStrokes	 7
U+544E.0	fCangjie	RSO
U+544E.0	fFourCorner	67087
U+544E.0	fHakka	 cak7
U+544E.0	fCantonese	cek3; jing1 (also)
U+544E.0	fMandarin	chi3
U+544E.0	fEnglish	foot
U+544F.0	fUTF8	呏
U+544F.0	fCNS11643	2-2342
U+544F.0	fBig5	CAA2
U+544F.0	fGB	E-2442
U+544F.0	fR/S	 30.4
U+544F.0	fTotalStrokes	 7
U+544F.0	fCangjie	RHT
U+544F.0	fFourCorner	64000
U+544F.0	fCantonese	sing1
U+544F.0	fMandarin	sheng1
U+544F.0	fEnglish	 gallon quart
U+5450.0	fUTF8	呐
U+5450.0	fCNS11643	3-247D
U+5450.0	fGB	0-4445
U+5450.0	fR/S	 30.4
U+5450.0	fTotalStrokes	 7
U+5450.0	fCangjie	ROB
U+5450.0	fFourCorner	64020
U+5450.0	fHakka	 nut8
U+5450.0	fCantonese	naap6; laap6 (also); neot6 (also); leot6 (also); nat6; lat6 (also); naa3; laa3 (also); ne1; le1 (also)
U+5450.0	fMandarin	na4; ne4; na5
U+5450.0	fEnglish	 raise voice; yell out loud; shout stammer
U+5450.1	fUTF8	呐
U+5450.1	fCNS11643	3-247D
U+5450.1	fGB	0-4445
U+5450.1	fR/S	 30.4
U+5450.1	fTotalStrokes	 7
U+5450.1	fCangjie	ROB
U+5450.1	fFourCorner	64020
U+5450.1	fMandarin	ne5
U+5451.0	fUTF8	呑
U+5451.0	fCNS11643	3-2522
U+5451.0	fGB	E-2443
U+5451.0	fR/S	 30.4
U+5451.0	fTotalStrokes	 7
U+5451.0	fCangjie	HKR
U+5451.0	fFourCorner	20600
U+5451.0	fHakka	  tun1
U+5451.0	fCantonese	tan1
U+5451.0	fMandarin	tun1
U+5451.0	fEnglish	 swallow absorb
U+5452.0	fUTF8	呒
U+5452.0	fGB	0-5F3C
U+5452.0	fR/S	 30.4
U+5452.0	fTotalStrokes	 7
U+5452.0	fCangjie	RMKU
U+5452.0	fCantonese	fu2; mou5; m4
U+5452.0	fMandarin	fu3; m1; wu3
U+5452.0	fEnglish	simplified form of &#x5638;
U+5453.0	fUTF8	呓
U+5453.0	fGB	0-5F3D
U+5453.0	fR/S	 30.4
U+5453.0	fTotalStrokes	 7
U+5453.0	fCangjie	RTN
U+5453.0	fHakka	 ngi5
U+5453.0	fCantonese	ngai6; ai6 (also)
U+5453.0	fMandarin	yi4
U+5453.0	fEnglish	simplified form of &#x56C8;
U+5454.0	fUTF8	呔
U+5454.0	fCNS11643	2-233B
U+5454.0	fBig5	CA79
U+5454.0	fGB	0-5F3E
U+5454.0	fR/S	 30.4
U+5454.0	fTotalStrokes	 7
U+5454.0	fCangjie	RKI
U+5454.0	fFourCorner	64030
U+5454.0	fCantonese	taai1; toi2; daai1
U+5454.0	fMandarin	dai1; tai3; tai4; fei4
U+5454.0	fEnglish	 tyre; tube; neck-tie
U+5455.0	fUTF8	呕
U+5455.0	fGB	0-453B
U+5455.0	fR/S	 30.4
U+5455.0	fTotalStrokes	 7
U+5455.0	fCangjie	RSK
U+5455.0	fHakka	 eu3
U+5455.0	fCantonese	au2; ngau2 (also); au1; ngau1 (also); au3; ngau3 (also); heoi1
U+5455.0	fMandarin	ou1; ou3; xu1; ou4; yu4
U+5455.0	fEnglish	simplified form of &#x5614;
U+5456.0	fUTF8	呖
U+5456.0	fGB	0-5F3F
U+5456.0	fR/S	 30.4
U+5456.0	fTotalStrokes	 7
U+5456.0	fCangjie	RMKS
U+5456.0	fHakka	 lit8
U+5456.0	fCantonese	lik6; lik1 (also)
U+5456.0	fMandarin	li4
U+5456.0	fEnglish	simplified form of &#x56A6;
U+5457.0	fUTF8	呗
U+5457.0	fGB	0-5F42
U+5457.0	fR/S	 30.4
U+5457.0	fTotalStrokes	 7
U+5457.0	fCangjie	XRBO
U+5457.0	fHakka	 pai5
U+5457.0	fCantonese	baai6; be6; bui6 (also)
U+5457.0	fMandarin	bai4; bei5
U+5457.0	fEnglish	simplified form of &#x5504;
U+5458.0	fUTF8	员
U+5458.0	fGB	0-5431
U+5458.0	fR/S	 30.4
U+5458.0	fTotalStrokes	 7
U+5458.0	fCangjie	RBO
U+5458.0	fHakka	 jen2
U+5458.0	fCantonese	jyun4; jyun2 (also); wan6; wan4
U+5458.0	fMandarin	yuan2; yun4; yun2
U+5458.0	fEnglish	simplified form of &#x54E1;
U+5459.0	fUTF8	呙
U+5459.0	fGB	0-5F43
U+5459.0	fR/S	 30.4
U+5459.0	fTotalStrokes	 7
U+5459.0	fCangjie	XXROB
U+5459.0	fHakka	 vo1; va2
U+5459.0	fCantonese	waa1; gwo1; go1 (also); gwaa3
U+5459.0	fMandarin	kuai1; guo1; wai1
U+5459.0	fEnglish	simplified form of &#x54BC;
U+545A.0	fUTF8	呚
U+545A.0	fCNS11643	3-2523
U+545A.0	fGB	E-2444
U+545A.0	fR/S	 30.4
U+545A.0	fTotalStrokes	 7
U+545A.0	fCangjie	ROK
U+545A.0	fFourCorner	68040
U+545A.0	fCantonese	mui4
U+545A.0	fMandarin	qi4
U+545B.0	fUTF8	呛
U+545B.0	fGB	0-473A
U+545B.0	fR/S	 30.4
U+545B.0	fTotalStrokes	 7
U+545B.0	fCangjie	ROSU
U+545B.0	fHakka	 cong1
U+545B.0	fCantonese	coeng1; coeng3; coeg3
U+545B.0	fMandarin	qiang1; cang1; qiang4
U+545B.0	fEnglish	simplified form of &#x55C6;
U+545C.0	fUTF8	呜
U+545C.0	fGB	0-4E58
U+545C.0	fR/S	 30.4
U+545C.0	fTotalStrokes	 7
U+545C.0	fCangjie	RPVM
U+545C.0	fHakka	 vu1
U+545C.0	fCantonese	wu1
U+545C.0	fMandarin	wu1; wu4
U+545C.0	fEnglish	simplified form of &#x55DA;
U+545D.0	fUTF8	呝
U+545D.0	fCNS11643	4-2564
U+545D.0	fGB	5-3831
U+545D.0	fR/S	  30.5
U+545D.0	fTotalStrokes	 8; 7; 6
U+545D.0	fCangjie	RISN
U+545D.0	fFourCorner	63010
U+545D.0	fHakka	  ak7; ai3
U+545D.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+545D.0	fMandarin	sao4; e4; ai4
U+545D.0	fEnglish	the cry of birds
U+545E.0	fUTF8	呞
U+545E.0	fCNS11643	4-2563
U+545E.0	fGB	3-3575
U+545E.0	fR/S	 30.5
U+545E.0	fTotalStrokes	 8; 7
U+545E.0	fCangjie	RSMR
U+545E.0	fFourCorner	67020
U+545E.0	fHakka	 chi1; shi1; sii1; cih1; chii1
U+545E.0	fCantonese	si1; ci1
U+545E.0	fMandarin	shi1; chi1
U+545E.0	fEnglish	 chew the cud
U+545F.0	fUTF8	呟
U+545F.0	fCNS11643	3-276E
U+545F.0	fGB	E-2445
U+545F.0	fR/S	 30.5
U+545F.0	fTotalStrokes	 8
U+545F.0	fCangjie	RYVI
U+545F.0	fFourCorner	60030
U+545F.0	fCantonese	gyun2
U+545F.0	fMandarin	quan3
U+545F.0	fEnglish	to mutter, grumble, murmur
U+5460.0	fUTF8	呠
U+5460.0	fCNS11643	2-256B
U+5460.0	fBig5	CBEB
U+5460.0	fGB	5-3827
U+5460.0	fR/S	 30.5
U+5460.0	fTotalStrokes	 8
U+5460.0	fCangjie	RDM
U+5460.0	fFourCorner	65030
U+5460.0	fHakka	  pun5
U+5460.0	fCantonese	pan3
U+5460.0	fMandarin	pen3; pen1
U+5460.0	fEnglish	 spurt blow out puff out snort
U+5461.0	fUTF8	呡
U+5461.0	fCNS11643	2-256A
U+5461.0	fBig5	CBEA
U+5461.0	fGB	5-3833
U+5461.0	fR/S	 30.5
U+5461.0	fTotalStrokes	 8
U+5461.0	fCangjie	RRVP
U+5461.0	fFourCorner	67047
U+5461.0	fHakka	 wut7; vut7
U+5461.0	fCantonese	man5; man4; man3
U+5461.0	fMandarin	wen3
U+5461.0	fEnglish	the corners of the mouth the lips
U+5462.0	fUTF8	呢
U+5462.0	fCNS11643	1-4C51
U+5462.0	fBig5	A94F
U+5462.0	fGB	0-4458
U+5462.0	fR/S	 30.5
U+5462.0	fTotalStrokes	 8
U+5462.0	fCangjie	RSP
U+5462.0	fFourCorner	67011
U+5462.0	fHakka	  ni2; li1; ne2; ni3; ni5; ne5
U+5462.0	fCantonese	nei4; ne1; lei4 (also); nai4 (also); lai4 (also); ni1; nei1 (also); le1 (also); li1 (also); lei1 (also)
U+5462.0	fMandarin	ni2; ne1; ni3
U+5462.0	fEnglish	[1] woolen fabric [2] murmur [3] this
U+5462.1	fUTF8	呢
U+5462.1	fCNS11643	1-4C51
U+5462.1	fBig5	A94F
U+5462.1	fGB	0-4458
U+5462.1	fR/S	 30.5
U+5462.1	fTotalStrokes	 8
U+5462.1	fCangjie	RSP
U+5462.1	fFourCorner	67011
U+5462.1	fMandarin	ne5
U+5462.1	fEnglish	an interrogative or emphatic particle used as final
U+5463.0	fUTF8	呣
U+5463.0	fCNS11643	2-256D
U+5463.0	fBig5	CBED
U+5463.0	fGB	3-3578
U+5463.0	fR/S	 30.5
U+5463.0	fTotalStrokes	 8
U+5463.0	fCangjie	RWYI
U+5463.0	fFourCorner	67050
U+5463.0	fCantonese	ngm2 (also); ngm6 (also); m2; m6; m4
U+5463.0	fMandarin	mou2
U+5464.0	fUTF8	呤
U+5464.0	fCNS11643	2-256F
U+5464.0	fBig5	CBEF
U+5464.0	fGB	0-5F4A
U+5464.0	fR/S	 30.5
U+5464.0	fTotalStrokes	 8
U+5464.0	fCangjie	ROII
U+5464.0	fFourCorner	68037
U+5464.0	fCantonese	ling4; ling6; ling5
U+5464.0	fMandarin	ling2; ling4; ling3
U+5464.0	fEnglish	purine
U+5465.0	fUTF8	呥
U+5465.0	fCNS11643	2-2564
U+5465.0	fBig5	CBE4
U+5465.0	fGB	3-356F
U+5465.0	fR/S	 30.5
U+5465.0	fTotalStrokes	 8
U+5465.0	fCangjie	RGB
U+5465.0	fFourCorner	65047
U+5465.0	fCantonese	jim4; jam1 (also)
U+5465.0	fMandarin	ran3; ran2
U+5465.0	fEnglish	intensifying particle
U+5466.0	fUTF8	呦
U+5466.0	fCNS11643	2-2567
U+5466.0	fBig5	CBE7
U+5466.0	fGB	0-5F4F
U+5466.0	fR/S	 30.5
U+5466.0	fTotalStrokes	 8
U+5466.0	fCangjie	RVIS
U+5466.0	fFourCorner	64027
U+5466.0	fHakka	  ju2; riu2; jiu2
U+5466.0	fCantonese	jau1
U+5466.0	fMandarin	you1; yao4; you3
U+5466.0	fEnglish	the bleating of the deer
U+5467.0	fUTF8	呧
U+5467.0	fCNS11643	2-256E
U+5467.0	fBig5	CBEE
U+5467.0	fGB	5-382D
U+5467.0	fR/S	 30.5
U+5467.0	fTotalStrokes	 8
U+5467.0	fCangjie	RHPM
U+5467.0	fFourCorner	62040
U+5467.0	fHakka	 di3
U+5467.0	fCantonese	dai2
U+5467.0	fMandarin	di3
U+5467.0	fEnglish	 rebuke scold slander
U+5468.0	fUTF8	周
U+5468.0	fCNS11643	1-4C52
U+5468.0	fBig5	A950
U+5468.0	fGB	0-565C
U+5468.0	fR/S	 30.5
U+5468.0	fTotalStrokes	 8
U+5468.0	fCangjie	BGR
U+5468.0	fFourCorner	77220
U+5468.0	fHakka	 zu1; zhiu1;  ziu1; zhu1
U+5468.0	fCantonese	zau1
U+5468.0	fMandarin	zhou1
U+5468.0	fEnglish	[1] Chou dynasty [2] circumference; circuit
U+5469.0	fUTF8	呩
U+5469.0	fCNS11643	3-2774
U+5469.0	fGB	5-3824
U+5469.0	fR/S	 30.5
U+5469.0	fTotalStrokes	 8
U+5469.0	fCangjie	RMMF
U+5469.0	fFourCorner	61090
U+5469.0	fHakka	 she5
U+5469.0	fCantonese	si3
U+5469.0	fMandarin	shi4
U+546A.0	fUTF8	呪
U+546A.0	fCNS11643	3-2778
U+546A.0	fGB	E-2446
U+546A.0	fR/S	  30.5
U+546A.0	fTotalStrokes	 8
U+546A.0	fCangjie	RRHU
U+546A.0	fFourCorner	66010
U+546A.0	fHakka	 zu5; ziu5; zhu5; zhiu5
U+546A.0	fCantonese	zau3; zau2
U+546A.0	fMandarin	zhou4
U+546A.0	fEnglish	curse; damn; incontation
U+546B.0	fUTF8	呫
U+546B.0	fCNS11643	2-2561
U+546B.0	fBig5	CBE1
U+546B.0	fGB	3-3569
U+546B.0	fR/S	 30.5
U+546B.0	fTotalStrokes	 8
U+546B.0	fCangjie	RYR
U+546B.0	fFourCorner	61060
U+546B.0	fHakka	 tiap7;  tiam1
U+546B.0	fCantonese	tip3; cip3; zim1
U+546B.0	fMandarin	che4; tie1; zhan1; tian1
U+546B.0	fEnglish	 whisper lick; taste petty
U+546C.0	fUTF8	呬
U+546C.0	fCNS11643	2-2565
U+546C.0	fBig5	CBE5
U+546C.0	fGB	3-3570
U+546C.0	fR/S	 30.5
U+546C.0	fTotalStrokes	 8
U+546C.0	fCangjie	RWC
U+546C.0	fFourCorner	66000
U+546C.0	fHakka	  si5
U+546C.0	fCantonese	hei3
U+546C.0	fMandarin	ling2; xi4
U+546C.0	fEnglish	 rest
U+546D.0	fUTF8	呭
U+546D.0	fCNS11643	3-2775
U+546D.0	fGB	5-3826
U+546D.0	fR/S	  30.5
U+546D.0	fTotalStrokes	 8
U+546D.0	fCangjie	RPT
U+546D.0	fFourCorner	64010
U+546D.0	fHakka	 siet7; ri3; ji3; ji5; i5
U+546D.0	fCantonese	jai6
U+546D.0	fMandarin	yi4
U+546D.0	fEnglish	 final particle expressing consent talkative
U+546E.0	fUTF8	呮
U+546E.0	fCNS11643	4-255F
U+546E.0	fGB	E-2447
U+546E.0	fR/S	 30.5
U+546E.0	fTotalStrokes	 8; 7
U+546E.0	fCangjie	RRC
U+546E.0	fFourCorner	66080
U+546E.0	fCantonese	kei4; ze1; zek1
U+546E.0	fMandarin	qi4
U+546E.0	fEnglish	to sit
U+546F.0	fUTF8	呯
U+546F.0	fCNS11643	2-2569
U+546F.0	fBig5	CBE9
U+546F.0	fGB	5-382A
U+546F.0	fR/S	 30.5
U+546F.0	fTotalStrokes	 8
U+546F.0	fCangjie	RMFJ
U+546F.0	fFourCorner	61049
U+546F.0	fCantonese	ping4; peng4
U+546F.0	fMandarin	ping2
U+546F.0	fEnglish	onomatopoetic, the sound of a gong
U+5470.0	fUTF8	呰
U+5470.0	fCNS11643	2-294A
U+5470.0	fBig5	CE49
U+5470.0	fGB	3-3624
U+5470.0	fR/S	 30.5
U+5470.0	fTotalStrokes	 8
U+5470.0	fCangjie	YPR
U+5470.0	fFourCorner	21601
U+5470.0	fHakka	 zih1; zii3; zih3
U+5470.0	fCantonese	zi2
U+5470.0	fMandarin	zi3
U+5470.0	fEnglish	 slander
U+5471.0	fUTF8	呱
U+5471.0	fCNS11643	1-4C4D
U+5471.0	fBig5	A94B
U+5471.0	fGB	0-5F49
U+5471.0	fR/S	 30.5
U+5471.0	fTotalStrokes	 8
U+5471.0	fCangjie	RHVO
U+5471.0	fFourCorner	62030
U+5471.0	fHakka	 gu1;  wa1; gwa1; kwak8; ga1; va1; gu2; wa3
U+5471.0	fCantonese	gu1; gwaa1; gwu1 (also); gwaa2; waa1
U+5471.0	fMandarin	gu1; gua1; wa1; gua3
U+5471.0	fEnglish	[1] wail [2] crying of child; cries of an infant
U+5472.0	fUTF8	呲
U+5472.0	fCNS11643	2-294E
U+5472.0	fBig5	CE4D
U+5472.0	fGB	0-5F5A
U+5472.0	fR/S	 30.5
U+5472.0	fTotalStrokes	 8
U+5472.0	fCangjie	RYMP
U+5472.0	fFourCorner	61010
U+5472.0	fCantonese	zi1; ci1 (also)
U+5472.0	fMandarin	zi1; ci2; ci1; yi2
U+5473.0	fUTF8	味
U+5473.0	fCNS11643	1-4C40
U+5473.0	fBig5	A8FD
U+5473.0	fGB	0-4E36
U+5473.0	fR/S	 30.5
U+5473.0	fTotalStrokes	 8
U+5473.0	fCangjie	RJD
U+5473.0	fFourCorner	65090
U+5473.0	fHakka	 mui5; mui6; mui3; mi5
U+5473.0	fCantonese	mei6; mei1 (also); mei2 (also)
U+5473.0	fMandarin	wei4; mai4; mei4; mo4
U+5473.0	fEnglish	[1] [ taste; flavor [2] smell; odor [3] delicacy; dainty
U+5474.0	fUTF8	呴
U+5474.0	fCNS11643	2-2566
U+5474.0	fBig5	CBE6
U+5474.0	fGB	3-3572
U+5474.0	fR/S	 30.5
U+5474.0	fTotalStrokes	 8
U+5474.0	fCangjie	RPR
U+5474.0	fFourCorner	67020
U+5474.0	fHakka	  hi1; kieu1; geu1
U+5474.0	fCantonese	heoi1 (also); hau3; heoi3; heoi2 (also); hau1; jyu3
U+5474.0	fMandarin	xu1; hou1; gou4; hou3; xu3; xu4
U+5474.0	fEnglish	 breathe on yawn roar
U+5475.0	fUTF8	呵
U+5475.0	fCNS11643	1-4C41
U+5475.0	fBig5	A8FE
U+5475.0	fGB	0-3A47
U+5475.0	fR/S	 30.5
U+5475.0	fTotalStrokes	 8
U+5475.0	fCangjie	RMNR
U+5475.0	fFourCorner	61020
U+5475.0	fHakka	 ho1; o1; ho5; ha1; ko1; ha5
U+5475.0	fCantonese	ho1; ho2 (also); aa3; ngaa3 (also); o1; ou1
U+5475.0	fMandarin	he1; ke1; a1; a2; a3; a4; a5; he2; wo2
U+5475.0	fEnglish	 scold in a loud voice; yawn
U+5475.1	fUTF8	呵
U+5475.1	fCNS11643	1-4C41
U+5475.1	fBig5	A8FE
U+5475.1	fGB	0-3A47
U+5475.1	fR/S	 30.5
U+5475.1	fTotalStrokes	 8
U+5475.1	fCangjie	RMNR
U+5475.1	fFourCorner	61020
U+5475.1	fMandarin	o5
U+5475.1	fEnglish	a particle used after a phrase to express surprise
U+5476.0	fUTF8	呶
U+5476.0	fCNS11643	1-4C4E
U+5476.0	fBig5	A94C
U+5476.0	fGB	0-5F4E
U+5476.0	fR/S	 30.5
U+5476.0	fTotalStrokes	 8
U+5476.0	fCangjie	RVE
U+5476.0	fFourCorner	67040
U+5476.0	fHakka	  nau2;  nu2; lu2; lau2
U+5476.0	fCantonese	naau4; laau4 (also); nou4
U+5476.0	fMandarin	nao2; na2
U+5476.0	fEnglish	 talkative clamour hubbub
U+5477.0	fUTF8	呷
U+5477.0	fCNS11643	1-4C47
U+5477.0	fBig5	A945
U+5477.0	fGB	0-5F48
U+5477.0	fR/S	 30.5
U+5477.0	fTotalStrokes	 8
U+5477.0	fCangjie	RWL
U+5477.0	fFourCorner	66050
U+5477.0	fHakka	 ap7; hap7; gap7; kap8; kiap7; gat7
U+5477.0	fCantonese	haap3; gaat3
U+5477.0	fMandarin	xia1; ga1; xia2
U+5477.0	fEnglish	[1] suck, swallow, drink [2] [Cantonese sip
U+5478.0	fUTF8	呸
U+5478.0	fCNS11643	1-4C43
U+5478.0	fBig5	A941
U+5478.0	fGB	0-455E
U+5478.0	fR/S	 30.5
U+5478.0	fTotalStrokes	 8
U+5478.0	fCangjie	RMFM
U+5478.0	fFourCorner	61019
U+5478.0	fHakka	 pi1; pui1; pi5; poi5
U+5478.0	fCantonese	pei1
U+5478.0	fMandarin	pei1; pi1
U+5478.0	fEnglish	expression of reprimand
U+5479.0	fUTF8	呹
U+5479.0	fCNS11643	3-2771
U+5479.0	fGB	3-3571
U+5479.0	fR/S	  30.5
U+5479.0	fTotalStrokes	 8
U+5479.0	fCangjie	RHQO
U+5479.0	fFourCorner	65030
U+5479.0	fCantonese	jat6; cik1
U+5479.0	fMandarin	yi4
U+5479.0	fEnglish	(of animals) to eat grass
U+547A.0	fUTF8	呺
U+547A.0	fCNS11643	2-2562
U+547A.0	fBig5	CBE2
U+547A.0	fGB	3-356D
U+547A.0	fR/S	 30.5
U+547A.0	fTotalStrokes	 8
U+547A.0	fCangjie	RRMS
U+547A.0	fFourCorner	66027
U+547A.0	fHakka	  hiau1; hau2; hieu1; hau5
U+547A.0	fCantonese	hiu1; hou4
U+547A.0	fMandarin	xiao1; hao2; hao4; hou3
U+547A.0	fEnglish	voice of anger vast; spacious
U+547B.0	fUTF8	呻
U+547B.0	fCNS11643	1-4C46
U+547B.0	fBig5	A944
U+547B.0	fGB	0-496B
U+547B.0	fR/S	 30.5
U+547B.0	fTotalStrokes	 8
U+547B.0	fCangjie	RLWL
U+547B.0	fFourCorner	65006
U+547B.0	fHakka	  shin1; siin1; sin1; cen1
U+547B.0	fCantonese	san1
U+547B.0	fMandarin	shen1
U+547B.0	fEnglish	 groan; moan
U+547C.0	fUTF8	呼
U+547C.0	fCNS11643	1-4C4B
U+547C.0	fBig5	A949
U+547C.0	fGB	0-3A74
U+547C.0	fR/S	 30.5
U+547C.0	fTotalStrokes	 8
U+547C.0	fCangjie	RHFD
U+547C.0	fFourCorner	62049
U+547C.0	fHakka	  fu1
U+547C.0	fCantonese	fu1
U+547C.0	fMandarin	hu1; he4; xu1; hu4; xia4; xiao1
U+547C.0	fEnglish	breathe sigh; exhale; call; shout
U+547C.1	fUTF8	呼
U+547C.1	fCNS11643	1-4C4B
U+547C.1	fBig5	A949
U+547C.1	fGB	0-3A74
U+547C.1	fR/S	 30.5
U+547C.1	fTotalStrokes	 8
U+547C.1	fCangjie	RHFD
U+547C.1	fFourCorner	62049
U+547C.1	fMandarin	hu5
U+547D.0	fUTF8	命
U+547D.0	fCNS11643	1-4C54
U+547D.0	fBig5	A952
U+547D.0	fGB	0-437C
U+547D.0	fR/S	 30.5
U+547D.0	fTotalStrokes	 8
U+547D.0	fCangjie	OMRL
U+547D.0	fFourCorner	80627
U+547D.0	fHakka	  min5; miang5; min6; miang3; miang6; min3
U+547D.0	fCantonese	ming6; meng6 (also); meng2 (also)
U+547D.0	fMandarin	ming4; man4
U+547D.0	fEnglish	[1] life [2] destiny; fate; lot [3] the ordinances of Heaven [4] orders; command
U+547E.0	fUTF8	呾
U+547E.0	fCNS11643	2-2563
U+547E.0	fBig5	CBE3
U+547E.0	fGB	3-356C
U+547E.0	fR/S	 30.5
U+547E.0	fTotalStrokes	 8
U+547E.0	fCangjie	RAM
U+547E.0	fFourCorner	66010
U+547E.0	fHakka	  dat7;  dan5; da5
U+547E.0	fCantonese	daat3; taan2
U+547E.0	fMandarin	da2; ta4
U+547E.0	fEnglish	[1] call out [2] hum
U+547F.0	fUTF8	呿
U+547F.0	fCNS11643	2-255C
U+547F.0	fBig5	CBDC
U+547F.0	fGB	3-3564
U+547F.0	fR/S	 30.5
U+547F.0	fTotalStrokes	 8
U+547F.0	fCangjie	RGI
U+547F.0	fFourCorner	64031
U+547F.0	fHakka	  kiap7; ki5; ki1; hi1
U+547F.0	fCantonese	keoi1; heoi3
U+547F.0	fMandarin	qu1; qie4; qu4
U+547F.0	fEnglish	 yawn
U+5480.0	fUTF8	咀
U+5480.0	fCNS11643	1-4C45
U+5480.0	fBig5	A943
U+5480.0	fGB	0-3E57
U+5480.0	fR/S	 30.5
U+5480.0	fTotalStrokes	 8
U+5480.0	fCangjie	RBM
U+5480.0	fFourCorner	67010
U+5480.0	fHakka	 zu3; zui3; zih1
U+5480.0	fCantonese	zeoi2; zo3
U+5480.0	fMandarin	ju3; ju1; zu3; zui3
U+5480.0	fEnglish	 chew; masticate
U+5481.0	fUTF8	咁
U+5481.0	fCNS11643	2-255D
U+5481.0	fBig5	CBDD
U+5481.0	fGB	3-3565
U+5481.0	fR/S	 30.5
U+5481.0	fTotalStrokes	 8
U+5481.0	fCangjie	RTM
U+5481.0	fFourCorner	64070
U+5481.0	fHakka	 an2; gam1; an5; gan5; an3; gan3; gan2
U+5481.0	fCantonese	gam2; gam3
U+5481.0	fMandarin	gan1; xian2
U+5481.0	fEnglish	[1] so [2] so (used only to modify adjective or adverbs, never verbs)
U+5482.0	fUTF8	咂
U+5482.0	fCNS11643	2-255F
U+5482.0	fBig5	CBDF
U+5482.0	fGB	0-5F46
U+5482.0	fR/S	 30.5
U+5482.0	fTotalStrokes	 8
U+5482.0	fCangjie	RSLB
U+5482.0	fFourCorner	61012
U+5482.0	fHakka	 zap7;  sap7
U+5482.0	fCantonese	zaap3; sap1
U+5482.0	fMandarin	za1
U+5482.0	fEnglish	 suck; sip; take in food with the tongue
U+5483.0	fUTF8	咃
U+5483.0	fCNS11643	4-2560
U+5483.0	fGB	E-2448
U+5483.0	fR/S	  30.5
U+5483.0	fTotalStrokes	 8; 7
U+5483.0	fCangjie	ROPD
U+5483.0	fFourCorner	67010
U+5483.0	fCantonese	to1
U+5483.0	fMandarin	tuo1
U+5484.0	fUTF8	咄
U+5484.0	fCNS11643	1-4C48
U+5484.0	fBig5	A946
U+5484.0	fGB	0-5F4D
U+5484.0	fR/S	 30.5
U+5484.0	fTotalStrokes	 8
U+5484.0	fCangjie	RUU
U+5484.0	fFourCorner	62072
U+5484.0	fHakka	 zot7; dot7; doi2; chot7; zot8; tut7
U+5484.0	fCantonese	deot1; cyut3 (also); zyut3 (also); dyut1 (also)
U+5484.0	fMandarin	duo1; duo4
U+5484.0	fEnglish	[1] cry out in anger; an angry cry [2] scold in a loud voice
U+5485.0	fUTF8	咅
U+5485.0	fCNS11643	3-276C
U+5485.0	fGB	5-5D7A
U+5485.0	fR/S	 30.5
U+5485.0	fTotalStrokes	 8
U+5485.0	fCangjie	YTR
U+5485.0	fFourCorner	00600
U+5485.0	fHakka	  teu5
U+5485.0	fCantonese	tau3
U+5485.0	fMandarin	pou3; pou4
U+5485.0	fEnglish	to spit out
U+5486.0	fUTF8	咆
U+5486.0	fCNS11643	1-4C4A
U+5486.0	fBig5	A948
U+5486.0	fGB	0-4558
U+5486.0	fR/S	 30.5
U+5486.0	fTotalStrokes	 8
U+5486.0	fCangjie	RPRU
U+5486.0	fFourCorner	67012
U+5486.0	fHakka	  pau1; pau2
U+5486.0	fCantonese	paau4; paau3
U+5486.0	fMandarin	pao2; bao4
U+5486.0	fEnglish	 roar
U+5487.0	fUTF8	咇
U+5487.0	fCNS11643	2-255B
U+5487.0	fBig5	CBDB
U+5487.0	fGB	5-3832
U+5487.0	fR/S	 30.5
U+5487.0	fTotalStrokes	 8
U+5487.0	fCangjie	RPH
U+5487.0	fFourCorner	63000
U+5487.0	fHakka	  bit7; pi5
U+5487.0	fCantonese	bit1; bit6; bat6 (also); bat1
U+5487.0	fMandarin	bi4; bie2; bi3
U+5487.0	fEnglish	fragrant
U+5488.0	fUTF8	咈
U+5488.0	fCNS11643	2-2560
U+5488.0	fBig5	CBE0
U+5488.0	fGB	3-3576
U+5488.0	fR/S	 30.5
U+5488.0	fTotalStrokes	 8
U+5488.0	fCangjie	RLLN
U+5488.0	fFourCorner	65027
U+5488.0	fHakka	 fut7; fut8
U+5488.0	fCantonese	fat6; fat1
U+5488.0	fMandarin	fu2; bi4
U+5488.0	fEnglish	oppose
U+5489.0	fUTF8	咉
U+5489.0	fCNS11643	3-2770
U+5489.0	fGB	3-356E
U+5489.0	fR/S	  30.5
U+5489.0	fTotalStrokes	 8
U+5489.0	fCangjie	RLBK
U+5489.0	fFourCorner	65030
U+5489.0	fHakka	 jong1; rong1; hiet7
U+5489.0	fCantonese	joeng1
U+5489.0	fMandarin	yang1; yang3
U+5489.0	fEnglish	an echo; a sound to vomit; as an infant
U+548A.0	fUTF8	咊
U+548A.0	fCNS11643	3-2779
U+548A.0	fGB	E-2449
U+548A.0	fR/S	  30.5
U+548A.0	fTotalStrokes	 8
U+548A.0	fCangjie	RHD
U+548A.0	fFourCorner	62090
U+548A.0	fHakka	 fo2; fo5; wo2
U+548A.0	fCantonese	wo4; wo6; wu4; gwo3; go3 (also)
U+548A.0	fMandarin	he2; he4; huo4; huo2
U+548A.0	fEnglish	 harmony peace conciliation be on good terms with kindly mild
U+548B.0	fUTF8	咋
U+548B.0	fCNS11643	1-4C53
U+548B.0	fBig5	A951
U+548B.0	fGB	0-5526
U+548B.0	fR/S	 30.5
U+548B.0	fTotalStrokes	 8
U+548B.0	fCangjie	ROS
U+548B.0	fFourCorner	68011
U+548B.0	fHakka	  zak7; za5
U+548B.0	fCantonese	zaa3; zaa1; zaak3; zaa2 (also); zaa6; zaak6
U+548B.0	fMandarin	zha4; za3; zha1; zuo4
U+548B.0	fEnglish	[1] all of a sudden; suddenly [2] [final particle] contraction of "tse1 a3", used at end of statements with idea of a limitation of some kind [3] [final particle] contraction of "tse1 a4", used at end of questions with a limitation of some some kind
U+548B.1	fUTF8	咋
U+548B.1	fCNS11643	1-4C53
U+548B.1	fBig5	A951
U+548B.1	fGB	0-5526
U+548B.1	fR/S	 30.5
U+548B.1	fTotalStrokes	 8
U+548B.1	fCangjie	ROS
U+548B.1	fFourCorner	68011
U+548B.1	fMandarin	ze2
U+548B.1	fEnglish	[1] bite; gnaw [2] loud noise
U+548C.0	fUTF8	和
U+548C.0	fCNS11643	1-4C4F
U+548C.0	fBig5	A94D
U+548C.0	fGB	0-3A4D
U+548C.0	fR/S	 30.5
U+548C.0	fTotalStrokes	 8
U+548C.0	fCangjie	HDR
U+548C.0	fFourCorner	26900
U+548C.0	fHakka	  fo2;  wo2; fo3; wo1
U+548C.0	fCantonese	wo4; wo6; wu4; wo1 (also); gwo3; go3 (also)
U+548C.0	fMandarin	he2; huo1; huo2
U+548C.0	fEnglish	[1] and [2] harmony; harmonious [3] peace; peaceful; calm [4] be affable [5] sum; aggregate [6] of Japan
U+548C.1	fUTF8	和
U+548C.1	fCNS11643	1-4C4F
U+548C.1	fBig5	A94D
U+548C.1	fGB	0-3A4D
U+548C.1	fR/S	 30.5
U+548C.1	fTotalStrokes	 8
U+548C.1	fCangjie	HDR
U+548C.1	fFourCorner	26900
U+548C.1	fHakka	 fo5
U+548C.1	fMandarin	he4
U+548C.1	fEnglish	[1] match; harmonise [2] write a poem in reply
U+548C.2	fUTF8	和
U+548C.2	fCNS11643	1-4C4F
U+548C.2	fBig5	A94D
U+548C.2	fGB	0-3A4D
U+548C.2	fR/S	 30.5
U+548C.2	fTotalStrokes	 8
U+548C.2	fCangjie	HDR
U+548C.2	fFourCorner	26900
U+548C.2	fMandarin	huo4
U+548C.2	fEnglish	 knead (dough, clay, etc.); mix
U+548C.3	fUTF8	和
U+548C.3	fCNS11643	1-4C4F
U+548C.3	fBig5	A94D
U+548C.3	fGB	0-3A4D
U+548C.3	fR/S	 30.5
U+548C.3	fTotalStrokes	 8
U+548C.3	fCangjie	HDR
U+548C.3	fFourCorner	26900
U+548C.3	fMandarin	huo5
U+548C.4	fUTF8	和
U+548C.4	fCNS11643	1-4C4F
U+548C.4	fBig5	A94D
U+548C.4	fGB	0-3A4D
U+548C.4	fR/S	 30.5
U+548C.4	fTotalStrokes	 8
U+548C.4	fCangjie	HDR
U+548C.4	fFourCorner	26900
U+548C.4	fMandarin	he5
U+548D.0	fUTF8	咍
U+548D.0	fCNS11643	2-2568
U+548D.0	fBig5	CBE8
U+548D.0	fGB	3-3577
U+548D.0	fR/S	 30.5
U+548D.0	fTotalStrokes	 8
U+548D.0	fCangjie	RIR
U+548D.0	fFourCorner	63060
U+548D.0	fHakka	 toi2; ri2; ji2; ji1; i1
U+548D.0	fCantonese	hoi1; haai1
U+548D.0	fMandarin	hai1; tai1
U+548D.0	fEnglish	laugh
U+548E.0	fUTF8	咎
U+548E.0	fCNS11643	1-4C55
U+548E.0	fBig5	A953
U+548E.0	fGB	0-3E4C
U+548E.0	fR/S	 30.5
U+548E.0	fTotalStrokes	 8
U+548E.0	fCangjie	HOR
U+548E.0	fFourCorner	28604
U+548E.0	fHakka	  giu5; kiu5
U+548E.0	fCantonese	gau3; gou1
U+548E.0	fMandarin	jiu4; gao1
U+548E.0	fEnglish	[1] disaster; calamity [2] fault; defect; error; mistake [3] blame; punish; censure
U+548F.0	fUTF8	咏
U+548F.0	fCNS11643	3-2773
U+548F.0	fGB	0-533D
U+548F.0	fR/S	  30.5
U+548F.0	fTotalStrokes	 8
U+548F.0	fCangjie	RINE
U+548F.0	fFourCorner	63030
U+548F.0	fHakka	  jun3; run3; jun6
U+548F.0	fCantonese	wing6
U+548F.0	fMandarin	yong3
U+548F.0	fEnglish	sing song or poem; hum; chant
U+5490.0	fUTF8	咐
U+5490.0	fCNS11643	1-4C4C
U+5490.0	fBig5	A94A
U+5490.0	fGB	0-3840
U+5490.0	fR/S	 30.5
U+5490.0	fTotalStrokes	 8
U+5490.0	fCangjie	RODI
U+5490.0	fFourCorner	64000
U+5490.0	fHakka	  fu5
U+5490.0	fCantonese	fu3; fu6
U+5490.0	fMandarin	fu4; fu1; fu5
U+5490.0	fEnglish	 instruct; order; tell (someone to do something); bid
U+5491.0	fUTF8	咑
U+5491.0	fCNS11643	2-255E
U+5491.0	fBig5	CBDE
U+5491.0	fGB	3-3568
U+5491.0	fR/S	 30.5
U+5491.0	fTotalStrokes	 8
U+5491.0	fCangjie	RQMN
U+5491.0	fFourCorner	61020
U+5491.0	fCantonese	daa1; daak1; dak1
U+5491.0	fMandarin	que4; da1; da5; que1; jue2
U+5491.0	fEnglish	啲咑, a trumpet
U+5492.0	fUTF8	咒
U+5492.0	fCNS11643	1-4C49
U+5492.0	fBig5	A947
U+5492.0	fGB	0-5664
U+5492.0	fR/S	 30.5
U+5492.0	fTotalStrokes	 8
U+5492.0	fCangjie	RRHN
U+5492.0	fFourCorner	66217
U+5492.0	fHakka	  zhiu5; zu5; ziu5; zhu5
U+5492.0	fCantonese	zau3
U+5492.0	fMandarin	zhou4
U+5492.0	fEnglish	[1] curse; damn; swear [2] words used as charms by Buddhist monks or Taoists priests to exorcise ghosts
U+5493.0	fUTF8	咓
U+5493.0	fCNS11643	3-276F
U+5493.0	fGB	E-244A
U+5493.0	fR/S	  30.5
U+5493.0	fTotalStrokes	 8
U+5493.0	fCangjie	RMVN
U+5493.0	fFourCorner	61010
U+5493.0	fCantonese	aa5; ngaa5 (also)
U+5493.0	fMandarin	wa3
U+5493.0	fEnglish	final particle
U+5494.0	fUTF8	咔
U+5494.0	fCNS11643	3-2777
U+5494.0	fGB	0-5F47
U+5494.0	fR/S	 30.5
U+5494.0	fTotalStrokes	 8
U+5494.0	fCangjie	RYMY
U+5494.0	fFourCorner	61030
U+5494.0	fCantonese	kaa1; kaa4; kaa2
U+5494.0	fMandarin	long4; ka3; nong4; ka1
U+5494.0	fEnglish	used in translation
U+5495.0	fUTF8	咕
U+5495.0	fCNS11643	1-4C44
U+5495.0	fBig5	A942
U+5495.0	fGB	0-393E
U+5495.0	fR/S	 30.5
U+5495.0	fTotalStrokes	 8
U+5495.0	fCangjie	RJR
U+5495.0	fFourCorner	64060
U+5495.0	fHakka	  gu3; gu1
U+5495.0	fCantonese	gu1; gwu1 (also); gu2
U+5495.0	fMandarin	gu1; gu3
U+5495.0	fEnglish	[1] mumble; mutter; murmur [2] cluck (said of hens); coo (said of turtledoves, etc.)
U+5496.0	fUTF8	咖
U+5496.0	fCNS11643	1-4C42
U+5496.0	fBig5	A940
U+5496.0	fGB	0-3F27
U+5496.0	fR/S	 30.5
U+5496.0	fTotalStrokes	 8
U+5496.0	fCangjie	RKSR
U+5496.0	fFourCorner	66000
U+5496.0	fHakka	 ga1
U+5496.0	fCantonese	gaa3; kaa1 (also); gaa1
U+5496.0	fMandarin	ka1; jia1
U+5496.0	fEnglish	used in transliteration
U+5496.1	fUTF8	咖
U+5496.1	fCNS11643	1-4C42
U+5496.1	fBig5	A940
U+5496.1	fGB	0-3F27
U+5496.1	fR/S	 30.5
U+5496.1	fTotalStrokes	 8
U+5496.1	fCangjie	RKSR
U+5496.1	fFourCorner	66000
U+5496.1	fMandarin	ga1
U+5496.1	fEnglish	used in transliteration
U+5497.0	fUTF8	咗
U+5497.0	fGB	1-7C25
U+5497.0	fR/S	 30.5
U+5497.0	fTotalStrokes	 8
U+5497.0	fCangjie	RKM
U+5497.0	fCantonese	zo2
U+5497.0	fMandarin	zuo3
U+5497.0	fEnglish	verbal particle of perfective aspect
U+5498.0	fUTF8	咘
U+5498.0	fCNS11643	2-256C
U+5498.0	fBig5	CBEC
U+5498.0	fGB	3-3567
U+5498.0	fR/S	 30.5
U+5498.0	fTotalStrokes	 8
U+5498.0	fCangjie	RKLB
U+5498.0	fFourCorner	64027
U+5498.0	fCantonese	bou3
U+5498.0	fMandarin	bu4
U+5499.0	fUTF8	咙
U+5499.0	fGB	0-417C
U+5499.0	fR/S	 30.5
U+5499.0	fTotalStrokes	 8
U+5499.0	fCangjie	RIKP
U+5499.0	fHakka	 lung2
U+5499.0	fCantonese	lung4
U+5499.0	fMandarin	long2
U+5499.0	fEnglish	simplified form of &#x56A8;
U+549A.0	fUTF8	咚
U+549A.0	fCNS11643	1-4C50
U+549A.0	fBig5	A94E
U+549A.0	fGB	0-5F4B
U+549A.0	fR/S	 30.5
U+549A.0	fTotalStrokes	 8
U+549A.0	fCangjie	RHEY
U+549A.0	fFourCorner	67033
U+549A.0	fHakka	 dung1
U+549A.0	fCantonese	dung1
U+549A.0	fMandarin	dong1
U+549A.0	fEnglish	[1] the sound of impact caused by a falling object [2] rub-a-dub [3] rat-tat; rat-a-tat [4] final particle
U+549B.0	fUTF8	咛
U+549B.0	fGB	0-5F4C
U+549B.0	fR/S	 30.5
U+549B.0	fTotalStrokes	 8
U+549B.0	fCangjie	RJMN
U+549B.0	fHakka	 nen2
U+549B.0	fCantonese	ning4; ling4 (also)
U+549B.0	fMandarin	ning2
U+549B.0	fEnglish	simplified form of &#x5680;
U+549C.0	fUTF8	咜
U+549C.0	fCNS11643	F-2571
U+549C.0	fGB	E-244B
U+549C.0	fR/S	 30.5
U+549C.0	fTotalStrokes	 8
U+549C.0	fCangjie	RJP
U+549C.0	fFourCorner	6301
U+549C.0	fCantonese	taa1
U+549C.0	fMandarin	zha4
U+549C.0	fEnglish	 scold
U+549D.0	fUTF8	咝
U+549D.0	fGB	0-5F50
U+549D.0	fR/S	 30.5
U+549D.0	fTotalStrokes	 8
U+549D.0	fCangjie	RVVM
U+549D.0	fCantonese	si1
U+549D.0	fMandarin	si1
U+549D.0	fEnglish	simplified form of &#x565D;
U+549E.0	fUTF8	咞
U+549E.0	fCNS11643	3-2B5C
U+549E.0	fGB	E-244C
U+549E.0	fR/S	 30.6
U+549E.0	fTotalStrokes	 9
U+549E.0	fCangjie	RMT
U+549E.0	fFourCorner	61040
U+549E.0	fCantonese	jin6
U+549E.0	fMandarin	xian4; xian2
U+549F.0	fUTF8	咟
U+549F.0	fCNS11643	3-2B5D
U+549F.0	fGB	5-3838
U+549F.0	fR/S	  30.6
U+549F.0	fTotalStrokes	 9
U+549F.0	fCangjie	RMA
U+549F.0	fFourCorner	61060
U+549F.0	fCantonese	waak1
U+549F.0	fMandarin	huo4
U+54A0.0	fUTF8	咠
U+54A0.0	fCNS11643	2-2949
U+54A0.0	fBig5	CE48
U+54A0.0	fGB	3-357B
U+54A0.0	fR/S	 30.6
U+54A0.0	fTotalStrokes	 9
U+54A0.0	fCangjie	RSJ
U+54A0.0	fFourCorner	60401
U+54A0.0	fHakka	  sip8; cip8
U+54A0.0	fCantonese	cap1
U+54A0.0	fMandarin	qi4; qi1; ji2; yi1
U+54A0.0	fEnglish	[1] whisper [2] blame to slander
U+54A1.0	fUTF8	咡
U+54A1.0	fCNS11643	2-293D
U+54A1.0	fBig5	CDFB
U+54A1.0	fGB	3-357A
U+54A1.0	fR/S	 30.6
U+54A1.0	fTotalStrokes	 9
U+54A1.0	fCangjie	RSJ
U+54A1.0	fFourCorner	61040
U+54A1.0	fHakka	 ngi3; ngi5
U+54A1.0	fCantonese	ji6; mai1; mai4
U+54A1.0	fMandarin	er4; er2
U+54A1.0	fEnglish	side of mouth
U+54A2.0	fUTF8	咢
U+54A2.0	fCNS11643	2-294C
U+54A2.0	fBig5	CE4B
U+54A2.0	fGB	3-3625
U+54A2.0	fR/S	 30.6
U+54A2.0	fTotalStrokes	 9
U+54A2.0	fCangjie	RRMMS
U+54A2.0	fFourCorner	66207
U+54A2.0	fHakka	 ngok8; hiau1
U+54A2.0	fCantonese	ngok6; ok6 (also)
U+54A2.0	fMandarin	e4
U+54A2.0	fEnglish	[1] sound; drumming [2] perk up the head
U+54A3.0	fUTF8	咣
U+54A3.0	fCNS11643	3-2B57
U+54A3.0	fGB	0-5F5B
U+54A3.0	fR/S	  30.6
U+54A3.0	fTotalStrokes	 9
U+54A3.0	fCangjie	RFMU
U+54A3.0	fFourCorner	69010
U+54A3.0	fCantonese	gwong1; gong1 (also)
U+54A3.0	fMandarin	gong1; guang1
U+54A4.0	fUTF8	咤
U+54A4.0	fCNS11643	3-2B59
U+54A4.0	fGB	0-5F65
U+54A4.0	fR/S	  30.6
U+54A4.0	fTotalStrokes	 9
U+54A4.0	fCangjie	RJHP
U+54A4.0	fFourCorner	63010
U+54A4.0	fHakka	  za5; ca5; za1; cet8
U+54A4.0	fCantonese	zaa3; caak1 (also); caa3; zaa1
U+54A4.0	fMandarin	zha4; cha4; zha1
U+54A4.0	fEnglish	 shout with anger
U+54A5.0	fUTF8	咥
U+54A5.0	fCNS11643	2-293F
U+54A5.0	fBig5	CDFD
U+54A5.0	fGB	3-3622
U+54A5.0	fR/S	 30.6
U+54A5.0	fTotalStrokes	 9
U+54A5.0	fCangjie	RMIG
U+54A5.0	fFourCorner	61014
U+54A5.0	fHakka	  hi5; zhit7; tsiit7
U+54A5.0	fCantonese	hei3; dit6; zat6
U+54A5.0	fMandarin	xi4; die2; xi1; zhi4
U+54A5.0	fEnglish	sound of cat; bite; laugh
U+54A6.0	fUTF8	咦
U+54A6.0	fCNS11643	1-503C
U+54A6.0	fBig5	AB78
U+54A6.0	fGB	0-5F57
U+54A6.0	fR/S	 30.6
U+54A6.0	fTotalStrokes	 9
U+54A6.0	fCangjie	RKN
U+54A6.0	fFourCorner	65032
U+54A6.0	fHakka	 ji2
U+54A6.0	fCantonese	ji4; ji2 (also); hi1
U+54A6.0	fMandarin	yi2; xi1
U+54A6.0	fEnglish	an interjection of surprise; well; why; hey
U+54A7.0	fUTF8	咧
U+54A7.0	fCNS11643	1-504A
U+54A7.0	fBig5	ABA8
U+54A7.0	fGB	0-5F56
U+54A7.0	fR/S	 30.6
U+54A7.0	fTotalStrokes	 9
U+54A7.0	fCangjie	RMNN
U+54A7.0	fFourCorner	62000
U+54A7.0	fHakka	  liet8; let8
U+54A7.0	fCantonese	le2; le1; lit6 (also); le3 (also); le4
U+54A7.0	fMandarin	lie3; lie1
U+54A7.0	fEnglish	 stretch the mouth horizontally
U+54A7.1	fUTF8	咧
U+54A7.1	fCNS11643	1-504A
U+54A7.1	fBig5	ABA8
U+54A7.1	fGB	0-5F56
U+54A7.1	fR/S	 30.6
U+54A7.1	fTotalStrokes	 9
U+54A7.1	fCangjie	RMNN
U+54A7.1	fFourCorner	62000
U+54A7.1	fMandarin	lie2; lie5
U+54A7.1	fEnglish	used in combination
U+54A8.0	fUTF8	咨
U+54A8.0	fCNS11643	1-5038
U+54A8.0	fBig5	AB74
U+54A8.0	fGB	0-5749
U+54A8.0	fR/S	 30.6
U+54A8.0	fTotalStrokes	 9
U+54A8.0	fCangjie	IOR
U+54A8.0	fFourCorner	37608
U+54A8.0	fHakka	 zih1; zu1; zii1
U+54A8.0	fCantonese	zi1
U+54A8.0	fMandarin	zi1; zi4
U+54A8.0	fEnglish	[1] inquire; consult; discuss [2] a very formal official communication between offices of equal rank
U+54A9.0	fUTF8	咩
U+54A9.0	fCNS11643	1-5049
U+54A9.0	fBig5	ABA7
U+54A9.0	fGB	0-5F63
U+54A9.0	fR/S	 30.6
U+54A9.0	fTotalStrokes	 9
U+54A9.0	fCangjie	RTQ
U+54A9.0	fFourCorner	68051
U+54A9.0	fHakka	 me1; mi3; me5
U+54A9.0	fCantonese	me1
U+54A9.0	fMandarin	mie1
U+54A9.0	fEnglish	[1] bleating of sheep [2] [final particle] transforms statements into questions that indicate doubt or surprise
U+54AA.0	fUTF8	咪
U+54AA.0	fCNS11643	1-5041
U+54AA.0	fBig5	AB7D
U+54AA.0	fGB	0-5F64
U+54AA.0	fR/S	 30.6
U+54AA.0	fTotalStrokes	 9
U+54AA.0	fCangjie	RFD
U+54AA.0	fFourCorner	69094
U+54AA.0	fHakka	 mi1
U+54AA.0	fCantonese	mai1; mei1; mai5; mai6; mi1; mi4; miu1
U+54AA.0	fMandarin	mi1
U+54AA.0	fEnglish	[1] sound of cat; cat's meow; miaow; miaou [2] smiling [3] [ Cantonese] don't!; shouldn't; mustn't [4] mile [5] microphone [6] close; shut; smile
U+54AB.0	fUTF8	咫
U+54AB.0	fCNS11643	1-5046
U+54AB.0	fBig5	ABA4
U+54AB.0	fGB	0-656B
U+54AB.0	fR/S	 30.6
U+54AB.0	fTotalStrokes	 9
U+54AB.0	fCangjie	SORC
U+54AB.0	fFourCorner	76808
U+54AB.0	fHakka	 zhi3; zi3; zih3; chit7; ts'iit7; cit7; tje3
U+54AB.0	fCantonese	zi2
U+54AB.0	fMandarin	zhi3
U+54AB.0	fEnglish	the foot measure of the Chou dynasty
U+54AC.0	fUTF8	咬
U+54AC.0	fCNS11643	1-5036
U+54AC.0	fBig5	AB72
U+54AC.0	fGB	0-5227
U+54AC.0	fR/S	 30.6
U+54AC.0	fTotalStrokes	 9
U+54AC.0	fCangjie	RYCK
U+54AC.0	fFourCorner	60048
U+54AC.0	fHakka	  ngau1;  gau1
U+54AC.0	fCantonese	ngaau5; gaau1; aau5 (also); aau1; ngaau1; au5
U+54AC.0	fMandarin	yao3; jiao1; ao1; jiao2
U+54AC.0	fEnglish	 bite; gnaw
U+54AD.0	fUTF8	咭
U+54AD.0	fCNS11643	2-293E
U+54AD.0	fBig5	CDFC
U+54AD.0	fGB	0-5F52
U+54AD.0	fR/S	 30.6
U+54AD.0	fTotalStrokes	 9
U+54AD.0	fCangjie	RGR
U+54AD.0	fFourCorner	64061
U+54AD.0	fHakka	 git7
U+54AD.0	fCantonese	gat1; kat1; kaat1 (also); hat1; kit1
U+54AD.0	fMandarin	ji1; xi1; qia4; jie2; ka3
U+54AD.0	fEnglish	 card
U+54AE.0	fUTF8	咮
U+54AE.0	fCNS11643	2-2944
U+54AE.0	fBig5	CE43
U+54AE.0	fGB	3-3628
U+54AE.0	fR/S	 30.6
U+54AE.0	fTotalStrokes	 9
U+54AE.0	fCangjie	RHJD
U+54AE.0	fFourCorner	65090
U+54AE.0	fHakka	 zhiu5; zhu1; zu1; ziu5
U+54AE.0	fCantonese	zau3; zyu1
U+54AE.0	fMandarin	zhou4; dou4; zhou1; zhu1; zhu4
U+54AE.0	fEnglish	beak
U+54AF.0	fUTF8	咯
U+54AF.0	fCNS11643	1-5045
U+54AF.0	fBig5	ABA3
U+54AF.0	fGB	0-3F29
U+54AF.0	fR/S	 30.6
U+54AF.0	fTotalStrokes	 9
U+54AF.0	fCangjie	RHER
U+54AF.0	fFourCorner	67064
U+54AF.0	fHakka	 lok7;  gok7; gok8; lo1; lok8
U+54AF.0	fCantonese	gok3 (also); gok1; lo1; lok3 (also); kaak3; lo3 (also); haak3 (also); lok6; kat1; gaak3
U+54AF.0	fMandarin	ka3; ka4; ke4; luo4
U+54AF.0	fEnglish	[1] cough [2] [Cantonese used as a sign of polite refusal, completed action, or a transition of one situation to another]
U+54AF.1	fUTF8	咯
U+54AF.1	fCNS11643	1-5045
U+54AF.1	fBig5	ABA3
U+54AF.1	fGB	0-3F29
U+54AF.1	fR/S	 30.6
U+54AF.1	fTotalStrokes	 9
U+54AF.1	fCangjie	RHER
U+54AF.1	fFourCorner	67064
U+54AF.1	fMandarin	lo5
U+54AF.1	fEnglish	a final particle
U+54AF.2	fUTF8	咯
U+54AF.2	fCNS11643	1-5045
U+54AF.2	fBig5	ABA3
U+54AF.2	fGB	0-3F29
U+54AF.2	fR/S	 30.6
U+54AF.2	fTotalStrokes	 9
U+54AF.2	fCangjie	RHER
U+54AF.2	fFourCorner	67064
U+54AF.2	fMandarin	ge1
U+54AF.2	fEnglish	used in combination
U+54AF.3	fUTF8	咯
U+54AF.3	fCNS11643	1-5045
U+54AF.3	fBig5	ABA3
U+54AF.3	fGB	0-3F29
U+54AF.3	fR/S	 30.6
U+54AF.3	fTotalStrokes	 9
U+54AF.3	fCangjie	RHER
U+54AF.3	fFourCorner	67064
U+54AF.3	fMandarin	ge2
U+54AF.3	fEnglish	used in combination
U+54B0.0	fUTF8	咰
U+54B0.0	fCNS11643	2-2950
U+54B0.0	fBig5	CE4F
U+54B0.0	fGB	5-3841
U+54B0.0	fR/S	 30.6
U+54B0.0	fTotalStrokes	 9
U+54B0.0	fCangjie	RPA
U+54B0.0	fFourCorner	67020
U+54B0.0	fCantonese	seot1; seon1; seot6
U+54B0.0	fMandarin	shuai4; xun2
U+54B1.0	fUTF8	咱
U+54B1.0	fCNS11643	1-5047
U+54B1.0	fBig5	ABA5
U+54B1.0	fGB	0-545B
U+54B1.0	fR/S	 30.6
U+54B1.0	fTotalStrokes	 9
U+54B1.0	fCangjie	RHBU
U+54B1.0	fFourCorner	66000
U+54B1.0	fHakka	  za1; zap7; zia1
U+54B1.0	fCantonese	zaa1; gau6
U+54B1.0	fMandarin	za2; zan2
U+54B1.0	fEnglish	[North China dialect] I; me
U+54B1.1	fUTF8	咱
U+54B1.1	fCNS11643	1-5047
U+54B1.1	fBig5	ABA5
U+54B1.1	fGB	0-545B
U+54B1.1	fR/S	 30.6
U+54B1.1	fTotalStrokes	 9
U+54B1.1	fCangjie	RHBU
U+54B1.1	fFourCorner	66000
U+54B1.1	fMandarin	zan5
U+54B2.0	fUTF8	咲
U+54B2.0	fCNS11643	3-2B5B
U+54B2.0	fGB	E-244D
U+54B2.0	fR/S	 30.6
U+54B2.0	fTotalStrokes	 9
U+54B2.0	fCangjie	RTK
U+54B2.0	fFourCorner	68030
U+54B2.0	fHakka	 siau5; seu5
U+54B2.0	fCantonese	siu3
U+54B2.0	fMandarin	xiao4
U+54B2.0	fEnglish	 smile; laugh; giggle snicker
U+54B3.0	fUTF8	咳
U+54B3.0	fCNS11643	1-503D
U+54B3.0	fBig5	AB79
U+54B3.0	fGB	0-3F48
U+54B3.0	fR/S	 30.6
U+54B3.0	fTotalStrokes	 9
U+54B3.0	fCangjie	RYVO
U+54B3.0	fFourCorner	60082
U+54B3.0	fHakka	  ket7; hai2; kiet7; kak7; kat8; kem3; kak8
U+54B3.0	fCantonese	hoi4; haai4 (also); koi3; kat1; haai1; hei1
U+54B3.0	fMandarin	ke2; hai1; hai2; kai4
U+54B3.0	fEnglish	 cough
U+54B3.1	fUTF8	咳
U+54B3.1	fCNS11643	1-503D
U+54B3.1	fBig5	AB79
U+54B3.1	fGB	0-3F48
U+54B3.1	fR/S	 30.6
U+54B3.1	fTotalStrokes	 9
U+54B3.1	fCangjie	RYVO
U+54B3.1	fFourCorner	60082
U+54B3.1	fMandarin	ka3
U+54B3.1	fEnglish	 cough up
U+54B3.2	fUTF8	咳
U+54B3.2	fCNS11643	1-503D
U+54B3.2	fBig5	AB79
U+54B3.2	fGB	0-3F48
U+54B3.2	fR/S	 30.6
U+54B3.2	fTotalStrokes	 9
U+54B3.2	fCangjie	RYVO
U+54B3.2	fFourCorner	60082
U+54B3.2	fMandarin	hai4
U+54B3.2	fEnglish	an interjection of regret or remorse
U+54B4.0	fUTF8	咴
U+54B4.0	fCNS11643	3-2B54
U+54B4.0	fGB	0-5F54
U+54B4.0	fR/S	  30.6
U+54B4.0	fTotalStrokes	 9
U+54B4.0	fCangjie	RKF
U+54B4.0	fFourCorner	64080
U+54B4.0	fCantonese	fui1
U+54B4.0	fMandarin	hui1
U+54B5.0	fUTF8	咵
U+54B5.0	fCNS11643	3-2B5E
U+54B5.0	fGB	5-3839
U+54B5.0	fR/S	  30.6
U+54B5.0	fTotalStrokes	 9
U+54B5.0	fCangjie	RKMS
U+54B5.0	fFourCorner	64020
U+54B5.0	fCantonese	kwaa1; kwaa2
U+54B5.0	fMandarin	kua3; kua1
U+54B5.0	fEnglish	 revile
U+54B6.0	fUTF8	咶
U+54B6.0	fCNS11643	2-2946
U+54B6.0	fBig5	CE45
U+54B6.0	fGB	3-3629
U+54B6.0	fR/S	 30.6
U+54B6.0	fTotalStrokes	 9
U+54B6.0	fCangjie	RHJR
U+54B6.0	fFourCorner	62064
U+54B6.0	fHakka	 wa3; se1
U+54B6.0	fCantonese	waat6; saai5; si6
U+54B6.0	fMandarin	huai4; shi4; hua2; hua4
U+54B7.0	fUTF8	咷
U+54B7.0	fCNS11643	2-2943
U+54B7.0	fBig5	CE42
U+54B7.0	fGB	5-3840
U+54B7.0	fR/S	 30.6
U+54B7.0	fTotalStrokes	 9
U+54B7.0	fCangjie	RLMO
U+54B7.0	fFourCorner	62013
U+54B7.0	fHakka	 tau2; to2; tiau5
U+54B7.0	fCantonese	tou4
U+54B7.0	fMandarin	tao2; di2
U+54B7.0	fEnglish	howl; cry loudly
U+54B8.0	fUTF8	咸
U+54B8.0	fCNS11643	1-503B
U+54B8.0	fBig5	AB77
U+54B8.0	fGB	0-4F4C
U+54B8.0	fR/S	 30.6
U+54B8.0	fTotalStrokes	 9
U+54B8.0	fCangjie	IHMR
U+54B8.0	fFourCorner	53200
U+54B8.0	fHakka	  ham2; ham3
U+54B8.0	fCantonese	haam4; haam2 (also)
U+54B8.0	fMandarin	xian2; han4; jian3
U+54B8.0	fEnglish	together; all; completely; united; fully; wholly
U+54B9.0	fUTF8	咹
U+54B9.0	fCNS11643	3-2B51
U+54B9.0	fGB	3-362D
U+54B9.0	fR/S	  30.6
U+54B9.0	fTotalStrokes	 9
U+54B9.0	fCangjie	RJV
U+54B9.0	fFourCorner	63040
U+54B9.0	fCantonese	hon1
U+54B9.0	fMandarin	e4
U+54B9.0	fEnglish	sound of an automobile horn
U+54BA.0	fUTF8	咺
U+54BA.0	fCNS11643	2-293C
U+54BA.0	fBig5	CDFA
U+54BA.0	fGB	3-357C
U+54BA.0	fR/S	 30.6
U+54BA.0	fTotalStrokes	 9
U+54BA.0	fCangjie	RMAM
U+54BA.0	fFourCorner	61016
U+54BA.0	fHakka	  sien1
U+54BA.0	fCantonese	hyun2; hyun1
U+54BA.0	fMandarin	xuan3; xuan1
U+54BA.0	fEnglish	majestic
U+54BB.0	fUTF8	咻
U+54BB.0	fCNS11643	1-5048
U+54BB.0	fBig5	ABA6
U+54BB.0	fGB	0-5F5D
U+54BB.0	fR/S	 30.6
U+54BB.0	fTotalStrokes	 9
U+54BB.0	fCangjie	ROD
U+54BB.0	fFourCorner	64090
U+54BB.0	fHakka	 hiu1
U+54BB.0	fCantonese	jau1; heoi2
U+54BB.0	fMandarin	xiu1; xu3; xiao1; xu4
U+54BB.0	fEnglish	shout
U+54BC.0	fUTF8	咼
U+54BC.0	fCNS11643	2-294B
U+54BC.0	fBig5	CE4A
U+54BC.0	fGB	1-5F43
U+54BC.0	fR/S	 30.6
U+54BC.0	fTotalStrokes	 9
U+54BC.0	fCangjie	BBR
U+54BC.0	fFourCorner	77227
U+54BC.0	fHakka	 wo1; wa1; fe3; vo1; va2
U+54BC.0	fCantonese	waa1; gwo1; go1 (also); kwaa1; wo4; gwaa3
U+54BC.0	fMandarin	wai1; guo1; kuai1
U+54BC.0	fEnglish	chat; jaw; gossip; talk; mouth
U+54BD.0	fUTF8	咽
U+54BD.0	fCNS11643	1-5040
U+54BD.0	fBig5	AB7C
U+54BD.0	fGB	0-514A
U+54BD.0	fR/S	 30.6
U+54BD.0	fTotalStrokes	 9
U+54BD.0	fCangjie	RWK
U+54BD.0	fFourCorner	66000
U+54BD.0	fHakka	  jen1; jet7; jin1; rin1; jan5; ren1; jen5; ret7; jan1
U+54BD.0	fCantonese	jin1; jin3; jit3
U+54BD.0	fMandarin	yan1; yin1
U+54BD.0	fEnglish	 throat; pharynx; larynx; gullet
U+54BD.1	fUTF8	咽
U+54BD.1	fCNS11643	1-5040
U+54BD.1	fBig5	AB7C
U+54BD.1	fGB	0-514A
U+54BD.1	fR/S	 30.6
U+54BD.1	fTotalStrokes	 9
U+54BD.1	fCangjie	RWK
U+54BD.1	fFourCorner	66000
U+54BD.1	fMandarin	yan4
U+54BD.1	fEnglish	 swallow; gulp
U+54BD.2	fUTF8	咽
U+54BD.2	fCNS11643	1-5040
U+54BD.2	fBig5	AB7C
U+54BD.2	fGB	0-514A
U+54BD.2	fR/S	 30.6
U+54BD.2	fTotalStrokes	 9
U+54BD.2	fCangjie	RWK
U+54BD.2	fFourCorner	66000
U+54BD.2	fMandarin	ye4
U+54BD.2	fEnglish	 be choked; weep or speak in a choked voice; sob
U+54BE.0	fUTF8	咾
U+54BE.0	fCNS11643	2-294D
U+54BE.0	fBig5	CE4C
U+54BE.0	fGB	5-3836
U+54BE.0	fR/S	 30.6
U+54BE.0	fTotalStrokes	 9
U+54BE.0	fCangjie	RJKP
U+54BE.0	fFourCorner	64011
U+54BE.0	fHakka	 lo3; lau33; lo1; lau3
U+54BE.0	fCantonese	lou5
U+54BE.0	fMandarin	lao3; liao3
U+54BE.0	fEnglish	[1] noise; sound [2] [3] final particle [4] final particle with no meaning
U+54BF.0	fUTF8	咿
U+54BF.0	fCNS11643	1-504B
U+54BF.0	fBig5	ABA9
U+54BF.0	fGB	0-5F5E
U+54BF.0	fR/S	 30.6
U+54BF.0	fTotalStrokes	 9
U+54BF.0	fCangjie	ROSK
U+54BF.0	fFourCorner	67027
U+54BF.0	fHakka	 ji2; ri2; i1; ji1
U+54BF.0	fCantonese	ji1
U+54BF.0	fMandarin	yi1
U+54BF.0	fEnglish	descriptive of creaking
U+54C0.0	fUTF8	哀
U+54C0.0	fCNS11643	1-5037
U+54C0.0	fBig5	AB73
U+54C0.0	fGB	0-3027
U+54C0.0	fR/S	 30.6
U+54C0.0	fTotalStrokes	 9
U+54C0.0	fCangjie	YRHV
U+54C0.0	fFourCorner	00732
U+54C0.0	fHakka	  oi1
U+54C0.0	fCantonese	oi1; ngoi1 (also)
U+54C0.0	fMandarin	ai1
U+54C0.0	fEnglish	[1] grieve; grieve for; mourn; lament [2] sad; mournful; sorrowful; lamentable [3] pity; sympathise; sympathise with; commiserate; compassion [4] sadness; sorrow; grief
U+54C1.0	fUTF8	品
U+54C1.0	fCNS11643	1-5042
U+54C1.0	fBig5	AB7E
U+54C1.0	fGB	0-4637
U+54C1.0	fR/S	 30.6
U+54C1.0	fTotalStrokes	 9
U+54C1.0	fCangjie	RRR
U+54C1.0	fFourCorner	60660
U+54C1.0	fHakka	  pin3
U+54C1.0	fCantonese	ban2
U+54C1.0	fMandarin	pin3
U+54C1.0	fEnglish	[1] personality; character [2] article; product; commodity [3] a rank or grade in the government service in former times [4] appraise; rate [5] find out
U+54C2.0	fUTF8	哂
U+54C2.0	fCNS11643	1-503F
U+54C2.0	fBig5	AB7B
U+54C2.0	fGB	0-5F53
U+54C2.0	fR/S	 30.6
U+54C2.0	fTotalStrokes	 9
U+54C2.0	fCangjie	RMCW
U+54C2.0	fFourCorner	61060
U+54C2.0	fHakka	 shin3; sin3; jin3; shin5; sin5; si1; rin3
U+54C2.0	fCantonese	can2; saai3 (also)
U+54C2.0	fMandarin	shen3
U+54C2.0	fEnglish	 give a sneering smile; sneer at
U+54C3.0	fUTF8	哃
U+54C3.0	fCNS11643	2-2941
U+54C3.0	fBig5	CE40
U+54C3.0	fGB	3-3627
U+54C3.0	fR/S	 30.6
U+54C3.0	fTotalStrokes	 9
U+54C3.0	fCangjie	RBMR
U+54C3.0	fFourCorner	67020
U+54C3.0	fCantonese	tung4
U+54C3.0	fMandarin	tong2
U+54C4.0	fUTF8	哄
U+54C4.0	fCNS11643	1-5043
U+54C4.0	fBig5	ABA1
U+54C4.0	fGB	0-3A65
U+54C4.0	fR/S	 30.6
U+54C4.0	fTotalStrokes	 9
U+54C4.0	fCangjie	RTC
U+54C4.0	fFourCorner	64081
U+54C4.0	fHakka	 giung1;  fung3; fung5; kiung3; hung3; fung6; giung3
U+54C4.0	fCantonese	hung1; hung3; hung6 (also); hung2
U+54C4.0	fMandarin	hong1; hong4; gong1
U+54C4.0	fEnglish	 beguile; cheat; deceive; defraud
U+54C4.1	fUTF8	哄
U+54C4.1	fCNS11643	1-5043
U+54C4.1	fBig5	ABA1
U+54C4.1	fGB	0-3A65
U+54C4.1	fR/S	 30.6
U+54C4.1	fTotalStrokes	 9
U+54C4.1	fCangjie	RTC
U+54C4.1	fFourCorner	64081
U+54C4.1	fMandarin	hong3
U+54C4.1	fEnglish	 make a roaring noise (said of a group of people)
U+54C5.0	fUTF8	哅
U+54C5.0	fCNS11643	2-2947
U+54C5.0	fBig5	CE46
U+54C5.0	fGB	3-362C
U+54C5.0	fR/S	 30.6
U+54C5.0	fTotalStrokes	 9
U+54C5.0	fCangjie	RPUK
U+54C5.0	fFourCorner	67020
U+54C5.0	fCantonese	hung1
U+54C5.0	fMandarin	xiong1; hong1
U+54C6.0	fUTF8	哆
U+54C6.0	fCNS11643	2-2948
U+54C6.0	fBig5	CE47
U+54C6.0	fGB	0-365F
U+54C6.0	fR/S	 30.6
U+54C6.0	fTotalStrokes	 9
U+54C6.0	fCangjie	RNIN
U+54C6.0	fFourCorner	67027
U+54C6.0	fHakka	 chi3; cih3; chii3
U+54C6.0	fCantonese	do1; ci2; ce2 (also); do2
U+54C6.0	fMandarin	duo1; cha4; chi3; chi1; chi4; duo3
U+54C6.0	fEnglish	 tremble; shiver; shudder; quiver
U+54C7.0	fUTF8	哇
U+54C7.0	fCNS11643	1-503E
U+54C7.0	fBig5	AB7A
U+54C7.0	fGB	0-4D5B
U+54C7.0	fR/S	 30.6
U+54C7.0	fTotalStrokes	 9
U+54C7.0	fCangjie	RGG
U+54C7.0	fFourCorner	64014
U+54C7.0	fHakka	  wa1; wa5; va1
U+54C7.0	fCantonese	waa1
U+54C7.0	fMandarin	wa1; gui1; huai2
U+54C7.0	fEnglish	[1] vomit [2] cry of an infant; sound of crying by a child
U+54C7.1	fUTF8	哇
U+54C7.1	fCNS11643	1-503E
U+54C7.1	fBig5	AB7A
U+54C7.1	fGB	0-4D5B
U+54C7.1	fR/S	 30.6
U+54C7.1	fTotalStrokes	 9
U+54C7.1	fCangjie	RGG
U+54C7.1	fFourCorner	64014
U+54C7.1	fMandarin	wa5
U+54C7.1	fEnglish	a phrase-final particle
U+54C8.0	fUTF8	哈
U+54C8.0	fCNS11643	1-5044
U+54C8.0	fBig5	ABA2
U+54C8.0	fGB	0-397E
U+54C8.0	fR/S	 30.6
U+54C8.0	fTotalStrokes	 9
U+54C8.0	fCangjie	ROMR
U+54C8.0	fFourCorner	68061
U+54C8.0	fHakka	  hap8;  ha2; ha1; hat8; hap7; hat7
U+54C8.0	fCantonese	haa1; haa5; kaa1; haai1
U+54C8.0	fMandarin	ha4; ha1; he1; ka1; sha4
U+54C8.0	fEnglish	[1] a form used in transliteration [2] sound of hearty laughter
U+54C8.1	fUTF8	哈
U+54C8.1	fCNS11643	1-5044
U+54C8.1	fBig5	ABA2
U+54C8.1	fGB	0-397E
U+54C8.1	fR/S	 30.6
U+54C8.1	fTotalStrokes	 9
U+54C8.1	fCangjie	ROMR
U+54C8.1	fFourCorner	68061
U+54C8.1	fMandarin	ha3
U+54C8.1	fEnglish	used in combination
U+54C9.0	fUTF8	哉
U+54C9.0	fCNS11643	1-503A
U+54C9.0	fBig5	AB76
U+54C9.0	fGB	0-5455
U+54C9.0	fR/S	 30.6
U+54C9.0	fTotalStrokes	 9
U+54C9.0	fCangjie	JIR
U+54C9.0	fFourCorner	43650
U+54C9.0	fHakka	  zai1; zai5
U+54C9.0	fCantonese	zoi1
U+54C9.0	fMandarin	zai1
U+54C9.0	fEnglish	a phrase-final particle expressing surprise, admiration, grief, doubt, etc. (a literary form)
U+54CA.0	fUTF8	哊
U+54CA.0	fCNS11643	3-2B55
U+54CA.0	fGB	3-357E
U+54CA.0	fR/S	  30.6
U+54CA.0	fTotalStrokes	 9
U+54CA.0	fCangjie	RKB
U+54CA.0	fFourCorner	64020
U+54CA.0	fHakka	 jok7; rok7; ruk7; juk7; juk8
U+54CA.0	fCantonese	juk6
U+54CA.0	fMandarin	yu4; you4
U+54CA.0	fEnglish	 vomit; belch
U+54CB.0	fUTF8	哋
U+54CB.0	fCNS11643	3-6454
U+54CB.0	fGB	1-7C26
U+54CB.0	fR/S	 30.6
U+54CB.0	fTotalStrokes	 9
U+54CB.0	fCangjie	RGPD
U+54CB.0	fFourCorner	6401
U+54CB.0	fCantonese	dei6; dei2; di4
U+54CB.0	fMandarin	di4; dei1
U+54CB.0	fEnglish	 used as a suffix to personal pronouns and the word &#4EBA; as a sign of the plural number
U+54CC.0	fUTF8	哌
U+54CC.0	fCNS11643	3-2B63
U+54CC.0	fGB	0-5F5F
U+54CC.0	fR/S	  30.6
U+54CC.0	fTotalStrokes	 9
U+54CC.0	fCangjie	RHHV
U+54CC.0	fFourCorner	62030
U+54CC.0	fCantonese	paai3
U+54CC.0	fMandarin	gu1; pai4
U+54CC.0	fEnglish	used in translation
U+54CD.0	fUTF8	响
U+54CD.0	fCNS11643	3-2B61
U+54CD.0	fGB	0-4F6C
U+54CD.0	fR/S	 30.6
U+54CD.0	fTotalStrokes	 9
U+54CD.0	fCangjie	RHBR
U+54CD.0	fFourCorner	67020
U+54CD.0	fHakka	 hiong3
U+54CD.0	fCantonese	hoeng2
U+54CD.0	fMandarin	xiang3
U+54CD.0	fEnglish	[1] be in; be at; be on in; at; on [2] simplified form of &#x97FF;
U+54CE.0	fUTF8	哎
U+54CE.0	fCNS11643	1-5039
U+54CE.0	fBig5	AB75
U+54CE.0	fGB	0-3025
U+54CE.0	fR/S	 30.6
U+54CE.0	fTotalStrokes	 9
U+54CE.0	fCangjie	RTK
U+54CE.0	fFourCorner	64040
U+54CE.0	fHakka	  ai1; ai3; ngai6; ngai5; ngiai5
U+54CE.0	fCantonese	aai1; ngaai1 (also); ai3 (also); ngai3 (also); ngaai6
U+54CE.0	fMandarin	ai1
U+54CE.0	fEnglish	an interjection of surprise, regret, etc.
U+54CF.0	fUTF8	哏
U+54CF.0	fCNS11643	2-2940
U+54CF.0	fBig5	CDFE
U+54CF.0	fGB	0-5F67
U+54CF.0	fR/S	 30.6
U+54CF.0	fTotalStrokes	 9
U+54CF.0	fCangjie	RAV
U+54CF.0	fFourCorner	67032
U+54CF.0	fHakka	 ken3; gen1
U+54CF.0	fCantonese	gan1; han2
U+54CF.0	fMandarin	gen2; hen3
U+54CF.0	fEnglish	the moise or din of people quarelling
U+54D0.0	fUTF8	哐
U+54D0.0	fCNS11643	3-2B53
U+54D0.0	fGB	0-5F51
U+54D0.0	fR/S	  30.6
U+54D0.0	fTotalStrokes	 9
U+54D0.0	fCangjie	RSMG
U+54D0.0	fFourCorner	61010
U+54D0.0	fCantonese	hong1
U+54D0.0	fMandarin	kuang1
U+54D0.0	fEnglish	syllable
U+54D1.0	fUTF8	哑
U+54D1.0	fGB	0-5146
U+54D1.0	fR/S	 30.6
U+54D1.0	fTotalStrokes	 9
U+54D1.0	fCangjie	RMTC
U+54D1.0	fHakka	 a3
U+54D1.0	fCantonese	aa2; aa1; ngaa2 (also); ngaa1 (also); ak1; ngak1 (also)
U+54D1.0	fMandarin	ya3; ya1; e4; ya5
U+54D1.0	fEnglish	simplified form of &#x555E;
U+54D2.0	fUTF8	哒
U+54D2.0	fGB	0-5F55
U+54D2.0	fR/S	 30.6
U+54D2.0	fTotalStrokes	 9
U+54D2.0	fCangjie	XRYK
U+54D2.0	fCantonese	daat6
U+54D2.0	fMandarin	da1; da2
U+54D2.0	fEnglish	simplified form of &#x5660;
U+54D3.0	fUTF8	哓
U+54D3.0	fGB	0-5F58
U+54D3.0	fR/S	 30.6
U+54D3.0	fTotalStrokes	 9
U+54D3.0	fCangjie	RJPU
U+54D3.0	fHakka	 hiau1
U+54D3.0	fCantonese	hiu1
U+54D3.0	fMandarin	xiao1
U+54D3.0	fEnglish	simplified form of &#x5635;
U+54D4.0	fUTF8	哔
U+54D4.0	fGB	0-5F59
U+54D4.0	fR/S	 30.6
U+54D4.0	fTotalStrokes	 9
U+54D4.0	fCangjie	RPPJ
U+54D4.0	fHakka	 bit7
U+54D4.0	fCantonese	bat1
U+54D4.0	fMandarin	bi4
U+54D4.0	fEnglish	simplified form of &#x55F6;
U+54D5.0	fUTF8	哕
U+54D5.0	fGB	0-5F5C
U+54D5.0	fR/S	 30.6
U+54D5.0	fTotalStrokes	 9
U+54D5.0	fCangjie	RUNI
U+54D5.0	fHakka	 vui3; oi5
U+54D5.0	fCantonese	jyut6; wai3
U+54D5.0	fMandarin	hui4; yue3; yue1; huai4
U+54D5.0	fEnglish	simplified form of &#x5666;
U+54D6.0	fUTF8	哖
U+54D6.0	fCNS11643	2-2945
U+54D6.0	fBig5	CE44
U+54D6.0	fGB	E-244E
U+54D6.0	fR/S	 30.6
U+54D6.0	fTotalStrokes	 9
U+54D6.0	fCangjie	ROQ
U+54D6.0	fFourCorner	68050
U+54D6.0	fCantonese	nin4; lin4 (also)
U+54D7.0	fUTF8	哗
U+54D7.0	fGB	0-3B29
U+54D7.0	fR/S	 30.6
U+54D7.0	fTotalStrokes	 9
U+54D7.0	fCangjie	ROPJ
U+54D7.0	fHakka	 fa2
U+54D7.0	fCantonese	waa1; waa4 (also)
U+54D7.0	fMandarin	hua1; wa2; hua2
U+54D7.0	fEnglish	simplified form of &#x5629;
U+54D8.0	fUTF8	哘
U+54D8.0	fGB	E-244F
U+54D8.0	fR/S	 30.6
U+54D8.0	fTotalStrokes	 9
U+54D8.0	fCangjie	RHON
U+54D8.0	fCantonese	hang4
U+54D9.0	fUTF8	哙
U+54D9.0	fGB	0-5F60
U+54D9.0	fR/S	 30.6
U+54D9.0	fTotalStrokes	 9
U+54D9.0	fCangjie	ROMI
U+54D9.0	fHakka	 kwai5
U+54D9.0	fCantonese	faai3
U+54D9.0	fMandarin	kuai4
U+54D9.0	fEnglish	simplified form of &#x5672;
U+54DA.0	fUTF8	哚
U+54DA.0	fCNS11643	3-2B58
U+54DA.0	fGB	0-5F61
U+54DA.0	fR/S	 30.6
U+54DA.0	fTotalStrokes	 9
U+54DA.0	fCangjie	RHND
U+54DA.0	fFourCorner	67090
U+54DA.0	fCantonese	doe2; do2; doe1
U+54DA.0	fMandarin	duo3
U+54DA.0	fEnglish	chemical element
U+54DB.0	fUTF8	哛
U+54DB.0	fGB	1-7D64
U+54DB.0	fR/S	 30.6
U+54DB.0	fTotalStrokes	 9
U+54DB.0	fCangjie	RPCSH
U+54DB.0	fCantonese	fan1
U+54DC.0	fUTF8	哜
U+54DC.0	fGB	0-5F62
U+54DC.0	fR/S	 30.6
U+54DC.0	fTotalStrokes	 9
U+54DC.0	fCangjie	RYKL
U+54DC.0	fHakka	 zi5
U+54DC.0	fCantonese	zai6; cai6
U+54DC.0	fMandarin	ji4; jie1; qi2; zhai1
U+54DC.0	fEnglish	simplified form of &#x568C;
U+54DD.0	fUTF8	哝
U+54DD.0	fGB	0-5F66
U+54DD.0	fR/S	 30.6
U+54DD.0	fTotalStrokes	 9
U+54DD.0	fCangjie	RHBV
U+54DD.0	fHakka	 nung2
U+54DD.0	fCantonese	nung4; lung4 (also)
U+54DD.0	fMandarin	nong2; nong5
U+54DD.0	fEnglish	simplified form of &#x5665;
U+54DE.0	fUTF8	哞
U+54DE.0	fCNS11643	2-294F
U+54DE.0	fBig5	CE4E
U+54DE.0	fGB	0-5F68
U+54DE.0	fR/S	 30.6
U+54DE.0	fTotalStrokes	 9
U+54DE.0	fCangjie	RIHQ
U+54DE.0	fFourCorner	63050
U+54DE.0	fCantonese	mau4 (also); mau1
U+54DE.0	fMandarin	mou1; mou2
U+54DF.0	fUTF8	哟
U+54DF.0	fGB	0-5334
U+54DF.0	fR/S	 30.6
U+54DF.0	fTotalStrokes	 9
U+54DF.0	fCangjie	RVMI
U+54DF.0	fCantonese	jo1
U+54DF.0	fMandarin	yo1; yao1
U+54DF.0	fEnglish	simplified form of &#x55B2;
U+54DF.1	fUTF8	哟
U+54DF.1	fGB	0-5334
U+54DF.1	fR/S	 30.6
U+54DF.1	fTotalStrokes	 9
U+54DF.1	fCangjie	RVMI
U+54DF.1	fMandarin	yo5
U+54E0.0	fUTF8	哠
U+54E0.0	fCNS11643	2-2E46
U+54E0.0	fBig5	D144
U+54E0.0	fGB	5-3850
U+54E0.0	fR/S	 30.7
U+54E0.0	fTotalStrokes	 10
U+54E0.0	fCangjie	RHGR
U+54E0.0	fFourCorner	64061
U+54E0.0	fCantonese	hou6
U+54E0.0	fMandarin	hao4
U+54E1.0	fUTF8	員
U+54E1.0	fCNS11643	1-545E
U+54E1.0	fBig5	ADFB
U+54E1.0	fGB	1-5431
U+54E1.0	fR/S	 30.7
U+54E1.0	fTotalStrokes	 10
U+54E1.0	fCangjie	RBUC
U+54E1.0	fFourCorner	60806
U+54E1.0	fHakka	  jen2; jan3; ren2; jan2
U+54E1.0	fCantonese	jyun4; wan4; wan6; jyun2 (also)
U+54E1.0	fMandarin	yuan2; yun4; yun2
U+54E1.0	fEnglish	[1] member (of an organization, etc.); personnel; staff member [2] a person engaged in some field of activity [3] outer limits (of land, space, etc.)
U+54E2.0	fUTF8	哢
U+54E2.0	fCNS11643	2-2E34
U+54E2.0	fBig5	D0F1
U+54E2.0	fGB	3-362F
U+54E2.0	fR/S	 30.7
U+54E2.0	fTotalStrokes	 10
U+54E2.0	fCangjie	RMGT
U+54E2.0	fFourCorner	61041
U+54E2.0	fHakka	 lung5; lung6; lung3
U+54E2.0	fCantonese	lung6
U+54E2.0	fMandarin	long4
U+54E2.0	fEnglish	syllable
U+54E3.0	fUTF8	哣
U+54E3.0	fCNS11643	4-2C2D
U+54E3.0	fGB	1-7C29
U+54E3.0	fR/S	 30.7
U+54E3.0	fTotalStrokes	 10
U+54E3.0	fCangjie	RMRT
U+54E3.0	fFourCorner	61010
U+54E3.0	fHakka	 teu5
U+54E3.0	fCantonese	dau6; dau2
U+54E3.0	fMandarin	pou3; tou4
U+54E3.0	fEnglish	sick, unwell
U+54E4.0	fUTF8	哤
U+54E4.0	fCNS11643	2-2E39
U+54E4.0	fBig5	D0F6
U+54E4.0	fGB	3-3634
U+54E4.0	fR/S	 30.7
U+54E4.0	fTotalStrokes	 10
U+54E4.0	fCangjie	RIUH
U+54E4.0	fFourCorner	63012
U+54E4.0	fHakka	  mang2
U+54E4.0	fCantonese	mong4
U+54E4.0	fMandarin	mang2
U+54E4.0	fEnglish	jargon
U+54E5.0	fUTF8	哥
U+54E5.0	fCNS11643	1-5457
U+54E5.0	fBig5	ADF4
U+54E5.0	fGB	0-3867
U+54E5.0	fR/S	 30.7
U+54E5.0	fTotalStrokes	 10
U+54E5.0	fCangjie	MRNR
U+54E5.0	fFourCorner	10621
U+54E5.0	fHakka	  go1
U+54E5.0	fCantonese	go1; go4 (also)
U+54E5.0	fMandarin	ge1
U+54E5.0	fEnglish	 elder brother
U+54E5.1	fUTF8	哥
U+54E5.1	fCNS11643	1-5457
U+54E5.1	fBig5	ADF4
U+54E5.1	fGB	0-3867
U+54E5.1	fR/S	 30.7
U+54E5.1	fTotalStrokes	 10
U+54E5.1	fCangjie	MRNR
U+54E5.1	fFourCorner	10621
U+54E5.1	fMandarin	ge5
U+54E6.0	fUTF8	哦
U+54E6.0	fCNS11643	1-5462
U+54E6.0	fBig5	AE40
U+54E6.0	fGB	0-4536
U+54E6.0	fR/S	 30.7
U+54E6.0	fTotalStrokes	 10
U+54E6.0	fCangjie	RHQI
U+54E6.0	fFourCorner	63050
U+54E6.0	fHakka	 ngo2
U+54E6.0	fCantonese	ngo4; ngo2 (also); ngo6 (also); o4; o2; o6; o3
U+54E6.0	fMandarin	o2
U+54E6.0	fEnglish	oh?; ah?; really?; is that so? (an interjection)
U+54E6.1	fUTF8	哦
U+54E6.1	fCNS11643	1-5462
U+54E6.1	fBig5	AE40
U+54E6.1	fGB	0-4536
U+54E6.1	fR/S	 30.7
U+54E6.1	fTotalStrokes	 10
U+54E6.1	fCangjie	RHQI
U+54E6.1	fFourCorner	63050
U+54E6.1	fMandarin	o4
U+54E6.2	fUTF8	哦
U+54E6.2	fCNS11643	1-5462
U+54E6.2	fBig5	AE40
U+54E6.2	fGB	0-4536
U+54E6.2	fR/S	 30.7
U+54E6.2	fTotalStrokes	 10
U+54E6.2	fCangjie	RHQI
U+54E6.2	fFourCorner	63050
U+54E6.2	fMandarin	e2
U+54E6.2	fEnglish	 recite (verses, etc.)
U+54E7.0	fUTF8	哧
U+54E7.0	fCNS11643	2-2E37
U+54E7.0	fBig5	D0F4
U+54E7.0	fGB	0-5F6A
U+54E7.0	fR/S	 30.7
U+54E7.0	fTotalStrokes	 10
U+54E7.0	fCangjie	RGLC
U+54E7.0	fFourCorner	64031
U+54E7.0	fCantonese	ci1; cik1 (also)
U+54E7.0	fMandarin	chi1; xia4; he4; chi4
U+54E8.0	fUTF8	哨
U+54E8.0	fCNS11643	1-5452
U+54E8.0	fBig5	ADEF
U+54E8.0	fGB	0-495A
U+54E8.0	fR/S	 30.7
U+54E8.0	fTotalStrokes	 10
U+54E8.0	fCangjie	RFB
U+54E8.0	fFourCorner	69027
U+54E8.0	fHakka	  sau5;  ciau5; seu1; siau5; siau1
U+54E8.0	fCantonese	saau3; saau2 (also)
U+54E8.0	fMandarin	shao4; xiao1
U+54E8.0	fEnglish	[1] whistle; blow whistle [2] patrol [3] outpost; guard station
U+54E9.0	fUTF8	哩
U+54E9.0	fCNS11643	1-545C
U+54E9.0	fBig5	ADF9
U+54E9.0	fGB	0-4128
U+54E9.0	fR/S	 30.7
U+54E9.0	fTotalStrokes	 10
U+54E9.0	fCangjie	RWG
U+54E9.0	fFourCorner	66014
U+54E9.0	fHakka	 li3;  li1; le1
U+54E9.0	fCantonese	le1; lei5; li1; jing1 (also)
U+54E9.0	fMandarin	li1; li5
U+54E9.0	fEnglish	 mile
U+54E9.1	fUTF8	哩
U+54E9.1	fCNS11643	1-545C
U+54E9.1	fBig5	ADF9
U+54E9.1	fGB	0-4128
U+54E9.1	fR/S	 30.7
U+54E9.1	fTotalStrokes	 10
U+54E9.1	fCangjie	RWG
U+54E9.1	fFourCorner	66014
U+54E9.1	fMandarin	li3
U+54E9.1	fEnglish	 speak indistinctly
U+54EA.0	fUTF8	哪
U+54EA.0	fCNS11643	1-5461
U+54EA.0	fBig5	ADFE
U+54EA.0	fGB	0-4444
U+54EA.0	fR/S	 30.7
U+54EA.0	fTotalStrokes	 10
U+54EA.0	fCangjie	RSQL
U+54EA.0	fFourCorner	67027
U+54EA.0	fHakka	 no2; la3; na3; la5; la2; na2; na1
U+54EA.0	fCantonese	naa5; naa1; naa4; laa5 (also); laa1 (also); laa4 (also); no4
U+54EA.0	fMandarin	na3; nei3; ne2; nai3; ne3
U+54EA.0	fEnglish	which?; where?; how?; what? (an interrogative particle)
U+54EA.1	fUTF8	哪
U+54EA.1	fCNS11643	1-5461
U+54EA.1	fBig5	ADFE
U+54EA.1	fGB	0-4444
U+54EA.1	fR/S	 30.7
U+54EA.1	fTotalStrokes	 10
U+54EA.1	fCangjie	RSQL
U+54EA.1	fFourCorner	67027
U+54EA.1	fMandarin	na5
U+54EA.1	fEnglish	a final particle
U+54EB.0	fUTF8	哫
U+54EB.0	fCNS11643	2-2E3E
U+54EB.0	fBig5	D0FB
U+54EB.0	fGB	3-3637
U+54EB.0	fR/S	 30.7
U+54EB.0	fTotalStrokes	 10
U+54EB.0	fCangjie	RRYO
U+54EB.0	fFourCorner	66081
U+54EB.0	fCantonese	zuk1
U+54EB.0	fMandarin	zu2; dou3
U+54EB.0	fEnglish	 cajole
U+54EC.0	fUTF8	哬
U+54EC.0	fCNS11643	4-2C26
U+54EC.0	fGB	5-3852
U+54EC.0	fR/S	  30.7
U+54EC.0	fTotalStrokes	 10
U+54EC.0	fCangjie	ROMR
U+54EC.0	fFourCorner	61020
U+54EC.0	fCantonese	ho4
U+54EC.0	fMandarin	he2
U+54ED.0	fUTF8	哭
U+54ED.0	fCNS11643	1-545D
U+54ED.0	fBig5	ADFA
U+54ED.0	fGB	0-3F5E
U+54ED.0	fR/S	 30.7
U+54ED.0	fTotalStrokes	 10
U+54ED.0	fCangjie	RRIK
U+54ED.0	fFourCorner	66430
U+54ED.0	fHakka	  kuk7; k'wuk7
U+54ED.0	fCantonese	huk1
U+54ED.0	fMandarin	ku1
U+54ED.0	fEnglish	 weep; cry; wail; sob
U+54EE.0	fUTF8	哮
U+54EE.0	fCNS11643	1-5460
U+54EE.0	fBig5	ADFD
U+54EE.0	fGB	0-4F78
U+54EE.0	fR/S	 30.7
U+54EE.0	fTotalStrokes	 10
U+54EE.0	fCangjie	RJKD
U+54EE.0	fFourCorner	64047
U+54EE.0	fHakka	  hau1
U+54EE.0	fCantonese	haau1
U+54EE.0	fMandarin	xiao4; xue1; xiao3
U+54EE.0	fEnglish	[1] pant; wheeze; gasp; breathe with difficulty [2] roar; howl
U+54EE.1	fUTF8	哮
U+54EE.1	fCNS11643	1-5460
U+54EE.1	fBig5	ADFD
U+54EE.1	fGB	0-4F78
U+54EE.1	fR/S	 30.7
U+54EE.1	fTotalStrokes	 10
U+54EE.1	fCangjie	RJKD
U+54EE.1	fFourCorner	64047
U+54EE.1	fMandarin	xiao1
U+54EE.1	fEnglish	 cry out in a loud voice
U+54EF.0	fUTF8	哯
U+54EF.0	fCNS11643	3-2F77
U+54EF.0	fGB	3-3558
U+54EF.0	fR/S	  30.7
U+54EF.0	fTotalStrokes	 10
U+54EF.0	fCangjie	RBUU
U+54EF.0	fFourCorner	66010
U+54EF.0	fHakka	 lien3; lien5; hien5; hien6
U+54EF.0	fCantonese	jin6
U+54EF.0	fMandarin	xian4
U+54EF.0	fEnglish	 throw up as a baby its milk
U+54F0.0	fUTF8	哰
U+54F0.0	fCNS11643	3-3021
U+54F0.0	fGB	3-3644
U+54F0.0	fR/S	  30.7
U+54F0.0	fTotalStrokes	 10
U+54F0.0	fCangjie	RJHQ
U+54F0.0	fFourCorner	63050
U+54F0.0	fHakka	  lau2; lo2
U+54F0.0	fCantonese	lou4
U+54F0.0	fMandarin	lao2
U+54F0.0	fEnglish	incoherent chatter
U+54F1.0	fUTF8	哱
U+54F1.0	fCNS11643	2-2E41
U+54F1.0	fBig5	D0FE
U+54F1.0	fGB	3-3633
U+54F1.0	fR/S	 30.7
U+54F1.0	fTotalStrokes	 10
U+54F1.0	fCangjie	RJBD
U+54F1.0	fFourCorner	64047
U+54F1.0	fHakka	  put8; bok8
U+54F1.0	fCantonese	but6; but3; po3
U+54F1.0	fMandarin	bo1; bei4; po4
U+54F1.0	fEnglish	a large trumpet, sounded on going to battle
U+54F2.0	fUTF8	哲
U+54F2.0	fCNS11643	1-5458
U+54F2.0	fBig5	ADF5
U+54F2.0	fGB	0-555C
U+54F2.0	fR/S	 30.7
U+54F2.0	fTotalStrokes	 10
U+54F2.0	fCangjie	QLR
U+54F2.0	fFourCorner	52602
U+54F2.0	fHakka	 zet7; zhat7; zat7; zhet7
U+54F2.0	fCantonese	zit3
U+54F2.0	fMandarin	zhe2
U+54F2.0	fEnglish	[1] sage; thinker; philosopher [2] wise; sagacious; wisdom; sagacity
U+54F3.0	fUTF8	哳
U+54F3.0	fCNS11643	2-2E38
U+54F3.0	fBig5	D0F5
U+54F3.0	fGB	0-5F6E
U+54F3.0	fR/S	 30.7
U+54F3.0	fTotalStrokes	 10
U+54F3.0	fCangjie	RQHL
U+54F3.0	fFourCorner	62021
U+54F3.0	fHakka	 zhat7; zhet7; zat7; zet7
U+54F3.0	fCantonese	zaat3
U+54F3.0	fMandarin	zha1; zha2
U+54F3.0	fEnglish	the crowing of a bird
U+54F4.0	fUTF8	哴
U+54F4.0	fCNS11643	3-2F74
U+54F4.0	fGB	3-3645
U+54F4.0	fR/S	  30.7
U+54F4.0	fTotalStrokes	 10
U+54F4.0	fCangjie	RIAV
U+54F4.0	fFourCorner	63030
U+54F4.0	fHakka	 liong5; long5; liong6; long6
U+54F4.0	fCantonese	loeng6; long4 (also)
U+54F4.0	fMandarin	liang4; lang2
U+54F4.0	fEnglish	crying of infants
U+54F5.0	fUTF8	哵
U+54F5.0	fCNS11643	3-2F78
U+54F5.0	fGB	3-3639
U+54F5.0	fR/S	  30.7
U+54F5.0	fTotalStrokes	 10
U+54F5.0	fCangjie	RRSN
U+54F5.0	fFourCorner	62000
U+54F5.0	fHakka	 bat7; bat8; pat8
U+54F5.0	fCantonese	baat3
U+54F5.0	fMandarin	ba1
U+54F5.0	fEnglish	the cry of a bird
U+54F6.0	fUTF8	哶
U+54F6.0	fCNS11643	3-3022
U+54F6.0	fGB	5-384E
U+54F6.0	fR/S	  30.7
U+54F6.0	fTotalStrokes	 10
U+54F6.0	fCangjie	RTMJ
U+54F6.0	fFourCorner	64040
U+54F6.0	fHakka	 me3;  mi3; me1
U+54F6.0	fCantonese	me1
U+54F6.0	fMandarin	mie1; mie3
U+54F6.0	fEnglish	 cries of sheep; baa; bleat
U+54F7.0	fUTF8	哷
U+54F7.0	fCNS11643	2-2E44
U+54F7.0	fBig5	D142
U+54F7.0	fGB	3-363D
U+54F7.0	fR/S	 30.7
U+54F7.0	fTotalStrokes	 10
U+54F7.0	fCangjie	RBDI
U+54F7.0	fFourCorner	62043
U+54F7.0	fCantonese	lyut3
U+54F7.0	fMandarin	le4
U+54F8.0	fUTF8	哸
U+54F8.0	fCNS11643	2-2E45
U+54F8.0	fBig5	D143
U+54F8.0	fGB	3-363E
U+54F8.0	fR/S	 30.7
U+54F8.0	fTotalStrokes	 10
U+54F8.0	fCangjie	RBV
U+54F8.0	fFourCorner	62047
U+54F8.0	fCantonese	ceoi1; seoi1
U+54F8.0	fMandarin	sui1
U+54F9.0	fUTF8	哹
U+54F9.0	fCNS11643	3-2F79
U+54F9.0	fGB	3-363F
U+54F9.0	fR/S	  30.7
U+54F9.0	fTotalStrokes	 10
U+54F9.0	fCangjie	RBND
U+54F9.0	fFourCorner	62040
U+54F9.0	fCantonese	fau4
U+54F9.0	fMandarin	fou2
U+54FA.0	fUTF8	哺
U+54FA.0	fCNS11643	1-545A
U+54FA.0	fBig5	ADF7
U+54FA.0	fGB	0-3238
U+54FA.0	fR/S	 30.7
U+54FA.0	fTotalStrokes	 10
U+54FA.0	fCangjie	RIJB
U+54FA.0	fFourCorner	63027
U+54FA.0	fHakka	  pu3; bu2; pu6; bu1; pu5
U+54FA.0	fCantonese	bou6; bou1
U+54FA.0	fMandarin	bu3; bu1; bu4; fu3
U+54FA.0	fEnglish	[1] chew (before swallowing) [2] feed (a baby, etc.)
U+54FB.0	fUTF8	哻
U+54FB.0	fCNS11643	2-2E43
U+54FB.0	fBig5	D141
U+54FB.0	fGB	5-384F
U+54FB.0	fR/S	 30.7
U+54FB.0	fTotalStrokes	 10
U+54FB.0	fCangjie	RAMJ
U+54FB.0	fFourCorner	66041
U+54FB.0	fCantonese	hon4
U+54FB.0	fMandarin	han4
U+54FC.0	fUTF8	哼
U+54FC.0	fCNS11643	1-5456
U+54FC.0	fBig5	ADF3
U+54FC.0	fGB	0-3A5F
U+54FC.0	fR/S	 30.7
U+54FC.0	fTotalStrokes	 10
U+54FC.0	fCangjie	RYRN
U+54FC.0	fFourCorner	60027
U+54FC.0	fHakka	 kien1; hen1; hem1; ken1
U+54FC.0	fCantonese	hang1; hng6; hm6 (also); hang6; ng1
U+54FC.0	fMandarin	heng1; hng5
U+54FC.0	fEnglish	[1] croon; hum [2] groan; moan
U+54FD.0	fUTF8	哽
U+54FD.0	fCNS11643	1-5465
U+54FD.0	fBig5	AE43
U+54FD.0	fGB	0-5F6C
U+54FD.0	fR/S	 30.7
U+54FD.0	fTotalStrokes	 10
U+54FD.0	fCangjie	RMLK
U+54FD.0	fFourCorner	61046
U+54FD.0	fHakka	  gang3; gien3; gen3
U+54FD.0	fCantonese	gang2; ang2; ngang2 (also)
U+54FD.0	fMandarin	geng3; ying3
U+54FD.0	fEnglish	 choke; feel a lump in one's throat
U+54FE.0	fUTF8	哾
U+54FE.0	fCNS11643	4-2C29
U+54FE.0	fGB	5-3857
U+54FE.0	fR/S	  30.7
U+54FE.0	fTotalStrokes	 10
U+54FE.0	fCangjie	RCRU
U+54FE.0	fFourCorner	68010
U+54FE.0	fCantonese	syut3
U+54FE.0	fMandarin	shuo1
U+54FF.0	fUTF8	哿
U+54FF.0	fCNS11643	2-2E3B
U+54FF.0	fBig5	D0F8
U+54FF.0	fGB	0-5B41
U+54FF.0	fR/S	 30.7
U+54FF.0	fTotalStrokes	 10
U+54FF.0	fCangjie	KRMNR
U+54FF.0	fFourCorner	46621
U+54FF.0	fHakka	 ko3
U+54FF.0	fCantonese	go2; ho2 (also); go1 (also)
U+54FF.0	fMandarin	ge3; ke3
U+54FF.0	fEnglish	 excellent commend to be able to
U+5500.0	fUTF8	唀
U+5500.0	fCNS11643	3-2F7B
U+5500.0	fGB	E-2450
U+5500.0	fR/S	  30.7
U+5500.0	fTotalStrokes	 10
U+5500.0	fCangjie	RHDS
U+5500.0	fFourCorner	62020
U+5500.0	fCantonese	jau5
U+5500.0	fMandarin	you3; you4
U+5501.0	fUTF8	唁
U+5501.0	fCNS11643	1-5454
U+5501.0	fBig5	ADF1
U+5501.0	fGB	0-5164
U+5501.0	fR/S	 30.7
U+5501.0	fTotalStrokes	 10
U+5501.0	fCangjie	RYMR
U+5501.0	fFourCorner	60061
U+5501.0	fHakka	 ngien5; ngan5; ngan6; ngan3; ngien6; ngian5
U+5501.0	fCantonese	jin6; jin3 (also)
U+5501.0	fMandarin	yan4
U+5501.0	fEnglish	 condole with; express sympathy for
U+5502.0	fUTF8	唂
U+5502.0	fCNS11643	3-2F7A
U+5502.0	fGB	3-363C
U+5502.0	fR/S	 30.7
U+5502.0	fTotalStrokes	 10
U+5502.0	fCangjie	RCOR
U+5502.0	fFourCorner	68060
U+5502.0	fHakka	 guk7; kwuk7
U+5502.0	fCantonese	guk1
U+5502.0	fMandarin	gu3
U+5502.0	fEnglish	[1] the cry of a pheasant [2] [Cantonese be moody [3] used in transliteration
U+5503.0	fUTF8	唃
U+5503.0	fCNS11643	2-2E48
U+5503.0	fBig5	D146
U+5503.0	fGB	3-3640
U+5503.0	fR/S	 30.7
U+5503.0	fTotalStrokes	 10
U+5503.0	fCangjie	RNBG
U+5503.0	fFourCorner	67027
U+5503.0	fCantonese	guk1
U+5503.0	fMandarin	gu3
U+5504.0	fUTF8	唄
U+5504.0	fCNS11643	2-2E3C
U+5504.0	fBig5	D0F9
U+5504.0	fGB	1-5F42
U+5504.0	fR/S	 30.7
U+5504.0	fTotalStrokes	 10
U+5504.0	fCangjie	RBUC
U+5504.0	fFourCorner	66080
U+5504.0	fHakka	  pai5
U+5504.0	fCantonese	baai6; bui6 (also); be6; baa6
U+5504.0	fMandarin	bai4; bei5
U+5504.0	fEnglish	final particle of assertion pathaka
U+5505.0	fUTF8	唅
U+5505.0	fCNS11643	2-2E40
U+5505.0	fBig5	D0FD
U+5505.0	fGB	5-3854
U+5505.0	fR/S	 30.7
U+5505.0	fTotalStrokes	 10
U+5505.0	fCangjie	ROIR
U+5505.0	fFourCorner	68062
U+5505.0	fHakka	 ham2; am3
U+5505.0	fCantonese	ham4; ham3; ham6; ham1; hom1; haam4; ngam4; ngam6; ngan4
U+5505.0	fMandarin	han1; han2; han4
U+5505.0	fEnglish	[1] put in the mouth [2] grunt; grumble [3] take out something [4] feel with the hand
U+5506.0	fUTF8	唆
U+5506.0	fCNS11643	1-5459
U+5506.0	fBig5	ADF6
U+5506.0	fGB	0-4B74
U+5506.0	fR/S	 30.7
U+5506.0	fTotalStrokes	 10
U+5506.0	fCangjie	RICE
U+5506.0	fFourCorner	63047
U+5506.0	fHakka	  so1; so2
U+5506.0	fCantonese	so1
U+5506.0	fMandarin	suo1
U+5506.0	fEnglish	 incite; instigate
U+5507.0	fUTF8	唇
U+5507.0	fCNS11643	1-5464
U+5507.0	fBig5	AE42
U+5507.0	fGB	0-343D
U+5507.0	fR/S	 30.7
U+5507.0	fTotalStrokes	 10
U+5507.0	fCangjie	MVR
U+5507.0	fFourCorner	71603
U+5507.0	fHakka	  sun2;  shun2; shun3
U+5507.0	fCantonese	seon4
U+5507.0	fMandarin	chun2; zhen1; zhen4
U+5507.0	fEnglish	lips
U+5508.0	fUTF8	唈
U+5508.0	fCNS11643	2-2E3D
U+5508.0	fBig5	D0FA
U+5508.0	fGB	3-3638
U+5508.0	fR/S	 30.7
U+5508.0	fTotalStrokes	 10
U+5508.0	fCangjie	RRAU
U+5508.0	fFourCorner	66017
U+5508.0	fHakka	 jip7; rip7
U+5508.0	fCantonese	jap1
U+5508.0	fMandarin	yi4
U+5508.0	fEnglish	 sob
U+5509.0	fUTF8	唉
U+5509.0	fCNS11643	1-545F
U+5509.0	fBig5	ADFC
U+5509.0	fGB	0-3026
U+5509.0	fR/S	 30.7
U+5509.0	fTotalStrokes	 10
U+5509.0	fCangjie	RIOK
U+5509.0	fFourCorner	63034
U+5509.0	fHakka	 ai1; ai2; ngai2; oi1
U+5509.0	fCantonese	aai1; ngaai1 (also); aai6; ngaai6 (also); aai3 (also); ngaai3 (also); oi1; ngoi1
U+5509.0	fMandarin	ai1; ai4; hai1; xi1; yi1; yi3
U+5509.0	fEnglish	alas; oh; well (an interjection of regret or disgust)
U+550A.0	fUTF8	唊
U+550A.0	fCNS11643	2-2E42
U+550A.0	fBig5	D140
U+550A.0	fGB	3-3621
U+550A.0	fR/S	 30.7
U+550A.0	fTotalStrokes	 10
U+550A.0	fCangjie	RKOO
U+550A.0	fFourCorner	64038
U+550A.0	fHakka	 giap8; giap7
U+550A.0	fCantonese	gaap3; him2; haap3
U+550A.0	fMandarin	jia2; qian3
U+550A.0	fEnglish	talkative
U+550B.0	fUTF8	唋
U+550B.0	fCNS11643	2-2E49
U+550B.0	fBig5	D147
U+550B.0	fGB	5-3853
U+550B.0	fR/S	 30.7
U+550B.0	fTotalStrokes	 10
U+550B.0	fCangjie	ROMD
U+550B.0	fFourCorner	68094
U+550B.0	fCantonese	tou1
U+550B.0	fMandarin	tu3
U+550C.0	fUTF8	唌
U+550C.0	fCNS11643	2-3424
U+550C.0	fBig5	D4A1
U+550C.0	fGB	3-362B
U+550C.0	fR/S	 30.7
U+550C.0	fTotalStrokes	 10
U+550C.0	fCangjie	RNKM
U+550C.0	fFourCorner	62031
U+550C.0	fCantonese	jin4
U+550C.0	fMandarin	xian2; yan2; dan4
U+550D.0	fUTF8	唍
U+550D.0	fCNS11643	3-2F73
U+550D.0	fGB	E-2451
U+550D.0	fR/S	 30.7
U+550D.0	fTotalStrokes	 10
U+550D.0	fCangjie	RJMU
U+550D.0	fFourCorner	63010
U+550D.0	fHakka	 wan3; fan3; von3
U+550D.0	fCantonese	waan5
U+550D.0	fMandarin	huan3; wan3
U+550D.0	fEnglish	 look pleased
U+550E.0	fUTF8	唎
U+550E.0	fCNS11643	2-2E47
U+550E.0	fBig5	D145
U+550E.0	fGB	3-363A
U+550E.0	fR/S	 30.7
U+550E.0	fTotalStrokes	 10
U+550E.0	fCangjie	RHDN
U+550E.0	fFourCorner	62000
U+550E.0	fHakka	 li5; li6
U+550E.0	fCantonese	li1; lei6; lei1
U+550E.0	fMandarin	li4; li1
U+550E.0	fEnglish	sound; noise final particle
U+550F.0	fUTF8	唏
U+550F.0	fCNS11643	1-5466
U+550F.0	fBig5	AE44
U+550F.0	fGB	0-5F71
U+550F.0	fR/S	 30.7
U+550F.0	fTotalStrokes	 10
U+550F.0	fCangjie	RKKB
U+550F.0	fFourCorner	64027
U+550F.0	fHakka	 hi1
U+550F.0	fCantonese	hei1
U+550F.0	fMandarin	xi1; xi3; xi4
U+550F.0	fEnglish	 weep with sorrow; sob with sorrow; grieve
U+5510.0	fUTF8	唐
U+5510.0	fCNS11643	1-5453
U+5510.0	fBig5	ADF0
U+5510.0	fGB	0-4C46
U+5510.0	fR/S	 30.7
U+5510.0	fTotalStrokes	 10
U+5510.0	fCangjie	ILR
U+5510.0	fFourCorner	00267
U+5510.0	fHakka	  tong2; tong3
U+5510.0	fCantonese	tong4; tong2 (also)
U+5510.0	fMandarin	tang2
U+5510.0	fEnglish	[1] the Tang dynasty; Chinese [2] a Chinese family name [3] abrupt; rude; preposterous; impertinent [4] Chinese; of China (Hakka)
U+5511.0	fUTF8	唑
U+5511.0	fCNS11643	2-2E3F
U+5511.0	fBig5	D0FC
U+5511.0	fGB	0-5F72
U+5511.0	fR/S	 30.7
U+5511.0	fTotalStrokes	 10
U+5511.0	fCangjie	ROOG
U+5511.0	fFourCorner	68014
U+5511.0	fHakka	 chi3; chi1; cih1; si5; ci3; cih3; chii3
U+5511.0	fCantonese	sai6
U+5511.0	fMandarin	shi4; yi1
U+5511.0	fEnglish	same as &#x566C;
U+5512.0	fUTF8	唒
U+5512.0	fCNS11643	2-2E36
U+5512.0	fBig5	D0F3
U+5512.0	fGB	E-2452
U+5512.0	fR/S	 30.7
U+5512.0	fTotalStrokes	 10
U+5512.0	fCangjie	RMCW
U+5512.0	fFourCorner	61060
U+5512.0	fCantonese	cau4
U+5512.0	fMandarin	qiu2; miu1; qi2
U+5513.0	fUTF8	唓
U+5513.0	fCNS11643	3-2F76
U+5513.0	fGB	3-3556
U+5513.0	fR/S	 30.7
U+5513.0	fTotalStrokes	 10
U+5513.0	fCangjie	RJWJ
U+5513.0	fFourCorner	65000
U+5513.0	fCantonese	ce1
U+5513.0	fMandarin	che1
U+5513.0	fEnglish	interjection
U+5514.0	fUTF8	唔
U+5514.0	fCNS11643	1-545B
U+5514.0	fBig5	ADF8
U+5514.0	fGB	0-5F6D
U+5514.0	fR/S	 30.7
U+5514.0	fTotalStrokes	 10
U+5514.0	fCangjie	RMMR
U+5514.0	fFourCorner	61061
U+5514.0	fHakka	 m1;  ng2; m2; ng3; ng1
U+5514.0	fCantonese	ng4; ngm4 (also); m4; ng2
U+5514.0	fMandarin	wu2; n2; ng2
U+5514.0	fEnglish	[1] hold in mouth; bite [2] not [3] non-classical form of ?;
U+5515.0	fUTF8	唕
U+5515.0	fCNS11643	4-2C28
U+5515.0	fGB	E-2453
U+5515.0	fR/S	  30.7
U+5515.0	fTotalStrokes	 10
U+5515.0	fCangjie	RHAJ
U+5515.0	fFourCorner	66040
U+5515.0	fCantonese	zou6
U+5515.0	fMandarin	zao4
U+5516.0	fUTF8	唖
U+5516.0	fCNS11643	4-2C2E
U+5516.0	fGB	E-2454
U+5516.0	fR/S	 30.7
U+5516.0	fTotalStrokes	 10
U+5516.0	fCangjie	RMLM
U+5516.0	fFourCorner	61010
U+5516.0	fCantonese	aa2; ngaa2 (also)
U+5516.0	fMandarin	ya3; ya1; e4
U+5516.0	fEnglish	 dumb; mute become hoarse
U+5517.0	fUTF8	唗
U+5517.0	fCNS11643	2-2E35
U+5517.0	fBig5	D0F2
U+5517.0	fGB	3-3631
U+5517.0	fR/S	 30.7
U+5517.0	fTotalStrokes	 10
U+5517.0	fCangjie	RGYO
U+5517.0	fFourCorner	64081
U+5517.0	fCantonese	dau1; dau2
U+5517.0	fMandarin	dou3; dou1
U+5518.0	fUTF8	唘
U+5518.0	fCNS11643	3-2F7E
U+5518.0	fGB	E-2455
U+5518.0	fR/S	  30.7
U+5518.0	fTotalStrokes	 10
U+5518.0	fCangjie	MER
U+5518.0	fFourCorner	17600
U+5518.0	fCantonese	kai2
U+5518.0	fMandarin	qi3
U+5519.0	fUTF8	唙
U+5519.0	fCNS11643	3-3025
U+5519.0	fGB	5-3855
U+5519.0	fR/S	 30.7
U+5519.0	fTotalStrokes	 10
U+5519.0	fCangjie	RKHF
U+5519.0	fFourCorner	69080
U+5519.0	fCantonese	dik6; dik1 (also); tik6
U+5519.0	fMandarin	di2
U+551A.0	fUTF8	唚
U+551A.0	fCNS11643	2-2E3A
U+551A.0	fBig5	D0F7
U+551A.0	fGB	3-3646
U+551A.0	fR/S	 30.7
U+551A.0	fTotalStrokes	 10
U+551A.0	fCangjie	RSME
U+551A.0	fFourCorner	67047
U+551A.0	fHakka	  cim3;  zim1
U+551A.0	fCantonese	cam3
U+551A.0	fMandarin	qin4
U+551A.0	fEnglish	[1] use bad language [2] long-winded
U+551B.0	fUTF8	唛
U+551B.0	fCNS11643	F-2954
U+551B.0	fGB	0-5F69
U+551B.0	fR/S	 30.7
U+551B.0	fTotalStrokes	 10
U+551B.0	fCangjie	RQME
U+551B.0	fCantonese	mak1
U+551B.0	fMandarin	ma4; ma3; mo4
U+551B.0	fEnglish	simplified form of &#x561C;
U+551C.0	fUTF8	唜
U+551C.0	fGB	E-2456
U+551C.0	fR/S	 30.7; 75.6
U+551C.0	fTotalStrokes	 10
U+551C.0	fCangjie	DJRP
U+551C.0	fCantonese	mut6
U+551D.0	fUTF8	唝
U+551D.0	fGB	8-7A61
U+551D.0	fR/S	 30.7; 30.10
U+551D.0	fTotalStrokes	 10
U+551D.0	fCangjie	RMBO
U+551D.0	fCantonese	fung2; hung2 (also); gung3
U+551D.0	fMandarin	hong3; gong4
U+551E.0	fUTF8	唞
U+551E.0	fGB	1-7C28
U+551E.0	fR/S	 30.7
U+551E.0	fTotalStrokes	 10
U+551E.0	fCangjie	RQYJ
U+551E.0	fCantonese	tau2
U+551E.0	fMandarin	dou3
U+551E.0	fEnglish	 rest; take a rest
U+551F.0	fUTF8	唟
U+551F.0	fGB	1-7D26
U+551F.0	fR/S	 30.7
U+551F.0	fTotalStrokes	 10
U+551F.0	fCangjie	GIRP
U+551F.0	fCantonese	heoi3
U+551F.0	fEnglish	 place name
U+5520.0	fUTF8	唠
U+5520.0	fCNS11643	F-2E42
U+5520.0	fGB	0-5F6B
U+5520.0	fR/S	 30.7
U+5520.0	fTotalStrokes	 10
U+5520.0	fCangjie	RTBS
U+5520.0	fCantonese	lou4
U+5520.0	fMandarin	lao2; lao4; cha1; chang4; chao1; xiao1
U+5520.0	fEnglish	simplified form of &#x562E;
U+5521.0	fUTF8	唡
U+5521.0	fCNS11643	F-2E4A
U+5521.0	fGB	8-2875
U+5521.0	fR/S	 30.7
U+5521.0	fTotalStrokes	 10
U+5521.0	fCangjie	RMOB
U+5521.0	fCantonese	loeng2; jing1 (also)
U+5521.0	fMandarin	ying1liang3; liang3
U+5521.0	fEnglish	ounce
U+5522.0	fUTF8	唢
U+5522.0	fGB	0-5F6F
U+5522.0	fR/S	 30.7
U+5522.0	fTotalStrokes	 10
U+5522.0	fCangjie	RFBO
U+5522.0	fCantonese	so2
U+5522.0	fMandarin	suo3
U+5522.0	fEnglish	simplified form of &#x55E9;
U+5523.0	fUTF8	唣
U+5523.0	fGB	0-5F70
U+5523.0	fR/S	 30.7
U+5523.0	fTotalStrokes	 10
U+5523.0	fCangjie	RHAP
U+5523.0	fFourCorner	6601
U+5523.0	fCantonese	zou6
U+5523.0	fMandarin	zao4
U+5523.0	fEnglish	chatter
U+5524.0	fUTF8	唤
U+5524.0	fGB	0-3B3D
U+5524.0	fR/S	 30.7
U+5524.0	fTotalStrokes	 10
U+5524.0	fCangjie	XRNBK
U+5524.0	fHakka	 fon5
U+5524.0	fCantonese	wun6
U+5524.0	fMandarin	huan4
U+5524.0	fEnglish	simplified form of &#x559A;
U+5524.1	fUTF8	唤
U+5524.1	fGB	0-3B3D
U+5524.1	fR/S	 30.7
U+5524.1	fTotalStrokes	 10
U+5524.1	fCangjie	XRNBK
U+5524.1	fMandarin	huan5
U+5525.0	fUTF8	唥
U+5525.0	fGB	1-7C27
U+5525.0	fR/S	 30.7
U+5525.0	fTotalStrokes	 10
U+5525.0	fCangjie	RIMI
U+5525.0	fCantonese	laang6; laang4 (also); laang1; lang3
U+5525.0	fEnglish	 bundle
U+5526.0	fUTF8	唦
U+5526.0	fCNS11643	2-2E33
U+5526.0	fBig5	D0F0
U+5526.0	fGB	3-3643
U+5526.0	fR/S	 30.7
U+5526.0	fTotalStrokes	 10
U+5526.0	fCangjie	REFH
U+5526.0	fFourCorner	69020
U+5526.0	fCantonese	saa1
U+5526.0	fMandarin	gou4; sha1
U+5527.0	fUTF8	唧
U+5527.0	fCNS11643	1-5463
U+5527.0	fBig5	AE41
U+5527.0	fGB	0-5F73
U+5527.0	fR/S	 30.7
U+5527.0	fTotalStrokes	 10
U+5527.0	fCangjie	RAIL
U+5527.0	fFourCorner	67020
U+5527.0	fHakka	 zit8
U+5527.0	fCantonese	zik1; zek1; zit1
U+5527.0	fMandarin	ji1; ji2
U+5527.0	fEnglish	 chirping of insects pump
U+5528.0	fUTF8	唨
U+5528.0	fCNS11643	3-6721
U+5528.0	fGB	E-2457
U+5528.0	fR/S	 30.8
U+5528.0	fTotalStrokes	 11
U+5528.0	fCangjie	RNLM
U+5528.0	fCantonese	zo2
U+5528.0	fMandarin	zuo3; zu3
U+5528.0	fEnglish	 used as a sign of past or perfect tense
U+5529.0	fUTF8	唩
U+5529.0	fCNS11643	3-345F
U+5529.0	fGB	3-3655
U+5529.0	fR/S	  30.8
U+5529.0	fTotalStrokes	 11
U+5529.0	fCangjie	RHDV
U+5529.0	fFourCorner	62040
U+5529.0	fCantonese	wo1
U+5529.0	fMandarin	wo1
U+552A.0	fUTF8	唪
U+552A.0	fCNS11643	2-337A
U+552A.0	fBig5	D477
U+552A.0	fGB	0-5F74
U+552A.0	fR/S	 30.8
U+552A.0	fTotalStrokes	 11
U+552A.0	fCangjie	RQKQ
U+552A.0	fFourCorner	65053
U+552A.0	fHakka	  fung3; bung3
U+552A.0	fCantonese	fung2; baang6; bung2; fung6
U+552A.0	fMandarin	feng3; beng3
U+552A.0	fEnglish	 chant; recite
U+552B.0	fUTF8	唫
U+552B.0	fCNS11643	3-3468
U+552B.0	fGB	E-2458
U+552B.0	fR/S	  30.8
U+552B.0	fTotalStrokes	 11
U+552B.0	fCangjie	RC
U+552B.0	fFourCorner	68010
U+552B.0	fHakka	 ngim2; gim1; gim3
U+552B.0	fCantonese	gam3; jam4; ngam4 (also); am4 (also)
U+552B.0	fMandarin	jin4; yin2
U+552B.0	fEnglish	[1] hum; intone; etc. [2] close; shut
U+552C.0	fUTF8	唬
U+552C.0	fCNS11643	1-5949
U+552C.0	fBig5	B0E4
U+552C.0	fGB	0-3B23
U+552C.0	fR/S	 30.8
U+552C.0	fTotalStrokes	 11
U+552C.0	fCangjie	RYPU
U+552C.0	fFourCorner	61014
U+552C.0	fHakka	 hau1;  hak7; ho5; fu2
U+552C.0	fCantonese	fu2; haak3; haau1
U+552C.0	fMandarin	hu3; xia4
U+552C.0	fEnglish	[1] intimidate; scare [2] roar of a tiger
U+552D.0	fUTF8	唭
U+552D.0	fCNS11643	2-342A
U+552D.0	fBig5	D4A7
U+552D.0	fGB	5-3859
U+552D.0	fR/S	 30.8
U+552D.0	fTotalStrokes	 11
U+552D.0	fCangjie	RTMC
U+552D.0	fFourCorner	64081
U+552D.0	fCantonese	kei3; gei1
U+552D.0	fMandarin	qi1
U+552E.0	fUTF8	售
U+552E.0	fCNS11643	1-5947
U+552E.0	fBig5	B0E2
U+552E.0	fGB	0-4A5B
U+552E.0	fR/S	 30.8
U+552E.0	fTotalStrokes	 11
U+552E.0	fCangjie	OGR
U+552E.0	fFourCorner	20601
U+552E.0	fHakka	 su2; chiu2; cu2; chiu5; siu5; shiu6; su5; siu2; shu5
U+552E.0	fCantonese	sau6
U+552E.0	fMandarin	shou4; chou2
U+552E.0	fEnglish	sell
U+552F.0	fUTF8	唯
U+552F.0	fCNS11643	1-5944
U+552F.0	fBig5	B0DF
U+552F.0	fGB	0-4E28
U+552F.0	fR/S	 30.8
U+552F.0	fTotalStrokes	 11
U+552F.0	fCangjie	ROG
U+552F.0	fFourCorner	60014
U+552F.0	fHakka	 wui2; wi2; vui3
U+552F.0	fCantonese	wai4; wai2; wai5
U+552F.0	fMandarin	wei2; shei2
U+552F.0	fEnglish	only
U+552F.1	fUTF8	唯
U+552F.1	fCNS11643	1-5944
U+552F.1	fBig5	B0DF
U+552F.1	fGB	0-4E28
U+552F.1	fR/S	 30.8
U+552F.1	fTotalStrokes	 11
U+552F.1	fCangjie	ROG
U+552F.1	fFourCorner	60014
U+552F.1	fMandarin	wei3
U+552F.1	fEnglish	yes or no; the words of answer
U+5530.0	fUTF8	唰
U+5530.0	fCNS11643	2-3421
U+5530.0	fBig5	D47C
U+5530.0	fGB	0-6027
U+5530.0	fR/S	 30.8
U+5530.0	fTotalStrokes	 11
U+5530.0	fCangjie	RSBN
U+5530.0	fFourCorner	62000
U+5530.0	fHakka	  sot7
U+5530.0	fCantonese	caat3; syut3
U+5530.0	fMandarin	shua1
U+5530.0	fEnglish	 preen (as birds)
U+5531.0	fUTF8	唱
U+5531.0	fCNS11643	1-5940
U+5531.0	fBig5	B0DB
U+5531.0	fGB	0-332A
U+5531.0	fR/S	 30.8
U+5531.0	fTotalStrokes	 11
U+5531.0	fCangjie	RAA
U+5531.0	fFourCorner	66060
U+5531.0	fHakka	  cong5; chong5
U+5531.0	fCantonese	coeng3
U+5531.0	fMandarin	chang4
U+5531.0	fEnglish	[1] sing; chant; [2] row; cry [3] song; singing part of a Chinese opera
U+5532.0	fUTF8	唲
U+5532.0	fCNS11643	2-3425
U+5532.0	fBig5	D4A2
U+5532.0	fGB	3-3656
U+5532.0	fR/S	 30.8
U+5532.0	fTotalStrokes	 11
U+5532.0	fCangjie	RHXU
U+5532.0	fFourCorner	67017
U+5532.0	fHakka	 ji2; ri2; i2
U+5532.0	fCantonese	ji4; waa1; aai1; ngaai1
U+5532.0	fMandarin	er2
U+5532.0	fEnglish	forced laughter
U+5533.0	fUTF8	唳
U+5533.0	fCNS11643	1-594B
U+5533.0	fBig5	B0E6
U+5533.0	fGB	0-6026
U+5533.0	fR/S	 30.8
U+5533.0	fTotalStrokes	 11
U+5533.0	fCangjie	RHSK
U+5533.0	fFourCorner	63034
U+5533.0	fHakka	 lui5; lui6; li3; lui3
U+5533.0	fCantonese	leoi6; lai6; lit6; loei6
U+5533.0	fMandarin	li4
U+5533.0	fEnglish	 cry of crane, wild goose, etc.
U+5534.0	fUTF8	唴
U+5534.0	fCNS11643	2-3379
U+5534.0	fBig5	D476
U+5534.0	fGB	3-3642
U+5534.0	fR/S	 30.8
U+5534.0	fTotalStrokes	 11
U+5534.0	fCangjie	RTGU
U+5534.0	fFourCorner	68011
U+5534.0	fHakka	  giong1; giong3
U+5534.0	fCantonese	goeng6; goeng1
U+5534.0	fMandarin	qiang4
U+5534.0	fEnglish	crying of children
U+5535.0	fUTF8	唵
U+5535.0	fCNS11643	2-337E
U+5535.0	fBig5	D47B
U+5535.0	fGB	3-364E
U+5535.0	fR/S	 30.8
U+5535.0	fTotalStrokes	 11
U+5535.0	fCangjie	RKLU
U+5535.0	fFourCorner	64016
U+5535.0	fHakka	 am3; am1; ram1; jam1; jin2; jam3
U+5535.0	fCantonese	am2; ngam2 (also); jim2
U+5535.0	fMandarin	an3
U+5535.0	fEnglish	[1] syllable [2] feed oneself with the hand; eat with the hand [3] used in Buddhist texts to transliterate [4] syllable
U+5536.0	fUTF8	唶
U+5536.0	fCNS11643	2-337D
U+5536.0	fBig5	D47A
U+5536.0	fGB	3-364A
U+5536.0	fR/S	 30.8
U+5536.0	fTotalStrokes	 11
U+5536.0	fCangjie	RTA
U+5536.0	fFourCorner	64060
U+5536.0	fHakka	  zia5;  za5
U+5536.0	fCantonese	ze3; zaak3; zik1
U+5536.0	fMandarin	jie4; ze4; ji2; ji1
U+5536.0	fEnglish	 sigh; groan loud laughter
U+5537.0	fUTF8	唷
U+5537.0	fCNS11643	1-5455
U+5537.0	fBig5	ADF2
U+5537.0	fGB	0-6021
U+5537.0	fR/S	 30.8
U+5537.0	fTotalStrokes	 11
U+5537.0	fCangjie	RYIB
U+5537.0	fFourCorner	60027
U+5537.0	fHakka	 juk7
U+5537.0	fCantonese	jo1
U+5537.0	fMandarin	yo1
U+5537.0	fEnglish	an exclamation expressing surprise or pain
U+5538.0	fUTF8	唸
U+5538.0	fCNS11643	1-5946
U+5538.0	fBig5	B0E1
U+5538.0	fGB	E-2459
U+5538.0	fR/S	 30.8
U+5538.0	fTotalStrokes	 11
U+5538.0	fCangjie	ROIP
U+5538.0	fFourCorner	68032
U+5538.0	fHakka	 ngiam5; diam5; tien5; ngiam6; tien6; ngian5
U+5538.0	fCantonese	din6; nim6; pan3
U+5538.0	fMandarin	nian4; pen1; pen4
U+5538.0	fEnglish	[1] recite; read; chant [2] same as &#x5674;
U+5539.0	fUTF8	唹
U+5539.0	fCNS11643	2-3428
U+5539.0	fBig5	D4A5
U+5539.0	fGB	5-386B
U+5539.0	fR/S	 30.8
U+5539.0	fTotalStrokes	 11
U+5539.0	fCangjie	RYSY
U+5539.0	fFourCorner	68033
U+5539.0	fHakka	 i1
U+5539.0	fCantonese	jyu1
U+5539.0	fMandarin	yu1; yu2; yo1
U+5539.0	fEnglish	 smile at
U+553A.0	fUTF8	唺
U+553A.0	fCNS11643	3-3466
U+553A.0	fGB	3-3652
U+553A.0	fR/S	  30.8
U+553A.0	fTotalStrokes	 11
U+553A.0	fCangjie	RTBC
U+553A.0	fFourCorner	65080
U+553A.0	fCantonese	tin2
U+553A.0	fMandarin	tian3
U+553B.0	fUTF8	唻
U+553B.0	fCNS11643	2-342B
U+553B.0	fBig5	D4A8
U+553B.0	fGB	3-3635
U+553B.0	fR/S	 30.8
U+553B.0	fTotalStrokes	 11
U+553B.0	fCangjie	RDOO
U+553B.0	fFourCorner	64098
U+553B.0	fHakka	 loi2
U+553B.0	fCantonese	loi6; loi4
U+553B.0	fMandarin	lai3; lai2; lai4
U+553B.0	fEnglish	a sound in singing
U+553C.0	fUTF8	唼
U+553C.0	fCNS11643	2-3376
U+553C.0	fBig5	D473
U+553C.0	fGB	0-5F7E
U+553C.0	fR/S	 30.8
U+553C.0	fTotalStrokes	 11
U+553C.0	fCangjie	RYTV
U+553C.0	fFourCorner	60044
U+553C.0	fHakka	  ciap7; ziap7; sa5
U+553C.0	fCantonese	zaap3 (also); saap3; cip3 (also); zit6; dip6
U+553C.0	fMandarin	sha4; qie4; za1
U+553C.0	fEnglish	speaking evil. gobbling sound made by ducks
U+553D.0	fUTF8	唽
U+553D.0	fCNS11643	4-3077
U+553D.0	fGB	5-385C
U+553D.0	fR/S	 30.8
U+553D.0	fTotalStrokes	 11
U+553D.0	fCangjie	RDHL
U+553D.0	fFourCorner	62020
U+553D.0	fHakka	 sit7; zhat7; zat7
U+553D.0	fCantonese	sik1
U+553D.0	fMandarin	xi2; xi4; xi1
U+553D.0	fEnglish	 twitter
U+553E.0	fUTF8	唾
U+553E.0	fCNS11643	1-5E4E
U+553E.0	fBig5	B3E8
U+553E.0	fGB	0-4D59
U+553E.0	fR/S	 30.8
U+553E.0	fTotalStrokes	 11
U+553E.0	fCangjie	RHJM
U+553E.0	fFourCorner	62014
U+553E.0	fHakka	  tui5; to5;  tio5; pi5; pui5
U+553E.0	fCantonese	to3; toe3 (also); toe5 (also); tou3
U+553E.0	fMandarin	tuo4; tu4
U+553E.0	fEnglish	[1] spit; spit on [2] saliva
U+553F.0	fUTF8	唿
U+553F.0	fCNS11643	3-3467
U+553F.0	fGB	0-5F7C
U+553F.0	fR/S	  30.8
U+553F.0	fTotalStrokes	 11
U+553F.0	fCangjie	RPHP
U+553F.0	fFourCorner	67030
U+553F.0	fCantonese	fat1
U+553F.0	fMandarin	hu1
U+553F.0	fEnglish	sad
U+5540.0	fUTF8	啀
U+5540.0	fCNS11643	2-342C
U+5540.0	fBig5	D4A9
U+5540.0	fGB	3-364D
U+5540.0	fR/S	 30.8
U+5540.0	fTotalStrokes	 11
U+5540.0	fCangjie	RMGG
U+5540.0	fFourCorner	61014
U+5540.0	fHakka	  ngai2; ngat7
U+5540.0	fCantonese	ngaai4; aai4 (also)
U+5540.0	fMandarin	ai2; yai2
U+5540.0	fEnglish	gnaw
U+5541.0	fUTF8	啁
U+5541.0	fCNS11643	1-594C
U+5541.0	fBig5	B0E7
U+5541.0	fGB	0-5F7A
U+5541.0	fR/S	 30.8
U+5541.0	fTotalStrokes	 11
U+5541.0	fCangjie	RBGR
U+5541.0	fFourCorner	67020
U+5541.0	fHakka	 zau1; zhiu1; zu1
U+5541.0	fCantonese	zau1; zaau1
U+5541.0	fMandarin	zhao1; zhou1; tiao2; obs; diao4; dao1; zou1
U+5541.0	fEnglish	the chirping of a bird; twittering of a bird
U+5542.0	fUTF8	啂
U+5542.0	fCNS11643	4-3072
U+5542.0	fGB	5-3867
U+5542.0	fR/S	  30.8
U+5542.0	fTotalStrokes	 11
U+5542.0	fCangjie	RBDU
U+5542.0	fFourCorner	62010
U+5542.0	fCantonese	nau5; lau5 (also)
U+5542.0	fMandarin	nou4
U+5543.0	fUTF8	啃
U+5543.0	fCNS11643	1-593E
U+5543.0	fBig5	B0D9
U+5543.0	fGB	0-3F50
U+5543.0	fR/S	 30.8
U+5543.0	fTotalStrokes	 11
U+5543.0	fCangjie	RYMB
U+5543.0	fFourCorner	61027
U+5543.0	fHakka	 sep7; ken3
U+5543.0	fCantonese	hang2; han2; hang3; kang2
U+5543.0	fMandarin	ken3; ken4
U+5543.0	fEnglish	 gnaw; bite; nibble
U+5544.0	fUTF8	啄
U+5544.0	fCNS11643	1-593B
U+5544.0	fBig5	B0D6
U+5544.0	fGB	0-5744
U+5544.0	fR/S	 30.8
U+5544.0	fTotalStrokes	 11
U+5544.0	fCangjie	RMSO
U+5544.0	fFourCorner	61032
U+5544.0	fHakka	 duk7;  dok7; dui3; duk8
U+5544.0	fCantonese	doek3; doeng1 (also); zau3
U+5544.0	fMandarin	zhuo2; zhou4
U+5544.0	fEnglish	 peck (said of bird)
U+5545.0	fUTF8	啅
U+5545.0	fCNS11643	2-3423
U+5545.0	fBig5	D47E
U+5545.0	fGB	3-364F
U+5545.0	fR/S	 30.8
U+5545.0	fTotalStrokes	 11
U+5545.0	fCangjie	RYAJ
U+5545.0	fFourCorner	61046
U+5545.0	fHakka	 dok7; zok7
U+5545.0	fCantonese	doek3
U+5545.0	fMandarin	zhuo2
U+5545.0	fEnglish	a multitude of people disputing
U+5546.0	fUTF8	商
U+5546.0	fCNS11643	1-5938
U+5546.0	fBig5	B0D3
U+5546.0	fGB	0-494C
U+5546.0	fR/S	 30.8
U+5546.0	fTotalStrokes	 11
U+5546.0	fCangjie	YCBR
U+5546.0	fFourCorner	00227
U+5546.0	fHakka	  song1; shong1; dit7
U+5546.0	fCantonese	soeng1
U+5546.0	fMandarin	shang1
U+5546.0	fEnglish	[1] commerce; business; trade [2] merchant; trader; business man [3] discuss; exchange views; confer
U+5547.0	fUTF8	啇
U+5547.0	fCNS11643	4-3074
U+5547.0	fGB	5-3352
U+5547.0	fR/S	  30.8
U+5547.0	fTotalStrokes	 11
U+5547.0	fCangjie	YCBR
U+5547.0	fFourCorner	00220
U+5547.0	fHakka	  dit7; tit8
U+5547.0	fCantonese	dik1
U+5547.0	fMandarin	di2; di4; shi4
U+5547.0	fEnglish	 stalk the stem the foot the base
U+5548.0	fUTF8	啈
U+5548.0	fCNS11643	2-3429
U+5548.0	fBig5	D4A6
U+5548.0	fGB	3-3648
U+5548.0	fR/S	 30.8
U+5548.0	fTotalStrokes	 11
U+5548.0	fCangjie	RGTJ
U+5548.0	fFourCorner	64041
U+5548.0	fCantonese	hang1; hng6; hm6 (also)
U+5548.0	fMandarin	heng4
U+5549.0	fUTF8	啉
U+5549.0	fCNS11643	3-345B
U+5549.0	fGB	0-5F78
U+5549.0	fR/S	  30.8
U+5549.0	fTotalStrokes	 11
U+5549.0	fCangjie	RDD
U+5549.0	fFourCorner	64090
U+5549.0	fHakka	  lam2; lim2; lim1
U+5549.0	fCantonese	laam4; lam4; lam1
U+5549.0	fMandarin	lan2; lin2; lin4
U+5549.0	fEnglish	 stupid slow
U+554A.0	fUTF8	啊
U+554A.0	fCNS11643	1-593F
U+554A.0	fBig5	B0DA
U+554A.0	fGB	0-3021
U+554A.0	fR/S	 30.8
U+554A.0	fTotalStrokes	 11
U+554A.0	fCangjie	RNLR
U+554A.0	fFourCorner	61020
U+554A.0	fHakka	  o1;  a1
U+554A.0	fCantonese	aa1; aa2; ngaa1 (also); aa3; ngaa2 (also); aa6; aa4; ngaa6 (also); ngaa3 (also); haa2; haa4; ngaa4 (also)
U+554A.0	fMandarin	a1; a2; a3; a4
U+554A.0	fEnglish	[1] an exclamatory particle [2] oh
U+554A.1	fUTF8	啊
U+554A.1	fCNS11643	1-593F
U+554A.1	fBig5	B0DA
U+554A.1	fGB	0-3021
U+554A.1	fR/S	 30.8
U+554A.1	fTotalStrokes	 11
U+554A.1	fCangjie	RNLR
U+554A.1	fFourCorner	61020
U+554A.1	fMandarin	a5
U+554A.1	fEnglish	a phrase-final particle, used generally with interrogations
U+554B.0	fUTF8	啋
U+554B.0	fCNS11643	2-342D
U+554B.0	fBig5	D4AA
U+554B.0	fGB	5-3866
U+554B.0	fR/S	 30.8
U+554B.0	fTotalStrokes	 11
U+554B.0	fCangjie	RBD
U+554B.0	fFourCorner	62094
U+554B.0	fHakka	 cai1; hiau1; hieu1
U+554B.0	fCantonese	coi1
U+554B.0	fMandarin	xiao1; cai1
U+554B.0	fEnglish	an interjection
U+554C.0	fUTF8	啌
U+554C.0	fCNS11643	3-3458
U+554C.0	fGB	3-365D
U+554C.0	fR/S	 30.8
U+554C.0	fTotalStrokes	 11
U+554C.0	fCangjie	RJCM
U+554C.0	fFourCorner	63010
U+554C.0	fHakka	 kiong1; kung5
U+554C.0	fCantonese	hong1
U+554C.0	fMandarin	xiang1; qiang1
U+554C.0	fEnglish	animal disease
U+554D.0	fUTF8	啍
U+554D.0	fCNS11643	2-3377
U+554D.0	fBig5	D474
U+554D.0	fGB	3-3659
U+554D.0	fR/S	 30.8
U+554D.0	fTotalStrokes	 11
U+554D.0	fCangjie	RYRD
U+554D.0	fFourCorner	60047
U+554D.0	fHakka	 zun1; zhun1; dun1
U+554D.0	fCantonese	teon1; zeon1; tan1
U+554D.0	fMandarin	tun1; tui1; tun2; xiang1; zhun1
U+554D.0	fEnglish	creak
U+554E.0	fUTF8	啎
U+554E.0	fCNS11643	2-3427
U+554E.0	fBig5	D4A4
U+554E.0	fGB	E-245A
U+554E.0	fR/S	 30.8
U+554E.0	fTotalStrokes	 11
U+554E.0	fCangjie	OJMMR
U+554E.0	fFourCorner	81461
U+554E.0	fHakka	 ng3; ngu5; ngu6; ng2
U+554E.0	fCantonese	ng5 (also); ng6
U+554E.0	fMandarin	wu3
U+554E.0	fEnglish	obstinate; disobedient; intractable
U+554F.0	fUTF8	問
U+554F.0	fCNS11643	1-5942
U+554F.0	fBig5	B0DD
U+554F.0	fGB	1-4E4A
U+554F.0	fR/S	 30.8
U+554F.0	fTotalStrokes	 11
U+554F.0	fCangjie	ANR
U+554F.0	fFourCorner	77607
U+554F.0	fHakka	  mun5
U+554F.0	fCantonese	man6
U+554F.0	fMandarin	wen4
U+554F.0	fEnglish	[1] ask; inquire [2] ask about; inquire after [3] interrogate; examine [4] hold responsible
U+554F.1	fUTF8	問
U+554F.1	fCNS11643	1-5942
U+554F.1	fBig5	B0DD
U+554F.1	fGB	1-4E4A
U+554F.1	fR/S	 30.8
U+554F.1	fTotalStrokes	 11
U+554F.1	fCangjie	ANR
U+554F.1	fFourCorner	77607
U+554F.1	fMandarin	wen5
U+5550.0	fUTF8	啐
U+5550.0	fCNS11643	2-3378
U+5550.0	fBig5	D475
U+5550.0	fGB	0-5F7D
U+5550.0	fR/S	 30.8
U+5550.0	fTotalStrokes	 11
U+5550.0	fCangjie	RYOJ
U+5550.0	fFourCorner	60048
U+5550.0	fHakka	 cui5;  zut7; chut8; zut8
U+5550.0	fCantonese	ceoi3; seoi6
U+5550.0	fMandarin	cui4; shui4; sui4
U+5550.0	fEnglish	[1] taste; sip [2] spit
U+5550.1	fUTF8	啐
U+5550.1	fCNS11643	2-3378
U+5550.1	fBig5	D475
U+5550.1	fGB	0-5F7D
U+5550.1	fR/S	 30.8
U+5550.1	fTotalStrokes	 11
U+5550.1	fCangjie	RYOJ
U+5550.1	fFourCorner	60048
U+5550.1	fMandarin	q5; qi5
U+5550.1	fEnglish	fie! (an interjection expressing contempt)
U+5551.0	fUTF8	啑
U+5551.0	fCNS11643	2-337B
U+5551.0	fBig5	D478
U+5551.0	fGB	E-245B
U+5551.0	fR/S	 30.8
U+5551.0	fTotalStrokes	 11
U+5551.0	fCangjie	RJLO
U+5551.0	fFourCorner	65081
U+5551.0	fHakka	 ciap8
U+5551.0	fCantonese	saap3; zaap3 (also); zit6; cip3 (also); dip6; tip6
U+5551.0	fMandarin	sha4; jie1; cui4; die2
U+5551.0	fEnglish	[1] the noise of birds feeding [2] speak evil
U+5552.0	fUTF8	啒
U+5552.0	fCNS11643	2-3422
U+5552.0	fBig5	D47D
U+5552.0	fGB	5-386E
U+5552.0	fR/S	 30.8
U+5552.0	fTotalStrokes	 11
U+5552.0	fCangjie	RSUU
U+5552.0	fFourCorner	67077
U+5552.0	fCantonese	gwat1; fat1
U+5552.0	fMandarin	hu1
U+5553.0	fUTF8	啓
U+5553.0	fCNS11643	3-3462
U+5553.0	fGB	1-4674
U+5553.0	fR/S	 30.8
U+5553.0	fTotalStrokes	 11
U+5553.0	fCangjie	IKR
U+5553.0	fFourCorner	38600
U+5553.0	fHakka	  ki3
U+5553.0	fCantonese	kai2
U+5553.0	fMandarin	qi3
U+5553.0	fEnglish	[1] open [2] begin; start; commence [3] explain [4] inform; state [5] letter
U+5554.0	fUTF8	啔
U+5554.0	fCNS11643	3-3461
U+5554.0	fGB	5-386D
U+5554.0	fR/S	 30.8
U+5554.0	fTotalStrokes	 11
U+5554.0	fCangjie	HIR
U+5554.0	fFourCorner	33600
U+5554.0	fCantonese	kai2
U+5554.0	fMandarin	qi3
U+5555.0	fUTF8	啕
U+5555.0	fCNS11643	1-5943
U+5555.0	fBig5	B0DE
U+5555.0	fGB	0-5F7B
U+5555.0	fR/S	 30.8
U+5555.0	fTotalStrokes	 11
U+5555.0	fCangjie	RPOU
U+5555.0	fFourCorner	67020
U+5555.0	fHakka	 tau2; to2; rau2; jeu2
U+5555.0	fCantonese	tou4
U+5555.0	fMandarin	tao2
U+5555.0	fEnglish	wail
U+5556.0	fUTF8	啖
U+5556.0	fCNS11643	1-5941
U+5556.0	fBig5	B0DC
U+5556.0	fGB	0-6022
U+5556.0	fR/S	 30.8
U+5556.0	fTotalStrokes	 11
U+5556.0	fCangjie	RFF
U+5556.0	fFourCorner	69089
U+5556.0	fHakka	 tam3; jam3; tam6; tam5; tam1; ram3; tan2
U+5556.0	fCantonese	daam6
U+5556.0	fMandarin	dan4; dan1
U+5556.0	fEnglish	[1] eat (a literary expression) [2] feed
U+5557.0	fUTF8	啗
U+5557.0	fCNS11643	1-594D
U+5557.0	fBig5	B0E8
U+5557.0	fGB	8-2C7D
U+5557.0	fR/S	 30.8
U+5557.0	fTotalStrokes	 11
U+5557.0	fCangjie	RNHX
U+5557.0	fFourCorner	67077
U+5557.0	fHakka	 ham3; tam3; tam6; tam5
U+5557.0	fCantonese	daam6
U+5557.0	fMandarin	dan4
U+5557.0	fEnglish	 eat; chew; bite entice; lure
U+5558.0	fUTF8	啘
U+5558.0	fCNS11643	3-3457
U+5558.0	fGB	3-365E
U+5558.0	fR/S	  30.8
U+5558.0	fTotalStrokes	 11
U+5558.0	fCangjie	RJNU
U+5558.0	fFourCorner	63010
U+5558.0	fCantonese	waa1
U+5558.0	fMandarin	ye4; wa1
U+5559.0	fUTF8	啙
U+5559.0	fCNS11643	3-3A36
U+5559.0	fGB	3-4679
U+5559.0	fR/S	  30.9; 30.8
U+5559.0	fTotalStrokes	 12
U+5559.0	fCangjie	YPRR
U+5559.0	fFourCorner	21660
U+5559.0	fHakka	 zih3; zih1; zhi1; zhi3
U+5559.0	fCantonese	zi2
U+5559.0	fMandarin	zi3
U+5559.0	fEnglish	poor
U+555A.0	fUTF8	啚
U+555A.0	fCNS11643	3-3464
U+555A.0	fGB	5-3869
U+555A.0	fR/S	  30.8
U+555A.0	fTotalStrokes	 11
U+555A.0	fCangjie	RYWR
U+555A.0	fFourCorner	60600
U+555A.0	fCantonese	pei2; tou4
U+555A.0	fMandarin	tu2; bi3
U+555B.0	fUTF8	啛
U+555B.0	fCNS11643	4-306B
U+555B.0	fGB	5-385F
U+555B.0	fR/S	 30.8
U+555B.0	fTotalStrokes	 11
U+555B.0	fCangjie	RJLV
U+555B.0	fFourCorner	65040
U+555B.0	fCantonese	ceoi3
U+555B.0	fMandarin	cui4
U+555C.0	fUTF8	啜
U+555C.0	fCNS11643	1-5948
U+555C.0	fBig5	B0E3
U+555C.0	fGB	0-6028
U+555C.0	fR/S	 30.8
U+555C.0	fTotalStrokes	 11
U+555C.0	fCangjie	REEE
U+555C.0	fFourCorner	67047
U+555C.0	fHakka	 ch'ot7; zot7;  chot7; cot7; sot7; ziot7; ziot8
U+555C.0	fCantonese	zyut3; zyut1 (also); ceoi3; cyut3
U+555C.0	fMandarin	chuo4; chuai4; cui4; rui4; zhui4
U+555C.0	fEnglish	[1] sip; drink [2] sob; weep; cry in a subdued manner
U+555D.0	fUTF8	啝
U+555D.0	fCNS11643	3-345E
U+555D.0	fGB	5-3864
U+555D.0	fR/S	  30.8
U+555D.0	fTotalStrokes	 11
U+555D.0	fCangjie	RHDR
U+555D.0	fFourCorner	66000
U+555D.0	fCantonese	wo3; wo4; wo6
U+555D.0	fMandarin	he2
U+555D.0	fEnglish	final partical expressing surprise
U+555E.0	fUTF8	啞
U+555E.0	fCNS11643	1-593C
U+555E.0	fBig5	B0D7
U+555E.0	fGB	1-5146
U+555E.0	fR/S	 30.8
U+555E.0	fTotalStrokes	 11
U+555E.0	fCangjie	RMLM
U+555E.0	fFourCorner	61017
U+555E.0	fHakka	  a3; a1; e3
U+555E.0	fCantonese	aa2; aa1; ngaa2 (also); ak1; ngak1 (also); ngaa1 (also); ze3
U+555E.0	fMandarin	ya3; ya1; e4; ya5
U+555E.0	fEnglish	[1] dumb; mute [2] hoarse; husky [3] a phrase-final particle
U+555F.0	fUTF8	啟
U+555F.0	fCNS11643	1-5A76
U+555F.0	fBig5	B1D2
U+555F.0	fGB	E-245C
U+555F.0	fR/S	 30.8; 66.7
U+555F.0	fTotalStrokes	 11
U+555F.0	fCangjie	HROK
U+555F.0	fFourCorner	38604
U+555F.0	fHakka	  ki3
U+555F.0	fCantonese	kai2
U+555F.0	fMandarin	qi3
U+555F.0	fEnglish	 open begin
U+5560.0	fUTF8	啠
U+5560.0	fCNS11643	4-307A
U+5560.0	fGB	E-245D
U+5560.0	fR/S	 30.8
U+5560.0	fTotalStrokes	 11
U+5560.0	fCangjie	HLR
U+5560.0	fFourCorner	72600
U+5560.0	fCantonese	zit3
U+5560.0	fMandarin	zhe2
U+5561.0	fUTF8	啡
U+5561.0	fCNS11643	1-593D
U+5561.0	fBig5	B0D8
U+5561.0	fGB	0-3748
U+5561.0	fR/S	 30.8
U+5561.0	fTotalStrokes	 11
U+5561.0	fCangjie	RLMY
U+5561.0	fFourCorner	61011
U+5561.0	fHakka	 fui1; pai5; fi1; pai2
U+5561.0	fCantonese	fe1; fei1 (also); pei1
U+5561.0	fMandarin	fei1; pei1
U+5561.0	fEnglish	used in transliteration (as in morphine, coffee)
U+5562.0	fUTF8	啢
U+5562.0	fCNS11643	2-337C
U+5562.0	fBig5	D479
U+5562.0	fGB	E-245E
U+5562.0	fR/S	 30.8
U+5562.0	fTotalStrokes	 11
U+5562.0	fCangjie	RMLB
U+5562.0	fFourCorner	61027
U+5562.0	fCantonese	loeng2; jing1 (also); loeng5
U+5562.0	fMandarin	liang3; liang4
U+5562.0	fEnglish	ounce
U+5563.0	fUTF8	啣
U+5563.0	fCNS11643	1-594A
U+5563.0	fBig5	B0E5
U+5563.0	fGB	E-245F
U+5563.0	fR/S	 30.8
U+5563.0	fTotalStrokes	 11
U+5563.0	fCangjie	ROML
U+5563.0	fFourCorner	67020
U+5563.0	fHakka	 ham2
U+5563.0	fCantonese	haam4; ham4 (also)
U+5563.0	fMandarin	xian2
U+5563.0	fEnglish	[1] hold in the mouth [2] bridle [3] harbour
U+5564.0	fUTF8	啤
U+5564.0	fCNS11643	1-5945
U+5564.0	fBig5	B0E0
U+5564.0	fGB	0-4621
U+5564.0	fR/S	 30.8
U+5564.0	fTotalStrokes	 11
U+5564.0	fCangjie	RHWJ
U+5564.0	fFourCorner	66040
U+5564.0	fHakka	 bia1; pi2
U+5564.0	fCantonese	be1; bi1; bi4 (also); pe1; pei4
U+5564.0	fMandarin	pi2
U+5564.0	fEnglish	used in transliteration (beer, pear, baby)
U+5565.0	fUTF8	啥
U+5565.0	fCNS11643	2-3426
U+5565.0	fBig5	D4A3
U+5565.0	fGB	0-4936
U+5565.0	fR/S	 30.8
U+5565.0	fTotalStrokes	 11
U+5565.0	fCangjie	ROMR
U+5565.0	fFourCorner	68064
U+5565.0	fHakka	 sa5
U+5565.0	fCantonese	saa2; saa1
U+5565.0	fMandarin	sha4
U+5565.0	fEnglish	what?
U+5566.0	fUTF8	啦
U+5566.0	fCNS11643	1-593A
U+5566.0	fBig5	B0D5
U+5566.0	fGB	0-4032
U+5566.0	fR/S	 30.8
U+5566.0	fTotalStrokes	 11
U+5566.0	fCangjie	RQYT
U+5566.0	fFourCorner	60018
U+5566.0	fHakka	 la1
U+5566.0	fCantonese	laa1; laa2 (also); laa3 (also); laa4 (also)
U+5566.0	fMandarin	la1
U+5566.0	fEnglish	a phrase-final particle
U+5566.1	fUTF8	啦
U+5566.1	fCNS11643	1-593A
U+5566.1	fBig5	B0D5
U+5566.1	fGB	0-4032
U+5566.1	fR/S	 30.8
U+5566.1	fTotalStrokes	 11
U+5566.1	fCangjie	RQYT
U+5566.1	fFourCorner	60018
U+5566.1	fMandarin	la5
U+5567.0	fUTF8	啧
U+5567.0	fGB	0-5F75
U+5567.0	fR/S	 30.8
U+5567.0	fTotalStrokes	 11
U+5567.0	fCangjie	RQMO
U+5567.0	fHakka	 zit7
U+5567.0	fCantonese	zaak3; zik1 (also)
U+5567.0	fMandarin	ze2
U+5567.0	fEnglish	simplified form of &#x5616;
U+5568.0	fUTF8	啨
U+5568.0	fCNS11643	3-3459
U+5568.0	fGB	3-3647
U+5568.0	fR/S	 30.8
U+5568.0	fTotalStrokes	 11
U+5568.0	fCangjie	RQMB
U+5568.0	fFourCorner	65020
U+5568.0	fCantonese	jing1
U+5568.0	fMandarin	qing1
U+5569.0	fUTF8	啩
U+5569.0	fGB	1-7C2A
U+5569.0	fR/S	 30.8
U+5569.0	fTotalStrokes	 11
U+5569.0	fCangjie	RGGY
U+5569.0	fCantonese	gwaa3
U+5569.0	fMandarin	gua4
U+5569.0	fEnglish	 particle implying probability, uncertainty or feeling doubt
U+556A.0	fUTF8	啪
U+556A.0	fCNS11643	1-5939
U+556A.0	fBig5	B0D4
U+556A.0	fGB	0-453E
U+556A.0	fR/S	 30.8
U+556A.0	fTotalStrokes	 11
U+556A.0	fCangjie	RQHA
U+556A.0	fFourCorner	66000
U+556A.0	fHakka	 bak8; pak7; biak8
U+556A.0	fCantonese	paak1; paak2 (also); paak4 (also); paa1
U+556A.0	fMandarin	pa1; pai1
U+556A.0	fEnglish	syllable
U+556B.0	fUTF8	啫
U+556B.0	fCNS11643	3-6457
U+556B.0	fGB	1-7C2F
U+556B.0	fR/S	 30.9
U+556B.0	fTotalStrokes	 12
U+556B.0	fCangjie	RJKA
U+556B.0	fFourCorner	6406
U+556B.0	fCantonese	zek1; ze1; zoe1; ze3
U+556B.0	fMandarin	zhe3
U+556B.0	fEnglish	 interjection of warning
U+556C.0	fUTF8	啬
U+556C.0	fGB	0-5844
U+556C.0	fR/S	 24.9; 30.8
U+556C.0	fTotalStrokes	 11
U+556C.0	fCangjie	GCWR
U+556C.0	fHakka	 sep7; sit7
U+556C.0	fCantonese	sik1
U+556C.0	fMandarin	se4
U+556C.0	fEnglish	simplified form of &#x55C7;
U+556D.0	fUTF8	啭
U+556D.0	fGB	0-5F79
U+556D.0	fR/S	 30.8
U+556D.0	fTotalStrokes	 11
U+556D.0	fCangjie	RKQI
U+556D.0	fHakka	 zhon5
U+556D.0	fCantonese	zyun2
U+556D.0	fMandarin	zhuan4; zhuan3
U+556D.0	fEnglish	simplified form of &#x56C0;
U+556E.0	fUTF8	啮
U+556E.0	fGB	0-4476
U+556E.0	fR/S	 30.8
U+556E.0	fTotalStrokes	 11
U+556E.0	fCangjie	XRYMU
U+556E.0	fHakka	 ngat7; ngau1
U+556E.0	fCantonese	ngit6; jit6 (also); it6 (also); ngaat6; aat6 (also); ngat6
U+556E.0	fMandarin	nie4
U+556E.0	fEnglish	simplified form of &#x5699;
U+556F.0	fUTF8	啯
U+556F.0	fGB	8-7A63
U+556F.0	fR/S	 30.8; 30.11
U+556F.0	fTotalStrokes	 11
U+556F.0	fCangjie	RWMI
U+556F.0	fCantonese	gwok3; gok3 (also)
U+556F.0	fMandarin	guo1; guo5
U+5570.0	fUTF8	啰
U+5570.0	fGB	8-2F7C
U+5570.0	fR/S	 30.8; 122.3
U+5570.0	fTotalStrokes	 11
U+5570.0	fCangjie	RWLN
U+5570.0	fCantonese	lo1; lo4; lo3
U+5570.0	fMandarin	luo1; luo2; luo5
U+5571.0	fUTF8	啱
U+5571.0	fCNS11643	3-6456
U+5571.0	fGB	1-7C2C
U+5571.0	fR/S	 30.8
U+5571.0	fTotalStrokes	 11
U+5571.0	fCangjie	RUMR
U+5571.0	fFourCorner	6206
U+5571.0	fHakka	 ngam1
U+5571.0	fCantonese	ngaam1; aam1 (also); ngam1; am1 (also)
U+5571.0	fMandarin	yan1
U+5571.0	fEnglish	[1] [2] be suitable for [3] correct; right; suitable; fit
U+5572.0	fUTF8	啲
U+5572.0	fCNS11643	3-6722
U+5572.0	fGB	1-7C2D
U+5572.0	fR/S	 30.8
U+5572.0	fTotalStrokes	 11
U+5572.0	fCangjie	RHAI
U+5572.0	fCantonese	di1; dit1; di4; ti4
U+5572.0	fMandarin	di4
U+5572.0	fEnglish	[1] [Cantonese, Hakka] a few; bit [2] used as a prefix to countable nouns to denote the plural number, similar in meaning to the English "some" [3] used as the English definite article "the" when follow by uncountable nouns or countable nouns in the lural number [4] used as a suffix to adjectives to denote the comparative
U+5573.0	fUTF8	啳
U+5573.0	fCNS11643	3-6455
U+5573.0	fGB	E-2460
U+5573.0	fR/S	 30.8
U+5573.0	fTotalStrokes	 11
U+5573.0	fCangjie	RFQU
U+5573.0	fFourCorner	6901
U+5573.0	fCantonese	kyun4
U+5573.0	fMandarin	quan2; jue2
U+5574.0	fUTF8	啴
U+5574.0	fGB	8-2C38
U+5574.0	fR/S	 30.8
U+5574.0	fTotalStrokes	 11
U+5574.0	fCangjie	RCWJ
U+5574.0	fCantonese	taan1; cin2; zin2 (also)
U+5574.0	fMandarin	tan1; chan3
U+5574.0	fEnglish	pant; many
U+5575.0	fUTF8	啵
U+5575.0	fCNS11643	2-3374
U+5575.0	fBig5	D471
U+5575.0	fGB	0-6023
U+5575.0	fR/S	 30.8
U+5575.0	fTotalStrokes	 11
U+5575.0	fCangjie	REDE
U+5575.0	fFourCorner	64047
U+5575.0	fCantonese	bo1 (also); bo3
U+5575.0	fMandarin	bo5; bo1
U+5576.0	fUTF8	啶
U+5576.0	fCNS11643	2-3375
U+5576.0	fBig5	D472
U+5576.0	fGB	0-6024
U+5576.0	fR/S	 30.8
U+5576.0	fTotalStrokes	 11
U+5576.0	fCangjie	RJMO
U+5576.0	fFourCorner	63081
U+5576.0	fCantonese	ding6
U+5576.0	fMandarin	ding4
U+5577.0	fUTF8	啷
U+5577.0	fCNS11643	2-3A4E
U+5577.0	fBig5	D86A
U+5577.0	fGB	0-6025
U+5577.0	fR/S	 30.8
U+5577.0	fTotalStrokes	 11
U+5577.0	fCangjie	RIIL
U+5577.0	fFourCorner	67027
U+5577.0	fCantonese	long1; long4
U+5577.0	fMandarin	lang1; lang2
U+5578.0	fUTF8	啸
U+5578.0	fGB	0-5025
U+5578.0	fR/S	 30.8
U+5578.0	fTotalStrokes	 11
U+5578.0	fCangjie	XRLX
U+5578.0	fHakka	 sau5; siau5
U+5578.0	fCantonese	siu3
U+5578.0	fMandarin	xiao4; chi4; su4
U+5578.0	fEnglish	simplified form of &#x562F;
U+5579.0	fUTF8	啹
U+5579.0	fGB	1-7C2E
U+5579.0	fR/S	 30.8
U+5579.0	fTotalStrokes	 11
U+5579.0	fCangjie	RSJR
U+5579.0	fCantonese	goe1; koe1 (also); geoi1
U+5579.0	fEnglish	stupid, idiotic; to kill, slaughter; to roll or crumple into a ball; phonetic
U+557A.0	fUTF8	啺
U+557A.0	fCNS11643	3-3A3C
U+557A.0	fGB	5-376F
U+557A.0	fR/S	  30.9
U+557A.0	fTotalStrokes	 12
U+557A.0	fCangjie	RAMH
U+557A.0	fFourCorner	66020
U+557A.0	fHakka	 sit7
U+557A.0	fCantonese	tong4
U+557A.0	fMandarin	tang2
U+557A.0	fEnglish	a bird's notes
U+557B.0	fUTF8	啻
U+557B.0	fCNS11643	1-5E3D
U+557B.0	fBig5	B3D7
U+557B.0	fGB	0-6034
U+557B.0	fR/S	 30.9
U+557B.0	fTotalStrokes	 12
U+557B.0	fCangjie	YBR
U+557B.0	fFourCorner	00602
U+557B.0	fHakka	  di5; ti5; chii3; chi3
U+557B.0	fCantonese	ci3
U+557B.0	fMandarin	chi4; di4; ti4
U+557B.0	fEnglish	[1] only; merely [2] used in combination
U+557C.0	fUTF8	啼
U+557C.0	fCNS11643	1-5E40
U+557C.0	fBig5	B3DA
U+557C.0	fGB	0-4C64
U+557C.0	fR/S	 30.9
U+557C.0	fTotalStrokes	 12
U+557C.0	fCangjie	RYBB
U+557C.0	fFourCorner	60027
U+557C.0	fHakka	  tai2;  ti2; tai3; ti5; t'e2
U+557C.0	fCantonese	tai4
U+557C.0	fMandarin	ti2; xin1
U+557C.0	fEnglish	 crow; cry (especially said of birds)
U+557D.0	fUTF8	啽
U+557D.0	fCNS11643	2-3A59
U+557D.0	fBig5	D875
U+557D.0	fGB	3-366F
U+557D.0	fR/S	 30.9
U+557D.0	fTotalStrokes	 12
U+557D.0	fCangjie	RORT
U+557D.0	fFourCorner	68046
U+557D.0	fCantonese	ngam4 (also); am4; ngap1; ap1 (also); ngaam4; am2; am1
U+557D.0	fMandarin	an2
U+557D.0	fEnglish	 speak
U+557E.0	fUTF8	啾
U+557E.0	fCNS11643	1-5E54
U+557E.0	fBig5	B3EE
U+557E.0	fGB	0-6031
U+557E.0	fR/S	 30.9
U+557E.0	fTotalStrokes	 12
U+557E.0	fCangjie	RHDF
U+557E.0	fFourCorner	69080
U+557E.0	fHakka	 ziu1; ciu1
U+557E.0	fCantonese	zau1
U+557E.0	fMandarin	jiu1; chao2
U+557E.0	fEnglish	[1] wailing of a child; wailing of infants [2] chirp of birds or insects
U+557F.0	fUTF8	啿
U+557F.0	fCNS11643	2-3A5C
U+557F.0	fBig5	D878
U+557F.0	fGB	E-2461
U+557F.0	fR/S	 30.9
U+557F.0	fTotalStrokes	 12
U+557F.0	fCangjie	RTMV
U+557F.0	fFourCorner	64013
U+557F.0	fCantonese	daam6; taam5
U+557F.0	fMandarin	dan4
U+5580.0	fUTF8	喀
U+5580.0	fCNS11643	1-5E3E
U+5580.0	fBig5	B3D8
U+5580.0	fGB	0-3F26
U+5580.0	fR/S	 30.9
U+5580.0	fTotalStrokes	 12
U+5580.0	fCangjie	RJHR
U+5580.0	fFourCorner	63064
U+5580.0	fHakka	  kak7; hak7; kak8; ket7
U+5580.0	fCantonese	haak3 (also); kaa3 (also); kaa1; kaak3; haak1; kak1; kak3
U+5580.0	fMandarin	ka1; ke4; ka4; ke5
U+5580.0	fEnglish	used in transliteration
U+5581.0	fUTF8	喁
U+5581.0	fCNS11643	2-3A55
U+5581.0	fBig5	D871
U+5581.0	fGB	0-602F
U+5581.0	fR/S	 30.9
U+5581.0	fTotalStrokes	 12
U+5581.0	fCangjie	RWLB
U+5581.0	fFourCorner	66027
U+5581.0	fHakka	 ngi2; ngiung2;  jung2; ngi3; ngeu3
U+5581.0	fCantonese	jung4; jyu4
U+5581.0	fMandarin	yong2; yu2
U+5581.0	fEnglish	[1] the mouth of a fish at the surface of the water; the state of a fish putting its mouth above the water surface; gasping for breath [2] harmony or unison of sounds
U+5582.0	fUTF8	喂
U+5582.0	fCNS11643	1-5E44
U+5582.0	fBig5	B3DE
U+5582.0	fGB	0-4E39
U+5582.0	fR/S	 30.9
U+5582.0	fTotalStrokes	 12
U+5582.0	fCangjie	RWMV
U+5582.0	fFourCorner	66032
U+5582.0	fHakka	  wui5; oi1; wi5; woi5; wai1; wai5; vui5
U+5582.0	fCantonese	wai3; wai2; wai6
U+5582.0	fMandarin	wei4; wei1
U+5582.0	fEnglish	an interjection to call attention
U+5583.0	fUTF8	喃
U+5583.0	fCNS11643	1-5E4A
U+5583.0	fBig5	B3E4
U+5583.0	fGB	0-602B
U+5583.0	fR/S	 30.9
U+5583.0	fTotalStrokes	 12
U+5583.0	fCangjie	RJBJ
U+5583.0	fFourCorner	64027
U+5583.0	fHakka	  nam2; lam2
U+5583.0	fCantonese	naam4; laam4 (also)
U+5583.0	fMandarin	nan2
U+5583.0	fEnglish	[1] cries of a swallow [2] mumble; murmur; mutter
U+5584.0	fUTF8	善
U+5584.0	fCNS11643	1-6143
U+5584.0	fBig5	B5BD
U+5584.0	fGB	0-4946
U+5584.0	fR/S	 30.9
U+5584.0	fTotalStrokes	 12
U+5584.0	fCangjie	TGTR
U+5584.0	fFourCorner	80605
U+5584.0	fHakka	  shan5; sen3; sien5; sen5; shen5; shan3
U+5584.0	fCantonese	sin6
U+5584.0	fMandarin	shan4
U+5584.0	fEnglish	[1] good; virtuous; charitable; kind; goodness; virtue; good deeds; benevolent actions [2] be good at; be skilled in [3] perfect; make a success of [4] remedy; relieve [5] properly
U+5585.0	fUTF8	喅
U+5585.0	fCNS11643	3-3A31
U+5585.0	fGB	5-3877
U+5585.0	fR/S	  30.9
U+5585.0	fTotalStrokes	 12
U+5585.0	fCangjie	RAYT
U+5585.0	fFourCorner	66010
U+5585.0	fCantonese	juk6
U+5585.0	fMandarin	yu4
U+5586.0	fUTF8	喆
U+5586.0	fCNS11643	3-3A38
U+5586.0	fGB	8-2D5E
U+5586.0	fR/S	  30.9
U+5586.0	fTotalStrokes	 12
U+5586.0	fCangjie	GRGR
U+5586.0	fFourCorner	44660
U+5586.0	fHakka	 tsiit7; zhit7; zhat7; zat7; zhet7
U+5586.0	fCantonese	zit3
U+5586.0	fMandarin	zhe2
U+5586.0	fEnglish	same as &#x54F2;
U+5587.0	fUTF8	喇
U+5587.0	fCNS11643	1-5E48
U+5587.0	fBig5	B3E2
U+5587.0	fGB	0-402E
U+5587.0	fR/S	 30.9
U+5587.0	fTotalStrokes	 12
U+5587.0	fCangjie	RDLN
U+5587.0	fFourCorner	62000
U+5587.0	fHakka	  la1; lat8; la2
U+5587.0	fCantonese	laa3; laa1; laa5 (also); laat3
U+5587.0	fMandarin	la3; la2; la4
U+5587.0	fEnglish	[1] horn; bugle; trumpet [2] lama [3] used in transliteration
U+5587.1	fUTF8	喇
U+5587.1	fCNS11643	1-5E48
U+5587.1	fBig5	B3E2
U+5587.1	fGB	0-402E
U+5587.1	fR/S	 30.9
U+5587.1	fTotalStrokes	 12
U+5587.1	fCangjie	RDLN
U+5587.1	fFourCorner	62000
U+5587.1	fMandarin	la1
U+5587.1	fEnglish	used for sound
U+5588.0	fUTF8	喈
U+5588.0	fCNS11643	2-3A52
U+5588.0	fBig5	D86E
U+5588.0	fGB	0-602E
U+5588.0	fR/S	 30.9
U+5588.0	fTotalStrokes	 12
U+5588.0	fCangjie	RPPA
U+5588.0	fFourCorner	61061
U+5588.0	fHakka	 gai1; gi1
U+5588.0	fCantonese	gaai1
U+5588.0	fMandarin	jie1; jie4
U+5588.0	fEnglish	music; melody; harmonious sounds
U+5589.0	fUTF8	喉
U+5589.0	fCNS11643	1-5E55
U+5589.0	fBig5	B3EF
U+5589.0	fGB	0-3A6D
U+5589.0	fR/S	 30.9
U+5589.0	fTotalStrokes	 12
U+5589.0	fCangjie	RONK
U+5589.0	fFourCorner	67034
U+5589.0	fHakka	  heu2; heu3; hiu2
U+5589.0	fCantonese	hau4; hau2 (also)
U+5589.0	fMandarin	hou2
U+5589.0	fEnglish	 throat; gullet; larynx; guttural
U+558A.0	fUTF8	喊
U+558A.0	fCNS11643	1-5E41
U+558A.0	fBig5	B3DB
U+558A.0	fGB	0-3A30
U+558A.0	fR/S	 30.9
U+558A.0	fTotalStrokes	 12
U+558A.0	fCangjie	RIHR
U+558A.0	fFourCorner	63050
U+558A.0	fHakka	  ham5;  hem1; ham3
U+558A.0	fCantonese	haam3; ham6
U+558A.0	fMandarin	han3
U+558A.0	fEnglish	[1] shout; scream; call out; yell; cry; loud call; cry; scream; shout [2] call (a person)
U+558B.0	fUTF8	喋
U+558B.0	fCNS11643	1-5E49
U+558B.0	fBig5	B3E3
U+558B.0	fGB	0-6029
U+558B.0	fR/S	 30.9
U+558B.0	fTotalStrokes	 12
U+558B.0	fCangjie	RPTD
U+558B.0	fFourCorner	64094
U+558B.0	fHakka	 tiap8
U+558B.0	fCantonese	dip6; zaap3; zaap6
U+558B.0	fMandarin	die2; zha2; qie4
U+558B.0	fEnglish	 nag; chatter; babble; prattle; be very talkative; rattle
U+558C.0	fUTF8	喌
U+558C.0	fCNS11643	2-3A5A
U+558C.0	fBig5	D876
U+558C.0	fGB	5-387B
U+558C.0	fR/S	 30.9
U+558C.0	fTotalStrokes	 12
U+558C.0	fCangjie	RRILL
U+558C.0	fFourCorner	66320
U+558C.0	fHakka	 chuk7; zhiu1; zuk7; zu1
U+558C.0	fCantonese	zau1; zuk1 (also)
U+558C.0	fMandarin	zhou1; zhu4
U+558C.0	fEnglish	syllable
U+558D.0	fUTF8	喍
U+558D.0	fCNS11643	2-417C
U+558D.0	fBig5	DCD7
U+558D.0	fGB	E-2462
U+558D.0	fR/S	 30.10; 30.9
U+558D.0	fTotalStrokes	 13
U+558D.0	fCangjie	RYPD
U+558D.0	fFourCorner	61094
U+558D.0	fHakka	  cai2
U+558D.0	fCantonese	caai4
U+558D.0	fMandarin	chai2
U+558E.0	fUTF8	喎
U+558E.0	fCNS11643	2-3A5F
U+558E.0	fBig5	D87B
U+558E.0	fGB	E-2463
U+558E.0	fR/S	 30.9
U+558E.0	fTotalStrokes	 12
U+558E.0	fCangjie	RBBR
U+558E.0	fFourCorner	67027
U+558E.0	fHakka	  wo1; fe3; wo3; vo1; va2
U+558E.0	fCantonese	waa1; wo5; wo3; kwaa1; wo4; wo6
U+558E.0	fMandarin	wai1; guo1; kuai1
U+558E.0	fEnglish	[1] wry mouth [2] [final particle] expresses idea of discontant and/or astonishment [3] [final particle] indicates the end of a quotation (direct or indirect) [4] [final particle] expresses idea of contradicting or objecting in the form of a question [5] same as &#x54BC;
U+558F.0	fUTF8	喏
U+558F.0	fCNS11643	2-3A53
U+558F.0	fBig5	D86F
U+558F.0	fGB	0-5F76
U+558F.0	fR/S	 30.9
U+558F.0	fTotalStrokes	 12
U+558F.0	fCangjie	RTKR
U+558F.0	fFourCorner	64064
U+558F.0	fHakka	  ngia3; i5
U+558F.0	fCantonese	je5; nok6; lok6 (also); no6; lo6 (also)
U+558F.0	fMandarin	re3
U+558F.0	fEnglish	respectful reply of assent to superiors; an address or greeting of respect when meeting a superior, etc.
U+558F.1	fUTF8	喏
U+558F.1	fCNS11643	2-3A53
U+558F.1	fBig5	D86F
U+558F.1	fGB	0-5F76
U+558F.1	fR/S	 30.9
U+558F.1	fTotalStrokes	 12
U+558F.1	fCangjie	RTKR
U+558F.1	fFourCorner	64064
U+558F.1	fMandarin	nuo4
U+558F.1	fEnglish	a word used to attract other's attention
U+5590.0	fUTF8	喐
U+5590.0	fCNS11643	4-365C
U+5590.0	fGB	1-7C2B
U+5590.0	fR/S	  30.9
U+5590.0	fTotalStrokes	 12
U+5590.0	fCangjie	RKBL
U+5590.0	fFourCorner	67020
U+5590.0	fCantonese	juk1
U+5590.0	fMandarin	yu4
U+5590.0	fEnglish	move, touch, hit
U+5591.0	fUTF8	喑
U+5591.0	fCNS11643	2-3A4A
U+5591.0	fBig5	D866
U+5591.0	fGB	0-6033
U+5591.0	fR/S	 30.9
U+5591.0	fTotalStrokes	 12
U+5591.0	fCangjie	RYTA
U+5591.0	fFourCorner	60061
U+5591.0	fHakka	 jim1;  am1; rim1; rap7; jap7; jim5; am5
U+5591.0	fCantonese	jam1; ngam6; jam3; am6 (also)
U+5591.0	fMandarin	yin1; an1; yin4; an4; yin3
U+5591.0	fEnglish	 be dumb the sobbing of infants
U+5592.0	fUTF8	喒
U+5592.0	fCNS11643	2-3A57
U+5592.0	fBig5	D873
U+5592.0	fGB	E-2464
U+5592.0	fR/S	 30.9
U+5592.0	fTotalStrokes	 12
U+5592.0	fCangjie	RHOA
U+5592.0	fFourCorner	68064
U+5592.0	fHakka	 zam3
U+5592.0	fCantonese	zaa1
U+5592.0	fMandarin	zan2; zan3
U+5592.0	fEnglish	a dual pronoun; I; you and me; we two
U+5593.0	fUTF8	喓
U+5593.0	fCNS11643	2-3A51
U+5593.0	fBig5	D86D
U+5593.0	fGB	3-3662
U+5593.0	fR/S	 30.9
U+5593.0	fTotalStrokes	 12
U+5593.0	fCangjie	RMWV
U+5593.0	fFourCorner	61044
U+5593.0	fHakka	 rau1; jeu1; jau1
U+5593.0	fCantonese	jiu1; oe1
U+5593.0	fMandarin	yao1
U+5593.0	fEnglish	chirping; buzzing
U+5594.0	fUTF8	喔
U+5594.0	fCNS11643	1-5E47
U+5594.0	fBig5	B3E1
U+5594.0	fGB	0-6038
U+5594.0	fR/S	 30.9
U+5594.0	fTotalStrokes	 12
U+5594.0	fCangjie	RSMG
U+5594.0	fFourCorner	67014
U+5594.0	fHakka	  wok7; wo3; vok7
U+5594.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also); ngo1 (also); ngo2 (also); o1; ok1
U+5594.0	fMandarin	wo4; o4
U+5594.0	fEnglish	 crowing of a cock; cackling of fowls
U+5594.1	fUTF8	喔
U+5594.1	fCNS11643	1-5E47
U+5594.1	fBig5	B3E1
U+5594.1	fGB	0-6038
U+5594.1	fR/S	 30.9
U+5594.1	fTotalStrokes	 12
U+5594.1	fCangjie	RSMG
U+5594.1	fFourCorner	67014
U+5594.1	fMandarin	o1
U+5594.1	fEnglish	an exclamation; Oh!
U+5595.0	fUTF8	喕
U+5595.0	fCNS11643	2-3A5D
U+5595.0	fBig5	D879
U+5595.0	fGB	5-3874
U+5595.0	fR/S	 30.9
U+5595.0	fTotalStrokes	 12
U+5595.0	fCangjie	RMWL
U+5595.0	fFourCorner	61060
U+5595.0	fCantonese	min5
U+5595.0	fMandarin	mian3; mian2
U+5596.0	fUTF8	喖
U+5596.0	fCNS11643	3-3A2E
U+5596.0	fGB	5-3872
U+5596.0	fR/S	  30.9
U+5596.0	fTotalStrokes	 12
U+5596.0	fCangjie	RDJR
U+5596.0	fFourCorner	61060
U+5596.0	fCantonese	wu4
U+5596.0	fMandarin	hu2
U+5597.0	fUTF8	喗
U+5597.0	fCNS11643	3-3A2A
U+5597.0	fGB	3-362E
U+5597.0	fR/S	  30.9
U+5597.0	fTotalStrokes	 12
U+5597.0	fCangjie	RBJJ
U+5597.0	fFourCorner	67050
U+5597.0	fCantonese	wan5
U+5597.0	fMandarin	yun3
U+5598.0	fUTF8	喘
U+5598.0	fCNS11643	1-5E43
U+5598.0	fBig5	B3DD
U+5598.0	fGB	0-342D
U+5598.0	fR/S	 30.9
U+5598.0	fTotalStrokes	 12
U+5598.0	fCangjie	RUMB
U+5598.0	fFourCorner	62027
U+5598.0	fHakka	 con3;  chon3; chon6
U+5598.0	fCantonese	cyun2
U+5598.0	fMandarin	chuan3
U+5598.0	fEnglish	[1] pant; gasp; breathe heavily; breathe hard [2] asthma (medicine)
U+5599.0	fUTF8	喙
U+5599.0	fCNS11643	1-5E57
U+5599.0	fBig5	B3F1
U+5599.0	fGB	0-6039
U+5599.0	fR/S	 30.9
U+5599.0	fTotalStrokes	 12
U+5599.0	fCangjie	RVNO
U+5599.0	fFourCorner	67032
U+5599.0	fHakka	 chui3; zui3; wui3; wui6; wi5; vui3
U+5599.0	fCantonese	fui3; zau3
U+5599.0	fMandarin	hui4; cui4; dou4
U+5599.0	fEnglish	[1] beak; bill; snout [2] mouth
U+559A.0	fUTF8	喚
U+559A.0	fCNS11643	1-5E50
U+559A.0	fBig5	B3EA
U+559A.0	fGB	1-3B3D
U+559A.0	fR/S	 30.9
U+559A.0	fTotalStrokes	 12
U+559A.0	fCangjie	RNBK
U+559A.0	fFourCorner	67034
U+559A.0	fHakka	 fon5; fan5; fon6; fon3; won3
U+559A.0	fCantonese	wun6; fun3
U+559A.0	fMandarin	huan4
U+559A.0	fEnglish	[1] call [2] summon [3] arouse
U+559A.1	fUTF8	喚
U+559A.1	fCNS11643	1-5E50
U+559A.1	fBig5	B3EA
U+559A.1	fGB	1-3B3D
U+559A.1	fR/S	 30.9
U+559A.1	fTotalStrokes	 12
U+559A.1	fCangjie	RNBK
U+559A.1	fFourCorner	67034
U+559A.1	fMandarin	huan5
U+559B.0	fUTF8	喛
U+559B.0	fCNS11643	3-3A33
U+559B.0	fGB	3-3670
U+559B.0	fR/S	  30.9
U+559B.0	fTotalStrokes	 12
U+559B.0	fCangjie	RBME
U+559B.0	fFourCorner	62040
U+559B.0	fCantonese	wun6
U+559B.0	fMandarin	huan4; yuan2; xuan3; he2
U+559C.0	fUTF8	喜
U+559C.0	fCNS11643	1-5E45
U+559C.0	fBig5	B3DF
U+559C.0	fGB	0-4F32
U+559C.0	fR/S	 30.9
U+559C.0	fTotalStrokes	 12
U+559C.0	fCangjie	GRTR
U+559C.0	fFourCorner	40601
U+559C.0	fHakka	  hi3
U+559C.0	fCantonese	hei2; ci3
U+559C.0	fMandarin	xi3; xi4
U+559C.0	fEnglish	[1] like; love; enjoy; be fond of [2] joyful thing; happy event [3] joy [4] joyful; happy; delightful; pleasant; auspicious
U+559D.0	fUTF8	喝
U+559D.0	fCNS11643	1-5E42
U+559D.0	fBig5	B3DC
U+559D.0	fGB	0-3A48
U+559D.0	fR/S	 30.9; 52.8
U+559D.0	fTotalStrokes	 12
U+559D.0	fCangjie	RAPV
U+559D.0	fFourCorner	66027
U+559D.0	fHakka	  hot7; hot8; hok7
U+559D.0	fCantonese	hot3; jit3
U+559D.0	fMandarin	he1
U+559D.0	fEnglish	 drink
U+559D.1	fUTF8	喝
U+559D.1	fCNS11643	1-5E42
U+559D.1	fBig5	B3DC
U+559D.1	fGB	0-3A48
U+559D.1	fR/S	 30.9; 52.8
U+559D.1	fTotalStrokes	 12
U+559D.1	fCangjie	RAPV
U+559D.1	fFourCorner	66027
U+559D.1	fMandarin	he4
U+559D.1	fEnglish	 shout; call out aloud
U+559D.2	fUTF8	喝
U+559D.2	fCNS11643	1-5E42
U+559D.2	fBig5	B3DC
U+559D.2	fGB	0-3A48
U+559D.2	fR/S	 30.9; 52.8
U+559D.2	fTotalStrokes	 12
U+559D.2	fCangjie	RAPV
U+559D.2	fFourCorner	66027
U+559D.2	fMandarin	he5
U+559E.0	fUTF8	喞
U+559E.0	fCNS11643	3-3A40
U+559E.0	fGB	E-2465
U+559E.0	fR/S	 30.9
U+559E.0	fTotalStrokes	 12
U+559E.0	fCangjie	RHPL
U+559E.0	fFourCorner	67020
U+559E.0	fCantonese	zik1; zit1; zek1
U+559E.0	fMandarin	ji2; ji1
U+559E.0	fEnglish	[1] pump [2] the buzzing sound; chirping of insects [3] squirt
U+559F.0	fUTF8	喟
U+559F.0	fCNS11643	1-5E4D
U+559F.0	fBig5	B3E7
U+559F.0	fGB	0-6030
U+559F.0	fR/S	 30.9
U+559F.0	fTotalStrokes	 12
U+559F.0	fCangjie	RWB
U+559F.0	fFourCorner	66027
U+559F.0	fHakka	 kui3; wui3; kui1; wi3; vui3
U+559F.0	fCantonese	wai2; wai6 (also); faai3; wai3
U+559F.0	fMandarin	kui4; kuai4
U+559F.0	fEnglish	 sigh heavily
U+55A0.0	fUTF8	喠
U+55A0.0	fCNS11643	3-3A34
U+55A0.0	fGB	3-366C
U+55A0.0	fR/S	 30.9
U+55A0.0	fTotalStrokes	 12
U+55A0.0	fCangjie	RHJG
U+55A0.0	fFourCorner	62010
U+55A0.0	fCantonese	zung2
U+55A0.0	fMandarin	zhong3
U+55A1.0	fUTF8	喡
U+55A1.0	fCNS11643	2-3A5E
U+55A1.0	fBig5	D87A
U+55A1.0	fGB	3-3553
U+55A1.0	fR/S	 30.9
U+55A1.0	fTotalStrokes	 12
U+55A1.0	fCangjie	RDMQ
U+55A1.0	fFourCorner	64056
U+55A1.0	fCantonese	wai4; wai6
U+55A1.0	fMandarin	wei3; wei4
U+55A2.0	fUTF8	喢
U+55A2.0	fCNS11643	2-3A50
U+55A2.0	fBig5	D86C
U+55A2.0	fGB	5-387D
U+55A2.0	fR/S	 30.9
U+55A2.0	fTotalStrokes	 12
U+55A2.0	fCangjie	RHJX
U+55A2.0	fFourCorner	61077
U+55A2.0	fHakka	  cap7
U+55A2.0	fCantonese	saap3; sip3
U+55A2.0	fMandarin	sha4
U+55A2.0	fEnglish	loquacious; talkative
U+55A3.0	fUTF8	喣
U+55A3.0	fCNS11643	2-3A56
U+55A3.0	fBig5	D872
U+55A3.0	fGB	3-5062
U+55A3.0	fR/S	 30.9
U+55A3.0	fTotalStrokes	 12
U+55A3.0	fCangjie	RRF
U+55A3.0	fFourCorner	67332
U+55A3.0	fHakka	 hi1; hi3; hi5
U+55A3.0	fCantonese	heoi2; jyu3
U+55A3.0	fMandarin	xu3; xu1
U+55A3.0	fEnglish	 breathe upon
U+55A4.0	fUTF8	喤
U+55A4.0	fCNS11643	2-3A58
U+55A4.0	fBig5	D874
U+55A4.0	fGB	3-366D
U+55A4.0	fR/S	 30.9
U+55A4.0	fTotalStrokes	 12
U+55A4.0	fCangjie	RHAG
U+55A4.0	fFourCorner	66014
U+55A4.0	fHakka	  fong2; wong1
U+55A4.0	fCantonese	wong4; waang4
U+55A4.0	fMandarin	huang2; hong1; hong2
U+55A4.0	fEnglish	 ah harmony
U+55A5.0	fUTF8	喥
U+55A5.0	fCNS11643	2-3A4C
U+55A5.0	fBig5	D868
U+55A5.0	fGB	5-3923
U+55A5.0	fR/S	 30.9
U+55A5.0	fTotalStrokes	 12
U+55A5.0	fCangjie	RITE
U+55A5.0	fFourCorner	60047
U+55A5.0	fHakka	  tu5;  tok8
U+55A5.0	fCantonese	dok6; zaa3; dou6
U+55A5.0	fMandarin	du4
U+55A5.0	fEnglish	 a place
U+55A6.0	fUTF8	喦
U+55A6.0	fCNS11643	2-3A5B
U+55A6.0	fBig5	D877
U+55A6.0	fGB	5-387A
U+55A6.0	fR/S	 30.9
U+55A6.0	fTotalStrokes	 12
U+55A6.0	fCangjie	RRRU
U+55A6.0	fFourCorner	60776
U+55A6.0	fHakka	 ngiap7
U+55A6.0	fCantonese	ngaam4; aam4 (also); ngap6; jip6; nip6
U+55A6.0	fMandarin	yan2; nie4
U+55A6.0	fEnglish	talkative
U+55A7.0	fUTF8	喧
U+55A7.0	fCNS11643	1-5E3F
U+55A7.0	fBig5	B3D9
U+55A7.0	fGB	0-507A
U+55A7.0	fR/S	 30.9
U+55A7.0	fTotalStrokes	 12
U+55A7.0	fCangjie	RJMM
U+55A7.0	fFourCorner	63016
U+55A7.0	fHakka	 sien1; sen1; sian1
U+55A7.0	fCantonese	hyun1
U+55A7.0	fMandarin	xuan1; xuan2; xuan3
U+55A7.0	fEnglish	[1] noise; hubbub; uproar; noisy; [2] clamor; talk loud; talk noisily
U+55A8.0	fUTF8	喨
U+55A8.0	fCNS11643	2-3A4B
U+55A8.0	fBig5	D867
U+55A8.0	fGB	3-3672
U+55A8.0	fR/S	 30.9
U+55A8.0	fTotalStrokes	 12
U+55A8.0	fCangjie	RYRU
U+55A8.0	fFourCorner	60017
U+55A8.0	fHakka	 liong5; liong6; long5
U+55A8.0	fCantonese	loeng6
U+55A8.0	fMandarin	liang4
U+55A8.0	fEnglish	a clear, resonant sound
U+55A9.0	fUTF8	喩
U+55A9.0	fCNS11643	3-3A42
U+55A9.0	fGB	E-2466
U+55A9.0	fR/S	 30.9
U+55A9.0	fTotalStrokes	 12
U+55A9.0	fCangjie	ROMV
U+55A9.0	fFourCorner	68020
U+55A9.0	fHakka	 ji5; ri5
U+55A9.0	fCantonese	jyu6; jyu4
U+55A9.0	fMandarin	yu4; yu2
U+55A9.0	fEnglish	[1] liken; compare; use a figure of speech; illustration; parable; metaphor [2] know; be acquainted with [3] explain; make clear; tell the meaning of; instruct [4] a Chinese family name
U+55AA.0	fUTF8	喪
U+55AA.0	fCNS11643	1-5E46
U+55AA.0	fBig5	B3E0
U+55AA.0	fGB	1-4925
U+55AA.0	fR/S	 30.9
U+55AA.0	fTotalStrokes	 12
U+55AA.0	fCangjie	GRRV
U+55AA.0	fFourCorner	40732
U+55AA.0	fHakka	  song3;  song1; song6
U+55AA.0	fCantonese	song1; song3
U+55AA.0	fMandarin	sang1
U+55AA.0	fEnglish	[1] death; dying [2] mourn [3] funeral
U+55AA.1	fUTF8	喪
U+55AA.1	fCNS11643	1-5E46
U+55AA.1	fBig5	B3E0
U+55AA.1	fGB	1-4925
U+55AA.1	fR/S	 30.9
U+55AA.1	fTotalStrokes	 12
U+55AA.1	fCangjie	GRRV
U+55AA.1	fFourCorner	40732
U+55AA.1	fHakka	 song5
U+55AA.1	fMandarin	sang4
U+55AA.1	fEnglish	[1] lose; be deprived of [2] be defeated [3] decline; go down
U+55AB.0	fUTF8	喫
U+55AB.0	fCNS11643	1-5E56
U+55AB.0	fBig5	B3F0
U+55AB.0	fGB	E-2467
U+55AB.0	fR/S	 30.9
U+55AB.0	fTotalStrokes	 12
U+55AB.0	fCangjie	RQHK
U+55AB.0	fFourCorner	67034
U+55AB.0	fHakka	  ket7;  kiet7; k'ie5; nget7; ngiet7; ngat7
U+55AB.0	fCantonese	hek3; jaak3 (also); hat1 (also); gat1
U+55AB.0	fMandarin	chi1; ji2; ji1
U+55AB.0	fEnglish	 eat drink suffer; endure; bear
U+55AC.0	fUTF8	喬
U+55AC.0	fCNS11643	1-5E52
U+55AC.0	fBig5	B3EC
U+55AC.0	fGB	1-4747
U+55AC.0	fR/S	 30.9
U+55AC.0	fTotalStrokes	 12
U+55AC.0	fCangjie	HKRBR
U+55AC.0	fFourCorner	20227
U+55AC.0	fHakka	  kiau2; k'ieu2
U+55AC.0	fCantonese	kiu4
U+55AC.0	fMandarin	qiao2; jiao1
U+55AC.0	fEnglish	[1] tall [2] disguise; pretend [3] a Chinese family name
U+55AD.0	fUTF8	喭
U+55AD.0	fCNS11643	2-3A4D
U+55AD.0	fBig5	D869
U+55AD.0	fGB	3-3674
U+55AD.0	fR/S	 30.9
U+55AD.0	fTotalStrokes	 12
U+55AD.0	fCangjie	RYHH
U+55AD.0	fFourCorner	60022
U+55AD.0	fHakka	 ngien5; ngan5; ngien6; ngan6; ngian5
U+55AD.0	fCantonese	jin6; ngon6; jin5
U+55AD.0	fMandarin	yan4
U+55AD.0	fEnglish	 condole with coarse
U+55AE.0	fUTF8	單
U+55AE.0	fCNS11643	1-5E4C
U+55AE.0	fBig5	B3E6
U+55AE.0	fGB	1-3525
U+55AE.0	fR/S	 30.9
U+55AE.0	fTotalStrokes	 12
U+55AE.0	fCangjie	RRWJ
U+55AE.0	fFourCorner	66506
U+55AE.0	fHakka	  dan1; shan5; san5; sam3; sen3; sam2; dan3
U+55AE.0	fCantonese	sin6; sin4; sim4 (also); daan1
U+55AE.0	fMandarin	dan1; chan1; tan2; yue4; zhan4
U+55AE.0	fEnglish	[1] single; individual [2] only; alone; lone [3] of an odd number [4] a slip of paper [5] simple [6] list
U+55AE.1	fUTF8	單
U+55AE.1	fCNS11643	1-5E4C
U+55AE.1	fBig5	B3E6
U+55AE.1	fGB	1-3525
U+55AE.1	fR/S	 30.9
U+55AE.1	fTotalStrokes	 12
U+55AE.1	fCangjie	RRWJ
U+55AE.1	fFourCorner	66506
U+55AE.1	fMandarin	shan4
U+55AE.1	fEnglish	a Chinese family name
U+55AE.2	fUTF8	單
U+55AE.2	fCNS11643	1-5E4C
U+55AE.2	fBig5	B3E6
U+55AE.2	fGB	1-3525
U+55AE.2	fR/S	 30.9
U+55AE.2	fTotalStrokes	 12
U+55AE.2	fCangjie	RRWJ
U+55AE.2	fFourCorner	66506
U+55AE.2	fMandarin	chan2
U+55AE.2	fEnglish	the Chief of the Huns (a common term during the Han dynasty)
U+55AF.0	fUTF8	喯
U+55AF.0	fCNS11643	3-3A3B
U+55AF.0	fGB	5-385E
U+55AF.0	fR/S	 30.9; 30.8
U+55AF.0	fTotalStrokes	 12; 11
U+55AF.0	fCangjie	RKJT
U+55AF.0	fFourCorner	64040
U+55AF.0	fCantonese	pan3
U+55AF.0	fMandarin	pen1
U+55B0.0	fUTF8	喰
U+55B0.0	fCNS11643	F-3971
U+55B0.0	fGB	E-2468
U+55B0.0	fR/S	 30.9
U+55B0.0	fTotalStrokes	 12
U+55B0.0	fCangjie	ROIV
U+55B0.0	fCantonese	caan1
U+55B0.0	fMandarin	can1; sun1
U+55B0.0	fEnglish	[1] eat [2] drink
U+55B1.0	fUTF8	喱
U+55B1.0	fCNS11643	1-5E53
U+55B1.0	fBig5	B3ED
U+55B1.0	fGB	0-602C
U+55B1.0	fR/S	 30.9
U+55B1.0	fTotalStrokes	 12
U+55B1.0	fCangjie	RMWG
U+55B1.0	fFourCorner	61014
U+55B1.0	fHakka	 li2; li1
U+55B1.0	fCantonese	lei1; lei2 (also)
U+55B1.0	fMandarin	li2
U+55B1.0	fEnglish	 gramme syllable
U+55B2.0	fUTF8	喲
U+55B2.0	fCNS11643	1-5E4F
U+55B2.0	fBig5	B3E9
U+55B2.0	fGB	1-5334
U+55B2.0	fR/S	 30.9
U+55B2.0	fTotalStrokes	 12
U+55B2.0	fCangjie	RVFI
U+55B2.0	fFourCorner	67020
U+55B2.0	fHakka	 jo1
U+55B2.0	fCantonese	jo1
U+55B2.0	fMandarin	yo1yao1
U+55B2.0	fEnglish	ah; final particle
U+55B2.1	fUTF8	喲
U+55B2.1	fCNS11643	1-5E4F
U+55B2.1	fBig5	B3E9
U+55B2.1	fGB	1-5334
U+55B2.1	fR/S	 30.9
U+55B2.1	fTotalStrokes	 12
U+55B2.1	fCangjie	RVFI
U+55B2.1	fFourCorner	67020
U+55B2.1	fMandarin	yo5
U+55B3.0	fUTF8	喳
U+55B3.0	fCNS11643	1-5E4B
U+55B3.0	fBig5	B3E5
U+55B3.0	fGB	0-547B
U+55B3.0	fR/S	 30.9
U+55B3.0	fTotalStrokes	 12
U+55B3.0	fCangjie	RDAM
U+55B3.0	fFourCorner	64016
U+55B3.0	fHakka	 za1; za5
U+55B3.0	fCantonese	zaa1; caa1; caa4
U+55B3.0	fMandarin	zha1; cha1
U+55B3.0	fEnglish	 sound of chattering
U+55B4.0	fUTF8	喴
U+55B4.0	fCNS11643	3-3A2F
U+55B4.0	fGB	3-3663
U+55B4.0	fR/S	 30.9
U+55B4.0	fTotalStrokes	 12
U+55B4.0	fCangjie	RIHV
U+55B4.0	fFourCorner	63050
U+55B4.0	fCantonese	wai1; wi1
U+55B4.0	fMandarin	wei1
U+55B4.0	fEnglish	hello
U+55B5.0	fUTF8	喵
U+55B5.0	fCNS11643	2-3A54
U+55B5.0	fBig5	D870
U+55B5.0	fGB	0-5F77
U+55B5.0	fR/S	 30.9
U+55B5.0	fTotalStrokes	 12
U+55B5.0	fCangjie	RTW
U+55B5.0	fFourCorner	64060
U+55B5.0	fCantonese	miu1
U+55B5.0	fMandarin	miao1
U+55B5.0	fEnglish	the mew of the cat
U+55B6.0	fUTF8	営
U+55B6.0	fCNS11643	6-4729
U+55B6.0	fGB	E-2469
U+55B6.0	fR/S	 30.9; 42.9
U+55B6.0	fTotalStrokes	 12
U+55B6.0	fCangjie	FBRHR
U+55B6.0	fFourCorner	90600
U+55B6.0	fCantonese	jing4
U+55B6.0	fMandarin	ying2
U+55B6.0	fEnglish	 encampment; barracks manage
U+55B7.0	fUTF8	喷
U+55B7.0	fGB	0-4567
U+55B7.0	fR/S	 30.9
U+55B7.0	fTotalStrokes	 12
U+55B7.0	fCangjie	RJTO
U+55B7.0	fCantonese	pan3; fan5 (also); pan1
U+55B7.0	fMandarin	pen1; pen4
U+55B7.0	fEnglish	simplified form of &#x5674;
U+55B8.0	fUTF8	喸
U+55B8.0	fGB	1-7D62
U+55B8.0	fR/S	 30.9
U+55B8.0	fTotalStrokes	 12
U+55B8.0	fCangjie	IBRP
U+55B8.0	fCantonese	fu2
U+55B8.0	fEnglish	 place name
U+55B9.0	fUTF8	喹
U+55B9.0	fGB	0-602D
U+55B9.0	fR/S	 30.9
U+55B9.0	fTotalStrokes	 12
U+55B9.0	fCangjie	RKGG
U+55B9.0	fCantonese	fui1; kwai4 (also)
U+55B9.0	fMandarin	kui2
U+55B9.0	fEnglish	chemical compound
U+55BA.0	fUTF8	喺
U+55BA.0	fCNS11643	3-6723
U+55BA.0	fGB	E-246A
U+55BA.0	fR/S	 30.9
U+55BA.0	fTotalStrokes	 12
U+55BA.0	fCangjie	ROHF
U+55BA.0	fCantonese	hai2
U+55BA.0	fMandarin	xi4
U+55BA.0	fEnglish	 be located at; be in; be on; be at in; on; at; form
U+55BB.0	fUTF8	喻
U+55BB.0	fCNS11643	1-5E51
U+55BB.0	fBig5	B3EB
U+55BB.0	fGB	0-5377
U+55BB.0	fR/S	 30.9
U+55BB.0	fTotalStrokes	 12
U+55BB.0	fCangjie	ROMN
U+55BB.0	fFourCorner	68021
U+55BB.0	fHakka	 ji5; ji3
U+55BB.0	fCantonese	jyu6; jyu4
U+55BB.0	fMandarin	yu4; yu3; yu2
U+55BB.0	fEnglish	 metaphor; analogy example like
U+55BC.0	fUTF8	喼
U+55BC.0	fGB	1-7C30
U+55BC.0	fR/S	 30.9
U+55BC.0	fTotalStrokes	 12
U+55BC.0	fCangjie	RNSP
U+55BC.0	fHakka	 giap8
U+55BC.0	fCantonese	gip1; kip1; gep1; kep1
U+55BC.0	fMandarin	jie2
U+55BC.0	fEnglish	 bag; suitcase (for clothes)
U+55BD.0	fUTF8	喽
U+55BD.0	fGB	0-6036
U+55BD.0	fR/S	 30.9
U+55BD.0	fTotalStrokes	 12
U+55BD.0	fCangjie	RFDV
U+55BD.0	fHakka	 leu2
U+55BD.0	fCantonese	lau4; lau1
U+55BD.0	fMandarin	lou2; lou5; lou3
U+55BD.0	fEnglish	simplified form of &#x560D;
U+55BE.0	fUTF8	喾
U+55BE.0	fGB	0-6037
U+55BE.0	fR/S	 30.9
U+55BE.0	fTotalStrokes	 12
U+55BE.0	fCangjie	FBHGR
U+55BE.0	fHakka	 kwuk7
U+55BE.0	fCantonese	guk1
U+55BE.0	fMandarin	ku4
U+55BE.0	fEnglish	simplified form of &#x56B3;
U+55BF.0	fUTF8	喿
U+55BF.0	fCNS11643	2-417A
U+55BF.0	fBig5	DCD5
U+55BF.0	fGB	3-423D
U+55BF.0	fR/S	 30.10
U+55BF.0	fTotalStrokes	 13
U+55BF.0	fCangjie	RRRD
U+55BF.0	fFourCorner	60904
U+55BF.0	fHakka	 cau5
U+55BF.0	fCantonese	cou3; co3
U+55BF.0	fMandarin	sao4; cao1
U+55BF.0	fEnglish	chirping of birds
U+55C0.0	fUTF8	嗀
U+55C0.0	fCNS11643	2-4176
U+55C0.0	fBig5	DCD1
U+55C0.0	fGB	3-4F2E
U+55C0.0	fR/S	 30.10
U+55C0.0	fTotalStrokes	 13
U+55C0.0	fCangjie	GRHNE
U+55C0.0	fFourCorner	47647
U+55C0.0	fHakka	 ok7; ok8; hok7; guk7; kwuk7
U+55C0.0	fCantonese	hok3; fo3; hok6
U+55C0.0	fMandarin	huo4; hu4; xue4
U+55C0.0	fEnglish	 vomit
U+55C1.0	fUTF8	嗁
U+55C1.0	fCNS11643	3-4051
U+55C1.0	fGB	E-246B
U+55C1.0	fR/S	 30.10
U+55C1.0	fTotalStrokes	 13
U+55C1.0	fCangjie	RHYU
U+55C1.0	fFourCorner	62010
U+55C1.0	fHakka	 ti2; tai2
U+55C1.0	fCantonese	tai4
U+55C1.0	fMandarin	ti2
U+55C1.0	fEnglish	give forth sound; make noise
U+55C2.0	fUTF8	嗂
U+55C2.0	fCNS11643	2-4227
U+55C2.0	fBig5	DCE0
U+55C2.0	fGB	5-3933
U+55C2.0	fR/S	 30.10
U+55C2.0	fTotalStrokes	 13
U+55C2.0	fCangjie	RBOU
U+55C2.0	fFourCorner	67072
U+55C2.0	fHakka	 rau2; jeu2; jau2
U+55C2.0	fCantonese	jiu4
U+55C2.0	fMandarin	yao2
U+55C2.0	fEnglish	joyfull; merry
U+55C3.0	fUTF8	嗃
U+55C3.0	fCNS11643	2-416F
U+55C3.0	fBig5	DCCA
U+55C3.0	fGB	3-367D
U+55C3.0	fR/S	 30.10
U+55C3.0	fTotalStrokes	 13
U+55C3.0	fCangjie	RYRB
U+55C3.0	fFourCorner	60027
U+55C3.0	fHakka	 gau1; hau2; ko1
U+55C3.0	fCantonese	hok3; haau1; haau3; hok6; kok3
U+55C3.0	fMandarin	he4; xiao4; hu4; xiao1; xiao2; xue4
U+55C3.0	fEnglish	 flute
U+55C4.0	fUTF8	嗄
U+55C4.0	fCNS11643	2-4178
U+55C4.0	fBig5	DCD3
U+55C4.0	fGB	0-6044
U+55C4.0	fR/S	 30.10
U+55C4.0	fTotalStrokes	 13
U+55C4.0	fCangjie	RMUE
U+55C4.0	fFourCorner	61047
U+55C4.0	fHakka	  sa5; sia5; ha5; ha3
U+55C4.0	fCantonese	aai3; ngaai3
U+55C4.0	fMandarin	sha4; ai4
U+55C4.0	fEnglish	hoarse (said of a voice)
U+55C4.1	fUTF8	嗄
U+55C4.1	fCNS11643	2-4178
U+55C4.1	fBig5	DCD3
U+55C4.1	fGB	0-6044
U+55C4.1	fR/S	 30.10
U+55C4.1	fTotalStrokes	 13
U+55C4.1	fCangjie	RMUE
U+55C4.1	fFourCorner	61047
U+55C4.1	fCantonese	aa2; saa3
U+55C4.1	fMandarin	a2
U+55C4.1	fEnglish	What! (interjection)
U+55C5.0	fUTF8	嗅
U+55C5.0	fCNS11643	1-634C
U+55C5.0	fBig5	B6E5
U+55C5.0	fGB	0-5061
U+55C5.0	fR/S	 30.10
U+55C5.0	fTotalStrokes	 13
U+55C5.0	fCangjie	RHUK
U+55C5.0	fFourCorner	66034
U+55C5.0	fHakka	 cu5;  chiu5; ciu5; chu5
U+55C5.0	fCantonese	hung3 (also); cau3
U+55C5.0	fMandarin	xiu4
U+55C5.0	fEnglish	 smell; scent; sniff; olfactive
U+55C6.0	fUTF8	嗆
U+55C6.0	fCNS11643	1-634D
U+55C6.0	fBig5	B6E6
U+55C6.0	fGB	1-473A
U+55C6.0	fR/S	 30.10
U+55C6.0	fTotalStrokes	 13
U+55C6.0	fCangjie	ROIR
U+55C6.0	fFourCorner	68067
U+55C6.0	fHakka	 cong1
U+55C6.0	fCantonese	coeng1; coeng3
U+55C6.0	fMandarin	qiang1; cang1
U+55C6.0	fEnglish	[1] peck [2] stupid foolish
U+55C6.1	fUTF8	嗆
U+55C6.1	fCNS11643	1-634D
U+55C6.1	fBig5	B6E6
U+55C6.1	fGB	1-473A
U+55C6.1	fR/S	 30.10
U+55C6.1	fTotalStrokes	 13
U+55C6.1	fCangjie	ROIR
U+55C6.1	fFourCorner	68067
U+55C6.1	fMandarin	qiang4
U+55C6.1	fEnglish	[1] cough because of a temporary blockade of the nasal passage which sometimes happens during eating or drinking water [2] irritate the throat or nose (said of smoke, smell, etc.)
U+55C7.0	fUTF8	嗇
U+55C7.0	fCNS11643	1-6345
U+55C7.0	fBig5	B6DE
U+55C7.0	fGB	1-5844
U+55C7.0	fR/S	 30.10
U+55C7.0	fTotalStrokes	 13
U+55C7.0	fCangjie	GOWR
U+55C7.0	fFourCorner	40601
U+55C7.0	fHakka	 sep7; sit7; sek7; set7
U+55C7.0	fCantonese	sik1
U+55C7.0	fMandarin	se4
U+55C7.0	fEnglish	miserly; stingy; parsimonious
U+55C8.0	fUTF8	嗈
U+55C8.0	fCNS11643	2-4223
U+55C8.0	fBig5	DCDC
U+55C8.0	fGB	5-3939
U+55C8.0	fR/S	 30.10
U+55C8.0	fTotalStrokes	 13
U+55C8.0	fCangjie	RVVU
U+55C8.0	fFourCorner	62017
U+55C8.0	fHakka	 jung1; rung1
U+55C8.0	fCantonese	jung1
U+55C8.0	fMandarin	yong1
U+55C8.0	fEnglish	oh
U+55C9.0	fUTF8	嗉
U+55C9.0	fCNS11643	1-634F
U+55C9.0	fBig5	B6E8
U+55C9.0	fGB	0-603C
U+55C9.0	fR/S	 30.10
U+55C9.0	fTotalStrokes	 13
U+55C9.0	fCangjie	RQMF
U+55C9.0	fFourCorner	65093
U+55C9.0	fHakka	 su5; sio5; siu5; se5
U+55C9.0	fCantonese	sou3; seoi3 (also)
U+55C9.0	fMandarin	su4
U+55C9.0	fEnglish	crop (of a bird)
U+55CA.0	fUTF8	嗊
U+55CA.0	fCNS11643	2-4174
U+55CA.0	fBig5	DCCF
U+55CA.0	fGB	3-3632
U+55CA.0	fR/S	 30.10
U+55CA.0	fTotalStrokes	 13
U+55CA.0	fCangjie	RMBC
U+55CA.0	fFourCorner	61086
U+55CA.0	fHakka	 fung5; gung5; fung3
U+55CA.0	fCantonese	hung2 (also); gung3; fung2; gung3fung2
U+55CA.0	fMandarin	hong3; hong4; gong4
U+55CA.0	fEnglish	 sing a song, song
U+55CB.0	fUTF8	嗋
U+55CB.0	fCNS11643	2-4173
U+55CB.0	fBig5	DCCE
U+55CB.0	fGB	3-3721
U+55CB.0	fR/S	 30.10
U+55CB.0	fTotalStrokes	 13
U+55CB.0	fCangjie	RKSB
U+55CB.0	fFourCorner	64027
U+55CB.0	fCantonese	hip3; hip6
U+55CB.0	fMandarin	xie2; xie1; xie5
U+55CC.0	fUTF8	嗌
U+55CC.0	fCNS11643	2-4171
U+55CC.0	fBig5	DCCC
U+55CC.0	fGB	0-6049
U+55CC.0	fR/S	 30.10
U+55CC.0	fTotalStrokes	 13
U+55CC.0	fCangjie	RTCT
U+55CC.0	fFourCorner	68017
U+55CC.0	fHakka	 jit7; rit7
U+55CC.0	fCantonese	jik1; aai3; ngaai3 (also); ai3; ng2
U+55CC.0	fMandarin	yi4; ai4; e4; wo4
U+55CC.0	fEnglish	[1] throat [2] shout; yell; say something loudly
U+55CD.0	fUTF8	嗍
U+55CD.0	fCNS11643	2-4225
U+55CD.0	fBig5	DCDE
U+55CD.0	fGB	0-604A
U+55CD.0	fR/S	 30.10
U+55CD.0	fTotalStrokes	 13
U+55CD.0	fCangjie	RTUB
U+55CD.0	fFourCorner	67020
U+55CD.0	fHakka	  sok7
U+55CD.0	fCantonese	sok3
U+55CD.0	fMandarin	suo1; shuo4
U+55CD.0	fEnglish	another form of $ ( suck in; imbibe)
U+55CE.0	fUTF8	嗎
U+55CE.0	fCNS11643	1-6343
U+55CE.0	fBig5	B6DC
U+55CE.0	fGB	1-4270
U+55CE.0	fR/S	 30.10
U+55CE.0	fTotalStrokes	 13
U+55CE.0	fCangjie	RSQF
U+55CE.0	fFourCorner	61027
U+55CE.0	fHakka	  ma1;  ma3
U+55CE.0	fCantonese	maa1; maa3 (also); maa5; maa4
U+55CE.0	fMandarin	ma2; ma3; ma1; ma4
U+55CE.0	fEnglish	final interrogative particle used in questions
U+55CE.1	fUTF8	嗎
U+55CE.1	fCNS11643	1-6343
U+55CE.1	fBig5	B6DC
U+55CE.1	fGB	1-4270
U+55CE.1	fR/S	 30.10
U+55CE.1	fTotalStrokes	 13
U+55CE.1	fCangjie	RSQF
U+55CE.1	fFourCorner	61027
U+55CE.1	fMandarin	ma5
U+55CE.1	fEnglish	used in transliteration
U+55CF.0	fUTF8	嗏
U+55CF.0	fCNS11643	2-417D
U+55CF.0	fBig5	DCD8
U+55CF.0	fGB	3-3661
U+55CF.0	fR/S	 30.10
U+55CF.0	fTotalStrokes	 13
U+55CF.0	fCangjie	RTOD
U+55CF.0	fFourCorner	64098
U+55CF.0	fHakka	 ca2; ca1
U+55CF.0	fCantonese	caa1; caa2
U+55CF.0	fMandarin	cha1
U+55CF.0	fEnglish	a final sound, used for the sake of euphony
U+55D0.0	fUTF8	嗐
U+55D0.0	fCNS11643	2-4172
U+55D0.0	fBig5	DCCD
U+55D0.0	fGB	5-3937
U+55D0.0	fR/S	 30.10
U+55D0.0	fTotalStrokes	 13
U+55D0.0	fCangjie	RJQR
U+55D0.0	fFourCorner	63061
U+55D0.0	fHakka	 hat7; hoi5; hoi6; hai1; hai5; hoi2
U+55D0.0	fCantonese	haai6; hoi6 (also)
U+55D0.0	fMandarin	hai4; xia2
U+55D0.0	fEnglish	alas!
U+55D1.0	fUTF8	嗑
U+55D1.0	fCNS11643	1-6346
U+55D1.0	fBig5	B6DF
U+55D1.0	fGB	0-603E
U+55D1.0	fR/S	 30.10
U+55D1.0	fTotalStrokes	 13
U+55D1.0	fCangjie	RGIT
U+55D1.0	fFourCorner	64017
U+55D1.0	fHakka	 hap8;  kap8; gap8
U+55D1.0	fCantonese	haap3; hap6; gaap3; ngap1; kap3
U+55D1.0	fMandarin	ke4; he2; ke1
U+55D1.0	fEnglish	 crack something between the teeth
U+55D2.0	fUTF8	嗒
U+55D2.0	fCNS11643	2-417B
U+55D2.0	fBig5	DCD6
U+55D2.0	fGB	0-602A
U+55D2.0	fR/S	 30.10
U+55D2.0	fTotalStrokes	 13
U+55D2.0	fCangjie	RTOR
U+55D2.0	fFourCorner	64061
U+55D2.0	fHakka	  tap7; dap7
U+55D2.0	fCantonese	taap3; daap1; daat3; taap1 (also); daap3; daat6; tap3; dap1; dap3
U+55D2.0	fMandarin	ta4; da1
U+55D2.0	fEnglish	[1] depressed; dejected; despondent; in low spirits [2] hang down; bend one's head [3] [dak4]
U+55D3.0	fUTF8	嗓
U+55D3.0	fCNS11643	1-6341
U+55D3.0	fBig5	B6DA
U+55D3.0	fGB	0-4924
U+55D3.0	fR/S	 30.10
U+55D3.0	fTotalStrokes	 13
U+55D3.0	fCangjie	REED
U+55D3.0	fFourCorner	67094
U+55D3.0	fHakka	  song3; song1
U+55D3.0	fCantonese	song2; song1 (also)
U+55D3.0	fMandarin	sang3
U+55D3.0	fEnglish	[1] voice; one's voice [2] throat
U+55D4.0	fUTF8	嗔
U+55D4.0	fCNS11643	2-4177
U+55D4.0	fBig5	DCD2
U+55D4.0	fGB	0-6041
U+55D4.0	fR/S	 30.10
U+55D4.0	fTotalStrokes	 13
U+55D4.0	fCangjie	RJBC
U+55D4.0	fFourCorner	61081
U+55D4.0	fHakka	  tien2; ten2; chin1; shim3
U+55D4.0	fCantonese	can1; zan1 (also); tin4
U+55D4.0	fMandarin	chen1; tian2
U+55D4.0	fEnglish	[1] be angry; take offense; fly into temper [2] be annoyed with
U+55D5.0	fUTF8	嗕
U+55D5.0	fCNS11643	2-417E
U+55D5.0	fBig5	DCD9
U+55D5.0	fGB	3-367A
U+55D5.0	fR/S	 30.10
U+55D5.0	fTotalStrokes	 13
U+55D5.0	fCangjie	RMVI
U+55D5.0	fFourCorner	61043
U+55D5.0	fHakka	 juk8; ruk8
U+55D5.0	fCantonese	juk6
U+55D5.0	fMandarin	ru4; nou4
U+55D5.0	fEnglish	[1] pity [2] a Turkic tribe, under the Han dynasty
U+55D6.0	fUTF8	嗖
U+55D6.0	fCNS11643	2-4222
U+55D6.0	fBig5	DCDB
U+55D6.0	fGB	0-6032
U+55D6.0	fR/S	 30.10
U+55D6.0	fTotalStrokes	 13
U+55D6.0	fCangjie	RHXE
U+55D6.0	fFourCorner	67047
U+55D6.0	fHakka	  suk7;  seu5; siu5
U+55D6.0	fCantonese	sau1
U+55D6.0	fMandarin	sou1; shou4
U+55D6.0	fEnglish	[1] the laughing expression [2] the sound one makes to drive birds away
U+55D7.0	fUTF8	嗗
U+55D7.0	fCNS11643	3-404E
U+55D7.0	fGB	3-3669
U+55D7.0	fR/S	  30.10
U+55D7.0	fTotalStrokes	 13
U+55D7.0	fCangjie	RBBB
U+55D7.0	fFourCorner	67020
U+55D7.0	fCantonese	waat3; gut2; gwut2 (also); gut6; gwat1; gwut6 (also); gut4; gwut4 (also)
U+55D7.0	fMandarin	wa1; gu1
U+55D7.0	fEnglish	quarter of an hour
U+55D8.0	fUTF8	嗘
U+55D8.0	fCNS11643	4-3C4B
U+55D8.0	fGB	E-246C
U+55D8.0	fR/S	  30.10
U+55D8.0	fTotalStrokes	 13
U+55D8.0	fCangjie	RBVK
U+55D8.0	fFourCorner	62030
U+55D8.0	fHakka	 gai1;  hi1
U+55D8.0	fCantonese	gai1
U+55D8.0	fMandarin	ji1
U+55D9.0	fUTF8	嗙
U+55D9.0	fCNS11643	2-4226
U+55D9.0	fBig5	DCDF
U+55D9.0	fGB	3-367E
U+55D9.0	fR/S	 30.10
U+55D9.0	fTotalStrokes	 13
U+55D9.0	fCangjie	RYBS
U+55D9.0	fFourCorner	60027
U+55D9.0	fHakka	  pong2; bong3;  pang1
U+55D9.0	fCantonese	pong3 (also); pong5; bong2 (also); bang1; paang1
U+55D9.0	fMandarin	pang3; bang1; bang4
U+55D9.0	fEnglish	boast
U+55DA.0	fUTF8	嗚
U+55DA.0	fCNS11643	1-634A
U+55DA.0	fBig5	B6E3
U+55DA.0	fGB	1-4E58
U+55DA.0	fR/S	 30.10
U+55DA.0	fTotalStrokes	 13
U+55DA.0	fCangjie	RHRF
U+55DA.0	fFourCorner	67027
U+55DA.0	fHakka	 wu1; vu1
U+55DA.0	fCantonese	wu1
U+55DA.0	fMandarin	wu1; wu4
U+55DA.0	fEnglish	[1] weep; sob [2] toot; hoot; zoom [3] alas!
U+55DB.0	fUTF8	嗛
U+55DB.0	fCNS11643	2-4170
U+55DB.0	fBig5	DCCB
U+55DB.0	fGB	5-3936
U+55DB.0	fR/S	 30.10
U+55DB.0	fTotalStrokes	 13
U+55DB.0	fCangjie	RTXC
U+55DB.0	fFourCorner	68037
U+55DB.0	fHakka	  kiap7;  kiam1; ham2
U+55DB.0	fCantonese	haam4; ham4 (also); him2; him3; hip3; him1; hip6 (also); him5
U+55DB.0	fMandarin	qian4; xian2; qie4; qian1; qian3; qian2
U+55DB.0	fEnglish	 pouch hold content
U+55DC.0	fUTF8	嗜
U+55DC.0	fCNS11643	1-6344
U+55DC.0	fBig5	B6DD
U+55DC.0	fGB	0-4A48
U+55DC.0	fR/S	 30.10
U+55DC.0	fTotalStrokes	 13
U+55DC.0	fCangjie	RJPA
U+55DC.0	fFourCorner	64061
U+55DC.0	fHakka	 si5; shi5; sii5; she5
U+55DC.0	fCantonese	si3
U+55DC.0	fMandarin	shi4
U+55DC.0	fEnglish	to be fond of; have weakness for; delight in; relish; like; addict
U+55DD.0	fUTF8	嗝
U+55DD.0	fCNS11643	2-4175
U+55DD.0	fBig5	DCD0
U+55DD.0	fGB	0-6043
U+55DD.0	fR/S	 30.10
U+55DD.0	fTotalStrokes	 13
U+55DD.0	fCangjie	RMRB
U+55DD.0	fFourCorner	61027
U+55DD.0	fHakka	 gak7; het8
U+55DD.0	fCantonese	gaak3
U+55DD.0	fMandarin	ge2
U+55DD.0	fEnglish	 hiccup; hiccough; belch
U+55DE.0	fUTF8	嗞
U+55DE.0	fCNS11643	3-3A2B
U+55DE.0	fGB	3-3675
U+55DE.0	fR/S	 30.10
U+55DE.0	fTotalStrokes	 12; 13
U+55DE.0	fCangjie	RTVI
U+55DE.0	fFourCorner	68032
U+55DE.0	fHakka	 zih1
U+55DE.0	fCantonese	zi1
U+55DE.0	fMandarin	zi1
U+55DE.0	fEnglish	 consult about; plan report in writing to an equal despatch
U+55DF.0	fUTF8	嗟
U+55DF.0	fCNS11643	1-633F
U+55DF.0	fBig5	B6D8
U+55DF.0	fGB	0-6035
U+55DF.0	fR/S	 30.10
U+55DF.0	fTotalStrokes	 13
U+55DF.0	fCangjie	RTQM
U+55DF.0	fFourCorner	68011
U+55DF.0	fHakka	 zia1; zia2
U+55DF.0	fCantonese	ze1
U+55DF.0	fMandarin	jie1; jue1; jie4
U+55DF.0	fEnglish	sigh; alas; an exclamation expressing grief or regret
U+55E0.0	fUTF8	嗠
U+55E0.0	fCNS11643	6-5143
U+55E0.0	fGB	E-246D
U+55E0.0	fR/S	  30.10
U+55E0.0	fTotalStrokes	 13
U+55E0.0	fCangjie	BDHER
U+55E0.0	fFourCorner	27460
U+55E0.0	fCantonese	lok6
U+55E0.0	fMandarin	luo4
U+55E1.0	fUTF8	嗡
U+55E1.0	fCNS11643	1-634B
U+55E1.0	fBig5	B6E4
U+55E1.0	fGB	0-4E4B
U+55E1.0	fR/S	 30.10
U+55E1.0	fTotalStrokes	 13
U+55E1.0	fCangjie	RCIM
U+55E1.0	fFourCorner	68027
U+55E1.0	fHakka	  wung1; ang2; vung1; ong2; wung2; wung5; ong5; ong1
U+55E1.0	fCantonese	jung1; wang1
U+55E1.0	fMandarin	weng1; weng3
U+55E1.0	fEnglish	 sound of flying insects; hum or buzz of insects
U+55E2.0	fUTF8	嗢
U+55E2.0	fCNS11643	2-4221
U+55E2.0	fBig5	DCDA
U+55E2.0	fGB	3-3666
U+55E2.0	fR/S	 30.10
U+55E2.0	fTotalStrokes	 13
U+55E2.0	fCangjie	RWOT
U+55E2.0	fFourCorner	66017
U+55E2.0	fHakka	 wut7; vut7
U+55E2.0	fCantonese	wat1
U+55E2.0	fMandarin	wa4
U+55E2.0	fEnglish	 hawk; clear the throat
U+55E3.0	fUTF8	嗣
U+55E3.0	fCNS11643	1-6347
U+55E3.0	fBig5	B6E0
U+55E3.0	fGB	0-4B43
U+55E3.0	fR/S	 30.10
U+55E3.0	fTotalStrokes	 13
U+55E3.0	fCangjie	RBSMR
U+55E3.0	fFourCorner	67220
U+55E3.0	fHakka	 cih2; cu3; sii5; sii6; cu2; se5; ze5
U+55E3.0	fCantonese	zi6
U+55E3.0	fMandarin	si4
U+55E3.0	fEnglish	[1] inherit; succeed to [2] descendant; heir [3] continue; follow
U+55E4.0	fUTF8	嗤
U+55E4.0	fCNS11643	1-6348
U+55E4.0	fBig5	B6E1
U+55E4.0	fGB	0-604D
U+55E4.0	fR/S	 30.10
U+55E4.0	fTotalStrokes	 13
U+55E4.0	fCangjie	RUMI
U+55E4.0	fFourCorner	62036
U+55E4.0	fHakka	 ci1;  chi1; cih1; chii1
U+55E4.0	fCantonese	ci1
U+55E4.0	fMandarin	chi1
U+55E4.0	fEnglish	 laugh at; ridicule; sneer; chuckle sneeringly
U+55E5.0	fUTF8	嗥
U+55E5.0	fCNS11643	1-634E
U+55E5.0	fBig5	B6E7
U+55E5.0	fGB	0-6046
U+55E5.0	fR/S	 30.10
U+55E5.0	fTotalStrokes	 13
U+55E5.0	fCangjie	RHAJ
U+55E5.0	fFourCorner	66043
U+55E5.0	fHakka	 hau2; hau1
U+55E5.0	fCantonese	hou4
U+55E5.0	fMandarin	hao2
U+55E5.0	fEnglish	 roar; call out; wail bark; yelp
U+55E6.0	fUTF8	嗦
U+55E6.0	fCNS11643	1-6342
U+55E6.0	fBig5	B6DB
U+55E6.0	fGB	0-6042
U+55E6.0	fR/S	 30.10
U+55E6.0	fTotalStrokes	 13
U+55E6.0	fCangjie	RJBF
U+55E6.0	fFourCorner	64093
U+55E6.0	fHakka	 so1
U+55E6.0	fCantonese	so1; sok3
U+55E6.0	fMandarin	suo1; suo5
U+55E6.0	fEnglish	suck
U+55E7.0	fUTF8	嗧
U+55E7.0	fCNS11643	1-2321
U+55E7.0	fBig5	A25F
U+55E7.0	fGB	E-246E
U+55E7.0	fR/S	 30.10
U+55E7.0	fTotalStrokes	 13
U+55E7.0	fCangjie	KROMB
U+55E7.0	fFourCorner	4622
U+55E7.0	fCantonese	gaa1
U+55E7.0	fMandarin	jia1lun2
U+55E8.0	fUTF8	嗨
U+55E8.0	fCNS11643	1-6340
U+55E8.0	fBig5	B6D9
U+55E8.0	fGB	0-604B
U+55E8.0	fR/S	 30.10
U+55E8.0	fTotalStrokes	 13
U+55E8.0	fCangjie	REOY
U+55E8.0	fFourCorner	68057
U+55E8.0	fHakka	 hoi3; hai1
U+55E8.0	fCantonese	hoi1; hei1; haai1; hai1
U+55E8.0	fMandarin	hai1; hai3; hai4; hei1
U+55E8.0	fEnglish	hi
U+55E9.0	fUTF8	嗩
U+55E9.0	fCNS11643	2-4179
U+55E9.0	fBig5	DCD4
U+55E9.0	fGB	1-5F6F
U+55E9.0	fR/S	 30.10
U+55E9.0	fTotalStrokes	 13
U+55E9.0	fCangjie	RFBC
U+55E9.0	fFourCorner	69086
U+55E9.0	fHakka	 so3
U+55E9.0	fCantonese	so2
U+55E9.0	fMandarin	suo3
U+55E9.0	fEnglish	a trumpet-like wind instrument
U+55EA.0	fUTF8	嗪
U+55EA.0	fCNS11643	3-404C
U+55EA.0	fGB	0-603A
U+55EA.0	fR/S	 30.10
U+55EA.0	fTotalStrokes	 13
U+55EA.0	fCangjie	RQKD
U+55EA.0	fFourCorner	65090
U+55EA.0	fCantonese	ceon4
U+55EA.0	fMandarin	qin2
U+55EA.0	fEnglish	character used in translation
U+55EB.0	fUTF8	嗫
U+55EB.0	fGB	0-603F
U+55EB.0	fR/S	 30.10
U+55EB.0	fTotalStrokes	 13
U+55EB.0	fCangjie	RSJE
U+55EB.0	fHakka	 ngiap7
U+55EB.0	fCantonese	zip3; nip6 (also); lip6 (also); zip6
U+55EB.0	fMandarin	nie4; zhe2
U+55EB.0	fEnglish	simplified form of &#x56C1;
U+55EC.0	fUTF8	嗬
U+55EC.0	fGB	0-6040
U+55EC.0	fR/S	 30.10
U+55EC.0	fTotalStrokes	 13
U+55EC.0	fCangjie	RTOR
U+55EC.0	fCantonese	ho2; ho4
U+55EC.0	fMandarin	he1
U+55EC.0	fEnglish	interrogative particle
U+55ED.0	fUTF8	嗭
U+55ED.0	fGB	1-7D5D
U+55ED.0	fR/S	 30.10
U+55ED.0	fTotalStrokes	 13
U+55ED.0	fCangjie	JMRP
U+55ED.0	fCantonese	zik6
U+55ED.0	fEnglish	 place name
U+55EE.0	fUTF8	嗮
U+55EE.0	fCNS11643	3-6725
U+55EE.0	fGB	E-246F
U+55EE.0	fR/S	 30.10
U+55EE.0	fTotalStrokes	 13
U+55EE.0	fCangjie	RAMW
U+55EE.0	fCantonese	saai3
U+55EE.0	fMandarin	sai4
U+55EE.0	fEnglish	 entirely; completely; all
U+55EF.0	fUTF8	嗯
U+55EF.0	fCNS11643	1-6349
U+55EF.0	fBig5	B6E2
U+55EF.0	fGB	0-6045
U+55EF.0	fR/S	 30.10
U+55EF.0	fTotalStrokes	 13
U+55EF.0	fCangjie	RWKP
U+55EF.0	fFourCorner	66030
U+55EF.0	fHakka	 en5; en1; ng3
U+55EF.0	fCantonese	ng6; ng2 (also); ng5 (also)
U+55EF.0	fMandarin	en2; en1
U+55EF.0	fEnglish	the nasal sound used to express doubt
U+55EF.1	fUTF8	嗯
U+55EF.1	fCNS11643	1-6349
U+55EF.1	fBig5	B6E2
U+55EF.1	fGB	0-6045
U+55EF.1	fR/S	 30.10
U+55EF.1	fTotalStrokes	 13
U+55EF.1	fCangjie	RWKP
U+55EF.1	fFourCorner	66030
U+55EF.1	fMandarin	en3
U+55EF.1	fEnglish	the nasal sound used when one makes a response or promise
U+55EF.2	fUTF8	嗯
U+55EF.2	fCNS11643	1-6349
U+55EF.2	fBig5	B6E2
U+55EF.2	fGB	0-6045
U+55EF.2	fR/S	 30.10
U+55EF.2	fTotalStrokes	 13
U+55EF.2	fCangjie	RWKP
U+55EF.2	fFourCorner	66030
U+55EF.2	fMandarin	en4
U+55EF.2	fEnglish	the nasal sound used when one is surprised or indifferent
U+55EF.3	fUTF8	嗯
U+55EF.3	fCNS11643	1-6349
U+55EF.3	fBig5	B6E2
U+55EF.3	fGB	0-6045
U+55EF.3	fR/S	 30.10
U+55EF.3	fTotalStrokes	 13
U+55EF.3	fCangjie	RWKP
U+55EF.3	fFourCorner	66030
U+55EF.3	fMandarin	ng2
U+55EF.4	fUTF8	嗯
U+55EF.4	fCNS11643	1-6349
U+55EF.4	fBig5	B6E2
U+55EF.4	fGB	0-6045
U+55EF.4	fR/S	 30.10
U+55EF.4	fTotalStrokes	 13
U+55EF.4	fCangjie	RWKP
U+55EF.4	fFourCorner	66030
U+55EF.5	fUTF8	嗯
U+55EF.5	fCNS11643	1-6349
U+55EF.5	fBig5	B6E2
U+55EF.5	fGB	0-6045
U+55EF.5	fR/S	 30.10
U+55EF.5	fTotalStrokes	 13
U+55EF.5	fCangjie	RWKP
U+55EF.5	fFourCorner	66030
U+55F0.0	fUTF8	嗰
U+55F0.0	fCNS11643	3-6724
U+55F0.0	fGB	1-7C33
U+55F0.0	fR/S	 30.10
U+55F0.0	fTotalStrokes	 13
U+55F0.0	fCangjie	ROWR
U+55F0.0	fCantonese	go2; go3; go1
U+55F0.0	fMandarin	ge4
U+55F0.0	fEnglish	 that
U+55F1.0	fUTF8	嗱
U+55F1.0	fCNS11643	F-4062
U+55F1.0	fGB	1-7C32
U+55F1.0	fR/S	 30.10
U+55F1.0	fTotalStrokes	 13
U+55F1.0	fCangjie	RORQ
U+55F1.0	fFourCorner	6805
U+55F1.0	fCantonese	naa4; laa4 (also); naa3; laa3
U+55F1.0	fMandarin	na2
U+55F1.0	fEnglish	 exclamation; there! now!; there it is; there you are; look
U+55F2.0	fUTF8	嗲
U+55F2.0	fCNS11643	2-4224
U+55F2.0	fBig5	DCDD
U+55F2.0	fGB	0-6047
U+55F2.0	fR/S	 30.10
U+55F2.0	fTotalStrokes	 13
U+55F2.0	fCangjie	RCKN
U+55F2.0	fFourCorner	68027
U+55F2.0	fCantonese	de2; de4; de1
U+55F2.0	fMandarin	dia3; die1; dia1
U+55F2.0	fEnglish	[1] inviting; coquettish [2] [dia2] coquettish tone or attitude
U+55F3.0	fUTF8	嗳
U+55F3.0	fGB	0-6048
U+55F3.0	fR/S	 30.10
U+55F3.0	fTotalStrokes	 13
U+55F3.0	fCangjie	XRBBE
U+55F3.0	fHakka	 oi5
U+55F3.0	fCantonese	oi2; ngoi2 (also); aai1; ngaai1 (also); oi3; ai1
U+55F3.0	fMandarin	ai4; ai1; ai3
U+55F3.0	fEnglish	simplified form of &#x566F;
U+55F4.0	fUTF8	嗴
U+55F4.0	fCNS11643	3-6458
U+55F4.0	fGB	E-2470
U+55F4.0	fR/S	 30.10
U+55F4.0	fTotalStrokes	 13
U+55F4.0	fCangjie	RTUI
U+55F4.0	fFourCorner	6801
U+55F4.0	fCantonese	goeng1
U+55F5.0	fUTF8	嗵
U+55F5.0	fGB	0-604C
U+55F5.0	fR/S	 30.10
U+55F5.0	fTotalStrokes	 13
U+55F5.0	fCangjie	RYNB
U+55F5.0	fCantonese	tung1
U+55F5.0	fMandarin	tong1
U+55F5.0	fEnglish	used for sound
U+55F6.0	fUTF8	嗶
U+55F6.0	fCNS11643	1-6835
U+55F6.0	fBig5	B9CD
U+55F6.0	fGB	1-5F59
U+55F6.0	fR/S	 30.11
U+55F6.0	fTotalStrokes	 14
U+55F6.0	fCangjie	RWTJ
U+55F6.0	fFourCorner	66054
U+55F6.0	fHakka	 bit7
U+55F6.0	fCantonese	bat1
U+55F6.0	fMandarin	bi4
U+55F6.0	fEnglish	used in transliteration
U+55F7.0	fUTF8	嗷
U+55F7.0	fCNS11643	1-6830
U+55F7.0	fBig5	B9C8
U+55F7.0	fGB	0-603B
U+55F7.0	fR/S	 30.11
U+55F7.0	fTotalStrokes	 14
U+55F7.0	fCangjie	RGSK
U+55F7.0	fFourCorner	68040
U+55F7.0	fHakka	 ngau2; ngau1
U+55F7.0	fCantonese	ngou4; ou4 (also); ngou1
U+55F7.0	fMandarin	ao2
U+55F7.0	fEnglish	[1] cry of hunger; noise of people clamouring [2] buzz of voices
U+55F8.0	fUTF8	嗸
U+55F8.0	fCNS11643	3-462E
U+55F8.0	fGB	5-392E
U+55F8.0	fR/S	  30.11
U+55F8.0	fTotalStrokes	 14
U+55F8.0	fCangjie	GKR
U+55F8.0	fFourCorner	48600
U+55F8.0	fCantonese	ngou4; ou4 (also)
U+55F8.0	fMandarin	ao2
U+55F8.0	fEnglish	same as &#x55F7;
U+55F9.0	fUTF8	嗹
U+55F9.0	fCNS11643	2-493C
U+55F9.0	fBig5	E155
U+55F9.0	fGB	5-384D
U+55F9.0	fR/S	 30.11
U+55F9.0	fTotalStrokes	 14
U+55F9.0	fCangjie	RYJJ
U+55F9.0	fFourCorner	65030
U+55F9.0	fHakka	  lien2
U+55F9.0	fCantonese	lin4
U+55F9.0	fMandarin	lian2
U+55F9.0	fEnglish	chatter
U+55FA.0	fUTF8	嗺
U+55FA.0	fCNS11643	2-4938
U+55FA.0	fBig5	E151
U+55FA.0	fGB	3-3728
U+55FA.0	fR/S	 30.11
U+55FA.0	fTotalStrokes	 14
U+55FA.0	fCangjie	RUOG
U+55FA.0	fFourCorner	62014
U+55FA.0	fHakka	 cui1; chui1; zui1
U+55FA.0	fCantonese	ceoi1
U+55FA.0	fMandarin	cui1; sui1; zui1
U+55FA.0	fEnglish	 purse up the mouth
U+55FB.0	fUTF8	嗻
U+55FB.0	fCNS11643	3-4628
U+55FB.0	fGB	3-372D
U+55FB.0	fR/S	  30.11
U+55FB.0	fTotalStrokes	 14
U+55FB.0	fCangjie	RITF
U+55FB.0	fFourCorner	60037
U+55FB.0	fHakka	 zha5; za5; zha1
U+55FB.0	fCantonese	ze1; syu3
U+55FB.0	fMandarin	zhe4; zhe1
U+55FB.0	fEnglish	[1] screen [2] loquacious
U+55FC.0	fUTF8	嗼
U+55FC.0	fCNS11643	2-4932
U+55FC.0	fBig5	E14B
U+55FC.0	fGB	5-392F
U+55FC.0	fR/S	 30.11
U+55FC.0	fTotalStrokes	 14
U+55FC.0	fCangjie	RTAK
U+55FC.0	fFourCorner	64034
U+55FC.0	fHakka	 mok8; mok7
U+55FC.0	fCantonese	mok6; mak6
U+55FC.0	fMandarin	mo4
U+55FC.0	fEnglish	silent; still; peaceful
U+55FD.0	fUTF8	嗽
U+55FD.0	fCNS11643	1-682A
U+55FD.0	fBig5	B9C2
U+55FD.0	fGB	0-4B54
U+55FD.0	fR/S	 30.11
U+55FD.0	fTotalStrokes	 14
U+55FD.0	fCangjie	RDLO
U+55FD.0	fFourCorner	67082
U+55FD.0	fHakka	 seu5;  cuk8;  cuk7; siu5; sieu5
U+55FD.0	fCantonese	sau3; sok3; sau2
U+55FD.0	fMandarin	sou4; shou4; shu4; shuo4; su4
U+55FD.0	fEnglish	 cough; clear the throat
U+55FE.0	fUTF8	嗾
U+55FE.0	fCNS11643	1-6826
U+55FE.0	fBig5	B9BE
U+55FE.0	fGB	0-6055
U+55FE.0	fR/S	 30.11
U+55FE.0	fTotalStrokes	 14
U+55FE.0	fCangjie	RYSK
U+55FE.0	fFourCorner	68034
U+55FE.0	fHakka	  seu5; cuk8; cuk7
U+55FE.0	fCantonese	sau2; zuk6 (also)
U+55FE.0	fMandarin	sou3; zu2; sou4
U+55FE.0	fEnglish	[1] give vocal signals to a dog [2] instigate
U+55FF.0	fUTF8	嗿
U+55FF.0	fCNS11643	2-493B
U+55FF.0	fBig5	E154
U+55FF.0	fGB	3-3657
U+55FF.0	fR/S	 30.11
U+55FF.0	fTotalStrokes	 14
U+55FF.0	fCangjie	ROIC
U+55FF.0	fFourCorner	68086
U+55FF.0	fHakka	  tam3
U+55FF.0	fCantonese	taam2
U+55FF.0	fMandarin	tan3
U+55FF.0	fEnglish	gobble
U+5600.0	fUTF8	嘀
U+5600.0	fCNS11643	1-6827
U+5600.0	fBig5	B9BF
U+5600.0	fGB	0-6056
U+5600.0	fR/S	 30.11
U+5600.0	fTotalStrokes	 14
U+5600.0	fCangjie	RYCB
U+5600.0	fFourCorner	60027
U+5600.0	fHakka	 zak8; dit7
U+5600.0	fCantonese	dik6; dik1 (also)
U+5600.0	fMandarin	di2; di1; shang1
U+5600.0	fEnglish	backbite
U+5601.0	fUTF8	嘁
U+5601.0	fCNS11643	2-4935
U+5601.0	fBig5	E14E
U+5601.0	fGB	0-6052
U+5601.0	fR/S	 30.11
U+5601.0	fTotalStrokes	 14
U+5601.0	fCangjie	RIHF
U+5601.0	fFourCorner	63050
U+5601.0	fHakka	  zap7
U+5601.0	fCantonese	ci1; cik1 (also)
U+5601.0	fMandarin	qi1; za1
U+5601.0	fEnglish	 be grieved ashamed used to imitate sounds
U+5602.0	fUTF8	嘂
U+5602.0	fCNS11643	2-4937
U+5602.0	fBig5	E150
U+5602.0	fGB	3-3727
U+5602.0	fR/S	 30.11
U+5602.0	fTotalStrokes	 14
U+5602.0	fCangjie	RRVLR
U+5602.0	fFourCorner	66660
U+5602.0	fCantonese	giu3
U+5602.0	fMandarin	jiao4
U+5602.0	fEnglish	cry loudly; yell; scream
U+5603.0	fUTF8	嘃
U+5603.0	fCNS11643	3-4629
U+5603.0	fGB	E-2471
U+5603.0	fR/S	  30.11
U+5603.0	fTotalStrokes	 14
U+5603.0	fCangjie	RILB
U+5603.0	fFourCorner	60020
U+5603.0	fCantonese	cung1
U+5603.0	fMandarin	chong1
U+5604.0	fUTF8	嘄
U+5604.0	fCNS11643	2-493A
U+5604.0	fBig5	E153
U+5604.0	fGB	5-3868
U+5604.0	fR/S	 30.11
U+5604.0	fTotalStrokes	 14
U+5604.0	fCangjie	RHAD
U+5604.0	fFourCorner	67027
U+5604.0	fCantonese	hiu1; giu3; tou2
U+5604.0	fMandarin	jiao1; jiao4
U+5605.0	fUTF8	嘅
U+5605.0	fCNS11643	3-4636
U+5605.0	fGB	1-7C31
U+5605.0	fR/S	 30.11
U+5605.0	fTotalStrokes	 14
U+5605.0	fCangjie	RHPU
U+5605.0	fFourCorner	61010
U+5605.0	fHakka	  koi3;  kai3; gai5
U+5605.0	fCantonese	ge3; ge2; koi3
U+5605.0	fMandarin	kai4; kai3; ge2
U+5605.0	fEnglish	[1] sound of sighing [2] possessive [3] used in combination to form what: mak7kai5
U+5606.0	fUTF8	嘆
U+5606.0	fCNS11643	1-682C
U+5606.0	fBig5	B9C4
U+5606.0	fGB	1-4C3E
U+5606.0	fR/S	 30.11
U+5606.0	fTotalStrokes	 14
U+5606.0	fCangjie	RTLO
U+5606.0	fFourCorner	64034
U+5606.0	fHakka	  tan5
U+5606.0	fCantonese	taan3; taan1
U+5606.0	fMandarin	tan4; tan1
U+5606.0	fEnglish	sigh; admire
U+5607.0	fUTF8	嘇
U+5607.0	fCNS11643	3-462C
U+5607.0	fGB	3-3660
U+5607.0	fR/S	  30.11
U+5607.0	fTotalStrokes	 14
U+5607.0	fCangjie	RIIH
U+5607.0	fFourCorner	63020
U+5607.0	fCantonese	saam1
U+5607.0	fMandarin	san1
U+5608.0	fUTF8	嘈
U+5608.0	fCNS11643	1-6833
U+5608.0	fBig5	B9CB
U+5608.0	fGB	0-6050
U+5608.0	fR/S	 30.11
U+5608.0	fTotalStrokes	 14
U+5608.0	fCangjie	RTWA
U+5608.0	fFourCorner	64067
U+5608.0	fHakka	  cau2; cau3
U+5608.0	fCantonese	cou4
U+5608.0	fMandarin	cao2; qiao2; zao4
U+5608.0	fEnglish	noisy; clamorous
U+5609.0	fUTF8	嘉
U+5609.0	fCNS11643	1-682D
U+5609.0	fBig5	B9C5
U+5609.0	fGB	0-3C4E
U+5609.0	fR/S	 30.11
U+5609.0	fTotalStrokes	 14
U+5609.0	fCangjie	GRTR
U+5609.0	fFourCorner	40461
U+5609.0	fHakka	  ga1
U+5609.0	fCantonese	gaa1
U+5609.0	fMandarin	jia1; xia3
U+5609.0	fEnglish	[1] excellent; auspicious; good; fine [2] praise; commend; admire
U+560A.0	fUTF8	嘊
U+560A.0	fCNS11643	4-4257
U+560A.0	fGB	5-393D
U+560A.0	fR/S	  30.11
U+560A.0	fTotalStrokes	 14
U+560A.0	fCangjie	RUMG
U+560A.0	fFourCorner	62010
U+560A.0	fCantonese	ngaai4; aai4 (also)
U+560A.0	fMandarin	ai2
U+560B.0	fUTF8	嘋
U+560B.0	fCNS11643	3-462A
U+560B.0	fGB	5-393A
U+560B.0	fR/S	  30.11
U+560B.0	fTotalStrokes	 14
U+560B.0	fCangjie	RJDK
U+560B.0	fFourCorner	68040
U+560B.0	fCantonese	haau1
U+560B.0	fMandarin	xiao1
U+560C.0	fUTF8	嘌
U+560C.0	fCNS11643	2-4930
U+560C.0	fBig5	E149
U+560C.0	fGB	0-6051
U+560C.0	fR/S	 30.11
U+560C.0	fTotalStrokes	 14
U+560C.0	fCangjie	RMWF
U+560C.0	fFourCorner	61091
U+560C.0	fHakka	 piau1; peu1
U+560C.0	fCantonese	piu1; piu3
U+560C.0	fMandarin	piao1; piao4; biao1
U+560C.0	fEnglish	appearance of chariots wildly driven
U+560D.0	fUTF8	嘍
U+560D.0	fCNS11643	1-682E
U+560D.0	fBig5	B9C6
U+560D.0	fGB	1-6036
U+560D.0	fR/S	 30.11
U+560D.0	fTotalStrokes	 14
U+560D.0	fCangjie	RLLV
U+560D.0	fFourCorner	65044
U+560D.0	fHakka	  leu2; leu5; liu2; leu3
U+560D.0	fCantonese	lau4; lau1
U+560D.0	fMandarin	lou3
U+560D.0	fEnglish	a phrase-final particle; used in onomatopoetic expression
U+560D.1	fUTF8	嘍
U+560D.1	fCNS11643	1-682E
U+560D.1	fBig5	B9C6
U+560D.1	fGB	1-6036
U+560D.1	fR/S	 30.11
U+560D.1	fTotalStrokes	 14
U+560D.1	fCangjie	RLLV
U+560D.1	fFourCorner	65044
U+560D.1	fMandarin	lou5
U+560D.1	fEnglish	 bandit's lackey or follower
U+560D.2	fUTF8	嘍
U+560D.2	fCNS11643	1-682E
U+560D.2	fBig5	B9C6
U+560D.2	fGB	1-6036
U+560D.2	fR/S	 30.11
U+560D.2	fTotalStrokes	 14
U+560D.2	fCangjie	RLLV
U+560D.2	fFourCorner	65044
U+560D.2	fMandarin	lou2
U+560D.2	fEnglish	 call fowl
U+560E.0	fUTF8	嘎
U+560E.0	fCNS11643	1-682F
U+560E.0	fBig5	B9C7
U+560E.0	fGB	0-3842
U+560E.0	fR/S	 30.11
U+560E.0	fTotalStrokes	 14
U+560E.0	fCangjie	RMUI
U+560E.0	fFourCorner	61053
U+560E.0	fHakka	 kat7; ga1
U+560E.0	fCantonese	gaa1 (also); gaat3; gaa4; gaa2; gat1
U+560E.0	fMandarin	ga1; ga3; ga2
U+560E.0	fEnglish	[1] sound of laughter bad; malevole [2] this way; so
U+560F.0	fUTF8	嘏
U+560F.0	fCNS11643	2-4933
U+560F.0	fBig5	E14C
U+560F.0	fGB	0-5845
U+560F.0	fR/S	 30.11
U+560F.0	fTotalStrokes	 14
U+560F.0	fCangjie	JRRSE
U+560F.0	fFourCorner	47647
U+560F.0	fHakka	 gu3; ga5
U+560F.0	fCantonese	gu2; gaa2 (also); gwu2 (also)
U+560F.0	fMandarin	gu3; jia3
U+560F.0	fEnglish	felicity; blessings; prosperity; happiness
U+5610.0	fUTF8	嘐
U+5610.0	fCNS11643	1-6834
U+5610.0	fBig5	B9CC
U+5610.0	fGB	3-372F
U+5610.0	fR/S	 30.11
U+5610.0	fTotalStrokes	 14
U+5610.0	fCangjie	RSMH
U+5610.0	fFourCorner	67022
U+5610.0	fHakka	  gau1; lau2;  hau1; lau5
U+5610.0	fCantonese	haau1; gaau1
U+5610.0	fMandarin	xiao1; jiao1; bao4; lao2
U+5610.0	fEnglish	final exclamatory particle
U+5611.0	fUTF8	嘑
U+5611.0	fCNS11643	3-4634
U+5611.0	fGB	E-2472
U+5611.0	fR/S	 30.11
U+5611.0	fTotalStrokes	 14
U+5611.0	fCangjie	RYPD
U+5611.0	fFourCorner	61040
U+5611.0	fHakka	  fu1; fu5
U+5611.0	fCantonese	fu3; laa3; fu1
U+5611.0	fMandarin	hu1; hu4; la4
U+5611.0	fEnglish	 menace howl at to bawl
U+5612.0	fUTF8	嘒
U+5612.0	fCNS11643	2-4931
U+5612.0	fBig5	E14A
U+5612.0	fGB	3-3722
U+5612.0	fR/S	 30.11
U+5612.0	fTotalStrokes	 14
U+5612.0	fCangjie	RQJM
U+5612.0	fFourCorner	65071
U+5612.0	fHakka	 fi5; woi6; woi5; fui6; fui5; voi5
U+5612.0	fCantonese	wai3
U+5612.0	fMandarin	hui4
U+5612.0	fEnglish	sharp
U+5613.0	fUTF8	嘓
U+5613.0	fCNS11643	2-4936
U+5613.0	fBig5	E14F
U+5613.0	fGB	3-3651
U+5613.0	fR/S	 30.11
U+5613.0	fTotalStrokes	 14
U+5613.0	fCangjie	RWIM
U+5613.0	fFourCorner	66000
U+5613.0	fHakka	 kwet7; get8; get7; gok7; gwet8; gwet7
U+5613.0	fCantonese	gwok3; gok3 (also)
U+5613.0	fMandarin	guo1
U+5613.0	fEnglish	gurgling sound; chattering
U+5614.0	fUTF8	嘔
U+5614.0	fCNS11643	1-682B
U+5614.0	fBig5	B9C3
U+5614.0	fGB	1-453B
U+5614.0	fR/S	 30.11
U+5614.0	fTotalStrokes	 14
U+5614.0	fCangjie	RSRR
U+5614.0	fFourCorner	61016
U+5614.0	fHakka	  eu3; iu3
U+5614.0	fCantonese	au2; au1; ngau2 (also); ngau1 (also); au3; heoi1; ngau3 (also)
U+5614.0	fMandarin	ou3; ou1; xu1; yu4; ou5
U+5614.0	fEnglish	 vomit; throw up
U+5614.1	fUTF8	嘔
U+5614.1	fCNS11643	1-682B
U+5614.1	fBig5	B9C3
U+5614.1	fGB	1-453B
U+5614.1	fR/S	 30.11
U+5614.1	fTotalStrokes	 14
U+5614.1	fCangjie	RSRR
U+5614.1	fFourCorner	61016
U+5614.1	fMandarin	ou4
U+5614.1	fEnglish	annoy on purpose
U+5615.0	fUTF8	嘕
U+5615.0	fCNS11643	2-492F
U+5615.0	fBig5	E148
U+5615.0	fGB	3-3723
U+5615.0	fR/S	 30.11
U+5615.0	fTotalStrokes	 14
U+5615.0	fCangjie	RMYF
U+5615.0	fFourCorner	61027
U+5615.0	fCantonese	hin1
U+5615.0	fMandarin	xian1; qian1; yan1
U+5616.0	fUTF8	嘖
U+5616.0	fCNS11643	1-6831
U+5616.0	fBig5	B9C9
U+5616.0	fGB	1-5F75
U+5616.0	fR/S	 30.11
U+5616.0	fTotalStrokes	 14
U+5616.0	fCangjie	RQMC
U+5616.0	fFourCorner	65086
U+5616.0	fHakka	  zit7; zip8
U+5616.0	fCantonese	zaak3; zik1 (also); zaak6
U+5616.0	fMandarin	ze2
U+5616.0	fEnglish	[1] interjection of approval or admiration [2] argue; dispute
U+5617.0	fUTF8	嘗
U+5617.0	fCNS11643	1-6829
U+5617.0	fBig5	B9C1
U+5617.0	fGB	1-3322
U+5617.0	fR/S	 30.11
U+5617.0	fTotalStrokes	 14
U+5617.0	fCangjie	FBRPA
U+5617.0	fFourCorner	90601
U+5617.0	fHakka	  song2;  shong2
U+5617.0	fCantonese	soeng4
U+5617.0	fMandarin	chang2
U+5617.0	fEnglish	[1] taste [2] experience [3] try; experiment
U+5618.0	fUTF8	嘘
U+5618.0	fCNS11643	3-4633
U+5618.0	fGB	0-506A
U+5618.0	fR/S	 30.11
U+5618.0	fTotalStrokes	 14
U+5618.0	fCangjie	RYPC
U+5618.0	fFourCorner	61010
U+5618.0	fHakka	 hi1
U+5618.0	fCantonese	heoi1; syu4
U+5618.0	fMandarin	xu1; shi1
U+5618.0	fEnglish	[1] warm with exhaled air [2] speak well of (another); praise [3] deep sigh [4] hiss; boo
U+5619.0	fUTF8	嘙
U+5619.0	fCNS11643	4-424B
U+5619.0	fGB	E-2473
U+5619.0	fR/S	  30.11
U+5619.0	fTotalStrokes	 14
U+5619.0	fCangjie	REEV
U+5619.0	fFourCorner	64040
U+5619.0	fCantonese	po4
U+5619.0	fMandarin	po2
U+561A.0	fUTF8	嘚
U+561A.0	fGB	3-3729
U+561A.0	fR/S	 30.11
U+561A.0	fTotalStrokes	 14
U+561A.0	fCangjie	RHOI
U+561A.0	fFourCorner	6604
U+561A.0	fCantonese	dak1
U+561A.0	fMandarin	de2; de1; dai1
U+561B.0	fUTF8	嘛
U+561B.0	fCNS11643	1-6828
U+561B.0	fBig5	B9C0
U+561B.0	fGB	0-426F
U+561B.0	fR/S	 30.11
U+561B.0	fTotalStrokes	 14
U+561B.0	fCangjie	RIJC
U+561B.0	fFourCorner	60094
U+561B.0	fHakka	 ma2
U+561B.0	fCantonese	maa3; maa4; maa1
U+561B.0	fMandarin	ma5
U+561B.0	fEnglish	final exclamatory particle
U+561B.1	fUTF8	嘛
U+561B.1	fCNS11643	1-6828
U+561B.1	fBig5	B9C0
U+561B.1	fGB	0-426F
U+561B.1	fR/S	 30.11
U+561B.1	fTotalStrokes	 14
U+561B.1	fCangjie	RIJC
U+561B.1	fFourCorner	60094
U+561B.1	fMandarin	ma2
U+561C.0	fUTF8	嘜
U+561C.0	fCNS11643	2-4934
U+561C.0	fBig5	E14D
U+561C.0	fGB	1-5F69
U+561C.0	fR/S	 30.11
U+561C.0	fTotalStrokes	 14
U+561C.0	fCangjie	RJON
U+561C.0	fFourCorner	64027
U+561C.0	fHakka	 mak8
U+561C.0	fCantonese	mak1
U+561C.0	fMandarin	ma4; ma3; mo4
U+561C.0	fEnglish	mark
U+561D.0	fUTF8	嘝
U+561D.0	fCNS11643	2-4939
U+561D.0	fBig5	E152
U+561D.0	fGB	E-2474
U+561D.0	fR/S	 30.11
U+561D.0	fTotalStrokes	 14
U+561D.0	fCangjie	RNBJ
U+561D.0	fFourCorner	64000
U+561D.0	fCantonese	huk6
U+561D.0	fMandarin	hu2
U+561E.0	fUTF8	嘞
U+561E.0	fCNS11643	3-4631
U+561E.0	fGB	0-604F
U+561E.0	fR/S	 30.11
U+561E.0	fTotalStrokes	 14
U+561E.0	fCangjie	RTJS
U+561E.0	fFourCorner	64020
U+561E.0	fCantonese	laak3 (also); laa3; laa1; lak3
U+561E.0	fMandarin	le1; lei5
U+561E.0	fEnglish	 [final particle] used as a sign of polite refusal, completed action, or a transition from one situation to another
U+561F.0	fUTF8	嘟
U+561F.0	fCNS11643	1-6832
U+561F.0	fBig5	B9CA
U+561F.0	fGB	0-603D
U+561F.0	fR/S	 30.12
U+561F.0	fTotalStrokes	 15
U+561F.0	fCangjie	RJAL
U+561F.0	fFourCorner	67027
U+561F.0	fHakka	 du1
U+561F.0	fCantonese	dou1
U+561F.0	fMandarin	du1
U+561F.0	fEnglish	sound of horn tooting
U+5620.0	fUTF8	嘠
U+5620.0	fCNS11643	3-645A
U+5620.0	fGB	E-2475
U+5620.0	fR/S	 30.12
U+5620.0	fTotalStrokes	 15
U+5620.0	fCangjie	RMBI
U+5620.0	fFourCorner	6105
U+5620.0	fHakka	 ga1
U+5620.0	fCantonese	gaat3; gaa1 (also)
U+5620.0	fMandarin	ga1; ga3
U+5620.0	fEnglish	the chirping of birds loud laughter
U+5621.0	fUTF8	嘡
U+5621.0	fGB	3-3726
U+5621.0	fR/S	 30.11
U+5621.0	fTotalStrokes	 14
U+5621.0	fCangjie	RFBG
U+5621.0	fCantonese	tong1
U+5621.0	fMandarin	tang1
U+5622.0	fUTF8	嘢
U+5622.0	fCNS11643	3-6459
U+5622.0	fGB	1-7C34
U+5622.0	fR/S	 30.11
U+5622.0	fTotalStrokes	 14
U+5622.0	fCangjie	RWGN
U+5622.0	fFourCorner	6702
U+5622.0	fCantonese	je5
U+5622.0	fMandarin	ye3
U+5622.0	fEnglish	[1] thing; matter [2] disgusting fellow; annoying person; naughty child
U+5623.0	fUTF8	嘣
U+5623.0	fGB	0-6054
U+5623.0	fR/S	 30.11
U+5623.0	fTotalStrokes	 14
U+5623.0	fCangjie	RUBB
U+5623.0	fCantonese	bang1
U+5623.0	fMandarin	beng1
U+5623.0	fEnglish	syllable
U+5624.0	fUTF8	嘤
U+5624.0	fGB	0-6053
U+5624.0	fR/S	 30.11
U+5624.0	fTotalStrokes	 14
U+5624.0	fCangjie	RBOV
U+5624.0	fHakka	 jin1
U+5624.0	fCantonese	jing1; ang1
U+5624.0	fMandarin	ying1
U+5624.0	fEnglish	simplified form of &#x56B6;
U+5625.0	fUTF8	嘥
U+5625.0	fGB	1-7C35
U+5625.0	fR/S	 30.11
U+5625.0	fTotalStrokes	 14
U+5625.0	fCangjie	RHOO
U+5625.0	fCantonese	saai1; saai4; saai3
U+5625.0	fEnglish	[1] [2] waste [3] all; entirely
U+5626.0	fUTF8	嘦
U+5626.0	fCNS11643	3-4650
U+5626.0	fGB	3-372B
U+5626.0	fR/S	 38.10; 30.11; 38.11
U+5626.0	fTotalStrokes	 14
U+5626.0	fCangjie	RCMWV
U+5626.0	fFourCorner	60400
U+5626.0	fCantonese	ziu3
U+5626.0	fMandarin	jiao4
U+5627.0	fUTF8	嘧
U+5627.0	fCNS11643	2-492E
U+5627.0	fBig5	E147
U+5627.0	fGB	0-6057
U+5627.0	fR/S	 30.11
U+5627.0	fTotalStrokes	 14
U+5627.0	fCangjie	RJPU
U+5627.0	fFourCorner	63073
U+5627.0	fCantonese	mat6
U+5627.0	fMandarin	mi4
U+5627.0	fEnglish	pyrimidine
U+5628.0	fUTF8	嘨
U+5628.0	fCNS11643	3-4630
U+5628.0	fGB	E-2476
U+5628.0	fR/S	 30.12
U+5628.0	fTotalStrokes	 14; 15
U+5628.0	fCangjie	RLLL
U+5628.0	fFourCorner	65020
U+5628.0	fCantonese	siu3
U+5628.0	fMandarin	xiao4; chi4
U+5628.0	fEnglish	 roar; howl; scream whistle
U+5629.0	fUTF8	嘩
U+5629.0	fCNS11643	1-6C37
U+5629.0	fBig5	BC4D
U+5629.0	fGB	1-3B29
U+5629.0	fR/S	 30.12
U+5629.0	fTotalStrokes	 15
U+5629.0	fCangjie	RTMJ
U+5629.0	fFourCorner	64054
U+5629.0	fHakka	 wa2
U+5629.0	fCantonese	waa4 (also); jip6; faa1
U+5629.0	fEnglish	an onomatopoeia, such as gurgle, clang, crack, etc.
U+5629.1	fUTF8	嘩
U+5629.1	fCNS11643	1-6C37
U+5629.1	fBig5	BC4D
U+5629.1	fGB	1-3B29
U+5629.1	fR/S	 30.12
U+5629.1	fTotalStrokes	 15
U+5629.1	fCangjie	RTMJ
U+5629.1	fFourCorner	64054
U+5629.1	fHakka	 fa2
U+5629.1	fCantonese	waa1
U+5629.1	fMandarin	hua2; hua1; wa2
U+5629.1	fEnglish	[1] tumult; hubbub; clamor; uproar [2] same as &#x8B41;
U+562A.0	fUTF8	嘪
U+562A.0	fCNS11643	2-4F6E
U+562A.0	fBig5	E547
U+562A.0	fGB	5-3846
U+562A.0	fR/S	 30.12
U+562A.0	fTotalStrokes	 15
U+562A.0	fCangjie	RWLC
U+562A.0	fFourCorner	66086
U+562A.0	fHakka	 mai3; mai1; moi1
U+562A.0	fCantonese	maai5
U+562A.0	fMandarin	mai3
U+562A.0	fEnglish	bleating of sheep
U+562B.0	fUTF8	嘫
U+562B.0	fCNS11643	3-4B41
U+562B.0	fGB	5-394D
U+562B.0	fR/S	  30.12
U+562B.0	fTotalStrokes	 15
U+562B.0	fCangjie	RBKF
U+562B.0	fFourCorner	63030
U+562B.0	fCantonese	jin4
U+562B.0	fMandarin	ran2
U+562C.0	fUTF8	嘬
U+562C.0	fCNS11643	2-4F6B
U+562C.0	fBig5	E544
U+562C.0	fGB	0-605C
U+562C.0	fR/S	 30.12
U+562C.0	fTotalStrokes	 15
U+562C.0	fCangjie	RASE
U+562C.0	fFourCorner	66047
U+562C.0	fHakka	  zui5; zot8; cai3; cai5
U+562C.0	fCantonese	caai3; zyut3 (also); ceoi3
U+562C.0	fMandarin	chuai4; zuo1
U+562C.0	fEnglish	[1] bite [2] gobble up [3] lap; suck
U+562D.0	fUTF8	嘭
U+562D.0	fCNS11643	3-4B38
U+562D.0	fGB	0-6058
U+562D.0	fR/S	 30.12
U+562D.0	fTotalStrokes	 15
U+562D.0	fCangjie	RGTH
U+562D.0	fFourCorner	62020
U+562D.0	fCantonese	paang4; baang4 (also); paang1
U+562D.0	fMandarin	peng1; peng2
U+562D.0	fEnglish	syllable
U+562E.0	fUTF8	嘮
U+562E.0	fCNS11643	1-6C31
U+562E.0	fBig5	BC47
U+562E.0	fGB	1-5F6B
U+562E.0	fR/S	 30.12
U+562E.0	fTotalStrokes	 15
U+562E.0	fCangjie	RFFS
U+562E.0	fFourCorner	69027
U+562E.0	fHakka	 lau2; la3
U+562E.0	fCantonese	lou4; lou6
U+562E.0	fMandarin	lao2; lao4; cha1; chang4; chao1; xiao1
U+562E.0	fEnglish	loquacious; garrulous; voluble
U+562F.0	fUTF8	嘯
U+562F.0	fCNS11643	1-6C3D
U+562F.0	fBig5	BC53
U+562F.0	fGB	1-5025
U+562F.0	fR/S	 30.12
U+562F.0	fTotalStrokes	 15
U+562F.0	fCangjie	RLX
U+562F.0	fFourCorner	65027
U+562F.0	fHakka	 siau5; sau5; siu5; seu5
U+562F.0	fCantonese	siu3; cik1
U+562F.0	fMandarin	xiao4; chi4; su4
U+562F.0	fEnglish	[1] roar; howl; scream; cry or shout in a sustained voice [2] whistle
U+5630.0	fUTF8	嘰
U+5630.0	fCNS11643	1-6C3E
U+5630.0	fBig5	BC54
U+5630.0	fGB	1-5F34
U+5630.0	fR/S	 30.12
U+5630.0	fTotalStrokes	 15
U+5630.0	fCangjie	RVII
U+5630.0	fFourCorner	62053
U+5630.0	fCantonese	gei1
U+5630.0	fMandarin	ji1; ji4
U+5630.0	fEnglish	[1] talk indistinctly in a low voice [2] chirp
U+5631.0	fUTF8	嘱
U+5631.0	fCNS11643	3-4B3D
U+5631.0	fGB	0-5676
U+5631.0	fR/S	  30.12
U+5631.0	fTotalStrokes	 15
U+5631.0	fCangjie	RSHB
U+5631.0	fFourCorner	67020
U+5631.0	fHakka	 zhuk7; chuk7
U+5631.0	fCantonese	zuk1
U+5631.0	fMandarin	zhu3; bao4
U+5631.0	fEnglish	simplified form of &#x56D1;
U+5632.0	fUTF8	嘲
U+5632.0	fCNS11643	1-6C34
U+5632.0	fBig5	BC4A
U+5632.0	fGB	0-3330
U+5632.0	fR/S	 30.12
U+5632.0	fTotalStrokes	 15
U+5632.0	fCangjie	RJJB
U+5632.0	fFourCorner	67020
U+5632.0	fHakka	 zau1; sau1; sau3; sau2; cau2
U+5632.0	fCantonese	zaau1
U+5632.0	fMandarin	chao2; zhao1
U+5632.0	fEnglish	 ridicule; deride; jeer; sneer; scoff; mock
U+5633.0	fUTF8	嘳
U+5633.0	fCNS11643	2-4F69
U+5633.0	fBig5	E542
U+5633.0	fGB	5-3878
U+5633.0	fR/S	 30.12
U+5633.0	fTotalStrokes	 15
U+5633.0	fCangjie	RLMC
U+5633.0	fFourCorner	65086
U+5633.0	fCantonese	kwaai3 (also); wai2
U+5633.0	fMandarin	kui4; kuai4
U+5634.0	fUTF8	嘴
U+5634.0	fCNS11643	1-6C36
U+5634.0	fBig5	BC4C
U+5634.0	fGB	0-576C
U+5634.0	fR/S	 30.12
U+5634.0	fTotalStrokes	 15
U+5634.0	fCangjie	RYPB
U+5634.0	fFourCorner	61027
U+5634.0	fHakka	  zui3;  zoi5; choi5; zhoi5; zui2
U+5634.0	fCantonese	zeoi2
U+5634.0	fMandarin	zui3
U+5634.0	fEnglish	[1] the mouth; lips; bill or beak (of a bird); snout (of a pig, etc.) [2] nozzle
U+5635.0	fUTF8	嘵
U+5635.0	fCNS11643	2-4F61
U+5635.0	fBig5	E4F9
U+5635.0	fGB	1-5F58
U+5635.0	fR/S	 30.12
U+5635.0	fTotalStrokes	 15
U+5635.0	fCangjie	RGGU
U+5635.0	fFourCorner	64011
U+5635.0	fHakka	 hiau1; kiau2
U+5635.0	fCantonese	hiu1
U+5635.0	fMandarin	xiao1
U+5635.0	fEnglish	 garrulous disturbed restless
U+5636.0	fUTF8	嘶
U+5636.0	fCNS11643	1-6C3C
U+5636.0	fBig5	BC52
U+5636.0	fGB	0-4B3B
U+5636.0	fR/S	 30.12
U+5636.0	fTotalStrokes	 15
U+5636.0	fCangjie	RTCL
U+5636.0	fFourCorner	62021
U+5636.0	fHakka	  se1;  sii1; sai1; su1
U+5636.0	fCantonese	sai1; si1 (also)
U+5636.0	fMandarin	si1; xi1
U+5636.0	fEnglish	[1] the neighing of a horse [2] hoarse (said of a voice)
U+5637.0	fUTF8	嘷
U+5637.0	fCNS11643	3-4B44
U+5637.0	fGB	E-2477
U+5637.0	fR/S	  30.12
U+5637.0	fTotalStrokes	 15
U+5637.0	fCangjie	RHUJ
U+5637.0	fFourCorner	66040
U+5637.0	fHakka	  hau2; hau1
U+5637.0	fCantonese	hou4
U+5637.0	fMandarin	hao2
U+5637.0	fEnglish	the roaring of wild beasts; the frantic barks or howl of dogs or wolves; howl; yelp
U+5638.0	fUTF8	嘸
U+5638.0	fCNS11643	2-4F6D
U+5638.0	fBig5	E546
U+5638.0	fGB	1-5F3C
U+5638.0	fR/S	 30.12
U+5638.0	fTotalStrokes	 15
U+5638.0	fCangjie	ROTF
U+5638.0	fFourCorner	68031
U+5638.0	fHakka	 wu3; mo2
U+5638.0	fCantonese	fu2; mou5; m4
U+5638.0	fMandarin	fu3; m1; wu3
U+5638.0	fEnglish	[1] an expletive [2] [Soochow, Shanghai] not [3] Wu] [4] not
U+5639.0	fUTF8	嘹
U+5639.0	fCNS11643	1-6C33
U+5639.0	fBig5	BC49
U+5639.0	fGB	0-605A
U+5639.0	fR/S	 30.12
U+5639.0	fTotalStrokes	 15
U+5639.0	fCangjie	RKCF
U+5639.0	fFourCorner	64096
U+5639.0	fHakka	 liau2
U+5639.0	fCantonese	liu4
U+5639.0	fMandarin	liao2; liao4
U+5639.0	fEnglish	resonant (said of a voice)
U+563A.0	fUTF8	嘺
U+563A.0	fCNS11643	2-4F6F
U+563A.0	fBig5	E548
U+563A.0	fGB	3-362A
U+563A.0	fR/S	 30.12
U+563A.0	fTotalStrokes	 15
U+563A.0	fCangjie	RHKB
U+563A.0	fFourCorner	62027
U+563A.0	fCantonese	kiu4; kiu2
U+563A.0	fMandarin	qiao2
U+563A.0	fEnglish	 coincidental
U+563B.0	fUTF8	嘻
U+563B.0	fCNS11643	1-6C32
U+563B.0	fBig5	BC48
U+563B.0	fGB	0-4E7B
U+563B.0	fR/S	 30.12
U+563B.0	fTotalStrokes	 15
U+563B.0	fCangjie	RGRR
U+563B.0	fFourCorner	64061
U+563B.0	fHakka	 hi1; si1
U+563B.0	fCantonese	hei1
U+563B.0	fMandarin	xi1; xi4; yi1
U+563B.0	fEnglish	[1] laughing happily [2] a interjection of grief or surprise (in literary text)
U+563C.0	fUTF8	嘼
U+563C.0	fCNS11643	4-4931
U+563C.0	fGB	5-3949
U+563C.0	fR/S	 30.12
U+563C.0	fTotalStrokes	 15
U+563C.0	fCangjie	RRWMR
U+563C.0	fFourCorner	66600
U+563C.0	fHakka	 hiuk7; no2
U+563C.0	fCantonese	cuk1
U+563C.0	fMandarin	xiu4; chu4; xu4
U+563C.0	fEnglish	animals which put the mouth to the ground when feeding
U+563D.0	fUTF8	嘽
U+563D.0	fCNS11643	2-4F6A
U+563D.0	fBig5	E543
U+563D.0	fGB	3-365C
U+563D.0	fR/S	 30.12
U+563D.0	fTotalStrokes	 15
U+563D.0	fCangjie	RRRJ
U+563D.0	fFourCorner	66056
U+563D.0	fHakka	  tan1
U+563D.0	fCantonese	taan1; cin2; zin2 (also)
U+563D.0	fMandarin	chan3; tan1; chan1; dan1; dan3; ta1
U+563D.0	fEnglish	 pant many
U+563E.0	fUTF8	嘾
U+563E.0	fCNS11643	2-4F6C
U+563E.0	fBig5	E545
U+563E.0	fGB	3-3731
U+563E.0	fR/S	 30.12
U+563E.0	fTotalStrokes	 15
U+563E.0	fCangjie	RMWJ
U+563E.0	fFourCorner	61046
U+563E.0	fCantonese	daam6; taam4; taam5
U+563E.0	fMandarin	tan2; dan4
U+563F.0	fUTF8	嘿
U+563F.0	fCNS11643	1-6C35
U+563F.0	fBig5	BC4B
U+563F.0	fGB	0-3A59
U+563F.0	fR/S	 30.12
U+563F.0	fTotalStrokes	 15
U+563F.0	fCangjie	RWGF
U+563F.0	fFourCorner	66031
U+563F.0	fHakka	 met8; het7; met7
U+563F.0	fCantonese	hei1; mak6
U+563F.0	fMandarin	hei1
U+563F.0	fEnglish	[1] speechless; silent [2] quiet; still
U+563F.1	fUTF8	嘿
U+563F.1	fCNS11643	1-6C35
U+563F.1	fBig5	BC4B
U+563F.1	fGB	0-3A59
U+563F.1	fR/S	 30.12
U+563F.1	fTotalStrokes	 15
U+563F.1	fCangjie	RWGF
U+563F.1	fFourCorner	66031
U+563F.1	fMandarin	mo4
U+563F.1	fEnglish	an interjection; hey!
U+5640.0	fUTF8	噀
U+5640.0	fCNS11643	2-4F68
U+5640.0	fBig5	E541
U+5640.0	fGB	3-373A
U+5640.0	fR/S	 30.12
U+5640.0	fTotalStrokes	 15
U+5640.0	fCangjie	RRUC
U+5640.0	fFourCorner	67081
U+5640.0	fHakka	  sun5
U+5640.0	fCantonese	seon3
U+5640.0	fMandarin	xun4; sun4
U+5640.0	fEnglish	spirt out of the mouth
U+5641.0	fUTF8	噁
U+5641.0	fCNS11643	2-4F62
U+5641.0	fBig5	E4FA
U+5641.0	fGB	1-7830
U+5641.0	fR/S	 30.12
U+5641.0	fTotalStrokes	 15
U+5641.0	fCangjie	RMMP
U+5641.0	fFourCorner	61031
U+5641.0	fHakka	 ok7; wu5
U+5641.0	fCantonese	ok3; ngok3 (also); wu3
U+5641.0	fMandarin	e3; wu4; e4; wu1
U+5641.0	fEnglish	 nastiness nauseate; sickening
U+5642.0	fUTF8	噂
U+5642.0	fCNS11643	2-4F5F
U+5642.0	fBig5	E4F7
U+5642.0	fGB	3-3739
U+5642.0	fR/S	 30.12
U+5642.0	fTotalStrokes	 15
U+5642.0	fCangjie	RTWI
U+5642.0	fFourCorner	68046
U+5642.0	fHakka	  zun3
U+5642.0	fCantonese	zyun2
U+5642.0	fMandarin	zun3
U+5642.0	fEnglish	meet
U+5643.0	fUTF8	噃
U+5643.0	fCNS11643	4-492E
U+5643.0	fGB	5-394C
U+5643.0	fR/S	 30.12
U+5643.0	fTotalStrokes	 15
U+5643.0	fCangjie	RHDW
U+5643.0	fFourCorner	62060
U+5643.0	fHakka	 fan1
U+5643.0	fCantonese	bo3
U+5643.0	fMandarin	fan1; bo5
U+5643.0	fEnglish	[1] provoke [2] abuse [3] express idea of repeating or reminding
U+5644.0	fUTF8	噄
U+5644.0	fCNS11643	3-4B3E
U+5644.0	fGB	5-3944
U+5644.0	fR/S	 30.12
U+5644.0	fTotalStrokes	 15
U+5644.0	fCangjie	RQHF
U+5644.0	fFourCorner	67090
U+5644.0	fCantonese	hek3
U+5644.0	fMandarin	chi1
U+5645.0	fUTF8	噅
U+5645.0	fCNS11643	2-3A4F
U+5645.0	fBig5	D86B
U+5645.0	fGB	3-3561
U+5645.0	fR/S	 30.12; 30.9
U+5645.0	fTotalStrokes	 12
U+5645.0	fCangjie	RIKF
U+5645.0	fFourCorner	62027
U+5645.0	fCantonese	fai1
U+5645.0	fMandarin	hui1; kui1
U+5645.0	fEnglish	 speak falsely or wrongly ugly
U+5646.0	fUTF8	噆
U+5646.0	fCNS11643	2-4F65
U+5646.0	fBig5	E4FD
U+5646.0	fGB	3-3732
U+5646.0	fR/S	 30.12
U+5646.0	fTotalStrokes	 15
U+5646.0	fCangjie	RMUA
U+5646.0	fFourCorner	61061
U+5646.0	fHakka	  cam3;  zap7
U+5646.0	fCantonese	cam2; zam2 (also); caam2 (also); cam3; zaam1; zaap3
U+5646.0	fMandarin	can3; zan3
U+5646.0	fEnglish	 suck
U+5647.0	fUTF8	噇
U+5647.0	fCNS11643	3-4B37
U+5647.0	fGB	3-3737
U+5647.0	fR/S	  30.12
U+5647.0	fTotalStrokes	 15
U+5647.0	fCangjie	RYTG
U+5647.0	fFourCorner	60010
U+5647.0	fCantonese	zong6 (also); cong4
U+5647.0	fMandarin	chuang2
U+5648.0	fUTF8	噈
U+5648.0	fCNS11643	2-4F5E
U+5648.0	fBig5	E4F6
U+5648.0	fGB	3-3735
U+5648.0	fR/S	 30.12
U+5648.0	fTotalStrokes	 15
U+5648.0	fCangjie	RYFU
U+5648.0	fFourCorner	63014
U+5648.0	fHakka	 cuk7; ci3
U+5648.0	fCantonese	zuk1; zaap3; hap6; cuk1
U+5648.0	fMandarin	cu4; he2; za1
U+5648.0	fEnglish	 frown
U+5649.0	fUTF8	噉
U+5649.0	fCNS11643	2-4F64
U+5649.0	fBig5	E4FC
U+5649.0	fGB	E-2478
U+5649.0	fR/S	 30.12
U+5649.0	fTotalStrokes	 15
U+5649.0	fCangjie	RMJK
U+5649.0	fFourCorner	68040
U+5649.0	fHakka	 tam2; tam5; tam6
U+5649.0	fCantonese	gam2; gum2 (also); daam6
U+5649.0	fMandarin	dan4; dan3
U+5649.0	fEnglish	[1] bite, chew [2] well; very well in this way; in that case; like this; thus; so; well; then
U+564A.0	fUTF8	噊
U+564A.0	fCNS11643	2-4F63
U+564A.0	fBig5	E4FB
U+564A.0	fGB	3-373B
U+564A.0	fR/S	 30.12
U+564A.0	fTotalStrokes	 15
U+564A.0	fCangjie	RNHB
U+564A.0	fFourCorner	67027
U+564A.0	fHakka	 lut8; jut8; rut8
U+564A.0	fCantonese	jyut6; wat6; leot6
U+564A.0	fMandarin	yu4
U+564A.0	fEnglish	the note of a bird
U+564B.0	fUTF8	噋
U+564B.0	fCNS11643	3-4B3B
U+564B.0	fGB	3-3736
U+564B.0	fR/S	  30.12
U+564B.0	fTotalStrokes	 15
U+564B.0	fCangjie	RYDK
U+564B.0	fFourCorner	68040
U+564B.0	fHakka	 dun1
U+564B.0	fCantonese	teon1
U+564B.0	fMandarin	tun1; kuo4
U+564B.0	fEnglish	 swallow hastily gobble up
U+564C.0	fUTF8	噌
U+564C.0	fCNS11643	2-4F60
U+564C.0	fBig5	E4F8
U+564C.0	fGB	0-6061
U+564C.0	fR/S	 30.12; 30.11
U+564C.0	fTotalStrokes	 15
U+564C.0	fCangjie	RCWA
U+564C.0	fFourCorner	68066
U+564C.0	fHakka	  cen1; zen5; zen1
U+564C.0	fCantonese	zang1; cang1; caang1
U+564C.0	fMandarin	cheng1; ceng1; ceng2
U+564C.0	fEnglish	scold; shout at
U+564D.0	fUTF8	噍
U+564D.0	fCNS11643	3-4B43
U+564D.0	fGB	0-605D
U+564D.0	fR/S	  30.12
U+564D.0	fTotalStrokes	 15
U+564D.0	fCangjie	ROGF
U+564D.0	fFourCorner	60030
U+564D.0	fHakka	 ciau5
U+564D.0	fCantonese	ziu6; ziu3; ziu1; zau1
U+564D.0	fMandarin	jiao4
U+564D.0	fEnglish	to chew; eat; munch; nibble
U+564D.1	fUTF8	噍
U+564D.1	fCNS11643	3-4B43
U+564D.1	fGB	0-605D
U+564D.1	fR/S	  30.12
U+564D.1	fTotalStrokes	 15
U+564D.1	fCangjie	ROGF
U+564D.1	fFourCorner	60030
U+564D.1	fMandarin	jiao1
U+564D.1	fEnglish	high and unpleasant (voice)
U+564D.2	fUTF8	噍
U+564D.2	fCNS11643	3-4B43
U+564D.2	fGB	0-605D
U+564D.2	fR/S	  30.12
U+564D.2	fTotalStrokes	 15
U+564D.2	fCangjie	ROGF
U+564D.2	fFourCorner	60030
U+564D.2	fMandarin	jiu1
U+564D.2	fEnglish	chirping (birds)
U+564E.0	fUTF8	噎
U+564E.0	fCNS11643	1-6C39
U+564E.0	fBig5	BC4F
U+564E.0	fGB	0-522D
U+564E.0	fR/S	 30.12
U+564E.0	fTotalStrokes	 15
U+564E.0	fCangjie	RGBT
U+564E.0	fFourCorner	64018
U+564E.0	fHakka	 jit7; et7; rit7
U+564E.0	fCantonese	jit3
U+564E.0	fMandarin	ye1; yi1
U+564E.0	fEnglish	[1] choke off [2] be choked with food
U+564F.0	fUTF8	噏
U+564F.0	fCNS11643	3-4B45
U+564F.0	fGB	5-394B
U+564F.0	fR/S	  30.12
U+564F.0	fTotalStrokes	 15
U+564F.0	fCangjie	RORM
U+564F.0	fFourCorner	68027
U+564F.0	fHakka	 hip7;  kip7
U+564F.0	fCantonese	ngap1; kap1; ap1 (also)
U+564F.0	fMandarin	xi1; xi4
U+564F.0	fEnglish	 inhale; absorb; imbibe compress
U+5650.0	fUTF8	噐
U+5650.0	fCNS11643	3-4B40
U+5650.0	fGB	5-3948
U+5650.0	fR/S	 30.12
U+5650.0	fTotalStrokes	 15
U+5650.0	fCangjie	RRMRR
U+5650.0	fFourCorner	66660
U+5650.0	fHakka	 hi5;  ki5
U+5650.0	fCantonese	hei3
U+5650.0	fMandarin	qi4
U+5650.0	fEnglish	 receptacle; vessel instrument
U+5651.0	fUTF8	噑
U+5651.0	fCNS11643	3-4635
U+5651.0	fGB	E-2479
U+5651.0	fR/S	  30.11
U+5651.0	fTotalStrokes	 14
U+5651.0	fCangjie	RHAJ
U+5651.0	fFourCorner	66041
U+5651.0	fHakka	 hau2
U+5651.0	fCantonese	hou4
U+5651.0	fMandarin	hao2
U+5652.0	fUTF8	噒
U+5652.0	fCNS11643	3-4B3C
U+5652.0	fGB	E-247A
U+5652.0	fR/S	 30.12
U+5652.0	fTotalStrokes	 15
U+5652.0	fCangjie	RFDQ
U+5652.0	fFourCorner	69050
U+5652.0	fCantonese	leon1
U+5652.0	fMandarin	lian2; tan1
U+5652.0	fEnglish	to suck and gnaw on bones
U+5653.0	fUTF8	噓
U+5653.0	fCNS11643	1-6C38
U+5653.0	fBig5	BC4E
U+5653.0	fGB	E-247B
U+5653.0	fR/S	 30.11
U+5653.0	fTotalStrokes	 14
U+5653.0	fCangjie	RYPM
U+5653.0	fFourCorner	61017
U+5653.0	fHakka	 hi1
U+5653.0	fCantonese	heoi1; syu4
U+5653.0	fMandarin	xu1; shi1; xu4
U+5653.0	fEnglish	 exhale blow out deep sigh hiss praise; flatter lie
U+5654.0	fUTF8	噔
U+5654.0	fCNS11643	3-4B39
U+5654.0	fGB	0-6062
U+5654.0	fR/S	  30.12
U+5654.0	fTotalStrokes	 15
U+5654.0	fCangjie	RNOT
U+5654.0	fFourCorner	62010
U+5654.0	fHakka	 ngut7; jut7; tang3; rut7
U+5654.0	fCantonese	dang1; dang6
U+5654.0	fMandarin	deng1
U+5654.0	fEnglish	[1] name of a person [2] for (feelings of sympathy and pity)
U+5655.0	fUTF8	噕
U+5655.0	fCNS11643	7-2163
U+5655.0	fGB	1-7D69
U+5655.0	fR/S	 30.12; 87.11
U+5655.0	fTotalStrokes	 15
U+5655.0	fCangjie	BHFR
U+5655.0	fFourCorner	20600
U+5655.0	fCantonese	fai1
U+5655.0	fMandarin	hui1
U+5655.0	fEnglish	 speak falsely or wrongly ugly
U+5656.0	fUTF8	噖
U+5656.0	fCNS11643	3-4B3F
U+5656.0	fGB	5-3945
U+5656.0	fR/S	  30.12
U+5656.0	fTotalStrokes	 15
U+5656.0	fCangjie	RMGN
U+5656.0	fFourCorner	61020
U+5656.0	fCantonese	jam4
U+5656.0	fMandarin	yin2
U+5657.0	fUTF8	噗
U+5657.0	fCNS11643	1-6C3A
U+5657.0	fBig5	BC50
U+5657.0	fGB	0-605B
U+5657.0	fR/S	 30.12
U+5657.0	fTotalStrokes	 15
U+5657.0	fCangjie	RTCO
U+5657.0	fFourCorner	62035
U+5657.0	fHakka	 pok7
U+5657.0	fCantonese	pok3
U+5657.0	fMandarin	pu1
U+5657.0	fEnglish	burst
U+5658.0	fUTF8	噘
U+5658.0	fCNS11643	2-4F66
U+5658.0	fBig5	E4FE
U+5658.0	fGB	0-6059
U+5658.0	fR/S	 30.12
U+5658.0	fTotalStrokes	 15
U+5658.0	fCangjie	RMTO
U+5658.0	fFourCorner	61087
U+5658.0	fHakka	 kiet7; ket7
U+5658.0	fCantonese	kyut3
U+5658.0	fMandarin	jue1
U+5658.0	fEnglish	pouting
U+5659.0	fUTF8	噙
U+5659.0	fCNS11643	1-703B
U+5659.0	fBig5	BEB2
U+5659.0	fGB	0-605F
U+5659.0	fR/S	 30.12
U+5659.0	fTotalStrokes	 15
U+5659.0	fCangjie	ROYB
U+5659.0	fFourCorner	68027
U+5659.0	fHakka	  kim2
U+5659.0	fCantonese	kam4
U+5659.0	fMandarin	qin2
U+5659.0	fEnglish	[1] hold in mouth; bite [2] yesterday
U+565A.0	fUTF8	噚
U+565A.0	fCNS11643	2-4F67
U+565A.0	fBig5	E540
U+565A.0	fGB	E-247C
U+565A.0	fR/S	 30.12
U+565A.0	fTotalStrokes	 15
U+565A.0	fCangjie	RSMI
U+565A.0	fFourCorner	67046
U+565A.0	fCantonese	cam4; jing1 (also)
U+565A.0	fMandarin	xun2
U+565A.0	fEnglish	 yesterday
U+565B.0	fUTF8	噛
U+565B.0	fGB	E-247D
U+565B.0	fR/S	 30.12
U+565B.0	fTotalStrokes	 15
U+565B.0	fCangjie	XXRYM
U+565B.0	fCantonese	ngit6; it6 (also)
U+565B.0	fMandarin	nie4
U+565B.0	fEnglish	bite; gnaw
U+565C.0	fUTF8	噜
U+565C.0	fGB	0-6060
U+565C.0	fR/S	 30.12
U+565C.0	fTotalStrokes	 15
U+565C.0	fCangjie	XRNWA
U+565C.0	fCantonese	lou1
U+565C.0	fMandarin	lu1
U+565C.0	fEnglish	simplified form of &#x5695;
U+565D.0	fUTF8	噝
U+565D.0	fGB	1-5F50
U+565D.0	fR/S	 30.12
U+565D.0	fTotalStrokes	 15
U+565D.0	fCangjie	RVFF
U+565D.0	fCantonese	si1
U+565D.0	fMandarin	si1
U+565D.0	fEnglish	 hiss a call to come
U+565E.0	fUTF8	噞
U+565E.0	fCNS11643	2-564F
U+565E.0	fBig5	E945
U+565E.0	fGB	3-363B
U+565E.0	fR/S	 30.13
U+565E.0	fTotalStrokes	 16
U+565E.0	fCangjie	ROMO
U+565E.0	fFourCorner	68086
U+565E.0	fHakka	 ngiam5; ngiam6; ngiam1
U+565E.0	fCantonese	jim2; jim1 (also)
U+565E.0	fMandarin	yan4; yan3
U+565E.0	fEnglish	the movement of a fish's mouth at the surface of the water
U+565F.0	fUTF8	噟
U+565F.0	fCNS11643	3-503F
U+565F.0	fGB	5-3D46
U+565F.0	fR/S	  30.13
U+565F.0	fTotalStrokes	 16
U+565F.0	fCangjie	IOGR
U+565F.0	fFourCorner	30260
U+565F.0	fCantonese	jing3
U+565F.0	fMandarin	ying4
U+5660.0	fUTF8	噠
U+5660.0	fCNS11643	2-5648
U+5660.0	fBig5	E8FD
U+5660.0	fGB	1-5F55
U+5660.0	fR/S	 30.13
U+5660.0	fTotalStrokes	 16
U+5660.0	fCangjie	RYGQ
U+5660.0	fFourCorner	64034
U+5660.0	fHakka	 tat7
U+5660.0	fCantonese	daat3 (also); daat6; zim1
U+5660.0	fMandarin	da1; da2
U+5660.0	fEnglish	sound made get horse move forward
U+5661.0	fUTF8	噡
U+5661.0	fCNS11643	3-5040
U+5661.0	fGB	5-395A
U+5661.0	fR/S	 30.13
U+5661.0	fTotalStrokes	 16
U+5661.0	fCangjie	RNCR
U+5661.0	fFourCorner	67060
U+5661.0	fCantonese	zim1
U+5661.0	fMandarin	dan1; zhan1
U+5662.0	fUTF8	噢
U+5662.0	fCNS11643	1-7047
U+5662.0	fBig5	BEBE
U+5662.0	fGB	0-605E
U+5662.0	fR/S	 30.13
U+5662.0	fTotalStrokes	 16
U+5662.0	fCangjie	RHBK
U+5662.0	fFourCorner	67034
U+5662.0	fHakka	  juk7; ruk7; ji3; o1
U+5662.0	fCantonese	jyu2; ou3; ngo1 (also); ngou3 (also); o1; juk1
U+5662.0	fMandarin	yu3; o1; yu4; ou4; you3
U+5662.0	fEnglish	 moan interjection for pain; sad
U+5663.0	fUTF8	噣
U+5663.0	fCNS11643	2-564C
U+5663.0	fBig5	E942
U+5663.0	fGB	3-3741
U+5663.0	fR/S	 30.13
U+5663.0	fTotalStrokes	 16
U+5663.0	fCangjie	RWLI
U+5663.0	fFourCorner	66027
U+5663.0	fHakka	 zhiu5; zu5
U+5663.0	fCantonese	zau3; doek3; zyu1
U+5663.0	fMandarin	zhou4; zhuo2; dou4
U+5663.0	fEnglish	a beak; to peck; to order, enjoin upon
U+5664.0	fUTF8	噤
U+5664.0	fCNS11643	1-703F
U+5664.0	fBig5	BEB6
U+5664.0	fGB	0-6064
U+5664.0	fR/S	 30.13
U+5664.0	fTotalStrokes	 16
U+5664.0	fCangjie	RDDF
U+5664.0	fFourCorner	64091
U+5664.0	fHakka	  gim5
U+5664.0	fCantonese	gam3; kam1 (also); gam6
U+5664.0	fMandarin	jin4
U+5664.0	fEnglish	 keep the mouth shut; be silent shut up
U+5665.0	fUTF8	噥
U+5665.0	fCNS11643	1-7043
U+5665.0	fBig5	BEBA
U+5665.0	fGB	1-5F66
U+5665.0	fR/S	 30.13
U+5665.0	fTotalStrokes	 16
U+5665.0	fCangjie	RTWV
U+5665.0	fFourCorner	65032
U+5665.0	fHakka	  nung2; lung2
U+5665.0	fCantonese	nung4; lung4 (also)
U+5665.0	fMandarin	nong2; nong5
U+5665.0	fEnglish	whisper
U+5666.0	fUTF8	噦
U+5666.0	fCNS11643	2-564B
U+5666.0	fBig5	E941
U+5666.0	fGB	1-5F5C
U+5666.0	fR/S	 30.13
U+5666.0	fTotalStrokes	 16
U+5666.0	fCangjie	RYMH
U+5666.0	fFourCorner	61053
U+5666.0	fHakka	 oi5; we1; we2; woi5; wi3; wui3; vui3
U+5666.0	fCantonese	jyut6; wai3; jyut3
U+5666.0	fMandarin	hui4; yue3; yue1; huai4
U+5666.0	fEnglish	 belch vomit
U+5667.0	fUTF8	噧
U+5667.0	fCNS11643	3-503B
U+5667.0	fGB	3-354F
U+5667.0	fR/S	 30.13
U+5667.0	fTotalStrokes	 16
U+5667.0	fCangjie	RTWB
U+5667.0	fFourCorner	64020
U+5667.0	fCantonese	haai3
U+5667.0	fMandarin	hui4
U+5668.0	fUTF8	器
U+5668.0	fCNS11643	1-7042
U+5668.0	fBig5	BEB9
U+5668.0	fGB	0-4677
U+5668.0	fR/S	 30.13; 30.6
U+5668.0	fTotalStrokes	 16
U+5668.0	fCangjie	RRIKR
U+5668.0	fFourCorner	66663
U+5668.0	fHakka	  hi5;  ki5
U+5668.0	fCantonese	hei3
U+5668.0	fMandarin	qi4
U+5668.0	fEnglish	[1] an instrument; implement; utensil; tool; piece of apparatus [2] magnanimity [3] talent; ability; capacity [4] to think highly of (a person)
U+5669.0	fUTF8	噩
U+5669.0	fCNS11643	1-703E
U+5669.0	fBig5	BEB5
U+5669.0	fGB	0-582C
U+5669.0	fR/S	 30.13
U+5669.0	fTotalStrokes	 16
U+5669.0	fCangjie	MGRR
U+5669.0	fFourCorner	10106
U+5669.0	fHakka	 ngok8
U+5669.0	fCantonese	ngok6; ok6 (also)
U+5669.0	fMandarin	e4
U+5669.0	fEnglish	[1] startling awesome; dreadful; alarming [2] grave; serious
U+566A.0	fUTF8	噪
U+566A.0	fCNS11643	1-7041
U+566A.0	fBig5	BEB8
U+566A.0	fGB	0-546B
U+566A.0	fR/S	 30.13
U+566A.0	fTotalStrokes	 16
U+566A.0	fCangjie	RRRD
U+566A.0	fFourCorner	66094
U+566A.0	fHakka	 cau5
U+566A.0	fCantonese	cou3
U+566A.0	fMandarin	zao4
U+566A.0	fEnglish	[1] be noisy [2] chirp (birds, insects, etc.)
U+566B.0	fUTF8	噫
U+566B.0	fCNS11643	1-703C
U+566B.0	fBig5	BEB3
U+566B.0	fGB	0-6066
U+566B.0	fR/S	 30.13
U+566B.0	fTotalStrokes	 16
U+566B.0	fCangjie	RYTP
U+566B.0	fFourCorner	60036
U+566B.0	fHakka	 ji5; ji1; ri5; i1
U+566B.0	fCantonese	ji1; aai3; ngaai3
U+566B.0	fMandarin	yi1; ai4; yi3; yi4
U+566B.0	fEnglish	an interjection of sorrow or regret (in literary texts)
U+566C.0	fUTF8	噬
U+566C.0	fCNS11643	1-7046
U+566C.0	fBig5	BEBD
U+566C.0	fGB	0-4A49
U+566C.0	fR/S	 30.13
U+566C.0	fTotalStrokes	 16
U+566C.0	fCangjie	RHMO
U+566C.0	fFourCorner	68018
U+566C.0	fHakka	 chi3; chi1; cih1; si5; ci3; cih3; chii3
U+566C.0	fCantonese	sai6
U+566C.0	fMandarin	shi4; yi1
U+566C.0	fEnglish	 bite; gnaw; snap at
U+566D.0	fUTF8	噭
U+566D.0	fCNS11643	2-564D
U+566D.0	fBig5	E943
U+566D.0	fGB	3-3742
U+566D.0	fR/S	 30.13
U+566D.0	fTotalStrokes	 16
U+566D.0	fCangjie	RHSK
U+566D.0	fFourCorner	68040
U+566D.0	fHakka	 giau5; kieu5; giau3
U+566D.0	fCantonese	giu3; kiu3
U+566D.0	fMandarin	jiao4; qiao4
U+566D.0	fEnglish	shout
U+566E.0	fUTF8	噮
U+566E.0	fCNS11643	2-5649
U+566E.0	fBig5	E8FE
U+566E.0	fGB	E-247E
U+566E.0	fR/S	 30.13
U+566E.0	fTotalStrokes	 16
U+566E.0	fCangjie	RWLV
U+566E.0	fFourCorner	66032
U+566E.0	fCantonese	jyun3; jyun1
U+566E.0	fMandarin	yuan4; yuan1
U+566F.0	fUTF8	噯
U+566F.0	fCNS11643	1-7045
U+566F.0	fBig5	BEBC
U+566F.0	fGB	1-6048
U+566F.0	fR/S	 30.13
U+566F.0	fTotalStrokes	 16
U+566F.0	fCangjie	RBBE
U+566F.0	fFourCorner	62047
U+566F.0	fHakka	 oi5; ai3; ai1; oi3
U+566F.0	fCantonese	oi2; ngoi2 (also); aai1; ngaai1 (also); oi3; ngoi3; ai1
U+566F.0	fMandarin	ai4; ai1
U+566F.0	fEnglish	Oh!; an interjection; exclamation
U+566F.1	fUTF8	噯
U+566F.1	fCNS11643	1-7045
U+566F.1	fBig5	BEBC
U+566F.1	fGB	1-6048
U+566F.1	fR/S	 30.13
U+566F.1	fTotalStrokes	 16
U+566F.1	fCangjie	RBBE
U+566F.1	fFourCorner	62047
U+566F.1	fMandarin	ai3
U+566F.1	fEnglish	used in combination
U+5670.0	fUTF8	噰
U+5670.0	fCNS11643	2-5647
U+5670.0	fBig5	E8FC
U+5670.0	fGB	5-395B
U+5670.0	fR/S	 30.13
U+5670.0	fTotalStrokes	 16
U+5670.0	fCangjie	RYVG
U+5670.0	fFourCorner	60014
U+5670.0	fHakka	 jung1; rung1
U+5670.0	fCantonese	jung1
U+5670.0	fMandarin	yong1; yong3
U+5670.0	fEnglish	 choke; the cry of birds
U+5671.0	fUTF8	噱
U+5671.0	fCNS11643	1-7044
U+5671.0	fBig5	BEBB
U+5671.0	fGB	0-6065
U+5671.0	fR/S	 30.13
U+5671.0	fTotalStrokes	 16
U+5671.0	fCangjie	RYPO
U+5671.0	fFourCorner	61032
U+5671.0	fHakka	  kiok7; nak7; gi3
U+5671.0	fCantonese	koek6; coek3 (also); koek3 (also); hoe4 (also); kek6; kek3
U+5671.0	fMandarin	jue2; xue2; jue1
U+5671.0	fEnglish	laugh heartily; laugh aloud; loud laughter
U+5671.1	fUTF8	噱
U+5671.1	fCNS11643	1-7044
U+5671.1	fBig5	BEBB
U+5671.1	fGB	0-6065
U+5671.1	fR/S	 30.13
U+5671.1	fTotalStrokes	 16
U+5671.1	fCangjie	RYPO
U+5671.1	fFourCorner	61032
U+5671.1	fMandarin	xue1
U+5671.1	fEnglish	used in Shanghai dialect
U+5672.0	fUTF8	噲
U+5672.0	fCNS11643	2-564E
U+5672.0	fBig5	E944
U+5672.0	fGB	1-5F60
U+5672.0	fR/S	 30.13
U+5672.0	fTotalStrokes	 16
U+5672.0	fCangjie	ROMA
U+5672.0	fFourCorner	68066
U+5672.0	fHakka	 kwai5; woi3; woi6; kai5; woi5; foe5; voe5
U+5672.0	fCantonese	faai3
U+5672.0	fMandarin	kuai4
U+5672.0	fEnglish	 swallow; gulp down greedy able will
U+5673.0	fUTF8	噳
U+5673.0	fCNS11643	2-564A
U+5673.0	fBig5	E940
U+5673.0	fGB	3-373F
U+5673.0	fR/S	 30.13
U+5673.0	fTotalStrokes	 16
U+5673.0	fCangjie	RYPK
U+5673.0	fFourCorner	61034
U+5673.0	fHakka	 ngi3;  ngu5
U+5673.0	fCantonese	jyu5; koek3
U+5673.0	fMandarin	yu3
U+5673.0	fEnglish	many
U+5674.0	fUTF8	噴
U+5674.0	fCNS11643	1-6C3B
U+5674.0	fBig5	BC51
U+5674.0	fGB	1-4567
U+5674.0	fR/S	 30.12
U+5674.0	fTotalStrokes	 15
U+5674.0	fCangjie	RJTC
U+5674.0	fFourCorner	64086
U+5674.0	fHakka	 pun5; pun2; pun1
U+5674.0	fMandarin	pen1; pen4
U+5674.0	fEnglish	[1] spurt; gush [2] spray; sprinkle; blow out; puff out
U+5674.1	fUTF8	噴
U+5674.1	fCNS11643	1-6C3B
U+5674.1	fBig5	BC51
U+5674.1	fGB	1-4567
U+5674.1	fR/S	 30.12
U+5674.1	fTotalStrokes	 15
U+5674.1	fCangjie	RJTC
U+5674.1	fFourCorner	64086
U+5674.1	fCantonese	pan3; pan1; fan5
U+5674.1	fMandarin	fen4
U+5674.1	fEnglish	used in combination
U+5675.0	fUTF8	噵
U+5675.0	fCNS11643	4-4F71
U+5675.0	fGB	E-2521
U+5675.0	fR/S	 30.13
U+5675.0	fTotalStrokes	 16
U+5675.0	fCangjie	YUR
U+5675.0	fFourCorner	38600
U+5675.0	fCantonese	dou6
U+5675.0	fMandarin	dao4
U+5676.0	fUTF8	噶
U+5676.0	fCNS11643	1-7048
U+5676.0	fBig5	BEBF
U+5676.0	fGB	0-3841
U+5676.0	fR/S	 30.13
U+5676.0	fTotalStrokes	 16
U+5676.0	fCangjie	RTAV
U+5676.0	fFourCorner	64027
U+5676.0	fCantonese	gaa1; got3; gaat3 (also)
U+5676.0	fMandarin	ga2; ge2; ge1
U+5676.0	fEnglish	used in transliterations
U+5677.0	fUTF8	噷
U+5677.0	fCNS11643	2-5650
U+5677.0	fBig5	E946
U+5677.0	fGB	3-3743
U+5677.0	fR/S	 30.13
U+5677.0	fTotalStrokes	 16
U+5677.0	fCangjie	RYAO
U+5677.0	fFourCorner	67082
U+5677.0	fCantonese	hm1; am1; ham1
U+5677.0	fMandarin	xin1; hen1
U+5677.0	fEnglish	H'm! (an interjection)
U+5678.0	fUTF8	噸
U+5678.0	fCNS11643	1-7040
U+5678.0	fBig5	BEB7
U+5678.0	fGB	1-3656
U+5678.0	fR/S	 30.13
U+5678.0	fTotalStrokes	 16
U+5678.0	fCangjie	RPUC
U+5678.0	fFourCorner	61086
U+5678.0	fHakka	 dun1
U+5678.0	fCantonese	deon1; deon6
U+5678.0	fMandarin	dun1
U+5678.0	fEnglish	metric ton
U+5679.0	fUTF8	噹
U+5679.0	fCNS11643	1-703D
U+5679.0	fBig5	BEB4
U+5679.0	fGB	1-782B
U+5679.0	fR/S	 30.13
U+5679.0	fTotalStrokes	 16
U+5679.0	fCangjie	RFBW
U+5679.0	fFourCorner	69066
U+5679.0	fHakka	 dong1; dang1
U+5679.0	fCantonese	dong1
U+5679.0	fMandarin	dang1; dang4
U+5679.0	fEnglish	a loud, resonant metallic sound
U+567A.0	fUTF8	噺
U+567A.0	fCNS11643	F-535D
U+567A.0	fGB	E-2522
U+567A.0	fR/S	 30.13
U+567A.0	fTotalStrokes	 16
U+567A.0	fCangjie	RYDL
U+567A.0	fCantonese	san1
U+567A.0	fEnglish	 story, talk
U+567B.0	fUTF8	噻
U+567B.0	fCNS11643	3-503D
U+567B.0	fGB	0-6067
U+567B.0	fR/S	 30.13
U+567B.0	fTotalStrokes	 16
U+567B.0	fCangjie	RJTG
U+567B.0	fFourCorner	63010
U+567B.0	fCantonese	sak1
U+567B.0	fMandarin	sai1; sai5
U+567B.0	fEnglish	character used in translation
U+567C.0	fUTF8	噼
U+567C.0	fGB	0-6068
U+567C.0	fR/S	 30.13
U+567C.0	fTotalStrokes	 16
U+567C.0	fCangjie	RSRJ
U+567C.0	fFourCorner	6004
U+567C.0	fCantonese	pik1; pek1; pet1 (also); pet6
U+567C.0	fMandarin	pi1
U+567C.0	fEnglish	sound
U+567D.0	fUTF8	噽
U+567D.0	fCNS11643	3-5448
U+567D.0	fGB	5-3723
U+567D.0	fR/S	  30.14
U+567D.0	fTotalStrokes	 17
U+567D.0	fCangjie	GRMFM
U+567D.0	fFourCorner	41610
U+567D.0	fHakka	  pi3
U+567D.0	fCantonese	pei2
U+567D.0	fMandarin	pi3
U+567D.0	fEnglish	[1] great [2] a high officer on the state of Wu
U+567E.0	fUTF8	噾
U+567E.0	fCNS11643	2-5C52
U+567E.0	fBig5	ECC6
U+567E.0	fGB	5-3966
U+567E.0	fR/S	 30.14
U+567E.0	fTotalStrokes	 17
U+567E.0	fCangjie	RJCA
U+567E.0	fFourCorner	63061
U+567E.0	fHakka	 jim1;  am1; rim1; rap7; jap7; jim5; am5
U+567E.0	fCantonese	jam1; ngam6; jam3; am6 (also)
U+567E.0	fMandarin	yin1; an1; yin4; an4; yin3
U+567E.0	fEnglish	same as &#x5591;
U+567F.0	fUTF8	噿
U+567F.0	fCNS11643	2-5C54
U+567F.0	fBig5	ECC8
U+567F.0	fGB	5-3968
U+567F.0	fR/S	 30.14
U+567F.0	fTotalStrokes	 17
U+567F.0	fCangjie	RSMJ
U+567F.0	fFourCorner	67048
U+567F.0	fCantonese	zeoi2
U+567F.0	fMandarin	zui3
U+5680.0	fUTF8	嚀
U+5680.0	fCNS11643	1-7346
U+5680.0	fBig5	C07B
U+5680.0	fGB	1-5F4C
U+5680.0	fR/S	 30.14
U+5680.0	fTotalStrokes	 17
U+5680.0	fCangjie	RJPN
U+5680.0	fFourCorner	63021
U+5680.0	fHakka	  nen2; len2
U+5680.0	fCantonese	ning4; ling4 (also)
U+5680.0	fMandarin	ning2
U+5680.0	fEnglish	to enjoin; instruct
U+5681.0	fUTF8	嚁
U+5681.0	fCNS11643	2-5C55
U+5681.0	fBig5	ECC9
U+5681.0	fGB	5-3967
U+5681.0	fR/S	 30.14
U+5681.0	fTotalStrokes	 17
U+5681.0	fCangjie	RSMG
U+5681.0	fFourCorner	67014
U+5681.0	fCantonese	dik6
U+5681.0	fMandarin	di2
U+5682.0	fUTF8	嚂
U+5682.0	fCNS11643	2-5C53
U+5682.0	fBig5	ECC7
U+5682.0	fGB	3-367C
U+5682.0	fR/S	 30.14
U+5682.0	fTotalStrokes	 17
U+5682.0	fCangjie	RSIT
U+5682.0	fFourCorner	68017
U+5682.0	fHakka	 gam5
U+5682.0	fCantonese	laam6; laam4 (also)
U+5682.0	fMandarin	lan4; han3; lan2
U+5683.0	fUTF8	嚃
U+5683.0	fCNS11643	2-5C51
U+5683.0	fBig5	ECC5
U+5683.0	fGB	3-3740
U+5683.0	fR/S	 30.14
U+5683.0	fTotalStrokes	 17
U+5683.0	fCangjie	RYWE
U+5683.0	fFourCorner	66033
U+5683.0	fHakka	  tap7
U+5683.0	fCantonese	daap6; taap3 (also); daap3
U+5683.0	fMandarin	ta4; tui4
U+5683.0	fEnglish	gobble
U+5684.0	fUTF8	嚄
U+5684.0	fCNS11643	2-5C50
U+5684.0	fBig5	ECC4
U+5684.0	fGB	3-373D
U+5684.0	fR/S	 30.14
U+5684.0	fTotalStrokes	 17
U+5684.0	fCangjie	RTOE
U+5684.0	fFourCorner	64047
U+5684.0	fHakka	 wok7; fet8
U+5684.0	fCantonese	wok6; ngo2 (also); ho1; o2; waak6
U+5684.0	fMandarin	huo4; huo1; o3
U+5684.0	fEnglish	roar
U+5685.0	fUTF8	嚅
U+5685.0	fCNS11643	1-7348
U+5685.0	fBig5	C07D
U+5685.0	fGB	0-6069
U+5685.0	fR/S	 30.14
U+5685.0	fTotalStrokes	 17
U+5685.0	fCangjie	RMBB
U+5685.0	fFourCorner	61027
U+5685.0	fHakka	  si1; ji2; ri2
U+5685.0	fCantonese	jyu4
U+5685.0	fMandarin	ru2
U+5685.0	fEnglish	 talk indistinctly and falteringly
U+5686.0	fUTF8	嚆
U+5686.0	fCNS11643	2-5C4F
U+5686.0	fBig5	ECC3
U+5686.0	fGB	0-6063
U+5686.0	fR/S	 30.14
U+5686.0	fTotalStrokes	 17
U+5686.0	fCangjie	RTYB
U+5686.0	fFourCorner	64027
U+5686.0	fHakka	  hau1; gau1
U+5686.0	fCantonese	hou1; haau1
U+5686.0	fMandarin	hao1
U+5686.0	fEnglish	 cry; yell
U+5687.0	fUTF8	嚇
U+5687.0	fCNS11643	1-7349
U+5687.0	fBig5	C07E
U+5687.0	fGB	1-4F45
U+5687.0	fR/S	 30.14
U+5687.0	fTotalStrokes	 17
U+5687.0	fCangjie	RGCC
U+5687.0	fFourCorner	64031
U+5687.0	fHakka	  hak7;  het7
U+5687.0	fCantonese	haak3; haak1; haa1 (also)
U+5687.0	fMandarin	he4
U+5687.0	fEnglish	[1] intimidate; threaten [2] the sound of laughter
U+5687.1	fUTF8	嚇
U+5687.1	fCNS11643	1-7349
U+5687.1	fBig5	C07E
U+5687.1	fGB	1-4F45
U+5687.1	fR/S	 30.14
U+5687.1	fTotalStrokes	 17
U+5687.1	fCangjie	RGCC
U+5687.1	fFourCorner	64031
U+5687.1	fMandarin	xia4
U+5687.1	fEnglish	 frighten; startle; scare; intimidate
U+5688.0	fUTF8	嚈
U+5688.0	fCNS11643	4-5574
U+5688.0	fGB	3-357D
U+5688.0	fR/S	  30.14
U+5688.0	fTotalStrokes	 17; 16
U+5688.0	fCangjie	RMAK
U+5688.0	fFourCorner	61030
U+5688.0	fCantonese	jip3; aat3; ngaat3 (also)
U+5688.0	fMandarin	ya1; ye4; ya4
U+5689.0	fUTF8	嚉
U+5689.0	fCNS11643	3-5449
U+5689.0	fGB	E-2523
U+5689.0	fR/S	 30.14
U+5689.0	fTotalStrokes	 17
U+5689.0	fCangjie	RTGI
U+5689.0	fFourCorner	64000
U+5689.0	fCantonese	deoi3
U+5689.0	fMandarin	duo1; duo4
U+5689.0	fEnglish	swell
U+568A.0	fUTF8	嚊
U+568A.0	fCNS11643	3-5447
U+568A.0	fGB	5-3963
U+568A.0	fR/S	 30.14
U+568A.0	fTotalStrokes	 17
U+568A.0	fCangjie	RHUL
U+568A.0	fFourCorner	66041
U+568A.0	fHakka	 si5
U+568A.0	fCantonese	be6; bei6 (also)
U+568A.0	fMandarin	xi4; pi4
U+568A.0	fEnglish	 pant wife
U+568B.0	fUTF8	嚋
U+568B.0	fCNS11643	4-5578
U+568B.0	fGB	5-3848
U+568B.0	fR/S	 30.14
U+568B.0	fTotalStrokes	 17; 16
U+568B.0	fCangjie	RGNI
U+568B.0	fFourCorner	64040
U+568B.0	fCantonese	cau4
U+568B.0	fMandarin	chou2
U+568C.0	fUTF8	嚌
U+568C.0	fCNS11643	2-5C4D
U+568C.0	fBig5	ECC1
U+568C.0	fGB	1-5F62
U+568C.0	fR/S	 30.14
U+568C.0	fTotalStrokes	 17
U+568C.0	fCangjie	RYX
U+568C.0	fFourCorner	60023
U+568C.0	fHakka	 zi5;  ci2; zip8; zi1
U+568C.0	fCantonese	zai6; cai6
U+568C.0	fMandarin	ji4; jie1; qi2; zhai1
U+568C.0	fEnglish	sip
U+568D.0	fUTF8	嚍
U+568D.0	fCNS11643	2-5C4E
U+568D.0	fBig5	ECC2
U+568D.0	fGB	5-3845
U+568D.0	fR/S	 30.14
U+568D.0	fTotalStrokes	 17
U+568D.0	fCangjie	RLMT
U+568D.0	fFourCorner	65017
U+568D.0	fCantonese	zeon3
U+568D.0	fMandarin	jin4
U+568E.0	fUTF8	嚎
U+568E.0	fCNS11643	1-7345
U+568E.0	fBig5	C07A
U+568E.0	fGB	0-3A3F
U+568E.0	fR/S	 30.14
U+568E.0	fTotalStrokes	 17
U+568E.0	fCangjie	RYRO
U+568E.0	fFourCorner	60032
U+568E.0	fHakka	  hau2; ho2
U+568E.0	fCantonese	hou4
U+568E.0	fMandarin	hao2
U+568E.0	fEnglish	 cry loudly with abandon
U+568F.0	fUTF8	嚏
U+568F.0	fCNS11643	1-734A
U+568F.0	fBig5	C0A1
U+568F.0	fGB	0-4C67
U+568F.0	fR/S	 30.14
U+568F.0	fTotalStrokes	 17
U+568F.0	fCangjie	RJBO
U+568F.0	fFourCorner	64081
U+568F.0	fHakka	 ti5; ci5
U+568F.0	fCantonese	tai3; dai3
U+568F.0	fMandarin	ti4
U+568F.0	fEnglish	sneeze
U+5690.0	fUTF8	嚐
U+5690.0	fCNS11643	1-7347
U+5690.0	fBig5	C07C
U+5690.0	fGB	E-2524
U+5690.0	fR/S	 30.14
U+5690.0	fTotalStrokes	 17
U+5690.0	fCangjie	RFBA
U+5690.0	fFourCorner	69061
U+5690.0	fHakka	 song2
U+5690.0	fCantonese	soeng4
U+5690.0	fMandarin	chang2
U+5690.0	fEnglish	taste
U+5691.0	fUTF8	嚑
U+5691.0	fCNS11643	3-645D
U+5691.0	fGB	E-2525
U+5691.0	fR/S	 30.14
U+5691.0	fTotalStrokes	 17
U+5691.0	fCangjie	RHGF
U+5691.0	fFourCorner	6203
U+5691.0	fCantonese	fan1
U+5692.0	fUTF8	嚒
U+5692.0	fCNS11643	3-645B
U+5692.0	fGB	E-2526
U+5692.0	fR/S	 30.14
U+5692.0	fTotalStrokes	 17
U+5692.0	fCangjie	RIDI
U+5692.0	fFourCorner	6003
U+5692.0	fCantonese	mo1
U+5693.0	fUTF8	嚓
U+5693.0	fCNS11643	2-5C4C
U+5693.0	fBig5	ECC0
U+5693.0	fGB	0-606A
U+5693.0	fR/S	 30.14
U+5693.0	fTotalStrokes	 17
U+5693.0	fCangjie	RJBF
U+5693.0	fFourCorner	63091
U+5693.0	fCantonese	caat3
U+5693.0	fMandarin	ca1; cha1
U+5694.0	fUTF8	嚔
U+5694.0	fCNS11643	3-575B
U+5694.0	fGB	5-396B
U+5694.0	fR/S	 30.15
U+5694.0	fTotalStrokes	 18
U+5694.0	fCangjie	RJBO
U+5694.0	fFourCorner	64080
U+5694.0	fHakka	 chi5; ti2; cih5; ti5; chii5
U+5694.0	fCantonese	dai3; tai3
U+5694.0	fMandarin	ti4
U+5694.0	fEnglish	 sneeze; sneezing
U+5695.0	fUTF8	嚕
U+5695.0	fCNS11643	1-763C
U+5695.0	fBig5	C250
U+5695.0	fGB	1-6060
U+5695.0	fR/S	 30.15
U+5695.0	fTotalStrokes	 18
U+5695.0	fCangjie	RNWA
U+5695.0	fFourCorner	67063
U+5695.0	fHakka	 lu1
U+5695.0	fCantonese	lou1
U+5695.0	fMandarin	lu1
U+5695.0	fEnglish	[1] verbose; wordy [2] an indistinct speech sound
U+5696.0	fUTF8	嚖
U+5696.0	fCNS11643	3-575A
U+5696.0	fGB	5-396A
U+5696.0	fR/S	 30.15
U+5696.0	fTotalStrokes	 18
U+5696.0	fCangjie	RQJP
U+5696.0	fFourCorner	65030
U+5696.0	fCantonese	wai3
U+5696.0	fMandarin	hui4
U+5697.0	fUTF8	嚗
U+5697.0	fCNS11643	2-6148
U+5697.0	fBig5	EFBC
U+5697.0	fGB	3-3749
U+5697.0	fR/S	 30.15
U+5697.0	fTotalStrokes	 18
U+5697.0	fCangjie	RATE
U+5697.0	fFourCorner	66032
U+5697.0	fCantonese	bok3; bok6
U+5697.0	fMandarin	bao4; bo2; bo1; pao4; pu3
U+5698.0	fUTF8	嚘
U+5698.0	fCNS11643	2-6146
U+5698.0	fBig5	EFBA
U+5698.0	fGB	3-3747
U+5698.0	fR/S	 30.15
U+5698.0	fTotalStrokes	 18
U+5698.0	fCangjie	RMBE
U+5698.0	fFourCorner	61047
U+5698.0	fHakka	 ju2; riu2
U+5698.0	fCantonese	jau1
U+5698.0	fMandarin	you1
U+5698.0	fEnglish	[1] stammer [2] sign [3] be hoarse
U+5699.0	fUTF8	嚙
U+5699.0	fCNS11643	2-614B
U+5699.0	fBig5	EFBF
U+5699.0	fGB	1-4476
U+5699.0	fR/S	 30.15
U+5699.0	fTotalStrokes	 18
U+5699.0	fCangjie	RYMU
U+5699.0	fFourCorner	61072
U+5699.0	fHakka	 ngau1;  ngat7
U+5699.0	fCantonese	ngit6; jit6 (also); ngaat6; it6 (also); aat6 (also); ngat6
U+5699.0	fMandarin	nie4
U+5699.0	fEnglish	bite; gnaw
U+569A.0	fUTF8	嚚
U+569A.0	fCNS11643	2-6149
U+569A.0	fBig5	EFBD
U+569A.0	fGB	3-374A
U+569A.0	fR/S	 30.15
U+569A.0	fTotalStrokes	 18
U+569A.0	fCangjie	RRSLR
U+569A.0	fFourCorner	66661
U+569A.0	fHakka	 ngiun2; niun2
U+569A.0	fCantonese	ngan4; an4 (also)
U+569A.0	fMandarin	yin2
U+569A.0	fEnglish	argumentative; talkative
U+569B.0	fUTF8	嚛
U+569B.0	fCNS11643	3-5758
U+569B.0	fGB	3-3574
U+569B.0	fR/S	  30.15
U+569B.0	fTotalStrokes	 18
U+569B.0	fCangjie	RVID
U+569B.0	fFourCorner	62090
U+569B.0	fCantonese	huk1
U+569B.0	fMandarin	hu4
U+569C.0	fUTF8	嚜
U+569C.0	fCNS11643	2-6147
U+569C.0	fBig5	EFBB
U+569C.0	fGB	3-374B
U+569C.0	fR/S	 30.15
U+569C.0	fTotalStrokes	 18
U+569C.0	fCangjie	RWGG
U+569C.0	fFourCorner	66014
U+569C.0	fHakka	 met8
U+569C.0	fCantonese	maa1; mak6; mak1; maak1; maa3
U+569C.0	fMandarin	mo4; mei4; me5; ma5
U+569C.0	fEnglish	 be silent final particle
U+569D.0	fUTF8	嚝
U+569D.0	fCNS11643	2-614A
U+569D.0	fBig5	EFBE
U+569D.0	fGB	5-376D
U+569D.0	fR/S	 30.15
U+569D.0	fTotalStrokes	 18
U+569D.0	fCangjie	RITC
U+569D.0	fFourCorner	60086
U+569D.0	fCantonese	gwang1; gwong1
U+569D.0	fMandarin	huang1
U+569E.0	fUTF8	嚞
U+569E.0	fCNS11643	3-5759
U+569E.0	fGB	5-3724
U+569E.0	fR/S	 30.15
U+569E.0	fTotalStrokes	 18
U+569E.0	fCangjie	GRGRR
U+569E.0	fFourCorner	40660
U+569E.0	fHakka	 zet7; zhat7; zat7; zhet7
U+569E.0	fCantonese	zit3
U+569E.0	fMandarin	zhe2
U+569E.0	fEnglish	same as &#x54F2;
U+569F.0	fUTF8	嚟
U+569F.0	fCNS11643	3-5757
U+569F.0	fGB	E-2527
U+569F.0	fR/S	 30.15
U+569F.0	fTotalStrokes	 18
U+569F.0	fCangjie	RHHE
U+569F.0	fFourCorner	67130
U+569F.0	fCantonese	lai4; lei4 (also); lai2 (also)
U+569F.0	fMandarin	li2
U+569F.0	fEnglish	 come; order (food, drinks, etc.)
U+56A0.0	fUTF8	嚠
U+56A0.0	fCNS11643	3-575C
U+56A0.0	fGB	E-2528
U+56A0.0	fR/S	 30.15
U+56A0.0	fTotalStrokes	 18
U+56A0.0	fCangjie	RHCN
U+56A0.0	fFourCorner	62000
U+56A0.0	fHakka	 liu2
U+56A0.0	fCantonese	lau4
U+56A0.0	fMandarin	liu2; liu3; liu1
U+56A0.0	fEnglish	same as &#x700F;
U+56A1.0	fUTF8	嚡
U+56A1.0	fGB	1-7C36
U+56A1.0	fR/S	 30.15
U+56A1.0	fTotalStrokes	 18
U+56A1.0	fCangjie	RTJG
U+56A1.0	fCantonese	haai4
U+56A1.0	fEnglish	 rough (as opposed to fine)
U+56A2.0	fUTF8	嚢
U+56A2.0	fGB	E-2529
U+56A2.0	fR/S	 30.15
U+56A2.0	fTotalStrokes	 18
U+56A2.0	fCangjie	JCTTV
U+56A2.0	fCantonese	nong4; long4 (also)
U+56A2.0	fMandarin	nang2; nang1
U+56A2.0	fEnglish	 bag; purse; sack put in bag
U+56A3.0	fUTF8	嚣
U+56A3.0	fGB	0-4F79
U+56A3.0	fR/S	 30.15
U+56A3.0	fTotalStrokes	 18
U+56A3.0	fCangjie	RRMOR
U+56A3.0	fHakka	 hiau1
U+56A3.0	fCantonese	hiu1
U+56A3.0	fMandarin	xiao1; ao2
U+56A3.0	fEnglish	simplified form of &#x56C2;
U+56A4.0	fUTF8	嚤
U+56A4.0	fGB	1-7C58
U+56A4.0	fR/S	 30.15
U+56A4.0	fTotalStrokes	 18
U+56A4.0	fCangjie	RIDQ
U+56A4.0	fCantonese	mo1
U+56A4.0	fMandarin	mo2
U+56A4.0	fEnglish	[1] slow; late [2] used in transliteration
U+56A5.0	fUTF8	嚥
U+56A5.0	fCNS11643	1-782C
U+56A5.0	fBig5	C360
U+56A5.0	fGB	E-252A
U+56A5.0	fR/S	 30.16
U+56A5.0	fTotalStrokes	 19
U+56A5.0	fCangjie	RTLF
U+56A5.0	fFourCorner	64031
U+56A5.0	fHakka	 jen5; ren5; jen1; rin1; jin1; jan1
U+56A5.0	fCantonese	jin3; jin1; jit3
U+56A5.0	fMandarin	yan4; ye4
U+56A5.0	fEnglish	 swallow; gulp
U+56A6.0	fUTF8	嚦
U+56A6.0	fCNS11643	2-653E
U+56A6.0	fBig5	F1F2
U+56A6.0	fGB	1-5F3F
U+56A6.0	fR/S	 30.16
U+56A6.0	fTotalStrokes	 19
U+56A6.0	fCangjie	RMHM
U+56A6.0	fFourCorner	61011
U+56A6.0	fHakka	  lit8;  let8
U+56A6.0	fCantonese	lik6; lik1 (also)
U+56A6.0	fMandarin	li4
U+56A6.0	fEnglish	used in onomatopoetic expression
U+56A7.0	fUTF8	嚧
U+56A7.0	fCNS11643	2-653F
U+56A7.0	fBig5	F1F3
U+56A7.0	fGB	3-356A
U+56A7.0	fR/S	 30.16
U+56A7.0	fTotalStrokes	 19
U+56A7.0	fCangjie	RYPT
U+56A7.0	fFourCorner	61017
U+56A7.0	fHakka	 lu2
U+56A7.0	fCantonese	lou4
U+56A7.0	fMandarin	lu2
U+56A7.0	fEnglish	cry for calling pigs
U+56A8.0	fUTF8	嚨
U+56A8.0	fCNS11643	1-782D
U+56A8.0	fBig5	C456
U+56A8.0	fGB	1-417C
U+56A8.0	fR/S	 30.16
U+56A8.0	fTotalStrokes	 19
U+56A8.0	fCangjie	RYBP
U+56A8.0	fFourCorner	61011
U+56A8.0	fHakka	 lung2; lien2; lun3
U+56A8.0	fCantonese	lung4
U+56A8.0	fMandarin	long2
U+56A8.0	fEnglish	the throat
U+56A9.0	fUTF8	嚩
U+56A9.0	fCNS11643	4-5F4D
U+56A9.0	fGB	3-3744
U+56A9.0	fR/S	  30.16
U+56A9.0	fTotalStrokes	 19
U+56A9.0	fCangjie	RVFI
U+56A9.0	fFourCorner	63040
U+56A9.0	fCantonese	bok3
U+56A9.0	fMandarin	fu2; mo2; po2
U+56AA.0	fUTF8	嚪
U+56AA.0	fCNS11643	2-6540
U+56AA.0	fBig5	F1F4
U+56AA.0	fGB	5-3941
U+56AA.0	fR/S	 30.16
U+56AA.0	fTotalStrokes	 19
U+56AA.0	fCangjie	RANX
U+56AA.0	fFourCorner	67020
U+56AA.0	fCantonese	daam6; taam6
U+56AA.0	fMandarin	dan1; dan4
U+56AB.0	fUTF8	嚫
U+56AB.0	fCNS11643	2-653C
U+56AB.0	fBig5	F1F0
U+56AB.0	fGB	3-3673
U+56AB.0	fR/S	 30.16
U+56AB.0	fTotalStrokes	 19
U+56AB.0	fCangjie	RYDU
U+56AB.0	fFourCorner	66010
U+56AB.0	fHakka	 cang5;  cen5
U+56AB.0	fCantonese	can3
U+56AB.0	fMandarin	chen4
U+56AB.0	fEnglish	 give money especially for Buddhist services
U+56AC.0	fUTF8	嚬
U+56AC.0	fCNS11643	2-6541
U+56AC.0	fBig5	F1F5
U+56AC.0	fGB	5-3956
U+56AC.0	fR/S	 30.16
U+56AC.0	fTotalStrokes	 19
U+56AC.0	fCangjie	RYHC
U+56AC.0	fFourCorner	61086
U+56AC.0	fHakka	 pin2
U+56AC.0	fCantonese	pan4
U+56AC.0	fMandarin	pin2
U+56AC.0	fEnglish	frown
U+56AD.0	fUTF8	嚭
U+56AD.0	fCNS11643	2-653D
U+56AD.0	fBig5	F1F1
U+56AD.0	fGB	3-3523
U+56AD.0	fR/S	 30.16
U+56AD.0	fTotalStrokes	 19
U+56AD.0	fCangjie	GRMFR
U+56AD.0	fFourCorner	41669
U+56AD.0	fHakka	 pi3
U+56AD.0	fCantonese	pei2
U+56AD.0	fMandarin	pi3
U+56AD.0	fEnglish	 mound; lump stealthily
U+56AE.0	fUTF8	嚮
U+56AE.0	fCNS11643	1-763D
U+56AE.0	fBig5	C251
U+56AE.0	fGB	1-7875
U+56AE.0	fR/S	 30.16; 30.14
U+56AE.0	fTotalStrokes	 19
U+56AE.0	fCangjie	VLHBR
U+56AE.0	fFourCorner	27227
U+56AE.0	fHakka	  hiong5;  hiong3
U+56AE.0	fCantonese	hoeng3; hoeng2 (also)
U+56AE.0	fMandarin	xiang4; xiang3
U+56AE.0	fEnglish	[1] guide; direct; lead [2] lean forward; be inclined toward
U+56AF.0	fUTF8	嚯
U+56AF.0	fCNS11643	3-5A2D
U+56AF.0	fGB	0-606B
U+56AF.0	fR/S	 30.16
U+56AF.0	fTotalStrokes	 19
U+56AF.0	fCangjie	RMBG
U+56AF.0	fFourCorner	61010
U+56AF.0	fCantonese	ho1
U+56AF.0	fMandarin	jue2; huo4; xue4
U+56B0.0	fUTF8	嚰
U+56B0.0	fCNS11643	3-645E
U+56B0.0	fGB	E-252B
U+56B0.0	fR/S	 30.16
U+56B0.0	fTotalStrokes	 19
U+56B0.0	fCangjie	RIDR
U+56B0.0	fFourCorner	6006
U+56B0.0	fCantonese	jiu1
U+56B0.0	fMandarin	mo2
U+56B1.0	fUTF8	嚱
U+56B1.0	fCNS11643	4-6331
U+56B1.0	fGB	E-252C
U+56B1.0	fR/S	  30.17
U+56B1.0	fTotalStrokes	 20
U+56B1.0	fCangjie	RYTI
U+56B1.0	fFourCorner	63050
U+56B1.0	fHakka	  hi1;  hi5
U+56B1.0	fCantonese	hei3; hei1
U+56B1.0	fMandarin	xi4
U+56B1.0	fEnglish	a shrill noise; alas!
U+56B2.0	fUTF8	嚲
U+56B2.0	fCNS11643	2-686B
U+56B2.0	fBig5	F3FE
U+56B2.0	fGB	3-3259
U+56B2.0	fR/S	 30.17
U+56B2.0	fTotalStrokes	 20
U+56B2.0	fCangjie	YDRRJ
U+56B2.0	fFourCorner	06446
U+56B2.0	fHakka	  do3; to3; tan3; to2; lo3
U+56B2.0	fCantonese	do2
U+56B2.0	fMandarin	duo3
U+56B2.0	fEnglish	 read aloud; recite; chant droop
U+56B3.0	fUTF8	嚳
U+56B3.0	fCNS11643	2-686D
U+56B3.0	fBig5	F441
U+56B3.0	fGB	1-6037
U+56B3.0	fR/S	 30.17
U+56B3.0	fTotalStrokes	 20
U+56B3.0	fCangjie	HBHGR
U+56B3.0	fFourCorner	77601
U+56B3.0	fHakka	 kuk7; guk7; kwuk7
U+56B3.0	fCantonese	guk1; kuk1
U+56B3.0	fMandarin	ku4
U+56B3.0	fEnglish	 mythical emperor inform quickly urgent communication
U+56B4.0	fUTF8	嚴
U+56B4.0	fCNS11643	1-7964
U+56B4.0	fBig5	C459
U+56B4.0	fGB	1-514F
U+56B4.0	fR/S	 30.17
U+56B4.0	fTotalStrokes	 20
U+56B4.0	fCangjie	RRMMK
U+56B4.0	fFourCorner	66248
U+56B4.0	fHakka	  ngiam2; niam2; gniam3
U+56B4.0	fCantonese	jim4
U+56B4.0	fMandarin	yan2; yan3; yan4
U+56B4.0	fEnglish	stern; strict; severe; grim; inclement; inexorable; relentless; rigorous; rigorous; rigid; grave
U+56B5.0	fUTF8	嚵
U+56B5.0	fCNS11643	2-686C
U+56B5.0	fBig5	F440
U+56B5.0	fGB	3-374F
U+56B5.0	fR/S	 30.17
U+56B5.0	fTotalStrokes	 20
U+56B5.0	fCangjie	RNRI
U+56B5.0	fFourCorner	67013
U+56B5.0	fHakka	 cam2
U+56B5.0	fCantonese	caam4
U+56B5.0	fMandarin	chan2; chan1; chan4; jian2; zhan2
U+56B5.0	fEnglish	 gluttonous greedy
U+56B6.0	fUTF8	嚶
U+56B6.0	fCNS11643	1-7963
U+56B6.0	fBig5	C458
U+56B6.0	fGB	1-6053
U+56B6.0	fR/S	 30.17
U+56B6.0	fTotalStrokes	 20
U+56B6.0	fCangjie	RBCV
U+56B6.0	fFourCorner	66044
U+56B6.0	fHakka	 jin1; rin1
U+56B6.0	fCantonese	jing1; ang1
U+56B6.0	fMandarin	ying1
U+56B6.0	fEnglish	 seek friends also used in onomatopoeia
U+56B7.0	fUTF8	嚷
U+56B7.0	fCNS11643	1-7962
U+56B7.0	fBig5	C457
U+56B7.0	fGB	0-4842
U+56B7.0	fR/S	 30.17
U+56B7.0	fTotalStrokes	 20
U+56B7.0	fCangjie	RYRV
U+56B7.0	fFourCorner	60032
U+56B7.0	fHakka	 jong3; rong3; ngiong5; ngiong3; jong5
U+56B7.0	fCantonese	joeng6; joeng5
U+56B7.0	fMandarin	rang3
U+56B7.0	fEnglish	 shout; cry; call out loudly
U+56B7.1	fUTF8	嚷
U+56B7.1	fCNS11643	1-7962
U+56B7.1	fBig5	C457
U+56B7.1	fGB	0-4842
U+56B7.1	fR/S	 30.17
U+56B7.1	fTotalStrokes	 20
U+56B7.1	fCangjie	RYRV
U+56B7.1	fFourCorner	60032
U+56B7.1	fMandarin	rang1
U+56B7.1	fEnglish	used in combination
U+56B8.0	fUTF8	嚸
U+56B8.0	fCNS11643	3-6726
U+56B8.0	fGB	E-252D
U+56B8.0	fR/S	 30.17
U+56B8.0	fTotalStrokes	 20
U+56B8.0	fCangjie	RWFR
U+56B8.0	fCantonese	dim2
U+56B8.0	fMandarin	dian3
U+56B9.0	fUTF8	嚹
U+56B9.0	fCNS11643	3-6727
U+56B9.0	fGB	1-7C37
U+56B9.0	fR/S	 30.17
U+56B9.0	fTotalStrokes	 20
U+56B9.0	fCangjie	ROUD
U+56B9.0	fCantonese	laa3; laa4 (also)
U+56B9.0	fMandarin	la1
U+56B9.0	fEnglish	 a particle implying completion; certainty; or urgency
U+56BA.0	fUTF8	嚺
U+56BA.0	fCNS11643	3-645C
U+56BA.0	fGB	E-252E
U+56BA.0	fR/S	 30.14
U+56BA.0	fTotalStrokes	 17
U+56BA.0	fCangjie	RYWO
U+56BA.0	fFourCorner	6603
U+56BA.0	fCantonese	daap6
U+56BA.0	fMandarin	ta4
U+56BB.0	fUTF8	嚻
U+56BB.0	fCNS11643	3-5E26
U+56BB.0	fGB	E-252F
U+56BB.0	fR/S	 30.18
U+56BB.0	fTotalStrokes	 21
U+56BB.0	fCangjie	RRMCR
U+56BB.0	fFourCorner	66660
U+56BB.0	fHakka	 hiau1
U+56BB.0	fCantonese	hiu1
U+56BB.0	fMandarin	xiao1; ao2
U+56BB.0	fEnglish	same as &#x56C2;
U+56BC.0	fUTF8	嚼
U+56BC.0	fCNS11643	1-7965
U+56BC.0	fBig5	C45A
U+56BC.0	fGB	0-3D40
U+56BC.0	fR/S	 30.18
U+56BC.0	fTotalStrokes	 21
U+56BC.0	fCangjie	RBWI
U+56BC.0	fFourCorner	62046
U+56BC.0	fHakka	 ziok7; tsio5;  ciau5; ziau3; ciau3; ciau6
U+56BC.0	fCantonese	zoek3; ziu6 (also); ziu3; zoek6
U+56BC.0	fMandarin	jiao2; jiao4; jue2
U+56BC.0	fEnglish	 chew; masticate; munch
U+56BD.0	fUTF8	嚽
U+56BD.0	fCNS11643	2-6B53
U+56BD.0	fBig5	F5C5
U+56BD.0	fGB	5-395F
U+56BD.0	fR/S	 30.18
U+56BD.0	fTotalStrokes	 21
U+56BD.0	fCangjie	RGKC
U+56BD.0	fFourCorner	67086
U+56BD.0	fCantonese	zyut3
U+56BD.0	fMandarin	chuo4
U+56BE.0	fUTF8	嚾
U+56BE.0	fCNS11643	2-6B54
U+56BE.0	fBig5	F5C6
U+56BE.0	fGB	3-374E
U+56BE.0	fR/S	 30.18
U+56BE.0	fTotalStrokes	 21
U+56BE.0	fCangjie	RTRG
U+56BE.0	fFourCorner	64014
U+56BE.0	fCantonese	fun1
U+56BE.0	fMandarin	huan1; huan4; tuan4
U+56BF.0	fUTF8	嚿
U+56BF.0	fCNS11643	4-6634
U+56BF.0	fGB	1-7C38
U+56BF.0	fR/S	 30.18
U+56BF.0	fTotalStrokes	 21
U+56BF.0	fCangjie	RTOX
U+56BF.0	fFourCorner	64070
U+56BF.0	fCantonese	gau6
U+56BF.0	fMandarin	huo4
U+56BF.0	fEnglish	 a lump; noun classifier
U+56C0.0	fUTF8	囀
U+56C0.0	fCNS11643	1-7A65
U+56C0.0	fBig5	C4DA
U+56C0.0	fGB	1-5F79
U+56C0.0	fR/S	 30.18
U+56C0.0	fTotalStrokes	 21
U+56C0.0	fCangjie	RJJI
U+56C0.0	fFourCorner	65043
U+56C0.0	fHakka	 zon3; zhon3; zhon5
U+56C0.0	fCantonese	zyun2; zyun3
U+56C0.0	fMandarin	zhuan4; zhuan3
U+56C0.0	fEnglish	[1] chirp; warble; twitter [2] pleasing to the ear
U+56C1.0	fUTF8	囁
U+56C1.0	fCNS11643	1-7A64
U+56C1.0	fBig5	C4D9
U+56C1.0	fGB	1-603F
U+56C1.0	fR/S	 30.18
U+56C1.0	fTotalStrokes	 21
U+56C1.0	fCangjie	RSJJ
U+56C1.0	fFourCorner	61041
U+56C1.0	fHakka	 ngiap7; gniap7
U+56C1.0	fCantonese	zip3; nip6 (also); lip6 (also); jip6; zip6
U+56C1.0	fMandarin	nie4; zhe2
U+56C1.0	fEnglish	 move the mouth (when speaking); falter in speech
U+56C2.0	fUTF8	囂
U+56C2.0	fCNS11643	1-7A66
U+56C2.0	fBig5	C4DB
U+56C2.0	fGB	1-4F79
U+56C2.0	fR/S	 30.18
U+56C2.0	fTotalStrokes	 21
U+56C2.0	fCangjie	RRMCR
U+56C2.0	fFourCorner	66668
U+56C2.0	fHakka	 hiau1; hieu1
U+56C2.0	fCantonese	hiu1; ngou4
U+56C2.0	fMandarin	xiao1; ao2
U+56C2.0	fEnglish	[1] noisy; clamor; hubbub [2] be haughty or proud
U+56C3.0	fUTF8	囃
U+56C3.0	fCNS11643	2-6B52
U+56C3.0	fBig5	F5C4
U+56C3.0	fGB	E-2530
U+56C3.0	fR/S	 30.18
U+56C3.0	fTotalStrokes	 21
U+56C3.0	fCangjie	RYDG
U+56C3.0	fFourCorner	60014
U+56C3.0	fHakka	 cap8
U+56C3.0	fCantonese	caap3; zaa3; zaap6
U+56C3.0	fMandarin	ca4
U+56C3.0	fEnglish	the hissing noise by which applause is given in a theatre
U+56C4.0	fUTF8	囄
U+56C4.0	fCNS11643	3-6728
U+56C4.0	fGB	E-2531
U+56C4.0	fR/S	 30.18
U+56C4.0	fTotalStrokes	 21
U+56C4.0	fCangjie	RYBG
U+56C4.0	fCantonese	lei4; lai4
U+56C4.0	fMandarin	li2; lei1
U+56C4.0	fEnglish	come
U+56C5.0	fUTF8	囅
U+56C5.0	fCNS11643	2-6D46
U+56C5.0	fBig5	F6D8
U+56C5.0	fGB	1-5966
U+56C5.0	fR/S	 30.19
U+56C5.0	fTotalStrokes	 22
U+56C5.0	fCangjie	RJSTV
U+56C5.0	fFourCorner	67532
U+56C5.0	fHakka	 chan5; can5; zhan3; zhen3
U+56C5.0	fCantonese	cin2; zin2 (also)
U+56C5.0	fMandarin	chan3; che4
U+56C5.0	fEnglish	smile
U+56C6.0	fUTF8	囆
U+56C6.0	fCNS11643	2-6D45
U+56C6.0	fBig5	F6D7
U+56C6.0	fGB	5-3875
U+56C6.0	fR/S	 30.19
U+56C6.0	fTotalStrokes	 22
U+56C6.0	fCangjie	RTWI
U+56C6.0	fFourCorner	64036
U+56C6.0	fCantonese	caai3
U+56C6.0	fMandarin	chai4
U+56C7.0	fUTF8	囇
U+56C7.0	fCNS11643	3-5F40
U+56C7.0	fGB	E-2532
U+56C7.0	fR/S	 30.19
U+56C7.0	fTotalStrokes	 22; 21
U+56C7.0	fCangjie	RMMP
U+56C7.0	fFourCorner	61010
U+56C7.0	fCantonese	lai6
U+56C7.0	fMandarin	li4
U+56C7.0	fEnglish	same as &#x3611;
U+56C8.0	fUTF8	囈
U+56C8.0	fCNS11643	1-7B59
U+56C8.0	fBig5	C56D
U+56C8.0	fGB	1-5F3D
U+56C8.0	fR/S	 30.19
U+56C8.0	fTotalStrokes	 22
U+56C8.0	fCangjie	RTGI
U+56C8.0	fFourCorner	64031
U+56C8.0	fHakka	 ngi5; ngi6; ngi3
U+56C8.0	fCantonese	ngai6; ai6 (also)
U+56C8.0	fMandarin	yi4
U+56C8.0	fEnglish	 talk in one's sleep; somniloquy
U+56C9.0	fUTF8	囉
U+56C9.0	fCNS11643	1-7B5B
U+56C9.0	fBig5	C56F
U+56C9.0	fGB	1-7853
U+56C9.0	fR/S	 30.19
U+56C9.0	fTotalStrokes	 22
U+56C9.0	fCangjie	RWLG
U+56C9.0	fFourCorner	66014
U+56C9.0	fHakka	  lo1; lo2; lo5
U+56C9.0	fCantonese	lo1; lo4; lo3
U+56C9.0	fMandarin	luo1; luo4; lou2
U+56C9.0	fEnglish	 chatter
U+56C9.1	fUTF8	囉
U+56C9.1	fCNS11643	1-7B5B
U+56C9.1	fBig5	C56F
U+56C9.1	fGB	1-7853
U+56C9.1	fR/S	 30.19
U+56C9.1	fTotalStrokes	 22
U+56C9.1	fCangjie	RWLG
U+56C9.1	fFourCorner	66014
U+56C9.1	fMandarin	luo2
U+56C9.1	fEnglish	[1] noisiness [2] band of outlaws [3] used as a slightly argumentative final particle
U+56CA.0	fUTF8	囊
U+56CA.0	fCNS11643	1-7B5A
U+56CA.0	fBig5	C56E
U+56CA.0	fGB	0-4452
U+56CA.0	fR/S	 30.19
U+56CA.0	fTotalStrokes	 22
U+56CA.0	fCangjie	JBRRV
U+56CA.0	fFourCorner	00732
U+56CA.0	fHakka	  nong2;  long2; nang2; nong3
U+56CA.0	fCantonese	nong4; long4 (also); nong1 (also); long1 (also); nong2 (also); long2 (also)
U+56CA.0	fMandarin	nang2; nang1
U+56CA.0	fEnglish	[1] bag; purse; sack; travelling bag [2] put in a bag [3] a Chinese family name
U+56CB.0	fUTF8	囋
U+56CB.0	fCNS11643	2-6D47
U+56CB.0	fBig5	F6D9
U+56CB.0	fGB	3-374D
U+56CB.0	fR/S	 30.19
U+56CB.0	fTotalStrokes	 22
U+56CB.0	fCangjie	RHUC
U+56CB.0	fFourCorner	64086
U+56CB.0	fHakka	 zan8; cat8; zan5
U+56CB.0	fCantonese	zaan3; zaat6
U+56CB.0	fMandarin	zan4; can1
U+56CC.0	fUTF8	囌
U+56CC.0	fCNS11643	1-7C34
U+56CC.0	fBig5	C5C8
U+56CC.0	fGB	1-786A
U+56CC.0	fR/S	 30.20; 140.19; 30.19
U+56CC.0	fTotalStrokes	 23
U+56CC.0	fCangjie	RTND
U+56CC.0	fFourCorner	64094
U+56CC.0	fCantonese	sou1
U+56CC.0	fMandarin	su1
U+56CC.0	fEnglish	 loquacious nag
U+56CD.0	fUTF8	囍
U+56CD.0	fCNS11643	2-7034
U+56CD.0	fBig5	F8A6
U+56CD.0	fGB	1-7D6E
U+56CD.0	fR/S	 30.18
U+56CD.0	fTotalStrokes	 21
U+56CD.0	fCangjie	GRGRR
U+56CD.0	fFourCorner	44661
U+56CD.0	fCantonese	hei2
U+56CD.0	fMandarin	xi3
U+56CD.0	fEnglish	double happiness
U+56CE.0	fUTF8	囎
U+56CE.0	fGB	E-2533
U+56CE.0	fR/S	 30.19
U+56CE.0	fTotalStrokes	 22
U+56CE.0	fCangjie	RBCA
U+56CE.0	fCantonese	zang6
U+56CF.0	fUTF8	囏
U+56CF.0	fCNS11643	4-6A44
U+56CF.0	fGB	5-6A78
U+56CF.0	fR/S	  30.20
U+56CF.0	fTotalStrokes	 23
U+56CF.0	fCangjie	TOGRR
U+56CF.0	fFourCorner	44560
U+56CF.0	fCantonese	gaan1
U+56CF.0	fMandarin	jian1
U+56D0.0	fUTF8	囐
U+56D0.0	fCNS11643	4-6A43
U+56D0.0	fGB	5-3954
U+56D0.0	fR/S	  30.20
U+56D0.0	fTotalStrokes	 23
U+56D0.0	fCangjie	RYBK
U+56D0.0	fFourCorner	63030
U+56D0.0	fCantonese	zaan3
U+56D0.0	fMandarin	za2; zan2
U+56D1.0	fUTF8	囑
U+56D1.0	fCNS11643	1-7C5D
U+56D1.0	fBig5	C5F1
U+56D1.0	fGB	1-5676
U+56D1.0	fR/S	 30.21
U+56D1.0	fTotalStrokes	 24
U+56D1.0	fCangjie	RSEI
U+56D1.0	fFourCorner	67027
U+56D1.0	fHakka	 zuk7;  chuk7; zhuk7
U+56D1.0	fCantonese	zuk1
U+56D1.0	fMandarin	zhu3; bao4
U+56D1.0	fEnglish	 order; tell; instruct; ask another to do something; enjoin; direct; entrust; charge
U+56D2.0	fUTF8	囒
U+56D2.0	fCNS11643	3-6071
U+56D2.0	fGB	E-2534
U+56D2.0	fR/S	  30.21
U+56D2.0	fTotalStrokes	 24
U+56D2.0	fCangjie	RTAW
U+56D2.0	fFourCorner	64020
U+56D2.0	fHakka	 lan2
U+56D2.0	fCantonese	laan4; laan1
U+56D2.0	fMandarin	lan2
U+56D2.0	fEnglish	same as &#x3613;
U+56D3.0	fUTF8	囓
U+56D3.0	fCNS11643	2-7033
U+56D3.0	fBig5	F8A5
U+56D3.0	fGB	E-2535
U+56D3.0	fR/S	 30.21
U+56D3.0	fTotalStrokes	 24
U+56D3.0	fCangjie	RQHU
U+56D3.0	fFourCorner	61072
U+56D3.0	fHakka	 ngat7;  ngau1; nget7
U+56D3.0	fCantonese	ngit6; jit6 (also); ngaat6; it6 (also); aat6 (also)
U+56D3.0	fMandarin	nie4
U+56D3.0	fEnglish	gnaw
U+56D4.0	fUTF8	囔
U+56D4.0	fCNS11643	2-707C
U+56D4.0	fBig5	F8EE
U+56D4.0	fGB	0-606C
U+56D4.0	fR/S	 30.22
U+56D4.0	fTotalStrokes	 25
U+56D4.0	fCangjie	RJBV
U+56D4.0	fFourCorner	60032
U+56D4.0	fHakka	 nang5; long2; nang1
U+56D4.0	fCantonese	nong4; long4 (also)
U+56D4.0	fMandarin	nang1; nang2
U+56D4.0	fEnglish	muttering; indistinct speech
U+56D5.0	fUTF8	囕
U+56D5.0	fGB	1-7D3A
U+56D5.0	fR/S	 30.22
U+56D5.0	fTotalStrokes	 25
U+56D5.0	fCangjie	RSWU
U+56D5.0	fCantonese	hei3
U+56D6.0	fUTF8	囖
U+56D6.0	fGB	1-7C39
U+56D6.0	fR/S	 30.25
U+56D6.0	fTotalStrokes	 28
U+56D6.0	fCangjie	RHWG
U+56D6.0	fCantonese	lo1
U+56D6.0	fEnglish	final particle for pointing out the obvious
U+56D7.0	fUTF8	囗
U+56D7.0	fCNS11643	2-212A
U+56D7.0	fBig5	C949
U+56D7.0	fGB	0-606D
U+56D7.0	fR/S	 31.0
U+56D7.0	fTotalStrokes	 3
U+56D7.0	fCangjie	BM
U+56D7.0	fFourCorner	60000
U+56D7.0	fHakka	 wui2; wi2; wui3; vui2
U+56D7.0	fCantonese	wai4; gwok3
U+56D7.0	fMandarin	wei2; guo2
U+56D7.0	fEnglish	 erect; proud upright bald
U+56D8.0	fUTF8	囘
U+56D8.0	fCNS11643	3-2177
U+56D8.0	fGB	E-2536
U+56D8.0	fR/S	 13.3; 31.2
U+56D8.0	fTotalStrokes	 5
U+56D8.0	fCangjie	BRU
U+56D8.0	fFourCorner	77220
U+56D8.0	fHakka	 fui2; fi2
U+56D8.0	fCantonese	wui4
U+56D8.0	fMandarin	hui2
U+56D8.0	fEnglish	 return; turn around a time
U+56D9.0	fUTF8	囙
U+56D9.0	fCNS11643	4-216F
U+56D9.0	fGB	E-2537
U+56D9.0	fR/S	 31.2
U+56D9.0	fTotalStrokes	 5
U+56D9.0	fCangjie	WS
U+56D9.0	fFourCorner	60170
U+56D9.0	fHakka	 jin1
U+56D9.0	fCantonese	jan1
U+56D9.0	fMandarin	yin1
U+56DA.0	fUTF8	囚
U+56DA.0	fCNS11643	1-463F
U+56DA.0	fBig5	A57D
U+56DA.0	fGB	0-4774
U+56DA.0	fR/S	 31.2
U+56DA.0	fTotalStrokes	 5
U+56DA.0	fCangjie	WO
U+56DA.0	fFourCorner	60800
U+56DA.0	fHakka	  siu2; siu3; ciu2
U+56DA.0	fCantonese	cau4
U+56DA.0	fMandarin	qiu2
U+56DA.0	fEnglish	[1] prisoner; convict [2] confine; imprison
U+56DB.0	fUTF8	四
U+56DB.0	fCNS11643	1-463E
U+56DB.0	fBig5	A57C
U+56DB.0	fGB	0-4B44
U+56DB.0	fR/S	 31.2
U+56DB.0	fTotalStrokes	 5
U+56DB.0	fCangjie	WC
U+56DB.0	fFourCorner	60210
U+56DB.0	fHakka	  si5
U+56DB.0	fCantonese	sei3; si3 (also)
U+56DB.0	fMandarin	si4
U+56DB.0	fEnglish	[1] four; fourth [2] all around
U+56DC.0	fUTF8	囜
U+56DC.0	fCNS11643	4-216D
U+56DC.0	fGB	E-2538
U+56DC.0	fR/S	  31.2
U+56DC.0	fTotalStrokes	 5
U+56DC.0	fCangjie	WI
U+56DC.0	fFourCorner	60730
U+56DC.0	fCantonese	nan4; lan4 (also)
U+56DC.0	fMandarin	nin2
U+56DD.0	fUTF8	囝
U+56DD.0	fCNS11643	1-4760
U+56DD.0	fBig5	A65F
U+56DD.0	fGB	0-606E
U+56DD.0	fR/S	 31.3
U+56DD.0	fTotalStrokes	 6
U+56DD.0	fCangjie	WND
U+56DD.0	fFourCorner	60407
U+56DD.0	fHakka	 ngiet8; k'ie3; kian3; ken3
U+56DD.0	fCantonese	zai2; naam4; laam4 (also); zoi2
U+56DD.0	fMandarin	jian3; nan1; yue4
U+56DD.0	fEnglish	baby; infant
U+56DE.0	fUTF8	回
U+56DE.0	fCNS11643	1-475F
U+56DE.0	fBig5	A65E
U+56DE.0	fGB	0-3B58
U+56DE.0	fR/S	 31.3
U+56DE.0	fTotalStrokes	 6
U+56DE.0	fCangjie	WR
U+56DE.0	fFourCorner	60600
U+56DE.0	fHakka	  fui2; fi2; fui3; foi2
U+56DE.0	fCantonese	wui4; wui2 (also)
U+56DE.0	fMandarin	hui2; hui4
U+56DE.0	fEnglish	[1] return; go back; bring back [2] reply; answer [3] turn around [4] the number of times [5] a kind; sort [6] chapters in a novel [7] of Mohammedanism; Moslems
U+56DF.0	fUTF8	囟
U+56DF.0	fCNS11643	2-2227
U+56DF.0	fBig5	C9C7
U+56DF.0	fGB	0-5836
U+56DF.0	fR/S	 31.3
U+56DF.0	fTotalStrokes	 6
U+56DF.0	fCangjie	HWK
U+56DF.0	fFourCorner	26000
U+56DF.0	fHakka	  sim5;  sin5; sim3
U+56DF.0	fCantonese	seon3; seon2 (also)
U+56DF.0	fMandarin	xin4
U+56DF.0	fEnglish	the top top of the human head; skull
U+56E0.0	fUTF8	因
U+56E0.0	fCNS11643	1-475E
U+56E0.0	fBig5	A65D
U+56E0.0	fGB	0-5272
U+56E0.0	fR/S	 31.3
U+56E0.0	fTotalStrokes	 6
U+56E0.0	fCangjie	WK
U+56E0.0	fFourCorner	60430
U+56E0.0	fHakka	  jin1; rin1
U+56E0.0	fCantonese	jan1
U+56E0.0	fMandarin	yin1
U+56E0.0	fEnglish	[1] cause; reason [2] for because (of) [3] in accordance with; according to; on the basis of; in the light of [4] follow (a practice, convention, etc.)
U+56E1.0	fUTF8	囡
U+56E1.0	fCNS11643	2-2226
U+56E1.0	fBig5	C9C6
U+56E1.0	fGB	0-606F
U+56E1.0	fR/S	 31.3
U+56E1.0	fTotalStrokes	 6
U+56E1.0	fCangjie	WV
U+56E1.0	fFourCorner	60404
U+56E1.0	fHakka	 ngiap8; lam2; ngiap7
U+56E1.0	fCantonese	naam4; laam4 (also)
U+56E1.0	fMandarin	nan1
U+56E1.0	fEnglish	[1] one's daughter filch secrete [2] daughter [3] child
U+56E2.0	fUTF8	团
U+56E2.0	fGB	0-4D45
U+56E2.0	fR/S	 31.3
U+56E2.0	fTotalStrokes	 6
U+56E2.0	fCangjie	WDH
U+56E2.0	fFourCorner	6024
U+56E2.0	fHakka	 ton2
U+56E2.0	fCantonese	tyun4; tyun2 (also)
U+56E2.0	fMandarin	tuan2; shuan4; tuan1
U+56E2.0	fEnglish	simplified form of &#x5718;
U+56E3.0	fUTF8	団
U+56E3.0	fCNS11643	3-233E
U+56E3.0	fGB	E-2539
U+56E3.0	fR/S	 31.3
U+56E3.0	fTotalStrokes	 6
U+56E3.0	fCangjie	WDI
U+56E3.0	fFourCorner	60340
U+56E3.0	fCantonese	tyun4
U+56E3.0	fMandarin	tuan2
U+56E3.0	fEnglish	 sphere; ball; circle mass; lump
U+56E4.0	fUTF8	囤
U+56E4.0	fCNS11643	1-495B
U+56E4.0	fBig5	A779
U+56E4.0	fGB	0-365A
U+56E4.0	fR/S	 31.4
U+56E4.0	fTotalStrokes	 7
U+56E4.0	fCangjie	WPU
U+56E4.0	fFourCorner	60717
U+56E4.0	fHakka	 dun3;  tun2; tun1; tun3; tun5
U+56E4.0	fCantonese	deon6; tyun4; teon4
U+56E4.0	fMandarin	tun2; dun4
U+56E4.0	fEnglish	 store up; hoard; stockpile
U+56E5.0	fUTF8	囥
U+56E5.0	fCNS11643	2-2349
U+56E5.0	fBig5	CAA9
U+56E5.0	fGB	3-3753
U+56E5.0	fR/S	 31.4
U+56E5.0	fTotalStrokes	 7
U+56E5.0	fCangjie	WYHN
U+56E5.0	fFourCorner	60211
U+56E5.0	fHakka	  kong5
U+56E5.0	fCantonese	kong3
U+56E5.0	fMandarin	kang4
U+56E5.0	fEnglish	hide
U+56E6.0	fUTF8	囦
U+56E6.0	fCNS11643	3-252C
U+56E6.0	fGB	5-3977
U+56E6.0	fR/S	  31.4
U+56E6.0	fTotalStrokes	 7
U+56E6.0	fCangjie	WE
U+56E6.0	fFourCorner	60230
U+56E6.0	fCantonese	jyun1
U+56E6.0	fMandarin	yuan1
U+56E7.0	fUTF8	囧
U+56E7.0	fCNS11643	2-2348
U+56E7.0	fBig5	CAA8
U+56E7.0	fGB	5-3976
U+56E7.0	fR/S	 31.4
U+56E7.0	fTotalStrokes	 7
U+56E7.0	fCangjie	WCB
U+56E7.0	fFourCorner	60227
U+56E7.0	fHakka	 gin3;  kwen3
U+56E7.0	fCantonese	gwing2
U+56E7.0	fMandarin	jiong3; jing3
U+56E8.0	fUTF8	囨
U+56E8.0	fCNS11643	F-2350
U+56E8.0	fGB	E-253A
U+56E8.0	fR/S	 31.4
U+56E8.0	fTotalStrokes	 7
U+56E8.0	fCangjie	WMF
U+56E8.0	fCantonese	pin1
U+56E8.0	fMandarin	pian1
U+56E9.0	fUTF8	囩
U+56E9.0	fCNS11643	3-2526
U+56E9.0	fGB	3-3751
U+56E9.0	fR/S	  31.4
U+56E9.0	fTotalStrokes	 7
U+56E9.0	fCangjie	WMMI
U+56E9.0	fFourCorner	60730
U+56E9.0	fHakka	 jun2
U+56E9.0	fCantonese	wan4
U+56E9.0	fMandarin	yun4
U+56EA.0	fUTF8	囪
U+56EA.0	fCNS11643	1-4959
U+56EA.0	fBig5	A777
U+56EA.0	fGB	E-253B
U+56EA.0	fR/S	 31.4
U+56EA.0	fTotalStrokes	 7
U+56EA.0	fCangjie	HWKK
U+56EA.0	fFourCorner	26000
U+56EA.0	fHakka	 cung1
U+56EA.0	fCantonese	cung1; coeng1; tung1
U+56EA.0	fMandarin	cong1; chuang1
U+56EA.0	fEnglish	chimney; funnel
U+56EB.0	fUTF8	囫
U+56EB.0	fCNS11643	1-495C
U+56EB.0	fBig5	A77A
U+56EB.0	fGB	0-6071
U+56EB.0	fR/S	 31.4
U+56EB.0	fTotalStrokes	 7
U+56EB.0	fCangjie	WPHH
U+56EB.0	fFourCorner	60227
U+56EB.0	fHakka	 wut7; fut8; fut7; gwun2; vut7
U+56EB.0	fCantonese	fat1; wat6
U+56EB.0	fMandarin	hu2
U+56EB.0	fEnglish	entire; whole
U+56EC.0	fUTF8	囬
U+56EC.0	fCNS11643	3-252B
U+56EC.0	fGB	E-253C
U+56EC.0	fR/S	 31.4
U+56EC.0	fTotalStrokes	 7
U+56EC.0	fCangjie	WSL
U+56EC.0	fFourCorner	60220
U+56EC.0	fHakka	 fui2; fi2
U+56EC.0	fCantonese	wui4
U+56EC.0	fMandarin	hui2
U+56EC.0	fEnglish	to return to or from
U+56ED.0	fUTF8	园
U+56ED.0	fCNS11643	3-252A
U+56ED.0	fGB	0-5430
U+56ED.0	fR/S	  31.4
U+56ED.0	fTotalStrokes	 7
U+56ED.0	fCangjie	WMMU
U+56ED.0	fFourCorner	60210
U+56ED.0	fHakka	 ngien2; ren2; jen2; mu2
U+56ED.0	fCantonese	jyun4; jyun2 (also); mou4
U+56ED.0	fMandarin	tuan2; mo2; yuan2
U+56ED.0	fEnglish	simplified form of &#x5712;
U+56EE.0	fUTF8	囮
U+56EE.0	fCNS11643	2-2347
U+56EE.0	fBig5	CAA7
U+56EE.0	fGB	3-3752
U+56EE.0	fR/S	 31.4
U+56EE.0	fTotalStrokes	 7
U+56EE.0	fCangjie	WOP
U+56EE.0	fFourCorner	60211
U+56EE.0	fHakka	  ngo2
U+56EE.0	fCantonese	ngo4; jau4 (also); o4
U+56EE.0	fMandarin	e2; you2; you3
U+56EE.0	fEnglish	 inveigle; decoy cheat
U+56EF.0	fUTF8	囯
U+56EF.0	fCNS11643	3-2529
U+56EF.0	fGB	E-253D
U+56EF.0	fR/S	 31.4
U+56EF.0	fTotalStrokes	 7
U+56EF.0	fCangjie	WMG
U+56EF.0	fFourCorner	60100
U+56EF.0	fHakka	 kwet7;  gok7
U+56EF.0	fCantonese	gwok3; gok3 (also)
U+56EF.0	fMandarin	guo2
U+56EF.0	fEnglish	 kingdom; nation; country
U+56F0.0	fUTF8	困
U+56F0.0	fCNS11643	1-495A
U+56F0.0	fBig5	A778
U+56F0.0	fGB	0-4027
U+56F0.0	fR/S	 31.4
U+56F0.0	fTotalStrokes	 7
U+56F0.0	fCangjie	WD
U+56F0.0	fFourCorner	60904
U+56F0.0	fHakka	  kun5; kwun5
U+56F0.0	fCantonese	kwan3
U+56F0.0	fMandarin	kun4
U+56F0.0	fEnglish	[1] difficult; hard [2] poor [3] tired; weary; fatigued
U+56F1.0	fUTF8	囱
U+56F1.0	fCNS11643	3-2548
U+56F1.0	fGB	0-3451
U+56F1.0	fR/S	 31.4
U+56F1.0	fTotalStrokes	 7
U+56F1.0	fCangjie	HWNI
U+56F1.0	fFourCorner	26000
U+56F1.0	fHakka	  cung1
U+56F1.0	fCantonese	cung1; coeng1
U+56F1.0	fMandarin	cong1; chuang1
U+56F1.0	fEnglish	 chimney; flue
U+56F2.0	fUTF8	囲
U+56F2.0	fCNS11643	3-2527
U+56F2.0	fGB	E-253E
U+56F2.0	fR/S	  31.4
U+56F2.0	fTotalStrokes	 7
U+56F2.0	fCangjie	WTT
U+56F2.0	fFourCorner	60550
U+56F2.0	fCantonese	tung1
U+56F2.0	fMandarin	tong1; wei2
U+56F2.0	fEnglish	 surround; encircle; corral whip
U+56F3.0	fUTF8	図
U+56F3.0	fCNS11643	3-2528
U+56F3.0	fGB	E-253F
U+56F3.0	fR/S	 31.4
U+56F3.0	fTotalStrokes	 7
U+56F3.0	fCangjie	WYK
U+56F3.0	fFourCorner	60400
U+56F3.0	fCantonese	tou4
U+56F3.0	fMandarin	tu2
U+56F3.0	fEnglish	 diagram chart; map; picture
U+56F4.0	fUTF8	围
U+56F4.0	fGB	0-4E27
U+56F4.0	fR/S	 31.4
U+56F4.0	fTotalStrokes	 7
U+56F4.0	fCangjie	WQS
U+56F4.0	fHakka	 vui2
U+56F4.0	fCantonese	wai4
U+56F4.0	fMandarin	wei2; wei4
U+56F4.0	fEnglish	simplified form of &#x570D;
U+56F5.0	fUTF8	囵
U+56F5.0	fGB	0-6070
U+56F5.0	fR/S	 31.4
U+56F5.0	fTotalStrokes	 7
U+56F5.0	fCangjie	WOP
U+56F5.0	fHakka	 lun2
U+56F5.0	fCantonese	leon4
U+56F5.0	fMandarin	lun2
U+56F5.0	fEnglish	simplified form of &#x5707;
U+56F6.0	fUTF8	囶
U+56F6.0	fCNS11643	3-2822
U+56F6.0	fGB	E-2540
U+56F6.0	fR/S	 31.5
U+56F6.0	fTotalStrokes	 8
U+56F6.0	fCangjie	WCG
U+56F6.0	fFourCorner	60100
U+56F6.0	fHakka	 kwet7;  gok7; giet7; get7; kuet7; gwet7
U+56F6.0	fCantonese	gwok3; gok3 (also)
U+56F6.0	fMandarin	guo2
U+56F6.0	fEnglish	ancient form of &#x570B;
U+56F7.0	fUTF8	囷
U+56F7.0	fCNS11643	2-2570
U+56F7.0	fBig5	CBF0
U+56F7.0	fGB	3-3754
U+56F7.0	fR/S	 31.5
U+56F7.0	fTotalStrokes	 8
U+56F7.0	fCangjie	WHD
U+56F7.0	fFourCorner	60904
U+56F7.0	fHakka	 kun1; kwun1
U+56F7.0	fCantonese	kwan1
U+56F7.0	fMandarin	qun1; jun1
U+56F7.0	fEnglish	 granary
U+56F8.0	fUTF8	囸
U+56F8.0	fCNS11643	3-277D
U+56F8.0	fGB	5-3978
U+56F8.0	fR/S	 31.5
U+56F8.0	fTotalStrokes	 8
U+56F8.0	fCangjie	WMYM
U+56F8.0	fFourCorner	60100
U+56F8.0	fHakka	 ngit7; gnit7; ngit8
U+56F8.0	fCantonese	jat6; jat2 (also); mik6
U+56F8.0	fMandarin	ri4; mi4
U+56F8.0	fEnglish	ancient form of &#x65E5;
U+56F9.0	fUTF8	囹
U+56F9.0	fCNS11643	2-2571
U+56F9.0	fBig5	CBF1
U+56F9.0	fGB	0-6072
U+56F9.0	fR/S	 31.5
U+56F9.0	fTotalStrokes	 8
U+56F9.0	fCangjie	WOII
U+56F9.0	fFourCorner	60307
U+56F9.0	fHakka	 lin2; long1; lang2
U+56F9.0	fCantonese	ling4
U+56F9.0	fMandarin	ling2
U+56F9.0	fEnglish	 prison; enclosure; jail
U+56FA.0	fUTF8	固
U+56FA.0	fCNS11643	1-4C56
U+56FA.0	fBig5	A954
U+56FA.0	fGB	0-394C
U+56FA.0	fR/S	 31.5
U+56FA.0	fTotalStrokes	 8
U+56FA.0	fCangjie	WJR
U+56FA.0	fFourCorner	60604
U+56FA.0	fHakka	  gu5
U+56FA.0	fCantonese	gu3; gwu3 (also)
U+56FA.0	fMandarin	gu4
U+56FA.0	fEnglish	[1] become solid; solidify [2] stable; firm; sturdy; solid; strong [3] obstinate; stubborn [4] base; mean; ignorant [5] originally; certainly; assuredly; as a matter of course [6] chronic [7] indeed [8] admittedly; no doubt [9] strengthen; secure; consolidate [10] a Chinese family name
U+56FB.0	fUTF8	囻
U+56FB.0	fCNS11643	3-277C
U+56FB.0	fGB	E-2541
U+56FB.0	fR/S	  31.5
U+56FB.0	fTotalStrokes	 8
U+56FB.0	fCangjie	WRVP
U+56FB.0	fFourCorner	60740
U+56FB.0	fHakka	 kwet7;  gok7
U+56FB.0	fCantonese	gwok3; gok3 (also)
U+56FB.0	fMandarin	guo2
U+56FB.0	fEnglish	 kingdom; nation; country
U+56FC.0	fUTF8	囼
U+56FC.0	fCNS11643	3-2821
U+56FC.0	fGB	5-3979
U+56FC.0	fR/S	  31.5
U+56FC.0	fTotalStrokes	 8
U+56FC.0	fCangjie	WIR
U+56FC.0	fFourCorner	60600
U+56FC.0	fHakka	  toi1
U+56FC.0	fCantonese	toi1
U+56FC.0	fMandarin	tai1
U+56FC.0	fEnglish	same as &#x80CE;
U+56FD.0	fUTF8	国
U+56FD.0	fCNS11643	3-277B
U+56FD.0	fGB	0-397A
U+56FD.0	fR/S	 31.5
U+56FD.0	fTotalStrokes	 8
U+56FD.0	fCangjie	WMGI
U+56FD.0	fFourCorner	60100
U+56FD.0	fHakka	 kwet7; gok7
U+56FD.0	fCantonese	gwok3; gok3 (also)
U+56FD.0	fMandarin	guo2
U+56FD.0	fEnglish	simplified form of &#x570B;
U+56FE.0	fUTF8	图
U+56FE.0	fGB	0-4D3C
U+56FE.0	fR/S	 31.5
U+56FE.0	fTotalStrokes	 8
U+56FE.0	fCangjie	WHEY
U+56FE.0	fFourCorner	6030
U+56FE.0	fHakka	 tu2
U+56FE.0	fCantonese	tou4
U+56FE.0	fMandarin	tu2; tu1
U+56FE.0	fEnglish	simplified form of &#x5716;
U+56FF.0	fUTF8	囿
U+56FF.0	fCNS11643	1-504C
U+56FF.0	fBig5	ABAA
U+56FF.0	fGB	0-6073
U+56FF.0	fR/S	 31.6
U+56FF.0	fTotalStrokes	 9
U+56FF.0	fCangjie	WKB
U+56FF.0	fFourCorner	60227
U+56FF.0	fHakka	 ju3; ju5; riu3; jiu3
U+56FF.0	fCantonese	jau6
U+56FF.0	fMandarin	you4; you3; yu4
U+56FF.0	fEnglish	[1] enclosure for keeping animals; menagerie; garden [2] confine; enclose; limit; constrain
U+5700.0	fUTF8	圀
U+5700.0	fCNS11643	3-2B65
U+5700.0	fGB	5-397A
U+5700.0	fR/S	  31.6
U+5700.0	fTotalStrokes	 9
U+5700.0	fCangjie	WCYS
U+5700.0	fFourCorner	60220
U+5700.0	fHakka	 kwet7; gok7
U+5700.0	fCantonese	gwok3; gok3 (also)
U+5700.0	fMandarin	guo2
U+5700.0	fEnglish	ancient form of &#x570B;
U+5701.0	fUTF8	圁
U+5701.0	fCNS11643	2-2E4A
U+5701.0	fBig5	D148
U+5701.0	fGB	3-3757
U+5701.0	fR/S	 31.7
U+5701.0	fTotalStrokes	 10
U+5701.0	fCangjie	WYMR
U+5701.0	fFourCorner	60601
U+5701.0	fCantonese	ngan4; an4 (also)
U+5701.0	fMandarin	yin2; kun3
U+5702.0	fUTF8	圂
U+5702.0	fCNS11643	2-2E4B
U+5702.0	fBig5	D149
U+5702.0	fGB	3-3756
U+5702.0	fR/S	 31.7
U+5702.0	fTotalStrokes	 10
U+5702.0	fCangjie	WMSO
U+5702.0	fFourCorner	60232
U+5702.0	fHakka	 fun3; fun5
U+5702.0	fCantonese	wan6; waan6
U+5702.0	fMandarin	hun4
U+5702.0	fEnglish	 pig-sty privy
U+5703.0	fUTF8	圃
U+5703.0	fCNS11643	1-5467
U+5703.0	fBig5	AE45
U+5703.0	fGB	0-4654
U+5703.0	fR/S	 31.7
U+5703.0	fTotalStrokes	 10
U+5703.0	fCangjie	WIJB
U+5703.0	fFourCorner	60227
U+5703.0	fHakka	 pu3; pu1
U+5703.0	fCantonese	pou2; bou2 (also)
U+5703.0	fMandarin	pu3; bu3
U+5703.0	fEnglish	[1] vegetable garden (or plot); nursery; orchard; plantation [2] planter; gardener
U+5704.0	fUTF8	圄
U+5704.0	fCNS11643	1-5468
U+5704.0	fBig5	AE46
U+5704.0	fGB	0-6074
U+5704.0	fR/S	 31.7
U+5704.0	fTotalStrokes	 10
U+5704.0	fCangjie	WMMR
U+5704.0	fFourCorner	60601
U+5704.0	fHakka	 ngi3; ng2; ngu2; ngi2; ni3; ngi1
U+5704.0	fCantonese	jyu5
U+5704.0	fMandarin	yu3
U+5704.0	fEnglish	 prison; jail
U+5705.0	fUTF8	圅
U+5705.0	fCNS11643	3-3027
U+5705.0	fGB	E-2542
U+5705.0	fR/S	  31.7
U+5705.0	fTotalStrokes	 10
U+5705.0	fCangjie	NIWJ
U+5705.0	fFourCorner	17600
U+5705.0	fHakka	  ham2
U+5705.0	fCantonese	haam4
U+5705.0	fMandarin	han2
U+5705.0	fEnglish	 correspondence case; box
U+5706.0	fUTF8	圆
U+5706.0	fGB	0-5432
U+5706.0	fR/S	 31.7
U+5706.0	fTotalStrokes	 10
U+5706.0	fCangjie	WRBO
U+5706.0	fHakka	 jen2
U+5706.0	fCantonese	jyun4; jyun2 (also)
U+5706.0	fMandarin	yuan2; yun4
U+5706.0	fEnglish	simplified form of &#x5713;
U+5707.0	fUTF8	圇
U+5707.0	fCNS11643	2-342F
U+5707.0	fBig5	D4AC
U+5707.0	fGB	1-6070
U+5707.0	fR/S	 31.8
U+5707.0	fTotalStrokes	 11
U+5707.0	fCangjie	WOMB
U+5707.0	fFourCorner	60227
U+5707.0	fHakka	 lun2
U+5707.0	fCantonese	leon4
U+5707.0	fMandarin	lun2
U+5707.0	fEnglish	entire; whole
U+5708.0	fUTF8	圈
U+5708.0	fCNS11643	1-594E
U+5708.0	fBig5	B0E9
U+5708.0	fGB	0-4826
U+5708.0	fR/S	 31.8
U+5708.0	fTotalStrokes	 11
U+5708.0	fCangjie	WFQU
U+5708.0	fFourCorner	60712
U+5708.0	fHakka	 ken1; kien1; kien2; ken5; kian5; kwan1
U+5708.0	fCantonese	hyun1; gyun6
U+5708.0	fMandarin	quan1; quan4; quan3
U+5708.0	fEnglish	 encircle; confine
U+5708.1	fUTF8	圈
U+5708.1	fCNS11643	1-594E
U+5708.1	fBig5	B0E9
U+5708.1	fGB	0-4826
U+5708.1	fR/S	 31.8
U+5708.1	fTotalStrokes	 11
U+5708.1	fCangjie	WFQU
U+5708.1	fFourCorner	60712
U+5708.1	fHakka	 kian1
U+5708.1	fMandarin	juan1
U+5708.1	fEnglish	[1] circle; ring [2] with a return to the starting point; round
U+5708.2	fUTF8	圈
U+5708.2	fCNS11643	1-594E
U+5708.2	fBig5	B0E9
U+5708.2	fGB	0-4826
U+5708.2	fR/S	 31.8
U+5708.2	fTotalStrokes	 11
U+5708.2	fCangjie	WFQU
U+5708.2	fFourCorner	60712
U+5708.2	fMandarin	juan4
U+5708.2	fEnglish	 enclosure or a pen for keeping livestock; corral
U+5709.0	fUTF8	圉
U+5709.0	fCNS11643	1-5950
U+5709.0	fBig5	B0EB
U+5709.0	fGB	0-6076
U+5709.0	fR/S	 31.8
U+5709.0	fTotalStrokes	 11
U+5709.0	fCangjie	WGTJ
U+5709.0	fFourCorner	60401
U+5709.0	fHakka	 ngi3
U+5709.0	fCantonese	jyu5
U+5709.0	fMandarin	yu3; yu4
U+5709.0	fEnglish	 stable; corral; enclosure front
U+570A.0	fUTF8	圊
U+570A.0	fCNS11643	2-342E
U+570A.0	fBig5	D4AB
U+570A.0	fGB	0-6075
U+570A.0	fR/S	 31.8
U+570A.0	fTotalStrokes	 11
U+570A.0	fCangjie	WQMB
U+570A.0	fFourCorner	60227
U+570A.0	fHakka	 cin1
U+570A.0	fCantonese	cing1
U+570A.0	fMandarin	qing1
U+570A.0	fEnglish	rest room
U+570B.0	fUTF8	國
U+570B.0	fCNS11643	1-594F
U+570B.0	fBig5	B0EA
U+570B.0	fGB	1-397A
U+570B.0	fR/S	 31.8
U+570B.0	fTotalStrokes	 11
U+570B.0	fCangjie	WIRM
U+570B.0	fFourCorner	60153
U+570B.0	fHakka	 kwet7;  gok7; giet7; get7; kuet7; gwet7
U+570B.0	fCantonese	gwok3; gok3 (also)
U+570B.0	fMandarin	guo2
U+570B.0	fEnglish	[1] nation; country; nation-state; state; kingdom [2] national; governmental [3] Chinese [4] a Chinese family name
U+570C.0	fUTF8	圌
U+570C.0	fCNS11643	2-3A60
U+570C.0	fBig5	D87C
U+570C.0	fGB	3-3758
U+570C.0	fR/S	 31.9
U+570C.0	fTotalStrokes	 12
U+570C.0	fCangjie	WUMB
U+570C.0	fFourCorner	60227
U+570C.0	fCantonese	seoi4; cyun4; cyun2
U+570C.0	fMandarin	chui2; chuan2; chuan1
U+570C.0	fEnglish	fence
U+570D.0	fUTF8	圍
U+570D.0	fCNS11643	1-5E58
U+570D.0	fBig5	B3F2
U+570D.0	fGB	1-4E27
U+570D.0	fR/S	 31.9
U+570D.0	fTotalStrokes	 12
U+570D.0	fCangjie	WDMQ
U+570D.0	fFourCorner	60506
U+570D.0	fHakka	 wui2; wi2; wui3; vui2
U+570D.0	fCantonese	wai4
U+570D.0	fMandarin	wei2; wei4
U+570D.0	fEnglish	[1] surround; encircle; enclose; hem in [2] surroundings; environment [3] the circumference of a circle formed by a person's arms
U+570E.0	fUTF8	圎
U+570E.0	fCNS11643	3-3A44
U+570E.0	fGB	E-2543
U+570E.0	fR/S	 31.9
U+570E.0	fTotalStrokes	 12
U+570E.0	fCangjie	WIBC
U+570E.0	fFourCorner	60800
U+570E.0	fHakka	 jen2
U+570E.0	fCantonese	jyun4
U+570E.0	fMandarin	yuan2; yun4
U+570E.0	fEnglish	circle; round, circular; complete
U+570F.0	fUTF8	圏
U+570F.0	fCNS11643	3-3A43
U+570F.0	fGB	E-2544
U+570F.0	fR/S	 31.8
U+570F.0	fTotalStrokes	 11; 12
U+570F.0	fCangjie	XWFQU
U+570F.0	fFourCorner	60710
U+570F.0	fHakka	 ken1; ken5
U+570F.0	fCantonese	hyun1; gyun6
U+570F.0	fMandarin	quan1; quan4; quan3
U+570F.0	fEnglish	same as &#x5708;
U+5710.0	fUTF8	圐
U+5710.0	fCNS11643	4-3668
U+5710.0	fGB	3-3759
U+5710.0	fR/S	 31.9
U+5710.0	fTotalStrokes	 12
U+5710.0	fCangjie	WWLS
U+5710.0	fFourCorner	60220
U+5710.0	fCantonese	ku1; kwu1 (also)
U+5710.0	fMandarin	ku1
U+5711.0	fUTF8	圑
U+5711.0	fCNS11643	4-3C52
U+5711.0	fGB	E-2545
U+5711.0	fR/S	  31.10
U+5711.0	fTotalStrokes	 13
U+5711.0	fCangjie	WIBI
U+5711.0	fFourCorner	60340
U+5711.0	fCantonese	pou2
U+5711.0	fMandarin	fu4; pu3
U+5712.0	fUTF8	園
U+5712.0	fCNS11643	1-6350
U+5712.0	fBig5	B6E9
U+5712.0	fGB	1-5430
U+5712.0	fR/S	 31.10
U+5712.0	fTotalStrokes	 13
U+5712.0	fCangjie	WGRV
U+5712.0	fFourCorner	60232
U+5712.0	fHakka	 jen2; jan3; ren2; jan2
U+5712.0	fCantonese	jyun4; jyun2 (also)
U+5712.0	fMandarin	yuan2
U+5712.0	fEnglish	[1] a piece of ground used for growing flowers, fruit, or vegetables; garden; orchard; plantation [2] public garden; park; recreation ground
U+5713.0	fUTF8	圓
U+5713.0	fCNS11643	1-6351
U+5713.0	fBig5	B6EA
U+5713.0	fGB	1-5432
U+5713.0	fR/S	 31.10
U+5713.0	fTotalStrokes	 13
U+5713.0	fCangjie	WRBC
U+5713.0	fFourCorner	60806
U+5713.0	fHakka	 jen2; jan3; ren2; jan2
U+5713.0	fCantonese	jyun4; jyun2 (also)
U+5713.0	fMandarin	yuan2; yun4
U+5713.0	fEnglish	[1] round; circular; spherical [2] complete; complete; make plausible; justify [3] satisfactory; tactful [4] monetary unit [5] circle
U+5714.0	fUTF8	圔
U+5714.0	fCNS11643	2-4228
U+5714.0	fBig5	DCE1
U+5714.0	fGB	3-375A
U+5714.0	fR/S	 31.10
U+5714.0	fTotalStrokes	 13
U+5714.0	fCangjie	WGIT
U+5714.0	fFourCorner	60107
U+5714.0	fHakka	 rip7; jip7; jap7
U+5714.0	fCantonese	aap3; ngaap3 (also); jap1; jip3
U+5714.0	fMandarin	e4; ya4
U+5714.0	fEnglish	the shrill note of a flute
U+5715.0	fUTF8	圕
U+5715.0	fCNS11643	3-4052
U+5715.0	fGB	E-2546
U+5715.0	fR/S	 31.10
U+5715.0	fTotalStrokes	 13
U+5715.0	fCangjie	WLGA
U+5715.0	fFourCorner	60600
U+5715.0	fCantonese	syu1
U+5715.0	fMandarin	tu2shu1guan3
U+5715.0	fEnglish	 library
U+5716.0	fUTF8	圖
U+5716.0	fCNS11643	1-6837
U+5716.0	fBig5	B9CF
U+5716.0	fGB	1-4D3C
U+5716.0	fR/S	 31.11
U+5716.0	fTotalStrokes	 14
U+5716.0	fCangjie	WRYW
U+5716.0	fFourCorner	60600
U+5716.0	fHakka	  tu2; tu3
U+5716.0	fCantonese	tou4
U+5716.0	fMandarin	tu2; tu1
U+5716.0	fEnglish	[1] diagram; chart; map; picture; portrait [2] seek; pursue [3] plan; scheme; conspire [4] intention; aim; purpose
U+5717.0	fUTF8	圗
U+5717.0	fCNS11643	3-4637
U+5717.0	fGB	E-2547
U+5717.0	fR/S	 31.11
U+5717.0	fTotalStrokes	 14
U+5717.0	fCangjie	WIJW
U+5717.0	fFourCorner	60600
U+5717.0	fCantonese	tou4
U+5717.0	fMandarin	tu2
U+5717.0	fEnglish	 diagram chart; map; picture
U+5718.0	fUTF8	團
U+5718.0	fCNS11643	1-6836
U+5718.0	fBig5	B9CE
U+5718.0	fGB	1-4D45
U+5718.0	fR/S	 31.11
U+5718.0	fTotalStrokes	 14
U+5718.0	fCangjie	WJII
U+5718.0	fFourCorner	60343
U+5718.0	fHakka	  ton2; ton3
U+5718.0	fCantonese	tyun4; tyun2 (also)
U+5718.0	fMandarin	tuan2; shuan4; tuan1
U+5718.0	fEnglish	[1] sphere; ball; something shaped like a ball [2] mass; lump [3] group; party; mission; organization; society [4] regiment, consisting of three battalions of foot soldiers
U+5719.0	fUTF8	圙
U+5719.0	fGB	3-375C
U+5719.0	fR/S	 31.11
U+5719.0	fTotalStrokes	 14
U+5719.0	fCangjie	WCMW
U+5719.0	fCantonese	loek6
U+5719.0	fMandarin	luue4; le4
U+571A.0	fUTF8	圚
U+571A.0	fCNS11643	2-4F70
U+571A.0	fBig5	E549
U+571A.0	fGB	5-397C
U+571A.0	fR/S	 31.12
U+571A.0	fTotalStrokes	 15
U+571A.0	fCangjie	WLMC
U+571A.0	fFourCorner	60806
U+571A.0	fCantonese	kui2; kwui2 (also); wui6
U+571A.0	fMandarin	hui4
U+571B.0	fUTF8	圛
U+571B.0	fCNS11643	2-5652
U+571B.0	fBig5	E948
U+571B.0	fGB	3-3755
U+571B.0	fR/S	 31.13
U+571B.0	fTotalStrokes	 16
U+571B.0	fCangjie	WWLJ
U+571B.0	fFourCorner	60401
U+571B.0	fHakka	 jit8; rit8
U+571B.0	fCantonese	jik6
U+571B.0	fMandarin	yi4
U+571B.0	fEnglish	 used in place names stratus
U+571C.0	fUTF8	圜
U+571C.0	fCNS11643	2-5651
U+571C.0	fBig5	E947
U+571C.0	fGB	0-6077
U+571C.0	fR/S	 31.13
U+571C.0	fTotalStrokes	 16
U+571C.0	fCangjie	WWLV
U+571C.0	fFourCorner	60732
U+571C.0	fHakka	  fan2;  jen2; fan3; ren2; wan2; jan2
U+571C.0	fCantonese	jyun4; waan4
U+571C.0	fMandarin	yuan2; huan2
U+571C.0	fEnglish	 circle; surround encircle
U+571D.0	fUTF8	圝
U+571D.0	fCNS11643	3-5F41
U+571D.0	fGB	E-2548
U+571D.0	fR/S	 31.19
U+571D.0	fTotalStrokes	 22; 21
U+571D.0	fCangjie	WVFF
U+571D.0	fFourCorner	60990
U+571D.0	fHakka	 lan2; lien5
U+571D.0	fCantonese	lyun4
U+571D.0	fMandarin	luan2
U+571D.0	fEnglish	entire
U+571E.0	fUTF8	圞
U+571E.0	fCNS11643	2-715A
U+571E.0	fBig5	F96B
U+571E.0	fGB	3-375B
U+571E.0	fR/S	 31.23
U+571E.0	fTotalStrokes	 26
U+571E.0	fCangjie	WVFD
U+571E.0	fFourCorner	60904
U+571E.0	fHakka	 lan2; lien5
U+571E.0	fCantonese	lyun4
U+571E.0	fMandarin	luan2
U+571E.0	fEnglish	entire
U+571F.0	fUTF8	土
U+571F.0	fCNS11643	1-4448
U+571F.0	fBig5	A467
U+571F.0	fGB	0-4D41
U+571F.0	fR/S	 32.0
U+571F.0	fTotalStrokes	 3
U+571F.0	fCangjie	G
U+571F.0	fFourCorner	40100
U+571F.0	fHakka	  tu3; tu2
U+571F.0	fCantonese	tou2; dou6
U+571F.0	fMandarin	tu3; du4
U+571F.0	fEnglish	[1] soil; earth; [2] land; territory; domain [3] local; native; indigenous [4] unrefined; unenlightened [5] rustic; countrified [6] opium [7] an abbreviation for Turkey
U+5720.0	fUTF8	圠
U+5720.0	fCNS11643	2-2139
U+5720.0	fBig5	C959
U+5720.0	fGB	5-3554
U+5720.0	fR/S	 32.1
U+5720.0	fTotalStrokes	 4
U+5720.0	fCangjie	GU
U+5720.0	fFourCorner	42110
U+5720.0	fHakka	 jet7; zat7; jat7
U+5720.0	fCantonese	caak3 (also); aat3; ngaat3 (also); jat1
U+5720.0	fMandarin	ya4
U+5720.0	fEnglish	fine dust
U+5721.0	fUTF8	圡
U+5721.0	fCNS11643	3-2161
U+5721.0	fGB	5-3553
U+5721.0	fR/S	  32.1
U+5721.0	fTotalStrokes	 4
U+5721.0	fCangjie	GI
U+5721.0	fFourCorner	40100
U+5721.0	fHakka	 tu3
U+5721.0	fCantonese	tou2; dou6
U+5721.0	fMandarin	tu3; du4
U+5722.0	fUTF8	圢
U+5722.0	fCNS11643	2-214F
U+5722.0	fBig5	C96E
U+5722.0	fGB	5-3555
U+5722.0	fR/S	 32.2
U+5722.0	fTotalStrokes	 5
U+5722.0	fCangjie	GMN
U+5722.0	fFourCorner	41120
U+5722.0	fHakka	  tin3;  dien3; tang2
U+5722.0	fCantonese	ting5; ding1
U+5722.0	fMandarin	ting3; ting1; tian3
U+5722.0	fEnglish	 elevated surface; elevated area; platform
U+5723.0	fUTF8	圣
U+5723.0	fCNS11643	2-2150
U+5723.0	fBig5	C96F
U+5723.0	fGB	0-4A25
U+5723.0	fR/S	 32.2
U+5723.0	fTotalStrokes	 5
U+5723.0	fCangjie	EG
U+5723.0	fFourCorner	17104
U+5723.0	fHakka	 gwai5;  shin5; kut7; siin5
U+5723.0	fCantonese	sing3
U+5723.0	fMandarin	sheng4; ku1
U+5723.0	fEnglish	simplified form of &#x8056;
U+5724.0	fUTF8	圤
U+5724.0	fCNS11643	3-2234
U+5724.0	fGB	5-3556
U+5724.0	fR/S	  32.2
U+5724.0	fTotalStrokes	 5
U+5724.0	fCangjie	GY
U+5724.0	fFourCorner	43100
U+5724.0	fHakka	 pok7
U+5724.0	fCantonese	pok3
U+5724.0	fMandarin	pu3; pu2
U+5725.0	fUTF8	圥
U+5725.0	fCNS11643	4-2170
U+5725.0	fGB	E-2549
U+5725.0	fR/S	  32.2
U+5725.0	fTotalStrokes	 5
U+5725.0	fCangjie	GHU
U+5725.0	fFourCorner	40210
U+5725.0	fCantonese	luk6
U+5725.0	fMandarin	lu4
U+5725.0	fEnglish	mushroom
U+5726.0	fUTF8	圦
U+5726.0	fGB	E-254A
U+5726.0	fR/S	 32.2
U+5726.0	fTotalStrokes	 5
U+5726.0	fCangjie	GO
U+5726.0	fCantonese	faai3
U+5726.0	fEnglish	 (kokuji) water gate, spout
U+5727.0	fUTF8	圧
U+5727.0	fCNS11643	4-2171
U+5727.0	fGB	E-254B
U+5727.0	fR/S	 32.2
U+5727.0	fTotalStrokes	 5
U+5727.0	fCangjie	MG
U+5727.0	fFourCorner	71210
U+5727.0	fCantonese	zong1
U+5727.0	fMandarin	ya1; ya4
U+5727.0	fEnglish	same as &#x58D3;
U+5728.0	fUTF8	在
U+5728.0	fCNS11643	1-4763
U+5728.0	fBig5	A662
U+5728.0	fGB	0-545A
U+5728.0	fR/S	 32.3
U+5728.0	fTotalStrokes	 6
U+5728.0	fCangjie	KLG
U+5728.0	fFourCorner	40214
U+5728.0	fHakka	  coi1;  cai5; cai3; cai6; cai1; coi5; hoi1; toi1
U+5728.0	fCantonese	zoi6; zoi2 (also)
U+5728.0	fMandarin	zai4; zai3
U+5728.0	fEnglish	[1] at; in; on; up to [2] rest with; consist in; depend on [3] be alive; living; be present; exist [4] used to indicate a progressive tense
U+5729.0	fUTF8	圩
U+5729.0	fCNS11643	1-4767
U+5729.0	fBig5	A666
U+5729.0	fGB	0-5B57
U+5729.0	fR/S	 32.3
U+5729.0	fTotalStrokes	 6
U+5729.0	fCangjie	GMD
U+5729.0	fFourCorner	41140
U+5729.0	fHakka	 hi1; ji2; ri2; i2
U+5729.0	fCantonese	wai4; jyu4 (also); heoi1
U+5729.0	fMandarin	wei2; xu1; yu2
U+5729.0	fEnglish	dike; embankment
U+572A.0	fUTF8	圪
U+572A.0	fCNS11643	2-2229
U+572A.0	fBig5	C9C9
U+572A.0	fGB	0-5B59
U+572A.0	fR/S	 32.3
U+572A.0	fTotalStrokes	 6
U+572A.0	fCangjie	GON
U+572A.0	fFourCorner	48117
U+572A.0	fCantonese	ngat6; at6 (also); gat3 (also); gat6; gaak3
U+572A.0	fMandarin	ge1; yi1
U+572B.0	fUTF8	圫
U+572B.0	fCNS11643	4-2251
U+572B.0	fGB	5-355A
U+572B.0	fR/S	 32.3
U+572B.0	fTotalStrokes	 6
U+572B.0	fCangjie	GHP
U+572B.0	fFourCorner	42110
U+572B.0	fHakka	 au5; juk7; ruk7
U+572B.0	fCantonese	ou3; ngou3 (also)
U+572B.0	fMandarin	ao4; yu4
U+572B.0	fEnglish	ancient form of &#x58BA;
U+572C.0	fUTF8	圬
U+572C.0	fCNS11643	1-4765
U+572C.0	fBig5	A664
U+572C.0	fGB	0-5B58
U+572C.0	fR/S	 32.3
U+572C.0	fTotalStrokes	 6
U+572C.0	fCangjie	GMMS
U+572C.0	fFourCorner	41127
U+572C.0	fHakka	  wu1; vu1
U+572C.0	fCantonese	wu1
U+572C.0	fMandarin	wu1
U+572C.0	fEnglish	[1] a trowel used for plastering [2] plaster
U+572D.0	fUTF8	圭
U+572D.0	fCNS11643	1-4764
U+572D.0	fBig5	A663
U+572D.0	fGB	0-3967
U+572D.0	fR/S	 32.3
U+572D.0	fTotalStrokes	 6
U+572D.0	fCangjie	GG
U+572D.0	fFourCorner	40104
U+572D.0	fHakka	 gui1; gwi1; kwui1
U+572D.0	fCantonese	gwai1
U+572D.0	fMandarin	gui1
U+572D.0	fEnglish	a jade tablet with a square base and a pointed top used in official ceremonies in ancient China
U+572E.0	fUTF8	圮
U+572E.0	fCNS11643	2-2228
U+572E.0	fBig5	C9C8
U+572E.0	fGB	0-5B5C
U+572E.0	fR/S	 32.3
U+572E.0	fTotalStrokes	 6
U+572E.0	fCangjie	GSU
U+572E.0	fFourCorner	47117
U+572E.0	fHakka	  pi3; gi3
U+572E.0	fCantonese	pei2; pei5
U+572E.0	fMandarin	pi3
U+572E.0	fEnglish	 destroyed; ruined subvert; injure
U+572F.0	fUTF8	圯
U+572F.0	fCNS11643	1-4766
U+572F.0	fBig5	A665
U+572F.0	fGB	0-5B5D
U+572F.0	fR/S	 32.3
U+572F.0	fTotalStrokes	 6
U+572F.0	fCangjie	GRU
U+572F.0	fFourCorner	47117
U+572F.0	fHakka	 ji2; ri2
U+572F.0	fCantonese	ji4; zi4
U+572F.0	fMandarin	yi2
U+572F.0	fEnglish	 bridge; bank
U+5730.0	fUTF8	地
U+5730.0	fCNS11643	1-4762
U+5730.0	fBig5	A661
U+5730.0	fGB	0-3558
U+5730.0	fR/S	 32.3
U+5730.0	fTotalStrokes	 6
U+5730.0	fCangjie	GPD
U+5730.0	fFourCorner	44112
U+5730.0	fHakka	 ti5; ti3; ti6; ti2
U+5730.0	fCantonese	dei6; deng6 (also); dei2 (also)
U+5730.0	fMandarin	di4; de5; de1
U+5730.0	fEnglish	[1] earth [2] land; soil; ground [3] region; territory; belt; place; locality [4] position; place; situation [5] an adjunct after a word (usually adjective) to form an adverbial phrase
U+5730.1	fUTF8	地
U+5730.1	fCNS11643	1-4762
U+5730.1	fBig5	A661
U+5730.1	fGB	0-3558
U+5730.1	fR/S	 32.3
U+5730.1	fTotalStrokes	 6
U+5730.1	fCangjie	GPD
U+5730.1	fFourCorner	44112
U+5731.0	fUTF8	圱
U+5731.0	fCNS11643	3-2341
U+5731.0	fGB	5-3559
U+5731.0	fR/S	 32.3
U+5731.0	fTotalStrokes	 6
U+5731.0	fCangjie	HJG
U+5731.0	fFourCorner	24410
U+5731.0	fCantonese	cin1
U+5731.0	fMandarin	qian1
U+5732.0	fUTF8	圲
U+5732.0	fCNS11643	3-2342
U+5732.0	fGB	3-3361
U+5732.0	fR/S	 32.3
U+5732.0	fTotalStrokes	 6
U+5732.0	fCangjie	GHJ
U+5732.0	fFourCorner	42140
U+5732.0	fCantonese	cin1
U+5732.0	fMandarin	qian1
U+5732.0	fEnglish	same as &#x5731;
U+5733.0	fUTF8	圳
U+5733.0	fCNS11643	1-4761
U+5733.0	fBig5	A660
U+5733.0	fGB	0-5B5A
U+5733.0	fR/S	 32.3
U+5733.0	fTotalStrokes	 6
U+5733.0	fCangjie	GLLL
U+5733.0	fFourCorner	42100
U+5733.0	fHakka	 zun5;  zhun5
U+5733.0	fCantonese	zan3; hyun2
U+5733.0	fMandarin	zhen4; zun4; chou2; quan3
U+5733.0	fEnglish	furrow in field; small drainage
U+5734.0	fUTF8	圴
U+5734.0	fCNS11643	2-222A
U+5734.0	fBig5	C9CA
U+5734.0	fGB	3-3362
U+5734.0	fR/S	 32.3
U+5734.0	fTotalStrokes	 6
U+5734.0	fCangjie	GPI
U+5734.0	fFourCorner	47120
U+5734.0	fCantonese	zoek3
U+5734.0	fMandarin	zhuo2; shao2; shuo4
U+5735.0	fUTF8	圵
U+5735.0	fCNS11643	3-233F
U+5735.0	fGB	E-254C
U+5735.0	fR/S	  32.3
U+5735.0	fTotalStrokes	 6
U+5735.0	fCangjie	GYM
U+5735.0	fFourCorner	41110
U+5735.0	fCantonese	joeng6
U+5735.0	fMandarin	dang4
U+5736.0	fUTF8	圶
U+5736.0	fCNS11643	3-2340
U+5736.0	fGB	5-372E
U+5736.0	fR/S	 32.3
U+5736.0	fTotalStrokes	 6
U+5736.0	fCangjie	KG
U+5736.0	fFourCorner	40100
U+5736.0	fCantonese	haap3
U+5736.0	fMandarin	qia4
U+5737.0	fUTF8	圷
U+5737.0	fGB	E-254D
U+5737.0	fR/S	 32.3
U+5737.0	fTotalStrokes	 6
U+5737.0	fCangjie	GMY
U+5737.0	fCantonese	haa6
U+5738.0	fUTF8	圸
U+5738.0	fGB	E-254E
U+5738.0	fR/S	 32.3
U+5738.0	fTotalStrokes	 6
U+5738.0	fCangjie	XGU
U+5738.0	fFourCorner	4217
U+5738.0	fCantonese	saan1
U+5739.0	fUTF8	圹
U+5739.0	fGB	0-5B5B
U+5739.0	fR/S	 32.3
U+5739.0	fTotalStrokes	 6
U+5739.0	fCangjie	XGI
U+5739.0	fHakka	 k'wong3
U+5739.0	fCantonese	kwong3; kong3 (also)
U+5739.0	fMandarin	kuang4
U+5739.0	fEnglish	simplified form of &#x58D9;
U+573A.0	fUTF8	场
U+573A.0	fGB	0-3321
U+573A.0	fR/S	 32.3
U+573A.0	fTotalStrokes	 6
U+573A.0	fCangjie	GNSH
U+573A.0	fHakka	 chong2
U+573A.0	fCantonese	coeng4; coeng2 (also)
U+573A.0	fMandarin	chang2; chang3
U+573A.0	fEnglish	simplified form of &#x5834;
U+573A.1	fUTF8	场
U+573A.1	fGB	0-3321
U+573A.1	fR/S	 32.3
U+573A.1	fTotalStrokes	 6
U+573A.1	fCangjie	GNSH
U+573B.0	fUTF8	圻
U+573B.0	fCNS11643	1-4966
U+573B.0	fBig5	A7A6
U+573B.0	fGB	0-5B5F
U+573B.0	fR/S	 32.4
U+573B.0	fTotalStrokes	 7
U+573B.0	fCangjie	GHML
U+573B.0	fFourCorner	42121
U+573B.0	fHakka	 ki2; ngiun2
U+573B.0	fCantonese	kei4; ngan4; an4 (also)
U+573B.0	fMandarin	qi2; yin2
U+573B.0	fEnglish	border; boundary
U+573C.0	fUTF8	圼
U+573C.0	fCNS11643	3-2531
U+573C.0	fGB	5-494C
U+573C.0	fR/S	  32.4
U+573C.0	fTotalStrokes	 7
U+573C.0	fCangjie	AG
U+573C.0	fFourCorner	60100
U+573C.0	fHakka	  ngiap7
U+573C.0	fCantonese	nit6; lit6 (also)
U+573C.0	fMandarin	nie4
U+573C.0	fEnglish	[1] fill up (hole) [2] same as &#x363F;
U+573D.0	fUTF8	圽
U+573D.0	fCNS11643	3-253E
U+573D.0	fGB	5-3562
U+573D.0	fR/S	 32.4
U+573D.0	fTotalStrokes	 7
U+573D.0	fCangjie	GPHH
U+573D.0	fFourCorner	47120
U+573D.0	fHakka	 mut8
U+573D.0	fCantonese	mut6
U+573D.0	fMandarin	mo4
U+573E.0	fUTF8	圾
U+573E.0	fCNS11643	1-4963
U+573E.0	fBig5	A7A3
U+573E.0	fGB	0-3B78
U+573E.0	fR/S	 32.4
U+573E.0	fTotalStrokes	 7
U+573E.0	fCangjie	GNHE
U+573E.0	fFourCorner	47147
U+573E.0	fHakka	 sap7
U+573E.0	fCantonese	saap3; kap1; ngap6
U+573E.0	fMandarin	ji2; ji1; jie2
U+573E.0	fEnglish	garbage; rubbish; refuse; waste
U+573F.0	fUTF8	圿
U+573F.0	fCNS11643	3-2530
U+573F.0	fGB	5-3560
U+573F.0	fR/S	  32.4
U+573F.0	fTotalStrokes	 7
U+573F.0	fCangjie	GOLL
U+573F.0	fFourCorner	48120
U+573F.0	fHakka	  kat7
U+573F.0	fCantonese	git3
U+573F.0	fMandarin	jia2
U+573F.0	fEnglish	rubbish, refuse
U+5740.0	fUTF8	址
U+5740.0	fCNS11643	1-495F
U+5740.0	fBig5	A77D
U+5740.0	fGB	0-5637
U+5740.0	fR/S	 32.4
U+5740.0	fTotalStrokes	 7
U+5740.0	fCangjie	GYLM
U+5740.0	fFourCorner	41110
U+5740.0	fHakka	 zi3; zhi3; zih3; tje3
U+5740.0	fCantonese	zi2
U+5740.0	fMandarin	zhi3
U+5740.0	fEnglish	[1] site; location; land on which to build a house [2] foundation
U+5741.0	fUTF8	坁
U+5741.0	fCNS11643	2-234A
U+5741.0	fBig5	CAAA
U+5741.0	fGB	5-3561
U+5741.0	fR/S	 32.4
U+5741.0	fTotalStrokes	 7
U+5741.0	fCangjie	GHVP
U+5741.0	fFourCorner	42140
U+5741.0	fCantonese	zi2
U+5741.0	fMandarin	zhi3
U+5742.0	fUTF8	坂
U+5742.0	fCNS11643	3-2540
U+5742.0	fGB	0-5B60
U+5742.0	fR/S	  32.4
U+5742.0	fTotalStrokes	 7
U+5742.0	fCangjie	GHE
U+5742.0	fFourCorner	41140
U+5742.0	fHakka	  fan3; ban3
U+5742.0	fCantonese	baan2; faan2
U+5742.0	fMandarin	ban3
U+5742.0	fEnglish	 slope; hillside
U+5743.0	fUTF8	坃
U+5743.0	fCNS11643	3-2536
U+5743.0	fGB	5-355C
U+5743.0	fR/S	 32.4
U+5743.0	fTotalStrokes	 7
U+5743.0	fCangjie	GMMU
U+5743.0	fFourCorner	41110
U+5743.0	fHakka	  hiun1
U+5743.0	fCantonese	hyun1; fan1 (also)
U+5743.0	fMandarin	xun1; xuan1; xun4
U+5743.0	fEnglish	ancient form of &#x58CE;
U+5744.0	fUTF8	坄
U+5744.0	fCNS11643	3-252F
U+5744.0	fGB	3-336E
U+5744.0	fR/S	  32.4
U+5744.0	fTotalStrokes	 7
U+5744.0	fCangjie	GHNE
U+5744.0	fFourCorner	47140
U+5744.0	fHakka	 jit8; rit8
U+5744.0	fCantonese	jik6
U+5744.0	fMandarin	tou2
U+5744.0	fEnglish	the door of a kiln or furnace
U+5745.0	fUTF8	坅
U+5745.0	fCNS11643	2-234B
U+5745.0	fBig5	CAAB
U+5745.0	fGB	3-336C
U+5745.0	fR/S	 32.4
U+5745.0	fTotalStrokes	 7
U+5745.0	fCangjie	GOIN
U+5745.0	fFourCorner	48127
U+5745.0	fHakka	 kim3
U+5745.0	fCantonese	jam2; gam2
U+5745.0	fMandarin	qin3; qiu3
U+5745.0	fEnglish	pit
U+5746.0	fUTF8	坆
U+5746.0	fCNS11643	3-253C
U+5746.0	fGB	5-355F
U+5746.0	fR/S	 32.4
U+5746.0	fTotalStrokes	 7
U+5746.0	fCangjie	GOK
U+5746.0	fFourCorner	48140
U+5746.0	fHakka	 fun1; pun1
U+5746.0	fCantonese	mui4; fan5; fan4
U+5746.0	fMandarin	fen2
U+5746.0	fEnglish	a grave, tomb
U+5747.0	fUTF8	均
U+5747.0	fCNS11643	1-4961
U+5747.0	fBig5	A7A1
U+5747.0	fGB	0-3E79
U+5747.0	fR/S	 32.4
U+5747.0	fTotalStrokes	 7
U+5747.0	fCangjie	GPIM
U+5747.0	fFourCorner	47120
U+5747.0	fHakka	  giun1
U+5747.0	fCantonese	gwan1; kwan1 (also); wan6; wan5 (also)
U+5747.0	fMandarin	jun1; yun4
U+5747.0	fEnglish	[1] equal; equally; even; level [2] be fair [3] all; also; too [4] potter's wheel [5] an ancient musical instrument
U+5748.0	fUTF8	坈
U+5748.0	fCNS11643	3-252D
U+5748.0	fGB	3-336F
U+5748.0	fR/S	  32.4
U+5748.0	fTotalStrokes	 7
U+5748.0	fCangjie	GBHU
U+5748.0	fFourCorner	47110
U+5748.0	fCantonese	haang1
U+5748.0	fMandarin	rong3; keng1
U+5749.0	fUTF8	坉
U+5749.0	fCNS11643	2-234D
U+5749.0	fBig5	CAAD
U+5749.0	fGB	3-3366
U+5749.0	fR/S	 32.4
U+5749.0	fTotalStrokes	 7
U+5749.0	fCangjie	GPU
U+5749.0	fFourCorner	45117
U+5749.0	fHakka	  tun2; tun3; dun3
U+5749.0	fCantonese	tyun4; nang4; lang4 (also)
U+5749.0	fMandarin	tun2; dun4; dun2
U+5749.0	fEnglish	 marsh
U+574A.0	fUTF8	坊
U+574A.0	fCNS11643	1-495D
U+574A.0	fBig5	A77B
U+574A.0	fGB	0-373B
U+574A.0	fR/S	 32.4
U+574A.0	fTotalStrokes	 7
U+574A.0	fCangjie	GYHS
U+574A.0	fFourCorner	40127
U+574A.0	fHakka	  fong1; fong2; fong3
U+574A.0	fCantonese	fong1; fong4; fong2
U+574A.0	fMandarin	fang2; fang1
U+574A.0	fEnglish	[1] community; neighbourhood; urban subdivision; subdivision of a city; city quarter; street; lane [2] workshop of a trade; mill [3] an arch-like memorial building
U+574B.0	fUTF8	坋
U+574B.0	fCNS11643	2-234E
U+574B.0	fBig5	CAAE
U+574B.0	fGB	3-336D
U+574B.0	fR/S	 32.4
U+574B.0	fTotalStrokes	 7
U+574B.0	fCangjie	GCSH
U+574B.0	fFourCorner	48127
U+574B.0	fHakka	 fun3; fun5; pun1
U+574B.0	fCantonese	fan5; fan6; ban6; ban3
U+574B.0	fMandarin	fen4
U+574B.0	fEnglish	 dust; earth a bank of earth dig bring to-gether
U+574C.0	fUTF8	坌
U+574C.0	fCNS11643	2-234C
U+574C.0	fBig5	CAAC
U+574C.0	fGB	0-5B50
U+574C.0	fR/S	 32.4
U+574C.0	fTotalStrokes	 7
U+574C.0	fCangjie	CSHG
U+574C.0	fFourCorner	80104
U+574C.0	fHakka	 fun3
U+574C.0	fCantonese	ban6; ban3
U+574C.0	fMandarin	ben4; ben3; pen4
U+574C.0	fEnglish	 dust;earth a bank of earth dig bring together
U+574C.1	fUTF8	坌
U+574C.1	fCNS11643	2-234C
U+574C.1	fBig5	CAAC
U+574C.1	fGB	0-5B50
U+574C.1	fR/S	 32.4
U+574C.1	fTotalStrokes	 7
U+574C.1	fCangjie	CSHG
U+574C.1	fFourCorner	80104
U+574C.1	fMandarin	beng4
U+574C.1	fEnglish	
U+574D.0	fUTF8	坍
U+574D.0	fCNS11643	1-4960
U+574D.0	fBig5	A77E
U+574D.0	fGB	0-4C2E
U+574D.0	fR/S	 32.4
U+574D.0	fTotalStrokes	 7
U+574D.0	fCangjie	GBY
U+574D.0	fFourCorner	47140
U+574D.0	fHakka	 tam1;  tan1; tak7
U+574D.0	fCantonese	taan1
U+574D.0	fMandarin	tan1
U+574D.0	fEnglish	[1] sliding of earth; to collapse; landslide; fall into ruins; tumble [2] collapse [3] loose face
U+574E.0	fUTF8	坎
U+574E.0	fCNS11643	1-4962
U+574E.0	fBig5	A7A2
U+574E.0	fGB	0-3F32
U+574E.0	fR/S	 32.4
U+574E.0	fTotalStrokes	 7
U+574E.0	fCangjie	GNO
U+574E.0	fFourCorner	47182
U+574E.0	fHakka	  kam3;  kam5; kam1
U+574E.0	fCantonese	ham2; ham1 (also); ham3
U+574E.0	fMandarin	kan3
U+574E.0	fEnglish	[1] pit; hole; depression [2] one of the Eight diagrams in the Book of Changes snare [3] the sound of percussion [4] snare; danger; crisis
U+574F.0	fUTF8	坏
U+574F.0	fCNS11643	1-4965
U+574F.0	fBig5	A7A5
U+574F.0	fGB	0-3B35
U+574F.0	fR/S	 32.4
U+574F.0	fTotalStrokes	 7
U+574F.0	fCangjie	GMF
U+574F.0	fFourCorner	41190
U+574F.0	fHakka	  poi1; pi1; pui1; fai5
U+574F.0	fCantonese	waai6; waai2 (also); pui1; pui4
U+574F.0	fMandarin	huai4; pi1; guai4; pei2
U+574F.0	fEnglish	simplified form of &#x58DE;
U+5750.0	fUTF8	坐
U+5750.0	fCNS11643	1-4964
U+5750.0	fBig5	A7A4
U+5750.0	fGB	0-5778
U+5750.0	fR/S	 32.4
U+5750.0	fTotalStrokes	 7
U+5750.0	fCangjie	OOG
U+5750.0	fFourCorner	88104
U+5750.0	fHakka	  co1;  co3;  co5; co6
U+5750.0	fCantonese	zo6; co5 (also)
U+5750.0	fMandarin	zuo4
U+5750.0	fEnglish	[1] sit; a seat [2] ride (on a bus, train, etc.); travel by [3] kneel [4] reach [5] have its back towards (said of a building) [6] get (profit, etc.) without work
U+5751.0	fUTF8	坑
U+5751.0	fCNS11643	1-495E
U+5751.0	fBig5	A77C
U+5751.0	fGB	0-3F53
U+5751.0	fR/S	 32.4
U+5751.0	fTotalStrokes	 7
U+5751.0	fCangjie	GYHN
U+5751.0	fFourCorner	40117
U+5751.0	fHakka	  hang1;  kang1; gang1
U+5751.0	fCantonese	haang1
U+5751.0	fMandarin	keng1
U+5751.0	fEnglish	[1] pit; a hole in the ground [2] bury alive [3] trap; entrap
U+5752.0	fUTF8	坒
U+5752.0	fCNS11643	2-234F
U+5752.0	fBig5	CAAF
U+5752.0	fGB	5-4841
U+5752.0	fR/S	 32.4
U+5752.0	fTotalStrokes	 7
U+5752.0	fCangjie	PPG
U+5752.0	fFourCorner	21104
U+5752.0	fHakka	 pi5; pi6; pi3
U+5752.0	fCantonese	bai6 (also); bei6; bei2
U+5752.0	fMandarin	bi4
U+5752.0	fEnglish	 compare match equal
U+5753.0	fUTF8	坓
U+5753.0	fCNS11643	3-2532
U+5753.0	fGB	E-254F
U+5753.0	fR/S	 32.4
U+5753.0	fTotalStrokes	 7
U+5753.0	fCangjie	TTG
U+5753.0	fFourCorner	55100
U+5753.0	fHakka	  hin2
U+5753.0	fCantonese	jing4
U+5753.0	fMandarin	xing2; jia2
U+5753.0	fEnglish	ancient form of &#x578B;
U+5754.0	fUTF8	坔
U+5754.0	fCNS11643	3-2541
U+5754.0	fGB	5-5462
U+5754.0	fR/S	  32.4
U+5754.0	fTotalStrokes	 7
U+5754.0	fCangjie	EG
U+5754.0	fFourCorner	12100
U+5754.0	fHakka	 ti5; ti3; ti6; ti2
U+5754.0	fCantonese	dei6; deng6 (also); dei2 (also)
U+5754.0	fMandarin	di4; de5; de1
U+5754.0	fEnglish	same as &#x5730;
U+5755.0	fUTF8	坕
U+5755.0	fCNS11643	3-2542
U+5755.0	fGB	E-2550
U+5755.0	fR/S	 32.4
U+5755.0	fTotalStrokes	 7
U+5755.0	fCangjie	HOG
U+5755.0	fFourCorner	72100
U+5755.0	fCantonese	ging1
U+5755.0	fMandarin	jing1
U+5756.0	fUTF8	坖
U+5756.0	fCNS11643	3-2533
U+5756.0	fGB	E-2551
U+5756.0	fR/S	  32.4
U+5756.0	fTotalStrokes	 7
U+5756.0	fCangjie	MMUG
U+5756.0	fFourCorner	10100
U+5756.0	fHakka	  ki3; ki6; ki5; gi5
U+5756.0	fCantonese	gei6; gei2 (also)
U+5756.0	fMandarin	ji4
U+5756.0	fEnglish	ancient form of &#x5FCC;
U+5757.0	fUTF8	块
U+5757.0	fCNS11643	3-2538
U+5757.0	fGB	0-3F69
U+5757.0	fR/S	  32.4
U+5757.0	fTotalStrokes	 7
U+5757.0	fCangjie	GDK
U+5757.0	fFourCorner	45130
U+5757.0	fHakka	 kwai5; te5; kai5
U+5757.0	fCantonese	faai3
U+5757.0	fMandarin	kuai4
U+5757.0	fEnglish	simplified form of &#x584A;
U+5758.0	fUTF8	坘
U+5758.0	fCNS11643	3-2537
U+5758.0	fGB	E-2552
U+5758.0	fR/S	 32.4
U+5758.0	fTotalStrokes	 7
U+5758.0	fCangjie	GMVM
U+5758.0	fFourCorner	41110
U+5758.0	fHakka	 chi2; cih2; zhi3; zih3; chii2
U+5758.0	fCantonese	ci4; dai2; zi2
U+5758.0	fMandarin	di3; chi2
U+5758.0	fEnglish	same as &#x577B;
U+5759.0	fUTF8	坙
U+5759.0	fCNS11643	3-645F
U+5759.0	fGB	E-2553
U+5759.0	fR/S	 32.4
U+5759.0	fTotalStrokes	 7
U+5759.0	fCangjie	MVVG
U+5759.0	fFourCorner	1010
U+5759.0	fHakka	 gin1
U+5759.0	fCantonese	ging1
U+5759.0	fMandarin	jing1
U+5759.0	fEnglish	corrupted form of &#x5DE0;
U+575A.0	fUTF8	坚
U+575A.0	fGB	0-3C61
U+575A.0	fR/S	 32.4
U+575A.0	fTotalStrokes	 7
U+575A.0	fCangjie	LEG
U+575A.0	fHakka	 gen1; gien1
U+575A.0	fCantonese	gin1
U+575A.0	fMandarin	jian1
U+575A.0	fEnglish	simplified form of &#x5805;
U+575B.0	fUTF8	坛
U+575B.0	fCNS11643	F-2352
U+575B.0	fGB	0-4C33
U+575B.0	fR/S	 32.4
U+575B.0	fTotalStrokes	 7
U+575B.0	fCangjie	GMMI
U+575B.0	fFourCorner	4113
U+575B.0	fHakka	 tan2
U+575B.0	fCantonese	taan4; taam4
U+575B.0	fMandarin	tan2; dan4; shan4
U+575B.0	fEnglish	simplified form of &#x58C7;
U+575C.0	fUTF8	坜
U+575C.0	fGB	0-5B5E
U+575C.0	fR/S	 32.4
U+575C.0	fTotalStrokes	 7
U+575C.0	fCangjie	GMKS
U+575C.0	fHakka	 lak7; lak8
U+575C.0	fCantonese	lek6; lik6
U+575C.0	fMandarin	li4
U+575C.0	fEnglish	simplified form of &#x58E2;
U+575D.0	fUTF8	坝
U+575D.0	fGB	0-3053
U+575D.0	fR/S	 32.4
U+575D.0	fTotalStrokes	 7
U+575D.0	fCangjie	GBO
U+575D.0	fHakka	 ba5
U+575D.0	fCantonese	baa3
U+575D.0	fMandarin	ba4
U+575D.0	fEnglish	simplified form of &#x58E9;
U+575E.0	fUTF8	坞
U+575E.0	fGB	0-4E6B
U+575E.0	fR/S	 32.4
U+575E.0	fTotalStrokes	 7
U+575E.0	fCangjie	GPVM
U+575E.0	fHakka	 vu3
U+575E.0	fCantonese	wu2; ou3; ngou3 (also)
U+575E.0	fMandarin	wu4
U+575E.0	fEnglish	simplified form of &#x5862;
U+575F.0	fUTF8	坟
U+575F.0	fCNS11643	3-2535
U+575F.0	fGB	0-3758
U+575F.0	fR/S	 32.4
U+575F.0	fTotalStrokes	 7
U+575F.0	fCangjie	GYK
U+575F.0	fFourCorner	40140
U+575F.0	fHakka	 fun2;  pun2
U+575F.0	fCantonese	fan4
U+575F.0	fMandarin	fen2; ben4; fen4
U+575F.0	fEnglish	simplified form of &#x58B3;
U+5760.0	fUTF8	坠
U+5760.0	fGB	0-5739
U+5760.0	fR/S	 32.4
U+5760.0	fTotalStrokes	 7
U+5760.0	fCangjie	XNOG
U+5760.0	fHakka	 chui5; tui5
U+5760.0	fCantonese	zeoi6; zui6
U+5760.0	fMandarin	zhui4
U+5760.0	fEnglish	simplified form of &#x589C;
U+5761.0	fUTF8	坡
U+5761.0	fCNS11643	1-4C5B
U+5761.0	fBig5	A959
U+5761.0	fGB	0-4642
U+5761.0	fR/S	 32.5
U+5761.0	fTotalStrokes	 8
U+5761.0	fCangjie	GDHE
U+5761.0	fFourCorner	44147
U+5761.0	fHakka	  bo1;  bi1; bok7; po1
U+5761.0	fCantonese	po1; bo1 (also)
U+5761.0	fMandarin	po1; bi4; pi2
U+5761.0	fEnglish	 slope; bank; hillside
U+5762.0	fUTF8	坢
U+5762.0	fCNS11643	2-257E
U+5762.0	fBig5	CBFE
U+5762.0	fGB	3-3377
U+5762.0	fR/S	 32.5
U+5762.0	fTotalStrokes	 8
U+5762.0	fCangjie	GFQ
U+5762.0	fFourCorner	49150
U+5762.0	fCantonese	bun3; bun6; pun5
U+5762.0	fMandarin	pan3; pan4
U+5763.0	fUTF8	坣
U+5763.0	fCNS11643	3-2825
U+5763.0	fGB	5-356B
U+5763.0	fR/S	  32.5
U+5763.0	fTotalStrokes	 8
U+5763.0	fCangjie	FBG
U+5763.0	fFourCorner	90100
U+5763.0	fHakka	  tong2; tong3; to2
U+5763.0	fCantonese	tong4; tong2 (also)
U+5763.0	fMandarin	tang2 tang5
U+5763.0	fEnglish	ancient form of &#x5802;
U+5764.0	fUTF8	坤
U+5764.0	fCNS11643	1-4C5D
U+5764.0	fBig5	A95B
U+5764.0	fGB	0-4024
U+5764.0	fR/S	 32.5
U+5764.0	fTotalStrokes	 8
U+5764.0	fCangjie	GLWL
U+5764.0	fFourCorner	45106
U+5764.0	fHakka	  kun1; kwun1
U+5764.0	fCantonese	kwan1
U+5764.0	fMandarin	kun1
U+5764.0	fEnglish	[1] earth; one of the Eight diagrams [2] feminine; female [3] compliance; obedience
U+5765.0	fUTF8	坥
U+5765.0	fCNS11643	3-2823
U+5765.0	fGB	3-3372
U+5765.0	fR/S	 32.5
U+5765.0	fTotalStrokes	 8
U+5765.0	fCangjie	GBM
U+5765.0	fFourCorner	47110
U+5765.0	fCantonese	ceoi1
U+5765.0	fMandarin	qu1
U+5766.0	fUTF8	坦
U+5766.0	fCNS11643	1-4C5C
U+5766.0	fBig5	A95A
U+5766.0	fGB	0-4C39
U+5766.0	fR/S	 32.5
U+5766.0	fTotalStrokes	 8
U+5766.0	fCangjie	GAM
U+5766.0	fFourCorner	46110
U+5766.0	fHakka	  tan3
U+5766.0	fCantonese	taan2
U+5766.0	fMandarin	tan3
U+5766.0	fEnglish	[1] wide and smooth; flat; smooth; level [2] self-possessed; composed; calm; peaceful [3] frank; straightforward [4] son-in-law [5] a Chinese family name
U+5767.0	fUTF8	坧
U+5767.0	fCNS11643	3-2826
U+5767.0	fGB	5-3568
U+5767.0	fR/S	 32.5
U+5767.0	fTotalStrokes	 8
U+5767.0	fCangjie	GMR
U+5767.0	fFourCorner	41160
U+5767.0	fHakka	 chak7; zak7; zhak7
U+5767.0	fCantonese	zek3
U+5767.0	fMandarin	zhi1; zhi2; zhuo2
U+5767.0	fEnglish	same as &#x588C;
U+5768.0	fUTF8	坨
U+5768.0	fCNS11643	2-2621
U+5768.0	fBig5	CC40
U+5768.0	fGB	0-5B67
U+5768.0	fR/S	 32.5
U+5768.0	fTotalStrokes	 8
U+5768.0	fCangjie	GJP
U+5768.0	fFourCorner	43111
U+5768.0	fCantonese	to4
U+5768.0	fMandarin	tuo2; yi2; zheng1
U+5769.0	fUTF8	坩
U+5769.0	fCNS11643	1-4C5A
U+5769.0	fBig5	A958
U+5769.0	fGB	0-5B61
U+5769.0	fR/S	 32.5
U+5769.0	fTotalStrokes	 8
U+5769.0	fCangjie	GTM
U+5769.0	fFourCorner	44170
U+5769.0	fHakka	  kam1
U+5769.0	fCantonese	ham1; kim4 (also)
U+5769.0	fMandarin	gan1
U+5769.0	fEnglish	earthenware; earthenware vessel
U+576A.0	fUTF8	坪
U+576A.0	fCNS11643	1-4C59
U+576A.0	fBig5	A957
U+576A.0	fGB	0-463A
U+576A.0	fR/S	 32.5
U+576A.0	fTotalStrokes	 8
U+576A.0	fCangjie	GMFJ
U+576A.0	fFourCorner	41149
U+576A.0	fHakka	  piang2;  pin2; piang3
U+576A.0	fCantonese	ping4
U+576A.0	fMandarin	ping2
U+576A.0	fEnglish	[1] level ground; level piece of ground [2] an area of 6 feet square (Japanese measure)
U+576B.0	fUTF8	坫
U+576B.0	fCNS11643	2-2575
U+576B.0	fBig5	CBF5
U+576B.0	fGB	0-5B63
U+576B.0	fR/S	 32.5
U+576B.0	fTotalStrokes	 8
U+576B.0	fCangjie	GYR
U+576B.0	fFourCorner	41160
U+576B.0	fHakka	 diam5
U+576B.0	fCantonese	dim3
U+576B.0	fMandarin	dian4; zhen1
U+576B.0	fEnglish	a stand on which to replace goblets after drinking
U+576C.0	fUTF8	坬
U+576C.0	fCNS11643	4-2569
U+576C.0	fGB	3-3375
U+576C.0	fR/S	 32.5
U+576C.0	fTotalStrokes	 8; 7
U+576C.0	fCangjie	GHVO
U+576C.0	fFourCorner	42130
U+576C.0	fCantonese	gwaa3
U+576C.0	fMandarin	gua4
U+576D.0	fUTF8	坭
U+576D.0	fCNS11643	2-2574
U+576D.0	fBig5	CBF4
U+576D.0	fGB	0-5B68
U+576D.0	fR/S	 32.5
U+576D.0	fTotalStrokes	 8
U+576D.0	fCangjie	GSP
U+576D.0	fFourCorner	47111
U+576D.0	fHakka	  nai2; lai2; ne2
U+576D.0	fCantonese	nai4; lai4 (also); nai2 (also); lai2 (also)
U+576D.0	fMandarin	ni2; ni4; ni3
U+576D.0	fEnglish	 mud; mire paste; plaster
U+576E.0	fUTF8	坮
U+576E.0	fCNS11643	3-2828
U+576E.0	fGB	5-3575
U+576E.0	fR/S	  32.5
U+576E.0	fTotalStrokes	 8
U+576E.0	fCangjie	GIR
U+576E.0	fFourCorner	43160
U+576E.0	fHakka	 toi2
U+576E.0	fCantonese	toi4
U+576E.0	fMandarin	tai2; hu2
U+576E.0	fEnglish	ancient form of &#x81FA;
U+576F.0	fUTF8	坯
U+576F.0	fCNS11643	2-2572
U+576F.0	fBig5	CBF2
U+576F.0	fGB	0-4577
U+576F.0	fR/S	 32.5
U+576F.0	fTotalStrokes	 8
U+576F.0	fCangjie	GMFM
U+576F.0	fFourCorner	41119
U+576F.0	fHakka	  poi1; pi1
U+576F.0	fCantonese	pui1
U+576F.0	fMandarin	pi1; pei1
U+576F.0	fEnglish	 dam; embankment weir
U+5770.0	fUTF8	坰
U+5770.0	fCNS11643	2-2577
U+5770.0	fBig5	CBF7
U+5770.0	fGB	3-3374
U+5770.0	fR/S	 32.5
U+5770.0	fTotalStrokes	 8
U+5770.0	fCangjie	GBR
U+5770.0	fFourCorner	47120
U+5770.0	fHakka	 kwen3; gwen3; kwen1
U+5770.0	fCantonese	gwing1
U+5770.0	fMandarin	jiong1; jyong1
U+5770.0	fEnglish	wilds
U+5771.0	fUTF8	坱
U+5771.0	fCNS11643	2-2576
U+5771.0	fBig5	CBF6
U+5771.0	fGB	3-3373
U+5771.0	fR/S	 32.5
U+5771.0	fTotalStrokes	 8
U+5771.0	fCangjie	GLBK
U+5771.0	fFourCorner	45130
U+5771.0	fHakka	 jong1; rin5; jin5; rong1; jong5
U+5771.0	fCantonese	joeng5 (also); joeng2
U+5771.0	fMandarin	yang3; yang1
U+5771.0	fEnglish	dust
U+5772.0	fUTF8	坲
U+5772.0	fCNS11643	2-2573
U+5772.0	fBig5	CBF3
U+5772.0	fGB	3-337A
U+5772.0	fR/S	 32.5
U+5772.0	fTotalStrokes	 8
U+5772.0	fCangjie	GLLN
U+5772.0	fFourCorner	45127
U+5772.0	fCantonese	fat6
U+5772.0	fMandarin	fo2
U+5773.0	fUTF8	坳
U+5773.0	fCNS11643	2-257C
U+5773.0	fBig5	CBFC
U+5773.0	fGB	0-5B6A
U+5773.0	fR/S	 32.5
U+5773.0	fTotalStrokes	 8
U+5773.0	fCangjie	GVIS
U+5773.0	fFourCorner	44127
U+5773.0	fHakka	  au1; au5; au3; o3
U+5773.0	fCantonese	aau3; aau1 (also); ngaau3 (also); ngaau1 (also)
U+5773.0	fMandarin	ao4; ao1; you3
U+5773.0	fEnglish	 hollow in the ground; cavity
U+5774.0	fUTF8	坴
U+5774.0	fCNS11643	2-257D
U+5774.0	fBig5	CBFD
U+5774.0	fGB	5-3567
U+5774.0	fR/S	 32.5
U+5774.0	fTotalStrokes	 8
U+5774.0	fCangjie	GCG
U+5774.0	fFourCorner	40104
U+5774.0	fHakka	 liuk7
U+5774.0	fCantonese	luk6
U+5774.0	fMandarin	liu4
U+5774.0	fEnglish	 clod of earth land
U+5775.0	fUTF8	坵
U+5775.0	fCNS11643	2-257A
U+5775.0	fBig5	CBFA
U+5775.0	fGB	E-2554
U+5775.0	fR/S	 32.5
U+5775.0	fTotalStrokes	 8
U+5775.0	fCangjie	GOM
U+5775.0	fFourCorner	42111
U+5775.0	fHakka	  hiu1; kiu1
U+5775.0	fCantonese	jau1
U+5775.0	fMandarin	qiu1
U+5775.0	fEnglish	earthenware, earthenware vessel
U+5776.0	fUTF8	坶
U+5776.0	fCNS11643	2-2578
U+5776.0	fBig5	CBF8
U+5776.0	fGB	0-5B69
U+5776.0	fR/S	 32.5
U+5776.0	fTotalStrokes	 8
U+5776.0	fCangjie	GWYI
U+5776.0	fFourCorner	47150
U+5776.0	fCantonese	muk6
U+5776.0	fMandarin	mu4
U+5777.0	fUTF8	坷
U+5777.0	fCNS11643	1-4C58
U+5777.0	fBig5	A956
U+5777.0	fGB	0-3F40
U+5777.0	fR/S	 32.5
U+5777.0	fTotalStrokes	 8
U+5777.0	fCangjie	GMNR
U+5777.0	fFourCorner	41120
U+5777.0	fHakka	 ko3; ko1
U+5777.0	fCantonese	ho2; ho1
U+5777.0	fMandarin	ke3; ke1; ce3
U+5777.0	fEnglish	[1] bad luck; unfortunate [2] rugged; uneven (roads, etc.)
U+5778.0	fUTF8	坸
U+5778.0	fCNS11643	3-2829
U+5778.0	fGB	5-3570
U+5778.0	fR/S	  32.5
U+5778.0	fTotalStrokes	 8
U+5778.0	fCangjie	GPR
U+5778.0	fFourCorner	47120
U+5778.0	fHakka	 kieu5; geu5
U+5778.0	fCantonese	gau3
U+5778.0	fMandarin	gou4
U+5779.0	fUTF8	坹
U+5779.0	fCNS11643	4-2566
U+5779.0	fGB	5-3572
U+5779.0	fR/S	 32.5
U+5779.0	fTotalStrokes	 8; 7
U+5779.0	fCangjie	GJC
U+5779.0	fFourCorner	43180
U+5779.0	fCantonese	jyut6
U+5779.0	fMandarin	xue4
U+577A.0	fUTF8	坺
U+577A.0	fCNS11643	4-2568
U+577A.0	fGB	3-3371
U+577A.0	fR/S	 32.5
U+577A.0	fTotalStrokes	 8; 7
U+577A.0	fCangjie	GIKE
U+577A.0	fFourCorner	43140
U+577A.0	fHakka	 fat8
U+577A.0	fCantonese	paat6; baat6; pet6
U+577A.0	fMandarin	ba2; bo1
U+577A.0	fEnglish	classifier for soft masses
U+577B.0	fUTF8	坻
U+577B.0	fCNS11643	2-257B
U+577B.0	fBig5	CBFB
U+577B.0	fGB	0-5B66
U+577B.0	fR/S	 32.5
U+577B.0	fTotalStrokes	 8
U+577B.0	fCangjie	GHPM
U+577B.0	fFourCorner	42140
U+577B.0	fHakka	 chi2; cih2; zhi3; zih3; chii2
U+577B.0	fCantonese	ci4; dai2; zi2
U+577B.0	fMandarin	di3; chi2
U+577B.0	fEnglish	 islet; a rock in a river embankment stop
U+577C.0	fUTF8	坼
U+577C.0	fCNS11643	1-4C5E
U+577C.0	fBig5	A95C
U+577C.0	fGB	0-5B65
U+577C.0	fR/S	 32.5
U+577C.0	fTotalStrokes	 8
U+577C.0	fCangjie	GHMY
U+577C.0	fFourCorner	42141
U+577C.0	fHakka	 cak7
U+577C.0	fCantonese	caak3
U+577C.0	fMandarin	che4
U+577C.0	fEnglish	[1] crack; split [2] chap; tear; rip open
U+577D.0	fUTF8	坽
U+577D.0	fCNS11643	2-2622
U+577D.0	fBig5	CC41
U+577D.0	fGB	5-356E
U+577D.0	fR/S	 32.5
U+577D.0	fTotalStrokes	 8
U+577D.0	fCangjie	GOII
U+577D.0	fFourCorner	48137
U+577D.0	fHakka	 lang1;  lang3;  len1
U+577D.0	fCantonese	ling4
U+577D.0	fMandarin	ling2
U+577E.0	fUTF8	坾
U+577E.0	fCNS11643	4-2565
U+577E.0	fGB	5-3564
U+577E.0	fR/S	 32.5
U+577E.0	fTotalStrokes	 8; 7
U+577E.0	fCangjie	GJMN
U+577E.0	fFourCorner	43120
U+577E.0	fCantonese	cyu5
U+577E.0	fMandarin	zhu4
U+577F.0	fUTF8	坿
U+577F.0	fCNS11643	3-282A
U+577F.0	fGB	8-2D5D
U+577F.0	fR/S	 32.5
U+577F.0	fTotalStrokes	 8
U+577F.0	fCangjie	GODI
U+577F.0	fFourCorner	44100
U+577F.0	fCantonese	fu6
U+577F.0	fMandarin	fu4
U+577F.0	fEnglish	 mound used in place names
U+5780.0	fUTF8	垀
U+5780.0	fCNS11643	2-2579
U+5780.0	fBig5	CBF9
U+5780.0	fGB	3-3376
U+5780.0	fR/S	 32.5
U+5780.0	fTotalStrokes	 8
U+5780.0	fCangjie	GHFD
U+5780.0	fFourCorner	42149
U+5780.0	fCantonese	fu1
U+5780.0	fMandarin	hu1
U+5781.0	fUTF8	垁
U+5781.0	fCNS11643	3-2824
U+5781.0	fGB	5-356D
U+5781.0	fR/S	 32.5
U+5781.0	fTotalStrokes	 8
U+5781.0	fCangjie	GOK
U+5781.0	fFourCorner	48130
U+5781.0	fCantonese	zi6
U+5781.0	fMandarin	zhi4
U+5782.0	fUTF8	垂
U+5782.0	fCNS11643	1-504D
U+5782.0	fBig5	ABAB
U+5782.0	fGB	0-3439
U+5782.0	fR/S	 32.5
U+5782.0	fTotalStrokes	 8
U+5782.0	fCangjie	HJTM
U+5782.0	fFourCorner	20104
U+5782.0	fHakka	  sui2;  shui2; sui3; shui5; chui2
U+5782.0	fCantonese	seoi4
U+5782.0	fMandarin	chui2; shui4; zhui4
U+5782.0	fEnglish	[1] hang down; let fall [2] hand down; leave a name in history [3] nearly; almost; approaching [4] condescend
U+5783.0	fUTF8	垃
U+5783.0	fCNS11643	1-4C57
U+5783.0	fBig5	A955
U+5783.0	fGB	0-402C
U+5783.0	fR/S	 32.5
U+5783.0	fTotalStrokes	 8
U+5783.0	fCangjie	GYT
U+5783.0	fFourCorner	40118
U+5783.0	fHakka	 lat7; la3; la5; lap7; la6
U+5783.0	fCantonese	laap6; laai1; laa1 (also)
U+5783.0	fMandarin	la1; la5
U+5783.0	fEnglish	garbage; refuse; waste
U+5784.0	fUTF8	垄
U+5784.0	fGB	0-4222
U+5784.0	fR/S	 212.3; 32.5
U+5784.0	fTotalStrokes	 8
U+5784.0	fCangjie	XIPG
U+5784.0	fHakka	 liung1; lung3
U+5784.0	fCantonese	lung5
U+5784.0	fMandarin	long3; long1
U+5784.0	fEnglish	simplified form of &#x58DF;
U+5785.0	fUTF8	垅
U+5785.0	fGB	0-5B62
U+5785.0	fR/S	 32.5
U+5785.0	fTotalStrokes	 8
U+5785.0	fCangjie	GIKP
U+5785.0	fHakka	 lung2
U+5785.0	fCantonese	lung5
U+5785.0	fMandarin	long3
U+5785.0	fEnglish	simplified form of &#x58E0;
U+5786.0	fUTF8	垆
U+5786.0	fGB	0-5B64
U+5786.0	fR/S	 32.5
U+5786.0	fTotalStrokes	 8
U+5786.0	fCangjie	GYS
U+5786.0	fHakka	 lu2
U+5786.0	fCantonese	lou4
U+5786.0	fMandarin	lu2
U+5786.0	fEnglish	simplified form of &#x58DA;
U+5787.0	fUTF8	垇
U+5787.0	fCNS11643	4-256E
U+5787.0	fGB	5-356C
U+5787.0	fR/S	 32.5; 32.6
U+5787.0	fTotalStrokes	 8; 9
U+5787.0	fCangjie	GSSU
U+5787.0	fFourCorner	47170
U+5787.0	fCantonese	aau3; ngaau3 (also); aau1 (also); ngaau1 (also)
U+5787.0	fMandarin	ao4; ao1
U+5788.0	fUTF8	垈
U+5788.0	fCNS11643	F-2633
U+5788.0	fGB	1-7D35
U+5788.0	fR/S	 32.5
U+5788.0	fTotalStrokes	 8
U+5788.0	fCangjie	OPG
U+5788.0	fFourCorner	2310
U+5788.0	fCantonese	doi6
U+5789.0	fUTF8	垉
U+5789.0	fGB	E-2555
U+5789.0	fR/S	 32.5
U+5789.0	fTotalStrokes	 8
U+5789.0	fCangjie	GPRU
U+5789.0	fFourCorner	4711
U+5789.0	fCantonese	paau4
U+5789.0	fMandarin	pao2
U+578A.0	fUTF8	垊
U+578A.0	fCNS11643	3-6460
U+578A.0	fGB	E-2556
U+578A.0	fR/S	 32.5
U+578A.0	fTotalStrokes	 8
U+578A.0	fCangjie	GRVP
U+578A.0	fFourCorner	4714
U+578A.0	fCantonese	man4
U+578B.0	fUTF8	型
U+578B.0	fCNS11643	1-504E
U+578B.0	fBig5	ABAC
U+578B.0	fGB	0-504D
U+578B.0	fR/S	 32.6
U+578B.0	fTotalStrokes	 9
U+578B.0	fCangjie	MNG
U+578B.0	fFourCorner	12104
U+578B.0	fHakka	  hin2
U+578B.0	fCantonese	jing4
U+578B.0	fMandarin	xing2; jia2
U+578B.0	fEnglish	[1] pattern; model; type; standard [2] earthen mold for casting
U+578C.0	fUTF8	垌
U+578C.0	fCNS11643	2-2955
U+578C.0	fBig5	CE54
U+578C.0	fGB	0-5B6D
U+578C.0	fR/S	 32.6
U+578C.0	fTotalStrokes	 9
U+578C.0	fCangjie	GBMR
U+578C.0	fFourCorner	47120
U+578C.0	fHakka	 tung5
U+578C.0	fCantonese	dung6; tung4; tung2; tung6
U+578C.0	fMandarin	dong4; tong2; tong3
U+578C.0	fEnglish	a Chinese family name
U+578D.0	fUTF8	垍
U+578D.0	fCNS11643	3-2B69
U+578D.0	fGB	3-3422
U+578D.0	fR/S	 32.6
U+578D.0	fTotalStrokes	 9
U+578D.0	fCangjie	GHBU
U+578D.0	fFourCorner	47170
U+578D.0	fHakka	 ki5
U+578D.0	fCantonese	gei3 (also); gei6
U+578D.0	fMandarin	ji4
U+578E.0	fUTF8	垎
U+578E.0	fCNS11643	3-2B68
U+578E.0	fGB	5-3622
U+578E.0	fR/S	  32.6
U+578E.0	fTotalStrokes	 9
U+578E.0	fCangjie	GHER
U+578E.0	fFourCorner	47160
U+578E.0	fHakka	  kak7; giet7; get7
U+578E.0	fCantonese	haak6
U+578E.0	fMandarin	ke4
U+578E.0	fEnglish	water drying off land
U+578F.0	fUTF8	垏
U+578F.0	fCNS11643	2-295B
U+578F.0	fBig5	CE5A
U+578F.0	fGB	5-3625
U+578F.0	fR/S	 32.6
U+578F.0	fTotalStrokes	 9
U+578F.0	fCangjie	GLQ
U+578F.0	fFourCorner	45107
U+578F.0	fCantonese	leot6
U+578F.0	fMandarin	lu4
U+5790.0	fUTF8	垐
U+5790.0	fCNS11643	3-2B6C
U+5790.0	fGB	3-3427
U+5790.0	fR/S	  32.6
U+5790.0	fTotalStrokes	 9
U+5790.0	fCangjie	IOG
U+5790.0	fFourCorner	37100
U+5790.0	fCantonese	ci4
U+5790.0	fMandarin	ci2
U+5791.0	fUTF8	垑
U+5791.0	fCNS11643	3-2B67
U+5791.0	fGB	3-3425
U+5791.0	fR/S	 32.6
U+5791.0	fTotalStrokes	 9
U+5791.0	fCangjie	GNIN
U+5791.0	fFourCorner	47120
U+5791.0	fCantonese	ci2
U+5791.0	fMandarin	chi3
U+5792.0	fUTF8	垒
U+5792.0	fCNS11643	3-2B6A
U+5792.0	fGB	0-405D
U+5792.0	fR/S	  32.6
U+5792.0	fTotalStrokes	 9
U+5792.0	fCangjie	IIIG
U+5792.0	fFourCorner	23100
U+5792.0	fHakka	 lui3
U+5792.0	fCantonese	leoi5; leoi6; leoi4; leot6
U+5792.0	fMandarin	lei3; lei4; lei2; l4
U+5792.0	fEnglish	simplified form of &#x58D8;
U+5793.0	fUTF8	垓
U+5793.0	fCNS11643	1-5054
U+5793.0	fBig5	ABB2
U+5793.0	fGB	0-5B72
U+5793.0	fR/S	 32.6
U+5793.0	fTotalStrokes	 9
U+5793.0	fCangjie	GYVO
U+5793.0	fFourCorner	40182
U+5793.0	fHakka	 goi1; gai5
U+5793.0	fCantonese	goi1
U+5793.0	fMandarin	gai1; jie1
U+5793.0	fEnglish	[1] border; boundary; frontier [2] far and remote places; wilds; beyond the frontier [3] a hundred million
U+5794.0	fUTF8	垔
U+5794.0	fCNS11643	2-2959
U+5794.0	fBig5	CE58
U+5794.0	fGB	5-5F63
U+5794.0	fR/S	 32.6
U+5794.0	fTotalStrokes	 9
U+5794.0	fCangjie	MWG
U+5794.0	fFourCorner	10104
U+5794.0	fHakka	 jen1; ren1; jin1
U+5794.0	fCantonese	jan1
U+5794.0	fMandarin	yin1; dou4; shu4
U+5794.0	fEnglish	 restrain dam a stream and change its direction mound
U+5795.0	fUTF8	垕
U+5795.0	fCNS11643	2-295F
U+5795.0	fBig5	CE5E
U+5795.0	fGB	5-357E
U+5795.0	fR/S	 32.6
U+5795.0	fTotalStrokes	 9
U+5795.0	fCangjie	HRG
U+5795.0	fFourCorner	72214
U+5795.0	fCantonese	hau5; hau6
U+5795.0	fMandarin	hou4
U+5795.0	fEnglish	used in place names
U+5796.0	fUTF8	垖
U+5796.0	fCNS11643	3-2B71
U+5796.0	fGB	5-357D
U+5796.0	fR/S	 32.6
U+5796.0	fTotalStrokes	 9
U+5796.0	fCangjie	GHRR
U+5796.0	fFourCorner	47170
U+5796.0	fHakka	  doi1;  doi3; doi5
U+5796.0	fCantonese	deoi1; zeoi1
U+5796.0	fMandarin	dui1; zui1
U+5796.0	fEnglish	standard form of &#x5806;
U+5797.0	fUTF8	垗
U+5797.0	fCNS11643	2-2956
U+5797.0	fBig5	CE55
U+5797.0	fGB	3-3423
U+5797.0	fR/S	 32.6
U+5797.0	fTotalStrokes	 9
U+5797.0	fCangjie	GLMO
U+5797.0	fFourCorner	42113
U+5797.0	fHakka	 chau5; shau5; sau5; cau5
U+5797.0	fCantonese	ziu6; siu6
U+5797.0	fMandarin	zhao4
U+5797.0	fEnglish	sacrifice
U+5798.0	fUTF8	垘
U+5798.0	fCNS11643	2-295A
U+5798.0	fBig5	CE59
U+5798.0	fGB	3-337E
U+5798.0	fR/S	 32.6
U+5798.0	fTotalStrokes	 9
U+5798.0	fCangjie	GOIK
U+5798.0	fFourCorner	43130
U+5798.0	fCantonese	baak6; fuk6
U+5798.0	fMandarin	fu2
U+5799.0	fUTF8	垙
U+5799.0	fCNS11643	2-295C
U+5799.0	fBig5	CE5B
U+5799.0	fGB	5-357B
U+5799.0	fR/S	 32.6
U+5799.0	fTotalStrokes	 9
U+5799.0	fCangjie	GFMU
U+5799.0	fFourCorner	49111
U+5799.0	fHakka	 gong1
U+5799.0	fCantonese	gwong1; gong1 (also)
U+5799.0	fMandarin	guang1
U+579A.0	fUTF8	垚
U+579A.0	fCNS11643	2-295E
U+579A.0	fBig5	CE5D
U+579A.0	fGB	5-3577
U+579A.0	fR/S	 32.6
U+579A.0	fTotalStrokes	 9
U+579A.0	fCangjie	GGG
U+579A.0	fFourCorner	40114
U+579A.0	fCantonese	jiu4; jiu1
U+579A.0	fMandarin	yao2
U+579A.0	fEnglish	mound; roundish mass
U+579B.0	fUTF8	垛
U+579B.0	fCNS11643	2-2958
U+579B.0	fBig5	CE57
U+579B.0	fGB	0-3662
U+579B.0	fR/S	 32.6
U+579B.0	fTotalStrokes	 9
U+579B.0	fCangjie	GHND
U+579B.0	fFourCorner	47194
U+579B.0	fHakka	 do3; do1
U+579B.0	fCantonese	do2; do6
U+579B.0	fMandarin	duo3; duo4
U+579B.0	fEnglish	 heap; pile pile up; heap up
U+579C.0	fUTF8	垜
U+579C.0	fCNS11643	3-2B70
U+579C.0	fGB	E-2557
U+579C.0	fR/S	 32.6
U+579C.0	fTotalStrokes	 9
U+579C.0	fCangjie	GNSD
U+579C.0	fFourCorner	47190
U+579C.0	fHakka	 do3
U+579C.0	fCantonese	do2
U+579C.0	fMandarin	duo3; duo4
U+579C.0	fEnglish	heap
U+579D.0	fUTF8	垝
U+579D.0	fCNS11643	2-2957
U+579D.0	fBig5	CE56
U+579D.0	fGB	3-3424
U+579D.0	fR/S	 32.6
U+579D.0	fTotalStrokes	 9
U+579D.0	fCangjie	GNMU
U+579D.0	fFourCorner	47112
U+579D.0	fHakka	 gui3; gwi3; kwui3
U+579D.0	fCantonese	gwai2
U+579D.0	fMandarin	gui3; gui4; kui3
U+579D.0	fEnglish	delapidated; ruined
U+579E.0	fUTF8	垞
U+579E.0	fCNS11643	2-2952
U+579E.0	fBig5	CE51
U+579E.0	fGB	3-3429
U+579E.0	fR/S	 32.6
U+579E.0	fTotalStrokes	 9
U+579E.0	fCangjie	GJHP
U+579E.0	fFourCorner	43114
U+579E.0	fCantonese	caa4
U+579E.0	fMandarin	cha2
U+579E.0	fEnglish	 small mound a place name hillock
U+579F.0	fUTF8	垟
U+579F.0	fCNS11643	2-2953
U+579F.0	fBig5	CE52
U+579F.0	fGB	3-3428
U+579F.0	fR/S	 32.6
U+579F.0	fTotalStrokes	 9
U+579F.0	fCangjie	GTQ
U+579F.0	fFourCorner	48151
U+579F.0	fHakka	 jong2; rong2
U+579F.0	fCantonese	joeng4
U+579F.0	fMandarin	yang2
U+579F.0	fEnglish	[1] elf; sprite [2] field
U+57A0.0	fUTF8	垠
U+57A0.0	fCNS11643	1-504F
U+57A0.0	fBig5	ABAD
U+57A0.0	fGB	0-5B73
U+57A0.0	fR/S	 32.6
U+57A0.0	fTotalStrokes	 9
U+57A0.0	fCangjie	GAV
U+57A0.0	fFourCorner	47132
U+57A0.0	fHakka	  ngiun2; niun2
U+57A0.0	fCantonese	ngan4; an4 (also)
U+57A0.0	fMandarin	yin2; hen2; ken4
U+57A0.0	fEnglish	[1] boundary; limit [2] bank of a stream or river
U+57A1.0	fUTF8	垡
U+57A1.0	fCNS11643	3-2B6B
U+57A1.0	fGB	0-5B52
U+57A1.0	fR/S	  32.6
U+57A1.0	fTotalStrokes	 9
U+57A1.0	fCangjie	OIG
U+57A1.0	fFourCorner	23100
U+57A1.0	fHakka	  fat8
U+57A1.0	fCantonese	fat6
U+57A1.0	fMandarin	fa2
U+57A1.0	fEnglish	 plow soil place name
U+57A2.0	fUTF8	垢
U+57A2.0	fCNS11643	1-5051
U+57A2.0	fBig5	ABAF
U+57A2.0	fGB	0-3938
U+57A2.0	fR/S	 32.6
U+57A2.0	fTotalStrokes	 9
U+57A2.0	fCangjie	GHMR
U+57A2.0	fFourCorner	42161
U+57A2.0	fHakka	 kieu5; giu3; geu5
U+57A2.0	fCantonese	gau3; hau4
U+57A2.0	fMandarin	gou4; kou4
U+57A2.0	fEnglish	[1] dirt; filth; stains; dirty [2] disgrace; shame
U+57A3.0	fUTF8	垣
U+57A3.0	fCNS11643	1-5050
U+57A3.0	fBig5	ABAE
U+57A3.0	fGB	0-542B
U+57A3.0	fR/S	 32.6
U+57A3.0	fTotalStrokes	 9
U+57A3.0	fCangjie	GMAM
U+57A3.0	fFourCorner	41116
U+57A3.0	fHakka	 jen2; ren2; jan2
U+57A3.0	fCantonese	wun4
U+57A3.0	fMandarin	yuan2; huan2
U+57A3.0	fEnglish	low wall
U+57A4.0	fUTF8	垤
U+57A4.0	fCNS11643	2-2954
U+57A4.0	fBig5	CE53
U+57A4.0	fGB	0-5B6C
U+57A4.0	fR/S	 32.6
U+57A4.0	fTotalStrokes	 9
U+57A4.0	fCangjie	GMIG
U+57A4.0	fFourCorner	41114
U+57A4.0	fHakka	  diet7; det7
U+57A4.0	fCantonese	dit6
U+57A4.0	fMandarin	die2
U+57A4.0	fEnglish	 ant-hill; small mound hill
U+57A5.0	fUTF8	垥
U+57A5.0	fCNS11643	2-295D
U+57A5.0	fBig5	CE5C
U+57A5.0	fGB	5-3621
U+57A5.0	fR/S	 32.6
U+57A5.0	fTotalStrokes	 9
U+57A5.0	fCangjie	GOMR
U+57A5.0	fFourCorner	48161
U+57A5.0	fCantonese	hip3
U+57A5.0	fMandarin	xie2
U+57A6.0	fUTF8	垦
U+57A6.0	fCNS11643	3-2B6D
U+57A6.0	fGB	0-3F51
U+57A6.0	fR/S	 32.6
U+57A6.0	fTotalStrokes	 9
U+57A6.0	fCangjie	AVG
U+57A6.0	fFourCorner	77100
U+57A6.0	fHakka	 ngiun2
U+57A6.0	fCantonese	han2; ngan4
U+57A6.0	fMandarin	ken3; yin2; hen2; ken4
U+57A6.0	fEnglish	[1] same as &#x57A0; [2] simplified form of &#x58BE;
U+57A7.0	fUTF8	垧
U+57A7.0	fCNS11643	3-2B72
U+57A7.0	fGB	0-5B70
U+57A7.0	fR/S	 32.6
U+57A7.0	fTotalStrokes	 9
U+57A7.0	fCangjie	GHBR
U+57A7.0	fFourCorner	47120
U+57A7.0	fHakka	 kwen1; kwen3
U+57A7.0	fCantonese	hoeng2; gwing1
U+57A7.0	fMandarin	jiong1; jyong1; shang3
U+57A7.0	fEnglish	variable unit of land measure
U+57A8.0	fUTF8	垨
U+57A8.0	fCNS11643	3-2B6E
U+57A8.0	fGB	E-2558
U+57A8.0	fR/S	 32.6
U+57A8.0	fTotalStrokes	 9
U+57A8.0	fCangjie	GJDI
U+57A8.0	fFourCorner	43140
U+57A8.0	fHakka	 su3;  shiu3; siu3; shiu6; shu3
U+57A8.0	fCantonese	sau2; sau3
U+57A8.0	fMandarin	shou3; shou4
U+57A8.0	fEnglish	same as &#x5B88;
U+57A9.0	fUTF8	垩
U+57A9.0	fCNS11643	4-2860
U+57A9.0	fGB	0-5B51
U+57A9.0	fR/S	 32.6
U+57A9.0	fTotalStrokes	 9
U+57A9.0	fCangjie	MCG
U+57A9.0	fFourCorner	10100
U+57A9.0	fHakka	 ok7
U+57A9.0	fCantonese	ok3; ngok3 (also)
U+57A9.0	fMandarin	e4; wu4
U+57A9.0	fEnglish	simplified form of &#x580A;
U+57AA.0	fUTF8	垪
U+57AA.0	fCNS11643	F-296A
U+57AA.0	fGB	E-2559
U+57AA.0	fR/S	 32.6
U+57AA.0	fTotalStrokes	 9
U+57AA.0	fCangjie	GTT
U+57AA.0	fCantonese	bing6
U+57AB.0	fUTF8	垫
U+57AB.0	fCNS11643	F-2971
U+57AB.0	fGB	0-3566
U+57AB.0	fR/S	 32.6
U+57AB.0	fTotalStrokes	 9
U+57AB.0	fCangjie	QIG
U+57AB.0	fFourCorner	5410
U+57AB.0	fHakka	 diam5; tiap8
U+57AB.0	fCantonese	dim3; din6
U+57AB.0	fMandarin	dian4
U+57AB.0	fEnglish	simplified form of &#x588A;
U+57AC.0	fUTF8	垬
U+57AC.0	fCNS11643	3-2B66
U+57AC.0	fGB	E-255A
U+57AC.0	fR/S	  32.6
U+57AC.0	fTotalStrokes	 9
U+57AC.0	fCangjie	GTC
U+57AC.0	fFourCorner	44180
U+57AC.0	fCantonese	hung4
U+57AC.0	fMandarin	hong2
U+57AD.0	fUTF8	垭
U+57AD.0	fGB	0-5B6B
U+57AD.0	fR/S	 32.6
U+57AD.0	fTotalStrokes	 9
U+57AD.0	fCangjie	GMTC
U+57AD.0	fCantonese	aa3; ngaa3 (also)
U+57AD.0	fMandarin	wu4; ya4; wu3
U+57AD.0	fEnglish	simplified form of &#x57E1;
U+57AE.0	fUTF8	垮
U+57AE.0	fCNS11643	1-5053
U+57AE.0	fBig5	ABB1
U+57AE.0	fGB	0-3F65
U+57AE.0	fR/S	 32.6
U+57AE.0	fTotalStrokes	 9
U+57AE.0	fCangjie	GKMS
U+57AE.0	fFourCorner	44127
U+57AE.0	fHakka	 ka1; kwa1; kwa5
U+57AE.0	fCantonese	kwaa1
U+57AE.0	fMandarin	kua3
U+57AE.0	fEnglish	[1] topple; collapse; be defeated; fail [2] wear down [3] put to rout
U+57AF.0	fUTF8	垯
U+57AF.0	fGB	7-2121
U+57AF.0	fR/S	 32.6
U+57AF.0	fTotalStrokes	 9
U+57AF.0	fCangjie	XGYK
U+57AF.0	fCantonese	daat3; daat6 (also)
U+57AF.0	fMandarin	da5
U+57B0.0	fUTF8	垰
U+57B0.0	fGB	E-255B
U+57B0.0	fR/S	 32.6
U+57B0.0	fTotalStrokes	 9
U+57B0.0	fCangjie	GYMY
U+57B0.0	fCantonese	lung6
U+57B1.0	fUTF8	垱
U+57B1.0	fGB	8-7A49
U+57B1.0	fR/S	 32.6
U+57B1.0	fTotalStrokes	 9
U+57B1.0	fCangjie	GFSM
U+57B1.0	fCantonese	dong3
U+57B1.0	fMandarin	dang4
U+57B2.0	fUTF8	垲
U+57B2.0	fGB	0-5B6E
U+57B2.0	fR/S	 32.6
U+57B2.0	fTotalStrokes	 9
U+57B2.0	fCangjie	GUSU
U+57B2.0	fHakka	 koi3
U+57B2.0	fCantonese	hoi2
U+57B2.0	fMandarin	kai3
U+57B2.0	fEnglish	simplified form of &#x584F;
U+57B3.0	fUTF8	垳
U+57B3.0	fGB	E-255C
U+57B3.0	fR/S	 32.6
U+57B3.0	fTotalStrokes	 9
U+57B3.0	fCangjie	GHON
U+57B3.0	fCantonese	hang4
U+57B4.0	fUTF8	垴
U+57B4.0	fCNS11643	4-285E
U+57B4.0	fGB	0-5B71
U+57B4.0	fR/S	  32.6
U+57B4.0	fTotalStrokes	 9
U+57B4.0	fCangjie	GYUK
U+57B4.0	fFourCorner	40170
U+57B4.0	fHakka	 nau3
U+57B4.0	fCantonese	nou5; lou5 (also)
U+57B4.0	fMandarin	nao3
U+57B4.0	fEnglish	simplified form of &#x5816;
U+57B5.0	fUTF8	垵
U+57B5.0	fCNS11643	2-2951
U+57B5.0	fBig5	CE50
U+57B5.0	fGB	5-3623
U+57B5.0	fR/S	 32.6
U+57B5.0	fTotalStrokes	 9
U+57B5.0	fCangjie	GJV
U+57B5.0	fFourCorner	43144
U+57B5.0	fCantonese	am2; ngam2 (also); am1; jim2
U+57B5.0	fMandarin	an1; an3
U+57B6.0	fUTF8	垶
U+57B6.0	fCNS11643	2-2E55
U+57B6.0	fBig5	D153
U+57B6.0	fGB	5-3630
U+57B6.0	fR/S	 32.7
U+57B6.0	fTotalStrokes	 10
U+57B6.0	fCangjie	GYTJ
U+57B6.0	fFourCorner	40141
U+57B6.0	fCantonese	sing1
U+57B6.0	fMandarin	xing1
U+57B7.0	fUTF8	垷
U+57B7.0	fCNS11643	3-302A
U+57B7.0	fGB	3-3369
U+57B7.0	fR/S	 32.7
U+57B7.0	fTotalStrokes	 10
U+57B7.0	fCangjie	GBUU
U+57B7.0	fFourCorner	46110
U+57B7.0	fHakka	 hien5; hien6; hien3
U+57B7.0	fCantonese	jin6
U+57B7.0	fMandarin	xian4
U+57B7.0	fEnglish	[1] mud [2] mire [3] embankment
U+57B8.0	fUTF8	垸
U+57B8.0	fCNS11643	2-2E54
U+57B8.0	fBig5	D152
U+57B8.0	fGB	0-5B79
U+57B8.0	fR/S	 32.7
U+57B8.0	fTotalStrokes	 10
U+57B8.0	fCangjie	GJMU
U+57B8.0	fFourCorner	43111
U+57B8.0	fHakka	 wan2; van2
U+57B8.0	fCantonese	jyun6; wun2; jyun4
U+57B8.0	fMandarin	huan4; huan2; wan2; yuan4
U+57B8.0	fEnglish	lime and varnish mixed for laquering
U+57B9.0	fUTF8	垹
U+57B9.0	fCNS11643	2-2E59
U+57B9.0	fBig5	D157
U+57B9.0	fGB	5-3576
U+57B9.0	fR/S	 32.7
U+57B9.0	fTotalStrokes	 10
U+57B9.0	fCangjie	GQJL
U+57B9.0	fFourCorner	47127
U+57B9.0	fHakka	 bong1; bang1
U+57B9.0	fCantonese	bong1
U+57B9.0	fMandarin	bang1
U+57B9.0	fEnglish	a funus, like a child's hand, said to give strength when eaten
U+57BA.0	fUTF8	垺
U+57BA.0	fCNS11643	2-2E50
U+57BA.0	fBig5	D14E
U+57BA.0	fGB	3-342F
U+57BA.0	fR/S	 32.7
U+57BA.0	fTotalStrokes	 10
U+57BA.0	fCangjie	GBND
U+57BA.0	fFourCorner	42147
U+57BA.0	fCantonese	pau4; fu1; pui1
U+57BA.0	fMandarin	pei1; fu2; bu4; pou2
U+57BB.0	fUTF8	垻
U+57BB.0	fCNS11643	3-3030
U+57BB.0	fGB	E-255D
U+57BB.0	fR/S	 32.7
U+57BB.0	fTotalStrokes	 10
U+57BB.0	fCangjie	GBUC
U+57BB.0	fFourCorner	46180
U+57BB.0	fHakka	  ba5; bui5
U+57BB.0	fCantonese	baa3; bui3
U+57BB.0	fMandarin	ba4
U+57BB.0	fEnglish	 embankment a slope over which boat pass
U+57BC.0	fUTF8	垼
U+57BC.0	fCNS11643	2-2E53
U+57BC.0	fBig5	D151
U+57BC.0	fGB	3-342E
U+57BC.0	fR/S	 32.7
U+57BC.0	fTotalStrokes	 10
U+57BC.0	fCangjie	HEG
U+57BC.0	fFourCorner	27104
U+57BC.0	fCantonese	jik6
U+57BC.0	fMandarin	yi4
U+57BD.0	fUTF8	垽
U+57BD.0	fCNS11643	2-2E52
U+57BD.0	fBig5	D150
U+57BD.0	fGB	3-3432
U+57BD.0	fR/S	 32.7
U+57BD.0	fTotalStrokes	 10
U+57BD.0	fCangjie	ELG
U+57BD.0	fFourCorner	32104
U+57BD.0	fHakka	 ngiun5; ngiun6; niun5
U+57BD.0	fCantonese	ngan6; an6 (also); jan6
U+57BD.0	fMandarin	yin4
U+57BD.0	fEnglish	[1] slime [2] mud [3] dregs
U+57BE.0	fUTF8	垾
U+57BE.0	fCNS11643	3-302F
U+57BE.0	fGB	3-342C
U+57BE.0	fR/S	 32.7
U+57BE.0	fTotalStrokes	 10
U+57BE.0	fCangjie	GAMJ
U+57BE.0	fFourCorner	46140
U+57BE.0	fHakka	 hon5; hon6; ngan3
U+57BE.0	fCantonese	hon6
U+57BE.0	fMandarin	han4
U+57BE.0	fEnglish	bank; dyke
U+57BF.0	fUTF8	垿
U+57BF.0	fCNS11643	2-2E56
U+57BF.0	fBig5	D154
U+57BF.0	fGB	3-3431
U+57BF.0	fR/S	 32.7
U+57BF.0	fTotalStrokes	 10
U+57BF.0	fCangjie	GINN
U+57BF.0	fFourCorner	40147
U+57BF.0	fHakka	 si5
U+57BF.0	fCantonese	zeoi6
U+57BF.0	fMandarin	xu4
U+57C0.0	fUTF8	埀
U+57C0.0	fCNS11643	3-3038
U+57C0.0	fGB	E-255E
U+57C0.0	fR/S	 32.7
U+57C0.0	fTotalStrokes	 10
U+57C0.0	fCangjie	HJLG
U+57C0.0	fFourCorner	20100
U+57C0.0	fHakka	 shui2
U+57C0.0	fCantonese	seoi4
U+57C0.0	fMandarin	chui2; zhui4; shui4
U+57C0.0	fEnglish	let down; suspend, hand; down
U+57C1.0	fUTF8	埁
U+57C1.0	fCNS11643	2-2E5A
U+57C1.0	fBig5	D158
U+57C1.0	fGB	5-362D
U+57C1.0	fR/S	 32.7
U+57C1.0	fTotalStrokes	 10
U+57C1.0	fCangjie	GUON
U+57C1.0	fFourCorner	42127
U+57C1.0	fCantonese	cam4; sam4
U+57C1.0	fMandarin	cen2
U+57C2.0	fUTF8	埂
U+57C2.0	fCNS11643	1-5469
U+57C2.0	fBig5	AE47
U+57C2.0	fGB	0-3921
U+57C2.0	fR/S	 32.7
U+57C2.0	fTotalStrokes	 10
U+57C2.0	fCangjie	GMLK
U+57C2.0	fFourCorner	41146
U+57C2.0	fHakka	 gien5; gang3; gen5
U+57C2.0	fCantonese	gang2
U+57C2.0	fMandarin	geng3; geng1
U+57C2.0	fEnglish	[1] 1] pit; cave; hole [2] irrigation ditch [3] low bank of earth between fields
U+57C3.0	fUTF8	埃
U+57C3.0	fCNS11643	1-546C
U+57C3.0	fBig5	AE4A
U+57C3.0	fGB	0-3023
U+57C3.0	fR/S	 32.7
U+57C3.0	fTotalStrokes	 10
U+57C3.0	fCangjie	GIOK
U+57C3.0	fFourCorner	43134
U+57C3.0	fHakka	  ai1; jai1; zai1
U+57C3.0	fCantonese	oi1; aai1 (also); ngoi1 (also); ngaai1 (also)
U+57C3.0	fMandarin	ai1; zhi4
U+57C3.0	fEnglish	[1] dust; fine dust; dirt [2] dead [3] Egypt
U+57C4.0	fUTF8	埄
U+57C4.0	fCNS11643	3-3034
U+57C4.0	fGB	5-362F
U+57C4.0	fR/S	 32.7
U+57C4.0	fTotalStrokes	 10
U+57C4.0	fCangjie	GHEJ
U+57C4.0	fFourCorner	47150
U+57C4.0	fHakka	  pung2
U+57C4.0	fCantonese	bung6
U+57C4.0	fMandarin	peng2
U+57C4.0	fEnglish	whirling of dust in the wind
U+57C5.0	fUTF8	埅
U+57C5.0	fCNS11643	3-3035
U+57C5.0	fGB	5-3626
U+57C5.0	fR/S	 32.7
U+57C5.0	fTotalStrokes	 10
U+57C5.0	fCangjie	NSG
U+57C5.0	fFourCorner	70100
U+57C5.0	fCantonese	fong4
U+57C5.0	fMandarin	fang2; fang1; di4
U+57C6.0	fUTF8	埆
U+57C6.0	fCNS11643	2-2E51
U+57C6.0	fBig5	D14F
U+57C6.0	fGB	3-3430
U+57C6.0	fR/S	 32.7
U+57C6.0	fTotalStrokes	 10
U+57C6.0	fCangjie	GNBG
U+57C6.0	fFourCorner	47127
U+57C6.0	fHakka	  gok7
U+57C6.0	fCantonese	kok3; gok3
U+57C6.0	fMandarin	que4
U+57C6.0	fEnglish	stony
U+57C7.0	fUTF8	埇
U+57C7.0	fCNS11643	2-2E57
U+57C7.0	fBig5	D155
U+57C7.0	fGB	5-3633
U+57C7.0	fR/S	 32.7
U+57C7.0	fTotalStrokes	 10
U+57C7.0	fCangjie	GNIB
U+57C7.0	fFourCorner	47127
U+57C7.0	fHakka	 jung3; rung3
U+57C7.0	fCantonese	jung2; cung1; tung2
U+57C7.0	fMandarin	yong3
U+57C7.0	fEnglish	name of bridge
U+57C8.0	fUTF8	埈
U+57C8.0	fCNS11643	3-3033
U+57C8.0	fGB	E-255F
U+57C8.0	fR/S	 32.7
U+57C8.0	fTotalStrokes	 10
U+57C8.0	fCangjie	GICE
U+57C8.0	fFourCorner	43140
U+57C8.0	fCantonese	seon3
U+57C8.0	fMandarin	xun4; jun4
U+57C8.0	fEnglish	same as &#x9656;
U+57C9.0	fUTF8	埉
U+57C9.0	fCNS11643	3-3028
U+57C9.0	fGB	5-3579
U+57C9.0	fR/S	  32.7
U+57C9.0	fTotalStrokes	 10
U+57C9.0	fCangjie	GKOO
U+57C9.0	fFourCorner	44130
U+57C9.0	fCantonese	gaap3
U+57C9.0	fMandarin	jia2
U+57CA.0	fUTF8	埊
U+57CA.0	fCNS11643	3-3037
U+57CA.0	fGB	5-3B36
U+57CA.0	fR/S	 32.7; 46.7
U+57CA.0	fTotalStrokes	 10
U+57CA.0	fCangjie	UEG
U+57CA.0	fFourCorner	22100
U+57CA.0	fCantonese	dei6; deng6 (also)
U+57CA.0	fMandarin	di4
U+57CB.0	fUTF8	埋
U+57CB.0	fCNS11643	1-546B
U+57CB.0	fBig5	AE49
U+57CB.0	fGB	0-4271
U+57CB.0	fR/S	 32.7
U+57CB.0	fTotalStrokes	 10
U+57CB.0	fCangjie	GWG
U+57CB.0	fFourCorner	46114
U+57CB.0	fHakka	  mai2; mai3; me2
U+57CB.0	fCantonese	maai4
U+57CB.0	fMandarin	mai2
U+57CB.0	fEnglish	[1] bury [2] secrete; conceal; lie in wait
U+57CB.1	fUTF8	埋
U+57CB.1	fCNS11643	1-546B
U+57CB.1	fBig5	AE49
U+57CB.1	fGB	0-4271
U+57CB.1	fR/S	 32.7
U+57CB.1	fTotalStrokes	 10
U+57CB.1	fCangjie	GWG
U+57CB.1	fFourCorner	46114
U+57CB.1	fMandarin	man2
U+57CB.1	fEnglish	used in combination
U+57CC.0	fUTF8	埌
U+57CC.0	fCNS11643	2-2E4C
U+57CC.0	fBig5	D14A
U+57CC.0	fGB	3-3434
U+57CC.0	fR/S	 32.7
U+57CC.0	fTotalStrokes	 10
U+57CC.0	fCangjie	GIAV
U+57CC.0	fFourCorner	43132
U+57CC.0	fHakka	 long3; long5
U+57CC.0	fCantonese	long6; joeng5
U+57CC.0	fMandarin	lang4
U+57CC.0	fEnglish	waste
U+57CD.0	fUTF8	埍
U+57CD.0	fCNS11643	3-302B
U+57CD.0	fGB	3-342D
U+57CD.0	fR/S	 32.7
U+57CD.0	fTotalStrokes	 10
U+57CD.0	fCangjie	GRB
U+57CD.0	fFourCorner	46120
U+57CD.0	fHakka	 kien3; gien3; gian3; gen3
U+57CD.0	fCantonese	gyun2
U+57CD.0	fMandarin	xuan4
U+57CD.0	fEnglish	[1] police -station [2] jail for women
U+57CE.0	fUTF8	城
U+57CE.0	fCNS11643	1-5052
U+57CE.0	fBig5	ABB0
U+57CE.0	fGB	0-3347
U+57CE.0	fR/S	 32.6; 32.7
U+57CE.0	fTotalStrokes	 9
U+57CE.0	fCangjie	GIHS
U+57CE.0	fFourCorner	43150
U+57CE.0	fHakka	  sang2; s'ang2; s'ang3; shang2
U+57CE.0	fCantonese	sing4; seng4 (also); seng2 (also); sing2 (also)
U+57CE.0	fMandarin	cheng2
U+57CE.0	fEnglish	[1] city; town; municipality [2] castle; the walls of a city; surround a city with walls
U+57CF.0	fUTF8	埏
U+57CF.0	fCNS11643	2-343D
U+57CF.0	fBig5	D4BA
U+57CF.0	fGB	0-5B6F
U+57CF.0	fR/S	 32.7
U+57CF.0	fTotalStrokes	 10
U+57CF.0	fCangjie	GNKM
U+57CF.0	fFourCorner	42141
U+57CF.0	fHakka	 jen2; ren2; jan2
U+57CF.0	fCantonese	jin4; sin1; saan1 (also)
U+57CF.0	fMandarin	yan2; shan1
U+57CF.0	fEnglish	 boundary; limit
U+57D0.0	fUTF8	埐
U+57D0.0	fCNS11643	2-2E58
U+57D0.0	fBig5	D156
U+57D0.0	fGB	3-3435
U+57D0.0	fR/S	 32.7
U+57D0.0	fTotalStrokes	 10
U+57D0.0	fCangjie	GSME
U+57D0.0	fFourCorner	47147
U+57D0.0	fCantonese	zam1
U+57D0.0	fMandarin	jin1
U+57D1.0	fUTF8	埑
U+57D1.0	fCNS11643	3-3036
U+57D1.0	fGB	5-3629
U+57D1.0	fR/S	 32.7
U+57D1.0	fTotalStrokes	 10
U+57D1.0	fCangjie	QLG
U+57D1.0	fFourCorner	52100
U+57D1.0	fHakka	 zet7; zhat7; zat7; zhet7
U+57D1.0	fCantonese	zit3
U+57D1.0	fMandarin	zhe2
U+57D1.0	fEnglish	same as &#x54F2;
U+57D2.0	fUTF8	埒
U+57D2.0	fCNS11643	2-2E4F
U+57D2.0	fBig5	D14D
U+57D2.0	fGB	0-5B78
U+57D2.0	fR/S	 32.7
U+57D2.0	fTotalStrokes	 10
U+57D2.0	fCangjie	GBDI
U+57D2.0	fFourCorner	42149
U+57D2.0	fCantonese	lyut3; lyut6
U+57D2.0	fMandarin	lei4; lie4; le4
U+57D2.0	fEnglish	enclosure; dike; embankment
U+57D3.0	fUTF8	埓
U+57D3.0	fCNS11643	4-2C33
U+57D3.0	fGB	E-2560
U+57D3.0	fR/S	 32.7; 32.6
U+57D3.0	fTotalStrokes	 10
U+57D3.0	fCangjie	GBDI
U+57D3.0	fFourCorner	47140
U+57D3.0	fHakka	  lot7
U+57D3.0	fCantonese	lyut3
U+57D3.0	fMandarin	lie4; le4
U+57D3.0	fEnglish	enclosure; dike; embankment
U+57D4.0	fUTF8	埔
U+57D4.0	fCNS11643	1-546A
U+57D4.0	fBig5	AE48
U+57D4.0	fGB	0-4652
U+57D4.0	fR/S	 32.7
U+57D4.0	fTotalStrokes	 10
U+57D4.0	fCangjie	GIJB
U+57D4.0	fFourCorner	43127
U+57D4.0	fHakka	 bu1; pu1; bu5
U+57D4.0	fCantonese	bou3; pou2
U+57D4.0	fMandarin	pu3; bu4; bu3
U+57D4.0	fEnglish	[1] plain; arena [2] port; market; mart
U+57D5.0	fUTF8	埕
U+57D5.0	fCNS11643	2-2E4E
U+57D5.0	fBig5	D14C
U+57D5.0	fGB	0-5B74
U+57D5.0	fR/S	 32.7
U+57D5.0	fTotalStrokes	 10
U+57D5.0	fCangjie	GRHG
U+57D5.0	fFourCorner	46114
U+57D5.0	fHakka	 chin2; tang5; ch'ang2; cihn2; cin2; chin3; cang2; tang2; chang2
U+57D5.0	fCantonese	cing4
U+57D5.0	fMandarin	cheng2
U+57D5.0	fEnglish	pear-shaped earthenware jar without handles
U+57D6.0	fUTF8	埖
U+57D6.0	fGB	E-2561
U+57D6.0	fR/S	 32.7
U+57D6.0	fTotalStrokes	 10
U+57D6.0	fCangjie	GTOP
U+57D6.0	fCantonese	faa1
U+57D7.0	fUTF8	埗
U+57D7.0	fCNS11643	F-2E5C
U+57D7.0	fGB	5-362A
U+57D7.0	fR/S	 32.7
U+57D7.0	fTotalStrokes	 10
U+57D7.0	fCangjie	GYLH
U+57D7.0	fFourCorner	4112
U+57D7.0	fCantonese	bou6; bou2; bou4
U+57D7.0	fMandarin	bu4
U+57D7.0	fEnglish	same as &#x5E70;
U+57D8.0	fUTF8	埘
U+57D8.0	fGB	0-5B75
U+57D8.0	fR/S	 32.7
U+57D8.0	fTotalStrokes	 10
U+57D8.0	fCangjie	GADI
U+57D8.0	fHakka	 shi2
U+57D8.0	fCantonese	si4
U+57D8.0	fMandarin	shi2
U+57D8.0	fEnglish	simplified form of &#x5852;
U+57D9.0	fUTF8	埙
U+57D9.0	fGB	0-5B77
U+57D9.0	fR/S	 32.7
U+57D9.0	fTotalStrokes	 10
U+57D9.0	fCangjie	GRBO
U+57D9.0	fHakka	 hiun1
U+57D9.0	fCantonese	hyun1; fan1 (also)
U+57D9.0	fMandarin	xun1; xuan1
U+57D9.0	fEnglish	simplified form of &#x5864;
U+57DA.0	fUTF8	埚
U+57DA.0	fGB	0-5B76
U+57DA.0	fR/S	 32.7
U+57DA.0	fTotalStrokes	 10
U+57DA.0	fCangjie	GROB
U+57DA.0	fHakka	 ko1; vo1
U+57DA.0	fCantonese	wo1
U+57DA.0	fMandarin	guo1
U+57DA.0	fEnglish	simplified form of &#x581D;
U+57DB.0	fUTF8	埛
U+57DB.0	fCNS11643	3-3032
U+57DB.0	fGB	E-2562
U+57DB.0	fR/S	 32.7
U+57DB.0	fTotalStrokes	 10
U+57DB.0	fCangjie	GBCR
U+57DB.0	fFourCorner	47120
U+57DB.0	fCantonese	gwing1
U+57DB.0	fMandarin	jiong1
U+57DC.0	fUTF8	埜
U+57DC.0	fCNS11643	2-3434
U+57DC.0	fBig5	D4B1
U+57DC.0	fGB	E-2563
U+57DC.0	fR/S	 32.8
U+57DC.0	fTotalStrokes	 11
U+57DC.0	fCangjie	DDG
U+57DC.0	fFourCorner	44104
U+57DC.0	fHakka	 ja3; ra3; i5
U+57DC.0	fCantonese	je5
U+57DC.0	fMandarin	ye3; shu4
U+57DC.0	fEnglish	 open country; field wilderness
U+57DD.0	fUTF8	埝
U+57DD.0	fCNS11643	3-346E
U+57DD.0	fGB	0-5B7E
U+57DD.0	fR/S	 32.8
U+57DD.0	fTotalStrokes	 11
U+57DD.0	fCangjie	GOIP
U+57DD.0	fFourCorner	48130
U+57DD.0	fHakka	  diam5;  ngiap7
U+57DD.0	fCantonese	nim6; lim6 (also); nap1; lap1 (also)
U+57DD.0	fMandarin	nian4
U+57DD.0	fEnglish	 bank; dike purtuberance from
U+57DE.0	fUTF8	埞
U+57DE.0	fCNS11643	3-3471
U+57DE.0	fGB	5-363D
U+57DE.0	fR/S	 32.8
U+57DE.0	fTotalStrokes	 11
U+57DE.0	fCangjie	GJMO
U+57DE.0	fFourCorner	43180
U+57DE.0	fHakka	 tang5; ti2
U+57DE.0	fCantonese	deng6; tai4
U+57DE.0	fMandarin	di3; di1; ti2; shi2; xi1
U+57DE.0	fEnglish	same as &#x5824;
U+57DF.0	fUTF8	域
U+57DF.0	fCNS11643	1-5951
U+57DF.0	fBig5	B0EC
U+57DF.0	fGB	0-5372
U+57DF.0	fR/S	 32.8
U+57DF.0	fTotalStrokes	 11
U+57DF.0	fCangjie	GIRM
U+57DF.0	fFourCorner	43150
U+57DF.0	fHakka	  wet7; vet7
U+57DF.0	fCantonese	wik6
U+57DF.0	fMandarin	yu4
U+57DF.0	fEnglish	[1] frontier; boundary [2] district; region; country; land; area [3] live; stay
U+57E0.0	fUTF8	埠
U+57E0.0	fCNS11643	1-5955
U+57E0.0	fBig5	B0F0
U+57E0.0	fGB	0-323A
U+57E0.0	fR/S	 32.8
U+57E0.0	fTotalStrokes	 11
U+57E0.0	fCangjie	GHRJ
U+57E0.0	fFourCorner	47147
U+57E0.0	fHakka	 bu1; feu6;  feu5; fiu5; feu3; pu1; fiu3
U+57E0.0	fCantonese	bou6; fau6; fau2 (also)
U+57E0.0	fMandarin	bu4; fu4
U+57E0.0	fEnglish	[1] harbour; port; port city; pier [2] mart on the bank of a river or seacoast
U+57E1.0	fUTF8	埡
U+57E1.0	fCNS11643	2-3444
U+57E1.0	fBig5	D4C1
U+57E1.0	fGB	1-5B6B
U+57E1.0	fR/S	 32.8
U+57E1.0	fTotalStrokes	 11
U+57E1.0	fCangjie	GMLM
U+57E1.0	fFourCorner	41112
U+57E1.0	fCantonese	aa3; ngaa3 (also)
U+57E1.0	fMandarin	ya4; wu3; wu4
U+57E1.0	fEnglish	used in place names
U+57E2.0	fUTF8	埢
U+57E2.0	fCNS11643	2-3432
U+57E2.0	fBig5	D4AF
U+57E2.0	fGB	3-3443
U+57E2.0	fR/S	 32.8
U+57E2.0	fTotalStrokes	 11
U+57E2.0	fCangjie	GFQU
U+57E2.0	fFourCorner	49112
U+57E2.0	fHakka	 gen3; gien3; gian3; gen5
U+57E2.0	fCantonese	gyun3; gyun6; gyun2
U+57E2.0	fMandarin	juan3; quan2
U+57E2.0	fEnglish	wall or mound round a grave
U+57E3.0	fUTF8	埣
U+57E3.0	fCNS11643	2-3440
U+57E3.0	fBig5	D4BD
U+57E3.0	fGB	5-363B
U+57E3.0	fR/S	 32.8
U+57E3.0	fTotalStrokes	 11
U+57E3.0	fCangjie	GYOJ
U+57E3.0	fFourCorner	40148
U+57E3.0	fCantonese	seoi3; seoi4; seot1
U+57E3.0	fMandarin	sui4; su4
U+57E4.0	fUTF8	埤
U+57E4.0	fCNS11643	1-5956
U+57E4.0	fBig5	B0F1
U+57E4.0	fGB	0-5B7D
U+57E4.0	fR/S	 32.8
U+57E4.0	fTotalStrokes	 11
U+57E4.0	fCangjie	GHWJ
U+57E4.0	fFourCorner	46140
U+57E4.0	fHakka	  bi1
U+57E4.0	fCantonese	pei4; bei1; bai3 (also); pai6; bei3 (also); pai3
U+57E4.0	fMandarin	pi2; bi4; pi4
U+57E4.0	fEnglish	[1] add to; increase [2] low fence; low wall; parapet
U+57E4.1	fUTF8	埤
U+57E4.1	fCNS11643	1-5956
U+57E4.1	fBig5	B0F1
U+57E4.1	fGB	0-5B7D
U+57E4.1	fR/S	 32.8
U+57E4.1	fTotalStrokes	 11
U+57E4.1	fCangjie	GHWJ
U+57E4.1	fFourCorner	46140
U+57E4.1	fMandarin	bei1
U+57E4.1	fEnglish	low-lying
U+57E5.0	fUTF8	埥
U+57E5.0	fCNS11643	2-3442
U+57E5.0	fBig5	D4BF
U+57E5.0	fGB	5-3634
U+57E5.0	fR/S	 32.8
U+57E5.0	fTotalStrokes	 11
U+57E5.0	fCangjie	GQMB
U+57E5.0	fFourCorner	45127
U+57E5.0	fCantonese	cing1; zang1
U+57E5.0	fMandarin	cheng1; zheng1
U+57E6.0	fUTF8	埦
U+57E6.0	fCNS11643	3-3472
U+57E6.0	fGB	5-363E
U+57E6.0	fR/S	  32.8
U+57E6.0	fTotalStrokes	 11
U+57E6.0	fCangjie	GJNU
U+57E6.0	fFourCorner	43110
U+57E6.0	fCantonese	wun2
U+57E6.0	fMandarin	wan3
U+57E6.0	fEnglish	a bowl
U+57E7.0	fUTF8	埧
U+57E7.0	fCNS11643	2-3448
U+57E7.0	fBig5	D4C5
U+57E7.0	fGB	3-343B
U+57E7.0	fR/S	 32.8
U+57E7.0	fTotalStrokes	 11
U+57E7.0	fCangjie	GBMC
U+57E7.0	fFourCorner	47181
U+57E7.0	fHakka	  ki3;  gi3
U+57E7.0	fCantonese	geoi6; baa3; zik6
U+57E7.0	fMandarin	ju4
U+57E7.0	fEnglish	dam
U+57E8.0	fUTF8	埨
U+57E8.0	fCNS11643	3-3474
U+57E8.0	fGB	3-336B
U+57E8.0	fR/S	  32.8
U+57E8.0	fTotalStrokes	 11
U+57E8.0	fCangjie	GOMB
U+57E8.0	fFourCorner	48120
U+57E8.0	fHakka	 lun3
U+57E8.0	fCantonese	leon4
U+57E8.0	fMandarin	lun3; lun4
U+57E9.0	fUTF8	埩
U+57E9.0	fCNS11643	2-344C
U+57E9.0	fBig5	D4C9
U+57E9.0	fGB	3-3426
U+57E9.0	fR/S	 32.8
U+57E9.0	fTotalStrokes	 11
U+57E9.0	fCangjie	GBSD
U+57E9.0	fFourCorner	42157
U+57E9.0	fCantonese	zang1; caang4
U+57E9.0	fMandarin	zheng1; cheng2
U+57EA.0	fUTF8	埪
U+57EA.0	fCNS11643	3-3469
U+57EA.0	fGB	3-3446
U+57EA.0	fR/S	 32.9; 32.8
U+57EA.0	fTotalStrokes	 11; 12
U+57EA.0	fCangjie	GJCM
U+57EA.0	fFourCorner	43110
U+57EA.0	fCantonese	hung1
U+57EA.0	fMandarin	kong1
U+57EB.0	fUTF8	埫
U+57EB.0	fCNS11643	4-307B
U+57EB.0	fGB	5-363A
U+57EB.0	fR/S	  32.8
U+57EB.0	fTotalStrokes	 11
U+57EB.0	fCangjie	GFBR
U+57EB.0	fFourCorner	49120
U+57EB.0	fCantonese	cung2
U+57EB.0	fMandarin	chong3; shang3
U+57EC.0	fUTF8	埬
U+57EC.0	fCNS11643	2-3443
U+57EC.0	fBig5	D4C0
U+57EC.0	fGB	5-356A
U+57EC.0	fR/S	 32.8
U+57EC.0	fTotalStrokes	 11
U+57EC.0	fCangjie	GDW
U+57EC.0	fFourCorner	45196
U+57EC.0	fCantonese	dung1
U+57EC.0	fMandarin	dong1
U+57ED.0	fUTF8	埭
U+57ED.0	fCNS11643	2-3437
U+57ED.0	fBig5	D4B4
U+57ED.0	fGB	0-5C24
U+57ED.0	fR/S	 32.8
U+57ED.0	fTotalStrokes	 11
U+57ED.0	fCangjie	GLE
U+57ED.0	fFourCorner	45132
U+57ED.0	fHakka	 ti5; tai1; ti6
U+57ED.0	fCantonese	dai6; doi6 (also)
U+57ED.0	fMandarin	dai4
U+57ED.0	fEnglish	 dam; jock inclined plane on a canal; where boats can be hauled up or down
U+57EE.0	fUTF8	埮
U+57EE.0	fCNS11643	2-343F
U+57EE.0	fBig5	D4BC
U+57EE.0	fGB	3-3445
U+57EE.0	fR/S	 32.8
U+57EE.0	fTotalStrokes	 11
U+57EE.0	fCangjie	GFF
U+57EE.0	fFourCorner	49189
U+57EE.0	fCantonese	taam3; daam6; taam4
U+57EE.0	fMandarin	tan4; tan2
U+57EF.0	fUTF8	埯
U+57EF.0	fCNS11643	3-346B
U+57EF.0	fGB	0-5B7B
U+57EF.0	fR/S	  32.8
U+57EF.0	fTotalStrokes	 11
U+57EF.0	fCangjie	GKLU
U+57EF.0	fFourCorner	44110
U+57EF.0	fHakka	 jam3; ram3
U+57EF.0	fCantonese	am2; ngam2 (also); jim2
U+57EF.0	fMandarin	yan3; an3
U+57EF.0	fEnglish	 cover with earth pit hole
U+57F0.0	fUTF8	埰
U+57F0.0	fCNS11643	2-344D
U+57F0.0	fBig5	D4CA
U+57F0.0	fGB	3-3440
U+57F0.0	fR/S	 32.8
U+57F0.0	fTotalStrokes	 11
U+57F0.0	fCangjie	GBD
U+57F0.0	fFourCorner	42194
U+57F0.0	fHakka	 cai5; coi5; cai3
U+57F0.0	fCantonese	coi3
U+57F0.0	fMandarin	cai4
U+57F0.0	fEnglish	fief
U+57F1.0	fUTF8	埱
U+57F1.0	fCNS11643	2-344B
U+57F1.0	fBig5	D4C8
U+57F1.0	fGB	5-3638
U+57F1.0	fR/S	 32.8
U+57F1.0	fTotalStrokes	 11
U+57F1.0	fCangjie	GYFE
U+57F1.0	fFourCorner	47140
U+57F1.0	fCantonese	cuk1; tau3
U+57F1.0	fMandarin	shu2; tou4
U+57F2.0	fUTF8	埲
U+57F2.0	fCNS11643	2-3441
U+57F2.0	fBig5	D4BE
U+57F2.0	fGB	3-3437
U+57F2.0	fR/S	 32.8
U+57F2.0	fTotalStrokes	 11
U+57F2.0	fCangjie	GQKQ
U+57F2.0	fFourCorner	45153
U+57F2.0	fCantonese	bung6; bong6; pung1
U+57F2.0	fMandarin	beng3; bang4
U+57F2.0	fEnglish	classifier for walls
U+57F3.0	fUTF8	埳
U+57F3.0	fCNS11643	2-343C
U+57F3.0	fBig5	D4B9
U+57F3.0	fGB	E-2564
U+57F3.0	fR/S	 32.8
U+57F3.0	fTotalStrokes	 11
U+57F3.0	fCangjie	GNHX
U+57F3.0	fFourCorner	47177
U+57F3.0	fHakka	 kam3
U+57F3.0	fCantonese	ham2; ham3; ham1 (also)
U+57F3.0	fMandarin	kan3
U+57F3.0	fEnglish	same as &#x574E;
U+57F4.0	fUTF8	埴
U+57F4.0	fCNS11643	2-3435
U+57F4.0	fBig5	D4B2
U+57F4.0	fGB	0-5B7A
U+57F4.0	fR/S	 32.8
U+57F4.0	fTotalStrokes	 11
U+57F4.0	fCangjie	GJBM
U+57F4.0	fFourCorner	44116
U+57F4.0	fHakka	 chit7; shit7; ts'iit7; siit7
U+57F4.0	fCantonese	zik6
U+57F4.0	fMandarin	zhi2
U+57F4.0	fEnglish	soil with large clay content
U+57F5.0	fUTF8	埵
U+57F5.0	fCNS11643	2-3A68
U+57F5.0	fBig5	D8A6
U+57F5.0	fGB	3-343E
U+57F5.0	fR/S	 32.8
U+57F5.0	fTotalStrokes	 11
U+57F5.0	fCangjie	GHJM
U+57F5.0	fFourCorner	42114
U+57F5.0	fHakka	  do3
U+57F5.0	fCantonese	do2; do3
U+57F5.0	fMandarin	duo3
U+57F5.0	fEnglish	 hardened dirt or clay cluster
U+57F6.0	fUTF8	埶
U+57F6.0	fCNS11643	2-3433
U+57F6.0	fBig5	D4B0
U+57F6.0	fGB	3-3439
U+57F6.0	fR/S	 32.8
U+57F6.0	fTotalStrokes	 11
U+57F6.0	fCangjie	GGKNI
U+57F6.0	fFourCorner	45117
U+57F6.0	fHakka	 ngi5; ngi6
U+57F6.0	fCantonese	ngai6; sai3; ai6 (also); zap1
U+57F6.0	fMandarin	yi4; shi4
U+57F6.0	fEnglish	art
U+57F7.0	fUTF8	執
U+57F7.0	fCNS11643	1-595A
U+57F7.0	fBig5	B0F5
U+57F7.0	fGB	1-5634
U+57F7.0	fR/S	 32.8
U+57F7.0	fTotalStrokes	 11
U+57F7.0	fCangjie	GJKNI
U+57F7.0	fFourCorner	45417
U+57F7.0	fHakka	 zip7;  chip7; tsiip7; zhip7
U+57F7.0	fCantonese	zap1
U+57F7.0	fMandarin	zhi2
U+57F7.0	fEnglish	[1] hold in hand; keep; grasp; seize [2] detain; arrest [3] maintain or uphold a principle, etc.); hold on stubbornly to; stick to; persist in [4] shut up (gossipers etc.) [5] a Chinese family name
U+57F8.0	fUTF8	埸
U+57F8.0	fCNS11643	2-343A
U+57F8.0	fBig5	D4B7
U+57F8.0	fGB	0-5B7C
U+57F8.0	fR/S	 32.8
U+57F8.0	fTotalStrokes	 11
U+57F8.0	fCangjie	GAPH
U+57F8.0	fFourCorner	46127
U+57F8.0	fHakka	 jit8; rit8
U+57F8.0	fCantonese	jik6
U+57F8.0	fMandarin	yi4; tan4
U+57F8.0	fEnglish	[1] border; limit; dike; frontier; boundary [2] row in a field [3] time
U+57F9.0	fUTF8	培
U+57F9.0	fCNS11643	1-595B
U+57F9.0	fBig5	B0F6
U+57F9.0	fGB	0-4560
U+57F9.0	fR/S	 32.8
U+57F9.0	fTotalStrokes	 11
U+57F9.0	fCangjie	GYTR
U+57F9.0	fFourCorner	40161
U+57F9.0	fHakka	 pui2; pi2; poi2
U+57F9.0	fCantonese	pui4; pau2; bau6
U+57F9.0	fMandarin	pei2; pou3; bei4; fou2; fu3
U+57F9.0	fEnglish	[1] bank up with dirt or earth [2] nourish; strengthen; cultivate; foster
U+57FA.0	fUTF8	基
U+57FA.0	fCNS11643	1-5957
U+57FA.0	fBig5	B0F2
U+57FA.0	fGB	0-3B79
U+57FA.0	fR/S	 32.8
U+57FA.0	fTotalStrokes	 11
U+57FA.0	fCangjie	TCG
U+57FA.0	fFourCorner	44104
U+57FA.0	fHakka	  gi1
U+57FA.0	fCantonese	gei1
U+57FA.0	fMandarin	ji1
U+57FA.0	fEnglish	[1] foundation; base [2] origin; basis; root [3] on the basis of; according to; on the strength of [4] group; the base of a chemical compound
U+57FB.0	fUTF8	埻
U+57FB.0	fCNS11643	2-3430
U+57FB.0	fBig5	D4AD
U+57FB.0	fGB	3-3441
U+57FB.0	fR/S	 32.8
U+57FB.0	fTotalStrokes	 11
U+57FB.0	fCangjie	GYRD
U+57FB.0	fFourCorner	40147
U+57FB.0	fHakka	 zhun3; zun3
U+57FB.0	fCantonese	zeon2; deoi1
U+57FB.0	fMandarin	zhun3; guo1; dui1; duo1
U+57FB.0	fEnglish	target
U+57FC.0	fUTF8	埼
U+57FC.0	fCNS11643	2-3446
U+57FC.0	fBig5	D4C3
U+57FC.0	fGB	3-343A
U+57FC.0	fR/S	 32.8
U+57FC.0	fTotalStrokes	 11
U+57FC.0	fCangjie	GKMR
U+57FC.0	fFourCorner	44121
U+57FC.0	fCantonese	kei4
U+57FC.0	fMandarin	qi2
U+57FC.0	fEnglish	headland
U+57FD.0	fUTF8	埽
U+57FD.0	fCNS11643	2-3438
U+57FD.0	fBig5	D4B5
U+57FD.0	fGB	0-5C23
U+57FD.0	fR/S	 32.8
U+57FD.0	fTotalStrokes	 11
U+57FD.0	fCangjie	GSMB
U+57FD.0	fFourCorner	47127
U+57FD.0	fHakka	 so5; sau5
U+57FD.0	fCantonese	sou3; sou2
U+57FD.0	fMandarin	sao3; sao4
U+57FD.0	fEnglish	 broom sweep; clear away
U+57FE.0	fUTF8	埾
U+57FE.0	fCNS11643	3-346F
U+57FE.0	fGB	5-3635
U+57FE.0	fR/S	  32.8
U+57FE.0	fTotalStrokes	 11
U+57FE.0	fCangjie	SEG
U+57FE.0	fFourCorner	11104
U+57FE.0	fHakka	 ci5; ci6
U+57FE.0	fCantonese	zeoi6
U+57FE.0	fMandarin	ju4
U+57FE.0	fEnglish	 pile up earth
U+57FF.0	fUTF8	埿
U+57FF.0	fCNS11643	3-3475
U+57FF.0	fGB	5-363C
U+57FF.0	fR/S	  32.8
U+57FF.0	fTotalStrokes	 11
U+57FF.0	fCangjie	EPG
U+57FF.0	fFourCorner	37100
U+57FF.0	fHakka	 ni2; nai2
U+57FF.0	fCantonese	nai4; lai4 (also); baan6
U+57FF.0	fMandarin	ni2; ni4; ban4
U+5800.0	fUTF8	堀
U+5800.0	fCNS11643	2-3436
U+5800.0	fBig5	D4B3
U+5800.0	fGB	0-5C25
U+5800.0	fR/S	 32.8
U+5800.0	fTotalStrokes	 11
U+5800.0	fCangjie	GSUU
U+5800.0	fFourCorner	47172
U+5800.0	fHakka	 kut7; kiut7; fut7; kwut7
U+5800.0	fCantonese	fat1; gwat6 (also)
U+5800.0	fMandarin	ku1; jue2
U+5800.0	fEnglish	cave; hole
U+5801.0	fUTF8	堁
U+5801.0	fCNS11643	2-3449
U+5801.0	fBig5	D4C6
U+5801.0	fGB	3-343C
U+5801.0	fR/S	 32.8
U+5801.0	fTotalStrokes	 11
U+5801.0	fCangjie	GWD
U+5801.0	fFourCorner	46194
U+5801.0	fHakka	  ko5
U+5801.0	fCantonese	fo3; fo2 (also)
U+5801.0	fMandarin	ke4; ke3; kuo4
U+5802.0	fUTF8	堂
U+5802.0	fCNS11643	1-5958
U+5802.0	fBig5	B0F3
U+5802.0	fGB	0-4C43
U+5802.0	fR/S	 32.8
U+5802.0	fTotalStrokes	 11
U+5802.0	fCangjie	FBRG
U+5802.0	fFourCorner	90104
U+5802.0	fHakka	  tong2; tong3; to2
U+5802.0	fCantonese	tong4; tong2 (also)
U+5802.0	fMandarin	tang2
U+5802.0	fEnglish	 hall; government office; office; reception
U+5802.1	fUTF8	堂
U+5802.1	fCNS11643	1-5958
U+5802.1	fBig5	B0F3
U+5802.1	fGB	0-4C43
U+5802.1	fR/S	 32.8
U+5802.1	fTotalStrokes	 11
U+5802.1	fCangjie	FBRG
U+5802.1	fFourCorner	90104
U+5802.1	fMandarin	tang5
U+5803.0	fUTF8	堃
U+5803.0	fCNS11643	3-3476
U+5803.0	fGB	8-2F69
U+5803.0	fR/S	  32.8
U+5803.0	fTotalStrokes	 11
U+5803.0	fCangjie	YSG
U+5803.0	fFourCorner	00100
U+5803.0	fHakka	 kun1; kwun1
U+5803.0	fCantonese	kwan1
U+5803.0	fMandarin	kun1
U+5803.0	fEnglish	same as &#x5764;
U+5804.0	fUTF8	堄
U+5804.0	fCNS11643	2-344F
U+5804.0	fBig5	D4CC
U+5804.0	fGB	3-343F
U+5804.0	fR/S	 32.8
U+5804.0	fTotalStrokes	 11
U+5804.0	fCangjie	GHXU
U+5804.0	fFourCorner	47117
U+5804.0	fCantonese	ngai6; ai6 (also); ngai4
U+5804.0	fMandarin	ni4; ni3; yi4
U+5804.0	fEnglish	parapet
U+5805.0	fUTF8	堅
U+5805.0	fCNS11643	1-5952
U+5805.0	fBig5	B0ED
U+5805.0	fGB	1-3C61
U+5805.0	fR/S	 32.8
U+5805.0	fTotalStrokes	 11
U+5805.0	fCangjie	SEG
U+5805.0	fFourCorner	77104
U+5805.0	fHakka	  gien1;  gen1; gien2; kian1; zin1; gian1
U+5805.0	fCantonese	gin1
U+5805.0	fMandarin	jian1
U+5805.0	fEnglish	[1] strong; durable; [2] hard; solid; firm [3] dedicate to; devote to [4] calm; steady; stable; determined [5] close; intimate [6] armor [7] the strongest point or position of enemy troops [8] firmly; steadfastly; resolutely [9] a Chinese family name
U+5806.0	fUTF8	堆
U+5806.0	fCNS11643	1-5954
U+5806.0	fBig5	B0EF
U+5806.0	fGB	0-3651
U+5806.0	fR/S	 32.8
U+5806.0	fTotalStrokes	 11
U+5806.0	fCangjie	GOG
U+5806.0	fFourCorner	40114
U+5806.0	fHakka	  doi1;  doi3; doi5
U+5806.0	fCantonese	deoi1; zeoi1
U+5806.0	fMandarin	dui1; zui1
U+5806.0	fEnglish	[1] heap; pile; crowd; mass [2] heap up; pile up; stack
U+5807.0	fUTF8	堇
U+5807.0	fCNS11643	2-343E
U+5807.0	fBig5	D4BB
U+5807.0	fGB	0-5D40
U+5807.0	fR/S	 32.8
U+5807.0	fTotalStrokes	 11
U+5807.0	fCangjie	TLQM
U+5807.0	fFourCorner	44104
U+5807.0	fHakka	 giun3; giun5
U+5807.0	fCantonese	gan2; rg ; kan4; gan6
U+5807.0	fMandarin	jin3; qin2
U+5807.0	fEnglish	 yellow loam clay season few
U+5808.0	fUTF8	堈
U+5808.0	fCNS11643	2-3439
U+5808.0	fBig5	D4B6
U+5808.0	fGB	3-3367
U+5808.0	fR/S	 32.8
U+5808.0	fTotalStrokes	 11
U+5808.0	fCangjie	GBTU
U+5808.0	fFourCorner	47120
U+5808.0	fHakka	  gong1
U+5808.0	fCantonese	gong1
U+5808.0	fMandarin	gang1
U+5808.0	fEnglish	earthen jug; crock; cistern
U+5809.0	fUTF8	堉
U+5809.0	fCNS11643	1-546D
U+5809.0	fBig5	AE4B
U+5809.0	fGB	3-3442
U+5809.0	fR/S	 32.8
U+5809.0	fTotalStrokes	 11
U+5809.0	fCangjie	GYIB
U+5809.0	fFourCorner	44127
U+5809.0	fCantonese	juk6
U+5809.0	fMandarin	yu4
U+5809.0	fEnglish	ground; fertile land
U+580A.0	fUTF8	堊
U+580A.0	fCNS11643	1-5953
U+580A.0	fBig5	B0EE
U+580A.0	fGB	1-5B51
U+580A.0	fR/S	 32.8
U+580A.0	fTotalStrokes	 11
U+580A.0	fCangjie	MMG
U+580A.0	fFourCorner	10104
U+580A.0	fHakka	 ok7
U+580A.0	fCantonese	ok3; ngok3 (also)
U+580A.0	fMandarin	e4; wu4
U+580A.0	fEnglish	 white earth daub with white mud
U+580B.0	fUTF8	堋
U+580B.0	fCNS11643	2-343B
U+580B.0	fBig5	D4B8
U+580B.0	fGB	0-5C21
U+580B.0	fR/S	 32.8
U+580B.0	fTotalStrokes	 11
U+580B.0	fCangjie	GBB
U+580B.0	fFourCorner	47120
U+580B.0	fHakka	  pang5;  pang2
U+580B.0	fCantonese	pang4; bang1 (also); bang6; baang6; bing3; paang4
U+580B.0	fMandarin	peng2; beng4; peng4
U+580B.0	fEnglish	bury
U+580C.0	fUTF8	堌
U+580C.0	fCNS11643	2-344A
U+580C.0	fBig5	D4C7
U+580C.0	fGB	3-343D
U+580C.0	fR/S	 32.8
U+580C.0	fTotalStrokes	 11
U+580C.0	fCangjie	GWJR
U+580C.0	fFourCorner	46100
U+580C.0	fHakka	 gu5
U+580C.0	fCantonese	gu3; gwu3 (also)
U+580C.0	fMandarin	gu4
U+580C.0	fEnglish	 dam; dike used in place names
U+580D.0	fUTF8	堍
U+580D.0	fCNS11643	2-344E
U+580D.0	fBig5	D4CB
U+580D.0	fGB	0-5C22
U+580D.0	fR/S	 32.8
U+580D.0	fTotalStrokes	 11
U+580D.0	fCangjie	GNAI
U+580D.0	fFourCorner	47113
U+580D.0	fCantonese	tou3
U+580D.0	fMandarin	tu4
U+580E.0	fUTF8	堎
U+580E.0	fCNS11643	2-3445
U+580E.0	fBig5	D4C2
U+580E.0	fGB	3-3438
U+580E.0	fR/S	 32.8
U+580E.0	fTotalStrokes	 11
U+580E.0	fCangjie	GGCE
U+580E.0	fFourCorner	44147
U+580E.0	fCantonese	ling4; ling6
U+580E.0	fMandarin	leng4; ling2
U+580E.0	fEnglish	used for place names
U+580F.0	fUTF8	堏
U+580F.0	fCNS11643	3-6461
U+580F.0	fGB	E-2565
U+580F.0	fR/S	 32.8
U+580F.0	fTotalStrokes	 11
U+580F.0	fCangjie	DSG
U+580F.0	fFourCorner	4010
U+580F.0	fCantonese	fong4
U+5810.0	fUTF8	堐
U+5810.0	fCNS11643	2-3447
U+5810.0	fBig5	D4C4
U+5810.0	fGB	E-2566
U+5810.0	fR/S	 32.8
U+5810.0	fTotalStrokes	 11
U+5810.0	fCangjie	GMGG
U+5810.0	fFourCorner	41114
U+5810.0	fCantonese	ngaai4; aai4 (also)
U+5810.0	fMandarin	ya2
U+5811.0	fUTF8	堑
U+5811.0	fGB	0-4735
U+5811.0	fR/S	 32.8
U+5811.0	fTotalStrokes	 11
U+5811.0	fCangjie	XKLG
U+5811.0	fHakka	 ciam5
U+5811.0	fCantonese	cim3
U+5811.0	fMandarin	qian4
U+5811.0	fEnglish	simplified form of &#x5879;
U+5812.0	fUTF8	堒
U+5812.0	fCNS11643	3-6462
U+5812.0	fGB	E-2567
U+5812.0	fR/S	 32.8
U+5812.0	fTotalStrokes	 11
U+5812.0	fCangjie	GAPP
U+5812.0	fFourCorner	4611
U+5812.0	fCantonese	kwan1
U+5813.0	fUTF8	堓
U+5813.0	fCNS11643	3-6463
U+5813.0	fGB	E-2568
U+5813.0	fR/S	 32.8
U+5813.0	fTotalStrokes	 11
U+5813.0	fCangjie	GUMJ
U+5813.0	fFourCorner	4214
U+5813.0	fCantonese	ngon6; on6 (also)
U+5813.0	fMandarin	an4
U+5814.0	fUTF8	堔
U+5814.0	fCNS11643	2-3431
U+5814.0	fBig5	D4AE
U+5814.0	fGB	E-2569
U+5814.0	fR/S	 32.8
U+5814.0	fTotalStrokes	 11
U+5814.0	fCangjie	GBCD
U+5814.0	fFourCorner	47191
U+5814.0	fCantonese	sam1
U+5814.0	fMandarin	chen1
U+5815.0	fUTF8	堕
U+5815.0	fGB	0-3669
U+5815.0	fR/S	 32.8; 32.9
U+5815.0	fTotalStrokes	 11
U+5815.0	fCangjie	XXNBG
U+5815.0	fHakka	 to5
U+5815.0	fCantonese	do6; fai1
U+5815.0	fMandarin	duo4; hui1; hui3; tuo3
U+5815.0	fEnglish	simplified form of &#x58AE;
U+5816.0	fUTF8	堖
U+5816.0	fGB	1-5B71
U+5816.0	fR/S	 32.9
U+5816.0	fTotalStrokes	 12
U+5816.0	fCangjie	GVVW
U+5816.0	fHakka	 nau3; no3
U+5816.0	fCantonese	nou5; lou5 (also)
U+5816.0	fMandarin	nao3
U+5816.0	fEnglish	small; head shaped hill; used in place names
U+5817.0	fUTF8	堗
U+5817.0	fCNS11643	4-3669
U+5817.0	fGB	5-364F
U+5817.0	fR/S	  32.9
U+5817.0	fTotalStrokes	 12
U+5817.0	fCangjie	GJCK
U+5817.0	fFourCorner	43130
U+5817.0	fHakka	  tut7
U+5817.0	fCantonese	dat6
U+5817.0	fMandarin	tu1; tu2
U+5817.0	fEnglish	the flue of a furnace
U+5818.0	fUTF8	堘
U+5818.0	fCNS11643	3-3A4A
U+5818.0	fGB	E-256A
U+5818.0	fR/S	  32.9
U+5818.0	fTotalStrokes	 12
U+5818.0	fCangjie	GFQG
U+5818.0	fFourCorner	49110
U+5818.0	fHakka	 shin2; shun2; siin2; shun3; sun2
U+5818.0	fCantonese	sing4
U+5818.0	fMandarin	cheng2
U+5818.0	fEnglish	a raised path between field which acts as a dyke
U+5819.0	fUTF8	堙
U+5819.0	fCNS11643	2-3A63
U+5819.0	fBig5	D8A1
U+5819.0	fGB	0-5C27
U+5819.0	fR/S	 32.9
U+5819.0	fTotalStrokes	 12
U+5819.0	fCangjie	GMWG
U+5819.0	fFourCorner	41114
U+5819.0	fHakka	  jin1; jen1; ren1; rin1
U+5819.0	fCantonese	jan1
U+5819.0	fMandarin	yin1
U+5819.0	fEnglish	 bury dam; block up
U+581A.0	fUTF8	堚
U+581A.0	fCNS11643	3-3A45
U+581A.0	fGB	3-342A
U+581A.0	fR/S	  32.9
U+581A.0	fTotalStrokes	 12
U+581A.0	fCangjie	GBJJ
U+581A.0	fFourCorner	47150
U+581A.0	fMandarin	hun2
U+581A.0	fEnglish	same as &#x21366;
U+581B.0	fUTF8	堛
U+581B.0	fCNS11643	2-3A6C
U+581B.0	fBig5	D8AA
U+581B.0	fGB	5-3641
U+581B.0	fR/S	 32.9
U+581B.0	fTotalStrokes	 12
U+581B.0	fCangjie	GMRW
U+581B.0	fFourCorner	41166
U+581B.0	fCantonese	pik1; pei3
U+581B.0	fMandarin	bi4
U+581C.0	fUTF8	堜
U+581C.0	fCNS11643	2-3A6B
U+581C.0	fBig5	D8A9
U+581C.0	fGB	E-256B
U+581C.0	fR/S	 32.9
U+581C.0	fTotalStrokes	 12
U+581C.0	fCangjie	GDWF
U+581C.0	fFourCorner	45196
U+581C.0	fCantonese	lin6
U+581C.0	fMandarin	lian4; fei4
U+581D.0	fUTF8	堝
U+581D.0	fCNS11643	1-5E60
U+581D.0	fBig5	B3FA
U+581D.0	fGB	1-5B76
U+581D.0	fR/S	 32.9
U+581D.0	fTotalStrokes	 12
U+581D.0	fCangjie	GBBR
U+581D.0	fFourCorner	47127
U+581D.0	fHakka	  ko1; wo1; kui1; vo1
U+581D.0	fCantonese	wo1; gwo1
U+581D.0	fMandarin	guo1
U+581D.0	fEnglish	crucible
U+581E.0	fUTF8	堞
U+581E.0	fCNS11643	2-3A64
U+581E.0	fBig5	D8A2
U+581E.0	fGB	0-5C26
U+581E.0	fR/S	 32.9
U+581E.0	fTotalStrokes	 12
U+581E.0	fCangjie	GPTD
U+581E.0	fFourCorner	44194
U+581E.0	fHakka	  tiap8
U+581E.0	fCantonese	dip6
U+581E.0	fMandarin	die2
U+581E.0	fEnglish	plate
U+581F.0	fUTF8	堟
U+581F.0	fCNS11643	3-3A47
U+581F.0	fGB	5-3651
U+581F.0	fR/S	  32.9
U+581F.0	fTotalStrokes	 12
U+581F.0	fCangjie	GVNO
U+581F.0	fFourCorner	47130
U+581F.0	fHakka	 chon5; con5
U+581F.0	fCantonese	zyun6
U+581F.0	fMandarin	zhuan4
U+581F.0	fEnglish	 plough
U+5820.0	fUTF8	堠
U+5820.0	fCNS11643	1-5E61
U+5820.0	fBig5	B3FB
U+5820.0	fGB	0-5C29
U+5820.0	fR/S	 32.9
U+5820.0	fTotalStrokes	 12
U+5820.0	fCangjie	GONK
U+5820.0	fFourCorner	47134
U+5820.0	fHakka	 heu5; heu6; hiu5
U+5820.0	fCantonese	hau6
U+5820.0	fMandarin	hou4
U+5820.0	fEnglish	battlements; battlemented walls
U+5821.0	fUTF8	堡
U+5821.0	fCNS11643	1-5E5F
U+5821.0	fBig5	B3F9
U+5821.0	fGB	0-3124
U+5821.0	fR/S	 32.9
U+5821.0	fTotalStrokes	 12
U+5821.0	fCangjie	ODG
U+5821.0	fFourCorner	26104
U+5821.0	fHakka	  bau3;  bo3
U+5821.0	fCantonese	bou2; pou3
U+5821.0	fMandarin	bao3; bu3; pu4
U+5821.0	fEnglish	[1] walled village; town [2] petty military station; fort; fortress
U+5822.0	fUTF8	堢
U+5822.0	fCNS11643	3-3A50
U+5822.0	fGB	5-364B
U+5822.0	fR/S	 32.9
U+5822.0	fTotalStrokes	 12
U+5822.0	fCangjie	GORD
U+5822.0	fFourCorner	46190
U+5822.0	fCantonese	bou2
U+5822.0	fMandarin	bao3
U+5823.0	fUTF8	堣
U+5823.0	fCNS11643	2-3A66
U+5823.0	fBig5	D8A4
U+5823.0	fGB	5-3645
U+5823.0	fR/S	 32.9
U+5823.0	fTotalStrokes	 12
U+5823.0	fCangjie	GWLB
U+5823.0	fFourCorner	46127
U+5823.0	fCantonese	jyu4
U+5823.0	fMandarin	yu2
U+5824.0	fUTF8	堤
U+5824.0	fCNS11643	1-5E5C
U+5824.0	fBig5	B3F6
U+5824.0	fGB	0-354C
U+5824.0	fR/S	 32.9
U+5824.0	fTotalStrokes	 12
U+5824.0	fCangjie	GAMO
U+5824.0	fFourCorner	46181
U+5824.0	fHakka	  ti2
U+5824.0	fCantonese	tai4
U+5824.0	fMandarin	di1; ti2; di3; shi2; xi1
U+5824.0	fEnglish	dike
U+5825.0	fUTF8	堥
U+5825.0	fCNS11643	2-3A6A
U+5825.0	fBig5	D8A8
U+5825.0	fGB	3-3458
U+5825.0	fR/S	 32.9
U+5825.0	fTotalStrokes	 12
U+5825.0	fCangjie	NKG
U+5825.0	fFourCorner	18104
U+5825.0	fCantonese	mou4; mau4; mou5
U+5825.0	fMandarin	mao2; mao1; mou2; wu3
U+5826.0	fUTF8	堦
U+5826.0	fCNS11643	3-3A4C
U+5826.0	fGB	E-256C
U+5826.0	fR/S	  32.9
U+5826.0	fTotalStrokes	 12
U+5826.0	fCangjie	GPPA
U+5826.0	fFourCorner	41160
U+5826.0	fHakka	 gai1; giai1
U+5826.0	fCantonese	gaai1
U+5826.0	fMandarin	jie1
U+5826.0	fEnglish	same as &#x968E;
U+5827.0	fUTF8	堧
U+5827.0	fCNS11643	2-3A65
U+5827.0	fBig5	D8A3
U+5827.0	fGB	5-3643
U+5827.0	fR/S	 32.9
U+5827.0	fTotalStrokes	 12
U+5827.0	fCangjie	GMBK
U+5827.0	fFourCorner	41134
U+5827.0	fHakka	 ngion1; no5; no6; nion1
U+5827.0	fCantonese	jyun4
U+5827.0	fMandarin	ruan2; nuan4; nuo4
U+5827.0	fEnglish	adjoin
U+5828.0	fUTF8	堨
U+5828.0	fCNS11643	2-3A67
U+5828.0	fBig5	D8A5
U+5828.0	fGB	3-344D
U+5828.0	fR/S	 32.9
U+5828.0	fTotalStrokes	 12
U+5828.0	fCangjie	GAPV
U+5828.0	fFourCorner	46127
U+5828.0	fHakka	 ot7;  giet7
U+5828.0	fCantonese	aat3; ngaat3 (also); oi2; ngoi2 (also); oi3; ngoi3
U+5828.0	fMandarin	e4; ai4; nie4; ye4
U+5828.0	fEnglish	daub
U+5829.0	fUTF8	堩
U+5829.0	fCNS11643	2-3A61
U+5829.0	fBig5	D87D
U+5829.0	fGB	E-256D
U+5829.0	fR/S	 32.9
U+5829.0	fTotalStrokes	 12
U+5829.0	fCangjie	GPMM
U+5829.0	fFourCorner	41112
U+5829.0	fHakka	 kwen3
U+5829.0	fCantonese	gang2; gang3
U+5829.0	fMandarin	geng4
U+5829.0	fEnglish	path or avenue leading up to a grave
U+582A.0	fUTF8	堪
U+582A.0	fCNS11643	1-5E5A
U+582A.0	fBig5	B3F4
U+582A.0	fGB	0-3F30
U+582A.0	fR/S	 32.9
U+582A.0	fTotalStrokes	 12
U+582A.0	fCangjie	GTMV
U+582A.0	fFourCorner	44111
U+582A.0	fHakka	  kam1; ham1
U+582A.0	fCantonese	ham1
U+582A.0	fMandarin	kan1
U+582A.0	fEnglish	[1] sustain; bear; stand [2] fit for; worthy of; adequately capable of; worthy of; adequate for [3] a Chinese family name
U+582B.0	fUTF8	堫
U+582B.0	fCNS11643	4-366F
U+582B.0	fGB	3-3452
U+582B.0	fR/S	  32.9
U+582B.0	fTotalStrokes	 12
U+582B.0	fCangjie	GUCE
U+582B.0	fFourCorner	42140
U+582B.0	fHakka	  zung1
U+582B.0	fCantonese	zung1
U+582B.0	fMandarin	zong1
U+582B.0	fEnglish	 sow seed without first ploughing the ground
U+582C.0	fUTF8	堬
U+582C.0	fCNS11643	2-3A74
U+582C.0	fBig5	D8B2
U+582C.0	fGB	5-364C
U+582C.0	fR/S	 32.9
U+582C.0	fTotalStrokes	 12
U+582C.0	fCangjie	GOMN
U+582C.0	fFourCorner	48121
U+582C.0	fHakka	 ji2; ri2
U+582C.0	fCantonese	jyu4
U+582C.0	fMandarin	yu2; yu1
U+582C.0	fEnglish	 grave; gravemound
U+582D.0	fUTF8	堭
U+582D.0	fCNS11643	2-3A73
U+582D.0	fBig5	D8B1
U+582D.0	fGB	3-3450
U+582D.0	fR/S	 32.9
U+582D.0	fTotalStrokes	 12
U+582D.0	fCangjie	GHAG
U+582D.0	fFourCorner	46114
U+582D.0	fHakka	  fong2
U+582D.0	fCantonese	wong4
U+582D.0	fMandarin	huang2
U+582D.0	fEnglish	 a dry moat outside a city wall dry ditch
U+582E.0	fUTF8	堮
U+582E.0	fCNS11643	2-3A70
U+582E.0	fBig5	D8AE
U+582E.0	fGB	5-3647
U+582E.0	fR/S	 32.9
U+582E.0	fTotalStrokes	 12
U+582E.0	fCangjie	GRRS
U+582E.0	fFourCorner	46127
U+582E.0	fCantonese	ngok6; ok6 (also)
U+582E.0	fMandarin	e4
U+582E.0	fEnglish	 boundary border
U+582F.0	fUTF8	堯
U+582F.0	fCNS11643	1-5E59
U+582F.0	fBig5	B3F3
U+582F.0	fGB	1-5222
U+582F.0	fR/S	 32.9
U+582F.0	fTotalStrokes	 12
U+582F.0	fCangjie	GGGU
U+582F.0	fFourCorner	40211
U+582F.0	fHakka	  ngiau2; ngieu2; gniau3
U+582F.0	fCantonese	jiu4
U+582F.0	fMandarin	yao2
U+582F.0	fEnglish	[1] Yao, a legendary sage king in ancient China whose reign is said too have extended from 2357 to 2255 B.C. [2] high; eminent; lofty [3] a Chinese family name
U+5830.0	fUTF8	堰
U+5830.0	fCNS11643	1-5E5D
U+5830.0	fBig5	B3F7
U+5830.0	fGB	0-515F
U+5830.0	fR/S	 32.9
U+5830.0	fTotalStrokes	 12
U+5830.0	fCangjie	GSAV
U+5830.0	fFourCorner	41114
U+5830.0	fHakka	 jen3; jen5; ren3; jan3
U+5830.0	fCantonese	jin2
U+5830.0	fMandarin	yan4; yi1
U+5830.0	fEnglish	 dam embankment; dike; bank
U+5831.0	fUTF8	報
U+5831.0	fCNS11643	1-5E5E
U+5831.0	fBig5	B3F8
U+5831.0	fGB	1-3128
U+5831.0	fR/S	 32.9
U+5831.0	fTotalStrokes	 12
U+5831.0	fCangjie	GJSLE
U+5831.0	fFourCorner	47447
U+5831.0	fHakka	  bau5;  bo5
U+5831.0	fCantonese	bou3
U+5831.0	fMandarin	bao4
U+5831.0	fEnglish	[1] repay; recompense [2] reward; retribution [3] report; announce; report [4] newspaper; periodical; journal
U+5832.0	fUTF8	堲
U+5832.0	fCNS11643	2-2E4D
U+5832.0	fBig5	D14B
U+5832.0	fGB	3-3436
U+5832.0	fR/S	 32.9
U+5832.0	fTotalStrokes	 12
U+5832.0	fCangjie	ALG
U+5832.0	fFourCorner	77104
U+5832.0	fHakka	  zit7
U+5832.0	fCantonese	zik1; zat1; zik6
U+5832.0	fMandarin	ji2
U+5832.0	fEnglish	hate
U+5833.0	fUTF8	堳
U+5833.0	fCNS11643	2-3A6D
U+5833.0	fBig5	D8AB
U+5833.0	fGB	3-3457
U+5833.0	fR/S	 32.9
U+5833.0	fTotalStrokes	 12
U+5833.0	fCangjie	GAHU
U+5833.0	fFourCorner	47167
U+5833.0	fHakka	 mi2
U+5833.0	fCantonese	mei4; mui4
U+5833.0	fMandarin	mei2
U+5833.0	fEnglish	 land allotment feudal noble alta
U+5834.0	fUTF8	場
U+5834.0	fCNS11643	1-5E5B
U+5834.0	fBig5	B3F5
U+5834.0	fGB	1-3321
U+5834.0	fR/S	 32.9
U+5834.0	fTotalStrokes	 12
U+5834.0	fCangjie	GAMH
U+5834.0	fFourCorner	46127
U+5834.0	fHakka	  cong2;  chong2; chong3; jit8
U+5834.0	fCantonese	coeng4; coeng2 (also)
U+5834.0	fMandarin	chang2; chang3
U+5834.0	fEnglish	[1] an open space; field; area of level ground [2] an act of a play; act of an opera [3] the stage [4] an area for drill; a playground [5] a farm [6] a site or place for a special purpose (as an examination, meeting, etc.)
U+5834.1	fUTF8	場
U+5834.1	fCNS11643	1-5E5B
U+5834.1	fBig5	B3F5
U+5834.1	fGB	1-3321
U+5834.1	fR/S	 32.9
U+5834.1	fTotalStrokes	 12
U+5834.1	fCangjie	GAMH
U+5834.1	fFourCorner	46127
U+5835.0	fUTF8	堵
U+5835.0	fCNS11643	1-5959
U+5835.0	fBig5	B0F4
U+5835.0	fGB	0-3642
U+5835.0	fR/S	 32.9
U+5835.0	fTotalStrokes	 12
U+5835.0	fCangjie	GJKA
U+5835.0	fFourCorner	44160
U+5835.0	fHakka	  du3;  du2
U+5835.0	fCantonese	dou2; ze2
U+5835.0	fMandarin	du3; du1; she2; zhe3
U+5835.0	fEnglish	[1] wall [2] stop; prevent; stop up; block up; shut off [3] a Chinese family name
U+5836.0	fUTF8	堶
U+5836.0	fCNS11643	2-3A6F
U+5836.0	fBig5	D8AD
U+5836.0	fGB	5-3642
U+5836.0	fR/S	 32.9
U+5836.0	fTotalStrokes	 12
U+5836.0	fCangjie	GKMB
U+5836.0	fFourCorner	44127
U+5836.0	fHakka	  to2; to3
U+5836.0	fCantonese	to4
U+5836.0	fMandarin	tuo2; tuo1; hui1
U+5836.0	fEnglish	a game in which a heavy sstone weight is thrown from one to another
U+5837.0	fUTF8	堷
U+5837.0	fCNS11643	2-3A62
U+5837.0	fBig5	D87E
U+5837.0	fGB	E-256E
U+5837.0	fR/S	 32.9
U+5837.0	fTotalStrokes	 12
U+5837.0	fCangjie	GYTA
U+5837.0	fFourCorner	40161
U+5837.0	fHakka	 am1
U+5837.0	fCantonese	jam3; bau6
U+5837.0	fMandarin	yin4; an3; pou3
U+5837.0	fEnglish	a burying-place on a moor, such as is granted to the poor for free interment
U+5838.0	fUTF8	堸
U+5838.0	fCNS11643	2-3A72
U+5838.0	fBig5	D8B0
U+5838.0	fGB	5-3563
U+5838.0	fR/S	 32.9
U+5838.0	fTotalStrokes	 12
U+5838.0	fCangjie	GHNI
U+5838.0	fFourCorner	47110
U+5838.0	fCantonese	fung4
U+5838.0	fMandarin	feng2
U+5839.0	fUTF8	堹
U+5839.0	fCNS11643	2-3A71
U+5839.0	fBig5	D8AF
U+5839.0	fGB	5-3649
U+5839.0	fR/S	 32.9
U+5839.0	fTotalStrokes	 12
U+5839.0	fCangjie	GHJG
U+5839.0	fFourCorner	42114
U+5839.0	fCantonese	zung3
U+5839.0	fMandarin	zhong4
U+583A.0	fUTF8	堺
U+583A.0	fCNS11643	3-3A4E
U+583A.0	fGB	5-3646
U+583A.0	fR/S	  32.9
U+583A.0	fTotalStrokes	 12
U+583A.0	fCangjie	GWOL
U+583A.0	fFourCorner	46120
U+583A.0	fHakka	 giai5; gai5
U+583A.0	fCantonese	gaai3
U+583A.0	fMandarin	jie4
U+583A.0	fEnglish	same as &#x754C;
U+583B.0	fUTF8	堻
U+583B.0	fCNS11643	2-3A75
U+583B.0	fBig5	D8B3
U+583B.0	fGB	E-256F
U+583B.0	fR/S	 32.9
U+583B.0	fTotalStrokes	 12
U+583B.0	fCangjie	EQG
U+583B.0	fFourCorner	35104
U+583B.0	fCantonese	zeon1
U+583B.0	fMandarin	zhen1
U+583C.0	fUTF8	堼
U+583C.0	fCNS11643	3-3A49
U+583C.0	fGB	3-3449
U+583C.0	fR/S	 32.9
U+583C.0	fTotalStrokes	 12
U+583C.0	fCangjie	GIG
U+583C.0	fFourCorner	44100
U+583C.0	fCantonese	hang6
U+583C.0	fMandarin	feng1; heng4
U+583D.0	fUTF8	堽
U+583D.0	fCNS11643	2-4236
U+583D.0	fBig5	DCEF
U+583D.0	fGB	3-345B
U+583D.0	fR/S	 32.10; 32.9
U+583D.0	fTotalStrokes	 13
U+583D.0	fCangjie	GWLM
U+583D.0	fFourCorner	46111
U+583D.0	fCantonese	gong1
U+583D.0	fMandarin	gang1
U+583D.0	fEnglish	 mound used in place names
U+583E.0	fUTF8	堾
U+583E.0	fCNS11643	3-3A4B
U+583E.0	fGB	3-3448
U+583E.0	fR/S	 32.9
U+583E.0	fTotalStrokes	 12
U+583E.0	fCangjie	GQKA
U+583E.0	fFourCorner	45160
U+583E.0	fHakka	 chun3; cun3
U+583E.0	fCantonese	cyun2; ceon2; ceon1
U+583E.0	fMandarin	chun3; chuan3
U+583E.0	fEnglish	same as &#x8E33;
U+583F.0	fUTF8	堿
U+583F.0	fCNS11643	2-3A6E
U+583F.0	fBig5	D8AC
U+583F.0	fGB	E-2570
U+583F.0	fR/S	 32.9
U+583F.0	fTotalStrokes	 12
U+583F.0	fCangjie	GIHR
U+583F.0	fFourCorner	43150
U+583F.0	fCantonese	haam4; ham3 (also); gaan2; gan2
U+583F.0	fMandarin	jian3
U+583F.0	fEnglish	salty
U+5840.0	fUTF8	塀
U+5840.0	fCNS11643	F-397D
U+5840.0	fGB	E-2571
U+5840.0	fR/S	 32.9
U+5840.0	fTotalStrokes	 12
U+5840.0	fCangjie	GSTT
U+5840.0	fFourCorner	4714
U+5840.0	fCantonese	ping4
U+5840.0	fEnglish	wall; fence
U+5841.0	fUTF8	塁
U+5841.0	fGB	E-2572
U+5841.0	fR/S	 32.9; 102.7
U+5841.0	fTotalStrokes	 12
U+5841.0	fCangjie	WIOG
U+5841.0	fCantonese	leoi5
U+5841.0	fMandarin	lei3
U+5841.0	fEnglish	 rampart, military wall
U+5842.0	fUTF8	塂
U+5842.0	fCNS11643	F-397E
U+5842.0	fGB	8-2F68
U+5842.0	fR/S	 32.8; 32.9
U+5842.0	fTotalStrokes	 12
U+5842.0	fCangjie	GTCU
U+5842.0	fCantonese	hong4
U+5842.0	fMandarin	xiang4; jiang3
U+5843.0	fUTF8	塃
U+5843.0	fCNS11643	4-3C57
U+5843.0	fGB	3-344B
U+5843.0	fR/S	 32.10
U+5843.0	fTotalStrokes	 13
U+5843.0	fCangjie	GTYU
U+5843.0	fFourCorner	44110
U+5843.0	fCantonese	fong1
U+5843.0	fMandarin	huang1
U+5844.0	fUTF8	塄
U+5844.0	fCNS11643	4-366D
U+5844.0	fGB	0-5C28
U+5844.0	fR/S	  32.9
U+5844.0	fTotalStrokes	 12
U+5844.0	fCangjie	GWLS
U+5844.0	fFourCorner	46120
U+5844.0	fCantonese	ling4
U+5844.0	fMandarin	leng2
U+5844.0	fEnglish	elevated bank in field
U+5845.0	fUTF8	塅
U+5845.0	fCNS11643	4-3671
U+5845.0	fGB	3-344F
U+5845.0	fR/S	 32.9
U+5845.0	fTotalStrokes	 12
U+5845.0	fCangjie	GHJE
U+5845.0	fFourCorner	47140
U+5845.0	fCantonese	dyun6; tyun6
U+5845.0	fMandarin	duan4
U+5846.0	fUTF8	塆
U+5846.0	fGB	8-7A55
U+5846.0	fR/S	 32.9
U+5846.0	fTotalStrokes	 12
U+5846.0	fCangjie	GYCN
U+5846.0	fCantonese	waan1
U+5846.0	fMandarin	wan1
U+5847.0	fUTF8	塇
U+5847.0	fCNS11643	3-6464
U+5847.0	fGB	E-2573
U+5847.0	fR/S	 32.9
U+5847.0	fTotalStrokes	 12
U+5847.0	fCangjie	GJMM
U+5847.0	fFourCorner	4311
U+5847.0	fCantonese	syun1
U+5847.0	fMandarin	xuan1
U+5848.0	fUTF8	塈
U+5848.0	fCNS11643	2-3A69
U+5848.0	fBig5	D8A7
U+5848.0	fGB	3-3456
U+5848.0	fR/S	 32.9
U+5848.0	fTotalStrokes	 12
U+5848.0	fCangjie	AUG
U+5848.0	fFourCorner	71104
U+5848.0	fCantonese	gei3 (also); hei3; kei3 (also); gei6
U+5848.0	fMandarin	ji4; xi4
U+5848.0	fEnglish	paint; decorate
U+5849.0	fUTF8	塉
U+5849.0	fCNS11643	2-422E
U+5849.0	fBig5	DCE7
U+5849.0	fGB	3-3461
U+5849.0	fR/S	 32.10
U+5849.0	fTotalStrokes	 13
U+5849.0	fCangjie	GFCB
U+5849.0	fFourCorner	44127
U+5849.0	fHakka	  zit7
U+5849.0	fCantonese	zik6; zek1
U+5849.0	fMandarin	ji2
U+5849.0	fEnglish	poor land, as on hill-tops
U+584A.0	fUTF8	塊
U+584A.0	fCNS11643	1-635B
U+584A.0	fBig5	B6F4
U+584A.0	fGB	1-3F69
U+584A.0	fR/S	 32.10
U+584A.0	fTotalStrokes	 13
U+584A.0	fCangjie	GHUI
U+584A.0	fFourCorner	46113
U+584A.0	fHakka	  kwai5; kai5; te5; k'uai5; kwai2
U+584A.0	fCantonese	faai3
U+584A.0	fMandarin	kuai4
U+584A.0	fEnglish	[1] a lump (or clod) of earth [2] a piece; lump; cube [3] a dollar [4] a piece of (land, bread, etc.) [5] alone; to be all by oneself
U+584B.0	fUTF8	塋
U+584B.0	fCNS11643	1-635E
U+584B.0	fBig5	B6F7
U+584B.0	fGB	1-5C63
U+584B.0	fR/S	 32.10
U+584B.0	fTotalStrokes	 13
U+584B.0	fCangjie	FFBG
U+584B.0	fFourCorner	99104
U+584B.0	fHakka	 jin2; rin2
U+584B.0	fCantonese	jing4
U+584B.0	fMandarin	ying2
U+584B.0	fEnglish	a grave; tomb
U+584C.0	fUTF8	塌
U+584C.0	fCNS11643	1-6359
U+584C.0	fBig5	B6F2
U+584C.0	fGB	0-4B7A
U+584C.0	fR/S	 32.10
U+584C.0	fTotalStrokes	 13
U+584C.0	fCangjie	GASM
U+584C.0	fFourCorner	46127
U+584C.0	fHakka	  tap7; lap7
U+584C.0	fCantonese	taap3; dap1
U+584C.0	fMandarin	ta1
U+584C.0	fEnglish	[1] cave in; fall in ruins; collapse [2] sink; droop [3] a little house
U+584D.0	fUTF8	塍
U+584D.0	fCNS11643	2-422D
U+584D.0	fBig5	DCE6
U+584D.0	fGB	0-6B73
U+584D.0	fR/S	 32.10
U+584D.0	fTotalStrokes	 13
U+584D.0	fCangjie	BFQG
U+584D.0	fFourCorner	79214
U+584D.0	fHakka	 shin2; shun2; siin2; sun2
U+584D.0	fCantonese	sing4
U+584D.0	fMandarin	cheng2
U+584D.0	fEnglish	a low dyke between ricefields, used as a pathway
U+584E.0	fUTF8	塎
U+584E.0	fCNS11643	2-4231
U+584E.0	fBig5	DCEA
U+584E.0	fGB	3-3464
U+584E.0	fR/S	 32.10
U+584E.0	fTotalStrokes	 13
U+584E.0	fCangjie	GJCR
U+584E.0	fFourCorner	43168
U+584E.0	fCantonese	jung5
U+584E.0	fMandarin	yong3
U+584F.0	fUTF8	塏
U+584F.0	fCNS11643	2-422C
U+584F.0	fBig5	DCE5
U+584F.0	fGB	1-5B6E
U+584F.0	fR/S	 32.10
U+584F.0	fTotalStrokes	 13
U+584F.0	fCangjie	GUMT
U+584F.0	fFourCorner	42118
U+584F.0	fHakka	  koi3; kai3
U+584F.0	fCantonese	hoi2
U+584F.0	fMandarin	kai3
U+584F.0	fEnglish	high and dry place
U+5850.0	fUTF8	塐
U+5850.0	fCNS11643	3-4057
U+5850.0	fGB	5-3652
U+5850.0	fR/S	  32.10
U+5850.0	fTotalStrokes	 13
U+5850.0	fCangjie	GQMF
U+5850.0	fFourCorner	45190
U+5850.0	fHakka	 zok7; su5; sii5; sok7
U+5850.0	fCantonese	sou3
U+5850.0	fMandarin	su4
U+5850.0	fEnglish	same as &#x5851;
U+5851.0	fUTF8	塑
U+5851.0	fCNS11643	1-6353
U+5851.0	fBig5	B6EC
U+5851.0	fGB	0-4B5C
U+5851.0	fR/S	 32.10
U+5851.0	fTotalStrokes	 13
U+5851.0	fCangjie	TBG
U+5851.0	fFourCorner	87104
U+5851.0	fHakka	  sok7;  zok7; su5; sok8; sii5
U+5851.0	fCantonese	sou3; sok3 (also)
U+5851.0	fMandarin	su4
U+5851.0	fEnglish	[1] mold (in clay, etc.); sculpt [2] figure; model [3] plastics
U+5852.0	fUTF8	塒
U+5852.0	fCNS11643	1-635D
U+5852.0	fBig5	B6F6
U+5852.0	fGB	1-5B75
U+5852.0	fR/S	 32.10
U+5852.0	fTotalStrokes	 13
U+5852.0	fCangjie	GAGI
U+5852.0	fFourCorner	44141
U+5852.0	fHakka	 shi2; sii2; she2
U+5852.0	fCantonese	si4
U+5852.0	fMandarin	shi2
U+5852.0	fEnglish	roost
U+5853.0	fUTF8	塓
U+5853.0	fCNS11643	2-4229
U+5853.0	fBig5	DCE2
U+5853.0	fGB	3-3466
U+5853.0	fR/S	 32.10
U+5853.0	fTotalStrokes	 13
U+5853.0	fCangjie	GBAC
U+5853.0	fFourCorner	47180
U+5853.0	fHakka	  met7; men3; men
U+5853.0	fCantonese	mik6
U+5853.0	fMandarin	mi4
U+5853.0	fEnglish	plaster
U+5854.0	fUTF8	塔
U+5854.0	fCNS11643	1-6357
U+5854.0	fBig5	B6F0
U+5854.0	fGB	0-4B7E
U+5854.0	fR/S	 32.10
U+5854.0	fTotalStrokes	 13
U+5854.0	fCangjie	GTOR
U+5854.0	fFourCorner	44161
U+5854.0	fHakka	  tap7; dap8
U+5854.0	fCantonese	taap3; taap2 (also)
U+5854.0	fMandarin	ta3; da5
U+5854.0	fEnglish	[1] pagoda [2] tower; tall building
U+5855.0	fUTF8	塕
U+5855.0	fCNS11643	2-4230
U+5855.0	fBig5	DCE9
U+5855.0	fGB	3-345D
U+5855.0	fR/S	 32.10
U+5855.0	fTotalStrokes	 13
U+5855.0	fCangjie	GCIM
U+5855.0	fFourCorner	48127
U+5855.0	fHakka	 wung3; wung1; vung3
U+5855.0	fCantonese	jung2; jung1
U+5855.0	fMandarin	weng3; weng1
U+5855.0	fEnglish	a gust of wind
U+5856.0	fUTF8	塖
U+5856.0	fCNS11643	3-405B
U+5856.0	fGB	5-3656
U+5856.0	fR/S	  32.10
U+5856.0	fTotalStrokes	 13
U+5856.0	fCangjie	GHDP
U+5856.0	fFourCorner	42190
U+5856.0	fHakka	 shin2; shun2; siin2; shun3; sun2
U+5856.0	fCantonese	sing4
U+5856.0	fMandarin	cheng2
U+5856.0	fEnglish	same as &#x5818;
U+5857.0	fUTF8	塗
U+5857.0	fCNS11643	1-6355
U+5857.0	fBig5	B6EE
U+5857.0	fGB	1-4D3F
U+5857.0	fR/S	 32.10
U+5857.0	fTotalStrokes	 13
U+5857.0	fCangjie	EDG
U+5857.0	fFourCorner	38104
U+5857.0	fHakka	  tu2; tu3
U+5857.0	fCantonese	tou4; caa4
U+5857.0	fMandarin	tu2; cha2; du1; du4
U+5857.0	fEnglish	[1] smear; apply; spread (ointment on a wound); paint [2] scribble; scrawl [3] erase; blot out; efface; obliterate; cross out [4] mud; wire [5] way; road
U+5858.0	fUTF8	塘
U+5858.0	fCNS11643	1-6354
U+5858.0	fBig5	B6ED
U+5858.0	fGB	0-4C41
U+5858.0	fR/S	 32.10
U+5858.0	fTotalStrokes	 13
U+5858.0	fCangjie	GILR
U+5858.0	fFourCorner	40167
U+5858.0	fHakka	  tong2; tong3
U+5858.0	fCantonese	tong4
U+5858.0	fMandarin	tang2
U+5858.0	fEnglish	[1] pond; tank; square pool [2] dike; embankment; bound; bank
U+5859.0	fUTF8	塙
U+5859.0	fCNS11643	2-4233
U+5859.0	fBig5	DCEC
U+5859.0	fGB	3-345F
U+5859.0	fR/S	 32.10
U+5859.0	fTotalStrokes	 13
U+5859.0	fCangjie	GYRB
U+5859.0	fFourCorner	40127
U+5859.0	fCantonese	kok3
U+5859.0	fMandarin	que4; qiao1
U+5859.0	fEnglish	truly
U+585A.0	fUTF8	塚
U+585A.0	fCNS11643	1-6356
U+585A.0	fBig5	B6EF
U+585A.0	fGB	8-2E32
U+585A.0	fR/S	 32.10; 32.9
U+585A.0	fTotalStrokes	 13
U+585A.0	fCangjie	GBMO
U+585A.0	fFourCorner	47132
U+585A.0	fHakka	 cung3;  chung3
U+585A.0	fCantonese	cung2
U+585A.0	fMandarin	zhong3
U+585A.0	fEnglish	[1] high tomb; grave; (burial) mound [2] non-classical form of &#x51A2;
U+585B.0	fUTF8	塛
U+585B.0	fCNS11643	2-4235
U+585B.0	fBig5	DCEE
U+585B.0	fGB	E-2574
U+585B.0	fR/S	 32.10
U+585B.0	fTotalStrokes	 13
U+585B.0	fCangjie	GMWD
U+585B.0	fFourCorner	41194
U+585B.0	fCantonese	leot6
U+585B.0	fMandarin	li4
U+585C.0	fUTF8	塜
U+585C.0	fCNS11643	3-4056
U+585C.0	fGB	5-365A
U+585C.0	fR/S	 32.10
U+585C.0	fTotalStrokes	 13
U+585C.0	fCangjie	GBMO
U+585C.0	fFourCorner	47130
U+585C.0	fHakka	 pung2
U+585C.0	fCantonese	bung6
U+585C.0	fMandarin	peng2
U+585C.0	fEnglish	same as &#x5873;
U+585D.0	fUTF8	塝
U+585D.0	fCNS11643	2-4232
U+585D.0	fBig5	DCEB
U+585D.0	fGB	5-3658
U+585D.0	fR/S	 32.10
U+585D.0	fTotalStrokes	 13
U+585D.0	fCangjie	GYBS
U+585D.0	fFourCorner	40127
U+585D.0	fCantonese	bong6
U+585D.0	fMandarin	bang4
U+585D.0	fEnglish	flat bank; plateau
U+585E.0	fUTF8	塞
U+585E.0	fCNS11643	1-6352
U+585E.0	fBig5	B6EB
U+585E.0	fGB	0-487B
U+585E.0	fR/S	 32.10
U+585E.0	fTotalStrokes	 13
U+585E.0	fCangjie	JTCG
U+585E.0	fFourCorner	30104
U+585E.0	fHakka	  set7;  sai5; cet7
U+585E.0	fCantonese	sak1; coi3
U+585E.0	fMandarin	se4; sai4; sei1
U+585E.0	fEnglish	[1] stop up; block; clog [2] stuff; squeeze in; fill [3] seal; cork; stopper; cork
U+585E.1	fUTF8	塞
U+585E.1	fCNS11643	1-6352
U+585E.1	fBig5	B6EB
U+585E.1	fGB	0-487B
U+585E.1	fR/S	 32.10
U+585E.1	fTotalStrokes	 13
U+585E.1	fCangjie	JTCG
U+585E.1	fFourCorner	30104
U+585E.1	fMandarin	sai1
U+585E.1	fEnglish	strategic points along the frontiers
U+585F.0	fUTF8	塟
U+585F.0	fCNS11643	3-405E
U+585F.0	fGB	E-2575
U+585F.0	fR/S	 32.10
U+585F.0	fTotalStrokes	 13
U+585F.0	fCangjie	TMPG
U+585F.0	fFourCorner	44100
U+585F.0	fHakka	  zong5
U+585F.0	fCantonese	zong3
U+585F.0	fMandarin	zang4
U+585F.0	fEnglish	[1] bury; inter [2] funeral
U+5860.0	fUTF8	塠
U+5860.0	fCNS11643	3-405C
U+5860.0	fGB	3-3451
U+5860.0	fR/S	 32.10
U+5860.0	fTotalStrokes	 13
U+5860.0	fCangjie	GYHR
U+5860.0	fFourCorner	47130
U+5860.0	fHakka	 doi1
U+5860.0	fCantonese	deoi1
U+5860.0	fMandarin	dui1
U+5861.0	fUTF8	塡
U+5861.0	fCNS11643	3-4058
U+5861.0	fGB	E-2576
U+5861.0	fR/S	 32.10
U+5861.0	fTotalStrokes	 13
U+5861.0	fCangjie	GPBC
U+5861.0	fFourCorner	41180
U+5861.0	fHakka	 tien2;  tien5; }
U+5861.0	fCantonese	tin4
U+5861.0	fMandarin	tian2
U+5861.0	fEnglish	[1] fill in; fill up; stuff [2] the sound of drumbeats
U+5862.0	fUTF8	塢
U+5862.0	fCNS11643	1-635C
U+5862.0	fBig5	B6F5
U+5862.0	fGB	1-4E6B
U+5862.0	fR/S	 32.10
U+5862.0	fTotalStrokes	 13
U+5862.0	fCangjie	GHRF
U+5862.0	fFourCorner	47127
U+5862.0	fHakka	  wu1; wu3; vu3
U+5862.0	fCantonese	wu2; ou3; ngou3 (also)
U+5862.0	fMandarin	wu4
U+5862.0	fEnglish	[1] a low wall around a village for defense; entrenchment; fortified building; castle [2] a structure which slants to a lower center on all sides; depression; cove; recess
U+5863.0	fUTF8	塣
U+5863.0	fCNS11643	2-4237
U+5863.0	fBig5	DCF0
U+5863.0	fGB	E-2577
U+5863.0	fR/S	 32.10
U+5863.0	fTotalStrokes	 13
U+5863.0	fCangjie	EGG
U+5863.0	fFourCorner	36104
U+5863.0	fCantonese	zing6; cing2
U+5863.0	fMandarin	cheng3
U+5864.0	fUTF8	塤
U+5864.0	fCNS11643	2-422B
U+5864.0	fBig5	DCE4
U+5864.0	fGB	1-5B77
U+5864.0	fR/S	 32.10
U+5864.0	fTotalStrokes	 13
U+5864.0	fCangjie	GRBC
U+5864.0	fFourCorner	46186
U+5864.0	fHakka	 hiun1
U+5864.0	fCantonese	hyun1; fan1 (also)
U+5864.0	fMandarin	xun1; xuan1
U+5864.0	fEnglish	instrument
U+5865.0	fUTF8	塥
U+5865.0	fCNS11643	2-4234
U+5865.0	fBig5	DCED
U+5865.0	fGB	0-5C2A
U+5865.0	fR/S	 32.10
U+5865.0	fTotalStrokes	 13
U+5865.0	fCangjie	GMRB
U+5865.0	fFourCorner	41127
U+5865.0	fHakka	 gak7
U+5865.0	fCantonese	gaak3
U+5865.0	fMandarin	ge2
U+5866.0	fUTF8	塦
U+5866.0	fCNS11643	3-405D
U+5866.0	fGB	E-2578
U+5866.0	fR/S	 32.10
U+5866.0	fTotalStrokes	 13
U+5866.0	fCangjie	NJG
U+5866.0	fFourCorner	25100
U+5866.0	fHakka	 chin6; cihn5; cin3; cin5; chin5
U+5866.0	fCantonese	zan6; zan2 (also)
U+5866.0	fMandarin	zhen4
U+5866.0	fEnglish	same as &#x9663;
U+5867.0	fUTF8	塧
U+5867.0	fCNS11643	4-3C53
U+5867.0	fGB	5-3659
U+5867.0	fR/S	  32.10
U+5867.0	fTotalStrokes	 13
U+5867.0	fCangjie	GTCT
U+5867.0	fFourCorner	48110
U+5867.0	fCantonese	aai3; ngaai3 (also)
U+5867.0	fMandarin	ai4
U+5868.0	fUTF8	塨
U+5868.0	fCNS11643	2-422A
U+5868.0	fBig5	DCE3
U+5868.0	fGB	3-3459
U+5868.0	fR/S	 32.10
U+5868.0	fTotalStrokes	 13
U+5868.0	fCangjie	GTCP
U+5868.0	fFourCorner	44133
U+5868.0	fCantonese	gung1
U+5868.0	fMandarin	gong1
U+5869.0	fUTF8	塩
U+5869.0	fCNS11643	3-405A
U+5869.0	fGB	E-2579
U+5869.0	fR/S	 32.10; 32.8; 108.8
U+5869.0	fTotalStrokes	 13
U+5869.0	fCangjie	GORT
U+5869.0	fFourCorner	48117
U+5869.0	fHakka	 jam2; ram2; jam3
U+5869.0	fCantonese	jim4; jim6
U+5869.0	fMandarin	yan2
U+5869.0	fEnglish	salt
U+586A.0	fUTF8	塪
U+586A.0	fCNS11643	3-4059
U+586A.0	fGB	E-257A
U+586A.0	fR/S	  32.10
U+586A.0	fTotalStrokes	 13
U+586A.0	fCangjie	GBHX
U+586A.0	fFourCorner	42170
U+586A.0	fHakka	 kam3
U+586A.0	fCantonese	ham2; hom2
U+586A.0	fMandarin	kan3; xian4
U+586A.0	fEnglish	corrupted form of &#x57F3;
U+586B.0	fUTF8	填
U+586B.0	fCNS11643	1-6358
U+586B.0	fBig5	B6F1
U+586B.0	fGB	0-4C6E
U+586B.0	fR/S	 32.10
U+586B.0	fTotalStrokes	 13
U+586B.0	fCangjie	GJBC
U+586B.0	fFourCorner	41181
U+586B.0	fHakka	 ten2; tien2
U+586B.0	fCantonese	tin4; zan3
U+586B.0	fMandarin	tian2; chen2; dian4; tian3; zhen1; zhen4
U+586B.0	fEnglish	[1] to fill in; fill up; stuff [2] the sound of drumbeats
U+586C.0	fUTF8	塬
U+586C.0	fCNS11643	3-4054
U+586C.0	fGB	0-5C2B
U+586C.0	fR/S	 32.10
U+586C.0	fTotalStrokes	 13
U+586C.0	fCangjie	GMHF
U+586C.0	fFourCorner	41190
U+586C.0	fCantonese	jyun4
U+586C.0	fMandarin	yuan2
U+586C.0	fEnglish	plateau
U+586D.0	fUTF8	塭
U+586D.0	fCNS11643	1-635A
U+586D.0	fBig5	B6F3
U+586D.0	fGB	E-257B
U+586D.0	fR/S	 32.10
U+586D.0	fTotalStrokes	 13
U+586D.0	fCangjie	GWOT
U+586D.0	fFourCorner	46117
U+586D.0	fCantonese	wan1
U+586D.0	fMandarin	wen1
U+586E.0	fUTF8	塮
U+586E.0	fGB	3-345C
U+586E.0	fR/S	 32.10
U+586E.0	fTotalStrokes	 13
U+586E.0	fCangjie	GHHI
U+586E.0	fCantonese	ze6
U+586E.0	fMandarin	xie4
U+586F.0	fUTF8	塯
U+586F.0	fCNS11643	2-422F
U+586F.0	fBig5	DCE8
U+586F.0	fGB	E-257C
U+586F.0	fR/S	 32.10
U+586F.0	fTotalStrokes	 13
U+586F.0	fCangjie	GHHW
U+586F.0	fFourCorner	47160
U+586F.0	fHakka	  liu5
U+586F.0	fCantonese	lau6
U+586F.0	fMandarin	liu4
U+586F.0	fEnglish	beggar's almsbowl
U+5870.0	fUTF8	塰
U+5870.0	fGB	E-257D
U+5870.0	fR/S	 32.10
U+5870.0	fTotalStrokes	 13
U+5870.0	fCangjie	XEYG
U+5870.0	fCantonese	hoi2
U+5871.0	fUTF8	塱
U+5871.0	fCNS11643	2-4238
U+5871.0	fBig5	DCF1
U+5871.0	fGB	5-365B
U+5871.0	fR/S	 32.10
U+5871.0	fTotalStrokes	 13
U+5871.0	fCangjie	IBG
U+5871.0	fFourCorner	37104
U+5871.0	fHakka	 long6; long5; long3
U+5871.0	fCantonese	long5
U+5871.0	fMandarin	lang3
U+5871.0	fEnglish	place name in Kwangtung province
U+5872.0	fUTF8	塲
U+5872.0	fCNS11643	3-4643
U+5872.0	fGB	E-257E
U+5872.0	fR/S	 32.11
U+5872.0	fTotalStrokes	 14
U+5872.0	fCangjie	GOAH
U+5872.0	fFourCorner	48120
U+5872.0	fHakka	 cong2;  chong2
U+5872.0	fCantonese	coeng4; zoeng4
U+5872.0	fMandarin	shang1; chang2; chang3
U+5872.0	fEnglish	[1] an open space; field; area of level ground [2] an act of a play; act of an opera [3] the stage [4] an area for drill; a playground [5] a farm [6] a site or place for a special purpose (as an examination, meeting, etc.)
U+5873.0	fUTF8	塳
U+5873.0	fCNS11643	3-463E
U+5873.0	fGB	3-345E
U+5873.0	fR/S	  32.11
U+5873.0	fTotalStrokes	 14
U+5873.0	fCangjie	GYHJ
U+5873.0	fFourCorner	47130
U+5873.0	fHakka	  pung2
U+5873.0	fCantonese	bung6; pung1
U+5873.0	fMandarin	peng2
U+5873.0	fEnglish	 whirl (wind)
U+5874.0	fUTF8	塴
U+5874.0	fCNS11643	2-4944
U+5874.0	fBig5	E15D
U+5874.0	fGB	3-346B
U+5874.0	fR/S	 32.11
U+5874.0	fTotalStrokes	 14
U+5874.0	fCangjie	GUBB
U+5874.0	fFourCorner	42127
U+5874.0	fCantonese	bang1 (also); bang6; baang6; bing3; paang4
U+5874.0	fMandarin	beng4; beng1
U+5875.0	fUTF8	塵
U+5875.0	fCNS11643	1-6838
U+5875.0	fBig5	B9D0
U+5875.0	fGB	1-333E
U+5875.0	fR/S	 32.11
U+5875.0	fTotalStrokes	 14
U+5875.0	fCangjie	IPG
U+5875.0	fFourCorner	00214
U+5875.0	fHakka	 cin2;  chin2; cihn2; chin3
U+5875.0	fCantonese	can4
U+5875.0	fMandarin	chen2
U+5875.0	fEnglish	[1] dust; dirt [2] trace; trail [3] this world; ways of the world [4] vice; sensual pleasures [5] a lifetime (Taoism)
U+5876.0	fUTF8	塶
U+5876.0	fCNS11643	2-494A
U+5876.0	fBig5	E163
U+5876.0	fGB	E-2621
U+5876.0	fR/S	 32.11
U+5876.0	fTotalStrokes	 14
U+5876.0	fCangjie	GIXP
U+5876.0	fFourCorner	40111
U+5876.0	fCantonese	luk6
U+5876.0	fMandarin	cu4; lu4
U+5877.0	fUTF8	塷
U+5877.0	fCNS11643	3-4642
U+5877.0	fGB	5-362B
U+5877.0	fR/S	  32.11
U+5877.0	fTotalStrokes	 14
U+5877.0	fCangjie	GYWI
U+5877.0	fFourCorner	41160
U+5877.0	fHakka	  lu1;  lu3
U+5877.0	fCantonese	lou5
U+5877.0	fMandarin	lu3; lu2
U+5877.0	fEnglish	same as &#x9E75;
U+5878.0	fUTF8	塸
U+5878.0	fCNS11643	3-463C
U+5878.0	fGB	3-3365
U+5878.0	fR/S	  32.11
U+5878.0	fTotalStrokes	 14
U+5878.0	fCangjie	GSRR
U+5878.0	fFourCorner	41110
U+5878.0	fCantonese	au2; ngau2 (also)
U+5878.0	fMandarin	ou3; ou1
U+5879.0	fUTF8	塹
U+5879.0	fCNS11643	1-683D
U+5879.0	fBig5	B9D5
U+5879.0	fGB	1-4735
U+5879.0	fR/S	 32.11
U+5879.0	fTotalStrokes	 14
U+5879.0	fCangjie	JLG
U+5879.0	fFourCorner	52104
U+5879.0	fHakka	 ciam5; ciam6; ciam3
U+5879.0	fCantonese	cim3
U+5879.0	fMandarin	qian4
U+5879.0	fEnglish	[1] the moat around a city; chasm; trench [2] pit; cavity or hole in the ground
U+587A.0	fUTF8	塺
U+587A.0	fCNS11643	2-4946
U+587A.0	fBig5	E15F
U+587A.0	fGB	3-7C21
U+587A.0	fR/S	 32.11
U+587A.0	fTotalStrokes	 14
U+587A.0	fCangjie	IDG
U+587A.0	fFourCorner	00214
U+587A.0	fHakka	 moi2;  mo2; moe2
U+587A.0	fCantonese	mui4; mo6 (also); mo4
U+587A.0	fMandarin	mei2; mo4
U+587A.0	fEnglish	dust
U+587B.0	fUTF8	塻
U+587B.0	fCNS11643	2-494D
U+587B.0	fBig5	E166
U+587B.0	fGB	5-3654
U+587B.0	fR/S	 32.11
U+587B.0	fTotalStrokes	 14
U+587B.0	fCangjie	GTAK
U+587B.0	fFourCorner	44134
U+587B.0	fHakka	 mok8; mok7
U+587B.0	fCantonese	mok6
U+587B.0	fMandarin	mo4
U+587B.0	fEnglish	dust
U+587C.0	fUTF8	塼
U+587C.0	fCNS11643	2-493E
U+587C.0	fBig5	E157
U+587C.0	fGB	E-2622
U+587C.0	fR/S	 32.11
U+587C.0	fTotalStrokes	 14
U+587C.0	fCangjie	GJII
U+587C.0	fFourCorner	45143
U+587C.0	fHakka	 zon1; zhon1
U+587C.0	fCantonese	zyun1
U+587C.0	fMandarin	zhuan1; tuan2
U+587C.0	fEnglish	brick
U+587D.0	fUTF8	塽
U+587D.0	fCNS11643	1-683F
U+587D.0	fBig5	B9D7
U+587D.0	fGB	5-3663
U+587D.0	fR/S	 32.11
U+587D.0	fTotalStrokes	 14
U+587D.0	fCangjie	GKKK
U+587D.0	fFourCorner	44134
U+587D.0	fCantonese	song2
U+587D.0	fMandarin	shuang3
U+587D.0	fEnglish	plateau
U+587E.0	fUTF8	塾
U+587E.0	fCNS11643	1-6839
U+587E.0	fBig5	B9D1
U+587E.0	fGB	0-5B53
U+587E.0	fR/S	 32.11
U+587E.0	fTotalStrokes	 14
U+587E.0	fCangjie	YIG
U+587E.0	fFourCorner	05104
U+587E.0	fHakka	 suk8;  shuk8
U+587E.0	fCantonese	suk6
U+587E.0	fMandarin	shu2
U+587E.0	fEnglish	[1] village school; family school; private tutorage; private primary school [2] anteroom; vestibule
U+587F.0	fUTF8	塿
U+587F.0	fCNS11643	2-4943
U+587F.0	fBig5	E15C
U+587F.0	fGB	3-3454
U+587F.0	fR/S	 32.11
U+587F.0	fTotalStrokes	 14
U+587F.0	fCangjie	GLLV
U+587F.0	fFourCorner	45144
U+587F.0	fHakka	  leu3
U+587F.0	fCantonese	lau5; lau4
U+587F.0	fMandarin	lou3; lou2
U+587F.0	fEnglish	small mound
U+5880.0	fUTF8	墀
U+5880.0	fCNS11643	1-6C3F
U+5880.0	fBig5	BC55
U+5880.0	fGB	0-5C2F
U+5880.0	fR/S	 32.11
U+5880.0	fTotalStrokes	 14
U+5880.0	fCangjie	GSEQ
U+5880.0	fFourCorner	47153
U+5880.0	fHakka	 chi2; cih2; chi3; ci2; chii2
U+5880.0	fCantonese	ci4
U+5880.0	fMandarin	chi2
U+5880.0	fEnglish	 porch courtyard steps leading
U+5881.0	fUTF8	墁
U+5881.0	fCNS11643	2-4942
U+5881.0	fBig5	E15B
U+5881.0	fGB	0-5C2C
U+5881.0	fR/S	 32.11
U+5881.0	fTotalStrokes	 14
U+5881.0	fCangjie	GAWE
U+5881.0	fFourCorner	46147
U+5881.0	fHakka	 man5; man6
U+5881.0	fCantonese	maan6
U+5881.0	fMandarin	man4; man2
U+5881.0	fEnglish	 plaster pave
U+5882.0	fUTF8	墂
U+5882.0	fCNS11643	2-494B
U+5882.0	fBig5	E164
U+5882.0	fGB	5-3661
U+5882.0	fR/S	 32.11
U+5882.0	fTotalStrokes	 14
U+5882.0	fCangjie	GMWF
U+5882.0	fFourCorner	41191
U+5882.0	fCantonese	biu1
U+5882.0	fMandarin	biao1
U+5883.0	fUTF8	境
U+5883.0	fCNS11643	1-683A
U+5883.0	fBig5	B9D2
U+5883.0	fGB	0-3E33
U+5883.0	fR/S	 32.11
U+5883.0	fTotalStrokes	 14
U+5883.0	fCangjie	GYTU
U+5883.0	fFourCorner	40116
U+5883.0	fHakka	  gin3; gin5
U+5883.0	fCantonese	ging2
U+5883.0	fMandarin	jing4
U+5883.0	fEnglish	[1] boundary; frontier; border [2] place; area; region [3] state; situation; circumstances
U+5884.0	fUTF8	墄
U+5884.0	fCNS11643	3-463D
U+5884.0	fGB	3-346A
U+5884.0	fR/S	 32.11
U+5884.0	fTotalStrokes	 14
U+5884.0	fCangjie	GIHF
U+5884.0	fFourCorner	43150
U+5884.0	fHakka	 cit7
U+5884.0	fCantonese	cak1
U+5884.0	fMandarin	qi1; ce4
U+5884.0	fEnglish	the steps of a stairway
U+5885.0	fUTF8	墅
U+5885.0	fCNS11643	1-683E
U+5885.0	fBig5	B9D6
U+5885.0	fGB	0-4A7B
U+5885.0	fR/S	 32.11
U+5885.0	fTotalStrokes	 14
U+5885.0	fCangjie	WNG
U+5885.0	fFourCorner	67104
U+5885.0	fHakka	 ja3; su3; shu3; ra3; ja1; shu5
U+5885.0	fCantonese	seoi5 (also); seoi6
U+5885.0	fMandarin	shu4; ye3
U+5885.0	fEnglish	 villa; country house
U+5886.0	fUTF8	墆
U+5886.0	fCNS11643	2-4941
U+5886.0	fBig5	E15A
U+5886.0	fGB	3-344A
U+5886.0	fR/S	 32.11
U+5886.0	fTotalStrokes	 14
U+5886.0	fCangjie	GKPB
U+5886.0	fFourCorner	44127
U+5886.0	fHakka	  diet7
U+5886.0	fCantonese	dai3; dit6; zai6
U+5886.0	fMandarin	di4; zhi4; die2
U+5886.0	fEnglish	[1] hoard [2] engross
U+5887.0	fUTF8	墇
U+5887.0	fCNS11643	2-4947
U+5887.0	fBig5	E160
U+5887.0	fGB	3-346E
U+5887.0	fR/S	 32.11
U+5887.0	fTotalStrokes	 14
U+5887.0	fCangjie	GYTJ
U+5887.0	fFourCorner	40146
U+5887.0	fHakka	 zhong1; zong1; zhong5; zong5
U+5887.0	fCantonese	zoeng3; zoeng1; ging2
U+5887.0	fMandarin	zhang1; zhang4; zong3
U+5887.0	fEnglish	 embankment
U+5888.0	fUTF8	墈
U+5888.0	fCNS11643	2-494C
U+5888.0	fBig5	E165
U+5888.0	fGB	3-3467
U+5888.0	fR/S	 32.11
U+5888.0	fTotalStrokes	 14
U+5888.0	fCangjie	GTVS
U+5888.0	fFourCorner	44127
U+5888.0	fHakka	 kam5; ham5; kiam1; kam3
U+5888.0	fCantonese	ham3
U+5888.0	fMandarin	kan4
U+5888.0	fEnglish	cliff
U+5889.0	fUTF8	墉
U+5889.0	fCNS11643	2-493D
U+5889.0	fBig5	E156
U+5889.0	fGB	0-5C2D
U+5889.0	fR/S	 32.11
U+5889.0	fTotalStrokes	 14
U+5889.0	fCangjie	GILB
U+5889.0	fFourCorner	40127
U+5889.0	fHakka	 jung2; rung2
U+5889.0	fCantonese	jung4
U+5889.0	fMandarin	yong1; yong2
U+5889.0	fEnglish	a wall; fortified wall
U+588A.0	fUTF8	墊
U+588A.0	fCNS11643	1-683C
U+588A.0	fBig5	B9D4
U+588A.0	fGB	1-3566
U+588A.0	fR/S	 32.11
U+588A.0	fTotalStrokes	 14
U+588A.0	fCangjie	GIG
U+588A.0	fFourCorner	45104
U+588A.0	fHakka	 tiap7; diam5;  tiap8; t'ian6; den3; dien3; tien5
U+588A.0	fCantonese	din3; zin3 (also); din6; din2; zin2 (also); dip6; dim3
U+588A.0	fMandarin	dian4
U+588A.0	fEnglish	[1] advance (money); pay for another and expect to be paid back [2] cushion; pad; bed-mat [3] cushion; put something under another (to make level) [4] sink into [5] dig
U+588B.0	fUTF8	墋
U+588B.0	fCNS11643	2-4945
U+588B.0	fBig5	E15E
U+588B.0	fGB	5-363F
U+588B.0	fR/S	 32.11
U+588B.0	fTotalStrokes	 14
U+588B.0	fCangjie	GIIH
U+588B.0	fFourCorner	43122
U+588B.0	fHakka	  cam3
U+588B.0	fCantonese	cam2; caam2
U+588B.0	fMandarin	chen3
U+588B.0	fEnglish	[1] dirty; turbid [2] obscure
U+588C.0	fUTF8	墌
U+588C.0	fCNS11643	3-463A
U+588C.0	fGB	3-346D
U+588C.0	fR/S	 32.11
U+588C.0	fTotalStrokes	 14
U+588C.0	fCangjie	GITF
U+588C.0	fFourCorner	40130
U+588C.0	fHakka	 chak7; zak7; zhak7
U+588C.0	fCantonese	zek3
U+588C.0	fMandarin	zhi1; zhi2; zhuo2
U+588C.0	fEnglish	foundation of a wall
U+588D.0	fUTF8	墍
U+588D.0	fCNS11643	3-4645
U+588D.0	fGB	E-2623
U+588D.0	fR/S	 32.11
U+588D.0	fTotalStrokes	 14
U+588D.0	fCangjie	HUG
U+588D.0	fFourCorner	21100
U+588D.0	fCantonese	hei3; kei3 (also)
U+588D.0	fMandarin	ji4; xi4
U+588D.0	fEnglish	paint; decorate
U+588E.0	fUTF8	墎
U+588E.0	fCNS11643	2-4949
U+588E.0	fBig5	E162
U+588E.0	fGB	3-3460
U+588E.0	fR/S	 32.11
U+588E.0	fTotalStrokes	 14
U+588E.0	fCangjie	GYDL
U+588E.0	fFourCorner	47127
U+588E.0	fHakka	 gok7; gwok7
U+588E.0	fCantonese	gwok3; gok3 (also)
U+588E.0	fMandarin	guo1; kuo4; guo4
U+588F.0	fUTF8	墏
U+588F.0	fCNS11643	2-494F
U+588F.0	fBig5	E168
U+588F.0	fGB	5-364D
U+588F.0	fR/S	 32.11
U+588F.0	fTotalStrokes	 14
U+588F.0	fCangjie	VIG
U+588F.0	fFourCorner	27104
U+588F.0	fCantonese	coeng2
U+588F.0	fMandarin	qiang3
U+5890.0	fUTF8	墐
U+5890.0	fCNS11643	2-493F
U+5890.0	fBig5	E158
U+5890.0	fGB	3-3468
U+5890.0	fR/S	 32.11
U+5890.0	fTotalStrokes	 14
U+5890.0	fCangjie	GTLM
U+5890.0	fFourCorner	44114
U+5890.0	fHakka	  giun3
U+5890.0	fCantonese	gan6; gan2 (also)
U+5890.0	fMandarin	jin4; jin3; jin2
U+5890.0	fEnglish	build with soil; plaster over wi
U+5891.0	fUTF8	墑
U+5891.0	fCNS11643	2-4948
U+5891.0	fBig5	E161
U+5891.0	fGB	5-3669
U+5891.0	fR/S	 32.11
U+5891.0	fTotalStrokes	 14
U+5891.0	fCangjie	GYCB
U+5891.0	fFourCorner	40127
U+5891.0	fCantonese	dik1; soeng1
U+5891.0	fMandarin	di1
U+5892.0	fUTF8	墒
U+5892.0	fCNS11643	3-4638
U+5892.0	fGB	0-494A
U+5892.0	fR/S	  32.11
U+5892.0	fTotalStrokes	 14
U+5892.0	fCangjie	GYCB
U+5892.0	fFourCorner	40120
U+5892.0	fCantonese	soeng1
U+5892.0	fMandarin	shang1
U+5892.0	fEnglish	wet tilth
U+5893.0	fUTF8	墓
U+5893.0	fCNS11643	1-683B
U+5893.0	fBig5	B9D3
U+5893.0	fGB	0-4439
U+5893.0	fR/S	 32.11
U+5893.0	fTotalStrokes	 14
U+5893.0	fCangjie	TAKG
U+5893.0	fFourCorner	44104
U+5893.0	fHakka	  mu5; mu6
U+5893.0	fCantonese	mou6
U+5893.0	fMandarin	mu4; mu2
U+5893.0	fEnglish	 grave; tomb; mausoleum
U+5894.0	fUTF8	墔
U+5894.0	fCNS11643	2-494E
U+5894.0	fBig5	E167
U+5894.0	fGB	5-3667
U+5894.0	fR/S	 32.11
U+5894.0	fTotalStrokes	 14
U+5894.0	fCangjie	GUOG
U+5894.0	fFourCorner	42114
U+5894.0	fHakka	 cui1
U+5894.0	fCantonese	ceoi4; ceoi1
U+5894.0	fMandarin	cui1
U+5895.0	fUTF8	墕
U+5895.0	fCNS11643	3-4641
U+5895.0	fGB	5-365E
U+5895.0	fR/S	 32.11
U+5895.0	fTotalStrokes	 14
U+5895.0	fCangjie	GMYF
U+5895.0	fFourCorner	41120
U+5895.0	fHakka	 jen3; jen5; ren3; jan3
U+5895.0	fCantonese	jin2
U+5895.0	fMandarin	yan4; yi1
U+5895.0	fEnglish	same as &#x5830;
U+5896.0	fUTF8	墖
U+5896.0	fCNS11643	3-4644
U+5896.0	fGB	E-2624
U+5896.0	fR/S	 32.11
U+5896.0	fTotalStrokes	 14
U+5896.0	fCangjie	GORW
U+5896.0	fFourCorner	48160
U+5896.0	fHakka	 tap8; tap7
U+5896.0	fCantonese	taap3
U+5896.0	fMandarin	ta3; da5
U+5896.0	fEnglish	 pagoda; tower; lighthouse
U+5897.0	fUTF8	増
U+5897.0	fCNS11643	3-4640
U+5897.0	fGB	E-2625
U+5897.0	fR/S	 32.11
U+5897.0	fTotalStrokes	 14
U+5897.0	fCangjie	XGCWA
U+5897.0	fFourCorner	48160
U+5897.0	fCantonese	zang1
U+5897.0	fMandarin	zeng1
U+5897.0	fEnglish	increase
U+5898.0	fUTF8	墘
U+5898.0	fCNS11643	2-4940
U+5898.0	fBig5	E159
U+5898.0	fGB	3-3469
U+5898.0	fR/S	 32.11
U+5898.0	fTotalStrokes	 14
U+5898.0	fCangjie	GJJN
U+5898.0	fFourCorner	48117
U+5898.0	fHakka	 kien2; ken2
U+5898.0	fCantonese	kin4; gin4; king4
U+5898.0	fMandarin	qi2
U+5898.0	fEnglish	 shore; bank; edge
U+5899.0	fUTF8	墙
U+5899.0	fGB	0-473D
U+5899.0	fR/S	 32.11
U+5899.0	fTotalStrokes	 14
U+5899.0	fCangjie	GGCW
U+5899.0	fHakka	 siong2; ciong2
U+5899.0	fCantonese	coeng4
U+5899.0	fMandarin	qiang2
U+5899.0	fEnglish	simplified form of &#x58BB;
U+589A.0	fUTF8	墚
U+589A.0	fCNS11643	4-4259
U+589A.0	fGB	0-5C2E
U+589A.0	fR/S	 32.11
U+589A.0	fTotalStrokes	 14
U+589A.0	fCangjie	GECD
U+589A.0	fFourCorner	43190
U+589A.0	fCantonese	loeng4
U+589A.0	fMandarin	liang2
U+589A.0	fEnglish	mountain range
U+589B.0	fUTF8	墛
U+589B.0	fCNS11643	3-6465
U+589B.0	fGB	E-2626
U+589B.0	fR/S	 32.11
U+589B.0	fTotalStrokes	 14
U+589B.0	fCangjie	GSFI
U+589B.0	fFourCorner	4410
U+589B.0	fCantonese	wai3
U+589C.0	fUTF8	墜
U+589C.0	fCNS11643	1-6C43
U+589C.0	fBig5	BC59
U+589C.0	fGB	1-5739
U+589C.0	fR/S	 32.12
U+589C.0	fTotalStrokes	 15
U+589C.0	fCangjie	NOG
U+589C.0	fFourCorner	78104
U+589C.0	fHakka	 zui5; ch'ui6; cui5; tui3; chui6; chui5; zhui5; tui5
U+589C.0	fCantonese	zeoi6; zui6
U+589C.0	fMandarin	zhui4
U+589C.0	fEnglish	 fall down; drop; sink; fall (of a person, a state, etc.)
U+589D.0	fUTF8	墝
U+589D.0	fCNS11643	2-4F72
U+589D.0	fBig5	E54B
U+589D.0	fGB	5-357A
U+589D.0	fR/S	 32.12
U+589D.0	fTotalStrokes	 15
U+589D.0	fCangjie	GGGU
U+589D.0	fFourCorner	44111
U+589D.0	fHakka	  kau1; kau3; ngiau2
U+589D.0	fCantonese	haau1
U+589D.0	fMandarin	qiao1; qiao4
U+589D.0	fEnglish	stony
U+589E.0	fUTF8	增
U+589E.0	fCNS11643	1-6C41
U+589E.0	fBig5	BC57
U+589E.0	fGB	0-5476
U+589E.0	fR/S	 32.12
U+589E.0	fTotalStrokes	 15
U+589E.0	fCangjie	GCWA
U+589E.0	fFourCorner	48166
U+589E.0	fHakka	  zen1;  zen5; zien1
U+589E.0	fCantonese	zang1; cang4
U+589E.0	fMandarin	zeng1; zeng4
U+589E.0	fEnglish	 add to; increase; grow; augment; enlarge
U+589F.0	fUTF8	墟
U+589F.0	fCNS11643	1-6C40
U+589F.0	fBig5	BC56
U+589F.0	fGB	0-5066
U+589F.0	fR/S	 32.12
U+589F.0	fTotalStrokes	 15
U+589F.0	fCangjie	GYPM
U+589F.0	fFourCorner	41117
U+589F.0	fHakka	  hi1
U+589F.0	fCantonese	heoi1
U+589F.0	fMandarin	xu1
U+589F.0	fEnglish	[1] high mound; ancient town; ghost town [2] wild, waste land [3] periodical market place where goods are bartered [4] ruin; destroy
U+58A0.0	fUTF8	墠
U+58A0.0	fCNS11643	2-4F74
U+58A0.0	fBig5	E54D
U+58A0.0	fGB	3-3444
U+58A0.0	fR/S	 32.12
U+58A0.0	fTotalStrokes	 15
U+58A0.0	fCangjie	GRRJ
U+58A0.0	fFourCorner	46156
U+58A0.0	fHakka	 shan3; san3; shen5; shan5; shen3
U+58A0.0	fCantonese	sin6
U+58A0.0	fMandarin	shan4; tan1
U+58A0.0	fEnglish	smooth; hard spot made level for sacrificial altar
U+58A1.0	fUTF8	墡
U+58A1.0	fCNS11643	2-4F79
U+58A1.0	fBig5	E552
U+58A1.0	fGB	3-3477
U+58A1.0	fR/S	 32.12
U+58A1.0	fTotalStrokes	 15
U+58A1.0	fCangjie	GTGR
U+58A1.0	fFourCorner	48161
U+58A1.0	fHakka	 shan5; san2; shen5
U+58A1.0	fCantonese	sin6
U+58A1.0	fMandarin	shan4
U+58A1.0	fEnglish	fine white clay used for walls
U+58A2.0	fUTF8	墢
U+58A2.0	fCNS11643	3-4B46
U+58A2.0	fGB	5-3573
U+58A2.0	fR/S	  32.12
U+58A2.0	fTotalStrokes	 15
U+58A2.0	fCangjie	GNOE
U+58A2.0	fFourCorner	42140
U+58A2.0	fHakka	 fat8
U+58A2.0	fCantonese	baat6
U+58A2.0	fMandarin	ba2; fei4; bo1
U+58A3.0	fUTF8	墣
U+58A3.0	fCNS11643	2-4F75
U+58A3.0	fBig5	E54E
U+58A3.0	fGB	3-3471
U+58A3.0	fR/S	 32.12
U+58A3.0	fTotalStrokes	 15
U+58A3.0	fCangjie	GTCO
U+58A3.0	fFourCorner	42135
U+58A3.0	fHakka	  pok7; puk7
U+58A3.0	fCantonese	pok3; buk6
U+58A3.0	fMandarin	pu2; pu1
U+58A3.0	fEnglish	 clod of earth
U+58A4.0	fUTF8	墤
U+58A4.0	fCNS11643	3-4B50
U+58A4.0	fGB	E-2627
U+58A4.0	fR/S	 32.12
U+58A4.0	fTotalStrokes	 15
U+58A4.0	fCangjie	GLMC
U+58A4.0	fFourCorner	45180
U+58A4.0	fHakka	 tui3
U+58A4.0	fCantonese	faai3; teoi4
U+58A4.0	fMandarin	tui2; kuai4
U+58A4.0	fEnglish	 fall in ruins; collapse; overthrow decayed; ruined; lost
U+58A5.0	fUTF8	墥
U+58A5.0	fCNS11643	2-4F78
U+58A5.0	fBig5	E551
U+58A5.0	fGB	3-3476
U+58A5.0	fR/S	 32.12
U+58A5.0	fTotalStrokes	 15
U+58A5.0	fCangjie	GYTG
U+58A5.0	fFourCorner	40114
U+58A5.0	fCantonese	dung2; teon2
U+58A5.0	fMandarin	dong3; tong2
U+58A6.0	fUTF8	墦
U+58A6.0	fCNS11643	1-6C46
U+58A6.0	fBig5	BC5C
U+58A6.0	fGB	3-3475
U+58A6.0	fR/S	 32.12
U+58A6.0	fTotalStrokes	 15
U+58A6.0	fCangjie	GHDW
U+58A6.0	fFourCorner	42169
U+58A6.0	fHakka	 fan1; ban1; fan2
U+58A6.0	fCantonese	faan4
U+58A6.0	fMandarin	fan2; pan1
U+58A6.0	fEnglish	tomb
U+58A7.0	fUTF8	墧
U+58A7.0	fCNS11643	3-4B51
U+58A7.0	fGB	E-2628
U+58A7.0	fR/S	 32.12
U+58A7.0	fTotalStrokes	 15
U+58A7.0	fCangjie	GHKB
U+58A7.0	fFourCorner	42120
U+58A7.0	fCantonese	kok3
U+58A7.0	fMandarin	que4; qiao2
U+58A8.0	fUTF8	墨
U+58A8.0	fCNS11643	1-702E
U+58A8.0	fBig5	BEA5
U+58A8.0	fGB	0-442B
U+58A8.0	fR/S	 32.12; 203.3; 32.11
U+58A8.0	fTotalStrokes	 15
U+58A8.0	fCangjie	WGFG
U+58A8.0	fFourCorner	60104
U+58A8.0	fHakka	  met8; miet8
U+58A8.0	fCantonese	mak6; maak6
U+58A8.0	fMandarin	mo4; mei2; mei4
U+58A8.0	fEnglish	[1] black; dark [2] a black dye [3] ink; a Chinese inkstick [4] calligraphy; writing; handwriting; painting [5] literate; letters; learning [6] statutes; institutions [7] greedy; covetous; corrupt [8] tattoo the face (one of the five punishments in ancient China) [9] a Chinese family name
U+58A9.0	fUTF8	墩
U+58A9.0	fCNS11643	1-6C45
U+58A9.0	fBig5	BC5B
U+58A9.0	fGB	0-3655
U+58A9.0	fR/S	 32.12
U+58A9.0	fTotalStrokes	 15
U+58A9.0	fCangjie	GYDK
U+58A9.0	fFourCorner	48147
U+58A9.0	fHakka	  dun1
U+58A9.0	fCantonese	deon1; dan1 (also); dan2
U+58A9.0	fMandarin	dun1
U+58A9.0	fEnglish	[1] heap; mound [2] block of stone or wood [3] cluster [4] [5] stick out [6] squat
U+58AA.0	fUTF8	墪
U+58AA.0	fCNS11643	3-4B4B
U+58AA.0	fGB	E-2629
U+58AA.0	fR/S	  32.12
U+58AA.0	fTotalStrokes	 15
U+58AA.0	fCangjie	YKG
U+58AA.0	fFourCorner	08100
U+58AA.0	fCantonese	deon1; dan1 (also); dan2
U+58AA.0	fMandarin	dun1
U+58AB.0	fUTF8	墫
U+58AB.0	fCNS11643	2-4F71
U+58AB.0	fBig5	E54A
U+58AB.0	fGB	5-3722
U+58AB.0	fR/S	 32.12
U+58AB.0	fTotalStrokes	 15
U+58AB.0	fCangjie	GTWI
U+58AB.0	fFourCorner	48146
U+58AB.0	fHakka	 zun1
U+58AB.0	fCantonese	zeon1 (also); deon1; cyun4; cun1
U+58AB.0	fMandarin	zun1; zun3; cun2
U+58AB.0	fEnglish	cup
U+58AC.0	fUTF8	墬
U+58AC.0	fCNS11643	2-4F77
U+58AC.0	fBig5	E550
U+58AC.0	fGB	5-366D
U+58AC.0	fR/S	 32.12
U+58AC.0	fTotalStrokes	 15
U+58AC.0	fCangjie	NOG
U+58AC.0	fFourCorner	77104
U+58AC.0	fHakka	 di3; zui5
U+58AC.0	fCantonese	dei6; deng6 (also); zeoi6
U+58AC.0	fMandarin	di4; zui4; de5
U+58AD.0	fUTF8	墭
U+58AD.0	fCNS11643	3-463F
U+58AD.0	fGB	5-3664
U+58AD.0	fR/S	 32.12; 32.11
U+58AD.0	fTotalStrokes	 14; 15
U+58AD.0	fCangjie	GIST
U+58AD.0	fFourCorner	43117
U+58AD.0	fCantonese	sing6
U+58AD.0	fMandarin	sheng4
U+58AE.0	fUTF8	墮
U+58AE.0	fCNS11643	1-6C44
U+58AE.0	fBig5	BC5A
U+58AE.0	fGB	1-3669
U+58AE.0	fR/S	 32.12
U+58AE.0	fTotalStrokes	 15
U+58AE.0	fCangjie	NBG
U+58AE.0	fFourCorner	74104
U+58AE.0	fHakka	 to5; to6; to3
U+58AE.0	fCantonese	do6; fai1
U+58AE.0	fMandarin	duo4; hui1; hui3; tuo3
U+58AE.0	fEnglish	[1] fall; sink; let fall [2] indulge in evil ways [3] lazy; idle
U+58AF.0	fUTF8	墯
U+58AF.0	fCNS11643	2-4F76
U+58AF.0	fBig5	E54F
U+58AF.0	fGB	5-3671
U+58AF.0	fR/S	 32.12
U+58AF.0	fTotalStrokes	 15
U+58AF.0	fCangjie	PBG
U+58AF.0	fFourCorner	94104
U+58AF.0	fCantonese	do6; fai1
U+58AF.0	fMandarin	duo4; hui1
U+58B0.0	fUTF8	墰
U+58B0.0	fCNS11643	3-4B4D
U+58B0.0	fGB	5-366E
U+58B0.0	fR/S	 32.12
U+58B0.0	fTotalStrokes	 15
U+58B0.0	fCangjie	GMWJ
U+58B0.0	fFourCorner	41140
U+58B0.0	fHakka	 tam2
U+58B0.0	fCantonese	taam4
U+58B0.0	fMandarin	tan2
U+58B0.0	fEnglish	 earthenware jar; jug
U+58B1.0	fUTF8	墱
U+58B1.0	fCNS11643	2-4F73
U+58B1.0	fBig5	E54C
U+58B1.0	fGB	5-3672
U+58B1.0	fR/S	 32.12
U+58B1.0	fTotalStrokes	 15
U+58B1.0	fCangjie	GNOT
U+58B1.0	fFourCorner	47118
U+58B1.0	fHakka	  den5
U+58B1.0	fCantonese	dang3
U+58B1.0	fMandarin	deng4; yan4; deng1
U+58B1.0	fEnglish	steps; mount up
U+58B2.0	fUTF8	墲
U+58B2.0	fCNS11643	3-4B49
U+58B2.0	fGB	3-3363
U+58B2.0	fR/S	  32.12
U+58B2.0	fTotalStrokes	 15
U+58B2.0	fCangjie	GOTF
U+58B2.0	fFourCorner	48130
U+58B2.0	fCantonese	mou4
U+58B2.0	fMandarin	wu3; mu2; wu2
U+58B3.0	fUTF8	墳
U+58B3.0	fCNS11643	1-6C42
U+58B3.0	fBig5	BC58
U+58B3.0	fGB	1-3758
U+58B3.0	fR/S	 32.12
U+58B3.0	fTotalStrokes	 15
U+58B3.0	fCangjie	GJTC
U+58B3.0	fFourCorner	44186
U+58B3.0	fHakka	  fun2; pun2; fun3
U+58B3.0	fCantonese	fan4; fan5
U+58B3.0	fMandarin	fen2; ben4; fen4
U+58B3.0	fEnglish	[1] grave; mound [2] big; large; great [3] the banks of a river
U+58B4.0	fUTF8	墴
U+58B4.0	fCNS11643	3-4B4F
U+58B4.0	fGB	5-365F
U+58B4.0	fR/S	  32.12
U+58B4.0	fTotalStrokes	 15
U+58B4.0	fCangjie	GTMC
U+58B4.0	fFourCorner	44180
U+58B4.0	fCantonese	wong4
U+58B4.0	fMandarin	huang2
U+58B5.0	fUTF8	墵
U+58B5.0	fCNS11643	3-4B4C
U+58B5.0	fGB	E-262A
U+58B5.0	fR/S	  32.12
U+58B5.0	fTotalStrokes	 15
U+58B5.0	fCangjie	GMBI
U+58B5.0	fFourCorner	41130
U+58B5.0	fCantonese	taam4
U+58B5.0	fMandarin	tan2
U+58B6.0	fUTF8	墶
U+58B6.0	fGB	3-337C
U+58B6.0	fR/S	 32.12
U+58B6.0	fTotalStrokes	 15
U+58B6.0	fCangjie	GYGQ
U+58B6.0	fCantonese	daat3; daat6 (also)
U+58B6.0	fMandarin	da1; da5
U+58B7.0	fUTF8	墷
U+58B7.0	fCNS11643	3-4B47
U+58B7.0	fGB	3-3421
U+58B7.0	fR/S	  32.12
U+58B7.0	fTotalStrokes	 15
U+58B7.0	fCangjie	GTMJ
U+58B7.0	fFourCorner	44150
U+58B7.0	fCantonese	jip6
U+58B7.0	fMandarin	ye4
U+58B8.0	fUTF8	墸
U+58B8.0	fGB	E-262B
U+58B8.0	fR/S	 32.12
U+58B8.0	fTotalStrokes	 15
U+58B8.0	fCangjie	GTJA
U+58B8.0	fCantonese	zyu3
U+58B8.0	fMandarin	chu2
U+58B9.0	fUTF8	墹
U+58B9.0	fGB	E-262C
U+58B9.0	fR/S	 32.12
U+58B9.0	fTotalStrokes	 15
U+58B9.0	fCangjie	GANA
U+58B9.0	fCantonese	gaan1
U+58BA.0	fUTF8	墺
U+58BA.0	fCNS11643	2-5657
U+58BA.0	fBig5	E94D
U+58BA.0	fGB	3-3474
U+58BA.0	fR/S	 32.13
U+58BA.0	fTotalStrokes	 16
U+58BA.0	fCangjie	GHBK
U+58BA.0	fFourCorner	47134
U+58BA.0	fHakka	 au5; juk7; ruk7
U+58BA.0	fCantonese	ou3; ngou3 (also)
U+58BA.0	fMandarin	ao4; yu4
U+58BA.0	fEnglish	4 walls
U+58BB.0	fUTF8	墻
U+58BB.0	fCNS11643	3-504A
U+58BB.0	fGB	1-473D
U+58BB.0	fR/S	  32.13
U+58BB.0	fTotalStrokes	 16
U+58BB.0	fCangjie	GGOW
U+58BB.0	fFourCorner	44161
U+58BB.0	fHakka	  siong2; ciong2
U+58BB.0	fCantonese	coeng4
U+58BB.0	fMandarin	qiang2
U+58BB.0	fEnglish	wall
U+58BC.0	fUTF8	墼
U+58BC.0	fCNS11643	2-5659
U+58BC.0	fBig5	E94F
U+58BC.0	fGB	0-5B54
U+58BC.0	fR/S	 32.13
U+58BC.0	fTotalStrokes	 16
U+58BC.0	fCangjie	JEG
U+58BC.0	fFourCorner	57104
U+58BC.0	fHakka	  git7
U+58BC.0	fCantonese	gik1
U+58BC.0	fMandarin	ji1; ji2
U+58BC.0	fEnglish	unburnt bricks
U+58BD.0	fUTF8	墽
U+58BD.0	fCNS11643	2-5654
U+58BD.0	fBig5	E94A
U+58BD.0	fGB	3-3479
U+58BD.0	fR/S	 32.13
U+58BD.0	fTotalStrokes	 16
U+58BD.0	fCangjie	GHSK
U+58BD.0	fFourCorner	48140
U+58BD.0	fCantonese	hiu1; haau1
U+58BD.0	fMandarin	qiao1
U+58BE.0	fUTF8	墾
U+58BE.0	fCNS11643	1-704A
U+58BE.0	fBig5	BEC1
U+58BE.0	fGB	1-3F51
U+58BE.0	fR/S	 32.13
U+58BE.0	fTotalStrokes	 16
U+58BE.0	fCangjie	BVG
U+58BE.0	fFourCorner	27104
U+58BE.0	fHakka	  ken3; kien3
U+58BE.0	fCantonese	han2
U+58BE.0	fMandarin	ken3
U+58BE.0	fEnglish	 open new land for farming, etc.; cultivate; reclaim; farm land
U+58BF.0	fUTF8	墿
U+58BF.0	fCNS11643	2-5656
U+58BF.0	fBig5	E94C
U+58BF.0	fGB	5-3574
U+58BF.0	fR/S	 32.13
U+58BF.0	fTotalStrokes	 16
U+58BF.0	fCangjie	GWLJ
U+58BF.0	fFourCorner	46141
U+58BF.0	fCantonese	jik6; tou4
U+58BF.0	fMandarin	yi4; du4; tu2
U+58C0.0	fUTF8	壀
U+58C0.0	fCNS11643	3-504B
U+58C0.0	fGB	E-262D
U+58C0.0	fR/S	 32.13
U+58C0.0	fTotalStrokes	 16
U+58C0.0	fCangjie	GSRJ
U+58C0.0	fFourCorner	40140
U+58C0.0	fHakka	  bi1
U+58C0.0	fCantonese	pei4; bei1; bai3 (also); pai6; bei3 (also); pai3
U+58C0.0	fMandarin	pi2; bi4; pi4 bei1
U+58C0.0	fEnglish	same as &#x57E4;
U+58C1.0	fUTF8	壁
U+58C1.0	fCNS11643	1-7049
U+58C1.0	fBig5	BEC0
U+58C1.0	fGB	0-315A
U+58C1.0	fR/S	 32.13
U+58C1.0	fTotalStrokes	 16
U+58C1.0	fCangjie	SJG
U+58C1.0	fFourCorner	70104
U+58C1.0	fHakka	  biak7
U+58C1.0	fCantonese	bik1; bek3 (also); bik3 (also)
U+58C1.0	fMandarin	bi4; bi3
U+58C1.0	fEnglish	[1] partition wall; walls of a room [2] military breastwork [3] cliff
U+58C2.0	fUTF8	壂
U+58C2.0	fCNS11643	2-5658
U+58C2.0	fBig5	E94E
U+58C2.0	fGB	3-347A
U+58C2.0	fR/S	 32.13
U+58C2.0	fTotalStrokes	 16
U+58C2.0	fCangjie	SEG
U+58C2.0	fFourCorner	77104
U+58C2.0	fCantonese	din6
U+58C2.0	fMandarin	dian4
U+58C3.0	fUTF8	壃
U+58C3.0	fCNS11643	3-5048
U+58C3.0	fGB	E-262E
U+58C3.0	fR/S	 32.13
U+58C3.0	fTotalStrokes	 16
U+58C3.0	fCangjie	GMWM
U+58C3.0	fFourCorner	41110
U+58C3.0	fHakka	  giong1;  kiong1
U+58C3.0	fCantonese	goeng1
U+58C3.0	fMandarin	jiang1; meng4; qiang2; qiang3; jiang4
U+58C3.0	fEnglish	same as &#x7586;
U+58C4.0	fUTF8	壄
U+58C4.0	fCNS11643	3-504C
U+58C4.0	fGB	E-262F
U+58C4.0	fR/S	  32.13
U+58C4.0	fTotalStrokes	 16
U+58C4.0	fCangjie	DDG
U+58C4.0	fFourCorner	44100
U+58C4.0	fCantonese	je5
U+58C4.0	fMandarin	ye3; mou4
U+58C5.0	fUTF8	壅
U+58C5.0	fCNS11643	1-704C
U+58C5.0	fBig5	BEC3
U+58C5.0	fGB	0-5B55
U+58C5.0	fR/S	 32.13
U+58C5.0	fTotalStrokes	 16
U+58C5.0	fCangjie	YVGG
U+58C5.0	fFourCorner	00104
U+58C5.0	fHakka	  jung3;  jung1; rung1; rung3
U+58C5.0	fCantonese	jung1; jung2 (also); ngung1 (also); ung1
U+58C5.0	fMandarin	yong1; yong3; yong4
U+58C5.0	fEnglish	[1] stop; block up [2] bank up the roots of plants [3] obstruct; impede (flow, etc.)
U+58C6.0	fUTF8	壆
U+58C6.0	fCNS11643	2-565A
U+58C6.0	fBig5	E950
U+58C6.0	fGB	3-3378
U+58C6.0	fR/S	 32.13
U+58C6.0	fTotalStrokes	 16
U+58C6.0	fCangjie	HBG
U+58C6.0	fFourCorner	77104
U+58C6.0	fHakka	  bok7; gok7; hok7; hok8
U+58C6.0	fCantonese	bok3; hok6
U+58C6.0	fMandarin	xue2; bo2
U+58C6.0	fEnglish	 stiff; hard clay or rocky strata crack in a jar
U+58C7.0	fUTF8	壇
U+58C7.0	fCNS11643	1-704B
U+58C7.0	fBig5	BEC2
U+58C7.0	fGB	1-4C33
U+58C7.0	fR/S	 32.13
U+58C7.0	fTotalStrokes	 16
U+58C7.0	fCangjie	GYWM
U+58C7.0	fFourCorner	40116
U+58C7.0	fHakka	  tan2; tan3
U+58C7.0	fCantonese	taan4
U+58C7.0	fMandarin	tan2; dan4; shan4
U+58C7.0	fEnglish	[1] a platform for sacrificial rites; altar [2] a hall for important meetings and ceremonies in ancient China; examination hall
U+58C8.0	fUTF8	壈
U+58C8.0	fCNS11643	2-5653
U+58C8.0	fBig5	E949
U+58C8.0	fGB	E-2630
U+58C8.0	fR/S	 32.13
U+58C8.0	fTotalStrokes	 16
U+58C8.0	fCangjie	GYWD
U+58C8.0	fFourCorner	40194
U+58C8.0	fHakka	 lam3
U+58C8.0	fCantonese	lam5
U+58C8.0	fMandarin	lan3; lai3
U+58C8.0	fEnglish	disappointed
U+58C9.0	fUTF8	壉
U+58C9.0	fCNS11643	2-5655
U+58C9.0	fBig5	E94B
U+58C9.0	fGB	E-2631
U+58C9.0	fR/S	 32.13
U+58C9.0	fTotalStrokes	 16
U+58C9.0	fCangjie	GYPO
U+58C9.0	fFourCorner	41132
U+58C9.0	fCantonese	geoi6; geoi3
U+58C9.0	fMandarin	ju4
U+58CA.0	fUTF8	壊
U+58CA.0	fCNS11643	3-5049
U+58CA.0	fGB	E-2632
U+58CA.0	fR/S	 32.13
U+58CA.0	fTotalStrokes	 16
U+58CA.0	fCangjie	GJWV
U+58CA.0	fFourCorner	44130
U+58CA.0	fHakka	 fai5; jong5
U+58CA.0	fCantonese	waai6; joeng6
U+58CA.0	fMandarin	huai4; rang3
U+58CA.0	fEnglish	[1] broken down; bad; spoil(ed); decaying; rotten; out of order; useless [2] bad; poor (scores, marks, etc.) [3] vicious; mean; evil (persons, etc.) [4] simplified form of &#x58E4;
U+58CB.0	fUTF8	壋
U+58CB.0	fGB	3-337D
U+58CB.0	fR/S	 32.13
U+58CB.0	fTotalStrokes	 16
U+58CB.0	fCangjie	GFBW
U+58CB.0	fFourCorner	4916
U+58CB.0	fCantonese	dong2 (also); dong3
U+58CB.0	fMandarin	dang4
U+58CC.0	fUTF8	壌
U+58CC.0	fGB	E-2633
U+58CC.0	fR/S	 32.13
U+58CC.0	fTotalStrokes	 16
U+58CC.0	fCangjie	GYCV
U+58CC.0	fHakka	 jong5
U+58CC.0	fCantonese	joeng6
U+58CC.0	fMandarin	rang3
U+58CC.0	fEnglish	 soil; loam; earth rich
U+58CD.0	fUTF8	壍
U+58CD.0	fCNS11643	3-544B
U+58CD.0	fGB	5-366B
U+58CD.0	fR/S	 32.14
U+58CD.0	fTotalStrokes	 17
U+58CD.0	fCangjie	ELG
U+58CD.0	fFourCorner	32100
U+58CD.0	fHakka	 ciam5
U+58CD.0	fCantonese	cim3
U+58CD.0	fMandarin	qian4
U+58CD.0	fEnglish	moat around a city. a channel
U+58CE.0	fUTF8	壎
U+58CE.0	fCNS11643	1-734E
U+58CE.0	fBig5	C0A5
U+58CE.0	fGB	E-2634
U+58CE.0	fR/S	 32.14
U+58CE.0	fTotalStrokes	 17
U+58CE.0	fCangjie	GHGF
U+58CE.0	fFourCorner	42131
U+58CE.0	fHakka	  hiun1
U+58CE.0	fCantonese	hyun1; fan1 (also)
U+58CE.0	fMandarin	xun1; xuan1; xun4
U+58CE.0	fEnglish	an ancient Chinese wind instrument, made of porcelain and shaped like an egg
U+58CF.0	fUTF8	壏
U+58CF.0	fCNS11643	2-5C58
U+58CF.0	fBig5	ECCC
U+58CF.0	fGB	3-345A
U+58CF.0	fR/S	 32.14
U+58CF.0	fTotalStrokes	 17
U+58CF.0	fCangjie	GSIT
U+58CF.0	fFourCorner	48117
U+58CF.0	fCantonese	haam5
U+58CF.0	fMandarin	lan4; jian4; han3
U+58D0.0	fUTF8	壐
U+58D0.0	fCNS11643	3-544C
U+58D0.0	fGB	5-3571
U+58D0.0	fR/S	  32.14
U+58D0.0	fTotalStrokes	 17
U+58D0.0	fCangjie	MBG
U+58D0.0	fFourCorner	10100
U+58D0.0	fHakka	  sai3
U+58D0.0	fCantonese	saai2
U+58D0.0	fMandarin	xi3
U+58D0.0	fEnglish	 the Imperial signet the great seal
U+58D1.0	fUTF8	壑
U+58D1.0	fCNS11643	1-734D
U+58D1.0	fBig5	C0A4
U+58D1.0	fGB	0-5B56
U+58D1.0	fR/S	 32.14
U+58D1.0	fTotalStrokes	 17
U+58D1.0	fCangjie	YEG
U+58D1.0	fFourCorner	27104
U+58D1.0	fHakka	 kok7
U+58D1.0	fCantonese	kok3
U+58D1.0	fMandarin	he4; huo4; obs
U+58D1.0	fEnglish	[1] a gully; channel for water [2] a narrow ravine at the floor of a hill
U+58D2.0	fUTF8	壒
U+58D2.0	fCNS11643	2-5C59
U+58D2.0	fBig5	ECCD
U+58D2.0	fGB	5-3674
U+58D2.0	fR/S	 32.14
U+58D2.0	fTotalStrokes	 17
U+58D2.0	fCangjie	GTGT
U+58D2.0	fFourCorner	44117
U+58D2.0	fHakka	  ai5
U+58D2.0	fCantonese	oi2; ngoi2 (also); oi3; ngoi3
U+58D2.0	fMandarin	kai4; ai4
U+58D2.0	fEnglish	dust in the air; obscured as in a dust storm
U+58D3.0	fUTF8	壓
U+58D3.0	fCNS11643	1-734C
U+58D3.0	fBig5	C0A3
U+58D3.0	fGB	1-5139
U+58D3.0	fR/S	 32.14
U+58D3.0	fTotalStrokes	 17
U+58D3.0	fCangjie	MKG
U+58D3.0	fFourCorner	71214
U+58D3.0	fHakka	  ap7; a1; a2
U+58D3.0	fCantonese	aat3; ngaat3 (also)
U+58D3.0	fMandarin	ya1; ya4; yan4
U+58D3.0	fEnglish	[1] press; oppress; pressure [2] control; quell [3] crush [4] close in; press near (said of enemy troops) [5] hold without taking action (a document, etc.) pigeonhole; shelve [6] a way of making a stroke in calligraphy [7] excel; surpass others (in ability, etc.)
U+58D4.0	fUTF8	壔
U+58D4.0	fCNS11643	2-5C57
U+58D4.0	fBig5	ECCB
U+58D4.0	fGB	3-342B
U+58D4.0	fR/S	 32.14
U+58D4.0	fTotalStrokes	 17
U+58D4.0	fCangjie	GGNI
U+58D4.0	fFourCorner	44141
U+58D4.0	fHakka	  dau3; do3
U+58D4.0	fCantonese	dou2
U+58D4.0	fMandarin	dao3
U+58D4.0	fEnglish	a mound; a tumulus
U+58D5.0	fUTF8	壕
U+58D5.0	fCNS11643	1-734B
U+58D5.0	fBig5	C0A2
U+58D5.0	fGB	0-3A3E
U+58D5.0	fR/S	 32.14
U+58D5.0	fTotalStrokes	 17
U+58D5.0	fCangjie	GYRO
U+58D5.0	fFourCorner	40132
U+58D5.0	fHakka	  hau2; ho2
U+58D5.0	fCantonese	hou4
U+58D5.0	fMandarin	hao2
U+58D5.0	fEnglish	[1] trench; ditch; dugout [2] moat; the ditch around a city wall
U+58D6.0	fUTF8	壖
U+58D6.0	fCNS11643	2-5C56
U+58D6.0	fBig5	ECCA
U+58D6.0	fGB	3-347B
U+58D6.0	fR/S	 32.14
U+58D6.0	fTotalStrokes	 17
U+58D6.0	fCangjie	GMBB
U+58D6.0	fFourCorner	41127
U+58D6.0	fHakka	 ngion1; nion1
U+58D6.0	fCantonese	jyun4
U+58D6.0	fMandarin	ruan2
U+58D6.0	fEnglish	open space along water
U+58D7.0	fUTF8	壗
U+58D7.0	fGB	E-2635
U+58D7.0	fR/S	 32.14
U+58D7.0	fTotalStrokes	 17
U+58D7.0	fCangjie	GLMT
U+58D7.0	fCantonese	zeon6
U+58D8.0	fUTF8	壘
U+58D8.0	fCNS11643	1-763F
U+58D8.0	fBig5	C253
U+58D8.0	fGB	1-405D
U+58D8.0	fR/S	 32.15
U+58D8.0	fTotalStrokes	 18
U+58D8.0	fCangjie	WWWG
U+58D8.0	fFourCorner	60104
U+58D8.0	fHakka	  lui3
U+58D8.0	fCantonese	leoi5; leot6; leoi6; leoi4
U+58D8.0	fMandarin	lei3; lei4; lei2; l4
U+58D8.0	fEnglish	[1] rampart; military wall [2] pile up [3] base (on a baseball diamond) [4] a Chinese family name
U+58D9.0	fUTF8	壙
U+58D9.0	fCNS11643	1-763E
U+58D9.0	fBig5	C252
U+58D9.0	fGB	1-5B5B
U+58D9.0	fR/S	 32.15
U+58D9.0	fTotalStrokes	 18
U+58D9.0	fCangjie	GITC
U+58D9.0	fFourCorner	40186
U+58D9.0	fHakka	  kong5; kwong3; k'wong3
U+58D9.0	fCantonese	kwong3; kong3 (also)
U+58D9.0	fMandarin	kuang4
U+58D9.0	fEnglish	[1] vault; tomb; grave [2] field; prairie; open wide; open or wide space
U+58DA.0	fUTF8	壚
U+58DA.0	fCNS11643	2-6542
U+58DA.0	fBig5	F1F6
U+58DA.0	fGB	1-5B64
U+58DA.0	fR/S	 32.16
U+58DA.0	fTotalStrokes	 19
U+58DA.0	fCangjie	GYPT
U+58DA.0	fFourCorner	41117
U+58DA.0	fHakka	 lu2
U+58DA.0	fCantonese	lou4
U+58DA.0	fMandarin	lu2
U+58DA.0	fEnglish	 black clods of earth shop; hut
U+58DB.0	fUTF8	壛
U+58DB.0	fCNS11643	2-6544
U+58DB.0	fBig5	F1F8
U+58DB.0	fGB	5-366A
U+58DB.0	fR/S	 32.16
U+58DB.0	fTotalStrokes	 19
U+58DB.0	fCangjie	GANX
U+58DB.0	fFourCorner	47120
U+58DB.0	fCantonese	jim4; sim4 (also); jam4 (also)
U+58DB.0	fMandarin	yan2
U+58DC.0	fUTF8	壜
U+58DC.0	fCNS11643	3-5A2F
U+58DC.0	fGB	E-2636
U+58DC.0	fR/S	  32.16
U+58DC.0	fTotalStrokes	 19
U+58DC.0	fCangjie	GAMI
U+58DC.0	fFourCorner	46130
U+58DC.0	fHakka	  tam2
U+58DC.0	fCantonese	taam4; tam4
U+58DC.0	fMandarin	tan2
U+58DC.0	fEnglish	earthen jar or jug
U+58DD.0	fUTF8	壝
U+58DD.0	fCNS11643	2-6543
U+58DD.0	fBig5	F1F7
U+58DD.0	fGB	3-3473
U+58DD.0	fR/S	 32.16
U+58DD.0	fTotalStrokes	 19
U+58DD.0	fCangjie	GYLC
U+58DD.0	fFourCorner	45138
U+58DD.0	fHakka	 wui3; wi3; vui3
U+58DD.0	fCantonese	wai2 (also); wai4; wai5 (also)
U+58DD.0	fMandarin	wei2; wei3; wei4; yi3
U+58DD.0	fEnglish	a mound; an embankment the earthen altar to the god of the soil
U+58DE.0	fUTF8	壞
U+58DE.0	fCNS11643	1-782E
U+58DE.0	fBig5	C361
U+58DE.0	fGB	1-3B35
U+58DE.0	fR/S	 32.16
U+58DE.0	fTotalStrokes	 19
U+58DE.0	fCangjie	GYWV
U+58DE.0	fFourCorner	44132
U+58DE.0	fHakka	  fai3; fai5; fai6; pai6; pai5
U+58DE.0	fCantonese	waai6; waai2 (also)
U+58DE.0	fMandarin	huai4; pi1; guai4
U+58DE.0	fEnglish	bad; spoil(ed); ruin; destroy
U+58DF.0	fUTF8	壟
U+58DF.0	fCNS11643	1-782F
U+58DF.0	fBig5	C362
U+58DF.0	fGB	1-4222
U+58DF.0	fR/S	 32.16
U+58DF.0	fTotalStrokes	 19
U+58DF.0	fCangjie	YPG
U+58DF.0	fFourCorner	01104
U+58DF.0	fHakka	  lung3;  liung1; lung1; liung2
U+58DF.0	fCantonese	lung5
U+58DF.0	fMandarin	long3; long1
U+58DF.0	fEnglish	[1] grave; mound of earth [2] a high place in a field
U+58E0.0	fUTF8	壠
U+58E0.0	fCNS11643	3-5A2E
U+58E0.0	fGB	1-5B62
U+58E0.0	fR/S	  32.16
U+58E0.0	fTotalStrokes	 19
U+58E0.0	fCangjie	GYBP
U+58E0.0	fFourCorner	41110
U+58E0.0	fHakka	 liung1; lung2; lung5; lung3; liung2
U+58E0.0	fCantonese	lung5
U+58E0.0	fMandarin	long3
U+58E0.0	fEnglish	mound; grave; ridge in field
U+58E1.0	fUTF8	壡
U+58E1.0	fCNS11643	7-4645
U+58E1.0	fGB	E-2637
U+58E1.0	fR/S	 32.16; 29.17; 32.17
U+58E1.0	fTotalStrokes	 19
U+58E1.0	fCangjie	YUEG
U+58E1.0	fFourCorner	27610
U+58E1.0	fCantonese	jeoi6
U+58E1.0	fMandarin	rui4; ruo4
U+58E2.0	fUTF8	壢
U+58E2.0	fCNS11643	1-7830
U+58E2.0	fBig5	C363
U+58E2.0	fGB	1-5B5E
U+58E2.0	fR/S	 32.16
U+58E2.0	fTotalStrokes	 19
U+58E2.0	fCangjie	GMHM
U+58E2.0	fFourCorner	41111
U+58E2.0	fHakka	 lak7; lak8
U+58E2.0	fCantonese	lek6; lik6
U+58E2.0	fMandarin	li4
U+58E2.0	fEnglish	a drain or water channel
U+58E3.0	fUTF8	壣
U+58E3.0	fCNS11643	2-686E
U+58E3.0	fBig5	F442
U+58E3.0	fGB	E-2638
U+58E3.0	fR/S	 32.17
U+58E3.0	fTotalStrokes	 20
U+58E3.0	fCangjie	GSJT
U+58E3.0	fFourCorner	42172
U+58E3.0	fCantonese	leon4
U+58E3.0	fMandarin	lin2
U+58E4.0	fUTF8	壤
U+58E4.0	fCNS11643	1-7966
U+58E4.0	fBig5	C45B
U+58E4.0	fGB	0-4840
U+58E4.0	fR/S	 32.17
U+58E4.0	fTotalStrokes	 20
U+58E4.0	fCangjie	GYRV
U+58E4.0	fFourCorner	40132
U+58E4.0	fHakka	 rong3; jong3; ngiong5; ngiong3; jong5
U+58E4.0	fCantonese	joeng6; joeng5
U+58E4.0	fMandarin	rang3
U+58E4.0	fEnglish	[1] loose soil; loam [2] earth [3] a region; place; land [4] rich; abundant [5] a Chinese family name
U+58E5.0	fUTF8	壥
U+58E5.0	fGB	E-2639
U+58E5.0	fR/S	 32.17
U+58E5.0	fTotalStrokes	 20
U+58E5.0	fCangjie	XGMWG
U+58E5.0	fCantonese	cin4
U+58E5.0	fMandarin	chan2
U+58E6.0	fUTF8	壦
U+58E6.0	fCNS11643	3-5E28
U+58E6.0	fGB	E-263A
U+58E6.0	fR/S	 32.18
U+58E6.0	fTotalStrokes	 21
U+58E6.0	fCangjie	GTRG
U+58E6.0	fFourCorner	44110
U+58E6.0	fHakka	  hiun1
U+58E6.0	fCantonese	hyun1; fan1 (also)
U+58E6.0	fMandarin	xun1; xuan1; xun4
U+58E6.0	fEnglish	ancient form of &#x58CE;
U+58E7.0	fUTF8	壧
U+58E7.0	fCNS11643	2-6F22
U+58E7.0	fBig5	F7D3
U+58E7.0	fGB	5-3627
U+58E7.0	fR/S	 32.20
U+58E7.0	fTotalStrokes	 23
U+58E7.0	fCangjie	GRRK
U+58E7.0	fFourCorner	46148
U+58E7.0	fHakka	 ngam5; ngap7
U+58E7.0	fCantonese	ngaam4; aam4 (also)
U+58E7.0	fMandarin	yan2
U+58E8.0	fUTF8	壨
U+58E8.0	fCNS11643	2-6F21
U+58E8.0	fBig5	F7D2
U+58E8.0	fGB	E-263B
U+58E8.0	fR/S	 32.20
U+58E8.0	fTotalStrokes	 23
U+58E8.0	fCangjie	WWWWG
U+58E8.0	fFourCorner	66104
U+58E8.0	fCantonese	leoi4
U+58E8.0	fMandarin	lei2
U+58E9.0	fUTF8	壩
U+58E9.0	fCNS11643	1-7C5E
U+58E9.0	fBig5	C5F2
U+58E9.0	fGB	1-3053
U+58E9.0	fR/S	 32.21
U+58E9.0	fTotalStrokes	 24
U+58E9.0	fCangjie	GMBB
U+58E9.0	fFourCorner	41127
U+58E9.0	fHakka	 ba5
U+58E9.0	fCantonese	baa3
U+58E9.0	fMandarin	ba4
U+58E9.0	fEnglish	[1] an embankment; dike [2] a dam
U+58EA.0	fUTF8	壪
U+58EA.0	fGB	3-3453
U+58EA.0	fR/S	 32.22
U+58EA.0	fTotalStrokes	 25
U+58EA.0	fCangjie	GVFN
U+58EA.0	fCantonese	waan1
U+58EB.0	fUTF8	士
U+58EB.0	fCNS11643	1-4449
U+58EB.0	fBig5	A468
U+58EB.0	fGB	0-4A3F
U+58EB.0	fR/S	 33.0
U+58EB.0	fTotalStrokes	 3
U+58EB.0	fCangjie	JM
U+58EB.0	fFourCorner	40100
U+58EB.0	fHakka	 sii6; su3; sii5; su5; se5
U+58EB.0	fCantonese	si6; si2 (also)
U+58EB.0	fMandarin	shi4
U+58EB.0	fEnglish	[1] a scholar; man of learning; gentleman [2] an official rank in ancient China; officer [3] an incommissioned officer [4] person [5] name of a chess piece [6] a Chinese family name
U+58EB.1	fUTF8	士
U+58EB.1	fCNS11643	1-4449
U+58EB.1	fBig5	A468
U+58EB.1	fGB	0-4A3F
U+58EB.1	fR/S	 33.0
U+58EB.1	fTotalStrokes	 3
U+58EB.1	fCangjie	JM
U+58EB.1	fFourCorner	40100
U+58EB.1	fMandarin	shi5
U+58EC.0	fUTF8	壬
U+58EC.0	fCNS11643	1-4531
U+58EC.0	fBig5	A4D0
U+58EC.0	fGB	0-4849
U+58EC.0	fR/S	 33.1
U+58EC.0	fTotalStrokes	 4
U+58EC.0	fCangjie	HG
U+58EC.0	fFourCorner	20104
U+58EC.0	fHakka	 ngim2; jim2; gnim3; ngim3; rim2
U+58EC.0	fCantonese	jam4
U+58EC.0	fMandarin	ren2
U+58EC.0	fEnglish	[1] ninth of ten celestial stems [2] artful and crafty [3] great [4] pregnant
U+58ED.0	fUTF8	壭
U+58ED.0	fGB	1-7D47
U+58ED.0	fR/S	 33.2
U+58ED.0	fTotalStrokes	 5
U+58ED.0	fCangjie	GSL
U+58ED.0	fCantonese	si6
U+58ED.0	fEnglish	 kwukyel
U+58EE.0	fUTF8	壮
U+58EE.0	fCNS11643	3-2343
U+58EE.0	fGB	0-5733
U+58EE.0	fR/S	 33.3
U+58EE.0	fTotalStrokes	 6
U+58EE.0	fCangjie	LMG
U+58EE.0	fFourCorner	34110
U+58EE.0	fHakka	 zong5
U+58EE.0	fCantonese	zong3; zong1
U+58EE.0	fMandarin	zhuang4; zhuan4; zhuang1
U+58EE.0	fEnglish	simplified form of &#x58EF;
U+58EF.0	fUTF8	壯
U+58EF.0	fCNS11643	1-4967
U+58EF.0	fBig5	A7A7
U+58EF.0	fGB	1-5733
U+58EF.0	fR/S	 33.4
U+58EF.0	fTotalStrokes	 7
U+58EF.0	fCangjie	VMG
U+58EF.0	fFourCorner	24210
U+58EF.0	fHakka	  zong5
U+58EF.0	fCantonese	zong3; zong1; zong6
U+58EF.0	fMandarin	zhuang4; zhuan4; zhuang1
U+58EF.0	fEnglish	[1] big; large; great [2] strong; robust; vigorous; sturdy [3] portly; stout [4] strengthen; make better [5] the prime of one's life [6] name of tribe [7] another name for the eight moon of the lunar calendar
U+58F0.0	fUTF8	声
U+58F0.0	fCNS11643	3-2545
U+58F0.0	fGB	0-4979
U+58F0.0	fR/S	 33.4
U+58F0.0	fTotalStrokes	 7
U+58F0.0	fCangjie	GAH
U+58F0.0	fFourCorner	40270
U+58F0.0	fHakka	 s'ang1; shang1
U+58F0.0	fCantonese	sing1; seng1
U+58F0.0	fMandarin	sheng1
U+58F0.0	fEnglish	simplified form of &#x8072;
U+58F0.1	fUTF8	声
U+58F0.1	fCNS11643	3-2545
U+58F0.1	fGB	0-4979
U+58F0.1	fR/S	 33.4
U+58F0.1	fTotalStrokes	 7
U+58F0.1	fCangjie	GAH
U+58F0.1	fFourCorner	40270
U+58F0.1	fMandarin	sheng5
U+58F1.0	fUTF8	壱
U+58F1.0	fCNS11643	3-2543
U+58F1.0	fGB	E-263C
U+58F1.0	fR/S	 33.4
U+58F1.0	fTotalStrokes	 7
U+58F1.0	fCangjie	GBP
U+58F1.0	fFourCorner	40710
U+58F1.0	fCantonese	jat1
U+58F1.0	fMandarin	yi1
U+58F1.0	fEnglish	number one
U+58F2.0	fUTF8	売
U+58F2.0	fCNS11643	3-2544
U+58F2.0	fGB	E-263D
U+58F2.0	fR/S	 33.4
U+58F2.0	fTotalStrokes	 7
U+58F2.0	fCangjie	GBHU
U+58F2.0	fFourCorner	40210
U+58F2.0	fCantonese	maai6
U+58F2.0	fMandarin	mai4
U+58F2.0	fEnglish	 sell NOT casing; shell; husk
U+58F3.0	fUTF8	壳
U+58F3.0	fCNS11643	3-282D
U+58F3.0	fGB	0-3F47
U+58F3.0	fR/S	 33.4; 33.5
U+58F3.0	fTotalStrokes	 7; 8
U+58F3.0	fCangjie	GBHN
U+58F3.0	fFourCorner	40210
U+58F3.0	fHakka	  hok7;  kok7
U+58F3.0	fCantonese	hok3
U+58F3.0	fMandarin	ke2
U+58F3.0	fEnglish	simplified form of &#x6BBB;
U+58F3.1	fUTF8	壳
U+58F3.1	fCNS11643	3-282D
U+58F3.1	fGB	0-3F47
U+58F3.1	fR/S	 33.4; 33.5
U+58F3.1	fTotalStrokes	 7; 8
U+58F3.1	fCangjie	GBHN
U+58F3.1	fFourCorner	40210
U+58F3.1	fMandarin	qiao4
U+58F4.0	fUTF8	壴
U+58F4.0	fCNS11643	2-2960
U+58F4.0	fBig5	CE5F
U+58F4.0	fGB	3-3521
U+58F4.0	fR/S	 33.6
U+58F4.0	fTotalStrokes	 9
U+58F4.0	fCangjie	GRT
U+58F4.0	fFourCorner	40108
U+58F4.0	fCantonese	zyu3; syu6
U+58F4.0	fMandarin	zhu3
U+58F5.0	fUTF8	壵
U+58F5.0	fCNS11643	3-2B77
U+58F5.0	fGB	E-263E
U+58F5.0	fR/S	 33.6
U+58F5.0	fTotalStrokes	 9
U+58F5.0	fCangjie	GGG
U+58F5.0	fFourCorner	40110
U+58F5.0	fHakka	  zong5
U+58F5.0	fCantonese	zong3; zong1; zong6
U+58F5.0	fMandarin	zhuang4; zhuan4; zhuang1
U+58F5.0	fEnglish	same as &#x58EF;
U+58F6.0	fUTF8	壶
U+58F6.0	fGB	0-3A78
U+58F6.0	fR/S	 33.7
U+58F6.0	fTotalStrokes	 10
U+58F6.0	fCangjie	GBTC
U+58F6.0	fHakka	 fu2
U+58F6.0	fCantonese	wu4; wu2 (also)
U+58F6.0	fMandarin	hu2
U+58F6.0	fEnglish	simplified form of &#x58FA;
U+58F7.0	fUTF8	壷
U+58F7.0	fCNS11643	3-3477
U+58F7.0	fGB	E-263F
U+58F7.0	fR/S	  33.8
U+58F7.0	fTotalStrokes	 11
U+58F7.0	fCangjie	GBLM
U+58F7.0	fFourCorner	40100
U+58F7.0	fHakka	 fu2
U+58F7.0	fCantonese	wu4
U+58F7.0	fMandarin	hu2
U+58F7.0	fEnglish	same as &#x58FA;
U+58F8.0	fUTF8	壸
U+58F8.0	fGB	8-7A5A
U+58F8.0	fR/S	 33.8; 33.10
U+58F8.0	fTotalStrokes	 11
U+58F8.0	fCangjie	XGBMC
U+58F8.0	fCantonese	kwan2
U+58F8.0	fMandarin	kun3
U+58F8.0	fEnglish	simplified form of &#x58F9;
U+58F9.0	fUTF8	壹
U+58F9.0	fCNS11643	1-5E62
U+58F9.0	fBig5	B3FC
U+58F9.0	fGB	0-523C
U+58F9.0	fR/S	 33.9
U+58F9.0	fTotalStrokes	 12
U+58F9.0	fCangjie	GBMT
U+58F9.0	fFourCorner	40108
U+58F9.0	fHakka	 jit7; rit7
U+58F9.0	fCantonese	jat1; jik1
U+58F9.0	fMandarin	yi1; yin1; yi4
U+58F9.0	fEnglish	one, used in accounting and banking
U+58FA.0	fUTF8	壺
U+58FA.0	fCNS11643	1-5E63
U+58FA.0	fBig5	B3FD
U+58FA.0	fGB	1-3A78
U+58FA.0	fR/S	 33.9
U+58FA.0	fTotalStrokes	 12
U+58FA.0	fCangjie	GBLM
U+58FA.0	fFourCorner	40107
U+58FA.0	fHakka	  fu2; fu3; kun3
U+58FA.0	fCantonese	wu4; wu2 (also)
U+58FA.0	fMandarin	hu2
U+58FA.0	fEnglish	[1] a jar; pot; jug; wine vessel [2] any potbellied container with a small opening [3] a Chinese family name
U+58FB.0	fUTF8	壻
U+58FB.0	fCNS11643	3-3A52
U+58FB.0	fGB	E-2640
U+58FB.0	fR/S	 33.9; 32.9
U+58FB.0	fTotalStrokes	 12
U+58FB.0	fCangjie	GNOB
U+58FB.0	fFourCorner	47120
U+58FB.0	fHakka	  se5
U+58FB.0	fCantonese	sai3
U+58FB.0	fMandarin	xu4
U+58FB.0	fEnglish	[1] one's son-in-law [2] one's husband
U+58FC.0	fUTF8	壼
U+58FC.0	fCNS11643	2-4239
U+58FC.0	fBig5	DCF2
U+58FC.0	fGB	3-3522
U+58FC.0	fR/S	 33.10
U+58FC.0	fTotalStrokes	 13
U+58FC.0	fCangjie	GBMM
U+58FC.0	fFourCorner	40107
U+58FC.0	fHakka	 kun3; kwun3
U+58FC.0	fCantonese	kwan2
U+58FC.0	fMandarin	kun3
U+58FC.0	fEnglish	a palace corridor; palace passageway; lane
U+58FD.0	fUTF8	壽
U+58FD.0	fCNS11643	1-6840
U+58FD.0	fBig5	B9D8
U+58FD.0	fGB	1-4A59
U+58FD.0	fR/S	 33.11
U+58FD.0	fTotalStrokes	 14
U+58FD.0	fCangjie	GNMI
U+58FD.0	fFourCorner	40641
U+58FD.0	fHakka	 su5; shiu6; siu3; su3; shiu5; shu5
U+58FD.0	fCantonese	sau6
U+58FD.0	fMandarin	shou4
U+58FD.0	fEnglish	[1] the life span [2] old age; a long life; longevity [3] birthday [4] die of old age [5] present another with gold, silk, etc. [6] toast an elder [7] a Chinese family name
U+58FE.0	fUTF8	壾
U+58FE.0	fCNS11643	2-4950
U+58FE.0	fBig5	E169
U+58FE.0	fGB	5-367E
U+58FE.0	fR/S	 33.11
U+58FE.0	fTotalStrokes	 14
U+58FE.0	fCangjie	GTSS
U+58FE.0	fFourCorner	41117
U+58FE.0	fCantonese	mong5
U+58FE.0	fMandarin	mang3
U+58FF.0	fUTF8	壿
U+58FF.0	fCNS11643	2-4F7A
U+58FF.0	fBig5	E553
U+58FF.0	fGB	E-2641
U+58FF.0	fR/S	 33.12
U+58FF.0	fTotalStrokes	 15
U+58FF.0	fCangjie	GTWI
U+58FF.0	fFourCorner	48146
U+58FF.0	fHakka	  zun1
U+58FF.0	fCantonese	cyun4; deon1; cun1
U+58FF.0	fMandarin	zun1; zun3; cun1
U+58FF.0	fEnglish	[1] goble; decanter [2] bottle
U+5900.0	fUTF8	夀
U+5900.0	fCNS11643	3-6466
U+5900.0	fGB	E-2642
U+5900.0	fR/S	 33.12
U+5900.0	fTotalStrokes	 15
U+5900.0	fCangjie	QMRDI
U+5900.0	fHakka	 shu5; shiu5
U+5900.0	fCantonese	sau6
U+5900.0	fMandarin	shou4
U+5900.0	fEnglish	life; length of life; longevity
U+5901.0	fUTF8	夁
U+5901.0	fCNS11643	3-504D
U+5901.0	fGB	E-2643
U+5901.0	fR/S	  33.13
U+5901.0	fTotalStrokes	 16
U+5901.0	fCangjie	GWLW
U+5901.0	fFourCorner	40600
U+5901.0	fHakka	 jit7; rit7
U+5901.0	fCantonese	jat1; jik1
U+5901.0	fMandarin	yi1; yin1; yi4
U+5901.0	fEnglish	ancient form of &#x58F9;
U+5902.0	fUTF8	夂
U+5902.0	fCNS11643	3-2139
U+5902.0	fGB	0-623A
U+5902.0	fR/S	 34.0
U+5902.0	fTotalStrokes	 3
U+5902.0	fCangjie	HE
U+5902.0	fFourCorner	27400
U+5902.0	fHakka	 zhi3; zhi5; zih5; sui1; tje5
U+5902.0	fCantonese	zi2; zi1; gau2
U+5902.0	fMandarin	zhi3
U+5902.0	fEnglish	 go Kangxi radical number 34
U+5903.0	fUTF8	夃
U+5903.0	fCNS11643	2-213A
U+5903.0	fBig5	C95A
U+5903.0	fGB	5-3535
U+5903.0	fR/S	 34.1
U+5903.0	fTotalStrokes	 4
U+5903.0	fCangjie	NSHE
U+5903.0	fFourCorner	17227
U+5903.0	fCantonese	gu2; gwu2 (also); jing4
U+5903.0	fMandarin	gu1; gu3
U+5904.0	fUTF8	处
U+5904.0	fCNS11643	3-2235
U+5904.0	fGB	0-3426
U+5904.0	fR/S	 34.2
U+5904.0	fTotalStrokes	 5
U+5904.0	fCangjie	HEY
U+5904.0	fFourCorner	28400
U+5904.0	fHakka	 chu3; chu5
U+5904.0	fCantonese	cyu3; cyu5; cyu2 (also); syu2 (also); syu3
U+5904.0	fMandarin	chu4; ju4
U+5904.0	fEnglish	simplified form of &#x8655;
U+5904.1	fUTF8	处
U+5904.1	fCNS11643	3-2235
U+5904.1	fGB	0-3426
U+5904.1	fR/S	 34.2
U+5904.1	fTotalStrokes	 5
U+5904.1	fCangjie	HEY
U+5904.1	fFourCorner	28400
U+5904.1	fMandarin	chu3
U+5904.2	fUTF8	处
U+5904.2	fCNS11643	3-2235
U+5904.2	fGB	0-3426
U+5904.2	fR/S	 34.2
U+5904.2	fTotalStrokes	 5
U+5904.2	fCangjie	HEY
U+5904.2	fFourCorner	28400
U+5904.2	fMandarin	chu5
U+5905.0	fUTF8	夅
U+5905.0	fCNS11643	3-2344
U+5905.0	fGB	5-3C6B
U+5905.0	fR/S	  34.3
U+5905.0	fTotalStrokes	 6
U+5905.0	fCangjie	HEQ
U+5905.0	fFourCorner	27500
U+5905.0	fCantonese	gong3
U+5905.0	fMandarin	jiang4; xiang2
U+5905.0	fEnglish	 descend come down from heaven send down
U+5906.0	fUTF8	夆
U+5906.0	fCNS11643	2-2350
U+5906.0	fBig5	CAB0
U+5906.0	fGB	3-3A34
U+5906.0	fR/S	 34.4
U+5906.0	fTotalStrokes	 7
U+5906.0	fCangjie	HEQJ
U+5906.0	fFourCorner	27504
U+5906.0	fHakka	 fung2; fung1
U+5906.0	fCantonese	fung1; fung4
U+5906.0	fMandarin	feng2; pang2
U+5906.0	fEnglish	resist
U+5907.0	fUTF8	备
U+5907.0	fGB	0-3138
U+5907.0	fR/S	 34.5
U+5907.0	fTotalStrokes	 8
U+5907.0	fCangjie	HEW
U+5907.0	fHakka	 pi5
U+5907.0	fCantonese	bei6
U+5907.0	fMandarin	bei4
U+5907.0	fEnglish	simplified form of &#x5099;
U+5908.0	fUTF8	夈
U+5908.0	fCNS11643	3-2B7A
U+5908.0	fGB	E-2644
U+5908.0	fR/S	 34.6; 34.3; 119.3
U+5908.0	fTotalStrokes	 9
U+5908.0	fCangjie	HEFD
U+5908.0	fFourCorner	27900
U+5908.0	fHakka	  zai1; sai2
U+5908.0	fCantonese	zaai1; zai1
U+5908.0	fMandarin	zhai1; zi1
U+5908.0	fEnglish	same as &#x9F4B;
U+5909.0	fUTF8	変
U+5909.0	fCNS11643	3-2B79
U+5909.0	fGB	E-2645
U+5909.0	fR/S	 35.6; 34.6
U+5909.0	fTotalStrokes	 9
U+5909.0	fCangjie	YCHE
U+5909.0	fFourCorner	00400
U+5909.0	fHakka	 bien5; pien5
U+5909.0	fCantonese	bin3
U+5909.0	fMandarin	bian4
U+5909.0	fEnglish	 change; transform; alter rebel
U+590A.0	fUTF8	夊
U+590A.0	fCNS11643	3-213A
U+590A.0	fGB	8-287B
U+590A.0	fR/S	 35.0
U+590A.0	fTotalStrokes	 3
U+590A.0	fCangjie	HE
U+590A.0	fFourCorner	27400
U+590A.0	fHakka	 soi1;  sui1
U+590A.0	fCantonese	seoi1
U+590A.0	fMandarin	sui1
U+590A.0	fEnglish	 Kangxi radical number 35 go slowly
U+590B.0	fUTF8	夋
U+590B.0	fCNS11643	3-2546
U+590B.0	fGB	3-3351
U+590B.0	fR/S	 35.4; 34.4
U+590B.0	fTotalStrokes	 7
U+590B.0	fCangjie	ICHE
U+590B.0	fFourCorner	23400
U+590B.0	fHakka	  zun1
U+590B.0	fCantonese	ceon1
U+590B.0	fMandarin	qun1
U+590B.0	fEnglish	 dawdle; walk slowly
U+590C.0	fUTF8	夌
U+590C.0	fCNS11643	2-2623
U+590C.0	fBig5	CC42
U+590C.0	fGB	3-3A35
U+590C.0	fR/S	 35.5
U+590C.0	fTotalStrokes	 8
U+590C.0	fCangjie	GCHE
U+590C.0	fFourCorner	40407
U+590C.0	fCantonese	ling4
U+590C.0	fMandarin	ling2
U+590C.0	fEnglish	 dawdle the name of the father of the Emperor Yao
U+590D.0	fUTF8	复
U+590D.0	fCNS11643	2-2961
U+590D.0	fBig5	CE60
U+590D.0	fGB	0-3834
U+590D.0	fR/S	 35.6
U+590D.0	fTotalStrokes	 9
U+590D.0	fCangjie	OAHE
U+590D.0	fFourCorner	80407
U+590D.0	fHakka	 fuk7; fuk8
U+590D.0	fCantonese	fuk6; fuk1; fau6 (also)
U+590D.0	fMandarin	fu4
U+590D.0	fEnglish	simplified form of &#x5FA9;
U+590E.0	fUTF8	夎
U+590E.0	fCNS11643	2-2E5B
U+590E.0	fBig5	D159
U+590E.0	fGB	3-3A37
U+590E.0	fR/S	 35.7
U+590E.0	fTotalStrokes	 10
U+590E.0	fCangjie	OOGE
U+590E.0	fFourCorner	88407
U+590E.0	fHakka	  co5
U+590E.0	fCantonese	zo3; co3; zeoi3
U+590E.0	fMandarin	zuo4
U+590E.0	fEnglish	 stumble
U+590F.0	fUTF8	夏
U+590F.0	fCNS11643	1-546E
U+590F.0	fBig5	AE4C
U+590F.0	fGB	0-4F44
U+590F.0	fR/S	 35.7
U+590F.0	fTotalStrokes	 10
U+590F.0	fCangjie	MUHE
U+590F.0	fFourCorner	10247
U+590F.0	fHakka	 ha5; ha6; ha3
U+590F.0	fCantonese	haa6; gaa2; kaa2
U+590F.0	fMandarin	xia4; xia3
U+590F.0	fEnglish	[1] summer [2] great; grand; big; spacious [3] a big house; mansion [4] Cathay, the ancient name of China [5] a dynasty in Chinese history (2205-1782 B.C.)
U+590F.1	fUTF8	夏
U+590F.1	fCNS11643	1-546E
U+590F.1	fBig5	AE4C
U+590F.1	fGB	0-4F44
U+590F.1	fR/S	 35.7
U+590F.1	fTotalStrokes	 10
U+590F.1	fCangjie	MUHE
U+590F.1	fFourCorner	10247
U+590F.1	fMandarin	jia3
U+590F.1	fEnglish	used in combination
U+5910.0	fUTF8	夐
U+5910.0	fCNS11643	3-4646
U+5910.0	fGB	3-326F
U+5910.0	fR/S	 35.11; 34.11
U+5910.0	fTotalStrokes	 14
U+5910.0	fCangjie	NBBUE
U+5910.0	fFourCorner	27400
U+5910.0	fHakka	 kwen3; gwen3
U+5910.0	fCantonese	hyun3; hing3
U+5910.0	fMandarin	xiong4
U+5910.0	fEnglish	[1] to seek [2] superior; preeminent;
U+5911.0	fUTF8	夑
U+5911.0	fCNS11643	3-6560
U+5911.0	fGB	E-2646
U+5911.0	fR/S	 35.15; 86.15
U+5911.0	fTotalStrokes	 18
U+5911.0	fCangjie	FFHE
U+5911.0	fFourCorner	9940
U+5911.0	fCantonese	sit3
U+5912.0	fUTF8	夒
U+5912.0	fCNS11643	2-6545
U+5912.0	fBig5	F1F9
U+5912.0	fGB	3-3A38
U+5912.0	fR/S	 35.16
U+5912.0	fTotalStrokes	 19
U+5912.0	fCangjie	MCHE
U+5912.0	fFourCorner	10407
U+5912.0	fCantonese	naau4; nou4 (also); laau4 (also); lou4 (also)
U+5912.0	fMandarin	nao2
U+5913.0	fUTF8	夓
U+5913.0	fCNS11643	4-5F52
U+5913.0	fGB	E-2647
U+5913.0	fR/S	 35.15
U+5913.0	fTotalStrokes	 19; 18
U+5913.0	fCangjie	HCHE
U+5913.0	fFourCorner	77400
U+5913.0	fCantonese	haa6
U+5913.0	fMandarin	xia4
U+5914.0	fUTF8	夔
U+5914.0	fCNS11643	1-7A67
U+5914.0	fBig5	C4DC
U+5914.0	fGB	0-5967
U+5914.0	fR/S	 35.19
U+5914.0	fTotalStrokes	 22
U+5914.0	fCangjie	TCHE
U+5914.0	fFourCorner	44407
U+5914.0	fHakka	 kui2
U+5914.0	fCantonese	kwai4
U+5914.0	fMandarin	kui2
U+5914.0	fEnglish	[1] one-legged monster (mythology) [2] name of a court musician in the reign of Emperor Shun (2255 B.C.) [3] name of a feudal state in the Chou dynasty [4] a Chinese family name
U+5915.0	fUTF8	夕
U+5915.0	fCNS11643	1-444A
U+5915.0	fBig5	A469
U+5915.0	fGB	0-4F26
U+5915.0	fR/S	 36.0
U+5915.0	fTotalStrokes	 3
U+5915.0	fCangjie	NI
U+5915.0	fFourCorner	27200
U+5915.0	fHakka	  sip8; sit8
U+5915.0	fCantonese	zik6; zik2 (also)
U+5915.0	fMandarin	xi1; yi4; xi4
U+5915.0	fEnglish	[1] evening; dusk; sunset [2] night [3] slanted; oblique [4] meet in the evening [5] a Chinese family name
U+5916.0	fUTF8	外
U+5916.0	fCNS11643	1-4640
U+5916.0	fBig5	A57E
U+5916.0	fGB	0-4D62
U+5916.0	fR/S	 36.2
U+5916.0	fTotalStrokes	 5
U+5916.0	fCangjie	NIY
U+5916.0	fFourCorner	23200
U+5916.0	fHakka	 ngoi5; ngoi3; ngoi6; ngo6; ngo5; no5; no6; mo6; mo5; ngoe3
U+5916.0	fCantonese	ngoi6; oi6 (also); ngoi2 (also); oi2 (also)
U+5916.0	fMandarin	wai4
U+5916.0	fEnglish	[1] out; outside; external [2] foreign; alien [3] diplomatic; diplomacy [4] besides; in addition [5] a term referring to one's husband [6] a role n Chinese opera for old men [7] alienate [8] on wife's or mother's side [9] relatives of one's sisters or daughters
U+5917.0	fUTF8	夗
U+5917.0	fCNS11643	2-2151
U+5917.0	fBig5	C970
U+5917.0	fGB	3-3A30
U+5917.0	fR/S	 36.2
U+5917.0	fTotalStrokes	 5
U+5917.0	fCangjie	NISU
U+5917.0	fFourCorner	27212
U+5917.0	fHakka	 wan3; jan1; ren5; jen5; jen3
U+5917.0	fCantonese	jyun2; jyun3
U+5917.0	fMandarin	yuan4; wan3; yung4
U+5917.0	fEnglish	 turn over when asleep
U+5918.0	fUTF8	夘
U+5918.0	fCNS11643	3-2236
U+5918.0	fGB	E-2648
U+5918.0	fR/S	 36.2
U+5918.0	fTotalStrokes	 5
U+5918.0	fCangjie	NISL
U+5918.0	fFourCorner	27220
U+5918.0	fHakka	 ngoe5; jen3
U+5918.0	fCantonese	maau5; jyun2
U+5918.0	fMandarin	mao3; yuan4; wan3; yung4
U+5918.0	fEnglish	same as &#x5917;
U+5919.0	fUTF8	夙
U+5919.0	fCNS11643	1-4768
U+5919.0	fBig5	A667
U+5919.0	fGB	0-596D
U+5919.0	fR/S	 36.3
U+5919.0	fTotalStrokes	 6
U+5919.0	fCangjie	HNMNI
U+5919.0	fFourCorner	77210
U+5919.0	fHakka	  siuk7; suk7
U+5919.0	fCantonese	suk1
U+5919.0	fMandarin	su4
U+5919.0	fEnglish	[1] the early morning [2] old; original (desires, etc.) [3] inborn; inherited (Buddhism)
U+591A.0	fUTF8	多
U+591A.0	fCNS11643	1-4769
U+591A.0	fBig5	A668
U+591A.0	fGB	0-3660
U+591A.0	fR/S	 36.3
U+591A.0	fTotalStrokes	 6
U+591A.0	fCangjie	NINI
U+591A.0	fFourCorner	27207
U+591A.0	fHakka	  do1
U+591A.0	fCantonese	do1
U+591A.0	fMandarin	duo1
U+591A.0	fEnglish	[1] much; many; too much [2] more than; over; much more [3] much; greatly; highly; very [4] to praise [5] only [6] a Chinese family name
U+591A.1	fUTF8	多
U+591A.1	fCNS11643	1-4769
U+591A.1	fBig5	A668
U+591A.1	fGB	0-3660
U+591A.1	fR/S	 36.3
U+591A.1	fTotalStrokes	 6
U+591A.1	fCangjie	NINI
U+591A.1	fFourCorner	27207
U+591A.1	fMandarin	duo2
U+591A.1	fEnglish	how, what, etc. (in exclamatory statements)
U+591B.0	fUTF8	夛
U+591B.0	fCNS11643	4-2252
U+591B.0	fGB	E-2649
U+591B.0	fR/S	 36.3
U+591B.0	fTotalStrokes	 6
U+591B.0	fCangjie	NMMN
U+591B.0	fFourCorner	17200
U+591B.0	fHakka	 do1
U+591B.0	fCantonese	do1
U+591B.0	fMandarin	duo1
U+591B.0	fEnglish	much, many; more than, over
U+591C.0	fUTF8	夜
U+591C.0	fCNS11643	1-4C5F
U+591C.0	fBig5	A95D
U+591C.0	fGB	0-5239
U+591C.0	fR/S	 36.5
U+591C.0	fTotalStrokes	 8
U+591C.0	fCangjie	YONK
U+591C.0	fFourCorner	00247
U+591C.0	fHakka	 ja3;  ja5; ra6; ja6
U+591C.0	fCantonese	je6; je2 (also)
U+591C.0	fMandarin	ye4; yi4
U+591C.0	fEnglish	[1] night; dark; darkness; in night [2] night trip; night travelling; by night
U+591D.0	fUTF8	夝
U+591D.0	fCNS11643	4-2570
U+591D.0	fGB	5-3C61
U+591D.0	fR/S	 36.5
U+591D.0	fTotalStrokes	 8
U+591D.0	fCangjie	NIHQM
U+591D.0	fFourCorner	25210
U+591D.0	fCantonese	cing4; ceng4 (also)
U+591D.0	fMandarin	qing2
U+591E.0	fUTF8	夞
U+591E.0	fGB	1-7D7E
U+591E.0	fR/S	 36.7; 21.8
U+591E.0	fTotalStrokes	 10
U+591E.0	fCangjie	NYRP
U+591E.0	fCantonese	ngoi6; oi6 (also)
U+591F.0	fUTF8	够
U+591F.0	fCNS11643	3-3479
U+591F.0	fGB	0-393B
U+591F.0	fR/S	  36.8
U+591F.0	fTotalStrokes	 11
U+591F.0	fCangjie	PRNIN
U+591F.0	fFourCorner	27620
U+591F.0	fHakka	  geu5;  kieu5; giu5; la6; la5
U+591F.0	fCantonese	gau3
U+591F.0	fMandarin	gou4
U+591F.0	fEnglish	[1] enough; more than enough; adequate; too much; sufficient [2] fully; quite
U+5920.0	fUTF8	夠
U+5920.0	fCNS11643	1-595C
U+5920.0	fBig5	B0F7
U+5920.0	fGB	E-264A
U+5920.0	fR/S	 36.8
U+5920.0	fTotalStrokes	 11
U+5920.0	fCangjie	NNPR
U+5920.0	fFourCorner	27220
U+5920.0	fHakka	 kieu5; giu5; geu5
U+5920.0	fCantonese	gau3
U+5920.0	fMandarin	gou4; kou1
U+5920.0	fEnglish	[1] enough; more than enough; adequate; too much; sufficient [2] fully; quite
U+5921.0	fUTF8	夡
U+5921.0	fCNS11643	5-373D
U+5921.0	fGB	5-3C64
U+5921.0	fR/S	 36.9
U+5921.0	fTotalStrokes	 12
U+5921.0	fCangjie	NNGR
U+5921.0	fFourCorner	24260
U+5921.0	fCantonese	hei3
U+5921.0	fMandarin	qi4
U+5922.0	fUTF8	夢
U+5922.0	fCNS11643	1-6842
U+5922.0	fBig5	B9DA
U+5922.0	fGB	1-434E
U+5922.0	fR/S	 36.11
U+5922.0	fTotalStrokes	 14
U+5922.0	fCangjie	TWLN
U+5922.0	fFourCorner	44207
U+5922.0	fHakka	  mung5; mung6; mung3; mu5
U+5922.0	fCantonese	mung6; mung4
U+5922.0	fMandarin	meng4; meng2; jie2
U+5922.0	fEnglish	[1] dream; visionary [2] dream; see visions [3] wishful thinking; wishful
U+5923.0	fUTF8	夣
U+5923.0	fCNS11643	3-405F
U+5923.0	fGB	E-264B
U+5923.0	fR/S	 36.11
U+5923.0	fTotalStrokes	 14; 13
U+5923.0	fCangjie	YIWN
U+5923.0	fFourCorner	00200
U+5923.0	fHakka	 mung5; mung6; mu5
U+5923.0	fCantonese	mung6
U+5923.0	fMandarin	meng4; jie2; meng2
U+5923.0	fEnglish	a dream; to dream visionary; stupid
U+5924.0	fUTF8	夤
U+5924.0	fCNS11643	1-6843
U+5924.0	fBig5	B9DB
U+5924.0	fGB	0-6239
U+5924.0	fR/S	 36.11
U+5924.0	fTotalStrokes	 14
U+5924.0	fCangjie	NIJMC
U+5924.0	fFourCorner	27806
U+5924.0	fHakka	  jin2; rin2
U+5924.0	fCantonese	jan4
U+5924.0	fMandarin	yin2; ya1; yi2
U+5924.0	fEnglish	[1] distant place; remote place [2] hang on (power, glory, etc.) [3] respect
U+5925.0	fUTF8	夥
U+5925.0	fCNS11643	1-6841
U+5925.0	fBig5	B9D9
U+5925.0	fGB	0-6237
U+5925.0	fR/S	 36.11
U+5925.0	fTotalStrokes	 14
U+5925.0	fCangjie	WDNIN
U+5925.0	fFourCorner	67927
U+5925.0	fHakka	  fo3; fo2
U+5925.0	fCantonese	fo2
U+5925.0	fMandarin	huo3; huai4
U+5925.0	fEnglish	[1] many; much; plenty; lots of [2] companion; partner; assistant [3] waiter; clerk [4] crowd
U+5926.0	fUTF8	夦
U+5926.0	fCNS11643	3-4B53
U+5926.0	fGB	E-264C
U+5926.0	fR/S	  36.12
U+5926.0	fTotalStrokes	 15
U+5926.0	fCangjie	TVNIN
U+5926.0	fFourCorner	47720
U+5926.0	fCantonese	cam2
U+5926.0	fMandarin	chen4
U+5927.0	fUTF8	大
U+5927.0	fCNS11643	1-444B
U+5927.0	fBig5	A46A
U+5927.0	fGB	0-3473
U+5927.0	fR/S	 37.0
U+5927.0	fTotalStrokes	 3
U+5927.0	fCangjie	K
U+5927.0	fFourCorner	40030
U+5927.0	fHakka	  tai5; tai3; tai6
U+5927.0	fCantonese	daai6; taai3; daai1 (also); daai2 (also); tai3
U+5927.0	fMandarin	da4; tai4; duo4; tai3
U+5927.0	fEnglish	[1] big; vast; large [2] great [3] much [4] very; highly; extremely; really [5] your (politely) [6] the eldest; senior [7] full-grown; adult [8] before; after (referring to dates only) [9] make large; make great [10] a Chinese family name
U+5927.1	fUTF8	大
U+5927.1	fCNS11643	1-444B
U+5927.1	fBig5	A46A
U+5927.1	fGB	0-3473
U+5927.1	fR/S	 37.0
U+5927.1	fTotalStrokes	 3
U+5927.1	fCangjie	K
U+5927.1	fFourCorner	40030
U+5927.1	fMandarin	dai4
U+5927.1	fEnglish	used in combination
U+5928.0	fUTF8	夨
U+5928.0	fCNS11643	4-2149
U+5928.0	fGB	5-372B
U+5928.0	fR/S	 37.0; 37.1
U+5928.0	fTotalStrokes	 4; 3
U+5928.0	fCangjie	VK
U+5928.0	fFourCorner	20430
U+5928.0	fCantonese	zak1
U+5928.0	fMandarin	ze4; che4
U+5929.0	fUTF8	天
U+5929.0	fCNS11643	1-4532
U+5929.0	fBig5	A4D1
U+5929.0	fGB	0-4C6C
U+5929.0	fR/S	 37.1
U+5929.0	fTotalStrokes	 4
U+5929.0	fCangjie	MK
U+5929.0	fFourCorner	10430
U+5929.0	fHakka	 tien1; ten1; t'ian1
U+5929.0	fCantonese	tin1
U+5929.0	fMandarin	tian1
U+5929.0	fEnglish	[1] sky; heavens; heaven; vault of heavens; firmament [2] nature; god; heaven; celestial [3] nature; natural; not artificial [4] a day [5] seasons; climates; weather [6] father; husband [7] something indispensable; necessities
U+5929.1	fUTF8	天
U+5929.1	fCNS11643	1-4532
U+5929.1	fBig5	A4D1
U+5929.1	fGB	0-4C6C
U+5929.1	fR/S	 37.1
U+5929.1	fTotalStrokes	 4
U+5929.1	fCangjie	MK
U+5929.1	fFourCorner	10430
U+5929.1	fMandarin	tian5
U+592A.0	fUTF8	太
U+592A.0	fCNS11643	1-4534
U+592A.0	fBig5	A4D3
U+592A.0	fGB	0-4C2B
U+592A.0	fR/S	 37.1
U+592A.0	fTotalStrokes	 4
U+592A.0	fCangjie	KI
U+592A.0	fFourCorner	40030
U+592A.0	fHakka	  tai5; tai
U+592A.0	fCantonese	taai3; taai2 (also)
U+592A.0	fMandarin	tai4; tai3
U+592A.0	fEnglish	[1] very large; very big [2] too; much; extremely; over; excessively [3] a term of respect (used in titles) [4] a Chinese family name
U+592A.1	fUTF8	太
U+592A.1	fCNS11643	1-4534
U+592A.1	fBig5	A4D3
U+592A.1	fGB	0-4C2B
U+592A.1	fR/S	 37.1
U+592A.1	fTotalStrokes	 4
U+592A.1	fCangjie	KI
U+592A.1	fFourCorner	40030
U+592A.1	fMandarin	tai5
U+592B.0	fUTF8	夫
U+592B.0	fCNS11643	1-4533
U+592B.0	fBig5	A4D2
U+592B.0	fGB	0-3772
U+592B.0	fR/S	 37.1
U+592B.0	fTotalStrokes	 4
U+592B.0	fCangjie	QO
U+592B.0	fFourCorner	50030
U+592B.0	fHakka	  fu1; bu1; fu3; fu2
U+592B.0	fCantonese	fu1; fu4
U+592B.0	fMandarin	fu1; fu4
U+592B.0	fEnglish	[1] man; male adult [2] those eligible for military service [3] husband [4] master
U+592B.1	fUTF8	夫
U+592B.1	fCNS11643	1-4533
U+592B.1	fBig5	A4D2
U+592B.1	fGB	0-3772
U+592B.1	fR/S	 37.1
U+592B.1	fTotalStrokes	 4
U+592B.1	fCangjie	QO
U+592B.1	fFourCorner	50030
U+592B.1	fMandarin	fu2
U+592B.1	fEnglish	[1] a demonstrative pronoun; that [2] a final particle; particle
U+592B.2	fUTF8	夫
U+592B.2	fCNS11643	1-4533
U+592B.2	fBig5	A4D2
U+592B.2	fGB	0-3772
U+592B.2	fR/S	 37.1
U+592B.2	fTotalStrokes	 4
U+592B.2	fCangjie	QO
U+592B.2	fFourCorner	50030
U+592C.0	fUTF8	夬
U+592C.0	fCNS11643	2-213B
U+592C.0	fBig5	C95B
U+592C.0	fGB	3-302F
U+592C.0	fR/S	 37.1
U+592C.0	fTotalStrokes	 4
U+592C.0	fCangjie	DK
U+592C.0	fFourCorner	50037
U+592C.0	fHakka	 kwai5; gwai5
U+592C.0	fCantonese	gwaai3; kyut3; gwaai2
U+592C.0	fMandarin	guai4; jue2
U+592C.0	fEnglish	one of the eight diagrams in the Book of Changes
U+592D.0	fUTF8	夭
U+592D.0	fCNS11643	1-4535
U+592D.0	fBig5	A4D4
U+592D.0	fGB	0-5832
U+592D.0	fR/S	 37.1
U+592D.0	fTotalStrokes	 4
U+592D.0	fCangjie	HK
U+592D.0	fFourCorner	20430
U+592D.0	fCantonese	jiu2; ou2
U+592D.0	fMandarin	yao1; wo4; wai1
U+592D.0	fEnglish	[1] die young [2] suppress; repress
U+592D.1	fUTF8	夭
U+592D.1	fCNS11643	1-4535
U+592D.1	fBig5	A4D4
U+592D.1	fGB	0-5832
U+592D.1	fR/S	 37.1
U+592D.1	fTotalStrokes	 4
U+592D.1	fCangjie	HK
U+592D.1	fFourCorner	20430
U+592D.1	fHakka	 jau1; rau1; jeu1
U+592D.1	fCantonese	jiu1
U+592D.1	fMandarin	yao3
U+592D.1	fEnglish	young; freshlooking; tender
U+592E.0	fUTF8	央
U+592E.0	fCNS11643	1-4641
U+592E.0	fBig5	A5A1
U+592E.0	fGB	0-516B
U+592E.0	fR/S	 37.2
U+592E.0	fTotalStrokes	 5
U+592E.0	fCangjie	LBK
U+592E.0	fFourCorner	50030
U+592E.0	fHakka	  jong1; ong1; rong1
U+592E.0	fCantonese	joeng1; jing1
U+592E.0	fMandarin	yang1; ying1
U+592E.0	fEnglish	[1] center; central; middle [2] finish; conclusion [3] request; entreat; beg
U+592F.0	fUTF8	夯
U+592F.0	fCNS11643	2-2152
U+592F.0	fBig5	C971
U+592F.0	fGB	0-3A3B
U+592F.0	fR/S	 37.2
U+592F.0	fTotalStrokes	 5
U+592F.0	fCangjie	KKS
U+592F.0	fFourCorner	40427
U+592F.0	fHakka	  kong3; hong2; hong1
U+592F.0	fCantonese	haang1; ban6; hang1; pan6
U+592F.0	fMandarin	hang1; ben4
U+592F.0	fEnglish	[1] raise with force [2] heavy load; burden [3] fill cracks and leaks with earth (levee construction)
U+5930.0	fUTF8	夰
U+5930.0	fCNS11643	3-2238
U+5930.0	fGB	5-372D
U+5930.0	fR/S	  37.2
U+5930.0	fTotalStrokes	 5
U+5930.0	fCangjie	KLL
U+5930.0	fFourCorner	40220
U+5930.0	fHakka	  gau3; hau3; hau5
U+5930.0	fCantonese	gou2
U+5930.0	fMandarin	gao3
U+5930.0	fEnglish	[1] let go [2] divide
U+5931.0	fUTF8	失
U+5931.0	fCNS11643	1-4642
U+5931.0	fBig5	A5A2
U+5931.0	fGB	0-4A27
U+5931.0	fR/S	 37.2
U+5931.0	fTotalStrokes	 5
U+5931.0	fCangjie	HQO
U+5931.0	fFourCorner	25030
U+5931.0	fHakka	  shit7;  sit7; siit7
U+5931.0	fCantonese	sat1; jat6
U+5931.0	fMandarin	shi1; yi4
U+5931.0	fEnglish	[1] lose [2] make a mistake; mistake; omission [3] let slip; neglect; miss
U+5932.0	fUTF8	夲
U+5932.0	fCNS11643	3-2237
U+5932.0	fGB	5-372C
U+5932.0	fR/S	 37.2
U+5932.0	fTotalStrokes	 5
U+5932.0	fCangjie	KJ
U+5932.0	fFourCorner	40230
U+5932.0	fHakka	  tau1; to1
U+5932.0	fCantonese	tou1
U+5932.0	fMandarin	ben3; tao1
U+5932.0	fEnglish	 advance quickly go back and forth origin; source
U+5933.0	fUTF8	夳
U+5933.0	fCNS11643	4-2172
U+5933.0	fGB	E-264D
U+5933.0	fR/S	 37.2
U+5933.0	fTotalStrokes	 5
U+5933.0	fCangjie	KMM
U+5933.0	fFourCorner	40100
U+5933.0	fHakka	  tai5
U+5933.0	fCantonese	taai3
U+5933.0	fMandarin	tai4
U+5933.0	fEnglish	ancient form of &#x6CF0;
U+5934.0	fUTF8	头
U+5934.0	fCNS11643	3-2239
U+5934.0	fGB	0-4D37
U+5934.0	fR/S	 37.2
U+5934.0	fTotalStrokes	 5
U+5934.0	fCangjie	YK
U+5934.0	fFourCorner	34030
U+5934.0	fHakka	 teu2
U+5934.0	fCantonese	tau4; tau2 (also)
U+5934.0	fMandarin	tou2
U+5934.0	fEnglish	simplified form of &#x982D;
U+5934.1	fUTF8	头
U+5934.1	fCNS11643	3-2239
U+5934.1	fGB	0-4D37
U+5934.1	fR/S	 37.2
U+5934.1	fTotalStrokes	 5
U+5934.1	fCangjie	YK
U+5934.1	fFourCorner	34030
U+5934.1	fMandarin	tou5
U+5935.0	fUTF8	夵
U+5935.0	fCNS11643	4-2253
U+5935.0	fGB	5-372F
U+5935.0	fR/S	  37.3
U+5935.0	fTotalStrokes	 6
U+5935.0	fCangjie	KF
U+5935.0	fFourCorner	40900
U+5935.0	fCantonese	jim5
U+5935.0	fMandarin	yan3
U+5936.0	fUTF8	夶
U+5936.0	fCNS11643	3-2347
U+5936.0	fGB	E-264E
U+5936.0	fR/S	 37.3
U+5936.0	fTotalStrokes	 6
U+5936.0	fCangjie	KK
U+5936.0	fFourCorner	44030
U+5936.0	fHakka	  bi3
U+5936.0	fCantonese	bei2; bei6; pei4; bei3
U+5936.0	fMandarin	bi3; pi2; pi3 bi4
U+5936.0	fEnglish	ancient form of &#x6BD4;
U+5937.0	fUTF8	夷
U+5937.0	fCNS11643	1-476A
U+5937.0	fBig5	A669
U+5937.0	fGB	0-5244
U+5937.0	fR/S	 37.3
U+5937.0	fTotalStrokes	 6
U+5937.0	fCangjie	KN
U+5937.0	fFourCorner	50032
U+5937.0	fHakka	  ji2; gni3; ri2; i2
U+5937.0	fCantonese	ji4
U+5937.0	fMandarin	yi2
U+5937.0	fEnglish	[1] ancient barbarian tribes in the east [2] foreign tribes; foreigners [3] at ease; peaceful [4] level; make level; even; smooth [5] safe [6] eliminate; exterminate; kill; execute [7] injuries; wounds [8] grades; classes [9] common; ordinary [10] great; big [11] a Chinese family name
U+5938.0	fUTF8	夸
U+5938.0	fCNS11643	1-476B
U+5938.0	fBig5	A66A
U+5938.0	fGB	0-3F64
U+5938.0	fR/S	 37.3
U+5938.0	fTotalStrokes	 6
U+5938.0	fCangjie	KMMS
U+5938.0	fFourCorner	40207
U+5938.0	fHakka	 kwa1; ka1; kia1; kwa5
U+5938.0	fCantonese	kwaa1
U+5938.0	fMandarin	kua1; qu4
U+5938.0	fEnglish	[1] simplified form of &#x8A87; [2] good-looking; handsome; pleasant [3] a Chinese family name
U+5939.0	fUTF8	夹
U+5939.0	fGB	0-3C50
U+5939.0	fR/S	 37.3; 1.5
U+5939.0	fTotalStrokes	 6
U+5939.0	fCangjie	KT
U+5939.0	fHakka	 hiap8; gap7; giap7
U+5939.0	fCantonese	gaap3; gaap2 (also); gep6 (also); gip6 (also); gip2 (also); gap3 (also); gaak3 (also); hip3
U+5939.0	fMandarin	jia2
U+5939.0	fEnglish	simplified form of &#x593E;
U+5939.1	fUTF8	夹
U+5939.1	fGB	0-3C50
U+5939.1	fR/S	 37.3; 1.5
U+5939.1	fTotalStrokes	 6
U+5939.1	fCangjie	KT
U+5939.1	fMandarin	jia1
U+593A.0	fUTF8	夺
U+593A.0	fGB	0-3661
U+593A.0	fR/S	 37.3
U+593A.0	fTotalStrokes	 6
U+593A.0	fCangjie	KDI
U+593A.0	fHakka	 tot8
U+593A.0	fCantonese	dyut6
U+593A.0	fMandarin	duo2; duo4
U+593A.0	fEnglish	simplified form of &#x596A;
U+593B.0	fUTF8	夻
U+593B.0	fGB	1-7D6D
U+593B.0	fR/S	 37.3; 30.3
U+593B.0	fTotalStrokes	 6
U+593B.0	fCangjie	XKR
U+593B.0	fFourCorner	4060
U+593B.0	fCantonese	ng4
U+593B.0	fEnglish	 fish name
U+593C.0	fUTF8	夼
U+593C.0	fCNS11643	2-222B
U+593C.0	fBig5	C9CB
U+593C.0	fGB	0-5E45
U+593C.0	fR/S	 37.3
U+593C.0	fTotalStrokes	 6
U+593C.0	fCangjie	KLLL
U+593C.0	fFourCorner	40222
U+593C.0	fCantonese	kwong3; kong3 (also); kwaang2
U+593C.0	fMandarin	kuang3
U+593D.0	fUTF8	夽
U+593D.0	fCNS11643	3-2549
U+593D.0	fGB	3-3527
U+593D.0	fR/S	  37.4
U+593D.0	fTotalStrokes	 7
U+593D.0	fCangjie	KMMI
U+593D.0	fFourCorner	40730
U+593D.0	fCantonese	wan5
U+593D.0	fMandarin	yun4
U+593E.0	fUTF8	夾
U+593E.0	fCNS11643	1-4968
U+593E.0	fBig5	A7A8
U+593E.0	fGB	1-3C50
U+593E.0	fR/S	 37.4
U+593E.0	fTotalStrokes	 7
U+593E.0	fCangjie	KOO
U+593E.0	fFourCorner	40038
U+593E.0	fHakka	  gap7; hiap8;  kiap8;  giap7; kiap78; gap8; giap8
U+593E.0	fCantonese	gaap3; gep6 (also); gip6 (also); gaap2 (also); gap3 (also); gip2 (also); gaak3 (also); hip3
U+593E.0	fMandarin	jia1; jia2
U+593E.0	fEnglish	[1] be wedged between; be sandwiched [2] squeeze; press; occupy both sides of [3] pincers [4] of one or more layers; lined (garments, etc.) [5] a folder to keep sheets of paper, etc. [6] carry secretly [7] mix; mingle
U+593E.1	fUTF8	夾
U+593E.1	fCNS11643	1-4968
U+593E.1	fBig5	A7A8
U+593E.1	fGB	1-3C50
U+593E.1	fR/S	 37.4
U+593E.1	fTotalStrokes	 7
U+593E.1	fCangjie	KOO
U+593E.1	fFourCorner	40038
U+593E.1	fEnglish	used in combination
U+593E.2	fUTF8	夾
U+593E.2	fCNS11643	1-4968
U+593E.2	fBig5	A7A8
U+593E.2	fGB	1-3C50
U+593E.2	fR/S	 37.4
U+593E.2	fTotalStrokes	 7
U+593E.2	fCangjie	KOO
U+593E.2	fFourCorner	40038
U+593E.2	fMandarin	jia4
U+593F.0	fUTF8	夿
U+593F.0	fCNS11643	4-2362
U+593F.0	fGB	3-352B
U+593F.0	fR/S	  37.4
U+593F.0	fTotalStrokes	 7
U+593F.0	fCangjie	KAU
U+593F.0	fFourCorner	40710
U+593F.0	fCantonese	baa1
U+593F.0	fMandarin	pa1; ba1
U+5940.0	fUTF8	奀
U+5940.0	fCNS11643	2-2351
U+5940.0	fBig5	CAB1
U+5940.0	fGB	3-3528
U+5940.0	fR/S	 37.4
U+5940.0	fTotalStrokes	 7
U+5940.0	fCangjie	MFK
U+5940.0	fFourCorner	10439
U+5940.0	fHakka	 ngan1; ze1
U+5940.0	fCantonese	ngan1; an1 (also)
U+5940.0	fMandarin	en1; mang2
U+5940.0	fEnglish	[1] skinny [2] tiny; puny; little (money) [3] jiggle the feet
U+5941.0	fUTF8	奁
U+5941.0	fGB	0-5E46
U+5941.0	fR/S	 37.4
U+5941.0	fTotalStrokes	 7
U+5941.0	fCangjie	KSK
U+5941.0	fHakka	 liam2
U+5941.0	fCantonese	lim4
U+5941.0	fMandarin	lian2
U+5941.0	fEnglish	simplified form of &#x5969;
U+5942.0	fUTF8	奂
U+5942.0	fGB	0-5B3C
U+5942.0	fR/S	 37.4
U+5942.0	fTotalStrokes	 7
U+5942.0	fCangjie	XNBK
U+5942.0	fHakka	 fon5
U+5942.0	fCantonese	wun6
U+5942.0	fMandarin	huan4
U+5942.0	fEnglish	simplified form of &#x5950;
U+5943.0	fUTF8	奃
U+5943.0	fCNS11643	4-2575
U+5943.0	fGB	3-352D
U+5943.0	fR/S	  37.5
U+5943.0	fTotalStrokes	 8
U+5943.0	fCangjie	KHPM
U+5943.0	fFourCorner	40710
U+5943.0	fCantonese	dai1
U+5943.0	fMandarin	di4; di1; ti4
U+5944.0	fUTF8	奄
U+5944.0	fCNS11643	1-4C63
U+5944.0	fBig5	A961
U+5944.0	fGB	0-5159
U+5944.0	fR/S	 37.5
U+5944.0	fTotalStrokes	 8
U+5944.0	fCangjie	KLWU
U+5944.0	fFourCorner	40716
U+5944.0	fHakka	  jam1; jam3; ram1
U+5944.0	fCantonese	jim2; jim1
U+5944.0	fMandarin	yan3; yan4
U+5944.0	fEnglish	[1] cover; surround [2] suddenly; abruptly; rapidly [3] a Chinese family name
U+5944.1	fUTF8	奄
U+5944.1	fCNS11643	1-4C63
U+5944.1	fBig5	A961
U+5944.1	fGB	0-5159
U+5944.1	fR/S	 37.5
U+5944.1	fTotalStrokes	 8
U+5944.1	fCangjie	KLWU
U+5944.1	fFourCorner	40716
U+5944.1	fMandarin	yan1
U+5944.1	fEnglish	[1] soak; bathe; drown [2] remain [3] castrate; castrated man
U+5945.0	fUTF8	奅
U+5945.0	fCNS11643	2-2624
U+5945.0	fBig5	CC43
U+5945.0	fGB	3-352E
U+5945.0	fR/S	 37.5
U+5945.0	fTotalStrokes	 8
U+5945.0	fCangjie	KHHL
U+5945.0	fFourCorner	40727
U+5945.0	fHakka	  pau5
U+5945.0	fCantonese	paau3
U+5945.0	fMandarin	pao4
U+5945.0	fEnglish	[1] brag [2] swell
U+5946.0	fUTF8	奆
U+5946.0	fCNS11643	4-2572
U+5946.0	fGB	3-3529
U+5946.0	fR/S	  37.5
U+5946.0	fTotalStrokes	 8
U+5946.0	fCangjie	KSS
U+5946.0	fFourCorner	40710
U+5946.0	fCantonese	gyun6
U+5946.0	fMandarin	quan3; juan4
U+5947.0	fUTF8	奇
U+5947.0	fCNS11643	1-4C61
U+5947.0	fBig5	A95F
U+5947.0	fGB	0-4666
U+5947.0	fR/S	 37.5
U+5947.0	fTotalStrokes	 8
U+5947.0	fCangjie	KMNR
U+5947.0	fFourCorner	40621
U+5947.0	fHakka	  ki2; ki3
U+5947.0	fCantonese	kei4; gei1
U+5947.0	fMandarin	qi2; ai3; yi3
U+5947.0	fEnglish	[1] strange; unusual; uncanny; occult; rare [2] wonderful [3] feel strange about; wonder
U+5947.1	fUTF8	奇
U+5947.1	fCNS11643	1-4C61
U+5947.1	fBig5	A95F
U+5947.1	fGB	0-4666
U+5947.1	fR/S	 37.5
U+5947.1	fTotalStrokes	 8
U+5947.1	fCangjie	KMNR
U+5947.1	fFourCorner	40621
U+5947.1	fMandarin	ji1
U+5947.1	fEnglish	odd (numbers)
U+5948.0	fUTF8	奈
U+5948.0	fCNS11643	1-4C62
U+5948.0	fBig5	A960
U+5948.0	fGB	0-444E
U+5948.0	fR/S	 37.5
U+5948.0	fTotalStrokes	 8
U+5948.0	fCangjie	KMMF
U+5948.0	fFourCorner	40901
U+5948.0	fHakka	 lai5;  nai5; nai6; lai3
U+5948.0	fCantonese	noi6; loi6 (also)
U+5948.0	fMandarin	nai4; nai3; na4
U+5948.0	fEnglish	[1] but; how; what [2] bear; stand; endure
U+5949.0	fUTF8	奉
U+5949.0	fCNS11643	1-4C60
U+5949.0	fBig5	A95E
U+5949.0	fGB	0-376E
U+5949.0	fR/S	 37.5
U+5949.0	fTotalStrokes	 8
U+5949.0	fCangjie	QKQ
U+5949.0	fFourCorner	50503
U+5949.0	fHakka	 fung3; fung6;  fung5
U+5949.0	fCantonese	fung6; fung2 (also)
U+5949.0	fMandarin	feng4; feng2; feng3
U+5949.0	fEnglish	[1] offer; present [2] receive with respect [3] an expression of respect [4] serve; wait on [5] admire; love and respect [6] pay; salary [7] a Chinese family name
U+594A.0	fUTF8	奊
U+594A.0	fCNS11643	2-2E5C
U+594A.0	fBig5	D15A
U+594A.0	fGB	3-337B
U+594A.0	fR/S	 37.7
U+594A.0	fTotalStrokes	 10
U+594A.0	fCangjie	GGVK
U+594A.0	fFourCorner	24431
U+594A.0	fHakka	 giet7
U+594A.0	fCantonese	kwai4 (also); hai5; kit3
U+594A.0	fMandarin	xie2; xie3
U+594B.0	fUTF8	奋
U+594B.0	fCNS11643	3-282E
U+594B.0	fGB	0-375C
U+594B.0	fR/S	  37.5
U+594B.0	fTotalStrokes	 8
U+594B.0	fCangjie	KW
U+594B.0	fFourCorner	40600
U+594B.0	fHakka	 fun3
U+594B.0	fCantonese	fan5
U+594B.0	fMandarin	fen4; gang3
U+594B.0	fEnglish	simplified form of &#x596E;
U+594C.0	fUTF8	奌
U+594C.0	fCNS11643	3-282F
U+594C.0	fGB	E-264F
U+594C.0	fR/S	 37.5
U+594C.0	fTotalStrokes	 8
U+594C.0	fCangjie	YRK
U+594C.0	fFourCorner	21430
U+594C.0	fHakka	 diam3
U+594C.0	fCantonese	dim2
U+594C.0	fMandarin	dian3; dian4; zhan1
U+594C.0	fEnglish	simplified form of &#x9EDE;
U+594D.0	fUTF8	奍
U+594D.0	fCNS11643	3-6467
U+594D.0	fGB	E-2650
U+594D.0	fR/S	 37.5
U+594D.0	fTotalStrokes	 8
U+594D.0	fCangjie	FQLL
U+594D.0	fFourCorner	9022
U+594D.0	fCantonese	hyun3
U+594E.0	fUTF8	奎
U+594E.0	fCNS11643	1-5058
U+594E.0	fBig5	ABB6
U+594E.0	fGB	0-3F7C
U+594E.0	fR/S	 37.6
U+594E.0	fTotalStrokes	 9
U+594E.0	fCangjie	KGG
U+594E.0	fFourCorner	40104
U+594E.0	fHakka	  kui2; kui3; k'wui2
U+594E.0	fCantonese	fui1; kwai1 (also)
U+594E.0	fMandarin	kui2; kui3
U+594E.0	fEnglish	[1] between the buttocks; stride made by a man [2] one of the 28 constellations which ancient Chinese astrologers believed to control the literary trends of the world [3] a Chinese family name
U+594F.0	fUTF8	奏
U+594F.0	fCNS11643	1-5057
U+594F.0	fBig5	ABB5
U+594F.0	fGB	0-5760
U+594F.0	fR/S	 37.6
U+594F.0	fTotalStrokes	 9
U+594F.0	fCangjie	QKHK
U+594F.0	fFourCorner	50430
U+594F.0	fHakka	  zeu5; tsieu5; ziu5
U+594F.0	fCantonese	zau3; cau3
U+594F.0	fMandarin	zou4
U+594F.0	fEnglish	[1] report to the throne [2] play (music or musical instruments) [3] move; advance
U+5950.0	fUTF8	奐
U+5950.0	fCNS11643	1-5059
U+5950.0	fBig5	ABB7
U+5950.0	fGB	1-5B3C
U+5950.0	fR/S	 37.6
U+5950.0	fTotalStrokes	 9
U+5950.0	fCangjie	NBK
U+5950.0	fFourCorner	27430
U+5950.0	fHakka	  fon5; fan5; fon3
U+5950.0	fCantonese	wun6
U+5950.0	fMandarin	huan4
U+5950.0	fEnglish	[1] numerous; many [2] brilliant; colorful; gay; lively [3] leisurely [4] excellent; elegant [5] a Chinese family name
U+5951.0	fUTF8	契
U+5951.0	fCNS11643	1-5056
U+5951.0	fBig5	ABB4
U+5951.0	fGB	0-4675
U+5951.0	fR/S	 37.6
U+5951.0	fTotalStrokes	 9
U+5951.0	fCangjie	QHK
U+5951.0	fFourCorner	57432
U+5951.0	fHakka	 k'ie5;  ke5; siet7; ke3; giet7; k'e3; k'e5
U+5951.0	fCantonese	kai3; kit3; sit3; kai2; hat1
U+5951.0	fMandarin	qi4; xie4; jie3; qi3; qie1
U+5951.0	fEnglish	[1] contract; bond; agreement [2] a divining instrument in ancient China [3] engrave; cut; carve; notch [4] be compatible; harmonious in thought and aspiration [5] adopt
U+5951.1	fUTF8	契
U+5951.1	fCNS11643	1-5056
U+5951.1	fBig5	ABB4
U+5951.1	fGB	0-4675
U+5951.1	fR/S	 37.6
U+5951.1	fTotalStrokes	 9
U+5951.1	fCangjie	QHK
U+5951.1	fFourCorner	57432
U+5951.1	fMandarin	qie4
U+5951.1	fEnglish	[1] carve [2] be separated from
U+5952.0	fUTF8	奒
U+5952.0	fCNS11643	3-2B7C
U+5952.0	fGB	5-373B
U+5952.0	fR/S	  37.6
U+5952.0	fTotalStrokes	 9
U+5952.0	fCangjie	KYVO
U+5952.0	fFourCorner	40280
U+5952.0	fCantonese	hoi1
U+5952.0	fMandarin	kai1; zha1; zha4
U+5953.0	fUTF8	奓
U+5953.0	fCNS11643	2-2962
U+5953.0	fBig5	CE61
U+5953.0	fGB	3-3533
U+5953.0	fR/S	 37.6
U+5953.0	fTotalStrokes	 9
U+5953.0	fCangjie	KNIN
U+5953.0	fFourCorner	40207
U+5953.0	fHakka	 cha1; sa1; ca1; za1; zha1
U+5953.0	fCantonese	zaa3; se1 (also); ce1; zaa1; zaa6 (also); ci2; se2
U+5953.0	fMandarin	she1; chi3; zha4; zha1
U+5953.0	fEnglish	extravagant
U+5954.0	fUTF8	奔
U+5954.0	fCNS11643	1-4C64
U+5954.0	fBig5	A962
U+5954.0	fGB	0-313C
U+5954.0	fR/S	 37.6
U+5954.0	fTotalStrokes	 9
U+5954.0	fCangjie	KJT
U+5954.0	fFourCorner	40444
U+5954.0	fHakka	  bun1; pun1
U+5954.0	fCantonese	ban1; ban3
U+5954.0	fMandarin	ben1; fen4
U+5954.0	fEnglish	[1] move quickly; rush about; run; hurry [2] run for one's life; flee [3] elope
U+5954.1	fUTF8	奔
U+5954.1	fCNS11643	1-4C64
U+5954.1	fBig5	A962
U+5954.1	fGB	0-313C
U+5954.1	fR/S	 37.6
U+5954.1	fTotalStrokes	 9
U+5954.1	fCangjie	KJT
U+5954.1	fFourCorner	40444
U+5954.1	fMandarin	ben4
U+5954.1	fEnglish	 go straight forwards; head for
U+5955.0	fUTF8	奕
U+5955.0	fCNS11643	1-5055
U+5955.0	fBig5	ABB3
U+5955.0	fGB	0-5E48
U+5955.0	fR/S	 37.6
U+5955.0	fTotalStrokes	 9
U+5955.0	fCangjie	YCK
U+5955.0	fFourCorner	00430
U+5955.0	fHakka	  jit8; rit8
U+5955.0	fCantonese	jik6
U+5955.0	fMandarin	yi4
U+5955.0	fEnglish	[1] great; grand; abundant [2] gorgeous; elegant; good-looking [3] worried; unsettled; anxious [4] in good order; in sequence
U+5956.0	fUTF8	奖
U+5956.0	fGB	0-3D31
U+5956.0	fR/S	 37.6
U+5956.0	fTotalStrokes	 9
U+5956.0	fCangjie	LNK
U+5956.0	fHakka	 ziong3
U+5956.0	fCantonese	zoeng2
U+5956.0	fMandarin	jiang3
U+5956.0	fEnglish	simplified form of &#x596C;
U+5957.0	fUTF8	套
U+5957.0	fCNS11643	1-546F
U+5957.0	fBig5	AE4D
U+5957.0	fGB	0-4C57
U+5957.0	fR/S	 37.7
U+5957.0	fTotalStrokes	 10
U+5957.0	fCangjie	KSMI
U+5957.0	fFourCorner	40731
U+5957.0	fHakka	  tau5;  to5; lau5
U+5957.0	fCantonese	tou3; tou2 (also)
U+5957.0	fMandarin	tao4; tao3
U+5957.0	fEnglish	[1] case; cover; wrapper; envelope; sheath [2] trap; snare; noose [3] wear; slip on (a sweater, etc.) [4] trap or trick a person (into telling the truth) [5] harness [6] pattern or model after; a set pattern [7] convention; formula [8] classifier for clothes, tableware, etc.
U+5958.0	fUTF8	奘
U+5958.0	fCNS11643	1-5470
U+5958.0	fBig5	AE4E
U+5958.0	fGB	0-5E4A
U+5958.0	fR/S	 37.7
U+5958.0	fTotalStrokes	 10
U+5958.0	fCangjie	VGK
U+5958.0	fFourCorner	24430
U+5958.0	fHakka	  zong5; zong1
U+5958.0	fCantonese	zong6; zong1 (also); zong5; zong3
U+5958.0	fMandarin	zang4; zhuang3; zhuan3
U+5958.0	fEnglish	large; powerful; stout; thick
U+5959.0	fUTF8	奙
U+5959.0	fCNS11643	4-2C38
U+5959.0	fGB	E-2651
U+5959.0	fR/S	 37.7
U+5959.0	fTotalStrokes	 10
U+5959.0	fCangjie	XIKLU
U+5959.0	fFourCorner	23710
U+5959.0	fCantonese	bun2
U+5959.0	fMandarin	ben3
U+595A.0	fUTF8	奚
U+595A.0	fCNS11643	1-5471
U+595A.0	fBig5	AE4F
U+595A.0	fGB	0-5E49
U+595A.0	fR/S	 37.7
U+595A.0	fTotalStrokes	 10
U+595A.0	fCangjie	BVIK
U+595A.0	fFourCorner	20430
U+595A.0	fHakka	 hi1; he1; hi2; he2
U+595A.0	fCantonese	hai4
U+595A.0	fMandarin	xi1; xi2
U+595A.0	fEnglish	[1] where? what? how? why?; which? [2] servant [3] name of a Tartar tribe in ancient China [4] a Chinese family name
U+595B.0	fUTF8	奛
U+595B.0	fCNS11643	3-347B
U+595B.0	fGB	E-2652
U+595B.0	fR/S	  37.8
U+595B.0	fTotalStrokes	 11
U+595B.0	fCangjie	KAB
U+595B.0	fFourCorner	40620
U+595B.0	fCantonese	kong3
U+595B.0	fMandarin	hang3; huang3; xiang3
U+595B.0	fEnglish	opening; radiant
U+595C.0	fUTF8	奜
U+595C.0	fCNS11643	2-3450
U+595C.0	fBig5	D4CD
U+595C.0	fGB	3-7456
U+595C.0	fR/S	 37.8
U+595C.0	fTotalStrokes	 11
U+595C.0	fCangjie	LYK
U+595C.0	fFourCorner	11431
U+595C.0	fHakka	 fui3; fi3
U+595C.0	fCantonese	fei2
U+595C.0	fMandarin	fei3; fei1; fei4
U+595C.0	fEnglish	big
U+595D.0	fUTF8	奝
U+595D.0	fCNS11643	3-347C
U+595D.0	fGB	3-3537
U+595D.0	fR/S	  37.8
U+595D.0	fTotalStrokes	 11
U+595D.0	fCangjie	KBGR
U+595D.0	fFourCorner	40220
U+595D.0	fCantonese	diu1
U+595D.0	fMandarin	diao1
U+595D.0	fEnglish	large
U+595E.0	fUTF8	奞
U+595E.0	fCNS11643	3-347D
U+595E.0	fGB	3-3535
U+595E.0	fR/S	 37.8
U+595E.0	fTotalStrokes	 11
U+595E.0	fCangjie	KOG
U+595E.0	fFourCorner	40210
U+595E.0	fHakka	 sui1; kwi2
U+595E.0	fCantonese	seon3
U+595E.0	fMandarin	xun4
U+595E.0	fEnglish	the stride made by a man
U+595F.0	fUTF8	奟
U+595F.0	fCNS11643	4-3126
U+595F.0	fGB	3-3536
U+595F.0	fR/S	 37.9; 37.8
U+595F.0	fTotalStrokes	 11; 12
U+595F.0	fCangjie	KBB
U+595F.0	fFourCorner	40220
U+595F.0	fCantonese	bang1
U+595F.0	fMandarin	keng1; beng1
U+5960.0	fUTF8	奠
U+5960.0	fCNS11643	1-5E64
U+5960.0	fBig5	B3FE
U+5960.0	fGB	0-356C
U+5960.0	fR/S	 37.9
U+5960.0	fTotalStrokes	 12
U+5960.0	fCangjie	TWK
U+5960.0	fFourCorner	80430
U+5960.0	fHakka	 ten3; tien5; tien6; t'ian6; tin1
U+5960.0	fCantonese	din6
U+5960.0	fMandarin	dian4; ding4; ting2
U+5960.0	fEnglish	[1] settle; lay (foundation, etc.) [2] secure; consolidate [3] offer libations
U+5961.0	fUTF8	奡
U+5961.0	fCNS11643	2-3A76
U+5961.0	fBig5	D8B4
U+5961.0	fGB	3-3026
U+5961.0	fR/S	 37.9
U+5961.0	fTotalStrokes	 12
U+5961.0	fCangjie	MUKLL
U+5961.0	fFourCorner	10336
U+5961.0	fHakka	 ngau5; ngau6; ngo6; ngo5
U+5961.0	fCantonese	ngou6; ou6 (also)
U+5961.0	fMandarin	ao4
U+5961.0	fEnglish	arrogant
U+5962.0	fUTF8	奢
U+5962.0	fCNS11643	1-595D
U+5962.0	fBig5	B0F8
U+5962.0	fGB	0-495D
U+5962.0	fR/S	 37.9
U+5962.0	fTotalStrokes	 12
U+5962.0	fCangjie	KJKA
U+5962.0	fFourCorner	40604
U+5962.0	fHakka	  ca1;  cha1; sa1; za1; zha1
U+5962.0	fCantonese	ce1; se1 (also)
U+5962.0	fMandarin	she1
U+5962.0	fEnglish	[1] extravagant; wasteful; lavish [2] excess; excessive [3] exaggerate; brag
U+5963.0	fUTF8	奣
U+5963.0	fCNS11643	3-3A53
U+5963.0	fGB	3-476F
U+5963.0	fR/S	 37.9
U+5963.0	fTotalStrokes	 12
U+5963.0	fCangjie	MKAB
U+5963.0	fFourCorner	10620
U+5963.0	fHakka	 jang3; rang3
U+5963.0	fCantonese	wang2
U+5963.0	fMandarin	weng3; weng1
U+5963.0	fEnglish	the year opening brightly
U+5964.0	fUTF8	奤
U+5964.0	fCNS11643	3-3A54
U+5964.0	fGB	3-3538
U+5964.0	fR/S	  37.9
U+5964.0	fTotalStrokes	 12
U+5964.0	fCangjie	KMWL
U+5964.0	fFourCorner	40600
U+5964.0	fCantonese	haa5
U+5964.0	fMandarin	pan3; ha3; tai3
U+5965.0	fUTF8	奥
U+5965.0	fCNS11643	4-3679
U+5965.0	fGB	0-3042
U+5965.0	fR/S	 37.9
U+5965.0	fTotalStrokes	 12
U+5965.0	fCangjie	HBK
U+5965.0	fFourCorner	27430
U+5965.0	fHakka	 au5
U+5965.0	fCantonese	ou3; ngou3 (also); juk1; ou2 (also); ngou2 (also)
U+5965.0	fMandarin	ao4; yu4
U+5965.0	fEnglish	[1] mysterious; obscure; profound (learning) [2] secret cabin or corner of a house or palace [3] Austria [4] non-classical form of &#x5967;
U+5966.0	fUTF8	奦
U+5966.0	fCNS11643	3-4060
U+5966.0	fGB	3-353B
U+5966.0	fR/S	 37.10; 37.11
U+5966.0	fTotalStrokes	 13; 14
U+5966.0	fCangjie	NSK
U+5966.0	fFourCorner	18430
U+5966.0	fCantonese	mou6
U+5966.0	fMandarin	wu4
U+5967.0	fUTF8	奧
U+5967.0	fCNS11643	1-635F
U+5967.0	fBig5	B6F8
U+5967.0	fGB	E-2653
U+5967.0	fR/S	 37.10
U+5967.0	fTotalStrokes	 13
U+5967.0	fCangjie	HBK
U+5967.0	fFourCorner	27430
U+5967.0	fHakka	 au5; o5
U+5967.0	fCantonese	ou3; ngou3 (also); juk1; ou2 (also); ngou2 (also)
U+5967.0	fMandarin	ao4; yu4
U+5967.0	fEnglish	[1] mysterious; obscure; profound [2] the south-west corner, where the lares used to be placed [3] secret cabin or corner of a house or palace [4] Austria [5] quiet; retired [6] warm
U+5968.0	fUTF8	奨
U+5968.0	fCNS11643	3-4061
U+5968.0	fGB	E-2654
U+5968.0	fR/S	 37.10
U+5968.0	fTotalStrokes	 13
U+5968.0	fCangjie	LIK
U+5968.0	fFourCorner	32430
U+5968.0	fHakka	 ziong3
U+5968.0	fCantonese	zoeng2
U+5968.0	fMandarin	jiang3
U+5968.0	fEnglish	simplified form of &#x596C;
U+5969.0	fUTF8	奩
U+5969.0	fCNS11643	1-6845
U+5969.0	fBig5	B9DD
U+5969.0	fGB	1-5E46
U+5969.0	fR/S	 37.11
U+5969.0	fTotalStrokes	 14
U+5969.0	fCangjie	KSRR
U+5969.0	fFourCorner	40716
U+5969.0	fHakka	 liam2
U+5969.0	fCantonese	lim4
U+5969.0	fMandarin	lian2
U+5969.0	fEnglish	 lady's vanity case trousseau
U+596A.0	fUTF8	奪
U+596A.0	fCNS11643	1-6844
U+596A.0	fBig5	B9DC
U+596A.0	fGB	1-3661
U+596A.0	fR/S	 37.11
U+596A.0	fTotalStrokes	 14
U+596A.0	fCangjie	KOGI
U+596A.0	fFourCorner	40341
U+596A.0	fHakka	  tot8; tot7
U+596A.0	fCantonese	dyut6
U+596A.0	fMandarin	duo2; duo4
U+596A.0	fEnglish	[1] take by force; rob [2] snatch; grasp; carry away (the first prize, etc.) [3] settle; decide
U+596B.0	fUTF8	奫
U+596B.0	fCNS11643	2-4951
U+596B.0	fBig5	E16A
U+596B.0	fGB	3-353D
U+596B.0	fR/S	 37.11
U+596B.0	fTotalStrokes	 14
U+596B.0	fCangjie	KELL
U+596B.0	fFourCorner	40123
U+596B.0	fHakka	 wun1; vun1
U+596B.0	fCantonese	wan1; jyun1
U+596B.0	fMandarin	yun1
U+596B.0	fEnglish	vast; profound; deep
U+596C.0	fUTF8	奬
U+596C.0	fCNS11643	3-4648
U+596C.0	fGB	1-3D31
U+596C.0	fR/S	  37.11
U+596C.0	fTotalStrokes	 14
U+596C.0	fCangjie	VIK
U+596C.0	fFourCorner	27430
U+596C.0	fHakka	  ziong3
U+596C.0	fCantonese	zoeng2
U+596C.0	fMandarin	jiang3
U+596C.0	fEnglish	[1] encourage; exhort [2] praise; commend [3] prize; reward [4] cite or give a prize or reward to (for a merit, etc.)
U+596D.0	fUTF8	奭
U+596D.0	fCNS11643	1-6C47
U+596D.0	fBig5	BC5D
U+596D.0	fGB	3-353C
U+596D.0	fR/S	 37.12
U+596D.0	fTotalStrokes	 15
U+596D.0	fCangjie	KMAA
U+596D.0	fFourCorner	40036
U+596D.0	fHakka	 hit7;  sit7; shit7
U+596D.0	fCantonese	sik1
U+596D.0	fMandarin	shi4
U+596D.0	fEnglish	[1] red; [2] a Chinese family name
U+596D.1	fUTF8	奭
U+596D.1	fCNS11643	1-6C47
U+596D.1	fBig5	BC5D
U+596D.1	fGB	3-353C
U+596D.1	fR/S	 37.12
U+596D.1	fTotalStrokes	 15
U+596D.1	fCangjie	KMAA
U+596D.1	fFourCorner	40036
U+596D.1	fMandarin	he4
U+596D.1	fEnglish	 be angry
U+596E.0	fUTF8	奮
U+596E.0	fCNS11643	1-704D
U+596E.0	fBig5	BEC4
U+596E.0	fGB	1-375C
U+596E.0	fR/S	 37.13
U+596E.0	fTotalStrokes	 16
U+596E.0	fCangjie	KOGW
U+596E.0	fFourCorner	40601
U+596E.0	fHakka	  fun5; fun3; bin1; fun6
U+596E.0	fCantonese	fan5; fan3
U+596E.0	fMandarin	fen4
U+596E.0	fEnglish	[1] rise in force; arouse; exert with force [2] take wing (said of a bird) [3] advance; promote; invigorate; strive (a cause, etc.)
U+596F.0	fUTF8	奯
U+596F.0	fCNS11643	4-4F7B
U+596F.0	fGB	3-3531
U+596F.0	fR/S	  37.13
U+596F.0	fTotalStrokes	 16
U+596F.0	fCangjie	KYMH
U+596F.0	fFourCorner	40250
U+596F.0	fCantonese	kut3; kwut3 (also)
U+596F.0	fMandarin	huo4
U+5970.0	fUTF8	奰
U+5970.0	fCNS11643	2-614C
U+5970.0	fBig5	EFC0
U+5970.0	fGB	3-353E
U+5970.0	fR/S	 37.15
U+5970.0	fTotalStrokes	 18
U+5970.0	fCangjie	WLWWK
U+5970.0	fFourCorner	60436
U+5970.0	fHakka	 pi5; pi6
U+5970.0	fCantonese	bei6; bai3 (also)
U+5970.0	fMandarin	bi4; bei4
U+5970.0	fEnglish	anger
U+5971.0	fUTF8	奱
U+5971.0	fCNS11643	2-6D48
U+5971.0	fBig5	F6DA
U+5971.0	fGB	E-2655
U+5971.0	fR/S	 37.19
U+5971.0	fTotalStrokes	 22
U+5971.0	fCangjie	VFK
U+5971.0	fFourCorner	22801
U+5971.0	fCantonese	lyun4
U+5971.0	fMandarin	lian2
U+5972.0	fUTF8	奲
U+5972.0	fCNS11643	2-6F23
U+5972.0	fBig5	F7D4
U+5972.0	fGB	5-3748
U+5972.0	fR/S	 37.21
U+5972.0	fTotalStrokes	 24
U+5972.0	fCangjie	KARRJ
U+5972.0	fFourCorner	46656
U+5972.0	fCantonese	ce2; do2
U+5972.0	fMandarin	duo3; che3; chan3
U+5973.0	fUTF8	女
U+5973.0	fCNS11643	1-444C
U+5973.0	fBig5	A46B
U+5973.0	fGB	0-452E
U+5973.0	fR/S	 38.0
U+5973.0	fTotalStrokes	 3
U+5973.0	fCangjie	V
U+5973.0	fFourCorner	40400
U+5973.0	fHakka	  ngi3;  ng3; gni3; ngiu3
U+5973.0	fCantonese	neoi5; leoi5 (also); neoi6; neoi4 (also); leoi4 (also); jyu5; neoi2; leoi2 (also); leoi6 (also)
U+5973.0	fMandarin	ru3; nuu3; n3
U+5973.0	fEnglish	[1] woman; female [2] daughter; girl; maiden; lady [3] feminine [4] name of one of the 28 constellations [5] Kangxi radical 38
U+5973.1	fUTF8	女
U+5973.1	fCNS11643	1-444C
U+5973.1	fBig5	A46B
U+5973.1	fGB	0-452E
U+5973.1	fR/S	 38.0
U+5973.1	fTotalStrokes	 3
U+5973.1	fCangjie	V
U+5973.1	fFourCorner	40400
U+5973.1	fMandarin	nuu5
U+5974.0	fUTF8	奴
U+5974.0	fCNS11643	1-4643
U+5974.0	fBig5	A5A3
U+5974.0	fGB	0-452B
U+5974.0	fR/S	 38.2
U+5974.0	fTotalStrokes	 5
U+5974.0	fCangjie	VE
U+5974.0	fFourCorner	47440
U+5974.0	fHakka	  nu2;  lu2; nung2; nu3
U+5974.0	fCantonese	nou4; lou4 (also)
U+5974.0	fMandarin	nu2
U+5974.0	fEnglish	[1] slave; servant [2] a self-derogatory expression used by a girl to refer to herself in former times [3] despicable yes-man
U+5975.0	fUTF8	奵
U+5975.0	fCNS11643	3-223B
U+5975.0	fGB	3-3B7A
U+5975.0	fR/S	 38.2
U+5975.0	fTotalStrokes	 5
U+5975.0	fCangjie	VMN
U+5975.0	fFourCorner	41420
U+5975.0	fCantonese	ding2
U+5975.0	fMandarin	ding1; ding3
U+5976.0	fUTF8	奶
U+5976.0	fCNS11643	1-4644
U+5976.0	fBig5	A5A4
U+5976.0	fGB	0-444C
U+5976.0	fR/S	 38.2
U+5976.0	fTotalStrokes	 5
U+5976.0	fCangjie	VNHS
U+5976.0	fFourCorner	47427
U+5976.0	fHakka	  nai1;  lai1; nai3; nen5; len1
U+5976.0	fCantonese	naai5; laai5 (also); naai4 (also); laai4 (also); naai1; naai2 (also); laai2 (also); laai1 (also)
U+5976.0	fMandarin	nai3
U+5976.0	fEnglish	[1] milk [2] woman's breasts [3] grandma; nurse [4] feed with milk; breast-feed; nurse
U+5976.1	fUTF8	奶
U+5976.1	fCNS11643	1-4644
U+5976.1	fBig5	A5A4
U+5976.1	fGB	0-444C
U+5976.1	fR/S	 38.2
U+5976.1	fTotalStrokes	 5
U+5976.1	fCangjie	VNHS
U+5976.1	fFourCorner	47427
U+5976.1	fMandarin	nai5
U+5977.0	fUTF8	奷
U+5977.0	fCNS11643	2-2231
U+5977.0	fBig5	C9D1
U+5977.0	fGB	E-2656
U+5977.0	fR/S	 38.3
U+5977.0	fTotalStrokes	 6
U+5977.0	fCangjie	VHJ
U+5977.0	fFourCorner	42480
U+5977.0	fCantonese	cin1
U+5977.0	fMandarin	qian1
U+5977.0	fEnglish	used in a female name
U+5978.0	fUTF8	奸
U+5978.0	fCNS11643	1-476D
U+5978.0	fBig5	A66C
U+5978.0	fGB	0-3C69
U+5978.0	fR/S	 38.3
U+5978.0	fTotalStrokes	 6
U+5978.0	fCangjie	VMJ
U+5978.0	fFourCorner	41440
U+5978.0	fHakka	 gan1;  gien1; gian1
U+5978.0	fCantonese	gaan1
U+5978.0	fMandarin	jian1; gan1
U+5978.0	fEnglish	[1] crafty; villainous; wicked; cunning; evil [2] false [3] selfish [4] disloyal [5] adultery; fornication; licentiousness [6] traitor; villain
U+5979.0	fUTF8	她
U+5979.0	fCNS11643	1-4770
U+5979.0	fBig5	A66F
U+5979.0	fGB	0-4B7D
U+5979.0	fR/S	 38.3
U+5979.0	fTotalStrokes	 6
U+5979.0	fCangjie	VPD
U+5979.0	fFourCorner	44412
U+5979.0	fHakka	 ta1
U+5979.0	fCantonese	taa1; ji1
U+5979.0	fMandarin	ta1; jie3; yi1
U+5979.0	fEnglish	she; her
U+597A.0	fUTF8	奺
U+597A.0	fCNS11643	3-234B
U+597A.0	fGB	3-3B7D
U+597A.0	fR/S	 38.3
U+597A.0	fTotalStrokes	 6
U+597A.0	fCangjie	VNO
U+597A.0	fFourCorner	47480
U+597A.0	fCantonese	gau2
U+597A.0	fMandarin	jiu3
U+597B.0	fUTF8	奻
U+597B.0	fCNS11643	2-222F
U+597B.0	fBig5	C9CF
U+597B.0	fGB	5-3F42
U+597B.0	fR/S	 38.3
U+597B.0	fTotalStrokes	 6
U+597B.0	fCangjie	VV
U+597B.0	fFourCorner	44444
U+597B.0	fCantonese	nyun4; lyun4 (also); naan4
U+597B.0	fMandarin	nan2; nan4
U+597C.0	fUTF8	奼
U+597C.0	fCNS11643	2-222D
U+597C.0	fBig5	C9CD
U+597C.0	fGB	3-3B7C
U+597C.0	fR/S	 38.3
U+597C.0	fTotalStrokes	 6
U+597C.0	fCangjie	VHP
U+597C.0	fFourCorner	42411
U+597C.0	fHakka	  za5; ca3
U+597C.0	fCantonese	caa3; zaa3
U+597C.0	fMandarin	cha4; cha3; du4
U+597C.0	fEnglish	 handsome, elegant young woman
U+597D.0	fUTF8	好
U+597D.0	fCNS11643	1-476F
U+597D.0	fBig5	A66E
U+597D.0	fGB	0-3A43
U+597D.0	fR/S	 38.3
U+597D.0	fTotalStrokes	 6
U+597D.0	fCangjie	VND
U+597D.0	fFourCorner	47447
U+597D.0	fHakka	  hau3;  ho3
U+597D.0	fCantonese	hou2; hou3
U+597D.0	fMandarin	hao3
U+597D.0	fEnglish	[1] good; nice; excellent; fine; well [2] pleasing (looks, taste, etc.) [3] finish (dressing, eating, etc.) [4] very much [5] can; so that [6] all right! [7] an exclamatory expression [8] friendly meeting [9] fit; suitable; proper [10] able [11] ought; can
U+597D.1	fUTF8	好
U+597D.1	fCNS11643	1-476F
U+597D.1	fBig5	A66E
U+597D.1	fGB	0-3A43
U+597D.1	fR/S	 38.3
U+597D.1	fTotalStrokes	 6
U+597D.1	fCangjie	VND
U+597D.1	fFourCorner	47447
U+597D.1	fHakka	 hau5
U+597D.1	fMandarin	hao4
U+597D.1	fEnglish	[1] love; be fond of [2] be addicted to [3] what one likes
U+597D.2	fUTF8	好
U+597D.2	fCNS11643	1-476F
U+597D.2	fBig5	A66E
U+597D.2	fGB	0-3A43
U+597D.2	fR/S	 38.3
U+597D.2	fTotalStrokes	 6
U+597D.2	fCangjie	VND
U+597D.2	fFourCorner	47447
U+597D.2	fMandarin	hao1
U+597E.0	fUTF8	奾
U+597E.0	fCNS11643	2-2230
U+597E.0	fBig5	C9D0
U+597E.0	fGB	5-3F3F
U+597E.0	fR/S	 38.3
U+597E.0	fTotalStrokes	 6
U+597E.0	fCangjie	VU
U+597E.0	fFourCorner	42470
U+597E.0	fCantonese	sin1
U+597E.0	fMandarin	xian1
U+597E.0	fEnglish	used in a female name
U+597F.0	fUTF8	奿
U+597F.0	fCNS11643	2-2232
U+597F.0	fBig5	C9D2
U+597F.0	fGB	E-2657
U+597F.0	fR/S	 38.3
U+597F.0	fTotalStrokes	 6
U+597F.0	fCangjie	KIV
U+597F.0	fFourCorner	54014
U+597F.0	fCantonese	faan3
U+597F.0	fMandarin	fan4
U+5980.0	fUTF8	妀
U+5980.0	fCNS11643	2-222C
U+5980.0	fBig5	C9CC
U+5980.0	fGB	E-2658
U+5980.0	fR/S	 38.3
U+5980.0	fTotalStrokes	 6
U+5980.0	fCangjie	SUV
U+5980.0	fFourCorner	14740
U+5980.0	fCantonese	gei2
U+5980.0	fMandarin	ji3
U+5980.0	fEnglish	used in a female name
U+5981.0	fUTF8	妁
U+5981.0	fCNS11643	1-4772
U+5981.0	fBig5	A671
U+5981.0	fGB	0-6579
U+5981.0	fR/S	 38.3
U+5981.0	fTotalStrokes	 6
U+5981.0	fCangjie	VPI
U+5981.0	fFourCorner	47420
U+5981.0	fHakka	 shok7; zok7; chok7; sok7
U+5981.0	fCantonese	zoek3
U+5981.0	fMandarin	shuo4; zhuo2
U+5981.0	fEnglish	 matchmaker; go-between
U+5982.0	fUTF8	如
U+5982.0	fCNS11643	1-4771
U+5982.0	fBig5	A670
U+5982.0	fGB	0-4867
U+5982.0	fR/S	 38.3
U+5982.0	fTotalStrokes	 6
U+5982.0	fCangjie	VR
U+5982.0	fFourCorner	46400
U+5982.0	fHakka	 ji2; ji3; ri2; i2
U+5982.0	fCantonese	jyu4
U+5982.0	fMandarin	ru2
U+5982.0	fEnglish	[1] if; supposing [2] as if [3] like; as [4] as good as; equal to [5] follow (advice); listen to [6] go to; arrive at [7] should; ought [8] on time; in time
U+5983.0	fUTF8	妃
U+5983.0	fCNS11643	1-476E
U+5983.0	fBig5	A66D
U+5983.0	fGB	0-657A
U+5983.0	fR/S	 38.3
U+5983.0	fTotalStrokes	 6
U+5983.0	fCangjie	VSU
U+5983.0	fFourCorner	47417
U+5983.0	fHakka	 fui1; fi1
U+5983.0	fCantonese	fei1; pui3
U+5983.0	fMandarin	fei1; pei4; yi2
U+5983.0	fEnglish	[1] wife; spouse [2] imperial concubine; concubine of a king or an emperor [3] wife or spouse of a crown prince
U+5984.0	fUTF8	妄
U+5984.0	fCNS11643	1-476C
U+5984.0	fBig5	A66B
U+5984.0	fGB	0-4D7D
U+5984.0	fR/S	 38.3
U+5984.0	fTotalStrokes	 6
U+5984.0	fCangjie	YVV
U+5984.0	fFourCorner	00404
U+5984.0	fHakka	 mong3;  wong3; mong5; mong2; mong1; mong6; nong5; nong6; vong3
U+5984.0	fCantonese	mong5; mong6
U+5984.0	fMandarin	wang4; wang2
U+5984.0	fEnglish	[1] absurd; untrue; false [2] ignorant; foolish [3] reckless; rash [4] wild; frantic; frenetic
U+5985.0	fUTF8	妅
U+5985.0	fCNS11643	2-222E
U+5985.0	fBig5	C9CE
U+5985.0	fGB	5-3F3D
U+5985.0	fR/S	 38.3
U+5985.0	fTotalStrokes	 6
U+5985.0	fCangjie	VM
U+5985.0	fFourCorner	41410
U+5985.0	fCantonese	hung4
U+5985.0	fMandarin	hong2
U+5985.0	fEnglish	used in a female name
U+5986.0	fUTF8	妆
U+5986.0	fCNS11643	3-234D
U+5986.0	fGB	0-5731
U+5986.0	fR/S	 38.3
U+5986.0	fTotalStrokes	 6
U+5986.0	fCangjie	LMV
U+5986.0	fFourCorner	34140
U+5986.0	fHakka	 zong1
U+5986.0	fCantonese	zong1
U+5986.0	fMandarin	zhuang1; zhuan1
U+5986.0	fEnglish	simplified form of &#x599D;
U+5987.0	fUTF8	妇
U+5987.0	fGB	0-383E
U+5987.0	fR/S	 38.3
U+5987.0	fTotalStrokes	 6
U+5987.0	fCangjie	VSM
U+5987.0	fHakka	 fu5
U+5987.0	fCantonese	fu5
U+5987.0	fMandarin	fu4
U+5987.0	fEnglish	simplified form of &#x5A66;
U+5987.1	fUTF8	妇
U+5987.1	fGB	0-383E
U+5987.1	fR/S	 38.3
U+5987.1	fTotalStrokes	 6
U+5987.1	fCangjie	VSM
U+5987.1	fMandarin	fu5
U+5988.0	fUTF8	妈
U+5988.0	fGB	0-4268
U+5988.0	fR/S	 38.3
U+5988.0	fTotalStrokes	 6
U+5988.0	fCangjie	VNVM
U+5988.0	fHakka	 ma1
U+5988.0	fCantonese	maa1; maa4 (also); maa2 (also); maa5
U+5988.0	fMandarin	ma1; mu3
U+5988.0	fEnglish	simplified form of &#x5ABD;
U+5988.1	fUTF8	妈
U+5988.1	fGB	0-4268
U+5988.1	fR/S	 38.3
U+5988.1	fTotalStrokes	 6
U+5988.1	fCangjie	VNVM
U+5988.1	fMandarin	ma5
U+5989.0	fUTF8	妉
U+5989.0	fCNS11643	3-2555
U+5989.0	fGB	5-3F4C
U+5989.0	fR/S	 38.4
U+5989.0	fTotalStrokes	 7
U+5989.0	fCangjie	VLBU
U+5989.0	fFourCorner	44410
U+5989.0	fCantonese	daam1
U+5989.0	fMandarin	dan1
U+598A.0	fUTF8	妊
U+598A.0	fCNS11643	1-4973
U+598A.0	fBig5	A7B3
U+598A.0	fGB	0-4851
U+598A.0	fR/S	 38.4
U+598A.0	fTotalStrokes	 7
U+598A.0	fCangjie	VHG
U+598A.0	fFourCorner	42414
U+598A.0	fHakka	 ngim2;  ngim3; ngim5; rim2; rim3; rim6; jim2; jim3
U+598A.0	fCantonese	jam4 (also); jam6
U+598A.0	fMandarin	ren4; ren2
U+598A.0	fEnglish	 conceive; be pregnant; be expecting
U+598B.0	fUTF8	妋
U+598B.0	fCNS11643	4-2367
U+598B.0	fGB	5-3F44
U+598B.0	fR/S	  38.4
U+598B.0	fTotalStrokes	 7
U+598B.0	fCangjie	VQO
U+598B.0	fFourCorner	45430
U+598B.0	fCantonese	fu1
U+598B.0	fMandarin	fu1
U+598C.0	fUTF8	妌
U+598C.0	fCNS11643	3-254C
U+598C.0	fGB	3-3C22
U+598C.0	fR/S	  38.4
U+598C.0	fTotalStrokes	 7
U+598C.0	fCangjie	VTT
U+598C.0	fFourCorner	45400
U+598C.0	fHakka	 ziang3
U+598C.0	fCantonese	zing6
U+598C.0	fMandarin	jing4
U+598C.0	fEnglish	female virtue
U+598D.0	fUTF8	妍
U+598D.0	fCNS11643	1-4970
U+598D.0	fBig5	A7B0
U+598D.0	fGB	0-657B
U+598D.0	fR/S	 38.6
U+598D.0	fTotalStrokes	 9
U+598D.0	fCangjie	VMT
U+598D.0	fFourCorner	41440
U+598D.0	fHakka	 ngien2; ngan2; ngan1
U+598D.0	fCantonese	jin4
U+598D.0	fMandarin	yan2
U+598D.0	fEnglish	beautiful; handsome; seductive
U+598E.0	fUTF8	妎
U+598E.0	fCNS11643	2-2356
U+598E.0	fBig5	CAB6
U+598E.0	fGB	3-3C2A
U+598E.0	fR/S	 38.4
U+598E.0	fTotalStrokes	 7
U+598E.0	fCangjie	VOLL
U+598E.0	fFourCorner	48420
U+598E.0	fHakka	 hai5;  gai5
U+598E.0	fCantonese	hoi6; gaai3; hai6
U+598E.0	fMandarin	xie4; jie4; xi4
U+598E.0	fEnglish	envy
U+598F.0	fUTF8	妏
U+598F.0	fCNS11643	2-2359
U+598F.0	fBig5	CAB9
U+598F.0	fGB	5-3F4B
U+598F.0	fR/S	 38.4
U+598F.0	fTotalStrokes	 7
U+598F.0	fCangjie	VYK
U+598F.0	fFourCorner	40441
U+598F.0	fCantonese	man6
U+598F.0	fMandarin	wen4; wen2
U+598F.0	fEnglish	used in a female name
U+5990.0	fUTF8	妐
U+5990.0	fCNS11643	2-2358
U+5990.0	fBig5	CAB8
U+5990.0	fGB	3-3C2C
U+5990.0	fR/S	 38.4
U+5990.0	fTotalStrokes	 7
U+5990.0	fCangjie	VCI
U+5990.0	fFourCorner	48432
U+5990.0	fCantonese	zung1
U+5990.0	fMandarin	zhong1
U+5991.0	fUTF8	妑
U+5991.0	fCNS11643	4-2369
U+5991.0	fGB	3-3C30
U+5991.0	fR/S	  38.4
U+5991.0	fTotalStrokes	 7
U+5991.0	fCangjie	VAU
U+5991.0	fFourCorner	47410
U+5991.0	fHakka	  ba1
U+5991.0	fCantonese	paa1
U+5991.0	fMandarin	pa1
U+5991.0	fEnglish	a woman's name
U+5992.0	fUTF8	妒
U+5992.0	fCNS11643	1-496A
U+5992.0	fBig5	A7AA
U+5992.0	fGB	0-364A
U+5992.0	fR/S	 38.4
U+5992.0	fTotalStrokes	 7
U+5992.0	fCangjie	VHS
U+5992.0	fFourCorner	41460
U+5992.0	fHakka	 du5; du2
U+5992.0	fCantonese	dou3
U+5992.0	fMandarin	du4
U+5992.0	fEnglish	jealous; envious; jealousy; envy
U+5993.0	fUTF8	妓
U+5993.0	fCNS11643	1-4972
U+5993.0	fBig5	A7B2
U+5993.0	fGB	0-3C4B
U+5993.0	fR/S	 38.4
U+5993.0	fTotalStrokes	 7
U+5993.0	fCangjie	VJE
U+5993.0	fFourCorner	44447
U+5993.0	fHakka	  gi3; ki3; gi1; ki1
U+5993.0	fCantonese	gei6
U+5993.0	fMandarin	ji4
U+5993.0	fEnglish	[1] prostitute; whore [2] a young woman who sings or dances to amuse her customers
U+5994.0	fUTF8	妔
U+5994.0	fCNS11643	3-254B
U+5994.0	fGB	3-3C2E
U+5994.0	fR/S	 38.4
U+5994.0	fTotalStrokes	 7
U+5994.0	fCangjie	VYHN
U+5994.0	fFourCorner	40410
U+5994.0	fCantonese	haang1
U+5994.0	fMandarin	keng1
U+5994.0	fEnglish	used in a female name
U+5995.0	fUTF8	妕
U+5995.0	fCNS11643	4-236A
U+5995.0	fGB	E-2659
U+5995.0	fR/S	 38.4
U+5995.0	fTotalStrokes	 7
U+5995.0	fCangjie	VL
U+5995.0	fFourCorner	45400
U+5995.0	fCantonese	zung3
U+5995.0	fMandarin	zhong4
U+5996.0	fUTF8	妖
U+5996.0	fCNS11643	1-496F
U+5996.0	fBig5	A7AF
U+5996.0	fGB	0-517D
U+5996.0	fR/S	 38.4
U+5996.0	fTotalStrokes	 7
U+5996.0	fCangjie	VHK
U+5996.0	fFourCorner	42434
U+5996.0	fHakka	  jau1; jeu1; rau1
U+5996.0	fCantonese	jiu1; jiu2 (also)
U+5996.0	fMandarin	yao1
U+5996.0	fEnglish	[1] strange; weird; supernatural; unaccountable; monstrous [2] evil; monster; goblin; phantom; ghost [3] bewitching; seductive (usually said of a woman)
U+5997.0	fUTF8	妗
U+5997.0	fCNS11643	2-2355
U+5997.0	fBig5	CAB5
U+5997.0	fGB	0-6621
U+5997.0	fR/S	 38.4
U+5997.0	fTotalStrokes	 7
U+5997.0	fCangjie	VOIN
U+5997.0	fFourCorner	48427
U+5997.0	fHakka	 kim1; gim5; kim3
U+5997.0	fCantonese	kam5; kam2
U+5997.0	fMandarin	jin4; chan1; xian1
U+5997.0	fEnglish	wife of mother's brother
U+5998.0	fUTF8	妘
U+5998.0	fCNS11643	2-2353
U+5998.0	fBig5	CAB3
U+5998.0	fGB	3-3C25
U+5998.0	fR/S	 38.4
U+5998.0	fTotalStrokes	 7
U+5998.0	fCangjie	VMMI
U+5998.0	fFourCorner	41431
U+5998.0	fHakka	 jun2; run2
U+5998.0	fCantonese	wan4
U+5998.0	fMandarin	yun2
U+5998.0	fEnglish	a Chinese family name
U+5999.0	fUTF8	妙
U+5999.0	fCNS11643	1-496E
U+5999.0	fBig5	A7AE
U+5999.0	fGB	0-436E
U+5999.0	fR/S	 38.4
U+5999.0	fTotalStrokes	 7
U+5999.0	fCangjie	VFH
U+5999.0	fFourCorner	49420
U+5999.0	fHakka	  miau3; miau5; mieu5; miau6; meu5
U+5999.0	fCantonese	miu6
U+5999.0	fMandarin	miao4
U+5999.0	fEnglish	[1] wonderful; excellent [2] intriguing; very interesting [3] subtle; clever; ingenious
U+599A.0	fUTF8	妚
U+599A.0	fCNS11643	3-2556
U+599A.0	fGB	5-3F45
U+599A.0	fR/S	 38.4
U+599A.0	fTotalStrokes	 7
U+599A.0	fCangjie	VMF
U+599A.0	fFourCorner	41490
U+599A.0	fCantonese	pui1
U+599A.0	fMandarin	bi3
U+599A.0	fEnglish	same as &#x36B0;
U+599B.0	fUTF8	妛
U+599B.0	fCNS11643	3-2553
U+599B.0	fGB	E-265A
U+599B.0	fR/S	 38.4; 46.4
U+599B.0	fTotalStrokes	 7
U+599B.0	fCangjie	UMV
U+599B.0	fFourCorner	22400
U+599B.0	fHakka	 chi1; cih1; zhe3; ze3; chii1
U+599B.0	fCantonese	ci1
U+599B.0	fMandarin	chi1
U+599B.0	fEnglish	same as &#x5AB8;
U+599C.0	fUTF8	妜
U+599C.0	fCNS11643	3-254E
U+599C.0	fGB	3-3C2F
U+599C.0	fR/S	  38.4
U+599C.0	fTotalStrokes	 7
U+599C.0	fCangjie	VDK
U+599C.0	fFourCorner	45430
U+599C.0	fHakka	 giet7; get7
U+599C.0	fCantonese	jyut3
U+599C.0	fMandarin	yue4
U+599C.0	fEnglish	space between the eyebrows and the eyes
U+599D.0	fUTF8	妝
U+599D.0	fCNS11643	1-4969
U+599D.0	fBig5	A7A9
U+599D.0	fGB	1-5731
U+599D.0	fR/S	 38.4
U+599D.0	fTotalStrokes	 7
U+599D.0	fCangjie	VMV
U+599D.0	fFourCorner	24240
U+599D.0	fHakka	  zong1
U+599D.0	fCantonese	zong1
U+599D.0	fMandarin	zhuang1; zhuan1
U+599D.0	fEnglish	[1] doll up; adorn oneself; dress up; apply makeup [2] jewels, etc. for adornment [3] disguise; pretend
U+599E.0	fUTF8	妞
U+599E.0	fCNS11643	1-496C
U+599E.0	fBig5	A7AC
U+599E.0	fGB	0-6624
U+599E.0	fR/S	 38.4
U+599E.0	fTotalStrokes	 7
U+599E.0	fCangjie	VNG
U+599E.0	fFourCorner	47415
U+599E.0	fHakka	  ngiu3; niu3
U+599E.0	fCantonese	nau2; lau2 (also); nau5
U+599E.0	fMandarin	niu1
U+599E.0	fEnglish	 girl; little girl
U+599F.0	fUTF8	妟
U+599F.0	fCNS11643	3-2551
U+599F.0	fGB	5-494E
U+599F.0	fR/S	  38.4
U+599F.0	fTotalStrokes	 7
U+599F.0	fCangjie	AV
U+599F.0	fFourCorner	60400
U+599F.0	fCantonese	aan3; ngaan3 (also)
U+599F.0	fMandarin	yan4
U+59A0.0	fUTF8	妠
U+59A0.0	fCNS11643	2-2354
U+59A0.0	fBig5	CAB4
U+59A0.0	fGB	3-3C29
U+59A0.0	fR/S	 38.4
U+59A0.0	fTotalStrokes	 7
U+59A0.0	fCangjie	VOB
U+59A0.0	fFourCorner	44427
U+59A0.0	fHakka	 nap8; nep7
U+59A0.0	fCantonese	naap6; laap6 (also); laap3
U+59A0.0	fMandarin	na4; nan2; nan4
U+59A0.0	fEnglish	[1] take a wife [2] plump
U+59A1.0	fUTF8	妡
U+59A1.0	fCNS11643	2-235B
U+59A1.0	fBig5	CABB
U+59A1.0	fGB	5-3F48
U+59A1.0	fR/S	 38.4
U+59A1.0	fTotalStrokes	 7
U+59A1.0	fCangjie	VHML
U+59A1.0	fFourCorner	42421
U+59A1.0	fCantonese	jan1
U+59A1.0	fMandarin	xin1; xin4
U+59A1.0	fEnglish	used in a female name
U+59A2.0	fUTF8	妢
U+59A2.0	fCNS11643	2-2357
U+59A2.0	fBig5	CAB7
U+59A2.0	fGB	3-3C2B
U+59A2.0	fR/S	 38.4
U+59A2.0	fTotalStrokes	 7
U+59A2.0	fCangjie	VCSH
U+59A2.0	fFourCorner	48427
U+59A2.0	fHakka	 fun2
U+59A2.0	fCantonese	fan4
U+59A2.0	fMandarin	fen2
U+59A3.0	fUTF8	妣
U+59A3.0	fCNS11643	1-496D
U+59A3.0	fBig5	A7AD
U+59A3.0	fGB	0-657E
U+59A3.0	fR/S	 38.4
U+59A3.0	fTotalStrokes	 7
U+59A3.0	fCangjie	VPP
U+59A3.0	fFourCorner	41410
U+59A3.0	fHakka	  bi3
U+59A3.0	fCantonese	bei2
U+59A3.0	fMandarin	bi3
U+59A3.0	fEnglish	one's deceased mother
U+59A4.0	fUTF8	妤
U+59A4.0	fCNS11643	1-4971
U+59A4.0	fBig5	A7B1
U+59A4.0	fGB	0-6625
U+59A4.0	fR/S	 38.4
U+59A4.0	fTotalStrokes	 7
U+59A4.0	fCangjie	VNIN
U+59A4.0	fFourCorner	47422
U+59A4.0	fHakka	 ji2; ri2; shu1; su1; i2
U+59A4.0	fCantonese	jyu4
U+59A4.0	fMandarin	yu2
U+59A4.0	fEnglish	an official title
U+59A5.0	fUTF8	妥
U+59A5.0	fCNS11643	1-4974
U+59A5.0	fBig5	A7B4
U+59A5.0	fGB	0-4D57
U+59A5.0	fR/S	 38.4
U+59A5.0	fTotalStrokes	 7
U+59A5.0	fCangjie	BV
U+59A5.0	fFourCorner	20404
U+59A5.0	fHakka	  to3; to2
U+59A5.0	fCantonese	to5; to2
U+59A5.0	fMandarin	tuo3
U+59A5.0	fEnglish	[1] firm; safe; secure [2] appropriate [3] ready; set; settle
U+59A6.0	fUTF8	妦
U+59A6.0	fCNS11643	2-2352
U+59A6.0	fBig5	CAB2
U+59A6.0	fGB	5-3F43
U+59A6.0	fR/S	 38.4
U+59A6.0	fTotalStrokes	 7
U+59A6.0	fCangjie	VQJ
U+59A6.0	fFourCorner	45400
U+59A6.0	fHakka	 fung1
U+59A6.0	fCantonese	fung1
U+59A6.0	fMandarin	feng1
U+59A7.0	fUTF8	妧
U+59A7.0	fCNS11643	2-235A
U+59A7.0	fBig5	CABA
U+59A7.0	fGB	3-3C23
U+59A7.0	fR/S	 38.4
U+59A7.0	fTotalStrokes	 7
U+59A7.0	fCangjie	VMMU
U+59A7.0	fFourCorner	41411
U+59A7.0	fHakka	 ngan5
U+59A7.0	fCantonese	wun6; jyun4; jyn4
U+59A7.0	fMandarin	yuan2; wan2
U+59A7.0	fEnglish	beautiful woman
U+59A8.0	fUTF8	妨
U+59A8.0	fCNS11643	1-496B
U+59A8.0	fBig5	A7AB
U+59A8.0	fGB	0-3741
U+59A8.0	fR/S	 38.4
U+59A8.0	fTotalStrokes	 7
U+59A8.0	fCangjie	VYHS
U+59A8.0	fFourCorner	40427
U+59A8.0	fHakka	  fong2
U+59A8.0	fCantonese	fong4
U+59A8.0	fMandarin	fang2; fang4
U+59A8.0	fEnglish	[1] interfere with; hinder; impede; obstruct [2] undermine; harm
U+59A8.1	fUTF8	妨
U+59A8.1	fCNS11643	1-496B
U+59A8.1	fBig5	A7AB
U+59A8.1	fGB	0-3741
U+59A8.1	fR/S	 38.4
U+59A8.1	fTotalStrokes	 7
U+59A8.1	fCangjie	VYHS
U+59A8.1	fFourCorner	40427
U+59A8.1	fMandarin	fang1
U+59A9.0	fUTF8	妩
U+59A9.0	fGB	0-657C
U+59A9.0	fR/S	 38.4
U+59A9.0	fTotalStrokes	 7
U+59A9.0	fCangjie	VMKU
U+59A9.0	fHakka	 vu3
U+59A9.0	fCantonese	mou5; mou4 (also)
U+59A9.0	fMandarin	wu3
U+59A9.0	fEnglish	simplified form of &#x5AF5;
U+59AA.0	fUTF8	妪
U+59AA.0	fGB	0-657D
U+59AA.0	fR/S	 38.4
U+59AA.0	fTotalStrokes	 7
U+59AA.0	fCangjie	VSK
U+59AA.0	fHakka	 ki1; eu1; hi1
U+59AA.0	fCantonese	jyu2; jyu3
U+59AA.0	fMandarin	yu4; yu3; kou1
U+59AA.0	fEnglish	simplified form of &#x5AD7;
U+59AB.0	fUTF8	妫
U+59AB.0	fGB	0-6623
U+59AB.0	fR/S	 38.4
U+59AB.0	fTotalStrokes	 7
U+59AB.0	fCangjie	XVIKS
U+59AB.0	fCantonese	gwai1
U+59AB.0	fMandarin	gui1
U+59AB.0	fEnglish	simplified form of &#x5B00;
U+59AC.0	fUTF8	妬
U+59AC.0	fCNS11643	3-2841
U+59AC.0	fGB	E-265B
U+59AC.0	fR/S	 38.5
U+59AC.0	fTotalStrokes	 8
U+59AC.0	fCangjie	VMR
U+59AC.0	fFourCorner	41460
U+59AC.0	fHakka	  du5
U+59AC.0	fCantonese	dou3
U+59AC.0	fMandarin	du4
U+59AC.0	fEnglish	jealous; envious
U+59AD.0	fUTF8	妭
U+59AD.0	fCNS11643	3-2833
U+59AD.0	fGB	3-3C35
U+59AD.0	fR/S	  38.5
U+59AD.0	fTotalStrokes	 8
U+59AD.0	fCangjie	VIKK
U+59AD.0	fFourCorner	43440
U+59AD.0	fHakka	  pat8
U+59AD.0	fCantonese	bat6
U+59AD.0	fMandarin	ba2
U+59AD.0	fEnglish	wife
U+59AE.0	fUTF8	妮
U+59AE.0	fCNS11643	1-4C69
U+59AE.0	fBig5	A967
U+59AE.0	fGB	0-445D
U+59AE.0	fR/S	 38.5
U+59AE.0	fTotalStrokes	 8
U+59AE.0	fCangjie	VSP
U+59AE.0	fFourCorner	47411
U+59AE.0	fHakka	  ni2; ni3; li2
U+59AE.0	fCantonese	nei4; lei4 (also)
U+59AE.0	fMandarin	ni1; ni2; obs
U+59AE.0	fEnglish	[1] maid; servant girl [2] little girl; cute girl; little darling
U+59AF.0	fUTF8	妯
U+59AF.0	fCNS11643	1-4C71
U+59AF.0	fBig5	A96F
U+59AF.0	fGB	0-6628
U+59AF.0	fR/S	 38.5
U+59AF.0	fTotalStrokes	 8
U+59AF.0	fCangjie	VLW
U+59AF.0	fFourCorner	45460
U+59AF.0	fHakka	 cuk8; ch'uk8; ciu2; cu2; ciu1; chuk7; ciu3
U+59AF.0	fCantonese	zuk6; cau1; suk6
U+59AF.0	fMandarin	zhou2; chou1; zhu2; chou2; di2; lu4; zhu3
U+59AF.0	fEnglish	sister-in-law
U+59B0.0	fUTF8	妰
U+59B0.0	fCNS11643	3-2838
U+59B0.0	fGB	5-3F55
U+59B0.0	fR/S	 38.5
U+59B0.0	fTotalStrokes	 8
U+59B0.0	fCangjie	VOS
U+59B0.0	fFourCorner	48410
U+59B0.0	fCantonese	zoek3
U+59B0.0	fMandarin	zhuo2
U+59B1.0	fUTF8	妱
U+59B1.0	fCNS11643	2-2630
U+59B1.0	fBig5	CC4F
U+59B1.0	fGB	E-265C
U+59B1.0	fR/S	 38.5
U+59B1.0	fTotalStrokes	 8
U+59B1.0	fCangjie	VSHR
U+59B1.0	fFourCorner	47462
U+59B1.0	fCantonese	ziu1
U+59B1.0	fMandarin	zhao1
U+59B1.0	fEnglish	used in a female name
U+59B2.0	fUTF8	妲
U+59B2.0	fCNS11643	2-2629
U+59B2.0	fBig5	CC48
U+59B2.0	fGB	0-6627
U+59B2.0	fR/S	 38.5
U+59B2.0	fTotalStrokes	 8
U+59B2.0	fCangjie	VAM
U+59B2.0	fFourCorner	46410
U+59B2.0	fHakka	  dat7;  tan5; dan5
U+59B2.0	fCantonese	daat3; taan2 (also)
U+59B2.0	fMandarin	da2; dan4
U+59B2.0	fEnglish	[1] used in a female name [2] the concubine of Chou Hsin, the last ruler of the Shang dynasty
U+59B3.0	fUTF8	妳
U+59B3.0	fCNS11643	1-4C72
U+59B3.0	fBig5	A970
U+59B3.0	fGB	8-2D66
U+59B3.0	fR/S	 38.5
U+59B3.0	fTotalStrokes	 8
U+59B3.0	fCangjie	VNF
U+59B3.0	fFourCorner	47492
U+59B3.0	fHakka	 nai3; ni3; ngi2; nai1
U+59B3.0	fCantonese	naai5; nei5; laai5 (also); lei5 (also); naai4 (also); laai4 (also)
U+59B3.0	fMandarin	nai3; ni3; er3; ni4
U+59B3.0	fEnglish	[1] you (f.) [2] non-classical form of &#x5B2D;
U+59B4.0	fUTF8	妴
U+59B4.0	fCNS11643	2-2634
U+59B4.0	fBig5	CC53
U+59B4.0	fGB	3-3C3C
U+59B4.0	fR/S	 38.5
U+59B4.0	fTotalStrokes	 8
U+59B4.0	fCangjie	NUV
U+59B4.0	fFourCorner	27404
U+59B4.0	fHakka	 wan3
U+59B4.0	fCantonese	jyun2
U+59B4.0	fMandarin	yuan3; wan3; yuan1; yung4
U+59B5.0	fUTF8	妵
U+59B5.0	fCNS11643	2-2625
U+59B5.0	fBig5	CC44
U+59B5.0	fGB	3-3C3E
U+59B5.0	fR/S	 38.5
U+59B5.0	fTotalStrokes	 8
U+59B5.0	fCangjie	VYG
U+59B5.0	fFourCorner	40414
U+59B5.0	fCantonese	tau2
U+59B5.0	fMandarin	tou3
U+59B5.0	fEnglish	man
U+59B6.0	fUTF8	妶
U+59B6.0	fCNS11643	2-262C
U+59B6.0	fBig5	CC4B
U+59B6.0	fGB	3-3C3F
U+59B6.0	fR/S	 38.5
U+59B6.0	fTotalStrokes	 8
U+59B6.0	fCangjie	VYVI
U+59B6.0	fFourCorner	40432
U+59B6.0	fCantonese	jin4; jyun4
U+59B6.0	fMandarin	xuan2
U+59B7.0	fUTF8	妷
U+59B7.0	fCNS11643	3-2845
U+59B7.0	fGB	E-265D
U+59B7.0	fR/S	 38.5
U+59B7.0	fTotalStrokes	 8
U+59B7.0	fCangjie	VHQO
U+59B7.0	fFourCorner	45430
U+59B7.0	fCantonese	zat6
U+59B7.0	fMandarin	zhi4; zhi2; yi4
U+59B8.0	fUTF8	妸
U+59B8.0	fCNS11643	3-2840
U+59B8.0	fGB	5-3F50
U+59B8.0	fR/S	 38.5
U+59B8.0	fTotalStrokes	 8
U+59B8.0	fCangjie	VMNR
U+59B8.0	fFourCorner	41420
U+59B8.0	fHakka	  o1
U+59B8.0	fCantonese	ngo2 (also); ngo1 (also)
U+59B8.0	fMandarin	e1
U+59B8.0	fEnglish	person
U+59B9.0	fUTF8	妹
U+59B9.0	fCNS11643	1-4C68
U+59B9.0	fBig5	A966
U+59B9.0	fGB	0-4343
U+59B9.0	fR/S	 38.5
U+59B9.0	fTotalStrokes	 8
U+59B9.0	fCangjie	VJD
U+59B9.0	fFourCorner	45490
U+59B9.0	fHakka	  moi5; moe5
U+59B9.0	fCantonese	mui6; mui2 (also); mui4 (also); mui1
U+59B9.0	fMandarin	mei4
U+59B9.0	fEnglish	 younger sister
U+59B9.1	fUTF8	妹
U+59B9.1	fCNS11643	1-4C68
U+59B9.1	fBig5	A966
U+59B9.1	fGB	0-4343
U+59B9.1	fR/S	 38.5
U+59B9.1	fTotalStrokes	 8
U+59B9.1	fCangjie	VJD
U+59B9.1	fFourCorner	45490
U+59B9.1	fMandarin	mei5
U+59BA.0	fUTF8	妺
U+59BA.0	fCNS11643	2-2626
U+59BA.0	fBig5	CC45
U+59BA.0	fGB	3-3C31
U+59BA.0	fR/S	 38.5
U+59BA.0	fTotalStrokes	 8
U+59BA.0	fCangjie	VDJ
U+59BA.0	fFourCorner	45490
U+59BA.0	fHakka	 mat7; mat8
U+59BA.0	fCantonese	mut6
U+59BA.0	fMandarin	mo4
U+59BA.0	fEnglish	wife of the last ruler of the Xia dynasty
U+59BB.0	fUTF8	妻
U+59BB.0	fCNS11643	1-4C66
U+59BB.0	fBig5	A964
U+59BB.0	fGB	0-465E
U+59BB.0	fR/S	 38.5
U+59BB.0	fTotalStrokes	 8
U+59BB.0	fCangjie	JLV
U+59BB.0	fFourCorner	50404
U+59BB.0	fHakka	  ci1; ci5
U+59BB.0	fCantonese	cai1; cai3
U+59BB.0	fMandarin	qi1; ci1
U+59BB.0	fEnglish	one's formal or legal wife
U+59BB.1	fUTF8	妻
U+59BB.1	fCNS11643	1-4C66
U+59BB.1	fBig5	A964
U+59BB.1	fGB	0-465E
U+59BB.1	fR/S	 38.5
U+59BB.1	fTotalStrokes	 8
U+59BB.1	fCangjie	JLV
U+59BB.1	fFourCorner	50404
U+59BB.1	fMandarin	qi4
U+59BB.1	fEnglish	 marry one's daughter to someone
U+59BC.0	fUTF8	妼
U+59BC.0	fCNS11643	2-262D
U+59BC.0	fBig5	CC4C
U+59BC.0	fGB	3-3C42
U+59BC.0	fR/S	 38.5
U+59BC.0	fTotalStrokes	 8
U+59BC.0	fCangjie	VPH
U+59BC.0	fFourCorner	43400
U+59BC.0	fCantonese	bat6
U+59BC.0	fMandarin	bi4
U+59BD.0	fUTF8	妽
U+59BD.0	fCNS11643	2-2631
U+59BD.0	fBig5	CC50
U+59BD.0	fGB	E-265E
U+59BD.0	fR/S	 38.5
U+59BD.0	fTotalStrokes	 8
U+59BD.0	fCangjie	VLWL
U+59BD.0	fFourCorner	45406
U+59BD.0	fCantonese	san1
U+59BD.0	fMandarin	shen1
U+59BD.0	fEnglish	used in a female name
U+59BE.0	fUTF8	妾
U+59BE.0	fCNS11643	1-4C65
U+59BE.0	fBig5	A963
U+59BE.0	fGB	0-662A
U+59BE.0	fR/S	 38.5
U+59BE.0	fTotalStrokes	 8
U+59BE.0	fCangjie	YTV
U+59BE.0	fFourCorner	00404
U+59BE.0	fHakka	  ciap7; ziap7
U+59BE.0	fCantonese	cip3
U+59BE.0	fMandarin	qie4
U+59BE.0	fEnglish	[1] concubine [2] a polite term used by woman to refer to herself when speaking to her husband
U+59BF.0	fUTF8	妿
U+59BF.0	fCNS11643	3-283D
U+59BF.0	fGB	5-3F59
U+59BF.0	fR/S	  38.5
U+59BF.0	fTotalStrokes	 8
U+59BF.0	fCangjie	KRV
U+59BF.0	fFourCorner	46400
U+59BF.0	fCantonese	aa3; ngaa3 (also)
U+59BF.0	fMandarin	e1
U+59C0.0	fUTF8	姀
U+59C0.0	fCNS11643	2-2632
U+59C0.0	fBig5	CC51
U+59C0.0	fGB	5-3F56
U+59C0.0	fR/S	 38.5
U+59C0.0	fTotalStrokes	 8
U+59C0.0	fCangjie	VHD
U+59C0.0	fFourCorner	42494
U+59C0.0	fCantonese	wo4
U+59C0.0	fMandarin	he2
U+59C0.0	fEnglish	used in a female name
U+59C1.0	fUTF8	姁
U+59C1.0	fCNS11643	2-262B
U+59C1.0	fBig5	CC4A
U+59C1.0	fGB	3-3C3B
U+59C1.0	fR/S	 38.5
U+59C1.0	fTotalStrokes	 8
U+59C1.0	fCangjie	VPR
U+59C1.0	fFourCorner	47420
U+59C1.0	fHakka	 hi1; ki1
U+59C1.0	fCantonese	heoi1; heoi2
U+59C1.0	fMandarin	xu3; xu1; qu3; qu4; xu4
U+59C1.0	fEnglish	beauty
U+59C2.0	fUTF8	姂
U+59C2.0	fCNS11643	3-2837
U+59C2.0	fGB	3-3C2D
U+59C2.0	fR/S	  38.5
U+59C2.0	fTotalStrokes	 8
U+59C2.0	fCangjie	VHIO
U+59C2.0	fFourCorner	42430
U+59C2.0	fHakka	 fat8
U+59C2.0	fCantonese	fat6
U+59C2.0	fMandarin	fa2
U+59C2.0	fEnglish	handsome, ladylike
U+59C3.0	fUTF8	姃
U+59C3.0	fCNS11643	2-262E
U+59C3.0	fBig5	CC4D
U+59C3.0	fGB	3-3C32
U+59C3.0	fR/S	 38.5
U+59C3.0	fTotalStrokes	 8
U+59C3.0	fCangjie	VMYM
U+59C3.0	fFourCorner	41411
U+59C3.0	fHakka	 zhin1; zihn1
U+59C3.0	fCantonese	zing1
U+59C3.0	fMandarin	zheng1; zheng4
U+59C3.0	fEnglish	[1] name of a woman [2] correct deportment; modes demeanor
U+59C4.0	fUTF8	姄
U+59C4.0	fCNS11643	3-2835
U+59C4.0	fGB	E-265F
U+59C4.0	fR/S	 38.5
U+59C4.0	fTotalStrokes	 8
U+59C4.0	fCangjie	VRVP
U+59C4.0	fFourCorner	47440
U+59C4.0	fCantonese	man4
U+59C4.0	fMandarin	min2
U+59C4.0	fEnglish	used in a female name
U+59C5.0	fUTF8	姅
U+59C5.0	fCNS11643	1-4C74
U+59C5.0	fBig5	A972
U+59C5.0	fGB	3-3C40
U+59C5.0	fR/S	 38.5
U+59C5.0	fTotalStrokes	 8
U+59C5.0	fCangjie	VFQ
U+59C5.0	fFourCorner	49450
U+59C5.0	fHakka	 ban5; pan5
U+59C5.0	fCantonese	bun3; pun3 (also)
U+59C5.0	fMandarin	ban4
U+59C5.0	fEnglish	woman at time of menstruation
U+59C6.0	fUTF8	姆
U+59C6.0	fCNS11643	1-4C6B
U+59C6.0	fBig5	A969
U+59C6.0	fGB	0-4437
U+59C6.0	fR/S	 38.5
U+59C6.0	fTotalStrokes	 8
U+59C6.0	fCangjie	VWYI
U+59C6.0	fFourCorner	47450
U+59C6.0	fHakka	 me1;  mu1; mi1; mu3; men1
U+59C6.0	fCantonese	mou5
U+59C6.0	fMandarin	mu3; mou4
U+59C6.0	fEnglish	[1] governess; woman tutor [2] child's governess; matron who looks after small children
U+59C7.0	fUTF8	姇
U+59C7.0	fCNS11643	2-2635
U+59C7.0	fBig5	CC54
U+59C7.0	fGB	5-3F57
U+59C7.0	fR/S	 38.5
U+59C7.0	fTotalStrokes	 8
U+59C7.0	fCangjie	OIV
U+59C7.0	fFourCorner	24440
U+59C7.0	fCantonese	fu1; fau4
U+59C7.0	fMandarin	fu1; fu4
U+59C8.0	fUTF8	姈
U+59C8.0	fCNS11643	2-2633
U+59C8.0	fBig5	CC52
U+59C8.0	fGB	3-3C3A
U+59C8.0	fR/S	 38.5
U+59C8.0	fTotalStrokes	 8
U+59C8.0	fCangjie	VOII
U+59C8.0	fFourCorner	48427
U+59C8.0	fCantonese	ling4
U+59C8.0	fMandarin	ling2
U+59C8.0	fEnglish	used in a female name
U+59C9.0	fUTF8	姉
U+59C9.0	fCNS11643	3-283F
U+59C9.0	fGB	E-2660
U+59C9.0	fR/S	  38.5
U+59C9.0	fTotalStrokes	 8
U+59C9.0	fCangjie	VYLB
U+59C9.0	fFourCorner	40420
U+59C9.0	fHakka	  zi3;  ze2; zih3; ze3
U+59C9.0	fCantonese	zi2; ze2 (also)
U+59C9.0	fMandarin	jie3; zi3
U+59C9.0	fEnglish	elder sister
U+59CA.0	fUTF8	姊
U+59CA.0	fCNS11643	1-4C70
U+59CA.0	fBig5	A96E
U+59CA.0	fGB	0-6622
U+59CA.0	fR/S	 38.5
U+59CA.0	fTotalStrokes	 8
U+59CA.0	fCangjie	VLXH
U+59CA.0	fFourCorner	40427
U+59CA.0	fHakka	  zi3;  zih3; ze3; ze2
U+59CA.0	fCantonese	zi2; ze2 (also)
U+59CA.0	fMandarin	zi3; jie3
U+59CA.0	fEnglish	one's elder sister or sister
U+59CB.0	fUTF8	始
U+59CB.0	fCNS11643	1-4C6E
U+59CB.0	fBig5	A96C
U+59CB.0	fGB	0-4A3C
U+59CB.0	fR/S	 38.5
U+59CB.0	fTotalStrokes	 8
U+59CB.0	fCangjie	VIR
U+59CB.0	fFourCorner	43460
U+59CB.0	fHakka	 ci3; chi3; shi3; sii3; cih3; chii3
U+59CB.0	fCantonese	ci2
U+59CB.0	fMandarin	shi3; shi4
U+59CB.0	fEnglish	[1] beginning; start; first [2] begin; start; be the first [3] only then; [4] a Chinese family name
U+59CC.0	fUTF8	姌
U+59CC.0	fCNS11643	2-262A
U+59CC.0	fBig5	CC49
U+59CC.0	fGB	3-3C39
U+59CC.0	fR/S	 38.5
U+59CC.0	fTotalStrokes	 8
U+59CC.0	fCangjie	VGB
U+59CC.0	fFourCorner	45447
U+59CC.0	fCantonese	jim5
U+59CC.0	fMandarin	ran3
U+59CD.0	fUTF8	姍
U+59CD.0	fCNS11643	1-4C6D
U+59CD.0	fBig5	A96B
U+59CD.0	fGB	E-2661
U+59CD.0	fR/S	 38.5
U+59CD.0	fTotalStrokes	 8
U+59CD.0	fCangjie	VBT
U+59CD.0	fFourCorner	47440
U+59CD.0	fHakka	  san1
U+59CD.0	fCantonese	saan1; sin1
U+59CD.0	fMandarin	shan1; xian1; pan1; san1; shan4
U+59CD.0	fEnglish	[1] slander; proceed slowly; walk slowly (said of a woman) [2] ridicule; laugh at
U+59CE.0	fUTF8	姎
U+59CE.0	fCNS11643	2-2628
U+59CE.0	fBig5	CC47
U+59CE.0	fGB	3-3C38
U+59CE.0	fR/S	 38.5
U+59CE.0	fTotalStrokes	 8
U+59CE.0	fCangjie	VLBK
U+59CE.0	fFourCorner	45430
U+59CE.0	fCantonese	ong1; ngong1 (also); joeng1; joeng2
U+59CE.0	fMandarin	yang1; yang3; yang4
U+59CF.0	fUTF8	姏
U+59CF.0	fCNS11643	2-2627
U+59CF.0	fBig5	CC46
U+59CF.0	fGB	3-3C33
U+59CF.0	fR/S	 38.5
U+59CF.0	fTotalStrokes	 8
U+59CF.0	fCangjie	VTM
U+59CF.0	fFourCorner	44470
U+59CF.0	fCantonese	maan4
U+59CF.0	fMandarin	man2; qian2; gan1; man4
U+59D0.0	fUTF8	姐
U+59D0.0	fCNS11643	1-4C6C
U+59D0.0	fBig5	A96A
U+59D0.0	fGB	0-3D63
U+59D0.0	fR/S	 38.5
U+59D0.0	fTotalStrokes	 8
U+59D0.0	fCangjie	VBM
U+59D0.0	fFourCorner	47410
U+59D0.0	fHakka	  zia3; zi3; ze1; cia1
U+59D0.0	fCantonese	ze2; ze1 (also); ze4 (also)
U+59D0.0	fMandarin	jie3; ju1; ju4; zi3
U+59D0.0	fEnglish	[1] one's elder sister or sisters [2] a general term for young women; young lady
U+59D0.0	fUTF8	姐
U+59D0.0	fCNS11643	1-4C6C
U+59D0.0	fBig5	A96A
U+59D0.0	fGB	0-3D63
U+59D0.0	fR/S	 38.5
U+59D0.0	fTotalStrokes	 8
U+59D0.0	fCangjie	VBM
U+59D0.0	fFourCorner	47410
U+59D0.0	fHakka	  zia3; zi3; ze1; cia1
U+59D0.0	fCantonese	ze2; ze1 (also); ze4 (also)
U+59D0.0	fMandarin	jie3; ju1; ju4; zi3
U+59D0.0	fEnglish	[1] one's elder sister or sisters [2] a general term for young women; young lady
U+59D1.0	fUTF8	姑
U+59D1.0	fCNS11643	1-4C6A
U+59D1.0	fBig5	A968
U+59D1.0	fGB	0-3943
U+59D1.0	fR/S	 38.5
U+59D1.0	fTotalStrokes	 8
U+59D1.0	fCangjie	VJR
U+59D1.0	fFourCorner	44460
U+59D1.0	fHakka	  gu1
U+59D1.0	fCantonese	gu1; gwu1 (also)
U+59D1.0	fMandarin	gu1
U+59D1.0	fEnglish	[1] one's father's sister; aunts [2] one's husband's mother [3] the sister of one's husband [4] a general term for unmarried women [5] for the time being; meanwhile; for a while [6] nun
U+59D1.1	fUTF8	姑
U+59D1.1	fCNS11643	1-4C6A
U+59D1.1	fBig5	A968
U+59D1.1	fGB	0-3943
U+59D1.1	fR/S	 38.5
U+59D1.1	fTotalStrokes	 8
U+59D1.1	fCangjie	VJR
U+59D1.1	fFourCorner	44460
U+59D1.1	fMandarin	gu5
U+59D2.0	fUTF8	姒
U+59D2.0	fCNS11643	1-4C73
U+59D2.0	fBig5	A971
U+59D2.0	fGB	0-6626
U+59D2.0	fR/S	 38.5
U+59D2.0	fTotalStrokes	 8
U+59D2.0	fCangjie	VVIO
U+59D2.0	fFourCorner	48400
U+59D2.0	fHakka	 sii5; su5; se5
U+59D2.0	fCantonese	ci5
U+59D2.0	fMandarin	si4
U+59D2.0	fEnglish	[1] the wife of one's husband's elder brother [2] the elder of the twin sisters [3] a Chinese family name
U+59D3.0	fUTF8	姓
U+59D3.0	fCNS11643	1-4C6F
U+59D3.0	fBig5	A96D
U+59D3.0	fGB	0-5055
U+59D3.0	fR/S	 38.5
U+59D3.0	fTotalStrokes	 8
U+59D3.0	fCangjie	VHQM
U+59D3.0	fFourCorner	45410
U+59D3.0	fHakka	  siang5
U+59D3.0	fCantonese	sing3; seng3 (also)
U+59D3.0	fMandarin	xing4; sheng1
U+59D3.0	fEnglish	[1] one's family name; surname [2] clan; people; family
U+59D4.0	fUTF8	委
U+59D4.0	fCNS11643	1-4C67
U+59D4.0	fBig5	A965
U+59D4.0	fGB	0-4E2F
U+59D4.0	fR/S	 38.5
U+59D4.0	fTotalStrokes	 8
U+59D4.0	fCangjie	HDV
U+59D4.0	fFourCorner	20404
U+59D4.0	fHakka	  wui1; wi1; wui3; wi3; we6; we5; vui1; vui3
U+59D4.0	fCantonese	wai2; wai1
U+59D4.0	fMandarin	wei3; wei4
U+59D4.0	fEnglish	[1] depute; deputize [2] send; commission; put in charge of [3] give up; abandon [4] be frustrated; weakened; tired [5] really; truly; indeed [6] grievance; wrong [7] stoop; lower oneself (in order to avoid an open conflict, etc.)
U+59D4.1	fUTF8	委
U+59D4.1	fCNS11643	1-4C67
U+59D4.1	fBig5	A965
U+59D4.1	fGB	0-4E2F
U+59D4.1	fR/S	 38.5
U+59D4.1	fTotalStrokes	 8
U+59D4.1	fCangjie	HDV
U+59D4.1	fFourCorner	20404
U+59D4.1	fMandarin	wei1
U+59D4.1	fEnglish	used in combination
U+59D5.0	fUTF8	姕
U+59D5.0	fCNS11643	3-2C26
U+59D5.0	fGB	3-3C4B
U+59D5.0	fR/S	 38.5; 38.6
U+59D5.0	fTotalStrokes	 9; 8
U+59D5.0	fCangjie	YPV
U+59D5.0	fFourCorner	21400
U+59D5.0	fHakka	 zih1; zii1
U+59D5.0	fCantonese	zi1
U+59D5.0	fMandarin	zi1
U+59D5.0	fEnglish	 slatternly, worthless woman
U+59D6.0	fUTF8	姖
U+59D6.0	fCNS11643	2-262F
U+59D6.0	fBig5	CC4E
U+59D6.0	fGB	3-3C27
U+59D6.0	fR/S	 38.5
U+59D6.0	fTotalStrokes	 8
U+59D6.0	fCangjie	VSS
U+59D6.0	fFourCorner	41417
U+59D6.0	fCantonese	geoi6
U+59D6.0	fMandarin	ju4
U+59D7.0	fUTF8	姗
U+59D7.0	fCNS11643	3-2846
U+59D7.0	fGB	0-6629
U+59D7.0	fR/S	 38.5
U+59D7.0	fTotalStrokes	 8
U+59D7.0	fCangjie	VBBM
U+59D7.0	fFourCorner	47440
U+59D7.0	fHakka	 san1; san3
U+59D7.0	fCantonese	saan1
U+59D7.0	fMandarin	shan1; xian1; pan1; san1; shan4
U+59D7.0	fEnglish	slander; ridicule; proceed slowly
U+59D8.0	fUTF8	姘
U+59D8.0	fCNS11643	1-505B
U+59D8.0	fBig5	ABB9
U+59D8.0	fGB	0-6630
U+59D8.0	fR/S	 38.6
U+59D8.0	fTotalStrokes	 9
U+59D8.0	fCangjie	VTT
U+59D8.0	fFourCorner	48441
U+59D8.0	fHakka	  pin5; pin2
U+59D8.0	fCantonese	ping1
U+59D8.0	fMandarin	pin1; pin2
U+59D8.0	fEnglish	illicit sexual relations
U+59D9.0	fUTF8	姙
U+59D9.0	fCNS11643	3-2C2D
U+59D9.0	fGB	E-2662
U+59D9.0	fR/S	  38.6
U+59D9.0	fTotalStrokes	 9
U+59D9.0	fCangjie	VOHG
U+59D9.0	fFourCorner	42410
U+59D9.0	fHakka	 ngim3; ngim5; ngim2; ngim6; jim2
U+59D9.0	fCantonese	jam6; jam4 (also)
U+59D9.0	fMandarin	ren4; ren2
U+59D9.0	fEnglish	conceive; be pregnant
U+59DA.0	fUTF8	姚
U+59DA.0	fCNS11643	1-5062
U+59DA.0	fBig5	ABC0
U+59DA.0	fGB	0-5226
U+59DA.0	fR/S	 38.6
U+59DA.0	fTotalStrokes	 9
U+59DA.0	fCangjie	VLMO
U+59DA.0	fFourCorner	42413
U+59DA.0	fHakka	  jau2; rau2; jau3; jeu2
U+59DA.0	fCantonese	jiu4
U+59DA.0	fMandarin	yao2; tao2; tiao2; tiao4
U+59DA.0	fEnglish	[1] handsome; elegant; good-looking [2] a Chinese family name
U+59DB.0	fUTF8	姛
U+59DB.0	fCNS11643	2-2970
U+59DB.0	fBig5	CE6F
U+59DB.0	fGB	3-3C4D
U+59DB.0	fR/S	 38.6
U+59DB.0	fTotalStrokes	 9
U+59DB.0	fCangjie	VBMR
U+59DB.0	fFourCorner	47420
U+59DB.0	fHakka	 tung5; tung6
U+59DB.0	fCantonese	dung6
U+59DB.0	fMandarin	tong3; dong4
U+59DB.0	fEnglish	a graceful neck
U+59DC.0	fUTF8	姜
U+59DC.0	fCNS11643	1-505A
U+59DC.0	fBig5	ABB8
U+59DC.0	fGB	0-3D2A
U+59DC.0	fR/S	 38.6
U+59DC.0	fTotalStrokes	 9
U+59DC.0	fCangjie	TGV
U+59DC.0	fFourCorner	80404
U+59DC.0	fHakka	  giong1
U+59DC.0	fCantonese	goeng1
U+59DC.0	fMandarin	jiang1
U+59DC.0	fEnglish	[1] ginger [2] a Chinese family name
U+59DD.0	fUTF8	姝
U+59DD.0	fCNS11643	2-2968
U+59DD.0	fBig5	CE67
U+59DD.0	fGB	0-662D
U+59DD.0	fR/S	 38.6
U+59DD.0	fTotalStrokes	 9
U+59DD.0	fCangjie	VHJD
U+59DD.0	fFourCorner	45490
U+59DD.0	fHakka	 ki1; zu1; zhu1
U+59DD.0	fCantonese	syu1; zyu1 (also)
U+59DD.0	fMandarin	shu1; zhu1
U+59DD.0	fEnglish	 beautiful girl
U+59DE.0	fUTF8	姞
U+59DE.0	fCNS11643	2-2964
U+59DE.0	fBig5	CE63
U+59DE.0	fGB	3-3C46
U+59DE.0	fR/S	 38.6
U+59DE.0	fTotalStrokes	 9
U+59DE.0	fCangjie	VGR
U+59DE.0	fFourCorner	44461
U+59DE.0	fHakka	 giet7; get7; git7
U+59DE.0	fCantonese	gat1; git6
U+59DE.0	fMandarin	ji2
U+59DE.0	fEnglish	concubine
U+59DF.0	fUTF8	姟
U+59DF.0	fCNS11643	3-2C21
U+59DF.0	fGB	5-3F6F
U+59DF.0	fR/S	  38.6
U+59DF.0	fTotalStrokes	 9
U+59DF.0	fCangjie	VYVO
U+59DF.0	fFourCorner	40480
U+59DF.0	fCantonese	goi1
U+59DF.0	fMandarin	gai1
U+59E0.0	fUTF8	姠
U+59E0.0	fCNS11643	2-2974
U+59E0.0	fBig5	CE73
U+59E0.0	fGB	5-3F68
U+59E0.0	fR/S	 38.6
U+59E0.0	fTotalStrokes	 9
U+59E0.0	fCangjie	VHBR
U+59E0.0	fFourCorner	47420
U+59E0.0	fCantonese	hoeng3
U+59E0.0	fMandarin	shang4; xiang4
U+59E0.0	fEnglish	used in a girl's name
U+59E1.0	fUTF8	姡
U+59E1.0	fCNS11643	2-2963
U+59E1.0	fBig5	CE62
U+59E1.0	fGB	3-3C51
U+59E1.0	fR/S	 38.6
U+59E1.0	fTotalStrokes	 9
U+59E1.0	fCangjie	VHJR
U+59E1.0	fFourCorner	42464
U+59E1.0	fHakka	 gwat7
U+59E1.0	fCantonese	waat6; kut3; wut6
U+59E1.0	fMandarin	kuo4; huo2
U+59E2.0	fUTF8	姢
U+59E2.0	fCNS11643	3-2C2C
U+59E2.0	fGB	E-2663
U+59E2.0	fR/S	 38.6
U+59E2.0	fTotalStrokes	 9
U+59E2.0	fCangjie	VIB
U+59E2.0	fFourCorner	43420
U+59E2.0	fHakka	 gen1
U+59E2.0	fCantonese	gyun1
U+59E2.0	fMandarin	juan1; yuan1
U+59E2.0	fEnglish	beautiful, graceful
U+59E3.0	fUTF8	姣
U+59E3.0	fCNS11643	1-505D
U+59E3.0	fBig5	ABBB
U+59E3.0	fGB	0-662F
U+59E3.0	fR/S	 38.6
U+59E3.0	fTotalStrokes	 9
U+59E3.0	fCangjie	VYCK
U+59E3.0	fFourCorner	40448
U+59E3.0	fHakka	  gau3; giau1; hau2; hiau2
U+59E3.0	fCantonese	gaau2; haau4
U+59E3.0	fMandarin	jiao1; jiao3; xiao2; xiao4
U+59E3.0	fEnglish	[1] beautiful; handsome; pretty [2] coquettish [3] lewd; unchaste; morally loose
U+59E4.0	fUTF8	姤
U+59E4.0	fCNS11643	2-296D
U+59E4.0	fBig5	CE6C
U+59E4.0	fGB	3-3C54
U+59E4.0	fR/S	 38.6
U+59E4.0	fTotalStrokes	 9
U+59E4.0	fCangjie	VHMR
U+59E4.0	fFourCorner	42461
U+59E4.0	fHakka	  geu5; heu6; heu5
U+59E4.0	fCantonese	gau3
U+59E4.0	fMandarin	gou4; hou4
U+59E4.0	fEnglish	[1] pair off; mate; copulate [2] good; excellent [3] one of the eight diagrams in the Book of Changes
U+59E5.0	fUTF8	姥
U+59E5.0	fCNS11643	1-5060
U+59E5.0	fBig5	ABBE
U+59E5.0	fGB	0-4051
U+59E5.0	fR/S	 38.6
U+59E5.0	fTotalStrokes	 9
U+59E5.0	fCangjie	VJKP
U+59E5.0	fFourCorner	44411
U+59E5.0	fHakka	  mu3; lau3; lo3
U+59E5.0	fCantonese	mou5; lou5
U+59E5.0	fMandarin	lao3; mu3
U+59E5.0	fEnglish	[1] one's maternal grandmother [2] old woman
U+59E6.0	fUTF8	姦
U+59E6.0	fCNS11643	1-5063
U+59E6.0	fBig5	ABC1
U+59E6.0	fGB	E-2664
U+59E6.0	fR/S	 38.6
U+59E6.0	fTotalStrokes	 9
U+59E6.0	fCangjie	VVV
U+59E6.0	fFourCorner	40444
U+59E6.0	fHakka	 gan1;  gien1; kian1; gian1
U+59E6.0	fCantonese	gaan1
U+59E6.0	fMandarin	jian1
U+59E6.0	fEnglish	[1] adultery; debauchery; licentiousness [2] debauch; ravish; attack sexually (a woman) [3] crook
U+59E7.0	fUTF8	姧
U+59E7.0	fCNS11643	3-2C2E
U+59E7.0	fGB	5-3549
U+59E7.0	fR/S	 38.6
U+59E7.0	fTotalStrokes	 9
U+59E7.0	fCangjie	VVMJ
U+59E7.0	fFourCorner	41440
U+59E7.0	fCantonese	gaan1
U+59E7.0	fMandarin	jian1
U+59E8.0	fUTF8	姨
U+59E8.0	fCNS11643	1-505E
U+59E8.0	fBig5	ABBC
U+59E8.0	fGB	0-524C
U+59E8.0	fR/S	 38.6
U+59E8.0	fTotalStrokes	 9
U+59E8.0	fCangjie	VKN
U+59E8.0	fFourCorner	45432
U+59E8.0	fHakka	 ji2; ji3; ri2; i2
U+59E8.0	fCantonese	ji4; ji1
U+59E8.0	fMandarin	yi2
U+59E8.0	fEnglish	[1] one's mother's sister [2] one's wife's sister [3] concubine
U+59E9.0	fUTF8	姩
U+59E9.0	fCNS11643	2-2971
U+59E9.0	fBig5	CE70
U+59E9.0	fGB	3-3C4E
U+59E9.0	fR/S	 38.6
U+59E9.0	fTotalStrokes	 9
U+59E9.0	fCangjie	VOQ
U+59E9.0	fFourCorner	48450
U+59E9.0	fHakka	 len5
U+59E9.0	fCantonese	nin6; lin6 (also); nin4; nin1
U+59E9.0	fMandarin	nian4; nian2
U+59E9.0	fEnglish	[1] beautiful girl [2] used in a female name [3] milk
U+59EA.0	fUTF8	姪
U+59EA.0	fCNS11643	1-5061
U+59EA.0	fBig5	ABBF
U+59EA.0	fGB	E-2665
U+59EA.0	fR/S	 38.6
U+59EA.0	fTotalStrokes	 9
U+59EA.0	fCangjie	VMIG
U+59EA.0	fFourCorner	41414
U+59EA.0	fHakka	  chit8;  cit8; ts'iit8
U+59EA.0	fCantonese	zat6; zat2 (also)
U+59EA.0	fMandarin	zhi2; die2
U+59EA.0	fEnglish	[1] the children of one's brother; nephews or nieces [2] I; me (when speaking to a family friend of one's father generation
U+59EB.0	fUTF8	姫
U+59EB.0	fCNS11643	3-2C23
U+59EB.0	fGB	5-3F5F
U+59EB.0	fR/S	  38.6
U+59EB.0	fTotalStrokes	 9
U+59EB.0	fCangjie	VSLL
U+59EB.0	fFourCorner	41410
U+59EB.0	fCantonese	can2
U+59EB.0	fMandarin	zhen3; ji1
U+59EB.0	fEnglish	 beauty imperial concubine
U+59EC.0	fUTF8	姬
U+59EC.0	fCNS11643	1-5478
U+59EC.0	fBig5	AE56
U+59EC.0	fGB	0-3C27
U+59EC.0	fR/S	 38.7
U+59EC.0	fTotalStrokes	 10
U+59EC.0	fCangjie	VSLL
U+59EC.0	fFourCorner	41417
U+59EC.0	fHakka	  hi1; ki1; gi1
U+59EC.0	fCantonese	gei1
U+59EC.0	fMandarin	ji1
U+59EC.0	fEnglish	[1] beautiful lady; charming girl [2] imperial concubine [3] a Chinese family name
U+59ED.0	fUTF8	姭
U+59ED.0	fCNS11643	2-2977
U+59ED.0	fBig5	CE76
U+59ED.0	fGB	E-2666
U+59ED.0	fR/S	 38.6
U+59ED.0	fTotalStrokes	 9
U+59ED.0	fCangjie	KSKSV
U+59ED.0	fFourCorner	47404
U+59ED.0	fCantonese	him3
U+59ED.0	fMandarin	xian4
U+59EE.0	fUTF8	姮
U+59EE.0	fCNS11643	2-2965
U+59EE.0	fBig5	CE64
U+59EE.0	fGB	3-3C47
U+59EE.0	fR/S	 38.6
U+59EE.0	fTotalStrokes	 9
U+59EE.0	fCangjie	VMAM
U+59EE.0	fFourCorner	41416
U+59EE.0	fHakka	 hen2; shong2; song2
U+59EE.0	fCantonese	hang4
U+59EE.0	fMandarin	heng2
U+59EE.0	fEnglish	[1] used in a female name [2] female name
U+59EF.0	fUTF8	姯
U+59EF.0	fCNS11643	3-2C24
U+59EF.0	fGB	5-3F65
U+59EF.0	fR/S	 38.6
U+59EF.0	fTotalStrokes	 9
U+59EF.0	fCangjie	VFMU
U+59EF.0	fFourCorner	49410
U+59EF.0	fCantonese	gwong1; gong1 (also)
U+59EF.0	fMandarin	guang1
U+59EF.0	fEnglish	used in a female name
U+59F0.0	fUTF8	姰
U+59F0.0	fCNS11643	3-2C25
U+59F0.0	fGB	3-3C58
U+59F0.0	fR/S	 38.6
U+59F0.0	fTotalStrokes	 9
U+59F0.0	fCangjie	VPA
U+59F0.0	fFourCorner	47420
U+59F0.0	fCantonese	gwan1
U+59F0.0	fMandarin	jun1
U+59F1.0	fUTF8	姱
U+59F1.0	fCNS11643	2-2967
U+59F1.0	fBig5	CE66
U+59F1.0	fGB	3-3C49
U+59F1.0	fR/S	 38.6
U+59F1.0	fTotalStrokes	 9
U+59F1.0	fCangjie	VKMS
U+59F1.0	fFourCorner	44427
U+59F1.0	fHakka	 kwa1; kwa5
U+59F1.0	fCantonese	kwaa1
U+59F1.0	fMandarin	kua1; hu4; xu1
U+59F1.0	fEnglish	pretty; beautiful; handsome; elegant; good-looking; fascinating
U+59F2.0	fUTF8	姲
U+59F2.0	fCNS11643	2-296E
U+59F2.0	fBig5	CE6D
U+59F2.0	fGB	5-3F72
U+59F2.0	fR/S	 38.6
U+59F2.0	fTotalStrokes	 9
U+59F2.0	fCangjie	VJV
U+59F2.0	fFourCorner	43442
U+59F2.0	fCantonese	aan3; ngaan3 (also)
U+59F2.0	fMandarin	yan4
U+59F2.0	fEnglish	used in a female name
U+59F3.0	fUTF8	姳
U+59F3.0	fCNS11643	2-2972
U+59F3.0	fBig5	CE71
U+59F3.0	fGB	5-3F6C
U+59F3.0	fR/S	 38.6
U+59F3.0	fTotalStrokes	 9
U+59F3.0	fCangjie	VNIR
U+59F3.0	fFourCorner	47462
U+59F3.0	fCantonese	ming5
U+59F3.0	fMandarin	ming3
U+59F4.0	fUTF8	姴
U+59F4.0	fCNS11643	2-2976
U+59F4.0	fBig5	CE75
U+59F4.0	fGB	5-3F61
U+59F4.0	fR/S	 38.6
U+59F4.0	fTotalStrokes	 9
U+59F4.0	fCangjie	MNV
U+59F4.0	fFourCorner	12404
U+59F4.0	fCantonese	lit6
U+59F4.0	fMandarin	lie4
U+59F5.0	fUTF8	姵
U+59F5.0	fCNS11643	2-2973
U+59F5.0	fBig5	CE72
U+59F5.0	fGB	5-3F6B
U+59F5.0	fR/S	 38.6
U+59F5.0	fTotalStrokes	 9
U+59F5.0	fCangjie	VHNB
U+59F5.0	fFourCorner	47410
U+59F5.0	fCantonese	bui6; pui3
U+59F5.0	fMandarin	pei4
U+59F5.0	fEnglish	female name
U+59F6.0	fUTF8	姶
U+59F6.0	fCNS11643	2-296C
U+59F6.0	fBig5	CE6B
U+59F6.0	fGB	3-3C56
U+59F6.0	fR/S	 38.6
U+59F6.0	fTotalStrokes	 9
U+59F6.0	fCangjie	VOMR
U+59F6.0	fFourCorner	48461
U+59F6.0	fHakka	 ak7; rap7; jap7; ap7; hap8
U+59F6.0	fCantonese	ap1; ngap1 (also); hap6; jap1; haap6
U+59F6.0	fMandarin	yan3
U+59F6.0	fEnglish	 beautiful woman
U+59F7.0	fUTF8	姷
U+59F7.0	fCNS11643	2-296F
U+59F7.0	fBig5	CE6E
U+59F7.0	fGB	5-3F60
U+59F7.0	fR/S	 38.6
U+59F7.0	fTotalStrokes	 9
U+59F7.0	fCangjie	VKB
U+59F7.0	fFourCorner	44427
U+59F7.0	fHakka	 riu3; ju3; ju5
U+59F7.0	fCantonese	jau6; jau5
U+59F7.0	fMandarin	you4
U+59F7.0	fEnglish	[1] assist [2] pair; a couple
U+59F8.0	fUTF8	姸
U+59F8.0	fCNS11643	3-2C2B
U+59F8.0	fGB	E-2667
U+59F8.0	fR/S	 38.6
U+59F8.0	fTotalStrokes	 9
U+59F8.0	fCangjie	VMJJ
U+59F8.0	fFourCorner	41440
U+59F8.0	fHakka	 ngien2; ngan2
U+59F8.0	fCantonese	jin4
U+59F8.0	fMandarin	yan2
U+59F8.0	fEnglish	[1] beautiful; pretty; cute; attractive; charming [2] seductive; coquettish
U+59F9.0	fUTF8	姹
U+59F9.0	fCNS11643	3-2C29
U+59F9.0	fGB	0-6631
U+59F9.0	fR/S	  38.6
U+59F9.0	fTotalStrokes	 9
U+59F9.0	fCangjie	VJHP
U+59F9.0	fFourCorner	43410
U+59F9.0	fHakka	  za5; ca5
U+59F9.0	fCantonese	caa3
U+59F9.0	fMandarin	cha4
U+59F9.0	fEnglish	[1] charming; attractive; seductive [2] young (girl) [3] boast; talk big; lie
U+59FA.0	fUTF8	姺
U+59FA.0	fCNS11643	2-2969
U+59FA.0	fBig5	CE68
U+59FA.0	fGB	3-3C4F
U+59FA.0	fR/S	 38.6
U+59FA.0	fTotalStrokes	 9
U+59FA.0	fCangjie	VHGU
U+59FA.0	fFourCorner	44411
U+59FA.0	fHakka	 sin1
U+59FA.0	fCantonese	sin2 (also); san1
U+59FA.0	fMandarin	xian3; xian1; shen1; sun3; xi3; xin1
U+59FA.0	fEnglish	name of an ancient state
U+59FB.0	fUTF8	姻
U+59FB.0	fCNS11643	1-5065
U+59FB.0	fBig5	ABC3
U+59FB.0	fGB	0-5276
U+59FB.0	fR/S	 38.6
U+59FB.0	fTotalStrokes	 9
U+59FB.0	fCangjie	VWK
U+59FB.0	fFourCorner	46400
U+59FB.0	fHakka	  jen1; jin1; rin1; jan1; ren1
U+59FB.0	fCantonese	jan1
U+59FB.0	fMandarin	yin1
U+59FB.0	fEnglish	[1] relations or connections through marriage; relatives by marriage [2] one's husband family [3] marriage
U+59FC.0	fUTF8	姼
U+59FC.0	fCNS11643	2-296B
U+59FC.0	fBig5	CE6A
U+59FC.0	fGB	3-3C59
U+59FC.0	fR/S	 38.6
U+59FC.0	fTotalStrokes	 9
U+59FC.0	fCangjie	VNIN
U+59FC.0	fFourCorner	47427
U+59FC.0	fHakka	 ti1;  chi3; cih3; chii3
U+59FC.0	fCantonese	ci2; si4 (also)
U+59FC.0	fMandarin	chi3; duo1; ji4; shi2; shi4; ti2
U+59FC.0	fEnglish	 pretty but wanton woman
U+59FD.0	fUTF8	姽
U+59FD.0	fCNS11643	2-296A
U+59FD.0	fBig5	CE69
U+59FD.0	fGB	3-3C57
U+59FD.0	fR/S	 38.6
U+59FD.0	fTotalStrokes	 9
U+59FD.0	fCangjie	VNMU
U+59FD.0	fFourCorner	47412
U+59FD.0	fCantonese	gwai2; ngai5
U+59FD.0	fMandarin	gui3
U+59FD.0	fEnglish	good
U+59FE.0	fUTF8	姾
U+59FE.0	fCNS11643	2-2975
U+59FE.0	fBig5	CE74
U+59FE.0	fGB	E-2668
U+59FE.0	fR/S	 38.6
U+59FE.0	fTotalStrokes	 9
U+59FE.0	fCangjie	VOMG
U+59FE.0	fFourCorner	48414
U+59FE.0	fCantonese	cyun4
U+59FE.0	fMandarin	quan1; quan2
U+59FE.0	fEnglish	used in a female name
U+59FF.0	fUTF8	姿
U+59FF.0	fCNS11643	1-505C
U+59FF.0	fBig5	ABBA
U+59FF.0	fGB	0-574B
U+59FF.0	fR/S	 38.6
U+59FF.0	fTotalStrokes	 9
U+59FF.0	fCangjie	IOV
U+59FF.0	fFourCorner	37404
U+59FF.0	fHakka	 zih1; zu1; zii1
U+59FF.0	fCantonese	zi1
U+59FF.0	fMandarin	zi1; zi4
U+59FF.0	fEnglish	[1] one's manner; an air; carriage; bearing [2] looks
U+5A00.0	fUTF8	娀
U+5A00.0	fCNS11643	2-2966
U+5A00.0	fBig5	CE65
U+5A00.0	fGB	3-3C45
U+5A00.0	fR/S	 38.6
U+5A00.0	fTotalStrokes	 9
U+5A00.0	fCangjie	VIJ
U+5A00.0	fFourCorner	43450
U+5A00.0	fHakka	 jung1; rung1
U+5A00.0	fCantonese	sung1
U+5A00.0	fMandarin	song1
U+5A00.0	fEnglish	name of a concubine of Di Ku; father of the mythical Yao
U+5A01.0	fUTF8	威
U+5A01.0	fCNS11643	1-5064
U+5A01.0	fBig5	ABC2
U+5A01.0	fGB	0-4D7E
U+5A01.0	fR/S	 38.6
U+5A01.0	fTotalStrokes	 9
U+5A01.0	fCangjie	IHMV
U+5A01.0	fFourCorner	53200
U+5A01.0	fHakka	  wui1; wi1; vui1
U+5A01.0	fCantonese	wai1
U+5A01.0	fMandarin	wei1
U+5A01.0	fEnglish	[1] dignity; majesty [2] authority; might; power; powerful; dominate [3] awe; awe-inspiring [4] the mother of one's husband [5] a Chinese family name
U+5A02.0	fUTF8	娂
U+5A02.0	fCNS11643	3-2C2A
U+5A02.0	fGB	5-3F5E
U+5A02.0	fR/S	 38.6
U+5A02.0	fTotalStrokes	 9
U+5A02.0	fCangjie	VTC
U+5A02.0	fFourCorner	44480
U+5A02.0	fCantonese	hung4
U+5A02.0	fMandarin	hong2
U+5A02.0	fEnglish	same as &#x5985;
U+5A03.0	fUTF8	娃
U+5A03.0	fCNS11643	1-505F
U+5A03.0	fBig5	ABBD
U+5A03.0	fGB	0-4D5E
U+5A03.0	fR/S	 38.6
U+5A03.0	fTotalStrokes	 9
U+5A03.0	fCangjie	VGG
U+5A03.0	fFourCorner	44414
U+5A03.0	fHakka	 wa1; va1; nga2
U+5A03.0	fCantonese	waa1
U+5A03.0	fMandarin	wa2; wa1
U+5A03.0	fEnglish	[1] baby; child [2] doll [3] a beautiful woman; pretty girl [4] exquisite; fine
U+5A03.1	fUTF8	娃
U+5A03.1	fCNS11643	1-505F
U+5A03.1	fBig5	ABBD
U+5A03.1	fGB	0-4D5E
U+5A03.1	fR/S	 38.6
U+5A03.1	fTotalStrokes	 9
U+5A03.1	fCangjie	VGG
U+5A03.1	fFourCorner	44414
U+5A03.1	fMandarin	wa5
U+5A04.0	fUTF8	娄
U+5A04.0	fCNS11643	3-2C28
U+5A04.0	fGB	0-4226
U+5A04.0	fR/S	 38.6; 119.3
U+5A04.0	fTotalStrokes	 9
U+5A04.0	fCangjie	FDV
U+5A04.0	fFourCorner	90400
U+5A04.0	fHakka	 leu2
U+5A04.0	fCantonese	lau4; leoi5
U+5A04.0	fMandarin	lou2; lei2; lu3
U+5A04.0	fEnglish	simplified form of &#x5A41;
U+5A05.0	fUTF8	娅
U+5A05.0	fGB	0-662B
U+5A05.0	fR/S	 38.6
U+5A05.0	fTotalStrokes	 9
U+5A05.0	fCangjie	VMTC
U+5A05.0	fHakka	 a5
U+5A05.0	fCantonese	aa3; ngaa3 (also)
U+5A05.0	fMandarin	ya4
U+5A05.0	fEnglish	simplified form of &#x5A6D;
U+5A06.0	fUTF8	娆
U+5A06.0	fGB	0-662C
U+5A06.0	fR/S	 38.6
U+5A06.0	fTotalStrokes	 9
U+5A06.0	fCangjie	VJPU
U+5A06.0	fHakka	 ngiau2
U+5A06.0	fCantonese	jiu4; jiu2; niu5
U+5A06.0	fMandarin	rao3; rao2; nao2; nao4; niao3; xiao4; yao3
U+5A06.0	fEnglish	simplified form of &#x5B08;
U+5A07.0	fUTF8	娇
U+5A07.0	fGB	0-3D3F
U+5A07.0	fR/S	 38.6
U+5A07.0	fTotalStrokes	 9
U+5A07.0	fCangjie	VHKL
U+5A07.0	fHakka	 giau1
U+5A07.0	fCantonese	giu1
U+5A07.0	fMandarin	jiao1; jiao3; qiao2
U+5A07.0	fEnglish	simplified form of &#x5B0C;
U+5A08.0	fUTF8	娈
U+5A08.0	fGB	0-662E
U+5A08.0	fR/S	 38.6
U+5A08.0	fTotalStrokes	 9
U+5A08.0	fCangjie	YCV
U+5A08.0	fHakka	 lan2
U+5A08.0	fCantonese	lyun5 (also); lyun2
U+5A08.0	fMandarin	lian4; luan2; luan3; luan4
U+5A08.0	fEnglish	simplified form of &#x5B4C;
U+5A09.0	fUTF8	娉
U+5A09.0	fCNS11643	1-547E
U+5A09.0	fBig5	AE5C
U+5A09.0	fGB	0-6633
U+5A09.0	fR/S	 38.7
U+5A09.0	fTotalStrokes	 10
U+5A09.0	fCangjie	VLWS
U+5A09.0	fFourCorner	45427
U+5A09.0	fHakka	  pin5; pin3
U+5A09.0	fCantonese	ping1; ping3
U+5A09.0	fMandarin	ping1; ping4
U+5A09.0	fEnglish	beautiful; attractive; charming; graceful; good-looking
U+5A0A.0	fUTF8	娊
U+5A0A.0	fCNS11643	2-2E64
U+5A0A.0	fBig5	D162
U+5A0A.0	fGB	5-3F47
U+5A0A.0	fR/S	 38.7
U+5A0A.0	fTotalStrokes	 10
U+5A0A.0	fCangjie	VBUU
U+5A0A.0	fFourCorner	46410
U+5A0A.0	fHakka	  hien5; hien3
U+5A0A.0	fCantonese	jin6; daam1
U+5A0A.0	fMandarin	xian4
U+5A0A.0	fEnglish	graceful woman with a small waist
U+5A0B.0	fUTF8	娋
U+5A0B.0	fCNS11643	3-3040
U+5A0B.0	fGB	5-3F78
U+5A0B.0	fR/S	 38.7
U+5A0B.0	fTotalStrokes	 10
U+5A0B.0	fCangjie	VFB
U+5A0B.0	fFourCorner	49420
U+5A0B.0	fHakka	  sau1
U+5A0B.0	fCantonese	saau3
U+5A0B.0	fMandarin	shao4
U+5A0B.0	fEnglish	the eldest sister
U+5A0C.0	fUTF8	娌
U+5A0C.0	fCNS11643	1-547D
U+5A0C.0	fBig5	AE5B
U+5A0C.0	fGB	0-6632
U+5A0C.0	fR/S	 38.7
U+5A0C.0	fTotalStrokes	 10
U+5A0C.0	fCangjie	VWG
U+5A0C.0	fFourCorner	46414
U+5A0C.0	fHakka	 li3
U+5A0C.0	fCantonese	lei5
U+5A0C.0	fMandarin	li3
U+5A0C.0	fEnglish	[1] sisters-in-law [2] he; she
U+5A0D.0	fUTF8	娍
U+5A0D.0	fCNS11643	3-2C27
U+5A0D.0	fGB	3-3C4A
U+5A0D.0	fR/S	 38.7; 38.6
U+5A0D.0	fTotalStrokes	 9; 10
U+5A0D.0	fCangjie	VIHS
U+5A0D.0	fFourCorner	43450
U+5A0D.0	fCantonese	sing4
U+5A0D.0	fMandarin	cheng2
U+5A0E.0	fUTF8	娎
U+5A0E.0	fCNS11643	4-2C40
U+5A0E.0	fGB	3-3C64
U+5A0E.0	fR/S	 38.7
U+5A0E.0	fTotalStrokes	 10
U+5A0E.0	fCangjie	QLV
U+5A0E.0	fFourCorner	52400
U+5A0E.0	fCantonese	hit3
U+5A0E.0	fMandarin	xiao4; xie4; xie1
U+5A0F.0	fUTF8	娏
U+5A0F.0	fCNS11643	2-2E62
U+5A0F.0	fBig5	D160
U+5A0F.0	fGB	E-2669
U+5A0F.0	fR/S	 38.7
U+5A0F.0	fTotalStrokes	 10
U+5A0F.0	fCangjie	VIUH
U+5A0F.0	fFourCorner	43412
U+5A0F.0	fCantonese	mong4
U+5A0F.0	fMandarin	mang2
U+5A0F.0	fEnglish	name of a female spirit
U+5A10.0	fUTF8	娐
U+5A10.0	fCNS11643	4-2C3E
U+5A10.0	fGB	3-3C68
U+5A10.0	fR/S	  38.7
U+5A10.0	fTotalStrokes	 10
U+5A10.0	fCangjie	VBND
U+5A10.0	fFourCorner	42440
U+5A10.0	fCantonese	fu1
U+5A10.0	fMandarin	fu1
U+5A11.0	fUTF8	娑
U+5A11.0	fCNS11643	1-5472
U+5A11.0	fBig5	AE50
U+5A11.0	fGB	0-6636
U+5A11.0	fR/S	 38.7
U+5A11.0	fTotalStrokes	 10
U+5A11.0	fCangjie	EHV
U+5A11.0	fFourCorner	39404
U+5A11.0	fHakka	 sa1;  so1
U+5A11.0	fCantonese	so1
U+5A11.0	fMandarin	suo1; suo3; suo4
U+5A11.0	fEnglish	 dance; frolic; lounge; saunter
U+5A12.0	fUTF8	娒
U+5A12.0	fCNS11643	3-304F
U+5A12.0	fGB	5-3F7B
U+5A12.0	fR/S	 38.7
U+5A12.0	fTotalStrokes	 10
U+5A12.0	fCangjie	VOWY
U+5A12.0	fFourCorner	48450
U+5A12.0	fHakka	 me1; mi1; men1
U+5A12.0	fCantonese	mou5; mui4
U+5A12.0	fMandarin	mu3; wu3; mou4
U+5A12.0	fEnglish	same as &#x59C6;
U+5A13.0	fUTF8	娓
U+5A13.0	fCNS11643	1-5477
U+5A13.0	fBig5	AE55
U+5A13.0	fGB	0-6638
U+5A13.0	fR/S	 38.7
U+5A13.0	fTotalStrokes	 10
U+5A13.0	fCangjie	VSHU
U+5A13.0	fFourCorner	47414
U+5A13.0	fHakka	 mui3; mi3; mui1
U+5A13.0	fCantonese	mei5
U+5A13.0	fMandarin	wei3; hui4; mei2; mei4
U+5A13.0	fEnglish	[1] complying; subservient; agreeable [2] beautiful; attractive; pleasant [3] diligent and tireless
U+5A14.0	fUTF8	娔
U+5A14.0	fCNS11643	3-304A
U+5A14.0	fGB	3-3C5C
U+5A14.0	fR/S	  38.7
U+5A14.0	fTotalStrokes	 10
U+5A14.0	fCangjie	VJRU
U+5A14.0	fFourCorner	44410
U+5A14.0	fMandarin	ke4
U+5A14.0	fEnglish	same as &#x217A2;
U+5A15.0	fUTF8	娕
U+5A15.0	fCNS11643	2-2E61
U+5A15.0	fBig5	D15F
U+5A15.0	fGB	3-3C5E
U+5A15.0	fR/S	 38.7
U+5A15.0	fTotalStrokes	 10
U+5A15.0	fCangjie	VDL
U+5A15.0	fFourCorner	45496
U+5A15.0	fHakka	  cok7
U+5A15.0	fCantonese	cuk1; laai6; cok3
U+5A15.0	fMandarin	lai4
U+5A15.0	fEnglish	attentive; cautious; respectful
U+5A16.0	fUTF8	娖
U+5A16.0	fCNS11643	2-2E5E
U+5A16.0	fBig5	D15C
U+5A16.0	fGB	3-3C65
U+5A16.0	fR/S	 38.7
U+5A16.0	fTotalStrokes	 10
U+5A16.0	fCangjie	VRYO
U+5A16.0	fFourCorner	46481
U+5A16.0	fHakka	  cok7;  cuk7
U+5A16.0	fCantonese	cuk1 (also); cok3; coek3 (also); zuk6
U+5A16.0	fMandarin	chuo4
U+5A16.0	fEnglish	attentive; cautious; respectful
U+5A17.0	fUTF8	娗
U+5A17.0	fCNS11643	2-2E63
U+5A17.0	fBig5	D161
U+5A17.0	fGB	3-3C50
U+5A17.0	fR/S	 38.7
U+5A17.0	fTotalStrokes	 10
U+5A17.0	fCangjie	VNKG
U+5A17.0	fFourCorner	42441
U+5A17.0	fHakka	 tin2
U+5A17.0	fCantonese	ting5; tin2; ting4
U+5A17.0	fMandarin	ding4; tian3; ting2
U+5A17.0	fEnglish	used in a female name
U+5A18.0	fUTF8	娘
U+5A18.0	fCNS11643	1-5473
U+5A18.0	fBig5	AE51
U+5A18.0	fGB	0-446F
U+5A18.0	fR/S	 38.7
U+5A18.0	fTotalStrokes	 10
U+5A18.0	fCangjie	VIAV
U+5A18.0	fFourCorner	43432
U+5A18.0	fHakka	  ngiong2; niong2; gniong3
U+5A18.0	fCantonese	noeng4; loeng4 (also); noeng2 (also); loeng2 (also)
U+5A18.0	fMandarin	niang2
U+5A18.0	fEnglish	[1] mother [2] young girl; women; wife; girls
U+5A18.1	fUTF8	娘
U+5A18.1	fCNS11643	1-5473
U+5A18.1	fBig5	AE51
U+5A18.1	fGB	0-446F
U+5A18.1	fR/S	 38.7
U+5A18.1	fTotalStrokes	 10
U+5A18.1	fCangjie	VIAV
U+5A18.1	fFourCorner	43432
U+5A18.1	fMandarin	niang5
U+5A19.0	fUTF8	娙
U+5A19.0	fCNS11643	2-2E5D
U+5A19.0	fBig5	D15B
U+5A19.0	fGB	3-3C43
U+5A19.0	fR/S	 38.7
U+5A19.0	fTotalStrokes	 10
U+5A19.0	fCangjie	VMVM
U+5A19.0	fFourCorner	41411
U+5A19.0	fHakka	 gang5; gin1
U+5A19.0	fCantonese	jing4
U+5A19.0	fMandarin	xing2; keng1; ying2
U+5A19.0	fEnglish	handsome woman
U+5A1A.0	fUTF8	娚
U+5A1A.0	fCNS11643	3-304E
U+5A1A.0	fGB	E-266A
U+5A1A.0	fR/S	  38.7
U+5A1A.0	fTotalStrokes	 10
U+5A1A.0	fCangjie	VWKS
U+5A1A.0	fFourCorner	46420
U+5A1A.0	fHakka	 nam2
U+5A1A.0	fCantonese	naam4; laam4 (also)
U+5A1A.0	fMandarin	nan2
U+5A1B.0	fUTF8	娛
U+5A1B.0	fCNS11643	1-5476
U+5A1B.0	fBig5	AE54
U+5A1B.0	fGB	E-266B
U+5A1B.0	fR/S	 38.7
U+5A1B.0	fTotalStrokes	 10
U+5A1B.0	fCangjie	VRVK
U+5A1B.0	fFourCorner	46481
U+5A1B.0	fHakka	 ngu2; ngu3; ngi2
U+5A1B.0	fCantonese	jyu4
U+5A1B.0	fMandarin	yu2; wu4
U+5A1B.0	fEnglish	 amuse; give pleasure to; entertain; enjoyment; amusement; pleasure; entertainment
U+5A1C.0	fUTF8	娜
U+5A1C.0	fCNS11643	1-5474
U+5A1C.0	fBig5	AE52
U+5A1C.0	fGB	0-4448
U+5A1C.0	fR/S	 38.7
U+5A1C.0	fTotalStrokes	 10
U+5A1C.0	fCangjie	VSQL
U+5A1C.0	fFourCorner	47427
U+5A1C.0	fHakka	 na3; no3; lo2; no2; la3; la2
U+5A1C.0	fCantonese	no5; no4 (also); naa4; lo5 (also); lo4 (also); laa4 (also)
U+5A1C.0	fMandarin	nuo3
U+5A1C.0	fEnglish	tender; slender; elegant; graceful; delicate
U+5A1C.1	fUTF8	娜
U+5A1C.1	fCNS11643	1-5474
U+5A1C.1	fBig5	AE52
U+5A1C.1	fGB	0-4448
U+5A1C.1	fR/S	 38.7
U+5A1C.1	fTotalStrokes	 10
U+5A1C.1	fCangjie	VSQL
U+5A1C.1	fFourCorner	47427
U+5A1C.1	fMandarin	nuo2
U+5A1C.1	fEnglish	used in female names
U+5A1C.2	fUTF8	娜
U+5A1C.2	fCNS11643	1-5474
U+5A1C.2	fBig5	AE52
U+5A1C.2	fGB	0-4448
U+5A1C.2	fR/S	 38.7
U+5A1C.2	fTotalStrokes	 10
U+5A1C.2	fCangjie	VSQL
U+5A1C.2	fFourCorner	47427
U+5A1C.2	fMandarin	na4
U+5A1C.2	fEnglish	used in transliteration of western female names
U+5A1D.0	fUTF8	娝
U+5A1D.0	fCNS11643	3-303E
U+5A1D.0	fGB	3-3C62
U+5A1D.0	fR/S	  38.7
U+5A1D.0	fTotalStrokes	 10
U+5A1D.0	fCangjie	VMFR
U+5A1D.0	fFourCorner	41460
U+5A1D.0	fHakka	 pi3; pi6; pi5
U+5A1D.0	fCantonese	pau1
U+5A1D.0	fMandarin	pei1
U+5A1D.0	fEnglish	a Chinese family name
U+5A1E.0	fUTF8	娞
U+5A1E.0	fCNS11643	2-2E65
U+5A1E.0	fBig5	D163
U+5A1E.0	fGB	5-3F7C
U+5A1E.0	fR/S	 38.7
U+5A1E.0	fTotalStrokes	 10
U+5A1E.0	fCangjie	VBV
U+5A1E.0	fFourCorner	42444
U+5A1E.0	fCantonese	neoi5; leoi5 (also); seoi1
U+5A1E.0	fMandarin	nei3; sui1; nei2
U+5A1F.0	fUTF8	娟
U+5A1F.0	fCNS11643	1-5475
U+5A1F.0	fBig5	AE53
U+5A1F.0	fGB	0-3E6A
U+5A1F.0	fR/S	 38.7
U+5A1F.0	fTotalStrokes	 10
U+5A1F.0	fCangjie	VRB
U+5A1F.0	fFourCorner	46427
U+5A1F.0	fHakka	 gien1; gen1; kian1; gian1
U+5A1F.0	fCantonese	gyun1
U+5A1F.0	fMandarin	juan1; yuan1
U+5A1F.0	fEnglish	pretty; beautiful; graceful; attractive; good-looking
U+5A20.0	fUTF8	娠
U+5A20.0	fCNS11643	1-5479
U+5A20.0	fBig5	AE57
U+5A20.0	fGB	0-496F
U+5A20.0	fR/S	 38.7
U+5A20.0	fTotalStrokes	 10
U+5A20.0	fCangjie	VMMV
U+5A20.0	fFourCorner	41432
U+5A20.0	fHakka	 shin2; sin2; shin1; siin2; siin1; zhin3; zihn3
U+5A20.0	fCantonese	san1; zan3 (also)
U+5A20.0	fMandarin	shen1; chen2; zhen4
U+5A20.0	fEnglish	pregnant
U+5A21.0	fUTF8	娡
U+5A21.0	fCNS11643	3-303B
U+5A21.0	fGB	5-3F75
U+5A21.0	fR/S	 38.7
U+5A21.0	fTotalStrokes	 10
U+5A21.0	fCangjie	VGP
U+5A21.0	fFourCorner	44430
U+5A21.0	fCantonese	zi3
U+5A21.0	fMandarin	zhi4
U+5A22.0	fUTF8	娢
U+5A22.0	fCNS11643	3-3045
U+5A22.0	fGB	E-266C
U+5A22.0	fR/S	  38.7
U+5A22.0	fTotalStrokes	 10
U+5A22.0	fCangjie	VOIR
U+5A22.0	fFourCorner	48460
U+5A22.0	fCantonese	ham4
U+5A22.0	fMandarin	han2
U+5A22.0	fEnglish	used in a female name
U+5A23.0	fUTF8	娣
U+5A23.0	fCNS11643	1-547A
U+5A23.0	fBig5	AE58
U+5A23.0	fGB	0-6637
U+5A23.0	fR/S	 38.7
U+5A23.0	fTotalStrokes	 10
U+5A23.0	fCangjie	VCNH
U+5A23.0	fFourCorner	48427
U+5A23.0	fHakka	  tai1; ti5; ti6
U+5A23.0	fCantonese	tai5; dai6 (also); tai6
U+5A23.0	fMandarin	di4
U+5A23.0	fEnglish	the wife of a younger brother of one's husband
U+5A24.0	fUTF8	娤
U+5A24.0	fCNS11643	3-3048
U+5A24.0	fGB	E-266D
U+5A24.0	fR/S	  38.7
U+5A24.0	fTotalStrokes	 10
U+5A24.0	fCangjie	VGV
U+5A24.0	fFourCorner	24400
U+5A24.0	fHakka	  zong1
U+5A24.0	fCantonese	zong1
U+5A24.0	fMandarin	zhuang1; zhuan1
U+5A24.0	fEnglish	same as &#x599D;
U+5A25.0	fUTF8	娥
U+5A25.0	fCNS11643	1-547C
U+5A25.0	fBig5	AE5A
U+5A25.0	fGB	0-3670
U+5A25.0	fR/S	 38.7
U+5A25.0	fTotalStrokes	 10
U+5A25.0	fCangjie	VHQI
U+5A25.0	fFourCorner	43450
U+5A25.0	fHakka	 ngo2; ngo3
U+5A25.0	fCantonese	ngo4; o4
U+5A25.0	fMandarin	e2
U+5A25.0	fEnglish	[1] beautiful; good [2] a common name for a girl [3] a Chinese family name
U+5A26.0	fUTF8	娦
U+5A26.0	fCNS11643	3-3052
U+5A26.0	fGB	E-266E
U+5A26.0	fR/S	 38.7
U+5A26.0	fTotalStrokes	 10
U+5A26.0	fCangjie	VOMC
U+5A26.0	fFourCorner	42480
U+5A26.0	fHakka	 bin1
U+5A26.0	fCantonese	pan4; ban3 (also)
U+5A26.0	fMandarin	pin2
U+5A26.0	fEnglish	same as &#x5B2A;
U+5A27.0	fUTF8	娧
U+5A27.0	fCNS11643	4-2C39
U+5A27.0	fGB	3-3C6A
U+5A27.0	fR/S	  38.7
U+5A27.0	fTotalStrokes	 10
U+5A27.0	fCangjie	VCRU
U+5A27.0	fFourCorner	48410
U+5A27.0	fCantonese	teoi3
U+5A27.0	fMandarin	tui4
U+5A28.0	fUTF8	娨
U+5A28.0	fCNS11643	4-2C3D
U+5A28.0	fGB	5-3F79
U+5A28.0	fR/S	 38.7
U+5A28.0	fTotalStrokes	 10
U+5A28.0	fCangjie	VAMJ
U+5A28.0	fFourCorner	46440
U+5A28.0	fCantonese	haan6
U+5A28.0	fMandarin	han4; xian4
U+5A29.0	fUTF8	娩
U+5A29.0	fCNS11643	1-547B
U+5A29.0	fBig5	AE59
U+5A29.0	fGB	0-4364
U+5A29.0	fR/S	 38.7
U+5A29.0	fTotalStrokes	 10
U+5A29.0	fCangjie	VNAU
U+5A29.0	fFourCorner	47416
U+5A29.0	fHakka	  wan3; men1; wan1; mien1; men3; van3; fan5; van1
U+5A29.0	fCantonese	min5; maan5; faan3; man5
U+5A29.0	fMandarin	wan3; fan4; wen4
U+5A29.0	fEnglish	complaisant
U+5A29.1	fUTF8	娩
U+5A29.1	fCNS11643	1-547B
U+5A29.1	fBig5	AE59
U+5A29.1	fGB	0-4364
U+5A29.1	fR/S	 38.7
U+5A29.1	fTotalStrokes	 10
U+5A29.1	fCangjie	VNAU
U+5A29.1	fFourCorner	47416
U+5A29.1	fMandarin	mian3
U+5A29.1	fEnglish	 give birth to a child
U+5A2A.0	fUTF8	娪
U+5A2A.0	fCNS11643	3-303C
U+5A2A.0	fGB	3-3C5F
U+5A2A.0	fR/S	 38.7
U+5A2A.0	fTotalStrokes	 10
U+5A2A.0	fCangjie	VMMR
U+5A2A.0	fFourCorner	41460
U+5A2A.0	fCantonese	ng4
U+5A2A.0	fMandarin	wu2
U+5A2B.0	fUTF8	娫
U+5A2B.0	fCNS11643	3-3522
U+5A2B.0	fGB	3-3C52
U+5A2B.0	fR/S	 38.7; 38.8
U+5A2B.0	fTotalStrokes	 11; 10
U+5A2B.0	fCangjie	VNKM
U+5A2B.0	fFourCorner	42440
U+5A2B.0	fHakka	  tin2
U+5A2B.0	fCantonese	jin4
U+5A2B.0	fMandarin	yan2
U+5A2B.0	fEnglish	handsome
U+5A2C.0	fUTF8	娬
U+5A2C.0	fCNS11643	3-3538
U+5A2C.0	fGB	5-3F7D
U+5A2C.0	fR/S	 38.8
U+5A2C.0	fTotalStrokes	 11
U+5A2C.0	fCangjie	VMPM
U+5A2C.0	fFourCorner	43440
U+5A2C.0	fHakka	 wu3
U+5A2C.0	fCantonese	mou5; mou4 (also)
U+5A2C.0	fMandarin	wu3
U+5A2C.0	fEnglish	lovely; attractive
U+5A2D.0	fUTF8	娭
U+5A2D.0	fCNS11643	2-2E5F
U+5A2D.0	fBig5	D15D
U+5A2D.0	fGB	3-3C6C
U+5A2D.0	fR/S	 38.7
U+5A2D.0	fTotalStrokes	 10
U+5A2D.0	fCangjie	VIOK
U+5A2D.0	fFourCorner	43434
U+5A2D.0	fHakka	  hi1;  ai1
U+5A2D.0	fCantonese	oi1; hei1; aai1; ngaai1 (also); ngoi1 (also)
U+5A2D.0	fMandarin	xi1; ai1
U+5A2D.0	fEnglish	[1] a disrespectful term for a woman [2] slave-girl
U+5A2E.0	fUTF8	娮
U+5A2E.0	fCNS11643	2-2E60
U+5A2E.0	fBig5	D15E
U+5A2E.0	fGB	E-266F
U+5A2E.0	fR/S	 38.7
U+5A2E.0	fTotalStrokes	 10
U+5A2E.0	fCangjie	VYMR
U+5A2E.0	fFourCorner	40461
U+5A2E.0	fCantonese	jin4
U+5A2E.0	fMandarin	yan2
U+5A2E.0	fEnglish	used in a female name
U+5A2F.0	fUTF8	娯
U+5A2F.0	fCNS11643	3-304D
U+5A2F.0	fGB	E-2670
U+5A2F.0	fR/S	 38.7
U+5A2F.0	fTotalStrokes	 10
U+5A2F.0	fCangjie	VRVC
U+5A2F.0	fFourCorner	46481
U+5A2F.0	fCantonese	jyu4
U+5A2F.0	fMandarin	yu2
U+5A2F.0	fEnglish	pleasure; enjoyment; amusement
U+5A30.0	fUTF8	娰
U+5A30.0	fCNS11643	6-3657
U+5A30.0	fGB	E-2671
U+5A30.0	fR/S	 38.7
U+5A30.0	fTotalStrokes	 9; 10
U+5A30.0	fCangjie	VOVO
U+5A30.0	fFourCorner	48400
U+5A30.0	fCantonese	ci5
U+5A30.0	fMandarin	si4
U+5A31.0	fUTF8	娱
U+5A31.0	fCNS11643	3-304B
U+5A31.0	fGB	0-5369
U+5A31.0	fR/S	 38.7
U+5A31.0	fTotalStrokes	 10
U+5A31.0	fCangjie	VRMK
U+5A31.0	fFourCorner	46480
U+5A31.0	fCantonese	jyu4
U+5A31.0	fMandarin	yu2
U+5A31.0	fEnglish	pleasure; enjoyment; amusement
U+5A32.0	fUTF8	娲
U+5A32.0	fGB	0-6634
U+5A32.0	fR/S	 38.7
U+5A32.0	fTotalStrokes	 10
U+5A32.0	fCangjie	VROB
U+5A32.0	fHakka	 vo1
U+5A32.0	fCantonese	wo1
U+5A32.0	fMandarin	wa1; gua1; guai1
U+5A32.0	fEnglish	simplified form of &#x5AA7;
U+5A33.0	fUTF8	娳
U+5A33.0	fCNS11643	2-2E66
U+5A33.0	fBig5	D164
U+5A33.0	fGB	E-2672
U+5A33.0	fR/S	 38.7
U+5A33.0	fTotalStrokes	 10
U+5A33.0	fCangjie	VHDN
U+5A33.0	fFourCorner	42400
U+5A33.0	fCantonese	lei6
U+5A33.0	fMandarin	li4
U+5A34.0	fUTF8	娴
U+5A34.0	fGB	0-6635
U+5A34.0	fR/S	 38.7
U+5A34.0	fTotalStrokes	 10
U+5A34.0	fCangjie	VLSD
U+5A34.0	fCantonese	haan4
U+5A34.0	fMandarin	xian2
U+5A34.0	fEnglish	simplified form of &#x5AFB;
U+5A35.0	fUTF8	娵
U+5A35.0	fCNS11643	2-3457
U+5A35.0	fBig5	D4D4
U+5A35.0	fGB	3-3C71
U+5A35.0	fR/S	 38.8
U+5A35.0	fTotalStrokes	 11
U+5A35.0	fCangjie	VSJE
U+5A35.0	fFourCorner	41447
U+5A35.0	fHakka	  zeu1
U+5A35.0	fCantonese	zeoi1 (also); zyu1; zau1
U+5A35.0	fMandarin	ju1; zou1
U+5A35.0	fEnglish	star
U+5A36.0	fUTF8	娶
U+5A36.0	fCNS11643	1-595E
U+5A36.0	fBig5	B0F9
U+5A36.0	fGB	0-4822
U+5A36.0	fR/S	 38.8
U+5A36.0	fTotalStrokes	 11
U+5A36.0	fCangjie	SEV
U+5A36.0	fFourCorner	17404
U+5A36.0	fHakka	  ci3; cih6
U+5A36.0	fCantonese	ceoi2; ceoi3 (also); cou2 (also)
U+5A36.0	fMandarin	qu3; qu4; ju1; shu1; xu1
U+5A36.0	fEnglish	 marry; take a wife
U+5A37.0	fUTF8	娷
U+5A37.0	fCNS11643	2-3B26
U+5A37.0	fBig5	D8C2
U+5A37.0	fGB	3-3C7E
U+5A37.0	fR/S	 38.8
U+5A37.0	fTotalStrokes	 11
U+5A37.0	fCangjie	VHJM
U+5A37.0	fFourCorner	42414
U+5A37.0	fCantonese	zeoi3; seoi6; zeoi6
U+5A37.0	fMandarin	shui4; chui2; zhui4
U+5A38.0	fUTF8	娸
U+5A38.0	fCNS11643	2-3456
U+5A38.0	fBig5	D4D3
U+5A38.0	fGB	3-3C70
U+5A38.0	fR/S	 38.8
U+5A38.0	fTotalStrokes	 11
U+5A38.0	fCangjie	VTMC
U+5A38.0	fFourCorner	44481
U+5A38.0	fHakka	 ki2
U+5A38.0	fCantonese	hei1
U+5A38.0	fMandarin	qi1; qi2
U+5A38.0	fEnglish	ugly
U+5A39.0	fUTF8	娹
U+5A39.0	fCNS11643	2-3469
U+5A39.0	fBig5	D4E6
U+5A39.0	fGB	5-402B
U+5A39.0	fR/S	 38.8
U+5A39.0	fTotalStrokes	 11
U+5A39.0	fCangjie	VNYI
U+5A39.0	fFourCorner	40432
U+5A39.0	fCantonese	jin4
U+5A39.0	fMandarin	xian2
U+5A3A.0	fUTF8	娺
U+5A3A.0	fCNS11643	3-3528
U+5A3A.0	fGB	3-3D2B
U+5A3A.0	fR/S	 38.8
U+5A3A.0	fTotalStrokes	 11
U+5A3A.0	fCangjie	VEEE
U+5A3A.0	fFourCorner	47440
U+5A3A.0	fHakka	 dot7
U+5A3A.0	fCantonese	to3
U+5A3A.0	fMandarin	zhui1
U+5A3A.0	fEnglish	[1] noise of quarrelling [2] angry
U+5A3B.0	fUTF8	娻
U+5A3B.0	fCNS11643	3-3524
U+5A3B.0	fGB	3-3C36
U+5A3B.0	fR/S	  38.8
U+5A3B.0	fTotalStrokes	 11
U+5A3B.0	fCangjie	VDW
U+5A3B.0	fFourCorner	45490
U+5A3B.0	fCantonese	dung1
U+5A3B.0	fMandarin	dong1
U+5A3C.0	fUTF8	娼
U+5A3C.0	fCNS11643	1-5964
U+5A3C.0	fBig5	B140
U+5A3C.0	fGB	0-663D
U+5A3C.0	fR/S	 38.8
U+5A3C.0	fTotalStrokes	 11
U+5A3C.0	fCangjie	VAA
U+5A3C.0	fFourCorner	46460
U+5A3C.0	fHakka	  chong1; cong1
U+5A3C.0	fCantonese	coeng1
U+5A3C.0	fMandarin	chang1
U+5A3C.0	fEnglish	 prostitute; whore
U+5A3D.0	fUTF8	娽
U+5A3D.0	fCNS11643	3-3531
U+5A3D.0	fGB	3-3D29
U+5A3D.0	fR/S	 38.7; 38.8
U+5A3D.0	fTotalStrokes	 11; 10
U+5A3D.0	fCangjie	XVNME
U+5A3D.0	fFourCorner	47430
U+5A3D.0	fCantonese	luk6
U+5A3D.0	fMandarin	lu4
U+5A3E.0	fUTF8	娾
U+5A3E.0	fCNS11643	2-3467
U+5A3E.0	fBig5	D4E4
U+5A3E.0	fGB	3-3C77
U+5A3E.0	fR/S	 38.8
U+5A3E.0	fTotalStrokes	 11
U+5A3E.0	fCangjie	VMGG
U+5A3E.0	fFourCorner	41414
U+5A3E.0	fCantonese	ngaai6; aai6 (also)
U+5A3E.0	fMandarin	ai3; ai2; yai2
U+5A3F.0	fUTF8	娿
U+5A3F.0	fCNS11643	3-3533
U+5A3F.0	fGB	E-2673
U+5A3F.0	fR/S	  38.8
U+5A3F.0	fTotalStrokes	 11
U+5A3F.0	fCangjie	NRV
U+5A3F.0	fFourCorner	71400
U+5A3F.0	fHakka	  o1
U+5A3F.0	fCantonese	ngo2 (also); ngo1 (also)
U+5A3F.0	fMandarin	e1
U+5A3F.0	fEnglish	[1] undecide [2] unstable
U+5A40.0	fUTF8	婀
U+5A40.0	fCNS11643	1-5963
U+5A40.0	fBig5	B0FE
U+5A40.0	fGB	0-6639
U+5A40.0	fR/S	 38.8
U+5A40.0	fTotalStrokes	 11
U+5A40.0	fCangjie	VNLR
U+5A40.0	fFourCorner	41420
U+5A40.0	fHakka	 o1
U+5A40.0	fCantonese	ngo2 (also); ngo1 (also); o2; o1
U+5A40.0	fMandarin	e1
U+5A40.0	fEnglish	beautiful; graceful; elegant
U+5A41.0	fUTF8	婁
U+5A41.0	fCNS11643	1-595F
U+5A41.0	fBig5	B0FA
U+5A41.0	fGB	1-4226
U+5A41.0	fR/S	 38.8
U+5A41.0	fTotalStrokes	 11
U+5A41.0	fCangjie	LLV
U+5A41.0	fFourCorner	50404
U+5A41.0	fHakka	  leu2; leu3; lui5; liu2
U+5A41.0	fCantonese	lau4; leoi4; leoi5
U+5A41.0	fMandarin	lou2; lei2; lu3
U+5A41.0	fEnglish	[1] a Chinese family name [2] one of the 28 constellations
U+5A41.1	fUTF8	婁
U+5A41.1	fCNS11643	1-595F
U+5A41.1	fBig5	B0FA
U+5A41.1	fGB	1-4226
U+5A41.1	fR/S	 38.8
U+5A41.1	fTotalStrokes	 11
U+5A41.1	fCangjie	LLV
U+5A41.1	fFourCorner	50404
U+5A41.1	fMandarin	luu2
U+5A41.1	fEnglish	 wear; trail along
U+5A41.2	fUTF8	婁
U+5A41.2	fCNS11643	1-595F
U+5A41.2	fBig5	B0FA
U+5A41.2	fGB	1-4226
U+5A41.2	fR/S	 38.8
U+5A41.2	fTotalStrokes	 11
U+5A41.2	fCangjie	LLV
U+5A41.2	fFourCorner	50404
U+5A41.2	fMandarin	luu3
U+5A41.2	fEnglish	 tether (oxen)
U+5A42.0	fUTF8	婂
U+5A42.0	fCNS11643	2-3470
U+5A42.0	fBig5	D4ED
U+5A42.0	fGB	5-4028
U+5A42.0	fR/S	 38.8
U+5A42.0	fTotalStrokes	 11
U+5A42.0	fCangjie	VHAB
U+5A42.0	fFourCorner	47427
U+5A42.0	fHakka	 men2
U+5A42.0	fCantonese	min4
U+5A42.0	fMandarin	mian2
U+5A42.0	fEnglish	used in a female name
U+5A43.0	fUTF8	婃
U+5A43.0	fCNS11643	2-3460
U+5A43.0	fBig5	D4DD
U+5A43.0	fGB	E-2674
U+5A43.0	fR/S	 38.8
U+5A43.0	fTotalStrokes	 11
U+5A43.0	fCangjie	VJMF
U+5A43.0	fFourCorner	43491
U+5A43.0	fCantonese	cung4
U+5A43.0	fMandarin	cong2
U+5A43.0	fEnglish	used in a female name
U+5A44.0	fUTF8	婄
U+5A44.0	fCNS11643	2-3463
U+5A44.0	fBig5	D4E0
U+5A44.0	fGB	3-3D26
U+5A44.0	fR/S	 38.8
U+5A44.0	fTotalStrokes	 11
U+5A44.0	fCangjie	VYTR
U+5A44.0	fFourCorner	40461
U+5A44.0	fCantonese	pau3
U+5A44.0	fMandarin	pou3
U+5A44.0	fEnglish	soft, weak; to soak; unreliable
U+5A45.0	fUTF8	婅
U+5A45.0	fCNS11643	3-352E
U+5A45.0	fGB	E-2675
U+5A45.0	fR/S	 38.8
U+5A45.0	fTotalStrokes	 11
U+5A45.0	fCangjie	VPFD
U+5A45.0	fFourCorner	47420
U+5A45.0	fCantonese	guk1
U+5A45.0	fMandarin	ju2
U+5A45.0	fEnglish	female name
U+5A46.0	fUTF8	婆
U+5A46.0	fCNS11643	1-5967
U+5A46.0	fBig5	B143
U+5A46.0	fGB	0-4645
U+5A46.0	fR/S	 38.8
U+5A46.0	fTotalStrokes	 11
U+5A46.0	fCangjie	EEV
U+5A46.0	fFourCorner	34404
U+5A46.0	fHakka	  po2; po3
U+5A46.0	fCantonese	po4; po1 (also); po2 (also)
U+5A46.0	fMandarin	po2
U+5A46.0	fEnglish	[1] old woman [2] one's grandmother [3] the mother of one's husband
U+5A46.1	fUTF8	婆
U+5A46.1	fCNS11643	1-5967
U+5A46.1	fBig5	B143
U+5A46.1	fGB	0-4645
U+5A46.1	fR/S	 38.8
U+5A46.1	fTotalStrokes	 11
U+5A46.1	fCangjie	EEV
U+5A46.1	fFourCorner	34404
U+5A46.1	fMandarin	po5
U+5A47.0	fUTF8	婇
U+5A47.0	fCNS11643	2-346D
U+5A47.0	fBig5	D4EA
U+5A47.0	fGB	E-2676
U+5A47.0	fR/S	 38.8
U+5A47.0	fTotalStrokes	 11
U+5A47.0	fCangjie	VBD
U+5A47.0	fFourCorner	42494
U+5A47.0	fCantonese	coi2
U+5A47.0	fMandarin	cai3; cai4
U+5A47.0	fEnglish	used in a female name
U+5A48.0	fUTF8	婈
U+5A48.0	fCNS11643	2-3465
U+5A48.0	fBig5	D4E2
U+5A48.0	fGB	5-4021
U+5A48.0	fR/S	 38.8
U+5A48.0	fTotalStrokes	 11
U+5A48.0	fCangjie	VGCE
U+5A48.0	fFourCorner	44447
U+5A48.0	fCantonese	ling4
U+5A48.0	fMandarin	ding2; ling2
U+5A48.0	fEnglish	used in a female name
U+5A49.0	fUTF8	婉
U+5A49.0	fCNS11643	1-5960
U+5A49.0	fBig5	B0FB
U+5A49.0	fGB	0-4D71
U+5A49.0	fR/S	 38.8
U+5A49.0	fTotalStrokes	 11
U+5A49.0	fCangjie	VJNU
U+5A49.0	fFourCorner	43412
U+5A49.0	fHakka	  won3;  wan3; von3; van3
U+5A49.0	fCantonese	jyun2; wun2
U+5A49.0	fMandarin	wan3; yung4
U+5A49.0	fEnglish	[1] amiable; genial; agreeable; pleasant; gently; genially; amicable [2] good-looking; beautiful
U+5A4A.0	fUTF8	婊
U+5A4A.0	fCNS11643	1-5968
U+5A4A.0	fBig5	B144
U+5A4A.0	fGB	0-663B
U+5A4A.0	fR/S	 38.8
U+5A4A.0	fTotalStrokes	 11
U+5A4A.0	fCangjie	VQMV
U+5A4A.0	fFourCorner	45432
U+5A4A.0	fHakka	  biau3; beu3; biau1
U+5A4A.0	fCantonese	biu2
U+5A4A.0	fMandarin	biao3
U+5A4A.0	fEnglish	 whore; prostitute
U+5A4B.0	fUTF8	婋
U+5A4B.0	fCNS11643	3-352A
U+5A4B.0	fGB	5-4024
U+5A4B.0	fR/S	 38.8
U+5A4B.0	fTotalStrokes	 11
U+5A4B.0	fCangjie	VYPU
U+5A4B.0	fFourCorner	41410
U+5A4B.0	fCantonese	haau1
U+5A4B.0	fMandarin	xiao1
U+5A4C.0	fUTF8	婌
U+5A4C.0	fCNS11643	2-346A
U+5A4C.0	fBig5	D4E7
U+5A4C.0	fGB	5-4022
U+5A4C.0	fR/S	 38.8
U+5A4C.0	fTotalStrokes	 11
U+5A4C.0	fCangjie	VYFE
U+5A4C.0	fFourCorner	47440
U+5A4C.0	fCantonese	suk6
U+5A4C.0	fMandarin	shu3; shu2
U+5A4D.0	fUTF8	婍
U+5A4D.0	fCNS11643	2-3468
U+5A4D.0	fBig5	D4E5
U+5A4D.0	fGB	3-3C78
U+5A4D.0	fR/S	 38.8
U+5A4D.0	fTotalStrokes	 11
U+5A4D.0	fCangjie	VKMR
U+5A4D.0	fFourCorner	44421
U+5A4D.0	fCantonese	kei2; ji2
U+5A4D.0	fMandarin	qi3
U+5A4E.0	fUTF8	婎
U+5A4E.0	fCNS11643	3-352F
U+5A4E.0	fGB	3-3D23
U+5A4E.0	fR/S	 38.8
U+5A4E.0	fTotalStrokes	 11
U+5A4E.0	fCangjie	VOG
U+5A4E.0	fFourCorner	40410
U+5A4E.0	fCantonese	fai1
U+5A4E.0	fMandarin	hui1
U+5A4F.0	fUTF8	婏
U+5A4F.0	fCNS11643	3-353B
U+5A4F.0	fGB	3-3D25
U+5A4F.0	fR/S	  38.8
U+5A4F.0	fTotalStrokes	 11
U+5A4F.0	fCangjie	VNAI
U+5A4F.0	fFourCorner	47410
U+5A4F.0	fHakka	 wan3
U+5A4F.0	fCantonese	faan3
U+5A4F.0	fMandarin	fu4
U+5A50.0	fUTF8	婐
U+5A50.0	fCNS11643	2-3459
U+5A50.0	fBig5	D4D6
U+5A50.0	fGB	E-2677
U+5A50.0	fR/S	 38.8
U+5A50.0	fTotalStrokes	 11
U+5A50.0	fCangjie	VWD
U+5A50.0	fFourCorner	46494
U+5A50.0	fHakka	 ko5; go3; wo3; vo3
U+5A50.0	fCantonese	wo2; ngo2 (also); ngo1 (also); o1; lo2; gwo2
U+5A50.0	fMandarin	wo3; e1; gua1
U+5A50.0	fEnglish	maid
U+5A51.0	fUTF8	婑
U+5A51.0	fCNS11643	2-346E
U+5A51.0	fBig5	D4EB
U+5A51.0	fGB	3-3D21
U+5A51.0	fR/S	 38.8
U+5A51.0	fTotalStrokes	 11
U+5A51.0	fCangjie	VHDV
U+5A51.0	fFourCorner	42444
U+5A51.0	fCantonese	wo2; wo1
U+5A51.0	fMandarin	rui2; wo3
U+5A52.0	fUTF8	婒
U+5A52.0	fCNS11643	2-3462
U+5A52.0	fBig5	D4DF
U+5A52.0	fGB	E-2678
U+5A52.0	fR/S	 38.8
U+5A52.0	fTotalStrokes	 11
U+5A52.0	fCangjie	VFF
U+5A52.0	fFourCorner	49489
U+5A52.0	fCantonese	taam4
U+5A52.0	fMandarin	tan2
U+5A52.0	fEnglish	used in a female name
U+5A53.0	fUTF8	婓
U+5A53.0	fCNS11643	2-345D
U+5A53.0	fBig5	D4DA
U+5A53.0	fGB	3-7458
U+5A53.0	fR/S	 38.8
U+5A53.0	fTotalStrokes	 11
U+5A53.0	fCangjie	LYV
U+5A53.0	fFourCorner	11441
U+5A53.0	fHakka	 fui1; fi1; pui2; pi2
U+5A53.0	fCantonese	fei1; fei4 (also); pui4
U+5A53.0	fMandarin	fei1
U+5A53.0	fEnglish	 pace to and fro; undecided
U+5A54.0	fUTF8	婔
U+5A54.0	fCNS11643	3-353A
U+5A54.0	fGB	E-2679
U+5A54.0	fR/S	  38.8
U+5A54.0	fTotalStrokes	 11
U+5A54.0	fCangjie	VLMY
U+5A54.0	fFourCorner	41410
U+5A54.0	fHakka	 fui1; fi1; pui2; pi2
U+5A54.0	fCantonese	fei1; fei4 (also); pui4
U+5A54.0	fMandarin	fei1
U+5A54.0	fEnglish	same as &#x5A53;
U+5A55.0	fUTF8	婕
U+5A55.0	fCNS11643	2-3453
U+5A55.0	fBig5	D4D0
U+5A55.0	fGB	0-663C
U+5A55.0	fR/S	 38.8
U+5A55.0	fTotalStrokes	 11
U+5A55.0	fCangjie	VJLO
U+5A55.0	fFourCorner	45481
U+5A55.0	fHakka	  ciap8
U+5A55.0	fCantonese	zit3; zip3; zit6
U+5A55.0	fMandarin	jie2
U+5A55.0	fEnglish	used in female names
U+5A56.0	fUTF8	婖
U+5A56.0	fCNS11643	2-346F
U+5A56.0	fBig5	D4EC
U+5A56.0	fGB	E-267A
U+5A56.0	fR/S	 38.8
U+5A56.0	fTotalStrokes	 11
U+5A56.0	fCangjie	VHKP
U+5A56.0	fFourCorner	42433
U+5A56.0	fCantonese	tim1
U+5A56.0	fMandarin	tian1; tian3
U+5A57.0	fUTF8	婗
U+5A57.0	fCNS11643	2-345F
U+5A57.0	fBig5	D4DC
U+5A57.0	fGB	3-3D22
U+5A57.0	fR/S	 38.8
U+5A57.0	fTotalStrokes	 11
U+5A57.0	fCangjie	VHXU
U+5A57.0	fFourCorner	47417
U+5A57.0	fCantonese	ngai4; ai4 (also); nei4
U+5A57.0	fMandarin	ni2; ni3
U+5A57.0	fEnglish	a new-born child the whimper of an infant
U+5A58.0	fUTF8	婘
U+5A58.0	fCNS11643	2-3452
U+5A58.0	fBig5	D4CF
U+5A58.0	fGB	3-3D27
U+5A58.0	fR/S	 38.8
U+5A58.0	fTotalStrokes	 11
U+5A58.0	fCangjie	VFQU
U+5A58.0	fFourCorner	49412
U+5A58.0	fCantonese	kyun4; gyun3
U+5A58.0	fMandarin	quan2; juan4
U+5A59.0	fUTF8	婙
U+5A59.0	fCNS11643	3-3539
U+5A59.0	fGB	5-3F6D
U+5A59.0	fR/S	 38.8
U+5A59.0	fTotalStrokes	 11
U+5A59.0	fCangjie	VBSD
U+5A59.0	fFourCorner	42450
U+5A59.0	fHakka	  ciang5;  cin5; ziang1
U+5A59.0	fCantonese	zing6; cing5
U+5A59.0	fMandarin	jing4; jing1; qing4
U+5A59.0	fEnglish	same as &#x5A67;
U+5A5A.0	fUTF8	婚
U+5A5A.0	fCNS11643	1-5966
U+5A5A.0	fBig5	B142
U+5A5A.0	fGB	0-3B69
U+5A5A.0	fR/S	 38.8
U+5A5A.0	fTotalStrokes	 11
U+5A5A.0	fCangjie	VHPA
U+5A5A.0	fFourCorner	42464
U+5A5A.0	fHakka	  fun1
U+5A5A.0	fCantonese	fan1
U+5A5A.0	fMandarin	hun1; yu4
U+5A5A.0	fEnglish	[1] get married; wed; marry [2] marriage; wedding
U+5A5B.0	fUTF8	婛
U+5A5B.0	fCNS11643	2-3464
U+5A5B.0	fBig5	D4E1
U+5A5B.0	fGB	E-267B
U+5A5B.0	fR/S	 38.8
U+5A5B.0	fTotalStrokes	 11
U+5A5B.0	fCangjie	VYRF
U+5A5B.0	fFourCorner	40496
U+5A5B.0	fCantonese	ging1
U+5A5B.0	fMandarin	jing1
U+5A5B.0	fEnglish	used in a female name
U+5A5C.0	fUTF8	婜
U+5A5C.0	fCNS11643	2-3471
U+5A5C.0	fBig5	D4EE
U+5A5C.0	fGB	3-3C28
U+5A5C.0	fR/S	 38.8
U+5A5C.0	fTotalStrokes	 11
U+5A5C.0	fCangjie	SEV
U+5A5C.0	fFourCorner	77404
U+5A5C.0	fCantonese	haan1; gan2
U+5A5C.0	fMandarin	qian1
U+5A5D.0	fUTF8	婝
U+5A5D.0	fCNS11643	2-3461
U+5A5D.0	fBig5	D4DE
U+5A5D.0	fGB	E-267C
U+5A5D.0	fR/S	 38.8
U+5A5D.0	fTotalStrokes	 11
U+5A5D.0	fCangjie	VJMO
U+5A5D.0	fFourCorner	43481
U+5A5D.0	fCantonese	din6
U+5A5D.0	fMandarin	dian4
U+5A5D.0	fEnglish	used in a female name
U+5A5E.0	fUTF8	婞
U+5A5E.0	fCNS11643	2-3455
U+5A5E.0	fBig5	D4D2
U+5A5E.0	fGB	3-3C6F
U+5A5E.0	fR/S	 38.8
U+5A5E.0	fTotalStrokes	 11
U+5A5E.0	fCangjie	VGTJ
U+5A5E.0	fFourCorner	44441
U+5A5E.0	fHakka	 hen5; hen6; hen3
U+5A5E.0	fCantonese	hang6
U+5A5E.0	fMandarin	xing4
U+5A5E.0	fEnglish	hate
U+5A5F.0	fUTF8	婟
U+5A5F.0	fCNS11643	2-345A
U+5A5F.0	fBig5	D4D7
U+5A5F.0	fGB	3-3C7C
U+5A5F.0	fR/S	 38.8
U+5A5F.0	fTotalStrokes	 11
U+5A5F.0	fCangjie	VWJR
U+5A5F.0	fFourCorner	46400
U+5A5F.0	fCantonese	wu6
U+5A5F.0	fMandarin	hu4; gu4
U+5A60.0	fUTF8	婠
U+5A60.0	fCNS11643	2-3451
U+5A60.0	fBig5	D4CE
U+5A60.0	fGB	3-3D28
U+5A60.0	fR/S	 38.8
U+5A60.0	fTotalStrokes	 11
U+5A60.0	fCangjie	VJRR
U+5A60.0	fFourCorner	43477
U+5A60.0	fCantonese	wun1; waat3 (also); waan1
U+5A60.0	fMandarin	wa4; wan2; guan4; wan1; wu4
U+5A61.0	fUTF8	婡
U+5A61.0	fCNS11643	3-3527
U+5A61.0	fGB	3-3C63
U+5A61.0	fR/S	 38.8
U+5A61.0	fTotalStrokes	 11
U+5A61.0	fCangjie	VDOO
U+5A61.0	fFourCorner	44490
U+5A61.0	fCantonese	loi4
U+5A61.0	fMandarin	lai2
U+5A62.0	fUTF8	婢
U+5A62.0	fCNS11643	1-5965
U+5A62.0	fBig5	B141
U+5A62.0	fGB	0-663E
U+5A62.0	fR/S	 38.8
U+5A62.0	fTotalStrokes	 11
U+5A62.0	fCangjie	VHWJ
U+5A62.0	fFourCorner	46440
U+5A62.0	fHakka	 bi1; pi5; pi3; pi6; bi3; pi1
U+5A62.0	fCantonese	pei5
U+5A62.0	fMandarin	bi4; bei4
U+5A62.0	fEnglish	[1] servant girl; maidservant; female slave [2] a humble term used by a girl to refer to herself
U+5A63.0	fUTF8	婣
U+5A63.0	fCNS11643	3-353C
U+5A63.0	fGB	E-267D
U+5A63.0	fR/S	  38.8
U+5A63.0	fTotalStrokes	 11
U+5A63.0	fCangjie	VLXL
U+5A63.0	fFourCorner	42400
U+5A63.0	fHakka	 jin1; rin1; ren1; jen1
U+5A63.0	fCantonese	jan1
U+5A63.0	fMandarin	yin1
U+5A63.0	fEnglish	marriage connections; a bride
U+5A64.0	fUTF8	婤
U+5A64.0	fCNS11643	2-345E
U+5A64.0	fBig5	D4DB
U+5A64.0	fGB	3-3D24
U+5A64.0	fR/S	 38.8
U+5A64.0	fTotalStrokes	 11
U+5A64.0	fCangjie	VBGR
U+5A64.0	fFourCorner	47420
U+5A64.0	fCantonese	zau1; cau1
U+5A64.0	fMandarin	zhou1; chou1; diao1
U+5A64.0	fEnglish	lovely
U+5A65.0	fUTF8	婥
U+5A65.0	fCNS11643	2-345B
U+5A65.0	fBig5	D4D8
U+5A65.0	fGB	5-4023
U+5A65.0	fR/S	 38.8
U+5A65.0	fTotalStrokes	 11
U+5A65.0	fCangjie	VYAJ
U+5A65.0	fFourCorner	41446
U+5A65.0	fHakka	 zok7;  shok7; sok7; zhok7
U+5A65.0	fCantonese	coek3
U+5A65.0	fMandarin	chuo4; di4; nao4
U+5A65.0	fEnglish	beautiful
U+5A66.0	fUTF8	婦
U+5A66.0	fCNS11643	1-5961
U+5A66.0	fBig5	B0FC
U+5A66.0	fGB	1-383E
U+5A66.0	fR/S	 38.8
U+5A66.0	fTotalStrokes	 11
U+5A66.0	fCangjie	VSMB
U+5A66.0	fFourCorner	47427
U+5A66.0	fHakka	  fu5; fu3; fu6
U+5A66.0	fCantonese	fu5
U+5A66.0	fMandarin	fu4
U+5A66.0	fEnglish	[1] married woman; woman; female; matron [2] the wife of one's son [3] wife [4] married woman
U+5A66.1	fUTF8	婦
U+5A66.1	fCNS11643	1-5961
U+5A66.1	fBig5	B0FC
U+5A66.1	fGB	1-383E
U+5A66.1	fR/S	 38.8
U+5A66.1	fTotalStrokes	 11
U+5A66.1	fCangjie	VSMB
U+5A66.1	fFourCorner	47427
U+5A66.1	fMandarin	fu5
U+5A67.0	fUTF8	婧
U+5A67.0	fCNS11643	2-3454
U+5A67.0	fBig5	D4D1
U+5A67.0	fGB	0-663A
U+5A67.0	fR/S	 38.8
U+5A67.0	fTotalStrokes	 11
U+5A67.0	fCangjie	VQMB
U+5A67.0	fFourCorner	45427
U+5A67.0	fHakka	  ciang5;  cin5; ziang1
U+5A67.0	fCantonese	zing6; cing5
U+5A67.0	fMandarin	jing4; jing1; qing4
U+5A67.0	fEnglish	 modest supple
U+5A68.0	fUTF8	婨
U+5A68.0	fCNS11643	3-3530
U+5A68.0	fGB	5-3F49
U+5A68.0	fR/S	 38.8
U+5A68.0	fTotalStrokes	 11
U+5A68.0	fCangjie	VOMB
U+5A68.0	fFourCorner	48420
U+5A68.0	fCantonese	leon4
U+5A68.0	fMandarin	lun2
U+5A68.0	fEnglish	female name
U+5A69.0	fUTF8	婩
U+5A69.0	fCNS11643	2-346C
U+5A69.0	fBig5	D4E9
U+5A69.0	fGB	3-3C7D
U+5A69.0	fR/S	 38.8
U+5A69.0	fTotalStrokes	 11
U+5A69.0	fCangjie	VUMJ
U+5A69.0	fFourCorner	62441
U+5A69.0	fHakka	 ngan5; ngan6
U+5A69.0	fCantonese	ngon6; on6 (also); joek6
U+5A69.0	fMandarin	yan4; an4
U+5A69.0	fEnglish	an well-dressed, elegant woman
U+5A6A.0	fUTF8	婪
U+5A6A.0	fCNS11643	1-5962
U+5A6A.0	fBig5	B0FD
U+5A6A.0	fGB	0-4037
U+5A6A.0	fR/S	 38.8
U+5A6A.0	fTotalStrokes	 11
U+5A6A.0	fCangjie	DDV
U+5A6A.0	fFourCorner	44404
U+5A6A.0	fHakka	 lam2
U+5A6A.0	fCantonese	laam4
U+5A6A.0	fMandarin	lan2; lan3; lin2
U+5A6A.0	fEnglish	greedy; covetous; avaricious; avarice
U+5A6B.0	fUTF8	婫
U+5A6B.0	fCNS11643	3-352B
U+5A6B.0	fGB	3-3C7B
U+5A6B.0	fR/S	 38.8
U+5A6B.0	fTotalStrokes	 11
U+5A6B.0	fCangjie	VAPP
U+5A6B.0	fFourCorner	46410
U+5A6B.0	fCantonese	gwan1
U+5A6B.0	fMandarin	kun1
U+5A6C.0	fUTF8	婬
U+5A6C.0	fCNS11643	2-345C
U+5A6C.0	fBig5	D4D9
U+5A6C.0	fGB	E-267E
U+5A6C.0	fR/S	 38.8
U+5A6C.0	fTotalStrokes	 11
U+5A6C.0	fCangjie	VBHG
U+5A6C.0	fFourCorner	42414
U+5A6C.0	fHakka	 jim2; rim2; jim3
U+5A6C.0	fCantonese	jam4
U+5A6C.0	fMandarin	yin2
U+5A6C.0	fEnglish	obscene; licentious; lewd
U+5A6D.0	fUTF8	婭
U+5A6D.0	fCNS11643	2-3458
U+5A6D.0	fBig5	D4D5
U+5A6D.0	fGB	1-662B
U+5A6D.0	fR/S	 38.8
U+5A6D.0	fTotalStrokes	 11
U+5A6D.0	fCangjie	VMLM
U+5A6D.0	fFourCorner	41417
U+5A6D.0	fHakka	 a3; a5
U+5A6D.0	fCantonese	aa3; ngaa3 (also)
U+5A6D.0	fMandarin	ya4
U+5A6D.0	fEnglish	a mutual term of address between one's sons-in-law
U+5A6E.0	fUTF8	婮
U+5A6E.0	fCNS11643	3-3529
U+5A6E.0	fGB	E-2721
U+5A6E.0	fR/S	 38.8
U+5A6E.0	fTotalStrokes	 11
U+5A6E.0	fCangjie	VSJR
U+5A6E.0	fFourCorner	47460
U+5A6E.0	fCantonese	geoi1
U+5A6E.0	fMandarin	ju1
U+5A6E.0	fEnglish	used in a female name
U+5A6F.0	fUTF8	婯
U+5A6F.0	fCNS11643	3-3534
U+5A6F.0	fGB	E-2722
U+5A6F.0	fR/S	 38.8
U+5A6F.0	fTotalStrokes	 11
U+5A6F.0	fCangjie	MMBBV
U+5A6F.0	fFourCorner	11400
U+5A6F.0	fHakka	  li5; li6; li2; li3
U+5A6F.0	fCantonese	lai6; lei4; lai4
U+5A6F.0	fMandarin	li4; li2; li3; si1
U+5A6F.0	fEnglish	same as &#x9E97;
U+5A70.0	fUTF8	婰
U+5A70.0	fCNS11643	2-346B
U+5A70.0	fBig5	D4E8
U+5A70.0	fGB	5-4026
U+5A70.0	fR/S	 38.8
U+5A70.0	fTotalStrokes	 11
U+5A70.0	fCangjie	VTBC
U+5A70.0	fFourCorner	45481
U+5A70.0	fCantonese	din2
U+5A70.0	fMandarin	dian3
U+5A70.0	fEnglish	female name
U+5A71.0	fUTF8	婱
U+5A71.0	fCNS11643	3-3532
U+5A71.0	fGB	E-2723
U+5A71.0	fR/S	 38.8
U+5A71.0	fTotalStrokes	 11
U+5A71.0	fCangjie	NIV
U+5A71.0	fFourCorner	10400
U+5A71.0	fCantonese	jin4
U+5A71.0	fMandarin	xian2
U+5A71.0	fEnglish	same as &#x5A39;
U+5A72.0	fUTF8	婲
U+5A72.0	fCNS11643	F-3373
U+5A72.0	fGB	E-2724
U+5A72.0	fR/S	 38.8
U+5A72.0	fTotalStrokes	 11
U+5A72.0	fCangjie	VTOP
U+5A72.0	fFourCorner	4441
U+5A72.0	fCantonese	waa6
U+5A73.0	fUTF8	婳
U+5A73.0	fCNS11643	3-3537
U+5A73.0	fGB	8-7B39
U+5A73.0	fR/S	 38.8; 38.7
U+5A73.0	fTotalStrokes	 11
U+5A73.0	fCangjie	VMUW
U+5A73.0	fFourCorner	41470
U+5A73.0	fCantonese	waak6
U+5A73.0	fMandarin	hua4
U+5A73.0	fEnglish	tranquil
U+5A74.0	fUTF8	婴
U+5A74.0	fGB	0-5324
U+5A74.0	fR/S	 38.8
U+5A74.0	fTotalStrokes	 11
U+5A74.0	fCangjie	BOV
U+5A74.0	fHakka	 jin1
U+5A74.0	fCantonese	jing1
U+5A74.0	fMandarin	ying1; ying4
U+5A74.0	fEnglish	simplified form of &#x5B30;
U+5A75.0	fUTF8	婵
U+5A75.0	fCNS11643	3-3535
U+5A75.0	fGB	0-663F
U+5A75.0	fR/S	 38.8
U+5A75.0	fTotalStrokes	 11
U+5A75.0	fCangjie	VCWJ
U+5A75.0	fFourCorner	48450
U+5A75.0	fHakka	 sham2
U+5A75.0	fCantonese	sim4; sin4
U+5A75.0	fMandarin	chan2
U+5A75.0	fEnglish	simplified form of &#x5B0B;
U+5A76.0	fUTF8	婶
U+5A76.0	fGB	0-4974
U+5A76.0	fR/S	 38.8
U+5A76.0	fTotalStrokes	 11
U+5A76.0	fCangjie	VJLL
U+5A76.0	fHakka	 shim3
U+5A76.0	fCantonese	sam2
U+5A76.0	fMandarin	shen3
U+5A76.0	fEnglish	simplified form of &#x5B38;
U+5A76.1	fUTF8	婶
U+5A76.1	fGB	0-4974
U+5A76.1	fR/S	 38.8
U+5A76.1	fTotalStrokes	 11
U+5A76.1	fCangjie	VJLL
U+5A76.1	fMandarin	shen5
U+5A77.0	fUTF8	婷
U+5A77.0	fCNS11643	1-5E65
U+5A77.0	fBig5	B440
U+5A77.0	fGB	0-6643
U+5A77.0	fR/S	 38.9
U+5A77.0	fTotalStrokes	 12
U+5A77.0	fCangjie	VYRN
U+5A77.0	fFourCorner	40421
U+5A77.0	fHakka	 tin2
U+5A77.0	fCantonese	ting4; ting2 (also)
U+5A77.0	fMandarin	ting2
U+5A77.0	fEnglish	pretty; attractive; graceful
U+5A78.0	fUTF8	婸
U+5A78.0	fCNS11643	2-3A7D
U+5A78.0	fBig5	D8BB
U+5A78.0	fGB	3-3C21
U+5A78.0	fR/S	 38.9
U+5A78.0	fTotalStrokes	 12
U+5A78.0	fCangjie	VAMH
U+5A78.0	fFourCorner	46427
U+5A78.0	fHakka	 tong3; tong5
U+5A78.0	fCantonese	dong6; tong3
U+5A78.0	fMandarin	dang4; yang2
U+5A78.0	fEnglish	dissolute; wanton
U+5A79.0	fUTF8	婹
U+5A79.0	fCNS11643	3-3A75
U+5A79.0	fGB	5-4031
U+5A79.0	fR/S	 38.9
U+5A79.0	fTotalStrokes	 12
U+5A79.0	fCangjie	VMWV
U+5A79.0	fFourCorner	41440
U+5A79.0	fHakka	 miau3; rau3; jeu3; jau3
U+5A79.0	fCantonese	jiu2
U+5A79.0	fMandarin	yao3
U+5A79.0	fEnglish	slender; slim
U+5A7A.0	fUTF8	婺
U+5A7A.0	fCNS11643	2-3A7A
U+5A7A.0	fBig5	D8B8
U+5A7A.0	fGB	0-6644
U+5A7A.0	fR/S	 38.9
U+5A7A.0	fTotalStrokes	 12
U+5A7A.0	fCangjie	NKV
U+5A7A.0	fFourCorner	18404
U+5A7A.0	fHakka	 wu6; wu5; vu5
U+5A7A.0	fCantonese	mou6
U+5A7A.0	fMandarin	wu4; mou2; mu4
U+5A7A.0	fEnglish	beautiful; charming
U+5A7B.0	fUTF8	婻
U+5A7B.0	fCNS11643	2-3B2D
U+5A7B.0	fBig5	D8C9
U+5A7B.0	fGB	3-3D2E
U+5A7B.0	fR/S	 38.9
U+5A7B.0	fTotalStrokes	 12
U+5A7B.0	fCangjie	VJBJ
U+5A7B.0	fFourCorner	44427
U+5A7B.0	fCantonese	naam6; laam6 (also); nam6
U+5A7B.0	fMandarin	nan4
U+5A7C.0	fUTF8	婼
U+5A7C.0	fCNS11643	2-3B21
U+5A7C.0	fBig5	D8BD
U+5A7C.0	fGB	3-3C72
U+5A7C.0	fR/S	 38.9
U+5A7C.0	fTotalStrokes	 12
U+5A7C.0	fCangjie	VTKR
U+5A7C.0	fFourCorner	44464
U+5A7C.0	fHakka	 shok7; sok7
U+5A7C.0	fCantonese	coek3; joek6; je4; ji3
U+5A7C.0	fMandarin	chuo4; ruo4; er2
U+5A7C.0	fEnglish	person
U+5A7D.0	fUTF8	婽
U+5A7D.0	fCNS11643	2-3B2E
U+5A7D.0	fBig5	D8CA
U+5A7D.0	fGB	5-4040
U+5A7D.0	fR/S	 38.9
U+5A7D.0	fTotalStrokes	 12
U+5A7D.0	fCangjie	VRSE
U+5A7D.0	fFourCorner	47447
U+5A7D.0	fCantonese	gaa2
U+5A7D.0	fMandarin	jia3
U+5A7E.0	fUTF8	婾
U+5A7E.0	fCNS11643	3-3A7C
U+5A7E.0	fGB	E-2725
U+5A7E.0	fR/S	 38.9
U+5A7E.0	fTotalStrokes	 12
U+5A7E.0	fCangjie	VOMV
U+5A7E.0	fFourCorner	48420
U+5A7E.0	fCantonese	jyu4
U+5A7E.0	fMandarin	tou1; yu2
U+5A7E.0	fEnglish	handsome
U+5A7F.0	fUTF8	婿
U+5A7F.0	fCNS11643	1-5E67
U+5A7F.0	fBig5	B442
U+5A7F.0	fGB	0-5076
U+5A7F.0	fR/S	 38.9
U+5A7F.0	fTotalStrokes	 12
U+5A7F.0	fCangjie	VNOB
U+5A7F.0	fFourCorner	47427
U+5A7F.0	fHakka	  se5
U+5A7F.0	fCantonese	sai3; sai2 (also)
U+5A7F.0	fMandarin	xu4
U+5A7F.0	fEnglish	 son-in-law husband
U+5A80.0	fUTF8	媀
U+5A80.0	fCNS11643	3-3A63
U+5A80.0	fGB	3-3D33
U+5A80.0	fR/S	  38.9
U+5A80.0	fTotalStrokes	 12
U+5A80.0	fCangjie	VWLB
U+5A80.0	fFourCorner	46420
U+5A80.0	fCantonese	jyu6
U+5A80.0	fMandarin	yu2
U+5A81.0	fUTF8	媁
U+5A81.0	fCNS11643	3-3A61
U+5A81.0	fGB	3-3C24
U+5A81.0	fR/S	 38.9
U+5A81.0	fTotalStrokes	 12
U+5A81.0	fCangjie	VDMQ
U+5A81.0	fFourCorner	44450
U+5A81.0	fCantonese	wai4
U+5A81.0	fMandarin	wei1
U+5A82.0	fUTF8	媂
U+5A82.0	fCNS11643	3-3A58
U+5A82.0	fGB	3-3D3B
U+5A82.0	fR/S	 38.9
U+5A82.0	fTotalStrokes	 12
U+5A82.0	fCangjie	VYBB
U+5A82.0	fFourCorner	40420
U+5A82.0	fCantonese	dai3
U+5A82.0	fMandarin	ti2
U+5A83.0	fUTF8	媃
U+5A83.0	fCNS11643	2-3B2A
U+5A83.0	fBig5	D8C6
U+5A83.0	fGB	3-3D42
U+5A83.0	fR/S	 38.9
U+5A83.0	fTotalStrokes	 12
U+5A83.0	fCangjie	VNHD
U+5A83.0	fFourCorner	47494
U+5A83.0	fCantonese	jau4
U+5A83.0	fMandarin	rou2
U+5A83.0	fEnglish	female name
U+5A84.0	fUTF8	媄
U+5A84.0	fCNS11643	2-3B27
U+5A84.0	fBig5	D8C3
U+5A84.0	fGB	5-403C
U+5A84.0	fR/S	 38.9
U+5A84.0	fTotalStrokes	 12
U+5A84.0	fCangjie	VTGK
U+5A84.0	fFourCorner	48435
U+5A84.0	fHakka	 mui1
U+5A84.0	fCantonese	mei5
U+5A84.0	fMandarin	mei3
U+5A85.0	fUTF8	媅
U+5A85.0	fCNS11643	3-3A5B
U+5A85.0	fGB	3-3D2D
U+5A85.0	fR/S	  38.9
U+5A85.0	fTotalStrokes	 12
U+5A85.0	fCangjie	VTMV
U+5A85.0	fFourCorner	44410
U+5A85.0	fCantonese	daam1
U+5A85.0	fMandarin	dan1
U+5A86.0	fUTF8	媆
U+5A86.0	fCNS11643	3-3A77
U+5A86.0	fGB	5-4032
U+5A86.0	fR/S	 38.9
U+5A86.0	fTotalStrokes	 12
U+5A86.0	fCangjie	VMBK
U+5A86.0	fFourCorner	41430
U+5A86.0	fHakka	 ngion2; nun5; nun6
U+5A86.0	fCantonese	jyun5; nyun6
U+5A86.0	fMandarin	ruan3
U+5A86.0	fEnglish	 spit out
U+5A87.0	fUTF8	媇
U+5A87.0	fCNS11643	3-3A72
U+5A87.0	fGB	E-2726
U+5A87.0	fR/S	 38.9
U+5A87.0	fTotalStrokes	 12
U+5A87.0	fCangjie	VYTD
U+5A87.0	fFourCorner	40490
U+5A87.0	fHakka	  cin1; zim1
U+5A87.0	fCantonese	can1; can3 (also)
U+5A87.0	fMandarin	qin1; qin4; xin1 qing4 qin5
U+5A87.0	fEnglish	same as &#x89AA;
U+5A88.0	fUTF8	媈
U+5A88.0	fCNS11643	3-3A59
U+5A88.0	fGB	E-2727
U+5A88.0	fR/S	  38.9
U+5A88.0	fTotalStrokes	 12
U+5A88.0	fCangjie	VBJJ
U+5A88.0	fFourCorner	47450
U+5A88.0	fHakka	  fun2; fui1
U+5A88.0	fCantonese	fai1
U+5A88.0	fMandarin	hui1
U+5A88.0	fEnglish	used in a female name
U+5A89.0	fUTF8	媉
U+5A89.0	fCNS11643	3-3A60
U+5A89.0	fGB	3-3D41
U+5A89.0	fR/S	  38.9
U+5A89.0	fTotalStrokes	 12
U+5A89.0	fCangjie	VSMG
U+5A89.0	fFourCorner	47410
U+5A89.0	fCantonese	ak1; ngak1 (also)
U+5A89.0	fMandarin	wu1
U+5A8A.0	fUTF8	媊
U+5A8A.0	fCNS11643	2-3B28
U+5A8A.0	fBig5	D8C4
U+5A8A.0	fGB	3-3D3E
U+5A8A.0	fR/S	 38.9
U+5A8A.0	fTotalStrokes	 12
U+5A8A.0	fCangjie	VTBN
U+5A8A.0	fFourCorner	48421
U+5A8A.0	fHakka	  cien2; zien5
U+5A8A.0	fCantonese	zin2; cin4
U+5A8A.0	fMandarin	qian2; jian1; jian3; jian4; zi1
U+5A8A.0	fEnglish	star
U+5A8B.0	fUTF8	媋
U+5A8B.0	fCNS11643	2-3B2B
U+5A8B.0	fBig5	D8C7
U+5A8B.0	fGB	5-402C
U+5A8B.0	fR/S	 38.9
U+5A8B.0	fTotalStrokes	 12
U+5A8B.0	fCangjie	VQKA
U+5A8B.0	fFourCorner	45463
U+5A8B.0	fCantonese	ceon1
U+5A8B.0	fMandarin	chun1
U+5A8B.0	fEnglish	used in a female name
U+5A8C.0	fUTF8	媌
U+5A8C.0	fCNS11643	2-3B2F
U+5A8C.0	fBig5	D8CB
U+5A8C.0	fGB	3-3C73
U+5A8C.0	fR/S	 38.9
U+5A8C.0	fTotalStrokes	 12
U+5A8C.0	fCangjie	VTW
U+5A8C.0	fFourCorner	44460
U+5A8C.0	fHakka	 mau2;  miau2; miau3; meu2
U+5A8C.0	fCantonese	maau4; miu4
U+5A8C.0	fMandarin	mao2; mao1; miao2
U+5A8C.0	fEnglish	beautiful; elegant
U+5A8D.0	fUTF8	媍
U+5A8D.0	fCNS11643	3-3A7B
U+5A8D.0	fGB	E-2728
U+5A8D.0	fR/S	 38.9
U+5A8D.0	fTotalStrokes	 12
U+5A8D.0	fCangjie	VNBC
U+5A8D.0	fFourCorner	47480
U+5A8D.0	fHakka	 fu5; fu6
U+5A8D.0	fCantonese	fu5
U+5A8D.0	fMandarin	fu4
U+5A8D.0	fEnglish	 married woman; woman; lady; matron
U+5A8E.0	fUTF8	媎
U+5A8E.0	fCNS11643	2-3466
U+5A8E.0	fBig5	D4E3
U+5A8E.0	fGB	5-3F7E
U+5A8E.0	fR/S	 38.9
U+5A8E.0	fTotalStrokes	 12
U+5A8E.0	fCangjie	VJKA
U+5A8E.0	fFourCorner	44460
U+5A8E.0	fCantonese	ze2
U+5A8E.0	fMandarin	jie3
U+5A8F.0	fUTF8	媏
U+5A8F.0	fCNS11643	2-3B31
U+5A8F.0	fBig5	D8CD
U+5A8F.0	fGB	5-4035
U+5A8F.0	fR/S	 38.9
U+5A8F.0	fTotalStrokes	 12
U+5A8F.0	fCangjie	VUMB
U+5A8F.0	fFourCorner	42427
U+5A8F.0	fCantonese	dyun1
U+5A8F.0	fMandarin	duan1
U+5A8F.0	fEnglish	used in a female name
U+5A90.0	fUTF8	媐
U+5A90.0	fCNS11643	2-424D
U+5A90.0	fBig5	DD47
U+5A90.0	fGB	3-3D44
U+5A90.0	fR/S	 38.10
U+5A90.0	fTotalStrokes	 13
U+5A90.0	fCangjie	SUV
U+5A90.0	fFourCorner	77401
U+5A90.0	fCantonese	hei1
U+5A90.0	fMandarin	xi1; yi2
U+5A91.0	fUTF8	媑
U+5A91.0	fCNS11643	3-3A66
U+5A91.0	fGB	5-4037
U+5A91.0	fR/S	  38.9
U+5A91.0	fTotalStrokes	 12
U+5A91.0	fCangjie	VHJG
U+5A91.0	fFourCorner	42410
U+5A91.0	fCantonese	zung6
U+5A91.0	fMandarin	zhong4
U+5A91.0	fEnglish	used in a female name
U+5A92.0	fUTF8	媒
U+5A92.0	fCNS11643	1-5E68
U+5A92.0	fBig5	B443
U+5A92.0	fGB	0-433D
U+5A92.0	fR/S	 38.9
U+5A92.0	fTotalStrokes	 12
U+5A92.0	fCangjie	VTMD
U+5A92.0	fFourCorner	44494
U+5A92.0	fHakka	  moi2; moi3; moe2
U+5A92.0	fCantonese	mui4; mui2 (also)
U+5A92.0	fMandarin	mei2; mei4; meng2
U+5A92.0	fEnglish	[1] marriage go-between; matchmaker [2] medium
U+5A93.0	fUTF8	媓
U+5A93.0	fCNS11643	2-3B32
U+5A93.0	fBig5	D8CE
U+5A93.0	fGB	3-3D38
U+5A93.0	fR/S	 38.9
U+5A93.0	fTotalStrokes	 12
U+5A93.0	fCangjie	VHAG
U+5A93.0	fFourCorner	46414
U+5A93.0	fHakka	 fong2
U+5A93.0	fCantonese	wong4
U+5A93.0	fMandarin	huang2
U+5A93.0	fEnglish	name of a concubine of the Emperor Yao
U+5A94.0	fUTF8	媔
U+5A94.0	fCNS11643	2-3A78
U+5A94.0	fBig5	D8B6
U+5A94.0	fGB	3-3D30
U+5A94.0	fR/S	 38.9
U+5A94.0	fTotalStrokes	 12
U+5A94.0	fCangjie	VMWL
U+5A94.0	fFourCorner	41460
U+5A94.0	fCantonese	min4
U+5A94.0	fMandarin	mian2; mian3
U+5A95.0	fUTF8	媕
U+5A95.0	fCNS11643	2-3B24
U+5A95.0	fBig5	D8C0
U+5A95.0	fGB	3-3D3A
U+5A95.0	fR/S	 38.9
U+5A95.0	fTotalStrokes	 12
U+5A95.0	fCangjie	VORT
U+5A95.0	fFourCorner	48446
U+5A95.0	fCantonese	am1; ngam1 (also); om1
U+5A95.0	fMandarin	an1
U+5A95.0	fEnglish	undecided
U+5A96.0	fUTF8	媖
U+5A96.0	fCNS11643	3-3A62
U+5A96.0	fGB	3-3C74
U+5A96.0	fR/S	  38.9
U+5A96.0	fTotalStrokes	 12
U+5A96.0	fCangjie	VTLK
U+5A96.0	fFourCorner	44430
U+5A96.0	fCantonese	jing1
U+5A96.0	fMandarin	ying1
U+5A96.0	fEnglish	 beautiful woman; beautiful
U+5A97.0	fUTF8	媗
U+5A97.0	fCNS11643	2-3B29
U+5A97.0	fBig5	D8C5
U+5A97.0	fGB	5-403E
U+5A97.0	fR/S	 38.9
U+5A97.0	fTotalStrokes	 12
U+5A97.0	fCangjie	VJMM
U+5A97.0	fFourCorner	43416
U+5A97.0	fCantonese	hyun1
U+5A97.0	fMandarin	xuan1; xuan3
U+5A97.0	fEnglish	used in a female name
U+5A98.0	fUTF8	媘
U+5A98.0	fCNS11643	3-3A5C
U+5A98.0	fGB	E-2729
U+5A98.0	fR/S	  38.9
U+5A98.0	fTotalStrokes	 12
U+5A98.0	fCangjie	VPPA
U+5A98.0	fFourCorner	41460
U+5A98.0	fCantonese	gaai1
U+5A98.0	fMandarin	jie1
U+5A98.0	fEnglish	used in a female name
U+5A99.0	fUTF8	媙
U+5A99.0	fCNS11643	3-3A5E
U+5A99.0	fGB	E-272A
U+5A99.0	fR/S	 38.9
U+5A99.0	fTotalStrokes	 12
U+5A99.0	fCangjie	VIHV
U+5A99.0	fFourCorner	43450
U+5A99.0	fCantonese	wai1
U+5A99.0	fMandarin	wei1
U+5A9A.0	fUTF8	媚
U+5A9A.0	fCNS11643	1-5E66
U+5A9A.0	fBig5	B441
U+5A9A.0	fGB	0-4344
U+5A9A.0	fR/S	 38.9
U+5A9A.0	fTotalStrokes	 12
U+5A9A.0	fCangjie	VAHU
U+5A9A.0	fFourCorner	47467
U+5A9A.0	fHakka	 mi5; mi2; mi6
U+5A9A.0	fCantonese	mei6; mei4 (also)
U+5A9A.0	fMandarin	mei4
U+5A9A.0	fEnglish	[1] charming; attractive; fascinating; seductive [2] fawn on; flatter [3] please [4] love
U+5A9B.0	fUTF8	媛
U+5A9B.0	fCNS11643	1-5E69
U+5A9B.0	fBig5	B444
U+5A9B.0	fGB	0-6642
U+5A9B.0	fR/S	 38.9
U+5A9B.0	fTotalStrokes	 12
U+5A9B.0	fCangjie	VBME
U+5A9B.0	fFourCorner	42447
U+5A9B.0	fHakka	 jen5; ren3; jen3; jen2; jan2
U+5A9B.0	fCantonese	jyun6; jyun4; wun4 (also)
U+5A9B.0	fMandarin	yuan4; yuan2; yung4
U+5A9B.0	fEnglish	[1] beauty; beautiful woman [2] [Southern dialects] young lady; miss
U+5A9C.0	fUTF8	媜
U+5A9C.0	fCNS11643	2-3B30
U+5A9C.0	fBig5	D8CC
U+5A9C.0	fGB	3-3C4C
U+5A9C.0	fR/S	 38.9
U+5A9C.0	fTotalStrokes	 12
U+5A9C.0	fCangjie	VYBC
U+5A9C.0	fFourCorner	51486
U+5A9C.0	fCantonese	zing1
U+5A9C.0	fMandarin	zhen1
U+5A9C.0	fEnglish	used in a female name
U+5A9D.0	fUTF8	媝
U+5A9D.0	fCNS11643	2-3B33
U+5A9D.0	fBig5	D8CF
U+5A9D.0	fGB	5-4036
U+5A9D.0	fR/S	 38.9
U+5A9D.0	fTotalStrokes	 12
U+5A9D.0	fCangjie	HFV
U+5A9D.0	fFourCorner	29404
U+5A9D.0	fCantonese	cau1
U+5A9D.0	fMandarin	qiu1
U+5A9D.0	fEnglish	used in a female name
U+5A9E.0	fUTF8	媞
U+5A9E.0	fCNS11643	2-3A7C
U+5A9E.0	fBig5	D8BA
U+5A9E.0	fGB	3-3D31
U+5A9E.0	fR/S	 38.9
U+5A9E.0	fTotalStrokes	 12
U+5A9E.0	fCangjie	VAMO
U+5A9E.0	fFourCorner	46481
U+5A9E.0	fHakka	 ti2; di2; ti1
U+5A9E.0	fCantonese	tai4; tai5
U+5A9E.0	fMandarin	ti2; di4; shi4; zhi2
U+5A9E.0	fEnglish	at ease
U+5A9F.0	fUTF8	媟
U+5A9F.0	fCNS11643	2-3A79
U+5A9F.0	fBig5	D8B7
U+5A9F.0	fGB	5-402F
U+5A9F.0	fR/S	 38.9
U+5A9F.0	fTotalStrokes	 12
U+5A9F.0	fCangjie	VPTD
U+5A9F.0	fFourCorner	44491
U+5A9F.0	fHakka	  siet7
U+5A9F.0	fCantonese	sit3
U+5A9F.0	fMandarin	xie4
U+5A9F.0	fEnglish	 lust after; act indecently lewd
U+5AA0.0	fUTF8	媠
U+5AA0.0	fCNS11643	3-3A73
U+5AA0.0	fGB	3-3D2F
U+5AA0.0	fR/S	  38.9
U+5AA0.0	fTotalStrokes	 12
U+5AA0.0	fCangjie	VKMB
U+5AA0.0	fFourCorner	44420
U+5AA0.0	fHakka	 to5; to6; to3
U+5AA0.0	fCantonese	to5; do6
U+5AA0.0	fMandarin	tuo3; duo4
U+5AA0.0	fEnglish	[1] fascinating; beautiful [2] wanting in respect
U+5AA1.0	fUTF8	媡
U+5AA1.0	fCNS11643	3-3A5A
U+5AA1.0	fGB	E-272B
U+5AA1.0	fR/S	 38.9
U+5AA1.0	fTotalStrokes	 12
U+5AA1.0	fCangjie	VDWF
U+5AA1.0	fFourCorner	45490
U+5AA1.0	fCantonese	lin6
U+5AA1.0	fMandarin	lian4
U+5AA1.0	fEnglish	used in a female name
U+5AA2.0	fUTF8	媢
U+5AA2.0	fCNS11643	2-3A7B
U+5AA2.0	fBig5	D8B9
U+5AA2.0	fGB	3-3D32
U+5AA2.0	fR/S	 38.9
U+5AA2.0	fTotalStrokes	 12
U+5AA2.0	fCangjie	VABU
U+5AA2.0	fFourCorner	46460
U+5AA2.0	fHakka	 mau5; mo6; mo5; mau6
U+5AA2.0	fCantonese	mou6
U+5AA2.0	fMandarin	mao4; mao3; mei4; mo4
U+5AA2.0	fEnglish	be jealous of; be envious
U+5AA3.0	fUTF8	媣
U+5AA3.0	fCNS11643	3-3A57
U+5AA3.0	fGB	3-3D40
U+5AA3.0	fR/S	  38.9
U+5AA3.0	fTotalStrokes	 12
U+5AA3.0	fCangjie	XVEND
U+5AA3.0	fFourCorner	44490
U+5AA3.0	fCantonese	gam1
U+5AA3.0	fMandarin	ran3
U+5AA4.0	fUTF8	媤
U+5AA4.0	fCNS11643	3-3A7A
U+5AA4.0	fGB	1-7D51
U+5AA4.0	fR/S	 38.9
U+5AA4.0	fTotalStrokes	 12
U+5AA4.0	fCangjie	VWP
U+5AA4.0	fFourCorner	46430
U+5AA4.0	fCantonese	si1
U+5AA4.0	fMandarin	si1
U+5AA4.0	fEnglish	husband's family
U+5AA5.0	fUTF8	媥
U+5AA5.0	fCNS11643	2-3B22
U+5AA5.0	fBig5	D8BE
U+5AA5.0	fGB	5-403F
U+5AA5.0	fR/S	 38.9
U+5AA5.0	fTotalStrokes	 12
U+5AA5.0	fCangjie	VHSB
U+5AA5.0	fFourCorner	42427
U+5AA5.0	fCantonese	pin1
U+5AA5.0	fMandarin	pian1
U+5AA6.0	fUTF8	媦
U+5AA6.0	fCNS11643	2-3A7E
U+5AA6.0	fBig5	D8BC
U+5AA6.0	fGB	3-3D34
U+5AA6.0	fR/S	 38.9
U+5AA6.0	fTotalStrokes	 12
U+5AA6.0	fCangjie	VWB
U+5AA6.0	fFourCorner	46427
U+5AA6.0	fHakka	 wui5; wi5; vui5
U+5AA6.0	fCantonese	wai6
U+5AA6.0	fMandarin	wei4
U+5AA6.0	fEnglish	 younger sister
U+5AA7.0	fUTF8	媧
U+5AA7.0	fCNS11643	1-5E6A
U+5AA7.0	fBig5	B445
U+5AA7.0	fGB	1-6634
U+5AA7.0	fR/S	 38.9
U+5AA7.0	fTotalStrokes	 12
U+5AA7.0	fCangjie	VBBR
U+5AA7.0	fFourCorner	47427
U+5AA7.0	fHakka	 wo1; vo1
U+5AA7.0	fCantonese	wo1; gwaa1
U+5AA7.0	fMandarin	wa1; gua1; guai1
U+5AA7.0	fEnglish	the mythical sister and successor of Fu Hsi, the legendary emperor (2852-2738 B.C.)
U+5AA8.0	fUTF8	媨
U+5AA8.0	fCNS11643	3-3A56
U+5AA8.0	fGB	3-3D3F
U+5AA8.0	fR/S	 38.9
U+5AA8.0	fTotalStrokes	 12
U+5AA8.0	fCangjie	VTCW
U+5AA8.0	fFourCorner	48460
U+5AA8.0	fCantonese	cuk1
U+5AA8.0	fMandarin	jiu4
U+5AA9.0	fUTF8	媩
U+5AA9.0	fCNS11643	2-3B2C
U+5AA9.0	fBig5	D8C8
U+5AA9.0	fGB	5-4030
U+5AA9.0	fR/S	 38.9
U+5AA9.0	fTotalStrokes	 12
U+5AA9.0	fCangjie	VJRB
U+5AA9.0	fFourCorner	47420
U+5AA9.0	fCantonese	wu4
U+5AA9.0	fMandarin	hu2
U+5AA9.0	fEnglish	used in a female name
U+5AAA.0	fUTF8	媪
U+5AAA.0	fCNS11643	3-3A79
U+5AAA.0	fGB	0-6641
U+5AAA.0	fR/S	 38.9
U+5AAA.0	fTotalStrokes	 12
U+5AAA.0	fCangjie	VABT
U+5AAA.0	fFourCorner	46410
U+5AAA.0	fCantonese	ou2; ngou2 (also)
U+5AAA.0	fMandarin	ao3
U+5AAA.0	fEnglish	[1] old woman; general term for older women [2] lower-class woman [3] the goddess of earth
U+5AAB.0	fUTF8	媫
U+5AAB.0	fCNS11643	3-3A78
U+5AAB.0	fGB	E-272C
U+5AAB.0	fR/S	  38.9
U+5AAB.0	fTotalStrokes	 12
U+5AAB.0	fCangjie	VYSO
U+5AAB.0	fFourCorner	41480
U+5AAB.0	fCantonese	zit3
U+5AAC.0	fUTF8	媬
U+5AAC.0	fCNS11643	2-3B23
U+5AAC.0	fBig5	D8BF
U+5AAC.0	fGB	E-272D
U+5AAC.0	fR/S	 38.9
U+5AAC.0	fTotalStrokes	 12
U+5AAC.0	fCangjie	VORD
U+5AAC.0	fFourCorner	46494
U+5AAC.0	fCantonese	bou2
U+5AAC.0	fMandarin	bao3
U+5AAD.0	fUTF8	媭
U+5AAD.0	fGB	8-7B3B
U+5AAD.0	fR/S	 38.12; 38.9
U+5AAD.0	fTotalStrokes	 12
U+5AAD.0	fCangjie	HOV
U+5AAD.0	fCantonese	seoi1
U+5AAD.0	fMandarin	xu1
U+5AAE.0	fUTF8	媮
U+5AAE.0	fCNS11643	2-3B25
U+5AAE.0	fBig5	D8C1
U+5AAE.0	fGB	8-2D67
U+5AAE.0	fR/S	 38.9
U+5AAE.0	fTotalStrokes	 12
U+5AAE.0	fCangjie	VOMN
U+5AAE.0	fFourCorner	48421
U+5AAE.0	fHakka	 teu1; ji2; ri2; i2
U+5AAE.0	fCantonese	jyu4; tau1
U+5AAE.0	fMandarin	tou1; tou3; yu2
U+5AAE.0	fEnglish	 handsome; cheerful steal
U+5AAF.0	fUTF8	媯
U+5AAF.0	fCNS11643	2-3A77
U+5AAF.0	fBig5	D8B5
U+5AAF.0	fGB	E-272E
U+5AAF.0	fR/S	 38.9
U+5AAF.0	fTotalStrokes	 12
U+5AAF.0	fCangjie	VIKF
U+5AAF.0	fFourCorner	42427
U+5AAF.0	fCantonese	gwai1
U+5AAF.0	fMandarin	gui1; gui4
U+5AAF.0	fEnglish	family name
U+5AB0.0	fUTF8	媰
U+5AB0.0	fCNS11643	2-4241
U+5AB0.0	fBig5	DCFA
U+5AB0.0	fGB	3-3C3D
U+5AB0.0	fR/S	 38.10
U+5AB0.0	fTotalStrokes	 13
U+5AB0.0	fCangjie	VPUU
U+5AB0.0	fFourCorner	47427
U+5AB0.0	fHakka	 cih1; ze1
U+5AB0.0	fCantonese	co4; zau1
U+5AB0.0	fMandarin	zou1; chu2; zhou4; zhu4
U+5AB0.0	fEnglish	[1] pregnent woman [2] widow
U+5AB1.0	fUTF8	媱
U+5AB1.0	fCNS11643	2-423F
U+5AB1.0	fBig5	DCF8
U+5AB1.0	fGB	5-404F
U+5AB1.0	fR/S	 38.10
U+5AB1.0	fTotalStrokes	 13
U+5AB1.0	fCangjie	VBOU
U+5AB1.0	fFourCorner	47472
U+5AB1.0	fHakka	 jeu2; rau2; hiau2; hieu2; jau2
U+5AB1.0	fCantonese	jiu4
U+5AB1.0	fMandarin	yao2
U+5AB1.0	fEnglish	[1] handsome [2] make fun
U+5AB2.0	fUTF8	媲
U+5AB2.0	fCNS11643	1-6368
U+5AB2.0	fBig5	B742
U+5AB2.0	fGB	0-6647
U+5AB2.0	fR/S	 38.10
U+5AB2.0	fTotalStrokes	 13
U+5AB2.0	fCangjie	VHWP
U+5AB2.0	fFourCorner	46411
U+5AB2.0	fHakka	 pi1;  pi5; pi3; pi6
U+5AB2.0	fCantonese	pei3; bei2 (also)
U+5AB2.0	fMandarin	pi4; bi1; pi2
U+5AB2.0	fEnglish	 marry off; pair; match compare
U+5AB3.0	fUTF8	媳
U+5AB3.0	fCNS11643	1-6366
U+5AB3.0	fBig5	B740
U+5AB3.0	fGB	0-4F31
U+5AB3.0	fR/S	 38.10
U+5AB3.0	fTotalStrokes	 13
U+5AB3.0	fCangjie	VHUP
U+5AB3.0	fFourCorner	46430
U+5AB3.0	fHakka	  sit7
U+5AB3.0	fCantonese	sik1
U+5AB3.0	fMandarin	xi2
U+5AB3.0	fEnglish	 daughter-in-law
U+5AB4.0	fUTF8	媴
U+5AB4.0	fCNS11643	2-4249
U+5AB4.0	fBig5	DD43
U+5AB4.0	fGB	5-4047
U+5AB4.0	fR/S	 38.10
U+5AB4.0	fTotalStrokes	 13
U+5AB4.0	fCangjie	VGRV
U+5AB4.0	fFourCorner	44432
U+5AB4.0	fCantonese	jyun4
U+5AB4.0	fMandarin	yuan2
U+5AB4.0	fEnglish	female name
U+5AB5.0	fUTF8	媵
U+5AB5.0	fCNS11643	2-4240
U+5AB5.0	fBig5	DCF9
U+5AB5.0	fGB	0-6B74
U+5AB5.0	fR/S	 38.10
U+5AB5.0	fTotalStrokes	 13
U+5AB5.0	fCangjie	BFQV
U+5AB5.0	fFourCorner	79244
U+5AB5.0	fHakka	 jin3; rin3
U+5AB5.0	fCantonese	jing6; jan6
U+5AB5.0	fMandarin	ying4
U+5AB5.0	fEnglish	[1] escort ladies to the harem of a newly married wife [2] present as a gift
U+5AB6.0	fUTF8	媶
U+5AB6.0	fCNS11643	2-424A
U+5AB6.0	fBig5	DD44
U+5AB6.0	fGB	5-402E
U+5AB6.0	fR/S	 38.10
U+5AB6.0	fTotalStrokes	 13
U+5AB6.0	fCangjie	VTSJ
U+5AB6.0	fFourCorner	44441
U+5AB6.0	fCantonese	jung4
U+5AB6.0	fMandarin	rong2
U+5AB7.0	fUTF8	媷
U+5AB7.0	fCNS11643	2-4246
U+5AB7.0	fBig5	DD40
U+5AB7.0	fGB	5-404A
U+5AB7.0	fR/S	 38.10
U+5AB7.0	fTotalStrokes	 13
U+5AB7.0	fCangjie	VMVI
U+5AB7.0	fFourCorner	41443
U+5AB7.0	fHakka	 juk8
U+5AB7.0	fCantonese	juk6
U+5AB7.0	fMandarin	ru4
U+5AB8.0	fUTF8	媸
U+5AB8.0	fCNS11643	2-423E
U+5AB8.0	fBig5	DCF7
U+5AB8.0	fGB	0-664A
U+5AB8.0	fR/S	 38.10
U+5AB8.0	fTotalStrokes	 13
U+5AB8.0	fCangjie	VUMI
U+5AB8.0	fFourCorner	47436
U+5AB8.0	fHakka	 chi1; cih1; zhe3; ze3; chii1
U+5AB8.0	fCantonese	ci1
U+5AB8.0	fMandarin	chi1
U+5AB8.0	fEnglish	 ugly woman; ugly
U+5AB9.0	fUTF8	媹
U+5AB9.0	fCNS11643	2-424C
U+5AB9.0	fBig5	DD46
U+5AB9.0	fGB	5-4051
U+5AB9.0	fR/S	 38.10
U+5AB9.0	fTotalStrokes	 13
U+5AB9.0	fCangjie	VHHW
U+5AB9.0	fFourCorner	47462
U+5AB9.0	fCantonese	lau4
U+5AB9.0	fMandarin	liu2
U+5AB9.0	fEnglish	used in a female name
U+5ABA.0	fUTF8	媺
U+5ABA.0	fCNS11643	2-423D
U+5ABA.0	fBig5	DCF6
U+5ABA.0	fGB	5-404D
U+5ABA.0	fR/S	 38.10
U+5ABA.0	fTotalStrokes	 13
U+5ABA.0	fCangjie	VUUK
U+5ABA.0	fFourCorner	48440
U+5ABA.0	fHakka	 mi3
U+5ABA.0	fCantonese	mei5
U+5ABA.0	fMandarin	mei3
U+5ABA.0	fEnglish	beautiful
U+5ABB.0	fUTF8	媻
U+5ABB.0	fCNS11643	2-4244
U+5ABB.0	fBig5	DCFD
U+5ABB.0	fGB	5-404E
U+5ABB.0	fR/S	 38.10
U+5ABB.0	fTotalStrokes	 13
U+5ABB.0	fCangjie	HEV
U+5ABB.0	fFourCorner	27404
U+5ABB.0	fHakka	  pan2
U+5ABB.0	fCantonese	pun4; po4
U+5ABB.0	fMandarin	pan2; pan4; po2
U+5ABB.0	fEnglish	 move
U+5ABC.0	fUTF8	媼
U+5ABC.0	fCNS11643	1-6365
U+5ABC.0	fBig5	B6FE
U+5ABC.0	fGB	E-272F
U+5ABC.0	fR/S	 38.10; 38.9
U+5ABC.0	fTotalStrokes	 13
U+5ABC.0	fCangjie	VWOT
U+5ABC.0	fFourCorner	46417
U+5ABC.0	fHakka	  au3;  wun3; vun3
U+5ABC.0	fCantonese	ou2; ngou2 (also); wan2
U+5ABC.0	fMandarin	ao3; wen1; yun1; yun3; yun4
U+5ABC.0	fEnglish	[1] old woman; general term for older women [2] lower-class woman [3] the goddess of earth [4] same as &#x5ABC;
U+5ABD.0	fUTF8	媽
U+5ABD.0	fCNS11643	1-6364
U+5ABD.0	fBig5	B6FD
U+5ABD.0	fGB	1-4268
U+5ABD.0	fR/S	 38.10
U+5ABD.0	fTotalStrokes	 13
U+5ABD.0	fCangjie	VSQF
U+5ABD.0	fFourCorner	41427
U+5ABD.0	fHakka	  ma1; ma2; mu3
U+5ABD.0	fCantonese	maa1; maa2 (also); maa4 (also); maa5
U+5ABD.0	fMandarin	ma1; mu3
U+5ABD.0	fEnglish	[1] one's mother; mother; mama [2] woman servant
U+5ABD.1	fUTF8	媽
U+5ABD.1	fCNS11643	1-6364
U+5ABD.1	fBig5	B6FD
U+5ABD.1	fGB	1-4268
U+5ABD.1	fR/S	 38.10
U+5ABD.1	fTotalStrokes	 13
U+5ABD.1	fCangjie	VSQF
U+5ABD.1	fFourCorner	41427
U+5ABD.1	fMandarin	ma5
U+5ABE.0	fUTF8	媾
U+5ABE.0	fCNS11643	1-6363
U+5ABE.0	fBig5	B6FC
U+5ABE.0	fGB	0-6645
U+5ABE.0	fR/S	 38.10
U+5ABE.0	fTotalStrokes	 13
U+5ABE.0	fCangjie	VTTB
U+5ABE.0	fFourCorner	45457
U+5ABE.0	fHakka	 kieu5; giu5; geu5
U+5ABE.0	fCantonese	gau3; kau3 (also)
U+5ABE.0	fMandarin	gou4
U+5ABE.0	fEnglish	[1] marry; wed [2] negotiate peace; amity [3] couple; copulate
U+5ABF.0	fUTF8	媿
U+5ABF.0	fCNS11643	2-4242
U+5ABF.0	fBig5	DCFB
U+5ABF.0	fGB	E-2730
U+5ABF.0	fR/S	 38.10
U+5ABF.0	fTotalStrokes	 13
U+5ABF.0	fCangjie	VHUI
U+5ABF.0	fFourCorner	46413
U+5ABF.0	fHakka	 kui3; k'wui3
U+5ABF.0	fCantonese	gwai2; kwai3 (also); kwai5
U+5ABF.0	fMandarin	kui4; gui3
U+5ABF.0	fEnglish	ashamed; conscience-stricken
U+5AC0.0	fUTF8	嫀
U+5AC0.0	fCNS11643	2-4247
U+5AC0.0	fBig5	DD41
U+5AC0.0	fGB	5-4043
U+5AC0.0	fR/S	 38.10
U+5AC0.0	fTotalStrokes	 13
U+5AC0.0	fCangjie	VQKD
U+5AC0.0	fFourCorner	45494
U+5AC0.0	fCantonese	ceon4; san1
U+5AC0.0	fMandarin	qin2
U+5AC0.0	fEnglish	[1] used in a female name [2] name of a country
U+5AC1.0	fUTF8	嫁
U+5AC1.0	fCNS11643	1-6360
U+5AC1.0	fBig5	B6F9
U+5AC1.0	fGB	0-3C5E
U+5AC1.0	fR/S	 38.10
U+5AC1.0	fTotalStrokes	 13
U+5AC1.0	fCangjie	VJMO
U+5AC1.0	fFourCorner	43432
U+5AC1.0	fHakka	  ga5
U+5AC1.0	fCantonese	gaa3
U+5AC1.0	fMandarin	jia4
U+5AC1.0	fEnglish	[1] marry a man; get married (said of a woman); give a daughter in marriage; marry off a daughter [2] impute to another (blame, a crime, etc.)
U+5AC2.0	fUTF8	嫂
U+5AC2.0	fCNS11643	1-6367
U+5AC2.0	fBig5	B741
U+5AC2.0	fGB	0-4929
U+5AC2.0	fR/S	 38.10
U+5AC2.0	fTotalStrokes	 13
U+5AC2.0	fCangjie	VHXE
U+5AC2.0	fFourCorner	47447
U+5AC2.0	fHakka	  sau3; so3
U+5AC2.0	fCantonese	sou2
U+5AC2.0	fMandarin	sao3
U+5AC2.0	fEnglish	the wife of one's elder brother
U+5AC2.1	fUTF8	嫂
U+5AC2.1	fCNS11643	1-6367
U+5AC2.1	fBig5	B741
U+5AC2.1	fGB	0-4929
U+5AC2.1	fR/S	 38.10
U+5AC2.1	fTotalStrokes	 13
U+5AC2.1	fCangjie	VHXE
U+5AC2.1	fFourCorner	47447
U+5AC2.1	fMandarin	sao5
U+5AC3.0	fUTF8	嫃
U+5AC3.0	fCNS11643	3-4067
U+5AC3.0	fGB	5-4049
U+5AC3.0	fR/S	 38.10
U+5AC3.0	fTotalStrokes	 13
U+5AC3.0	fCangjie	VJBC
U+5AC3.0	fFourCorner	44480
U+5AC3.0	fCantonese	zan1
U+5AC3.0	fMandarin	zhen1
U+5AC3.0	fEnglish	used in a female name
U+5AC4.0	fUTF8	嫄
U+5AC4.0	fCNS11643	2-423B
U+5AC4.0	fBig5	DCF4
U+5AC4.0	fGB	5-404C
U+5AC4.0	fR/S	 38.10
U+5AC4.0	fTotalStrokes	 13
U+5AC4.0	fCangjie	VMHF
U+5AC4.0	fFourCorner	41496
U+5AC4.0	fHakka	  ngien2
U+5AC4.0	fCantonese	jyun4
U+5AC4.0	fMandarin	yuan2
U+5AC4.0	fEnglish	an ancient emperor's mother's name; Qiang Yuan; consort to Gu
U+5AC5.0	fUTF8	嫅
U+5AC5.0	fCNS11643	3-4066
U+5AC5.0	fGB	3-3D3C
U+5AC5.0	fR/S	  38.10
U+5AC5.0	fTotalStrokes	 13
U+5AC5.0	fCangjie	VTQM
U+5AC5.0	fFourCorner	48410
U+5AC5.0	fCantonese	ze1
U+5AC5.0	fMandarin	cha1
U+5AC6.0	fUTF8	嫆
U+5AC6.0	fCNS11643	2-4245
U+5AC6.0	fBig5	DCFE
U+5AC6.0	fGB	5-4054
U+5AC6.0	fR/S	 38.10
U+5AC6.0	fTotalStrokes	 13
U+5AC6.0	fCangjie	VJCR
U+5AC6.0	fFourCorner	43468
U+5AC6.0	fCantonese	jung4
U+5AC6.0	fMandarin	yong2
U+5AC6.0	fEnglish	used in a female name
U+5AC7.0	fUTF8	嫇
U+5AC7.0	fCNS11643	2-423A
U+5AC7.0	fBig5	DCF3
U+5AC7.0	fGB	3-3D48
U+5AC7.0	fR/S	 38.10
U+5AC7.0	fTotalStrokes	 13
U+5AC7.0	fCangjie	VBAC
U+5AC7.0	fFourCorner	47480
U+5AC7.0	fHakka	  men2
U+5AC7.0	fCantonese	ming4; mang4; ming5
U+5AC7.0	fMandarin	ming2; meng2
U+5AC7.0	fEnglish	young wife of sixteen
U+5AC8.0	fUTF8	嫈
U+5AC8.0	fCNS11643	2-4243
U+5AC8.0	fBig5	DCFC
U+5AC8.0	fGB	3-3C34
U+5AC8.0	fR/S	 38.10
U+5AC8.0	fTotalStrokes	 13
U+5AC8.0	fCangjie	FFBV
U+5AC8.0	fFourCorner	99404
U+5AC8.0	fHakka	 rin2; jin2; jin1
U+5AC8.0	fCantonese	jing1
U+5AC8.0	fMandarin	ying1; xing1; yang1; ying2; ying4
U+5AC8.0	fEnglish	lady
U+5AC9.0	fUTF8	嫉
U+5AC9.0	fCNS11643	1-6361
U+5AC9.0	fBig5	B6FA
U+5AC9.0	fGB	0-3C35
U+5AC9.0	fR/S	 38.10
U+5AC9.0	fTotalStrokes	 13
U+5AC9.0	fCangjie	VKOK
U+5AC9.0	fFourCorner	40434
U+5AC9.0	fHakka	 cit8
U+5AC9.0	fCantonese	zat6; zaat6
U+5AC9.0	fMandarin	ji2; zi4
U+5AC9.0	fEnglish	[1] jealous; jealousy; be jealous of; envious [2] hate; detest
U+5ACA.0	fUTF8	嫊
U+5ACA.0	fCNS11643	2-4248
U+5ACA.0	fBig5	DD42
U+5ACA.0	fGB	5-4045
U+5ACA.0	fR/S	 38.10
U+5ACA.0	fTotalStrokes	 13
U+5ACA.0	fCangjie	VQMF
U+5ACA.0	fFourCorner	45493
U+5ACA.0	fCantonese	sou3
U+5ACA.0	fMandarin	su4
U+5ACA.0	fEnglish	used in a female name
U+5ACB.0	fUTF8	嫋
U+5ACB.0	fCNS11643	2-423C
U+5ACB.0	fBig5	DCF5
U+5ACB.0	fGB	E-2731
U+5ACB.0	fR/S	 38.10
U+5ACB.0	fTotalStrokes	 13
U+5ACB.0	fCangjie	VNMM
U+5ACB.0	fFourCorner	47427
U+5ACB.0	fHakka	  ngiau3; niau3; ngiau1
U+5ACB.0	fCantonese	niu1; liu1 (also); niu5; liu5 (also)
U+5ACB.0	fMandarin	niao3; ni4; ruo4
U+5ACB.0	fEnglish	slender and delicate
U+5ACC.0	fUTF8	嫌
U+5ACC.0	fCNS11643	1-6362
U+5ACC.0	fBig5	B6FB
U+5ACC.0	fGB	0-4F53
U+5ACC.0	fR/S	 38.10
U+5ACC.0	fTotalStrokes	 13
U+5ACC.0	fCangjie	VTXC
U+5ACC.0	fFourCorner	48437
U+5ACC.0	fHakka	  hiam2; hiam3; hio2
U+5ACC.0	fCantonese	jim4
U+5ACC.0	fMandarin	xian2
U+5ACC.0	fEnglish	[1] hate; detest; dislike [2] ill will; grudge [3] suspect; suspicion [4] complain; reject; object
U+5ACD.0	fUTF8	嫍
U+5ACD.0	fCNS11643	2-424B
U+5ACD.0	fBig5	DD45
U+5ACD.0	fGB	E-2732
U+5ACD.0	fR/S	 38.10
U+5ACD.0	fTotalStrokes	 13
U+5ACD.0	fCangjie	VBHX
U+5ACD.0	fFourCorner	42477
U+5ACD.0	fCantonese	tou1
U+5ACD.0	fMandarin	tao1; yao3
U+5ACD.0	fEnglish	used in a female name
U+5ACE.0	fUTF8	嫎
U+5ACE.0	fCNS11643	3-4064
U+5ACE.0	fGB	3-3D46
U+5ACE.0	fR/S	 38.10
U+5ACE.0	fTotalStrokes	 13
U+5ACE.0	fCangjie	VYBS
U+5ACE.0	fFourCorner	40420
U+5ACE.0	fCantonese	pong4
U+5ACE.0	fMandarin	pang2
U+5ACF.0	fUTF8	嫏
U+5ACF.0	fCNS11643	3-3A6B
U+5ACF.0	fGB	5-4029
U+5ACF.0	fR/S	 38.9
U+5ACF.0	fTotalStrokes	 12
U+5ACF.0	fCangjie	VIIL
U+5ACF.0	fFourCorner	47420
U+5ACF.0	fCantonese	long4
U+5ACF.0	fMandarin	lang2
U+5ACF.0	fEnglish	the place where the Supreme stores his books library
U+5AD0.0	fUTF8	嫐
U+5AD0.0	fCNS11643	3-406A
U+5AD0.0	fGB	E-2733
U+5AD0.0	fR/S	  38.10
U+5AD0.0	fTotalStrokes	 13
U+5AD0.0	fCangjie	VWSV
U+5AD0.0	fFourCorner	44440
U+5AD0.0	fHakka	 liau6;  liau5
U+5AD0.0	fCantonese	nou5; lou5 (also); nat1
U+5AD0.0	fMandarin	nao3; niao3
U+5AD0.0	fEnglish	 frolic; play with flirt with
U+5AD1.0	fUTF8	嫑
U+5AD1.0	fCNS11643	3-6468
U+5AD1.0	fGB	E-2734
U+5AD1.0	fR/S	 38.10
U+5AD1.0	fTotalStrokes	 13
U+5AD1.0	fCangjie	MFMWV
U+5AD1.0	fFourCorner	1040
U+5AD1.0	fCantonese	biu3
U+5AD1.0	fMandarin	bao2; biao4
U+5AD1.0	fEnglish	 contraction of 'do not'
U+5AD2.0	fUTF8	嫒
U+5AD2.0	fGB	0-6648
U+5AD2.0	fR/S	 38.10
U+5AD2.0	fTotalStrokes	 13
U+5AD2.0	fCangjie	XVBBE
U+5AD2.0	fCantonese	oi3; ngoi3 (also); oi2 (also); ngoi2 (also)
U+5AD2.0	fMandarin	ai4
U+5AD2.0	fEnglish	simplified form of &#x5B21;
U+5AD3.0	fUTF8	嫓
U+5AD3.0	fCNS11643	3-6469
U+5AD3.0	fGB	E-2735
U+5AD3.0	fR/S	 38.10
U+5AD3.0	fTotalStrokes	 13
U+5AD3.0	fCangjie	VNBP
U+5AD3.0	fFourCorner	4741
U+5AD3.0	fCantonese	pei3
U+5AD3.0	fMandarin	pi4
U+5AD4.0	fUTF8	嫔
U+5AD4.0	fGB	0-6649
U+5AD4.0	fR/S	 38.10
U+5AD4.0	fTotalStrokes	 13
U+5AD4.0	fCangjie	VJOC
U+5AD4.0	fCantonese	pan4; ban3 (also)
U+5AD4.0	fMandarin	pin2
U+5AD4.0	fEnglish	simplified form of &#x5B2A;
U+5AD5.0	fUTF8	嫕
U+5AD5.0	fCNS11643	2-4955
U+5AD5.0	fBig5	E16E
U+5AD5.0	fGB	3-3D4C
U+5AD5.0	fR/S	 38.11
U+5AD5.0	fTotalStrokes	 14
U+5AD5.0	fCangjie	VSKP
U+5AD5.0	fFourCorner	41431
U+5AD5.0	fHakka	 ji5; ri5; i5
U+5AD5.0	fCantonese	ai3; ngai3 (also); ji3 (also)
U+5AD5.0	fMandarin	yi4
U+5AD5.0	fEnglish	 compiant; yielding easy-going
U+5AD6.0	fUTF8	嫖
U+5AD6.0	fCNS11643	1-684A
U+5AD6.0	fBig5	B9E2
U+5AD6.0	fGB	0-664E
U+5AD6.0	fR/S	 38.11
U+5AD6.0	fTotalStrokes	 14
U+5AD6.0	fCangjie	VMWF
U+5AD6.0	fFourCorner	41491
U+5AD6.0	fHakka	  piau2; peu2
U+5AD6.0	fCantonese	piu4; piu1; piu3 (also)
U+5AD6.0	fMandarin	piao2; piao4; biao1; piao1
U+5AD6.0	fEnglish	 patronise whorehouses; visit prostitutes; go whoring
U+5AD7.0	fUTF8	嫗
U+5AD7.0	fCNS11643	1-6849
U+5AD7.0	fBig5	B9E1
U+5AD7.0	fGB	1-657D
U+5AD7.0	fR/S	 38.11
U+5AD7.0	fTotalStrokes	 14
U+5AD7.0	fCangjie	VSRR
U+5AD7.0	fFourCorner	41416
U+5AD7.0	fHakka	  ki1; hi1; eu1
U+5AD7.0	fCantonese	jyu2; jyu3
U+5AD7.0	fMandarin	yu4; yu3; kou1
U+5AD7.0	fEnglish	 old woman
U+5AD8.0	fUTF8	嫘
U+5AD8.0	fCNS11643	1-684B
U+5AD8.0	fBig5	B9E3
U+5AD8.0	fGB	0-6650
U+5AD8.0	fR/S	 38.11
U+5AD8.0	fTotalStrokes	 14
U+5AD8.0	fCangjie	VWVF
U+5AD8.0	fFourCorner	46493
U+5AD8.0	fHakka	 lui1
U+5AD8.0	fCantonese	leoi4
U+5AD8.0	fMandarin	lei2; lei4
U+5AD8.0	fEnglish	a Chinese family name
U+5AD9.0	fUTF8	嫙
U+5AD9.0	fCNS11643	2-4961
U+5AD9.0	fBig5	E17A
U+5AD9.0	fGB	3-3D52
U+5AD9.0	fR/S	 38.11
U+5AD9.0	fTotalStrokes	 14
U+5AD9.0	fCangjie	VYSO
U+5AD9.0	fFourCorner	48481
U+5AD9.0	fCantonese	syun4
U+5AD9.0	fMandarin	xuan2; xuan4
U+5ADA.0	fUTF8	嫚
U+5ADA.0	fCNS11643	2-4957
U+5ADA.0	fBig5	E170
U+5ADA.0	fGB	3-3D4F
U+5ADA.0	fR/S	 38.11
U+5ADA.0	fTotalStrokes	 14
U+5ADA.0	fCangjie	VAWE
U+5ADA.0	fFourCorner	46447
U+5ADA.0	fHakka	 man5; man6
U+5ADA.0	fCantonese	maan6
U+5ADA.0	fMandarin	man4; man1; mian4
U+5ADA.0	fEnglish	[1] slight; despise; insult; affront [2] haughty; arrogant [3] slowly; negligently
U+5ADA.1	fUTF8	嫚
U+5ADA.1	fCNS11643	2-4957
U+5ADA.1	fBig5	E170
U+5ADA.1	fGB	3-3D4F
U+5ADA.1	fR/S	 38.11
U+5ADA.1	fTotalStrokes	 14
U+5ADA.1	fCangjie	VAWE
U+5ADA.1	fFourCorner	46447
U+5ADA.1	fMandarin	yuan1
U+5ADA.1	fEnglish	slender; tender
U+5ADB.0	fUTF8	嫛
U+5ADB.0	fCNS11643	2-495D
U+5ADB.0	fBig5	E176
U+5ADB.0	fGB	3-3D4B
U+5ADB.0	fR/S	 38.11
U+5ADB.0	fTotalStrokes	 14
U+5ADB.0	fCangjie	SEV
U+5ADB.0	fFourCorner	77404
U+5ADB.0	fHakka	 i1
U+5ADB.0	fCantonese	ji1
U+5ADB.0	fMandarin	yi1; yi4
U+5ADB.0	fEnglish	 compliant; yielding easy-going a newborn child
U+5ADC.0	fUTF8	嫜
U+5ADC.0	fCNS11643	2-4952
U+5ADC.0	fBig5	E16B
U+5ADC.0	fGB	0-6651
U+5ADC.0	fR/S	 38.11
U+5ADC.0	fTotalStrokes	 14
U+5ADC.0	fCangjie	VYTJ
U+5ADC.0	fFourCorner	40446
U+5ADC.0	fHakka	 zhong1; zong1
U+5ADC.0	fCantonese	zoeng1
U+5ADC.0	fMandarin	zhang1
U+5ADC.0	fEnglish	husband's father
U+5ADD.0	fUTF8	嫝
U+5ADD.0	fCNS11643	2-4960
U+5ADD.0	fBig5	E179
U+5ADD.0	fGB	E-2736
U+5ADD.0	fR/S	 38.11
U+5ADD.0	fTotalStrokes	 14
U+5ADD.0	fCangjie	VILE
U+5ADD.0	fFourCorner	40432
U+5ADD.0	fCantonese	hong1
U+5ADD.0	fMandarin	kang1
U+5ADE.0	fUTF8	嫞
U+5ADE.0	fCNS11643	2-495F
U+5ADE.0	fBig5	E178
U+5ADE.0	fGB	3-3D51
U+5ADE.0	fR/S	 38.11
U+5ADE.0	fTotalStrokes	 14
U+5ADE.0	fCangjie	VILB
U+5ADE.0	fFourCorner	40427
U+5ADE.0	fCantonese	sung4; cung1; jung4
U+5ADE.0	fMandarin	yong2; chong1; chong2
U+5ADF.0	fUTF8	嫟
U+5ADF.0	fCNS11643	2-4963
U+5ADF.0	fBig5	E17C
U+5ADF.0	fGB	5-4046
U+5ADF.0	fR/S	 38.11
U+5ADF.0	fTotalStrokes	 14
U+5ADF.0	fCangjie	VSTR
U+5ADF.0	fFourCorner	41416
U+5ADF.0	fHakka	 nit8
U+5ADF.0	fCantonese	nik1; lik1 (also)
U+5ADF.0	fMandarin	ni4
U+5AE0.0	fUTF8	嫠
U+5AE0.0	fCNS11643	2-495C
U+5AE0.0	fBig5	E175
U+5AE0.0	fGB	0-664B
U+5AE0.0	fR/S	 38.11
U+5AE0.0	fTotalStrokes	 14
U+5AE0.0	fCangjie	JKMV
U+5AE0.0	fFourCorner	58244
U+5AE0.0	fHakka	  li2
U+5AE0.0	fCantonese	lei4
U+5AE0.0	fMandarin	li2
U+5AE0.0	fEnglish	 widow
U+5AE1.0	fUTF8	嫡
U+5AE1.0	fCNS11643	1-6846
U+5AE1.0	fBig5	B9DE
U+5AE1.0	fGB	0-3555
U+5AE1.0	fR/S	 38.11
U+5AE1.0	fTotalStrokes	 14
U+5AE1.0	fCangjie	VYCB
U+5AE1.0	fFourCorner	40427
U+5AE1.0	fHakka	  dit7
U+5AE1.0	fCantonese	dik1
U+5AE1.0	fMandarin	di2; zu2
U+5AE1.0	fEnglish	[1] legal wife [2] child of a legal wife; sons born of the legal wife
U+5AE2.0	fUTF8	嫢
U+5AE2.0	fCNS11643	2-495B
U+5AE2.0	fBig5	E174
U+5AE2.0	fGB	3-3C6E
U+5AE2.0	fR/S	 38.11
U+5AE2.0	fTotalStrokes	 14
U+5AE2.0	fCangjie	QUV
U+5AE2.0	fFourCorner	56404
U+5AE2.0	fHakka	 gui1; kwui1
U+5AE2.0	fCantonese	kwai1
U+5AE2.0	fMandarin	gui1
U+5AE2.0	fEnglish	a woman with a small waist
U+5AE3.0	fUTF8	嫣
U+5AE3.0	fCNS11643	1-684C
U+5AE3.0	fBig5	B9E4
U+5AE3.0	fGB	0-664C
U+5AE3.0	fR/S	 38.11
U+5AE3.0	fTotalStrokes	 14
U+5AE3.0	fCangjie	VMYF
U+5AE3.0	fFourCorner	41427
U+5AE3.0	fHakka	 jen1; ren1; jan1
U+5AE3.0	fCantonese	jin1
U+5AE3.0	fMandarin	yan1; xian1; yan3; yan4
U+5AE3.0	fEnglish	charming; fascinating; captivating; lovely
U+5AE4.0	fUTF8	嫤
U+5AE4.0	fCNS11643	3-464C
U+5AE4.0	fGB	3-3D49
U+5AE4.0	fR/S	 38.11
U+5AE4.0	fTotalStrokes	 14
U+5AE4.0	fCangjie	VTLM
U+5AE4.0	fFourCorner	44410
U+5AE4.0	fCantonese	gan2
U+5AE4.0	fMandarin	jin4
U+5AE5.0	fUTF8	嫥
U+5AE5.0	fCNS11643	2-4954
U+5AE5.0	fBig5	E16D
U+5AE5.0	fGB	3-3C26
U+5AE5.0	fR/S	 38.11
U+5AE5.0	fTotalStrokes	 14
U+5AE5.0	fCangjie	VJII
U+5AE5.0	fFourCorner	45443
U+5AE5.0	fHakka	 zhon1; zon1
U+5AE5.0	fCantonese	zyun1
U+5AE5.0	fMandarin	zhuan1; tuan2
U+5AE5.0	fEnglish	[1] gentle [2] beautiful
U+5AE6.0	fUTF8	嫦
U+5AE6.0	fCNS11643	1-6847
U+5AE6.0	fBig5	B9DF
U+5AE6.0	fGB	0-664F
U+5AE6.0	fR/S	 38.11
U+5AE6.0	fTotalStrokes	 14
U+5AE6.0	fCangjie	VFBB
U+5AE6.0	fFourCorner	49427
U+5AE6.0	fHakka	 shong2; song2; shong3
U+5AE6.0	fCantonese	soeng4
U+5AE6.0	fMandarin	chang2; heng2
U+5AE6.0	fEnglish	the name of a moon goddess
U+5AE7.0	fUTF8	嫧
U+5AE7.0	fCNS11643	3-464A
U+5AE7.0	fGB	3-3C6D
U+5AE7.0	fR/S	  38.11
U+5AE7.0	fTotalStrokes	 14
U+5AE7.0	fCangjie	VQMC
U+5AE7.0	fFourCorner	45480
U+5AE7.0	fCantonese	zaak3
U+5AE7.0	fMandarin	ce4
U+5AE8.0	fUTF8	嫨
U+5AE8.0	fCNS11643	2-4962
U+5AE8.0	fBig5	E17B
U+5AE8.0	fGB	3-3D4A
U+5AE8.0	fR/S	 38.11
U+5AE8.0	fTotalStrokes	 14
U+5AE8.0	fCangjie	VTLO
U+5AE8.0	fFourCorner	44434
U+5AE8.0	fCantonese	hon1; naan5
U+5AE8.0	fMandarin	han1; ran3
U+5AE9.0	fUTF8	嫩
U+5AE9.0	fCNS11643	1-6848
U+5AE9.0	fBig5	B9E0
U+5AE9.0	fGB	0-445B
U+5AE9.0	fR/S	 38.11
U+5AE9.0	fTotalStrokes	 14
U+5AE9.0	fCangjie	VDLK
U+5AE9.0	fFourCorner	48440
U+5AE9.0	fHakka	 nun6; lun3;  nun5
U+5AE9.0	fCantonese	nyun6; lyun6 (also)
U+5AE9.0	fMandarin	nen4; nun4
U+5AE9.0	fEnglish	[1] soft; tender; delicate [2] young; immature [3] light (of color)
U+5AEA.0	fUTF8	嫪
U+5AEA.0	fCNS11643	2-4956
U+5AEA.0	fBig5	E16F
U+5AEA.0	fGB	3-3D53
U+5AEA.0	fR/S	 38.11
U+5AEA.0	fTotalStrokes	 14
U+5AEA.0	fCangjie	VSMH
U+5AEA.0	fFourCorner	47422
U+5AEA.0	fHakka	 lau3; liau2; lau5; liau3
U+5AEA.0	fCantonese	lou6
U+5AEA.0	fMandarin	lao4; lao2
U+5AEA.0	fEnglish	hanker
U+5AEB.0	fUTF8	嫫
U+5AEB.0	fCNS11643	2-4959
U+5AEB.0	fBig5	E172
U+5AEB.0	fGB	0-6646
U+5AEB.0	fR/S	 38.11
U+5AEB.0	fTotalStrokes	 14
U+5AEB.0	fCangjie	VTAK
U+5AEB.0	fFourCorner	44434
U+5AEB.0	fHakka	 mu2; mo2
U+5AEB.0	fCantonese	mou4
U+5AEB.0	fMandarin	mo2
U+5AEB.0	fEnglish	Huangdi's ugly concubine; nurse
U+5AEC.0	fUTF8	嫬
U+5AEC.0	fCNS11643	2-495E
U+5AEC.0	fBig5	E177
U+5AEC.0	fGB	5-405C
U+5AEC.0	fR/S	 38.11
U+5AEC.0	fTotalStrokes	 14
U+5AEC.0	fCangjie	VITF
U+5AEC.0	fFourCorner	40437
U+5AEC.0	fCantonese	ze1
U+5AEC.0	fMandarin	zhe1; zhe5
U+5AED.0	fUTF8	嫭
U+5AED.0	fCNS11643	2-4958
U+5AED.0	fBig5	E171
U+5AED.0	fGB	3-3D4E
U+5AED.0	fR/S	 38.11
U+5AED.0	fTotalStrokes	 14
U+5AED.0	fCangjie	VYPD
U+5AED.0	fFourCorner	41449
U+5AED.0	fHakka	 fu5; fu6
U+5AED.0	fCantonese	wu6; fu6
U+5AED.0	fMandarin	hu4
U+5AED.0	fEnglish	graceful; beautiful
U+5AEE.0	fUTF8	嫮
U+5AEE.0	fCNS11643	2-4953
U+5AEE.0	fBig5	E16C
U+5AEE.0	fGB	5-4059
U+5AEE.0	fR/S	 38.11
U+5AEE.0	fTotalStrokes	 14
U+5AEE.0	fCangjie	VMBS
U+5AEE.0	fFourCorner	41427
U+5AEE.0	fHakka	 fu5; fu6
U+5AEE.0	fCantonese	wu6
U+5AEE.0	fMandarin	hu4
U+5AEE.0	fEnglish	a place name (Conf).
U+5AEF.0	fUTF8	嫯
U+5AEF.0	fCNS11643	3-464F
U+5AEF.0	fGB	5-4044
U+5AEF.0	fR/S	  38.11
U+5AEF.0	fTotalStrokes	 14
U+5AEF.0	fCangjie	GKV
U+5AEF.0	fFourCorner	48404
U+5AEF.0	fCantonese	ngou6; ou6 (also)
U+5AEF.0	fMandarin	ao4
U+5AF0.0	fUTF8	嫰
U+5AF0.0	fCNS11643	3-4653
U+5AF0.0	fGB	E-2737
U+5AF0.0	fR/S	 38.11
U+5AF0.0	fTotalStrokes	 14
U+5AF0.0	fCangjie	VDLO
U+5AF0.0	fFourCorner	47480
U+5AF0.0	fHakka	 lun3; nun5
U+5AF0.0	fCantonese	nyun6; lyun6 (also)
U+5AF0.0	fMandarin	nen4; nun4
U+5AF1.0	fUTF8	嫱
U+5AF1.0	fGB	0-664D
U+5AF1.0	fR/S	 38.11
U+5AF1.0	fTotalStrokes	 14
U+5AF1.0	fCangjie	VGCW
U+5AF1.0	fCantonese	coeng4
U+5AF1.0	fMandarin	qiang2
U+5AF1.0	fEnglish	simplified form of &#x5B19;
U+5AF2.0	fUTF8	嫲
U+5AF2.0	fGB	1-7C3A
U+5AF2.0	fR/S	 38.11
U+5AF2.0	fTotalStrokes	 14
U+5AF2.0	fCangjie	VIDD
U+5AF2.0	fHakka	  ma2
U+5AF2.0	fCantonese	maa4
U+5AF2.0	fEnglish	 grandmother; grandma (father's mother)
U+5AF3.0	fUTF8	嫳
U+5AF3.0	fCNS11643	2-495A
U+5AF3.0	fBig5	E173
U+5AF3.0	fGB	5-405A
U+5AF3.0	fR/S	 38.12
U+5AF3.0	fTotalStrokes	 15
U+5AF3.0	fCangjie	FKV
U+5AF3.0	fFourCorner	98404
U+5AF3.0	fHakka	 pet7;  piet7
U+5AF3.0	fCantonese	pit3; pai6
U+5AF3.0	fMandarin	pie4; bi4; pie1
U+5AF3.0	fEnglish	irritable; hasty
U+5AF4.0	fUTF8	嫴
U+5AF4.0	fCNS11643	2-4F7C
U+5AF4.0	fBig5	E555
U+5AF4.0	fGB	5-4061
U+5AF4.0	fR/S	 38.12
U+5AF4.0	fTotalStrokes	 15
U+5AF4.0	fCangjie	VJRJ
U+5AF4.0	fFourCorner	44441
U+5AF4.0	fCantonese	gu1; gwu1 (also)
U+5AF4.0	fMandarin	gu1
U+5AF5.0	fUTF8	嫵
U+5AF5.0	fCNS11643	1-6C4B
U+5AF5.0	fBig5	BC61
U+5AF5.0	fGB	1-657C
U+5AF5.0	fR/S	 38.12
U+5AF5.0	fTotalStrokes	 15
U+5AF5.0	fCangjie	VOTF
U+5AF5.0	fFourCorner	48431
U+5AF5.0	fHakka	 wu3; vu3
U+5AF5.0	fCantonese	mou5; mou4 (also)
U+5AF5.0	fMandarin	wu3
U+5AF5.0	fEnglish	charming; enchanting
U+5AF6.0	fUTF8	嫶
U+5AF6.0	fCNS11643	2-5021
U+5AF6.0	fBig5	E558
U+5AF6.0	fGB	3-3D58
U+5AF6.0	fR/S	 38.12
U+5AF6.0	fTotalStrokes	 15
U+5AF6.0	fCangjie	VOGF
U+5AF6.0	fFourCorner	40431
U+5AF6.0	fCantonese	ciu4; ziu1 (also)
U+5AF6.0	fMandarin	qiao2; jiao2
U+5AF7.0	fUTF8	嫷
U+5AF7.0	fCNS11643	2-4F7E
U+5AF7.0	fBig5	E557
U+5AF7.0	fGB	5-405E
U+5AF7.0	fR/S	 38.12
U+5AF7.0	fTotalStrokes	 15
U+5AF7.0	fCangjie	VNLB
U+5AF7.0	fFourCorner	44427
U+5AF7.0	fHakka	 to5
U+5AF7.0	fCantonese	to5
U+5AF7.0	fMandarin	tuo3; duo4; tuo4
U+5AF8.0	fUTF8	嫸
U+5AF8.0	fCNS11643	2-5023
U+5AF8.0	fBig5	E55A
U+5AF8.0	fGB	3-3D5C
U+5AF8.0	fR/S	 38.12
U+5AF8.0	fTotalStrokes	 15
U+5AF8.0	fCangjie	VTGR
U+5AF8.0	fFourCorner	48461
U+5AF8.0	fCantonese	zin2
U+5AF8.0	fMandarin	zhan3
U+5AF9.0	fUTF8	嫹
U+5AF9.0	fCNS11643	2-5025
U+5AF9.0	fBig5	E55C
U+5AF9.0	fGB	E-2738
U+5AF9.0	fR/S	 38.12
U+5AF9.0	fTotalStrokes	 15
U+5AF9.0	fCangjie	VTLC
U+5AF9.0	fFourCorner	44486
U+5AF9.0	fCantonese	maau4
U+5AF9.0	fMandarin	mao2; mao1; mao3
U+5AFA.0	fUTF8	嫺
U+5AFA.0	fCNS11643	3-4B5C
U+5AFA.0	fGB	E-2739
U+5AFA.0	fR/S	  38.12
U+5AFA.0	fTotalStrokes	 15
U+5AFA.0	fCangjie	VANB
U+5AFA.0	fFourCorner	47420
U+5AFA.0	fHakka	 han2
U+5AFA.0	fCantonese	haan4; han4
U+5AFA.0	fMandarin	xian2
U+5AFA.0	fEnglish	[1] refined; gracious [2] skillful; skilled
U+5AFB.0	fUTF8	嫻
U+5AFB.0	fCNS11643	1-6C49
U+5AFB.0	fBig5	BC5F
U+5AFB.0	fGB	1-6635
U+5AFB.0	fR/S	 38.12
U+5AFB.0	fTotalStrokes	 15
U+5AFB.0	fCangjie	VAND
U+5AFB.0	fFourCorner	47420
U+5AFB.0	fHakka	 han2
U+5AFB.0	fCantonese	haan4; han4
U+5AFB.0	fMandarin	xian2
U+5AFB.0	fEnglish	same as &#x5AFA;
U+5AFC.0	fUTF8	嫼
U+5AFC.0	fCNS11643	3-4B58
U+5AFC.0	fGB	3-3D57
U+5AFC.0	fR/S	  38.12
U+5AFC.0	fTotalStrokes	 15
U+5AFC.0	fCangjie	VWGF
U+5AFC.0	fFourCorner	46430
U+5AFC.0	fCantonese	aat3; ngaat3 (also)
U+5AFC.0	fMandarin	mo4
U+5AFD.0	fUTF8	嫽
U+5AFD.0	fCNS11643	2-4F7D
U+5AFD.0	fBig5	E556
U+5AFD.0	fGB	3-3D56
U+5AFD.0	fR/S	 38.12
U+5AFD.0	fTotalStrokes	 15
U+5AFD.0	fCangjie	VKCF
U+5AFD.0	fFourCorner	44496
U+5AFD.0	fHakka	 liau2; liau3; lau3
U+5AFD.0	fCantonese	liu4; liu5
U+5AFD.0	fMandarin	liao2; liao3; liao4
U+5AFD.0	fEnglish	play with
U+5AFE.0	fUTF8	嫾
U+5AFE.0	fCNS11643	3-4B54
U+5AFE.0	fGB	E-273A
U+5AFE.0	fR/S	 38.12
U+5AFE.0	fTotalStrokes	 15
U+5AFE.0	fCangjie	VFDQ
U+5AFE.0	fFourCorner	49450
U+5AFE.0	fCantonese	lin4
U+5AFE.0	fMandarin	lian2
U+5AFF.0	fUTF8	嫿
U+5AFF.0	fCNS11643	2-4F7B
U+5AFF.0	fBig5	E554
U+5AFF.0	fGB	3-3C75
U+5AFF.0	fR/S	 38.12
U+5AFF.0	fTotalStrokes	 15
U+5AFF.0	fCangjie	VLGM
U+5AFF.0	fFourCorner	45416
U+5AFF.0	fCantonese	waak6
U+5AFF.0	fMandarin	hua4
U+5AFF.0	fEnglish	tranquil
U+5B00.0	fUTF8	嬀
U+5B00.0	fCNS11643	3-4B5E
U+5B00.0	fGB	1-6623
U+5B00.0	fR/S	 38.12
U+5B00.0	fTotalStrokes	 15
U+5B00.0	fCangjie	VBHF
U+5B00.0	fFourCorner	42420
U+5B00.0	fCantonese	gwai1
U+5B00.0	fMandarin	gui1
U+5B00.0	fEnglish	name of a river in Shanxi crafty
U+5B01.0	fUTF8	嬁
U+5B01.0	fCNS11643	2-5026
U+5B01.0	fBig5	E55D
U+5B01.0	fGB	E-273B
U+5B01.0	fR/S	 38.12
U+5B01.0	fTotalStrokes	 15
U+5B01.0	fCangjie	VNOT
U+5B01.0	fFourCorner	47418
U+5B01.0	fCantonese	dang1
U+5B01.0	fMandarin	deng1
U+5B02.0	fUTF8	嬂
U+5B02.0	fCNS11643	2-5024
U+5B02.0	fBig5	E55B
U+5B02.0	fGB	5-3F53
U+5B02.0	fR/S	 38.12
U+5B02.0	fTotalStrokes	 15
U+5B02.0	fCangjie	VYIA
U+5B02.0	fFourCorner	43450
U+5B02.0	fCantonese	zik1
U+5B02.0	fMandarin	zhi1; zhi2
U+5B03.0	fUTF8	嬃
U+5B03.0	fCNS11643	2-5022
U+5B03.0	fBig5	E559
U+5B03.0	fGB	3-3D39
U+5B03.0	fR/S	 38.12
U+5B03.0	fTotalStrokes	 15
U+5B03.0	fCangjie	HCV
U+5B03.0	fFourCorner	21404
U+5B03.0	fCantonese	seoi1
U+5B03.0	fMandarin	xu1
U+5B03.0	fEnglish	a term of address for elder sisters in the state of Chu in ancient China
U+5B04.0	fUTF8	嬄
U+5B04.0	fCNS11643	3-4B56
U+5B04.0	fGB	E-273C
U+5B04.0	fR/S	  38.12
U+5B04.0	fTotalStrokes	 15
U+5B04.0	fCangjie	VGBT
U+5B04.0	fFourCorner	44410
U+5B04.0	fCantonese	jat1
U+5B04.0	fMandarin	yi1
U+5B05.0	fUTF8	嬅
U+5B05.0	fCNS11643	2-5028
U+5B05.0	fBig5	E55F
U+5B05.0	fGB	3-3C53
U+5B05.0	fR/S	 38.12
U+5B05.0	fTotalStrokes	 15
U+5B05.0	fCangjie	VTMJ
U+5B05.0	fFourCorner	44454
U+5B05.0	fCantonese	waa6
U+5B05.0	fMandarin	hua4; hua2
U+5B06.0	fUTF8	嬆
U+5B06.0	fCNS11643	3-4B5A
U+5B06.0	fGB	3-3D59
U+5B06.0	fR/S	 38.12
U+5B06.0	fTotalStrokes	 15
U+5B06.0	fCangjie	VORM
U+5B06.0	fFourCorner	48420
U+5B06.0	fCantonese	jap1
U+5B06.0	fMandarin	xi1
U+5B07.0	fUTF8	嬇
U+5B07.0	fCNS11643	2-5027
U+5B07.0	fBig5	E55E
U+5B07.0	fGB	3-3D35
U+5B07.0	fR/S	 38.12
U+5B07.0	fTotalStrokes	 15
U+5B07.0	fCangjie	VLMC
U+5B07.0	fFourCorner	45486
U+5B07.0	fHakka	 kui5; k'wui5
U+5B07.0	fCantonese	wui6; faai3; kui2; gwai3
U+5B07.0	fMandarin	hui4; kui4
U+5B07.0	fEnglish	woman from whose womb six sons were removed by a surgical operation
U+5B08.0	fUTF8	嬈
U+5B08.0	fCNS11643	1-6C4D
U+5B08.0	fBig5	BC63
U+5B08.0	fGB	1-662C
U+5B08.0	fR/S	 38.12
U+5B08.0	fTotalStrokes	 15
U+5B08.0	fCangjie	VGGU
U+5B08.0	fFourCorner	44412
U+5B08.0	fHakka	  ngiau2; neu1; ngieu2; liau2; ngiau1
U+5B08.0	fCantonese	jiu4; jiu2; jiu5; niu5
U+5B08.0	fMandarin	rao3; rao2; yao3; nao2; nao4; niao3; xiao4
U+5B08.0	fEnglish	graceful; charming; fascinating
U+5B09.0	fUTF8	嬉
U+5B09.0	fCNS11643	1-6C48
U+5B09.0	fBig5	BC5E
U+5B09.0	fGB	0-6652
U+5B09.0	fR/S	 38.12
U+5B09.0	fTotalStrokes	 15
U+5B09.0	fCangjie	VGRR
U+5B09.0	fFourCorner	44465
U+5B09.0	fHakka	  hi1
U+5B09.0	fCantonese	hei1; hei2
U+5B09.0	fMandarin	xi1; xi3; xi4
U+5B09.0	fEnglish	 enjoy; have fun; play; amuse oneself; sport; frolic
U+5B0A.0	fUTF8	嬊
U+5B0A.0	fCNS11643	3-4B5B
U+5B0A.0	fGB	E-273D
U+5B0A.0	fR/S	 38.12; 38.11; 140.11
U+5B0A.0	fTotalStrokes	 15
U+5B0A.0	fCangjie	TLPV
U+5B0A.0	fFourCorner	44400
U+5B0A.0	fCantonese	jin2
U+5B0A.0	fMandarin	yan4
U+5B0B.0	fUTF8	嬋
U+5B0B.0	fCNS11643	1-6C4A
U+5B0B.0	fBig5	BC60
U+5B0B.0	fGB	1-663F
U+5B0B.0	fR/S	 38.12
U+5B0B.0	fTotalStrokes	 15
U+5B0B.0	fCangjie	VRRJ
U+5B0B.0	fFourCorner	46456
U+5B0B.0	fHakka	 sham2; sam2; sham3
U+5B0B.0	fCantonese	sim4; sin4
U+5B0B.0	fMandarin	chan2
U+5B0B.0	fEnglish	graceful; beautiful; lovely; pretty; grace; attractive; lady-like
U+5B0C.0	fUTF8	嬌
U+5B0C.0	fCNS11643	1-6C4C
U+5B0C.0	fBig5	BC62
U+5B0C.0	fGB	1-3D3F
U+5B0C.0	fR/S	 38.12
U+5B0C.0	fTotalStrokes	 15
U+5B0C.0	fCangjie	VHKB
U+5B0C.0	fFourCorner	42427
U+5B0C.0	fHakka	  giau1; ziau2; geu1
U+5B0C.0	fCantonese	giu1
U+5B0C.0	fMandarin	jiao1; jiao3; qiao2
U+5B0C.0	fEnglish	[1] tender; delicate; beautiful; lovely [2] spoiled; pampered; coddled
U+5B0D.0	fUTF8	嬍
U+5B0D.0	fCNS11643	3-4B5D
U+5B0D.0	fGB	E-273E
U+5B0D.0	fR/S	 38.12
U+5B0D.0	fTotalStrokes	 15
U+5B0D.0	fCangjie	VUGK
U+5B0D.0	fFourCorner	48440
U+5B0D.0	fCantonese	mei5
U+5B0D.0	fMandarin	mei3
U+5B0E.0	fUTF8	嬎
U+5B0E.0	fCNS11643	3-4B5F
U+5B0E.0	fGB	3-3D5A
U+5B0E.0	fR/S	  38.12
U+5B0E.0	fTotalStrokes	 15
U+5B0E.0	fCangjie	VNUM
U+5B0E.0	fFourCorner	45410
U+5B0E.0	fHakka	 fan1; fu5; fu6; fan5
U+5B0E.0	fCantonese	faan3
U+5B0E.0	fMandarin	fan4
U+5B0E.0	fEnglish	 bear multiple identical offspring
U+5B0F.0	fUTF8	嬏
U+5B0F.0	fCNS11643	2-5029
U+5B0F.0	fBig5	E560
U+5B0F.0	fGB	5-4063
U+5B0F.0	fR/S	 38.12
U+5B0F.0	fTotalStrokes	 15
U+5B0F.0	fCangjie	VHDW
U+5B0F.0	fFourCorner	42469
U+5B0F.0	fCantonese	faan1
U+5B0F.0	fMandarin	fan1; fan2; pan1
U+5B10.0	fUTF8	嬐
U+5B10.0	fCNS11643	2-5661
U+5B10.0	fBig5	E957
U+5B10.0	fGB	3-3C66
U+5B10.0	fR/S	 38.13
U+5B10.0	fTotalStrokes	 16
U+5B10.0	fCangjie	VOMO
U+5B10.0	fFourCorner	48486
U+5B10.0	fCantonese	cim1
U+5B10.0	fMandarin	xian1; yan3; yin3
U+5B11.0	fUTF8	嬑
U+5B11.0	fCNS11643	3-5050
U+5B11.0	fGB	E-273F
U+5B11.0	fR/S	 38.13
U+5B11.0	fTotalStrokes	 16
U+5B11.0	fCangjie	VYTP
U+5B11.0	fFourCorner	40430
U+5B11.0	fCantonese	ji3
U+5B11.0	fMandarin	yi4
U+5B12.0	fUTF8	嬒
U+5B12.0	fCNS11643	3-5053
U+5B12.0	fGB	3-3C55
U+5B12.0	fR/S	  38.13
U+5B12.0	fTotalStrokes	 16
U+5B12.0	fCangjie	VOMA
U+5B12.0	fFourCorner	48460
U+5B12.0	fCantonese	wai3
U+5B12.0	fMandarin	wei4
U+5B13.0	fUTF8	嬓
U+5B13.0	fCNS11643	2-5660
U+5B13.0	fBig5	E956
U+5B13.0	fGB	E-2740
U+5B13.0	fR/S	 38.13
U+5B13.0	fTotalStrokes	 16
U+5B13.0	fCangjie	VHSK
U+5B13.0	fFourCorner	48440
U+5B13.0	fCantonese	giu3
U+5B13.0	fMandarin	jiao4; chan1
U+5B13.0	fEnglish	used in names
U+5B14.0	fUTF8	嬔
U+5B14.0	fCNS11643	2-565F
U+5B14.0	fBig5	E955
U+5B14.0	fGB	E-2741
U+5B14.0	fR/S	 38.13
U+5B14.0	fTotalStrokes	 16
U+5B14.0	fCangjie	VNUM
U+5B14.0	fFourCorner	45413
U+5B14.0	fCantonese	faan3; fu6
U+5B14.0	fMandarin	fu4; fan4
U+5B14.0	fEnglish	baby rabbit
U+5B15.0	fUTF8	嬕
U+5B15.0	fCNS11643	3-5051
U+5B15.0	fGB	5-3F5A
U+5B15.0	fR/S	  38.13
U+5B15.0	fTotalStrokes	 16
U+5B15.0	fCangjie	VWLJ
U+5B15.0	fFourCorner	46440
U+5B15.0	fCantonese	sik1
U+5B15.0	fMandarin	shi4
U+5B16.0	fUTF8	嬖
U+5B16.0	fCNS11643	2-5662
U+5B16.0	fBig5	E958
U+5B16.0	fGB	0-6654
U+5B16.0	fR/S	 38.13
U+5B16.0	fTotalStrokes	 16
U+5B16.0	fCangjie	SJV
U+5B16.0	fFourCorner	70404
U+5B16.0	fHakka	  pit7; bi5
U+5B16.0	fCantonese	pei3; bai3
U+5B16.0	fMandarin	bi4
U+5B16.0	fEnglish	[1] minion [2] enjoy the favor of a powerful person (said of people of mean birth)
U+5B17.0	fUTF8	嬗
U+5B17.0	fCNS11643	2-565B
U+5B17.0	fBig5	E951
U+5B17.0	fGB	0-6653
U+5B17.0	fR/S	 38.13
U+5B17.0	fTotalStrokes	 16
U+5B17.0	fCangjie	VYWM
U+5B17.0	fFourCorner	40416
U+5B17.0	fHakka	 tan1;  sham2; sam2
U+5B17.0	fCantonese	sin6
U+5B17.0	fMandarin	shan4; dan3; tan1
U+5B17.0	fEnglish	 change; be replaced
U+5B18.0	fUTF8	嬘
U+5B18.0	fCNS11643	3-504F
U+5B18.0	fGB	E-2742
U+5B18.0	fR/S	 38.13
U+5B18.0	fTotalStrokes	 16
U+5B18.0	fCangjie	VYTO
U+5B18.0	fFourCorner	48430
U+5B18.0	fCantonese	seoi6
U+5B18.0	fMandarin	sui4
U+5B19.0	fUTF8	嬙
U+5B19.0	fCNS11643	2-565C
U+5B19.0	fBig5	E952
U+5B19.0	fGB	1-664D
U+5B19.0	fR/S	 38.13
U+5B19.0	fTotalStrokes	 16
U+5B19.0	fCangjie	VGOW
U+5B19.0	fFourCorner	44461
U+5B19.0	fHakka	 siong2; ciong2
U+5B19.0	fCantonese	coeng4
U+5B19.0	fMandarin	qiang2; se4; xiang2
U+5B19.0	fEnglish	 court lady
U+5B1A.0	fUTF8	嬚
U+5B1A.0	fCNS11643	2-5664
U+5B1A.0	fBig5	E95A
U+5B1A.0	fGB	3-3D60
U+5B1A.0	fR/S	 38.13
U+5B1A.0	fTotalStrokes	 16
U+5B1A.0	fCangjie	VITC
U+5B1A.0	fFourCorner	40437
U+5B1A.0	fCantonese	lim5
U+5B1A.0	fMandarin	lian3; lian2
U+5B1B.0	fUTF8	嬛
U+5B1B.0	fCNS11643	2-565D
U+5B1B.0	fBig5	E953
U+5B1B.0	fGB	3-3D5F
U+5B1B.0	fR/S	 38.13
U+5B1B.0	fTotalStrokes	 16
U+5B1B.0	fCangjie	VWLV
U+5B1B.0	fFourCorner	46432
U+5B1B.0	fHakka	 kiun2; wan2; van2
U+5B1B.0	fCantonese	king4; waan4; hyun1; wan4
U+5B1B.0	fMandarin	huan2; qiong2; yuan1
U+5B1B.0	fEnglish	lonely; solitary
U+5B1B.1	fUTF8	嬛
U+5B1B.1	fCNS11643	2-565D
U+5B1B.1	fBig5	E953
U+5B1B.1	fGB	3-3D5F
U+5B1B.1	fR/S	 38.13
U+5B1B.1	fTotalStrokes	 16
U+5B1B.1	fCangjie	VWLV
U+5B1B.1	fFourCorner	46432
U+5B1B.1	fMandarin	xuan1
U+5B1B.1	fEnglish	fickle; frivolous
U+5B1C.0	fUTF8	嬜
U+5B1C.0	fCNS11643	3-5054
U+5B1C.0	fGB	3-3D61
U+5B1C.0	fR/S	 38.13
U+5B1C.0	fTotalStrokes	 16
U+5B1C.0	fCangjie	YOV
U+5B1C.0	fFourCorner	07400
U+5B1C.0	fCantonese	jam1
U+5B1C.0	fMandarin	xin1
U+5B1D.0	fUTF8	嬝
U+5B1D.0	fCNS11643	1-704E
U+5B1D.0	fBig5	BEC5
U+5B1D.0	fGB	E-2743
U+5B1D.0	fR/S	 38.13
U+5B1D.0	fTotalStrokes	 16
U+5B1D.0	fCangjie	VHAV
U+5B1D.0	fFourCorner	47427
U+5B1D.0	fCantonese	niu5; liu5 (also)
U+5B1D.0	fMandarin	niao3
U+5B1D.0	fEnglish	delicate; graceful
U+5B1E.0	fUTF8	嬞
U+5B1E.0	fCNS11643	2-5666
U+5B1E.0	fBig5	E95C
U+5B1E.0	fGB	5-4060
U+5B1E.0	fR/S	 38.13
U+5B1E.0	fTotalStrokes	 16
U+5B1E.0	fCangjie	VTHG
U+5B1E.0	fFourCorner	44414
U+5B1E.0	fCantonese	dung2
U+5B1E.0	fMandarin	dong3
U+5B1F.0	fUTF8	嬟
U+5B1F.0	fCNS11643	3-504E
U+5B1F.0	fGB	3-3B7E
U+5B1F.0	fR/S	 38.13
U+5B1F.0	fTotalStrokes	 16
U+5B1F.0	fCangjie	VTGI
U+5B1F.0	fFourCorner	48450
U+5B1F.0	fCantonese	ji6
U+5B1F.0	fMandarin	yi4
U+5B20.0	fUTF8	嬠
U+5B20.0	fCNS11643	2-5665
U+5B20.0	fBig5	E95B
U+5B20.0	fGB	5-4066
U+5B20.0	fR/S	 38.13
U+5B20.0	fTotalStrokes	 16
U+5B20.0	fCangjie	VRRD
U+5B20.0	fFourCorner	46494
U+5B20.0	fCantonese	caam2; cam1
U+5B20.0	fMandarin	can2
U+5B21.0	fUTF8	嬡
U+5B21.0	fCNS11643	2-565E
U+5B21.0	fBig5	E954
U+5B21.0	fGB	1-6648
U+5B21.0	fR/S	 38.13
U+5B21.0	fTotalStrokes	 16
U+5B21.0	fCangjie	VBBE
U+5B21.0	fFourCorner	42447
U+5B21.0	fHakka	 oi5
U+5B21.0	fCantonese	oi3; ngoi3 (also); oi2 (also); ngoi2 (also)
U+5B21.0	fMandarin	ai4
U+5B21.0	fEnglish	 daughter (a complimentary term referring to the daughter of the person one is speaking to)
U+5B22.0	fUTF8	嬢
U+5B22.0	fCNS11643	3-5056
U+5B22.0	fGB	E-2744
U+5B22.0	fR/S	 38.13
U+5B22.0	fTotalStrokes	 16
U+5B22.0	fCangjie	VYCV
U+5B22.0	fFourCorner	40430
U+5B22.0	fCantonese	noeng4; loeng4 (also)
U+5B22.0	fMandarin	niang2
U+5B22.0	fEnglish	 troubled; oppressed fat mother
U+5B23.0	fUTF8	嬣
U+5B23.0	fCNS11643	2-5C5D
U+5B23.0	fBig5	ECD1
U+5B23.0	fGB	3-3C41
U+5B23.0	fR/S	 38.14
U+5B23.0	fTotalStrokes	 17
U+5B23.0	fCangjie	VJPN
U+5B23.0	fFourCorner	43421
U+5B23.0	fCantonese	ning4; ling4 (also)
U+5B23.0	fMandarin	neng2; ning2
U+5B24.0	fUTF8	嬤
U+5B24.0	fCNS11643	1-7351
U+5B24.0	fBig5	C0A8
U+5B24.0	fGB	5-406E
U+5B24.0	fR/S	 38.14
U+5B24.0	fTotalStrokes	 17
U+5B24.0	fCangjie	VICI
U+5B24.0	fFourCorner	40432
U+5B24.0	fHakka	  mo2; mo3; mo1; ma1
U+5B24.0	fCantonese	maa1; maa4 (also); mo5
U+5B24.0	fMandarin	ma1
U+5B24.0	fEnglish	mother
U+5B25.0	fUTF8	嬥
U+5B25.0	fCNS11643	2-5C5B
U+5B25.0	fBig5	ECCF
U+5B25.0	fGB	3-3D63
U+5B25.0	fR/S	 38.14
U+5B25.0	fTotalStrokes	 17
U+5B25.0	fCangjie	VSMG
U+5B25.0	fFourCorner	47414
U+5B25.0	fHakka	 tiau1; tiau5; tiau2; tiau3
U+5B25.0	fCantonese	tiu5; tiu4
U+5B25.0	fMandarin	tiao3; diao4; tiao2; tiao1; zhuo1
U+5B25.0	fEnglish	[1] upright; honest [2] feminine; seductive
U+5B26.0	fUTF8	嬦
U+5B26.0	fCNS11643	2-5C60
U+5B26.0	fBig5	ECD4
U+5B26.0	fGB	5-3F74
U+5B26.0	fR/S	 38.14
U+5B26.0	fTotalStrokes	 17
U+5B26.0	fCangjie	VGNI
U+5B26.0	fFourCorner	44441
U+5B26.0	fCantonese	cau4
U+5B26.0	fMandarin	chou2; shou4
U+5B27.0	fUTF8	嬧
U+5B27.0	fCNS11643	2-5C5F
U+5B27.0	fBig5	ECD3
U+5B27.0	fGB	5-3F73
U+5B27.0	fR/S	 38.14
U+5B27.0	fTotalStrokes	 17
U+5B27.0	fCangjie	VLMT
U+5B27.0	fFourCorner	45417
U+5B27.0	fCantonese	zeon6
U+5B27.0	fMandarin	jin4
U+5B28.0	fUTF8	嬨
U+5B28.0	fCNS11643	2-5663
U+5B28.0	fBig5	E959
U+5B28.0	fGB	5-4069
U+5B28.0	fR/S	 38.14
U+5B28.0	fTotalStrokes	 17
U+5B28.0	fCangjie	VTVP
U+5B28.0	fFourCorner	44433
U+5B28.0	fCantonese	ci4
U+5B28.0	fMandarin	ci2
U+5B29.0	fUTF8	嬩
U+5B29.0	fCNS11643	3-5450
U+5B29.0	fGB	3-3B7B
U+5B29.0	fR/S	  38.14
U+5B29.0	fTotalStrokes	 17
U+5B29.0	fCangjie	VHXC
U+5B29.0	fFourCorner	47480
U+5B29.0	fCantonese	jyu4
U+5B29.0	fMandarin	yu2
U+5B2A.0	fUTF8	嬪
U+5B2A.0	fCNS11643	1-7350
U+5B2A.0	fBig5	C0A7
U+5B2A.0	fGB	1-6649
U+5B2A.0	fR/S	 38.14
U+5B2A.0	fTotalStrokes	 17
U+5B2A.0	fCangjie	VJMC
U+5B2A.0	fFourCorner	43486
U+5B2A.0	fHakka	 bin1
U+5B2A.0	fCantonese	pan4; ban3 (also)
U+5B2A.0	fMandarin	pin2
U+5B2A.0	fEnglish	[1] court lady; palace maid [2] be married to (said of a woman)
U+5B2B.0	fUTF8	嬫
U+5B2B.0	fCNS11643	3-544E
U+5B2B.0	fGB	E-2745
U+5B2B.0	fR/S	 38.14
U+5B2B.0	fTotalStrokes	 17
U+5B2B.0	fCangjie	VFFD
U+5B2B.0	fFourCorner	49490
U+5B2B.0	fCantonese	wing4
U+5B2B.0	fMandarin	yong2
U+5B2C.0	fUTF8	嬬
U+5B2C.0	fCNS11643	2-5C5E
U+5B2C.0	fBig5	ECD2
U+5B2C.0	fGB	3-3D62
U+5B2C.0	fR/S	 38.14
U+5B2C.0	fTotalStrokes	 17
U+5B2C.0	fCangjie	VMBB
U+5B2C.0	fFourCorner	41427
U+5B2C.0	fCantonese	jyu4
U+5B2C.0	fMandarin	xu1; ru2
U+5B2C.0	fEnglish	 mistress; concubine weak
U+5B2D.0	fUTF8	嬭
U+5B2D.0	fCNS11643	2-5C5A
U+5B2D.0	fBig5	ECCE
U+5B2D.0	fGB	E-2746
U+5B2D.0	fR/S	 38.14
U+5B2D.0	fTotalStrokes	 17
U+5B2D.0	fCangjie	VMFB
U+5B2D.0	fFourCorner	41427
U+5B2D.0	fHakka	  nai1; ni3; lai1; nai3
U+5B2D.0	fCantonese	naai5; laai5 (also); naai4 (also); laai4 (also)
U+5B2D.0	fMandarin	nai3; er3; ni3; ni4
U+5B2D.0	fEnglish	 milk suckle breasts
U+5B2E.0	fUTF8	嬮
U+5B2E.0	fCNS11643	2-5C62
U+5B2E.0	fBig5	ECD6
U+5B2E.0	fGB	3-3037
U+5B2E.0	fR/S	 38.14
U+5B2E.0	fTotalStrokes	 17
U+5B2E.0	fCangjie	MKV
U+5B2E.0	fFourCorner	71404
U+5B2E.0	fHakka	 ram5; jam5; jam3
U+5B2E.0	fCantonese	jim1; jim3 (also); jim2
U+5B2E.0	fMandarin	yan1; yan4; ye1
U+5B2E.0	fEnglish	amiable; lovable
U+5B2F.0	fUTF8	嬯
U+5B2F.0	fCNS11643	2-5C61
U+5B2F.0	fBig5	ECD5
U+5B2F.0	fGB	5-406C
U+5B2F.0	fR/S	 38.14
U+5B2F.0	fTotalStrokes	 17
U+5B2F.0	fCangjie	VGRG
U+5B2F.0	fFourCorner	44414
U+5B2F.0	fCantonese	toi4
U+5B2F.0	fMandarin	tai2
U+5B30.0	fUTF8	嬰
U+5B30.0	fCNS11643	1-734F
U+5B30.0	fBig5	C0A6
U+5B30.0	fGB	1-5324
U+5B30.0	fR/S	 38.14
U+5B30.0	fTotalStrokes	 17
U+5B30.0	fCangjie	BCV
U+5B30.0	fFourCorner	66404
U+5B30.0	fHakka	  jin1; rin1
U+5B30.0	fCantonese	jing1
U+5B30.0	fMandarin	ying1; ying4
U+5B30.0	fEnglish	 baby; infant; suckling
U+5B31.0	fUTF8	嬱
U+5B31.0	fCNS11643	3-5452
U+5B31.0	fGB	5-405D
U+5B31.0	fR/S	  38.14
U+5B31.0	fTotalStrokes	 17
U+5B31.0	fCangjie	ELV
U+5B31.0	fFourCorner	32400
U+5B31.0	fCantonese	cim3; cam4
U+5B31.0	fMandarin	can2; qian4
U+5B31.0	fEnglish	same as &#x371E;
U+5B32.0	fUTF8	嬲
U+5B32.0	fCNS11643	2-5C5C
U+5B32.0	fBig5	ECD0
U+5B32.0	fGB	0-6655
U+5B32.0	fR/S	 38.14
U+5B32.0	fTotalStrokes	 17
U+5B32.0	fCangjie	WSVWS
U+5B32.0	fFourCorner	66427
U+5B32.0	fHakka	  liau5; liau6; liau2; lau1
U+5B32.0	fCantonese	niu5; nau1; liu5 (also); lau1 (also); niu4
U+5B32.0	fMandarin	niao3; nao3
U+5B32.0	fEnglish	[1] frolic; play with; flirt with; dally with [2] angry
U+5B33.0	fUTF8	嬳
U+5B33.0	fCNS11643	3-544F
U+5B33.0	fGB	3-3D5E
U+5B33.0	fR/S	  38.14
U+5B33.0	fTotalStrokes	 17
U+5B33.0	fCangjie	VTOE
U+5B33.0	fFourCorner	44440
U+5B33.0	fCantonese	wok6
U+5B33.0	fMandarin	wo4
U+5B34.0	fUTF8	嬴
U+5B34.0	fCNS11643	1-704F
U+5B34.0	fBig5	BEC6
U+5B34.0	fGB	0-5978
U+5B34.0	fR/S	 38.13; 38.14
U+5B34.0	fTotalStrokes	 16
U+5B34.0	fCangjie	YNV
U+5B34.0	fFourCorner	00217
U+5B34.0	fHakka	 jin2; rin2; jin3; jang3
U+5B34.0	fCantonese	jing4
U+5B34.0	fMandarin	ying2
U+5B34.0	fEnglish	[1] win [2] have surplus [3] a Chinese family name [4] same as &#x218CD;
U+5B35.0	fUTF8	嬵
U+5B35.0	fCNS11643	3-5451
U+5B35.0	fGB	E-2747
U+5B35.0	fR/S	  38.14
U+5B35.0	fTotalStrokes	 17
U+5B35.0	fCangjie	VVFB
U+5B35.0	fFourCorner	46420
U+5B35.0	fCantonese	min4
U+5B35.0	fMandarin	mian2
U+5B36.0	fUTF8	嬶
U+5B36.0	fGB	E-2748
U+5B36.0	fR/S	 38.14
U+5B36.0	fTotalStrokes	 17
U+5B36.0	fCangjie	VHUL
U+5B36.0	fCantonese	bei6
U+5B36.0	fEnglish	a wife
U+5B37.0	fUTF8	嬷
U+5B37.0	fCNS11643	3-5453
U+5B37.0	fGB	0-6656
U+5B37.0	fR/S	 38.14
U+5B37.0	fTotalStrokes	 17
U+5B37.0	fCangjie	VIDI
U+5B37.0	fFourCorner	40430
U+5B37.0	fHakka	 mo1; ma1
U+5B37.0	fCantonese	maa1; maa4 (also)
U+5B37.0	fMandarin	ma1
U+5B37.0	fEnglish	mother
U+5B38.0	fUTF8	嬸
U+5B38.0	fCNS11643	1-7640
U+5B38.0	fBig5	C254
U+5B38.0	fGB	1-4974
U+5B38.0	fR/S	 38.15
U+5B38.0	fTotalStrokes	 18
U+5B38.0	fCangjie	VJHW
U+5B38.0	fFourCorner	43469
U+5B38.0	fHakka	  shim3; sim3; zim2; siim3
U+5B38.0	fCantonese	sam2
U+5B38.0	fMandarin	shen3
U+5B38.0	fEnglish	[1] aunt (the wife of one's father's younger brother) [2] sister-in-law (the wife of one's husband younger brother)
U+5B38.1	fUTF8	嬸
U+5B38.1	fCNS11643	1-7640
U+5B38.1	fBig5	C254
U+5B38.1	fGB	1-4974
U+5B38.1	fR/S	 38.15
U+5B38.1	fTotalStrokes	 18
U+5B38.1	fCangjie	VJHW
U+5B38.1	fFourCorner	43469
U+5B38.1	fMandarin	shen5
U+5B39.0	fUTF8	嬹
U+5B39.0	fCNS11643	3-5A31
U+5B39.0	fGB	3-3D66
U+5B39.0	fR/S	 38.16
U+5B39.0	fTotalStrokes	 19
U+5B39.0	fCangjie	VHXC
U+5B39.0	fFourCorner	47480
U+5B39.0	fCantonese	hing1
U+5B39.0	fMandarin	xing4
U+5B3A.0	fUTF8	嬺
U+5B3A.0	fCNS11643	3-5761
U+5B3A.0	fGB	5-406B
U+5B3A.0	fR/S	  38.15
U+5B3A.0	fTotalStrokes	 18
U+5B3A.0	fCangjie	VSRP
U+5B3A.0	fFourCorner	41430
U+5B3A.0	fCantonese	nik1; lik1 (also)
U+5B3A.0	fMandarin	ni4
U+5B3B.0	fUTF8	嬻
U+5B3B.0	fCNS11643	3-575E
U+5B3B.0	fGB	3-3C76
U+5B3B.0	fR/S	  38.15
U+5B3B.0	fTotalStrokes	 18
U+5B3B.0	fCangjie	VGWC
U+5B3B.0	fFourCorner	44480
U+5B3B.0	fHakka	 tuk8
U+5B3B.0	fCantonese	duk6
U+5B3B.0	fMandarin	du2
U+5B3C.0	fUTF8	嬼
U+5B3C.0	fCNS11643	2-614D
U+5B3C.0	fBig5	EFC1
U+5B3C.0	fGB	3-3C5A
U+5B3C.0	fR/S	 38.15
U+5B3C.0	fTotalStrokes	 18
U+5B3C.0	fCangjie	VHCN
U+5B3C.0	fFourCorner	42400
U+5B3C.0	fCantonese	lau5; lau6; liu5
U+5B3C.0	fMandarin	liu3; liu2; liu4
U+5B3D.0	fUTF8	嬽
U+5B3D.0	fCNS11643	2-6546
U+5B3D.0	fBig5	F1FA
U+5B3D.0	fGB	E-2749
U+5B3D.0	fR/S	 38.15
U+5B3D.0	fTotalStrokes	 18
U+5B3D.0	fCangjie	VWLK
U+5B3D.0	fFourCorner	42434
U+5B3D.0	fCantonese	bei6; jyun1
U+5B3D.0	fMandarin	yuan1; wan1
U+5B3E.0	fUTF8	嬾
U+5B3E.0	fCNS11643	2-6547
U+5B3E.0	fBig5	F1FB
U+5B3E.0	fGB	E-274A
U+5B3E.0	fR/S	 38.16
U+5B3E.0	fTotalStrokes	 19
U+5B3E.0	fCangjie	VDLC
U+5B3E.0	fFourCorner	47486
U+5B3E.0	fHakka	 lan2
U+5B3E.0	fCantonese	laan5
U+5B3E.0	fMandarin	lan3
U+5B3E.0	fEnglish	lazy; languid; listless
U+5B3F.0	fUTF8	嬿
U+5B3F.0	fCNS11643	2-6548
U+5B3F.0	fBig5	F1FC
U+5B3F.0	fGB	3-3D64
U+5B3F.0	fR/S	 38.16
U+5B3F.0	fTotalStrokes	 19
U+5B3F.0	fCangjie	VTLF
U+5B3F.0	fFourCorner	44431
U+5B3F.0	fCantonese	jin3 (also); jin2
U+5B3F.0	fMandarin	yan4; yan3
U+5B3F.0	fEnglish	lovely
U+5B40.0	fUTF8	孀
U+5B40.0	fCNS11643	1-7967
U+5B40.0	fBig5	C45C
U+5B40.0	fGB	0-6657
U+5B40.0	fR/S	 38.17
U+5B40.0	fTotalStrokes	 20
U+5B40.0	fCangjie	VMBU
U+5B40.0	fFourCorner	41463
U+5B40.0	fHakka	  song1
U+5B40.0	fCantonese	soeng1; sung1
U+5B40.0	fMandarin	shuang1; shuan1; shuang4
U+5B40.0	fEnglish	 widow
U+5B41.0	fUTF8	孁
U+5B41.0	fCNS11643	4-6334
U+5B41.0	fGB	E-274B
U+5B41.0	fR/S	 38.17; 173.12
U+5B41.0	fTotalStrokes	 20
U+5B41.0	fCangjie	MBRRV
U+5B41.0	fFourCorner	10400
U+5B41.0	fCantonese	ling4
U+5B41.0	fMandarin	ling2
U+5B42.0	fUTF8	孂
U+5B42.0	fCNS11643	3-5C36
U+5B42.0	fGB	3-3D68
U+5B42.0	fR/S	  38.17
U+5B42.0	fTotalStrokes	 20
U+5B42.0	fCangjie	VHAT
U+5B42.0	fFourCorner	48410
U+5B42.0	fCantonese	giu2
U+5B42.0	fMandarin	jiao3
U+5B43.0	fUTF8	孃
U+5B43.0	fCNS11643	1-7968
U+5B43.0	fBig5	C45D
U+5B43.0	fGB	8-7E59
U+5B43.0	fR/S	 38.17
U+5B43.0	fTotalStrokes	 20
U+5B43.0	fCangjie	VYRV
U+5B43.0	fFourCorner	40432
U+5B43.0	fHakka	  ngiong2; jong2; rong2; niong2
U+5B43.0	fCantonese	noeng4; loeng4 (also)
U+5B43.0	fMandarin	niang2; nang2; rang2; rang3
U+5B43.0	fEnglish	 troubled; oppressed fat mother
U+5B44.0	fUTF8	孄
U+5B44.0	fCNS11643	3-5C39
U+5B44.0	fGB	5-4064
U+5B44.0	fR/S	 38.17
U+5B44.0	fTotalStrokes	 20
U+5B44.0	fCangjie	VANW
U+5B44.0	fFourCorner	47420
U+5B44.0	fHakka	 lan2
U+5B44.0	fCantonese	laan5
U+5B44.0	fMandarin	lan3
U+5B44.0	fEnglish	same as &#x5B3E;
U+5B45.0	fUTF8	孅
U+5B45.0	fCNS11643	2-686F
U+5B45.0	fBig5	F443
U+5B45.0	fGB	3-3D69
U+5B45.0	fR/S	 38.17
U+5B45.0	fTotalStrokes	 20
U+5B45.0	fCangjie	VOIM
U+5B45.0	fFourCorner	43450
U+5B45.0	fHakka	  siam2
U+5B45.0	fCantonese	cim1
U+5B45.0	fMandarin	xian1; qian1
U+5B45.0	fEnglish	slender; thin; small; fine; delicate
U+5B46.0	fUTF8	孆
U+5B46.0	fCNS11643	3-5C3A
U+5B46.0	fGB	5-405B
U+5B46.0	fR/S	  38.17
U+5B46.0	fTotalStrokes	 20
U+5B46.0	fCangjie	VBCV
U+5B46.0	fFourCorner	46440
U+5B46.0	fHakka	  jin1; rin1
U+5B46.0	fCantonese	jing1
U+5B46.0	fMandarin	ying1; ying4
U+5B46.0	fEnglish	same as &#x5B30;
U+5B47.0	fUTF8	孇
U+5B47.0	fCNS11643	2-6B56
U+5B47.0	fBig5	F5C8
U+5B47.0	fGB	5-3F4E
U+5B47.0	fR/S	 38.18
U+5B47.0	fTotalStrokes	 21
U+5B47.0	fCangjie	VOGE
U+5B47.0	fFourCorner	40447
U+5B47.0	fCantonese	soeng1
U+5B47.0	fMandarin	shuang1
U+5B48.0	fUTF8	孈
U+5B48.0	fCNS11643	2-6B55
U+5B48.0	fBig5	F5C7
U+5B48.0	fGB	3-3D6B
U+5B48.0	fR/S	 38.18
U+5B48.0	fTotalStrokes	 21
U+5B48.0	fCangjie	VUOB
U+5B48.0	fFourCorner	42427
U+5B48.0	fCantonese	wai3; seoi1
U+5B48.0	fMandarin	shuai1; hua4; hui1; wei3; yi3
U+5B49.0	fUTF8	孉
U+5B49.0	fCNS11643	3-5E29
U+5B49.0	fGB	5-4072
U+5B49.0	fR/S	 38.18
U+5B49.0	fTotalStrokes	 21
U+5B49.0	fCangjie	VTRG
U+5B49.0	fFourCorner	44410
U+5B49.0	fCantonese	kyun4; gyun3
U+5B49.0	fMandarin	quan2; juan4
U+5B49.0	fEnglish	same as &#x5A58;
U+5B4A.0	fUTF8	孊
U+5B4A.0	fCNS11643	3-5F42
U+5B4A.0	fGB	3-3D6C
U+5B4A.0	fR/S	 38.19
U+5B4A.0	fTotalStrokes	 22
U+5B4A.0	fCangjie	VIDY
U+5B4A.0	fFourCorner	40410
U+5B4A.0	fCantonese	mei5
U+5B4A.0	fMandarin	mi3
U+5B4B.0	fUTF8	孋
U+5B4B.0	fCNS11643	2-6D49
U+5B4B.0	fBig5	F6DB
U+5B4B.0	fGB	3-3C61
U+5B4B.0	fR/S	 38.19
U+5B4B.0	fTotalStrokes	 22
U+5B4B.0	fCangjie	VMMP
U+5B4B.0	fFourCorner	41411
U+5B4B.0	fHakka	 li2
U+5B4B.0	fCantonese	lei4
U+5B4B.0	fMandarin	li2; li4
U+5B4B.0	fEnglish	a daughter of the Western barbarians captured by the Duke of Chin B.C. 672
U+5B4C.0	fUTF8	孌
U+5B4C.0	fCNS11643	2-6D4A
U+5B4C.0	fBig5	F6DC
U+5B4C.0	fGB	1-662E
U+5B4C.0	fR/S	 38.19
U+5B4C.0	fTotalStrokes	 22
U+5B4C.0	fCangjie	VFV
U+5B4C.0	fFourCorner	22401
U+5B4C.0	fHakka	  lan2; len2; lien5
U+5B4C.0	fCantonese	lyun5 (also); lyun2
U+5B4C.0	fMandarin	lian4; luan2; luan3; luan4
U+5B4C.0	fEnglish	[1] lovely; beautiful; handsome [2] docile; obedient
U+5B4D.0	fUTF8	孍
U+5B4D.0	fCNS11643	2-6F24
U+5B4D.0	fBig5	F7D5
U+5B4D.0	fGB	3-3C5B
U+5B4D.0	fR/S	 38.20
U+5B4D.0	fTotalStrokes	 23
U+5B4D.0	fCangjie	VRRK
U+5B4D.0	fFourCorner	46448
U+5B4D.0	fHakka	 ngiam2
U+5B4D.0	fCantonese	jim4; jim5
U+5B4D.0	fMandarin	yan2; li2; yan3
U+5B4E.0	fUTF8	孎
U+5B4E.0	fCNS11643	2-7035
U+5B4E.0	fBig5	F8A7
U+5B4E.0	fGB	3-3D5D
U+5B4E.0	fR/S	 38.21
U+5B4E.0	fTotalStrokes	 24
U+5B4E.0	fCangjie	VSEI
U+5B4E.0	fFourCorner	47427
U+5B4E.0	fHakka	 chuk7; zuk7; zhuk7
U+5B4E.0	fCantonese	zuk1; doek3; cuk1; suk6
U+5B4E.0	fMandarin	zhu3; chuo4
U+5B4E.0	fEnglish	respectful deportments of a wife
U+5B4F.0	fUTF8	孏
U+5B4F.0	fCNS11643	3-6073
U+5B4F.0	fGB	E-274C
U+5B4F.0	fR/S	 38.21
U+5B4F.0	fTotalStrokes	 24
U+5B4F.0	fCangjie	VTAW
U+5B4F.0	fFourCorner	44420
U+5B4F.0	fCantonese	laan5
U+5B4F.0	fMandarin	lan3
U+5B50.0	fUTF8	子
U+5B50.0	fCNS11643	1-444D
U+5B50.0	fBig5	A46C
U+5B50.0	fGB	0-5753
U+5B50.0	fR/S	 39.0
U+5B50.0	fTotalStrokes	 3
U+5B50.0	fCangjie	ND
U+5B50.0	fFourCorner	17407
U+5B50.0	fHakka	  zih3;  zu3; zii3
U+5B50.0	fCantonese	zi2; ci4
U+5B50.0	fMandarin	zi3; zi2; ci2
U+5B50.0	fEnglish	[1] offspring; child; son [2] seed; egg [3] the first of the twelve Terrestrial Branches [4] a rank of the nobility equivalent to a viscount [5] a designation used in speaking of or to a man in former times
U+5B50.1	fUTF8	子
U+5B50.1	fCNS11643	1-444D
U+5B50.1	fBig5	A46C
U+5B50.1	fGB	0-5753
U+5B50.1	fR/S	 39.0
U+5B50.1	fTotalStrokes	 3
U+5B50.1	fCangjie	ND
U+5B50.1	fFourCorner	17407
U+5B50.1	fMandarin	zi5
U+5B51.0	fUTF8	孑
U+5B51.0	fCNS11643	1-444E
U+5B51.0	fBig5	A46D
U+5B51.0	fGB	0-665D
U+5B51.0	fR/S	 39.0
U+5B51.0	fTotalStrokes	 3
U+5B51.0	fCangjie	NNM
U+5B51.0	fFourCorner	17407
U+5B51.0	fHakka	  giet7; get7
U+5B51.0	fCantonese	kit3; git3
U+5B51.0	fMandarin	jie2
U+5B51.0	fEnglish	[1] larvae of mosquitoes [2] lonely; solitary; unaccompanied
U+5B52.0	fUTF8	孒
U+5B52.0	fCNS11643	4-2132
U+5B52.0	fGB	E-274D
U+5B52.0	fR/S	 39.0
U+5B52.0	fTotalStrokes	 3
U+5B52.0	fCangjie	NNM
U+5B52.0	fFourCorner	17210
U+5B52.0	fCantonese	kyut3
U+5B52.0	fMandarin	jue2
U+5B52.0	fEnglish	larvae of the mosquito
U+5B53.0	fUTF8	孓
U+5B53.0	fCNS11643	1-444F
U+5B53.0	fBig5	A46E
U+5B53.0	fGB	0-665E
U+5B53.0	fR/S	 39.0
U+5B53.0	fTotalStrokes	 3
U+5B53.0	fCangjie	NNO
U+5B53.0	fFourCorner	17400
U+5B53.0	fHakka	 ket7
U+5B53.0	fCantonese	kyut3
U+5B53.0	fMandarin	jue2; gong3
U+5B53.0	fEnglish	 larvae of the mosquitoes
U+5B54.0	fUTF8	孔
U+5B54.0	fCNS11643	1-4536
U+5B54.0	fBig5	A4D5
U+5B54.0	fGB	0-3F57
U+5B54.0	fR/S	 39.1
U+5B54.0	fTotalStrokes	 4
U+5B54.0	fCangjie	NDU
U+5B54.0	fFourCorner	12410
U+5B54.0	fHakka	  kung3; kung1
U+5B54.0	fCantonese	hung2
U+5B54.0	fMandarin	kong3
U+5B54.0	fEnglish	[1] opening; hole; orifice; great; aperture [2] very; exceedingly [3] of or pertaining to Confucius or Confucianism [4] urgent; badly [5] a Chinese family name
U+5B55.0	fUTF8	孕
U+5B55.0	fCNS11643	1-4645
U+5B55.0	fBig5	A5A5
U+5B55.0	fGB	0-5450
U+5B55.0	fR/S	 39.2
U+5B55.0	fTotalStrokes	 5
U+5B55.0	fCangjie	NSND
U+5B55.0	fFourCorner	17407
U+5B55.0	fHakka	  jin3; rin3; jun3
U+5B55.0	fCantonese	jan6
U+5B55.0	fMandarin	yun4
U+5B55.0	fEnglish	 be pregnant; conceive; pregnancy
U+5B56.0	fUTF8	孖
U+5B56.0	fCNS11643	2-2233
U+5B56.0	fBig5	C9D3
U+5B56.0	fGB	3-3D6E
U+5B56.0	fR/S	 39.3
U+5B56.0	fTotalStrokes	 6
U+5B56.0	fCangjie	NDND
U+5B56.0	fFourCorner	17447
U+5B56.0	fHakka	 zih1; cih5; zu1; la1; ma1; mo1; zii1
U+5B56.0	fCantonese	maa1; zi1
U+5B56.0	fMandarin	zi1; ma1
U+5B56.0	fEnglish	[1] twins [2] measure word for pairs
U+5B57.0	fUTF8	字
U+5B57.0	fCNS11643	1-4773
U+5B57.0	fBig5	A672
U+5B57.0	fGB	0-5756
U+5B57.0	fR/S	 39.3
U+5B57.0	fTotalStrokes	 6
U+5B57.0	fCangjie	JND
U+5B57.0	fFourCorner	30407
U+5B57.0	fHakka	 su5; sii6; sii5; cih5; su3; cih6; se5; ze3
U+5B57.0	fCantonese	zi6; zi2 (also)
U+5B57.0	fMandarin	zi4
U+5B57.0	fEnglish	[1] word; letter; character; word; logograph; glyph [2] betroth a girl [3] a name or style taken at the age of 20, by which one was sometimes called
U+5B57.1	fUTF8	字
U+5B57.1	fCNS11643	1-4773
U+5B57.1	fBig5	A672
U+5B57.1	fGB	0-5756
U+5B57.1	fR/S	 39.3
U+5B57.1	fTotalStrokes	 6
U+5B57.1	fCangjie	JND
U+5B57.1	fFourCorner	30407
U+5B57.1	fMandarin	zi5
U+5B58.0	fUTF8	存
U+5B58.0	fCNS11643	1-4774
U+5B58.0	fBig5	A673
U+5B58.0	fGB	0-3466
U+5B58.0	fR/S	 39.3
U+5B58.0	fTotalStrokes	 6
U+5B58.0	fCangjie	KLND
U+5B58.0	fFourCorner	40247
U+5B58.0	fHakka	  sun2; cun2; sun3
U+5B58.0	fCantonese	cyun4
U+5B58.0	fMandarin	cun2
U+5B58.0	fEnglish	[1] exist; live; survive; remain [2] keep; deposit
U+5B59.0	fUTF8	孙
U+5B59.0	fGB	0-4B6F
U+5B59.0	fR/S	 39.3
U+5B59.0	fTotalStrokes	 6
U+5B59.0	fCangjie	NDF
U+5B59.0	fHakka	 sun1; zih3
U+5B59.0	fCantonese	syun1; seon3; zi2
U+5B59.0	fMandarin	sun1; zi5; zi2; xun4; ci2; zi3
U+5B59.0	fEnglish	[1] ancient form of &#x5B50; [2] simplified form of &#x5B6B;
U+5B5A.0	fUTF8	孚
U+5B5A.0	fCNS11643	1-4977
U+5B5A.0	fBig5	A7B7
U+5B5A.0	fGB	0-665A
U+5B5A.0	fR/S	 39.4
U+5B5A.0	fTotalStrokes	 7
U+5B5A.0	fCangjie	BND
U+5B5A.0	fFourCorner	20407
U+5B5A.0	fHakka	 fu1
U+5B5A.0	fCantonese	fu1
U+5B5A.0	fMandarin	fu2; fu1; fu4
U+5B5A.0	fEnglish	[1] confidence; trust [2] a Chinese family name
U+5B5B.0	fUTF8	孛
U+5B5B.0	fCNS11643	1-4978
U+5B5B.0	fBig5	A7B8
U+5B5B.0	fGB	0-5843
U+5B5B.0	fR/S	 39.4
U+5B5B.0	fTotalStrokes	 7
U+5B5B.0	fCangjie	JBND
U+5B5B.0	fFourCorner	40407
U+5B5B.0	fHakka	  put8
U+5B5B.0	fCantonese	bui6; but6; baak6
U+5B5B.0	fMandarin	bei4; bo2
U+5B5B.0	fEnglish	 comet
U+5B5C.0	fUTF8	孜
U+5B5C.0	fCNS11643	1-4976
U+5B5C.0	fBig5	A7B6
U+5B5C.0	fGB	0-574E
U+5B5C.0	fR/S	 39.4
U+5B5C.0	fTotalStrokes	 7
U+5B5C.0	fCangjie	NDOK
U+5B5C.0	fFourCorner	18440
U+5B5C.0	fHakka	 zih1; sii1; zu1; zii1
U+5B5C.0	fCantonese	zi1
U+5B5C.0	fMandarin	zi1
U+5B5C.0	fEnglish	never weary; be as diligent as possible; unwearied and diligent
U+5B5D.0	fUTF8	孝
U+5B5D.0	fCNS11643	1-4975
U+5B5D.0	fBig5	A7B5
U+5B5D.0	fGB	0-5022
U+5B5D.0	fR/S	 39.4
U+5B5D.0	fTotalStrokes	 7
U+5B5D.0	fCangjie	JKND
U+5B5D.0	fFourCorner	44407
U+5B5D.0	fHakka	  hau5
U+5B5D.0	fCantonese	haau3
U+5B5D.0	fMandarin	xiao4
U+5B5D.0	fEnglish	filial piety; obedience; of or having to do with filial piety
U+5B5E.0	fUTF8	孞
U+5B5E.0	fGB	E-274E
U+5B5E.0	fR/S	 39.4
U+5B5E.0	fTotalStrokes	 7
U+5B5E.0	fCangjie	NDP
U+5B5E.0	fFourCorner	1733
U+5B5E.0	fCantonese	seon3
U+5B5E.0	fMandarin	xin4
U+5B5F.0	fUTF8	孟
U+5B5F.0	fCNS11643	1-4C75
U+5B5F.0	fBig5	A973
U+5B5F.0	fGB	0-434F
U+5B5F.0	fR/S	 39.5
U+5B5F.0	fTotalStrokes	 8
U+5B5F.0	fCangjie	NDBT
U+5B5F.0	fFourCorner	17107
U+5B5F.0	fHakka	 men5; men6; mang3; men3
U+5B5F.0	fCantonese	maang6; maang2 (also)
U+5B5F.0	fMandarin	meng4; man4; mang4
U+5B5F.0	fEnglish	[1] a Chinese family name [2] the eldest of children [3] rude; rough
U+5B60.0	fUTF8	孠
U+5B60.0	fCNS11643	3-2848
U+5B60.0	fGB	E-274F
U+5B60.0	fR/S	 39.5
U+5B60.0	fTotalStrokes	 8
U+5B60.0	fCangjie	SRND
U+5B60.0	fFourCorner	17400
U+5B60.0	fHakka	 cih2; cu3; sii5; sii6; cu2; se5; ze5
U+5B60.0	fCantonese	zi6
U+5B60.0	fMandarin	si4
U+5B60.0	fEnglish	ancient form of &#x55E3;
U+5B61.0	fUTF8	孡
U+5B61.0	fCNS11643	4-2622
U+5B61.0	fGB	5-4122
U+5B61.0	fR/S	  39.5
U+5B61.0	fTotalStrokes	 8
U+5B61.0	fCangjie	NDIR
U+5B61.0	fFourCorner	13460
U+5B61.0	fCantonese	toi1
U+5B61.0	fMandarin	tai1
U+5B62.0	fUTF8	孢
U+5B62.0	fCNS11643	2-2636
U+5B62.0	fBig5	CC55
U+5B62.0	fGB	0-665F
U+5B62.0	fR/S	 39.5
U+5B62.0	fTotalStrokes	 8
U+5B62.0	fCangjie	NDPRU
U+5B62.0	fFourCorner	17412
U+5B62.0	fHakka	 bau1
U+5B62.0	fCantonese	baau1
U+5B62.0	fMandarin	bao1; bao4
U+5B63.0	fUTF8	季
U+5B63.0	fCNS11643	1-4C77
U+5B63.0	fBig5	A975
U+5B63.0	fGB	0-3C3E
U+5B63.0	fR/S	 39.5
U+5B63.0	fTotalStrokes	 8
U+5B63.0	fCangjie	HDND
U+5B63.0	fFourCorner	20407
U+5B63.0	fHakka	  gui5;  gi5; gwi5; kwui5
U+5B63.0	fCantonese	gwai3
U+5B63.0	fMandarin	ji4
U+5B63.0	fEnglish	[1] quarter of a year; season [2] last (month of a season) [3] the youngest (of brothers) [4] a Chinese family name
U+5B64.0	fUTF8	孤
U+5B64.0	fCNS11643	1-4C76
U+5B64.0	fBig5	A974
U+5B64.0	fGB	0-3942
U+5B64.0	fR/S	 39.5
U+5B64.0	fTotalStrokes	 8
U+5B64.0	fCangjie	NDHVO
U+5B64.0	fFourCorner	12430
U+5B64.0	fHakka	  gu1; gu2
U+5B64.0	fCantonese	gu1; gwu1 (also)
U+5B64.0	fMandarin	gu1
U+5B64.0	fEnglish	[1] orphan; fatherless [2] solitary; lone; lonely; friendless; helpless; unaided
U+5B65.0	fUTF8	孥
U+5B65.0	fCNS11643	2-2637
U+5B65.0	fBig5	CC56
U+5B65.0	fGB	0-665B
U+5B65.0	fR/S	 39.5
U+5B65.0	fTotalStrokes	 8
U+5B65.0	fCangjie	VEND
U+5B65.0	fFourCorner	47407
U+5B65.0	fHakka	  nu2
U+5B65.0	fCantonese	nou4; lou4 (also)
U+5B65.0	fMandarin	nu2; nu4
U+5B65.0	fEnglish	one's children
U+5B66.0	fUTF8	学
U+5B66.0	fCNS11643	4-2623
U+5B66.0	fGB	0-5127
U+5B66.0	fR/S	 39.5
U+5B66.0	fTotalStrokes	 8
U+5B66.0	fCangjie	FBND
U+5B66.0	fFourCorner	32400
U+5B66.0	fHakka	 hok8
U+5B66.0	fCantonese	hok6; hok2 (also)
U+5B66.0	fMandarin	xue2; xiao2; jiao4
U+5B66.0	fEnglish	simplified form of &#x5B78;
U+5B67.0	fUTF8	孧
U+5B67.0	fCNS11643	3-646A
U+5B67.0	fGB	E-2750
U+5B67.0	fR/S	 39.5
U+5B67.0	fTotalStrokes	 8
U+5B67.0	fCangjie	VSND
U+5B67.0	fFourCorner	2440
U+5B67.0	fCantonese	jau3
U+5B68.0	fUTF8	孨
U+5B68.0	fCNS11643	4-286E
U+5B68.0	fGB	5-4126
U+5B68.0	fR/S	  39.6
U+5B68.0	fTotalStrokes	 9
U+5B68.0	fCangjie	NDNDD
U+5B68.0	fFourCorner	17440
U+5B68.0	fCantonese	zyun2
U+5B68.0	fMandarin	zhuan3; ni4
U+5B69.0	fUTF8	孩
U+5B69.0	fCNS11643	1-5066
U+5B69.0	fBig5	ABC4
U+5B69.0	fGB	0-3A22
U+5B69.0	fR/S	 39.6
U+5B69.0	fTotalStrokes	 9
U+5B69.0	fCangjie	NDYVO
U+5B69.0	fFourCorner	10482
U+5B69.0	fHakka	  hai2; hai3; ha2
U+5B69.0	fCantonese	haai4; hoi4 (also)
U+5B69.0	fMandarin	hai2
U+5B69.0	fEnglish	[1] baby; child; infant; children [2] young; small
U+5B6A.0	fUTF8	孪
U+5B6A.0	fCNS11643	3-2C2F
U+5B6A.0	fGB	0-424F
U+5B6A.0	fR/S	 39.6
U+5B6A.0	fTotalStrokes	 9
U+5B6A.0	fCangjie	YCND
U+5B6A.0	fFourCorner	00400
U+5B6A.0	fHakka	 lien2; lan2
U+5B6A.0	fCantonese	lyun4
U+5B6A.0	fMandarin	luan2; shuan4
U+5B6A.0	fEnglish	simplified form of &#x5B7F;
U+5B6B.0	fUTF8	孫
U+5B6B.0	fCNS11643	1-5521
U+5B6B.0	fBig5	AE5D
U+5B6B.0	fGB	1-4B6F
U+5B6B.0	fR/S	 39.7
U+5B6B.0	fTotalStrokes	 10
U+5B6B.0	fCangjie	NDHVF
U+5B6B.0	fFourCorner	12493
U+5B6B.0	fHakka	  sun1; sun5
U+5B6B.0	fCantonese	syun1; seon3
U+5B6B.0	fMandarin	sun1; xun4; sun4
U+5B6B.0	fEnglish	[1] grandchild; descendent [2] a Chinese family name
U+5B6C.0	fUTF8	孬
U+5B6C.0	fCNS11643	2-2E67
U+5B6C.0	fBig5	D165
U+5B6C.0	fGB	0-582B
U+5B6C.0	fR/S	 39.7
U+5B6C.0	fTotalStrokes	 10
U+5B6C.0	fCangjie	MFVND
U+5B6C.0	fFourCorner	10447
U+5B6C.0	fHakka	 wai1; fai3; fai6; wai6; wai5; lau1; vai5
U+5B6C.0	fCantonese	bou2; naau1; laau1 (also); jai5; naau6; l; aau1
U+5B6C.0	fMandarin	huai4; nao1
U+5B6D.0	fUTF8	孭
U+5B6D.0	fGB	1-7C3B
U+5B6D.0	fR/S	 39.7
U+5B6D.0	fTotalStrokes	 10
U+5B6D.0	fCangjie	NDBUC
U+5B6D.0	fCantonese	me1
U+5B6D.0	fMandarin	mie1
U+5B6D.0	fEnglish	 carry on the back
U+5B6E.0	fUTF8	孮
U+5B6E.0	fCNS11643	2-3473
U+5B6E.0	fBig5	D4F0
U+5B6E.0	fGB	5-4128
U+5B6E.0	fR/S	 39.8
U+5B6E.0	fTotalStrokes	 11
U+5B6E.0	fCangjie	NDJMF
U+5B6E.0	fFourCorner	13491
U+5B6E.0	fCantonese	cung4
U+5B6E.0	fMandarin	cong2
U+5B6F.0	fUTF8	孯
U+5B6F.0	fCNS11643	4-312C
U+5B6F.0	fGB	E-2751
U+5B6F.0	fR/S	 39.8
U+5B6F.0	fTotalStrokes	 11
U+5B6F.0	fCangjie	SEND
U+5B6F.0	fFourCorner	77400
U+5B6F.0	fCantonese	haan1
U+5B6F.0	fMandarin	qian1; wan4
U+5B70.0	fUTF8	孰
U+5B70.0	fCNS11643	1-5969
U+5B70.0	fBig5	B145
U+5B70.0	fGB	0-4A6B
U+5B70.0	fR/S	 39.8
U+5B70.0	fTotalStrokes	 11
U+5B70.0	fCangjie	YDKNI
U+5B70.0	fFourCorner	05417
U+5B70.0	fHakka	  shuk8; suk8
U+5B70.0	fCantonese	suk6
U+5B70.0	fMandarin	shu2
U+5B70.0	fEnglish	[1] who? whom [2] which? what? which one? (in literary texts)
U+5B71.0	fUTF8	孱
U+5B71.0	fCNS11643	1-5E6C
U+5B71.0	fBig5	B447
U+5B71.0	fGB	0-656E
U+5B71.0	fR/S	 39.9
U+5B71.0	fTotalStrokes	 12
U+5B71.0	fCangjie	SNDD
U+5B71.0	fFourCorner	77247
U+5B71.0	fHakka	 zan1; shan1; san1
U+5B71.0	fCantonese	saan4; caan3; zaan6
U+5B71.0	fMandarin	chan2; can4; jian1; zhan1; zhan4
U+5B71.0	fEnglish	weak; unfit; frail; feeble
U+5B72.0	fUTF8	孲
U+5B72.0	fCNS11643	2-3472
U+5B72.0	fBig5	D4EF
U+5B72.0	fGB	E-2752
U+5B72.0	fR/S	 39.8; 39.9
U+5B72.0	fTotalStrokes	 11
U+5B72.0	fCangjie	NDMLM
U+5B72.0	fFourCorner	11417
U+5B72.0	fHakka	 o2
U+5B72.0	fCantonese	aa1; ngaa1 (also)
U+5B72.0	fMandarin	ya1
U+5B73.0	fUTF8	孳
U+5B73.0	fCNS11643	1-5E6B
U+5B73.0	fBig5	B446
U+5B73.0	fGB	0-665C
U+5B73.0	fR/S	 39.10
U+5B73.0	fTotalStrokes	 13
U+5B73.0	fCangjie	TVID
U+5B73.0	fFourCorner	44407
U+5B73.0	fHakka	 zih1; zu1; zii1
U+5B73.0	fCantonese	zi1; zi3
U+5B73.0	fMandarin	zi1; zi4
U+5B73.0	fEnglish	[1] bear or beget in large numbers [2] work with sustained diligence
U+5B74.0	fUTF8	孴
U+5B74.0	fCNS11643	3-4071
U+5B74.0	fGB	3-4825
U+5B74.0	fR/S	 39.10
U+5B74.0	fTotalStrokes	 13
U+5B74.0	fCangjie	NDNDA
U+5B74.0	fFourCorner	17600
U+5B74.0	fCantonese	ji5
U+5B74.0	fMandarin	ni3
U+5B75.0	fUTF8	孵
U+5B75.0	fCNS11643	1-684D
U+5B75.0	fBig5	B9E5
U+5B75.0	fGB	0-3775
U+5B75.0	fR/S	 39.11
U+5B75.0	fTotalStrokes	 14
U+5B75.0	fCangjie	HHSLD
U+5B75.0	fFourCorner	72747
U+5B75.0	fHakka	 fu1;  pu5; pu6; pu3
U+5B75.0	fCantonese	fu1
U+5B75.0	fMandarin	fu1; fu4
U+5B75.0	fEnglish	[1] sit on eggs; hatch (eggs); incubate [2] emerge from eggs or spawn
U+5B76.0	fUTF8	孶
U+5B76.0	fCNS11643	3-4072
U+5B76.0	fGB	E-2753
U+5B76.0	fR/S	 39.11
U+5B76.0	fTotalStrokes	 13; 14
U+5B76.0	fCangjie	YIND
U+5B76.0	fFourCorner	44400
U+5B76.0	fCantonese	zi1; zi3
U+5B76.0	fMandarin	zi1; zi4
U+5B76.0	fEnglish	variant of &#x5B73;
U+5B77.0	fUTF8	孷
U+5B77.0	fCNS11643	2-4964
U+5B77.0	fBig5	E17D
U+5B77.0	fGB	5-4944
U+5B77.0	fR/S	 39.11
U+5B77.0	fTotalStrokes	 14
U+5B77.0	fCangjie	JKMND
U+5B77.0	fFourCorner	58247
U+5B77.0	fCantonese	lei4
U+5B77.0	fMandarin	li2; li1
U+5B78.0	fUTF8	學
U+5B78.0	fCNS11643	1-7050
U+5B78.0	fBig5	BEC7
U+5B78.0	fGB	1-5127
U+5B78.0	fR/S	 39.13
U+5B78.0	fTotalStrokes	 16
U+5B78.0	fCangjie	HBND
U+5B78.0	fFourCorner	77407
U+5B78.0	fHakka	  hok8
U+5B78.0	fCantonese	hok6; hok2 (also)
U+5B78.0	fMandarin	xue2; xiao2; jiao4
U+5B78.0	fEnglish	[1] learn; study; imitate [2] of or having to do with learning; knowledge; school; academic [3] tell; inform (v)
U+5B79.0	fUTF8	孹
U+5B79.0	fCNS11643	3-5057
U+5B79.0	fGB	E-2754
U+5B79.0	fR/S	 39.13
U+5B79.0	fTotalStrokes	 16
U+5B79.0	fCangjie	SJND
U+5B79.0	fFourCorner	70400
U+5B79.0	fHakka	  mak7;  pit7; piak7;  bak7; phit7; mak8
U+5B79.0	fCantonese	maak3; baai1; mak3
U+5B79.0	fMandarin	bo4; pi4
U+5B79.0	fEnglish	same as &#x64D8;
U+5B7A.0	fUTF8	孺
U+5B7A.0	fCNS11643	1-7352
U+5B7A.0	fBig5	C0A9
U+5B7A.0	fGB	0-4866
U+5B7A.0	fR/S	 39.14
U+5B7A.0	fTotalStrokes	 17
U+5B7A.0	fCangjie	NDMBB
U+5B7A.0	fFourCorner	11427
U+5B7A.0	fHakka	 ji2; ji3; ri1; ji1; i2
U+5B7A.0	fCantonese	jyu4 (also); jyu6
U+5B7A.0	fMandarin	ru2; ru4
U+5B7A.0	fEnglish	 young child; infant
U+5B7B.0	fUTF8	孻
U+5B7B.0	fCNS11643	2-5C63
U+5B7B.0	fBig5	ECD7
U+5B7B.0	fGB	5-4125
U+5B7B.0	fR/S	 39.14
U+5B7B.0	fTotalStrokes	 17
U+5B7B.0	fCangjie	NDLMT
U+5B7B.0	fFourCorner	15417
U+5B7B.0	fHakka	  lai5
U+5B7B.0	fCantonese	laai1
U+5B7B.0	fMandarin	nai2; lai2
U+5B7B.0	fEnglish	last
U+5B7C.0	fUTF8	孼
U+5B7C.0	fCNS11643	3-5A34
U+5B7C.0	fGB	E-2755
U+5B7C.0	fR/S	  39.16
U+5B7C.0	fTotalStrokes	 19
U+5B7C.0	fCangjie	UJND
U+5B7C.0	fFourCorner	20400
U+5B7C.0	fHakka	  ngiet7; ngiap7; nget7
U+5B7C.0	fCantonese	jit6; jip6 (also)
U+5B7C.0	fMandarin	nie4
U+5B7C.0	fEnglish	 misfortune sin; evil
U+5B7D.0	fUTF8	孽
U+5B7D.0	fCNS11643	1-7969
U+5B7D.0	fBig5	C45E
U+5B7D.0	fGB	0-4475
U+5B7D.0	fR/S	 39.17
U+5B7D.0	fTotalStrokes	 20
U+5B7D.0	fCangjie	TJND
U+5B7D.0	fFourCorner	44407
U+5B7D.0	fHakka	  ngiet7; ngiap7; jet7; nget7
U+5B7D.0	fCantonese	jit6; jip6 (also)
U+5B7D.0	fMandarin	nie4
U+5B7D.0	fEnglish	[1] evil; sin [2] the son of a concubine [3] monster [4] calamity [5] non-classical form of &#x5B7C;
U+5B7E.0	fUTF8	孾
U+5B7E.0	fCNS11643	3-5C3B
U+5B7E.0	fGB	E-2756
U+5B7E.0	fR/S	  39.17
U+5B7E.0	fTotalStrokes	 20
U+5B7E.0	fCangjie	NDBCV
U+5B7E.0	fFourCorner	16440
U+5B7E.0	fHakka	 jin1; rin1
U+5B7E.0	fCantonese	jing1
U+5B7E.0	fMandarin	ying1
U+5B7E.0	fEnglish	a baby; especially a girl; an infnt
U+5B7F.0	fUTF8	孿
U+5B7F.0	fCNS11643	1-7B5C
U+5B7F.0	fBig5	C570
U+5B7F.0	fGB	1-424F
U+5B7F.0	fR/S	 39.19
U+5B7F.0	fTotalStrokes	 22
U+5B7F.0	fCangjie	VFND
U+5B7F.0	fFourCorner	22407
U+5B7F.0	fHakka	  lan2; lien2; len2
U+5B7F.0	fCantonese	lyun4; syun3
U+5B7F.0	fMandarin	luan2; shuan4
U+5B7F.0	fEnglish	twin
U+5B80.0	fUTF8	宀
U+5B80.0	fCNS11643	3-213B
U+5B80.0	fGB	0-6532
U+5B80.0	fR/S	 40.0
U+5B80.0	fTotalStrokes	 3
U+5B80.0	fCangjie	IB
U+5B80.0	fFourCorner	30370
U+5B80.0	fHakka	  mien2; men2
U+5B80.0	fCantonese	min4
U+5B80.0	fMandarin	mian2; mian1
U+5B80.0	fEnglish	[1] roof [2] Kangxi radical no. 40
U+5B81.0	fUTF8	宁
U+5B81.0	fCNS11643	2-2153
U+5B81.0	fBig5	C972
U+5B81.0	fGB	0-447E
U+5B81.0	fR/S	 40.2
U+5B81.0	fTotalStrokes	 5
U+5B81.0	fCangjie	JMN
U+5B81.0	fFourCorner	30201
U+5B81.0	fHakka	  du3; len2; nen2
U+5B81.0	fCantonese	ning4; cyu5; ling4 (also); ning6; ling6 (also)
U+5B81.0	fMandarin	ning2; zhu4
U+5B81.0	fEnglish	[1] store; save; stockpile [2] stand [3] simplified form of &#x5BE7;
U+5B81.1	fUTF8	宁
U+5B81.1	fCNS11643	2-2153
U+5B81.1	fBig5	C972
U+5B81.1	fGB	0-447E
U+5B81.1	fR/S	 40.2
U+5B81.1	fTotalStrokes	 5
U+5B81.1	fCangjie	JMN
U+5B81.1	fFourCorner	30201
U+5B81.1	fMandarin	ning4
U+5B82.0	fUTF8	宂
U+5B82.0	fCNS11643	3-223D
U+5B82.0	fGB	E-2757
U+5B82.0	fR/S	 40.2
U+5B82.0	fTotalStrokes	 5
U+5B82.0	fCangjie	JHU
U+5B82.0	fFourCorner	30210
U+5B82.0	fHakka	 jung3; shiu3
U+5B82.0	fCantonese	jung2; sau2; sau3
U+5B82.0	fMandarin	rong3; shou3; shou4
U+5B82.0	fEnglish	[1] redundant; superfluous [2] to stand [3] same as &#x5B88;
U+5B83.0	fUTF8	它
U+5B83.0	fCNS11643	1-4646
U+5B83.0	fBig5	A5A6
U+5B83.0	fGB	0-4B7C
U+5B83.0	fR/S	 40.2
U+5B83.0	fTotalStrokes	 5
U+5B83.0	fCangjie	JP
U+5B83.0	fFourCorner	30711
U+5B83.0	fHakka	  to2; ta1
U+5B83.0	fCantonese	taa1; to1 (also)
U+5B83.0	fMandarin	ta1; tuo1; te1
U+5B83.0	fEnglish	it; that; this
U+5B84.0	fUTF8	宄
U+5B84.0	fCNS11643	2-2154
U+5B84.0	fBig5	C973
U+5B84.0	fGB	0-6533
U+5B84.0	fR/S	 40.2
U+5B84.0	fTotalStrokes	 5
U+5B84.0	fCangjie	JKN
U+5B84.0	fFourCorner	30417
U+5B84.0	fHakka	 gui3; kui3; kwui3
U+5B84.0	fCantonese	gwai2
U+5B84.0	fMandarin	gui3
U+5B84.0	fEnglish	 traitor; villain; thief; treacherous fellow
U+5B85.0	fUTF8	宅
U+5B85.0	fCNS11643	1-4777
U+5B85.0	fBig5	A676
U+5B85.0	fGB	0-552C
U+5B85.0	fR/S	 40.3
U+5B85.0	fTotalStrokes	 6
U+5B85.0	fCangjie	JHP
U+5B85.0	fFourCorner	30714
U+5B85.0	fHakka	 cet8;  cak8; can3
U+5B85.0	fCantonese	zaak6; zaak2 (also)
U+5B85.0	fMandarin	zhai2; zhe4
U+5B85.0	fEnglish	 residence; dwelling; house; home
U+5B86.0	fUTF8	宆
U+5B86.0	fCNS11643	3-2350
U+5B86.0	fGB	E-2758
U+5B86.0	fR/S	 40.3
U+5B86.0	fTotalStrokes	 6
U+5B86.0	fCangjie	JN
U+5B86.0	fFourCorner	30200
U+5B86.0	fHakka	 giung1
U+5B86.0	fCantonese	kung4; hung1; gung1
U+5B86.0	fMandarin	qiong2; qiong1; kong1
U+5B86.0	fEnglish	same as &#x7A79;
U+5B87.0	fUTF8	宇
U+5B87.0	fCNS11643	1-4775
U+5B87.0	fBig5	A674
U+5B87.0	fGB	0-536E
U+5B87.0	fR/S	 40.3
U+5B87.0	fTotalStrokes	 6
U+5B87.0	fCangjie	JMD
U+5B87.0	fFourCorner	30401
U+5B87.0	fHakka	 ji3; ji1; ri3; i3
U+5B87.0	fCantonese	jyu5
U+5B87.0	fMandarin	yu3
U+5B87.0	fEnglish	[1] house; building; roof; structure [2] look; appearance; countenance [3] space
U+5B88.0	fUTF8	守
U+5B88.0	fCNS11643	1-4776
U+5B88.0	fBig5	A675
U+5B88.0	fGB	0-4A58
U+5B88.0	fR/S	 40.3
U+5B88.0	fTotalStrokes	 6
U+5B88.0	fCangjie	JDI
U+5B88.0	fFourCorner	30342
U+5B88.0	fHakka	 su3;  shiu3; siu3; shiu6; shu3
U+5B88.0	fCantonese	sau2; sau3
U+5B88.0	fMandarin	shou3; shou4
U+5B88.0	fEnglish	[1] defend; protect; guard; conserve [2] wait [3] keep (a secret, etc.) [4] stick to; maintain [5] abide by
U+5B89.0	fUTF8	安
U+5B89.0	fCNS11643	1-4778
U+5B89.0	fBig5	A677
U+5B89.0	fGB	0-3032
U+5B89.0	fR/S	 40.3
U+5B89.0	fTotalStrokes	 6
U+5B89.0	fCangjie	JV
U+5B89.0	fFourCorner	30404
U+5B89.0	fHakka	  on1
U+5B89.0	fCantonese	on1; ngon1 (also)
U+5B89.0	fMandarin	an1
U+5B89.0	fEnglish	[1] peaceful; tranquil; quiet; calm [2] quiet; stabilize; pacify; console [3] put; place; arrange [4] be content with [5] how; why [6] safe; secure; stable [7] mend [8] a Chinese family name
U+5B8A.0	fUTF8	宊
U+5B8A.0	fCNS11643	4-236F
U+5B8A.0	fGB	5-3D72
U+5B8A.0	fR/S	  40.4
U+5B8A.0	fTotalStrokes	 7
U+5B8A.0	fCangjie	JIK
U+5B8A.0	fFourCorner	30430
U+5B8A.0	fCantonese	dat6
U+5B8A.0	fMandarin	tu2; tu1; jia1
U+5B8B.0	fUTF8	宋
U+5B8B.0	fCNS11643	1-497A
U+5B8B.0	fBig5	A7BA
U+5B8B.0	fGB	0-4B4E
U+5B8B.0	fR/S	 40.4
U+5B8B.0	fTotalStrokes	 7
U+5B8B.0	fCangjie	JD
U+5B8B.0	fFourCorner	30904
U+5B8B.0	fHakka	  sung5
U+5B8B.0	fCantonese	sung3
U+5B8B.0	fMandarin	song4
U+5B8B.0	fEnglish	[1] Sung dynasty (960-1279 A.D.) of, or having to do with the Sung dynasty [2] a state in the Warring States period [3] a Chinese family name
U+5B8C.0	fUTF8	完
U+5B8C.0	fCNS11643	1-4979
U+5B8C.0	fBig5	A7B9
U+5B8C.0	fGB	0-4D6A
U+5B8C.0	fR/S	 40.4
U+5B8C.0	fTotalStrokes	 7
U+5B8C.0	fCangjie	JMMU
U+5B8C.0	fFourCorner	30211
U+5B8C.0	fHakka	  wan2; won2;  jan2; wan3; zan2; van2; jen2; von2
U+5B8C.0	fCantonese	jyun4
U+5B8C.0	fMandarin	wan2
U+5B8C.0	fEnglish	[1] complete; finish; bring to a conclusion [2] whole [3] run out; use up [4] a Chinese family name
U+5B8D.0	fUTF8	宍
U+5B8D.0	fCNS11643	3-255A
U+5B8D.0	fGB	5-3D76
U+5B8D.0	fR/S	  40.4
U+5B8D.0	fTotalStrokes	 7
U+5B8D.0	fCangjie	JYC
U+5B8D.0	fFourCorner	30800
U+5B8D.0	fCantonese	juk6
U+5B8D.0	fMandarin	rou4; rou2
U+5B8D.0	fEnglish	 flesh meat Kangxi radical number 130
U+5B8E.0	fUTF8	宎
U+5B8E.0	fCNS11643	2-235C
U+5B8E.0	fBig5	CABC
U+5B8E.0	fGB	5-3D74
U+5B8E.0	fR/S	 40.4
U+5B8E.0	fTotalStrokes	 7
U+5B8E.0	fCangjie	JHK
U+5B8E.0	fFourCorner	30432
U+5B8E.0	fCantonese	jiu2; jiu3 (also); ngaau5; miu5
U+5B8E.0	fMandarin	yao3; yao1; yao4
U+5B8F.0	fUTF8	宏
U+5B8F.0	fCNS11643	1-497B
U+5B8F.0	fBig5	A7BB
U+5B8F.0	fGB	0-3A6A
U+5B8F.0	fR/S	 40.4
U+5B8F.0	fTotalStrokes	 7
U+5B8F.0	fCangjie	JKI
U+5B8F.0	fFourCorner	30432
U+5B8F.0	fHakka	  fen2; fen3
U+5B8F.0	fCantonese	wang4
U+5B8F.0	fMandarin	hong2
U+5B8F.0	fEnglish	[1] wide; spacious; great; vast; ample [2] a Chinese family name
U+5B90.0	fUTF8	宐
U+5B90.0	fCNS11643	4-2625
U+5B90.0	fGB	E-2759
U+5B90.0	fR/S	 40.4
U+5B90.0	fTotalStrokes	 7; 8
U+5B90.0	fCangjie	JNIM
U+5B90.0	fFourCorner	30100
U+5B90.0	fHakka	 ni2
U+5B90.0	fCantonese	ji4
U+5B90.0	fMandarin	yi2
U+5B90.0	fEnglish	standard form of &#x5B9C;
U+5B91.0	fUTF8	宑
U+5B91.0	fGB	E-275A
U+5B91.0	fR/S	 40.4
U+5B91.0	fTotalStrokes	 7
U+5B91.0	fCangjie	JTT
U+5B91.0	fFourCorner	3055
U+5B91.0	fCantonese	jing2
U+5B91.0	fMandarin	jing3
U+5B92.0	fUTF8	宒
U+5B92.0	fCNS11643	2-235D
U+5B92.0	fBig5	CABD
U+5B92.0	fGB	E-275B
U+5B92.0	fR/S	 40.4
U+5B92.0	fTotalStrokes	 7
U+5B92.0	fCangjie	JHQU
U+5B92.0	fFourCorner	30714
U+5B92.0	fCantonese	zeon1
U+5B92.0	fMandarin	zhun1
U+5B93.0	fUTF8	宓
U+5B93.0	fCNS11643	2-2638
U+5B93.0	fBig5	CC57
U+5B93.0	fGB	0-6535
U+5B93.0	fR/S	 40.5
U+5B93.0	fTotalStrokes	 8
U+5B93.0	fCangjie	JPH
U+5B93.0	fFourCorner	30332
U+5B93.0	fHakka	 mit8; met8
U+5B93.0	fCantonese	mat6; fuk6
U+5B93.0	fMandarin	mi4; fu2
U+5B93.0	fEnglish	[1] quiet; silent; still [2] a Chinese family name
U+5B94.0	fUTF8	宔
U+5B94.0	fCNS11643	3-284C
U+5B94.0	fGB	3-3B39
U+5B94.0	fR/S	 40.5
U+5B94.0	fTotalStrokes	 8
U+5B94.0	fCangjie	JYG
U+5B94.0	fFourCorner	30100
U+5B94.0	fHakka	 zhu3; zu3
U+5B94.0	fCantonese	zyu2
U+5B94.0	fMandarin	zhu3
U+5B95.0	fUTF8	宕
U+5B95.0	fCNS11643	2-2639
U+5B95.0	fBig5	CC58
U+5B95.0	fGB	0-6534
U+5B95.0	fR/S	 40.5
U+5B95.0	fTotalStrokes	 8
U+5B95.0	fCangjie	JMR
U+5B95.0	fFourCorner	30601
U+5B95.0	fHakka	  tong5
U+5B95.0	fCantonese	dong6; dong1
U+5B95.0	fMandarin	dang4
U+5B95.0	fEnglish	[1] quarryman [2] delay; procrastinate [3] dissolute [4] hesitate [5] hang
U+5B96.0	fUTF8	宖
U+5B96.0	fCNS11643	3-284A
U+5B96.0	fGB	5-3D77
U+5B96.0	fR/S	  40.5
U+5B96.0	fTotalStrokes	 8
U+5B96.0	fCangjie	JNI
U+5B96.0	fFourCorner	30230
U+5B96.0	fHakka	 fen2
U+5B96.0	fCantonese	wang4
U+5B96.0	fMandarin	hong2
U+5B97.0	fUTF8	宗
U+5B97.0	fCNS11643	1-4C78
U+5B97.0	fBig5	A976
U+5B97.0	fGB	0-575A
U+5B97.0	fR/S	 40.5
U+5B97.0	fTotalStrokes	 8
U+5B97.0	fCangjie	JMMF
U+5B97.0	fFourCorner	30901
U+5B97.0	fHakka	  zung1
U+5B97.0	fCantonese	zung1
U+5B97.0	fMandarin	zong1
U+5B97.0	fEnglish	[1] lineage; ancestry; ancestor; clan [2] sect; religion [3] believe in [4] a Chinese family name
U+5B97.1	fUTF8	宗
U+5B97.1	fCNS11643	1-4C78
U+5B97.1	fBig5	A976
U+5B97.1	fGB	0-575A
U+5B97.1	fR/S	 40.5
U+5B97.1	fTotalStrokes	 8
U+5B97.1	fCangjie	JMMF
U+5B97.1	fFourCorner	30901
U+5B97.1	fMandarin	zong5
U+5B98.0	fUTF8	官
U+5B98.0	fCNS11643	1-4C7A
U+5B98.0	fBig5	A978
U+5B98.0	fGB	0-3959
U+5B98.0	fR/S	 40.5
U+5B98.0	fTotalStrokes	 8
U+5B98.0	fCangjie	JRLR
U+5B98.0	fFourCorner	30777
U+5B98.0	fHakka	  gon1;  kwan1; gwon1
U+5B98.0	fCantonese	gun1; gwun1 (also)
U+5B98.0	fMandarin	guan1
U+5B98.0	fEnglish	[1] government official; public servant [2] of, or having to do with the government or the state [3] organ (biology) [4] a Chinese family name
U+5B99.0	fUTF8	宙
U+5B99.0	fCNS11643	1-4C7C
U+5B99.0	fBig5	A97A
U+5B99.0	fGB	0-5666
U+5B99.0	fR/S	 40.5
U+5B99.0	fTotalStrokes	 8
U+5B99.0	fCangjie	JLW
U+5B99.0	fFourCorner	30602
U+5B99.0	fHakka	  ciu5; diu3; ciu3; ciu6
U+5B99.0	fCantonese	zau6
U+5B99.0	fMandarin	zhou4
U+5B99.0	fEnglish	infinite time; time without beginning or end; eternity
U+5B9A.0	fUTF8	定
U+5B9A.0	fCNS11643	1-4C79
U+5B9A.0	fBig5	A977
U+5B9A.0	fGB	0-3628
U+5B9A.0	fR/S	 40.5
U+5B9A.0	fTotalStrokes	 8
U+5B9A.0	fCangjie	JMYO
U+5B9A.0	fFourCorner	30801
U+5B9A.0	fHakka	  tin5; tin6; tin3
U+5B9A.0	fCantonese	ding6; deng6; ding2 (also); deng2 (also); ding3
U+5B9A.0	fMandarin	ding4
U+5B9A.0	fEnglish	[1] decide; settle; fix [2] definite; sure [3] stable [4] remain
U+5B9B.0	fUTF8	宛
U+5B9B.0	fCNS11643	1-4C7D
U+5B9B.0	fBig5	A97B
U+5B9B.0	fGB	0-4D70
U+5B9B.0	fR/S	 40.5
U+5B9B.0	fTotalStrokes	 8
U+5B9B.0	fCangjie	JNIU
U+5B9B.0	fFourCorner	30212
U+5B9B.0	fHakka	 wan3;  won3; jen3; jen1; jang1; von3
U+5B9B.0	fCantonese	jyun2; jyun1; wun2
U+5B9B.0	fMandarin	wan3; yuan1; yu4; yung4
U+5B9B.0	fEnglish	[1] seem; as if; as though [2] crooked; roundabout [3] a Chinese family name
U+5B9C.0	fUTF8	宜
U+5B9C.0	fCNS11643	1-4C7B
U+5B9C.0	fBig5	A979
U+5B9C.0	fGB	0-524B
U+5B9C.0	fR/S	 40.5
U+5B9C.0	fTotalStrokes	 8
U+5B9C.0	fCangjie	JBM
U+5B9C.0	fFourCorner	30107
U+5B9C.0	fHakka	  ni2;  ngi2; gni2; gni3; ji2
U+5B9C.0	fCantonese	ji4; ji2 (also)
U+5B9C.0	fMandarin	yi2
U+5B9C.0	fEnglish	[1] suitable; right; fitting; proper; good [2] should; ought to; had better [3] matter [4] fit; suit; put in order [5] a Chinese family name
U+5B9C.1	fUTF8	宜
U+5B9C.1	fCNS11643	1-4C7B
U+5B9C.1	fBig5	A979
U+5B9C.1	fGB	0-524B
U+5B9C.1	fR/S	 40.5
U+5B9C.1	fTotalStrokes	 8
U+5B9C.1	fCangjie	JBM
U+5B9C.1	fFourCorner	30107
U+5B9C.1	fMandarin	yi5
U+5B9D.0	fUTF8	宝
U+5B9D.0	fCNS11643	3-284E
U+5B9D.0	fGB	0-3126
U+5B9D.0	fR/S	 40.5
U+5B9D.0	fTotalStrokes	 8
U+5B9D.0	fCangjie	JMGI
U+5B9D.0	fFourCorner	30100
U+5B9D.0	fHakka	  bau3;  bo3
U+5B9D.0	fCantonese	bou2
U+5B9D.0	fMandarin	bao3
U+5B9D.0	fEnglish	simplified form of &#x5BF6;
U+5B9E.0	fUTF8	实
U+5B9E.0	fCNS11643	3-284B
U+5B9E.0	fGB	0-4A35
U+5B9E.0	fR/S	 40.5
U+5B9E.0	fTotalStrokes	 8
U+5B9E.0	fCangjie	JYK
U+5B9E.0	fFourCorner	30430
U+5B9E.0	fHakka	 shit8; siit8
U+5B9E.0	fCantonese	sat6
U+5B9E.0	fMandarin	shi2; zhi1
U+5B9E.0	fEnglish	simplified form of &#x5BE6;
U+5B9E.1	fUTF8	实
U+5B9E.1	fCNS11643	3-284B
U+5B9E.1	fGB	0-4A35
U+5B9E.1	fR/S	 40.5
U+5B9E.1	fTotalStrokes	 8
U+5B9E.1	fCangjie	JYK
U+5B9E.1	fFourCorner	30430
U+5B9E.1	fMandarin	shi5
U+5B9F.0	fUTF8	実
U+5B9F.0	fCNS11643	3-284D
U+5B9F.0	fGB	E-275C
U+5B9F.0	fR/S	 40.5
U+5B9F.0	fTotalStrokes	 8
U+5B9F.0	fCangjie	JQK
U+5B9F.0	fFourCorner	30530
U+5B9F.0	fHakka	 shit8
U+5B9F.0	fCantonese	sat6
U+5B9F.0	fMandarin	shi2; zhi1
U+5B9F.0	fEnglish	simplified form of &#x5BE6;
U+5BA0.0	fUTF8	宠
U+5BA0.0	fGB	0-3368
U+5BA0.0	fR/S	 40.5
U+5BA0.0	fTotalStrokes	 8
U+5BA0.0	fCangjie	JIKP
U+5BA0.0	fHakka	 chung3
U+5BA0.0	fCantonese	cung2
U+5BA0.0	fMandarin	chong3; long2
U+5BA0.0	fEnglish	simplified form of &#x5BF5;
U+5BA1.0	fUTF8	审
U+5BA1.0	fGB	0-4973
U+5BA1.0	fR/S	 40.5
U+5BA1.0	fTotalStrokes	 8
U+5BA1.0	fCangjie	JLWL
U+5BA1.0	fHakka	 shim3
U+5BA1.0	fCantonese	sam2
U+5BA1.0	fMandarin	shen3; pan2
U+5BA1.0	fEnglish	simplified form of &#x5BE9;
U+5BA2.0	fUTF8	客
U+5BA2.0	fCNS11643	1-506A
U+5BA2.0	fBig5	ABC8
U+5BA2.0	fGB	0-3F4D
U+5BA2.0	fR/S	 40.6
U+5BA2.0	fTotalStrokes	 9
U+5BA2.0	fCangjie	JHER
U+5BA2.0	fFourCorner	30604
U+5BA2.0	fHakka	  hak7;  kak7; kak8; ket7
U+5BA2.0	fCantonese	haak3; haak2 (also)
U+5BA2.0	fMandarin	ke4
U+5BA2.0	fEnglish	[1] guest [2] customer [3] stranger; alien; foreigner [4] spectator; audience [5] foreign; strange; alien [6] adventurer [7] a Chinese family name [8] Hakka
U+5BA3.0	fUTF8	宣
U+5BA3.0	fCNS11643	1-5067
U+5BA3.0	fBig5	ABC5
U+5BA3.0	fGB	0-507B
U+5BA3.0	fR/S	 40.6
U+5BA3.0	fTotalStrokes	 9
U+5BA3.0	fCangjie	JMAM
U+5BA3.0	fFourCorner	30106
U+5BA3.0	fHakka	 sien1; sen1; sien2; sian1
U+5BA3.0	fCantonese	syun1
U+5BA3.0	fMandarin	xuan1
U+5BA3.0	fEnglish	[1] declare; announce; proclaim [2] propagate; circulate [3] a Chinese family name
U+5BA4.0	fUTF8	室
U+5BA4.0	fCNS11643	1-5069
U+5BA4.0	fBig5	ABC7
U+5BA4.0	fGB	0-4A52
U+5BA4.0	fR/S	 40.6
U+5BA4.0	fTotalStrokes	 9
U+5BA4.0	fCangjie	JMIG
U+5BA4.0	fFourCorner	30104
U+5BA4.0	fHakka	 sit7; shit7; siit7; shi3; sii3
U+5BA4.0	fCantonese	sat1
U+5BA4.0	fMandarin	shi4
U+5BA4.0	fEnglish	[1] room; home; house; chamber; apartment [2] wife
U+5BA5.0	fUTF8	宥
U+5BA5.0	fCNS11643	1-506B
U+5BA5.0	fBig5	ABC9
U+5BA5.0	fGB	0-6536
U+5BA5.0	fR/S	 40.6
U+5BA5.0	fTotalStrokes	 9
U+5BA5.0	fCangjie	JKB
U+5BA5.0	fFourCorner	30227
U+5BA5.0	fHakka	 ju5; riu6; jiu6; ju3; jiu3
U+5BA5.0	fCantonese	jau6
U+5BA5.0	fMandarin	you4
U+5BA5.0	fEnglish	 forgive; pardon; lenient
U+5BA6.0	fUTF8	宦
U+5BA6.0	fCNS11643	1-5068
U+5BA6.0	fBig5	ABC6
U+5BA6.0	fGB	0-3B42
U+5BA6.0	fR/S	 40.6
U+5BA6.0	fTotalStrokes	 9
U+5BA6.0	fCangjie	JSLL
U+5BA6.0	fFourCorner	30717
U+5BA6.0	fHakka	 fan5; fong5;  fon5; fon3
U+5BA6.0	fCantonese	waan6
U+5BA6.0	fMandarin	huan4
U+5BA6.0	fEnglish	[1] government official; government service [2] castrated
U+5BA7.0	fUTF8	宧
U+5BA7.0	fCNS11643	2-2E68
U+5BA7.0	fBig5	D166
U+5BA7.0	fGB	3-3B3D
U+5BA7.0	fR/S	 40.7; 40.6
U+5BA7.0	fTotalStrokes	 10
U+5BA7.0	fCangjie	JSLL
U+5BA7.0	fFourCorner	30717
U+5BA7.0	fHakka	 fan5
U+5BA7.0	fCantonese	ji4
U+5BA7.0	fMandarin	yi2
U+5BA7.0	fEnglish	corner
U+5BA8.0	fUTF8	宨
U+5BA8.0	fCNS11643	2-2978
U+5BA8.0	fBig5	CE77
U+5BA8.0	fGB	E-275D
U+5BA8.0	fR/S	 40.6
U+5BA8.0	fTotalStrokes	 9
U+5BA8.0	fCangjie	JLMO
U+5BA8.0	fFourCorner	30113
U+5BA8.0	fCantonese	tiu2
U+5BA8.0	fMandarin	tiao3
U+5BA9.0	fUTF8	宩
U+5BA9.0	fCNS11643	F-2A39
U+5BA9.0	fGB	E-275E
U+5BA9.0	fR/S	 40.6
U+5BA9.0	fTotalStrokes	 9
U+5BA9.0	fCangjie	JFD
U+5BA9.0	fFourCorner	3090
U+5BA9.0	fCantonese	si2
U+5BA9.0	fMandarin	shi3
U+5BAA.0	fUTF8	宪
U+5BAA.0	fGB	0-4F5C
U+5BAA.0	fR/S	 40.6
U+5BAA.0	fTotalStrokes	 9
U+5BAA.0	fCangjie	JHGU
U+5BAA.0	fFourCorner	3021
U+5BAA.0	fHakka	 hien5
U+5BAA.0	fCantonese	hin3
U+5BAA.0	fMandarin	xian4; xian3
U+5BAA.0	fEnglish	simplified form of &#x61B2;
U+5BAB.0	fUTF8	宫
U+5BAB.0	fCNS11643	3-2C32
U+5BAB.0	fGB	0-392C
U+5BAB.0	fR/S	 40.6
U+5BAB.0	fTotalStrokes	 9
U+5BAB.0	fCangjie	JRR
U+5BAB.0	fFourCorner	30600
U+5BAB.0	fCantonese	gung1
U+5BAB.0	fMandarin	gong1
U+5BAB.0	fEnglish	palace
U+5BAC.0	fUTF8	宬
U+5BAC.0	fCNS11643	2-2E6A
U+5BAC.0	fBig5	D168
U+5BAC.0	fGB	3-3B3B
U+5BAC.0	fR/S	 40.7
U+5BAC.0	fTotalStrokes	 10
U+5BAC.0	fCangjie	JIHS
U+5BAC.0	fFourCorner	30253
U+5BAC.0	fHakka	 shin2; siin2
U+5BAC.0	fCantonese	sing4
U+5BAC.0	fMandarin	cheng2
U+5BAC.0	fEnglish	 archives a Chinese family name
U+5BAD.0	fUTF8	宭
U+5BAD.0	fCNS11643	2-2E69
U+5BAD.0	fBig5	D167
U+5BAD.0	fGB	3-3B3F
U+5BAD.0	fR/S	 40.7
U+5BAD.0	fTotalStrokes	 10
U+5BAD.0	fCangjie	JSKR
U+5BAD.0	fFourCorner	30607
U+5BAD.0	fHakka	 kiun2
U+5BAD.0	fCantonese	kwan4
U+5BAD.0	fMandarin	qun2; jun1
U+5BAE.0	fUTF8	宮
U+5BAE.0	fCNS11643	1-5527
U+5BAE.0	fBig5	AE63
U+5BAE.0	fGB	E-275F
U+5BAE.0	fR/S	 40.7
U+5BAE.0	fTotalStrokes	 10
U+5BAE.0	fCangjie	JRHR
U+5BAE.0	fFourCorner	30606
U+5BAE.0	fHakka	  giung1
U+5BAE.0	fCantonese	gung1
U+5BAE.0	fMandarin	gong1
U+5BAE.0	fEnglish	[1] palace [2] castration
U+5BAF.0	fUTF8	宯
U+5BAF.0	fCNS11643	4-2C46
U+5BAF.0	fGB	E-2760
U+5BAF.0	fR/S	 40.7
U+5BAF.0	fTotalStrokes	 10
U+5BAF.0	fCangjie	JJKD
U+5BAF.0	fFourCorner	30400
U+5BAF.0	fCantonese	haau1
U+5BAF.0	fMandarin	xiao1
U+5BB0.0	fUTF8	宰
U+5BB0.0	fCNS11643	1-5523
U+5BB0.0	fBig5	AE5F
U+5BB0.0	fGB	0-5457
U+5BB0.0	fR/S	 40.7
U+5BB0.0	fTotalStrokes	 10
U+5BB0.0	fCangjie	JYTJ
U+5BB0.0	fFourCorner	30401
U+5BB0.0	fHakka	  zai3
U+5BB0.0	fCantonese	zoi2
U+5BB0.0	fMandarin	zai3
U+5BB0.0	fEnglish	[1] preside; govern; rule [2] slaughter; butcher [3] a Chinese family name
U+5BB1.0	fUTF8	宱
U+5BB1.0	fCNS11643	4-2C49
U+5BB1.0	fGB	5-3E24
U+5BB1.0	fR/S	 40.7
U+5BB1.0	fTotalStrokes	 10
U+5BB1.0	fCangjie	JOOS
U+5BB1.0	fFourCorner	30210
U+5BB1.0	fCantonese	zaa6
U+5BB1.0	fMandarin	zha4
U+5BB2.0	fUTF8	宲
U+5BB2.0	fCNS11643	6-3666
U+5BB2.0	fGB	E-2761
U+5BB2.0	fR/S	  40.7
U+5BB2.0	fTotalStrokes	 10; 9
U+5BB2.0	fCangjie	JRD
U+5BB2.0	fFourCorner	30900
U+5BB2.0	fCantonese	bou2
U+5BB2.0	fMandarin	bao3
U+5BB3.0	fUTF8	害
U+5BB3.0	fCNS11643	1-5524
U+5BB3.0	fBig5	AE60
U+5BB3.0	fGB	0-3A26
U+5BB3.0	fR/S	 40.7
U+5BB3.0	fTotalStrokes	 10
U+5BB3.0	fCangjie	JQMR
U+5BB3.0	fFourCorner	30601
U+5BB3.0	fHakka	 hoi5;  hoi3;  hot7; hoi6
U+5BB3.0	fCantonese	hoi6; hot3; hot6
U+5BB3.0	fMandarin	hai4; he2
U+5BB3.0	fEnglish	[1] injure; harm; destroy; damage [2] kill [3] damage; injury; harm; detriment [4] vital point
U+5BB3.1	fUTF8	害
U+5BB3.1	fCNS11643	1-5524
U+5BB3.1	fBig5	AE60
U+5BB3.1	fGB	0-3A26
U+5BB3.1	fR/S	 40.7
U+5BB3.1	fTotalStrokes	 10
U+5BB3.1	fCangjie	JQMR
U+5BB3.1	fFourCorner	30601
U+5BB3.1	fMandarin	hai5
U+5BB4.0	fUTF8	宴
U+5BB4.0	fCNS11643	1-5526
U+5BB4.0	fBig5	AE62
U+5BB4.0	fGB	0-5167
U+5BB4.0	fR/S	 40.7
U+5BB4.0	fTotalStrokes	 10
U+5BB4.0	fCangjie	JAV
U+5BB4.0	fFourCorner	30404
U+5BB4.0	fHakka	  jen5; jan5; ren5
U+5BB4.0	fCantonese	jin3
U+5BB4.0	fMandarin	yan4
U+5BB4.0	fEnglish	[1] entertain; feast [2] leisurely; comfort; ease
U+5BB5.0	fUTF8	宵
U+5BB5.0	fCNS11643	1-5528
U+5BB5.0	fBig5	AE64
U+5BB5.0	fGB	0-4F7C
U+5BB5.0	fR/S	 40.7
U+5BB5.0	fTotalStrokes	 10
U+5BB5.0	fCangjie	JFB
U+5BB5.0	fFourCorner	30227
U+5BB5.0	fHakka	  siau1; seu1
U+5BB5.0	fCantonese	siu1
U+5BB5.0	fMandarin	xiao1
U+5BB5.0	fEnglish	night; evening; dark
U+5BB6.0	fUTF8	家
U+5BB6.0	fCNS11643	1-5525
U+5BB6.0	fBig5	AE61
U+5BB6.0	fGB	0-3C52
U+5BB6.0	fR/S	 40.7
U+5BB6.0	fTotalStrokes	 10
U+5BB6.0	fCangjie	JMSO
U+5BB6.0	fFourCorner	30232
U+5BB6.0	fHakka	  ga1; gu1
U+5BB6.0	fCantonese	gaa1; gu1; gwu1 (also); gaa3; ze1
U+5BB6.0	fMandarin	jia1; gu1; jie5
U+5BB6.0	fEnglish	[1] house; home; residence; family; of a household; domestic [2] specialist (in any branch of art or science)
U+5BB6.1	fUTF8	家
U+5BB6.1	fCNS11643	1-5525
U+5BB6.1	fBig5	AE61
U+5BB6.1	fGB	0-3C52
U+5BB6.1	fR/S	 40.7
U+5BB6.1	fTotalStrokes	 10
U+5BB6.1	fCangjie	JMSO
U+5BB6.1	fFourCorner	30232
U+5BB6.1	fMandarin	jia5
U+5BB7.0	fUTF8	宷
U+5BB7.0	fCNS11643	3-3057
U+5BB7.0	fGB	E-2762
U+5BB7.0	fR/S	 40.7
U+5BB7.0	fTotalStrokes	 10
U+5BB7.0	fCangjie	JHFD
U+5BB7.0	fFourCorner	30900
U+5BB7.0	fHakka	  shim3;  sim3; siim3
U+5BB7.0	fCantonese	sam2
U+5BB7.0	fMandarin	shen3; pan2
U+5BB7.0	fEnglish	same as &#x5BE9;
U+5BB8.0	fUTF8	宸
U+5BB8.0	fCNS11643	1-552A
U+5BB8.0	fBig5	AE66
U+5BB8.0	fGB	0-6537
U+5BB8.0	fR/S	 40.7
U+5BB8.0	fTotalStrokes	 10
U+5BB8.0	fCangjie	JMMV
U+5BB8.0	fFourCorner	30232
U+5BB8.0	fHakka	 shin2; siin2; sin2
U+5BB8.0	fCantonese	san4
U+5BB8.0	fMandarin	chen2
U+5BB8.0	fEnglish	[1] abode of the emperor [2] large mansion
U+5BB9.0	fUTF8	容
U+5BB9.0	fCNS11643	1-5529
U+5BB9.0	fBig5	AE65
U+5BB9.0	fGB	0-485D
U+5BB9.0	fR/S	 40.7
U+5BB9.0	fTotalStrokes	 10
U+5BB9.0	fCangjie	JCOR
U+5BB9.0	fFourCorner	30608
U+5BB9.0	fHakka	  jung2; rung2; jung3
U+5BB9.0	fCantonese	jung4
U+5BB9.0	fMandarin	rong2
U+5BB9.0	fEnglish	[1] looks; appearance; figure; form; face; expression; countenance [2] contain; hold [3] allow; permit [4] forgive; pardon [5] forbear; forbearance
U+5BBA.0	fUTF8	宺
U+5BBA.0	fCNS11643	4-2C48
U+5BBA.0	fGB	E-2763
U+5BBA.0	fR/S	  40.7
U+5BBA.0	fTotalStrokes	 10
U+5BBA.0	fCangjie	JRMU
U+5BBA.0	fFourCorner	30210
U+5BBA.0	fCantonese	fong2
U+5BBA.0	fMandarin	huang3
U+5BBB.0	fUTF8	宻
U+5BBB.0	fCNS11643	3-3056
U+5BBB.0	fGB	E-2764
U+5BBB.0	fR/S	 40.7
U+5BBB.0	fTotalStrokes	 10
U+5BBB.0	fCangjie	JECU
U+5BBB.0	fFourCorner	30770
U+5BBB.0	fHakka	  met8;  mit8
U+5BBB.0	fCantonese	mat6
U+5BBB.0	fMandarin	mi4
U+5BBB.0	fEnglish	dense, thick, close; intimate
U+5BBC.0	fUTF8	宼
U+5BBC.0	fCNS11643	3-3055
U+5BBC.0	fGB	E-2765
U+5BBC.0	fR/S	 40.7
U+5BBC.0	fTotalStrokes	 10
U+5BBC.0	fCangjie	JMUV
U+5BBC.0	fFourCorner	30210
U+5BBC.0	fHakka	 keu5
U+5BBC.0	fCantonese	kau3
U+5BBC.0	fMandarin	kou4
U+5BBC.0	fEnglish	tyrannical, cruel; to rob to plunder; banditti thieves; highwaymen
U+5BBD.0	fUTF8	宽
U+5BBD.0	fGB	0-3F6D
U+5BBD.0	fR/S	 40.7
U+5BBD.0	fTotalStrokes	 10
U+5BBD.0	fCangjie	XJTBU
U+5BBD.0	fHakka	 kwon1; k'wan1
U+5BBD.0	fCantonese	fun1
U+5BBD.0	fMandarin	kuan1
U+5BBD.0	fEnglish	simplified form of &#x5BEC;
U+5BBE.0	fUTF8	宾
U+5BBE.0	fCNS11643	F-2E7E
U+5BBE.0	fGB	0-3176
U+5BBE.0	fR/S	 40.7
U+5BBE.0	fTotalStrokes	 10
U+5BBE.0	fCangjie	JOMC
U+5BBE.0	fFourCorner	3080
U+5BBE.0	fHakka	 bin1
U+5BBE.0	fCantonese	ban1; ban3
U+5BBE.0	fMandarin	bin1; bin4
U+5BBE.0	fEnglish	simplified form of &#x8CD3;
U+5BBF.0	fUTF8	宿
U+5BBF.0	fCNS11643	1-596E
U+5BBF.0	fBig5	B14A
U+5BBF.0	fGB	0-4B5E
U+5BBF.0	fR/S	 40.8
U+5BBF.0	fTotalStrokes	 11
U+5BBF.0	fCangjie	JOMA
U+5BBF.0	fFourCorner	30261
U+5BBF.0	fHakka	  siuk7; suk7; siu5
U+5BBF.0	fCantonese	suk1; sau3
U+5BBF.0	fMandarin	su4
U+5BBF.0	fEnglish	[1] lodge; stay overnight; sojourn [2] long-harboured; long-cherished [3] of the former life; inborn; innate; destined [4] veteran; old [5] a Chinese family name
U+5BBF.1	fUTF8	宿
U+5BBF.1	fCNS11643	1-596E
U+5BBF.1	fBig5	B14A
U+5BBF.1	fGB	0-4B5E
U+5BBF.1	fR/S	 40.8
U+5BBF.1	fTotalStrokes	 11
U+5BBF.1	fCangjie	JOMA
U+5BBF.1	fFourCorner	30261
U+5BBF.1	fMandarin	xiu4
U+5BBF.1	fEnglish	an ancient term for a constellation; asterism
U+5BBF.2	fUTF8	宿
U+5BBF.2	fCNS11643	1-596E
U+5BBF.2	fBig5	B14A
U+5BBF.2	fGB	0-4B5E
U+5BBF.2	fR/S	 40.8
U+5BBF.2	fTotalStrokes	 11
U+5BBF.2	fCangjie	JOMA
U+5BBF.2	fFourCorner	30261
U+5BBF.2	fMandarin	xiu3
U+5BBF.2	fEnglish	night
U+5BC0.0	fUTF8	寀
U+5BC0.0	fCNS11643	2-3475
U+5BC0.0	fBig5	D4F2
U+5BC0.0	fGB	E-2766
U+5BC0.0	fR/S	 40.8
U+5BC0.0	fTotalStrokes	 11
U+5BC0.0	fCangjie	JBD
U+5BC0.0	fFourCorner	30904
U+5BC0.0	fHakka	 cai3
U+5BC0.0	fCantonese	coi2; coi3
U+5BC0.0	fMandarin	cai4; cai3; shen3
U+5BC0.0	fEnglish	feudal estate
U+5BC1.0	fUTF8	寁
U+5BC1.0	fCNS11643	2-3474
U+5BC1.0	fBig5	D4F1
U+5BC1.0	fGB	3-3B40
U+5BC1.0	fR/S	 40.8
U+5BC1.0	fTotalStrokes	 11
U+5BC1.0	fCangjie	JJLO
U+5BC1.0	fFourCorner	30801
U+5BC1.0	fHakka	 ciap8; zam3
U+5BC1.0	fCantonese	zaam2; zit6 (also); zaan2
U+5BC1.0	fMandarin	zan3; jie2
U+5BC1.0	fEnglish	quick; fast
U+5BC2.0	fUTF8	寂
U+5BC2.0	fCNS11643	1-596D
U+5BC2.0	fBig5	B149
U+5BC2.0	fGB	0-3C45
U+5BC2.0	fR/S	 40.8
U+5BC2.0	fTotalStrokes	 11
U+5BC2.0	fCangjie	JYFE
U+5BC2.0	fFourCorner	30947
U+5BC2.0	fHakka	 sip8;  cip8; siuk8; sit7; zit7; sip7; cip7; cit7
U+5BC2.0	fCantonese	zik6
U+5BC2.0	fMandarin	ji4
U+5BC2.0	fEnglish	[1] still; silent; quiet; desolate; peaceful; serene [2] death of a Buddhist monk or nun
U+5BC3.0	fUTF8	寃
U+5BC3.0	fCNS11643	3-3541
U+5BC3.0	fGB	E-2767
U+5BC3.0	fR/S	  40.8
U+5BC3.0	fTotalStrokes	 11
U+5BC3.0	fCangjie	JNAI
U+5BC3.0	fFourCorner	30410
U+5BC3.0	fHakka	 jen1; jan1; ren1
U+5BC3.0	fCantonese	jyun1; zeoi6; zeoi3
U+5BC3.0	fMandarin	yuan1
U+5BC3.0	fEnglish	grievance; injustice; wrong
U+5BC4.0	fUTF8	寄
U+5BC4.0	fCNS11643	1-596C
U+5BC4.0	fBig5	B148
U+5BC4.0	fGB	0-3C44
U+5BC4.0	fR/S	 40.8
U+5BC4.0	fTotalStrokes	 11
U+5BC4.0	fCangjie	JKMR
U+5BC4.0	fFourCorner	30621
U+5BC4.0	fHakka	  gi5
U+5BC4.0	fCantonese	gei3
U+5BC4.0	fMandarin	ji4
U+5BC4.0	fEnglish	[1] send; transmit; mail [2] entrust; consign; commit; deposit
U+5BC5.0	fUTF8	寅
U+5BC5.0	fCNS11643	1-596B
U+5BC5.0	fBig5	B147
U+5BC5.0	fGB	0-527A
U+5BC5.0	fR/S	 40.8
U+5BC5.0	fTotalStrokes	 11
U+5BC5.0	fCangjie	JMLC
U+5BC5.0	fFourCorner	30806
U+5BC5.0	fHakka	  jin2; ji2; rin2; ji3; jen5; ri2; i2
U+5BC5.0	fCantonese	jan4
U+5BC5.0	fMandarin	yin2; yi2
U+5BC5.0	fEnglish	 respect; reverence respectfully
U+5BC6.0	fUTF8	密
U+5BC6.0	fCNS11643	1-596F
U+5BC6.0	fBig5	B14B
U+5BC6.0	fGB	0-435C
U+5BC6.0	fR/S	 40.8
U+5BC6.0	fTotalStrokes	 11
U+5BC6.0	fCangjie	JPHU
U+5BC6.0	fFourCorner	30773
U+5BC6.0	fHakka	  mit8;  met8; mi1
U+5BC6.0	fCantonese	mat6
U+5BC6.0	fMandarin	mi4
U+5BC6.0	fEnglish	[1] dense; thick; tight [2] close; intimate
U+5BC7.0	fUTF8	寇
U+5BC7.0	fCNS11643	1-596A
U+5BC7.0	fBig5	B146
U+5BC7.0	fGB	0-3F5C
U+5BC7.0	fR/S	 40.8
U+5BC7.0	fTotalStrokes	 11
U+5BC7.0	fCangjie	JMUE
U+5BC7.0	fFourCorner	30214
U+5BC7.0	fHakka	 keu5; k'ieu2; kiu5
U+5BC7.0	fCantonese	kau3
U+5BC7.0	fMandarin	kou4
U+5BC7.0	fEnglish	[1] bandits; thieves; enemy; robbers [2] invade; pillage; plunder [3] a Chinese family name
U+5BC8.0	fUTF8	寈
U+5BC8.0	fCNS11643	3-353F
U+5BC8.0	fGB	E-2768
U+5BC8.0	fR/S	 40.8
U+5BC8.0	fTotalStrokes	 11
U+5BC8.0	fCangjie	JQMB
U+5BC8.0	fFourCorner	30220
U+5BC8.0	fHakka	 ciang1; ziang1
U+5BC8.0	fCantonese	cing1; ceng1 (also)
U+5BC8.0	fMandarin	qing1; jing1
U+5BC8.0	fEnglish	ancient form of &#x9752;
U+5BC9.0	fUTF8	寉
U+5BC9.0	fGB	5-3E2A
U+5BC9.0	fR/S	 40.8
U+5BC9.0	fTotalStrokes	 11
U+5BC9.0	fCangjie	JOG
U+5BC9.0	fFourCorner	3021
U+5BC9.0	fCantonese	hok6
U+5BC9.0	fMandarin	que4; he4
U+5BC9.0	fEnglish	a bird flying high
U+5BCA.0	fUTF8	寊
U+5BCA.0	fCNS11643	2-3B39
U+5BCA.0	fBig5	D8D5
U+5BCA.0	fGB	5-3D79
U+5BCA.0	fR/S	 40.9
U+5BCA.0	fTotalStrokes	 12
U+5BCA.0	fCangjie	JYBC
U+5BCA.0	fFourCorner	30806
U+5BCA.0	fCantonese	zing1
U+5BCA.0	fMandarin	zhen1
U+5BCB.0	fUTF8	寋
U+5BCB.0	fCNS11643	2-3B36
U+5BCB.0	fBig5	D8D2
U+5BCB.0	fGB	5-337B
U+5BCB.0	fR/S	 40.9
U+5BCB.0	fTotalStrokes	 12
U+5BCB.0	fCangjie	JTCU
U+5BCB.0	fFourCorner	30712
U+5BCB.0	fHakka	 kien3; ken3
U+5BCB.0	fCantonese	gin6; gin2
U+5BCB.0	fMandarin	jian3; jian4
U+5BCB.0	fEnglish	men whose duty it is to strike the musical stones
U+5BCC.0	fUTF8	富
U+5BCC.0	fCNS11643	1-5E6E
U+5BCC.0	fBig5	B449
U+5BCC.0	fGB	0-383B
U+5BCC.0	fR/S	 40.9
U+5BCC.0	fTotalStrokes	 12
U+5BCC.0	fCangjie	JMRW
U+5BCC.0	fFourCorner	30606
U+5BCC.0	fHakka	  fu5
U+5BCC.0	fCantonese	fu3
U+5BCC.0	fMandarin	fu4
U+5BCC.0	fEnglish	[1] abundant; rich; wealthy; affluent; plentiful [2] a Chinese family name
U+5BCD.0	fUTF8	寍
U+5BCD.0	fCNS11643	2-3B35
U+5BCD.0	fBig5	D8D1
U+5BCD.0	fGB	5-3E30
U+5BCD.0	fR/S	 40.9
U+5BCD.0	fTotalStrokes	 12
U+5BCD.0	fCangjie	JPBT
U+5BCD.0	fFourCorner	30107
U+5BCD.0	fHakka	  nen2; nen3
U+5BCD.0	fCantonese	ning4; ling4 (also); ning6
U+5BCD.0	fMandarin	ning2; ning4
U+5BCD.0	fEnglish	[1] used as a term of comparison:-rather; it is better; would that [2] ancient form of &#x5BE7;
U+5BCE.0	fUTF8	寎
U+5BCE.0	fCNS11643	2-3B3A
U+5BCE.0	fBig5	D8D6
U+5BCE.0	fGB	3-3B43
U+5BCE.0	fR/S	 40.9
U+5BCE.0	fTotalStrokes	 12
U+5BCE.0	fCangjie	JVMB
U+5BCE.0	fFourCorner	30227
U+5BCE.0	fHakka	 biang5; biang3
U+5BCE.0	fCantonese	bing2 (also); bing3
U+5BCE.0	fMandarin	bing4; bing3
U+5BCE.0	fEnglish	drowsy
U+5BCF.0	fUTF8	寏
U+5BCF.0	fCNS11643	4-3724
U+5BCF.0	fGB	5-3E27
U+5BCF.0	fR/S	  40.9
U+5BCF.0	fTotalStrokes	 12
U+5BCF.0	fCangjie	JNBK
U+5BCF.0	fFourCorner	30430
U+5BCF.0	fCantonese	jyun4
U+5BCF.0	fMandarin	huan2
U+5BD0.0	fUTF8	寐
U+5BD0.0	fCNS11643	1-5E70
U+5BD0.0	fBig5	B44B
U+5BD0.0	fGB	0-4342
U+5BD0.0	fR/S	 40.9
U+5BD0.0	fTotalStrokes	 12
U+5BD0.0	fCangjie	JVMD
U+5BD0.0	fFourCorner	30295
U+5BD0.0	fHakka	 mui6;  mui3; mi5; mi3
U+5BD0.0	fCantonese	mei6
U+5BD0.0	fMandarin	mei4
U+5BD0.0	fEnglish	 sound sleep; deep sleep; be asleep; sleep; drowse; doze
U+5BD1.0	fUTF8	寑
U+5BD1.0	fCNS11643	2-3B38
U+5BD1.0	fBig5	D8D4
U+5BD1.0	fGB	E-2769
U+5BD1.0	fR/S	 40.9
U+5BD1.0	fTotalStrokes	 12
U+5BD1.0	fCangjie	JOSE
U+5BD1.0	fFourCorner	30147
U+5BD1.0	fCantonese	cam2
U+5BD1.0	fMandarin	qin3; qiu3
U+5BD2.0	fUTF8	寒
U+5BD2.0	fCNS11643	1-5E6D
U+5BD2.0	fBig5	B448
U+5BD2.0	fGB	0-3A2E
U+5BD2.0	fR/S	 40.9
U+5BD2.0	fTotalStrokes	 12
U+5BD2.0	fCangjie	JTCY
U+5BD2.0	fFourCorner	30303
U+5BD2.0	fHakka	  hon2; hon3
U+5BD2.0	fCantonese	hon4
U+5BD2.0	fMandarin	han2
U+5BD2.0	fEnglish	[1] cold; wintry; chilly [2] poor
U+5BD3.0	fUTF8	寓
U+5BD3.0	fCNS11643	1-5E6F
U+5BD3.0	fBig5	B44A
U+5BD3.0	fGB	0-5422
U+5BD3.0	fR/S	 40.9
U+5BD3.0	fTotalStrokes	 12
U+5BD3.0	fCangjie	JWLB
U+5BD3.0	fFourCorner	30427
U+5BD3.0	fHakka	  ngi5; ngi6; ngi3; gni6
U+5BD3.0	fCantonese	jyu6
U+5BD3.0	fMandarin	yu4; yu3
U+5BD3.0	fEnglish	[1] dwell; live temporarily; sojourn [2] consign
U+5BD4.0	fUTF8	寔
U+5BD4.0	fCNS11643	2-3B37
U+5BD4.0	fBig5	D8D3
U+5BD4.0	fGB	E-276A
U+5BD4.0	fR/S	 40.9
U+5BD4.0	fTotalStrokes	 12
U+5BD4.0	fCangjie	JAMO
U+5BD4.0	fFourCorner	30801
U+5BD4.0	fHakka	 shit8; siit8; sit8
U+5BD4.0	fCantonese	sat6; zik6
U+5BD4.0	fMandarin	shi2; zi2
U+5BD4.0	fEnglish	real; true; solid; honest
U+5BD5.0	fUTF8	寕
U+5BD5.0	fCNS11643	3-3A7E
U+5BD5.0	fGB	E-276B
U+5BD5.0	fR/S	 40.9
U+5BD5.0	fTotalStrokes	 12
U+5BD5.0	fCangjie	JMWN
U+5BD5.0	fFourCorner	30200
U+5BD5.0	fHakka	 nen2
U+5BD5.0	fCantonese	ning4; ling4 (also); ning6; ling6 (also)
U+5BD5.0	fMandarin	ning2; ning4
U+5BD5.0	fEnglish	 repose; serentiy; peace peaceful
U+5BD6.0	fUTF8	寖
U+5BD6.0	fCNS11643	2-424E
U+5BD6.0	fBig5	DD48
U+5BD6.0	fGB	5-3E35
U+5BD6.0	fR/S	 40.10
U+5BD6.0	fTotalStrokes	 13
U+5BD6.0	fCangjie	JESE
U+5BD6.0	fFourCorner	30147
U+5BD6.0	fHakka	  zim5
U+5BD6.0	fCantonese	zam3
U+5BD6.0	fMandarin	jin4; jin1; qin4
U+5BD6.0	fEnglish	gradually; step by step; little
U+5BD7.0	fUTF8	寗
U+5BD7.0	fCNS11643	3-4074
U+5BD7.0	fGB	E-276C
U+5BD7.0	fR/S	  40.10
U+5BD7.0	fTotalStrokes	 13
U+5BD7.0	fCangjie	JPHB
U+5BD7.0	fFourCorner	30557
U+5BD7.0	fCantonese	ning4; ling4 (also); ning6; ling6 (also)
U+5BD7.0	fMandarin	ning2; ning4
U+5BD7.0	fEnglish	[1] peaceful; rather [2] a Chinese family name
U+5BD8.0	fUTF8	寘
U+5BD8.0	fCNS11643	2-424F
U+5BD8.0	fBig5	DD49
U+5BD8.0	fGB	E-276D
U+5BD8.0	fR/S	 40.10
U+5BD8.0	fTotalStrokes	 13
U+5BD8.0	fCangjie	JJBC
U+5BD8.0	fFourCorner	30801
U+5BD8.0	fHakka	 zhi5; zih5; tje5
U+5BD8.0	fCantonese	tin4; zi3
U+5BD8.0	fMandarin	zhi4
U+5BD8.0	fEnglish	[1] put; put down; place [2] discard [3] make; cause [4] full
U+5BD9.0	fUTF8	寙
U+5BD9.0	fCNS11643	2-4250
U+5BD9.0	fBig5	DD4A
U+5BD9.0	fGB	5-3E34
U+5BD9.0	fR/S	 40.10
U+5BD9.0	fTotalStrokes	 13
U+5BD9.0	fCangjie	JHOO
U+5BD9.0	fFourCorner	30232
U+5BD9.0	fCantonese	jyu5
U+5BD9.0	fMandarin	yu3
U+5BDA.0	fUTF8	寚
U+5BDA.0	fCNS11643	3-4075
U+5BDA.0	fGB	E-276E
U+5BDA.0	fR/S	 40.10
U+5BDA.0	fTotalStrokes	 13
U+5BDA.0	fCangjie	JMGU
U+5BDA.0	fFourCorner	30770
U+5BDA.0	fCantonese	bou2
U+5BDA.0	fMandarin	bao3
U+5BDA.0	fEnglish	 precious rear valuable treasure bullion a jewel
U+5BDB.0	fUTF8	寛
U+5BDB.0	fCNS11643	3-4076
U+5BDB.0	fGB	E-276F
U+5BDB.0	fR/S	 40.10
U+5BDB.0	fTotalStrokes	 13
U+5BDB.0	fCangjie	JTBU
U+5BDB.0	fFourCorner	30210
U+5BDB.0	fHakka	 kwon1; k'wan1
U+5BDB.0	fCantonese	fun1
U+5BDB.0	fMandarin	kuan1
U+5BDB.0	fEnglish	broad
U+5BDC.0	fUTF8	寜
U+5BDC.0	fCNS11643	3-4073
U+5BDC.0	fGB	E-2770
U+5BDC.0	fR/S	 40.10
U+5BDC.0	fTotalStrokes	 13
U+5BDC.0	fCangjie	XJPBN
U+5BDC.0	fFourCorner	30200
U+5BDC.0	fCantonese	ning4; ling4 (also)
U+5BDC.0	fMandarin	ning2; ning4
U+5BDC.0	fEnglish	 repose; serentiy; peace peaceful
U+5BDD.0	fUTF8	寝
U+5BDD.0	fCNS11643	4-3C6E
U+5BDD.0	fGB	0-475E
U+5BDD.0	fR/S	 40.10
U+5BDD.0	fTotalStrokes	 13
U+5BDD.0	fCangjie	JLME
U+5BDD.0	fFourCorner	30140
U+5BDD.0	fHakka	 cim3
U+5BDD.0	fCantonese	cam2
U+5BDD.0	fMandarin	qin3; qiu3
U+5BDD.0	fEnglish	simplified form of &#x5BE2;
U+5BDE.0	fUTF8	寞
U+5BDE.0	fCNS11643	1-684E
U+5BDE.0	fBig5	B9E6
U+5BDE.0	fGB	0-442F
U+5BDE.0	fR/S	 40.11
U+5BDE.0	fTotalStrokes	 14
U+5BDE.0	fCangjie	JTAK
U+5BDE.0	fFourCorner	30430
U+5BDE.0	fHakka	  mok7; mok8; mo5
U+5BDE.0	fCantonese	mok6
U+5BDE.0	fMandarin	mo4
U+5BDE.0	fEnglish	silent; still; quiet; lonely; solitary
U+5BDF.0	fUTF8	察
U+5BDF.0	fCNS11643	1-6856
U+5BDF.0	fBig5	B9EE
U+5BDF.0	fGB	0-326C
U+5BDF.0	fR/S	 40.11
U+5BDF.0	fTotalStrokes	 14
U+5BDF.0	fCangjie	JBOF
U+5BDF.0	fFourCorner	30901
U+5BDF.0	fHakka	  cat7
U+5BDF.0	fCantonese	caat3
U+5BDF.0	fMandarin	cha2
U+5BDF.0	fEnglish	 examine; investigate; notice; observe; survey; study; scrutinise
U+5BE0.0	fUTF8	寠
U+5BE0.0	fCNS11643	2-4965
U+5BE0.0	fBig5	E17E
U+5BE0.0	fGB	5-3E2F
U+5BE0.0	fR/S	 40.11
U+5BE0.0	fTotalStrokes	 14
U+5BE0.0	fCangjie	JLLV
U+5BE0.0	fFourCorner	30404
U+5BE0.0	fHakka	  ki3
U+5BE0.0	fCantonese	geoi6; lau4; keoi6; leoi6
U+5BE0.0	fMandarin	ju4; lou2
U+5BE0.0	fEnglish	poor; impoverished
U+5BE1.0	fUTF8	寡
U+5BE1.0	fCNS11643	1-6850
U+5BE1.0	fBig5	B9E8
U+5BE1.0	fGB	0-3951
U+5BE1.0	fR/S	 40.11
U+5BE1.0	fTotalStrokes	 14
U+5BE1.0	fCangjie	JMCH
U+5BE1.0	fFourCorner	30227
U+5BE1.0	fHakka	  gwa3;  ga3; kua3
U+5BE1.0	fCantonese	gwaa2
U+5BE1.0	fMandarin	gua3; gua2
U+5BE1.0	fEnglish	[1] widowed; surviving the spouse [2] alone; lonely; solitary [3] little; few; scant; rare
U+5BE2.0	fUTF8	寢
U+5BE2.0	fCNS11643	1-6854
U+5BE2.0	fBig5	B9EC
U+5BE2.0	fGB	1-475E
U+5BE2.0	fR/S	 40.11
U+5BE2.0	fTotalStrokes	 14
U+5BE2.0	fCangjie	JVME
U+5BE2.0	fFourCorner	30247
U+5BE2.0	fHakka	  cim3; cim6
U+5BE2.0	fCantonese	cam2
U+5BE2.0	fMandarin	qin3; qiu3
U+5BE2.0	fEnglish	[1] sleep; rest [2] tomb; residence [3] stop; end
U+5BE3.0	fUTF8	寣
U+5BE3.0	fCNS11643	2-4966
U+5BE3.0	fBig5	E1A1
U+5BE3.0	fGB	3-3B48
U+5BE3.0	fR/S	 40.11
U+5BE3.0	fTotalStrokes	 14
U+5BE3.0	fCangjie	JVMR
U+5BE3.0	fFourCorner	30261
U+5BE3.0	fCantonese	fat1; fu1
U+5BE3.0	fMandarin	hu1
U+5BE3.0	fEnglish	 one sleep
U+5BE4.0	fUTF8	寤
U+5BE4.0	fCNS11643	1-6855
U+5BE4.0	fBig5	B9ED
U+5BE4.0	fGB	0-653B
U+5BE4.0	fR/S	 40.11
U+5BE4.0	fTotalStrokes	 14
U+5BE4.0	fCangjie	JVMR
U+5BE4.0	fFourCorner	30261
U+5BE4.0	fHakka	 ngu5; ngu6; ngu3
U+5BE4.0	fCantonese	ng6
U+5BE4.0	fMandarin	wu4
U+5BE4.0	fEnglish	 awake from a sound sleep
U+5BE5.0	fUTF8	寥
U+5BE5.0	fCNS11643	1-6851
U+5BE5.0	fBig5	B9E9
U+5BE5.0	fGB	0-4148
U+5BE5.0	fR/S	 40.11
U+5BE5.0	fTotalStrokes	 14
U+5BE5.0	fCangjie	JSMH
U+5BE5.0	fFourCorner	30202
U+5BE5.0	fHakka	  liau2; liau3
U+5BE5.0	fCantonese	liu4
U+5BE5.0	fMandarin	liao2; lu4
U+5BE5.0	fEnglish	[1] few; scarce; not many [2] empty; deserted; desolate
U+5BE6.0	fUTF8	實
U+5BE6.0	fCNS11643	1-6852
U+5BE6.0	fBig5	B9EA
U+5BE6.0	fGB	1-4A35
U+5BE6.0	fR/S	 40.11
U+5BE6.0	fTotalStrokes	 14
U+5BE6.0	fCangjie	JWJC
U+5BE6.0	fFourCorner	30806
U+5BE6.0	fHakka	  shit8;  sit8; siit8
U+5BE6.0	fCantonese	sat6
U+5BE6.0	fMandarin	shi2; zhi1
U+5BE6.0	fEnglish	[1] real; true; actual [2] practically
U+5BE6.1	fUTF8	實
U+5BE6.1	fCNS11643	1-6852
U+5BE6.1	fBig5	B9EA
U+5BE6.1	fGB	1-4A35
U+5BE6.1	fR/S	 40.11
U+5BE6.1	fTotalStrokes	 14
U+5BE6.1	fCangjie	JWJC
U+5BE6.1	fFourCorner	30806
U+5BE6.1	fMandarin	shi5
U+5BE7.0	fUTF8	寧
U+5BE7.0	fCNS11643	1-684F
U+5BE7.0	fBig5	B9E7
U+5BE7.0	fGB	1-447E
U+5BE7.0	fR/S	 40.11
U+5BE7.0	fTotalStrokes	 14
U+5BE7.0	fCangjie	JPBN
U+5BE7.0	fFourCorner	30201
U+5BE7.0	fHakka	  nen2; len2; lien2
U+5BE7.0	fCantonese	ning4; ning6; ling4 (also); ling6 (also)
U+5BE7.0	fMandarin	ning2; zhu4
U+5BE7.0	fEnglish	[1] repose; serenity; peace; peaceful; tranquility [2] would rather; had rather; would sooner [3] could there be
U+5BE7.1	fUTF8	寧
U+5BE7.1	fCNS11643	1-684F
U+5BE7.1	fBig5	B9E7
U+5BE7.1	fGB	1-447E
U+5BE7.1	fR/S	 40.11
U+5BE7.1	fTotalStrokes	 14
U+5BE7.1	fCangjie	JPBN
U+5BE7.1	fFourCorner	30201
U+5BE7.1	fMandarin	ning4
U+5BE8.0	fUTF8	寨
U+5BE8.0	fCNS11643	1-6853
U+5BE8.0	fBig5	B9EB
U+5BE8.0	fGB	0-552F
U+5BE8.0	fR/S	 40.11
U+5BE8.0	fTotalStrokes	 14
U+5BE8.0	fCangjie	JTCD
U+5BE8.0	fFourCorner	30904
U+5BE8.0	fHakka	  cai5; zai5; cai6
U+5BE8.0	fCantonese	zaai6; zaai2 (also); zai6
U+5BE8.0	fMandarin	zhai4; se4
U+5BE8.0	fEnglish	 stockade
U+5BE9.0	fUTF8	審
U+5BE9.0	fCNS11643	1-6C50
U+5BE9.0	fBig5	BC66
U+5BE9.0	fGB	1-4973
U+5BE9.0	fR/S	 40.12
U+5BE9.0	fTotalStrokes	 15
U+5BE9.0	fCangjie	JHDW
U+5BE9.0	fFourCorner	30609
U+5BE9.0	fHakka	  shim3;  sim3; siim3
U+5BE9.0	fCantonese	sam2
U+5BE9.0	fMandarin	shen3; pan2
U+5BE9.0	fEnglish	[1] examine; investigate; review [2] know; discern; appreciate [3] cautious; judicious; careful [4] really; indeed
U+5BEA.0	fUTF8	寪
U+5BEA.0	fCNS11643	2-3B34
U+5BEA.0	fBig5	D8D0
U+5BEA.0	fGB	3-3B38
U+5BEA.0	fR/S	 40.9
U+5BEA.0	fTotalStrokes	 12
U+5BEA.0	fCangjie	JIKF
U+5BEA.0	fFourCorner	30227
U+5BEA.0	fHakka	 wui3; wi3; vui3
U+5BEA.0	fCantonese	wai2; wai6
U+5BEA.0	fMandarin	wei3
U+5BEA.0	fEnglish	a Chinese family name
U+5BEB.0	fUTF8	寫
U+5BEB.0	fCNS11643	1-6C51
U+5BEB.0	fBig5	BC67
U+5BEB.0	fGB	1-5034
U+5BEB.0	fR/S	 40.12
U+5BEB.0	fTotalStrokes	 15
U+5BEB.0	fCangjie	JHXF
U+5BEB.0	fFourCorner	30327
U+5BEB.0	fHakka	  sia3
U+5BEB.0	fCantonese	se2; se3
U+5BEB.0	fMandarin	xie3
U+5BEB.0	fEnglish	 write; draw; sketch; compose; represent
U+5BEB.1	fUTF8	寫
U+5BEB.1	fCNS11643	1-6C51
U+5BEB.1	fBig5	BC67
U+5BEB.1	fGB	1-5034
U+5BEB.1	fR/S	 40.12
U+5BEB.1	fTotalStrokes	 15
U+5BEB.1	fCangjie	JHXF
U+5BEB.1	fFourCorner	30327
U+5BEB.1	fMandarin	xie4
U+5BEB.1	fEnglish	unrestrained
U+5BEC.0	fUTF8	寬
U+5BEC.0	fCNS11643	1-6C4F
U+5BEC.0	fBig5	BC65
U+5BEC.0	fGB	1-3F6D
U+5BEC.0	fR/S	 40.12
U+5BEC.0	fTotalStrokes	 15
U+5BEC.0	fCangjie	JTBI
U+5BEC.0	fFourCorner	30216
U+5BEC.0	fHakka	  kon1;  k'wan1; kwon1; kwan1
U+5BEC.0	fCantonese	fun1
U+5BEC.0	fMandarin	kuan1
U+5BEC.0	fEnglish	[1] broad; wide; spacious; vast [2] magnanimous; lenient; tolerant; liberal; forgiving; indulgent
U+5BED.0	fUTF8	寭
U+5BED.0	fCNS11643	4-4944
U+5BED.0	fGB	E-2771
U+5BED.0	fR/S	 40.12
U+5BED.0	fTotalStrokes	 15
U+5BED.0	fCangjie	JJIP
U+5BED.0	fFourCorner	30330
U+5BED.0	fCantonese	wai6
U+5BED.0	fMandarin	hui4
U+5BEE.0	fUTF8	寮
U+5BEE.0	fCNS11643	1-6C4E
U+5BEE.0	fBig5	BC64
U+5BEE.0	fGB	0-653C
U+5BEE.0	fR/S	 40.12
U+5BEE.0	fTotalStrokes	 15
U+5BEE.0	fCangjie	JKCF
U+5BEE.0	fFourCorner	30906
U+5BEE.0	fHakka	 liau2; liau3
U+5BEE.0	fCantonese	liu4; liu2 (also)
U+5BEE.0	fMandarin	liao2
U+5BEE.0	fEnglish	[1] fellow officer or official; colleague [2] hut; shack
U+5BEF.0	fUTF8	寯
U+5BEF.0	fCNS11643	2-5667
U+5BEF.0	fBig5	E95D
U+5BEF.0	fGB	E-2772
U+5BEF.0	fR/S	 40.13
U+5BEF.0	fTotalStrokes	 16
U+5BEF.0	fCangjie	JOGS
U+5BEF.0	fFourCorner	30227
U+5BEF.0	fCantonese	zeon3
U+5BEF.0	fMandarin	jun4
U+5BF0.0	fUTF8	寰
U+5BF0.0	fCNS11643	1-7051
U+5BF0.0	fBig5	BEC8
U+5BF0.0	fGB	0-653E
U+5BF0.0	fR/S	 40.13
U+5BF0.0	fTotalStrokes	 16
U+5BF0.0	fCangjie	JWLV
U+5BF0.0	fFourCorner	30732
U+5BF0.0	fHakka	 fan5; fan3; wan2; fan2
U+5BF0.0	fCantonese	waan4
U+5BF0.0	fMandarin	huan2; xian4
U+5BF0.0	fEnglish	 great domain; vast space
U+5BF1.0	fUTF8	寱
U+5BF1.0	fCNS11643	2-5C64
U+5BF1.0	fBig5	ECD8
U+5BF1.0	fGB	5-3E3B
U+5BF1.0	fR/S	 40.14
U+5BF1.0	fTotalStrokes	 17
U+5BF1.0	fCangjie	JVMD
U+5BF1.0	fFourCorner	30294
U+5BF1.0	fHakka	 ngi5; ngi6
U+5BF1.0	fCantonese	ngai6; ai6 (also)
U+5BF1.0	fMandarin	yi4
U+5BF1.0	fEnglish	 talk in sleep
U+5BF2.0	fUTF8	寲
U+5BF2.0	fCNS11643	2-5C65
U+5BF2.0	fBig5	ECD9
U+5BF2.0	fGB	5-3E3A
U+5BF2.0	fR/S	 40.14
U+5BF2.0	fTotalStrokes	 17
U+5BF2.0	fCangjie	JPKO
U+5BF2.0	fFourCorner	30481
U+5BF2.0	fCantonese	ji4
U+5BF2.0	fMandarin	yi2
U+5BF3.0	fUTF8	寳
U+5BF3.0	fCNS11643	3-5A35
U+5BF3.0	fGB	E-2773
U+5BF3.0	fR/S	 40.16
U+5BF3.0	fTotalStrokes	 19
U+5BF3.0	fCangjie	JMFC
U+5BF3.0	fFourCorner	30800
U+5BF3.0	fHakka	  bau3; bo3
U+5BF3.0	fCantonese	bou2
U+5BF3.0	fMandarin	bao3
U+5BF3.0	fEnglish	 treasure; jewel precious; rare
U+5BF4.0	fUTF8	寴
U+5BF4.0	fCNS11643	4-5F55
U+5BF4.0	fGB	3-3B42
U+5BF4.0	fR/S	 40.16
U+5BF4.0	fTotalStrokes	 19
U+5BF4.0	fCangjie	JYDU
U+5BF4.0	fFourCorner	30910
U+5BF4.0	fCantonese	can1
U+5BF4.0	fMandarin	qin4; qin1
U+5BF5.0	fUTF8	寵
U+5BF5.0	fCNS11643	1-7831
U+5BF5.0	fBig5	C364
U+5BF5.0	fGB	1-3368
U+5BF5.0	fR/S	 40.16
U+5BF5.0	fTotalStrokes	 19
U+5BF5.0	fCangjie	JYBP
U+5BF5.0	fFourCorner	30211
U+5BF5.0	fHakka	  cung3;  chung3
U+5BF5.0	fCantonese	cung2
U+5BF5.0	fMandarin	chong3; long2
U+5BF5.0	fEnglish	[1] favor; dote on; patronise [2] concubine [3] favour; love (especially of the emperor
U+5BF6.0	fUTF8	寶
U+5BF6.0	fCNS11643	1-796A
U+5BF6.0	fBig5	C45F
U+5BF6.0	fGB	1-3126
U+5BF6.0	fR/S	 40.17
U+5BF6.0	fTotalStrokes	 20
U+5BF6.0	fCangjie	JMUC
U+5BF6.0	fFourCorner	30806
U+5BF6.0	fHakka	 bau3;  bo3
U+5BF6.0	fCantonese	bou2
U+5BF6.0	fMandarin	bao3
U+5BF6.0	fEnglish	[1] treasure; jewel [2] precious; valuable [3] respectable; honorable
U+5BF7.0	fUTF8	寷
U+5BF7.0	fCNS11643	3-5E2A
U+5BF7.0	fGB	3-3B37
U+5BF7.0	fR/S	 40.18
U+5BF7.0	fTotalStrokes	 21
U+5BF7.0	fCangjie	JUJT
U+5BF7.0	fFourCorner	30100
U+5BF7.0	fCantonese	fung1
U+5BF7.0	fMandarin	feng1
U+5BF8.0	fUTF8	寸
U+5BF8.0	fCNS11643	1-4450
U+5BF8.0	fBig5	A46F
U+5BF8.0	fGB	0-3467
U+5BF8.0	fR/S	 41.0
U+5BF8.0	fTotalStrokes	 3
U+5BF8.0	fCangjie	DI
U+5BF8.0	fFourCorner	40300
U+5BF8.0	fHakka	  cun5
U+5BF8.0	fCantonese	cyun3; cyun2 (also)
U+5BF8.0	fMandarin	cun4
U+5BF8.0	fEnglish	[1] inch; measure of length [2] small; tiny; little [3] Kangxi radical 41
U+5BF8.1	fUTF8	寸
U+5BF8.1	fCNS11643	1-4450
U+5BF8.1	fBig5	A46F
U+5BF8.1	fGB	0-3467
U+5BF8.1	fR/S	 41.0
U+5BF8.1	fTotalStrokes	 3
U+5BF8.1	fCangjie	DI
U+5BF8.1	fFourCorner	40300
U+5BF8.1	fMandarin	cun5
U+5BF9.0	fUTF8	对
U+5BF9.0	fCNS11643	3-223E
U+5BF9.0	fGB	0-3654
U+5BF9.0	fR/S	 41.2
U+5BF9.0	fTotalStrokes	 5
U+5BF9.0	fCangjie	EDI
U+5BF9.0	fFourCorner	74400
U+5BF9.0	fHakka	 dui5
U+5BF9.0	fCantonese	deoi3; deoi2 (also)
U+5BF9.0	fMandarin	dui4
U+5BF9.0	fEnglish	simplified form of &#x5C0D;
U+5BFA.0	fUTF8	寺
U+5BFA.0	fCNS11643	1-4779
U+5BFA.0	fBig5	A678
U+5BFA.0	fGB	0-4B42
U+5BFA.0	fR/S	 41.3
U+5BFA.0	fTotalStrokes	 6
U+5BFA.0	fCangjie	GDI
U+5BFA.0	fFourCorner	40341
U+5BFA.0	fHakka	 sii5; su3; cih5; cu3; sii6; cih6; se5
U+5BFA.0	fCantonese	zi6; zi2 (also)
U+5BFA.0	fMandarin	si4
U+5BFA.0	fEnglish	 temple; monastery; mosque; shrine
U+5BFB.0	fUTF8	寻
U+5BFB.0	fGB	0-5130
U+5BFB.0	fR/S	 41.3; 58.3
U+5BFB.0	fTotalStrokes	 6
U+5BFB.0	fCangjie	SMDI
U+5BFB.0	fHakka	 cim2
U+5BFB.0	fCantonese	cam4
U+5BFB.0	fMandarin	xun2
U+5BFB.0	fEnglish	simplified form of &#x5C0B;
U+5BFB.1	fUTF8	寻
U+5BFB.1	fGB	0-5130
U+5BFB.1	fR/S	 41.3; 58.3
U+5BFB.1	fTotalStrokes	 6
U+5BFB.1	fCangjie	SMDI
U+5BFB.1	fMandarin	xin2
U+5BFC.0	fUTF8	导
U+5BFC.0	fGB	0-353C
U+5BFC.0	fR/S	 41.3
U+5BFC.0	fTotalStrokes	 6
U+5BFC.0	fCangjie	RUDI
U+5BFC.0	fHakka	 tau3
U+5BFC.0	fCantonese	dou6
U+5BFC.0	fMandarin	dao3; dao4
U+5BFC.0	fEnglish	simplified form of &#x5C0E;
U+5BFD.0	fUTF8	寽
U+5BFD.0	fCNS11643	3-255E
U+5BFD.0	fGB	5-4E67
U+5BFD.0	fR/S	 41.4
U+5BFD.0	fTotalStrokes	 7
U+5BFD.0	fCangjie	BDI
U+5BFD.0	fFourCorner	20340
U+5BFD.0	fHakka	 lot8; lut8; lot7
U+5BFD.0	fCantonese	lyut3
U+5BFD.0	fMandarin	luu4; l4; luue4
U+5BFD.0	fEnglish	 a handful pinch
U+5BFE.0	fUTF8	対
U+5BFE.0	fCNS11643	3-255C
U+5BFE.0	fGB	E-2774
U+5BFE.0	fR/S	 41.4
U+5BFE.0	fTotalStrokes	 7
U+5BFE.0	fCangjie	YKDI
U+5BFE.0	fFourCorner	04400
U+5BFE.0	fHakka	 dui5
U+5BFE.0	fCantonese	deoi3
U+5BFE.0	fMandarin	dui4
U+5BFE.0	fEnglish	simplified form of &#x5C0D;
U+5BFF.0	fUTF8	寿
U+5BFF.0	fCNS11643	3-255D
U+5BFF.0	fGB	0-4A59
U+5BFF.0	fR/S	 41.4
U+5BFF.0	fTotalStrokes	 7
U+5BFF.0	fCangjie	QKDI
U+5BFF.0	fFourCorner	50340
U+5BFF.0	fHakka	 shiu5
U+5BFF.0	fCantonese	sau6
U+5BFF.0	fMandarin	shou4
U+5BFF.0	fEnglish	simplified form of &#x58FD;
U+5C00.0	fUTF8	尀
U+5C00.0	fCNS11643	3-2850
U+5C00.0	fGB	5-375B
U+5C00.0	fR/S	 41.5; 41.6; 22.6
U+5C00.0	fTotalStrokes	 8
U+5C00.0	fCangjie	SRDI
U+5C00.0	fFourCorner	74700
U+5C00.0	fHakka	  po3
U+5C00.0	fCantonese	po2
U+5C00.0	fMandarin	po3
U+5C00.0	fEnglish	same as &#x53F5;
U+5C01.0	fUTF8	封
U+5C01.0	fCNS11643	1-506C
U+5C01.0	fBig5	ABCA
U+5C01.0	fGB	0-3762
U+5C01.0	fR/S	 41.6
U+5C01.0	fTotalStrokes	 9
U+5C01.0	fCangjie	GGDI
U+5C01.0	fFourCorner	44100
U+5C01.0	fHakka	  fung1; ga5
U+5C01.0	fCantonese	fung1
U+5C01.0	fMandarin	feng1; bian4; feng4
U+5C01.0	fEnglish	[1] a classifier for letters [2] install as a feudal lord or nobleman [3] seal; block; close completely [4] covering; wrapper; envelope
U+5C02.0	fUTF8	専
U+5C02.0	fGB	E-2775
U+5C02.0	fR/S	 41.6
U+5C02.0	fTotalStrokes	 9
U+5C02.0	fCangjie	JWDI
U+5C02.0	fHakka	 zhon1; zon1
U+5C02.0	fCantonese	zyun1
U+5C02.0	fMandarin	zhuan1
U+5C02.0	fEnglish	monopolise; take sole possession
U+5C03.0	fUTF8	尃
U+5C03.0	fCNS11643	2-2E6B
U+5C03.0	fBig5	D169
U+5C03.0	fGB	3-354B
U+5C03.0	fR/S	 41.7
U+5C03.0	fTotalStrokes	 10
U+5C03.0	fCangjie	IBDI
U+5C03.0	fFourCorner	53342
U+5C03.0	fHakka	 fu1; fu6; fu5
U+5C03.0	fCantonese	fu1
U+5C03.0	fMandarin	fu1; bu4; fu3
U+5C03.0	fEnglish	 state to;announce
U+5C04.0	fUTF8	射
U+5C04.0	fCNS11643	1-552B
U+5C04.0	fBig5	AE67
U+5C04.0	fGB	0-4964
U+5C04.0	fR/S	 41.7
U+5C04.0	fTotalStrokes	 10
U+5C04.0	fCangjie	HHDI
U+5C04.0	fFourCorner	24200
U+5C04.0	fHakka	  sa5;  sha5; jit8; sha6; rit8
U+5C04.0	fCantonese	se6; jik6; je6; sik6
U+5C04.0	fMandarin	she4; yi4; ye4; shi2
U+5C04.0	fEnglish	[1] shoot; eject [2] issue forth; emit; send out (light, heat, etc.) [3] archery (listed by Confucius as one of six arts required of a scholar)
U+5C05.0	fUTF8	尅
U+5C05.0	fCNS11643	3-3058
U+5C05.0	fGB	8-2D61
U+5C05.0	fR/S	  41.7
U+5C05.0	fTotalStrokes	 10
U+5C05.0	fCangjie	JUDI
U+5C05.0	fFourCorner	44210
U+5C05.0	fHakka	 kiet7; ket7
U+5C05.0	fCantonese	hak1; haak1 (also)
U+5C05.0	fMandarin	ke4; kei1
U+5C05.0	fEnglish	[1] subdue; destroy; overcome [2] same as &#x524B;
U+5C06.0	fUTF8	将
U+5C06.0	fCNS11643	3-3059
U+5C06.0	fGB	0-3D2B
U+5C06.0	fR/S	 41.6; 90.6; 41.7
U+5C06.0	fTotalStrokes	 9; 10
U+5C06.0	fCangjie	LMNII
U+5C06.0	fFourCorner	32140
U+5C06.0	fHakka	 ziong1; ziong5
U+5C06.0	fCantonese	zoeng1; zoeng3; coeng1
U+5C06.0	fMandarin	jiang1; qiang1; yang2
U+5C06.0	fEnglish	simplified form of &#x5C07;
U+5C06.1	fUTF8	将
U+5C06.1	fCNS11643	3-3059
U+5C06.1	fGB	0-3D2B
U+5C06.1	fR/S	 41.6; 90.6; 41.7
U+5C06.1	fTotalStrokes	 9; 10
U+5C06.1	fCangjie	LMNII
U+5C06.1	fFourCorner	32140
U+5C06.1	fMandarin	jiang4
U+5C07.0	fUTF8	將
U+5C07.0	fCNS11643	1-5972
U+5C07.0	fBig5	B14E
U+5C07.0	fGB	1-3D2B
U+5C07.0	fR/S	 41.8
U+5C07.0	fTotalStrokes	 11
U+5C07.0	fCangjie	VMBDI
U+5C07.0	fFourCorner	27242
U+5C07.0	fHakka	  ziong1
U+5C07.0	fCantonese	zoeng1; zoeng3; coeng1
U+5C07.0	fMandarin	jiang1; qiang1; yang2
U+5C07.0	fEnglish	[1] will; going to; future; about to [2] used with a noun functioning as a direct object [3] nourish
U+5C07.1	fUTF8	將
U+5C07.1	fCNS11643	1-5972
U+5C07.1	fBig5	B14E
U+5C07.1	fGB	1-3D2B
U+5C07.1	fR/S	 41.8
U+5C07.1	fTotalStrokes	 11
U+5C07.1	fCangjie	VMBDI
U+5C07.1	fFourCorner	27242
U+5C07.1	fHakka	 ziong5
U+5C07.1	fMandarin	jiang4
U+5C07.1	fEnglish	[1] general; admiral; military leader of high rank [2] lead (soldiers)
U+5C08.0	fUTF8	專
U+5C08.0	fCNS11643	1-5971
U+5C08.0	fBig5	B14D
U+5C08.0	fGB	1-5728
U+5C08.0	fR/S	 41.8
U+5C08.0	fTotalStrokes	 11
U+5C08.0	fCangjie	JIDI
U+5C08.0	fFourCorner	50343
U+5C08.0	fHakka	 zon1;  zhon1
U+5C08.0	fCantonese	zyun1
U+5C08.0	fMandarin	zhuan1; tuan1
U+5C08.0	fEnglish	[1] concentrate; focus [2] monopolise; take sole possession [3] specialise [4] exclusive; special
U+5C09.0	fUTF8	尉
U+5C09.0	fCNS11643	1-5970
U+5C09.0	fBig5	B14C
U+5C09.0	fGB	0-4E3E
U+5C09.0	fR/S	 41.8
U+5C09.0	fTotalStrokes	 11
U+5C09.0	fCangjie	SFDI
U+5C09.0	fFourCorner	74200
U+5C09.0	fHakka	 wui3; wi3; wui6; wui5; vui3
U+5C09.0	fCantonese	wai3; wat1; wan6
U+5C09.0	fMandarin	wei4
U+5C09.0	fEnglish	[1] company-grade military officer [2] a grade of military official
U+5C09.1	fUTF8	尉
U+5C09.1	fCNS11643	1-5970
U+5C09.1	fBig5	B14C
U+5C09.1	fGB	0-4E3E
U+5C09.1	fR/S	 41.8
U+5C09.1	fTotalStrokes	 11
U+5C09.1	fCangjie	SFDI
U+5C09.1	fFourCorner	74200
U+5C09.1	fMandarin	yu4
U+5C09.1	fEnglish	[1] a Chinese double family name
U+5C0A.0	fUTF8	尊
U+5C0A.0	fCNS11643	1-5E71
U+5C0A.0	fBig5	B44C
U+5C0A.0	fGB	0-5770
U+5C0A.0	fR/S	 41.9
U+5C0A.0	fTotalStrokes	 12
U+5C0A.0	fCangjie	TWDI
U+5C0A.0	fFourCorner	80346
U+5C0A.0	fHakka	  zun1
U+5C0A.0	fCantonese	zyun1; zeon1
U+5C0A.0	fMandarin	zun1
U+5C0A.0	fEnglish	[1] container for wine [2] respect; revere; venerate; honor; esteem [3] honored; honorable; noble; esteemed; respectable [4] measure word
U+5C0B.0	fUTF8	尋
U+5C0B.0	fCNS11643	1-5E72
U+5C0B.0	fBig5	B44D
U+5C0B.0	fGB	1-5130
U+5C0B.0	fR/S	 41.9
U+5C0B.0	fTotalStrokes	 12
U+5C0B.0	fCangjie	SMMRI
U+5C0B.0	fFourCorner	17346
U+5C0B.0	fHakka	  cim2; cim3
U+5C0B.0	fCantonese	cam4
U+5C0B.0	fMandarin	xun2
U+5C0B.0	fEnglish	[1] seek; search; look for [2] a measure of length in former times
U+5C0B.1	fUTF8	尋
U+5C0B.1	fCNS11643	1-5E72
U+5C0B.1	fBig5	B44D
U+5C0B.1	fGB	1-5130
U+5C0B.1	fR/S	 41.9
U+5C0B.1	fTotalStrokes	 12
U+5C0B.1	fCangjie	SMMRI
U+5C0B.1	fFourCorner	17346
U+5C0B.1	fMandarin	xin2
U+5C0B.1	fEnglish	 beg; entreat
U+5C0B.2	fUTF8	尋
U+5C0B.2	fCNS11643	1-5E72
U+5C0B.2	fBig5	B44D
U+5C0B.2	fGB	1-5130
U+5C0B.2	fR/S	 41.9
U+5C0B.2	fTotalStrokes	 12
U+5C0B.2	fCangjie	SMMRI
U+5C0B.2	fFourCorner	17346
U+5C0B.2	fMandarin	xue2
U+5C0B.2	fEnglish	 look around
U+5C0C.0	fUTF8	尌
U+5C0C.0	fCNS11643	2-3B3B
U+5C0C.0	fBig5	D8D7
U+5C0C.0	fGB	5-375E
U+5C0C.0	fR/S	 41.9
U+5C0C.0	fTotalStrokes	 12
U+5C0C.0	fCangjie	GTDI
U+5C0C.0	fFourCorner	44100
U+5C0C.0	fHakka	 shu6; su5; shu5
U+5C0C.0	fCantonese	syu6
U+5C0C.0	fMandarin	shu4; zhu4
U+5C0C.0	fEnglish	standing (something) up
U+5C0D.0	fUTF8	對
U+5C0D.0	fCNS11643	1-6857
U+5C0D.0	fBig5	B9EF
U+5C0D.0	fGB	1-3654
U+5C0D.0	fR/S	 41.11
U+5C0D.0	fTotalStrokes	 14
U+5C0D.0	fCangjie	TGDI
U+5C0D.0	fFourCorner	34100
U+5C0D.0	fHakka	  dui5
U+5C0D.0	fCantonese	deoi3; deoi2 (also)
U+5C0D.0	fMandarin	dui4
U+5C0D.0	fEnglish	[1] correct; right; proper [2] facing; opposed; parallel [3] pair; couple [4] check; ascertain [5] to; as to; with regard to [6] be directed at
U+5C0E.0	fUTF8	導
U+5C0E.0	fCNS11643	1-7052
U+5C0E.0	fBig5	BEC9
U+5C0E.0	fGB	1-353C
U+5C0E.0	fR/S	 41.12
U+5C0E.0	fTotalStrokes	 15
U+5C0E.0	fCangjie	YUDI
U+5C0E.0	fFourCorner	38343
U+5C0E.0	fHakka	 tau3; to1; tau5; to3; tau1; to6; to5
U+5C0E.0	fCantonese	dou6
U+5C0E.0	fMandarin	dao3; dao4
U+5C0E.0	fEnglish	direct; guide; lead; conduct; instruct; direct
U+5C0F.0	fUTF8	小
U+5C0F.0	fCNS11643	1-4451
U+5C0F.0	fBig5	A470
U+5C0F.0	fGB	0-5021
U+5C0F.0	fR/S	 42.0
U+5C0F.0	fTotalStrokes	 3
U+5C0F.0	fCangjie	NC
U+5C0F.0	fFourCorner	90000
U+5C0F.0	fHakka	  siau3; seu3
U+5C0F.0	fCantonese	siu2
U+5C0F.0	fMandarin	xiao3
U+5C0F.0	fEnglish	[1] small; tiny; little [2] minor [3] young; junior [4] humble; mean; lowly [5] insignificant; light; unimportant; trivial; petty
U+5C10.0	fUTF8	尐
U+5C10.0	fCNS11643	2-213C
U+5C10.0	fBig5	C95C
U+5C10.0	fGB	E-2776
U+5C10.0	fR/S	 42.1
U+5C10.0	fTotalStrokes	 4
U+5C10.0	fCangjie	FO
U+5C10.0	fFourCorner	90300
U+5C10.0	fCantonese	zit3; zik1
U+5C10.0	fMandarin	ji1; jie2
U+5C11.0	fUTF8	少
U+5C11.0	fCNS11643	1-4537
U+5C11.0	fBig5	A4D6
U+5C11.0	fGB	0-4959
U+5C11.0	fR/S	 42.1
U+5C11.0	fTotalStrokes	 4
U+5C11.0	fCangjie	FH
U+5C11.0	fFourCorner	90200
U+5C11.0	fHakka	  shau5; shau3;  sau3; seu3; seu5; s'eu3; s'eu5
U+5C11.0	fCantonese	siu2; siu3
U+5C11.0	fMandarin	shao3
U+5C11.0	fEnglish	[1] small; little; few; less (in number, quantity, duration) [2] missing; absent; inadequate; wanting; lost [3] cannot do without [4] stop; quit
U+5C11.1	fUTF8	少
U+5C11.1	fCNS11643	1-4537
U+5C11.1	fBig5	A4D6
U+5C11.1	fGB	0-4959
U+5C11.1	fR/S	 42.1
U+5C11.1	fTotalStrokes	 4
U+5C11.1	fCangjie	FH
U+5C11.1	fFourCorner	90200
U+5C11.1	fHakka	 sau5
U+5C11.1	fMandarin	shao4
U+5C11.1	fEnglish	young; youthful; junior; juvenile
U+5C11.2	fUTF8	少
U+5C11.2	fCNS11643	1-4537
U+5C11.2	fBig5	A4D6
U+5C11.2	fGB	0-4959
U+5C11.2	fR/S	 42.1
U+5C11.2	fTotalStrokes	 4
U+5C11.2	fCangjie	FH
U+5C11.2	fFourCorner	90200
U+5C11.2	fMandarin	shao5
U+5C12.0	fUTF8	尒
U+5C12.0	fCNS11643	2-2155
U+5C12.0	fBig5	C974
U+5C12.0	fGB	E-2777
U+5C12.0	fR/S	 42.2
U+5C12.0	fTotalStrokes	 5
U+5C12.0	fCangjie	OF
U+5C12.0	fFourCorner	80900
U+5C12.0	fHakka	 ni1; ni3
U+5C12.0	fCantonese	ji5
U+5C12.0	fMandarin	er3; ying1
U+5C12.0	fEnglish	you; your
U+5C13.0	fUTF8	尓
U+5C13.0	fCNS11643	4-2173
U+5C13.0	fGB	E-2778
U+5C13.0	fR/S	  42.2
U+5C13.0	fTotalStrokes	 5
U+5C13.0	fCangjie	OF
U+5C13.0	fFourCorner	80900
U+5C13.0	fHakka	 ni3
U+5C13.0	fCantonese	ji5
U+5C13.0	fMandarin	er3; ying1
U+5C13.0	fEnglish	same as &#x5C12;
U+5C14.0	fUTF8	尔
U+5C14.0	fCNS11643	3-223F
U+5C14.0	fGB	0-367B
U+5C14.0	fR/S	 42.2
U+5C14.0	fTotalStrokes	 5
U+5C14.0	fCangjie	NF
U+5C14.0	fFourCorner	27900
U+5C14.0	fHakka	  ni3; ni1; ngi3
U+5C14.0	fCantonese	ji5
U+5C14.0	fMandarin	er3; ying1
U+5C14.0	fEnglish	[1] same as &#x5C12; [2] simplified form of &#x723E;
U+5C15.0	fUTF8	尕
U+5C15.0	fCNS11643	2-2234
U+5C15.0	fBig5	C9D4
U+5C15.0	fGB	0-6658
U+5C15.0	fR/S	 42.2
U+5C15.0	fTotalStrokes	 5
U+5C15.0	fCangjie	NSF
U+5C15.0	fFourCorner	17902
U+5C15.0	fCantonese	gaa1
U+5C15.0	fMandarin	ga3
U+5C15.0	fEnglish	[1] [2] small
U+5C16.0	fUTF8	尖
U+5C16.0	fCNS11643	1-477A
U+5C16.0	fBig5	A679
U+5C16.0	fGB	0-3C62
U+5C16.0	fR/S	 42.3
U+5C16.0	fTotalStrokes	 6
U+5C16.0	fCangjie	FK
U+5C16.0	fFourCorner	90430
U+5C16.0	fHakka	  ziam1
U+5C16.0	fCantonese	zim1
U+5C16.0	fMandarin	jian1
U+5C16.0	fEnglish	sharp; pointed; acute; keen
U+5C17.0	fUTF8	尗
U+5C17.0	fCNS11643	4-225C
U+5C17.0	fGB	5-4073
U+5C17.0	fR/S	 42.3
U+5C17.0	fTotalStrokes	 6
U+5C17.0	fCangjie	YMF
U+5C17.0	fFourCorner	21900
U+5C17.0	fCantonese	suk6
U+5C17.0	fMandarin	shu2
U+5C17.0	fEnglish	 younger of brothers father's younger brother younger brother of a husband
U+5C18.0	fUTF8	尘
U+5C18.0	fCNS11643	3-2352
U+5C18.0	fGB	0-333E
U+5C18.0	fR/S	 42.3
U+5C18.0	fTotalStrokes	 6
U+5C18.0	fCangjie	FG
U+5C18.0	fFourCorner	90100
U+5C18.0	fHakka	 chin2
U+5C18.0	fCantonese	can4
U+5C18.0	fMandarin	chen2
U+5C18.0	fEnglish	[1] ancient form of &#x5875; [2] simplified form of &#x5875;
U+5C19.0	fUTF8	尙
U+5C19.0	fCNS11643	3-2851
U+5C19.0	fGB	E-2779
U+5C19.0	fR/S	 42.5
U+5C19.0	fTotalStrokes	 8
U+5C19.0	fCangjie	FBR
U+5C19.0	fFourCorner	90220
U+5C19.0	fHakka	 shong6; song5; shong5; shong3
U+5C19.0	fCantonese	soeng6
U+5C19.0	fMandarin	shang4; chang2
U+5C19.0	fEnglish	[1] still; yet; even [2] uphold; honor; esteem [3] a Chinese family name
U+5C1A.0	fUTF8	尚
U+5C1A.0	fCNS11643	1-4C7E
U+5C1A.0	fBig5	A97C
U+5C1A.0	fGB	0-4950
U+5C1A.0	fR/S	 42.5
U+5C1A.0	fTotalStrokes	 8
U+5C1A.0	fCangjie	FBR
U+5C1A.0	fFourCorner	90227
U+5C1A.0	fHakka	 song3; song5; shong6
U+5C1A.0	fCantonese	soeng6; soeng2 (also)
U+5C1A.0	fMandarin	shang4; chang2
U+5C1A.0	fEnglish	[1] still; yet; even [2] uphold; honor; esteem [3] a Chinese family name
U+5C1A.1	fUTF8	尚
U+5C1A.1	fCNS11643	1-4C7E
U+5C1A.1	fBig5	A97C
U+5C1A.1	fGB	0-4950
U+5C1A.1	fR/S	 42.5
U+5C1A.1	fTotalStrokes	 8
U+5C1A.1	fCangjie	FBR
U+5C1A.1	fFourCorner	90227
U+5C1A.1	fMandarin	shang5
U+5C1B.0	fUTF8	尛
U+5C1B.0	fCNS11643	3-2C34
U+5C1B.0	fGB	5-4076
U+5C1B.0	fR/S	  42.6
U+5C1B.0	fTotalStrokes	 9
U+5C1B.0	fCangjie	FFF
U+5C1B.0	fFourCorner	90990
U+5C1B.0	fHakka	  mo1; mo2; mak7; ma3; man3; jau1; mut7
U+5C1B.0	fCantonese	mo1; jiu1; maa1
U+5C1B.0	fMandarin	me0; ma5; me5; yao1; mo5; mo3; mo2
U+5C1B.0	fEnglish	ancient form of &#x9EBD;
U+5C1C.0	fUTF8	尜
U+5C1C.0	fGB	0-6659
U+5C1C.0	fR/S	 42.6
U+5C1C.0	fTotalStrokes	 9
U+5C1C.0	fCangjie	FKF
U+5C1C.0	fFourCorner	9090
U+5C1C.0	fCantonese	gaat3; gaa1 (also)
U+5C1C.0	fMandarin	ga2
U+5C1C.0	fEnglish	a child toy
U+5C1D.0	fUTF8	尝
U+5C1D.0	fGB	0-3322
U+5C1D.0	fR/S	 42.6
U+5C1D.0	fTotalStrokes	 9
U+5C1D.0	fCangjie	FBMMI
U+5C1D.0	fHakka	 shong2
U+5C1D.0	fCantonese	soeng4
U+5C1D.0	fMandarin	chang2
U+5C1D.0	fEnglish	simplified form of &#x5617;
U+5C1E.0	fUTF8	尞
U+5C1E.0	fCNS11643	3-3B24
U+5C1E.0	fGB	5-4079
U+5C1E.0	fR/S	 42.9
U+5C1E.0	fTotalStrokes	 12
U+5C1E.0	fCangjie	KCAF
U+5C1E.0	fFourCorner	40900
U+5C1E.0	fHakka	 liau5
U+5C1E.0	fCantonese	liu4; liu6; liu5
U+5C1E.0	fMandarin	liao4; liao2
U+5C1E.0	fEnglish	[1] a Chinese family name [2] same as &#x71CE;
U+5C1F.0	fUTF8	尟
U+5C1F.0	fCNS11643	2-4251
U+5C1F.0	fBig5	DD4B
U+5C1F.0	fGB	E-277A
U+5C1F.0	fR/S	 42.10
U+5C1F.0	fTotalStrokes	 13
U+5C1F.0	fCangjie	AOFH
U+5C1F.0	fFourCorner	49720
U+5C1F.0	fHakka	 sien1;  sien3
U+5C1F.0	fCantonese	sin2
U+5C1F.0	fMandarin	xian3
U+5C1F.0	fEnglish	 a Chinese family name fresh
U+5C20.0	fUTF8	尠
U+5C20.0	fCNS11643	3-4078
U+5C20.0	fGB	E-277B
U+5C20.0	fR/S	  42.10
U+5C20.0	fTotalStrokes	 13
U+5C20.0	fCangjie	TVFH
U+5C20.0	fFourCorner	49720
U+5C20.0	fHakka	 sien1; sien3
U+5C20.0	fCantonese	sin2
U+5C20.0	fMandarin	xian3
U+5C20.0	fEnglish	[1] very few; seldom; rarely [2] simplified form of &#x5C1F;
U+5C21.0	fUTF8	尡
U+5C21.0	fCNS11643	3-6525
U+5C21.0	fGB	E-277C
U+5C21.0	fR/S	 42.11; 72.10
U+5C21.0	fTotalStrokes	 14
U+5C21.0	fCangjie	FUAPP
U+5C21.0	fFourCorner	9621
U+5C21.0	fCantonese	kwan1
U+5C22.0	fUTF8	尢
U+5C22.0	fCNS11643	1-4452
U+5C22.0	fBig5	A471
U+5C22.0	fGB	0-5E4C
U+5C22.0	fR/S	 43.0
U+5C22.0	fTotalStrokes	 3
U+5C22.0	fCangjie	KU
U+5C22.0	fFourCorner	40010
U+5C22.0	fHakka	  wong1; ju2
U+5C22.0	fCantonese	wong1; jau4
U+5C22.0	fMandarin	you2; wang1
U+5C22.0	fEnglish	standard form of &#x5C24;
U+5C23.0	fUTF8	尣
U+5C23.0	fCNS11643	4-214B
U+5C23.0	fGB	E-277D
U+5C23.0	fR/S	 43.0
U+5C23.0	fTotalStrokes	 4; 3
U+5C23.0	fCangjie	CHU
U+5C23.0	fFourCorner	80210
U+5C23.0	fHakka	 wong1; ju2
U+5C23.0	fCantonese	wong1; jau4
U+5C23.0	fMandarin	wang1; you2
U+5C23.0	fEnglish	same as &#x5C24;
U+5C24.0	fUTF8	尤
U+5C24.0	fCNS11643	1-4538
U+5C24.0	fBig5	A4D7
U+5C24.0	fGB	0-5348
U+5C24.0	fR/S	 43.1
U+5C24.0	fTotalStrokes	 4
U+5C24.0	fCangjie	IKU
U+5C24.0	fFourCorner	43010
U+5C24.0	fHakka	  ju2; jiu2; jeu2; jiu3; riu2
U+5C24.0	fCantonese	jau4
U+5C24.0	fMandarin	you2
U+5C24.0	fEnglish	[1] especially; particularly [2] feel bitter against; reproach; blame [3] mistake; error [4] outstanding; special [5] a Chinese family name
U+5C25.0	fUTF8	尥
U+5C25.0	fCNS11643	2-2235
U+5C25.0	fBig5	C9D5
U+5C25.0	fGB	0-5E4D
U+5C25.0	fR/S	 43.3
U+5C25.0	fTotalStrokes	 6
U+5C25.0	fCangjie	KUPI
U+5C25.0	fFourCorner	47012
U+5C25.0	fHakka	 liau2; liau5
U+5C25.0	fCantonese	liu6
U+5C25.0	fMandarin	liao4
U+5C25.0	fEnglish	[1] cross the legs in walking [2] cross-kneed
U+5C26.0	fUTF8	尦
U+5C26.0	fGB	E-277E
U+5C26.0	fR/S	 43.3
U+5C26.0	fTotalStrokes	 6
U+5C26.0	fCangjie	CUPI
U+5C26.0	fCantonese	liu6
U+5C26.0	fMandarin	liao4
U+5C27.0	fUTF8	尧
U+5C27.0	fGB	0-5222
U+5C27.0	fR/S	 43.3; 10.4
U+5C27.0	fTotalStrokes	 6
U+5C27.0	fCangjie	JPMU
U+5C27.0	fHakka	 ngiau2
U+5C27.0	fCantonese	jiu4
U+5C27.0	fMandarin	yao2
U+5C27.0	fEnglish	simplified form of &#x582F;
U+5C28.0	fUTF8	尨
U+5C28.0	fCNS11643	2-235E
U+5C28.0	fBig5	CABE
U+5C28.0	fGB	3-5876
U+5C28.0	fR/S	 43.4
U+5C28.0	fTotalStrokes	 7
U+5C28.0	fCangjie	IUHHH
U+5C28.0	fFourCorner	42012
U+5C28.0	fHakka	 mang2; mong2
U+5C28.0	fCantonese	mong4; mung4; pong4
U+5C28.0	fMandarin	mang2; pang2
U+5C28.0	fEnglish	[1] shaggy haired dog; shaggy dog [2] variegated; blended
U+5C28.1	fUTF8	尨
U+5C28.1	fCNS11643	2-235E
U+5C28.1	fBig5	CABE
U+5C28.1	fGB	3-5876
U+5C28.1	fR/S	 43.4
U+5C28.1	fTotalStrokes	 7
U+5C28.1	fCangjie	IUHHH
U+5C28.1	fFourCorner	42012
U+5C28.1	fMandarin	meng2
U+5C28.1	fEnglish	disorderly; confused
U+5C29.0	fUTF8	尩
U+5C29.0	fCNS11643	4-2629
U+5C29.0	fGB	E-2821
U+5C29.0	fR/S	 43.4; 43.5
U+5C29.0	fTotalStrokes	 8; 7
U+5C29.0	fCangjie	CUMG
U+5C29.0	fFourCorner	81210
U+5C29.0	fCantonese	wong1
U+5C29.0	fMandarin	wang1
U+5C29.0	fEnglish	 weak a rickety person emaciated
U+5C2A.0	fUTF8	尪
U+5C2A.0	fCNS11643	2-235F
U+5C2A.0	fBig5	CABF
U+5C2A.0	fGB	3-353F
U+5C2A.0	fR/S	 43.4
U+5C2A.0	fTotalStrokes	 7
U+5C2A.0	fCangjie	KUMG
U+5C2A.0	fFourCorner	41011
U+5C2A.0	fHakka	 wong1; vong1
U+5C2A.0	fCantonese	wong1
U+5C2A.0	fMandarin	wang1
U+5C2A.0	fEnglish	same as &#x5C29;
U+5C2B.0	fUTF8	尫
U+5C2B.0	fCNS11643	3-255F
U+5C2B.0	fGB	E-2822
U+5C2B.0	fR/S	 43.4; 43.3; 96.3
U+5C2B.0	fTotalStrokes	 7; 8
U+5C2B.0	fCangjie	MUMG
U+5C2B.0	fFourCorner	11211
U+5C2B.0	fCantonese	wong1
U+5C2B.0	fMandarin	wang1
U+5C2B.0	fEnglish	same as &#x5C29;
U+5C2C.0	fUTF8	尬
U+5C2C.0	fCNS11643	1-497C
U+5C2C.0	fBig5	A7BC
U+5C2C.0	fGB	0-5E4E
U+5C2C.0	fR/S	 43.4
U+5C2C.0	fTotalStrokes	 7
U+5C2C.0	fCangjie	KUOLL
U+5C2C.0	fFourCorner	48012
U+5C2C.0	fHakka	 gai5; kie5; giai5
U+5C2C.0	fCantonese	gaai3
U+5C2C.0	fMandarin	ga4
U+5C2C.0	fEnglish	embarrassed; ill at ease
U+5C2D.0	fUTF8	尭
U+5C2D.0	fCNS11643	F-2554
U+5C2D.0	fGB	E-2823
U+5C2D.0	fR/S	 43.5; 10.6
U+5C2D.0	fTotalStrokes	 8
U+5C2D.0	fCangjie	JTHU
U+5C2D.0	fCantonese	jiu4
U+5C2D.0	fMandarin	yao2
U+5C2D.0	fEnglish	a legendary ancient emperor-sage
U+5C2E.0	fUTF8	尮
U+5C2E.0	fGB	E-2824
U+5C2E.0	fR/S	 43.6
U+5C2E.0	fTotalStrokes	 9
U+5C2E.0	fCangjie	KUNSD
U+5C2E.0	fFourCorner	4701
U+5C2E.0	fCantonese	do3
U+5C2E.0	fMandarin	duo4
U+5C2F.0	fUTF8	尯
U+5C2F.0	fCNS11643	3-2C35
U+5C2F.0	fGB	3-3542
U+5C2F.0	fR/S	 43.6
U+5C2F.0	fTotalStrokes	 9
U+5C2F.0	fCangjie	KUNMU
U+5C2F.0	fFourCorner	47010
U+5C2F.0	fCantonese	kwai3
U+5C2F.0	fMandarin	kui3; kui4
U+5C30.0	fUTF8	尰
U+5C30.0	fCNS11643	2-3B3C
U+5C30.0	fBig5	D8D8
U+5C30.0	fGB	5-3753
U+5C30.0	fR/S	 43.9
U+5C30.0	fTotalStrokes	 12
U+5C30.0	fCangjie	KUHJG
U+5C30.0	fFourCorner	42011
U+5C30.0	fHakka	 zhung3; zung3
U+5C30.0	fCantonese	zung2; cung3; zung6; zung1
U+5C30.0	fMandarin	zhong3; zhong4
U+5C30.0	fEnglish	swell
U+5C31.0	fUTF8	就
U+5C31.0	fCNS11643	1-5E73
U+5C31.0	fBig5	B44E
U+5C31.0	fGB	0-3E4D
U+5C31.0	fR/S	 43.9
U+5C31.0	fTotalStrokes	 12
U+5C31.0	fCangjie	YFIKU
U+5C31.0	fFourCorner	03914
U+5C31.0	fHakka	 ciu6; ciu3;  ciu5; ts'ieu5
U+5C31.0	fCantonese	zau6
U+5C31.0	fMandarin	jiu4
U+5C31.0	fEnglish	[1] receive [2] undergo [3] assume [4] follow [5] come or go [6] suit; fit; accommodate oneself [7] fortwidth; right away [8] exactly; precisely [9] namely [10] even if
U+5C32.0	fUTF8	尲
U+5C32.0	fGB	5-3755
U+5C32.0	fR/S	 43.10
U+5C32.0	fTotalStrokes	 13
U+5C32.0	fCangjie	KUTXC
U+5C32.0	fFourCorner	4801
U+5C32.0	fHakka	 giam1; gam1
U+5C32.0	fCantonese	gaam1; gaam3 (also); kam1; kam3
U+5C32.0	fMandarin	gan1
U+5C32.0	fEnglish	 limp be embarrassed
U+5C33.0	fUTF8	尳
U+5C33.0	fCNS11643	2-4252
U+5C33.0	fBig5	DD4C
U+5C33.0	fGB	5-3752
U+5C33.0	fR/S	 43.10
U+5C33.0	fTotalStrokes	 13
U+5C33.0	fCangjie	KUBBB
U+5C33.0	fFourCorner	47012
U+5C33.0	fCantonese	gwat1; wat6
U+5C33.0	fMandarin	gu3
U+5C34.0	fUTF8	尴
U+5C34.0	fGB	0-5E4F
U+5C34.0	fR/S	 43.10
U+5C34.0	fTotalStrokes	 13
U+5C34.0	fCangjie	KULIT
U+5C34.0	fHakka	 giam1
U+5C34.0	fCantonese	gaam1; gaam3
U+5C34.0	fMandarin	gan1; jian1
U+5C34.0	fEnglish	simplified form of &#x5C37;
U+5C35.0	fUTF8	尵
U+5C35.0	fCNS11643	4-4949
U+5C35.0	fGB	5-3751
U+5C35.0	fR/S	 43.12
U+5C35.0	fTotalStrokes	 15
U+5C35.0	fCangjie	KULMC
U+5C35.0	fFourCorner	45010
U+5C35.0	fHakka	 tui2; tui5
U+5C35.0	fCantonese	teoi4
U+5C35.0	fMandarin	tui2; zhuai4
U+5C35.0	fEnglish	jaded
U+5C36.0	fUTF8	尶
U+5C36.0	fCNS11643	3-5454
U+5C36.0	fGB	E-2825
U+5C36.0	fR/S	  43.14
U+5C36.0	fTotalStrokes	 17
U+5C36.0	fCangjie	MUSIT
U+5C36.0	fFourCorner	18210
U+5C36.0	fHakka	 giam1
U+5C36.0	fCantonese	gaam1; kam1; kam3
U+5C36.0	fMandarin	gan1
U+5C36.0	fEnglish	embarrassed; ill at ease
U+5C37.0	fUTF8	尷
U+5C37.0	fCNS11643	1-7353
U+5C37.0	fBig5	C0AA
U+5C37.0	fGB	1-5E4F
U+5C37.0	fR/S	 43.14
U+5C37.0	fTotalStrokes	 17
U+5C37.0	fCangjie	KUSIT
U+5C37.0	fFourCorner	48011
U+5C37.0	fHakka	 gam1;  giam1
U+5C37.0	fCantonese	gaam3 (also); gaam1
U+5C37.0	fMandarin	gan1; jian1
U+5C37.0	fEnglish	embarrassed; ill at ease
U+5C38.0	fUTF8	尸
U+5C38.0	fCNS11643	1-4453
U+5C38.0	fBig5	A472
U+5C38.0	fGB	0-4A2C
U+5C38.0	fR/S	 44.0
U+5C38.0	fTotalStrokes	 3
U+5C38.0	fCangjie	S
U+5C38.0	fFourCorner	77207
U+5C38.0	fHakka	 shi1; si1; sii1; she1
U+5C38.0	fCantonese	si1; baai1
U+5C38.0	fMandarin	shi1
U+5C38.0	fEnglish	[1] corpse [2] preside [3] Kangxi radical 44 [4] simplified form of &#x21C23;
U+5C39.0	fUTF8	尹
U+5C39.0	fCNS11643	1-4467
U+5C39.0	fBig5	A4A8
U+5C39.0	fGB	0-527C
U+5C39.0	fR/S	 44.1; 4.3
U+5C39.0	fTotalStrokes	 4
U+5C39.0	fCangjie	SK
U+5C39.0	fFourCorner	17507
U+5C39.0	fHakka	  jun3; run3; jun1
U+5C39.0	fCantonese	wan5
U+5C39.0	fMandarin	yin3
U+5C39.0	fEnglish	[1] govern; rule [2] a Chinese family name
U+5C3A.0	fUTF8	尺
U+5C3A.0	fCNS11643	1-4539
U+5C3A.0	fBig5	A4D8
U+5C3A.0	fGB	0-335F
U+5C3A.0	fR/S	 44.1
U+5C3A.0	fTotalStrokes	 4
U+5C3A.0	fCangjie	SO
U+5C3A.0	fFourCorner	77807
U+5C3A.0	fHakka	  cak7;  ch'ak7; chak7
U+5C3A.0	fCantonese	cek3; ce2; cek2 (also)
U+5C3A.0	fMandarin	chi3
U+5C3A.0	fEnglish	a Chinese measuring unit
U+5C3A.1	fUTF8	尺
U+5C3A.1	fCNS11643	1-4539
U+5C3A.1	fBig5	A4D8
U+5C3A.1	fGB	0-335F
U+5C3A.1	fR/S	 44.1
U+5C3A.1	fTotalStrokes	 4
U+5C3A.1	fCangjie	SO
U+5C3A.1	fFourCorner	77807
U+5C3A.1	fMandarin	chi2
U+5C3A.1	fEnglish	[1] ruler; rule [2] used in combination
U+5C3A.2	fUTF8	尺
U+5C3A.2	fCNS11643	1-4539
U+5C3A.2	fBig5	A4D8
U+5C3A.2	fGB	0-335F
U+5C3A.2	fR/S	 44.1
U+5C3A.2	fTotalStrokes	 4
U+5C3A.2	fCangjie	SO
U+5C3A.2	fFourCorner	77807
U+5C3A.2	fMandarin	che3
U+5C3A.2	fEnglish	a note in an old Chinese musical scale
U+5C3B.0	fUTF8	尻
U+5C3B.0	fCNS11643	2-2156
U+5C3B.0	fBig5	C975
U+5C3B.0	fGB	0-656A
U+5C3B.0	fR/S	 44.2
U+5C3B.0	fTotalStrokes	 5
U+5C3B.0	fCangjie	SKN
U+5C3B.0	fFourCorner	77217
U+5C3B.0	fHakka	 kau1; gau1
U+5C3B.0	fCantonese	haau1; hou1
U+5C3B.0	fMandarin	kao1
U+5C3B.0	fEnglish	 sacrum
U+5C3C.0	fUTF8	尼
U+5C3C.0	fCNS11643	1-4647
U+5C3C.0	fBig5	A5A7
U+5C3C.0	fGB	0-4461
U+5C3C.0	fR/S	 44.2
U+5C3C.0	fTotalStrokes	 5
U+5C3C.0	fCangjie	SP
U+5C3C.0	fFourCorner	77211
U+5C3C.0	fHakka	  ni2; li2; nit8; ni3
U+5C3C.0	fCantonese	nei4; lei4 (also); nik1; lik1
U+5C3C.0	fMandarin	ni2; ni3; ni4
U+5C3C.0	fEnglish	[1] Buddhist nun [2] used in transliteration
U+5C3D.0	fUTF8	尽
U+5C3D.0	fCNS11643	3-2354
U+5C3D.0	fGB	0-3E21
U+5C3D.0	fR/S	 44.3
U+5C3D.0	fTotalStrokes	 6
U+5C3D.0	fCangjie	SOY
U+5C3D.0	fFourCorner	77230
U+5C3D.0	fHakka	  cin5; cin6
U+5C3D.0	fCantonese	zeon6; zeon2
U+5C3D.0	fMandarin	jin3
U+5C3D.0	fEnglish	simplified form of &#x76E1;
U+5C3D.1	fUTF8	尽
U+5C3D.1	fCNS11643	3-2354
U+5C3D.1	fGB	0-3E21
U+5C3D.1	fR/S	 44.3
U+5C3D.1	fTotalStrokes	 6
U+5C3D.1	fCangjie	SOY
U+5C3D.1	fFourCorner	77230
U+5C3D.1	fMandarin	jin4
U+5C3E.0	fUTF8	尾
U+5C3E.0	fCNS11643	1-4A22
U+5C3E.0	fBig5	A7C0
U+5C3E.0	fGB	0-4E32
U+5C3E.0	fR/S	 44.4
U+5C3E.0	fTotalStrokes	 7
U+5C3E.0	fCangjie	SHQU
U+5C3E.0	fFourCorner	77214
U+5C3E.0	fHakka	  mui3; mui1; mi1; mi3; mui2; mi2
U+5C3E.0	fCantonese	mei5; mei1 (also)
U+5C3E.0	fMandarin	wei3; yi3
U+5C3E.0	fEnglish	[1] tail; rear; extremity; end; stern (of a ship) [2] last final [3] remaining [4] a Chinese family name
U+5C3F.0	fUTF8	尿
U+5C3F.0	fCNS11643	1-4A21
U+5C3F.0	fBig5	A7BF
U+5C3F.0	fGB	0-4472
U+5C3F.0	fR/S	 44.4
U+5C3F.0	fTotalStrokes	 7
U+5C3F.0	fCangjie	SE
U+5C3F.0	fFourCorner	77232
U+5C3F.0	fHakka	  ngiau5; ngiau3; ngiau6; gniau6
U+5C3F.0	fCantonese	niu6; seoi1; liu6 (also); niu2 (also); liu2 (also)
U+5C3F.0	fMandarin	niao4; sui1
U+5C3F.0	fEnglish	 urine; urinate
U+5C40.0	fUTF8	局
U+5C40.0	fCNS11643	1-497D
U+5C40.0	fBig5	A7BD
U+5C40.0	fGB	0-3E56
U+5C40.0	fR/S	 44.4
U+5C40.0	fTotalStrokes	 7
U+5C40.0	fCangjie	SSR
U+5C40.0	fFourCorner	77227
U+5C40.0	fHakka	  kiuk8
U+5C40.0	fCantonese	guk6; guk2 (also)
U+5C40.0	fMandarin	ju2
U+5C40.0	fEnglish	[1] bureau; office [2] situation; state of affairs; circumstance [3] proving [4] game
U+5C41.0	fUTF8	屁
U+5C41.0	fCNS11643	1-497E
U+5C41.0	fBig5	A7BE
U+5C41.0	fGB	0-4628
U+5C41.0	fR/S	 44.4
U+5C41.0	fTotalStrokes	 7
U+5C41.0	fCangjie	SPP
U+5C41.0	fFourCorner	77211
U+5C41.0	fHakka	  pi5
U+5C41.0	fCantonese	pei3
U+5C41.0	fMandarin	pi4
U+5C41.0	fEnglish	break wind; fart; buttocks
U+5C42.0	fUTF8	层
U+5C42.0	fGB	0-3263
U+5C42.0	fR/S	 44.4
U+5C42.0	fTotalStrokes	 7
U+5C42.0	fCangjie	SMMI
U+5C42.0	fHakka	 cen2
U+5C42.0	fCantonese	cang4
U+5C42.0	fMandarin	ceng2; zeng1
U+5C42.0	fEnglish	simplified form of &#x5C64;
U+5C43.0	fUTF8	屃
U+5C43.0	fGB	7-2123
U+5C43.0	fR/S	 44.4
U+5C43.0	fTotalStrokes	 7
U+5C43.0	fCangjie	SBO
U+5C43.0	fCantonese	ai3; ngai3 (also); hei3 (also)
U+5C43.0	fMandarin	xi4
U+5C43.0	fEnglish	gigantic strength; hercules
U+5C44.0	fUTF8	屄
U+5C44.0	fCNS11643	2-263A
U+5C44.0	fBig5	CC59
U+5C44.0	fGB	3-3B52
U+5C44.0	fR/S	 44.5
U+5C44.0	fTotalStrokes	 8
U+5C44.0	fCangjie	SJC
U+5C44.0	fFourCorner	77287
U+5C44.0	fHakka	  bi1;  bai2; biet7; bet7
U+5C44.0	fCantonese	bei1; hai1
U+5C44.0	fMandarin	bi1; bi3
U+5C44.0	fEnglish	 vagina
U+5C45.0	fUTF8	居
U+5C45.0	fCNS11643	1-4D22
U+5C45.0	fBig5	A97E
U+5C45.0	fGB	0-3E53
U+5C45.0	fR/S	 44.5
U+5C45.0	fTotalStrokes	 8
U+5C45.0	fCangjie	SJR
U+5C45.0	fFourCorner	77264
U+5C45.0	fHakka	  gi1
U+5C45.0	fCantonese	geoi1; gei1
U+5C45.0	fMandarin	ju1; ji1
U+5C45.0	fEnglish	[1] live; dwell; reside; inhabit; occupy; abode; dwelling [2] stay put; be at standstill [3] a Chinese family name
U+5C46.0	fUTF8	屆
U+5C46.0	fCNS11643	1-4D23
U+5C46.0	fBig5	A9A1
U+5C46.0	fGB	E-2826
U+5C46.0	fR/S	 44.5
U+5C46.0	fTotalStrokes	 8
U+5C46.0	fCangjie	SUG
U+5C46.0	fFourCorner	77272
U+5C46.0	fHakka	  gai5; kie5; kiai5; giai5
U+5C46.0	fCantonese	gaai3
U+5C46.0	fMandarin	jie4
U+5C46.0	fEnglish	[1] numerary adjunct for periodic terms or events [2] arrive (said of an appointed date) [3] expire (said of a term)
U+5C47.0	fUTF8	屇
U+5C47.0	fCNS11643	2-263B
U+5C47.0	fBig5	CC5A
U+5C47.0	fGB	5-3E4F
U+5C47.0	fR/S	 44.5
U+5C47.0	fTotalStrokes	 8
U+5C47.0	fCangjie	SW
U+5C47.0	fFourCorner	77267
U+5C47.0	fHakka	 giai5
U+5C47.0	fCantonese	tin4
U+5C47.0	fMandarin	tian2; tian1
U+5C47.0	fEnglish	 cave hole
U+5C48.0	fUTF8	屈
U+5C48.0	fCNS11643	1-4D21
U+5C48.0	fBig5	A97D
U+5C48.0	fGB	0-477C
U+5C48.0	fR/S	 44.5
U+5C48.0	fTotalStrokes	 8
U+5C48.0	fCangjie	SUU
U+5C48.0	fFourCorner	77272
U+5C48.0	fHakka	 kiut7;  kut7
U+5C48.0	fCantonese	wat1; gwat6
U+5C48.0	fMandarin	qu1; que4
U+5C48.0	fEnglish	[1] bend; flex; bent; crook; bow [2] humiliate; humble; submit; subdue [3] wrong; injustice; [4] in the wrong [5] be in an inferior or uncomfortable position [6] a Chinese family name
U+5C48.1	fUTF8	屈
U+5C48.1	fCNS11643	1-4D21
U+5C48.1	fBig5	A97D
U+5C48.1	fGB	0-477C
U+5C48.1	fR/S	 44.5
U+5C48.1	fTotalStrokes	 8
U+5C48.1	fCangjie	SUU
U+5C48.1	fFourCorner	77272
U+5C48.1	fMandarin	qu5
U+5C49.0	fUTF8	屉
U+5C49.0	fCNS11643	3-2854
U+5C49.0	fGB	0-4C6B
U+5C49.0	fR/S	  44.5
U+5C49.0	fTotalStrokes	 8
U+5C49.0	fCangjie	SPT
U+5C49.0	fFourCorner	77210
U+5C49.0	fHakka	 ti5
U+5C49.0	fCantonese	tai3
U+5C49.0	fMandarin	ti4
U+5C49.0	fEnglish	 drawer tray pad screen
U+5C4A.0	fUTF8	届
U+5C4A.0	fCNS11643	3-2855
U+5C4A.0	fGB	0-3D6C
U+5C4A.0	fR/S	 44.5
U+5C4A.0	fTotalStrokes	 8
U+5C4A.0	fCangjie	SLW
U+5C4A.0	fFourCorner	77260
U+5C4A.0	fHakka	 gai5; kie5; kiai5
U+5C4A.0	fCantonese	gaai3
U+5C4A.0	fMandarin	jie4
U+5C4A.0	fEnglish	numerary adjunct for time, term
U+5C4B.0	fUTF8	屋
U+5C4B.0	fCNS11643	1-5070
U+5C4B.0	fBig5	ABCE
U+5C4B.0	fGB	0-4E5D
U+5C4B.0	fR/S	 44.6
U+5C4B.0	fTotalStrokes	 9
U+5C4B.0	fCangjie	SMIG
U+5C4B.0	fFourCorner	77214
U+5C4B.0	fHakka	  wuk7; vuk7
U+5C4B.0	fCantonese	uk1; nguk1 (also); nuk1; guk1
U+5C4B.0	fMandarin	wu1
U+5C4B.0	fEnglish	 house; room; building; shelter
U+5C4C.0	fUTF8	屌
U+5C4C.0	fCNS11643	2-2979
U+5C4C.0	fBig5	CE78
U+5C4C.0	fGB	3-3B53
U+5C4C.0	fR/S	 44.6
U+5C4C.0	fTotalStrokes	 9
U+5C4C.0	fCangjie	SRLB
U+5C4C.0	fFourCorner	77227
U+5C4C.0	fHakka	  diau3
U+5C4C.0	fCantonese	diu2
U+5C4C.0	fMandarin	diao3
U+5C4C.0	fEnglish	[1] the penis [2] a term much used in vile scolding [3] have carnal intercourse
U+5C4D.0	fUTF8	屍
U+5C4D.0	fCNS11643	1-506F
U+5C4D.0	fBig5	ABCD
U+5C4D.0	fGB	E-2827
U+5C4D.0	fR/S	 44.6
U+5C4D.0	fTotalStrokes	 9
U+5C4D.0	fCangjie	SMNP
U+5C4D.0	fFourCorner	77211
U+5C4D.0	fHakka	 si1; shi1; sii1; she1
U+5C4D.0	fCantonese	si1
U+5C4D.0	fMandarin	shi1; shi4
U+5C4D.0	fEnglish	 corpse; carcass
U+5C4E.0	fUTF8	屎
U+5C4E.0	fCNS11643	1-506D
U+5C4E.0	fBig5	ABCB
U+5C4E.0	fGB	0-4A3A
U+5C4E.0	fR/S	 44.6
U+5C4E.0	fTotalStrokes	 9
U+5C4E.0	fCangjie	SFD
U+5C4E.0	fFourCorner	77294
U+5C4E.0	fHakka	 si3;  shi3; sii3; she3
U+5C4E.0	fCantonese	si2; hei1
U+5C4E.0	fMandarin	shi3; xi1
U+5C4E.0	fEnglish	 excrement; dung
U+5C4F.0	fUTF8	屏
U+5C4F.0	fCNS11643	1-506E
U+5C4F.0	fBig5	ABCC
U+5C4F.0	fGB	0-4641
U+5C4F.0	fR/S	 44.6
U+5C4F.0	fTotalStrokes	 9
U+5C4F.0	fCangjie	STT
U+5C4F.0	fFourCorner	77241
U+5C4F.0	fHakka	 pin2; pin3
U+5C4F.0	fCantonese	ping4; bing2; bing1; ping2 (also); bing3
U+5C4F.0	fMandarin	ping2; bing3; bing4
U+5C4F.0	fEnglish	folding screen, shield
U+5C50.0	fUTF8	屐
U+5C50.0	fCNS11643	1-552E
U+5C50.0	fBig5	AE6A
U+5C50.0	fGB	0-656C
U+5C50.0	fR/S	 44.7
U+5C50.0	fTotalStrokes	 10
U+5C50.0	fCangjie	SHOE
U+5C50.0	fFourCorner	77247
U+5C50.0	fHakka	  kiak8; ki5; hi5
U+5C50.0	fCantonese	kek6
U+5C50.0	fMandarin	ji1; ji4
U+5C50.0	fEnglish	wooden shoes; clogs
U+5C51.0	fUTF8	屑
U+5C51.0	fCNS11643	1-552C
U+5C51.0	fBig5	AE68
U+5C51.0	fGB	0-503C
U+5C51.0	fR/S	 44.7
U+5C51.0	fTotalStrokes	 10
U+5C51.0	fCangjie	SFB
U+5C51.0	fFourCorner	77227
U+5C51.0	fHakka	  siet7;  sut7; sit7; set7; siau1
U+5C51.0	fCantonese	sit3
U+5C51.0	fMandarin	xie4
U+5C51.0	fEnglish	[1] bits; chips; crumbs; fragments; odds and ends; trifles [2] care; mind
U+5C52.0	fUTF8	屒
U+5C52.0	fCNS11643	3-305A
U+5C52.0	fGB	3-3B56
U+5C52.0	fR/S	  44.7
U+5C52.0	fTotalStrokes	 10
U+5C52.0	fCangjie	SMMV
U+5C52.0	fFourCorner	77230
U+5C52.0	fCantonese	zan2
U+5C52.0	fMandarin	chen2
U+5C53.0	fUTF8	屓
U+5C53.0	fCNS11643	3-305B
U+5C53.0	fGB	3-3B4D
U+5C53.0	fR/S	 44.7
U+5C53.0	fTotalStrokes	 10
U+5C53.0	fCangjie	SBUC
U+5C53.0	fFourCorner	77280
U+5C53.0	fHakka	 kiak8; hi5
U+5C53.0	fCantonese	ai3; ngai3 (also); hei3 (also)
U+5C53.0	fMandarin	xi4
U+5C53.0	fEnglish	 gigantic strength Hercules
U+5C54.0	fUTF8	屔
U+5C54.0	fCNS11643	2-2E6D
U+5C54.0	fBig5	D16B
U+5C54.0	fGB	5-3E58
U+5C54.0	fR/S	 44.7
U+5C54.0	fTotalStrokes	 10
U+5C54.0	fCangjie	OMSP
U+5C54.0	fFourCorner	07111
U+5C54.0	fCantonese	nai4; lai4 (also); nei4
U+5C54.0	fMandarin	ni2
U+5C55.0	fUTF8	展
U+5C55.0	fCNS11643	1-552D
U+5C55.0	fBig5	AE69
U+5C55.0	fGB	0-5539
U+5C55.0	fR/S	 44.7
U+5C55.0	fTotalStrokes	 10
U+5C55.0	fCangjie	STV
U+5C55.0	fFourCorner	77232
U+5C55.0	fHakka	 zen3; dien3; zhan3; zien3; zan3; zhen3
U+5C55.0	fCantonese	zin2
U+5C55.0	fMandarin	zhan3
U+5C55.0	fEnglish	[1] open [2] unfold; unroll [3] stretch; extend [4] expand; dilate [5] prolong [6] visit
U+5C55.1	fUTF8	展
U+5C55.1	fCNS11643	1-552D
U+5C55.1	fBig5	AE69
U+5C55.1	fGB	0-5539
U+5C55.1	fR/S	 44.7
U+5C55.1	fTotalStrokes	 10
U+5C55.1	fCangjie	STV
U+5C55.1	fFourCorner	77232
U+5C55.1	fMandarin	zhan5
U+5C56.0	fUTF8	屖
U+5C56.0	fCNS11643	2-2E6C
U+5C56.0	fBig5	D16A
U+5C56.0	fGB	3-3B58
U+5C56.0	fR/S	 44.7
U+5C56.0	fTotalStrokes	 10
U+5C56.0	fCangjie	SYTJ
U+5C56.0	fFourCorner	77241
U+5C56.0	fCantonese	sai1; cai1
U+5C56.0	fMandarin	xi1
U+5C57.0	fUTF8	屗
U+5C57.0	fCNS11643	3-646B
U+5C57.0	fGB	E-2828
U+5C57.0	fR/S	 44.7; 41.7
U+5C57.0	fTotalStrokes	 10
U+5C57.0	fCangjie	SUDI
U+5C57.0	fFourCorner	7421
U+5C57.0	fCantonese	cyun3
U+5C58.0	fUTF8	屘
U+5C58.0	fCNS11643	1-5522
U+5C58.0	fBig5	AE5E
U+5C58.0	fGB	E-2829
U+5C58.0	fR/S	 44.7; 39.7
U+5C58.0	fTotalStrokes	 10
U+5C58.0	fCangjie	SUND
U+5C58.0	fFourCorner	7121
U+5C58.0	fCantonese	mei1
U+5C58.0	fMandarin	man3
U+5C58.0	fEnglish	the youngest
U+5C59.0	fUTF8	屙
U+5C59.0	fCNS11643	2-3476
U+5C59.0	fBig5	D4F3
U+5C59.0	fGB	0-656D
U+5C59.0	fR/S	 44.8
U+5C59.0	fTotalStrokes	 11
U+5C59.0	fCangjie	SNLR
U+5C59.0	fFourCorner	77221
U+5C59.0	fHakka	  o1
U+5C59.0	fCantonese	ngo1 (also); o1
U+5C59.0	fMandarin	e1; dong1
U+5C59.0	fEnglish	 to ease nature; discharge excrement or urine
U+5C5A.0	fUTF8	屚
U+5C5A.0	fCNS11643	4-3135
U+5C5A.0	fGB	5-3E5B
U+5C5A.0	fR/S	  44.8
U+5C5A.0	fTotalStrokes	 11
U+5C5A.0	fCangjie	SMLB
U+5C5A.0	fFourCorner	77220
U+5C5A.0	fCantonese	lau6
U+5C5A.0	fMandarin	lou4
U+5C5B.0	fUTF8	屛
U+5C5B.0	fCNS11643	3-3543
U+5C5B.0	fGB	E-282A
U+5C5B.0	fR/S	 44.8
U+5C5B.0	fTotalStrokes	 11
U+5C5B.0	fCangjie	SYJJ
U+5C5B.0	fFourCorner	77240
U+5C5B.0	fHakka	  pin2; bin5
U+5C5B.0	fCantonese	ping4; bing2; bing3; bing1
U+5C5B.0	fMandarin	ping2; bing4; bing1
U+5C5B.0	fEnglish	 folding screen; screen; shield; shield; screen; guard
U+5C5B.1	fUTF8	屛
U+5C5B.1	fCNS11643	3-3543
U+5C5B.1	fGB	E-282A
U+5C5B.1	fR/S	 44.8
U+5C5B.1	fTotalStrokes	 11
U+5C5B.1	fCangjie	SYJJ
U+5C5B.1	fFourCorner	77240
U+5C5B.1	fMandarin	bing3
U+5C5B.1	fEnglish	 reject; discard; dismiss; get rid of; abandon
U+5C5C.0	fUTF8	屜
U+5C5C.0	fCNS11643	1-5974
U+5C5C.0	fBig5	B150
U+5C5C.0	fGB	5-3E5D
U+5C5C.0	fR/S	 44.8
U+5C5C.0	fTotalStrokes	 11
U+5C5C.0	fCangjie	SHOT
U+5C5C.0	fFourCorner	77217
U+5C5C.0	fHakka	  ti5; tai5
U+5C5C.0	fCantonese	tai3
U+5C5C.0	fMandarin	ti4; xie4
U+5C5C.0	fEnglish	 drawer
U+5C5D.0	fUTF8	屝
U+5C5D.0	fCNS11643	1-5975
U+5C5D.0	fBig5	B151
U+5C5D.0	fGB	3-3B5A
U+5C5D.0	fR/S	 44.8
U+5C5D.0	fTotalStrokes	 11
U+5C5D.0	fCangjie	SLMY
U+5C5D.0	fFourCorner	77211
U+5C5D.0	fHakka	 fui1; fi1; fui5
U+5C5D.0	fCantonese	fei2; fai6; fei6
U+5C5D.0	fMandarin	fei4
U+5C5D.0	fEnglish	 straw shoes straw door
U+5C5E.0	fUTF8	属
U+5C5E.0	fCNS11643	3-3B26
U+5C5E.0	fGB	0-4A74
U+5C5E.0	fR/S	 44.9
U+5C5E.0	fTotalStrokes	 12
U+5C5E.0	fCangjie	SHLB
U+5C5E.0	fFourCorner	77220
U+5C5E.0	fHakka	 shuk8; suk8; zhuk7; chuk7
U+5C5E.0	fCantonese	suk6; zuk1
U+5C5E.0	fMandarin	shu3; zhu4; zhu3
U+5C5E.0	fEnglish	[1] non-classical form of &#x5C6C; [2] simplified form of &#x5C6C;
U+5C5F.0	fUTF8	屟
U+5C5F.0	fCNS11643	3-3B25
U+5C5F.0	fGB	5-3E5E
U+5C5F.0	fR/S	 44.9
U+5C5F.0	fTotalStrokes	 12
U+5C5F.0	fCangjie	SPTD
U+5C5F.0	fFourCorner	77290
U+5C5F.0	fCantonese	sit3; sip3 (also)
U+5C5F.0	fMandarin	ti4; xie4
U+5C60.0	fUTF8	屠
U+5C60.0	fCNS11643	1-5973
U+5C60.0	fBig5	B14F
U+5C60.0	fGB	0-4D40
U+5C60.0	fR/S	 44.9
U+5C60.0	fTotalStrokes	 12
U+5C60.0	fCangjie	SJKA
U+5C60.0	fFourCorner	77264
U+5C60.0	fHakka	  tu2; tu3
U+5C60.0	fCantonese	tou4; ceoi4
U+5C60.0	fMandarin	tu2; chu3
U+5C60.0	fEnglish	[1] butcher; slaughter; massacre [2] a Chinese family name
U+5C61.0	fUTF8	屡
U+5C61.0	fCNS11643	3-3544
U+5C61.0	fGB	0-4245
U+5C61.0	fR/S	 44.9
U+5C61.0	fTotalStrokes	 12; 11
U+5C61.0	fCangjie	SFDV
U+5C61.0	fFourCorner	77240
U+5C61.0	fHakka	 li5; lui1; lui3
U+5C61.0	fCantonese	leoi5
U+5C61.0	fMandarin	luu3; l3; lu3
U+5C61.0	fEnglish	[1] non-classical form of &#x5C62; [2] simplified form of &#x5C62;
U+5C62.0	fUTF8	屢
U+5C62.0	fCNS11643	1-6858
U+5C62.0	fBig5	B9F0
U+5C62.0	fGB	1-4245
U+5C62.0	fR/S	 44.11
U+5C62.0	fTotalStrokes	 14
U+5C62.0	fCangjie	SLLV
U+5C62.0	fFourCorner	77244
U+5C62.0	fHakka	 lui1; lui3; li5
U+5C62.0	fCantonese	leoi5
U+5C62.0	fMandarin	l3; luu3; lu3
U+5C62.0	fEnglish	frequently; often; again and again; repeatedly; time after time; time and again
U+5C63.0	fUTF8	屣
U+5C63.0	fCNS11643	2-4967
U+5C63.0	fBig5	E1A2
U+5C63.0	fGB	0-656F
U+5C63.0	fR/S	 44.11
U+5C63.0	fTotalStrokes	 14
U+5C63.0	fCangjie	SHOO
U+5C63.0	fFourCorner	77281
U+5C63.0	fHakka	  sai3; si3
U+5C63.0	fCantonese	saai2
U+5C63.0	fMandarin	xi3; xi4
U+5C63.0	fEnglish	shoes; sandals
U+5C64.0	fUTF8	層
U+5C64.0	fCNS11643	1-6C52
U+5C64.0	fBig5	BC68
U+5C64.0	fGB	1-3263
U+5C64.0	fR/S	 44.12; 44.11
U+5C64.0	fTotalStrokes	 15
U+5C64.0	fCangjie	SCWA
U+5C64.0	fFourCorner	77266
U+5C64.0	fHakka	  cen2; zan5; cien2; cen3; cen5
U+5C64.0	fCantonese	cang4
U+5C64.0	fMandarin	ceng2; zeng1
U+5C64.0	fEnglish	 story (of a building); layer; floor; stratum
U+5C65.0	fUTF8	履
U+5C65.0	fCNS11643	1-6C53
U+5C65.0	fBig5	BC69
U+5C65.0	fGB	0-4244
U+5C65.0	fR/S	 44.12
U+5C65.0	fTotalStrokes	 15
U+5C65.0	fCangjie	SHOE
U+5C65.0	fFourCorner	77247
U+5C65.0	fHakka	  li3;  li1
U+5C65.0	fCantonese	lei5; leoi5 (also)
U+5C65.0	fMandarin	l3; luu3; li3; lu3
U+5C65.0	fEnglish	[1] footwear; shoes [2] walk; walk on; tread on; step on; follow
U+5C66.0	fUTF8	屦
U+5C66.0	fGB	0-6570
U+5C66.0	fR/S	 44.12
U+5C66.0	fTotalStrokes	 15
U+5C66.0	fCangjie	XSHOV
U+5C66.0	fHakka	 li5
U+5C66.0	fCantonese	geoi3
U+5C66.0	fMandarin	ju4
U+5C66.0	fEnglish	simplified form of &#x5C68;
U+5C67.0	fUTF8	屧
U+5C67.0	fCNS11643	2-502A
U+5C67.0	fBig5	E561
U+5C67.0	fGB	5-3E69
U+5C67.0	fR/S	 44.12
U+5C67.0	fTotalStrokes	 15
U+5C67.0	fCangjie	SHOD
U+5C67.0	fFourCorner	77294
U+5C67.0	fHakka	 siet7; tiap8; siap8
U+5C67.0	fCantonese	sit3; sip3 (also)
U+5C67.0	fMandarin	xie4; ti4
U+5C67.0	fEnglish	wooden shoes; clogs
U+5C68.0	fUTF8	屨
U+5C68.0	fCNS11643	1-7354
U+5C68.0	fBig5	C0AB
U+5C68.0	fGB	1-6570
U+5C68.0	fR/S	 44.14
U+5C68.0	fTotalStrokes	 17
U+5C68.0	fCangjie	SHOV
U+5C68.0	fFourCorner	77247
U+5C68.0	fHakka	 li5; gi5
U+5C68.0	fCantonese	geoi3
U+5C68.0	fMandarin	ju4
U+5C68.0	fEnglish	 sandals; straw sandals; shoes made of coarse material
U+5C69.0	fUTF8	屩
U+5C69.0	fCNS11643	2-614E
U+5C69.0	fBig5	EFC2
U+5C69.0	fGB	3-3B5C
U+5C69.0	fR/S	 44.15
U+5C69.0	fTotalStrokes	 18
U+5C69.0	fCangjie	SHOB
U+5C69.0	fFourCorner	77227
U+5C69.0	fHakka	  giok7
U+5C69.0	fCantonese	goek3
U+5C69.0	fMandarin	jue1; jue2; ju2
U+5C69.0	fEnglish	straw sandals
U+5C6A.0	fUTF8	屪
U+5C6A.0	fCNS11643	2-614F
U+5C6A.0	fBig5	EFC3
U+5C6A.0	fGB	5-3E6C
U+5C6A.0	fR/S	 44.15
U+5C6A.0	fTotalStrokes	 18
U+5C6A.0	fCangjie	SJKF
U+5C6A.0	fFourCorner	77296
U+5C6A.0	fHakka	 liau2
U+5C6A.0	fCantonese	liu4
U+5C6A.0	fMandarin	liao2
U+5C6A.0	fEnglish	penis
U+5C6B.0	fUTF8	屫
U+5C6B.0	fCNS11643	3-5A38
U+5C6B.0	fGB	5-3E65
U+5C6B.0	fR/S	 44.16
U+5C6B.0	fTotalStrokes	 19
U+5C6B.0	fCangjie	SVMB
U+5C6B.0	fFourCorner	77220
U+5C6B.0	fHakka	  giok7
U+5C6B.0	fCantonese	goek3
U+5C6B.0	fMandarin	jue1; jue2; ju2
U+5C6B.0	fEnglish	same as &#x5C69;
U+5C6C.0	fUTF8	屬
U+5C6C.0	fCNS11643	1-7A68
U+5C6C.0	fBig5	C4DD
U+5C6C.0	fGB	1-4A74
U+5C6C.0	fR/S	 44.18
U+5C6C.0	fTotalStrokes	 21
U+5C6C.0	fCangjie	SEWI
U+5C6C.0	fFourCorner	77227
U+5C6C.0	fHakka	  suk8; shuk8; zhuk7; chuk7
U+5C6C.0	fCantonese	suk6; zuk1; zuk6
U+5C6C.0	fMandarin	shu3; zhu4
U+5C6C.0	fEnglish	[1] class; category; type; kind [2] belong to; subordinate to; be governed by
U+5C6C.1	fUTF8	屬
U+5C6C.1	fCNS11643	1-7A68
U+5C6C.1	fBig5	C4DD
U+5C6C.1	fGB	1-4A74
U+5C6C.1	fR/S	 44.18
U+5C6C.1	fTotalStrokes	 21
U+5C6C.1	fCangjie	SEWI
U+5C6C.1	fFourCorner	77227
U+5C6C.1	fMandarin	zhu3
U+5C6C.1	fEnglish	[1] compose (a piece of writing) [2] instruct; direct
U+5C6D.0	fUTF8	屭
U+5C6D.0	fCNS11643	2-7036
U+5C6D.0	fBig5	F8A8
U+5C6D.0	fGB	5-3E68
U+5C6D.0	fR/S	 44.21
U+5C6D.0	fTotalStrokes	 24
U+5C6D.0	fCangjie	SBCC
U+5C6D.0	fFourCorner	77286
U+5C6D.0	fHakka	  hi5
U+5C6D.0	fCantonese	ai3; ngai3 (also); hei3 (also)
U+5C6D.0	fMandarin	xi4
U+5C6D.0	fEnglish	 gigantic strength Hercules
U+5C6E.0	fUTF8	屮
U+5C6E.0	fCNS11643	2-212B
U+5C6E.0	fBig5	C94B
U+5C6E.0	fGB	0-6578
U+5C6E.0	fR/S	 45.0
U+5C6E.0	fTotalStrokes	 3
U+5C6E.0	fCangjie	UL
U+5C6E.0	fFourCorner	22407
U+5C6E.0	fHakka	 co3; chet8; cet8; che5; ce5
U+5C6E.0	fCantonese	cit3
U+5C6E.0	fMandarin	che4; cao3
U+5C6E.0	fEnglish	 sprout Kangxi radical number 45
U+5C6F.0	fUTF8	屯
U+5C6F.0	fCNS11643	1-453A
U+5C6F.0	fBig5	A4D9
U+5C6F.0	fGB	0-4D4D
U+5C6F.0	fR/S	 45.1
U+5C6F.0	fTotalStrokes	 4
U+5C6F.0	fCangjie	PU
U+5C6F.0	fFourCorner	50717
U+5C6F.0	fHakka	  tun2; zun1; zhun1
U+5C6F.0	fCantonese	tyun4; zeon1; seon4
U+5C6F.0	fMandarin	tun2; zhun1
U+5C6F.0	fEnglish	[1] camp; station (army) [2] stockpile
U+5C70.0	fUTF8	屰
U+5C70.0	fCNS11643	4-225F
U+5C70.0	fGB	5-3139
U+5C70.0	fR/S	  45.3
U+5C70.0	fTotalStrokes	 6
U+5C70.0	fCangjie	TU
U+5C70.0	fFourCorner	80740
U+5C70.0	fHakka	  ngiak8
U+5C70.0	fCantonese	jik6
U+5C70.0	fMandarin	ni4
U+5C70.0	fEnglish	disobedient
U+5C71.0	fUTF8	山
U+5C71.0	fCNS11643	1-4454
U+5C71.0	fBig5	A473
U+5C71.0	fGB	0-493D
U+5C71.0	fR/S	 46.0
U+5C71.0	fTotalStrokes	 3
U+5C71.0	fCangjie	U
U+5C71.0	fFourCorner	22770
U+5C71.0	fHakka	  san1
U+5C71.0	fCantonese	saan1
U+5C71.0	fMandarin	shan1
U+5C71.0	fEnglish	 mountain; hill; peak
U+5C72.0	fUTF8	屲
U+5C72.0	fGB	5-3A53
U+5C72.0	fR/S	 46.1
U+5C72.0	fTotalStrokes	 4
U+5C72.0	fCangjie	HU
U+5C72.0	fCantonese	saan1
U+5C72.0	fMandarin	gai4
U+5C73.0	fUTF8	屳
U+5C73.0	fCNS11643	2-2158
U+5C73.0	fBig5	C977
U+5C73.0	fGB	5-3142
U+5C73.0	fR/S	 46.2
U+5C73.0	fTotalStrokes	 5
U+5C73.0	fCangjie	OU
U+5C73.0	fFourCorner	80770
U+5C73.0	fHakka	 sen1
U+5C73.0	fCantonese	sin1
U+5C73.0	fMandarin	xian1
U+5C74.0	fUTF8	屴
U+5C74.0	fCNS11643	2-2157
U+5C74.0	fBig5	C976
U+5C74.0	fGB	3-383F
U+5C74.0	fR/S	 46.2
U+5C74.0	fTotalStrokes	 5
U+5C74.0	fCangjie	UKS
U+5C74.0	fFourCorner	22227
U+5C74.0	fHakka	  lit8
U+5C74.0	fCantonese	lik6; lak6 (also)
U+5C74.0	fMandarin	li4; le4
U+5C74.0	fEnglish	lofty
U+5C75.0	fUTF8	屵
U+5C75.0	fCNS11643	4-2176
U+5C75.0	fGB	5-3A54
U+5C75.0	fR/S	  46.2
U+5C75.0	fTotalStrokes	 5
U+5C75.0	fCangjie	UM
U+5C75.0	fFourCorner	22200
U+5C75.0	fCantonese	ngaat6; aat6 (also)
U+5C75.0	fMandarin	xue1; e4; yan3
U+5C76.0	fUTF8	屶
U+5C76.0	fGB	E-282B
U+5C76.0	fR/S	 46.2
U+5C76.0	fTotalStrokes	 5
U+5C76.0	fCangjie	USH
U+5C76.0	fCantonese	wui6
U+5C76.0	fEnglish	 lofty
U+5C77.0	fUTF8	屷
U+5C77.0	fCNS11643	3-646C
U+5C77.0	fGB	E-282C
U+5C77.0	fR/S	 46.2
U+5C77.0	fTotalStrokes	 5
U+5C77.0	fCangjie	UNHS
U+5C77.0	fFourCorner	2772
U+5C77.0	fCantonese	wui6
U+5C78.0	fUTF8	屸
U+5C78.0	fCNS11643	3-2355
U+5C78.0	fGB	3-3840
U+5C78.0	fR/S	  46.3
U+5C78.0	fTotalStrokes	 6
U+5C78.0	fCangjie	UM
U+5C78.0	fFourCorner	21710
U+5C78.0	fCantonese	lung4
U+5C78.0	fMandarin	long2
U+5C79.0	fUTF8	屹
U+5C79.0	fCNS11643	1-477B
U+5C79.0	fBig5	A67A
U+5C79.0	fGB	0-5259
U+5C79.0	fR/S	 46.3
U+5C79.0	fTotalStrokes	 6
U+5C79.0	fCangjie	UON
U+5C79.0	fFourCorner	28717
U+5C79.0	fHakka	 ngiet7; nget7; jat7
U+5C79.0	fCantonese	ngat6; at6 (also); gat1
U+5C79.0	fMandarin	yi4; ge1
U+5C79.0	fEnglish	 rise high; stand erect or majestical (said of a mountain, building, etc.)
U+5C7A.0	fUTF8	屺
U+5C7A.0	fCNS11643	2-2237
U+5C7A.0	fBig5	C9D7
U+5C7A.0	fGB	0-6128
U+5C7A.0	fR/S	 46.3
U+5C7A.0	fTotalStrokes	 6
U+5C7A.0	fCangjie	USU
U+5C7A.0	fFourCorner	27717
U+5C7A.0	fHakka	  ki3;  gi5; hi3
U+5C7A.0	fCantonese	hei2; gei2
U+5C7A.0	fMandarin	qi3
U+5C7A.0	fEnglish	 bare mountain or hill
U+5C7B.0	fUTF8	屻
U+5C7B.0	fCNS11643	2-2238
U+5C7B.0	fBig5	C9D8
U+5C7B.0	fGB	5-3A5C
U+5C7B.0	fR/S	 46.3
U+5C7B.0	fTotalStrokes	 6
U+5C7B.0	fCangjie	USHI
U+5C7B.0	fFourCorner	27720
U+5C7B.0	fHakka	 ngiun3
U+5C7B.0	fCantonese	jan6
U+5C7B.0	fMandarin	ren4
U+5C7B.0	fEnglish	sharp
U+5C7C.0	fUTF8	屼
U+5C7C.0	fCNS11643	2-2236
U+5C7C.0	fBig5	C9D6
U+5C7C.0	fGB	3-3841
U+5C7C.0	fR/S	 46.3
U+5C7C.0	fTotalStrokes	 6
U+5C7C.0	fCangjie	UMU
U+5C7C.0	fFourCorner	21710
U+5C7C.0	fHakka	 ngut7; ngiet7
U+5C7C.0	fCantonese	ngat6; at6 (also)
U+5C7C.0	fMandarin	wu4
U+5C7C.0	fEnglish	 mountain bare hill
U+5C7D.0	fUTF8	屽
U+5C7D.0	fCNS11643	4-2260
U+5C7D.0	fGB	5-3A58
U+5C7D.0	fR/S	  46.3
U+5C7D.0	fTotalStrokes	 6
U+5C7D.0	fCangjie	UMJ
U+5C7D.0	fFourCorner	21740
U+5C7D.0	fHakka	 ngan3
U+5C7D.0	fCantonese	hon6
U+5C7D.0	fMandarin	han4
U+5C7E.0	fUTF8	屾
U+5C7E.0	fCNS11643	2-2239
U+5C7E.0	fBig5	C9D9
U+5C7E.0	fGB	5-3A59
U+5C7E.0	fR/S	 46.3
U+5C7E.0	fTotalStrokes	 6
U+5C7E.0	fCangjie	UU
U+5C7E.0	fFourCorner	22770
U+5C7E.0	fCantonese	san1
U+5C7E.0	fMandarin	shen1
U+5C7F.0	fUTF8	屿
U+5C7F.0	fGB	0-536C
U+5C7F.0	fR/S	 46.3
U+5C7F.0	fTotalStrokes	 6
U+5C7F.0	fCangjie	UYSM
U+5C7F.0	fHakka	 ji3; sii5
U+5C7F.0	fCantonese	zeoi6; jyu4; jyu5 (also)
U+5C7F.0	fMandarin	yu3; xu4
U+5C7F.0	fEnglish	simplified form of &#x5DBC;
U+5C80.0	fUTF8	岀
U+5C80.0	fCNS11643	3-235A
U+5C80.0	fGB	E-282D
U+5C80.0	fR/S	 46.3
U+5C80.0	fTotalStrokes	 6
U+5C80.0	fCangjie	UU
U+5C80.0	fFourCorner	22770
U+5C80.0	fHakka	 ch'ut7
U+5C80.0	fCantonese	ceot1
U+5C80.0	fMandarin	chu1; chui4
U+5C80.0	fEnglish	[1] go out [2] non-classical form of &#x51FA;
U+5C81.0	fUTF8	岁
U+5C81.0	fCNS11643	F-2247
U+5C81.0	fGB	0-4B6A
U+5C81.0	fR/S	 46.3
U+5C81.0	fTotalStrokes	 6
U+5C81.0	fCangjie	UNI
U+5C81.0	fFourCorner	2220
U+5C81.0	fHakka	 soi5; sui5; se5
U+5C81.0	fCantonese	seoi3
U+5C81.0	fMandarin	sui4; xue3; suo4
U+5C81.0	fEnglish	simplified form of &#x6B72;
U+5C82.0	fUTF8	岂
U+5C82.0	fGB	0-4671
U+5C82.0	fR/S	 46.3
U+5C82.0	fTotalStrokes	 6
U+5C82.0	fCangjie	USU
U+5C82.0	fHakka	 ki3
U+5C82.0	fCantonese	hei2; hoi2
U+5C82.0	fMandarin	qi3; kai3
U+5C82.0	fEnglish	simplified form of &#x8C48;
U+5C83.0	fUTF8	岃
U+5C83.0	fGB	1-7C59
U+5C83.0	fR/S	 46.3
U+5C83.0	fTotalStrokes	 6
U+5C83.0	fCangjie	USHI
U+5C83.0	fHakka	 jin5; rin5; rin1; jin1
U+5C83.0	fCantonese	jan6
U+5C83.0	fEnglish	[1] hills [2] the contour of hills (goematic)
U+5C84.0	fUTF8	岄
U+5C84.0	fCNS11643	3-2561
U+5C84.0	fGB	E-282E
U+5C84.0	fR/S	  46.4
U+5C84.0	fTotalStrokes	 7
U+5C84.0	fCangjie	UB
U+5C84.0	fFourCorner	27720
U+5C84.0	fCantonese	jyut6
U+5C84.0	fMandarin	yue4
U+5C85.0	fUTF8	岅
U+5C85.0	fCNS11643	3-2566
U+5C85.0	fGB	E-282F
U+5C85.0	fR/S	 46.4
U+5C85.0	fTotalStrokes	 7
U+5C85.0	fCangjie	UHE
U+5C85.0	fFourCorner	21740
U+5C85.0	fCantonese	baan2; faan2
U+5C85.0	fMandarin	ban3
U+5C86.0	fUTF8	岆
U+5C86.0	fCNS11643	2-2367
U+5C86.0	fBig5	CAC7
U+5C86.0	fGB	E-2830
U+5C86.0	fR/S	 46.4
U+5C86.0	fTotalStrokes	 7
U+5C86.0	fCangjie	UHK
U+5C86.0	fFourCorner	22734
U+5C86.0	fCantonese	jiu2
U+5C86.0	fMandarin	yao3
U+5C87.0	fUTF8	岇
U+5C87.0	fCNS11643	4-237A
U+5C87.0	fGB	E-2831
U+5C87.0	fR/S	  46.4
U+5C87.0	fTotalStrokes	 7
U+5C87.0	fCangjie	UHVL
U+5C87.0	fFourCorner	22720
U+5C87.0	fCantonese	ngong4; ong4 (also)
U+5C87.0	fMandarin	ang2
U+5C88.0	fUTF8	岈
U+5C88.0	fCNS11643	2-2362
U+5C88.0	fBig5	CAC2
U+5C88.0	fGB	0-612C
U+5C88.0	fR/S	 46.4
U+5C88.0	fTotalStrokes	 7
U+5C88.0	fCangjie	UMVH
U+5C88.0	fFourCorner	21740
U+5C88.0	fCantonese	ngaa4; haa1; aa4 (also)
U+5C88.0	fMandarin	ya2; xia1
U+5C89.0	fUTF8	岉
U+5C89.0	fCNS11643	2-2364
U+5C89.0	fBig5	CAC4
U+5C89.0	fGB	E-2832
U+5C89.0	fR/S	 46.4
U+5C89.0	fTotalStrokes	 7
U+5C89.0	fCangjie	UPHH
U+5C89.0	fFourCorner	27720
U+5C89.0	fHakka	 wut7; vut7
U+5C89.0	fCantonese	mat6; wat1
U+5C89.0	fMandarin	wu4
U+5C89.0	fEnglish	high; lofty
U+5C8A.0	fUTF8	岊
U+5C8A.0	fCNS11643	2-2366
U+5C8A.0	fBig5	CAC6
U+5C8A.0	fGB	3-384C
U+5C8A.0	fR/S	 46.4
U+5C8A.0	fTotalStrokes	 7
U+5C8A.0	fCangjie	AUU
U+5C8A.0	fFourCorner	77771
U+5C8A.0	fHakka	  ziet7
U+5C8A.0	fCantonese	zit3; git3
U+5C8A.0	fMandarin	jie2
U+5C8A.0	fEnglish	foothill
U+5C8B.0	fUTF8	岋
U+5C8B.0	fCNS11643	2-2363
U+5C8B.0	fBig5	CAC3
U+5C8B.0	fGB	5-3A5A
U+5C8B.0	fR/S	 46.4
U+5C8B.0	fTotalStrokes	 7
U+5C8B.0	fCangjie	UNHE
U+5C8B.0	fFourCorner	27247
U+5C8B.0	fCantonese	ngap6; ap6 (also)
U+5C8B.0	fMandarin	e4
U+5C8C.0	fUTF8	岌
U+5C8C.0	fCNS11643	1-4A26
U+5C8C.0	fBig5	A7C4
U+5C8C.0	fGB	0-6127
U+5C8C.0	fR/S	 46.4
U+5C8C.0	fTotalStrokes	 7
U+5C8C.0	fCangjie	UNHE
U+5C8C.0	fFourCorner	22247
U+5C8C.0	fHakka	 kip7
U+5C8C.0	fCantonese	kap1; ngap6; gap6 (also); ap6 (also)
U+5C8C.0	fMandarin	ji2
U+5C8C.0	fEnglish	[1] perilous; hazardous [2] rising high above others (said of a peak)
U+5C8D.0	fUTF8	岍
U+5C8D.0	fCNS11643	2-2360
U+5C8D.0	fBig5	CAC0
U+5C8D.0	fGB	0-6129
U+5C8D.0	fR/S	 46.6
U+5C8D.0	fTotalStrokes	 9
U+5C8D.0	fCangjie	UMT
U+5C8D.0	fFourCorner	21740
U+5C8D.0	fCantonese	hin1
U+5C8D.0	fMandarin	qian1
U+5C8E.0	fUTF8	岎
U+5C8E.0	fCNS11643	3-2562
U+5C8E.0	fGB	3-384A
U+5C8E.0	fR/S	  46.4
U+5C8E.0	fTotalStrokes	 7
U+5C8E.0	fCangjie	UCSH
U+5C8E.0	fFourCorner	28720
U+5C8E.0	fCantonese	fan4
U+5C8E.0	fMandarin	fen1
U+5C8F.0	fUTF8	岏
U+5C8F.0	fCNS11643	2-2361
U+5C8F.0	fBig5	CAC1
U+5C8F.0	fGB	3-3843
U+5C8F.0	fR/S	 46.4
U+5C8F.0	fTotalStrokes	 7
U+5C8F.0	fCangjie	UMMU
U+5C8F.0	fFourCorner	21711
U+5C8F.0	fHakka	 ngan2; ngien2
U+5C8F.0	fCantonese	jyun4
U+5C8F.0	fMandarin	wan2; yuan2
U+5C90.0	fUTF8	岐
U+5C90.0	fCNS11643	1-4A23
U+5C90.0	fBig5	A7C1
U+5C90.0	fGB	0-612A
U+5C90.0	fR/S	 46.4
U+5C90.0	fTotalStrokes	 7
U+5C90.0	fCangjie	UJE
U+5C90.0	fFourCorner	24747
U+5C90.0	fHakka	  ki1; ki2
U+5C90.0	fCantonese	kei4
U+5C90.0	fMandarin	qi2
U+5C90.0	fEnglish	[1] Chi, name of a mountain [2] diverge; fork in road; branch
U+5C91.0	fUTF8	岑
U+5C91.0	fCNS11643	1-4A24
U+5C91.0	fBig5	A7C2
U+5C91.0	fGB	0-612F
U+5C91.0	fR/S	 46.4
U+5C91.0	fTotalStrokes	 7
U+5C91.0	fCangjie	UOIN
U+5C91.0	fFourCorner	22207
U+5C91.0	fHakka	 cem2; sam2; kim2
U+5C91.0	fCantonese	sam4
U+5C91.0	fMandarin	cen2
U+5C91.0	fEnglish	steep; precipitous; peak
U+5C92.0	fUTF8	岒
U+5C92.0	fCNS11643	2-2365
U+5C92.0	fBig5	CAC5
U+5C92.0	fGB	5-3A5F
U+5C92.0	fR/S	 46.4
U+5C92.0	fTotalStrokes	 7
U+5C92.0	fCangjie	UOIN
U+5C92.0	fFourCorner	28727
U+5C92.0	fCantonese	kim4
U+5C92.0	fMandarin	qian2
U+5C93.0	fUTF8	岓
U+5C93.0	fCNS11643	2-2368
U+5C93.0	fBig5	CAC8
U+5C93.0	fGB	5-3A5E
U+5C93.0	fR/S	 46.4
U+5C93.0	fTotalStrokes	 7
U+5C93.0	fCangjie	UHML
U+5C93.0	fFourCorner	22721
U+5C93.0	fCantonese	kei4
U+5C93.0	fMandarin	qi2
U+5C94.0	fUTF8	岔
U+5C94.0	fCNS11643	1-4A25
U+5C94.0	fBig5	A7C3
U+5C94.0	fGB	0-326D
U+5C94.0	fR/S	 46.4
U+5C94.0	fTotalStrokes	 7
U+5C94.0	fCangjie	CSHU
U+5C94.0	fFourCorner	80772
U+5C94.0	fHakka	 ca5;  ca1
U+5C94.0	fCantonese	caa3; caa1
U+5C94.0	fMandarin	cha4; cha1; neng4
U+5C94.0	fEnglish	[1] diverge; branch (off); fork [2] fork in road; branching point; fork
U+5C95.0	fUTF8	岕
U+5C95.0	fCNS11643	2-2369
U+5C95.0	fBig5	CAC9
U+5C95.0	fGB	3-3847
U+5C95.0	fR/S	 46.4
U+5C95.0	fTotalStrokes	 7
U+5C95.0	fCangjie	UOLL
U+5C95.0	fFourCorner	22228
U+5C95.0	fCantonese	gaai3
U+5C95.0	fMandarin	jie4
U+5C96.0	fUTF8	岖
U+5C96.0	fGB	0-612B
U+5C96.0	fR/S	 46.4
U+5C96.0	fTotalStrokes	 7
U+5C96.0	fCangjie	USK
U+5C96.0	fHakka	 ki1
U+5C96.0	fCantonese	keoi1
U+5C96.0	fMandarin	qu1
U+5C96.0	fEnglish	simplified form of &#x5D87;
U+5C97.0	fUTF8	岗
U+5C97.0	fCNS11643	F-237C
U+5C97.0	fGB	0-385A
U+5C97.0	fR/S	 46.4
U+5C97.0	fTotalStrokes	 7
U+5C97.0	fCangjie	UBK
U+5C97.0	fHakka	 gong1
U+5C97.0	fCantonese	gong1
U+5C97.0	fMandarin	gang3
U+5C97.0	fEnglish	simplified form of &#x5D17;
U+5C97.1	fUTF8	岗
U+5C97.1	fCNS11643	F-237C
U+5C97.1	fGB	0-385A
U+5C97.1	fR/S	 46.4
U+5C97.1	fTotalStrokes	 7
U+5C97.1	fCangjie	UBK
U+5C97.1	fMandarin	gang1
U+5C98.0	fUTF8	岘
U+5C98.0	fGB	0-612D
U+5C98.0	fR/S	 46.4
U+5C98.0	fTotalStrokes	 7
U+5C98.0	fCangjie	UBHU
U+5C98.0	fHakka	 hien5; hien3
U+5C98.0	fCantonese	jin6
U+5C98.0	fMandarin	xian4
U+5C98.0	fEnglish	simplified form of &#x5CF4;
U+5C99.0	fUTF8	岙
U+5C99.0	fCNS11643	F-2378
U+5C99.0	fGB	0-612E
U+5C99.0	fR/S	 46.4
U+5C99.0	fTotalStrokes	 7
U+5C99.0	fCangjie	HKU
U+5C99.0	fFourCorner	2077
U+5C99.0	fCantonese	ou3; ngou3 (also); aau3 (also); ngaau3 (also)
U+5C99.0	fMandarin	ao4
U+5C99.0	fEnglish	island
U+5C9A.0	fUTF8	岚
U+5C9A.0	fGB	0-6130
U+5C9A.0	fR/S	 46.4
U+5C9A.0	fTotalStrokes	 7
U+5C9A.0	fCangjie	UHNK
U+5C9A.0	fHakka	 lam2
U+5C9A.0	fCantonese	laam4; laam2 (also)
U+5C9A.0	fMandarin	lan2
U+5C9A.0	fEnglish	simplified form of &#x5D50;
U+5C9B.0	fUTF8	岛
U+5C9B.0	fGB	0-353A
U+5C9B.0	fR/S	 46.4
U+5C9B.0	fTotalStrokes	 7
U+5C9B.0	fCangjie	PYSU
U+5C9B.0	fHakka	 dau3
U+5C9B.0	fCantonese	dou2
U+5C9B.0	fMandarin	dao3
U+5C9B.0	fEnglish	simplified form of &#x5CF6;
U+5C9C.0	fUTF8	岜
U+5C9C.0	fCNS11643	3-2563
U+5C9C.0	fGB	0-6131
U+5C9C.0	fR/S	  46.4
U+5C9C.0	fTotalStrokes	 7
U+5C9C.0	fCangjie	UAU
U+5C9C.0	fFourCorner	22710
U+5C9C.0	fCantonese	baa1
U+5C9C.0	fMandarin	ba1
U+5C9C.0	fEnglish	rock mountain
U+5C9D.0	fUTF8	岝
U+5C9D.0	fCNS11643	2-2649
U+5C9D.0	fBig5	CC68
U+5C9D.0	fGB	5-3A64
U+5C9D.0	fR/S	 46.5
U+5C9D.0	fTotalStrokes	 8
U+5C9D.0	fCangjie	UOS
U+5C9D.0	fFourCorner	22811
U+5C9D.0	fCantonese	zok3; zaak3; zaak6
U+5C9D.0	fMandarin	zuo4; zhai3; ze2
U+5C9D.0	fEnglish	a mountain in Shandong
U+5C9E.0	fUTF8	岞
U+5C9E.0	fCNS11643	3-285D
U+5C9E.0	fGB	3-3858
U+5C9E.0	fR/S	 46.5
U+5C9E.0	fTotalStrokes	 8
U+5C9E.0	fCangjie	UOS
U+5C9E.0	fFourCorner	28710
U+5C9E.0	fCantonese	zok3; zaak3; zaak6
U+5C9E.0	fMandarin	zuo4; zhai3; ze2
U+5C9E.0	fEnglish	same as &#x5C9D;
U+5C9F.0	fUTF8	岟
U+5C9F.0	fCNS11643	2-2643
U+5C9F.0	fBig5	CC62
U+5C9F.0	fGB	3-3857
U+5C9F.0	fR/S	 46.5
U+5C9F.0	fTotalStrokes	 8
U+5C9F.0	fCangjie	ULBK
U+5C9F.0	fFourCorner	25730
U+5C9F.0	fCantonese	joeng2
U+5C9F.0	fMandarin	yang3
U+5CA0.0	fUTF8	岠
U+5CA0.0	fCNS11643	2-263E
U+5CA0.0	fBig5	CC5D
U+5CA0.0	fGB	5-3A5D
U+5CA0.0	fR/S	 46.5
U+5CA0.0	fTotalStrokes	 8
U+5CA0.0	fCangjie	USS
U+5CA0.0	fFourCorner	21717
U+5CA0.0	fHakka	 ki3;  gi3
U+5CA0.0	fCantonese	geoi6; keoi5
U+5CA0.0	fMandarin	ju4
U+5CA0.0	fEnglish	a fabulous animal with a serpent's body and a deer's head
U+5CA1.0	fUTF8	岡
U+5CA1.0	fCNS11643	1-4D25
U+5CA1.0	fBig5	A9A3
U+5CA1.0	fGB	1-3854
U+5CA1.0	fR/S	 46.5
U+5CA1.0	fTotalStrokes	 8
U+5CA1.0	fCangjie	BTU
U+5CA1.0	fFourCorner	77220
U+5CA1.0	fHakka	  gong1
U+5CA1.0	fCantonese	gong1
U+5CA1.0	fMandarin	gang1; gang3
U+5CA1.0	fEnglish	 ridge or crest of a hill or mountain
U+5CA2.0	fUTF8	岢
U+5CA2.0	fCNS11643	2-2646
U+5CA2.0	fBig5	CC65
U+5CA2.0	fGB	0-6133
U+5CA2.0	fR/S	 46.5
U+5CA2.0	fTotalStrokes	 8
U+5CA2.0	fCangjie	UMNR
U+5CA2.0	fFourCorner	22621
U+5CA2.0	fHakka	 ko3
U+5CA2.0	fCantonese	ho2
U+5CA2.0	fMandarin	ke3
U+5CA3.0	fUTF8	岣
U+5CA3.0	fCNS11643	2-2644
U+5CA3.0	fBig5	CC63
U+5CA3.0	fGB	0-6138
U+5CA3.0	fR/S	 46.5
U+5CA3.0	fTotalStrokes	 8
U+5CA3.0	fCangjie	UPR
U+5CA3.0	fFourCorner	27720
U+5CA3.0	fHakka	 kieu3; gi5; giu3; geu3
U+5CA3.0	fCantonese	gau2; keoi1
U+5CA3.0	fMandarin	gou3; gou1; ju1
U+5CA3.0	fEnglish	a hill in Hunan
U+5CA4.0	fUTF8	岤
U+5CA4.0	fCNS11643	2-263D
U+5CA4.0	fBig5	CC5C
U+5CA4.0	fGB	5-3A6A
U+5CA4.0	fR/S	 46.5
U+5CA4.0	fTotalStrokes	 8
U+5CA4.0	fCangjie	UJC
U+5CA4.0	fFourCorner	23782
U+5CA4.0	fCantonese	jyut6; hyut3
U+5CA4.0	fMandarin	xue4
U+5CA5.0	fUTF8	岥
U+5CA5.0	fCNS11643	2-264A
U+5CA5.0	fBig5	CC69
U+5CA5.0	fGB	5-3A6F
U+5CA5.0	fR/S	 46.5
U+5CA5.0	fTotalStrokes	 8
U+5CA5.0	fCangjie	UDHE
U+5CA5.0	fFourCorner	24747
U+5CA5.0	fHakka	 bo1
U+5CA5.0	fCantonese	po1; bei1; pei4
U+5CA5.0	fMandarin	bei1; bo1; po1; pi2
U+5CA6.0	fUTF8	岦
U+5CA6.0	fCNS11643	2-264D
U+5CA6.0	fBig5	CC6C
U+5CA6.0	fGB	5-3A69
U+5CA6.0	fR/S	 46.5
U+5CA6.0	fTotalStrokes	 8
U+5CA6.0	fCangjie	UYT
U+5CA6.0	fFourCorner	22108
U+5CA6.0	fHakka	  lip8
U+5CA6.0	fCantonese	laap6; lap6
U+5CA6.0	fMandarin	li4
U+5CA6.0	fEnglish	hilly; mountainous
U+5CA7.0	fUTF8	岧
U+5CA7.0	fCNS11643	2-2648
U+5CA7.0	fBig5	CC67
U+5CA7.0	fGB	3-385D
U+5CA7.0	fR/S	 46.5
U+5CA7.0	fTotalStrokes	 8
U+5CA7.0	fCangjie	USHR
U+5CA7.0	fFourCorner	22602
U+5CA7.0	fHakka	  tiau2
U+5CA7.0	fCantonese	tiu4
U+5CA7.0	fMandarin	tiao2; tiao1
U+5CA7.0	fEnglish	 peak cliff precipice
U+5CA8.0	fUTF8	岨
U+5CA8.0	fCNS11643	2-2641
U+5CA8.0	fBig5	CC60
U+5CA8.0	fGB	3-3855
U+5CA8.0	fR/S	 46.5
U+5CA8.0	fTotalStrokes	 8
U+5CA8.0	fCangjie	UBM
U+5CA8.0	fFourCorner	27710
U+5CA8.0	fHakka	 zu5; zih3
U+5CA8.0	fCantonese	zeoi1; zeoi2; zo2
U+5CA8.0	fMandarin	ju3; qu1; ju1
U+5CA8.0	fEnglish	uneven
U+5CA9.0	fUTF8	岩
U+5CA9.0	fCNS11643	1-4D27
U+5CA9.0	fBig5	A9A5
U+5CA9.0	fGB	0-5152
U+5CA9.0	fR/S	 46.5
U+5CA9.0	fTotalStrokes	 8
U+5CA9.0	fCangjie	UMR
U+5CA9.0	fFourCorner	22601
U+5CA9.0	fHakka	  ngam2; ngam1
U+5CA9.0	fCantonese	ngaam4; aam4 (also); am4
U+5CA9.0	fMandarin	yan2
U+5CA9.0	fEnglish	[1] large rock [2] mountain
U+5CAA.0	fUTF8	岪
U+5CAA.0	fCNS11643	2-2647
U+5CAA.0	fBig5	CC66
U+5CAA.0	fGB	3-385C
U+5CAA.0	fR/S	 46.5
U+5CAA.0	fTotalStrokes	 8
U+5CAA.0	fCangjie	ULLN
U+5CAA.0	fFourCorner	22527
U+5CAA.0	fCantonese	fat6
U+5CAA.0	fMandarin	fu2
U+5CAB.0	fUTF8	岫
U+5CAB.0	fCNS11643	1-4D28
U+5CAB.0	fBig5	A9A6
U+5CAB.0	fGB	0-6136
U+5CAB.0	fR/S	 46.5
U+5CAB.0	fTotalStrokes	 8
U+5CAB.0	fCangjie	ULW
U+5CAB.0	fFourCorner	25760
U+5CAB.0	fHakka	 ciu5; ciu6; ciu3
U+5CAB.0	fCantonese	zau6
U+5CAB.0	fMandarin	xiu4; you2
U+5CAB.0	fEnglish	[1] mountain peak [2] cave; cavern
U+5CAC.0	fUTF8	岬
U+5CAC.0	fCNS11643	2-2642
U+5CAC.0	fBig5	CC61
U+5CAC.0	fGB	0-6135
U+5CAC.0	fR/S	 46.5
U+5CAC.0	fTotalStrokes	 8
U+5CAC.0	fCangjie	UWL
U+5CAC.0	fFourCorner	26750
U+5CAC.0	fHakka	 gap7
U+5CAC.0	fCantonese	gaap3
U+5CAC.0	fMandarin	jia3
U+5CAC.0	fEnglish	 cape; promontory; headland; point
U+5CAD.0	fUTF8	岭
U+5CAD.0	fCNS11643	2-2645
U+5CAD.0	fBig5	CC64
U+5CAD.0	fGB	0-416B
U+5CAD.0	fR/S	 46.5
U+5CAD.0	fTotalStrokes	 8
U+5CAD.0	fCangjie	UOII
U+5CAD.0	fFourCorner	28727
U+5CAD.0	fHakka	 liang1; liang3
U+5CAD.0	fCantonese	ling4; ling5; leng5 (also)
U+5CAD.0	fMandarin	ling3; ling2
U+5CAD.0	fEnglish	[1] waste and remote (mountains) [2] simplified form of &#x5DBA;
U+5CAE.0	fUTF8	岮
U+5CAE.0	fCNS11643	2-263C
U+5CAE.0	fBig5	CC5B
U+5CAE.0	fGB	5-3A6B
U+5CAE.0	fR/S	 46.5
U+5CAE.0	fTotalStrokes	 8
U+5CAE.0	fCangjie	UJP
U+5CAE.0	fFourCorner	23711
U+5CAE.0	fHakka	 to2
U+5CAE.0	fCantonese	to4
U+5CAE.0	fMandarin	tuo2
U+5CAF.0	fUTF8	岯
U+5CAF.0	fCNS11643	2-2640
U+5CAF.0	fBig5	CC5F
U+5CAF.0	fGB	3-384F
U+5CAF.0	fR/S	 46.5
U+5CAF.0	fTotalStrokes	 8
U+5CAF.0	fCangjie	UMFM
U+5CAF.0	fFourCorner	21719
U+5CAF.0	fCantonese	pei4
U+5CAF.0	fMandarin	pei1; pei2
U+5CB0.0	fUTF8	岰
U+5CB0.0	fCNS11643	2-264C
U+5CB0.0	fBig5	CC6B
U+5CB0.0	fGB	3-3860
U+5CB0.0	fR/S	 46.5
U+5CB0.0	fTotalStrokes	 8
U+5CB0.0	fCangjie	UVIS
U+5CB0.0	fFourCorner	24727
U+5CB0.0	fCantonese	au3; ngau3 (also); aau3
U+5CB0.0	fMandarin	you3; ao4
U+5CB1.0	fUTF8	岱
U+5CB1.0	fCNS11643	1-4D29
U+5CB1.0	fBig5	A9A7
U+5CB1.0	fGB	0-6137
U+5CB1.0	fR/S	 46.5
U+5CB1.0	fTotalStrokes	 8
U+5CB1.0	fCangjie	OPU
U+5CB1.0	fFourCorner	23772
U+5CB1.0	fHakka	 toi5; toi6; toi3
U+5CB1.0	fCantonese	doi6
U+5CB1.0	fMandarin	dai4
U+5CB1.0	fEnglish	Mt. Tai (in Shantung province, one of the five sacred mountains)
U+5CB2.0	fUTF8	岲
U+5CB2.0	fCNS11643	4-262F
U+5CB2.0	fGB	E-2833
U+5CB2.0	fR/S	  46.5
U+5CB2.0	fTotalStrokes	 8
U+5CB2.0	fCangjie	URHU
U+5CB2.0	fFourCorner	26710
U+5CB2.0	fCantonese	fong3
U+5CB2.0	fMandarin	kuang4
U+5CB3.0	fUTF8	岳
U+5CB3.0	fCNS11643	1-4D2A
U+5CB3.0	fBig5	A9A8
U+5CB3.0	fGB	0-5440
U+5CB3.0	fR/S	 46.5
U+5CB3.0	fTotalStrokes	 8
U+5CB3.0	fCangjie	OMU
U+5CB3.0	fFourCorner	72772
U+5CB3.0	fHakka	  ngok8
U+5CB3.0	fCantonese	ngok6; ok6 (also)
U+5CB3.0	fMandarin	yue4
U+5CB3.0	fEnglish	[1] high mountain; great mountain [2] parents of one's wife [3] a Chinese family name
U+5CB4.0	fUTF8	岴
U+5CB4.0	fCNS11643	3-285E
U+5CB4.0	fGB	5-3A65
U+5CB4.0	fR/S	 46.5
U+5CB4.0	fTotalStrokes	 8
U+5CB4.0	fCangjie	UOM
U+5CB4.0	fFourCorner	22710
U+5CB4.0	fHakka	 ki1
U+5CB4.0	fCantonese	keoi1
U+5CB4.0	fMandarin	qu1
U+5CB4.0	fEnglish	same as &#x5D87;
U+5CB5.0	fUTF8	岵
U+5CB5.0	fCNS11643	2-263F
U+5CB5.0	fBig5	CC5E
U+5CB5.0	fGB	0-6132
U+5CB5.0	fR/S	 46.5
U+5CB5.0	fTotalStrokes	 8
U+5CB5.0	fCangjie	UJR
U+5CB5.0	fFourCorner	24760
U+5CB5.0	fHakka	 gu3; fu5; fu6; fu3
U+5CB5.0	fCantonese	wu6
U+5CB5.0	fMandarin	hu4
U+5CB5.0	fEnglish	hill
U+5CB6.0	fUTF8	岶
U+5CB6.0	fCNS11643	2-264B
U+5CB6.0	fBig5	CC6A
U+5CB6.0	fGB	5-3A66
U+5CB6.0	fR/S	 46.5
U+5CB6.0	fTotalStrokes	 8
U+5CB6.0	fCangjie	UHA
U+5CB6.0	fFourCorner	26700
U+5CB6.0	fCantonese	paak3
U+5CB6.0	fMandarin	po4; pai1
U+5CB7.0	fUTF8	岷
U+5CB7.0	fCNS11643	1-4D24
U+5CB7.0	fBig5	A9A2
U+5CB7.0	fGB	0-613A
U+5CB7.0	fR/S	 46.5
U+5CB7.0	fTotalStrokes	 8
U+5CB7.0	fCangjie	URVP
U+5CB7.0	fFourCorner	27747
U+5CB7.0	fHakka	  min2; men2
U+5CB7.0	fCantonese	man4
U+5CB7.0	fMandarin	min2
U+5CB7.0	fEnglish	[1] Mt. Min (in Szechwan) [2] Min river (in Szechwan)
U+5CB8.0	fUTF8	岸
U+5CB8.0	fCNS11643	1-4D26
U+5CB8.0	fBig5	A9A4
U+5CB8.0	fGB	0-3036
U+5CB8.0	fR/S	 46.5
U+5CB8.0	fTotalStrokes	 8
U+5CB8.0	fCangjie	UMMJ
U+5CB8.0	fFourCorner	22241
U+5CB8.0	fHakka	  ngan5;  ngan3; ngan6
U+5CB8.0	fCantonese	ngon6; on6 (also)
U+5CB8.0	fMandarin	an4
U+5CB8.0	fEnglish	[1] bank; shore; beach; coast [2] majestic [3] proud [4] high cliff [5] end of a journey; goal [6] private lock-up
U+5CB9.0	fUTF8	岹
U+5CB9.0	fCNS11643	3-285C
U+5CB9.0	fGB	5-3A6E
U+5CB9.0	fR/S	  46.5
U+5CB9.0	fTotalStrokes	 8
U+5CB9.0	fCangjie	USHR
U+5CB9.0	fFourCorner	27760
U+5CB9.0	fHakka	  tiau2
U+5CB9.0	fCantonese	tiu4
U+5CB9.0	fMandarin	tiao2; tiao1
U+5CB9.0	fEnglish	same as &#x5CA7;
U+5CBA.0	fUTF8	岺
U+5CBA.0	fCNS11643	3-285F
U+5CBA.0	fGB	5-3A67
U+5CBA.0	fR/S	  46.5
U+5CBA.0	fTotalStrokes	 8
U+5CBA.0	fCangjie	UOII
U+5CBA.0	fFourCorner	22300
U+5CBA.0	fCantonese	ling4; ling5
U+5CBA.0	fMandarin	ling2; ling3
U+5CBA.0	fEnglish	same as &#x5CAD;
U+5CBB.0	fUTF8	岻
U+5CBB.0	fCNS11643	3-2857
U+5CBB.0	fGB	3-3859
U+5CBB.0	fR/S	  46.5
U+5CBB.0	fTotalStrokes	 8
U+5CBB.0	fCangjie	UHPM
U+5CBB.0	fFourCorner	22740
U+5CBB.0	fCantonese	ci4
U+5CBB.0	fMandarin	chi2
U+5CBC.0	fUTF8	岼
U+5CBC.0	fGB	E-2834
U+5CBC.0	fR/S	 46.5
U+5CBC.0	fTotalStrokes	 8
U+5CBC.0	fCangjie	UMFJ
U+5CBC.0	fCantonese	ping4
U+5CBD.0	fUTF8	岽
U+5CBD.0	fGB	0-6134
U+5CBD.0	fR/S	 46.5
U+5CBD.0	fTotalStrokes	 8
U+5CBD.0	fCangjie	UKD
U+5CBD.0	fHakka	 dung3
U+5CBD.0	fCantonese	dung1
U+5CBD.0	fMandarin	dong1
U+5CBD.0	fEnglish	simplified form of &#x5D2C;
U+5CBE.0	fUTF8	岾
U+5CBE.0	fGB	1-7D5A
U+5CBE.0	fR/S	 46.5
U+5CBE.0	fTotalStrokes	 8
U+5CBE.0	fCangjie	UYR
U+5CBE.0	fCantonese	zim1
U+5CBE.0	fEnglish	 mountain pass
U+5CBF.0	fUTF8	岿
U+5CBF.0	fGB	0-3F79
U+5CBF.0	fR/S	 46.5
U+5CBF.0	fTotalStrokes	 8
U+5CBF.0	fCangjie	ULLS
U+5CBF.0	fHakka	 kwui1
U+5CBF.0	fCantonese	kwai1
U+5CBF.0	fMandarin	kui1; wei1
U+5CBF.0	fEnglish	simplified form of &#x5DCB;
U+5CC0.0	fUTF8	峀
U+5CC0.0	fCNS11643	4-2634
U+5CC0.0	fGB	E-2835
U+5CC0.0	fR/S	 46.5
U+5CC0.0	fTotalStrokes	 8
U+5CC0.0	fCangjie	XULW
U+5CC0.0	fFourCorner	22600
U+5CC0.0	fCantonese	bong1
U+5CC0.0	fMandarin	xiu4; bang1
U+5CC0.0	fEnglish	 mountain peak cave; cavern
U+5CC1.0	fUTF8	峁
U+5CC1.0	fCNS11643	3-2858
U+5CC1.0	fGB	0-6139
U+5CC1.0	fR/S	 46.5
U+5CC1.0	fTotalStrokes	 8
U+5CC1.0	fCangjie	UHHL
U+5CC1.0	fFourCorner	22220
U+5CC1.0	fCantonese	maau5
U+5CC1.0	fMandarin	mao3
U+5CC1.0	fEnglish	yellow dirt mount
U+5CC2.0	fUTF8	峂
U+5CC2.0	fGB	3-385A
U+5CC2.0	fR/S	 46.5
U+5CC2.0	fTotalStrokes	 8
U+5CC2.0	fCangjie	UHEY
U+5CC2.0	fCantonese	tung4
U+5CC2.0	fMandarin	tong2
U+5CC2.0	fEnglish	name of place
U+5CC3.0	fUTF8	峃
U+5CC3.0	fGB	8-7A72
U+5CC3.0	fR/S	 46.13; 46.5
U+5CC3.0	fTotalStrokes	 8
U+5CC3.0	fCangjie	FBU
U+5CC3.0	fCantonese	hok6
U+5CC3.0	fMandarin	xue2
U+5CC4.0	fUTF8	峄
U+5CC4.0	fGB	0-613B
U+5CC4.0	fR/S	 46.5
U+5CC4.0	fTotalStrokes	 8
U+5CC4.0	fCangjie	UEQ
U+5CC4.0	fHakka	 jit8
U+5CC4.0	fCantonese	jik6
U+5CC4.0	fMandarin	yi4
U+5CC4.0	fEnglish	simplified form of &#x5DA7;
U+5CC5.0	fUTF8	峅
U+5CC5.0	fGB	E-2836
U+5CC5.0	fR/S	 46.5
U+5CC5.0	fTotalStrokes	 8
U+5CC5.0	fCangjie	UIT
U+5CC5.0	fCantonese	bin6
U+5CC6.0	fUTF8	峆
U+5CC6.0	fCNS11643	2-2A2C
U+5CC6.0	fBig5	CEAB
U+5CC6.0	fGB	5-3A7D
U+5CC6.0	fR/S	 46.6
U+5CC6.0	fTotalStrokes	 9
U+5CC6.0	fCangjie	UOMR
U+5CC6.0	fFourCorner	28761
U+5CC6.0	fCantonese	hap6; gap3
U+5CC6.0	fMandarin	he1; he2
U+5CC7.0	fUTF8	峇
U+5CC7.0	fCNS11643	2-2A25
U+5CC7.0	fBig5	CEA4
U+5CC7.0	fGB	5-3A7E
U+5CC7.0	fR/S	 46.6
U+5CC7.0	fTotalStrokes	 9
U+5CC7.0	fCangjie	UOMR
U+5CC7.0	fFourCorner	22601
U+5CC7.0	fHakka	  hap8; ba1
U+5CC7.0	fCantonese	baa1; hap1; hap6
U+5CC7.0	fMandarin	ke1; ba1
U+5CC7.0	fEnglish	cave; cavern
U+5CC8.0	fUTF8	峈
U+5CC8.0	fCNS11643	2-2A2B
U+5CC8.0	fBig5	CEAA
U+5CC8.0	fGB	E-2837
U+5CC8.0	fR/S	 46.6
U+5CC8.0	fTotalStrokes	 9
U+5CC8.0	fCangjie	UHER
U+5CC8.0	fFourCorner	27764
U+5CC8.0	fCantonese	lok6
U+5CC8.0	fMandarin	luo4
U+5CC9.0	fUTF8	峉
U+5CC9.0	fCNS11643	2-2A24
U+5CC9.0	fBig5	CEA3
U+5CC9.0	fGB	3-386B
U+5CC9.0	fR/S	 46.6
U+5CC9.0	fTotalStrokes	 9
U+5CC9.0	fCangjie	UHER
U+5CC9.0	fFourCorner	22604
U+5CC9.0	fCantonese	ngaak6; aak6 (also)
U+5CC9.0	fMandarin	e1; e4
U+5CCA.0	fUTF8	峊
U+5CCA.0	fCNS11643	2-2A26
U+5CCA.0	fBig5	CEA5
U+5CCA.0	fGB	5-3A7C
U+5CCA.0	fR/S	 46.6
U+5CCA.0	fTotalStrokes	 9
U+5CCA.0	fCangjie	HRU
U+5CCA.0	fFourCorner	27770
U+5CCA.0	fCantonese	fau6
U+5CCA.0	fMandarin	fu4; nie4
U+5CCB.0	fUTF8	峋
U+5CCB.0	fCNS11643	2-297E
U+5CCB.0	fBig5	CE7D
U+5CCB.0	fGB	0-613E
U+5CCB.0	fR/S	 46.6
U+5CCB.0	fTotalStrokes	 9
U+5CCB.0	fCangjie	UPA
U+5CCB.0	fFourCorner	27720
U+5CCB.0	fHakka	 sun2
U+5CCB.0	fCantonese	seon1
U+5CCB.0	fMandarin	xun2; xun1
U+5CCB.0	fEnglish	ranges of hills stretching on beyond another irregular peaks; irregular stretches of mountains
U+5CCC.0	fUTF8	峌
U+5CCC.0	fCNS11643	2-297C
U+5CCC.0	fBig5	CE7B
U+5CCC.0	fGB	3-3866
U+5CCC.0	fR/S	 46.6
U+5CCC.0	fTotalStrokes	 9
U+5CCC.0	fCangjie	UMIG
U+5CCC.0	fFourCorner	21714
U+5CCC.0	fCantonese	dit6
U+5CCC.0	fMandarin	die2
U+5CCD.0	fUTF8	峍
U+5CCD.0	fCNS11643	3-2C3F
U+5CCD.0	fGB	E-2838
U+5CCD.0	fR/S	  46.6
U+5CCD.0	fTotalStrokes	 9
U+5CCD.0	fCangjie	ULQ
U+5CCD.0	fFourCorner	25700
U+5CCD.0	fCantonese	leot6
U+5CCD.0	fMandarin	lu4
U+5CCE.0	fUTF8	峎
U+5CCE.0	fCNS11643	2-2A2D
U+5CCE.0	fBig5	CEAC
U+5CCE.0	fGB	3-386F
U+5CCE.0	fR/S	 46.6
U+5CCE.0	fTotalStrokes	 9
U+5CCE.0	fCangjie	UAV
U+5CCE.0	fFourCorner	22737
U+5CCE.0	fHakka	 gien5; gen5
U+5CCE.0	fCantonese	jan2; gan3
U+5CCE.0	fMandarin	an1; lang3
U+5CCF.0	fUTF8	峏
U+5CCF.0	fCNS11643	2-2A2A
U+5CCF.0	fBig5	CEA9
U+5CCF.0	fGB	5-3A74
U+5CCF.0	fR/S	 46.6
U+5CCF.0	fTotalStrokes	 9
U+5CCF.0	fCangjie	UMBL
U+5CCF.0	fFourCorner	21727
U+5CCF.0	fCantonese	ji4
U+5CCF.0	fMandarin	er3
U+5CD0.0	fUTF8	峐
U+5CD0.0	fCNS11643	2-297A
U+5CD0.0	fBig5	CE79
U+5CD0.0	fGB	3-386E
U+5CD0.0	fR/S	 46.6
U+5CD0.0	fTotalStrokes	 9
U+5CD0.0	fCangjie	UYVO
U+5CD0.0	fFourCorner	20782
U+5CD0.0	fHakka	  goi1; gai1
U+5CD0.0	fCantonese	goi1; gaai1
U+5CD0.0	fMandarin	gai1
U+5CD0.0	fEnglish	barren hill
U+5CD1.0	fUTF8	峑
U+5CD1.0	fCNS11643	3-2C3B
U+5CD1.0	fGB	E-2839
U+5CD1.0	fR/S	 46.6
U+5CD1.0	fTotalStrokes	 9
U+5CD1.0	fCangjie	UOMG
U+5CD1.0	fFourCorner	22100
U+5CD1.0	fCantonese	cyun1
U+5CD1.0	fMandarin	quan2
U+5CD2.0	fUTF8	峒
U+5CD2.0	fCNS11643	1-5072
U+5CD2.0	fBig5	ABD0
U+5CD2.0	fGB	0-613C
U+5CD2.0	fR/S	 46.6
U+5CD2.0	fTotalStrokes	 9
U+5CD2.0	fCangjie	UBMR
U+5CD2.0	fFourCorner	27720
U+5CD2.0	fHakka	 tung2
U+5CD2.0	fCantonese	tung4; dung6
U+5CD2.0	fMandarin	tong2; dong4; gu3
U+5CD2.0	fEnglish	Mt. Kungtung (in Kansu province)
U+5CD3.0	fUTF8	峓
U+5CD3.0	fCNS11643	2-2A28
U+5CD3.0	fBig5	CEA7
U+5CD3.0	fGB	3-3864
U+5CD3.0	fR/S	 46.6
U+5CD3.0	fTotalStrokes	 9
U+5CD3.0	fCangjie	UKN
U+5CD3.0	fFourCorner	25732
U+5CD3.0	fHakka	 ji2; ri2
U+5CD3.0	fCantonese	ji4
U+5CD3.0	fMandarin	yi2
U+5CD3.0	fEnglish	a place in Corea, selected by Yao as an astronomical station
U+5CD4.0	fUTF8	峔
U+5CD4.0	fCNS11643	2-2A29
U+5CD4.0	fBig5	CEA8
U+5CD4.0	fGB	5-3A72
U+5CD4.0	fR/S	 46.6
U+5CD4.0	fTotalStrokes	 9
U+5CD4.0	fCangjie	UJKP
U+5CD4.0	fFourCorner	24711
U+5CD4.0	fCantonese	mou5
U+5CD4.0	fMandarin	mu3
U+5CD5.0	fUTF8	峕
U+5CD5.0	fCNS11643	3-646D
U+5CD5.0	fGB	E-283A
U+5CD5.0	fR/S	 46.6
U+5CD5.0	fTotalStrokes	 9
U+5CD5.0	fCangjie	UYA
U+5CD5.0	fFourCorner	2260
U+5CD5.0	fHakka	 shi2; sii2
U+5CD5.0	fCantonese	si4
U+5CD5.0	fMandarin	shi2
U+5CD5.0	fEnglish	[1] time; season; opportunity [2] an hour
U+5CD6.0	fUTF8	峖
U+5CD6.0	fCNS11643	2-2A27
U+5CD6.0	fBig5	CEA6
U+5CD6.0	fGB	5-3B24
U+5CD6.0	fR/S	 46.6
U+5CD6.0	fTotalStrokes	 9
U+5CD6.0	fCangjie	UJV
U+5CD6.0	fFourCorner	23744
U+5CD6.0	fCantonese	on1; ngon1 (also)
U+5CD6.0	fMandarin	an1; an4; e4
U+5CD7.0	fUTF8	峗
U+5CD7.0	fCNS11643	2-297D
U+5CD7.0	fBig5	CE7C
U+5CD7.0	fGB	3-386A
U+5CD7.0	fR/S	 46.6
U+5CD7.0	fTotalStrokes	 9
U+5CD7.0	fCangjie	UNMU
U+5CD7.0	fFourCorner	27712
U+5CD7.0	fHakka	 ngui2
U+5CD7.0	fCantonese	ngai4; ai4 (also); ngai5
U+5CD7.0	fMandarin	wei2; wei3; yu4
U+5CD7.0	fEnglish	[1] a mountain in Kansu [2] a mountain in Szechuan
U+5CD8.0	fUTF8	峘
U+5CD8.0	fCNS11643	2-297B
U+5CD8.0	fBig5	CE7A
U+5CD8.0	fGB	3-3862
U+5CD8.0	fR/S	 46.6
U+5CD8.0	fTotalStrokes	 9
U+5CD8.0	fCangjie	UMAM
U+5CD8.0	fFourCorner	21716
U+5CD8.0	fHakka	 fan2
U+5CD8.0	fCantonese	wun4
U+5CD8.0	fMandarin	hu1; huan2
U+5CD8.0	fEnglish	 high hill
U+5CD9.0	fUTF8	峙
U+5CD9.0	fCNS11643	1-5071
U+5CD9.0	fBig5	ABCF
U+5CD9.0	fGB	0-5645
U+5CD9.0	fR/S	 46.6
U+5CD9.0	fTotalStrokes	 9
U+5CD9.0	fCangjie	UGDI
U+5CD9.0	fFourCorner	24741
U+5CD9.0	fHakka	 sii5; su3; sii6; chi6; cih5; ci3; si5; tje3; chi5; she3
U+5CD9.0	fCantonese	ci5 (also); si6; zi6
U+5CD9.0	fMandarin	zhi4; shi4
U+5CD9.0	fEnglish	 stand erect (like a mountain); stand up
U+5CDA.0	fUTF8	峚
U+5CDA.0	fCNS11643	2-2A23
U+5CDA.0	fBig5	CEA2
U+5CDA.0	fGB	5-3A75
U+5CDA.0	fR/S	 46.6
U+5CDA.0	fTotalStrokes	 9
U+5CDA.0	fCangjie	UKG
U+5CDA.0	fFourCorner	22104
U+5CDA.0	fCantonese	mat6
U+5CDA.0	fMandarin	mi4
U+5CDB.0	fUTF8	峛
U+5CDB.0	fCNS11643	2-2A21
U+5CDB.0	fBig5	CE7E
U+5CDB.0	fGB	3-3863
U+5CDB.0	fR/S	 46.6
U+5CDB.0	fTotalStrokes	 9
U+5CDB.0	fCangjie	UMNN
U+5CDB.0	fFourCorner	22227
U+5CDB.0	fCantonese	lei5
U+5CDB.0	fMandarin	li3
U+5CDC.0	fUTF8	峜
U+5CDC.0	fCNS11643	3-2C3C
U+5CDC.0	fGB	5-3B21
U+5CDC.0	fR/S	  46.6
U+5CDC.0	fTotalStrokes	 9
U+5CDC.0	fCangjie	UOYM
U+5CDC.0	fFourCorner	22100
U+5CDC.0	fCantonese	gai3
U+5CDC.0	fMandarin	ji1
U+5CDD.0	fUTF8	峝
U+5CDD.0	fCNS11643	3-2C42
U+5CDD.0	fGB	E-283B
U+5CDD.0	fR/S	  46.6
U+5CDD.0	fTotalStrokes	 9
U+5CDD.0	fCangjie	UBMR
U+5CDD.0	fFourCorner	22220
U+5CDD.0	fCantonese	tung4; dung6
U+5CDD.0	fMandarin	dong4; tong2
U+5CDD.0	fEnglish	a mountain in Gansu province
U+5CDE.0	fUTF8	峞
U+5CDE.0	fCNS11643	2-2A22
U+5CDE.0	fBig5	CEA1
U+5CDE.0	fGB	5-3B23
U+5CDE.0	fR/S	 46.6
U+5CDE.0	fTotalStrokes	 9
U+5CDE.0	fCangjie	UNMU
U+5CDE.0	fFourCorner	22712
U+5CDE.0	fCantonese	ngai4; ai4 (also); ngai5
U+5CDE.0	fMandarin	wei2; kui3; kui1; wei1
U+5CDE.0	fEnglish	high and uneven
U+5CDF.0	fUTF8	峟
U+5CDF.0	fCNS11643	2-2A2E
U+5CDF.0	fBig5	CEAD
U+5CDF.0	fGB	5-3A73
U+5CDF.0	fR/S	 46.6
U+5CDF.0	fTotalStrokes	 9
U+5CDF.0	fCangjie	UKB
U+5CDF.0	fFourCorner	22427
U+5CDF.0	fCantonese	jau6
U+5CDF.0	fMandarin	you4
U+5CE0.0	fUTF8	峠
U+5CE0.0	fGB	E-283C
U+5CE0.0	fR/S	 46.6
U+5CE0.0	fTotalStrokes	 9
U+5CE0.0	fCangjie	UYMY
U+5CE0.0	fFourCorner	2172
U+5CE0.0	fCantonese	kaa1
U+5CE0.0	fEnglish	 mountain pass crisis; climax
U+5CE1.0	fUTF8	峡
U+5CE1.0	fCNS11643	3-2C3E
U+5CE1.0	fGB	0-4F3F
U+5CE1.0	fR/S	 46.6
U+5CE1.0	fTotalStrokes	 9
U+5CE1.0	fCangjie	UKT
U+5CE1.0	fFourCorner	25730
U+5CE1.0	fHakka	 hiap8
U+5CE1.0	fCantonese	haap6; haap3 (also); hap6 (also)
U+5CE1.0	fMandarin	xia2
U+5CE1.0	fEnglish	simplified form of &#x5CFD;
U+5CE2.0	fUTF8	峢
U+5CE2.0	fCNS11643	3-2C41
U+5CE2.0	fGB	E-283D
U+5CE2.0	fR/S	 46.6
U+5CE2.0	fTotalStrokes	 9
U+5CE2.0	fCangjie	UMNN
U+5CE2.0	fFourCorner	22700
U+5CE2.0	fCantonese	lit6
U+5CE2.0	fMandarin	li3
U+5CE3.0	fUTF8	峣
U+5CE3.0	fGB	8-7A74
U+5CE3.0	fR/S	 46.6; 46.12
U+5CE3.0	fTotalStrokes	 9
U+5CE3.0	fCangjie	UJPU
U+5CE3.0	fCantonese	jiu4
U+5CE3.0	fMandarin	yao2
U+5CE3.0	fEnglish	high or tall
U+5CE4.0	fUTF8	峤
U+5CE4.0	fGB	0-613D
U+5CE4.0	fR/S	 46.6
U+5CE4.0	fTotalStrokes	 9
U+5CE4.0	fCangjie	UHKL
U+5CE4.0	fHakka	 kiau2; kiau5
U+5CE4.0	fCantonese	giu6; kiu4
U+5CE4.0	fMandarin	jiao4; qiao2
U+5CE4.0	fEnglish	simplified form of &#x5DA0;
U+5CE5.0	fUTF8	峥
U+5CE5.0	fCNS11643	3-2C43
U+5CE5.0	fGB	0-613F
U+5CE5.0	fR/S	 46.6
U+5CE5.0	fTotalStrokes	 9
U+5CE5.0	fCangjie	UNSD
U+5CE5.0	fFourCorner	27750
U+5CE5.0	fHakka	 zen1
U+5CE5.0	fCantonese	zang1; zaang1 (also)
U+5CE5.0	fMandarin	zheng1; cheng2
U+5CE5.0	fEnglish	simplified form of &#x5D22;
U+5CE6.0	fUTF8	峦
U+5CE6.0	fCNS11643	3-2C3D
U+5CE6.0	fGB	0-424D
U+5CE6.0	fR/S	 46.6
U+5CE6.0	fTotalStrokes	 9
U+5CE6.0	fCangjie	YCU
U+5CE6.0	fFourCorner	00770
U+5CE6.0	fHakka	 lan2
U+5CE6.0	fCantonese	lyun4
U+5CE6.0	fMandarin	luan2
U+5CE6.0	fEnglish	simplified form of &#x5DD2;
U+5CE7.0	fUTF8	峧
U+5CE7.0	fGB	3-386C
U+5CE7.0	fR/S	 46.6
U+5CE7.0	fTotalStrokes	 9
U+5CE7.0	fCangjie	UYCK
U+5CE7.0	fCantonese	gaau1
U+5CE7.0	fMandarin	jiao1
U+5CE8.0	fUTF8	峨
U+5CE8.0	fCNS11643	1-5533
U+5CE8.0	fBig5	AE6F
U+5CE8.0	fGB	0-366B
U+5CE8.0	fR/S	 46.7
U+5CE8.0	fTotalStrokes	 10
U+5CE8.0	fCangjie	UHQI
U+5CE8.0	fFourCorner	23750
U+5CE8.0	fHakka	  ngo2
U+5CE8.0	fCantonese	ngo4; o4
U+5CE8.0	fMandarin	e2
U+5CE8.0	fEnglish	lofty
U+5CE9.0	fUTF8	峩
U+5CE9.0	fCNS11643	3-3063
U+5CE9.0	fGB	E-283E
U+5CE9.0	fR/S	  46.7
U+5CE9.0	fTotalStrokes	 10
U+5CE9.0	fCangjie	UHQI
U+5CE9.0	fFourCorner	22550
U+5CE9.0	fHakka	 ngo2
U+5CE9.0	fCantonese	ngo4; o4
U+5CE9.0	fMandarin	e2
U+5CE9.0	fEnglish	[1] lofty [2] same as &#x5CE8;
U+5CEA.0	fUTF8	峪
U+5CEA.0	fCNS11643	1-5532
U+5CEA.0	fBig5	AE6E
U+5CEA.0	fGB	0-5378
U+5CEA.0	fR/S	 46.7
U+5CEA.0	fTotalStrokes	 10
U+5CEA.0	fCangjie	UCOR
U+5CEA.0	fFourCorner	28768
U+5CEA.0	fHakka	 juk8; ruk8
U+5CEA.0	fCantonese	juk6; jyu6 (also)
U+5CEA.0	fMandarin	yu4
U+5CEA.0	fEnglish	[1] valley; ravine [2] Chiayuu in Kansu province, where the Great Wall terminates
U+5CEB.0	fUTF8	峫
U+5CEB.0	fCNS11643	3-305F
U+5CEB.0	fGB	3-3867
U+5CEB.0	fR/S	  46.7
U+5CEB.0	fTotalStrokes	 10
U+5CEB.0	fCangjie	UMHL
U+5CEB.0	fFourCorner	27720
U+5CEB.0	fCantonese	je4
U+5CEB.0	fMandarin	ye2
U+5CEB.0	fEnglish	mountain in Shantung province
U+5CEC.0	fUTF8	峬
U+5CEC.0	fCNS11643	2-2E6E
U+5CEC.0	fBig5	D16C
U+5CEC.0	fGB	5-3B28
U+5CEC.0	fR/S	 46.7
U+5CEC.0	fTotalStrokes	 10
U+5CEC.0	fCangjie	UIJB
U+5CEC.0	fFourCorner	23727
U+5CEC.0	fCantonese	bou1
U+5CEC.0	fMandarin	bu1
U+5CED.0	fUTF8	峭
U+5CED.0	fCNS11643	1-552F
U+5CED.0	fBig5	AE6B
U+5CED.0	fGB	0-474D
U+5CED.0	fR/S	 46.7
U+5CED.0	fTotalStrokes	 10
U+5CED.0	fCangjie	UFB
U+5CED.0	fFourCorner	29727
U+5CED.0	fHakka	 ciau5
U+5CED.0	fCantonese	ciu3
U+5CED.0	fMandarin	qiao4
U+5CED.0	fEnglish	steep; precipitous; rugged
U+5CEE.0	fUTF8	峮
U+5CEE.0	fCNS11643	2-2E70
U+5CEE.0	fBig5	D16E
U+5CEE.0	fGB	3-3922
U+5CEE.0	fR/S	 46.7
U+5CEE.0	fTotalStrokes	 10
U+5CEE.0	fCangjie	USKR
U+5CEE.0	fFourCorner	27767
U+5CEE.0	fCantonese	kwan1
U+5CEE.0	fMandarin	qun1; jun1
U+5CEF.0	fUTF8	峯
U+5CEF.0	fCNS11643	3-3064
U+5CEF.0	fGB	E-283F
U+5CEF.0	fR/S	  46.7
U+5CEF.0	fTotalStrokes	 10
U+5CEF.0	fCangjie	UHEJ
U+5CEF.0	fFourCorner	22500
U+5CEF.0	fHakka	 fung1
U+5CEF.0	fCantonese	fung1
U+5CEF.0	fMandarin	feng1
U+5CEF.0	fEnglish	[1] peak; summit [2] hump (e.g. of a camel)
U+5CF0.0	fUTF8	峰
U+5CF0.0	fCNS11643	1-5534
U+5CF0.0	fBig5	AE70
U+5CF0.0	fGB	0-3765
U+5CF0.0	fR/S	 46.7
U+5CF0.0	fTotalStrokes	 10
U+5CF0.0	fCangjie	UHEJ
U+5CF0.0	fFourCorner	27754
U+5CF0.0	fHakka	  fung1
U+5CF0.0	fCantonese	fung1
U+5CF0.0	fMandarin	feng1
U+5CF0.0	fEnglish	peak, summit; hump of camel
U+5CF1.0	fUTF8	峱
U+5CF1.0	fCNS11643	2-2E71
U+5CF1.0	fBig5	D16F
U+5CF1.0	fGB	3-4436
U+5CF1.0	fR/S	 46.7
U+5CF1.0	fTotalStrokes	 10
U+5CF1.0	fCangjie	KHNGU
U+5CF1.0	fFourCorner	47271
U+5CF1.0	fHakka	 nau2; no2
U+5CF1.0	fCantonese	naau4; laau4 (also); nau4; nou4; lau4
U+5CF1.0	fMandarin	nao2; nao1
U+5CF1.0	fEnglish	name of a mountain
U+5CF2.0	fUTF8	峲
U+5CF2.0	fCNS11643	4-2C60
U+5CF2.0	fGB	5-3B2F
U+5CF2.0	fR/S	  46.7
U+5CF2.0	fTotalStrokes	 10
U+5CF2.0	fCangjie	UHDN
U+5CF2.0	fFourCorner	22920
U+5CF2.0	fCantonese	lei5
U+5CF2.0	fMandarin	li3
U+5CF3.0	fUTF8	峳
U+5CF3.0	fCNS11643	3-3061
U+5CF3.0	fGB	3-3876
U+5CF3.0	fR/S	  46.7
U+5CF3.0	fTotalStrokes	 10
U+5CF3.0	fCangjie	UOLK
U+5CF3.0	fFourCorner	22240
U+5CF3.0	fCantonese	jau4
U+5CF3.0	fMandarin	you2
U+5CF4.0	fUTF8	峴
U+5CF4.0	fCNS11643	1-5537
U+5CF4.0	fBig5	AE73
U+5CF4.0	fGB	1-612D
U+5CF4.0	fR/S	 46.7
U+5CF4.0	fTotalStrokes	 10
U+5CF4.0	fCangjie	UBUU
U+5CF4.0	fFourCorner	26710
U+5CF4.0	fHakka	  hien5; hen3; hien3
U+5CF4.0	fCantonese	jin6
U+5CF4.0	fMandarin	xian4
U+5CF4.0	fEnglish	[1] steep hill [2] mountain in Hupeh province
U+5CF5.0	fUTF8	峵
U+5CF5.0	fCNS11643	3-3062
U+5CF5.0	fGB	5-3B35
U+5CF5.0	fR/S	  46.7
U+5CF5.0	fTotalStrokes	 10
U+5CF5.0	fCangjie	UJKI
U+5CF5.0	fFourCorner	23730
U+5CF5.0	fHakka	 jin2;  jung2; rung2; rin2
U+5CF5.0	fCantonese	wing4; jing4; wang4
U+5CF5.0	fMandarin	rong2; heng2
U+5CF5.0	fEnglish	same as &#x5DB8;
U+5CF6.0	fUTF8	島
U+5CF6.0	fCNS11643	1-5535
U+5CF6.0	fBig5	AE71
U+5CF6.0	fGB	1-353A
U+5CF6.0	fR/S	 46.7
U+5CF6.0	fTotalStrokes	 10
U+5CF6.0	fCangjie	HAYU
U+5CF6.0	fFourCorner	27727
U+5CF6.0	fHakka	  dau3;  do3
U+5CF6.0	fCantonese	dou2
U+5CF6.0	fMandarin	dao3
U+5CF6.0	fEnglish	 island; isle
U+5CF7.0	fUTF8	峷
U+5CF7.0	fCNS11643	2-2E72
U+5CF7.0	fBig5	D170
U+5CF7.0	fGB	5-3B32
U+5CF7.0	fR/S	 46.7
U+5CF7.0	fTotalStrokes	 10
U+5CF7.0	fCangjie	UYTJ
U+5CF7.0	fFourCorner	22504
U+5CF7.0	fCantonese	san1
U+5CF7.0	fMandarin	shen1
U+5CF8.0	fUTF8	峸
U+5CF8.0	fCNS11643	2-2A2F
U+5CF8.0	fBig5	CEAE
U+5CF8.0	fGB	E-2840
U+5CF8.0	fR/S	 46.7
U+5CF8.0	fTotalStrokes	 10
U+5CF8.0	fCangjie	UIHS
U+5CF8.0	fFourCorner	23750
U+5CF8.0	fCantonese	sing4
U+5CF8.0	fMandarin	cheng2
U+5CF9.0	fUTF8	峹
U+5CF9.0	fCNS11643	2-2E74
U+5CF9.0	fBig5	D172
U+5CF9.0	fGB	5-3B30
U+5CF9.0	fR/S	 46.7
U+5CF9.0	fTotalStrokes	 10
U+5CF9.0	fCangjie	OMDU
U+5CF9.0	fFourCorner	80779
U+5CF9.0	fHakka	 tu2
U+5CF9.0	fCantonese	tou4
U+5CF9.0	fMandarin	tu2
U+5CF9.0	fEnglish	[1] the mountain and region whence came the wife of Thai Yi [2] same as &#x5D5E;
U+5CFA.0	fUTF8	峺
U+5CFA.0	fCNS11643	3-305E
U+5CFA.0	fGB	5-3B29
U+5CFA.0	fR/S	  46.7
U+5CFA.0	fTotalStrokes	 10
U+5CFA.0	fCangjie	UMLK
U+5CFA.0	fFourCorner	21740
U+5CFA.0	fCantonese	gang2
U+5CFA.0	fMandarin	geng3
U+5CFB.0	fUTF8	峻
U+5CFB.0	fCNS11643	1-5531
U+5CFB.0	fBig5	AE6D
U+5CFB.0	fGB	0-3E7E
U+5CFB.0	fR/S	 46.7
U+5CFB.0	fTotalStrokes	 10
U+5CFB.0	fCangjie	UICE
U+5CFB.0	fFourCorner	23747
U+5CFB.0	fHakka	 zun5
U+5CFB.0	fCantonese	zeon3
U+5CFB.0	fMandarin	jun4; xun1
U+5CFB.0	fEnglish	[1] high; lofty [2] steep [3] stern; severe; harsh; rigorous
U+5CFC.0	fUTF8	峼
U+5CFC.0	fCNS11643	4-2C5C
U+5CFC.0	fGB	3-3875
U+5CFC.0	fR/S	  46.7
U+5CFC.0	fTotalStrokes	 10
U+5CFC.0	fCangjie	UHGR
U+5CFC.0	fFourCorner	24760
U+5CFC.0	fCantonese	kuk1
U+5CFC.0	fMandarin	hao4
U+5CFD.0	fUTF8	峽
U+5CFD.0	fCNS11643	1-5530
U+5CFD.0	fBig5	AE6C
U+5CFD.0	fGB	1-4F3F
U+5CFD.0	fR/S	 46.7
U+5CFD.0	fTotalStrokes	 10
U+5CFD.0	fCangjie	UKOO
U+5CFD.0	fFourCorner	24738
U+5CFD.0	fHakka	 hiap8;  kiap8; giap7
U+5CFD.0	fCantonese	haap6; hap6 (also); haap3 (also)
U+5CFD.0	fMandarin	xia2
U+5CFD.0	fEnglish	[1] gorge [2] straits [3] isthmus
U+5CFE.0	fUTF8	峾
U+5CFE.0	fCNS11643	4-2C61
U+5CFE.0	fGB	3-387D
U+5CFE.0	fR/S	  46.7
U+5CFE.0	fTotalStrokes	 10
U+5CFE.0	fCangjie	ELU
U+5CFE.0	fFourCorner	32770
U+5CFE.0	fCantonese	ngan4; an4 (also)
U+5CFE.0	fMandarin	yin1; yin2
U+5CFF.0	fUTF8	峿
U+5CFF.0	fCNS11643	2-2E6F
U+5CFF.0	fBig5	D16D
U+5CFF.0	fGB	3-3872
U+5CFF.0	fR/S	 46.7
U+5CFF.0	fTotalStrokes	 10
U+5CFF.0	fCangjie	UMMR
U+5CFF.0	fFourCorner	21761
U+5CFF.0	fHakka	 ng2
U+5CFF.0	fCantonese	jyu5; ng4
U+5CFF.0	fMandarin	wu2; yu3; yu1
U+5CFF.0	fEnglish	mountain
U+5D00.0	fUTF8	崀
U+5D00.0	fCNS11643	2-2E73
U+5D00.0	fBig5	D171
U+5D00.0	fGB	3-3921
U+5D00.0	fR/S	 46.7
U+5D00.0	fTotalStrokes	 10
U+5D00.0	fCangjie	UIAV
U+5D00.0	fFourCorner	22732
U+5D00.0	fHakka	 long2
U+5D00.0	fCantonese	long5 (also); long6; long4
U+5D00.0	fMandarin	lang4; lang3; lang2
U+5D00.0	fEnglish	place name in Hunan province
U+5D01.0	fUTF8	崁
U+5D01.0	fCNS11643	1-5536
U+5D01.0	fBig5	AE72
U+5D01.0	fGB	3-3871
U+5D01.0	fR/S	 46.7
U+5D01.0	fTotalStrokes	 10
U+5D01.0	fCangjie	UGNO
U+5D01.0	fFourCorner	22187
U+5D01.0	fHakka	 kam5
U+5D01.0	fCantonese	ham3
U+5D01.0	fMandarin	kan3; kan4
U+5D01.0	fEnglish	a place in Taiwan Tainan
U+5D02.0	fUTF8	崂
U+5D02.0	fGB	0-6140
U+5D02.0	fR/S	 46.7
U+5D02.0	fTotalStrokes	 10
U+5D02.0	fCangjie	UTBS
U+5D02.0	fCantonese	lou4; dang3
U+5D02.0	fMandarin	lao2; lao1
U+5D02.0	fEnglish	simplified form of &#x5D97;
U+5D03.0	fUTF8	崃
U+5D03.0	fGB	0-6141
U+5D03.0	fR/S	 46.7
U+5D03.0	fTotalStrokes	 10
U+5D03.0	fCangjie	UDT
U+5D03.0	fHakka	 loi2
U+5D03.0	fCantonese	loi4
U+5D03.0	fMandarin	lai2
U+5D03.0	fEnglish	simplified form of &#x5D0D;
U+5D04.0	fUTF8	崄
U+5D04.0	fGB	8-7A77
U+5D04.0	fR/S	 46.13; 46.7
U+5D04.0	fTotalStrokes	 10
U+5D04.0	fCangjie	UOMM
U+5D04.0	fCantonese	him2
U+5D04.0	fMandarin	xian3
U+5D04.0	fEnglish	high, steep, precipitous
U+5D05.0	fUTF8	崅
U+5D05.0	fGB	8-2F70
U+5D05.0	fR/S	 46.7
U+5D05.0	fTotalStrokes	 10
U+5D05.0	fCangjie	XUNBQ
U+5D05.0	fCantonese	gok3
U+5D05.0	fMandarin	que4
U+5D06.0	fUTF8	崆
U+5D06.0	fCNS11643	1-5977
U+5D06.0	fBig5	B153
U+5D06.0	fGB	0-6147
U+5D06.0	fR/S	 46.8
U+5D06.0	fTotalStrokes	 11
U+5D06.0	fCangjie	UJCM
U+5D06.0	fFourCorner	23711
U+5D06.0	fHakka	  kung1; kung3; kung5
U+5D06.0	fCantonese	hung1
U+5D06.0	fMandarin	kong1; qiang1
U+5D06.0	fEnglish	Kongtong mountain
U+5D07.0	fUTF8	崇
U+5D07.0	fCNS11643	1-5976
U+5D07.0	fBig5	B152
U+5D07.0	fGB	0-3367
U+5D07.0	fR/S	 46.8
U+5D07.0	fTotalStrokes	 11
U+5D07.0	fCangjie	UJMF
U+5D07.0	fFourCorner	22901
U+5D07.0	fHakka	  cung2; chung3
U+5D07.0	fCantonese	sung4
U+5D07.0	fMandarin	chong2
U+5D07.0	fEnglish	[1] esteem; honor; revere; venerate; respect; adore; worship [2] high; lofty; noble; dignified; exalted [3] a Chinese family name
U+5D08.0	fUTF8	崈
U+5D08.0	fCNS11643	3-354B
U+5D08.0	fGB	E-2841
U+5D08.0	fR/S	 46.8
U+5D08.0	fTotalStrokes	 11
U+5D08.0	fCangjie	JMMU
U+5D08.0	fFourCorner	30770
U+5D08.0	fHakka	  cung2; chung3
U+5D08.0	fCantonese	sung4
U+5D08.0	fMandarin	chong2
U+5D08.0	fEnglish	same as &#x5D07;
U+5D09.0	fUTF8	崉
U+5D09.0	fCNS11643	3-3548
U+5D09.0	fGB	E-2842
U+5D09.0	fR/S	  46.8
U+5D09.0	fTotalStrokes	 11
U+5D09.0	fCangjie	UEA
U+5D09.0	fFourCorner	22760
U+5D09.0	fHakka	 tap7
U+5D09.0	fCantonese	daap6
U+5D09.0	fMandarin	ta4
U+5D0A.0	fUTF8	崊
U+5D0A.0	fCNS11643	3-3547
U+5D0A.0	fGB	E-2843
U+5D0A.0	fR/S	  46.8
U+5D0A.0	fTotalStrokes	 11
U+5D0A.0	fCangjie	UDD
U+5D0A.0	fFourCorner	24790
U+5D0A.0	fCantonese	lam4
U+5D0A.0	fMandarin	lin2
U+5D0B.0	fUTF8	崋
U+5D0B.0	fCNS11643	2-3478
U+5D0B.0	fBig5	D4F5
U+5D0B.0	fGB	E-2844
U+5D0B.0	fR/S	 46.8
U+5D0B.0	fTotalStrokes	 11
U+5D0B.0	fCangjie	UMTJ
U+5D0B.0	fFourCorner	22504
U+5D0B.0	fHakka	  fa2;  fa5; fa6
U+5D0B.0	fCantonese	waa4; waa6 (also)
U+5D0B.0	fMandarin	hua2; hua4
U+5D0B.0	fEnglish	 flowery illustrious Chinese
U+5D0C.0	fUTF8	崌
U+5D0C.0	fCNS11643	2-347C
U+5D0C.0	fBig5	D4F9
U+5D0C.0	fGB	3-3930
U+5D0C.0	fR/S	 46.8
U+5D0C.0	fTotalStrokes	 11
U+5D0C.0	fCangjie	USJR
U+5D0C.0	fFourCorner	27264
U+5D0C.0	fHakka	 gi1
U+5D0C.0	fCantonese	geoi1
U+5D0C.0	fMandarin	ju1
U+5D0C.0	fEnglish	[1] Mt. Chuu in Szechwan [2] Mt. Chuulai in Szechwan
U+5D0D.0	fUTF8	崍
U+5D0D.0	fCNS11643	2-347E
U+5D0D.0	fBig5	D4FB
U+5D0D.0	fGB	1-6141
U+5D0D.0	fR/S	 46.8
U+5D0D.0	fTotalStrokes	 11
U+5D0D.0	fCangjie	UDOO
U+5D0D.0	fFourCorner	24798
U+5D0D.0	fHakka	  loi2
U+5D0D.0	fCantonese	loi4
U+5D0D.0	fMandarin	lai2
U+5D0D.0	fEnglish	Mt. Lai (in Szechwan province)
U+5D0E.0	fUTF8	崎
U+5D0E.0	fCNS11643	1-5978
U+5D0E.0	fBig5	B154
U+5D0E.0	fGB	0-4669
U+5D0E.0	fR/S	 46.8
U+5D0E.0	fTotalStrokes	 11
U+5D0E.0	fCangjie	UKMR
U+5D0E.0	fFourCorner	24721
U+5D0E.0	fHakka	 ki1;  ki2; gia5
U+5D0E.0	fCantonese	kei1; kei4
U+5D0E.0	fMandarin	qi1; qi3; yi1
U+5D0E.0	fEnglish	rough; uneven; jagged; rugged
U+5D0E.1	fUTF8	崎
U+5D0E.1	fCNS11643	1-5978
U+5D0E.1	fBig5	B154
U+5D0E.1	fGB	0-4669
U+5D0E.1	fR/S	 46.8
U+5D0E.1	fTotalStrokes	 11
U+5D0E.1	fCangjie	UKMR
U+5D0E.1	fFourCorner	24721
U+5D0E.1	fMandarin	qi2
U+5D0E.1	fEnglish	 banks of a winding river
U+5D0F.0	fUTF8	崏
U+5D0F.0	fCNS11643	2-3523
U+5D0F.0	fBig5	D4FE
U+5D0F.0	fGB	E-2845
U+5D0F.0	fR/S	 46.8
U+5D0F.0	fTotalStrokes	 11
U+5D0F.0	fCangjie	UHPA
U+5D0F.0	fFourCorner	22764
U+5D0F.0	fCantonese	man4
U+5D0F.0	fMandarin	min2
U+5D10.0	fUTF8	崐
U+5D10.0	fCNS11643	3-3551
U+5D10.0	fGB	E-2846
U+5D10.0	fR/S	  46.8
U+5D10.0	fTotalStrokes	 11
U+5D10.0	fCangjie	UAPP
U+5D10.0	fFourCorner	26710
U+5D10.0	fHakka	 kun1; kwun1
U+5D10.0	fCantonese	kwan1
U+5D10.0	fMandarin	kun1
U+5D10.0	fEnglish	same as &#x5D11;
U+5D11.0	fUTF8	崑
U+5D11.0	fCNS11643	1-597C
U+5D11.0	fBig5	B158
U+5D11.0	fGB	8-2D63
U+5D11.0	fR/S	 46.8
U+5D11.0	fTotalStrokes	 11
U+5D11.0	fCangjie	UAPP
U+5D11.0	fFourCorner	22711
U+5D11.0	fHakka	  kun1; kwun1
U+5D11.0	fCantonese	kwan1; gwan1
U+5D11.0	fMandarin	kun1
U+5D11.0	fEnglish	[1] Kunlun mountains [2] Kunshan, name of a county and mountain in Kiangsu province
U+5D12.0	fUTF8	崒
U+5D12.0	fCNS11643	2-3525
U+5D12.0	fBig5	D541
U+5D12.0	fGB	5-3B40
U+5D12.0	fR/S	 46.8
U+5D12.0	fTotalStrokes	 11
U+5D12.0	fCangjie	UYOJ
U+5D12.0	fFourCorner	22408
U+5D12.0	fHakka	  cui5; zut7
U+5D12.0	fCantonese	zeot1 (also); zyut6; zyut3
U+5D12.0	fMandarin	zu2; cui4; zu1; zui1
U+5D12.0	fEnglish	 rocky peaks lofty and dangerous
U+5D13.0	fUTF8	崓
U+5D13.0	fCNS11643	3-3552
U+5D13.0	fGB	3-3928
U+5D13.0	fR/S	  46.8
U+5D13.0	fTotalStrokes	 11
U+5D13.0	fCangjie	UWJR
U+5D13.0	fFourCorner	26700
U+5D13.0	fCantonese	gu3; gwu3 (also)
U+5D13.0	fMandarin	gu4
U+5D14.0	fUTF8	崔
U+5D14.0	fCNS11643	1-597E
U+5D14.0	fBig5	B15A
U+5D14.0	fGB	0-345E
U+5D14.0	fR/S	 46.8
U+5D14.0	fTotalStrokes	 11
U+5D14.0	fCangjie	UOG
U+5D14.0	fFourCorner	22214
U+5D14.0	fHakka	  cui1; kok7
U+5D14.0	fCantonese	ceoi1; ceoi4
U+5D14.0	fMandarin	cui1; cui3; zui1
U+5D14.0	fEnglish	[1] high and steep [2] a Chinese family name
U+5D15.0	fUTF8	崕
U+5D15.0	fCNS11643	3-354F
U+5D15.0	fGB	5-3B39
U+5D15.0	fR/S	  46.8
U+5D15.0	fTotalStrokes	 11
U+5D15.0	fCangjie	UMGG
U+5D15.0	fFourCorner	21710
U+5D15.0	fHakka	 ngai2
U+5D15.0	fCantonese	ngaai4; aai4 (also); ngai4; ai4
U+5D15.0	fMandarin	yi2; ya2; yai2
U+5D15.0	fEnglish	same as &#x5D16;
U+5D16.0	fUTF8	崖
U+5D16.0	fCNS11643	1-597A
U+5D16.0	fBig5	B156
U+5D16.0	fGB	0-5142
U+5D16.0	fR/S	 46.8
U+5D16.0	fTotalStrokes	 11
U+5D16.0	fCangjie	UMGG
U+5D16.0	fFourCorner	22214
U+5D16.0	fHakka	  ngai2; na2
U+5D16.0	fCantonese	ngaai4; aai4 (also); ngai4; ai4
U+5D16.0	fMandarin	ya2; yai2
U+5D16.0	fEnglish	[1] cliff; precipice [2] precipitous; high and steep; forbidding [3] brink; verge
U+5D17.0	fUTF8	崗
U+5D17.0	fCNS11643	1-5A24
U+5D17.0	fBig5	B15E
U+5D17.0	fGB	1-385A
U+5D17.0	fR/S	 46.8
U+5D17.0	fTotalStrokes	 11
U+5D17.0	fCangjie	UBTU
U+5D17.0	fFourCorner	22227
U+5D17.0	fHakka	  gong1
U+5D17.0	fCantonese	gong1
U+5D17.0	fMandarin	gang3
U+5D17.0	fEnglish	[1] place where a sentry is posted [2] position [3] non-classical form of &#x5CA1;
U+5D17.1	fUTF8	崗
U+5D17.1	fCNS11643	1-5A24
U+5D17.1	fBig5	B15E
U+5D17.1	fGB	1-385A
U+5D17.1	fR/S	 46.8
U+5D17.1	fTotalStrokes	 11
U+5D17.1	fCangjie	UBTU
U+5D17.1	fFourCorner	22227
U+5D17.1	fMandarin	gang1
U+5D18.0	fUTF8	崘
U+5D18.0	fCNS11643	3-3555
U+5D18.0	fGB	E-2847
U+5D18.0	fR/S	  46.8
U+5D18.0	fTotalStrokes	 11
U+5D18.0	fCangjie	UOMB
U+5D18.0	fFourCorner	28720
U+5D18.0	fHakka	 lun2
U+5D18.0	fCantonese	leon4
U+5D18.0	fMandarin	lun2
U+5D18.0	fEnglish	same as &#x5D19;
U+5D19.0	fUTF8	崙
U+5D19.0	fCNS11643	1-5A21
U+5D19.0	fBig5	B15B
U+5D19.0	fGB	8-2D64
U+5D19.0	fR/S	 46.8
U+5D19.0	fTotalStrokes	 11
U+5D19.0	fCangjie	UOMB
U+5D19.0	fFourCorner	22227
U+5D19.0	fHakka	  lun2; lun3
U+5D19.0	fCantonese	leon4
U+5D19.0	fMandarin	lun2
U+5D19.0	fEnglish	the Kunlun mountains
U+5D1A.0	fUTF8	崚
U+5D1A.0	fCNS11643	2-347A
U+5D1A.0	fBig5	D4F7
U+5D1A.0	fGB	3-3924
U+5D1A.0	fR/S	 46.8
U+5D1A.0	fTotalStrokes	 11
U+5D1A.0	fCangjie	UGCE
U+5D1A.0	fFourCorner	24747
U+5D1A.0	fHakka	  lin2
U+5D1A.0	fCantonese	ling4
U+5D1A.0	fMandarin	leng2; ling1
U+5D1A.0	fEnglish	hilly; steep; rugged
U+5D1B.0	fUTF8	崛
U+5D1B.0	fCNS11643	1-5979
U+5D1B.0	fBig5	B155
U+5D1B.0	fGB	0-6148
U+5D1B.0	fR/S	 46.8
U+5D1B.0	fTotalStrokes	 11
U+5D1B.0	fCangjie	USUU
U+5D1B.0	fFourCorner	27772
U+5D1B.0	fHakka	  kiut7; kut7
U+5D1B.0	fCantonese	gwat6
U+5D1B.0	fMandarin	jue2; yu4
U+5D1B.0	fEnglish	 rise abruptly
U+5D1C.0	fUTF8	崜
U+5D1C.0	fCNS11643	3-3B2B
U+5D1C.0	fGB	3-3929
U+5D1C.0	fR/S	 46.8; 46.9
U+5D1C.0	fTotalStrokes	 12; 11
U+5D1C.0	fCangjie	UHJM
U+5D1C.0	fFourCorner	22710
U+5D1C.0	fCantonese	do1
U+5D1C.0	fMandarin	duo1
U+5D1D.0	fUTF8	崝
U+5D1D.0	fCNS11643	2-3479
U+5D1D.0	fBig5	D4F6
U+5D1D.0	fGB	3-3923
U+5D1D.0	fR/S	 46.8
U+5D1D.0	fTotalStrokes	 11
U+5D1D.0	fCangjie	UQMB
U+5D1D.0	fFourCorner	25721
U+5D1D.0	fHakka	 zen1
U+5D1D.0	fCantonese	zang1; zaang1 (also)
U+5D1D.0	fMandarin	zheng1; cheng1; cheng2
U+5D1E.0	fUTF8	崞
U+5D1E.0	fCNS11643	2-3477
U+5D1E.0	fBig5	D4F4
U+5D1E.0	fGB	0-6146
U+5D1E.0	fR/S	 46.8
U+5D1E.0	fTotalStrokes	 11
U+5D1E.0	fCangjie	UYRD
U+5D1E.0	fFourCorner	20747
U+5D1E.0	fHakka	 gok7; gwok7
U+5D1E.0	fCantonese	gwok3; gok3 (also)
U+5D1E.0	fMandarin	guo1
U+5D1E.0	fEnglish	mountain in Shanxi
U+5D1F.0	fUTF8	崟
U+5D1F.0	fCNS11643	2-3527
U+5D1F.0	fBig5	D543
U+5D1F.0	fGB	3-392C
U+5D1F.0	fR/S	 46.8
U+5D1F.0	fTotalStrokes	 11
U+5D1F.0	fCangjie	UC
U+5D1F.0	fFourCorner	22109
U+5D1F.0	fHakka	 ngim2; jin5
U+5D1F.0	fCantonese	jam4
U+5D1F.0	fMandarin	yin2
U+5D1F.0	fEnglish	cliffs
U+5D20.0	fUTF8	崠
U+5D20.0	fCNS11643	2-347B
U+5D20.0	fBig5	D4F8
U+5D20.0	fGB	3-3851
U+5D20.0	fR/S	 46.8
U+5D20.0	fTotalStrokes	 11
U+5D20.0	fCangjie	UDW
U+5D20.0	fFourCorner	25796
U+5D20.0	fHakka	 dung3; dung5
U+5D20.0	fCantonese	dung3
U+5D20.0	fMandarin	dong1; dong4; cheng2; ju1
U+5D20.0	fEnglish	a place name in Guangxi province
U+5D21.0	fUTF8	崡
U+5D21.0	fCNS11643	3-3546
U+5D21.0	fGB	5-3B45
U+5D21.0	fR/S	  46.8
U+5D21.0	fTotalStrokes	 11
U+5D21.0	fCangjie	UUNE
U+5D21.0	fFourCorner	27770
U+5D21.0	fHakka	 ham2
U+5D21.0	fCantonese	haam4; ham4
U+5D21.0	fMandarin	han2
U+5D21.0	fEnglish	[1] name of a checkpoint during Chin and Han dynasties, located in what is Honan today [2] name of a mountain in Honan
U+5D22.0	fUTF8	崢
U+5D22.0	fCNS11643	1-597B
U+5D22.0	fBig5	B157
U+5D22.0	fGB	1-613F
U+5D22.0	fR/S	 46.8
U+5D22.0	fTotalStrokes	 11
U+5D22.0	fCangjie	UBSD
U+5D22.0	fFourCorner	22757
U+5D22.0	fHakka	 zang1;  zen1
U+5D22.0	fCantonese	zang1; zaang1 (also); caang4
U+5D22.0	fMandarin	zheng1; cheng2
U+5D22.0	fEnglish	[1] high; lofty [2] outstanding; distinguished [3] steep; perilous; dangerous [4] harsh; severe; rigorous
U+5D23.0	fUTF8	崣
U+5D23.0	fCNS11643	2-3526
U+5D23.0	fBig5	D542
U+5D23.0	fGB	3-392A
U+5D23.0	fR/S	 46.8
U+5D23.0	fTotalStrokes	 11
U+5D23.0	fCangjie	UHDV
U+5D23.0	fFourCorner	22404
U+5D23.0	fCantonese	wai2
U+5D23.0	fMandarin	wei3
U+5D24.0	fUTF8	崤
U+5D24.0	fCNS11643	1-5A22
U+5D24.0	fBig5	B15C
U+5D24.0	fGB	0-6145
U+5D24.0	fR/S	 46.8
U+5D24.0	fTotalStrokes	 11
U+5D24.0	fCangjie	UKKB
U+5D24.0	fFourCorner	24727
U+5D24.0	fHakka	  ngau2
U+5D24.0	fCantonese	ngaau4; aau4 (also)
U+5D24.0	fMandarin	yao2; xiao2
U+5D24.0	fEnglish	Mt. Yao (in Honan province)
U+5D25.0	fUTF8	崥
U+5D25.0	fCNS11643	2-3522
U+5D25.0	fBig5	D4FD
U+5D25.0	fGB	3-392B
U+5D25.0	fR/S	 46.8
U+5D25.0	fTotalStrokes	 11
U+5D25.0	fCangjie	UHWJ
U+5D25.0	fFourCorner	26740
U+5D25.0	fHakka	  pi2;  bi1; pai2
U+5D25.0	fCantonese	pei4; bei2; bi1; pai4; bi2
U+5D25.0	fMandarin	pi3; bi1; pi2
U+5D25.0	fEnglish	hilly
U+5D26.0	fUTF8	崦
U+5D26.0	fCNS11643	2-3521
U+5D26.0	fBig5	D4FC
U+5D26.0	fGB	0-6143
U+5D26.0	fR/S	 46.8
U+5D26.0	fTotalStrokes	 11
U+5D26.0	fCangjie	UKLU
U+5D26.0	fFourCorner	24716
U+5D26.0	fHakka	 jam1; ram1; jam3
U+5D26.0	fCantonese	jim1
U+5D26.0	fMandarin	yan1; yan3
U+5D26.0	fEnglish	Mt Yen, (in Kansu, where there is a cave into which the sun is said to sink at night)
U+5D27.0	fUTF8	崧
U+5D27.0	fCNS11643	1-5A23
U+5D27.0	fBig5	B15D
U+5D27.0	fGB	0-6142
U+5D27.0	fR/S	 46.8
U+5D27.0	fTotalStrokes	 11
U+5D27.0	fCangjie	UDCI
U+5D27.0	fFourCorner	22938
U+5D27.0	fHakka	 sung1;  cung2; ciung2
U+5D27.0	fCantonese	sung1
U+5D27.0	fMandarin	song1
U+5D27.0	fEnglish	[1] high mountain [2] lofty
U+5D28.0	fUTF8	崨
U+5D28.0	fCNS11643	2-347D
U+5D28.0	fBig5	D4FA
U+5D28.0	fGB	3-3926
U+5D28.0	fR/S	 46.8
U+5D28.0	fTotalStrokes	 11
U+5D28.0	fCangjie	UJLO
U+5D28.0	fFourCorner	25781
U+5D28.0	fCantonese	zip6; zit6
U+5D28.0	fMandarin	jie2; jie4
U+5D29.0	fUTF8	崩
U+5D29.0	fCNS11643	1-597D
U+5D29.0	fBig5	B159
U+5D29.0	fGB	0-3140
U+5D29.0	fR/S	 46.8
U+5D29.0	fTotalStrokes	 11
U+5D29.0	fCangjie	UBB
U+5D29.0	fFourCorner	22227
U+5D29.0	fHakka	  ben1; bien1
U+5D29.0	fCantonese	bang1
U+5D29.0	fMandarin	beng1
U+5D29.0	fEnglish	[1] collapse; disintegrate; fall [2] die (said of an emperor)
U+5D2A.0	fUTF8	崪
U+5D2A.0	fCNS11643	3-354E
U+5D2A.0	fGB	3-392D
U+5D2A.0	fR/S	 46.8
U+5D2A.0	fTotalStrokes	 11
U+5D2A.0	fCangjie	UYOJ
U+5D2A.0	fFourCorner	20740
U+5D2A.0	fHakka	 cui5
U+5D2A.0	fCantonese	zyut6; zeot1 (also); zyut3
U+5D2A.0	fMandarin	zu2; cui4; zu1; zui1
U+5D2A.0	fEnglish	same as &#x5D12;
U+5D2B.0	fUTF8	崫
U+5D2B.0	fCNS11643	6-3E76
U+5D2B.0	fGB	3-3931
U+5D2B.0	fR/S	 46.8
U+5D2B.0	fTotalStrokes	 11
U+5D2B.0	fCangjie	XUSUU
U+5D2B.0	fFourCorner	22270
U+5D2B.0	fCantonese	fat1
U+5D2B.0	fMandarin	jue2
U+5D2B.0	fEnglish	high and lofty (mountains)
U+5D2C.0	fUTF8	崬
U+5D2C.0	fCNS11643	3-354A
U+5D2C.0	fGB	1-6134
U+5D2C.0	fR/S	  46.8
U+5D2C.0	fTotalStrokes	 11
U+5D2C.0	fCangjie	UDW
U+5D2C.0	fFourCorner	22900
U+5D2C.0	fHakka	 dung3
U+5D2C.0	fCantonese	dung1
U+5D2C.0	fMandarin	dong1
U+5D2C.0	fEnglish	 top; summit
U+5D2D.0	fUTF8	崭
U+5D2D.0	fGB	0-5538
U+5D2D.0	fR/S	 46.8
U+5D2D.0	fTotalStrokes	 11
U+5D2D.0	fCangjie	UKQL
U+5D2D.0	fHakka	 cam3
U+5D2D.0	fCantonese	zaam2; zaam3 (also); caam4
U+5D2D.0	fMandarin	zhan3; chan2
U+5D2D.0	fEnglish	simplified form of &#x5D84;
U+5D2E.0	fUTF8	崮
U+5D2E.0	fCNS11643	2-3528
U+5D2E.0	fBig5	D544
U+5D2E.0	fGB	0-6144
U+5D2E.0	fR/S	 46.8
U+5D2E.0	fTotalStrokes	 11
U+5D2E.0	fCangjie	UWJR
U+5D2E.0	fFourCorner	22607
U+5D2E.0	fCantonese	gu3; gwu3 (also)
U+5D2E.0	fMandarin	gu4
U+5D2F.0	fUTF8	崯
U+5D2F.0	fCNS11643	3-3554
U+5D2F.0	fGB	5-3B3E
U+5D2F.0	fR/S	 46.8
U+5D2F.0	fTotalStrokes	 11
U+5D2F.0	fCangjie	UC
U+5D2F.0	fFourCorner	28710
U+5D2F.0	fCantonese	jam4
U+5D2F.0	fMandarin	yin2
U+5D30.0	fUTF8	崰
U+5D30.0	fCNS11643	2-3524
U+5D30.0	fBig5	D540
U+5D30.0	fGB	E-2848
U+5D30.0	fR/S	 46.8
U+5D30.0	fTotalStrokes	 11
U+5D30.0	fCangjie	UVVW
U+5D30.0	fFourCorner	22760
U+5D30.0	fCantonese	zi1
U+5D30.0	fMandarin	zi1
U+5D31.0	fUTF8	崱
U+5D31.0	fCNS11643	2-3B4B
U+5D31.0	fBig5	D8E7
U+5D31.0	fGB	3-3868
U+5D31.0	fR/S	 46.9
U+5D31.0	fTotalStrokes	 12
U+5D31.0	fCangjie	UBCN
U+5D31.0	fFourCorner	22827
U+5D31.0	fHakka	  zet7
U+5D31.0	fCantonese	zak1; sik6
U+5D31.0	fMandarin	ze2; ze4
U+5D31.0	fEnglish	lofty
U+5D32.0	fUTF8	崲
U+5D32.0	fCNS11643	2-3B52
U+5D32.0	fBig5	D8EE
U+5D32.0	fGB	5-3B4E
U+5D32.0	fR/S	 46.9
U+5D32.0	fTotalStrokes	 12
U+5D32.0	fCangjie	UHAG
U+5D32.0	fFourCorner	26714
U+5D32.0	fCantonese	wong4
U+5D32.0	fMandarin	huang2
U+5D33.0	fUTF8	崳
U+5D33.0	fCNS11643	2-3B47
U+5D33.0	fBig5	D8E3
U+5D33.0	fGB	5-3B51
U+5D33.0	fR/S	 46.9
U+5D33.0	fTotalStrokes	 12
U+5D33.0	fCangjie	UOMN
U+5D33.0	fFourCorner	28721
U+5D33.0	fCantonese	jyu4
U+5D33.0	fMandarin	yu2
U+5D33.0	fEnglish	county in Shandong province
U+5D34.0	fUTF8	崴
U+5D34.0	fCNS11643	1-5E76
U+5D34.0	fBig5	B451
U+5D34.0	fGB	0-614B
U+5D34.0	fR/S	 46.9
U+5D34.0	fTotalStrokes	 12
U+5D34.0	fCangjie	UIHV
U+5D34.0	fFourCorner	22253
U+5D34.0	fHakka	 wui1; wi2; vui1
U+5D34.0	fCantonese	wai1; waai1; waai2
U+5D34.0	fMandarin	wei1; wai3; wa1
U+5D34.0	fEnglish	high; lofty
U+5D35.0	fUTF8	崵
U+5D35.0	fCNS11643	2-3B43
U+5D35.0	fBig5	D8DF
U+5D35.0	fGB	5-3A5B
U+5D35.0	fR/S	 46.9
U+5D35.0	fTotalStrokes	 12
U+5D35.0	fCangjie	UAMH
U+5D35.0	fFourCorner	26727
U+5D35.0	fHakka	 jong2; rong2; jong1
U+5D35.0	fCantonese	joeng4; dong6
U+5D35.0	fMandarin	yang2
U+5D35.0	fEnglish	a mountain in Shensi
U+5D36.0	fUTF8	崶
U+5D36.0	fCNS11643	2-3B53
U+5D36.0	fBig5	D8EF
U+5D36.0	fGB	3-3932
U+5D36.0	fR/S	 46.9
U+5D36.0	fTotalStrokes	 12
U+5D36.0	fCangjie	UGGI
U+5D36.0	fFourCorner	22147
U+5D36.0	fHakka	  fung1
U+5D36.0	fCantonese	fung1
U+5D36.0	fMandarin	feng1
U+5D36.0	fEnglish	name of a fabulous hill; emblem of literary success
U+5D37.0	fUTF8	崷
U+5D37.0	fCNS11643	2-3B3D
U+5D37.0	fBig5	D8D9
U+5D37.0	fGB	3-3945
U+5D37.0	fR/S	 46.9
U+5D37.0	fTotalStrokes	 12
U+5D37.0	fCangjie	UTCW
U+5D37.0	fFourCorner	28761
U+5D37.0	fCantonese	cau4
U+5D37.0	fMandarin	qiu2
U+5D38.0	fUTF8	崸
U+5D38.0	fCNS11643	2-3B50
U+5D38.0	fBig5	D8EC
U+5D38.0	fGB	E-2849
U+5D38.0	fR/S	 46.9
U+5D38.0	fTotalStrokes	 12
U+5D38.0	fCangjie	UMBC
U+5D38.0	fFourCorner	21786
U+5D38.0	fCantonese	ngong4; ong4 (also); dan6
U+5D38.0	fMandarin	dun4
U+5D39.0	fUTF8	崹
U+5D39.0	fCNS11643	2-3B4E
U+5D39.0	fBig5	D8EA
U+5D39.0	fGB	3-3942
U+5D39.0	fR/S	 46.9
U+5D39.0	fTotalStrokes	 12
U+5D39.0	fCangjie	UYBB
U+5D39.0	fFourCorner	20727
U+5D39.0	fCantonese	tai4
U+5D39.0	fMandarin	ti2
U+5D3A.0	fUTF8	崺
U+5D3A.0	fCNS11643	2-3B48
U+5D3A.0	fBig5	D8E4
U+5D3A.0	fGB	3-3943
U+5D3A.0	fR/S	 46.9
U+5D3A.0	fTotalStrokes	 12
U+5D3A.0	fCangjie	UYSD
U+5D3A.0	fFourCorner	22212
U+5D3A.0	fCantonese	ji5; ji4
U+5D3A.0	fMandarin	yi3
U+5D3B.0	fUTF8	崻
U+5D3B.0	fCNS11643	3-3B36
U+5D3B.0	fGB	5-3B50
U+5D3B.0	fR/S	 46.9
U+5D3B.0	fTotalStrokes	 12
U+5D3B.0	fCangjie	UHOI
U+5D3B.0	fFourCorner	24740
U+5D3B.0	fCantonese	zi6
U+5D3B.0	fMandarin	zhi4
U+5D3C.0	fUTF8	崼
U+5D3C.0	fCNS11643	2-3B51
U+5D3C.0	fBig5	D8ED
U+5D3C.0	fGB	3-3937
U+5D3C.0	fR/S	 46.9
U+5D3C.0	fTotalStrokes	 12
U+5D3C.0	fCangjie	UAMO
U+5D3C.0	fFourCorner	26781
U+5D3C.0	fCantonese	si6
U+5D3C.0	fMandarin	shi4
U+5D3D.0	fUTF8	崽
U+5D3D.0	fCNS11643	2-3B4A
U+5D3D.0	fBig5	D8E6
U+5D3D.0	fGB	0-614C
U+5D3D.0	fR/S	 46.9
U+5D3D.0	fTotalStrokes	 12
U+5D3D.0	fCangjie	UWP
U+5D3D.0	fFourCorner	22336
U+5D3D.0	fHakka	  zai3;  sii1; sai3
U+5D3D.0	fCantonese	zai2; zoi2 (also)
U+5D3D.0	fMandarin	zai3; shai1
U+5D3D.0	fEnglish	[1] son [2] Chinese servant of a foreigner (derogatory term) [3] young animal; whelp
U+5D3E.0	fUTF8	崾
U+5D3E.0	fCNS11643	3-3B28
U+5D3E.0	fGB	0-614A
U+5D3E.0	fR/S	  46.9
U+5D3E.0	fTotalStrokes	 12
U+5D3E.0	fCangjie	UMWV
U+5D3E.0	fFourCorner	21740
U+5D3E.0	fCantonese	jiu5
U+5D3E.0	fMandarin	yao3
U+5D3E.0	fEnglish	place name in Shanxi province
U+5D3F.0	fUTF8	崿
U+5D3F.0	fCNS11643	2-3B42
U+5D3F.0	fBig5	D8DE
U+5D3F.0	fGB	3-393B
U+5D3F.0	fR/S	 46.9
U+5D3F.0	fTotalStrokes	 12
U+5D3F.0	fCangjie	URRS
U+5D3F.0	fFourCorner	26727
U+5D3F.0	fHakka	  ngok8
U+5D3F.0	fCantonese	ngok6; ok6 (also)
U+5D3F.0	fMandarin	e4
U+5D3F.0	fEnglish	 cliff; precipice
U+5D40.0	fUTF8	嵀
U+5D40.0	fCNS11643	2-3B54
U+5D40.0	fBig5	D8F0
U+5D40.0	fGB	5-3B48
U+5D40.0	fR/S	 46.9
U+5D40.0	fTotalStrokes	 12
U+5D40.0	fCangjie	UDYG
U+5D40.0	fFourCorner	22914
U+5D40.0	fCantonese	cyu5
U+5D40.0	fMandarin	zhu4
U+5D41.0	fUTF8	嵁
U+5D41.0	fCNS11643	2-3B40
U+5D41.0	fBig5	D8DC
U+5D41.0	fGB	3-3933
U+5D41.0	fR/S	 46.9
U+5D41.0	fTotalStrokes	 12
U+5D41.0	fCangjie	UTMV
U+5D41.0	fFourCorner	24711
U+5D41.0	fHakka	  kam1
U+5D41.0	fCantonese	ham1
U+5D41.0	fMandarin	kan1; an2; qian1; zhan4
U+5D41.0	fEnglish	 bear; sustain; be capable, be worthy of
U+5D42.0	fUTF8	嵂
U+5D42.0	fCNS11643	2-3B4D
U+5D42.0	fBig5	D8E9
U+5D42.0	fGB	3-393E
U+5D42.0	fR/S	 46.9
U+5D42.0	fTotalStrokes	 12
U+5D42.0	fCangjie	UHOQ
U+5D42.0	fFourCorner	22257
U+5D42.0	fHakka	 lut8; lit8
U+5D42.0	fCantonese	leot6
U+5D42.0	fMandarin	l4; luu4; lu4
U+5D42.0	fEnglish	lofty
U+5D43.0	fUTF8	嵃
U+5D43.0	fCNS11643	2-3B3E
U+5D43.0	fBig5	D8DA
U+5D43.0	fGB	3-3941
U+5D43.0	fR/S	 46.9
U+5D43.0	fTotalStrokes	 12
U+5D43.0	fCangjie	UYHH
U+5D43.0	fFourCorner	20722
U+5D43.0	fCantonese	jin5; jin6
U+5D43.0	fMandarin	yan3; yan4
U+5D44.0	fUTF8	嵄
U+5D44.0	fCNS11643	3-3B27
U+5D44.0	fGB	E-284A
U+5D44.0	fR/S	  46.9
U+5D44.0	fTotalStrokes	 12
U+5D44.0	fCangjie	UTGK
U+5D44.0	fFourCorner	28730
U+5D44.0	fCantonese	mei5
U+5D44.0	fMandarin	mei3
U+5D45.0	fUTF8	嵅
U+5D45.0	fCNS11643	2-3B55
U+5D45.0	fBig5	D8F1
U+5D45.0	fGB	E-284B
U+5D45.0	fR/S	 46.9
U+5D45.0	fTotalStrokes	 12
U+5D45.0	fCangjie	UIHR
U+5D45.0	fFourCorner	22253
U+5D45.0	fCantonese	haam4; ham4
U+5D45.0	fMandarin	gan1; gan4; yin1
U+5D46.0	fUTF8	嵆
U+5D46.0	fCNS11643	3-3B37
U+5D46.0	fGB	5-3B4D
U+5D46.0	fR/S	  46.9
U+5D46.0	fTotalStrokes	 12
U+5D46.0	fCangjie	HUU
U+5D46.0	fFourCorner	23770
U+5D46.0	fCantonese	kai1
U+5D46.0	fMandarin	xi1; ji1; obs; xi2
U+5D46.0	fEnglish	[1] mountain in Honan [2] a Chinese family name
U+5D47.0	fUTF8	嵇
U+5D47.0	fCNS11643	1-5E77
U+5D47.0	fBig5	B452
U+5D47.0	fGB	0-6F7A
U+5D47.0	fR/S	 46.9
U+5D47.0	fTotalStrokes	 12
U+5D47.0	fCangjie	HDIUU
U+5D47.0	fFourCorner	23971
U+5D47.0	fHakka	  hi1
U+5D47.0	fCantonese	kai1; hai4 (also)
U+5D47.0	fMandarin	ji1; xi2; obs
U+5D47.0	fEnglish	same as &#x5D46;
U+5D48.0	fUTF8	嵈
U+5D48.0	fCNS11643	3-3B2A
U+5D48.0	fGB	E-284C
U+5D48.0	fR/S	  46.9
U+5D48.0	fTotalStrokes	 12
U+5D48.0	fCangjie	UBME
U+5D48.0	fFourCorner	22740
U+5D48.0	fCantonese	wun6
U+5D48.0	fMandarin	huan3; huan4
U+5D49.0	fUTF8	嵉
U+5D49.0	fCNS11643	2-3B4F
U+5D49.0	fBig5	D8EB
U+5D49.0	fGB	5-3B53
U+5D49.0	fR/S	 46.9
U+5D49.0	fTotalStrokes	 12
U+5D49.0	fCangjie	UYRN
U+5D49.0	fFourCorner	20721
U+5D49.0	fCantonese	ting4
U+5D49.0	fMandarin	ting2
U+5D4A.0	fUTF8	嵊
U+5D4A.0	fCNS11643	2-4255
U+5D4A.0	fBig5	DD4F
U+5D4A.0	fGB	0-6153
U+5D4A.0	fR/S	 46.10
U+5D4A.0	fTotalStrokes	 13
U+5D4A.0	fCangjie	UHDP
U+5D4A.0	fFourCorner	22791
U+5D4A.0	fHakka	 shin6; siin5; shin5
U+5D4A.0	fCantonese	sing6
U+5D4A.0	fMandarin	sheng4; cheng2; lang4
U+5D4A.0	fEnglish	[1] name of a mountain in Chekiang [2] name of a county in Chekiang
U+5D4B.0	fUTF8	嵋
U+5D4B.0	fCNS11643	2-3B41
U+5D4B.0	fBig5	D8DD
U+5D4B.0	fGB	0-6152
U+5D4B.0	fR/S	 46.9
U+5D4B.0	fTotalStrokes	 12
U+5D4B.0	fCangjie	UAHU
U+5D4B.0	fFourCorner	27767
U+5D4B.0	fHakka	 mi2
U+5D4B.0	fCantonese	mei4
U+5D4B.0	fMandarin	mei2
U+5D4B.0	fEnglish	Mt. Omei, a Buddhistic resort in Szechwan
U+5D4C.0	fUTF8	嵌
U+5D4C.0	fCNS11643	1-5E74
U+5D4C.0	fBig5	B44F
U+5D4C.0	fGB	0-4736
U+5D4C.0	fR/S	 46.9
U+5D4C.0	fTotalStrokes	 12
U+5D4C.0	fCangjie	UTMO
U+5D4C.0	fFourCorner	22782
U+5D4C.0	fHakka	  kiam2; kam5; giam1; kam1
U+5D4C.0	fCantonese	ham3; ham6 (also); haam1 (also); ham2
U+5D4C.0	fMandarin	qian4; qian1; han3; han2; kan4; qian2; zan4
U+5D4C.0	fEnglish	 inlay; set in
U+5D4D.0	fUTF8	嵍
U+5D4D.0	fCNS11643	3-3B2E
U+5D4D.0	fGB	E-284D
U+5D4D.0	fR/S	  46.9
U+5D4D.0	fTotalStrokes	 12
U+5D4D.0	fCangjie	NKU
U+5D4D.0	fFourCorner	18770
U+5D4D.0	fCantonese	mou6
U+5D4D.0	fMandarin	wu4
U+5D4E.0	fUTF8	嵎
U+5D4E.0	fCNS11643	2-3B45
U+5D4E.0	fBig5	D8E1
U+5D4E.0	fGB	3-3938
U+5D4E.0	fR/S	 46.9
U+5D4E.0	fTotalStrokes	 12
U+5D4E.0	fCangjie	UWLB
U+5D4E.0	fFourCorner	26727
U+5D4E.0	fHakka	 ngi2
U+5D4E.0	fCantonese	jyu4
U+5D4E.0	fMandarin	yu2
U+5D4E.0	fEnglish	 mountain recess; curved place in the mountains; corner of a hill
U+5D4F.0	fUTF8	嵏
U+5D4F.0	fCNS11643	3-3B34
U+5D4F.0	fGB	3-393F
U+5D4F.0	fR/S	  46.9
U+5D4F.0	fTotalStrokes	 12
U+5D4F.0	fCangjie	UUCE
U+5D4F.0	fFourCorner	22400
U+5D4F.0	fHakka	 zung1
U+5D4F.0	fCantonese	zung1
U+5D4F.0	fMandarin	zong1
U+5D4F.0	fEnglish	a mountain in Shenxi
U+5D50.0	fUTF8	嵐
U+5D50.0	fCNS11643	1-5E75
U+5D50.0	fBig5	B450
U+5D50.0	fGB	1-6130
U+5D50.0	fR/S	 46.9
U+5D50.0	fTotalStrokes	 12
U+5D50.0	fCangjie	UHNI
U+5D50.0	fFourCorner	22217
U+5D50.0	fHakka	 lam2; lim3
U+5D50.0	fCantonese	laam4; laam2 (also)
U+5D50.0	fMandarin	lan2
U+5D50.0	fEnglish	mountain mist; mountain haze; mountain vapor; mist
U+5D51.0	fUTF8	嵑
U+5D51.0	fCNS11643	2-3B44
U+5D51.0	fBig5	D8E0
U+5D51.0	fGB	3-3939
U+5D51.0	fR/S	 46.9
U+5D51.0	fTotalStrokes	 12
U+5D51.0	fCangjie	UAPV
U+5D51.0	fFourCorner	26727
U+5D51.0	fHakka	 giet7
U+5D51.0	fCantonese	hot3; kit3
U+5D51.0	fMandarin	jue2; jie2; ke3; he2
U+5D51.0	fEnglish	lofty (mountains)
U+5D52.0	fUTF8	嵒
U+5D52.0	fCNS11643	2-3B49
U+5D52.0	fBig5	D8E5
U+5D52.0	fGB	E-284E
U+5D52.0	fR/S	 46.9
U+5D52.0	fTotalStrokes	 12
U+5D52.0	fCangjie	RRRU
U+5D52.0	fFourCorner	60772
U+5D52.0	fHakka	 ngam2
U+5D52.0	fCantonese	ngaam4; aam4 (also)
U+5D52.0	fMandarin	yan2
U+5D52.0	fEnglish	 cliff soar
U+5D53.0	fUTF8	嵓
U+5D53.0	fCNS11643	3-3B33
U+5D53.0	fGB	5-3B4C
U+5D53.0	fR/S	 46.9
U+5D53.0	fTotalStrokes	 12
U+5D53.0	fCangjie	URRR
U+5D53.0	fFourCorner	22600
U+5D53.0	fHakka	 ngam2
U+5D53.0	fCantonese	ngaam4; aam4 (also)
U+5D53.0	fMandarin	yan2
U+5D53.0	fEnglish	same as &#x5D52;
U+5D54.0	fUTF8	嵔
U+5D54.0	fCNS11643	4-3735
U+5D54.0	fGB	5-3B4A
U+5D54.0	fR/S	  46.9
U+5D54.0	fTotalStrokes	 12
U+5D54.0	fCangjie	UWMV
U+5D54.0	fFourCorner	22730
U+5D54.0	fCantonese	wai3
U+5D54.0	fMandarin	wei3; wei4
U+5D55.0	fUTF8	嵕
U+5D55.0	fCNS11643	2-3B46
U+5D55.0	fBig5	D8E2
U+5D55.0	fGB	5-3B52
U+5D55.0	fR/S	 46.9
U+5D55.0	fTotalStrokes	 12
U+5D55.0	fCangjie	UUCE
U+5D55.0	fFourCorner	22747
U+5D55.0	fHakka	 zung1
U+5D55.0	fCantonese	zung1
U+5D55.0	fMandarin	zong1
U+5D55.0	fEnglish	same as &#x5D4F;
U+5D56.0	fUTF8	嵖
U+5D56.0	fCNS11643	4-372F
U+5D56.0	fGB	3-3936
U+5D56.0	fR/S	  46.9
U+5D56.0	fTotalStrokes	 12
U+5D56.0	fCangjie	UDAM
U+5D56.0	fFourCorner	24710
U+5D56.0	fCantonese	zaa6; caa4
U+5D56.0	fMandarin	cha2
U+5D56.0	fEnglish	name of a mountain
U+5D57.0	fUTF8	嵗
U+5D57.0	fCNS11643	3-3B31
U+5D57.0	fGB	E-284F
U+5D57.0	fR/S	 46.9
U+5D57.0	fTotalStrokes	 12
U+5D57.0	fCangjie	UIHH
U+5D57.0	fFourCorner	22250
U+5D57.0	fHakka	 soi5; sui5; se5
U+5D57.0	fCantonese	seoi3
U+5D57.0	fMandarin	sui4; xue3; suo4
U+5D57.0	fEnglish	year; age; harvest
U+5D58.0	fUTF8	嵘
U+5D58.0	fGB	0-6149
U+5D58.0	fR/S	 46.9
U+5D58.0	fTotalStrokes	 12
U+5D58.0	fCangjie	UTBD
U+5D58.0	fHakka	 jin2; jung2
U+5D58.0	fCantonese	wing4
U+5D58.0	fMandarin	rong2; heng2
U+5D58.0	fEnglish	simplified form of &#x5DB8;
U+5D59.0	fUTF8	嵙
U+5D59.0	fCNS11643	2-3B4C
U+5D59.0	fBig5	D8E8
U+5D59.0	fGB	E-2850
U+5D59.0	fR/S	 46.9
U+5D59.0	fTotalStrokes	 12
U+5D59.0	fCangjie	UHDJ
U+5D59.0	fFourCorner	22947
U+5D59.0	fCantonese	fo1
U+5D59.0	fMandarin	ke1; wu4
U+5D59.0	fEnglish	place name
U+5D5A.0	fUTF8	嵚
U+5D5A.0	fGB	8-7A7C
U+5D5A.0	fR/S	 46.12; 46.9
U+5D5A.0	fTotalStrokes	 12
U+5D5A.0	fCangjie	XUCNO
U+5D5A.0	fCantonese	jam1
U+5D5A.0	fMandarin	qin1
U+5D5A.0	fEnglish	lofty, towering
U+5D5B.0	fUTF8	嵛
U+5D5B.0	fCNS11643	3-3B2D
U+5D5B.0	fGB	0-614E
U+5D5B.0	fR/S	  46.9
U+5D5B.0	fTotalStrokes	 12
U+5D5B.0	fCangjie	UOMN
U+5D5B.0	fFourCorner	22220
U+5D5B.0	fCantonese	jyu4
U+5D5B.0	fMandarin	yu2
U+5D5B.0	fEnglish	county in Shandong province
U+5D5C.0	fUTF8	嵜
U+5D5C.0	fGB	E-2851
U+5D5C.0	fR/S	 46.9
U+5D5C.0	fTotalStrokes	 12
U+5D5C.0	fCangjie	UYTR
U+5D5C.0	fCantonese	kei1
U+5D5C.0	fMandarin	qi2
U+5D5C.0	fEnglish	 rough, uneven, jagged, rugged
U+5D5D.0	fUTF8	嵝
U+5D5D.0	fGB	0-6150
U+5D5D.0	fR/S	 46.9
U+5D5D.0	fTotalStrokes	 12
U+5D5D.0	fCangjie	UFDV
U+5D5D.0	fHakka	 leu3
U+5D5D.0	fCantonese	lau5
U+5D5D.0	fMandarin	lou3; lou4; l3
U+5D5D.0	fEnglish	simplified form of &#x5D81;
U+5D5E.0	fUTF8	嵞
U+5D5E.0	fCNS11643	2-4259
U+5D5E.0	fBig5	DD53
U+5D5E.0	fGB	E-2852
U+5D5E.0	fR/S	 46.10
U+5D5E.0	fTotalStrokes	 13
U+5D5E.0	fCangjie	OMDU
U+5D5E.0	fFourCorner	80779
U+5D5E.0	fHakka	 tu2
U+5D5E.0	fCantonese	tou4
U+5D5E.0	fMandarin	tu2
U+5D5E.0	fEnglish	the mountain and region whence came the wife of Thai Yi
U+5D5F.0	fUTF8	嵟
U+5D5F.0	fCNS11643	4-3C79
U+5D5F.0	fGB	5-3B59
U+5D5F.0	fR/S	 46.10
U+5D5F.0	fTotalStrokes	 13
U+5D5F.0	fCangjie	UMOG
U+5D5F.0	fFourCorner	22210
U+5D5F.0	fCantonese	deoi1
U+5D5F.0	fMandarin	dui1
U+5D60.0	fUTF8	嵠
U+5D60.0	fCNS11643	3-4121
U+5D60.0	fGB	5-3B5D
U+5D60.0	fR/S	 46.10
U+5D60.0	fTotalStrokes	 13
U+5D60.0	fCangjie	UBVK
U+5D60.0	fFourCorner	22730
U+5D60.0	fHakka	  hai1; k'ie1; kai1; k'e1; ke1
U+5D60.0	fCantonese	kai1; kai4
U+5D60.0	fMandarin	xi1; qi1; ji1
U+5D60.0	fEnglish	same as &#x8C3F;
U+5D61.0	fUTF8	嵡
U+5D61.0	fCNS11643	4-3C7B
U+5D61.0	fGB	5-3B5E
U+5D61.0	fR/S	 46.10
U+5D61.0	fTotalStrokes	 13
U+5D61.0	fCangjie	UCIM
U+5D61.0	fFourCorner	22120
U+5D61.0	fCantonese	jung2
U+5D61.0	fMandarin	weng1
U+5D62.0	fUTF8	嵢
U+5D62.0	fCNS11643	2-425C
U+5D62.0	fBig5	DD56
U+5D62.0	fGB	5-3A60
U+5D62.0	fR/S	 46.10
U+5D62.0	fTotalStrokes	 13
U+5D62.0	fCangjie	UOIR
U+5D62.0	fFourCorner	28767
U+5D62.0	fHakka	 cong1
U+5D62.0	fCantonese	cong1
U+5D62.0	fMandarin	cang1
U+5D63.0	fUTF8	嵣
U+5D63.0	fCNS11643	2-4254
U+5D63.0	fBig5	DD4E
U+5D63.0	fGB	3-394C
U+5D63.0	fR/S	 46.10
U+5D63.0	fTotalStrokes	 13
U+5D63.0	fCangjie	UILR
U+5D63.0	fFourCorner	20767
U+5D63.0	fCantonese	dong6; tong4
U+5D63.0	fMandarin	dang1; tang2
U+5D64.0	fUTF8	嵤
U+5D64.0	fCNS11643	3-407C
U+5D64.0	fGB	5-3A61
U+5D64.0	fR/S	  46.10
U+5D64.0	fTotalStrokes	 13
U+5D64.0	fCangjie	FFBU
U+5D64.0	fFourCorner	99770
U+5D64.0	fCantonese	wing4
U+5D64.0	fMandarin	hong2; rong2; ying2
U+5D65.0	fUTF8	嵥
U+5D65.0	fCNS11643	2-4256
U+5D65.0	fBig5	DD50
U+5D65.0	fGB	5-3B5F
U+5D65.0	fR/S	 46.10
U+5D65.0	fTotalStrokes	 13
U+5D65.0	fCangjie	UNQD
U+5D65.0	fFourCorner	25794
U+5D65.0	fCantonese	git6
U+5D65.0	fMandarin	jie2
U+5D66.0	fUTF8	嵦
U+5D66.0	fCNS11643	3-407A
U+5D66.0	fGB	5-3A79
U+5D66.0	fR/S	  46.10
U+5D66.0	fTotalStrokes	 13
U+5D66.0	fCangjie	UUMT
U+5D66.0	fFourCorner	22710
U+5D66.0	fCantonese	hoi2
U+5D66.0	fMandarin	ai2
U+5D66.0	fEnglish	a raised site; pleasant
U+5D67.0	fUTF8	嵧
U+5D67.0	fCNS11643	2-425B
U+5D67.0	fBig5	DD55
U+5D67.0	fGB	5-3B60
U+5D67.0	fR/S	 46.10
U+5D67.0	fTotalStrokes	 13
U+5D67.0	fCangjie	UHHW
U+5D67.0	fFourCorner	27762
U+5D67.0	fCantonese	lau4
U+5D67.0	fMandarin	liu2
U+5D68.0	fUTF8	嵨
U+5D68.0	fCNS11643	2-425A
U+5D68.0	fBig5	DD54
U+5D68.0	fGB	3-384B
U+5D68.0	fR/S	 46.10
U+5D68.0	fTotalStrokes	 13
U+5D68.0	fCangjie	UHRF
U+5D68.0	fFourCorner	27727
U+5D68.0	fCantonese	wu2
U+5D68.0	fMandarin	wu3
U+5D69.0	fUTF8	嵩
U+5D69.0	fCNS11643	1-6369
U+5D69.0	fBig5	B743
U+5D69.0	fGB	0-6154
U+5D69.0	fR/S	 46.10
U+5D69.0	fTotalStrokes	 13
U+5D69.0	fCangjie	UYRB
U+5D69.0	fFourCorner	22227
U+5D69.0	fHakka	  sung1;  siung1; sung2; ciung2; cung1
U+5D69.0	fCantonese	sung1
U+5D69.0	fMandarin	song1
U+5D69.0	fEnglish	[1] high; lofty [2] Mt. Sung, the highest and central peak of the five Sacred mountains situated in Honan
U+5D6A.0	fUTF8	嵪
U+5D6A.0	fCNS11643	4-3C75
U+5D6A.0	fGB	5-3B61
U+5D6A.0	fR/S	 46.10
U+5D6A.0	fTotalStrokes	 13
U+5D6A.0	fCangjie	UYRB
U+5D6A.0	fFourCorner	20720
U+5D6A.0	fHakka	 kiau1
U+5D6A.0	fCantonese	hiu1
U+5D6A.0	fMandarin	qiao1
U+5D6B.0	fUTF8	嵫
U+5D6B.0	fCNS11643	2-3B3F
U+5D6B.0	fBig5	D8DB
U+5D6B.0	fGB	0-6151
U+5D6B.0	fR/S	 46.10
U+5D6B.0	fTotalStrokes	 13
U+5D6B.0	fCangjie	UTVI
U+5D6B.0	fFourCorner	28732
U+5D6B.0	fHakka	 zih1; zu1; zii1
U+5D6B.0	fCantonese	zi1
U+5D6B.0	fMandarin	zi1
U+5D6B.0	fEnglish	a hill in Shantung
U+5D6C.0	fUTF8	嵬
U+5D6C.0	fCNS11643	2-4258
U+5D6C.0	fBig5	DD52
U+5D6C.0	fGB	0-614D
U+5D6C.0	fR/S	 46.10
U+5D6C.0	fTotalStrokes	 13
U+5D6C.0	fCangjie	UHUI
U+5D6C.0	fFourCorner	22213
U+5D6C.0	fHakka	 ngui2
U+5D6C.0	fCantonese	ngai4; ai4 (also)
U+5D6C.0	fMandarin	wei2; wei1; wei3
U+5D6C.0	fEnglish	high and uneven; rugged; rocky (said of a mountain)
U+5D6D.0	fUTF8	嵭
U+5D6D.0	fCNS11643	3-4079
U+5D6D.0	fGB	E-2853
U+5D6D.0	fR/S	  46.10
U+5D6D.0	fTotalStrokes	 13
U+5D6D.0	fCangjie	UYBS
U+5D6D.0	fFourCorner	20720
U+5D6D.0	fCantonese	bang1
U+5D6D.0	fMandarin	beng1
U+5D6E.0	fUTF8	嵮
U+5D6E.0	fCNS11643	3-407D
U+5D6E.0	fGB	5-3B57
U+5D6E.0	fR/S	  46.10
U+5D6E.0	fTotalStrokes	 13
U+5D6E.0	fCangjie	UJBC
U+5D6E.0	fFourCorner	24780
U+5D6E.0	fCantonese	din1
U+5D6E.0	fMandarin	dian1
U+5D6F.0	fUTF8	嵯
U+5D6F.0	fCNS11643	1-636A
U+5D6F.0	fBig5	B744
U+5D6F.0	fGB	0-614F
U+5D6F.0	fR/S	 46.10
U+5D6F.0	fTotalStrokes	 13
U+5D6F.0	fCangjie	UTQM
U+5D6F.0	fFourCorner	28711
U+5D6F.0	fHakka	 co2; co1; ca1; co5
U+5D6F.0	fCantonese	co1; ci1; co4
U+5D6F.0	fMandarin	cuo2; ci1; cuo1
U+5D6F.0	fEnglish	high and steep; precipitous (said of mountains)
U+5D70.0	fUTF8	嵰
U+5D70.0	fCNS11643	4-3C74
U+5D70.0	fGB	3-394D
U+5D70.0	fR/S	 46.10
U+5D70.0	fTotalStrokes	 13
U+5D70.0	fCangjie	UTXC
U+5D70.0	fFourCorner	28730
U+5D70.0	fCantonese	him2
U+5D70.0	fMandarin	qian3
U+5D71.0	fUTF8	嵱
U+5D71.0	fCNS11643	2-4253
U+5D71.0	fBig5	DD4D
U+5D71.0	fGB	3-394E
U+5D71.0	fR/S	 46.10
U+5D71.0	fTotalStrokes	 13
U+5D71.0	fCangjie	UJCR
U+5D71.0	fFourCorner	23768
U+5D71.0	fCantonese	jung4 (also); jung2
U+5D71.0	fMandarin	yong3; yong2
U+5D72.0	fUTF8	嵲
U+5D72.0	fCNS11643	2-4257
U+5D72.0	fBig5	DD51
U+5D72.0	fGB	3-394A
U+5D72.0	fR/S	 46.10
U+5D72.0	fTotalStrokes	 13
U+5D72.0	fCangjie	UHUD
U+5D72.0	fFourCorner	26794
U+5D72.0	fCantonese	jit6
U+5D72.0	fMandarin	nie4
U+5D73.0	fUTF8	嵳
U+5D73.0	fCNS11643	4-3C7C
U+5D73.0	fGB	E-2854
U+5D73.0	fR/S	 46.10
U+5D73.0	fTotalStrokes	 13
U+5D73.0	fCangjie	XUTQM
U+5D73.0	fFourCorner	22100
U+5D73.0	fHakka	 co2
U+5D73.0	fCantonese	co1
U+5D73.0	fMandarin	cuo2; ci1; cuo1
U+5D73.0	fEnglish	same as &#x5D6F;
U+5D74.0	fUTF8	嵴
U+5D74.0	fCNS11643	4-3C78
U+5D74.0	fGB	0-6155
U+5D74.0	fR/S	 46.10
U+5D74.0	fTotalStrokes	 13
U+5D74.0	fCangjie	UFCB
U+5D74.0	fFourCorner	22720
U+5D74.0	fCantonese	zek3
U+5D74.0	fMandarin	ji2
U+5D74.0	fEnglish	mountain ridge
U+5D75.0	fUTF8	嵵
U+5D75.0	fCNS11643	3-646E
U+5D75.0	fGB	E-2855
U+5D75.0	fR/S	 46.10
U+5D75.0	fTotalStrokes	 13
U+5D75.0	fCangjie	UAGI
U+5D75.0	fFourCorner	2264
U+5D75.0	fCantonese	zi6
U+5D76.0	fUTF8	嵶
U+5D76.0	fGB	E-2856
U+5D76.0	fR/S	 46.10
U+5D76.0	fTotalStrokes	 13
U+5D76.0	fCangjie	UNMM
U+5D76.0	fCantonese	joek6
U+5D76.0	fEnglish	 low part of a mountain
U+5D77.0	fUTF8	嵷
U+5D77.0	fCNS11643	2-496E
U+5D77.0	fBig5	E1A9
U+5D77.0	fGB	3-3848
U+5D77.0	fR/S	 46.11
U+5D77.0	fTotalStrokes	 14
U+5D77.0	fCangjie	UHOO
U+5D77.0	fFourCorner	28781
U+5D77.0	fCantonese	sung2
U+5D77.0	fMandarin	song3; zong1; zong3
U+5D78.0	fUTF8	嵸
U+5D78.0	fCNS11643	3-465C
U+5D78.0	fGB	3-3849
U+5D78.0	fR/S	  46.11
U+5D78.0	fTotalStrokes	 14
U+5D78.0	fCangjie	UHOO
U+5D78.0	fFourCorner	22280
U+5D78.0	fCantonese	sung2
U+5D78.0	fMandarin	song3; zong1; zong3
U+5D78.0	fEnglish	same as &#x5D77;
U+5D79.0	fUTF8	嵹
U+5D79.0	fCNS11643	2-4975
U+5D79.0	fBig5	E1B0
U+5D79.0	fGB	3-3964
U+5D79.0	fR/S	 46.11
U+5D79.0	fTotalStrokes	 14
U+5D79.0	fCangjie	UNII
U+5D79.0	fFourCorner	22236
U+5D79.0	fCantonese	koeng4
U+5D79.0	fMandarin	jiang4; qiang2
U+5D7A.0	fUTF8	嵺
U+5D7A.0	fCNS11643	2-496C
U+5D7A.0	fBig5	E1A7
U+5D7A.0	fGB	3-3958
U+5D7A.0	fR/S	 46.11
U+5D7A.0	fTotalStrokes	 14
U+5D7A.0	fCangjie	USMH
U+5D7A.0	fFourCorner	27722
U+5D7A.0	fHakka	 liau2
U+5D7A.0	fCantonese	liu4
U+5D7A.0	fMandarin	liao2
U+5D7B.0	fUTF8	嵻
U+5D7B.0	fCNS11643	3-4654
U+5D7B.0	fGB	E-2857
U+5D7B.0	fR/S	  46.11
U+5D7B.0	fTotalStrokes	 14
U+5D7B.0	fCangjie	UILE
U+5D7B.0	fFourCorner	20730
U+5D7B.0	fCantonese	hong1
U+5D7B.0	fMandarin	kang1
U+5D7C.0	fUTF8	嵼
U+5D7C.0	fCNS11643	2-4973
U+5D7C.0	fBig5	E1AE
U+5D7C.0	fGB	3-386D
U+5D7C.0	fR/S	 46.11
U+5D7C.0	fTotalStrokes	 14
U+5D7C.0	fCangjie	UYHM
U+5D7C.0	fFourCorner	20715
U+5D7C.0	fHakka	  san3; can3
U+5D7C.0	fCantonese	caan2
U+5D7C.0	fMandarin	chan3
U+5D7C.0	fEnglish	a winding road among hills
U+5D7D.0	fUTF8	嵽
U+5D7D.0	fCNS11643	2-496A
U+5D7D.0	fBig5	E1A5
U+5D7D.0	fGB	3-3934
U+5D7D.0	fR/S	 46.11
U+5D7D.0	fTotalStrokes	 14
U+5D7D.0	fCangjie	UKPB
U+5D7D.0	fFourCorner	24727
U+5D7D.0	fCantonese	dit6; dai6; dai3
U+5D7D.0	fMandarin	di4; die2; di1
U+5D7E.0	fUTF8	嵾
U+5D7E.0	fCNS11643	2-4972
U+5D7E.0	fBig5	E1AD
U+5D7E.0	fGB	E-2858
U+5D7E.0	fR/S	 46.11
U+5D7E.0	fTotalStrokes	 14
U+5D7E.0	fCangjie	UIIH
U+5D7E.0	fFourCorner	22202
U+5D7E.0	fHakka	  cam1
U+5D7E.0	fCantonese	cam1; caam1 (also)
U+5D7E.0	fMandarin	liao2; cen1
U+5D7E.0	fEnglish	same as &#x5D7A;
U+5D7F.0	fUTF8	嵿
U+5D7F.0	fCNS11643	2-4976
U+5D7F.0	fBig5	E1B1
U+5D7F.0	fGB	3-3927
U+5D7F.0	fR/S	 46.11
U+5D7F.0	fTotalStrokes	 14
U+5D7F.0	fCangjie	UMNC
U+5D7F.0	fFourCorner	22286
U+5D7F.0	fCantonese	ding2
U+5D7F.0	fMandarin	ding3
U+5D80.0	fUTF8	嶀
U+5D80.0	fCNS11643	2-4969
U+5D80.0	fBig5	E1A4
U+5D80.0	fGB	E-2859
U+5D80.0	fR/S	 46.11
U+5D80.0	fTotalStrokes	 14
U+5D80.0	fCangjie	UMBS
U+5D80.0	fFourCorner	21771
U+5D80.0	fCantonese	tou1
U+5D80.0	fMandarin	tu1; die2
U+5D81.0	fUTF8	嶁
U+5D81.0	fCNS11643	2-496D
U+5D81.0	fBig5	E1A8
U+5D81.0	fGB	1-6150
U+5D81.0	fR/S	 46.11
U+5D81.0	fTotalStrokes	 14
U+5D81.0	fCangjie	ULLV
U+5D81.0	fFourCorner	25744
U+5D81.0	fHakka	  leu3; li3
U+5D81.0	fCantonese	lau5
U+5D81.0	fMandarin	lou3; lou4; l3
U+5D81.0	fEnglish	Goulou mountain peak in Hunan
U+5D82.0	fUTF8	嶂
U+5D82.0	fCNS11643	2-4968
U+5D82.0	fBig5	E1A3
U+5D82.0	fGB	0-6156
U+5D82.0	fR/S	 46.11
U+5D82.0	fTotalStrokes	 14
U+5D82.0	fCangjie	UYTJ
U+5D82.0	fFourCorner	20746
U+5D82.0	fHakka	  zhong5; zong5
U+5D82.0	fCantonese	zoeng3
U+5D82.0	fMandarin	zhang4
U+5D82.0	fEnglish	 precipitous mountain
U+5D83.0	fUTF8	嶃
U+5D83.0	fCNS11643	3-4656
U+5D83.0	fGB	E-285A
U+5D83.0	fR/S	 46.11
U+5D83.0	fTotalStrokes	 14
U+5D83.0	fCangjie	UJJL
U+5D83.0	fFourCorner	22720
U+5D83.0	fHakka	 cam3; cam2
U+5D83.0	fCantonese	zaam2; caam4
U+5D83.0	fMandarin	zhan3
U+5D83.0	fEnglish	same as &#x5D83;
U+5D84.0	fUTF8	嶄
U+5D84.0	fCNS11643	1-6859
U+5D84.0	fBig5	B9F1
U+5D84.0	fGB	1-5538
U+5D84.0	fR/S	 46.11
U+5D84.0	fTotalStrokes	 14
U+5D84.0	fCangjie	UJJL
U+5D84.0	fFourCorner	22521
U+5D84.0	fHakka	 cam3; zam5; zam3
U+5D84.0	fCantonese	zaam3 (also); zaam2; caam4; caam5
U+5D84.0	fMandarin	zhan3; chan2
U+5D84.0	fEnglish	[1] high and steep (said of a mountain) [2] towering (above) [3] new; novel
U+5D85.0	fUTF8	嶅
U+5D85.0	fCNS11643	4-432C
U+5D85.0	fGB	8-7A7D
U+5D85.0	fR/S	 46.11
U+5D85.0	fTotalStrokes	 14
U+5D85.0	fCangjie	GKU
U+5D85.0	fFourCorner	48770
U+5D85.0	fCantonese	ngou4; ou4 (also)
U+5D85.0	fMandarin	ao2
U+5D86.0	fUTF8	嶆
U+5D86.0	fCNS11643	2-496B
U+5D86.0	fBig5	E1A6
U+5D86.0	fGB	3-3951
U+5D86.0	fR/S	 46.11
U+5D86.0	fTotalStrokes	 14
U+5D86.0	fCangjie	UTWA
U+5D86.0	fFourCorner	24767
U+5D86.0	fCantonese	cou4
U+5D86.0	fMandarin	cao2
U+5D87.0	fUTF8	嶇
U+5D87.0	fCNS11643	1-685A
U+5D87.0	fBig5	B9F2
U+5D87.0	fGB	1-612B
U+5D87.0	fR/S	 46.11
U+5D87.0	fTotalStrokes	 14
U+5D87.0	fCangjie	USRR
U+5D87.0	fFourCorner	21716
U+5D87.0	fHakka	 ki1
U+5D87.0	fCantonese	keoi1
U+5D87.0	fMandarin	qu1
U+5D87.0	fEnglish	rugged; rough; uneven; irregular
U+5D88.0	fUTF8	嶈
U+5D88.0	fCNS11643	2-4971
U+5D88.0	fBig5	E1AC
U+5D88.0	fGB	3-3940
U+5D88.0	fR/S	 46.11
U+5D88.0	fTotalStrokes	 14
U+5D88.0	fCangjie	UVMI
U+5D88.0	fFourCorner	22243
U+5D88.0	fCantonese	coeng1
U+5D88.0	fMandarin	qiang1
U+5D89.0	fUTF8	嶉
U+5D89.0	fCNS11643	2-4970
U+5D89.0	fBig5	E1AB
U+5D89.0	fGB	3-3954
U+5D89.0	fR/S	 46.11
U+5D89.0	fTotalStrokes	 14
U+5D89.0	fCangjie	UROG
U+5D89.0	fFourCorner	22614
U+5D89.0	fCantonese	ceoi1; zeoi1; zeoi2
U+5D89.0	fMandarin	zui1
U+5D8A.0	fUTF8	嶊
U+5D8A.0	fCNS11643	2-496F
U+5D8A.0	fBig5	E1AA
U+5D8A.0	fGB	3-3953
U+5D8A.0	fR/S	 46.11
U+5D8A.0	fTotalStrokes	 14
U+5D8A.0	fCangjie	UQOG
U+5D8A.0	fFourCorner	22514
U+5D8A.0	fCantonese	zeoi2
U+5D8A.0	fMandarin	zui3
U+5D8B.0	fUTF8	嶋
U+5D8B.0	fCNS11643	3-4659
U+5D8B.0	fGB	5-3A68
U+5D8B.0	fR/S	  46.11
U+5D8B.0	fTotalStrokes	 14
U+5D8B.0	fCangjie	UHAF
U+5D8B.0	fFourCorner	27720
U+5D8B.0	fHakka	 dau3; do3
U+5D8B.0	fCantonese	dou2
U+5D8B.0	fMandarin	dao3
U+5D8B.0	fEnglish	same as &#x3800;
U+5D8C.0	fUTF8	嶌
U+5D8C.0	fCNS11643	3-465A
U+5D8C.0	fGB	E-285B
U+5D8C.0	fR/S	 46.11
U+5D8C.0	fTotalStrokes	 14
U+5D8C.0	fCangjie	UHAF
U+5D8C.0	fFourCorner	22320
U+5D8C.0	fCantonese	dou2
U+5D8C.0	fMandarin	dao3
U+5D8C.0	fEnglish	island
U+5D8D.0	fUTF8	嶍
U+5D8D.0	fCNS11643	2-4974
U+5D8D.0	fBig5	E1AF
U+5D8D.0	fGB	3-3957
U+5D8D.0	fR/S	 46.11
U+5D8D.0	fTotalStrokes	 14
U+5D8D.0	fCangjie	USMA
U+5D8D.0	fFourCorner	27762
U+5D8D.0	fHakka	 sip8
U+5D8D.0	fCantonese	zaap6
U+5D8D.0	fMandarin	xi2
U+5D8E.0	fUTF8	嶎
U+5D8E.0	fCNS11643	3-646F
U+5D8E.0	fGB	E-285C
U+5D8E.0	fR/S	 46.11
U+5D8E.0	fTotalStrokes	 14
U+5D8E.0	fCangjie	USFI
U+5D8E.0	fFourCorner	2224
U+5D8E.0	fCantonese	wai3
U+5D8E.0	fMandarin	yu4
U+5D8E.0	fEnglish	luxuriant; elegant
U+5D8F.0	fUTF8	嶏
U+5D8F.0	fCNS11643	3-4B65
U+5D8F.0	fGB	5-3B6B
U+5D8F.0	fR/S	  46.12
U+5D8F.0	fTotalStrokes	 15
U+5D8F.0	fCangjie	UMMU
U+5D8F.0	fFourCorner	22210
U+5D8F.0	fCantonese	pui3
U+5D8F.0	fMandarin	bo2
U+5D90.0	fUTF8	嶐
U+5D90.0	fCNS11643	3-4B66
U+5D90.0	fGB	E-285D
U+5D90.0	fR/S	  46.12
U+5D90.0	fTotalStrokes	 15
U+5D90.0	fCangjie	UNLM
U+5D90.0	fFourCorner	22210
U+5D90.0	fCantonese	lung4
U+5D90.0	fMandarin	long2
U+5D91.0	fUTF8	嶑
U+5D91.0	fCNS11643	3-4B63
U+5D91.0	fGB	E-285E
U+5D91.0	fR/S	 46.12
U+5D91.0	fTotalStrokes	 15
U+5D91.0	fCangjie	UNAO
U+5D91.0	fFourCorner	27730
U+5D91.0	fCantonese	zoeng6
U+5D91.0	fMandarin	xiang3
U+5D92.0	fUTF8	嶒
U+5D92.0	fCNS11643	2-502E
U+5D92.0	fBig5	E565
U+5D92.0	fGB	3-3963
U+5D92.0	fR/S	 46.12
U+5D92.0	fTotalStrokes	 15
U+5D92.0	fCangjie	UCWA
U+5D92.0	fFourCorner	28766
U+5D92.0	fHakka	  cen2;  zen1
U+5D92.0	fCantonese	cang4
U+5D92.0	fMandarin	ceng2; cheng2
U+5D92.0	fEnglish	lofty; steep
U+5D93.0	fUTF8	嶓
U+5D93.0	fCNS11643	2-5030
U+5D93.0	fBig5	E567
U+5D93.0	fGB	3-3960
U+5D93.0	fR/S	 46.12
U+5D93.0	fTotalStrokes	 15
U+5D93.0	fCangjie	UHDW
U+5D93.0	fFourCorner	22769
U+5D93.0	fHakka	 bo1; bo5
U+5D93.0	fCantonese	bo1
U+5D93.0	fMandarin	bo1; bo4; guo1; zeng1
U+5D93.0	fEnglish	Mt. Boshan (in Shenxi)
U+5D94.0	fUTF8	嶔
U+5D94.0	fCNS11643	1-6C55
U+5D94.0	fBig5	BC6B
U+5D94.0	fGB	3-393C
U+5D94.0	fR/S	 46.12
U+5D94.0	fTotalStrokes	 15
U+5D94.0	fCangjie	UCNO
U+5D94.0	fFourCorner	22182
U+5D94.0	fHakka	  kim1
U+5D94.0	fCantonese	jam1
U+5D94.0	fMandarin	qin1; qian1
U+5D94.0	fEnglish	lofty (mountain)
U+5D95.0	fUTF8	嶕
U+5D95.0	fCNS11643	2-5031
U+5D95.0	fBig5	E568
U+5D95.0	fGB	3-395E
U+5D95.0	fR/S	 46.12
U+5D95.0	fTotalStrokes	 15
U+5D95.0	fCangjie	UOGF
U+5D95.0	fFourCorner	20731
U+5D95.0	fHakka	 ziau1; zeu1
U+5D95.0	fCantonese	ciu4; ziu1
U+5D95.0	fMandarin	jiao1; qiao2
U+5D95.0	fEnglish	grieved; depressed; melancholy
U+5D96.0	fUTF8	嶖
U+5D96.0	fCNS11643	3-4B6B
U+5D96.0	fGB	5-3B71
U+5D96.0	fR/S	  46.12
U+5D96.0	fTotalStrokes	 15
U+5D96.0	fCangjie	UORM
U+5D96.0	fFourCorner	28727
U+5D96.0	fHakka	 jam1; ram1; jam3
U+5D96.0	fCantonese	jim1
U+5D96.0	fMandarin	yan1; yan3
U+5D96.0	fEnglish	same as &#x5D26;
U+5D97.0	fUTF8	嶗
U+5D97.0	fCNS11643	2-502C
U+5D97.0	fBig5	E563
U+5D97.0	fGB	1-6140
U+5D97.0	fR/S	 46.12
U+5D97.0	fTotalStrokes	 15
U+5D97.0	fCangjie	UFFS
U+5D97.0	fFourCorner	29727
U+5D97.0	fCantonese	lou4; dang3
U+5D97.0	fMandarin	lao2; lao1
U+5D97.0	fEnglish	Laoshan; mountain in Shandong
U+5D98.0	fUTF8	嶘
U+5D98.0	fCNS11643	3-4B67
U+5D98.0	fGB	5-3B47
U+5D98.0	fR/S	 46.12
U+5D98.0	fTotalStrokes	 15
U+5D98.0	fCangjie	UDII
U+5D98.0	fFourCorner	22950
U+5D98.0	fCantonese	zaan6
U+5D98.0	fMandarin	zhan4
U+5D99.0	fUTF8	嶙
U+5D99.0	fCNS11643	2-502B
U+5D99.0	fBig5	E562
U+5D99.0	fGB	0-6157
U+5D99.0	fR/S	 46.12
U+5D99.0	fTotalStrokes	 15
U+5D99.0	fCangjie	UFDQ
U+5D99.0	fFourCorner	29759
U+5D99.0	fHakka	  lin2
U+5D99.0	fCantonese	leon4
U+5D99.0	fMandarin	lin2; lin3
U+5D99.0	fEnglish	rugged (said of mountains)
U+5D9A.0	fUTF8	嶚
U+5D9A.0	fCNS11643	2-5035
U+5D9A.0	fBig5	E56C
U+5D9A.0	fGB	5-3B6C
U+5D9A.0	fR/S	 46.12
U+5D9A.0	fTotalStrokes	 15
U+5D9A.0	fCangjie	UKCF
U+5D9A.0	fFourCorner	22906
U+5D9A.0	fCantonese	liu4
U+5D9A.0	fMandarin	liao2
U+5D9B.0	fUTF8	嶛
U+5D9B.0	fCNS11643	3-4B69
U+5D9B.0	fGB	3-395B
U+5D9B.0	fR/S	 46.12
U+5D9B.0	fTotalStrokes	 15
U+5D9B.0	fCangjie	UKCF
U+5D9B.0	fFourCorner	24790
U+5D9B.0	fCantonese	liu4
U+5D9B.0	fMandarin	liao2
U+5D9B.0	fEnglish	same as &#x5D9A;
U+5D9C.0	fUTF8	嶜
U+5D9C.0	fCNS11643	2-5033
U+5D9C.0	fBig5	E56A
U+5D9C.0	fGB	3-395C
U+5D9C.0	fR/S	 46.12
U+5D9C.0	fTotalStrokes	 15
U+5D9C.0	fCangjie	UMUA
U+5D9C.0	fFourCorner	22601
U+5D9C.0	fCantonese	zam1
U+5D9C.0	fMandarin	jin1; jin4; qian2
U+5D9D.0	fUTF8	嶝
U+5D9D.0	fCNS11643	1-6C54
U+5D9D.0	fBig5	BC6A
U+5D9D.0	fGB	0-6158
U+5D9D.0	fR/S	 46.12
U+5D9D.0	fTotalStrokes	 15
U+5D9D.0	fCangjie	UNOT
U+5D9D.0	fFourCorner	22718
U+5D9D.0	fHakka	  den5;  den1
U+5D9D.0	fCantonese	dang3
U+5D9D.0	fMandarin	deng4
U+5D9D.0	fEnglish	 path leading up a mountain
U+5D9E.0	fUTF8	嶞
U+5D9E.0	fCNS11643	2-5036
U+5D9E.0	fBig5	E56D
U+5D9E.0	fGB	3-3956
U+5D9E.0	fR/S	 46.12
U+5D9E.0	fTotalStrokes	 15
U+5D9E.0	fCangjie	NBU
U+5D9E.0	fFourCorner	74771
U+5D9E.0	fHakka	 to5; to6
U+5D9E.0	fCantonese	do6
U+5D9E.0	fMandarin	duo4
U+5D9E.0	fEnglish	ridge
U+5D9F.0	fUTF8	嶟
U+5D9F.0	fCNS11643	2-502D
U+5D9F.0	fBig5	E564
U+5D9F.0	fGB	3-3962
U+5D9F.0	fR/S	 46.12
U+5D9F.0	fTotalStrokes	 15
U+5D9F.0	fCangjie	UTWI
U+5D9F.0	fFourCorner	28746
U+5D9F.0	fHakka	 zun1
U+5D9F.0	fCantonese	zeon1
U+5D9F.0	fMandarin	zun1; cun1
U+5D9F.0	fEnglish	lofty (mountain peak)
U+5DA0.0	fUTF8	嶠
U+5DA0.0	fCNS11643	2-5032
U+5DA0.0	fBig5	E569
U+5DA0.0	fGB	1-613D
U+5DA0.0	fR/S	 46.12
U+5DA0.0	fTotalStrokes	 15
U+5DA0.0	fCangjie	UHKB
U+5DA0.0	fFourCorner	22727
U+5DA0.0	fHakka	 kiau5; kiau2
U+5DA0.0	fCantonese	giu6; kiu4; giu2
U+5DA0.0	fMandarin	jiao4; qiao2
U+5DA0.0	fEnglish	 high pointed mountain
U+5DA1.0	fUTF8	嶡
U+5DA1.0	fCNS11643	2-5034
U+5DA1.0	fBig5	E56B
U+5DA1.0	fGB	3-395A
U+5DA1.0	fR/S	 46.12
U+5DA1.0	fTotalStrokes	 15
U+5DA1.0	fCangjie	UMTO
U+5DA1.0	fFourCorner	22282
U+5DA1.0	fHakka	 kui1; kiet7; ket7
U+5DA1.0	fCantonese	gwai3; kyut3
U+5DA1.0	fMandarin	gui4; gui1; jue1; jue2
U+5DA1.0	fEnglish	table
U+5DA2.0	fUTF8	嶢
U+5DA2.0	fCNS11643	2-502F
U+5DA2.0	fBig5	E566
U+5DA2.0	fGB	3-3865
U+5DA2.0	fR/S	 46.12
U+5DA2.0	fTotalStrokes	 15
U+5DA2.0	fCangjie	UGGU
U+5DA2.0	fFourCorner	24711
U+5DA2.0	fHakka	 ngiau2; gniau3
U+5DA2.0	fCantonese	jiu4
U+5DA2.0	fMandarin	yao2
U+5DA2.0	fEnglish	high; tall (said of mountains)
U+5DA3.0	fUTF8	嶣
U+5DA3.0	fCNS11643	3-4B6A
U+5DA3.0	fGB	5-3B6F
U+5DA3.0	fR/S	 46.12
U+5DA3.0	fTotalStrokes	 15
U+5DA3.0	fCangjie	UOGF
U+5DA3.0	fFourCorner	22331
U+5DA3.0	fHakka	 ziau1; zeu1
U+5DA3.0	fCantonese	ciu4; ziu1
U+5DA3.0	fMandarin	jiao1; qiao2
U+5DA3.0	fEnglish	same as &#x5D95;
U+5DA4.0	fUTF8	嶤
U+5DA4.0	fCNS11643	3-4B68
U+5DA4.0	fGB	5-3A77
U+5DA4.0	fR/S	 46.12
U+5DA4.0	fTotalStrokes	 15
U+5DA4.0	fCangjie	UGGU
U+5DA4.0	fFourCorner	22210
U+5DA4.0	fHakka	 ngiau2; gniau3
U+5DA4.0	fCantonese	jiu4
U+5DA4.0	fMandarin	yao2
U+5DA4.0	fEnglish	same as &#x5DA2;
U+5DA5.0	fUTF8	嶥
U+5DA5.0	fCNS11643	3-6470
U+5DA5.0	fGB	E-285F
U+5DA5.0	fR/S	 46.12
U+5DA5.0	fTotalStrokes	 15
U+5DA5.0	fCangjie	XUMTO
U+5DA5.0	fFourCorner	2178
U+5DA5.0	fCantonese	gwai3
U+5DA5.0	fMandarin	jue2
U+5DA6.0	fUTF8	嶦
U+5DA6.0	fCNS11643	3-5058
U+5DA6.0	fGB	3-396B
U+5DA6.0	fR/S	 46.13
U+5DA6.0	fTotalStrokes	 16
U+5DA6.0	fCangjie	UNCR
U+5DA6.0	fFourCorner	27760
U+5DA6.0	fCantonese	zim1
U+5DA6.0	fMandarin	zhan1
U+5DA7.0	fUTF8	嶧
U+5DA7.0	fCNS11643	2-566B
U+5DA7.0	fBig5	E961
U+5DA7.0	fGB	1-613B
U+5DA7.0	fR/S	 46.13
U+5DA7.0	fTotalStrokes	 16
U+5DA7.0	fCangjie	UWLJ
U+5DA7.0	fFourCorner	26741
U+5DA7.0	fHakka	 jit8; rit8
U+5DA7.0	fCantonese	jik6
U+5DA7.0	fMandarin	yi4
U+5DA7.0	fEnglish	range of peaks; peaks rising one upon another
U+5DA8.0	fUTF8	嶨
U+5DA8.0	fCNS11643	2-5670
U+5DA8.0	fBig5	E966
U+5DA8.0	fGB	3-385B
U+5DA8.0	fR/S	 46.13
U+5DA8.0	fTotalStrokes	 16
U+5DA8.0	fCangjie	HBU
U+5DA8.0	fFourCorner	77772
U+5DA8.0	fCantonese	hok6; bok3
U+5DA8.0	fMandarin	xue2; xue1; xiao2
U+5DA8.0	fEnglish	big rocky mountain
U+5DA9.0	fUTF8	嶩
U+5DA9.0	fCNS11643	2-566A
U+5DA9.0	fBig5	E960
U+5DA9.0	fGB	5-3B26
U+5DA9.0	fR/S	 46.13
U+5DA9.0	fTotalStrokes	 16
U+5DA9.0	fCangjie	UTWV
U+5DA9.0	fFourCorner	25732
U+5DA9.0	fHakka	 nau2; no2
U+5DA9.0	fCantonese	naau4; laau4 (also); nau4; nou4; lau4
U+5DA9.0	fMandarin	nao2; nao1
U+5DA9.0	fEnglish	same as &#x5CF1;
U+5DAA.0	fUTF8	嶪
U+5DAA.0	fCNS11643	2-566F
U+5DAA.0	fBig5	E965
U+5DAA.0	fGB	3-3854
U+5DAA.0	fR/S	 46.13
U+5DAA.0	fTotalStrokes	 16
U+5DAA.0	fCangjie	UTCD
U+5DAA.0	fFourCorner	22904
U+5DAA.0	fCantonese	jip6
U+5DAA.0	fMandarin	ye4
U+5DAB.0	fUTF8	嶫
U+5DAB.0	fCNS11643	3-505B
U+5DAB.0	fGB	E-2860
U+5DAB.0	fR/S	 46.13
U+5DAB.0	fTotalStrokes	 16
U+5DAB.0	fCangjie	UTCD
U+5DAB.0	fFourCorner	23790
U+5DAB.0	fCantonese	jip6
U+5DAB.0	fMandarin	ye4
U+5DAB.0	fEnglish	same as &#x5DAA;
U+5DAC.0	fUTF8	嶬
U+5DAC.0	fCNS11643	2-5668
U+5DAC.0	fBig5	E95E
U+5DAC.0	fGB	3-3842
U+5DAC.0	fR/S	 46.13
U+5DAC.0	fTotalStrokes	 16
U+5DAC.0	fCangjie	UTGI
U+5DAC.0	fFourCorner	28753
U+5DAC.0	fHakka	  ngi2
U+5DAC.0	fCantonese	ngai4; ai4 (also); ji4; ngai5
U+5DAC.0	fMandarin	yi2; yi1; yi3
U+5DAC.0	fEnglish	steep and irregular mountain peeks
U+5DAD.0	fUTF8	嶭
U+5DAD.0	fCNS11643	2-5672
U+5DAD.0	fBig5	E968
U+5DAD.0	fGB	3-396A
U+5DAD.0	fR/S	 46.13
U+5DAD.0	fTotalStrokes	 16
U+5DAD.0	fCangjie	UHRJ
U+5DAD.0	fFourCorner	22741
U+5DAD.0	fHakka	 ngiet7; nget7
U+5DAD.0	fCantonese	jit6; sit3; jip6
U+5DAD.0	fMandarin	e4; nie4
U+5DAD.0	fEnglish	elevated; lofty
U+5DAE.0	fUTF8	嶮
U+5DAE.0	fCNS11643	2-566E
U+5DAE.0	fBig5	E964
U+5DAE.0	fGB	3-3877
U+5DAE.0	fR/S	 46.13
U+5DAE.0	fTotalStrokes	 16
U+5DAE.0	fCangjie	UOMO
U+5DAE.0	fFourCorner	28786
U+5DAE.0	fCantonese	him2
U+5DAE.0	fMandarin	xian3; yan3
U+5DAE.0	fEnglish	high; steep; precipitous; lofty
U+5DAF.0	fUTF8	嶯
U+5DAF.0	fCNS11643	2-5673
U+5DAF.0	fBig5	E969
U+5DAF.0	fGB	E-2861
U+5DAF.0	fR/S	 46.13
U+5DAF.0	fTotalStrokes	 16
U+5DAF.0	fCangjie	URJI
U+5DAF.0	fFourCorner	22153
U+5DAF.0	fCantonese	cap1
U+5DAF.0	fMandarin	ji2; zha3
U+5DB0.0	fUTF8	嶰
U+5DB0.0	fCNS11643	2-566D
U+5DB0.0	fBig5	E963
U+5DB0.0	fGB	3-396C
U+5DB0.0	fR/S	 46.13
U+5DB0.0	fTotalStrokes	 16
U+5DB0.0	fCangjie	UNBQ
U+5DB0.0	fFourCorner	27752
U+5DB0.0	fHakka	 hai5; hai6
U+5DB0.0	fCantonese	haai5
U+5DB0.0	fMandarin	xie4; jie4
U+5DB0.0	fEnglish	gorge
U+5DB1.0	fUTF8	嶱
U+5DB1.0	fCNS11643	2-5669
U+5DB1.0	fBig5	E95F
U+5DB1.0	fGB	3-3959
U+5DB1.0	fR/S	 46.13
U+5DB1.0	fTotalStrokes	 16
U+5DB1.0	fCangjie	UTAV
U+5DB1.0	fFourCorner	24727
U+5DB1.0	fCantonese	got3; ho1
U+5DB1.0	fMandarin	ke3; ge2
U+5DB2.0	fUTF8	嶲
U+5DB2.0	fCNS11643	2-5671
U+5DB2.0	fBig5	E967
U+5DB2.0	fGB	E-2862
U+5DB2.0	fR/S	 46.13; 172.8
U+5DB2.0	fTotalStrokes	 16
U+5DB2.0	fCangjie	UOGS
U+5DB2.0	fFourCorner	22227
U+5DB2.0	fHakka	 sui1
U+5DB2.0	fCantonese	seoi5; kwai1; seoi2; kwai4
U+5DB2.0	fMandarin	sui3; xi1
U+5DB2.0	fEnglish	 a name of an old town in Szechuan cuckoo revolution of a wheel
U+5DB3.0	fUTF8	嶳
U+5DB3.0	fCNS11643	3-505A
U+5DB3.0	fGB	5-3B75
U+5DB3.0	fR/S	  46.13
U+5DB3.0	fTotalStrokes	 16
U+5DB3.0	fCangjie	UMOG
U+5DB3.0	fFourCorner	22110
U+5DB3.0	fHakka	 ti5; ti3; ti6; ti2
U+5DB3.0	fCantonese	dei6; deng6 (also); dei2 (also)
U+5DB3.0	fMandarin	di4; de5; de1
U+5DB3.0	fEnglish	ancient form of &#x5730;
U+5DB4.0	fUTF8	嶴
U+5DB4.0	fCNS11643	2-5674
U+5DB4.0	fBig5	E96A
U+5DB4.0	fGB	5-3B70
U+5DB4.0	fR/S	 46.13
U+5DB4.0	fTotalStrokes	 16
U+5DB4.0	fCangjie	HKU
U+5DB4.0	fFourCorner	27772
U+5DB4.0	fHakka	 au5
U+5DB4.0	fCantonese	ou3; ngou3 (also); aau3 (also); ngaau3 (also); juk1
U+5DB4.0	fMandarin	ao4; yu4
U+5DB4.0	fEnglish	same as &#x5967;
U+5DB5.0	fUTF8	嶵
U+5DB5.0	fCNS11643	2-566C
U+5DB5.0	fBig5	E962
U+5DB5.0	fGB	E-2863
U+5DB5.0	fR/S	 46.13
U+5DB5.0	fTotalStrokes	 16
U+5DB5.0	fCangjie	UWLY
U+5DB5.0	fFourCorner	26711
U+5DB5.0	fHakka	 cui5
U+5DB5.0	fCantonese	zeoi2
U+5DB5.0	fMandarin	zui4; zui3
U+5DB6.0	fUTF8	嶶
U+5DB6.0	fCNS11643	3-6471
U+5DB6.0	fGB	E-2864
U+5DB6.0	fR/S	 46.13
U+5DB6.0	fTotalStrokes	 16
U+5DB6.0	fCangjie	UHOK
U+5DB6.0	fFourCorner	2224
U+5DB6.0	fCantonese	mei4
U+5DB7.0	fUTF8	嶷
U+5DB7.0	fCNS11643	2-5C66
U+5DB7.0	fBig5	ECDA
U+5DB7.0	fGB	0-615A
U+5DB7.0	fR/S	 46.14
U+5DB7.0	fTotalStrokes	 17
U+5DB7.0	fCangjie	UPKO
U+5DB7.0	fFourCorner	22481
U+5DB7.0	fHakka	  ngi2
U+5DB7.0	fCantonese	jik6; ji4
U+5DB7.0	fMandarin	yi2; ni4; ni1
U+5DB7.0	fEnglish	range of mountains in hunan prov
U+5DB8.0	fUTF8	嶸
U+5DB8.0	fCNS11643	1-7358
U+5DB8.0	fBig5	C0AF
U+5DB8.0	fGB	1-6149
U+5DB8.0	fR/S	 46.14
U+5DB8.0	fTotalStrokes	 17
U+5DB8.0	fCangjie	UFFD
U+5DB8.0	fFourCorner	29794
U+5DB8.0	fHakka	 jin2;  jung2; rung2; rin2
U+5DB8.0	fCantonese	wing4; jing4; wang4
U+5DB8.0	fMandarin	rong2; heng2
U+5DB8.0	fEnglish	lofty (said of mountains)
U+5DB9.0	fUTF8	嶹
U+5DB9.0	fCNS11643	3-5457
U+5DB9.0	fGB	5-3B27
U+5DB9.0	fR/S	  46.14
U+5DB9.0	fTotalStrokes	 17
U+5DB9.0	fCangjie	UGNI
U+5DB9.0	fFourCorner	24740
U+5DB9.0	fHakka	  min2; min3
U+5DB9.0	fCantonese	ming4
U+5DB9.0	fMandarin	ming2; ming4
U+5DB9.0	fEnglish	same as &#x9CF4;
U+5DBA.0	fUTF8	嶺
U+5DBA.0	fCNS11643	1-7356
U+5DBA.0	fBig5	C0AD
U+5DBA.0	fGB	1-416B
U+5DBA.0	fR/S	 46.14
U+5DBA.0	fTotalStrokes	 17
U+5DBA.0	fCangjie	UOIC
U+5DBA.0	fFourCorner	22386
U+5DBA.0	fHakka	  liang1;  liang3
U+5DBA.0	fCantonese	ling5; leng5 (also)
U+5DBA.0	fMandarin	ling3
U+5DBA.0	fEnglish	[1] mountain ridge; mountain range [2] mountain
U+5DBB.0	fUTF8	嶻
U+5DBB.0	fCNS11643	3-5459
U+5DBB.0	fGB	3-396E
U+5DBB.0	fR/S	 46.16; 46.14
U+5DBB.0	fTotalStrokes	 17; 19
U+5DBB.0	fCangjie	UJIG
U+5DBB.0	fFourCorner	22150
U+5DBB.0	fCantonese	zit3
U+5DBB.0	fMandarin	za2
U+5DBC.0	fUTF8	嶼
U+5DBC.0	fCNS11643	1-7355
U+5DBC.0	fBig5	C0AC
U+5DBC.0	fGB	1-536C
U+5DBC.0	fR/S	 46.14
U+5DBC.0	fTotalStrokes	 17
U+5DBC.0	fCangjie	UHXC
U+5DBC.0	fFourCorner	27781
U+5DBC.0	fHakka	 ji3; sii5; sii6; se6; ri3; se5
U+5DBC.0	fCantonese	zeoi6; jyu4; jyu5 (also)
U+5DBC.0	fMandarin	yu3; xu4
U+5DBC.0	fEnglish	 island; islet
U+5DBD.0	fUTF8	嶽
U+5DBD.0	fCNS11643	1-7357
U+5DBD.0	fBig5	C0AE
U+5DBD.0	fGB	E-2865
U+5DBD.0	fR/S	 46.14
U+5DBD.0	fTotalStrokes	 17
U+5DBD.0	fCangjie	UKHK
U+5DBD.0	fFourCorner	22234
U+5DBD.0	fHakka	  ngok8
U+5DBD.0	fCantonese	ngok6; ok6 (also)
U+5DBD.0	fMandarin	yue4
U+5DBD.0	fEnglish	 high mountain
U+5DBE.0	fUTF8	嶾
U+5DBE.0	fCNS11643	3-5455
U+5DBE.0	fGB	3-396F
U+5DBE.0	fR/S	  46.14
U+5DBE.0	fTotalStrokes	 17
U+5DBE.0	fCangjie	UBMP
U+5DBE.0	fFourCorner	22730
U+5DBE.0	fHakka	 jun3; run3
U+5DBE.0	fCantonese	jan2
U+5DBE.0	fMandarin	yin3
U+5DBE.0	fEnglish	mountainous
U+5DBF.0	fUTF8	嶿
U+5DBF.0	fCNS11643	3-6472
U+5DBF.0	fGB	E-2866
U+5DBF.0	fR/S	 46.14
U+5DBF.0	fTotalStrokes	 17
U+5DBF.0	fCangjie	UMBB
U+5DBF.0	fFourCorner	2177
U+5DBF.0	fCantonese	seoi1
U+5DC0.0	fUTF8	巀
U+5DC0.0	fCNS11643	2-6150
U+5DC0.0	fBig5	EFC4
U+5DC0.0	fGB	E-2867
U+5DC0.0	fR/S	 46.15
U+5DC0.0	fTotalStrokes	 18
U+5DC0.0	fCangjie	UFGI
U+5DC0.0	fFourCorner	22253
U+5DC0.0	fCantonese	zit3; zit6
U+5DC0.0	fMandarin	jie1; jie2
U+5DC1.0	fUTF8	巁
U+5DC1.0	fCNS11643	3-5762
U+5DC1.0	fGB	3-384E
U+5DC1.0	fR/S	 46.15
U+5DC1.0	fTotalStrokes	 18
U+5DC1.0	fCangjie	UMTB
U+5DC1.0	fFourCorner	21720
U+5DC1.0	fCantonese	lai6
U+5DC1.0	fMandarin	li4
U+5DC2.0	fUTF8	巂
U+5DC2.0	fCNS11643	2-643E
U+5DC2.0	fBig5	F172
U+5DC2.0	fGB	5-3F37
U+5DC2.0	fR/S	 46.15
U+5DC2.0	fTotalStrokes	 18
U+5DC2.0	fCangjie	UOGB
U+5DC2.0	fFourCorner	22227
U+5DC2.0	fCantonese	kwai1; seoi5; seoi2; sai1; kwai4
U+5DC2.0	fMandarin	sui3; xi1; gui1
U+5DC2.0	fEnglish	place name
U+5DC3.0	fUTF8	巃
U+5DC3.0	fCNS11643	2-6549
U+5DC3.0	fBig5	F1FD
U+5DC3.0	fGB	3-3850
U+5DC3.0	fR/S	 46.16
U+5DC3.0	fTotalStrokes	 19
U+5DC3.0	fCangjie	UYBP
U+5DC3.0	fFourCorner	22211
U+5DC3.0	fHakka	 lung1; lung2
U+5DC3.0	fCantonese	lung4
U+5DC3.0	fMandarin	long2; long1; long3
U+5DC3.0	fEnglish	lofty
U+5DC4.0	fUTF8	巄
U+5DC4.0	fCNS11643	3-5A39
U+5DC4.0	fGB	5-3A63
U+5DC4.0	fR/S	  46.16
U+5DC4.0	fTotalStrokes	 19
U+5DC4.0	fCangjie	UYBP
U+5DC4.0	fFourCorner	21710
U+5DC4.0	fHakka	 lung1; lung2
U+5DC4.0	fCantonese	lung4
U+5DC4.0	fMandarin	long2; long1; long3
U+5DC4.0	fEnglish	same as &#x5DC3;
U+5DC5.0	fUTF8	巅
U+5DC5.0	fGB	0-615B
U+5DC5.0	fR/S	 46.16
U+5DC5.0	fTotalStrokes	 19
U+5DC5.0	fCangjie	UJCO
U+5DC5.0	fHakka	 dien1
U+5DC5.0	fCantonese	din1
U+5DC5.0	fMandarin	dian1
U+5DC5.0	fEnglish	simplified form of &#x5DD4;
U+5DC6.0	fUTF8	巆
U+5DC6.0	fCNS11643	2-6870
U+5DC6.0	fBig5	F444
U+5DC6.0	fGB	3-3950
U+5DC6.0	fR/S	 46.17
U+5DC6.0	fTotalStrokes	 20
U+5DC6.0	fCangjie	UFFR
U+5DC6.0	fFourCorner	29766
U+5DC6.0	fCantonese	wing4; jing4
U+5DC6.0	fMandarin	ying2; hong2
U+5DC7.0	fUTF8	巇
U+5DC7.0	fCNS11643	2-6871
U+5DC7.0	fBig5	F445
U+5DC7.0	fGB	3-3974
U+5DC7.0	fR/S	 46.17
U+5DC7.0	fTotalStrokes	 20
U+5DC7.0	fCangjie	UYTI
U+5DC7.0	fFourCorner	23750
U+5DC7.0	fCantonese	hei1; hei3
U+5DC7.0	fMandarin	xi1
U+5DC7.0	fEnglish	[1] crack [2] hazardous (said of a mountain path)
U+5DC8.0	fUTF8	巈
U+5DC8.0	fCNS11643	3-5C3D
U+5DC8.0	fGB	E-2868
U+5DC8.0	fR/S	 46.17
U+5DC8.0	fTotalStrokes	 20
U+5DC8.0	fCangjie	UTJD
U+5DC8.0	fFourCorner	22520
U+5DC8.0	fCantonese	guk1
U+5DC8.0	fMandarin	ju2
U+5DC9.0	fUTF8	巉
U+5DC9.0	fCNS11643	1-796B
U+5DC9.0	fBig5	C460
U+5DC9.0	fGB	3-3975
U+5DC9.0	fR/S	 46.17
U+5DC9.0	fTotalStrokes	 20
U+5DC9.0	fCangjie	UNRI
U+5DC9.0	fFourCorner	27716
U+5DC9.0	fHakka	  cam2
U+5DC9.0	fCantonese	caam4
U+5DC9.0	fMandarin	chan2; zhan3
U+5DC9.0	fEnglish	precipitous
U+5DCA.0	fUTF8	巊
U+5DCA.0	fCNS11643	3-5C3C
U+5DCA.0	fGB	3-3955
U+5DCA.0	fR/S	  46.17
U+5DCA.0	fTotalStrokes	 20
U+5DCA.0	fCangjie	UBCV
U+5DCA.0	fFourCorner	26740
U+5DCA.0	fCantonese	jing2
U+5DCA.0	fMandarin	ying3
U+5DCB.0	fUTF8	巋
U+5DCB.0	fCNS11643	2-6B57
U+5DCB.0	fBig5	F5C9
U+5DCB.0	fGB	1-3F79
U+5DCB.0	fR/S	 46.17
U+5DCB.0	fTotalStrokes	 20
U+5DCB.0	fCangjie	UHMB
U+5DCB.0	fFourCorner	22127
U+5DCB.0	fHakka	 kui3; gui1; kwui1
U+5DCB.0	fCantonese	kwai1; kwai2
U+5DCB.0	fMandarin	kui1; wei1
U+5DCB.0	fEnglish	[1] grand and secure; stately and lasting [2] rows after rows of small hills
U+5DCC.0	fUTF8	巌
U+5DCC.0	fCNS11643	3-5C3E
U+5DCC.0	fGB	E-2869
U+5DCC.0	fR/S	 46.17
U+5DCC.0	fTotalStrokes	 20
U+5DCC.0	fCangjie	UFMK
U+5DCC.0	fFourCorner	22240
U+5DCC.0	fHakka	 ngam2
U+5DCC.0	fCantonese	ngaam4; aam4 (also)
U+5DCC.0	fMandarin	yan2
U+5DCC.0	fEnglish	simplified form of &#x5DD6;
U+5DCD.0	fUTF8	巍
U+5DCD.0	fCNS11643	1-7A69
U+5DCD.0	fBig5	C4DE
U+5DCD.0	fGB	0-4E21
U+5DCD.0	fR/S	 46.18
U+5DCD.0	fTotalStrokes	 21
U+5DCD.0	fCangjie	UHVI
U+5DCD.0	fFourCorner	22413
U+5DCD.0	fHakka	 ngui2; ngui3
U+5DCD.0	fCantonese	ngai4; ai4 (also)
U+5DCD.0	fMandarin	wei1; wei2
U+5DCD.0	fEnglish	towering; lofty; majestic
U+5DCE.0	fUTF8	巎
U+5DCE.0	fCNS11643	4-663B
U+5DCE.0	fGB	5-3C22
U+5DCE.0	fR/S	 46.19
U+5DCE.0	fTotalStrokes	 22; 21
U+5DCE.0	fCangjie	UMCE
U+5DCE.0	fFourCorner	21740
U+5DCE.0	fCantonese	naau4; laau4 (also)
U+5DCE.0	fMandarin	nao2
U+5DCF.0	fUTF8	巏
U+5DCF.0	fCNS11643	2-6B58
U+5DCF.0	fBig5	F5CA
U+5DCF.0	fGB	3-3973
U+5DCF.0	fR/S	 46.18
U+5DCF.0	fTotalStrokes	 21
U+5DCF.0	fCangjie	UTRG
U+5DCF.0	fFourCorner	24714
U+5DCF.0	fCantonese	kyun4
U+5DCF.0	fMandarin	quan2; guan4
U+5DD0.0	fUTF8	巐
U+5DD0.0	fCNS11643	3-5E2D
U+5DD0.0	fGB	5-3C21
U+5DD0.0	fR/S	  46.18
U+5DD0.0	fTotalStrokes	 21
U+5DD0.0	fCangjie	UYBT
U+5DD0.0	fFourCorner	22240
U+5DD0.0	fCantonese	ciu2
U+5DD0.0	fMandarin	chao3
U+5DD1.0	fUTF8	巑
U+5DD1.0	fCNS11643	2-6D4C
U+5DD1.0	fBig5	F6DE
U+5DD1.0	fGB	3-3972
U+5DD1.0	fR/S	 46.19
U+5DD1.0	fTotalStrokes	 22
U+5DD1.0	fCangjie	UHUC
U+5DD1.0	fFourCorner	24786
U+5DD1.0	fHakka	  zon1
U+5DD1.0	fCantonese	cyun4; zan1
U+5DD1.0	fMandarin	cuan2
U+5DD1.0	fEnglish	high mountains
U+5DD2.0	fUTF8	巒
U+5DD2.0	fCNS11643	1-7B5E
U+5DD2.0	fBig5	C572
U+5DD2.0	fGB	1-424D
U+5DD2.0	fR/S	 46.19
U+5DD2.0	fTotalStrokes	 22
U+5DD2.0	fCangjie	VFU
U+5DD2.0	fFourCorner	22772
U+5DD2.0	fHakka	 lan2
U+5DD2.0	fCantonese	lyun4
U+5DD2.0	fMandarin	luan2
U+5DD2.0	fEnglish	 pointed mountain; pointed hill
U+5DD3.0	fUTF8	巓
U+5DD3.0	fCNS11643	3-5F43
U+5DD3.0	fGB	E-286A
U+5DD3.0	fR/S	 46.19
U+5DD3.0	fTotalStrokes	 22
U+5DD3.0	fCangjie	UPCC
U+5DD3.0	fFourCorner	22880
U+5DD3.0	fHakka	 din1; din3
U+5DD3.0	fCantonese	din1; tin1
U+5DD3.0	fMandarin	dian1
U+5DD3.0	fEnglish	[1] summit of a mountain; mountain top; peak [2] treetop
U+5DD4.0	fUTF8	巔
U+5DD4.0	fCNS11643	1-7B5D
U+5DD4.0	fBig5	C571
U+5DD4.0	fGB	1-615B
U+5DD4.0	fR/S	 46.19
U+5DD4.0	fTotalStrokes	 22
U+5DD4.0	fCangjie	UJCC
U+5DD4.0	fFourCorner	22886
U+5DD4.0	fHakka	  dien1; zon1; den1
U+5DD4.0	fCantonese	din1
U+5DD4.0	fMandarin	dian1
U+5DD4.0	fEnglish	summit of mountain; mountain top
U+5DD5.0	fUTF8	巕
U+5DD5.0	fCNS11643	2-6D4B
U+5DD5.0	fBig5	F6DD
U+5DD5.0	fGB	E-286B
U+5DD5.0	fR/S	 46.19
U+5DD5.0	fTotalStrokes	 22
U+5DD5.0	fCangjie	UTHV
U+5DD5.0	fFourCorner	24744
U+5DD5.0	fCantonese	jit6
U+5DD5.0	fMandarin	nie4
U+5DD6.0	fUTF8	巖
U+5DD6.0	fCNS11643	1-7C35
U+5DD6.0	fBig5	C5C9
U+5DD6.0	fGB	8-2D65
U+5DD6.0	fR/S	 46.20
U+5DD6.0	fTotalStrokes	 23
U+5DD6.0	fCangjie	URRK
U+5DD6.0	fFourCorner	22248
U+5DD6.0	fHakka	  ngam2; ngam3
U+5DD6.0	fCantonese	ngaam4; aam4 (also); jim4
U+5DD6.0	fMandarin	yan2
U+5DD6.0	fEnglish	[1] rock; crag [2] cave; grotto
U+5DD7.0	fUTF8	巗
U+5DD7.0	fCNS11643	3-6030
U+5DD7.0	fGB	E-286C
U+5DD7.0	fR/S	 46.19; 46.20
U+5DD7.0	fTotalStrokes	 23; 22
U+5DD7.0	fCangjie	URRK
U+5DD7.0	fFourCorner	26740
U+5DD7.0	fHakka	  ngam2; ngam3
U+5DD7.0	fCantonese	ngaam4; aam4 (also); jim4
U+5DD7.0	fMandarin	yan2
U+5DD7.0	fEnglish	same as &#x5DD6;
U+5DD8.0	fUTF8	巘
U+5DD8.0	fCNS11643	2-6F25
U+5DD8.0	fBig5	F7D6
U+5DD8.0	fGB	3-3967
U+5DD8.0	fR/S	 46.20
U+5DD8.0	fTotalStrokes	 23
U+5DD8.0	fCangjie	UYBK
U+5DD8.0	fFourCorner	23734
U+5DD8.0	fHakka	 hien5; hien3
U+5DD8.0	fCantonese	jin5 (also); jin2
U+5DD8.0	fMandarin	yan3
U+5DD8.0	fEnglish	 summit; peak; hilltop
U+5DD9.0	fUTF8	巙
U+5DD9.0	fCNS11643	4-6B68
U+5DD9.0	fGB	E-286D
U+5DD9.0	fR/S	 46.20; 46.21
U+5DD9.0	fTotalStrokes	 24; 23
U+5DD9.0	fCangjie	UTCE
U+5DD9.0	fFourCorner	28740
U+5DD9.0	fCantonese	naau4; laau4 (also)
U+5DD9.0	fMandarin	kui2; nao2
U+5DDA.0	fUTF8	巚
U+5DDA.0	fCNS11643	4-6A46
U+5DDA.0	fGB	E-286E
U+5DDA.0	fR/S	  46.20
U+5DDA.0	fTotalStrokes	 23
U+5DDA.0	fCangjie	UYBK
U+5DDA.0	fFourCorner	22230
U+5DDA.0	fHakka	 hien5; hien3
U+5DDA.0	fCantonese	jin5 (also); jin2
U+5DDA.0	fMandarin	yan3
U+5DDA.0	fEnglish	same as &#x5DD8;
U+5DDB.0	fUTF8	巛
U+5DDB.0	fCNS11643	3-213C
U+5DDB.0	fGB	0-675D
U+5DDB.0	fR/S	 47.0
U+5DDB.0	fTotalStrokes	 3
U+5DDB.0	fCangjie	VVV
U+5DDB.0	fFourCorner	22330
U+5DDB.0	fHakka	 chon1; con1; cam1
U+5DDB.0	fCantonese	cyun1; kwan1; caam1
U+5DDB.0	fMandarin	chuan1; cen1; kun1
U+5DDB.0	fEnglish	[1] river [2] Kangxi radical 47 [3] standard form of &#x5D7E;
U+5DDC.0	fUTF8	巜
U+5DDC.0	fCNS11643	4-212C
U+5DDC.0	fGB	5-303D
U+5DDC.0	fR/S	 47.0
U+5DDC.0	fTotalStrokes	 2; 3
U+5DDC.0	fCangjie	VV
U+5DDC.0	fFourCorner	22330
U+5DDC.0	fCantonese	kui2; kwui2 (also)
U+5DDC.0	fMandarin	gui4; kuai4
U+5DDD.0	fUTF8	川
U+5DDD.0	fCNS11643	1-4455
U+5DDD.0	fBig5	A474
U+5DDD.0	fGB	0-3428
U+5DDD.0	fR/S	 47.0
U+5DDD.0	fTotalStrokes	 3
U+5DDD.0	fCangjie	LLL
U+5DDD.0	fFourCorner	22000
U+5DDD.0	fHakka	  con1; chon1
U+5DDD.0	fCantonese	cyun1
U+5DDD.0	fMandarin	chuan1
U+5DDD.0	fEnglish	[1] stream; river [2] flow; constant flow [3] boil with water [4] Szechwan province
U+5DDE.0	fUTF8	州
U+5DDE.0	fCNS11643	1-477C
U+5DDE.0	fBig5	A67B
U+5DDE.0	fGB	0-565D
U+5DDE.0	fR/S	 47.3
U+5DDE.0	fTotalStrokes	 6
U+5DDE.0	fCangjie	ILIL
U+5DDE.0	fFourCorner	32000
U+5DDE.0	fHakka	 zu1; zhiu1; ziu1; zhu1
U+5DDE.0	fCantonese	zau1
U+5DDE.0	fMandarin	zhou1
U+5DDE.0	fEnglish	[1] administrative district in ancient China [2] region with 2,500 families [3] county [4] place surrounded by water; islet; sand bar [5] state (in the U.S.A.) [6] a Chinese family name
U+5DDF.0	fUTF8	巟
U+5DDF.0	fCNS11643	2-223A
U+5DDF.0	fBig5	C9DA
U+5DDF.0	fGB	5-334C
U+5DDF.0	fR/S	 47.3
U+5DDF.0	fTotalStrokes	 6
U+5DDF.0	fCangjie	YVHLU
U+5DDF.0	fFourCorner	00211
U+5DDF.0	fHakka	 fong1; mong2; miong2
U+5DDF.0	fCantonese	fong1; mong4; mong1
U+5DDF.0	fMandarin	huang1; mang2; wang2; huang3; mang3
U+5DDF.0	fEnglish	same as &#x8292;
U+5DE0.0	fUTF8	巠
U+5DE0.0	fCNS11643	2-236A
U+5DE0.0	fBig5	CACA
U+5DE0.0	fGB	E-286F
U+5DE0.0	fR/S	 47.4
U+5DE0.0	fTotalStrokes	 7
U+5DE0.0	fCangjie	MVVM
U+5DE0.0	fFourCorner	10101
U+5DE0.0	fHakka	  gin1
U+5DE0.0	fCantonese	ging1
U+5DE0.0	fMandarin	jing1
U+5DE0.0	fEnglish	 streams running underground flowing water
U+5DE1.0	fUTF8	巡
U+5DE1.0	fCNS11643	1-4B56
U+5DE1.0	fBig5	A8B5
U+5DE1.0	fGB	0-5132
U+5DE1.0	fR/S	 47.4; 162.3
U+5DE1.0	fTotalStrokes	 7
U+5DE1.0	fCangjie	YVVV
U+5DE1.0	fFourCorner	32303
U+5DE1.0	fHakka	  sun2; sun3; cun2
U+5DE1.0	fCantonese	ceon4
U+5DE1.0	fMandarin	xun2
U+5DE1.0	fEnglish	[1] patrol; inspect; go on circuit; cruise [2] round (of drinks) [3] policeman; cop
U+5DE2.0	fUTF8	巢
U+5DE2.0	fCNS11643	1-5A25
U+5DE2.0	fBig5	B15F
U+5DE2.0	fGB	0-3332
U+5DE2.0	fR/S	 47.8
U+5DE2.0	fTotalStrokes	 11
U+5DE2.0	fCangjie	VVWD
U+5DE2.0	fFourCorner	32904
U+5DE2.0	fHakka	  cau2; ciu2
U+5DE2.0	fCantonese	caau4; cau4
U+5DE2.0	fMandarin	chao2; jiao3; zhao4
U+5DE2.0	fEnglish	[1] bird's nest [2] living quarter in tree [3] haunt; den; hideout (for bandits, etc.) [4] a Chinese family name
U+5DE3.0	fUTF8	巣
U+5DE3.0	fCNS11643	3-3557
U+5DE3.0	fGB	E-2870
U+5DE3.0	fR/S	 47.8
U+5DE3.0	fTotalStrokes	 11
U+5DE3.0	fCangjie	FWD
U+5DE3.0	fFourCorner	23900
U+5DE3.0	fHakka	 cau2
U+5DE3.0	fCantonese	caau4; cau4
U+5DE3.0	fMandarin	chao2; jiao3; zhao4
U+5DE3.0	fEnglish	simplified form of &#x5DE2;
U+5DE4.0	fUTF8	巤
U+5DE4.0	fCNS11643	3-4B6C
U+5DE4.0	fGB	5-4165
U+5DE4.0	fR/S	  47.12
U+5DE4.0	fTotalStrokes	 15
U+5DE4.0	fCangjie	VVWKV
U+5DE4.0	fFourCorner	22717
U+5DE4.0	fHakka	  lap8;  liap8
U+5DE4.0	fCantonese	lip6
U+5DE4.0	fMandarin	lie1; lie4
U+5DE4.0	fEnglish	 mane bristles (as on a hog) dorsal fins
U+5DE5.0	fUTF8	工
U+5DE5.0	fCNS11643	1-4456
U+5DE5.0	fBig5	A475
U+5DE5.0	fGB	0-3924
U+5DE5.0	fR/S	 48.0
U+5DE5.0	fTotalStrokes	 3
U+5DE5.0	fCangjie	MLM
U+5DE5.0	fFourCorner	10100
U+5DE5.0	fHakka	  gung1
U+5DE5.0	fCantonese	gung1
U+5DE5.0	fMandarin	gong1
U+5DE5.0	fEnglish	[1] labor; work [2] worker; laborer [3] shift; time used in doing a piece of work [4] day's work [5] engineering or building project [6] defense work [7] fine; delicate [8] be skilled in
U+5DE6.0	fUTF8	左
U+5DE6.0	fCNS11643	1-464A
U+5DE6.0	fBig5	A5AA
U+5DE6.0	fGB	0-5773
U+5DE6.0	fR/S	 48.2
U+5DE6.0	fTotalStrokes	 5
U+5DE6.0	fCangjie	KM
U+5DE6.0	fFourCorner	40011
U+5DE6.0	fHakka	  zo3
U+5DE6.0	fCantonese	zo2
U+5DE6.0	fMandarin	zuo3; zuo4
U+5DE6.0	fEnglish	[1] left; left side [2] east; east side [3] unorthodox; improper [4] supporting; assist [5] be demoted; descend [6] inconvenience; inconvenient [7] erroneous; mistaken [8] unduly stubborn [9] disobey; disregard [10] a Chinese family name
U+5DE7.0	fUTF8	巧
U+5DE7.0	fCNS11643	1-4649
U+5DE7.0	fBig5	A5A9
U+5DE7.0	fGB	0-4749
U+5DE7.0	fR/S	 48.2
U+5DE7.0	fTotalStrokes	 5
U+5DE7.0	fCangjie	MMVS
U+5DE7.0	fFourCorner	11127
U+5DE7.0	fHakka	  kau3
U+5DE7.0	fCantonese	haau2; kiu2 (also)
U+5DE7.0	fMandarin	qiao3; qiao4
U+5DE7.0	fEnglish	skillful; ingenious; clever
U+5DE8.0	fUTF8	巨
U+5DE8.0	fCNS11643	1-4648
U+5DE8.0	fBig5	A5A8
U+5DE8.0	fGB	0-3E5E
U+5DE8.0	fR/S	 48.2
U+5DE8.0	fTotalStrokes	 5
U+5DE8.0	fCangjie	SS
U+5DE8.0	fFourCorner	71717
U+5DE8.0	fHakka	 ki1; ki3;  gi3; gi1; gi5
U+5DE8.0	fCantonese	geoi6
U+5DE8.0	fMandarin	ju4
U+5DE8.0	fEnglish	[1] large; great; enormous; big [2] very [3] a Chinese family name
U+5DE9.0	fUTF8	巩
U+5DE9.0	fCNS11643	4-2263
U+5DE9.0	fGB	0-392E
U+5DE9.0	fR/S	 48.3
U+5DE9.0	fTotalStrokes	 6
U+5DE9.0	fCangjie	MHNI
U+5DE9.0	fFourCorner	17110
U+5DE9.0	fHakka	 giung3
U+5DE9.0	fCantonese	gung2
U+5DE9.0	fMandarin	gong3; xie2
U+5DE9.0	fEnglish	simplified form of &#x978F;
U+5DEA.0	fUTF8	巪
U+5DEA.0	fGB	1-7D23
U+5DEA.0	fR/S	 48.3
U+5DEA.0	fTotalStrokes	 6
U+5DEA.0	fCangjie	XSSN
U+5DEA.0	fCantonese	geoi6
U+5DEA.0	fEnglish	 person's name
U+5DEB.0	fUTF8	巫
U+5DEB.0	fCNS11643	1-4A27
U+5DEB.0	fBig5	A7C5
U+5DEB.0	fGB	0-4E57
U+5DEB.0	fR/S	 48.4
U+5DEB.0	fTotalStrokes	 7
U+5DEB.0	fCangjie	MOO
U+5DEB.0	fFourCorner	10108
U+5DEB.0	fHakka	  mu2; wu2; mu3
U+5DEB.0	fCantonese	mou4
U+5DEB.0	fMandarin	wu1; wu2
U+5DEB.0	fEnglish	[1] wizard; sorcerer; witch; shaman [2] witchcraft; sorcery [3] a Chinese family name
U+5DEC.0	fUTF8	巬
U+5DEC.0	fGB	1-7D43
U+5DEC.0	fR/S	 48.6; 16.7
U+5DEC.0	fTotalStrokes	 9
U+5DEC.0	fCangjie	MNQO
U+5DEC.0	fCantonese	gei1
U+5DED.0	fUTF8	巭
U+5DED.0	fGB	1-7D44
U+5DED.0	fR/S	 48.6; 19.7
U+5DED.0	fTotalStrokes	 9
U+5DED.0	fCangjie	MSQO
U+5DED.0	fCantonese	gung1
U+5DED.0	fEnglish	 study
U+5DEE.0	fUTF8	差
U+5DEE.0	fCNS11643	1-5538
U+5DEE.0	fBig5	AE74
U+5DEE.0	fGB	0-326E
U+5DEE.0	fR/S	 48.7
U+5DEE.0	fTotalStrokes	 10
U+5DEE.0	fCangjie	TQM
U+5DEE.0	fFourCorner	80211
U+5DEE.0	fHakka	  cai1; ci1
U+5DEE.0	fCantonese	caa1; caai1; ci1; co1
U+5DEE.0	fMandarin	cha4; chai4; cuo1
U+5DEE.0	fEnglish	[1] differ [2] wrong [3] want; fall short of [4] not up to standard; poor
U+5DEE.1	fUTF8	差
U+5DEE.1	fCNS11643	1-5538
U+5DEE.1	fBig5	AE74
U+5DEE.1	fGB	0-326E
U+5DEE.1	fR/S	 48.7
U+5DEE.1	fTotalStrokes	 10
U+5DEE.1	fCangjie	TQM
U+5DEE.1	fFourCorner	80211
U+5DEE.1	fHakka	 ca1
U+5DEE.1	fMandarin	cha1
U+5DEE.1	fEnglish	[1] error; mistake [2] difference; discrepancy [3] difference (mathematics)
U+5DEE.2	fUTF8	差
U+5DEE.2	fCNS11643	1-5538
U+5DEE.2	fBig5	AE74
U+5DEE.2	fGB	0-326E
U+5DEE.2	fR/S	 48.7
U+5DEE.2	fTotalStrokes	 10
U+5DEE.2	fCangjie	TQM
U+5DEE.2	fFourCorner	80211
U+5DEE.2	fMandarin	chai1
U+5DEE.2	fEnglish	[1] messenger [2] dispatch; send (a person) [3] one's duty; official assignment [4] errand
U+5DEF.0	fUTF8	巯
U+5DEF.0	fGB	0-5B4F
U+5DEF.0	fR/S	 48.9
U+5DEF.0	fTotalStrokes	 12
U+5DEF.0	fCangjie	XNMYI
U+5DEF.0	fCantonese	kau4
U+5DEF.0	fMandarin	qiu2; liu2
U+5DEF.0	fEnglish	simplified form of &#x5DF0;
U+5DF0.0	fUTF8	巰
U+5DF0.0	fCNS11643	2-425D
U+5DF0.0	fBig5	DD57
U+5DF0.0	fGB	1-5B4F
U+5DF0.0	fR/S	 48.10
U+5DF0.0	fTotalStrokes	 13
U+5DF0.0	fCangjie	MMYIU
U+5DF0.0	fFourCorner	10112
U+5DF0.0	fCantonese	kau4
U+5DF0.0	fMandarin	qiu2; liu2
U+5DF0.0	fEnglish	 atom group
U+5DF1.0	fUTF8	己
U+5DF1.0	fCNS11643	1-4457
U+5DF1.0	fBig5	A476
U+5DF1.0	fGB	0-3C3A
U+5DF1.0	fR/S	 49.0
U+5DF1.0	fTotalStrokes	 3
U+5DF1.0	fCangjie	SU
U+5DF1.0	fFourCorner	17717
U+5DF1.0	fHakka	  gi3
U+5DF1.0	fCantonese	gei2; gei1 (also)
U+5DF1.0	fMandarin	ji3; qi3
U+5DF1.0	fEnglish	[1] self; oneself; personal; private; one's own [2] the sixth of the celestial stems
U+5DF2.0	fUTF8	已
U+5DF2.0	fCNS11643	1-4458
U+5DF2.0	fBig5	A477
U+5DF2.0	fGB	0-5251
U+5DF2.0	fR/S	 49.0
U+5DF2.0	fTotalStrokes	 3
U+5DF2.0	fCangjie	SU
U+5DF2.0	fFourCorner	17717
U+5DF2.0	fHakka	 ji3; gi3; ri3; gi5; i1; i5; cu5; ji1; i3
U+5DF2.0	fCantonese	ji5
U+5DF2.0	fMandarin	yi3
U+5DF2.0	fEnglish	[1] already [2] come to an end; finish; complete; finished [3] stop; cease [4] used to indicate the past [5] excessive; very; much [6] a final particle to add emphasis
U+5DF3.0	fUTF8	巳
U+5DF3.0	fCNS11643	1-4459
U+5DF3.0	fBig5	A478
U+5DF3.0	fGB	0-4B48
U+5DF3.0	fR/S	 49.0
U+5DF3.0	fTotalStrokes	 3
U+5DF3.0	fCangjie	RU
U+5DF3.0	fFourCorner	77717
U+5DF3.0	fHakka	 cih6; sii5; sii6; gi5; su5; cih5; se5
U+5DF3.0	fCantonese	zi6
U+5DF3.0	fMandarin	si4
U+5DF3.0	fEnglish	[1] the sixth of the twelve Terrestrial branches [2] 9 to 11 a.m. [3] a Chinese family name
U+5DF4.0	fUTF8	巴
U+5DF4.0	fCNS11643	1-453B
U+5DF4.0	fBig5	A4DA
U+5DF4.0	fGB	0-304D
U+5DF4.0	fR/S	 49.1
U+5DF4.0	fTotalStrokes	 4
U+5DF4.0	fCangjie	AU
U+5DF4.0	fFourCorner	77717
U+5DF4.0	fHakka	  ba1
U+5DF4.0	fCantonese	baa1; baa2 (also)
U+5DF4.0	fMandarin	ba1
U+5DF4.0	fEnglish	[1] name of an ancient state which occupied today's eastern Szechwan [2] crust formed as a result of heat or dryness [3] greatly desire; anxiously hope; expect [4] used with parts of human body (hands, cheeks, chin, etc.) [5] a final particle implying closeness or adhesion [6] be close to [7] bar (physics) [8] a Chinese family name
U+5DF4.1	fUTF8	巴
U+5DF4.1	fCNS11643	1-453B
U+5DF4.1	fBig5	A4DA
U+5DF4.1	fGB	0-304D
U+5DF4.1	fR/S	 49.1
U+5DF4.1	fTotalStrokes	 4
U+5DF4.1	fCangjie	AU
U+5DF4.1	fFourCorner	77717
U+5DF4.1	fMandarin	ba5
U+5DF5.0	fUTF8	巵
U+5DF5.0	fCNS11643	3-2567
U+5DF5.0	fGB	E-2871
U+5DF5.0	fR/S	 49.4
U+5DF5.0	fTotalStrokes	 7
U+5DF5.0	fCangjie	HMAU
U+5DF5.0	fFourCorner	72210
U+5DF5.0	fHakka	 zhi1; zi1; zih1; tje1
U+5DF5.0	fCantonese	zi1; zi6
U+5DF5.0	fMandarin	zhi1
U+5DF5.0	fEnglish	measuring cup; wine container
U+5DF6.0	fUTF8	巶
U+5DF6.0	fCNS11643	3-2860
U+5DF6.0	fGB	5-3E70
U+5DF6.0	fR/S	 49.5
U+5DF6.0	fTotalStrokes	 8
U+5DF6.0	fCangjie	SRRU
U+5DF6.0	fFourCorner	17610
U+5DF6.0	fHakka	 shau5; sau5
U+5DF6.0	fCantonese	siu6
U+5DF6.0	fMandarin	shao1; shao4; zhao1
U+5DF6.0	fEnglish	same as &#x4F4B;
U+5DF7.0	fUTF8	巷
U+5DF7.0	fCNS11643	1-5073
U+5DF7.0	fBig5	ABD1
U+5DF7.0	fGB	0-4F6F
U+5DF7.0	fR/S	 49.6
U+5DF7.0	fTotalStrokes	 9
U+5DF7.0	fCangjie	TCRU
U+5DF7.0	fFourCorner	44717
U+5DF7.0	fHakka	  hong5; hong6; hong3
U+5DF7.0	fCantonese	hong6; hong2 (also)
U+5DF7.0	fMandarin	xiang4; hang4
U+5DF7.0	fEnglish	 alley; lane
U+5DF8.0	fUTF8	巸
U+5DF8.0	fCNS11643	4-2C64
U+5DF8.0	fGB	3-3B61
U+5DF8.0	fR/S	 49.6; 49.7
U+5DF8.0	fTotalStrokes	 9; 10
U+5DF8.0	fCangjie	SLRU
U+5DF8.0	fFourCorner	77710
U+5DF8.0	fCantonese	ji4
U+5DF8.0	fMandarin	yi2
U+5DF9.0	fUTF8	巹
U+5DF9.0	fCNS11643	2-2A30
U+5DF9.0	fBig5	CEAF
U+5DF9.0	fGB	5-3E71
U+5DF9.0	fR/S	 49.6
U+5DF9.0	fTotalStrokes	 9
U+5DF9.0	fCangjie	NEMSU
U+5DF9.0	fFourCorner	17717
U+5DF9.0	fHakka	 giun3
U+5DF9.0	fCantonese	gan2
U+5DF9.0	fMandarin	jin3
U+5DFA.0	fUTF8	巺
U+5DFA.0	fCNS11643	3-2C45
U+5DFA.0	fGB	E-2872
U+5DFA.0	fR/S	 49.6
U+5DFA.0	fTotalStrokes	 9
U+5DFA.0	fCangjie	RUMC
U+5DFA.0	fFourCorner	77800
U+5DFA.0	fHakka	 sun5
U+5DFA.0	fCantonese	seon3
U+5DFA.0	fMandarin	xun4; sun4; zhuan4
U+5DFA.0	fEnglish	standard form of &#x5DFD;
U+5DFB.0	fUTF8	巻
U+5DFB.0	fCNS11643	3-2C44
U+5DFB.0	fGB	E-2873
U+5DFB.0	fR/S	 26.6; 49.6
U+5DFB.0	fTotalStrokes	 9
U+5DFB.0	fCangjie	XFQSU
U+5DFB.0	fFourCorner	90710
U+5DFB.0	fHakka	 gien3; kian2; gen3; kien2
U+5DFB.0	fCantonese	gyun2; gyun3; kyun4
U+5DFB.0	fMandarin	juan4; gun3; juan3
U+5DFB.0	fEnglish	scroll; curl; make a comeback
U+5DFC.0	fUTF8	巼
U+5DFC.0	fGB	1-7D60
U+5DFC.0	fR/S	 49.7
U+5DFC.0	fTotalStrokes	 10
U+5DFC.0	fCangjie	AURP
U+5DFC.0	fCantonese	baa1
U+5DFC.0	fEnglish	 place name
U+5DFD.0	fUTF8	巽
U+5DFD.0	fCNS11643	1-5E78
U+5DFD.0	fBig5	B453
U+5DFD.0	fGB	0-5963
U+5DFD.0	fR/S	 49.9
U+5DFD.0	fTotalStrokes	 12
U+5DFD.0	fCangjie	RUTC
U+5DFD.0	fFourCorner	77801
U+5DFD.0	fHakka	 sun5
U+5DFD.0	fCantonese	seon3
U+5DFD.0	fMandarin	xun4; sun4; zhuan4
U+5DFD.0	fEnglish	[1] 5th of the Eight diagrams [2] mild; bland; subservient; submissive
U+5DFE.0	fUTF8	巾
U+5DFE.0	fCNS11643	1-445A
U+5DFE.0	fBig5	A479
U+5DFE.0	fGB	0-3D6D
U+5DFE.0	fR/S	 50.0
U+5DFE.0	fTotalStrokes	 3
U+5DFE.0	fCangjie	LB
U+5DFE.0	fFourCorner	40227
U+5DFE.0	fHakka	  gin1
U+5DFE.0	fCantonese	gan1
U+5DFE.0	fMandarin	jin1
U+5DFE.0	fEnglish	[1] handkerchief; napkin; towel [2] headgear; articles for dressing the hair [3] Kangxi radical 50
U+5DFF.0	fUTF8	巿
U+5DFF.0	fCNS11643	2-213D
U+5DFF.0	fBig5	C95D
U+5DFF.0	fGB	5-397E
U+5DFF.0	fR/S	 50.1
U+5DFF.0	fTotalStrokes	 4
U+5DFF.0	fCangjie	JB
U+5DFF.0	fFourCorner	50227
U+5DFF.0	fHakka	 fut7
U+5DFF.0	fCantonese	fat1; si5
U+5DFF.0	fMandarin	fu2; bei4; fei4; po1
U+5DFF.0	fEnglish	revolve; make circuit; turn
U+5E00.0	fUTF8	帀
U+5E00.0	fCNS11643	3-2163
U+5E00.0	fGB	E-2874
U+5E00.0	fR/S	  50.1
U+5E00.0	fTotalStrokes	 4
U+5E00.0	fCangjie	MLB
U+5E00.0	fFourCorner	10220
U+5E00.0	fHakka	 zap7
U+5E00.0	fCantonese	zaap3
U+5E00.0	fMandarin	za1
U+5E00.0	fEnglish	 go round; make a circuit; make a revolution; turn round
U+5E01.0	fUTF8	币
U+5E01.0	fCNS11643	4-214C
U+5E01.0	fGB	0-3152
U+5E01.0	fR/S	 50.1
U+5E01.0	fTotalStrokes	 4
U+5E01.0	fCangjie	HLB
U+5E01.0	fFourCorner	20220
U+5E01.0	fHakka	 jin5
U+5E01.0	fCantonese	bai6; jan3
U+5E01.0	fMandarin	bi4; yin4
U+5E01.0	fEnglish	[1] same as &#x5370; [2] simplified form of &#x5E63;
U+5E02.0	fUTF8	市
U+5E02.0	fCNS11643	1-464B
U+5E02.0	fBig5	A5AB
U+5E02.0	fGB	0-4A50
U+5E02.0	fR/S	 50.2
U+5E02.0	fTotalStrokes	 5
U+5E02.0	fCangjie	YLB
U+5E02.0	fFourCorner	00227
U+5E02.0	fHakka	 si5;  shi5; sii5; she5
U+5E02.0	fCantonese	si5; si2 (also)
U+5E02.0	fMandarin	shi4
U+5E02.0	fEnglish	[1] market; marketplace; place for battering goods; fair [2] city; town [3] buy; sell; trade [4] a Chinese family name
U+5E03.0	fUTF8	布
U+5E03.0	fCNS11643	1-464C
U+5E03.0	fBig5	A5AC
U+5E03.0	fGB	0-323C
U+5E03.0	fR/S	 50.2
U+5E03.0	fTotalStrokes	 5
U+5E03.0	fCangjie	KLB
U+5E03.0	fFourCorner	40227
U+5E03.0	fHakka	  bu5
U+5E03.0	fCantonese	bou3
U+5E03.0	fMandarin	bu4; bian4
U+5E03.0	fEnglish	[1] (cotton) cloth; textiles [2] declare; announce; proclaim [3] display; distribute; disseminate; spread out [4] a Chinese family name
U+5E04.0	fUTF8	帄
U+5E04.0	fCNS11643	2-2159
U+5E04.0	fBig5	C978
U+5E04.0	fGB	5-3A21
U+5E04.0	fR/S	 50.2
U+5E04.0	fTotalStrokes	 5
U+5E04.0	fCangjie	LBMN
U+5E04.0	fFourCorner	41220
U+5E04.0	fCantonese	ding1
U+5E04.0	fMandarin	ding1
U+5E04.0	fEnglish	 mend clothes
U+5E05.0	fUTF8	帅
U+5E05.0	fGB	0-4B27
U+5E05.0	fR/S	 50.2
U+5E05.0	fTotalStrokes	 5
U+5E05.0	fCangjie	LLLB
U+5E05.0	fHakka	 sai5; soi5; sut7
U+5E05.0	fCantonese	seoi3; seot1
U+5E05.0	fMandarin	shuai4; shuo4
U+5E05.0	fEnglish	simplified form of &#x5E25;
U+5E06.0	fUTF8	帆
U+5E06.0	fCNS11643	1-477D
U+5E06.0	fBig5	A67C
U+5E06.0	fGB	0-372B
U+5E06.0	fR/S	 50.3
U+5E06.0	fTotalStrokes	 6
U+5E06.0	fCangjie	LBHNI
U+5E06.0	fFourCorner	47210
U+5E06.0	fHakka	  fam2
U+5E06.0	fCantonese	faan4; faan6
U+5E06.0	fMandarin	fan1; fan2; fan4
U+5E06.0	fEnglish	[1] sail (of a boat) [2] boat
U+5E07.0	fUTF8	帇
U+5E07.0	fCNS11643	5-225C
U+5E07.0	fGB	5-6737
U+5E07.0	fR/S	  50.3
U+5E07.0	fTotalStrokes	 6
U+5E07.0	fCangjie	LB
U+5E07.0	fFourCorner	50220
U+5E07.0	fCantonese	nip6; lip6 (also)
U+5E07.0	fMandarin	nie4
U+5E08.0	fUTF8	师
U+5E08.0	fCNS11643	3-235C
U+5E08.0	fGB	0-4A26
U+5E08.0	fR/S	 50.3
U+5E08.0	fTotalStrokes	 6
U+5E08.0	fCangjie	LLMB
U+5E08.0	fFourCorner	21020
U+5E08.0	fHakka	 sii1
U+5E08.0	fCantonese	si1
U+5E08.0	fMandarin	shi1
U+5E08.0	fEnglish	simplified form of &#x5E2B;
U+5E09.0	fUTF8	帉
U+5E09.0	fCNS11643	3-2568
U+5E09.0	fGB	5-3A26
U+5E09.0	fR/S	  50.4
U+5E09.0	fTotalStrokes	 7
U+5E09.0	fCangjie	LBCSH
U+5E09.0	fFourCorner	48220
U+5E09.0	fHakka	 fun1
U+5E09.0	fCantonese	fan1
U+5E09.0	fMandarin	fen1
U+5E09.0	fEnglish	towel hung at the girdle
U+5E0A.0	fUTF8	帊
U+5E0A.0	fCNS11643	2-236B
U+5E0A.0	fBig5	CACB
U+5E0A.0	fGB	3-3765
U+5E0A.0	fR/S	 50.4
U+5E0A.0	fTotalStrokes	 7
U+5E0A.0	fCangjie	LBAU
U+5E0A.0	fFourCorner	47217
U+5E0A.0	fHakka	  pa5
U+5E0A.0	fCantonese	paa3
U+5E0A.0	fMandarin	pa4; pa1
U+5E0A.0	fEnglish	 napkin
U+5E0B.0	fUTF8	帋
U+5E0B.0	fCNS11643	3-256B
U+5E0B.0	fGB	8-2D62
U+5E0B.0	fR/S	  50.4
U+5E0B.0	fTotalStrokes	 7
U+5E0B.0	fCangjie	HPLB
U+5E0B.0	fFourCorner	72220
U+5E0B.0	fHakka	 zhi3; zih3
U+5E0B.0	fCantonese	zi2
U+5E0B.0	fMandarin	zhi3
U+5E0B.0	fEnglish	same as &#x7D19;
U+5E0C.0	fUTF8	希
U+5E0C.0	fCNS11643	1-4A28
U+5E0C.0	fBig5	A7C6
U+5E0C.0	fGB	0-4F23
U+5E0C.0	fR/S	 50.4
U+5E0C.0	fTotalStrokes	 7
U+5E0C.0	fCangjie	KKLB
U+5E0C.0	fFourCorner	40227
U+5E0C.0	fHakka	  hi1
U+5E0C.0	fCantonese	hei1
U+5E0C.0	fMandarin	xi1
U+5E0C.0	fEnglish	[1] rare; strange; precious [2] hope; expect; wish; desire; long for [3] come to a stop gradually [4] become silent [5] very; much [6] a Chinese family name
U+5E0D.0	fUTF8	帍
U+5E0D.0	fCNS11643	4-2423
U+5E0D.0	fGB	5-5263
U+5E0D.0	fR/S	  50.4
U+5E0D.0	fTotalStrokes	 7
U+5E0D.0	fCangjie	HSLB
U+5E0D.0	fFourCorner	30227
U+5E0D.0	fHakka	 fu5
U+5E0D.0	fCantonese	wu6
U+5E0D.0	fMandarin	hu4
U+5E0D.0	fEnglish	 fine napkin
U+5E0E.0	fUTF8	帎
U+5E0E.0	fCNS11643	2-236C
U+5E0E.0	fBig5	CACC
U+5E0E.0	fGB	5-3A28
U+5E0E.0	fR/S	 50.4
U+5E0E.0	fTotalStrokes	 7
U+5E0E.0	fCangjie	LBLBU
U+5E0E.0	fFourCorner	44212
U+5E0E.0	fCantonese	daam3; zam2
U+5E0E.0	fMandarin	dan4
U+5E0E.0	fEnglish	a way of playing snooker with the stick on top of one's thumb
U+5E0F.0	fUTF8	帏
U+5E0F.0	fGB	0-6078
U+5E0F.0	fR/S	 50.4
U+5E0F.0	fTotalStrokes	 7
U+5E0F.0	fCangjie	LBQS
U+5E0F.0	fHakka	 vui2
U+5E0F.0	fCantonese	wai4
U+5E0F.0	fMandarin	wei2; hui1
U+5E0F.0	fEnglish	simplified form of &#x5E43;
U+5E10.0	fUTF8	帐
U+5E10.0	fGB	0-554A
U+5E10.0	fR/S	 50.4
U+5E10.0	fTotalStrokes	 7
U+5E10.0	fCangjie	LBPO
U+5E10.0	fHakka	 zhong5
U+5E10.0	fCantonese	zoeng3; zoeng2 (also)
U+5E10.0	fMandarin	zhang4
U+5E10.0	fEnglish	simplified form of &#x5E33;
U+5E11.0	fUTF8	帑
U+5E11.0	fCNS11643	1-4D30
U+5E11.0	fBig5	A9AE
U+5E11.0	fGB	0-607B
U+5E11.0	fR/S	 50.5
U+5E11.0	fTotalStrokes	 8
U+5E11.0	fCangjie	VELB
U+5E11.0	fFourCorner	47227
U+5E11.0	fHakka	 tong2; nu2; tong3; lu2
U+5E11.0	fCantonese	tong2; nou4; lou4 (also)
U+5E11.0	fMandarin	tang3; nu2; nu3
U+5E11.0	fEnglish	[1] treasury [2] public funds; public money
U+5E12.0	fUTF8	帒
U+5E12.0	fCNS11643	3-2862
U+5E12.0	fGB	5-3A2B
U+5E12.0	fR/S	 50.5
U+5E12.0	fTotalStrokes	 8
U+5E12.0	fCangjie	OPLB
U+5E12.0	fFourCorner	23220
U+5E12.0	fHakka	 toi6;  toi5
U+5E12.0	fCantonese	doi6
U+5E12.0	fMandarin	dai4; dai1
U+5E12.0	fEnglish	 bag; sack case pocket; pouch
U+5E13.0	fUTF8	帓
U+5E13.0	fCNS11643	3-2861
U+5E13.0	fGB	3-3766
U+5E13.0	fR/S	  50.5
U+5E13.0	fTotalStrokes	 8
U+5E13.0	fCangjie	LBDJ
U+5E13.0	fFourCorner	45290
U+5E13.0	fHakka	  mat7
U+5E13.0	fCantonese	mat6
U+5E13.0	fMandarin	ma4
U+5E13.0	fEnglish	[1] socks [2] head gear
U+5E14.0	fUTF8	帔
U+5E14.0	fCNS11643	2-264F
U+5E14.0	fBig5	CC6E
U+5E14.0	fGB	0-607A
U+5E14.0	fR/S	 50.5
U+5E14.0	fTotalStrokes	 8
U+5E14.0	fCangjie	LBDHE
U+5E14.0	fFourCorner	44247
U+5E14.0	fHakka	 pi1
U+5E14.0	fCantonese	pei3; pei1 (also)
U+5E14.0	fMandarin	pei4; bei1
U+5E14.0	fEnglish	 cape (usually worn by a woman)
U+5E15.0	fUTF8	帕
U+5E15.0	fCNS11643	1-4D2E
U+5E15.0	fBig5	A9AC
U+5E15.0	fGB	0-4541
U+5E15.0	fR/S	 50.5
U+5E15.0	fTotalStrokes	 8
U+5E15.0	fCangjie	LBHA
U+5E15.0	fFourCorner	46200
U+5E15.0	fHakka	  pa5
U+5E15.0	fCantonese	paak3; mak6; paa3; mat6
U+5E15.0	fMandarin	pa4; mo4; ma4
U+5E15.0	fEnglish	[1] turban [2] handkerchief [3] veil [4] wrap and bind [5] curtain made of cloth
U+5E16.0	fUTF8	帖
U+5E16.0	fCNS11643	1-4D2D
U+5E16.0	fBig5	A9AB
U+5E16.0	fGB	0-4C7B
U+5E16.0	fR/S	 50.5
U+5E16.0	fTotalStrokes	 8
U+5E16.0	fCangjie	LBYR
U+5E16.0	fFourCorner	41260
U+5E16.0	fHakka	  tiap7; diap7
U+5E16.0	fCantonese	tip3; tip2 (also)
U+5E16.0	fMandarin	tie1
U+5E16.0	fEnglish	[1] invitation card [2] label; placard; document [3] copybook (of calligraphy) [4] medical prescription
U+5E16.1	fUTF8	帖
U+5E16.1	fCNS11643	1-4D2D
U+5E16.1	fBig5	A9AB
U+5E16.1	fGB	0-4C7B
U+5E16.1	fR/S	 50.5
U+5E16.1	fTotalStrokes	 8
U+5E16.1	fCangjie	LBYR
U+5E16.1	fFourCorner	41260
U+5E16.1	fMandarin	tie3
U+5E16.1	fEnglish	[1] submissive; obedient [2] proper
U+5E16.2	fUTF8	帖
U+5E16.2	fCNS11643	1-4D2D
U+5E16.2	fBig5	A9AB
U+5E16.2	fGB	0-4C7B
U+5E16.2	fR/S	 50.5
U+5E16.2	fTotalStrokes	 8
U+5E16.2	fCangjie	LBYR
U+5E16.2	fFourCorner	41260
U+5E16.2	fMandarin	tie4
U+5E16.2	fEnglish	[1] write on silk [2] engrave characters of famous calligraphers on stone
U+5E17.0	fUTF8	帗
U+5E17.0	fCNS11643	2-264E
U+5E17.0	fBig5	CC6D
U+5E17.0	fGB	3-3768
U+5E17.0	fR/S	 50.5
U+5E17.0	fTotalStrokes	 8
U+5E17.0	fCangjie	LBIKK
U+5E17.0	fFourCorner	43247
U+5E17.0	fHakka	 fut7; pat7
U+5E17.0	fCantonese	fat1
U+5E17.0	fMandarin	fu2; ke1
U+5E17.0	fEnglish	 multicolored prop used in dancing rituals (in ancient China)
U+5E18.0	fUTF8	帘
U+5E18.0	fCNS11643	1-4D2B
U+5E18.0	fBig5	A9A9
U+5E18.0	fGB	0-4131
U+5E18.0	fR/S	 50.5
U+5E18.0	fTotalStrokes	 8
U+5E18.0	fCangjie	JCLB
U+5E18.0	fFourCorner	30227
U+5E18.0	fHakka	  liam2
U+5E18.0	fCantonese	lim4; lim2 (also)
U+5E18.0	fMandarin	lian2
U+5E18.0	fEnglish	simplified form of &#x7C3E;
U+5E19.0	fUTF8	帙
U+5E19.0	fCNS11643	2-2650
U+5E19.0	fBig5	CC6F
U+5E19.0	fGB	0-6079
U+5E19.0	fR/S	 50.5
U+5E19.0	fTotalStrokes	 8
U+5E19.0	fCangjie	LBHQO
U+5E19.0	fFourCorner	45230
U+5E19.0	fHakka	 zhit7; tsiit7; zit7
U+5E19.0	fCantonese	dit6
U+5E19.0	fMandarin	zhi4
U+5E19.0	fEnglish	book cover; book wrapper; book casing
U+5E1A.0	fUTF8	帚
U+5E1A.0	fCNS11643	1-4D2C
U+5E1A.0	fBig5	A9AA
U+5E1A.0	fGB	0-5663
U+5E1A.0	fR/S	 50.5
U+5E1A.0	fTotalStrokes	 8
U+5E1A.0	fCangjie	SMBLB
U+5E1A.0	fFourCorner	17227
U+5E1A.0	fHakka	 sau5; so5; ziu3; tsieu3; zau3; zhiu3; zhiu5; zu5; chiu3; cu3; zhu3
U+5E1A.0	fCantonese	zau2; zaau2 (also)
U+5E1A.0	fMandarin	zhou3
U+5E1A.0	fEnglish	 broom; besom
U+5E1B.0	fUTF8	帛
U+5E1B.0	fCNS11643	1-4D2F
U+5E1B.0	fBig5	A9AD
U+5E1B.0	fGB	0-322F
U+5E1B.0	fR/S	 50.5
U+5E1B.0	fTotalStrokes	 8
U+5E1B.0	fCangjie	HALB
U+5E1B.0	fFourCorner	26227
U+5E1B.0	fHakka	 pet8; pet7; piet7; pak8; tet7; det7; het7
U+5E1B.0	fCantonese	baak6
U+5E1B.0	fMandarin	bo2; he2
U+5E1B.0	fEnglish	[1] silks; silk fabrics (collectively) [2] a Chinese family name
U+5E1C.0	fUTF8	帜
U+5E1C.0	fGB	0-5644
U+5E1C.0	fR/S	 50.5
U+5E1C.0	fTotalStrokes	 8
U+5E1C.0	fCangjie	LBRC
U+5E1C.0	fHakka	 zhi5
U+5E1C.0	fCantonese	ci3
U+5E1C.0	fMandarin	zhi4; chi4
U+5E1C.0	fEnglish	simplified form of &#x5E5F;
U+5E1D.0	fUTF8	帝
U+5E1D.0	fCNS11643	1-5074
U+5E1D.0	fBig5	ABD2
U+5E1D.0	fGB	0-355B
U+5E1D.0	fR/S	 50.6
U+5E1D.0	fTotalStrokes	 9
U+5E1D.0	fCangjie	YBLB
U+5E1D.0	fFourCorner	00227
U+5E1D.0	fHakka	  di5; ti5
U+5E1D.0	fCantonese	dai3
U+5E1D.0	fMandarin	di4
U+5E1D.0	fEnglish	[1] supreme ruler; emperor [2] god; defied being [3] Heaven (as a divine being) [4] imperial
U+5E1E.0	fUTF8	帞
U+5E1E.0	fCNS11643	3-2C46
U+5E1E.0	fGB	3-376B
U+5E1E.0	fR/S	  50.6
U+5E1E.0	fTotalStrokes	 9
U+5E1E.0	fCangjie	LBMA
U+5E1E.0	fFourCorner	41260
U+5E1E.0	fCantonese	mak6
U+5E1E.0	fMandarin	mo4; mo1
U+5E1F.0	fUTF8	帟
U+5E1F.0	fCNS11643	1-5076
U+5E1F.0	fBig5	ABD4
U+5E1F.0	fGB	3-3770
U+5E1F.0	fR/S	 50.6
U+5E1F.0	fTotalStrokes	 9
U+5E1F.0	fCangjie	YCLB
U+5E1F.0	fFourCorner	00223
U+5E1F.0	fHakka	 jit8; rit8
U+5E1F.0	fCantonese	jik6
U+5E1F.0	fMandarin	yi4
U+5E1F.0	fEnglish	tent
U+5E20.0	fUTF8	帠
U+5E20.0	fCNS11643	2-2A34
U+5E20.0	fBig5	CEB3
U+5E20.0	fGB	E-2875
U+5E20.0	fR/S	 50.6
U+5E20.0	fTotalStrokes	 9
U+5E20.0	fCangjie	HXLB
U+5E20.0	fFourCorner	77227
U+5E20.0	fHakka	 ni3; ji5; ri5; ngi5
U+5E20.0	fCantonese	jit6; ngai6
U+5E20.0	fMandarin	yi4
U+5E21.0	fUTF8	帡
U+5E21.0	fCNS11643	2-2A31
U+5E21.0	fBig5	CEB0
U+5E21.0	fGB	3-3771
U+5E21.0	fR/S	 50.6
U+5E21.0	fTotalStrokes	 9
U+5E21.0	fCangjie	LBTT
U+5E21.0	fFourCorner	48241
U+5E21.0	fHakka	 pin2
U+5E21.0	fCantonese	ping4
U+5E21.0	fMandarin	ping2
U+5E21.0	fEnglish	shelter; screen; awning
U+5E22.0	fUTF8	帢
U+5E22.0	fCNS11643	2-2A32
U+5E22.0	fBig5	CEB1
U+5E22.0	fGB	3-376E
U+5E22.0	fR/S	 50.6
U+5E22.0	fTotalStrokes	 9
U+5E22.0	fCangjie	LBOMR
U+5E22.0	fFourCorner	48261
U+5E22.0	fHakka	 kap7
U+5E22.0	fCantonese	hap1
U+5E22.0	fMandarin	qia4
U+5E22.0	fEnglish	a scholar's cap used about 300 AD
U+5E23.0	fUTF8	帣
U+5E23.0	fCNS11643	2-2A33
U+5E23.0	fBig5	CEB2
U+5E23.0	fGB	3-3772
U+5E23.0	fR/S	 50.6
U+5E23.0	fTotalStrokes	 9
U+5E23.0	fCangjie	FQLB
U+5E23.0	fFourCorner	90227
U+5E23.0	fHakka	 ken5; gien5; kian5
U+5E23.0	fCantonese	gyun3; gyun2
U+5E23.0	fMandarin	juan4
U+5E23.0	fEnglish	bag
U+5E24.0	fUTF8	帤
U+5E24.0	fCNS11643	2-2A35
U+5E24.0	fBig5	CEB4
U+5E24.0	fGB	3-3773
U+5E24.0	fR/S	 50.6
U+5E24.0	fTotalStrokes	 9
U+5E24.0	fCangjie	VRLB
U+5E24.0	fFourCorner	46227
U+5E24.0	fHakka	 ji2; ri2
U+5E24.0	fCantonese	jyu4
U+5E24.0	fMandarin	ru2
U+5E24.0	fEnglish	[1] the binding at the end of a bow [2] flag
U+5E25.0	fUTF8	帥
U+5E25.0	fCNS11643	1-5075
U+5E25.0	fBig5	ABD3
U+5E25.0	fGB	1-4B27
U+5E25.0	fR/S	 50.6
U+5E25.0	fTotalStrokes	 9
U+5E25.0	fCangjie	HRLB
U+5E25.0	fFourCorner	24727
U+5E25.0	fHakka	 soi5; sai5; sut7; sai3
U+5E25.0	fCantonese	seoi3; seot1
U+5E25.0	fMandarin	shuai4; shuo4
U+5E25.0	fEnglish	[1] commander; commander-in-chief; commanding general [2] command; lead [3] follow; comply (with instructions or orders) [4] dashing; smartlooking [5] a Chinese family name
U+5E26.0	fUTF8	带
U+5E26.0	fGB	0-3478
U+5E26.0	fR/S	 50.6
U+5E26.0	fTotalStrokes	 9
U+5E26.0	fCangjie	TJBLB
U+5E26.0	fHakka	 dai5
U+5E26.0	fCantonese	daai3; daai2 (also)
U+5E26.0	fMandarin	dai4
U+5E26.0	fEnglish	simplified form of &#x5E36;
U+5E27.0	fUTF8	帧
U+5E27.0	fGB	0-5621
U+5E27.0	fR/S	 50.6
U+5E27.0	fTotalStrokes	 9
U+5E27.0	fCangjie	LBYBO
U+5E27.0	fCantonese	zing3
U+5E27.0	fMandarin	zheng4; men2; zhen1
U+5E27.0	fEnglish	simplified form of &#x5E40;
U+5E28.0	fUTF8	帨
U+5E28.0	fCNS11643	2-2E76
U+5E28.0	fBig5	D174
U+5E28.0	fGB	3-3777
U+5E28.0	fR/S	 50.7
U+5E28.0	fTotalStrokes	 10
U+5E28.0	fCangjie	LBCRU
U+5E28.0	fFourCorner	48216
U+5E28.0	fHakka	 tot7; s'oi5; soi5; choe5
U+5E28.0	fCantonese	seoi3; ceoi3
U+5E28.0	fMandarin	shui4; sui4; yue4
U+5E28.0	fEnglish	 handkerchief; kerchief
U+5E29.0	fUTF8	帩
U+5E29.0	fCNS11643	2-2E75
U+5E29.0	fBig5	D173
U+5E29.0	fGB	3-3776
U+5E29.0	fR/S	 50.7
U+5E29.0	fTotalStrokes	 10
U+5E29.0	fCangjie	LBFB
U+5E29.0	fFourCorner	49227
U+5E29.0	fCantonese	ciu3
U+5E29.0	fMandarin	qiao4; qiao1
U+5E2A.0	fUTF8	帪
U+5E2A.0	fCNS11643	3-3066
U+5E2A.0	fGB	3-3775
U+5E2A.0	fR/S	 50.7
U+5E2A.0	fTotalStrokes	 10
U+5E2A.0	fCangjie	LBMMV
U+5E2A.0	fFourCorner	41230
U+5E2A.0	fHakka	 zhin1; zhin3; zihn3; zihn1; zhen3
U+5E2A.0	fCantonese	zan1
U+5E2A.0	fMandarin	zhen1
U+5E2B.0	fUTF8	師
U+5E2B.0	fCNS11643	1-553A
U+5E2B.0	fBig5	AE76
U+5E2B.0	fGB	1-4A26
U+5E2B.0	fR/S	 50.7
U+5E2B.0	fTotalStrokes	 10
U+5E2B.0	fCangjie	HRMLB
U+5E2B.0	fFourCorner	21727
U+5E2B.0	fHakka	  sii1;  su1; se1
U+5E2B.0	fCantonese	si1
U+5E2B.0	fMandarin	shi1
U+5E2B.0	fEnglish	[1] teacher; master; tutor [2] specialist (especially of medicine; painting, music, divining, etc.) [3] division in the Chinese army; [4] army [5] model; example [6] teach [7] pattern; model after [8] local administrative chief [9] a surname
U+5E2C.0	fUTF8	帬
U+5E2C.0	fCNS11643	3-3069
U+5E2C.0	fGB	E-2876
U+5E2C.0	fR/S	 50.7
U+5E2C.0	fTotalStrokes	 10
U+5E2C.0	fCangjie	XSRLB
U+5E2C.0	fFourCorner	17220
U+5E2C.0	fCantonese	kwan4
U+5E2C.0	fMandarin	qun2
U+5E2C.0	fEnglish	the skirt of a lady's dress petticoat
U+5E2D.0	fUTF8	席
U+5E2D.0	fCNS11643	1-5539
U+5E2D.0	fBig5	AE75
U+5E2D.0	fGB	0-4F2F
U+5E2D.0	fR/S	 50.7
U+5E2D.0	fTotalStrokes	 10
U+5E2D.0	fCangjie	ITLB
U+5E2D.0	fFourCorner	00227
U+5E2D.0	fHakka	  cit8;  sit8
U+5E2D.0	fCantonese	zik6; zek6 (also)
U+5E2D.0	fMandarin	xi2
U+5E2D.0	fEnglish	[1] seat; take a seat [2] mat [3] banquet; feast [4] rely on [5] sail (rarely) [6] a Chinese family name
U+5E2E.0	fUTF8	帮
U+5E2E.0	fCNS11643	3-3068
U+5E2E.0	fGB	0-306F
U+5E2E.0	fR/S	  50.7
U+5E2E.0	fTotalStrokes	 10
U+5E2E.0	fCangjie	QLLB
U+5E2E.0	fFourCorner	57220
U+5E2E.0	fHakka	  bong1; bong3
U+5E2E.0	fCantonese	bong1
U+5E2E.0	fMandarin	bang1
U+5E2E.0	fEnglish	[1] simplified form of &#x5E6B; [2] non-classical form of &#x5E6B;
U+5E2F.0	fUTF8	帯
U+5E2F.0	fCNS11643	3-306A
U+5E2F.0	fGB	E-2877
U+5E2F.0	fR/S	 50.7
U+5E2F.0	fTotalStrokes	 10
U+5E2F.0	fCangjie	XTJBL
U+5E2F.0	fFourCorner	44220
U+5E2F.0	fHakka	 dai5
U+5E2F.0	fCantonese	daai3; daai2
U+5E2F.0	fMandarin	dai4
U+5E2F.0	fEnglish	belt
U+5E30.0	fUTF8	帰
U+5E30.0	fCNS11643	3-3067
U+5E30.0	fGB	E-2878
U+5E30.0	fR/S	 50.7; 58.7
U+5E30.0	fTotalStrokes	 10
U+5E30.0	fCangjie	LLSMB
U+5E30.0	fFourCorner	37227
U+5E30.0	fHakka	 kwui1
U+5E30.0	fCantonese	gwai1; gwai6; kwai5
U+5E30.0	fMandarin	gui1; kui4
U+5E30.0	fEnglish	return; return to, revert to
U+5E31.0	fUTF8	帱
U+5E31.0	fGB	0-607C
U+5E31.0	fR/S	 50.7
U+5E31.0	fTotalStrokes	 10
U+5E31.0	fCangjie	LBQKI
U+5E31.0	fHakka	 dau3
U+5E31.0	fCantonese	cau4; dou6; tou4 (also)
U+5E31.0	fMandarin	chou2; dao4
U+5E31.0	fEnglish	simplified form of &#x5E6C;
U+5E32.0	fUTF8	帲
U+5E32.0	fCNS11643	3-355B
U+5E32.0	fGB	E-2879
U+5E32.0	fR/S	 50.8
U+5E32.0	fTotalStrokes	 11
U+5E32.0	fCangjie	LBYJJ
U+5E32.0	fFourCorner	48240
U+5E32.0	fHakka	 pin2; ben3; bin3
U+5E32.0	fCantonese	ping4
U+5E32.0	fMandarin	ping2
U+5E32.0	fEnglish	shelter
U+5E33.0	fUTF8	帳
U+5E33.0	fCNS11643	1-5A28
U+5E33.0	fBig5	B162
U+5E33.0	fGB	1-554A
U+5E33.0	fR/S	 50.8
U+5E33.0	fTotalStrokes	 11
U+5E33.0	fCangjie	LBSMV
U+5E33.0	fFourCorner	41232
U+5E33.0	fHakka	  zong5; zhong5
U+5E33.0	fCantonese	zoeng3; zoeng2 (also)
U+5E33.0	fMandarin	zhang4
U+5E33.0	fEnglish	[1] tent; [2] screen; mosquito net; curtain [3] canopy above the bed [4] scroll (sent as a gift for a wedding, birthday party, funeral, etc.)
U+5E34.0	fUTF8	帴
U+5E34.0	fCNS11643	2-352A
U+5E34.0	fBig5	D546
U+5E34.0	fGB	3-3767
U+5E34.0	fR/S	 50.8
U+5E34.0	fTotalStrokes	 11
U+5E34.0	fCangjie	LBII
U+5E34.0	fFourCorner	43253
U+5E34.0	fCantonese	zin1; zin2
U+5E34.0	fMandarin	sha1; can2; jian1; xian1; jian3
U+5E35.0	fUTF8	帵
U+5E35.0	fCNS11643	3-3559
U+5E35.0	fGB	3-377D
U+5E35.0	fR/S	  50.8
U+5E35.0	fTotalStrokes	 11
U+5E35.0	fCangjie	LBJNU
U+5E35.0	fFourCorner	43210
U+5E35.0	fHakka	 won1; wan1; van1
U+5E35.0	fCantonese	wun1; jyun2 (also)
U+5E35.0	fMandarin	wan1
U+5E35.0	fEnglish	remnants; tailor's cutting
U+5E36.0	fUTF8	帶
U+5E36.0	fCNS11643	1-5A27
U+5E36.0	fBig5	B161
U+5E36.0	fGB	1-3478
U+5E36.0	fR/S	 50.8
U+5E36.0	fTotalStrokes	 11
U+5E36.0	fCangjie	KPBLB
U+5E36.0	fFourCorner	44227
U+5E36.0	fHakka	  dai5
U+5E36.0	fCantonese	daai3; daai2 (also)
U+5E36.0	fMandarin	dai4
U+5E36.0	fEnglish	 belt; girdle; sash; band; ribbon; string; tie
U+5E37.0	fUTF8	帷
U+5E37.0	fCNS11643	1-5A29
U+5E37.0	fBig5	B163
U+5E37.0	fGB	0-6121
U+5E37.0	fR/S	 50.8
U+5E37.0	fTotalStrokes	 11
U+5E37.0	fCangjie	LBOG
U+5E37.0	fFourCorner	40214
U+5E37.0	fHakka	 wui2; wi2; wui3; vui2
U+5E37.0	fCantonese	wai4
U+5E37.0	fMandarin	wei2
U+5E37.0	fEnglish	 tent; curtain; screen
U+5E38.0	fUTF8	常
U+5E38.0	fCNS11643	1-5A26
U+5E38.0	fBig5	B160
U+5E38.0	fGB	0-3323
U+5E38.0	fR/S	 50.8
U+5E38.0	fTotalStrokes	 11
U+5E38.0	fCangjie	FBRLB
U+5E38.0	fFourCorner	90227
U+5E38.0	fHakka	  song2;  shong2; shong3
U+5E38.0	fCantonese	soeng4
U+5E38.0	fMandarin	chang2
U+5E38.0	fEnglish	[1] common; normal; ordinary [2] frequent; frequently; regular; regularly; often [3] long; lasting; permanent (jobs, etc.); eternal [4] ordinarily; usually; on ordinary occasions [5] rule; principle [6] a Chinese family name
U+5E39.0	fUTF8	帹
U+5E39.0	fCNS11643	3-3558
U+5E39.0	fGB	3-377B
U+5E39.0	fR/S	  50.8
U+5E39.0	fTotalStrokes	 11
U+5E39.0	fCangjie	LBYTV
U+5E39.0	fFourCorner	40240
U+5E39.0	fCantonese	saap3
U+5E39.0	fMandarin	sha4
U+5E3A.0	fUTF8	帺
U+5E3A.0	fCNS11643	3-355A
U+5E3A.0	fGB	3-3779
U+5E3A.0	fR/S	  50.8
U+5E3A.0	fTotalStrokes	 11
U+5E3A.0	fCangjie	LBTMC
U+5E3A.0	fFourCorner	44280
U+5E3A.0	fCantonese	kei4
U+5E3A.0	fMandarin	qi2
U+5E3B.0	fUTF8	帻
U+5E3B.0	fGB	0-607D
U+5E3B.0	fR/S	 50.8
U+5E3B.0	fTotalStrokes	 11
U+5E3B.0	fCangjie	LBQMO
U+5E3B.0	fHakka	 zit7
U+5E3B.0	fCantonese	zik1; zaak3 (also)
U+5E3B.0	fMandarin	ze2; ze1
U+5E3B.0	fEnglish	simplified form of &#x5E58;
U+5E3C.0	fUTF8	帼
U+5E3C.0	fGB	0-607E
U+5E3C.0	fR/S	 50.8
U+5E3C.0	fTotalStrokes	 11
U+5E3C.0	fCangjie	LBWMI
U+5E3C.0	fHakka	 kwet7
U+5E3C.0	fCantonese	gwok3; gok3 (also); kok3
U+5E3C.0	fMandarin	guo2; gui1; gui4
U+5E3C.0	fEnglish	simplified form of &#x5E57;
U+5E3D.0	fUTF8	帽
U+5E3D.0	fCNS11643	1-5E7A
U+5E3D.0	fBig5	B455
U+5E3D.0	fGB	0-4331
U+5E3D.0	fR/S	 50.9
U+5E3D.0	fTotalStrokes	 12
U+5E3D.0	fCangjie	LBABU
U+5E3D.0	fFourCorner	46260
U+5E3D.0	fHakka	 mau5; mo6; mau3; mo5
U+5E3D.0	fCantonese	mou6; mou2 (also)
U+5E3D.0	fMandarin	mao4
U+5E3D.0	fEnglish	[1] hat; headwear; cap-like tops [2] cap (of a fountain pen, screw, etc.)
U+5E3E.0	fUTF8	帾
U+5E3E.0	fCNS11643	2-3529
U+5E3E.0	fBig5	D545
U+5E3E.0	fGB	3-3778
U+5E3E.0	fR/S	 50.9
U+5E3E.0	fTotalStrokes	 12
U+5E3E.0	fCangjie	LBJKA
U+5E3E.0	fFourCorner	44260
U+5E3E.0	fCantonese	dou2
U+5E3E.0	fMandarin	du3
U+5E3F.0	fUTF8	帿
U+5E3F.0	fCNS11643	5-377C
U+5E3F.0	fGB	E-287A
U+5E3F.0	fR/S	  50.9
U+5E3F.0	fTotalStrokes	 12
U+5E3F.0	fCangjie	LBONK
U+5E3F.0	fFourCorner	47230
U+5E3F.0	fCantonese	hau4
U+5E3F.0	fMandarin	hou2
U+5E3F.0	fEnglish	 marquis; lord target in archery
U+5E40.0	fUTF8	幀
U+5E40.0	fCNS11643	1-5E7B
U+5E40.0	fBig5	B456
U+5E40.0	fGB	1-5621
U+5E40.0	fR/S	 50.9
U+5E40.0	fTotalStrokes	 12
U+5E40.0	fCangjie	LBYBC
U+5E40.0	fFourCorner	41286
U+5E40.0	fHakka	 zin1; zin5
U+5E40.0	fCantonese	zing3
U+5E40.0	fMandarin	zheng4; men2; zhen1
U+5E40.0	fEnglish	[1] a noun classifier for paintings, pictures, photos, etc. [2] one of a pair (as of scrolls)
U+5E41.0	fUTF8	幁
U+5E41.0	fCNS11643	2-3B57
U+5E41.0	fBig5	D8F3
U+5E41.0	fGB	E-287B
U+5E41.0	fR/S	 50.9
U+5E41.0	fTotalStrokes	 12
U+5E41.0	fCangjie	LBMBC
U+5E41.0	fFourCorner	41286
U+5E41.0	fCantonese	seoi1
U+5E41.0	fMandarin	xu1
U+5E42.0	fUTF8	幂
U+5E42.0	fCNS11643	3-3979
U+5E42.0	fGB	0-435D
U+5E42.0	fR/S	 14.10; 50.9; 14.9
U+5E42.0	fTotalStrokes	 12
U+5E42.0	fCangjie	BAKB
U+5E42.0	fFourCorner	37220
U+5E42.0	fHakka	 met7; met8
U+5E42.0	fCantonese	mik6
U+5E42.0	fMandarin	mi4
U+5E42.0	fEnglish	cover-cloth; cover with cloth
U+5E43.0	fUTF8	幃
U+5E43.0	fCNS11643	1-5E7C
U+5E43.0	fBig5	B457
U+5E43.0	fGB	1-6078
U+5E43.0	fR/S	 50.9
U+5E43.0	fTotalStrokes	 12
U+5E43.0	fCangjie	LBDMQ
U+5E43.0	fFourCorner	44256
U+5E43.0	fHakka	 wi2; wui2; wui3; vui2
U+5E43.0	fCantonese	wai4
U+5E43.0	fMandarin	wei2; hui1
U+5E43.0	fEnglish	[1] curtain [2] perfume bag
U+5E44.0	fUTF8	幄
U+5E44.0	fCNS11643	2-3B56
U+5E44.0	fBig5	D8F2
U+5E44.0	fGB	0-6122
U+5E44.0	fR/S	 50.9
U+5E44.0	fTotalStrokes	 12
U+5E44.0	fCangjie	LBSMG
U+5E44.0	fFourCorner	47214
U+5E44.0	fHakka	  wok7; vok7
U+5E44.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+5E44.0	fMandarin	wo4
U+5E44.0	fEnglish	 big tent
U+5E45.0	fUTF8	幅
U+5E45.0	fCNS11643	1-5E79
U+5E45.0	fBig5	B454
U+5E45.0	fGB	0-3779
U+5E45.0	fR/S	 50.9
U+5E45.0	fTotalStrokes	 12
U+5E45.0	fCangjie	LBMRW
U+5E45.0	fFourCorner	41266
U+5E45.0	fHakka	  fuk7; buk7
U+5E45.0	fCantonese	fuk1
U+5E45.0	fMandarin	fu2; bi1
U+5E45.0	fEnglish	[1] the breadth of cloth or paper; a width (of cloth) [2] border [3] a noun classifier for pictures, scrolls, etc.
U+5E46.0	fUTF8	幆
U+5E46.0	fCNS11643	3-3B3A
U+5E46.0	fGB	3-3824
U+5E46.0	fR/S	  50.9
U+5E46.0	fTotalStrokes	 12
U+5E46.0	fCangjie	LBAPV
U+5E46.0	fFourCorner	46220
U+5E46.0	fCantonese	ai3; ngai3 (also)
U+5E46.0	fMandarin	yi4
U+5E47.0	fUTF8	幇
U+5E47.0	fCNS11643	3-3B3B
U+5E47.0	fGB	E-287C
U+5E47.0	fR/S	 50.9
U+5E47.0	fTotalStrokes	 12
U+5E47.0	fCangjie	GILB
U+5E47.0	fFourCorner	44220
U+5E47.0	fHakka	  bong1
U+5E47.0	fCantonese	bong1
U+5E47.0	fMandarin	bang1
U+5E47.0	fEnglish	same as &#x5E6B;
U+5E48.0	fUTF8	幈
U+5E48.0	fCNS11643	3-3B3D
U+5E48.0	fGB	5-3A40
U+5E48.0	fR/S	 50.11; 50.9
U+5E48.0	fTotalStrokes	 14; 12
U+5E48.0	fCangjie	LBSTT
U+5E48.0	fFourCorner	47240
U+5E48.0	fCantonese	ping4
U+5E48.0	fMandarin	ping2
U+5E49.0	fUTF8	幉
U+5E49.0	fGB	E-287D
U+5E49.0	fR/S	 50.9
U+5E49.0	fTotalStrokes	 12
U+5E49.0	fCangjie	LBPTD
U+5E49.0	fCantonese	jip6
U+5E4A.0	fUTF8	幊
U+5E4A.0	fCNS11643	2-4260
U+5E4A.0	fBig5	DD5A
U+5E4A.0	fGB	E-287E
U+5E4A.0	fR/S	 50.10
U+5E4A.0	fTotalStrokes	 13
U+5E4A.0	fCangjie	LBMBC
U+5E4A.0	fFourCorner	41086
U+5E4A.0	fCantonese	gung1
U+5E4A.0	fMandarin	gong1
U+5E4B.0	fUTF8	幋
U+5E4B.0	fCNS11643	2-4262
U+5E4B.0	fBig5	DD5C
U+5E4B.0	fGB	5-3A43
U+5E4B.0	fR/S	 50.10
U+5E4B.0	fTotalStrokes	 13
U+5E4B.0	fCangjie	HELB
U+5E4B.0	fFourCorner	27227
U+5E4B.0	fCantonese	pun4
U+5E4B.0	fMandarin	pan2
U+5E4B.0	fEnglish	belt
U+5E4C.0	fUTF8	幌
U+5E4C.0	fCNS11643	1-636B
U+5E4C.0	fBig5	B745
U+5E4C.0	fGB	0-3B4F
U+5E4C.0	fR/S	 50.10
U+5E4C.0	fTotalStrokes	 13
U+5E4C.0	fCangjie	LBAFU
U+5E4C.0	fFourCorner	46211
U+5E4C.0	fHakka	 fong3
U+5E4C.0	fCantonese	fong2
U+5E4C.0	fMandarin	huang3
U+5E4C.0	fEnglish	 curtain; cloth screen; strip of cloth
U+5E4D.0	fUTF8	幍
U+5E4D.0	fCNS11643	2-4261
U+5E4D.0	fBig5	DD5B
U+5E4D.0	fGB	5-3A44
U+5E4D.0	fR/S	 50.10
U+5E4D.0	fTotalStrokes	 13
U+5E4D.0	fCangjie	LBBHX
U+5E4D.0	fFourCorner	42277
U+5E4D.0	fCantonese	tou1
U+5E4D.0	fMandarin	dao1; tao1
U+5E4E.0	fUTF8	幎
U+5E4E.0	fCNS11643	2-425F
U+5E4E.0	fBig5	DD59
U+5E4E.0	fGB	3-3831
U+5E4E.0	fR/S	 50.10
U+5E4E.0	fTotalStrokes	 13
U+5E4E.0	fCangjie	LBBAC
U+5E4E.0	fFourCorner	47280
U+5E4E.0	fHakka	  met7
U+5E4E.0	fCantonese	mik6
U+5E4E.0	fMandarin	mi4; mi1; mian2; mian4; ming2
U+5E4E.0	fEnglish	cover-cloth; cover with cloth
U+5E4F.0	fUTF8	幏
U+5E4F.0	fCNS11643	2-425E
U+5E4F.0	fBig5	DD58
U+5E4F.0	fGB	3-3830
U+5E4F.0	fR/S	 50.10
U+5E4F.0	fTotalStrokes	 13
U+5E4F.0	fCangjie	LBJMO
U+5E4F.0	fFourCorner	43232
U+5E4F.0	fCantonese	gaa3
U+5E4F.0	fMandarin	jia1; jia4
U+5E50.0	fUTF8	幐
U+5E50.0	fCNS11643	4-3D2A
U+5E50.0	fGB	3-4D56
U+5E50.0	fR/S	 50.10; 130.9
U+5E50.0	fTotalStrokes	 13
U+5E50.0	fCangjie	BFQB
U+5E50.0	fFourCorner	79227
U+5E50.0	fCantonese	tang4
U+5E50.0	fMandarin	teng2; teng4
U+5E51.0	fUTF8	幑
U+5E51.0	fCNS11643	4-4335
U+5E51.0	fGB	E-2921
U+5E51.0	fR/S	 50.11
U+5E51.0	fTotalStrokes	 14
U+5E51.0	fCangjie	HOUBK
U+5E51.0	fFourCorner	28240
U+5E51.0	fCantonese	fai1
U+5E51.0	fMandarin	hui1
U+5E52.0	fUTF8	幒
U+5E52.0	fCNS11643	3-465D
U+5E52.0	fGB	3-3834
U+5E52.0	fR/S	 50.11
U+5E52.0	fTotalStrokes	 14
U+5E52.0	fCangjie	LBHWP
U+5E52.0	fFourCorner	46230
U+5E52.0	fCantonese	zung1
U+5E52.0	fMandarin	zhong1
U+5E53.0	fUTF8	幓
U+5E53.0	fCNS11643	2-4979
U+5E53.0	fBig5	E1B4
U+5E53.0	fGB	3-3821
U+5E53.0	fR/S	 50.11
U+5E53.0	fTotalStrokes	 14
U+5E53.0	fCangjie	LBIIH
U+5E53.0	fFourCorner	43222
U+5E53.0	fCantonese	saam1
U+5E53.0	fMandarin	shan1; sen1; qiao1; shan4
U+5E54.0	fUTF8	幔
U+5E54.0	fCNS11643	1-685F
U+5E54.0	fBig5	B9F7
U+5E54.0	fGB	0-6123
U+5E54.0	fR/S	 50.11
U+5E54.0	fTotalStrokes	 14
U+5E54.0	fCangjie	LBAWE
U+5E54.0	fFourCorner	46247
U+5E54.0	fHakka	 man5; man3; man6
U+5E54.0	fCantonese	maan6; maam6
U+5E54.0	fMandarin	man4
U+5E54.0	fEnglish	[1] curtain; screen [2] tent
U+5E55.0	fUTF8	幕
U+5E55.0	fCNS11643	1-685D
U+5E55.0	fBig5	B9F5
U+5E55.0	fGB	0-443B
U+5E55.0	fR/S	 50.11
U+5E55.0	fTotalStrokes	 14
U+5E55.0	fCangjie	TAKB
U+5E55.0	fFourCorner	44227
U+5E55.0	fHakka	  mok7
U+5E55.0	fCantonese	mok6
U+5E55.0	fMandarin	mu4; man4; mo4
U+5E55.0	fEnglish	[1] curtain; screen [2] tent [3] advisor; staffs; private secretaries
U+5E56.0	fUTF8	幖
U+5E56.0	fCNS11643	4-4330
U+5E56.0	fGB	3-3833
U+5E56.0	fR/S	  50.11
U+5E56.0	fTotalStrokes	 14
U+5E56.0	fCangjie	LBMWF
U+5E56.0	fFourCorner	41290
U+5E56.0	fHakka	  biau1; beu1
U+5E56.0	fCantonese	biu1
U+5E56.0	fMandarin	biao1
U+5E56.0	fEnglish	 streamer; pennant
U+5E57.0	fUTF8	幗
U+5E57.0	fCNS11643	1-685E
U+5E57.0	fBig5	B9F6
U+5E57.0	fGB	1-607E
U+5E57.0	fR/S	 50.11
U+5E57.0	fTotalStrokes	 14
U+5E57.0	fCangjie	LBWIM
U+5E57.0	fFourCorner	46200
U+5E57.0	fHakka	 kwet7; get7
U+5E57.0	fCantonese	gwok3; gok3 (also); kok3
U+5E57.0	fMandarin	guo2; gui1; gui4
U+5E57.0	fEnglish	 women's headgear; women's headdress
U+5E58.0	fUTF8	幘
U+5E58.0	fCNS11643	2-4977
U+5E58.0	fBig5	E1B2
U+5E58.0	fGB	1-607D
U+5E58.0	fR/S	 50.11
U+5E58.0	fTotalStrokes	 14
U+5E58.0	fCangjie	LBQMC
U+5E58.0	fFourCorner	45286
U+5E58.0	fHakka	 zit7
U+5E58.0	fCantonese	zik1; zaak3 (also)
U+5E58.0	fMandarin	ze2; ze1
U+5E58.0	fEnglish	 turban; headdress
U+5E59.0	fUTF8	幙
U+5E59.0	fCNS11643	2-4978
U+5E59.0	fBig5	E1B3
U+5E59.0	fGB	E-2922
U+5E59.0	fR/S	 50.11
U+5E59.0	fTotalStrokes	 14
U+5E59.0	fCangjie	LBTAK
U+5E59.0	fFourCorner	44234
U+5E59.0	fHakka	 mok7
U+5E59.0	fCantonese	mok6
U+5E59.0	fMandarin	mu4; mo2
U+5E59.0	fEnglish	a curtain; a screen
U+5E5A.0	fUTF8	幚
U+5E5A.0	fCNS11643	3-4B6D
U+5E5A.0	fGB	E-2923
U+5E5A.0	fR/S	 50.12
U+5E5A.0	fTotalStrokes	 15
U+5E5A.0	fCangjie	QLHAB
U+5E5A.0	fFourCorner	57220
U+5E5A.0	fHakka	 bong1
U+5E5A.0	fCantonese	bong1
U+5E5A.0	fMandarin	bang1
U+5E5A.0	fEnglish	corrupted form of &#x5E6B;
U+5E5B.0	fUTF8	幛
U+5E5B.0	fCNS11643	1-685B
U+5E5B.0	fBig5	B9F3
U+5E5B.0	fGB	0-6124
U+5E5B.0	fR/S	 50.11
U+5E5B.0	fTotalStrokes	 14
U+5E5B.0	fCangjie	LBYTJ
U+5E5B.0	fFourCorner	40246
U+5E5B.0	fHakka	 zong5
U+5E5B.0	fCantonese	zoeng3
U+5E5B.0	fMandarin	zhang4
U+5E5B.0	fEnglish	 scroll of silk or cloth mounted or embroidered with appropriate wording sent as a gift for a wedding, funerals, etc.
U+5E5C.0	fUTF8	幜
U+5E5C.0	fCNS11643	2-503A
U+5E5C.0	fBig5	E571
U+5E5C.0	fGB	5-3A4B
U+5E5C.0	fR/S	 50.12
U+5E5C.0	fTotalStrokes	 15
U+5E5C.0	fCangjie	LBAYF
U+5E5C.0	fFourCorner	46296
U+5E5C.0	fCantonese	ging2
U+5E5C.0	fMandarin	jiong3; jing3
U+5E5D.0	fUTF8	幝
U+5E5D.0	fCNS11643	2-5038
U+5E5D.0	fBig5	E56F
U+5E5D.0	fGB	3-377C
U+5E5D.0	fR/S	 50.12
U+5E5D.0	fTotalStrokes	 15
U+5E5D.0	fCangjie	LBRRJ
U+5E5D.0	fFourCorner	46256
U+5E5D.0	fHakka	 shan3; san3; shen5; shan5; shen3
U+5E5D.0	fCantonese	cin2; zin2
U+5E5D.0	fMandarin	chan3; chan1; chan4; tan1
U+5E5D.0	fEnglish	an old worn-out carriage
U+5E5E.0	fUTF8	幞
U+5E5E.0	fCNS11643	3-4B70
U+5E5E.0	fGB	0-6125
U+5E5E.0	fR/S	 50.12
U+5E5E.0	fTotalStrokes	 15
U+5E5E.0	fCangjie	LBTCO
U+5E5E.0	fFourCorner	42230
U+5E5E.0	fHakka	  puk7
U+5E5E.0	fCantonese	fuk6; buk6 (also)
U+5E5E.0	fMandarin	fu2; pu2
U+5E5E.0	fEnglish	 turban; headdress; scarf
U+5E5F.0	fUTF8	幟
U+5E5F.0	fCNS11643	1-6C57
U+5E5F.0	fBig5	BC6D
U+5E5F.0	fGB	1-5644
U+5E5F.0	fR/S	 50.12
U+5E5F.0	fTotalStrokes	 15
U+5E5F.0	fCangjie	LBYIA
U+5E5F.0	fFourCorner	43250
U+5E5F.0	fHakka	 zit7; zhi5; zhit7; zi5; zih5; tje5
U+5E5F.0	fCantonese	ci3
U+5E5F.0	fMandarin	zhi4; chi4
U+5E5F.0	fEnglish	[1] flag; pennant; pennon [2] sign; mark
U+5E60.0	fUTF8	幠
U+5E60.0	fCNS11643	2-5039
U+5E60.0	fBig5	E570
U+5E60.0	fGB	3-3763
U+5E60.0	fR/S	 50.12
U+5E60.0	fTotalStrokes	 15
U+5E60.0	fCangjie	LBOTF
U+5E60.0	fFourCorner	48231
U+5E60.0	fHakka	  fu1; wu1
U+5E60.0	fCantonese	fu1
U+5E60.0	fMandarin	hu1; wu3
U+5E60.0	fEnglish	cover
U+5E61.0	fUTF8	幡
U+5E61.0	fCNS11643	1-6C58
U+5E61.0	fBig5	BC6E
U+5E61.0	fGB	0-6126
U+5E61.0	fR/S	 50.12
U+5E61.0	fTotalStrokes	 15
U+5E61.0	fCangjie	LBHDW
U+5E61.0	fFourCorner	42269
U+5E61.0	fHakka	 fan1
U+5E61.0	fCantonese	faan1
U+5E61.0	fMandarin	fan1; fan2
U+5E61.0	fEnglish	[1] flag; pennant; pennon; banner; streamer [2] sudden; suddenly
U+5E62.0	fUTF8	幢
U+5E62.0	fCNS11643	1-6C56
U+5E62.0	fBig5	BC6C
U+5E62.0	fGB	0-3431
U+5E62.0	fR/S	 50.12
U+5E62.0	fTotalStrokes	 15
U+5E62.0	fCangjie	LBYTG
U+5E62.0	fFourCorner	40214
U+5E62.0	fHakka	  tung2; cong2; cong5; cong3; cong6
U+5E62.0	fCantonese	cong4; tong4; zong6
U+5E62.0	fMandarin	chuang2; zhuang4; chuan2; tong2; zhuan4
U+5E62.0	fEnglish	[1] carriage curtain [2] a noun classifier for buildings of more than one story [3] flags; pennants; streamers [4] flickering; waving
U+5E63.0	fUTF8	幣
U+5E63.0	fCNS11643	1-685C
U+5E63.0	fBig5	B9F4
U+5E63.0	fGB	1-3152
U+5E63.0	fR/S	 50.12
U+5E63.0	fTotalStrokes	 15
U+5E63.0	fCangjie	FKLB
U+5E63.0	fFourCorner	98227
U+5E63.0	fHakka	  bi5; pet7
U+5E63.0	fCantonese	bai6
U+5E63.0	fMandarin	bi4
U+5E63.0	fEnglish	[1] currency; money; coins; legal tender [2] present; offering
U+5E64.0	fUTF8	幤
U+5E64.0	fCNS11643	F-4E5E
U+5E64.0	fGB	E-2924
U+5E64.0	fR/S	 50.12
U+5E64.0	fTotalStrokes	 15
U+5E64.0	fCangjie	XFKLB
U+5E64.0	fCantonese	bai6
U+5E64.0	fMandarin	bi4
U+5E64.0	fEnglish	 evil, wrong, bad; criminal
U+5E65.0	fUTF8	幥
U+5E65.0	fCNS11643	3-6473
U+5E65.0	fGB	E-2925
U+5E65.0	fR/S	 50.12
U+5E65.0	fTotalStrokes	 15
U+5E65.0	fCangjie	LBFBQ
U+5E65.0	fFourCorner	4925
U+5E65.0	fCantonese	zoeng2
U+5E66.0	fUTF8	幦
U+5E66.0	fCNS11643	2-5677
U+5E66.0	fBig5	E96D
U+5E66.0	fGB	3-3839
U+5E66.0	fR/S	 50.13
U+5E66.0	fTotalStrokes	 16
U+5E66.0	fCangjie	SJLB
U+5E66.0	fFourCorner	70227
U+5E66.0	fHakka	  met7
U+5E66.0	fCantonese	mik6; mat6; mik2
U+5E66.0	fMandarin	mi4; mi1
U+5E66.0	fEnglish	cover
U+5E67.0	fUTF8	幧
U+5E67.0	fCNS11643	2-5675
U+5E67.0	fBig5	E96B
U+5E67.0	fGB	5-3A4E
U+5E67.0	fR/S	 50.13
U+5E67.0	fTotalStrokes	 16
U+5E67.0	fCangjie	LBRRD
U+5E67.0	fFourCorner	46294
U+5E67.0	fHakka	  ciau1;  cau1
U+5E67.0	fCantonese	ciu1; ciu4
U+5E67.0	fMandarin	qiao1; cao1
U+5E67.0	fEnglish	a kind of turban
U+5E68.0	fUTF8	幨
U+5E68.0	fCNS11643	2-5676
U+5E68.0	fBig5	E96C
U+5E68.0	fGB	3-3838
U+5E68.0	fR/S	 50.13
U+5E68.0	fTotalStrokes	 16
U+5E68.0	fCangjie	LBNCR
U+5E68.0	fFourCorner	47261
U+5E68.0	fHakka	 zham1; zam1
U+5E68.0	fCantonese	cim1; zim1 (also); cim3
U+5E68.0	fMandarin	chan1; chan4; dan4
U+5E68.0	fEnglish	curtain
U+5E69.0	fUTF8	幩
U+5E69.0	fCNS11643	2-5037
U+5E69.0	fBig5	E56E
U+5E69.0	fGB	3-3822
U+5E69.0	fR/S	 50.13
U+5E69.0	fTotalStrokes	 16
U+5E69.0	fCangjie	LBJTC
U+5E69.0	fFourCorner	40286
U+5E69.0	fHakka	 fun2
U+5E69.0	fCantonese	fan4
U+5E69.0	fMandarin	fen2; bi4; fen4
U+5E69.0	fEnglish	trappings
U+5E6A.0	fUTF8	幪
U+5E6A.0	fCNS11643	2-5C68
U+5E6A.0	fBig5	ECDC
U+5E6A.0	fGB	3-3837
U+5E6A.0	fR/S	 50.14
U+5E6A.0	fTotalStrokes	 17
U+5E6A.0	fCangjie	LBTBO
U+5E6A.0	fFourCorner	44232
U+5E6A.0	fHakka	 mung2; miang1
U+5E6A.0	fCantonese	mung4; mung2; mung5
U+5E6A.0	fMandarin	meng2; meng3
U+5E6A.0	fEnglish	 cover; screen
U+5E6B.0	fUTF8	幫
U+5E6B.0	fCNS11643	1-7359
U+5E6B.0	fBig5	C0B0
U+5E6B.0	fGB	1-306F
U+5E6B.0	fR/S	 50.14
U+5E6B.0	fTotalStrokes	 17
U+5E6B.0	fCangjie	GIHAB
U+5E6B.0	fFourCorner	44227
U+5E6B.0	fHakka	  bong1
U+5E6B.0	fCantonese	bong1
U+5E6B.0	fMandarin	bang1
U+5E6B.0	fEnglish	[1] help; assist [2] gang; group; class; fleet [3] the sides of a shoe or gutter
U+5E6C.0	fUTF8	幬
U+5E6C.0	fCNS11643	2-5C67
U+5E6C.0	fBig5	ECDB
U+5E6C.0	fGB	1-607C
U+5E6C.0	fR/S	 50.14
U+5E6C.0	fTotalStrokes	 17
U+5E6C.0	fCangjie	LBGNI
U+5E6C.0	fFourCorner	44241
U+5E6C.0	fHakka	  dau3; do3; cu2; chiu2
U+5E6C.0	fCantonese	cau4; dou6; tou4 (also)
U+5E6C.0	fMandarin	chou2
U+5E6C.0	fEnglish	 cover up
U+5E6C.1	fUTF8	幬
U+5E6C.1	fCNS11643	2-5C67
U+5E6C.1	fBig5	ECDB
U+5E6C.1	fGB	1-607C
U+5E6C.1	fR/S	 50.14
U+5E6C.1	fTotalStrokes	 17
U+5E6C.1	fCangjie	LBGNI
U+5E6C.1	fFourCorner	44241
U+5E6C.1	fMandarin	dao4
U+5E6C.1	fEnglish	 curtain; canopy
U+5E6D.0	fUTF8	幭
U+5E6D.0	fCNS11643	2-6151
U+5E6D.0	fBig5	EFC5
U+5E6D.0	fGB	3-383A
U+5E6D.0	fR/S	 50.15
U+5E6D.0	fTotalStrokes	 18
U+5E6D.0	fCangjie	LBTWI
U+5E6D.0	fFourCorner	44253
U+5E6D.0	fHakka	  met8
U+5E6D.0	fCantonese	mit6; mik6
U+5E6D.0	fMandarin	mie4
U+5E6D.0	fEnglish	cover
U+5E6E.0	fUTF8	幮
U+5E6E.0	fCNS11643	2-6152
U+5E6E.0	fBig5	EFC6
U+5E6E.0	fGB	E-2926
U+5E6E.0	fR/S	 50.15
U+5E6E.0	fTotalStrokes	 18
U+5E6E.0	fCangjie	LBIGI
U+5E6E.0	fFourCorner	40240
U+5E6E.0	fHakka	 chu2; cu2
U+5E6E.0	fCantonese	cyu4; ceoi4 (also); cyu1; ceoi1
U+5E6E.0	fMandarin	chu2
U+5E6E.0	fEnglish	[1] bed-screen or mosquito net that looks like a small cabinet [2] screen put up to make a temporary kitchen
U+5E6F.0	fUTF8	幯
U+5E6F.0	fCNS11643	2-5678
U+5E6F.0	fBig5	E96E
U+5E6F.0	fGB	5-3A2A
U+5E6F.0	fR/S	 50.15
U+5E6F.0	fTotalStrokes	 18
U+5E6F.0	fCangjie	LBHAL
U+5E6F.0	fFourCorner	48227
U+5E6F.0	fCantonese	zit3
U+5E6F.0	fMandarin	jie2; jie1
U+5E70.0	fUTF8	幰
U+5E70.0	fCNS11643	2-654A
U+5E70.0	fBig5	F1FE
U+5E70.0	fGB	3-383B
U+5E70.0	fR/S	 50.16
U+5E70.0	fTotalStrokes	 19
U+5E70.0	fCangjie	LBJQP
U+5E70.0	fFourCorner	43236
U+5E70.0	fHakka	 hien3
U+5E70.0	fCantonese	hin2; hin3
U+5E70.0	fMandarin	xian3
U+5E70.0	fEnglish	curtain at front of carriage
U+5E71.0	fUTF8	幱
U+5E71.0	fCNS11643	3-6677
U+5E71.0	fGB	5-3A4C
U+5E71.0	fR/S	 50.17
U+5E71.0	fTotalStrokes	 20
U+5E71.0	fCangjie	LBANW
U+5E71.0	fFourCorner	4722
U+5E71.0	fCantonese	laan4
U+5E71.0	fMandarin	lan2
U+5E71.0	fEnglish	official dress
U+5E72.0	fUTF8	干
U+5E72.0	fCNS11643	1-445B
U+5E72.0	fBig5	A47A
U+5E72.0	fGB	0-3849
U+5E72.0	fR/S	 51.0
U+5E72.0	fTotalStrokes	 3
U+5E72.0	fCangjie	MJ
U+5E72.0	fFourCorner	10400
U+5E72.0	fHakka	  gon1; gon5
U+5E72.0	fCantonese	gon1; gon3
U+5E72.0	fMandarin	gan4; han2; guan3
U+5E72.0	fEnglish	[1] oppose; offend; invade [2] jam (radio broadcasts, etc.); interfere; intervene [3] concern; involve [4] seek; beseech [5] bank (of a river, etc.) [6] shield [7] (how) many; (how) much; group [8] stem; the celestial stems [9] a Chinese family name [10] simplified form of &#x5E79;
U+5E72.1	fUTF8	干
U+5E72.1	fCNS11643	1-445B
U+5E72.1	fBig5	A47A
U+5E72.1	fGB	0-3849
U+5E72.1	fR/S	 51.0
U+5E72.1	fTotalStrokes	 3
U+5E72.1	fCangjie	MJ
U+5E72.1	fFourCorner	10400
U+5E72.1	fMandarin	gan1
U+5E73.0	fUTF8	平
U+5E73.0	fCNS11643	1-464D
U+5E73.0	fBig5	A5AD
U+5E73.0	fGB	0-463D
U+5E73.0	fR/S	 51.2
U+5E73.0	fTotalStrokes	 5
U+5E73.0	fCangjie	MFJ
U+5E73.0	fFourCorner	10409
U+5E73.0	fHakka	  pin2;  piang2; pe2; pin3; piang3
U+5E73.0	fCantonese	ping4; peng4 (also)
U+5E73.0	fMandarin	ping2; bing4; pian2
U+5E73.0	fEnglish	[1] flat; level; even [2] peaceful; amity [3] equal; equality [4] conquer; quell (a revolt); calm down [5] control; regulate [6] go back to normal after sharp rises (said of prices) [7] make the same score; tie; draw (sports) draw [8] pacifiy; bring peace to [9] short for Peiping [10] a Chinese family name
U+5E74.0	fUTF8	年
U+5E74.0	fCNS11643	1-4821
U+5E74.0	fBig5	A67E
U+5E74.0	fGB	0-446A
U+5E74.0	fR/S	 51.3
U+5E74.0	fTotalStrokes	 6
U+5E74.0	fCangjie	OQ
U+5E74.0	fFourCorner	80500
U+5E74.0	fHakka	 ngien2; ngen2; gnian3; ngian2
U+5E74.0	fCantonese	nin4; lin4 (also); nin2 (also); lin2 (also)
U+5E74.0	fMandarin	nian2
U+5E74.0	fEnglish	[1] year [2] person's age [3] a Chinese family name
U+5E75.0	fUTF8	幵
U+5E75.0	fCNS11643	2-223B
U+5E75.0	fBig5	C9DB
U+5E75.0	fGB	E-2927
U+5E75.0	fR/S	 51.3
U+5E75.0	fTotalStrokes	 6
U+5E75.0	fCangjie	MJMJ
U+5E75.0	fFourCorner	10440
U+5E75.0	fHakka	 gien1; gian1; gen1
U+5E75.0	fCantonese	hin1; gin1
U+5E75.0	fMandarin	jian1; qian1
U+5E75.0	fEnglish	[1] even level [2] raise in both hands
U+5E76.0	fUTF8	并
U+5E76.0	fCNS11643	1-477E
U+5E76.0	fBig5	A67D
U+5E76.0	fGB	0-3222
U+5E76.0	fR/S	 51.5
U+5E76.0	fTotalStrokes	 8
U+5E76.0	fCangjie	TT
U+5E76.0	fFourCorner	80441
U+5E76.0	fHakka	 bin5
U+5E76.0	fCantonese	bing6; bing3; bing1; ping3 (also)
U+5E76.0	fMandarin	bing4; bing1
U+5E76.0	fEnglish	 combine; annex also; whats more
U+5E77.0	fUTF8	幷
U+5E77.0	fCNS11643	3-2863
U+5E77.0	fGB	E-2928
U+5E77.0	fR/S	 51.5
U+5E77.0	fTotalStrokes	 8
U+5E77.0	fCangjie	YJYJ
U+5E77.0	fFourCorner	33440
U+5E77.0	fHakka	  bin5
U+5E77.0	fCantonese	bing3; bing6; ping3 (also); bin3; bing1
U+5E77.0	fMandarin	bing4; bing1
U+5E77.0	fEnglish	[1] on a level with; even; equal [2] and; also; or; at the same time; together with
U+5E77.1	fUTF8	幷
U+5E77.1	fCNS11643	3-2863
U+5E77.1	fGB	E-2928
U+5E77.1	fR/S	 51.5
U+5E77.1	fTotalStrokes	 8
U+5E77.1	fCangjie	YJYJ
U+5E77.1	fFourCorner	33440
U+5E77.1	fEnglish	Pingchou, one of the ancient Chinese administrative divisions, consisting of parts of today's Hopeh, Shansi
U+5E78.0	fUTF8	幸
U+5E78.0	fCNS11643	1-4D31
U+5E78.0	fBig5	A9AF
U+5E78.0	fGB	0-5052
U+5E78.0	fR/S	 51.5
U+5E78.0	fTotalStrokes	 8
U+5E78.0	fCangjie	GTJ
U+5E78.0	fFourCorner	40401
U+5E78.0	fHakka	  hen5; hen6; hen3
U+5E78.0	fCantonese	hang6
U+5E78.0	fMandarin	xing4
U+5E78.0	fEnglish	[1] well-being and happiness [2] luck(ily); fortunately; thanks to [3] feel happy about; favor [4] imperial tour or progress [5] a Chinese family name
U+5E79.0	fUTF8	幹
U+5E79.0	fCNS11643	1-636C
U+5E79.0	fBig5	B746
U+5E79.0	fGB	1-3849
U+5E79.0	fR/S	 51.10
U+5E79.0	fTotalStrokes	 13
U+5E79.0	fCangjie	JJOMJ
U+5E79.0	fFourCorner	48441
U+5E79.0	fHakka	  gon5; kon5; gan5
U+5E79.0	fCantonese	gon3; hon4; gon1
U+5E79.0	fMandarin	gan4; han2; gan1; guan3
U+5E79.0	fEnglish	[1] trunk (of a tree or of a human body) [2] the main park of anything [3] capabilities; talents; capable; skillful [4] do; attend to business; manage [5] kill (slang)
U+5E7A.0	fUTF8	幺
U+5E7A.0	fCNS11643	3-213D
U+5E7A.0	fGB	0-675B
U+5E7A.0	fR/S	 52.0
U+5E7A.0	fTotalStrokes	 3
U+5E7A.0	fCangjie	VI
U+5E7A.0	fFourCorner	22730
U+5E7A.0	fHakka	  jau1; rau1; jeu1
U+5E7A.0	fCantonese	jiu1
U+5E7A.0	fMandarin	yao1
U+5E7A.0	fEnglish	[1] one; one on dice [2] tiny; small; insignificant [3] the youngest son or daughter of a family [4] lone; alone [5] a Chinese family name
U+5E7B.0	fUTF8	幻
U+5E7B.0	fCNS11643	1-453C
U+5E7B.0	fBig5	A4DB
U+5E7B.0	fGB	0-3B43
U+5E7B.0	fR/S	 52.1
U+5E7B.0	fTotalStrokes	 4
U+5E7B.0	fCangjie	VIS
U+5E7B.0	fFourCorner	27720
U+5E7B.0	fHakka	  fam5; fam6; fan5
U+5E7B.0	fCantonese	waan6
U+5E7B.0	fMandarin	huan4
U+5E7B.0	fEnglish	[1] illusion; hallucination; fantasy; magic [2] illusory; unreal; deceptive; changeable
U+5E7C.0	fUTF8	幼
U+5E7C.0	fCNS11643	1-464E
U+5E7C.0	fBig5	A5AE
U+5E7C.0	fGB	0-5357
U+5E7C.0	fR/S	 52.2
U+5E7C.0	fTotalStrokes	 5
U+5E7C.0	fCangjie	VIKS
U+5E7C.0	fFourCorner	24727
U+5E7C.0	fHakka	  ju5;  jiu5; riu5
U+5E7C.0	fCantonese	jau3; jiu3
U+5E7C.0	fMandarin	you4; yao4
U+5E7C.0	fEnglish	[1] young; delicate; tender; immature [2] take care of the young
U+5E7D.0	fUTF8	幽
U+5E7D.0	fCNS11643	1-5077
U+5E7D.0	fBig5	ABD5
U+5E7D.0	fGB	0-5344
U+5E7D.0	fR/S	 52.6
U+5E7D.0	fTotalStrokes	 9
U+5E7D.0	fCangjie	UVII
U+5E7D.0	fFourCorner	22770
U+5E7D.0	fHakka	  ju2; jeu2; riu2; jiu3; jiu1; jiu2
U+5E7D.0	fCantonese	jau1
U+5E7D.0	fMandarin	you1
U+5E7D.0	fEnglish	[1] quiet; tranquil [2] dark; gloomy; obscure [3] lonely; solitary; secluded [4] deep; profound [5] hidden; secret [6] imprison; confine
U+5E7E.0	fUTF8	幾
U+5E7E.0	fCNS11643	1-5E7D
U+5E7E.0	fBig5	B458
U+5E7E.0	fGB	1-3C38
U+5E7E.0	fR/S	 52.9
U+5E7E.0	fTotalStrokes	 12
U+5E7E.0	fCangjie	VIHI
U+5E7E.0	fFourCorner	22253
U+5E7E.0	fHakka	  gi3; git7; gia6; gia5
U+5E7E.0	fCantonese	gei2; gei1
U+5E7E.0	fMandarin	ji3; ji4; qi2
U+5E7E.0	fEnglish	[1] how many; how much [2] a few; some; several
U+5E7E.1	fUTF8	幾
U+5E7E.1	fCNS11643	1-5E7D
U+5E7E.1	fBig5	B458
U+5E7E.1	fGB	1-3C38
U+5E7E.1	fR/S	 52.9
U+5E7E.1	fTotalStrokes	 12
U+5E7E.1	fCangjie	VIHI
U+5E7E.1	fFourCorner	22253
U+5E7E.1	fHakka	 gi1
U+5E7E.1	fMandarin	ji1
U+5E7E.1	fEnglish	[1] small; minute; tiny; slight [2] nearly; almost
U+5E7F.0	fUTF8	广
U+5E7F.0	fCNS11643	3-213E
U+5E7F.0	fGB	0-3963
U+5E7F.0	fR/S	 53.0
U+5E7F.0	fTotalStrokes	 3
U+5E7F.0	fCangjie	YH
U+5E7F.0	fFourCorner	00200
U+5E7F.0	fHakka	 ram3; jam3; kwong3
U+5E7F.0	fCantonese	jim5; gwong2; am1; ngam1 (also); gong2 (also); jim2
U+5E7F.0	fMandarin	yan3; an1; guang3; guang1; guang4
U+5E7F.0	fEnglish	simplified form of &#x5EE3;
U+5E80.0	fUTF8	庀
U+5E80.0	fCNS11643	2-215A
U+5E80.0	fBig5	C979
U+5E80.0	fGB	0-624F
U+5E80.0	fR/S	 53.2
U+5E80.0	fTotalStrokes	 5
U+5E80.0	fCangjie	IP
U+5E80.0	fFourCorner	00211
U+5E80.0	fHakka	 pi3
U+5E80.0	fCantonese	pei2; pei5
U+5E80.0	fMandarin	pi3
U+5E80.0	fEnglish	 prepare; provide; arrange
U+5E81.0	fUTF8	庁
U+5E81.0	fCNS11643	3-2247
U+5E81.0	fGB	5-3C75
U+5E81.0	fR/S	  53.2
U+5E81.0	fTotalStrokes	 5
U+5E81.0	fCangjie	IMN
U+5E81.0	fFourCorner	00220
U+5E81.0	fCantonese	ting1
U+5E81.0	fMandarin	ting1
U+5E81.0	fEnglish	hall; central room
U+5E82.0	fUTF8	庂
U+5E82.0	fCNS11643	2-215B
U+5E82.0	fBig5	C97A
U+5E82.0	fGB	5-3C77
U+5E82.0	fR/S	 53.2
U+5E82.0	fTotalStrokes	 5
U+5E82.0	fCangjie	IO
U+5E82.0	fFourCorner	00280
U+5E82.0	fCantonese	zak1
U+5E82.0	fMandarin	ze4
U+5E83.0	fUTF8	広
U+5E83.0	fCNS11643	3-2248
U+5E83.0	fGB	E-2929
U+5E83.0	fR/S	  53.2
U+5E83.0	fTotalStrokes	 5
U+5E83.0	fCangjie	II
U+5E83.0	fFourCorner	00230
U+5E83.0	fHakka	 kwong3
U+5E83.0	fCantonese	gwong2; gong2 (also)
U+5E83.0	fMandarin	guang3; an1; guang1; guang4
U+5E83.0	fEnglish	simplified form of &#x5EE3;
U+5E84.0	fUTF8	庄
U+5E84.0	fCNS11643	2-223C
U+5E84.0	fBig5	C9DC
U+5E84.0	fGB	0-572F
U+5E84.0	fR/S	 53.3
U+5E84.0	fTotalStrokes	 6
U+5E84.0	fCangjie	IG
U+5E84.0	fFourCorner	00214
U+5E84.0	fHakka	  zong1
U+5E84.0	fCantonese	zong1
U+5E84.0	fMandarin	zhuang1; zhuan1; zhuang4
U+5E84.0	fEnglish	[1] farmhouse [2] marketplace [3] banker (in gambling games) [4] cottage [5] simplified form of &#x838A;
U+5E85.0	fUTF8	庅
U+5E85.0	fCNS11643	3-6678
U+5E85.0	fGB	E-292A
U+5E85.0	fR/S	 53.3
U+5E85.0	fTotalStrokes	 6
U+5E85.0	fCangjie	XIHI
U+5E85.0	fHakka	 dai3; gi3; te3
U+5E85.0	fCantonese	mo1; dai2
U+5E85.0	fMandarin	mo5; di3; de1; de5
U+5E85.0	fEnglish	same as &#x5E95;
U+5E86.0	fUTF8	庆
U+5E86.0	fGB	0-476C
U+5E86.0	fR/S	 53.3
U+5E86.0	fTotalStrokes	 6
U+5E86.0	fCangjie	IK
U+5E86.0	fHakka	 kin1; kin5; ken1
U+5E86.0	fCantonese	hing3
U+5E86.0	fMandarin	qing4; qiang1; qing1
U+5E86.0	fEnglish	simplified form of &#x6176;
U+5E87.0	fUTF8	庇
U+5E87.0	fCNS11643	1-4A2A
U+5E87.0	fBig5	A7C8
U+5E87.0	fGB	0-3153
U+5E87.0	fR/S	 53.4
U+5E87.0	fTotalStrokes	 7
U+5E87.0	fCangjie	IPP
U+5E87.0	fFourCorner	00211
U+5E87.0	fHakka	 bi3; bi5; pi5; pi1
U+5E87.0	fCantonese	bei3
U+5E87.0	fMandarin	bi4
U+5E87.0	fEnglish	hide; protect; conceal; harbour
U+5E88.0	fUTF8	庈
U+5E88.0	fCNS11643	2-2370
U+5E88.0	fBig5	CAD0
U+5E88.0	fGB	3-3A40
U+5E88.0	fR/S	 53.4
U+5E88.0	fTotalStrokes	 7
U+5E88.0	fCangjie	IOIN
U+5E88.0	fFourCorner	00227
U+5E88.0	fHakka	 kim2; kim1
U+5E88.0	fCantonese	kam4; jam1
U+5E88.0	fMandarin	qin2
U+5E88.0	fEnglish	personal name
U+5E89.0	fUTF8	庉
U+5E89.0	fCNS11643	2-236E
U+5E89.0	fBig5	CACE
U+5E89.0	fGB	5-3C7C
U+5E89.0	fR/S	 53.4
U+5E89.0	fTotalStrokes	 7
U+5E89.0	fCangjie	IPU
U+5E89.0	fFourCorner	00217
U+5E89.0	fHakka	 tun2;  tun3; tun5
U+5E89.0	fCantonese	deon6; tyun4
U+5E89.0	fMandarin	dun4; tun2
U+5E89.0	fEnglish	 village dwell together
U+5E8A.0	fUTF8	床
U+5E8A.0	fCNS11643	1-4A2B
U+5E8A.0	fBig5	A7C9
U+5E8A.0	fGB	0-3432
U+5E8A.0	fR/S	 53.4
U+5E8A.0	fTotalStrokes	 7
U+5E8A.0	fCangjie	ID
U+5E8A.0	fFourCorner	00294
U+5E8A.0	fHakka	  cong2; cong3
U+5E8A.0	fCantonese	cong4
U+5E8A.0	fMandarin	chuang2
U+5E8A.0	fEnglish	bed, couch; framework, chassis
U+5E8B.0	fUTF8	庋
U+5E8B.0	fCNS11643	2-236D
U+5E8B.0	fBig5	CACD
U+5E8B.0	fGB	0-6251
U+5E8B.0	fR/S	 53.4
U+5E8B.0	fTotalStrokes	 7
U+5E8B.0	fCangjie	IJE
U+5E8B.0	fFourCorner	00247
U+5E8B.0	fHakka	 gui3; gi3; kwui3; gi1
U+5E8B.0	fCantonese	gwai2; gei2 (also)
U+5E8B.0	fMandarin	gui3; gui4; ji3
U+5E8B.0	fEnglish	[1] cupboard; closet [2] put into a cupboard or closet; put into the proper place; put away; store up
U+5E8C.0	fUTF8	庌
U+5E8C.0	fCNS11643	2-236F
U+5E8C.0	fBig5	CACF
U+5E8C.0	fGB	3-3A3E
U+5E8C.0	fR/S	 53.4
U+5E8C.0	fTotalStrokes	 7
U+5E8C.0	fCangjie	IMVH
U+5E8C.0	fFourCorner	00241
U+5E8C.0	fHakka	  nga3
U+5E8C.0	fCantonese	ngaa5; aa5 (also)
U+5E8C.0	fMandarin	ya3; ya1; ya2
U+5E8C.0	fEnglish	[1] verandah [2] shed
U+5E8D.0	fUTF8	庍
U+5E8D.0	fCNS11643	2-2371
U+5E8D.0	fBig5	CAD1
U+5E8D.0	fGB	5-3C7E
U+5E8D.0	fR/S	 53.4
U+5E8D.0	fTotalStrokes	 7
U+5E8D.0	fCangjie	IHML
U+5E8D.0	fFourCorner	00221
U+5E8D.0	fCantonese	baai3; jan3
U+5E8D.0	fMandarin	bai4; xian1
U+5E8E.0	fUTF8	庎
U+5E8E.0	fCNS11643	4-2428
U+5E8E.0	fGB	5-3D21
U+5E8E.0	fR/S	  53.4
U+5E8E.0	fTotalStrokes	 7
U+5E8E.0	fCangjie	IOLL
U+5E8E.0	fFourCorner	00220
U+5E8E.0	fCantonese	gaai3
U+5E8E.0	fMandarin	jie4
U+5E8F.0	fUTF8	序
U+5E8F.0	fCNS11643	1-4A29
U+5E8F.0	fBig5	A7C7
U+5E8F.0	fGB	0-5072
U+5E8F.0	fR/S	 53.4
U+5E8F.0	fTotalStrokes	 7
U+5E8F.0	fCangjie	ININ
U+5E8F.0	fFourCorner	00222
U+5E8F.0	fHakka	  si5; cih5
U+5E8F.0	fCantonese	zeoi6; zeoi2 (also)
U+5E8F.0	fMandarin	xu4
U+5E8F.0	fEnglish	[1] series; serial order; sequence [2] preface; foreword; introduction
U+5E90.0	fUTF8	庐
U+5E90.0	fCNS11643	3-256E
U+5E90.0	fGB	0-422E
U+5E90.0	fR/S	 53.4
U+5E90.0	fTotalStrokes	 7
U+5E90.0	fCangjie	IIS
U+5E90.0	fFourCorner	00200
U+5E90.0	fHakka	 lu2
U+5E90.0	fCantonese	lou4
U+5E90.0	fMandarin	lu2
U+5E90.0	fEnglish	[1] non-classical form of &#x5EEC; [2] simplified form of &#x5EEC;
U+5E91.0	fUTF8	庑
U+5E91.0	fGB	0-6250
U+5E91.0	fR/S	 53.4
U+5E91.0	fTotalStrokes	 7
U+5E91.0	fCangjie	IMKU
U+5E91.0	fHakka	 vu3
U+5E91.0	fCantonese	mou5; mou4
U+5E91.0	fMandarin	wu3; wu2
U+5E91.0	fEnglish	simplified form of &#x5EE1;
U+5E92.0	fUTF8	庒
U+5E92.0	fCNS11643	3-256D
U+5E92.0	fGB	E-292B
U+5E92.0	fR/S	  53.4
U+5E92.0	fTotalStrokes	 7
U+5E92.0	fCangjie	IGI
U+5E92.0	fFourCorner	00210
U+5E92.0	fCantonese	zong1
U+5E93.0	fUTF8	库
U+5E93.0	fGB	0-3F62
U+5E93.0	fR/S	 53.4
U+5E93.0	fTotalStrokes	 7
U+5E93.0	fCangjie	IKQ
U+5E93.0	fHakka	 ku5
U+5E93.0	fCantonese	fu3
U+5E93.0	fMandarin	ku4; she4
U+5E93.0	fEnglish	simplified form of &#x5EAB;
U+5E94.0	fUTF8	应
U+5E94.0	fGB	0-5326
U+5E94.0	fR/S	 53.4
U+5E94.0	fTotalStrokes	 7
U+5E94.0	fCangjie	IFM
U+5E94.0	fHakka	 jin1; jin5
U+5E94.0	fCantonese	jing3; jing1
U+5E94.0	fMandarin	ying1
U+5E94.0	fEnglish	simplified form of &#x61C9;
U+5E94.1	fUTF8	应
U+5E94.1	fGB	0-5326
U+5E94.1	fR/S	 53.4
U+5E94.1	fTotalStrokes	 7
U+5E94.1	fCangjie	IFM
U+5E94.1	fMandarin	ying4
U+5E94.2	fUTF8	应
U+5E94.2	fGB	0-5326
U+5E94.2	fR/S	 53.4
U+5E94.2	fTotalStrokes	 7
U+5E94.2	fCangjie	IFM
U+5E94.2	fMandarin	ying5
U+5E95.0	fUTF8	底
U+5E95.0	fCNS11643	1-4D35
U+5E95.0	fBig5	A9B3
U+5E95.0	fGB	0-3557
U+5E95.0	fR/S	 53.5
U+5E95.0	fTotalStrokes	 8
U+5E95.0	fCangjie	IHPM
U+5E95.0	fFourCorner	00242
U+5E95.0	fHakka	  dai3; di3; gi3; te3
U+5E95.0	fCantonese	dai2; dai1 (also); dik1
U+5E95.0	fMandarin	di3; de5; de1
U+5E95.0	fEnglish	[1] bottom; underneath; underside; base; basis [2] end; last part
U+5E96.0	fUTF8	庖
U+5E96.0	fCNS11643	1-4D36
U+5E96.0	fBig5	A9B4
U+5E96.0	fGB	0-6252
U+5E96.0	fR/S	 53.5
U+5E96.0	fTotalStrokes	 8
U+5E96.0	fCangjie	IPRU
U+5E96.0	fFourCorner	00212
U+5E96.0	fHakka	 pau2
U+5E96.0	fCantonese	paau4
U+5E96.0	fMandarin	pao2
U+5E96.0	fEnglish	 kitchen; cuisine
U+5E97.0	fUTF8	店
U+5E97.0	fCNS11643	1-4D33
U+5E97.0	fBig5	A9B1
U+5E97.0	fGB	0-356A
U+5E97.0	fR/S	 53.5
U+5E97.0	fTotalStrokes	 8
U+5E97.0	fCangjie	IYR
U+5E97.0	fFourCorner	00261
U+5E97.0	fHakka	  diam5
U+5E97.0	fCantonese	dim3
U+5E97.0	fMandarin	dian4
U+5E97.0	fEnglish	[1] shop; store; commercial establishment [2] inn; hotel
U+5E98.0	fUTF8	庘
U+5E98.0	fCNS11643	3-2865
U+5E98.0	fGB	3-3A42
U+5E98.0	fR/S	 53.4; 53.5
U+5E98.0	fTotalStrokes	 8; 7
U+5E98.0	fCangjie	IWL
U+5E98.0	fFourCorner	00260
U+5E98.0	fCantonese	aap3; ngaap3 (also)
U+5E98.0	fMandarin	ya1
U+5E99.0	fUTF8	庙
U+5E99.0	fCNS11643	3-2867
U+5E99.0	fGB	0-436D
U+5E99.0	fR/S	  53.5
U+5E99.0	fTotalStrokes	 8
U+5E99.0	fCangjie	ILW
U+5E99.0	fFourCorner	00260
U+5E99.0	fHakka	 miau5; miau6; meu5
U+5E99.0	fCantonese	miu6; miu2 (also)
U+5E99.0	fMandarin	miao4
U+5E99.0	fEnglish	[1] non-classical form of &#x5EDF; [2] simplified form of &#x5EDF;
U+5E9A.0	fUTF8	庚
U+5E9A.0	fCNS11643	1-4D32
U+5E9A.0	fBig5	A9B0
U+5E9A.0	fGB	0-387D
U+5E9A.0	fR/S	 53.5
U+5E9A.0	fTotalStrokes	 8
U+5E9A.0	fCangjie	ILO
U+5E9A.0	fFourCorner	00237
U+5E9A.0	fHakka	  gang1
U+5E9A.0	fCantonese	gang1
U+5E9A.0	fMandarin	geng1
U+5E9A.0	fEnglish	[1] the seventh of the ten celestial stems [2] the age (of a person)
U+5E9B.0	fUTF8	庛
U+5E9B.0	fCNS11643	2-2A39
U+5E9B.0	fBig5	CEB8
U+5E9B.0	fGB	3-3A49
U+5E9B.0	fR/S	 53.5
U+5E9B.0	fTotalStrokes	 8
U+5E9B.0	fCangjie	IYMP
U+5E9B.0	fFourCorner	00111
U+5E9B.0	fHakka	 cih5; zih1; ze5
U+5E9B.0	fCantonese	ci3
U+5E9B.0	fMandarin	ci1; ci4
U+5E9B.0	fEnglish	the wooden part of the plough to which the share is fastened
U+5E9C.0	fUTF8	府
U+5E9C.0	fCNS11643	1-4D34
U+5E9C.0	fBig5	A9B2
U+5E9C.0	fGB	0-382E
U+5E9C.0	fR/S	 53.5
U+5E9C.0	fTotalStrokes	 8
U+5E9C.0	fCangjie	IODI
U+5E9C.0	fFourCorner	00240
U+5E9C.0	fHakka	  fu3
U+5E9C.0	fCantonese	fu2
U+5E9C.0	fMandarin	fu3
U+5E9C.0	fEnglish	[1] mansion [2] government agency; government office [3] prefecture; administrative district in former times [4] your home [5] treasury; archives
U+5E9D.0	fUTF8	庝
U+5E9D.0	fCNS11643	3-2868
U+5E9D.0	fGB	3-3A46
U+5E9D.0	fR/S	  53.5
U+5E9D.0	fTotalStrokes	 8
U+5E9D.0	fCangjie	IHEY
U+5E9D.0	fFourCorner	00230
U+5E9D.0	fCantonese	tung4
U+5E9D.0	fMandarin	tong2
U+5E9E.0	fUTF8	庞
U+5E9E.0	fGB	0-4553
U+5E9E.0	fR/S	 53.5
U+5E9E.0	fTotalStrokes	 8
U+5E9E.0	fCangjie	IIKP
U+5E9E.0	fHakka	 pong2; lung2
U+5E9E.0	fCantonese	pong4
U+5E9E.0	fMandarin	pang2; long2; peng2
U+5E9E.0	fEnglish	simplified form of &#x9F90;
U+5E9F.0	fUTF8	废
U+5E9F.0	fGB	0-374F
U+5E9F.0	fR/S	 53.5
U+5E9F.0	fTotalStrokes	 8
U+5E9F.0	fCangjie	IIVE
U+5E9F.0	fHakka	 fui5
U+5E9F.0	fCantonese	fai3
U+5E9F.0	fMandarin	fei4
U+5E9F.0	fEnglish	simplified form of &#x5EE2;
U+5EA0.0	fUTF8	庠
U+5EA0.0	fCNS11643	1-5078
U+5EA0.0	fBig5	ABD6
U+5EA0.0	fGB	0-6254
U+5EA0.0	fR/S	 53.6
U+5EA0.0	fTotalStrokes	 9
U+5EA0.0	fCangjie	ITQ
U+5EA0.0	fFourCorner	00251
U+5EA0.0	fHakka	 siong2; siong3
U+5EA0.0	fCantonese	coeng4
U+5EA0.0	fMandarin	xiang2
U+5EA0.0	fEnglish	a school during the Chou dynasty
U+5EA1.0	fUTF8	庡
U+5EA1.0	fCNS11643	3-2C48
U+5EA1.0	fGB	3-3A4E
U+5EA1.0	fR/S	  53.6
U+5EA1.0	fTotalStrokes	 9
U+5EA1.0	fCangjie	IYHV
U+5EA1.0	fFourCorner	00230
U+5EA1.0	fCantonese	ji2
U+5EA1.0	fMandarin	yi3
U+5EA2.0	fUTF8	庢
U+5EA2.0	fCNS11643	2-2A38
U+5EA2.0	fBig5	CEB7
U+5EA2.0	fGB	5-3D26
U+5EA2.0	fR/S	 53.6
U+5EA2.0	fTotalStrokes	 9
U+5EA2.0	fCangjie	IMIG
U+5EA2.0	fFourCorner	00214
U+5EA2.0	fHakka	 zhit7; tsiit7
U+5EA2.0	fCantonese	zat6; zat1
U+5EA2.0	fMandarin	zhi4; di3
U+5EA3.0	fUTF8	庣
U+5EA3.0	fCNS11643	2-2A3A
U+5EA3.0	fBig5	CEB9
U+5EA3.0	fGB	3-3A4D
U+5EA3.0	fR/S	 53.6
U+5EA3.0	fTotalStrokes	 9
U+5EA3.0	fCangjie	ILMO
U+5EA3.0	fFourCorner	00213
U+5EA3.0	fHakka	  tiau1
U+5EA3.0	fCantonese	tiu1
U+5EA3.0	fMandarin	tiao1; tiao2
U+5EA3.0	fEnglish	 hollow; cavity; sinus
U+5EA4.0	fUTF8	庤
U+5EA4.0	fCNS11643	2-2A37
U+5EA4.0	fBig5	CEB6
U+5EA4.0	fGB	3-3A47
U+5EA4.0	fR/S	 53.6
U+5EA4.0	fTotalStrokes	 9
U+5EA4.0	fCangjie	IGDI
U+5EA4.0	fFourCorner	00241
U+5EA4.0	fHakka	 su1; cu1; chi1; cih1; sii5; she5; shi5
U+5EA4.0	fCantonese	zi6
U+5EA4.0	fMandarin	zhi4
U+5EA4.0	fEnglish	prepare
U+5EA5.0	fUTF8	庥
U+5EA5.0	fCNS11643	2-2A3B
U+5EA5.0	fBig5	CEBA
U+5EA5.0	fGB	0-6253
U+5EA5.0	fR/S	 53.6
U+5EA5.0	fTotalStrokes	 9
U+5EA5.0	fCangjie	IOD
U+5EA5.0	fFourCorner	00294
U+5EA5.0	fHakka	  hiu1
U+5EA5.0	fCantonese	jau1
U+5EA5.0	fMandarin	xiu1
U+5EA5.0	fEnglish	 shelter; protect
U+5EA6.0	fUTF8	度
U+5EA6.0	fCNS11643	1-5079
U+5EA6.0	fBig5	ABD7
U+5EA6.0	fGB	0-3648
U+5EA6.0	fR/S	 53.6
U+5EA6.0	fTotalStrokes	 9
U+5EA6.0	fCangjie	ITE
U+5EA6.0	fFourCorner	00247
U+5EA6.0	fHakka	  tu5; tok8; tu6; dok7
U+5EA6.0	fCantonese	dou6; dok6; dou2 (also)
U+5EA6.0	fMandarin	du4; duo2
U+5EA6.0	fEnglish	[1] degree [2] kilowatt-hour [3] an instrument form measuring length [4] (number of) times [5] system [6] manner; bearing [7] pass [8] consideration; careful thought
U+5EA6.1	fUTF8	度
U+5EA6.1	fCNS11643	1-5079
U+5EA6.1	fBig5	ABD7
U+5EA6.1	fGB	0-3648
U+5EA6.1	fR/S	 53.6
U+5EA6.1	fTotalStrokes	 9
U+5EA6.1	fCangjie	ITE
U+5EA6.1	fFourCorner	00247
U+5EA6.1	fMandarin	duo4
U+5EA6.1	fEnglish	 consider; measure; infer
U+5EA6.2	fUTF8	度
U+5EA6.2	fCNS11643	1-5079
U+5EA6.2	fBig5	ABD7
U+5EA6.2	fGB	0-3648
U+5EA6.2	fR/S	 53.6
U+5EA6.2	fTotalStrokes	 9
U+5EA6.2	fCangjie	ITE
U+5EA6.2	fFourCorner	00247
U+5EA6.2	fMandarin	du5
U+5EA7.0	fUTF8	座
U+5EA7.0	fCNS11643	1-553D
U+5EA7.0	fBig5	AE79
U+5EA7.0	fGB	0-5779
U+5EA7.0	fR/S	 53.7
U+5EA7.0	fTotalStrokes	 10
U+5EA7.0	fCangjie	IOOG
U+5EA7.0	fFourCorner	00214
U+5EA7.0	fHakka	  co5; co6; co3
U+5EA7.0	fCantonese	zo6; zo2 (also)
U+5EA7.0	fMandarin	zuo4
U+5EA7.0	fEnglish	 seat; stand
U+5EA8.0	fUTF8	庨
U+5EA8.0	fCNS11643	2-2E77
U+5EA8.0	fBig5	D175
U+5EA8.0	fGB	3-3A51
U+5EA8.0	fR/S	 53.7
U+5EA8.0	fTotalStrokes	 10
U+5EA8.0	fCangjie	IJKD
U+5EA8.0	fFourCorner	00247
U+5EA8.0	fHakka	  hau1
U+5EA8.0	fCantonese	haau1
U+5EA8.0	fMandarin	xiao1; qiao1
U+5EA8.0	fEnglish	grand; imposing as lofty buildings
U+5EA9.0	fUTF8	庩
U+5EA9.0	fCNS11643	3-306C
U+5EA9.0	fGB	3-3A56
U+5EA9.0	fR/S	  53.7
U+5EA9.0	fTotalStrokes	 10
U+5EA9.0	fCangjie	IOMD
U+5EA9.0	fFourCorner	00290
U+5EA9.0	fCantonese	tou4
U+5EA9.0	fMandarin	tu2
U+5EAA.0	fUTF8	庪
U+5EAA.0	fCNS11643	2-2E79
U+5EAA.0	fBig5	D177
U+5EAA.0	fGB	3-3A55
U+5EAA.0	fR/S	 53.7
U+5EAA.0	fTotalStrokes	 10
U+5EAA.0	fCangjie	IQJE
U+5EAA.0	fFourCorner	00247
U+5EAA.0	fHakka	 gui3; kwui3
U+5EAA.0	fCantonese	gwai2; gei2 (also)
U+5EAA.0	fMandarin	gui3; gui2; ji3
U+5EAB.0	fUTF8	庫
U+5EAB.0	fCNS11643	1-553B
U+5EAB.0	fBig5	AE77
U+5EAB.0	fGB	1-3F62
U+5EAB.0	fR/S	 53.7
U+5EAB.0	fTotalStrokes	 10
U+5EAB.0	fCangjie	IJWJ
U+5EAB.0	fFourCorner	00256
U+5EAB.0	fHakka	  ku5
U+5EAB.0	fCantonese	fu3; se3
U+5EAB.0	fMandarin	ku4; she4
U+5EAB.0	fEnglish	[1] treasury [2] storehouse; storeroom; warehouse; granary
U+5EAC.0	fUTF8	庬
U+5EAC.0	fCNS11643	2-2E7A
U+5EAC.0	fBig5	D178
U+5EAC.0	fGB	5-3D31
U+5EAC.0	fR/S	 53.7
U+5EAC.0	fTotalStrokes	 10
U+5EAC.0	fCangjie	IIUH
U+5EAC.0	fFourCorner	00212
U+5EAC.0	fHakka	 mung3;  mang2
U+5EAC.0	fCantonese	pong4; mong4
U+5EAC.0	fMandarin	pang2; mang2; mang3; meng3; xi4
U+5EAC.0	fEnglish	 abundant mixed
U+5EAD.0	fUTF8	庭
U+5EAD.0	fCNS11643	1-553C
U+5EAD.0	fBig5	AE78
U+5EAD.0	fGB	0-4D25
U+5EAD.0	fR/S	 53.7
U+5EAD.0	fTotalStrokes	 10
U+5EAD.0	fCangjie	INKG
U+5EAD.0	fFourCorner	00241
U+5EAD.0	fHakka	  tin2; tin3
U+5EAD.0	fCantonese	ting4; ting6; ting3
U+5EAD.0	fMandarin	ting2; ting4
U+5EAD.0	fEnglish	[1] court of justice [2] yard [3] hall [4] imperial court
U+5EAE.0	fUTF8	庮
U+5EAE.0	fCNS11643	2-2E78
U+5EAE.0	fBig5	D176
U+5EAE.0	fGB	3-3A53
U+5EAE.0	fR/S	 53.7
U+5EAE.0	fTotalStrokes	 10
U+5EAE.0	fCangjie	IMCW
U+5EAE.0	fFourCorner	00261
U+5EAE.0	fHakka	 ju3; riu3; ju2
U+5EAE.0	fCantonese	jau5; jau4 (also)
U+5EAE.0	fMandarin	you3; you2
U+5EAE.0	fEnglish	[1] rot; decay [2] smell offensive
U+5EAF.0	fUTF8	庯
U+5EAF.0	fCNS11643	3-306B
U+5EAF.0	fGB	3-3A52
U+5EAF.0	fR/S	  53.7
U+5EAF.0	fTotalStrokes	 10
U+5EAF.0	fCangjie	IIJB
U+5EAF.0	fFourCorner	00220
U+5EAF.0	fCantonese	bou1
U+5EAF.0	fMandarin	bu1
U+5EB0.0	fUTF8	庰
U+5EB0.0	fCNS11643	2-2A36
U+5EB0.0	fBig5	CEB5
U+5EB0.0	fGB	3-3A4F
U+5EB0.0	fR/S	 53.8
U+5EB0.0	fTotalStrokes	 11
U+5EB0.0	fCangjie	ITT
U+5EB0.0	fFourCorner	00244
U+5EB0.0	fHakka	 biang5
U+5EB0.0	fCantonese	bing6; bing2; bing3
U+5EB0.0	fMandarin	ding1; bing3
U+5EB1.0	fUTF8	庱
U+5EB1.0	fCNS11643	2-352B
U+5EB1.0	fBig5	D547
U+5EB1.0	fGB	3-3A59
U+5EB1.0	fR/S	 53.8
U+5EB1.0	fTotalStrokes	 11
U+5EB1.0	fCangjie	IGCE
U+5EB1.0	fFourCorner	00247
U+5EB1.0	fCantonese	cing2
U+5EB1.0	fMandarin	cheng3; cheng1; ling3
U+5EB2.0	fUTF8	庲
U+5EB2.0	fCNS11643	2-352E
U+5EB2.0	fBig5	D54A
U+5EB2.0	fGB	3-3A54
U+5EB2.0	fR/S	 53.8
U+5EB2.0	fTotalStrokes	 11
U+5EB2.0	fCangjie	IDOO
U+5EB2.0	fFourCorner	00298
U+5EB2.0	fHakka	 lo2
U+5EB2.0	fCantonese	loi4
U+5EB2.0	fMandarin	lai2
U+5EB3.0	fUTF8	庳
U+5EB3.0	fCNS11643	2-352F
U+5EB3.0	fBig5	D54B
U+5EB3.0	fGB	0-6258
U+5EB3.0	fR/S	 53.8
U+5EB3.0	fTotalStrokes	 11
U+5EB3.0	fCangjie	IHWJ
U+5EB3.0	fFourCorner	00246
U+5EB3.0	fHakka	 bi1; pi3; bi3; bi5; pi5
U+5EB3.0	fCantonese	bei1; pei5 (also)
U+5EB3.0	fMandarin	bei1; bi4; bi3; bi1; pi2
U+5EB3.0	fEnglish	a low-built house
U+5EB4.0	fUTF8	庴
U+5EB4.0	fCNS11643	2-352C
U+5EB4.0	fBig5	D548
U+5EB4.0	fGB	3-3A5A
U+5EB4.0	fR/S	 53.8
U+5EB4.0	fTotalStrokes	 11
U+5EB4.0	fCangjie	ITA
U+5EB4.0	fFourCorner	00261
U+5EB4.0	fCantonese	zik6; zik1
U+5EB4.0	fMandarin	ji2; ji1; qi4; que4
U+5EB5.0	fUTF8	庵
U+5EB5.0	fCNS11643	1-5A2D
U+5EB5.0	fBig5	B167
U+5EB5.0	fGB	0-6256
U+5EB5.0	fR/S	 53.8
U+5EB5.0	fTotalStrokes	 11
U+5EB5.0	fCangjie	IKLU
U+5EB5.0	fFourCorner	00216
U+5EB5.0	fHakka	  am1
U+5EB5.0	fCantonese	am1; ngam1 (also)
U+5EB5.0	fMandarin	an1
U+5EB5.0	fEnglish	[1] hut; cottage; chalet [2] Buddhist nunnery; Buddhist monastery
U+5EB6.0	fUTF8	庶
U+5EB6.0	fCNS11643	1-5A2C
U+5EB6.0	fBig5	B166
U+5EB6.0	fGB	0-4A7C
U+5EB6.0	fR/S	 53.8
U+5EB6.0	fTotalStrokes	 11
U+5EB6.0	fCangjie	ITF
U+5EB6.0	fFourCorner	00237
U+5EB6.0	fHakka	 su5;  shu5; shu6; za5
U+5EB6.0	fCantonese	syu3
U+5EB6.0	fMandarin	shu4; shu1; shu3; zhi1; zhu4
U+5EB6.0	fEnglish	[1] numerous; various; populous [2] born of a concubine [3] general; common [4] the common people; the commoners [5] almost; nearly
U+5EB7.0	fUTF8	康
U+5EB7.0	fCNS11643	1-5A2A
U+5EB7.0	fBig5	B164
U+5EB7.0	fGB	0-3F35
U+5EB7.0	fR/S	 53.8
U+5EB7.0	fTotalStrokes	 11
U+5EB7.0	fCangjie	ILE
U+5EB7.0	fFourCorner	00232
U+5EB7.0	fHakka	  kong1
U+5EB7.0	fCantonese	hong1
U+5EB7.0	fMandarin	kang1; kang4
U+5EB7.0	fEnglish	[1] peaceful [2] healthy [3] abundant [4] level; even and smooth (road, etc.)
U+5EB8.0	fUTF8	庸
U+5EB8.0	fCNS11643	1-5A2B
U+5EB8.0	fBig5	B165
U+5EB8.0	fGB	0-5339
U+5EB8.0	fR/S	 53.8
U+5EB8.0	fTotalStrokes	 11
U+5EB8.0	fCangjie	ILB
U+5EB8.0	fFourCorner	00227
U+5EB8.0	fHakka	  jung2; rung2; jung3
U+5EB8.0	fCantonese	jung4
U+5EB8.0	fMandarin	yong1; yong2
U+5EB8.0	fEnglish	[1] mediocre; common [2] stupid [3] hired laborer [4] require; need [5] reward [6] an interrogative (as, how, etc.)
U+5EB9.0	fUTF8	庹
U+5EB9.0	fCNS11643	2-352D
U+5EB9.0	fBig5	D549
U+5EB9.0	fGB	0-6255
U+5EB9.0	fR/S	 53.8
U+5EB9.0	fTotalStrokes	 11
U+5EB9.0	fCangjie	ITSO
U+5EB9.0	fFourCorner	00237
U+5EB9.0	fHakka	 to2; tu5
U+5EB9.0	fCantonese	tok3
U+5EB9.0	fMandarin	tuo3; tuo1; tuo2
U+5EB9.0	fEnglish	the distance between one's two middle fingertips when one outstretches one's arms
U+5EBA.0	fUTF8	庺
U+5EBA.0	fCNS11643	3-3561
U+5EBA.0	fGB	5-3D37
U+5EBA.0	fR/S	  53.8
U+5EBA.0	fTotalStrokes	 11
U+5EBA.0	fCangjie	ICID
U+5EBA.0	fFourCorner	00290
U+5EBA.0	fHakka	 ciung2; cung2; sung1; ciung3
U+5EBA.0	fCantonese	cung4; cung2 (also); sung1
U+5EBA.0	fMandarin	song1; gong1; song4
U+5EBA.0	fEnglish	ancient form of &#x677E;
U+5EBB.0	fUTF8	庻
U+5EBB.0	fCNS11643	3-3560
U+5EBB.0	fGB	E-292C
U+5EBB.0	fR/S	 53.8
U+5EBB.0	fTotalStrokes	 11
U+5EBB.0	fCangjie	ITOO
U+5EBB.0	fFourCorner	00280
U+5EBB.0	fCantonese	syu3
U+5EBB.0	fMandarin	shu4; zhu4; zhe1
U+5EBB.0	fEnglish	 numerous; various multitude
U+5EBC.0	fUTF8	庼
U+5EBC.0	fGB	8-7B2A
U+5EBC.0	fR/S	 53.11; 53.8
U+5EBC.0	fTotalStrokes	 11
U+5EBC.0	fCangjie	IPMO
U+5EBC.0	fCantonese	king2
U+5EBC.0	fMandarin	qing3
U+5EBD.0	fUTF8	庽
U+5EBD.0	fCNS11643	3-3B3F
U+5EBD.0	fGB	E-292D
U+5EBD.0	fR/S	  53.9
U+5EBD.0	fTotalStrokes	 12
U+5EBD.0	fCangjie	IWLB
U+5EBD.0	fFourCorner	00220
U+5EBD.0	fHakka	 ngi1; ngi5
U+5EBD.0	fCantonese	jyu6
U+5EBD.0	fMandarin	yu4; yu3
U+5EBD.0	fEnglish	[1] lodge [2] sojourn; reside [3] borrow as a metaphor [4] belong to; attack or hang on [5] same as &#x5BD3;
U+5EBE.0	fUTF8	庾
U+5EBE.0	fCNS11643	1-5A2E
U+5EBE.0	fBig5	B168
U+5EBE.0	fGB	0-6257
U+5EBE.0	fR/S	 53.9
U+5EBE.0	fTotalStrokes	 12
U+5EBE.0	fCangjie	IHXO
U+5EBE.0	fFourCorner	00237
U+5EBE.0	fHakka	 ji3; ri3; ji1; i2; i3
U+5EBE.0	fCantonese	jyu4; jyu5 (also)
U+5EBE.0	fMandarin	yu3
U+5EBE.0	fEnglish	[1] an enclosed placed for storing grain; granary [2] a ancient unit of measure of capacity [3] a Chinese family name
U+5EBF.0	fUTF8	庿
U+5EBF.0	fCNS11643	3-3B40
U+5EBF.0	fGB	5-3D33
U+5EBF.0	fR/S	 53.9
U+5EBF.0	fTotalStrokes	 12
U+5EBF.0	fCangjie	ITW
U+5EBF.0	fFourCorner	00260
U+5EBF.0	fHakka	  miau5; miau6; miau3; meu5
U+5EBF.0	fCantonese	miu6; miu2 (also)
U+5EBF.0	fMandarin	miao4
U+5EBF.0	fEnglish	ancient form of &#x5EDF;
U+5EC0.0	fUTF8	廀
U+5EC0.0	fCNS11643	3-3B3E
U+5EC0.0	fGB	E-292E
U+5EC0.0	fR/S	 53.9
U+5EC0.0	fTotalStrokes	 12
U+5EC0.0	fCangjie	IJFE
U+5EC0.0	fFourCorner	00240
U+5EC0.0	fCantonese	sau1
U+5EC0.0	fMandarin	sou1
U+5EC0.0	fEnglish	 conceal search into
U+5EC1.0	fUTF8	廁
U+5EC1.0	fCNS11643	1-5F21
U+5EC1.0	fBig5	B45A
U+5EC1.0	fGB	E-292F
U+5EC1.0	fR/S	 53.9
U+5EC1.0	fTotalStrokes	 12
U+5EC1.0	fCangjie	IBCN
U+5EC1.0	fFourCorner	00220
U+5EC1.0	fHakka	 ci5;  cih5;  sa3; ca5; tsej7; tsez7; ze5
U+5EC1.0	fCantonese	ci3; cak1; zak1
U+5EC1.0	fMandarin	ce4
U+5EC1.0	fEnglish	 toilet; lavatory
U+5EC1.1	fUTF8	廁
U+5EC1.1	fCNS11643	1-5F21
U+5EC1.1	fBig5	B45A
U+5EC1.1	fGB	E-292F
U+5EC1.1	fR/S	 53.9
U+5EC1.1	fTotalStrokes	 12
U+5EC1.1	fCangjie	IBCN
U+5EC1.1	fFourCorner	00220
U+5EC1.1	fMandarin	ci4
U+5EC1.1	fEnglish	 mingle with; be a member of
U+5EC1.2	fUTF8	廁
U+5EC1.2	fCNS11643	1-5F21
U+5EC1.2	fBig5	B45A
U+5EC1.2	fGB	E-292F
U+5EC1.2	fR/S	 53.9
U+5EC1.2	fTotalStrokes	 12
U+5EC1.2	fCangjie	IBCN
U+5EC1.2	fFourCorner	00220
U+5EC1.2	fMandarin	si4
U+5EC1.2	fEnglish	used in combination
U+5EC2.0	fUTF8	廂
U+5EC2.0	fCNS11643	1-5F22
U+5EC2.0	fBig5	B45B
U+5EC2.0	fGB	E-2930
U+5EC2.0	fR/S	 53.9
U+5EC2.0	fTotalStrokes	 12
U+5EC2.0	fCangjie	IDBU
U+5EC2.0	fFourCorner	00260
U+5EC2.0	fHakka	  siong1
U+5EC2.0	fCantonese	soeng1
U+5EC2.0	fMandarin	xiang1
U+5EC2.0	fEnglish	[1] side-room; wing (of a building) [2] theatre box [3] the vicinity or outskirts of a city
U+5EC3.0	fUTF8	廃
U+5EC3.0	fCNS11643	6-4845
U+5EC3.0	fGB	E-2931
U+5EC3.0	fR/S	 53.9
U+5EC3.0	fTotalStrokes	 12
U+5EC3.0	fCangjie	INOP
U+5EC3.0	fFourCorner	00210
U+5EC3.0	fHakka	 fui5
U+5EC3.0	fCantonese	fai3
U+5EC3.0	fMandarin	fei4
U+5EC3.0	fEnglish	simplified form of &#x5EE2;
U+5EC4.0	fUTF8	廄
U+5EC4.0	fCNS11643	1-5F23
U+5EC4.0	fBig5	B45C
U+5EC4.0	fGB	5-3D3B
U+5EC4.0	fR/S	 53.11
U+5EC4.0	fTotalStrokes	 14
U+5EC4.0	fCangjie	IAIE
U+5EC4.0	fFourCorner	00247
U+5EC4.0	fCantonese	gau3
U+5EC4.0	fMandarin	jiu4
U+5EC4.0	fEnglish	 stable barnyard
U+5EC5.0	fUTF8	廅
U+5EC5.0	fCNS11643	2-4263
U+5EC5.0	fBig5	DD5D
U+5EC5.0	fGB	3-3A68
U+5EC5.0	fR/S	 53.10
U+5EC5.0	fTotalStrokes	 13
U+5EC5.0	fCangjie	IGIT
U+5EC5.0	fFourCorner	00213
U+5EC5.0	fHakka	 hap8;  kap8
U+5EC5.0	fCantonese	ap1; ngap1 (also); hap6
U+5EC5.0	fMandarin	he2
U+5EC6.0	fUTF8	廆
U+5EC6.0	fCNS11643	2-4265
U+5EC6.0	fBig5	DD5F
U+5EC6.0	fGB	3-3A66
U+5EC6.0	fR/S	 53.10
U+5EC6.0	fTotalStrokes	 13
U+5EC6.0	fCangjie	IHUI
U+5EC6.0	fFourCorner	00213
U+5EC6.0	fCantonese	gwai1; wai2
U+5EC6.0	fMandarin	hui4; gui1; wei3
U+5EC6.0	fEnglish	 room the wall of a house a man's name
U+5EC7.0	fUTF8	廇
U+5EC7.0	fCNS11643	2-4267
U+5EC7.0	fBig5	DD61
U+5EC7.0	fGB	3-3A6A
U+5EC7.0	fR/S	 53.10
U+5EC7.0	fTotalStrokes	 13
U+5EC7.0	fCangjie	IHHW
U+5EC7.0	fFourCorner	00262
U+5EC7.0	fHakka	  liu5
U+5EC7.0	fCantonese	lau6
U+5EC7.0	fMandarin	liu4
U+5EC7.0	fEnglish	beam in the eaves of a roof, main hall in a house
U+5EC8.0	fUTF8	廈
U+5EC8.0	fCNS11643	1-636E
U+5EC8.0	fBig5	B748
U+5EC8.0	fGB	8-2D5B
U+5EC8.0	fR/S	 53.10
U+5EC8.0	fTotalStrokes	 13
U+5EC8.0	fCangjie	IMUE
U+5EC8.0	fFourCorner	00247
U+5EC8.0	fHakka	 ha5; ha6; ha3
U+5EC8.0	fCantonese	haa6
U+5EC8.0	fMandarin	xia4; sha4
U+5EC8.0	fEnglish	 big building; tall building; edifice
U+5EC9.0	fUTF8	廉
U+5EC9.0	fCNS11643	1-636D
U+5EC9.0	fBig5	B747
U+5EC9.0	fGB	0-412E
U+5EC9.0	fR/S	 53.10
U+5EC9.0	fTotalStrokes	 13
U+5EC9.0	fCangjie	ITXC
U+5EC9.0	fFourCorner	00237
U+5EC9.0	fHakka	  liam2; liam3; liam5
U+5EC9.0	fCantonese	lim4; lim1 (also)
U+5EC9.0	fMandarin	lian2
U+5EC9.0	fEnglish	[1] upright; honest; incorrupt; incorruptible [2] inexpensive; cheap [3] examine; inspect [4] a Chinese family name
U+5ECA.0	fUTF8	廊
U+5ECA.0	fCNS11643	1-5E7E
U+5ECA.0	fBig5	B459
U+5ECA.0	fGB	0-4048
U+5ECA.0	fR/S	 53.9
U+5ECA.0	fTotalStrokes	 12
U+5ECA.0	fCangjie	IIIL
U+5ECA.0	fFourCorner	00227
U+5ECA.0	fHakka	  long2; long3
U+5ECA.0	fCantonese	long4; long2 (also)
U+5ECA.0	fMandarin	lang2
U+5ECA.0	fEnglish	 corridor; portico; hallway
U+5ECB.0	fUTF8	廋
U+5ECB.0	fCNS11643	2-4266
U+5ECB.0	fBig5	DD60
U+5ECB.0	fGB	3-3A65
U+5ECB.0	fR/S	 53.10
U+5ECB.0	fTotalStrokes	 13
U+5ECB.0	fCangjie	IHXE
U+5ECB.0	fFourCorner	00247
U+5ECB.0	fHakka	 seu1; seu3; siu5
U+5ECB.0	fCantonese	sau1
U+5ECB.0	fMandarin	sou1; sou3
U+5ECB.0	fEnglish	[1] conceal; hide [2] search (for); seek
U+5ECC.0	fUTF8	廌
U+5ECC.0	fCNS11643	2-4264
U+5ECC.0	fBig5	DD5E
U+5ECC.0	fGB	3-3A6B
U+5ECC.0	fR/S	 53.10
U+5ECC.0	fTotalStrokes	 13
U+5ECC.0	fCangjie	IXF
U+5ECC.0	fFourCorner	00227
U+5ECC.0	fHakka	 sai6; sai5; sai3; chi3
U+5ECC.0	fCantonese	zi6; zaai6 (also); zai6
U+5ECC.0	fMandarin	zhi4; jian1; jian4; zhai4
U+5ECC.0	fEnglish	unicorn
U+5ECD.0	fUTF8	廍
U+5ECD.0	fCNS11643	3-6474
U+5ECD.0	fGB	E-2932
U+5ECD.0	fR/S	 53.11
U+5ECD.0	fTotalStrokes	 14
U+5ECD.0	fCangjie	IYRL
U+5ECD.0	fFourCorner	0022
U+5ECD.0	fCantonese	bou6
U+5ECD.0	fMandarin	pou3
U+5ECE.0	fUTF8	廎
U+5ECE.0	fCNS11643	2-497D
U+5ECE.0	fBig5	E1B8
U+5ECE.0	fGB	3-3A5D
U+5ECE.0	fR/S	 53.11
U+5ECE.0	fTotalStrokes	 14
U+5ECE.0	fCangjie	IPMC
U+5ECE.0	fFourCorner	00286
U+5ECE.0	fCantonese	king2
U+5ECE.0	fMandarin	qing3; qing1; qing4
U+5ECE.0	fEnglish	room
U+5ECF.0	fUTF8	廏
U+5ECF.0	fCNS11643	3-4661
U+5ECF.0	fGB	E-2933
U+5ECF.0	fR/S	 53.11
U+5ECF.0	fTotalStrokes	 14
U+5ECF.0	fCangjie	IHPE
U+5ECF.0	fFourCorner	00240
U+5ECF.0	fHakka	 giu5
U+5ECF.0	fCantonese	gau3
U+5ECF.0	fMandarin	jiu4
U+5ECF.0	fEnglish	 stable
U+5ED0.0	fUTF8	廐
U+5ED0.0	fCNS11643	3-4660
U+5ED0.0	fGB	E-2934
U+5ED0.0	fR/S	 53.11
U+5ED0.0	fTotalStrokes	 14
U+5ED0.0	fCangjie	IHPU
U+5ED0.0	fFourCorner	00210
U+5ED0.0	fHakka	 giu5
U+5ED0.0	fCantonese	gau3
U+5ED0.0	fMandarin	jiu4
U+5ED0.0	fEnglish	stable; barnyard
U+5ED1.0	fUTF8	廑
U+5ED1.0	fCNS11643	2-497B
U+5ED1.0	fBig5	E1B6
U+5ED1.0	fGB	0-625B
U+5ED1.0	fR/S	 53.11
U+5ED1.0	fTotalStrokes	 14
U+5ED1.0	fCangjie	ITLM
U+5ED1.0	fFourCorner	00214
U+5ED1.0	fHakka	 kiun2; giun3
U+5ED1.0	fCantonese	kan4; gan2; gan6
U+5ED1.0	fMandarin	jin3; qin2; jin4
U+5ED1.0	fEnglish	[1] hut; cottage [2] eager
U+5ED2.0	fUTF8	廒
U+5ED2.0	fCNS11643	2-4A23
U+5ED2.0	fBig5	E1BC
U+5ED2.0	fGB	0-625A
U+5ED2.0	fR/S	 53.11
U+5ED2.0	fTotalStrokes	 14
U+5ED2.0	fCangjie	IGSK
U+5ED2.0	fFourCorner	00248
U+5ED2.0	fHakka	  ngau2
U+5ED2.0	fCantonese	ngou4; ou4 (also)
U+5ED2.0	fMandarin	ao2
U+5ED2.0	fEnglish	granary; storehouse for grain
U+5ED3.0	fUTF8	廓
U+5ED3.0	fCNS11643	1-6860
U+5ED3.0	fBig5	B9F8
U+5ED3.0	fGB	0-402A
U+5ED3.0	fR/S	 53.11
U+5ED3.0	fTotalStrokes	 14
U+5ED3.0	fCangjie	IYDL
U+5ED3.0	fFourCorner	00227
U+5ED3.0	fHakka	  kok7;  gok7; gwok7; kwok7
U+5ED3.0	fCantonese	gwok3; kwok3 (also); gok3 (also); kok3 (also)
U+5ED3.0	fMandarin	kuo4
U+5ED3.0	fEnglish	[1] open; wide [2] empty
U+5ED4.0	fUTF8	廔
U+5ED4.0	fCNS11643	2-4A24
U+5ED4.0	fBig5	E1BD
U+5ED4.0	fGB	5-3D3A
U+5ED4.0	fR/S	 53.11
U+5ED4.0	fTotalStrokes	 14
U+5ED4.0	fCangjie	ILLV
U+5ED4.0	fFourCorner	00244
U+5ED4.0	fHakka	 liu2
U+5ED4.0	fCantonese	lau4
U+5ED4.0	fMandarin	lou2
U+5ED5.0	fUTF8	廕
U+5ED5.0	fCNS11643	2-4A21
U+5ED5.0	fBig5	E1BA
U+5ED5.0	fGB	E-2935
U+5ED5.0	fR/S	 53.11
U+5ED5.0	fTotalStrokes	 14
U+5ED5.0	fCangjie	INLI
U+5ED5.0	fFourCorner	00231
U+5ED5.0	fHakka	 jim5; rim5
U+5ED5.0	fCantonese	jam3
U+5ED5.0	fMandarin	yin4; yin1
U+5ED5.0	fEnglish	 shade; protect; harbour
U+5ED6.0	fUTF8	廖
U+5ED6.0	fCNS11643	1-6861
U+5ED6.0	fBig5	B9F9
U+5ED6.0	fGB	0-414E
U+5ED6.0	fR/S	 53.11
U+5ED6.0	fTotalStrokes	 14
U+5ED6.0	fCangjie	ISMH
U+5ED6.0	fFourCorner	00222
U+5ED6.0	fHakka	  liau5; liau6; liau2; liau3
U+5ED6.0	fCantonese	liu6
U+5ED6.0	fMandarin	liao4; chou1; liao2; liu4
U+5ED6.0	fEnglish	a Chinese family name
U+5ED7.0	fUTF8	廗
U+5ED7.0	fCNS11643	2-497C
U+5ED7.0	fBig5	E1B7
U+5ED7.0	fGB	3-3A62
U+5ED7.0	fR/S	 53.11
U+5ED7.0	fTotalStrokes	 14
U+5ED7.0	fCangjie	IKPB
U+5ED7.0	fFourCorner	00227
U+5ED7.0	fCantonese	daai3
U+5ED7.0	fMandarin	dai4
U+5ED8.0	fUTF8	廘
U+5ED8.0	fCNS11643	2-497A
U+5ED8.0	fBig5	E1B5
U+5ED8.0	fGB	3-3A6E
U+5ED8.0	fR/S	 53.11
U+5ED8.0	fTotalStrokes	 14
U+5ED8.0	fCangjie	IIXP
U+5ED8.0	fFourCorner	00211
U+5ED8.0	fCantonese	luk6
U+5ED8.0	fMandarin	lu4
U+5ED9.0	fUTF8	廙
U+5ED9.0	fCNS11643	2-4A22
U+5ED9.0	fBig5	E1BB
U+5ED9.0	fGB	3-3A6D
U+5ED9.0	fR/S	 53.12
U+5ED9.0	fTotalStrokes	 15
U+5ED9.0	fCangjie	IWTC
U+5ED9.0	fFourCorner	00281
U+5ED9.0	fCantonese	ji6; jik6
U+5ED9.0	fMandarin	yi4
U+5EDA.0	fUTF8	廚
U+5EDA.0	fCNS11643	1-6C5A
U+5EDA.0	fBig5	BC70
U+5EDA.0	fGB	E-2936
U+5EDA.0	fR/S	 53.12
U+5EDA.0	fTotalStrokes	 15
U+5EDA.0	fCangjie	IGTI
U+5EDA.0	fFourCorner	00240
U+5EDA.0	fHakka	 cu2;  chu2; chu3
U+5EDA.0	fCantonese	cyu4; ceoi4 (also); cyu2 (also); ceoi2 (also); cyu1
U+5EDA.0	fMandarin	chu2
U+5EDA.0	fEnglish	[1] kitchen; cookroom [2] closet; chest; wardrobe; cupboard
U+5EDB.0	fUTF8	廛
U+5EDB.0	fCNS11643	2-503C
U+5EDB.0	fBig5	E573
U+5EDB.0	fGB	0-625C
U+5EDB.0	fR/S	 53.12
U+5EDB.0	fTotalStrokes	 15
U+5EDB.0	fCangjie	IWCG
U+5EDB.0	fFourCorner	00214
U+5EDB.0	fHakka	 chan2; can2; chin3; chen2
U+5EDB.0	fCantonese	cin4
U+5EDB.0	fMandarin	chan2
U+5EDB.0	fEnglish	[1] store; shop [2] living space for one family in ancient times
U+5EDC.0	fUTF8	廜
U+5EDC.0	fCNS11643	2-497E
U+5EDC.0	fBig5	E1B9
U+5EDC.0	fGB	3-3A6F
U+5EDC.0	fR/S	 53.12
U+5EDC.0	fTotalStrokes	 15
U+5EDC.0	fCangjie	ISJA
U+5EDC.0	fFourCorner	00264
U+5EDC.0	fCantonese	tou4
U+5EDC.0	fMandarin	tu1; tu2
U+5EDD.0	fUTF8	廝
U+5EDD.0	fCNS11643	1-6C5C
U+5EDD.0	fBig5	BC72
U+5EDD.0	fGB	E-2937
U+5EDD.0	fR/S	 53.12
U+5EDD.0	fTotalStrokes	 15
U+5EDD.0	fCangjie	ITCL
U+5EDD.0	fFourCorner	00221
U+5EDD.0	fHakka	 sii1; su1; se1
U+5EDD.0	fCantonese	si1
U+5EDD.0	fMandarin	si1
U+5EDD.0	fEnglish	[1] servant; a person who performs mean labor [2] fellow; guy [3] each other; together
U+5EDE.0	fUTF8	廞
U+5EDE.0	fCNS11643	2-503D
U+5EDE.0	fBig5	E574
U+5EDE.0	fGB	3-3A63
U+5EDE.0	fR/S	 53.12
U+5EDE.0	fTotalStrokes	 15
U+5EDE.0	fCangjie	ICNO
U+5EDE.0	fFourCorner	00282
U+5EDE.0	fHakka	  him1;  kim1
U+5EDE.0	fCantonese	jam1
U+5EDE.0	fMandarin	xin1; qian1; qin1; xin3; yin3
U+5EDE.0	fEnglish	 prepare horses and chariots for battle
U+5EDF.0	fUTF8	廟
U+5EDF.0	fCNS11643	1-6C5B
U+5EDF.0	fBig5	BC71
U+5EDF.0	fGB	1-436D
U+5EDF.0	fR/S	 53.12
U+5EDF.0	fTotalStrokes	 15
U+5EDF.0	fCangjie	IJJB
U+5EDF.0	fFourCorner	00227
U+5EDF.0	fHakka	  miau5; miau6; miau3; meu5
U+5EDF.0	fCantonese	miu6; miu2 (also)
U+5EDF.0	fMandarin	miao4
U+5EDF.0	fEnglish	 temple; shrine
U+5EE0.0	fUTF8	廠
U+5EE0.0	fCNS11643	1-6C5E
U+5EE0.0	fBig5	BC74
U+5EE0.0	fGB	1-3327
U+5EE0.0	fR/S	 53.12
U+5EE0.0	fTotalStrokes	 15
U+5EE0.0	fCangjie	IFBK
U+5EE0.0	fFourCorner	00248
U+5EE0.0	fHakka	  chong3; cong3
U+5EE0.0	fCantonese	cong2
U+5EE0.0	fMandarin	chang3; an1
U+5EE0.0	fEnglish	 factory; plant; workshop
U+5EE1.0	fUTF8	廡
U+5EE1.0	fCNS11643	2-503E
U+5EE1.0	fBig5	E575
U+5EE1.0	fGB	1-6250
U+5EE1.0	fR/S	 53.12
U+5EE1.0	fTotalStrokes	 15
U+5EE1.0	fCangjie	IOTF
U+5EE1.0	fFourCorner	00231
U+5EE1.0	fHakka	 wu3; vu3
U+5EE1.0	fCantonese	mou5; mou4
U+5EE1.0	fMandarin	wu3; wu2
U+5EE1.0	fEnglish	[1] small rooms around a main building [2] dense; luxuriant (said of vegetation)
U+5EE2.0	fUTF8	廢
U+5EE2.0	fCNS11643	1-6C59
U+5EE2.0	fBig5	BC6F
U+5EE2.0	fGB	1-374F
U+5EE2.0	fR/S	 53.12
U+5EE2.0	fTotalStrokes	 15
U+5EE2.0	fCangjie	INOE
U+5EE2.0	fFourCorner	00247
U+5EE2.0	fHakka	  fui5; fi5; fut7; fit8; fit7
U+5EE2.0	fCantonese	fai3
U+5EE2.0	fMandarin	fei4
U+5EE2.0	fEnglish	[1] give up; abandon; discontinue; abolish [2] reject [3] useless; disused
U+5EE3.0	fUTF8	廣
U+5EE3.0	fCNS11643	1-6C5D
U+5EE3.0	fBig5	BC73
U+5EE3.0	fGB	1-3963
U+5EE3.0	fR/S	 53.12; 53.11
U+5EE3.0	fTotalStrokes	 15
U+5EE3.0	fCangjie	ITLC
U+5EE3.0	fFourCorner	00286
U+5EE3.0	fHakka	  gong3;  kwong3; gwong3
U+5EE3.0	fCantonese	gwong2; gong2 (also)
U+5EE3.0	fMandarin	guang3; an1; guang1; guang4
U+5EE3.0	fEnglish	[1] broad; wide; extensive; spacious [2] stretch; extend [3] Kwangtung; Kwangsi
U+5EE4.0	fUTF8	廤
U+5EE4.0	fGB	1-7D28
U+5EE4.0	fR/S	 53.12
U+5EE4.0	fTotalStrokes	 15
U+5EE4.0	fCangjie	IJJP
U+5EE4.0	fCantonese	fu3
U+5EE4.0	fEnglish	 place
U+5EE5.0	fUTF8	廥
U+5EE5.0	fCNS11643	2-567D
U+5EE5.0	fBig5	E973
U+5EE5.0	fGB	3-3A4C
U+5EE5.0	fR/S	 53.13
U+5EE5.0	fTotalStrokes	 16
U+5EE5.0	fCangjie	IOMA
U+5EE5.0	fFourCorner	00266
U+5EE5.0	fHakka	  kwai5
U+5EE5.0	fCantonese	kui2; kwui2 (also); kui3
U+5EE5.0	fMandarin	guai4; kuai4
U+5EE5.0	fEnglish	[1] outhouse [2] barn
U+5EE6.0	fUTF8	廦
U+5EE6.0	fCNS11643	2-567B
U+5EE6.0	fBig5	E971
U+5EE6.0	fGB	3-3A72
U+5EE6.0	fR/S	 53.13
U+5EE6.0	fTotalStrokes	 16
U+5EE6.0	fCangjie	ISRJ
U+5EE6.0	fFourCorner	00241
U+5EE6.0	fCantonese	bik1; pik1
U+5EE6.0	fMandarin	bi4; pi4
U+5EE7.0	fUTF8	廧
U+5EE7.0	fCNS11643	2-567A
U+5EE7.0	fBig5	E970
U+5EE7.0	fGB	5-3D40
U+5EE7.0	fR/S	 53.13
U+5EE7.0	fTotalStrokes	 16
U+5EE7.0	fCangjie	IGOW
U+5EE7.0	fFourCorner	00261
U+5EE7.0	fHakka	  siong2;  ciong2
U+5EE7.0	fCantonese	coeng4
U+5EE7.0	fMandarin	qiang2
U+5EE7.0	fEnglish	wall
U+5EE8.0	fUTF8	廨
U+5EE8.0	fCNS11643	2-567C
U+5EE8.0	fBig5	E972
U+5EE8.0	fGB	0-625D
U+5EE8.0	fR/S	 53.13
U+5EE8.0	fTotalStrokes	 16
U+5EE8.0	fCangjie	INBQ
U+5EE8.0	fFourCorner	00252
U+5EE8.0	fHakka	 gai3; hai3; kie3; kiai3; hai5; giai5; gai5
U+5EE8.0	fCantonese	haai6; gaai3 (also)
U+5EE8.0	fMandarin	xie4; jie4
U+5EE8.0	fEnglish	government building; government office; public office
U+5EE9.0	fUTF8	廩
U+5EE9.0	fCNS11643	2-5679
U+5EE9.0	fBig5	E96F
U+5EE9.0	fGB	E-2938
U+5EE9.0	fR/S	 53.13
U+5EE9.0	fTotalStrokes	 16
U+5EE9.0	fCangjie	IYWD
U+5EE9.0	fFourCorner	00294
U+5EE9.0	fHakka	 lim1; lim3
U+5EE9.0	fCantonese	lam5
U+5EE9.0	fMandarin	lin3
U+5EE9.0	fEnglish	[1] granary [2] stockpile [3] supply (foodstuffs)
U+5EEA.0	fUTF8	廪
U+5EEA.0	fCNS11643	3-505C
U+5EEA.0	fGB	0-625E
U+5EEA.0	fR/S	 53.13
U+5EEA.0	fTotalStrokes	 16
U+5EEA.0	fCangjie	IYWF
U+5EEA.0	fFourCorner	00290
U+5EEA.0	fHakka	 lim3
U+5EEA.0	fCantonese	lam5
U+5EEA.0	fMandarin	lin3; bing3; lan3
U+5EEA.0	fEnglish	 granary stockpile
U+5EEB.0	fUTF8	廫
U+5EEB.0	fCNS11643	4-5B21
U+5EEB.0	fGB	E-2939
U+5EEB.0	fR/S	  53.15
U+5EEB.0	fTotalStrokes	 18
U+5EEB.0	fCangjie	IBSH
U+5EEB.0	fFourCorner	00220
U+5EEB.0	fCantonese	liu4
U+5EEB.0	fMandarin	liao2
U+5EEC.0	fUTF8	廬
U+5EEC.0	fCNS11643	1-7833
U+5EEC.0	fBig5	C366
U+5EEC.0	fGB	1-422E
U+5EEC.0	fR/S	 53.16
U+5EEC.0	fTotalStrokes	 19
U+5EEC.0	fCangjie	IYPT
U+5EEC.0	fFourCorner	00217
U+5EEC.0	fHakka	  lu2; lu3
U+5EEC.0	fCantonese	lou4
U+5EEC.0	fMandarin	lu2
U+5EEC.0	fEnglish	[1] thatched cottage [2] Mt. Lu (in Kiangsi)
U+5EED.0	fUTF8	廭
U+5EED.0	fCNS11643	3-6475
U+5EED.0	fGB	E-293A
U+5EED.0	fR/S	 53.16
U+5EED.0	fTotalStrokes	 19
U+5EED.0	fCangjie	IHDC
U+5EED.0	fFourCorner	0028
U+5EED.0	fCantonese	zik1
U+5EEE.0	fUTF8	廮
U+5EEE.0	fCNS11643	2-6872
U+5EEE.0	fBig5	F446
U+5EEE.0	fGB	5-3D41
U+5EEE.0	fR/S	 53.17
U+5EEE.0	fTotalStrokes	 20
U+5EEE.0	fCangjie	IBCV
U+5EEE.0	fFourCorner	00244
U+5EEE.0	fCantonese	jing2
U+5EEE.0	fMandarin	ying2
U+5EEF.0	fUTF8	廯
U+5EEF.0	fCNS11643	2-6873
U+5EEF.0	fBig5	F447
U+5EEF.0	fGB	5-3D48
U+5EEF.0	fR/S	 53.17
U+5EEF.0	fTotalStrokes	 20
U+5EEF.0	fCangjie	INFQ
U+5EEF.0	fFourCorner	00151
U+5EEF.0	fHakka	  sien3; sian3
U+5EEF.0	fCantonese	sin1; sin2
U+5EEF.0	fMandarin	xian1; xian2; xian3
U+5EEF.0	fEnglish	 granary
U+5EF0.0	fUTF8	廰
U+5EF0.0	fCNS11643	3-5C3F
U+5EF0.0	fGB	E-293B
U+5EF0.0	fR/S	 53.17
U+5EF0.0	fTotalStrokes	 20
U+5EF0.0	fCangjie	ISJP
U+5EF0.0	fFourCorner	00230
U+5EF0.0	fHakka	 ten1; tang1; tin1
U+5EF0.0	fCantonese	teng1
U+5EF0.0	fMandarin	ting1
U+5EF0.0	fEnglish	abbreviated form of &#x5EF3;
U+5EF1.0	fUTF8	廱
U+5EF1.0	fCNS11643	2-6B59
U+5EF1.0	fBig5	F5CB
U+5EF1.0	fGB	5-3D4B
U+5EF1.0	fR/S	 53.18
U+5EF1.0	fTotalStrokes	 21
U+5EF1.0	fCangjie	IVUG
U+5EF1.0	fFourCorner	00214
U+5EF1.0	fHakka	  jung1; rung1
U+5EF1.0	fCantonese	jung1
U+5EF1.0	fMandarin	yong1
U+5EF1.0	fEnglish	[1] harmonious [2] imperial university or academy
U+5EF2.0	fUTF8	廲
U+5EF2.0	fCNS11643	2-6D4D
U+5EF2.0	fBig5	F6DF
U+5EF2.0	fGB	5-3D2F
U+5EF2.0	fR/S	 53.19
U+5EF2.0	fTotalStrokes	 22
U+5EF2.0	fCangjie	IMMP
U+5EF2.0	fFourCorner	00211
U+5EF2.0	fCantonese	lai4; lai6
U+5EF2.0	fMandarin	li2
U+5EF3.0	fUTF8	廳
U+5EF3.0	fCNS11643	1-7D22
U+5EF3.0	fBig5	C655
U+5EF3.0	fGB	1-4C7C
U+5EF3.0	fR/S	 53.22
U+5EF3.0	fTotalStrokes	 25
U+5EF3.0	fCangjie	ISGP
U+5EF3.0	fFourCorner	00231
U+5EF3.0	fHakka	 tang1;  tiang1; ten1;  tin1; tang5; tang3
U+5EF3.0	fCantonese	teng1; ting1 (also)
U+5EF3.0	fMandarin	ting1
U+5EF3.0	fEnglish	[1] hall [2] central room of a house; main room of a house [3] government agency
U+5EF4.0	fUTF8	廴
U+5EF4.0	fCNS11643	3-213F
U+5EF4.0	fGB	0-5B48
U+5EF4.0	fR/S	 54.0
U+5EF4.0	fTotalStrokes	 3
U+5EF4.0	fCangjie	NNK
U+5EF4.0	fFourCorner	17400
U+5EF4.0	fHakka	 jin3; rin3
U+5EF4.0	fCantonese	jan5; jan2
U+5EF4.0	fMandarin	yin3
U+5EF4.0	fEnglish	 go Kangxi radical number 54
U+5EF5.0	fUTF8	廵
U+5EF5.0	fCNS11643	3-2361
U+5EF5.0	fGB	E-293C
U+5EF5.0	fR/S	  54.3
U+5EF5.0	fTotalStrokes	 6
U+5EF5.0	fCangjie	NKVVV
U+5EF5.0	fFourCorner	12400
U+5EF5.0	fHakka	 sun2
U+5EF5.0	fCantonese	ceon4
U+5EF5.0	fMandarin	xun2
U+5EF5.0	fEnglish	corrupted form of &#x5DE1;
U+5EF6.0	fUTF8	延
U+5EF6.0	fCNS11643	1-4D37
U+5EF6.0	fBig5	A9B5
U+5EF6.0	fGB	0-5153
U+5EF6.0	fR/S	 54.4
U+5EF6.0	fTotalStrokes	 7
U+5EF6.0	fCangjie	NKHYM
U+5EF6.0	fFourCorner	12401
U+5EF6.0	fHakka	 jen2; jan3; ren2; jan2
U+5EF6.0	fCantonese	jin4
U+5EF6.0	fMandarin	yan2; yan3; yan4
U+5EF6.0	fEnglish	[1] delay; postpone; defer [2] extend; spread; lengthen [3] prolong [4] invite [5] procrastinate
U+5EF7.0	fUTF8	廷
U+5EF7.0	fCNS11643	1-4A2C
U+5EF7.0	fBig5	A7CA
U+5EF7.0	fGB	0-4D22
U+5EF7.0	fR/S	 54.4
U+5EF7.0	fTotalStrokes	 7
U+5EF7.0	fCangjie	NKHG
U+5EF7.0	fFourCorner	12401
U+5EF7.0	fHakka	  tin2; tin3
U+5EF7.0	fCantonese	ting4
U+5EF7.0	fMandarin	ting2; ding4
U+5EF7.0	fEnglish	 court; imperial court
U+5EF8.0	fUTF8	廸
U+5EF8.0	fCNS11643	3-2869
U+5EF8.0	fGB	E-293D
U+5EF8.0	fR/S	 54.5
U+5EF8.0	fTotalStrokes	 8
U+5EF8.0	fCangjie	NKLW
U+5EF8.0	fFourCorner	15400
U+5EF8.0	fHakka	  tit7
U+5EF8.0	fCantonese	dik6
U+5EF8.0	fMandarin	di2
U+5EF8.0	fEnglish	 enlighten; advance progress
U+5EF9.0	fUTF8	廹
U+5EF9.0	fCNS11643	3-286B
U+5EF9.0	fGB	E-293E
U+5EF9.0	fR/S	 54.5
U+5EF9.0	fTotalStrokes	 8
U+5EF9.0	fCangjie	NKHA
U+5EF9.0	fFourCorner	16400
U+5EF9.0	fHakka	  bet7; bit7
U+5EF9.0	fCantonese	baak1; bik1; pik1; baak3; pak1
U+5EF9.0	fMandarin	po4; pai3
U+5EF9.0	fEnglish	 persecute oppress in difficulties embarrassed
U+5EFA.0	fUTF8	建
U+5EFA.0	fCNS11643	1-507A
U+5EFA.0	fBig5	ABD8
U+5EFA.0	fGB	0-3D28
U+5EFA.0	fR/S	 54.6
U+5EFA.0	fTotalStrokes	 9
U+5EFA.0	fCangjie	NKLQ
U+5EFA.0	fFourCorner	15400
U+5EFA.0	fHakka	  gien5;  gen5; kian5; gian5
U+5EFA.0	fCantonese	gin3
U+5EFA.0	fMandarin	jian4; jian3
U+5EFA.0	fEnglish	[1] build; establish; erect; found; construct [2] bring up; propose; suggest
U+5EFB.0	fUTF8	廻
U+5EFB.0	fCNS11643	3-2C4C
U+5EFB.0	fGB	E-293F
U+5EFB.0	fR/S	  54.6
U+5EFB.0	fTotalStrokes	 9
U+5EFB.0	fCangjie	NKWR
U+5EFB.0	fFourCorner	16400
U+5EFB.0	fHakka	 fui2; fi2; fui3
U+5EFB.0	fCantonese	wui4; wui6
U+5EFB.0	fMandarin	hui2
U+5EFB.0	fEnglish	 circle around return curving
U+5EFC.0	fUTF8	廼
U+5EFC.0	fCNS11643	3-2C4B
U+5EFC.0	fGB	E-2940
U+5EFC.0	fR/S	  54.6
U+5EFC.0	fTotalStrokes	 9
U+5EFC.0	fCangjie	NKMCW
U+5EFC.0	fFourCorner	11400
U+5EFC.0	fHakka	 nai3; nai1; ne3
U+5EFC.0	fCantonese	naai5; laai5 (also); oi2
U+5EFC.0	fMandarin	nai3
U+5EFC.0	fEnglish	then, thereupon, only then
U+5EFD.0	fUTF8	廽
U+5EFD.0	fCNS11643	3-306E
U+5EFD.0	fGB	E-2941
U+5EFD.0	fR/S	 54.7
U+5EFD.0	fTotalStrokes	 10
U+5EFD.0	fCangjie	NKWSL
U+5EFD.0	fFourCorner	16400
U+5EFD.0	fHakka	 fui2
U+5EFD.0	fCantonese	wui4
U+5EFD.0	fMandarin	hui2
U+5EFD.0	fEnglish	return, turn around; a time
U+5EFE.0	fUTF8	廾
U+5EFE.0	fCNS11643	1-445C
U+5EFE.0	fBig5	A47B
U+5EFE.0	fGB	0-5E43
U+5EFE.0	fR/S	 55.0
U+5EFE.0	fTotalStrokes	 3
U+5EFE.0	fCangjie	JJ
U+5EFE.0	fFourCorner	44000
U+5EFE.0	fHakka	 giung3
U+5EFE.0	fCantonese	gung2; jaa6; je6
U+5EFE.0	fMandarin	gong3; ru4; gong4; nian4
U+5EFE.0	fEnglish	 two hands Kangxi radical number 55
U+5EFF.0	fUTF8	廿
U+5EFF.0	fCNS11643	1-453D
U+5EFF.0	fBig5	A4DC
U+5EFF.0	fGB	0-5825
U+5EFF.0	fR/S	 55.1
U+5EFF.0	fTotalStrokes	 4
U+5EFF.0	fCangjie	T
U+5EFF.0	fFourCorner	44770
U+5EFF.0	fHakka	 ngip8;  ngiam5
U+5EFF.0	fCantonese	nim6; je6 (also); jaa6 (also); lim6 (also); jap6
U+5EFF.0	fMandarin	nian4; shi2
U+5EFF.0	fEnglish	twenty; twentieth
U+5F00.0	fUTF8	开
U+5F00.0	fCNS11643	4-214D
U+5F00.0	fGB	0-3F2A
U+5F00.0	fR/S	 55.1
U+5F00.0	fTotalStrokes	 4
U+5F00.0	fCangjie	MT
U+5F00.0	fFourCorner	10440
U+5F00.0	fHakka	 ki2; koi1; hoi1
U+5F00.0	fCantonese	hoi1
U+5F00.0	fMandarin	kai1; qian1
U+5F00.0	fEnglish	simplified form of &#x958B;
U+5F01.0	fUTF8	弁
U+5F01.0	fCNS11643	1-464F
U+5F01.0	fBig5	A5AF
U+5F01.0	fGB	0-5B4D
U+5F01.0	fR/S	 55.2
U+5F01.0	fTotalStrokes	 5
U+5F01.0	fCangjie	IT
U+5F01.0	fFourCorner	23440
U+5F01.0	fHakka	 pien5; pan5; pan6; pen3; pien6; pan3
U+5F01.0	fCantonese	bin6; pun4
U+5F01.0	fMandarin	bian4; pan2
U+5F01.0	fEnglish	[1] conical cap worn on certain ceremonious occasions in ancient times [2] in the Ching Dynasty low-ranking military officers [3] a Chinese family name
U+5F02.0	fUTF8	异
U+5F02.0	fCNS11643	2-223D
U+5F02.0	fBig5	C9DD
U+5F02.0	fGB	0-526C
U+5F02.0	fR/S	 55.3
U+5F02.0	fTotalStrokes	 6
U+5F02.0	fCangjie	SUT
U+5F02.0	fFourCorner	17441
U+5F02.0	fHakka	 ji5; ri2; ji2; ri5; i2
U+5F02.0	fCantonese	ji6; ji4
U+5F02.0	fMandarin	yi4
U+5F02.0	fEnglish	different; unusual; strange
U+5F03.0	fUTF8	弃
U+5F03.0	fCNS11643	3-256F
U+5F03.0	fGB	0-467A
U+5F03.0	fR/S	 55.4; 55.3
U+5F03.0	fTotalStrokes	 7; 6
U+5F03.0	fCangjie	YIT
U+5F03.0	fFourCorner	00440
U+5F03.0	fHakka	 hi5; ki5
U+5F03.0	fCantonese	hei3
U+5F03.0	fMandarin	qi4
U+5F03.0	fEnglish	[1] ancient form of &#x68C4; [2] simplified form of &#x68C4;
U+5F04.0	fUTF8	弄
U+5F04.0	fCNS11643	1-4A2D
U+5F04.0	fBig5	A7CB
U+5F04.0	fGB	0-452A
U+5F04.0	fR/S	 55.4
U+5F04.0	fTotalStrokes	 7
U+5F04.0	fCangjie	MGT
U+5F04.0	fFourCorner	10441
U+5F04.0	fHakka	  lung3; lung5; lung6; nung3; nung6; nung5
U+5F04.0	fCantonese	lung6; nung6 (also)
U+5F04.0	fMandarin	nong4; long4; neng4
U+5F04.0	fEnglish	[1] do; handle; perform [2] play with; fiddle with; sport with [3] make fun of; mock
U+5F05.0	fUTF8	弅
U+5F05.0	fCNS11643	2-2372
U+5F05.0	fBig5	CAD2
U+5F05.0	fGB	E-2942
U+5F05.0	fR/S	 55.4
U+5F05.0	fTotalStrokes	 7
U+5F05.0	fCangjie	CSHT
U+5F05.0	fFourCorner	80442
U+5F05.0	fHakka	 fun2
U+5F05.0	fCantonese	fan5
U+5F05.0	fMandarin	fen2; fen4
U+5F06.0	fUTF8	弆
U+5F06.0	fCNS11643	3-286D
U+5F06.0	fGB	3-3524
U+5F06.0	fR/S	 55.5
U+5F06.0	fTotalStrokes	 8
U+5F06.0	fCangjie	GIT
U+5F06.0	fFourCorner	40440
U+5F06.0	fCantonese	geoi2
U+5F06.0	fMandarin	ju3
U+5F07.0	fUTF8	弇
U+5F07.0	fCNS11643	2-2A3C
U+5F07.0	fBig5	CEBB
U+5F07.0	fGB	3-3526
U+5F07.0	fR/S	 55.6
U+5F07.0	fTotalStrokes	 9
U+5F07.0	fCangjie	ORT
U+5F07.0	fFourCorner	80446
U+5F07.0	fHakka	 jam3;  kem2; ram3; kep7
U+5F07.0	fCantonese	jim2; jim1
U+5F07.0	fMandarin	yan3; yan1
U+5F07.0	fEnglish	[1] cover [2] narrow-necked container [3] profound
U+5F08.0	fUTF8	弈
U+5F08.0	fCNS11643	1-507B
U+5F08.0	fBig5	ABD9
U+5F08.0	fGB	0-5E44
U+5F08.0	fR/S	 55.6
U+5F08.0	fTotalStrokes	 9
U+5F08.0	fCangjie	YCT
U+5F08.0	fFourCorner	00443
U+5F08.0	fHakka	  jit8; rit8
U+5F08.0	fCantonese	jik6
U+5F08.0	fMandarin	yi4
U+5F08.0	fEnglish	Chinese chess
U+5F09.0	fUTF8	弉
U+5F09.0	fGB	E-2943
U+5F09.0	fR/S	 55.7
U+5F09.0	fTotalStrokes	 10
U+5F09.0	fCangjie	VGT
U+5F09.0	fCantonese	zong6
U+5F09.0	fMandarin	zang4; zhuang3; zhuan3
U+5F09.0	fEnglish	large; powerful; stout; thick
U+5F0A.0	fUTF8	弊
U+5F0A.0	fCNS11643	1-6862
U+5F0A.0	fBig5	B9FA
U+5F0A.0	fGB	0-3157
U+5F0A.0	fR/S	 55.12
U+5F0A.0	fTotalStrokes	 15
U+5F0A.0	fCangjie	FKT
U+5F0A.0	fFourCorner	98444
U+5F0A.0	fHakka	  bi5
U+5F0A.0	fCantonese	bai6
U+5F0A.0	fMandarin	bi4; bie2
U+5F0A.0	fEnglish	[1] bad; undesirable [2] dishonesty; fraud [3] exhausted; tired; worn-out [4] disadvantages; harm
U+5F0B.0	fUTF8	弋
U+5F0B.0	fCNS11643	1-445D
U+5F0B.0	fBig5	A47C
U+5F0B.0	fGB	0-5F2E
U+5F0B.0	fR/S	 56.0
U+5F0B.0	fTotalStrokes	 3
U+5F0B.0	fCangjie	IP
U+5F0B.0	fFourCorner	43000
U+5F0B.0	fHakka	  jit8; rit8
U+5F0B.0	fCantonese	jik6
U+5F0B.0	fMandarin	yi4
U+5F0B.0	fEnglish	[1] catch; take [2] shoot with arrow and bow
U+5F0C.0	fUTF8	弌
U+5F0C.0	fCNS11643	3-2164
U+5F0C.0	fGB	5-375F
U+5F0C.0	fR/S	 56.1
U+5F0C.0	fTotalStrokes	 4
U+5F0C.0	fCangjie	IPM
U+5F0C.0	fFourCorner	43100
U+5F0C.0	fHakka	 jit7; rit7
U+5F0C.0	fCantonese	jat1
U+5F0C.0	fMandarin	yi1; yi2; yi4
U+5F0C.0	fEnglish	ancient form of &#x4E00;
U+5F0D.0	fUTF8	弍
U+5F0D.0	fCNS11643	3-2249
U+5F0D.0	fGB	5-3760
U+5F0D.0	fR/S	  56.2
U+5F0D.0	fTotalStrokes	 5
U+5F0D.0	fCangjie	IPMM
U+5F0D.0	fFourCorner	43100
U+5F0D.0	fHakka	  ngi5; ngi6
U+5F0D.0	fCantonese	ji6
U+5F0D.0	fMandarin	er4
U+5F0D.0	fEnglish	ancient form of &#x4E8C;
U+5F0E.0	fUTF8	弎
U+5F0E.0	fCNS11643	3-2362
U+5F0E.0	fGB	5-3761
U+5F0E.0	fR/S	 56.3
U+5F0E.0	fTotalStrokes	 6
U+5F0E.0	fCangjie	IPMMM
U+5F0E.0	fFourCorner	43100
U+5F0E.0	fHakka	 sam1; sam5
U+5F0E.0	fCantonese	saam1; saam3
U+5F0E.0	fMandarin	san1; san4; sa1; sen1
U+5F0E.0	fEnglish	ancient form of &#x4E09;
U+5F0F.0	fUTF8	式
U+5F0F.0	fCNS11643	1-4822
U+5F0F.0	fBig5	A6A1
U+5F0F.0	fGB	0-4A3D
U+5F0F.0	fR/S	 56.3
U+5F0F.0	fTotalStrokes	 6
U+5F0F.0	fCangjie	IPM
U+5F0F.0	fFourCorner	43100
U+5F0F.0	fHakka	 sit7;  shit7; chit7; siit7; ts'iit7
U+5F0F.0	fCantonese	sik1
U+5F0F.0	fMandarin	shi4; chi4; te4
U+5F0F.0	fEnglish	[1] style; fashion; form; mode [2] pattern; type; model; example [3] system [4] ceremony
U+5F0F.1	fUTF8	式
U+5F0F.1	fCNS11643	1-4822
U+5F0F.1	fBig5	A6A1
U+5F0F.1	fGB	0-4A3D
U+5F0F.1	fR/S	 56.3
U+5F0F.1	fTotalStrokes	 6
U+5F0F.1	fCangjie	IPM
U+5F0F.1	fFourCorner	43100
U+5F0F.1	fMandarin	shi5
U+5F10.0	fUTF8	弐
U+5F10.0	fCNS11643	F-224E
U+5F10.0	fGB	E-2944
U+5F10.0	fR/S	 56.3
U+5F10.0	fTotalStrokes	 6
U+5F10.0	fCangjie	MPMM
U+5F10.0	fCantonese	ji6
U+5F10.0	fMandarin	er4
U+5F10.0	fEnglish	number two
U+5F11.0	fUTF8	弑
U+5F11.0	fCNS11643	3-3B42
U+5F11.0	fGB	0-5F31
U+5F11.0	fR/S	 56.9
U+5F11.0	fTotalStrokes	 12
U+5F11.0	fCangjie	KDIPM
U+5F11.0	fFourCorner	43900
U+5F11.0	fHakka	 shi5; si5; sii5; she5
U+5F11.0	fCantonese	si3
U+5F11.0	fMandarin	shi4
U+5F11.0	fEnglish	 kill or murder one's superior, senior, etc.
U+5F12.0	fUTF8	弒
U+5F12.0	fCNS11643	1-636F
U+5F12.0	fBig5	B749
U+5F12.0	fGB	E-2945
U+5F12.0	fR/S	 56.10
U+5F12.0	fTotalStrokes	 13
U+5F12.0	fCangjie	KCIPM
U+5F12.0	fFourCorner	43940
U+5F12.0	fHakka	 si5; shi5
U+5F12.0	fCantonese	si3
U+5F12.0	fMandarin	shi4
U+5F12.0	fEnglish	standard form of &#x5F11;
U+5F13.0	fUTF8	弓
U+5F13.0	fCNS11643	1-445E
U+5F13.0	fBig5	A47D
U+5F13.0	fGB	0-392D
U+5F13.0	fR/S	 57.0
U+5F13.0	fTotalStrokes	 3
U+5F13.0	fCangjie	N
U+5F13.0	fFourCorner	17207
U+5F13.0	fHakka	  giung1
U+5F13.0	fCantonese	gung1
U+5F13.0	fMandarin	gong1; nai3
U+5F13.0	fEnglish	[1] bow [2] curved; arched; bent [3] a measure of length (equal to five Chinese feet) [4] Kangxi radical 57
U+5F14.0	fUTF8	弔
U+5F14.0	fCNS11643	1-453E
U+5F14.0	fBig5	A4DD
U+5F14.0	fGB	E-2946
U+5F14.0	fR/S	 57.1
U+5F14.0	fTotalStrokes	 4
U+5F14.0	fCangjie	NL
U+5F14.0	fFourCorner	17527
U+5F14.0	fHakka	  diau5; dit7; diau1
U+5F14.0	fCantonese	diu3; dik1
U+5F14.0	fMandarin	diao4; di4
U+5F14.0	fEnglish	[1] condole; console [2] mourn [3] hang; suspend; suspended
U+5F15.0	fUTF8	引
U+5F15.0	fCNS11643	1-453F
U+5F15.0	fBig5	A4DE
U+5F15.0	fGB	0-527D
U+5F15.0	fR/S	 57.1
U+5F15.0	fTotalStrokes	 4
U+5F15.0	fCangjie	NL
U+5F15.0	fFourCorner	12200
U+5F15.0	fHakka	  jin3; jin1; rin1; rin3
U+5F15.0	fCantonese	jan5
U+5F15.0	fMandarin	yin3; yin4
U+5F15.0	fEnglish	[1] pull; attract [2] guide [3] introduce [4] quote [5] retire [6] cause; make [7] a unit of length
U+5F16.0	fUTF8	弖
U+5F16.0	fGB	E-2947
U+5F16.0	fR/S	 57.1
U+5F16.0	fTotalStrokes	 4
U+5F16.0	fCangjie	XNM
U+5F16.0	fFourCorner	1710
U+5F16.0	fCantonese	gung1
U+5F16.0	fMandarin	hu4
U+5F16.0	fEnglish	phonetic for "te" (Japanese)
U+5F17.0	fUTF8	弗
U+5F17.0	fCNS11643	1-4651
U+5F17.0	fBig5	A5B1
U+5F17.0	fGB	0-3825
U+5F17.0	fR/S	 57.2
U+5F17.0	fTotalStrokes	 5
U+5F17.0	fCangjie	LLN
U+5F17.0	fFourCorner	55027
U+5F17.0	fHakka	  fut7; fut8; wut7; wut8
U+5F17.0	fCantonese	fat1; fik1
U+5F17.0	fMandarin	fu2; bi4
U+5F17.0	fEnglish	[1] not [2] not
U+5F18.0	fUTF8	弘
U+5F18.0	fCNS11643	1-4650
U+5F18.0	fBig5	A5B0
U+5F18.0	fGB	0-3A6B
U+5F18.0	fR/S	 57.2
U+5F18.0	fTotalStrokes	 5
U+5F18.0	fCangjie	NI
U+5F18.0	fFourCorner	12230
U+5F18.0	fHakka	 fen2; hiong2
U+5F18.0	fCantonese	wang4
U+5F18.0	fMandarin	hong2
U+5F18.0	fEnglish	[1] enlarge; expand; broaden [2] great; magnanimous; capacious
U+5F19.0	fUTF8	弙
U+5F19.0	fCNS11643	4-226B
U+5F19.0	fGB	3-3B63
U+5F19.0	fR/S	  57.3
U+5F19.0	fTotalStrokes	 6
U+5F19.0	fCangjie	NMD
U+5F19.0	fFourCorner	11240
U+5F19.0	fHakka	 wu1; vu1
U+5F19.0	fCantonese	wu1
U+5F19.0	fMandarin	wu1
U+5F19.0	fEnglish	[1] draw a bow [2] whirr of an arrow
U+5F1A.0	fUTF8	弚
U+5F1A.0	fCNS11643	2-223E
U+5F1A.0	fBig5	C9DE
U+5F1A.0	fGB	5-3137
U+5F1A.0	fR/S	 57.3
U+5F1A.0	fTotalStrokes	 6
U+5F1A.0	fCangjie	CNL
U+5F1A.0	fFourCorner	80527
U+5F1A.0	fCantonese	dai6; teoi4
U+5F1A.0	fMandarin	tui2
U+5F1B.0	fUTF8	弛
U+5F1B.0	fCNS11643	1-4823
U+5F1B.0	fBig5	A6A2
U+5F1B.0	fGB	0-335A
U+5F1B.0	fR/S	 57.3
U+5F1B.0	fTotalStrokes	 6
U+5F1B.0	fCangjie	NPD
U+5F1B.0	fFourCorner	14212
U+5F1B.0	fHakka	 chi3; ci3; cih3; ze1; je1; chii3; chi1
U+5F1B.0	fCantonese	ci4; ci2
U+5F1B.0	fMandarin	chi2; shi3
U+5F1B.0	fEnglish	 loosen; relax; unstring; neglect
U+5F1C.0	fUTF8	弜
U+5F1C.0	fCNS11643	4-226C
U+5F1C.0	fGB	3-3B65
U+5F1C.0	fR/S	  57.3
U+5F1C.0	fTotalStrokes	 6
U+5F1C.0	fCangjie	NN
U+5F1C.0	fFourCorner	17220
U+5F1C.0	fCantonese	koeng5
U+5F1C.0	fMandarin	jiang4; qiang2
U+5F1D.0	fUTF8	弝
U+5F1D.0	fCNS11643	2-2373
U+5F1D.0	fBig5	CAD3
U+5F1D.0	fGB	5-3E77
U+5F1D.0	fR/S	 57.4
U+5F1D.0	fTotalStrokes	 7
U+5F1D.0	fCangjie	NAU
U+5F1D.0	fFourCorner	17217
U+5F1D.0	fHakka	  ba5
U+5F1D.0	fCantonese	baa3
U+5F1D.0	fMandarin	ba4
U+5F1D.0	fEnglish	the part of a bow grasped when shooting
U+5F1E.0	fUTF8	弞
U+5F1E.0	fCNS11643	3-2571
U+5F1E.0	fGB	3-3B67
U+5F1E.0	fR/S	  57.4
U+5F1E.0	fTotalStrokes	 7
U+5F1E.0	fCangjie	NNO
U+5F1E.0	fFourCorner	17280
U+5F1E.0	fCantonese	can2
U+5F1E.0	fMandarin	shen3
U+5F1F.0	fUTF8	弟
U+5F1F.0	fCNS11643	1-4A2E
U+5F1F.0	fBig5	A7CC
U+5F1F.0	fGB	0-355C
U+5F1F.0	fR/S	 57.4
U+5F1F.0	fTotalStrokes	 7
U+5F1F.0	fCangjie	CNLH
U+5F1F.0	fFourCorner	80227
U+5F1F.0	fHakka	 tai1;  ti5; ti3; ti6; tai3
U+5F1F.0	fCantonese	dai6; dai2 (also); tai5 (also); dai4 (also); di2 (also); di4 (also); tai6; teoi4
U+5F1F.0	fMandarin	di4
U+5F1F.0	fEnglish	[1] young brother [2] junior
U+5F1F.1	fUTF8	弟
U+5F1F.1	fCNS11643	1-4A2E
U+5F1F.1	fBig5	A7CC
U+5F1F.1	fGB	0-355C
U+5F1F.1	fR/S	 57.4
U+5F1F.1	fTotalStrokes	 7
U+5F1F.1	fCangjie	CNLH
U+5F1F.1	fFourCorner	80227
U+5F1F.1	fMandarin	ti4
U+5F1F.1	fEnglish	 show brotherly love
U+5F1F.2	fUTF8	弟
U+5F1F.2	fCNS11643	1-4A2E
U+5F1F.2	fBig5	A7CC
U+5F1F.2	fGB	0-355C
U+5F1F.2	fR/S	 57.4
U+5F1F.2	fTotalStrokes	 7
U+5F1F.2	fCangjie	CNLH
U+5F1F.2	fFourCorner	80227
U+5F1F.2	fMandarin	di5
U+5F20.0	fUTF8	张
U+5F20.0	fGB	0-5545
U+5F20.0	fR/S	 57.4
U+5F20.0	fTotalStrokes	 7
U+5F20.0	fCangjie	NPO
U+5F20.0	fHakka	 zhong1
U+5F20.0	fCantonese	zoeng1; zoeng3
U+5F20.0	fMandarin	zhang1; zhang4
U+5F20.0	fEnglish	simplified form of &#x5F35;
U+5F21.0	fUTF8	弡
U+5F21.0	fCNS11643	3-286E
U+5F21.0	fGB	5-3E76
U+5F21.0	fR/S	  57.5
U+5F21.0	fTotalStrokes	 8
U+5F21.0	fCangjie	NSS
U+5F21.0	fFourCorner	11210
U+5F21.0	fCantonese	gwat6
U+5F21.0	fMandarin	jue2
U+5F22.0	fUTF8	弢
U+5F22.0	fCNS11643	2-2652
U+5F22.0	fBig5	CC71
U+5F22.0	fGB	3-3B6B
U+5F22.0	fR/S	 57.5
U+5F22.0	fTotalStrokes	 8
U+5F22.0	fCangjie	NUE
U+5F22.0	fFourCorner	12247
U+5F22.0	fHakka	 tau1; to1
U+5F22.0	fCantonese	tou1
U+5F22.0	fMandarin	tao1
U+5F22.0	fEnglish	 bow case scabbard sheathe
U+5F23.0	fUTF8	弣
U+5F23.0	fCNS11643	2-2653
U+5F23.0	fBig5	CC72
U+5F23.0	fGB	3-3B68
U+5F23.0	fR/S	 57.5
U+5F23.0	fTotalStrokes	 8
U+5F23.0	fCangjie	NODI
U+5F23.0	fFourCorner	14240
U+5F23.0	fHakka	 fu5; fu3
U+5F23.0	fCantonese	fu2
U+5F23.0	fMandarin	fu3
U+5F23.0	fEnglish	middle
U+5F24.0	fUTF8	弤
U+5F24.0	fCNS11643	2-2654
U+5F24.0	fBig5	CC73
U+5F24.0	fGB	3-3B69
U+5F24.0	fR/S	 57.5
U+5F24.0	fTotalStrokes	 8
U+5F24.0	fCangjie	NHPM
U+5F24.0	fFourCorner	12240
U+5F24.0	fHakka	  di3
U+5F24.0	fCantonese	dai2
U+5F24.0	fMandarin	di3; diao1
U+5F24.0	fEnglish	bow
U+5F25.0	fUTF8	弥
U+5F25.0	fCNS11643	3-2870
U+5F25.0	fGB	0-4356
U+5F25.0	fR/S	 57.5
U+5F25.0	fTotalStrokes	 8
U+5F25.0	fCangjie	NNF
U+5F25.0	fFourCorner	18290
U+5F25.0	fHakka	  ni2;  ni3
U+5F25.0	fCantonese	nei4; mei4 (also); lei4 (also)
U+5F25.0	fMandarin	mi2; mi3; yi2
U+5F25.0	fEnglish	simplified form of &#x5F4C;
U+5F26.0	fUTF8	弦
U+5F26.0	fCNS11643	1-4D38
U+5F26.0	fBig5	A9B6
U+5F26.0	fGB	0-4F52
U+5F26.0	fR/S	 57.5
U+5F26.0	fTotalStrokes	 8
U+5F26.0	fCangjie	NYVI
U+5F26.0	fFourCorner	10232
U+5F26.0	fHakka	  hien2;  hen2; hian3
U+5F26.0	fCantonese	jin4; jyun4 (also)
U+5F26.0	fMandarin	xian2
U+5F26.0	fEnglish	[1] string (of bows, musical instruments) [2] chord of an arc [3] the first or last quarter of a lunar month
U+5F27.0	fUTF8	弧
U+5F27.0	fCNS11643	1-4D39
U+5F27.0	fBig5	A9B7
U+5F27.0	fGB	0-3B21
U+5F27.0	fR/S	 57.5
U+5F27.0	fTotalStrokes	 8
U+5F27.0	fCangjie	NHVO
U+5F27.0	fFourCorner	12230
U+5F27.0	fHakka	 fu2
U+5F27.0	fCantonese	wu4
U+5F27.0	fMandarin	hu2; wu1
U+5F27.0	fEnglish	[1] wooden bow [2] segment of a circle
U+5F28.0	fUTF8	弨
U+5F28.0	fCNS11643	2-2651
U+5F28.0	fBig5	CC70
U+5F28.0	fGB	3-3B6C
U+5F28.0	fR/S	 57.5
U+5F28.0	fTotalStrokes	 8
U+5F28.0	fCangjie	NSHR
U+5F28.0	fFourCorner	17262
U+5F28.0	fHakka	 chau1; cau1
U+5F28.0	fCantonese	ciu1
U+5F28.0	fMandarin	chao1; chao3; zhao1
U+5F28.0	fEnglish	 bow
U+5F29.0	fUTF8	弩
U+5F29.0	fCNS11643	1-4D3A
U+5F29.0	fBig5	A9B8
U+5F29.0	fGB	0-6573
U+5F29.0	fR/S	 57.5
U+5F29.0	fTotalStrokes	 8
U+5F29.0	fCangjie	VEN
U+5F29.0	fFourCorner	47207
U+5F29.0	fHakka	  nu3; lu2
U+5F29.0	fCantonese	nou5; lou5 (also)
U+5F29.0	fMandarin	nu3
U+5F29.0	fEnglish	 cross-bow
U+5F2A.0	fUTF8	弪
U+5F2A.0	fGB	0-6572
U+5F2A.0	fR/S	 57.5
U+5F2A.0	fTotalStrokes	 8
U+5F2A.0	fCangjie	NNOM
U+5F2A.0	fCantonese	ging3
U+5F2A.0	fMandarin	jing4
U+5F2A.0	fEnglish	simplified form of &#x5F33;
U+5F2B.0	fUTF8	弫
U+5F2B.0	fCNS11643	4-292C
U+5F2B.0	fGB	E-2948
U+5F2B.0	fR/S	 57.6
U+5F2B.0	fTotalStrokes	 9
U+5F2B.0	fCangjie	NSLL
U+5F2B.0	fFourCorner	11210
U+5F2B.0	fCantonese	can2
U+5F2B.0	fMandarin	zhen3
U+5F2C.0	fUTF8	弬
U+5F2C.0	fCNS11643	4-2C70
U+5F2C.0	fGB	E-2949
U+5F2C.0	fR/S	 57.6; 57.7
U+5F2C.0	fTotalStrokes	 10; 9
U+5F2C.0	fCangjie	NSLL
U+5F2C.0	fFourCorner	11210
U+5F2C.0	fCantonese	ji4
U+5F2C.0	fMandarin	yi2
U+5F2C.0	fEnglish	name of a bow
U+5F2D.0	fUTF8	弭
U+5F2D.0	fCNS11643	1-507C
U+5F2D.0	fBig5	ABDA
U+5F2D.0	fGB	0-6574
U+5F2D.0	fR/S	 57.6
U+5F2D.0	fTotalStrokes	 9
U+5F2D.0	fCangjie	NSJ
U+5F2D.0	fFourCorner	11240
U+5F2D.0	fHakka	  ngi3;  mi2; mi3
U+5F2D.0	fCantonese	mai5; mei5 (also)
U+5F2D.0	fMandarin	mi3
U+5F2D.0	fEnglish	[1] stop; end [2] ends of a bow
U+5F2E.0	fUTF8	弮
U+5F2E.0	fCNS11643	2-2A3D
U+5F2E.0	fBig5	CEBC
U+5F2E.0	fGB	3-3B6F
U+5F2E.0	fR/S	 57.6
U+5F2E.0	fTotalStrokes	 9
U+5F2E.0	fCangjie	FQN
U+5F2E.0	fFourCorner	90727
U+5F2E.0	fHakka	 kien1; kian1; ken1; kien2
U+5F2E.0	fCantonese	hyun1; gyun2; gyun3
U+5F2E.0	fMandarin	quan1
U+5F2E.0	fEnglish	 bowstring
U+5F2F.0	fUTF8	弯
U+5F2F.0	fCNS11643	3-2C4D
U+5F2F.0	fGB	0-4D64
U+5F2F.0	fR/S	 57.6
U+5F2F.0	fTotalStrokes	 9
U+5F2F.0	fCangjie	YCN
U+5F2F.0	fFourCorner	00200
U+5F2F.0	fHakka	 van1
U+5F2F.0	fCantonese	waan1
U+5F2F.0	fMandarin	wan1
U+5F2F.0	fEnglish	simplified form of &#x5F4E;
U+5F30.0	fUTF8	弰
U+5F30.0	fCNS11643	2-2E7C
U+5F30.0	fBig5	D17A
U+5F30.0	fGB	3-3B70
U+5F30.0	fR/S	 57.7
U+5F30.0	fTotalStrokes	 10
U+5F30.0	fCangjie	NFB
U+5F30.0	fFourCorner	19227
U+5F30.0	fHakka	 sau1; sau3
U+5F30.0	fCantonese	saau1
U+5F30.0	fMandarin	shao1
U+5F30.0	fEnglish	 ends of bow
U+5F31.0	fUTF8	弱
U+5F31.0	fCNS11643	1-553E
U+5F31.0	fBig5	AE7A
U+5F31.0	fGB	0-4875
U+5F31.0	fR/S	 57.7
U+5F31.0	fTotalStrokes	 10
U+5F31.0	fCangjie	NMNIM
U+5F31.0	fFourCorner	17127
U+5F31.0	fHakka	  ngiok8; gniok8; niok8
U+5F31.0	fCantonese	joek6
U+5F31.0	fMandarin	ruo4
U+5F31.0	fEnglish	[1] weak; fragile; feeble; tender; delicate; infirm [2] inferior [3] young [4] a little less than
U+5F32.0	fUTF8	弲
U+5F32.0	fCNS11643	3-306F
U+5F32.0	fGB	3-3B71
U+5F32.0	fR/S	 57.7
U+5F32.0	fTotalStrokes	 10
U+5F32.0	fCangjie	NRB
U+5F32.0	fFourCorner	16220
U+5F32.0	fCantonese	hyun1
U+5F32.0	fMandarin	xuan1
U+5F33.0	fUTF8	弳
U+5F33.0	fCNS11643	2-2E7B
U+5F33.0	fBig5	D179
U+5F33.0	fGB	1-6572
U+5F33.0	fR/S	 57.7
U+5F33.0	fTotalStrokes	 10
U+5F33.0	fCangjie	NMVM
U+5F33.0	fFourCorner	11211
U+5F33.0	fCantonese	ging3
U+5F33.0	fMandarin	jing4
U+5F33.0	fEnglish	circular measure
U+5F34.0	fUTF8	弴
U+5F34.0	fCNS11643	3-3562
U+5F34.0	fGB	5-3E7E
U+5F34.0	fR/S	  57.8
U+5F34.0	fTotalStrokes	 11
U+5F34.0	fCangjie	NYRD
U+5F34.0	fFourCorner	10240
U+5F34.0	fHakka	  dun1; diau1
U+5F34.0	fCantonese	diu1
U+5F34.0	fMandarin	dun1; diao1
U+5F34.0	fEnglish	red lacquer bow
U+5F35.0	fUTF8	張
U+5F35.0	fCNS11643	1-5A2F
U+5F35.0	fBig5	B169
U+5F35.0	fGB	1-5545
U+5F35.0	fR/S	 57.8
U+5F35.0	fTotalStrokes	 11
U+5F35.0	fCangjie	NSMV
U+5F35.0	fFourCorner	11232
U+5F35.0	fHakka	 zong1; zhong1
U+5F35.0	fCantonese	zoeng3; zoeng1
U+5F35.0	fMandarin	zhang1; zhang4
U+5F35.0	fEnglish	[1] open; stretch; extend [2] display [3] sheet (of paper); leaf (of a book) [4] a Chinese family name [5] trap animals; catch animals
U+5F36.0	fUTF8	弶
U+5F36.0	fCNS11643	2-3530
U+5F36.0	fBig5	D54C
U+5F36.0	fGB	3-3B74
U+5F36.0	fR/S	 57.8
U+5F36.0	fTotalStrokes	 11
U+5F36.0	fCangjie	NYRF
U+5F36.0	fFourCorner	10296
U+5F36.0	fHakka	  giong3;  kiong5
U+5F36.0	fCantonese	koeng5 (also); koeng6; goeng6; zoeng3
U+5F36.0	fMandarin	jiang4; yang3
U+5F36.0	fEnglish	[1] trap for animals [2] net for birds [3] simple trap for birds
U+5F37.0	fUTF8	強
U+5F37.0	fCNS11643	1-5A30
U+5F37.0	fBig5	B16A
U+5F37.0	fGB	E-294A
U+5F37.0	fR/S	 57.8
U+5F37.0	fTotalStrokes	 11
U+5F37.0	fCangjie	NILI
U+5F37.0	fFourCorner	13236
U+5F37.0	fHakka	  kiong2; giong3; kiong3; giong1
U+5F37.0	fCantonese	koeng4; koeng5; goeng6; koeng3; goeng3
U+5F37.0	fMandarin	qiang2; jiang3
U+5F37.0	fEnglish	[1] strong; powerful; vigorous [2] better [3] violent
U+5F37.1	fUTF8	強
U+5F37.1	fCNS11643	1-5A30
U+5F37.1	fBig5	B16A
U+5F37.1	fGB	E-294A
U+5F37.1	fR/S	 57.8
U+5F37.1	fTotalStrokes	 11
U+5F37.1	fCangjie	NILI
U+5F37.1	fFourCorner	13236
U+5F37.1	fMandarin	qiang3
U+5F37.1	fEnglish	[1] force [2] make an effort; strive
U+5F38.0	fUTF8	弸
U+5F38.0	fCNS11643	2-3531
U+5F38.0	fBig5	D54D
U+5F38.0	fGB	3-3B73
U+5F38.0	fR/S	 57.8
U+5F38.0	fTotalStrokes	 11
U+5F38.0	fCangjie	NBB
U+5F38.0	fFourCorner	17220
U+5F38.0	fHakka	  pen2;  ben1
U+5F38.0	fCantonese	pang4 (also); paang4; ping1 (also); bang1
U+5F38.0	fMandarin	peng2; beng1; bing1; peng1
U+5F38.0	fEnglish	 bow stretched full tensely draw
U+5F39.0	fUTF8	弹
U+5F39.0	fCNS11643	3-3563
U+5F39.0	fGB	0-352F
U+5F39.0	fR/S	 57.8
U+5F39.0	fTotalStrokes	 11
U+5F39.0	fCangjie	NCWJ
U+5F39.0	fFourCorner	18250
U+5F39.0	fHakka	 tan3; tan2
U+5F39.0	fCantonese	daan6; taan4; daan2 (also)
U+5F39.0	fMandarin	dan4
U+5F39.0	fEnglish	simplified form of &#x5F48;
U+5F39.1	fUTF8	弹
U+5F39.1	fCNS11643	3-3563
U+5F39.1	fGB	0-352F
U+5F39.1	fR/S	 57.8
U+5F39.1	fTotalStrokes	 11
U+5F39.1	fCangjie	NCWJ
U+5F39.1	fFourCorner	18250
U+5F39.1	fMandarin	tan2
U+5F39.2	fUTF8	弹
U+5F39.2	fCNS11643	3-3563
U+5F39.2	fGB	0-352F
U+5F39.2	fR/S	 57.8
U+5F39.2	fTotalStrokes	 11
U+5F39.2	fCangjie	NCWJ
U+5F39.2	fFourCorner	18250
U+5F39.2	fMandarin	dan5
U+5F3A.0	fUTF8	强
U+5F3A.0	fCNS11643	3-3B47
U+5F3A.0	fGB	0-473F
U+5F3A.0	fR/S	 57.9
U+5F3A.0	fTotalStrokes	 12
U+5F3A.0	fCangjie	NRLI
U+5F3A.0	fFourCorner	16230
U+5F3A.0	fHakka	  kiong2;  giong3; giong1; kiong3; kiong1
U+5F3A.0	fCantonese	koeng4; koeng5; goeng6; goeng1; koeg6
U+5F3A.0	fMandarin	qiang2; jiang4
U+5F3A.0	fEnglish	[1] strong; powerful; energetic [2] contradict
U+5F3A.1	fUTF8	强
U+5F3A.1	fCNS11643	3-3B47
U+5F3A.1	fGB	0-473F
U+5F3A.1	fR/S	 57.9
U+5F3A.1	fTotalStrokes	 12
U+5F3A.1	fCangjie	NRLI
U+5F3A.1	fFourCorner	16230
U+5F3A.1	fMandarin	qiang3
U+5F3B.0	fUTF8	弻
U+5F3B.0	fCNS11643	3-3B45
U+5F3B.0	fGB	5-3F24
U+5F3B.0	fR/S	 57.9
U+5F3B.0	fTotalStrokes	 12
U+5F3B.0	fCangjie	NMWN
U+5F3B.0	fFourCorner	17220
U+5F3B.0	fHakka	 pit7;  pet7; bet7; bit7
U+5F3B.0	fCantonese	bat6
U+5F3B.0	fMandarin	bi4
U+5F3B.0	fEnglish	same as &#x5F3C;
U+5F3C.0	fUTF8	弼
U+5F3C.0	fCNS11643	1-5F24
U+5F3C.0	fBig5	B45D
U+5F3C.0	fGB	0-6576
U+5F3C.0	fR/S	 57.9
U+5F3C.0	fTotalStrokes	 12
U+5F3C.0	fCangjie	NMAN
U+5F3C.0	fFourCorner	17227
U+5F3C.0	fHakka	 pit7;  pet7; bet7; bit7
U+5F3C.0	fCantonese	bat6
U+5F3C.0	fMandarin	bi4
U+5F3C.0	fEnglish	[1] device for regulating bows; bow regulator [2] correct [3] aid; assist; help
U+5F3D.0	fUTF8	弽
U+5F3D.0	fCNS11643	4-3745
U+5F3D.0	fGB	5-3F22
U+5F3D.0	fR/S	  57.9
U+5F3D.0	fTotalStrokes	 12
U+5F3D.0	fCangjie	NPTD
U+5F3D.0	fFourCorner	14290
U+5F3D.0	fCantonese	sip3
U+5F3D.0	fMandarin	she4
U+5F3E.0	fUTF8	弾
U+5F3E.0	fCNS11643	3-3B44
U+5F3E.0	fGB	E-294B
U+5F3E.0	fR/S	 57.9
U+5F3E.0	fTotalStrokes	 12
U+5F3E.0	fCangjie	NFWJ
U+5F3E.0	fFourCorner	12250
U+5F3E.0	fHakka	 tan3; tan2
U+5F3E.0	fCantonese	daan6; taan4
U+5F3E.0	fMandarin	dan4
U+5F3E.0	fEnglish	simplified form of &#x5F48;
U+5F3F.0	fUTF8	弿
U+5F3F.0	fCNS11643	4-3D30
U+5F3F.0	fGB	5-3F28
U+5F3F.0	fR/S	  57.10
U+5F3F.0	fTotalStrokes	 13
U+5F3F.0	fCangjie	JTCN
U+5F3F.0	fFourCorner	30200
U+5F3F.0	fCantonese	gin2
U+5F3F.0	fMandarin	jian3
U+5F40.0	fUTF8	彀
U+5F40.0	fCNS11643	2-4268
U+5F40.0	fBig5	DD62
U+5F40.0	fGB	0-6C30
U+5F40.0	fR/S	 57.10
U+5F40.0	fTotalStrokes	 13
U+5F40.0	fCangjie	GNHNE
U+5F40.0	fFourCorner	47247
U+5F40.0	fHakka	 kieu5;  geu5; giu5
U+5F40.0	fCantonese	gau3
U+5F40.0	fMandarin	gou4; kou1
U+5F40.0	fEnglish	[1] draw a bow to the full [2] trap; snare [3] sufficiently; enough; adequate
U+5F41.0	fUTF8	彁
U+5F41.0	fCNS11643	F-413F
U+5F41.0	fGB	E-294C
U+5F41.0	fR/S	 57.10
U+5F41.0	fTotalStrokes	 13
U+5F41.0	fCangjie	NMRR
U+5F41.0	fCantonese	go1
U+5F42.0	fUTF8	彂
U+5F42.0	fCNS11643	3-4122
U+5F42.0	fGB	E-294D
U+5F42.0	fR/S	 57.10
U+5F42.0	fTotalStrokes	 13
U+5F42.0	fCangjie	TCNOK
U+5F42.0	fFourCorner	23230
U+5F42.0	fCantonese	faat3
U+5F42.0	fMandarin	fa1; fa4; bo1
U+5F42.0	fEnglish	issue; dispatch; send out; emit
U+5F43.0	fUTF8	彃
U+5F43.0	fCNS11643	2-4A26
U+5F43.0	fBig5	E1BF
U+5F43.0	fGB	3-3B6E
U+5F43.0	fR/S	 57.11
U+5F43.0	fTotalStrokes	 14
U+5F43.0	fCangjie	NWTJ
U+5F43.0	fFourCorner	16254
U+5F43.0	fCantonese	bat1
U+5F43.0	fMandarin	bi4
U+5F44.0	fUTF8	彄
U+5F44.0	fCNS11643	2-4A25
U+5F44.0	fBig5	E1BE
U+5F44.0	fGB	3-3B66
U+5F44.0	fR/S	 57.11
U+5F44.0	fTotalStrokes	 14
U+5F44.0	fCangjie	NSRR
U+5F44.0	fFourCorner	11216
U+5F44.0	fHakka	 k'ieu1; kiu1; keu1; keu2
U+5F44.0	fCantonese	kau1
U+5F44.0	fMandarin	kou1
U+5F44.0	fEnglish	 nock at end of bow stretch
U+5F45.0	fUTF8	彅
U+5F45.0	fGB	E-294E
U+5F45.0	fR/S	 57.11
U+5F45.0	fTotalStrokes	 14
U+5F45.0	fCangjie	NTBH
U+5F45.0	fCantonese	zin2
U+5F46.0	fUTF8	彆
U+5F46.0	fCNS11643	1-6863
U+5F46.0	fBig5	B9FB
U+5F46.0	fGB	1-7824
U+5F46.0	fR/S	 57.12
U+5F46.0	fTotalStrokes	 15
U+5F46.0	fCangjie	FKN
U+5F46.0	fFourCorner	97727
U+5F46.0	fHakka	 pet7; pit7
U+5F46.0	fCantonese	bit3
U+5F46.0	fMandarin	bie4; bi4
U+5F46.0	fEnglish	awkward
U+5F47.0	fUTF8	彇
U+5F47.0	fCNS11643	4-495E
U+5F47.0	fGB	5-3F21
U+5F47.0	fR/S	 57.12
U+5F47.0	fTotalStrokes	 15
U+5F47.0	fCangjie	NLX
U+5F47.0	fFourCorner	15220
U+5F47.0	fCantonese	siu1
U+5F47.0	fMandarin	xiao1
U+5F48.0	fUTF8	彈
U+5F48.0	fCNS11643	1-6C5F
U+5F48.0	fBig5	BC75
U+5F48.0	fGB	1-352F
U+5F48.0	fR/S	 57.12
U+5F48.0	fTotalStrokes	 15
U+5F48.0	fCangjie	NRRJ
U+5F48.0	fFourCorner	16256
U+5F48.0	fHakka	 tan3;  tan5; tan6
U+5F48.0	fCantonese	daan6; taan4; daan2 (also)
U+5F48.0	fMandarin	dan4
U+5F48.0	fEnglish	 pellet; bullet; bomb
U+5F48.1	fUTF8	彈
U+5F48.1	fCNS11643	1-6C5F
U+5F48.1	fBig5	BC75
U+5F48.1	fGB	1-352F
U+5F48.1	fR/S	 57.12
U+5F48.1	fTotalStrokes	 15
U+5F48.1	fCangjie	NRRJ
U+5F48.1	fFourCorner	16256
U+5F48.1	fHakka	 tan2
U+5F48.1	fMandarin	tan2
U+5F48.1	fEnglish	[1] rebound [2] play (a stringed instrument sounded by snapping action) [3] impeach
U+5F48.2	fUTF8	彈
U+5F48.2	fCNS11643	1-6C5F
U+5F48.2	fBig5	BC75
U+5F48.2	fGB	1-352F
U+5F48.2	fR/S	 57.12
U+5F48.2	fTotalStrokes	 15
U+5F48.2	fCangjie	NRRJ
U+5F48.2	fFourCorner	16256
U+5F48.2	fMandarin	dan5
U+5F49.0	fUTF8	彉
U+5F49.0	fCNS11643	2-503F
U+5F49.0	fBig5	E576
U+5F49.0	fGB	5-3F29
U+5F49.0	fR/S	 57.12
U+5F49.0	fTotalStrokes	 15
U+5F49.0	fCangjie	NTLC
U+5F49.0	fFourCorner	14286
U+5F49.0	fCantonese	kwok3; gwok3 (also); kok3 (also); gok3 (also); fok3
U+5F49.0	fMandarin	kuo4; kuang4; huo4
U+5F4A.0	fUTF8	彊
U+5F4A.0	fCNS11643	1-7053
U+5F4A.0	fBig5	BECA
U+5F4A.0	fGB	E-294F
U+5F4A.0	fR/S	 57.13
U+5F4A.0	fTotalStrokes	 16
U+5F4A.0	fCangjie	NMWM
U+5F4A.0	fFourCorner	11216
U+5F4A.0	fHakka	  kiong2;  giong3; giong1
U+5F4A.0	fCantonese	goeng1; goeng6; koeng4; koeng5; koeng3
U+5F4A.0	fMandarin	qiang2; jiang1; qiang3; jiang4
U+5F4A.0	fEnglish	stubborn; uncompromising
U+5F4B.0	fUTF8	彋
U+5F4B.0	fCNS11643	2-567E
U+5F4B.0	fBig5	E974
U+5F4B.0	fGB	5-3F2E
U+5F4B.0	fR/S	 57.13
U+5F4B.0	fTotalStrokes	 16
U+5F4B.0	fCangjie	NWLV
U+5F4B.0	fFourCorner	16232
U+5F4B.0	fCantonese	wang4; waang4 (also); gwang1
U+5F4B.0	fMandarin	hong2; heng2
U+5F4C.0	fUTF8	彌
U+5F4C.0	fCNS11643	1-735A
U+5F4C.0	fBig5	C0B1
U+5F4C.0	fGB	1-4356
U+5F4C.0	fR/S	 57.14
U+5F4C.0	fTotalStrokes	 17
U+5F4C.0	fCangjie	NMFB
U+5F4C.0	fFourCorner	11227
U+5F4C.0	fHakka	  ni3; ni2; li1; li2; mi2
U+5F4C.0	fCantonese	nei4; mei4 (also); lei4 (also)
U+5F4C.0	fMandarin	mi2; mi3; yi2
U+5F4C.0	fEnglish	[1] fill; complete [2] more
U+5F4D.0	fUTF8	彍
U+5F4D.0	fCNS11643	3-5763
U+5F4D.0	fGB	3-3B64
U+5F4D.0	fR/S	 57.15
U+5F4D.0	fTotalStrokes	 18
U+5F4D.0	fCangjie	NITC
U+5F4D.0	fFourCorner	10286
U+5F4D.0	fHakka	 kok7; gwok7; kwok7
U+5F4D.0	fCantonese	kwok3; kok3 (also); gwok3 (also); gok3 (also)
U+5F4D.0	fMandarin	guo1; kuo4
U+5F4D.0	fEnglish	 draw a bow to the full
U+5F4E.0	fUTF8	彎
U+5F4E.0	fCNS11643	1-7B5F
U+5F4E.0	fBig5	C573
U+5F4E.0	fGB	1-4D64
U+5F4E.0	fR/S	 57.19
U+5F4E.0	fTotalStrokes	 22
U+5F4E.0	fCangjie	VFN
U+5F4E.0	fFourCorner	22207
U+5F4E.0	fHakka	  wan1; van1
U+5F4E.0	fCantonese	waan1
U+5F4E.0	fMandarin	wan1
U+5F4E.0	fEnglish	 bend; curve
U+5F4F.0	fUTF8	彏
U+5F4F.0	fCNS11643	2-6F27
U+5F4F.0	fBig5	F7D8
U+5F4F.0	fGB	3-3B79
U+5F4F.0	fR/S	 57.20
U+5F4F.0	fTotalStrokes	 23
U+5F4F.0	fCangjie	NBUE
U+5F4F.0	fFourCorner	16247
U+5F4F.0	fCantonese	fok3; gok3 (also); gwok3
U+5F4F.0	fMandarin	jue2
U+5F50.0	fUTF8	彐
U+5F50.0	fCNS11643	3-2140
U+5F50.0	fGB	0-6566
U+5F50.0	fR/S	 58.0
U+5F50.0	fTotalStrokes	 3
U+5F50.0	fCangjie	NMM
U+5F50.0	fFourCorner	17100
U+5F50.0	fHakka	 gi5; gi6
U+5F50.0	fCantonese	gai3
U+5F50.0	fMandarin	ji4
U+5F50.0	fEnglish	 snout Kangxi radical number 58
U+5F51.0	fUTF8	彑
U+5F51.0	fCNS11643	3-2141
U+5F51.0	fGB	E-2950
U+5F51.0	fR/S	 58.0
U+5F51.0	fTotalStrokes	 3
U+5F51.0	fCangjie	VNM
U+5F51.0	fFourCorner	27100
U+5F51.0	fHakka	 gi5
U+5F51.0	fCantonese	gai3
U+5F51.0	fMandarin	ji4
U+5F51.0	fEnglish	same as &#x5F50;
U+5F52.0	fUTF8	归
U+5F52.0	fCNS11643	3-224A
U+5F52.0	fGB	0-3969
U+5F52.0	fR/S	 58.2
U+5F52.0	fTotalStrokes	 5
U+5F52.0	fCangjie	LLSM
U+5F52.0	fFourCorner	27070
U+5F52.0	fHakka	 kwui1
U+5F52.0	fCantonese	gwai1; gwai6; kwai5
U+5F52.0	fMandarin	gui1; kui4
U+5F52.0	fEnglish	[1] non-classical form of &#x6B78; [2] simplified form of &#x6B78;
U+5F53.0	fUTF8	当
U+5F53.0	fCNS11643	3-2351
U+5F53.0	fGB	0-3531
U+5F53.0	fR/S	 42.3; 58.3
U+5F53.0	fTotalStrokes	 6
U+5F53.0	fCangjie	FSM
U+5F53.0	fFourCorner	90170
U+5F53.0	fHakka	 dong1; dong5
U+5F53.0	fCantonese	dong1; dong3; dong2
U+5F53.0	fMandarin	dang1; dang2
U+5F53.0	fEnglish	simplified form of &#x7576;
U+5F53.1	fUTF8	当
U+5F53.1	fCNS11643	3-2351
U+5F53.1	fGB	0-3531
U+5F53.1	fR/S	 42.3; 58.3
U+5F53.1	fTotalStrokes	 6
U+5F53.1	fCangjie	FSM
U+5F53.1	fFourCorner	90170
U+5F53.1	fMandarin	dang4
U+5F53.2	fUTF8	当
U+5F53.2	fCNS11643	3-2351
U+5F53.2	fGB	0-3531
U+5F53.2	fR/S	 42.3; 58.3
U+5F53.2	fTotalStrokes	 6
U+5F53.2	fCangjie	FSM
U+5F53.2	fFourCorner	90170
U+5F53.2	fMandarin	dang5
U+5F54.0	fUTF8	彔
U+5F54.0	fCNS11643	2-2655
U+5F54.0	fBig5	CC74
U+5F54.0	fGB	E-2951
U+5F54.0	fR/S	 58.5
U+5F54.0	fTotalStrokes	 8
U+5F54.0	fCangjie	VNME
U+5F54.0	fFourCorner	27232
U+5F54.0	fHakka	 liuk8
U+5F54.0	fCantonese	luk6
U+5F54.0	fMandarin	lu4
U+5F54.0	fEnglish	[1] carve wood [2] an abbreviated form of
U+5F55.0	fUTF8	录
U+5F55.0	fCNS11643	3-2871
U+5F55.0	fGB	0-423C
U+5F55.0	fR/S	 58.5
U+5F55.0	fTotalStrokes	 8
U+5F55.0	fCangjie	NMME
U+5F55.0	fFourCorner	17130
U+5F55.0	fCantonese	luk6; luk2 (also)
U+5F55.0	fMandarin	lu4; l4
U+5F55.0	fEnglish	simplified form of &#x9332;
U+5F56.0	fUTF8	彖
U+5F56.0	fCNS11643	2-2A3E
U+5F56.0	fBig5	CEBD
U+5F56.0	fGB	0-6568
U+5F56.0	fR/S	 58.6
U+5F56.0	fTotalStrokes	 9
U+5F56.0	fCangjie	VNMO
U+5F56.0	fFourCorner	27232
U+5F56.0	fHakka	 ton5;  chon5; ton3; con5
U+5F56.0	fCantonese	teon3; teon5
U+5F56.0	fMandarin	tuan4
U+5F56.0	fEnglish	[1] hog; hedgehog; porcupine [2] a chapter of generalisation on divination in the Book of Changes; a work expounding the Book of Changes
U+5F57.0	fUTF8	彗
U+5F57.0	fCNS11643	1-5A31
U+5F57.0	fBig5	B16B
U+5F57.0	fGB	0-6567
U+5F57.0	fR/S	 58.8
U+5F57.0	fTotalStrokes	 11
U+5F57.0	fCangjie	QJSM
U+5F57.0	fFourCorner	55174
U+5F57.0	fHakka	  sui5; fui5; fui6; fi5; sui6
U+5F57.0	fCantonese	wai6; seoi6 (also)
U+5F57.0	fMandarin	hui4; sui4
U+5F57.0	fEnglish	[1] broom [2] comet [3] expose to sunlight
U+5F58.0	fUTF8	彘
U+5F58.0	fCNS11643	2-3B58
U+5F58.0	fBig5	D8F4
U+5F58.0	fGB	0-6569
U+5F58.0	fR/S	 58.9
U+5F58.0	fTotalStrokes	 12
U+5F58.0	fCangjie	VMPOP
U+5F58.0	fFourCorner	27711
U+5F58.0	fHakka	 chi3; cih3; cih6; ci3; chii3
U+5F58.0	fCantonese	zi6; zai6 (also)
U+5F58.0	fMandarin	zhi4
U+5F58.0	fEnglish	 swine; hog; pig
U+5F59.0	fUTF8	彙
U+5F59.0	fCNS11643	1-6370
U+5F59.0	fBig5	B74A
U+5F59.0	fGB	1-783D
U+5F59.0	fR/S	 58.10
U+5F59.0	fTotalStrokes	 13
U+5F59.0	fCangjie	VMBWD
U+5F59.0	fFourCorner	27904
U+5F59.0	fHakka	 lui5; lui6; lui3
U+5F59.0	fCantonese	wai6; wui6
U+5F59.0	fMandarin	hui4; wei4
U+5F59.0	fEnglish	[1] category; class; series [2] categorise; classify [3] collect (materials, data, etc.)
U+5F5A.0	fUTF8	彚
U+5F5A.0	fCNS11643	3-4123
U+5F5A.0	fGB	E-2952
U+5F5A.0	fR/S	 58.10
U+5F5A.0	fTotalStrokes	 13
U+5F5A.0	fCangjie	NMBWD
U+5F5A.0	fFourCorner	17900
U+5F5A.0	fHakka	 lui5
U+5F5A.0	fCantonese	wai6; wui6
U+5F5A.0	fMandarin	hui4; wei4
U+5F5A.0	fEnglish	same as &#x5F59;
U+5F5B.0	fUTF8	彛
U+5F5B.0	fCNS11643	3-505E
U+5F5B.0	fGB	E-2953
U+5F5B.0	fR/S	 58.13
U+5F5B.0	fTotalStrokes	 16
U+5F5B.0	fCangjie	SMFHT
U+5F5B.0	fFourCorner	17442
U+5F5B.0	fHakka	 ji2; ri2
U+5F5B.0	fCantonese	ji4
U+5F5B.0	fMandarin	yi2
U+5F5B.0	fEnglish	yi; tripod, wine vessel; rule
U+5F5C.0	fUTF8	彜
U+5F5C.0	fCNS11643	F-5443
U+5F5C.0	fGB	5-3E47
U+5F5C.0	fR/S	 58.13
U+5F5C.0	fTotalStrokes	 16
U+5F5C.0	fCangjie	VMFHT
U+5F5C.0	fFourCorner	2744
U+5F5C.0	fCantonese	ji4
U+5F5C.0	fMandarin	yi2
U+5F5C.0	fEnglish	 yi tripod; wine vessel rule
U+5F5D.0	fUTF8	彝
U+5F5D.0	fCNS11643	3-5764
U+5F5D.0	fBig5	C255
U+5F5D.0	fGB	0-524D
U+5F5D.0	fR/S	  58.15
U+5F5D.0	fTotalStrokes	 18
U+5F5D.0	fCangjie	VMFFT
U+5F5D.0	fFourCorner	27449
U+5F5D.0	fHakka	 ji2; i2
U+5F5D.0	fCantonese	ji4
U+5F5D.0	fMandarin	yi2
U+5F5D.0	fEnglish	 yi tripod; wine vessel rule
U+5F5E.0	fUTF8	彞
U+5F5E.0	fCNS11643	1-7641
U+5F5E.0	fGB	E-2954
U+5F5E.0	fR/S	 58.15
U+5F5E.0	fTotalStrokes	 18
U+5F5E.0	fCangjie	NMFFT
U+5F5E.0	fCantonese	ji4
U+5F5E.0	fMandarin	yi2
U+5F5E.0	fEnglish	[1] vessel for wine; goblet [2] laws or regulations [3] normal nature [4] regular; constant [5] the vessels or items used in ceremonies at temples, etc., such as bells, tripods, chalices, etc.
U+5F5F.0	fUTF8	彟
U+5F5F.0	fGB	8-2E3D
U+5F5F.0	fR/S	 58.16; 58.23
U+5F5F.0	fTotalStrokes	 19
U+5F5F.0	fCangjie	XSITO
U+5F5F.0	fCantonese	wok6; wok3 (also)
U+5F5F.0	fMandarin	huo4
U+5F60.0	fUTF8	彠
U+5F60.0	fCNS11643	3-6164
U+5F60.0	fGB	5-3E46
U+5F60.0	fR/S	 58.23
U+5F60.0	fTotalStrokes	 26
U+5F60.0	fCangjie	SITOE
U+5F60.0	fFourCorner	14340
U+5F60.0	fHakka	 wok7; vok7
U+5F60.0	fCantonese	wok6
U+5F60.0	fMandarin	wo4; huo4; yue1
U+5F60.0	fEnglish	same as &#x84A6;
U+5F61.0	fUTF8	彡
U+5F61.0	fCNS11643	3-2142
U+5F61.0	fGB	0-616A
U+5F61.0	fR/S	 59.0
U+5F61.0	fTotalStrokes	 3
U+5F61.0	fCangjie	HHH
U+5F61.0	fFourCorner	20200
U+5F61.0	fHakka	 sam1
U+5F61.0	fCantonese	saam1
U+5F61.0	fMandarin	shan1; xian1
U+5F61.0	fEnglish	 hair Kangxi radical number 59
U+5F62.0	fUTF8	形
U+5F62.0	fCNS11643	1-4A30
U+5F62.0	fBig5	A7CE
U+5F62.0	fGB	0-504E
U+5F62.0	fR/S	 59.4
U+5F62.0	fTotalStrokes	 7
U+5F62.0	fCangjie	MTHHH
U+5F62.0	fFourCorner	12422
U+5F62.0	fHakka	  hin2; hin3
U+5F62.0	fCantonese	jing4
U+5F62.0	fMandarin	xing2
U+5F62.0	fEnglish	[1] form; shape; appearance; figure [2] complexion [3] terrain; contour [4] expression; description; describe [5] in comparison [6] show; manifest
U+5F62.1	fUTF8	形
U+5F62.1	fCNS11643	1-4A30
U+5F62.1	fBig5	A7CE
U+5F62.1	fGB	0-504E
U+5F62.1	fR/S	 59.4
U+5F62.1	fTotalStrokes	 7
U+5F62.1	fCangjie	MTHHH
U+5F62.1	fFourCorner	12422
U+5F62.1	fMandarin	xing5
U+5F63.0	fUTF8	彣
U+5F63.0	fCNS11643	3-2572
U+5F63.0	fGB	5-5076
U+5F63.0	fR/S	  59.4
U+5F63.0	fTotalStrokes	 7
U+5F63.0	fCangjie	YKHHH
U+5F63.0	fFourCorner	02420
U+5F63.0	fCantonese	man4
U+5F63.0	fMandarin	wen2
U+5F64.0	fUTF8	彤
U+5F64.0	fCNS11643	1-4A2F
U+5F64.0	fBig5	A7CD
U+5F64.0	fGB	0-4D2E
U+5F64.0	fR/S	 59.4
U+5F64.0	fTotalStrokes	 7
U+5F64.0	fCangjie	BYHHH
U+5F64.0	fFourCorner	72422
U+5F64.0	fHakka	 tung2
U+5F64.0	fCantonese	tung4
U+5F64.0	fMandarin	tong2
U+5F64.0	fEnglish	[1] red; vermilion [2] name of an ancient state
U+5F65.0	fUTF8	彥
U+5F65.0	fCNS11643	1-507D
U+5F65.0	fBig5	ABDB
U+5F65.0	fGB	E-2955
U+5F65.0	fR/S	 59.6
U+5F65.0	fTotalStrokes	 9
U+5F65.0	fCangjie	YKMHH
U+5F65.0	fFourCorner	00222
U+5F65.0	fHakka	 ngien5;  ngan5; ngan6; ngan3; ngien6; ngian5
U+5F65.0	fCantonese	jin6
U+5F65.0	fMandarin	yan4
U+5F65.0	fEnglish	 man of ability and virtue; erudite scholar
U+5F66.0	fUTF8	彦
U+5F66.0	fCNS11643	3-2C50
U+5F66.0	fGB	0-5165
U+5F66.0	fR/S	 59.6
U+5F66.0	fTotalStrokes	 9
U+5F66.0	fCangjie	YHHHH
U+5F66.0	fFourCorner	00220
U+5F66.0	fHakka	 ngien5; ngan5
U+5F66.0	fCantonese	jin6
U+5F66.0	fMandarin	yan4
U+5F66.0	fEnglish	elegant
U+5F67.0	fUTF8	彧
U+5F67.0	fCNS11643	2-2E7D
U+5F67.0	fBig5	D17B
U+5F67.0	fGB	3-4630
U+5F67.0	fR/S	 59.7
U+5F67.0	fTotalStrokes	 10
U+5F67.0	fCangjie	IKRM
U+5F67.0	fFourCorner	53100
U+5F67.0	fHakka	  juk7; ruk7; juk8
U+5F67.0	fCantonese	juk1
U+5F67.0	fMandarin	yu4
U+5F67.0	fEnglish	refined; cultured; polished; learned and accomplished; abundant
U+5F68.0	fUTF8	彨
U+5F68.0	fGB	8-2E3C
U+5F68.0	fR/S	 59.19; 59.7
U+5F68.0	fTotalStrokes	 10
U+5F68.0	fCangjie	XMBHH
U+5F68.0	fCantonese	ci1
U+5F68.0	fMandarin	chi1
U+5F69.0	fUTF8	彩
U+5F69.0	fCNS11643	1-5A33
U+5F69.0	fBig5	B16D
U+5F69.0	fGB	0-324A
U+5F69.0	fR/S	 59.8
U+5F69.0	fTotalStrokes	 11
U+5F69.0	fCangjie	BDHHH
U+5F69.0	fFourCorner	22922
U+5F69.0	fHakka	  cai3; coi3
U+5F69.0	fCantonese	coi2
U+5F69.0	fMandarin	cai3
U+5F69.0	fEnglish	[1] colors; variegated colors [2] makeup in various Chinese operas [3] special feats or stunts in Chinese operas [4] ornamental; brilliant; gay [5] stakes in a gambling game; prize money
U+5F69.1	fUTF8	彩
U+5F69.1	fCNS11643	1-5A33
U+5F69.1	fBig5	B16D
U+5F69.1	fGB	0-324A
U+5F69.1	fR/S	 59.8
U+5F69.1	fTotalStrokes	 11
U+5F69.1	fCangjie	BDHHH
U+5F69.1	fFourCorner	22922
U+5F69.1	fMandarin	cai5
U+5F6A.0	fUTF8	彪
U+5F6A.0	fCNS11643	1-5D29
U+5F6A.0	fBig5	B343
U+5F6A.0	fGB	0-316B
U+5F6A.0	fR/S	 59.8
U+5F6A.0	fTotalStrokes	 11
U+5F6A.0	fCangjie	YUHHH
U+5F6A.0	fFourCorner	22212
U+5F6A.0	fHakka	  biau1; beu1; piau1
U+5F6A.0	fCantonese	biu1
U+5F6A.0	fMandarin	biao1
U+5F6A.0	fEnglish	[1] tiger cub [2] tiger stripes; stripes or streaks on the skin of a tiger [3] tall and big; shining and brilliant; outstanding [4] a Chinese family name
U+5F6B.0	fUTF8	彫
U+5F6B.0	fCNS11643	1-5A34
U+5F6B.0	fBig5	B16E
U+5F6B.0	fGB	E-2956
U+5F6B.0	fR/S	 59.8
U+5F6B.0	fTotalStrokes	 11
U+5F6B.0	fCangjie	BRHHH
U+5F6B.0	fFourCorner	72222
U+5F6B.0	fHakka	 diau1; tiau1
U+5F6B.0	fCantonese	diu1; tiu1
U+5F6B.0	fMandarin	diao1
U+5F6B.0	fEnglish	[1] carve; paint; engrave; tattoo [2] adorn or decorate [3] wither; emaciated
U+5F6C.0	fUTF8	彬
U+5F6C.0	fCNS11643	1-5A32
U+5F6C.0	fBig5	B16C
U+5F6C.0	fGB	0-3172
U+5F6C.0	fR/S	 59.8
U+5F6C.0	fTotalStrokes	 11
U+5F6C.0	fCangjie	DDHH
U+5F6C.0	fFourCorner	42922
U+5F6C.0	fHakka	  bin1
U+5F6C.0	fCantonese	ban1
U+5F6C.0	fMandarin	bin1; ban1
U+5F6C.0	fEnglish	[1] intelligent; refined and gentle [2] a Chinese family name
U+5F6D.0	fUTF8	彭
U+5F6D.0	fCNS11643	1-5F25
U+5F6D.0	fBig5	B45E
U+5F6D.0	fGB	0-456D
U+5F6D.0	fR/S	 59.9
U+5F6D.0	fTotalStrokes	 12
U+5F6D.0	fCangjie	GTHHH
U+5F6D.0	fFourCorner	42122
U+5F6D.0	fHakka	 pang2; pang3; pang5
U+5F6D.0	fCantonese	paang4; bong1; pong4; pang4
U+5F6D.0	fMandarin	peng2; bang1; beng1; pang2
U+5F6D.0	fEnglish	[1] a Chinese family name [2] big [3] longevity [4] proud
U+5F6E.0	fUTF8	彮
U+5F6E.0	fCNS11643	3-4124
U+5F6E.0	fGB	5-3C5E
U+5F6E.0	fR/S	  59.10
U+5F6E.0	fTotalStrokes	 13
U+5F6E.0	fCangjie	JRHHH
U+5F6E.0	fFourCorner	32620
U+5F6E.0	fCantonese	jung5
U+5F6E.0	fMandarin	yong3
U+5F6F.0	fUTF8	彯
U+5F6F.0	fCNS11643	2-4A27
U+5F6F.0	fBig5	E1C0
U+5F6F.0	fGB	5-3C5F
U+5F6F.0	fR/S	 59.11
U+5F6F.0	fTotalStrokes	 14
U+5F6F.0	fCangjie	MFHHH
U+5F6F.0	fFourCorner	12922
U+5F6F.0	fHakka	 piau1; p'ieu1; piau5; peu5
U+5F6F.0	fCantonese	piu1
U+5F6F.0	fMandarin	piao1; piao4
U+5F6F.0	fEnglish	ornamented (as with a tassel)
U+5F70.0	fUTF8	彰
U+5F70.0	fCNS11643	1-6864
U+5F70.0	fBig5	B9FC
U+5F70.0	fGB	0-5543
U+5F70.0	fR/S	 59.11
U+5F70.0	fTotalStrokes	 14
U+5F70.0	fCangjie	YJHHH
U+5F70.0	fFourCorner	02422
U+5F70.0	fHakka	 zong1;  zhong1
U+5F70.0	fCantonese	zoeng1
U+5F70.0	fMandarin	zhang1
U+5F70.0	fEnglish	[1] ornamental; colorful [2] evident; clear; obvious [3] manifest; make known; display
U+5F71.0	fUTF8	影
U+5F71.0	fCNS11643	1-6C60
U+5F71.0	fBig5	BC76
U+5F71.0	fGB	0-5330
U+5F71.0	fR/S	 59.12
U+5F71.0	fTotalStrokes	 15
U+5F71.0	fCangjie	AFHHH
U+5F71.0	fFourCorner	62922
U+5F71.0	fHakka	  jang3; rang3; jang5
U+5F71.0	fCantonese	jing2; jeng2
U+5F71.0	fMandarin	ying3
U+5F71.0	fEnglish	[1] shadow; image; reflection [2] trace; vague impression [3] copy and imitate [4] hide; conceal [5] sundial
U+5F72.0	fUTF8	彲
U+5F72.0	fCNS11643	3-5F45
U+5F72.0	fGB	5-3C5A
U+5F72.0	fR/S	  59.19
U+5F72.0	fTotalStrokes	 22
U+5F72.0	fCangjie	MPHHH
U+5F72.0	fFourCorner	12220
U+5F72.0	fCantonese	ci1
U+5F72.0	fMandarin	chi1
U+5F72.0	fEnglish	yellow dragon
U+5F73.0	fUTF8	彳
U+5F73.0	fCNS11643	2-212C
U+5F73.0	fBig5	C94C
U+5F73.0	fGB	0-615C
U+5F73.0	fR/S	 60.0
U+5F73.0	fTotalStrokes	 3
U+5F73.0	fCangjie	HO
U+5F73.0	fFourCorner	20202
U+5F73.0	fHakka	 chit7; ts'iit7; cit7
U+5F73.0	fCantonese	cik1
U+5F73.0	fMandarin	chi4
U+5F73.0	fEnglish	[1] step taken with the left foot [2] short steps or paces [3] Zhuyin symbol [4] Kangxi radical 60
U+5F74.0	fUTF8	彴
U+5F74.0	fCNS11643	2-223F
U+5F74.0	fBig5	C9DF
U+5F74.0	fGB	3-3978
U+5F74.0	fR/S	 60.3
U+5F74.0	fTotalStrokes	 6
U+5F74.0	fCangjie	HOPI
U+5F74.0	fFourCorner	27220
U+5F74.0	fHakka	 shok7; ch'ok7; cok7; sok7
U+5F74.0	fCantonese	zoek3; coek3
U+5F74.0	fMandarin	zhuo2
U+5F74.0	fEnglish	bridge
U+5F75.0	fUTF8	彵
U+5F75.0	fCNS11643	4-226D
U+5F75.0	fGB	E-2957
U+5F75.0	fR/S	  60.3
U+5F75.0	fTotalStrokes	 6
U+5F75.0	fCangjie	HOPD
U+5F75.0	fFourCorner	24210
U+5F75.0	fCantonese	to5
U+5F75.0	fMandarin	tuo3
U+5F76.0	fUTF8	彶
U+5F76.0	fCNS11643	2-2375
U+5F76.0	fBig5	CAD5
U+5F76.0	fGB	5-3C26
U+5F76.0	fR/S	 60.4
U+5F76.0	fTotalStrokes	 7
U+5F76.0	fCangjie	HONHE
U+5F76.0	fFourCorner	27247
U+5F76.0	fHakka	 giak7; giak8
U+5F76.0	fCantonese	gap1
U+5F76.0	fMandarin	ji2
U+5F76.0	fEnglish	[1] walk fast; hurry; run [2] same as &#x20AF3; [3] same as &#x6C72;
U+5F77.0	fUTF8	彷
U+5F77.0	fCNS11643	1-4A31
U+5F77.0	fBig5	A7CF
U+5F77.0	fGB	0-615D
U+5F77.0	fR/S	 60.4
U+5F77.0	fTotalStrokes	 7
U+5F77.0	fCangjie	HOYHS
U+5F77.0	fFourCorner	20227
U+5F77.0	fHakka	  fong3;  pong2
U+5F77.0	fCantonese	pong4; fong2
U+5F77.0	fMandarin	pang2
U+5F77.0	fEnglish	hesitating; unsettled
U+5F77.1	fUTF8	彷
U+5F77.1	fCNS11643	1-4A31
U+5F77.1	fBig5	A7CF
U+5F77.1	fGB	0-615D
U+5F77.1	fR/S	 60.4
U+5F77.1	fTotalStrokes	 7
U+5F77.1	fCangjie	HOYHS
U+5F77.1	fFourCorner	20227
U+5F77.1	fMandarin	fang3
U+5F77.1	fEnglish	like; resembling; similar to; resemble
U+5F78.0	fUTF8	彸
U+5F78.0	fCNS11643	2-2374
U+5F78.0	fBig5	CAD4
U+5F78.0	fGB	3-3979
U+5F78.0	fR/S	 60.4
U+5F78.0	fTotalStrokes	 7
U+5F78.0	fCangjie	HOCI
U+5F78.0	fFourCorner	28232
U+5F78.0	fHakka	 zhung1; zung1
U+5F78.0	fCantonese	zung1
U+5F78.0	fMandarin	zhong1
U+5F78.0	fEnglish	restless; agitated
U+5F79.0	fUTF8	役
U+5F79.0	fCNS11643	1-4A32
U+5F79.0	fBig5	A7D0
U+5F79.0	fGB	0-525B
U+5F79.0	fR/S	 60.4
U+5F79.0	fTotalStrokes	 7
U+5F79.0	fCangjie	HOHNE
U+5F79.0	fFourCorner	27247
U+5F79.0	fHakka	  jit8; rit8
U+5F79.0	fCantonese	jik6
U+5F79.0	fMandarin	yi4
U+5F79.0	fEnglish	[1] military service [2] servant; laborer; serve [3] guard the frontier [4] dispatch [5] employ a s a servant [6] do; undertake
U+5F7A.0	fUTF8	彺
U+5F7A.0	fCNS11643	4-242C
U+5F7A.0	fGB	E-2958
U+5F7A.0	fR/S	  60.4
U+5F7A.0	fTotalStrokes	 7
U+5F7A.0	fCangjie	HOMG
U+5F7A.0	fFourCorner	21210
U+5F7A.0	fCantonese	wong4
U+5F7A.0	fMandarin	wang2
U+5F7B.0	fUTF8	彻
U+5F7B.0	fGB	0-3339
U+5F7B.0	fR/S	 60.4
U+5F7B.0	fTotalStrokes	 7
U+5F7B.0	fCangjie	HOPSH
U+5F7B.0	fHakka	 chet8
U+5F7B.0	fCantonese	cit3
U+5F7B.0	fMandarin	che4; zhe2
U+5F7B.0	fEnglish	simplified form of &#x5FB9;
U+5F7C.0	fUTF8	彼
U+5F7C.0	fCNS11643	1-4D3E
U+5F7C.0	fBig5	A9BC
U+5F7C.0	fGB	0-314B
U+5F7C.0	fR/S	 60.5
U+5F7C.0	fTotalStrokes	 8
U+5F7C.0	fCangjie	HODHE
U+5F7C.0	fFourCorner	24247
U+5F7C.0	fHakka	  bi3; bi1; pi1
U+5F7C.0	fCantonese	bei2
U+5F7C.0	fMandarin	bi3
U+5F7C.0	fEnglish	[1] that; those [2] the other; another [3] there
U+5F7D.0	fUTF8	彽
U+5F7D.0	fCNS11643	2-2658
U+5F7D.0	fBig5	CC77
U+5F7D.0	fGB	5-3C2E
U+5F7D.0	fR/S	 60.5
U+5F7D.0	fTotalStrokes	 8
U+5F7D.0	fCangjie	HOHPM
U+5F7D.0	fFourCorner	22240
U+5F7D.0	fHakka	 chi2; cih2; chii2
U+5F7D.0	fCantonese	dai1 (also); ci4
U+5F7D.0	fMandarin	chi2; di1; tui1
U+5F7D.0	fEnglish	 go to and fro
U+5F7E.0	fUTF8	彾
U+5F7E.0	fCNS11643	2-2657
U+5F7E.0	fBig5	CC76
U+5F7E.0	fGB	5-3C2C
U+5F7E.0	fR/S	 60.5
U+5F7E.0	fTotalStrokes	 8
U+5F7E.0	fCangjie	HOOII
U+5F7E.0	fFourCorner	28227
U+5F7E.0	fCantonese	ling4; ling5
U+5F7E.0	fMandarin	ling3; ling2
U+5F7F.0	fUTF8	彿
U+5F7F.0	fCNS11643	1-4D3D
U+5F7F.0	fBig5	A9BB
U+5F7F.0	fGB	E-2959
U+5F7F.0	fR/S	 60.5
U+5F7F.0	fTotalStrokes	 8
U+5F7F.0	fCangjie	HOLLN
U+5F7F.0	fFourCorner	25227
U+5F7F.0	fHakka	 fut7; fut8
U+5F7F.0	fCantonese	fat1
U+5F7F.0	fMandarin	fu2
U+5F7F.0	fEnglish	resembling; like; as if; similar to
U+5F80.0	fUTF8	往
U+5F80.0	fCNS11643	1-4D3B
U+5F80.0	fBig5	A9B9
U+5F80.0	fGB	0-4D79
U+5F80.0	fR/S	 60.5
U+5F80.0	fTotalStrokes	 8
U+5F80.0	fCangjie	HOYG
U+5F80.0	fFourCorner	20214
U+5F80.0	fHakka	  wong1; wong3; gong1; vong3; vong1
U+5F80.0	fCantonese	wong5
U+5F80.0	fMandarin	wang3; wang2
U+5F80.0	fEnglish	[1] go toward; depart; be bound for [2] past; formerly; bygone
U+5F80.1	fUTF8	往
U+5F80.1	fCNS11643	1-4D3B
U+5F80.1	fBig5	A9B9
U+5F80.1	fGB	0-4D79
U+5F80.1	fR/S	 60.5
U+5F80.1	fTotalStrokes	 8
U+5F80.1	fCangjie	HOYG
U+5F80.1	fFourCorner	20214
U+5F80.1	fMandarin	wang4
U+5F80.1	fEnglish	an adverb indicating time or direction
U+5F81.0	fUTF8	征
U+5F81.0	fCNS11643	1-4D3C
U+5F81.0	fBig5	A9BA
U+5F81.0	fGB	0-5577
U+5F81.0	fR/S	 60.5
U+5F81.0	fTotalStrokes	 8
U+5F81.0	fCangjie	HOMYM
U+5F81.0	fFourCorner	21211
U+5F81.0	fHakka	  zhin1; zin1; zihn1
U+5F81.0	fCantonese	zing1; cing4
U+5F81.0	fMandarin	zheng1
U+5F81.0	fEnglish	[1] invade; attack; conquer; reduce to submission; tame [2] journey far away [3] levy taxes; collect taxes [4] take; snatch [5] a Chinese family name
U+5F82.0	fUTF8	徂
U+5F82.0	fCNS11643	2-2656
U+5F82.0	fBig5	CC75
U+5F82.0	fGB	0-615E
U+5F82.0	fR/S	 60.5
U+5F82.0	fTotalStrokes	 8
U+5F82.0	fCangjie	HOBM
U+5F82.0	fFourCorner	27210
U+5F82.0	fHakka	 zu1; cu2; zii1; zih1
U+5F82.0	fCantonese	cou4
U+5F82.0	fMandarin	cu2
U+5F82.0	fEnglish	[1] go to; advance; go ahead [2] to [3] die [4] past
U+5F83.0	fUTF8	徃
U+5F83.0	fCNS11643	3-2876
U+5F83.0	fGB	E-295A
U+5F83.0	fR/S	 60.5
U+5F83.0	fTotalStrokes	 8
U+5F83.0	fCangjie	HOHQM
U+5F83.0	fFourCorner	25210
U+5F83.0	fHakka	 wong1; wong3; gong3; vong3; vong1
U+5F83.0	fCantonese	wong5
U+5F83.0	fMandarin	wang3; wang2; wang4
U+5F83.0	fEnglish	go, depart; past, formerly
U+5F84.0	fUTF8	径
U+5F84.0	fCNS11643	3-2875
U+5F84.0	fGB	0-3E36
U+5F84.0	fR/S	 60.5
U+5F84.0	fTotalStrokes	 8
U+5F84.0	fCangjie	HONOM
U+5F84.0	fFourCorner	27210
U+5F84.0	fHakka	 gang5
U+5F84.0	fCantonese	ging3; gaang3 (also)
U+5F84.0	fMandarin	jing4; fa2; pang2
U+5F84.0	fEnglish	simplified form of &#x5F91;
U+5F85.0	fUTF8	待
U+5F85.0	fCNS11643	1-5121
U+5F85.0	fBig5	ABDD
U+5F85.0	fGB	0-347D
U+5F85.0	fR/S	 60.6
U+5F85.0	fTotalStrokes	 9
U+5F85.0	fCangjie	HOGDI
U+5F85.0	fFourCorner	24241
U+5F85.0	fHakka	  toi5;  tai5; toi3; toi6; tai6
U+5F85.0	fCantonese	doi6; daai1
U+5F85.0	fMandarin	dai4
U+5F85.0	fEnglish	[1] treat; entertain [2] await; wait for [3] need [4] until [5] a Chinese family name
U+5F85.1	fUTF8	待
U+5F85.1	fCNS11643	1-5121
U+5F85.1	fBig5	ABDD
U+5F85.1	fGB	0-347D
U+5F85.1	fR/S	 60.6
U+5F85.1	fTotalStrokes	 9
U+5F85.1	fCangjie	HOGDI
U+5F85.1	fFourCorner	24241
U+5F85.1	fMandarin	dai1
U+5F85.1	fEnglish	[1] stay [2] later
U+5F86.0	fUTF8	徆
U+5F86.0	fCNS11643	2-2A3F
U+5F86.0	fBig5	CEBE
U+5F86.0	fGB	5-3C2F
U+5F86.0	fR/S	 60.6
U+5F86.0	fTotalStrokes	 9
U+5F86.0	fCangjie	HOMCW
U+5F86.0	fFourCorner	21260
U+5F86.0	fCantonese	sai1
U+5F86.0	fMandarin	xi1
U+5F87.0	fUTF8	徇
U+5F87.0	fCNS11643	1-5124
U+5F87.0	fBig5	ABE0
U+5F87.0	fGB	0-615F
U+5F87.0	fR/S	 60.6
U+5F87.0	fTotalStrokes	 9
U+5F87.0	fCangjie	HOPA
U+5F87.0	fFourCorner	27220
U+5F87.0	fHakka	  sun2
U+5F87.0	fCantonese	seon1; seon6; ceon4
U+5F87.0	fMandarin	xun4
U+5F87.0	fEnglish	[1] comply with; follow [2] show; display [3] issue orders in the army [4] die for a cause
U+5F87.1	fUTF8	徇
U+5F87.1	fCNS11643	1-5124
U+5F87.1	fBig5	ABE0
U+5F87.1	fGB	0-615F
U+5F87.1	fR/S	 60.6
U+5F87.1	fTotalStrokes	 9
U+5F87.1	fCangjie	HOPA
U+5F87.1	fFourCorner	27220
U+5F87.1	fMandarin	xun2
U+5F87.1	fEnglish	[1] pervading [2] profit
U+5F88.0	fUTF8	很
U+5F88.0	fCNS11643	1-507E
U+5F88.0	fBig5	ABDC
U+5F88.0	fGB	0-3A5C
U+5F88.0	fR/S	 60.6
U+5F88.0	fTotalStrokes	 9
U+5F88.0	fCangjie	HOAV
U+5F88.0	fFourCorner	27232
U+5F88.0	fHakka	  hen3;  han5; han2; hen6; ken3; hen2
U+5F88.0	fCantonese	han2
U+5F88.0	fMandarin	hen3
U+5F88.0	fEnglish	[1] very; quite; much [2] fierce; cruel; truculent [3] disobedient; intractable; quarrelsome [4] dispute; quarrel
U+5F89.0	fUTF8	徉
U+5F89.0	fCNS11643	1-5126
U+5F89.0	fBig5	ABE2
U+5F89.0	fGB	0-6160
U+5F89.0	fR/S	 60.6
U+5F89.0	fTotalStrokes	 9
U+5F89.0	fCangjie	HOTQ
U+5F89.0	fFourCorner	28251
U+5F89.0	fHakka	  jong2; rong2
U+5F89.0	fCantonese	joeng4
U+5F89.0	fMandarin	yang2
U+5F89.0	fEnglish	[1] stray; roam [2] hesitating; unsettled
U+5F8A.0	fUTF8	徊
U+5F8A.0	fCNS11643	1-5122
U+5F8A.0	fBig5	ABDE
U+5F8A.0	fGB	0-3B32
U+5F8A.0	fR/S	 60.6
U+5F8A.0	fTotalStrokes	 9
U+5F8A.0	fCangjie	HOWR
U+5F8A.0	fFourCorner	26200
U+5F8A.0	fHakka	 fui2; fi2; fui3
U+5F8A.0	fCantonese	wui4
U+5F8A.0	fMandarin	huai2; hui2
U+5F8A.0	fEnglish	linger; walk to and fro; hesitating
U+5F8B.0	fUTF8	律
U+5F8B.0	fCNS11643	1-5123
U+5F8B.0	fBig5	ABDF
U+5F8B.0	fGB	0-4249
U+5F8B.0	fR/S	 60.6
U+5F8B.0	fTotalStrokes	 9
U+5F8B.0	fCangjie	HOLQ
U+5F8B.0	fFourCorner	25207
U+5F8B.0	fHakka	  lut8; lit8
U+5F8B.0	fCantonese	leot6; leot2 (also)
U+5F8B.0	fMandarin	l4; luu4; lu4
U+5F8B.0	fEnglish	[1] statute; law; regulation; rule [2] bind by law; control or restrain; discipline [3] a series of standard bamboo tuning pitch pipes used in ancient music [4] a form in Chinese poetry - a stanza of eight lines
U+5F8C.0	fUTF8	後
U+5F8C.0	fCNS11643	1-5125
U+5F8C.0	fBig5	ABE1
U+5F8C.0	fGB	0-6161
U+5F8C.0	fR/S	 60.6
U+5F8C.0	fTotalStrokes	 9
U+5F8C.0	fCangjie	HOVIE
U+5F8C.0	fFourCorner	22247
U+5F8C.0	fHakka	 heu5; heu6; hiu3; hieu5; heu1
U+5F8C.0	fCantonese	hau6; hau2 (also)
U+5F8C.0	fMandarin	hou4
U+5F8C.0	fEnglish	[1] behind; rear; at the back of [2] afterwards; come after [3] descendents; posterity [4] an auxiliary verb to indicate 'then' or 'afterwards' [5] a Chinese family name [6] simplified form of &#x540E;
U+5F8D.0	fUTF8	徍
U+5F8D.0	fCNS11643	4-2931
U+5F8D.0	fGB	E-295B
U+5F8D.0	fR/S	  60.6
U+5F8D.0	fTotalStrokes	 9
U+5F8D.0	fCangjie	HOGG
U+5F8D.0	fFourCorner	24210
U+5F8D.0	fCantonese	wong5
U+5F8D.0	fMandarin	wa1; wang3
U+5F8E.0	fUTF8	徎
U+5F8E.0	fCNS11643	3-3070
U+5F8E.0	fGB	3-397D
U+5F8E.0	fR/S	  60.7
U+5F8E.0	fTotalStrokes	 10
U+5F8E.0	fCangjie	HORMG
U+5F8E.0	fFourCorner	26210
U+5F8E.0	fHakka	 chin3; cihn3
U+5F8E.0	fCantonese	cing2
U+5F8E.0	fMandarin	cheng3
U+5F8E.0	fEnglish	by-path
U+5F8F.0	fUTF8	徏
U+5F8F.0	fCNS11643	3-3072
U+5F8F.0	fGB	E-295C
U+5F8F.0	fR/S	 60.7
U+5F8F.0	fTotalStrokes	 10
U+5F8F.0	fCangjie	HOYLH
U+5F8F.0	fFourCorner	21220
U+5F8F.0	fHakka	 zhit7; tsiit7; zit7
U+5F8F.0	fCantonese	zik1
U+5F8F.0	fMandarin	zhi4; de2; shan3
U+5F8F.0	fEnglish	same as &#x965F;
U+5F90.0	fUTF8	徐
U+5F90.0	fCNS11643	1-5541
U+5F90.0	fBig5	AE7D
U+5F90.0	fGB	0-506C
U+5F90.0	fR/S	 60.7
U+5F90.0	fTotalStrokes	 10
U+5F90.0	fCangjie	HOOMD
U+5F90.0	fFourCorner	28294
U+5F90.0	fHakka	  ci2; ci3
U+5F90.0	fCantonese	ceoi4
U+5F90.0	fMandarin	xu2
U+5F90.0	fEnglish	[1] slow; calm; composed; gently [2] Hsuu - a state in ancient China occupying part of today's Anhwei Province
U+5F91.0	fUTF8	徑
U+5F91.0	fCNS11643	1-5540
U+5F91.0	fBig5	AE7C
U+5F91.0	fGB	1-3E36
U+5F91.0	fR/S	 60.7
U+5F91.0	fTotalStrokes	 10
U+5F91.0	fCangjie	HOMVM
U+5F91.0	fFourCorner	21211
U+5F91.0	fHakka	  gang5;  gin5
U+5F91.0	fCantonese	ging3; gaang3 (also)
U+5F91.0	fMandarin	jing4; fa2; pang2
U+5F91.0	fEnglish	[1] narrow path; byway [2] diameter [3] decide and proceed (to do something, etc.) without getting orders; direct; straight [4] already - implying a sense of surprise
U+5F92.0	fUTF8	徒
U+5F92.0	fCNS11643	1-553F
U+5F92.0	fBig5	AE7B
U+5F92.0	fGB	0-4D3D
U+5F92.0	fR/S	 60.7
U+5F92.0	fTotalStrokes	 10
U+5F92.0	fCangjie	HOGYO
U+5F92.0	fFourCorner	24281
U+5F92.0	fHakka	  tu2; tu3
U+5F92.0	fCantonese	tou4; tou2 (also)
U+5F92.0	fMandarin	tu2
U+5F92.0	fEnglish	[1] disciple; follower; pupil; apprentice [2] go on foot [3] crowd; gang; group of people [4] punishment [5] only; merely; in vain [6] empty; as empty-handed; barehanded [7] foot soldiers; infantry
U+5F93.0	fUTF8	従
U+5F93.0	fCNS11643	3-3071
U+5F93.0	fGB	E-295D
U+5F93.0	fR/S	 60.7
U+5F93.0	fTotalStrokes	 10
U+5F93.0	fCangjie	HOTYO
U+5F93.0	fFourCorner	28280
U+5F93.0	fHakka	 ciung2; ziung3; siung5
U+5F93.0	fCantonese	cung4
U+5F93.0	fMandarin	cong2; chuang2
U+5F93.0	fEnglish	simplified form of &#x5F9E;
U+5F94.0	fUTF8	徔
U+5F94.0	fCNS11643	3-6476
U+5F94.0	fGB	E-295E
U+5F94.0	fR/S	 60.6
U+5F94.0	fTotalStrokes	 9
U+5F94.0	fCangjie	HOTIO
U+5F94.0	fFourCorner	2423
U+5F94.0	fCantonese	zi1
U+5F95.0	fUTF8	徕
U+5F95.0	fGB	0-6162
U+5F95.0	fR/S	 60.7
U+5F95.0	fTotalStrokes	 10
U+5F95.0	fCangjie	HODT
U+5F95.0	fHakka	 loi3; loi5
U+5F95.0	fCantonese	loi4; loi6
U+5F95.0	fMandarin	lai2; lai4
U+5F95.0	fEnglish	simplified form of &#x5FA0;
U+5F96.0	fUTF8	徖
U+5F96.0	fCNS11643	2-3533
U+5F96.0	fBig5	D54F
U+5F96.0	fGB	E-295F
U+5F96.0	fR/S	 60.8
U+5F96.0	fTotalStrokes	 11
U+5F96.0	fCangjie	HOJMF
U+5F96.0	fFourCorner	23291
U+5F96.0	fCantonese	cung4
U+5F96.0	fMandarin	cong2
U+5F97.0	fUTF8	得
U+5F97.0	fCNS11643	1-5A35
U+5F97.0	fBig5	B16F
U+5F97.0	fGB	0-3543
U+5F97.0	fR/S	 60.8
U+5F97.0	fTotalStrokes	 11
U+5F97.0	fCangjie	HOAMI
U+5F97.0	fFourCorner	26241
U+5F97.0	fHakka	  det7; diet7
U+5F97.0	fCantonese	dak1
U+5F97.0	fMandarin	de2; de1
U+5F97.0	fEnglish	[1] get; obtain; acquire; gain; attain; effect; win [2] complacent [3] agreement; harmony [4] can; may; be able to [5] all right!
U+5F97.1	fUTF8	得
U+5F97.1	fCNS11643	1-5A35
U+5F97.1	fBig5	B16F
U+5F97.1	fGB	0-3543
U+5F97.1	fR/S	 60.8
U+5F97.1	fTotalStrokes	 11
U+5F97.1	fCangjie	HOAMI
U+5F97.1	fFourCorner	26241
U+5F97.1	fMandarin	dei3
U+5F97.1	fEnglish	[1] must; should; ought to [2] need; take
U+5F97.2	fUTF8	得
U+5F97.2	fCNS11643	1-5A35
U+5F97.2	fBig5	B16F
U+5F97.2	fGB	0-3543
U+5F97.2	fR/S	 60.8
U+5F97.2	fTotalStrokes	 11
U+5F97.2	fCangjie	HOAMI
U+5F97.2	fFourCorner	26241
U+5F97.2	fMandarin	de5
U+5F97.2	fEnglish	an adverbial expletive
U+5F98.0	fUTF8	徘
U+5F98.0	fCNS11643	1-5A38
U+5F98.0	fBig5	B172
U+5F98.0	fGB	0-4547
U+5F98.0	fR/S	 60.8
U+5F98.0	fTotalStrokes	 11
U+5F98.0	fCangjie	HOLMY
U+5F98.0	fFourCorner	21211
U+5F98.0	fHakka	  pai2; fui1; fi1; pi2
U+5F98.0	fCantonese	pui4
U+5F98.0	fMandarin	pai2; pei2
U+5F98.0	fEnglish	[1] walk to and fro; move around; linger around; pace up and down [2] hesitating; irresolute; indecisive
U+5F99.0	fUTF8	徙
U+5F99.0	fCNS11643	1-5A36
U+5F99.0	fBig5	B170
U+5F99.0	fGB	0-6163
U+5F99.0	fR/S	 60.8
U+5F99.0	fTotalStrokes	 11
U+5F99.0	fCangjie	HOYLO
U+5F99.0	fFourCorner	21281
U+5F99.0	fHakka	  sai3
U+5F99.0	fCantonese	saai2
U+5F99.0	fMandarin	xi3; si1
U+5F99.0	fEnglish	[1] move one's abode; shift; migrate [2] be exiled
U+5F9A.0	fUTF8	徚
U+5F9A.0	fCNS11643	3-6477
U+5F9A.0	fGB	E-2960
U+5F9A.0	fR/S	 60.9
U+5F9A.0	fTotalStrokes	 12
U+5F9A.0	fCangjie	HODWF
U+5F9A.0	fFourCorner	2429
U+5F9A.0	fCantonese	gaan2
U+5F9B.0	fUTF8	徛
U+5F9B.0	fCNS11643	2-3532
U+5F9B.0	fBig5	D54E
U+5F9B.0	fGB	3-3A22
U+5F9B.0	fR/S	 60.8
U+5F9B.0	fTotalStrokes	 11
U+5F9B.0	fCangjie	HOKMR
U+5F9B.0	fFourCorner	24221
U+5F9B.0	fHakka	  ki1
U+5F9B.0	fCantonese	gei3 (also); kei5 (also); gei6
U+5F9B.0	fMandarin	qi4; qi1; ji4
U+5F9B.0	fEnglish	[1] stepping stones [2] bridge
U+5F9C.0	fUTF8	徜
U+5F9C.0	fCNS11643	1-5A3B
U+5F9C.0	fBig5	B175
U+5F9C.0	fGB	0-6164
U+5F9C.0	fR/S	 60.8
U+5F9C.0	fTotalStrokes	 11
U+5F9C.0	fCangjie	HOFBR
U+5F9C.0	fFourCorner	29227
U+5F9C.0	fHakka	 shong2; song2
U+5F9C.0	fCantonese	soeng4
U+5F9C.0	fMandarin	chang2
U+5F9C.0	fEnglish	going to and fro; lingering; loitering
U+5F9D.0	fUTF8	徝
U+5F9D.0	fCNS11643	4-3153
U+5F9D.0	fGB	5-3C3C
U+5F9D.0	fR/S	  60.8
U+5F9D.0	fTotalStrokes	 11
U+5F9D.0	fCangjie	HOJBM
U+5F9D.0	fFourCorner	24210
U+5F9D.0	fCantonese	zi3
U+5F9D.0	fMandarin	zhi4
U+5F9E.0	fUTF8	從
U+5F9E.0	fCNS11643	1-5A37
U+5F9E.0	fBig5	B171
U+5F9E.0	fGB	1-3453
U+5F9E.0	fR/S	 60.8
U+5F9E.0	fTotalStrokes	 11
U+5F9E.0	fCangjie	HOOOO
U+5F9E.0	fFourCorner	28281
U+5F9E.0	fHakka	  ciung2;  ziung3; cung2; ciung3; siung6; ziung6; ziung5; siung3; siung5
U+5F9E.0	fCantonese	sung1; zung1; zung3; cung4; cung1; zung6
U+5F9E.0	fMandarin	cong2; chuang2
U+5F9E.0	fEnglish	[1] from; by; whence; through [2] undertake; manage; dedicate oneself to; attend to; engage in
U+5F9E.1	fUTF8	從
U+5F9E.1	fCNS11643	1-5A37
U+5F9E.1	fBig5	B171
U+5F9E.1	fGB	1-3453
U+5F9E.1	fR/S	 60.8
U+5F9E.1	fTotalStrokes	 11
U+5F9E.1	fCangjie	HOOOO
U+5F9E.1	fFourCorner	28281
U+5F9E.1	fMandarin	zong4
U+5F9E.1	fEnglish	[1] entourage; attendant; servant [2] secondary; the relation other than one's direct blood relatives [3] accessory [4] vice; deputy
U+5F9E.2	fUTF8	從
U+5F9E.2	fCNS11643	1-5A37
U+5F9E.2	fBig5	B171
U+5F9E.2	fGB	1-3453
U+5F9E.2	fR/S	 60.8
U+5F9E.2	fTotalStrokes	 11
U+5F9E.2	fCangjie	HOOOO
U+5F9E.2	fFourCorner	28281
U+5F9E.2	fMandarin	cong1
U+5F9E.2	fEnglish	[1] lax; easy [2] plentiful; abundant [3] urge; persuade strongly
U+5F9E.3	fUTF8	從
U+5F9E.3	fCNS11643	1-5A37
U+5F9E.3	fBig5	B171
U+5F9E.3	fGB	1-3453
U+5F9E.3	fR/S	 60.8
U+5F9E.3	fTotalStrokes	 11
U+5F9E.3	fCangjie	HOOOO
U+5F9E.3	fFourCorner	28281
U+5F9E.3	fMandarin	zong1
U+5F9E.3	fEnglish	from north to south
U+5F9F.0	fUTF8	徟
U+5F9F.0	fCNS11643	2-3534
U+5F9F.0	fBig5	D550
U+5F9F.0	fGB	E-2961
U+5F9F.0	fR/S	 60.8
U+5F9F.0	fTotalStrokes	 11
U+5F9F.0	fCangjie	HOBGR
U+5F9F.0	fFourCorner	27220
U+5F9F.0	fHakka	 zhiu1; zu1
U+5F9F.0	fCantonese	zau1
U+5F9F.0	fMandarin	zhou1
U+5F9F.0	fEnglish	[1] hurried [2] excited
U+5FA0.0	fUTF8	徠
U+5FA0.0	fCNS11643	1-5A3A
U+5FA0.0	fBig5	B174
U+5FA0.0	fGB	1-6162
U+5FA0.0	fR/S	 60.8
U+5FA0.0	fTotalStrokes	 11
U+5FA0.0	fCangjie	HODOO
U+5FA0.0	fFourCorner	24298
U+5FA0.0	fHakka	  loi3;  loi2; loi5
U+5FA0.0	fCantonese	loi4; loi6
U+5FA0.0	fMandarin	lai2; lai4
U+5FA0.0	fEnglish	 induce; encourage (customers) to come; solicit (customers or business); encourage (customers) to come and buy
U+5FA1.0	fUTF8	御
U+5FA1.0	fCNS11643	1-5A39
U+5FA1.0	fBig5	B173
U+5FA1.0	fGB	0-5379
U+5FA1.0	fR/S	 60.8
U+5FA1.0	fTotalStrokes	 11
U+5FA1.0	fCangjie	HOOML
U+5FA1.0	fFourCorner	27220
U+5FA1.0	fHakka	  ngi5; ngi6; gni6; ngi3; ham2
U+5FA1.0	fCantonese	jyu6; ngaa6; aa6 (also)
U+5FA1.0	fMandarin	yu4; ya4; yu3
U+5FA1.0	fEnglish	[1] drive a chariot or carriage [2] driver; attendant [3] resist; keep out [4] control; manage; superintend; tame (a shrew); harness [5] imperial [6] wait on; set before, ass food; offer to; present to [7] a Chinese family name
U+5FA2.0	fUTF8	徢
U+5FA2.0	fCNS11643	4-3152
U+5FA2.0	fGB	E-2962
U+5FA2.0	fR/S	 60.8
U+5FA2.0	fTotalStrokes	 11
U+5FA2.0	fCangjie	HOJLO
U+5FA2.0	fFourCorner	25280
U+5FA2.0	fCantonese	sip3
U+5FA2.0	fMandarin	xie4
U+5FA3.0	fUTF8	徣
U+5FA3.0	fCNS11643	4-3157
U+5FA3.0	fGB	E-2963
U+5FA3.0	fR/S	 60.8
U+5FA3.0	fTotalStrokes	 11
U+5FA3.0	fCangjie	HOTA
U+5FA3.0	fFourCorner	24260
U+5FA3.0	fHakka	  zia5
U+5FA3.0	fCantonese	ze3; ze2 (also)
U+5FA3.0	fMandarin	jie4
U+5FA3.0	fEnglish	same as &#x501F;
U+5FA4.0	fUTF8	徤
U+5FA4.0	fCNS11643	4-374A
U+5FA4.0	fGB	E-2964
U+5FA4.0	fR/S	 60.8; 60.9
U+5FA4.0	fTotalStrokes	 12; 11
U+5FA4.0	fCangjie	HONKQ
U+5FA4.0	fFourCorner	25240
U+5FA4.0	fCantonese	gin6
U+5FA4.0	fMandarin	jian4
U+5FA5.0	fUTF8	徥
U+5FA5.0	fCNS11643	2-3B5A
U+5FA5.0	fBig5	D8F6
U+5FA5.0	fGB	3-3A24
U+5FA5.0	fR/S	 60.9
U+5FA5.0	fTotalStrokes	 12
U+5FA5.0	fCangjie	HOAMO
U+5FA5.0	fFourCorner	26281
U+5FA5.0	fCantonese	si6; zi6
U+5FA5.0	fMandarin	chi2; shi4; chi1
U+5FA6.0	fUTF8	徦
U+5FA6.0	fCNS11643	2-3B59
U+5FA6.0	fBig5	D8F5
U+5FA6.0	fGB	5-3C43
U+5FA6.0	fR/S	 60.9
U+5FA6.0	fTotalStrokes	 12
U+5FA6.0	fCangjie	HORSE
U+5FA6.0	fFourCorner	27247
U+5FA6.0	fCantonese	gaa2; haa4
U+5FA6.0	fMandarin	jia3; ge2; xia2
U+5FA7.0	fUTF8	徧
U+5FA7.0	fCNS11643	3-3B4B
U+5FA7.0	fGB	E-2965
U+5FA7.0	fR/S	  60.9
U+5FA7.0	fTotalStrokes	 12
U+5FA7.0	fCangjie	HOHSB
U+5FA7.0	fFourCorner	23220
U+5FA7.0	fHakka	  bien5;  pien1; pen1
U+5FA7.0	fCantonese	pin3; opin3
U+5FA7.0	fMandarin	bian4
U+5FA7.0	fEnglish	[1] everywhere; all over; all around [2] the whole (world, etc.)
U+5FA8.0	fUTF8	徨
U+5FA8.0	fCNS11643	1-5F28
U+5FA8.0	fBig5	B461
U+5FA8.0	fGB	0-6165
U+5FA8.0	fR/S	 60.9
U+5FA8.0	fTotalStrokes	 12
U+5FA8.0	fCangjie	HOHAG
U+5FA8.0	fFourCorner	26214
U+5FA8.0	fHakka	 fong2; fong3
U+5FA8.0	fCantonese	wong4
U+5FA8.0	fMandarin	huang2; wang2
U+5FA8.0	fEnglish	[1] doubtful; irresolute [2] agitated; alarmed
U+5FA9.0	fUTF8	復
U+5FA9.0	fCNS11643	1-5F26
U+5FA9.0	fBig5	B45F
U+5FA9.0	fGB	1-3834
U+5FA9.0	fR/S	 60.9
U+5FA9.0	fTotalStrokes	 12
U+5FA9.0	fCangjie	HOOAE
U+5FA9.0	fFourCorner	28247
U+5FA9.0	fHakka	  fuk8; fuk7
U+5FA9.0	fCantonese	fuk6; fau6 (also); fuk1
U+5FA9.0	fMandarin	fu4; fou4
U+5FA9.0	fEnglish	[1] return; come back [2] repeat; repeatedly; again [3] answer; reply [4] recover; recovery [5] return to a normal or original state [6] a surname
U+5FAA.0	fUTF8	循
U+5FAA.0	fCNS11643	1-5F27
U+5FAA.0	fBig5	B460
U+5FAA.0	fGB	0-512D
U+5FAA.0	fR/S	 60.9
U+5FAA.0	fTotalStrokes	 12
U+5FAA.0	fCangjie	HOHJU
U+5FAA.0	fFourCorner	22264
U+5FAA.0	fHakka	 sun2;  cun2; sun3
U+5FAA.0	fCantonese	ceon4
U+5FAA.0	fMandarin	xun2
U+5FAA.0	fEnglish	[1] comply with; follow [2] postpone; procrastinate [3] in orderly fashion [4] touch [5] inspect
U+5FAB.0	fUTF8	徫
U+5FAB.0	fCNS11643	2-3B5B
U+5FAB.0	fBig5	D8F7
U+5FAB.0	fGB	5-3C27
U+5FAB.0	fR/S	 60.9
U+5FAB.0	fTotalStrokes	 12
U+5FAB.0	fCangjie	HODMQ
U+5FAB.0	fFourCorner	24256
U+5FAB.0	fCantonese	wai5
U+5FAB.0	fMandarin	wei3
U+5FAC.0	fUTF8	徬
U+5FAC.0	fCNS11643	1-6371
U+5FAC.0	fBig5	B74B
U+5FAC.0	fGB	5-3C46
U+5FAC.0	fR/S	 60.10
U+5FAC.0	fTotalStrokes	 13
U+5FAC.0	fCangjie	HOYBS
U+5FAC.0	fFourCorner	20227
U+5FAC.0	fHakka	  pong2;  pong5; pong3
U+5FAC.0	fCantonese	pong4; bong6
U+5FAC.0	fMandarin	pang2; bang4
U+5FAC.0	fEnglish	anxious; agitated and indecisive
U+5FAD.0	fUTF8	徭
U+5FAD.0	fCNS11643	2-426A
U+5FAD.0	fBig5	DD64
U+5FAD.0	fGB	0-6166
U+5FAD.0	fR/S	 60.10
U+5FAD.0	fTotalStrokes	 13
U+5FAD.0	fCangjie	HOBOU
U+5FAD.0	fFourCorner	27272
U+5FAD.0	fHakka	 jau2; rau2; jeu2
U+5FAD.0	fCantonese	jiu4
U+5FAD.0	fMandarin	yao2
U+5FAD.0	fEnglish	conscript labor; compulsory labor
U+5FAE.0	fUTF8	微
U+5FAE.0	fCNS11643	1-6372
U+5FAE.0	fBig5	B74C
U+5FAE.0	fGB	0-4E22
U+5FAE.0	fR/S	 60.10
U+5FAE.0	fTotalStrokes	 13
U+5FAE.0	fCangjie	HOUUK
U+5FAE.0	fFourCorner	28240
U+5FAE.0	fHakka	  mi2; mi3
U+5FAE.0	fCantonese	mei4; mei1 (also); mei2 (also)
U+5FAE.0	fMandarin	wei1; wei2; obs
U+5FAE.0	fEnglish	[1] small; minute; diminutive; trifling; little; slight [2] low; mean; humble [3] a polite expression for I, my, me [4] weak; sickly; feeble [5] subtle [6] obscure [7] hidden; concealed [8] spy [9] if not; but for [10] a Chinese family name
U+5FAF.0	fUTF8	徯
U+5FAF.0	fCNS11643	2-4269
U+5FAF.0	fBig5	DD63
U+5FAF.0	fGB	3-3A25
U+5FAF.0	fR/S	 60.10
U+5FAF.0	fTotalStrokes	 13
U+5FAF.0	fCangjie	HOBVK
U+5FAF.0	fFourCorner	22234
U+5FAF.0	fHakka	  hi1
U+5FAF.0	fCantonese	hai4; hai5
U+5FAF.0	fMandarin	xi1; xi4; xi2
U+5FAF.0	fEnglish	[1] wait for; await; expect [2] narrow path; shortcut
U+5FB0.0	fUTF8	徰
U+5FB0.0	fCNS11643	4-3D34
U+5FB0.0	fGB	E-2966
U+5FB0.0	fR/S	 60.10
U+5FB0.0	fTotalStrokes	 13
U+5FB0.0	fCangjie	HOMMM
U+5FB0.0	fFourCorner	21210
U+5FB0.0	fHakka	  zhin1; zin1; zihn1
U+5FB0.0	fCantonese	zing1; cing4
U+5FB0.0	fMandarin	zheng1
U+5FB0.0	fEnglish	ancient form of &#x5F81;
U+5FB1.0	fUTF8	徱
U+5FB1.0	fCNS11643	4-433F
U+5FB1.0	fGB	E-2967
U+5FB1.0	fR/S	 60.11
U+5FB1.0	fTotalStrokes	 14
U+5FB1.0	fCangjie	HOMWF
U+5FB1.0	fFourCorner	21290
U+5FB1.0	fHakka	 piau5
U+5FB1.0	fCantonese	piu3 (also); piu1; piu5
U+5FB1.0	fMandarin	piao4; biao1; piao1
U+5FB1.0	fEnglish	same as &#x50C4;
U+5FB2.0	fUTF8	徲
U+5FB2.0	fCNS11643	2-5040
U+5FB2.0	fBig5	E577
U+5FB2.0	fGB	3-3A29
U+5FB2.0	fR/S	 60.11; 60.12
U+5FB2.0	fTotalStrokes	 15
U+5FB2.0	fCangjie	HOSEQ
U+5FB2.0	fFourCorner	27253
U+5FB2.0	fCantonese	tai4; ci4
U+5FB2.0	fMandarin	chi2; ti2
U+5FB3.0	fUTF8	徳
U+5FB3.0	fCNS11643	3-4663
U+5FB3.0	fGB	E-2968
U+5FB3.0	fR/S	 60.11
U+5FB3.0	fTotalStrokes	 14
U+5FB3.0	fCangjie	HOJWP
U+5FB3.0	fFourCorner	24230
U+5FB3.0	fHakka	 det7
U+5FB3.0	fCantonese	dak1
U+5FB3.0	fMandarin	de2; duo2
U+5FB3.0	fEnglish	simplified form of &#x5FB7;
U+5FB4.0	fUTF8	徴
U+5FB4.0	fCNS11643	3-4664
U+5FB4.0	fGB	E-2969
U+5FB4.0	fR/S	 60.11
U+5FB4.0	fTotalStrokes	 14
U+5FB4.0	fCangjie	HOUGK
U+5FB4.0	fFourCorner	28240
U+5FB4.0	fHakka	 zhin1
U+5FB4.0	fCantonese	zing1; zi2
U+5FB4.0	fMandarin	zheng1
U+5FB4.0	fEnglish	[1] summon [2] levy; raise (taxes) [3] call to arms [4] ask; inquire [5] request; seek for [6] prove; evidence [7] signs [8] simplified form of &#x5FB5;
U+5FB4.1	fUTF8	徴
U+5FB4.1	fCNS11643	3-4664
U+5FB4.1	fGB	E-2969
U+5FB4.1	fR/S	 60.11
U+5FB4.1	fTotalStrokes	 14
U+5FB4.1	fCangjie	HOUGK
U+5FB4.1	fFourCorner	28240
U+5FB4.1	fMandarin	zhi3
U+5FB4.1	fEnglish	one of the five musical notes in Chinese scale
U+5FB5.0	fUTF8	徵
U+5FB5.0	fCNS11643	1-6C62
U+5FB5.0	fBig5	BC78
U+5FB5.0	fGB	0-6167
U+5FB5.0	fR/S	 60.12
U+5FB5.0	fTotalStrokes	 15
U+5FB5.0	fCangjie	HOUGK
U+5FB5.0	fFourCorner	28240
U+5FB5.0	fHakka	 zhin1; zin1; zihn1
U+5FB5.0	fCantonese	zi2; zing1; cing4
U+5FB5.0	fMandarin	zheng1; zhi3
U+5FB5.0	fEnglish	 summon; recruit musical note
U+5FB6.0	fUTF8	徶
U+5FB6.0	fCNS11643	2-4A28
U+5FB6.0	fBig5	E1C1
U+5FB6.0	fGB	3-3A27
U+5FB6.0	fR/S	 60.12
U+5FB6.0	fTotalStrokes	 15
U+5FB6.0	fCangjie	HOFBK
U+5FB6.0	fFourCorner	28240
U+5FB6.0	fHakka	 piet8
U+5FB6.0	fCantonese	pit3 (also); bit3
U+5FB6.0	fMandarin	bie2; bie4
U+5FB7.0	fUTF8	德
U+5FB7.0	fCNS11643	1-6C61
U+5FB7.0	fBig5	BC77
U+5FB7.0	fGB	0-3542
U+5FB7.0	fR/S	 60.12
U+5FB7.0	fTotalStrokes	 15
U+5FB7.0	fCangjie	HOJWP
U+5FB7.0	fFourCorner	24236
U+5FB7.0	fHakka	  det7; diet7
U+5FB7.0	fCantonese	dak1
U+5FB7.0	fMandarin	de2
U+5FB7.0	fEnglish	[1] morality; decency; virtues [2] favor; kindness [3] behavior; conduct [4] feel grateful [5] Germany; German
U+5FB8.0	fUTF8	徸
U+5FB8.0	fCNS11643	3-4B72
U+5FB8.0	fGB	5-3C4C
U+5FB8.0	fR/S	  60.12
U+5FB8.0	fTotalStrokes	 15
U+5FB8.0	fCangjie	HOYTG
U+5FB8.0	fFourCorner	20210
U+5FB8.0	fCantonese	cung1
U+5FB8.0	fMandarin	chong1
U+5FB9.0	fUTF8	徹
U+5FB9.0	fCNS11643	1-6865
U+5FB9.0	fBig5	B9FD
U+5FB9.0	fGB	1-3339
U+5FB9.0	fR/S	 60.12
U+5FB9.0	fTotalStrokes	 15
U+5FB9.0	fCangjie	HOYBK
U+5FB9.0	fFourCorner	28240
U+5FB9.0	fHakka	 cet8;  chat7; chat8; chet8; cat8; cat7
U+5FB9.0	fCantonese	cit3
U+5FB9.0	fMandarin	che4; zhe2
U+5FB9.0	fEnglish	[1] penetrating; discerning [2] remove [3] tax in tithe [4] manage; cultivate (farms) [5] destroy [6] deprive; take away
U+5FBA.0	fUTF8	徺
U+5FBA.0	fCNS11643	3-6478
U+5FBA.0	fGB	E-296A
U+5FBA.0	fR/S	 60.12
U+5FBA.0	fTotalStrokes	 15
U+5FBA.0	fCangjie	HOGGU
U+5FBA.0	fFourCorner	2421
U+5FBA.0	fCantonese	hiu1
U+5FBA.0	fMandarin	jiao3
U+5FBB.0	fUTF8	徻
U+5FBB.0	fCNS11643	2-5722
U+5FBB.0	fBig5	ECDE
U+5FBB.0	fGB	E-296B
U+5FBB.0	fR/S	 60.13
U+5FBB.0	fTotalStrokes	 16
U+5FBB.0	fCangjie	HOOMA
U+5FBB.0	fFourCorner	28266
U+5FBB.0	fHakka	 kwai5
U+5FBB.0	fCantonese	wai3; wui3; kwui2
U+5FBB.0	fMandarin	wei4
U+5FBB.0	fEnglish	a wide room
U+5FBC.0	fUTF8	徼
U+5FBC.0	fCNS11643	2-5721
U+5FBC.0	fBig5	E975
U+5FBC.0	fGB	0-6168
U+5FBC.0	fR/S	 60.13
U+5FBC.0	fTotalStrokes	 16
U+5FBC.0	fCangjie	HOHSK
U+5FBC.0	fFourCorner	28240
U+5FBC.0	fHakka	 giau5;  hiau1; hieu1; giau1
U+5FBC.0	fCantonese	giu3; giu1; giu2 (also); hiu1 (also); jiu1
U+5FBC.0	fMandarin	jiao4; jiao1; yao2; jiao2
U+5FBC.0	fEnglish	[1] frontiers; boundaries; border [2] inspect; patrol; take an inspection trip
U+5FBC.1	fUTF8	徼
U+5FBC.1	fCNS11643	2-5721
U+5FBC.1	fBig5	E975
U+5FBC.1	fGB	0-6168
U+5FBC.1	fR/S	 60.13
U+5FBC.1	fTotalStrokes	 16
U+5FBC.1	fCangjie	HOHSK
U+5FBC.1	fFourCorner	28240
U+5FBC.1	fMandarin	jiao3
U+5FBC.1	fEnglish	 be lucky; fortunate
U+5FBC.2	fUTF8	徼
U+5FBC.2	fCNS11643	2-5721
U+5FBC.2	fBig5	E975
U+5FBC.2	fGB	0-6168
U+5FBC.2	fR/S	 60.13
U+5FBC.2	fTotalStrokes	 16
U+5FBC.2	fCangjie	HOHSK
U+5FBC.2	fFourCorner	28240
U+5FBC.2	fMandarin	yao1
U+5FBC.2	fEnglish	[1] pray for [2] shade
U+5FBD.0	fUTF8	徽
U+5FBD.0	fCNS11643	1-735B
U+5FBD.0	fBig5	C0B2
U+5FBD.0	fGB	0-3B55
U+5FBD.0	fR/S	 60.14
U+5FBD.0	fTotalStrokes	 17
U+5FBD.0	fCangjie	HOUFK
U+5FBD.0	fFourCorner	28240
U+5FBD.0	fHakka	  fui1; fi1
U+5FBD.0	fCantonese	fai1
U+5FBD.0	fMandarin	hui1
U+5FBD.0	fEnglish	[1] badge; insignia; emblem [2] good and beautiful; honorable [3] stops on alute [4] streamer; flag; pennant; flag-sign [5] Anhwei; Huichow
U+5FBE.0	fUTF8	徾
U+5FBE.0	fCNS11643	2-5C69
U+5FBE.0	fBig5	ECDD
U+5FBE.0	fGB	E-296C
U+5FBE.0	fR/S	 60.14
U+5FBE.0	fTotalStrokes	 17
U+5FBE.0	fCangjie	HOURK
U+5FBE.0	fFourCorner	28240
U+5FBE.0	fCantonese	mei4
U+5FBE.0	fMandarin	mei2
U+5FBF.0	fUTF8	徿
U+5FBF.0	fCNS11643	2-654B
U+5FBF.0	fBig5	F240
U+5FBF.0	fGB	5-3C29
U+5FBF.0	fR/S	 60.16
U+5FBF.0	fTotalStrokes	 19
U+5FBF.0	fCangjie	HOYBP
U+5FBF.0	fFourCorner	21211
U+5FBF.0	fHakka	 lung3; lung1; lung6; lung5
U+5FBF.0	fCantonese	lung6; lung5
U+5FBF.0	fMandarin	long4; long3
U+5FBF.0	fEnglish	 walk awry
U+5FC0.0	fUTF8	忀
U+5FC0.0	fCNS11643	2-6874
U+5FC0.0	fBig5	F448
U+5FC0.0	fGB	5-3C56
U+5FC0.0	fR/S	 60.17
U+5FC0.0	fTotalStrokes	 20
U+5FC0.0	fCangjie	HOYRV
U+5FC0.0	fFourCorner	20232
U+5FC0.0	fHakka	  siong1
U+5FC0.0	fCantonese	soeng1; nong5
U+5FC0.0	fMandarin	xiang1
U+5FC0.0	fEnglish	 stroll ramble
U+5FC1.0	fUTF8	忁
U+5FC1.0	fCNS11643	2-6875
U+5FC1.0	fBig5	F449
U+5FC1.0	fGB	E-296D
U+5FC1.0	fR/S	 60.17; 60.16
U+5FC1.0	fTotalStrokes	 20
U+5FC1.0	fCangjie	HOAUE
U+5FC1.0	fFourCorner	26232
U+5FC1.0	fCantonese	bou3
U+5FC1.0	fMandarin	bao4
U+5FC2.0	fUTF8	忂
U+5FC2.0	fCNS11643	3-5E2F
U+5FC2.0	fGB	5-3C57
U+5FC2.0	fR/S	  60.18
U+5FC2.0	fTotalStrokes	 21
U+5FC2.0	fCangjie	HOBUG
U+5FC2.0	fFourCorner	26210
U+5FC2.0	fCantonese	keoi4
U+5FC2.0	fMandarin	qu2
U+5FC3.0	fUTF8	心
U+5FC3.0	fCNS11643	1-4540
U+5FC3.0	fBig5	A4DF
U+5FC3.0	fGB	0-5044
U+5FC3.0	fR/S	 61.0
U+5FC3.0	fTotalStrokes	 4
U+5FC3.0	fCangjie	P
U+5FC3.0	fFourCorner	33000
U+5FC3.0	fHakka	  sim1
U+5FC3.0	fCantonese	sam1
U+5FC3.0	fMandarin	xin1
U+5FC3.0	fEnglish	[1] heart [2] mind [3] conscience; moral nature [4] intention; idea; ambition; design [5] core; middle; center; inside [6] one of the 28 constellations [7] Kangxi radical 61
U+5FC3.1	fUTF8	心
U+5FC3.1	fCNS11643	1-4540
U+5FC3.1	fBig5	A4DF
U+5FC3.1	fGB	0-5044
U+5FC3.1	fR/S	 61.0
U+5FC3.1	fTotalStrokes	 4
U+5FC3.1	fCangjie	P
U+5FC3.1	fFourCorner	33000
U+5FC3.1	fMandarin	xin5
U+5FC4.0	fUTF8	忄
U+5FC4.0	fCNS11643	4-2136
U+5FC4.0	fGB	0-6260
U+5FC4.0	fR/S	 61.0
U+5FC4.0	fTotalStrokes	 3
U+5FC4.0	fCangjie	HY
U+5FC4.0	fFourCorner	90000
U+5FC4.0	fHakka	 sim1
U+5FC4.0	fCantonese	syu6; sam1
U+5FC4.0	fMandarin	xin1
U+5FC4.0	fEnglish	same as &#x5FC3;
U+5FC5.0	fUTF8	必
U+5FC5.0	fCNS11643	1-4652
U+5FC5.0	fBig5	A5B2
U+5FC5.0	fGB	0-3158
U+5FC5.0	fR/S	 61.1
U+5FC5.0	fTotalStrokes	 5; 4
U+5FC5.0	fCangjie	PH
U+5FC5.0	fFourCorner	33000
U+5FC5.0	fHakka	  bit7
U+5FC5.0	fCantonese	bit1
U+5FC5.0	fMandarin	bi4
U+5FC5.0	fEnglish	[1] surely; most certainly; must; necessarily [2] an emphatic particle
U+5FC6.0	fUTF8	忆
U+5FC6.0	fGB	0-5264
U+5FC6.0	fR/S	 61.1
U+5FC6.0	fTotalStrokes	 4
U+5FC6.0	fCangjie	PN
U+5FC6.0	fHakka	 jit7
U+5FC6.0	fCantonese	jik1
U+5FC6.0	fMandarin	yi4
U+5FC6.0	fEnglish	simplified form of &#x61B6;
U+5FC7.0	fUTF8	忇
U+5FC7.0	fCNS11643	3-224D
U+5FC7.0	fGB	3-512E
U+5FC7.0	fR/S	  61.2
U+5FC7.0	fTotalStrokes	 5
U+5FC7.0	fCangjie	PKS
U+5FC7.0	fFourCorner	94020
U+5FC7.0	fCantonese	lak6
U+5FC7.0	fMandarin	le4
U+5FC8.0	fUTF8	忈
U+5FC8.0	fCNS11643	4-2271
U+5FC8.0	fGB	5-5268
U+5FC8.0	fR/S	  61.2
U+5FC8.0	fTotalStrokes	 6
U+5FC8.0	fCangjie	MMP
U+5FC8.0	fFourCorner	10330
U+5FC8.0	fCantonese	jan4
U+5FC8.0	fMandarin	ren2
U+5FC9.0	fUTF8	忉
U+5FC9.0	fCNS11643	2-215C
U+5FC9.0	fBig5	C97B
U+5FC9.0	fGB	0-6261
U+5FC9.0	fR/S	 61.2
U+5FC9.0	fTotalStrokes	 5
U+5FC9.0	fCangjie	PSH
U+5FC9.0	fFourCorner	97020
U+5FC9.0	fHakka	  dau1; do1
U+5FC9.0	fCantonese	dou1; tou1 (also)
U+5FC9.0	fMandarin	dao1
U+5FC9.0	fEnglish	grieved; distressed
U+5FCA.0	fUTF8	忊
U+5FCA.0	fCNS11643	3-224C
U+5FCA.0	fGB	3-512D
U+5FCA.0	fR/S	  61.2
U+5FCA.0	fTotalStrokes	 5
U+5FCA.0	fCangjie	PMN
U+5FCA.0	fFourCorner	91020
U+5FCA.0	fCantonese	ding6
U+5FCA.0	fMandarin	ding4
U+5FCB.0	fUTF8	忋
U+5FCB.0	fCNS11643	5-2266
U+5FCB.0	fGB	5-5346
U+5FCB.0	fR/S	  61.3
U+5FCB.0	fTotalStrokes	 6
U+5FCB.0	fCangjie	PSU
U+5FCB.0	fFourCorner	97010
U+5FCB.0	fCantonese	goi2
U+5FCB.0	fMandarin	gai3
U+5FCB.0	fEnglish	 rely on look up to
U+5FCC.0	fUTF8	忌
U+5FCC.0	fCNS11643	1-4A34
U+5FCC.0	fBig5	A7D2
U+5FCC.0	fGB	0-3C49
U+5FCC.0	fR/S	 61.3
U+5FCC.0	fTotalStrokes	 7
U+5FCC.0	fCangjie	SUP
U+5FCC.0	fFourCorner	17331
U+5FCC.0	fHakka	  ki3; ki6; ki5; gi5
U+5FCC.0	fCantonese	gei6; gei2 (also)
U+5FCC.0	fMandarin	ji4
U+5FCC.0	fEnglish	[1] jealous; envious; envy [2] fear (usually superstitious) [3] shun [4] prohibit; proscribe (usually for superstitious purpose); taboo [5] death anniversaries of one's parents or grandparents
U+5FCD.0	fUTF8	忍
U+5FCD.0	fCNS11643	1-4A36
U+5FCD.0	fBig5	A7D4
U+5FCD.0	fGB	0-484C
U+5FCD.0	fR/S	 61.3
U+5FCD.0	fTotalStrokes	 7
U+5FCD.0	fCangjie	SIP
U+5FCD.0	fFourCorner	17332
U+5FCD.0	fHakka	  ngiun1; ngiun3; gniun1; niun1
U+5FCD.0	fCantonese	jan2; jan5
U+5FCD.0	fMandarin	ren3; ren4
U+5FCD.0	fEnglish	[1] endure; bear; tolerate; suffer; put up with; stand [2] merciless; truculence [3] forbear; repress
U+5FCE.0	fUTF8	忎
U+5FCE.0	fCNS11643	3-2578
U+5FCE.0	fGB	E-296E
U+5FCE.0	fR/S	  61.3
U+5FCE.0	fTotalStrokes	 7
U+5FCE.0	fCangjie	HJP
U+5FCE.0	fFourCorner	20334
U+5FCE.0	fHakka	 jin2; rin2; jin3; ngin2
U+5FCE.0	fCantonese	jan4
U+5FCE.0	fMandarin	ren2
U+5FCE.0	fEnglish	ancient form of &#x4EC1;
U+5FCF.0	fUTF8	忏
U+5FCF.0	fCNS11643	2-2242
U+5FCF.0	fBig5	C9E2
U+5FCF.0	fGB	0-6263
U+5FCF.0	fR/S	 61.3
U+5FCF.0	fTotalStrokes	 6
U+5FCF.0	fCangjie	PHJ
U+5FCF.0	fFourCorner	92040
U+5FCF.0	fHakka	 cam5
U+5FCF.0	fCantonese	caam3; saam3
U+5FCF.0	fMandarin	chan4
U+5FCF.0	fEnglish	simplified form of &#x61FA;
U+5FD0.0	fUTF8	忐
U+5FD0.0	fCNS11643	2-2378
U+5FD0.0	fBig5	CAD8
U+5FD0.0	fGB	0-6C7E
U+5FD0.0	fR/S	 61.3
U+5FD0.0	fTotalStrokes	 7
U+5FD0.0	fCangjie	YMP
U+5FD0.0	fFourCorner	21331
U+5FD0.0	fHakka	 tam3; tam5; tan3
U+5FD0.0	fCantonese	taan2
U+5FD0.0	fMandarin	tan3
U+5FD0.0	fEnglish	[1] timid; apprehensive [2] indecisive; vacillating
U+5FD1.0	fUTF8	忑
U+5FD1.0	fCNS11643	2-2377
U+5FD1.0	fBig5	CAD7
U+5FD1.0	fGB	0-6C7D
U+5FD1.0	fR/S	 61.3
U+5FD1.0	fTotalStrokes	 7
U+5FD1.0	fCangjie	MYP
U+5FD1.0	fFourCorner	10333
U+5FD1.0	fHakka	 tit7
U+5FD1.0	fCantonese	tik1
U+5FD1.0	fMandarin	te4
U+5FD1.0	fEnglish	[1] apprehensive; nervous [2] indecisive
U+5FD2.0	fUTF8	忒
U+5FD2.0	fCNS11643	2-2376
U+5FD2.0	fBig5	CAD6
U+5FD2.0	fGB	0-5F2F
U+5FD2.0	fR/S	 61.3
U+5FD2.0	fTotalStrokes	 7
U+5FD2.0	fCangjie	IPP
U+5FD2.0	fFourCorner	43300
U+5FD2.0	fHakka	  tet7; tit7;  tiet7; tit8; tiet8
U+5FD2.0	fCantonese	tik1
U+5FD2.0	fMandarin	te4; qu4; tei1; tui1
U+5FD2.0	fEnglish	[1] excessive; too; very-usually of objectionable things [2] err; mistake [3] change [4] too
U+5FD2.1	fUTF8	忒
U+5FD2.1	fCNS11643	2-2376
U+5FD2.1	fBig5	CAD6
U+5FD2.1	fGB	0-5F2F
U+5FD2.1	fR/S	 61.3
U+5FD2.1	fTotalStrokes	 7
U+5FD2.1	fCangjie	IPP
U+5FD2.1	fFourCorner	43300
U+5FD2.1	fMandarin	te1
U+5FD2.1	fEnglish	a word descriptive of sound
U+5FD3.0	fUTF8	忓
U+5FD3.0	fCNS11643	3-2366
U+5FD3.0	fGB	3-512F
U+5FD3.0	fR/S	  61.3
U+5FD3.0	fTotalStrokes	 6
U+5FD3.0	fCangjie	PMJ
U+5FD3.0	fFourCorner	91040
U+5FD3.0	fHakka	 gon1
U+5FD3.0	fCantonese	gon1
U+5FD3.0	fMandarin	gan1
U+5FD3.0	fEnglish	concerned about
U+5FD4.0	fUTF8	忔
U+5FD4.0	fCNS11643	2-2241
U+5FD4.0	fBig5	C9E1
U+5FD4.0	fGB	3-5132
U+5FD4.0	fR/S	 61.3
U+5FD4.0	fTotalStrokes	 6
U+5FD4.0	fCangjie	PON
U+5FD4.0	fFourCorner	98017
U+5FD4.0	fHakka	 ngiet7; ze2; je2; hit7; nget7
U+5FD4.0	fCantonese	ngat6; hat1 (also); at6 (also); jat6
U+5FD4.0	fMandarin	qi4; yi4
U+5FD4.0	fEnglish	[1] discontented [2] dislike
U+5FD5.0	fUTF8	忕
U+5FD5.0	fCNS11643	2-2240
U+5FD5.0	fBig5	C9E0
U+5FD5.0	fGB	3-5130
U+5FD5.0	fR/S	 61.3
U+5FD5.0	fTotalStrokes	 6
U+5FD5.0	fCangjie	PK
U+5FD5.0	fFourCorner	94030
U+5FD5.0	fCantonese	taai3; sai6; si6; tai3
U+5FD5.0	fMandarin	shi4; dai4; she2; tai4; tai3
U+5FD6.0	fUTF8	忖
U+5FD6.0	fCNS11643	1-4825
U+5FD6.0	fBig5	A6A4
U+5FD6.0	fGB	0-6262
U+5FD6.0	fR/S	 61.3
U+5FD6.0	fTotalStrokes	 6
U+5FD6.0	fCangjie	PDI
U+5FD6.0	fFourCorner	94000
U+5FD6.0	fHakka	  cun3; cun5
U+5FD6.0	fCantonese	cyun2
U+5FD6.0	fMandarin	cun3
U+5FD6.0	fEnglish	[1] surmise; consider; presume; suppose [2] think
U+5FD7.0	fUTF8	志
U+5FD7.0	fCNS11643	1-4A35
U+5FD7.0	fBig5	A7D3
U+5FD7.0	fGB	0-563E
U+5FD7.0	fR/S	 61.3
U+5FD7.0	fTotalStrokes	 7
U+5FD7.0	fCangjie	GP
U+5FD7.0	fFourCorner	40331
U+5FD7.0	fHakka	 zi5;  zhi5; zih5; tje5
U+5FD7.0	fCantonese	zi3
U+5FD7.0	fMandarin	zhi4
U+5FD7.0	fEnglish	[1] purpose; will; determination [2] make up one's mind to pursue some object; be bent on doing something [3] ideal; desire; ambition; interest; wish [4] annals; records
U+5FD8.0	fUTF8	忘
U+5FD8.0	fCNS11643	1-4A33
U+5FD8.0	fBig5	A7D1
U+5FD8.0	fGB	0-4D7C
U+5FD8.0	fR/S	 61.3
U+5FD8.0	fTotalStrokes	 7; 6
U+5FD8.0	fCangjie	YVP
U+5FD8.0	fFourCorner	00331
U+5FD8.0	fHakka	  mong2; mong3; mong5; mong6; wong5
U+5FD8.0	fCantonese	mong4; mong6
U+5FD8.0	fMandarin	wang4
U+5FD8.0	fEnglish	[1] forget [2] neglect; overlook [3] miss (a line, etc.); omit
U+5FD8.1	fUTF8	忘
U+5FD8.1	fCNS11643	1-4A33
U+5FD8.1	fBig5	A7D1
U+5FD8.1	fGB	0-4D7C
U+5FD8.1	fR/S	 61.3
U+5FD8.1	fTotalStrokes	 7; 6
U+5FD8.1	fCangjie	YVP
U+5FD8.1	fFourCorner	00331
U+5FD8.1	fMandarin	wang2
U+5FD8.1	fEnglish	reading sound of &#x5FD8;
U+5FD9.0	fUTF8	忙
U+5FD9.0	fCNS11643	1-4824
U+5FD9.0	fBig5	A6A3
U+5FD9.0	fGB	0-4326
U+5FD9.0	fR/S	 61.3
U+5FD9.0	fTotalStrokes	 6; 7
U+5FD9.0	fCangjie	PYV
U+5FD9.0	fFourCorner	90010
U+5FD9.0	fHakka	  mong2; mong3
U+5FD9.0	fCantonese	mong4
U+5FD9.0	fMandarin	mang2
U+5FD9.0	fEnglish	[1] busy; short of time; pressed for time [2] hurried; in haste; make haste; hurry
U+5FDA.0	fUTF8	忚
U+5FDA.0	fCNS11643	3-2367
U+5FDA.0	fGB	3-5135
U+5FDA.0	fR/S	 61.3
U+5FDA.0	fTotalStrokes	 6
U+5FDA.0	fCangjie	PPD
U+5FDA.0	fFourCorner	94010
U+5FDA.0	fHakka	 hi1
U+5FDA.0	fCantonese	hei1
U+5FDA.0	fMandarin	xi1
U+5FDA.0	fEnglish	contemptuous, insulting
U+5FDB.0	fUTF8	忛
U+5FDB.0	fCNS11643	3-2369
U+5FDB.0	fGB	E-296F
U+5FDB.0	fR/S	 61.3
U+5FDB.0	fTotalStrokes	 6
U+5FDB.0	fCangjie	PHNI
U+5FDB.0	fFourCorner	97010
U+5FDB.0	fHakka	 fam2
U+5FDB.0	fCantonese	faan4
U+5FDB.0	fMandarin	fan2; fan1; fan4
U+5FDB.0	fEnglish	corrupted form of &#x5E06;
U+5FDC.0	fUTF8	応
U+5FDC.0	fCNS11643	3-2577
U+5FDC.0	fGB	E-2970
U+5FDC.0	fR/S	 61.3
U+5FDC.0	fTotalStrokes	 7
U+5FDC.0	fCangjie	IP
U+5FDC.0	fFourCorner	00230
U+5FDC.0	fHakka	 jin1; jin5
U+5FDC.0	fCantonese	jing3; jing1
U+5FDC.0	fMandarin	ying1; ying4
U+5FDC.0	fEnglish	simplified form of &#x61C9;
U+5FDD.0	fUTF8	忝
U+5FDD.0	fCNS11643	1-4D3F
U+5FDD.0	fBig5	A9BD
U+5FDD.0	fGB	0-6343
U+5FDD.0	fR/S	 61.4
U+5FDD.0	fTotalStrokes	 8; 7
U+5FDD.0	fCangjie	HKP
U+5FDD.0	fFourCorner	20333
U+5FDD.0	fHakka	 tiam3
U+5FDD.0	fCantonese	tim2; tim5
U+5FDD.0	fMandarin	tian3
U+5FDD.0	fEnglish	[1] ashamed; disgrace [2] a depreciatory expression referring to oneself [3] be unworthy of the honor
U+5FDE.0	fUTF8	忞
U+5FDE.0	fCNS11643	2-2659
U+5FDE.0	fBig5	CC78
U+5FDE.0	fGB	3-4F3E
U+5FDE.0	fR/S	 61.4
U+5FDE.0	fTotalStrokes	 8
U+5FDE.0	fCangjie	YKP
U+5FDE.0	fFourCorner	00334
U+5FDE.0	fHakka	  min2; men2
U+5FDE.0	fCantonese	man4; man2 (also)
U+5FDE.0	fMandarin	min2; min3
U+5FDE.0	fEnglish	[1] make an effort [2] self-siscipline [3] anxious
U+5FDF.0	fUTF8	忟
U+5FDF.0	fCNS11643	3-257B
U+5FDF.0	fGB	E-2971
U+5FDF.0	fR/S	 61.4
U+5FDF.0	fTotalStrokes	 7
U+5FDF.0	fCangjie	PYK
U+5FDF.0	fFourCorner	90040
U+5FDF.0	fHakka	 min2
U+5FDF.0	fCantonese	man4; mang2
U+5FDF.0	fMandarin	min2; min3
U+5FDF.0	fEnglish	same as &#x5FDE;
U+5FE0.0	fUTF8	忠
U+5FE0.0	fCNS11643	1-4D40
U+5FE0.0	fBig5	A9BE
U+5FE0.0	fGB	0-5652
U+5FE0.0	fR/S	 61.4
U+5FE0.0	fTotalStrokes	 8; 7
U+5FE0.0	fCangjie	LP
U+5FE0.0	fFourCorner	50336
U+5FE0.0	fHakka	 zung1; zhung1
U+5FE0.0	fCantonese	zung1
U+5FE0.0	fMandarin	zhong1
U+5FE0.0	fEnglish	[1] faithful; loyalty; patriotic; constant; sincerity [2] devoted; honest (advice, etc.)
U+5FE1.0	fUTF8	忡
U+5FE1.0	fCNS11643	2-237D
U+5FE1.0	fBig5	CADD
U+5FE1.0	fGB	0-6267
U+5FE1.0	fR/S	 61.4
U+5FE1.0	fTotalStrokes	 7
U+5FE1.0	fCangjie	PL
U+5FE1.0	fFourCorner	95006
U+5FE1.0	fHakka	 cung1;  chung1
U+5FE1.0	fCantonese	cung1
U+5FE1.0	fMandarin	chong1
U+5FE1.0	fEnglish	worried; anxious; uneasy; sad
U+5FE2.0	fUTF8	忢
U+5FE2.0	fCNS11643	4-2650
U+5FE2.0	fGB	E-2972
U+5FE2.0	fR/S	 61.4
U+5FE2.0	fTotalStrokes	 8
U+5FE2.0	fCangjie	MMP
U+5FE2.0	fFourCorner	10330
U+5FE2.0	fHakka	 ngu5; ngu3; ng3
U+5FE2.0	fCantonese	ng6; ng4 (also)
U+5FE2.0	fMandarin	wu4
U+5FE2.0	fEnglish	ancient form of &#x609F;
U+5FE3.0	fUTF8	忣
U+5FE3.0	fCNS11643	2-2421
U+5FE3.0	fBig5	CADF
U+5FE3.0	fGB	5-5345
U+5FE3.0	fR/S	 61.4
U+5FE3.0	fTotalStrokes	 7
U+5FE3.0	fCangjie	PNHE
U+5FE3.0	fFourCorner	97047
U+5FE3.0	fHakka	 gip7
U+5FE3.0	fCantonese	gap1
U+5FE3.0	fMandarin	ji2
U+5FE3.0	fEnglish	same as &#x3902;
U+5FE4.0	fUTF8	忤
U+5FE4.0	fCNS11643	2-237E
U+5FE4.0	fBig5	CADE
U+5FE4.0	fGB	0-6268
U+5FE4.0	fR/S	 61.4
U+5FE4.0	fTotalStrokes	 7
U+5FE4.0	fCangjie	POJ
U+5FE4.0	fFourCorner	98040
U+5FE4.0	fHakka	 ng3; ngu6
U+5FE4.0	fCantonese	ng5; ng6
U+5FE4.0	fMandarin	wu3; wu4
U+5FE4.0	fEnglish	[1] recalcitrant; stubbornly; defiant; disobedient; uncongenial [2] blunder; wrong; mistake
U+5FE5.0	fUTF8	忥
U+5FE5.0	fCNS11643	2-265A
U+5FE5.0	fBig5	CC79
U+5FE5.0	fGB	3-4C23
U+5FE5.0	fR/S	 61.4
U+5FE5.0	fTotalStrokes	 8
U+5FE5.0	fCangjie	ONP
U+5FE5.0	fFourCorner	80317
U+5FE5.0	fCantonese	hei3
U+5FE5.0	fMandarin	xi4
U+5FE6.0	fUTF8	忦
U+5FE6.0	fCNS11643	3-2576
U+5FE6.0	fGB	3-513E
U+5FE6.0	fR/S	  61.4
U+5FE6.0	fTotalStrokes	 7
U+5FE6.0	fCangjie	POLL
U+5FE6.0	fFourCorner	98020
U+5FE6.0	fCantonese	git3
U+5FE6.0	fMandarin	ye4
U+5FE7.0	fUTF8	忧
U+5FE7.0	fCNS11643	3-2574
U+5FE7.0	fGB	0-5347
U+5FE7.0	fR/S	 61.4
U+5FE7.0	fTotalStrokes	 7
U+5FE7.0	fCangjie	PIKU
U+5FE7.0	fFourCorner	93010
U+5FE7.0	fHakka	 ju2
U+5FE7.0	fCantonese	jau1
U+5FE7.0	fMandarin	you4; you1
U+5FE7.0	fEnglish	[1] simplified form of &#x6182; [2] simplified form of &#x6182;
U+5FE8.0	fUTF8	忨
U+5FE8.0	fCNS11643	2-237A
U+5FE8.0	fBig5	CADA
U+5FE8.0	fGB	5-5347
U+5FE8.0	fR/S	 61.4
U+5FE8.0	fTotalStrokes	 7
U+5FE8.0	fCangjie	PMMU
U+5FE8.0	fFourCorner	91011
U+5FE8.0	fHakka	  ngan5
U+5FE8.0	fCantonese	wun6; jyun4
U+5FE8.0	fMandarin	wan4; wan2
U+5FE8.0	fEnglish	 desire; long for
U+5FE9.0	fUTF8	忩
U+5FE9.0	fCNS11643	3-287E
U+5FE9.0	fGB	E-2973
U+5FE9.0	fR/S	 61.4
U+5FE9.0	fTotalStrokes	 8; 7
U+5FE9.0	fCangjie	CIP
U+5FE9.0	fFourCorner	80330
U+5FE9.0	fHakka	 cung1
U+5FE9.0	fCantonese	cung1
U+5FE9.0	fMandarin	cong1
U+5FE9.0	fEnglish	same as &#x6031;
U+5FEA.0	fUTF8	忪
U+5FEA.0	fCNS11643	1-4A3A
U+5FEA.0	fBig5	A7D8
U+5FEA.0	fGB	0-626C
U+5FEA.0	fR/S	 61.4
U+5FEA.0	fTotalStrokes	 7
U+5FEA.0	fCangjie	PCI
U+5FEA.0	fFourCorner	98032
U+5FEA.0	fHakka	 chung1; cung1; zung1; zhung1
U+5FEA.0	fCantonese	zung1; sung1
U+5FEA.0	fMandarin	zhong1
U+5FEA.0	fEnglish	[1] agitated [2] frightened
U+5FEA.1	fUTF8	忪
U+5FEA.1	fCNS11643	1-4A3A
U+5FEA.1	fBig5	A7D8
U+5FEA.1	fGB	0-626C
U+5FEA.1	fR/S	 61.4
U+5FEA.1	fTotalStrokes	 7
U+5FEA.1	fCangjie	PCI
U+5FEA.1	fFourCorner	98032
U+5FEA.1	fMandarin	song1
U+5FEA.1	fEnglish	used in combination
U+5FEB.0	fUTF8	快
U+5FEB.0	fCNS11643	1-4A38
U+5FEB.0	fBig5	A7D6
U+5FEB.0	fGB	0-3F6C
U+5FEB.0	fR/S	 61.4
U+5FEB.0	fTotalStrokes	 7
U+5FEB.0	fCangjie	PDK
U+5FEB.0	fFourCorner	95030
U+5FEB.0	fHakka	 kai5; kwai5; k'uai5; fai5
U+5FEB.0	fCantonese	faai3; faai1 (also); faai2 (also)
U+5FEB.0	fMandarin	kuai4
U+5FEB.0	fEnglish	[1] rapid; quickly; fast; soon; hasty; prompt [2] nearly; near [3] hurry up; make haste [4] quick-witted; ingenious [5] sharp (blades, etc.); keen [6] pleasant; happy; make happy; pleasurable [7] used in combination [8] honest; straightforward
U+5FEB.1	fUTF8	快
U+5FEB.1	fCNS11643	1-4A38
U+5FEB.1	fBig5	A7D6
U+5FEB.1	fGB	0-3F6C
U+5FEB.1	fR/S	 61.4
U+5FEB.1	fTotalStrokes	 7
U+5FEB.1	fCangjie	PDK
U+5FEB.1	fFourCorner	95030
U+5FEB.1	fMandarin	kuai5
U+5FEC.0	fUTF8	忬
U+5FEC.0	fCNS11643	3-257C
U+5FEC.0	fGB	5-5353
U+5FEC.0	fR/S	  61.4
U+5FEC.0	fTotalStrokes	 7
U+5FEC.0	fCangjie	PNIN
U+5FEC.0	fFourCorner	97020
U+5FEC.0	fHakka	 i5
U+5FEC.0	fCantonese	jyu6
U+5FEC.0	fMandarin	yu4
U+5FED.0	fUTF8	忭
U+5FED.0	fCNS11643	2-2379
U+5FED.0	fBig5	CAD9
U+5FED.0	fGB	0-626D
U+5FED.0	fR/S	 61.4
U+5FED.0	fTotalStrokes	 7
U+5FED.0	fCangjie	PYY
U+5FED.0	fFourCorner	90030
U+5FED.0	fHakka	 pien5; pien6; pen3; pien3
U+5FED.0	fCantonese	bin6
U+5FED.0	fMandarin	bian4
U+5FED.0	fEnglish	overjoyed; delighted; pleased
U+5FEE.0	fUTF8	忮
U+5FEE.0	fCNS11643	2-237B
U+5FEE.0	fBig5	CADB
U+5FEE.0	fGB	0-6265
U+5FEE.0	fR/S	 61.4
U+5FEE.0	fTotalStrokes	 7
U+5FEE.0	fCangjie	PJE
U+5FEE.0	fFourCorner	94047
U+5FEE.0	fHakka	 ki5; zhi5; zih5; zi5; ki6; tje5
U+5FEE.0	fCantonese	zi3; gei6
U+5FEE.0	fMandarin	zhi4; ji4; qi2; qi3; zhi1
U+5FEE.0	fEnglish	jealous; jealousy; dislike
U+5FEF.0	fUTF8	忯
U+5FEF.0	fCNS11643	2-2423
U+5FEF.0	fBig5	CAE1
U+5FEF.0	fGB	3-5141
U+5FEF.0	fR/S	 61.4
U+5FEF.0	fTotalStrokes	 7
U+5FEF.0	fCangjie	PHVP
U+5FEF.0	fFourCorner	92040
U+5FEF.0	fHakka	 ki2
U+5FEF.0	fCantonese	kei4; si6; kei2
U+5FEF.0	fMandarin	qi2; chi2; di4; shi4
U+5FEF.0	fEnglish	 respect; esteem
U+5FF0.0	fUTF8	忰
U+5FF0.0	fCNS11643	3-2622
U+5FF0.0	fGB	E-2974
U+5FF0.0	fR/S	 61.4
U+5FF0.0	fTotalStrokes	 7
U+5FF0.0	fCangjie	PKNJ
U+5FF0.0	fFourCorner	94041
U+5FF0.0	fHakka	 cui5
U+5FF0.0	fCantonese	seoi6; seoi5
U+5FF0.0	fMandarin	cui4; zu2
U+5FF0.0	fEnglish	suffer, become emaciated, haggard
U+5FF1.0	fUTF8	忱
U+5FF1.0	fCNS11643	1-4A37
U+5FF1.0	fBig5	A7D5
U+5FF1.0	fGB	0-3340
U+5FF1.0	fR/S	 61.4
U+5FF1.0	fTotalStrokes	 7
U+5FF1.0	fCangjie	PLBU
U+5FF1.0	fFourCorner	94012
U+5FF1.0	fHakka	 zhim1; sim3; shim1; zihm1; cim2; zim1; zim3
U+5FF1.0	fCantonese	sam4
U+5FF1.0	fMandarin	chen2
U+5FF1.0	fEnglish	[1] sincerity; sincere [2] rely on
U+5FF2.0	fUTF8	忲
U+5FF2.0	fCNS11643	3-2621
U+5FF2.0	fGB	5-5349
U+5FF2.0	fR/S	 61.4
U+5FF2.0	fTotalStrokes	 7
U+5FF2.0	fCangjie	PKI
U+5FF2.0	fFourCorner	94030
U+5FF2.0	fHakka	  tai5
U+5FF2.0	fCantonese	taai3; tai3; sai6
U+5FF2.0	fMandarin	tai4; dai4; she2; shi4; tai3
U+5FF2.0	fEnglish	same as &#x5FD5;
U+5FF3.0	fUTF8	忳
U+5FF3.0	fCNS11643	2-237C
U+5FF3.0	fBig5	CADC
U+5FF3.0	fGB	3-513B
U+5FF3.0	fR/S	 61.4
U+5FF3.0	fTotalStrokes	 7
U+5FF3.0	fCangjie	PPU
U+5FF3.0	fFourCorner	95017
U+5FF3.0	fHakka	  tun2; tun6
U+5FF3.0	fCantonese	tyun4; zeon1
U+5FF3.0	fMandarin	tun2; zhun1; chun2; dun3; dun4
U+5FF3.0	fEnglish	sad; melancholy
U+5FF4.0	fUTF8	忴
U+5FF4.0	fCNS11643	2-2427
U+5FF4.0	fBig5	CAE5
U+5FF4.0	fGB	3-5140
U+5FF4.0	fR/S	 61.4
U+5FF4.0	fTotalStrokes	 7
U+5FF4.0	fCangjie	POIN
U+5FF4.0	fFourCorner	98027
U+5FF4.0	fCantonese	kim4; kam4
U+5FF4.0	fMandarin	qian2; qin2
U+5FF5.0	fUTF8	念
U+5FF5.0	fCNS11643	1-4D42
U+5FF5.0	fBig5	A9C0
U+5FF5.0	fGB	0-446E
U+5FF5.0	fR/S	 61.4
U+5FF5.0	fTotalStrokes	 8
U+5FF5.0	fCangjie	OINP
U+5FF5.0	fFourCorner	80332
U+5FF5.0	fHakka	 ngiam5; niam5; ngiam6; gniam6; ngiam3
U+5FF5.0	fCantonese	nim6; lim6 (also); je6 (also); din6; jaa6 (also)
U+5FF5.0	fMandarin	nian4
U+5FF5.0	fEnglish	[1] think of; miss; remember [2] study; attend school [3] read out aloud; chant [4] twenty [5] a Chinese family name [6] twenty [7] speak
U+5FF6.0	fUTF8	忶
U+5FF6.0	fCNS11643	3-257D
U+5FF6.0	fGB	5-5348
U+5FF6.0	fR/S	  61.4
U+5FF6.0	fTotalStrokes	 7
U+5FF6.0	fCangjie	PMMI
U+5FF6.0	fFourCorner	91030
U+5FF6.0	fHakka	  fun2
U+5FF6.0	fCantonese	wan4
U+5FF6.0	fMandarin	hun2
U+5FF6.0	fEnglish	melancholy, dull
U+5FF7.0	fUTF8	忷
U+5FF7.0	fCNS11643	2-2424
U+5FF7.0	fBig5	CAE2
U+5FF7.0	fGB	5-534E
U+5FF7.0	fR/S	 61.4
U+5FF7.0	fTotalStrokes	 7
U+5FF7.0	fCangjie	PUK
U+5FF7.0	fFourCorner	92070
U+5FF7.0	fHakka	 hiung1
U+5FF7.0	fCantonese	hung1
U+5FF7.0	fMandarin	xiong1; xiong3
U+5FF7.0	fEnglish	same as &#x605F;
U+5FF8.0	fUTF8	忸
U+5FF8.0	fCNS11643	1-4A39
U+5FF8.0	fBig5	A7D7
U+5FF8.0	fGB	0-626E
U+5FF8.0	fR/S	 61.4
U+5FF8.0	fTotalStrokes	 7
U+5FF8.0	fCangjie	PNG
U+5FF8.0	fFourCorner	97015
U+5FF8.0	fHakka	  ngiu3;  nuk7;  nuk8; luk8
U+5FF8.0	fCantonese	nau2; nuk6; lau2 (also); luk6 (also); nau5
U+5FF8.0	fMandarin	niu3; nuu4; n4
U+5FF8.0	fEnglish	[1] bashful; ashamed [2] be accustomed to; be inclined to (evils, etc.)
U+5FF9.0	fUTF8	忹
U+5FF9.0	fCNS11643	3-2623
U+5FF9.0	fGB	3-5137
U+5FF9.0	fR/S	 61.4
U+5FF9.0	fTotalStrokes	 7
U+5FF9.0	fCangjie	PMG
U+5FF9.0	fFourCorner	92010
U+5FF9.0	fHakka	 an3; ngim2;  ngim5; ngim3; ngim6; an2; an5; jim5
U+5FF9.0	fCantonese	jam6; jam5
U+5FF9.0	fMandarin	ren4; nin2; nen4
U+5FF9.0	fEnglish	same as &#x6041;
U+5FFA.0	fUTF8	忺
U+5FFA.0	fCNS11643	2-2422
U+5FFA.0	fBig5	CAE0
U+5FFA.0	fGB	3-5142
U+5FFA.0	fR/S	 61.4
U+5FFA.0	fTotalStrokes	 7
U+5FFA.0	fCangjie	PNO
U+5FFA.0	fFourCorner	97082
U+5FFA.0	fHakka	 kiam1; zii5
U+5FFA.0	fCantonese	him1 (also); hin1
U+5FFA.0	fMandarin	xian1
U+5FFB.0	fUTF8	忻
U+5FFB.0	fCNS11643	2-2425
U+5FFB.0	fBig5	CAE3
U+5FFB.0	fGB	0-5043
U+5FFB.0	fR/S	 61.4
U+5FFB.0	fTotalStrokes	 7
U+5FFB.0	fCangjie	PHML
U+5FFB.0	fFourCorner	92021
U+5FFB.0	fHakka	 hiun1
U+5FFB.0	fCantonese	jan1
U+5FFB.0	fMandarin	xin1
U+5FFB.0	fEnglish	happy; delight; joy
U+5FFC.0	fUTF8	忼
U+5FFC.0	fCNS11643	3-257A
U+5FFC.0	fGB	5-5350
U+5FFC.0	fR/S	  61.4
U+5FFC.0	fTotalStrokes	 7
U+5FFC.0	fCangjie	PYHN
U+5FFC.0	fFourCorner	90010
U+5FFC.0	fHakka	  kong3; hong2
U+5FFC.0	fCantonese	kong2; hong2 (also); hung2
U+5FFC.0	fMandarin	kang3; kang1; hang1
U+5FFC.0	fEnglish	[1] excited [2] annoyed
U+5FFD.0	fUTF8	忽
U+5FFD.0	fCNS11643	1-4D41
U+5FFD.0	fBig5	A9BF
U+5FFD.0	fGB	0-3A76
U+5FFD.0	fR/S	 61.4
U+5FFD.0	fTotalStrokes	 8
U+5FFD.0	fCangjie	PHP
U+5FFD.0	fFourCorner	27332
U+5FFD.0	fHakka	  fut7
U+5FFD.0	fCantonese	fat1
U+5FFD.0	fMandarin	hu1
U+5FFD.0	fEnglish	[1] suddenly; abruptly; unexpectedly [2] neglect; disregard; be careless; be indifferent [3] forget [4] one millionth of a tael [5] a Chinese family name
U+5FFE.0	fUTF8	忾
U+5FFE.0	fCNS11643	3-2575
U+5FFE.0	fGB	0-6269
U+5FFE.0	fR/S	 61.4
U+5FFE.0	fTotalStrokes	 7
U+5FFE.0	fCangjie	XPOMN
U+5FFE.0	fFourCorner	98010
U+5FFE.0	fHakka	 hi5; koi3; kai3
U+5FFE.0	fCantonese	koi3; koi3hei3
U+5FFE.0	fMandarin	xi1; kai4; qi4; xi4
U+5FFE.0	fEnglish	simplified form of &#x613E;
U+5FFF.0	fUTF8	忿
U+5FFF.0	fCNS11643	1-4D43
U+5FFF.0	fBig5	A9C1
U+5FFF.0	fGB	0-375E
U+5FFF.0	fR/S	 61.4
U+5FFF.0	fTotalStrokes	 8
U+5FFF.0	fCangjie	CSHP
U+5FFF.0	fFourCorner	80332
U+5FFF.0	fHakka	 fun3; fun5
U+5FFF.0	fCantonese	fan5; fan6 (also); fan2
U+5FFF.0	fMandarin	fen4; fen3
U+5FFF.0	fEnglish	[1] anger; fury; indignation [2] complaining; hatred; grudge
U+6000.0	fUTF8	怀
U+6000.0	fCNS11643	2-2426
U+6000.0	fBig5	CAE4
U+6000.0	fGB	0-3B33
U+6000.0	fR/S	 61.4
U+6000.0	fTotalStrokes	 7
U+6000.0	fCangjie	PMF
U+6000.0	fFourCorner	91090
U+6000.0	fHakka	 fai2; pi1
U+6000.0	fCantonese	waai4
U+6000.0	fMandarin	huai2
U+6000.0	fEnglish	simplified form of &#x61F7;
U+6001.0	fUTF8	态
U+6001.0	fGB	0-4C2C
U+6001.0	fR/S	 61.4
U+6001.0	fTotalStrokes	 8
U+6001.0	fCangjie	KIP
U+6001.0	fHakka	 tai5
U+6001.0	fCantonese	taai3
U+6001.0	fMandarin	tai4; tai3
U+6001.0	fEnglish	simplified form of &#x614B;
U+6002.0	fUTF8	怂
U+6002.0	fGB	0-4B4B
U+6002.0	fR/S	 61.4
U+6002.0	fTotalStrokes	 8
U+6002.0	fCangjie	XOOP
U+6002.0	fHakka	 ziung3; sung3; siung3
U+6002.0	fCantonese	sung2
U+6002.0	fMandarin	song3
U+6002.0	fEnglish	simplified form of &#x616B;
U+6003.0	fUTF8	怃
U+6003.0	fGB	0-6264
U+6003.0	fR/S	 61.5; 61.4
U+6003.0	fTotalStrokes	 7
U+6003.0	fCangjie	PMKU
U+6003.0	fHakka	 vu3
U+6003.0	fCantonese	mou5
U+6003.0	fMandarin	wu3; hu1; wu1; xu3
U+6003.0	fEnglish	simplified form of &#x61AE;
U+6004.0	fUTF8	怄
U+6004.0	fGB	0-6266
U+6004.0	fR/S	 61.4
U+6004.0	fTotalStrokes	 7
U+6004.0	fCangjie	PSK
U+6004.0	fHakka	 eu1
U+6004.0	fCantonese	au3; ngau3 (also); au1 (also); ngau1 (also)
U+6004.0	fMandarin	ou4; ou1; kou1
U+6004.0	fEnglish	simplified form of &#x616A;
U+6005.0	fUTF8	怅
U+6005.0	fGB	0-626A
U+6005.0	fR/S	 61.4
U+6005.0	fTotalStrokes	 7
U+6005.0	fCangjie	PPO
U+6005.0	fHakka	 zhong5
U+6005.0	fCantonese	coeng3; zoeng3 (also)
U+6005.0	fMandarin	chang4
U+6005.0	fEnglish	simplified form of &#x60B5;
U+6006.0	fUTF8	怆
U+6006.0	fGB	0-626B
U+6006.0	fR/S	 61.4
U+6006.0	fTotalStrokes	 7
U+6006.0	fCangjie	POSU
U+6006.0	fHakka	 cong3; cong1
U+6006.0	fCantonese	cong3
U+6006.0	fMandarin	chuang4; chuan4; chuang1; chuang3
U+6006.0	fEnglish	simplified form of &#x6134;
U+6007.0	fUTF8	怇
U+6007.0	fCNS11643	3-2878
U+6007.0	fGB	5-534B
U+6007.0	fR/S	 61.5
U+6007.0	fTotalStrokes	 8
U+6007.0	fCangjie	PSS
U+6007.0	fFourCorner	91010
U+6007.0	fHakka	 ki3
U+6007.0	fCantonese	geoi6
U+6007.0	fMandarin	ju4
U+6007.0	fEnglish	[1] reach [2] go to
U+6008.0	fUTF8	怈
U+6008.0	fCNS11643	3-2922
U+6008.0	fGB	5-5355
U+6008.0	fR/S	  61.5
U+6008.0	fTotalStrokes	 8
U+6008.0	fCangjie	PPT
U+6008.0	fFourCorner	94010
U+6008.0	fHakka	 i5
U+6008.0	fMandarin	yi4
U+6008.0	fEnglish	same as &#x2263D;
U+6009.0	fUTF8	怉
U+6009.0	fCNS11643	2-266E
U+6009.0	fBig5	CCAF
U+6009.0	fGB	3-514C
U+6009.0	fR/S	 61.5
U+6009.0	fTotalStrokes	 8
U+6009.0	fCangjie	PPRU
U+6009.0	fFourCorner	97012
U+6009.0	fCantonese	baau2; pou5
U+6009.0	fMandarin	bao3; bao4
U+600A.0	fUTF8	怊
U+600A.0	fCNS11643	2-2661
U+600A.0	fBig5	CCA2
U+600A.0	fGB	0-6277
U+600A.0	fR/S	 61.5
U+600A.0	fTotalStrokes	 8
U+600A.0	fCangjie	PSHR
U+600A.0	fFourCorner	97062
U+600A.0	fHakka	 chau1; cau1
U+600A.0	fCantonese	ciu1; tiu4 (also)
U+600A.0	fMandarin	chao1
U+600A.0	fEnglish	 be grieved
U+600B.0	fUTF8	怋
U+600B.0	fCNS11643	2-265F
U+600B.0	fBig5	CC7E
U+600B.0	fGB	3-514E
U+600B.0	fR/S	 61.5
U+600B.0	fTotalStrokes	 8
U+600B.0	fCangjie	PRVP
U+600B.0	fFourCorner	97047
U+600B.0	fCantonese	man4; man5
U+600B.0	fMandarin	min2; hun1; men2; mian4
U+600C.0	fUTF8	怌
U+600C.0	fCNS11643	2-266D
U+600C.0	fBig5	CCAE
U+600C.0	fGB	5-5358
U+600C.0	fR/S	 61.5
U+600C.0	fTotalStrokes	 8
U+600C.0	fCangjie	PMFM
U+600C.0	fFourCorner	91019
U+600C.0	fHakka	 pi1
U+600C.0	fCantonese	pei1; bei1
U+600C.0	fMandarin	pei1; pi1
U+600C.0	fEnglish	timid; frightened
U+600D.0	fUTF8	怍
U+600D.0	fCNS11643	2-2668
U+600D.0	fBig5	CCA9
U+600D.0	fGB	0-6274
U+600D.0	fR/S	 61.5
U+600D.0	fTotalStrokes	 8
U+600D.0	fCangjie	POS
U+600D.0	fFourCorner	98011
U+600D.0	fHakka	  cok7; cok8
U+600D.0	fCantonese	zok6 (also); zok3
U+600D.0	fMandarin	zuo4
U+600D.0	fEnglish	[1] sham [2] change color; blush
U+600E.0	fUTF8	怎
U+600E.0	fCNS11643	1-512B
U+600E.0	fBig5	ABE7
U+600E.0	fGB	0-5475
U+600E.0	fR/S	 61.5
U+600E.0	fTotalStrokes	 9
U+600E.0	fCangjie	OSP
U+600E.0	fFourCorner	80331
U+600E.0	fHakka	 zen1; zam3
U+600E.0	fCantonese	zam2; zaam2; dim2
U+600E.0	fMandarin	zen3; jin1; zou3
U+600E.0	fEnglish	what; why; how
U+600E.1	fUTF8	怎
U+600E.1	fCNS11643	1-512B
U+600E.1	fBig5	ABE7
U+600E.1	fGB	0-5475
U+600E.1	fR/S	 61.5
U+600E.1	fTotalStrokes	 9
U+600E.1	fCangjie	OSP
U+600E.1	fFourCorner	80331
U+600E.1	fMandarin	ze3
U+600E.1	fEnglish	what?; why?; how?
U+600F.0	fUTF8	怏
U+600F.0	fCNS11643	1-4D44
U+600F.0	fBig5	A9C2
U+600F.0	fGB	0-6273
U+600F.0	fR/S	 61.5
U+600F.0	fTotalStrokes	 8
U+600F.0	fCangjie	PLBK
U+600F.0	fFourCorner	95030
U+600F.0	fHakka	 jong3; rong3; jong6; jong5
U+600F.0	fCantonese	joeng2; joeng3 (also)
U+600F.0	fMandarin	yang4; yang1; yang3
U+600F.0	fEnglish	discontented; dispirited; disheartened
U+6010.0	fUTF8	怐
U+6010.0	fCNS11643	2-2669
U+6010.0	fBig5	CCAA
U+6010.0	fGB	3-5149
U+6010.0	fR/S	 61.5
U+6010.0	fTotalStrokes	 8
U+6010.0	fCangjie	PPR
U+6010.0	fFourCorner	97020
U+6010.0	fHakka	 k'ieu5; kiu5; keu5
U+6010.0	fCantonese	kau3; hau6; ngau6
U+6010.0	fMandarin	kou4; gou4; hou4; ju4
U+6010.0	fEnglish	simple; stupid
U+6011.0	fUTF8	怑
U+6011.0	fCNS11643	2-266C
U+6011.0	fBig5	CCAD
U+6011.0	fGB	5-5361
U+6011.0	fR/S	 61.5
U+6011.0	fTotalStrokes	 8
U+6011.0	fCangjie	PFQ
U+6011.0	fFourCorner	99050
U+6011.0	fCantonese	bun6
U+6011.0	fMandarin	ban4
U+6012.0	fUTF8	怒
U+6012.0	fCNS11643	1-5127
U+6012.0	fBig5	ABE3
U+6012.0	fGB	0-452D
U+6012.0	fR/S	 61.5
U+6012.0	fTotalStrokes	 9
U+6012.0	fCangjie	VEP
U+6012.0	fFourCorner	47334
U+6012.0	fHakka	 nu5; lu5; lu3
U+6012.0	fCantonese	nou6; lou6 (also)
U+6012.0	fMandarin	nu4; nu3
U+6012.0	fEnglish	[1] temper; anger; rage; angry; furious [2] put forth with vigor (as plants, etc.); sprout; spring up
U+6013.0	fUTF8	怓
U+6013.0	fCNS11643	2-266B
U+6013.0	fBig5	CCAC
U+6013.0	fGB	3-5150
U+6013.0	fR/S	 61.5
U+6013.0	fTotalStrokes	 8
U+6013.0	fCangjie	PVE
U+6013.0	fFourCorner	97040
U+6013.0	fHakka	  nau2; nau5
U+6013.0	fCantonese	naau4; laau4 (also)
U+6013.0	fMandarin	nao2
U+6013.0	fEnglish	confusion; confused; wild
U+6014.0	fUTF8	怔
U+6014.0	fCNS11643	1-4D45
U+6014.0	fBig5	A9C3
U+6014.0	fGB	0-557A
U+6014.0	fR/S	 61.5
U+6014.0	fTotalStrokes	 8
U+6014.0	fCangjie	PMYM
U+6014.0	fFourCorner	91011
U+6014.0	fHakka	 zhin1; zihn1; zin1; zhin5
U+6014.0	fCantonese	zing1; ling6
U+6014.0	fMandarin	zheng1; zheng4
U+6014.0	fEnglish	terrified; stunned; scared
U+6015.0	fUTF8	怕
U+6015.0	fCNS11643	1-4D4A
U+6015.0	fBig5	A9C8
U+6015.0	fGB	0-4542
U+6015.0	fR/S	 61.5
U+6015.0	fTotalStrokes	 8
U+6015.0	fCangjie	PHA
U+6015.0	fFourCorner	96000
U+6015.0	fHakka	  pa5
U+6015.0	fCantonese	paa3; paa2 (also)
U+6015.0	fMandarin	pa4; bo2; po4
U+6015.0	fEnglish	[1] fear; dread; afraid; scared or frightened; apprehensive [2] maybe; perhaps; I am afraid ...; I suppose
U+6016.0	fUTF8	怖
U+6016.0	fCNS11643	1-4D48
U+6016.0	fBig5	A9C6
U+6016.0	fGB	0-3240
U+6016.0	fR/S	 61.5
U+6016.0	fTotalStrokes	 8
U+6016.0	fCangjie	PKLB
U+6016.0	fFourCorner	94027
U+6016.0	fHakka	  bu5
U+6016.0	fCantonese	bou3
U+6016.0	fMandarin	bu4; pu4
U+6016.0	fEnglish	[1] terrified; frightened [2] frighten; threaten
U+6017.0	fUTF8	怗
U+6017.0	fCNS11643	2-2662
U+6017.0	fBig5	CCA3
U+6017.0	fGB	3-5145
U+6017.0	fR/S	 61.5
U+6017.0	fTotalStrokes	 8
U+6017.0	fCangjie	PYR
U+6017.0	fFourCorner	91060
U+6017.0	fHakka	  tiap7
U+6017.0	fCantonese	tip3; zim1
U+6017.0	fMandarin	tie1; chan1
U+6017.0	fEnglish	[1] observant; submissive; subservient; compliant [2] peaceful; quiet
U+6018.0	fUTF8	怘
U+6018.0	fCNS11643	4-2936
U+6018.0	fGB	5-526D
U+6018.0	fR/S	  61.5
U+6018.0	fTotalStrokes	 9
U+6018.0	fCangjie	JRP
U+6018.0	fFourCorner	40330
U+6018.0	fCantonese	gu3; gwu3 (also)
U+6018.0	fMandarin	hu4; gu4
U+6019.0	fUTF8	怙
U+6019.0	fCNS11643	2-265D
U+6019.0	fBig5	CC7C
U+6019.0	fGB	0-626F
U+6019.0	fR/S	 61.5
U+6019.0	fTotalStrokes	 8
U+6019.0	fCangjie	PJR
U+6019.0	fFourCorner	94060
U+6019.0	fHakka	 fu3; fu5; fu6
U+6019.0	fCantonese	wu6
U+6019.0	fMandarin	hu4
U+6019.0	fEnglish	[1] rely on; presume on [2] one's father; things or persons that one relies on
U+601A.0	fUTF8	怚
U+601A.0	fCNS11643	2-2664
U+601A.0	fBig5	CCA5
U+601A.0	fGB	3-5146
U+601A.0	fR/S	 61.5
U+601A.0	fTotalStrokes	 8
U+601A.0	fCangjie	PBM
U+601A.0	fFourCorner	97010
U+601A.0	fHakka	 zi1; cu1; zii1; zih1
U+601A.0	fCantonese	zeoi6; cou1; zi1
U+601A.0	fMandarin	ju4
U+601A.0	fEnglish	dull; stupid; suspicious
U+601B.0	fUTF8	怛
U+601B.0	fCNS11643	1-4D4F
U+601B.0	fBig5	A9CD
U+601B.0	fGB	0-6272
U+601B.0	fR/S	 61.5
U+601B.0	fTotalStrokes	 8
U+601B.0	fCangjie	PAM
U+601B.0	fFourCorner	96010
U+601B.0	fHakka	  dat7;  tan3; }
U+601B.0	fCantonese	daat3; taan2 (also)
U+601B.0	fMandarin	da2; dan4
U+601B.0	fEnglish	[1] grieved; distressed [2] surprised; shocked; alarmed [3] striving and toiling
U+601C.0	fUTF8	怜
U+601C.0	fCNS11643	2-266F
U+601C.0	fBig5	CCB0
U+601C.0	fGB	0-412F
U+601C.0	fR/S	 61.5
U+601C.0	fTotalStrokes	 8
U+601C.0	fCangjie	POII
U+601C.0	fFourCorner	98037
U+601C.0	fHakka	 lin2; lien2
U+601C.0	fCantonese	lin4; ling4
U+601C.0	fMandarin	lin2
U+601C.0	fEnglish	[1] pity [2] simplified form of &#x6190;
U+601C.1	fUTF8	怜
U+601C.1	fCNS11643	2-266F
U+601C.1	fBig5	CCB0
U+601C.1	fGB	0-412F
U+601C.1	fR/S	 61.5
U+601C.1	fTotalStrokes	 8
U+601C.1	fCangjie	POII
U+601C.1	fFourCorner	98037
U+601C.1	fMandarin	lian2
U+601C.1	fEnglish	agile; nimble; bright; cute and pleasing (often said of a child, young girl, etc.)
U+601D.0	fUTF8	思
U+601D.0	fCNS11643	1-5128
U+601D.0	fBig5	ABE4
U+601D.0	fGB	0-4B3C
U+601D.0	fR/S	 61.5
U+601D.0	fTotalStrokes	 9
U+601D.0	fCangjie	WP
U+601D.0	fFourCorner	60330
U+601D.0	fHakka	  sii1;  su1; sii5; se1; se5
U+601D.0	fCantonese	si1; si3; soi1
U+601D.0	fMandarin	si1; sai1; si4
U+601D.0	fEnglish	[1] think; consider; ponder; contemplate [2] memory; remembrance; remember; recall; think of [3] mourn; grieve [4] admire [5] pine for [6] a final particle to sound off an expression
U+601D.1	fUTF8	思
U+601D.1	fCNS11643	1-5128
U+601D.1	fBig5	ABE4
U+601D.1	fGB	0-4B3C
U+601D.1	fR/S	 61.5
U+601D.1	fTotalStrokes	 9
U+601D.1	fCangjie	WP
U+601D.1	fFourCorner	60330
U+601D.1	fMandarin	si5
U+601E.0	fUTF8	怞
U+601E.0	fCNS11643	2-2665
U+601E.0	fBig5	CCA6
U+601E.0	fGB	3-5147
U+601E.0	fR/S	 61.5
U+601E.0	fTotalStrokes	 8
U+601E.0	fCangjie	PLW
U+601E.0	fFourCorner	95060
U+601E.0	fHakka	 ciu5; ciu6; cu5; chiu6; chiu2
U+601E.0	fCantonese	jau4; zau6 (also); cau4
U+601E.0	fMandarin	you2; zhou4; chou1; zhou1
U+601E.0	fEnglish	 grieve sorrowful
U+601F.0	fUTF8	怟
U+601F.0	fCNS11643	4-264F
U+601F.0	fGB	5-535F
U+601F.0	fR/S	  61.5
U+601F.0	fTotalStrokes	 8
U+601F.0	fCangjie	PHPM
U+601F.0	fFourCorner	92014
U+601F.0	fHakka	 di5
U+601F.0	fCantonese	dai3
U+601F.0	fMandarin	di4
U+6020.0	fUTF8	怠
U+6020.0	fCNS11643	1-5129
U+6020.0	fBig5	ABE5
U+6020.0	fGB	0-3521
U+6020.0	fR/S	 61.5
U+6020.0	fTotalStrokes	 9
U+6020.0	fCangjie	IRP
U+6020.0	fFourCorner	23336
U+6020.0	fHakka	 tai5; tai6; tai3; toi3
U+6020.0	fCantonese	toi5; doi6 (also)
U+6020.0	fMandarin	dai4; tai1; tai3
U+6020.0	fEnglish	[1] idle; remiss; lax; negligent [2] treat coldly 
U+6021.0	fUTF8	怡
U+6021.0	fCNS11643	1-4D4B
U+6021.0	fBig5	A9C9
U+6021.0	fGB	0-6279
U+6021.0	fR/S	 61.5
U+6021.0	fTotalStrokes	 8
U+6021.0	fCangjie	PIR
U+6021.0	fFourCorner	93060
U+6021.0	fHakka	 ji2; ji3; ri2; i2
U+6021.0	fCantonese	ji4
U+6021.0	fMandarin	yi2
U+6021.0	fEnglish	[1] harmony; on good terms [2] pleasure; joy [3] a Chinese family name
U+6022.0	fUTF8	怢
U+6022.0	fCNS11643	2-2667
U+6022.0	fBig5	CCA8
U+6022.0	fGB	3-5148
U+6022.0	fR/S	 61.5
U+6022.0	fTotalStrokes	 8
U+6022.0	fCangjie	PHQO
U+6022.0	fFourCorner	95030
U+6022.0	fCantonese	dat6
U+6022.0	fMandarin	tu2; tui4; tu1
U+6023.0	fUTF8	怣
U+6023.0	fCNS11643	3-2C5A
U+6023.0	fGB	5-526F
U+6023.0	fR/S	  61.5
U+6023.0	fTotalStrokes	 9
U+6023.0	fCangjie	HOP
U+6023.0	fFourCorner	25330
U+6023.0	fHakka	  ju2; jiu2; jeu2; jiu3; riu2
U+6023.0	fCantonese	jau4
U+6023.0	fMandarin	you2
U+6023.0	fEnglish	ancient form of &#x5C24;
U+6024.0	fUTF8	怤
U+6024.0	fCNS11643	2-2A4E
U+6024.0	fBig5	CECD
U+6024.0	fGB	3-5072
U+6024.0	fR/S	 61.5
U+6024.0	fTotalStrokes	 9
U+6024.0	fCangjie	OIP
U+6024.0	fFourCorner	24330
U+6024.0	fHakka	  fu1
U+6024.0	fCantonese	fu1
U+6024.0	fMandarin	fu1; fu4
U+6024.0	fEnglish	 think on with pleasure
U+6025.0	fUTF8	急
U+6025.0	fCNS11643	1-512A
U+6025.0	fBig5	ABE6
U+6025.0	fGB	0-3C31
U+6025.0	fR/S	 61.5
U+6025.0	fTotalStrokes	 9
U+6025.0	fCangjie	NSP
U+6025.0	fFourCorner	27337
U+6025.0	fHakka	  gip7;  kip8; giap7
U+6025.0	fCantonese	gap1
U+6025.0	fMandarin	ji2
U+6025.0	fEnglish	[1] quick; quickly; with expedition [2] urgent; pressing; hurried; hasty [3] anxious; very eager; worried
U+6026.0	fUTF8	怦
U+6026.0	fCNS11643	2-265C
U+6026.0	fBig5	CC7B
U+6026.0	fGB	0-6271
U+6026.0	fR/S	 61.5
U+6026.0	fTotalStrokes	 8
U+6026.0	fCangjie	PMFJ
U+6026.0	fFourCorner	91049
U+6026.0	fHakka	 pin2
U+6026.0	fCantonese	ping1; paang1
U+6026.0	fMandarin	peng1
U+6026.0	fEnglish	eager; ardent; impulsive; anxious
U+6027.0	fUTF8	性
U+6027.0	fCNS11643	1-4D4C
U+6027.0	fBig5	A9CA
U+6027.0	fGB	0-5054
U+6027.0	fR/S	 61.5
U+6027.0	fTotalStrokes	 8
U+6027.0	fCangjie	PHQM
U+6027.0	fFourCorner	95010
U+6027.0	fHakka	  sin5; siang5
U+6027.0	fCantonese	sing3
U+6027.0	fMandarin	xing4
U+6027.0	fEnglish	[1] nature; character; natural property; disposition; temper [2] a quality or property [3] sex
U+6027.1	fUTF8	性
U+6027.1	fCNS11643	1-4D4C
U+6027.1	fBig5	A9CA
U+6027.1	fGB	0-5054
U+6027.1	fR/S	 61.5
U+6027.1	fTotalStrokes	 8
U+6027.1	fCangjie	PHQM
U+6027.1	fFourCorner	95010
U+6027.1	fMandarin	xing5
U+6028.0	fUTF8	怨
U+6028.0	fCNS11643	1-512C
U+6028.0	fBig5	ABE8
U+6028.0	fGB	0-5439
U+6028.0	fR/S	 61.5
U+6028.0	fTotalStrokes	 9
U+6028.0	fCangjie	NUP
U+6028.0	fFourCorner	27331
U+6028.0	fHakka	  jen5; jan5; ren5
U+6028.0	fCantonese	jyun3
U+6028.0	fMandarin	yuan4; yuan1; yun3; yung4
U+6028.0	fEnglish	[1] ill will; hatred; enmity; resentment; animus; resent [2] complain; blame (others)
U+6028.1	fUTF8	怨
U+6028.1	fCNS11643	1-512C
U+6028.1	fBig5	ABE8
U+6028.1	fGB	0-5439
U+6028.1	fR/S	 61.5
U+6028.1	fTotalStrokes	 9
U+6028.1	fCangjie	NUP
U+6028.1	fFourCorner	27331
U+6028.1	fMandarin	yuan5
U+6029.0	fUTF8	怩
U+6029.0	fCNS11643	1-4D4D
U+6029.0	fBig5	A9CB
U+6029.0	fGB	0-6275
U+6029.0	fR/S	 61.5
U+6029.0	fTotalStrokes	 8
U+6029.0	fCangjie	PSP
U+6029.0	fFourCorner	97011
U+6029.0	fHakka	  ni2;  ni3; li2
U+6029.0	fCantonese	nei4; lei4 (also)
U+6029.0	fMandarin	ni2; ni4; wan3
U+6029.0	fEnglish	[1] shy; coyly; bashful [2] blush; look embarrassed
U+602A.0	fUTF8	怪
U+602A.0	fCNS11643	1-4D49
U+602A.0	fBig5	A9C7
U+602A.0	fGB	0-3956
U+602A.0	fR/S	 61.5
U+602A.0	fTotalStrokes	 8
U+602A.0	fCangjie	PEG
U+602A.0	fFourCorner	97014
U+602A.0	fHakka	 gai5;  gwai5; kuai5; gwai3
U+602A.0	fCantonese	gwaai3; gwaai2 (also)
U+602A.0	fMandarin	guai4
U+602A.0	fEnglish	strange; unusual; peculiar; queer; monstrous; odd
U+602B.0	fUTF8	怫
U+602B.0	fCNS11643	1-4D4E
U+602B.0	fBig5	A9CC
U+602B.0	fGB	0-6276
U+602B.0	fR/S	 61.5
U+602B.0	fTotalStrokes	 8
U+602B.0	fCangjie	PLLN
U+602B.0	fFourCorner	95027
U+602B.0	fHakka	  fut7; fui5
U+602B.0	fCantonese	fat6; fai3 (also); bui6; bui3
U+602B.0	fMandarin	fu2; pei4
U+602B.0	fEnglish	depressed and discontented
U+602B.1	fUTF8	怫
U+602B.1	fCNS11643	1-4D4E
U+602B.1	fBig5	A9CC
U+602B.1	fGB	0-6276
U+602B.1	fR/S	 61.5
U+602B.1	fTotalStrokes	 8
U+602B.1	fCangjie	PLLN
U+602B.1	fFourCorner	95027
U+602B.1	fMandarin	fei4
U+602B.1	fEnglish	angry; annoyed; indignant
U+602C.0	fUTF8	怬
U+602C.0	fCNS11643	2-2666
U+602C.0	fBig5	CCA7
U+602C.0	fGB	5-535C
U+602C.0	fR/S	 61.5
U+602C.0	fTotalStrokes	 8
U+602C.0	fCangjie	PWC
U+602C.0	fFourCorner	96000
U+602C.0	fHakka	 si5
U+602C.0	fCantonese	hei3
U+602C.0	fMandarin	xi4
U+602D.0	fUTF8	怭
U+602D.0	fCNS11643	2-265B
U+602D.0	fBig5	CC7A
U+602D.0	fGB	3-514D
U+602D.0	fR/S	 61.5
U+602D.0	fTotalStrokes	 8
U+602D.0	fCangjie	PPH
U+602D.0	fFourCorner	93000
U+602D.0	fHakka	  bit7
U+602D.0	fCantonese	bit1; bat6
U+602D.0	fMandarin	bi4
U+602D.0	fEnglish	rude
U+602E.0	fUTF8	怮
U+602E.0	fCNS11643	2-266A
U+602E.0	fBig5	CCAB
U+602E.0	fGB	3-5152
U+602E.0	fR/S	 61.5
U+602E.0	fTotalStrokes	 8
U+602E.0	fCangjie	PVIS
U+602E.0	fFourCorner	94027
U+602E.0	fHakka	 ju2; riu2
U+602E.0	fCantonese	jau1; jau4
U+602E.0	fMandarin	you1
U+602E.0	fEnglish	sulky; morose
U+602F.0	fUTF8	怯
U+602F.0	fCNS11643	1-4D46
U+602F.0	fBig5	A9C4
U+602F.0	fGB	0-4753
U+602F.0	fR/S	 61.5
U+602F.0	fTotalStrokes	 8
U+602F.0	fCangjie	PGI
U+602F.0	fFourCorner	94031
U+602F.0	fHakka	 kiap7; hiap7; giap7; kiok7
U+602F.0	fCantonese	hip3
U+602F.0	fMandarin	qie4
U+602F.0	fEnglish	[1] lacking in courage; cowardly [2] nervous; socially timid; fright; fear; afraid
U+6030.0	fUTF8	怰
U+6030.0	fCNS11643	3-2877
U+6030.0	fGB	5-5360
U+6030.0	fR/S	 61.5
U+6030.0	fTotalStrokes	 8
U+6030.0	fCangjie	PYVI
U+6030.0	fFourCorner	90030
U+6030.0	fCantonese	jyun6
U+6030.0	fMandarin	xuan4
U+6031.0	fUTF8	怱
U+6031.0	fCNS11643	3-2C5B
U+6031.0	fGB	E-2975
U+6031.0	fR/S	  61.5
U+6031.0	fTotalStrokes	 9
U+6031.0	fCangjie	PKP
U+6031.0	fFourCorner	27330
U+6031.0	fHakka	  cung1
U+6031.0	fCantonese	cung1
U+6031.0	fMandarin	cong1; zong1
U+6031.0	fEnglish	hasty; hastily; in haste; hurriedly
U+6032.0	fUTF8	怲
U+6032.0	fCNS11643	2-265E
U+6032.0	fBig5	CC7D
U+6032.0	fGB	3-5143
U+6032.0	fR/S	 61.5
U+6032.0	fTotalStrokes	 8
U+6032.0	fCangjie	PMOB
U+6032.0	fFourCorner	91027
U+6032.0	fHakka	  biang5; biang3
U+6032.0	fCantonese	bing2; bing3; ping2
U+6032.0	fMandarin	bing3; bing4
U+6032.0	fEnglish	be anxious; be grieved; be sad
U+6033.0	fUTF8	怳
U+6033.0	fCNS11643	2-2663
U+6033.0	fBig5	CCA4
U+6033.0	fGB	E-2976
U+6033.0	fR/S	 61.5
U+6033.0	fTotalStrokes	 8
U+6033.0	fCangjie	PRHU
U+6033.0	fFourCorner	96010
U+6033.0	fHakka	  fong3
U+6033.0	fCantonese	fong2; fong3
U+6033.0	fMandarin	huang3; xing3
U+6033.0	fEnglish	[1] wild; mad; flurried [2] despondent; dejected
U+6034.0	fUTF8	怴
U+6034.0	fCNS11643	2-2660
U+6034.0	fBig5	CCA1
U+6034.0	fGB	3-5144
U+6034.0	fR/S	 61.5
U+6034.0	fTotalStrokes	 8
U+6034.0	fCangjie	PIV
U+6034.0	fFourCorner	93050
U+6034.0	fCantonese	wat1; hyut3
U+6034.0	fMandarin	xu4; yue4
U+6035.0	fUTF8	怵
U+6035.0	fCNS11643	1-4D47
U+6035.0	fBig5	A9C5
U+6035.0	fGB	0-6270
U+6035.0	fR/S	 61.5
U+6035.0	fTotalStrokes	 8
U+6035.0	fCangjie	PIJC
U+6035.0	fFourCorner	93094
U+6035.0	fHakka	  tut7; chot7
U+6035.0	fCantonese	zeot1; ceot1; seot1
U+6035.0	fMandarin	chu4; xu4
U+6035.0	fEnglish	[1] fear; afraid; timorous; scared [2] entice; induce
U+6036.0	fUTF8	怶
U+6036.0	fCNS11643	3-287A
U+6036.0	fGB	3-5151
U+6036.0	fR/S	  61.5
U+6036.0	fTotalStrokes	 8
U+6036.0	fCangjie	PDHE
U+6036.0	fFourCorner	94040
U+6036.0	fCantonese	bei3; pe5
U+6036.0	fMandarin	pi1
U+6036.0	fEnglish	have no zeal at all
U+6037.0	fUTF8	怷
U+6037.0	fCNS11643	2-2A40
U+6037.0	fBig5	CEBF
U+6037.0	fGB	E-2977
U+6037.0	fR/S	 61.5
U+6037.0	fTotalStrokes	 9
U+6037.0	fCangjie	ICP
U+6037.0	fFourCorner	43331
U+6037.0	fCantonese	sik1; seot6
U+6037.0	fMandarin	xi1; zhu2; zhu3
U+6038.0	fUTF8	怸
U+6038.0	fCNS11643	4-2945
U+6038.0	fGB	5-526E
U+6038.0	fR/S	 61.5
U+6038.0	fTotalStrokes	 9
U+6038.0	fCangjie	IDP
U+6038.0	fFourCorner	43330
U+6038.0	fCantonese	sik1
U+6038.0	fMandarin	xi1
U+6039.0	fUTF8	怹
U+6039.0	fCNS11643	2-2A41
U+6039.0	fBig5	CEC0
U+6039.0	fGB	3-5073
U+6039.0	fR/S	 61.5
U+6039.0	fTotalStrokes	 9
U+6039.0	fCangjie	ODP
U+6039.0	fFourCorner	24332
U+6039.0	fCantonese	taa1
U+6039.0	fMandarin	tan1
U+6039.0	fEnglish	a polite version of he
U+603A.0	fUTF8	怺
U+603A.0	fGB	E-2978
U+603A.0	fR/S	 61.5
U+603A.0	fTotalStrokes	 8
U+603A.0	fCangjie	PINE
U+603A.0	fCantonese	wing5
U+603A.0	fEnglish	 endure
U+603B.0	fUTF8	总
U+603B.0	fGB	0-575C
U+603B.0	fR/S	 12.7; 61.5
U+603B.0	fTotalStrokes	 9; 8
U+603B.0	fCangjie	CRP
U+603B.0	fHakka	 zung3
U+603B.0	fCantonese	zung2
U+603B.0	fMandarin	zong3; zong1; cong1
U+603B.0	fEnglish	simplified form of &#x7E3D;
U+603C.0	fUTF8	怼
U+603C.0	fGB	0-6D21
U+603C.0	fR/S	 61.5
U+603C.0	fTotalStrokes	 9; 8
U+603C.0	fCangjie	EIP
U+603C.0	fHakka	 dui5; chui5
U+603C.0	fCantonese	deoi6; zeoi6 (also)
U+603C.0	fMandarin	dui4; zhui4
U+603C.0	fEnglish	simplified form of &#x61DF;
U+603D.0	fUTF8	怽
U+603D.0	fCNS11643	3-2921
U+603D.0	fGB	E-2979
U+603D.0	fR/S	  61.5
U+603D.0	fTotalStrokes	 8
U+603D.0	fCangjie	PJD
U+603D.0	fFourCorner	95090
U+603D.0	fCantonese	mei6
U+603E.0	fUTF8	怾
U+603E.0	fGB	1-7D2A
U+603E.0	fR/S	 61.5
U+603E.0	fTotalStrokes	 8
U+603E.0	fCangjie	PRC
U+603E.0	fCantonese	sik1; ci3
U+603E.0	fEnglish	 mountain name
U+603F.0	fUTF8	怿
U+603F.0	fGB	0-6278
U+603F.0	fR/S	 61.5
U+603F.0	fTotalStrokes	 8
U+603F.0	fCangjie	PEQ
U+603F.0	fHakka	 jit8
U+603F.0	fCantonese	jik6
U+603F.0	fMandarin	yi4
U+603F.0	fEnglish	simplified form of &#x61CC;
U+6040.0	fUTF8	恀
U+6040.0	fCNS11643	2-2A4B
U+6040.0	fBig5	CECA
U+6040.0	fGB	3-5162
U+6040.0	fR/S	 61.6
U+6040.0	fTotalStrokes	 9
U+6040.0	fCangjie	PNIN
U+6040.0	fFourCorner	97027
U+6040.0	fHakka	 chi3; cih3; chii3
U+6040.0	fCantonese	ci2 (also); zi2; si6 (also)
U+6040.0	fMandarin	chi3; shi4; zhi3
U+6040.0	fEnglish	 cling to (as a child to its mother)
U+6041.0	fUTF8	恁
U+6041.0	fCNS11643	2-2F23
U+6041.0	fBig5	D1A1
U+6041.0	fGB	0-6D25
U+6041.0	fR/S	 61.6
U+6041.0	fTotalStrokes	 10
U+6041.0	fCangjie	OGP
U+6041.0	fFourCorner	22331
U+6041.0	fHakka	 an3; ngim2;  ngim5; ngim3; ngim6; an2; an5; jim5
U+6041.0	fCantonese	jam6; jam5
U+6041.0	fMandarin	ren4; nin2; nen4
U+6041.0	fEnglish	[1] this; thus; so; such [2] how; what
U+6042.0	fUTF8	恂
U+6042.0	fCNS11643	2-2A4C
U+6042.0	fBig5	CECB
U+6042.0	fGB	0-627E
U+6042.0	fR/S	 61.6
U+6042.0	fTotalStrokes	 9
U+6042.0	fCangjie	PPA
U+6042.0	fFourCorner	97020
U+6042.0	fHakka	  sun2; sun3
U+6042.0	fCantonese	seon1
U+6042.0	fMandarin	xun2; sun3; xun1; xun4
U+6042.0	fEnglish	[1] sincere [2] suddenly [3] honest; trust; have faith in [4] afraid; scared
U+6043.0	fUTF8	恃
U+6043.0	fCNS11643	1-5132
U+6043.0	fBig5	ABEE
U+6043.0	fGB	0-4A51
U+6043.0	fR/S	 61.6
U+6043.0	fTotalStrokes	 9
U+6043.0	fCangjie	PGDI
U+6043.0	fFourCorner	94041
U+6043.0	fHakka	 shi6; sii5; si5; she5; shi5
U+6043.0	fCantonese	ci5; si5
U+6043.0	fMandarin	shi4; zhi4
U+6043.0	fEnglish	 rely on; presume upon; to depend on
U+6044.0	fUTF8	恄
U+6044.0	fCNS11643	2-2A4F
U+6044.0	fBig5	CECE
U+6044.0	fGB	E-297A
U+6044.0	fR/S	 61.6
U+6044.0	fTotalStrokes	 9
U+6044.0	fCangjie	PGR
U+6044.0	fFourCorner	94061
U+6044.0	fCantonese	hat1
U+6044.0	fMandarin	xi4
U+6045.0	fUTF8	恅
U+6045.0	fCNS11643	2-2A45
U+6045.0	fBig5	CEC4
U+6045.0	fGB	3-5155
U+6045.0	fR/S	 61.6
U+6045.0	fTotalStrokes	 9
U+6045.0	fCangjie	PJKP
U+6045.0	fFourCorner	94011
U+6045.0	fHakka	 lau3
U+6045.0	fCantonese	lou5
U+6045.0	fMandarin	lao3
U+6045.0	fEnglish	confused; disturbed
U+6046.0	fUTF8	恆
U+6046.0	fCNS11643	1-5131
U+6046.0	fBig5	ABED
U+6046.0	fGB	E-297B
U+6046.0	fR/S	 61.6
U+6046.0	fTotalStrokes	 9
U+6046.0	fCangjie	PMBM
U+6046.0	fFourCorner	91017
U+6046.0	fHakka	 hen2
U+6046.0	fCantonese	hang4; gang2 (also); gang1; gang3
U+6046.0	fMandarin	heng2; geng4; geng1
U+6046.0	fEnglish	[1] constant; regular; persevering [2] lasting; continual; continually
U+6047.0	fUTF8	恇
U+6047.0	fCNS11643	2-2A47
U+6047.0	fBig5	CEC6
U+6047.0	fGB	3-5153
U+6047.0	fR/S	 61.6
U+6047.0	fTotalStrokes	 9
U+6047.0	fCangjie	PSMG
U+6047.0	fFourCorner	91011
U+6047.0	fHakka	  kiong1
U+6047.0	fCantonese	hong1
U+6047.0	fMandarin	kuang1; kuang3
U+6047.0	fEnglish	fear
U+6048.0	fUTF8	恈
U+6048.0	fCNS11643	4-293F
U+6048.0	fGB	3-516A
U+6048.0	fR/S	  61.6
U+6048.0	fTotalStrokes	 9
U+6048.0	fCangjie	PIHQ
U+6048.0	fFourCorner	93050
U+6048.0	fCantonese	mau4
U+6048.0	fMandarin	mu2; mou2
U+6049.0	fUTF8	恉
U+6049.0	fCNS11643	2-2A48
U+6049.0	fBig5	CEC7
U+6049.0	fGB	5-536E
U+6049.0	fR/S	 61.6
U+6049.0	fTotalStrokes	 9
U+6049.0	fCangjie	PPA
U+6049.0	fFourCorner	91061
U+6049.0	fCantonese	zi2
U+6049.0	fMandarin	zhi3; zhi4
U+6049.0	fEnglish	meaning; purport; intention
U+604A.0	fUTF8	恊
U+604A.0	fCNS11643	3-2C60
U+604A.0	fGB	5-5373
U+604A.0	fR/S	  61.6
U+604A.0	fTotalStrokes	 9
U+604A.0	fCangjie	PKSS
U+604A.0	fFourCorner	94020
U+604A.0	fHakka	  hiap8
U+604A.0	fCantonese	hip3
U+604A.0	fMandarin	xie2
U+604A.0	fEnglish	[1] intimidate [2] fearful and timid
U+604B.0	fUTF8	恋
U+604B.0	fCNS11643	3-3079
U+604B.0	fGB	0-4135
U+604B.0	fR/S	 61.6
U+604B.0	fTotalStrokes	 10
U+604B.0	fCangjie	YCP
U+604B.0	fFourCorner	00330
U+604B.0	fHakka	 lien5; lien6
U+604B.0	fCantonese	lyun2; lyun5 (also)
U+604B.0	fMandarin	lian4
U+604B.0	fEnglish	[1] non-classical form of &#x6200; [2] simplified form of &#x6200;
U+604C.0	fUTF8	恌
U+604C.0	fCNS11643	2-2A4A
U+604C.0	fBig5	CEC9
U+604C.0	fGB	3-5161
U+604C.0	fR/S	 61.6
U+604C.0	fTotalStrokes	 9
U+604C.0	fCangjie	PLMO
U+604C.0	fFourCorner	92013
U+604C.0	fHakka	 tiau1
U+604C.0	fCantonese	tiu1
U+604C.0	fMandarin	tiao1; yao2
U+604C.0	fEnglish	flippant; playful; sportive; frivolous
U+604D.0	fUTF8	恍
U+604D.0	fCNS11643	1-512D
U+604D.0	fBig5	ABE9
U+604D.0	fGB	0-3B50
U+604D.0	fR/S	 61.6
U+604D.0	fTotalStrokes	 9
U+604D.0	fCangjie	PFMU
U+604D.0	fFourCorner	99011
U+604D.0	fHakka	 fong3
U+604D.0	fCantonese	fong2
U+604D.0	fMandarin	huang3; guang1
U+604D.0	fEnglish	[1] seemingly; seem; as if [2] absent-minded; unconscious [3] all of a sudden; suddenly
U+604E.0	fUTF8	恎
U+604E.0	fCNS11643	3-2C56
U+604E.0	fGB	3-515C
U+604E.0	fR/S	  61.6
U+604E.0	fTotalStrokes	 9
U+604E.0	fCangjie	PMIG
U+604E.0	fFourCorner	91010
U+604E.0	fCantonese	dit6
U+604E.0	fMandarin	die2
U+604F.0	fUTF8	恏
U+604F.0	fCNS11643	3-3073
U+604F.0	fGB	E-297C
U+604F.0	fR/S	  61.6
U+604F.0	fTotalStrokes	 10
U+604F.0	fCangjie	VDP
U+604F.0	fFourCorner	47330
U+604F.0	fCantonese	hou2
U+604F.0	fMandarin	hao3
U+6050.0	fUTF8	恐
U+6050.0	fCNS11643	1-5545
U+6050.0	fBig5	AEA3
U+6050.0	fGB	0-3F56
U+6050.0	fR/S	 61.6
U+6050.0	fTotalStrokes	 10
U+6050.0	fCangjie	MNP
U+6050.0	fFourCorner	17331
U+6050.0	fHakka	  kiung3; kung3
U+6050.0	fCantonese	hung2
U+6050.0	fMandarin	kong3; kong4
U+6050.0	fEnglish	[1] fear; dread [2] I am afraid ...
U+6051.0	fUTF8	恑
U+6051.0	fCNS11643	3-2C57
U+6051.0	fGB	5-536D
U+6051.0	fR/S	  61.6
U+6051.0	fTotalStrokes	 9
U+6051.0	fCangjie	PNMU
U+6051.0	fFourCorner	97010
U+6051.0	fHakka	 gui3; gwi3; kwui3
U+6051.0	fCantonese	gwai2
U+6051.0	fMandarin	gui3
U+6051.0	fEnglish	 change alter accommodate oneself to
U+6052.0	fUTF8	恒
U+6052.0	fCNS11643	3-2C5D
U+6052.0	fGB	0-3A63
U+6052.0	fR/S	  61.6
U+6052.0	fTotalStrokes	 9
U+6052.0	fCangjie	PMAM
U+6052.0	fFourCorner	91016
U+6052.0	fHakka	  hen2; hen3
U+6052.0	fCantonese	hang4; hang2 (also); gang2 (also)
U+6052.0	fMandarin	heng1; heng2; geng4; geng1
U+6052.0	fEnglish	constant, regular, persistent
U+6053.0	fUTF8	恓
U+6053.0	fCNS11643	2-2A46
U+6053.0	fBig5	CEC5
U+6053.0	fGB	3-5158
U+6053.0	fR/S	 61.6
U+6053.0	fTotalStrokes	 9
U+6053.0	fCangjie	PMCW
U+6053.0	fFourCorner	91060
U+6053.0	fHakka	  si1; zi1
U+6053.0	fCantonese	sai1
U+6053.0	fMandarin	xi1
U+6053.0	fEnglish	frightened and worried
U+6054.0	fUTF8	恔
U+6054.0	fCNS11643	2-2A42
U+6054.0	fBig5	CEC1
U+6054.0	fGB	3-5165
U+6054.0	fR/S	 61.6
U+6054.0	fTotalStrokes	 9
U+6054.0	fCangjie	PYCK
U+6054.0	fFourCorner	90048
U+6054.0	fHakka	 gau3; hau5
U+6054.0	fCantonese	haau6; hau6
U+6054.0	fMandarin	xiao4; jiao3; xiao2
U+6054.0	fEnglish	 cheerful bright; sagacious
U+6055.0	fUTF8	恕
U+6055.0	fCNS11643	1-5546
U+6055.0	fBig5	AEA4
U+6055.0	fGB	0-4B21
U+6055.0	fR/S	 61.6
U+6055.0	fTotalStrokes	 10
U+6055.0	fCangjie	VRP
U+6055.0	fFourCorner	46330
U+6055.0	fHakka	  su5;  shu5; shu6
U+6055.0	fCantonese	syu3
U+6055.0	fMandarin	shu4
U+6055.0	fEnglish	[1] forgive; excuse [2] excuse me [3] benevolence
U+6056.0	fUTF8	恖
U+6056.0	fCNS11643	3-307E
U+6056.0	fGB	E-297D
U+6056.0	fR/S	 61.6
U+6056.0	fTotalStrokes	 10
U+6056.0	fCangjie	HWP
U+6056.0	fFourCorner	26330
U+6056.0	fHakka	  sii1;  su1; sii5; se1; se5
U+6056.0	fCantonese	si1; si3; soi1
U+6056.0	fMandarin	si1; sai1; si4 si5
U+6056.0	fEnglish	standard form of &#x601D;
U+6057.0	fUTF8	恗
U+6057.0	fCNS11643	4-293E
U+6057.0	fGB	3-5159
U+6057.0	fR/S	  61.6
U+6057.0	fTotalStrokes	 9
U+6057.0	fCangjie	PKMS
U+6057.0	fFourCorner	94020
U+6057.0	fHakka	 kwa1; fu1; k'ua1; kwa5
U+6057.0	fCantonese	fu1
U+6057.0	fMandarin	hu1; kua3
U+6057.0	fEnglish	self-satisfied; compacent
U+6058.0	fUTF8	恘
U+6058.0	fCNS11643	2-2A50
U+6058.0	fBig5	CECF
U+6058.0	fGB	E-297E
U+6058.0	fR/S	 61.6
U+6058.0	fTotalStrokes	 9
U+6058.0	fCangjie	POD
U+6058.0	fFourCorner	94090
U+6058.0	fCantonese	jau1; cau4
U+6058.0	fMandarin	qiu1
U+6059.0	fUTF8	恙
U+6059.0	fCNS11643	1-5542
U+6059.0	fBig5	AE7E
U+6059.0	fGB	0-6D26
U+6059.0	fR/S	 61.6
U+6059.0	fTotalStrokes	 10
U+6059.0	fCangjie	TGP
U+6059.0	fFourCorner	80331
U+6059.0	fHakka	 jong5; rong6; ngiong5
U+6059.0	fCantonese	joeng6
U+6059.0	fMandarin	yang4
U+6059.0	fEnglish	[1] illness; sickness [2] worry
U+605A.0	fUTF8	恚
U+605A.0	fCNS11643	2-2F21
U+605A.0	fBig5	D17D
U+605A.0	fGB	0-6D23
U+605A.0	fR/S	 61.6
U+605A.0	fTotalStrokes	 10
U+605A.0	fCangjie	GGP
U+605A.0	fFourCorner	40331
U+605A.0	fHakka	 fui6;  fui5; fi5
U+605A.0	fCantonese	wai6; waai6; wai3
U+605A.0	fMandarin	hui4
U+605A.0	fEnglish	 anger; rage
U+605B.0	fUTF8	恛
U+605B.0	fCNS11643	2-2A49
U+605B.0	fBig5	CEC8
U+605B.0	fGB	3-515E
U+605B.0	fR/S	 61.6
U+605B.0	fTotalStrokes	 9
U+605B.0	fCangjie	PWR
U+605B.0	fFourCorner	96000
U+605B.0	fHakka	  fui2; fi2
U+605B.0	fCantonese	wui4
U+605B.0	fMandarin	hui2
U+605B.0	fEnglish	 disordered indistinct doubtful blurred
U+605C.0	fUTF8	恜
U+605C.0	fCNS11643	3-2C54
U+605C.0	fGB	3-5154
U+605C.0	fR/S	 61.6
U+605C.0	fTotalStrokes	 9; 10
U+605C.0	fCangjie	PIPM
U+605C.0	fFourCorner	93040
U+605C.0	fHakka	 chit7; cit7
U+605C.0	fCantonese	cik1; sik1 (also)
U+605C.0	fMandarin	chi4
U+605C.0	fEnglish	 fear; venerate
U+605D.0	fUTF8	恝
U+605D.0	fCNS11643	2-2E7E
U+605D.0	fBig5	D17C
U+605D.0	fGB	0-6D22
U+605D.0	fR/S	 61.6
U+605D.0	fTotalStrokes	 10
U+605D.0	fCangjie	QHP
U+605D.0	fFourCorner	57332
U+605D.0	fHakka	  kat7; gat7; gai5
U+605D.0	fCantonese	aat3 (also); gaat3; ngaat3 (also); git3; gaat1
U+605D.0	fMandarin	jia2
U+605D.0	fEnglish	carefree; indifferent; unworried; unconcerned
U+605E.0	fUTF8	恞
U+605E.0	fCNS11643	2-2A44
U+605E.0	fBig5	CEC3
U+605E.0	fGB	5-5368
U+605E.0	fR/S	 61.6
U+605E.0	fTotalStrokes	 9
U+605E.0	fCangjie	PKN
U+605E.0	fFourCorner	95032
U+605E.0	fHakka	 ji2; ri2; i2
U+605E.0	fCantonese	ji4
U+605E.0	fMandarin	yi2
U+605E.0	fEnglish	pleased; gratified
U+605F.0	fUTF8	恟
U+605F.0	fCNS11643	2-2A4D
U+605F.0	fBig5	CECC
U+605F.0	fGB	5-536F
U+605F.0	fR/S	 61.6
U+605F.0	fTotalStrokes	 9
U+605F.0	fCangjie	PPUK
U+605F.0	fFourCorner	97020
U+605F.0	fHakka	  hiung1
U+605F.0	fCantonese	hung1
U+605F.0	fMandarin	xiong1; xiong3
U+605F.0	fEnglish	[1] afraid; frightened [2] noisy
U+6060.0	fUTF8	恠
U+6060.0	fCNS11643	3-2C5F
U+6060.0	fGB	E-2A21
U+6060.0	fR/S	  61.6
U+6060.0	fTotalStrokes	 9
U+6060.0	fCangjie	PKLG
U+6060.0	fFourCorner	94010
U+6060.0	fHakka	 gwai5; gwai3
U+6060.0	fCantonese	gwaai3
U+6060.0	fMandarin	guai4
U+6060.0	fEnglish	strange, unusual, peculiar
U+6061.0	fUTF8	恡
U+6061.0	fCNS11643	3-2C61
U+6061.0	fGB	E-2A22
U+6061.0	fR/S	  61.6
U+6061.0	fTotalStrokes	 9
U+6061.0	fCangjie	PKKI
U+6061.0	fFourCorner	94030
U+6061.0	fHakka	 nin5; nin6
U+6061.0	fCantonese	leon6
U+6061.0	fMandarin	lin4
U+6061.0	fEnglish	 stingy; sparing of close-fisted
U+6062.0	fUTF8	恢
U+6062.0	fCNS11643	1-5130
U+6062.0	fBig5	ABEC
U+6062.0	fGB	0-3B56
U+6062.0	fR/S	 61.6
U+6062.0	fTotalStrokes	 9
U+6062.0	fCangjie	PKF
U+6062.0	fFourCorner	94089
U+6062.0	fHakka	 foi1; kui1; fi1; foe1; k'wui1
U+6062.0	fCantonese	fui1
U+6062.0	fMandarin	hui1; kui3
U+6062.0	fEnglish	[1] restore; recover; regain [2] big; great; immense; vast; extensive; enormous
U+6063.0	fUTF8	恣
U+6063.0	fCNS11643	1-5543
U+6063.0	fBig5	AEA1
U+6063.0	fGB	0-6D27
U+6063.0	fR/S	 61.6
U+6063.0	fTotalStrokes	 10
U+6063.0	fCangjie	IOP
U+6063.0	fFourCorner	37338
U+6063.0	fHakka	 zih1; zu1; zii5; zih3
U+6063.0	fCantonese	zi3; ci3 (also); zi1 (also); ci1
U+6063.0	fMandarin	zi4; zi1
U+6063.0	fEnglish	 throw off restraint; dissipate; debauch
U+6063.1	fUTF8	恣
U+6063.1	fCNS11643	1-5543
U+6063.1	fBig5	AEA1
U+6063.1	fGB	0-6D27
U+6063.1	fR/S	 61.6
U+6063.1	fTotalStrokes	 10
U+6063.1	fCangjie	IOP
U+6063.1	fFourCorner	37338
U+6063.1	fMandarin	ci1
U+6063.1	fEnglish	used in combination
U+6064.0	fUTF8	恤
U+6064.0	fCNS11643	1-5136
U+6064.0	fBig5	ABF2
U+6064.0	fGB	0-5074
U+6064.0	fR/S	 61.6
U+6064.0	fTotalStrokes	 9
U+6064.0	fCangjie	PHBT
U+6064.0	fFourCorner	97010
U+6064.0	fHakka	  sut7; sit7; sut8; sot8
U+6064.0	fCantonese	seot1
U+6064.0	fMandarin	xu4
U+6064.0	fEnglish	[1] relieve; help [2] sympathise;
U+6065.0	fUTF8	恥
U+6065.0	fCNS11643	1-5544
U+6065.0	fBig5	AEA2
U+6065.0	fGB	E-2A23
U+6065.0	fR/S	 61.6
U+6065.0	fTotalStrokes	 10
U+6065.0	fCangjie	SJP
U+6065.0	fFourCorner	13100
U+6065.0	fHakka	 ci3;  chi3; cih3; chii3
U+6065.0	fCantonese	ci2
U+6065.0	fMandarin	chi3
U+6065.0	fEnglish	shame; humiliation; disgrace; feel ashamed
U+6066.0	fUTF8	恦
U+6066.0	fCNS11643	2-2A51
U+6066.0	fBig5	CED0
U+6066.0	fGB	E-2A24
U+6066.0	fR/S	 61.6
U+6066.0	fTotalStrokes	 9
U+6066.0	fCangjie	PHBR
U+6066.0	fFourCorner	97020
U+6066.0	fCantonese	soeng3; hoeng2
U+6066.0	fMandarin	xiang4
U+6067.0	fUTF8	恧
U+6067.0	fCNS11643	2-2F22
U+6067.0	fBig5	D17E
U+6067.0	fGB	0-6D24
U+6067.0	fR/S	 61.6
U+6067.0	fTotalStrokes	 10
U+6067.0	fCangjie	MBP
U+6067.0	fFourCorner	10332
U+6067.0	fHakka	  nuk7; nut7
U+6067.0	fCantonese	nuk6; luk6 (also); nuk1
U+6067.0	fMandarin	nuu4; n4
U+6067.0	fEnglish	ashamed
U+6068.0	fUTF8	恨
U+6068.0	fCNS11643	1-512F
U+6068.0	fBig5	ABEB
U+6068.0	fGB	0-3A5E
U+6068.0	fR/S	 61.6
U+6068.0	fTotalStrokes	 9
U+6068.0	fCangjie	PAV
U+6068.0	fFourCorner	97032
U+6068.0	fHakka	 han5;  hen5; hen6; han3
U+6068.0	fCantonese	han6
U+6068.0	fMandarin	hen4
U+6068.0	fEnglish	[1] hatred; hate; dislike; resent; hate [2] regret
U+6069.0	fUTF8	恩
U+6069.0	fCNS11643	1-5548
U+6069.0	fBig5	AEA6
U+6069.0	fGB	0-3677
U+6069.0	fR/S	 61.6
U+6069.0	fTotalStrokes	 10
U+6069.0	fCangjie	WKP
U+6069.0	fFourCorner	60330
U+6069.0	fHakka	  en1
U+6069.0	fCantonese	jan1
U+6069.0	fMandarin	en1
U+6069.0	fEnglish	kindness; mercy; charity; favor; grace; gratitude; kindness
U+606A.0	fUTF8	恪
U+606A.0	fCNS11643	1-5135
U+606A.0	fBig5	ABF1
U+606A.0	fGB	0-6321
U+606A.0	fR/S	 61.6
U+606A.0	fTotalStrokes	 9
U+606A.0	fCangjie	PHER
U+606A.0	fFourCorner	97064
U+606A.0	fHakka	  kok7; gok7
U+606A.0	fCantonese	kok3
U+606A.0	fMandarin	ke4; que4
U+606A.0	fEnglish	respectful; reverent; respect; respectfully
U+606B.0	fUTF8	恫
U+606B.0	fCNS11643	1-5134
U+606B.0	fBig5	ABF0
U+606B.0	fGB	0-3632
U+606B.0	fR/S	 61.6
U+606B.0	fTotalStrokes	 9
U+606B.0	fCangjie	PBMR
U+606B.0	fFourCorner	97020
U+606B.0	fHakka	 tung2; tung3
U+606B.0	fCantonese	dung6; tung1
U+606B.0	fMandarin	tong1
U+606B.0	fEnglish	pain
U+606B.1	fUTF8	恫
U+606B.1	fCNS11643	1-5134
U+606B.1	fBig5	ABF0
U+606B.1	fGB	0-3632
U+606B.1	fR/S	 61.6
U+606B.1	fTotalStrokes	 9
U+606B.1	fCangjie	PBMR
U+606B.1	fFourCorner	97020
U+606B.1	fMandarin	dong4
U+606B.1	fEnglish	 threaten; intimidate; scare
U+606C.0	fUTF8	恬
U+606C.0	fCNS11643	1-5133
U+606C.0	fBig5	ABEF
U+606C.0	fGB	0-4C71
U+606C.0	fR/S	 61.6
U+606C.0	fTotalStrokes	 9
U+606C.0	fCangjie	PHJR
U+606C.0	fFourCorner	92064
U+606C.0	fHakka	 tiam2; diam1
U+606C.0	fCantonese	tim4; tim5 (also)
U+606C.0	fMandarin	tian2
U+606C.0	fEnglish	quiet; calm; tranquil; peaceful
U+606D.0	fUTF8	恭
U+606D.0	fCNS11643	1-5547
U+606D.0	fBig5	AEA5
U+606D.0	fGB	0-3927
U+606D.0	fR/S	 61.6
U+606D.0	fTotalStrokes	 10
U+606D.0	fCangjie	TCP
U+606D.0	fFourCorner	44338
U+606D.0	fHakka	  giung1
U+606D.0	fCantonese	gung1
U+606D.0	fMandarin	gong1
U+606D.0	fEnglish	respectful; reverent; deferential
U+606E.0	fUTF8	恮
U+606E.0	fCNS11643	2-2A52
U+606E.0	fBig5	CED1
U+606E.0	fGB	3-515F
U+606E.0	fR/S	 61.6
U+606E.0	fTotalStrokes	 9
U+606E.0	fCangjie	POMG
U+606E.0	fFourCorner	98014
U+606E.0	fCantonese	cyun4
U+606E.0	fMandarin	quan2; quan1
U+606F.0	fUTF8	息
U+606F.0	fCNS11643	1-5549
U+606F.0	fBig5	AEA7
U+606F.0	fGB	0-4F22
U+606F.0	fR/S	 61.6
U+606F.0	fTotalStrokes	 10
U+606F.0	fCangjie	HUP
U+606F.0	fFourCorner	26330
U+606F.0	fHakka	  sit7
U+606F.0	fCantonese	sik1; sak1
U+606F.0	fMandarin	xi1
U+606F.0	fEnglish	rest; put stop to; end; cease
U+606F.1	fUTF8	息
U+606F.1	fCNS11643	1-5549
U+606F.1	fBig5	AEA7
U+606F.1	fGB	0-4F22
U+606F.1	fR/S	 61.6
U+606F.1	fTotalStrokes	 10
U+606F.1	fCangjie	HUP
U+606F.1	fFourCorner	26330
U+606F.1	fMandarin	xi5
U+6070.0	fUTF8	恰
U+6070.0	fCNS11643	1-512E
U+6070.0	fBig5	ABEA
U+6070.0	fGB	0-4721
U+6070.0	fR/S	 61.6
U+6070.0	fTotalStrokes	 9
U+6070.0	fCangjie	POMR
U+6070.0	fFourCorner	98061
U+6070.0	fHakka	  kap7; gip7
U+6070.0	fCantonese	hap1; aam1
U+6070.0	fMandarin	qia4
U+6070.0	fEnglish	appropriate; proper; suitable
U+6071.0	fUTF8	恱
U+6071.0	fCNS11643	4-294B
U+6071.0	fGB	E-2A25
U+6071.0	fR/S	 61.6
U+6071.0	fTotalStrokes	 9
U+6071.0	fCangjie	PCIU
U+6071.0	fFourCorner	98010
U+6071.0	fHakka	 jet8
U+6071.0	fCantonese	jyut6
U+6071.0	fMandarin	yue4
U+6072.0	fUTF8	恲
U+6072.0	fCNS11643	2-2A43
U+6072.0	fBig5	CEC2
U+6072.0	fGB	3-5168
U+6072.0	fR/S	 61.6
U+6072.0	fTotalStrokes	 9
U+6072.0	fCangjie	PTT
U+6072.0	fFourCorner	98041
U+6072.0	fCantonese	paang1
U+6072.0	fMandarin	peng1
U+6073.0	fUTF8	恳
U+6073.0	fCNS11643	3-307A
U+6073.0	fGB	0-3F52
U+6073.0	fR/S	 61.6
U+6073.0	fTotalStrokes	 10
U+6073.0	fCangjie	AVP
U+6073.0	fFourCorner	77330
U+6073.0	fHakka	 ken3
U+6073.0	fCantonese	han2
U+6073.0	fMandarin	ken3
U+6073.0	fEnglish	simplified form of &#x61C7;
U+6074.0	fUTF8	恴
U+6074.0	fCNS11643	3-3078
U+6074.0	fGB	E-2A26
U+6074.0	fR/S	  61.6
U+6074.0	fTotalStrokes	 10
U+6074.0	fCangjie	YAP
U+6074.0	fFourCorner	00330
U+6074.0	fHakka	  det7; diet7
U+6074.0	fCantonese	dak1
U+6074.0	fMandarin	de2
U+6074.0	fEnglish	same as &#x5FB7;
U+6075.0	fUTF8	恵
U+6075.0	fCNS11643	3-307B
U+6075.0	fGB	E-2A27
U+6075.0	fR/S	 61.6
U+6075.0	fTotalStrokes	 10
U+6075.0	fCangjie	JWP
U+6075.0	fFourCorner	50330
U+6075.0	fHakka	 fui5
U+6075.0	fCantonese	wai6
U+6075.0	fMandarin	hui4
U+6075.0	fEnglish	simplified form of &#x60E0;
U+6076.0	fUTF8	恶
U+6076.0	fGB	0-3671
U+6076.0	fR/S	 61.6
U+6076.0	fTotalStrokes	 10
U+6076.0	fCangjie	MCP
U+6076.0	fHakka	 ok7
U+6076.0	fCantonese	ok3; ngok3 (also); wu3; wu1
U+6076.0	fMandarin	e4
U+6076.0	fEnglish	simplified form of &#x60E1;
U+6076.1	fUTF8	恶
U+6076.1	fGB	0-3671
U+6076.1	fR/S	 61.6
U+6076.1	fTotalStrokes	 10
U+6076.1	fCangjie	MCP
U+6076.1	fHakka	vu5
U+6076.1	fMandarin	wu1; wu4
U+6077.0	fUTF8	恷
U+6077.0	fGB	E-2A28
U+6077.0	fR/S	 61.6
U+6077.0	fTotalStrokes	 10
U+6077.0	fCangjie	ODP
U+6077.0	fFourCorner	2433
U+6077.0	fCantonese	jau1
U+6078.0	fUTF8	恸
U+6078.0	fGB	0-627A
U+6078.0	fR/S	 61.6
U+6078.0	fTotalStrokes	 9
U+6078.0	fCangjie	PMIS
U+6078.0	fHakka	 tung5
U+6078.0	fCantonese	dung6
U+6078.0	fMandarin	tong4; dong4
U+6078.0	fEnglish	simplified form of &#x615F;
U+6079.0	fUTF8	恹
U+6079.0	fGB	0-627B
U+6079.0	fR/S	 61.6
U+6079.0	fTotalStrokes	 9
U+6079.0	fCangjie	PMIK
U+6079.0	fCantonese	jim1
U+6079.0	fMandarin	yan4; chu2; yan1; ye4
U+6079.0	fEnglish	simplified form of &#x61E8;
U+607A.0	fUTF8	恺
U+607A.0	fGB	0-627D
U+607A.0	fR/S	 61.6
U+607A.0	fTotalStrokes	 9
U+607A.0	fCangjie	PUSU
U+607A.0	fHakka	 koi3; kai3
U+607A.0	fCantonese	hoi2
U+607A.0	fMandarin	kai3
U+607A.0	fEnglish	simplified form of &#x6137;
U+607B.0	fUTF8	恻
U+607B.0	fGB	0-627C
U+607B.0	fR/S	 61.6
U+607B.0	fTotalStrokes	 9
U+607B.0	fCangjie	PBON
U+607B.0	fHakka	 cet7
U+607B.0	fCantonese	cak1; caak1 (also)
U+607B.0	fMandarin	ce4
U+607B.0	fEnglish	simplified form of &#x60FB;
U+607C.0	fUTF8	恼
U+607C.0	fCNS11643	3-2C5C
U+607C.0	fGB	0-4455
U+607C.0	fR/S	 61.6
U+607C.0	fTotalStrokes	 9
U+607C.0	fCangjie	PYUK
U+607C.0	fFourCorner	90070
U+607C.0	fHakka	 nau3
U+607C.0	fCantonese	nou5; lou5 (also)
U+607C.0	fMandarin	nao3
U+607C.0	fEnglish	simplified form of &#x60F1;
U+607D.0	fUTF8	恽
U+607D.0	fGB	0-6322
U+607D.0	fR/S	 61.6
U+607D.0	fTotalStrokes	 9
U+607D.0	fCangjie	PBKQ
U+607D.0	fHakka	 vun3
U+607D.0	fCantonese	wan6
U+607D.0	fMandarin	yun4; jun3; yun3
U+607D.0	fEnglish	simplified form of &#x60F2;
U+607E.0	fUTF8	恾
U+607E.0	fCNS11643	3-3076
U+607E.0	fGB	3-5156
U+607E.0	fR/S	 61.7
U+607E.0	fTotalStrokes	 10
U+607E.0	fCangjie	PTYV
U+607E.0	fFourCorner	94010
U+607E.0	fHakka	  mong2
U+607E.0	fCantonese	mong4
U+607E.0	fMandarin	mang2
U+607E.0	fEnglish	hurried; flurried
U+607F.0	fUTF8	恿
U+607F.0	fCNS11643	1-5A3C
U+607F.0	fBig5	B176
U+607F.0	fGB	0-5341
U+607F.0	fR/S	 61.7
U+607F.0	fTotalStrokes	 11
U+607F.0	fCangjie	NBP
U+607F.0	fFourCorner	17332
U+607F.0	fHakka	  jung3; rung3
U+607F.0	fCantonese	jung2; jung5
U+607F.0	fMandarin	yong3
U+607F.0	fEnglish	 instigate; incite alarm
U+6080.0	fUTF8	悀
U+6080.0	fCNS11643	2-2F26
U+6080.0	fBig5	D1A4
U+6080.0	fGB	3-5222
U+6080.0	fR/S	 61.7
U+6080.0	fTotalStrokes	 10
U+6080.0	fCangjie	PNIB
U+6080.0	fFourCorner	97027
U+6080.0	fCantonese	jung5
U+6080.0	fMandarin	yong3
U+6081.0	fUTF8	悁
U+6081.0	fCNS11643	2-2F28
U+6081.0	fBig5	D1A6
U+6081.0	fGB	3-5175
U+6081.0	fR/S	 61.7
U+6081.0	fTotalStrokes	 10
U+6081.0	fCangjie	PRB
U+6081.0	fFourCorner	96027
U+6081.0	fHakka	 gan1; gien3; gen1; gian1
U+6081.0	fCantonese	gyun1 (also); gyun3; jyun1
U+6081.0	fMandarin	juan4; yuan1
U+6081.0	fEnglish	irritable; impatient
U+6081.1	fUTF8	悁
U+6081.1	fCNS11643	2-2F28
U+6081.1	fBig5	D1A6
U+6081.1	fGB	3-5175
U+6081.1	fR/S	 61.7
U+6081.1	fTotalStrokes	 10
U+6081.1	fCangjie	PRB
U+6081.1	fFourCorner	96027
U+6081.1	fMandarin	juan1
U+6081.1	fEnglish	indignant; angry
U+6082.0	fUTF8	悂
U+6082.0	fCNS11643	4-2D25
U+6082.0	fGB	3-5173
U+6082.0	fR/S	  61.7
U+6082.0	fTotalStrokes	 10
U+6082.0	fCangjie	PPPG
U+6082.0	fFourCorner	91010
U+6082.0	fCantonese	pei1; pai1 (also)
U+6082.0	fMandarin	pi1; bi1
U+6083.0	fUTF8	悃
U+6083.0	fCNS11643	2-2F2A
U+6083.0	fBig5	D1A8
U+6083.0	fGB	0-6327
U+6083.0	fR/S	 61.7
U+6083.0	fTotalStrokes	 10
U+6083.0	fCangjie	PWD
U+6083.0	fFourCorner	96000
U+6083.0	fHakka	  kun3; kwun3
U+6083.0	fCantonese	kwan2
U+6083.0	fMandarin	kun3
U+6083.0	fEnglish	sincere; genuine; honest
U+6084.0	fUTF8	悄
U+6084.0	fCNS11643	1-554A
U+6084.0	fBig5	AEA8
U+6084.0	fGB	0-4744
U+6084.0	fR/S	 61.7
U+6084.0	fTotalStrokes	 10
U+6084.0	fCangjie	PFB
U+6084.0	fFourCorner	99027
U+6084.0	fHakka	 ciau3; ciau1
U+6084.0	fCantonese	ciu2; ciu5 (also); ciu3 (also)
U+6084.0	fMandarin	qiao3; qiao1; qiao4
U+6084.0	fEnglish	silent; quiet
U+6085.0	fUTF8	悅
U+6085.0	fCNS11643	1-5550
U+6085.0	fBig5	AEAE
U+6085.0	fGB	E-2A29
U+6085.0	fR/S	 61.7
U+6085.0	fTotalStrokes	 10
U+6085.0	fCangjie	PCRU
U+6085.0	fFourCorner	98016
U+6085.0	fHakka	  jet8; jat8; zat8
U+6085.0	fCantonese	jyut6
U+6085.0	fMandarin	yue4
U+6085.0	fEnglish	 delight; gratify; please; contended; pleased; gratified; happy; happily; glad; gladly
U+6086.0	fUTF8	悆
U+6086.0	fCNS11643	2-3537
U+6086.0	fBig5	D553
U+6086.0	fGB	5-527D
U+6086.0	fR/S	 61.7
U+6086.0	fTotalStrokes	 11
U+6086.0	fCangjie	OMDP
U+6086.0	fFourCorner	80339
U+6086.0	fHakka	 i2
U+6086.0	fCantonese	jyu6
U+6086.0	fMandarin	yu4; shu1
U+6086.0	fEnglish	happy
U+6087.0	fUTF8	悇
U+6087.0	fCNS11643	2-2F2E
U+6087.0	fBig5	D1AC
U+6087.0	fGB	3-5178
U+6087.0	fR/S	 61.7
U+6087.0	fTotalStrokes	 10
U+6087.0	fCangjie	POMD
U+6087.0	fFourCorner	98094
U+6087.0	fHakka	 tu2; ri2; ji2
U+6087.0	fCantonese	tou4
U+6087.0	fMandarin	yu4; tu2; tu1
U+6087.0	fEnglish	anxious; careworn
U+6088.0	fUTF8	悈
U+6088.0	fCNS11643	2-2F25
U+6088.0	fBig5	D1A3
U+6088.0	fGB	3-516C
U+6088.0	fR/S	 61.7
U+6088.0	fTotalStrokes	 10
U+6088.0	fCangjie	PIT
U+6088.0	fFourCorner	93050
U+6088.0	fHakka	 gai5; kie5; kiai5; giai5
U+6088.0	fCantonese	gaai3; hak1
U+6088.0	fMandarin	jie4
U+6088.0	fEnglish	[1] enjoin [2] charge
U+6089.0	fUTF8	悉
U+6089.0	fCNS11643	1-5A3E
U+6089.0	fBig5	B178
U+6089.0	fGB	0-4F24
U+6089.0	fR/S	 61.7
U+6089.0	fTotalStrokes	 11
U+6089.0	fCangjie	HDP
U+6089.0	fFourCorner	20339
U+6089.0	fHakka	  sit7
U+6089.0	fCantonese	sik1
U+6089.0	fMandarin	xi1
U+6089.0	fEnglish	[1] know [2] all; whole; total; entire
U+608A.0	fUTF8	悊
U+608A.0	fCNS11643	2-3535
U+608A.0	fBig5	D551
U+608A.0	fGB	5-527B
U+608A.0	fR/S	 61.7
U+608A.0	fTotalStrokes	 11
U+608A.0	fCangjie	QLP
U+608A.0	fFourCorner	52332
U+608A.0	fHakka	 zhat7; zat7; zhet7
U+608A.0	fCantonese	zit3
U+608A.0	fMandarin	zhe2; qi1; zhe1
U+608A.0	fEnglish	wise. to know intuitively
U+608B.0	fUTF8	悋
U+608B.0	fCNS11643	3-3122
U+608B.0	fGB	5-5377
U+608B.0	fR/S	  61.7
U+608B.0	fTotalStrokes	 10
U+608B.0	fCangjie	PYKR
U+608B.0	fFourCorner	90060
U+608B.0	fHakka	  nin5; lin5
U+608B.0	fCantonese	leon6
U+608B.0	fMandarin	lin4
U+608B.0	fEnglish	 stingy; sparing of close-fisted
U+608C.0	fUTF8	悌
U+608C.0	fCNS11643	1-554F
U+608C.0	fBig5	AEAD
U+608C.0	fGB	0-6329
U+608C.0	fR/S	 61.7
U+608C.0	fTotalStrokes	 10
U+608C.0	fCangjie	PCNH
U+608C.0	fFourCorner	98027
U+608C.0	fHakka	  ti5; ti6; ti3
U+608C.0	fCantonese	dai6; tai5; tai6
U+608C.0	fMandarin	ti4; dai4; di4
U+608C.0	fEnglish	 show brotherly love; love and respect for one's elder brother
U+608D.0	fUTF8	悍
U+608D.0	fCNS11643	1-554D
U+608D.0	fBig5	AEAB
U+608D.0	fGB	0-3A37
U+608D.0	fR/S	 61.7
U+608D.0	fTotalStrokes	 10
U+608D.0	fCangjie	PAMJ
U+608D.0	fFourCorner	96041
U+608D.0	fHakka	 gon5;  hon5; hon3; hon6
U+608D.0	fCantonese	hon5; hon6 (also)
U+608D.0	fMandarin	han4; han3; huan3
U+608D.0	fEnglish	[1] violent; fierce; cruel [2] courageous; brave; audacious [3] stubborn
U+608E.0	fUTF8	悎
U+608E.0	fCNS11643	2-2F30
U+608E.0	fBig5	D1AE
U+608E.0	fGB	3-5177
U+608E.0	fR/S	 61.7
U+608E.0	fTotalStrokes	 10
U+608E.0	fCangjie	PHGR
U+608E.0	fFourCorner	94061
U+608E.0	fHakka	 hau3; hau5
U+608E.0	fCantonese	hou6; gaau3
U+608E.0	fMandarin	hao4; jiao4
U+608E.0	fEnglish	fear; anxiety
U+608F.0	fUTF8	悏
U+608F.0	fCNS11643	4-2D2D
U+608F.0	fGB	3-515B
U+608F.0	fR/S	 61.7
U+608F.0	fTotalStrokes	 10
U+608F.0	fCangjie	PKOO
U+608F.0	fFourCorner	94030
U+608F.0	fCantonese	hip3; hap6
U+608F.0	fMandarin	qie4; qu4
U+608F.0	fEnglish	same as &#x3932;
U+6090.0	fUTF8	悐
U+6090.0	fCNS11643	2-3536
U+6090.0	fBig5	D552
U+6090.0	fGB	5-527E
U+6090.0	fR/S	 61.7
U+6090.0	fTotalStrokes	 11
U+6090.0	fCangjie	KFP
U+6090.0	fFourCorner	49338
U+6090.0	fHakka	 tit8
U+6090.0	fCantonese	tik1
U+6090.0	fMandarin	ti4
U+6090.0	fEnglish	 respect; regard stand in awe of; be alarmed
U+6091.0	fUTF8	悑
U+6091.0	fCNS11643	4-2D2C
U+6091.0	fGB	E-2A2A
U+6091.0	fR/S	 61.7
U+6091.0	fTotalStrokes	 10
U+6091.0	fCangjie	PIJB
U+6091.0	fFourCorner	93020
U+6091.0	fHakka	 bu1
U+6091.0	fCantonese	bou3
U+6091.0	fMandarin	bu4
U+6092.0	fUTF8	悒
U+6092.0	fCNS11643	2-2F27
U+6092.0	fBig5	D1A5
U+6092.0	fGB	0-6328
U+6092.0	fR/S	 61.7
U+6092.0	fTotalStrokes	 10
U+6092.0	fCangjie	PRAU
U+6092.0	fFourCorner	96017
U+6092.0	fHakka	  jip7; rip7
U+6092.0	fCantonese	jap1
U+6092.0	fMandarin	yi4
U+6092.0	fEnglish	sorrowful; depressed; unhappy
U+6093.0	fUTF8	悓
U+6093.0	fCNS11643	3-3125
U+6093.0	fGB	E-2A2B
U+6093.0	fR/S	 61.7
U+6093.0	fTotalStrokes	 10
U+6093.0	fCangjie	PBUU
U+6093.0	fFourCorner	96010
U+6093.0	fHakka	 hien5
U+6093.0	fCantonese	jin5; jin6
U+6093.0	fMandarin	xian4; qian4
U+6093.0	fEnglish	same as &#x4FD4;
U+6094.0	fUTF8	悔
U+6094.0	fCNS11643	1-554E
U+6094.0	fBig5	AEAC
U+6094.0	fGB	0-3B5A
U+6094.0	fR/S	 61.7; 61.6
U+6094.0	fTotalStrokes	 10
U+6094.0	fCangjie	POWY
U+6094.0	fFourCorner	98055
U+6094.0	fHakka	 fui5; fi5; fui6; fui3; fi3
U+6094.0	fCantonese	fui3
U+6094.0	fMandarin	hui3; hui4; jia3
U+6094.0	fEnglish	 repent; remorse; regret
U+6095.0	fUTF8	悕
U+6095.0	fCNS11643	2-2F2B
U+6095.0	fBig5	D1A9
U+6095.0	fGB	3-5179
U+6095.0	fR/S	 61.7
U+6095.0	fTotalStrokes	 10
U+6095.0	fCangjie	PKKB
U+6095.0	fFourCorner	94027
U+6095.0	fHakka	  hi1
U+6095.0	fCantonese	hei1
U+6095.0	fMandarin	xi1
U+6095.0	fEnglish	[1] consider [2] reflect
U+6096.0	fUTF8	悖
U+6096.0	fCNS11643	1-5551
U+6096.0	fBig5	AEAF
U+6096.0	fGB	0-6323
U+6096.0	fR/S	 61.7
U+6096.0	fTotalStrokes	 10
U+6096.0	fCangjie	PJBD
U+6096.0	fFourCorner	94047
U+6096.0	fHakka	 put8; poi5; poi6; put7
U+6096.0	fCantonese	bui6; bui3 (also); but6
U+6096.0	fMandarin	bei4; bo2
U+6096.0	fEnglish	 go against to; go counter to; revolt against; be contradictory to
U+6097.0	fUTF8	悗
U+6097.0	fCNS11643	2-2F2D
U+6097.0	fBig5	D1AB
U+6097.0	fGB	3-517C
U+6097.0	fR/S	 61.7
U+6097.0	fTotalStrokes	 10
U+6097.0	fCangjie	PNAU
U+6097.0	fFourCorner	97016
U+6097.0	fHakka	 man2;  mun3
U+6097.0	fCantonese	mun5; mun4; maan5
U+6097.0	fMandarin	man2; men3; men4
U+6097.0	fEnglish	[1] frightened [2] alarmed [3] suspicious
U+6098.0	fUTF8	悘
U+6098.0	fCNS11643	3-3568
U+6098.0	fGB	3-5076
U+6098.0	fR/S	  61.7
U+6098.0	fTotalStrokes	 11
U+6098.0	fCangjie	SKP
U+6098.0	fFourCorner	71331
U+6098.0	fCantonese	ji1
U+6098.0	fMandarin	yi1; yi2
U+6099.0	fUTF8	悙
U+6099.0	fCNS11643	3-3074
U+6099.0	fGB	3-517D
U+6099.0	fR/S	  61.7
U+6099.0	fTotalStrokes	 10
U+6099.0	fCangjie	PYRN
U+6099.0	fFourCorner	90020
U+6099.0	fCantonese	hang1
U+6099.0	fMandarin	heng1
U+609A.0	fUTF8	悚
U+609A.0	fCNS11643	1-554C
U+609A.0	fBig5	AEAA
U+609A.0	fGB	0-6324
U+609A.0	fR/S	 61.7
U+609A.0	fTotalStrokes	 10
U+609A.0	fCangjie	PDL
U+609A.0	fFourCorner	95096
U+609A.0	fHakka	 sung3; cet7
U+609A.0	fCantonese	sung2
U+609A.0	fMandarin	song3
U+609A.0	fEnglish	terrified; fearful; frightened
U+609B.0	fUTF8	悛
U+609B.0	fCNS11643	2-2F2C
U+609B.0	fBig5	D1AA
U+609B.0	fGB	0-632A
U+609B.0	fR/S	 61.7
U+609B.0	fTotalStrokes	 10
U+609B.0	fCangjie	PICE
U+609B.0	fFourCorner	93047
U+609B.0	fHakka	  zun1; cien2; cen2; zun5
U+609B.0	fCantonese	syun1; cyun1
U+609B.0	fMandarin	quan1; xun2
U+609B.0	fEnglish	 repent; reform
U+609C.0	fUTF8	悜
U+609C.0	fCNS11643	2-2F2F
U+609C.0	fBig5	D1AD
U+609C.0	fGB	3-5174
U+609C.0	fR/S	 61.7
U+609C.0	fTotalStrokes	 10
U+609C.0	fCangjie	PRHG
U+609C.0	fFourCorner	96014
U+609C.0	fHakka	 chin3; cihn3
U+609C.0	fCantonese	cing2
U+609C.0	fMandarin	cheng3
U+609C.0	fEnglish	obscure
U+609D.0	fUTF8	悝
U+609D.0	fCNS11643	2-2F29
U+609D.0	fBig5	D1A7
U+609D.0	fGB	0-6326
U+609D.0	fR/S	 61.7
U+609D.0	fTotalStrokes	 10
U+609D.0	fCangjie	PWG
U+609D.0	fFourCorner	96014
U+609D.0	fHakka	 foi1; kui1; k'wui1
U+609D.0	fCantonese	fui1; lei5; kwai1
U+609D.0	fMandarin	li3; hui1
U+609D.0	fEnglish	sad; grieved; worried
U+609D.1	fUTF8	悝
U+609D.1	fCNS11643	2-2F29
U+609D.1	fBig5	D1A7
U+609D.1	fGB	0-6326
U+609D.1	fR/S	 61.7
U+609D.1	fTotalStrokes	 10
U+609D.1	fCangjie	PWG
U+609D.1	fFourCorner	96014
U+609D.1	fMandarin	kui1
U+609D.1	fEnglish	 ridicule
U+609E.0	fUTF8	悞
U+609E.0	fCNS11643	3-3127
U+609E.0	fGB	E-2A2C
U+609E.0	fR/S	  61.7
U+609E.0	fTotalStrokes	 10
U+609E.0	fCangjie	PRVK
U+609E.0	fFourCorner	96030
U+609E.0	fHakka	 ngu5
U+609E.0	fCantonese	ng6
U+609E.0	fMandarin	wu4
U+609E.0	fEnglish	 impede neglect; delay
U+609F.0	fUTF8	悟
U+609F.0	fCNS11643	1-554B
U+609F.0	fBig5	AEA9
U+609F.0	fGB	0-4E72
U+609F.0	fR/S	 61.7
U+609F.0	fTotalStrokes	 10
U+609F.0	fCangjie	PMMR
U+609F.0	fFourCorner	91061
U+609F.0	fHakka	 ngu5; ngu3; ng3
U+609F.0	fCantonese	ng6; ng4 (also)
U+609F.0	fMandarin	wu4
U+609F.0	fEnglish	 comprehend; realise; become aware of; awake to
U+60A0.0	fUTF8	悠
U+60A0.0	fCNS11643	1-5A3F
U+60A0.0	fBig5	B179
U+60A0.0	fGB	0-5346
U+60A0.0	fR/S	 61.7
U+60A0.0	fTotalStrokes	 11
U+60A0.0	fCangjie	OKP
U+60A0.0	fFourCorner	28334
U+60A0.0	fHakka	  ju2; jiu2; riu2; jiu3
U+60A0.0	fCantonese	jau4
U+60A0.0	fMandarin	you1; you2
U+60A0.0	fEnglish	[1] long; far; remote; distant; vast; extensive [2] das; pensive; meditative [3] gentle; slow; soft [4] swing
U+60A1.0	fUTF8	悡
U+60A1.0	fCNS11643	3-3571
U+60A1.0	fGB	E-2A2D
U+60A1.0	fR/S	  61.7
U+60A1.0	fTotalStrokes	 11
U+60A1.0	fCangjie	HNP
U+60A1.0	fFourCorner	22330
U+60A1.0	fCantonese	lei4; laai4
U+60A1.0	fMandarin	li2; lai2
U+60A1.0	fEnglish	same as &#x394E;
U+60A2.0	fUTF8	悢
U+60A2.0	fCNS11643	2-2F24
U+60A2.0	fBig5	D1A2
U+60A2.0	fGB	3-5221
U+60A2.0	fR/S	 61.7
U+60A2.0	fTotalStrokes	 10
U+60A2.0	fCangjie	PIAV
U+60A2.0	fFourCorner	93032
U+60A2.0	fCantonese	loeng6; long5
U+60A2.0	fMandarin	liang4; lang3
U+60A3.0	fUTF8	患
U+60A3.0	fCNS11643	1-5A3D
U+60A3.0	fBig5	B177
U+60A3.0	fGB	0-3B3C
U+60A3.0	fR/S	 61.7
U+60A3.0	fTotalStrokes	 11
U+60A3.0	fCangjie	LLP
U+60A3.0	fFourCorner	50336
U+60A3.0	fHakka	 fam5; fam6; fan5; fam1
U+60A3.0	fCantonese	waan6
U+60A3.0	fMandarin	huan4
U+60A3.0	fEnglish	[1] trouble; worry [2] suffering; adversity; disaster; peril
U+60A4.0	fUTF8	悤
U+60A4.0	fCNS11643	3-3572
U+60A4.0	fGB	E-2A2E
U+60A4.0	fR/S	 61.7
U+60A4.0	fTotalStrokes	 11
U+60A4.0	fCangjie	HWP
U+60A4.0	fFourCorner	26330
U+60A4.0	fHakka	  cung1
U+60A4.0	fCantonese	cung1
U+60A4.0	fMandarin	cong1
U+60A4.0	fEnglish	agitated; excited; hurried
U+60A5.0	fUTF8	悥
U+60A5.0	fCNS11643	4-316B
U+60A5.0	fGB	E-2A2F
U+60A5.0	fR/S	 61.7
U+60A5.0	fTotalStrokes	 11
U+60A5.0	fCangjie	YMMP
U+60A5.0	fFourCorner	00330
U+60A5.0	fHakka	 ji5
U+60A5.0	fCantonese	ji3
U+60A5.0	fMandarin	yi4; yi1
U+60A6.0	fUTF8	悦
U+60A6.0	fCNS11643	3-3123
U+60A6.0	fGB	0-5443
U+60A6.0	fR/S	 61.7
U+60A6.0	fTotalStrokes	 10
U+60A6.0	fCangjie	PCRU
U+60A6.0	fFourCorner	98010
U+60A6.0	fHakka	 jet8
U+60A6.0	fCantonese	jyut6
U+60A6.0	fMandarin	yue4
U+60A6.0	fEnglish	same as &#x6085;
U+60A7.0	fUTF8	悧
U+60A7.0	fCNS11643	3-3128
U+60A7.0	fGB	5-5376
U+60A7.0	fR/S	 61.7
U+60A7.0	fTotalStrokes	 10
U+60A7.0	fCangjie	PHDN
U+60A7.0	fFourCorner	92000
U+60A7.0	fHakka	 li5
U+60A7.0	fCantonese	lei4; lei6
U+60A7.0	fMandarin	li4
U+60A7.0	fEnglish	same as &#x4FD0;
U+60A8.0	fUTF8	您
U+60A8.0	fCNS11643	1-5A40
U+60A8.0	fBig5	B17A
U+60A8.0	fGB	0-447A
U+60A8.0	fR/S	 61.7
U+60A8.0	fTotalStrokes	 11
U+60A8.0	fCangjie	OFP
U+60A8.0	fFourCorner	27339
U+60A8.0	fHakka	 nin2; li1; nen2; ni3
U+60A8.0	fCantonese	nei5; lei5 (also)
U+60A8.0	fMandarin	ni3; nin2
U+60A8.0	fEnglish	[1] non-classical form of &#x4F60; [2] honorific for "you" in Northern Chinese dialects
U+60A9.0	fUTF8	悩
U+60A9.0	fCNS11643	3-3121
U+60A9.0	fGB	E-2A30
U+60A9.0	fR/S	 61.7
U+60A9.0	fTotalStrokes	 10
U+60A9.0	fCangjie	PFUK
U+60A9.0	fFourCorner	93070
U+60A9.0	fHakka	 nau3
U+60A9.0	fCantonese	nou5; lou5 (also)
U+60A9.0	fMandarin	nao3
U+60A9.0	fEnglish	simplified form of &#x60F1;
U+60AA.0	fUTF8	悪
U+60AA.0	fCNS11643	3-356F
U+60AA.0	fGB	E-2A31
U+60AA.0	fR/S	 61.7
U+60AA.0	fTotalStrokes	 11
U+60AA.0	fCangjie	MMP
U+60AA.0	fFourCorner	10330
U+60AA.0	fHakka	 vu5; ok7
U+60AA.0	fCantonese	ok3; ngok3 (also); wu1; wu3
U+60AA.0	fMandarin	e4
U+60AA.0	fEnglish	evil, wicked, bad, foul
U+60AB.0	fUTF8	悫
U+60AB.0	fGB	0-6D28
U+60AB.0	fR/S	 61.7
U+60AB.0	fTotalStrokes	 11; 10
U+60AB.0	fCangjie	GBHP
U+60AB.0	fCantonese	kok3
U+60AB.0	fMandarin	que4
U+60AB.0	fEnglish	simplified form of &#x6164;
U+60AC.0	fUTF8	悬
U+60AC.0	fGB	0-507C
U+60AC.0	fR/S	 61.7
U+60AC.0	fTotalStrokes	 11
U+60AC.0	fCangjie	BIP
U+60AC.0	fHakka	 hien2
U+60AC.0	fCantonese	jyun4
U+60AC.0	fMandarin	xuan2
U+60AC.0	fEnglish	simplified form of &#x61F8;
U+60AD.0	fUTF8	悭
U+60AD.0	fGB	0-6325
U+60AD.0	fR/S	 61.7
U+60AD.0	fTotalStrokes	 10
U+60AD.0	fCangjie	PLEG
U+60AD.0	fHakka	 gen1; han1; gien1
U+60AD.0	fCantonese	haan1; han1
U+60AD.0	fMandarin	qian1
U+60AD.0	fEnglish	simplified form of &#x6173;
U+60AE.0	fUTF8	悮
U+60AE.0	fCNS11643	3-3126
U+60AE.0	fGB	8-2D5A
U+60AE.0	fR/S	 61.7
U+60AE.0	fTotalStrokes	 10
U+60AE.0	fCangjie	PRMK
U+60AE.0	fFourCorner	96030
U+60AE.0	fCantonese	ng6
U+60AE.0	fMandarin	wu4
U+60AE.0	fEnglish	impede; neglect, delay
U+60AF.0	fUTF8	悯
U+60AF.0	fGB	0-4375
U+60AF.0	fR/S	 61.7
U+60AF.0	fTotalStrokes	 10
U+60AF.0	fCangjie	PLSK
U+60AF.0	fHakka	 men1; men3; mien3; mien1
U+60AF.0	fCantonese	man5
U+60AF.0	fMandarin	min3
U+60AF.0	fEnglish	simplified form of &#x61AB;
U+60B0.0	fUTF8	悰
U+60B0.0	fCNS11643	2-3539
U+60B0.0	fBig5	D555
U+60B0.0	fGB	3-5238
U+60B0.0	fR/S	 61.8
U+60B0.0	fTotalStrokes	 11
U+60B0.0	fCangjie	PJMF
U+60B0.0	fFourCorner	93091
U+60B0.0	fHakka	  cung1
U+60B0.0	fCantonese	cung4; cung6; zung1
U+60B0.0	fMandarin	cong2
U+60B0.0	fEnglish	 enjoy; amuse; please joy
U+60B1.0	fUTF8	悱
U+60B1.0	fCNS11643	2-3542
U+60B1.0	fBig5	D55E
U+60B1.0	fGB	0-632D
U+60B1.0	fR/S	 61.8
U+60B1.0	fTotalStrokes	 11
U+60B1.0	fCangjie	PLMY
U+60B1.0	fFourCorner	91011
U+60B1.0	fHakka	 fui3; fi3; fui1; fi1
U+60B1.0	fCantonese	fei2
U+60B1.0	fMandarin	fei3
U+60B1.0	fEnglish	[1] inarticulate; unable to vent one's emotion; unable to explain one's self; be at a loss for words [2] sorrowful
U+60B2.0	fUTF8	悲
U+60B2.0	fCNS11643	1-5F2B
U+60B2.0	fBig5	B464
U+60B2.0	fGB	0-312F
U+60B2.0	fR/S	 61.8
U+60B2.0	fTotalStrokes	 12
U+60B2.0	fCangjie	LYP
U+60B2.0	fFourCorner	11331
U+60B2.0	fHakka	  bui1;  bi1
U+60B2.0	fCantonese	bei1
U+60B2.0	fMandarin	bei1
U+60B2.0	fEnglish	sorrowful; mournful; sad; woeful; rueful; doleful
U+60B3.0	fUTF8	悳
U+60B3.0	fCNS11643	3-3B54
U+60B3.0	fGB	E-2A32
U+60B3.0	fR/S	 61.8
U+60B3.0	fTotalStrokes	 12
U+60B3.0	fCangjie	JVP
U+60B3.0	fFourCorner	40330
U+60B3.0	fHakka	 det7
U+60B3.0	fCantonese	dak1
U+60B3.0	fMandarin	de2; duo2
U+60B3.0	fEnglish	ethics; morality; virtue
U+60B4.0	fUTF8	悴
U+60B4.0	fCNS11643	1-5A42
U+60B4.0	fBig5	B17C
U+60B4.0	fGB	0-6332
U+60B4.0	fR/S	 61.8
U+60B4.0	fTotalStrokes	 11
U+60B4.0	fCangjie	PYOJ
U+60B4.0	fFourCorner	90048
U+60B4.0	fHakka	 cui5; sui5
U+60B4.0	fCantonese	seoi6; seoi5 (also)
U+60B4.0	fMandarin	cui4; zu2
U+60B4.0	fEnglish	[1] haggard; worn-out; tired out [2] worried; sad
U+60B5.0	fUTF8	悵
U+60B5.0	fCNS11643	1-5A47
U+60B5.0	fBig5	B1A3
U+60B5.0	fGB	1-626A
U+60B5.0	fR/S	 61.8
U+60B5.0	fTotalStrokes	 11
U+60B5.0	fCangjie	PSMV
U+60B5.0	fFourCorner	91032
U+60B5.0	fHakka	 zhong5; cong3; zong5; chong5
U+60B5.0	fCantonese	coeng3; zoeng3 (also)
U+60B5.0	fMandarin	chang4
U+60B5.0	fEnglish	disappointed; dissatisfied; frustrated; sorry
U+60B6.0	fUTF8	悶
U+60B6.0	fCNS11643	1-5F2C
U+60B6.0	fBig5	B465
U+60B6.0	fGB	1-4346
U+60B6.0	fR/S	 61.8
U+60B6.0	fTotalStrokes	 12
U+60B6.0	fCangjie	ANP
U+60B6.0	fFourCorner	77337
U+60B6.0	fHakka	  mun3; mun5; mun6
U+60B6.0	fCantonese	mun6
U+60B6.0	fMandarin	men4; men2; men3
U+60B6.0	fEnglish	depressed; melancholy; bored; in low spirits
U+60B6.1	fUTF8	悶
U+60B6.1	fCNS11643	1-5F2C
U+60B6.1	fBig5	B465
U+60B6.1	fGB	1-4346
U+60B6.1	fR/S	 61.8
U+60B6.1	fTotalStrokes	 12
U+60B6.1	fCangjie	ANP
U+60B6.1	fFourCorner	77337
U+60B6.1	fMandarin	men1
U+60B6.1	fEnglish	[1] oppressive; suffocating (said of weather); stuffy [2] muffled (said of a sound) [3] shut oneself or somebody indoors [4] cover the tea pot for a while when one makes tea with boiling water
U+60B7.0	fUTF8	悷
U+60B7.0	fCNS11643	2-3544
U+60B7.0	fBig5	D560
U+60B7.0	fGB	3-523A
U+60B7.0	fR/S	 61.8
U+60B7.0	fTotalStrokes	 11
U+60B7.0	fCangjie	PHSK
U+60B7.0	fFourCorner	93034
U+60B7.0	fHakka	 lui3; li5; lui5; li6
U+60B7.0	fCantonese	lai6; leoi6 (also)
U+60B7.0	fMandarin	li4
U+60B7.0	fEnglish	[1] sad [2] distressed
U+60B8.0	fUTF8	悸
U+60B8.0	fCNS11643	1-5A4E
U+60B8.0	fBig5	B1AA
U+60B8.0	fGB	0-3C42
U+60B8.0	fR/S	 61.8
U+60B8.0	fTotalStrokes	 11
U+60B8.0	fCangjie	PHDD
U+60B8.0	fFourCorner	92047
U+60B8.0	fHakka	  gi5; git8; gui5; kui3; kwui5; gwi5
U+60B8.0	fCantonese	gwai3; gwai6
U+60B8.0	fMandarin	ji4
U+60B8.0	fEnglish	[1] palpitation of the heart [2] fear
U+60B9.0	fUTF8	悹
U+60B9.0	fCNS11643	2-3B5D
U+60B9.0	fBig5	D8F9
U+60B9.0	fGB	3-507C
U+60B9.0	fR/S	 61.8
U+60B9.0	fTotalStrokes	 12
U+60B9.0	fCangjie	JRRP
U+60B9.0	fFourCorner	00337
U+60B9.0	fHakka	 gon5; gon3; gwon3; gwon5
U+60B9.0	fCantonese	gun3; gwun3 (also); gun2
U+60B9.0	fMandarin	guan4; guan1
U+60BA.0	fUTF8	悺
U+60BA.0	fCNS11643	2-353A
U+60BA.0	fBig5	D556
U+60BA.0	fGB	5-5421
U+60BA.0	fR/S	 61.8
U+60BA.0	fTotalStrokes	 11
U+60BA.0	fCangjie	PJRR
U+60BA.0	fFourCorner	93077
U+60BA.0	fHakka	 gon5; gon3; gwon3; gwon5
U+60BA.0	fCantonese	gun3; gwun3 (also); gun2
U+60BA.0	fMandarin	guan4; guan1
U+60BA.0	fEnglish	same as &#x60B9;
U+60BB.0	fUTF8	悻
U+60BB.0	fCNS11643	1-5A46
U+60BB.0	fBig5	B1A2
U+60BB.0	fGB	0-632C
U+60BB.0	fR/S	 61.8
U+60BB.0	fTotalStrokes	 11
U+60BB.0	fCangjie	PGTJ
U+60BB.0	fFourCorner	94041
U+60BB.0	fHakka	 hen5; hen6; hen3
U+60BB.0	fCantonese	hang6; hang5 (also)
U+60BB.0	fMandarin	xing4
U+60BB.0	fEnglish	angry; indignant; enraged
U+60BC.0	fUTF8	悼
U+60BC.0	fCNS11643	1-5A49
U+60BC.0	fBig5	B1A5
U+60BC.0	fGB	0-353F
U+60BC.0	fR/S	 61.8
U+60BC.0	fTotalStrokes	 11
U+60BC.0	fCangjie	PYAJ
U+60BC.0	fFourCorner	91046
U+60BC.0	fHakka	  tau5; do6; do5; tau6
U+60BC.0	fCantonese	dou6
U+60BC.0	fMandarin	dao4
U+60BC.0	fEnglish	 mourn (for, over); lament; grieve; regret
U+60BD.0	fUTF8	悽
U+60BD.0	fCNS11643	1-5A44
U+60BD.0	fBig5	B17E
U+60BD.0	fGB	E-2A33
U+60BD.0	fR/S	 61.8
U+60BD.0	fTotalStrokes	 11
U+60BD.0	fCangjie	PJLV
U+60BD.0	fFourCorner	95047
U+60BD.0	fHakka	  ci1
U+60BD.0	fCantonese	cai1
U+60BD.0	fMandarin	qi1; qi4
U+60BD.0	fEnglish	[1] sorrowful; grieved; afflicted [2] tragic; pathetic; pitiful; grievous
U+60BE.0	fUTF8	悾
U+60BE.0	fCNS11643	2-3538
U+60BE.0	fBig5	D554
U+60BE.0	fGB	3-5239
U+60BE.0	fR/S	 61.8
U+60BE.0	fTotalStrokes	 11
U+60BE.0	fCangjie	PJCM
U+60BE.0	fFourCorner	93011
U+60BE.0	fHakka	  kung1
U+60BE.0	fCantonese	hung1
U+60BE.0	fMandarin	kong1; kong4; nong3; qiang1
U+60BE.0	fEnglish	sincere; candid
U+60BF.0	fUTF8	悿
U+60BF.0	fCNS11643	2-3546
U+60BF.0	fBig5	D562
U+60BF.0	fGB	3-5224
U+60BF.0	fR/S	 61.8
U+60BF.0	fTotalStrokes	 11
U+60BF.0	fCangjie	PHKP
U+60BF.0	fFourCorner	92033
U+60BF.0	fCantonese	tim2; tin2
U+60BF.0	fMandarin	tian3
U+60C0.0	fUTF8	惀
U+60C0.0	fCNS11643	2-3549
U+60C0.0	fBig5	D565
U+60C0.0	fGB	3-513F
U+60C0.0	fR/S	 61.8
U+60C0.0	fTotalStrokes	 11
U+60C0.0	fCangjie	POMB
U+60C0.0	fFourCorner	98027
U+60C0.0	fCantonese	leon4; leon6; leoi4
U+60C0.0	fMandarin	lun2; lun3; lun4
U+60C1.0	fUTF8	惁
U+60C1.0	fCNS11643	2-3B6C
U+60C1.0	fBig5	D949
U+60C1.0	fGB	3-507A
U+60C1.0	fR/S	 61.8
U+60C1.0	fTotalStrokes	 12
U+60C1.0	fCangjie	DLP
U+60C1.0	fFourCorner	42332
U+60C1.0	fHakka	 sit7
U+60C1.0	fCantonese	sik1; sik3
U+60C1.0	fMandarin	xi1; qi1
U+60C1.0	fEnglish	[1] sorrowful [2] venerate
U+60C2.0	fUTF8	惂
U+60C2.0	fCNS11643	4-3168
U+60C2.0	fGB	E-2A34
U+60C2.0	fR/S	  61.8
U+60C2.0	fTotalStrokes	 11
U+60C2.0	fCangjie	PNHX
U+60C2.0	fFourCorner	97070
U+60C2.0	fCantonese	ham2
U+60C2.0	fMandarin	kan3
U+60C3.0	fUTF8	惃
U+60C3.0	fCNS11643	2-3547
U+60C3.0	fBig5	D563
U+60C3.0	fGB	3-522C
U+60C3.0	fR/S	 61.8
U+60C3.0	fTotalStrokes	 11
U+60C3.0	fCangjie	PAPP
U+60C3.0	fFourCorner	96011
U+60C3.0	fHakka	 gun3; kun1; kwun1
U+60C3.0	fCantonese	gwan2; kwan1
U+60C3.0	fMandarin	kun1
U+60C3.0	fEnglish	a kind of red steel called after the name of the old state where it was found
U+60C4.0	fUTF8	惄
U+60C4.0	fCNS11643	2-3B61
U+60C4.0	fBig5	D8FD
U+60C4.0	fGB	3-507B
U+60C4.0	fR/S	 61.8
U+60C4.0	fTotalStrokes	 12
U+60C4.0	fCangjie	YEP
U+60C4.0	fFourCorner	27334
U+60C4.0	fHakka	  nit8; lit7; net8; lit8; net7
U+60C4.0	fCantonese	nik6; lik6 (also)
U+60C4.0	fMandarin	ni4; ni1; nu4
U+60C4.0	fEnglish	 long for hungry
U+60C5.0	fUTF8	情
U+60C5.0	fCNS11643	1-5A45
U+60C5.0	fBig5	B1A1
U+60C5.0	fGB	0-4769
U+60C5.0	fR/S	 61.8
U+60C5.0	fTotalStrokes	 11
U+60C5.0	fCangjie	PQMB
U+60C5.0	fFourCorner	95027
U+60C5.0	fHakka	  cin2; cin3
U+60C5.0	fCantonese	cing4; cing2 (also)
U+60C5.0	fMandarin	qing2
U+60C5.0	fEnglish	[1] feeling; sentiment; emotion [2] fact; detail; situation; condition [3] love; affection; passion [4] nature; reason
U+60C5.1	fUTF8	情
U+60C5.1	fCNS11643	1-5A45
U+60C5.1	fBig5	B1A1
U+60C5.1	fGB	0-4769
U+60C5.1	fR/S	 61.8
U+60C5.1	fTotalStrokes	 11
U+60C5.1	fCangjie	PQMB
U+60C5.1	fFourCorner	95027
U+60C5.1	fMandarin	qing5
U+60C6.0	fUTF8	惆
U+60C6.0	fCNS11643	1-5A4C
U+60C6.0	fBig5	B1A8
U+60C6.0	fGB	0-6330
U+60C6.0	fR/S	 61.8
U+60C6.0	fTotalStrokes	 11
U+60C6.0	fCangjie	PBGR
U+60C6.0	fFourCorner	97020
U+60C6.0	fHakka	 cu2;  chiu2; ciu2; chu2
U+60C6.0	fCantonese	cau4
U+60C6.0	fMandarin	chou2; chou1; chou4; jiu4; qiu1
U+60C6.0	fEnglish	[1] regretful; rueful; disconsolate; melancholy [2] disappointed; frustrated
U+60C7.0	fUTF8	惇
U+60C7.0	fCNS11643	1-5A50
U+60C7.0	fBig5	B1AC
U+60C7.0	fGB	3-5235
U+60C7.0	fR/S	 61.8
U+60C7.0	fTotalStrokes	 11
U+60C7.0	fCangjie	PYRD
U+60C7.0	fFourCorner	90047
U+60C7.0	fHakka	  dun1; zhun1; zun1
U+60C7.0	fCantonese	deon1
U+60C7.0	fMandarin	dun1; chun2; zhun1
U+60C7.0	fEnglish	kind; generous; sincere
U+60C8.0	fUTF8	惈
U+60C8.0	fCNS11643	2-3541
U+60C8.0	fBig5	D55D
U+60C8.0	fGB	3-522B
U+60C8.0	fR/S	 61.8
U+60C8.0	fTotalStrokes	 11
U+60C8.0	fCangjie	PWD
U+60C8.0	fFourCorner	96094
U+60C8.0	fHakka	 go3; gwo3; kwo3
U+60C8.0	fCantonese	gwo2; go2 (also)
U+60C8.0	fMandarin	guo3
U+60C8.0	fEnglish	courageous; brave
U+60C9.0	fUTF8	惉
U+60C9.0	fCNS11643	2-3B5C
U+60C9.0	fBig5	D8F8
U+60C9.0	fGB	5-5328
U+60C9.0	fR/S	 61.8
U+60C9.0	fTotalStrokes	 12
U+60C9.0	fCangjie	ERP
U+60C9.0	fFourCorner	31336
U+60C9.0	fHakka	 zham1; zam1
U+60C9.0	fCantonese	zim1
U+60C9.0	fMandarin	chan1; zhan1
U+60C9.0	fEnglish	musical discord
U+60CA.0	fUTF8	惊
U+60CA.0	fCNS11643	2-3545
U+60CA.0	fBig5	D561
U+60CA.0	fGB	0-3E2A
U+60CA.0	fR/S	 61.8
U+60CA.0	fTotalStrokes	 11
U+60CA.0	fCangjie	PYRF
U+60CA.0	fFourCorner	90096
U+60CA.0	fHakka	 liong2; liong3; giang1; oi1
U+60CA.0	fCantonese	ging1; geng1 (also); loeng6
U+60CA.0	fMandarin	jing1; liang2
U+60CA.0	fEnglish	simplified form of &#x9A5A;
U+60CB.0	fUTF8	惋
U+60CB.0	fCNS11643	1-5A41
U+60CB.0	fBig5	B17B
U+60CB.0	fGB	0-4D6F
U+60CB.0	fR/S	 61.8
U+60CB.0	fTotalStrokes	 11
U+60CB.0	fCangjie	PJNU
U+60CB.0	fFourCorner	93012
U+60CB.0	fHakka	  wan3; won3; van3
U+60CB.0	fCantonese	wun2; jyun2 (also); j
U+60CB.0	fMandarin	wan3
U+60CB.0	fEnglish	[1] regret; be sorry [2] be alarmed
U+60CC.0	fUTF8	惌
U+60CC.0	fCNS11643	2-3B5E
U+60CC.0	fBig5	D8FA
U+60CC.0	fGB	5-5329
U+60CC.0	fR/S	 61.8
U+60CC.0	fTotalStrokes	 12
U+60CC.0	fCangjie	JNUP
U+60CC.0	fFourCorner	30331
U+60CC.0	fHakka	 jen1; ren1; jan1
U+60CC.0	fCantonese	jyun3; jyun1
U+60CC.0	fMandarin	yuan1; yu4
U+60CC.0	fEnglish	 bear a grudge against
U+60CD.0	fUTF8	惍
U+60CD.0	fCNS11643	2-3548
U+60CD.0	fBig5	D564
U+60CD.0	fGB	E-2A35
U+60CD.0	fR/S	 61.8
U+60CD.0	fTotalStrokes	 11
U+60CD.0	fCangjie	PC
U+60CD.0	fFourCorner	98019
U+60CD.0	fCantonese	gam1
U+60CD.0	fMandarin	jin1
U+60CE.0	fUTF8	惎
U+60CE.0	fCNS11643	2-3B60
U+60CE.0	fBig5	D8FC
U+60CE.0	fGB	3-5079
U+60CE.0	fR/S	 61.8
U+60CE.0	fTotalStrokes	 12
U+60CE.0	fCangjie	TCP
U+60CE.0	fFourCorner	44338
U+60CE.0	fHakka	 ki3; ki5
U+60CE.0	fCantonese	gei6; gei3
U+60CE.0	fMandarin	ji4
U+60CE.0	fEnglish	injure; harm; murder
U+60CF.0	fUTF8	惏
U+60CF.0	fCNS11643	2-353D
U+60CF.0	fBig5	D559
U+60CF.0	fGB	E-2A36
U+60CF.0	fR/S	 61.8
U+60CF.0	fTotalStrokes	 11
U+60CF.0	fCangjie	PDD
U+60CF.0	fFourCorner	94090
U+60CF.0	fHakka	  lam2
U+60CF.0	fCantonese	lam4; laam4
U+60CF.0	fMandarin	lin2; lan2; lan3
U+60CF.0	fEnglish	 avaricious greedy cold
U+60D0.0	fUTF8	惐
U+60D0.0	fCNS11643	3-356A
U+60D0.0	fGB	3-5226
U+60D0.0	fR/S	  61.8
U+60D0.0	fTotalStrokes	 11
U+60D0.0	fCangjie	PIRM
U+60D0.0	fFourCorner	93050
U+60D0.0	fCantonese	wik6
U+60D0.0	fMandarin	yu4
U+60D1.0	fUTF8	惑
U+60D1.0	fCNS11643	1-5F29
U+60D1.0	fBig5	B462
U+60D1.0	fGB	0-3B73
U+60D1.0	fR/S	 61.8
U+60D1.0	fTotalStrokes	 12
U+60D1.0	fCangjie	IMP
U+60D1.0	fFourCorner	53330
U+60D1.0	fHakka	  fet8; fiet8
U+60D1.0	fCantonese	waak6
U+60D1.0	fMandarin	huo4
U+60D1.0	fEnglish	 confuse; mislead; baffle doubt
U+60D1.1	fUTF8	惑
U+60D1.1	fCNS11643	1-5F29
U+60D1.1	fBig5	B462
U+60D1.1	fGB	0-3B73
U+60D1.1	fR/S	 61.8
U+60D1.1	fTotalStrokes	 12
U+60D1.1	fCangjie	IMP
U+60D1.1	fFourCorner	53330
U+60D1.1	fMandarin	huo5
U+60D2.0	fUTF8	惒
U+60D2.0	fCNS11643	3-3B56
U+60D2.0	fGB	5-5325
U+60D2.0	fR/S	  61.8
U+60D2.0	fTotalStrokes	 12
U+60D2.0	fCangjie	HRP
U+60D2.0	fFourCorner	26330
U+60D2.0	fHakka	 fo2; fo5
U+60D2.0	fCantonese	wo4; wo6
U+60D2.0	fMandarin	he2; han4; huo; huo1; he5; huo5
U+60D3.0	fUTF8	惓
U+60D3.0	fCNS11643	2-353B
U+60D3.0	fBig5	D557
U+60D3.0	fGB	3-5236
U+60D3.0	fR/S	 61.8
U+60D3.0	fTotalStrokes	 11
U+60D3.0	fCangjie	PFQU
U+60D3.0	fFourCorner	99012
U+60D3.0	fHakka	 gien3; kien3; ken2; kian2; gen3; kian3; ken3
U+60D3.0	fCantonese	kyun4; gyun3
U+60D3.0	fMandarin	quan2; quan1; juan4
U+60D3.0	fEnglish	sincere; candid
U+60D4.0	fUTF8	惔
U+60D4.0	fCNS11643	2-353C
U+60D4.0	fBig5	D558
U+60D4.0	fGB	3-5237
U+60D4.0	fR/S	 61.8
U+60D4.0	fTotalStrokes	 11
U+60D4.0	fCangjie	PFF
U+60D4.0	fFourCorner	99089
U+60D4.0	fHakka	  tam2
U+60D4.0	fCantonese	taam4; daam6; tam4
U+60D4.0	fMandarin	tan2; yan3; dan4; yan2
U+60D4.0	fEnglish	cheerful
U+60D5.0	fUTF8	惕
U+60D5.0	fCNS11643	1-5A4B
U+60D5.0	fBig5	B1A7
U+60D5.0	fGB	0-4C68
U+60D5.0	fR/S	 61.8
U+60D5.0	fTotalStrokes	 11
U+60D5.0	fCangjie	PAPH
U+60D5.0	fFourCorner	96027
U+60D5.0	fHakka	 tit7
U+60D5.0	fCantonese	tik1
U+60D5.0	fMandarin	ti4
U+60D5.0	fEnglish	cautious; careful; prudent; watchout; be on alert
U+60D6.0	fUTF8	惖
U+60D6.0	fCNS11643	3-3B55
U+60D6.0	fGB	5-5322
U+60D6.0	fR/S	  61.8
U+60D6.0	fTotalStrokes	 12
U+60D6.0	fCangjie	AHP
U+60D6.0	fFourCorner	60330
U+60D6.0	fHakka	 tit7
U+60D6.0	fCantonese	tik1; tik4
U+60D6.0	fMandarin	hu1; ti4
U+60D6.0	fEnglish	 respect fear
U+60D7.0	fUTF8	惗
U+60D7.0	fCNS11643	3-356E
U+60D7.0	fGB	3-5233
U+60D7.0	fR/S	  61.8
U+60D7.0	fTotalStrokes	 11
U+60D7.0	fCangjie	POIP
U+60D7.0	fFourCorner	98030
U+60D7.0	fHakka	 ngiap7
U+60D7.0	fCantonese	nam2; lam2 (also)
U+60D7.0	fMandarin	nie1
U+60D7.0	fEnglish	think; consider
U+60D8.0	fUTF8	惘
U+60D8.0	fCNS11643	1-5A4A
U+60D8.0	fBig5	B1A6
U+60D8.0	fGB	0-632F
U+60D8.0	fR/S	 61.8
U+60D8.0	fTotalStrokes	 11
U+60D8.0	fCangjie	PBTV
U+60D8.0	fFourCorner	97020
U+60D8.0	fHakka	  wong3; miong3; vong3
U+60D8.0	fCantonese	mong5
U+60D8.0	fMandarin	wang3
U+60D8.0	fEnglish	frustrated; dejected; discouraged
U+60D9.0	fUTF8	惙
U+60D9.0	fCNS11643	2-353F
U+60D9.0	fBig5	D55B
U+60D9.0	fGB	3-523D
U+60D9.0	fR/S	 61.8
U+60D9.0	fTotalStrokes	 11
U+60D9.0	fCangjie	PEEE
U+60D9.0	fFourCorner	97047
U+60D9.0	fHakka	  dot7
U+60D9.0	fCantonese	zyut3
U+60D9.0	fMandarin	chuo4; zhuo2
U+60D9.0	fEnglish	sad; melancholy; grieving; mourn
U+60DA.0	fUTF8	惚
U+60DA.0	fCNS11643	1-5A4F
U+60DA.0	fBig5	B1AB
U+60DA.0	fGB	0-6331
U+60DA.0	fR/S	 61.8
U+60DA.0	fTotalStrokes	 11
U+60DA.0	fCangjie	PPHP
U+60DA.0	fFourCorner	97032
U+60DA.0	fHakka	 fut7
U+60DA.0	fCantonese	fat1
U+60DA.0	fMandarin	hu1; bu1
U+60DA.0	fEnglish	absent-minded; entranced
U+60DB.0	fUTF8	惛
U+60DB.0	fCNS11643	2-3543
U+60DB.0	fBig5	D55F
U+60DB.0	fGB	3-5234
U+60DB.0	fR/S	 61.8
U+60DB.0	fTotalStrokes	 11
U+60DB.0	fCangjie	PHPA
U+60DB.0	fFourCorner	92064
U+60DB.0	fHakka	  fun1
U+60DB.0	fCantonese	fan1; man5
U+60DB.0	fMandarin	hun1; hun3; hun4; men4; min3
U+60DB.0	fEnglish	[1] confused; [2] senile
U+60DC.0	fUTF8	惜
U+60DC.0	fCNS11643	1-5A48
U+60DC.0	fBig5	B1A4
U+60DC.0	fGB	0-4F27
U+60DC.0	fR/S	 61.8
U+60DC.0	fTotalStrokes	 11
U+60DC.0	fCangjie	PTA
U+60DC.0	fFourCorner	94061
U+60DC.0	fHakka	  sit7;  siak7
U+60DC.0	fCantonese	sik1; sek3 (also)
U+60DC.0	fMandarin	xi1
U+60DC.0	fEnglish	 pity; regret; sympathize; feel sorry for somebody
U+60DD.0	fUTF8	惝
U+60DD.0	fCNS11643	2-3540
U+60DD.0	fBig5	D55C
U+60DD.0	fGB	0-632E
U+60DD.0	fR/S	 61.8
U+60DD.0	fTotalStrokes	 11
U+60DD.0	fCangjie	PFBR
U+60DD.0	fFourCorner	99027
U+60DD.0	fHakka	 chong3; cong3
U+60DD.0	fCantonese	cong2; tong2
U+60DD.0	fMandarin	chang3; tang3
U+60DD.0	fEnglish	dispirited; disheartened; discouraged
U+60DE.0	fUTF8	惞
U+60DE.0	fCNS11643	3-3579
U+60DE.0	fGB	E-2A37
U+60DE.0	fR/S	  61.8
U+60DE.0	fTotalStrokes	 11
U+60DE.0	fCangjie	PHLO
U+60DE.0	fFourCorner	97080
U+60DE.0	fHakka	 hiun1
U+60DE.0	fCantonese	jan1
U+60DE.0	fMandarin	xin1
U+60DE.0	fEnglish	joy; delight; elation
U+60DF.0	fUTF8	惟
U+60DF.0	fCNS11643	1-5A4D
U+60DF.0	fBig5	B1A9
U+60DF.0	fGB	0-4E29
U+60DF.0	fR/S	 61.8
U+60DF.0	fTotalStrokes	 11
U+60DF.0	fCangjie	POG
U+60DF.0	fFourCorner	90014
U+60DF.0	fHakka	 wui2; wi2; wui3; vui2
U+60DF.0	fCantonese	wai4
U+60DF.0	fMandarin	wei2
U+60DF.0	fEnglish	[1] think; meditate [2] only; alone [3] but; however; nevertheless
U+60E0.0	fUTF8	惠
U+60E0.0	fCNS11643	1-5F2D
U+60E0.0	fBig5	B466
U+60E0.0	fGB	0-3B5D
U+60E0.0	fR/S	 61.8
U+60E0.0	fTotalStrokes	 12
U+60E0.0	fCangjie	JIP
U+60E0.0	fFourCorner	50333
U+60E0.0	fHakka	  fui5; fui6; fi3
U+60E0.0	fCantonese	wai6
U+60E0.0	fMandarin	hui4
U+60E0.0	fEnglish	[1] favor; benefit; profit [2] kind; benevolent; gracious [3] gentle and yielding
U+60E1.0	fUTF8	惡
U+60E1.0	fCNS11643	1-5F2A
U+60E1.0	fBig5	B463
U+60E1.0	fGB	1-3671
U+60E1.0	fR/S	 61.8
U+60E1.0	fTotalStrokes	 12
U+60E1.0	fCangjie	MMP
U+60E1.0	fFourCorner	10331
U+60E1.0	fHakka	  ok7; ngo5
U+60E1.0	fCantonese	ok3; ngok3 (also); fu1
U+60E1.0	fMandarin	e4
U+60E1.0	fEnglish	[1] evil; wicked; bad; vice [2] fierce; ferocious
U+60E1.1	fUTF8	惡
U+60E1.1	fCNS11643	1-5F2A
U+60E1.1	fBig5	B463
U+60E1.1	fGB	1-3671
U+60E1.1	fR/S	 61.8
U+60E1.1	fTotalStrokes	 12
U+60E1.1	fCangjie	MMP
U+60E1.1	fFourCorner	10331
U+60E1.1	fHakka	vu4; vou5; wu5; vu5
U+60E1.1	fCantonese	wu3; wu1
U+60E1.1	fMandarin	e3
U+60E1.1	fEnglish	 disgust; sicken; scorn
U+60E1.2	fUTF8	惡
U+60E1.2	fCNS11643	1-5F2A
U+60E1.2	fBig5	B463
U+60E1.2	fGB	1-3671
U+60E1.2	fR/S	 61.8
U+60E1.2	fTotalStrokes	 12
U+60E1.2	fCangjie	MMP
U+60E1.2	fFourCorner	10331
U+60E1.2	fMandarin	wu4
U+60E1.2	fEnglish	 hate; detest; dislike; abhor; loathe
U+60E1.3	fUTF8	惡
U+60E1.3	fCNS11643	1-5F2A
U+60E1.3	fBig5	B463
U+60E1.3	fGB	1-3671
U+60E1.3	fR/S	 61.8
U+60E1.3	fTotalStrokes	 12
U+60E1.3	fCangjie	MMP
U+60E1.3	fFourCorner	10331
U+60E1.3	fMandarin	wu1
U+60E1.3	fEnglish	[1] how; where [2] O; oh; ah
U+60E2.0	fUTF8	惢
U+60E2.0	fCNS11643	2-3B5F
U+60E2.0	fBig5	D8FB
U+60E2.0	fGB	3-507D
U+60E2.0	fR/S	 61.8
U+60E2.0	fTotalStrokes	 12
U+60E2.0	fCangjie	PPP
U+60E2.0	fFourCorner	33330
U+60E2.0	fHakka	 so3;  lui3
U+60E2.0	fCantonese	so2
U+60E2.0	fMandarin	rui3; suo3; zui1; zui3
U+60E2.0	fEnglish	suspicious, doubtful
U+60E3.0	fUTF8	惣
U+60E3.0	fCNS11643	3-3B58
U+60E3.0	fGB	E-2A38
U+60E3.0	fR/S	  61.8
U+60E3.0	fTotalStrokes	 12
U+60E3.0	fCangjie	HHP
U+60E3.0	fFourCorner	27330
U+60E3.0	fCantonese	zung2
U+60E3.0	fMandarin	zong3
U+60E3.0	fEnglish	overall
U+60E4.0	fUTF8	惤
U+60E4.0	fCNS11643	2-353E
U+60E4.0	fBig5	D55A
U+60E4.0	fGB	E-2A39
U+60E4.0	fR/S	 61.8
U+60E4.0	fTotalStrokes	 11
U+60E4.0	fCangjie	PNYI
U+60E4.0	fFourCorner	90032
U+60E4.0	fCantonese	gin1
U+60E4.0	fMandarin	jian1
U+60E5.0	fUTF8	惥
U+60E5.0	fCNS11643	3-3B59
U+60E5.0	fGB	E-2A3A
U+60E5.0	fR/S	  61.8
U+60E5.0	fTotalStrokes	 12
U+60E5.0	fCangjie	HOP
U+60E5.0	fFourCorner	77330
U+60E5.0	fHakka	 jung3
U+60E5.0	fCantonese	jung2
U+60E5.0	fMandarin	yong3
U+60E6.0	fUTF8	惦
U+60E6.0	fCNS11643	1-5A43
U+60E6.0	fBig5	B17D
U+60E6.0	fGB	0-356B
U+60E6.0	fR/S	 61.8
U+60E6.0	fTotalStrokes	 11
U+60E6.0	fCangjie	PIYR
U+60E6.0	fFourCorner	90061
U+60E6.0	fHakka	  diam5
U+60E6.0	fCantonese	dim3
U+60E6.0	fMandarin	dian4
U+60E6.0	fEnglish	 remember; bear in mind; miss; be concerned about; keep thinking of
U+60E7.0	fUTF8	惧
U+60E7.0	fCNS11643	3-3576
U+60E7.0	fGB	0-3E65
U+60E7.0	fR/S	  61.8
U+60E7.0	fTotalStrokes	 11
U+60E7.0	fCangjie	PBMC
U+60E7.0	fFourCorner	97081
U+60E7.0	fHakka	 ki3
U+60E7.0	fCantonese	geoi6
U+60E7.0	fMandarin	ju4; ju3; qu2
U+60E7.0	fEnglish	[1] non-classical form of &#x61FC; [2] simplified form of &#x61FC;
U+60E8.0	fUTF8	惨
U+60E8.0	fCNS11643	3-3578
U+60E8.0	fGB	0-3252
U+60E8.0	fR/S	  61.8
U+60E8.0	fTotalStrokes	 11
U+60E8.0	fCangjie	PIKH
U+60E8.0	fFourCorner	93010
U+60E8.0	fHakka	 cam3
U+60E8.0	fCantonese	caam2
U+60E8.0	fMandarin	can3; can4; cen3
U+60E8.0	fEnglish	[1] non-classical form of &#x6158; [2] simplified form of &#x6158;
U+60E9.0	fUTF8	惩
U+60E9.0	fCNS11643	3-3B5A
U+60E9.0	fGB	0-334D
U+60E9.0	fR/S	 61.8
U+60E9.0	fTotalStrokes	 12
U+60E9.0	fCangjie	HMP
U+60E9.0	fFourCorner	21330
U+60E9.0	fHakka	 zhin1; zihn1; zhin3
U+60E9.0	fCantonese	cing4
U+60E9.0	fMandarin	cheng2
U+60E9.0	fEnglish	simplified form of &#x61F2;
U+60EA.0	fUTF8	惪
U+60EA.0	fCNS11643	3-3B4D
U+60EA.0	fGB	E-2A3B
U+60EA.0	fR/S	  61.8
U+60EA.0	fTotalStrokes	 12; 11
U+60EA.0	fCangjie	JMP
U+60EA.0	fFourCorner	40330
U+60EA.0	fCantonese	dak1
U+60EA.0	fMandarin	de2
U+60EA.0	fEnglish	ethics
U+60EB.0	fUTF8	惫
U+60EB.0	fGB	0-3139
U+60EB.0	fR/S	 34.9; 61.8
U+60EB.0	fTotalStrokes	 12
U+60EB.0	fCangjie	HEWP
U+60EB.0	fHakka	 pai5; pi5
U+60EB.0	fCantonese	bei6; baai6 (also)
U+60EB.0	fMandarin	bei4; bi4; bo2
U+60EB.0	fEnglish	simplified form of &#x618A;
U+60EC.0	fUTF8	惬
U+60EC.0	fGB	0-632B
U+60EC.0	fR/S	 61.8
U+60EC.0	fTotalStrokes	 11
U+60EC.0	fCangjie	PSKT
U+60EC.0	fHakka	 giap7
U+60EC.0	fCantonese	hip3; hip6 (also)
U+60EC.0	fMandarin	qie4
U+60EC.0	fEnglish	simplified form of &#x611C;
U+60ED.0	fUTF8	惭
U+60ED.0	fGB	0-3251
U+60ED.0	fR/S	 61.8
U+60ED.0	fTotalStrokes	 11
U+60ED.0	fCangjie	PKQL
U+60ED.0	fHakka	 cam1; cam2
U+60ED.0	fCantonese	caam4
U+60ED.0	fMandarin	can2
U+60ED.0	fEnglish	simplified form of &#x615A;
U+60EE.0	fUTF8	惮
U+60EE.0	fCNS11643	3-3574
U+60EE.0	fGB	0-352C
U+60EE.0	fR/S	 61.8
U+60EE.0	fTotalStrokes	 11
U+60EE.0	fCangjie	PCWJ
U+60EE.0	fFourCorner	98050
U+60EE.0	fHakka	 tan3
U+60EE.0	fCantonese	daan6
U+60EE.0	fMandarin	dan4; chan3; chan4; da2; dan3; duo4; tan2
U+60EE.0	fEnglish	simplified form of &#x619A;
U+60EF.0	fUTF8	惯
U+60EF.0	fGB	0-395F
U+60EF.0	fR/S	 61.8
U+60EF.0	fTotalStrokes	 11
U+60EF.0	fCangjie	PWJO
U+60EF.0	fHakka	 kwan5
U+60EF.0	fCantonese	gwaan3
U+60EF.0	fMandarin	guan4
U+60EF.0	fEnglish	simplified form of &#x6163;
U+60F0.0	fUTF8	惰
U+60F0.0	fCNS11643	1-5F32
U+60F0.0	fBig5	B46B
U+60F0.0	fGB	0-3668
U+60F0.0	fR/S	 61.9
U+60F0.0	fTotalStrokes	 12
U+60F0.0	fCangjie	PKMB
U+60F0.0	fFourCorner	94027
U+60F0.0	fHakka	 to5; to6; to3; to2
U+60F0.0	fCantonese	do6
U+60F0.0	fMandarin	duo4; tuo2; duo3
U+60F0.0	fEnglish	lazy; idle; indolent
U+60F1.0	fUTF8	惱
U+60F1.0	fCNS11643	1-5F36
U+60F1.0	fBig5	B46F
U+60F1.0	fGB	1-4455
U+60F1.0	fR/S	 61.9
U+60F1.0	fTotalStrokes	 12
U+60F1.0	fCangjie	PVVW
U+60F1.0	fFourCorner	92063
U+60F1.0	fHakka	 nau1;  nau3; lau3; lau1; no3; lau5
U+60F1.0	fCantonese	nou5; lou5 (also)
U+60F1.0	fMandarin	nao3
U+60F1.0	fEnglish	[1] angered; offended; vexed; annoyed [2] anger; exasperate; annoy; irritate; vex; trouble
U+60F2.0	fUTF8	惲
U+60F2.0	fCNS11643	2-3B63
U+60F2.0	fBig5	D940
U+60F2.0	fGB	1-6322
U+60F2.0	fR/S	 61.9
U+60F2.0	fTotalStrokes	 12
U+60F2.0	fCangjie	PBJJ
U+60F2.0	fFourCorner	97052
U+60F2.0	fHakka	 wun3; vun3
U+60F2.0	fCantonese	wan6; wan2
U+60F2.0	fMandarin	yun4; jun3; yun3
U+60F2.0	fEnglish	[1] devise; plan; consider; deliberate [2] a Chinese family name
U+60F3.0	fUTF8	想
U+60F3.0	fCNS11643	1-6377
U+60F3.0	fBig5	B751
U+60F3.0	fGB	0-4F6B
U+60F3.0	fR/S	 61.9
U+60F3.0	fTotalStrokes	 13
U+60F3.0	fCangjie	DUP
U+60F3.0	fFourCorner	46330
U+60F3.0	fHakka	  siong3; siong6
U+60F3.0	fCantonese	soeng2
U+60F3.0	fMandarin	xiang3
U+60F3.0	fEnglish	[1] think; consider; suppose [2] hope; expect [3] plan [4] remember with longing; miss [5] want; would like to
U+60F4.0	fUTF8	惴
U+60F4.0	fCNS11643	1-5F34
U+60F4.0	fBig5	B46D
U+60F4.0	fGB	0-6337
U+60F4.0	fR/S	 61.9
U+60F4.0	fTotalStrokes	 12
U+60F4.0	fCangjie	PUMB
U+60F4.0	fFourCorner	92027
U+60F4.0	fHakka	 cui3; zui3; chui3; zhui5; nion1; cui5; ch'ui5
U+60F4.0	fCantonese	zeoi3; cyun2; cyun3
U+60F4.0	fMandarin	zhui4; gua4
U+60F4.0	fEnglish	afraid; worried; apprehensive; anxious
U+60F5.0	fUTF8	惵
U+60F5.0	fCNS11643	2-3B67
U+60F5.0	fBig5	D944
U+60F5.0	fGB	5-5425
U+60F5.0	fR/S	 61.9
U+60F5.0	fTotalStrokes	 12
U+60F5.0	fCangjie	PPTD
U+60F5.0	fFourCorner	94094
U+60F5.0	fHakka	  tiap8; tiap7
U+60F5.0	fCantonese	dip6
U+60F5.0	fMandarin	die4; die2; xie4
U+60F5.0	fEnglish	afraid; fearful; timorous
U+60F6.0	fUTF8	惶
U+60F6.0	fCNS11643	1-5F38
U+60F6.0	fBig5	B471
U+60F6.0	fGB	0-3B4C
U+60F6.0	fR/S	 61.9
U+60F6.0	fTotalStrokes	 12
U+60F6.0	fCangjie	PHAG
U+60F6.0	fFourCorner	96014
U+60F6.0	fHakka	 fong2; fong3
U+60F6.0	fCantonese	wong4
U+60F6.0	fMandarin	huang2; wang2
U+60F6.0	fEnglish	[1] fearful; afraid [2] anxious; uneasy [3] flurried; hurried
U+60F7.0	fUTF8	惷
U+60F7.0	fCNS11643	2-426B
U+60F7.0	fBig5	DD65
U+60F7.0	fGB	E-2A3C
U+60F7.0	fR/S	 61.9
U+60F7.0	fTotalStrokes	 13
U+60F7.0	fCangjie	QAP
U+60F7.0	fFourCorner	40336
U+60F7.0	fCantonese	ceon2; cyun2
U+60F7.0	fMandarin	chun3
U+60F7.0	fEnglish	 wriggle stupid silly fat
U+60F8.0	fUTF8	惸
U+60F8.0	fCNS11643	2-3B69
U+60F8.0	fBig5	D946
U+60F8.0	fGB	5-542F
U+60F8.0	fR/S	 61.9
U+60F8.0	fTotalStrokes	 12
U+60F8.0	fCangjie	PPAD
U+60F8.0	fFourCorner	97047
U+60F8.0	fHakka	  kiun2
U+60F8.0	fCantonese	king4
U+60F8.0	fMandarin	qiong2
U+60F8.0	fEnglish	[1] troubled; worried; distressed [2] brotherless; friendless
U+60F9.0	fUTF8	惹
U+60F9.0	fCNS11643	1-6379
U+60F9.0	fBig5	B753
U+60F9.0	fGB	0-4847
U+60F9.0	fR/S	 61.9
U+60F9.0	fTotalStrokes	 13
U+60F9.0	fCangjie	TKRP
U+60F9.0	fFourCorner	44336
U+60F9.0	fHakka	  ngia1; ngia3;  ngia5; gnia1; ngia2
U+60F9.0	fCantonese	je5
U+60F9.0	fMandarin	re3; ruo4
U+60F9.0	fEnglish	 irritate; incite; provoke; rouse; induce; attract; cause; bring upon oneself; offend; incur
U+60FA.0	fUTF8	惺
U+60FA.0	fCNS11643	1-5F30
U+60FA.0	fBig5	B469
U+60FA.0	fGB	0-504A
U+60FA.0	fR/S	 61.9
U+60FA.0	fTotalStrokes	 12
U+60FA.0	fCangjie	PAHM
U+60FA.0	fFourCorner	96014
U+60FA.0	fHakka	 sen1; sin1; siang3
U+60FA.0	fCantonese	sing1; sing2
U+60FA.0	fMandarin	xing1
U+60FA.0	fEnglish	[1] intelligent; clever; wise [2] wavering; indecisive
U+60FA.1	fUTF8	惺
U+60FA.1	fCNS11643	1-5F30
U+60FA.1	fBig5	B469
U+60FA.1	fGB	0-504A
U+60FA.1	fR/S	 61.9
U+60FA.1	fTotalStrokes	 12
U+60FA.1	fCangjie	PAHM
U+60FA.1	fFourCorner	96014
U+60FA.1	fMandarin	xing3
U+60FA.1	fEnglish	 become aware of; awake from ignorance
U+60FB.0	fUTF8	惻
U+60FB.0	fCNS11643	1-5F33
U+60FB.0	fBig5	B46C
U+60FB.0	fGB	1-627C
U+60FB.0	fR/S	 61.9
U+60FB.0	fTotalStrokes	 12
U+60FB.0	fCangjie	PBCN
U+60FB.0	fFourCorner	92000
U+60FB.0	fHakka	 cet7
U+60FB.0	fCantonese	cak1; caak1 (also)
U+60FB.0	fMandarin	ce4
U+60FB.0	fEnglish	 feel anguish
U+60FC.0	fUTF8	惼
U+60FC.0	fCNS11643	2-3B6A
U+60FC.0	fBig5	D947
U+60FC.0	fGB	3-524E
U+60FC.0	fR/S	 61.9
U+60FC.0	fTotalStrokes	 12
U+60FC.0	fCangjie	PHSB
U+60FC.0	fFourCorner	93027
U+60FC.0	fHakka	  bien3
U+60FC.0	fCantonese	bin2; pin2 (also)
U+60FC.0	fMandarin	bian3
U+60FC.0	fEnglish	[1] irritable [2] hasty in temper
U+60FD.0	fUTF8	惽
U+60FD.0	fCNS11643	3-3B5E
U+60FD.0	fGB	5-5434
U+60FD.0	fR/S	 61.9
U+60FD.0	fTotalStrokes	 12
U+60FD.0	fCangjie	PRPA
U+60FD.0	fFourCorner	97060
U+60FD.0	fHakka	 fun1
U+60FD.0	fCantonese	fan1; man5
U+60FD.0	fMandarin	hun1
U+60FD.0	fEnglish	confused
U+60FE.0	fUTF8	惾
U+60FE.0	fCNS11643	2-3B6B
U+60FE.0	fBig5	D948
U+60FE.0	fGB	3-5249
U+60FE.0	fR/S	 61.9
U+60FE.0	fTotalStrokes	 12
U+60FE.0	fCangjie	PUCE
U+60FE.0	fFourCorner	92047
U+60FE.0	fCantonese	zung1; sau3
U+60FE.0	fMandarin	zong1; sou4
U+60FF.0	fUTF8	惿
U+60FF.0	fCNS11643	2-3B71
U+60FF.0	fBig5	D94E
U+60FF.0	fGB	3-5243
U+60FF.0	fR/S	 61.9
U+60FF.0	fTotalStrokes	 12
U+60FF.0	fCangjie	PAMO
U+60FF.0	fFourCorner	96081
U+60FF.0	fCantonese	tai4; si6
U+60FF.0	fMandarin	ti2; shi4
U+6100.0	fUTF8	愀
U+6100.0	fCNS11643	1-5F3A
U+6100.0	fBig5	B473
U+6100.0	fGB	0-6338
U+6100.0	fR/S	 61.9
U+6100.0	fTotalStrokes	 12
U+6100.0	fCangjie	PHDF
U+6100.0	fFourCorner	99080
U+6100.0	fHakka	  ciau3; ziu5
U+6100.0	fCantonese	ciu2; cau5
U+6100.0	fMandarin	qiao3; jiao3; jiu3; qiao1; qiu1
U+6100.0	fEnglish	[1] change one's countenance; show a sudden change of expression [2] anxious-looking
U+6101.0	fUTF8	愁
U+6101.0	fCNS11643	1-637A
U+6101.0	fBig5	B754
U+6101.0	fGB	0-336E
U+6101.0	fR/S	 61.9
U+6101.0	fTotalStrokes	 13
U+6101.0	fCangjie	HFP
U+6101.0	fFourCorner	29338
U+6101.0	fHakka	 seu2; sieu2; seu3; siu2; cau2
U+6101.0	fCantonese	sau4
U+6101.0	fMandarin	chou2; cao2; jiu1
U+6101.0	fEnglish	[1] sad; distressed; worried; unhappy; melancholy [2] depressing; saddening; gloomy [3] be anxious about; worry about [4] sorrow; woe
U+6102.0	fUTF8	愂
U+6102.0	fCNS11643	4-3D3D
U+6102.0	fGB	E-2A3D
U+6102.0	fR/S	 61.9
U+6102.0	fTotalStrokes	 13
U+6102.0	fCangjie	JSP
U+6102.0	fFourCorner	44330
U+6102.0	fCantonese	bui6
U+6102.0	fMandarin	bei4; bei3
U+6103.0	fUTF8	愃
U+6103.0	fCNS11643	2-3B6D
U+6103.0	fBig5	D94A
U+6103.0	fGB	3-524D
U+6103.0	fR/S	 61.9
U+6103.0	fTotalStrokes	 12
U+6103.0	fCangjie	PJMM
U+6103.0	fFourCorner	93016
U+6103.0	fCantonese	syun1; hyun1; syun2
U+6103.0	fMandarin	xuan1; xuan3; le4
U+6103.0	fEnglish	relax
U+6104.0	fUTF8	愄
U+6104.0	fCNS11643	2-3B72
U+6104.0	fBig5	D94F
U+6104.0	fGB	5-542B
U+6104.0	fR/S	 61.9
U+6104.0	fTotalStrokes	 12
U+6104.0	fCangjie	PWMV
U+6104.0	fFourCorner	96032
U+6104.0	fCantonese	wui1; wui4
U+6104.0	fMandarin	wei1
U+6105.0	fUTF8	愅
U+6105.0	fCNS11643	2-3B66
U+6105.0	fBig5	D943
U+6105.0	fGB	5-5424
U+6105.0	fR/S	 61.9
U+6105.0	fTotalStrokes	 12
U+6105.0	fCangjie	PTLJ
U+6105.0	fFourCorner	94056
U+6105.0	fCantonese	gaak3
U+6105.0	fMandarin	ge2
U+6106.0	fUTF8	愆
U+6106.0	fCNS11643	1-6426
U+6106.0	fBig5	B75E
U+6106.0	fGB	0-6D29
U+6106.0	fR/S	 61.9
U+6106.0	fTotalStrokes	 13
U+6106.0	fCangjie	HNP
U+6106.0	fFourCorner	21332
U+6106.0	fHakka	 kien1; ken1; kian1
U+6106.0	fCantonese	hin1
U+6106.0	fMandarin	qian1
U+6106.0	fEnglish	[1] fault; mistake; error; misdemeanor [2] lose [3] malignant disease
U+6107.0	fUTF8	愇
U+6107.0	fCNS11643	3-3B4F
U+6107.0	fGB	3-5138
U+6107.0	fR/S	 61.9
U+6107.0	fTotalStrokes	 12
U+6107.0	fCangjie	PDMQ
U+6107.0	fFourCorner	94050
U+6107.0	fCantonese	wai5
U+6107.0	fMandarin	wei3
U+6108.0	fUTF8	愈
U+6108.0	fCNS11643	1-637B
U+6108.0	fBig5	B755
U+6108.0	fGB	0-537A
U+6108.0	fR/S	 61.9
U+6108.0	fTotalStrokes	 13
U+6108.0	fCangjie	OMBP
U+6108.0	fFourCorner	80332
U+6108.0	fHakka	  ji3;  ji5; ri3; ri5; i5
U+6108.0	fCantonese	jyu6
U+6108.0	fMandarin	yu4; yu2; yu3
U+6108.0	fEnglish	[1] more and more; even more; to a greater degree [2] recover (from illness); heal
U+6109.0	fUTF8	愉
U+6109.0	fCNS11643	1-5F39
U+6109.0	fBig5	B472
U+6109.0	fGB	0-5364
U+6109.0	fR/S	 61.9
U+6109.0	fTotalStrokes	 12
U+6109.0	fCangjie	POMN
U+6109.0	fFourCorner	98021
U+6109.0	fHakka	  ji2; ji3; ri2; i2
U+6109.0	fCantonese	jyu4; jyu6 (also); tau1
U+6109.0	fMandarin	yu2; tou1; yu3
U+6109.0	fEnglish	pleasant; delightful; pleased; contended; happy
U+610A.0	fUTF8	愊
U+610A.0	fCNS11643	2-3B64
U+610A.0	fBig5	D941
U+610A.0	fGB	3-5242
U+610A.0	fR/S	 61.9
U+610A.0	fTotalStrokes	 12
U+610A.0	fCangjie	PMRW
U+610A.0	fFourCorner	91066
U+610A.0	fHakka	  bit7; bet7
U+610A.0	fCantonese	bik1
U+610A.0	fMandarin	bi4
U+610A.0	fEnglish	[1] sincere; honest [2] depressed; melancholy
U+610B.0	fUTF8	愋
U+610B.0	fCNS11643	2-3B73
U+610B.0	fBig5	D950
U+610B.0	fGB	3-5248
U+610B.0	fR/S	 61.9
U+610B.0	fTotalStrokes	 12
U+610B.0	fCangjie	PBME
U+610B.0	fFourCorner	92047
U+610B.0	fHakka	  sien1; hien1
U+610B.0	fCantonese	hyun1; syun1
U+610B.0	fMandarin	xuan1; yuan2
U+610B.0	fEnglish	[1] forget [2] dislike
U+610C.0	fUTF8	愌
U+610C.0	fCNS11643	3-3B51
U+610C.0	fGB	3-517B
U+610C.0	fR/S	 61.9
U+610C.0	fTotalStrokes	 12
U+610C.0	fCangjie	PNBK
U+610C.0	fFourCorner	97030
U+610C.0	fCantonese	wun6
U+610C.0	fMandarin	huan4
U+610D.0	fUTF8	愍
U+610D.0	fCNS11643	1-6425
U+610D.0	fBig5	B75D
U+610D.0	fGB	0-6D2A
U+610D.0	fR/S	 61.9
U+610D.0	fTotalStrokes	 13
U+610D.0	fCangjie	RKP
U+610D.0	fFourCorner	78334
U+610D.0	fHakka	  men3; men1
U+610D.0	fCantonese	man5
U+610D.0	fMandarin	min3; fen1; min4
U+610D.0	fEnglish	 pity; commiserate
U+610E.0	fUTF8	愎
U+610E.0	fCNS11643	1-5F37
U+610E.0	fBig5	B470
U+610E.0	fGB	0-6339
U+610E.0	fR/S	 61.9
U+610E.0	fTotalStrokes	 12
U+610E.0	fCangjie	POAE
U+610E.0	fFourCorner	98047
U+610E.0	fHakka	  buk7;  bet7;  pit7; bit7
U+610E.0	fCantonese	bik1; fuk1
U+610E.0	fMandarin	bi4
U+610E.0	fEnglish	obstinate; stubborn; perverse; self-willed
U+610F.0	fUTF8	意
U+610F.0	fCNS11643	1-6374
U+610F.0	fBig5	B74E
U+610F.0	fGB	0-5262
U+610F.0	fR/S	 61.9
U+610F.0	fTotalStrokes	 13
U+610F.0	fCangjie	YTAP
U+610F.0	fFourCorner	00336
U+610F.0	fHakka	  ji5; ri5; i5
U+610F.0	fCantonese	ji3; ji1 (also); ji2 (also)
U+610F.0	fMandarin	yi4; yi1
U+610F.0	fEnglish	[1] thought; idea; sentiments; opinion [2] intention; inclination [3] expectations [4] meaning [5] hint; suggestion
U+610F.1	fUTF8	意
U+610F.1	fCNS11643	1-6374
U+610F.1	fBig5	B74E
U+610F.1	fGB	0-5262
U+610F.1	fR/S	 61.9
U+610F.1	fTotalStrokes	 13
U+610F.1	fCangjie	YTAP
U+610F.1	fFourCorner	00336
U+610F.1	fMandarin	yi5
U+6110.0	fUTF8	愐
U+6110.0	fCNS11643	2-3B70
U+6110.0	fBig5	D94D
U+6110.0	fGB	5-5428
U+6110.0	fR/S	 61.9
U+6110.0	fTotalStrokes	 12
U+6110.0	fCangjie	PMWL
U+6110.0	fFourCorner	91060
U+6110.0	fHakka	 men3;  mien3;  min3
U+6110.0	fCantonese	min5
U+6110.0	fMandarin	mian3; ding3
U+6110.0	fEnglish	[1] remember; give thought to [2] shy
U+6111.0	fUTF8	愑
U+6111.0	fCNS11643	3-3B5D
U+6111.0	fGB	5-5436
U+6111.0	fR/S	  61.9
U+6111.0	fTotalStrokes	 12
U+6111.0	fCangjie	PNBS
U+6111.0	fFourCorner	97020
U+6111.0	fCantonese	jung5
U+6111.0	fMandarin	yong3
U+6111.0	fEnglish	same as &#x6080;
U+6112.0	fUTF8	愒
U+6112.0	fCNS11643	1-5F3B
U+6112.0	fBig5	B474
U+6112.0	fGB	3-5244
U+6112.0	fR/S	 61.9
U+6112.0	fTotalStrokes	 12
U+6112.0	fCangjie	PAPV
U+6112.0	fFourCorner	96027
U+6112.0	fHakka	 ki1; hi1; hi5; kai3; hot7; ki5; k'e5
U+6112.0	fCantonese	koi3; kai3; hei3 (also); hot3
U+6112.0	fMandarin	qi4; kai4; he1; kai3; qie4
U+6112.0	fEnglish	 idle away (time)
U+6113.0	fUTF8	愓
U+6113.0	fCNS11643	2-3B68
U+6113.0	fBig5	D945
U+6113.0	fGB	3-5136
U+6113.0	fR/S	 61.9
U+6113.0	fTotalStrokes	 12
U+6113.0	fCangjie	PAMH
U+6113.0	fFourCorner	98027
U+6113.0	fHakka	  tong3
U+6113.0	fCantonese	dong6; soeng1
U+6113.0	fMandarin	dang4; shang1; yang2
U+6113.0	fEnglish	profligate
U+6114.0	fUTF8	愔
U+6114.0	fCNS11643	2-3B62
U+6114.0	fBig5	D8FE
U+6114.0	fGB	3-524A
U+6114.0	fR/S	 61.9
U+6114.0	fTotalStrokes	 12
U+6114.0	fCangjie	PYTA
U+6114.0	fFourCorner	90061
U+6114.0	fHakka	 jim1; rim1
U+6114.0	fCantonese	jam1; jam4
U+6114.0	fMandarin	yin1; an1
U+6114.0	fEnglish	composed; peaceful; serene
U+6115.0	fUTF8	愕
U+6115.0	fCNS11643	1-5F31
U+6115.0	fBig5	B46A
U+6115.0	fGB	0-6335
U+6115.0	fR/S	 61.9
U+6115.0	fTotalStrokes	 12
U+6115.0	fCangjie	PRRS
U+6115.0	fFourCorner	96027
U+6115.0	fHakka	  ngok8; ngok7
U+6115.0	fCantonese	ngok6; ok6 (also)
U+6115.0	fMandarin	e4; wu4
U+6115.0	fEnglish	startled; amazed; astonished
U+6116.0	fUTF8	愖
U+6116.0	fCNS11643	2-3B65
U+6116.0	fBig5	D942
U+6116.0	fGB	3-5240
U+6116.0	fR/S	 61.9
U+6116.0	fTotalStrokes	 12
U+6116.0	fCangjie	PTMV
U+6116.0	fFourCorner	94011
U+6116.0	fHakka	 shim3; cam3; sim3; shim1
U+6116.0	fCantonese	sam4; sam6
U+6116.0	fMandarin	chen2; dan1; xin4; zhen4
U+6116.0	fEnglish	sincere
U+6117.0	fUTF8	愗
U+6117.0	fCNS11643	5-3F30
U+6117.0	fGB	3-5121
U+6117.0	fR/S	  61.9
U+6117.0	fTotalStrokes	 13
U+6117.0	fCangjie	NKP
U+6117.0	fFourCorner	18334
U+6117.0	fCantonese	mau6
U+6117.0	fMandarin	mou4
U+6118.0	fUTF8	愘
U+6118.0	fCNS11643	2-3B6E
U+6118.0	fBig5	D94B
U+6118.0	fGB	5-5432
U+6118.0	fR/S	 61.9
U+6118.0	fTotalStrokes	 12
U+6118.0	fCangjie	PJHR
U+6118.0	fFourCorner	93064
U+6118.0	fHakka	 ka5; ka6
U+6118.0	fCantonese	kok3; kaa3
U+6118.0	fMandarin	ke4; qia4
U+6118.0	fEnglish	 hide
U+6119.0	fUTF8	愙
U+6119.0	fCNS11643	3-412B
U+6119.0	fGB	5-532F
U+6119.0	fR/S	 61.9
U+6119.0	fTotalStrokes	 13; 12
U+6119.0	fCangjie	JHRP
U+6119.0	fFourCorner	30330
U+6119.0	fCantonese	kok3
U+6119.0	fMandarin	ke4
U+611A.0	fUTF8	愚
U+611A.0	fCNS11643	1-6373
U+611A.0	fBig5	B74D
U+611A.0	fGB	0-535E
U+611A.0	fR/S	 61.9
U+611A.0	fTotalStrokes	 13
U+611A.0	fCangjie	WBP
U+611A.0	fFourCorner	60332
U+611A.0	fHakka	  ngi2; gni2; gni3
U+611A.0	fCantonese	jyu4
U+611A.0	fMandarin	yu2
U+611A.0	fEnglish	[1] stupid; foolish; silly; unwise; unintelligent [2] fool; cheat; deceive [3] I, me (courteous self-reference)
U+611B.0	fUTF8	愛
U+611B.0	fCNS11643	1-6378
U+611B.0	fBig5	B752
U+611B.0	fGB	1-302E
U+611B.0	fR/S	 61.9
U+611B.0	fTotalStrokes	 13
U+611B.0	fCangjie	BBPE
U+611B.0	fFourCorner	20247
U+611B.0	fHakka	  oi5
U+611B.0	fCantonese	oi3; ngoi3 (also)
U+611B.0	fMandarin	ai4
U+611B.0	fEnglish	[1] love; be fond of; like; be kind to [2] love; affection; kindness; benevolence; likes [3] be apt to
U+611C.0	fUTF8	愜
U+611C.0	fCNS11643	1-5F2E
U+611C.0	fBig5	B467
U+611C.0	fGB	1-632B
U+611C.0	fR/S	 61.9
U+611C.0	fTotalStrokes	 12
U+611C.0	fCangjie	PSKO
U+611C.0	fFourCorner	91013
U+611C.0	fHakka	 giap7; kiap7; hiap7
U+611C.0	fCantonese	hip3; hip6 (also)
U+611C.0	fMandarin	qie4
U+611C.0	fEnglish	satisfied; cheerful; contended
U+611D.0	fUTF8	愝
U+611D.0	fCNS11643	2-3B6F
U+611D.0	fBig5	D94C
U+611D.0	fGB	5-5427
U+611D.0	fR/S	 61.9
U+611D.0	fTotalStrokes	 12
U+611D.0	fCangjie	PSAV
U+611D.0	fFourCorner	91014
U+611D.0	fCantonese	jin2
U+611D.0	fMandarin	yan3
U+611E.0	fUTF8	愞
U+611E.0	fCNS11643	3-3B61
U+611E.0	fGB	5-5429
U+611E.0	fR/S	  61.9
U+611E.0	fTotalStrokes	 12
U+611E.0	fCangjie	PMBK
U+611E.0	fFourCorner	91030
U+611E.0	fHakka	 no5; ngion3
U+611E.0	fCantonese	no6; lo6 (also); jyun5 (also)
U+611E.0	fMandarin	run3; nuan3; nuo4
U+611E.0	fEnglish	timid; apprehensive
U+611F.0	fUTF8	感
U+611F.0	fCNS11643	1-6376
U+611F.0	fBig5	B750
U+611F.0	fGB	0-3850
U+611F.0	fR/S	 61.9
U+611F.0	fTotalStrokes	 13
U+611F.0	fCangjie	IRP
U+611F.0	fFourCorner	53330
U+611F.0	fHakka	  gam3; gam1
U+611F.0	fCantonese	gam2; ham6
U+611F.0	fMandarin	gan3; han4
U+611F.0	fEnglish	[1] feel; find; perceive; sense; respond to [2] emotion; sensation
U+6120.0	fUTF8	愠
U+6120.0	fCNS11643	3-3B62
U+6120.0	fGB	0-6333
U+6120.0	fR/S	 61.9
U+6120.0	fTotalStrokes	 12
U+6120.0	fCangjie	PABT
U+6120.0	fFourCorner	96010
U+6120.0	fHakka	 wun3; vun3
U+6120.0	fCantonese	wan3
U+6120.0	fMandarin	yun4; wen3; yu4
U+6120.0	fEnglish	same as &#x614D;
U+6121.0	fUTF8	愡
U+6121.0	fCNS11643	3-3B63
U+6121.0	fGB	E-2A3E
U+6121.0	fR/S	  61.9
U+6121.0	fTotalStrokes	 12; 13
U+6121.0	fCangjie	PPKP
U+6121.0	fFourCorner	97030
U+6121.0	fHakka	 zung5
U+6121.0	fCantonese	zung2
U+6121.0	fMandarin	zong3
U+6121.0	fEnglish	absent-minded, confused
U+6122.0	fUTF8	愢
U+6122.0	fCNS11643	3-3B50
U+6122.0	fGB	3-5246
U+6122.0	fR/S	  61.9
U+6122.0	fTotalStrokes	 12
U+6122.0	fCangjie	PWP
U+6122.0	fFourCorner	96030
U+6122.0	fHakka	 soi1;  sii1; se1
U+6122.0	fCantonese	soi1
U+6122.0	fMandarin	sai1
U+6122.0	fEnglish	hesitant; modest
U+6123.0	fUTF8	愣
U+6123.0	fCNS11643	1-5F2F
U+6123.0	fBig5	B468
U+6123.0	fGB	0-6336
U+6123.0	fR/S	 61.9
U+6123.0	fTotalStrokes	 12
U+6123.0	fCangjie	PWLS
U+6123.0	fFourCorner	96027
U+6123.0	fCantonese	ling6; ling4 (also); leng1
U+6123.0	fMandarin	leng4; leng2
U+6123.0	fEnglish	[1] dumbfounded; agape with horror; stupefied [2] reckless; irresponsible; rash; rude [3] outspoken
U+6124.0	fUTF8	愤
U+6124.0	fGB	0-375F
U+6124.0	fR/S	 61.9
U+6124.0	fTotalStrokes	 12
U+6124.0	fCangjie	PJTO
U+6124.0	fHakka	 fun3
U+6124.0	fCantonese	fan5
U+6124.0	fMandarin	fen4
U+6124.0	fEnglish	simplified form of &#x61A4;
U+6125.0	fUTF8	愥
U+6125.0	fCNS11643	3-6479
U+6125.0	fGB	E-2A3F
U+6125.0	fR/S	 61.9
U+6125.0	fTotalStrokes	 12
U+6125.0	fCangjie	PTLK
U+6125.0	fFourCorner	9403
U+6125.0	fCantonese	jing1
U+6126.0	fUTF8	愦
U+6126.0	fGB	0-6334
U+6126.0	fR/S	 61.9
U+6126.0	fTotalStrokes	 12
U+6126.0	fCangjie	PLMO
U+6126.0	fHakka	 k'wui5
U+6126.0	fCantonese	kui2; kwui2 (also)
U+6126.0	fMandarin	kui4; gui4; hui3; hui4; kui3
U+6126.0	fEnglish	simplified form of &#x6192;
U+6127.0	fUTF8	愧
U+6127.0	fCNS11643	1-6424
U+6127.0	fBig5	B75C
U+6127.0	fGB	0-4022
U+6127.0	fR/S	 61.10
U+6127.0	fTotalStrokes	 13
U+6127.0	fCangjie	PHUI
U+6127.0	fFourCorner	96013
U+6127.0	fHakka	  kui3; kui5; kwi3; k'wui3
U+6127.0	fCantonese	kwai5; kwai3 (also)
U+6127.0	fMandarin	kui4
U+6127.0	fEnglish	ashamed; conscience-stricken; shameful; abashed; have something on one's conscience
U+6128.0	fUTF8	愨
U+6128.0	fCNS11643	2-4A2A
U+6128.0	fBig5	E1C3
U+6128.0	fGB	5-5332
U+6128.0	fR/S	 61.10
U+6128.0	fTotalStrokes	 14
U+6128.0	fCangjie	GEP
U+6128.0	fFourCorner	47334
U+6128.0	fHakka	 kok7
U+6128.0	fCantonese	kok3
U+6128.0	fMandarin	que4
U+6128.0	fEnglish	 sincerity; honesty cautious
U+6129.0	fUTF8	愩
U+6129.0	fCNS11643	2-4276
U+6129.0	fBig5	DD70
U+6129.0	fGB	3-516D
U+6129.0	fR/S	 61.10
U+6129.0	fTotalStrokes	 13
U+6129.0	fCangjie	PMBC
U+6129.0	fFourCorner	91086
U+6129.0	fCantonese	gung1; gung3; hung2
U+6129.0	fMandarin	gong1; gong4; hong3
U+612A.0	fUTF8	愪
U+612A.0	fCNS11643	3-4129
U+612A.0	fGB	3-5176
U+612A.0	fR/S	  61.10
U+612A.0	fTotalStrokes	 13
U+612A.0	fCangjie	PRBC
U+612A.0	fFourCorner	96080
U+612A.0	fHakka	 run3; jun3; jun2
U+612A.0	fCantonese	wan4
U+612A.0	fMandarin	yun2
U+612A.0	fEnglish	 be grieved; sad
U+612B.0	fUTF8	愫
U+612B.0	fCNS11643	2-426E
U+612B.0	fBig5	DD68
U+612B.0	fGB	0-633A
U+612B.0	fR/S	 61.10
U+612B.0	fTotalStrokes	 13
U+612B.0	fCangjie	PQMF
U+612B.0	fFourCorner	95093
U+612B.0	fHakka	 su5; se5; sii5
U+612B.0	fCantonese	sou3
U+612B.0	fMandarin	su4
U+612B.0	fEnglish	sincerity; honesty
U+612C.0	fUTF8	愬
U+612C.0	fCNS11643	2-4A29
U+612C.0	fBig5	E1C2
U+612C.0	fGB	E-2A40
U+612C.0	fR/S	 61.10
U+612C.0	fTotalStrokes	 14
U+612C.0	fCangjie	TBP
U+612C.0	fFourCorner	87332
U+612C.0	fHakka	 su5; sii5
U+612C.0	fCantonese	saak3; sou3
U+612C.0	fMandarin	su4; se4; shuo4
U+612C.0	fEnglish	[1] tell; complain [2] scared; frightened
U+612D.0	fUTF8	愭
U+612D.0	fCNS11643	4-3D39
U+612D.0	fGB	3-5252
U+612D.0	fR/S	 61.10
U+612D.0	fTotalStrokes	 13
U+612D.0	fCangjie	PJPA
U+612D.0	fFourCorner	94060
U+612D.0	fHakka	 ki2
U+612D.0	fCantonese	kei4
U+612D.0	fMandarin	qi2
U+612E.0	fUTF8	愮
U+612E.0	fCNS11643	2-4272
U+612E.0	fBig5	DD6C
U+612E.0	fGB	3-5257
U+612E.0	fR/S	 61.10
U+612E.0	fTotalStrokes	 13
U+612E.0	fCangjie	PBOU
U+612E.0	fFourCorner	97072
U+612E.0	fHakka	 rau2; jeu2; jau3; jau2
U+612E.0	fCantonese	jiu4
U+612E.0	fMandarin	yao2; yao4
U+612E.0	fEnglish	distressed; agitated
U+612F.0	fUTF8	愯
U+612F.0	fCNS11643	2-4274
U+612F.0	fBig5	DD6E
U+612F.0	fGB	E-2A41
U+612F.0	fR/S	 61.10
U+612F.0	fTotalStrokes	 13
U+612F.0	fCangjie	POGE
U+612F.0	fFourCorner	93047
U+612F.0	fCantonese	sung2
U+612F.0	fMandarin	song3
U+6130.0	fUTF8	愰
U+6130.0	fCNS11643	3-4127
U+6130.0	fGB	3-5254
U+6130.0	fR/S	 61.10
U+6130.0	fTotalStrokes	 13
U+6130.0	fCangjie	PAFU
U+6130.0	fFourCorner	96010
U+6130.0	fHakka	  fong3; fong5
U+6130.0	fCantonese	fong2; jiu6
U+6130.0	fMandarin	huang4; huang3
U+6130.0	fEnglish	 clearness of mind doubtful uncertain
U+6131.0	fUTF8	愱
U+6131.0	fCNS11643	3-412C
U+6131.0	fGB	E-2A42
U+6131.0	fR/S	  61.10
U+6131.0	fTotalStrokes	 13
U+6131.0	fCangjie	PKOK
U+6131.0	fFourCorner	90030
U+6131.0	fCantonese	zat6
U+6131.0	fMandarin	ji2
U+6132.0	fUTF8	愲
U+6132.0	fCNS11643	2-4271
U+6132.0	fBig5	DD6B
U+6132.0	fGB	3-5247
U+6132.0	fR/S	 61.10
U+6132.0	fTotalStrokes	 13
U+6132.0	fCangjie	PBBB
U+6132.0	fFourCorner	97027
U+6132.0	fHakka	 gut7; gwut7
U+6132.0	fCantonese	gwat1
U+6132.0	fMandarin	gu3
U+6132.0	fEnglish	[1] perturbed [2] distressed
U+6133.0	fUTF8	愳
U+6133.0	fCNS11643	3-466F
U+6133.0	fGB	E-2A43
U+6133.0	fR/S	 61.10
U+6133.0	fTotalStrokes	 14
U+6133.0	fCangjie	BUP
U+6133.0	fFourCorner	66330
U+6133.0	fHakka	 ki3; gi3
U+6133.0	fCantonese	geoi6; geoi2
U+6133.0	fMandarin	ju4; ju3; qu2
U+6133.0	fEnglish	same as &#x61FC;
U+6134.0	fUTF8	愴
U+6134.0	fCNS11643	1-6423
U+6134.0	fBig5	B75B
U+6134.0	fGB	1-626B
U+6134.0	fR/S	 61.10
U+6134.0	fTotalStrokes	 13
U+6134.0	fCangjie	POIR
U+6134.0	fFourCorner	98067
U+6134.0	fHakka	 cong1; cong3
U+6134.0	fCantonese	cong3; cong2
U+6134.0	fMandarin	chuang4; chuan4; chuang1; chuang3
U+6134.0	fEnglish	sad; broken-hearted; sorrowful
U+6135.0	fUTF8	愵
U+6135.0	fCNS11643	4-3D38
U+6135.0	fGB	5-5444
U+6135.0	fR/S	  61.10
U+6135.0	fTotalStrokes	 13
U+6135.0	fCangjie	PNMM
U+6135.0	fFourCorner	97020
U+6135.0	fCantonese	nik6; lik6 (also)
U+6135.0	fMandarin	ni4
U+6136.0	fUTF8	愶
U+6136.0	fCNS11643	2-4270
U+6136.0	fBig5	DD6A
U+6136.0	fGB	5-5445
U+6136.0	fR/S	 61.10
U+6136.0	fTotalStrokes	 13
U+6136.0	fCangjie	PKSB
U+6136.0	fFourCorner	94027
U+6136.0	fHakka	  hiap8; hiap7
U+6136.0	fCantonese	hip3
U+6136.0	fMandarin	xie2; xie5
U+6136.0	fEnglish	 intimidate; overawe
U+6137.0	fUTF8	愷
U+6137.0	fCNS11643	1-6427
U+6137.0	fBig5	B75F
U+6137.0	fGB	1-627D
U+6137.0	fR/S	 61.10
U+6137.0	fTotalStrokes	 13
U+6137.0	fCangjie	PUMT
U+6137.0	fFourCorner	92018
U+6137.0	fHakka	  koi3; kai3
U+6137.0	fCantonese	hoi2
U+6137.0	fMandarin	kai3
U+6137.0	fEnglish	joyful; good; kind; gentle
U+6138.0	fUTF8	愸
U+6138.0	fCNS11643	3-466E
U+6138.0	fGB	5-532D
U+6138.0	fR/S	 61.10
U+6138.0	fTotalStrokes	 14
U+6138.0	fCangjie	DSP
U+6138.0	fFourCorner	44330
U+6138.0	fHakka	 zhin3; chang3
U+6138.0	fCantonese	zing2
U+6138.0	fMandarin	zheng3
U+6139.0	fUTF8	愹
U+6139.0	fCNS11643	6-5264
U+6139.0	fGB	E-2A44
U+6139.0	fR/S	 61.10; 61.19
U+6139.0	fTotalStrokes	 13
U+6139.0	fCangjie	PJCR
U+6139.0	fFourCorner	93060
U+6139.0	fCantonese	jung5
U+6139.0	fMandarin	yong3
U+613A.0	fUTF8	愺
U+613A.0	fCNS11643	3-4128
U+613A.0	fGB	3-5241
U+613A.0	fR/S	  61.10
U+613A.0	fTotalStrokes	 13
U+613A.0	fCangjie	PTAJ
U+613A.0	fFourCorner	94040
U+613A.0	fHakka	 cau3
U+613A.0	fCantonese	cou2
U+613A.0	fMandarin	cao3
U+613B.0	fUTF8	愻
U+613B.0	fCNS11643	2-4A39
U+613B.0	fBig5	E1D2
U+613B.0	fGB	5-5278
U+613B.0	fR/S	 61.10
U+613B.0	fTotalStrokes	 14
U+613B.0	fCangjie	NFP
U+613B.0	fFourCorner	12333
U+613B.0	fHakka	  sun5
U+613B.0	fCantonese	seon3
U+613B.0	fMandarin	sun4; xun4
U+613C.0	fUTF8	愼
U+613C.0	fCNS11643	3-466C
U+613C.0	fGB	E-2A45
U+613C.0	fR/S	 61.10
U+613C.0	fTotalStrokes	 13; 14
U+613C.0	fCangjie	PPBC
U+613C.0	fFourCorner	91081
U+613C.0	fHakka	 shim3; sim3; siim3
U+613C.0	fCantonese	san6
U+613C.0	fMandarin	shen4
U+613C.0	fEnglish	cautious; careful; scrupulous; prudent
U+613D.0	fUTF8	愽
U+613D.0	fCNS11643	3-412E
U+613D.0	fGB	E-2A46
U+613D.0	fR/S	  61.10
U+613D.0	fTotalStrokes	 13
U+613D.0	fCangjie	PIBI
U+613D.0	fFourCorner	93040
U+613D.0	fHakka	 bok7
U+613D.0	fCantonese	bok3
U+613D.0	fMandarin	bo2
U+613D.0	fEnglish	gamble, play games; wide, broad
U+613E.0	fUTF8	愾
U+613E.0	fCNS11643	1-6422
U+613E.0	fBig5	B75A
U+613E.0	fGB	1-6269
U+613E.0	fR/S	 61.10
U+613E.0	fTotalStrokes	 13
U+613E.0	fCangjie	POND
U+613E.0	fFourCorner	98017
U+613E.0	fHakka	 koi3;  hi5; kai3
U+613E.0	fCantonese	koi3; hei3; koi3hei3
U+613E.0	fMandarin	kai4; qi4; xi4
U+613E.0	fEnglish	anger; wrath; hatred; enmity
U+613F.0	fUTF8	愿
U+613F.0	fCNS11643	1-6867
U+613F.0	fBig5	BA40
U+613F.0	fGB	0-5438
U+613F.0	fR/S	 61.10
U+613F.0	fTotalStrokes	 14
U+613F.0	fCangjie	MFP
U+613F.0	fFourCorner	71339
U+613F.0	fHakka	 ngien5; gnian6; ngen3; ngien6; ngian5
U+613F.0	fCantonese	jyun6; jyun2 (also)
U+613F.0	fMandarin	yuan4; ruan3
U+613F.0	fEnglish	[1] sincere; honest; faithful; virtuous [2] simplified form of &#x9858;
U+6140.0	fUTF8	慀
U+6140.0	fCNS11643	2-4277
U+6140.0	fBig5	DD71
U+6140.0	fGB	3-5259
U+6140.0	fR/S	 61.10
U+6140.0	fTotalStrokes	 13
U+6140.0	fCangjie	PBVK
U+6140.0	fFourCorner	92034
U+6140.0	fCantonese	hai6; haai4
U+6140.0	fMandarin	xie2; xi4
U+6141.0	fUTF8	慁
U+6141.0	fCNS11643	2-4A2B
U+6141.0	fBig5	E1C4
U+6141.0	fGB	3-5122
U+6141.0	fR/S	 61.10
U+6141.0	fTotalStrokes	 14
U+6141.0	fCangjie	WOP
U+6141.0	fFourCorner	60330
U+6141.0	fHakka	 fun5; fun6
U+6141.0	fCantonese	wan6
U+6141.0	fMandarin	hun4
U+6141.0	fEnglish	[1] disturb; distress; upset [2] disgrace [3] worry
U+6142.0	fUTF8	慂
U+6142.0	fCNS11643	3-466D
U+6142.0	fGB	E-2A47
U+6142.0	fR/S	  61.10
U+6142.0	fTotalStrokes	 14
U+6142.0	fCangjie	EBP
U+6142.0	fFourCorner	37330
U+6142.0	fHakka	  jung3; rung3
U+6142.0	fCantonese	jung2; jung5
U+6142.0	fMandarin	yong3
U+6142.0	fEnglish	 urge; persuade
U+6143.0	fUTF8	慃
U+6143.0	fCNS11643	3-412A
U+6143.0	fGB	3-525A
U+6143.0	fR/S	  61.10
U+6143.0	fTotalStrokes	 13
U+6143.0	fCangjie	PCIM
U+6143.0	fFourCorner	98020
U+6143.0	fHakka	 wung3
U+6143.0	fCantonese	ong2; ngong2 (also)
U+6143.0	fMandarin	yang3
U+6144.0	fUTF8	慄
U+6144.0	fCNS11643	1-637E
U+6144.0	fBig5	B758
U+6144.0	fGB	E-2A48
U+6144.0	fR/S	 61.10
U+6144.0	fTotalStrokes	 13
U+6144.0	fCangjie	PMWD
U+6144.0	fFourCorner	91094
U+6144.0	fHakka	 lit8
U+6144.0	fCantonese	leot6
U+6144.0	fMandarin	li4
U+6144.0	fEnglish	 shiver; shudder; tremble
U+6145.0	fUTF8	慅
U+6145.0	fCNS11643	2-426F
U+6145.0	fBig5	DD69
U+6145.0	fGB	3-5250
U+6145.0	fR/S	 61.10
U+6145.0	fTotalStrokes	 13
U+6145.0	fCangjie	PEII
U+6145.0	fFourCorner	97036
U+6145.0	fHakka	 sau1;  cau5; cau3
U+6145.0	fCantonese	cou2; sou1
U+6145.0	fMandarin	sao1; cao3; cao4; xiao1
U+6145.0	fEnglish	agitated
U+6146.0	fUTF8	慆
U+6146.0	fCNS11643	2-4273
U+6146.0	fBig5	DD6D
U+6146.0	fGB	3-5258
U+6146.0	fR/S	 61.10
U+6146.0	fTotalStrokes	 13
U+6146.0	fCangjie	PBHX
U+6146.0	fFourCorner	92077
U+6146.0	fHakka	  tau1; to1
U+6146.0	fCantonese	tou1
U+6146.0	fMandarin	tao1
U+6146.0	fEnglish	[1] fleeting [2] for an extended period; long [3] delighted; happy [4] suspicious; doubtful [5] cover up; hide
U+6147.0	fUTF8	慇
U+6147.0	fCNS11643	1-6866
U+6147.0	fBig5	B9FE
U+6147.0	fGB	E-2A49
U+6147.0	fR/S	 61.10
U+6147.0	fTotalStrokes	 14
U+6147.0	fCangjie	HEP
U+6147.0	fFourCorner	77334
U+6147.0	fHakka	 jin1; hin2; rin1
U+6147.0	fCantonese	jan1
U+6147.0	fMandarin	yin1; yin3
U+6147.0	fEnglish	[1] mournful; sorrowful [2] regardful; respectful
U+6148.0	fUTF8	慈
U+6148.0	fCNS11643	1-6375
U+6148.0	fBig5	B74F
U+6148.0	fGB	0-3448
U+6148.0	fR/S	 61.10; 61.9
U+6148.0	fTotalStrokes	 14
U+6148.0	fCangjie	TVIP
U+6148.0	fFourCorner	80333
U+6148.0	fHakka	 cu2;  cih2; cih3; ze2
U+6148.0	fCantonese	ci4
U+6148.0	fMandarin	ci2
U+6148.0	fEnglish	[1] kind; charitable; benevolent; benignant; merciful; loving; fond [2] of one's mother; maternal
U+6149.0	fUTF8	慉
U+6149.0	fCNS11643	2-426C
U+6149.0	fBig5	DD66
U+6149.0	fGB	5-5442
U+6149.0	fR/S	 61.10
U+6149.0	fTotalStrokes	 13
U+6149.0	fCangjie	PYVW
U+6149.0	fFourCorner	90063
U+6149.0	fHakka	  hiuk7
U+6149.0	fCantonese	cuk1; huk1; zuk6
U+6149.0	fMandarin	xu4; chu4
U+6149.0	fEnglish	 bring up; raise
U+614A.0	fUTF8	慊
U+614A.0	fCNS11643	2-426D
U+614A.0	fBig5	DD67
U+614A.0	fGB	0-633B
U+614A.0	fR/S	 61.10
U+614A.0	fTotalStrokes	 13
U+614A.0	fCangjie	PTXC
U+614A.0	fFourCorner	98037
U+614A.0	fHakka	  kiap7;  kiam1; hiam2
U+614A.0	fCantonese	him3; hip3; jim4; him2
U+614A.0	fMandarin	qian4; qian3; xian2
U+614A.0	fEnglish	 resent
U+614A.1	fUTF8	慊
U+614A.1	fCNS11643	2-426D
U+614A.1	fBig5	DD67
U+614A.1	fGB	0-633B
U+614A.1	fR/S	 61.10
U+614A.1	fTotalStrokes	 13
U+614A.1	fCangjie	PTXC
U+614A.1	fFourCorner	98037
U+614A.1	fMandarin	qie4
U+614A.1	fEnglish	contended; satisfied; gratified; pleased
U+614B.0	fUTF8	態
U+614B.0	fCNS11643	1-6868
U+614B.0	fBig5	BA41
U+614B.0	fGB	1-4C2C
U+614B.0	fR/S	 61.10
U+614B.0	fTotalStrokes	 14
U+614B.0	fCangjie	IPP
U+614B.0	fFourCorner	21331
U+614B.0	fHakka	  tai5
U+614B.0	fCantonese	taai3
U+614B.0	fMandarin	tai4; tai3
U+614B.0	fEnglish	[1] manner; bearing; carriage; deportment [2] attitude; position [3] situation; condition; circumstances [4] state of matter (physics)
U+614C.0	fUTF8	慌
U+614C.0	fCNS11643	1-637D
U+614C.0	fBig5	B757
U+614C.0	fGB	0-3B45
U+614C.0	fR/S	 61.10
U+614C.0	fTotalStrokes	 13
U+614C.0	fCangjie	PTYU
U+614C.0	fFourCorner	94011
U+614C.0	fHakka	 fong1; fong3
U+614C.0	fCantonese	fong1; fong2
U+614C.0	fMandarin	huang1; huang3; huang4
U+614C.0	fEnglish	 lose self-possession; lose one's head; panic; confused
U+614C.1	fUTF8	慌
U+614C.1	fCNS11643	1-637D
U+614C.1	fBig5	B757
U+614C.1	fGB	0-3B45
U+614C.1	fR/S	 61.10
U+614C.1	fTotalStrokes	 13
U+614C.1	fCangjie	PTYU
U+614C.1	fFourCorner	94011
U+614C.1	fMandarin	huang5
U+614D.0	fUTF8	慍
U+614D.0	fCNS11643	1-6421
U+614D.0	fBig5	B759
U+614D.0	fGB	E-2A4A
U+614D.0	fR/S	 61.10
U+614D.0	fTotalStrokes	 13
U+614D.0	fCangjie	PWOT
U+614D.0	fFourCorner	96017
U+614D.0	fHakka	 wun6; wun3
U+614D.0	fCantonese	wan3; wan2
U+614D.0	fMandarin	yun4; wen3; yu4
U+614D.0	fEnglish	angry; indignant; resentful
U+614E.0	fUTF8	慎
U+614E.0	fCNS11643	1-637C
U+614E.0	fBig5	B756
U+614E.0	fGB	0-4977
U+614E.0	fR/S	 61.10
U+614E.0	fTotalStrokes	 13
U+614E.0	fCangjie	PJBC
U+614E.0	fFourCorner	94081
U+614E.0	fHakka	 shim6; sim3; sin3; sem3
U+614E.0	fCantonese	san6
U+614E.0	fMandarin	shen4; chen2
U+614E.0	fEnglish	act with care; be cautious
U+614F.0	fUTF8	慏
U+614F.0	fCNS11643	2-4275
U+614F.0	fBig5	DD6F
U+614F.0	fGB	E-2A4B
U+614F.0	fR/S	 61.10
U+614F.0	fTotalStrokes	 13
U+614F.0	fCangjie	PBAC
U+614F.0	fFourCorner	97080
U+614F.0	fHakka	 met7; men2; men3
U+614F.0	fCantonese	ming5
U+614F.0	fMandarin	ming3
U+614F.0	fEnglish	reticent
U+6150.0	fUTF8	慐
U+6150.0	fCNS11643	3-647A
U+6150.0	fGB	E-2A4C
U+6150.0	fR/S	 61.10
U+6150.0	fTotalStrokes	 14
U+6150.0	fCangjie	MBCP
U+6150.0	fFourCorner	1033
U+6150.0	fCantonese	gung1
U+6151.0	fUTF8	慑
U+6151.0	fGB	0-4965
U+6151.0	fR/S	 61.10
U+6151.0	fTotalStrokes	 13
U+6151.0	fCangjie	PSJE
U+6151.0	fHakka	 ngiap7; zhap7
U+6151.0	fCantonese	sip3; zip3 (also)
U+6151.0	fMandarin	she4; zhe2
U+6151.0	fEnglish	simplified form of &#x61FE;
U+6152.0	fUTF8	慒
U+6152.0	fCNS11643	2-4A2F
U+6152.0	fBig5	E1C8
U+6152.0	fGB	3-525C
U+6152.0	fR/S	 61.11
U+6152.0	fTotalStrokes	 14
U+6152.0	fCangjie	PTWA
U+6152.0	fFourCorner	94067
U+6152.0	fHakka	 cau2; cung1; zau1
U+6152.0	fCantonese	cung4; zou1; cou4
U+6152.0	fMandarin	cong2; qiu2; zao1
U+6152.0	fEnglish	in confusion
U+6153.0	fUTF8	慓
U+6153.0	fCNS11643	2-4A30
U+6153.0	fBig5	E1C9
U+6153.0	fGB	5-5447
U+6153.0	fR/S	 61.11
U+6153.0	fTotalStrokes	 14
U+6153.0	fCangjie	PMWF
U+6153.0	fFourCorner	91091
U+6153.0	fCantonese	piu3; piu5 (also); piu4 (also)
U+6153.0	fMandarin	piao4; piao1; piao3
U+6154.0	fUTF8	慔
U+6154.0	fCNS11643	2-4A35
U+6154.0	fBig5	E1CE
U+6154.0	fGB	5-543B
U+6154.0	fR/S	 61.11
U+6154.0	fTotalStrokes	 14
U+6154.0	fCangjie	PTAK
U+6154.0	fFourCorner	94034
U+6154.0	fHakka	 mu5; mok7
U+6154.0	fCantonese	mou6; mok6
U+6154.0	fMandarin	mo4; mu4
U+6154.0	fEnglish	 exert oneself
U+6155.0	fUTF8	慕
U+6155.0	fCNS11643	1-6C67
U+6155.0	fBig5	BC7D
U+6155.0	fGB	0-443D
U+6155.0	fR/S	 61.11
U+6155.0	fTotalStrokes	 15
U+6155.0	fCangjie	TAKP
U+6155.0	fFourCorner	44333
U+6155.0	fHakka	  mu5
U+6155.0	fCantonese	mou6
U+6155.0	fMandarin	mu4
U+6155.0	fEnglish	[1] yearn for; long for [2] adore; admire
U+6156.0	fUTF8	慖
U+6156.0	fCNS11643	2-4A3C
U+6156.0	fBig5	E1D5
U+6156.0	fGB	3-522D
U+6156.0	fR/S	 61.11
U+6156.0	fTotalStrokes	 14
U+6156.0	fCangjie	PWIM
U+6156.0	fFourCorner	96000
U+6156.0	fCantonese	gwok3; gok3 (also)
U+6156.0	fMandarin	guo2; gui4
U+6157.0	fUTF8	慗
U+6157.0	fCNS11643	3-4B73
U+6157.0	fGB	3-5125
U+6157.0	fR/S	 61.11
U+6157.0	fTotalStrokes	 15
U+6157.0	fCangjie	DKP
U+6157.0	fFourCorner	58330
U+6157.0	fCantonese	cik1
U+6157.0	fMandarin	chi4
U+6158.0	fUTF8	慘
U+6158.0	fCNS11643	1-686E
U+6158.0	fBig5	BA47
U+6158.0	fGB	1-3252
U+6158.0	fR/S	 61.11
U+6158.0	fTotalStrokes	 14
U+6158.0	fCangjie	PIIH
U+6158.0	fFourCorner	93022
U+6158.0	fHakka	  cam3
U+6158.0	fCantonese	caam2
U+6158.0	fMandarin	can3; can4; cen3
U+6158.0	fEnglish	[1] sad; sorrowful; miserable; tragic [2] cruel; merciless; brutal [3] dark; gloomy; dull [4] disastrously
U+6159.0	fUTF8	慙
U+6159.0	fCNS11643	3-4B7E
U+6159.0	fGB	E-2A4D
U+6159.0	fR/S	 61.11
U+6159.0	fTotalStrokes	 15
U+6159.0	fCangjie	JLP
U+6159.0	fFourCorner	52330
U+6159.0	fHakka	  cam2; ciam3; cam1
U+6159.0	fCantonese	caam4
U+6159.0	fMandarin	can2
U+6159.0	fEnglish	 ashamed; humiliated shameful
U+615A.0	fUTF8	慚
U+615A.0	fCNS11643	1-686D
U+615A.0	fBig5	BA46
U+615A.0	fGB	1-3251
U+615A.0	fR/S	 61.11
U+615A.0	fTotalStrokes	 14
U+615A.0	fCangjie	PJJL
U+615A.0	fFourCorner	92021
U+615A.0	fHakka	  cam2;  cam1
U+615A.0	fCantonese	caam4
U+615A.0	fMandarin	can2
U+615A.0	fEnglish	ashamed; humiliated; mortified
U+615B.0	fUTF8	慛
U+615B.0	fCNS11643	2-4A37
U+615B.0	fBig5	E1D0
U+615B.0	fGB	5-544B
U+615B.0	fR/S	 61.11
U+615B.0	fTotalStrokes	 14
U+615B.0	fCangjie	PUOG
U+615B.0	fFourCorner	92014
U+615B.0	fCantonese	ceoi4
U+615B.0	fMandarin	cui2; cui1
U+615C.0	fUTF8	慜
U+615C.0	fCNS11643	3-4B74
U+615C.0	fGB	5-5337
U+615C.0	fR/S	  61.11
U+615C.0	fTotalStrokes	 15
U+615C.0	fCangjie	OKP
U+615C.0	fFourCorner	88330
U+615C.0	fCantonese	man5
U+615C.0	fMandarin	min3
U+615C.0	fEnglish	name of a monk in tang dynasty
U+615D.0	fUTF8	慝
U+615D.0	fCNS11643	1-6C66
U+615D.0	fBig5	BC7C
U+615D.0	fGB	0-6D2B
U+615D.0	fR/S	 61.11
U+615D.0	fTotalStrokes	 15
U+615D.0	fCangjie	SRP
U+615D.0	fFourCorner	71331
U+615D.0	fHakka	 nit7; tit8; nit8; tit7
U+615D.0	fCantonese	tik1; nik1
U+615D.0	fMandarin	ni4; te4
U+615D.0	fEnglish	 evil idea; vice; iniquity
U+615E.0	fUTF8	慞
U+615E.0	fCNS11643	2-4A2C
U+615E.0	fBig5	E1C5
U+615E.0	fGB	5-544E
U+615E.0	fR/S	 61.11
U+615E.0	fTotalStrokes	 14
U+615E.0	fCangjie	PYTJ
U+615E.0	fFourCorner	90046
U+615E.0	fHakka	 zhong1; zong1
U+615E.0	fCantonese	zoeng1
U+615E.0	fMandarin	zhang1
U+615E.0	fEnglish	terrified
U+615F.0	fUTF8	慟
U+615F.0	fCNS11643	1-686C
U+615F.0	fBig5	BA45
U+615F.0	fGB	1-627A
U+615F.0	fR/S	 61.11
U+615F.0	fTotalStrokes	 14
U+615F.0	fCangjie	PHGS
U+615F.0	fFourCorner	94027
U+615F.0	fHakka	 tung5; tung6; tung3
U+615F.0	fCantonese	dung6
U+615F.0	fMandarin	tong4; dong4
U+615F.0	fEnglish	extreme grief
U+6160.0	fUTF8	慠
U+6160.0	fCNS11643	3-4671
U+6160.0	fGB	5-5438
U+6160.0	fR/S	  61.11
U+6160.0	fTotalStrokes	 14
U+6160.0	fCangjie	PGSK
U+6160.0	fFourCorner	98040
U+6160.0	fHakka	 ngau5
U+6160.0	fCantonese	ngou6; ou6 (also)
U+6160.0	fMandarin	ao4
U+6160.0	fEnglish	proud; arrogant; scornful; rude
U+6161.0	fUTF8	慡
U+6161.0	fCNS11643	2-4A3B
U+6161.0	fBig5	E1D4
U+6161.0	fGB	5-5448
U+6161.0	fR/S	 61.11
U+6161.0	fTotalStrokes	 14
U+6161.0	fCangjie	PKKK
U+6161.0	fFourCorner	94034
U+6161.0	fCantonese	song2
U+6161.0	fMandarin	shuang3
U+6162.0	fUTF8	慢
U+6162.0	fCNS11643	1-686A
U+6162.0	fBig5	BA43
U+6162.0	fGB	0-427D
U+6162.0	fR/S	 61.11
U+6162.0	fTotalStrokes	 14
U+6162.0	fCangjie	PAWE
U+6162.0	fFourCorner	96047
U+6162.0	fHakka	 man5; man3; man6
U+6162.0	fCantonese	maan6; maan2 (also)
U+6162.0	fMandarin	man4; man2; mian4
U+6162.0	fEnglish	[1] slow; sluggish [2] negligent [3] haughty; rude; disrespectful; arrogant; supercilious [4] postpone; defer
U+6163.0	fUTF8	慣
U+6163.0	fCNS11643	1-686B
U+6163.0	fBig5	BA44
U+6163.0	fGB	1-395F
U+6163.0	fR/S	 61.11
U+6163.0	fTotalStrokes	 14
U+6163.0	fCangjie	PWJC
U+6163.0	fFourCorner	97086
U+6163.0	fHakka	 gan5;  kwan5; gon5; gwan5
U+6163.0	fCantonese	gwaan3
U+6163.0	fMandarin	guan4
U+6163.0	fEnglish	[1] habitual; customary; usual; accustomed [2] spoil (a child) [3] be accustomed to; be used to
U+6164.0	fUTF8	慤
U+6164.0	fCNS11643	3-4B7C
U+6164.0	fGB	1-6D28
U+6164.0	fR/S	 61.11
U+6164.0	fTotalStrokes	 15
U+6164.0	fCangjie	GEP
U+6164.0	fFourCorner	47330
U+6164.0	fHakka	 kok7
U+6164.0	fCantonese	kok3
U+6164.0	fMandarin	que4
U+6164.0	fEnglish	sincerity, honesty; modest
U+6165.0	fUTF8	慥
U+6165.0	fCNS11643	2-4A38
U+6165.0	fBig5	E1D1
U+6165.0	fGB	3-5255
U+6165.0	fR/S	 61.11
U+6165.0	fTotalStrokes	 14
U+6165.0	fCangjie	PYHR
U+6165.0	fFourCorner	94036
U+6165.0	fHakka	  cau5; co5
U+6165.0	fCantonese	zou6; cou3 (also)
U+6165.0	fMandarin	zao4; cao4
U+6165.0	fEnglish	sincere; earnest; kindhearted
U+6166.0	fUTF8	慦
U+6166.0	fCNS11643	2-5051
U+6166.0	fBig5	E5AA
U+6166.0	fGB	3-5127
U+6166.0	fR/S	 61.11
U+6166.0	fTotalStrokes	 15
U+6166.0	fCangjie	IKP
U+6166.0	fFourCorner	48334
U+6166.0	fHakka	 giu5
U+6166.0	fCantonese	gau3
U+6166.0	fMandarin	jiu4
U+6166.0	fEnglish	[1] be pleased [2] diligent [3] attentive
U+6167.0	fUTF8	慧
U+6167.0	fCNS11643	1-6C64
U+6167.0	fBig5	BC7A
U+6167.0	fGB	0-3B5B
U+6167.0	fR/S	 61.11
U+6167.0	fTotalStrokes	 15
U+6167.0	fCangjie	QJSMP
U+6167.0	fFourCorner	55337
U+6167.0	fHakka	  fui5; fui6; fi5
U+6167.0	fCantonese	wai6; wai3 (also); wai2 (also); wai4 (also)
U+6167.0	fMandarin	hui4
U+6167.0	fEnglish	bright; intelligent; wise
U+6168.0	fUTF8	慨
U+6168.0	fCNS11643	1-5F35
U+6168.0	fBig5	B46E
U+6168.0	fGB	0-3F2E
U+6168.0	fR/S	 61.9; 61.11
U+6168.0	fTotalStrokes	 12
U+6168.0	fCangjie	PAIU
U+6168.0	fFourCorner	91014
U+6168.0	fHakka	 kai3; koi3; goi3
U+6168.0	fCantonese	koi3; koi2
U+6168.0	fMandarin	kai3; kai4
U+6168.0	fEnglish	[1] sigh emotionally [2] generous; magnanimous
U+6169.0	fUTF8	慩
U+6169.0	fCNS11643	3-4672
U+6169.0	fGB	5-5374
U+6169.0	fR/S	  61.11
U+6169.0	fTotalStrokes	 14
U+6169.0	fCangjie	PYJJ
U+6169.0	fFourCorner	95030
U+6169.0	fCantonese	lin4
U+6169.0	fMandarin	lian2
U+6169.0	fEnglish	same as &#x3981;
U+616A.0	fUTF8	慪
U+616A.0	fCNS11643	2-4A3A
U+616A.0	fBig5	E1D3
U+616A.0	fGB	1-6266
U+616A.0	fR/S	 61.11
U+616A.0	fTotalStrokes	 14
U+616A.0	fCangjie	PSRR
U+616A.0	fFourCorner	91016
U+616A.0	fHakka	 eu1; eu5
U+616A.0	fCantonese	au3; au1 (also); ngau1 (also); ngau3 (also)
U+616A.0	fMandarin	ou4; ou1; kou1
U+616A.0	fEnglish	[1] irritate; exasperate [2] be stingy about something
U+616B.0	fUTF8	慫
U+616B.0	fCNS11643	1-6C6B
U+616B.0	fBig5	BCA3
U+616B.0	fGB	1-4B4B
U+616B.0	fR/S	 61.11
U+616B.0	fTotalStrokes	 15
U+616B.0	fCangjie	HOP
U+616B.0	fFourCorner	28338
U+616B.0	fHakka	 sung3;  ziung3; cung3; siung3
U+616B.0	fCantonese	sung2
U+616B.0	fMandarin	song3
U+616B.0	fEnglish	 instigate; incite
U+616C.0	fUTF8	慬
U+616C.0	fCNS11643	2-4A32
U+616C.0	fBig5	E1CB
U+616C.0	fGB	3-525B
U+616C.0	fR/S	 61.11
U+616C.0	fTotalStrokes	 14
U+616C.0	fCangjie	PTLM
U+616C.0	fFourCorner	94014
U+616C.0	fHakka	  kiun2; giun3
U+616C.0	fCantonese	kan4; kwan4
U+616C.0	fMandarin	jin3; jin4; qin2
U+616C.0	fEnglish	 brave cautious sad
U+616D.0	fUTF8	慭
U+616D.0	fCNS11643	4-496F
U+616D.0	fGB	7-214A
U+616D.0	fR/S	 61.12; 61.11
U+616D.0	fTotalStrokes	 15; 16
U+616D.0	fCangjie	DKP
U+616D.0	fFourCorner	53330
U+616D.0	fCantonese	jan6
U+616D.0	fMandarin	yin4
U+616D.0	fEnglish	 cautious willing but
U+616E.0	fUTF8	慮
U+616E.0	fCNS11643	1-6C65
U+616E.0	fBig5	BC7B
U+616E.0	fGB	1-4247
U+616E.0	fR/S	 61.11
U+616E.0	fTotalStrokes	 15
U+616E.0	fCangjie	YPWP
U+616E.0	fFourCorner	21236
U+616E.0	fHakka	 li5; li6; li3
U+616E.0	fCantonese	leoi6; lou4; luk6
U+616E.0	fMandarin	l4; luu4; lu4
U+616E.0	fEnglish	[1] be concerned; worry about; anxious about [2] consider; take into account
U+616F.0	fUTF8	慯
U+616F.0	fCNS11643	3-466A
U+616F.0	fGB	5-535D
U+616F.0	fR/S	 61.11
U+616F.0	fTotalStrokes	 14
U+616F.0	fCangjie	POAH
U+616F.0	fFourCorner	98020
U+616F.0	fCantonese	soeng1
U+616F.0	fMandarin	shang1; shang4
U+6170.0	fUTF8	慰
U+6170.0	fCNS11643	1-6C6A
U+6170.0	fBig5	BCA2
U+6170.0	fGB	0-4E3F
U+6170.0	fR/S	 61.11
U+6170.0	fTotalStrokes	 15
U+6170.0	fCangjie	SIP
U+6170.0	fFourCorner	74330
U+6170.0	fHakka	 wui3; wi5; wi3; wui6; wui1; wi1; vui3
U+6170.0	fCantonese	wai3
U+6170.0	fMandarin	wei4
U+6170.0	fEnglish	 comfort; console; soothe; assuage; relieve
U+6171.0	fUTF8	慱
U+6171.0	fCNS11643	2-4A2D
U+6171.0	fBig5	E1C6
U+6171.0	fGB	3-5139
U+6171.0	fR/S	 61.11
U+6171.0	fTotalStrokes	 14
U+6171.0	fCangjie	PJII
U+6171.0	fFourCorner	95043
U+6171.0	fHakka	  ton2
U+6171.0	fCantonese	tyun4
U+6171.0	fMandarin	tuan2; quan2
U+6171.0	fEnglish	sad
U+6172.0	fUTF8	慲
U+6172.0	fCNS11643	2-4A31
U+6172.0	fBig5	E1CA
U+6172.0	fGB	5-543A
U+6172.0	fR/S	 61.11
U+6172.0	fTotalStrokes	 14
U+6172.0	fCangjie	PTLB
U+6172.0	fFourCorner	94027
U+6172.0	fHakka	 man2
U+6172.0	fCantonese	mun4
U+6172.0	fMandarin	man2
U+6173.0	fUTF8	慳
U+6173.0	fCNS11643	2-4A2E
U+6173.0	fBig5	E1C7
U+6173.0	fGB	1-6325
U+6173.0	fR/S	 61.11
U+6173.0	fTotalStrokes	 14
U+6173.0	fCangjie	PSEG
U+6173.0	fFourCorner	97014
U+6173.0	fHakka	 gien1; han1; gen1
U+6173.0	fCantonese	haan1; han1
U+6173.0	fMandarin	qian1
U+6173.0	fEnglish	[1] parsimonious; stingy; niggardly; close [2] deficient [3] ] save; be thrifty; be economical
U+6174.0	fUTF8	慴
U+6174.0	fCNS11643	2-4A34
U+6174.0	fBig5	E1CD
U+6174.0	fGB	E-2A4E
U+6174.0	fR/S	 61.11
U+6174.0	fTotalStrokes	 14
U+6174.0	fCangjie	PSMA
U+6174.0	fFourCorner	97062
U+6174.0	fHakka	 ngiap7
U+6174.0	fCantonese	sip3; zip3 (also)
U+6174.0	fMandarin	zhe2; she4; die2; xi2
U+6174.0	fEnglish	terrified; fearful; frightened
U+6175.0	fUTF8	慵
U+6175.0	fCNS11643	1-686F
U+6175.0	fBig5	BA48
U+6175.0	fGB	0-633C
U+6175.0	fR/S	 61.11
U+6175.0	fTotalStrokes	 14
U+6175.0	fCangjie	PILB
U+6175.0	fFourCorner	90027
U+6175.0	fHakka	  jung2; rung2; jung3
U+6175.0	fCantonese	jung4
U+6175.0	fMandarin	yong1; yong2; chong2; yong3
U+6175.0	fEnglish	indolent; idle; lazy
U+6176.0	fUTF8	慶
U+6176.0	fCNS11643	1-6C63
U+6176.0	fBig5	BC79
U+6176.0	fGB	1-476C
U+6176.0	fR/S	 61.11
U+6176.0	fTotalStrokes	 15; 14
U+6176.0	fCangjie	IXE
U+6176.0	fFourCorner	00247
U+6176.0	fHakka	  kin5;  kin1; kiang5; ken1
U+6176.0	fCantonese	hing3; goeng1
U+6176.0	fMandarin	qing4; qiang1; qing1
U+6176.0	fEnglish	[1] congratulate; celebrate; rejoice [2] festivity; blessing; felicity; joy
U+6177.0	fUTF8	慷
U+6177.0	fCNS11643	1-6869
U+6177.0	fBig5	BA42
U+6177.0	fGB	0-3F36
U+6177.0	fR/S	 61.11
U+6177.0	fTotalStrokes	 14
U+6177.0	fCangjie	PILE
U+6177.0	fFourCorner	90032
U+6177.0	fHakka	 kong3; kong1; k'wong3; kong5
U+6177.0	fCantonese	kong2; hong2 (also); hong1; hung2
U+6177.0	fMandarin	kang1; kang3; kang4
U+6177.0	fEnglish	[1] ardent; impassioned [2] generous; magnanimous; liberal; unselfish
U+6178.0	fUTF8	慸
U+6178.0	fCNS11643	3-4C21
U+6178.0	fGB	5-532C
U+6178.0	fR/S	  61.11
U+6178.0	fTotalStrokes	 15
U+6178.0	fCangjie	KPBLP
U+6178.0	fFourCorner	44330
U+6178.0	fHakka	  dai5; di5
U+6178.0	fCantonese	dai3
U+6178.0	fMandarin	di4
U+6178.0	fEnglish	high; exalted
U+6179.0	fUTF8	慹
U+6179.0	fCNS11643	2-5043
U+6179.0	fBig5	E57A
U+6179.0	fGB	5-5274
U+6179.0	fR/S	 61.11
U+6179.0	fTotalStrokes	 15
U+6179.0	fCangjie	GIP
U+6179.0	fFourCorner	45331
U+6179.0	fHakka	 chip7; tsiip7; zhip7; zhit7
U+6179.0	fCantonese	zap1; zip3 (also)
U+6179.0	fMandarin	zhi2; zhe2; ji2; nie1
U+6179.0	fEnglish	[1] lose heart [2] yield
U+617A.0	fUTF8	慺
U+617A.0	fCNS11643	2-4A36
U+617A.0	fBig5	E1CF
U+617A.0	fGB	3-524B
U+617A.0	fR/S	 61.11
U+617A.0	fTotalStrokes	 14
U+617A.0	fCangjie	PLLV
U+617A.0	fFourCorner	95044
U+617A.0	fHakka	  leu2
U+617A.0	fCantonese	lau4
U+617A.0	fMandarin	lou2; luu2; lu2
U+617A.0	fEnglish	[1] diligent [2] contented
U+617B.0	fUTF8	慻
U+617B.0	fCNS11643	3-4668
U+617B.0	fGB	E-2A4F
U+617B.0	fR/S	  61.11
U+617B.0	fTotalStrokes	 14
U+617B.0	fCangjie	PFQU
U+617B.0	fFourCorner	99060
U+617B.0	fHakka	 gian5
U+617B.0	fCantonese	gyun3
U+617B.0	fMandarin	juan4
U+617C.0	fUTF8	慼
U+617C.0	fCNS11643	1-6C69
U+617C.0	fBig5	BCA1
U+617C.0	fGB	E-2A50
U+617C.0	fR/S	 61.11
U+617C.0	fTotalStrokes	 15
U+617C.0	fCangjie	IFP
U+617C.0	fFourCorner	53330
U+617C.0	fHakka	 cit7
U+617C.0	fCantonese	cik1
U+617C.0	fMandarin	qi1
U+617C.0	fEnglish	[1] mournful; woeful [2] ashamed
U+617D.0	fUTF8	慽
U+617D.0	fCNS11643	3-4673
U+617D.0	fGB	E-2A51
U+617D.0	fR/S	 61.11
U+617D.0	fTotalStrokes	 14
U+617D.0	fCangjie	PIHF
U+617D.0	fFourCorner	93050
U+617D.0	fHakka	 cit7
U+617D.0	fCantonese	cik1
U+617D.0	fMandarin	qi1
U+617D.0	fEnglish	 sorrowful; mournful sorrow
U+617E.0	fUTF8	慾
U+617E.0	fCNS11643	1-6C6C
U+617E.0	fBig5	BCA4
U+617E.0	fGB	E-2A52
U+617E.0	fR/S	 61.11
U+617E.0	fTotalStrokes	 15
U+617E.0	fCangjie	COP
U+617E.0	fFourCorner	87338
U+617E.0	fHakka	  juk8; ruk8
U+617E.0	fCantonese	juk6
U+617E.0	fMandarin	yu4
U+617E.0	fEnglish	lust; desire; passion; appetite; greed
U+617F.0	fUTF8	慿
U+617F.0	fCNS11643	7-2271
U+617F.0	fGB	E-2A53
U+617F.0	fR/S	 61.11
U+617F.0	fTotalStrokes	 15; 14
U+617F.0	fCangjie	ITP
U+617F.0	fFourCorner	31330
U+617F.0	fCantonese	pang4
U+617F.0	fMandarin	ping2
U+6180.0	fUTF8	憀
U+6180.0	fCNS11643	2-4A33
U+6180.0	fBig5	E1CC
U+6180.0	fGB	3-5264
U+6180.0	fR/S	 61.11
U+6180.0	fTotalStrokes	 14
U+6180.0	fCangjie	PSMH
U+6180.0	fFourCorner	97022
U+6180.0	fHakka	  liau2
U+6180.0	fCantonese	liu4
U+6180.0	fMandarin	liao2; liu2
U+6180.0	fEnglish	[1] rely; depend [2] disappointed; sad; forlorn [3] clear; easy to understand [4] for the time being
U+6181.0	fUTF8	憁
U+6181.0	fCNS11643	3-466B
U+6181.0	fGB	3-525F
U+6181.0	fR/S	  61.11
U+6181.0	fTotalStrokes	 14; 15
U+6181.0	fCangjie	PHWP
U+6181.0	fFourCorner	96030
U+6181.0	fHakka	 zung3; zung5
U+6181.0	fCantonese	zung2
U+6181.0	fMandarin	zong3; cong1
U+6181.0	fEnglish	disappointed
U+6182.0	fUTF8	憂
U+6182.0	fCNS11643	1-6C68
U+6182.0	fBig5	BC7E
U+6182.0	fGB	1-5347
U+6182.0	fR/S	 61.11
U+6182.0	fTotalStrokes	 15
U+6182.0	fCangjie	MBPHE
U+6182.0	fFourCorner	10247
U+6182.0	fHakka	  ju2; jeu2; riu2; jiu3; jiu1; jiu2
U+6182.0	fCantonese	jau1
U+6182.0	fMandarin	you1; you4
U+6182.0	fEnglish	[1] sad; pensive; grieved; mournful [2] anxiety [3] worry about; concerned about; anxious; apprehensive
U+6183.0	fUTF8	憃
U+6183.0	fCNS11643	2-5042
U+6183.0	fBig5	E579
U+6183.0	fGB	3-5124
U+6183.0	fR/S	 61.11
U+6183.0	fTotalStrokes	 15
U+6183.0	fCangjie	QXP
U+6183.0	fFourCorner	50337
U+6183.0	fHakka	 zhung1; chong1; zung1; cong1
U+6183.0	fCantonese	cung1; zung1 (also)
U+6183.0	fMandarin	chong1; chuang1; chuang4; zhuang4
U+6183.0	fEnglish	foolish; stupid; dull; silly
U+6184.0	fUTF8	憄
U+6184.0	fCNS11643	5-4D2A
U+6184.0	fGB	E-2A54
U+6184.0	fR/S	 61.11
U+6184.0	fTotalStrokes	 15
U+6184.0	fCangjie	XHVP
U+6184.0	fFourCorner	24330
U+6184.0	fCantonese	zi3
U+6184.0	fMandarin	zhi4
U+6185.0	fUTF8	憅
U+6185.0	fCNS11643	3-4C23
U+6185.0	fGB	E-2A55
U+6185.0	fR/S	 61.11
U+6185.0	fTotalStrokes	 15
U+6185.0	fCangjie	HSP
U+6185.0	fFourCorner	24330
U+6185.0	fHakka	 tung5; tung6; tung3
U+6185.0	fCantonese	dung6
U+6185.0	fMandarin	tong4; dong4
U+6185.0	fEnglish	same as &#x615F;
U+6186.0	fUTF8	憆
U+6186.0	fCNS11643	3-4674
U+6186.0	fGB	5-544A
U+6186.0	fR/S	 61.11
U+6186.0	fTotalStrokes	 14
U+6186.0	fCangjie	PFBG
U+6186.0	fFourCorner	99010
U+6186.0	fCantonese	caang1
U+6186.0	fMandarin	tang2; cheng1; zheng4
U+6187.0	fUTF8	憇
U+6187.0	fCNS11643	3-4C24
U+6187.0	fGB	E-2A56
U+6187.0	fR/S	  61.11
U+6187.0	fTotalStrokes	 15
U+6187.0	fCangjie	HTP
U+6187.0	fFourCorner	24330
U+6187.0	fHakka	  hi5; ki5; tiam2
U+6187.0	fCantonese	hei3
U+6187.0	fMandarin	qi4
U+6187.0	fEnglish	rest, take rest
U+6188.0	fUTF8	憈
U+6188.0	fCNS11643	3-4B77
U+6188.0	fGB	5-5449
U+6188.0	fR/S	  61.11
U+6188.0	fTotalStrokes	 15; 14
U+6188.0	fCangjie	PYPM
U+6188.0	fFourCorner	91010
U+6188.0	fCantonese	keoi1
U+6188.0	fMandarin	qu1
U+6189.0	fUTF8	憉
U+6189.0	fCNS11643	2-5047
U+6189.0	fBig5	E57E
U+6189.0	fGB	E-2A57
U+6189.0	fR/S	 61.12
U+6189.0	fTotalStrokes	 15
U+6189.0	fCangjie	PGTH
U+6189.0	fFourCorner	92022
U+6189.0	fCantonese	paang4
U+6189.0	fMandarin	peng2
U+618A.0	fUTF8	憊
U+618A.0	fCNS11643	1-7057
U+618A.0	fBig5	BECE
U+618A.0	fGB	1-3139
U+618A.0	fR/S	 61.12
U+618A.0	fTotalStrokes	 16
U+618A.0	fCangjie	OBP
U+618A.0	fFourCorner	24332
U+618A.0	fHakka	 pai5; pi5; pi3; pi6; pai6; pai3
U+618A.0	fCantonese	bei6; baai6 (also)
U+618A.0	fMandarin	bei4; bi4; bo2
U+618A.0	fEnglish	tired; weary; fatigued; exhausted
U+618B.0	fUTF8	憋
U+618B.0	fCNS11643	2-5041
U+618B.0	fBig5	E578
U+618B.0	fGB	0-316F
U+618B.0	fR/S	 61.12
U+618B.0	fTotalStrokes	 16
U+618B.0	fCangjie	FKP
U+618B.0	fFourCorner	97330
U+618B.0	fHakka	  biet7; piet7; pet7; bet7
U+618B.0	fCantonese	bit3
U+618B.0	fMandarin	bie1; pie1
U+618B.0	fEnglish	[1] suppress inner feelings with efforts [2] feel oppressed
U+618C.0	fUTF8	憌
U+618C.0	fCNS11643	2-572E
U+618C.0	fBig5	E9A3
U+618C.0	fGB	5-532E
U+618C.0	fR/S	 61.12
U+618C.0	fTotalStrokes	 16
U+618C.0	fCangjie	CMP
U+618C.0	fFourCorner	87332
U+618C.0	fCantonese	king4
U+618C.0	fMandarin	chun2; qiong2
U+618D.0	fUTF8	憍
U+618D.0	fCNS11643	2-5050
U+618D.0	fBig5	E5A9
U+618D.0	fGB	5-536B
U+618D.0	fR/S	 61.12
U+618D.0	fTotalStrokes	 15
U+618D.0	fCangjie	PHKB
U+618D.0	fFourCorner	92027
U+618D.0	fHakka	 giau1; kieu1
U+618D.0	fCantonese	giu1; kiu1
U+618D.0	fMandarin	jiao1; qiao2
U+618D.0	fEnglish	self-conceited; arrogant
U+618E.0	fUTF8	憎
U+618E.0	fCNS11643	1-6C70
U+618E.0	fBig5	BCA8
U+618E.0	fGB	0-5477
U+618E.0	fR/S	 61.12; 61.11
U+618E.0	fTotalStrokes	 15
U+618E.0	fCangjie	PCWA
U+618E.0	fFourCorner	98066
U+618E.0	fHakka	  zen1
U+618E.0	fCantonese	zang1
U+618E.0	fMandarin	zeng1
U+618E.0	fEnglish	 hate; detest; abhor; loathe; abominate
U+618F.0	fUTF8	憏
U+618F.0	fCNS11643	3-4B7A
U+618F.0	fGB	5-544D
U+618F.0	fR/S	 61.12; 61.11
U+618F.0	fTotalStrokes	 15; 14
U+618F.0	fCangjie	PBOF
U+618F.0	fFourCorner	97090
U+618F.0	fCantonese	cai3; zai3; lin4
U+618F.0	fMandarin	chi4
U+6190.0	fUTF8	憐
U+6190.0	fCNS11643	1-6C6E
U+6190.0	fBig5	BCA6
U+6190.0	fGB	1-412F
U+6190.0	fR/S	 61.12
U+6190.0	fTotalStrokes	 15
U+6190.0	fCangjie	PFDQ
U+6190.0	fFourCorner	99059
U+6190.0	fHakka	  lin2; lin3; len2; lien2
U+6190.0	fCantonese	lin4
U+6190.0	fMandarin	lian2; lin2
U+6190.0	fEnglish	[1] pity; sympathise; commiserate [2] feel tender regard for [3] touching
U+6191.0	fUTF8	憑
U+6191.0	fCNS11643	1-7055
U+6191.0	fBig5	BECC
U+6191.0	fGB	1-463E
U+6191.0	fR/S	 61.12
U+6191.0	fTotalStrokes	 16
U+6191.0	fCangjie	IFP
U+6191.0	fFourCorner	31332
U+6191.0	fHakka	 pung2;  pin2; ben5; pin3; pen5; pen6
U+6191.0	fCantonese	pang4; bang6 (also); peng1 (also)
U+6191.0	fMandarin	ping2
U+6191.0	fEnglish	[1] lean on [2] depend on; rely upon [3] be based on; go by; base on; take as a basis [4] basis; proof; evidence [5] no matter (what, how, etc.)
U+6192.0	fUTF8	憒
U+6192.0	fCNS11643	2-504D
U+6192.0	fBig5	E5A6
U+6192.0	fGB	1-6334
U+6192.0	fR/S	 61.12
U+6192.0	fTotalStrokes	 15
U+6192.0	fCangjie	PLMC
U+6192.0	fFourCorner	95086
U+6192.0	fHakka	 kui5; kui3; fui3; fi3; kwi5; k'wui5
U+6192.0	fCantonese	kui2; kwui2 (also); kui3
U+6192.0	fMandarin	kui4; gui4; hui3; hui4; kui3
U+6192.0	fEnglish	confused in one's mind; muddleheaded
U+6193.0	fUTF8	憓
U+6193.0	fCNS11643	2-5049
U+6193.0	fBig5	E5A2
U+6193.0	fGB	3-5265
U+6193.0	fR/S	 61.12
U+6193.0	fTotalStrokes	 15
U+6193.0	fCangjie	PJIP
U+6193.0	fFourCorner	95033
U+6193.0	fHakka	 fui5; fi5; fui3
U+6193.0	fCantonese	wai6
U+6193.0	fMandarin	hui4
U+6194.0	fUTF8	憔
U+6194.0	fCNS11643	1-6C74
U+6194.0	fBig5	BCAC
U+6194.0	fGB	0-633E
U+6194.0	fR/S	 61.12
U+6194.0	fTotalStrokes	 15
U+6194.0	fCangjie	POGF
U+6194.0	fFourCorner	90031
U+6194.0	fHakka	  ziau1; zeu1
U+6194.0	fCantonese	ciu4
U+6194.0	fMandarin	qiao2
U+6194.0	fEnglish	worn; emaciated; haggard
U+6195.0	fUTF8	憕
U+6195.0	fCNS11643	3-4B76
U+6195.0	fGB	3-526F
U+6195.0	fR/S	  61.12
U+6195.0	fTotalStrokes	 15
U+6195.0	fCangjie	PNOT
U+6195.0	fFourCorner	92018
U+6195.0	fCantonese	cing4
U+6195.0	fMandarin	cheng2
U+6196.0	fUTF8	憖
U+6196.0	fCNS11643	2-5725
U+6196.0	fBig5	E978
U+6196.0	fGB	3-5126
U+6196.0	fR/S	 61.12
U+6196.0	fTotalStrokes	 16
U+6196.0	fCangjie	DKP
U+6196.0	fFourCorner	43333
U+6196.0	fHakka	 ngiun3; giun3; niun5; ngiun5
U+6196.0	fCantonese	jan6; ngan6
U+6196.0	fMandarin	yin4; xin4; yin2
U+6196.0	fEnglish	[1] cautious [2] willing
U+6197.0	fUTF8	憗
U+6197.0	fCNS11643	3-5068
U+6197.0	fGB	E-2A58
U+6197.0	fR/S	 61.12
U+6197.0	fTotalStrokes	 16
U+6197.0	fCangjie	DKP
U+6197.0	fFourCorner	48330
U+6197.0	fHakka	 ngiun3; giun3; niun5; ngiun5
U+6197.0	fCantonese	jan6
U+6197.0	fMandarin	yin4; xin4; yin2
U+6198.0	fUTF8	憘
U+6198.0	fCNS11643	3-4C28
U+6198.0	fGB	E-2A59
U+6198.0	fR/S	 61.12
U+6198.0	fTotalStrokes	 15
U+6198.0	fCangjie	PGRR
U+6198.0	fFourCorner	94060
U+6198.0	fHakka	 hi3
U+6198.0	fCantonese	hei2
U+6198.0	fMandarin	xi3; xi4
U+6198.0	fEnglish	ancient form of &#x559C;
U+6199.0	fUTF8	憙
U+6199.0	fCNS11643	3-5067
U+6199.0	fGB	3-5128
U+6199.0	fR/S	 61.12
U+6199.0	fTotalStrokes	 16
U+6199.0	fCangjie	GRTP
U+6199.0	fFourCorner	40330
U+6199.0	fHakka	 hi3; hi5
U+6199.0	fCantonese	hei2
U+6199.0	fMandarin	xi3; xi4
U+6199.0	fEnglish	joyful; happy; glad; delighted; pleased
U+619A.0	fUTF8	憚
U+619A.0	fCNS11643	1-6C72
U+619A.0	fBig5	BCAA
U+619A.0	fGB	1-352C
U+619A.0	fR/S	 61.12
U+619A.0	fTotalStrokes	 15
U+619A.0	fCangjie	PRRJ
U+619A.0	fFourCorner	96056
U+619A.0	fHakka	 tan3; tan5
U+619A.0	fCantonese	daan6
U+619A.0	fMandarin	dan4; chan3; chan4; da2; dan3; duo4; tan2
U+619A.0	fEnglish	 dread; shirk; fear
U+619B.0	fUTF8	憛
U+619B.0	fCNS11643	2-5048
U+619B.0	fBig5	E5A1
U+619B.0	fGB	3-5266
U+619B.0	fR/S	 61.12
U+619B.0	fTotalStrokes	 15
U+619B.0	fCangjie	PMWJ
U+619B.0	fFourCorner	91046
U+619B.0	fCantonese	taam4
U+619B.0	fMandarin	tan2; tan4; yan1; yan2
U+619C.0	fUTF8	憜
U+619C.0	fCNS11643	3-4C29
U+619C.0	fGB	5-5450
U+619C.0	fR/S	 61.12
U+619C.0	fTotalStrokes	 15
U+619C.0	fCangjie	PNLB
U+619C.0	fFourCorner	94020
U+619C.0	fCantonese	do6
U+619C.0	fMandarin	duo3; duo4; tuo2
U+619D.0	fUTF8	憝
U+619D.0	fCNS11643	2-5723
U+619D.0	fBig5	E976
U+619D.0	fGB	0-6D2D
U+619D.0	fR/S	 61.12
U+619D.0	fTotalStrokes	 16
U+619D.0	fCangjie	YKP
U+619D.0	fFourCorner	08334
U+619D.0	fHakka	  dui5
U+619D.0	fCantonese	deoi6
U+619D.0	fMandarin	dui4
U+619D.0	fEnglish	[1] wicked or vicious person [2] hate
U+619E.0	fUTF8	憞
U+619E.0	fCNS11643	3-4C25
U+619E.0	fGB	3-526D
U+619E.0	fR/S	 61.12
U+619E.0	fTotalStrokes	 15
U+619E.0	fCangjie	PYDK
U+619E.0	fFourCorner	98040
U+619E.0	fHakka	 dui5
U+619E.0	fCantonese	deoi6
U+619E.0	fMandarin	dun4; dui4
U+619F.0	fUTF8	憟
U+619F.0	fCNS11643	2-504C
U+619F.0	fBig5	E5A5
U+619F.0	fGB	5-5453
U+619F.0	fR/S	 61.12
U+619F.0	fTotalStrokes	 15
U+619F.0	fCangjie	PMWD
U+619F.0	fFourCorner	91094
U+619F.0	fHakka	  siuk7
U+619F.0	fCantonese	suk1
U+619F.0	fMandarin	su4
U+619F.0	fEnglish	[1] play the sycophant [2] toady
U+61A0.0	fUTF8	憠
U+61A0.0	fCNS11643	4-503B
U+61A0.0	fGB	E-2A5A
U+61A0.0	fR/S	  61.12
U+61A0.0	fTotalStrokes	 16
U+61A0.0	fCangjie	XMOP
U+61A0.0	fFourCorner	71230
U+61A0.0	fCantonese	kyut3
U+61A0.0	fMandarin	jue2
U+61A1.0	fUTF8	憡
U+61A1.0	fCNS11643	2-504F
U+61A1.0	fBig5	E5A8
U+61A1.0	fGB	5-5456
U+61A1.0	fR/S	 61.12
U+61A1.0	fTotalStrokes	 15
U+61A1.0	fCangjie	PHDB
U+61A1.0	fFourCorner	98092
U+61A1.0	fCantonese	caak3
U+61A1.0	fMandarin	ce4
U+61A2.0	fUTF8	憢
U+61A2.0	fCNS11643	2-5046
U+61A2.0	fBig5	E57D
U+61A2.0	fGB	5-5369
U+61A2.0	fR/S	 61.12
U+61A2.0	fTotalStrokes	 15
U+61A2.0	fCangjie	PGGU
U+61A2.0	fFourCorner	94011
U+61A2.0	fCantonese	hiu1
U+61A2.0	fMandarin	xiao1
U+61A3.0	fUTF8	憣
U+61A3.0	fCNS11643	3-4B79
U+61A3.0	fGB	3-526A
U+61A3.0	fR/S	  61.12
U+61A3.0	fTotalStrokes	 15
U+61A3.0	fCangjie	PHDW
U+61A3.0	fFourCorner	92060
U+61A3.0	fHakka	 fan1
U+61A3.0	fCantonese	faan1
U+61A3.0	fMandarin	fan2; fan1
U+61A4.0	fUTF8	憤
U+61A4.0	fCNS11643	1-6C73
U+61A4.0	fBig5	BCAB
U+61A4.0	fGB	1-375F
U+61A4.0	fR/S	 61.12
U+61A4.0	fTotalStrokes	 15
U+61A4.0	fCangjie	PJTC
U+61A4.0	fFourCorner	94086
U+61A4.0	fHakka	 fun3; fun5
U+61A4.0	fCantonese	fan5
U+61A4.0	fMandarin	fen4
U+61A4.0	fEnglish	 resent; angry; indignant; indignation
U+61A5.0	fUTF8	憥
U+61A5.0	fCNS11643	4-5037
U+61A5.0	fGB	5-5279
U+61A5.0	fR/S	  61.12
U+61A5.0	fTotalStrokes	 16
U+61A5.0	fCangjie	FFBKP
U+61A5.0	fFourCorner	99330
U+61A5.0	fCantonese	lou4
U+61A5.0	fMandarin	lao2
U+61A6.0	fUTF8	憦
U+61A6.0	fCNS11643	3-4B75
U+61A6.0	fGB	3-5170
U+61A6.0	fR/S	  61.12
U+61A6.0	fTotalStrokes	 15
U+61A6.0	fCangjie	PFFS
U+61A6.0	fFourCorner	99020
U+61A6.0	fHakka	 lau6; lau5
U+61A6.0	fCantonese	lou6
U+61A6.0	fMandarin	lao4
U+61A6.0	fEnglish	 regret; feel remorse
U+61A7.0	fUTF8	憧
U+61A7.0	fCNS11643	1-6C6D
U+61A7.0	fBig5	BCA5
U+61A7.0	fGB	0-633F
U+61A7.0	fR/S	 61.12
U+61A7.0	fTotalStrokes	 15
U+61A7.0	fCangjie	PYTG
U+61A7.0	fFourCorner	90014
U+61A7.0	fHakka	 cung1;  chung1
U+61A7.0	fCantonese	cung1; zung1; tung4
U+61A7.0	fMandarin	chong1; chong2; dong4; tong2; zhuang4
U+61A7.0	fEnglish	[1] irresolute; indecisive [2] yearn; aspire
U+61A8.0	fUTF8	憨
U+61A8.0	fCNS11643	2-5724
U+61A8.0	fBig5	E977
U+61A8.0	fGB	0-3A29
U+61A8.0	fR/S	 61.12
U+61A8.0	fTotalStrokes	 16
U+61A8.0	fCangjie	MKP
U+61A8.0	fFourCorner	18334
U+61A8.0	fHakka	  ham1
U+61A8.0	fCantonese	ham1; hom1
U+61A8.0	fMandarin	han1; han4
U+61A8.0	fEnglish	[1] foolish; silly; stupid [2] naive; straightforward
U+61A9.0	fUTF8	憩
U+61A9.0	fCNS11643	1-7056
U+61A9.0	fBig5	BECD
U+61A9.0	fGB	0-6D2C
U+61A9.0	fR/S	 61.12
U+61A9.0	fTotalStrokes	 16
U+61A9.0	fCangjie	HUP
U+61A9.0	fFourCorner	24337
U+61A9.0	fHakka	 hi5; ki5; k'e5
U+61A9.0	fCantonese	hei3
U+61A9.0	fMandarin	qi4
U+61A9.0	fEnglish	 rest; take rest; repose
U+61AA.0	fUTF8	憪
U+61AA.0	fCNS11643	2-504E
U+61AA.0	fBig5	E5A7
U+61AA.0	fGB	3-517E
U+61AA.0	fR/S	 61.12
U+61AA.0	fTotalStrokes	 15
U+61AA.0	fCangjie	PANB
U+61AA.0	fFourCorner	97020
U+61AA.0	fHakka	 han2;  han3
U+61AA.0	fCantonese	haan4; haan6 (also); haan5
U+61AA.0	fMandarin	xian2; xian4
U+61AA.0	fEnglish	composed; contented
U+61AB.0	fUTF8	憫
U+61AB.0	fCNS11643	1-6C6F
U+61AB.0	fBig5	BCA7
U+61AB.0	fGB	1-4375
U+61AB.0	fR/S	 61.12
U+61AB.0	fTotalStrokes	 15
U+61AB.0	fCangjie	PANK
U+61AB.0	fFourCorner	97020
U+61AB.0	fHakka	 men3; men1; mien3; mien1
U+61AB.0	fCantonese	man5
U+61AB.0	fMandarin	min3
U+61AB.0	fEnglish	[1] pity; commiserate; feel concerned over [2] sorrow; grieve
U+61AC.0	fUTF8	憬
U+61AC.0	fCNS11643	1-6C71
U+61AC.0	fBig5	BCA9
U+61AC.0	fGB	0-633D
U+61AC.0	fR/S	 61.12
U+61AC.0	fTotalStrokes	 15
U+61AC.0	fCangjie	PAYF
U+61AC.0	fFourCorner	96096
U+61AC.0	fHakka	 gin3; kwen3
U+61AC.0	fCantonese	ging2; gwing2
U+61AC.0	fMandarin	jing3; jyong3; qing3
U+61AC.0	fEnglish	 awake; become conscious; realize; come to understand
U+61AD.0	fUTF8	憭
U+61AD.0	fCNS11643	2-504B
U+61AD.0	fBig5	E5A4
U+61AD.0	fGB	3-5267
U+61AD.0	fR/S	 61.12
U+61AD.0	fTotalStrokes	 15
U+61AD.0	fCangjie	PKCF
U+61AD.0	fFourCorner	94096
U+61AD.0	fHakka	 liau3;  liau2
U+61AD.0	fCantonese	liu5; liu4
U+61AD.0	fMandarin	liao2
U+61AD.0	fEnglish	severe; inclement
U+61AD.1	fUTF8	憭
U+61AD.1	fCNS11643	2-504B
U+61AD.1	fBig5	E5A4
U+61AD.1	fGB	3-5267
U+61AD.1	fR/S	 61.12
U+61AD.1	fTotalStrokes	 15
U+61AD.1	fCangjie	PKCF
U+61AD.1	fFourCorner	94096
U+61AD.1	fMandarin	liao3
U+61AD.1	fEnglish	clear; intelligible
U+61AE.0	fUTF8	憮
U+61AE.0	fCNS11643	1-6C75
U+61AE.0	fBig5	BCAD
U+61AE.0	fGB	1-6264
U+61AE.0	fR/S	 61.12
U+61AE.0	fTotalStrokes	 15
U+61AE.0	fCangjie	POTF
U+61AE.0	fFourCorner	98031
U+61AE.0	fHakka	 wu3; vu3
U+61AE.0	fCantonese	mou5
U+61AE.0	fMandarin	wu3; hu1; wu1; xu3
U+61AE.0	fEnglish	regretful; disappointed
U+61AF.0	fUTF8	憯
U+61AF.0	fCNS11643	2-504A
U+61AF.0	fBig5	E5A3
U+61AF.0	fGB	3-5268
U+61AF.0	fR/S	 61.12
U+61AF.0	fTotalStrokes	 15
U+61AF.0	fCangjie	PMUA
U+61AF.0	fFourCorner	91061
U+61AF.0	fHakka	  cam3
U+61AF.0	fCantonese	caam2
U+61AF.0	fMandarin	can3; qian3
U+61AF.0	fEnglish	sorrow; sad; grieved
U+61B0.0	fUTF8	憰
U+61B0.0	fCNS11643	2-5045
U+61B0.0	fBig5	E57C
U+61B0.0	fGB	3-5270
U+61B0.0	fR/S	 61.12
U+61B0.0	fTotalStrokes	 15
U+61B0.0	fCangjie	PNHB
U+61B0.0	fFourCorner	97027
U+61B0.0	fHakka	 kiet7; giet7; get7
U+61B0.0	fCantonese	kyut3; cuk1; gwat1; gwat3
U+61B0.0	fMandarin	jue2
U+61B0.0	fEnglish	 wily; time-serving feign; act the hypocrite
U+61B1.0	fUTF8	憱
U+61B1.0	fCNS11643	2-5044
U+61B1.0	fBig5	E57B
U+61B1.0	fGB	3-526C
U+61B1.0	fR/S	 61.12
U+61B1.0	fTotalStrokes	 15
U+61B1.0	fCangjie	PYFU
U+61B1.0	fFourCorner	93014
U+61B1.0	fCantonese	cuk1; cau3
U+61B1.0	fMandarin	cu4; chou4; zao4
U+61B2.0	fUTF8	憲
U+61B2.0	fCNS11643	1-7054
U+61B2.0	fBig5	BECB
U+61B2.0	fGB	1-4F5C
U+61B2.0	fR/S	 61.12
U+61B2.0	fTotalStrokes	 16; 15
U+61B2.0	fCangjie	JQMP
U+61B2.0	fFourCorner	30336
U+61B2.0	fHakka	 hien5;  hien3; hen5
U+61B2.0	fCantonese	hin3
U+61B2.0	fMandarin	xian4; xian3
U+61B2.0	fEnglish	[1] constitution; statute; law; code; ordinance [2] intelligent [3] reference to superiors
U+61B3.0	fUTF8	憳
U+61B3.0	fCNS11643	2-5052
U+61B3.0	fBig5	E5AB
U+61B3.0	fGB	E-2A5B
U+61B3.0	fR/S	 61.12
U+61B3.0	fTotalStrokes	 15
U+61B3.0	fCangjie	PSMI
U+61B3.0	fFourCorner	97046
U+61B3.0	fHakka	 tam3
U+61B3.0	fCantonese	taam2
U+61B3.0	fMandarin	tan3; qiao1
U+61B3.0	fEnglish	anxious; timid
U+61B4.0	fUTF8	憴
U+61B4.0	fCNS11643	2-5727
U+61B4.0	fBig5	E97A
U+61B4.0	fGB	5-537B
U+61B4.0	fR/S	 61.13
U+61B4.0	fTotalStrokes	 16
U+61B4.0	fCangjie	PRXU
U+61B4.0	fFourCorner	97017
U+61B4.0	fHakka	 shun2; shin2; sun2; siin2
U+61B4.0	fCantonese	sing4
U+61B4.0	fMandarin	sheng2
U+61B4.0	fEnglish	 be careful
U+61B5.0	fUTF8	憵
U+61B5.0	fCNS11643	2-5C6B
U+61B5.0	fBig5	ECE0
U+61B5.0	fGB	5-533C
U+61B5.0	fR/S	 61.13
U+61B5.0	fTotalStrokes	 17
U+61B5.0	fCangjie	SJP
U+61B5.0	fFourCorner	70334
U+61B5.0	fCantonese	pik1; bik1
U+61B5.0	fMandarin	pi1
U+61B5.0	fEnglish	rush
U+61B6.0	fUTF8	憶
U+61B6.0	fCNS11643	1-7059
U+61B6.0	fBig5	BED0
U+61B6.0	fGB	1-5264
U+61B6.0	fR/S	 61.13
U+61B6.0	fTotalStrokes	 16
U+61B6.0	fCangjie	PYTP
U+61B6.0	fFourCorner	90036
U+61B6.0	fHakka	 jit8; ji5; jit7; ri5; et7; rit7; i5
U+61B6.0	fCantonese	jik1
U+61B6.0	fMandarin	yi4
U+61B6.0	fEnglish	 remember; reflect upon memory
U+61B7.0	fUTF8	憷
U+61B7.0	fCNS11643	3-5061
U+61B7.0	fGB	0-6340
U+61B7.0	fR/S	  61.13
U+61B7.0	fTotalStrokes	 16
U+61B7.0	fCangjie	PDDO
U+61B7.0	fFourCorner	94080
U+61B7.0	fHakka	 cu3; co3; ze3; cih3
U+61B7.0	fCantonese	co2; zeot1 (also)
U+61B7.0	fMandarin	chu3; chu4
U+61B7.0	fEnglish	 painful suffering privation
U+61B8.0	fUTF8	憸
U+61B8.0	fCNS11643	2-572D
U+61B8.0	fBig5	E9A2
U+61B8.0	fGB	3-517A
U+61B8.0	fR/S	 61.13
U+61B8.0	fTotalStrokes	 16
U+61B8.0	fCangjie	POMO
U+61B8.0	fFourCorner	98086
U+61B8.0	fHakka	  siam2;  ciam1; siam1
U+61B8.0	fCantonese	cim1; nau4; lau4
U+61B8.0	fMandarin	xian1; qian1; qian3; xian3
U+61B8.0	fEnglish	 flatter crafty; having specious
U+61B9.0	fUTF8	憹
U+61B9.0	fCNS11643	3-5063
U+61B9.0	fGB	3-5169
U+61B9.0	fR/S	  61.13
U+61B9.0	fTotalStrokes	 16
U+61B9.0	fCangjie	PTWV
U+61B9.0	fFourCorner	95030
U+61B9.0	fHakka	  nung2;  nau3
U+61B9.0	fCantonese	nou4; lou4 (also); nou5; lou5 (also); nung4 (also); lung4 (also)
U+61B9.0	fMandarin	nong2; nao2
U+61B9.0	fEnglish	[1] pleased; glad [2] vexed [3] troubled
U+61BA.0	fUTF8	憺
U+61BA.0	fCNS11643	2-572B
U+61BA.0	fBig5	E97E
U+61BA.0	fGB	3-5277
U+61BA.0	fR/S	 61.13
U+61BA.0	fTotalStrokes	 16
U+61BA.0	fCangjie	PNCR
U+61BA.0	fFourCorner	97061
U+61BA.0	fHakka	 tam5; tam6
U+61BA.0	fCantonese	daam6; daam3
U+61BA.0	fMandarin	dan4; tan2
U+61BA.0	fEnglish	[1] content and stable [2] fear
U+61BB.0	fUTF8	憻
U+61BB.0	fCNS11643	3-506A
U+61BB.0	fGB	3-5278
U+61BB.0	fR/S	 61.14; 61.13
U+61BB.0	fTotalStrokes	 16; 17
U+61BB.0	fCangjie	PYWM
U+61BB.0	fFourCorner	90010
U+61BB.0	fCantonese	taan2
U+61BB.0	fMandarin	tan3
U+61BC.0	fUTF8	憼
U+61BC.0	fCNS11643	2-5C6C
U+61BC.0	fBig5	ECE1
U+61BC.0	fGB	5-5339
U+61BC.0	fR/S	 61.13
U+61BC.0	fTotalStrokes	 17
U+61BC.0	fCangjie	TKP
U+61BC.0	fFourCorner	48334
U+61BC.0	fCantonese	ging2; ging3
U+61BC.0	fMandarin	jing3; jiang3; jing4
U+61BD.0	fUTF8	憽
U+61BD.0	fCNS11643	3-5064
U+61BD.0	fGB	E-2A5C
U+61BD.0	fR/S	  61.13
U+61BD.0	fTotalStrokes	 16
U+61BD.0	fCangjie	PTPP
U+61BD.0	fFourCorner	94030
U+61BD.0	fHakka	  sung1; cung1
U+61BD.0	fCantonese	sung1
U+61BD.0	fMandarin	song1
U+61BD.0	fEnglish	intelligent
U+61BE.0	fUTF8	憾
U+61BE.0	fCNS11643	1-705A
U+61BE.0	fBig5	BED1
U+61BE.0	fGB	0-3A36
U+61BE.0	fR/S	 61.13
U+61BE.0	fTotalStrokes	 16
U+61BE.0	fCangjie	PIRP
U+61BE.0	fFourCorner	93035
U+61BE.0	fHakka	 ham5; ham3; ham6
U+61BE.0	fCantonese	ham6
U+61BE.0	fMandarin	han4; dan3
U+61BE.0	fEnglish	regret; remorse; dissatisfaction
U+61BF.0	fUTF8	憿
U+61BF.0	fCNS11643	2-572C
U+61BF.0	fBig5	E9A1
U+61BF.0	fGB	3-5276
U+61BF.0	fR/S	 61.13
U+61BF.0	fTotalStrokes	 16
U+61BF.0	fCangjie	PHSK
U+61BF.0	fFourCorner	98040
U+61BF.0	fHakka	 hiau1; hieu1
U+61BF.0	fCantonese	giu1; gik1; giu2; hiu1
U+61BF.0	fMandarin	jiao1; ji1; jiao3
U+61BF.0	fEnglish	lucky; prosperous
U+61C0.0	fUTF8	懀
U+61C0.0	fCNS11643	3-5066
U+61C0.0	fGB	3-5160
U+61C0.0	fR/S	 61.13
U+61C0.0	fTotalStrokes	 16
U+61C0.0	fCangjie	POMA
U+61C0.0	fFourCorner	98060
U+61C0.0	fCantonese	wai3
U+61C0.0	fMandarin	wai4; wei4
U+61C1.0	fUTF8	懁
U+61C1.0	fCNS11643	2-5729
U+61C1.0	fBig5	E97C
U+61C1.0	fGB	3-5275
U+61C1.0	fR/S	 61.13
U+61C1.0	fTotalStrokes	 16
U+61C1.0	fCangjie	PWLV
U+61C1.0	fFourCorner	90032
U+61C1.0	fHakka	 gen1
U+61C1.0	fCantonese	hyun1; gyun3; gyun1
U+61C1.0	fMandarin	huan2; xuan1; juan4
U+61C1.0	fEnglish	anxious, distressed
U+61C2.0	fUTF8	懂
U+61C2.0	fCNS11643	1-735D
U+61C2.0	fBig5	C0B4
U+61C2.0	fGB	0-362E
U+61C2.0	fR/S	 61.13
U+61C2.0	fTotalStrokes	 16
U+61C2.0	fCangjie	PTHG
U+61C2.0	fFourCorner	94014
U+61C2.0	fHakka	  dung3
U+61C2.0	fCantonese	dung2
U+61C2.0	fMandarin	dong3
U+61C2.0	fEnglish	 understand; comprehend; know
U+61C3.0	fUTF8	懃
U+61C3.0	fCNS11643	2-5C6A
U+61C3.0	fBig5	ECDF
U+61C3.0	fGB	E-2A5D
U+61C3.0	fR/S	 61.13
U+61C3.0	fTotalStrokes	 17; 16
U+61C3.0	fCangjie	TSP
U+61C3.0	fFourCorner	44332
U+61C3.0	fHakka	 kiun2; kin2
U+61C3.0	fCantonese	kan4
U+61C3.0	fMandarin	qin2
U+61C3.0	fEnglish	cordial; hearty
U+61C4.0	fUTF8	懄
U+61C4.0	fCNS11643	3-506C
U+61C4.0	fGB	E-2A5E
U+61C4.0	fR/S	  61.13
U+61C4.0	fTotalStrokes	 16
U+61C4.0	fCangjie	PTMS
U+61C4.0	fFourCorner	94020
U+61C4.0	fHakka	 kiun2
U+61C4.0	fCantonese	kan4; kwan4
U+61C4.0	fMandarin	qin2; jin3; jin4
U+61C4.0	fEnglish	same as &#x616C;
U+61C5.0	fUTF8	懅
U+61C5.0	fCNS11643	2-5726
U+61C5.0	fBig5	E979
U+61C5.0	fGB	3-5273
U+61C5.0	fR/S	 61.13
U+61C5.0	fTotalStrokes	 16
U+61C5.0	fCangjie	PYPO
U+61C5.0	fFourCorner	91032
U+61C5.0	fHakka	  gi3;  ki2; gi5
U+61C5.0	fCantonese	keoi4; geoi6; gei2; ki2; gei3
U+61C5.0	fMandarin	qu2; ju4
U+61C5.0	fEnglish	 bashful ashamed
U+61C6.0	fUTF8	懆
U+61C6.0	fCNS11643	2-5728
U+61C6.0	fBig5	E97B
U+61C6.0	fGB	3-5274
U+61C6.0	fR/S	 61.13
U+61C6.0	fTotalStrokes	 16
U+61C6.0	fCangjie	PRRD
U+61C6.0	fFourCorner	96094
U+61C6.0	fHakka	 cau5
U+61C6.0	fCantonese	cou2; cou3; co3
U+61C6.0	fMandarin	cao3; can3; cao1; cao4; sao1; sao4
U+61C6.0	fEnglish	anxious; apprehensive; uneasy
U+61C7.0	fUTF8	懇
U+61C7.0	fCNS11643	1-735E
U+61C7.0	fBig5	C0B5
U+61C7.0	fGB	1-3F52
U+61C7.0	fR/S	 61.13
U+61C7.0	fTotalStrokes	 17; 16
U+61C7.0	fCangjie	BVP
U+61C7.0	fFourCorner	27333
U+61C7.0	fHakka	  ken3; kien3
U+61C7.0	fCantonese	han2
U+61C7.0	fMandarin	ken3
U+61C7.0	fEnglish	sincere; earnest; cordial
U+61C8.0	fUTF8	懈
U+61C8.0	fCNS11643	1-705C
U+61C8.0	fBig5	BED3
U+61C8.0	fGB	0-5038
U+61C8.0	fR/S	 61.13
U+61C8.0	fTotalStrokes	 16
U+61C8.0	fCangjie	PNBQ
U+61C8.0	fFourCorner	97052
U+61C8.0	fHakka	  hai3;  hai5; hai6
U+61C8.0	fCantonese	haai6; haai5 (also); gaai3 (also)
U+61C8.0	fMandarin	xie4; jie4
U+61C8.0	fEnglish	negligent; relaxed; remiss; inattentive
U+61C9.0	fUTF8	應
U+61C9.0	fCNS11643	1-735C
U+61C9.0	fBig5	C0B3
U+61C9.0	fGB	1-5326
U+61C9.0	fR/S	 61.13
U+61C9.0	fTotalStrokes	 17; 16
U+61C9.0	fCangjie	IOGP
U+61C9.0	fFourCorner	00231
U+61C9.0	fHakka	 jin1; en5; rin5; rin1; rin3; jin3
U+61C9.0	fCantonese	jing3; jing1
U+61C9.0	fMandarin	ying1
U+61C9.0	fEnglish	should; ought to; need; must
U+61C9.1	fUTF8	應
U+61C9.1	fCNS11643	1-735C
U+61C9.1	fBig5	C0B3
U+61C9.1	fGB	1-5326
U+61C9.1	fR/S	 61.13
U+61C9.1	fTotalStrokes	 17; 16
U+61C9.1	fCangjie	IOGP
U+61C9.1	fFourCorner	00231
U+61C9.1	fHakka	 jin5
U+61C9.1	fMandarin	ying4
U+61C9.1	fEnglish	[1] respond to; answer; echo; react to [2] comply with; grant [3] deal with; cope with [4] assent to [5] a Chinese family name
U+61C9.2	fUTF8	應
U+61C9.2	fCNS11643	1-735C
U+61C9.2	fBig5	C0B3
U+61C9.2	fGB	1-5326
U+61C9.2	fR/S	 61.13
U+61C9.2	fTotalStrokes	 17; 16
U+61C9.2	fCangjie	IOGP
U+61C9.2	fFourCorner	00231
U+61C9.2	fMandarin	ying5
U+61CA.0	fUTF8	懊
U+61CA.0	fCNS11643	1-705B
U+61CA.0	fBig5	BED2
U+61CA.0	fGB	0-3043
U+61CA.0	fR/S	 61.13; 61.12
U+61CA.0	fTotalStrokes	 16
U+61CA.0	fCangjie	PHBK
U+61CA.0	fFourCorner	97034
U+61CA.0	fHakka	  au5; au3
U+61CA.0	fCantonese	ou3; ngou3 (also); mau6; jik6
U+61CA.0	fMandarin	ao4; ao3; yu4
U+61CA.0	fEnglish	 regret; resent; regretful; remorseful; resentful
U+61CB.0	fUTF8	懋
U+61CB.0	fCNS11643	1-7360
U+61CB.0	fBig5	C0B7
U+61CB.0	fGB	0-6D2E
U+61CB.0	fR/S	 61.13
U+61CB.0	fTotalStrokes	 17
U+61CB.0	fCangjie	DDP
U+61CB.0	fFourCorner	44339
U+61CB.0	fHakka	 meu5; meu6; miu3
U+61CB.0	fCantonese	mau6
U+61CB.0	fMandarin	mao4; die2; mou4
U+61CB.0	fEnglish	[1] grand; majestic; great [2] trade [3] encourage
U+61CC.0	fUTF8	懌
U+61CC.0	fCNS11643	2-572A
U+61CC.0	fBig5	E97D
U+61CC.0	fGB	1-6278
U+61CC.0	fR/S	 61.13
U+61CC.0	fTotalStrokes	 16
U+61CC.0	fCangjie	PWLJ
U+61CC.0	fFourCorner	96041
U+61CC.0	fHakka	 jit8; rit8
U+61CC.0	fCantonese	jik6
U+61CC.0	fMandarin	yi4
U+61CC.0	fEnglish	enjoy; be glad; be pleased
U+61CD.0	fUTF8	懍
U+61CD.0	fCNS11643	1-7058
U+61CD.0	fBig5	BECF
U+61CD.0	fGB	E-2A5F
U+61CD.0	fR/S	 61.13
U+61CD.0	fTotalStrokes	 16
U+61CD.0	fCangjie	PYWD
U+61CD.0	fFourCorner	90094
U+61CD.0	fHakka	  lim3
U+61CD.0	fCantonese	lam5
U+61CD.0	fMandarin	lin3; jin4; lan3
U+61CD.0	fEnglish	[1] filled with awe; awe-struck [2] inspiring awe; awful
U+61CE.0	fUTF8	懎
U+61CE.0	fCNS11643	3-5062
U+61CE.0	fGB	E-2A60
U+61CE.0	fR/S	  61.13
U+61CE.0	fTotalStrokes	 16
U+61CE.0	fCangjie	PGOW
U+61CE.0	fFourCorner	94060
U+61CE.0	fHakka	 sep7
U+61CE.0	fCantonese	sik1
U+61CE.0	fMandarin	se4
U+61CF.0	fUTF8	懏
U+61CF.0	fCNS11643	3-5065
U+61CF.0	fGB	E-2A61
U+61CF.0	fR/S	  61.13
U+61CF.0	fTotalStrokes	 16
U+61CF.0	fCangjie	POGS
U+61CF.0	fFourCorner	90020
U+61CF.0	fHakka	  zun5
U+61CF.0	fCantonese	zeon3
U+61CF.0	fMandarin	jun4
U+61CF.0	fEnglish	intelligent; quick of apprehension
U+61D0.0	fUTF8	懐
U+61D0.0	fCNS11643	3-506B
U+61D0.0	fGB	E-2A62
U+61D0.0	fR/S	 61.13
U+61D0.0	fTotalStrokes	 16
U+61D0.0	fCangjie	PYWV
U+61D0.0	fFourCorner	90030
U+61D0.0	fHakka	 fai2
U+61D0.0	fCantonese	waai4
U+61D0.0	fMandarin	huai2
U+61D0.0	fEnglish	ancient form of &#x61F7;
U+61D1.0	fUTF8	懑
U+61D1.0	fGB	0-6D2F
U+61D1.0	fR/S	 61.13
U+61D1.0	fTotalStrokes	 17
U+61D1.0	fCangjie	XEBP
U+61D1.0	fHakka	 mun3
U+61D1.0	fCantonese	mun6
U+61D1.0	fMandarin	men4
U+61D1.0	fEnglish	simplified form of &#x61E3;
U+61D2.0	fUTF8	懒
U+61D2.0	fGB	0-4041
U+61D2.0	fR/S	 61.13
U+61D2.0	fTotalStrokes	 16
U+61D2.0	fCangjie	PDLO
U+61D2.0	fHakka	 lan1; lan3
U+61D2.0	fCantonese	laan5
U+61D2.0	fMandarin	lan3; lai4
U+61D2.0	fEnglish	simplified form of &#x61F6;
U+61D3.0	fUTF8	懓
U+61D3.0	fCNS11643	3-6679
U+61D3.0	fGB	E-2A63
U+61D3.0	fR/S	 61.13
U+61D3.0	fTotalStrokes	 16
U+61D3.0	fCangjie	PBBE
U+61D3.0	fFourCorner	9204
U+61D3.0	fCantonese	oi3; ngoi3 (also)
U+61D3.0	fMandarin	ai4
U+61D4.0	fUTF8	懔
U+61D4.0	fGB	0-6341
U+61D4.0	fR/S	 61.13
U+61D4.0	fTotalStrokes	 16
U+61D4.0	fCangjie	PYWF
U+61D4.0	fCantonese	lam5
U+61D4.0	fMandarin	lin3
U+61D4.0	fEnglish	be afraid of
U+61D5.0	fUTF8	懕
U+61D5.0	fCNS11643	4-5B2B
U+61D5.0	fGB	3-303D
U+61D5.0	fR/S	  61.14
U+61D5.0	fTotalStrokes	 18
U+61D5.0	fCangjie	MKP
U+61D5.0	fFourCorner	71333
U+61D5.0	fHakka	 ram5; jam5; jam3
U+61D5.0	fCantonese	jim1
U+61D5.0	fMandarin	yan1
U+61D5.0	fEnglish	[1] sickly; in poor health [2] peaceful; tranquil
U+61D6.0	fUTF8	懖
U+61D6.0	fCNS11643	2-615B
U+61D6.0	fBig5	EFCF
U+61D6.0	fGB	5-5336
U+61D6.0	fR/S	 61.14
U+61D6.0	fTotalStrokes	 18
U+61D6.0	fCangjie	CRP
U+61D6.0	fFourCorner	82336
U+61D6.0	fCantonese	kut3; kwut3 (also)
U+61D6.0	fMandarin	gua1
U+61D7.0	fUTF8	懗
U+61D7.0	fCNS11643	4-5633
U+61D7.0	fGB	E-2A64
U+61D7.0	fR/S	 61.14
U+61D7.0	fTotalStrokes	 17; 16
U+61D7.0	fCangjie	PGCC
U+61D7.0	fFourCorner	94030
U+61D7.0	fHakka	 ha5
U+61D7.0	fCantonese	haa3
U+61D7.0	fMandarin	xia4
U+61D8.0	fUTF8	懘
U+61D8.0	fCNS11643	2-6153
U+61D8.0	fBig5	EFC7
U+61D8.0	fGB	3-512A
U+61D8.0	fR/S	 61.14
U+61D8.0	fTotalStrokes	 18
U+61D8.0	fCangjie	EBP
U+61D8.0	fFourCorner	34332
U+61D8.0	fCantonese	cit3; dai3; ding3
U+61D8.0	fMandarin	chi4; chi3
U+61D9.0	fUTF8	懙
U+61D9.0	fCNS11643	3-5461
U+61D9.0	fGB	3-5131
U+61D9.0	fR/S	 61.14
U+61D9.0	fTotalStrokes	 17
U+61D9.0	fCangjie	PHXC
U+61D9.0	fFourCorner	97080
U+61D9.0	fCantonese	jyu5; jyu4 (also)
U+61D9.0	fMandarin	yu3; yu2
U+61DA.0	fUTF8	懚
U+61DA.0	fCNS11643	3-545F
U+61DA.0	fGB	E-2A65
U+61DA.0	fR/S	 61.14
U+61DA.0	fTotalStrokes	 17
U+61DA.0	fCangjie	PBMP
U+61DA.0	fFourCorner	92030
U+61DA.0	fCantonese	jan3
U+61DA.0	fMandarin	yin4
U+61DB.0	fUTF8	懛
U+61DB.0	fCNS11643	3-545C
U+61DB.0	fGB	5-5458
U+61DB.0	fR/S	  61.14
U+61DB.0	fTotalStrokes	 17
U+61DB.0	fCangjie	PGRG
U+61DB.0	fFourCorner	94014
U+61DB.0	fHakka	  toi2
U+61DB.0	fCantonese	daai1
U+61DB.0	fMandarin	dai1
U+61DB.0	fEnglish	 alarmed; scared stupid
U+61DC.0	fUTF8	懜
U+61DC.0	fCNS11643	3-545E
U+61DC.0	fGB	E-2A66
U+61DC.0	fR/S	  61.14
U+61DC.0	fTotalStrokes	 17
U+61DC.0	fCangjie	PTWN
U+61DC.0	fFourCorner	94020
U+61DC.0	fHakka	 mung3;  mung5; mung6; mung2
U+61DC.0	fCantonese	mung2; mung4 (also); mung5 (also); mung1
U+61DC.0	fMandarin	meng4; meng2
U+61DC.0	fEnglish	dull; stupid; doltish
U+61DD.0	fUTF8	懝
U+61DD.0	fCNS11643	3-545D
U+61DD.0	fGB	E-2A67
U+61DD.0	fR/S	  61.14
U+61DD.0	fTotalStrokes	 17
U+61DD.0	fCangjie	PPKO
U+61DD.0	fFourCorner	97080
U+61DD.0	fCantonese	ngoi6; oi6 (also)
U+61DD.0	fMandarin	ai4
U+61DE.0	fUTF8	懞
U+61DE.0	fCNS11643	2-5C72
U+61DE.0	fBig5	ECE7
U+61DE.0	fGB	1-7856
U+61DE.0	fR/S	 61.14
U+61DE.0	fTotalStrokes	 17
U+61DE.0	fCangjie	PTBO
U+61DE.0	fFourCorner	94032
U+61DE.0	fHakka	 mung3; mung6; mung5
U+61DE.0	fCantonese	mung4; mung5; mung2
U+61DE.0	fMandarin	meng2; meng3; meng1
U+61DF.0	fUTF8	懟
U+61DF.0	fCNS11643	2-6154
U+61DF.0	fBig5	EFC8
U+61DF.0	fGB	1-6D21
U+61DF.0	fR/S	 61.14
U+61DF.0	fTotalStrokes	 18
U+61DF.0	fCangjie	TIP
U+61DF.0	fFourCorner	34330
U+61DF.0	fHakka	 dui5; dui6; chui5
U+61DF.0	fCantonese	deoi6; zeoi6 (also)
U+61DF.0	fMandarin	dui4; zhui4
U+61DF.0	fEnglish	 hate; rancor; resent
U+61E0.0	fUTF8	懠
U+61E0.0	fCNS11643	2-5C6E
U+61E0.0	fBig5	ECE3
U+61E0.0	fGB	3-5164
U+61E0.0	fR/S	 61.14
U+61E0.0	fTotalStrokes	 17
U+61E0.0	fCangjie	PYX
U+61E0.0	fFourCorner	90023
U+61E0.0	fHakka	  zi5; chi2; ci2
U+61E0.0	fCantonese	cai4; zai6; zai3
U+61E0.0	fMandarin	qi2; ci2; ji4
U+61E0.0	fEnglish	angry
U+61E1.0	fUTF8	懡
U+61E1.0	fCNS11643	4-5632
U+61E1.0	fGB	3-527B
U+61E1.0	fR/S	  61.14
U+61E1.0	fTotalStrokes	 17; 16
U+61E1.0	fCangjie	PIDI
U+61E1.0	fFourCorner	90032
U+61E1.0	fCantonese	mo5
U+61E1.0	fMandarin	mo3
U+61E2.0	fUTF8	懢
U+61E2.0	fCNS11643	3-545B
U+61E2.0	fGB	5-543D
U+61E2.0	fR/S	 61.14
U+61E2.0	fTotalStrokes	 17
U+61E2.0	fCangjie	PSIT
U+61E2.0	fFourCorner	98010
U+61E2.0	fHakka	  lam5
U+61E2.0	fCantonese	laam4
U+61E2.0	fMandarin	lan2
U+61E2.0	fEnglish	greedy, covetous
U+61E3.0	fUTF8	懣
U+61E3.0	fCNS11643	1-7642
U+61E3.0	fBig5	C256
U+61E3.0	fGB	1-6D2F
U+61E3.0	fR/S	 61.14
U+61E3.0	fTotalStrokes	 18
U+61E3.0	fCangjie	EBP
U+61E3.0	fFourCorner	34332
U+61E3.0	fHakka	 mun3; mun6; mun5
U+61E3.0	fCantonese	mun6; mun5
U+61E3.0	fMandarin	men4
U+61E3.0	fEnglish	resentful; sullen; sulky
U+61E4.0	fUTF8	懤
U+61E4.0	fCNS11643	2-5C70
U+61E4.0	fBig5	ECE5
U+61E4.0	fGB	3-516B
U+61E4.0	fR/S	 61.14
U+61E4.0	fTotalStrokes	 17
U+61E4.0	fCangjie	PGNI
U+61E4.0	fFourCorner	94041
U+61E4.0	fHakka	 dau3; do3; cu2; chu2
U+61E4.0	fCantonese	cau4
U+61E4.0	fMandarin	chou2; dao3; zhou4
U+61E4.0	fEnglish	grieved; pained
U+61E5.0	fUTF8	懥
U+61E5.0	fCNS11643	2-5C6F
U+61E5.0	fBig5	ECE4
U+61E5.0	fGB	3-527A
U+61E5.0	fR/S	 61.14
U+61E5.0	fTotalStrokes	 17
U+61E5.0	fCangjie	PJBO
U+61E5.0	fFourCorner	94081
U+61E5.0	fHakka	 ti5; chi5; cih5; zhi5
U+61E5.0	fCantonese	zi3; ci3
U+61E5.0	fMandarin	zhi4
U+61E5.0	fEnglish	 angry; enraged; resentful hate
U+61E6.0	fUTF8	懦
U+61E6.0	fCNS11643	1-735F
U+61E6.0	fBig5	C0B6
U+61E6.0	fGB	0-4533
U+61E6.0	fR/S	 61.14
U+61E6.0	fTotalStrokes	 17
U+61E6.0	fCangjie	PMBB
U+61E6.0	fFourCorner	91027
U+61E6.0	fHakka	 no5; no6; lo3
U+61E6.0	fCantonese	no6; lo6 (also)
U+61E6.0	fMandarin	nuo4; nuan4; ru2; ruan3
U+61E6.0	fEnglish	weak; timid; cowardly
U+61E7.0	fUTF8	懧
U+61E7.0	fCNS11643	2-5C6D
U+61E7.0	fBig5	ECE2
U+61E7.0	fGB	E-2A68
U+61E7.0	fR/S	 61.14
U+61E7.0	fTotalStrokes	 17
U+61E7.0	fCangjie	PJPN
U+61E7.0	fFourCorner	93021
U+61E7.0	fCantonese	no6
U+61E7.0	fMandarin	nuo4; nuan4; ru2; ruan3
U+61E8.0	fUTF8	懨
U+61E8.0	fCNS11643	2-5C71
U+61E8.0	fBig5	ECE6
U+61E8.0	fGB	1-627B
U+61E8.0	fR/S	 61.14
U+61E8.0	fTotalStrokes	 17
U+61E8.0	fCangjie	PMAK
U+61E8.0	fFourCorner	91034
U+61E8.0	fHakka	 jam5; jam3
U+61E8.0	fCantonese	jim1
U+61E8.0	fMandarin	yan1
U+61E8.0	fEnglish	same as &#x61D5;
U+61E9.0	fUTF8	懩
U+61E9.0	fCNS11643	2-615C
U+61E9.0	fBig5	EFD0
U+61E9.0	fGB	E-2A69
U+61E9.0	fR/S	 61.15
U+61E9.0	fTotalStrokes	 18
U+61E9.0	fCangjie	PTOV
U+61E9.0	fFourCorner	98032
U+61E9.0	fHakka	 jong3; rong3
U+61E9.0	fCantonese	joeng5
U+61E9.0	fMandarin	yang3
U+61E9.0	fEnglish	 long for
U+61EA.0	fUTF8	懪
U+61EA.0	fCNS11643	2-6158
U+61EA.0	fBig5	EFCC
U+61EA.0	fGB	5-5459
U+61EA.0	fR/S	 61.15
U+61EA.0	fTotalStrokes	 18
U+61EA.0	fCangjie	PATE
U+61EA.0	fFourCorner	96032
U+61EA.0	fCantonese	bok6; bok3
U+61EA.0	fMandarin	bo2; bao2; bo1
U+61EB.0	fUTF8	懫
U+61EB.0	fCNS11643	2-615A
U+61EB.0	fBig5	EFCE
U+61EB.0	fGB	3-5230
U+61EB.0	fR/S	 61.15
U+61EB.0	fTotalStrokes	 18
U+61EB.0	fCangjie	PHLC
U+61EB.0	fFourCorner	92086
U+61EB.0	fHakka	 zhit7; zhi3; tsiit7; zih3; tje5
U+61EB.0	fCantonese	zi3
U+61EB.0	fMandarin	zhi2; zhi4; chi4
U+61EB.0	fEnglish	 enraged resentful; hate desist
U+61EC.0	fUTF8	懬
U+61EC.0	fCNS11643	4-5F60
U+61EC.0	fGB	E-2A6A
U+61EC.0	fR/S	  61.15
U+61EC.0	fTotalStrokes	 19
U+61EC.0	fCangjie	ITMP
U+61EC.0	fFourCorner	00338
U+61EC.0	fCantonese	kwong3; kong3 (also)
U+61EC.0	fMandarin	kuang4
U+61ED.0	fUTF8	懭
U+61ED.0	fCNS11643	2-6155
U+61ED.0	fBig5	EFC9
U+61ED.0	fGB	3-5134
U+61ED.0	fR/S	 61.15
U+61ED.0	fTotalStrokes	 18
U+61ED.0	fCangjie	PITC
U+61ED.0	fFourCorner	90086
U+61ED.0	fHakka	 kong5; kwong3; k'wong3
U+61ED.0	fCantonese	kong3; kwong3 (also)
U+61ED.0	fMandarin	kuang4; gong3; kuang3
U+61ED.0	fEnglish	[1] hate [2] abhor
U+61EE.0	fUTF8	懮
U+61EE.0	fCNS11643	2-6156
U+61EE.0	fBig5	EFCA
U+61EE.0	fGB	3-527C
U+61EE.0	fR/S	 61.15
U+61EE.0	fTotalStrokes	 18
U+61EE.0	fCangjie	PMBE
U+61EE.0	fFourCorner	91047
U+61EE.0	fHakka	 ju2; riu2
U+61EE.0	fCantonese	jau1; jau2; jau4
U+61EE.0	fMandarin	you1; you3
U+61EE.0	fEnglish	calm
U+61EF.0	fUTF8	懯
U+61EF.0	fCNS11643	4-5F61
U+61EF.0	fGB	5-533F
U+61EF.0	fR/S	  61.15
U+61EF.0	fTotalStrokes	 19
U+61EF.0	fCangjie	IKP
U+61EF.0	fFourCorner	58330
U+61EF.0	fHakka	  fu1
U+61EF.0	fCantonese	fu1
U+61EF.0	fMandarin	fu1
U+61EF.0	fEnglish	hasty; urgent
U+61F0.0	fUTF8	懰
U+61F0.0	fCNS11643	2-6159
U+61F0.0	fBig5	EFCD
U+61F0.0	fGB	3-5163
U+61F0.0	fR/S	 61.15
U+61F0.0	fTotalStrokes	 18
U+61F0.0	fCangjie	PHCN
U+61F0.0	fFourCorner	92000
U+61F0.0	fHakka	 liu2;  liu3
U+61F0.0	fCantonese	lau4; lau5; lau2
U+61F0.0	fMandarin	liu2
U+61F0.0	fEnglish	be glad; pleased; delighted
U+61F1.0	fUTF8	懱
U+61F1.0	fCNS11643	2-6157
U+61F1.0	fBig5	EFCB
U+61F1.0	fGB	3-5279
U+61F1.0	fR/S	 61.15
U+61F1.0	fTotalStrokes	 18
U+61F1.0	fCangjie	PTWI
U+61F1.0	fFourCorner	93050
U+61F1.0	fCantonese	mit6
U+61F1.0	fMandarin	mie4
U+61F2.0	fUTF8	懲
U+61F2.0	fCNS11643	1-7834
U+61F2.0	fBig5	C367
U+61F2.0	fGB	1-334D
U+61F2.0	fR/S	 61.15; 61.14
U+61F2.0	fTotalStrokes	 19
U+61F2.0	fCangjie	HKP
U+61F2.0	fFourCorner	28334
U+61F2.0	fHakka	 cin2;  zhin3; zhin1; zihn1; zihn3; zin1
U+61F2.0	fCantonese	cing4
U+61F2.0	fMandarin	cheng2
U+61F2.0	fEnglish	 punish; reprimand; warn; chastise; reprove
U+61F3.0	fUTF8	懳
U+61F3.0	fCNS11643	3-5765
U+61F3.0	fGB	E-2A6B
U+61F3.0	fR/S	  61.15
U+61F3.0	fTotalStrokes	 18; 19
U+61F3.0	fCangjie	PQJP
U+61F3.0	fFourCorner	95030
U+61F3.0	fCantonese	wai6
U+61F4.0	fUTF8	懴
U+61F4.0	fGB	E-2A6C
U+61F4.0	fR/S	 61.15
U+61F4.0	fTotalStrokes	 18
U+61F4.0	fCangjie	PJIM
U+61F4.0	fCantonese	caam3
U+61F4.0	fMandarin	chan4
U+61F4.0	fEnglish	 regret; repent confess sins
U+61F5.0	fUTF8	懵
U+61F5.0	fCNS11643	1-7837
U+61F5.0	fBig5	C36A
U+61F5.0	fGB	0-6342
U+61F5.0	fR/S	 61.16
U+61F5.0	fTotalStrokes	 19
U+61F5.0	fCangjie	PTWU
U+61F5.0	fFourCorner	94062
U+61F5.0	fHakka	  mung3;  mung5; mung6; mung2
U+61F5.0	fCantonese	mung2; mung4 (also); mung5 (also)
U+61F5.0	fMandarin	meng2
U+61F5.0	fEnglish	ignorant
U+61F5.1	fUTF8	懵
U+61F5.1	fCNS11643	1-7837
U+61F5.1	fBig5	C36A
U+61F5.1	fGB	0-6342
U+61F5.1	fR/S	 61.16
U+61F5.1	fTotalStrokes	 19
U+61F5.1	fCangjie	PTWU
U+61F5.1	fFourCorner	94062
U+61F5.1	fMandarin	meng3
U+61F5.1	fEnglish	muddleheaded; confused
U+61F6.0	fUTF8	懶
U+61F6.0	fCNS11643	1-7836
U+61F6.0	fBig5	C369
U+61F6.0	fGB	1-4041
U+61F6.0	fR/S	 61.16
U+61F6.0	fTotalStrokes	 19
U+61F6.0	fCangjie	PDLC
U+61F6.0	fFourCorner	97086
U+61F6.0	fHakka	  lan1;  lan3; nan1; nan3
U+61F6.0	fCantonese	laan5
U+61F6.0	fMandarin	lan3; lai4
U+61F6.0	fEnglish	lazy; listless; indolent; inactive; reluctant; disinclined
U+61F7.0	fUTF8	懷
U+61F7.0	fCNS11643	1-7835
U+61F7.0	fBig5	C368
U+61F7.0	fGB	1-3B33
U+61F7.0	fR/S	 61.16
U+61F7.0	fTotalStrokes	 19
U+61F7.0	fCangjie	PYWV
U+61F7.0	fFourCorner	94032
U+61F7.0	fHakka	  fai2; fai3
U+61F7.0	fCantonese	waai4
U+61F7.0	fMandarin	huai2
U+61F7.0	fEnglish	[1] bosom; breast [2] hold; harbour [3] think of; recollect [4] conceive (a child) [5] mind [6] used in combination [7] a Chinese family name
U+61F8.0	fUTF8	懸
U+61F8.0	fCNS11643	1-796C
U+61F8.0	fBig5	C461
U+61F8.0	fGB	1-507C
U+61F8.0	fR/S	 61.16
U+61F8.0	fTotalStrokes	 20
U+61F8.0	fCangjie	BFP
U+61F8.0	fFourCorner	22339
U+61F8.0	fHakka	  hien2; hian3; jen2; hen2
U+61F8.0	fCantonese	jyun4
U+61F8.0	fMandarin	xuan2
U+61F8.0	fEnglish	[1] hang; be hanged; be hung; suspend; be suspended [2] be in suspension; be in suspense; unsettled; unsolved
U+61F9.0	fUTF8	懹
U+61F9.0	fCNS11643	2-6876
U+61F9.0	fBig5	F44A
U+61F9.0	fGB	3-5321
U+61F9.0	fR/S	 61.17
U+61F9.0	fTotalStrokes	 20
U+61F9.0	fCangjie	PYRV
U+61F9.0	fFourCorner	90032
U+61F9.0	fCantonese	joeng6
U+61F9.0	fMandarin	rang4; rang2; shang4
U+61FA.0	fUTF8	懺
U+61FA.0	fCNS11643	1-796D
U+61FA.0	fBig5	C462
U+61FA.0	fGB	1-6263
U+61FA.0	fR/S	 61.17
U+61FA.0	fTotalStrokes	 20
U+61FA.0	fCangjie	POIM
U+61FA.0	fFourCorner	93050
U+61FA.0	fHakka	  cam5; cam6
U+61FA.0	fCantonese	caam3; cim3
U+61FA.0	fMandarin	chan4
U+61FA.0	fEnglish	 repent; confess one's sins
U+61FB.0	fUTF8	懻
U+61FB.0	fCNS11643	2-654C
U+61FB.0	fBig5	F241
U+61FB.0	fGB	5-545A
U+61FB.0	fR/S	 61.17
U+61FB.0	fTotalStrokes	 20
U+61FB.0	fCangjie	PLPC
U+61FB.0	fFourCorner	94081
U+61FB.0	fHakka	  gi3;  gi5; gi1
U+61FB.0	fCantonese	gei3; keoi5; kei3
U+61FB.0	fMandarin	ji4
U+61FB.0	fEnglish	[1] violent, crafty [2] over-bearing
U+61FC.0	fUTF8	懼
U+61FC.0	fCNS11643	1-7A6A
U+61FC.0	fBig5	C4DF
U+61FC.0	fGB	1-3E65
U+61FC.0	fR/S	 61.18
U+61FC.0	fTotalStrokes	 21
U+61FC.0	fCangjie	PBUG
U+61FC.0	fFourCorner	96014
U+61FC.0	fHakka	 ki3; gi3
U+61FC.0	fCantonese	geoi6; geoi2
U+61FC.0	fMandarin	ju4; ju3; qu2
U+61FC.0	fEnglish	[1] fear; be afraid of; dread; in fear of [2] frighten
U+61FD.0	fUTF8	懽
U+61FD.0	fCNS11643	2-6B5A
U+61FD.0	fBig5	F5CC
U+61FD.0	fGB	E-2A6D
U+61FD.0	fR/S	 61.18
U+61FD.0	fTotalStrokes	 21
U+61FD.0	fCangjie	PTRG
U+61FD.0	fFourCorner	94014
U+61FD.0	fHakka	 fon1; fan1;  kon1
U+61FD.0	fCantonese	fun1
U+61FD.0	fMandarin	huan1; guan4
U+61FD.0	fEnglish	happy; glad; joyous
U+61FE.0	fUTF8	懾
U+61FE.0	fCNS11643	1-7A6B
U+61FE.0	fBig5	C4E0
U+61FE.0	fGB	1-4965
U+61FE.0	fR/S	 61.18
U+61FE.0	fTotalStrokes	 21
U+61FE.0	fCangjie	PSJJ
U+61FE.0	fFourCorner	91041
U+61FE.0	fHakka	 ngiap7; liap7; zap7; zhap7
U+61FE.0	fCantonese	sip3; zip3 (also)
U+61FE.0	fMandarin	she4; zhe2
U+61FE.0	fEnglish	fearful; awe-struck
U+61FF.0	fUTF8	懿
U+61FF.0	fCNS11643	1-7B60
U+61FF.0	fBig5	C574
U+61FF.0	fGB	0-5C32
U+61FF.0	fR/S	 61.18
U+61FF.0	fTotalStrokes	 22
U+61FF.0	fCangjie	GTIOP
U+61FF.0	fFourCorner	47138
U+61FF.0	fHakka	  ji5; ri5; i5
U+61FF.0	fCantonese	ji3
U+61FF.0	fMandarin	yi4; yi2
U+61FF.0	fEnglish	[1] virtuous; fine; good; exemplary [2] having to do with womanly virtue; chaste
U+6200.0	fUTF8	戀
U+6200.0	fCNS11643	1-7C36
U+6200.0	fBig5	C5CA
U+6200.0	fGB	1-4135
U+6200.0	fR/S	 61.19
U+6200.0	fTotalStrokes	 23
U+6200.0	fCangjie	VFP
U+6200.0	fFourCorner	22339
U+6200.0	fHakka	  lien5; lien6; len3; lian6
U+6200.0	fCantonese	lyun2; lyun5 (also)
U+6200.0	fMandarin	lian4
U+6200.0	fEnglish	[1] love (one of the other sex); be in love [2] feel a persistent attachment (for a thing)
U+6201.0	fUTF8	戁
U+6201.0	fCNS11643	2-6F28
U+6201.0	fBig5	F7D9
U+6201.0	fGB	3-5123
U+6201.0	fR/S	 61.19
U+6201.0	fTotalStrokes	 23
U+6201.0	fCangjie	TGP
U+6201.0	fFourCorner	40331
U+6201.0	fHakka	  nan3
U+6201.0	fCantonese	naan5; laan5 (also); naan2
U+6201.0	fMandarin	nan3; ran3
U+6201.0	fEnglish	fear
U+6202.0	fUTF8	戂
U+6202.0	fCNS11643	3-5F46
U+6202.0	fGB	3-5322
U+6202.0	fR/S	 61.19
U+6202.0	fTotalStrokes	 22
U+6202.0	fCangjie	PIDY
U+6202.0	fFourCorner	90010
U+6202.0	fCantonese	mei4
U+6202.0	fMandarin	mi2
U+6203.0	fUTF8	戃
U+6203.0	fCNS11643	2-6F29
U+6203.0	fBig5	F7DA
U+6203.0	fGB	5-543E
U+6203.0	fR/S	 61.20
U+6203.0	fTotalStrokes	 23
U+6203.0	fCangjie	PFBF
U+6203.0	fFourCorner	99031
U+6203.0	fCantonese	tong2; cong2 (also)
U+6203.0	fMandarin	tang3; chang3
U+6204.0	fUTF8	戄
U+6204.0	fCNS11643	2-6F2A
U+6204.0	fBig5	F7DB
U+6204.0	fGB	3-5323
U+6204.0	fR/S	 61.20
U+6204.0	fTotalStrokes	 23
U+6204.0	fCangjie	PBUE
U+6204.0	fFourCorner	96047
U+6204.0	fHakka	  kiok7; wok7
U+6204.0	fCantonese	fok3; koek3
U+6204.0	fMandarin	jue2; ju4
U+6204.0	fEnglish	[1] awe-struck [2] respectful
U+6205.0	fUTF8	戅
U+6205.0	fCNS11643	3-613E
U+6205.0	fGB	5-5343
U+6205.0	fR/S	 61.21
U+6205.0	fTotalStrokes	 25
U+6205.0	fCangjie	YCP
U+6205.0	fFourCorner	01330
U+6205.0	fHakka	 gung5; {ngong5}
U+6205.0	fCantonese	zong3; ngong6 (also); ong6 (also)
U+6205.0	fMandarin	zhuang4; gang4; chong4; han4
U+6206.0	fUTF8	戆
U+6206.0	fGB	0-6D30
U+6206.0	fR/S	 61.21
U+6206.0	fTotalStrokes	 25
U+6206.0	fCangjie	YOP
U+6206.0	fHakka	 gung5; {ngong5}
U+6206.0	fCantonese	zong3; ngong6 (also); ong6 (also)
U+6206.0	fMandarin	gang4; zhuang4; chong4; han4
U+6206.0	fEnglish	simplified form of &#x6207;
U+6207.0	fUTF8	戇
U+6207.0	fCNS11643	2-722A
U+6207.0	fBig5	F9BA
U+6207.0	fGB	1-6D30
U+6207.0	fR/S	 61.24
U+6207.0	fTotalStrokes	 28
U+6207.0	fCangjie	YCP
U+6207.0	fFourCorner	07338
U+6207.0	fHakka	  ngong5;  ong5; gung5; {ngong5}; zhong5; ngong1
U+6207.0	fCantonese	zong3; ngong6 (also); ong6 (also)
U+6207.0	fMandarin	gang4; zhuang4; chong4; han4
U+6207.0	fEnglish	stupid; simple; simple-minded
U+6208.0	fUTF8	戈
U+6208.0	fCNS11643	1-4541
U+6208.0	fBig5	A4E0
U+6208.0	fGB	0-386A
U+6208.0	fR/S	 62.0
U+6208.0	fTotalStrokes	 4
U+6208.0	fCangjie	I
U+6208.0	fFourCorner	53000
U+6208.0	fHakka	  ko1; go1
U+6208.0	fCantonese	gwo1; go1 (also); wo1
U+6208.0	fMandarin	ge1
U+6208.0	fEnglish	[1] halberd; spear; lance; javelin [2] a Chinese family name [3] Kangxi radical 62
U+6209.0	fUTF8	戉
U+6209.0	fCNS11643	2-215D
U+6209.0	fBig5	C97C
U+6209.0	fGB	5-4822
U+6209.0	fR/S	 62.1
U+6209.0	fTotalStrokes	 5
U+6209.0	fCangjie	IV
U+6209.0	fFourCorner	53700
U+6209.0	fHakka	 jet8; zat8; jat8
U+6209.0	fCantonese	jyut6
U+6209.0	fMandarin	yue4
U+6209.0	fEnglish	 battle-axe; large ax
U+620A.0	fUTF8	戊
U+620A.0	fCNS11643	1-4653
U+620A.0	fBig5	A5B3
U+620A.0	fGB	0-4E6C
U+620A.0	fR/S	 62.1
U+620A.0	fTotalStrokes	 5
U+620A.0	fCangjie	IH
U+620A.0	fFourCorner	53200
U+620A.0	fHakka	 wu5; wu6; meu3; vu5
U+620A.0	fCantonese	mou6
U+620A.0	fMandarin	wu4; mou4; mu3
U+620A.0	fEnglish	the fifth of the Ten Celestial stems
U+620B.0	fUTF8	戋
U+620B.0	fGB	0-6A27
U+620B.0	fR/S	 62.1
U+620B.0	fTotalStrokes	 5
U+620B.0	fCangjie	XIJ
U+620B.0	fHakka	 zien1
U+620B.0	fCantonese	zin1
U+620B.0	fMandarin	jian1; can2; chan3; jin1; san1
U+620B.0	fEnglish	simplified form of &#x6214;
U+620C.0	fUTF8	戌
U+620C.0	fCNS11643	1-4827
U+620C.0	fBig5	A6A6
U+620C.0	fGB	0-5067
U+620C.0	fR/S	 62.2
U+620C.0	fTotalStrokes	 6
U+620C.0	fCangjie	IHM
U+620C.0	fFourCorner	53200
U+620C.0	fHakka	  sut7; sut8; su5
U+620C.0	fCantonese	seot1
U+620C.0	fMandarin	xu1; qu5; xu4
U+620C.0	fEnglish	the eleventh of the twelve Terrestrial branches
U+620D.0	fUTF8	戍
U+620D.0	fCNS11643	1-4828
U+620D.0	fBig5	A6A7
U+620D.0	fGB	0-4A79
U+620D.0	fR/S	 62.2
U+620D.0	fTotalStrokes	 6
U+620D.0	fCangjie	IHI
U+620D.0	fFourCorner	53200
U+620D.0	fHakka	 su3; shiu3; shu5; shu3
U+620D.0	fCantonese	syu3
U+620D.0	fMandarin	shu4
U+620D.0	fEnglish	 defend; guard
U+620E.0	fUTF8	戎
U+620E.0	fCNS11643	1-4826
U+620E.0	fBig5	A6A5
U+620E.0	fGB	0-4856
U+620E.0	fR/S	 62.2
U+620E.0	fTotalStrokes	 6
U+620E.0	fCangjie	IJ
U+620E.0	fFourCorner	53400
U+620E.0	fHakka	 jung1; jung2; rung1
U+620E.0	fCantonese	jung4
U+620E.0	fMandarin	rong2
U+620E.0	fEnglish	[1] war; fighting [2] arms; armaments; apparatus of war [3] military affair; army [4] barbarians to the west [5] great [6] you [7] help
U+620F.0	fUTF8	戏
U+620F.0	fCNS11643	3-236B
U+620F.0	fGB	0-4F37
U+620F.0	fR/S	 62.2
U+620F.0	fTotalStrokes	 6
U+620F.0	fCangjie	EI
U+620F.0	fFourCorner	73450
U+620F.0	fHakka	 hi5
U+620F.0	fCantonese	hei3; hei1 (also); fu1; fai1
U+620F.0	fMandarin	xi4; xi1; kui1; suo1; hu1; hui1; yi1
U+620F.0	fEnglish	simplified form of &#x6232;
U+6210.0	fUTF8	成
U+6210.0	fCNS11643	1-4829
U+6210.0	fBig5	A6A8
U+6210.0	fGB	0-3349
U+6210.0	fR/S	 62.3; 62.2
U+6210.0	fTotalStrokes	 7
U+6210.0	fCangjie	IHS
U+6210.0	fFourCorner	53200
U+6210.0	fHakka	 sin2;  shin2; sang2; s'ang2; siin2; shin3; siang2; shang2; zhang5
U+6210.0	fCantonese	sing4; seng4 (also); cing4 (also)
U+6210.0	fMandarin	cheng2
U+6210.0	fEnglish	[1] completed; accomplished; finished; fixed; settled; accomplish; succeed; complete [2] achieve [3] become [4] acceptable; all right [5] able; capable [6] one tenth [7] a Chinese family name
U+6210.1	fUTF8	成
U+6210.1	fCNS11643	1-4829
U+6210.1	fBig5	A6A8
U+6210.1	fGB	0-3349
U+6210.1	fR/S	 62.3; 62.2
U+6210.1	fTotalStrokes	 7
U+6210.1	fCangjie	IHS
U+6210.1	fFourCorner	53200
U+6210.1	fMandarin	cheng5
U+6211.0	fUTF8	我
U+6211.0	fCNS11643	1-4A3C
U+6211.0	fBig5	A7DA
U+6211.0	fGB	0-4E52
U+6211.0	fR/S	 62.3
U+6211.0	fTotalStrokes	 7
U+6211.0	fCangjie	HQI
U+6211.0	fFourCorner	23550
U+6211.0	fHakka	  ngo1; ngai2;  ngo3; nga1; {ngai2}; ngoe2
U+6211.0	fCantonese	ngo5; o5
U+6211.0	fMandarin	wo3; e3
U+6211.0	fEnglish	[1] I; me; my [2] we; our; us [3] self
U+6212.0	fUTF8	戒
U+6212.0	fCNS11643	1-4A3B
U+6212.0	fBig5	A7D9
U+6212.0	fGB	0-3D64
U+6212.0	fR/S	 62.3
U+6212.0	fTotalStrokes	 7
U+6212.0	fCangjie	IT
U+6212.0	fFourCorner	53400
U+6212.0	fHakka	  gai5; kie5; kiai5; giai5
U+6212.0	fCantonese	gaai3
U+6212.0	fMandarin	jie4
U+6212.0	fEnglish	[1] warn; caution; admonish [2] abstain from; refrain from; give up [3] guard against; avoid [4] commandment; Buddhist monastic discipline
U+6213.0	fUTF8	戓
U+6213.0	fCNS11643	3-2625
U+6213.0	fGB	E-2A6E
U+6213.0	fR/S	  62.3
U+6213.0	fTotalStrokes	 7
U+6213.0	fCangjie	IR
U+6213.0	fFourCorner	53600
U+6213.0	fCantonese	gap3
U+6213.0	fMandarin	ge1
U+6214.0	fUTF8	戔
U+6214.0	fCNS11643	2-2670
U+6214.0	fBig5	CCB1
U+6214.0	fGB	1-6A27
U+6214.0	fR/S	 62.4
U+6214.0	fTotalStrokes	 8
U+6214.0	fCangjie	II
U+6214.0	fFourCorner	53503
U+6214.0	fHakka	  zien1; zen1
U+6214.0	fCantonese	zin1
U+6214.0	fMandarin	jian1; can2; chan3; jin1; san1
U+6214.0	fEnglish	small; tiny; little
U+6215.0	fUTF8	戕
U+6215.0	fCNS11643	1-4D51
U+6215.0	fBig5	A9CF
U+6215.0	fGB	0-635E
U+6215.0	fR/S	 62.4
U+6215.0	fTotalStrokes	 8
U+6215.0	fCangjie	VMI
U+6215.0	fFourCorner	23250
U+6215.0	fHakka	  cong2; cong3; ciong2; ciong3
U+6215.0	fCantonese	coeng4
U+6215.0	fMandarin	qiang1; qiang2; zang1; can2; jiang1; qing2
U+6215.0	fEnglish	[1] kill; slay; destroy [2] be injurious
U+6216.0	fUTF8	或
U+6216.0	fCNS11643	1-4D50
U+6216.0	fBig5	A9CE
U+6216.0	fGB	0-3B72
U+6216.0	fR/S	 62.4
U+6216.0	fTotalStrokes	 8
U+6216.0	fCangjie	IRM
U+6216.0	fFourCorner	53100
U+6216.0	fHakka	  fet8; fiet8
U+6216.0	fCantonese	waak6; wik6
U+6216.0	fMandarin	huo4; yu4
U+6216.0	fEnglish	[1] a certain; some [2] or [3] perhaps; maybe; probably
U+6217.0	fUTF8	戗
U+6217.0	fGB	0-6A28
U+6217.0	fR/S	 62.4
U+6217.0	fTotalStrokes	 8
U+6217.0	fCangjie	OUI
U+6217.0	fHakka	 cong3
U+6217.0	fCantonese	coeng3; coeng1
U+6217.0	fMandarin	qiang1; chuang1; qiang4
U+6217.0	fEnglish	simplified form of &#x6227;
U+6218.0	fUTF8	战
U+6218.0	fCNS11643	3-2C63
U+6218.0	fGB	0-553D
U+6218.0	fR/S	  62.5
U+6218.0	fTotalStrokes	 9
U+6218.0	fCangjie	YRI
U+6218.0	fFourCorner	23650
U+6218.0	fHakka	 zhen5
U+6218.0	fCantonese	zin3
U+6218.0	fMandarin	zhan4
U+6218.0	fEnglish	simplified form of &#x6230;
U+6219.0	fUTF8	戙
U+6219.0	fCNS11643	2-2F31
U+6219.0	fBig5	D1AF
U+6219.0	fGB	5-4829
U+6219.0	fR/S	 62.6
U+6219.0	fTotalStrokes	 10
U+6219.0	fCangjie	BRI
U+6219.0	fFourCorner	73250
U+6219.0	fHakka	 tung5; tung6; tung3
U+6219.0	fCantonese	dung6; dong6 (also); dung2 (also); tung2
U+6219.0	fMandarin	dong4; tong2
U+6219.0	fEnglish	boards used in building a boat
U+621A.0	fUTF8	戚
U+621A.0	fCNS11643	1-5A51
U+621A.0	fBig5	B1AD
U+621A.0	fGB	0-465D
U+621A.0	fR/S	 62.7
U+621A.0	fTotalStrokes	 11
U+621A.0	fCangjie	IHYMF
U+621A.0	fFourCorner	53200
U+621A.0	fHakka	  cit7
U+621A.0	fCantonese	cik1; cuk1
U+621A.0	fMandarin	qi1; cu4; zu2
U+621A.0	fEnglish	[1] relatives by marriage [2] sad; mournful; woeful; sorrow; woe [3] battle-ax [4] a Chinese family name
U+621B.0	fUTF8	戛
U+621B.0	fCNS11643	1-5A52
U+621B.0	fBig5	B1AE
U+621B.0	fGB	0-6A29
U+621B.0	fR/S	 62.7
U+621B.0	fTotalStrokes	 11
U+621B.0	fCangjie	MUI
U+621B.0	fFourCorner	10503
U+621B.0	fHakka	  kat7; giet7; get7
U+621B.0	fCantonese	gaat3; aat3; git3
U+621B.0	fMandarin	jia2
U+621B.0	fEnglish	[1] lance [2] tap; strike lightly [3] natural principles
U+621C.0	fUTF8	戜
U+621C.0	fCNS11643	4-316E
U+621C.0	fGB	5-482C
U+621C.0	fR/S	  62.7
U+621C.0	fTotalStrokes	 11
U+621C.0	fCangjie	IRMG
U+621C.0	fFourCorner	53100
U+621C.0	fHakka	 diet7
U+621C.0	fCantonese	dit6
U+621C.0	fMandarin	die2
U+621C.0	fEnglish	 scrape advantageous
U+621D.0	fUTF8	戝
U+621D.0	fCNS11643	4-3170
U+621D.0	fGB	E-2A6F
U+621D.0	fR/S	 62.4; 62.7; 154.4
U+621D.0	fTotalStrokes	 11
U+621D.0	fCangjie	BCI
U+621D.0	fFourCorner	63850
U+621D.0	fHakka	 cet8
U+621D.0	fCantonese	caak3; caak6
U+621D.0	fMandarin	zei2; ze2
U+621D.0	fEnglish	same as &#x8CCA;
U+621E.0	fUTF8	戞
U+621E.0	fCNS11643	3-3B64
U+621E.0	fGB	E-2A70
U+621E.0	fR/S	  62.8
U+621E.0	fTotalStrokes	 12
U+621E.0	fCangjie	MBI
U+621E.0	fFourCorner	10500
U+621E.0	fHakka	 kat7; kat8; get7
U+621E.0	fCantonese	gaat3; aat3; aat1
U+621E.0	fMandarin	jia2
U+621E.0	fEnglish	lance; tap or strike lightly
U+621F.0	fUTF8	戟
U+621F.0	fCNS11643	1-5F3C
U+621F.0	fBig5	B475
U+621F.0	fGB	0-6A2A
U+621F.0	fR/S	 62.8
U+621F.0	fTotalStrokes	 12
U+621F.0	fCangjie	JJI
U+621F.0	fFourCorner	43450
U+621F.0	fHakka	  git7
U+621F.0	fCantonese	gik1
U+621F.0	fMandarin	ji3
U+621F.0	fEnglish	 halberd with crescent blade; two-pronged spear or lance
U+6220.0	fUTF8	戠
U+6220.0	fCNS11643	2-4278
U+6220.0	fBig5	DD72
U+6220.0	fGB	5-7878
U+6220.0	fR/S	 62.9
U+6220.0	fTotalStrokes	 13
U+6220.0	fCangjie	YAI
U+6220.0	fFourCorner	03650
U+6220.0	fHakka	 zhit7; tsiit7
U+6220.0	fCantonese	zik1; zat6
U+6220.0	fMandarin	shi4; chi4; zhi2
U+6220.0	fEnglish	[1] sword [2] potter's clay [3] gather
U+6221.0	fUTF8	戡
U+6221.0	fCNS11643	1-6428
U+6221.0	fBig5	B760
U+6221.0	fGB	0-6A2C
U+6221.0	fR/S	 62.9
U+6221.0	fTotalStrokes	 13
U+6221.0	fCangjie	TVI
U+6221.0	fFourCorner	43750
U+6221.0	fHakka	 kam1
U+6221.0	fCantonese	ham1; zam3
U+6221.0	fMandarin	kan1; zhen3
U+6221.0	fEnglish	 subjugate; subdue; quell kill
U+6222.0	fUTF8	戢
U+6222.0	fCNS11643	1-6429
U+6222.0	fBig5	B761
U+6222.0	fGB	0-6A2B
U+6222.0	fR/S	 62.9
U+6222.0	fTotalStrokes	 13
U+6222.0	fCangjie	RJI
U+6222.0	fFourCorner	63150
U+6222.0	fHakka	  sip8;  cip8
U+6222.0	fCantonese	cap1; zap1
U+6222.0	fMandarin	ji2
U+6222.0	fEnglish	[1] put away; store up [2] fold; cease; restrain
U+6223.0	fUTF8	戣
U+6223.0	fCNS11643	2-427A
U+6223.0	fBig5	DD74
U+6223.0	fGB	3-4635
U+6223.0	fR/S	 62.9
U+6223.0	fTotalStrokes	 13
U+6223.0	fCangjie	NKI
U+6223.0	fFourCorner	13450
U+6223.0	fHakka	 kui2; kwi2; k'wui2
U+6223.0	fCantonese	kwai4
U+6223.0	fMandarin	kui2; gui4
U+6223.0	fEnglish	lance
U+6224.0	fUTF8	戤
U+6224.0	fCNS11643	2-427C
U+6224.0	fBig5	DD76
U+6224.0	fGB	0-6A2E
U+6224.0	fR/S	 62.9
U+6224.0	fTotalStrokes	 13
U+6224.0	fCangjie	NTI
U+6224.0	fFourCorner	13150
U+6224.0	fCantonese	koi3
U+6224.0	fMandarin	gai4
U+6224.0	fEnglish	infringe a trade-mark to pledge an article
U+6225.0	fUTF8	戥
U+6225.0	fCNS11643	2-427B
U+6225.0	fBig5	DD75
U+6225.0	fGB	0-6A2D
U+6225.0	fR/S	 62.9
U+6225.0	fTotalStrokes	 13
U+6225.0	fCangjie	AMI
U+6225.0	fFourCorner	63150
U+6225.0	fHakka	  din3; chin5; cihn5; den3
U+6225.0	fCantonese	dang2; dang6
U+6225.0	fMandarin	deng3
U+6225.0	fEnglish	 small steelyard for weighing gold, jewels, money, etc.
U+6226.0	fUTF8	戦
U+6226.0	fCNS11643	4-3D43
U+6226.0	fGB	E-2A71
U+6226.0	fR/S	 62.8; 62.9
U+6226.0	fTotalStrokes	 13; 12
U+6226.0	fCangjie	FJI
U+6226.0	fFourCorner	23550
U+6226.0	fHakka	 zhen5
U+6226.0	fCantonese	zin3
U+6226.0	fMandarin	zhan4
U+6226.0	fEnglish	simplified form of &#x6230;
U+6227.0	fUTF8	戧
U+6227.0	fCNS11643	2-4A3E
U+6227.0	fBig5	E1D7
U+6227.0	fGB	1-6A28
U+6227.0	fR/S	 62.10
U+6227.0	fTotalStrokes	 14
U+6227.0	fCangjie	ORI
U+6227.0	fFourCorner	83650
U+6227.0	fHakka	 cong1;  cong3
U+6227.0	fCantonese	coeng1; coeng3; cong3; cong1
U+6227.0	fMandarin	qiang1; chuang4; chuang1; chuan4; tiao3
U+6227.0	fEnglish	[1] be broken off; clash [2] against; in an opposite direction [3] ancient form of &#x5275;
U+6227.1	fUTF8	戧
U+6227.1	fCNS11643	2-4A3E
U+6227.1	fBig5	E1D7
U+6227.1	fGB	1-6A28
U+6227.1	fR/S	 62.10
U+6227.1	fTotalStrokes	 14
U+6227.1	fCangjie	ORI
U+6227.1	fFourCorner	83650
U+6227.1	fMandarin	qiang4
U+6227.1	fEnglish	 support; prop up; shore up
U+6228.0	fUTF8	戨
U+6228.0	fCNS11643	3-4676
U+6228.0	fGB	5-4837
U+6228.0	fR/S	  62.10
U+6228.0	fTotalStrokes	 14
U+6228.0	fCangjie	MRI
U+6228.0	fFourCorner	13650
U+6228.0	fCantonese	go1
U+6228.0	fMandarin	ge1
U+6229.0	fUTF8	戩
U+6229.0	fCNS11643	2-4A3D
U+6229.0	fBig5	E1D6
U+6229.0	fGB	1-6A2F
U+6229.0	fR/S	 62.10
U+6229.0	fTotalStrokes	 14
U+6229.0	fCangjie	MAI
U+6229.0	fFourCorner	13650
U+6229.0	fHakka	  zin3; zien3; zen3; tsian3
U+6229.0	fCantonese	zin2
U+6229.0	fMandarin	jian3; jian4
U+6229.0	fEnglish	[1] exterminate [2] blessing [3] full; complete; entirely
U+622A.0	fUTF8	截
U+622A.0	fCNS11643	1-6870
U+622A.0	fBig5	BA49
U+622A.0	fGB	0-3D58
U+622A.0	fR/S	 62.10
U+622A.0	fTotalStrokes	 14
U+622A.0	fCangjie	JIOG
U+622A.0	fFourCorner	43250
U+622A.0	fHakka	 ciet8;  ciet7; cet8; ziet7; cat7
U+622A.0	fCantonese	zit6; zit2 (also)
U+622A.0	fMandarin	jie2
U+622A.0	fEnglish	[1] cut; section; truncate [2] slice; division; section; segment; chunk [3] detain; withhold [4] keep; set in order [5] stop; intercept; close; end
U+622B.0	fUTF8	戫
U+622B.0	fCNS11643	2-4A3F
U+622B.0	fBig5	E1D8
U+622B.0	fGB	5-4F7A
U+622B.0	fR/S	 62.10
U+622B.0	fTotalStrokes	 14
U+622B.0	fCangjie	KBIRM
U+622B.0	fFourCorner	43250
U+622B.0	fCantonese	juk1; wik6
U+622B.0	fMandarin	yu4
U+622C.0	fUTF8	戬
U+622C.0	fCNS11643	3-4675
U+622C.0	fGB	0-6A2F
U+622C.0	fR/S	 62.10
U+622C.0	fTotalStrokes	 14
U+622C.0	fCangjie	MAI
U+622C.0	fFourCorner	13650
U+622C.0	fHakka	 zien3; zin3
U+622C.0	fCantonese	zin2
U+622C.0	fMandarin	jian3; jian4
U+622C.0	fEnglish	simplified form of &#x6229;
U+622D.0	fUTF8	戭
U+622D.0	fCNS11643	2-5053
U+622D.0	fBig5	E5AC
U+622D.0	fGB	3-4637
U+622D.0	fR/S	 62.11
U+622D.0	fTotalStrokes	 15
U+622D.0	fCangjie	JCI
U+622D.0	fFourCorner	33850
U+622D.0	fHakka	 jin3; rin3
U+622D.0	fCantonese	jan5; jin2 (also)
U+622D.0	fMandarin	yan3; yin3; yin4; you3; zhuan4
U+622D.0	fEnglish	spear
U+622E.0	fUTF8	戮
U+622E.0	fCNS11643	1-6C76
U+622E.0	fBig5	BCAE
U+622E.0	fGB	0-423E
U+622E.0	fR/S	 62.11
U+622E.0	fTotalStrokes	 15
U+622E.0	fCangjie	SHI
U+622E.0	fFourCorner	13250
U+622E.0	fHakka	  liuk8; luk8
U+622E.0	fCantonese	luk6
U+622E.0	fMandarin	lu4; liao2; liu4
U+622E.0	fEnglish	[1] slay; kill; massacre; slaughter [2] unite; join
U+622F.0	fUTF8	戯
U+622F.0	fCNS11643	3-4C2C
U+622F.0	fGB	E-2A72
U+622F.0	fR/S	 62.11
U+622F.0	fTotalStrokes	 15
U+622F.0	fCangjie	YCI
U+622F.0	fFourCorner	23250
U+622F.0	fHakka	  hi5
U+622F.0	fCantonese	hei3; fu1; fai1
U+622F.0	fMandarin	xi4; xi1; kui1; suo1; hu1; hui1; yi1
U+622F.0	fEnglish	theatrical play; show
U+6230.0	fUTF8	戰
U+6230.0	fCNS11643	1-705D
U+6230.0	fBig5	BED4
U+6230.0	fGB	1-553D
U+6230.0	fR/S	 62.12
U+6230.0	fTotalStrokes	 16
U+6230.0	fCangjie	RJI
U+6230.0	fFourCorner	63550
U+6230.0	fHakka	 zhan5; zen5; zien5; zan5; zhen5
U+6230.0	fCantonese	zin3
U+6230.0	fMandarin	zhan4
U+6230.0	fEnglish	[1] war; warfare; fighting; battle [2] contest; fight; contend [3] shudder; shiver; tremble [4] a Chinese family name
U+6231.0	fUTF8	戱
U+6231.0	fCNS11643	3-506D
U+6231.0	fGB	E-2A73
U+6231.0	fR/S	 62.11
U+6231.0	fTotalStrokes	 16; 15
U+6231.0	fCangjie	YMI
U+6231.0	fFourCorner	23240
U+6231.0	fCantonese	hei3; fu1
U+6231.0	fMandarin	xi4; xi1; hu1; hui1
U+6231.0	fEnglish	theatrical play
U+6232.0	fUTF8	戲
U+6232.0	fCNS11643	1-7361
U+6232.0	fBig5	C0B8
U+6232.0	fGB	1-4F37
U+6232.0	fR/S	 62.13
U+6232.0	fTotalStrokes	 17
U+6232.0	fCangjie	YTI
U+6232.0	fFourCorner	23250
U+6232.0	fHakka	  hi5
U+6232.0	fCantonese	hei3; fu1; hei1 (also); fai1
U+6232.0	fMandarin	xi4; xi1; kui1; suo1; yi1
U+6232.0	fEnglish	[1] play; toy; sport [2] jest; have fun; make fun [3] theatrical play; show; drama; play
U+6232.1	fUTF8	戲
U+6232.1	fCNS11643	1-7361
U+6232.1	fBig5	C0B8
U+6232.1	fGB	1-4F37
U+6232.1	fR/S	 62.13
U+6232.1	fTotalStrokes	 17
U+6232.1	fCangjie	YTI
U+6232.1	fFourCorner	23250
U+6232.1	fMandarin	hu1
U+6232.1	fEnglish	alas; oh; o; ah
U+6233.0	fUTF8	戳
U+6233.0	fCNS11643	1-7643
U+6233.0	fBig5	C257
U+6233.0	fGB	0-3441
U+6233.0	fR/S	 62.14
U+6233.0	fTotalStrokes	 18
U+6233.0	fCangjie	SGI
U+6233.0	fFourCorner	13250
U+6233.0	fHakka	  cok7; cok8
U+6233.0	fCantonese	coek3; cok3 (also); zoek3
U+6233.0	fMandarin	chuo1
U+6233.0	fEnglish	[1] prick; stab; jab; poke; pierce [2] stamp; seal; chop
U+6234.0	fUTF8	戴
U+6234.0	fCNS11643	1-7362
U+6234.0	fBig5	C0B9
U+6234.0	fGB	0-3477
U+6234.0	fR/S	 62.14; 62.13
U+6234.0	fTotalStrokes	 18
U+6234.0	fCangjie	JIWTC
U+6234.0	fFourCorner	43850
U+6234.0	fHakka	  dai5
U+6234.0	fCantonese	daai3
U+6234.0	fMandarin	dai4; zai4
U+6234.0	fEnglish	[1] wear on the head; put on [2] support; sustain; bear [3] a Chinese family name [4] respect; honor
U+6235.0	fUTF8	戵
U+6235.0	fCNS11643	3-5F48
U+6235.0	fGB	5-483F
U+6235.0	fR/S	 62.18
U+6235.0	fTotalStrokes	 22
U+6235.0	fCangjie	BGI
U+6235.0	fFourCorner	63150
U+6235.0	fCantonese	keoi4
U+6235.0	fMandarin	qu2
U+6235.0	fEnglish	a halberd
U+6236.0	fUTF8	戶
U+6236.0	fCNS11643	1-4542
U+6236.0	fBig5	A4E1
U+6236.0	fGB	E-2A74
U+6236.0	fR/S	 63.0
U+6236.0	fTotalStrokes	 4
U+6236.0	fCangjie	HS
U+6236.0	fFourCorner	30277
U+6236.0	fHakka	 fu5; fu3; fu1
U+6236.0	fCantonese	wu6
U+6236.0	fMandarin	hu4
U+6236.0	fEnglish	[1] door [2] family; household [3] a Chinese family name
U+6236.1	fUTF8	戶
U+6236.1	fCNS11643	1-4542
U+6236.1	fBig5	A4E1
U+6236.1	fGB	E-2A74
U+6236.1	fR/S	 63.0
U+6236.1	fTotalStrokes	 4
U+6236.1	fCangjie	HS
U+6236.1	fFourCorner	30277
U+6236.1	fMandarin	hu5
U+6237.0	fUTF8	户
U+6237.0	fCNS11643	3-2165
U+6237.0	fGB	0-3B27
U+6237.0	fR/S	 63.0
U+6237.0	fTotalStrokes	 4
U+6237.0	fCangjie	IS
U+6237.0	fFourCorner	30270
U+6237.0	fHakka	 fu3; fu5; fu1
U+6237.0	fCantonese	wu6
U+6237.0	fMandarin	hu4
U+6237.0	fEnglish	 door family
U+6237.1	fUTF8	户
U+6237.1	fCNS11643	3-2165
U+6237.1	fGB	0-3B27
U+6237.1	fR/S	 63.0
U+6237.1	fTotalStrokes	 4
U+6237.1	fCangjie	IS
U+6237.1	fFourCorner	30270
U+6237.1	fMandarin	hu5
U+6238.0	fUTF8	戸
U+6238.0	fCNS11643	3-2166
U+6238.0	fGB	E-2A75
U+6238.0	fR/S	 63.0
U+6238.0	fTotalStrokes	 4
U+6238.0	fCangjie	MS
U+6238.0	fFourCorner	30270
U+6238.0	fHakka	 fu5; {fu2}
U+6238.0	fCantonese	wu6
U+6238.0	fMandarin	hu4
U+6238.0	fEnglish	 door family
U+6239.0	fUTF8	戹
U+6239.0	fCNS11643	3-224E
U+6239.0	fGB	E-2A76
U+6239.0	fR/S	 63.1
U+6239.0	fTotalStrokes	 5
U+6239.0	fCangjie	ISN
U+6239.0	fFourCorner	30210
U+6239.0	fHakka	 ak8; ak7
U+6239.0	fCantonese	ak1; aak1 (also); ngaak1 (also); ngak1 (also)
U+6239.0	fMandarin	e4; ai4
U+6239.0	fEnglish	narrow; strait; straits; difficult
U+623A.0	fUTF8	戺
U+623A.0	fCNS11643	2-2428
U+623A.0	fBig5	CAE6
U+623A.0	fGB	3-506B
U+623A.0	fR/S	 63.3
U+623A.0	fTotalStrokes	 7
U+623A.0	fCangjie	HSRU
U+623A.0	fFourCorner	77217
U+623A.0	fHakka	 sii5; sii6
U+623A.0	fCantonese	si6
U+623A.0	fMandarin	shi4; shi1
U+623A.0	fEnglish	 stone boarder steps
U+623B.0	fUTF8	戻
U+623B.0	fCNS11643	3-2628
U+623B.0	fGB	E-2A77
U+623B.0	fR/S	  63.3
U+623B.0	fTotalStrokes	 7
U+623B.0	fCangjie	HSK
U+623B.0	fFourCorner	30234
U+623B.0	fCantonese	dai6
U+623B.0	fMandarin	tai4; li4; ti4
U+623B.0	fEnglish	perverse
U+623C.0	fUTF8	戼
U+623C.0	fCNS11643	3-2629
U+623C.0	fGB	E-2A78
U+623C.0	fR/S	 63.3
U+623C.0	fTotalStrokes	 7
U+623C.0	fCangjie	NSMLS
U+623C.0	fFourCorner	11770
U+623C.0	fHakka	 mau1; mau3
U+623C.0	fCantonese	maau5
U+623C.0	fMandarin	mao3
U+623C.0	fEnglish	standard form of &#x536F;
U+623D.0	fUTF8	戽
U+623D.0	fCNS11643	2-2671
U+623D.0	fBig5	CCB2
U+623D.0	fGB	0-6C66
U+623D.0	fR/S	 63.4
U+623D.0	fTotalStrokes	 8
U+623D.0	fCangjie	HSYJ
U+623D.0	fFourCorner	30247
U+623D.0	fHakka	  fu5
U+623D.0	fCantonese	fu3
U+623D.0	fMandarin	hu4
U+623D.0	fEnglish	 pail; bucket
U+623E.0	fUTF8	戾
U+623E.0	fCNS11643	1-4D53
U+623E.0	fBig5	A9D1
U+623E.0	fGB	0-6C65
U+623E.0	fR/S	 63.4
U+623E.0	fTotalStrokes	 8
U+623E.0	fCangjie	HSIK
U+623E.0	fFourCorner	30234
U+623E.0	fHakka	  lui5; lui6; lui3; li3
U+623E.0	fCantonese	leoi6; lit6; lai2; lai6
U+623E.0	fMandarin	li4; lie4
U+623E.0	fEnglish	[1] perverse; recalcitrant; irregular; abnormal [2] atrocity; atrocious; violence; violent [3] come to; up to
U+623F.0	fUTF8	房
U+623F.0	fCNS11643	1-4D52
U+623F.0	fBig5	A9D0
U+623F.0	fGB	0-373F
U+623F.0	fR/S	 63.4
U+623F.0	fTotalStrokes	 8
U+623F.0	fCangjie	HSYHS
U+623F.0	fFourCorner	30227
U+623F.0	fHakka	  fong2; fong3
U+623F.0	fCantonese	fong4; fong2 (also); pong4 (also)
U+623F.0	fMandarin	fang2; pang2
U+623F.0	fEnglish	[1] house; building [2] room; chamber [3] compartmentalised structure [4] wife; concubine [5] a Chinese family name
U+6240.0	fUTF8	所
U+6240.0	fCNS11643	1-4D54
U+6240.0	fBig5	A9D2
U+6240.0	fGB	0-4B79
U+6240.0	fR/S	 63.4
U+6240.0	fTotalStrokes	 8
U+6240.0	fCangjie	HSHML
U+6240.0	fFourCorner	72221
U+6240.0	fHakka	  so3
U+6240.0	fCantonese	so2
U+6240.0	fMandarin	suo3
U+6240.0	fEnglish	[1] place; location; position [2] building; office [3] that which
U+6241.0	fUTF8	扁
U+6241.0	fCNS11643	1-5137
U+6241.0	fBig5	ABF3
U+6241.0	fGB	0-3162
U+6241.0	fR/S	 63.5
U+6241.0	fTotalStrokes	 9
U+6241.0	fCangjie	HSBT
U+6241.0	fFourCorner	30227
U+6241.0	fHakka	 bien3; ben3; pian3
U+6241.0	fCantonese	bin2; pin1
U+6241.0	fMandarin	bian3; bian4; fan1; pian2; pian3; pin4
U+6241.0	fEnglish	[1] flat [2] tablet; signboard [3] an ancient surname
U+6241.1	fUTF8	扁
U+6241.1	fCNS11643	1-5137
U+6241.1	fBig5	ABF3
U+6241.1	fGB	0-3162
U+6241.1	fR/S	 63.5
U+6241.1	fTotalStrokes	 9
U+6241.1	fCangjie	HSBT
U+6241.1	fFourCorner	30227
U+6241.1	fMandarin	pian1
U+6241.1	fEnglish	small
U+6242.0	fUTF8	扂
U+6242.0	fCNS11643	2-2A53
U+6242.0	fBig5	CED2
U+6242.0	fGB	3-506E
U+6242.0	fR/S	 63.5
U+6242.0	fTotalStrokes	 9
U+6242.0	fCangjie	HSYR
U+6242.0	fFourCorner	72261
U+6242.0	fHakka	  diam5
U+6242.0	fCantonese	dim3; dim6; tim5
U+6242.0	fMandarin	dian4
U+6242.0	fEnglish	 bar a gate
U+6243.0	fUTF8	扃
U+6243.0	fCNS11643	2-2A54
U+6243.0	fBig5	CED3
U+6243.0	fGB	0-6C67
U+6243.0	fR/S	 63.5
U+6243.0	fTotalStrokes	 9
U+6243.0	fCangjie	HSBR
U+6243.0	fFourCorner	37227
U+6243.0	fHakka	 kwen1; giung1; gwen1
U+6243.0	fCantonese	gwing1; gwing2
U+6243.0	fMandarin	jiong1
U+6243.0	fEnglish	 bolt (doors); shut a door
U+6243.1	fUTF8	扃
U+6243.1	fCNS11643	2-2A54
U+6243.1	fBig5	CED3
U+6243.1	fGB	0-6C67
U+6243.1	fR/S	 63.5
U+6243.1	fTotalStrokes	 9
U+6243.1	fCangjie	HSBR
U+6243.1	fFourCorner	37227
U+6243.1	fMandarin	jiong3
U+6243.1	fEnglish	perceiving; discerning
U+6244.0	fUTF8	扄
U+6244.0	fGB	E-2A79
U+6244.0	fR/S	 63.6
U+6244.0	fTotalStrokes	 10
U+6244.0	fCangjie	ISHBR
U+6244.0	fFourCorner	7222
U+6244.0	fHakka	 shong3; song3
U+6244.0	fCantonese	soeng2
U+6244.0	fMandarin	shang3
U+6244.0	fEnglish	large ring on a front door used as a knocker
U+6245.0	fUTF8	扅
U+6245.0	fCNS11643	3-3129
U+6245.0	fGB	3-506F
U+6245.0	fR/S	  63.6
U+6245.0	fTotalStrokes	 10
U+6245.0	fCangjie	ISNIN
U+6245.0	fFourCorner	30220
U+6245.0	fHakka	 ji2; ri2; i2
U+6245.0	fCantonese	ji4
U+6245.0	fMandarin	yi2
U+6245.0	fEnglish	gate bar; bolt
U+6246.0	fUTF8	扆
U+6246.0	fCNS11643	2-2F32
U+6246.0	fBig5	D1B0
U+6246.0	fGB	3-5070
U+6246.0	fR/S	 63.6
U+6246.0	fTotalStrokes	 10
U+6246.0	fCangjie	HSYHV
U+6246.0	fFourCorner	30232
U+6246.0	fHakka	 ji3; ri3; i3
U+6246.0	fCantonese	ji2
U+6246.0	fMandarin	yi3
U+6246.0	fEnglish	a screen decorated with a design in hatchets placed behind the emperor in the imperial audience chamber
U+6247.0	fUTF8	扇
U+6247.0	fCNS11643	1-5552
U+6247.0	fBig5	AEB0
U+6247.0	fGB	0-4948
U+6247.0	fR/S	 63.6
U+6247.0	fTotalStrokes	 10
U+6247.0	fCangjie	HSSMM
U+6247.0	fFourCorner	30227
U+6247.0	fHakka	 shan5; sen5; sien5; san5; shen5
U+6247.0	fCantonese	sin3; sin2 (also)
U+6247.0	fMandarin	shan4
U+6247.0	fEnglish	[1] fan; door panel [2] a numerary auxiliary for door or gate leaves
U+6247.1	fUTF8	扇
U+6247.1	fCNS11643	1-5552
U+6247.1	fBig5	AEB0
U+6247.1	fGB	0-4948
U+6247.1	fR/S	 63.6
U+6247.1	fTotalStrokes	 10
U+6247.1	fCangjie	HSSMM
U+6247.1	fFourCorner	30227
U+6247.1	fMandarin	shan1
U+6247.1	fEnglish	 fan; instigate; incite
U+6248.0	fUTF8	扈
U+6248.0	fCNS11643	1-5A53
U+6248.0	fBig5	B1AF
U+6248.0	fGB	0-6C68
U+6248.0	fR/S	 63.7
U+6248.0	fTotalStrokes	 11
U+6248.0	fCangjie	HSRAU
U+6248.0	fFourCorner	72217
U+6248.0	fHakka	 fu5; fu6; fu3; gu3
U+6248.0	fCantonese	wu6
U+6248.0	fMandarin	hu4
U+6248.0	fEnglish	[1] follow as escort or retinue [2] insolent; impertinent [3] a Chinese family name
U+6249.0	fUTF8	扉
U+6249.0	fCNS11643	1-5F3D
U+6249.0	fBig5	B476
U+6249.0	fGB	0-6C69
U+6249.0	fR/S	 63.8
U+6249.0	fTotalStrokes	 12
U+6249.0	fCangjie	HSLMY
U+6249.0	fFourCorner	30211
U+6249.0	fHakka	  fui1; fi1
U+6249.0	fCantonese	fei1
U+6249.0	fMandarin	fei1
U+6249.0	fEnglish	 door leaf
U+624A.0	fUTF8	扊
U+624A.0	fCNS11643	2-3B74
U+624A.0	fBig5	D951
U+624A.0	fGB	3-5071
U+624A.0	fR/S	 63.8
U+624A.0	fTotalStrokes	 12
U+624A.0	fCangjie	HSFF
U+624A.0	fFourCorner	30289
U+624A.0	fHakka	 ram2; jam2; jam3
U+624A.0	fCantonese	jim5
U+624A.0	fMandarin	yan3
U+624A.0	fEnglish	upright bar for fastening door
U+624B.0	fUTF8	手
U+624B.0	fCNS11643	1-4543
U+624B.0	fBig5	A4E2
U+624B.0	fGB	0-4A56
U+624B.0	fR/S	 64.0
U+624B.0	fTotalStrokes	 4
U+624B.0	fCangjie	Q
U+624B.0	fFourCorner	20500
U+624B.0	fHakka	 su3; shiu3; siu3; shu3
U+624B.0	fCantonese	sau2
U+624B.0	fMandarin	shou3
U+624B.0	fEnglish	[1] hand; of the hand; having to do with the hand [2] have in one's hand; hold [3] skilled person; person [4] action [5] personally
U+624B.1	fUTF8	手
U+624B.1	fCNS11643	1-4543
U+624B.1	fBig5	A4E2
U+624B.1	fGB	0-4A56
U+624B.1	fR/S	 64.0
U+624B.1	fTotalStrokes	 4
U+624B.1	fCangjie	Q
U+624B.1	fFourCorner	20500
U+624B.1	fMandarin	shou5
U+624C.0	fUTF8	扌
U+624C.0	fCNS11643	4-2137
U+624C.0	fGB	0-5E50
U+624C.0	fR/S	 64.0
U+624C.0	fTotalStrokes	 3
U+624C.0	fCangjie	DM
U+624C.0	fFourCorner	50000
U+624C.0	fCantonese	sau2
U+624C.0	fMandarin	shou3zi4pang2
U+624C.0	fEnglish	 hand Kangxi radical number 64
U+624D.0	fUTF8	才
U+624D.0	fCNS11643	1-445F
U+624D.0	fBig5	A47E
U+624D.0	fGB	0-3245
U+624D.0	fR/S	 64.0
U+624D.0	fTotalStrokes	 3
U+624D.0	fCangjie	DH
U+624D.0	fFourCorner	40200
U+624D.0	fHakka	  coi2; coi3
U+624D.0	fCantonese	coi4; coi2 (also)
U+624D.0	fMandarin	cai2
U+624D.0	fEnglish	[1] talent; natural abilities; gift; mental faculty [2] gifted person; talented person; brilliant man; talent [3] people of a certain type [4] certainly; indeed [5] just; just now
U+624D.1	fUTF8	才
U+624D.1	fCNS11643	1-445F
U+624D.1	fBig5	A47E
U+624D.1	fGB	0-3245
U+624D.1	fR/S	 64.0
U+624D.1	fTotalStrokes	 3
U+624D.1	fCangjie	DH
U+624D.1	fFourCorner	40200
U+624D.1	fMandarin	zai1
U+624D.2	fUTF8	才
U+624D.2	fCNS11643	1-445F
U+624D.2	fBig5	A47E
U+624D.2	fGB	0-3245
U+624D.2	fR/S	 64.0
U+624D.2	fTotalStrokes	 3
U+624D.2	fCangjie	DH
U+624D.2	fFourCorner	40200
U+624E.0	fUTF8	扎
U+624E.0	fCNS11643	1-4544
U+624E.0	fBig5	A4E3
U+624E.0	fGB	0-547A
U+624E.0	fR/S	 64.1
U+624E.0	fTotalStrokes	 4
U+624E.0	fCangjie	QU
U+624E.0	fFourCorner	52010
U+624E.0	fHakka	  zap7; zat7; zhap7; soi1
U+624E.0	fCantonese	zaat3; zaat2 (also); zaap3
U+624E.0	fMandarin	zha1; za1; zha3
U+624E.0	fEnglish	 pierce; prick
U+624E.1	fUTF8	扎
U+624E.1	fCNS11643	1-4544
U+624E.1	fBig5	A4E3
U+624E.1	fGB	0-547A
U+624E.1	fR/S	 64.1
U+624E.1	fTotalStrokes	 4
U+624E.1	fCangjie	QU
U+624E.1	fFourCorner	52010
U+624E.1	fMandarin	zha2
U+624E.1	fEnglish	 struggle; strive
U+624F.0	fUTF8	扏
U+624F.0	fCNS11643	3-2250
U+624F.0	fGB	3-4953
U+624F.0	fR/S	 64.3; 64.2
U+624F.0	fTotalStrokes	 5; 6
U+624F.0	fCangjie	XQKN
U+624F.0	fFourCorner	54010
U+624F.0	fCantonese	kau4
U+624F.0	fMandarin	qiu2
U+6250.0	fUTF8	扐
U+6250.0	fCNS11643	2-215E
U+6250.0	fBig5	C97D
U+6250.0	fGB	3-4954
U+6250.0	fR/S	 64.2
U+6250.0	fTotalStrokes	 5
U+6250.0	fCangjie	QKS
U+6250.0	fFourCorner	54027
U+6250.0	fHakka	  let8;  lit8; net8
U+6250.0	fCantonese	lak6; lik1 (also)
U+6250.0	fMandarin	le4
U+6250.0	fEnglish	divine
U+6251.0	fUTF8	扑
U+6251.0	fCNS11643	1-4657
U+6251.0	fBig5	A5B7
U+6251.0	fGB	0-464B
U+6251.0	fR/S	 64.2
U+6251.0	fTotalStrokes	 5
U+6251.0	fCangjie	QY
U+6251.0	fFourCorner	43000
U+6251.0	fHakka	  pok7; puk7
U+6251.0	fCantonese	pok3
U+6251.0	fMandarin	pu1; bao2; bu1; bo2; bu2; pu2
U+6251.0	fEnglish	simplified form of &#x64B2;
U+6252.0	fUTF8	扒
U+6252.0	fCNS11643	1-4656
U+6252.0	fBig5	A5B6
U+6252.0	fGB	0-3047
U+6252.0	fR/S	 64.2
U+6252.0	fTotalStrokes	 5
U+6252.0	fCangjie	QC
U+6252.0	fFourCorner	58000
U+6252.0	fHakka	  bai5;  bat7; pa2; bak7
U+6252.0	fCantonese	paa4; paa1; paa2 (also); baat3
U+6252.0	fMandarin	ba1; pa1; bai4
U+6252.0	fEnglish	[1] claw; strip [2] rake [3] climb; scale
U+6252.1	fUTF8	扒
U+6252.1	fCNS11643	1-4656
U+6252.1	fBig5	A5B6
U+6252.1	fGB	0-3047
U+6252.1	fR/S	 64.2
U+6252.1	fTotalStrokes	 5
U+6252.1	fCangjie	QC
U+6252.1	fFourCorner	58000
U+6252.1	fMandarin	pa2
U+6252.1	fEnglish	[1] gather up; rake up [2] stew; braise [3] scratch; claw
U+6253.0	fUTF8	打
U+6253.0	fCNS11643	1-4654
U+6253.0	fBig5	A5B4
U+6253.0	fGB	0-3472
U+6253.0	fR/S	 64.2
U+6253.0	fTotalStrokes	 5
U+6253.0	fCangjie	QMN
U+6253.0	fFourCorner	51020
U+6253.0	fHakka	  da3; da1
U+6253.0	fCantonese	daa2; daa1; ding2
U+6253.0	fMandarin	da3; deng3
U+6253.0	fEnglish	[1] strike; hit; beat [2] fight; attack [3] smash [4] make, do, get, fetch, play, buy, etc. [5] from; to; toward [6] wash with soap
U+6253.1	fUTF8	打
U+6253.1	fCNS11643	1-4654
U+6253.1	fBig5	A5B4
U+6253.1	fGB	0-3472
U+6253.1	fR/S	 64.2
U+6253.1	fTotalStrokes	 5
U+6253.1	fCangjie	QMN
U+6253.1	fFourCorner	51020
U+6253.1	fMandarin	da2
U+6253.1	fEnglish	 dozen
U+6253.2	fUTF8	打
U+6253.2	fCNS11643	1-4654
U+6253.2	fBig5	A5B4
U+6253.2	fGB	0-3472
U+6253.2	fR/S	 64.2
U+6253.2	fTotalStrokes	 5
U+6253.2	fCangjie	QMN
U+6253.2	fFourCorner	51020
U+6254.0	fUTF8	扔
U+6254.0	fCNS11643	1-4655
U+6254.0	fBig5	A5B5
U+6254.0	fGB	0-4853
U+6254.0	fR/S	 64.2
U+6254.0	fTotalStrokes	 5
U+6254.0	fCangjie	QNHS
U+6254.0	fFourCorner	57027
U+6254.0	fHakka	  jin2; jin3; rin2; {vang5}
U+6254.0	fCantonese	jing4; wing1 (also); wing4
U+6254.0	fMandarin	reng1; reng3; reng4; reng2
U+6254.0	fEnglish	[1] throw; hurl [2] abandon; throw away; discard
U+6255.0	fUTF8	払
U+6255.0	fGB	5-4B54
U+6255.0	fR/S	 64.2
U+6255.0	fTotalStrokes	 5
U+6255.0	fCangjie	XXQI
U+6255.0	fCantonese	faan2
U+6255.0	fMandarin	fu2; bi4
U+6255.0	fEnglish	 shake off; brush away dust
U+6256.0	fUTF8	扖
U+6256.0	fGB	E-2A7A
U+6256.0	fR/S	 64.2
U+6256.0	fTotalStrokes	 5
U+6256.0	fCangjie	XQO
U+6256.0	fCantonese	jap6
U+6257.0	fUTF8	扗
U+6257.0	fCNS11643	3-236F
U+6257.0	fGB	E-2A7B
U+6257.0	fR/S	 64.3
U+6257.0	fTotalStrokes	 6
U+6257.0	fCangjie	QG
U+6257.0	fFourCorner	54010
U+6257.0	fHakka	  coi1;  cai5; cai3; cai6; cai1; coi5; hoi1; toi1
U+6257.0	fCantonese	zoi6; zoi2 (also)
U+6257.0	fMandarin	zai4; zai3
U+6257.0	fEnglish	standard form of &#x5728;
U+6258.0	fUTF8	托
U+6258.0	fCNS11643	1-482C
U+6258.0	fBig5	A6AB
U+6258.0	fGB	0-4D50
U+6258.0	fR/S	 64.3
U+6258.0	fTotalStrokes	 6
U+6258.0	fCangjie	QHP
U+6258.0	fFourCorner	52014
U+6258.0	fHakka	  tok7
U+6258.0	fCantonese	tok3; tok2 (also)
U+6258.0	fMandarin	tuo1
U+6258.0	fEnglish	[1] hold on the palm; lift on the palm [2] entrust; charge; consign; rely; ask [3] tray; pad
U+6259.0	fUTF8	扙
U+6259.0	fCNS11643	2-2249
U+6259.0	fBig5	C9E9
U+6259.0	fGB	5-4B55
U+6259.0	fR/S	 64.3
U+6259.0	fTotalStrokes	 6
U+6259.0	fCangjie	QJK
U+6259.0	fFourCorner	55000
U+6259.0	fCantonese	zoeng6
U+6259.0	fMandarin	zhang4
U+625A.0	fUTF8	扚
U+625A.0	fCNS11643	2-224B
U+625A.0	fBig5	C9EB
U+625A.0	fGB	3-495B
U+625A.0	fR/S	 64.3
U+625A.0	fTotalStrokes	 6
U+625A.0	fCangjie	QPI
U+625A.0	fFourCorner	57020
U+625A.0	fHakka	 dit7
U+625A.0	fCantonese	tiu2; dik1; lik6; diu2; joek6
U+625A.0	fMandarin	diao1; di4; yue1; zhuo2
U+625A.0	fEnglish	 lead
U+625B.0	fUTF8	扛
U+625B.0	fCNS11643	1-482B
U+625B.0	fBig5	A6AA
U+625B.0	fGB	0-3F38
U+625B.0	fR/S	 64.3
U+625B.0	fTotalStrokes	 6
U+625B.0	fCangjie	QM
U+625B.0	fFourCorner	51010
U+625B.0	fHakka	  gong1
U+625B.0	fCantonese	gong1; kong1
U+625B.0	fMandarin	kang2; gang1; jiang1; xiang3
U+625B.0	fEnglish	 carry on the shoulders; shoulder; lift (especially when only a single person is involved)
U+625C.0	fUTF8	扜
U+625C.0	fCNS11643	2-2243
U+625C.0	fBig5	C9E3
U+625C.0	fGB	E-2A7C
U+625C.0	fR/S	 64.3
U+625C.0	fTotalStrokes	 6
U+625C.0	fCangjie	QMD
U+625C.0	fFourCorner	51040
U+625C.0	fHakka	 i1
U+625C.0	fCantonese	jyu1
U+625C.0	fMandarin	yu1; xu1
U+625D.0	fUTF8	扝
U+625D.0	fCNS11643	3-236C
U+625D.0	fGB	3-4956
U+625D.0	fR/S	 64.3
U+625D.0	fTotalStrokes	 6
U+625D.0	fCangjie	QMMS
U+625D.0	fFourCorner	51020
U+625D.0	fHakka	 i1
U+625D.0	fCantonese	jyu1
U+625D.0	fMandarin	yu1; xu1
U+625D.0	fEnglish	same as &#x625C;
U+625E.0	fUTF8	扞
U+625E.0	fCNS11643	2-2244
U+625E.0	fBig5	C9E4
U+625E.0	fGB	E-2A7D
U+625E.0	fR/S	 64.3
U+625E.0	fTotalStrokes	 6
U+625E.0	fCangjie	QMJ
U+625E.0	fFourCorner	51040
U+625E.0	fHakka	  gon5;  hon5; hon6; hon3
U+625E.0	fCantonese	gon2; hon6; hon2 (also); hon5 (also)
U+625E.0	fMandarin	han4; gan3
U+625E.0	fEnglish	[1] defend; guard; ward off; withstand [2] oppose; resist; obstruct
U+625F.0	fUTF8	扟
U+625F.0	fCNS11643	3-236D
U+625F.0	fGB	3-495C
U+625F.0	fR/S	 64.4; 64.3
U+625F.0	fTotalStrokes	 6; 7
U+625F.0	fCangjie	QNJ
U+625F.0	fFourCorner	57010
U+625F.0	fCantonese	san1
U+625F.0	fMandarin	shen1
U+6260.0	fUTF8	扠
U+6260.0	fCNS11643	2-224A
U+6260.0	fBig5	C9EA
U+6260.0	fGB	3-495E
U+6260.0	fR/S	 64.3
U+6260.0	fTotalStrokes	 6
U+6260.0	fCangjie	QEI
U+6260.0	fFourCorner	57040
U+6260.0	fHakka	  ca1
U+6260.0	fCantonese	caa1; zaa3
U+6260.0	fMandarin	cha1; cha2; zha3; cha3
U+6260.0	fEnglish	 harpoon
U+6261.0	fUTF8	扡
U+6261.0	fCNS11643	2-2246
U+6261.0	fBig5	C9E6
U+6261.0	fGB	3-495D
U+6261.0	fR/S	 64.3
U+6261.0	fTotalStrokes	 6
U+6261.0	fCangjie	QPD
U+6261.0	fFourCorner	54012
U+6261.0	fHakka	 to3;  to1; ja3; to2; ra3; shi5
U+6261.0	fCantonese	ci2; to1; si6
U+6261.0	fMandarin	yi3; tuo1; chi3; shi4; zhai4; zhi4
U+6261.0	fEnglish	drag along
U+6262.0	fUTF8	扢
U+6262.0	fCNS11643	2-2248
U+6262.0	fBig5	C9E8
U+6262.0	fGB	3-4959
U+6262.0	fR/S	 64.3
U+6262.0	fTotalStrokes	 6
U+6262.0	fCangjie	QON
U+6262.0	fFourCorner	58017
U+6262.0	fHakka	 gut7;  ngiet7; kut7; jat7; gwut7
U+6262.0	fCantonese	gwat1; ngat6; at6 (also); hei3; goi3; ngat1
U+6262.0	fMandarin	gu3; xi4; he4; ge1; hu2; ji4
U+6262.0	fEnglish	caper
U+6263.0	fUTF8	扣
U+6263.0	fCNS11643	1-482A
U+6263.0	fBig5	A6A9
U+6263.0	fGB	0-3F5B
U+6263.0	fR/S	 64.3
U+6263.0	fTotalStrokes	 6
U+6263.0	fCangjie	QR
U+6263.0	fFourCorner	56000
U+6263.0	fHakka	  keu5;  k'ieu5; kiu5; gep7; kep7; kem2; gem1; gem3
U+6263.0	fCantonese	kau3; kau2 (also)
U+6263.0	fMandarin	kou4; kou3
U+6263.0	fEnglish	[1] knock; strike; rap; tap; pull [2] fasten; button; buckle [3] detain; confine [4] deduct [5] button; hook; buckle [6] impound; withhold [7] cover on top
U+6264.0	fUTF8	扤
U+6264.0	fCNS11643	2-2245
U+6264.0	fBig5	C9E5
U+6264.0	fGB	3-4957
U+6264.0	fR/S	 64.3
U+6264.0	fTotalStrokes	 6
U+6264.0	fCangjie	QMU
U+6264.0	fFourCorner	51010
U+6264.0	fHakka	 ngut7
U+6264.0	fCantonese	ngaat6; ngat1; aat6 (also); ngat6 (also); at6 (also); wat6 (also); at1 (also)
U+6264.0	fMandarin	wu4; yue4
U+6264.0	fEnglish	[1] move [2] stuff; cram in
U+6265.0	fUTF8	扥
U+6265.0	fCNS11643	2-224C
U+6265.0	fBig5	C9EC
U+6265.0	fGB	E-2A7E
U+6265.0	fR/S	 64.3
U+6265.0	fTotalStrokes	 6
U+6265.0	fCangjie	QQU
U+6265.0	fFourCorner	55010
U+6265.0	fCantonese	dan3; deon6
U+6265.0	fMandarin	tuo1
U+6266.0	fUTF8	扦
U+6266.0	fCNS11643	2-2247
U+6266.0	fBig5	C9E7
U+6266.0	fGB	0-4724
U+6266.0	fR/S	 64.3
U+6266.0	fTotalStrokes	 6
U+6266.0	fCangjie	QHJ
U+6266.0	fFourCorner	52040
U+6266.0	fHakka	  cien1
U+6266.0	fCantonese	cin1
U+6266.0	fMandarin	qian1
U+6266.0	fEnglish	[1] penetrate; poke; pick; pierce [2] pick
U+6267.0	fUTF8	执
U+6267.0	fCNS11643	3-236E
U+6267.0	fGB	0-5634
U+6267.0	fR/S	 64.3
U+6267.0	fTotalStrokes	 6
U+6267.0	fCangjie	QKNI
U+6267.0	fFourCorner	55010
U+6267.0	fHakka	 chip7
U+6267.0	fCantonese	zap1
U+6267.0	fMandarin	zhi2
U+6267.0	fEnglish	simplified form of &#x57F7;
U+6268.0	fUTF8	扨
U+6268.0	fCNS11643	F-2251
U+6268.0	fGB	E-2B21
U+6268.0	fR/S	 64.3
U+6268.0	fTotalStrokes	 6
U+6268.0	fCangjie	QSHI
U+6268.0	fFourCorner	5702
U+6268.0	fCantonese	jan6
U+6268.0	fMandarin	ren4
U+6268.0	fEnglish	pick up with fork or pincers
U+6269.0	fUTF8	扩
U+6269.0	fGB	0-4029
U+6269.0	fR/S	 64.3
U+6269.0	fTotalStrokes	 6
U+6269.0	fCangjie	XQI
U+6269.0	fHakka	 kwok7
U+6269.0	fCantonese	kwok3; kok3 (also); kwong3 (also); kong3 (also)
U+6269.0	fMandarin	kuo4
U+6269.0	fEnglish	simplified form of &#x64F4;
U+626A.0	fUTF8	扪
U+626A.0	fGB	0-5E51
U+626A.0	fR/S	 64.3
U+626A.0	fTotalStrokes	 6
U+626A.0	fCangjie	QLS
U+626A.0	fHakka	 mun2
U+626A.0	fCantonese	mun4
U+626A.0	fMandarin	men2
U+626A.0	fEnglish	simplified form of &#x636B;
U+626B.0	fUTF8	扫
U+626B.0	fGB	0-4928
U+626B.0	fR/S	 64.3
U+626B.0	fTotalStrokes	 6
U+626B.0	fCangjie	QSM
U+626B.0	fHakka	 sau5
U+626B.0	fCantonese	sou3; sou2
U+626B.0	fMandarin	sao3
U+626B.0	fEnglish	simplified form of &#x6383;
U+626B.1	fUTF8	扫
U+626B.1	fGB	0-4928
U+626B.1	fR/S	 64.3
U+626B.1	fTotalStrokes	 6
U+626B.1	fCangjie	QSM
U+626B.1	fMandarin	sao4
U+626C.0	fUTF8	扬
U+626C.0	fGB	0-516F
U+626C.0	fR/S	 64.3
U+626C.0	fTotalStrokes	 6
U+626C.0	fCangjie	QNSH
U+626C.0	fHakka	 jong2
U+626C.0	fCantonese	joeng4; joeng2 (also)
U+626C.0	fMandarin	yang2
U+626C.0	fEnglish	simplified form of &#x63DA;
U+626D.0	fUTF8	扭
U+626D.0	fCNS11643	1-4A43
U+626D.0	fBig5	A7E1
U+626D.0	fGB	0-4524
U+626D.0	fR/S	 64.4
U+626D.0	fTotalStrokes	 7
U+626D.0	fCangjie	QNG
U+626D.0	fFourCorner	57015
U+626D.0	fHakka	 ngiu3;  neu3; gniu5; niu2; niu3; niu5
U+626D.0	fCantonese	nau2; lau2 (also); nau5
U+626D.0	fMandarin	niu3
U+626D.0	fEnglish	[1] turn; twist; wrench [2] seize; grasp
U+626D.1	fUTF8	扭
U+626D.1	fCNS11643	1-4A43
U+626D.1	fBig5	A7E1
U+626D.1	fGB	0-4524
U+626D.1	fR/S	 64.4
U+626D.1	fTotalStrokes	 7
U+626D.1	fCangjie	QNG
U+626D.1	fFourCorner	57015
U+626D.1	fMandarin	niu5
U+626E.0	fUTF8	扮
U+626E.0	fCNS11643	1-4A4C
U+626E.0	fBig5	A7EA
U+626E.0	fGB	0-3067
U+626E.0	fR/S	 64.4
U+626E.0	fTotalStrokes	 7
U+626E.0	fCangjie	QCSH
U+626E.0	fFourCorner	58027
U+626E.0	fHakka	  ban5; pan3; pan5; pan6
U+626E.0	fCantonese	baan6; baan3; fan5; ban6; ban3
U+626E.0	fMandarin	ban4
U+626E.0	fEnglish	[1] dress up (as); disguise; play [2] corrupted form of &#x6274;
U+626E.1	fUTF8	扮
U+626E.1	fCNS11643	1-4A4C
U+626E.1	fBig5	A7EA
U+626E.1	fGB	0-3067
U+626E.1	fR/S	 64.4
U+626E.1	fTotalStrokes	 7
U+626E.1	fCangjie	QCSH
U+626E.1	fFourCorner	58027
U+626E.1	fMandarin	ban5
U+626F.0	fUTF8	扯
U+626F.0	fCNS11643	1-4A4A
U+626F.0	fBig5	A7E8
U+626F.0	fGB	0-3336
U+626F.0	fR/S	 64.4
U+626F.0	fTotalStrokes	 7
U+626F.0	fCangjie	QYLM
U+626F.0	fFourCorner	51010
U+626F.0	fHakka	 ca3;  cha3; che1; ce1; cha1
U+626F.0	fCantonese	ce2
U+626F.0	fMandarin	che3
U+626F.0	fEnglish	[1] rip up; tear down [2] pull; drag; haul; strain [3] lump [4] talk nonsense; lie; prevaricate; disgress
U+626F.1	fUTF8	扯
U+626F.1	fCNS11643	1-4A4A
U+626F.1	fBig5	A7E8
U+626F.1	fGB	0-3336
U+626F.1	fR/S	 64.4
U+626F.1	fTotalStrokes	 7
U+626F.1	fCangjie	QYLM
U+626F.1	fFourCorner	51010
U+626F.1	fMandarin	che5
U+6270.0	fUTF8	扰
U+6270.0	fCNS11643	2-2432
U+6270.0	fBig5	CAF0
U+6270.0	fGB	0-4845
U+6270.0	fR/S	 64.4
U+6270.0	fTotalStrokes	 7
U+6270.0	fCangjie	QIKU
U+6270.0	fFourCorner	53014
U+6270.0	fHakka	 jau3; jau1
U+6270.0	fCantonese	jiu5 (also); jiu2
U+6270.0	fMandarin	rao3; rou2
U+6270.0	fEnglish	simplified form of &#x64FE;
U+6271.0	fUTF8	扱
U+6271.0	fCNS11643	2-242F
U+6271.0	fBig5	CAED
U+6271.0	fGB	3-495A
U+6271.0	fR/S	 64.4
U+6271.0	fTotalStrokes	 7
U+6271.0	fCangjie	QNHE
U+6271.0	fFourCorner	57047
U+6271.0	fHakka	 cap7; kip7; giak7
U+6271.0	fCantonese	kap1; caap3
U+6271.0	fMandarin	xi1; ji2; qi4; qie4; yi4
U+6271.0	fEnglish	[1] gather; collect [2] kneel and bow with both hands touching the ground or the floor
U+6271.1	fUTF8	扱
U+6271.1	fCNS11643	2-242F
U+6271.1	fBig5	CAED
U+6271.1	fGB	3-495A
U+6271.1	fR/S	 64.4
U+6271.1	fTotalStrokes	 7
U+6271.1	fCangjie	QNHE
U+6271.1	fFourCorner	57047
U+6271.1	fMandarin	cha1
U+6271.1	fEnglish	 insert
U+6272.0	fUTF8	扲
U+6272.0	fCNS11643	2-2437
U+6272.0	fBig5	CAF5
U+6272.0	fGB	3-496A
U+6272.0	fR/S	 64.4
U+6272.0	fTotalStrokes	 7
U+6272.0	fCangjie	QOIN
U+6272.0	fFourCorner	58027
U+6272.0	fCantonese	ngam4; am4 (also); kam4; gam6
U+6272.0	fMandarin	qian2; jin4; qin2
U+6272.0	fEnglish	pull out
U+6273.0	fUTF8	扳
U+6273.0	fCNS11643	1-4A48
U+6273.0	fBig5	A7E6
U+6273.0	fGB	0-3062
U+6273.0	fR/S	 64.4
U+6273.0	fTotalStrokes	 7
U+6273.0	fCangjie	QHE
U+6273.0	fFourCorner	51047
U+6273.0	fHakka	  ban1; ban3
U+6273.0	fCantonese	paan1; baan2; baan1
U+6273.0	fMandarin	ban1; pan1; pan4
U+6273.0	fEnglish	[1] pull; drag [2] count (on one's fingers)
U+6274.0	fUTF8	扴
U+6274.0	fCNS11643	2-2438
U+6274.0	fBig5	CAF6
U+6274.0	fGB	5-4B60
U+6274.0	fR/S	 64.4
U+6274.0	fTotalStrokes	 7
U+6274.0	fCangjie	QOLL
U+6274.0	fFourCorner	58020
U+6274.0	fCantonese	git3
U+6274.0	fMandarin	jia2
U+6275.0	fUTF8	扵
U+6275.0	fCNS11643	3-2639
U+6275.0	fGB	5-4B61
U+6275.0	fR/S	 64.4
U+6275.0	fTotalStrokes	 7
U+6275.0	fCangjie	QOY
U+6275.0	fFourCorner	58030
U+6275.0	fHakka	 ji1; ri1; i1; vu1
U+6275.0	fCantonese	jyu1; wu1
U+6275.0	fMandarin	yu2; wu1; yu1
U+6275.0	fEnglish	in, on, at, by, from; than; with reference to; compared with
U+6276.0	fUTF8	扶
U+6276.0	fCNS11643	1-4A41
U+6276.0	fBig5	A7DF
U+6276.0	fGB	0-3776
U+6276.0	fR/S	 64.4
U+6276.0	fTotalStrokes	 7
U+6276.0	fCangjie	QQO
U+6276.0	fFourCorner	55030
U+6276.0	fHakka	  fu2;  pu2; fu3; ku5
U+6276.0	fCantonese	fu4; pou4
U+6276.0	fMandarin	fu2; pu2
U+6276.0	fEnglish	[1] support; help; protect; prop up; aid; shield; shelter; harbour [2] lean upon
U+6277.0	fUTF8	扷
U+6277.0	fCNS11643	2-2435
U+6277.0	fBig5	CAF3
U+6277.0	fGB	E-2B22
U+6277.0	fR/S	 64.4
U+6277.0	fTotalStrokes	 7
U+6277.0	fCangjie	QHK
U+6277.0	fFourCorner	52034
U+6277.0	fCantonese	ou3; ngou3 (also)
U+6277.0	fMandarin	ao4
U+6278.0	fUTF8	扸
U+6278.0	fCNS11643	3-262E
U+6278.0	fGB	5-4B5F
U+6278.0	fR/S	 64.5; 64.4
U+6278.0	fTotalStrokes	 7; 8
U+6278.0	fCangjie	QLLL
U+6278.0	fFourCorner	52020
U+6278.0	fCantonese	sik1
U+6278.0	fMandarin	xi1
U+6279.0	fUTF8	批
U+6279.0	fCNS11643	1-4A47
U+6279.0	fBig5	A7E5
U+6279.0	fGB	0-457A
U+6279.0	fR/S	 64.4
U+6279.0	fTotalStrokes	 7
U+6279.0	fCangjie	QPP
U+6279.0	fFourCorner	51010
U+6279.0	fHakka	 pi1;  p'e1; pi3; poi1; biang1; pai1
U+6279.0	fCantonese	pai1; pei4
U+6279.0	fMandarin	pi1; bie2; pi2
U+6279.0	fEnglish	[1] comment; criticize; judge [2] a whole batch (of things or people); a large quantity or number; wholesale
U+627A.0	fUTF8	扺
U+627A.0	fCNS11643	2-2431
U+627A.0	fBig5	CAEF
U+627A.0	fGB	3-496D
U+627A.0	fR/S	 64.4
U+627A.0	fTotalStrokes	 7
U+627A.0	fCangjie	QHVP
U+627A.0	fFourCorner	52040
U+627A.0	fCantonese	zi2
U+627A.0	fMandarin	zhi3; qi2; di3
U+627B.0	fUTF8	扻
U+627B.0	fCNS11643	2-2430
U+627B.0	fBig5	CAEE
U+627B.0	fGB	5-4B62
U+627B.0	fR/S	 64.4
U+627B.0	fTotalStrokes	 7
U+627B.0	fCangjie	QNO
U+627B.0	fFourCorner	57082
U+627B.0	fHakka	  kam3
U+627B.0	fCantonese	ham2; ham6 (also); zit3
U+627B.0	fMandarin	zi4; jie2; kan3; zhi4; sun3
U+627B.0	fEnglish	 strike run against; throw (as a stone)
U+627C.0	fUTF8	扼
U+627C.0	fCNS11643	1-4A45
U+627C.0	fBig5	A7E3
U+627C.0	fGB	0-3673
U+627C.0	fR/S	 64.4
U+627C.0	fTotalStrokes	 7
U+627C.0	fCangjie	QMSU
U+627C.0	fFourCorner	51012
U+627C.0	fHakka	  ak7; ngak7; nak8; nak7
U+627C.0	fCantonese	aak1 (also); ak1; ngak1 (also); ngaak1 (also); aa1
U+627C.0	fMandarin	e4
U+627C.0	fEnglish	[1] repress; restrain; control [2] grasp; clutch; grip [3] hold and defend (a city, etc.)
U+627D.0	fUTF8	扽
U+627D.0	fCNS11643	2-2436
U+627D.0	fBig5	CAF4
U+627D.0	fGB	3-4963
U+627D.0	fR/S	 64.4
U+627D.0	fTotalStrokes	 7
U+627D.0	fCangjie	QPU
U+627D.0	fFourCorner	50017
U+627D.0	fHakka	  dun5; dun3
U+627D.0	fCantonese	dan3; dan6 (also); deon6
U+627D.0	fMandarin	dun4; den4
U+627D.0	fEnglish	 move; shake
U+627E.0	fUTF8	找
U+627E.0	fCNS11643	1-4A46
U+627E.0	fBig5	A7E4
U+627E.0	fGB	0-5552
U+627E.0	fR/S	 64.4
U+627E.0	fTotalStrokes	 7
U+627E.0	fCangjie	QI
U+627E.0	fFourCorner	53050
U+627E.0	fHakka	  zau3
U+627E.0	fCantonese	zaau2
U+627E.0	fMandarin	zhao3; hua2
U+627E.0	fEnglish	[1] search (for); seek; look for; find [2] return (change)
U+627F.0	fUTF8	承
U+627F.0	fCNS11643	1-4D55
U+627F.0	fBig5	A9D3
U+627F.0	fGB	0-3350
U+627F.0	fR/S	 64.4
U+627F.0	fTotalStrokes	 8
U+627F.0	fCangjie	NNQO
U+627F.0	fFourCorner	17232
U+627F.0	fHakka	 sin2;  shin2; siin2; shin3
U+627F.0	fCantonese	sing4; cing2
U+627F.0	fMandarin	cheng2; zheng3; zeng4; zheng1; zheng4
U+627F.0	fEnglish	[1] receive; inherit; succeed (another in a task, etc.) [2] undertake; make it one's responsibility [3] by (order of) [4] continue; carry on, as a theme [5] hold; contain; support; bear [6] confess [7] obliged; with thanks [8] please; flatter
U+6280.0	fUTF8	技
U+6280.0	fCNS11643	1-4A40
U+6280.0	fBig5	A7DE
U+6280.0	fGB	0-3C3C
U+6280.0	fR/S	 64.4
U+6280.0	fTotalStrokes	 7
U+6280.0	fCangjie	QJE
U+6280.0	fFourCorner	54047
U+6280.0	fHakka	  ki3; gi1; gi3
U+6280.0	fCantonese	gei6
U+6280.0	fMandarin	ji4
U+6280.0	fEnglish	skill; dexterity; ability; talent; special ability; tricks; ingenuity
U+6281.0	fUTF8	抁
U+6281.0	fCNS11643	2-2433
U+6281.0	fBig5	CAF1
U+6281.0	fGB	3-4972
U+6281.0	fR/S	 64.4
U+6281.0	fTotalStrokes	 7
U+6281.0	fCangjie	QIHU
U+6281.0	fFourCorner	53010
U+6281.0	fCantonese	jin5
U+6281.0	fMandarin	yan3; juan4; rui4; wei3; yung4
U+6282.0	fUTF8	抂
U+6282.0	fCNS11643	3-262A
U+6282.0	fGB	E-2B23
U+6282.0	fR/S	 64.4
U+6282.0	fTotalStrokes	 7
U+6282.0	fCangjie	QMG
U+6282.0	fFourCorner	51014
U+6282.0	fCantonese	kwong4; kong4 (also)
U+6282.0	fMandarin	kuang2
U+6283.0	fUTF8	抃
U+6283.0	fCNS11643	2-2429
U+6283.0	fBig5	CAE7
U+6283.0	fGB	3-496E
U+6283.0	fR/S	 64.4
U+6283.0	fTotalStrokes	 7
U+6283.0	fCangjie	QYY
U+6283.0	fFourCorner	50030
U+6283.0	fHakka	 pien5; pien6; pien3
U+6283.0	fCantonese	bin6
U+6283.0	fMandarin	bian4; fan1; pan4
U+6283.0	fEnglish	 clap hands; cheer
U+6284.0	fUTF8	抄
U+6284.0	fCNS11643	1-4A3D
U+6284.0	fBig5	A7DB
U+6284.0	fGB	0-332D
U+6284.0	fR/S	 64.4
U+6284.0	fTotalStrokes	 7
U+6284.0	fCangjie	QFH
U+6284.0	fFourCorner	59020
U+6284.0	fHakka	  cau1
U+6284.0	fCantonese	caau1
U+6284.0	fMandarin	chao1
U+6284.0	fEnglish	[1] copy; transcribe; transcription [2] confiscate [3] seize; take
U+6285.0	fUTF8	抅
U+6285.0	fCNS11643	3-2637
U+6285.0	fGB	5-4B64
U+6285.0	fR/S	 64.4
U+6285.0	fTotalStrokes	 7
U+6285.0	fCangjie	QPI
U+6285.0	fFourCorner	57020
U+6285.0	fHakka	 gi1;  ki1; geu1
U+6285.0	fCantonese	keoi1
U+6285.0	fMandarin	ju1; gou1; ju3; ju4
U+6285.0	fEnglish	to collect; to join together
U+6286.0	fUTF8	抆
U+6286.0	fCNS11643	1-4A50
U+6286.0	fBig5	A7EE
U+6286.0	fGB	3-496F
U+6286.0	fR/S	 64.4
U+6286.0	fTotalStrokes	 7
U+6286.0	fCangjie	QYK
U+6286.0	fFourCorner	50041
U+6286.0	fHakka	 wun3;  mun3; vun3
U+6286.0	fCantonese	man5; man2 (also)
U+6286.0	fMandarin	wen4; wen3
U+6286.0	fEnglish	wipe; rub off
U+6287.0	fUTF8	抇
U+6287.0	fCNS11643	2-242E
U+6287.0	fBig5	CAEC
U+6287.0	fGB	3-4964
U+6287.0	fR/S	 64.4
U+6287.0	fTotalStrokes	 7
U+6287.0	fCangjie	QA
U+6287.0	fFourCorner	56000
U+6287.0	fCantonese	wat6
U+6287.0	fMandarin	hu2; jue2; gu3; hu1
U+6288.0	fUTF8	抈
U+6288.0	fCNS11643	2-2434
U+6288.0	fBig5	CAF2
U+6288.0	fGB	3-496C
U+6288.0	fR/S	 64.4
U+6288.0	fTotalStrokes	 7
U+6288.0	fCangjie	QB
U+6288.0	fFourCorner	57020
U+6288.0	fHakka	 ngiet7; je1; ze1; nget8
U+6288.0	fCantonese	jyut6
U+6288.0	fMandarin	yue4
U+6288.0	fEnglish	 bend
U+6289.0	fUTF8	抉
U+6289.0	fCNS11643	1-4A42
U+6289.0	fBig5	A7E0
U+6289.0	fGB	0-3E71
U+6289.0	fR/S	 64.4
U+6289.0	fTotalStrokes	 7
U+6289.0	fCangjie	QDK
U+6289.0	fFourCorner	55030
U+6289.0	fHakka	  giet7; kiet8; get7
U+6289.0	fCantonese	kyut3
U+6289.0	fMandarin	jue2
U+6289.0	fEnglish	[1] choose; select; pick [2] gouge; dig
U+628A.0	fUTF8	把
U+628A.0	fCNS11643	1-4A44
U+628A.0	fBig5	A7E2
U+628A.0	fGB	0-3051
U+628A.0	fR/S	 64.4
U+628A.0	fTotalStrokes	 7
U+628A.0	fCangjie	QAU
U+628A.0	fFourCorner	57017
U+628A.0	fHakka	  ba3; ba5; ba2
U+628A.0	fCantonese	baa2; baa3; paa4
U+628A.0	fMandarin	ba3; ba1
U+628A.0	fEnglish	[1] handle; hold [2] hold [3] take [4] guard; watch over; keep under surveillance [5] bundle; grasp; handful [6] around; about; approximate; more or less [7] sworn [8] give [9] used before a direct object, followed by a transitive verb (colloquial)
U+628A.1	fUTF8	把
U+628A.1	fCNS11643	1-4A44
U+628A.1	fBig5	A7E2
U+628A.1	fGB	0-3051
U+628A.1	fR/S	 64.4
U+628A.1	fTotalStrokes	 7
U+628A.1	fCangjie	QAU
U+628A.1	fFourCorner	57017
U+628A.1	fMandarin	ba4
U+628A.1	fEnglish	 handle
U+628B.0	fUTF8	抋
U+628B.0	fCNS11643	3-262F
U+628B.0	fGB	3-4971
U+628B.0	fR/S	 64.4
U+628B.0	fTotalStrokes	 7; 8
U+628B.0	fCangjie	QP
U+628B.0	fFourCorner	53000
U+628B.0	fCantonese	cam3
U+628B.0	fMandarin	qin4
U+628C.0	fUTF8	抌
U+628C.0	fCNS11643	2-242A
U+628C.0	fBig5	CAE8
U+628C.0	fGB	3-4970
U+628C.0	fR/S	 64.4
U+628C.0	fTotalStrokes	 7
U+628C.0	fCangjie	QLBU
U+628C.0	fFourCorner	54012
U+628C.0	fHakka	 zem3; zhim1; jau3; zihm1
U+628C.0	fCantonese	dam2
U+628C.0	fMandarin	zhen3; shen4; dan3; you2; yu3
U+628C.0	fEnglish	[1] bale out water [2] dip
U+628D.0	fUTF8	抍
U+628D.0	fCNS11643	3-2636
U+628D.0	fGB	3-4967
U+628D.0	fR/S	 64.4
U+628D.0	fTotalStrokes	 7
U+628D.0	fCangjie	QHT
U+628D.0	fFourCorner	54000
U+628D.0	fHakka	 zhin1
U+628D.0	fCantonese	sing1
U+628D.0	fMandarin	zheng3
U+628E.0	fUTF8	抎
U+628E.0	fCNS11643	2-242B
U+628E.0	fBig5	CAE9
U+628E.0	fGB	3-4960
U+628E.0	fR/S	 64.4
U+628E.0	fTotalStrokes	 7
U+628E.0	fCangjie	QMMI
U+628E.0	fFourCorner	51031
U+628E.0	fHakka	 jun3; run3
U+628E.0	fCantonese	wan5; sin6
U+628E.0	fMandarin	yun3
U+628E.0	fEnglish	 lose; be vanquished
U+628F.0	fUTF8	抏
U+628F.0	fCNS11643	2-242C
U+628F.0	fBig5	CAEA
U+628F.0	fGB	3-495F
U+628F.0	fR/S	 64.4
U+628F.0	fTotalStrokes	 7
U+628F.0	fCangjie	QMMU
U+628F.0	fFourCorner	51011
U+628F.0	fCantonese	jyun4; waan4; wun6 (also)
U+628F.0	fMandarin	wan2; wan4
U+6290.0	fUTF8	抐
U+6290.0	fCNS11643	4-2439
U+6290.0	fGB	3-4965
U+6290.0	fR/S	  64.4
U+6290.0	fTotalStrokes	 7
U+6290.0	fCangjie	QOB
U+6290.0	fFourCorner	54020
U+6290.0	fHakka	  nap8;  nut8
U+6290.0	fCantonese	neot6; leot6 (also)
U+6290.0	fMandarin	nu4; ne4; ni4
U+6290.0	fEnglish	[1] dip in [2] immerse
U+6291.0	fUTF8	抑
U+6291.0	fCNS11643	1-4A4F
U+6291.0	fBig5	A7ED
U+6291.0	fGB	0-5256
U+6291.0	fR/S	 64.4
U+6291.0	fTotalStrokes	 7
U+6291.0	fCangjie	QHVL
U+6291.0	fFourCorner	57020
U+6291.0	fHakka	  jit7; et7; ngiong3; rit7; jit8; ra6; ja5
U+6291.0	fCantonese	jik1
U+6291.0	fMandarin	yi4
U+6291.0	fEnglish	[1] press down; repress [2] restrain; force to (do, perform, etc.) [3] bend or lower (one's head) [4] or; or if; still; else; either; then [5] but; an opening particle of an expression [6] an exclamatory; oh; alas [7] stop
U+6292.0	fUTF8	抒
U+6292.0	fCNS11643	1-4A49
U+6292.0	fBig5	A7E7
U+6292.0	fGB	0-4A63
U+6292.0	fR/S	 64.4
U+6292.0	fTotalStrokes	 7
U+6292.0	fCangjie	QNIN
U+6292.0	fFourCorner	57022
U+6292.0	fHakka	 shu3; su3; su1
U+6292.0	fCantonese	syu1
U+6292.0	fMandarin	shu1; shu4; zhu4
U+6292.0	fEnglish	[1] express; give expression to [2] relieve; ease; lighten; unburden
U+6293.0	fUTF8	抓
U+6293.0	fCNS11643	1-4A4E
U+6293.0	fBig5	A7EC
U+6293.0	fGB	0-5725
U+6293.0	fR/S	 64.4
U+6293.0	fTotalStrokes	 7
U+6293.0	fCangjie	QHLO
U+6293.0	fFourCorner	52030
U+6293.0	fHakka	 zau3; za1
U+6293.0	fCantonese	zaau2; zaa1 (also); aau1 (also); ngaau1 (also); waa2 (also)
U+6293.0	fMandarin	zhua1; zhao1; zhao3; zhao4
U+6293.0	fEnglish	[1] scratch [2] seize; grab; grasp; take; snatch; make a snatch at; clutch; catch [3] arrest
U+6294.0	fUTF8	抔
U+6294.0	fCNS11643	2-242D
U+6294.0	fBig5	CAEB
U+6294.0	fGB	3-4962
U+6294.0	fR/S	 64.4
U+6294.0	fTotalStrokes	 7
U+6294.0	fCangjie	QMF
U+6294.0	fFourCorner	51090
U+6294.0	fHakka	  piau1;  but7; bui1; bi1
U+6294.0	fCantonese	pau4
U+6294.0	fMandarin	pou2; pei1; bao4
U+6294.0	fEnglish	 scoop up with both hands
U+6295.0	fUTF8	投
U+6295.0	fCNS11643	1-4A4D
U+6295.0	fBig5	A7EB
U+6295.0	fGB	0-4D36
U+6295.0	fR/S	 64.4
U+6295.0	fTotalStrokes	 7
U+6295.0	fCangjie	QHNE
U+6295.0	fFourCorner	57047
U+6295.0	fHakka	 teu2; dau2; teu3; tiu2; teu5
U+6295.0	fCantonese	tau4
U+6295.0	fMandarin	tou2
U+6295.0	fEnglish	[1] throw; cast; fling; pitch; toss [2] present as a gift [3] lodge; stay [4] head (west, etc.) [5] agreeable; congenial; harmonious; fit in with; cater to [6] join; submit to [7] project; cast [8] deliver (mail, etc.); send (letters, scripts, etc.)
U+6296.0	fUTF8	抖
U+6296.0	fCNS11643	1-4A3F
U+6296.0	fBig5	A7DD
U+6296.0	fGB	0-3636
U+6296.0	fR/S	 64.4
U+6296.0	fTotalStrokes	 7
U+6296.0	fCangjie	QYJ
U+6296.0	fFourCorner	54000
U+6296.0	fHakka	 deu3; diu3; shu1; shiu1
U+6296.0	fCantonese	dau2
U+6296.0	fMandarin	dou3
U+6296.0	fEnglish	[1] shiver; tremble [2] shake [3] rouse [4] make good; become well-to-do (colloquial) [5] work in wood [6] touch
U+6297.0	fUTF8	抗
U+6297.0	fCNS11643	1-4A3E
U+6297.0	fBig5	A7DC
U+6297.0	fGB	0-3F39
U+6297.0	fR/S	 64.4
U+6297.0	fTotalStrokes	 7
U+6297.0	fCangjie	QYHN
U+6297.0	fFourCorner	50017
U+6297.0	fHakka	  kong5
U+6297.0	fCantonese	kong3
U+6297.0	fMandarin	kang4; kang2
U+6297.0	fEnglish	[1] resist; oppose [2] defy; reject; refute; rebuke [3] high and virtuous [4] raise; set up [5] hide; conceal; screen; secrete [6] a Chinese family name
U+6298.0	fUTF8	折
U+6298.0	fCNS11643	1-4A4B
U+6298.0	fBig5	A7E9
U+6298.0	fGB	0-555B
U+6298.0	fR/S	 64.4
U+6298.0	fTotalStrokes	 7
U+6298.0	fCangjie	QHML
U+6298.0	fFourCorner	52021
U+6298.0	fHakka	 zet7; zhat7; cak7; ziet7; zat7; zhat8; zat8; zap7; zhet7
U+6298.0	fCantonese	zit3; zip3; sit6
U+6298.0	fMandarin	zhe2; ti2
U+6298.0	fEnglish	[1] break; snap [2] bend; humble; bow [3] decide a course; judge [4] sell; barter; exchange [5] discount in price [6] fold [7] tear into halves; destroy
U+6298.1	fUTF8	折
U+6298.1	fCNS11643	1-4A4B
U+6298.1	fBig5	A7E9
U+6298.1	fGB	0-555B
U+6298.1	fR/S	 64.4
U+6298.1	fTotalStrokes	 7
U+6298.1	fCangjie	QHML
U+6298.1	fFourCorner	52021
U+6298.1	fMandarin	she2
U+6298.1	fEnglish	[1] lose money; fail in business [2] break; snap
U+6298.2	fUTF8	折
U+6298.2	fCNS11643	1-4A4B
U+6298.2	fBig5	A7E9
U+6298.2	fGB	0-555B
U+6298.2	fR/S	 64.4
U+6298.2	fTotalStrokes	 7
U+6298.2	fCangjie	QHML
U+6298.2	fFourCorner	52021
U+6298.2	fMandarin	zhe1
U+6298.2	fEnglish	[1] turn upside down; fall head over heels; somersault; [2] pour all out
U+6299.0	fUTF8	抙
U+6299.0	fCNS11643	3-2635
U+6299.0	fGB	5-4B5D
U+6299.0	fR/S	  64.4
U+6299.0	fTotalStrokes	 7
U+6299.0	fCangjie	QQ
U+6299.0	fFourCorner	52050
U+6299.0	fHakka	 feu2; piau2; p'ieu2; piau1
U+6299.0	fCantonese	pau4
U+6299.0	fMandarin	pou2; fou1; fou2; fu2; po1
U+6299.0	fEnglish	same as &#x634A;
U+629A.0	fUTF8	抚
U+629A.0	fCNS11643	4-243C
U+629A.0	fGB	0-3827
U+629A.0	fR/S	 64.4
U+629A.0	fTotalStrokes	 7
U+629A.0	fCangjie	QMKU
U+629A.0	fFourCorner	51010
U+629A.0	fHakka	 vu3
U+629A.0	fCantonese	fu2
U+629A.0	fMandarin	fu3; mu2; hu1
U+629A.0	fEnglish	[1] non-classical form of &#x64AB; [2] simplified form of &#x64AB;
U+629B.0	fUTF8	抛
U+629B.0	fCNS11643	3-2633
U+629B.0	fGB	0-4557
U+629B.0	fR/S	 64.4
U+629B.0	fTotalStrokes	 7
U+629B.0	fCangjie	QKNS
U+629B.0	fFourCorner	54010
U+629B.0	fHakka	 pau1
U+629B.0	fCantonese	paau1
U+629B.0	fMandarin	pao1; pao4
U+629B.0	fEnglish	throw (away)
U+629C.0	fUTF8	抜
U+629C.0	fCNS11643	3-2634
U+629C.0	fGB	E-2B24
U+629C.0	fR/S	 64.4
U+629C.0	fTotalStrokes	 7
U+629C.0	fCangjie	QKE
U+629C.0	fFourCorner	54040
U+629C.0	fHakka	 pat8
U+629C.0	fCantonese	bat6
U+629C.0	fMandarin	ba2; bie2; fa2; jie3; pei4
U+629C.0	fEnglish	simplified form of &#x62D4;
U+629D.0	fUTF8	抝
U+629D.0	fCNS11643	3-2638
U+629D.0	fGB	E-2B25
U+629D.0	fR/S	 64.4
U+629D.0	fTotalStrokes	 7
U+629D.0	fCangjie	QVIS
U+629D.0	fFourCorner	57020
U+629D.0	fHakka	 au3; au5
U+629D.0	fCantonese	aau2; aau3; ngaau2 (also); ngaau3 (also)
U+629D.0	fMandarin	ao3; niu4
U+629E.0	fUTF8	択
U+629E.0	fCNS11643	3-2631
U+629E.0	fGB	E-2B26
U+629E.0	fR/S	 64.4
U+629E.0	fTotalStrokes	 7
U+629E.0	fCangjie	QSO
U+629E.0	fFourCorner	57030
U+629E.0	fHakka	 cak8; cet8; tok8
U+629E.0	fCantonese	zaak6
U+629E.0	fMandarin	ze2; zhai2; yi4
U+629E.0	fEnglish	simplified form of &#x64C7;
U+629F.0	fUTF8	抟
U+629F.0	fGB	0-5E52
U+629F.0	fR/S	 64.4
U+629F.0	fTotalStrokes	 7
U+629F.0	fCangjie	QQNI
U+629F.0	fHakka	 ton2
U+629F.0	fCantonese	tyun4; zyun1
U+629F.0	fMandarin	tuan2; zhuan1; gua1; zhuan4
U+629F.0	fEnglish	simplified form of &#x6476;
U+62A0.0	fUTF8	抠
U+62A0.0	fGB	0-3F59
U+62A0.0	fR/S	 64.4
U+62A0.0	fTotalStrokes	 7
U+62A0.0	fCangjie	QSK
U+62A0.0	fHakka	 keu1
U+62A0.0	fCantonese	kau1
U+62A0.0	fMandarin	kou1; ou3; qu1
U+62A0.0	fEnglish	simplified form of &#x6473;
U+62A1.0	fUTF8	抡
U+62A1.0	fGB	0-4255
U+62A1.0	fR/S	 64.4
U+62A1.0	fTotalStrokes	 7
U+62A1.0	fCangjie	QOP
U+62A1.0	fHakka	 lun2
U+62A1.0	fCantonese	leon4
U+62A1.0	fMandarin	lun2; lun1
U+62A1.0	fEnglish	simplified form of &#x6384;
U+62A2.0	fUTF8	抢
U+62A2.0	fGB	0-4740
U+62A2.0	fR/S	 64.4
U+62A2.0	fTotalStrokes	 7
U+62A2.0	fCangjie	QOSU
U+62A2.0	fHakka	 ciong3; cong5
U+62A2.0	fCantonese	coeng2; coeng1
U+62A2.0	fMandarin	qiang3; cheng1; qiang1; chuang3; cang1; chuan3; qiang4
U+62A2.0	fEnglish	simplified form of &#x6436;
U+62A3.0	fUTF8	抣
U+62A3.0	fCNS11643	3-647B
U+62A3.0	fGB	E-2B27
U+62A3.0	fR/S	 64.4
U+62A3.0	fTotalStrokes	 7
U+62A3.0	fCangjie	XXQPM
U+62A3.0	fFourCorner	5702
U+62A3.0	fCantonese	gwan1
U+62A4.0	fUTF8	护
U+62A4.0	fCNS11643	F-2439
U+62A4.0	fGB	0-3B24
U+62A4.0	fR/S	 64.4
U+62A4.0	fTotalStrokes	 7
U+62A4.0	fCangjie	QIS
U+62A4.0	fHakka	 fu5
U+62A4.0	fCantonese	wu6
U+62A4.0	fMandarin	hu4
U+62A4.0	fEnglish	simplified form of &#x8B77;
U+62A5.0	fUTF8	报
U+62A5.0	fCNS11643	3-2632
U+62A5.0	fGB	0-3128
U+62A5.0	fR/S	 64.4
U+62A5.0	fTotalStrokes	 7
U+62A5.0	fCangjie	QSLE
U+62A5.0	fFourCorner	57040
U+62A5.0	fHakka	 bau5
U+62A5.0	fCantonese	bou3
U+62A5.0	fMandarin	bao4
U+62A5.0	fEnglish	simplified form of &#x5831;
U+62A6.0	fUTF8	抦
U+62A6.0	fCNS11643	3-2931
U+62A6.0	fGB	5-4B69
U+62A6.0	fR/S	 64.5
U+62A6.0	fTotalStrokes	 8
U+62A6.0	fCangjie	QMOB
U+62A6.0	fFourCorner	51020
U+62A6.0	fHakka	 biang1; bin3
U+62A6.0	fCantonese	bing2
U+62A6.0	fMandarin	bing3
U+62A6.0	fEnglish	beat
U+62A7.0	fUTF8	抧
U+62A7.0	fCNS11643	3-292A
U+62A7.0	fGB	5-4B6D
U+62A7.0	fR/S	 64.5
U+62A7.0	fTotalStrokes	 8
U+62A7.0	fCangjie	QRC
U+62A7.0	fFourCorner	56080
U+62A7.0	fCantonese	zi2
U+62A7.0	fMandarin	zhi3
U+62A8.0	fUTF8	抨
U+62A8.0	fCNS11643	1-4D63
U+62A8.0	fBig5	A9E1
U+62A8.0	fGB	0-456A
U+62A8.0	fR/S	 64.5
U+62A8.0	fTotalStrokes	 8
U+62A8.0	fCangjie	QMFJ
U+62A8.0	fFourCorner	51049
U+62A8.0	fHakka	  pin2; biang1; piang1; bin3
U+62A8.0	fCantonese	ping1; ping4 (also); paang1 (also)
U+62A8.0	fMandarin	peng1; beng1
U+62A8.0	fEnglish	 impeach; censure; attack or assail by words
U+62A9.0	fUTF8	抩
U+62A9.0	fCNS11643	2-267D
U+62A9.0	fBig5	CCBE
U+62A9.0	fGB	5-4B6E
U+62A9.0	fR/S	 64.5
U+62A9.0	fTotalStrokes	 8
U+62A9.0	fCangjie	QGB
U+62A9.0	fFourCorner	55057
U+62A9.0	fCantonese	naam4; laam4 (also)
U+62A9.0	fMandarin	tan1
U+62AA.0	fUTF8	抪
U+62AA.0	fCNS11643	2-2676
U+62AA.0	fBig5	CCB7
U+62AA.0	fGB	3-4976
U+62AA.0	fR/S	 64.5
U+62AA.0	fTotalStrokes	 8
U+62AA.0	fCangjie	QKLB
U+62AA.0	fFourCorner	54027
U+62AA.0	fHakka	 bu5; pu2
U+62AA.0	fCantonese	bo3; bou3
U+62AA.0	fMandarin	pu1; bu4
U+62AA.0	fEnglish	 scatter; disperse
U+62AB.0	fUTF8	披
U+62AB.0	fCNS11643	1-4D5E
U+62AB.0	fBig5	A9DC
U+62AB.0	fGB	0-457B
U+62AB.0	fR/S	 64.5
U+62AB.0	fTotalStrokes	 8
U+62AB.0	fCangjie	QDHE
U+62AB.0	fFourCorner	54047
U+62AB.0	fHakka	  pi1;  pia1; pi6; bi3
U+62AB.0	fCantonese	pei1
U+62AB.0	fMandarin	pi1; bi4; pa4; pei1; pi2
U+62AB.0	fEnglish	[1] open (a book, scroll, etc.) unroll [2] spread out; disperse [3] thumb through; read casually [4] throw on (a garment, etc.); wear untidily
U+62AC.0	fUTF8	抬
U+62AC.0	fCNS11643	1-4D71
U+62AC.0	fBig5	A9EF
U+62AC.0	fGB	0-4C27
U+62AC.0	fR/S	 64.5
U+62AC.0	fTotalStrokes	 8
U+62AC.0	fCangjie	QIR
U+62AC.0	fFourCorner	53060
U+62AC.0	fHakka	  toi2
U+62AC.0	fCantonese	toi4
U+62AC.0	fMandarin	tai2; chi1
U+62AC.0	fEnglish	lift; carry
U+62AD.0	fUTF8	抭
U+62AD.0	fCNS11643	2-2672
U+62AD.0	fBig5	CCB3
U+62AD.0	fGB	3-4A23
U+62AD.0	fR/S	 64.5
U+62AD.0	fTotalStrokes	 8
U+62AD.0	fCangjie	QJHU
U+62AD.0	fFourCorner	50012
U+62AD.0	fCantonese	jiu5
U+62AD.0	fMandarin	yao3; you2; tao1; yu2
U+62AE.0	fUTF8	抮
U+62AE.0	fCNS11643	2-2679
U+62AE.0	fBig5	CCBA
U+62AE.0	fGB	3-497D
U+62AE.0	fR/S	 64.5
U+62AE.0	fTotalStrokes	 8
U+62AE.0	fCangjie	QOHH
U+62AE.0	fFourCorner	58022
U+62AE.0	fHakka	 zhin3; zin3
U+62AE.0	fCantonese	zan2; hin2 (also); can2
U+62AE.0	fMandarin	zhen3; nian3; tian3; xian3
U+62AF.0	fUTF8	抯
U+62AF.0	fCNS11643	2-267B
U+62AF.0	fBig5	CCBC
U+62AF.0	fGB	5-4B6C
U+62AF.0	fR/S	 64.5
U+62AF.0	fTotalStrokes	 8
U+62AF.0	fCangjie	QBM
U+62AF.0	fFourCorner	57010
U+62AF.0	fHakka	 za1
U+62AF.0	fCantonese	zaa1; ce2
U+62AF.0	fMandarin	zha1; qie3; xie4
U+62AF.0	fEnglish	grasp, hold
U+62B0.0	fUTF8	抰
U+62B0.0	fCNS11643	2-267E
U+62B0.0	fBig5	CCBF
U+62B0.0	fGB	3-497A
U+62B0.0	fR/S	 64.5
U+62B0.0	fTotalStrokes	 8
U+62B0.0	fCangjie	QLBK
U+62B0.0	fFourCorner	55030
U+62B0.0	fHakka	 jong3; rong3; gniong3
U+62B0.0	fCantonese	joeng2; joeng1
U+62B0.0	fMandarin	yang3; yang1
U+62B0.0	fEnglish	 whip; beat
U+62B1.0	fUTF8	抱
U+62B1.0	fCNS11643	1-4D6C
U+62B1.0	fBig5	A9EA
U+62B1.0	fGB	0-3127
U+62B1.0	fR/S	 64.5
U+62B1.0	fTotalStrokes	 8
U+62B1.0	fCangjie	QPRU
U+62B1.0	fFourCorner	57012
U+62B1.0	fHakka	  pau5; pau6; pau3; bau1
U+62B1.0	fCantonese	pou5; bou6; paau1
U+62B1.0	fMandarin	bao4; bao1; fou2; ni4; pao2; pou2
U+62B1.0	fEnglish	[1] embrace; hold in the arms; enfold [2] harbour; cherish; bosom [3] adopt; aspirations; ambition
U+62B2.0	fUTF8	抲
U+62B2.0	fCNS11643	3-2926
U+62B2.0	fGB	3-4975
U+62B2.0	fR/S	 64.5
U+62B2.0	fTotalStrokes	 8; 14
U+62B2.0	fCangjie	QMNR
U+62B2.0	fFourCorner	51020
U+62B2.0	fCantonese	ho1
U+62B2.0	fMandarin	he1
U+62B3.0	fUTF8	抳
U+62B3.0	fCNS11643	2-267A
U+62B3.0	fBig5	CCBB
U+62B3.0	fGB	5-4B73
U+62B3.0	fR/S	 64.5
U+62B3.0	fTotalStrokes	 8
U+62B3.0	fCangjie	QSP
U+62B3.0	fFourCorner	57011
U+62B3.0	fHakka	 ni3
U+62B3.0	fCantonese	nei5; lei5 (also); ni4; nai4
U+62B3.0	fMandarin	ni3; ni2; ni4
U+62B3.0	fEnglish	crush; mash; grind
U+62B4.0	fUTF8	抴
U+62B4.0	fCNS11643	2-2673
U+62B4.0	fBig5	CCB4
U+62B4.0	fGB	5-4B68
U+62B4.0	fR/S	 64.5
U+62B4.0	fTotalStrokes	 8
U+62B4.0	fCangjie	QPT
U+62B4.0	fFourCorner	54017
U+62B4.0	fCantonese	jit6; jai6; zai6
U+62B4.0	fMandarin	yi4; ye4; she2
U+62B5.0	fUTF8	抵
U+62B5.0	fCNS11643	1-4D6A
U+62B5.0	fBig5	A9E8
U+62B5.0	fGB	0-3556
U+62B5.0	fR/S	 64.5
U+62B5.0	fTotalStrokes	 8
U+62B5.0	fCangjie	QHPM
U+62B5.0	fFourCorner	52040
U+62B5.0	fHakka	  di3; du2; dai3; dai1; din3
U+62B5.0	fCantonese	dai2; zi2
U+62B5.0	fMandarin	di3; di2; qu2; zhi3
U+62B5.0	fEnglish	[1] resist; oppose [2] prop; sustain [3] offset; balance [4] substitute; give as an equivalent [5] offer as collateral [6] arrive at; reach (a place) [7] go against; offend against (the law and regulations)
U+62B6.0	fUTF8	抶
U+62B6.0	fCNS11643	2-2677
U+62B6.0	fBig5	CCB8
U+62B6.0	fGB	3-497B
U+62B6.0	fR/S	 64.5
U+62B6.0	fTotalStrokes	 8
U+62B6.0	fCangjie	QHQO
U+62B6.0	fFourCorner	55030
U+62B6.0	fHakka	 zhit7; tsiit7
U+62B6.0	fCantonese	cik1; maat3; mut3
U+62B6.0	fMandarin	chi4; zhi4
U+62B6.0	fEnglish	beat
U+62B7.0	fUTF8	抷
U+62B7.0	fCNS11643	3-2928
U+62B7.0	fGB	E-2B28
U+62B7.0	fR/S	  64.5
U+62B7.0	fTotalStrokes	 8
U+62B7.0	fCangjie	QMFM
U+62B7.0	fFourCorner	51010
U+62B7.0	fHakka	 piau1
U+62B7.0	fCantonese	pei1
U+62B7.0	fMandarin	pi1
U+62B8.0	fUTF8	抸
U+62B8.0	fCNS11643	2-2721
U+62B8.0	fBig5	CCC0
U+62B8.0	fGB	3-496B
U+62B8.0	fR/S	 64.5
U+62B8.0	fTotalStrokes	 8
U+62B8.0	fCangjie	QHIO
U+62B8.0	fFourCorner	42037
U+62B8.0	fHakka	 hiap8; giap8; kiap8
U+62B8.0	fCantonese	hip3; zaap3
U+62B8.0	fMandarin	za1
U+62B9.0	fUTF8	抹
U+62B9.0	fCNS11643	1-4D5B
U+62B9.0	fBig5	A9D9
U+62B9.0	fGB	0-4428
U+62B9.0	fR/S	 64.5
U+62B9.0	fTotalStrokes	 8
U+62B9.0	fCangjie	QDJ
U+62B9.0	fFourCorner	55090
U+62B9.0	fHakka	  mat7; mat8
U+62B9.0	fCantonese	mut3; maat3 (also); mut6
U+62B9.0	fMandarin	mo3; mo4; wa4
U+62B9.0	fEnglish	[1] smear; apply to [2] wipe; mop; rub [3] obliterate; blot out
U+62B9.1	fUTF8	抹
U+62B9.1	fCNS11643	1-4D5B
U+62B9.1	fBig5	A9D9
U+62B9.1	fGB	0-4428
U+62B9.1	fR/S	 64.5
U+62B9.1	fTotalStrokes	 8
U+62B9.1	fCangjie	QDJ
U+62B9.1	fFourCorner	55090
U+62B9.1	fMandarin	ma1
U+62B9.1	fEnglish	[1] plaster [2] tight undergarment [3] turn
U+62BA.0	fUTF8	抺
U+62BA.0	fCNS11643	3-2925
U+62BA.0	fGB	E-2B29
U+62BA.0	fR/S	  64.5
U+62BA.0	fTotalStrokes	 8
U+62BA.0	fCangjie	QJD
U+62BA.0	fFourCorner	55090
U+62BA.0	fHakka	 mi2; wui6; wi5; mui6; mi5; mui5
U+62BA.0	fCantonese	mui6
U+62BA.0	fMandarin	mo3; mei4
U+62BA.0	fEnglish	 feel with the hand
U+62BB.0	fUTF8	抻
U+62BB.0	fCNS11643	2-267C
U+62BB.0	fBig5	CCBD
U+62BB.0	fGB	0-5E53
U+62BB.0	fR/S	 64.5
U+62BB.0	fTotalStrokes	 8
U+62BB.0	fCangjie	QLWL
U+62BB.0	fFourCorner	55006
U+62BB.0	fCantonese	can2
U+62BB.0	fMandarin	shen1; chen1; shen4
U+62BB.0	fEnglish	pull
U+62BC.0	fUTF8	押
U+62BC.0	fCNS11643	1-4D65
U+62BC.0	fBig5	A9E3
U+62BC.0	fGB	0-513A
U+62BC.0	fR/S	 64.5
U+62BC.0	fTotalStrokes	 8
U+62BC.0	fCangjie	QWL
U+62BC.0	fFourCorner	56050
U+62BC.0	fHakka	  ap7
U+62BC.0	fCantonese	aat3; ngaat3 (also); aap3; aat2 (also); ngaap3 (also); ngaat2 (also)
U+62BC.0	fMandarin	ya1; xia2; ya2; jia3
U+62BC.0	fEnglish	[1] mortgage; pledge; deposit; pawn; obtain loans against securities [2] detain; imprison (temporarily) [3] escort [4] signature
U+62BD.0	fUTF8	抽
U+62BD.0	fCNS11643	1-4D64
U+62BD.0	fBig5	A9E2
U+62BD.0	fGB	0-3369
U+62BD.0	fR/S	 64.5
U+62BD.0	fTotalStrokes	 8
U+62BD.0	fCangjie	QLW
U+62BD.0	fFourCorner	55060
U+62BD.0	fHakka	 cu1;  ciu1;  chiu1; chu1
U+62BD.0	fCantonese	cau1; zau1
U+62BD.0	fMandarin	chou1; chou2
U+62BD.0	fEnglish	[1] draw out; pull out or open [2] sprout; put forth shoots; bud [3] rid; take way [4] whip; lash [5] smoke (cigarettes, etc.) [6] shrink
U+62BE.0	fUTF8	抾
U+62BE.0	fCNS11643	2-2675
U+62BE.0	fBig5	CCB6
U+62BE.0	fGB	3-4973
U+62BE.0	fR/S	 64.5
U+62BE.0	fTotalStrokes	 8
U+62BE.0	fCangjie	QGI
U+62BE.0	fFourCorner	54031
U+62BE.0	fHakka	 ki1; giap7
U+62BE.0	fCantonese	keoi1; gip3; hei1
U+62BE.0	fMandarin	qu1; jie2; qi1; qie4
U+62BE.0	fEnglish	[1] feel for a thing [2] grasp
U+62BF.0	fUTF8	抿
U+62BF.0	fCNS11643	1-4D59
U+62BF.0	fBig5	A9D7
U+62BF.0	fGB	0-4372
U+62BF.0	fR/S	 64.5
U+62BF.0	fTotalStrokes	 8
U+62BF.0	fCangjie	QRVP
U+62BF.0	fFourCorner	57047
U+62BF.0	fHakka	  min2; men3; man2; min3
U+62BF.0	fCantonese	man5; man2 (also); man4
U+62BF.0	fMandarin	min3; min2; wen3
U+62BF.0	fEnglish	[1] smooth (hair); stroke; caress [2] purse up (lips); contract; tuck [3] sip
U+62C0.0	fUTF8	拀
U+62C0.0	fCNS11643	3-2934
U+62C0.0	fGB	E-2B2A
U+62C0.0	fR/S	 64.5
U+62C0.0	fTotalStrokes	 8
U+62C0.0	fCangjie	QRHU
U+62C0.0	fFourCorner	56010
U+62C0.0	fHakka	 zhuk7; chuk7
U+62C0.0	fCantonese	zuk1
U+62C0.0	fMandarin	chu4; zhu4
U+62C0.0	fEnglish	corrupted form of &#x67F7;
U+62C1.0	fUTF8	拁
U+62C1.0	fCNS11643	3-2932
U+62C1.0	fGB	E-2B2B
U+62C1.0	fR/S	  64.5
U+62C1.0	fTotalStrokes	 8
U+62C1.0	fCangjie	QKSR
U+62C1.0	fFourCorner	56000
U+62C1.0	fHakka	 ga1
U+62C1.0	fCantonese	ke2; gaa1
U+62C1.0	fMandarin	jia1
U+62C1.0	fEnglish	corrupted form of &#x67B7;
U+62C2.0	fUTF8	拂
U+62C2.0	fCNS11643	1-4D5A
U+62C2.0	fBig5	A9D8
U+62C2.0	fGB	0-3777
U+62C2.0	fR/S	 64.5
U+62C2.0	fTotalStrokes	 8
U+62C2.0	fCangjie	QLLN
U+62C2.0	fFourCorner	55027
U+62C2.0	fHakka	  fut7;  fit8;  fin5
U+62C2.0	fCantonese	fat1; bat6; faak1
U+62C2.0	fMandarin	fu2; fei4; pi3
U+62C2.0	fEnglish	[1] shake; brush; whisk [2] dust; duster [3] oppose; disobey [4] expel; drive away
U+62C2.1	fUTF8	拂
U+62C2.1	fCNS11643	1-4D5A
U+62C2.1	fBig5	A9D8
U+62C2.1	fGB	0-3777
U+62C2.1	fR/S	 64.5
U+62C2.1	fTotalStrokes	 8
U+62C2.1	fCangjie	QLLN
U+62C2.1	fFourCorner	55027
U+62C2.1	fMandarin	bi4
U+62C2.1	fEnglish	[1] aid; assist [2] make correct or right
U+62C3.0	fUTF8	拃
U+62C3.0	fCNS11643	F-2730
U+62C3.0	fGB	3-497C
U+62C3.0	fR/S	 64.5
U+62C3.0	fTotalStrokes	 8
U+62C3.0	fCangjie	QOS
U+62C3.0	fFourCorner	5801
U+62C3.0	fHakka	 zan3;  cok7; ngiam5; zak7
U+62C3.0	fCantonese	zaa6; zaa3
U+62C3.0	fMandarin	zhan3; zha4
U+62C3.0	fEnglish	[1] grope; press; span [2] obstruct; interrupt; handful; quantity held by a hand
U+62C4.0	fUTF8	拄
U+62C4.0	fCNS11643	1-4D58
U+62C4.0	fBig5	A9D6
U+62C4.0	fGB	0-5674
U+62C4.0	fR/S	 64.5
U+62C4.0	fTotalStrokes	 8
U+62C4.0	fCangjie	QYG
U+62C4.0	fFourCorner	50014
U+62C4.0	fHakka	 zhu5; ku3; zhu3; gu3; zu5; pu3; cu5; chu5; zu3; gu2
U+62C4.0	fCantonese	zyu2
U+62C4.0	fMandarin	zhu3
U+62C4.0	fEnglish	[1] lean on (a stick, etc.) [2] post; prop [3] ridicule; make sarcastic remarks
U+62C5.0	fUTF8	担
U+62C5.0	fCNS11643	3-2933
U+62C5.0	fGB	0-3523
U+62C5.0	fR/S	  64.5
U+62C5.0	fTotalStrokes	 8
U+62C5.0	fCangjie	QAM
U+62C5.0	fFourCorner	56010
U+62C5.0	fHakka	  dam1;  dam5
U+62C5.0	fCantonese	daam1; daam3; daam2 (also)
U+62C5.0	fMandarin	dan1; dan3; qie4
U+62C5.0	fEnglish	simplified form of &#x64D4;
U+62C5.1	fUTF8	担
U+62C5.1	fCNS11643	3-2933
U+62C5.1	fGB	0-3523
U+62C5.1	fR/S	  64.5
U+62C5.1	fTotalStrokes	 8
U+62C5.1	fCangjie	QAM
U+62C5.1	fFourCorner	56010
U+62C5.1	fMandarin	dan4
U+62C5.2	fUTF8	担
U+62C5.2	fCNS11643	3-2933
U+62C5.2	fGB	0-3523
U+62C5.2	fR/S	  64.5
U+62C5.2	fTotalStrokes	 8
U+62C5.2	fCangjie	QAM
U+62C5.2	fFourCorner	56010
U+62C5.2	fMandarin	dan5
U+62C6.0	fUTF8	拆
U+62C6.0	fCNS11643	1-4D70
U+62C6.0	fBig5	A9EE
U+62C6.0	fGB	0-3270
U+62C6.0	fR/S	 64.5
U+62C6.0	fTotalStrokes	 8
U+62C6.0	fCangjie	QHMY
U+62C6.0	fFourCorner	52031
U+62C6.0	fHakka	  cak7
U+62C6.0	fCantonese	caak3
U+62C6.0	fMandarin	chai1; ca1; che4; chi3
U+62C6.0	fEnglish	[1] break; split; rip open [2] take down; tear down (a house, tec.); destroy; dismantle; take apart
U+62C7.0	fUTF8	拇
U+62C7.0	fCNS11643	1-4D68
U+62C7.0	fBig5	A9E6
U+62C7.0	fGB	0-4434
U+62C7.0	fR/S	 64.5
U+62C7.0	fTotalStrokes	 8
U+62C7.0	fCangjie	QWYI
U+62C7.0	fFourCorner	57050
U+62C7.0	fHakka	  mu3; mu1; meu6; meu5; meu3; meu1
U+62C7.0	fCantonese	mou5
U+62C7.0	fMandarin	mu3; mou3
U+62C7.0	fEnglish	[1] thumb [2] big toe
U+62C8.0	fUTF8	拈
U+62C8.0	fCNS11643	1-4D62
U+62C8.0	fBig5	A9E0
U+62C8.0	fGB	0-4469
U+62C8.0	fR/S	 64.5
U+62C8.0	fTotalStrokes	 8
U+62C8.0	fCangjie	QYR
U+62C8.0	fFourCorner	51060
U+62C8.0	fHakka	 ngiam1; zam1; zham1; gniam1; giam3
U+62C8.0	fCantonese	nim1; lim1 (also); nin2 (also); nim2; lim2 (also); lin2 (also); nin5 (also); lin5 (also); nim4; dim1
U+62C8.0	fMandarin	nian1; nian2; nian3; zhan3; dian1
U+62C8.0	fEnglish	[1] take, hold, or pick with fingers [2] draw lots
U+62C9.0	fUTF8	拉
U+62C9.0	fCNS11643	1-4D56
U+62C9.0	fBig5	A9D4
U+62C9.0	fGB	0-402D
U+62C9.0	fR/S	 64.5
U+62C9.0	fTotalStrokes	 8
U+62C9.0	fCangjie	QYT
U+62C9.0	fFourCorner	50018
U+62C9.0	fHakka	  la1;  lai1; lo1;  lai2;  lap8; la3; lat7
U+62C9.0	fCantonese	laai1; laap6; laat6; laai6; laa1; lap6
U+62C9.0	fMandarin	la1; la3; la4; ba2
U+62C9.0	fEnglish	[1] pull; drag; seize; draw; hold [2] discharge (especially stool, urine, etc.) [3] lengthen; elongate [4] play
U+62C9.1	fUTF8	拉
U+62C9.1	fCNS11643	1-4D56
U+62C9.1	fBig5	A9D4
U+62C9.1	fGB	0-402D
U+62C9.1	fR/S	 64.5
U+62C9.1	fTotalStrokes	 8
U+62C9.1	fCangjie	QYT
U+62C9.1	fFourCorner	50018
U+62C9.1	fMandarin	la2
U+62C9.1	fEnglish	used in combination
U+62C9.2	fUTF8	拉
U+62C9.2	fCNS11643	1-4D56
U+62C9.2	fBig5	A9D4
U+62C9.2	fGB	0-402D
U+62C9.2	fR/S	 64.5
U+62C9.2	fTotalStrokes	 8
U+62C9.2	fCangjie	QYT
U+62C9.2	fFourCorner	50018
U+62C9.2	fMandarin	la5
U+62CA.0	fUTF8	拊
U+62CA.0	fCNS11643	2-2678
U+62CA.0	fBig5	CCB9
U+62CA.0	fGB	0-5E54
U+62CA.0	fR/S	 64.5
U+62CA.0	fTotalStrokes	 8
U+62CA.0	fCangjie	QODI
U+62CA.0	fFourCorner	54000
U+62CA.0	fHakka	  fu5; fu3; fu1
U+62CA.0	fCantonese	fu2
U+62CA.0	fMandarin	fu3; bo1; fu1; fu4
U+62CA.0	fEnglish	[1] slap; tap [2] handle of a vessel or utensil [3] touch with hand lightly or tenderly; pat [4] indulge
U+62CB.0	fUTF8	拋
U+62CB.0	fCNS11643	1-4D61
U+62CB.0	fBig5	A9DF
U+62CB.0	fGB	E-2B2C
U+62CB.0	fR/S	 64.5; 64.4
U+62CB.0	fTotalStrokes	 8
U+62CB.0	fCangjie	QKUS
U+62CB.0	fFourCorner	54012
U+62CB.0	fHakka	  pau1
U+62CB.0	fCantonese	paau1
U+62CB.0	fMandarin	pao1; pao4
U+62CB.0	fEnglish	[1] throw (away); hurt [2] abandon; reject; give up [3] cast aside; leave behind
U+62CC.0	fUTF8	拌
U+62CC.0	fCNS11643	1-4D57
U+62CC.0	fBig5	A9D5
U+62CC.0	fGB	0-3068
U+62CC.0	fR/S	 64.5
U+62CC.0	fTotalStrokes	 8
U+62CC.0	fCangjie	QFQ
U+62CC.0	fFourCorner	59050
U+62CC.0	fHakka	  pan1;  pan5; pan3; biang1; pan2
U+62CC.0	fCantonese	bun6; pun3; pun1; bun3
U+62CC.0	fMandarin	ban4; pan4; pan1
U+62CC.0	fEnglish	 mix
U+62CC.1	fUTF8	拌
U+62CC.1	fCNS11643	1-4D57
U+62CC.1	fBig5	A9D5
U+62CC.1	fGB	0-3068
U+62CC.1	fR/S	 64.5
U+62CC.1	fTotalStrokes	 8
U+62CC.1	fCangjie	QFQ
U+62CC.1	fFourCorner	59050
U+62CC.1	fEnglish	 throw away; abandon
U+62CD.0	fUTF8	拍
U+62CD.0	fCNS11643	1-4D69
U+62CD.0	fBig5	A9E7
U+62CD.0	fGB	0-4544
U+62CD.0	fR/S	 64.5
U+62CD.0	fTotalStrokes	 8
U+62CD.0	fCangjie	QHA
U+62CD.0	fFourCorner	56000
U+62CD.0	fHakka	  pok7;  pak7
U+62CD.0	fCantonese	paak3; paak2 (also)
U+62CD.0	fMandarin	pai1; bo2; po4}; po4; mo4
U+62CD.0	fEnglish	[1] strike with the hand; clap; slap; pat; swat [2] the time or beat of a piece of music [3] fawn; flatter
U+62CE.0	fUTF8	拎
U+62CE.0	fCNS11643	1-4D72
U+62CE.0	fBig5	A9F0
U+62CE.0	fGB	0-4160
U+62CE.0	fR/S	 64.5
U+62CE.0	fTotalStrokes	 8
U+62CE.0	fCangjie	QOII
U+62CE.0	fFourCorner	58037
U+62CE.0	fHakka	  lang2
U+62CE.0	fCantonese	ling1; ning1 (also); ling4
U+62CE.0	fMandarin	ling1; ling2
U+62CE.0	fEnglish	 haul; lift; take; carry
U+62CF.0	fUTF8	拏
U+62CF.0	fCNS11643	2-2A55
U+62CF.0	fBig5	CED4
U+62CF.0	fGB	E-2B2D
U+62CF.0	fR/S	 64.5
U+62CF.0	fTotalStrokes	 9
U+62CF.0	fCangjie	VEQ
U+62CF.0	fFourCorner	47502
U+62CF.0	fHakka	  na1; la1
U+62CF.0	fCantonese	naa4; laa4 (also)
U+62CF.0	fMandarin	na2
U+62CF.0	fEnglish	 take bring grasp; hold arrest
U+62D0.0	fUTF8	拐
U+62D0.0	fCNS11643	1-4D66
U+62D0.0	fBig5	A9E4
U+62D0.0	fGB	0-3955
U+62D0.0	fR/S	 64.5
U+62D0.0	fTotalStrokes	 8
U+62D0.0	fCangjie	QRSH
U+62D0.0	fFourCorner	56027
U+62D0.0	fHakka	 gai3; gwai3; gwai1; kuai3; gwai2
U+62D0.0	fCantonese	gwaai2
U+62D0.0	fMandarin	guai3
U+62D0.0	fEnglish	[1] kidnap; abduct [2] turn; change direction (in walking, driving, etc.) [3] swindle [4] staff for an old person; cane
U+62D1.0	fUTF8	拑
U+62D1.0	fCNS11643	2-2674
U+62D1.0	fBig5	CCB5
U+62D1.0	fGB	5-4B67
U+62D1.0	fR/S	 64.5
U+62D1.0	fTotalStrokes	 8
U+62D1.0	fCangjie	QTM
U+62D1.0	fFourCorner	54070
U+62D1.0	fHakka	 kiam2
U+62D1.0	fCantonese	kim4
U+62D1.0	fMandarin	qian2
U+62D1.0	fEnglish	 hold; grasp
U+62D2.0	fUTF8	拒
U+62D2.0	fCNS11643	1-4D5C
U+62D2.0	fBig5	A9DA
U+62D2.0	fGB	0-3E5C
U+62D2.0	fR/S	 64.5
U+62D2.0	fTotalStrokes	 8
U+62D2.0	fCangjie	QSS
U+62D2.0	fFourCorner	51017
U+62D2.0	fHakka	 ki1; gi3;  ki3; gi1
U+62D2.0	fCantonese	keoi5
U+62D2.0	fMandarin	ju4; ju3; ju1
U+62D2.0	fEnglish	[1] ward off; defend [2] refuse; reject
U+62D3.0	fUTF8	拓
U+62D3.0	fCNS11643	1-4D5F
U+62D3.0	fBig5	A9DD
U+62D3.0	fGB	0-4D58
U+62D3.0	fR/S	 64.5
U+62D3.0	fTotalStrokes	 8
U+62D3.0	fCangjie	QMR
U+62D3.0	fFourCorner	51060
U+62D3.0	fHakka	  tok7; zak7; chak7; tap7
U+62D3.0	fCantonese	tok3; taap3; zek3
U+62D3.0	fMandarin	tuo4; zhi2; shi2
U+62D3.0	fEnglish	[1] expand; open up (new frontiers, etc.); aggrandize; develop [2] push with hands
U+62D3.1	fUTF8	拓
U+62D3.1	fCNS11643	1-4D5F
U+62D3.1	fBig5	A9DD
U+62D3.1	fGB	0-4D58
U+62D3.1	fR/S	 64.5
U+62D3.1	fTotalStrokes	 8
U+62D3.1	fCangjie	QMR
U+62D3.1	fFourCorner	51060
U+62D3.1	fMandarin	ta4
U+62D3.1	fEnglish	 copy characters from an ancient tablet or tomb by rubbing over a paper placed on its surface; make rubbings of inscriptions
U+62D4.0	fUTF8	拔
U+62D4.0	fCNS11643	1-4D60
U+62D4.0	fBig5	A9DE
U+62D4.0	fGB	0-304E
U+62D4.0	fR/S	 64.5
U+62D4.0	fTotalStrokes	 8
U+62D4.0	fCangjie	QIKK
U+62D4.0	fFourCorner	53047
U+62D4.0	fHakka	  pat8; bat8
U+62D4.0	fCantonese	bat6
U+62D4.0	fMandarin	ba2; bie2; bei4; fa2; jie3; pei4
U+62D4.0	fEnglish	[1] uproot; pull out [2] promote (another to a higher position, etc.) [3] stand out; outstanding; remarkable [4] attack and take (a city); capture [5] elevation; above
U+62D4.1	fUTF8	拔
U+62D4.1	fCNS11643	1-4D60
U+62D4.1	fBig5	A9DE
U+62D4.1	fGB	0-304E
U+62D4.1	fR/S	 64.5
U+62D4.1	fTotalStrokes	 8
U+62D4.1	fCangjie	QIKK
U+62D4.1	fFourCorner	53047
U+62D4.1	fMandarin	ba5
U+62D5.0	fUTF8	拕
U+62D5.0	fCNS11643	3-292E
U+62D5.0	fGB	5-4B71
U+62D5.0	fR/S	  64.5
U+62D5.0	fTotalStrokes	 8
U+62D5.0	fCangjie	QJP
U+62D5.0	fFourCorner	53010
U+62D5.0	fHakka	 to1
U+62D5.0	fCantonese	to1
U+62D5.0	fMandarin	tuo1; duo3; tuo4; yi3
U+62D5.0	fEnglish	same as &#x62D6;
U+62D6.0	fUTF8	拖
U+62D6.0	fCNS11643	1-4D6E
U+62D6.0	fBig5	A9EC
U+62D6.0	fGB	0-4D4F
U+62D6.0	fR/S	 64.5
U+62D6.0	fTotalStrokes	 8
U+62D6.0	fCangjie	QOPD
U+62D6.0	fFourCorner	58012
U+62D6.0	fHakka	  to1; to5
U+62D6.0	fCantonese	to1
U+62D6.0	fMandarin	tuo1; duo3; tuo4; yi3
U+62D6.0	fEnglish	[1] drag along, after or out; tow [2] delay; prolong; procrastinate; drag out [3] involve; implicate
U+62D7.0	fUTF8	拗
U+62D7.0	fCNS11643	1-4D6F
U+62D7.0	fBig5	A9ED
U+62D7.0	fGB	0-5E56
U+62D7.0	fR/S	 64.5
U+62D7.0	fTotalStrokes	 8
U+62D7.0	fCangjie	QVIS
U+62D7.0	fFourCorner	54027
U+62D7.0	fHakka	  au3;  au5; eu2
U+62D7.0	fCantonese	aau2; aau3; ngaau2 (also); ngaau3 (also); juk6
U+62D7.0	fMandarin	ao3
U+62D7.0	fEnglish	 bend or twist so as to break
U+62D7.1	fUTF8	拗
U+62D7.1	fCNS11643	1-4D6F
U+62D7.1	fBig5	A9ED
U+62D7.1	fGB	0-5E56
U+62D7.1	fR/S	 64.5
U+62D7.1	fTotalStrokes	 8
U+62D7.1	fCangjie	QVIS
U+62D7.1	fFourCorner	54027
U+62D7.1	fMandarin	ao4; yao4; niu4
U+62D7.1	fEnglish	[1] obstinate; stubborn; unmanageable; recalcitrant [2] hard to pronounce; awkward-sounding
U+62D7.2	fUTF8	拗
U+62D7.2	fCNS11643	1-4D6F
U+62D7.2	fBig5	A9ED
U+62D7.2	fGB	0-5E56
U+62D7.2	fR/S	 64.5
U+62D7.2	fTotalStrokes	 8
U+62D7.2	fCangjie	QVIS
U+62D7.2	fFourCorner	54027
U+62D7.2	fMandarin	yu4
U+62D7.2	fEnglish	 restrain; repress; curb; suppress
U+62D8.0	fUTF8	拘
U+62D8.0	fCNS11643	1-4D6D
U+62D8.0	fBig5	A9EB
U+62D8.0	fGB	0-3E50
U+62D8.0	fR/S	 64.5
U+62D8.0	fTotalStrokes	 8
U+62D8.0	fCangjie	QPR
U+62D8.0	fFourCorner	57020
U+62D8.0	fHakka	  gi1;  ki1
U+62D8.0	fCantonese	keoi1
U+62D8.0	fMandarin	ju1; gou1; ju3; ju4
U+62D8.0	fEnglish	[1] apprehend; arrest; restrain; detain [2] inflexible; adhere rigidly to (conventions, etc.) [3] confined; restricted; not free; restrained
U+62D9.0	fUTF8	拙
U+62D9.0	fCNS11643	1-4D67
U+62D9.0	fBig5	A9E5
U+62D9.0	fGB	0-573E
U+62D9.0	fR/S	 64.5
U+62D9.0	fTotalStrokes	 8
U+62D9.0	fCangjie	QUU
U+62D9.0	fFourCorner	52072
U+62D9.0	fHakka	 zot7; chot7; cut7; zhot7
U+62D9.0	fCantonese	zyut3; zyut6 (also)
U+62D9.0	fMandarin	zhuo1; zhuo2
U+62D9.0	fEnglish	[1] stupid; crude; poor (works, etc.) [2] a conventional term referring to oneself
U+62DA.0	fUTF8	拚
U+62DA.0	fCNS11643	1-4D6B
U+62DA.0	fBig5	A9E9
U+62DA.0	fGB	0-5E55
U+62DA.0	fR/S	 64.5
U+62DA.0	fTotalStrokes	 8
U+62DA.0	fCangjie	QIT
U+62DA.0	fFourCorner	53040
U+62DA.0	fHakka	  pien5; biang5; fan5; ban3; pin5; ban5; fin5; min5
U+62DA.0	fCantonese	pun3; ping1; pun2 (also); baai3; bin6; ziu1; baai1 (also); fan3; faan1; pun1; dai2; ji5; ji4 (also)
U+62DA.0	fMandarin	pan4; fan1; fen4; pin1; bian4; fen1
U+62DA.0	fEnglish	[1] at the risk of; disregard [2] go all out; try very hard to [3] reject; abandon; discard
U+62DB.0	fUTF8	招
U+62DB.0	fCNS11643	1-4D5D
U+62DB.0	fBig5	A9DB
U+62DB.0	fGB	0-5550
U+62DB.0	fR/S	 64.5
U+62DB.0	fTotalStrokes	 8
U+62DB.0	fCangjie	QSHR
U+62DB.0	fFourCorner	57062
U+62DB.0	fHakka	  zhau1;  zau1; zeu1; cheu1
U+62DB.0	fCantonese	ziu1
U+62DB.0	fMandarin	zhao1; qiao2; shao2
U+62DB.0	fEnglish	[1] beckon with one's hand; summon [2] raise (an army; capital, etc.); recruit [3] confess; admit [4] poster; notice; signboard [5] cause; effect; incite; incur; invite [6] entice; induce [7] welcome; receive [8] infect; be infectious [9] target; bull's eye [10] move; trick; device [11] a Chinese family name
U+62DC.0	fUTF8	拜
U+62DC.0	fCNS11643	1-5138
U+62DC.0	fBig5	ABF4
U+62DC.0	fGB	0-305D
U+62DC.0	fR/S	 64.5
U+62DC.0	fTotalStrokes	 9
U+62DC.0	fCangjie	HQMQJ
U+62DC.0	fFourCorner	21550
U+62DC.0	fHakka	  bai5
U+62DC.0	fCantonese	baai3; baai1
U+62DC.0	fMandarin	bai4
U+62DC.0	fEnglish	[1] do obeisance; salute; pay respects to [2] appoint (as a government official); appointment [3] visit; pay a visit; call on or at [4] a Chinese family name
U+62DD.0	fUTF8	拝
U+62DD.0	fCNS11643	3-2930
U+62DD.0	fGB	E-2B2E
U+62DD.0	fR/S	 64.5
U+62DD.0	fTotalStrokes	 8
U+62DD.0	fCangjie	QMQJ
U+62DD.0	fFourCorner	52050
U+62DD.0	fHakka	 bai5
U+62DD.0	fCantonese	baai3
U+62DD.0	fMandarin	bai4
U+62DD.0	fEnglish	simplified form of &#x62DC;
U+62DE.0	fUTF8	拞
U+62DE.0	fCNS11643	3-2937
U+62DE.0	fGB	E-2B2F
U+62DE.0	fR/S	 64.5
U+62DE.0	fTotalStrokes	 8
U+62DE.0	fCangjie	QOM
U+62DE.0	fFourCorner	52010
U+62DE.0	fHakka	  di3; du2; dai3; dai1; din3
U+62DE.0	fCantonese	dai2; zi2
U+62DE.0	fMandarin	di3; di2; qu2; zhi3
U+62DE.0	fEnglish	same as &#x62B5;
U+62DF.0	fUTF8	拟
U+62DF.0	fCNS11643	3-2935
U+62DF.0	fGB	0-4462
U+62DF.0	fR/S	 64.5
U+62DF.0	fTotalStrokes	 8
U+62DF.0	fCangjie	QVIO
U+62DF.0	fFourCorner	58000
U+62DF.0	fHakka	 ngi3
U+62DF.0	fCantonese	ji5
U+62DF.0	fMandarin	ni3; yi2
U+62DF.0	fEnglish	[1] non-classical form of &#x64EC; [2] simplified form of &#x64EC;
U+62E0.0	fUTF8	拠
U+62E0.0	fCNS11643	3-2936
U+62E0.0	fGB	E-2B30
U+62E0.0	fR/S	 64.5
U+62E0.0	fTotalStrokes	 8
U+62E0.0	fCangjie	QHEN
U+62E0.0	fFourCorner	57040
U+62E0.0	fHakka	 gi3
U+62E0.0	fCantonese	geoi3
U+62E0.0	fMandarin	ju4; ju1; ji3
U+62E0.0	fEnglish	to occupy, take possession of; a base, position
U+62E1.0	fUTF8	拡
U+62E1.0	fCNS11643	3-292F
U+62E1.0	fGB	E-2B31
U+62E1.0	fR/S	 64.5
U+62E1.0	fTotalStrokes	 8
U+62E1.0	fCangjie	QII
U+62E1.0	fFourCorner	50030
U+62E1.0	fHakka	 kwok7
U+62E1.0	fCantonese	kwok3; kok3 (also); kwong3
U+62E1.0	fMandarin	kuo4
U+62E1.0	fEnglish	simplified form of &#x64F4;
U+62E2.0	fUTF8	拢
U+62E2.0	fGB	0-4223
U+62E2.0	fR/S	 64.5
U+62E2.0	fTotalStrokes	 8
U+62E2.0	fCangjie	QIKP
U+62E2.0	fHakka	 lung3
U+62E2.0	fCantonese	lung5; lung2 (also)
U+62E2.0	fMandarin	long3
U+62E2.0	fEnglish	simplified form of &#x650F;
U+62E3.0	fUTF8	拣
U+62E3.0	fGB	0-3C70
U+62E3.0	fR/S	 64.5
U+62E3.0	fTotalStrokes	 8
U+62E3.0	fCangjie	QKVC
U+62E3.0	fHakka	 gan3
U+62E3.0	fCantonese	gaan2
U+62E3.0	fMandarin	jian3; lian4
U+62E3.0	fEnglish	simplified form of &#x63C0;
U+62E4.0	fUTF8	拤
U+62E4.0	fCNS11643	3-2929
U+62E4.0	fGB	3-4978
U+62E4.0	fR/S	  64.5
U+62E4.0	fTotalStrokes	 8
U+62E4.0	fCangjie	QYMY
U+62E4.0	fFourCorner	51030
U+62E4.0	fCantonese	kaa1
U+62E4.0	fMandarin	qia2
U+62E5.0	fUTF8	拥
U+62E5.0	fGB	0-5335
U+62E5.0	fR/S	 64.5
U+62E5.0	fTotalStrokes	 8
U+62E5.0	fCangjie	QBQ
U+62E5.0	fHakka	 jung3
U+62E5.0	fCantonese	jung2
U+62E5.0	fMandarin	yong3; yong1
U+62E5.0	fEnglish	simplified form of &#x64C1;
U+62E6.0	fUTF8	拦
U+62E6.0	fGB	0-4039
U+62E6.0	fR/S	 64.5
U+62E6.0	fTotalStrokes	 8
U+62E6.0	fCangjie	QTMM
U+62E6.0	fHakka	 lan2
U+62E6.0	fCantonese	laan4
U+62E6.0	fMandarin	lan2
U+62E6.0	fEnglish	simplified form of &#x6514;
U+62E7.0	fUTF8	拧
U+62E7.0	fGB	0-4521
U+62E7.0	fR/S	 64.5
U+62E7.0	fTotalStrokes	 8
U+62E7.0	fCangjie	QJMN
U+62E7.0	fHakka	 nen2
U+62E7.0	fCantonese	ning6; ling6 (also); ning4; ning2 (also); ling2 (also); ling4 (also)
U+62E7.0	fMandarin	ning2; ning3; ning4
U+62E7.0	fEnglish	simplified form of &#x64F0;
U+62E8.0	fUTF8	拨
U+62E8.0	fGB	0-3226
U+62E8.0	fR/S	 64.5
U+62E8.0	fTotalStrokes	 8
U+62E8.0	fCangjie	QIVE
U+62E8.0	fHakka	 pat7
U+62E8.0	fCantonese	but6; but2 (also)
U+62E8.0	fMandarin	bo1; fa2; fa1
U+62E8.0	fEnglish	simplified form of &#x64A5;
U+62E9.0	fUTF8	择
U+62E9.0	fGB	0-5471
U+62E9.0	fR/S	 64.5
U+62E9.0	fTotalStrokes	 8
U+62E9.0	fCangjie	QEQ
U+62E9.0	fHakka	 cak8; cet8; tok8
U+62E9.0	fCantonese	zaak6
U+62E9.0	fMandarin	ze2; zhai2; yi4
U+62E9.0	fEnglish	simplified form of &#x64C7;
U+62EA.0	fUTF8	拪
U+62EA.0	fCNS11643	3-2C6D
U+62EA.0	fGB	5-4B78
U+62EA.0	fR/S	 64.6
U+62EA.0	fTotalStrokes	 9
U+62EA.0	fCangjie	QMCW
U+62EA.0	fFourCorner	51060
U+62EA.0	fCantonese	cin1
U+62EA.0	fMandarin	qian1
U+62EB.0	fUTF8	拫
U+62EB.0	fCNS11643	2-2A5B
U+62EB.0	fBig5	CEDA
U+62EB.0	fGB	3-4A35
U+62EB.0	fR/S	 64.6
U+62EB.0	fTotalStrokes	 9
U+62EB.0	fCangjie	QAV
U+62EB.0	fFourCorner	57032
U+62EB.0	fHakka	  hen2; hen1; hien2; dui1
U+62EB.0	fCantonese	hang4; han4; han6
U+62EB.0	fMandarin	hen2; hen3; hen4
U+62EB.0	fEnglish	 pull drag stop
U+62EC.0	fUTF8	括
U+62EC.0	fCNS11643	1-5144
U+62EC.0	fBig5	AC41
U+62EC.0	fGB	0-4028
U+62EC.0	fR/S	 64.6
U+62EC.0	fTotalStrokes	 9
U+62EC.0	fCangjie	QHJR
U+62EC.0	fFourCorner	52064
U+62EC.0	fHakka	 gat7;  kwat7; gwat7
U+62EC.0	fCantonese	kut3; gwaat3; kwut3 (also)
U+62EC.0	fMandarin	kuo4; huo2; gua1
U+62EC.0	fEnglish	[1] include; embrace; enclose; sum up [2] seek; search for; ransack [3] come; arrive [4] bound; tie [5] restrain [6] hit; dismiss
U+62ED.0	fUTF8	拭
U+62ED.0	fCNS11643	1-513C
U+62ED.0	fBig5	ABF8
U+62ED.0	fGB	0-4A43
U+62ED.0	fR/S	 64.6
U+62ED.0	fTotalStrokes	 9
U+62ED.0	fCangjie	QIPM
U+62ED.0	fFourCorner	53040
U+62ED.0	fHakka	 sit7;  chit7; cit7; shit7; siit7; ts'iit7; rsut8; cut8
U+62ED.0	fCantonese	sik1
U+62ED.0	fMandarin	shi4
U+62ED.0	fEnglish	 wipe; rub (eyes, etc.)
U+62EE.0	fUTF8	拮
U+62EE.0	fCNS11643	1-513E
U+62EE.0	fBig5	ABFA
U+62EE.0	fGB	0-5E57
U+62EE.0	fR/S	 64.6
U+62EE.0	fTotalStrokes	 9
U+62EE.0	fCangjie	QGR
U+62EE.0	fFourCorner	54061
U+62EE.0	fHakka	  git7; giet7; kit8; get7
U+62EE.0	fCantonese	git3; gaat3; gat1
U+62EE.0	fMandarin	jie2; jia2; ji2; qie4
U+62EE.0	fEnglish	laboring hard; occupied
U+62EF.0	fUTF8	拯
U+62EF.0	fCNS11643	1-5143
U+62EF.0	fBig5	AC40
U+62EF.0	fGB	0-557C
U+62EF.0	fR/S	 64.6
U+62EF.0	fTotalStrokes	 9
U+62EF.0	fCangjie	QNEM
U+62EF.0	fFourCorner	57013
U+62EF.0	fHakka	 zhin3; zihn3; zin3; zin2; zhin1
U+62EF.0	fCantonese	cing2
U+62EF.0	fMandarin	zheng3
U+62EF.0	fEnglish	[1] deliver; save [2] lift up; raise
U+62F0.0	fUTF8	拰
U+62F0.0	fCNS11643	2-2A67
U+62F0.0	fBig5	CEE6
U+62F0.0	fGB	3-4A2E
U+62F0.0	fR/S	 64.6
U+62F0.0	fTotalStrokes	 9
U+62F0.0	fCangjie	QOHG
U+62F0.0	fFourCorner	52014
U+62F0.0	fCantonese	nam5; lam5 (also)
U+62F0.0	fMandarin	nin3
U+62F1.0	fUTF8	拱
U+62F1.0	fCNS11643	1-5141
U+62F1.0	fBig5	ABFD
U+62F1.0	fGB	0-3930
U+62F1.0	fR/S	 64.6
U+62F1.0	fTotalStrokes	 9
U+62F1.0	fCangjie	QTC
U+62F1.0	fFourCorner	54081
U+62F1.0	fHakka	  giung3; gung3; giung1
U+62F1.0	fCantonese	gung2
U+62F1.0	fMandarin	gong3; gong1; gong4; ju2
U+62F1.0	fEnglish	[1] fold hands before the breast when making a bow; salute [2] encircle with the hands [3] surround [4] arched (doors, windows, etc. in architecture) [5] raise up (in the middle); hump up; arch
U+62F2.0	fUTF8	拲
U+62F2.0	fCNS11643	2-2F33
U+62F2.0	fBig5	D1B1
U+62F2.0	fGB	5-4B42
U+62F2.0	fR/S	 64.6
U+62F2.0	fTotalStrokes	 10
U+62F2.0	fCangjie	TCQ
U+62F2.0	fFourCorner	44503
U+62F2.0	fHakka	  giung3
U+62F2.0	fCantonese	gung2
U+62F2.0	fMandarin	gong3; ju2
U+62F2.0	fEnglish	 put wooden handcuffs on a prisoner
U+62F3.0	fUTF8	拳
U+62F3.0	fCNS11643	1-5553
U+62F3.0	fBig5	AEB1
U+62F3.0	fGB	0-482D
U+62F3.0	fR/S	 64.6
U+62F3.0	fTotalStrokes	 10
U+62F3.0	fCangjie	FQQ
U+62F3.0	fFourCorner	90502
U+62F3.0	fHakka	  kien2;  ken2; k'ian3; kian2
U+62F3.0	fCantonese	kyun4; kyun2 (also)
U+62F3.0	fMandarin	quan2; juan1; quan1; quan3
U+62F3.0	fEnglish	[1] fist [2] various forms of boxing; sparring reats [3] strength
U+62F4.0	fUTF8	拴
U+62F4.0	fCNS11643	1-5146
U+62F4.0	fBig5	AC43
U+62F4.0	fGB	0-4B29
U+62F4.0	fR/S	 64.6
U+62F4.0	fTotalStrokes	 9
U+62F4.0	fCangjie	QOMG
U+62F4.0	fFourCorner	58014
U+62F4.0	fHakka	  cien2; cion2; cen2; con1; zien1
U+62F4.0	fCantonese	saan1; cyun4 (also)
U+62F4.0	fMandarin	shuan1; quan1
U+62F4.0	fEnglish	[1] tie up; fasten [2] drive a wedge between two parties
U+62F5.0	fUTF8	拵
U+62F5.0	fCNS11643	2-2A58
U+62F5.0	fBig5	CED7
U+62F5.0	fGB	3-4A29
U+62F5.0	fR/S	 64.6
U+62F5.0	fTotalStrokes	 9
U+62F5.0	fCangjie	QKLD
U+62F5.0	fFourCorner	54047
U+62F5.0	fCantonese	cyun4
U+62F5.0	fMandarin	cun2; tun2; cun4
U+62F5.0	fEnglish	to make, prepare, manufacture; workmanship; an arrangement
U+62F6.0	fUTF8	拶
U+62F6.0	fCNS11643	2-2A60
U+62F6.0	fBig5	CEDF
U+62F6.0	fGB	0-5E59
U+62F6.0	fR/S	 64.6
U+62F6.0	fTotalStrokes	 9
U+62F6.0	fCangjie	QVVN
U+62F6.0	fFourCorner	52027
U+62F6.0	fHakka	 zat7;  zat8
U+62F6.0	fCantonese	zaat3; zaan2
U+62F6.0	fMandarin	zan3; za1
U+62F6.0	fEnglish	a torture device in old China consisting of several contractible wooden sticks, in between which the fingers of a suspect were placed and pressed to extort confessions
U+62F7.0	fUTF8	拷
U+62F7.0	fCNS11643	1-5142
U+62F7.0	fBig5	ABFE
U+62F7.0	fGB	0-3F3D
U+62F7.0	fR/S	 64.6
U+62F7.0	fTotalStrokes	 9
U+62F7.0	fCangjie	QJKS
U+62F7.0	fFourCorner	54027
U+62F7.0	fHakka	 kau5;  kau3
U+62F7.0	fCantonese	haau2; haau1 (also)
U+62F7.0	fMandarin	kao3
U+62F7.0	fEnglish	 flog; whip; torture (in order to get a confession)
U+62F8.0	fUTF8	拸
U+62F8.0	fCNS11643	2-2A5F
U+62F8.0	fBig5	CEDE
U+62F8.0	fGB	3-4A31
U+62F8.0	fR/S	 64.6
U+62F8.0	fTotalStrokes	 9
U+62F8.0	fCangjie	QNIN
U+62F8.0	fFourCorner	57027
U+62F8.0	fCantonese	ji4; ci2
U+62F8.0	fMandarin	chi3; yi2; tuo1; tuo3
U+62F9.0	fUTF8	拹
U+62F9.0	fCNS11643	2-2A5C
U+62F9.0	fBig5	CEDB
U+62F9.0	fGB	3-4A36
U+62F9.0	fR/S	 64.6
U+62F9.0	fTotalStrokes	 9
U+62F9.0	fCangjie	QKSS
U+62F9.0	fFourCorner	54027
U+62F9.0	fHakka	 hiap8
U+62F9.0	fCantonese	hip3; hip6
U+62F9.0	fMandarin	xie2; la4
U+62F9.0	fEnglish	[1] fold [2] double up [3] drag; pull
U+62FA.0	fUTF8	拺
U+62FA.0	fCNS11643	2-2A64
U+62FA.0	fBig5	CEE3
U+62FA.0	fGB	3-4A27
U+62FA.0	fR/S	 64.6
U+62FA.0	fTotalStrokes	 9
U+62FA.0	fCangjie	QDB
U+62FA.0	fFourCorner	54092
U+62FA.0	fHakka	 sit7;  cet7
U+62FA.0	fCantonese	caak3; saak3; cuk1; sik1
U+62FA.0	fMandarin	ce4
U+62FA.0	fEnglish	[1] support; assist [2] riding whip
U+62FB.0	fUTF8	拻
U+62FB.0	fCNS11643	2-2A66
U+62FB.0	fBig5	CEE5
U+62FB.0	fGB	E-2B32
U+62FB.0	fR/S	 64.6
U+62FB.0	fTotalStrokes	 9
U+62FB.0	fCangjie	QKF
U+62FB.0	fFourCorner	54089
U+62FB.0	fCantonese	fui1
U+62FB.0	fMandarin	hui1
U+62FC.0	fUTF8	拼
U+62FC.0	fCNS11643	1-513B
U+62FC.0	fBig5	ABF7
U+62FC.0	fGB	0-4634
U+62FC.0	fR/S	 64.6
U+62FC.0	fTotalStrokes	 9
U+62FC.0	fCangjie	QTT
U+62FC.0	fFourCorner	58041
U+62FC.0	fHakka	 pin5; pin2;  bin5; pin1; ben1; biang5; bin3
U+62FC.0	fCantonese	ping1; ping3 (also); pun3; pun2 (also)
U+62FC.0	fMandarin	pin1
U+62FC.0	fEnglish	[1] join together; incorporate; put together; make a whole [2] spell (a word) [3] risk
U+62FD.0	fUTF8	拽
U+62FD.0	fCNS11643	1-513F
U+62FD.0	fBig5	ABFB
U+62FD.0	fGB	0-5727
U+62FD.0	fR/S	 64.6
U+62FD.0	fTotalStrokes	 9
U+62FD.0	fCangjie	QLWP
U+62FD.0	fFourCorner	55006
U+62FD.0	fHakka	 ji3; ri3
U+62FD.0	fCantonese	jai6; jit6; zai6
U+62FD.0	fMandarin	zhuai1; shi4
U+62FD.0	fEnglish	 pull; drag; catch
U+62FD.1	fUTF8	拽
U+62FD.1	fCNS11643	1-513F
U+62FD.1	fBig5	ABFB
U+62FD.1	fGB	0-5727
U+62FD.1	fR/S	 64.6
U+62FD.1	fTotalStrokes	 9
U+62FD.1	fCangjie	QLWP
U+62FD.1	fFourCorner	55006
U+62FD.1	fMandarin	zhuai4
U+62FD.1	fEnglish	 fling; hurl
U+62FD.2	fUTF8	拽
U+62FD.2	fCNS11643	1-513F
U+62FD.2	fBig5	ABFB
U+62FD.2	fGB	0-5727
U+62FD.2	fR/S	 64.6
U+62FD.2	fTotalStrokes	 9
U+62FD.2	fCangjie	QLWP
U+62FD.2	fFourCorner	55006
U+62FD.2	fMandarin	ye4; yi4
U+62FD.2	fEnglish	[1] trail; drag after [2] pull; drag
U+62FE.0	fUTF8	拾
U+62FE.0	fCNS11643	1-5145
U+62FE.0	fBig5	AC42
U+62FE.0	fGB	0-4A30
U+62FE.0	fR/S	 64.6
U+62FE.0	fTotalStrokes	 9
U+62FE.0	fCangjie	QOMR
U+62FE.0	fFourCorner	58061
U+62FE.0	fHakka	  ship8;  sip8; siip8
U+62FE.0	fCantonese	sap6; sip3; gip6
U+62FE.0	fMandarin	shi2; jie4
U+62FE.0	fEnglish	[1] pick up; collect [2] put away [3] formal form of the figure ten used in accountancy [4] an armlet used by archers
U+62FE.1	fUTF8	拾
U+62FE.1	fCNS11643	1-5145
U+62FE.1	fBig5	AC42
U+62FE.1	fGB	0-4A30
U+62FE.1	fR/S	 64.6
U+62FE.1	fTotalStrokes	 9
U+62FE.1	fCangjie	QOMR
U+62FE.1	fFourCorner	58061
U+62FE.1	fMandarin	she4
U+62FE.1	fEnglish	 go up; ascend
U+62FE.2	fUTF8	拾
U+62FE.2	fCNS11643	1-5145
U+62FE.2	fBig5	AC42
U+62FE.2	fGB	0-4A30
U+62FE.2	fR/S	 64.6
U+62FE.2	fTotalStrokes	 9
U+62FE.2	fCangjie	QOMR
U+62FE.2	fFourCorner	58061
U+62FE.2	fMandarin	shi1
U+62FE.2	fEnglish	used in combination
U+62FE.3	fUTF8	拾
U+62FE.3	fCNS11643	1-5145
U+62FE.3	fBig5	AC42
U+62FE.3	fGB	0-4A30
U+62FE.3	fR/S	 64.6
U+62FE.3	fTotalStrokes	 9
U+62FE.3	fCangjie	QOMR
U+62FE.3	fFourCorner	58061
U+62FE.3	fMandarin	shi5
U+62FF.0	fUTF8	拿
U+62FF.0	fCNS11643	1-5555
U+62FF.0	fBig5	AEB3
U+62FF.0	fGB	0-4443
U+62FF.0	fR/S	 64.6
U+62FF.0	fTotalStrokes	 10
U+62FF.0	fCangjie	ORQ
U+62FF.0	fFourCorner	80502
U+62FF.0	fHakka	 na1;  la1
U+62FF.0	fCantonese	naa4; laa4 (also); naa2 (also); laa2 (also)
U+62FF.0	fMandarin	na2
U+62FF.0	fEnglish	[1] take; hold in one's hand; grasp [2] with [3] arrest; apprehend; capture [4] use; employ (a method, device, etc) [5] to be confirmed or restrained
U+6300.0	fUTF8	挀
U+6300.0	fCNS11643	2-2A61
U+6300.0	fBig5	CEE0
U+6300.0	fGB	E-2B33
U+6300.0	fR/S	 64.6
U+6300.0	fTotalStrokes	 9
U+6300.0	fCangjie	QHHV
U+6300.0	fFourCorner	52032
U+6300.0	fCantonese	baak3
U+6300.0	fMandarin	bo4
U+6301.0	fUTF8	持
U+6301.0	fCNS11643	1-513D
U+6301.0	fBig5	ABF9
U+6301.0	fGB	0-3356
U+6301.0	fR/S	 64.6
U+6301.0	fTotalStrokes	 9
U+6301.0	fCangjie	QGDI
U+6301.0	fFourCorner	54041
U+6301.0	fHakka	 ci2;  chi2; cih2; chi3; chii2
U+6301.0	fCantonese	ci4; ci2 (also)
U+6301.0	fMandarin	chi2
U+6301.0	fEnglish	[1] maintain; sustain; support; keep; uphold [2] hold; grasp
U+6302.0	fUTF8	挂
U+6302.0	fCNS11643	1-5148
U+6302.0	fBig5	AC45
U+6302.0	fGB	0-3952
U+6302.0	fR/S	 64.6
U+6302.0	fTotalStrokes	 9
U+6302.0	fCangjie	QGG
U+6302.0	fFourCorner	54014
U+6302.0	fHakka	  gwa5; ga5
U+6302.0	fCantonese	kwaa3 (also); gwaa3
U+6302.0	fMandarin	gua4; gui1
U+6302.0	fEnglish	 hang; suspend suspense
U+6303.0	fUTF8	挃
U+6303.0	fCNS11643	2-2A5A
U+6303.0	fBig5	CED9
U+6303.0	fGB	3-4A2A
U+6303.0	fR/S	 64.6
U+6303.0	fTotalStrokes	 9
U+6303.0	fCangjie	QMIG
U+6303.0	fFourCorner	51014
U+6303.0	fHakka	 zhit7; tsiit7
U+6303.0	fCantonese	zat6; zat1
U+6303.0	fMandarin	zhi4; die2
U+6303.0	fEnglish	beat
U+6304.0	fUTF8	挄
U+6304.0	fCNS11643	3-2C6F
U+6304.0	fGB	5-4B7D
U+6304.0	fR/S	  64.6
U+6304.0	fTotalStrokes	 9
U+6304.0	fCangjie	QFMU
U+6304.0	fFourCorner	59010
U+6304.0	fHakka	 kok7; kong5; kwok7
U+6304.0	fCantonese	gwok3; kwong3; kwok3; kong3
U+6304.0	fMandarin	kuo4
U+6304.0	fEnglish	same as &#x64F4;
U+6305.0	fUTF8	挅
U+6305.0	fCNS11643	3-2C70
U+6305.0	fGB	E-2B34
U+6305.0	fR/S	  64.6
U+6305.0	fTotalStrokes	 9
U+6305.0	fCangjie	QNSD
U+6305.0	fFourCorner	57090
U+6305.0	fHakka	 do3
U+6305.0	fCantonese	do2
U+6305.0	fMandarin	duo3
U+6306.0	fUTF8	挆
U+6306.0	fCNS11643	5-2945
U+6306.0	fGB	5-4C24
U+6306.0	fR/S	  64.6
U+6306.0	fTotalStrokes	 9
U+6306.0	fCangjie	QHND
U+6306.0	fFourCorner	57090
U+6306.0	fHakka	 do3
U+6306.0	fCantonese	do2
U+6306.0	fMandarin	duo3
U+6306.0	fEnglish	same as &#x6305;
U+6307.0	fUTF8	指
U+6307.0	fCNS11643	1-5140
U+6307.0	fBig5	ABFC
U+6307.0	fGB	0-5638
U+6307.0	fR/S	 64.6
U+6307.0	fTotalStrokes	 9
U+6307.0	fCangjie	QPA
U+6307.0	fFourCorner	51061
U+6307.0	fHakka	 zi3;  zhi3; zih3; tje3
U+6307.0	fCantonese	zi2
U+6307.0	fMandarin	zhi3
U+6307.0	fEnglish	[1] finger [2] point; direct [3] indicate; refer to; mean [4] number of people [5] intentions [6] main theme [7] hope [8] depend on
U+6307.1	fUTF8	指
U+6307.1	fCNS11643	1-5140
U+6307.1	fBig5	ABFC
U+6307.1	fGB	0-5638
U+6307.1	fR/S	 64.6
U+6307.1	fTotalStrokes	 9
U+6307.1	fCangjie	QPA
U+6307.1	fFourCorner	51061
U+6307.1	fMandarin	zhi2
U+6307.1	fEnglish	used in combination with &#x982D;
U+6307.2	fUTF8	指
U+6307.2	fCNS11643	1-5140
U+6307.2	fBig5	ABFC
U+6307.2	fGB	0-5638
U+6307.2	fR/S	 64.6
U+6307.2	fTotalStrokes	 9
U+6307.2	fCangjie	QPA
U+6307.2	fFourCorner	51061
U+6307.2	fMandarin	zhi1
U+6307.2	fEnglish	used in combination with &#x7532;
U+6308.0	fUTF8	挈
U+6308.0	fCNS11643	1-5554
U+6308.0	fBig5	AEB2
U+6308.0	fGB	0-6A7C
U+6308.0	fR/S	 64.6
U+6308.0	fTotalStrokes	 10
U+6308.0	fCangjie	QHQ
U+6308.0	fFourCorner	57502
U+6308.0	fHakka	 giet7; k'ie5; get7; ke5; k'e5
U+6308.0	fCantonese	kit3; hit3
U+6308.0	fMandarin	qie4; qi4; chi4
U+6308.0	fEnglish	[1] lead [2] rise above; raise
U+6309.0	fUTF8	按
U+6309.0	fCNS11643	1-513A
U+6309.0	fBig5	ABF6
U+6309.0	fGB	0-3034
U+6309.0	fR/S	 64.6
U+6309.0	fTotalStrokes	 9
U+6309.0	fCangjie	QJV
U+6309.0	fFourCorner	53044
U+6309.0	fHakka	  on5
U+6309.0	fCantonese	on3; ngon3 (also)
U+6309.0	fMandarin	an4; e4
U+6309.0	fEnglish	[1] put the hand on; place the hand on; press, control, etc. with one's hand [2] examine [3] stop; halt; repress; leave aside [4] impeach; censure [5] according to; in (good order); as [6] follow (a map, river, etc.) [7] note; comment
U+630A.0	fUTF8	挊
U+630A.0	fCNS11643	4-2959
U+630A.0	fGB	E-2B35
U+630A.0	fR/S	 64.6
U+630A.0	fTotalStrokes	 9
U+630A.0	fCangjie	QYMY
U+630A.0	fFourCorner	51030
U+630A.0	fHakka	 lung5; lung3
U+630A.0	fCantonese	lung6
U+630A.0	fMandarin	nong4; long4; neng4
U+630B.0	fUTF8	挋
U+630B.0	fCNS11643	2-2A57
U+630B.0	fBig5	CED6
U+630B.0	fGB	3-4A26
U+630B.0	fR/S	 64.6; 64.7
U+630B.0	fTotalStrokes	 9
U+630B.0	fCangjie	QSLL
U+630B.0	fFourCorner	51017
U+630B.0	fHakka	 zhin3; zihn3
U+630B.0	fCantonese	zan3; zin3
U+630B.0	fMandarin	zhen4; jin4
U+630B.0	fEnglish	[1] shake [2] brush clean
U+630C.0	fUTF8	挌
U+630C.0	fCNS11643	2-2A5E
U+630C.0	fBig5	CEDD
U+630C.0	fGB	5-4C27
U+630C.0	fR/S	 64.6
U+630C.0	fTotalStrokes	 9
U+630C.0	fCangjie	QHER
U+630C.0	fFourCorner	57064
U+630C.0	fHakka	  gak7; get7; gok7
U+630C.0	fCantonese	gaak3; gak3
U+630C.0	fMandarin	ge2; he4; luo4
U+630C.0	fEnglish	fight
U+630D.0	fUTF8	挍
U+630D.0	fCNS11643	2-2A56
U+630D.0	fBig5	CED5
U+630D.0	fGB	5-4C29
U+630D.0	fR/S	 64.6
U+630D.0	fTotalStrokes	 9
U+630D.0	fCangjie	QYCK
U+630D.0	fFourCorner	50048
U+630D.0	fHakka	 gau5; gau3
U+630D.0	fCantonese	haau6; gaau3; gaau2; ngaau1
U+630D.0	fMandarin	jiao4; jiao1; jiao3
U+630D.0	fEnglish	collate
U+630E.0	fUTF8	挎
U+630E.0	fCNS11643	2-2A59
U+630E.0	fBig5	CED8
U+630E.0	fGB	0-3F66
U+630E.0	fR/S	 64.6
U+630E.0	fTotalStrokes	 9
U+630E.0	fCangjie	QKMS
U+630E.0	fFourCorner	54027
U+630E.0	fHakka	 ku1
U+630E.0	fCantonese	kwaa3; fu1
U+630E.0	fMandarin	ku1; kua4; kou1
U+630E.0	fEnglish	carry
U+630F.0	fUTF8	挏
U+630F.0	fCNS11643	2-2A5D
U+630F.0	fBig5	CEDC
U+630F.0	fGB	3-4A2C
U+630F.0	fR/S	 64.6
U+630F.0	fTotalStrokes	 9
U+630F.0	fCangjie	QBMR
U+630F.0	fFourCorner	57020
U+630F.0	fHakka	  tung2; dung2; dong2; tung3; tung5
U+630F.0	fCantonese	dung6; tung4 (also); tung2
U+630F.0	fMandarin	dong4; tong2
U+630F.0	fEnglish	[1] draw off; skim [2] shake
U+6310.0	fUTF8	挐
U+6310.0	fCNS11643	2-2F34
U+6310.0	fBig5	D1B2
U+6310.0	fGB	E-2B36
U+6310.0	fR/S	 64.6
U+6310.0	fTotalStrokes	 10
U+6310.0	fCangjie	VRQ
U+6310.0	fFourCorner	46502
U+6310.0	fHakka	  na1
U+6310.0	fCantonese	jyu4; naau4; naa4; laau4 (also); laa4 (also)
U+6310.0	fMandarin	ru2; na2; nu2; rao2
U+6310.0	fEnglish	drag
U+6311.0	fUTF8	挑
U+6311.0	fCNS11643	1-5147
U+6311.0	fBig5	AC44
U+6311.0	fGB	0-4C74
U+6311.0	fR/S	 64.6
U+6311.0	fTotalStrokes	 9
U+6311.0	fCangjie	QLMO
U+6311.0	fFourCorner	52013
U+6311.0	fHakka	  tiau1
U+6311.0	fCantonese	tiu1; tou1; tiu5
U+6311.0	fMandarin	tiao1; diao4; tiao2
U+6311.0	fEnglish	[1] stir; provoke; arouse [2] dally; make a pass at; seduce
U+6311.1	fUTF8	挑
U+6311.1	fCNS11643	1-5147
U+6311.1	fBig5	AC44
U+6311.1	fGB	0-4C74
U+6311.1	fR/S	 64.6
U+6311.1	fTotalStrokes	 9
U+6311.1	fCangjie	QLMO
U+6311.1	fFourCorner	52013
U+6311.1	fMandarin	tiao3
U+6311.1	fEnglish	[1] carry things with a pole one's shoulder; shoulder [2] select; choose; pick [3] pick by pitchfork
U+6311.2	fUTF8	挑
U+6311.2	fCNS11643	1-5147
U+6311.2	fBig5	AC44
U+6311.2	fGB	0-4C74
U+6311.2	fR/S	 64.6
U+6311.2	fTotalStrokes	 9
U+6311.2	fCangjie	QLMO
U+6311.2	fFourCorner	52013
U+6311.2	fMandarin	tao1
U+6311.2	fEnglish	light and frivolous
U+6312.0	fUTF8	挒
U+6312.0	fCNS11643	4-2951
U+6312.0	fGB	5-4B7A
U+6312.0	fR/S	  64.6
U+6312.0	fTotalStrokes	 9
U+6312.0	fCangjie	QMNN
U+6312.0	fFourCorner	52000
U+6312.0	fHakka	 liet8
U+6312.0	fCantonese	lit6
U+6312.0	fMandarin	lie4
U+6313.0	fUTF8	挓
U+6313.0	fCNS11643	2-2A62
U+6313.0	fBig5	CEE1
U+6313.0	fGB	5-4C2B
U+6313.0	fR/S	 64.6
U+6313.0	fTotalStrokes	 9
U+6313.0	fCangjie	QJHP
U+6313.0	fFourCorner	53014
U+6313.0	fHakka	  za1
U+6313.0	fCantonese	zaa1; zat3
U+6313.0	fMandarin	zha1
U+6313.0	fEnglish	used in combination
U+6314.0	fUTF8	挔
U+6314.0	fCNS11643	2-2A63
U+6314.0	fBig5	CEE2
U+6314.0	fGB	E-2B37
U+6314.0	fR/S	 64.6
U+6314.0	fTotalStrokes	 9
U+6314.0	fCangjie	QYHV
U+6314.0	fFourCorner	50032
U+6314.0	fHakka	  li3;  li1
U+6314.0	fCantonese	leoi5; leoi3
U+6314.0	fMandarin	l3; luu3; lu2; lu3
U+6314.0	fEnglish	same as &#x65C5;
U+6315.0	fUTF8	挕
U+6315.0	fCNS11643	2-2A65
U+6315.0	fBig5	CEE4
U+6315.0	fGB	5-4B77
U+6315.0	fR/S	 64.6
U+6315.0	fTotalStrokes	 9
U+6315.0	fCangjie	QSJ
U+6315.0	fFourCorner	51040
U+6315.0	fCantonese	dip3; dip6; zip3
U+6315.0	fMandarin	die2
U+6316.0	fUTF8	挖
U+6316.0	fCNS11643	1-5139
U+6316.0	fBig5	ABF5
U+6316.0	fGB	0-4D5A
U+6316.0	fR/S	 64.6
U+6316.0	fTotalStrokes	 9
U+6316.0	fCangjie	QJCN
U+6316.0	fFourCorner	53017
U+6316.0	fHakka	  wat7;  jet7; wet7; wak7; ret7; wa1; vet7; vat7
U+6316.0	fCantonese	waat3; waa1 (also); waat2 (also)
U+6316.0	fMandarin	wa1; wa2
U+6316.0	fEnglish	[1] dig out; scoop out [2] engrave with a knife; cut; gouge
U+6317.0	fUTF8	挗
U+6317.0	fCNS11643	3-2C6C
U+6317.0	fGB	E-2B38
U+6317.0	fR/S	  64.6
U+6317.0	fTotalStrokes	 9
U+6317.0	fCangjie	QKN
U+6317.0	fFourCorner	55030
U+6317.0	fHakka	  giet7; kiet8; get7
U+6317.0	fCantonese	kyut3
U+6317.0	fMandarin	jue2
U+6317.0	fEnglish	same as &#x6289;
U+6318.0	fUTF8	挘
U+6318.0	fGB	E-2B39
U+6318.0	fR/S	 64.6
U+6318.0	fTotalStrokes	 9
U+6318.0	fCangjie	QFHS
U+6318.0	fCantonese	lyut3
U+6318.0	fEnglish	[1] pluck; pick; tear [2] rubbing action
U+6319.0	fUTF8	挙
U+6319.0	fCNS11643	3-3132
U+6319.0	fGB	E-2B3A
U+6319.0	fR/S	 64.6
U+6319.0	fTotalStrokes	 10
U+6319.0	fCangjie	FCQ
U+6319.0	fFourCorner	23503
U+6319.0	fHakka	 gi3
U+6319.0	fCantonese	geoi2
U+6319.0	fMandarin	ju3
U+6319.0	fEnglish	simplified form of &#x8209;
U+631A.0	fUTF8	挚
U+631A.0	fGB	0-563F
U+631A.0	fR/S	 64.6
U+631A.0	fTotalStrokes	 10
U+631A.0	fCangjie	QIQ
U+631A.0	fHakka	 zhi5
U+631A.0	fCantonese	zi3
U+631A.0	fMandarin	zhi4; zhi2
U+631A.0	fEnglish	simplified form of &#x646F;
U+631B.0	fUTF8	挛
U+631B.0	fCNS11643	3-3133
U+631B.0	fGB	0-424E
U+631B.0	fR/S	 64.6
U+631B.0	fTotalStrokes	 10
U+631B.0	fCangjie	YCQ
U+631B.0	fFourCorner	00500
U+631B.0	fHakka	 lan2; lien5
U+631B.0	fCantonese	lyun4; lyun1 (also)
U+631B.0	fMandarin	luan2; lian4; luan4
U+631B.0	fEnglish	simplified form of &#x6523;
U+631C.0	fUTF8	挜
U+631C.0	fGB	8-7C4C
U+631C.0	fR/S	 64.6; 64.8
U+631C.0	fTotalStrokes	 9
U+631C.0	fCangjie	QMTC
U+631C.0	fCantonese	aa3; ngaa6 (also); aa6 (also); ngaa3 (also)
U+631C.0	fMandarin	ya4
U+631C.0	fEnglish	take; hold; give forcefully
U+631D.0	fUTF8	挝
U+631D.0	fGB	0-4E4E
U+631D.0	fR/S	 64.6
U+631D.0	fTotalStrokes	 9
U+631D.0	fCangjie	QYDI
U+631D.0	fHakka	 ko1
U+631D.0	fCantonese	zaa1; wo1; gwo1
U+631D.0	fMandarin	zhua1; wo1; ge1
U+631D.0	fEnglish	simplified form of &#x64BE;
U+631E.0	fUTF8	挞
U+631E.0	fGB	0-4C22
U+631E.0	fR/S	 64.6
U+631E.0	fTotalStrokes	 9
U+631E.0	fCangjie	XQYK
U+631E.0	fHakka	 dat7
U+631E.0	fCantonese	taat3; taat1
U+631E.0	fMandarin	ta4
U+631E.0	fEnglish	simplified form of &#x64BB;
U+631F.0	fUTF8	挟
U+631F.0	fCNS11643	3-2C6B
U+631F.0	fGB	0-502E
U+631F.0	fR/S	 64.6
U+631F.0	fTotalStrokes	 9
U+631F.0	fCangjie	QKT
U+631F.0	fFourCorner	55030
U+631F.0	fHakka	 hiap8; kiap8; giap7
U+631F.0	fCantonese	hip3; haap6 (also); hip6; gaap3; gaap6
U+631F.0	fMandarin	xie2; xia2; jia1; jia2; jie1; xie3; za1
U+631F.0	fEnglish	simplified form of &#x633E;
U+6320.0	fUTF8	挠
U+6320.0	fGB	0-4453
U+6320.0	fR/S	 64.6
U+6320.0	fTotalStrokes	 9
U+6320.0	fCangjie	QJPU
U+6320.0	fHakka	 nau2
U+6320.0	fCantonese	naau4; laau4 (also); naau2 (also); naau5 (also); laau2 (also); laau5 (also)
U+6320.0	fMandarin	nao2; hao1; nao4; rao4; xiao1
U+6320.0	fEnglish	simplified form of &#x6493;
U+6321.0	fUTF8	挡
U+6321.0	fCNS11643	3-2C6E
U+6321.0	fGB	0-3532
U+6321.0	fR/S	 64.6
U+6321.0	fTotalStrokes	 9
U+6321.0	fCangjie	QFSM
U+6321.0	fFourCorner	59070
U+6321.0	fHakka	 dong3; dong5
U+6321.0	fCantonese	dong2; dong3
U+6321.0	fMandarin	dang3; dang4; tang3
U+6321.0	fEnglish	simplified form of &#x64CB;
U+6322.0	fUTF8	挢
U+6322.0	fGB	0-5E58
U+6322.0	fR/S	 64.6
U+6322.0	fTotalStrokes	 9
U+6322.0	fCangjie	QHKL
U+6322.0	fHakka	 giau3
U+6322.0	fCantonese	giu2; giu6
U+6322.0	fMandarin	jiao3; jia3; jiao1; kao3
U+6322.0	fEnglish	simplified form of &#x649F;
U+6323.0	fUTF8	挣
U+6323.0	fCNS11643	4-295B
U+6323.0	fGB	0-5575
U+6323.0	fR/S	 64.6
U+6323.0	fTotalStrokes	 9
U+6323.0	fCangjie	QNSD
U+6323.0	fFourCorner	57050
U+6323.0	fHakka	 zang5; nang5
U+6323.0	fCantonese	zang1; zaang1 (also); zaang6; zang6
U+6323.0	fMandarin	zheng1
U+6323.0	fEnglish	simplified form of &#x6399;
U+6323.1	fUTF8	挣
U+6323.1	fCNS11643	4-295B
U+6323.1	fGB	0-5575
U+6323.1	fR/S	 64.6
U+6323.1	fTotalStrokes	 9
U+6323.1	fCangjie	QNSD
U+6323.1	fFourCorner	57050
U+6323.1	fMandarin	zheng4
U+6324.0	fUTF8	挤
U+6324.0	fGB	0-3C37
U+6324.0	fR/S	 64.6
U+6324.0	fTotalStrokes	 9
U+6324.0	fCangjie	QYKL
U+6324.0	fHakka	 zi1
U+6324.0	fCantonese	zai1
U+6324.0	fMandarin	ji3; ji1; ji4
U+6324.0	fEnglish	simplified form of &#x64E0;
U+6325.0	fUTF8	挥
U+6325.0	fGB	0-3B53
U+6325.0	fR/S	 64.6
U+6325.0	fTotalStrokes	 9
U+6325.0	fCangjie	QBKQ
U+6325.0	fHakka	 fui1
U+6325.0	fCantonese	fai1
U+6325.0	fMandarin	hui1; hun2; xun4
U+6325.0	fEnglish	simplified form of &#x63EE;
U+6326.0	fUTF8	挦
U+6326.0	fGB	8-7C4D
U+6326.0	fR/S	 64.6; 64.12
U+6326.0	fTotalStrokes	 9
U+6326.0	fCangjie	XQSMI
U+6326.0	fCantonese	cam4; cim4 (also)
U+6326.0	fMandarin	xun2; xian2
U+6326.0	fEnglish	pull out, pluck; take hold of
U+6327.0	fUTF8	挧
U+6327.0	fGB	E-2B3B
U+6327.0	fR/S	 64.6
U+6327.0	fTotalStrokes	 9
U+6327.0	fCangjie	QSMM
U+6327.0	fCantonese	jyu5
U+6328.0	fUTF8	挨
U+6328.0	fCNS11643	1-5563
U+6328.0	fBig5	AEC1
U+6328.0	fGB	0-3024
U+6328.0	fR/S	 64.7
U+6328.0	fTotalStrokes	 10
U+6328.0	fCangjie	QIOK
U+6328.0	fFourCorner	53034
U+6328.0	fHakka	  ai1; jai1; ngai1; ngai2; zai1
U+6328.0	fCantonese	aai1; ngaai1 (also); ngaai4; aai4 (also); oi2; ngoi2; ngai4; ai4
U+6328.0	fMandarin	ai1; ai2; ai3
U+6328.0	fEnglish	[1] (to stay) near; close to; side by side; next to; lean to; lean on; trust to [2] suffer [3] wait; delay; put off [4] according to order; in good order [5] rub; scratch [6] force; strike on the back
U+6329.0	fUTF8	挩
U+6329.0	fCNS11643	2-2F40
U+6329.0	fBig5	D1BE
U+6329.0	fGB	E-2B3C
U+6329.0	fR/S	 64.7
U+6329.0	fTotalStrokes	 10
U+6329.0	fCangjie	QCRU
U+6329.0	fFourCorner	58016
U+6329.0	fHakka	 tot7
U+6329.0	fCantonese	tyut3; seoi3
U+6329.0	fMandarin	tuo1; shui4; shui2; duo2
U+6329.0	fEnglish	 exclude; remove
U+632A.0	fUTF8	挪
U+632A.0	fCNS11643	1-5561
U+632A.0	fBig5	AEBF
U+632A.0	fGB	0-4532
U+632A.0	fR/S	 64.7
U+632A.0	fTotalStrokes	 10
U+632A.0	fCangjie	QSQL
U+632A.0	fFourCorner	57027
U+632A.0	fHakka	  no2; la2; na1; lo2
U+632A.0	fCantonese	no4; lo4 (also); no1
U+632A.0	fMandarin	nuo2; ruo2
U+632A.0	fEnglish	 move; shift; transfer
U+632B.0	fUTF8	挫
U+632B.0	fCNS11643	1-5562
U+632B.0	fBig5	AEC0
U+632B.0	fGB	0-346C
U+632B.0	fR/S	 64.7
U+632B.0	fTotalStrokes	 10
U+632B.0	fCangjie	QOOG
U+632B.0	fFourCorner	58014
U+632B.0	fHakka	 co5; co3
U+632B.0	fCantonese	co3
U+632B.0	fMandarin	cuo4; cuo1; zuo4
U+632B.0	fEnglish	[1] defeat; frustrate [2] damp [3] humiliate; treat harshly
U+632C.0	fUTF8	挬
U+632C.0	fCNS11643	2-2F36
U+632C.0	fBig5	D1B4
U+632C.0	fGB	5-4C30
U+632C.0	fR/S	 64.7
U+632C.0	fTotalStrokes	 10
U+632C.0	fCangjie	QJBD
U+632C.0	fFourCorner	54047
U+632C.0	fHakka	 put8
U+632C.0	fCantonese	but6
U+632C.0	fMandarin	bo2
U+632C.0	fEnglish	 take out; pluck up
U+632D.0	fUTF8	挭
U+632D.0	fCNS11643	2-2F46
U+632D.0	fBig5	D1C4
U+632D.0	fGB	5-4C31
U+632D.0	fR/S	 64.7
U+632D.0	fTotalStrokes	 10
U+632D.0	fCangjie	QMLK
U+632D.0	fFourCorner	51046
U+632D.0	fHakka	 gien3; gen3
U+632D.0	fCantonese	gaang3; gang2
U+632D.0	fMandarin	geng3
U+632D.0	fEnglish	 fish-bones; things that stick in the throat unyielding; blunt of speech
U+632E.0	fUTF8	挮
U+632E.0	fCNS11643	3-312C
U+632E.0	fGB	3-4A45
U+632E.0	fR/S	 64.7
U+632E.0	fTotalStrokes	 10
U+632E.0	fCangjie	QCNH
U+632E.0	fFourCorner	58020
U+632E.0	fCantonese	tai2
U+632E.0	fMandarin	ti3
U+632F.0	fUTF8	振
U+632F.0	fCNS11643	1-5558
U+632F.0	fBig5	AEB6
U+632F.0	fGB	0-5571
U+632F.0	fR/S	 64.7
U+632F.0	fTotalStrokes	 10
U+632F.0	fCangjie	QMMV
U+632F.0	fFourCorner	51032
U+632F.0	fHakka	 zhin3; zen3; tin1; zhin6; zihn3
U+632F.0	fCantonese	zan3; zan1
U+632F.0	fMandarin	zhen4; zhen3
U+632F.0	fEnglish	[1] raise; excite; arouse to action [2] pull up; save; relieve [3] shake; flap as wings
U+632F.1	fUTF8	振
U+632F.1	fCNS11643	1-5558
U+632F.1	fBig5	AEB6
U+632F.1	fGB	0-5571
U+632F.1	fR/S	 64.7
U+632F.1	fTotalStrokes	 10
U+632F.1	fCangjie	QMMV
U+632F.1	fFourCorner	51032
U+632F.1	fMandarin	zhen1
U+632F.1	fEnglish	benevolent and generous
U+6330.0	fUTF8	挰
U+6330.0	fCNS11643	3-312F
U+6330.0	fGB	3-4A3D
U+6330.0	fR/S	  64.7
U+6330.0	fTotalStrokes	 10; 11
U+6330.0	fCangjie	QRHG
U+6330.0	fFourCorner	56010
U+6330.0	fCantonese	caang4
U+6330.0	fMandarin	cheng2
U+6331.0	fUTF8	挱
U+6331.0	fCNS11643	3-3135
U+6331.0	fGB	E-2B3D
U+6331.0	fR/S	 64.7
U+6331.0	fTotalStrokes	 10
U+6331.0	fCangjie	QEFH
U+6331.0	fFourCorner	59020
U+6331.0	fHakka	  so1; sa1
U+6331.0	fCantonese	so1; saa1; sat3
U+6331.0	fMandarin	suo1
U+6331.0	fEnglish	 feel; feel with hands
U+6332.0	fUTF8	挲
U+6332.0	fCNS11643	2-354A
U+6332.0	fBig5	D566
U+6332.0	fGB	0-6A7D
U+6332.0	fR/S	 64.7
U+6332.0	fTotalStrokes	 11
U+6332.0	fCangjie	EHQ
U+6332.0	fFourCorner	39502
U+6332.0	fHakka	  so1; sa1
U+6332.0	fCantonese	so1; saa1; sat3
U+6332.0	fMandarin	suo1
U+6332.0	fEnglish	same as &#x6331;
U+6333.0	fUTF8	挳
U+6333.0	fCNS11643	2-2F48
U+6333.0	fBig5	D1C6
U+6333.0	fGB	E-2B3E
U+6333.0	fR/S	 64.7
U+6333.0	fTotalStrokes	 10
U+6333.0	fCangjie	QMVM
U+6333.0	fFourCorner	51011
U+6333.0	fHakka	 gien1; kien1; gen1; ken1
U+6333.0	fCantonese	hang1
U+6333.0	fMandarin	keng1
U+6333.0	fEnglish	same as &#x647C;
U+6334.0	fUTF8	挴
U+6334.0	fCNS11643	2-2F42
U+6334.0	fBig5	D1C0
U+6334.0	fGB	3-4A3F
U+6334.0	fR/S	 64.7
U+6334.0	fTotalStrokes	 10
U+6334.0	fCangjie	QOWY
U+6334.0	fFourCorner	58057
U+6334.0	fCantonese	mui5
U+6334.0	fMandarin	mei3
U+6335.0	fUTF8	挵
U+6335.0	fCNS11643	3-3138
U+6335.0	fGB	E-2B3F
U+6335.0	fR/S	 64.7
U+6335.0	fTotalStrokes	 10
U+6335.0	fCangjie	QMGT
U+6335.0	fFourCorner	51040
U+6335.0	fCantonese	lung6
U+6335.0	fMandarin	long4; nong4
U+6335.0	fEnglish	to play with, handle things
U+6336.0	fUTF8	挶
U+6336.0	fCNS11643	2-2F39
U+6336.0	fBig5	D1B7
U+6336.0	fGB	3-4A47
U+6336.0	fR/S	 64.7
U+6336.0	fTotalStrokes	 10
U+6336.0	fCangjie	QSSR
U+6336.0	fFourCorner	57027
U+6336.0	fHakka	 kiuk8
U+6336.0	fCantonese	guk6; guk1
U+6336.0	fMandarin	ju2
U+6336.0	fEnglish	 receptacle for earth
U+6337.0	fUTF8	挷
U+6337.0	fCNS11643	3-3137
U+6337.0	fGB	5-4B75
U+6337.0	fR/S	 64.7
U+6337.0	fTotalStrokes	 10
U+6337.0	fCangjie	QQJL
U+6337.0	fFourCorner	57020
U+6337.0	fHakka	 bong3;  bang1; bong1
U+6337.0	fCantonese	pong3; bong1
U+6337.0	fMandarin	bang4; peng2; beng1; beng4; nang4; bang3
U+6337.0	fEnglish	to propel a boat; to beat
U+6338.0	fUTF8	挸
U+6338.0	fCNS11643	2-2F4B
U+6338.0	fBig5	D1C9
U+6338.0	fGB	5-4B5C
U+6338.0	fR/S	 64.7
U+6338.0	fTotalStrokes	 10
U+6338.0	fCangjie	QBUU
U+6338.0	fFourCorner	56010
U+6338.0	fCantonese	gin2; gaan2
U+6338.0	fMandarin	jian3
U+6339.0	fUTF8	挹
U+6339.0	fCNS11643	2-2F3C
U+6339.0	fBig5	D1BA
U+6339.0	fGB	0-5E5A
U+6339.0	fR/S	 64.7
U+6339.0	fTotalStrokes	 10
U+6339.0	fCangjie	QRAU
U+6339.0	fFourCorner	56017
U+6339.0	fHakka	 jip7; rip7; gip7
U+6339.0	fCantonese	jap1
U+6339.0	fMandarin	yi4
U+6339.0	fEnglish	[1] decant liquids (especially wine) [2] retreat
U+633A.0	fUTF8	挺
U+633A.0	fCNS11643	1-555E
U+633A.0	fBig5	AEBC
U+633A.0	fGB	0-4D26
U+633A.0	fR/S	 64.7
U+633A.0	fTotalStrokes	 10
U+633A.0	fCangjie	QNKG
U+633A.0	fFourCorner	52041
U+633A.0	fHakka	  tin3;  ten3; {tin1}; tin2; tiang3; ciam1
U+633A.0	fCantonese	ting5; ting2 (also)
U+633A.0	fMandarin	ting3; ting2; zheng4
U+633A.0	fEnglish	[1] stand straight for; stand upright; straighten; square; rigid [2] pull up [3] unyielding; unbending; tough [4] outstanding; remarkable; eminent; prominent [5] sustain; endure; pull through; stand; hold out [6] thrust forward (as one's breast) [7] very; pretty [8] noun classifier for machine guns
U+633B.0	fUTF8	挻
U+633B.0	fCNS11643	2-3561
U+633B.0	fBig5	D57D
U+633B.0	fGB	3-4A2D
U+633B.0	fR/S	 64.7
U+633B.0	fTotalStrokes	 10
U+633B.0	fCangjie	QNKM
U+633B.0	fFourCorner	52041
U+633B.0	fHakka	 jen2; zan1; zhan1; zan2; shan1; zhen1; jan2
U+633B.0	fCantonese	saan1; sin1
U+633B.0	fMandarin	shan1; yan2
U+633B.0	fEnglish	 lead on; delay
U+633C.0	fUTF8	挼
U+633C.0	fCNS11643	2-2F3F
U+633C.0	fBig5	D1BD
U+633C.0	fGB	8-2F6F
U+633C.0	fR/S	 64.7
U+633C.0	fTotalStrokes	 10
U+633C.0	fCangjie	QBV
U+633C.0	fFourCorner	52047
U+633C.0	fHakka	  no2; lo2
U+633C.0	fCantonese	no4; lo4 (also)
U+633C.0	fMandarin	nuo4; ruo2; hui1; nuo2; ne2; sui1
U+633C.0	fEnglish	[1] rub; stroke [2] crumple (paper into a ball, etc.)
U+633D.0	fUTF8	挽
U+633D.0	fCNS11643	1-5560
U+633D.0	fBig5	AEBE
U+633D.0	fGB	0-4D6C
U+633D.0	fR/S	 64.7
U+633D.0	fTotalStrokes	 10
U+633D.0	fCangjie	QNAU
U+633D.0	fFourCorner	57016
U+633D.0	fHakka	  wan3;  wan1; van3; van1
U+633D.0	fCantonese	waan5; maan5; wan5
U+633D.0	fMandarin	wan3; mian3
U+633D.0	fEnglish	[1] draw (a bow, etc.) [2] restore [3] seize [4] roll up (sleeves, etc.)
U+633E.0	fUTF8	挾
U+633E.0	fCNS11643	1-5557
U+633E.0	fBig5	AEB5
U+633E.0	fGB	1-502E
U+633E.0	fR/S	 64.7
U+633E.0	fTotalStrokes	 10
U+633E.0	fCangjie	QKOO
U+633E.0	fFourCorner	54038
U+633E.0	fHakka	 hiap8;  kiap8; giap8; gap7; giap7
U+633E.0	fCantonese	haap6 (also); hip3; gaap3; hip6; gaap6
U+633E.0	fMandarin	xie2; jia2; jia1; jie1; xia2; xie3; za1
U+633E.0	fEnglish	[1] clasp under arm; hold under the arm [2] embrace; bosom [3] presume upon (one's influence; advantage, etc.) [4] exhort; blackmail; hold as a hostage (a crown prince, etc.)
U+633F.0	fUTF8	挿
U+633F.0	fCNS11643	3-313C
U+633F.0	fGB	E-2B40
U+633F.0	fR/S	 64.7
U+633F.0	fTotalStrokes	 10
U+633F.0	fCangjie	QHJL
U+633F.0	fFourCorner	52050
U+633F.0	fHakka	 cap7
U+633F.0	fCantonese	caap3
U+633F.0	fMandarin	cha1; zha3
U+633F.0	fEnglish	insert, stick into; plant
U+6340.0	fUTF8	捀
U+6340.0	fCNS11643	2-2F4D
U+6340.0	fBig5	D1CB
U+6340.0	fGB	3-4A42
U+6340.0	fR/S	 64.7
U+6340.0	fTotalStrokes	 10
U+6340.0	fCangjie	QHEJ
U+6340.0	fFourCorner	57054
U+6340.0	fCantonese	fung4; fung6
U+6340.0	fMandarin	feng1; feng2; feng4
U+6341.0	fUTF8	捁
U+6341.0	fCNS11643	2-2F41
U+6341.0	fBig5	D1BF
U+6341.0	fGB	3-4A3E
U+6341.0	fR/S	 64.7
U+6341.0	fTotalStrokes	 10
U+6341.0	fCangjie	QHGR
U+6341.0	fFourCorner	54061
U+6341.0	fCantonese	gaau2
U+6341.0	fMandarin	jiao3; jia3; ku4
U+6342.0	fUTF8	捂
U+6342.0	fCNS11643	1-555A
U+6342.0	fBig5	AEB8
U+6342.0	fGB	0-4E66
U+6342.0	fR/S	 64.7
U+6342.0	fTotalStrokes	 10
U+6342.0	fCangjie	QMMR
U+6342.0	fFourCorner	51061
U+6342.0	fHakka	 ngu5; ngu6; ngu3
U+6342.0	fCantonese	wu2; ng6 (also); ng5
U+6342.0	fMandarin	wu3; wu2; wu4
U+6342.0	fEnglish	resist
U+6343.0	fUTF8	捃
U+6343.0	fCNS11643	2-2F3A
U+6343.0	fBig5	D1B8
U+6343.0	fGB	0-5E5C
U+6343.0	fR/S	 64.7
U+6343.0	fTotalStrokes	 10
U+6343.0	fCangjie	QSKR
U+6343.0	fFourCorner	57067
U+6343.0	fHakka	 giun3; kiun3; kiun2
U+6343.0	fCantonese	kwan2; kwan3 (also); gwan3
U+6343.0	fMandarin	jun4
U+6343.0	fEnglish	 pick up; gather; collect
U+6344.0	fUTF8	捄
U+6344.0	fCNS11643	2-2F37
U+6344.0	fBig5	D1B5
U+6344.0	fGB	E-2B41
U+6344.0	fR/S	 64.7
U+6344.0	fTotalStrokes	 10
U+6344.0	fCangjie	QIJE
U+6344.0	fFourCorner	53032
U+6344.0	fHakka	  gi1;  kiu2; {giu2}; giu2
U+6344.0	fCantonese	kau4; geoi1; gau3; haau6; kau3
U+6344.0	fMandarin	jiu4; jiu1; ju1; qiu2
U+6344.0	fEnglish	long
U+6345.0	fUTF8	捅
U+6345.0	fCNS11643	2-2F38
U+6345.0	fBig5	D1B6
U+6345.0	fGB	0-4D31
U+6345.0	fR/S	 64.7
U+6345.0	fTotalStrokes	 10
U+6345.0	fCangjie	QNIB
U+6345.0	fFourCorner	57027
U+6345.0	fHakka	  tung3; dung3
U+6345.0	fCantonese	tung2; tong2
U+6345.0	fMandarin	tong3
U+6345.0	fEnglish	jab
U+6346.0	fUTF8	捆
U+6346.0	fCNS11643	1-555B
U+6346.0	fBig5	AEB9
U+6346.0	fGB	0-4026
U+6346.0	fR/S	 64.7
U+6346.0	fTotalStrokes	 10
U+6346.0	fCangjie	QWD
U+6346.0	fFourCorner	56000
U+6346.0	fHakka	  kun3; kun2; kwun3
U+6346.0	fCantonese	kwan2
U+6346.0	fMandarin	kun3; hun2; kun4
U+6346.0	fEnglish	[1] tie up; bind [2] bundle
U+6347.0	fUTF8	捇
U+6347.0	fCNS11643	2-2F47
U+6347.0	fBig5	D1C5
U+6347.0	fGB	5-4C2E
U+6347.0	fR/S	 64.7
U+6347.0	fTotalStrokes	 10
U+6347.0	fCangjie	QGLC
U+6347.0	fFourCorner	54031
U+6347.0	fCantonese	waak6; cik3
U+6347.0	fMandarin	huo4; he4; qi4
U+6348.0	fUTF8	捈
U+6348.0	fCNS11643	2-2F4E
U+6348.0	fBig5	D1CC
U+6348.0	fGB	3-4A40
U+6348.0	fR/S	 64.7
U+6348.0	fTotalStrokes	 10
U+6348.0	fCangjie	QOMD
U+6348.0	fFourCorner	58091
U+6348.0	fCantonese	tou4; caa4; syu1
U+6348.0	fMandarin	tu2; chu1; tu1; ye2
U+6349.0	fUTF8	捉
U+6349.0	fCNS11643	1-555D
U+6349.0	fBig5	AEBB
U+6349.0	fGB	0-573D
U+6349.0	fR/S	 64.7
U+6349.0	fTotalStrokes	 10
U+6349.0	fCangjie	QRYO
U+6349.0	fFourCorner	56081
U+6349.0	fHakka	  zuk7;  zok7
U+6349.0	fCantonese	zuk1; zuk3 (also)
U+6349.0	fMandarin	zhuo1
U+6349.0	fEnglish	[1] grasp; catch; seize; hold [2] apprehend; arrest
U+634A.0	fUTF8	捊
U+634A.0	fCNS11643	2-2F3E
U+634A.0	fBig5	D1BC
U+634A.0	fGB	5-4C3C
U+634A.0	fR/S	 64.7
U+634A.0	fTotalStrokes	 10
U+634A.0	fCangjie	QBND
U+634A.0	fFourCorner	52047
U+634A.0	fHakka	 feu2; piau2; p'ieu2; piau1
U+634A.0	fCantonese	pau4
U+634A.0	fMandarin	pou2; fou1; fou2; fu2; po1
U+634B.0	fUTF8	捋
U+634B.0	fCNS11643	2-2F3D
U+634B.0	fBig5	D1BB
U+634B.0	fGB	0-5E5B
U+634B.0	fR/S	 64.7
U+634B.0	fTotalStrokes	 10
U+634B.0	fCangjie	QBDI
U+634B.0	fFourCorner	52043
U+634B.0	fHakka	 lot8
U+634B.0	fCantonese	lyut3; lyut6; tyut3
U+634B.0	fMandarin	le4; lun4; l3
U+634B.0	fEnglish	 pluck; gather in the fingers
U+634B.1	fUTF8	捋
U+634B.1	fCNS11643	2-2F3D
U+634B.1	fBig5	D1BB
U+634B.1	fGB	0-5E5B
U+634B.1	fR/S	 64.7
U+634B.1	fTotalStrokes	 10
U+634B.1	fCangjie	QBDI
U+634B.1	fFourCorner	52043
U+634B.1	fHakka	 lot7
U+634B.1	fMandarin	luu3
U+634B.1	fEnglish	 stroke (one's beard, etc.); smooth out with fingers
U+634B.2	fUTF8	捋
U+634B.2	fCNS11643	2-2F3D
U+634B.2	fBig5	D1BB
U+634B.2	fGB	0-5E5B
U+634B.2	fR/S	 64.7
U+634B.2	fTotalStrokes	 10
U+634B.2	fCangjie	QBDI
U+634B.2	fFourCorner	52043
U+634B.2	fMandarin	luo1
U+634B.2	fEnglish	[1] rub one's palm along (something long) [2] squeeze with hands
U+634C.0	fUTF8	捌
U+634C.0	fCNS11643	1-5565
U+634C.0	fBig5	AEC3
U+634C.0	fGB	0-3046
U+634C.0	fR/S	 64.7
U+634C.0	fTotalStrokes	 10
U+634C.0	fCangjie	QRSN
U+634C.0	fFourCorner	52000
U+634C.0	fHakka	 bat7
U+634C.0	fCantonese	baat3
U+634C.0	fMandarin	ba1; bie2
U+634C.0	fEnglish	[1] break; split open; pull apart [2] eight used in accountancy
U+634D.0	fUTF8	捍
U+634D.0	fCNS11643	1-5564
U+634D.0	fBig5	AEC2
U+634D.0	fGB	0-3A34
U+634D.0	fR/S	 64.7
U+634D.0	fTotalStrokes	 10
U+634D.0	fCangjie	QAMJ
U+634D.0	fFourCorner	56041
U+634D.0	fHakka	  gon5;  hon5; hon6; hon3
U+634D.0	fCantonese	hon6; hon2 (also); hon5 (also); kon2
U+634D.0	fMandarin	han4; gan3; han3; xian3
U+634D.0	fEnglish	 ward off; guard defend
U+634E.0	fUTF8	捎
U+634E.0	fCNS11643	1-5556
U+634E.0	fBig5	AEB4
U+634E.0	fGB	0-4953
U+634E.0	fR/S	 64.7
U+634E.0	fTotalStrokes	 10
U+634E.0	fCangjie	QFB
U+634E.0	fFourCorner	59027
U+634E.0	fHakka	  sau1; seu1; siau1; shau1
U+634E.0	fCantonese	saau1; siu1
U+634E.0	fMandarin	shao1; shao3; xiao1
U+634E.0	fEnglish	[1] take along at one's convenience; bring along at one's convenience [2] brush over lightly [3] wipe out
U+634E.1	fUTF8	捎
U+634E.1	fCNS11643	1-5556
U+634E.1	fBig5	AEB4
U+634E.1	fGB	0-4953
U+634E.1	fR/S	 64.7
U+634E.1	fTotalStrokes	 10
U+634E.1	fCangjie	QFB
U+634E.1	fFourCorner	59027
U+634E.1	fMandarin	shao4
U+634E.1	fEnglish	[1] fade; discolor (said of colors) [2] sprinkle water [3] watch secretly; spy on; glance at
U+634F.0	fUTF8	捏
U+634F.0	fCNS11643	1-555C
U+634F.0	fBig5	AEBA
U+634F.0	fGB	0-4473
U+634F.0	fR/S	 64.7
U+634F.0	fTotalStrokes	 10
U+634F.0	fCangjie	QHXG
U+634F.0	fFourCorner	56014
U+634F.0	fHakka	  ngiap7; net7; let7
U+634F.0	fCantonese	nip6; lip6 (also)
U+634F.0	fMandarin	nie1
U+634F.0	fEnglish	[1] squeeze with fingers; press with fingers; knead; pinch [2] mold (mud, etc.) [3] fabricate; trump up; make up [4] shake [5] take
U+6350.0	fUTF8	捐
U+6350.0	fCNS11643	1-555F
U+6350.0	fBig5	AEBD
U+6350.0	fGB	0-3E68
U+6350.0	fR/S	 64.7
U+6350.0	fTotalStrokes	 10
U+6350.0	fCangjie	QRB
U+6350.0	fFourCorner	56027
U+6350.0	fHakka	  gien1;  gen1; {gien3}; kian1; gian1
U+6350.0	fCantonese	gyun1
U+6350.0	fMandarin	juan1; yuan2; yan2
U+6350.0	fEnglish	[1] contribute; donate; subscribe [2] tax; duty; charge; dues [3] buy (an official rank) [4] give up (one's life for a cause, etc.) [5] remove
U+6351.0	fUTF8	捑
U+6351.0	fCNS11643	2-2F4A
U+6351.0	fBig5	D1C8
U+6351.0	fGB	E-2B42
U+6351.0	fR/S	 64.7
U+6351.0	fTotalStrokes	 10
U+6351.0	fCangjie	QAVK
U+6351.0	fFourCorner	56034
U+6351.0	fCantonese	zik6; sik6
U+6351.0	fMandarin	ze2; she4
U+6352.0	fUTF8	捒
U+6352.0	fCNS11643	3-312E
U+6352.0	fGB	5-4C32
U+6352.0	fR/S	  64.7
U+6352.0	fTotalStrokes	 10
U+6352.0	fCangjie	QDL
U+6352.0	fFourCorner	55090
U+6352.0	fHakka	  sung3;  suk7
U+6352.0	fCantonese	cuk1
U+6352.0	fMandarin	song3; shu4; sou1
U+6352.0	fEnglish	(sung3) respect (suk7) bind
U+6353.0	fUTF8	捓
U+6353.0	fCNS11643	3-3139
U+6353.0	fGB	5-4B7B
U+6353.0	fR/S	  64.7; 64.8
U+6353.0	fTotalStrokes	 10
U+6353.0	fCangjie	QMHL
U+6353.0	fFourCorner	57020
U+6353.0	fHakka	 ja2; ra2
U+6353.0	fCantonese	je4
U+6353.0	fMandarin	ye2
U+6353.0	fEnglish	strive; endeavor
U+6354.0	fUTF8	捔
U+6354.0	fCNS11643	2-2F44
U+6354.0	fBig5	D1C2
U+6354.0	fGB	3-4A41
U+6354.0	fR/S	 64.7
U+6354.0	fTotalStrokes	 10
U+6354.0	fCangjie	QNBG
U+6354.0	fFourCorner	57027
U+6354.0	fHakka	  gok7
U+6354.0	fCantonese	gok3
U+6354.0	fMandarin	jue2; chuo4; zhuo2
U+6354.0	fEnglish	[1] seize by the horns [2] stab
U+6355.0	fUTF8	捕
U+6355.0	fCNS11643	1-5559
U+6355.0	fBig5	AEB7
U+6355.0	fGB	0-3236
U+6355.0	fR/S	 64.7
U+6355.0	fTotalStrokes	 10
U+6355.0	fCangjie	QIJB
U+6355.0	fFourCorner	53027
U+6355.0	fHakka	  pu3; pu5; pu6
U+6355.0	fCantonese	bou6
U+6355.0	fMandarin	bu3; bu4; fu4
U+6355.0	fEnglish	[1] arrest; catch; seize; apprehend [2] policeman
U+6356.0	fUTF8	捖
U+6356.0	fCNS11643	2-2F35
U+6356.0	fBig5	D1B3
U+6356.0	fGB	5-4C3F
U+6356.0	fR/S	 64.7
U+6356.0	fTotalStrokes	 10
U+6356.0	fCangjie	QJMU
U+6356.0	fFourCorner	53011
U+6356.0	fHakka	 wan2;  fan5; van2; van3
U+6356.0	fCantonese	gwaat3; jyun4
U+6356.0	fMandarin	huan2; wan2; huan3; gua1; kuan3
U+6356.0	fEnglish	[1] rub to polish gems [2] strike [3] hang up on a hook
U+6357.0	fUTF8	捗
U+6357.0	fCNS11643	2-2F4C
U+6357.0	fBig5	D1CA
U+6357.0	fGB	3-4A3B
U+6357.0	fR/S	 64.7
U+6357.0	fTotalStrokes	 10
U+6357.0	fCangjie	QYLH
U+6357.0	fFourCorner	51021
U+6357.0	fCantonese	bou6; zik1; pou4
U+6357.0	fMandarin	bu4; pu2; zhi4
U+6357.0	fEnglish	make progress
U+6358.0	fUTF8	捘
U+6358.0	fCNS11643	2-2F43
U+6358.0	fBig5	D1C1
U+6358.0	fGB	3-4A48
U+6358.0	fR/S	 64.7
U+6358.0	fTotalStrokes	 10
U+6358.0	fCangjie	QICE
U+6358.0	fFourCorner	53047
U+6358.0	fHakka	 zun3;  zun5
U+6358.0	fCantonese	zyun3; zeon3 (also); zeoi3
U+6358.0	fMandarin	zun4; qun1; zui1; zui4
U+6358.0	fEnglish	push
U+6359.0	fUTF8	捙
U+6359.0	fCNS11643	2-2F45
U+6359.0	fBig5	D1C3
U+6359.0	fGB	E-2B43
U+6359.0	fR/S	 64.7
U+6359.0	fTotalStrokes	 10
U+6359.0	fCangjie	QJWJ
U+6359.0	fFourCorner	55000
U+6359.0	fCantonese	jai6
U+6359.0	fMandarin	yi4
U+635A.0	fUTF8	捚
U+635A.0	fCNS11643	2-2F49
U+635A.0	fBig5	D1C7
U+635A.0	fGB	5-4C39
U+635A.0	fR/S	 64.7
U+635A.0	fTotalStrokes	 10
U+635A.0	fCangjie	QWG
U+635A.0	fFourCorner	56014
U+635A.0	fCantonese	zaai1
U+635A.0	fMandarin	zhai1
U+635B.0	fUTF8	捛
U+635B.0	fCNS11643	3-3130
U+635B.0	fGB	E-2B44
U+635B.0	fR/S	 64.7
U+635B.0	fTotalStrokes	 10
U+635B.0	fCangjie	QRHR
U+635B.0	fFourCorner	56060
U+635B.0	fCantonese	leoi5
U+635B.0	fMandarin	luu3; l3
U+635C.0	fUTF8	捜
U+635C.0	fCNS11643	3-313B
U+635C.0	fGB	E-2B45
U+635C.0	fR/S	 64.7
U+635C.0	fTotalStrokes	 10
U+635C.0	fCangjie	QLWE
U+635C.0	fFourCorner	55040
U+635C.0	fCantonese	sau1
U+635C.0	fMandarin	sou1
U+635C.0	fEnglish	 search; seek investigate
U+635D.0	fUTF8	捝
U+635D.0	fCNS11643	3-3136
U+635D.0	fGB	3-4A44
U+635D.0	fR/S	 64.7
U+635D.0	fTotalStrokes	 10
U+635D.0	fCangjie	QCRU
U+635D.0	fFourCorner	58010
U+635D.0	fCantonese	tyut3; seoi3
U+635D.0	fMandarin	yan3; tuo1; shui2
U+635E.0	fUTF8	捞
U+635E.0	fGB	0-404C
U+635E.0	fR/S	 64.7
U+635E.0	fTotalStrokes	 10
U+635E.0	fCangjie	QTBS
U+635E.0	fHakka	 leu2; lau5
U+635E.0	fCantonese	laau4; laau1 (also); lou4 (also); lou1
U+635E.0	fMandarin	lao1; lao2; lao4; liao2; liao4
U+635E.0	fEnglish	simplified form of &#x6488;
U+635F.0	fUTF8	损
U+635F.0	fGB	0-4B70
U+635F.0	fR/S	 64.7
U+635F.0	fTotalStrokes	 10
U+635F.0	fCangjie	QRBO
U+635F.0	fHakka	 sun3
U+635F.0	fCantonese	syun2
U+635F.0	fMandarin	sun3
U+635F.0	fEnglish	simplified form of &#x640D;
U+6360.0	fUTF8	捠
U+6360.0	fCNS11643	3-647C
U+6360.0	fGB	E-2B46
U+6360.0	fR/S	 64.7; 64.6
U+6360.0	fTotalStrokes	 10
U+6360.0	fCangjie	QOMC
U+6360.0	fFourCorner	5208
U+6360.0	fCantonese	bang1
U+6360.0	fMandarin	bang1
U+6361.0	fUTF8	捡
U+6361.0	fGB	0-3C71
U+6361.0	fR/S	 64.7
U+6361.0	fTotalStrokes	 10
U+6361.0	fCangjie	QOMM
U+6361.0	fHakka	 giam3
U+6361.0	fCantonese	gim2
U+6361.0	fMandarin	jian3; lian4
U+6361.0	fEnglish	simplified form of &#x64BF;
U+6362.0	fUTF8	换
U+6362.0	fGB	0-3B3B
U+6362.0	fR/S	 64.7
U+6362.0	fTotalStrokes	 10
U+6362.0	fCangjie	XQNBK
U+6362.0	fHakka	 von3; fan5; van5; fon5
U+6362.0	fCantonese	wun6
U+6362.0	fMandarin	huan4
U+6362.0	fEnglish	simplified form of &#x63DB;
U+6363.0	fUTF8	捣
U+6363.0	fGB	0-3537
U+6363.0	fR/S	 64.7
U+6363.0	fTotalStrokes	 10
U+6363.0	fCangjie	QPYU
U+6363.0	fHakka	 dau3; dau5
U+6363.0	fCantonese	dou2
U+6363.0	fMandarin	dao3
U+6363.0	fEnglish	simplified form of &#x6417;
U+6364.0	fUTF8	捤
U+6364.0	fCNS11643	3-647D
U+6364.0	fGB	E-2B47
U+6364.0	fR/S	 64.7
U+6364.0	fTotalStrokes	 10
U+6364.0	fCangjie	QSHU
U+6364.0	fFourCorner	5701
U+6364.0	fCantonese	mei5
U+6365.0	fUTF8	捥
U+6365.0	fCNS11643	2-354B
U+6365.0	fBig5	D567
U+6365.0	fGB	5-4C59
U+6365.0	fR/S	 64.8
U+6365.0	fTotalStrokes	 11
U+6365.0	fCangjie	QJNU
U+6365.0	fFourCorner	53012
U+6365.0	fHakka	 wan3; won3; von3
U+6365.0	fCantonese	wun2
U+6365.0	fMandarin	wan4; wan1; wan3; yu4
U+6365.0	fEnglish	 bend the wrist
U+6366.0	fUTF8	捦
U+6366.0	fCNS11643	3-3631
U+6366.0	fGB	5-4C51
U+6366.0	fR/S	 64.8
U+6366.0	fTotalStrokes	 11
U+6366.0	fCangjie	QC
U+6366.0	fFourCorner	58010
U+6366.0	fHakka	 kim2; kim3
U+6366.0	fCantonese	kam4
U+6366.0	fMandarin	qin2
U+6366.0	fEnglish	 seize clutch arrest capture
U+6367.0	fUTF8	捧
U+6367.0	fCNS11643	1-5A5B
U+6367.0	fBig5	B1B7
U+6367.0	fGB	0-4575
U+6367.0	fR/S	 64.8
U+6367.0	fTotalStrokes	 11
U+6367.0	fCangjie	QQKQ
U+6367.0	fFourCorner	55053
U+6367.0	fHakka	  bung3
U+6367.0	fCantonese	pung2; bung2 (also); fung2
U+6367.0	fMandarin	peng3; feng2; feng3; feng4
U+6367.0	fEnglish	[1] hold something in both hands; scoop up something with one's hands [2] boost; flatter; treat [3] support; cheer or render assistance; by one's presence
U+6368.0	fUTF8	捨
U+6368.0	fCNS11643	1-5A6F
U+6368.0	fBig5	B1CB
U+6368.0	fGB	1-4961
U+6368.0	fR/S	 64.8
U+6368.0	fTotalStrokes	 11
U+6368.0	fCangjie	QOMR
U+6368.0	fFourCorner	58061
U+6368.0	fHakka	  sa3; sha3; sa5
U+6368.0	fCantonese	se2; se5 (also)
U+6368.0	fMandarin	she3; she4
U+6368.0	fEnglish	[1] reject; discard; give up; abandon; renounce; relinquish; part with; forsake; let go [2] give alms; give to charity
U+6369.0	fUTF8	捩
U+6369.0	fCNS11643	1-5A6E
U+6369.0	fBig5	B1CA
U+6369.0	fGB	0-5E66
U+6369.0	fR/S	 64.8
U+6369.0	fTotalStrokes	 11
U+6369.0	fCangjie	QHSK
U+6369.0	fFourCorner	53034
U+6369.0	fHakka	 lit7; lui5; lui6; li6; li5; lui3; li3; liet8
U+6369.0	fCantonese	lit6; lai6; leoi6 (also); loei6
U+6369.0	fMandarin	lie4; li4
U+6369.0	fEnglish	[1] twist with hands [2] snap; tear apart; rip apart
U+636A.0	fUTF8	捪
U+636A.0	fCNS11643	3-3627
U+636A.0	fGB	5-4C53
U+636A.0	fR/S	  64.8
U+636A.0	fTotalStrokes	 11
U+636A.0	fCangjie	QHPA
U+636A.0	fFourCorner	52060
U+636A.0	fHakka	 min2; mien3; men3
U+636A.0	fCantonese	man4; maan5
U+636A.0	fMandarin	min2; min3
U+636A.0	fEnglish	same as &#x3A09;
U+636B.0	fUTF8	捫
U+636B.0	fCNS11643	1-5A63
U+636B.0	fBig5	B1BF
U+636B.0	fGB	1-5E51
U+636B.0	fR/S	 64.8
U+636B.0	fTotalStrokes	 11
U+636B.0	fCangjie	QAN
U+636B.0	fFourCorner	57020
U+636B.0	fHakka	  mun2; mun3
U+636B.0	fCantonese	mun4; mun1
U+636B.0	fMandarin	men2
U+636B.0	fEnglish	[1] feel or touch with hands; hold [2] search (in one's pocket, etc.)
U+636C.0	fUTF8	捬
U+636C.0	fCNS11643	3-3629
U+636C.0	fGB	5-4C55
U+636C.0	fR/S	  64.8
U+636C.0	fTotalStrokes	 11
U+636C.0	fCangjie	QIOI
U+636C.0	fFourCorner	50040
U+636C.0	fHakka	 wu3; vu3; mu3
U+636C.0	fCantonese	fu2
U+636C.0	fMandarin	fu3
U+636C.0	fEnglish	press down heavily with hands
U+636D.0	fUTF8	捭
U+636D.0	fCNS11643	2-355D
U+636D.0	fBig5	D579
U+636D.0	fGB	0-5E63
U+636D.0	fR/S	 64.8
U+636D.0	fTotalStrokes	 11
U+636D.0	fCangjie	QHWJ
U+636D.0	fFourCorner	56040
U+636D.0	fHakka	  pai2; bak7
U+636D.0	fCantonese	baai2
U+636D.0	fMandarin	bai3; bai4; pi3
U+636D.0	fEnglish	[1] open; spread out [2] strike with both hands
U+636E.0	fUTF8	据
U+636E.0	fCNS11643	2-3559
U+636E.0	fBig5	D575
U+636E.0	fGB	0-3E5D
U+636E.0	fR/S	 64.8
U+636E.0	fTotalStrokes	 11
U+636E.0	fCangjie	QSJR
U+636E.0	fFourCorner	57064
U+636E.0	fHakka	 gi1
U+636E.0	fCantonese	geoi1
U+636E.0	fMandarin	ju1
U+636E.0	fEnglish	used in combination
U+636E.1	fUTF8	据
U+636E.1	fCNS11643	2-3559
U+636E.1	fBig5	D575
U+636E.1	fGB	0-3E5D
U+636E.1	fR/S	 64.8
U+636E.1	fTotalStrokes	 11
U+636E.1	fCangjie	QSJR
U+636E.1	fFourCorner	57064
U+636E.1	fHakka	 gi3; gi5
U+636E.1	fCantonese	geoi3
U+636E.1	fMandarin	ju4
U+636E.1	fEnglish	[1] simplified form of &#x64DA; [2] same as &#x5028;
U+636F.0	fUTF8	捯
U+636F.0	fCNS11643	2-3556
U+636F.0	fBig5	D572
U+636F.0	fGB	3-4A4F
U+636F.0	fR/S	 64.8
U+636F.0	fTotalStrokes	 11
U+636F.0	fCangjie	QMGN
U+636F.0	fFourCorner	52020
U+636F.0	fHakka	 dau3; dau5
U+636F.0	fCantonese	dou3; dou2
U+636F.0	fMandarin	dao2; dao3
U+6370.0	fUTF8	捰
U+6370.0	fCNS11643	2-3568
U+6370.0	fBig5	D5A6
U+6370.0	fGB	E-2B48
U+6370.0	fR/S	 64.8
U+6370.0	fTotalStrokes	 11
U+6370.0	fCangjie	QWD
U+6370.0	fFourCorner	56094
U+6370.0	fCantonese	wo2
U+6370.0	fMandarin	wo3
U+6371.0	fUTF8	捱
U+6371.0	fCNS11643	1-5A5E
U+6371.0	fBig5	B1BA
U+6371.0	fGB	0-5E5F
U+6371.0	fR/S	 64.8
U+6371.0	fTotalStrokes	 11
U+6371.0	fCangjie	QMGG
U+6371.0	fFourCorner	51014
U+6371.0	fHakka	  ngai2; ngai5; ngai1; ngai6; ngai3; nai6; nai5; ai1
U+6371.0	fCantonese	ngaai4; aai4 (also); ai4
U+6371.0	fMandarin	ai2; ai1
U+6371.0	fEnglish	[1] suffer; endure [2] procrastinate; put off [3] rub (shoulders) [4] draw near; come close to
U+6372.0	fUTF8	捲
U+6372.0	fCNS11643	1-5A56
U+6372.0	fBig5	B1B2
U+6372.0	fGB	1-7847
U+6372.0	fR/S	 64.8
U+6372.0	fTotalStrokes	 11
U+6372.0	fCangjie	QFQU
U+6372.0	fFourCorner	59012
U+6372.0	fHakka	 gien3; kien2; gen3; kien3; gian3; kian2
U+6372.0	fCantonese	gyun2
U+6372.0	fMandarin	juan3; quan2; juan4; quan1; quan3
U+6372.0	fEnglish	[1] curl; curly (hair, etc.) [2] roll up roll [3] sweep off
U+6373.0	fUTF8	捳
U+6373.0	fCNS11643	3-3626
U+6373.0	fGB	3-4A53
U+6373.0	fR/S	  64.8
U+6373.0	fTotalStrokes	 11
U+6373.0	fCangjie	QOMU
U+6373.0	fFourCorner	52070
U+6373.0	fCantonese	ngok6; ok6 (also)
U+6373.0	fMandarin	yue4
U+6374.0	fUTF8	捴
U+6374.0	fCNS11643	3-3632
U+6374.0	fGB	E-2B49
U+6374.0	fR/S	 64.8
U+6374.0	fTotalStrokes	 11
U+6374.0	fCangjie	QCIP
U+6374.0	fFourCorner	58030
U+6374.0	fHakka	  zung3
U+6374.0	fCantonese	zung2
U+6374.0	fMandarin	zong3; zong1
U+6374.0	fEnglish	same as &#x6460;
U+6375.0	fUTF8	捵
U+6375.0	fCNS11643	2-355B
U+6375.0	fBig5	D577
U+6375.0	fGB	5-4C4F
U+6375.0	fR/S	 64.8
U+6375.0	fTotalStrokes	 11
U+6375.0	fCangjie	QTBC
U+6375.0	fFourCorner	55081
U+6375.0	fHakka	  dien3; zhen3
U+6375.0	fCantonese	din2; tin2; can2; nin2
U+6375.0	fMandarin	chen3; chen1; nian3; tian3
U+6375.0	fEnglish	[1] stretch; lengthen (as in making noodles, etc.) [2] drag out; draw out
U+6376.0	fUTF8	捶
U+6376.0	fCNS11643	1-5F4D
U+6376.0	fBig5	B4A8
U+6376.0	fGB	0-3437
U+6376.0	fR/S	 64.8
U+6376.0	fTotalStrokes	 11
U+6376.0	fCangjie	QHJM
U+6376.0	fFourCorner	52014
U+6376.0	fHakka	 cui2; ch'ui2; chui3; zui3; chui2
U+6376.0	fCantonese	ceoi4
U+6376.0	fMandarin	chui2; chui4; duo3; zhui3
U+6376.0	fEnglish	[1] beat; thrash; pound [2] stick for beating
U+6377.0	fUTF8	捷
U+6377.0	fCNS11643	1-5A5A
U+6377.0	fBig5	B1B6
U+6377.0	fGB	0-3D5D
U+6377.0	fR/S	 64.8
U+6377.0	fTotalStrokes	 11
U+6377.0	fCangjie	QJLO
U+6377.0	fFourCorner	55081
U+6377.0	fHakka	  ciap8
U+6377.0	fCantonese	zit6; zit3 (also); cip3
U+6377.0	fMandarin	jie2; qie4; cha1
U+6377.0	fEnglish	[1] win; victory; triumph; prices of a victory [2] swift; quick; rapid; agile; fast; alert
U+6378.0	fUTF8	捸
U+6378.0	fCNS11643	2-3563
U+6378.0	fBig5	D5A1
U+6378.0	fGB	E-2B4A
U+6378.0	fR/S	 64.8
U+6378.0	fTotalStrokes	 11
U+6378.0	fCangjie	QLE
U+6378.0	fFourCorner	55032
U+6378.0	fCantonese	teot1; dat6
U+6378.0	fMandarin	tu1
U+6379.0	fUTF8	捹
U+6379.0	fCNS11643	3-3622
U+6379.0	fGB	E-2B4B
U+6379.0	fR/S	  64.8
U+6379.0	fTotalStrokes	 11
U+6379.0	fCangjie	QKJT
U+6379.0	fFourCorner	54040
U+6379.0	fHakka	 bun1;  pun3
U+6379.0	fCantonese	ban3; fing6; fang6 (also); fing3
U+6379.0	fMandarin	ben4
U+6379.0	fEnglish	[1] throw into confusion [2] mix up
U+637A.0	fUTF8	捺
U+637A.0	fCNS11643	1-5A70
U+637A.0	fBig5	B1CC
U+637A.0	fGB	0-5E60
U+637A.0	fR/S	 64.8
U+637A.0	fTotalStrokes	 11
U+637A.0	fCangjie	QKMF
U+637A.0	fFourCorner	54091
U+637A.0	fHakka	 nak8; nat7; lat7; lai5; lak8; nak7
U+637A.0	fCantonese	naat6; laat6 (also)
U+637A.0	fMandarin	na4
U+637A.0	fEnglish	[1] press hard with hands; press down [2] downstroke slanting toward the right in Chinese writing [3] repress; restrain
U+637B.0	fUTF8	捻
U+637B.0	fCNS11643	1-5A6D
U+637B.0	fBig5	B1C9
U+637B.0	fGB	0-446D
U+637B.0	fR/S	 64.8
U+637B.0	fTotalStrokes	 11
U+637B.0	fCangjie	QOIP
U+637B.0	fFourCorner	58032
U+637B.0	fHakka	  ngiap7; ngiam1; ngen3; jap7; ngiam3; ban5; ngien3
U+637B.0	fCantonese	nim2; nin2 (also); nip6; lim2 (also); lin2 (also); lip6 (also); nan2; nim1; nim5
U+637B.0	fMandarin	nian3; nie4
U+637B.0	fEnglish	[1] nip with fingers [2] the "Nien Bandits" [3] twist [4] follow up [5] something made by twisting
U+637B.1	fUTF8	捻
U+637B.1	fCNS11643	1-5A6D
U+637B.1	fBig5	B1C9
U+637B.1	fGB	0-446D
U+637B.1	fR/S	 64.8
U+637B.1	fTotalStrokes	 11
U+637B.1	fCangjie	QOIP
U+637B.1	fFourCorner	58032
U+637B.1	fMandarin	nie1
U+637B.1	fEnglish	 pinch or knead with the fingers
U+637C.0	fUTF8	捼
U+637C.0	fCNS11643	2-355F
U+637C.0	fBig5	D57B
U+637C.0	fGB	E-2B4C
U+637C.0	fR/S	 64.8
U+637C.0	fTotalStrokes	 11
U+637C.0	fCangjie	QHDV
U+637C.0	fFourCorner	52044
U+637C.0	fHakka	  no2; lo2
U+637C.0	fCantonese	no4; lo4 (also)
U+637C.0	fMandarin	nuo2; ruan2; ruo2; rui2; wei3; wo1
U+637C.0	fEnglish	 rub crumple
U+637D.0	fUTF8	捽
U+637D.0	fCNS11643	2-354E
U+637D.0	fBig5	D56A
U+637D.0	fGB	3-4A58
U+637D.0	fR/S	 64.8
U+637D.0	fTotalStrokes	 11
U+637D.0	fCangjie	QYOJ
U+637D.0	fFourCorner	50048
U+637D.0	fHakka	 cut8; zut8; cut7
U+637D.0	fCantonese	zeot1; cyut3 (also); zyut6
U+637D.0	fMandarin	zu2; zuo2; zui4
U+637D.0	fEnglish	[1] hold with hands; seize; grasp [2] pull up [3] go against; contradict; be in conflict with
U+637E.0	fUTF8	捾
U+637E.0	fCNS11643	3-357C
U+637E.0	fGB	3-4A5B
U+637E.0	fR/S	  64.8
U+637E.0	fTotalStrokes	 11
U+637E.0	fCangjie	QJRR
U+637E.0	fFourCorner	53070
U+637E.0	fHakka	  gwon3; wat7; vat7
U+637E.0	fCantonese	wut6
U+637E.0	fMandarin	wo4
U+637E.0	fEnglish	 take up; lift
U+637F.0	fUTF8	捿
U+637F.0	fCNS11643	3-362B
U+637F.0	fGB	5-4C4B
U+637F.0	fR/S	 64.8
U+637F.0	fTotalStrokes	 11
U+637F.0	fCangjie	QJLV
U+637F.0	fFourCorner	55040
U+637F.0	fCantonese	cai1
U+637F.0	fMandarin	xi1; qi1
U+637F.0	fEnglish	 perch roost stay
U+6380.0	fUTF8	掀
U+6380.0	fCNS11643	1-5A6C
U+6380.0	fBig5	B1C8
U+6380.0	fGB	0-4F46
U+6380.0	fR/S	 64.8
U+6380.0	fTotalStrokes	 11
U+6380.0	fCangjie	QHLO
U+6380.0	fFourCorner	57082
U+6380.0	fHakka	  hien1; hen1; hiun1
U+6380.0	fCantonese	hin1; hin3
U+6380.0	fMandarin	xian1; hen2; qin4; xi4; xin1
U+6380.0	fEnglish	[1] lift with the hands; raise [2] stir; stir up; cause; rise; expose; reveal
U+6381.0	fUTF8	掁
U+6381.0	fCNS11643	2-3565
U+6381.0	fBig5	D5A3
U+6381.0	fGB	3-4968
U+6381.0	fR/S	 64.8
U+6381.0	fTotalStrokes	 11
U+6381.0	fCangjie	QSMV
U+6381.0	fFourCorner	51032
U+6381.0	fCantonese	caang4
U+6381.0	fMandarin	cheng2; cheng1
U+6382.0	fUTF8	掂
U+6382.0	fCNS11643	2-354D
U+6382.0	fBig5	D569
U+6382.0	fGB	0-3560
U+6382.0	fR/S	 64.8
U+6382.0	fTotalStrokes	 11
U+6382.0	fCangjie	QIYR
U+6382.0	fFourCorner	50061
U+6382.0	fHakka	 diam5; diam1; tiam3
U+6382.0	fCantonese	dim1; dim6; dim3
U+6382.0	fMandarin	dian1
U+6382.0	fEnglish	[1] estimate the weight of something by holding or weighing it with hands [2] shake; touch [3] satisfactory; in good order; straight; vertical
U+6383.0	fUTF8	掃
U+6383.0	fCNS11643	1-5A61
U+6383.0	fBig5	B1BD
U+6383.0	fGB	1-4928
U+6383.0	fR/S	 64.8
U+6383.0	fTotalStrokes	 11
U+6383.0	fCangjie	QSMB
U+6383.0	fFourCorner	57027
U+6383.0	fHakka	  sau5;  so5
U+6383.0	fCantonese	sou3; sou2
U+6383.0	fMandarin	sao3
U+6383.0	fEnglish	[1] sweep with a broom; clear away; clean [2] exterminate; wipe out; weed out; mop up [3] sweepingly; totally [4] paint (the eyebrows, etc) [5] pass over to examine; sweep
U+6383.1	fUTF8	掃
U+6383.1	fCNS11643	1-5A61
U+6383.1	fBig5	B1BD
U+6383.1	fGB	1-4928
U+6383.1	fR/S	 64.8
U+6383.1	fTotalStrokes	 11
U+6383.1	fCangjie	QSMB
U+6383.1	fFourCorner	57027
U+6383.1	fMandarin	sao4
U+6383.1	fEnglish	 broom
U+6384.0	fUTF8	掄
U+6384.0	fCNS11643	1-5A65
U+6384.0	fBig5	B1C1
U+6384.0	fGB	1-4255
U+6384.0	fR/S	 64.8
U+6384.0	fTotalStrokes	 11
U+6384.0	fCangjie	QOMB
U+6384.0	fFourCorner	58027
U+6384.0	fHakka	 lun2; lun3
U+6384.0	fCantonese	leon4
U+6384.0	fMandarin	lun2
U+6384.0	fEnglish	 select; choose
U+6384.1	fUTF8	掄
U+6384.1	fCNS11643	1-5A65
U+6384.1	fBig5	B1C1
U+6384.1	fGB	1-4255
U+6384.1	fR/S	 64.8
U+6384.1	fTotalStrokes	 11
U+6384.1	fCangjie	QOMB
U+6384.1	fFourCorner	58027
U+6384.1	fMandarin	lun1
U+6384.1	fEnglish	[1] swing; turn or spin with hands or arms [2] brandish [3] squander
U+6385.0	fUTF8	掅
U+6385.0	fCNS11643	2-3564
U+6385.0	fBig5	D5A2
U+6385.0	fGB	3-4A49
U+6385.0	fR/S	 64.8
U+6385.0	fTotalStrokes	 11
U+6385.0	fCangjie	QQMB
U+6385.0	fFourCorner	55027
U+6385.0	fCantonese	cing3
U+6385.0	fMandarin	qing4
U+6385.0	fEnglish	to lift up; to seize, grab
U+6386.0	fUTF8	掆
U+6386.0	fCNS11643	3-3623
U+6386.0	fGB	E-2B4D
U+6386.0	fR/S	  64.8
U+6386.0	fTotalStrokes	 11
U+6386.0	fCangjie	QBTU
U+6386.0	fFourCorner	57020
U+6386.0	fHakka	  gong1
U+6386.0	fCantonese	gong1
U+6386.0	fMandarin	gang1
U+6386.0	fEnglish	[1] carry with a pole on the shoulders of two or more men [2] bear with both hands
U+6387.0	fUTF8	掇
U+6387.0	fCNS11643	2-3557
U+6387.0	fBig5	D573
U+6387.0	fGB	0-365E
U+6387.0	fR/S	 64.8
U+6387.0	fTotalStrokes	 11
U+6387.0	fCangjie	QEEE
U+6387.0	fFourCorner	57047
U+6387.0	fHakka	  dot7; dok7
U+6387.0	fCantonese	zyut3; dyut3 (also)
U+6387.0	fMandarin	duo2; duo1; zhuo1; chuo2; zhuo2
U+6387.0	fEnglish	[1] collect; gather [2] plagiarize; pirate
U+6388.0	fUTF8	授
U+6388.0	fCNS11643	1-5A66
U+6388.0	fBig5	B1C2
U+6388.0	fGB	0-4A5A
U+6388.0	fR/S	 64.8
U+6388.0	fTotalStrokes	 11
U+6388.0	fCangjie	QBBE
U+6388.0	fFourCorner	52047
U+6388.0	fHakka	 shiu6; su3; siu3;  shiu3; su5; shu3
U+6388.0	fCantonese	sau6
U+6388.0	fMandarin	shou4
U+6388.0	fEnglish	[1] give to; hand over to; transmit; confer (a degree; prize, etc.) [2] give up (one's life, etc.)
U+6389.0	fUTF8	掉
U+6389.0	fCNS11643	1-5A60
U+6389.0	fBig5	B1BC
U+6389.0	fGB	0-3574
U+6389.0	fR/S	 64.8
U+6389.0	fTotalStrokes	 11
U+6389.0	fCangjie	QYAJ
U+6389.0	fFourCorner	51046
U+6389.0	fHakka	  tiau5; tiau3; cau5
U+6389.0	fCantonese	diu6; zaau6 (also); deu6 (also)
U+6389.0	fMandarin	diao4; nao4; nuo4; tiao3
U+6389.0	fEnglish	[1] turn [2] fall; drop; shed [3] lose; be missing [4] fall behind; lag behind [5] change; substitute [6] move; shake; wag [7] used as an adverbial particle after verbs expressing conditions of fulfillment
U+638A.0	fUTF8	掊
U+638A.0	fCNS11643	2-354C
U+638A.0	fBig5	D568
U+638A.0	fGB	0-5E65
U+638A.0	fR/S	 64.8
U+638A.0	fTotalStrokes	 11
U+638A.0	fCangjie	QYTR
U+638A.0	fFourCorner	50061
U+638A.0	fHakka	 peu3;  feu2; piu2; piau1; piu3; piau3; pui2
U+638A.0	fCantonese	pau2; pau4; fau2
U+638A.0	fMandarin	pou3; fu4; bao4; bo2; fou2; fu2; pao2
U+638A.0	fEnglish	 strike; cut; cudgel; brak; attack
U+638A.1	fUTF8	掊
U+638A.1	fCNS11643	2-354C
U+638A.1	fBig5	D568
U+638A.1	fGB	0-5E65
U+638A.1	fR/S	 64.8
U+638A.1	fTotalStrokes	 11
U+638A.1	fCangjie	QYTR
U+638A.1	fFourCorner	50061
U+638A.1	fMandarin	pou2
U+638A.1	fEnglish	 extract; collect (taxes)
U+638B.0	fUTF8	掋
U+638B.0	fCNS11643	3-357D
U+638B.0	fGB	5-4C56
U+638B.0	fR/S	  64.8
U+638B.0	fTotalStrokes	 11
U+638B.0	fCangjie	QIHM
U+638B.0	fFourCorner	50014
U+638B.0	fCantonese	dai2
U+638B.0	fMandarin	di3
U+638C.0	fUTF8	掌
U+638C.0	fCNS11643	1-5F3F
U+638C.0	fBig5	B478
U+638C.0	fGB	0-5546
U+638C.0	fR/S	 64.8
U+638C.0	fTotalStrokes	 12
U+638C.0	fCangjie	FBRQ
U+638C.0	fFourCorner	90502
U+638C.0	fHakka	  zhong3;  zong3; cong3; cang1
U+638C.0	fCantonese	zoeng2
U+638C.0	fMandarin	zhang3
U+638C.0	fEnglish	[1] palm of the hand [2] slap with one's hand; smack [3] have charge of; supervise; control [4] stand; bear [5] a Chinese family name
U+638C.1	fUTF8	掌
U+638C.1	fCNS11643	1-5F3F
U+638C.1	fBig5	B478
U+638C.1	fGB	0-5546
U+638C.1	fR/S	 64.8
U+638C.1	fTotalStrokes	 12
U+638C.1	fCangjie	FBRQ
U+638C.1	fFourCorner	90502
U+638C.1	fMandarin	zhang5
U+638D.0	fUTF8	掍
U+638D.0	fCNS11643	2-3567
U+638D.0	fBig5	D5A5
U+638D.0	fGB	3-4A52
U+638D.0	fR/S	 64.8
U+638D.0	fTotalStrokes	 11
U+638D.0	fCangjie	QAPP
U+638D.0	fFourCorner	56011
U+638D.0	fHakka	 gun3; gwun3
U+638D.0	fCantonese	wan6; gwan2 (also); gun2
U+638D.0	fMandarin	gun3; hun4; hun3
U+638D.0	fEnglish	[1] {1] same; similar [2] inlay [3] root up
U+638E.0	fUTF8	掎
U+638E.0	fCNS11643	2-3555
U+638E.0	fBig5	D571
U+638E.0	fGB	0-5E61
U+638E.0	fR/S	 64.8
U+638E.0	fTotalStrokes	 11
U+638E.0	fCangjie	QKMR
U+638E.0	fFourCorner	54021
U+638E.0	fHakka	 ji3; ri3
U+638E.0	fCantonese	gei2; gei1 (also)
U+638E.0	fMandarin	ji3; ge1; ji1; yi3
U+638E.0	fEnglish	 drag aside; pull; drag
U+638F.0	fUTF8	掏
U+638F.0	fCNS11643	1-5A6B
U+638F.0	fBig5	B1C7
U+638F.0	fGB	0-4C4D
U+638F.0	fR/S	 64.8
U+638F.0	fTotalStrokes	 11
U+638F.0	fCangjie	QPOU
U+638F.0	fFourCorner	57020
U+638F.0	fHakka	 tau2; jau2; rau2; tau1
U+638F.0	fCantonese	tou4; tou1
U+638F.0	fMandarin	tao1; tao2
U+638F.0	fEnglish	[1] take out; pull out [2] dig; scoop out [3] steal from somebody's pocket
U+6390.0	fUTF8	掐
U+6390.0	fCNS11643	2-3558
U+6390.0	fBig5	D574
U+6390.0	fGB	0-467E
U+6390.0	fR/S	 64.8
U+6390.0	fTotalStrokes	 11
U+6390.0	fCangjie	QNHX
U+6390.0	fFourCorner	57077
U+6390.0	fHakka	 kap7; kang5; gip7; ngiap7; kat8
U+6390.0	fCantonese	haap3
U+6390.0	fMandarin	qia1
U+6390.0	fEnglish	[1] dig the nail into [2] cut with fingernails; nip; pinch; give a pinch [3] cut with fingernails; nip; pinch; give a pinch [4] hold; gather with the hand; grasp; clutch
U+6391.0	fUTF8	掑
U+6391.0	fCNS11643	2-3566
U+6391.0	fBig5	D5A4
U+6391.0	fGB	E-2B4E
U+6391.0	fR/S	 64.8
U+6391.0	fTotalStrokes	 11
U+6391.0	fCangjie	QTMC
U+6391.0	fFourCorner	54081
U+6391.0	fCantonese	kei4
U+6391.0	fMandarin	qi2
U+6392.0	fUTF8	排
U+6392.0	fCNS11643	1-5A6A
U+6392.0	fBig5	B1C6
U+6392.0	fGB	0-4545
U+6392.0	fR/S	 64.8
U+6392.0	fTotalStrokes	 11
U+6392.0	fCangjie	QLMY
U+6392.0	fFourCorner	51011
U+6392.0	fHakka	  pai2; pai3
U+6392.0	fCantonese	paai4; paai2 (also); bik1
U+6392.0	fMandarin	pai2; pai3; bei4; bai4
U+6392.0	fEnglish	[1] row; rank; line [2] arrange; put in order; fall in line [3] platoon (military) [4] clear out; drain away; discharge [5] reject; expel; exclude [6] rehearse [7] raft [8] push
U+6393.0	fUTF8	掓
U+6393.0	fCNS11643	3-3624
U+6393.0	fGB	5-4C4C
U+6393.0	fR/S	  64.8
U+6393.0	fTotalStrokes	 11
U+6393.0	fCangjie	QYFE
U+6393.0	fFourCorner	57040
U+6393.0	fCantonese	suk1
U+6393.0	fMandarin	shu2
U+6394.0	fUTF8	掔
U+6394.0	fCNS11643	2-3B75
U+6394.0	fBig5	D952
U+6394.0	fGB	3-4949
U+6394.0	fR/S	 64.8; 64.9
U+6394.0	fTotalStrokes	 12
U+6394.0	fCangjie	SEQ
U+6394.0	fFourCorner	77502
U+6394.0	fHakka	 kien1; kian1; ken1
U+6394.0	fCantonese	haan1; hin1 (also); hang1
U+6394.0	fMandarin	qian1; wan4; qian4
U+6394.0	fEnglish	[1] thick; firm; substantial [2] drag along; pull [3] lead
U+6395.0	fUTF8	掕
U+6395.0	fCNS11643	4-3177
U+6395.0	fGB	3-4A4A
U+6395.0	fR/S	  64.8
U+6395.0	fTotalStrokes	 11
U+6395.0	fCangjie	QGCE
U+6395.0	fFourCorner	54040
U+6395.0	fHakka	 lin2
U+6395.0	fCantonese	ling4
U+6395.0	fMandarin	ling4; ling2
U+6396.0	fUTF8	掖
U+6396.0	fCNS11643	1-5A57
U+6396.0	fBig5	B1B3
U+6396.0	fGB	0-5234
U+6396.0	fR/S	 64.8
U+6396.0	fTotalStrokes	 11
U+6396.0	fCangjie	QYOK
U+6396.0	fFourCorner	50047
U+6396.0	fHakka	  jit8; rit8; ze5; je5
U+6396.0	fCantonese	jik6; jip3; jat6
U+6396.0	fMandarin	ye4; yi4; ye3
U+6396.0	fEnglish	[1] support another; extend a helping hand; promote [2] armpits [3] side by side; side [4] side apartments in the palace
U+6396.1	fUTF8	掖
U+6396.1	fCNS11643	1-5A57
U+6396.1	fBig5	B1B3
U+6396.1	fGB	0-5234
U+6396.1	fR/S	 64.8
U+6396.1	fTotalStrokes	 11
U+6396.1	fCangjie	QYOK
U+6396.1	fFourCorner	50047
U+6396.1	fMandarin	ye1
U+6396.1	fEnglish	[1] conceal; tuck away; hide [2] fold; roll up (part of one's clothing)
U+6397.0	fUTF8	掗
U+6397.0	fCNS11643	2-3553
U+6397.0	fBig5	D56F
U+6397.0	fGB	3-4A25
U+6397.0	fR/S	 64.8
U+6397.0	fTotalStrokes	 11
U+6397.0	fCangjie	QMLM
U+6397.0	fFourCorner	51017
U+6397.0	fHakka	 a3; a1
U+6397.0	fCantonese	aa3; ngaa6 (also); ngaa3 (also); aa6 (also); aa2; ngaa2
U+6397.0	fMandarin	ya4; ya3; e3
U+6397.0	fEnglish	 take hold give forcefully
U+6398.0	fUTF8	掘
U+6398.0	fCNS11643	1-5A5C
U+6398.0	fBig5	B1B8
U+6398.0	fGB	0-3E72
U+6398.0	fR/S	 64.8
U+6398.0	fTotalStrokes	 11
U+6398.0	fCangjie	QSUU
U+6398.0	fFourCorner	57072
U+6398.0	fHakka	  kiut7;  kut8; kut7; kiat7; kwut7; kwut8
U+6398.0	fCantonese	gwat6
U+6398.0	fMandarin	jue2; hu2; zhuo2
U+6398.0	fEnglish	 dig; excavate; make a hole or cave
U+6399.0	fUTF8	掙
U+6399.0	fCNS11643	1-5A67
U+6399.0	fBig5	B1C3
U+6399.0	fGB	1-5575
U+6399.0	fR/S	 64.8
U+6399.0	fTotalStrokes	 11
U+6399.0	fCangjie	QBSD
U+6399.0	fFourCorner	52057
U+6399.0	fHakka	  zang1; zen1; {nang5}; zang5; zen5; nang5
U+6399.0	fCantonese	zang1; zaang1 (also); zaang6; zang6
U+6399.0	fMandarin	zheng1
U+6399.0	fEnglish	[1] strive; endeavor; make efforts [2] get free from
U+6399.1	fUTF8	掙
U+6399.1	fCNS11643	1-5A67
U+6399.1	fBig5	B1C3
U+6399.1	fGB	1-5575
U+6399.1	fR/S	 64.8
U+6399.1	fTotalStrokes	 11
U+6399.1	fCangjie	QBSD
U+6399.1	fFourCorner	52057
U+6399.1	fMandarin	zheng4
U+6399.1	fEnglish	[1] struggle (for one's life) [2] earn (money, etc.)
U+639A.0	fUTF8	掚
U+639A.0	fCNS11643	4-3178
U+639A.0	fGB	5-4C34
U+639A.0	fR/S	  64.8
U+639A.0	fTotalStrokes	 11
U+639A.0	fCangjie	QMLB
U+639A.0	fFourCorner	51020
U+639A.0	fCantonese	loeng5
U+639A.0	fMandarin	liang3
U+639B.0	fUTF8	掛
U+639B.0	fCNS11643	1-5A62
U+639B.0	fBig5	B1BE
U+639B.0	fGB	E-2B4F
U+639B.0	fR/S	 64.8
U+639B.0	fTotalStrokes	 11
U+639B.0	fCangjie	QGGY
U+639B.0	fFourCorner	53000
U+639B.0	fHakka	  gwa5; ga5; kua5
U+639B.0	fCantonese	gwaa3; kwaa3 (also)
U+639B.0	fMandarin	gua4
U+639B.0	fEnglish	[1] hang up; suspend [2] ring off [3] worry; think of; anxious [4] with one's name registered or listed; recorded; register [5] hitch; get caught
U+639C.0	fUTF8	掜
U+639C.0	fCNS11643	2-355C
U+639C.0	fBig5	D578
U+639C.0	fGB	3-4A54
U+639C.0	fR/S	 64.8
U+639C.0	fTotalStrokes	 11
U+639C.0	fCangjie	QHXU
U+639C.0	fFourCorner	57017
U+639C.0	fCantonese	ngai5; nip6; ai5 (also); lip6 (also)
U+639C.0	fMandarin	yi3; nie4; ni3; yi4
U+639D.0	fUTF8	掝
U+639D.0	fCNS11643	2-3552
U+639D.0	fBig5	D56E
U+639D.0	fGB	3-4A4D
U+639D.0	fR/S	 64.8
U+639D.0	fTotalStrokes	 11
U+639D.0	fCangjie	QIRM
U+639D.0	fFourCorner	53050
U+639D.0	fHakka	 fet8; wet7
U+639D.0	fCantonese	waak6; kwik1; wak6
U+639D.0	fMandarin	huo4; xu4
U+639D.0	fEnglish	deluded, beguiled
U+639E.0	fUTF8	掞
U+639E.0	fCNS11643	2-3550
U+639E.0	fBig5	D56C
U+639E.0	fGB	3-4A59
U+639E.0	fR/S	 64.8
U+639E.0	fTotalStrokes	 11
U+639E.0	fCangjie	QFF
U+639E.0	fFourCorner	59089
U+639E.0	fHakka	  jam3;  jam5; ram3; ram6
U+639E.0	fCantonese	sim3; jim5; jim6
U+639E.0	fMandarin	shan4; yan4; yan3
U+639E.0	fEnglish	easy; smooth; comfortable; suave
U+639F.0	fUTF8	掟
U+639F.0	fCNS11643	2-3562
U+639F.0	fBig5	D57E
U+639F.0	fGB	3-4A5A
U+639F.0	fR/S	 64.8
U+639F.0	fTotalStrokes	 11
U+639F.0	fCangjie	QJMO
U+639F.0	fFourCorner	53081
U+639F.0	fHakka	 zen3; tin5; tin6
U+639F.0	fCantonese	deng3; deng6; ting6
U+639F.0	fMandarin	zheng3; ding4; ding1
U+639F.0	fEnglish	[1] whirl about; smash [2] pelt; throw
U+63A0.0	fUTF8	掠
U+63A0.0	fCNS11643	1-5A54
U+63A0.0	fBig5	B1B0
U+63A0.0	fGB	0-4253
U+63A0.0	fR/S	 64.8
U+63A0.0	fTotalStrokes	 11
U+63A0.0	fCangjie	QYRF
U+63A0.0	fFourCorner	50096
U+63A0.0	fHakka	  liok8
U+63A0.0	fCantonese	loek6
U+63A0.0	fMandarin	le4; luue4; liang4; le3; lun4
U+63A0.0	fEnglish	[1] take by force; rob; plunder; pillage [2] brush; pass lightly on the side; sweep past; skim over [3] whip; flog [4] long stroke to the right in Chinese writing
U+63A1.0	fUTF8	採
U+63A1.0	fCNS11643	1-5A68
U+63A1.0	fBig5	B1C4
U+63A1.0	fGB	E-2B50
U+63A1.0	fR/S	 64.8
U+63A1.0	fTotalStrokes	 11
U+63A1.0	fCangjie	QBD
U+63A1.0	fFourCorner	52094
U+63A1.0	fHakka	  cai3
U+63A1.0	fCantonese	coi2
U+63A1.0	fMandarin	cai3; cai4
U+63A1.0	fEnglish	[1] pluck (flowers, etc.); gather; collect; extract [2] pick; select; adopt [3] drag [4] beckon; take notice of
U+63A2.0	fUTF8	探
U+63A2.0	fCNS11643	1-5A58
U+63A2.0	fBig5	B1B4
U+63A2.0	fGB	0-4C3D
U+63A2.0	fR/S	 64.8
U+63A2.0	fTotalStrokes	 11
U+63A2.0	fCangjie	QBCD
U+63A2.0	fFourCorner	57094
U+63A2.0	fHakka	  tam5;  tam1; tam3
U+63A2.0	fCantonese	taam3; taam1
U+63A2.0	fMandarin	tan4; chan2
U+63A2.0	fEnglish	[1] find; locate; search; prospect; feel (in a pocket or bag) [2] watch; spy; spy; detective; investigate; secret agent [3] try; venture; tempt [4] stick out [5] explore [6] visit; inquiry about
U+63A2.1	fUTF8	探
U+63A2.1	fCNS11643	1-5A58
U+63A2.1	fBig5	B1B4
U+63A2.1	fGB	0-4C3D
U+63A2.1	fR/S	 64.8
U+63A2.1	fTotalStrokes	 11
U+63A2.1	fCangjie	QBCD
U+63A2.1	fFourCorner	57094
U+63A2.1	fMandarin	tan1
U+63A2.1	fEnglish	 try; attempt; test
U+63A2.2	fUTF8	探
U+63A2.2	fCNS11643	1-5A58
U+63A2.2	fBig5	B1B4
U+63A2.2	fGB	0-4C3D
U+63A2.2	fR/S	 64.8
U+63A2.2	fTotalStrokes	 11
U+63A2.2	fCangjie	QBCD
U+63A2.2	fFourCorner	57094
U+63A3.0	fUTF8	掣
U+63A3.0	fCNS11643	1-5F3E
U+63A3.0	fBig5	B477
U+63A3.0	fGB	0-3338
U+63A3.0	fR/S	 64.8
U+63A3.0	fTotalStrokes	 12
U+63A3.0	fCangjie	HNQ
U+63A3.0	fFourCorner	22502
U+63A3.0	fHakka	 chat7; zhi5; cat7; zai5; zih5; cet7; chet8
U+63A3.0	fCantonese	zai3; cit3
U+63A3.0	fMandarin	che4; chi4
U+63A3.0	fEnglish	[1] drag; pull; draw [2] hinder [3] snatch away
U+63A4.0	fUTF8	掤
U+63A4.0	fCNS11643	2-3560
U+63A4.0	fBig5	D57C
U+63A4.0	fGB	3-4A55
U+63A4.0	fR/S	 64.8
U+63A4.0	fTotalStrokes	 11
U+63A4.0	fCangjie	QBB
U+63A4.0	fFourCorner	57020
U+63A4.0	fHakka	 ben1
U+63A4.0	fCantonese	bing1; bang1; maang1; mang1; mang3
U+63A4.0	fMandarin	bing1
U+63A4.0	fEnglish	quiver
U+63A5.0	fUTF8	接
U+63A5.0	fCNS11643	1-5A59
U+63A5.0	fBig5	B1B5
U+63A5.0	fGB	0-3D53
U+63A5.0	fR/S	 64.8
U+63A5.0	fTotalStrokes	 11
U+63A5.0	fCangjie	QYTV
U+63A5.0	fFourCorner	50044
U+63A5.0	fHakka	  ziap7; ziak7
U+63A5.0	fCantonese	zip3; kam4; kwaai5; daam1; daam3; daam2 (also)
U+63A5.0	fMandarin	jie1; cha2; sha4; xie2
U+63A5.0	fEnglish	[1] receive; accept; take with the hand [2] welcome; meet [3] join; connect [4] graft [5] come close to; make contact with [6] succeed to
U+63A6.0	fUTF8	掦
U+63A6.0	fCNS11643	4-3223
U+63A6.0	fGB	5-4C4E
U+63A6.0	fR/S	  64.8
U+63A6.0	fTotalStrokes	 11
U+63A6.0	fCangjie	QAPH
U+63A6.0	fFourCorner	56020
U+63A6.0	fCantonese	tik1; dik1
U+63A6.0	fMandarin	ti4
U+63A6.0	fEnglish	to take up; to put by
U+63A7.0	fUTF8	控
U+63A7.0	fCNS11643	1-5A55
U+63A7.0	fBig5	B1B1
U+63A7.0	fGB	0-3F58
U+63A7.0	fR/S	 64.8
U+63A7.0	fTotalStrokes	 11
U+63A7.0	fCangjie	QJCM
U+63A7.0	fFourCorner	53011
U+63A7.0	fHakka	  kung5; kung3; kiong1; kiong5
U+63A7.0	fCantonese	hung3
U+63A7.0	fMandarin	kong4; qiang1; kong1; qiang3
U+63A7.0	fEnglish	[1] accuse; charge; sue [2] control [3] draw (a bow) [4] throw; hit
U+63A8.0	fUTF8	推
U+63A8.0	fCNS11643	1-5A64
U+63A8.0	fBig5	B1C0
U+63A8.0	fGB	0-4D46
U+63A8.0	fR/S	 64.8
U+63A8.0	fTotalStrokes	 11
U+63A8.0	fCangjie	QOG
U+63A8.0	fFourCorner	50014
U+63A8.0	fHakka	  tui1;  cui1;  toi1; kok7; dui3; tui2
U+63A8.0	fCantonese	teoi1; ceoi1
U+63A8.0	fMandarin	tui1; chui1
U+63A8.0	fEnglish	[1] push; shove [2] look into; find out; ponder; infer; deduce [3] shirk; shift (responsibility, etc.); refuse [4] elect; recommend; praise; esteem [5] move along; change in succession (as seasons) [6] to extend; enlarge
U+63A9.0	fUTF8	掩
U+63A9.0	fCNS11643	1-5A5F
U+63A9.0	fBig5	B1BB
U+63A9.0	fGB	0-515A
U+63A9.0	fR/S	 64.8
U+63A9.0	fTotalStrokes	 11
U+63A9.0	fCangjie	QKLU
U+63A9.0	fFourCorner	54016
U+63A9.0	fHakka	  jam3; am1; ram3; jim5
U+63A9.0	fCantonese	jim2; am2
U+63A9.0	fMandarin	yan3; yan4; ye4
U+63A9.0	fEnglish	[1] cover; conceal; hide; cover up [2] shut; close [3] mount a surprise attack; ambush; take or catch by surprise
U+63AA.0	fUTF8	措
U+63AA.0	fCNS11643	1-5A5D
U+63AA.0	fBig5	B1B9
U+63AA.0	fGB	0-346B
U+63AA.0	fR/S	 64.8
U+63AA.0	fTotalStrokes	 11
U+63AA.0	fCangjie	QTA
U+63AA.0	fFourCorner	54061
U+63AA.0	fHakka	  cu5; co5; cih5; ze5; chu5
U+63AA.0	fCantonese	cou3; zaak3
U+63AA.0	fMandarin	cuo4; ze2; cu4; qi4; zhai3
U+63AA.0	fEnglish	[1] place [2] collect; arrange; manage; handle [3] abandon; renounce [4] make preparation for an undertaking; make plans
U+63AB.0	fUTF8	掫
U+63AB.0	fCNS11643	2-3554
U+63AB.0	fBig5	D570
U+63AB.0	fGB	3-4A4B
U+63AB.0	fR/S	 64.8
U+63AB.0	fTotalStrokes	 11
U+63AB.0	fCangjie	QSJE
U+63AB.0	fFourCorner	51047
U+63AB.0	fHakka	  zeu1
U+63AB.0	fCantonese	zau1
U+63AB.0	fMandarin	zou1; chou1; cou3; ju1; zhou1; zhou3
U+63AB.0	fEnglish	 be on night watch
U+63AC.0	fUTF8	掬
U+63AC.0	fCNS11643	1-5A69
U+63AC.0	fBig5	B1C5
U+63AC.0	fGB	0-5E64
U+63AC.0	fR/S	 64.8
U+63AC.0	fTotalStrokes	 11
U+63AC.0	fCangjie	QPFD
U+63AC.0	fFourCorner	57020
U+63AC.0	fHakka	  kiuk7
U+63AC.0	fCantonese	guk1
U+63AC.0	fMandarin	ju1; ju2
U+63AC.0	fEnglish	[1] hold in both hands [2] a double handful of (noun classifier)
U+63AD.0	fUTF8	掭
U+63AD.0	fCNS11643	2-3551
U+63AD.0	fBig5	D56D
U+63AD.0	fGB	0-5E5D
U+63AD.0	fR/S	 64.8
U+63AD.0	fTotalStrokes	 11
U+63AD.0	fCangjie	QHKP
U+63AD.0	fFourCorner	52033
U+63AD.0	fHakka	 tiam3; tiam5
U+63AD.0	fCantonese	tim3 (also); tim5
U+63AD.0	fMandarin	tian4
U+63AD.0	fEnglish	 manipulate a pricker for a lamp-wick
U+63AE.0	fUTF8	掮
U+63AE.0	fCNS11643	2-355E
U+63AE.0	fBig5	D57A
U+63AE.0	fGB	0-5E67
U+63AE.0	fR/S	 64.8
U+63AE.0	fTotalStrokes	 11
U+63AE.0	fCangjie	QHSB
U+63AE.0	fFourCorner	53027
U+63AE.0	fHakka	 kai1; ken2
U+63AE.0	fCantonese	kin4
U+63AE.0	fMandarin	qian2
U+63AE.0	fEnglish	[1] bear a load on the shoulder [2] move things on the shoulder
U+63AF.0	fUTF8	掯
U+63AF.0	fCNS11643	2-355A
U+63AF.0	fBig5	D576
U+63AF.0	fGB	3-4A50
U+63AF.0	fR/S	 64.8
U+63AF.0	fTotalStrokes	 11
U+63AF.0	fCangjie	QYMB
U+63AF.0	fFourCorner	51027
U+63AF.0	fHakka	  ken5; ken3
U+63AF.0	fCantonese	kang3
U+63AF.0	fMandarin	ken4
U+63AF.0	fEnglish	oppress; extort; take by force
U+63B0.0	fUTF8	掰
U+63B0.0	fCNS11643	2-3B77
U+63B0.0	fBig5	D954
U+63B0.0	fGB	0-6A7E
U+63B0.0	fR/S	 64.8
U+63B0.0	fTotalStrokes	 12; 11
U+63B0.0	fCangjie	QCHQ
U+63B0.0	fFourCorner	22550
U+63B0.0	fHakka	 mak7
U+63B0.0	fCantonese	baai1 (also); maak3; baai2; mak3; bai1
U+63B0.0	fMandarin	bai1; zhai4
U+63B0.0	fEnglish	 pull apart with hands
U+63B1.0	fUTF8	掱
U+63B1.0	fCNS11643	2-3B76
U+63B1.0	fBig5	D953
U+63B1.0	fGB	3-494D
U+63B1.0	fR/S	 64.8
U+63B1.0	fTotalStrokes	 12
U+63B1.0	fCangjie	QQQ
U+63B1.0	fFourCorner	20552
U+63B1.0	fCantonese	paa4
U+63B1.0	fMandarin	shou3; pa2
U+63B1.0	fEnglish	pickpocket
U+63B2.0	fUTF8	掲
U+63B2.0	fCNS11643	3-362D
U+63B2.0	fGB	E-2B51
U+63B2.0	fR/S	 64.9; 64.8
U+63B2.0	fTotalStrokes	 11; 12
U+63B2.0	fCangjie	QAPP
U+63B2.0	fFourCorner	56020
U+63B2.0	fCantonese	kit3
U+63B2.0	fMandarin	jie1; qi4
U+63B2.0	fEnglish	to set (a table); to install, equip; to lay (a foundation)
U+63B3.0	fUTF8	掳
U+63B3.0	fGB	0-4230
U+63B3.0	fR/S	 64.8
U+63B3.0	fTotalStrokes	 11
U+63B3.0	fCangjie	XQYPS
U+63B3.0	fHakka	 lu3; lu1
U+63B3.0	fCantonese	lou5
U+63B3.0	fMandarin	lu3
U+63B3.0	fEnglish	simplified form of &#x64C4;
U+63B4.0	fUTF8	掴
U+63B4.0	fGB	0-5E62
U+63B4.0	fR/S	 64.7; 64.8
U+63B4.0	fTotalStrokes	 11
U+63B4.0	fCangjie	QWMI
U+63B4.0	fHakka	 kwet8
U+63B4.0	fCantonese	gwaak3; gwok3 (also); gok3 (also)
U+63B4.0	fMandarin	guo2
U+63B4.0	fEnglish	simplified form of &#x6451;
U+63B5.0	fUTF8	掵
U+63B5.0	fGB	E-2B52
U+63B5.0	fR/S	 64.8
U+63B5.0	fTotalStrokes	 11
U+63B5.0	fCangjie	QOML
U+63B5.0	fCantonese	ming6
U+63B6.0	fUTF8	掶
U+63B6.0	fCNS11643	3-647E
U+63B6.0	fGB	E-2B53
U+63B6.0	fR/S	 64.8
U+63B6.0	fTotalStrokes	 11
U+63B6.0	fCangjie	QILO
U+63B6.0	fFourCorner	5308
U+63B6.0	fCantonese	zit6
U+63B7.0	fUTF8	掷
U+63B7.0	fGB	0-5640
U+63B7.0	fR/S	 64.8
U+63B7.0	fTotalStrokes	 11
U+63B7.0	fCangjie	XQTKL
U+63B7.0	fHakka	 chit7
U+63B7.0	fCantonese	zaak6
U+63B7.0	fMandarin	zhi2; zhi4; zhi1
U+63B7.0	fEnglish	simplified form of &#x64F2;
U+63B8.0	fUTF8	掸
U+63B8.0	fGB	0-3527
U+63B8.0	fR/S	 64.8
U+63B8.0	fTotalStrokes	 11
U+63B8.0	fCangjie	QCWJ
U+63B8.0	fHakka	 tan3; tan5
U+63B8.0	fCantonese	daan6 (also); daan5; sin6
U+63B8.0	fMandarin	dan3; shan3; chan2; shan4; dan4; tan2; zhan3
U+63B8.0	fEnglish	simplified form of &#x64A3;
U+63B9.0	fUTF8	掹
U+63B9.0	fGB	1-7C3C
U+63B9.0	fR/S	 64.8
U+63B9.0	fTotalStrokes	 11
U+63B9.0	fCangjie	QNDT
U+63B9.0	fFourCorner	5701
U+63B9.0	fHakka	  bang1;  man3
U+63B9.0	fCantonese	mang1; mang3; maang1
U+63B9.0	fEnglish	 pluck; pull
U+63BA.0	fUTF8	掺
U+63BA.0	fCNS11643	3-362E
U+63BA.0	fGB	0-3274
U+63BA.0	fR/S	 64.8
U+63BA.0	fTotalStrokes	 11
U+63BA.0	fCangjie	QIKH
U+63BA.0	fFourCorner	53020
U+63BA.0	fHakka	 cam1
U+63BA.0	fCantonese	caam1; saam1; cam1; cam3; caam4; saam2
U+63BA.0	fMandarin	shan3; can4; xian1; shan1; chan1; can1; san1; san4
U+63BA.0	fEnglish	simplified form of &#x647B;
U+63BB.0	fUTF8	掻
U+63BB.0	fGB	E-2B54
U+63BB.0	fR/S	 64.8
U+63BB.0	fTotalStrokes	 11
U+63BB.0	fCangjie	QELI
U+63BB.0	fCantonese	sou1
U+63BB.0	fMandarin	sao1; zhao3
U+63BB.0	fEnglish	scratch lightly
U+63BC.0	fUTF8	掼
U+63BC.0	fGB	0-5E68
U+63BC.0	fR/S	 64.8
U+63BC.0	fTotalStrokes	 11
U+63BC.0	fCangjie	QWJO
U+63BC.0	fHakka	 gwon5
U+63BC.0	fCantonese	gwaan3
U+63BC.0	fMandarin	guan4
U+63BC.0	fEnglish	simplified form of &#x645C;
U+63BD.0	fUTF8	掽
U+63BD.0	fCNS11643	2-354F
U+63BD.0	fBig5	D56B
U+63BD.0	fGB	E-2B55
U+63BD.0	fR/S	 64.8; 64.9
U+63BD.0	fTotalStrokes	 11
U+63BD.0	fCangjie	QTTC
U+63BD.0	fFourCorner	58017
U+63BD.0	fHakka	  pang5; pong1
U+63BD.0	fCantonese	pung3
U+63BD.0	fMandarin	peng4
U+63BD.0	fEnglish	 collide meet unexpectedly
U+63BE.0	fUTF8	掾
U+63BE.0	fCNS11643	2-3C29
U+63BE.0	fBig5	D964
U+63BE.0	fGB	0-5E72
U+63BE.0	fR/S	 64.9
U+63BE.0	fTotalStrokes	 12
U+63BE.0	fCangjie	QVNO
U+63BE.0	fFourCorner	57032
U+63BE.0	fHakka	 jen2; ren2; jen5; jan5; jeu5
U+63BE.0	fCantonese	jyun6
U+63BE.0	fMandarin	yuan4; chuan2; yung4
U+63BE.0	fEnglish	a general designation of public officials in ancient China
U+63BF.0	fUTF8	掿
U+63BF.0	fCNS11643	3-3B71
U+63BF.0	fGB	5-4C46
U+63BF.0	fR/S	  64.9
U+63BF.0	fTotalStrokes	 12
U+63BF.0	fCangjie	QTKR
U+63BF.0	fFourCorner	54060
U+63BF.0	fCantonese	nok6; lok6 (also); nik1; lik1
U+63BF.0	fMandarin	nuo4
U+63C0.0	fUTF8	揀
U+63C0.0	fCNS11643	1-5F41
U+63C0.0	fBig5	B47A
U+63C0.0	fGB	1-3C70
U+63C0.0	fR/S	 64.9
U+63C0.0	fTotalStrokes	 12
U+63C0.0	fCangjie	QDWF
U+63C0.0	fFourCorner	55096
U+63C0.0	fHakka	  gan3; gien3; kian3; gian3
U+63C0.0	fCantonese	gaan2
U+63C0.0	fMandarin	jian3; lian4
U+63C0.0	fEnglish	[1] choose; select; pick [2] pick up (something another has left behind, etc.)
U+63C1.0	fUTF8	揁
U+63C1.0	fCNS11643	3-3B70
U+63C1.0	fGB	5-4B7C
U+63C1.0	fR/S	 64.9
U+63C1.0	fTotalStrokes	 12
U+63C1.0	fCangjie	QYBC
U+63C1.0	fFourCorner	51080
U+63C1.0	fCantonese	zing1
U+63C1.0	fMandarin	zhen1; zheng1
U+63C1.0	fEnglish	sound of setting up a koto
U+63C2.0	fUTF8	揂
U+63C2.0	fCNS11643	2-3C2F
U+63C2.0	fBig5	D96A
U+63C2.0	fGB	5-4C71
U+63C2.0	fR/S	 64.9
U+63C2.0	fTotalStrokes	 12
U+63C2.0	fCangjie	QTCW
U+63C2.0	fFourCorner	58060
U+63C2.0	fHakka	 ziu3
U+63C2.0	fCantonese	zau1; jau4
U+63C2.0	fMandarin	jiu1; qiu2; you2
U+63C3.0	fUTF8	揃
U+63C3.0	fCNS11643	2-3B7C
U+63C3.0	fBig5	D959
U+63C3.0	fGB	3-4A72
U+63C3.0	fR/S	 64.9
U+63C3.0	fTotalStrokes	 12
U+63C3.0	fCangjie	QTBN
U+63C3.0	fFourCorner	58021
U+63C3.0	fHakka	  zien5
U+63C3.0	fCantonese	zin2; zin1 (also); zin3
U+63C3.0	fMandarin	jian1; jian3; qian1
U+63C3.0	fEnglish	shear
U+63C4.0	fUTF8	揄
U+63C4.0	fCNS11643	2-3C2C
U+63C4.0	fBig5	D967
U+63C4.0	fGB	0-5E6D
U+63C4.0	fR/S	 64.9
U+63C4.0	fTotalStrokes	 12
U+63C4.0	fCangjie	QOMN
U+63C4.0	fFourCorner	58021
U+63C4.0	fHakka	 ji2; ri2; i2
U+63C4.0	fCantonese	jyu4; jau4; jiu5
U+63C4.0	fMandarin	yu2; you2; chou1; shu1; tou2; yao2
U+63C4.0	fEnglish	[1] draw out; scoop out (grain from a mortar) [2] praise; show merits of [3] hang
U+63C5.0	fUTF8	揅
U+63C5.0	fCNS11643	2-427D
U+63C5.0	fBig5	DD77
U+63C5.0	fGB	5-4B4B
U+63C5.0	fR/S	 64.11; 64.9
U+63C5.0	fTotalStrokes	 15
U+63C5.0	fCangjie	MTQ
U+63C5.0	fFourCorner	11502
U+63C5.0	fHakka	 ngan1; ngion1
U+63C5.0	fCantonese	jin4; ngaan4
U+63C5.0	fMandarin	yan2
U+63C5.0	fEnglish	[1] grind; powder [2] study thoroughly
U+63C6.0	fUTF8	揆
U+63C6.0	fCNS11643	1-5F44
U+63C6.0	fBig5	B47D
U+63C6.0	fGB	0-5E71
U+63C6.0	fR/S	 64.9
U+63C6.0	fTotalStrokes	 12
U+63C6.0	fCangjie	QNOK
U+63C6.0	fFourCorner	52034
U+63C6.0	fHakka	  kui2; kwi2; k'wui2
U+63C6.0	fCantonese	kwai4; kwai5 (also)
U+63C6.0	fMandarin	kui2; kui3
U+63C6.0	fEnglish	[1] survey and weigh; consider; investigate; estimate [2] prime minister; premier [3] important affair
U+63C7.0	fUTF8	揇
U+63C7.0	fCNS11643	2-3C30
U+63C7.0	fBig5	D96B
U+63C7.0	fGB	5-4C5C
U+63C7.0	fR/S	 64.9
U+63C7.0	fTotalStrokes	 12
U+63C7.0	fCangjie	QJBJ
U+63C7.0	fFourCorner	54027
U+63C7.0	fHakka	 nam3; lam3; niam3
U+63C7.0	fCantonese	naam5; laam5 (also); naam3; laam3 (also); naam2
U+63C7.0	fMandarin	nan3
U+63C7.0	fEnglish	 grasp with the hand
U+63C8.0	fUTF8	揈
U+63C8.0	fCNS11643	2-3C33
U+63C8.0	fBig5	D96E
U+63C8.0	fGB	5-4C6A
U+63C8.0	fR/S	 64.9
U+63C8.0	fTotalStrokes	 12
U+63C8.0	fCangjie	QPYR
U+63C8.0	fFourCorner	57020
U+63C8.0	fHakka	 fen2; wang1; fen1; kiuk7
U+63C8.0	fCantonese	fang4; gwang1
U+63C8.0	fMandarin	hong1; ju2; xuan4
U+63C9.0	fUTF8	揉
U+63C9.0	fCNS11643	1-5F43
U+63C9.0	fBig5	B47C
U+63C9.0	fGB	0-4860
U+63C9.0	fR/S	 64.9
U+63C9.0	fTotalStrokes	 12
U+63C9.0	fCangjie	QNHD
U+63C9.0	fFourCorner	57094
U+63C9.0	fHakka	  ngiu3; ju2;  ngio1; jiu2; riu2
U+63C9.0	fCantonese	jau4; jau6
U+63C9.0	fMandarin	rou2; niu3; rao3; rou3; rou4
U+63C9.0	fEnglish	[1] rub; knead [2] crumple by hand [3] massage [4] subdue; make smooth or peaceful [5] mixed-up; confused
U+63CA.0	fUTF8	揊
U+63CA.0	fCNS11643	2-3C21
U+63CA.0	fBig5	D95C
U+63CA.0	fGB	5-4C5D
U+63CA.0	fR/S	 64.9
U+63CA.0	fTotalStrokes	 12
U+63CA.0	fCangjie	QMRW
U+63CA.0	fFourCorner	51066
U+63CA.0	fCantonese	bik1
U+63CA.0	fMandarin	pi4; bi4; che4
U+63CB.0	fUTF8	揋
U+63CB.0	fCNS11643	2-3C32
U+63CB.0	fBig5	D96D
U+63CB.0	fGB	5-4C61
U+63CB.0	fR/S	 64.9
U+63CB.0	fTotalStrokes	 12
U+63CB.0	fCangjie	QWMV
U+63CB.0	fFourCorner	56032
U+63CB.0	fCantonese	wui1
U+63CB.0	fMandarin	wei1
U+63CC.0	fUTF8	揌
U+63CC.0	fCNS11643	2-3C31
U+63CC.0	fBig5	D96C
U+63CC.0	fGB	5-4C62
U+63CC.0	fR/S	 64.9
U+63CC.0	fTotalStrokes	 12
U+63CC.0	fCangjie	QWP
U+63CC.0	fFourCorner	56030
U+63CC.0	fHakka	 soi1; sai1
U+63CC.0	fCantonese	sak1; si1
U+63CC.0	fMandarin	sai1; cai2
U+63CC.0	fEnglish	 shake
U+63CD.0	fUTF8	揍
U+63CD.0	fCNS11643	1-5F45
U+63CD.0	fBig5	B47E
U+63CD.0	fGB	0-5761
U+63CD.0	fR/S	 64.9
U+63CD.0	fTotalStrokes	 12
U+63CD.0	fCangjie	QQKK
U+63CD.0	fFourCorner	55034
U+63CD.0	fHakka	 ciu5
U+63CD.0	fCantonese	zau3; cau3
U+63CD.0	fMandarin	zou4; cou4
U+63CD.0	fEnglish	[1] hit hard; beat; smash; slug (somebody) [2] break (a glass, etc.)
U+63CE.0	fUTF8	揎
U+63CE.0	fCNS11643	2-3B78
U+63CE.0	fBig5	D955
U+63CE.0	fGB	0-5E6F
U+63CE.0	fR/S	 64.9
U+63CE.0	fTotalStrokes	 12
U+63CE.0	fCangjie	QJMM
U+63CE.0	fFourCorner	53016
U+63CE.0	fHakka	  sien1
U+63CE.0	fCantonese	syun1
U+63CE.0	fMandarin	xuan1; shuan1
U+63CE.0	fEnglish	[1] pull up the sleeves and show the arms [2] fight with bare hands [3] push with hands [4] hit; strike
U+63CF.0	fUTF8	描
U+63CF.0	fCNS11643	1-5F40
U+63CF.0	fBig5	B479
U+63CF.0	fGB	0-4368
U+63CF.0	fR/S	 64.9
U+63CF.0	fTotalStrokes	 12
U+63CF.0	fCangjie	QTW
U+63CF.0	fFourCorner	54060
U+63CF.0	fHakka	  miau2; meu2
U+63CF.0	fCantonese	miu4
U+63CF.0	fMandarin	miao2; mao1; mao4
U+63CF.0	fEnglish	[1] trace; draw; sketch; retouch [2] depict
U+63D0.0	fUTF8	提
U+63D0.0	fCNS11643	1-5F48
U+63D0.0	fBig5	B4A3
U+63D0.0	fGB	0-4C61
U+63D0.0	fR/S	 64.9
U+63D0.0	fTotalStrokes	 12
U+63D0.0	fCangjie	QAMO
U+63D0.0	fFourCorner	56081
U+63D0.0	fHakka	  ti2; ti3
U+63D0.0	fCantonese	tai4; si4; dai2
U+63D0.0	fMandarin	ti2; di3; shi2; chi2
U+63D0.0	fEnglish	[1] lift by hand; pull up; arouse; raise; carry [2] cause to rise or happen [3] mention; bring forward; suggest; propose (a motion, etc.) [4] obtain; make delivery; draw out [5] mange; control [6] a rising stroke in Chinese writing [7] surname
U+63D0.1	fUTF8	提
U+63D0.1	fCNS11643	1-5F48
U+63D0.1	fBig5	B4A3
U+63D0.1	fGB	0-4C61
U+63D0.1	fR/S	 64.9
U+63D0.1	fTotalStrokes	 12
U+63D0.1	fCangjie	QAMO
U+63D0.1	fFourCorner	56081
U+63D0.1	fMandarin	di1
U+63D0.1	fEnglish	 hold or take in the hand
U+63D1.0	fUTF8	揑
U+63D1.0	fCNS11643	3-3B73
U+63D1.0	fGB	E-2B56
U+63D1.0	fR/S	 64.9
U+63D1.0	fTotalStrokes	 12
U+63D1.0	fCangjie	QHXM
U+63D1.0	fFourCorner	57010
U+63D1.0	fHakka	 ngiap7; net7; get7
U+63D1.0	fCantonese	nip6; lip6 (also)
U+63D1.0	fMandarin	nie1
U+63D1.0	fEnglish	[1] squeeze with fingers; press with fingers; knead; pinch [2] mold (mud, etc.) [3] fabricate; trump up; make up [4] shake [5] take
U+63D2.0	fUTF8	插
U+63D2.0	fCNS11643	1-5F46
U+63D2.0	fBig5	B4A1
U+63D2.0	fGB	0-3265
U+63D2.0	fR/S	 64.9
U+63D2.0	fTotalStrokes	 12
U+63D2.0	fCangjie	QHJX
U+63D2.0	fFourCorner	52077
U+63D2.0	fHakka	  cap7
U+63D2.0	fCantonese	caap3; caap2 (also)
U+63D2.0	fMandarin	cha1
U+63D2.0	fEnglish	[1] insert; stick into; put in [2] interpose; get a word in edgeways [3] plant
U+63D3.0	fUTF8	揓
U+63D3.0	fCNS11643	2-3C2E
U+63D3.0	fBig5	D969
U+63D3.0	fGB	5-4C6E
U+63D3.0	fR/S	 64.9
U+63D3.0	fTotalStrokes	 12
U+63D3.0	fCangjie	QYSD
U+63D3.0	fFourCorner	58012
U+63D3.0	fCantonese	ci3; to1
U+63D3.0	fMandarin	shi4; chi4
U+63D4.0	fUTF8	揔
U+63D4.0	fCNS11643	4-3771
U+63D4.0	fGB	5-4C69
U+63D4.0	fR/S	  64.9
U+63D4.0	fTotalStrokes	 12
U+63D4.0	fCangjie	QPKP
U+63D4.0	fFourCorner	57030
U+63D4.0	fCantonese	zung2
U+63D4.0	fMandarin	zong3; zong1
U+63D5.0	fUTF8	揕
U+63D5.0	fCNS11643	2-3C24
U+63D5.0	fBig5	D95F
U+63D5.0	fGB	3-4A60
U+63D5.0	fR/S	 64.9
U+63D5.0	fTotalStrokes	 12
U+63D5.0	fCangjie	QTMV
U+63D5.0	fFourCorner	54011
U+63D5.0	fHakka	 zem1; zhim5; zihm5; zim5; zem3
U+63D5.0	fCantonese	zam3
U+63D5.0	fMandarin	zhen4; zhen1
U+63D5.0	fEnglish	 strike; stab; thrust
U+63D6.0	fUTF8	揖
U+63D6.0	fCNS11643	1-5F4A
U+63D6.0	fBig5	B4A5
U+63D6.0	fGB	0-523E
U+63D6.0	fR/S	 64.9
U+63D6.0	fTotalStrokes	 12
U+63D6.0	fCangjie	QRSJ
U+63D6.0	fFourCorner	56041
U+63D6.0	fHakka	  jip7; cip8; rip7; jip8; cap7
U+63D6.0	fCantonese	jap1; cap1
U+63D6.0	fMandarin	yi1; ji2; ji4; yi4
U+63D6.0	fEnglish	[1] salute; bow with hands folding in front [2] yield politely; defer to
U+63D7.0	fUTF8	揗
U+63D7.0	fCNS11643	2-3C35
U+63D7.0	fBig5	D970
U+63D7.0	fGB	E-2B57
U+63D7.0	fR/S	 64.9
U+63D7.0	fTotalStrokes	 12
U+63D7.0	fCangjie	QHJU
U+63D7.0	fFourCorner	52064
U+63D7.0	fHakka	 sun3; sun2
U+63D7.0	fCantonese	tan4; tang4; seon5; seon6; ceon4
U+63D7.0	fMandarin	shun3; shun4; xun4
U+63D7.0	fEnglish	 strike; hit with hand tap
U+63D8.0	fUTF8	揘
U+63D8.0	fCNS11643	2-3C2D
U+63D8.0	fBig5	D968
U+63D8.0	fGB	3-4A6D
U+63D8.0	fR/S	 64.9
U+63D8.0	fTotalStrokes	 12
U+63D8.0	fCangjie	QHAG
U+63D8.0	fFourCorner	56014
U+63D8.0	fCantonese	wing4; waang4
U+63D8.0	fMandarin	heng2; yong2
U+63D9.0	fUTF8	揙
U+63D9.0	fCNS11643	2-3C36
U+63D9.0	fBig5	D971
U+63D9.0	fGB	3-4A75
U+63D9.0	fR/S	 64.9
U+63D9.0	fTotalStrokes	 12
U+63D9.0	fCangjie	QHSB
U+63D9.0	fFourCorner	53027
U+63D9.0	fHakka	 bien3; bien1
U+63D9.0	fCantonese	bin1; pin1; pin4
U+63D9.0	fMandarin	bian4; bian1
U+63DA.0	fUTF8	揚
U+63DA.0	fCNS11643	1-5F52
U+63DA.0	fBig5	B4AD
U+63DA.0	fGB	1-516F
U+63DA.0	fR/S	 64.9
U+63DA.0	fTotalStrokes	 12
U+63DA.0	fCangjie	QAMH
U+63DA.0	fFourCorner	56027
U+63DA.0	fHakka	  jong2; rong2; jong3; tit8
U+63DA.0	fCantonese	joeng4; joeng2 (also)
U+63DA.0	fMandarin	yang2
U+63DA.0	fEnglish	[1] raise [2] blazing (said of flames) [3] wave; flutter [4] praise; acclaim
U+63DB.0	fUTF8	換
U+63DB.0	fCNS11643	1-5F50
U+63DB.0	fBig5	B4AB
U+63DB.0	fGB	1-3B3B
U+63DB.0	fR/S	 64.9
U+63DB.0	fTotalStrokes	 12
U+63DB.0	fCangjie	QNBK
U+63DB.0	fFourCorner	57034
U+63DB.0	fHakka	 won5; won3; won6; fon6;  fon5; fan5; fan6; von3; van5
U+63DB.0	fCantonese	wun6
U+63DB.0	fMandarin	huan4
U+63DB.0	fEnglish	 change; exchange; substitute; alter
U+63DC.0	fUTF8	揜
U+63DC.0	fCNS11643	2-3C2B
U+63DC.0	fBig5	D966
U+63DC.0	fGB	3-4A6F
U+63DC.0	fR/S	 64.9
U+63DC.0	fTotalStrokes	 12
U+63DC.0	fCangjie	QORT
U+63DC.0	fFourCorner	58046
U+63DC.0	fHakka	 ram5; jam5; jam3
U+63DC.0	fCantonese	jim2; am2; ngam2
U+63DC.0	fMandarin	yan3
U+63DC.0	fEnglish	[1] cover up; conceal [2] rob; take by force [3] shut
U+63DD.0	fUTF8	揝
U+63DD.0	fCNS11643	2-3C2A
U+63DD.0	fBig5	D965
U+63DD.0	fGB	5-4C6B
U+63DD.0	fR/S	 64.9
U+63DD.0	fTotalStrokes	 12
U+63DD.0	fCangjie	QHOA
U+63DD.0	fFourCorner	58064
U+63DD.0	fHakka	  zam5; zem3
U+63DD.0	fCantonese	zaan6; zaan2; zyut3; zaam3
U+63DD.0	fMandarin	zuan4
U+63DD.0	fEnglish	 hold or seize with the hand; grasp; grip; clench
U+63DD.1	fUTF8	揝
U+63DD.1	fCNS11643	2-3C2A
U+63DD.1	fBig5	D965
U+63DD.1	fGB	5-4C6B
U+63DD.1	fR/S	 64.9
U+63DD.1	fTotalStrokes	 12
U+63DD.1	fCangjie	QHOA
U+63DD.1	fFourCorner	58064
U+63DD.1	fMandarin	zan3
U+63DD.1	fEnglish	 save money; save up
U+63DE.0	fUTF8	揞
U+63DE.0	fCNS11643	3-3B67
U+63DE.0	fGB	0-5E6E
U+63DE.0	fR/S	 64.9
U+63DE.0	fTotalStrokes	 12
U+63DE.0	fCangjie	QYTA
U+63DE.0	fFourCorner	50060
U+63DE.0	fHakka	  em1;  am3; am1
U+63DE.0	fCantonese	am2; ngam2 (also)
U+63DE.0	fMandarin	an3
U+63DE.0	fEnglish	[1] cover with the hand [2] hide [3] press on; tap [4] extinguish
U+63DF.0	fUTF8	揟
U+63DF.0	fCNS11643	2-3C28
U+63DF.0	fBig5	D963
U+63DF.0	fGB	3-4A78
U+63DF.0	fR/S	 64.9
U+63DF.0	fTotalStrokes	 12
U+63DF.0	fCangjie	QNOB
U+63DF.0	fFourCorner	57027
U+63DF.0	fCantonese	seoi1; zeoi1
U+63DF.0	fMandarin	xu1; ju1
U+63E0.0	fUTF8	揠
U+63E0.0	fCNS11643	2-3C22
U+63E0.0	fBig5	D95D
U+63E0.0	fGB	0-5E6B
U+63E0.0	fR/S	 64.9
U+63E0.0	fTotalStrokes	 12
U+63E0.0	fCangjie	QSAV
U+63E0.0	fFourCorner	51014
U+63E0.0	fHakka	 jet7; wat7; ret7; jat7; vat7
U+63E0.0	fCantonese	aat3; ngaat3 (also); aat1
U+63E0.0	fMandarin	ya4
U+63E0.0	fEnglish	 pull up or out
U+63E1.0	fUTF8	握
U+63E1.0	fCNS11643	1-5F49
U+63E1.0	fBig5	B4A4
U+63E1.0	fGB	0-4E55
U+63E1.0	fR/S	 64.9
U+63E1.0	fTotalStrokes	 12
U+63E1.0	fCangjie	QSMG
U+63E1.0	fFourCorner	57014
U+63E1.0	fHakka	  wok7; vok7; but7; wut7
U+63E1.0	fCantonese	ak1; aak1 (also); ngak1 (also); ngaak1 (also); au3; ngau3 (also)
U+63E1.0	fMandarin	wo4; ou1; wu1
U+63E1.0	fEnglish	[1] grasp; hold fast; grip; take by hand [2] a handful
U+63E2.0	fUTF8	揢
U+63E2.0	fCNS11643	3-3B66
U+63E2.0	fGB	3-4A74
U+63E2.0	fR/S	  64.9
U+63E2.0	fTotalStrokes	 12
U+63E2.0	fCangjie	QJHR
U+63E2.0	fFourCorner	53060
U+63E2.0	fHakka	  kak8; ak8
U+63E2.0	fCantonese	kaa1; kik1; kak1 (also); kaak3 (also); kak3
U+63E2.0	fMandarin	ke4
U+63E2.0	fEnglish	 seize
U+63E3.0	fUTF8	揣
U+63E3.0	fCNS11643	1-5F47
U+63E3.0	fBig5	B4A2
U+63E3.0	fGB	0-3427
U+63E3.0	fR/S	 64.9
U+63E3.0	fTotalStrokes	 12
U+63E3.0	fCangjie	QUMB
U+63E3.0	fFourCorner	52027
U+63E3.0	fHakka	  cui3;  ton2; chui3; zui3; coi3; ch'ui3; zhui5; chui5; cui5
U+63E3.0	fCantonese	ceoi2; cyun2 (also); caai1; cai1
U+63E3.0	fMandarin	chuai3; chuai1; chuai4; tuan2; zhui1; chuan4; duo3; zhui4
U+63E3.0	fEnglish	[1] measure; weigh; estimate; calculate; reckon [2] try; probe (for possibilities); put out a feeler
U+63E4.0	fUTF8	揤
U+63E4.0	fCNS11643	2-2F3B
U+63E4.0	fBig5	D1B9
U+63E4.0	fGB	3-4A46
U+63E4.0	fR/S	 64.9
U+63E4.0	fTotalStrokes	 12
U+63E4.0	fCangjie	QAIL
U+63E4.0	fFourCorner	57020
U+63E4.0	fCantonese	zat1; zeot1 (also)
U+63E4.0	fMandarin	ji2; jie2
U+63E5.0	fUTF8	揥
U+63E5.0	fCNS11643	2-3B79
U+63E5.0	fBig5	D956
U+63E5.0	fGB	3-4A71
U+63E5.0	fR/S	 64.9
U+63E5.0	fTotalStrokes	 12
U+63E5.0	fCangjie	QYBB
U+63E5.0	fFourCorner	50027
U+63E5.0	fHakka	  ti5; di2; di5
U+63E5.0	fCantonese	dai3; tai3; zaai6
U+63E5.0	fMandarin	ti4; di4; chi4
U+63E5.0	fEnglish	scratcher
U+63E6.0	fUTF8	揦
U+63E6.0	fCNS11643	3-3B68
U+63E6.0	fGB	3-4A64
U+63E6.0	fR/S	 64.9
U+63E6.0	fTotalStrokes	 12
U+63E6.0	fCangjie	QDLN
U+63E6.0	fFourCorner	52000
U+63E6.0	fHakka	 lat8; la1
U+63E6.0	fCantonese	laa2; laa5; laa3
U+63E6.0	fMandarin	la2
U+63E6.0	fEnglish	 clutch grab at rub or scrape tear
U+63E7.0	fUTF8	揧
U+63E7.0	fCNS11643	2-433D
U+63E7.0	fBig5	DDB7
U+63E7.0	fGB	5-4B4A
U+63E7.0	fR/S	 64.9
U+63E7.0	fTotalStrokes	 13
U+63E7.0	fCangjie	DNQ
U+63E7.0	fFourCorner	52502
U+63E7.0	fHakka	 lat8
U+63E7.0	fCantonese	laat6
U+63E7.0	fMandarin	la4
U+63E8.0	fUTF8	揨
U+63E8.0	fCNS11643	2-3B7A
U+63E8.0	fBig5	D957
U+63E8.0	fGB	5-4C6D
U+63E8.0	fR/S	 64.9
U+63E8.0	fTotalStrokes	 12
U+63E8.0	fCangjie	QYRN
U+63E8.0	fFourCorner	50021
U+63E8.0	fCantonese	caang4
U+63E8.0	fMandarin	cheng2
U+63E9.0	fUTF8	揩
U+63E9.0	fCNS11643	1-5F42
U+63E9.0	fBig5	B47B
U+63E9.0	fGB	0-3F2B
U+63E9.0	fR/S	 64.9
U+63E9.0	fTotalStrokes	 12
U+63E9.0	fCangjie	QPPA
U+63E9.0	fFourCorner	51061
U+63E9.0	fHakka	  gai1; kai1; giai1
U+63E9.0	fCantonese	haai1
U+63E9.0	fMandarin	kai1; jia2
U+63E9.0	fEnglish	 wipe; dust; clean; scrub; rub
U+63EA.0	fUTF8	揪
U+63EA.0	fCNS11643	1-5F4F
U+63EA.0	fBig5	B4AA
U+63EA.0	fGB	0-3E3E
U+63EA.0	fR/S	 64.9
U+63EA.0	fTotalStrokes	 12
U+63EA.0	fCangjie	QHDF
U+63EA.0	fFourCorner	59080
U+63EA.0	fHakka	 ziu1;  ciu1; kiu3; ts'ieu1; ziu3
U+63EA.0	fCantonese	zau1
U+63EA.0	fMandarin	jiu1
U+63EA.0	fEnglish	[1] clutch; grasp with one's hand; grab; hold fast; seize [2] pull; drag [3] pick on
U+63EB.0	fUTF8	揫
U+63EB.0	fCNS11643	2-4321
U+63EB.0	fBig5	DD79
U+63EB.0	fGB	E-2B58
U+63EB.0	fR/S	 64.9
U+63EB.0	fTotalStrokes	 13
U+63EB.0	fCangjie	HFQ
U+63EB.0	fFourCorner	29502
U+63EB.0	fHakka	 ziu1; kiu3; ciu1; ziu3
U+63EB.0	fCantonese	zau1; cau1
U+63EB.0	fMandarin	jiu1; qiu2
U+63EB.0	fEnglish	grasp with hand; pinch
U+63EC.0	fUTF8	揬
U+63EC.0	fCNS11643	4-3760
U+63EC.0	fGB	5-4C74
U+63EC.0	fR/S	  64.9
U+63EC.0	fTotalStrokes	 12
U+63EC.0	fCangjie	QJCK
U+63EC.0	fFourCorner	53030
U+63EC.0	fHakka	  tut7; tut8; dut7
U+63EC.0	fCantonese	dat6; dat1
U+63EC.0	fMandarin	tu2
U+63EC.0	fEnglish	brusque; forward
U+63ED.0	fUTF8	揭
U+63ED.0	fCNS11643	1-5F4B
U+63ED.0	fBig5	B4A6
U+63ED.0	fGB	0-3D52
U+63ED.0	fR/S	 64.9
U+63ED.0	fTotalStrokes	 12
U+63ED.0	fCangjie	QAPV
U+63ED.0	fFourCorner	56027
U+63ED.0	fHakka	 giet7; ket7; jat7; get7
U+63ED.0	fCantonese	kit3; gai6 (also); kai3; hei3
U+63ED.0	fMandarin	jie1; chi4; he2; ji4; jie2; qi4; qian1
U+63ED.0	fEnglish	[1] raise high; lift up or off [2] unveil; uncover; unearth; expose; tear off; take off [3] announce; publicize [4] a Chinese family name
U+63EE.0	fUTF8	揮
U+63EE.0	fCNS11643	1-5F4C
U+63EE.0	fBig5	B4A7
U+63EE.0	fGB	1-3B53
U+63EE.0	fR/S	 64.9
U+63EE.0	fTotalStrokes	 12
U+63EE.0	fCangjie	QBJJ
U+63EE.0	fFourCorner	57056
U+63EE.0	fHakka	  fui1; fi1; wun3
U+63EE.0	fCantonese	fai1
U+63EE.0	fMandarin	hui1; hun2; xun4
U+63EE.0	fEnglish	[1] wield (a sword, pen, etc.); move; shake; wave; brandish; make a light or rapid stroke [2] conduct; direct (troop movements, a concert, course of action, etc.) [3] wipe away (sweat, tears, etc) [4] scatter; sprinkle [5] squander (money, etc.) [6] swing (fists)
U+63EF.0	fUTF8	揯
U+63EF.0	fCNS11643	2-3B7B
U+63EF.0	fBig5	D958
U+63EF.0	fGB	E-2B59
U+63EF.0	fR/S	 64.9
U+63EF.0	fTotalStrokes	 12
U+63EF.0	fCangjie	QPMM
U+63EF.0	fFourCorner	51012
U+63EF.0	fCantonese	gang1; gang3; hang4
U+63EF.0	fMandarin	geng1; geng4
U+63EF.0	fEnglish	tight, distended
U+63F0.0	fUTF8	揰
U+63F0.0	fCNS11643	2-3C34
U+63F0.0	fBig5	D96F
U+63F0.0	fGB	5-4C63
U+63F0.0	fR/S	 64.9
U+63F0.0	fTotalStrokes	 12
U+63F0.0	fCangjie	QHJG
U+63F0.0	fFourCorner	52014
U+63F0.0	fHakka	 chung5; cung5
U+63F0.0	fCantonese	cung3; zung6
U+63F0.0	fMandarin	chong4; dong3
U+63F0.0	fEnglish	 push into
U+63F1.0	fUTF8	揱
U+63F1.0	fCNS11643	2-427E
U+63F1.0	fBig5	DD78
U+63F1.0	fGB	3-494E
U+63F1.0	fR/S	 64.9
U+63F1.0	fTotalStrokes	 13
U+63F1.0	fCangjie	FNQ
U+63F1.0	fFourCorner	92502
U+63F1.0	fHakka	  sok7; siok7
U+63F1.0	fCantonese	siu1; sok3 (also)
U+63F1.0	fMandarin	shuo4; xiao1; shan1; shao4; xue4
U+63F1.0	fEnglish	[1] long and tapering arms [2] tapering
U+63F2.0	fUTF8	揲
U+63F2.0	fCNS11643	2-3C25
U+63F2.0	fBig5	D960
U+63F2.0	fGB	0-5E69
U+63F2.0	fR/S	 64.9
U+63F2.0	fTotalStrokes	 12
U+63F2.0	fCangjie	QPTD
U+63F2.0	fFourCorner	54094
U+63F2.0	fHakka	 tiap8; sat8; tiap7; shat7; s'et7
U+63F2.0	fCantonese	sit3; dip6; jip6; sip3; sit6
U+63F2.0	fMandarin	she2; die2; ye4; tie2; she4; xue1
U+63F2.0	fEnglish	 sort out the stalks used in divination
U+63F3.0	fUTF8	揳
U+63F3.0	fCNS11643	2-3B7E
U+63F3.0	fBig5	D95B
U+63F3.0	fGB	3-4A5F
U+63F3.0	fR/S	 64.9
U+63F3.0	fTotalStrokes	 12
U+63F3.0	fCangjie	QQHK
U+63F3.0	fFourCorner	57034
U+63F3.0	fHakka	  siet7
U+63F3.0	fCantonese	sit3; sip3 (also); git3; se2
U+63F3.0	fMandarin	xie4; xie2; xue1; jia2; xie1
U+63F3.0	fEnglish	[1] rub; wipe [2] estimate
U+63F4.0	fUTF8	援
U+63F4.0	fCNS11643	1-5F4E
U+63F4.0	fBig5	B4A9
U+63F4.0	fGB	0-542E
U+63F4.0	fR/S	 64.9
U+63F4.0	fTotalStrokes	 12
U+63F4.0	fCangjie	QBME
U+63F4.0	fFourCorner	52047
U+63F4.0	fHakka	 jen1; jen2; ren2; ren1; jan1
U+63F4.0	fCantonese	wun4; jyun4 (also); jyun6 (also)
U+63F4.0	fMandarin	yuan2; huan4; yuan4; yung4
U+63F4.0	fEnglish	[1] aid; assist; help; reinforce; rescue [2] lead [3] take hold of; pull by hand [4] invoke (a law, precedent, etc.)
U+63F5.0	fUTF8	揵
U+63F5.0	fCNS11643	2-3C26
U+63F5.0	fBig5	D961
U+63F5.0	fGB	3-4A5C
U+63F5.0	fR/S	 64.9
U+63F5.0	fTotalStrokes	 12
U+63F5.0	fCangjie	QNKQ
U+63F5.0	fFourCorner	55044
U+63F5.0	fHakka	 kien2; kien5; ken2; kian3; ken3
U+63F5.0	fCantonese	kin4; gin6; gin4
U+63F5.0	fMandarin	qian2; jian4; jian3
U+63F5.0	fEnglish	carry
U+63F6.0	fUTF8	揶
U+63F6.0	fCNS11643	2-3C23
U+63F6.0	fBig5	D95E
U+63F6.0	fGB	0-5E5E
U+63F6.0	fR/S	 64.9
U+63F6.0	fTotalStrokes	 12
U+63F6.0	fCangjie	QSJL
U+63F6.0	fFourCorner	57027
U+63F6.0	fHakka	  ja2; jak8; ra2; zak8
U+63F6.0	fCantonese	je4
U+63F6.0	fMandarin	ye2
U+63F6.0	fEnglish	 make fun of; ridicule; jeer at; play a joke on
U+63F7.0	fUTF8	揷
U+63F7.0	fCNS11643	3-3B72
U+63F7.0	fGB	E-2B5A
U+63F7.0	fR/S	 64.9
U+63F7.0	fTotalStrokes	 12
U+63F7.0	fCangjie	QHJL
U+63F7.0	fFourCorner	52070
U+63F7.0	fHakka	 cap7
U+63F7.0	fCantonese	caap3
U+63F7.0	fMandarin	cha1
U+63F7.0	fEnglish	insert
U+63F8.0	fUTF8	揸
U+63F8.0	fCNS11643	3-3B6D
U+63F8.0	fGB	0-5E6A
U+63F8.0	fR/S	  64.9
U+63F8.0	fTotalStrokes	 12
U+63F8.0	fCangjie	QDAM
U+63F8.0	fFourCorner	54010
U+63F8.0	fHakka	  za1
U+63F8.0	fCantonese	zaa1; zaa2
U+63F8.0	fMandarin	zha1
U+63F8.0	fEnglish	 pick up with fingers; grasp by hand
U+63F9.0	fUTF8	揹
U+63F9.0	fCNS11643	1-5F53
U+63F9.0	fBig5	B4AE
U+63F9.0	fGB	E-2B5B
U+63F9.0	fR/S	 64.9
U+63F9.0	fTotalStrokes	 12
U+63F9.0	fCangjie	QLPB
U+63F9.0	fFourCorner	51027
U+63F9.0	fHakka	 boi5
U+63F9.0	fCantonese	bui3
U+63F9.0	fMandarin	bei1; bei4
U+63F9.0	fEnglish	 carry on the back; shoulder (a load, responsibility, etc.)
U+63FA.0	fUTF8	揺
U+63FA.0	fCNS11643	4-376F
U+63FA.0	fGB	E-2B5C
U+63FA.0	fR/S	 64.9
U+63FA.0	fTotalStrokes	 12
U+63FA.0	fCangjie	QBMU
U+63FA.0	fFourCorner	52070
U+63FA.0	fCantonese	jiu4
U+63FA.0	fMandarin	yao2
U+63FA.0	fEnglish	 wag; swing; wave shake scull
U+63FB.0	fUTF8	揻
U+63FB.0	fGB	3-4A66
U+63FB.0	fR/S	 64.9
U+63FB.0	fTotalStrokes	 12
U+63FB.0	fCangjie	QIHV
U+63FB.0	fCantonese	wai1
U+63FC.0	fUTF8	揼
U+63FC.0	fGB	1-7C3D
U+63FC.0	fR/S	 64.9
U+63FC.0	fTotalStrokes	 12
U+63FC.0	fCangjie	QMRE
U+63FC.0	fCantonese	dam1; dam2; dam3; dap6 (also); dam4
U+63FC.0	fEnglish	to delay, waste time; to throw down, beat; to hang down
U+63FD.0	fUTF8	揽
U+63FD.0	fGB	0-403F
U+63FD.0	fR/S	 64.9
U+63FD.0	fTotalStrokes	 12
U+63FD.0	fCangjie	QLIU
U+63FD.0	fHakka	 lam3
U+63FD.0	fCantonese	laam5; laam2 (also)
U+63FD.0	fMandarin	lan3
U+63FD.0	fEnglish	simplified form of &#x652C;
U+63FE.0	fUTF8	揾
U+63FE.0	fCNS11643	3-3B6F
U+63FE.0	fGB	3-4A69
U+63FE.0	fR/S	 64.9
U+63FE.0	fTotalStrokes	 12
U+63FE.0	fCangjie	QABT
U+63FE.0	fFourCorner	56010
U+63FE.0	fHakka	 wut8
U+63FE.0	fCantonese	wan2; wan3
U+63FE.0	fMandarin	wen4
U+63FE.0	fEnglish	 wipe off press down with knuckle
U+63FF.0	fUTF8	揿
U+63FF.0	fGB	0-5E6C
U+63FF.0	fR/S	 64.9
U+63FF.0	fTotalStrokes	 12
U+63FF.0	fCangjie	XQCNO
U+63FF.0	fCantonese	gam6
U+63FF.0	fMandarin	qin4
U+63FF.0	fEnglish	simplified form of &#x64B3;
U+6400.0	fUTF8	搀
U+6400.0	fGB	0-3273
U+6400.0	fR/S	 64.9
U+6400.0	fTotalStrokes	 12
U+6400.0	fCangjie	QNUY
U+6400.0	fHakka	 cam1
U+6400.0	fCantonese	caam1; caam4
U+6400.0	fMandarin	chan1; chan2; chan4; chen1
U+6400.0	fEnglish	simplified form of &#x6519;
U+6401.0	fUTF8	搁
U+6401.0	fGB	0-3869
U+6401.0	fR/S	 64.9
U+6401.0	fTotalStrokes	 12
U+6401.0	fCangjie	QLSR
U+6401.0	fHakka	 gok7
U+6401.0	fCantonese	gok3
U+6401.0	fMandarin	ge1; ge2
U+6401.0	fEnglish	simplified form of &#x64F1;
U+6401.1	fUTF8	搁
U+6401.1	fGB	0-3869
U+6401.1	fR/S	 64.9
U+6401.1	fTotalStrokes	 12
U+6401.1	fCangjie	QLSR
U+6401.1	fMandarin	ge5
U+6402.0	fUTF8	搂
U+6402.0	fGB	0-4227
U+6402.0	fR/S	 64.9
U+6402.0	fTotalStrokes	 12
U+6402.0	fCangjie	QFDV
U+6402.0	fCantonese	lau1; lau2; lau4; lau5
U+6402.0	fMandarin	lou2; lou3; lou1
U+6402.0	fEnglish	simplified form of &#x645F;
U+6403.0	fUTF8	搃
U+6403.0	fCNS11643	3-6521
U+6403.0	fGB	E-2B5D
U+6403.0	fR/S	 64.9
U+6403.0	fTotalStrokes	 12
U+6403.0	fCangjie	QCRP
U+6403.0	fHakka	 zung3
U+6403.0	fCantonese	zung2
U+6403.0	fMandarin	zong3
U+6403.0	fEnglish	 all; general summarise the chief manage
U+6404.0	fUTF8	搄
U+6404.0	fGB	3-4A73
U+6404.0	fR/S	 64.9
U+6404.0	fTotalStrokes	 12
U+6404.0	fCangjie	XQPMM
U+6404.0	fHakka	 hen2;  kwen3
U+6404.0	fCantonese	gang1
U+6404.0	fMandarin	geng1
U+6405.0	fUTF8	搅
U+6405.0	fGB	0-3D41
U+6405.0	fR/S	 64.9
U+6405.0	fTotalStrokes	 12
U+6405.0	fCangjie	QFBU
U+6405.0	fHakka	 gau3
U+6405.0	fCantonese	gaau2
U+6405.0	fMandarin	jiao3; xiao3
U+6405.0	fEnglish	simplified form of &#x652A;
U+6406.0	fUTF8	搆
U+6406.0	fCNS11643	1-6438
U+6406.0	fBig5	B770
U+6406.0	fGB	E-2B5E
U+6406.0	fR/S	 64.10
U+6406.0	fTotalStrokes	 13
U+6406.0	fCangjie	QTTB
U+6406.0	fFourCorner	55047
U+6406.0	fHakka	 geu5; kieu5; giu5
U+6406.0	fCantonese	kau3; gau3 (also)
U+6406.0	fMandarin	gou4; gou1
U+6406.0	fEnglish	[1] pull; drag [2] reach [3] implicate [4] make (war, peace, etc.); bring (a disaster, a grudge, etc.); incur (animosity, etc.) [5] compose
U+6407.0	fUTF8	搇
U+6407.0	fCNS11643	3-4149
U+6407.0	fGB	E-2B5F
U+6407.0	fR/S	  64.10
U+6407.0	fTotalStrokes	 13
U+6407.0	fCangjie	QOIV
U+6407.0	fFourCorner	58030
U+6407.0	fHakka	 kim5;  kim3; kim6
U+6407.0	fCantonese	kam2; gam6
U+6407.0	fMandarin	qin4
U+6407.0	fEnglish	 press down with the hand lead on
U+6408.0	fUTF8	搈
U+6408.0	fCNS11643	3-4132
U+6408.0	fGB	3-4B24
U+6408.0	fR/S	  64.10
U+6408.0	fTotalStrokes	 13
U+6408.0	fCangjie	QJCR
U+6408.0	fFourCorner	53060
U+6408.0	fCantonese	jung4
U+6408.0	fMandarin	yong3
U+6409.0	fUTF8	搉
U+6409.0	fCNS11643	2-4324
U+6409.0	fBig5	DD7C
U+6409.0	fGB	E-2B60
U+6409.0	fR/S	 64.10
U+6409.0	fTotalStrokes	 13
U+6409.0	fCangjie	QOBG
U+6409.0	fFourCorner	54014
U+6409.0	fHakka	  kok7;  kok8
U+6409.0	fCantonese	kok1; kok3; gok3
U+6409.0	fMandarin	que4; huo4; jue2
U+6409.0	fEnglish	[1] knock; strike [2] discuss; negotiate; consult
U+640A.0	fUTF8	搊
U+640A.0	fCNS11643	2-4337
U+640A.0	fBig5	DDB1
U+640A.0	fGB	3-497E
U+640A.0	fR/S	 64.10
U+640A.0	fTotalStrokes	 13
U+640A.0	fCangjie	QPUU
U+640A.0	fFourCorner	57027
U+640A.0	fHakka	  ceu1
U+640A.0	fCantonese	cau1; cai1 (also)
U+640A.0	fMandarin	chou1; zhou3; zhu1
U+640A.0	fEnglish	pluck
U+640B.0	fUTF8	搋
U+640B.0	fCNS11643	2-433C
U+640B.0	fBig5	DDB6
U+640B.0	fGB	0-5E75
U+640B.0	fR/S	 64.10
U+640B.0	fTotalStrokes	 13
U+640B.0	fCangjie	QHYU
U+640B.0	fFourCorner	51011
U+640B.0	fHakka	 cai1; chi3; cih3; ja2
U+640B.0	fCantonese	caai1; ci2; cai1
U+640B.0	fMandarin	chi3; chuai1; cha1; yi2; yi3
U+640B.0	fEnglish	[1] conceal something in the bosom [2] knead (dough); rub
U+640C.0	fUTF8	搌
U+640C.0	fCNS11643	2-4330
U+640C.0	fBig5	DDAA
U+640C.0	fGB	0-5E78
U+640C.0	fR/S	 64.10
U+640C.0	fTotalStrokes	 13
U+640C.0	fCangjie	QSTV
U+640C.0	fFourCorner	57032
U+640C.0	fHakka	 zan3; zhen3; zen3
U+640C.0	fCantonese	zin2
U+640C.0	fMandarin	zhan3; chan3; zhan4
U+640C.0	fEnglish	[1] bind [2] wipe; mop
U+640D.0	fUTF8	損
U+640D.0	fCNS11643	1-6434
U+640D.0	fBig5	B76C
U+640D.0	fGB	1-4B70
U+640D.0	fR/S	 64.10
U+640D.0	fTotalStrokes	 13
U+640D.0	fCangjie	QRBC
U+640D.0	fFourCorner	56086
U+640D.0	fHakka	  sun3
U+640D.0	fCantonese	syun2
U+640D.0	fMandarin	sun3
U+640D.0	fEnglish	[1] detract; damage; injure; destroy; harm [2] lose; loss [3] diminish; reduce; decrease [4] weak; emaciated [5] ridicule; jeer at [6] wicked and mean; cruel [7] long-term emaciation (Chinese medicine)
U+640E.0	fUTF8	搎
U+640E.0	fCNS11643	2-4341
U+640E.0	fBig5	DDBB
U+640E.0	fGB	3-4A37
U+640E.0	fR/S	 64.10
U+640E.0	fTotalStrokes	 13
U+640E.0	fCangjie	QNDF
U+640E.0	fFourCorner	52093
U+640E.0	fHakka	 sun1
U+640E.0	fCantonese	syun1
U+640E.0	fMandarin	sun1
U+640E.0	fEnglish	 rub with the hand stroke
U+640F.0	fUTF8	搏
U+640F.0	fCNS11643	1-6431
U+640F.0	fBig5	B769
U+640F.0	fGB	0-322B
U+640F.0	fR/S	 64.10
U+640F.0	fTotalStrokes	 13
U+640F.0	fCangjie	QIBI
U+640F.0	fFourCorner	53042
U+640F.0	fHakka	  bok7
U+640F.0	fCantonese	bok3
U+640F.0	fMandarin	bo2; bu4; fu4; po4
U+640F.0	fEnglish	[1] pounce on (or at); spring upon [2] strike; box; engage in a hand-to-hand combat [3] grasp; catch; arrest; seize
U+6410.0	fUTF8	搐
U+6410.0	fCNS11643	2-4322
U+6410.0	fBig5	DD7A
U+6410.0	fGB	0-3424
U+6410.0	fR/S	 64.10
U+6410.0	fTotalStrokes	 13
U+6410.0	fCangjie	QYVW
U+6410.0	fFourCorner	50063
U+6410.0	fHakka	 hiuk7
U+6410.0	fCantonese	cuk1; caau1; cau1
U+6410.0	fMandarin	chu4; chou1
U+6410.0	fEnglish	cramp; spasm; convulsion; twitch; shake involuntarily
U+6411.0	fUTF8	搑
U+6411.0	fCNS11643	3-413A
U+6411.0	fGB	3-4A61
U+6411.0	fR/S	  64.10
U+6411.0	fTotalStrokes	 13
U+6411.0	fCangjie	QTSJ
U+6411.0	fFourCorner	54040
U+6411.0	fHakka	 jung2; rung2
U+6411.0	fCantonese	jung4
U+6411.0	fMandarin	rong3
U+6411.0	fEnglish	[1] push; beat [2] stuff
U+6412.0	fUTF8	搒
U+6412.0	fCNS11643	2-4323
U+6412.0	fBig5	DD7B
U+6412.0	fGB	3-4B22
U+6412.0	fR/S	 64.10
U+6412.0	fTotalStrokes	 13
U+6412.0	fCangjie	QYBS
U+6412.0	fFourCorner	50027
U+6412.0	fHakka	 bong3; pong1; bang1
U+6412.0	fCantonese	pong3; paang4; pang4; bong2; bong3; bong1
U+6412.0	fMandarin	beng4; peng2; beng1; bang4; nang4
U+6412.0	fEnglish	 pole beat
U+6413.0	fUTF8	搓
U+6413.0	fCNS11643	1-642A
U+6413.0	fBig5	B762
U+6413.0	fGB	0-346A
U+6413.0	fR/S	 64.10
U+6413.0	fTotalStrokes	 13
U+6413.0	fCangjie	QTQM
U+6413.0	fFourCorner	58011
U+6413.0	fHakka	  co1;  cai1; co5; cai3; co2
U+6413.0	fCantonese	co1; caai1
U+6413.0	fMandarin	cuo1; chai1; cuo3
U+6413.0	fEnglish	[1] rub hands; rub between the hands [2] scrub [3] twist (a thread, etc.) between the hands
U+6414.0	fUTF8	搔
U+6414.0	fCNS11643	1-6433
U+6414.0	fBig5	B76B
U+6414.0	fGB	0-4926
U+6414.0	fR/S	 64.10
U+6414.0	fTotalStrokes	 13
U+6414.0	fCangjie	QEII
U+6414.0	fFourCorner	57036
U+6414.0	fHakka	 zau3; sau1; {zau3}
U+6414.0	fCantonese	sou1
U+6414.0	fMandarin	sao1; zhao3; sao4
U+6414.0	fEnglish	[1] scratch lightly [2] irritate; annoy
U+6415.0	fUTF8	搕
U+6415.0	fCNS11643	2-432A
U+6415.0	fBig5	DDA4
U+6415.0	fGB	5-4C7D
U+6415.0	fR/S	 64.10
U+6415.0	fTotalStrokes	 13
U+6415.0	fCangjie	QGIT
U+6415.0	fFourCorner	54017
U+6415.0	fHakka	 kap7; ep7; kap8; hap8
U+6415.0	fCantonese	hap6; ap1; ngap1 (also); hap1
U+6415.0	fMandarin	ke4
U+6415.0	fEnglish	 strike take in the hand
U+6416.0	fUTF8	搖
U+6416.0	fCNS11643	1-6436
U+6416.0	fBig5	B76E
U+6416.0	fGB	E-2B61
U+6416.0	fR/S	 64.10
U+6416.0	fTotalStrokes	 13
U+6416.0	fCangjie	QBOU
U+6416.0	fFourCorner	52072
U+6416.0	fHakka	  jau2; jeu2; rau2; jau3
U+6416.0	fCantonese	jiu4; jiu2 (also)
U+6416.0	fMandarin	yao2; yao4
U+6416.0	fEnglish	[1] wag; swing; wave; shake; rock [2] sway (said of one's confidence, determination); wobble; shake [3] scull; row (a boat, etc.) [4] agitate; incite; annoy
U+6417.0	fUTF8	搗
U+6417.0	fCNS11643	1-6437
U+6417.0	fBig5	B76F
U+6417.0	fGB	1-3537
U+6417.0	fR/S	 64.10
U+6417.0	fTotalStrokes	 13
U+6417.0	fCangjie	QHAU
U+6417.0	fFourCorner	57027
U+6417.0	fHakka	 dau3; do3; dau5
U+6417.0	fCantonese	dou2
U+6417.0	fMandarin	dao3
U+6417.0	fEnglish	[1] thresh (grains); hull; unhusk [2] beat; pound [3] drive; attack [4] sabotage
U+6418.0	fUTF8	搘
U+6418.0	fCNS11643	2-432B
U+6418.0	fBig5	DDA5
U+6418.0	fGB	5-4C7C
U+6418.0	fR/S	 64.10
U+6418.0	fTotalStrokes	 13
U+6418.0	fCangjie	QJPA
U+6418.0	fFourCorner	54061
U+6418.0	fHakka	 zhi1; zih1; tje1
U+6418.0	fCantonese	zi1
U+6418.0	fMandarin	zhi1
U+6418.0	fEnglish	 support; prop up
U+6419.0	fUTF8	搙
U+6419.0	fCNS11643	3-4138
U+6419.0	fGB	3-4A7A
U+6419.0	fR/S	  64.10
U+6419.0	fTotalStrokes	 13
U+6419.0	fCangjie	QMVI
U+6419.0	fFourCorner	51040
U+6419.0	fHakka	 nuk8; ngieu3
U+6419.0	fCantonese	nuk6; luk6 (also)
U+6419.0	fMandarin	nu4
U+6419.0	fEnglish	 handle play with
U+641A.0	fUTF8	搚
U+641A.0	fCNS11643	2-4338
U+641A.0	fBig5	DDB2
U+641A.0	fGB	5-4D34
U+641A.0	fR/S	 64.10
U+641A.0	fTotalStrokes	 13
U+641A.0	fCangjie	QKSB
U+641A.0	fFourCorner	54027
U+641A.0	fCantonese	lap6; laap6 (also); hip6
U+641A.0	fMandarin	xie2; la1; la4; xian4
U+641B.0	fUTF8	搛
U+641B.0	fCNS11643	2-433E
U+641B.0	fBig5	DDB8
U+641B.0	fGB	0-5E76
U+641B.0	fR/S	 64.10
U+641B.0	fTotalStrokes	 13
U+641B.0	fCangjie	QTXC
U+641B.0	fFourCorner	58037
U+641B.0	fHakka	 liam2;  giam1; kiam2
U+641B.0	fCantonese	gim1; lim4
U+641B.0	fMandarin	jian1; lian2
U+641B.0	fEnglish	[1] strike a drum [2] pick up food with chopsticks [3] [4] pick up food with chopsticks
U+641C.0	fUTF8	搜
U+641C.0	fCNS11643	1-6432
U+641C.0	fBig5	B76A
U+641C.0	fGB	0-4B51
U+641C.0	fR/S	 64.9; 64.10
U+641C.0	fTotalStrokes	 13
U+641C.0	fCangjie	QHXE
U+641C.0	fFourCorner	57047
U+641C.0	fHakka	 seu1; sau1; siu1
U+641C.0	fCantonese	sau1; sau2
U+641C.0	fMandarin	sou1; sao1; shao3; sou3; xiao1
U+641C.0	fEnglish	[1] search; seek [2] inquiry into; investigate
U+641D.0	fUTF8	搝
U+641D.0	fCNS11643	3-413E
U+641D.0	fGB	5-4D27
U+641D.0	fR/S	  64.10
U+641D.0	fTotalStrokes	 13
U+641D.0	fCangjie	QHUK
U+641D.0	fFourCorner	56034
U+641D.0	fCantonese	hau2
U+641D.0	fMandarin	qiu3
U+641E.0	fUTF8	搞
U+641E.0	fCNS11643	1-642C
U+641E.0	fBig5	B764
U+641E.0	fGB	0-3863
U+641E.0	fR/S	 64.10
U+641E.0	fTotalStrokes	 13
U+641E.0	fCangjie	QYRB
U+641E.0	fFourCorner	50027
U+641E.0	fHakka	 gau3
U+641E.0	fCantonese	gaau2
U+641E.0	fMandarin	gao3; qiao1
U+641E.0	fEnglish	[1] stir up; cause trouble; make a mess of; mix up [2] do; carry on; be engaged in [3] get; secure [4] set up; start; organise
U+641F.0	fUTF8	搟
U+641F.0	fCNS11643	2-4329
U+641F.0	fBig5	DDA3
U+641F.0	fGB	5-4C35
U+641F.0	fR/S	 64.10
U+641F.0	fTotalStrokes	 13
U+641F.0	fCangjie	QJJJ
U+641F.0	fFourCorner	51040
U+641F.0	fCantonese	hin2
U+641F.0	fMandarin	xian3
U+6420.0	fUTF8	搠
U+6420.0	fCNS11643	2-4325
U+6420.0	fBig5	DD7D
U+6420.0	fGB	0-5E77
U+6420.0	fR/S	 64.10
U+6420.0	fTotalStrokes	 13
U+6420.0	fCangjie	QTUB
U+6420.0	fFourCorner	57020
U+6420.0	fHakka	  sok7; shok7
U+6420.0	fCantonese	sok3
U+6420.0	fMandarin	shuo4
U+6420.0	fEnglish	[1] thrust (at one's enemy) [2] smear; daub
U+6421.0	fUTF8	搡
U+6421.0	fCNS11643	2-4340
U+6421.0	fBig5	DDBA
U+6421.0	fGB	0-5E7A
U+6421.0	fR/S	 64.10
U+6421.0	fTotalStrokes	 13
U+6421.0	fCangjie	QEED
U+6421.0	fFourCorner	57094
U+6421.0	fHakka	  song3; sung3
U+6421.0	fCantonese	song2
U+6421.0	fMandarin	sang3; sang4
U+6421.0	fEnglish	[1] push over or push back [2] put down heavily
U+6422.0	fUTF8	搢
U+6422.0	fCNS11643	2-432E
U+6422.0	fBig5	DDA8
U+6422.0	fGB	3-4A79
U+6422.0	fR/S	 64.10
U+6422.0	fTotalStrokes	 13
U+6422.0	fCangjie	QMIA
U+6422.0	fFourCorner	51061
U+6422.0	fHakka	 zin3; zin5
U+6422.0	fCantonese	zeon3
U+6422.0	fMandarin	jin4; ji1; jian4; jiang3
U+6422.0	fEnglish	[1] stick into [2] shake
U+6423.0	fUTF8	搣
U+6423.0	fCNS11643	2-432F
U+6423.0	fBig5	DDA9
U+6423.0	fGB	3-4A7B
U+6423.0	fR/S	 64.10
U+6423.0	fTotalStrokes	 13
U+6423.0	fCangjie	QIHF
U+6423.0	fFourCorner	53050
U+6423.0	fHakka	 met8;  met7; miet8
U+6423.0	fCantonese	mit1; mit6
U+6423.0	fMandarin	mie4
U+6423.0	fEnglish	[1] pluck [2] pull out [3] peel [4] pinch; peel off; tear
U+6424.0	fUTF8	搤
U+6424.0	fCNS11643	2-4326
U+6424.0	fBig5	DD7E
U+6424.0	fGB	E-2B62
U+6424.0	fR/S	 64.10
U+6424.0	fTotalStrokes	 13
U+6424.0	fCangjie	QTCT
U+6424.0	fFourCorner	58017
U+6424.0	fHakka	 ak7; jit7; rit7; jet7
U+6424.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+6424.0	fMandarin	e4; ai4
U+6424.0	fEnglish	seize
U+6425.0	fUTF8	搥
U+6425.0	fCNS11643	2-433A
U+6425.0	fBig5	DDB4
U+6425.0	fGB	E-2B63
U+6425.0	fR/S	 64.10
U+6425.0	fTotalStrokes	 13
U+6425.0	fCangjie	QYHR
U+6425.0	fFourCorner	57037
U+6425.0	fHakka	 cui2; ch'ui2; ch'ui3; chui2
U+6425.0	fCantonese	ceoi4
U+6425.0	fMandarin	chui2; dui1
U+6425.0	fEnglish	 beat; pound; strike with a stick or fist
U+6426.0	fUTF8	搦
U+6426.0	fCNS11643	2-4331
U+6426.0	fBig5	DDAB
U+6426.0	fGB	0-5E79
U+6426.0	fR/S	 64.10
U+6426.0	fTotalStrokes	 13
U+6426.0	fCangjie	QNMM
U+6426.0	fFourCorner	57027
U+6426.0	fHakka	  nit8; lak8; lit7; lit8
U+6426.0	fCantonese	nik1; nok6 (also); lik1 (also); lok6 (also)
U+6426.0	fMandarin	nuo4; ne4; ni4
U+6426.0	fEnglish	[1] grasp; seize; take in hand; hold; carry [2] challenge [3] incite [4] suppress; restrain
U+6427.0	fUTF8	搧
U+6427.0	fCNS11643	2-433B
U+6427.0	fBig5	DDB5
U+6427.0	fGB	5-4D32
U+6427.0	fR/S	 64.10
U+6427.0	fTotalStrokes	 13
U+6427.0	fCangjie	QHSM
U+6427.0	fFourCorner	53027
U+6427.0	fHakka	  sen5; san1; san5; shen5; shan5
U+6427.0	fCantonese	sin3
U+6427.0	fMandarin	shan1; shan4
U+6427.0	fEnglish	[1] fan [2] strike on the face; slap on the face [3] stir up; incite
U+6428.0	fUTF8	搨
U+6428.0	fCNS11643	2-4333
U+6428.0	fBig5	DDAD
U+6428.0	fGB	E-2B64
U+6428.0	fR/S	 64.10
U+6428.0	fTotalStrokes	 13
U+6428.0	fCangjie	QASM
U+6428.0	fFourCorner	56027
U+6428.0	fHakka	  tap7;  dap7
U+6428.0	fCantonese	taap3
U+6428.0	fMandarin	ta4; da1
U+6428.0	fEnglish	[1] take a rubbing of an inscription on stone, etc. [2] make an exact copy with paper and a writing brush
U+6429.0	fUTF8	搩
U+6429.0	fCNS11643	3-413D
U+6429.0	fGB	E-2B65
U+6429.0	fR/S	  64.10
U+6429.0	fTotalStrokes	 13
U+6429.0	fCangjie	QNQD
U+6429.0	fFourCorner	55090
U+6429.0	fHakka	 kiet8
U+6429.0	fCantonese	git6
U+6429.0	fMandarin	jie2
U+642A.0	fUTF8	搪
U+642A.0	fCNS11643	1-642D
U+642A.0	fBig5	B765
U+642A.0	fGB	0-4C42
U+642A.0	fR/S	 64.10
U+642A.0	fTotalStrokes	 13
U+642A.0	fCangjie	QILR
U+642A.0	fFourCorner	50067
U+642A.0	fHakka	  tong2; tong5
U+642A.0	fCantonese	tong4; tong2 (also); tong5 (also)
U+642A.0	fMandarin	tang2; tang1
U+642A.0	fEnglish	[1] ward off; evade; parry; block [2] draw; paint [3] wash with little water
U+642B.0	fUTF8	搫
U+642B.0	fCNS11643	2-4A40
U+642B.0	fBig5	E1D9
U+642B.0	fGB	3-4950
U+642B.0	fR/S	 64.10
U+642B.0	fTotalStrokes	 14
U+642B.0	fCangjie	HEQ
U+642B.0	fFourCorner	27502
U+642B.0	fHakka	 pan2
U+642B.0	fCantonese	pun4; po4; bun1
U+642B.0	fMandarin	pan2; po2
U+642C.0	fUTF8	搬
U+642C.0	fCNS11643	1-6430
U+642C.0	fBig5	B768
U+642C.0	fGB	0-3061
U+642C.0	fR/S	 64.10
U+642C.0	fTotalStrokes	 13
U+642C.0	fCangjie	QHYE
U+642C.0	fFourCorner	57047
U+642C.0	fHakka	  ban1
U+642C.0	fCantonese	bun1
U+642C.0	fMandarin	ban1; pan2
U+642C.0	fEnglish	[1] transfer; move; transport; shift [2] present
U+642D.0	fUTF8	搭
U+642D.0	fCNS11643	1-642E
U+642D.0	fBig5	B766
U+642D.0	fGB	0-346E
U+642D.0	fR/S	 64.10
U+642D.0	fTotalStrokes	 13
U+642D.0	fCangjie	QTOR
U+642D.0	fFourCorner	54061
U+642D.0	fHakka	  dap7
U+642D.0	fCantonese	daap3
U+642D.0	fMandarin	da1; ta4; ta3
U+642D.0	fEnglish	[1] join together; attach to; add to [2] hand over [3] raise; build (a shed, etc.); put up; pitch (a tent, etc.) [4] travel by; take (a passage on a bus, train, boat, etc.) [5] help; rescue [6] short garment [7] cover
U+642E.0	fUTF8	搮
U+642E.0	fCNS11643	2-433F
U+642E.0	fBig5	DDB9
U+642E.0	fGB	E-2B66
U+642E.0	fR/S	 64.10
U+642E.0	fTotalStrokes	 13
U+642E.0	fCangjie	QMWD
U+642E.0	fFourCorner	51094
U+642E.0	fHakka	 lit8
U+642E.0	fCantonese	leot6
U+642E.0	fMandarin	li4
U+642E.0	fEnglish	 smooth by drawing through the hands
U+642F.0	fUTF8	搯
U+642F.0	fCNS11643	2-4336
U+642F.0	fBig5	DDB0
U+642F.0	fGB	5-4D29
U+642F.0	fR/S	 64.10
U+642F.0	fTotalStrokes	 13
U+642F.0	fCangjie	QBHX
U+642F.0	fFourCorner	52077
U+642F.0	fHakka	  tau1
U+642F.0	fCantonese	tou1; tou4
U+642F.0	fMandarin	tao1
U+642F.0	fEnglish	 take out (money, cigarettes, etc.); pull out
U+6430.0	fUTF8	搰
U+6430.0	fCNS11643	2-4332
U+6430.0	fBig5	DDAC
U+6430.0	fGB	3-4A6B
U+6430.0	fR/S	 64.10
U+6430.0	fTotalStrokes	 13
U+6430.0	fCangjie	QBBB
U+6430.0	fFourCorner	57027
U+6430.0	fHakka	 gut7; gwut7
U+6430.0	fCantonese	wat6; fat1
U+6430.0	fMandarin	hu2; ku1; gu3
U+6430.0	fEnglish	 dig turbid exert
U+6431.0	fUTF8	搱
U+6431.0	fCNS11643	3-4136
U+6431.0	fGB	3-4B26
U+6431.0	fR/S	 64.12; 64.10
U+6431.0	fTotalStrokes	 13; 15
U+6431.0	fCangjie	QSYJ
U+6431.0	fFourCorner	57040
U+6431.0	fCantonese	zi6
U+6431.0	fMandarin	zhi4
U+6432.0	fUTF8	搲
U+6432.0	fCNS11643	3-4133
U+6432.0	fGB	3-4B23
U+6432.0	fR/S	  64.10
U+6432.0	fTotalStrokes	 13
U+6432.0	fCangjie	QJCO
U+6432.0	fFourCorner	53030
U+6432.0	fHakka	 wa1; wa6; wa5; va1; va5; na5
U+6432.0	fCantonese	waa2; we2
U+6432.0	fMandarin	wa1
U+6432.0	fEnglish	 seize; grasp
U+6433.0	fUTF8	搳
U+6433.0	fCNS11643	2-4327
U+6433.0	fBig5	DDA1
U+6433.0	fGB	5-4D31
U+6433.0	fR/S	 64.10
U+6433.0	fTotalStrokes	 13
U+6433.0	fCangjie	QJQR
U+6433.0	fFourCorner	53061
U+6433.0	fCantonese	waak6; hat6; waa1
U+6433.0	fMandarin	xia2; ya4; hua2
U+6434.0	fUTF8	搴
U+6434.0	fCNS11643	1-687A
U+6434.0	fBig5	BA53
U+6434.0	fGB	0-653A
U+6434.0	fR/S	 64.10
U+6434.0	fTotalStrokes	 14
U+6434.0	fCangjie	JTCQ
U+6434.0	fFourCorner	30502
U+6434.0	fHakka	 kien1; kian1; ken1
U+6434.0	fCantonese	hin1; gin2
U+6434.0	fMandarin	qian1; jian3; sun4; xian3
U+6434.0	fEnglish	 pull; pluck up
U+6435.0	fUTF8	搵
U+6435.0	fCNS11643	2-4335
U+6435.0	fBig5	DDAF
U+6435.0	fGB	E-2B67
U+6435.0	fR/S	 64.10
U+6435.0	fTotalStrokes	 13
U+6435.0	fCangjie	QWOT
U+6435.0	fFourCorner	56017
U+6435.0	fHakka	  wun5; wun3; vun5; wut8; ziam3
U+6435.0	fCantonese	wan2; wan3; man5
U+6435.0	fMandarin	wen4; yun1; yun3
U+6435.0	fEnglish	[1] wipe off (tears) [2] press with fingers
U+6436.0	fUTF8	搶
U+6436.0	fCNS11643	1-6435
U+6436.0	fBig5	B76D
U+6436.0	fGB	1-4740
U+6436.0	fR/S	 64.10
U+6436.0	fTotalStrokes	 13
U+6436.0	fCangjie	QOIR
U+6436.0	fFourCorner	58067
U+6436.0	fHakka	  ciong3; cong3; cong5
U+6436.0	fCantonese	coeng2; coeng1; cong2; cong1
U+6436.0	fMandarin	qiang3; cheng1; chuang3; cang1; chuan3; qiang4
U+6436.0	fEnglish	[1] plunder; rob; take by force; snatch; loot [2] do something in haste, as in an emergency; rush [3] oppose [4] try to beat others in performance; vie for; scramble for a ball, position, etc.
U+6436.1	fUTF8	搶
U+6436.1	fCNS11643	1-6435
U+6436.1	fBig5	B76D
U+6436.1	fGB	1-4740
U+6436.1	fR/S	 64.10
U+6436.1	fTotalStrokes	 13
U+6436.1	fCangjie	QOIR
U+6436.1	fFourCorner	58067
U+6436.1	fMandarin	qiang1
U+6436.1	fEnglish	[1] head (winds); adverse [2] strike; hit; knock
U+6437.0	fUTF8	搷
U+6437.0	fCNS11643	2-432D
U+6437.0	fBig5	DDA7
U+6437.0	fGB	5-4D21
U+6437.0	fR/S	 64.10
U+6437.0	fTotalStrokes	 13
U+6437.0	fCangjie	QJBC
U+6437.0	fFourCorner	54081
U+6437.0	fHakka	 tien5; tien2
U+6437.0	fCantonese	tin4
U+6437.0	fMandarin	tian2; chen1; shen1
U+6437.0	fEnglish	 beat winnow
U+6438.0	fUTF8	搸
U+6438.0	fCNS11643	3-4134
U+6438.0	fGB	5-4C78
U+6438.0	fR/S	 64.10
U+6438.0	fTotalStrokes	 13
U+6438.0	fCangjie	QQKD
U+6438.0	fFourCorner	55090
U+6438.0	fCantonese	zeon1
U+6438.0	fMandarin	zhen1
U+6439.0	fUTF8	搹
U+6439.0	fCNS11643	2-432C
U+6439.0	fBig5	DDA6
U+6439.0	fGB	5-4D23
U+6439.0	fR/S	 64.10
U+6439.0	fTotalStrokes	 13
U+6439.0	fCangjie	QMRB
U+6439.0	fFourCorner	51027
U+6439.0	fCantonese	ngak1; ak1 (also); gaak3 (also); aak1
U+6439.0	fMandarin	e4; ge2
U+643A.0	fUTF8	携
U+643A.0	fCNS11643	3-4148
U+643A.0	fGB	0-502F
U+643A.0	fR/S	 64.10; 64.11
U+643A.0	fTotalStrokes	 13; 14
U+643A.0	fCangjie	QOGS
U+643A.0	fFourCorner	50020
U+643A.0	fHakka	 hi1; kui2
U+643A.0	fCantonese	kwai4; kaa1; kaat3; gaat3; pik1; jip6; pek6; maak3; baai1; ngou1; ou1 (also); geoi3
U+643A.0	fMandarin	xi1; xi2; xie2
U+643A.0	fEnglish	 lead by hand; take with carry
U+643B.0	fUTF8	搻
U+643B.0	fCNS11643	3-4725
U+643B.0	fGB	5-4B4C
U+643B.0	fR/S	 64.10
U+643B.0	fTotalStrokes	 14
U+643B.0	fCangjie	ORQQ
U+643B.0	fFourCorner	80550
U+643B.0	fHakka	  nit8; lak8; lit7; lit8
U+643B.0	fCantonese	nik1; nok6 (also); lik1 (also); lok6 (also)
U+643B.0	fMandarin	nuo4; ne4; ni4
U+643B.0	fEnglish	same as &#x6426;
U+643C.0	fUTF8	搼
U+643C.0	fCNS11643	3-4142
U+643C.0	fGB	5-4D30
U+643C.0	fR/S	 64.10
U+643C.0	fTotalStrokes	 13
U+643C.0	fCangjie	QFQQ
U+643C.0	fFourCorner	59050
U+643C.0	fHakka	  kien2;  ken2; k'ian3; kian2
U+643C.0	fCantonese	kyun4; kyun2 (also)
U+643C.0	fMandarin	quan2; juan1; quan1; quan3
U+643C.0	fEnglish	same as &#x62F3;
U+643D.0	fUTF8	搽
U+643D.0	fCNS11643	1-642F
U+643D.0	fBig5	B767
U+643D.0	fGB	0-326B
U+643D.0	fR/S	 64.10; 64.9
U+643D.0	fTotalStrokes	 13
U+643D.0	fCangjie	QTOD
U+643D.0	fFourCorner	54094
U+643D.0	fHakka	  ca2; ca3
U+643D.0	fCantonese	caa4
U+643D.0	fMandarin	cha2; ca1; ca4
U+643D.0	fEnglish	 rub on (ointment, etc.); smear; anoint; paint
U+643E.0	fUTF8	搾
U+643E.0	fCNS11643	1-642B
U+643E.0	fBig5	B763
U+643E.0	fGB	E-2B68
U+643E.0	fR/S	 64.10
U+643E.0	fTotalStrokes	 13
U+643E.0	fCangjie	QJCS
U+643E.0	fFourCorner	53011
U+643E.0	fHakka	 za5
U+643E.0	fCantonese	zaa3
U+643E.0	fMandarin	zha4
U+643E.0	fEnglish	 press (for juice or oil); extract; squeeze; wring
U+643F.0	fUTF8	搿
U+643F.0	fCNS11643	2-4A55
U+643F.0	fBig5	E1EE
U+643F.0	fGB	0-6B21
U+643F.0	fR/S	 64.10
U+643F.0	fTotalStrokes	 14; 13
U+643F.0	fCangjie	QORQ
U+643F.0	fFourCorner	22550
U+643F.0	fHakka	 bung3
U+643F.0	fCantonese	gaap3; gap3 (also)
U+643F.0	fMandarin	ge2
U+643F.0	fEnglish	 this; that
U+6440.0	fUTF8	摀
U+6440.0	fCNS11643	2-4339
U+6440.0	fBig5	DDB3
U+6440.0	fGB	E-2B69
U+6440.0	fR/S	 64.10
U+6440.0	fTotalStrokes	 13
U+6440.0	fCangjie	QHRF
U+6440.0	fFourCorner	57027
U+6440.0	fHakka	 wu3; vu3
U+6440.0	fCantonese	wu2; ng6 (also)
U+6440.0	fMandarin	wu3
U+6440.0	fEnglish	[1] cover; conceal; hide [2] put into an airtight container (in cooking)
U+6441.0	fUTF8	摁
U+6441.0	fCNS11643	2-4334
U+6441.0	fBig5	DDAE
U+6441.0	fGB	0-5E74
U+6441.0	fR/S	 64.10
U+6441.0	fTotalStrokes	 13
U+6441.0	fCangjie	QWKP
U+6441.0	fFourCorner	56030
U+6441.0	fCantonese	on3; ngon3 (also); jan1
U+6441.0	fMandarin	en4
U+6441.0	fEnglish	[1] press (a doorbell, etc.) [2] delay; hold
U+6442.0	fUTF8	摂
U+6442.0	fGB	E-2B6A
U+6442.0	fR/S	 64.10
U+6442.0	fTotalStrokes	 13
U+6442.0	fCangjie	QSJO
U+6442.0	fHakka	 ngiap7
U+6442.0	fCantonese	sip3; nip6; lip6
U+6442.0	fMandarin	she4; nie2; nie4; sha4; zhe2
U+6442.0	fEnglish	simplified form of &#x651D;
U+6443.0	fUTF8	摃
U+6443.0	fCNS11643	2-4328
U+6443.0	fBig5	DDA2
U+6443.0	fGB	E-2B6B
U+6443.0	fR/S	 64.10
U+6443.0	fTotalStrokes	 13
U+6443.0	fCangjie	QMBC
U+6443.0	fFourCorner	51082
U+6443.0	fCantonese	kong1; gong1
U+6443.0	fMandarin	kang2; gong4; gang1
U+6444.0	fUTF8	摄
U+6444.0	fGB	0-4963
U+6444.0	fR/S	 64.10
U+6444.0	fTotalStrokes	 13
U+6444.0	fCangjie	XQSJE
U+6444.0	fHakka	 ngiap7
U+6444.0	fCantonese	sip3; zip3; nip6; lip6 (also)
U+6444.0	fMandarin	she4; nie2; nie4; sha4; zhe2
U+6444.0	fEnglish	simplified form of &#x651D;
U+6445.0	fUTF8	摅
U+6445.0	fGB	0-5E73
U+6445.0	fR/S	 64.10
U+6445.0	fTotalStrokes	 13
U+6445.0	fCangjie	XQYPP
U+6445.0	fHakka	 shu1
U+6445.0	fCantonese	syu1
U+6445.0	fMandarin	shu1; chu1
U+6445.0	fEnglish	simplified form of &#x6504;
U+6446.0	fUTF8	摆
U+6446.0	fCNS11643	3-4144
U+6446.0	fGB	0-305A
U+6446.0	fR/S	 64.10
U+6446.0	fTotalStrokes	 13
U+6446.0	fCangjie	QWLI
U+6446.0	fFourCorner	56030
U+6446.0	fHakka	 bai3
U+6446.0	fCantonese	baai2; bei1
U+6446.0	fMandarin	bai3
U+6446.0	fEnglish	simplified form of &#x64FA;
U+6447.0	fUTF8	摇
U+6447.0	fCNS11643	3-4145
U+6447.0	fGB	0-5221
U+6447.0	fR/S	 64.10
U+6447.0	fTotalStrokes	 13
U+6447.0	fCangjie	QBOU
U+6447.0	fFourCorner	52070
U+6447.0	fHakka	 jau2
U+6447.0	fCantonese	jiu4
U+6447.0	fMandarin	yao2
U+6447.0	fEnglish	 wag; swing; wave shake scull
U+6448.0	fUTF8	摈
U+6448.0	fGB	0-3177
U+6448.0	fR/S	 64.10
U+6448.0	fTotalStrokes	 13
U+6448.0	fCangjie	QJOC
U+6448.0	fCantonese	ban3
U+6448.0	fMandarin	bin4; bin1
U+6448.0	fEnglish	simplified form of &#x64EF;
U+6449.0	fUTF8	摉
U+6449.0	fCNS11643	3-6676
U+6449.0	fGB	E-2B6C
U+6449.0	fR/S	 64.10
U+6449.0	fTotalStrokes	 13
U+6449.0	fCangjie	XQJCN
U+6449.0	fFourCorner	5302
U+6449.0	fCantonese	sau1
U+6449.0	fMandarin	sou1
U+644A.0	fUTF8	摊
U+644A.0	fGB	0-4C2F
U+644A.0	fR/S	 64.10
U+644A.0	fTotalStrokes	 13
U+644A.0	fCangjie	QEOG
U+644A.0	fHakka	 tan1
U+644A.0	fCantonese	taan1
U+644A.0	fMandarin	tan1; nan3; nan4
U+644A.0	fEnglish	simplified form of &#x6524;
U+644B.0	fUTF8	摋
U+644B.0	fCNS11643	2-4A50
U+644B.0	fBig5	E1E9
U+644B.0	fGB	3-4A2F
U+644B.0	fR/S	 64.11
U+644B.0	fTotalStrokes	 14
U+644B.0	fCangjie	QKCE
U+644B.0	fFourCorner	57047
U+644B.0	fHakka	 sat7
U+644B.0	fCantonese	saat3; sat3
U+644B.0	fMandarin	sa4; sha1; shai3; xue1
U+644B.0	fEnglish	[1] slap [2] disperse [3] cast away [4] sow
U+644C.0	fUTF8	摌
U+644C.0	fCNS11643	3-4678
U+644C.0	fGB	3-4A32
U+644C.0	fR/S	  64.11
U+644C.0	fTotalStrokes	 14
U+644C.0	fCangjie	QYHM
U+644C.0	fFourCorner	50010
U+644C.0	fCantonese	caan2
U+644C.0	fMandarin	chan3
U+644D.0	fUTF8	摍
U+644D.0	fCNS11643	2-4A41
U+644D.0	fBig5	E1DA
U+644D.0	fGB	5-4D45
U+644D.0	fR/S	 64.11
U+644D.0	fTotalStrokes	 14
U+644D.0	fCangjie	QJOA
U+644D.0	fFourCorner	53061
U+644D.0	fHakka	 suk7
U+644D.0	fCantonese	suk1
U+644D.0	fMandarin	suo1
U+644E.0	fUTF8	摎
U+644E.0	fCNS11643	2-4A4C
U+644E.0	fBig5	E1E5
U+644E.0	fGB	3-4B38
U+644E.0	fR/S	 64.11
U+644E.0	fTotalStrokes	 14
U+644E.0	fCangjie	QSMH
U+644E.0	fFourCorner	57022
U+644E.0	fHakka	 kieu1; kiu1; geu1
U+644E.0	fCantonese	gau1; lau4; kau1
U+644E.0	fMandarin	liao2; jiu1; liu2; jiao1; jiao3; nao3
U+644E.0	fEnglish	entwined (as branches of a tree)
U+644F.0	fUTF8	摏
U+644F.0	fCNS11643	3-4728
U+644F.0	fGB	5-4D37
U+644F.0	fR/S	  64.11
U+644F.0	fTotalStrokes	 14
U+644F.0	fCangjie	QQKX
U+644F.0	fFourCorner	55070
U+644F.0	fHakka	 zung1;  zhung1
U+644F.0	fCantonese	zung1; sung1
U+644F.0	fMandarin	chong1
U+644F.0	fEnglish	 pound strike against ram
U+6450.0	fUTF8	摐
U+6450.0	fCNS11643	2-4A53
U+6450.0	fBig5	E1EC
U+6450.0	fGB	3-4969
U+6450.0	fR/S	 64.11
U+6450.0	fTotalStrokes	 14
U+6450.0	fCangjie	QHOO
U+6450.0	fFourCorner	58081
U+6450.0	fHakka	  cung1
U+6450.0	fCantonese	cung1; coeng1 (also)
U+6450.0	fMandarin	chuang1; cong1
U+6450.0	fEnglish	[1] beat; strike [2] beckon
U+6451.0	fUTF8	摑
U+6451.0	fCNS11643	1-6878
U+6451.0	fBig5	BA51
U+6451.0	fGB	1-5E62
U+6451.0	fR/S	 64.11
U+6451.0	fTotalStrokes	 14
U+6451.0	fCangjie	QWIM
U+6451.0	fFourCorner	56000
U+6451.0	fHakka	 kwet8; kwet7; dok8; get7; gwet8
U+6451.0	fCantonese	gwaak3; gwok3 (also); gok3 (also)
U+6451.0	fMandarin	guo2
U+6451.0	fEnglish	 slap another on his face
U+6452.0	fUTF8	摒
U+6452.0	fCNS11643	1-5F51
U+6452.0	fBig5	B4AC
U+6452.0	fGB	0-5E70
U+6452.0	fR/S	 64.11
U+6452.0	fTotalStrokes	 14
U+6452.0	fCangjie	QSTT
U+6452.0	fFourCorner	57041
U+6452.0	fHakka	 bin5;  bin3; biang5
U+6452.0	fCantonese	bing3; bing2
U+6452.0	fMandarin	bing3; bing4
U+6452.0	fEnglish	[1] get rid of; expel; remove; dismiss; cast off; brush aside [2] arrange in order
U+6453.0	fUTF8	摓
U+6453.0	fCNS11643	2-4A51
U+6453.0	fBig5	E1EA
U+6453.0	fGB	5-4D2B
U+6453.0	fR/S	 64.11
U+6453.0	fTotalStrokes	 14
U+6453.0	fCangjie	QYHJ
U+6453.0	fFourCorner	57035
U+6453.0	fHakka	  fung2; pung2
U+6453.0	fCantonese	fung4; hung4
U+6453.0	fMandarin	feng2
U+6453.0	fEnglish	 sew; baste
U+6454.0	fUTF8	摔
U+6454.0	fCNS11643	1-6873
U+6454.0	fBig5	BA4C
U+6454.0	fGB	0-4B24
U+6454.0	fR/S	 64.11
U+6454.0	fTotalStrokes	 14
U+6454.0	fCangjie	QYIJ
U+6454.0	fFourCorner	50043
U+6454.0	fHakka	 sut7; sui1; soi1
U+6454.0	fCantonese	seot1; seoi1
U+6454.0	fMandarin	shuai1
U+6454.0	fEnglish	[1] throw to the ground; fling; break [2] get rid of; shake off (a tail, etc.) [3] fall down; stumble; trip; lose one's balance
U+6455.0	fUTF8	摕
U+6455.0	fCNS11643	3-467E
U+6455.0	fGB	3-4A62
U+6455.0	fR/S	  64.11
U+6455.0	fTotalStrokes	 14
U+6455.0	fCangjie	QKPB
U+6455.0	fFourCorner	54020
U+6455.0	fCantonese	dai3; cai3
U+6455.0	fMandarin	di4
U+6455.0	fEnglish	hit
U+6456.0	fUTF8	摖
U+6456.0	fCNS11643	3-4724
U+6456.0	fGB	3-4B34
U+6456.0	fR/S	 64.12; 64.11
U+6456.0	fTotalStrokes	 15; 14
U+6456.0	fCangjie	QBOF
U+6456.0	fFourCorner	57090
U+6456.0	fCantonese	cai3
U+6456.0	fMandarin	qi4
U+6457.0	fUTF8	摗
U+6457.0	fCNS11643	3-467A
U+6457.0	fGB	3-4B2A
U+6457.0	fR/S	  64.11
U+6457.0	fTotalStrokes	 14
U+6457.0	fCangjie	QDLO
U+6457.0	fFourCorner	57082
U+6457.0	fCantonese	cuk1
U+6457.0	fMandarin	sou1
U+6458.0	fUTF8	摘
U+6458.0	fCNS11643	1-6872
U+6458.0	fBig5	BA4B
U+6458.0	fGB	0-552A
U+6458.0	fR/S	 64.11
U+6458.0	fTotalStrokes	 14
U+6458.0	fCangjie	QYCB
U+6458.0	fFourCorner	50027
U+6458.0	fHakka	  zak7; dit7
U+6458.0	fCantonese	zaak6
U+6458.0	fMandarin	zhai1; zhe2; ti4
U+6458.0	fEnglish	[1] take off (one's hat, etc.) [2] choose; select [3] jot down (down) [4] expose; unveil (a conspiracy, etc.)
U+6459.0	fUTF8	摙
U+6459.0	fCNS11643	2-4A58
U+6459.0	fBig5	E1F1
U+6459.0	fGB	3-4A3A
U+6459.0	fR/S	 64.11
U+6459.0	fTotalStrokes	 14
U+6459.0	fCangjie	QYJJ
U+6459.0	fFourCorner	55030
U+6459.0	fHakka	 lien5;  lien3
U+6459.0	fCantonese	lin5; lin2; lin4; lin6
U+6459.0	fMandarin	lian3; lian4
U+6459.0	fEnglish	 transport remove; take
U+645A.0	fUTF8	摚
U+645A.0	fCNS11643	4-435D
U+645A.0	fGB	3-4B31
U+645A.0	fR/S	  64.11
U+645A.0	fTotalStrokes	 14
U+645A.0	fCangjie	QFBG
U+645A.0	fFourCorner	59010
U+645A.0	fCantonese	tong3
U+645A.0	fMandarin	tang2; cheng1
U+645A.0	fEnglish	to pull open (a drawer)
U+645B.0	fUTF8	摛
U+645B.0	fCNS11643	2-4A42
U+645B.0	fBig5	E1DB
U+645B.0	fGB	E-2B6D
U+645B.0	fR/S	 64.11
U+645B.0	fTotalStrokes	 14
U+645B.0	fCangjie	QYUB
U+645B.0	fFourCorner	50027
U+645B.0	fHakka	 li2
U+645B.0	fCantonese	ci1; lei4 (also)
U+645B.0	fMandarin	chi1; li2
U+645B.0	fEnglish	[1] spread (said of fame, news, etc.) [2] brandish (a pen) [3] known widely
U+645C.0	fUTF8	摜
U+645C.0	fCNS11643	2-4A4F
U+645C.0	fBig5	E1E8
U+645C.0	fGB	1-5E68
U+645C.0	fR/S	 64.11
U+645C.0	fTotalStrokes	 14
U+645C.0	fCangjie	QWJC
U+645C.0	fFourCorner	55086
U+645C.0	fHakka	 gon5; kwan5; gwon5
U+645C.0	fCantonese	gwaan3
U+645C.0	fMandarin	guan4
U+645C.0	fEnglish	[1] throw to the ground [2] be accustomed to; be used to [3] throw
U+645D.0	fUTF8	摝
U+645D.0	fCNS11643	2-4A43
U+645D.0	fBig5	E1DC
U+645D.0	fGB	3-4B35
U+645D.0	fR/S	 64.11
U+645D.0	fTotalStrokes	 14
U+645D.0	fCangjie	QIXP
U+645D.0	fFourCorner	50011
U+645D.0	fHakka	 luk7;  luk8; liuk7
U+645D.0	fCantonese	luk6; luk1
U+645D.0	fMandarin	lu4; long4; lu3
U+645D.0	fEnglish	another form of the character for pulley, etc.
U+645E.0	fUTF8	摞
U+645E.0	fCNS11643	2-4A4E
U+645E.0	fBig5	E1E7
U+645E.0	fGB	0-5E7B
U+645E.0	fR/S	 64.11
U+645E.0	fTotalStrokes	 14
U+645E.0	fCangjie	QWVF
U+645E.0	fFourCorner	56093
U+645E.0	fHakka	  lo2; no2
U+645E.0	fCantonese	lo3; lo6 (also); lo4
U+645E.0	fMandarin	luo4; luo2
U+645E.0	fEnglish	 pile up
U+645F.0	fUTF8	摟
U+645F.0	fCNS11643	1-6876
U+645F.0	fBig5	BA4F
U+645F.0	fGB	1-4227
U+645F.0	fR/S	 64.11
U+645F.0	fTotalStrokes	 14
U+645F.0	fCangjie	QLLV
U+645F.0	fFourCorner	55044
U+645F.0	fHakka	 leu5
U+645F.0	fMandarin	lou2
U+645F.0	fEnglish	 drag; pull; drag away
U+645F.1	fUTF8	摟
U+645F.1	fCNS11643	1-6876
U+645F.1	fBig5	BA4F
U+645F.1	fGB	1-4227
U+645F.1	fR/S	 64.11
U+645F.1	fTotalStrokes	 14
U+645F.1	fCangjie	QLLV
U+645F.1	fFourCorner	55044
U+645F.1	fHakka	  leu3;  leu2; liu1; leu1; liu2
U+645F.1	fCantonese	lau5; lau1; lau2; lau4
U+645F.1	fMandarin	lou1
U+645F.1	fEnglish	[1] hold up; tuck up [2] squeeze; extort (money, etc.) [3] gather up; collect; rake together
U+645F.2	fUTF8	摟
U+645F.2	fCNS11643	1-6876
U+645F.2	fBig5	BA4F
U+645F.2	fGB	1-4227
U+645F.2	fR/S	 64.11
U+645F.2	fTotalStrokes	 14
U+645F.2	fCangjie	QLLV
U+645F.2	fFourCorner	55044
U+645F.2	fMandarin	lou3
U+645F.2	fEnglish	 hold in the arms; embrace; hug
U+6460.0	fUTF8	摠
U+6460.0	fCNS11643	2-4A52
U+6460.0	fBig5	E1EB
U+6460.0	fGB	5-4D3F
U+6460.0	fR/S	 64.11
U+6460.0	fTotalStrokes	 14
U+6460.0	fCangjie	QHWP
U+6460.0	fFourCorner	56030
U+6460.0	fHakka	  zung3
U+6460.0	fCantonese	zung2
U+6460.0	fMandarin	zong3; zong1
U+6460.0	fEnglish	general
U+6461.0	fUTF8	摡
U+6461.0	fCNS11643	2-3C27
U+6461.0	fBig5	D962
U+6461.0	fGB	3-4A76
U+6461.0	fR/S	 64.11; 64.12
U+6461.0	fTotalStrokes	 14
U+6461.0	fCangjie	QAIU
U+6461.0	fFourCorner	51011
U+6461.0	fHakka	 koi3;  kai3
U+6461.0	fCantonese	goi3; hei3
U+6461.0	fMandarin	xi4; gai4
U+6461.0	fEnglish	 irrigate flood water flowing scour
U+6462.0	fUTF8	摢
U+6462.0	fCNS11643	3-4721
U+6462.0	fGB	3-4B30
U+6462.0	fR/S	  64.11
U+6462.0	fTotalStrokes	 14
U+6462.0	fCangjie	QYPD
U+6462.0	fFourCorner	51040
U+6462.0	fCantonese	wu6
U+6462.0	fMandarin	hu4
U+6463.0	fUTF8	摣
U+6463.0	fCNS11643	4-435E
U+6463.0	fGB	3-4B2F
U+6463.0	fR/S	 64.11
U+6463.0	fTotalStrokes	 14
U+6463.0	fCangjie	QYPM
U+6463.0	fFourCorner	51010
U+6463.0	fHakka	 za1
U+6463.0	fCantonese	zaa1
U+6463.0	fMandarin	zha1; za1
U+6463.0	fEnglish	[1] handful [2] pick up with the fingers [3] seize
U+6464.0	fUTF8	摤
U+6464.0	fCNS11643	3-472A
U+6464.0	fGB	5-4D3B
U+6464.0	fR/S	  64.11
U+6464.0	fTotalStrokes	 14
U+6464.0	fCangjie	QKKK
U+6464.0	fFourCorner	54030
U+6464.0	fCantonese	coeng2
U+6464.0	fMandarin	chuang3
U+6465.0	fUTF8	摥
U+6465.0	fCNS11643	2-4A59
U+6465.0	fBig5	E1F2
U+6465.0	fGB	E-2B6E
U+6465.0	fR/S	 64.11
U+6465.0	fTotalStrokes	 14
U+6465.0	fCangjie	QOAH
U+6465.0	fFourCorner	58027
U+6465.0	fHakka	 tong5
U+6465.0	fCantonese	tong3
U+6465.0	fMandarin	tang4
U+6465.0	fEnglish	[1] part [2] separate
U+6466.0	fUTF8	摦
U+6466.0	fCNS11643	2-4A4A
U+6466.0	fBig5	E1E3
U+6466.0	fGB	3-4B2E
U+6466.0	fR/S	 64.11
U+6466.0	fTotalStrokes	 14
U+6466.0	fCangjie	QKSO
U+6466.0	fFourCorner	52030
U+6466.0	fCantonese	waa6
U+6466.0	fMandarin	hua4; hua2
U+6466.0	fEnglish	wide
U+6467.0	fUTF8	摧
U+6467.0	fCNS11643	1-6879
U+6467.0	fBig5	BA52
U+6467.0	fGB	0-345D
U+6467.0	fR/S	 64.11
U+6467.0	fTotalStrokes	 14
U+6467.0	fCangjie	QUOG
U+6467.0	fFourCorner	52014
U+6467.0	fHakka	  cui1
U+6467.0	fCantonese	ceoi1; ceoi4
U+6467.0	fMandarin	cui1; cuo4; zui1
U+6467.0	fEnglish	[1] destroy; break; injure; harm; smash [2] damp [3] cause to cease; extinguish [4] be sad and sorrowful; grieve
U+6468.0	fUTF8	摨
U+6468.0	fCNS11643	2-5061
U+6468.0	fBig5	E5BA
U+6468.0	fGB	5-4D57
U+6468.0	fR/S	 64.11; 64.12
U+6468.0	fTotalStrokes	 14
U+6468.0	fCangjie	QSEQ
U+6468.0	fFourCorner	57053
U+6468.0	fCantonese	naai4; laai4 (also); zi6
U+6468.0	fMandarin	nai2; zhi4
U+6469.0	fUTF8	摩
U+6469.0	fCNS11643	1-6C77
U+6469.0	fBig5	BCAF
U+6469.0	fGB	0-4426
U+6469.0	fR/S	 64.11
U+6469.0	fTotalStrokes	 15; 14
U+6469.0	fCangjie	IDQ
U+6469.0	fFourCorner	00252
U+6469.0	fHakka	  mo2; mo1; mo3
U+6469.0	fCantonese	mo1; maa1; mo4
U+6469.0	fMandarin	mo2; ma1; mi2; mo4
U+6469.0	fEnglish	[1] rub; scour; chafe [2] friction [3] feel with the hand [4] work and encourage each other (especially in study)
U+646A.0	fUTF8	摪
U+646A.0	fCNS11643	3-4723
U+646A.0	fGB	5-4C6C
U+646A.0	fR/S	  64.11
U+646A.0	fTotalStrokes	 14
U+646A.0	fCangjie	QVMI
U+646A.0	fFourCorner	57042
U+646A.0	fCantonese	zoeng1
U+646A.0	fMandarin	jiang1
U+646B.0	fUTF8	摫
U+646B.0	fCNS11643	2-4A57
U+646B.0	fBig5	E1F0
U+646B.0	fGB	5-4C43
U+646B.0	fR/S	 64.11
U+646B.0	fTotalStrokes	 14
U+646B.0	fCangjie	QQOU
U+646B.0	fFourCorner	56010
U+646B.0	fHakka	 gui1; gwi1; kwui1
U+646B.0	fCantonese	kwai1
U+646B.0	fMandarin	gui1
U+646B.0	fEnglish	 cut cloth for making clothes
U+646C.0	fUTF8	摬
U+646C.0	fCNS11643	2-4A56
U+646C.0	fBig5	E1EF
U+646C.0	fGB	3-4B36
U+646C.0	fR/S	 64.11
U+646C.0	fTotalStrokes	 14
U+646C.0	fCangjie	QYTU
U+646C.0	fFourCorner	50016
U+646C.0	fCantonese	jing2
U+646C.0	fMandarin	ying4; jiang3; yang4; ying3
U+646D.0	fUTF8	摭
U+646D.0	fCNS11643	1-687B
U+646D.0	fBig5	BA54
U+646D.0	fGB	0-5E7D
U+646D.0	fR/S	 64.11
U+646D.0	fTotalStrokes	 14
U+646D.0	fCangjie	QITF
U+646D.0	fFourCorner	50037
U+646D.0	fHakka	 chak7; zhit7; zak7; tsiit7; zhak7
U+646D.0	fCantonese	zek3
U+646D.0	fMandarin	zhi2; tuo4
U+646D.0	fEnglish	 pick up from the ground; take up; gather; collect
U+646E.0	fUTF8	摮
U+646E.0	fCNS11643	2-5054
U+646E.0	fBig5	E5AD
U+646E.0	fGB	3-494F
U+646E.0	fR/S	 64.11
U+646E.0	fTotalStrokes	 15
U+646E.0	fCangjie	GKQ
U+646E.0	fFourCorner	58502
U+646E.0	fHakka	 ngau5
U+646E.0	fCantonese	ngou4; ou4 (also)
U+646E.0	fMandarin	ao2; qiao1; yao2
U+646E.0	fEnglish	 rattle shake smite
U+646F.0	fUTF8	摯
U+646F.0	fCNS11643	1-6C78
U+646F.0	fBig5	BCB0
U+646F.0	fGB	1-563F
U+646F.0	fR/S	 64.11
U+646F.0	fTotalStrokes	 15
U+646F.0	fCangjie	GIQ
U+646F.0	fFourCorner	44502
U+646F.0	fHakka	 zhi5; zih5; zi5
U+646F.0	fCantonese	zi3
U+646F.0	fMandarin	zhi4; zhi2
U+646F.0	fEnglish	[1] sincere; warm; cordial [2] a Chinese family name
U+6470.0	fUTF8	摰
U+6470.0	fCNS11643	2-5055
U+6470.0	fBig5	E5AE
U+6470.0	fGB	3-4951
U+6470.0	fR/S	 64.11
U+6470.0	fTotalStrokes	 15
U+6470.0	fCangjie	GIQ
U+6470.0	fFourCorner	45502
U+6470.0	fCantonese	jit6; zi3
U+6470.0	fMandarin	nie4; chi4
U+6470.0	fEnglish	 seize with the hand grasp advance breakdown
U+6471.0	fUTF8	摱
U+6471.0	fCNS11643	3-4722
U+6471.0	fGB	3-4B32
U+6471.0	fR/S	 64.11
U+6471.0	fTotalStrokes	 14
U+6471.0	fCangjie	QAWE
U+6471.0	fFourCorner	56040
U+6471.0	fCantonese	maan1
U+6471.0	fMandarin	man2
U+6471.0	fEnglish	to climb, hold on to, cling to; to pull the trigger on a gun
U+6472.0	fUTF8	摲
U+6472.0	fCNS11643	2-4A46
U+6472.0	fBig5	E1DF
U+6472.0	fGB	3-4A4E
U+6472.0	fR/S	 64.11
U+6472.0	fTotalStrokes	 14
U+6472.0	fCangjie	QJJL
U+6472.0	fFourCorner	52021
U+6472.0	fHakka	 cam6; cam5; cam1
U+6472.0	fCantonese	saam3; caam3; saan3; caam1
U+6472.0	fMandarin	shan4; chan4; zhan4; jian4; shan1; shan3
U+6472.0	fEnglish	raze
U+6473.0	fUTF8	摳
U+6473.0	fCNS11643	2-4A47
U+6473.0	fBig5	E1E0
U+6473.0	fGB	1-3F59
U+6473.0	fR/S	 64.11
U+6473.0	fTotalStrokes	 14
U+6473.0	fCangjie	QSRR
U+6473.0	fFourCorner	51016
U+6473.0	fHakka	 ki1; k'ieu1; k'ieu5; keu5; keu1; kiu1; keu2
U+6473.0	fCantonese	kau1
U+6473.0	fMandarin	kou1; ou3; qu1
U+6473.0	fEnglish	[1] raise; lift up [2] grope for [3] inquire; into; delve into [4] dig with fingers (or something pointed) [5] put something in; mix something with
U+6474.0	fUTF8	摴
U+6474.0	fCNS11643	2-4A44
U+6474.0	fBig5	E1DD
U+6474.0	fGB	5-4D3C
U+6474.0	fR/S	 64.11
U+6474.0	fTotalStrokes	 14
U+6474.0	fCangjie	QMBS
U+6474.0	fFourCorner	51027
U+6474.0	fHakka	 ki1; chi1; cih1; shu1
U+6474.0	fCantonese	syu1
U+6474.0	fMandarin	shu1; chi1; chu1
U+6474.0	fEnglish	[1] comfortable; easy [2] an ancient Chinese gambling game roughly resembling today's dice game
U+6475.0	fUTF8	摵
U+6475.0	fCNS11643	2-4A49
U+6475.0	fBig5	E1E2
U+6475.0	fGB	3-4B2D
U+6475.0	fR/S	 64.11
U+6475.0	fTotalStrokes	 14
U+6475.0	fCangjie	QIHF
U+6475.0	fFourCorner	53050
U+6475.0	fHakka	  cit7; cuk7; sit7
U+6475.0	fCantonese	saak3; cik1; sik1
U+6475.0	fMandarin	suo3; cu4; she4; mi2; suo1
U+6475.0	fEnglish	[1] the rustling of falling leaves [2] mournful
U+6476.0	fUTF8	摶
U+6476.0	fCNS11643	2-4A45
U+6476.0	fBig5	E1DE
U+6476.0	fGB	1-5E52
U+6476.0	fR/S	 64.11
U+6476.0	fTotalStrokes	 14
U+6476.0	fCangjie	QJII
U+6476.0	fFourCorner	55043
U+6476.0	fHakka	  ton2; ton3
U+6476.0	fCantonese	tyun4; zyun1
U+6476.0	fMandarin	tuan2; zhuan1; gua1; zhuan4
U+6476.0	fEnglish	[1] roll around with the hand; knead [2] rely on [3] take or follow (a trail, etc.)
U+6477.0	fUTF8	摷
U+6477.0	fCNS11643	2-4A5A
U+6477.0	fBig5	E1F3
U+6477.0	fGB	5-4D48
U+6477.0	fR/S	 64.11
U+6477.0	fTotalStrokes	 14
U+6477.0	fCangjie	QVVD
U+6477.0	fFourCorner	52094
U+6477.0	fHakka	 cau1
U+6477.0	fCantonese	ziu6; ciu6 (also); caau3
U+6477.0	fMandarin	jiao3; chao1; lao2; liao2; zhao1
U+6477.0	fEnglish	to search for, look for; to beat up
U+6478.0	fUTF8	摸
U+6478.0	fCNS11643	1-6875
U+6478.0	fBig5	BA4E
U+6478.0	fGB	0-437E
U+6478.0	fR/S	 64.11
U+6478.0	fTotalStrokes	 14
U+6478.0	fCangjie	QTAK
U+6478.0	fFourCorner	54034
U+6478.0	fHakka	  mo1; mu2; mia1; mia3; mio1; mok7
U+6478.0	fCantonese	mo2; mou4; mok3; mok6
U+6478.0	fMandarin	mo1; mo2; mao1; mu2
U+6478.0	fEnglish	[1] gently touch with the hand; caress; feel or touch lightly with fingers [2] grope [3] try to find out; feel out [4] seek after; try to get at
U+6478.1	fUTF8	摸
U+6478.1	fCNS11643	1-6875
U+6478.1	fBig5	BA4E
U+6478.1	fGB	0-437E
U+6478.1	fR/S	 64.11
U+6478.1	fTotalStrokes	 14
U+6478.1	fCangjie	QTAK
U+6478.1	fFourCorner	54034
U+6478.1	fMandarin	mo5
U+6479.0	fUTF8	摹
U+6479.0	fCNS11643	1-6C79
U+6479.0	fBig5	BCB1
U+6479.0	fGB	0-4421
U+6479.0	fR/S	 64.11
U+6479.0	fTotalStrokes	 15
U+6479.0	fCangjie	TAKQ
U+6479.0	fFourCorner	44502
U+6479.0	fHakka	 mu2;  mo1; mo2
U+6479.0	fCantonese	mou4
U+6479.0	fMandarin	mo2; mu2; mu4
U+6479.0	fEnglish	[1] trace; copy; duplicate [2] pattern after; model after; imitate
U+647A.0	fUTF8	摺
U+647A.0	fCNS11643	1-6877
U+647A.0	fBig5	BA50
U+647A.0	fGB	0-5F21
U+647A.0	fR/S	 64.11
U+647A.0	fTotalStrokes	 14
U+647A.0	fCangjie	QSMA
U+647A.0	fFourCorner	57062
U+647A.0	fHakka	  zap7; zhap7
U+647A.0	fCantonese	zip3; zaap3; zip2 (also); zit3
U+647A.0	fMandarin	zhe2; la1; zhe3
U+647A.0	fEnglish	[1] fold (paper, etc.); plait [2] folder; folded brochure [3] curved and winding [4] pull and break
U+647B.0	fUTF8	摻
U+647B.0	fCNS11643	1-687C
U+647B.0	fBig5	BA55
U+647B.0	fGB	1-3274
U+647B.0	fR/S	 64.11
U+647B.0	fTotalStrokes	 14
U+647B.0	fCangjie	QIIH
U+647B.0	fFourCorner	53022
U+647B.0	fHakka	 cam1
U+647B.0	fCantonese	caam1; saam1; cam3; cam1; saam2; caam4
U+647B.0	fMandarin	shan3; can4; xian1; shan1; chan1; can1; sen1; san1; san4
U+647B.0	fEnglish	 a delicate hand mix; blend; adulterate
U+647C.0	fUTF8	摼
U+647C.0	fCNS11643	3-467B
U+647C.0	fGB	3-4A3C
U+647C.0	fR/S	  64.11
U+647C.0	fTotalStrokes	 14
U+647C.0	fCangjie	QSEG
U+647C.0	fFourCorner	57010
U+647C.0	fHakka	 gien1; kien1; gen1; ken1
U+647C.0	fCantonese	hang1
U+647C.0	fMandarin	keng1
U+647C.0	fEnglish	[1] thump [2] but against
U+647D.0	fUTF8	摽
U+647D.0	fCNS11643	2-4A48
U+647D.0	fBig5	E1E1
U+647D.0	fGB	3-4B2C
U+647D.0	fR/S	 64.11
U+647D.0	fTotalStrokes	 14
U+647D.0	fCangjie	QMWF
U+647D.0	fFourCorner	51091
U+647D.0	fHakka	 piau3; piau1; biau1; peu3
U+647D.0	fCantonese	piu1 (also); biu1; piu5; piu3
U+647D.0	fMandarin	piao1; biao1; biao4; pao1; pi1; piao4
U+647D.0	fEnglish	[1] motion (somebody) to go out [2] strike [3] high; lofty [4] to glorify [5] razor of a sword [6] throw away; give up; abandon
U+647D.1	fUTF8	摽
U+647D.1	fCNS11643	2-4A48
U+647D.1	fBig5	E1E1
U+647D.1	fGB	3-4B2C
U+647D.1	fR/S	 64.11
U+647D.1	fTotalStrokes	 14
U+647D.1	fCangjie	QMWF
U+647D.1	fFourCorner	51091
U+647D.1	fMandarin	piao3
U+647D.1	fEnglish	[1] fall; falling [2] lock together
U+647E.0	fUTF8	摾
U+647E.0	fCNS11643	3-4729
U+647E.0	fGB	5-4D59
U+647E.0	fR/S	  64.11
U+647E.0	fTotalStrokes	 14
U+647E.0	fCangjie	QNII
U+647E.0	fFourCorner	52030
U+647E.0	fCantonese	koeng6
U+647E.0	fMandarin	jiang4
U+647F.0	fUTF8	摿
U+647F.0	fCNS11643	2-4A54
U+647F.0	fBig5	E1ED
U+647F.0	fGB	E-2B6F
U+647F.0	fR/S	 64.11
U+647F.0	fTotalStrokes	 14
U+647F.0	fCangjie	QOYA
U+647F.0	fFourCorner	58061
U+647F.0	fCantonese	jiu4
U+647F.0	fMandarin	yin1
U+6480.0	fUTF8	撀
U+6480.0	fCNS11643	3-4C2D
U+6480.0	fGB	E-2B70
U+6480.0	fR/S	  64.11
U+6480.0	fTotalStrokes	 15
U+6480.0	fCangjie	GEQ
U+6480.0	fFourCorner	47500
U+6480.0	fCantonese	gau3
U+6480.0	fMandarin	gou4
U+6481.0	fUTF8	撁
U+6481.0	fCNS11643	3-4726
U+6481.0	fGB	E-2B71
U+6481.0	fR/S	 64.11
U+6481.0	fTotalStrokes	 14
U+6481.0	fCangjie	QYVQ
U+6481.0	fFourCorner	50050
U+6481.0	fHakka	  kien1;  ken1; k'ian1; kian1; kian5
U+6481.0	fCantonese	hin1
U+6481.0	fMandarin	qian1; qian4
U+6481.0	fEnglish	same as &#x727D;
U+6482.0	fUTF8	撂
U+6482.0	fCNS11643	2-4A4D
U+6482.0	fBig5	E1E6
U+6482.0	fGB	0-414C
U+6482.0	fR/S	 64.11
U+6482.0	fTotalStrokes	 14
U+6482.0	fCangjie	QWHR
U+6482.0	fFourCorner	56064
U+6482.0	fHakka	 liok8; liau5
U+6482.0	fCantonese	liu1; loek6
U+6482.0	fMandarin	luue4; liao4; le4; liao1; liao2
U+6482.0	fEnglish	[1] put down; lay down [2] leave behind; drop [3] non-classical form of &#x3A3C;
U+6483.0	fUTF8	撃
U+6483.0	fCNS11643	3-4C35
U+6483.0	fGB	E-2B72
U+6483.0	fR/S	 64.11
U+6483.0	fTotalStrokes	 15
U+6483.0	fCangjie	JEQ
U+6483.0	fFourCorner	57500
U+6483.0	fHakka	 git7
U+6483.0	fCantonese	gik1
U+6483.0	fMandarin	ji2; ji1; ji4
U+6483.0	fEnglish	simplified form of &#x64CA;
U+6484.0	fUTF8	撄
U+6484.0	fGB	0-5E7C
U+6484.0	fR/S	 64.11
U+6484.0	fTotalStrokes	 14
U+6484.0	fCangjie	QBOV
U+6484.0	fHakka	 jin1
U+6484.0	fCantonese	jing1
U+6484.0	fMandarin	ying1; ying2
U+6484.0	fEnglish	simplified form of &#x6516;
U+6485.0	fUTF8	撅
U+6485.0	fCNS11643	2-5058
U+6485.0	fBig5	E5B1
U+6485.0	fGB	0-3E6F
U+6485.0	fR/S	 64.12
U+6485.0	fTotalStrokes	 15
U+6485.0	fCangjie	QMTO
U+6485.0	fFourCorner	51082
U+6485.0	fHakka	  kiet7; kiet8; ket7
U+6485.0	fCantonese	kyut3; gwat6; gwai3
U+6485.0	fMandarin	jue1; gui4; jue2
U+6485.0	fEnglish	[1] protrude; stick up [2] snap; break
U+6486.0	fUTF8	撆
U+6486.0	fCNS11643	3-4C38
U+6486.0	fGB	5-4B4D
U+6486.0	fR/S	 64.12; 64.11
U+6486.0	fTotalStrokes	 16; 15
U+6486.0	fCangjie	FKQ
U+6486.0	fFourCorner	98500
U+6486.0	fHakka	  piet7; pet7; bak7
U+6486.0	fCantonese	pit3
U+6486.0	fMandarin	pie1; bi4 pie3
U+6486.0	fEnglish	standard form of &#x6487;
U+6487.0	fUTF8	撇
U+6487.0	fCNS11643	1-6871
U+6487.0	fBig5	BA4A
U+6487.0	fGB	0-4632
U+6487.0	fR/S	 64.12
U+6487.0	fTotalStrokes	 15
U+6487.0	fCangjie	QFBK
U+6487.0	fFourCorner	58040
U+6487.0	fHakka	  piet7; pet7; bak7
U+6487.0	fCantonese	pit3
U+6487.0	fMandarin	pie1; bi4
U+6487.0	fEnglish	[1] discard; abandon; throw away; cast away; neglect [2] skim
U+6487.1	fUTF8	撇
U+6487.1	fCNS11643	1-6871
U+6487.1	fBig5	BA4A
U+6487.1	fGB	0-4632
U+6487.1	fR/S	 64.12
U+6487.1	fTotalStrokes	 15
U+6487.1	fCangjie	QFBK
U+6487.1	fFourCorner	58040
U+6487.1	fMandarin	pie3
U+6487.1	fEnglish	[1] stroke made in the lower left direction (calligraphy) [2] purse the mouth (in contempt or to resist an impulse to cry)
U+6488.0	fUTF8	撈
U+6488.0	fCNS11643	1-6C7C
U+6488.0	fBig5	BCB4
U+6488.0	fGB	1-404C
U+6488.0	fR/S	 64.12
U+6488.0	fTotalStrokes	 15
U+6488.0	fCangjie	QFFS
U+6488.0	fFourCorner	59027
U+6488.0	fHakka	  lau2; leu2; lo1; liu2; lau5
U+6488.0	fCantonese	laau4; laau1 (also); lou1; lou4 (also)
U+6488.0	fMandarin	lao1; lao2; lao4; liao2; liao4
U+6488.0	fEnglish	[1] pull or drag out of the water; dredge [2] fish up; get by improper means
U+6489.0	fUTF8	撉
U+6489.0	fCNS11643	2-5735
U+6489.0	fBig5	E9AA
U+6489.0	fGB	5-4B4E
U+6489.0	fR/S	 64.12
U+6489.0	fTotalStrokes	 16
U+6489.0	fCangjie	YKQ
U+6489.0	fFourCorner	08504
U+6489.0	fHakka	  dun1;  dun5
U+6489.0	fCantonese	dan3; deon1; doe1
U+6489.0	fMandarin	dun1
U+6489.0	fEnglish	 strike with the fist
U+648A.0	fUTF8	撊
U+648A.0	fCNS11643	2-505D
U+648A.0	fBig5	E5B6
U+648A.0	fGB	3-4A43
U+648A.0	fR/S	 64.12
U+648A.0	fTotalStrokes	 15
U+648A.0	fCangjie	QANB
U+648A.0	fFourCorner	57020
U+648A.0	fHakka	 han2; han3
U+648A.0	fCantonese	haan6; haan5 (also)
U+648A.0	fMandarin	xian4; jian4
U+648A.0	fEnglish	 fierce wrathful
U+648B.0	fUTF8	撋
U+648B.0	fCNS11643	2-505C
U+648B.0	fBig5	E5B5
U+648B.0	fGB	5-4C3E
U+648B.0	fR/S	 64.12
U+648B.0	fTotalStrokes	 15
U+648B.0	fCangjie	QANG
U+648B.0	fFourCorner	57020
U+648B.0	fHakka	 ngion1; ngio1; no2; nio1
U+648B.0	fCantonese	no4; jyun4; lo4 (also); jeon6; jyun1
U+648B.0	fMandarin	ruan2; nuo2; rui2; run2
U+648B.0	fEnglish	 rub between the hands
U+648C.0	fUTF8	撌
U+648C.0	fCNS11643	2-505E
U+648C.0	fBig5	E5B7
U+648C.0	fGB	3-4A6A
U+648C.0	fR/S	 64.12
U+648C.0	fTotalStrokes	 15
U+648C.0	fCangjie	QLMC
U+648C.0	fFourCorner	55086
U+648C.0	fCantonese	gwai6
U+648C.0	fMandarin	kui4; gui4
U+648D.0	fUTF8	撍
U+648D.0	fCNS11643	3-4C30
U+648D.0	fGB	3-4B3D
U+648D.0	fR/S	  64.12
U+648D.0	fTotalStrokes	 15
U+648D.0	fCangjie	QMUA
U+648D.0	fFourCorner	51060
U+648D.0	fCantonese	cim4; zaan2
U+648D.0	fMandarin	zan3
U+648D.0	fEnglish	 hold in the hand; grasp wring
U+648E.0	fUTF8	撎
U+648E.0	fCNS11643	3-4C2F
U+648E.0	fGB	3-4B3A
U+648E.0	fR/S	  64.12
U+648E.0	fTotalStrokes	 15
U+648E.0	fCangjie	QGBT
U+648E.0	fFourCorner	54010
U+648E.0	fHakka	 ji5; ri5; jip8; rit7; jit7; jip7
U+648E.0	fCantonese	ji3
U+648E.0	fMandarin	yi4
U+648E.0	fEnglish	same as $
U+648F.0	fUTF8	撏
U+648F.0	fCNS11643	2-505B
U+648F.0	fBig5	E5B4
U+648F.0	fGB	3-4A34
U+648F.0	fR/S	 64.12
U+648F.0	fTotalStrokes	 15
U+648F.0	fCangjie	QSMI
U+648F.0	fFourCorner	57046
U+648F.0	fHakka	  cim2
U+648F.0	fCantonese	cam4; cim4 (also)
U+648F.0	fMandarin	xun2; xian2; can2
U+648F.0	fEnglish	 pluck; take; pick
U+6490.0	fUTF8	撐
U+6490.0	fCNS11643	1-6C7D
U+6490.0	fBig5	BCB5
U+6490.0	fGB	E-2B73
U+6490.0	fR/S	 64.12
U+6490.0	fTotalStrokes	 15
U+6490.0	fCangjie	QFBH
U+6490.0	fFourCorner	59052
U+6490.0	fHakka	  cang5; cang1; zhong1
U+6490.0	fCantonese	caang1; caang3 (also)
U+6490.0	fMandarin	cheng1
U+6490.0	fEnglish	[1] prop; support [2] stretch tight; burst [3] pole; punt (a raft or a boat) [4] maintain; keep up; go on with [5] open
U+6490.1	fUTF8	撐
U+6490.1	fCNS11643	1-6C7D
U+6490.1	fBig5	BCB5
U+6490.1	fGB	E-2B73
U+6490.1	fR/S	 64.12
U+6490.1	fTotalStrokes	 15
U+6490.1	fCangjie	QFBH
U+6490.1	fFourCorner	59052
U+6490.1	fMandarin	cheng5
U+6491.0	fUTF8	撑
U+6491.0	fCNS11643	3-4C3B
U+6491.0	fGB	0-3345
U+6491.0	fR/S	 64.12
U+6491.0	fTotalStrokes	 15
U+6491.0	fCangjie	QFBQ
U+6491.0	fFourCorner	59050
U+6491.0	fHakka	 cang1; cang5; ciang5; {cang5}
U+6491.0	fCantonese	caang1
U+6491.0	fMandarin	cheng1
U+6491.0	fEnglish	 prop up; support brace
U+6491.1	fUTF8	撑
U+6491.1	fCNS11643	3-4C3B
U+6491.1	fGB	0-3345
U+6491.1	fR/S	 64.12
U+6491.1	fTotalStrokes	 15
U+6491.1	fCangjie	QFBQ
U+6491.1	fFourCorner	59050
U+6491.1	fMandarin	cheng5
U+6492.0	fUTF8	撒
U+6492.0	fCNS11643	1-6D25
U+6492.0	fBig5	BCBB
U+6492.0	fGB	0-4876
U+6492.0	fR/S	 64.12
U+6492.0	fTotalStrokes	 15
U+6492.0	fCangjie	QTBK
U+6492.0	fFourCorner	58040
U+6492.0	fMandarin	sa3
U+6492.0	fEnglish	 scatter; sprinkle; disperse; spread; spill
U+6492.1	fUTF8	撒
U+6492.1	fCNS11643	1-6D25
U+6492.1	fBig5	BCBB
U+6492.1	fGB	0-4876
U+6492.1	fR/S	 64.12
U+6492.1	fTotalStrokes	 15
U+6492.1	fCangjie	QTBK
U+6492.1	fFourCorner	58040
U+6492.1	fHakka	 sat7; sot7; sat8; {we6}; {we5}
U+6492.1	fCantonese	saat3
U+6492.1	fMandarin	sa1
U+6492.1	fEnglish	[1] release; let go; loosen; unleash [2] relax; ease [3] exhibit; display; show
U+6493.0	fUTF8	撓
U+6493.0	fCNS11643	1-6D22
U+6493.0	fBig5	BCB8
U+6493.0	fGB	1-4453
U+6493.0	fR/S	 64.12
U+6493.0	fTotalStrokes	 15
U+6493.0	fCangjie	QGGU
U+6493.0	fFourCorner	54011
U+6493.0	fHakka	  nau2;  ngiau2; nau3; neu1; jau3; lau2; ngiau3
U+6493.0	fCantonese	naau4; naau5 (also); laau4 (also); naau2 (also); laau2 (also); laau5 (also); naau6
U+6493.0	fMandarin	nao2; hao1; nao4; rao4; xiao1
U+6493.0	fEnglish	[1] bend; daunt; subjugate; yield; submit to superior force [2] scratch; rub [3] hinder; bother; obstruct
U+6494.0	fUTF8	撔
U+6494.0	fCNS11643	3-4C31
U+6494.0	fGB	5-4D50
U+6494.0	fR/S	  64.12
U+6494.0	fTotalStrokes	 15
U+6494.0	fCangjie	QAYF
U+6494.0	fFourCorner	56090
U+6494.0	fCantonese	wang6
U+6494.0	fMandarin	heng4
U+6495.0	fUTF8	撕
U+6495.0	fCNS11643	1-6D23
U+6495.0	fBig5	BCB9
U+6495.0	fGB	0-4B3A
U+6495.0	fR/S	 64.12
U+6495.0	fTotalStrokes	 15
U+6495.0	fCangjie	QTCL
U+6495.0	fFourCorner	52021
U+6495.0	fHakka	 sii1; si1; su1; se1
U+6495.0	fCantonese	si1; sai1
U+6495.0	fMandarin	si1; xi1
U+6495.0	fEnglish	 rip; tear
U+6496.0	fUTF8	撖
U+6496.0	fCNS11643	2-5056
U+6496.0	fBig5	E5AF
U+6496.0	fGB	0-5E7E
U+6496.0	fR/S	 64.12
U+6496.0	fTotalStrokes	 15
U+6496.0	fCangjie	QMJK
U+6496.0	fFourCorner	58040
U+6496.0	fHakka	 kam3; gam6; gam5
U+6496.0	fCantonese	hon6; ham5 (also); ham1; haam5
U+6496.0	fMandarin	qian3; han4; kan1
U+6496.0	fEnglish	a Chinese family name
U+6497.0	fUTF8	撗
U+6497.0	fCNS11643	2-5059
U+6497.0	fBig5	E5B2
U+6497.0	fGB	5-4D39
U+6497.0	fR/S	 64.12
U+6497.0	fTotalStrokes	 15
U+6497.0	fCangjie	QTLC
U+6497.0	fFourCorner	54086
U+6497.0	fHakka	 kong5; wang2; gwok7
U+6497.0	fCantonese	gwong3; gong3 (also)
U+6497.0	fMandarin	huang2; guang4; guo1
U+6498.0	fUTF8	撘
U+6498.0	fCNS11643	2-5063
U+6498.0	fBig5	E5BC
U+6498.0	fGB	E-2B74
U+6498.0	fR/S	 64.12
U+6498.0	fTotalStrokes	 15
U+6498.0	fCangjie	QHOR
U+6498.0	fFourCorner	58061
U+6498.0	fHakka	 dap7
U+6498.0	fCantonese	daap3; dap3
U+6498.0	fMandarin	da1
U+6498.0	fEnglish	strike; pound
U+6499.0	fUTF8	撙
U+6499.0	fCNS11643	1-6D2B
U+6499.0	fBig5	BCC1
U+6499.0	fGB	0-5F24
U+6499.0	fR/S	 64.12
U+6499.0	fTotalStrokes	 15
U+6499.0	fCangjie	QTWI
U+6499.0	fFourCorner	48046
U+6499.0	fHakka	  zun3
U+6499.0	fCantonese	zyun2
U+6499.0	fMandarin	zun3
U+6499.0	fEnglish	[1] economize; save [2] comply with
U+649A.0	fUTF8	撚
U+649A.0	fCNS11643	1-6D29
U+649A.0	fBig5	BCBF
U+649A.0	fGB	E-2B75
U+649A.0	fR/S	 64.12
U+649A.0	fTotalStrokes	 15
U+649A.0	fCangjie	QBKF
U+649A.0	fFourCorner	53033
U+649A.0	fHakka	 ngien3; ngien5; ngian3
U+649A.0	fCantonese	nan2; lan2 (also); nin2 (also); nim2; lim2 (also); nin5 (also); lin2 (also); lin5 (also)
U+649A.0	fMandarin	nian3
U+649A.0	fEnglish	 twirl with fingers; toy with
U+649B.0	fUTF8	撛
U+649B.0	fCNS11643	3-4C2E
U+649B.0	fGB	3-4B45
U+649B.0	fR/S	  64.12; 64.13
U+649B.0	fTotalStrokes	 15
U+649B.0	fCangjie	QFDQ
U+649B.0	fFourCorner	59050
U+649B.0	fHakka	 lin2
U+649B.0	fCantonese	leon5
U+649B.0	fMandarin	lin3
U+649C.0	fUTF8	撜
U+649C.0	fCNS11643	2-505A
U+649C.0	fBig5	E5B3
U+649C.0	fGB	3-4B47
U+649C.0	fR/S	 64.12
U+649C.0	fTotalStrokes	 15
U+649C.0	fCangjie	QNOT
U+649C.0	fFourCorner	52018
U+649C.0	fCantonese	cing2; caang4
U+649C.0	fMandarin	zheng3; cheng2; cheng4; sheng1
U+649D.0	fUTF8	撝
U+649D.0	fCNS11643	2-3B7D
U+649D.0	fBig5	D95A
U+649D.0	fGB	5-4B66
U+649D.0	fR/S	 64.12; 64.9
U+649D.0	fTotalStrokes	 15
U+649D.0	fCangjie	QBHF
U+649D.0	fFourCorner	42027
U+649D.0	fCantonese	fai1
U+649D.0	fMandarin	hui1; wei2; wei3
U+649D.0	fEnglish	[1] wave; brandish [2] modest; humble
U+649E.0	fUTF8	撞
U+649E.0	fCNS11643	1-6C7A
U+649E.0	fBig5	BCB2
U+649E.0	fGB	0-5732
U+649E.0	fR/S	 64.12
U+649E.0	fTotalStrokes	 15
U+649E.0	fCangjie	QYTG
U+649E.0	fFourCorner	50014
U+649E.0	fHakka	 cong5; cong1; cong3; cong6
U+649E.0	fCantonese	zong6; zong2 (also); cong4
U+649E.0	fMandarin	zhuang4; chuang2; chuan4; zhuan2
U+649E.0	fEnglish	[1] knock against; bump into; collide; dash [2] meet by chance
U+649F.0	fUTF8	撟
U+649F.0	fCNS11643	2-5060
U+649F.0	fBig5	E5B9
U+649F.0	fGB	1-5E58
U+649F.0	fR/S	 64.12
U+649F.0	fTotalStrokes	 15
U+649F.0	fCangjie	QHKB
U+649F.0	fFourCorner	52027
U+649F.0	fHakka	  giau3; kieu3
U+649F.0	fCantonese	giu2; giu6
U+649F.0	fMandarin	jiao3; jia3; jiao1; kao3
U+649F.0	fEnglish	 put right; set right
U+649F.1	fUTF8	撟
U+649F.1	fCNS11643	2-5060
U+649F.1	fBig5	E5B9
U+649F.1	fGB	1-5E58
U+649F.1	fR/S	 64.12
U+649F.1	fTotalStrokes	 15
U+649F.1	fCangjie	QHKB
U+649F.1	fFourCorner	52027
U+649F.1	fMandarin	jiao4
U+649F.1	fEnglish	 raise; lift up
U+64A0.0	fUTF8	撠
U+64A0.0	fCNS11643	2-5057
U+64A0.0	fBig5	E5B0
U+64A0.0	fGB	3-4B3C
U+64A0.0	fR/S	 64.12
U+64A0.0	fTotalStrokes	 15
U+64A0.0	fCangjie	QJJI
U+64A0.0	fFourCorner	53050
U+64A0.0	fHakka	 git7
U+64A0.0	fCantonese	gik1
U+64A0.0	fMandarin	ji3
U+64A1.0	fUTF8	撡
U+64A1.0	fCNS11643	3-4C3E
U+64A1.0	fGB	5-4D5B
U+64A1.0	fR/S	  64.12
U+64A1.0	fTotalStrokes	 15
U+64A1.0	fCangjie	QIIP
U+64A1.0	fFourCorner	53030
U+64A1.0	fCantonese	cou1; cou3
U+64A1.0	fMandarin	cao1
U+64A2.0	fUTF8	撢
U+64A2.0	fCNS11643	1-6D2C
U+64A2.0	fBig5	BCC2
U+64A2.0	fGB	5-4D4E
U+64A2.0	fR/S	 64.12
U+64A2.0	fTotalStrokes	 15
U+64A2.0	fCangjie	QMWJ
U+64A2.0	fFourCorner	51046
U+64A2.0	fHakka	 tam5;  tam1; tam2; tam3
U+64A2.0	fCantonese	taam3; taam2 (also); ngam4; am4 (also); daan6 (also); daan5; taam1
U+64A2.0	fMandarin	dan3; tan4; tan1; xin2; yin2
U+64A2.0	fEnglish	[1] dust; brush lightly; whisk [2] duster
U+64A3.0	fUTF8	撣
U+64A3.0	fCNS11643	2-505F
U+64A3.0	fBig5	E5B8
U+64A3.0	fGB	1-3527
U+64A3.0	fR/S	 64.12
U+64A3.0	fTotalStrokes	 15
U+64A3.0	fCangjie	QRRJ
U+64A3.0	fFourCorner	56056
U+64A3.0	fHakka	 tan5; tan6; tan3
U+64A3.0	fCantonese	daan6 (also); sin6; daan5
U+64A3.0	fMandarin	dan3; chan2; shan4; shan3; dan4; tan2; zhan3
U+64A3.0	fEnglish	 dust duster
U+64A4.0	fUTF8	撤
U+64A4.0	fCNS11643	1-6874
U+64A4.0	fBig5	BA4D
U+64A4.0	fGB	0-3337
U+64A4.0	fR/S	 64.12
U+64A4.0	fTotalStrokes	 15
U+64A4.0	fCangjie	QYBK
U+64A4.0	fFourCorner	58040
U+64A4.0	fHakka	 cet8; chat7; cat8; chat8; chet8
U+64A4.0	fCantonese	cit3
U+64A4.0	fMandarin	che4; zhe2
U+64A4.0	fEnglish	 remove; withdraw; take back; take away; evacuate
U+64A5.0	fUTF8	撥
U+64A5.0	fCNS11643	1-6D21
U+64A5.0	fBig5	BCB7
U+64A5.0	fGB	1-3226
U+64A5.0	fR/S	 64.12
U+64A5.0	fTotalStrokes	 15
U+64A5.0	fCangjie	QNOE
U+64A5.0	fFourCorner	52047
U+64A5.0	fHakka	  pat7;  bat7; pat8; bat8
U+64A5.0	fCantonese	but6; but2 (also); put3; but3
U+64A5.0	fMandarin	bo1; fa2; fa1
U+64A5.0	fEnglish	[1] move; transfer [2] dispel; remove [3] distribute; issue [4] poke [5] set aside; set apart; appropriate; allocate [6] give [7] give; allow
U+64A6.0	fUTF8	撦
U+64A6.0	fCNS11643	2-4A4B
U+64A6.0	fBig5	E1E4
U+64A6.0	fGB	E-2B76
U+64A6.0	fR/S	 64.12
U+64A6.0	fTotalStrokes	 15
U+64A6.0	fCangjie	QKJA
U+64A6.0	fFourCorner	54064
U+64A6.0	fHakka	  cha3; ca1; ca3; cha1
U+64A6.0	fCantonese	ce2
U+64A6.0	fMandarin	che3
U+64A6.0	fEnglish	 rip up; tear
U+64A7.0	fUTF8	撧
U+64A7.0	fCNS11643	3-4C3F
U+64A7.0	fGB	E-2B77
U+64A7.0	fR/S	  64.12
U+64A7.0	fTotalStrokes	 15
U+64A7.0	fCangjie	QVFU
U+64A7.0	fFourCorner	57010
U+64A7.0	fHakka	 ciet8; cet8; ts'iat8
U+64A7.0	fCantonese	zyut6; zyut2 (also); zyut3
U+64A7.0	fMandarin	jue2
U+64A7.0	fEnglish	same as &#x7D55;
U+64A8.0	fUTF8	撨
U+64A8.0	fCNS11643	3-4C34
U+64A8.0	fGB	3-4B3F
U+64A8.0	fR/S	 64.12
U+64A8.0	fTotalStrokes	 15
U+64A8.0	fCangjie	QOGF
U+64A8.0	fFourCorner	50030
U+64A8.0	fCantonese	siu1
U+64A8.0	fMandarin	xiao1
U+64A9.0	fUTF8	撩
U+64A9.0	fCNS11643	1-6D24
U+64A9.0	fBig5	BCBA
U+64A9.0	fGB	0-4143
U+64A9.0	fR/S	 64.12
U+64A9.0	fTotalStrokes	 15
U+64A9.0	fCangjie	QKCF
U+64A9.0	fFourCorner	54096
U+64A9.0	fHakka	  liau2;  liau1; liau3
U+64A9.0	fCantonese	liu4; liu1; liu6
U+64A9.0	fMandarin	liao2; liao4; lao3; lao4; liao3
U+64A9.0	fEnglish	[1] provoke; excite; stir up; tease; tantalize [2] disorderly; confused
U+64A9.1	fUTF8	撩
U+64A9.1	fCNS11643	1-6D24
U+64A9.1	fBig5	BCBA
U+64A9.1	fGB	0-4143
U+64A9.1	fR/S	 64.12
U+64A9.1	fTotalStrokes	 15
U+64A9.1	fCangjie	QKCF
U+64A9.1	fFourCorner	54096
U+64A9.1	fMandarin	liao1
U+64A9.1	fEnglish	[1] lift up; raise; hold up [2] sprinkle
U+64AA.0	fUTF8	撪
U+64AA.0	fCNS11643	3-4C3D
U+64AA.0	fGB	E-2B78
U+64AA.0	fR/S	 64.12
U+64AA.0	fTotalStrokes	 15
U+64AA.0	fCangjie	QIKJ
U+64AA.0	fFourCorner	53050
U+64AA.0	fCantonese	ban6; bun2
U+64AA.0	fMandarin	ben4
U+64AA.0	fEnglish	same as &#x3A27;
U+64AB.0	fUTF8	撫
U+64AB.0	fCNS11643	1-6D28
U+64AB.0	fBig5	BCBE
U+64AB.0	fGB	1-3827
U+64AB.0	fR/S	 64.12
U+64AB.0	fTotalStrokes	 15
U+64AB.0	fCangjie	QOTF
U+64AB.0	fFourCorner	58031
U+64AB.0	fHakka	 wu3; fu3; vu3
U+64AB.0	fCantonese	fu2
U+64AB.0	fMandarin	fu3; mu2; hu1
U+64AB.0	fEnglish	[1] stroke; touch [2] soothe; console; comfort; relieve [3] bring up; rear; nurture; foster
U+64AC.0	fUTF8	撬
U+64AC.0	fCNS11643	1-6D2A
U+64AC.0	fBig5	BCC0
U+64AC.0	fGB	0-474B
U+64AC.0	fR/S	 64.12
U+64AC.0	fTotalStrokes	 15
U+64AC.0	fCangjie	QHUU
U+64AC.0	fFourCorner	52014
U+64AC.0	fHakka	  kiau5; kiau3; kiau6; k'ieu5; ke3; ge3
U+64AC.0	fCantonese	giu6; hiu3 (also)
U+64AC.0	fMandarin	qiao4
U+64AC.0	fEnglish	 pry; prize
U+64AC.1	fUTF8	撬
U+64AC.1	fCNS11643	1-6D2A
U+64AC.1	fBig5	BCC0
U+64AC.1	fGB	0-474B
U+64AC.1	fR/S	 64.12
U+64AC.1	fTotalStrokes	 15
U+64AC.1	fCangjie	QHUU
U+64AC.1	fFourCorner	52014
U+64AC.1	fMandarin	qiao1
U+64AC.1	fEnglish	 lift; raise
U+64AD.0	fUTF8	播
U+64AD.0	fCNS11643	1-6D27
U+64AD.0	fBig5	BCBD
U+64AD.0	fGB	0-3225
U+64AD.0	fR/S	 64.12
U+64AD.0	fTotalStrokes	 15
U+64AD.0	fCangjie	QHDW
U+64AD.0	fFourCorner	52069
U+64AD.0	fHakka	 bo5
U+64AD.0	fCantonese	bo3
U+64AD.0	fMandarin	bo1; bo3; bo4
U+64AD.0	fEnglish	[1] sow; seed [2] spread; propagate [3] move [4] cast away; abandon
U+64AE.0	fUTF8	撮
U+64AE.0	fCNS11643	1-6D26
U+64AE.0	fBig5	BCBC
U+64AE.0	fGB	0-3469
U+64AE.0	fR/S	 64.12
U+64AE.0	fTotalStrokes	 15
U+64AE.0	fCangjie	QASE
U+64AE.0	fFourCorner	56047
U+64AE.0	fHakka	  cot7; zep7; zut7; zot7
U+64AE.0	fCantonese	cyut3; caai2
U+64AE.0	fMandarin	cuo1; cuo4; zui2; chai3; jue2; zuan1; zui4
U+64AE.0	fEnglish	[1] take with fingers [2] gather [3] extract; summarize [4] a pinch of; small amount [5] grasp [6] take with fingers [7] fool
U+64AE.1	fUTF8	撮
U+64AE.1	fCNS11643	1-6D26
U+64AE.1	fBig5	BCBC
U+64AE.1	fGB	0-3469
U+64AE.1	fR/S	 64.12
U+64AE.1	fTotalStrokes	 15
U+64AE.1	fCangjie	QASE
U+64AE.1	fFourCorner	56047
U+64AE.1	fMandarin	zuo3
U+64AE.1	fEnglish	tuft (of hair, grass, etc.)
U+64AF.0	fUTF8	撯
U+64AF.0	fCNS11643	3-4C3A
U+64AF.0	fGB	5-4D43
U+64AF.0	fR/S	 64.12
U+64AF.0	fTotalStrokes	 15
U+64AF.0	fCangjie	QTQU
U+64AF.0	fFourCorner	58060
U+64AF.0	fCantonese	zoek3
U+64AF.0	fMandarin	zhuo2
U+64B0.0	fUTF8	撰
U+64B0.0	fCNS11643	1-6C7E
U+64B0.0	fBig5	BCB6
U+64B0.0	fGB	0-572B
U+64B0.0	fR/S	 64.12
U+64B0.0	fTotalStrokes	 15
U+64B0.0	fCangjie	QRUC
U+64B0.0	fFourCorner	57081
U+64B0.0	fHakka	  con3; sien3; con6; zon5; sen3; cion5; con5
U+64B0.0	fCantonese	zaan6; zaan3 (also); syun2
U+64B0.0	fMandarin	zhuan4; suan4; xuan3; suan3; xuan4; zhuan1
U+64B0.0	fEnglish	 compose; write; compile
U+64B1.0	fUTF8	撱
U+64B1.0	fCNS11643	2-5062
U+64B1.0	fBig5	E5BB
U+64B1.0	fGB	E-2B79
U+64B1.0	fR/S	 64.12
U+64B1.0	fTotalStrokes	 15
U+64B1.0	fCangjie	QNLB
U+64B1.0	fFourCorner	54027
U+64B1.0	fHakka	 to5; to3
U+64B1.0	fCantonese	wai5; to2; to5
U+64B1.0	fMandarin	tuo3; hui1; wei3
U+64B1.0	fEnglish	 shorten; clip throw away
U+64B2.0	fUTF8	撲
U+64B2.0	fCNS11643	1-6C7B
U+64B2.0	fBig5	BCB3
U+64B2.0	fGB	1-464B
U+64B2.0	fR/S	 64.12
U+64B2.0	fTotalStrokes	 15
U+64B2.0	fCangjie	QTCO
U+64B2.0	fFourCorner	52034
U+64B2.0	fHakka	  pok7; puk7; pu1; puk8
U+64B2.0	fCantonese	puk1; pok3
U+64B2.0	fMandarin	pu1; bao2; bo2; bu2; pu2
U+64B2.0	fEnglish	[1] pound; beat; strike [2] dash; smash [3] throw oneself on; spring at [4] flap; flutter
U+64B3.0	fUTF8	撳
U+64B3.0	fCNS11643	1-6D2D
U+64B3.0	fBig5	BCC3
U+64B3.0	fGB	1-5E6C
U+64B3.0	fR/S	 64.12
U+64B3.0	fTotalStrokes	 15
U+64B3.0	fCangjie	QCNO
U+64B3.0	fFourCorner	57080
U+64B3.0	fHakka	 kim3; kim6; kim5
U+64B3.0	fCantonese	gam6; kam2
U+64B3.0	fMandarin	qin4
U+64B3.0	fEnglish	 press with the hand; push with the hand
U+64B4.0	fUTF8	撴
U+64B4.0	fCNS11643	3-4C39
U+64B4.0	fGB	3-4B44
U+64B4.0	fR/S	  64.12
U+64B4.0	fTotalStrokes	 15
U+64B4.0	fCangjie	QYDK
U+64B4.0	fFourCorner	58040
U+64B4.0	fCantonese	dan3; deon1
U+64B4.0	fMandarin	dun1
U+64B4.0	fEnglish	 jolt thump
U+64B5.0	fUTF8	撵
U+64B5.0	fGB	0-446C
U+64B5.0	fR/S	 64.12
U+64B5.0	fTotalStrokes	 15
U+64B5.0	fCangjie	QQOQ
U+64B5.0	fHakka	 lien1; lien3
U+64B5.0	fCantonese	lin5
U+64B5.0	fMandarin	nian3
U+64B5.0	fEnglish	simplified form of &#x6506;
U+64B6.0	fUTF8	撶
U+64B6.0	fCNS11643	3-6522
U+64B6.0	fGB	E-2B7A
U+64B6.0	fR/S	 64.12
U+64B6.0	fTotalStrokes	 15
U+64B6.0	fCangjie	QTMJ
U+64B6.0	fFourCorner	5405
U+64B6.0	fCantonese	waa1
U+64B7.0	fUTF8	撷
U+64B7.0	fGB	0-5F22
U+64B7.0	fR/S	 64.12
U+64B7.0	fTotalStrokes	 15
U+64B7.0	fCangjie	QGRO
U+64B7.0	fHakka	 get7; giet7
U+64B7.0	fCantonese	kit3; git3 (also)
U+64B7.0	fMandarin	xie2; jie2
U+64B7.0	fEnglish	simplified form of &#x64F7;
U+64B8.0	fUTF8	撸
U+64B8.0	fGB	0-5F23
U+64B8.0	fR/S	 64.12
U+64B8.0	fTotalStrokes	 15
U+64B8.0	fCangjie	XQNWA
U+64B8.0	fCantonese	lou5
U+64B8.0	fMandarin	lu3; lu1
U+64B8.0	fEnglish	simplified form of &#x64FC;
U+64B9.0	fUTF8	撹
U+64B9.0	fCNS11643	3-4C37
U+64B9.0	fGB	E-2B7B
U+64B9.0	fR/S	 64.12
U+64B9.0	fTotalStrokes	 15
U+64B9.0	fCangjie	QFBU
U+64B9.0	fFourCorner	53016
U+64B9.0	fCantonese	gaau2
U+64B9.0	fMandarin	jiao3; jia3
U+64B9.0	fEnglish	disturb; agitate; stir up
U+64BA.0	fUTF8	撺
U+64BA.0	fGB	0-5F25
U+64BA.0	fR/S	 64.12
U+64BA.0	fTotalStrokes	 15
U+64BA.0	fCangjie	QJCL
U+64BA.0	fHakka	 con3; chon3
U+64BA.0	fCantonese	cyun3; cyun2 (also)
U+64BA.0	fMandarin	cuan1; cuan4
U+64BA.0	fEnglish	simplified form of &#x651B;
U+64BB.0	fUTF8	撻
U+64BB.0	fCNS11643	1-7061
U+64BB.0	fBig5	BED8
U+64BB.0	fGB	1-4C22
U+64BB.0	fR/S	 64.13
U+64BB.0	fTotalStrokes	 16
U+64BB.0	fCangjie	QYGQ
U+64BB.0	fFourCorner	54034
U+64BB.0	fHakka	 dat7; tat7; dat8
U+64BB.0	fCantonese	taat3; taat1; daat3
U+64BB.0	fMandarin	ta4
U+64BB.0	fEnglish	[1] flog; strike; chastise [2] show; throw down
U+64BC.0	fUTF8	撼
U+64BC.0	fCNS11643	1-7062
U+64BC.0	fBig5	BED9
U+64BC.0	fGB	0-3A33
U+64BC.0	fR/S	 64.13
U+64BC.0	fTotalStrokes	 16
U+64BC.0	fCangjie	QIRP
U+64BC.0	fFourCorner	53035
U+64BC.0	fHakka	 ham5;  ham3
U+64BC.0	fCantonese	ham6; ham2 (also)
U+64BC.0	fMandarin	han4
U+64BC.0	fEnglish	 rock; shake; jolt; joggle
U+64BD.0	fUTF8	撽
U+64BD.0	fCNS11643	2-5734
U+64BD.0	fBig5	E9A9
U+64BD.0	fGB	3-4B4A
U+64BD.0	fR/S	 64.13
U+64BD.0	fTotalStrokes	 16
U+64BD.0	fCangjie	QHSK
U+64BD.0	fFourCorner	58040
U+64BD.0	fHakka	 git7; kau1; kau5
U+64BD.0	fCantonese	gik1; kiu3; haau3 (also); git1
U+64BD.0	fMandarin	qiao4; ji1; jiao1; jiao3
U+64BD.0	fEnglish	[1] whip as a horse [2] screen [3] lay hold of
U+64BE.0	fUTF8	撾
U+64BE.0	fCNS11643	1-706B
U+64BE.0	fBig5	BEE2
U+64BE.0	fGB	1-4E4E
U+64BE.0	fR/S	 64.13
U+64BE.0	fTotalStrokes	 16
U+64BE.0	fCangjie	QYBB
U+64BE.0	fFourCorner	57032
U+64BE.0	fHakka	 ko1
U+64BE.0	fCantonese	zaa1; wo1; gwo1
U+64BE.0	fMandarin	zhua1; wo1; ge1
U+64BE.0	fEnglish	 beat; strike
U+64BF.0	fUTF8	撿
U+64BF.0	fCNS11643	1-7068
U+64BF.0	fBig5	BEDF
U+64BF.0	fGB	1-3C71
U+64BF.0	fR/S	 64.13
U+64BF.0	fTotalStrokes	 16
U+64BF.0	fCangjie	QOMO
U+64BF.0	fFourCorner	58086
U+64BF.0	fHakka	  giam3
U+64BF.0	fCantonese	gim2
U+64BF.0	fMandarin	jian3; lian4
U+64BF.0	fEnglish	 to pick up; collect; gather
U+64C0.0	fUTF8	擀
U+64C0.0	fCNS11643	3-5070
U+64C0.0	fGB	0-5F26
U+64C0.0	fR/S	  64.13
U+64C0.0	fTotalStrokes	 16
U+64C0.0	fCangjie	QJJJ
U+64C0.0	fFourCorner	58040
U+64C0.0	fHakka	  gon5; gon3; hon5
U+64C0.0	fCantonese	gon2
U+64C0.0	fMandarin	gan3; han4
U+64C0.0	fEnglish	[1] stretch out with a rolling pin; roll flat [2] polish; shine
U+64C1.0	fUTF8	擁
U+64C1.0	fCNS11643	1-705F
U+64C1.0	fBig5	BED6
U+64C1.0	fGB	1-5335
U+64C1.0	fR/S	 64.13
U+64C1.0	fTotalStrokes	 16
U+64C1.0	fCangjie	QYVG
U+64C1.0	fFourCorner	50014
U+64C1.0	fHakka	  jung3; rung3; jung5
U+64C1.0	fCantonese	jung2; ung2
U+64C1.0	fMandarin	yong1; yong3
U+64C1.0	fEnglish	[1] embrace; hug; [2] crowd; thong; swarm [3] v] have; possess [4] follow; support
U+64C2.0	fUTF8	擂
U+64C2.0	fCNS11643	1-7066
U+64C2.0	fBig5	BEDD
U+64C2.0	fGB	0-405E
U+64C2.0	fR/S	 64.13
U+64C2.0	fTotalStrokes	 16
U+64C2.0	fCangjie	QMBW
U+64C2.0	fFourCorner	51063
U+64C2.0	fHakka	  lui2;  lui3; lui1; lui5
U+64C2.0	fCantonese	leoi4
U+64C2.0	fMandarin	lei2; lei1
U+64C2.0	fEnglish	[1] grind; pestle [2] beat; hit
U+64C2.1	fUTF8	擂
U+64C2.1	fCNS11643	1-7066
U+64C2.1	fBig5	BEDD
U+64C2.1	fGB	0-405E
U+64C2.1	fR/S	 64.13
U+64C2.1	fTotalStrokes	 16
U+64C2.1	fCangjie	QMBW
U+64C2.1	fFourCorner	51063
U+64C2.1	fMandarin	lei4
U+64C2.1	fEnglish	used in combination
U+64C3.0	fUTF8	擃
U+64C3.0	fCNS11643	2-5736
U+64C3.0	fBig5	E9AB
U+64C3.0	fGB	5-4C2C
U+64C3.0	fR/S	 64.13
U+64C3.0	fTotalStrokes	 16
U+64C3.0	fCangjie	QTWV
U+64C3.0	fFourCorner	55032
U+64C3.0	fHakka	 nong2; {kang5}; nong3
U+64C3.0	fCantonese	nong5; long5 (also); nong6
U+64C3.0	fMandarin	kuo3
U+64C3.0	fEnglish	[1] ward off [2] stick [3] stab
U+64C4.0	fUTF8	擄
U+64C4.0	fCNS11643	1-7064
U+64C4.0	fBig5	BEDB
U+64C4.0	fGB	1-4230
U+64C4.0	fR/S	 64.13
U+64C4.0	fTotalStrokes	 16
U+64C4.0	fCangjie	QYPS
U+64C4.0	fFourCorner	51027
U+64C4.0	fHakka	 lu1;  lu3
U+64C4.0	fCantonese	lou5
U+64C4.0	fMandarin	lu3
U+64C4.0	fEnglish	 capture; seize; take captive
U+64C5.0	fUTF8	擅
U+64C5.0	fCNS11643	1-705E
U+64C5.0	fBig5	BED5
U+64C5.0	fGB	0-4943
U+64C5.0	fR/S	 64.13
U+64C5.0	fTotalStrokes	 16
U+64C5.0	fCangjie	QYWM
U+64C5.0	fFourCorner	50016
U+64C5.0	fHakka	 shan3; san3; sen3; shen3; chan3
U+64C5.0	fCantonese	sin6
U+64C5.0	fMandarin	shan4
U+64C5.0	fEnglish	[1] monopolise; take exclusive possession [2] unauthorised; arbitrarily; unilateral [3] be good at; be expert in
U+64C6.0	fUTF8	擆
U+64C6.0	fCNS11643	3-4C32
U+64C6.0	fGB	3-4B28
U+64C6.0	fR/S	  64.12
U+64C6.0	fTotalStrokes	 15
U+64C6.0	fCangjie	QTJA
U+64C6.0	fFourCorner	54060
U+64C6.0	fCantonese	zoek3
U+64C6.0	fMandarin	zhuo2
U+64C7.0	fUTF8	擇
U+64C7.0	fCNS11643	1-7065
U+64C7.0	fBig5	BEDC
U+64C7.0	fGB	1-5471
U+64C7.0	fR/S	 64.13
U+64C7.0	fTotalStrokes	 16
U+64C7.0	fCangjie	QWLJ
U+64C7.0	fFourCorner	56041
U+64C7.0	fHakka	 cet8; cak8; tok8
U+64C7.0	fCantonese	zaak6
U+64C7.0	fMandarin	ze2; zhai2; yi4
U+64C7.0	fEnglish	 select; choose; pick out
U+64C8.0	fUTF8	擈
U+64C8.0	fCNS11643	3-5077
U+64C8.0	fGB	E-2B7C
U+64C8.0	fR/S	 64.13
U+64C8.0	fTotalStrokes	 16
U+64C8.0	fCangjie	QTCD
U+64C8.0	fFourCorner	52090
U+64C8.0	fHakka	  pok7; puk7; pu1; puk8
U+64C8.0	fCantonese	puk1; pok3
U+64C8.0	fMandarin	pu1; bao2; bo2; bu2; pu2
U+64C8.0	fEnglish	same as &#x64B2;
U+64C9.0	fUTF8	擉
U+64C9.0	fCNS11643	2-5733
U+64C9.0	fBig5	E9A8
U+64C9.0	fGB	5-4D64
U+64C9.0	fR/S	 64.13
U+64C9.0	fTotalStrokes	 16
U+64C9.0	fCangjie	QWLI
U+64C9.0	fFourCorner	56027
U+64C9.0	fHakka	 cuk8; cok8; cok7
U+64C9.0	fCantonese	cok3; cuk1; zuk1; zuk6
U+64C9.0	fMandarin	chuo4; chu4; chuo1; shu3; zhuo1; zhuo2
U+64C9.0	fEnglish	 pierce break through
U+64CA.0	fUTF8	擊
U+64CA.0	fCNS11643	1-7364
U+64CA.0	fBig5	C0BB
U+64CA.0	fGB	1-3B77
U+64CA.0	fR/S	 64.13
U+64CA.0	fTotalStrokes	 17; 16
U+64CA.0	fCangjie	JEQ
U+64CA.0	fFourCorner	57502
U+64CA.0	fHakka	  git7
U+64CA.0	fCantonese	gik1
U+64CA.0	fMandarin	ji1; ji4; ji2
U+64CA.0	fEnglish	 strike; hit; beat; attack; fight; bump into
U+64CA.1	fUTF8	擊
U+64CA.1	fCNS11643	1-7364
U+64CA.1	fBig5	C0BB
U+64CA.1	fGB	1-3B77
U+64CA.1	fR/S	 64.13
U+64CA.1	fTotalStrokes	 17; 16
U+64CA.1	fCangjie	JEQ
U+64CA.1	fFourCorner	57502
U+64CA.1	fEnglish	 lift; support; prop up
U+64CB.0	fUTF8	擋
U+64CB.0	fCNS11643	1-7060
U+64CB.0	fBig5	BED7
U+64CB.0	fGB	1-3532
U+64CB.0	fR/S	 64.13
U+64CB.0	fTotalStrokes	 16
U+64CB.0	fCangjie	QFBW
U+64CB.0	fFourCorner	59066
U+64CB.0	fHakka	  dong3;  dong5
U+64CB.0	fCantonese	dong2; dong3
U+64CB.0	fMandarin	dang4; tang3
U+64CB.0	fEnglish	[1] obstruct; impede; stop; resist; ward off [2] fender; blind [3] gear
U+64CB.1	fUTF8	擋
U+64CB.1	fCNS11643	1-7060
U+64CB.1	fBig5	BED7
U+64CB.1	fGB	1-3532
U+64CB.1	fR/S	 64.13
U+64CB.1	fTotalStrokes	 16
U+64CB.1	fCangjie	QFBW
U+64CB.1	fFourCorner	59066
U+64CB.1	fMandarin	dang3
U+64CB.1	fEnglish	used in combination
U+64CC.0	fUTF8	擌
U+64CC.0	fCNS11643	5-545E
U+64CC.0	fGB	E-2B7D
U+64CC.0	fR/S	  64.13
U+64CC.0	fTotalStrokes	 16
U+64CC.0	fCangjie	QHKO
U+64CC.0	fFourCorner	58030
U+64CC.0	fCantonese	saak3
U+64CC.0	fMandarin	suo3; se4
U+64CD.0	fUTF8	操
U+64CD.0	fCNS11643	1-7067
U+64CD.0	fBig5	BEDE
U+64CD.0	fGB	0-3259
U+64CD.0	fR/S	 64.13
U+64CD.0	fTotalStrokes	 16
U+64CD.0	fCangjie	QRRD
U+64CD.0	fFourCorner	56094
U+64CD.0	fHakka	  cau1; cau5; chau5
U+64CD.0	fCantonese	cou1; cou3
U+64CD.0	fMandarin	cao1; cao4; luo3; sou3
U+64CD.0	fEnglish	[1] handle; manage [2] hold; grasp [3] exercise; drill [4] speak [5] a Chinese family name
U+64CE.0	fUTF8	擎
U+64CE.0	fCNS11643	1-7363
U+64CE.0	fBig5	C0BA
U+64CE.0	fGB	0-4766
U+64CE.0	fR/S	 64.13; 64.12
U+64CE.0	fTotalStrokes	 17
U+64CE.0	fCangjie	TKQ
U+64CE.0	fFourCorner	48502
U+64CE.0	fHakka	  kiang2;  kia2; kiang3; kia3; {kia2}
U+64CE.0	fCantonese	king4
U+64CE.0	fMandarin	qing2; jing4
U+64CE.0	fEnglish	lift up; hold up; support
U+64CF.0	fUTF8	擏
U+64CF.0	fCNS11643	2-5732
U+64CF.0	fBig5	E9A7
U+64CF.0	fGB	5-4D4C
U+64CF.0	fR/S	 64.13
U+64CF.0	fTotalStrokes	 16
U+64CF.0	fCangjie	QTRK
U+64CF.0	fFourCorner	58040
U+64CF.0	fCantonese	king4; gwaan3; gwaan1 (also); geng6
U+64CF.0	fMandarin	jing4; qing2; jing3
U+64CF.0	fEnglish	to guard against, take precautions
U+64D0.0	fUTF8	擐
U+64D0.0	fCNS11643	2-5731
U+64D0.0	fBig5	E9A6
U+64D0.0	fGB	0-5F27
U+64D0.0	fR/S	 64.13
U+64D0.0	fTotalStrokes	 16
U+64D0.0	fCangjie	QWLV
U+64D0.0	fFourCorner	56032
U+64D0.0	fHakka	  fan2; kia2; k'wan5; k'wan6; kiang2; fan5
U+64D0.0	fCantonese	gwaan3; waan6
U+64D0.0	fMandarin	huan4; guan1; xuan1; guan4; juan3; quan3
U+64D0.0	fEnglish	 put on; wear
U+64D1.0	fUTF8	擑
U+64D1.0	fCNS11643	3-5076
U+64D1.0	fGB	3-4B3E
U+64D1.0	fR/S	 64.13
U+64D1.0	fTotalStrokes	 16
U+64D1.0	fCangjie	QRJI
U+64D1.0	fFourCorner	53050
U+64D1.0	fHakka	  ziap7; ziak7
U+64D1.0	fCantonese	zip3; kam4; kwaai5; daam1; daam3; daam2 (also)
U+64D1.0	fMandarin	jie1; cha2; sha4; xie2
U+64D1.0	fEnglish	same as &#x63A5;
U+64D2.0	fUTF8	擒
U+64D2.0	fCNS11643	1-7069
U+64D2.0	fBig5	BEE0
U+64D2.0	fGB	0-475C
U+64D2.0	fR/S	 64.13
U+64D2.0	fTotalStrokes	 16
U+64D2.0	fCangjie	QOYB
U+64D2.0	fFourCorner	58027
U+64D2.0	fHakka	  kim2
U+64D2.0	fCantonese	kam4
U+64D2.0	fMandarin	qin2
U+64D2.0	fEnglish	 catch; capture; seize; arrest
U+64D3.0	fUTF8	擓
U+64D3.0	fCNS11643	3-506F
U+64D3.0	fGB	E-2B7E
U+64D3.0	fR/S	 64.13
U+64D3.0	fTotalStrokes	 16
U+64D3.0	fCangjie	QSEG
U+64D3.0	fFourCorner	51010
U+64D3.0	fHakka	 kwai3; kwai1; kwai5
U+64D3.0	fCantonese	kwai5
U+64D3.0	fMandarin	kuai3
U+64D3.0	fEnglish	[1] scratch (lightly) [2] carry on the arm
U+64D4.0	fUTF8	擔
U+64D4.0	fCNS11643	1-706A
U+64D4.0	fBig5	BEE1
U+64D4.0	fGB	1-3523
U+64D4.0	fR/S	 64.13
U+64D4.0	fTotalStrokes	 16
U+64D4.0	fCangjie	QNCR
U+64D4.0	fFourCorner	57061
U+64D4.0	fHakka	  dam5
U+64D4.0	fCantonese	daam1; daam3
U+64D4.0	fMandarin	dan1; dan3
U+64D4.0	fEnglish	 shoulder; carry; bear; undertake; suffer; endure; take upon oneself
U+64D4.1	fUTF8	擔
U+64D4.1	fCNS11643	1-706A
U+64D4.1	fBig5	BEE1
U+64D4.1	fGB	1-3523
U+64D4.1	fR/S	 64.13
U+64D4.1	fTotalStrokes	 16
U+64D4.1	fCangjie	QNCR
U+64D4.1	fFourCorner	57061
U+64D4.1	fHakka	 dam1
U+64D4.1	fMandarin	dan4
U+64D4.1	fEnglish	[1] load; burden [2] used in combination [3] a unit of weight or capacity
U+64D4.2	fUTF8	擔
U+64D4.2	fCNS11643	1-706A
U+64D4.2	fBig5	BEE1
U+64D4.2	fGB	1-3523
U+64D4.2	fR/S	 64.13
U+64D4.2	fTotalStrokes	 16
U+64D4.2	fCangjie	QNCR
U+64D4.2	fFourCorner	57061
U+64D4.2	fMandarin	dan5
U+64D5.0	fUTF8	擕
U+64D5.0	fCNS11643	3-5079
U+64D5.0	fGB	E-2C21
U+64D5.0	fR/S	 64.13
U+64D5.0	fTotalStrokes	 16
U+64D5.0	fCangjie	QOGS
U+64D5.0	fFourCorner	50020
U+64D5.0	fHakka	 hi1
U+64D5.0	fCantonese	kwai4
U+64D5.0	fMandarin	xi1; xie2; xi2
U+64D6.0	fUTF8	擖
U+64D6.0	fCNS11643	2-5730
U+64D6.0	fBig5	E9A5
U+64D6.0	fGB	3-4B3B
U+64D6.0	fR/S	 64.13
U+64D6.0	fTotalStrokes	 16
U+64D6.0	fCangjie	QTAV
U+64D6.0	fFourCorner	54027
U+64D6.0	fHakka	 giet7; gat7; got7; kia5; kiet8
U+64D6.0	fCantonese	kaa1; gaa3; jip6; kat6
U+64D6.0	fMandarin	ge3; ye4; ge2; ge4; la4; lie4; qi4; zha2
U+64D6.0	fEnglish	scrape
U+64D7.0	fUTF8	擗
U+64D7.0	fCNS11643	2-572F
U+64D7.0	fBig5	E9A4
U+64D7.0	fGB	0-5F28
U+64D7.0	fR/S	 64.13
U+64D7.0	fTotalStrokes	 16
U+64D7.0	fCangjie	QSRJ
U+64D7.0	fFourCorner	50041
U+64D7.0	fHakka	 pit7;  piak7; mak7; bak7
U+64D7.0	fCantonese	pik1; pek6; bik1
U+64D7.0	fMandarin	pi3; pi4; bo4; pi1
U+64D7.0	fEnglish	 beat the breast
U+64D8.0	fUTF8	擘
U+64D8.0	fCNS11643	1-7365
U+64D8.0	fBig5	C0BC
U+64D8.0	fGB	0-6B22
U+64D8.0	fR/S	 64.13
U+64D8.0	fTotalStrokes	 17; 16
U+64D8.0	fCangjie	SJQ
U+64D8.0	fFourCorner	70502
U+64D8.0	fHakka	  mak7;  pit7; piak7;  bak7; phit7; mak8
U+64D8.0	fCantonese	maak3; baai1; mak3
U+64D8.0	fMandarin	bo4; pi4
U+64D8.0	fEnglish	[1] thumb [2] break; tear apart; rip; rend apart; cleave; split [3] outstanding person; authority in a certain field
U+64D9.0	fUTF8	擙
U+64D9.0	fCNS11643	2-5739
U+64D9.0	fBig5	E9AE
U+64D9.0	fGB	E-2C22
U+64D9.0	fR/S	 64.13
U+64D9.0	fTotalStrokes	 16
U+64D9.0	fCangjie	QHBK
U+64D9.0	fFourCorner	57032
U+64D9.0	fHakka	  au3; au6; au5
U+64D9.0	fCantonese	ngou1; aau2; ou1
U+64D9.0	fMandarin	ao4; dian4
U+64D9.0	fEnglish	 grind
U+64DA.0	fUTF8	據
U+64DA.0	fCNS11643	1-7063
U+64DA.0	fBig5	BEDA
U+64DA.0	fGB	1-3E5D
U+64DA.0	fR/S	 64.13
U+64DA.0	fTotalStrokes	 16
U+64DA.0	fCangjie	QYPO
U+64DA.0	fFourCorner	51032
U+64DA.0	fHakka	  gi3; gi5
U+64DA.0	fCantonese	geoi3
U+64DA.0	fMandarin	ju4; ju1; ji3
U+64DA.0	fEnglish	[1] occupy; take possession of; seize [2] according to; on the basis of; on grounds of [3] depend on [4] proof; evidence [5] a Chinese family name
U+64DB.0	fUTF8	擛
U+64DB.0	fCNS11643	2-5737
U+64DB.0	fBig5	E9AC
U+64DB.0	fGB	E-2C23
U+64DB.0	fR/S	 64.13
U+64DB.0	fTotalStrokes	 16
U+64DB.0	fCangjie	QTPD
U+64DB.0	fFourCorner	54094
U+64DB.0	fHakka	 jak8; zak8; rap8; jap8
U+64DB.0	fCantonese	jip6
U+64DB.0	fMandarin	ye4
U+64DC.0	fUTF8	擜
U+64DC.0	fCNS11643	3-5075
U+64DC.0	fGB	E-2C24
U+64DC.0	fR/S	 64.13
U+64DC.0	fTotalStrokes	 16
U+64DC.0	fCangjie	QJBK
U+64DC.0	fFourCorner	53030
U+64DC.0	fCantonese	kwai4
U+64DC.0	fMandarin	nie4
U+64DD.0	fUTF8	擝
U+64DD.0	fGB	1-7C3E
U+64DD.0	fR/S	 64.13
U+64DD.0	fTotalStrokes	 16
U+64DD.0	fCangjie	QABT
U+64DD.0	fCantonese	mang1; mang3
U+64DD.0	fEnglish	 pull; stretch; pluck (from pockets, boxes, etc.)
U+64DE.0	fUTF8	擞
U+64DE.0	fGB	0-4B53
U+64DE.0	fR/S	 64.13
U+64DE.0	fTotalStrokes	 16; 17
U+64DE.0	fCangjie	QFVK
U+64DE.0	fHakka	 seu3
U+64DE.0	fCantonese	sau2
U+64DE.0	fMandarin	sou3; sou4
U+64DE.0	fEnglish	simplified form of &#x64FB;
U+64DF.0	fUTF8	擟
U+64DF.0	fCNS11643	3-5464
U+64DF.0	fGB	E-2C25
U+64DF.0	fR/S	  64.14
U+64DF.0	fTotalStrokes	 17
U+64DF.0	fCangjie	QMFB
U+64DF.0	fFourCorner	51020
U+64DF.0	fMandarin	mi2
U+64E0.0	fUTF8	擠
U+64E0.0	fCNS11643	1-7366
U+64E0.0	fBig5	C0BD
U+64E0.0	fGB	1-3C37
U+64E0.0	fR/S	 64.14
U+64E0.0	fTotalStrokes	 17
U+64E0.0	fCangjie	QYX
U+64E0.0	fFourCorner	50023
U+64E0.0	fHakka	 zi1; zi5; kit8; za1
U+64E0.0	fCantonese	zai1
U+64E0.0	fMandarin	ji3; ji1; ji4
U+64E0.0	fEnglish	[1] crowd; throng; cram [2] wring; squeeze; press; twist [3] push; jostle
U+64E1.0	fUTF8	擡
U+64E1.0	fCNS11643	3-5467
U+64E1.0	fGB	5-4D68
U+64E1.0	fR/S	  64.14
U+64E1.0	fTotalStrokes	 17
U+64E1.0	fCangjie	QGRG
U+64E1.0	fFourCorner	54010
U+64E1.0	fHakka	 toi2; toi3
U+64E1.0	fCantonese	toi4
U+64E1.0	fMandarin	tai2
U+64E1.0	fEnglish	 carry; raise; lift
U+64E2.0	fUTF8	擢
U+64E2.0	fCNS11643	1-736B
U+64E2.0	fBig5	C0C2
U+64E2.0	fGB	0-5F2A
U+64E2.0	fR/S	 64.14
U+64E2.0	fTotalStrokes	 17
U+64E2.0	fCangjie	QSMG
U+64E2.0	fFourCorner	57014
U+64E2.0	fHakka	  cok8; cok7
U+64E2.0	fCantonese	zok6; zaak6
U+64E2.0	fMandarin	zhuo2
U+64E2.0	fEnglish	[1] pull out; take out; extract; pick out [2] promote; raise (in rank)
U+64E3.0	fUTF8	擣
U+64E3.0	fCNS11643	2-5C75
U+64E3.0	fBig5	ECEA
U+64E3.0	fGB	E-2C26
U+64E3.0	fR/S	 64.14
U+64E3.0	fTotalStrokes	 17
U+64E3.0	fCangjie	QGNI
U+64E3.0	fFourCorner	54041
U+64E3.0	fHakka	  dau3; do3; do1; dau5
U+64E3.0	fCantonese	dou2
U+64E3.0	fMandarin	dao3; chou2
U+64E3.0	fEnglish	[1] pound; beat [2] attack [3] harass; disturb
U+64E4.0	fUTF8	擤
U+64E4.0	fCNS11643	2-5C77
U+64E4.0	fBig5	ECEC
U+64E4.0	fGB	0-5F29
U+64E4.0	fR/S	 64.14
U+64E4.0	fTotalStrokes	 17
U+64E4.0	fCangjie	QHUL
U+64E4.0	fFourCorner	56046
U+64E4.0	fHakka	  sen5; sem5; ken3; sen3
U+64E4.0	fCantonese	sang3
U+64E4.0	fMandarin	xing3
U+64E4.0	fEnglish	 blow (the nose)
U+64E5.0	fUTF8	擥
U+64E5.0	fCNS11643	3-576B
U+64E5.0	fGB	5-4B41
U+64E5.0	fR/S	 64.15; 64.14
U+64E5.0	fTotalStrokes	 18; 19; 17
U+64E5.0	fCangjie	SWQ
U+64E5.0	fFourCorner	78500
U+64E5.0	fHakka	 lam3
U+64E5.0	fCantonese	laam5; laam2 (also)
U+64E5.0	fMandarin	lan3
U+64E5.0	fEnglish	drive away; expel; oust
U+64E6.0	fUTF8	擦
U+64E6.0	fCNS11643	1-7368
U+64E6.0	fBig5	C0BF
U+64E6.0	fGB	0-3241
U+64E6.0	fR/S	 64.14
U+64E6.0	fTotalStrokes	 17
U+64E6.0	fCangjie	QJBF
U+64E6.0	fFourCorner	53091
U+64E6.0	fHakka	  cat7
U+64E6.0	fCantonese	caat3; caat2 (also)
U+64E6.0	fMandarin	ca1
U+64E6.0	fEnglish	[1] wipe; scrub; mop; polish [2] brush; shave [3] rub; graze; scratch [4] spread on; put on
U+64E7.0	fUTF8	擧
U+64E7.0	fCNS11643	3-576D
U+64E7.0	fGB	E-2C27
U+64E7.0	fR/S	 64.14
U+64E7.0	fTotalStrokes	 18; 17
U+64E7.0	fCangjie	HCQ
U+64E7.0	fFourCorner	77500
U+64E7.0	fHakka	 gi3
U+64E7.0	fCantonese	geoi2; geng6; keoi2
U+64E7.0	fMandarin	ju3
U+64E7.0	fEnglish	 raise; lift up recommend
U+64E8.0	fUTF8	擨
U+64E8.0	fCNS11643	2-5C78
U+64E8.0	fBig5	ECED
U+64E8.0	fGB	5-4D6B
U+64E8.0	fR/S	 64.14
U+64E8.0	fTotalStrokes	 17
U+64E8.0	fCangjie	QHUO
U+64E8.0	fFourCorner	57082
U+64E8.0	fCantonese	je4; jyu5; jip3; jeoi5 (also); jyu6; aap3; ngaap3 (also); ji5; ji4 (also); wok6; zit6; ban3; ning6; ling6 (also); ning2 (also); ling2 (also); ning4; ling4 (also)
U+64E8.0	fMandarin	ye2; yi2
U+64E9.0	fUTF8	擩
U+64E9.0	fCNS11643	2-5C74
U+64E9.0	fBig5	ECE9
U+64E9.0	fGB	3-4B50
U+64E9.0	fR/S	 64.14
U+64E9.0	fTotalStrokes	 17
U+64E9.0	fCangjie	QMBB
U+64E9.0	fFourCorner	51027
U+64E9.0	fHakka	 ji5; ji2; ri5; nio1
U+64E9.0	fCantonese	jyu5; jeoi5; jau4; jyu4
U+64E9.0	fMandarin	ru3; nou4; rui4; nu3; ru4; rui2; ruo4
U+64E9.0	fEnglish	 stain; dye
U+64EA.0	fUTF8	擪
U+64EA.0	fCNS11643	3-576C
U+64EA.0	fGB	3-303C
U+64EA.0	fR/S	  64.14
U+64EA.0	fTotalStrokes	 18; 17
U+64EA.0	fCangjie	MKQ
U+64EA.0	fFourCorner	71250
U+64EA.0	fHakka	 ngiap7; ap7; jap7; rap7
U+64EA.0	fCantonese	jip3; ngaap3
U+64EA.0	fMandarin	ye4
U+64EA.0	fEnglish	 press with a finger
U+64EB.0	fUTF8	擫
U+64EB.0	fCNS11643	2-5C76
U+64EB.0	fBig5	ECEB
U+64EB.0	fGB	5-4B79
U+64EB.0	fR/S	 64.14
U+64EB.0	fTotalStrokes	 17
U+64EB.0	fCangjie	QMAK
U+64EB.0	fFourCorner	51034
U+64EB.0	fHakka	 rap8; jap8; jap7
U+64EB.0	fCantonese	jip3
U+64EB.0	fMandarin	ye4
U+64EB.0	fEnglish	same as &#x64EA;
U+64EC.0	fUTF8	擬
U+64EC.0	fCNS11643	1-7369
U+64EC.0	fBig5	C0C0
U+64EC.0	fGB	1-4462
U+64EC.0	fR/S	 64.14
U+64EC.0	fTotalStrokes	 17
U+64EC.0	fCangjie	QPKO
U+64EC.0	fFourCorner	57081
U+64EC.0	fHakka	 ngi3; gni3; ji3
U+64EC.0	fCantonese	ji5; ji4
U+64EC.0	fMandarin	ni3; yi2
U+64EC.0	fEnglish	[1] draft; draw up; design [2] intend; plan; decide; determine [3] imitate
U+64ED.0	fUTF8	擭
U+64ED.0	fCNS11643	1-736C
U+64ED.0	fBig5	C0C3
U+64ED.0	fGB	3-4B48
U+64ED.0	fR/S	 64.14; 64.13
U+64ED.0	fTotalStrokes	 17
U+64ED.0	fCangjie	QTOE
U+64ED.0	fFourCorner	54041
U+64ED.0	fHakka	 fa5; fu5; fu6; fa1
U+64ED.0	fCantonese	wok6; faa6
U+64ED.0	fMandarin	huo4; hu4; wo4; hu2; hua2
U+64ED.0	fEnglish	[1] trap; snare [2] trap; catch; seize
U+64EE.0	fUTF8	擮
U+64EE.0	fCNS11643	3-5469
U+64EE.0	fGB	E-2C28
U+64EE.0	fR/S	  64.14
U+64EE.0	fTotalStrokes	 17
U+64EE.0	fCangjie	QJIG
U+64EE.0	fFourCorner	53050
U+64EE.0	fMandarin	ji2
U+64EF.0	fUTF8	擯
U+64EF.0	fCNS11643	2-5C73
U+64EF.0	fBig5	ECE8
U+64EF.0	fGB	1-3177
U+64EF.0	fR/S	 64.14
U+64EF.0	fTotalStrokes	 17
U+64EF.0	fCangjie	QJMC
U+64EF.0	fFourCorner	53086
U+64EF.0	fHakka	 bin5
U+64EF.0	fCantonese	ban3
U+64EF.0	fMandarin	bin4; bin1
U+64EF.0	fEnglish	[1] reject; oust; get rid of; discard [2] usher
U+64F0.0	fUTF8	擰
U+64F0.0	fCNS11643	1-7367
U+64F0.0	fBig5	C0BE
U+64F0.0	fGB	1-4521
U+64F0.0	fR/S	 64.14
U+64F0.0	fTotalStrokes	 17
U+64F0.0	fCangjie	QJPN
U+64F0.0	fFourCorner	53021
U+64F0.0	fHakka	  nen2; len2
U+64F0.0	fCantonese	ling6; ling2; ning4; ning6; ning1
U+64F0.0	fMandarin	ning2
U+64F0.0	fEnglish	 pinch; twist; wring
U+64F0.1	fUTF8	擰
U+64F0.1	fCNS11643	1-7367
U+64F0.1	fBig5	C0BE
U+64F0.1	fGB	1-4521
U+64F0.1	fR/S	 64.14
U+64F0.1	fTotalStrokes	 17
U+64F0.1	fCangjie	QJPN
U+64F0.1	fFourCorner	53021
U+64F0.1	fMandarin	ning3
U+64F0.1	fEnglish	[1] wrench; twist; screw [2] wrong; mistaken [3] differ; disagree
U+64F0.2	fUTF8	擰
U+64F0.2	fCNS11643	1-7367
U+64F0.2	fBig5	C0BE
U+64F0.2	fGB	1-4521
U+64F0.2	fR/S	 64.14
U+64F0.2	fTotalStrokes	 17
U+64F0.2	fCangjie	QJPN
U+64F0.2	fFourCorner	53021
U+64F0.2	fMandarin	ning4
U+64F0.2	fEnglish	determined; dogged; stubborn
U+64F1.0	fUTF8	擱
U+64F1.0	fCNS11643	1-736A
U+64F1.0	fBig5	C0C1
U+64F1.0	fGB	1-3869
U+64F1.0	fR/S	 64.14
U+64F1.0	fTotalStrokes	 17
U+64F1.0	fCangjie	QANR
U+64F1.0	fFourCorner	57020
U+64F1.0	fHakka	  gok7
U+64F1.0	fCantonese	gok3
U+64F1.0	fMandarin	ge1; ge2
U+64F1.0	fEnglish	[1] leave; put; lay [2] file [3] delay; put aside
U+64F1.1	fUTF8	擱
U+64F1.1	fCNS11643	1-736A
U+64F1.1	fBig5	C0C1
U+64F1.1	fGB	1-3869
U+64F1.1	fR/S	 64.14
U+64F1.1	fTotalStrokes	 17
U+64F1.1	fCangjie	QANR
U+64F1.1	fFourCorner	57020
U+64F1.1	fMandarin	ge5
U+64F2.0	fUTF8	擲
U+64F2.0	fCNS11643	1-7645
U+64F2.0	fBig5	C259
U+64F2.0	fGB	1-5640
U+64F2.0	fR/S	 64.15
U+64F2.0	fTotalStrokes	 18
U+64F2.0	fCangjie	QTKL
U+64F2.0	fFourCorner	57027
U+64F2.0	fHakka	 chit7; ts'iit7; tiat7; cit7; wang3; {zhim3}; {zihm3}
U+64F2.0	fCantonese	zaak6
U+64F2.0	fMandarin	zhi4; zhi2; zhi1
U+64F2.0	fEnglish	 throw; cast
U+64F3.0	fUTF8	擳
U+64F3.0	fCNS11643	2-5738
U+64F3.0	fBig5	E9AD
U+64F3.0	fGB	3-4974
U+64F3.0	fR/S	 64.15
U+64F3.0	fTotalStrokes	 18
U+64F3.0	fCangjie	QHAL
U+64F3.0	fFourCorner	58027
U+64F3.0	fHakka	 ziet8
U+64F3.0	fCantonese	zit1; kwok3; kok3 (also); kong3 (also); kwong3 (also); mo1; zin3; zit3; kit3; git3 (also)
U+64F3.0	fMandarin	jie2
U+64F3.0	fEnglish	to squeeze out; tickle
U+64F4.0	fUTF8	擴
U+64F4.0	fCNS11643	1-7644
U+64F4.0	fBig5	C258
U+64F4.0	fGB	1-4029
U+64F4.0	fR/S	 64.15
U+64F4.0	fTotalStrokes	 18
U+64F4.0	fCangjie	QITC
U+64F4.0	fFourCorner	50086
U+64F4.0	fHakka	 kok7; kong5; kwok7
U+64F4.0	fCantonese	gwok3; kwong3; kwok3; kong3
U+64F4.0	fMandarin	kuo4
U+64F4.0	fEnglish	 expand; enlarge; stretch; magnify; extend
U+64F5.0	fUTF8	擵
U+64F5.0	fCNS11643	7-3F62
U+64F5.0	fGB	E-2C29
U+64F5.0	fR/S	 64.15
U+64F5.0	fTotalStrokes	 18
U+64F5.0	fCangjie	QIDQ
U+64F5.0	fFourCorner	50050
U+64F5.0	fHakka	 6520
U+64F5.0	fCantonese	mo1; mo2
U+64F5.0	fMandarin	mo2; ma1; mi2
U+64F5.0	fEnglish	to touch, feel with the hand
U+64F6.0	fUTF8	擶
U+64F6.0	fCNS11643	3-576A
U+64F6.0	fGB	E-2C2A
U+64F6.0	fR/S	  64.15
U+64F6.0	fTotalStrokes	 18
U+64F6.0	fCangjie	QHTN
U+64F6.0	fFourCorner	58020
U+64F6.0	fMandarin	jian4
U+64F7.0	fUTF8	擷
U+64F7.0	fCNS11643	1-764A
U+64F7.0	fBig5	C25E
U+64F7.0	fGB	1-5F22
U+64F7.0	fR/S	 64.15
U+64F7.0	fTotalStrokes	 18
U+64F7.0	fCangjie	QGRC
U+64F7.0	fFourCorner	51086
U+64F7.0	fHakka	  giet7; get7
U+64F7.0	fCantonese	kit3; git3
U+64F7.0	fMandarin	xie2; jie2
U+64F7.0	fEnglish	 pick; collect; gather
U+64F8.0	fUTF8	擸
U+64F8.0	fCNS11643	2-6160
U+64F8.0	fBig5	EFD4
U+64F8.0	fGB	3-4B56
U+64F8.0	fR/S	 64.15
U+64F8.0	fTotalStrokes	 18
U+64F8.0	fCangjie	QVVV
U+64F8.0	fFourCorner	52016
U+64F8.0	fHakka	 liap8;  lap8; lap7
U+64F8.0	fCantonese	laap3; laap6; lip6; lip3
U+64F8.0	fMandarin	lie4; la4; la1
U+64F8.0	fEnglish	 hold; grasp to hold the hair pull at
U+64F9.0	fUTF8	擹
U+64F9.0	fCNS11643	3-5770
U+64F9.0	fGB	E-2C2B
U+64F9.0	fR/S	  64.15
U+64F9.0	fTotalStrokes	 18
U+64F9.0	fCangjie	QTOO
U+64F9.0	fFourCorner	57080
U+64F9.0	fHakka	  tan1; lan3
U+64F9.0	fCantonese	taan1
U+64F9.0	fMandarin	tan1; nan3; nan4
U+64F9.0	fEnglish	same as &#x6524;
U+64FA.0	fUTF8	擺
U+64FA.0	fCNS11643	1-7648
U+64FA.0	fBig5	C25C
U+64FA.0	fGB	1-305A
U+64FA.0	fR/S	 64.15
U+64FA.0	fTotalStrokes	 18
U+64FA.0	fCangjie	QWLP
U+64FA.0	fFourCorner	56011
U+64FA.0	fHakka	  bai3; bai2; bai1
U+64FA.0	fCantonese	baai2
U+64FA.0	fMandarin	bai3
U+64FA.0	fEnglish	[1] put; place; display; arrange [2] wave; swing; sway; oscillate; wag
U+64FB.0	fUTF8	擻
U+64FB.0	fCNS11643	1-7649
U+64FB.0	fBig5	C25D
U+64FB.0	fGB	1-4B53
U+64FB.0	fR/S	 64.15
U+64FB.0	fTotalStrokes	 18
U+64FB.0	fCangjie	QLVK
U+64FB.0	fFourCorner	58040
U+64FB.0	fHakka	 seu5; seu3
U+64FB.0	fCantonese	sau2; sau3; mei4; zai1
U+64FB.0	fMandarin	sou3; sou4
U+64FB.0	fEnglish	 shake; tremble; quake; flutter
U+64FC.0	fUTF8	擼
U+64FC.0	fCNS11643	2-6163
U+64FC.0	fBig5	EFD7
U+64FC.0	fGB	1-5F23
U+64FC.0	fR/S	 64.15
U+64FC.0	fTotalStrokes	 18
U+64FC.0	fCangjie	QNWA
U+64FC.0	fFourCorner	57063
U+64FC.0	fCantonese	lou5
U+64FC.0	fMandarin	lu3; lu1
U+64FC.0	fEnglish	[1] wipe [2] roll up
U+64FD.0	fUTF8	擽
U+64FD.0	fCNS11643	2-615F
U+64FD.0	fBig5	EFD3
U+64FD.0	fGB	3-4A21
U+64FD.0	fR/S	 64.15
U+64FD.0	fTotalStrokes	 18
U+64FD.0	fCangjie	QVID
U+64FD.0	fFourCorner	52094
U+64FD.0	fHakka	 liok8
U+64FD.0	fCantonese	loek6 (also); lik6; lik1
U+64FD.0	fMandarin	li4; luo4; le4; luue4; lun4; yao4; yue4
U+64FD.0	fEnglish	 tickle ticklish; funny
U+64FE.0	fUTF8	擾
U+64FE.0	fCNS11643	1-7646
U+64FE.0	fBig5	C25A
U+64FE.0	fGB	1-4845
U+64FE.0	fR/S	 64.15
U+64FE.0	fTotalStrokes	 18
U+64FE.0	fCangjie	QMBE
U+64FE.0	fFourCorner	51047
U+64FE.0	fHakka	 jau3; rau3;  jau1; jeu3; jeu1; ngiau1
U+64FE.0	fCantonese	jiu2; jiu5 (also)
U+64FE.0	fMandarin	rao3; rou2
U+64FE.0	fEnglish	[1] disturb; agitate; harass; trouble [2] trespass on somebody's hospitality [3] disorder
U+64FF.0	fUTF8	擿
U+64FF.0	fCNS11643	2-615D
U+64FF.0	fBig5	EFD1
U+64FF.0	fGB	3-4B51
U+64FF.0	fR/S	 64.15
U+64FF.0	fTotalStrokes	 18
U+64FF.0	fCangjie	QYYB
U+64FF.0	fFourCorner	50032
U+64FF.0	fHakka	 chit7; zak7; ts'iit7; dit7; tit7
U+64FF.0	fCantonese	tik1; zaak6; zek3; zak6
U+64FF.0	fMandarin	zhi2; ti4; di4; zhi4; zhai1; zhe2
U+64FF.0	fEnglish	[1] instigate; incite [2] expose
U+6500.0	fUTF8	攀
U+6500.0	fCNS11643	1-7838
U+6500.0	fBig5	C36B
U+6500.0	fGB	0-454A
U+6500.0	fR/S	 64.15
U+6500.0	fTotalStrokes	 19
U+6500.0	fCangjie	DDKQ
U+6500.0	fFourCorner	44502
U+6500.0	fHakka	 ban1; pan1
U+6500.0	fCantonese	paan1
U+6500.0	fMandarin	pan1
U+6500.0	fEnglish	[1] hold to; climb; hang on; clamber [2] involve
U+6501.0	fUTF8	攁
U+6501.0	fCNS11643	2-6161
U+6501.0	fBig5	EFD5
U+6501.0	fGB	5-4D6E
U+6501.0	fR/S	 64.15
U+6501.0	fTotalStrokes	 18
U+6501.0	fCangjie	QTOV
U+6501.0	fFourCorner	58032
U+6501.0	fHakka	 jong3
U+6501.0	fCantonese	joeng5
U+6501.0	fMandarin	yang3
U+6502.0	fUTF8	攂
U+6502.0	fCNS11643	4-5B36
U+6502.0	fGB	5-4D73
U+6502.0	fR/S	  64.15
U+6502.0	fTotalStrokes	 18
U+6502.0	fCangjie	QWWW
U+6502.0	fFourCorner	56060
U+6502.0	fHakka	  lui2; lui6; lui5; lui1
U+6502.0	fCantonese	leoi4
U+6502.0	fMandarin	lei4; lei2
U+6502.0	fEnglish	 beat a drum
U+6503.0	fUTF8	攃
U+6503.0	fCNS11643	2-6162
U+6503.0	fBig5	EFD6
U+6503.0	fGB	3-4B4F
U+6503.0	fR/S	 64.15
U+6503.0	fTotalStrokes	 18
U+6503.0	fCangjie	QTBF
U+6503.0	fFourCorner	54091
U+6503.0	fHakka	 sat7; cat7
U+6503.0	fCantonese	caat3; saat3
U+6503.0	fMandarin	sa4; ca3
U+6503.0	fEnglish	preceding
U+6504.0	fUTF8	攄
U+6504.0	fCNS11643	2-615E
U+6504.0	fBig5	EFD2
U+6504.0	fGB	1-5E73
U+6504.0	fR/S	 64.15
U+6504.0	fTotalStrokes	 18
U+6504.0	fCangjie	QYPP
U+6504.0	fFourCorner	51036
U+6504.0	fHakka	 shu1; su1; chi1; li6; cih1
U+6504.0	fCantonese	syu1
U+6504.0	fMandarin	shu1; chu1
U+6504.0	fEnglish	 spread; disperse; vent; make known
U+6505.0	fUTF8	攅
U+6505.0	fCNS11643	3-576E
U+6505.0	fGB	5-4D4B
U+6505.0	fR/S	 64.15
U+6505.0	fTotalStrokes	 18
U+6505.0	fCangjie	QQOC
U+6505.0	fFourCorner	55080
U+6505.0	fHakka	 zon1; zan1
U+6505.0	fCantonese	cyun4; zaan2
U+6505.0	fMandarin	zan3; zuan1; zuan4
U+6505.0	fEnglish	save, hoard
U+6506.0	fUTF8	攆
U+6506.0	fCNS11643	1-7647
U+6506.0	fBig5	C25B
U+6506.0	fGB	1-446C
U+6506.0	fR/S	 64.15
U+6506.0	fTotalStrokes	 18
U+6506.0	fCangjie	QQOJ
U+6506.0	fFourCorner	55053
U+6506.0	fHakka	  lien3;  lien1; len1; len3
U+6506.0	fCantonese	lin5
U+6506.0	fMandarin	nian3
U+6506.0	fEnglish	[1] drive away; expel; oust [2] catch up
U+6507.0	fUTF8	攇
U+6507.0	fCNS11643	2-654D
U+6507.0	fBig5	F242
U+6507.0	fGB	3-4B59
U+6507.0	fR/S	 64.16
U+6507.0	fTotalStrokes	 19
U+6507.0	fCangjie	QJQP
U+6507.0	fFourCorner	53036
U+6507.0	fCantonese	hin2
U+6507.0	fMandarin	xian3
U+6508.0	fUTF8	攈
U+6508.0	fCNS11643	3-5A3B
U+6508.0	fGB	5-4D77
U+6508.0	fR/S	  64.16
U+6508.0	fTotalStrokes	 19
U+6508.0	fCangjie	QIPD
U+6508.0	fFourCorner	50094
U+6508.0	fCantonese	kwan2; kwan3 (also)
U+6508.0	fMandarin	jun4
U+6509.0	fUTF8	攉
U+6509.0	fCNS11643	2-6550
U+6509.0	fBig5	F245
U+6509.0	fGB	0-5F2B
U+6509.0	fR/S	 64.16
U+6509.0	fTotalStrokes	 19
U+6509.0	fCangjie	QMBG
U+6509.0	fFourCorner	51014
U+6509.0	fHakka	 kok7; jak8
U+6509.0	fCantonese	fok3; kok3; wok6
U+6509.0	fMandarin	huo4; que4; huo1
U+6509.0	fEnglish	 beckon urge
U+650A.0	fUTF8	攊
U+650A.0	fCNS11643	3-5A3C
U+650A.0	fGB	E-2C2C
U+650A.0	fR/S	 64.16
U+650A.0	fTotalStrokes	 19
U+650A.0	fCangjie	QMHM
U+650A.0	fFourCorner	51010
U+650A.0	fCantonese	lik6
U+650A.0	fMandarin	li4; luue4
U+650B.0	fUTF8	攋
U+650B.0	fCNS11643	3-5A3A
U+650B.0	fGB	3-4B49
U+650B.0	fR/S	  64.16
U+650B.0	fTotalStrokes	 19
U+650B.0	fCangjie	QDLC
U+650B.0	fFourCorner	57080
U+650B.0	fHakka	  lat8; {lai2}; lai6; lai5
U+650B.0	fCantonese	laai6; laa2
U+650B.0	fMandarin	la4
U+650B.0	fEnglish	 clutch grab at rub; scrape tear
U+650C.0	fUTF8	攌
U+650C.0	fCNS11643	2-6551
U+650C.0	fBig5	F246
U+650C.0	fGB	3-4B58
U+650C.0	fR/S	 64.16
U+650C.0	fTotalStrokes	 19
U+650C.0	fCangjie	QWWV
U+650C.0	fFourCorner	56000
U+650C.0	fCantonese	waan6; waan5
U+650C.0	fMandarin	han4; wan3; huan3
U+650D.0	fUTF8	攍
U+650D.0	fCNS11643	2-654F
U+650D.0	fBig5	F244
U+650D.0	fGB	5-4D78
U+650D.0	fR/S	 64.16
U+650D.0	fTotalStrokes	 19
U+650D.0	fCangjie	QYNV
U+650D.0	fFourCorner	50017
U+650D.0	fCantonese	jing4
U+650D.0	fMandarin	ying2
U+650E.0	fUTF8	攎
U+650E.0	fCNS11643	2-6552
U+650E.0	fBig5	F247
U+650E.0	fGB	3-4979
U+650E.0	fR/S	 64.16
U+650E.0	fTotalStrokes	 19
U+650E.0	fCangjie	QYPT
U+650E.0	fFourCorner	51017
U+650E.0	fHakka	  lu2; lu1
U+650E.0	fCantonese	lou4; lo4
U+650E.0	fMandarin	lu2; lu4; luo2
U+650E.0	fEnglish	[1] grasp; [2] choose [3] spread out
U+650F.0	fUTF8	攏
U+650F.0	fCNS11643	1-7839
U+650F.0	fBig5	C36C
U+650F.0	fGB	1-4223
U+650F.0	fR/S	 64.16
U+650F.0	fTotalStrokes	 19
U+650F.0	fCangjie	QYBP
U+650F.0	fFourCorner	51011
U+650F.0	fHakka	 lung1;  lung3; nung1; nung3
U+650F.0	fCantonese	lung5; lung2 (also); lung4
U+650F.0	fMandarin	long3
U+650F.0	fEnglish	[1] collect; bring together; gather [2] lean [3] come alongside; approach; reach [4] add up; sum up [5] comb (hair)
U+650F.1	fUTF8	攏
U+650F.1	fCNS11643	1-7839
U+650F.1	fBig5	C36C
U+650F.1	fGB	1-4223
U+650F.1	fR/S	 64.16
U+650F.1	fTotalStrokes	 19
U+650F.1	fCangjie	QYBP
U+650F.1	fFourCorner	51011
U+650F.1	fMandarin	long2
U+650F.1	fEnglish	a special fingering in playing the lute
U+6510.0	fUTF8	攐
U+6510.0	fCNS11643	2-654E
U+6510.0	fBig5	F243
U+6510.0	fGB	5-4D7A
U+6510.0	fR/S	 64.16
U+6510.0	fTotalStrokes	 19
U+6510.0	fCangjie	QJTV
U+6510.0	fFourCorner	53032
U+6510.0	fHakka	 kien1; ken1
U+6510.0	fCantonese	hin1; hang1
U+6510.0	fMandarin	qian1
U+6510.0	fEnglish	[1] pluck up; seize [2] underclothes [3] hold up one's skirts
U+6511.0	fUTF8	攑
U+6511.0	fCNS11643	3-5E31
U+6511.0	fGB	5-4C72
U+6511.0	fR/S	 64.17
U+6511.0	fTotalStrokes	 20; 21; 19
U+6511.0	fCangjie	QHCQ
U+6511.0	fFourCorner	57050
U+6511.0	fCantonese	hin1
U+6511.0	fMandarin	qian1
U+6512.0	fUTF8	攒
U+6512.0	fGB	0-545C
U+6512.0	fR/S	 64.16
U+6512.0	fTotalStrokes	 19
U+6512.0	fCangjie	XQHUO
U+6512.0	fHakka	 zon1; zan1
U+6512.0	fCantonese	cyun4; zaan2; zyun2 (also)
U+6512.0	fMandarin	zan3; zuan1; zuan4
U+6512.0	fEnglish	simplified form of &#x6522;
U+6513.0	fUTF8	攓
U+6513.0	fCNS11643	2-687A
U+6513.0	fBig5	F44E
U+6513.0	fGB	5-4E21
U+6513.0	fR/S	 64.17
U+6513.0	fTotalStrokes	 20
U+6513.0	fCangjie	QJTO
U+6513.0	fFourCorner	53081
U+6513.0	fCantonese	hin1
U+6513.0	fMandarin	qian1; jian3
U+6514.0	fUTF8	攔
U+6514.0	fCNS11643	1-796F
U+6514.0	fBig5	C464
U+6514.0	fGB	1-4039
U+6514.0	fR/S	 64.17
U+6514.0	fTotalStrokes	 20
U+6514.0	fCangjie	QANW
U+6514.0	fFourCorner	57020
U+6514.0	fHakka	  lan2; lan3
U+6514.0	fCantonese	laan4
U+6514.0	fMandarin	lan2
U+6514.0	fEnglish	 obstruct; impede; bar; hinder; block; hold back
U+6515.0	fUTF8	攕
U+6515.0	fCNS11643	2-6879
U+6515.0	fBig5	F44D
U+6515.0	fGB	5-4D7C
U+6515.0	fR/S	 64.17
U+6515.0	fTotalStrokes	 20
U+6515.0	fCangjie	QOIM
U+6515.0	fFourCorner	53050
U+6515.0	fCantonese	cim1; saam1 (also)
U+6515.0	fMandarin	san1; xian1; jian1
U+6516.0	fUTF8	攖
U+6516.0	fCNS11643	2-6878
U+6516.0	fBig5	F44C
U+6516.0	fGB	1-5E7C
U+6516.0	fR/S	 64.17
U+6516.0	fTotalStrokes	 20
U+6516.0	fCangjie	QBCV
U+6516.0	fFourCorner	56044
U+6516.0	fHakka	 jin1; rin1
U+6516.0	fCantonese	jing1
U+6516.0	fMandarin	ying1; ying2
U+6516.0	fEnglish	[1] irritate; offend [2] disturb; stir up
U+6517.0	fUTF8	攗
U+6517.0	fCNS11643	2-6877
U+6517.0	fBig5	F44B
U+6517.0	fGB	3-4B5B
U+6517.0	fR/S	 64.17
U+6517.0	fTotalStrokes	 20
U+6517.0	fCangjie	QIPD
U+6517.0	fFourCorner	50094
U+6517.0	fCantonese	mei4; kwan2
U+6517.0	fMandarin	mei2
U+6518.0	fUTF8	攘
U+6518.0	fCNS11643	1-796E
U+6518.0	fBig5	C463
U+6518.0	fGB	0-4841
U+6518.0	fR/S	 64.17
U+6518.0	fTotalStrokes	 20
U+6518.0	fCangjie	QYRV
U+6518.0	fFourCorner	50032
U+6518.0	fHakka	 jong2; jong5; rong2; ngiong5; ngiong3; rong5; ngion2
U+6518.0	fCantonese	joeng4; joeng5; joeng6 (also)
U+6518.0	fMandarin	rang4; rang3; shang4; ning2; xiang3
U+6518.0	fEnglish	confused; disorderly
U+6518.1	fUTF8	攘
U+6518.1	fCNS11643	1-796E
U+6518.1	fBig5	C463
U+6518.1	fGB	0-4841
U+6518.1	fR/S	 64.17
U+6518.1	fTotalStrokes	 20
U+6518.1	fCangjie	QYRV
U+6518.1	fFourCorner	50032
U+6518.1	fMandarin	rang2
U+6518.1	fEnglish	[1] seize; take by force [2] repel; eliminate; resist
U+6519.0	fUTF8	攙
U+6519.0	fCNS11643	1-7970
U+6519.0	fBig5	C465
U+6519.0	fGB	1-3273
U+6519.0	fR/S	 64.17
U+6519.0	fTotalStrokes	 20
U+6519.0	fCangjie	QNRI
U+6519.0	fFourCorner	57013
U+6519.0	fHakka	  cam1;  cam2; cen2
U+6519.0	fCantonese	caam1; caam4
U+6519.0	fMandarin	chan1; chan2; chan4; chen1
U+6519.0	fEnglish	[1] give helping hand; lead (a person) by the hand; support or assist somebody while standing or walking [2] mix; blend
U+651A.0	fUTF8	攚
U+651A.0	fCNS11643	3-6523
U+651A.0	fGB	E-2C2D
U+651A.0	fR/S	 64.17
U+651A.0	fTotalStrokes	 20
U+651A.0	fCangjie	QFFR
U+651A.0	fFourCorner	5906
U+651A.0	fCantonese	jing4
U+651B.0	fUTF8	攛
U+651B.0	fCNS11643	2-6B5B
U+651B.0	fBig5	F5CD
U+651B.0	fGB	1-5F25
U+651B.0	fR/S	 64.18
U+651B.0	fTotalStrokes	 21
U+651B.0	fCangjie	QJCV
U+651B.0	fFourCorner	43017
U+651B.0	fHakka	 con3; chon3; con5
U+651B.0	fCantonese	cyun3; cyun2 (also); cyun1; syu2
U+651B.0	fMandarin	cuan1; cuan4
U+651B.0	fEnglish	[1] hurry; do in a hurry [2] throw; fling [3] urge; persuade
U+651C.0	fUTF8	攜
U+651C.0	fCNS11643	1-7A6D
U+651C.0	fBig5	C4E2
U+651C.0	fGB	E-2C2E
U+651C.0	fR/S	 64.18
U+651C.0	fTotalStrokes	 21
U+651C.0	fCangjie	QUOB
U+651C.0	fFourCorner	52027
U+651C.0	fHakka	  hi1
U+651C.0	fCantonese	kwai4
U+651C.0	fMandarin	xie2; xi1; xi2
U+651C.0	fEnglish	[1] lead; help [2] take; carry
U+651D.0	fUTF8	攝
U+651D.0	fCNS11643	1-7A6C
U+651D.0	fBig5	C4E1
U+651D.0	fGB	1-4963
U+651D.0	fR/S	 64.18
U+651D.0	fTotalStrokes	 21
U+651D.0	fCangjie	QSJJ
U+651D.0	fFourCorner	51041
U+651D.0	fHakka	  ngiap7; gniap7; jap7
U+651D.0	fCantonese	sip3; nip6; zip3; lip6 (also)
U+651D.0	fMandarin	she4; nie2; nie4; sha4; zhe2
U+651D.0	fEnglish	[1] take in; absorb; attract [2] take a photograph (or a shot) of [3] regulate [4] represent
U+651E.0	fUTF8	攞
U+651E.0	fCNS11643	3-5F4A
U+651E.0	fGB	5-4C50
U+651E.0	fR/S	  64.19
U+651E.0	fTotalStrokes	 22
U+651E.0	fCangjie	QWLG
U+651E.0	fFourCorner	56010
U+651E.0	fHakka	 lo1;  lo2; lo3
U+651E.0	fCantonese	lo2; lyut3; lo1
U+651E.0	fMandarin	luo3; luo2; luo1; l3
U+651E.0	fEnglish	 tuck; pull
U+651F.0	fUTF8	攟
U+651F.0	fCNS11643	3-5F4B
U+651F.0	fGB	5-4E25
U+651F.0	fR/S	 64.19
U+651F.0	fTotalStrokes	 22
U+651F.0	fCangjie	QIPW
U+651F.0	fFourCorner	50060
U+651F.0	fCantonese	kwan2; kwan3 (also)
U+651F.0	fMandarin	jun4
U+6520.0	fUTF8	攠
U+6520.0	fCNS11643	2-6D4F
U+6520.0	fBig5	F6E1
U+6520.0	fGB	3-4B5D
U+6520.0	fR/S	 64.19
U+6520.0	fTotalStrokes	 22
U+6520.0	fCangjie	QIDY
U+6520.0	fFourCorner	50011
U+6520.0	fCantonese	mei4
U+6520.0	fMandarin	mi2; ma1; mo2
U+6521.0	fUTF8	攡
U+6521.0	fCNS11643	2-6D4E
U+6521.0	fBig5	F6E0
U+6521.0	fGB	3-4B21
U+6521.0	fR/S	 64.19
U+6521.0	fTotalStrokes	 22
U+6521.0	fCangjie	QYBG
U+6521.0	fFourCorner	50014
U+6521.0	fCantonese	ci1; lei4 (also)
U+6521.0	fMandarin	li2; chi1
U+6522.0	fUTF8	攢
U+6522.0	fCNS11643	2-6D51
U+6522.0	fBig5	F6E3
U+6522.0	fGB	1-545C
U+6522.0	fR/S	 64.19
U+6522.0	fTotalStrokes	 22
U+6522.0	fCangjie	QHUC
U+6522.0	fFourCorner	54086
U+6522.0	fHakka	 zon5; zon1;  zan1; zan3; can3
U+6522.0	fCantonese	cyun4; zaan2; zyun2 (also); zaan6; zyun1; zyun6
U+6522.0	fMandarin	zan3; zuan1; zuan4
U+6522.0	fEnglish	 save; hoard; accumulate
U+6522.1	fUTF8	攢
U+6522.1	fCNS11643	2-6D51
U+6522.1	fBig5	F6E3
U+6522.1	fGB	1-545C
U+6522.1	fR/S	 64.19
U+6522.1	fTotalStrokes	 22
U+6522.1	fCangjie	QHUC
U+6522.1	fFourCorner	54086
U+6522.1	fMandarin	cuan2
U+6522.1	fEnglish	 bring together; gather; assemble; collect
U+6523.0	fUTF8	攣
U+6523.0	fCNS11643	1-7C37
U+6523.0	fBig5	C5CB
U+6523.0	fGB	1-424E
U+6523.0	fR/S	 64.19
U+6523.0	fTotalStrokes	 23; 22
U+6523.0	fCangjie	VFQ
U+6523.0	fFourCorner	22502
U+6523.0	fHakka	  lan2; lien2; len2; lien5
U+6523.0	fCantonese	lyun4; lyun1 (also)
U+6523.0	fMandarin	luan2; lian4; luan4
U+6523.0	fEnglish	[1] tangled; entwined [2] crooked [3] contraction
U+6524.0	fUTF8	攤
U+6524.0	fCNS11643	1-7B61
U+6524.0	fBig5	C575
U+6524.0	fGB	1-4C2F
U+6524.0	fR/S	 64.19
U+6524.0	fTotalStrokes	 22
U+6524.0	fCangjie	QTOG
U+6524.0	fFourCorner	50014
U+6524.0	fHakka	  tan1; lan3
U+6524.0	fCantonese	taan1
U+6524.0	fMandarin	tan1; nan3; nan4
U+6524.0	fEnglish	[1] spread; open [2] divide equally; apportion [3] booth; stand; stall [4] collection of liquid; pool of (water; mud; blood, etc.)
U+6525.0	fUTF8	攥
U+6525.0	fCNS11643	2-6F2C
U+6525.0	fBig5	F7DD
U+6525.0	fGB	0-5F2C
U+6525.0	fR/S	 64.20
U+6525.0	fTotalStrokes	 23
U+6525.0	fCangjie	QHBF
U+6525.0	fFourCorner	58093
U+6525.0	fCantonese	zaan6; zyut3
U+6525.0	fMandarin	zuan4
U+6525.0	fEnglish	 grasp; grip; clutch
U+6526.0	fUTF8	攦
U+6526.0	fCNS11643	2-6D50
U+6526.0	fBig5	F6E2
U+6526.0	fGB	3-4A39
U+6526.0	fR/S	 64.19
U+6526.0	fTotalStrokes	 22
U+6526.0	fCangjie	QMMP
U+6526.0	fFourCorner	51011
U+6526.0	fCantonese	lai6
U+6526.0	fMandarin	li4; xi3; li3; lie4; shai4
U+6527.0	fUTF8	攧
U+6527.0	fCNS11643	3-5F49
U+6527.0	fGB	3-4B57
U+6527.0	fR/S	  64.19
U+6527.0	fTotalStrokes	 22
U+6527.0	fCangjie	QJCC
U+6527.0	fFourCorner	51086
U+6527.0	fCantonese	din1; dit6
U+6527.0	fMandarin	dian1; die2
U+6527.0	fEnglish	 stumble; fall
U+6528.0	fUTF8	攨
U+6528.0	fCNS11643	3-6032
U+6528.0	fGB	E-2C2F
U+6528.0	fR/S	 64.20
U+6528.0	fTotalStrokes	 23
U+6528.0	fCangjie	QJCO
U+6528.0	fFourCorner	53030
U+6528.0	fHakka	 wa1; wa6; wa5; va1; va5; na5
U+6528.0	fCantonese	waa2; we2
U+6528.0	fMandarin	wa1
U+6528.0	fEnglish	same as &#x6432;
U+6529.0	fUTF8	攩
U+6529.0	fCNS11643	2-6F2B
U+6529.0	fBig5	F7DC
U+6529.0	fGB	E-2C30
U+6529.0	fR/S	 64.20
U+6529.0	fTotalStrokes	 23
U+6529.0	fCangjie	QFBF
U+6529.0	fFourCorner	59031
U+6529.0	fHakka	 dong3; dung3
U+6529.0	fCantonese	dong2; dong3
U+6529.0	fMandarin	dang3; dang4; tang3; huang4; zhang1
U+6529.0	fEnglish	[1] obstruct; impede; hinder; block; stop; resist [2] party; faction; clique; association
U+652A.0	fUTF8	攪
U+652A.0	fCNS11643	1-7C39
U+652A.0	fBig5	C5CD
U+652A.0	fGB	1-3D41
U+652A.0	fR/S	 64.20
U+652A.0	fTotalStrokes	 23
U+652A.0	fCangjie	QHBU
U+652A.0	fFourCorner	57016
U+652A.0	fHakka	  gau3; gau2; giau1; giau3
U+652A.0	fCantonese	gaau2
U+652A.0	fMandarin	jiao3; xiao3
U+652A.0	fEnglish	[1] disturb; agitate; annoy [2] stir; mix
U+652A.1	fUTF8	攪
U+652A.1	fCNS11643	1-7C39
U+652A.1	fBig5	C5CD
U+652A.1	fGB	1-3D41
U+652A.1	fR/S	 64.20
U+652A.1	fTotalStrokes	 23
U+652A.1	fCangjie	QHBU
U+652A.1	fFourCorner	57016
U+652A.1	fMandarin	gao3
U+652A.1	fEnglish	 do; manage; handle
U+652B.0	fUTF8	攫
U+652B.0	fCNS11643	1-7C38
U+652B.0	fBig5	C5CC
U+652B.0	fGB	0-3E70
U+652B.0	fR/S	 64.20
U+652B.0	fTotalStrokes	 23
U+652B.0	fCangjie	QBUE
U+652B.0	fFourCorner	56047
U+652B.0	fHakka	 kiok7; giok7
U+652B.0	fCantonese	fok3; gwok3
U+652B.0	fMandarin	jue2; ju1
U+652B.0	fEnglish	 snatch; seize; catch; take hold
U+652C.0	fUTF8	攬
U+652C.0	fCNS11643	1-7C5F
U+652C.0	fBig5	C5F3
U+652C.0	fGB	1-403F
U+652C.0	fR/S	 64.21
U+652C.0	fTotalStrokes	 24
U+652C.0	fCangjie	QSWU
U+652C.0	fFourCorner	58016
U+652C.0	fHakka	  lam3;  nam3; nam1; lam1
U+652C.0	fCantonese	laam5; laam2 (also)
U+652C.0	fMandarin	lan3
U+652C.0	fEnglish	[1] grasp; monopolise [2] be in full possession of [3] take into one's arms [4] make a selective selection of [5] take on; undertake [6] round up
U+652D.0	fUTF8	攭
U+652D.0	fCNS11643	2-7037
U+652D.0	fBig5	F8A9
U+652D.0	fGB	5-4E26
U+652D.0	fR/S	 64.21
U+652D.0	fTotalStrokes	 24
U+652D.0	fCangjie	QVOI
U+652D.0	fFourCorner	57036
U+652D.0	fCantonese	lai5; lai6 (also)
U+652D.0	fMandarin	li4; luo3
U+652E.0	fUTF8	攮
U+652E.0	fCNS11643	2-707D
U+652E.0	fBig5	F8EF
U+652E.0	fGB	0-5F2D
U+652E.0	fR/S	 64.22
U+652E.0	fTotalStrokes	 25
U+652E.0	fCangjie	QJBV
U+652E.0	fFourCorner	50032
U+652E.0	fHakka	  nong3; {nang5}
U+652E.0	fCantonese	nong5; long5 (also)
U+652E.0	fMandarin	nang3; sa3
U+652E.0	fEnglish	 thrust; stab
U+652F.0	fUTF8	支
U+652F.0	fCNS11643	1-4545
U+652F.0	fBig5	A4E4
U+652F.0	fGB	0-5627
U+652F.0	fR/S	 65.0
U+652F.0	fTotalStrokes	 4
U+652F.0	fCangjie	JE
U+652F.0	fFourCorner	40407
U+652F.0	fHakka	 zi1;  zhi1; gi1; zih1; tje1
U+652F.0	fCantonese	zi1
U+652F.0	fMandarin	zhi1
U+652F.0	fEnglish	[1] pay; disburse; defray [2] support; sustain [3] prop up; put up [4] prick up; raise [5] send away; put somebody off [6] branch; subdivision; offshoot [7] numerical classifier (pens, candles, textile) [8] used in combination
U+6530.0	fUTF8	攰
U+6530.0	fCNS11643	6-236C
U+6530.0	fGB	1-7C3F
U+6530.0	fR/S	 19.4; 65.2; 19.2
U+6530.0	fTotalStrokes	 6
U+6530.0	fCangjie	JEKS
U+6530.0	fFourCorner	44400
U+6530.0	fHakka	 kwi5
U+6530.0	fCantonese	gui6; gwui6 (also)
U+6530.0	fMandarin	gui4; qi1
U+6530.0	fEnglish	totally exhausted
U+6531.0	fUTF8	攱
U+6531.0	fCNS11643	3-2C72
U+6531.0	fGB	5-5D7B
U+6531.0	fR/S	 65.5
U+6531.0	fTotalStrokes	 9
U+6531.0	fCangjie	YTJE
U+6531.0	fFourCorner	04140
U+6531.0	fCantonese	gwai2
U+6531.0	fMandarin	gui3
U+6532.0	fUTF8	攲
U+6532.0	fCNS11643	2-3C37
U+6532.0	fBig5	D972
U+6532.0	fGB	3-4368
U+6532.0	fR/S	 65.8
U+6532.0	fTotalStrokes	 12
U+6532.0	fCangjie	KRJE
U+6532.0	fFourCorner	44647
U+6532.0	fHakka	 ki2
U+6532.0	fCantonese	kei1
U+6532.0	fMandarin	qi1; ji3; ji1; yi1
U+6532.0	fEnglish	 incline; lean; slant
U+6533.0	fUTF8	攳
U+6533.0	fCNS11643	2-573A
U+6533.0	fBig5	E9AF
U+6533.0	fGB	5-4642
U+6533.0	fR/S	 65.12
U+6533.0	fTotalStrokes	 16
U+6533.0	fCangjie	JESMI
U+6533.0	fFourCorner	47404
U+6533.0	fCantonese	cam4
U+6533.0	fMandarin	xin2; xun2
U+6534.0	fUTF8	攴
U+6534.0	fCNS11643	3-2167
U+6534.0	fGB	0-6A37
U+6534.0	fR/S	 66.0
U+6534.0	fTotalStrokes	 4
U+6534.0	fCangjie	YE
U+6534.0	fFourCorner	21400
U+6534.0	fHakka	 puk7;  pok7
U+6534.0	fCantonese	pok3; bok1 (also)
U+6534.0	fMandarin	pu1; po1
U+6534.0	fEnglish	 rap; tap lightly Kangxi radical number 66
U+6535.0	fUTF8	攵
U+6535.0	fCNS11643	3-2168
U+6535.0	fGB	0-6B36
U+6535.0	fR/S	 66.0
U+6535.0	fTotalStrokes	 4
U+6535.0	fCangjie	OK
U+6535.0	fFourCorner	80400
U+6535.0	fCantonese	pok3; man4; man6
U+6535.0	fMandarin	pu1; sui1
U+6535.0	fEnglish	 rap; tap Kangxi radical number 66
U+6536.0	fUTF8	收
U+6536.0	fCNS11643	1-482D
U+6536.0	fBig5	A6AC
U+6536.0	fGB	0-4A55
U+6536.0	fR/S	 66.2
U+6536.0	fTotalStrokes	 6
U+6536.0	fCangjie	VLOK
U+6536.0	fFourCorner	28740
U+6536.0	fHakka	 su1; shiu1; siu1; shu1
U+6536.0	fCantonese	sau1
U+6536.0	fMandarin	shou1; shou4
U+6536.0	fEnglish	[1] draw together; gather; collect [2] contain [3] receive; accept; take [4] end; come to a close [5] retrieve; take back; put away
U+6537.0	fUTF8	攷
U+6537.0	fCNS11643	2-224D
U+6537.0	fBig5	CAF7
U+6537.0	fGB	E-2C31
U+6537.0	fR/S	 66.2
U+6537.0	fTotalStrokes	 6
U+6537.0	fCangjie	MSOK
U+6537.0	fFourCorner	18240
U+6537.0	fHakka	 kau3; ko3
U+6537.0	fCantonese	haau2
U+6537.0	fMandarin	kao3; kao2
U+6537.0	fEnglish	 examine; test investigate
U+6538.0	fUTF8	攸
U+6538.0	fCNS11643	1-4A53
U+6538.0	fBig5	A7F1
U+6538.0	fGB	0-587C
U+6538.0	fR/S	 66.3
U+6538.0	fTotalStrokes	 7
U+6538.0	fCangjie	OLOK
U+6538.0	fFourCorner	28240
U+6538.0	fHakka	  ju2; riu2; jiu3; jiu2
U+6538.0	fCantonese	jau4
U+6538.0	fMandarin	you1
U+6538.0	fEnglish	[1] distant; far [2] fast fleeting [3] the place where [4] concern [5] a Chinese family name
U+6539.0	fUTF8	改
U+6539.0	fCNS11643	1-4A51
U+6539.0	fBig5	A7EF
U+6539.0	fGB	0-3844
U+6539.0	fR/S	 66.3
U+6539.0	fTotalStrokes	 7
U+6539.0	fCangjie	SUOK
U+6539.0	fFourCorner	18740
U+6539.0	fHakka	  goi3; goi5
U+6539.0	fCantonese	goi2
U+6539.0	fMandarin	gai3
U+6539.0	fEnglish	[1] change; alter; transform; convert; modify [2] correct [3] revise [4] switch over to
U+653A.0	fUTF8	攺
U+653A.0	fCNS11643	4-243F
U+653A.0	fGB	5-3E6E
U+653A.0	fR/S	  66.3
U+653A.0	fTotalStrokes	 7
U+653A.0	fCangjie	RUOK
U+653A.0	fFourCorner	78740
U+653A.0	fHakka	 goi3
U+653A.0	fCantonese	ji5
U+653A.0	fMandarin	yi3
U+653B.0	fUTF8	攻
U+653B.0	fCNS11643	1-4A52
U+653B.0	fBig5	A7F0
U+653B.0	fGB	0-3925
U+653B.0	fR/S	 66.3
U+653B.0	fTotalStrokes	 7
U+653B.0	fCangjie	MOK
U+653B.0	fFourCorner	18140
U+653B.0	fHakka	  gung1
U+653B.0	fCantonese	gung1
U+653B.0	fMandarin	gong1
U+653B.0	fEnglish	[1] attack; raid; assault [2] accuse; charge; assail; criticise; rebuke [3] work at; apply oneself to; study
U+653C.0	fUTF8	攼
U+653C.0	fCNS11643	4-243E
U+653C.0	fGB	5-3547
U+653C.0	fR/S	  66.3
U+653C.0	fTotalStrokes	 7
U+653C.0	fCangjie	MJOK
U+653C.0	fFourCorner	18440
U+653C.0	fCantonese	gon1
U+653C.0	fMandarin	gan1
U+653D.0	fUTF8	攽
U+653D.0	fCNS11643	2-2722
U+653D.0	fBig5	CCC1
U+653D.0	fGB	5-4935
U+653D.0	fR/S	 66.4
U+653D.0	fTotalStrokes	 8
U+653D.0	fCangjie	CHOK
U+653D.0	fFourCorner	88240
U+653D.0	fHakka	 ban1;  bin1
U+653D.0	fCantonese	baan1; ban1 (also)
U+653D.0	fMandarin	ban1; bin1
U+653D.0	fEnglish	[1] divide [2] reduce
U+653E.0	fUTF8	放
U+653E.0	fCNS11643	1-4D73
U+653E.0	fBig5	A9F1
U+653E.0	fGB	0-3745
U+653E.0	fR/S	 66.4
U+653E.0	fTotalStrokes	 8
U+653E.0	fCangjie	YSOK
U+653E.0	fFourCorner	08240
U+653E.0	fHakka	  fong5; piong5; bong5; biong5
U+653E.0	fCantonese	fong3; fong2
U+653E.0	fMandarin	fang4; fang3; fang1
U+653E.0	fEnglish	[1] let go; release; free; liberate; loosen [2] put; place [3] put in; add [4] dissipate; debauch; indulge
U+653F.0	fUTF8	政
U+653F.0	fCNS11643	1-5149
U+653F.0	fBig5	AC46
U+653F.0	fGB	0-557E
U+653F.0	fR/S	 66.4
U+653F.0	fTotalStrokes	 8
U+653F.0	fCangjie	MMOK
U+653F.0	fFourCorner	18140
U+653F.0	fHakka	  zhin5;  zin5; zihn5
U+653F.0	fCantonese	zing3
U+653F.0	fMandarin	zheng4; zheng1
U+653F.0	fEnglish	[1] government [2] administration; management [3] politics; political affairs
U+6540.0	fUTF8	敀
U+6540.0	fCNS11643	3-2C74
U+6540.0	fGB	5-5B53
U+6540.0	fR/S	  66.5
U+6540.0	fTotalStrokes	 9
U+6540.0	fCangjie	HAOK
U+6540.0	fFourCorner	28640
U+6540.0	fCantonese	baak3
U+6540.0	fMandarin	bo2
U+6541.0	fUTF8	敁
U+6541.0	fCNS11643	2-2A68
U+6541.0	fBig5	CEE7
U+6541.0	fGB	5-486F
U+6541.0	fR/S	 66.5
U+6541.0	fTotalStrokes	 9
U+6541.0	fCangjie	YRYE
U+6541.0	fFourCorner	21647
U+6541.0	fHakka	 zham1; zam1
U+6541.0	fCantonese	dim1; dim6
U+6541.0	fMandarin	dian1
U+6541.0	fEnglish	 weigh in the hand; estimate shake
U+6542.0	fUTF8	敂
U+6542.0	fCNS11643	3-2C76
U+6542.0	fGB	E-2C32
U+6542.0	fR/S	  66.5
U+6542.0	fTotalStrokes	 9
U+6542.0	fCangjie	PROK
U+6542.0	fFourCorner	28640
U+6542.0	fHakka	 keu5
U+6542.0	fCantonese	kau3
U+6542.0	fMandarin	kou4
U+6542.0	fEnglish	deduct
U+6543.0	fUTF8	敃
U+6543.0	fCNS11643	2-2A69
U+6543.0	fBig5	CEE8
U+6543.0	fGB	5-4936
U+6543.0	fR/S	 66.5
U+6543.0	fTotalStrokes	 9
U+6543.0	fCangjie	RPOK
U+6543.0	fFourCorner	78740
U+6543.0	fHakka	  men3
U+6543.0	fCantonese	man5
U+6543.0	fMandarin	min3; fen1; min2
U+6543.0	fEnglish	 strong robust vigorous
U+6544.0	fUTF8	敄
U+6544.0	fCNS11643	3-2C73
U+6544.0	fGB	5-5E7C
U+6544.0	fR/S	  66.5
U+6544.0	fTotalStrokes	 9
U+6544.0	fCangjie	NHOK
U+6544.0	fFourCorner	18240
U+6544.0	fCantonese	mou6
U+6544.0	fMandarin	wu4; mou4
U+6544.0	fEnglish	 put forth effort
U+6545.0	fUTF8	故
U+6545.0	fCNS11643	1-514A
U+6545.0	fBig5	AC47
U+6545.0	fGB	0-394A
U+6545.0	fR/S	 66.5
U+6545.0	fTotalStrokes	 9
U+6545.0	fCangjie	JROK
U+6545.0	fFourCorner	48640
U+6545.0	fHakka	  gu5
U+6545.0	fCantonese	gu3; gwu3 (also); gu2; gwu2 (also)
U+6545.0	fMandarin	gu4; gu3
U+6545.0	fEnglish	[1] ancient; old; former; past; earlier; previous; antique [2] cause; reason [3] intentional; willful; on purpose; knowingly [4] die [5] incident; event; matter; happening [6] consequently; hence; therefore; [7] friend; acquaintance
U+6546.0	fUTF8	敆
U+6546.0	fCNS11643	2-2F50
U+6546.0	fBig5	D1CE
U+6546.0	fGB	3-4725
U+6546.0	fR/S	 66.6
U+6546.0	fTotalStrokes	 10
U+6546.0	fCangjie	ORYE
U+6546.0	fFourCorner	81647
U+6546.0	fCantonese	hap6; gap3
U+6546.0	fMandarin	he2; ge2; xia2
U+6547.0	fUTF8	敇
U+6547.0	fCNS11643	4-2D43
U+6547.0	fGB	E-2C33
U+6547.0	fR/S	  66.6
U+6547.0	fTotalStrokes	 10
U+6547.0	fCangjie	DBOK
U+6547.0	fFourCorner	58940
U+6547.0	fCantonese	caak3
U+6547.0	fMandarin	ce4
U+6548.0	fUTF8	效
U+6548.0	fCNS11643	1-5566
U+6548.0	fBig5	AEC4
U+6548.0	fGB	0-5027
U+6548.0	fR/S	 66.6
U+6548.0	fTotalStrokes	 10
U+6548.0	fCangjie	YKOK
U+6548.0	fFourCorner	08440
U+6548.0	fHakka	 hau5; hau3; hau6
U+6548.0	fCantonese	haau6
U+6548.0	fMandarin	xiao4; jiao3
U+6548.0	fEnglish	[1] imitate; mimic; follow [2] result; effect; effectiveness; efficacy [3] devote; [4] offer
U+6549.0	fUTF8	敉
U+6549.0	fCNS11643	1-5567
U+6549.0	fBig5	AEC5
U+6549.0	fGB	0-744D
U+6549.0	fR/S	 66.6
U+6549.0	fTotalStrokes	 10
U+6549.0	fCangjie	FDOK
U+6549.0	fFourCorner	98940
U+6549.0	fHakka	  mi3; mia1
U+6549.0	fCantonese	mei5; mai5 (also); mei3
U+6549.0	fMandarin	mi3
U+6549.0	fEnglish	 pacify; soothe; stabilise; quiet
U+654A.0	fUTF8	敊
U+654A.0	fCNS11643	2-2F4F
U+654A.0	fBig5	D1CD
U+654A.0	fGB	3-4723
U+654A.0	fR/S	 66.6
U+654A.0	fTotalStrokes	 10
U+654A.0	fCangjie	YFYE
U+654A.0	fFourCorner	21947
U+654A.0	fCantonese	cuk1
U+654A.0	fMandarin	chu4; shi4
U+654B.0	fUTF8	敋
U+654B.0	fCNS11643	3-313D
U+654B.0	fGB	E-2C34
U+654B.0	fR/S	  66.6
U+654B.0	fTotalStrokes	 10
U+654B.0	fCangjie	XXHRO
U+654B.0	fFourCorner	28640
U+654B.0	fHakka	 gok8
U+654B.0	fCantonese	gok3
U+654B.0	fMandarin	ge2; guo2
U+654B.0	fEnglish	 strike
U+654C.0	fUTF8	敌
U+654C.0	fCNS11643	4-2D44
U+654C.0	fGB	0-3550
U+654C.0	fR/S	 66.6; 135.4
U+654C.0	fTotalStrokes	 10
U+654C.0	fCangjie	HROK
U+654C.0	fFourCorner	28640
U+654C.0	fHakka	 tit8
U+654C.0	fCantonese	dik6
U+654C.0	fMandarin	di2; hua2
U+654C.0	fEnglish	simplified form of &#x6575;
U+654D.0	fUTF8	敍
U+654D.0	fCNS11643	3-3637
U+654D.0	fGB	E-2C35
U+654D.0	fR/S	 66.7
U+654D.0	fTotalStrokes	 11
U+654D.0	fCangjie	ODYE
U+654D.0	fFourCorner	81940
U+654D.0	fHakka	 sii5; si5
U+654D.0	fCantonese	zeoi6
U+654D.0	fMandarin	xu4
U+654D.0	fEnglish	[1] tell; express; state; relate; narrate; describe [2] talk about; chat [3] arrange in order [4] rate; evaluate (as a basis for reward, appointment, etc.); assess
U+654E.0	fUTF8	敎
U+654E.0	fCNS11643	3-3636
U+654E.0	fGB	E-2C36
U+654E.0	fR/S	 66.7
U+654E.0	fTotalStrokes	 11
U+654E.0	fCangjie	KDOK
U+654E.0	fFourCorner	48440
U+654E.0	fHakka	 gau5;  gau1
U+654E.0	fCantonese	gaau3; gaau1
U+654E.0	fMandarin	jiao4
U+654E.0	fEnglish	[1] religion [2] order; directive [3] educate [4] incite; urge; bid; instigate [5] instruct; instruction(s); advise; advice [6] make; let; have
U+654E.1	fUTF8	敎
U+654E.1	fCNS11643	3-3636
U+654E.1	fGB	E-2C36
U+654E.1	fR/S	 66.7
U+654E.1	fTotalStrokes	 11
U+654E.1	fCangjie	KDOK
U+654E.1	fFourCorner	48440
U+654E.1	fMandarin	jiao1
U+654E.1	fEnglish	 teach; guide
U+654F.0	fUTF8	敏
U+654F.0	fCNS11643	1-5A77
U+654F.0	fBig5	B1D3
U+654F.0	fGB	0-4374
U+654F.0	fR/S	 66.7; 66.6
U+654F.0	fTotalStrokes	 11
U+654F.0	fCangjie	OYOK
U+654F.0	fFourCorner	88540
U+654F.0	fHakka	 men3; men1; mien1; mian3; mien3
U+654F.0	fCantonese	man5
U+654F.0	fMandarin	min3
U+654F.0	fEnglish	[1] fast; quick; clever; smart; agile; speedy; nimble; sensitive [2] diligent; industrious; earnest; eager
U+6550.0	fUTF8	敐
U+6550.0	fCNS11643	4-322B
U+6550.0	fGB	5-6E38
U+6550.0	fR/S	  66.7
U+6550.0	fTotalStrokes	 11
U+6550.0	fCangjie	MVOK
U+6550.0	fFourCorner	78240
U+6550.0	fCantonese	san4
U+6550.0	fMandarin	chen2
U+6551.0	fUTF8	救
U+6551.0	fCNS11643	1-5A73
U+6551.0	fBig5	B1CF
U+6551.0	fGB	0-3E48
U+6551.0	fR/S	 66.7
U+6551.0	fTotalStrokes	 11
U+6551.0	fCangjie	IEOK
U+6551.0	fFourCorner	48140
U+6551.0	fHakka	  giu5; kieu5
U+6551.0	fCantonese	gau3; geoi1
U+6551.0	fMandarin	jiu4; ju1
U+6551.0	fEnglish	 save; rescue; relieve; help; aid; deliver
U+6552.0	fUTF8	敒
U+6552.0	fCNS11643	4-322F
U+6552.0	fGB	E-2C37
U+6552.0	fR/S	  66.7
U+6552.0	fTotalStrokes	 11
U+6552.0	fCangjie	OLLK
U+6552.0	fFourCorner	28240
U+6552.0	fCantonese	san1
U+6552.0	fMandarin	zhen4; shen1
U+6553.0	fUTF8	敓
U+6553.0	fCNS11643	2-3569
U+6553.0	fBig5	D5A7
U+6553.0	fGB	E-2C38
U+6553.0	fR/S	 66.7
U+6553.0	fTotalStrokes	 11
U+6553.0	fCangjie	CUOK
U+6553.0	fFourCorner	88240
U+6553.0	fHakka	  tot8; tot7
U+6553.0	fCantonese	dyut6
U+6553.0	fMandarin	duo2
U+6553.0	fEnglish	 plunder
U+6554.0	fUTF8	敔
U+6554.0	fCNS11643	1-5A7A
U+6554.0	fBig5	B1D6
U+6554.0	fGB	3-4740
U+6554.0	fR/S	 66.7
U+6554.0	fTotalStrokes	 11
U+6554.0	fCangjie	MROK
U+6554.0	fFourCorner	18640
U+6554.0	fHakka	 ngi3
U+6554.0	fCantonese	jyu5
U+6554.0	fMandarin	yu3
U+6554.0	fEnglish	an ancient musical instrument
U+6555.0	fUTF8	敕
U+6555.0	fCNS11643	1-5A79
U+6555.0	fBig5	B1D5
U+6555.0	fGB	0-6B37
U+6555.0	fR/S	 66.7
U+6555.0	fTotalStrokes	 11
U+6555.0	fCangjie	DLOK
U+6555.0	fFourCorner	58940
U+6555.0	fHakka	 chit7; ts'iit7; cit7; sok7
U+6555.0	fCantonese	cik1
U+6555.0	fMandarin	chi4
U+6555.0	fEnglish	[1] imperial order or decree [2] cautious [3] warn; caution
U+6556.0	fUTF8	敖
U+6556.0	fCNS11643	1-5A72
U+6556.0	fBig5	B1CE
U+6556.0	fGB	0-303D
U+6556.0	fR/S	 66.7
U+6556.0	fTotalStrokes	 11
U+6556.0	fCangjie	GSOK
U+6556.0	fFourCorner	48240
U+6556.0	fHakka	 ngau2; ngau3; ngau6; ngau5
U+6556.0	fCantonese	ngou4; ngou6; ou4 (also); ou6 (also)
U+6556.0	fMandarin	ao2
U+6556.0	fEnglish	leisurely; idle
U+6556.1	fUTF8	敖
U+6556.1	fCNS11643	1-5A72
U+6556.1	fBig5	B1CE
U+6556.1	fGB	0-303D
U+6556.1	fR/S	 66.7
U+6556.1	fTotalStrokes	 11
U+6556.1	fCangjie	GSOK
U+6556.1	fFourCorner	48240
U+6556.1	fMandarin	ao4
U+6556.1	fEnglish	[1] make fun of; play about [2] proud; haughty
U+6557.0	fUTF8	敗
U+6557.0	fCNS11643	1-5A75
U+6557.0	fBig5	B1D1
U+6557.0	fGB	1-305C
U+6557.0	fR/S	 66.7
U+6557.0	fTotalStrokes	 11
U+6557.0	fCangjie	BCOK
U+6557.0	fFourCorner	68840
U+6557.0	fHakka	 pai5; pai3; pai2; pai6
U+6557.0	fCantonese	baai6
U+6557.0	fMandarin	bai4
U+6557.0	fEnglish	[1] defeat; be defeated; thwart; be thwarted; go down; fail; lose [2] decline; decay; wither [3] spoil; be spoiled; corrupt; be corrupted
U+6558.0	fUTF8	敘
U+6558.0	fCNS11643	1-5A78
U+6558.0	fBig5	B1D4
U+6558.0	fGB	E-2C39
U+6558.0	fR/S	 66.7
U+6558.0	fTotalStrokes	 11
U+6558.0	fCangjie	ODOK
U+6558.0	fFourCorner	88940
U+6558.0	fHakka	 si5
U+6558.0	fCantonese	zeoi6
U+6558.0	fMandarin	xu4
U+6558.0	fEnglish	express, state, relate, narrate
U+6559.0	fUTF8	教
U+6559.0	fCNS11643	1-5A74
U+6559.0	fBig5	B1D0
U+6559.0	fGB	0-3D4C
U+6559.0	fR/S	 66.7
U+6559.0	fTotalStrokes	 11
U+6559.0	fCangjie	JDOK
U+6559.0	fFourCorner	48440
U+6559.0	fHakka	  gau5;  gau1
U+6559.0	fCantonese	gaau3; gaau1
U+6559.0	fMandarin	jiao4
U+6559.0	fEnglish	teach, class
U+6559.1	fUTF8	教
U+6559.1	fCNS11643	1-5A74
U+6559.1	fBig5	B1D0
U+6559.1	fGB	0-3D4C
U+6559.1	fR/S	 66.7
U+6559.1	fTotalStrokes	 11
U+6559.1	fCangjie	JDOK
U+6559.1	fFourCorner	48440
U+6559.1	fMandarin	jiao1
U+655A.0	fUTF8	敚
U+655A.0	fCNS11643	3-3634
U+655A.0	fGB	5-493D
U+655A.0	fR/S	 66.7
U+655A.0	fTotalStrokes	 11
U+655A.0	fCangjie	CUOK
U+655A.0	fFourCorner	88240
U+655A.0	fCantonese	dyut6
U+655A.0	fMandarin	duo2
U+655B.0	fUTF8	敛
U+655B.0	fGB	0-4132
U+655B.0	fR/S	 66.7
U+655B.0	fTotalStrokes	 11
U+655B.0	fCangjie	OMOK
U+655B.0	fHakka	 liam1; liam3
U+655B.0	fCantonese	lim5; lim6 (also)
U+655B.0	fMandarin	lian4; lian3; lian2
U+655B.0	fEnglish	simplified form of &#x6582;
U+655C.0	fUTF8	敜
U+655C.0	fCNS11643	2-3C3B
U+655C.0	fBig5	D976
U+655C.0	fGB	3-4742
U+655C.0	fR/S	 66.8
U+655C.0	fTotalStrokes	 12
U+655C.0	fCangjie	OPOK
U+655C.0	fFourCorner	88340
U+655C.0	fHakka	  ngiap7
U+655C.0	fCantonese	nip6; lip6 (also); nip1
U+655C.0	fMandarin	nie4; nie2
U+655C.0	fEnglish	fill up
U+655D.0	fUTF8	敝
U+655D.0	fCNS11643	1-5A71
U+655D.0	fBig5	B1CD
U+655D.0	fGB	0-3156
U+655D.0	fR/S	 66.8
U+655D.0	fTotalStrokes	 12
U+655D.0	fCangjie	FBOK
U+655D.0	fFourCorner	98240
U+655D.0	fHakka	  bi5
U+655D.0	fCantonese	bai6
U+655D.0	fMandarin	bi4
U+655D.0	fEnglish	[1] worn-out; broken; tattered [2] exhausted; tired [3] my; our (a self-depreciatory term)
U+655E.0	fUTF8	敞
U+655E.0	fCNS11643	1-5F54
U+655E.0	fBig5	B4AF
U+655E.0	fGB	0-3328
U+655E.0	fR/S	 66.8
U+655E.0	fTotalStrokes	 12
U+655E.0	fCangjie	FBOK
U+655E.0	fFourCorner	98240
U+655E.0	fHakka	 chon3; chong3; con3; cong3
U+655E.0	fCantonese	cong2
U+655E.0	fMandarin	chang3; cheng2; cheng4
U+655E.0	fEnglish	[1] open; uncovered [2] roomy; spacious; broad
U+655F.0	fUTF8	敟
U+655F.0	fCNS11643	3-3B74
U+655F.0	fGB	5-4940
U+655F.0	fR/S	  66.8
U+655F.0	fTotalStrokes	 12
U+655F.0	fCangjie	TCOK
U+655F.0	fFourCorner	58840
U+655F.0	fCantonese	din2
U+655F.0	fMandarin	dian3; tian3
U+6560.0	fUTF8	敠
U+6560.0	fCNS11643	3-3B75
U+6560.0	fGB	5-4921
U+6560.0	fR/S	  66.8
U+6560.0	fTotalStrokes	 12
U+6560.0	fCangjie	EEEEE
U+6560.0	fFourCorner	11440
U+6560.0	fHakka	 dot7
U+6560.0	fCantonese	zyut3; dyut3 (also)
U+6560.0	fMandarin	duo2
U+6560.0	fEnglish	[1] weigh (a thing) in the hand [2] weigh and consider (a matter)
U+6561.0	fUTF8	敡
U+6561.0	fCNS11643	3-3B76
U+6561.0	fGB	5-487C
U+6561.0	fR/S	  66.8
U+6561.0	fTotalStrokes	 12
U+6561.0	fCangjie	AHYE
U+6561.0	fFourCorner	61240
U+6561.0	fCantonese	ji6
U+6561.0	fMandarin	yi4
U+6561.0	fEnglish	rebuke
U+6562.0	fUTF8	敢
U+6562.0	fCNS11643	1-5F56
U+6562.0	fBig5	B4B1
U+6562.0	fGB	0-3852
U+6562.0	fR/S	 66.8
U+6562.0	fTotalStrokes	 12
U+6562.0	fCangjie	MJOK
U+6562.0	fFourCorner	18140
U+6562.0	fHakka	  gam3; giam3
U+6562.0	fCantonese	gam2
U+6562.0	fMandarin	gan3
U+6562.0	fEnglish	[1] dare [2] bold; brave; daring [3] make bold; venture [4] have the confidence; be sure; be certain
U+6563.0	fUTF8	散
U+6563.0	fCNS11643	1-5F57
U+6563.0	fBig5	B4B2
U+6563.0	fGB	0-4922
U+6563.0	fR/S	 66.8
U+6563.0	fTotalStrokes	 12
U+6563.0	fCangjie	TBOK
U+6563.0	fFourCorner	48240
U+6563.0	fHakka	  san3; sang1
U+6563.0	fCantonese	saan3; saan2
U+6563.0	fMandarin	san4; san1
U+6563.0	fEnglish	[1] scatter; disperse [2] end; be over; stop [3] disseminate; give out
U+6563.1	fUTF8	散
U+6563.1	fCNS11643	1-5F57
U+6563.1	fBig5	B4B2
U+6563.1	fGB	0-4922
U+6563.1	fR/S	 66.8
U+6563.1	fTotalStrokes	 12
U+6563.1	fCangjie	TBOK
U+6563.1	fFourCorner	48240
U+6563.1	fHakka	 san5
U+6563.1	fMandarin	san3
U+6563.1	fEnglish	[1] loose; loosened [2] idle; leisurely [3] powdered medicine
U+6564.0	fUTF8	敤
U+6564.0	fCNS11643	2-3C3A
U+6564.0	fBig5	D975
U+6564.0	fGB	5-487B
U+6564.0	fR/S	 66.8
U+6564.0	fTotalStrokes	 12
U+6564.0	fCangjie	WDYE
U+6564.0	fFourCorner	61947
U+6564.0	fCantonese	fo2; fo3
U+6564.0	fMandarin	ke3; ke4
U+6565.0	fUTF8	敥
U+6565.0	fCNS11643	2-3C3D
U+6565.0	fBig5	D978
U+6565.0	fGB	E-2C3A
U+6565.0	fR/S	 66.8
U+6565.0	fTotalStrokes	 12
U+6565.0	fCangjie	FFYE
U+6565.0	fFourCorner	92861
U+6565.0	fHakka	 jam5
U+6565.0	fCantonese	jim6; gaau2
U+6565.0	fMandarin	yan4
U+6566.0	fUTF8	敦
U+6566.0	fCNS11643	1-5F55
U+6566.0	fBig5	B4B0
U+6566.0	fGB	0-3658
U+6566.0	fR/S	 66.8
U+6566.0	fTotalStrokes	 12
U+6566.0	fCangjie	YDOK
U+6566.0	fFourCorner	08440
U+6566.0	fHakka	  dun1; doi5
U+6566.0	fCantonese	deon1; deoi1; deoi3; tyun4; deoi6; deon6; diu1
U+6566.0	fMandarin	dun1; diao1; dui1; tuan2; chou2; dun3; dun4; tun2; zhun3
U+6566.0	fEnglish	[1] honest; candid; sincere [2] deepen; strengthen (relations) [3] urge
U+6566.1	fUTF8	敦
U+6566.1	fCNS11643	1-5F55
U+6566.1	fBig5	B4B0
U+6566.1	fGB	0-3658
U+6566.1	fR/S	 66.8
U+6566.1	fTotalStrokes	 12
U+6566.1	fCangjie	YDOK
U+6566.1	fFourCorner	08440
U+6566.1	fMandarin	dui4
U+6566.1	fEnglish	a sort of container
U+6567.0	fUTF8	敧
U+6567.0	fCNS11643	2-3C38
U+6567.0	fBig5	D973
U+6567.0	fGB	3-4727
U+6567.0	fR/S	 66.8
U+6567.0	fTotalStrokes	 12
U+6567.0	fCangjie	KRYE
U+6567.0	fFourCorner	41647
U+6567.0	fHakka	 ki1;  gi1; ji3
U+6567.0	fCantonese	gei1; ji2
U+6567.0	fMandarin	qi3; ji1
U+6567.0	fEnglish	 pick up thing with chopsticks or pincers.
U+6568.0	fUTF8	敨
U+6568.0	fCNS11643	2-3C3C
U+6568.0	fBig5	D977
U+6568.0	fGB	E-2C3B
U+6568.0	fR/S	 66.8
U+6568.0	fTotalStrokes	 12
U+6568.0	fCangjie	YROK
U+6568.0	fFourCorner	08640
U+6568.0	fHakka	 teu3
U+6568.0	fCantonese	tau2; dau2
U+6568.0	fMandarin	dou3; tou3
U+6569.0	fUTF8	敩
U+6569.0	fGB	8-7C37
U+6569.0	fR/S	 66.16; 66.8
U+6569.0	fTotalStrokes	 12
U+6569.0	fCangjie	XFDOK
U+6569.0	fCantonese	haau6; hok6
U+6569.0	fMandarin	xiao4
U+656A.0	fUTF8	敪
U+656A.0	fCNS11643	2-3C39
U+656A.0	fBig5	D974
U+656A.0	fGB	E-2C3C
U+656A.0	fR/S	 66.8
U+656A.0	fTotalStrokes	 12
U+656A.0	fCangjie	EEEEK
U+656A.0	fFourCorner	18440
U+656A.0	fCantonese	zyut3
U+656A.0	fMandarin	duo2
U+656A.0	fEnglish	 weigh cut come without being invited
U+656B.0	fUTF8	敫
U+656B.0	fCNS11643	3-414B
U+656B.0	fGB	0-6B38
U+656B.0	fR/S	 66.9; 66.8; 106.8
U+656B.0	fTotalStrokes	 13
U+656B.0	fCangjie	HSOK
U+656B.0	fFourCorner	28240
U+656B.0	fHakka	  git7; giau3; kieu3; giau5; jok8
U+656B.0	fCantonese	giu3; giu2
U+656B.0	fMandarin	jiao4; jiao3; qiao1; jia3
U+656B.0	fEnglish	ancient musical instrument
U+656C.0	fUTF8	敬
U+656C.0	fCNS11643	1-6439
U+656C.0	fBig5	B771
U+656C.0	fGB	0-3E34
U+656C.0	fR/S	 66.9
U+656C.0	fTotalStrokes	 13
U+656C.0	fCangjie	TROK
U+656C.0	fFourCorner	48640
U+656C.0	fHakka	  gin5
U+656C.0	fCantonese	ging3
U+656C.0	fMandarin	jing4
U+656C.0	fEnglish	[1] respect; honor; revere; esteem; respectfully [2] present; offer [3] a Chinese family name
U+656D.0	fUTF8	敭
U+656D.0	fCNS11643	3-414E
U+656D.0	fGB	E-2C3D
U+656D.0	fR/S	 66.8; 66.9
U+656D.0	fTotalStrokes	 13; 12
U+656D.0	fCangjie	AHOK
U+656D.0	fFourCorner	68240
U+656D.0	fHakka	 ji5; ri5; rit7; jit7
U+656D.0	fCantonese	joeng4
U+656D.0	fMandarin	yang2
U+656D.0	fEnglish	 scatter; spread praise raise
U+656E.0	fUTF8	敮
U+656E.0	fCNS11643	4-3D5D
U+656E.0	fGB	3-4732
U+656E.0	fR/S	 66.9
U+656E.0	fTotalStrokes	 13
U+656E.0	fCangjie	MXYE
U+656E.0	fFourCorner	11740
U+656E.0	fCantonese	hap1
U+656E.0	fMandarin	xia2; gui1
U+656F.0	fUTF8	敯
U+656F.0	fCNS11643	2-4342
U+656F.0	fBig5	DDBC
U+656F.0	fGB	5-492A
U+656F.0	fR/S	 66.9
U+656F.0	fTotalStrokes	 13
U+656F.0	fCangjie	RAYE
U+656F.0	fFourCorner	71647
U+656F.0	fCantonese	man5; fan1; man4
U+656F.0	fMandarin	min2; hun1; min3
U+656F.0	fEnglish	 defy; dare strong suffocate
U+6570.0	fUTF8	数
U+6570.0	fCNS11643	3-414D
U+6570.0	fGB	0-4A7D
U+6570.0	fR/S	 66.9
U+6570.0	fTotalStrokes	 13
U+6570.0	fCangjie	FVOK
U+6570.0	fFourCorner	98440
U+6570.0	fHakka	 sii5; su5
U+6570.0	fCantonese	sou3; sou2; sok3; cuk1
U+6570.0	fMandarin	shu4; cu4; su4; suo1; shuo4
U+6570.0	fEnglish	[1] variant of &#x6578; [2] simplified form of &#x6578;
U+6570.1	fUTF8	数
U+6570.1	fCNS11643	3-414D
U+6570.1	fGB	0-4A7D
U+6570.1	fR/S	 66.9
U+6570.1	fTotalStrokes	 13
U+6570.1	fCangjie	FVOK
U+6570.1	fFourCorner	98440
U+6570.1	fMandarin	shu3
U+6570.2	fUTF8	数
U+6570.2	fCNS11643	3-414D
U+6570.2	fGB	0-4A7D
U+6570.2	fR/S	 66.9
U+6570.2	fTotalStrokes	 13
U+6570.2	fCangjie	FVOK
U+6570.2	fFourCorner	98440
U+6570.2	fMandarin	shu5
U+6571.0	fUTF8	敱
U+6571.0	fCNS11643	3-4730
U+6571.0	fGB	3-4724
U+6571.0	fR/S	 66.10
U+6571.0	fTotalStrokes	 14
U+6571.0	fCangjie	UTYE
U+6571.0	fFourCorner	21140
U+6571.0	fCantonese	ngoi4; oi4 (also)
U+6571.0	fMandarin	ai2; zhu2
U+6572.0	fUTF8	敲
U+6572.0	fCNS11643	1-687D
U+6572.0	fBig5	BA56
U+6572.0	fGB	0-4743
U+6572.0	fR/S	 66.10
U+6572.0	fTotalStrokes	 14
U+6572.0	fCangjie	YBYE
U+6572.0	fFourCorner	01247
U+6572.0	fHakka	 kau5;  kau1; kiau1
U+6572.0	fCantonese	haau1
U+6572.0	fMandarin	qiao1; qiao4
U+6572.0	fEnglish	[1] strike; beat; pound; hammer; rap; knock; tap [2] used in combination [3] extort; blackmail; overcharge
U+6573.0	fUTF8	敳
U+6573.0	fCNS11643	2-4A5B
U+6573.0	fBig5	E1F4
U+6573.0	fGB	E-2C3E
U+6573.0	fR/S	 66.10
U+6573.0	fTotalStrokes	 14
U+6573.0	fCangjie	UTOK
U+6573.0	fFourCorner	28140
U+6573.0	fHakka	 ngoi2
U+6573.0	fCantonese	ngoi4; oi4 (also); zuk1
U+6573.0	fMandarin	ai2; ai3; gui1; zhu1
U+6573.0	fEnglish	 able to regulate or administer
U+6574.0	fUTF8	整
U+6574.0	fCNS11643	1-706C
U+6574.0	fBig5	BEE3
U+6574.0	fGB	0-557B
U+6574.0	fR/S	 66.11
U+6574.0	fTotalStrokes	 15
U+6574.0	fCangjie	DKMYM
U+6574.0	fFourCorner	58101
U+6574.0	fHakka	 zin3; zhin3; zang3; chang3; zihn3; zhang3
U+6574.0	fCantonese	zing2
U+6574.0	fMandarin	zheng3
U+6574.0	fEnglish	[1] orderly; neat; tidy; systematically [2] sharp [3] whole; complete; entire; intact [4] tidy; set in orer; adjust; arrange; repair; make ready
U+6575.0	fUTF8	敵
U+6575.0	fCNS11643	1-6D2E
U+6575.0	fBig5	BCC4
U+6575.0	fGB	1-3550
U+6575.0	fR/S	 66.11
U+6575.0	fTotalStrokes	 15
U+6575.0	fCangjie	YBOK
U+6575.0	fFourCorner	08240
U+6575.0	fHakka	  tit8
U+6575.0	fCantonese	dik6
U+6575.0	fMandarin	di2
U+6575.0	fEnglish	[1] enemy; foe; rival [2] oppose; resist [3] match; rival; equal
U+6576.0	fUTF8	敶
U+6576.0	fCNS11643	2-5064
U+6576.0	fBig5	E5BD
U+6576.0	fGB	5-493F
U+6576.0	fR/S	 66.11
U+6576.0	fTotalStrokes	 15
U+6576.0	fCangjie	NLDWK
U+6576.0	fFourCorner	78240
U+6576.0	fCantonese	can4; zan6
U+6576.0	fMandarin	zhen4; chen2
U+6577.0	fUTF8	敷
U+6577.0	fCNS11643	1-6D2F
U+6577.0	fBig5	BCC5
U+6577.0	fGB	0-3773
U+6577.0	fR/S	 66.11
U+6577.0	fTotalStrokes	 15
U+6577.0	fCangjie	ISOK
U+6577.0	fFourCorner	58240
U+6577.0	fHakka	  fu1
U+6577.0	fCantonese	fu1
U+6577.0	fMandarin	fu1
U+6577.0	fEnglish	[1] spread (over a surface); diffuse; apply (over a surface); paint [2] suffice; be enough [3] state; explain; expound
U+6578.0	fUTF8	數
U+6578.0	fCNS11643	1-6D30
U+6578.0	fBig5	BCC6
U+6578.0	fGB	1-4A7D
U+6578.0	fR/S	 66.11
U+6578.0	fTotalStrokes	 15
U+6578.0	fCangjie	LVOK
U+6578.0	fFourCorner	58440
U+6578.0	fHakka	  su5;  sii5; sii6; se5; se3
U+6578.0	fCantonese	sou3; sou2; sok3; cuk1
U+6578.0	fMandarin	shu4; cu4; su4; suo1
U+6578.0	fEnglish	[1] number; quantity; sum [2] numbers (mathematics) [3] several; a few [4] destiny; fate [5] plan; idea [6] art
U+6578.1	fUTF8	數
U+6578.1	fCNS11643	1-6D30
U+6578.1	fBig5	BCC6
U+6578.1	fGB	1-4A7D
U+6578.1	fR/S	 66.11
U+6578.1	fTotalStrokes	 15
U+6578.1	fCangjie	LVOK
U+6578.1	fFourCorner	58440
U+6578.1	fHakka	 si5
U+6578.1	fMandarin	shu3
U+6578.1	fEnglish	[1] count; enumerate [2] count (as best, etc.); be reckoned as exceptionally (good, bad, etc.)
U+6578.2	fUTF8	數
U+6578.2	fCNS11643	1-6D30
U+6578.2	fBig5	BCC6
U+6578.2	fGB	1-4A7D
U+6578.2	fR/S	 66.11
U+6578.2	fTotalStrokes	 15
U+6578.2	fCangjie	LVOK
U+6578.2	fFourCorner	58440
U+6578.2	fMandarin	shuo4
U+6578.2	fEnglish	often; frequently
U+6578.3	fUTF8	數
U+6578.3	fCNS11643	1-6D30
U+6578.3	fBig5	BCC6
U+6578.3	fGB	1-4A7D
U+6578.3	fR/S	 66.11
U+6578.3	fTotalStrokes	 15
U+6578.3	fCangjie	LVOK
U+6578.3	fFourCorner	58440
U+6578.3	fMandarin	shu5
U+6579.0	fUTF8	敹
U+6579.0	fCNS11643	2-5066
U+6579.0	fBig5	E5BF
U+6579.0	fGB	3-4746
U+6579.0	fR/S	 66.11
U+6579.0	fTotalStrokes	 15
U+6579.0	fCangjie	YDOK
U+6579.0	fFourCorner	28940
U+6579.0	fHakka	  liau2
U+6579.0	fCantonese	liu4
U+6579.0	fMandarin	liao2
U+6579.0	fEnglish	 sew keep tidy and repaired
U+657A.0	fUTF8	敺
U+657A.0	fCNS11643	2-5065
U+657A.0	fBig5	E5BE
U+657A.0	fGB	E-2C3F
U+657A.0	fR/S	 66.11
U+657A.0	fTotalStrokes	 15
U+657A.0	fCangjie	SRYE
U+657A.0	fFourCorner	71747
U+657A.0	fHakka	  ki1
U+657A.0	fCantonese	keoi1; au2; ngau2
U+657A.0	fMandarin	qu1
U+657A.0	fEnglish	 expel; drive away
U+657A.1	fUTF8	敺
U+657A.1	fCNS11643	2-5065
U+657A.1	fBig5	E5BE
U+657A.1	fGB	E-2C3F
U+657A.1	fR/S	 66.11
U+657A.1	fTotalStrokes	 15
U+657A.1	fCangjie	SRYE
U+657A.1	fFourCorner	71747
U+657A.1	fMandarin	ou1
U+657A.1	fEnglish	 beat; assault physically
U+657B.0	fUTF8	敻
U+657B.0	fCNS11643	2-5067
U+657B.0	fBig5	E5C0
U+657B.0	fGB	E-2C40
U+657B.0	fR/S	 66.11
U+657B.0	fTotalStrokes	 15
U+657B.0	fCangjie	NBBUK
U+657B.0	fFourCorner	27407
U+657B.0	fCantonese	hing3; hyun3
U+657B.0	fMandarin	xiong4; xuan4
U+657B.0	fEnglish	long
U+657C.0	fUTF8	敼
U+657C.0	fCNS11643	2-573C
U+657C.0	fBig5	E9B1
U+657C.0	fGB	E-2C41
U+657C.0	fR/S	 66.12
U+657C.0	fTotalStrokes	 16
U+657C.0	fCangjie	GRYE
U+657C.0	fFourCorner	41647
U+657C.0	fHakka	 shan5
U+657C.0	fCantonese	ji2; hei2
U+657C.0	fMandarin	xi3
U+657D.0	fUTF8	敽
U+657D.0	fGB	E-2C42
U+657D.0	fR/S	 66.12
U+657D.0	fTotalStrokes	 16
U+657D.0	fCangjie	HBYE
U+657D.0	fHakka	 giau3; kieu3
U+657D.0	fCantonese	giu2
U+657D.0	fMandarin	jiao3
U+657D.0	fEnglish	bind
U+657E.0	fUTF8	敾
U+657E.0	fGB	E-2C43
U+657E.0	fR/S	 66.12
U+657E.0	fTotalStrokes	 16
U+657E.0	fCangjie	XTROK
U+657E.0	fCantonese	sin6
U+657E.0	fMandarin	shan4
U+657E.0	fEnglish	 rule
U+657F.0	fUTF8	敿
U+657F.0	fCNS11643	2-573B
U+657F.0	fBig5	E9B0
U+657F.0	fGB	3-473F
U+657F.0	fR/S	 66.12
U+657F.0	fTotalStrokes	 16
U+657F.0	fCangjie	HBOK
U+657F.0	fFourCorner	28240
U+657F.0	fHakka	 giau3
U+657F.0	fCantonese	giu2
U+657F.0	fMandarin	jiao3; qiao2; jiao4
U+657F.0	fEnglish	bind
U+6580.0	fUTF8	斀
U+6580.0	fCNS11643	2-5C7A
U+6580.0	fBig5	ECEF
U+6580.0	fGB	3-4738
U+6580.0	fR/S	 66.13
U+6580.0	fTotalStrokes	 17
U+6580.0	fCangjie	WIYE
U+6580.0	fFourCorner	61147
U+6580.0	fCantonese	zoek3; doek3; zuk1
U+6580.0	fMandarin	zhuo2; shu3; zhu2
U+6581.0	fUTF8	斁
U+6581.0	fCNS11643	2-5C79
U+6581.0	fBig5	ECEE
U+6581.0	fGB	3-473D
U+6581.0	fR/S	 66.13
U+6581.0	fTotalStrokes	 17
U+6581.0	fCangjie	WJOK
U+6581.0	fFourCorner	68440
U+6581.0	fHakka	  du5; rit8; jit8
U+6581.0	fCantonese	jik6; dou3
U+6581.0	fMandarin	yi4
U+6581.0	fEnglish	 dislike; be weary of; be tired
U+6581.1	fUTF8	斁
U+6581.1	fCNS11643	2-5C79
U+6581.1	fBig5	ECEE
U+6581.1	fGB	3-473D
U+6581.1	fR/S	 66.13
U+6581.1	fTotalStrokes	 17
U+6581.1	fCangjie	WJOK
U+6581.1	fFourCorner	68440
U+6581.1	fMandarin	du4
U+6581.1	fEnglish	 corrupt; spoil
U+6582.0	fUTF8	斂
U+6582.0	fCNS11643	1-736D
U+6582.0	fBig5	C0C4
U+6582.0	fGB	1-4132
U+6582.0	fR/S	 66.13
U+6582.0	fTotalStrokes	 17
U+6582.0	fCangjie	OOOK
U+6582.0	fFourCorner	88840
U+6582.0	fHakka	  liam3; liam1
U+6582.0	fCantonese	lim5; lim6 (also)
U+6582.0	fMandarin	lian3; lian4; lian2
U+6582.0	fEnglish	[1] draw together; contract [2] hold back; restrain [3] collect; gather
U+6583.0	fUTF8	斃
U+6583.0	fCNS11643	1-736E
U+6583.0	fBig5	C0C5
U+6583.0	fGB	1-3150
U+6583.0	fR/S	 66.14
U+6583.0	fTotalStrokes	 18
U+6583.0	fCangjie	FKMNP
U+6583.0	fFourCorner	98211
U+6583.0	fHakka	  bi5
U+6583.0	fCantonese	bai6
U+6583.0	fMandarin	bi4
U+6583.0	fEnglish	[1] come to a bad end; decline; destruction [2] fall; prostrate [3] dead; come to the end of life [4] kill; shoot; execute by shooting (colloquial)
U+6584.0	fUTF8	斄
U+6584.0	fCNS11643	2-6553
U+6584.0	fBig5	F248
U+6584.0	fGB	3-4748
U+6584.0	fR/S	 66.15
U+6584.0	fTotalStrokes	 19
U+6584.0	fCangjie	JKMDO
U+6584.0	fFourCorner	47298
U+6584.0	fCantonese	lei4; toi4
U+6584.0	fMandarin	li4; lai2; li2; tai1; tai2
U+6585.0	fUTF8	斅
U+6585.0	fCNS11643	3-5C41
U+6585.0	fGB	5-487E
U+6585.0	fR/S	  66.16
U+6585.0	fTotalStrokes	 20
U+6585.0	fCangjie	HDYE
U+6585.0	fFourCorner	71447
U+6585.0	fHakka	 hau5; hau6
U+6585.0	fCantonese	haau6; hok6
U+6585.0	fMandarin	xiao4; xue2; jiao3
U+6585.0	fEnglish	[1] teach; instruct [2] be awakened; realise
U+6586.0	fUTF8	斆
U+6586.0	fGB	3-4743
U+6586.0	fR/S	 66.16
U+6586.0	fTotalStrokes	 20
U+6586.0	fCangjie	HDOK
U+6586.0	fHakka	 hau6; hau5; hok8
U+6586.0	fCantonese	hau6; haau6
U+6586.0	fMandarin	xiao4; jiao3
U+6586.0	fEnglish	 teach; instruct be aroused awake intelligent
U+6587.0	fUTF8	文
U+6587.0	fCNS11643	1-4546
U+6587.0	fBig5	A4E5
U+6587.0	fGB	0-4E44
U+6587.0	fR/S	 67.0
U+6587.0	fTotalStrokes	 4
U+6587.0	fCangjie	YK
U+6587.0	fFourCorner	00400
U+6587.0	fHakka	  wun2; wun3; vun2
U+6587.0	fCantonese	man4; man6; man1; man2 (also)
U+6587.0	fMandarin	wen2
U+6587.0	fEnglish	[1] composition; article; writing [2] language [3] literature; culture; education [4] elegant; cultured; polished; suave; civil; polite; urbane; mild [5] civilian; civil [6] monetary unit [7] used in combination [8] a Chinese family name
U+6587.1	fUTF8	文
U+6587.1	fCNS11643	1-4546
U+6587.1	fBig5	A4E5
U+6587.1	fGB	0-4E44
U+6587.1	fR/S	 67.0
U+6587.1	fTotalStrokes	 4
U+6587.1	fCangjie	YK
U+6587.1	fFourCorner	00400
U+6587.1	fMandarin	wen4
U+6587.1	fEnglish	 cover up; conceal; gloss over
U+6588.0	fUTF8	斈
U+6588.0	fCNS11643	3-2558
U+6588.0	fGB	5-5077
U+6588.0	fR/S	 67.3; 39.4
U+6588.0	fTotalStrokes	 7
U+6588.0	fCangjie	YKND
U+6588.0	fFourCorner	00400
U+6588.0	fHakka	 hok8
U+6588.0	fCantonese	hok6
U+6588.0	fMandarin	xue2; xiao2; jiao4
U+6588.0	fEnglish	learning, knowledge; school
U+6589.0	fUTF8	斉
U+6589.0	fCNS11643	3-2939
U+6589.0	fGB	E-2C44
U+6589.0	fR/S	 210.0; 67.4
U+6589.0	fTotalStrokes	 8
U+6589.0	fCangjie	YKSL
U+6589.0	fFourCorner	00220
U+6589.0	fHakka	 ce2; zai1; ci2
U+6589.0	fCantonese	cai4; zai6; zi1
U+6589.0	fMandarin	qi2; ji4; zi1; jian3
U+6589.0	fEnglish	even, uniform, of equal length
U+658A.0	fUTF8	斊
U+658A.0	fCNS11643	4-2D47
U+658A.0	fGB	E-2C45
U+658A.0	fR/S	 67.6
U+658A.0	fTotalStrokes	 10
U+658A.0	fCangjie	YKSJ
U+658A.0	fFourCorner	00400
U+658A.0	fCantonese	cai4
U+658A.0	fMandarin	qi2; ji4
U+658B.0	fUTF8	斋
U+658B.0	fCNS11643	3-3140
U+658B.0	fGB	0-552B
U+658B.0	fR/S	  67.6
U+658B.0	fTotalStrokes	 10
U+658B.0	fCangjie	YKMBL
U+658B.0	fFourCorner	00220
U+658B.0	fHakka	 zai1
U+658B.0	fCantonese	zaai1; zai1
U+658B.0	fMandarin	zhai1; zi1
U+658B.0	fEnglish	[1] same as &#x9F4B; [2] simplified form of &#x9F4B;
U+658C.0	fUTF8	斌
U+658C.0	fCNS11643	2-3C3E
U+658C.0	fBig5	D979
U+658C.0	fGB	0-3173
U+658C.0	fR/S	 67.7
U+658C.0	fTotalStrokes	 11
U+658C.0	fCangjie	YKMPM
U+658C.0	fFourCorner	03440
U+658C.0	fHakka	 bin1
U+658C.0	fCantonese	ban1
U+658C.0	fMandarin	bin1
U+658C.0	fEnglish	equally fine in external accomplishments and internal qualities
U+658D.0	fUTF8	斍
U+658D.0	fCNS11643	3-3638
U+658D.0	fGB	E-2C46
U+658D.0	fR/S	 67.7
U+658D.0	fTotalStrokes	 11
U+658D.0	fCangjie	YKBUU
U+658D.0	fFourCorner	00210
U+658D.0	fHakka	 gok7; gau5
U+658D.0	fCantonese	gok3; gaau3
U+658D.0	fMandarin	jue2
U+658D.0	fEnglish	perceive; feel
U+658E.0	fUTF8	斎
U+658E.0	fCNS11643	3-3639
U+658E.0	fGB	E-2C47
U+658E.0	fR/S	 67.7
U+658E.0	fTotalStrokes	 11
U+658E.0	fCangjie	YKLML
U+658E.0	fFourCorner	00220
U+658E.0	fHakka	 zai1
U+658E.0	fCantonese	zaai1; zai1
U+658E.0	fMandarin	zhai1; zi1
U+658E.0	fEnglish	vegetarian diet; study
U+658F.0	fUTF8	斏
U+658F.0	fGB	8-2D69
U+658F.0	fR/S	 67.7
U+658F.0	fTotalStrokes	 11
U+658F.0	fCangjie	YKIAV
U+658F.0	fCantonese	loeng4
U+658F.0	fMandarin	lang3
U+6590.0	fUTF8	斐
U+6590.0	fCNS11643	1-5F59
U+6590.0	fBig5	B4B4
U+6590.0	fGB	0-6C33
U+6590.0	fR/S	 67.8
U+6590.0	fTotalStrokes	 12
U+6590.0	fCangjie	LYYK
U+6590.0	fFourCorner	11400
U+6590.0	fHakka	  fui3; fui1; fi3
U+6590.0	fCantonese	fei2; fei1
U+6590.0	fMandarin	fei3
U+6590.0	fEnglish	graceful; elegant; beautiful
U+6590.1	fUTF8	斐
U+6590.1	fCNS11643	1-5F59
U+6590.1	fBig5	B4B4
U+6590.1	fGB	0-6C33
U+6590.1	fR/S	 67.8
U+6590.1	fTotalStrokes	 12
U+6590.1	fCangjie	LYYK
U+6590.1	fFourCorner	11400
U+6590.1	fMandarin	fei1
U+6590.1	fEnglish	a Chinese family name
U+6591.0	fUTF8	斑
U+6591.0	fCNS11643	1-5F58
U+6591.0	fBig5	B4B3
U+6591.0	fGB	0-305F
U+6591.0	fR/S	 67.8
U+6591.0	fTotalStrokes	 12
U+6591.0	fCangjie	MGYKG
U+6591.0	fFourCorner	11114
U+6591.0	fHakka	  ban1
U+6591.0	fCantonese	baan1
U+6591.0	fMandarin	ban1
U+6591.0	fEnglish	[1] speckles; spots; mottles [2] mottled; variegated; motley
U+6592.0	fUTF8	斒
U+6592.0	fCNS11643	2-4343
U+6592.0	fBig5	DDBD
U+6592.0	fGB	5-507A
U+6592.0	fR/S	 67.9
U+6592.0	fTotalStrokes	 13
U+6592.0	fCangjie	YKHSB
U+6592.0	fFourCorner	02427
U+6592.0	fHakka	 ban1; ben1
U+6592.0	fCantonese	baan1
U+6592.0	fMandarin	ban1
U+6592.0	fEnglish	[1] variegated; motley [2] gorgeous; resplendent
U+6593.0	fUTF8	斓
U+6593.0	fGB	0-6C35
U+6593.0	fR/S	 67.12
U+6593.0	fTotalStrokes	 16
U+6593.0	fCangjie	YKLSW
U+6593.0	fHakka	 lan2
U+6593.0	fCantonese	laan4; laan6 (also)
U+6593.0	fMandarin	lan2
U+6593.0	fEnglish	simplified form of &#x6595;
U+6594.0	fUTF8	斔
U+6594.0	fCNS11643	2-6164
U+6594.0	fBig5	EFD8
U+6594.0	fGB	5-507B
U+6594.0	fR/S	 67.15
U+6594.0	fTotalStrokes	 19
U+6594.0	fCangjie	YIHXO
U+6594.0	fFourCorner	07137
U+6594.0	fCantonese	jyu4; jyu5
U+6594.0	fMandarin	yu3; zhong1
U+6595.0	fUTF8	斕
U+6595.0	fCNS11643	1-7A6E
U+6595.0	fBig5	C4E3
U+6595.0	fGB	1-6C35
U+6595.0	fR/S	 67.17
U+6595.0	fTotalStrokes	 21
U+6595.0	fCangjie	YKANW
U+6595.0	fFourCorner	07420
U+6595.0	fHakka	 lan2
U+6595.0	fCantonese	laan4; laan6 (also)
U+6595.0	fMandarin	lan2
U+6595.0	fEnglish	multicolored
U+6596.0	fUTF8	斖
U+6596.0	fCNS11643	2-6F2D
U+6596.0	fBig5	F7DE
U+6596.0	fGB	E-2C48
U+6596.0	fR/S	 67.19
U+6596.0	fTotalStrokes	 23
U+6596.0	fCangjie	YKHBM
U+6596.0	fFourCorner	00107
U+6596.0	fCantonese	mei5
U+6596.0	fMandarin	wei3; men2
U+6597.0	fUTF8	斗
U+6597.0	fCNS11643	1-4547
U+6597.0	fBig5	A4E6
U+6597.0	fGB	0-3637
U+6597.0	fR/S	 68.0
U+6597.0	fTotalStrokes	 4
U+6597.0	fCangjie	YJ
U+6597.0	fFourCorner	34000
U+6597.0	fHakka	  deu3; deu5; diu3
U+6597.0	fCantonese	dau2; dau3
U+6597.0	fMandarin	dou4; dou3
U+6597.0	fEnglish	[1] Chinese peck [2] large container for wine [3] one of the 28 constellations [4] simplified form of &#x9B25;
U+6598.0	fUTF8	斘
U+6598.0	fCNS11643	4-2441
U+6598.0	fGB	E-2C49
U+6598.0	fR/S	  68.3
U+6598.0	fTotalStrokes	 7
U+6598.0	fCangjie	NIYJ
U+6598.0	fFourCorner	24200
U+6598.0	fCantonese	sing1
U+6598.0	fMandarin	sheng1
U+6599.0	fUTF8	料
U+6599.0	fCNS11643	1-5568
U+6599.0	fBig5	AEC6
U+6599.0	fGB	0-414F
U+6599.0	fR/S	 68.6
U+6599.0	fTotalStrokes	 10
U+6599.0	fCangjie	FDYJ
U+6599.0	fFourCorner	94900
U+6599.0	fHakka	 liau5;  liau3; liau6
U+6599.0	fCantonese	liu6; liu2 (also)
U+6599.0	fMandarin	liao4; liao2
U+6599.0	fEnglish	[1] reckon; conjecture; estimate [2] infer; anticipate; foresee [3] consider; calculate [4] manage; handle; care [5] material; stuff [6] makings; ingredients; stuff
U+659A.0	fUTF8	斚
U+659A.0	fCNS11643	3-3142
U+659A.0	fGB	E-2C4A
U+659A.0	fR/S	 68.6
U+659A.0	fTotalStrokes	 10
U+659A.0	fCangjie	IIBYJ
U+659A.0	fFourCorner	23400
U+659A.0	fHakka	 ga3; ga5
U+659A.0	fCantonese	gaa2
U+659A.0	fMandarin	jia3
U+659A.0	fEnglish	a small cup of stone with ears, used in ancient times for libations
U+659B.0	fUTF8	斛
U+659B.0	fCNS11643	1-5A7C
U+659B.0	fBig5	B1D8
U+659B.0	fGB	0-757A
U+659B.0	fR/S	 68.7
U+659B.0	fTotalStrokes	 11
U+659B.0	fCangjie	NBYJ
U+659B.0	fFourCorner	24200
U+659B.0	fHakka	  fuk8
U+659B.0	fCantonese	huk6; fuk6
U+659B.0	fMandarin	hu2
U+659B.0	fEnglish	dry measure 10 or 5 times of dou3
U+659C.0	fUTF8	斜
U+659C.0	fCNS11643	1-5A7B
U+659C.0	fBig5	B1D7
U+659C.0	fGB	0-5031
U+659C.0	fR/S	 68.7
U+659C.0	fTotalStrokes	 11
U+659C.0	fCangjie	ODYJ
U+659C.0	fFourCorner	84900
U+659C.0	fHakka	  sia2;  cia5; cia2; zia2; cia3
U+659C.0	fCantonese	ce4; je4; ce2 (also); ce3 (also)
U+659C.0	fMandarin	xie2; xia2; ye2; zha2
U+659C.0	fEnglish	slanting; sloping; inclined; leaning; oblique; diagonal
U+659D.0	fUTF8	斝
U+659D.0	fCNS11643	2-3C3F
U+659D.0	fBig5	D97A
U+659D.0	fGB	3-5065
U+659D.0	fR/S	 68.8
U+659D.0	fTotalStrokes	 12
U+659D.0	fCangjie	RRBYJ
U+659D.0	fFourCorner	66402
U+659D.0	fHakka	  ga5
U+659D.0	fCantonese	gaa2
U+659D.0	fMandarin	jia3
U+659D.0	fEnglish	 small jade wine cup
U+659E.0	fUTF8	斞
U+659E.0	fCNS11643	2-3C40
U+659E.0	fBig5	D97B
U+659E.0	fGB	5-525E
U+659E.0	fR/S	 68.9
U+659E.0	fTotalStrokes	 13
U+659E.0	fCangjie	HOYJ
U+659E.0	fFourCorner	74400
U+659E.0	fHakka	 ri2; ji2; i3; ji3
U+659E.0	fCantonese	jyu5 (also); jyu4
U+659E.0	fMandarin	yu3
U+659E.0	fEnglish	 a stack of grain a measure of 16 dou3
U+659F.0	fUTF8	斟
U+659F.0	fCNS11643	1-643A
U+659F.0	fBig5	B772
U+659F.0	fGB	0-5565
U+659F.0	fR/S	 68.9
U+659F.0	fTotalStrokes	 13
U+659F.0	fCangjie	TVYJ
U+659F.0	fFourCorner	44700
U+659F.0	fHakka	  zim1;  zhim1; zihm1
U+659F.0	fCantonese	zam1
U+659F.0	fMandarin	zhen1
U+659F.0	fEnglish	[1] pour wine or tea into cup [2] consider
U+65A0.0	fUTF8	斠
U+65A0.0	fCNS11643	2-4A5C
U+65A0.0	fBig5	E1F5
U+65A0.0	fGB	3-5067
U+65A0.0	fR/S	 68.10
U+65A0.0	fTotalStrokes	 14
U+65A0.0	fCangjie	TBYJ
U+65A0.0	fFourCorner	54500
U+65A0.0	fCantonese	gaau3; gok3 (also); gau3
U+65A0.0	fMandarin	jiao4
U+65A0.0	fEnglish	a leveling stick (used for measuring the volume of grain, etc.)
U+65A1.0	fUTF8	斡
U+65A1.0	fCNS11643	1-687E
U+65A1.0	fBig5	BA57
U+65A1.0	fGB	0-4E53
U+65A1.0	fR/S	 68.10
U+65A1.0	fTotalStrokes	 14
U+65A1.0	fCangjie	JJOYJ
U+65A1.0	fFourCorner	48440
U+65A1.0	fHakka	  wat7; ren5; jen5; vat7; gwon3; gon3
U+65A1.0	fCantonese	waat3; gun2; gwun2 (also); wat3
U+65A1.0	fMandarin	wo4; guan3
U+65A1.0	fEnglish	 revolve; rotate; turn
U+65A2.0	fUTF8	斢
U+65A2.0	fCNS11643	2-573D
U+65A2.0	fBig5	E9B2
U+65A2.0	fGB	3-7B75
U+65A2.0	fR/S	 68.12
U+65A2.0	fTotalStrokes	 16
U+65A2.0	fCangjie	TCYJ
U+65A2.0	fFourCorner	44800
U+65A2.0	fHakka	 teu3; teu5
U+65A2.0	fCantonese	tau3; tau2; dau6
U+65A2.0	fMandarin	tou3; tiao3; tou4
U+65A2.0	fEnglish	 loot; plunder
U+65A3.0	fUTF8	斣
U+65A3.0	fCNS11643	4-563F
U+65A3.0	fGB	3-5069
U+65A3.0	fR/S	  68.13
U+65A3.0	fTotalStrokes	 17; 16
U+65A3.0	fCangjie	WIYJ
U+65A3.0	fFourCorner	64100
U+65A3.0	fCantonese	dau3
U+65A3.0	fMandarin	chu4; dou4
U+65A4.0	fUTF8	斤
U+65A4.0	fCNS11643	1-4548
U+65A4.0	fBig5	A4E7
U+65A4.0	fGB	0-3D6F
U+65A4.0	fR/S	 69.0
U+65A4.0	fTotalStrokes	 4
U+65A4.0	fCangjie	HML
U+65A4.0	fFourCorner	72221
U+65A4.0	fHakka	  gin1
U+65A4.0	fCantonese	gan1; gan3
U+65A4.0	fMandarin	jin1; jin5; jin4
U+65A4.0	fEnglish	[1] catty (unit of weight) [2] axe [3] keen in observation; shrewd [4] Kangxi radical 69
U+65A5.0	fUTF8	斥
U+65A5.0	fCNS11643	1-4658
U+65A5.0	fBig5	A5B8
U+65A5.0	fGB	0-3362
U+65A5.0	fR/S	 69.1
U+65A5.0	fTotalStrokes	 5
U+65A5.0	fCangjie	HMY
U+65A5.0	fFourCorner	72231
U+65A5.0	fHakka	 chit7; cit7; ts'iit7; cak7
U+65A5.0	fCantonese	cik1
U+65A5.0	fMandarin	chi4
U+65A5.0	fEnglish	[1] blame; accuse; reproach; reprove; censure [2] expel; drive off; banish; eject [3] survey; observe; reconnoiter
U+65A6.0	fUTF8	斦
U+65A6.0	fCNS11643	3-293B
U+65A6.0	fGB	5-4E5D
U+65A6.0	fR/S	  69.4
U+65A6.0	fTotalStrokes	 8
U+65A6.0	fCangjie	HLHML
U+65A6.0	fFourCorner	72220
U+65A6.0	fCantonese	ngan4; an4 (also)
U+65A6.0	fMandarin	yin2
U+65A7.0	fUTF8	斧
U+65A7.0	fCNS11643	1-4D74
U+65A7.0	fBig5	A9F2
U+65A7.0	fGB	0-382B
U+65A7.0	fR/S	 69.4
U+65A7.0	fTotalStrokes	 8
U+65A7.0	fCangjie	CKHML
U+65A7.0	fFourCorner	80221
U+65A7.0	fHakka	  bu3; bu1; fu3
U+65A7.0	fCantonese	fu2
U+65A7.0	fMandarin	fu3
U+65A7.0	fEnglish	[1] axe; hatchet [2] chop; hew; cut [3] trim
U+65A8.0	fUTF8	斨
U+65A8.0	fCNS11643	2-2723
U+65A8.0	fBig5	CCC2
U+65A8.0	fGB	3-3A39
U+65A8.0	fR/S	 69.4
U+65A8.0	fTotalStrokes	 8
U+65A8.0	fCangjie	VMHML
U+65A8.0	fFourCorner	22221
U+65A8.0	fHakka	 cong2
U+65A8.0	fCantonese	coeng1
U+65A8.0	fMandarin	qiang1
U+65A8.0	fEnglish	axe
U+65A9.0	fUTF8	斩
U+65A9.0	fGB	0-5536
U+65A9.0	fR/S	 159.4; 69.4
U+65A9.0	fTotalStrokes	 8
U+65A9.0	fCangjie	KQHML
U+65A9.0	fHakka	 zam3
U+65A9.0	fCantonese	zaam2
U+65A9.0	fMandarin	zhan3; zhan4
U+65A9.0	fEnglish	simplified form of &#x65AC;
U+65AA.0	fUTF8	斪
U+65AA.0	fCNS11643	2-2A6A
U+65AA.0	fBig5	CEE9
U+65AA.0	fGB	5-4E5F
U+65AA.0	fR/S	 69.5
U+65AA.0	fTotalStrokes	 9
U+65AA.0	fCangjie	PRHML
U+65AA.0	fFourCorner	22621
U+65AA.0	fCantonese	keoi4
U+65AA.0	fMandarin	qu2; ju1; gou1
U+65AB.0	fUTF8	斫
U+65AB.0	fCNS11643	1-514B
U+65AB.0	fBig5	AC48
U+65AB.0	fGB	0-6D3D
U+65AB.0	fR/S	 69.5
U+65AB.0	fTotalStrokes	 9
U+65AB.0	fCangjie	MRHML
U+65AB.0	fFourCorner	12621
U+65AB.0	fHakka	 zok7; chok7; zhok7
U+65AB.0	fCantonese	zoek3
U+65AB.0	fMandarin	zhuo2
U+65AB.0	fEnglish	 cut; chop (wood)
U+65AC.0	fUTF8	斬
U+65AC.0	fCNS11643	1-5A7D
U+65AC.0	fBig5	B1D9
U+65AC.0	fGB	1-5536
U+65AC.0	fR/S	 69.7
U+65AC.0	fTotalStrokes	 11
U+65AC.0	fCangjie	JJHML
U+65AC.0	fFourCorner	52021
U+65AC.0	fHakka	  zam3
U+65AC.0	fCantonese	zaam2
U+65AC.0	fMandarin	zhan3; zhan4
U+65AC.0	fEnglish	[1] cut; chop [2] kill; behead
U+65AD.0	fUTF8	断
U+65AD.0	fCNS11643	3-363A
U+65AD.0	fGB	0-364F
U+65AD.0	fR/S	 69.7
U+65AD.0	fTotalStrokes	 11
U+65AD.0	fCangjie	VDHML
U+65AD.0	fFourCorner	22720
U+65AD.0	fHakka	 don5; ton1
U+65AD.0	fCantonese	dyun6; tyun5 (also); dyun3
U+65AD.0	fMandarin	duan4
U+65AD.0	fEnglish	[1] non-classical form of &#x65B7; [2] simplified form of &#x65B7;
U+65AE.0	fUTF8	斮
U+65AE.0	fCNS11643	2-3C41
U+65AE.0	fBig5	D97C
U+65AE.0	fGB	E-2C4B
U+65AE.0	fR/S	 69.8
U+65AE.0	fTotalStrokes	 12
U+65AE.0	fCangjie	TAHML
U+65AE.0	fFourCorner	42621
U+65AE.0	fHakka	  cok7
U+65AE.0	fCantonese	zoek3
U+65AE.0	fMandarin	zhuo2; zhuo1
U+65AE.0	fEnglish	 cut; pair
U+65AF.0	fUTF8	斯
U+65AF.0	fCNS11643	1-5F5A
U+65AF.0	fBig5	B4B5
U+65AF.0	fGB	0-4B39
U+65AF.0	fR/S	 69.8
U+65AF.0	fTotalStrokes	 12
U+65AF.0	fCangjie	TCHML
U+65AF.0	fFourCorner	42821
U+65AF.0	fHakka	  sii1; su1; se1
U+65AF.0	fCantonese	si1
U+65AF.0	fMandarin	si1; si4
U+65AF.0	fEnglish	[1] this; these; such; here [2] then; thus (connecting particle) [3] tear; rip; cut away
U+65B0.0	fUTF8	新
U+65B0.0	fCNS11643	1-643B
U+65B0.0	fBig5	B773
U+65B0.0	fGB	0-5042
U+65B0.0	fR/S	 69.9
U+65B0.0	fTotalStrokes	 13
U+65B0.0	fCangjie	YDHML
U+65B0.0	fFourCorner	02921
U+65B0.0	fHakka	  sin1
U+65B0.0	fCantonese	san1
U+65B0.0	fMandarin	xin1
U+65B0.0	fEnglish	[1] new; novel; fresh [2] beginning; starting [3] recent; modern [4] neo- [5] the Hsin dynasty (8-22 A.D.)
U+65B1.0	fUTF8	斱
U+65B1.0	fCNS11643	4-3D63
U+65B1.0	fGB	E-2C4C
U+65B1.0	fR/S	 69.9
U+65B1.0	fTotalStrokes	 13
U+65B1.0	fCangjie	JAHML
U+65B1.0	fFourCorner	42620
U+65B1.0	fHakka	 zhok7
U+65B1.0	fCantonese	zoek3
U+65B1.0	fMandarin	zhuo2
U+65B2.0	fUTF8	斲
U+65B2.0	fCNS11643	2-5068
U+65B2.0	fBig5	E5C1
U+65B2.0	fGB	E-2C4D
U+65B2.0	fR/S	 69.10; 69.11
U+65B2.0	fTotalStrokes	 14
U+65B2.0	fCangjie	RMHML
U+65B2.0	fFourCorner	72121
U+65B2.0	fHakka	 dok8; dok7
U+65B2.0	fCantonese	zoek3; doek3
U+65B2.0	fMandarin	zhuo2
U+65B2.0	fEnglish	 cut; chop; hew
U+65B3.0	fUTF8	斳
U+65B3.0	fCNS11643	2-5069
U+65B3.0	fBig5	E5C2
U+65B3.0	fGB	5-4E64
U+65B3.0	fR/S	 69.11
U+65B3.0	fTotalStrokes	 15
U+65B3.0	fCangjie	TMHML
U+65B3.0	fFourCorner	42121
U+65B3.0	fHakka	 giun3
U+65B3.0	fCantonese	kan4
U+65B3.0	fMandarin	qin2; jin3
U+65B4.0	fUTF8	斴
U+65B4.0	fCNS11643	4-504E
U+65B4.0	fGB	E-2C4E
U+65B4.0	fR/S	  69.12
U+65B4.0	fTotalStrokes	 16
U+65B4.0	fCangjie	FQHML
U+65B4.0	fFourCorner	92220
U+65B4.0	fCantonese	leon4
U+65B4.0	fMandarin	lin2
U+65B4.0	fEnglish	the sound of water flowing among rocks
U+65B5.0	fUTF8	斵
U+65B5.0	fCNS11643	3-546B
U+65B5.0	fGB	E-2C4F
U+65B5.0	fR/S	 69.13
U+65B5.0	fTotalStrokes	 17
U+65B5.0	fCangjie	HMHML
U+65B5.0	fFourCorner	72120
U+65B5.0	fHakka	  dok7
U+65B5.0	fCantonese	zoek3; doek3
U+65B5.0	fMandarin	zhuo2
U+65B5.0	fEnglish	to cut to pieces; to hack; to chop or hew; to carve for ornaments
U+65B6.0	fUTF8	斶
U+65B6.0	fCNS11643	2-5C7B
U+65B6.0	fBig5	ECF0
U+65B6.0	fGB	3-4C36
U+65B6.0	fR/S	 69.13
U+65B6.0	fTotalStrokes	 17
U+65B6.0	fCangjie	HLWLI
U+65B6.0	fFourCorner	76227
U+65B6.0	fCantonese	cuk1; zuk1 (also)
U+65B6.0	fMandarin	chu4
U+65B7.0	fUTF8	斷
U+65B7.0	fCNS11643	1-764B
U+65B7.0	fBig5	C25F
U+65B7.0	fGB	1-364F
U+65B7.0	fR/S	 69.14
U+65B7.0	fTotalStrokes	 18
U+65B7.0	fCangjie	VIHML
U+65B7.0	fFourCorner	22721
U+65B7.0	fHakka	  don5;  ton1; don3; {ton2}; ton5; dun3; ton2
U+65B7.0	fCantonese	dyun6; tyun5 (also); dyun3
U+65B7.0	fMandarin	duan4
U+65B7.0	fEnglish	[1] cut apart; sever [2] give up; abstain from [3] judge; decide; conclude [4] cut off; interrupt; break; broken
U+65B8.0	fUTF8	斸
U+65B8.0	fCNS11643	2-707E
U+65B8.0	fBig5	F8F0
U+65B8.0	fGB	3-4C35
U+65B8.0	fR/S	 69.21
U+65B8.0	fTotalStrokes	 25
U+65B8.0	fCangjie	SIHML
U+65B8.0	fFourCorner	72221
U+65B8.0	fHakka	 dok7; chuk7; zuk7; zhuk7
U+65B8.0	fCantonese	zuk1
U+65B8.0	fMandarin	zhu3; zhu2; zhuo2
U+65B8.0	fEnglish	cut
U+65B9.0	fUTF8	方
U+65B9.0	fCNS11643	1-4549
U+65B9.0	fBig5	A4E8
U+65B9.0	fGB	0-373D
U+65B9.0	fR/S	 70.0
U+65B9.0	fTotalStrokes	 4
U+65B9.0	fCangjie	YHS
U+65B9.0	fFourCorner	00227
U+65B9.0	fHakka	  fong1
U+65B9.0	fCantonese	fong1
U+65B9.0	fMandarin	fang1; feng1; pang2; wang3
U+65B9.0	fEnglish	[1] square; rectangular [2] honest; morally upright [3] region; area; place [4] prescription; recipe [5] direction [6] occultism [7] just now; just then; just [8] power (mathematics) [9] short for square meter or cubic meter (classifier) [10] method; way [11] aspect [12] one side [13] a Chinese family name
U+65B9.1	fUTF8	方
U+65B9.1	fCNS11643	1-4549
U+65B9.1	fBig5	A4E8
U+65B9.1	fGB	0-373D
U+65B9.1	fR/S	 70.0
U+65B9.1	fTotalStrokes	 4
U+65B9.1	fCangjie	YHS
U+65B9.1	fFourCorner	00227
U+65B9.1	fMandarin	fang5
U+65BA.0	fUTF8	斺
U+65BA.0	fCNS11643	3-293C
U+65BA.0	fGB	3-4F40
U+65BA.0	fR/S	 70.4
U+65BA.0	fTotalStrokes	 8
U+65BA.0	fCangjie	YSOLL
U+65BA.0	fFourCorner	08220
U+65BA.0	fCantonese	cin2
U+65BA.0	fMandarin	xie4
U+65BB.0	fUTF8	斻
U+65BB.0	fCNS11643	2-2724
U+65BB.0	fBig5	CCC3
U+65BB.0	fGB	5-5121
U+65BB.0	fR/S	 70.4
U+65BB.0	fTotalStrokes	 8
U+65BB.0	fCangjie	YSOHN
U+65BB.0	fFourCorner	08217
U+65BB.0	fCantonese	hong4
U+65BB.0	fMandarin	hang2
U+65BC.0	fUTF8	於
U+65BC.0	fCNS11643	1-4D75
U+65BC.0	fBig5	A9F3
U+65BC.0	fGB	0-6C36
U+65BC.0	fR/S	 70.4
U+65BC.0	fTotalStrokes	 8
U+65BC.0	fCangjie	YSOY
U+65BC.0	fFourCorner	08233
U+65BC.0	fHakka	  ji1; ri1; wu1; vu1; i1
U+65BC.0	fCantonese	jyu1; wu1; jyu4
U+65BC.0	fMandarin	yu2; wu1; yu1
U+65BC.0	fEnglish	[1] in; at; on [2] than; then; to; with reference to [3] compared with [4] a Chinese family name
U+65BD.0	fUTF8	施
U+65BD.0	fCNS11643	1-514C
U+65BD.0	fBig5	AC49
U+65BD.0	fGB	0-4A29
U+65BD.0	fR/S	 70.5
U+65BD.0	fTotalStrokes	 9
U+65BD.0	fCangjie	YSOPD
U+65BD.0	fFourCorner	08212
U+65BD.0	fHakka	 si1; shi5; shi1; si3; sii1; sii5; she1
U+65BD.0	fCantonese	si1; ji6; si3
U+65BD.0	fMandarin	shi1; shi3; yi4; yi2; shi4
U+65BD.0	fEnglish	[1] grant; bestow; give (alms, etc.) [2] act; do; make
U+65BE.0	fUTF8	斾
U+65BE.0	fCNS11643	3-2C77
U+65BE.0	fGB	E-2C50
U+65BE.0	fR/S	 70.5
U+65BE.0	fTotalStrokes	 9
U+65BE.0	fCangjie	YSOLB
U+65BE.0	fFourCorner	08220
U+65BE.0	fHakka	 pai5;  pui5; pi5
U+65BE.0	fCantonese	pui3
U+65BE.0	fMandarin	pei4
U+65BE.0	fEnglish	corrupted form of &#x65C6;
U+65BF.0	fUTF8	斿
U+65BF.0	fCNS11643	2-2A6B
U+65BF.0	fBig5	CEEA
U+65BF.0	fGB	5-5122
U+65BF.0	fR/S	 70.5
U+65BF.0	fTotalStrokes	 9
U+65BF.0	fCangjie	YSOND
U+65BF.0	fFourCorner	08247
U+65BF.0	fHakka	 ju2; riu2
U+65BF.0	fCantonese	jau4; lau4; jau6
U+65BF.0	fMandarin	you2; liu2
U+65BF.0	fEnglish	[1] swim [2] to rove about freely [3] ornaments hanging own from banners
U+65C0.0	fUTF8	旀
U+65C0.0	fGB	1-7D3C
U+65C0.0	fR/S	 70.5
U+65C0.0	fTotalStrokes	 9
U+65C0.0	fCangjie	YSIF
U+65C0.0	fCantonese	ji5
U+65C0.0	fEnglish	 place name
U+65C1.0	fUTF8	旁
U+65C1.0	fCNS11643	1-5569
U+65C1.0	fBig5	AEC7
U+65C1.0	fGB	0-4554
U+65C1.0	fR/S	 70.6
U+65C1.0	fTotalStrokes	 10
U+65C1.0	fCangjie	YBYHS
U+65C1.0	fFourCorner	00227
U+65C1.0	fHakka	  pong2; pong3
U+65C1.0	fCantonese	pong4; bong6
U+65C1.0	fMandarin	pang2; bang1; beng1; jing1; pang1; peng2
U+65C1.0	fEnglish	[1] side (branches, doors, ways, etc.); the side [2] by the side; close by; near; nearby [3] other; else
U+65C1.1	fUTF8	旁
U+65C1.1	fCNS11643	1-5569
U+65C1.1	fBig5	AEC7
U+65C1.1	fGB	0-4554
U+65C1.1	fR/S	 70.6
U+65C1.1	fTotalStrokes	 10
U+65C1.1	fCangjie	YBYHS
U+65C1.1	fFourCorner	00227
U+65C1.1	fMandarin	bang4
U+65C1.1	fEnglish	[1] busy complicated [2] criss-cross
U+65C2.0	fUTF8	旂
U+65C2.0	fCNS11643	2-2F54
U+65C2.0	fBig5	D1D2
U+65C2.0	fGB	8-2D7E
U+65C2.0	fR/S	 70.6
U+65C2.0	fTotalStrokes	 10
U+65C2.0	fCangjie	YSOHL
U+65C2.0	fFourCorner	08221
U+65C2.0	fHakka	  ki2
U+65C2.0	fCantonese	kei4
U+65C2.0	fMandarin	qi2
U+65C2.0	fEnglish	flag
U+65C3.0	fUTF8	旃
U+65C3.0	fCNS11643	2-2F52
U+65C3.0	fBig5	D1D0
U+65C3.0	fGB	0-6C39
U+65C3.0	fR/S	 70.6
U+65C3.0	fTotalStrokes	 10
U+65C3.0	fCangjie	YSOBY
U+65C3.0	fFourCorner	08247
U+65C3.0	fHakka	 zhan1; zan1; zen1; zhen1
U+65C3.0	fCantonese	zin1
U+65C3.0	fMandarin	zhan1
U+65C3.0	fEnglish	silk banner with bent pole
U+65C4.0	fUTF8	旄
U+65C4.0	fCNS11643	2-2F53
U+65C4.0	fBig5	D1D1
U+65C4.0	fGB	0-6C38
U+65C4.0	fR/S	 70.6
U+65C4.0	fTotalStrokes	 10
U+65C4.0	fCangjie	YSOHU
U+65C4.0	fFourCorner	08214
U+65C4.0	fHakka	 mau2; mau3; mau1
U+65C4.0	fCantonese	mou4; mou6
U+65C4.0	fMandarin	mao1
U+65C4.0	fEnglish	a kind of ancient flag with the oxtails set up at the top of its staff
U+65C4.1	fUTF8	旄
U+65C4.1	fCNS11643	2-2F53
U+65C4.1	fBig5	D1D1
U+65C4.1	fGB	0-6C38
U+65C4.1	fR/S	 70.6
U+65C4.1	fTotalStrokes	 10
U+65C4.1	fCangjie	YSOHU
U+65C4.1	fFourCorner	08214
U+65C4.1	fMandarin	mao4; mao2
U+65C4.1	fEnglish	old
U+65C5.0	fUTF8	旅
U+65C5.0	fCNS11643	1-556A
U+65C5.0	fBig5	AEC8
U+65C5.0	fGB	0-4243
U+65C5.0	fR/S	 70.6
U+65C5.0	fTotalStrokes	 10
U+65C5.0	fCangjie	YSOHV
U+65C5.0	fFourCorner	08232
U+65C5.0	fHakka	  li3;  li1
U+65C5.0	fCantonese	leoi5; leoi3
U+65C5.0	fMandarin	l3; luu3; lu2; lu3
U+65C5.0	fEnglish	[1] travel; lodge [2] traveler; passenger; lodger [3] multitude; people [4] disciples; pupils; followers; subordinates [5] order; sequence; arrange [6] brigade; troops (military) [7] sacrifice to the mountains [8] proceed together; do things together
U+65C6.0	fUTF8	旆
U+65C6.0	fCNS11643	2-2F51
U+65C6.0	fBig5	D1CF
U+65C6.0	fGB	0-6C37
U+65C6.0	fR/S	 70.6
U+65C6.0	fTotalStrokes	 10
U+65C6.0	fCangjie	YSOJB
U+65C6.0	fFourCorner	08227
U+65C6.0	fHakka	 pai5;  pui5; pi5
U+65C6.0	fCantonese	pui3; bui3
U+65C6.0	fMandarin	pei4
U+65C6.0	fEnglish	 flag ornament flags; banners
U+65C7.0	fUTF8	旇
U+65C7.0	fCNS11643	3-363C
U+65C7.0	fGB	3-4F44
U+65C7.0	fR/S	  70.7
U+65C7.0	fTotalStrokes	 11
U+65C7.0	fCangjie	YSODE
U+65C7.0	fFourCorner	08240
U+65C7.0	fHakka	  pi1
U+65C7.0	fCantonese	pei1
U+65C7.0	fMandarin	pi1
U+65C7.0	fEnglish	 flutter
U+65C8.0	fUTF8	旈
U+65C8.0	fCNS11643	4-3234
U+65C8.0	fGB	E-2C51
U+65C8.0	fR/S	 70.7; 70.6
U+65C8.0	fTotalStrokes	 11; 10
U+65C8.0	fCangjie	YSYIU
U+65C8.0	fFourCorner	00210
U+65C8.0	fCantonese	lau4
U+65C8.0	fMandarin	liu2
U+65C8.0	fEnglish	 fringes of pearls on crowns pennant
U+65C9.0	fUTF8	旉
U+65C9.0	fCNS11643	3-363F
U+65C9.0	fGB	5-5124
U+65C9.0	fR/S	  70.7
U+65C9.0	fTotalStrokes	 11
U+65C9.0	fCangjie	IBYHS
U+65C9.0	fFourCorner	53220
U+65C9.0	fHakka	  fu1
U+65C9.0	fCantonese	fu1
U+65C9.0	fMandarin	fu1
U+65C9.0	fEnglish	ancient form of &#x6577;
U+65CA.0	fUTF8	旊
U+65CA.0	fCNS11643	3-363D
U+65CA.0	fGB	3-4F43
U+65CA.0	fR/S	 70.6; 70.7
U+65CA.0	fTotalStrokes	 11; 10
U+65CA.0	fCangjie	YSOMN
U+65CA.0	fFourCorner	08210
U+65CA.0	fCantonese	fong2
U+65CA.0	fMandarin	fang3
U+65CB.0	fUTF8	旋
U+65CB.0	fCNS11643	1-5B21
U+65CB.0	fBig5	B1DB
U+65CB.0	fGB	0-507D
U+65CB.0	fR/S	 70.7
U+65CB.0	fTotalStrokes	 11
U+65CB.0	fCangjie	YSONO
U+65CB.0	fFourCorner	08281
U+65CB.0	fHakka	  sien2; sion2; cion5; sian1; son2; cion6; sen2
U+65CB.0	fCantonese	syun4; syun6 (also)
U+65CB.0	fMandarin	xuan2
U+65CB.0	fEnglish	[1] revolve; move in orbit; circle; spin [2] return; turn back
U+65CB.1	fUTF8	旋
U+65CB.1	fCNS11643	1-5B21
U+65CB.1	fBig5	B1DB
U+65CB.1	fGB	0-507D
U+65CB.1	fR/S	 70.7
U+65CB.1	fTotalStrokes	 11
U+65CB.1	fCangjie	YSONO
U+65CB.1	fFourCorner	08281
U+65CB.1	fMandarin	xuan4
U+65CB.1	fEnglish	[1] whirl [2] at the time; at the last moment; as soon as
U+65CC.0	fUTF8	旌
U+65CC.0	fCNS11643	1-5B22
U+65CC.0	fBig5	B1DC
U+65CC.0	fGB	0-6C3A
U+65CC.0	fR/S	 70.7
U+65CC.0	fTotalStrokes	 11
U+65CC.0	fCangjie	YSOHM
U+65CC.0	fFourCorner	08214
U+65CC.0	fHakka	  zin1
U+65CC.0	fCantonese	zing1; sing1 (also)
U+65CC.0	fMandarin	jing1
U+65CC.0	fEnglish	[1] a kind standard, banner or flag adorned with feathers [2] cite (one's merits, virtues, etc.); make manifest
U+65CD.0	fUTF8	旍
U+65CD.0	fCNS11643	2-356A
U+65CD.0	fBig5	D5A8
U+65CD.0	fGB	5-5128
U+65CD.0	fR/S	 70.7
U+65CD.0	fTotalStrokes	 11
U+65CD.0	fCangjie	YSOOI
U+65CD.0	fFourCorner	08237
U+65CD.0	fCantonese	zing1; sing1 (also)
U+65CD.0	fMandarin	jing1
U+65CE.0	fUTF8	旎
U+65CE.0	fCNS11643	1-5B23
U+65CE.0	fBig5	B1DD
U+65CE.0	fGB	0-6C3B
U+65CE.0	fR/S	 70.7
U+65CE.0	fTotalStrokes	 11
U+65CE.0	fCangjie	YSOSP
U+65CE.0	fFourCorner	08211
U+65CE.0	fHakka	  ni3; li3
U+65CE.0	fCantonese	nei5; lei5 (also)
U+65CE.0	fMandarin	ni3; ni2
U+65CE.0	fEnglish	[1] fluttering of flags [2] romantic
U+65CF.0	fUTF8	族
U+65CF.0	fCNS11643	1-5A7E
U+65CF.0	fBig5	B1DA
U+65CF.0	fGB	0-5765
U+65CF.0	fR/S	 70.7
U+65CF.0	fTotalStrokes	 11
U+65CF.0	fCangjie	YSOOK
U+65CF.0	fFourCorner	08234
U+65CF.0	fHakka	  cuk8
U+65CF.0	fCantonese	zuk6; cau3
U+65CF.0	fMandarin	zu2; cou4; zou4
U+65CF.0	fEnglish	[1] family clan; tribe; family; clan [2] race (of people); ethnic group [3] class or group of things with common features
U+65D0.0	fUTF8	旐
U+65D0.0	fCNS11643	2-3C42
U+65D0.0	fBig5	D97D
U+65D0.0	fGB	3-4F46
U+65D0.0	fR/S	 70.8
U+65D0.0	fTotalStrokes	 12
U+65D0.0	fCangjie	YSOLO
U+65D0.0	fFourCorner	08213
U+65D0.0	fHakka	 chau6; cau5; chau5; ch'eu5
U+65D0.0	fCantonese	siu6; ziu6
U+65D0.0	fMandarin	zhao4
U+65D0.0	fEnglish	 embroidered pennant; flag with tortoises and snakes emblazoned on it
U+65D1.0	fUTF8	旑
U+65D1.0	fCNS11643	3-3B79
U+65D1.0	fGB	E-2C52
U+65D1.0	fR/S	 70.8
U+65D1.0	fTotalStrokes	 12
U+65D1.0	fCangjie	YSKMR
U+65D1.0	fFourCorner	04220
U+65D1.0	fHakka	 ji3; ri3; i3
U+65D1.0	fCantonese	ji2
U+65D1.0	fMandarin	yi3; yi1; yi4
U+65D1.0	fEnglish	same as &#x65D6;
U+65D2.0	fUTF8	旒
U+65D2.0	fCNS11643	2-3C43
U+65D2.0	fBig5	D97E
U+65D2.0	fGB	0-6C3C
U+65D2.0	fR/S	 70.9
U+65D2.0	fTotalStrokes	 13
U+65D2.0	fCangjie	YSOYU
U+65D2.0	fFourCorner	08213
U+65D2.0	fHakka	 liu2; liu3
U+65D2.0	fCantonese	lau4
U+65D2.0	fMandarin	liu2
U+65D2.0	fEnglish	[1] a silk string used to hold a piece of jade hung on ancient ceremonial caps [2] a colored streamer hung on a flag
U+65D3.0	fUTF8	旓
U+65D3.0	fCNS11643	2-4344
U+65D3.0	fBig5	DDBE
U+65D3.0	fGB	3-4F47
U+65D3.0	fR/S	 70.9
U+65D3.0	fTotalStrokes	 13
U+65D3.0	fCangjie	YSOFB
U+65D3.0	fFourCorner	08227
U+65D3.0	fHakka	 sau1
U+65D3.0	fCantonese	saau1
U+65D3.0	fMandarin	shao1
U+65D3.0	fEnglish	serrated edges on a Chinese flag
U+65D4.0	fUTF8	旔
U+65D4.0	fCNS11643	4-3D64
U+65D4.0	fGB	E-2C53
U+65D4.0	fR/S	 70.9
U+65D4.0	fTotalStrokes	 13
U+65D4.0	fCangjie	YSNKQ
U+65D4.0	fFourCorner	05240
U+65D4.0	fCantonese	gin3
U+65D4.0	fMandarin	jian4
U+65D5.0	fUTF8	旕
U+65D5.0	fGB	1-7D76
U+65D5.0	fR/S	 70.9
U+65D5.0	fTotalStrokes	 13
U+65D5.0	fCangjie	YYRP
U+65D5.0	fCantonese	jyu1
U+65D5.0	fEnglish	 place name
U+65D6.0	fUTF8	旖
U+65D6.0	fCNS11643	1-6922
U+65D6.0	fBig5	BA59
U+65D6.0	fGB	0-6C3D
U+65D6.0	fR/S	 70.10
U+65D6.0	fTotalStrokes	 14
U+65D6.0	fCangjie	YSOKR
U+65D6.0	fFourCorner	08221
U+65D6.0	fHakka	 ji3; ri3; i3
U+65D6.0	fCantonese	ji2
U+65D6.0	fMandarin	yi3; yi1; yi4
U+65D6.0	fEnglish	[1] romantic; tender [2] charming; lovely; attractive; graceful [3] the fluttering of flags
U+65D7.0	fUTF8	旗
U+65D7.0	fCNS11643	1-6921
U+65D7.0	fBig5	BA58
U+65D7.0	fGB	0-466C
U+65D7.0	fR/S	 70.10
U+65D7.0	fTotalStrokes	 14
U+65D7.0	fCangjie	YSOTC
U+65D7.0	fFourCorner	08281
U+65D7.0	fHakka	  ki2; ki3
U+65D7.0	fCantonese	kei4; kei2 (also)
U+65D7.0	fMandarin	qi2
U+65D7.0	fEnglish	[1] banner; flag; streamer; pennant [2] sign; insignia; emblem [3] administrative division of Mongolia and Tsinghai [4] the Manchus
U+65D8.0	fUTF8	旘
U+65D8.0	fCNS11643	4-504F
U+65D8.0	fGB	E-2C54
U+65D8.0	fR/S	 70.12
U+65D8.0	fTotalStrokes	 16
U+65D8.0	fCangjie	YSYIA
U+65D8.0	fFourCorner	03250
U+65D8.0	fCantonese	ci3
U+65D8.0	fMandarin	zhi4
U+65D8.0	fEnglish	 pennon flag fasten
U+65D9.0	fUTF8	旙
U+65D9.0	fCNS11643	F-5464
U+65D9.0	fGB	E-2C55
U+65D9.0	fR/S	 70.12
U+65D9.0	fTotalStrokes	 16
U+65D9.0	fCangjie	YSHDW
U+65D9.0	fCantonese	faan1
U+65D9.0	fMandarin	fan1
U+65D9.0	fEnglish	a pennant; a banner
U+65DA.0	fUTF8	旚
U+65DA.0	fCNS11643	2-5C7C
U+65DA.0	fBig5	ECF1
U+65DA.0	fGB	5-512B
U+65DA.0	fR/S	 70.13
U+65DA.0	fTotalStrokes	 17
U+65DA.0	fCangjie	YSOMF
U+65DA.0	fFourCorner	08291
U+65DA.0	fCantonese	piu1
U+65DA.0	fMandarin	piao1
U+65DB.0	fUTF8	旛
U+65DB.0	fCNS11643	2-6165
U+65DB.0	fBig5	EFD9
U+65DB.0	fGB	3-4F49
U+65DB.0	fR/S	 70.14
U+65DB.0	fTotalStrokes	 18
U+65DB.0	fCangjie	YSOHW
U+65DB.0	fFourCorner	08269
U+65DB.0	fHakka	  fan1
U+65DB.0	fCantonese	faan1
U+65DB.0	fMandarin	fan1; fan2
U+65DB.0	fEnglish	[1] pennant; banner; general name of flags or streamers [2] a silk (usually rectangular) pennant for hanging
U+65DC.0	fUTF8	旜
U+65DC.0	fCNS11643	3-5A3F
U+65DC.0	fGB	5-512E
U+65DC.0	fR/S	 70.15
U+65DC.0	fTotalStrokes	 19
U+65DC.0	fCangjie	YSOYM
U+65DC.0	fFourCorner	08210
U+65DC.0	fHakka	 zhan1; zan1; zhen1
U+65DC.0	fCantonese	zin1
U+65DC.0	fMandarin	zhan1
U+65DC.0	fEnglish	flag
U+65DD.0	fUTF8	旝
U+65DD.0	fCNS11643	2-6555
U+65DD.0	fBig5	F24A
U+65DD.0	fGB	3-4F45
U+65DD.0	fR/S	 70.15
U+65DD.0	fTotalStrokes	 19
U+65DD.0	fCangjie	YSOOA
U+65DD.0	fFourCorner	08266
U+65DD.0	fHakka	  kwai5
U+65DD.0	fCantonese	kui2; kwui2 (also); kui3
U+65DD.0	fMandarin	guai4; gui4; kuai4
U+65DD.0	fEnglish	[1] banner, flag, or streamer used by a general [2] a coach used in ancient warfare for shooting stones
U+65DE.0	fUTF8	旞
U+65DE.0	fCNS11643	2-6554
U+65DE.0	fBig5	F249
U+65DE.0	fGB	3-4F4A
U+65DE.0	fR/S	 70.15
U+65DE.0	fTotalStrokes	 19
U+65DE.0	fCangjie	YSYOO
U+65DE.0	fFourCorner	07233
U+65DE.0	fCantonese	seoi6
U+65DE.0	fMandarin	sui4
U+65DF.0	fUTF8	旟
U+65DF.0	fCNS11643	2-687B
U+65DF.0	fBig5	F44F
U+65DF.0	fGB	3-4F41
U+65DF.0	fR/S	 70.16; 70.15
U+65DF.0	fTotalStrokes	 20
U+65DF.0	fCangjie	YSOHC
U+65DF.0	fFourCorner	08281
U+65DF.0	fHakka	 ji2; ri2; i2
U+65DF.0	fCantonese	jyu4
U+65DF.0	fMandarin	yu2
U+65DF.0	fEnglish	[1] military flag or standard; flag with falcons emblazoned on it [2] fluttering (said of flags)
U+65E0.0	fUTF8	无
U+65E0.0	fCNS11643	3-2169
U+65E0.0	fGB	0-4E5E
U+65E0.0	fR/S	 71.0
U+65E0.0	fTotalStrokes	 4
U+65E0.0	fCangjie	MKU
U+65E0.0	fFourCorner	10410
U+65E0.0	fHakka	  wu2;  mo2; mau2; vu2
U+65E0.0	fCantonese	mou4; mo4
U+65E0.0	fMandarin	wu2; mo2
U+65E0.0	fEnglish	[1] Kangxi radical 71 [2] simplified form of &#x7121;
U+65E1.0	fUTF8	旡
U+65E1.0	fCNS11643	2-213E
U+65E1.0	fBig5	C95E
U+65E1.0	fGB	5-4278
U+65E1.0	fR/S	 71.1
U+65E1.0	fTotalStrokes	 5
U+65E1.0	fCangjie	MVKU
U+65E1.0	fFourCorner	10410
U+65E1.0	fHakka	  gi5
U+65E1.0	fCantonese	gei3
U+65E1.0	fMandarin	ji4
U+65E1.0	fEnglish	 choke in eating
U+65E2.0	fUTF8	既
U+65E2.0	fCNS11643	1-514D
U+65E2.0	fBig5	AC4A
U+65E2.0	fGB	0-3C48
U+65E2.0	fR/S	 71.5
U+65E2.0	fTotalStrokes	 9
U+65E2.0	fCangjie	AIMVU
U+65E2.0	fFourCorner	71714
U+65E2.0	fHakka	  gi5
U+65E2.0	fCantonese	gei3
U+65E2.0	fMandarin	ji4; xi4
U+65E2.0	fEnglish	already; de facto; since; then
U+65E3.0	fUTF8	旣
U+65E3.0	fCNS11643	3-3641
U+65E3.0	fGB	E-2C56
U+65E3.0	fR/S	 71.7
U+65E3.0	fTotalStrokes	 11
U+65E3.0	fCangjie	HPMVU
U+65E3.0	fFourCorner	21710
U+65E3.0	fHakka	  gi5
U+65E3.0	fCantonese	gei3
U+65E3.0	fMandarin	ji4; xi4
U+65E3.0	fEnglish	[1] already [2] since; as; now that; inasmuch as [3] all [4] finish [5] a Chinese family name
U+65E4.0	fUTF8	旤
U+65E4.0	fCNS11643	3-4150
U+65E4.0	fGB	E-2C57
U+65E4.0	fR/S	  71.9
U+65E4.0	fTotalStrokes	 13
U+65E4.0	fCangjie	BBMVU
U+65E4.0	fFourCorner	71210
U+65E4.0	fHakka	  fo5; fo3; wo3
U+65E4.0	fCantonese	wo6; wo5
U+65E4.0	fMandarin	huo4
U+65E4.0	fEnglish	ancient form of &#x798D;
U+65E5.0	fUTF8	日
U+65E5.0	fCNS11643	1-454A
U+65E5.0	fBig5	A4E9
U+65E5.0	fGB	0-4855
U+65E5.0	fR/S	 72.0
U+65E5.0	fTotalStrokes	 4
U+65E5.0	fCangjie	A
U+65E5.0	fFourCorner	60100
U+65E5.0	fHakka	 ngit7; gnit7; ngit8
U+65E5.0	fCantonese	jat6; jat2 (also); mik6
U+65E5.0	fMandarin	ri4; mi4
U+65E5.0	fEnglish	[1] sun [2] day [3] daily; every day [4] Japan; Japanese [5] time [6] day; daytime
U+65E6.0	fUTF8	旦
U+65E6.0	fCNS11643	1-4659
U+65E6.0	fBig5	A5B9
U+65E6.0	fGB	0-3529
U+65E6.0	fR/S	 72.1
U+65E6.0	fTotalStrokes	 5
U+65E6.0	fCangjie	AM
U+65E6.0	fFourCorner	60100
U+65E6.0	fHakka	  dan5; dan3
U+65E6.0	fCantonese	daan3; daan2; daan6 (also)
U+65E6.0	fMandarin	dan4
U+65E6.0	fEnglish	[1] dawn; daybreak [2] morning; day [3] a female role in a Chinese opera
U+65E7.0	fUTF8	旧
U+65E7.0	fCNS11643	3-2251
U+65E7.0	fGB	0-3E49
U+65E7.0	fR/S	 72.1
U+65E7.0	fTotalStrokes	 5
U+65E7.0	fCangjie	LA
U+65E7.0	fFourCorner	26000
U+65E7.0	fHakka	 kiu5
U+65E7.0	fCantonese	gau6
U+65E7.0	fMandarin	jiu4; xiu1
U+65E7.0	fEnglish	simplified form of &#x820A;
U+65E8.0	fUTF8	旨
U+65E8.0	fCNS11643	1-482F
U+65E8.0	fBig5	A6AE
U+65E8.0	fGB	0-563C
U+65E8.0	fR/S	 72.2
U+65E8.0	fTotalStrokes	 6
U+65E8.0	fCangjie	PA
U+65E8.0	fFourCorner	21601
U+65E8.0	fHakka	 zi3;  zhi3; zih3; tje3
U+65E8.0	fCantonese	zi2
U+65E8.0	fMandarin	zhi3
U+65E8.0	fEnglish	[1] purpose; aim; will; intention [2] imperial decree [3] good; excellent; beautiful [4] tasty; pleasant to the palate; delicious
U+65E9.0	fUTF8	早
U+65E9.0	fCNS11643	1-482E
U+65E9.0	fBig5	A6AD
U+65E9.0	fGB	0-5467
U+65E9.0	fR/S	 72.2
U+65E9.0	fTotalStrokes	 6
U+65E9.0	fCangjie	AJ
U+65E9.0	fFourCorner	60400
U+65E9.0	fHakka	  zau3;  zo3
U+65E9.0	fCantonese	zou2
U+65E9.0	fMandarin	zao3
U+65E9.0	fEnglish	[1] early; earlier; soon; beforehand; previous; premature [2] morning [3] ago; before [4] Good morning!
U+65EA.0	fUTF8	旪
U+65EA.0	fCNS11643	3-2373
U+65EA.0	fGB	5-4A36
U+65EA.0	fR/S	 72.2
U+65EA.0	fTotalStrokes	 6
U+65EA.0	fCangjie	AJ
U+65EA.0	fFourCorner	64000
U+65EA.0	fHakka	 hiap8; hiap7
U+65EA.0	fCantonese	hip3; hip6
U+65EA.0	fMandarin	xie2
U+65EA.0	fEnglish	same as &#x5354;
U+65EB.0	fUTF8	旫
U+65EB.0	fCNS11643	3-2372
U+65EB.0	fGB	E-2C58
U+65EB.0	fR/S	 72.2; 72.4; 18.4
U+65EB.0	fTotalStrokes	 6
U+65EB.0	fCangjie	ASH
U+65EB.0	fFourCorner	67020
U+65EB.0	fCantonese	tiu1
U+65EB.0	fMandarin	tiao1
U+65EC.0	fUTF8	旬
U+65EC.0	fCNS11643	1-4830
U+65EC.0	fBig5	A6AF
U+65EC.0	fGB	0-512E
U+65EC.0	fR/S	 72.2
U+65EC.0	fTotalStrokes	 6
U+65EC.0	fCangjie	PA
U+65EC.0	fFourCorner	27620
U+65EC.0	fHakka	  sun2; sun3
U+65EC.0	fCantonese	ceon4
U+65EC.0	fMandarin	xun2
U+65EC.0	fEnglish	[1] ten-day period [2] period of ten years (usually used to indicate a person's age) [3] widespread; throughout [4] tour; inspect
U+65ED.0	fUTF8	旭
U+65ED.0	fCNS11643	1-4831
U+65ED.0	fBig5	A6B0
U+65ED.0	fGB	0-5071
U+65ED.0	fR/S	 72.2
U+65ED.0	fTotalStrokes	 6
U+65ED.0	fCangjie	KNA
U+65ED.0	fFourCorner	46010
U+65ED.0	fHakka	  hiuk7
U+65ED.0	fCantonese	juk1
U+65ED.0	fMandarin	xu4; hao3; xuan1
U+65ED.0	fEnglish	[1] rising sun [2] brilliance or radiance of daybreak [3] proud; smug; complacent
U+65EE.0	fUTF8	旮
U+65EE.0	fCNS11643	2-224F
U+65EE.0	fBig5	C9EE
U+65EE.0	fGB	0-6A38
U+65EE.0	fR/S	 72.2
U+65EE.0	fTotalStrokes	 6
U+65EE.0	fCangjie	KNA
U+65EE.0	fFourCorner	47602
U+65EE.0	fHakka	 hiuk7
U+65EE.0	fCantonese	go1; kaa3 (also); gok3 (also); juk1
U+65EE.0	fMandarin	xu4; hao3; ga1; xuan1
U+65EE.0	fEnglish	[1] corner [2] same as &#x65ED;
U+65EF.0	fUTF8	旯
U+65EF.0	fCNS11643	2-224E
U+65EF.0	fBig5	C9ED
U+65EF.0	fGB	0-6A39
U+65EF.0	fR/S	 72.2
U+65EF.0	fTotalStrokes	 6
U+65EF.0	fCangjie	AKN
U+65EF.0	fFourCorner	60417
U+65EF.0	fCantonese	lo1; laa1 (also); lok1 (also)
U+65EF.0	fMandarin	la2; xu4
U+65EF.0	fEnglish	 corner
U+65F0.0	fUTF8	旰
U+65F0.0	fCNS11643	2-2439
U+65F0.0	fBig5	CAF8
U+65F0.0	fGB	0-6A3A
U+65F0.0	fR/S	 72.3
U+65F0.0	fTotalStrokes	 7
U+65F0.0	fCangjie	AMJ
U+65F0.0	fFourCorner	61040
U+65F0.0	fHakka	  gon5; hon6; hon5
U+65F0.0	fCantonese	gon3; hon6; hon5
U+65F0.0	fMandarin	gan4; gan1; han4
U+65F0.0	fEnglish	evening; late
U+65F1.0	fUTF8	旱
U+65F1.0	fCNS11643	1-4A54
U+65F1.0	fBig5	A7F2
U+65F1.0	fGB	0-3A35
U+65F1.0	fR/S	 72.3
U+65F1.0	fTotalStrokes	 7
U+65F1.0	fCangjie	AMJ
U+65F1.0	fFourCorner	60401
U+65F1.0	fHakka	  hon1;  hon3; {hon1}; han3
U+65F1.0	fCantonese	hon5
U+65F1.0	fMandarin	han4
U+65F1.0	fEnglish	[1] drought; dry; dry land [2] by land route (as opposed to waterway)
U+65F2.0	fUTF8	旲
U+65F2.0	fCNS11643	2-243C
U+65F2.0	fBig5	CAFB
U+65F2.0	fGB	3-474A
U+65F2.0	fR/S	 72.3
U+65F2.0	fTotalStrokes	 7
U+65F2.0	fCangjie	AK
U+65F2.0	fFourCorner	60430
U+65F2.0	fCantonese	toi4; jing2
U+65F2.0	fMandarin	tai2; ying3
U+65F3.0	fUTF8	旳
U+65F3.0	fCNS11643	2-243B
U+65F3.0	fBig5	CAFA
U+65F3.0	fGB	5-494D
U+65F3.0	fR/S	 72.3
U+65F3.0	fTotalStrokes	 7
U+65F3.0	fCangjie	API
U+65F3.0	fFourCorner	67020
U+65F3.0	fCantonese	dik1
U+65F3.0	fMandarin	di4
U+65F4.0	fUTF8	旴
U+65F4.0	fCNS11643	2-243A
U+65F4.0	fBig5	CAF9
U+65F4.0	fGB	3-4749
U+65F4.0	fR/S	 72.3
U+65F4.0	fTotalStrokes	 7
U+65F4.0	fCangjie	AMD
U+65F4.0	fFourCorner	61040
U+65F4.0	fCantonese	heoi1
U+65F4.0	fMandarin	xu1
U+65F4.0	fEnglish	dawn
U+65F5.0	fUTF8	旵
U+65F5.0	fCNS11643	2-243D
U+65F5.0	fBig5	CAFC
U+65F5.0	fGB	3-474B
U+65F5.0	fR/S	 72.3
U+65F5.0	fTotalStrokes	 7
U+65F5.0	fCangjie	AU
U+65F5.0	fFourCorner	60770
U+65F5.0	fCantonese	caam2; caam3
U+65F5.0	fMandarin	chan3
U+65F6.0	fUTF8	时
U+65F6.0	fCNS11643	3-263D
U+65F6.0	fGB	0-4A31
U+65F6.0	fR/S	 72.3
U+65F6.0	fTotalStrokes	 7
U+65F6.0	fCangjie	ADI
U+65F6.0	fFourCorner	64000
U+65F6.0	fHakka	 shi2
U+65F6.0	fCantonese	si4; si2 (also)
U+65F6.0	fMandarin	shi2
U+65F6.0	fEnglish	[1] non-classical form of &#x6642; [2] simplified form of &#x6642;
U+65F7.0	fUTF8	旷
U+65F7.0	fGB	0-3F75
U+65F7.0	fR/S	 72.3
U+65F7.0	fTotalStrokes	 7
U+65F7.0	fCangjie	AI
U+65F7.0	fHakka	 k'wong3
U+65F7.0	fCantonese	kwong3; kong3 (also)
U+65F7.0	fMandarin	kuang4
U+65F7.0	fEnglish	simplified form of &#x66E0;
U+65F8.0	fUTF8	旸
U+65F8.0	fGB	8-7C39
U+65F8.0	fR/S	 72.3
U+65F8.0	fTotalStrokes	 7
U+65F8.0	fCangjie	ANSH
U+65F8.0	fCantonese	joeng4
U+65F8.0	fMandarin	yang2
U+65F8.0	fEnglish	rising sun; sunshine
U+65F9.0	fUTF8	旹
U+65F9.0	fCNS11643	4-2662
U+65F9.0	fGB	E-2C59
U+65F9.0	fR/S	 72.4
U+65F9.0	fTotalStrokes	 8
U+65F9.0	fCangjie	UMA
U+65F9.0	fFourCorner	22600
U+65F9.0	fCantonese	si4
U+65F9.0	fMandarin	shi2
U+65F9.0	fEnglish	 time; season era; age; period
U+65FA.0	fUTF8	旺
U+65FA.0	fCNS11643	1-4D76
U+65FA.0	fBig5	A9F4
U+65FA.0	fGB	0-4D7A
U+65FA.0	fR/S	 72.4
U+65FA.0	fTotalStrokes	 8
U+65FA.0	fCangjie	AMG
U+65FA.0	fFourCorner	61014
U+65FA.0	fHakka	 wong3;  wong5; wong6; vong5
U+65FA.0	fCantonese	wong6
U+65FA.0	fMandarin	wang4
U+65FA.0	fEnglish	[1] prosper; flourish; prosperous [2] vigorous; prolific; productive [3] brilliant; bright; brightly (said of light, fires, etc.)
U+65FB.0	fUTF8	旻
U+65FB.0	fCNS11643	2-272A
U+65FB.0	fBig5	CCC9
U+65FB.0	fGB	3-4755
U+65FB.0	fR/S	 72.4
U+65FB.0	fTotalStrokes	 8
U+65FB.0	fCangjie	AYK
U+65FB.0	fFourCorner	60400
U+65FB.0	fHakka	 men1; min2; men2; men3
U+65FB.0	fCantonese	man4
U+65FB.0	fMandarin	min2
U+65FB.0	fEnglish	autumn
U+65FC.0	fUTF8	旼
U+65FC.0	fCNS11643	2-2726
U+65FC.0	fBig5	CCC5
U+65FC.0	fGB	3-4754
U+65FC.0	fR/S	 72.4
U+65FC.0	fTotalStrokes	 8
U+65FC.0	fCangjie	AYK
U+65FC.0	fFourCorner	60040
U+65FC.0	fCantonese	man4
U+65FC.0	fMandarin	min2
U+65FC.0	fEnglish	gentle and affable
U+65FD.0	fUTF8	旽
U+65FD.0	fCNS11643	2-272F
U+65FD.0	fBig5	CCCE
U+65FD.0	fGB	5-4953
U+65FD.0	fR/S	 72.4
U+65FD.0	fTotalStrokes	 8
U+65FD.0	fCangjie	APU
U+65FD.0	fFourCorner	65017
U+65FD.0	fCantonese	tan1; zan3
U+65FD.0	fMandarin	tun1; zhun4
U+65FD.0	fEnglish	morning sun; sunrise
U+65FE.0	fUTF8	旾
U+65FE.0	fCNS11643	3-2943
U+65FE.0	fGB	E-2C5A
U+65FE.0	fR/S	 72.4
U+65FE.0	fTotalStrokes	 8
U+65FE.0	fCangjie	PUA
U+65FE.0	fFourCorner	50600
U+65FE.0	fHakka	  cun1; chun1
U+65FE.0	fCantonese	ceon1
U+65FE.0	fMandarin	chun1; chun3
U+65FE.0	fEnglish	ancient form of &#x6625;
U+65FF.0	fUTF8	旿
U+65FF.0	fCNS11643	3-293E
U+65FF.0	fGB	3-474F
U+65FF.0	fR/S	 72.4
U+65FF.0	fTotalStrokes	 8
U+65FF.0	fCangjie	AOJ
U+65FF.0	fFourCorner	68040
U+65FF.0	fHakka	 ng3
U+65FF.0	fCantonese	ng5
U+65FF.0	fMandarin	wu3; wu4
U+65FF.0	fEnglish	noontide
U+6600.0	fUTF8	昀
U+6600.0	fCNS11643	1-4D7D
U+6600.0	fBig5	A9FB
U+6600.0	fGB	0-6A40
U+6600.0	fR/S	 72.4
U+6600.0	fTotalStrokes	 8
U+6600.0	fCangjie	APIM
U+6600.0	fFourCorner	67020
U+6600.0	fHakka	 jun2
U+6600.0	fCantonese	wan4
U+6600.0	fMandarin	yun2
U+6600.0	fEnglish	[1] daybreak; sunrise; dawn [2] sunshine
U+6601.0	fUTF8	昁
U+6601.0	fCNS11643	3-2942
U+6601.0	fGB	E-2C5B
U+6601.0	fR/S	  72.4
U+6601.0	fTotalStrokes	 8
U+6601.0	fCangjie	AJB
U+6601.0	fFourCorner	65027
U+6601.0	fCantonese	bui3
U+6601.0	fMandarin	bei4
U+6602.0	fUTF8	昂
U+6602.0	fCNS11643	1-4D7B
U+6602.0	fBig5	A9F9
U+6602.0	fGB	0-303A
U+6602.0	fR/S	 72.4
U+6602.0	fTotalStrokes	 8
U+6602.0	fCangjie	AHVL
U+6602.0	fFourCorner	60127
U+6602.0	fHakka	 ngong1; ngong2; {ngong2}; ngo5; ngong5
U+6602.0	fCantonese	ngong4; ngong5 (also); ong4 (also); ong5 (also)
U+6602.0	fMandarin	ang2; yang4
U+6602.0	fEnglish	[1] rise; raise; rise higher and higher [2] lofty; proud; bold and not easily bent; straightforward [3] high [4] expensive; costly
U+6603.0	fUTF8	昃
U+6603.0	fCNS11643	2-272B
U+6603.0	fBig5	CCCA
U+6603.0	fGB	0-6A3E
U+6603.0	fR/S	 72.4
U+6603.0	fTotalStrokes	 8
U+6603.0	fCangjie	AMO
U+6603.0	fFourCorner	60281
U+6603.0	fHakka	 zet7
U+6603.0	fCantonese	zak1
U+6603.0	fMandarin	ze4
U+6603.0	fEnglish	after noon; afternoon; the sun on the western side of the sky
U+6604.0	fUTF8	昄
U+6604.0	fCNS11643	2-2727
U+6604.0	fBig5	CCC6
U+6604.0	fGB	3-4751
U+6604.0	fR/S	 72.4
U+6604.0	fTotalStrokes	 8
U+6604.0	fCangjie	AHE
U+6604.0	fFourCorner	61047
U+6604.0	fHakka	  ban3
U+6604.0	fCantonese	baan2; baan6
U+6604.0	fMandarin	ban3; pan1; pan4; pian4
U+6604.0	fEnglish	big
U+6605.0	fUTF8	昅
U+6605.0	fCNS11643	2-272E
U+6605.0	fBig5	CCCD
U+6605.0	fGB	E-2C5C
U+6605.0	fR/S	 72.4
U+6605.0	fTotalStrokes	 8
U+6605.0	fCangjie	ANHE
U+6605.0	fFourCorner	67047
U+6605.0	fCantonese	gip6; gip3
U+6605.0	fMandarin	jie2; ji2
U+6606.0	fUTF8	昆
U+6606.0	fCNS11643	1-4D7A
U+6606.0	fBig5	A9F8
U+6606.0	fGB	0-4025
U+6606.0	fR/S	 72.4
U+6606.0	fTotalStrokes	 8
U+6606.0	fCangjie	APP
U+6606.0	fFourCorner	60711
U+6606.0	fHakka	  kun1; kwun1
U+6606.0	fCantonese	gwan1 (also); kwan1
U+6606.0	fMandarin	kun1; hun4; hun2
U+6606.0	fEnglish	[1] elder brother [2] descendants; posterity [3] multitudes [4] together; in unison
U+6607.0	fUTF8	昇
U+6607.0	fCNS11643	1-4E23
U+6607.0	fBig5	AA40
U+6607.0	fGB	8-2D70
U+6607.0	fR/S	 72.4
U+6607.0	fTotalStrokes	 8
U+6607.0	fCangjie	AHT
U+6607.0	fFourCorner	60440
U+6607.0	fHakka	  shin1; siin1; sin1
U+6607.0	fCantonese	sing1
U+6607.0	fMandarin	sheng1
U+6607.0	fEnglish	[1] rise; ascent [2] peaceful; peace [3] a Chinese family name
U+6608.0	fUTF8	昈
U+6608.0	fCNS11643	2-2729
U+6608.0	fBig5	CCC8
U+6608.0	fGB	3-4757
U+6608.0	fR/S	 72.4
U+6608.0	fTotalStrokes	 8
U+6608.0	fCangjie	AHS
U+6608.0	fFourCorner	63007
U+6608.0	fCantonese	wu6
U+6608.0	fMandarin	hu4
U+6609.0	fUTF8	昉
U+6609.0	fCNS11643	2-2725
U+6609.0	fBig5	CCC4
U+6609.0	fGB	3-4756
U+6609.0	fR/S	 72.4
U+6609.0	fTotalStrokes	 8
U+6609.0	fCangjie	AYHS
U+6609.0	fFourCorner	60027
U+6609.0	fHakka	 fong3
U+6609.0	fCantonese	fong2
U+6609.0	fMandarin	fang3
U+6609.0	fEnglish	[1] dawn; daybreak [2] beginning
U+660A.0	fUTF8	昊
U+660A.0	fCNS11643	1-4E22
U+660A.0	fBig5	A9FE
U+660A.0	fGB	0-6A3B
U+660A.0	fR/S	 72.4
U+660A.0	fTotalStrokes	 8
U+660A.0	fCangjie	AMK
U+660A.0	fFourCorner	60430
U+660A.0	fHakka	 hau1;  hau3; hau6
U+660A.0	fCantonese	hou6
U+660A.0	fMandarin	hao4
U+660A.0	fEnglish	[1] summer time [2] sky; heavens
U+660B.0	fUTF8	昋
U+660B.0	fCNS11643	2-272C
U+660B.0	fBig5	CCCB
U+660B.0	fGB	3-474D
U+660B.0	fR/S	 72.4
U+660B.0	fTotalStrokes	 8
U+660B.0	fCangjie	HKA
U+660B.0	fFourCorner	20603
U+660B.0	fHakka	 gui3; kwui5
U+660B.0	fCantonese	gwai3
U+660B.0	fMandarin	gui4; jing3
U+660B.0	fEnglish	[1] look [2] a Chinese family name
U+660C.0	fUTF8	昌
U+660C.0	fCNS11643	1-4D79
U+660C.0	fBig5	A9F7
U+660C.0	fGB	0-327D
U+660C.0	fR/S	 72.4
U+660C.0	fTotalStrokes	 8
U+660C.0	fCangjie	AA
U+660C.0	fFourCorner	60600
U+660C.0	fHakka	  cong1;  chong1
U+660C.0	fCantonese	coeng1
U+660C.0	fMandarin	chang1; chang4
U+660C.0	fEnglish	[1] proper; good; straight (talk) [2] prosperous; robust; vigorous; make prosperous; glorify [3] light; brightness [4] a Chinese family name
U+660D.0	fUTF8	昍
U+660D.0	fCNS11643	2-272D
U+660D.0	fBig5	CCCC
U+660D.0	fGB	5-4954
U+660D.0	fR/S	 72.4
U+660D.0	fTotalStrokes	 8
U+660D.0	fCangjie	AA
U+660D.0	fFourCorner	66000
U+660D.0	fCantonese	hyun1
U+660D.0	fMandarin	xuan1
U+660E.0	fUTF8	明
U+660E.0	fCNS11643	1-4D7C
U+660E.0	fBig5	A9FA
U+660E.0	fGB	0-4377
U+660E.0	fR/S	 72.4
U+660E.0	fTotalStrokes	 8
U+660E.0	fCangjie	AB
U+660E.0	fFourCorner	67020
U+660E.0	fHakka	  min2; miang2; mang2; miang3; min3
U+660E.0	fCantonese	ming4; ming2 (also)
U+660E.0	fMandarin	ming2
U+660E.0	fEnglish	[1] bright; light; brilliant [2] clear; understandable; clarify; understand; obvious; evident [3] intelligent; clever [4] eyesight; seeing faculty [5] day; daybreak; dawn [6] state; show; assert [7] next (day or year) [8] the Ming dynasty (1368-1644 A.D.) [9] a Chinese family name
U+660E.1	fUTF8	明
U+660E.1	fCNS11643	1-4D7C
U+660E.1	fBig5	A9FA
U+660E.1	fGB	0-4377
U+660E.1	fR/S	 72.4
U+660E.1	fTotalStrokes	 8
U+660E.1	fCangjie	AB
U+660E.1	fFourCorner	67020
U+660E.1	fMandarin	ming5
U+660F.0	fUTF8	昏
U+660F.0	fCNS11643	1-4D7E
U+660F.0	fBig5	A9FC
U+660F.0	fGB	0-3B68
U+660F.0	fR/S	 72.4
U+660F.0	fTotalStrokes	 8
U+660F.0	fCangjie	HPA
U+660F.0	fFourCorner	72604
U+660F.0	fHakka	  fun1
U+660F.0	fCantonese	fan1
U+660F.0	fMandarin	hun1; hun4; min3
U+660F.0	fEnglish	[1] dusk; dark; nightfall; twilight [2] confused; muddled; mixed-up; demented [3] unclear of sight; dizzy [4] same as ?; marry [5] a Chinese family name
U+660F.1	fUTF8	昏
U+660F.1	fCNS11643	1-4D7E
U+660F.1	fBig5	A9FC
U+660F.1	fGB	0-3B68
U+660F.1	fR/S	 72.4
U+660F.1	fTotalStrokes	 8
U+660F.1	fCangjie	HPA
U+660F.1	fFourCorner	72604
U+660F.1	fMandarin	hun2
U+6610.0	fUTF8	昐
U+6610.0	fCNS11643	2-2731
U+6610.0	fBig5	CCD0
U+6610.0	fGB	5-4956
U+6610.0	fR/S	 72.4
U+6610.0	fTotalStrokes	 8
U+6610.0	fCangjie	ACSH
U+6610.0	fFourCorner	68027
U+6610.0	fHakka	  fun1
U+6610.0	fCantonese	fan1
U+6610.0	fMandarin	fen1
U+6610.0	fEnglish	the light of the sun issuing forth
U+6611.0	fUTF8	昑
U+6611.0	fCNS11643	2-2730
U+6611.0	fBig5	CCCF
U+6611.0	fGB	5-4955
U+6611.0	fR/S	 72.4
U+6611.0	fTotalStrokes	 8
U+6611.0	fCangjie	AOIN
U+6611.0	fFourCorner	68027
U+6611.0	fCantonese	jam2
U+6611.0	fMandarin	qin3; qiu3
U+6611.0	fEnglish	bright
U+6612.0	fUTF8	昒
U+6612.0	fCNS11643	2-2728
U+6612.0	fBig5	CCC7
U+6612.0	fGB	3-4752
U+6612.0	fR/S	 72.4
U+6612.0	fTotalStrokes	 8
U+6612.0	fCangjie	APHH
U+6612.0	fFourCorner	67020
U+6612.0	fHakka	 fut7
U+6612.0	fCantonese	fat1; mat6 (also)
U+6612.0	fMandarin	hu1; wu4
U+6613.0	fUTF8	易
U+6613.0	fCNS11643	1-4D78
U+6613.0	fBig5	A9F6
U+6613.0	fGB	0-5257
U+6613.0	fR/S	 72.4
U+6613.0	fTotalStrokes	 8
U+6613.0	fCangjie	APHH
U+6613.0	fFourCorner	60227
U+6613.0	fHakka	  jit8; ji5; ji3; ri5; rit8; i5
U+6613.0	fCantonese	ji6; jik6
U+6613.0	fMandarin	yi4
U+6613.0	fEnglish	[1] exchange; barter [2] change (places, jobs, owners, etc.) [3] easy [4] amiable; lenient [5] the Book of Changes [6] a Chinese family name
U+6614.0	fUTF8	昔
U+6614.0	fCNS11643	1-4D77
U+6614.0	fBig5	A9F5
U+6614.0	fGB	0-4E74
U+6614.0	fR/S	 72.4
U+6614.0	fTotalStrokes	 8
U+6614.0	fCangjie	TA
U+6614.0	fFourCorner	44601
U+6614.0	fHakka	  sit7; sit8
U+6614.0	fCantonese	sik1
U+6614.0	fMandarin	xi1; xi2; cuo4; cuo2
U+6614.0	fEnglish	[1] formerly; ancient; of old; bygone [2] end [3] night; evening
U+6615.0	fUTF8	昕
U+6615.0	fCNS11643	1-4E21
U+6615.0	fBig5	A9FD
U+6615.0	fGB	0-6A3F
U+6615.0	fR/S	 72.4
U+6615.0	fTotalStrokes	 8
U+6615.0	fCangjie	AHML
U+6615.0	fFourCorner	62021
U+6615.0	fHakka	 hiun1
U+6615.0	fCantonese	jan1
U+6615.0	fMandarin	xin1; xi1
U+6615.0	fEnglish	dawn; early morning; daybreak
U+6616.0	fUTF8	昖
U+6616.0	fCNS11643	4-2661
U+6616.0	fGB	5-4957
U+6616.0	fR/S	 72.4
U+6616.0	fTotalStrokes	 8
U+6616.0	fCangjie	ACI
U+6616.0	fFourCorner	68030
U+6616.0	fCantonese	jyun4
U+6616.0	fMandarin	yan2
U+6617.0	fUTF8	昗
U+6617.0	fCNS11643	3-2940
U+6617.0	fGB	5-4958
U+6617.0	fR/S	  72.4
U+6617.0	fTotalStrokes	 8
U+6617.0	fCangjie	AYC
U+6617.0	fFourCorner	60800
U+6617.0	fHakka	 zet7
U+6617.0	fCantonese	zak1
U+6617.0	fMandarin	ze4
U+6617.0	fEnglish	same as &#x6603;
U+6618.0	fUTF8	昘
U+6618.0	fCNS11643	3-2941
U+6618.0	fGB	E-2C5D
U+6618.0	fR/S	 72.4
U+6618.0	fTotalStrokes	 8
U+6618.0	fCangjie	AYHS
U+6618.0	fFourCorner	60220
U+6618.0	fHakka	  fong3; fong5
U+6618.0	fCantonese	fong2
U+6618.0	fMandarin	fang3; fang4
U+6618.0	fEnglish	same as &#x8A2A;
U+6619.0	fUTF8	昙
U+6619.0	fCNS11643	F-273E
U+6619.0	fGB	0-6A3C
U+6619.0	fR/S	 72.4
U+6619.0	fTotalStrokes	 8
U+6619.0	fCangjie	AMMI
U+6619.0	fFourCorner	6073
U+6619.0	fHakka	 tam2
U+6619.0	fCantonese	taam4
U+6619.0	fMandarin	tan2
U+6619.0	fEnglish	simplified form of &#x66C7;
U+661A.0	fUTF8	昚
U+661A.0	fCNS11643	3-2C7E
U+661A.0	fGB	E-2C5E
U+661A.0	fR/S	 72.5
U+661A.0	fTotalStrokes	 9
U+661A.0	fCangjie	KCA
U+661A.0	fFourCorner	40600
U+661A.0	fHakka	 shim3; sim3; siim3
U+661A.0	fCantonese	san6
U+661A.0	fMandarin	shen4
U+661A.0	fEnglish	ancient form of &#x613C;
U+661B.0	fUTF8	昛
U+661B.0	fCNS11643	3-2C7A
U+661B.0	fGB	E-2C5F
U+661B.0	fR/S	 72.5
U+661B.0	fTotalStrokes	 9
U+661B.0	fCangjie	ASS
U+661B.0	fFourCorner	61010
U+661B.0	fCantonese	geoi6
U+661B.0	fMandarin	ju4
U+661C.0	fUTF8	昜
U+661C.0	fCNS11643	2-2A70
U+661C.0	fBig5	CEEF
U+661C.0	fGB	5-495D
U+661C.0	fR/S	 72.5
U+661C.0	fTotalStrokes	 9
U+661C.0	fCangjie	AMSH
U+661C.0	fFourCorner	60227
U+661C.0	fHakka	 jong2; rong2
U+661C.0	fCantonese	joeng4
U+661C.0	fMandarin	yang2
U+661C.0	fEnglish	bright; glorious
U+661D.0	fUTF8	昝
U+661D.0	fCNS11643	2-2A76
U+661D.0	fBig5	CEF5
U+661D.0	fGB	0-6A43
U+661D.0	fR/S	 72.5
U+661D.0	fTotalStrokes	 9
U+661D.0	fCangjie	HOA
U+661D.0	fFourCorner	28604
U+661D.0	fHakka	 zam3
U+661D.0	fCantonese	zaan2; zaam2
U+661D.0	fMandarin	zan3; can3
U+661D.0	fEnglish	a Chinese family name
U+661E.0	fUTF8	昞
U+661E.0	fCNS11643	3-2D23
U+661E.0	fGB	5-495B
U+661E.0	fR/S	 72.5
U+661E.0	fTotalStrokes	 9
U+661E.0	fCangjie	AMOB
U+661E.0	fFourCorner	41020
U+661E.0	fHakka	 biang3
U+661E.0	fCantonese	bing2; fong2
U+661E.0	fMandarin	bing3
U+661E.0	fEnglish	bright; luminous; glorious; radiant; brilliant 
U+661F.0	fUTF8	星
U+661F.0	fCNS11643	1-5153
U+661F.0	fBig5	AC50
U+661F.0	fGB	0-5047
U+661F.0	fR/S	 72.5
U+661F.0	fTotalStrokes	 9
U+661F.0	fCangjie	AHQM
U+661F.0	fFourCorner	60104
U+661F.0	fHakka	  sin1;  sen1; siang1; sang1
U+661F.0	fCantonese	sing1; seng1
U+661F.0	fMandarin	xing1
U+661F.0	fEnglish	[1] any heavenly body that shines; star; planet; satellite; any point of light at the sky [2] spark; sparks [3] droplets; small particles of anything; very tiny [4] name of one of the 28 constellations [5] movie star [6] by night; nocturnal [7] an ancient percussion instrument [8] a Chinese family name
U+6620.0	fUTF8	映
U+6620.0	fCNS11643	1-5150
U+6620.0	fBig5	AC4D
U+6620.0	fGB	0-5333
U+6620.0	fR/S	 72.5
U+6620.0	fTotalStrokes	 9
U+6620.0	fCangjie	ALBK
U+6620.0	fFourCorner	64034
U+6620.0	fHakka	 jang3; jang5; rang3; jong5; rang6; rong3; jong3
U+6620.0	fCantonese	jing2; joeng2
U+6620.0	fMandarin	ying4; ang3
U+6620.0	fEnglish	[1] mirror; reflect [2] project (slides, pictures, etc.) [3] shine; shining; blind; blinding (glare, light, etc.)
U+6621.0	fUTF8	昡
U+6621.0	fCNS11643	2-2A6D
U+6621.0	fBig5	CEEC
U+6621.0	fGB	3-475F
U+6621.0	fR/S	 72.5
U+6621.0	fTotalStrokes	 9
U+6621.0	fCangjie	AYVI
U+6621.0	fFourCorner	60032
U+6621.0	fHakka	 hien2; hien5
U+6621.0	fCantonese	jyun6; jyun4
U+6621.0	fMandarin	xuan4
U+6621.0	fEnglish	 long day extended relaxed
U+6622.0	fUTF8	昢
U+6622.0	fCNS11643	2-2A72
U+6622.0	fBig5	CEF1
U+6622.0	fGB	3-4761
U+6622.0	fR/S	 72.5
U+6622.0	fTotalStrokes	 9
U+6622.0	fCangjie	AUU
U+6622.0	fFourCorner	62077
U+6622.0	fHakka	 put8
U+6622.0	fCantonese	pui3; put6 (also); but6
U+6622.0	fMandarin	pei3; pei4
U+6622.0	fEnglish	the sun rising
U+6623.0	fUTF8	昣
U+6623.0	fCNS11643	3-2C7C
U+6623.0	fGB	E-2C60
U+6623.0	fR/S	  72.5
U+6623.0	fTotalStrokes	 9
U+6623.0	fCangjie	AOHH
U+6623.0	fFourCorner	68020
U+6623.0	fCantonese	can2
U+6623.0	fMandarin	zhen3
U+6624.0	fUTF8	昤
U+6624.0	fCNS11643	1-5156
U+6624.0	fBig5	AC53
U+6624.0	fGB	5-495F
U+6624.0	fR/S	 72.5
U+6624.0	fTotalStrokes	 9
U+6624.0	fCangjie	AOII
U+6624.0	fFourCorner	68037
U+6624.0	fCantonese	ling4
U+6624.0	fMandarin	ling1; ling2
U+6624.0	fEnglish	sunshine
U+6625.0	fUTF8	春
U+6625.0	fCNS11643	1-514E
U+6625.0	fBig5	AC4B
U+6625.0	fGB	0-343A
U+6625.0	fR/S	 72.5
U+6625.0	fTotalStrokes	 9
U+6625.0	fCangjie	QKA
U+6625.0	fFourCorner	50603
U+6625.0	fHakka	  cun1; chun1
U+6625.0	fCantonese	ceon1
U+6625.0	fMandarin	chun1; chun3
U+6625.0	fEnglish	[1] spring; first of the four seasons [2] sensuality; lustful; lewd; pornographic [3] alive; vitality; living [4] joyful [5] wine (especially in the Tang dynasty)
U+6626.0	fUTF8	昦
U+6626.0	fCNS11643	2-2A71
U+6626.0	fBig5	CEF0
U+6626.0	fGB	5-495C
U+6626.0	fR/S	 72.5
U+6626.0	fTotalStrokes	 9
U+6626.0	fCangjie	AKLL
U+6626.0	fFourCorner	60223
U+6626.0	fHakka	 hau1;  hau3; hau6
U+6626.0	fCantonese	hou6
U+6626.0	fMandarin	hao4
U+6626.0	fEnglish	standard form of &#x660A;
U+6627.0	fUTF8	昧
U+6627.0	fCNS11643	1-5151
U+6627.0	fBig5	AC4E
U+6627.0	fGB	0-4341
U+6627.0	fR/S	 72.5
U+6627.0	fTotalStrokes	 9
U+6627.0	fCangjie	AJD
U+6627.0	fFourCorner	65090
U+6627.0	fHakka	 mui3; mi3; mui6; mi5
U+6627.0	fCantonese	mui6
U+6627.0	fMandarin	mei4; mi4
U+6627.0	fEnglish	[1] obscure; dark; darken [2] hide; pocket [3] ignore (one's conscience, etc.) [4] blind; ignorant [5] faint; fainting
U+6628.0	fUTF8	昨
U+6628.0	fCNS11643	1-5154
U+6628.0	fBig5	AC51
U+6628.0	fGB	0-5772
U+6628.0	fR/S	 72.5
U+6628.0	fTotalStrokes	 9
U+6628.0	fCangjie	AOS
U+6628.0	fFourCorner	68011
U+6628.0	fHakka	  cok7; co1; ca1; zok7; cam5
U+6628.0	fCantonese	zok3 (also); zok6
U+6628.0	fMandarin	zuo2
U+6628.0	fEnglish	yesterday; in former times; past; lately
U+6629.0	fUTF8	昩
U+6629.0	fCNS11643	3-2C79
U+6629.0	fGB	3-4758
U+6629.0	fR/S	  72.5
U+6629.0	fTotalStrokes	 9
U+6629.0	fCangjie	ADJ
U+6629.0	fFourCorner	65090
U+6629.0	fCantonese	mut6
U+6629.0	fMandarin	mo4
U+662A.0	fUTF8	昪
U+662A.0	fCNS11643	4-2967
U+662A.0	fGB	3-4762
U+662A.0	fR/S	  72.5
U+662A.0	fTotalStrokes	 9
U+662A.0	fCangjie	AIT
U+662A.0	fFourCorner	60440
U+662A.0	fHakka	 pien5; pien6; pien3
U+662A.0	fCantonese	bin6
U+662A.0	fMandarin	bian4
U+662A.0	fEnglish	[1] happy; joy; pleasing [2] sunshine [3] light; brightness; bright
U+662B.0	fUTF8	昫
U+662B.0	fCNS11643	2-2A74
U+662B.0	fBig5	CEF3
U+662B.0	fGB	3-475E
U+662B.0	fR/S	 72.5
U+662B.0	fTotalStrokes	 9
U+662B.0	fCangjie	APR
U+662B.0	fFourCorner	67020
U+662B.0	fHakka	  hi1;  gi1
U+662B.0	fCantonese	heoi2; heoi3 (also)
U+662B.0	fMandarin	xu4; xu3; xu1; xiong3
U+662B.0	fEnglish	warmth of the rising sun
U+662C.0	fUTF8	昬
U+662C.0	fCNS11643	3-2C7D
U+662C.0	fGB	E-2C61
U+662C.0	fR/S	  72.5
U+662C.0	fTotalStrokes	 9
U+662C.0	fCangjie	RPA
U+662C.0	fFourCorner	77600
U+662C.0	fHakka	 fun1
U+662C.0	fCantonese	fan1
U+662C.0	fMandarin	hun1; min3
U+662C.0	fEnglish	 dusk; dark confused stupid
U+662D.0	fUTF8	昭
U+662D.0	fCNS11643	1-514F
U+662D.0	fBig5	AC4C
U+662D.0	fGB	0-5551
U+662D.0	fR/S	 72.5
U+662D.0	fTotalStrokes	 9
U+662D.0	fCangjie	ASHR
U+662D.0	fFourCorner	67062
U+662D.0	fHakka	  zhau1;  zau1; zeu1; shau1
U+662D.0	fCantonese	ciu1; ziu1 (also)
U+662D.0	fMandarin	zhao1; zhao4
U+662D.0	fEnglish	[1] bright; brightness; luminous; illustrious [2] prominent; eminent; evident; obvious [3] make open; show; display [4] a Chinese family name
U+662E.0	fUTF8	昮
U+662E.0	fCNS11643	2-2A79
U+662E.0	fBig5	CEF8
U+662E.0	fGB	5-495A
U+662E.0	fR/S	 72.5
U+662E.0	fTotalStrokes	 9
U+662E.0	fCangjie	AMKS
U+662E.0	fFourCorner	60127
U+662E.0	fCantonese	zung6; cung6
U+662E.0	fMandarin	zong4
U+662F.0	fUTF8	是
U+662F.0	fCNS11643	1-5152
U+662F.0	fBig5	AC4F
U+662F.0	fGB	0-4A47
U+662F.0	fR/S	 72.5
U+662F.0	fTotalStrokes	 9
U+662F.0	fCangjie	AMYO
U+662F.0	fFourCorner	60801
U+662F.0	fHakka	 shi6; si3; si5; sii5; she5; shi5
U+662F.0	fCantonese	si6; si2 (also)
U+662F.0	fMandarin	shi4; ti2
U+662F.0	fEnglish	[1] indeed; yes; right; positive [2] be [3] this; that; which [4] a Chinese family name
U+662F.1	fUTF8	是
U+662F.1	fCNS11643	1-5152
U+662F.1	fBig5	AC4F
U+662F.1	fGB	0-4A47
U+662F.1	fR/S	 72.5
U+662F.1	fTotalStrokes	 9
U+662F.1	fCangjie	AMYO
U+662F.1	fFourCorner	60801
U+662F.1	fMandarin	shi5
U+6630.0	fUTF8	昰
U+6630.0	fCNS11643	3-2D21
U+6630.0	fGB	E-2C62
U+6630.0	fR/S	 72.5
U+6630.0	fTotalStrokes	 9
U+6630.0	fCangjie	AMYM
U+6630.0	fFourCorner	60100
U+6630.0	fCantonese	haa6; si6
U+6630.0	fMandarin	shi4; xia4
U+6630.0	fEnglish	 variant of "to be"; "summer" name
U+6631.0	fUTF8	昱
U+6631.0	fCNS11643	1-5155
U+6631.0	fBig5	AC52
U+6631.0	fGB	0-6A45
U+6631.0	fR/S	 72.5
U+6631.0	fTotalStrokes	 9
U+6631.0	fCangjie	AYT
U+6631.0	fFourCorner	60108
U+6631.0	fHakka	  juk7; ruk7
U+6631.0	fCantonese	juk1
U+6631.0	fMandarin	yu4; yi4
U+6631.0	fEnglish	[1] sunshine; bright light; sunlight; dazzling; shining; brightness [2] tomorrow
U+6632.0	fUTF8	昲
U+6632.0	fCNS11643	2-2A6E
U+6632.0	fBig5	CEED
U+6632.0	fGB	3-4760
U+6632.0	fR/S	 72.5
U+6632.0	fTotalStrokes	 9
U+6632.0	fCangjie	ALLN
U+6632.0	fFourCorner	65027
U+6632.0	fCantonese	fai3; fai1
U+6632.0	fMandarin	fei4; fu2; fei1; fen4; po1
U+6633.0	fUTF8	昳
U+6633.0	fCNS11643	2-2A73
U+6633.0	fBig5	CEF2
U+6633.0	fGB	3-475C
U+6633.0	fR/S	 72.5
U+6633.0	fTotalStrokes	 9
U+6633.0	fCangjie	AHQO
U+6633.0	fFourCorner	65030
U+6633.0	fHakka	  diet7; zhit7; tsiit7
U+6633.0	fCantonese	dit6; jat6
U+6633.0	fMandarin	die2
U+6633.0	fEnglish	 declining sun in the west; setting sun
U+6633.1	fUTF8	昳
U+6633.1	fCNS11643	2-2A73
U+6633.1	fBig5	CEF2
U+6633.1	fGB	3-475C
U+6633.1	fR/S	 72.5
U+6633.1	fTotalStrokes	 9
U+6633.1	fCangjie	AHQO
U+6633.1	fFourCorner	65030
U+6633.1	fMandarin	yi4
U+6633.1	fEnglish	bright
U+6634.0	fUTF8	昴
U+6634.0	fCNS11643	2-2A77
U+6634.0	fBig5	CEF6
U+6634.0	fGB	0-6A44
U+6634.0	fR/S	 72.5
U+6634.0	fTotalStrokes	 9
U+6634.0	fCangjie	AHHL
U+6634.0	fFourCorner	60727
U+6634.0	fHakka	  mau1
U+6634.0	fCantonese	maau5; mau5 (also)
U+6634.0	fMandarin	mao3; mao1
U+6634.0	fEnglish	one of the 28 constellations
U+6635.0	fUTF8	昵
U+6635.0	fCNS11643	2-2A6F
U+6635.0	fBig5	CEEE
U+6635.0	fGB	0-6A47
U+6635.0	fR/S	 72.5
U+6635.0	fTotalStrokes	 9
U+6635.0	fCangjie	ASP
U+6635.0	fFourCorner	67011
U+6635.0	fHakka	 nit8;  ni3; lit7; lit8; li3
U+6635.0	fCantonese	nik1; lik1 (also); nik6
U+6635.0	fMandarin	ni4; ni3; zhi4; ri4; zhi2
U+6635.0	fEnglish	 intimate; close approach
U+6636.0	fUTF8	昶
U+6636.0	fCNS11643	2-2A6C
U+6636.0	fBig5	CEEB
U+6636.0	fGB	0-6A46
U+6636.0	fR/S	 72.5
U+6636.0	fTotalStrokes	 9
U+6636.0	fCangjie	IEA
U+6636.0	fFourCorner	36230
U+6636.0	fHakka	 cong3;  chong3
U+6636.0	fCantonese	cong2; coeng3
U+6636.0	fMandarin	chang3; chang4
U+6636.0	fEnglish	[1] long day [2] comfortable and easy
U+6637.0	fUTF8	昷
U+6637.0	fCNS11643	3-2D24
U+6637.0	fGB	5-4A3A
U+6637.0	fR/S	 72.5
U+6637.0	fTotalStrokes	 9
U+6637.0	fCangjie	ABT
U+6637.0	fFourCorner	60100
U+6637.0	fHakka	 vun1
U+6637.0	fCantonese	wan1
U+6637.0	fMandarin	wen1; yun4
U+6637.0	fEnglish	same as &#x25055;
U+6638.0	fUTF8	昸
U+6638.0	fCNS11643	3-2D25
U+6638.0	fGB	E-2C63
U+6638.0	fR/S	 72.5
U+6638.0	fTotalStrokes	 9
U+6638.0	fCangjie	AHEY
U+6638.0	fFourCorner	67030
U+6638.0	fHakka	  dung1
U+6638.0	fCantonese	dung1
U+6638.0	fMandarin	dong1
U+6638.0	fEnglish	same as &#x51AC;
U+6639.0	fUTF8	昹
U+6639.0	fCNS11643	2-2A78
U+6639.0	fBig5	CEF7
U+6639.0	fGB	5-4960
U+6639.0	fR/S	 72.5
U+6639.0	fTotalStrokes	 9
U+6639.0	fCangjie	AINE
U+6639.0	fFourCorner	63032
U+6639.0	fCantonese	oi2; ngoi2 (also)
U+6639.0	fMandarin	ai3
U+663A.0	fUTF8	昺
U+663A.0	fCNS11643	2-2A75
U+663A.0	fBig5	CEF4
U+663A.0	fGB	3-4759
U+663A.0	fR/S	 72.5
U+663A.0	fTotalStrokes	 9
U+663A.0	fCangjie	AMOB
U+663A.0	fFourCorner	60227
U+663A.0	fHakka	  biang3
U+663A.0	fCantonese	bing2
U+663A.0	fMandarin	bing3
U+663A.0	fEnglish	same as &#x661E;
U+663B.0	fUTF8	昻
U+663B.0	fCNS11643	3-2D22
U+663B.0	fGB	E-2C64
U+663B.0	fR/S	 72.5
U+663B.0	fTotalStrokes	 9
U+663B.0	fCangjie	AMSL
U+663B.0	fFourCorner	60120
U+663B.0	fHakka	 ngong1
U+663B.0	fCantonese	ngong4; ong4 (also)
U+663B.0	fMandarin	ang2; yang4
U+663B.0	fEnglish	same as &#x6602;
U+663C.0	fUTF8	昼
U+663C.0	fCNS11643	3-2C37
U+663C.0	fGB	0-5667
U+663C.0	fR/S	 44.6; 72.5
U+663C.0	fTotalStrokes	 9
U+663C.0	fCangjie	SOAM
U+663C.0	fFourCorner	77100
U+663C.0	fHakka	 zhiu5
U+663C.0	fCantonese	zau3
U+663C.0	fMandarin	zhou4
U+663C.0	fEnglish	[1] non-classical form of &#x665D; [2] simplified form of &#x665D;
U+663D.0	fUTF8	昽
U+663D.0	fGB	8-7C3C
U+663D.0	fR/S	 72.5; 72.16
U+663D.0	fTotalStrokes	 9
U+663D.0	fCangjie	AIKP
U+663D.0	fCantonese	lung4
U+663D.0	fMandarin	long2
U+663D.0	fEnglish	vague, dim; twilight
U+663E.0	fUTF8	显
U+663E.0	fGB	0-4F54
U+663E.0	fR/S	 72.5
U+663E.0	fTotalStrokes	 9
U+663E.0	fCangjie	ATC
U+663E.0	fHakka	 hien3
U+663E.0	fCantonese	hin2
U+663E.0	fMandarin	xian3; xian4
U+663E.0	fEnglish	simplified form of &#x986F;
U+663F.0	fUTF8	昿
U+663F.0	fGB	E-2C65
U+663F.0	fR/S	 72.5
U+663F.0	fTotalStrokes	 9
U+663F.0	fCangjie	AII
U+663F.0	fCantonese	kwong3; kong3 (also)
U+663F.0	fMandarin	kuang4
U+663F.0	fEnglish	 extensive; wide; broad empty
U+6640.0	fUTF8	晀
U+6640.0	fCNS11643	3-3146
U+6640.0	fGB	E-2C66
U+6640.0	fR/S	  72.6
U+6640.0	fTotalStrokes	 10
U+6640.0	fCangjie	XALMO
U+6640.0	fFourCorner	62010
U+6640.0	fCantonese	tiu2
U+6640.0	fMandarin	tiao3
U+6641.0	fUTF8	晁
U+6641.0	fCNS11643	1-5572
U+6641.0	fBig5	AED0
U+6641.0	fGB	0-6A4B
U+6641.0	fR/S	 72.6
U+6641.0	fTotalStrokes	 10
U+6641.0	fCangjie	ALMO
U+6641.0	fFourCorner	60113
U+6641.0	fHakka	 zhau1; chau2; cau3; zau1; cau2; cheu1; ch'eu2
U+6641.0	fCantonese	ciu4; siu6; ziu1
U+6641.0	fMandarin	chao2; zhao1; zhu1
U+6641.0	fEnglish	[1] ancient form of &#x671D; [2] a Chinese family name
U+6642.0	fUTF8	時
U+6642.0	fCNS11643	1-556B
U+6642.0	fBig5	AEC9
U+6642.0	fGB	1-4A31
U+6642.0	fR/S	 72.6
U+6642.0	fTotalStrokes	 10
U+6642.0	fCangjie	AGDI
U+6642.0	fFourCorner	64041
U+6642.0	fHakka	 si2;  shi2; sii2; shi3; she2
U+6642.0	fCantonese	si4; si2 (also)
U+6642.0	fMandarin	shi2
U+6642.0	fEnglish	[1] time; fixed time [2] season [3] era; age; period; epoch [4] hours [5] often; frequently [6] fashionable [7] proper and adequate [8] opportune (moments); opportunity [9] timely; seasonable [10] now ... now ...; sometimes ... sometimes ... [11] a Chinese family name
U+6643.0	fUTF8	晃
U+6643.0	fCNS11643	1-556E
U+6643.0	fBig5	AECC
U+6643.0	fGB	0-3B4E
U+6643.0	fR/S	 72.6
U+6643.0	fTotalStrokes	 10
U+6643.0	fCangjie	AFMU
U+6643.0	fFourCorner	60211
U+6643.0	fCantonese	fong2
U+6643.0	fMandarin	huang3
U+6643.0	fEnglish	[1] brightness [2] dazzling; glaring [3] flash; flash past; appear and disappear very quickly
U+6643.1	fUTF8	晃
U+6643.1	fCNS11643	1-556E
U+6643.1	fBig5	AECC
U+6643.1	fGB	0-3B4E
U+6643.1	fR/S	 72.6
U+6643.1	fTotalStrokes	 10
U+6643.1	fCangjie	AFMU
U+6643.1	fFourCorner	60211
U+6643.1	fHakka	 fong3
U+6643.1	fMandarin	huang4; huang3}
U+6643.1	fEnglish	 rock; sway; shake
U+6643.2	fUTF8	晃
U+6643.2	fCNS11643	1-556E
U+6643.2	fBig5	AECC
U+6643.2	fGB	0-3B4E
U+6643.2	fR/S	 72.6
U+6643.2	fTotalStrokes	 10
U+6643.2	fCangjie	AFMU
U+6643.2	fFourCorner	60211
U+6644.0	fUTF8	晄
U+6644.0	fCNS11643	3-314C
U+6644.0	fGB	5-4963
U+6644.0	fR/S	  72.6
U+6644.0	fTotalStrokes	 10
U+6644.0	fCangjie	AFMU
U+6644.0	fFourCorner	69010
U+6644.0	fCantonese	fong2
U+6644.0	fMandarin	huang3; ge2
U+6644.0	fEnglish	 bright; dazzling sway; shake
U+6645.0	fUTF8	晅
U+6645.0	fCNS11643	1-5571
U+6645.0	fBig5	AECF
U+6645.0	fGB	3-4764
U+6645.0	fR/S	 72.6
U+6645.0	fTotalStrokes	 10
U+6645.0	fCangjie	AMAM
U+6645.0	fFourCorner	61016
U+6645.0	fHakka	  sien1
U+6645.0	fCantonese	hyun1; syun2 (also); hyun2
U+6645.0	fMandarin	xuan1; xuan3; xian3
U+6645.0	fEnglish	 light of the sun dry in the sun
U+6646.0	fUTF8	晆
U+6646.0	fCNS11643	3-3144
U+6646.0	fGB	E-2C67
U+6646.0	fR/S	 72.6
U+6646.0	fTotalStrokes	 10
U+6646.0	fCangjie	AGG
U+6646.0	fFourCorner	64010
U+6646.0	fCantonese	kwai1
U+6646.0	fMandarin	kui2
U+6647.0	fUTF8	晇
U+6647.0	fCNS11643	2-2F57
U+6647.0	fBig5	D1D5
U+6647.0	fGB	3-4765
U+6647.0	fR/S	 72.6
U+6647.0	fTotalStrokes	 10
U+6647.0	fCangjie	AKMS
U+6647.0	fFourCorner	64027
U+6647.0	fCantonese	heoi1; kwaa1
U+6647.0	fMandarin	xu1; kua1; hu1
U+6648.0	fUTF8	晈
U+6648.0	fCNS11643	3-3149
U+6648.0	fGB	5-4966
U+6648.0	fR/S	  72.6
U+6648.0	fTotalStrokes	 10
U+6648.0	fCangjie	AYCK
U+6648.0	fFourCorner	60040
U+6648.0	fHakka	 gau3
U+6648.0	fCantonese	gaau2
U+6648.0	fMandarin	jiao3
U+6649.0	fUTF8	晉
U+6649.0	fCNS11643	1-556C
U+6649.0	fBig5	AECA
U+6649.0	fGB	1-3D7A
U+6649.0	fR/S	 72.6
U+6649.0	fTotalStrokes	 10
U+6649.0	fCangjie	MIIA
U+6649.0	fFourCorner	10601
U+6649.0	fHakka	  zin3;  zin5
U+6649.0	fCantonese	zeon3
U+6649.0	fMandarin	jin4; jian4
U+6649.0	fEnglish	[1] advance; increase; flourish; promote [2] Tsin state [3] another name for Shansi province [4] Tsin dynasty (265-420 A.D.) [5] a Chinese family name
U+664A.0	fUTF8	晊
U+664A.0	fCNS11643	2-2F55
U+664A.0	fBig5	D1D3
U+664A.0	fGB	5-4962
U+664A.0	fR/S	 72.6
U+664A.0	fTotalStrokes	 10
U+664A.0	fCangjie	AMIG
U+664A.0	fFourCorner	61014
U+664A.0	fHakka	 zhit7; tsiit7
U+664A.0	fCantonese	zat1
U+664A.0	fMandarin	zhi3
U+664B.0	fUTF8	晋
U+664B.0	fCNS11643	3-3148
U+664B.0	fGB	0-3D7A
U+664B.0	fR/S	 72.6
U+664B.0	fTotalStrokes	 10
U+664B.0	fCangjie	MCA
U+664B.0	fFourCorner	10600
U+664B.0	fHakka	  zin3;  zin5; zin6
U+664B.0	fCantonese	zeon3
U+664B.0	fMandarin	jin4; jian4
U+664B.0	fEnglish	[1] non-classical form of &#x6649; [2] simplified form of &#x6649;
U+664C.0	fUTF8	晌
U+664C.0	fCNS11643	1-5570
U+664C.0	fBig5	AECE
U+664C.0	fGB	0-494E
U+664C.0	fR/S	 72.6
U+664C.0	fTotalStrokes	 10
U+664C.0	fCangjie	AHBR
U+664C.0	fFourCorner	67020
U+664C.0	fHakka	  shong3; song3; hiong3; hiong5
U+664C.0	fCantonese	hoeng2
U+664C.0	fMandarin	shang3
U+664C.0	fEnglish	[1] high noon; midday [2] a day's work (northeast China dialect) [3] a certain duration or interval of time
U+664D.0	fUTF8	晍
U+664D.0	fCNS11643	3-314D
U+664D.0	fGB	E-2C68
U+664D.0	fR/S	  72.6
U+664D.0	fTotalStrokes	 10
U+664D.0	fCangjie	ABMR
U+664D.0	fFourCorner	67020
U+664D.0	fHakka	 tung2
U+664D.0	fCantonese	tung4
U+664D.0	fMandarin	tong2; tong1; tong3
U+664D.0	fEnglish	same as &#x66C8;
U+664E.0	fUTF8	晎
U+664E.0	fCNS11643	3-3145
U+664E.0	fGB	E-2C69
U+664E.0	fR/S	  72.6
U+664E.0	fTotalStrokes	 10
U+664E.0	fCangjie	ATC
U+664E.0	fFourCorner	64080
U+664E.0	fCantonese	hung2
U+664E.0	fMandarin	hong3
U+664F.0	fUTF8	晏
U+664F.0	fCNS11643	1-556D
U+664F.0	fBig5	AECB
U+664F.0	fGB	0-6A4C
U+664F.0	fR/S	 72.6
U+664F.0	fTotalStrokes	 10
U+664F.0	fCangjie	AJV
U+664F.0	fFourCorner	60404
U+664F.0	fHakka	  on5; jen5; ngan5; an5
U+664F.0	fCantonese	aan3; ngaan3 (also)
U+664F.0	fMandarin	yan4
U+664F.0	fEnglish	[1] peaceful; quiet [2] clear (sky, sea, water, etc.) [3] late [4] a Chinese family name
U+6650.0	fUTF8	晐
U+6650.0	fCNS11643	3-3143
U+6650.0	fGB	3-4767
U+6650.0	fR/S	  72.6
U+6650.0	fTotalStrokes	 10
U+6650.0	fCangjie	AYVO
U+6650.0	fFourCorner	60080
U+6650.0	fHakka	  goi1
U+6650.0	fCantonese	goi1
U+6650.0	fMandarin	gai1
U+6650.0	fEnglish	[1] bright light over-spreading the world [2] all; the whole
U+6651.0	fUTF8	晑
U+6651.0	fCNS11643	2-2F58
U+6651.0	fBig5	D1D6
U+6651.0	fGB	E-2C6A
U+6651.0	fR/S	 72.6
U+6651.0	fTotalStrokes	 10
U+6651.0	fCangjie	AHBR
U+6651.0	fFourCorner	60227
U+6651.0	fCantonese	hoeng2
U+6651.0	fMandarin	xiang3
U+6652.0	fUTF8	晒
U+6652.0	fCNS11643	1-556F
U+6652.0	fBig5	AECD
U+6652.0	fGB	0-4939
U+6652.0	fR/S	 72.6
U+6652.0	fTotalStrokes	 10
U+6652.0	fCangjie	AMCW
U+6652.0	fFourCorner	61060
U+6652.0	fHakka	  sai5
U+6652.0	fCantonese	saai3
U+6652.0	fMandarin	shai4; chi1; sha4; shi4
U+6652.0	fEnglish	[1] same as &#x66EC; [2] simplified form of &#x66EC;
U+6653.0	fUTF8	晓
U+6653.0	fGB	0-4F7E
U+6653.0	fR/S	 72.6
U+6653.0	fTotalStrokes	 10
U+6653.0	fCangjie	AJPU
U+6653.0	fHakka	 hiau3
U+6653.0	fCantonese	hiu2
U+6653.0	fMandarin	xiao3
U+6653.0	fEnglish	simplified form of &#x66C9;
U+6654.0	fUTF8	晔
U+6654.0	fGB	0-6A4A
U+6654.0	fR/S	 72.6
U+6654.0	fTotalStrokes	 10
U+6654.0	fCangjie	AOPJ
U+6654.0	fHakka	 jap7; sap7
U+6654.0	fCantonese	jip6
U+6654.0	fMandarin	ye4; ye1; yi4
U+6654.0	fEnglish	simplified form of &#x66C4;
U+6655.0	fUTF8	晕
U+6655.0	fGB	0-544E
U+6655.0	fR/S	 72.6
U+6655.0	fTotalStrokes	 10
U+6655.0	fCangjie	ABKQ
U+6655.0	fHakka	 jun5
U+6655.0	fCantonese	wan4; wan6
U+6655.0	fMandarin	yun1; yin4; yun4
U+6655.0	fEnglish	simplified form of &#x6688;
U+6656.0	fUTF8	晖
U+6656.0	fGB	0-6A4D
U+6656.0	fR/S	 72.6
U+6656.0	fTotalStrokes	 10
U+6656.0	fCangjie	XABKQ
U+6656.0	fHakka	 fui1
U+6656.0	fCantonese	fai1
U+6656.0	fMandarin	hui1
U+6656.0	fEnglish	simplified form of &#x6689;
U+6657.0	fUTF8	晗
U+6657.0	fCNS11643	3-3642
U+6657.0	fGB	0-6A4F
U+6657.0	fR/S	  72.7
U+6657.0	fTotalStrokes	 11
U+6657.0	fCangjie	AOIR
U+6657.0	fFourCorner	68060
U+6657.0	fCantonese	ham4
U+6657.0	fMandarin	han2
U+6657.0	fEnglish	pre-dawn
U+6658.0	fUTF8	晘
U+6658.0	fCNS11643	4-3239
U+6658.0	fGB	5-4969
U+6658.0	fR/S	  72.7
U+6658.0	fTotalStrokes	 11
U+6658.0	fCangjie	AAMJ
U+6658.0	fFourCorner	66040
U+6658.0	fCantonese	waan5
U+6658.0	fMandarin	han4
U+6659.0	fUTF8	晙
U+6659.0	fCNS11643	2-356E
U+6659.0	fBig5	D5AC
U+6659.0	fGB	3-476E
U+6659.0	fR/S	 72.7
U+6659.0	fTotalStrokes	 11
U+6659.0	fCangjie	AICE
U+6659.0	fFourCorner	63047
U+6659.0	fHakka	 zun5
U+6659.0	fCantonese	zeon3
U+6659.0	fMandarin	jun4; zun4
U+6659.0	fEnglish	dawn; bright; clear
U+665A.0	fUTF8	晚
U+665A.0	fCNS11643	1-5B25
U+665A.0	fBig5	B1DF
U+665A.0	fGB	0-4D6D
U+665A.0	fR/S	 72.7
U+665A.0	fTotalStrokes	 11
U+665A.0	fCangjie	ANAU
U+665A.0	fFourCorner	67016
U+665A.0	fHakka	  wan1;  man1; wan3; man3; van3; van1
U+665A.0	fCantonese	maan5; maan1 (also); maan2 (also)
U+665A.0	fMandarin	wan3
U+665A.0	fEnglish	[1] sunset; night; evening [2] late [3] drawing toward the end [4] younger; junior
U+665B.0	fUTF8	晛
U+665B.0	fCNS11643	2-356D
U+665B.0	fBig5	D5AB
U+665B.0	fGB	3-474E
U+665B.0	fR/S	 72.7
U+665B.0	fTotalStrokes	 11
U+665B.0	fCangjie	ABUU
U+665B.0	fFourCorner	66010
U+665B.0	fHakka	 hien6; hien5; hien3
U+665B.0	fCantonese	jin5; jin2 (also); nin6; jin6
U+665B.0	fMandarin	xian4; nian4
U+665B.0	fEnglish	sunlight
U+665C.0	fUTF8	晜
U+665C.0	fCNS11643	2-356F
U+665C.0	fBig5	D5AD
U+665C.0	fGB	5-496C
U+665C.0	fR/S	 72.7
U+665C.0	fTotalStrokes	 11
U+665C.0	fCangjie	ACNH
U+665C.0	fFourCorner	60227
U+665C.0	fCantonese	kwan1; gwan1
U+665C.0	fMandarin	kun1
U+665C.0	fEnglish	insect
U+665D.0	fUTF8	晝
U+665D.0	fCNS11643	1-5B24
U+665D.0	fBig5	B1DE
U+665D.0	fGB	1-5667
U+665D.0	fR/S	 72.7
U+665D.0	fTotalStrokes	 11
U+665D.0	fCangjie	LGAM
U+665D.0	fFourCorner	50106
U+665D.0	fHakka	 zu5; zhiu5; ziu5; zhu5
U+665D.0	fCantonese	zau3
U+665D.0	fMandarin	zhou4
U+665D.0	fEnglish	[1] day; daytime; daylight [2] a Chinese family name
U+665E.0	fUTF8	晞
U+665E.0	fCNS11643	1-5B29
U+665E.0	fBig5	B1E3
U+665E.0	fGB	3-476C
U+665E.0	fR/S	 72.7
U+665E.0	fTotalStrokes	 11
U+665E.0	fCangjie	AKKB
U+665E.0	fFourCorner	64027
U+665E.0	fHakka	  hi1
U+665E.0	fCantonese	hei1
U+665E.0	fMandarin	xi1
U+665E.0	fEnglish	[1] dry in the sun; dry [2] sunshine at daybreak
U+665F.0	fUTF8	晟
U+665F.0	fCNS11643	2-2F56
U+665F.0	fBig5	D1D4
U+665F.0	fGB	0-6A49
U+665F.0	fR/S	 72.7
U+665F.0	fTotalStrokes	 11
U+665F.0	fCangjie	AIHS
U+665F.0	fFourCorner	60253
U+665F.0	fHakka	 shin2; shin5; siin2; siin5
U+665F.0	fCantonese	sing4; sing6 (also)
U+665F.0	fMandarin	sheng4; cheng2; jing1
U+665F.0	fEnglish	light; splendor; brightness of the sun
U+6660.0	fUTF8	晠
U+6660.0	fCNS11643	3-314B
U+6660.0	fGB	E-2C6B
U+6660.0	fR/S	 72.7; 72.6
U+6660.0	fTotalStrokes	 10; 11
U+6660.0	fCangjie	AIHS
U+6660.0	fFourCorner	63050
U+6660.0	fHakka	 shin2; shin5; siin2; siin5
U+6660.0	fCantonese	sing4; sing6 (also)
U+6660.0	fMandarin	sheng4; cheng2; jing1
U+6660.0	fEnglish	same as &#x665F;
U+6661.0	fUTF8	晡
U+6661.0	fCNS11643	2-356C
U+6661.0	fBig5	D5AA
U+6661.0	fGB	0-6A4E
U+6661.0	fR/S	 72.7
U+6661.0	fTotalStrokes	 11
U+6661.0	fCangjie	AIJB
U+6661.0	fFourCorner	63027
U+6661.0	fHakka	  bu1
U+6661.0	fCantonese	bou1
U+6661.0	fMandarin	bu1
U+6661.0	fEnglish	afternoon; late afternoon
U+6662.0	fUTF8	晢
U+6662.0	fCNS11643	2-3570
U+6662.0	fBig5	D5AE
U+6662.0	fGB	3-476B
U+6662.0	fR/S	 72.7
U+6662.0	fTotalStrokes	 11
U+6662.0	fCangjie	QLA
U+6662.0	fFourCorner	52602
U+6662.0	fHakka	 zhat7; zat7; zhi5; zih5; zhet7
U+6662.0	fCantonese	zit3; zai3 (also); sik1
U+6662.0	fMandarin	zhe2; zhe1; zhi4; xi1; zhe5
U+6662.0	fEnglish	 light of stars shine
U+6663.0	fUTF8	晣
U+6663.0	fCNS11643	3-3644
U+6663.0	fGB	5-4968
U+6663.0	fR/S	 72.7
U+6663.0	fTotalStrokes	 11
U+6663.0	fCangjie	AQHL
U+6663.0	fFourCorner	62020
U+6663.0	fHakka	 zhat7; zat7; zhi5; zih5; zhet7
U+6663.0	fCantonese	zit3; zai3 (also); sik1
U+6663.0	fMandarin	zhe2; zhe1; zhi4; xi1; zhe5
U+6663.0	fEnglish	same as &#x6662;
U+6664.0	fUTF8	晤
U+6664.0	fCNS11643	1-5B26
U+6664.0	fBig5	B1E0
U+6664.0	fGB	0-4E6E
U+6664.0	fR/S	 72.7
U+6664.0	fTotalStrokes	 11
U+6664.0	fCangjie	AMMR
U+6664.0	fFourCorner	61061
U+6664.0	fHakka	 ngu3; ngu5
U+6664.0	fCantonese	ng6
U+6664.0	fMandarin	wu4
U+6664.0	fEnglish	[1] have interview with; meet; see face to face [2] enlightened; wise
U+6665.0	fUTF8	晥
U+6665.0	fCNS11643	2-356B
U+6665.0	fBig5	D5A9
U+6665.0	fGB	5-496D
U+6665.0	fR/S	 72.7
U+6665.0	fTotalStrokes	 11
U+6665.0	fCangjie	AJMU
U+6665.0	fFourCorner	63011
U+6665.0	fHakka	  fan3; wan3; fon3
U+6665.0	fCantonese	wun5; faan2
U+6665.0	fMandarin	han4; huan3
U+6665.0	fEnglish	 Anhui province name of ancient
U+6666.0	fUTF8	晦
U+6666.0	fCNS11643	1-5B28
U+6666.0	fBig5	B1E2
U+6666.0	fGB	0-3B5E
U+6666.0	fR/S	 72.7
U+6666.0	fTotalStrokes	 11
U+6666.0	fCangjie	AOWY
U+6666.0	fFourCorner	68057
U+6666.0	fHakka	  fui5; fi5; fui3
U+6666.0	fCantonese	fui3
U+6666.0	fMandarin	hui4
U+6666.0	fEnglish	[1] the last day of every month in the lunar calendar [2] night; evening; dark [3] obscure; indistinguishable [4] unlucky; bad luck
U+6667.0	fUTF8	晧
U+6667.0	fCNS11643	3-3645
U+6667.0	fGB	5-496A
U+6667.0	fR/S	  72.7
U+6667.0	fTotalStrokes	 11
U+6667.0	fCangjie	AHGR
U+6667.0	fFourCorner	65060
U+6667.0	fCantonese	hou6
U+6667.0	fMandarin	hao4
U+6667.0	fEnglish	[1] daybreak [2] bright and brilliant
U+6668.0	fUTF8	晨
U+6668.0	fCNS11643	1-5B27
U+6668.0	fBig5	B1E1
U+6668.0	fGB	0-333F
U+6668.0	fR/S	 72.7
U+6668.0	fTotalStrokes	 11
U+6668.0	fCangjie	AMMV
U+6668.0	fFourCorner	60232
U+6668.0	fHakka	 sin2; shin2; siin2; shin3; sen2
U+6668.0	fCantonese	san4
U+6668.0	fMandarin	chen2; shen2
U+6668.0	fEnglish	[1] early morning; daybreak; morning [2] announce the arrival of morning (said of a cock); crow at down
U+6668.1	fUTF8	晨
U+6668.1	fCNS11643	1-5B27
U+6668.1	fBig5	B1E1
U+6668.1	fGB	0-333F
U+6668.1	fR/S	 72.7
U+6668.1	fTotalStrokes	 11
U+6668.1	fCangjie	AMMV
U+6668.1	fFourCorner	60232
U+6669.0	fUTF8	晩
U+6669.0	fCNS11643	3-3C26
U+6669.0	fGB	E-2C6C
U+6669.0	fR/S	 72.7
U+6669.0	fTotalStrokes	 12; 11
U+6669.0	fCangjie	XXANA
U+6669.0	fFourCorner	67010
U+6669.0	fCantonese	maan5; man5
U+6669.0	fMandarin	wan3
U+6669.0	fEnglish	[1] sunset; night; evening [2] late [3] drawing toward the end [4] younger; junior
U+666A.0	fUTF8	晪
U+666A.0	fCNS11643	2-3C4A
U+666A.0	fBig5	D9A7
U+666A.0	fGB	E-2C6D
U+666A.0	fR/S	 72.8
U+666A.0	fTotalStrokes	 12
U+666A.0	fCangjie	ATBC
U+666A.0	fFourCorner	65081
U+666A.0	fCantonese	long4; tin2
U+666A.0	fMandarin	tian3
U+666B.0	fUTF8	晫
U+666B.0	fCNS11643	3-3B7D
U+666B.0	fGB	3-4771
U+666B.0	fR/S	 72.8
U+666B.0	fTotalStrokes	 12
U+666B.0	fCangjie	AYAJ
U+666B.0	fFourCorner	61040
U+666B.0	fCantonese	zoek3
U+666B.0	fMandarin	zhuo2
U+666C.0	fUTF8	晬
U+666C.0	fCNS11643	2-3C45
U+666C.0	fBig5	D9A2
U+666C.0	fGB	3-4775
U+666C.0	fR/S	 72.8
U+666C.0	fTotalStrokes	 12
U+666C.0	fCangjie	AYOJ
U+666C.0	fFourCorner	60048
U+666C.0	fHakka	 cui5
U+666C.0	fCantonese	zeoi3
U+666C.0	fMandarin	zui4
U+666C.0	fEnglish	[1] first birthday of a child [2] anniversary
U+666D.0	fUTF8	晭
U+666D.0	fCNS11643	3-3C25
U+666D.0	fGB	5-4971
U+666D.0	fR/S	 72.8
U+666D.0	fTotalStrokes	 12
U+666D.0	fCangjie	ABGR
U+666D.0	fFourCorner	67020
U+666D.0	fCantonese	zau2
U+666D.0	fMandarin	zhou3; cou3
U+666D.0	fEnglish	same as &#x3AF6;
U+666E.0	fUTF8	普
U+666E.0	fCNS11643	1-5F5B
U+666E.0	fBig5	B4B6
U+666E.0	fGB	0-4655
U+666E.0	fR/S	 72.8
U+666E.0	fTotalStrokes	 12
U+666E.0	fCangjie	TCA
U+666E.0	fFourCorner	80601
U+666E.0	fHakka	  pu1; pu3
U+666E.0	fCantonese	pou2
U+666E.0	fMandarin	pu3
U+666E.0	fEnglish	[1] universal; general; widespread; everywhere; all [2] Prussia [3] a Chinese family name
U+666F.0	fUTF8	景
U+666F.0	fCNS11643	1-5F5F
U+666F.0	fBig5	B4BA
U+666F.0	fGB	0-3E30
U+666F.0	fR/S	 72.8
U+666F.0	fTotalStrokes	 12
U+666F.0	fCangjie	AYRF
U+666F.0	fFourCorner	60906
U+666F.0	fHakka	  gin3; jang3; rang3
U+666F.0	fCantonese	ging2; jing2
U+666F.0	fMandarin	jing3
U+666F.0	fEnglish	[1] scenery; views [2] conditions; prospects; circumstances; situations [3] settings; background scenes (in movies, stage shows, etc.) [4] big and strong [5] great [6] high [7] bright and luminous [8] admire; respect [9] a Chinese family name
U+666F.1	fUTF8	景
U+666F.1	fCNS11643	1-5F5F
U+666F.1	fBig5	B4BA
U+666F.1	fGB	0-3E30
U+666F.1	fR/S	 72.8
U+666F.1	fTotalStrokes	 12
U+666F.1	fCangjie	AYRF
U+666F.1	fFourCorner	60906
U+666F.1	fMandarin	ying3
U+666F.1	fEnglish	shadow; reflection; image
U+6670.0	fUTF8	晰
U+6670.0	fCNS11643	1-5F5C
U+6670.0	fBig5	B4B7
U+6670.0	fGB	0-4E7A
U+6670.0	fR/S	 72.8
U+6670.0	fTotalStrokes	 12
U+6670.0	fCangjie	ADHL
U+6670.0	fFourCorner	62021
U+6670.0	fHakka	  sit7
U+6670.0	fCantonese	sik1; cik1
U+6670.0	fMandarin	xi1
U+6670.0	fEnglish	[1] clear; evident; clearly; distinct [2] same as &#x6673;
U+6671.0	fUTF8	晱
U+6671.0	fCNS11643	2-3C48
U+6671.0	fBig5	D9A5
U+6671.0	fGB	5-4973
U+6671.0	fR/S	 72.8
U+6671.0	fTotalStrokes	 12
U+6671.0	fCangjie	AFF
U+6671.0	fFourCorner	69089
U+6671.0	fCantonese	sim2
U+6671.0	fMandarin	shan3
U+6672.0	fUTF8	晲
U+6672.0	fCNS11643	2-3C4B
U+6672.0	fBig5	D9A8
U+6672.0	fGB	3-4773
U+6672.0	fR/S	 72.8
U+6672.0	fTotalStrokes	 12
U+6672.0	fCangjie	AHXU
U+6672.0	fFourCorner	67017
U+6672.0	fHakka	 fon3
U+6672.0	fCantonese	ngai5; ai5 (also); ngai6
U+6672.0	fMandarin	yi3; ni4
U+6673.0	fUTF8	晳
U+6673.0	fCNS11643	3-3C23
U+6673.0	fGB	8-2D72
U+6673.0	fR/S	  72.8
U+6673.0	fTotalStrokes	 12
U+6673.0	fCangjie	DLA
U+6673.0	fFourCorner	42600
U+6673.0	fHakka	  sit7
U+6673.0	fCantonese	sik1
U+6673.0	fMandarin	xi4; xi1
U+6673.0	fEnglish	[1] fair; white (said of one's skin) [2] same as &#x6670;
U+6674.0	fUTF8	晴
U+6674.0	fCNS11643	1-5F5D
U+6674.0	fBig5	B4B8
U+6674.0	fGB	0-4767
U+6674.0	fR/S	 72.8
U+6674.0	fTotalStrokes	 12
U+6674.0	fCangjie	AQMB
U+6674.0	fFourCorner	65027
U+6674.0	fHakka	 ciang2; ciang3; ziang3; zin1
U+6674.0	fCantonese	cing4; ceng4 (also)
U+6674.0	fMandarin	qing2
U+6674.0	fEnglish	[1] clear weather; fine weather; bright; clear; not raining [2] when the rain stops
U+6675.0	fUTF8	晵
U+6675.0	fCNS11643	4-3824
U+6675.0	fGB	3-4777
U+6675.0	fR/S	 72.8
U+6675.0	fTotalStrokes	 12
U+6675.0	fCangjie	HKA
U+6675.0	fFourCorner	38600
U+6675.0	fHakka	 ki3
U+6675.0	fCantonese	kai2
U+6675.0	fMandarin	qi3
U+6676.0	fUTF8	晶
U+6676.0	fCNS11643	1-5F5E
U+6676.0	fBig5	B4B9
U+6676.0	fGB	0-3E27
U+6676.0	fR/S	 72.8
U+6676.0	fTotalStrokes	 12
U+6676.0	fCangjie	AAA
U+6676.0	fFourCorner	60660
U+6676.0	fHakka	  zin1
U+6676.0	fCantonese	zing1
U+6676.0	fMandarin	jing1
U+6676.0	fEnglish	[1] crystal [2] clear; bright; radiant; glittering
U+6677.0	fUTF8	晷
U+6677.0	fCNS11643	1-5F63
U+6677.0	fBig5	B4BE
U+6677.0	fGB	0-6A50
U+6677.0	fR/S	 72.8
U+6677.0	fTotalStrokes	 12
U+6677.0	fCangjie	AHOR
U+6677.0	fFourCorner	60604
U+6677.0	fHakka	 gui3; kui3; gwi3; kwui3
U+6677.0	fCantonese	gwai2
U+6677.0	fMandarin	gui3
U+6677.0	fEnglish	[1] shadows caused by the sun [2] time [3] sundial
U+6678.0	fUTF8	晸
U+6678.0	fCNS11643	2-434D
U+6678.0	fBig5	DDC7
U+6678.0	fGB	3-4779
U+6678.0	fR/S	 72.8
U+6678.0	fTotalStrokes	 12
U+6678.0	fCangjie	AMMK
U+6678.0	fFourCorner	60141
U+6678.0	fHakka	 zhin3; zihn3
U+6678.0	fCantonese	zing2; zan2
U+6678.0	fMandarin	zhen3; zheng3
U+6678.0	fEnglish	the sun rising
U+6679.0	fUTF8	晹
U+6679.0	fCNS11643	2-3C49
U+6679.0	fBig5	D9A6
U+6679.0	fGB	3-4772
U+6679.0	fR/S	 72.8
U+6679.0	fTotalStrokes	 12
U+6679.0	fCangjie	AAPH
U+6679.0	fFourCorner	66027
U+6679.0	fCantonese	sik1; jik6
U+6679.0	fMandarin	yi4; shi4
U+667A.0	fUTF8	智
U+667A.0	fCNS11643	1-5F61
U+667A.0	fBig5	B4BC
U+667A.0	fGB	0-5647
U+667A.0	fR/S	 72.8
U+667A.0	fTotalStrokes	 12
U+667A.0	fCangjie	ORA
U+667A.0	fFourCorner	86600
U+667A.0	fHakka	 zhi5; zi5; zih5; tje5
U+667A.0	fCantonese	zi3
U+667A.0	fMandarin	zhi4; zhi1
U+667A.0	fEnglish	talented; wisdom; knowledge; intelligence; capable; clever; wise; wit
U+667B.0	fUTF8	晻
U+667B.0	fCNS11643	2-3C46
U+667B.0	fBig5	D9A3
U+667B.0	fGB	E-2C6E
U+667B.0	fR/S	 72.8
U+667B.0	fTotalStrokes	 12
U+667B.0	fCangjie	AKLU
U+667B.0	fFourCorner	64016
U+667B.0	fHakka	  jam3; ram3
U+667B.0	fCantonese	am2; ngam2 (also); jim2; am3; ngam3 (also)
U+667B.0	fMandarin	an3; yan3; an4
U+667B.0	fEnglish	[1] obscured; hidden from sight [2] screen
U+667C.0	fUTF8	晼
U+667C.0	fCNS11643	2-3C44
U+667C.0	fBig5	D9A1
U+667C.0	fGB	3-4776
U+667C.0	fR/S	 72.8
U+667C.0	fTotalStrokes	 12
U+667C.0	fCangjie	AJNU
U+667C.0	fFourCorner	63012
U+667C.0	fHakka	 won3; wan3; von3
U+667C.0	fCantonese	jyun2; wun2
U+667C.0	fMandarin	wan3
U+667C.0	fEnglish	the declining sun
U+667D.0	fUTF8	晽
U+667D.0	fCNS11643	3-3B7C
U+667D.0	fGB	3-4770
U+667D.0	fR/S	 72.8
U+667D.0	fTotalStrokes	 12
U+667D.0	fCangjie	ADD
U+667D.0	fFourCorner	64090
U+667D.0	fCantonese	lam4
U+667D.0	fMandarin	lin2
U+667E.0	fUTF8	晾
U+667E.0	fCNS11643	1-5F62
U+667E.0	fBig5	B4BD
U+667E.0	fGB	0-4140
U+667E.0	fR/S	 72.8
U+667E.0	fTotalStrokes	 12
U+667E.0	fCangjie	AYRF
U+667E.0	fFourCorner	60096
U+667E.0	fHakka	 liong5; lang2; liong6; liong2; long5
U+667E.0	fCantonese	long6; long3
U+667E.0	fMandarin	liang4
U+667E.0	fEnglish	[1] dry in the air; air; hang in the wind to dry [2] dry in the sun
U+667F.0	fUTF8	晿
U+667F.0	fCNS11643	3-3B7E
U+667F.0	fGB	E-2C6F
U+667F.0	fR/S	  72.8
U+667F.0	fTotalStrokes	 12
U+667F.0	fCangjie	AAA
U+667F.0	fFourCorner	66060
U+667F.0	fCantonese	coeng1
U+667F.0	fMandarin	chang1
U+6680.0	fUTF8	暀
U+6680.0	fCNS11643	2-3C47
U+6680.0	fBig5	D9A4
U+6680.0	fGB	5-4970
U+6680.0	fR/S	 72.8
U+6680.0	fTotalStrokes	 12
U+6680.0	fCangjie	AHOG
U+6680.0	fFourCorner	60014
U+6680.0	fCantonese	wong6; wong2; wong5
U+6680.0	fMandarin	wang3; wang4; guang4; huang4
U+6681.0	fUTF8	暁
U+6681.0	fCNS11643	3-3C24
U+6681.0	fGB	E-2C70
U+6681.0	fR/S	 72.8
U+6681.0	fTotalStrokes	 12
U+6681.0	fCangjie	AJTU
U+6681.0	fFourCorner	64010
U+6681.0	fHakka	 hiau3
U+6681.0	fCantonese	hiu2
U+6681.0	fMandarin	xiao3
U+6681.0	fEnglish	dawn, daybreak; clear, explicit
U+6682.0	fUTF8	暂
U+6682.0	fGB	0-545D
U+6682.0	fR/S	 72.8
U+6682.0	fTotalStrokes	 12
U+6682.0	fCangjie	KLA
U+6682.0	fHakka	 ciam5
U+6682.0	fCantonese	zaam6
U+6682.0	fMandarin	zan4; zhan4
U+6682.0	fEnglish	simplified form of &#x66AB;
U+6683.0	fUTF8	暃
U+6683.0	fGB	E-2C71
U+6683.0	fR/S	 72.8
U+6683.0	fTotalStrokes	 12
U+6683.0	fCangjie	ALMY
U+6683.0	fCantonese	fei1
U+6684.0	fUTF8	暄
U+6684.0	fCNS11643	1-6441
U+6684.0	fBig5	B779
U+6684.0	fGB	0-6A51
U+6684.0	fR/S	 72.9
U+6684.0	fTotalStrokes	 13
U+6684.0	fCangjie	AJMM
U+6684.0	fFourCorner	63016
U+6684.0	fHakka	 sien1; sian1; sen1
U+6684.0	fCantonese	hyun1
U+6684.0	fMandarin	xuan1
U+6684.0	fEnglish	[1] warm; comfortable and genial (climates); genial [2] fluffy
U+6685.0	fUTF8	暅
U+6685.0	fCNS11643	4-3D69
U+6685.0	fGB	3-4823
U+6685.0	fR/S	 72.9
U+6685.0	fTotalStrokes	 13
U+6685.0	fCangjie	APMM
U+6685.0	fFourCorner	61010
U+6685.0	fCantonese	hyun1; syun2 (also)
U+6685.0	fMandarin	xuan1; xuan3; geng4; geng3
U+6686.0	fUTF8	暆
U+6686.0	fCNS11643	2-4345
U+6686.0	fBig5	DDBF
U+6686.0	fGB	3-4821
U+6686.0	fR/S	 72.9
U+6686.0	fTotalStrokes	 13
U+6686.0	fCangjie	AYSD
U+6686.0	fFourCorner	68012
U+6686.0	fHakka	 ji2; ri2; i2
U+6686.0	fCantonese	ji4
U+6686.0	fMandarin	yi2
U+6686.0	fEnglish	the sun's course after it has passed the meridian
U+6687.0	fUTF8	暇
U+6687.0	fCNS11643	1-643E
U+6687.0	fBig5	B776
U+6687.0	fGB	0-4F3E
U+6687.0	fR/S	 72.9
U+6687.0	fTotalStrokes	 13
U+6687.0	fCangjie	ARSE
U+6687.0	fFourCorner	67047
U+6687.0	fHakka	 ha3; ha2; ha5; ha6
U+6687.0	fCantonese	haa6; haa4 (also); haa1 (also)
U+6687.0	fMandarin	xia2; xia4
U+6687.0	fEnglish	leisure; relaxation; spare time; free time
U+6688.0	fUTF8	暈
U+6688.0	fCNS11643	1-643F
U+6688.0	fBig5	B777
U+6688.0	fGB	1-544E
U+6688.0	fR/S	 72.9
U+6688.0	fTotalStrokes	 13
U+6688.0	fCangjie	ABJJ
U+6688.0	fFourCorner	60506
U+6688.0	fHakka	  fun2; hin2;  jun5; fun3; jun2; win2; run6
U+6688.0	fCantonese	wan4; wan6; jan3
U+6688.0	fMandarin	yun1; yin4
U+6688.0	fEnglish	[1] dizzy; giddy [2] faint; swoon [3] do things without a purpose (sarcastically)
U+6688.1	fUTF8	暈
U+6688.1	fCNS11643	1-643F
U+6688.1	fBig5	B777
U+6688.1	fGB	1-544E
U+6688.1	fR/S	 72.9
U+6688.1	fTotalStrokes	 13
U+6688.1	fCangjie	ABJJ
U+6688.1	fFourCorner	60506
U+6688.1	fMandarin	yun4
U+6688.1	fEnglish	[1] halo in sky; vapors; mist [2] dazzled; feel faint or dizzy
U+6689.0	fUTF8	暉
U+6689.0	fCNS11643	1-643D
U+6689.0	fBig5	B775
U+6689.0	fGB	1-6A4D
U+6689.0	fR/S	 72.9
U+6689.0	fTotalStrokes	 13
U+6689.0	fCangjie	ABJJ
U+6689.0	fFourCorner	67056
U+6689.0	fHakka	 fui1; fi1
U+6689.0	fCantonese	fai1
U+6689.0	fMandarin	hui1
U+6689.0	fEnglish	[1] sunshine; light of the sun [2] bright; radiant
U+668A.0	fUTF8	暊
U+668A.0	fCNS11643	2-434A
U+668A.0	fBig5	DDC4
U+668A.0	fGB	5-4961
U+668A.0	fR/S	 72.9
U+668A.0	fTotalStrokes	 13
U+668A.0	fCangjie	AMBC
U+668A.0	fFourCorner	61086
U+668A.0	fCantonese	seoi2; fu2
U+668A.0	fMandarin	fu3
U+668B.0	fUTF8	暋
U+668B.0	fCNS11643	2-4349
U+668B.0	fBig5	DDC3
U+668B.0	fGB	3-4824
U+668B.0	fR/S	 72.9
U+668B.0	fTotalStrokes	 13
U+668B.0	fCangjie	RKA
U+668B.0	fFourCorner	78604
U+668B.0	fHakka	  men3; man3
U+668B.0	fCantonese	man5; man4
U+668B.0	fMandarin	min3; min2; hui1
U+668B.0	fEnglish	strong; robust; tough
U+668C.0	fUTF8	暌
U+668C.0	fCNS11643	2-4346
U+668C.0	fBig5	DDC0
U+668C.0	fGB	0-6A52
U+668C.0	fR/S	 72.9
U+668C.0	fTotalStrokes	 13
U+668C.0	fCangjie	ANOK
U+668C.0	fFourCorner	62034
U+668C.0	fHakka	 kui2; kwi2; k'wui2
U+668C.0	fCantonese	kwai4
U+668C.0	fMandarin	kui2
U+668C.0	fEnglish	[1] in opposition [2] part; separate; distant from
U+668D.0	fUTF8	暍
U+668D.0	fCNS11643	1-6443
U+668D.0	fBig5	B77B
U+668D.0	fGB	3-477C
U+668D.0	fR/S	 72.9
U+668D.0	fTotalStrokes	 13
U+668D.0	fCangjie	AAPV
U+668D.0	fFourCorner	66027
U+668D.0	fHakka	 giet7; hot7; jat7; jet7
U+668D.0	fCantonese	hot3; jit3
U+668D.0	fMandarin	he4; ye4; ye1
U+668D.0	fEnglish	sunstroke
U+668E.0	fUTF8	暎
U+668E.0	fCNS11643	3-415B
U+668E.0	fGB	8-2D71
U+668E.0	fR/S	 72.9
U+668E.0	fTotalStrokes	 13
U+668E.0	fCangjie	ATLK
U+668E.0	fFourCorner	64037
U+668E.0	fHakka	 jang3; rong3; jong3; rang3; jang5
U+668E.0	fCantonese	jing2
U+668E.0	fMandarin	ying4; ang3
U+668E.0	fEnglish	sun beginning decline; reflect
U+668F.0	fUTF8	暏
U+668F.0	fCNS11643	3-4156
U+668F.0	fGB	5-496E
U+668F.0	fR/S	 72.9; 72.8
U+668F.0	fTotalStrokes	 13; 12
U+668F.0	fCangjie	AJKA
U+668F.0	fFourCorner	64060
U+668F.0	fHakka	  du5; du3
U+668F.0	fCantonese	syu6
U+668F.0	fMandarin	du3
U+668F.0	fEnglish	morning; dawn
U+6690.0	fUTF8	暐
U+6690.0	fCNS11643	2-4348
U+6690.0	fBig5	DDC2
U+6690.0	fGB	5-4950
U+6690.0	fR/S	 72.9
U+6690.0	fTotalStrokes	 13
U+6690.0	fCangjie	ADMQ
U+6690.0	fFourCorner	64056
U+6690.0	fHakka	 wi3; wui3; vui3
U+6690.0	fCantonese	wai5
U+6690.0	fMandarin	wei3
U+6690.0	fEnglish	the bright shining of the sun
U+6691.0	fUTF8	暑
U+6691.0	fCNS11643	1-5F60
U+6691.0	fBig5	B4BB
U+6691.0	fGB	0-4A6E
U+6691.0	fR/S	 72.8; 72.9
U+6691.0	fTotalStrokes	 12
U+6691.0	fCangjie	AJKA
U+6691.0	fFourCorner	60604
U+6691.0	fHakka	 cu3;  chu3;  shu3; su3
U+6691.0	fCantonese	syu2
U+6691.0	fMandarin	shu3
U+6691.0	fEnglish	[1] hot heat of summer [2] mid-summer; summer
U+6692.0	fUTF8	暒
U+6692.0	fCNS11643	3-4159
U+6692.0	fGB	5-4977
U+6692.0	fR/S	 72.9
U+6692.0	fTotalStrokes	 13
U+6692.0	fCangjie	AAHM
U+6692.0	fFourCorner	66010
U+6692.0	fCantonese	cing4
U+6692.0	fMandarin	qing2
U+6693.0	fUTF8	暓
U+6693.0	fCNS11643	3-4151
U+6693.0	fGB	5-4979
U+6693.0	fR/S	  72.9
U+6693.0	fTotalStrokes	 13
U+6693.0	fCangjie	NKA
U+6693.0	fFourCorner	18600
U+6693.0	fCantonese	mau6
U+6693.0	fMandarin	mao4
U+6694.0	fUTF8	暔
U+6694.0	fCNS11643	2-434C
U+6694.0	fBig5	DDC6
U+6694.0	fGB	3-477A
U+6694.0	fR/S	 72.9
U+6694.0	fTotalStrokes	 13
U+6694.0	fCangjie	AJBJ
U+6694.0	fFourCorner	64027
U+6694.0	fCantonese	naam4; laam4 (also)
U+6694.0	fMandarin	nan2
U+6695.0	fUTF8	暕
U+6695.0	fCNS11643	2-4347
U+6695.0	fBig5	DDC1
U+6695.0	fGB	3-477B
U+6695.0	fR/S	 72.9
U+6695.0	fTotalStrokes	 13
U+6695.0	fCangjie	ADWF
U+6695.0	fFourCorner	65096
U+6695.0	fCantonese	gaan2; laan4
U+6695.0	fMandarin	jian3; lan2
U+6696.0	fUTF8	暖
U+6696.0	fCNS11643	1-6440
U+6696.0	fBig5	B778
U+6696.0	fGB	0-452F
U+6696.0	fR/S	 72.9
U+6696.0	fTotalStrokes	 13
U+6696.0	fCangjie	ABME
U+6696.0	fFourCorner	62047
U+6696.0	fHakka	 non1;  lon1;  non3; nun2
U+6696.0	fCantonese	nyun5; lyun5 (also); nyun2 (also); lyun2 (also); hyun1
U+6696.0	fMandarin	nuan3; xuan1; xuan3
U+6696.0	fEnglish	warm; genial (weather)
U+6697.0	fUTF8	暗
U+6697.0	fCNS11643	1-643C
U+6697.0	fBig5	B774
U+6697.0	fGB	0-3035
U+6697.0	fR/S	 72.9
U+6697.0	fTotalStrokes	 13
U+6697.0	fCangjie	AYTA
U+6697.0	fFourCorner	60061
U+6697.0	fHakka	  am5; em3
U+6697.0	fCantonese	am3; ngam3 (also)
U+6697.0	fMandarin	an4; an3
U+6697.0	fEnglish	[1] dark; dim; obscure [2] in secret; clandestine; stealthy [3] stupid; ignorant [4] invisible; hidden (meanings, drainage systems, rocks, etc.)
U+6698.0	fUTF8	暘
U+6698.0	fCNS11643	1-6442
U+6698.0	fBig5	B77A
U+6698.0	fGB	3-474C
U+6698.0	fR/S	 72.9
U+6698.0	fTotalStrokes	 13
U+6698.0	fCangjie	AAMH
U+6698.0	fFourCorner	66027
U+6698.0	fHakka	  jong2; rong2; jong3
U+6698.0	fCantonese	joeng4
U+6698.0	fMandarin	yang2
U+6698.0	fEnglish	 rising sun sunshine
U+6699.0	fUTF8	暙
U+6699.0	fCNS11643	2-434B
U+6699.0	fBig5	DDC5
U+6699.0	fGB	5-4976
U+6699.0	fR/S	 72.9
U+6699.0	fTotalStrokes	 13
U+6699.0	fCangjie	AQKA
U+6699.0	fFourCorner	65068
U+6699.0	fCantonese	ceon1
U+6699.0	fMandarin	chun1
U+669A.0	fUTF8	暚
U+669A.0	fCNS11643	3-4734
U+669A.0	fGB	3-4827
U+669A.0	fR/S	 72.10
U+669A.0	fTotalStrokes	 14
U+669A.0	fCangjie	ABOU
U+669A.0	fFourCorner	67070
U+669A.0	fCantonese	jiu4
U+669A.0	fMandarin	yao2
U+669B.0	fUTF8	暛
U+669B.0	fCNS11643	4-436B
U+669B.0	fGB	3-4822
U+669B.0	fR/S	  72.10
U+669B.0	fTotalStrokes	 14
U+669B.0	fCangjie	ATQM
U+669B.0	fFourCorner	68010
U+669B.0	fCantonese	so2
U+669B.0	fMandarin	suo3
U+669C.0	fUTF8	暜
U+669C.0	fCNS11643	3-4735
U+669C.0	fGB	E-2C72
U+669C.0	fR/S	 72.10
U+669C.0	fTotalStrokes	 14
U+669C.0	fCangjie	YTA
U+669C.0	fFourCorner	00600
U+669C.0	fHakka	  pu1; pu3
U+669C.0	fCantonese	pou2
U+669C.0	fMandarin	pu3
U+669C.0	fEnglish	standard form of &#x666E;
U+669D.0	fUTF8	暝
U+669D.0	fCNS11643	1-6925
U+669D.0	fBig5	BA5C
U+669D.0	fGB	0-6A54
U+669D.0	fR/S	 72.10
U+669D.0	fTotalStrokes	 14
U+669D.0	fCangjie	ABAC
U+669D.0	fFourCorner	67080
U+669D.0	fHakka	 men2
U+669D.0	fCantonese	ming4; ming6 (also); ming5 (also)
U+669D.0	fMandarin	ming2; ming3
U+669D.0	fEnglish	dark; obscure
U+669D.1	fUTF8	暝
U+669D.1	fCNS11643	1-6925
U+669D.1	fBig5	BA5C
U+669D.1	fGB	0-6A54
U+669D.1	fR/S	 72.10
U+669D.1	fTotalStrokes	 14
U+669D.1	fCangjie	ABAC
U+669D.1	fFourCorner	67080
U+669D.1	fMandarin	ming4
U+669D.1	fEnglish	night
U+669E.0	fUTF8	暞
U+669E.0	fCNS11643	4-436F
U+669E.0	fGB	E-2C73
U+669E.0	fR/S	  72.10
U+669E.0	fTotalStrokes	 14
U+669E.0	fCangjie	AHUD
U+669E.0	fFourCorner	66090
U+669E.0	fCantonese	giu2
U+669E.0	fMandarin	jiao3
U+669F.0	fUTF8	暟
U+669F.0	fCNS11643	2-4A5F
U+669F.0	fBig5	E1F8
U+669F.0	fGB	3-4766
U+669F.0	fR/S	 72.10
U+669F.0	fTotalStrokes	 14
U+669F.0	fCangjie	AUMT
U+669F.0	fFourCorner	62018
U+669F.0	fCantonese	hoi2
U+669F.0	fMandarin	kai3
U+66A0.0	fUTF8	暠
U+66A0.0	fCNS11643	2-4A5E
U+66A0.0	fBig5	E1F7
U+66A0.0	fGB	E-2C74
U+66A0.0	fR/S	 72.10
U+66A0.0	fTotalStrokes	 14
U+66A0.0	fCangjie	AYRB
U+66A0.0	fFourCorner	60227
U+66A0.0	fHakka	 gau3;  hau3; hau1
U+66A0.0	fCantonese	gou2; hou6; hou1
U+66A0.0	fMandarin	gao3; hao4
U+66A0.0	fEnglish	 daybreak bright and brilliant
U+66A1.0	fUTF8	暡
U+66A1.0	fCNS11643	2-4A5D
U+66A1.0	fBig5	E1F6
U+66A1.0	fGB	5-4A23
U+66A1.0	fR/S	 72.10
U+66A1.0	fTotalStrokes	 14
U+66A1.0	fCangjie	ACIM
U+66A1.0	fFourCorner	68027
U+66A1.0	fCantonese	jung2
U+66A1.0	fMandarin	weng3
U+66A2.0	fUTF8	暢
U+66A2.0	fCNS11643	1-6923
U+66A2.0	fBig5	BA5A
U+66A2.0	fGB	1-3329
U+66A2.0	fR/S	 72.10
U+66A2.0	fTotalStrokes	 14
U+66A2.0	fCangjie	LLAMH
U+66A2.0	fFourCorner	56027
U+66A2.0	fHakka	 cong3;  chong3; t'iong5; chong6
U+66A2.0	fCantonese	coeng3
U+66A2.0	fMandarin	chang4
U+66A2.0	fEnglish	[1] smoothly; fluently [2] easily accessible [3] freely; unrestrained; clear (to state or elaborate) [4] with gusto; to one's heart's content [5] long; expanding [6] luxuriant; luxuriance [7] very [8] a Chinese family name
U+66A3.0	fUTF8	暣
U+66A3.0	fCNS11643	4-436D
U+66A3.0	fGB	3-4750
U+66A3.0	fR/S	 72.10
U+66A3.0	fTotalStrokes	 14
U+66A3.0	fCangjie	AOND
U+66A3.0	fFourCorner	68010
U+66A3.0	fCantonese	hei3
U+66A3.0	fMandarin	qi4
U+66A4.0	fUTF8	暤
U+66A4.0	fCNS11643	3-4739
U+66A4.0	fGB	5-4A22
U+66A4.0	fR/S	  72.10
U+66A4.0	fTotalStrokes	 14
U+66A4.0	fCangjie	AHAJ
U+66A4.0	fFourCorner	66043
U+66A4.0	fCantonese	hou6
U+66A4.0	fMandarin	hao4
U+66A5.0	fUTF8	暥
U+66A5.0	fCNS11643	3-4733
U+66A5.0	fGB	5-4A21
U+66A5.0	fR/S	  72.10
U+66A5.0	fTotalStrokes	 14
U+66A5.0	fCangjie	AAJV
U+66A5.0	fFourCorner	66040
U+66A5.0	fCantonese	aan3; ngaan3 (also)
U+66A5.0	fMandarin	yan4
U+66A6.0	fUTF8	暦
U+66A6.0	fCNS11643	3-4737
U+66A6.0	fGB	E-2C75
U+66A6.0	fR/S	  72.10
U+66A6.0	fTotalStrokes	 14
U+66A6.0	fCangjie	MDDA
U+66A6.0	fFourCorner	17269
U+66A6.0	fHakka	 lak8; let8; lit8
U+66A6.0	fCantonese	lik6
U+66A6.0	fMandarin	li4
U+66A6.0	fEnglish	calendar, era
U+66A7.0	fUTF8	暧
U+66A7.0	fGB	0-6A53
U+66A7.0	fR/S	 72.10
U+66A7.0	fTotalStrokes	 14
U+66A7.0	fCangjie	XABBE
U+66A7.0	fHakka	 oi5
U+66A7.0	fCantonese	oi3; ngoi3 (also); oi2; ngoi2 (also)
U+66A7.0	fMandarin	ai4
U+66A7.0	fEnglish	simplified form of &#x66D6;
U+66A8.0	fUTF8	暨
U+66A8.0	fCNS11643	1-6924
U+66A8.0	fBig5	BA5B
U+66A8.0	fGB	0-745F
U+66A8.0	fR/S	 72.10
U+66A8.0	fTotalStrokes	 14
U+66A8.0	fCangjie	AUAM
U+66A8.0	fFourCorner	21106
U+66A8.0	fHakka	 gi5; kai3
U+66A8.0	fCantonese	kei3; gei6
U+66A8.0	fMandarin	ji4; jie4
U+66A8.0	fEnglish	 and attain; reach confines
U+66A9.0	fUTF8	暩
U+66A9.0	fCNS11643	2-506C
U+66A9.0	fBig5	E5C5
U+66A9.0	fGB	E-2C76
U+66A9.0	fR/S	 72.11
U+66A9.0	fTotalStrokes	 15
U+66A9.0	fCangjie	ABOF
U+66A9.0	fFourCorner	87091
U+66A9.0	fCantonese	zai3; gwai3
U+66A9.0	fMandarin	gui4
U+66AA.0	fUTF8	暪
U+66AA.0	fCNS11643	2-506F
U+66AA.0	fBig5	E5C8
U+66AA.0	fGB	5-497B
U+66AA.0	fR/S	 72.11
U+66AA.0	fTotalStrokes	 15
U+66AA.0	fCangjie	ATLB
U+66AA.0	fFourCorner	64027
U+66AA.0	fHakka	 man2
U+66AA.0	fCantonese	mun5
U+66AA.0	fMandarin	men3
U+66AB.0	fUTF8	暫
U+66AB.0	fCNS11643	1-6D32
U+66AB.0	fBig5	BCC8
U+66AB.0	fGB	1-545D
U+66AB.0	fR/S	 72.11
U+66AB.0	fTotalStrokes	 15
U+66AB.0	fCangjie	JLA
U+66AB.0	fFourCorner	52602
U+66AB.0	fHakka	 ciam5; ciam3; ciam6
U+66AB.0	fCantonese	zaam6
U+66AB.0	fMandarin	zan4
U+66AB.0	fEnglish	[1] temporary; for a short time [2] suddenly; abruptly
U+66AC.0	fUTF8	暬
U+66AC.0	fCNS11643	3-4C44
U+66AC.0	fGB	3-4829
U+66AC.0	fR/S	 72.11
U+66AC.0	fTotalStrokes	 15
U+66AC.0	fCangjie	GIA
U+66AC.0	fFourCorner	45600
U+66AC.0	fCantonese	sit3
U+66AC.0	fMandarin	xie4
U+66AD.0	fUTF8	暭
U+66AD.0	fCNS11643	3-4C46
U+66AD.0	fGB	E-2C77
U+66AD.0	fR/S	 72.11
U+66AD.0	fTotalStrokes	 15
U+66AD.0	fCangjie	AHAJ
U+66AD.0	fFourCorner	66040
U+66AD.0	fCantonese	hou6
U+66AD.0	fMandarin	hao4; yi4
U+66AE.0	fUTF8	暮
U+66AE.0	fCNS11643	1-6D31
U+66AE.0	fBig5	BCC7
U+66AE.0	fGB	0-443A
U+66AE.0	fR/S	 72.11
U+66AE.0	fTotalStrokes	 15
U+66AE.0	fCangjie	TAKA
U+66AE.0	fFourCorner	44603
U+66AE.0	fHakka	  mu5
U+66AE.0	fCantonese	mou6
U+66AE.0	fMandarin	mu4
U+66AE.0	fEnglish	[1] evening; dusk; sunset [2] ending; closing (years); late
U+66AF.0	fUTF8	暯
U+66AF.0	fCNS11643	2-5070
U+66AF.0	fBig5	E5C9
U+66AF.0	fGB	5-497C
U+66AF.0	fR/S	 72.11
U+66AF.0	fTotalStrokes	 15
U+66AF.0	fCangjie	ATAK
U+66AF.0	fFourCorner	64034
U+66AF.0	fHakka	 mok8; mok7
U+66AF.0	fCantonese	mok6
U+66AF.0	fMandarin	mo4
U+66AF.0	fEnglish	dark; obscure
U+66B0.0	fUTF8	暰
U+66B0.0	fCNS11643	2-506B
U+66B0.0	fBig5	E5C4
U+66B0.0	fGB	E-2C78
U+66B0.0	fR/S	 72.11
U+66B0.0	fTotalStrokes	 15
U+66B0.0	fCangjie	AHOO
U+66B0.0	fFourCorner	68081
U+66B0.0	fCantonese	cung1
U+66B0.0	fMandarin	cong1
U+66B1.0	fUTF8	暱
U+66B1.0	fCNS11643	1-6D34
U+66B1.0	fBig5	BCCA
U+66B1.0	fGB	E-2C79
U+66B1.0	fR/S	 72.11
U+66B1.0	fTotalStrokes	 15
U+66B1.0	fCangjie	ASTR
U+66B1.0	fFourCorner	61016
U+66B1.0	fHakka	  nit8; lit8; lit7
U+66B1.0	fCantonese	nik1; lik1 (also); nik6
U+66B1.0	fMandarin	ni4; ri4
U+66B1.0	fEnglish	intimate; close
U+66B2.0	fUTF8	暲
U+66B2.0	fCNS11643	2-506D
U+66B2.0	fBig5	E5C6
U+66B2.0	fGB	5-4A27
U+66B2.0	fR/S	 72.11
U+66B2.0	fTotalStrokes	 15
U+66B2.0	fCangjie	AYTJ
U+66B2.0	fFourCorner	60046
U+66B2.0	fCantonese	zoeng1
U+66B2.0	fMandarin	zhang1
U+66B3.0	fUTF8	暳
U+66B3.0	fCNS11643	3-4C41
U+66B3.0	fGB	5-4A26
U+66B3.0	fR/S	  72.11
U+66B3.0	fTotalStrokes	 15
U+66B3.0	fCangjie	AQJM
U+66B3.0	fFourCorner	65077
U+66B3.0	fHakka	 fui5; fi5
U+66B3.0	fCantonese	wai3
U+66B3.0	fMandarin	hui4
U+66B3.0	fEnglish	fine; minute
U+66B4.0	fUTF8	暴
U+66B4.0	fCNS11643	1-6D33
U+66B4.0	fBig5	BCC9
U+66B4.0	fGB	0-3129
U+66B4.0	fR/S	 72.11
U+66B4.0	fTotalStrokes	 15
U+66B4.0	fCangjie	ATCE
U+66B4.0	fFourCorner	60132
U+66B4.0	fHakka	  pau5;  bau5;  pok7
U+66B4.0	fCantonese	bou6; buk6; bou2 (also); bok3
U+66B4.0	fMandarin	bao4; bo2; bo1
U+66B4.0	fEnglish	[1] cruel; savage; fierce; atrocious; violent [2] sudden [3] short-tempered; hot-tempered
U+66B4.1	fUTF8	暴
U+66B4.1	fCNS11643	1-6D33
U+66B4.1	fBig5	BCC9
U+66B4.1	fGB	0-3129
U+66B4.1	fR/S	 72.11
U+66B4.1	fTotalStrokes	 15
U+66B4.1	fCangjie	ATCE
U+66B4.1	fFourCorner	60132
U+66B4.1	fMandarin	pu4
U+66B4.1	fEnglish	 expose
U+66B5.0	fUTF8	暵
U+66B5.0	fCNS11643	2-506A
U+66B5.0	fBig5	E5C3
U+66B5.0	fGB	3-482A
U+66B5.0	fR/S	 72.11
U+66B5.0	fTotalStrokes	 15
U+66B5.0	fCangjie	ATLO
U+66B5.0	fFourCorner	64034
U+66B5.0	fHakka	 hon5; hon3
U+66B5.0	fCantonese	hon3; hon2
U+66B5.0	fMandarin	han4; han3
U+66B5.0	fEnglish	 dry; expose to sunshine; be scorched up; withered; wizened
U+66B6.0	fUTF8	暶
U+66B6.0	fCNS11643	3-4C40
U+66B6.0	fGB	3-482C
U+66B6.0	fR/S	 72.11
U+66B6.0	fTotalStrokes	 15
U+66B6.0	fCangjie	AYSO
U+66B6.0	fFourCorner	68080
U+66B6.0	fCantonese	syun4
U+66B6.0	fMandarin	xuan2
U+66B7.0	fUTF8	暷
U+66B7.0	fCNS11643	2-506E
U+66B7.0	fBig5	E5C7
U+66B7.0	fGB	5-4951
U+66B7.0	fR/S	 72.11
U+66B7.0	fTotalStrokes	 15
U+66B7.0	fCangjie	AJII
U+66B7.0	fFourCorner	65043
U+66B7.0	fCantonese	cyun4
U+66B7.0	fMandarin	chuan2
U+66B8.0	fUTF8	暸
U+66B8.0	fCNS11643	1-7072
U+66B8.0	fBig5	BEE9
U+66B8.0	fGB	5-4A2B
U+66B8.0	fR/S	 72.12
U+66B8.0	fTotalStrokes	 16
U+66B8.0	fCangjie	AKCF
U+66B8.0	fFourCorner	64096
U+66B8.0	fHakka	 liau2; liau3
U+66B8.0	fCantonese	liu4; liu5
U+66B8.0	fMandarin	liao3; liao2
U+66B8.0	fEnglish	bright; clear
U+66B9.0	fUTF8	暹
U+66B9.0	fCNS11643	1-706F
U+66B9.0	fBig5	BEE6
U+66B9.0	fGB	0-655F
U+66B9.0	fR/S	 72.12
U+66B9.0	fTotalStrokes	 16
U+66B9.0	fCangjie	YAOG
U+66B9.0	fFourCorner	36301
U+66B9.0	fHakka	  siam2; siam1
U+66B9.0	fCantonese	cim3; cim1 (also)
U+66B9.0	fMandarin	xian1
U+66B9.0	fEnglish	 rise (said of the sun)
U+66BA.0	fUTF8	暺
U+66BA.0	fCNS11643	2-5746
U+66BA.0	fBig5	E9BB
U+66BA.0	fGB	5-4972
U+66BA.0	fR/S	 72.12
U+66BA.0	fTotalStrokes	 16
U+66BA.0	fCangjie	ARRJ
U+66BA.0	fFourCorner	66046
U+66BA.0	fCantonese	taan2; daan6
U+66BA.0	fMandarin	dan4
U+66BB.0	fUTF8	暻
U+66BB.0	fCNS11643	2-5745
U+66BB.0	fBig5	E9BA
U+66BB.0	fGB	5-4A2C
U+66BB.0	fR/S	 72.12
U+66BB.0	fTotalStrokes	 16
U+66BB.0	fCangjie	AAYF
U+66BB.0	fFourCorner	66096
U+66BB.0	fCantonese	ging2
U+66BB.0	fMandarin	jing3; liao2
U+66BB.0	fEnglish	bright
U+66BC.0	fUTF8	暼
U+66BC.0	fCNS11643	3-4C43
U+66BC.0	fGB	E-2C7A
U+66BC.0	fR/S	 72.12; 72.11
U+66BC.0	fTotalStrokes	 16; 15
U+66BC.0	fCangjie	FKA
U+66BC.0	fFourCorner	98600
U+66BC.0	fCantonese	pit3
U+66BC.0	fMandarin	pie1
U+66BC.0	fEnglish	take fleeting glance at
U+66BD.0	fUTF8	暽
U+66BD.0	fCNS11643	2-5744
U+66BD.0	fBig5	E9B9
U+66BD.0	fGB	5-4A2E
U+66BD.0	fR/S	 72.12
U+66BD.0	fTotalStrokes	 16
U+66BD.0	fCangjie	AFDQ
U+66BD.0	fFourCorner	69059
U+66BD.0	fCantonese	leon4
U+66BD.0	fMandarin	lin2
U+66BE.0	fUTF8	暾
U+66BE.0	fCNS11643	2-573F
U+66BE.0	fBig5	E9B4
U+66BE.0	fGB	0-6A55
U+66BE.0	fR/S	 72.12
U+66BE.0	fTotalStrokes	 16
U+66BE.0	fCangjie	AYDK
U+66BE.0	fFourCorner	68040
U+66BE.0	fHakka	 dun1
U+66BE.0	fCantonese	tan1
U+66BE.0	fMandarin	tun1
U+66BE.0	fEnglish	sunrise
U+66BF.0	fUTF8	暿
U+66BF.0	fCNS11643	3-507D
U+66BF.0	fGB	5-4A28
U+66BF.0	fR/S	  72.12
U+66BF.0	fTotalStrokes	 16
U+66BF.0	fCangjie	AGRR
U+66BF.0	fFourCorner	64060
U+66BF.0	fCantonese	hei2
U+66BF.0	fMandarin	xi3
U+66C0.0	fUTF8	曀
U+66C0.0	fCNS11643	2-5740
U+66C0.0	fBig5	E9B5
U+66C0.0	fGB	3-482E
U+66C0.0	fR/S	 72.12
U+66C0.0	fTotalStrokes	 16
U+66C0.0	fCangjie	AGBT
U+66C0.0	fFourCorner	64018
U+66C0.0	fHakka	 ji5; ri5; i5
U+66C0.0	fCantonese	ai3; ngai3 (also)
U+66C0.0	fMandarin	yi4; yi1; ye1
U+66C0.0	fEnglish	obscure; misty; dim; cloudy
U+66C1.0	fUTF8	曁
U+66C1.0	fCNS11643	3-5123
U+66C1.0	fGB	E-2C7B
U+66C1.0	fR/S	 72.12
U+66C1.0	fTotalStrokes	 16
U+66C1.0	fCangjie	HUAM
U+66C1.0	fFourCorner	21100
U+66C1.0	fHakka	  kai3; koi2; gi5; koi3
U+66C1.0	fCantonese	kei3
U+66C1.0	fMandarin	ji4; jie4
U+66C1.0	fEnglish	[1] and [2] attain; reach; overtake [3] up to; till
U+66C2.0	fUTF8	曂
U+66C2.0	fCNS11643	4-5050
U+66C2.0	fGB	E-2C7C
U+66C2.0	fR/S	  72.12
U+66C2.0	fTotalStrokes	 16
U+66C2.0	fCangjie	ATLC
U+66C2.0	fFourCorner	64080
U+66C2.0	fCantonese	wong6
U+66C2.0	fMandarin	huang4
U+66C3.0	fUTF8	曃
U+66C3.0	fCNS11643	3-507A
U+66C3.0	fGB	3-482D
U+66C3.0	fR/S	  72.12
U+66C3.0	fTotalStrokes	 16
U+66C3.0	fCangjie	AYLE
U+66C3.0	fFourCorner	65030
U+66C3.0	fCantonese	doi6
U+66C3.0	fMandarin	tai4; dai4
U+66C4.0	fUTF8	曄
U+66C4.0	fCNS11643	1-7070
U+66C4.0	fBig5	BEE7
U+66C4.0	fGB	1-6A4A
U+66C4.0	fR/S	 72.12
U+66C4.0	fTotalStrokes	 16
U+66C4.0	fCangjie	ATMJ
U+66C4.0	fFourCorner	64054
U+66C4.0	fHakka	 jap7; ngiap7; rap7; sap7
U+66C4.0	fCantonese	jip6
U+66C4.0	fMandarin	ye4; yi4
U+66C4.0	fEnglish	[1] bright; radiant [2] prosperous; thriving
U+66C5.0	fUTF8	曅
U+66C5.0	fCNS11643	3-507E
U+66C5.0	fGB	5-4964
U+66C5.0	fR/S	 72.12
U+66C5.0	fTotalStrokes	 16
U+66C5.0	fCangjie	XATMJ
U+66C5.0	fFourCorner	60500
U+66C5.0	fHakka	 jap7; sap7
U+66C5.0	fCantonese	jip6
U+66C5.0	fMandarin	ye4; yi4
U+66C5.0	fEnglish	same as &#x66C4;
U+66C6.0	fUTF8	曆
U+66C6.0	fCNS11643	1-706D
U+66C6.0	fBig5	BEE4
U+66C6.0	fGB	1-784F
U+66C6.0	fR/S	 72.12
U+66C6.0	fTotalStrokes	 16
U+66C6.0	fCangjie	MHDA
U+66C6.0	fFourCorner	71269
U+66C6.0	fHakka	  lak8;  lit8;  let8
U+66C6.0	fCantonese	lik6
U+66C6.0	fMandarin	li4
U+66C6.0	fEnglish	[1] calendar [2] age; era [3] calculate; count
U+66C7.0	fUTF8	曇
U+66C7.0	fCNS11643	1-7071
U+66C7.0	fBig5	BEE8
U+66C7.0	fGB	1-6A3C
U+66C7.0	fR/S	 72.12
U+66C7.0	fTotalStrokes	 16
U+66C7.0	fCangjie	AMBI
U+66C7.0	fFourCorner	60731
U+66C7.0	fHakka	 tam2
U+66C7.0	fCantonese	taam4
U+66C7.0	fMandarin	tan2
U+66C7.0	fEnglish	clouds; overcast
U+66C8.0	fUTF8	曈
U+66C8.0	fCNS11643	2-573E
U+66C8.0	fBig5	E9B3
U+66C8.0	fGB	3-4831
U+66C8.0	fR/S	 72.12
U+66C8.0	fTotalStrokes	 16
U+66C8.0	fCangjie	AYTG
U+66C8.0	fFourCorner	60014
U+66C8.0	fHakka	 tung2
U+66C8.0	fCantonese	tung4
U+66C8.0	fMandarin	tong2; tong1; tong3
U+66C8.0	fEnglish	twilight before sunrise
U+66C9.0	fUTF8	曉
U+66C9.0	fCNS11643	1-706E
U+66C9.0	fBig5	BEE5
U+66C9.0	fGB	1-4F7E
U+66C9.0	fR/S	 72.12
U+66C9.0	fTotalStrokes	 16
U+66C9.0	fCangjie	AGGU
U+66C9.0	fFourCorner	64011
U+66C9.0	fHakka	  hiau3; hieu3
U+66C9.0	fCantonese	hiu2
U+66C9.0	fMandarin	xiao3
U+66C9.0	fEnglish	[1] dawn; daybreak [2] tell; explain [3] know; understand
U+66CA.0	fUTF8	曊
U+66CA.0	fCNS11643	2-5741
U+66CA.0	fBig5	E9B6
U+66CA.0	fGB	5-4978
U+66CA.0	fR/S	 72.12
U+66CA.0	fTotalStrokes	 16
U+66CA.0	fCangjie	ALNC
U+66CA.0	fFourCorner	65086
U+66CA.0	fCantonese	fai3; fei1
U+66CA.0	fMandarin	fei4; fei1; fen4; fu2; po1
U+66CB.0	fUTF8	曋
U+66CB.0	fCNS11643	2-5742
U+66CB.0	fBig5	E9B7
U+66CB.0	fGB	5-4A2A
U+66CB.0	fR/S	 72.12
U+66CB.0	fTotalStrokes	 16
U+66CB.0	fCangjie	AMWJ
U+66CB.0	fFourCorner	61046
U+66CB.0	fCantonese	sam2; cam2
U+66CB.0	fMandarin	qin3; qiu3
U+66CC.0	fUTF8	曌
U+66CC.0	fCNS11643	2-5747
U+66CC.0	fBig5	E9BC
U+66CC.0	fGB	3-4830
U+66CC.0	fR/S	 72.12
U+66CC.0	fTotalStrokes	 16
U+66CC.0	fCangjie	ABJCM
U+66CC.0	fFourCorner	67101
U+66CC.0	fHakka	  zau5; zhau5; zeu5
U+66CC.0	fCantonese	ziu3
U+66CC.0	fMandarin	zhao4
U+66CC.0	fEnglish	same as &#x7167;
U+66CD.0	fUTF8	曍
U+66CD.0	fCNS11643	3-5122
U+66CD.0	fGB	E-2C7D
U+66CD.0	fR/S	  72.12
U+66CD.0	fTotalStrokes	 16
U+66CD.0	fCangjie	AHUJ
U+66CD.0	fFourCorner	66041
U+66CD.0	fCantonese	hou6
U+66CD.0	fMandarin	hao4
U+66CD.0	fEnglish	bright; brilliant
U+66CE.0	fUTF8	曎
U+66CE.0	fCNS11643	3-5471
U+66CE.0	fGB	E-2C7E
U+66CE.0	fR/S	 72.13
U+66CE.0	fTotalStrokes	 17
U+66CE.0	fCangjie	AWLJ
U+66CE.0	fFourCorner	66040
U+66CE.0	fCantonese	jik6
U+66CE.0	fMandarin	yi4
U+66CF.0	fUTF8	曏
U+66CF.0	fCNS11643	2-5743
U+66CF.0	fBig5	E9B8
U+66CF.0	fGB	E-2D21
U+66CF.0	fR/S	 72.13
U+66CF.0	fTotalStrokes	 17
U+66CF.0	fCangjie	AVHL
U+66CF.0	fFourCorner	60227
U+66CF.0	fHakka	 hiong5
U+66CF.0	fCantonese	hoeng3
U+66CF.0	fMandarin	xiang4; shang3; shang4; xiang3
U+66CF.0	fEnglish	[1] once upon time [2] period of time
U+66D0.0	fUTF8	曐
U+66D0.0	fCNS11643	3-546F
U+66D0.0	fGB	E-2D22
U+66D0.0	fR/S	 72.13
U+66D0.0	fTotalStrokes	 17
U+66D0.0	fCangjie	AAAM
U+66D0.0	fFourCorner	60100
U+66D0.0	fHakka	  sin1;  sen1; siang1; sang1
U+66D0.0	fCantonese	sing1; seng1
U+66D0.0	fMandarin	xing1
U+66D0.0	fEnglish	ancient form of &#x661F;
U+66D1.0	fUTF8	曑
U+66D1.0	fCNS11643	3-5470
U+66D1.0	fGB	E-2D23
U+66D1.0	fR/S	  72.13
U+66D1.0	fTotalStrokes	 17
U+66D1.0	fCangjie	AAAH
U+66D1.0	fFourCorner	60200
U+66D1.0	fCantonese	sam1
U+66D1.0	fMandarin	sen1
U+66D2.0	fUTF8	曒
U+66D2.0	fCNS11643	2-5C7D
U+66D2.0	fBig5	ECF2
U+66D2.0	fGB	3-4834
U+66D2.0	fR/S	 72.13
U+66D2.0	fTotalStrokes	 17
U+66D2.0	fCangjie	AHSK
U+66D2.0	fFourCorner	68040
U+66D2.0	fHakka	 gau3
U+66D2.0	fCantonese	giu2
U+66D2.0	fMandarin	jiao3; hao4
U+66D3.0	fUTF8	曓
U+66D3.0	fCNS11643	4-5641
U+66D3.0	fGB	E-2D24
U+66D3.0	fR/S	  72.13
U+66D3.0	fTotalStrokes	 17; 16
U+66D3.0	fCangjie	AUUJ
U+66D3.0	fFourCorner	60500
U+66D3.0	fCantonese	bou6
U+66D3.0	fMandarin	bao4; bo2
U+66D4.0	fUTF8	曔
U+66D4.0	fCNS11643	3-546C
U+66D4.0	fGB	3-482F
U+66D4.0	fR/S	  72.13
U+66D4.0	fTotalStrokes	 17
U+66D4.0	fCangjie	ATRK
U+66D4.0	fFourCorner	68040
U+66D4.0	fCantonese	ging6
U+66D4.0	fMandarin	jing4
U+66D5.0	fUTF8	曕
U+66D5.0	fCNS11643	3-546D
U+66D5.0	fGB	E-2D25
U+66D5.0	fR/S	  72.13
U+66D5.0	fTotalStrokes	 17
U+66D5.0	fCangjie	ANCR
U+66D5.0	fFourCorner	67060
U+66D5.0	fCantonese	jim6
U+66D5.0	fMandarin	yan4
U+66D5.0	fEnglish	dry or cook in the sun
U+66D6.0	fUTF8	曖
U+66D6.0	fCNS11643	1-7370
U+66D6.0	fBig5	C0C7
U+66D6.0	fGB	1-6A53
U+66D6.0	fR/S	 72.13
U+66D6.0	fTotalStrokes	 17
U+66D6.0	fCangjie	ABBE
U+66D6.0	fFourCorner	62047
U+66D6.0	fHakka	 oi5
U+66D6.0	fCantonese	oi3; oi2; ngoi3 (also); ngoi2 (also)
U+66D6.0	fMandarin	ai4
U+66D6.0	fEnglish	[1] obscure; dim; indistinct; ambiguous; vague [2] th sun hidden by clouds
U+66D7.0	fUTF8	曗
U+66D7.0	fCNS11643	3-5472
U+66D7.0	fGB	E-2D26
U+66D7.0	fR/S	 72.13
U+66D7.0	fTotalStrokes	 17
U+66D7.0	fCangjie	ATCD
U+66D7.0	fFourCorner	63090
U+66D7.0	fHakka	 jap7; ngiap7; rap7; sap7
U+66D7.0	fCantonese	jip6
U+66D7.0	fMandarin	ye4; yi4
U+66D7.0	fEnglish	same as &#x66C4;
U+66D8.0	fUTF8	曘
U+66D8.0	fCNS11643	2-6168
U+66D8.0	fBig5	EFDC
U+66D8.0	fGB	3-4835
U+66D8.0	fR/S	 72.14
U+66D8.0	fTotalStrokes	 18
U+66D8.0	fCangjie	AMBB
U+66D8.0	fFourCorner	61027
U+66D8.0	fCantonese	jyu4
U+66D8.0	fMandarin	ru2; liu3
U+66D9.0	fUTF8	曙
U+66D9.0	fCNS11643	1-736F
U+66D9.0	fBig5	C0C6
U+66D9.0	fGB	0-4A6F
U+66D9.0	fR/S	 72.14
U+66D9.0	fTotalStrokes	 18
U+66D9.0	fCangjie	AWLA
U+66D9.0	fFourCorner	66064
U+66D9.0	fHakka	 shu5; su3; shu3; su2; zu5; zhu6; zhu5
U+66D9.0	fCantonese	cyu5; syu5 (also); syu6
U+66D9.0	fMandarin	shu3; shu4
U+66D9.0	fEnglish	dawn; daybreak
U+66DA.0	fUTF8	曚
U+66DA.0	fCNS11643	2-6166
U+66DA.0	fBig5	EFDA
U+66DA.0	fGB	5-4A31
U+66DA.0	fR/S	 72.13; 72.14
U+66DA.0	fTotalStrokes	 17
U+66DA.0	fCangjie	ATBO
U+66DA.0	fFourCorner	64032
U+66DA.0	fHakka	 mung2; mung3
U+66DA.0	fCantonese	mung4; mung1 (also)
U+66DA.0	fMandarin	meng2
U+66DA.0	fEnglish	dim obscure
U+66DB.0	fUTF8	曛
U+66DB.0	fCNS11643	2-6167
U+66DB.0	fBig5	EFDB
U+66DB.0	fGB	0-6A56
U+66DB.0	fR/S	 72.14
U+66DB.0	fTotalStrokes	 18
U+66DB.0	fCangjie	AHGF
U+66DB.0	fFourCorner	62031
U+66DB.0	fHakka	 hiun1
U+66DB.0	fCantonese	fan1
U+66DB.0	fMandarin	xun1
U+66DB.0	fEnglish	[1] twilight; sunset [2] dim glow of the setting sun
U+66DC.0	fUTF8	曜
U+66DC.0	fCNS11643	1-764C
U+66DC.0	fBig5	C260
U+66DC.0	fGB	0-6A57
U+66DC.0	fR/S	 72.14
U+66DC.0	fTotalStrokes	 18
U+66DC.0	fCangjie	ASMG
U+66DC.0	fFourCorner	67014
U+66DC.0	fHakka	  jau5; rau5; jau3; jeu5
U+66DC.0	fCantonese	jiu6
U+66DC.0	fMandarin	yao4; yue4
U+66DC.0	fEnglish	[1] sunshine; daylight [2] shine; glisten
U+66DD.0	fUTF8	曝
U+66DD.0	fCNS11643	1-783B
U+66DD.0	fBig5	C36E
U+66DD.0	fGB	0-4658
U+66DD.0	fR/S	 72.15
U+66DD.0	fTotalStrokes	 19
U+66DD.0	fCangjie	AATE
U+66DD.0	fFourCorner	66032
U+66DD.0	fHakka	  pau5;  pok7
U+66DD.0	fCantonese	buk6; bou6
U+66DD.0	fMandarin	pu4; bao4
U+66DD.0	fEnglish	 sun; expose to sunlight
U+66DE.0	fUTF8	曞
U+66DE.0	fCNS11643	2-6556
U+66DE.0	fBig5	F24B
U+66DE.0	fGB	E-2D27
U+66DE.0	fR/S	 72.15
U+66DE.0	fTotalStrokes	 19
U+66DE.0	fCangjie	AMTB
U+66DE.0	fFourCorner	61027
U+66DE.0	fCantonese	lai6
U+66DE.0	fMandarin	li4
U+66DF.0	fUTF8	曟
U+66DF.0	fCNS11643	3-5A43
U+66DF.0	fGB	E-2D28
U+66DF.0	fR/S	  72.15
U+66DF.0	fTotalStrokes	 19
U+66DF.0	fCangjie	AAAV
U+66DF.0	fFourCorner	60730
U+66DF.0	fHakka	 sin2; shin2; siin2; shin3; sen2
U+66DF.0	fCantonese	san4
U+66DF.0	fMandarin	chen2; shen2
U+66DF.0	fEnglish	same as &#x6668;
U+66E0.0	fUTF8	曠
U+66E0.0	fCNS11643	1-783A
U+66E0.0	fBig5	C36D
U+66E0.0	fGB	1-3F75
U+66E0.0	fR/S	 72.15
U+66E0.0	fTotalStrokes	 19
U+66E0.0	fCangjie	AITC
U+66E0.0	fFourCorner	60086
U+66E0.0	fHakka	  kong3; kong5; gwong3; k'wong3
U+66E0.0	fCantonese	kwong3; kong3 (also)
U+66E0.0	fMandarin	kuang4
U+66E0.0	fEnglish	[1] extensive; wide; broad; empty; open; vast; spacious; unoccupied [2] free from worries and petty ideas [3] neglect
U+66E1.0	fUTF8	曡
U+66E1.0	fCNS11643	3-5A42
U+66E1.0	fGB	E-2D29
U+66E1.0	fR/S	 72.15; 72.16
U+66E1.0	fTotalStrokes	 19; 20
U+66E1.0	fCangjie	AAAM
U+66E1.0	fFourCorner	60100
U+66E1.0	fCantonese	dip6; daap6 (also); tip6
U+66E1.0	fMandarin	die2
U+66E1.0	fEnglish	 repeat; duplicate repititious
U+66E2.0	fUTF8	曢
U+66E2.0	fCNS11643	3-6526
U+66E2.0	fGB	E-2D2A
U+66E2.0	fR/S	 72.15
U+66E2.0	fTotalStrokes	 19
U+66E2.0	fCangjie	AJKF
U+66E2.0	fFourCorner	6309
U+66E2.0	fCantonese	liu4
U+66E3.0	fUTF8	曣
U+66E3.0	fCNS11643	2-687D
U+66E3.0	fBig5	F451
U+66E3.0	fGB	3-4837
U+66E3.0	fR/S	 72.16
U+66E3.0	fTotalStrokes	 20
U+66E3.0	fCangjie	ATLF
U+66E3.0	fFourCorner	64031
U+66E3.0	fCantonese	jin3
U+66E3.0	fMandarin	yan4
U+66E4.0	fUTF8	曤
U+66E4.0	fCNS11643	2-687E
U+66E4.0	fBig5	F452
U+66E4.0	fGB	E-2D2B
U+66E4.0	fR/S	 72.16
U+66E4.0	fTotalStrokes	 20
U+66E4.0	fCangjie	AMBG
U+66E4.0	fFourCorner	61014
U+66E4.0	fCantonese	fok3
U+66E4.0	fMandarin	huo4
U+66E5.0	fUTF8	曥
U+66E5.0	fCNS11643	3-5C42
U+66E5.0	fGB	3-475B
U+66E5.0	fR/S	  72.16
U+66E5.0	fTotalStrokes	 20
U+66E5.0	fCangjie	AYPT
U+66E5.0	fFourCorner	61010
U+66E5.0	fCantonese	lou4
U+66E5.0	fMandarin	lu2; luu2
U+66E6.0	fUTF8	曦
U+66E6.0	fCNS11643	1-7971
U+66E6.0	fBig5	C466
U+66E6.0	fGB	0-6A58
U+66E6.0	fR/S	 72.16
U+66E6.0	fTotalStrokes	 20
U+66E6.0	fCangjie	ATGS
U+66E6.0	fFourCorner	68053
U+66E6.0	fHakka	 hi1
U+66E6.0	fCantonese	hei1
U+66E6.0	fMandarin	xi1
U+66E6.0	fEnglish	sunlight; sunshine
U+66E7.0	fUTF8	曧
U+66E7.0	fCNS11643	3-5C43
U+66E7.0	fGB	E-2D2C
U+66E7.0	fR/S	  72.16
U+66E7.0	fTotalStrokes	 20
U+66E7.0	fCangjie	AMBI
U+66E7.0	fFourCorner	60230
U+66E7.0	fCantonese	jung4
U+66E7.0	fMandarin	rong2
U+66E8.0	fUTF8	曨
U+66E8.0	fCNS11643	2-687C
U+66E8.0	fBig5	F450
U+66E8.0	fGB	3-475A
U+66E8.0	fR/S	 72.16
U+66E8.0	fTotalStrokes	 20
U+66E8.0	fCangjie	AYBP
U+66E8.0	fFourCorner	61011
U+66E8.0	fHakka	 lung2; lun3
U+66E8.0	fCantonese	lung4
U+66E8.0	fMandarin	long2; long3
U+66E8.0	fEnglish	[1] vague; dim [2] bright
U+66E9.0	fUTF8	曩
U+66E9.0	fCNS11643	1-7A6F
U+66E9.0	fBig5	C4E4
U+66E9.0	fGB	0-6A59
U+66E9.0	fR/S	 72.17
U+66E9.0	fTotalStrokes	 21
U+66E9.0	fCangjie	AYRV
U+66E9.0	fFourCorner	60732
U+66E9.0	fHakka	  nong3;  siong1; long2; long3
U+66E9.0	fCantonese	nong5; long5 (also)
U+66E9.0	fMandarin	nang3
U+66E9.0	fEnglish	in ancient times; of old; former; past
U+66EA.0	fUTF8	曪
U+66EA.0	fCNS11643	4-6A4C
U+66EA.0	fGB	E-2D2D
U+66EA.0	fR/S	  72.19
U+66EA.0	fTotalStrokes	 23
U+66EA.0	fCangjie	AWLG
U+66EA.0	fFourCorner	66010
U+66EA.0	fCantonese	lo2
U+66EA.0	fMandarin	luo3
U+66EB.0	fUTF8	曫
U+66EB.0	fCNS11643	2-6F2E
U+66EB.0	fBig5	F7DF
U+66EB.0	fGB	E-2D2E
U+66EB.0	fR/S	 72.19
U+66EB.0	fTotalStrokes	 23
U+66EB.0	fCangjie	VFA
U+66EB.0	fFourCorner	22603
U+66EB.0	fCantonese	lyun4
U+66EB.0	fMandarin	luan2; lian4; man2
U+66EC.0	fUTF8	曬
U+66EC.0	fCNS11643	1-7C3A
U+66EC.0	fBig5	C5CE
U+66EC.0	fGB	1-4939
U+66EC.0	fR/S	 72.19
U+66EC.0	fTotalStrokes	 23
U+66EC.0	fCangjie	AMMP
U+66EC.0	fFourCorner	67011
U+66EC.0	fHakka	  sai5
U+66EC.0	fCantonese	saai3
U+66EC.0	fMandarin	shai4; chi1; sha4; shi4
U+66EC.0	fEnglish	 dry in the sun; expose to sunlight
U+66ED.0	fUTF8	曭
U+66ED.0	fCNS11643	2-7038
U+66ED.0	fBig5	F8AA
U+66ED.0	fGB	3-4826
U+66ED.0	fR/S	 72.20
U+66ED.0	fTotalStrokes	 24
U+66ED.0	fCangjie	AFBF
U+66ED.0	fFourCorner	69031
U+66ED.0	fHakka	  dong3; tong3
U+66ED.0	fCantonese	tong2; dong2
U+66ED.0	fMandarin	tang3
U+66ED.0	fEnglish	dark; cloudy
U+66EE.0	fUTF8	曮
U+66EE.0	fCNS11643	2-7039
U+66EE.0	fBig5	F8AB
U+66EE.0	fGB	3-4769
U+66EE.0	fR/S	 72.20
U+66EE.0	fTotalStrokes	 24
U+66EE.0	fCangjie	ARRK
U+66EE.0	fFourCorner	66048
U+66EE.0	fCantonese	jim5
U+66EE.0	fMandarin	yan3
U+66EF.0	fUTF8	曯
U+66EF.0	fCNS11643	3-613F
U+66EF.0	fGB	E-2D2F
U+66EF.0	fR/S	 72.21
U+66EF.0	fTotalStrokes	 25
U+66EF.0	fCangjie	ASEI
U+66EF.0	fFourCorner	67020
U+66EF.0	fCantonese	zuk1
U+66EF.0	fMandarin	zhu2; chu2
U+66F0.0	fUTF8	曰
U+66F0.0	fCNS11643	1-454B
U+66F0.0	fBig5	A4EA
U+66F0.0	fGB	0-543B
U+66F0.0	fR/S	 73.0
U+66F0.0	fTotalStrokes	 4
U+66F0.0	fCangjie	A
U+66F0.0	fFourCorner	60100
U+66F0.0	fHakka	 jet8; jat7; jet7; ret7
U+66F0.0	fCantonese	joek6; jyut6 (also); koek6
U+66F0.0	fMandarin	yue1
U+66F0.0	fEnglish	[1] say [2] call; name [3] Kangxi radical 73
U+66F1.0	fUTF8	曱
U+66F1.0	fCNS11643	4-217C
U+66F1.0	fGB	1-7C21
U+66F1.0	fR/S	 73.1; 102.0
U+66F1.0	fTotalStrokes	 5
U+66F1.0	fCangjie	WMLL
U+66F1.0	fFourCorner	60400
U+66F1.0	fHakka	 gat7
U+66F1.0	fCantonese	gaat6; gat6 (also); gaak6; gaap3; zaat6
U+66F1.0	fMandarin	ya1; yue1
U+66F1.0	fEnglish	[1] [2] cockroach [3] cockroach (from Cantonese)
U+66F2.0	fUTF8	曲
U+66F2.0	fCNS11643	1-4832
U+66F2.0	fBig5	A6B1
U+66F2.0	fGB	0-477A
U+66F2.0	fR/S	 73.2
U+66F2.0	fTotalStrokes	 6
U+66F2.0	fCangjie	TW
U+66F2.0	fFourCorner	55600
U+66F2.0	fHakka	  kiuk7; kiut7
U+66F2.0	fCantonese	kuk1; guk1 (also)
U+66F2.0	fMandarin	qu1
U+66F2.0	fEnglish	[1] crooked; bent; twisted; winding [2] wrong; false; unjustifiable [3] little known; obscure
U+66F2.1	fUTF8	曲
U+66F2.1	fCNS11643	1-4832
U+66F2.1	fBig5	A6B1
U+66F2.1	fGB	0-477A
U+66F2.1	fR/S	 73.2
U+66F2.1	fTotalStrokes	 6
U+66F2.1	fCangjie	TW
U+66F2.1	fFourCorner	55600
U+66F2.1	fMandarin	qu3
U+66F2.1	fEnglish	[1] a type of verse for singing, which emerged in the Southern Sung and Kin dynasties and became popular in the Yuan dynasty [2] piece of music; song
U+66F3.0	fUTF8	曳
U+66F3.0	fCNS11643	1-4833
U+66F3.0	fBig5	A6B2
U+66F3.0	fGB	0-5237
U+66F3.0	fR/S	 73.2
U+66F3.0	fTotalStrokes	 6
U+66F3.0	fCangjie	LWP
U+66F3.0	fFourCorner	50006
U+66F3.0	fHakka	 ji3; ri3; i5
U+66F3.0	fCantonese	jai6; jai5
U+66F3.0	fMandarin	ye4; yi4
U+66F3.0	fEnglish	 haul; tug; drag; trail
U+66F4.0	fUTF8	更
U+66F4.0	fCNS11643	1-4A55
U+66F4.0	fBig5	A7F3
U+66F4.0	fGB	0-387C
U+66F4.0	fR/S	 73.3
U+66F4.0	fTotalStrokes	 7
U+66F4.0	fCangjie	MLWK
U+66F4.0	fFourCorner	10506
U+66F4.0	fCantonese	gaang1; ang1
U+66F4.0	fMandarin	geng4
U+66F4.0	fEnglish	more; still further; much more; to a greater degree; further
U+66F4.1	fUTF8	更
U+66F4.1	fCNS11643	1-4A55
U+66F4.1	fBig5	A7F3
U+66F4.1	fGB	0-387C
U+66F4.1	fR/S	 73.3
U+66F4.1	fTotalStrokes	 7
U+66F4.1	fCangjie	MLWK
U+66F4.1	fFourCorner	10506
U+66F4.1	fHakka	  gang1;  gien3;  gen5; gien5; gen3; gen1; gin3
U+66F4.1	fCantonese	gang1; gang3
U+66F4.1	fMandarin	geng1; jing1
U+66F4.1	fEnglish	[1] the watches of the night [2] night watchman [3] change; alter; shift [4] experience [5] alternate
U+66F5.0	fUTF8	曵
U+66F5.0	fCNS11643	3-2641
U+66F5.0	fGB	E-2D30
U+66F5.0	fR/S	 73.3
U+66F5.0	fTotalStrokes	 7
U+66F5.0	fCangjie	ILWP
U+66F5.0	fFourCorner	53000
U+66F5.0	fHakka	 i5; ji3
U+66F5.0	fCantonese	jai6
U+66F5.0	fMandarin	yi4; ye4
U+66F5.0	fEnglish	trail, tow, drag, pull
U+66F6.0	fUTF8	曶
U+66F6.0	fCNS11643	2-2732
U+66F6.0	fBig5	CCD1
U+66F6.0	fGB	3-4838
U+66F6.0	fR/S	 73.4
U+66F6.0	fTotalStrokes	 8
U+66F6.0	fCangjie	PHA
U+66F6.0	fFourCorner	27602
U+66F6.0	fCantonese	fat1; mat6 (also)
U+66F6.0	fMandarin	hu1; wu4
U+66F7.0	fUTF8	曷
U+66F7.0	fCNS11643	1-5157
U+66F7.0	fBig5	AC54
U+66F7.0	fGB	0-6A42
U+66F7.0	fR/S	 73.5
U+66F7.0	fTotalStrokes	 9
U+66F7.0	fCangjie	APVO
U+66F7.0	fFourCorner	60727
U+66F7.0	fHakka	  hot7; hot8
U+66F7.0	fCantonese	hot3; aat3; ngaat3 (also); hot6
U+66F7.0	fMandarin	he2
U+66F7.0	fEnglish	[1] why? what? where? [2] why not [3] how
U+66F8.0	fUTF8	書
U+66F8.0	fCNS11643	1-5573
U+66F8.0	fBig5	AED1
U+66F8.0	fGB	1-4A69
U+66F8.0	fR/S	 73.6
U+66F8.0	fTotalStrokes	 10
U+66F8.0	fCangjie	LGA
U+66F8.0	fFourCorner	50601
U+66F8.0	fHakka	  su1; shu1
U+66F8.0	fCantonese	syu1
U+66F8.0	fMandarin	shu1
U+66F8.0	fEnglish	[1] book; writings [2] write [3] letter [4] document; certificate [5] the style of the calligraphy script
U+66F9.0	fUTF8	曹
U+66F9.0	fCNS11643	1-5B2A
U+66F9.0	fBig5	B1E4
U+66F9.0	fGB	0-325C
U+66F9.0	fR/S	 73.7
U+66F9.0	fTotalStrokes	 11
U+66F9.0	fCangjie	TWA
U+66F9.0	fFourCorner	55606
U+66F9.0	fHakka	  cau2;  co2; co3
U+66F9.0	fCantonese	cou4
U+66F9.0	fMandarin	cao2
U+66F9.0	fEnglish	[1] a plural particle [2] a Chinese family name
U+66FA.0	fUTF8	曺
U+66FA.0	fCNS11643	F-3273
U+66FA.0	fGB	E-2D31
U+66FA.0	fR/S	 73.6
U+66FA.0	fTotalStrokes	 10
U+66FA.0	fCangjie	JWA
U+66FA.0	fCantonese	cou4
U+66FA.0	fMandarin	cao2
U+66FB.0	fUTF8	曻
U+66FB.0	fGB	E-2D32
U+66FB.0	fR/S	 73.7; 72.7
U+66FB.0	fTotalStrokes	 11
U+66FB.0	fCangjie	ANIQ
U+66FB.0	fFourCorner	6025
U+66FB.0	fCantonese	cyun2
U+66FB.0	fMandarin	sheng1
U+66FC.0	fUTF8	曼
U+66FC.0	fCNS11643	1-5937
U+66FC.0	fBig5	B0D2
U+66FC.0	fGB	0-427C
U+66FC.0	fR/S	 73.7; 72.7
U+66FC.0	fTotalStrokes	 11
U+66FC.0	fCangjie	AWLE
U+66FC.0	fFourCorner	60407
U+66FC.0	fHakka	 man5; man6; man3; wan5; wan6; van5
U+66FC.0	fCantonese	maan6; maan4; mun5
U+66FC.0	fMandarin	man4; man1; man2; wan4
U+66FC.0	fEnglish	[1] long; vast; extended; prolonged; long-drawn-out [2] delicately; beautiful [3] a Chinese family name
U+66FD.0	fUTF8	曽
U+66FD.0	fGB	E-2D33
U+66FD.0	fR/S	 72.7
U+66FD.0	fTotalStrokes	 11
U+66FD.0	fCangjie	XCWA
U+66FD.0	fCantonese	zang1
U+66FD.0	fMandarin	ceng1; zeng1; ceng2
U+66FD.0	fEnglish	 already sign of past
U+66FE.0	fUTF8	曾
U+66FE.0	fCNS11643	1-5F64
U+66FE.0	fBig5	B4BF
U+66FE.0	fGB	0-5478
U+66FE.0	fR/S	 73.8
U+66FE.0	fTotalStrokes	 12
U+66FE.0	fCangjie	CWA
U+66FE.0	fFourCorner	80606
U+66FE.0	fHakka	  zen1; cien2
U+66FE.0	fCantonese	zang1; cang4
U+66FE.0	fMandarin	zeng1
U+66FE.0	fEnglish	[1] older or younger by three generations [2] a Chinese family name
U+66FE.1	fUTF8	曾
U+66FE.1	fCNS11643	1-5F64
U+66FE.1	fBig5	B4BF
U+66FE.1	fGB	0-5478
U+66FE.1	fR/S	 73.8
U+66FE.1	fTotalStrokes	 12
U+66FE.1	fCangjie	CWA
U+66FE.1	fFourCorner	80606
U+66FE.1	fHakka	 cen2
U+66FE.1	fMandarin	ceng2
U+66FE.1	fEnglish	ever; once
U+66FF.0	fUTF8	替
U+66FF.0	fCNS11643	1-5F65
U+66FF.0	fBig5	B4C0
U+66FF.0	fGB	0-4C66
U+66FF.0	fR/S	 73.8
U+66FF.0	fTotalStrokes	 12
U+66FF.0	fCangjie	QOA
U+66FF.0	fFourCorner	55603
U+66FF.0	fHakka	  tai5; ti5; toi5; t'e5
U+66FF.0	fCantonese	tai3
U+66FF.0	fMandarin	ti4
U+66FF.0	fEnglish	[1] take the place of; replace; substitute [2] for; on behalf of [3] decay; decline [4] neglect
U+6700.0	fUTF8	最
U+6700.0	fCNS11643	1-5E32
U+6700.0	fBig5	B3CC
U+6700.0	fGB	0-576E
U+6700.0	fR/S	 73.8; 13.10
U+6700.0	fTotalStrokes	 12
U+6700.0	fCangjie	ASJE
U+6700.0	fFourCorner	60147
U+6700.0	fHakka	  zui5
U+6700.0	fCantonese	zeoi3
U+6700.0	fMandarin	zui4
U+6700.0	fEnglish	most; extremely; exceedingly; extreme; superlative
U+6701.0	fUTF8	朁
U+6701.0	fCNS11643	2-3C4C
U+6701.0	fBig5	D9A9
U+6701.0	fGB	3-4839
U+6701.0	fR/S	 73.8
U+6701.0	fTotalStrokes	 12
U+6701.0	fCangjie	MUA
U+6701.0	fFourCorner	11601
U+6701.0	fHakka	 cam3
U+6701.0	fCantonese	caam2
U+6701.0	fMandarin	can3; qian2
U+6701.0	fEnglish	if; supposing; nevertheless
U+6702.0	fUTF8	朂
U+6702.0	fCNS11643	3-6527
U+6702.0	fGB	E-2D34
U+6702.0	fR/S	 73.8
U+6702.0	fTotalStrokes	 12
U+6702.0	fCangjie	ASJS
U+6702.0	fHakka	 hiuk7
U+6702.0	fCantonese	juk1
U+6702.0	fMandarin	xu4
U+6703.0	fUTF8	會
U+6703.0	fCNS11643	1-6444
U+6703.0	fBig5	B77C
U+6703.0	fGB	1-3B61
U+6703.0	fR/S	 73.9
U+6703.0	fTotalStrokes	 13
U+6703.0	fCangjie	OMWA
U+6703.0	fFourCorner	80606
U+6703.0	fHakka	 fui3; fui6; fui5; foi5; woi6; woi5; kai5; woi3; kwai5
U+6703.0	fCantonese	wui6; kui2; wui2 (also); wui5; kwui2 (also); kui3; wui3; wui4
U+6703.0	fMandarin	hui4; gua1; guai4
U+6703.0	fEnglish	[1] assemble; gather; meet together; converge [2] meet [3] meeting; convention [4] association; society [5] private banking cooperative [6] be able to [7] understand; comprehend; realize [8] shall; will [9] chief city; capital
U+6703.1	fUTF8	會
U+6703.1	fCNS11643	1-6444
U+6703.1	fBig5	B77C
U+6703.1	fGB	1-3B61
U+6703.1	fR/S	 73.9
U+6703.1	fTotalStrokes	 13
U+6703.1	fCangjie	OMWA
U+6703.1	fFourCorner	80606
U+6703.1	fHakka	 fi5
U+6703.1	fMandarin	hui3
U+6703.1	fEnglish	a brief period of time; moment
U+6703.2	fUTF8	會
U+6703.2	fCNS11643	1-6444
U+6703.2	fBig5	B77C
U+6703.2	fGB	1-3B61
U+6703.2	fR/S	 73.9
U+6703.2	fTotalStrokes	 13
U+6703.2	fCangjie	OMWA
U+6703.2	fFourCorner	80606
U+6703.2	fMandarin	kuai4
U+6703.2	fEnglish	 add; compute
U+6703.3	fUTF8	會
U+6703.3	fCNS11643	1-6444
U+6703.3	fBig5	B77C
U+6703.3	fGB	1-3B61
U+6703.3	fR/S	 73.9
U+6703.3	fTotalStrokes	 13
U+6703.3	fCangjie	OMWA
U+6703.3	fFourCorner	80606
U+6703.3	fMandarin	gui4
U+6703.3	fEnglish	used in Kueichi, Chekiang province
U+6704.0	fUTF8	朄
U+6704.0	fCNS11643	2-4A61
U+6704.0	fBig5	E1FA
U+6704.0	fGB	3-3029
U+6704.0	fR/S	 73.10
U+6704.0	fTotalStrokes	 14
U+6704.0	fCangjie	LLDWF
U+6704.0	fFourCorner	55096
U+6704.0	fCantonese	jan5; jan6 (also)
U+6704.0	fMandarin	yin4; yin3
U+6705.0	fUTF8	朅
U+6705.0	fCNS11643	2-4A60
U+6705.0	fBig5	E1F9
U+6705.0	fGB	3-346F
U+6705.0	fR/S	 73.10
U+6705.0	fTotalStrokes	 14
U+6705.0	fCangjie	GIAPV
U+6705.0	fFourCorner	46727
U+6705.0	fHakka	  giet7;  kiet7; get7
U+6705.0	fCantonese	kit3; hot3
U+6705.0	fMandarin	qie4
U+6705.0	fEnglish	brave
U+6706.0	fUTF8	朆
U+6706.0	fCNS11643	3-5124
U+6706.0	fGB	5-313B
U+6706.0	fR/S	 73.12; 73.14; 20.14
U+6706.0	fTotalStrokes	 16
U+6706.0	fCangjie	CAPHH
U+6706.0	fFourCorner	87620
U+6706.0	fCantonese	fan1
U+6706.0	fMandarin	fen1
U+6706.0	fEnglish	 not
U+6707.0	fUTF8	朇
U+6707.0	fCNS11643	7-5347
U+6707.0	fGB	E-2D35
U+6707.0	fR/S	  73.17
U+6707.0	fTotalStrokes	 21
U+6707.0	fCangjie	OAHWJ
U+6707.0	fFourCorner	86640
U+6707.0	fCantonese	pei4
U+6707.0	fMandarin	pi2; bi4
U+6708.0	fUTF8	月
U+6708.0	fCNS11643	1-454C
U+6708.0	fBig5	A4EB
U+6708.0	fGB	0-5442
U+6708.0	fR/S	 74.0
U+6708.0	fTotalStrokes	 4
U+6708.0	fCangjie	B
U+6708.0	fFourCorner	77220
U+6708.0	fHakka	  ngiet8; nget8; niet8; gniat8
U+6708.0	fCantonese	jyut6; jyut2 (also)
U+6708.0	fMandarin	yue4
U+6708.0	fEnglish	[1] moon [2] month [3] Kangxi radical 74
U+6708.1	fUTF8	月
U+6708.1	fCNS11643	1-454C
U+6708.1	fBig5	A4EB
U+6708.1	fGB	0-5442
U+6708.1	fR/S	 74.0
U+6708.1	fTotalStrokes	 4
U+6708.1	fCangjie	B
U+6708.1	fFourCorner	77220
U+6708.1	fMandarin	yue5
U+6709.0	fUTF8	有
U+6709.0	fCNS11643	1-4834
U+6709.0	fBig5	A6B3
U+6709.0	fGB	0-5350
U+6709.0	fR/S	 74.2
U+6709.0	fTotalStrokes	 6
U+6709.0	fCangjie	KB
U+6709.0	fFourCorner	40227
U+6709.0	fHakka	  ju1;  jiu1;  ju3; jeu1; riu1; riu3; jiu3; jiu2
U+6709.0	fCantonese	jau5
U+6709.0	fMandarin	you3
U+6709.0	fEnglish	have; be present; own; possess; exist; there is
U+6709.1	fUTF8	有
U+6709.1	fCNS11643	1-4834
U+6709.1	fBig5	A6B3
U+6709.1	fGB	0-5350
U+6709.1	fR/S	 74.2
U+6709.1	fTotalStrokes	 6
U+6709.1	fCangjie	KB
U+6709.1	fFourCorner	40227
U+6709.1	fHakka	 ju5
U+6709.1	fCantonese	jau6
U+6709.1	fMandarin	you4
U+6709.1	fEnglish	same as &#x53C8;
U+670A.0	fUTF8	朊
U+670A.0	fCNS11643	2-2733
U+670A.0	fBig5	CCD2
U+670A.0	fGB	0-6B43
U+670A.0	fR/S	 74.4; 130.4
U+670A.0	fTotalStrokes	 8
U+670A.0	fCangjie	BMMU
U+670A.0	fFourCorner	71211
U+670A.0	fCantonese	jyun2; gun2; gwun2 (also)
U+670A.0	fMandarin	ruan3
U+670B.0	fUTF8	朋
U+670B.0	fCNS11643	1-4E25
U+670B.0	fBig5	AA42
U+670B.0	fGB	0-4573
U+670B.0	fR/S	 74.4
U+670B.0	fTotalStrokes	 8
U+670B.0	fCangjie	BB
U+670B.0	fFourCorner	77220
U+670B.0	fHakka	  pen2; pien2; pen3; pung2
U+670B.0	fCantonese	pang4; pang2 (also)
U+670B.0	fMandarin	peng2
U+670B.0	fEnglish	[1] friend; pal; acquaintance [2] group; clique
U+670C.0	fUTF8	朌
U+670C.0	fCNS11643	3-2945
U+670C.0	fGB	5-4E79
U+670C.0	fR/S	 74.4
U+670C.0	fTotalStrokes	 8
U+670C.0	fCangjie	BCSH
U+670C.0	fFourCorner	78220
U+670C.0	fCantonese	fan4; baan1
U+670C.0	fMandarin	ban1; fen2
U+670C.0	fEnglish	distribute; large head
U+670D.0	fUTF8	服
U+670D.0	fCNS11643	1-4E24
U+670D.0	fBig5	AA41
U+670D.0	fGB	0-377E
U+670D.0	fR/S	 74.4
U+670D.0	fTotalStrokes	 8
U+670D.0	fCangjie	BSLE
U+670D.0	fFourCorner	77247
U+670D.0	fHakka	  fuk8; fuk7
U+670D.0	fCantonese	fuk6
U+670D.0	fMandarin	fu2; bao2; bo2; bi4
U+670D.0	fEnglish	[1] clothes; garments; dress; costume [2] mourning [3] wear (clothes) [4] obey; be convinced; yield; concede; admit [5] serve [6] take (medicine) [7] be accustomed to; be acclimated to
U+670D.1	fUTF8	服
U+670D.1	fCNS11643	1-4E24
U+670D.1	fBig5	AA41
U+670D.1	fGB	0-377E
U+670D.1	fR/S	 74.4
U+670D.1	fTotalStrokes	 8
U+670D.1	fCangjie	BSLE
U+670D.1	fFourCorner	77247
U+670D.1	fMandarin	fu4
U+670D.1	fEnglish	numerical classifier for Chinese medicine; dose
U+670D.2	fUTF8	服
U+670D.2	fCNS11643	1-4E24
U+670D.2	fBig5	AA41
U+670D.2	fGB	0-377E
U+670D.2	fR/S	 74.4
U+670D.2	fTotalStrokes	 8
U+670D.2	fCangjie	BSLE
U+670D.2	fFourCorner	77247
U+670D.2	fMandarin	fu5
U+670E.0	fUTF8	朎
U+670E.0	fCNS11643	3-2D27
U+670E.0	fGB	E-2D36
U+670E.0	fR/S	 74.5; 130.5
U+670E.0	fTotalStrokes	 9
U+670E.0	fCangjie	BOII
U+670E.0	fFourCorner	78230
U+670E.0	fCantonese	ling4
U+670E.0	fMandarin	ling2
U+670F.0	fUTF8	朏
U+670F.0	fCNS11643	2-2A7A
U+670F.0	fBig5	CEF9
U+670F.0	fGB	3-4C67
U+670F.0	fR/S	 74.5
U+670F.0	fTotalStrokes	 9
U+670F.0	fCangjie	BUU
U+670F.0	fFourCorner	72272
U+670F.0	fHakka	 fui5; fi5; fui3
U+670F.0	fCantonese	fei2
U+670F.0	fMandarin	fei3; pei4; ku1; pei3
U+670F.0	fEnglish	 light of a crescent moon
U+6710.0	fUTF8	朐
U+6710.0	fCNS11643	2-2A7B
U+6710.0	fBig5	CEFA
U+6710.0	fGB	0-6B54
U+6710.0	fR/S	 74.5
U+6710.0	fTotalStrokes	 9
U+6710.0	fCangjie	BPR
U+6710.0	fFourCorner	77220
U+6710.0	fCantonese	keoi4
U+6710.0	fMandarin	qu2
U+6710.0	fEnglish	warm
U+6711.0	fUTF8	朑
U+6711.0	fCNS11643	3-6632
U+6711.0	fGB	E-2D37
U+6711.0	fR/S	 74.5; 130.5
U+6711.0	fTotalStrokes	 9
U+6711.0	fCangjie	BPT
U+6711.0	fFourCorner	7421
U+6711.0	fCantonese	sai3
U+6712.0	fUTF8	朒
U+6712.0	fCNS11643	2-2F59
U+6712.0	fBig5	D1D7
U+6712.0	fGB	3-4C71
U+6712.0	fR/S	 74.6
U+6712.0	fTotalStrokes	 10
U+6712.0	fCangjie	BOBO
U+6712.0	fFourCorner	74227
U+6712.0	fCantonese	nuk6; luk6 (also); muk6 (also)
U+6712.0	fMandarin	nuu4; n4
U+6713.0	fUTF8	朓
U+6713.0	fCNS11643	2-2F5A
U+6713.0	fBig5	D1D8
U+6713.0	fGB	3-4C76
U+6713.0	fR/S	 74.6
U+6713.0	fTotalStrokes	 10
U+6713.0	fCangjie	BLMO
U+6713.0	fFourCorner	72213
U+6713.0	fHakka	  tiau5
U+6713.0	fCantonese	tiu3; tiu5 (also); tiu1 (also); tiu2
U+6713.0	fMandarin	tiao4; tiao3
U+6713.0	fEnglish	at the end of a lunar month when the moon is setting in the west
U+6714.0	fUTF8	朔
U+6714.0	fCNS11643	1-5574
U+6714.0	fBig5	AED2
U+6714.0	fGB	0-4B37
U+6714.0	fR/S	 74.6
U+6714.0	fTotalStrokes	 10
U+6714.0	fCangjie	TUB
U+6714.0	fFourCorner	87420
U+6714.0	fHakka	  sok7
U+6714.0	fCantonese	sok3
U+6714.0	fMandarin	shuo4
U+6714.0	fEnglish	[1] begin [2] north [3] the first day of the lunar month
U+6715.0	fUTF8	朕
U+6715.0	fCNS11643	1-5575
U+6715.0	fBig5	AED3
U+6715.0	fGB	0-6B5E
U+6715.0	fR/S	 74.6
U+6715.0	fTotalStrokes	 10
U+6715.0	fCangjie	BTK
U+6715.0	fFourCorner	78234
U+6715.0	fHakka	 zhim3; zihm3; zhin5; zim3; dim3
U+6715.0	fCantonese	zam6
U+6715.0	fMandarin	zhen4
U+6715.0	fEnglish	[1] royal "we" (used exclusively by the emperor or king to mean "I") [2] omens; auguries; portents; signs
U+6716.0	fUTF8	朖
U+6716.0	fCNS11643	3-3648
U+6716.0	fGB	E-2D38
U+6716.0	fR/S	 74.7
U+6716.0	fTotalStrokes	 11
U+6716.0	fCangjie	BIAV
U+6716.0	fFourCorner	73230
U+6716.0	fHakka	 long5; long3
U+6716.0	fCantonese	long5
U+6716.0	fMandarin	lang3
U+6716.0	fEnglish	same as &#x6717;
U+6717.0	fUTF8	朗
U+6717.0	fCNS11643	1-5576
U+6717.0	fBig5	AED4
U+6717.0	fGB	0-404A
U+6717.0	fR/S	 74.7
U+6717.0	fTotalStrokes	 11
U+6717.0	fCangjie	IIB
U+6717.0	fFourCorner	37720
U+6717.0	fHakka	  long3;  long5; long6
U+6717.0	fCantonese	long5
U+6717.0	fMandarin	lang3
U+6717.0	fEnglish	[1] clear; bright; distinct [2] resonant; sonorously; sonorous
U+6718.0	fUTF8	朘
U+6718.0	fCNS11643	2-3571
U+6718.0	fBig5	D5AF
U+6718.0	fGB	3-4D2E
U+6718.0	fR/S	 74.7
U+6718.0	fTotalStrokes	 11
U+6718.0	fCangjie	BICE
U+6718.0	fFourCorner	73247
U+6718.0	fCantonese	zeoi1; zeon1 (also); zyun1; syun1 (also); zeon3
U+6718.0	fMandarin	juan1; zui1
U+6718.0	fEnglish	wane; reduce; contract; exploit
U+6719.0	fUTF8	朙
U+6719.0	fCNS11643	3-3647
U+6719.0	fGB	5-4F33
U+6719.0	fR/S	 74.7; 74.8
U+6719.0	fTotalStrokes	 11; 12
U+6719.0	fCangjie	WBB
U+6719.0	fFourCorner	67020
U+6719.0	fHakka	 min2; miang2
U+6719.0	fCantonese	ming4
U+6719.0	fMandarin	ming2
U+6719.0	fEnglish	ancient form of &#x660E;
U+671A.0	fUTF8	朚
U+671A.0	fCNS11643	3-3646
U+671A.0	fGB	5-496B
U+671A.0	fR/S	  74.7
U+671A.0	fTotalStrokes	 11
U+671A.0	fCangjie	YVAB
U+671A.0	fFourCorner	00620
U+671A.0	fCantonese	fong1
U+671A.0	fMandarin	huang1
U+671B.0	fUTF8	望
U+671B.0	fCNS11643	1-5B2C
U+671B.0	fBig5	B1E6
U+671B.0	fGB	0-4D7B
U+671B.0	fR/S	 74.7
U+671B.0	fTotalStrokes	 11
U+671B.0	fCangjie	YBHG
U+671B.0	fFourCorner	07104
U+671B.0	fHakka	  mong5; mong6; mong3
U+671B.0	fCantonese	mong6; mong2 (also); mong4
U+671B.0	fMandarin	wang4; wang2
U+671B.0	fEnglish	[1] look at; view; watch; gaze into the distance [2] hope; expect [3] the 15th day of each month [4] reputation; prestige [5] call on; visit
U+671B.1	fUTF8	望
U+671B.1	fCNS11643	1-5B2C
U+671B.1	fBig5	B1E6
U+671B.1	fGB	0-4D7B
U+671B.1	fR/S	 74.7
U+671B.1	fTotalStrokes	 11
U+671B.1	fCangjie	YBHG
U+671B.1	fFourCorner	07104
U+671B.1	fMandarin	wang5
U+671C.0	fUTF8	朜
U+671C.0	fCNS11643	4-3829
U+671C.0	fGB	E-2D39
U+671C.0	fR/S	  74.8
U+671C.0	fTotalStrokes	 12
U+671C.0	fCangjie	BYRD
U+671C.0	fFourCorner	70240
U+671C.0	fCantonese	tan1
U+671C.0	fMandarin	tun1
U+671D.0	fUTF8	朝
U+671D.0	fCNS11643	1-5F67
U+671D.0	fBig5	B4C2
U+671D.0	fGB	0-332F
U+671D.0	fR/S	 74.8
U+671D.0	fTotalStrokes	 12
U+671D.0	fCangjie	JJB
U+671D.0	fFourCorner	47420
U+671D.0	fHakka	  cau2; zhau1;  zau1;  chau2; chau3; zeu1; ceu2; cheu1; ch'eu2
U+671D.0	fCantonese	ciu4; ziu1; zyu1
U+671D.0	fMandarin	chao2
U+671D.0	fEnglish	[1] imperial court [2] dynasty [3] go to imperial court [4] face
U+671D.1	fUTF8	朝
U+671D.1	fCNS11643	1-5F67
U+671D.1	fBig5	B4C2
U+671D.1	fGB	0-332F
U+671D.1	fR/S	 74.8
U+671D.1	fTotalStrokes	 12
U+671D.1	fCangjie	JJB
U+671D.1	fFourCorner	47420
U+671D.1	fMandarin	zhao1; zhu1
U+671D.1	fEnglish	[1] morning [2] day
U+671E.0	fUTF8	朞
U+671E.0	fCNS11643	3-3C29
U+671E.0	fGB	E-2D3A
U+671E.0	fR/S	  74.8
U+671E.0	fTotalStrokes	 12
U+671E.0	fCangjie	TCB
U+671E.0	fFourCorner	44220
U+671E.0	fCantonese	gei1
U+671E.0	fMandarin	ji1; qi1; qi2
U+671E.0	fEnglish	full year; anniversary
U+671F.0	fUTF8	期
U+671F.0	fCNS11643	1-5F66
U+671F.0	fBig5	B4C1
U+671F.0	fGB	0-465A
U+671F.0	fR/S	 74.8
U+671F.0	fTotalStrokes	 12
U+671F.0	fCangjie	TCB
U+671F.0	fFourCorner	47820
U+671F.0	fHakka	  ki2; ki3
U+671F.0	fCantonese	kei4; gei1
U+671F.0	fMandarin	qi1; qi2
U+671F.0	fEnglish	[1] periods; times [2] designated time; time limit [3] expect; hope; wait
U+671F.1	fUTF8	期
U+671F.1	fCNS11643	1-5F66
U+671F.1	fBig5	B4C1
U+671F.1	fGB	0-465A
U+671F.1	fR/S	 74.8
U+671F.1	fTotalStrokes	 12
U+671F.1	fCangjie	TCB
U+671F.1	fFourCorner	47820
U+671F.1	fMandarin	ji1
U+671F.1	fEnglish	one year
U+6720.0	fUTF8	朠
U+6720.0	fCNS11643	2-434E
U+6720.0	fBig5	DDC8
U+6720.0	fGB	E-2D3B
U+6720.0	fR/S	 74.9; 74.8
U+6720.0	fTotalStrokes	 13
U+6720.0	fCangjie	BTLK
U+6720.0	fFourCorner	74235
U+6720.0	fCantonese	jing1
U+6720.0	fMandarin	ying1
U+6720.0	fEnglish	color of the moon
U+6721.0	fUTF8	朡
U+6721.0	fCNS11643	2-4641
U+6721.0	fBig5	DF7A
U+6721.0	fGB	3-4D52
U+6721.0	fR/S	 74.9
U+6721.0	fTotalStrokes	 13
U+6721.0	fCangjie	BUCE
U+6721.0	fFourCorner	72247
U+6721.0	fCantonese	zung1
U+6721.0	fMandarin	zong1; zong3
U+6722.0	fUTF8	朢
U+6722.0	fCNS11643	2-4A62
U+6722.0	fBig5	E1FB
U+6722.0	fGB	E-2D3C
U+6722.0	fR/S	 74.10
U+6722.0	fTotalStrokes	 14
U+6722.0	fCangjie	SBHG
U+6722.0	fFourCorner	77104
U+6722.0	fHakka	 mong5; mong6
U+6722.0	fCantonese	mong6; mong4
U+6722.0	fMandarin	wang4
U+6722.0	fEnglish	standard form of @ the 15th day of each month of the lunar calendar
U+6723.0	fUTF8	朣
U+6723.0	fCNS11643	2-5748
U+6723.0	fBig5	E9BD
U+6723.0	fGB	5-4F6D
U+6723.0	fR/S	 74.12
U+6723.0	fTotalStrokes	 16
U+6723.0	fCangjie	BYTG
U+6723.0	fFourCorner	70214
U+6723.0	fHakka	  tung2
U+6723.0	fCantonese	tung4
U+6723.0	fMandarin	tong2
U+6723.0	fEnglish	 rising moon
U+6724.0	fUTF8	朤
U+6724.0	fCNS11643	3-5126
U+6724.0	fGB	E-2D3D
U+6724.0	fR/S	 74.12
U+6724.0	fTotalStrokes	 16
U+6724.0	fCangjie	BBBB
U+6724.0	fFourCorner	77220
U+6724.0	fHakka	  long3;  long5; long6
U+6724.0	fCantonese	long5
U+6724.0	fMandarin	lang3
U+6724.0	fEnglish	same as &#x6717;
U+6725.0	fUTF8	朥
U+6725.0	fCNS11643	3-6633
U+6725.0	fGB	E-2D3E
U+6725.0	fR/S	 130.12; 74.12
U+6725.0	fTotalStrokes	 16
U+6725.0	fCangjie	BFFS
U+6725.0	fFourCorner	7922
U+6725.0	fCantonese	lou4
U+6726.0	fUTF8	朦
U+6726.0	fCNS11643	1-764D
U+6726.0	fBig5	C261
U+6726.0	fGB	0-6B7C
U+6726.0	fR/S	 74.14
U+6726.0	fTotalStrokes	 18
U+6726.0	fCangjie	BTBO
U+6726.0	fFourCorner	74232
U+6726.0	fHakka	 mung2; mung3
U+6726.0	fCantonese	mung4
U+6726.0	fMandarin	meng2
U+6726.0	fEnglish	[1] the state of the moon just before setting [2] dim; vague; hazy [3] deceive; swindle
U+6727.0	fUTF8	朧
U+6727.0	fCNS11643	1-7972
U+6727.0	fBig5	C467
U+6727.0	fGB	1-6B4A
U+6727.0	fR/S	 74.16
U+6727.0	fTotalStrokes	 20
U+6727.0	fCangjie	BYBP
U+6727.0	fFourCorner	71211
U+6727.0	fHakka	 lung2; lun3
U+6727.0	fCantonese	lung4
U+6727.0	fMandarin	long2; long3
U+6727.0	fEnglish	 the moon's brightness
U+6728.0	fUTF8	木
U+6728.0	fCNS11643	1-454D
U+6728.0	fBig5	A4EC
U+6728.0	fGB	0-443E
U+6728.0	fR/S	 75.0
U+6728.0	fTotalStrokes	 4
U+6728.0	fCangjie	D
U+6728.0	fFourCorner	40900
U+6728.0	fHakka	  muk7
U+6728.0	fCantonese	muk6
U+6728.0	fMandarin	mu4
U+6728.0	fEnglish	[1] tree [2] wood; lumber; timber [3] made of wood; wooden [4] simple; honest [5] senseless; benumbed; dull [6] coffin
U+6729.0	fUTF8	朩
U+6729.0	fGB	1-7D33
U+6729.0	fR/S	 75.0
U+6729.0	fTotalStrokes	 4
U+6729.0	fCangjie	XDC
U+6729.0	fCantonese	muk6
U+6729.0	fMandarin	pin4; deng3
U+6729.0	fEnglish	 rank; grade wait equal "etc."
U+672A.0	fUTF8	未
U+672A.0	fCNS11643	1-465C
U+672A.0	fBig5	A5BC
U+672A.0	fGB	0-4E34
U+672A.0	fR/S	 75.1
U+672A.0	fTotalStrokes	 5
U+672A.0	fCangjie	JD
U+672A.0	fFourCorner	50900
U+672A.0	fHakka	 wi5; wui5; mui5; wui6; mui3; mui6; vui5
U+672A.0	fCantonese	mei6
U+672A.0	fMandarin	wei4
U+672A.0	fEnglish	[1] not yet [2] not [3] the eight of the twelve Terrestrial branches [4] 1;00-3:00 p.m. [5] a Chinese family name
U+672B.0	fUTF8	末
U+672B.0	fCNS11643	1-465D
U+672B.0	fBig5	A5BD
U+672B.0	fGB	0-4429
U+672B.0	fR/S	 75.1
U+672B.0	fTotalStrokes	 5
U+672B.0	fCangjie	DJ
U+672B.0	fFourCorner	50900
U+672B.0	fHakka	  mat8; mat7
U+672B.0	fCantonese	mut6; mut2 (also); mo1
U+672B.0	fMandarin	mo4; mie4
U+672B.0	fEnglish	[1] final; last; end [2] late; recent [3] trivial; insignificant; unimportant [4] end; tip [5] the four limbs [6] final particle
U+672C.0	fUTF8	本
U+672C.0	fCNS11643	1-465B
U+672C.0	fBig5	A5BB
U+672C.0	fGB	0-313E
U+672C.0	fR/S	 75.1
U+672C.0	fTotalStrokes	 5
U+672C.0	fCangjie	DM
U+672C.0	fFourCorner	50230
U+672C.0	fHakka	  bun3
U+672C.0	fCantonese	bun2
U+672C.0	fMandarin	ben3; ben1
U+672C.0	fEnglish	[1] root of a plant [2] root; origin; source; basis; foundation [3] original [4] book; copy [5] capital (in business) [6] our; this; the present [7] according to; based on [8] beginning; starting point
U+672D.0	fUTF8	札
U+672D.0	fCNS11643	1-465E
U+672D.0	fBig5	A5BE
U+672D.0	fGB	0-547D
U+672D.0	fR/S	 75.1
U+672D.0	fTotalStrokes	 5
U+672D.0	fCangjie	DU
U+672D.0	fFourCorner	42910
U+672D.0	fHakka	  zap7;  zat7; zhap7
U+672D.0	fCantonese	zaat3; zaat2 (also); zaap3 (also); aat3; ngaat3 (also)
U+672D.0	fMandarin	zha2; ya4
U+672D.0	fEnglish	[1] thin wooden tablet for writing [2] letter; note; correspondence [3] documents or instructions to a subordinate [4] die before one comes of age
U+672E.0	fUTF8	朮
U+672E.0	fCNS11643	1-465A
U+672E.0	fBig5	A5BA
U+672E.0	fGB	E-2D3F
U+672E.0	fR/S	 75.1
U+672E.0	fTotalStrokes	 5
U+672E.0	fCangjie	IJC
U+672E.0	fFourCorner	43210
U+672E.0	fHakka	 sut8;  shut8
U+672E.0	fCantonese	seot6
U+672E.0	fMandarin	zhu2; shu4
U+672E.0	fEnglish	Podophyllum versipell, a plant with violet, green or red flowers and white roots, used in herbal medicine
U+672F.0	fUTF8	术
U+672F.0	fCNS11643	3-2252
U+672F.0	fGB	0-4A75
U+672F.0	fR/S	 75.1
U+672F.0	fTotalStrokes	 5
U+672F.0	fCangjie	ID
U+672F.0	fFourCorner	43900
U+672F.0	fHakka	 shut8; sut8
U+672F.0	fCantonese	seot6; seot2 (also); seoi6
U+672F.0	fMandarin	zhu2; shu4
U+672F.0	fEnglish	simplified form of &#x8853;
U+6730.0	fUTF8	朰
U+6730.0	fGB	1-7D34
U+6730.0	fR/S	 75.1; 5.4
U+6730.0	fTotalStrokes	 5
U+6730.0	fCangjie	XDN
U+6730.0	fCantonese	zaat3
U+6730.0	fEnglish	 kwukyel
U+6731.0	fUTF8	朱
U+6731.0	fCNS11643	1-4837
U+6731.0	fBig5	A6B6
U+6731.0	fGB	0-566C
U+6731.0	fR/S	 75.2
U+6731.0	fTotalStrokes	 6
U+6731.0	fCangjie	HJD
U+6731.0	fFourCorner	25900
U+6731.0	fHakka	  zu1; zhu1
U+6731.0	fCantonese	zyu1; syu4
U+6731.0	fMandarin	zhu1; shu1; shu2
U+6731.0	fEnglish	[1] red; cinnabar; vermilion [2] a Chinese family name
U+6732.0	fUTF8	朲
U+6732.0	fCNS11643	3-2377
U+6732.0	fGB	3-4045
U+6732.0	fR/S	  75.2
U+6732.0	fTotalStrokes	 6
U+6732.0	fCangjie	DO
U+6732.0	fFourCorner	48900
U+6732.0	fHakka	 ngin2
U+6732.0	fCantonese	jan4
U+6732.0	fMandarin	ren2
U+6732.0	fEnglish	rafters on which tiles are laid
U+6733.0	fUTF8	朳
U+6733.0	fCNS11643	2-2257
U+6733.0	fBig5	C9F6
U+6733.0	fGB	3-4046
U+6733.0	fR/S	 75.2
U+6733.0	fTotalStrokes	 6
U+6733.0	fCangjie	DC
U+6733.0	fFourCorner	48900
U+6733.0	fHakka	 bat8;  bat7; pat8; pa3; kwa3
U+6733.0	fCantonese	baat3; paa4
U+6733.0	fMandarin	ba1; bie1
U+6733.0	fEnglish	tree
U+6734.0	fUTF8	朴
U+6734.0	fCNS11643	1-4836
U+6734.0	fBig5	A6B5
U+6734.0	fGB	0-4653
U+6734.0	fR/S	 75.2
U+6734.0	fTotalStrokes	 6
U+6734.0	fCangjie	DY
U+6734.0	fFourCorner	43900
U+6734.0	fHakka	 pok7
U+6734.0	fCantonese	pok2 (also); piu4; po1
U+6734.0	fMandarin	pu3; piao2; po1; pou4; pu1
U+6734.0	fEnglish	saltpeter
U+6734.1	fUTF8	朴
U+6734.1	fCNS11643	1-4836
U+6734.1	fBig5	A6B5
U+6734.1	fGB	0-4653
U+6734.1	fR/S	 75.2
U+6734.1	fTotalStrokes	 6
U+6734.1	fCangjie	DY
U+6734.1	fFourCorner	43900
U+6734.1	fHakka	 pok7; puk7
U+6734.1	fCantonese	pok3
U+6734.1	fMandarin	pu2; po4; bu2
U+6734.1	fEnglish	simplified form of &#x6A38;
U+6735.0	fUTF8	朵
U+6735.0	fCNS11643	1-4838
U+6735.0	fBig5	A6B7
U+6735.0	fGB	0-3664
U+6735.0	fR/S	 75.2
U+6735.0	fTotalStrokes	 6
U+6735.0	fCangjie	HND
U+6735.0	fFourCorner	17904
U+6735.0	fHakka	  do3; to2
U+6735.0	fCantonese	do2; doe2 (also)
U+6735.0	fMandarin	duo3
U+6735.0	fEnglish	[1] cluster of flowers; flower [2] earlobe; lobe of the ear
U+6735.1	fUTF8	朵
U+6735.1	fCNS11643	1-4838
U+6735.1	fBig5	A6B7
U+6735.1	fGB	0-3664
U+6735.1	fR/S	 75.2
U+6735.1	fTotalStrokes	 6
U+6735.1	fCangjie	HND
U+6735.1	fFourCorner	17904
U+6735.1	fMandarin	duo5
U+6736.0	fUTF8	朶
U+6736.0	fCNS11643	3-2378
U+6736.0	fGB	E-2D40
U+6736.0	fR/S	 75.2
U+6736.0	fTotalStrokes	 6
U+6736.0	fCangjie	NSD
U+6736.0	fFourCorner	77900
U+6736.0	fHakka	  do3; to2; do2; dau2
U+6736.0	fCantonese	do2; doe2 (also); do3
U+6736.0	fMandarin	duo3
U+6736.0	fEnglish	cluster of flowers; earlobe
U+6737.0	fUTF8	朷
U+6737.0	fCNS11643	3-2375
U+6737.0	fGB	3-4048
U+6737.0	fR/S	  75.2
U+6737.0	fTotalStrokes	 6
U+6737.0	fCangjie	DSH
U+6737.0	fFourCorner	47920
U+6737.0	fCantonese	dou1
U+6737.0	fMandarin	dao1
U+6738.0	fUTF8	朸
U+6738.0	fCNS11643	2-2252
U+6738.0	fBig5	C9F1
U+6738.0	fGB	3-4049
U+6738.0	fR/S	 75.2
U+6738.0	fTotalStrokes	 6
U+6738.0	fCangjie	DKS
U+6738.0	fFourCorner	44927
U+6738.0	fCantonese	lak6; lik6 (also)
U+6738.0	fMandarin	li4; le4
U+6739.0	fUTF8	朹
U+6739.0	fCNS11643	2-2251
U+6739.0	fBig5	C9F0
U+6739.0	fGB	3-4047
U+6739.0	fR/S	 75.2
U+6739.0	fTotalStrokes	 6
U+6739.0	fCangjie	DKN
U+6739.0	fFourCorner	44917
U+6739.0	fHakka	 kiu2
U+6739.0	fCantonese	kau4; gwai2
U+6739.0	fMandarin	qiu2; gui3
U+6739.0	fEnglish	a kind of wild plum
U+673A.0	fUTF8	机
U+673A.0	fCNS11643	2-2254
U+673A.0	fBig5	C9F3
U+673A.0	fGB	0-3B7A
U+673A.0	fR/S	 75.2
U+673A.0	fTotalStrokes	 6
U+673A.0	fCangjie	DHN
U+673A.0	fFourCorner	47910
U+673A.0	fHakka	 gi1; ri3; ji3
U+673A.0	fCantonese	gei1; gei2
U+673A.0	fMandarin	ji1
U+673A.0	fEnglish	simplified form of &#x6A5F;
U+673B.0	fUTF8	朻
U+673B.0	fCNS11643	2-2253
U+673B.0	fBig5	C9F2
U+673B.0	fGB	5-433A
U+673B.0	fR/S	 75.2
U+673B.0	fTotalStrokes	 6
U+673B.0	fCangjie	DVL
U+673B.0	fFourCorner	42900
U+673B.0	fHakka	 kiu2; giu5
U+673B.0	fCantonese	gau1; kau4
U+673B.0	fMandarin	jiu1
U+673B.0	fEnglish	having drooping branches like the weeping willow
U+673C.0	fUTF8	朼
U+673C.0	fCNS11643	2-2256
U+673C.0	fBig5	C9F5
U+673C.0	fGB	5-4339
U+673C.0	fR/S	 75.2
U+673C.0	fTotalStrokes	 6
U+673C.0	fCangjie	DP
U+673C.0	fFourCorner	51010
U+673C.0	fHakka	 bi3
U+673C.0	fCantonese	pei4; bei2
U+673C.0	fMandarin	bi3
U+673D.0	fUTF8	朽
U+673D.0	fCNS11643	1-4835
U+673D.0	fBig5	A6B4
U+673D.0	fGB	0-5060
U+673D.0	fR/S	 75.2
U+673D.0	fTotalStrokes	 6
U+673D.0	fCangjie	DMVS
U+673D.0	fFourCorner	41927
U+673D.0	fHakka	 hiu3;  hiu1; kau3
U+673D.0	fCantonese	nau2; jau2 (also); lau2 (also); nau5
U+673D.0	fMandarin	xiu3
U+673D.0	fEnglish	[1] decayed; rotten; rot; decay [2] old and useless
U+673E.0	fUTF8	朾
U+673E.0	fCNS11643	2-2250
U+673E.0	fBig5	C9EF
U+673E.0	fGB	3-4044
U+673E.0	fR/S	 75.2
U+673E.0	fTotalStrokes	 6
U+673E.0	fCangjie	DMN
U+673E.0	fFourCorner	41920
U+673E.0	fHakka	 cen2;  tin2
U+673E.0	fCantonese	caang4; ting1
U+673E.0	fMandarin	ting2; ting1; cheng2
U+673E.0	fEnglish	 bump
U+673F.0	fUTF8	朿
U+673F.0	fCNS11643	2-2255
U+673F.0	fBig5	C9F4
U+673F.0	fGB	3-3023
U+673F.0	fR/S	 75.2
U+673F.0	fTotalStrokes	 6
U+673F.0	fCangjie	DB
U+673F.0	fFourCorner	50902
U+673F.0	fHakka	 cih5; ze5
U+673F.0	fCantonese	ci3
U+673F.0	fMandarin	ci4
U+673F.0	fEnglish	stab
U+6740.0	fUTF8	杀
U+6740.0	fCNS11643	F-2258
U+6740.0	fGB	0-4931
U+6740.0	fR/S	 75.2
U+6740.0	fTotalStrokes	 6
U+6740.0	fCangjie	XKD
U+6740.0	fHakka	 sat7
U+6740.0	fCantonese	saat3; saai3
U+6740.0	fMandarin	sha1; sa4; xue1; shai4; shi4; she4
U+6740.0	fEnglish	simplified form of &#x6BBA;
U+6741.0	fUTF8	杁
U+6741.0	fGB	E-2D41
U+6741.0	fR/S	 75.2
U+6741.0	fTotalStrokes	 6
U+6741.0	fCangjie	XDO
U+6741.0	fCantonese	jan4
U+6742.0	fUTF8	杂
U+6742.0	fCNS11643	F-2257
U+6742.0	fGB	0-5453
U+6742.0	fR/S	 75.2
U+6742.0	fTotalStrokes	 6
U+6742.0	fCangjie	KND
U+6742.0	fHakka	 cap8
U+6742.0	fCantonese	zaap6; zaap2 (also)
U+6742.0	fMandarin	za2
U+6742.0	fEnglish	simplified form of &#x96DC;
U+6743.0	fUTF8	权
U+6743.0	fGB	0-4828
U+6743.0	fR/S	 75.2
U+6743.0	fTotalStrokes	 6
U+6743.0	fCangjie	DE
U+6743.0	fHakka	 ken2; kien2
U+6743.0	fCantonese	kyun4
U+6743.0	fMandarin	quan2; guan4
U+6743.0	fEnglish	simplified form of &#x6B0A;
U+6744.0	fUTF8	杄
U+6744.0	fCNS11643	3-2648
U+6744.0	fGB	3-404E
U+6744.0	fR/S	  75.3
U+6744.0	fTotalStrokes	 7
U+6744.0	fCangjie	DHJ
U+6744.0	fFourCorner	42940
U+6744.0	fCantonese	cin1
U+6744.0	fMandarin	qian1
U+6745.0	fUTF8	杅
U+6745.0	fCNS11643	2-243E
U+6745.0	fBig5	CAFD
U+6745.0	fGB	3-404A
U+6745.0	fR/S	 75.3
U+6745.0	fTotalStrokes	 7
U+6745.0	fCangjie	DMD
U+6745.0	fFourCorner	41940
U+6745.0	fHakka	 ri1; ji1; i2; ji2
U+6745.0	fCantonese	jyu1; jyu4
U+6745.0	fMandarin	yu2; yu4
U+6745.0	fEnglish	tub
U+6746.0	fUTF8	杆
U+6746.0	fCNS11643	1-4A5F
U+6746.0	fBig5	A7FD
U+6746.0	fGB	0-384B
U+6746.0	fR/S	 75.3
U+6746.0	fTotalStrokes	 7
U+6746.0	fCangjie	DMJ
U+6746.0	fFourCorner	41940
U+6746.0	fHakka	  gon1
U+6746.0	fCantonese	gon1; gon2
U+6746.0	fMandarin	gan1; gan4
U+6746.0	fEnglish	 pole shaft of spear
U+6746.1	fUTF8	杆
U+6746.1	fCNS11643	1-4A5F
U+6746.1	fBig5	A7FD
U+6746.1	fGB	0-384B
U+6746.1	fR/S	 75.3
U+6746.1	fTotalStrokes	 7
U+6746.1	fCangjie	DMJ
U+6746.1	fFourCorner	41940
U+6746.1	fMandarin	gan3
U+6747.0	fUTF8	杇
U+6747.0	fCNS11643	2-243F
U+6747.0	fBig5	CAFE
U+6747.0	fGB	3-404B
U+6747.0	fR/S	 75.3
U+6747.0	fTotalStrokes	 7
U+6747.0	fCangjie	DMMS
U+6747.0	fFourCorner	41927
U+6747.0	fHakka	 wu1; vu1
U+6747.0	fCantonese	wu1
U+6747.0	fMandarin	wu1; hu2; wu4
U+6747.0	fEnglish	loquat
U+6748.0	fUTF8	杈
U+6748.0	fCNS11643	2-2443
U+6748.0	fBig5	CB43
U+6748.0	fGB	0-683E
U+6748.0	fR/S	 75.3
U+6748.0	fTotalStrokes	 7
U+6748.0	fCangjie	DEI
U+6748.0	fFourCorner	47940
U+6748.0	fHakka	 ca1;  ca5
U+6748.0	fCantonese	caa1; caa3
U+6748.0	fMandarin	cha1; chai1; chai4
U+6748.0	fEnglish	[1] fork of a tree; branches of a tree [2] pitchfork; fishfork; wooden fork; hayfork [3] any fork-like object [4] a kind of weapon in ancient China
U+6748.1	fUTF8	杈
U+6748.1	fCNS11643	2-2443
U+6748.1	fBig5	CB43
U+6748.1	fGB	0-683E
U+6748.1	fR/S	 75.3
U+6748.1	fTotalStrokes	 7
U+6748.1	fCangjie	DEI
U+6748.1	fFourCorner	47940
U+6748.1	fMandarin	cha4
U+6748.1	fEnglish	[1] cheval-de-frise; chevaux-de-frise
U+6749.0	fUTF8	杉
U+6749.0	fCNS11643	1-4A5E
U+6749.0	fBig5	A7FC
U+6749.0	fGB	0-493C
U+6749.0	fR/S	 75.3
U+6749.0	fTotalStrokes	 7
U+6749.0	fCangjie	DHHH
U+6749.0	fFourCorner	42922
U+6749.0	fHakka	  cam5;  cam1; cam6; cam3
U+6749.0	fCantonese	caam3; saam1
U+6749.0	fMandarin	shan1; sha1
U+6749.0	fEnglish	 various species of pine and fir; China fir
U+674A.0	fUTF8	杊
U+674A.0	fCNS11643	3-2649
U+674A.0	fGB	E-2D42
U+674A.0	fR/S	 75.3
U+674A.0	fTotalStrokes	 7
U+674A.0	fCangjie	DLLL
U+674A.0	fFourCorner	42900
U+674A.0	fCantonese	ceon4
U+674A.0	fMandarin	xun2
U+674B.0	fUTF8	杋
U+674B.0	fCNS11643	2-2447
U+674B.0	fBig5	CB47
U+674B.0	fGB	E-2D43
U+674B.0	fR/S	 75.3
U+674B.0	fTotalStrokes	 7
U+674B.0	fCangjie	DHNI
U+674B.0	fFourCorner	47910
U+674B.0	fHakka	 fam2
U+674B.0	fCantonese	faan4
U+674B.0	fMandarin	fan1; fan2
U+674B.0	fEnglish	a large tree the bark of which floats in water
U+674C.0	fUTF8	杌
U+674C.0	fCNS11643	2-2442
U+674C.0	fBig5	CB42
U+674C.0	fGB	0-683B
U+674C.0	fR/S	 75.3
U+674C.0	fTotalStrokes	 7
U+674C.0	fCangjie	DMU
U+674C.0	fFourCorner	41910
U+674C.0	fHakka	 ngut7
U+674C.0	fCantonese	ngat6; at6 (also)
U+674C.0	fMandarin	wu4; wu1; yue4
U+674C.0	fEnglish	[1] stump of a tree; tree without a branch [2] worried and anxious [3] square stool
U+674D.0	fUTF8	杍
U+674D.0	fCNS11643	2-2445
U+674D.0	fBig5	CB45
U+674D.0	fGB	5-433E
U+674D.0	fR/S	 75.3
U+674D.0	fTotalStrokes	 7
U+674D.0	fCangjie	DND
U+674D.0	fFourCorner	47940
U+674D.0	fCantonese	zi2
U+674D.0	fMandarin	zi3; li3
U+674E.0	fUTF8	李
U+674E.0	fCNS11643	1-4A57
U+674E.0	fBig5	A7F5
U+674E.0	fGB	0-406E
U+674E.0	fR/S	 75.3
U+674E.0	fTotalStrokes	 7
U+674E.0	fCangjie	DND
U+674E.0	fFourCorner	40407
U+674E.0	fHakka	  li3
U+674E.0	fCantonese	lei5; lei2 (also)
U+674E.0	fMandarin	li3
U+674E.0	fEnglish	[1] plum [2] judge; justice [3] a Chinese family name
U+674F.0	fUTF8	杏
U+674F.0	fCNS11643	1-4A58
U+674F.0	fBig5	A7F6
U+674F.0	fGB	0-5053
U+674F.0	fR/S	 75.3
U+674F.0	fTotalStrokes	 7
U+674F.0	fCangjie	DR
U+674F.0	fFourCorner	40609
U+674F.0	fHakka	  hen5; hen3; hen6; hem6
U+674F.0	fCantonese	hang6
U+674F.0	fMandarin	xing4
U+674F.0	fEnglish	[1] apricot [2] almond; apricot kernels [3] apricot flowers
U+6750.0	fUTF8	材
U+6750.0	fCNS11643	1-4A59
U+6750.0	fBig5	A7F7
U+6750.0	fGB	0-3244
U+6750.0	fR/S	 75.3
U+6750.0	fTotalStrokes	 7
U+6750.0	fCangjie	DDH
U+6750.0	fFourCorner	44900
U+6750.0	fHakka	  coi2;  cai2; coi3
U+6750.0	fCantonese	coi4
U+6750.0	fMandarin	cai2
U+6750.0	fEnglish	[1] materials, especially timber for building houses, furniture, etc. [2] material in its broadest sense [3] ability; talent; aptitude [4] coffin [5] properties of a substance
U+6750.1	fUTF8	材
U+6750.1	fCNS11643	1-4A59
U+6750.1	fBig5	A7F7
U+6750.1	fGB	0-3244
U+6750.1	fR/S	 75.3
U+6750.1	fTotalStrokes	 7
U+6750.1	fCangjie	DDH
U+6750.1	fFourCorner	44900
U+6750.1	fMandarin	cai5
U+6751.0	fUTF8	村
U+6751.0	fCNS11643	1-4A5A
U+6751.0	fBig5	A7F8
U+6751.0	fGB	0-3465
U+6751.0	fR/S	 75.3
U+6751.0	fTotalStrokes	 7
U+6751.0	fCangjie	DDI
U+6751.0	fFourCorner	44900
U+6751.0	fHakka	  cun1
U+6751.0	fCantonese	cyun1
U+6751.0	fMandarin	cun1
U+6751.0	fEnglish	[1] village; countryside; hamlet [2] vulgar; coarse [3] naive; simple-minded [4] embarrass [5] scold
U+6752.0	fUTF8	杒
U+6752.0	fCNS11643	3-2645
U+6752.0	fGB	3-4054
U+6752.0	fR/S	  75.3
U+6752.0	fTotalStrokes	 7
U+6752.0	fCangjie	DSHI
U+6752.0	fFourCorner	47920
U+6752.0	fHakka	 niun5
U+6752.0	fCantonese	jan6
U+6752.0	fMandarin	ren4
U+6753.0	fUTF8	杓
U+6753.0	fCNS11643	1-4A61
U+6753.0	fBig5	A840
U+6753.0	fGB	0-683C
U+6753.0	fR/S	 75.3
U+6753.0	fTotalStrokes	 7
U+6753.0	fCangjie	DPI
U+6753.0	fFourCorner	47920
U+6753.0	fHakka	 shok8
U+6753.0	fCantonese	biu1
U+6753.0	fMandarin	shao2; shuo2
U+6753.0	fEnglish	 handle of a cup, receptacle, or container; receptacle; container; ladle; spoon
U+6753.1	fUTF8	杓
U+6753.1	fCNS11643	1-4A61
U+6753.1	fBig5	A840
U+6753.1	fGB	0-683C
U+6753.1	fR/S	 75.3
U+6753.1	fTotalStrokes	 7
U+6753.1	fCangjie	DPI
U+6753.1	fFourCorner	47920
U+6753.1	fHakka	 shok8; sok8; piau1
U+6753.1	fCantonese	soek3; zoek3
U+6753.1	fMandarin	biao1; di2; diao4; piao1; shuo4
U+6753.1	fEnglish	name of a constellation; the handle of the Dipper
U+6754.0	fUTF8	杔
U+6754.0	fCNS11643	3-2644
U+6754.0	fGB	3-404F
U+6754.0	fR/S	  75.3
U+6754.0	fTotalStrokes	 7
U+6754.0	fCangjie	DHP
U+6754.0	fFourCorner	42910
U+6754.0	fCantonese	tok3
U+6754.0	fMandarin	tuo1
U+6755.0	fUTF8	杕
U+6755.0	fCNS11643	2-2441
U+6755.0	fBig5	CB41
U+6755.0	fGB	3-404C
U+6755.0	fR/S	 75.3
U+6755.0	fTotalStrokes	 7
U+6755.0	fCangjie	DK
U+6755.0	fFourCorner	44930
U+6755.0	fHakka	 ti5; ti6
U+6755.0	fCantonese	dai6
U+6755.0	fMandarin	di4; duo4
U+6755.0	fEnglish	alone
U+6756.0	fUTF8	杖
U+6756.0	fCNS11643	1-4A5C
U+6756.0	fBig5	A7FA
U+6756.0	fGB	0-5548
U+6756.0	fR/S	 75.3
U+6756.0	fTotalStrokes	 7
U+6756.0	fCangjie	DJK
U+6756.0	fFourCorner	45900
U+6756.0	fHakka	  cong3; chong6;  chong3; cong5
U+6756.0	fCantonese	zoeng6; zoeng2 (also)
U+6756.0	fMandarin	zhang4
U+6756.0	fEnglish	[1] cane; walking stick; staff; stick [2] beat with a cane (an old punishment); flogging with a stick [3] mourning staff [4] presume on (one's connections, influence, etc.)
U+6757.0	fUTF8	杗
U+6757.0	fCNS11643	1-4A62
U+6757.0	fBig5	A841
U+6757.0	fGB	3-4052
U+6757.0	fR/S	 75.3
U+6757.0	fTotalStrokes	 7
U+6757.0	fCangjie	YVD
U+6757.0	fFourCorner	00904
U+6757.0	fHakka	  mong2
U+6757.0	fCantonese	mong4
U+6757.0	fMandarin	mang2; wang2
U+6757.0	fEnglish	beam
U+6758.0	fUTF8	杘
U+6758.0	fCNS11643	3-2647
U+6758.0	fGB	3-3B4C
U+6758.0	fR/S	 75.3
U+6758.0	fTotalStrokes	 7
U+6758.0	fCangjie	SD
U+6758.0	fFourCorner	77290
U+6758.0	fCantonese	ci3
U+6758.0	fMandarin	chi4; chi1
U+6759.0	fUTF8	杙
U+6759.0	fCNS11643	2-2440
U+6759.0	fBig5	CB40
U+6759.0	fGB	3-404D
U+6759.0	fR/S	 75.3
U+6759.0	fTotalStrokes	 7
U+6759.0	fCangjie	DIP
U+6759.0	fFourCorner	43940
U+6759.0	fHakka	 jit8; rit8
U+6759.0	fCantonese	jik6; dak6
U+6759.0	fMandarin	yi4
U+6759.0	fEnglish	 tiny wooden post; boundary mark or fence
U+675A.0	fUTF8	杚
U+675A.0	fCNS11643	2-2446
U+675A.0	fBig5	CB46
U+675A.0	fGB	5-433C
U+675A.0	fR/S	 75.3
U+675A.0	fTotalStrokes	 7
U+675A.0	fCangjie	DON
U+675A.0	fFourCorner	58917
U+675A.0	fCantonese	koi3; gwat1
U+675A.0	fMandarin	gu3; gai4
U+675B.0	fUTF8	杛
U+675B.0	fCNS11643	3-2642
U+675B.0	fGB	E-2D44
U+675B.0	fR/S	  75.3
U+675B.0	fTotalStrokes	 7
U+675B.0	fCangjie	DN
U+675B.0	fFourCorner	47920
U+675B.0	fCantonese	gung1
U+675B.0	fMandarin	gong1
U+675C.0	fUTF8	杜
U+675C.0	fCNS11643	1-4A5B
U+675C.0	fBig5	A7F9
U+675C.0	fGB	0-3645
U+675C.0	fR/S	 75.3
U+675C.0	fTotalStrokes	 7
U+675C.0	fCangjie	DG
U+675C.0	fFourCorner	44910
U+675C.0	fHakka	 tu5; tu6
U+675C.0	fCantonese	dou6
U+675C.0	fMandarin	du4
U+675C.0	fEnglish	[1] plug (a hole, leak, etc.); stop; prevent; put an end to something [2] shut out; restrict; impede [3] the russet pear (Pyrus betalaefolia) [4] fabricate; practice forgery [5] a Chinese family name
U+675D.0	fUTF8	杝
U+675D.0	fCNS11643	2-2444
U+675D.0	fBig5	CB44
U+675D.0	fGB	3-4053
U+675D.0	fR/S	 75.3
U+675D.0	fTotalStrokes	 7
U+675D.0	fCangjie	DPD
U+675D.0	fFourCorner	44912
U+675D.0	fHakka	 to2; ji2; chi1; ri2; cih1; i2; to3
U+675D.0	fCantonese	ci2; ji4; lei4; zi2 (also); to4
U+675D.0	fMandarin	yi2; chi3; li2; chi2; tuo1; zhi4
U+675D.0	fEnglish	tree
U+675E.0	fUTF8	杞
U+675E.0	fCNS11643	1-4A5D
U+675E.0	fBig5	A7FB
U+675E.0	fGB	0-683D
U+675E.0	fR/S	 75.3
U+675E.0	fTotalStrokes	 7
U+675E.0	fCangjie	DSU
U+675E.0	fFourCorner	47917
U+675E.0	fHakka	 gi3; ki3
U+675E.0	fCantonese	gei2; hei2
U+675E.0	fMandarin	qi3; hai4; si1
U+675E.0	fEnglish	[1] a species of willow [2] medlar (tree) [3] Chi, name of a feudal state in the Chou dynasty in Honan province
U+675F.0	fUTF8	束
U+675F.0	fCNS11643	1-4A56
U+675F.0	fBig5	A7F4
U+675F.0	fGB	0-4A78
U+675F.0	fR/S	 75.3
U+675F.0	fTotalStrokes	 7
U+675F.0	fCangjie	DL
U+675F.0	fFourCorner	50906
U+675F.0	fHakka	  cuk7;  suk7; sok7; {zih3}; ch'uk7; zo3; chuk7
U+675F.0	fCantonese	cuk1; ci3
U+675F.0	fMandarin	shu4; chu4
U+675F.0	fEnglish	[1] bind [2] bunch; bundle [3] control; restrain; restrained [4] a Chinese family name
U+6760.0	fUTF8	杠
U+6760.0	fCNS11643	1-4A60
U+6760.0	fBig5	A7FE
U+6760.0	fGB	0-385C
U+6760.0	fR/S	 75.3
U+6760.0	fTotalStrokes	 7
U+6760.0	fCangjie	DM
U+6760.0	fFourCorner	41910
U+6760.0	fHakka	 gong1;  gong5
U+6760.0	fCantonese	gong3; gong1
U+6760.0	fMandarin	gang1; gang4; gong1
U+6760.0	fEnglish	[1] lever; flag pole [2] bridge
U+6761.0	fUTF8	条
U+6761.0	fCNS11643	3-264A
U+6761.0	fGB	0-4C75
U+6761.0	fR/S	 75.3
U+6761.0	fTotalStrokes	 7
U+6761.0	fCangjie	HED
U+6761.0	fFourCorner	27904
U+6761.0	fHakka	 tiau2
U+6761.0	fCantonese	tiu4; tiu2 (also); tiu5 (also)
U+6761.0	fMandarin	tiao2; di2
U+6761.0	fEnglish	[1] non-classical form of &#x689D; [2] simplified form of &#x689D;
U+6762.0	fUTF8	杢
U+6762.0	fCNS11643	F-2422
U+6762.0	fGB	E-2D45
U+6762.0	fR/S	 75.3
U+6762.0	fTotalStrokes	 7
U+6762.0	fCangjie	DM
U+6762.0	fFourCorner	4010
U+6762.0	fCantonese	gong3
U+6762.0	fEnglish	 woodworker
U+6763.0	fUTF8	杣
U+6763.0	fGB	E-2D46
U+6763.0	fR/S	 75.3
U+6763.0	fTotalStrokes	 7
U+6763.0	fCangjie	XDU
U+6763.0	fFourCorner	4292
U+6763.0	fCantonese	min4
U+6763.0	fEnglish	timber forest; timber, lumber; woodcutter
U+6764.0	fUTF8	杤
U+6764.0	fGB	E-2D47
U+6764.0	fR/S	 75.3
U+6764.0	fTotalStrokes	 7
U+6764.0	fCangjie	DMS
U+6764.0	fCantonese	maan6
U+6764.0	fEnglish	 type of oak; stable
U+6765.0	fUTF8	来
U+6765.0	fCNS11643	4-2446
U+6765.0	fGB	0-4034
U+6765.0	fR/S	 75.3
U+6765.0	fTotalStrokes	 7
U+6765.0	fCangjie	DT
U+6765.0	fFourCorner	50900
U+6765.0	fHakka	 loi2
U+6765.0	fCantonese	loi4; loi2 (also); loi6; lai4; lai2 (also)
U+6765.0	fMandarin	lai2; lai4
U+6765.0	fEnglish	[1] non-classical form of &#x4F86; [2] simplified form of &#x4F86;
U+6766.0	fUTF8	杦
U+6766.0	fCNS11643	F-2453
U+6766.0	fGB	E-2D48
U+6766.0	fR/S	 75.3
U+6766.0	fTotalStrokes	 7
U+6766.0	fCangjie	XDNO
U+6766.0	fFourCorner	4792
U+6766.0	fCantonese	gau2
U+6766.0	fMandarin	shan1
U+6767.0	fUTF8	杧
U+6767.0	fGB	3-4051
U+6767.0	fR/S	 75.3
U+6767.0	fTotalStrokes	 7
U+6767.0	fCangjie	DYV
U+6767.0	fFourCorner	4091
U+6767.0	fCantonese	mong4 (also); mong1
U+6767.0	fMandarin	mang2
U+6767.0	fEnglish	 mango (Mangifera indica)
U+6768.0	fUTF8	杨
U+6768.0	fGB	0-516E
U+6768.0	fR/S	 75.3
U+6768.0	fTotalStrokes	 7
U+6768.0	fCangjie	DNSH
U+6768.0	fHakka	 jong2
U+6768.0	fCantonese	joeng4; joeng2 (also)
U+6768.0	fMandarin	yang2
U+6768.0	fEnglish	simplified form of &#x694A;
U+6769.0	fUTF8	杩
U+6769.0	fGB	0-683F
U+6769.0	fR/S	 75.3
U+6769.0	fTotalStrokes	 7
U+6769.0	fCangjie	DNVM
U+6769.0	fHakka	 ma5
U+6769.0	fCantonese	maa5; maa6
U+6769.0	fMandarin	ma4; ma3
U+6769.0	fEnglish	simplified form of &#x69AA;
U+676A.0	fUTF8	杪
U+676A.0	fCNS11643	1-4E3A
U+676A.0	fBig5	AA57
U+676A.0	fGB	0-6842
U+676A.0	fR/S	 75.4
U+676A.0	fTotalStrokes	 8
U+676A.0	fCangjie	DFH
U+676A.0	fFourCorner	49920
U+676A.0	fHakka	  miau3; cau1; miau1; meu3
U+676A.0	fCantonese	miu5
U+676A.0	fMandarin	miao3; chao1
U+676A.0	fEnglish	[1] tip of twig; tip of a small branch [2] end (of a period)
U+676B.0	fUTF8	杫
U+676B.0	fCNS11643	3-2947
U+676B.0	fGB	3-405D
U+676B.0	fR/S	  75.4
U+676B.0	fTotalStrokes	 8
U+676B.0	fCangjie	DYLM
U+676B.0	fFourCorner	41910
U+676B.0	fCantonese	ci3
U+676B.0	fMandarin	si4
U+676C.0	fUTF8	杬
U+676C.0	fCNS11643	2-2735
U+676C.0	fBig5	CCD4
U+676C.0	fGB	3-4057
U+676C.0	fR/S	 75.4
U+676C.0	fTotalStrokes	 8
U+676C.0	fCangjie	DMMU
U+676C.0	fFourCorner	41911
U+676C.0	fHakka	 ngien2; ngian2
U+676C.0	fCantonese	jyun4; laam5; laam2 (also); wun6; nin4
U+676C.0	fMandarin	yuan2; wan2
U+676C.0	fEnglish	a tall tree found in Kiansi, it has a thick, red bitter bark
U+676D.0	fUTF8	杭
U+676D.0	fCNS11643	1-4E26
U+676D.0	fBig5	AA43
U+676D.0	fGB	0-3A3C
U+676D.0	fR/S	 75.4
U+676D.0	fTotalStrokes	 8
U+676D.0	fCangjie	DYHN
U+676D.0	fFourCorner	40917
U+676D.0	fHakka	  hong2
U+676D.0	fCantonese	hong4
U+676D.0	fMandarin	hang2; kang4; kang1
U+676D.0	fEnglish	[1] same as $; cross stream; navigate [2] Hangchow
U+676E.0	fUTF8	杮
U+676E.0	fCNS11643	4-2669
U+676E.0	fGB	3-405A
U+676E.0	fR/S	 75.4
U+676E.0	fTotalStrokes	 8
U+676E.0	fCangjie	DJB
U+676E.0	fFourCorner	45920
U+676E.0	fHakka	 sii6; pui5
U+676E.0	fCantonese	fai3; ci5 (also); ci3
U+676E.0	fMandarin	lin2; fei4; bei4
U+676E.0	fEnglish	wood shavings
U+676F.0	fUTF8	杯
U+676F.0	fCNS11643	1-4E30
U+676F.0	fBig5	AA4D
U+676F.0	fGB	0-312D
U+676F.0	fR/S	 75.4
U+676F.0	fTotalStrokes	 8
U+676F.0	fCangjie	DMF
U+676F.0	fFourCorner	41990
U+676F.0	fHakka	  bui1; bi1
U+676F.0	fCantonese	bui1
U+676F.0	fMandarin	bei1
U+676F.0	fEnglish	 cup; glass; goblet; tumbler
U+6770.0	fUTF8	杰
U+6770.0	fCNS11643	1-4E31
U+6770.0	fBig5	AA4E
U+6770.0	fGB	0-3D5C
U+6770.0	fR/S	 75.4
U+6770.0	fTotalStrokes	 8
U+6770.0	fCangjie	DF
U+6770.0	fFourCorner	40339
U+6770.0	fHakka	 kiet8;  ket8
U+6770.0	fCantonese	git6
U+6770.0	fMandarin	jie2
U+6770.0	fEnglish	 hero; outstanding person
U+6771.0	fUTF8	東
U+6771.0	fCNS11643	1-4E29
U+6771.0	fBig5	AA46
U+6771.0	fGB	1-362B
U+6771.0	fR/S	 75.4
U+6771.0	fTotalStrokes	 8
U+6771.0	fCangjie	DW
U+6771.0	fFourCorner	50906
U+6771.0	fHakka	  dung1
U+6771.0	fCantonese	dung1
U+6771.0	fMandarin	dong1
U+6771.0	fEnglish	[1] east; eastern [2] travel eastward [3] host; master; owner [4] a Chinese family name
U+6772.0	fUTF8	杲
U+6772.0	fCNS11643	1-4E3B
U+6772.0	fBig5	AA58
U+6772.0	fGB	0-6A3D
U+6772.0	fR/S	 75.4
U+6772.0	fTotalStrokes	 8
U+6772.0	fCangjie	AD
U+6772.0	fFourCorner	60904
U+6772.0	fHakka	 gau3
U+6772.0	fCantonese	gou2
U+6772.0	fMandarin	gao3; hao4
U+6772.0	fEnglish	[1] bright sun; brilliant; bright [2] high
U+6773.0	fUTF8	杳
U+6773.0	fCNS11643	1-4E2B
U+6773.0	fBig5	AA48
U+6773.0	fGB	0-6843
U+6773.0	fR/S	 75.4
U+6773.0	fTotalStrokes	 8
U+6773.0	fCangjie	DA
U+6773.0	fFourCorner	40609
U+6773.0	fHakka	  miau3; miau1; meu3; jau3
U+6773.0	fCantonese	miu5; jiu2 (also)
U+6773.0	fMandarin	yao3; miao3; niao3
U+6773.0	fEnglish	[1] deep and expansive [2] quiet; silent
U+6774.0	fUTF8	杴
U+6774.0	fCNS11643	2-273D
U+6774.0	fBig5	CCDC
U+6774.0	fGB	5-434A
U+6774.0	fR/S	 75.4
U+6774.0	fTotalStrokes	 8
U+6774.0	fCangjie	DNO
U+6774.0	fFourCorner	47982
U+6774.0	fHakka	 kiam1; hiam1
U+6774.0	fCantonese	hin1 (also); him1; hin3; jan1
U+6774.0	fMandarin	xian1; qian1
U+6774.0	fEnglish	 shovel trough used to carry water
U+6775.0	fUTF8	杵
U+6775.0	fCNS11643	1-4E36
U+6775.0	fBig5	AA53
U+6775.0	fGB	0-6846
U+6775.0	fR/S	 75.4
U+6775.0	fTotalStrokes	 8
U+6775.0	fCangjie	DOJ
U+6775.0	fFourCorner	48940
U+6775.0	fHakka	 cu3;  chu3
U+6775.0	fCantonese	cyu5; cyu2
U+6775.0	fMandarin	chu3
U+6775.0	fEnglish	[1] pestle; baton used to beat clothes; baton used to pound the laundry [2] poke
U+6776.0	fUTF8	杶
U+6776.0	fCNS11643	2-2738
U+6776.0	fBig5	CCD7
U+6776.0	fGB	3-405B
U+6776.0	fR/S	 75.4
U+6776.0	fTotalStrokes	 8
U+6776.0	fCangjie	DPU
U+6776.0	fFourCorner	45917
U+6776.0	fHakka	 cun1;  chun1
U+6776.0	fCantonese	ceon1
U+6776.0	fMandarin	chun1; qun1
U+6776.0	fEnglish	varnish
U+6777.0	fUTF8	杷
U+6777.0	fCNS11643	1-4E2C
U+6777.0	fBig5	AA49
U+6777.0	fGB	0-684B
U+6777.0	fR/S	 75.4
U+6777.0	fTotalStrokes	 8
U+6777.0	fCangjie	DAU
U+6777.0	fFourCorner	47917
U+6777.0	fHakka	  pa2; pa3
U+6777.0	fCantonese	paa4
U+6777.0	fMandarin	pa2; ba4; ba5; bai4; pa5
U+6777.0	fEnglish	loquats
U+6778.0	fUTF8	杸
U+6778.0	fCNS11643	2-2747
U+6778.0	fBig5	CCE6
U+6778.0	fGB	5-434B
U+6778.0	fR/S	 75.4
U+6778.0	fTotalStrokes	 8
U+6778.0	fCangjie	DHNE
U+6778.0	fFourCorner	47947
U+6778.0	fHakka	 shu2; su2
U+6778.0	fCantonese	syu4; su4
U+6778.0	fMandarin	shu1; dui4
U+6778.0	fEnglish	 kill spear
U+6779.0	fUTF8	杹
U+6779.0	fCNS11643	2-2748
U+6779.0	fBig5	CCE7
U+6779.0	fGB	3-405F
U+6779.0	fR/S	 75.4
U+6779.0	fTotalStrokes	 8
U+6779.0	fCangjie	DOP
U+6779.0	fFourCorner	41910
U+6779.0	fCantonese	faa3
U+6779.0	fMandarin	hua4
U+677A.0	fUTF8	杺
U+677A.0	fCNS11643	2-2740
U+677A.0	fBig5	CCDF
U+677A.0	fGB	5-434C
U+677A.0	fR/S	 75.4
U+677A.0	fTotalStrokes	 8
U+677A.0	fCangjie	DP
U+677A.0	fFourCorner	43900
U+677A.0	fCantonese	sam1
U+677A.0	fMandarin	xin1; xin2
U+677B.0	fUTF8	杻
U+677B.0	fCNS11643	2-2739
U+677B.0	fBig5	CCD8
U+677B.0	fGB	3-4067
U+677B.0	fR/S	 75.4
U+677B.0	fTotalStrokes	 8
U+677B.0	fCangjie	DNG
U+677B.0	fFourCorner	47915
U+677B.0	fHakka	 ngiu3;  chiu3; cu3; chu3
U+677B.0	fCantonese	cau2; nau2; lau2 (also); nau5; nau4
U+677B.0	fMandarin	chou3; niu3
U+677B.0	fEnglish	ligustrum sinenese; tree
U+677C.0	fUTF8	杼
U+677C.0	fCNS11643	1-4E39
U+677C.0	fBig5	AA56
U+677C.0	fGB	0-684C
U+677C.0	fR/S	 75.4
U+677C.0	fTotalStrokes	 8
U+677C.0	fCangjie	DNIN
U+677C.0	fFourCorner	47922
U+677C.0	fHakka	 chu3; shu3; cu3; shu1; su3; chu1; zhu3
U+677C.0	fCantonese	cyu5
U+677C.0	fMandarin	zhu4; shu4; mao4; shu3
U+677C.0	fEnglish	 shuttle of a loom
U+677D.0	fUTF8	杽
U+677D.0	fCNS11643	2-2745
U+677D.0	fBig5	CCE4
U+677D.0	fGB	5-4343
U+677D.0	fR/S	 75.4
U+677D.0	fTotalStrokes	 8
U+677D.0	fCangjie	DQ
U+677D.0	fFourCorner	42950
U+677D.0	fCantonese	cau2
U+677D.0	fMandarin	chou3; shou3
U+677E.0	fUTF8	松
U+677E.0	fCNS11643	1-4E34
U+677E.0	fBig5	AA51
U+677E.0	fGB	0-4B49
U+677E.0	fR/S	 75.4
U+677E.0	fTotalStrokes	 8
U+677E.0	fCangjie	DCI
U+677E.0	fFourCorner	48932
U+677E.0	fHakka	 ciung2; cung2; sung1; ciung3
U+677E.0	fCantonese	cung4; cung2 (also); sung1
U+677E.0	fMandarin	song1; gong1; song4
U+677E.0	fEnglish	[1] pine tree; fir tree [2] a Chinese family name [3] simplified form of &#x9B06;
U+677F.0	fUTF8	板
U+677F.0	fCNS11643	1-4E32
U+677F.0	fBig5	AA4F
U+677F.0	fGB	0-3065
U+677F.0	fR/S	 75.4
U+677F.0	fTotalStrokes	 8
U+677F.0	fCangjie	DHE
U+677F.0	fFourCorner	41947
U+677F.0	fHakka	  ban3
U+677F.0	fCantonese	baan2
U+677F.0	fMandarin	ban3; ban4
U+677F.0	fEnglish	[1] plank; board [2] plate (of iron, tin, aluminum, etc.); slab [3] printing blocks
U+6780.0	fUTF8	枀
U+6780.0	fCNS11643	3-2952
U+6780.0	fGB	5-4349
U+6780.0	fR/S	  75.4
U+6780.0	fTotalStrokes	 8
U+6780.0	fCangjie	CID
U+6780.0	fFourCorner	80900
U+6780.0	fHakka	 ciung2; cung2; sung1; ciung3
U+6780.0	fCantonese	cung4; cung2 (also); sung1
U+6780.0	fMandarin	song1; gong1; song4
U+6780.0	fEnglish	same as &#x677E;
U+6781.0	fUTF8	极
U+6781.0	fCNS11643	2-2746
U+6781.0	fBig5	CCE5
U+6781.0	fGB	0-3C2B
U+6781.0	fR/S	 75.3
U+6781.0	fTotalStrokes	 7
U+6781.0	fCangjie	DNHE
U+6781.0	fFourCorner	47947
U+6781.0	fHakka	 kip7; kit8
U+6781.0	fCantonese	gik6; kap1
U+6781.0	fMandarin	ji2; ji3
U+6781.0	fEnglish	simplified form of &#x6975;
U+6782.0	fUTF8	枂
U+6782.0	fCNS11643	3-2949
U+6782.0	fGB	3-4062
U+6782.0	fR/S	  75.4
U+6782.0	fTotalStrokes	 8
U+6782.0	fCangjie	DB
U+6782.0	fFourCorner	47920
U+6782.0	fCantonese	wut6
U+6782.0	fMandarin	yue4
U+6783.0	fUTF8	枃
U+6783.0	fCNS11643	2-2744
U+6783.0	fBig5	CCE3
U+6783.0	fGB	3-4063
U+6783.0	fR/S	 75.4
U+6783.0	fTotalStrokes	 8
U+6783.0	fCangjie	DPMM
U+6783.0	fFourCorner	47920
U+6783.0	fCantonese	zeon3
U+6783.0	fMandarin	jin4; yun2; yin4; yun3; yun4
U+6784.0	fUTF8	构
U+6784.0	fCNS11643	2-273C
U+6784.0	fBig5	CCDB
U+6784.0	fGB	0-3939
U+6784.0	fR/S	 75.4
U+6784.0	fTotalStrokes	 8
U+6784.0	fCangjie	DPI
U+6784.0	fFourCorner	47920
U+6784.0	fHakka	 giu1; keu5; giu5; geu5
U+6784.0	fCantonese	kau3; gau3 (also)
U+6784.0	fMandarin	gou4; gou1; jue2
U+6784.0	fEnglish	simplified form of &#x69CB;
U+6785.0	fUTF8	枅
U+6785.0	fCNS11643	2-2734
U+6785.0	fBig5	CCD3
U+6785.0	fGB	3-4055
U+6785.0	fR/S	 75.4
U+6785.0	fTotalStrokes	 8
U+6785.0	fCangjie	DMT
U+6785.0	fFourCorner	41940
U+6785.0	fCantonese	gai1
U+6785.0	fMandarin	ji1; jian1
U+6785.0	fEnglish	a tie-beam connecting two posts or supporting the roof
U+6786.0	fUTF8	枆
U+6786.0	fCNS11643	2-273B
U+6786.0	fBig5	CCDA
U+6786.0	fGB	5-4344
U+6786.0	fR/S	 75.4
U+6786.0	fTotalStrokes	 8
U+6786.0	fCangjie	DHQU
U+6786.0	fFourCorner	42914
U+6786.0	fCantonese	mou4
U+6786.0	fMandarin	mao2; mao1
U+6787.0	fUTF8	枇
U+6787.0	fCNS11643	1-4E2D
U+6787.0	fBig5	AA4A
U+6787.0	fGB	0-6841
U+6787.0	fR/S	 75.4
U+6787.0	fTotalStrokes	 8
U+6787.0	fCangjie	DPP
U+6787.0	fFourCorner	41910
U+6787.0	fHakka	  pi2
U+6787.0	fCantonese	pei4
U+6787.0	fMandarin	pi2; bi3; bi4; pi1; pi3
U+6787.0	fEnglish	loquats
U+6788.0	fUTF8	枈
U+6788.0	fCNS11643	4-2665
U+6788.0	fGB	3-4638
U+6788.0	fR/S	  75.4
U+6788.0	fTotalStrokes	 8
U+6788.0	fCangjie	PPD
U+6788.0	fFourCorner	21900
U+6788.0	fCantonese	bei3
U+6788.0	fMandarin	bi4
U+6789.0	fUTF8	枉
U+6789.0	fCNS11643	1-4E33
U+6789.0	fBig5	AA50
U+6789.0	fGB	0-4D77
U+6789.0	fR/S	 75.4
U+6789.0	fTotalStrokes	 8
U+6789.0	fCangjie	DMG
U+6789.0	fFourCorner	41914
U+6789.0	fHakka	  wong3;  nong5; nong3; vong3
U+6789.0	fCantonese	wong2
U+6789.0	fMandarin	wang3; kuang2
U+6789.0	fEnglish	[1] waste; useless; in vain [2] bent; crooked [3] wrong; do or suffer wrong; aggrieved; oppression [4] request another to deign or condescend to (in polite language)
U+678A.0	fUTF8	枊
U+678A.0	fCNS11643	3-294A
U+678A.0	fGB	3-4065
U+678A.0	fR/S	  75.4
U+678A.0	fTotalStrokes	 8
U+678A.0	fCangjie	DHVL
U+678A.0	fFourCorner	47920
U+678A.0	fHakka	 ngong5
U+678A.0	fCantonese	ngong6; ong6 (also)
U+678A.0	fMandarin	ang4
U+678A.0	fEnglish	[1] post [2] hard; strong
U+678B.0	fUTF8	枋
U+678B.0	fCNS11643	1-4E27
U+678B.0	fBig5	AA44
U+678B.0	fGB	0-684A
U+678B.0	fR/S	 75.4
U+678B.0	fTotalStrokes	 8
U+678B.0	fCangjie	DYHS
U+678B.0	fFourCorner	40927
U+678B.0	fHakka	 fong1; piong1; biang5; biong1
U+678B.0	fCantonese	fong1; beng3
U+678B.0	fMandarin	fang1; bing4
U+678B.0	fEnglish	sandalwood; tree used as timber
U+678C.0	fUTF8	枌
U+678C.0	fCNS11643	2-273F
U+678C.0	fBig5	CCDE
U+678C.0	fGB	3-4061
U+678C.0	fR/S	 75.4
U+678C.0	fTotalStrokes	 8
U+678C.0	fCangjie	DCSH
U+678C.0	fFourCorner	48927
U+678C.0	fHakka	  fun1; fun2
U+678C.0	fCantonese	fan4; fan1
U+678C.0	fMandarin	fen2
U+678C.0	fEnglish	a variety of elm with small seeds and white bark
U+678D.0	fUTF8	枍
U+678D.0	fCNS11643	2-273E
U+678D.0	fBig5	CCDD
U+678D.0	fGB	5-4347
U+678D.0	fR/S	 75.4
U+678D.0	fTotalStrokes	 8
U+678D.0	fCangjie	DCMS
U+678D.0	fFourCorner	48927
U+678D.0	fCantonese	jai6
U+678D.0	fMandarin	yi4
U+678E.0	fUTF8	枎
U+678E.0	fCNS11643	2-2736
U+678E.0	fBig5	CCD5
U+678E.0	fGB	3-4056
U+678E.0	fR/S	 75.4
U+678E.0	fTotalStrokes	 8
U+678E.0	fCangjie	DQO
U+678E.0	fFourCorner	45930
U+678E.0	fHakka	  fu2
U+678E.0	fCantonese	fu4
U+678E.0	fMandarin	fu2; fu1
U+678E.0	fEnglish	t spread out
U+678F.0	fUTF8	枏
U+678F.0	fCNS11643	3-294F
U+678F.0	fGB	E-2D49
U+678F.0	fR/S	 75.4
U+678F.0	fTotalStrokes	 8
U+678F.0	fCangjie	DBMM
U+678F.0	fFourCorner	47950
U+678F.0	fCantonese	naam4; laam4 (also)
U+678F.0	fMandarin	nan2
U+678F.0	fEnglish	a variety of evergreen tree, commonly known as cedar; (Machilus nanmu)
U+6790.0	fUTF8	析
U+6790.0	fCNS11643	1-4E35
U+6790.0	fBig5	AA52
U+6790.0	fGB	0-4E76
U+6790.0	fR/S	 75.4
U+6790.0	fTotalStrokes	 8
U+6790.0	fCangjie	DHML
U+6790.0	fFourCorner	42921
U+6790.0	fHakka	  sit7; sak7
U+6790.0	fCantonese	sik1; cik1 (also)
U+6790.0	fMandarin	xi1; si1
U+6790.0	fEnglish	[1] split; rip or break apart; divide; separate [2] interpret; explain; analyse [3] a Chinese family name
U+6791.0	fUTF8	枑
U+6791.0	fCNS11643	2-2742
U+6791.0	fBig5	CCE1
U+6791.0	fGB	3-405C
U+6791.0	fR/S	 75.4
U+6791.0	fTotalStrokes	 8
U+6791.0	fCangjie	DMVM
U+6791.0	fFourCorner	41917
U+6791.0	fHakka	  fu5
U+6791.0	fCantonese	wu6
U+6791.0	fMandarin	hu4
U+6791.0	fEnglish	railings; fence
U+6792.0	fUTF8	枒
U+6792.0	fCNS11643	2-2737
U+6792.0	fBig5	CCD6
U+6792.0	fGB	E-2D4A
U+6792.0	fR/S	 75.4
U+6792.0	fTotalStrokes	 8
U+6792.0	fCangjie	DMVH
U+6792.0	fFourCorner	41940
U+6792.0	fHakka	  nga2
U+6792.0	fCantonese	aa1; ngaa1 (also); ngaa4
U+6792.0	fMandarin	ya2; ya1; ye2
U+6792.0	fEnglish	[1] the coconut tree [2] felloe of a wheel [3] disorderly; growth of twigs
U+6793.0	fUTF8	枓
U+6793.0	fCNS11643	1-4E38
U+6793.0	fBig5	AA55
U+6793.0	fGB	3-4066
U+6793.0	fR/S	 75.4
U+6793.0	fTotalStrokes	 8
U+6793.0	fCangjie	DYJ
U+6793.0	fFourCorner	44900
U+6793.0	fHakka	  deu3; zhu3; zu3
U+6793.0	fCantonese	dau2; zyu2
U+6793.0	fMandarin	dou3; zhu3; dou4
U+6793.0	fEnglish	capital
U+6794.0	fUTF8	枔
U+6794.0	fCNS11643	2-2749
U+6794.0	fBig5	CCE8
U+6794.0	fGB	5-4348
U+6794.0	fR/S	 75.4
U+6794.0	fTotalStrokes	 8
U+6794.0	fCangjie	DOIN
U+6794.0	fFourCorner	48927
U+6794.0	fCantonese	cam4
U+6794.0	fMandarin	xun2
U+6795.0	fUTF8	枕
U+6795.0	fCNS11643	1-4E28
U+6795.0	fBig5	AA45
U+6795.0	fGB	0-556D
U+6795.0	fR/S	 75.4
U+6795.0	fTotalStrokes	 8
U+6795.0	fCangjie	DLBU
U+6795.0	fFourCorner	44912
U+6795.0	fHakka	  zhim3;  zim3; zihm3
U+6795.0	fCantonese	zam2; zam3
U+6795.0	fMandarin	zhen3; chen2
U+6795.0	fEnglish	 pillow
U+6795.1	fUTF8	枕
U+6795.1	fCNS11643	1-4E28
U+6795.1	fBig5	AA45
U+6795.1	fGB	0-556D
U+6795.1	fR/S	 75.4
U+6795.1	fTotalStrokes	 8
U+6795.1	fCangjie	DLBU
U+6795.1	fFourCorner	44912
U+6795.1	fMandarin	zhen4
U+6795.1	fEnglish	 use something as a pillow; pillow
U+6796.0	fUTF8	枖
U+6796.0	fCNS11643	3-2948
U+6796.0	fGB	5-4345
U+6796.0	fR/S	  75.4
U+6796.0	fTotalStrokes	 8
U+6796.0	fCangjie	DHK
U+6796.0	fFourCorner	42930
U+6796.0	fCantonese	jiu1
U+6796.0	fMandarin	yao1; yao3
U+6797.0	fUTF8	林
U+6797.0	fCNS11643	1-4E2F
U+6797.0	fBig5	AA4C
U+6797.0	fGB	0-4156
U+6797.0	fR/S	 75.4
U+6797.0	fTotalStrokes	 8
U+6797.0	fCangjie	DD
U+6797.0	fFourCorner	44990
U+6797.0	fHakka	  lim2; lim3
U+6797.0	fCantonese	lam4; lam2 (also)
U+6797.0	fMandarin	lin2
U+6797.0	fEnglish	[1] forest; grove; copse [2] collection of books, works, literary extracts, etc. [3] circles; many; numerous; a great body of (capable persons, etc.) [4] a Chinese family name
U+6798.0	fUTF8	枘
U+6798.0	fCNS11643	2-273A
U+6798.0	fBig5	CCD9
U+6798.0	fGB	0-6844
U+6798.0	fR/S	 75.4
U+6798.0	fTotalStrokes	 8
U+6798.0	fCangjie	DOB
U+6798.0	fFourCorner	44927
U+6798.0	fHakka	 nui5; nui6; jui3; jui5
U+6798.0	fCantonese	jeoi6
U+6798.0	fMandarin	rui4
U+6798.0	fEnglish	 wooden handle (tools, etc.)
U+6799.0	fUTF8	枙
U+6799.0	fCNS11643	2-2743
U+6799.0	fBig5	CCE2
U+6799.0	fGB	5-4341
U+6799.0	fR/S	 75.4
U+6799.0	fTotalStrokes	 8
U+6799.0	fCangjie	DMSU
U+6799.0	fFourCorner	41912
U+6799.0	fHakka	 ak7
U+6799.0	fCantonese	ngo4; ak1
U+6799.0	fMandarin	e4; e2; e3
U+679A.0	fUTF8	枚
U+679A.0	fCNS11643	1-4E37
U+679A.0	fBig5	AA54
U+679A.0	fGB	0-4336
U+679A.0	fR/S	 75.4
U+679A.0	fTotalStrokes	 8
U+679A.0	fCangjie	DOK
U+679A.0	fFourCorner	48940
U+679A.0	fHakka	 moi2; mi2; mui2; moi3; moe2
U+679A.0	fCantonese	mui4; mui2 (also)
U+679A.0	fMandarin	mei2
U+679A.0	fEnglish	[1] stalk; trunk of a tree (as opposed to branches) [2] numerical classifier (coins, fruits, stamps, bombs, etc.) [3] a gag for troops marching at night when silence means a lot [4] a Chinese family name
U+679B.0	fUTF8	枛
U+679B.0	fCNS11643	4-2667
U+679B.0	fGB	E-2D4B
U+679B.0	fR/S	 75.4
U+679B.0	fTotalStrokes	 8
U+679B.0	fCangjie	DHLO
U+679B.0	fFourCorner	42930
U+679B.0	fCantonese	zaau3
U+679B.0	fMandarin	zhao4
U+679C.0	fUTF8	果
U+679C.0	fCNS11643	1-4E2A
U+679C.0	fBig5	AA47
U+679C.0	fGB	0-397B
U+679C.0	fR/S	 75.4
U+679C.0	fTotalStrokes	 8
U+679C.0	fCangjie	WD
U+679C.0	fFourCorner	60904
U+679C.0	fHakka	  go3; gwo3; kwo3
U+679C.0	fCantonese	gwo2; go2 (also); gu2
U+679C.0	fMandarin	guo3; luo3; guan4; ke4; wo3
U+679C.0	fEnglish	[1] fruit of a plant [2] effect (in cause and effect); result; fruit; consequence [3] surely; really; truly; exactly [4] stuff; fill [5] succeed [6] a Chinese family name
U+679D.0	fUTF8	枝
U+679D.0	fCNS11643	1-4E2E
U+679D.0	fBig5	AA4B
U+679D.0	fGB	0-5626
U+679D.0	fR/S	 75.4
U+679D.0	fTotalStrokes	 8
U+679D.0	fCangjie	DJE
U+679D.0	fFourCorner	44947
U+679D.0	fHakka	  gi1;  zi1;  zhi1; zih1; ki1; tje1; {gi1}
U+679D.0	fCantonese	zi1; kei4
U+679D.0	fMandarin	zhi1; gui4
U+679D.0	fEnglish	[1] branches of a tree; branch [2] limbs [3] branch off
U+679D.1	fUTF8	枝
U+679D.1	fCNS11643	1-4E2E
U+679D.1	fBig5	AA4B
U+679D.1	fGB	0-5626
U+679D.1	fR/S	 75.4
U+679D.1	fTotalStrokes	 8
U+679D.1	fCangjie	DJE
U+679D.1	fFourCorner	44947
U+679D.1	fMandarin	qi2
U+679D.1	fEnglish	[1] an additional finger; forked finger
U+679E.0	fUTF8	枞
U+679E.0	fCNS11643	4-266A
U+679E.0	fGB	0-6848
U+679E.0	fR/S	 75.4
U+679E.0	fTotalStrokes	 8
U+679E.0	fCangjie	DOO
U+679E.0	fFourCorner	48900
U+679E.0	fHakka	 ciung2; ziung1
U+679E.0	fCantonese	cung1; zung1
U+679E.0	fMandarin	cong1; zong1
U+679E.0	fEnglish	[1] same as &#x6A05; [2] simplified form of &#x6A05;
U+679F.0	fUTF8	枟
U+679F.0	fCNS11643	2-2741
U+679F.0	fBig5	CCE0
U+679F.0	fGB	3-4058
U+679F.0	fR/S	 75.4
U+679F.0	fTotalStrokes	 8
U+679F.0	fCangjie	DMMI
U+679F.0	fFourCorner	41931
U+679F.0	fCantonese	taan4; wan4
U+679F.0	fMandarin	yun4; yun2
U+679F.0	fEnglish	wood streaks
U+67A0.0	fUTF8	枠
U+67A0.0	fGB	E-2D4C
U+67A0.0	fR/S	 75.4
U+67A0.0	fTotalStrokes	 8
U+67A0.0	fCangjie	DKNJ
U+67A0.0	fFourCorner	4494
U+67A0.0	fCantonese	zyut6
U+67A0.0	fEnglish	a frame; a reel, spindle, spool
U+67A1.0	fUTF8	枡
U+67A1.0	fCNS11643	F-274E
U+67A1.0	fGB	E-2D4D
U+67A1.0	fR/S	 75.4
U+67A1.0	fTotalStrokes	 8
U+67A1.0	fCangjie	DHT
U+67A1.0	fFourCorner	4490
U+67A1.0	fCantonese	sing1
U+67A1.0	fMandarin	dou3
U+67A1.0	fEnglish	measuring box
U+67A2.0	fUTF8	枢
U+67A2.0	fCNS11643	3-294E
U+67A2.0	fGB	0-4A60
U+67A2.0	fR/S	 75.4
U+67A2.0	fTotalStrokes	 8
U+67A2.0	fCangjie	DSK
U+67A2.0	fFourCorner	41910
U+67A2.0	fHakka	 ki1
U+67A2.0	fCantonese	syu1
U+67A2.0	fMandarin	shu1; chu1; ou1
U+67A2.0	fEnglish	simplified form of &#x6A1E;
U+67A3.0	fUTF8	枣
U+67A3.0	fCNS11643	3-294C
U+67A3.0	fGB	0-5466
U+67A3.0	fR/S	 75.4
U+67A3.0	fTotalStrokes	 8
U+67A3.0	fCangjie	DBY
U+67A3.0	fFourCorner	50300
U+67A3.0	fHakka	 zau3
U+67A3.0	fCantonese	zou2
U+67A3.0	fMandarin	zao3
U+67A3.0	fEnglish	simplified form of &#x68D7;
U+67A4.0	fUTF8	枤
U+67A4.0	fCNS11643	3-6528
U+67A4.0	fGB	E-2D4E
U+67A4.0	fR/S	 75.4
U+67A4.0	fTotalStrokes	 8
U+67A4.0	fCangjie	DIK
U+67A4.0	fFourCorner	4393
U+67A4.0	fCantonese	dai6
U+67A5.0	fUTF8	枥
U+67A5.0	fGB	0-6840
U+67A5.0	fR/S	 75.4
U+67A5.0	fTotalStrokes	 8
U+67A5.0	fCangjie	DMKS
U+67A5.0	fHakka	 lit8
U+67A5.0	fCantonese	lik1
U+67A5.0	fMandarin	li4
U+67A5.0	fEnglish	simplified form of &#x6AEA;
U+67A6.0	fUTF8	枦
U+67A6.0	fCNS11643	F-2750
U+67A6.0	fGB	E-2D4F
U+67A6.0	fR/S	 75.4
U+67A6.0	fTotalStrokes	 8
U+67A6.0	fCangjie	DIS
U+67A6.0	fCantonese	lou4
U+67A6.0	fMandarin	lu2
U+67A7.0	fUTF8	枧
U+67A7.0	fGB	0-6845
U+67A7.0	fR/S	 75.4
U+67A7.0	fTotalStrokes	 8
U+67A7.0	fCangjie	DBHU
U+67A7.0	fHakka	 gan3; gen5
U+67A7.0	fCantonese	gaan2
U+67A7.0	fMandarin	jian3; jian4; xian4
U+67A7.0	fEnglish	simplified form of &#x6898;
U+67A8.0	fUTF8	枨
U+67A8.0	fGB	0-6847
U+67A8.0	fR/S	 75.4
U+67A8.0	fTotalStrokes	 8
U+67A8.0	fCangjie	DPO
U+67A8.0	fHakka	 cen2
U+67A8.0	fCantonese	caang4; caam4; caan5
U+67A8.0	fMandarin	cheng2
U+67A8.0	fEnglish	simplified form of &#x68D6;
U+67A9.0	fUTF8	枩
U+67A9.0	fGB	E-2D50
U+67A9.0	fR/S	 75.4
U+67A9.0	fTotalStrokes	 8
U+67A9.0	fCangjie	XDCI
U+67A9.0	fCantonese	cung4
U+67A9.0	fMandarin	song1
U+67A9.0	fEnglish	 pine tree; fir tree
U+67AA.0	fUTF8	枪
U+67AA.0	fGB	0-4739
U+67AA.0	fR/S	 75.4
U+67AA.0	fTotalStrokes	 8
U+67AA.0	fCangjie	DOSU
U+67AA.0	fHakka	 ciong1
U+67AA.0	fCantonese	coeng1
U+67AA.0	fMandarin	qiang1; cheng1
U+67AA.0	fEnglish	simplified form of &#x69CD;
U+67AB.0	fUTF8	枫
U+67AB.0	fGB	0-3763
U+67AB.0	fR/S	 75.4
U+67AB.0	fTotalStrokes	 8
U+67AB.0	fCangjie	DHNK
U+67AB.0	fHakka	 fung1
U+67AB.0	fCantonese	fung1
U+67AB.0	fMandarin	feng1; bian1; fan1
U+67AB.0	fEnglish	simplified form of &#x6953;
U+67AC.0	fUTF8	枬
U+67AC.0	fCNS11643	3-6529
U+67AC.0	fGB	E-2D51
U+67AC.0	fR/S	 75.4
U+67AC.0	fTotalStrokes	 8
U+67AC.0	fCangjie	DBY
U+67AC.0	fFourCorner	4794
U+67AC.0	fHakka	 nam2
U+67AC.0	fCantonese	zin1
U+67AC.0	fMandarin	nan2
U+67AC.0	fEnglish	 an even-grained yellowish fine wood used for furniture cedar
U+67AD.0	fUTF8	枭
U+67AD.0	fGB	0-6849
U+67AD.0	fR/S	 75.4
U+67AD.0	fTotalStrokes	 8
U+67AD.0	fCangjie	PYSD
U+67AD.0	fHakka	 hiau1
U+67AD.0	fCantonese	hiu1
U+67AD.0	fMandarin	xiao1; jiao1
U+67AD.0	fEnglish	simplified form of &#x689F;
U+67AE.0	fUTF8	枮
U+67AE.0	fCNS11643	2-2B3D
U+67AE.0	fBig5	CF5B
U+67AE.0	fGB	5-4354
U+67AE.0	fR/S	 75.5
U+67AE.0	fTotalStrokes	 9
U+67AE.0	fCangjie	DYR
U+67AE.0	fFourCorner	41960
U+67AE.0	fHakka	 zem1
U+67AE.0	fCantonese	cim1; zam1
U+67AE.0	fMandarin	xian1; zhen1
U+67AF.0	fUTF8	枯
U+67AF.0	fCNS11643	1-515F
U+67AF.0	fBig5	AC5C
U+67AF.0	fGB	0-3F5D
U+67AF.0	fR/S	 75.5
U+67AF.0	fTotalStrokes	 9
U+67AF.0	fCangjie	DJR
U+67AF.0	fFourCorner	44960
U+67AF.0	fHakka	  ku1
U+67AF.0	fCantonese	fu1
U+67AF.0	fMandarin	ku1
U+67AF.0	fEnglish	[1] dry; withered; decayed [2] dried wood [3] ill health; emaciated
U+67B0.0	fUTF8	枰
U+67B0.0	fCNS11643	1-516C
U+67B0.0	fBig5	AC69
U+67B0.0	fGB	0-6852
U+67B0.0	fR/S	 75.5
U+67B0.0	fTotalStrokes	 9
U+67B0.0	fCangjie	DMFJ
U+67B0.0	fFourCorner	4194
U+67B0.0	fHakka	 pin2; pin3
U+67B0.0	fCantonese	ping4
U+67B0.0	fMandarin	ping2
U+67B0.0	fEnglish	[1] smooth board; chessboard [2] chess; chess game
U+67B1.0	fUTF8	枱
U+67B1.0	fCNS11643	3-2D36
U+67B1.0	fGB	5-435F
U+67B1.0	fR/S	  75.5
U+67B1.0	fTotalStrokes	 9
U+67B1.0	fCangjie	DIR
U+67B1.0	fFourCorner	43960
U+67B1.0	fHakka	  toi2; ji2; ri2; toi1
U+67B1.0	fCantonese	toi4 (also); toi2
U+67B1.0	fMandarin	yi2; si4; ci2; tai2
U+67B1.0	fEnglish	[1] same as $ [2] an ancient unit of measurement [3] handle of a plough
U+67B2.0	fUTF8	枲
U+67B2.0	fCNS11643	2-2B38
U+67B2.0	fBig5	CF56
U+67B2.0	fGB	3-3353
U+67B2.0	fR/S	 75.5
U+67B2.0	fTotalStrokes	 9
U+67B2.0	fCangjie	IRD
U+67B2.0	fFourCorner	23904
U+67B2.0	fHakka	  sai3
U+67B2.0	fCantonese	saai2; sai2
U+67B2.0	fMandarin	xi3
U+67B2.0	fEnglish	 male nettle-hemp
U+67B3.0	fUTF8	枳
U+67B3.0	fCNS11643	2-2B2E
U+67B3.0	fBig5	CF4C
U+67B3.0	fGB	0-6857
U+67B3.0	fR/S	 75.5
U+67B3.0	fTotalStrokes	 9
U+67B3.0	fCangjie	DRC
U+67B3.0	fFourCorner	46980
U+67B3.0	fHakka	 zhit7; zhi3; zih3; tsiit7
U+67B3.0	fCantonese	zi2; zik1; zat1; zeot1
U+67B3.0	fMandarin	zhi1; zhi3; zhi2; ji3; qi2
U+67B3.0	fEnglish	[1] trifoliate orange [2] a variety of orange with a very thick skin
U+67B4.0	fUTF8	枴
U+67B4.0	fCNS11643	1-5165
U+67B4.0	fBig5	AC62
U+67B4.0	fGB	E-2D52
U+67B4.0	fR/S	 75.5
U+67B4.0	fTotalStrokes	 9
U+67B4.0	fCangjie	DRSH
U+67B4.0	fFourCorner	46927
U+67B4.0	fHakka	 kuai3; gai3
U+67B4.0	fCantonese	gwaai2
U+67B4.0	fMandarin	guai3; gua3; zhai4
U+67B4.0	fEnglish	 cane; staff for an old person
U+67B5.0	fUTF8	枵
U+67B5.0	fCNS11643	2-2B2C
U+67B5.0	fBig5	CF4A
U+67B5.0	fGB	0-6855
U+67B5.0	fR/S	 75.5
U+67B5.0	fTotalStrokes	 9
U+67B5.0	fCangjie	DRMS
U+67B5.0	fFourCorner	46927
U+67B5.0	fHakka	  hiau1; iau1
U+67B5.0	fCantonese	hiu1
U+67B5.0	fMandarin	xiao1
U+67B5.0	fEnglish	empty
U+67B6.0	fUTF8	架
U+67B6.0	fCNS11643	1-515E
U+67B6.0	fBig5	AC5B
U+67B6.0	fGB	0-3C5C
U+67B6.0	fR/S	 75.5
U+67B6.0	fTotalStrokes	 9
U+67B6.0	fCangjie	KRD
U+67B6.0	fFourCorner	46904
U+67B6.0	fHakka	  ga5
U+67B6.0	fCantonese	gaa3; gaa2; gaa1 (also)
U+67B6.0	fMandarin	jia4
U+67B6.0	fEnglish	[1] rack; stand; prop; frame [2] prop up; set up; support [3] framework; scaffold [4] frame up (a charge, etc.); fabricate [5] lay something on [6] quarrel
U+67B7.0	fUTF8	枷
U+67B7.0	fCNS11643	2-2B27
U+67B7.0	fBig5	CF45
U+67B7.0	fGB	0-3C4F
U+67B7.0	fR/S	 75.5
U+67B7.0	fTotalStrokes	 9
U+67B7.0	fCangjie	DKSR
U+67B7.0	fFourCorner	46900
U+67B7.0	fHakka	  ga1; ka2
U+67B7.0	fCantonese	gaa1
U+67B7.0	fMandarin	jia1
U+67B7.0	fEnglish	 cangue; pillory
U+67B8.0	fUTF8	枸
U+67B8.0	fCNS11643	1-5168
U+67B8.0	fBig5	AC65
U+67B8.0	fGB	0-685B
U+67B8.0	fR/S	 75.5
U+67B8.0	fTotalStrokes	 9
U+67B8.0	fCangjie	DPR
U+67B8.0	fFourCorner	47920
U+67B8.0	fHakka	 kieu3; gi3; giu3; geu3; geu1; gi1
U+67B8.0	fCantonese	gau2; geoi2; gau1
U+67B8.0	fMandarin	gou3; gou2; ju1; qu2
U+67B8.0	fEnglish	medlar
U+67B8.1	fUTF8	枸
U+67B8.1	fCNS11643	1-5168
U+67B8.1	fBig5	AC65
U+67B8.1	fGB	0-685B
U+67B8.1	fR/S	 75.5
U+67B8.1	fTotalStrokes	 9
U+67B8.1	fCangjie	DPR
U+67B8.1	fFourCorner	47920
U+67B8.1	fMandarin	gou1
U+67B8.1	fEnglish	used in combination
U+67B8.2	fUTF8	枸
U+67B8.2	fCNS11643	1-5168
U+67B8.2	fBig5	AC65
U+67B8.2	fGB	0-685B
U+67B8.2	fR/S	 75.5
U+67B8.2	fTotalStrokes	 9
U+67B8.2	fCangjie	DPR
U+67B8.2	fFourCorner	47920
U+67B8.2	fMandarin	ju3
U+67B8.2	fEnglish	used in combination
U+67B9.0	fUTF8	枹
U+67B9.0	fCNS11643	2-2B34
U+67B9.0	fBig5	CF52
U+67B9.0	fGB	3-4079
U+67B9.0	fR/S	 75.5
U+67B9.0	fTotalStrokes	 9
U+67B9.0	fCangjie	DPRU
U+67B9.0	fFourCorner	47912
U+67B9.0	fHakka	 fu1; bau1
U+67B9.0	fCantonese	baau1; fu1; fau4
U+67B9.0	fMandarin	bao1; fu1; fu2; fou2
U+67B9.0	fEnglish	 drumstick
U+67BA.0	fUTF8	枺
U+67BA.0	fCNS11643	2-2B21
U+67BA.0	fBig5	CEFE
U+67BA.0	fGB	3-4068
U+67BA.0	fR/S	 75.5
U+67BA.0	fTotalStrokes	 9
U+67BA.0	fCangjie	DDJ
U+67BA.0	fFourCorner	45990
U+67BA.0	fCantonese	mut6
U+67BA.0	fMandarin	mo4
U+67BB.0	fUTF8	枻
U+67BB.0	fCNS11643	2-2B23
U+67BB.0	fBig5	CF41
U+67BB.0	fGB	3-406A
U+67BB.0	fR/S	 75.5
U+67BB.0	fTotalStrokes	 9
U+67BB.0	fCangjie	DPT
U+67BB.0	fFourCorner	44917
U+67BB.0	fHakka	 siet7; ri3; ji3; ji5; i5
U+67BB.0	fCantonese	jai6; ji6; sit3
U+67BB.0	fMandarin	yi4; xie4
U+67BB.0	fEnglish	[1] rowing sweep [2] an instrument for correcting a bow
U+67BC.0	fUTF8	枼
U+67BC.0	fCNS11643	3-2D2E
U+67BC.0	fGB	5-4352
U+67BC.0	fR/S	  75.5
U+67BC.0	fTotalStrokes	 9
U+67BC.0	fCangjie	PTD
U+67BC.0	fFourCorner	44900
U+67BC.0	fCantonese	jip6
U+67BC.0	fMandarin	ye4
U+67BC.0	fEnglish	a table; flat pieces of wood; a slip; a leaf
U+67BD.0	fUTF8	枽
U+67BD.0	fCNS11643	4-2D59
U+67BD.0	fGB	E-2D53
U+67BD.0	fR/S	 75.5
U+67BD.0	fTotalStrokes	 10; 9
U+67BD.0	fCangjie	JTD
U+67BD.0	fFourCorner	40900
U+67BD.0	fCantonese	jip6
U+67BD.0	fMandarin	ye4
U+67BD.0	fEnglish	same as &#x67BC;
U+67BE.0	fUTF8	枾
U+67BE.0	fCNS11643	4-296E
U+67BE.0	fGB	E-2D54
U+67BE.0	fR/S	  75.5
U+67BE.0	fTotalStrokes	 9
U+67BE.0	fCangjie	DQLL
U+67BE.0	fFourCorner	45920
U+67BE.0	fCantonese	si6
U+67BE.0	fMandarin	shi4
U+67BE.0	fEnglish	persimmon
U+67BF.0	fUTF8	枿
U+67BF.0	fCNS11643	3-2D30
U+67BF.0	fGB	3-4069
U+67BF.0	fR/S	 75.5
U+67BF.0	fTotalStrokes	 9
U+67BF.0	fCangjie	DJT
U+67BF.0	fFourCorner	44940
U+67BF.0	fCantonese	jit6; jip6 (also)
U+67BF.0	fMandarin	nie4
U+67C0.0	fUTF8	柀
U+67C0.0	fCNS11643	2-2B26
U+67C0.0	fBig5	CF44
U+67C0.0	fGB	3-4122
U+67C0.0	fR/S	 75.5
U+67C0.0	fTotalStrokes	 9
U+67C0.0	fCangjie	DDHE
U+67C0.0	fFourCorner	44947
U+67C0.0	fHakka	 bi3
U+67C0.0	fCantonese	bei2; bei3
U+67C0.0	fMandarin	bi3; pi3
U+67C0.0	fEnglish	Chinese pine Cuninghamia Sinensis
U+67C1.0	fUTF8	柁
U+67C1.0	fCNS11643	2-2A7C
U+67C1.0	fBig5	CEFB
U+67C1.0	fGB	0-685E
U+67C1.0	fR/S	 75.5
U+67C1.0	fTotalStrokes	 9
U+67C1.0	fCangjie	DJP
U+67C1.0	fFourCorner	43911
U+67C1.0	fHakka	  to5; to3; to2
U+67C1.0	fCantonese	to4; to5 (also); taai5
U+67C1.0	fMandarin	duo4; duo1; tuo3
U+67C1.0	fEnglish	 rudder or helm of a ship; rudder
U+67C1.1	fUTF8	柁
U+67C1.1	fCNS11643	2-2A7C
U+67C1.1	fBig5	CEFB
U+67C1.1	fGB	0-685E
U+67C1.1	fR/S	 75.5
U+67C1.1	fTotalStrokes	 9
U+67C1.1	fCangjie	DJP
U+67C1.1	fFourCorner	43911
U+67C1.1	fMandarin	tuo2
U+67C1.1	fEnglish	large tie beams; girders
U+67C2.0	fUTF8	柂
U+67C2.0	fCNS11643	2-2B33
U+67C2.0	fBig5	CF51
U+67C2.0	fGB	3-4073
U+67C2.0	fR/S	 75.5
U+67C2.0	fTotalStrokes	 9
U+67C2.0	fCangjie	DOPD
U+67C2.0	fFourCorner	48912
U+67C2.0	fHakka	 to5
U+67C2.0	fCantonese	ji4; to4; to5 (also); ci2
U+67C2.0	fMandarin	yi2; duo4; li2
U+67C2.0	fEnglish	helm
U+67C3.0	fUTF8	柃
U+67C3.0	fCNS11643	2-2B43
U+67C3.0	fBig5	CF61
U+67C3.0	fGB	0-685A
U+67C3.0	fR/S	 75.5
U+67C3.0	fTotalStrokes	 9
U+67C3.0	fCangjie	DOII
U+67C3.0	fFourCorner	48927
U+67C3.0	fHakka	 lin2;  lang2
U+67C3.0	fCantonese	ling4
U+67C3.0	fMandarin	ling2; ling3
U+67C3.0	fEnglish	Eurya japonica, the ashes of which are used in dyeing
U+67C4.0	fUTF8	柄
U+67C4.0	fCNS11643	1-5163
U+67C4.0	fBig5	AC60
U+67C4.0	fGB	0-317A
U+67C4.0	fR/S	 75.5
U+67C4.0	fTotalStrokes	 9
U+67C4.0	fCangjie	DMOB
U+67C4.0	fFourCorner	41927
U+67C4.0	fHakka	  biang5
U+67C4.0	fCantonese	beng3; bing3 (also)
U+67C4.0	fMandarin	bing3; bing4
U+67C4.0	fEnglish	[1] handle of something; knob [2] authority; power [3] operate; handle; control [4] handle
U+67C5.0	fUTF8	柅
U+67C5.0	fCNS11643	2-2B28
U+67C5.0	fBig5	CF46
U+67C5.0	fGB	3-407D
U+67C5.0	fR/S	 75.5
U+67C5.0	fTotalStrokes	 9
U+67C5.0	fCangjie	DSP
U+67C5.0	fFourCorner	47911
U+67C5.0	fHakka	 ni3; ni2
U+67C5.0	fCantonese	nei5; lei5 (also)
U+67C5.0	fMandarin	ni3; chi4; ni2
U+67C5.0	fEnglish	flourish
U+67C6.0	fUTF8	柆
U+67C6.0	fCNS11643	2-2B3A
U+67C6.0	fBig5	CF58
U+67C6.0	fGB	5-435B
U+67C6.0	fR/S	 75.5
U+67C6.0	fTotalStrokes	 9
U+67C6.0	fCangjie	DYT
U+67C6.0	fFourCorner	40918
U+67C6.0	fCantonese	laap6
U+67C6.0	fMandarin	la1
U+67C7.0	fUTF8	柇
U+67C7.0	fCNS11643	3-2D2D
U+67C7.0	fGB	3-4072
U+67C7.0	fR/S	  75.5
U+67C7.0	fTotalStrokes	 9
U+67C7.0	fCangjie	DHD
U+67C7.0	fFourCorner	42990
U+67C7.0	fCantonese	wo4
U+67C7.0	fMandarin	he2
U+67C8.0	fUTF8	柈
U+67C8.0	fCNS11643	2-2A7E
U+67C8.0	fBig5	CEFD
U+67C8.0	fGB	3-407B
U+67C8.0	fR/S	 75.5
U+67C8.0	fTotalStrokes	 9
U+67C8.0	fCangjie	DFQ
U+67C8.0	fFourCorner	49950
U+67C8.0	fCantonese	bun6; pun6 (also); pun4
U+67C8.0	fMandarin	pan2; pan4
U+67C8.0	fEnglish	container
U+67C9.0	fUTF8	柉
U+67C9.0	fCNS11643	2-2B41
U+67C9.0	fBig5	CF5F
U+67C9.0	fGB	E-2D55
U+67C9.0	fR/S	 75.5; 75.4
U+67C9.0	fTotalStrokes	 9
U+67C9.0	fCangjie	DHIO
U+67C9.0	fFourCorner	42937
U+67C9.0	fCantonese	faan4
U+67C9.0	fMandarin	fan2; bian1
U+67CA.0	fUTF8	柊
U+67CA.0	fCNS11643	2-2B42
U+67CA.0	fBig5	CF60
U+67CA.0	fGB	3-4076
U+67CA.0	fR/S	 75.5
U+67CA.0	fTotalStrokes	 9
U+67CA.0	fCangjie	DHEY
U+67CA.0	fFourCorner	47933
U+67CA.0	fCantonese	dung1; zung1
U+67CA.0	fMandarin	zhong1
U+67CA.0	fEnglish	holly
U+67CB.0	fUTF8	柋
U+67CB.0	fCNS11643	2-2B45
U+67CB.0	fBig5	CF63
U+67CB.0	fGB	E-2D56
U+67CB.0	fR/S	 75.5
U+67CB.0	fTotalStrokes	 9
U+67CB.0	fCangjie	OPD
U+67CB.0	fFourCorner	23904
U+67CB.0	fCantonese	doi6
U+67CB.0	fMandarin	dai4
U+67CC.0	fUTF8	柌
U+67CC.0	fCNS11643	2-2B3C
U+67CC.0	fBig5	CF5A
U+67CC.0	fGB	5-435C
U+67CC.0	fR/S	 75.5
U+67CC.0	fTotalStrokes	 9
U+67CC.0	fCangjie	DSMR
U+67CC.0	fFourCorner	47920
U+67CC.0	fCantonese	ci4
U+67CC.0	fMandarin	ci2
U+67CD.0	fUTF8	柍
U+67CD.0	fCNS11643	2-2B2D
U+67CD.0	fBig5	CF4B
U+67CD.0	fGB	3-406E
U+67CD.0	fR/S	 75.5
U+67CD.0	fTotalStrokes	 9
U+67CD.0	fCangjie	DLBK
U+67CD.0	fFourCorner	45930
U+67CD.0	fCantonese	joeng1; joeng2
U+67CD.0	fMandarin	yang1; ang4; yang4; ying1
U+67CE.0	fUTF8	柎
U+67CE.0	fCNS11643	2-2B35
U+67CE.0	fBig5	CF53
U+67CE.0	fGB	3-4074
U+67CE.0	fR/S	 75.5
U+67CE.0	fTotalStrokes	 9
U+67CE.0	fCangjie	DODI
U+67CE.0	fFourCorner	44900
U+67CE.0	fHakka	 fu5
U+67CE.0	fCantonese	fu1; fu2
U+67CE.0	fMandarin	fu1; fu4; fu3
U+67CE.0	fEnglish	 calyx of a flower
U+67CF.0	fUTF8	柏
U+67CF.0	fCNS11643	1-5169
U+67CF.0	fBig5	AC66
U+67CF.0	fGB	0-3058
U+67CF.0	fR/S	 75.5
U+67CF.0	fTotalStrokes	 9
U+67CF.0	fCangjie	DHA
U+67CF.0	fFourCorner	46900
U+67CF.0	fHakka	 bak7; pak7
U+67CF.0	fCantonese	paak3; baak3
U+67CF.0	fMandarin	bai3; bo2; bo4
U+67CF.0	fEnglish	[1] cypress; cedar [2] a Chinese family name
U+67D0.0	fUTF8	某
U+67D0.0	fCNS11643	1-515C
U+67D0.0	fBig5	AC59
U+67D0.0	fGB	0-4433
U+67D0.0	fR/S	 75.5
U+67D0.0	fTotalStrokes	 9
U+67D0.0	fCangjie	TMD
U+67D0.0	fFourCorner	44904
U+67D0.0	fHakka	  meu1; meu3; miu1; mu1
U+67D0.0	fCantonese	mau5
U+67D0.0	fMandarin	mou3; mei2; mu3
U+67D0.0	fEnglish	[1] a certain thing or person [2] formerly used in place of "I"
U+67D1.0	fUTF8	柑
U+67D1.0	fCNS11643	1-5164
U+67D1.0	fBig5	AC61
U+67D1.0	fGB	0-384C
U+67D1.0	fR/S	 75.5
U+67D1.0	fTotalStrokes	 9
U+67D1.0	fCangjie	DTM
U+67D1.0	fFourCorner	44970
U+67D1.0	fHakka	  gam1
U+67D1.0	fCantonese	gam1; kim4
U+67D1.0	fMandarin	gan1; qian2
U+67D1.0	fEnglish	tangerine; loose-skinned orange; mandarin orange
U+67D2.0	fUTF8	柒
U+67D2.0	fCNS11643	1-5170
U+67D2.0	fBig5	AC6D
U+67D2.0	fGB	0-4662
U+67D2.0	fR/S	 75.5
U+67D2.0	fTotalStrokes	 9
U+67D2.0	fCangjie	EPD
U+67D2.0	fFourCorner	34904
U+67D2.0	fHakka	 cit7
U+67D2.0	fCantonese	cat1
U+67D2.0	fMandarin	qi1
U+67D2.0	fEnglish	seven (used in accountancy)
U+67D3.0	fUTF8	染
U+67D3.0	fCNS11643	1-5159
U+67D3.0	fBig5	AC56
U+67D3.0	fGB	0-483E
U+67D3.0	fR/S	 75.5
U+67D3.0	fTotalStrokes	 9
U+67D3.0	fCangjie	END
U+67D3.0	fFourCorner	34904
U+67D3.0	fHakka	 ngiam3;  ngiam5; ngiam6; gniam6
U+67D3.0	fCantonese	jim5
U+67D3.0	fMandarin	ran3; ran4
U+67D3.0	fEnglish	[1] dye [2] be contagious; infect; get infected; catch a disease; infectious [3] soil; pollute [4] have an affair with [5] make strokes
U+67D4.0	fUTF8	柔
U+67D4.0	fCNS11643	1-515B
U+67D4.0	fBig5	AC58
U+67D4.0	fGB	0-4861
U+67D4.0	fR/S	 75.5
U+67D4.0	fTotalStrokes	 9
U+67D4.0	fCangjie	NHD
U+67D4.0	fFourCorner	17904
U+67D4.0	fHakka	  ju2;  jiu2; jeu2; riu2
U+67D4.0	fCantonese	jau4
U+67D4.0	fMandarin	rou2
U+67D4.0	fEnglish	[1] soft; tender [2] amiable; gentle; pliant; yielding; submissive; supple [3] the new grass budding in spring
U+67D5.0	fUTF8	柕
U+67D5.0	fCNS11643	3-2D31
U+67D5.0	fGB	E-2D57
U+67D5.0	fR/S	  75.5
U+67D5.0	fTotalStrokes	 9
U+67D5.0	fCangjie	DNIH
U+67D5.0	fFourCorner	47920
U+67D5.0	fHakka	 muk8; miu3; meu5; miau3
U+67D5.0	fCantonese	mau6
U+67D5.0	fMandarin	mao4
U+67D5.0	fEnglish	same as &#x6959;
U+67D6.0	fUTF8	柖
U+67D6.0	fCNS11643	3-2D2B
U+67D6.0	fGB	3-4121
U+67D6.0	fR/S	 75.5
U+67D6.0	fTotalStrokes	 9
U+67D6.0	fCangjie	DSHR
U+67D6.0	fFourCorner	47960
U+67D6.0	fCantonese	siu4
U+67D6.0	fMandarin	zhao1
U+67D7.0	fUTF8	柗
U+67D7.0	fCNS11643	3-2D3A
U+67D7.0	fGB	E-2D58
U+67D7.0	fR/S	 75.5
U+67D7.0	fTotalStrokes	 9
U+67D7.0	fCangjie	DCR
U+67D7.0	fFourCorner	48960
U+67D7.0	fHakka	 ciung2; cung2; sung1; ciung3
U+67D7.0	fCantonese	cung4; cung2 (also); sung1
U+67D7.0	fMandarin	song1; gong1; song4
U+67D7.0	fEnglish	same as &#x677E;
U+67D8.0	fUTF8	柘
U+67D8.0	fCNS11643	2-2B25
U+67D8.0	fBig5	CF43
U+67D8.0	fGB	0-684F
U+67D8.0	fR/S	 75.5
U+67D8.0	fTotalStrokes	 9
U+67D8.0	fCangjie	DMR
U+67D8.0	fFourCorner	41960
U+67D8.0	fHakka	 za5;  zha5
U+67D8.0	fCantonese	ze3
U+67D8.0	fMandarin	zhe4
U+67D8.0	fEnglish	[1] a thorny tree (Cudrania tricuspidata) [2] sugarcane
U+67D9.0	fUTF8	柙
U+67D9.0	fCNS11643	1-516D
U+67D9.0	fBig5	AC6A
U+67D9.0	fGB	0-6854
U+67D9.0	fR/S	 75.5
U+67D9.0	fTotalStrokes	 9
U+67D9.0	fCangjie	DWL
U+67D9.0	fFourCorner	46950
U+67D9.0	fHakka	  ap7
U+67D9.0	fCantonese	haap6; gaap3; aat3 (also); ngaat3 (also)
U+67D9.0	fMandarin	xia2; jia3; ya1
U+67D9.0	fEnglish	[1] cage; pen for wild animals, especially the fierce ones [2] scabbard; case for a sword
U+67DA.0	fUTF8	柚
U+67DA.0	fCNS11643	1-5166
U+67DA.0	fBig5	AC63
U+67DA.0	fGB	0-6856
U+67DA.0	fR/S	 75.5
U+67DA.0	fTotalStrokes	 9
U+67DA.0	fCangjie	DLW
U+67DA.0	fFourCorner	45960
U+67DA.0	fHakka	 ju5; jiu3; riu6; ju3; jiu6; cuk8
U+67DA.0	fCantonese	jau2; jau6 (also); jau4 (also); zuk6; jau2zuk6
U+67DA.0	fMandarin	you4; you2; zhu2
U+67DA.0	fEnglish	[1] pumelo; pomelo [2] teak; teak tree
U+67DB.0	fUTF8	柛
U+67DB.0	fCNS11643	2-2B3F
U+67DB.0	fBig5	CF5D
U+67DB.0	fGB	5-4358
U+67DB.0	fR/S	 75.5
U+67DB.0	fTotalStrokes	 9
U+67DB.0	fCangjie	DLWL
U+67DB.0	fFourCorner	45906
U+67DB.0	fHakka	 shin1; siin1
U+67DB.0	fCantonese	san1
U+67DB.0	fMandarin	shen1
U+67DB.0	fEnglish	trees that die of themselves
U+67DC.0	fUTF8	柜
U+67DC.0	fCNS11643	2-2B22
U+67DC.0	fBig5	CF40
U+67DC.0	fGB	0-3971
U+67DC.0	fR/S	 75.5
U+67DC.0	fTotalStrokes	 9
U+67DC.0	fCangjie	DSS
U+67DC.0	fFourCorner	41917
U+67DC.0	fHakka	 gi3; kui5; kui3; k'wui5
U+67DC.0	fCantonese	geoi2; gwai6; gwai2 (also)
U+67DC.0	fMandarin	gui4; ju3
U+67DC.0	fEnglish	[1] a tree of the willow family [2] same as $; a very large tree whose beautiful fine-grained wood is good for making furniture [3] simplified form of &#x6AC3;
U+67DD.0	fUTF8	柝
U+67DD.0	fCNS11643	1-516F
U+67DD.0	fBig5	AC6C
U+67DD.0	fGB	0-6858
U+67DD.0	fR/S	 75.5
U+67DD.0	fTotalStrokes	 9
U+67DD.0	fCangjie	DHMY
U+67DD.0	fFourCorner	42941
U+67DD.0	fHakka	  tok8
U+67DD.0	fCantonese	tok3
U+67DD.0	fMandarin	tuo4
U+67DD.0	fEnglish	 watchman's rattle; watchman's clapper; knocker
U+67DE.0	fUTF8	柞
U+67DE.0	fCNS11643	1-516A
U+67DE.0	fBig5	AC67
U+67DE.0	fGB	0-5775
U+67DE.0	fR/S	 75.5
U+67DE.0	fTotalStrokes	 9
U+67DE.0	fCangjie	DOS
U+67DE.0	fFourCorner	48911
U+67DE.0	fHakka	 cok7; zia5; cok8
U+67DE.0	fCantonese	zok6; zaak3; zaa3; zaa6 (also); zok3
U+67DE.0	fMandarin	zuo4; zha4; ze2; zhai3; zuo2
U+67DE.0	fEnglish	an evergreen thorny tree with small leaves, fine and sturdy wood (Xylosma congesta); oak
U+67DF.0	fUTF8	柟
U+67DF.0	fCNS11643	2-2B2B
U+67DF.0	fBig5	CF49
U+67DF.0	fGB	E-2D59
U+67DF.0	fR/S	 75.5
U+67DF.0	fTotalStrokes	 9
U+67DF.0	fCangjie	DGB
U+67DF.0	fFourCorner	45957
U+67DF.0	fCantonese	naam4; laam4 (also); naam1; jim4
U+67DF.0	fMandarin	nan2
U+67DF.0	fEnglish	standard form of &#x678F;
U+67E0.0	fUTF8	柠
U+67E0.0	fCNS11643	4-296D
U+67E0.0	fGB	0-447B
U+67E0.0	fR/S	 75.5
U+67E0.0	fTotalStrokes	 9
U+67E0.0	fCangjie	DJMN
U+67E0.0	fFourCorner	43920
U+67E0.0	fHakka	 nen2
U+67E0.0	fCantonese	ning4; ling4 (also)
U+67E0.0	fMandarin	ning2; neng3
U+67E0.0	fEnglish	simplified form of &#x6AB8;
U+67E1.0	fUTF8	柡
U+67E1.0	fCNS11643	4-2D4E
U+67E1.0	fGB	E-2D5A
U+67E1.0	fR/S	  75.5
U+67E1.0	fTotalStrokes	 9; 10
U+67E1.0	fCangjie	DMME
U+67E1.0	fFourCorner	41930
U+67E1.0	fCantonese	wing5
U+67E1.0	fMandarin	yong3
U+67E2.0	fUTF8	柢
U+67E2.0	fCNS11643	1-516E
U+67E2.0	fBig5	AC6B
U+67E2.0	fGB	0-685C
U+67E2.0	fR/S	 75.5
U+67E2.0	fTotalStrokes	 9
U+67E2.0	fCangjie	DHPM
U+67E2.0	fFourCorner	42940
U+67E2.0	fHakka	 di3
U+67E2.0	fCantonese	dai2
U+67E2.0	fMandarin	di3; chi2; di1; di4
U+67E2.0	fEnglish	 root; base; bottom of object; foundation
U+67E3.0	fUTF8	柣
U+67E3.0	fCNS11643	2-2B32
U+67E3.0	fBig5	CF50
U+67E3.0	fGB	3-4071
U+67E3.0	fR/S	 75.5
U+67E3.0	fTotalStrokes	 9
U+67E3.0	fCangjie	DHQO
U+67E3.0	fFourCorner	45930
U+67E3.0	fCantonese	dit6; cit3
U+67E3.0	fMandarin	zhi2; die2; zhi4
U+67E3.0	fEnglish	sill
U+67E4.0	fUTF8	柤
U+67E4.0	fCNS11643	2-2B2A
U+67E4.0	fBig5	CF48
U+67E4.0	fGB	5-4357
U+67E4.0	fR/S	 75.5
U+67E4.0	fTotalStrokes	 9
U+67E4.0	fCangjie	DBM
U+67E4.0	fFourCorner	47910
U+67E4.0	fHakka	  za1
U+67E4.0	fCantonese	zaa1; zo2
U+67E4.0	fMandarin	zha1; ju1; cha1; zu3; zhu2
U+67E4.0	fEnglish	hawthorn
U+67E5.0	fUTF8	查
U+67E5.0	fCNS11643	1-5167
U+67E5.0	fBig5	AC64
U+67E5.0	fGB	0-3269
U+67E5.0	fR/S	 75.5
U+67E5.0	fTotalStrokes	 9
U+67E5.0	fCangjie	DAM
U+67E5.0	fFourCorner	40107
U+67E5.0	fHakka	  ca2; ca3; za5; za1
U+67E5.0	fCantonese	caa4; zaa1; caa2 (also)
U+67E5.0	fMandarin	cha2; chai2
U+67E5.0	fEnglish	[1] investigate; examine; seek into; seek out; look into; check [2] It appears ...; It seems ...; It is known ...; It is found that (used at the beginning of the official correspondence) [3] wooden raft
U+67E5.1	fUTF8	查
U+67E5.1	fCNS11643	1-5167
U+67E5.1	fBig5	AC64
U+67E5.1	fGB	0-3269
U+67E5.1	fR/S	 75.5
U+67E5.1	fTotalStrokes	 9
U+67E5.1	fCangjie	DAM
U+67E5.1	fFourCorner	40107
U+67E5.1	fMandarin	zha1
U+67E5.1	fEnglish	a Chinese family name
U+67E6.0	fUTF8	柦
U+67E6.0	fCNS11643	2-2B3E
U+67E6.0	fBig5	CF5C
U+67E6.0	fGB	5-4356
U+67E6.0	fR/S	 75.5
U+67E6.0	fTotalStrokes	 9
U+67E6.0	fCangjie	DAM
U+67E6.0	fFourCorner	46910
U+67E6.0	fCantonese	daan6
U+67E6.0	fMandarin	dan4
U+67E7.0	fUTF8	柧
U+67E7.0	fCNS11643	2-2B36
U+67E7.0	fBig5	CF54
U+67E7.0	fGB	5-435A
U+67E7.0	fR/S	 75.5
U+67E7.0	fTotalStrokes	 9
U+67E7.0	fCangjie	DHVO
U+67E7.0	fFourCorner	42930
U+67E7.0	fHakka	  gu1
U+67E7.0	fCantonese	gu1; gwu1 (also)
U+67E7.0	fMandarin	gu1; gua1
U+67E7.0	fEnglish	[1] corner; angle [2] triangular
U+67E8.0	fUTF8	柨
U+67E8.0	fCNS11643	3-2D2A
U+67E8.0	fGB	3-406C
U+67E8.0	fR/S	  75.5
U+67E8.0	fTotalStrokes	 9
U+67E8.0	fCangjie	DKLB
U+67E8.0	fFourCorner	44920
U+67E8.0	fCantonese	bou3
U+67E8.0	fMandarin	pu1
U+67E9.0	fUTF8	柩
U+67E9.0	fCNS11643	1-5161
U+67E9.0	fBig5	AC5E
U+67E9.0	fGB	0-6851
U+67E9.0	fR/S	 75.5
U+67E9.0	fTotalStrokes	 9
U+67E9.0	fCangjie	DSNO
U+67E9.0	fFourCorner	41918
U+67E9.0	fHakka	  giu5; giu3
U+67E9.0	fCantonese	gau6; gau3 (also)
U+67E9.0	fMandarin	jiu4
U+67E9.0	fEnglish	 coffin which contains a corpse
U+67EA.0	fUTF8	柪
U+67EA.0	fCNS11643	2-2B44
U+67EA.0	fBig5	CF62
U+67EA.0	fGB	3-4125
U+67EA.0	fR/S	 75.5
U+67EA.0	fTotalStrokes	 9
U+67EA.0	fCangjie	DVIS
U+67EA.0	fFourCorner	44927
U+67EA.0	fCantonese	au1; ngau1 (also); aau1; aau3
U+67EA.0	fMandarin	ao1; ao4
U+67EB.0	fUTF8	柫
U+67EB.0	fCNS11643	2-2B29
U+67EB.0	fBig5	CF47
U+67EB.0	fGB	E-2D5B
U+67EB.0	fR/S	 75.5
U+67EB.0	fTotalStrokes	 9
U+67EB.0	fCangjie	DLLN
U+67EB.0	fFourCorner	45927
U+67EB.0	fCantonese	fat1
U+67EB.0	fMandarin	fu2; bi4
U+67EC.0	fUTF8	柬
U+67EC.0	fCNS11643	1-515D
U+67EC.0	fBig5	AC5A
U+67EC.0	fGB	0-3C6D
U+67EC.0	fR/S	 75.5
U+67EC.0	fTotalStrokes	 9
U+67EC.0	fCangjie	DWF
U+67EC.0	fFourCorner	50906
U+67EC.0	fHakka	 gan3;  kian3; gan1; gian3
U+67EC.0	fCantonese	gaan2
U+67EC.0	fMandarin	jian3
U+67EC.0	fEnglish	[1] letter; invitation; visiting card [2] choose; select; pick
U+67ED.0	fUTF8	柭
U+67ED.0	fCNS11643	2-2B3B
U+67ED.0	fBig5	CF59
U+67ED.0	fGB	3-406D
U+67ED.0	fR/S	 75.5
U+67ED.0	fTotalStrokes	 9
U+67ED.0	fCangjie	DIKK
U+67ED.0	fFourCorner	43947
U+67ED.0	fCantonese	baat3; bat6; fat1; but6
U+67ED.0	fMandarin	bo1; ba1; ba2; fu2
U+67EE.0	fUTF8	柮
U+67EE.0	fCNS11643	2-2B31
U+67EE.0	fBig5	CF4F
U+67EE.0	fGB	3-407E
U+67EE.0	fR/S	 75.5
U+67EE.0	fTotalStrokes	 9
U+67EE.0	fCangjie	DUU
U+67EE.0	fFourCorner	42972
U+67EE.0	fHakka	  tut7;  nut8
U+67EE.0	fCantonese	deot1; dut1 (also); zyut3
U+67EE.0	fMandarin	duo4; wu4
U+67EE.0	fEnglish	 firewood
U+67EF.0	fUTF8	柯
U+67EF.0	fCNS11643	1-5162
U+67EF.0	fBig5	AC5F
U+67EF.0	fGB	0-3F42
U+67EF.0	fR/S	 75.5
U+67EF.0	fTotalStrokes	 9
U+67EF.0	fCangjie	DMNR
U+67EF.0	fFourCorner	41920
U+67EF.0	fHakka	 ko1; ho1
U+67EF.0	fCantonese	ngo1 (also); o1
U+67EF.0	fMandarin	ke1; ge1
U+67EF.0	fEnglish	[1] axe-handle [2] stalk of a plant; trunk of a plant [3] a Chinese family name [4] a tall evergreen tree (Pasania cuspidata)
U+67F0.0	fUTF8	柰
U+67F0.0	fCNS11643	2-2B37
U+67F0.0	fBig5	CF55
U+67F0.0	fGB	0-684D
U+67F0.0	fR/S	 75.5
U+67F0.0	fTotalStrokes	 9
U+67F0.0	fCangjie	DMMF
U+67F0.0	fFourCorner	40901
U+67F0.0	fHakka	 nai5; nai6; lai5; lai3
U+67F0.0	fCantonese	noi6; loi6 (also)
U+67F0.0	fMandarin	nai4
U+67F0.0	fEnglish	[1] crab-apple tree [2] how [3] then what? [4] endure; bear [5] sometimes used with a negative-(no) way out of a dilemma
U+67F1.0	fUTF8	柱
U+67F1.0	fCNS11643	1-515A
U+67F1.0	fBig5	AC57
U+67F1.0	fGB	0-5679
U+67F1.0	fR/S	 75.5
U+67F1.0	fTotalStrokes	 9
U+67F1.0	fCangjie	DYG
U+67F1.0	fFourCorner	40914
U+67F1.0	fHakka	 cu1;  chu1; zu3; cu3; chu3; {chu1}; chu5
U+67F1.0	fCantonese	cyu5; cyu2 (also)
U+67F1.0	fMandarin	zhu4; zhu3
U+67F1.0	fEnglish	[1] pillar; post [2] support; lean on [3] cylinder [4] stab; pierce
U+67F2.0	fUTF8	柲
U+67F2.0	fCNS11643	2-2A7D
U+67F2.0	fBig5	CEFC
U+67F2.0	fGB	3-407C
U+67F2.0	fR/S	 75.5
U+67F2.0	fTotalStrokes	 9
U+67F2.0	fCangjie	DPH
U+67F2.0	fFourCorner	43900
U+67F2.0	fCantonese	bei3
U+67F2.0	fMandarin	bi4; bie2; tao4
U+67F2.0	fEnglish	handle
U+67F3.0	fUTF8	柳
U+67F3.0	fCNS11643	1-516B
U+67F3.0	fBig5	AC68
U+67F3.0	fGB	0-4178
U+67F3.0	fR/S	 75.5
U+67F3.0	fTotalStrokes	 9
U+67F3.0	fCangjie	DHHL
U+67F3.0	fFourCorner	47920
U+67F3.0	fHakka	  liu3; liu1; lieu3; leu1
U+67F3.0	fCantonese	lau5
U+67F3.0	fMandarin	liu3; ping2
U+67F3.0	fEnglish	[1] willow tree [2] name of one of the 28 constellations [3] singsong house; red-light district [4] tumor; swelling; lump [5] a Chinese family name
U+67F4.0	fUTF8	柴
U+67F4.0	fCNS11643	1-5627
U+67F4.0	fBig5	AEE3
U+67F4.0	fGB	0-3271
U+67F4.0	fR/S	 75.5
U+67F4.0	fTotalStrokes	 9
U+67F4.0	fCangjie	YPD
U+67F4.0	fFourCorner	21904
U+67F4.0	fHakka	  cai2;  ciau2; ciau3; ceu2
U+67F4.0	fCantonese	caai4; caai2 (also); zaai6
U+67F4.0	fMandarin	chai2; zhai4; zi4; cha2; ci1
U+67F4.0	fEnglish	[1] firewood; faggots; fuel; brushwood [2] thin; emaciated [3] fence [4] a Chinese family name [5] how
U+67F5.0	fUTF8	柵
U+67F5.0	fCNS11643	1-5160
U+67F5.0	fBig5	AC5D
U+67F5.0	fGB	E-2D5C
U+67F5.0	fR/S	 75.5
U+67F5.0	fTotalStrokes	 9
U+67F5.0	fCangjie	DBT
U+67F5.0	fFourCorner	47940
U+67F5.0	fHakka	 cak8
U+67F5.0	fCantonese	caak3; saan1; zaap3; saan3
U+67F5.0	fMandarin	zha4; ce4; shan4; sha4; shuan4; shan1
U+67F5.0	fEnglish	 fence of bamboos or wood; palisade; railing of posts; window-bars
U+67F6.0	fUTF8	柶
U+67F6.0	fCNS11643	2-2B30
U+67F6.0	fBig5	CF4E
U+67F6.0	fGB	3-4070
U+67F6.0	fR/S	 75.5
U+67F6.0	fTotalStrokes	 9
U+67F6.0	fCangjie	DWC
U+67F6.0	fFourCorner	56900
U+67F6.0	fCantonese	si3; sei3
U+67F6.0	fMandarin	si4; qie4
U+67F6.0	fEnglish	a dagger-shaped ritual vessel in ancient China, made of horn
U+67F7.0	fUTF8	柷
U+67F7.0	fCNS11643	2-2B2F
U+67F7.0	fBig5	CF4D
U+67F7.0	fGB	3-406F
U+67F7.0	fR/S	 75.5
U+67F7.0	fTotalStrokes	 9
U+67F7.0	fCangjie	DRHU
U+67F7.0	fFourCorner	46910
U+67F7.0	fHakka	 chuk7; zuk7; zhuk7
U+67F7.0	fCantonese	zuk1; cuk1
U+67F7.0	fMandarin	zhu4; chu4
U+67F7.0	fEnglish	an ancient instrument made of wood which is played at the start of an orchestra
U+67F8.0	fUTF8	柸
U+67F8.0	fCNS11643	2-2B24
U+67F8.0	fBig5	CF42
U+67F8.0	fGB	3-406B
U+67F8.0	fR/S	 75.5
U+67F8.0	fTotalStrokes	 9
U+67F8.0	fCangjie	DMFM
U+67F8.0	fFourCorner	41919
U+67F8.0	fCantonese	pui1
U+67F8.0	fMandarin	pei1
U+67F8.0	fEnglish	 collect; gather shed built cont
U+67F9.0	fUTF8	柹
U+67F9.0	fCNS11643	3-2951
U+67F9.0	fGB	E-2D5D
U+67F9.0	fR/S	 75.5; 75.4
U+67F9.0	fTotalStrokes	 8; 9
U+67F9.0	fCangjie	DLXH
U+67F9.0	fFourCorner	45900
U+67F9.0	fCantonese	ci2
U+67F9.0	fMandarin	shi4
U+67F9.0	fEnglish	persimmon
U+67FA.0	fUTF8	柺
U+67FA.0	fCNS11643	2-2B40
U+67FA.0	fBig5	CF5E
U+67FA.0	fGB	E-2D5E
U+67FA.0	fR/S	 75.5
U+67FA.0	fTotalStrokes	 9
U+67FA.0	fCangjie	DRKS
U+67FA.0	fFourCorner	46927
U+67FA.0	fCantonese	gwaai2
U+67FA.0	fMandarin	guai3
U+67FA.0	fEnglish	an old man's staff
U+67FB.0	fUTF8	査
U+67FB.0	fCNS11643	3-2D34
U+67FB.0	fGB	E-2D5F
U+67FB.0	fR/S	 75.5
U+67FB.0	fTotalStrokes	 9
U+67FB.0	fCangjie	DBM
U+67FB.0	fFourCorner	40100
U+67FB.0	fCantonese	caa4
U+67FB.0	fMandarin	cha2; zha1
U+67FB.0	fEnglish	investigate; examine; seek into
U+67FC.0	fUTF8	柼
U+67FC.0	fCNS11643	2-2B39
U+67FC.0	fBig5	CF57
U+67FC.0	fGB	E-2D60
U+67FC.0	fR/S	 75.5
U+67FC.0	fTotalStrokes	 9
U+67FC.0	fCangjie	DJC
U+67FC.0	fFourCorner	43982
U+67FC.0	fCantonese	jiu2
U+67FC.0	fMandarin	yao3; yao2
U+67FD.0	fUTF8	柽
U+67FD.0	fGB	0-685F
U+67FD.0	fR/S	 75.5
U+67FD.0	fTotalStrokes	 9
U+67FD.0	fCangjie	DEG
U+67FD.0	fHakka	 shin5
U+67FD.0	fCantonese	cing1
U+67FD.0	fMandarin	cheng1; jue2
U+67FD.0	fEnglish	simplified form of &#x6A89;
U+67FE.0	fUTF8	柾
U+67FE.0	fCNS11643	3-2D32
U+67FE.0	fGB	E-2D61
U+67FE.0	fR/S	 75.5
U+67FE.0	fTotalStrokes	 9
U+67FE.0	fCangjie	DMYM
U+67FE.0	fFourCorner	41910
U+67FE.0	fHakka	 giu5
U+67FE.0	fCantonese	gau6
U+67FE.0	fMandarin	jiu4
U+67FE.0	fEnglish	same as &#x67E9;
U+67FF.0	fUTF8	柿
U+67FF.0	fCNS11643	1-5158
U+67FF.0	fBig5	AC55
U+67FF.0	fGB	0-4A41
U+67FF.0	fR/S	 75.5
U+67FF.0	fTotalStrokes	 9
U+67FF.0	fCangjie	DYLB
U+67FF.0	fFourCorner	45927
U+67FF.0	fHakka	 se2;  sai3; shi6; si5; ki6; ki5; s'e2; sii5; se1; {sai3}; se5
U+67FF.0	fCantonese	ci5 (also); si5 (also); ci2
U+67FF.0	fMandarin	shi4
U+67FF.0	fEnglish	[1] the persimmon [2] non-classical form of &#x67F9;
U+6800.0	fUTF8	栀
U+6800.0	fCNS11643	3-2D39
U+6800.0	fGB	0-6859
U+6800.0	fR/S	 75.5
U+6800.0	fTotalStrokes	 9
U+6800.0	fCangjie	DHMU
U+6800.0	fFourCorner	42910
U+6800.0	fCantonese	zi1
U+6800.0	fMandarin	zhi1
U+6800.0	fEnglish	gardenia
U+6801.0	fUTF8	栁
U+6801.0	fCNS11643	3-2D37
U+6801.0	fGB	E-2D62
U+6801.0	fR/S	 75.5; 75.6
U+6801.0	fTotalStrokes	 9; 10
U+6801.0	fCangjie	DNIL
U+6801.0	fFourCorner	47920
U+6801.0	fHakka	  liu3; liu1; lieu3; leu1
U+6801.0	fCantonese	lau5
U+6801.0	fMandarin	liu3; ping2
U+6801.0	fEnglish	same as &#x67F3;
U+6802.0	fUTF8	栂
U+6802.0	fGB	E-2D63
U+6802.0	fR/S	 75.5
U+6802.0	fTotalStrokes	 9
U+6802.0	fCangjie	DWYI
U+6802.0	fFourCorner	4795
U+6802.0	fCantonese	mui4
U+6802.0	fMandarin	mei2
U+6802.0	fEnglish	a kind of evergreen tree
U+6803.0	fUTF8	栃
U+6803.0	fGB	E-2D64
U+6803.0	fR/S	 75.5
U+6803.0	fTotalStrokes	 9
U+6803.0	fCangjie	DHMS
U+6803.0	fFourCorner	4192
U+6803.0	fCantonese	lai6
U+6803.0	fEnglish	 type of oak stable
U+6804.0	fUTF8	栄
U+6804.0	fCNS11643	6-3172
U+6804.0	fGB	E-2D65
U+6804.0	fR/S	 75.5
U+6804.0	fTotalStrokes	 9
U+6804.0	fCangjie	FBD
U+6804.0	fFourCorner	90900
U+6804.0	fHakka	 jin2; jung2
U+6804.0	fCantonese	wing4
U+6804.0	fMandarin	rong2; ying2
U+6804.0	fEnglish	glory, honor; flourish, prosper
U+6805.0	fUTF8	栅
U+6805.0	fCNS11643	3-2D38
U+6805.0	fGB	0-5524
U+6805.0	fR/S	 75.5
U+6805.0	fTotalStrokes	 9
U+6805.0	fCangjie	DBBM
U+6805.0	fFourCorner	47940
U+6805.0	fHakka	  cak8; ch'ak7; san1; cak7
U+6805.0	fCantonese	caak3; saan1; cak3; san1; saan3
U+6805.0	fMandarin	zha4; ce4; shan1; sha4
U+6805.0	fEnglish	 fence palisade grid
U+6806.0	fUTF8	栆
U+6806.0	fCNS11643	3-652B
U+6806.0	fGB	E-2D66
U+6806.0	fR/S	 75.5
U+6806.0	fTotalStrokes	 9
U+6806.0	fCangjie	KLY
U+6806.0	fFourCorner	5030
U+6806.0	fHakka	  zau3;  zo3
U+6806.0	fCantonese	zou2
U+6806.0	fMandarin	zao3
U+6806.0	fEnglish	simplified form of &#x68D7;
U+6807.0	fUTF8	标
U+6807.0	fGB	0-316A
U+6807.0	fR/S	 75.5
U+6807.0	fTotalStrokes	 9
U+6807.0	fCangjie	DMMF
U+6807.0	fHakka	 piau1; biau1
U+6807.0	fCantonese	biu1
U+6807.0	fMandarin	biao1; biao3
U+6807.0	fEnglish	simplified form of &#x6A19;
U+6808.0	fUTF8	栈
U+6808.0	fGB	0-553B
U+6808.0	fR/S	 75.5
U+6808.0	fTotalStrokes	 9
U+6808.0	fCangjie	DIJ
U+6808.0	fHakka	 zan5; zan3
U+6808.0	fCantonese	zaan6; zaan2 (also); zaan3
U+6808.0	fMandarin	zhan4; chen2
U+6808.0	fEnglish	simplified form of &#x68E7;
U+6809.0	fUTF8	栉
U+6809.0	fGB	0-684E
U+6809.0	fR/S	 75.5
U+6809.0	fTotalStrokes	 9
U+6809.0	fCangjie	DTSL
U+6809.0	fHakka	 ziet7
U+6809.0	fCantonese	zit3; zit1
U+6809.0	fMandarin	zhi4; jie2
U+6809.0	fEnglish	simplified form of &#x6ADB;
U+680A.0	fUTF8	栊
U+680A.0	fGB	0-6850
U+680A.0	fR/S	 75.5
U+680A.0	fTotalStrokes	 9
U+680A.0	fCangjie	DIKP
U+680A.0	fHakka	 lung2
U+680A.0	fCantonese	lung4
U+680A.0	fMandarin	long2
U+680A.0	fEnglish	simplified form of &#x6AF3;
U+680B.0	fUTF8	栋
U+680B.0	fGB	0-3630
U+680B.0	fR/S	 75.5
U+680B.0	fTotalStrokes	 9
U+680B.0	fCangjie	DKD
U+680B.0	fHakka	 dung5
U+680B.0	fCantonese	dung3 (also); dung6
U+680B.0	fMandarin	dong4
U+680B.0	fEnglish	simplified form of &#x68DF;
U+680C.0	fUTF8	栌
U+680C.0	fGB	0-6853
U+680C.0	fR/S	 75.5
U+680C.0	fTotalStrokes	 9
U+680C.0	fCangjie	DYS
U+680C.0	fHakka	 lu2
U+680C.0	fCantonese	lou4; leoi4
U+680C.0	fMandarin	lu2
U+680C.0	fEnglish	simplified form of &#x6AE8;
U+680D.0	fUTF8	栍
U+680D.0	fGB	1-7D52
U+680D.0	fR/S	 75.5
U+680D.0	fTotalStrokes	 9
U+680D.0	fCangjie	DHQM
U+680D.0	fFourCorner	4591
U+680D.0	fCantonese	sang1
U+680D.0	fEnglish	 a lot; a lable; a slip of bamboo engraved with signs
U+680E.0	fUTF8	栎
U+680E.0	fGB	0-685D
U+680E.0	fR/S	 75.5
U+680E.0	fTotalStrokes	 9
U+680E.0	fCangjie	DHVD
U+680E.0	fHakka	 lit8
U+680E.0	fCantonese	lik1; joek6
U+680E.0	fMandarin	li4; lao2; yue4; luo4; yao4; shuo4
U+680E.0	fEnglish	simplified form of &#x6ADF;
U+680F.0	fUTF8	栏
U+680F.0	fGB	0-4038
U+680F.0	fR/S	 75.5
U+680F.0	fTotalStrokes	 9
U+680F.0	fCangjie	DTMM
U+680F.0	fHakka	 lan2
U+680F.0	fCantonese	laan4; laan1 (also)
U+680F.0	fMandarin	lan2; lian4
U+680F.0	fEnglish	simplified form of &#x6B04;
U+6810.0	fUTF8	栐
U+6810.0	fCNS11643	3-652A
U+6810.0	fGB	E-2D67
U+6810.0	fR/S	 75.5
U+6810.0	fTotalStrokes	 9
U+6810.0	fCangjie	DINE
U+6810.0	fFourCorner	4393
U+6810.0	fCantonese	wing5
U+6810.0	fMandarin	yong3
U+6811.0	fUTF8	树
U+6811.0	fCNS11643	F-2B48
U+6811.0	fGB	0-4A77
U+6811.0	fR/S	 75.5
U+6811.0	fTotalStrokes	 9
U+6811.0	fCangjie	DEDI
U+6811.0	fHakka	 shu5
U+6811.0	fCantonese	syu6
U+6811.0	fMandarin	shu4
U+6811.0	fEnglish	simplified form of &#x6A39;
U+6812.0	fUTF8	栒
U+6812.0	fCNS11643	2-2F6E
U+6812.0	fBig5	D1EC
U+6812.0	fGB	3-413A
U+6812.0	fR/S	 75.6
U+6812.0	fTotalStrokes	 10
U+6812.0	fCangjie	DPA
U+6812.0	fFourCorner	47920
U+6812.0	fHakka	  sun3
U+6812.0	fCantonese	ceon4; seon2; seon1
U+6812.0	fMandarin	xun2; sun1; sun3
U+6812.0	fEnglish	cross bar
U+6813.0	fUTF8	栓
U+6813.0	fCNS11643	1-562E
U+6813.0	fBig5	AEEA
U+6813.0	fGB	0-4B28
U+6813.0	fR/S	 75.6
U+6813.0	fTotalStrokes	 10
U+6813.0	fCangjie	DOMG
U+6813.0	fFourCorner	48914
U+6813.0	fHakka	  cien2;  zien1; cion2; cen2
U+6813.0	fCantonese	saan1
U+6813.0	fMandarin	shuan1; shan1; shuan4
U+6813.0	fEnglish	[1] wooden peg; wooden pin [2] bolt; plug [3] stopper; cork
U+6814.0	fUTF8	栔
U+6814.0	fCNS11643	2-2F6F
U+6814.0	fBig5	D1ED
U+6814.0	fGB	5-4360
U+6814.0	fR/S	 75.6
U+6814.0	fTotalStrokes	 10
U+6814.0	fCangjie	QHD
U+6814.0	fFourCorner	57904
U+6814.0	fHakka	 k'ie5; ke5; giet7
U+6814.0	fCantonese	kai3; kit3
U+6814.0	fMandarin	qi4; qie4
U+6814.0	fEnglish	carve; engrave
U+6815.0	fUTF8	栕
U+6815.0	fCNS11643	3-315D
U+6815.0	fGB	5-4364
U+6815.0	fR/S	 75.6
U+6815.0	fTotalStrokes	 10
U+6815.0	fCangjie	DSLL
U+6815.0	fFourCorner	41910
U+6815.0	fCantonese	san4
U+6815.0	fMandarin	zhen1
U+6816.0	fUTF8	栖
U+6816.0	fCNS11643	2-2F63
U+6816.0	fBig5	D1E1
U+6816.0	fGB	0-465C
U+6816.0	fR/S	 75.6
U+6816.0	fTotalStrokes	 10
U+6816.0	fCangjie	DMCW
U+6816.0	fFourCorner	41960
U+6816.0	fHakka	 zi1; ci1; zi5
U+6816.0	fCantonese	cai1; sai1
U+6816.0	fMandarin	qi1; xi1
U+6816.0	fEnglish	simplified form of &#x68F2;
U+6817.0	fUTF8	栗
U+6817.0	fCNS11643	1-5623
U+6817.0	fBig5	AEDF
U+6817.0	fGB	0-4075
U+6817.0	fR/S	 75.6
U+6817.0	fTotalStrokes	 10
U+6817.0	fCangjie	MWD
U+6817.0	fFourCorner	10904
U+6817.0	fHakka	  lit8; lak8
U+6817.0	fCantonese	leot6; leot2 (also)
U+6817.0	fMandarin	li4; lie4
U+6817.0	fEnglish	[1] chestnut tree [2] strong and tough; firm; durable [3] respectful; fearful; awe-inspiring [4] tremble [5] dignified; majestic [6] a Chinese family name
U+6818.0	fUTF8	栘
U+6818.0	fCNS11643	1-562F
U+6818.0	fBig5	AEEB
U+6818.0	fGB	3-413C
U+6818.0	fR/S	 75.6
U+6818.0	fTotalStrokes	 10
U+6818.0	fCangjie	DNIN
U+6818.0	fFourCorner	47927
U+6818.0	fHakka	 ji2; ri2; i2
U+6818.0	fCantonese	ji4
U+6818.0	fMandarin	yi2; yi3; chi2; xi1
U+6818.0	fEnglish	fruit tree
U+6819.0	fUTF8	栙
U+6819.0	fCNS11643	3-3157
U+6819.0	fGB	3-413B
U+6819.0	fR/S	 75.6
U+6819.0	fTotalStrokes	 10
U+6819.0	fCangjie	DHEQ
U+6819.0	fFourCorner	47950
U+6819.0	fCantonese	hong4; gong3
U+6819.0	fMandarin	xiang2
U+681A.0	fUTF8	栚
U+681A.0	fCNS11643	2-2F5C
U+681A.0	fBig5	D1DA
U+681A.0	fGB	3-4141
U+681A.0	fR/S	 75.6
U+681A.0	fTotalStrokes	 10
U+681A.0	fCangjie	DTK
U+681A.0	fFourCorner	48935
U+681A.0	fHakka	 zhim3; zihm3
U+681A.0	fCantonese	zam6; zam3
U+681A.0	fMandarin	zhen4
U+681A.0	fEnglish	a plant the leaves of which are used for fixing colours
U+681B.0	fUTF8	栛
U+681B.0	fCNS11643	3-3153
U+681B.0	fGB	3-4144
U+681B.0	fR/S	  75.6
U+681B.0	fTotalStrokes	 10
U+681B.0	fCangjie	DKSS
U+681B.0	fFourCorner	44920
U+681B.0	fHakka	 lai5;  li5; lai1; li6
U+681B.0	fCantonese	lai6
U+681B.0	fMandarin	li4
U+681B.0	fEnglish	the lichee; a fruit which grows in South China
U+681C.0	fUTF8	栜
U+681C.0	fCNS11643	2-2F65
U+681C.0	fBig5	D1E3
U+681C.0	fGB	3-4128
U+681C.0	fR/S	 75.6
U+681C.0	fTotalStrokes	 10
U+681C.0	fCangjie	DDB
U+681C.0	fFourCorner	45992
U+681C.0	fCantonese	saak3
U+681C.0	fMandarin	su4; chi4; she4; ci4; xi2
U+681D.0	fUTF8	栝
U+681D.0	fCNS11643	2-2F6D
U+681D.0	fBig5	D1EB
U+681D.0	fGB	0-6869
U+681D.0	fR/S	 75.6
U+681D.0	fTotalStrokes	 10
U+681D.0	fCangjie	DHJR
U+681D.0	fFourCorner	42964
U+681D.0	fHakka	 kwat7; tiam5; tiam3; gat7
U+681D.0	fCantonese	gwaa1; kut3; kwut3 (also)
U+681D.0	fMandarin	gua1; kuo4; tian4; kuai4; tian3
U+681D.0	fEnglish	[1] same as $ , a Chinese junior [2] pointed end of an arrow
U+681E.0	fUTF8	栞
U+681E.0	fCNS11643	3-315B
U+681E.0	fGB	E-2D68
U+681E.0	fR/S	 75.6
U+681E.0	fTotalStrokes	 10
U+681E.0	fCangjie	MJD
U+681E.0	fFourCorner	11900
U+681E.0	fHakka	 kan1; kan3
U+681E.0	fCantonese	hon1; hon2 (also); jing4
U+681E.0	fMandarin	kan1
U+681E.0	fEnglish	 publication; periodical publish
U+681F.0	fUTF8	栟
U+681F.0	fCNS11643	2-2F5B
U+681F.0	fBig5	D1D9
U+681F.0	fGB	3-4140
U+681F.0	fR/S	 75.6
U+681F.0	fTotalStrokes	 10
U+681F.0	fCangjie	DTT
U+681F.0	fFourCorner	48941
U+681F.0	fHakka	  bin1
U+681F.0	fCantonese	bing1; ban1
U+681F.0	fMandarin	bing1; ben1
U+681F.0	fEnglish	hemp palm
U+6820.0	fUTF8	栠
U+6820.0	fCNS11643	2-2F76
U+6820.0	fBig5	D1F4
U+6820.0	fGB	5-436C
U+6820.0	fR/S	 75.6
U+6820.0	fTotalStrokes	 10
U+6820.0	fCangjie	OGD
U+6820.0	fFourCorner	22904
U+6820.0	fCantonese	jam6; jam5
U+6820.0	fMandarin	ren3
U+6821.0	fUTF8	校
U+6821.0	fCNS11643	1-5577
U+6821.0	fBig5	AED5
U+6821.0	fGB	0-5023
U+6821.0	fR/S	 75.6
U+6821.0	fTotalStrokes	 10
U+6821.0	fCangjie	DYCK
U+6821.0	fFourCorner	40948
U+6821.0	fHakka	 gau5; gau3
U+6821.0	fCantonese	haau6; gaau3
U+6821.0	fMandarin	xiao4; jiao3; xiao2; yao2
U+6821.0	fEnglish	[1] school [2] military field officer
U+6821.1	fUTF8	校
U+6821.1	fCNS11643	1-5577
U+6821.1	fBig5	AED5
U+6821.1	fGB	0-5023
U+6821.1	fR/S	 75.6
U+6821.1	fTotalStrokes	 10
U+6821.1	fCangjie	DYCK
U+6821.1	fFourCorner	40948
U+6821.1	fHakka	 gau3
U+6821.1	fMandarin	jiao4
U+6821.1	fEnglish	[1] compare [2] proofread; proofs [3] revise (books, etc.); collate
U+6822.0	fUTF8	栢
U+6822.0	fCNS11643	3-315E
U+6822.0	fGB	E-2D69
U+6822.0	fR/S	  75.6
U+6822.0	fTotalStrokes	 10
U+6822.0	fCangjie	DMA
U+6822.0	fFourCorner	41960
U+6822.0	fHakka	  bak7; pak7
U+6822.0	fCantonese	paak3; baak3
U+6822.0	fMandarin	bo2; bai3; bo4
U+6822.0	fEnglish	cypress, cedar
U+6823.0	fUTF8	栣
U+6823.0	fCNS11643	3-3164
U+6823.0	fGB	E-2D6A
U+6823.0	fR/S	  75.6
U+6823.0	fTotalStrokes	 10
U+6823.0	fCangjie	DOHG
U+6823.0	fFourCorner	41910
U+6823.0	fCantonese	jam6; jam5
U+6823.0	fMandarin	ren3
U+6823.0	fEnglish	same as &#x6820;
U+6824.0	fUTF8	栤
U+6824.0	fCNS11643	3-3150
U+6824.0	fGB	E-2D6B
U+6824.0	fR/S	  75.6
U+6824.0	fTotalStrokes	 10
U+6824.0	fCangjie	DIME
U+6824.0	fFourCorner	42930
U+6824.0	fCantonese	bing6
U+6824.0	fMandarin	bing4
U+6825.0	fUTF8	栥
U+6825.0	fCNS11643	2-2F75
U+6825.0	fBig5	D1F3
U+6825.0	fGB	E-2D6C
U+6825.0	fR/S	 75.6
U+6825.0	fTotalStrokes	 10
U+6825.0	fCangjie	IOD
U+6825.0	fFourCorner	37904
U+6825.0	fCantonese	zi1
U+6825.0	fMandarin	zi1
U+6826.0	fUTF8	栦
U+6826.0	fCNS11643	2-2F70
U+6826.0	fBig5	D1EE
U+6826.0	fGB	E-2D6D
U+6826.0	fR/S	 75.6
U+6826.0	fTotalStrokes	 10
U+6826.0	fCangjie	DILL
U+6826.0	fFourCorner	42900
U+6826.0	fCantonese	cau4
U+6826.0	fMandarin	chou2
U+6827.0	fUTF8	栧
U+6827.0	fCNS11643	3-315F
U+6827.0	fGB	5-4369
U+6827.0	fR/S	  75.6
U+6827.0	fTotalStrokes	 10
U+6827.0	fCangjie	DLWP
U+6827.0	fFourCorner	45900
U+6827.0	fHakka	 siet7; ri3; ji3; ji5; i5
U+6827.0	fCantonese	jai6; ji6; sit3
U+6827.0	fMandarin	yi4; xie4
U+6827.0	fEnglish	same as &#x67BB;
U+6828.0	fUTF8	栨
U+6828.0	fCNS11643	2-2F71
U+6828.0	fBig5	D1EF
U+6828.0	fGB	5-4371
U+6828.0	fR/S	 75.6
U+6828.0	fTotalStrokes	 10
U+6828.0	fCangjie	DIMO
U+6828.0	fFourCorner	47982
U+6828.0	fCantonese	ci3
U+6828.0	fMandarin	jie2; ci4
U+6829.0	fUTF8	栩
U+6829.0	fCNS11643	1-5621
U+6829.0	fBig5	AEDD
U+6829.0	fGB	0-6872
U+6829.0	fR/S	 75.6
U+6829.0	fTotalStrokes	 10
U+6829.0	fCangjie	DSMM
U+6829.0	fFourCorner	47920
U+6829.0	fHakka	 hi3
U+6829.0	fCantonese	heoi2
U+6829.0	fMandarin	xu3; yu3
U+6829.0	fEnglish	[1] a species of oak [2] glad; pleased
U+682A.0	fUTF8	株
U+682A.0	fCNS11643	1-562C
U+682A.0	fBig5	AEE8
U+682A.0	fGB	0-566A
U+682A.0	fR/S	 75.6
U+682A.0	fTotalStrokes	 10
U+682A.0	fCangjie	DHJD
U+682A.0	fFourCorner	45990
U+682A.0	fHakka	 zu1;  zhu1
U+682A.0	fCantonese	zyu1
U+682A.0	fMandarin	zhu1
U+682A.0	fEnglish	[1] numerical classifier for trees [2] roots that grow above the ground
U+682B.0	fUTF8	栫
U+682B.0	fCNS11643	2-2F67
U+682B.0	fBig5	D1E5
U+682B.0	fGB	3-412B
U+682B.0	fR/S	 75.6
U+682B.0	fTotalStrokes	 10
U+682B.0	fCangjie	DKLD
U+682B.0	fFourCorner	44947
U+682B.0	fHakka	  zien5;  zun3
U+682B.0	fCantonese	zin3
U+682B.0	fMandarin	jian4; huo4; zun4
U+682B.0	fEnglish	fence
U+682C.0	fUTF8	栬
U+682C.0	fCNS11643	3-3156
U+682C.0	fGB	3-413D
U+682C.0	fR/S	 75.6
U+682C.0	fTotalStrokes	 10
U+682C.0	fCangjie	DNAU
U+682C.0	fFourCorner	47910
U+682C.0	fCantonese	zeoi3
U+682C.0	fMandarin	zui4
U+682D.0	fUTF8	栭
U+682D.0	fCNS11643	2-2F68
U+682D.0	fBig5	D1E6
U+682D.0	fGB	3-412A
U+682D.0	fR/S	 75.6
U+682D.0	fTotalStrokes	 10
U+682D.0	fCangjie	DMBL
U+682D.0	fFourCorner	41927
U+682D.0	fHakka	 ji2; ri2; i2
U+682D.0	fCantonese	ji4
U+682D.0	fMandarin	er2
U+682D.0	fEnglish	king-post
U+682E.0	fUTF8	栮
U+682E.0	fCNS11643	2-2F72
U+682E.0	fBig5	D1F0
U+682E.0	fGB	5-4363
U+682E.0	fR/S	 75.6
U+682E.0	fTotalStrokes	 10
U+682E.0	fCangjie	DSJ
U+682E.0	fFourCorner	41940
U+682E.0	fCantonese	ji5
U+682E.0	fMandarin	er3
U+682F.0	fUTF8	栯
U+682F.0	fCNS11643	2-2F69
U+682F.0	fBig5	D1E7
U+682F.0	fGB	3-4129
U+682F.0	fR/S	 75.6
U+682F.0	fTotalStrokes	 10
U+682F.0	fCangjie	DKB
U+682F.0	fFourCorner	44927
U+682F.0	fCantonese	jau5; juk1
U+682F.0	fMandarin	you3; yu4; you4
U+6830.0	fUTF8	栰
U+6830.0	fCNS11643	3-3163
U+6830.0	fGB	E-2D6E
U+6830.0	fR/S	  75.6
U+6830.0	fTotalStrokes	 10
U+6830.0	fCangjie	DOI
U+6830.0	fFourCorner	43950
U+6830.0	fCantonese	fat6
U+6830.0	fMandarin	fa2
U+6830.0	fEnglish	a bamboo raft
U+6831.0	fUTF8	栱
U+6831.0	fCNS11643	2-2F64
U+6831.0	fBig5	D1E2
U+6831.0	fGB	3-4127
U+6831.0	fR/S	 75.6
U+6831.0	fTotalStrokes	 10
U+6831.0	fCangjie	DTC
U+6831.0	fFourCorner	44981
U+6831.0	fHakka	  giung3
U+6831.0	fCantonese	gung2
U+6831.0	fMandarin	gong3
U+6831.0	fEnglish	 large peg; big stake; post; prop
U+6832.0	fUTF8	栲
U+6832.0	fCNS11643	2-2F5E
U+6832.0	fBig5	D1DC
U+6832.0	fGB	0-6860
U+6832.0	fR/S	 75.6
U+6832.0	fTotalStrokes	 10
U+6832.0	fCangjie	DJKS
U+6832.0	fFourCorner	44927
U+6832.0	fHakka	 kau3; kau5
U+6832.0	fCantonese	haau2
U+6832.0	fMandarin	kao3; kao1
U+6832.0	fEnglish	Castanopsis hystrix
U+6833.0	fUTF8	栳
U+6833.0	fCNS11643	2-2F5F
U+6833.0	fBig5	D1DD
U+6833.0	fGB	0-6861
U+6833.0	fR/S	 75.6
U+6833.0	fTotalStrokes	 10
U+6833.0	fCangjie	DJKP
U+6833.0	fFourCorner	44911
U+6833.0	fHakka	 lau3
U+6833.0	fCantonese	lou5
U+6833.0	fMandarin	lao3
U+6833.0	fEnglish	a basket
U+6834.0	fUTF8	栴
U+6834.0	fCNS11643	2-2F6C
U+6834.0	fBig5	D1EA
U+6834.0	fGB	5-436B
U+6834.0	fR/S	 75.6
U+6834.0	fTotalStrokes	 10
U+6834.0	fCangjie	DOBY
U+6834.0	fFourCorner	48947
U+6834.0	fHakka	 zhan1; zan1; zen1; zhen1
U+6834.0	fCantonese	zin1
U+6834.0	fMandarin	zhan1
U+6834.0	fEnglish	sandalwood
U+6835.0	fUTF8	栵
U+6835.0	fCNS11643	2-2F66
U+6835.0	fBig5	D1E4
U+6835.0	fGB	3-412D
U+6835.0	fR/S	 75.6
U+6835.0	fTotalStrokes	 10
U+6835.0	fCangjie	DMNN
U+6835.0	fFourCorner	42900
U+6835.0	fHakka	  liet8
U+6835.0	fCantonese	lai6; lit6
U+6835.0	fMandarin	li4; lie4
U+6835.0	fEnglish	hedge
U+6836.0	fUTF8	栶
U+6836.0	fCNS11643	3-3154
U+6836.0	fGB	E-2D6F
U+6836.0	fR/S	 75.6
U+6836.0	fTotalStrokes	 10
U+6836.0	fCangjie	DWK
U+6836.0	fFourCorner	46900
U+6836.0	fCantonese	jan1
U+6836.0	fMandarin	yin1
U+6837.0	fUTF8	样
U+6837.0	fCNS11643	3-314F
U+6837.0	fGB	0-5179
U+6837.0	fR/S	  75.6
U+6837.0	fTotalStrokes	 10
U+6837.0	fCangjie	DTQ
U+6837.0	fFourCorner	48950
U+6837.0	fHakka	 jong5
U+6837.0	fCantonese	joeng4; joeng6; joeng2 (also)
U+6837.0	fMandarin	yang2; yang4; xiang4
U+6837.0	fEnglish	simplified form of &#x6A23;
U+6838.0	fUTF8	核
U+6838.0	fCNS11643	1-5578
U+6838.0	fBig5	AED6
U+6838.0	fGB	0-3A4B
U+6838.0	fR/S	 75.6
U+6838.0	fTotalStrokes	 10
U+6838.0	fCangjie	DYVO
U+6838.0	fFourCorner	40982
U+6838.0	fHakka	  fut8;  het8; hak8; het7; ep7
U+6838.0	fCantonese	hat6; wat6 (also); wat2 (also); goi1
U+6838.0	fMandarin	he2; hu2; gai1; jie1; kai4
U+6838.0	fEnglish	[1] kernel; core; nut; walnut; fruit stone; pit [2] investigate; examine; verify; study; check [3] nucleus; nuclear [4] hard lump
U+6839.0	fUTF8	根
U+6839.0	fCNS11643	1-557C
U+6839.0	fBig5	AEDA
U+6839.0	fGB	0-3879
U+6839.0	fR/S	 75.6
U+6839.0	fTotalStrokes	 10
U+6839.0	fCangjie	DAV
U+6839.0	fFourCorner	47932
U+6839.0	fHakka	  gin1;  gen1; gien1; {gen1}
U+6839.0	fCantonese	gan1
U+6839.0	fMandarin	gen1
U+6839.0	fEnglish	[1] root of a plant [2] base; foundation [3] root of a number (mathematics) [4] beginning cause; source of something [5] radical (chemistry) a piece of (string, rope, etc.); a stick, spear or something of slender shape
U+683A.0	fUTF8	栺
U+683A.0	fCNS11643	2-2F74
U+683A.0	fBig5	D1F2
U+683A.0	fGB	3-4139
U+683A.0	fR/S	 75.6
U+683A.0	fTotalStrokes	 10
U+683A.0	fCangjie	DPA
U+683A.0	fFourCorner	41961
U+683A.0	fCantonese	ngai6; ai6 (also); zi1; zi2
U+683A.0	fMandarin	zhi3; zhi4
U+683B.0	fUTF8	栻
U+683B.0	fCNS11643	2-2F60
U+683B.0	fBig5	D1DE
U+683B.0	fGB	3-4126
U+683B.0	fR/S	 75.6
U+683B.0	fTotalStrokes	 10
U+683B.0	fCangjie	DIPM
U+683B.0	fFourCorner	43940
U+683B.0	fHakka	 shit7; chit7; ts'iit7; siit7
U+683B.0	fCantonese	sik1; cik1
U+683B.0	fMandarin	chi4; shi4
U+683B.0	fEnglish	[1] name of a kind of tree [2] a wooden object used in divination
U+683C.0	fUTF8	格
U+683C.0	fCNS11643	1-562A
U+683C.0	fBig5	AEE6
U+683C.0	fGB	0-3871
U+683C.0	fR/S	 75.6
U+683C.0	fTotalStrokes	 10
U+683C.0	fCangjie	DHER
U+683C.0	fFourCorner	47964
U+683C.0	fHakka	  gak7;  giet7; get7
U+683C.0	fCantonese	gaak3; gaak2 (also)
U+683C.0	fMandarin	ge2; ge1; ge4; luo4
U+683C.0	fEnglish	[1] correct; adjust; regulate [2] reach; come; go to [3] influence [4] resist; attack; fight [5] obstruct; block [6] study thoroughly; search to the very source; investigate [7] pattern; standard; form; style; rule [8] possessive [9] [10] this; that [11] box
U+683C.1	fUTF8	格
U+683C.1	fCNS11643	1-562A
U+683C.1	fBig5	AEE6
U+683C.1	fGB	0-3871
U+683C.1	fR/S	 75.6
U+683C.1	fTotalStrokes	 10
U+683C.1	fCangjie	DHER
U+683C.1	fFourCorner	47964
U+683C.1	fMandarin	ge5
U+683D.0	fUTF8	栽
U+683D.0	fCNS11643	1-5626
U+683D.0	fBig5	AEE2
U+683D.0	fGB	0-5454
U+683D.0	fR/S	 75.6
U+683D.0	fTotalStrokes	 10
U+683D.0	fCangjie	JID
U+683D.0	fFourCorner	43950
U+683D.0	fHakka	  zai1; cai2
U+683D.0	fCantonese	zoi1
U+683D.0	fMandarin	zai1; zai4; zai3
U+683D.0	fEnglish	[1] cultivate; plant [2] care; assist [3] fall; fail [4] young trees, saplings, cuttings for planting
U+683E.0	fUTF8	栾
U+683E.0	fCNS11643	3-315A
U+683E.0	fGB	0-686F
U+683E.0	fR/S	  75.6
U+683E.0	fTotalStrokes	 10
U+683E.0	fCangjie	YCD
U+683E.0	fFourCorner	00900
U+683E.0	fHakka	 lan2
U+683E.0	fCantonese	lyun4
U+683E.0	fMandarin	luan2
U+683E.0	fEnglish	[1] non-classical form of &#x6B12; [2] simplified form of &#x6B12;
U+683F.0	fUTF8	栿
U+683F.0	fCNS11643	4-2D55
U+683F.0	fGB	3-4133
U+683F.0	fR/S	  75.6
U+683F.0	fTotalStrokes	 10
U+683F.0	fCangjie	DOIK
U+683F.0	fFourCorner	43930
U+683F.0	fHakka	 fuk8
U+683F.0	fCantonese	fuk6
U+683F.0	fMandarin	fu2
U+6840.0	fUTF8	桀
U+6840.0	fCNS11643	1-5629
U+6840.0	fBig5	AEE5
U+6840.0	fGB	0-686E
U+6840.0	fR/S	 75.6
U+6840.0	fTotalStrokes	 10
U+6840.0	fCangjie	NQD
U+6840.0	fFourCorner	25904
U+6840.0	fHakka	 kiet8; ket8
U+6840.0	fCantonese	git6
U+6840.0	fMandarin	jie2
U+6840.0	fEnglish	 chicken roost ancient emperor
U+6841.0	fUTF8	桁
U+6841.0	fCNS11643	1-5630
U+6841.0	fBig5	AEEC
U+6841.0	fGB	0-686C
U+6841.0	fR/S	 75.6
U+6841.0	fTotalStrokes	 10
U+6841.0	fCangjie	DHON
U+6841.0	fFourCorner	41921
U+6841.0	fHakka	 hang2; hang3; en5
U+6841.0	fCantonese	hang4; haang4; hong4; hong6; haang2 (also)
U+6841.0	fMandarin	heng2; hang2; hang4
U+6841.0	fEnglish	 cross-beams of roof; purlins of a roof
U+6841.1	fUTF8	桁
U+6841.1	fCNS11643	1-5630
U+6841.1	fBig5	AEEC
U+6841.1	fGB	0-686C
U+6841.1	fR/S	 75.6
U+6841.1	fTotalStrokes	 10
U+6841.1	fCangjie	DHON
U+6841.1	fFourCorner	41921
U+6841.1	fMandarin	hang2hang4}
U+6841.1	fEnglish	 big wooden collar (for punishing a criminal)
U+6841.2	fUTF8	桁
U+6841.2	fCNS11643	1-5630
U+6841.2	fBig5	AEEC
U+6841.2	fGB	0-686C
U+6841.2	fR/S	 75.6
U+6841.2	fTotalStrokes	 10
U+6841.2	fCangjie	DHON
U+6841.2	fFourCorner	41921
U+6841.2	fMandarin	hang4heng2}hang2}
U+6841.2	fEnglish	 rack for hanging clothes
U+6842.0	fUTF8	桂
U+6842.0	fCNS11643	1-557D
U+6842.0	fBig5	AEDB
U+6842.0	fGB	0-3970
U+6842.0	fR/S	 75.6
U+6842.0	fTotalStrokes	 10
U+6842.0	fCangjie	DGG
U+6842.0	fFourCorner	44914
U+6842.0	fHakka	 gui5; gwi5; kwui5
U+6842.0	fCantonese	gwai3
U+6842.0	fMandarin	gui4
U+6842.0	fEnglish	[1] cassia or cinnamon [2] a short name of Kwangsi province [3] a Chinese family name
U+6843.0	fUTF8	桃
U+6843.0	fCNS11643	1-562B
U+6843.0	fBig5	AEE7
U+6843.0	fGB	0-4C52
U+6843.0	fR/S	 75.6
U+6843.0	fTotalStrokes	 10
U+6843.0	fCangjie	DLMO
U+6843.0	fFourCorner	42913
U+6843.0	fHakka	  tau2;  to2; to3
U+6843.0	fCantonese	tou4; tou2 (also)
U+6843.0	fMandarin	tao2; shu4; tao1; tiao1; zhao4
U+6843.0	fEnglish	[1] peach [2] a Chinese family name
U+6844.0	fUTF8	桄
U+6844.0	fCNS11643	2-2F6B
U+6844.0	fBig5	D1E9
U+6844.0	fGB	0-6866
U+6844.0	fR/S	 75.6
U+6844.0	fTotalStrokes	 10
U+6844.0	fCangjie	DFMU
U+6844.0	fFourCorner	49911
U+6844.0	fHakka	 gong1; gang1; gwang1; kwong1
U+6844.0	fCantonese	gwong3; gwong1; gong3 (also); waang6 (also); gong1 (also); gwong2 (also); gong2 (also); kwong3
U+6844.0	fMandarin	guang4; guang1
U+6844.0	fEnglish	[1] crossbeam [2] reel
U+6845.0	fUTF8	桅
U+6845.0	fCNS11643	1-562D
U+6845.0	fBig5	AEE9
U+6845.0	fGB	0-4E26
U+6845.0	fR/S	 75.6
U+6845.0	fTotalStrokes	 10
U+6845.0	fCangjie	DNMU
U+6845.0	fFourCorner	47912
U+6845.0	fHakka	 ngui2; mui2; ngui3; wui2; mi2; nui2
U+6845.0	fCantonese	wai4; ngai4 (also); ai4 (also); ngai2 (also); ai2 (also)
U+6845.0	fMandarin	wei2
U+6845.0	fEnglish	 mast of a ship
U+6846.0	fUTF8	框
U+6846.0	fCNS11643	1-557A
U+6846.0	fBig5	AED8
U+6846.0	fGB	0-3F72
U+6846.0	fR/S	 75.6
U+6846.0	fTotalStrokes	 10
U+6846.0	fCangjie	DSMG
U+6846.0	fFourCorner	41911
U+6846.0	fHakka	 kiong1
U+6846.0	fCantonese	kwaang1; hong1 (also)
U+6846.0	fMandarin	kuang1
U+6846.0	fEnglish	[1] frame; framework [2] door frame [3] skeleton (of a lantern, etc.)
U+6846.1	fUTF8	框
U+6846.1	fCNS11643	1-557A
U+6846.1	fBig5	AED8
U+6846.1	fGB	0-3F72
U+6846.1	fR/S	 75.6
U+6846.1	fTotalStrokes	 10
U+6846.1	fCangjie	DSMG
U+6846.1	fFourCorner	41911
U+6846.1	fMandarin	kuang4
U+6847.0	fUTF8	桇
U+6847.0	fCNS11643	3-3159
U+6847.0	fGB	E-2D70
U+6847.0	fR/S	 75.6
U+6847.0	fTotalStrokes	 10
U+6847.0	fCangjie	VRD
U+6847.0	fFourCorner	46900
U+6847.0	fCantonese	jyu4
U+6847.0	fMandarin	ru2
U+6848.0	fUTF8	案
U+6848.0	fCNS11643	1-5579
U+6848.0	fBig5	AED7
U+6848.0	fGB	0-3038
U+6848.0	fR/S	 75.6
U+6848.0	fTotalStrokes	 10
U+6848.0	fCangjie	JVD
U+6848.0	fFourCorner	30904
U+6848.0	fHakka	  on5
U+6848.0	fCantonese	on3; ngon3 (also); on2 (also); ngon2 (also)
U+6848.0	fMandarin	an4
U+6848.0	fEnglish	[1] narrow long table; bench; judge's desk [2] legal case; legal records; legal offense [3] according to; on the strength of; following this precedent [4] press
U+6849.0	fUTF8	桉
U+6849.0	fCNS11643	2-2F5D
U+6849.0	fBig5	D1DB
U+6849.0	fGB	0-6871
U+6849.0	fR/S	 75.6
U+6849.0	fTotalStrokes	 10
U+6849.0	fCangjie	DJV
U+6849.0	fFourCorner	43944
U+6849.0	fHakka	 on5
U+6849.0	fCantonese	on1; on3; ngon1 (also); ngon3 (also)
U+6849.0	fMandarin	an1; an4
U+684A.0	fUTF8	桊
U+684A.0	fCNS11643	4-2D4F
U+684A.0	fGB	0-6870
U+684A.0	fR/S	 75.6
U+684A.0	fTotalStrokes	 10
U+684A.0	fCangjie	FQD
U+684A.0	fFourCorner	90900
U+684A.0	fHakka	 kien5;  gien5
U+684A.0	fCantonese	gyun3
U+684A.0	fMandarin	juan4; quan1; quan2
U+684A.0	fEnglish	a ring through an animals' nose by which it is led
U+684B.0	fUTF8	桋
U+684B.0	fCNS11643	2-2F61
U+684B.0	fBig5	D1DF
U+684B.0	fGB	3-412F
U+684B.0	fR/S	 75.6
U+684B.0	fTotalStrokes	 10
U+684B.0	fCangjie	DKN
U+684B.0	fFourCorner	45932
U+684B.0	fHakka	 ji2; ri2; i2
U+684B.0	fCantonese	ji4
U+684B.0	fMandarin	yi2; ti2
U+684B.0	fEnglish	tree
U+684C.0	fUTF8	桌
U+684C.0	fCNS11643	1-5624
U+684C.0	fBig5	AEE0
U+684C.0	fGB	0-5740
U+684C.0	fR/S	 75.6
U+684C.0	fTotalStrokes	 10
U+684C.0	fCangjie	YAD
U+684C.0	fFourCorner	21904
U+684C.0	fHakka	  zok7
U+684C.0	fCantonese	coek3; zoek3 (also)
U+684C.0	fMandarin	zhuo1
U+684C.0	fEnglish	[1] table; desk; stand [2] dishes for guests around the table, usually consisting of 20 courses [3] a tableful of guests (10 to 12 persons at a round table)
U+684D.0	fUTF8	桍
U+684D.0	fCNS11643	2-2F73
U+684D.0	fBig5	D1F1
U+684D.0	fGB	3-412C
U+684D.0	fR/S	 75.6
U+684D.0	fTotalStrokes	 10
U+684D.0	fCangjie	DKMS
U+684D.0	fFourCorner	44927
U+684D.0	fHakka	 kwa3; k'ua5; ku1; ka3
U+684D.0	fCantonese	fu1
U+684D.0	fMandarin	ku1
U+684E.0	fUTF8	桎
U+684E.0	fCNS11643	2-2F6A
U+684E.0	fBig5	D1E8
U+684E.0	fGB	0-6864
U+684E.0	fR/S	 75.6
U+684E.0	fTotalStrokes	 10
U+684E.0	fCangjie	DMIG
U+684E.0	fFourCorner	41914
U+684E.0	fHakka	 zhit7; tsiit7; zit7
U+684E.0	fCantonese	zat6; zat1
U+684E.0	fMandarin	zhi4; zhi2; zhi3
U+684E.0	fEnglish	[1] fetters; shackles; fetter [2] suffocate
U+684F.0	fUTF8	桏
U+684F.0	fCNS11643	2-2F62
U+684F.0	fBig5	D1E0
U+684F.0	fGB	E-2D71
U+684F.0	fR/S	 75.6
U+684F.0	fTotalStrokes	 10
U+684F.0	fCangjie	DMNL
U+684F.0	fFourCorner	47927
U+684F.0	fHakka	 kiung2
U+684F.0	fCantonese	kung4; hung4
U+684F.0	fMandarin	qiong2
U+684F.0	fEnglish	tallow tree, a kind of choir palm, willow
U+6850.0	fUTF8	桐
U+6850.0	fCNS11643	1-5628
U+6850.0	fBig5	AEE4
U+6850.0	fGB	0-4D29
U+6850.0	fR/S	 75.6
U+6850.0	fTotalStrokes	 10
U+6850.0	fCangjie	DBMR
U+6850.0	fFourCorner	47920
U+6850.0	fHakka	  tung2; tung3
U+6850.0	fCantonese	tung4
U+6850.0	fMandarin	tong2; dong4
U+6850.0	fEnglish	[1] a paulownia [2] a Chinese family name
U+6851.0	fUTF8	桑
U+6851.0	fCNS11643	1-5625
U+6851.0	fBig5	AEE1
U+6851.0	fGB	0-4923
U+6851.0	fR/S	 75.6
U+6851.0	fTotalStrokes	 10
U+6851.0	fCangjie	EEED
U+6851.0	fFourCorner	17904
U+6851.0	fHakka	  song1
U+6851.0	fCantonese	song1
U+6851.0	fMandarin	sang1
U+6851.0	fEnglish	[1] mulberry tree [2] a Chinese family name
U+6852.0	fUTF8	桒
U+6852.0	fCNS11643	3-2D2F
U+6852.0	fGB	E-2D72
U+6852.0	fR/S	 75.6
U+6852.0	fTotalStrokes	 10; 9
U+6852.0	fCangjie	JTD
U+6852.0	fFourCorner	40900
U+6852.0	fCantonese	song1
U+6852.0	fMandarin	sang1
U+6853.0	fUTF8	桓
U+6853.0	fCNS11643	1-557B
U+6853.0	fBig5	AED9
U+6853.0	fGB	0-3B38
U+6853.0	fR/S	 75.6
U+6853.0	fTotalStrokes	 10
U+6853.0	fCangjie	DMAM
U+6853.0	fFourCorner	41916
U+6853.0	fHakka	 fan2; hen2
U+6853.0	fCantonese	wun4
U+6853.0	fMandarin	huan2
U+6853.0	fEnglish	[1] a tree with leaves like a willow and white bark [2] a Chinese family name [3] greatly; effectively
U+6854.0	fUTF8	桔
U+6854.0	fCNS11643	1-557E
U+6854.0	fBig5	AEDC
U+6854.0	fGB	0-3D5B
U+6854.0	fR/S	 75.6
U+6854.0	fTotalStrokes	 10
U+6854.0	fCangjie	DGR
U+6854.0	fFourCorner	44961
U+6854.0	fHakka	  git7; giet7; get7
U+6854.0	fCantonese	gat1; git3; kit3; aat1; gwat1
U+6854.0	fMandarin	jie2; xie2
U+6854.0	fEnglish	 well-sweep
U+6854.1	fUTF8	桔
U+6854.1	fCNS11643	1-557E
U+6854.1	fBig5	AEDC
U+6854.1	fGB	0-3D5B
U+6854.1	fR/S	 75.6
U+6854.1	fTotalStrokes	 10
U+6854.1	fCangjie	DGR
U+6854.1	fFourCorner	44961
U+6854.1	fMandarin	ju2
U+6854.1	fEnglish	abbreviated form of @; a mandarin orange or tangerine
U+6855.0	fUTF8	桕
U+6855.0	fCNS11643	3-3162
U+6855.0	fGB	0-686A
U+6855.0	fR/S	  75.6
U+6855.0	fTotalStrokes	 10
U+6855.0	fCangjie	DHX
U+6855.0	fFourCorner	47970
U+6855.0	fHakka	 kiu1; kiu3; gau3; gau1
U+6855.0	fCantonese	kau5; kau3 (also)
U+6855.0	fMandarin	jiu4; gao1
U+6855.0	fEnglish	non-classical form of &#x234CC;
U+6856.0	fUTF8	桖
U+6856.0	fCNS11643	3-3158
U+6856.0	fGB	3-4135
U+6856.0	fR/S	 75.6
U+6856.0	fTotalStrokes	 10
U+6856.0	fCangjie	DHBT
U+6856.0	fFourCorner	47910
U+6856.0	fCantonese	hyut3
U+6856.0	fMandarin	xue4
U+6857.0	fUTF8	桗
U+6857.0	fCNS11643	F-3024
U+6857.0	fGB	E-2D73
U+6857.0	fR/S	 75.6
U+6857.0	fTotalStrokes	 10
U+6857.0	fCangjie	DNSD
U+6857.0	fFourCorner	4799
U+6857.0	fCantonese	do3
U+6857.0	fMandarin	duo4; duo3
U+6858.0	fUTF8	桘
U+6858.0	fCNS11643	3-3161
U+6858.0	fGB	E-2D74
U+6858.0	fR/S	 75.6
U+6858.0	fTotalStrokes	 10
U+6858.0	fCangjie	DHRR
U+6858.0	fFourCorner	43970
U+6858.0	fHakka	 cui2; ch'ui2; ch'ui3; chui2
U+6858.0	fCantonese	ceoi4
U+6858.0	fMandarin	chui2; zhui4; dui1
U+6858.0	fEnglish	same as &#x69CC;
U+6859.0	fUTF8	桙
U+6859.0	fCNS11643	3-3160
U+6859.0	fGB	E-2D75
U+6859.0	fR/S	 75.6
U+6859.0	fTotalStrokes	 10
U+6859.0	fCangjie	DIHQ
U+6859.0	fFourCorner	43950
U+6859.0	fHakka	 ri1; ji1; i2; ji2
U+6859.0	fCantonese	jyu1; jyu4
U+6859.0	fMandarin	yu2; yu4
U+6859.0	fEnglish	same as &#x6745;
U+685A.0	fUTF8	桚
U+685A.0	fCNS11643	3-652D
U+685A.0	fGB	E-2D76
U+685A.0	fR/S	 75.6
U+685A.0	fTotalStrokes	 10
U+685A.0	fCangjie	DVVN
U+685A.0	fFourCorner	4292
U+685A.0	fCantonese	zaat3; cat3
U+685A.0	fMandarin	zan3
U+685A.0	fEnglish	 press; squeeze hard force
U+685B.0	fUTF8	桛
U+685B.0	fGB	E-2D77
U+685B.0	fR/S	 75.6
U+685B.0	fTotalStrokes	 10
U+685B.0	fCangjie	DYMY
U+685B.0	fFourCorner	4193
U+685B.0	fCantonese	lung6
U+685B.0	fEnglish	a real, hank, skein
U+685C.0	fUTF8	桜
U+685C.0	fGB	E-2D78
U+685C.0	fR/S	 75.6
U+685C.0	fTotalStrokes	 10
U+685C.0	fCangjie	DFV
U+685C.0	fHakka	 jin1
U+685C.0	fCantonese	jing1
U+685C.0	fMandarin	ying1
U+685C.0	fEnglish	cherry, cherry blossom
U+685D.0	fUTF8	桝
U+685D.0	fGB	E-2D79
U+685D.0	fR/S	 75.6
U+685D.0	fTotalStrokes	 10
U+685D.0	fCangjie	DNIQ
U+685D.0	fFourCorner	4590
U+685D.0	fCantonese	git6
U+685D.0	fEnglish	variant of &#x67A1;
U+685E.0	fUTF8	桞
U+685E.0	fCNS11643	3-652C
U+685E.0	fGB	E-2D7A
U+685E.0	fR/S	 75.6
U+685E.0	fTotalStrokes	 10
U+685E.0	fCangjie	DNIL
U+685E.0	fFourCorner	4792
U+685E.0	fCantonese	lau5
U+685F.0	fUTF8	桟
U+685F.0	fGB	E-2D7B
U+685F.0	fR/S	 75.6
U+685F.0	fTotalStrokes	 10
U+685F.0	fCangjie	DIQ
U+685F.0	fCantonese	zaan6
U+685F.0	fMandarin	zhan4
U+685F.0	fEnglish	a crosspiece, cleat, frame, door bolt; shelf
U+6860.0	fUTF8	桠
U+6860.0	fGB	0-6862
U+6860.0	fR/S	 75.6
U+6860.0	fTotalStrokes	 10
U+6860.0	fCangjie	DMTC
U+6860.0	fHakka	 a3
U+6860.0	fCantonese	aa1; ngaa1 (also); ngaa4
U+6860.0	fMandarin	ya2; ya1; e3; e4
U+6860.0	fEnglish	simplified form of &#x690F;
U+6861.0	fUTF8	桡
U+6861.0	fGB	0-6863
U+6861.0	fR/S	 75.6
U+6861.0	fTotalStrokes	 10
U+6861.0	fCangjie	XDJPU
U+6861.0	fHakka	 ngiau2; nau2
U+6861.0	fCantonese	naau4; laau4 (also); jiu4; lau4
U+6861.0	fMandarin	nao2; rao2; nao4; nao3; rao3
U+6861.0	fEnglish	simplified form of &#x6A48;
U+6862.0	fUTF8	桢
U+6862.0	fGB	0-6865
U+6862.0	fR/S	 75.6
U+6862.0	fTotalStrokes	 10
U+6862.0	fCangjie	DYBO
U+6862.0	fHakka	 zhin1
U+6862.0	fCantonese	zing1
U+6862.0	fMandarin	zhen1; zheng1
U+6862.0	fEnglish	simplified form of &#x6968;
U+6863.0	fUTF8	档
U+6863.0	fGB	0-3535
U+6863.0	fR/S	 75.6
U+6863.0	fTotalStrokes	 10
U+6863.0	fCangjie	DFSM
U+6863.0	fHakka	 dong5
U+6863.0	fCantonese	dong2; dong3
U+6863.0	fMandarin	dang4; dang1; dang3
U+6863.0	fEnglish	simplified form of &#x6A94;
U+6864.0	fUTF8	桤
U+6864.0	fGB	0-6867
U+6864.0	fR/S	 75.6
U+6864.0	fTotalStrokes	 10
U+6864.0	fCangjie	DUSU
U+6864.0	fHakka	 ki1
U+6864.0	fCantonese	kei1 (also); hei2; hei1 (also)
U+6864.0	fMandarin	qi1; ai2; kai3
U+6864.0	fEnglish	simplified form of &#x69BF;
U+6865.0	fUTF8	桥
U+6865.0	fGB	0-4745
U+6865.0	fR/S	 75.6
U+6865.0	fTotalStrokes	 10
U+6865.0	fCangjie	DHKL
U+6865.0	fHakka	 kiau2
U+6865.0	fCantonese	kiu4; kiu2 (also)
U+6865.0	fMandarin	qiao2; qiao1; jiao4; gao1jiao3
U+6865.0	fEnglish	simplified form of &#x6A4B;
U+6866.0	fUTF8	桦
U+6866.0	fGB	0-686B
U+6866.0	fR/S	 75.6
U+6866.0	fTotalStrokes	 10
U+6866.0	fCangjie	DOPJ
U+6866.0	fHakka	 fa5; fa2
U+6866.0	fCantonese	waa4; waa6
U+6866.0	fMandarin	hua4; hua2; jia2
U+6866.0	fEnglish	simplified form of &#x6A3A;
U+6867.0	fUTF8	桧
U+6867.0	fGB	0-686D
U+6867.0	fR/S	 75.6
U+6867.0	fTotalStrokes	 10
U+6867.0	fCangjie	DOMI
U+6867.0	fHakka	 kwai5
U+6867.0	fCantonese	kui2; kwui2 (also)
U+6867.0	fMandarin	kuai4; gui4; gua1; hui4
U+6867.0	fEnglish	simplified form of &#x6A9C;
U+6868.0	fUTF8	桨
U+6868.0	fGB	0-3D30
U+6868.0	fR/S	 75.6
U+6868.0	fTotalStrokes	 10
U+6868.0	fCangjie	LND
U+6868.0	fHakka	 ziong3
U+6868.0	fCantonese	zoeng2
U+6868.0	fMandarin	jiang3; jiang1
U+6868.0	fEnglish	simplified form of &#x69F3;
U+6869.0	fUTF8	桩
U+6869.0	fGB	0-572E
U+6869.0	fR/S	 75.6
U+6869.0	fTotalStrokes	 10
U+6869.0	fCangjie	DIG
U+6869.0	fHakka	 zhung1
U+6869.0	fCantonese	zong1
U+6869.0	fMandarin	zhuang1; chong1
U+6869.0	fEnglish	simplified form of &#x6A01;
U+686A.0	fUTF8	桪
U+686A.0	fGB	8-7B6C
U+686A.0	fR/S	 75.6
U+686A.0	fTotalStrokes	 10
U+686A.0	fCangjie	XDSMI
U+686A.0	fCantonese	cam4
U+686A.0	fMandarin	xun2; xin2
U+686A.0	fEnglish	mythical giant tree
U+686B.0	fUTF8	桫
U+686B.0	fCNS11643	2-3628
U+686B.0	fBig5	D5C4
U+686B.0	fGB	0-6878
U+686B.0	fR/S	 75.7
U+686B.0	fTotalStrokes	 11
U+686B.0	fCangjie	DEFH
U+686B.0	fFourCorner	49920
U+686B.0	fHakka	  so1
U+686B.0	fCantonese	so1
U+686B.0	fMandarin	suo1
U+686B.0	fEnglish	[1] horse chestnut (Stewartia pseudocamellia) [2] sal (Cyathea spinulosa)
U+686C.0	fUTF8	桬
U+686C.0	fCNS11643	3-3655
U+686C.0	fGB	5-442C
U+686C.0	fR/S	  75.7
U+686C.0	fTotalStrokes	 11
U+686C.0	fCangjie	EHD
U+686C.0	fFourCorner	39900
U+686C.0	fHakka	 sa1
U+686C.0	fCantonese	saa1
U+686C.0	fMandarin	sha1
U+686C.0	fEnglish	pear and other trees
U+686D.0	fUTF8	桭
U+686D.0	fCNS11643	2-3576
U+686D.0	fBig5	D5B4
U+686D.0	fGB	3-414F
U+686D.0	fR/S	 75.7
U+686D.0	fTotalStrokes	 11
U+686D.0	fCangjie	DMMV
U+686D.0	fFourCorner	41932
U+686D.0	fCantonese	san4
U+686D.0	fMandarin	zhen1; zhen4; chen2
U+686E.0	fUTF8	桮
U+686E.0	fCNS11643	2-3577
U+686E.0	fBig5	D5B5
U+686E.0	fGB	E-2D7C
U+686E.0	fR/S	 75.7
U+686E.0	fTotalStrokes	 11
U+686E.0	fCangjie	DMFR
U+686E.0	fFourCorner	41969
U+686E.0	fHakka	 bui1; bi1
U+686E.0	fCantonese	bui1
U+686E.0	fMandarin	bei1
U+686E.0	fEnglish	cup; glass; tumbler
U+686F.0	fUTF8	桯
U+686F.0	fCNS11643	2-357B
U+686F.0	fBig5	D5B9
U+686F.0	fGB	3-4153
U+686F.0	fR/S	 75.7
U+686F.0	fTotalStrokes	 11
U+686F.0	fCangjie	DRHG
U+686F.0	fFourCorner	46914
U+686F.0	fHakka	 tin2; tin1
U+686F.0	fCantonese	ting1; jing4
U+686F.0	fMandarin	ting1; xing2; ying2
U+686F.0	fEnglish	table
U+6870.0	fUTF8	桰
U+6870.0	fCNS11643	3-3660
U+6870.0	fGB	5-4426
U+6870.0	fR/S	  75.7
U+6870.0	fTotalStrokes	 11
U+6870.0	fCangjie	DHPR
U+6870.0	fFourCorner	42960
U+6870.0	fHakka	 kwat7
U+6870.0	fCantonese	kut3; kwut3 (also)
U+6870.0	fMandarin	gua1; kuo4
U+6870.0	fEnglish	measuring frame used by builders
U+6871.0	fUTF8	桱
U+6871.0	fCNS11643	2-362C
U+6871.0	fBig5	D5C8
U+6871.0	fGB	3-4124
U+6871.0	fR/S	 75.7
U+6871.0	fTotalStrokes	 11
U+6871.0	fCangjie	DMVM
U+6871.0	fFourCorner	41911
U+6871.0	fHakka	 gin5; kwang1
U+6871.0	fCantonese	ging3
U+6871.0	fMandarin	jing4; jing1
U+6871.0	fEnglish	[1] a kind of hard timber [2] roller used by silk-dryers
U+6872.0	fUTF8	桲
U+6872.0	fCNS11643	2-3629
U+6872.0	fBig5	D5C5
U+6872.0	fGB	3-414B
U+6872.0	fR/S	 75.7
U+6872.0	fTotalStrokes	 11
U+6872.0	fCangjie	DJBD
U+6872.0	fFourCorner	44947
U+6872.0	fHakka	 put8
U+6872.0	fCantonese	but6
U+6872.0	fMandarin	bo2; po5
U+6872.0	fEnglish	[1] flail [2] a fruit
U+6873.0	fUTF8	桳
U+6873.0	fCNS11643	3-364E
U+6873.0	fGB	E-2D7D
U+6873.0	fR/S	  75.7
U+6873.0	fTotalStrokes	 11
U+6873.0	fCangjie	DIKJ
U+6873.0	fFourCorner	43940
U+6873.0	fCantonese	ban6
U+6873.0	fMandarin	ben4
U+6874.0	fUTF8	桴
U+6874.0	fCNS11643	2-3622
U+6874.0	fBig5	D5BE
U+6874.0	fGB	0-6875
U+6874.0	fR/S	 75.7
U+6874.0	fTotalStrokes	 11
U+6874.0	fCangjie	DBND
U+6874.0	fFourCorner	42947
U+6874.0	fHakka	  fu1; feu5
U+6874.0	fCantonese	fu1; fau4
U+6874.0	fMandarin	fu2; fou2; fu1
U+6874.0	fEnglish	[1] raft (of wood or bamboo) [2] drumstick [3] ridge pole on a roof
U+6875.0	fUTF8	桵
U+6875.0	fCNS11643	2-3621
U+6875.0	fBig5	D5BD
U+6875.0	fGB	3-4161
U+6875.0	fR/S	 75.7
U+6875.0	fTotalStrokes	 11
U+6875.0	fCangjie	DBV
U+6875.0	fFourCorner	42944
U+6875.0	fCantonese	jeoi4; jeoi1
U+6875.0	fMandarin	rui3
U+6876.0	fUTF8	桶
U+6876.0	fCNS11643	1-5B33
U+6876.0	fBig5	B1ED
U+6876.0	fGB	0-4D30
U+6876.0	fR/S	 75.7
U+6876.0	fTotalStrokes	 11
U+6876.0	fCangjie	DNIB
U+6876.0	fFourCorner	47927
U+6876.0	fHakka	  tung3
U+6876.0	fCantonese	tung2
U+6876.0	fMandarin	tong3; dong4; yong3
U+6876.0	fEnglish	 pail; bucket; tub; cask; keg; barrel
U+6877.0	fUTF8	桷
U+6877.0	fCNS11643	2-3625
U+6877.0	fBig5	D5C1
U+6877.0	fGB	0-6876
U+6877.0	fR/S	 75.7
U+6877.0	fTotalStrokes	 11
U+6877.0	fCangjie	DNBG
U+6877.0	fFourCorner	47927
U+6877.0	fHakka	  gok7
U+6877.0	fCantonese	gok3; gok2 (also)
U+6877.0	fMandarin	jue2
U+6877.0	fEnglish	[1] rafter [2] Malus toringo
U+6878.0	fUTF8	桸
U+6878.0	fCNS11643	2-3634
U+6878.0	fBig5	D5D0
U+6878.0	fGB	3-415E
U+6878.0	fR/S	 75.7
U+6878.0	fTotalStrokes	 11
U+6878.0	fCangjie	DKKB
U+6878.0	fFourCorner	44927
U+6878.0	fCantonese	hei1
U+6878.0	fMandarin	xi1
U+6879.0	fUTF8	桹
U+6879.0	fCNS11643	2-3572
U+6879.0	fBig5	D5B0
U+6879.0	fGB	5-442D
U+6879.0	fR/S	 75.7
U+6879.0	fTotalStrokes	 11
U+6879.0	fCangjie	DIAV
U+6879.0	fFourCorner	43932
U+6879.0	fHakka	 long2; nong3
U+6879.0	fCantonese	long4
U+6879.0	fMandarin	lang2
U+6879.0	fEnglish	palm
U+687A.0	fUTF8	桺
U+687A.0	fCNS11643	3-365B
U+687A.0	fGB	E-2D7E
U+687A.0	fR/S	 75.7
U+687A.0	fTotalStrokes	 11
U+687A.0	fCangjie	DMLS
U+687A.0	fFourCorner	41970
U+687A.0	fHakka	  liu3; liu1; lieu3; leu1
U+687A.0	fCantonese	lau5
U+687A.0	fMandarin	liu3; ping2
U+687A.0	fEnglish	standard form of &#x67F3;
U+687B.0	fUTF8	桻
U+687B.0	fCNS11643	2-3635
U+687B.0	fBig5	D5D1
U+687B.0	fGB	5-4428
U+687B.0	fR/S	 75.7
U+687B.0	fTotalStrokes	 11
U+687B.0	fCangjie	DHEJ
U+687B.0	fFourCorner	47954
U+687B.0	fCantonese	fung1
U+687B.0	fMandarin	feng1
U+687C.0	fUTF8	桼
U+687C.0	fCNS11643	2-3627
U+687C.0	fBig5	D5C3
U+687C.0	fGB	5-4425
U+687C.0	fR/S	 75.7
U+687C.0	fTotalStrokes	 11
U+687C.0	fCangjie	DOE
U+687C.0	fFourCorner	40232
U+687C.0	fHakka	  cit7
U+687C.0	fCantonese	cat1
U+687C.0	fMandarin	qi1
U+687C.0	fEnglish	[1] the varnish tree (Rhus vernicifera) [2] lacquer
U+687D.0	fUTF8	桽
U+687D.0	fCNS11643	2-3639
U+687D.0	fBig5	D5D5
U+687D.0	fGB	E-2E21
U+687D.0	fR/S	 75.7
U+687D.0	fTotalStrokes	 11
U+687D.0	fCangjie	DOOG
U+687D.0	fFourCorner	40104
U+687D.0	fCantonese	wan2
U+687D.0	fMandarin	wen3
U+687E.0	fUTF8	桾
U+687E.0	fCNS11643	2-362D
U+687E.0	fBig5	D5C9
U+687E.0	fGB	5-442E
U+687E.0	fR/S	 75.7
U+687E.0	fTotalStrokes	 11
U+687E.0	fCangjie	DSKR
U+687E.0	fFourCorner	47967
U+687E.0	fCantonese	gwan1
U+687E.0	fMandarin	jun1; jun4
U+687F.0	fUTF8	桿
U+687F.0	fCNS11643	1-5B32
U+687F.0	fBig5	B1EC
U+687F.0	fGB	E-2E22
U+687F.0	fR/S	 75.7
U+687F.0	fTotalStrokes	 11
U+687F.0	fCangjie	DAMJ
U+687F.0	fFourCorner	46941
U+687F.0	fHakka	  gon5; gon1; gon3; hon6; hon5; hon3
U+687F.0	fCantonese	gon3; gon2; gon1 (also)
U+687F.0	fMandarin	gan3; han4
U+687F.0	fEnglish	 wooden pole; stick; club
U+6880.0	fUTF8	梀
U+6880.0	fCNS11643	2-362B
U+6880.0	fBig5	D5C7
U+6880.0	fGB	3-414C
U+6880.0	fR/S	 75.7
U+6880.0	fTotalStrokes	 11
U+6880.0	fCangjie	DDL
U+6880.0	fFourCorner	45996
U+6880.0	fHakka	 suk7
U+6880.0	fCantonese	saak3; jan3; suk1; cuk1
U+6880.0	fMandarin	cu4; su4; yin4
U+6880.0	fEnglish	a kind of birch tree
U+6881.0	fUTF8	梁
U+6881.0	fCNS11643	1-5B2D
U+6881.0	fBig5	B1E7
U+6881.0	fGB	0-413A
U+6881.0	fR/S	 75.7
U+6881.0	fTotalStrokes	 11
U+6881.0	fCangjie	EID
U+6881.0	fFourCorner	33904
U+6881.0	fHakka	  liong2; liong3
U+6881.0	fCantonese	loeng4; loeng2 (also)
U+6881.0	fMandarin	liang2
U+6881.0	fEnglish	[1] bridge [2] beam of a house [3] ridge; swelling [4] Liang dynasty (502-557 A.D.) [5] a state during the Warring States, also known as Wei (@) [6] a Chinese family name
U+6881.1	fUTF8	梁
U+6881.1	fCNS11643	1-5B2D
U+6881.1	fBig5	B1E7
U+6881.1	fGB	0-413A
U+6881.1	fR/S	 75.7
U+6881.1	fTotalStrokes	 11
U+6881.1	fCangjie	EID
U+6881.1	fFourCorner	33904
U+6881.1	fMandarin	liang5
U+6882.0	fUTF8	梂
U+6882.0	fCNS11643	1-5B42
U+6882.0	fBig5	B1FC
U+6882.0	fGB	3-4152
U+6882.0	fR/S	 75.7
U+6882.0	fTotalStrokes	 11
U+6882.0	fCangjie	DIJE
U+6882.0	fFourCorner	43932
U+6882.0	fHakka	  kiu2
U+6882.0	fCantonese	kau4
U+6882.0	fMandarin	qiu2; ju1; ju2
U+6882.0	fEnglish	the cup of an acorn
U+6883.0	fUTF8	梃
U+6883.0	fCNS11643	1-5B38
U+6883.0	fBig5	B1F2
U+6883.0	fGB	0-6868
U+6883.0	fR/S	 75.7
U+6883.0	fTotalStrokes	 11
U+6883.0	fCangjie	DNKG
U+6883.0	fFourCorner	42941
U+6883.0	fHakka	 tin3; tiang3
U+6883.0	fCantonese	ting5
U+6883.0	fMandarin	ting3; ting4; ting2
U+6883.0	fEnglish	[1] club; cudgel; stick; cane [2] stalk; branch; stem [3] straight and strong
U+6884.0	fUTF8	梄
U+6884.0	fCNS11643	3-3657
U+6884.0	fGB	5-4379
U+6884.0	fR/S	 75.7
U+6884.0	fTotalStrokes	 11
U+6884.0	fCangjie	DMCW
U+6884.0	fFourCorner	41960
U+6884.0	fHakka	 ju3; riu3
U+6884.0	fCantonese	jau5
U+6884.0	fMandarin	you3; you2; you4
U+6884.0	fEnglish	same as &#x69F1;
U+6885.0	fUTF8	梅
U+6885.0	fCNS11643	1-5B3C
U+6885.0	fBig5	B1F6
U+6885.0	fGB	0-4337
U+6885.0	fR/S	 75.7; 75.6
U+6885.0	fTotalStrokes	 11
U+6885.0	fCangjie	DOWY
U+6885.0	fFourCorner	48957
U+6885.0	fHakka	  moi2; moi3; moe2
U+6885.0	fCantonese	mui4; mui1 (also); mui2 (also)
U+6885.0	fMandarin	mei2; mei3
U+6885.0	fEnglish	[1] plums; prunes [2] a Chinese family name
U+6886.0	fUTF8	梆
U+6886.0	fCNS11643	1-5B3B
U+6886.0	fBig5	B1F5
U+6886.0	fGB	0-3070
U+6886.0	fR/S	 75.7
U+6886.0	fTotalStrokes	 11
U+6886.0	fCangjie	DQJL
U+6886.0	fFourCorner	47927
U+6886.0	fHakka	 bang1;  bong1; bok8
U+6886.0	fCantonese	bong1
U+6886.0	fMandarin	bang1
U+6886.0	fEnglish	 watchman's rattle made of wood or bamboo
U+6887.0	fUTF8	梇
U+6887.0	fCNS11643	2-3573
U+6887.0	fBig5	D5B1
U+6887.0	fGB	3-4146
U+6887.0	fR/S	 75.7
U+6887.0	fTotalStrokes	 11
U+6887.0	fCangjie	DMGT
U+6887.0	fFourCorner	41941
U+6887.0	fCantonese	lung6
U+6887.0	fMandarin	long4; nong4
U+6888.0	fUTF8	梈
U+6888.0	fCNS11643	4-323F
U+6888.0	fGB	3-4162
U+6888.0	fR/S	 75.7
U+6888.0	fTotalStrokes	 11
U+6888.0	fCangjie	DYRN
U+6888.0	fFourCorner	40920
U+6888.0	fCantonese	paang1
U+6888.0	fMandarin	peng1
U+6889.0	fUTF8	梉
U+6889.0	fCNS11643	2-3632
U+6889.0	fBig5	D5CE
U+6889.0	fGB	E-2E23
U+6889.0	fR/S	 75.7
U+6889.0	fTotalStrokes	 11
U+6889.0	fCangjie	DVMG
U+6889.0	fFourCorner	44910
U+6889.0	fCantonese	zong1
U+6889.0	fMandarin	zhuang1
U+688A.0	fUTF8	梊
U+688A.0	fCNS11643	2-3638
U+688A.0	fBig5	D5D4
U+688A.0	fGB	5-437C
U+688A.0	fR/S	 75.7
U+688A.0	fTotalStrokes	 11
U+688A.0	fCangjie	QLD
U+688A.0	fFourCorner	52904
U+688A.0	fCantonese	dai3; dai6
U+688A.0	fMandarin	di4; die2; ti4
U+688A.0	fEnglish	 squeeze out of extract
U+688B.0	fUTF8	梋
U+688B.0	fCNS11643	2-3630
U+688B.0	fBig5	D5CC
U+688B.0	fGB	3-4158
U+688B.0	fR/S	 75.7
U+688B.0	fTotalStrokes	 11
U+688B.0	fCangjie	DRB
U+688B.0	fFourCorner	46927
U+688B.0	fCantonese	hyun1
U+688B.0	fMandarin	xuan1; xi1; yan3
U+688C.0	fUTF8	梌
U+688C.0	fCNS11643	2-3637
U+688C.0	fBig5	D5D3
U+688C.0	fGB	3-415D
U+688C.0	fR/S	 75.7
U+688C.0	fTotalStrokes	 11
U+688C.0	fCangjie	DOMD
U+688C.0	fFourCorner	48994
U+688C.0	fHakka	 tu2
U+688C.0	fCantonese	tou4
U+688C.0	fMandarin	tu2; cha2; jia2; tu1; yu2
U+688C.0	fEnglish	a tree, the wood of which is used for making forniture
U+688D.0	fUTF8	梍
U+688D.0	fGB	5-4422
U+688D.0	fR/S	 75.7
U+688D.0	fTotalStrokes	 11
U+688D.0	fCangjie	XDHAP
U+688D.0	fFourCorner	4691
U+688D.0	fCantonese	zou6
U+688D.0	fMandarin	zao4
U+688D.0	fEnglish	tree name
U+688E.0	fUTF8	梎
U+688E.0	fCNS11643	3-3651
U+688E.0	fGB	3-415C
U+688E.0	fR/S	  75.7
U+688E.0	fTotalStrokes	 11
U+688E.0	fCangjie	DHAP
U+688E.0	fFourCorner	46910
U+688E.0	fCantonese	au1; ngau1 (also)
U+688E.0	fMandarin	zao4; ao1; you4
U+688E.0	fEnglish	sickle handle; branch
U+688F.0	fUTF8	梏
U+688F.0	fCNS11643	2-3624
U+688F.0	fBig5	D5C0
U+688F.0	fGB	0-6874
U+688F.0	fR/S	 75.7
U+688F.0	fTotalStrokes	 11
U+688F.0	fCangjie	DHGR
U+688F.0	fFourCorner	44961
U+688F.0	fHakka	  guk7;  kuk7; gok7; kwuk7
U+688F.0	fCantonese	guk1; gok3
U+688F.0	fMandarin	gu4; jiao3
U+688F.0	fEnglish	[1] handcuffs; hand-shackles; manacles [2] detain; imprison
U+688F.1	fUTF8	梏
U+688F.1	fCNS11643	2-3624
U+688F.1	fBig5	D5C0
U+688F.1	fGB	0-6874
U+688F.1	fR/S	 75.7
U+688F.1	fTotalStrokes	 11
U+688F.1	fCangjie	DHGR
U+688F.1	fFourCorner	44961
U+688F.1	fMandarin	jue2
U+688F.1	fEnglish	[1] great [2] straightforward; upright
U+6890.0	fUTF8	梐
U+6890.0	fCNS11643	2-3574
U+6890.0	fBig5	D5B2
U+6890.0	fGB	3-4151
U+6890.0	fR/S	 75.7
U+6890.0	fTotalStrokes	 11
U+6890.0	fCangjie	DPPG
U+6890.0	fFourCorner	41914
U+6890.0	fHakka	 pi3; bi3; pi5
U+6890.0	fCantonese	bai6
U+6890.0	fMandarin	bi4; bi1
U+6890.0	fEnglish	fence
U+6891.0	fUTF8	梑
U+6891.0	fCNS11643	2-3636
U+6891.0	fBig5	D5D2
U+6891.0	fGB	5-4427
U+6891.0	fR/S	 75.7
U+6891.0	fTotalStrokes	 11
U+6891.0	fCangjie	DKHF
U+6891.0	fFourCorner	49980
U+6891.0	fCantonese	dik6
U+6891.0	fMandarin	di2
U+6892.0	fUTF8	梒
U+6892.0	fCNS11643	2-3626
U+6892.0	fBig5	D5C2
U+6892.0	fGB	5-4424
U+6892.0	fR/S	 75.7
U+6892.0	fTotalStrokes	 11
U+6892.0	fCangjie	DOIR
U+6892.0	fFourCorner	48962
U+6892.0	fHakka	 ham2
U+6892.0	fCantonese	ham4
U+6892.0	fMandarin	han2
U+6893.0	fUTF8	梓
U+6893.0	fCNS11643	1-5B30
U+6893.0	fBig5	B1EA
U+6893.0	fGB	0-6877
U+6893.0	fR/S	 75.7
U+6893.0	fTotalStrokes	 11
U+6893.0	fCangjie	DYTJ
U+6893.0	fFourCorner	40941
U+6893.0	fHakka	 zih3; zu3; zii3
U+6893.0	fCantonese	zi2
U+6893.0	fMandarin	zi3
U+6893.0	fEnglish	[1] Catalpa ovata [2] one's native place or hometown [3] make furniture; furniture [4] carve words on a woodboard; printing blocks [5] a Chinese family name
U+6894.0	fUTF8	梔
U+6894.0	fCNS11643	1-5B3D
U+6894.0	fBig5	B1F7
U+6894.0	fGB	E-2E24
U+6894.0	fR/S	 75.7
U+6894.0	fTotalStrokes	 11
U+6894.0	fCangjie	DHMU
U+6894.0	fFourCorner	42917
U+6894.0	fHakka	 gi1; zi1; zhi1; zih1
U+6894.0	fCantonese	zi1
U+6894.0	fMandarin	zhi1
U+6894.0	fEnglish	 gardenia
U+6895.0	fUTF8	梕
U+6895.0	fCNS11643	3-365A
U+6895.0	fGB	5-4431
U+6895.0	fR/S	  75.7
U+6895.0	fTotalStrokes	 11
U+6895.0	fCangjie	DSIP
U+6895.0	fFourCorner	47930
U+6895.0	fHakka	 niun5
U+6895.0	fCantonese	jan6
U+6895.0	fMandarin	ren4
U+6895.0	fEnglish	same as &#x6752;
U+6896.0	fUTF8	梖
U+6896.0	fCNS11643	2-362F
U+6896.0	fBig5	D5CB
U+6896.0	fGB	5-4342
U+6896.0	fR/S	 75.7
U+6896.0	fTotalStrokes	 11
U+6896.0	fCangjie	DBUC
U+6896.0	fFourCorner	46980
U+6896.0	fHakka	  bi5;  bui5
U+6896.0	fCantonese	bui3
U+6896.0	fMandarin	bei4
U+6896.0	fEnglish	a palm-tree
U+6897.0	fUTF8	梗
U+6897.0	fCNS11643	1-5B36
U+6897.0	fBig5	B1F0
U+6897.0	fGB	0-3923
U+6897.0	fR/S	 75.7
U+6897.0	fTotalStrokes	 11
U+6897.0	fCangjie	DMLK
U+6897.0	fFourCorner	41946
U+6897.0	fHakka	 gien3; gang3; kwang3; gen3
U+6897.0	fCantonese	gang2; gwaang2 (also)
U+6897.0	fMandarin	geng3
U+6897.0	fEnglish	[1] stem of a plants; branch of a plant [2] prick or pierce with a thorn; thorny [3] outline; synopsis; summary [4] block; obstruct [5] stubborn; stiff [6] fierce and fearless [7] ailment; bane; distress [8] straighten [9] honest; straight [10] eat to the full
U+6898.0	fUTF8	梘
U+6898.0	fCNS11643	3-364D
U+6898.0	fGB	1-6845
U+6898.0	fR/S	 75.7
U+6898.0	fTotalStrokes	 11
U+6898.0	fCangjie	DBUU
U+6898.0	fFourCorner	46910
U+6898.0	fHakka	 gan3; gien5; hien5; gian3; gen5
U+6898.0	fCantonese	gaan2
U+6898.0	fMandarin	jian4; jian3; xian4
U+6898.0	fEnglish	 bamboo tube; wooden peg spout
U+6899.0	fUTF8	梙
U+6899.0	fCNS11643	3-365C
U+6899.0	fGB	5-437D
U+6899.0	fR/S	  75.7
U+6899.0	fTotalStrokes	 11
U+6899.0	fCangjie	DLL
U+6899.0	fFourCorner	45900
U+6899.0	fHakka	 fam6;  fam5
U+6899.0	fCantonese	waan6
U+6899.0	fMandarin	huan4
U+6899.0	fEnglish	same as &#x69F5;
U+689A.0	fUTF8	梚
U+689A.0	fCNS11643	3-3650
U+689A.0	fGB	E-2E25
U+689A.0	fR/S	  75.7
U+689A.0	fTotalStrokes	 11
U+689A.0	fCangjie	DNAU
U+689A.0	fFourCorner	47910
U+689A.0	fHakka	 tu2
U+689A.0	fCantonese	maan5
U+689A.0	fMandarin	wan3
U+689B.0	fUTF8	梛
U+689B.0	fCNS11643	2-362E
U+689B.0	fBig5	D5CA
U+689B.0	fGB	5-4373
U+689B.0	fR/S	 75.7
U+689B.0	fTotalStrokes	 11
U+689B.0	fCangjie	DSQL
U+689B.0	fFourCorner	47927
U+689B.0	fCantonese	naa4; laa4 (also); no4
U+689B.0	fMandarin	nuo2
U+689C.0	fUTF8	梜
U+689C.0	fCNS11643	2-3575
U+689C.0	fBig5	D5B3
U+689C.0	fGB	3-412E
U+689C.0	fR/S	 75.7
U+689C.0	fTotalStrokes	 11
U+689C.0	fCangjie	DKOO
U+689C.0	fFourCorner	54038
U+689C.0	fHakka	  giap7; kiap8
U+689C.0	fCantonese	gaap3; zit6; hip3
U+689C.0	fMandarin	jia2; jia1
U+689C.0	fEnglish	chopsitck
U+689D.0	fUTF8	條
U+689D.0	fCNS11643	1-5B3E
U+689D.0	fBig5	B1F8
U+689D.0	fGB	1-4C75
U+689D.0	fR/S	 75.7
U+689D.0	fTotalStrokes	 11
U+689D.0	fCangjie	OLOD
U+689D.0	fFourCorner	28294
U+689D.0	fHakka	  tiau2; tiau3
U+689D.0	fCantonese	tiu4; tiu5 (also); tiu2 (also)
U+689D.0	fMandarin	tiao2; di2
U+689D.0	fEnglish	[1] article, clause, section of an agreement, pact, treaty, law, etc. [2] stripe [3] in good order; (present) one by one [4] numerical classifier for narrow and long things (roads, fish, ropes, dogs, snakes, etc.)
U+689E.0	fUTF8	梞
U+689E.0	fCNS11643	3-364B
U+689E.0	fGB	3-4169
U+689E.0	fR/S	  75.7
U+689E.0	fTotalStrokes	 11
U+689E.0	fCangjie	DSUP
U+689E.0	fFourCorner	47930
U+689E.0	fCantonese	gei6
U+689E.0	fMandarin	ji4
U+689F.0	fUTF8	梟
U+689F.0	fCNS11643	1-5B40
U+689F.0	fBig5	B1FA
U+689F.0	fGB	1-6849
U+689F.0	fR/S	 75.7
U+689F.0	fTotalStrokes	 11
U+689F.0	fCangjie	HAYD
U+689F.0	fFourCorner	27904
U+689F.0	fHakka	  hiau1; hieu1
U+689F.0	fCantonese	hiu1
U+689F.0	fMandarin	xiao1; jiao1
U+689F.0	fEnglish	[1] owl thus; a legendary bird said to eat its own mother [2] smuggler of contraband, narcotics, etc. [3] brave and unscrupulous
U+68A0.0	fUTF8	梠
U+68A0.0	fCNS11643	2-3631
U+68A0.0	fBig5	D5CD
U+68A0.0	fGB	3-4131
U+68A0.0	fR/S	 75.7
U+68A0.0	fTotalStrokes	 11
U+68A0.0	fCangjie	DRHR
U+68A0.0	fFourCorner	46960
U+68A0.0	fHakka	  li3
U+68A0.0	fCantonese	leoi5
U+68A0.0	fMandarin	l3; luu3; lu3
U+68A0.0	fEnglish	a small beam supporting the rafters at the eaves
U+68A1.0	fUTF8	梡
U+68A1.0	fCNS11643	1-5B41
U+68A1.0	fBig5	B1FB
U+68A1.0	fGB	3-4164
U+68A1.0	fR/S	 75.7
U+68A1.0	fTotalStrokes	 11
U+68A1.0	fCangjie	DJMU
U+68A1.0	fFourCorner	43911
U+68A1.0	fHakka	 fan3;  k'wan3
U+68A1.0	fCantonese	fun2
U+68A1.0	fMandarin	hun2; kuan3; hun1; huan2; huan3; huan4; hun4; nuan4
U+68A1.0	fEnglish	tray for carrying sacrificial meat
U+68A2.0	fUTF8	梢
U+68A2.0	fCNS11643	1-5B2F
U+68A2.0	fBig5	B1E9
U+68A2.0	fGB	0-4952
U+68A2.0	fR/S	 75.7
U+68A2.0	fTotalStrokes	 11
U+68A2.0	fCangjie	DFB
U+68A2.0	fFourCorner	49927
U+68A2.0	fHakka	 sau1; siau1
U+68A2.0	fCantonese	saau1
U+68A2.0	fMandarin	shao1; shao4; xiao1; sao4; shuo4
U+68A2.0	fEnglish	[1] pointed tip of something long like a branch; the tip of a branch or things of similar shape [2] rudder [3] the end of something; result [4] the sound of wind
U+68A3.0	fUTF8	梣
U+68A3.0	fCNS11643	2-357C
U+68A3.0	fBig5	D5BA
U+68A3.0	fGB	3-415B
U+68A3.0	fR/S	 75.7
U+68A3.0	fTotalStrokes	 11
U+68A3.0	fCangjie	DUON
U+68A3.0	fFourCorner	42927
U+68A3.0	fCantonese	sam4; zam1; cam4
U+68A3.0	fMandarin	cen2; chen2; qin2
U+68A4.0	fUTF8	梤
U+68A4.0	fCNS11643	2-3633
U+68A4.0	fBig5	D5CF
U+68A4.0	fGB	5-4430
U+68A4.0	fR/S	 75.7
U+68A4.0	fTotalStrokes	 11
U+68A4.0	fCangjie	DUCH
U+68A4.0	fFourCorner	42927
U+68A4.0	fCantonese	fan4
U+68A4.0	fMandarin	fen2; fen1
U+68A5.0	fUTF8	梥
U+68A5.0	fCNS11643	3-3654
U+68A5.0	fGB	5-3E2B
U+68A5.0	fR/S	  75.7
U+68A5.0	fTotalStrokes	 11
U+68A5.0	fCangjie	JCID
U+68A5.0	fFourCorner	30900
U+68A5.0	fHakka	 ciung2; cung2; sung1; ciung3
U+68A5.0	fCantonese	cung4; cung2 (also); sung1
U+68A5.0	fMandarin	song1; gong1; song4
U+68A5.0	fEnglish	ancient form of &#x677E;
U+68A6.0	fUTF8	梦
U+68A6.0	fCNS11643	3-3478
U+68A6.0	fGB	0-434E
U+68A6.0	fR/S	 75.7; 36.7
U+68A6.0	fTotalStrokes	 11
U+68A6.0	fCangjie	DDNI
U+68A6.0	fFourCorner	44200
U+68A6.0	fHakka	 mung5
U+68A6.0	fCantonese	mung6; mung4
U+68A6.0	fMandarin	meng4; meng2; jie2
U+68A6.0	fEnglish	[1] non-classical form of &#x5922; [2] simplified form of &#x5922;
U+68A7.0	fUTF8	梧
U+68A7.0	fCNS11643	1-5B35
U+68A7.0	fBig5	B1EF
U+68A7.0	fGB	0-4E60
U+68A7.0	fR/S	 75.7
U+68A7.0	fTotalStrokes	 11
U+68A7.0	fCangjie	DMMR
U+68A7.0	fFourCorner	41961
U+68A7.0	fHakka	  ng2;  ngu2; ng3
U+68A7.0	fCantonese	ng4; ng6 (also)
U+68A7.0	fMandarin	wu2; wu4; yu3; wu3
U+68A7.0	fEnglish	[1] a firmiana (Firmiana platanifolia) [2] support; prop [3] support; prop [4] a Chinese family name
U+68A8.0	fUTF8	梨
U+68A8.0	fCNS11643	1-5B3F
U+68A8.0	fBig5	B1F9
U+68A8.0	fGB	0-4066
U+68A8.0	fR/S	 75.7
U+68A8.0	fTotalStrokes	 11
U+68A8.0	fCangjie	HND
U+68A8.0	fFourCorner	22904
U+68A8.0	fHakka	  li2
U+68A8.0	fCantonese	lei4; lei2 (also)
U+68A8.0	fMandarin	li2
U+68A8.0	fEnglish	 pear opera cut; slash
U+68A9.0	fUTF8	梩
U+68A9.0	fCNS11643	2-357E
U+68A9.0	fBig5	D5BC
U+68A9.0	fGB	3-4155
U+68A9.0	fR/S	 75.7
U+68A9.0	fTotalStrokes	 11
U+68A9.0	fCangjie	DWG
U+68A9.0	fFourCorner	46914
U+68A9.0	fHakka	 li2
U+68A9.0	fCantonese	lei4; lei5
U+68A9.0	fMandarin	li2; dai1; si4; si3; zhan1
U+68A9.0	fEnglish	hod
U+68AA.0	fUTF8	梪
U+68AA.0	fCNS11643	2-362A
U+68AA.0	fBig5	D5C6
U+68AA.0	fGB	3-414D
U+68AA.0	fR/S	 75.7
U+68AA.0	fTotalStrokes	 11
U+68AA.0	fCangjie	DMRT
U+68AA.0	fFourCorner	41918
U+68AA.0	fHakka	 teu5; teu6
U+68AA.0	fCantonese	dau6
U+68AA.0	fMandarin	dou4
U+68AA.0	fEnglish	 sacrificial vessel
U+68AB.0	fUTF8	梫
U+68AB.0	fCNS11643	2-3579
U+68AB.0	fBig5	D5B7
U+68AB.0	fGB	3-4165
U+68AB.0	fR/S	 75.7
U+68AB.0	fTotalStrokes	 11
U+68AB.0	fCangjie	DSME
U+68AB.0	fFourCorner	47947
U+68AB.0	fHakka	 cam1;  cim1
U+68AB.0	fCantonese	caam1; cam2 (also); cam1
U+68AB.0	fMandarin	cen1; jin4; qin1; qiu3
U+68AB.0	fEnglish	the Cassia lignea tree
U+68AC.0	fUTF8	梬
U+68AC.0	fCNS11643	2-357D
U+68AC.0	fBig5	D5BB
U+68AC.0	fGB	3-4156
U+68AC.0	fR/S	 75.7
U+68AC.0	fTotalStrokes	 11
U+68AC.0	fCangjie	DLWS
U+68AC.0	fFourCorner	45927
U+68AC.0	fCantonese	jing5
U+68AC.0	fMandarin	ying3; cheng3
U+68AD.0	fUTF8	梭
U+68AD.0	fCNS11643	1-5B3A
U+68AD.0	fBig5	B1F4
U+68AD.0	fGB	0-4B73
U+68AD.0	fR/S	 75.7
U+68AD.0	fTotalStrokes	 11
U+68AD.0	fCangjie	DICE
U+68AD.0	fFourCorner	43947
U+68AD.0	fHakka	  so1; zun5; sii1
U+68AD.0	fCantonese	so1
U+68AD.0	fMandarin	suo1; xun4; quan1; xun1
U+68AD.0	fEnglish	[1] weaver's shuttle [2] move to and fro; shuttle
U+68AE.0	fUTF8	梮
U+68AE.0	fCNS11643	2-3578
U+68AE.0	fBig5	D5B6
U+68AE.0	fGB	3-4168
U+68AE.0	fR/S	 75.7
U+68AE.0	fTotalStrokes	 11
U+68AE.0	fCangjie	DSSR
U+68AE.0	fFourCorner	47927
U+68AE.0	fHakka	  kiuk8
U+68AE.0	fCantonese	guk6; guk1 (also)
U+68AE.0	fMandarin	ju2
U+68AE.0	fEnglish	 put spikes on shoes to prevent slipping
U+68AF.0	fUTF8	梯
U+68AF.0	fCNS11643	1-5B2E
U+68AF.0	fBig5	B1E8
U+68AF.0	fGB	0-4C5D
U+68AF.0	fR/S	 75.7
U+68AF.0	fTotalStrokes	 11
U+68AF.0	fCangjie	DCNH
U+68AF.0	fFourCorner	48927
U+68AF.0	fHakka	  toi1; ti1; ti2; {toi1}
U+68AF.0	fCantonese	tai1; tai2
U+68AF.0	fMandarin	ti1; ti2; fou2
U+68AF.0	fEnglish	[1] ladder; steps; stairs; [2] something to lean or depend on [3] terraced [4] private; intimate [5] phase (in the induction of military draftees)
U+68B0.0	fUTF8	械
U+68B0.0	fCNS11643	1-5B37
U+68B0.0	fBig5	B1F1
U+68B0.0	fGB	0-5035
U+68B0.0	fR/S	 75.7
U+68B0.0	fTotalStrokes	 11
U+68B0.0	fCangjie	DIT
U+68B0.0	fFourCorner	43950
U+68B0.0	fHakka	 hai5; hai3; hai6
U+68B0.0	fCantonese	haai6
U+68B0.0	fMandarin	xie4; jie4
U+68B0.0	fEnglish	[1] weapons [2] implements; machinery; machines [3] shackles; fetters [4] arrest and put in prison
U+68B1.0	fUTF8	梱
U+68B1.0	fCNS11643	1-5B34
U+68B1.0	fBig5	B1EE
U+68B1.0	fGB	3-4157
U+68B1.0	fR/S	 75.7
U+68B1.0	fTotalStrokes	 11
U+68B1.0	fCangjie	DWD
U+68B1.0	fFourCorner	46900
U+68B1.0	fHakka	 kun5; kun3; kwun5
U+68B1.0	fCantonese	kwan2
U+68B1.0	fMandarin	kun3; gun4; kun4; wen2
U+68B1.0	fEnglish	 doorsill doorjamb
U+68B2.0	fUTF8	梲
U+68B2.0	fCNS11643	2-3623
U+68B2.0	fBig5	D5BF
U+68B2.0	fGB	E-2E26
U+68B2.0	fR/S	 75.7
U+68B2.0	fTotalStrokes	 11
U+68B2.0	fCangjie	DCRU
U+68B2.0	fFourCorner	48916
U+68B2.0	fHakka	 tot7; chot7; zot7; zhot7
U+68B2.0	fCantonese	zyut3; tyut3; jeoi6
U+68B2.0	fMandarin	zhuo2; rui4; tu2; tuo1
U+68B2.0	fEnglish	[1] joist [2] cane; club
U+68B3.0	fUTF8	梳
U+68B3.0	fCNS11643	1-5622
U+68B3.0	fBig5	AEDE
U+68B3.0	fGB	0-4A61
U+68B3.0	fR/S	 75.7
U+68B3.0	fTotalStrokes	 11
U+68B3.0	fCangjie	DYIU
U+68B3.0	fFourCorner	40913
U+68B3.0	fHakka	  so1;  sii1; liu3; se1
U+68B3.0	fCantonese	so1
U+68B3.0	fMandarin	shu1; shu4
U+68B3.0	fEnglish	[1] comb; coarse comb; brush [2] comb
U+68B4.0	fUTF8	梴
U+68B4.0	fCNS11643	2-3C63
U+68B4.0	fBig5	D9C0
U+68B4.0	fGB	3-4134
U+68B4.0	fR/S	 75.7
U+68B4.0	fTotalStrokes	 11
U+68B4.0	fCangjie	DNKM
U+68B4.0	fFourCorner	42941
U+68B4.0	fHakka	 jen2; ren2; chan2; can2
U+68B4.0	fCantonese	cin1
U+68B4.0	fMandarin	chan1
U+68B4.0	fEnglish	long
U+68B5.0	fUTF8	梵
U+68B5.0	fCNS11643	1-5B31
U+68B5.0	fBig5	B1EB
U+68B5.0	fGB	0-6873
U+68B5.0	fR/S	 75.7
U+68B5.0	fTotalStrokes	 11
U+68B5.0	fCangjie	DDHNI
U+68B5.0	fFourCorner	44217
U+68B5.0	fHakka	 fam2;  fam5; fan5
U+68B5.0	fCantonese	faan6; faan4 (also)
U+68B5.0	fMandarin	fan4; feng1; feng2
U+68B5.0	fEnglish	[1] clean and pure [2] Brahman [3] Sanskrit [4] anything pertaining to Buddhism
U+68B6.0	fUTF8	梶
U+68B6.0	fCNS11643	3-364C
U+68B6.0	fGB	3-4167
U+68B6.0	fR/S	 75.7
U+68B6.0	fTotalStrokes	 11
U+68B6.0	fCangjie	DSHU
U+68B6.0	fFourCorner	47910
U+68B6.0	fCantonese	mei5
U+68B6.0	fMandarin	wei3
U+68B6.0	fEnglish	oar; shaft
U+68B7.0	fUTF8	梷
U+68B7.0	fCNS11643	3-3652
U+68B7.0	fGB	E-2E27
U+68B7.0	fR/S	  75.7
U+68B7.0	fTotalStrokes	 11
U+68B7.0	fCangjie	YED
U+68B7.0	fFourCorner	27900
U+68B7.0	fCantonese	zing6
U+68B7.0	fMandarin	jing4
U+68B8.0	fUTF8	梸
U+68B8.0	fCNS11643	3-365E
U+68B8.0	fGB	5-437E
U+68B8.0	fR/S	  75.7
U+68B8.0	fTotalStrokes	 11
U+68B8.0	fCangjie	DHDN
U+68B8.0	fFourCorner	42900
U+68B8.0	fHakka	  li2
U+68B8.0	fCantonese	lei4; lei2 (also)
U+68B8.0	fMandarin	li2
U+68B8.0	fEnglish	same as &#x68A8;
U+68B9.0	fUTF8	梹
U+68B9.0	fCNS11643	3-365F
U+68B9.0	fGB	5-4421
U+68B9.0	fR/S	 75.7
U+68B9.0	fTotalStrokes	 11
U+68B9.0	fCangjie	DOMC
U+68B9.0	fFourCorner	42980
U+68B9.0	fHakka	 bin1
U+68B9.0	fCantonese	ban1
U+68B9.0	fMandarin	bing1; bin1
U+68B9.0	fEnglish	same as &#x6AB3;
U+68BA.0	fUTF8	梺
U+68BA.0	fGB	E-2E28
U+68BA.0	fR/S	 75.7
U+68BA.0	fTotalStrokes	 11
U+68BA.0	fCangjie	DDMY
U+68BA.0	fFourCorner	4423
U+68BA.0	fCantonese	haa6
U+68BB.0	fUTF8	梻
U+68BB.0	fGB	E-2E29
U+68BB.0	fR/S	 75.7
U+68BB.0	fTotalStrokes	 11
U+68BB.0	fCangjie	DOLN
U+68BB.0	fCantonese	fat6
U+68BB.0	fEnglish	a tree whose branches are placed on Buddhist graves
U+68BC.0	fUTF8	梼
U+68BC.0	fGB	8-7B6D
U+68BC.0	fR/S	 75.7
U+68BC.0	fTotalStrokes	 11
U+68BC.0	fCangjie	DQKI
U+68BC.0	fCantonese	tou4; to4
U+68BC.0	fMandarin	tao2; dao3
U+68BC.0	fEnglish	 block of wood blockhead stupid
U+68BD.0	fUTF8	梽
U+68BD.0	fCNS11643	F-3557
U+68BD.0	fGB	3-4147
U+68BD.0	fR/S	 75.7
U+68BD.0	fTotalStrokes	 11
U+68BD.0	fCangjie	DGP
U+68BD.0	fFourCorner	4493
U+68BD.0	fCantonese	zi3
U+68BD.0	fMandarin	zhi4
U+68BE.0	fUTF8	梾
U+68BE.0	fGB	7-2124
U+68BE.0	fR/S	 75.7
U+68BE.0	fTotalStrokes	 11
U+68BE.0	fCangjie	DDT
U+68BE.0	fCantonese	loi4
U+68BE.0	fMandarin	lai2
U+68BF.0	fUTF8	梿
U+68BF.0	fGB	8-2D6F
U+68BF.0	fR/S	 75.7
U+68BF.0	fTotalStrokes	 11
U+68BF.0	fCangjie	DYKQ
U+68BF.0	fCantonese	lin4; lin2 (also)
U+68BF.0	fMandarin	lian2
U+68C0.0	fUTF8	检
U+68C0.0	fGB	0-3C6C
U+68C0.0	fR/S	 75.7
U+68C0.0	fTotalStrokes	 11
U+68C0.0	fCangjie	DOMM
U+68C0.0	fHakka	 giam3
U+68C0.0	fCantonese	gim2
U+68C0.0	fMandarin	jian3
U+68C0.0	fEnglish	simplified form of &#x6AA2;
U+68C1.0	fUTF8	棁
U+68C1.0	fCNS11643	3-3656
U+68C1.0	fGB	3-4163
U+68C1.0	fR/S	 75.7
U+68C1.0	fTotalStrokes	 11
U+68C1.0	fCangjie	DCRU
U+68C1.0	fFourCorner	48910
U+68C1.0	fCantonese	zyut3; tyut3
U+68C1.0	fMandarin	zhuo2; tuo1; zhuo1
U+68C1.0	fEnglish	 joist cane
U+68C2.0	fUTF8	棂
U+68C2.0	fGB	0-6879
U+68C2.0	fR/S	 75.7
U+68C2.0	fTotalStrokes	 11
U+68C2.0	fCangjie	DSMF
U+68C2.0	fHakka	 lin2
U+68C2.0	fCantonese	ling4
U+68C2.0	fMandarin	ling2
U+68C2.0	fEnglish	simplified form of &#x6B1E;
U+68C3.0	fUTF8	棃
U+68C3.0	fCNS11643	3-3C45
U+68C3.0	fGB	E-2E2A
U+68C3.0	fR/S	  75.8
U+68C3.0	fTotalStrokes	 12
U+68C3.0	fCangjie	HHD
U+68C3.0	fFourCorner	27900
U+68C3.0	fHakka	 li2; li3
U+68C3.0	fCantonese	lei4
U+68C3.0	fMandarin	li2
U+68C3.0	fEnglish	[1] pear [2] Chinese opera
U+68C4.0	fUTF8	棄
U+68C4.0	fCNS11643	1-5B39
U+68C4.0	fBig5	B1F3
U+68C4.0	fGB	1-467A
U+68C4.0	fR/S	 75.8
U+68C4.0	fTotalStrokes	 12
U+68C4.0	fCangjie	YITD
U+68C4.0	fFourCorner	00904
U+68C4.0	fHakka	  hi5;  ki5
U+68C4.0	fCantonese	hei3
U+68C4.0	fMandarin	qi4
U+68C4.0	fEnglish	[1] reject; abandon; desert; forsake [2] discard; cast aside [3] forget [4] throw away one's own life
U+68C5.0	fUTF8	棅
U+68C5.0	fCNS11643	3-3C41
U+68C5.0	fGB	5-443E
U+68C5.0	fR/S	 75.8
U+68C5.0	fTotalStrokes	 12
U+68C5.0	fCangjie	DHDL
U+68C5.0	fFourCorner	42990
U+68C5.0	fHakka	  biang5
U+68C5.0	fCantonese	bing3; beng3 (also)
U+68C5.0	fMandarin	bing4; bing3
U+68C5.0	fEnglish	 handle; lever; knob authority
U+68C6.0	fUTF8	棆
U+68C6.0	fCNS11643	2-3C66
U+68C6.0	fBig5	D9C3
U+68C6.0	fGB	3-4060
U+68C6.0	fR/S	 75.8
U+68C6.0	fTotalStrokes	 12
U+68C6.0	fCangjie	DOMB
U+68C6.0	fFourCorner	48927
U+68C6.0	fCantonese	leon4
U+68C6.0	fMandarin	lun2; zhun1
U+68C6.0	fEnglish	camphor tree
U+68C7.0	fUTF8	棇
U+68C7.0	fCNS11643	2-3C7C
U+68C7.0	fBig5	D9D9
U+68C7.0	fGB	E-2E2B
U+68C7.0	fR/S	 75.8
U+68C7.0	fTotalStrokes	 12
U+68C7.0	fCangjie	DCIP
U+68C7.0	fFourCorner	48933
U+68C7.0	fHakka	  cung1
U+68C7.0	fCantonese	sung2; cung1
U+68C7.0	fMandarin	cong1; song1; cong4; song3
U+68C7.0	fEnglish	[1] the pricky ash [2] Aralia spinosa
U+68C8.0	fUTF8	棈
U+68C8.0	fCNS11643	2-3C71
U+68C8.0	fBig5	D9CE
U+68C8.0	fGB	5-4432
U+68C8.0	fR/S	 75.8
U+68C8.0	fTotalStrokes	 12
U+68C8.0	fCangjie	DQMB
U+68C8.0	fFourCorner	45927
U+68C8.0	fCantonese	sin3
U+68C8.0	fMandarin	qian4
U+68C9.0	fUTF8	棉
U+68C9.0	fCNS11643	1-5F7B
U+68C9.0	fBig5	B4D6
U+68C9.0	fGB	0-435E
U+68C9.0	fR/S	 75.8
U+68C9.0	fTotalStrokes	 12
U+68C9.0	fCangjie	DHAB
U+68C9.0	fFourCorner	46927
U+68C9.0	fHakka	 mien2; men2
U+68C9.0	fCantonese	min4; min2 (also)
U+68C9.0	fMandarin	mian2
U+68C9.0	fEnglish	 cotton
U+68CA.0	fUTF8	棊
U+68CA.0	fCNS11643	3-3C38
U+68CA.0	fGB	E-2E2C
U+68CA.0	fR/S	  75.8
U+68CA.0	fTotalStrokes	 12
U+68CA.0	fCangjie	TCD
U+68CA.0	fFourCorner	44900
U+68CA.0	fHakka	 ki2
U+68CA.0	fCantonese	kei4
U+68CA.0	fMandarin	qi2
U+68CA.0	fEnglish	chess; any game similar to chess
U+68CB.0	fUTF8	棋
U+68CB.0	fCNS11643	1-5F76
U+68CB.0	fBig5	B4D1
U+68CB.0	fGB	0-4665
U+68CB.0	fR/S	 75.8
U+68CB.0	fTotalStrokes	 12
U+68CB.0	fCangjie	DTMC
U+68CB.0	fFourCorner	44981
U+68CB.0	fHakka	  ki2; ki3
U+68CB.0	fCantonese	kei4; kei2 (also)
U+68CB.0	fMandarin	qi2; ji1
U+68CB.0	fEnglish	[1] chess; any game similar to chess [2] any piece used in the game of chess
U+68CC.0	fUTF8	棌
U+68CC.0	fCNS11643	2-3C60
U+68CC.0	fBig5	D9BD
U+68CC.0	fGB	5-4440
U+68CC.0	fR/S	 75.8
U+68CC.0	fTotalStrokes	 12
U+68CC.0	fCangjie	DBD
U+68CC.0	fFourCorner	52994
U+68CC.0	fHakka	  cai3
U+68CC.0	fCantonese	coi3; coi4
U+68CC.0	fMandarin	cai3
U+68CC.0	fEnglish	a species of oak
U+68CD.0	fUTF8	棍
U+68CD.0	fCNS11643	1-5F77
U+68CD.0	fBig5	B4D2
U+68CD.0	fGB	0-3977
U+68CD.0	fR/S	 75.8
U+68CD.0	fTotalStrokes	 12
U+68CD.0	fCangjie	DAPP
U+68CD.0	fFourCorner	46911
U+68CD.0	fHakka	  gun5; gwun5
U+68CD.0	fCantonese	gwan3
U+68CD.0	fMandarin	gun4; hun4; hong3; ao1; gun3
U+68CD.0	fEnglish	[1] club; stick; cudgel [2] scoundrel; rascal; villain; ruffian
U+68CE.0	fUTF8	棎
U+68CE.0	fCNS11643	2-3C70
U+68CE.0	fBig5	D9CD
U+68CE.0	fGB	5-4448
U+68CE.0	fR/S	 75.8
U+68CE.0	fTotalStrokes	 12
U+68CE.0	fCangjie	DBCD
U+68CE.0	fFourCorner	47994
U+68CE.0	fHakka	 sham2; sam2
U+68CE.0	fCantonese	sim4; sam4
U+68CE.0	fMandarin	chan2
U+68CE.0	fEnglish	a tree bearing a small pear-shaped fruit
U+68CF.0	fUTF8	棏
U+68CF.0	fCNS11643	3-3C3E
U+68CF.0	fGB	E-2E2D
U+68CF.0	fR/S	 75.8
U+68CF.0	fTotalStrokes	 12
U+68CF.0	fCangjie	DAMI
U+68CF.0	fFourCorner	46940
U+68CF.0	fCantonese	dak6
U+68CF.0	fMandarin	te4; de2; zhe2
U+68CF.0	fEnglish	corrupted form of &#x68CF;
U+68D0.0	fUTF8	棐
U+68D0.0	fCNS11643	2-3C69
U+68D0.0	fBig5	D9C6
U+68D0.0	fGB	3-745A
U+68D0.0	fR/S	 75.8
U+68D0.0	fTotalStrokes	 12
U+68D0.0	fCangjie	LYD
U+68D0.0	fFourCorner	11904
U+68D0.0	fHakka	 fui1; fi1; fui3
U+68D0.0	fCantonese	fei2
U+68D0.0	fMandarin	fei3; fei2
U+68D0.0	fEnglish	[1] a species of yew found in North China [2] bamboo products [3] assist
U+68D1.0	fUTF8	棑
U+68D1.0	fCNS11643	2-3C76
U+68D1.0	fBig5	D9D3
U+68D1.0	fGB	3-4172
U+68D1.0	fR/S	 75.8
U+68D1.0	fTotalStrokes	 12
U+68D1.0	fCangjie	DLMY
U+68D1.0	fFourCorner	41911
U+68D1.0	fHakka	 pai2
U+68D1.0	fCantonese	paai4
U+68D1.0	fMandarin	pai2; bei4; bai4; pei4
U+68D1.0	fEnglish	raft
U+68D2.0	fUTF8	棒
U+68D2.0	fCNS11643	1-5F73
U+68D2.0	fBig5	B4CE
U+68D2.0	fGB	0-3074
U+68D2.0	fR/S	 75.8
U+68D2.0	fTotalStrokes	 12
U+68D2.0	fCangjie	DQKQ
U+68D2.0	fFourCorner	45953
U+68D2.0	fHakka	  pong5; pong3; pen3
U+68D2.0	fCantonese	paang5
U+68D2.0	fMandarin	bang4
U+68D2.0	fEnglish	[1] stick; club; truncheon [2] hit with a club [3] good; strong; wonderful
U+68D3.0	fUTF8	棓
U+68D3.0	fCNS11643	2-3C4E
U+68D3.0	fBig5	D9AB
U+68D3.0	fGB	3-4222
U+68D3.0	fR/S	 75.8
U+68D3.0	fTotalStrokes	 12
U+68D3.0	fCangjie	DYTR
U+68D3.0	fFourCorner	40961
U+68D3.0	fHakka	 poi5; pui5; pi5
U+68D3.0	fCantonese	paang5; bui6; pui5 (also); pui4; pau2; bui4; pang5
U+68D3.0	fMandarin	pou3; bang4; pou2; bei4
U+68D3.0	fEnglish	hit; strike
U+68D4.0	fUTF8	棔
U+68D4.0	fCNS11643	2-3C78
U+68D4.0	fBig5	D9D5
U+68D4.0	fGB	3-417B
U+68D4.0	fR/S	 75.8
U+68D4.0	fTotalStrokes	 12
U+68D4.0	fCangjie	DHPA
U+68D4.0	fFourCorner	42964
U+68D4.0	fHakka	 fun1
U+68D4.0	fCantonese	fan1
U+68D4.0	fMandarin	hun1
U+68D5.0	fUTF8	棕
U+68D5.0	fCNS11643	1-5F69
U+68D5.0	fBig5	B4C4
U+68D5.0	fGB	0-5758
U+68D5.0	fR/S	 75.8
U+68D5.0	fTotalStrokes	 12
U+68D5.0	fCangjie	DJMF
U+68D5.0	fFourCorner	43991
U+68D5.0	fHakka	  zung1
U+68D5.0	fCantonese	zung1
U+68D5.0	fMandarin	zong1; huan1
U+68D5.0	fEnglish	the palm tree
U+68D6.0	fUTF8	棖
U+68D6.0	fCNS11643	2-3C56
U+68D6.0	fBig5	D9B3
U+68D6.0	fGB	1-6847
U+68D6.0	fR/S	 75.8
U+68D6.0	fTotalStrokes	 12
U+68D6.0	fCangjie	DSMV
U+68D6.0	fFourCorner	51932
U+68D6.0	fHakka	  cen2
U+68D6.0	fCantonese	caang4; caam4; caam5; caan5
U+68D6.0	fMandarin	cheng2
U+68D6.0	fEnglish	[1] door posts [2] stop [3] touch [4] a Chinese family name
U+68D7.0	fUTF8	棗
U+68D7.0	fCNS11643	1-5F6C
U+68D7.0	fBig5	B4C7
U+68D7.0	fGB	1-5466
U+68D7.0	fR/S	 75.8
U+68D7.0	fTotalStrokes	 12
U+68D7.0	fCangjie	DBDB
U+68D7.0	fFourCorner	50902
U+68D7.0	fHakka	  zau3;  zo3
U+68D7.0	fCantonese	zou2
U+68D7.0	fMandarin	zao3
U+68D7.0	fEnglish	[1] date tree; dates; jujube (Zizyphus vulgaris) [2] a Chinese family name
U+68D8.0	fUTF8	棘
U+68D8.0	fCNS11643	1-5F6B
U+68D8.0	fBig5	B4C6
U+68D8.0	fGB	0-3C2C
U+68D8.0	fR/S	 75.8
U+68D8.0	fTotalStrokes	 12
U+68D8.0	fCangjie	DBDB
U+68D8.0	fFourCorner	55992
U+68D8.0	fHakka	 git7
U+68D8.0	fCantonese	gik1; kik1
U+68D8.0	fMandarin	ji2; ji4
U+68D8.0	fEnglish	[1] buckthorns; thorny brambles [2] urgent [3] a Chinese family name
U+68D9.0	fUTF8	棙
U+68D9.0	fCNS11643	4-3832
U+68D9.0	fGB	3-422B
U+68D9.0	fR/S	  75.8
U+68D9.0	fTotalStrokes	 12
U+68D9.0	fCangjie	DHSK
U+68D9.0	fFourCorner	43930
U+68D9.0	fHakka	 lui5; lui6; li6; li5
U+68D9.0	fCantonese	lai6; lit6
U+68D9.0	fMandarin	li4
U+68D9.0	fEnglish	 strike, as the strings of a guitar
U+68DA.0	fUTF8	棚
U+68DA.0	fCNS11643	1-5F7C
U+68DA.0	fBig5	B4D7
U+68DA.0	fGB	0-456F
U+68DA.0	fR/S	 75.8
U+68DA.0	fTotalStrokes	 12
U+68DA.0	fCangjie	DBB
U+68DA.0	fFourCorner	47920
U+68DA.0	fHakka	  pang2; pang3
U+68DA.0	fCantonese	paang4; paang2 (also)
U+68DA.0	fMandarin	peng2
U+68DA.0	fEnglish	 tent; awning; mat awning; shed
U+68DA.1	fUTF8	棚
U+68DA.1	fCNS11643	1-5F7C
U+68DA.1	fBig5	B4D7
U+68DA.1	fGB	0-456F
U+68DA.1	fR/S	 75.8
U+68DA.1	fTotalStrokes	 12
U+68DA.1	fCangjie	DBB
U+68DA.1	fFourCorner	47920
U+68DA.1	fMandarin	peng5
U+68DB.0	fUTF8	棛
U+68DB.0	fCNS11643	3-3653
U+68DB.0	fGB	5-4446
U+68DB.0	fR/S	 75.8; 75.7
U+68DB.0	fTotalStrokes	 12; 11
U+68DB.0	fCangjie	DYIB
U+68DB.0	fFourCorner	40920
U+68DB.0	fCantonese	juk6
U+68DB.0	fMandarin	yu4
U+68DC.0	fUTF8	棜
U+68DC.0	fCNS11643	2-3C50
U+68DC.0	fBig5	D9AD
U+68DC.0	fGB	3-4224
U+68DC.0	fR/S	 75.8
U+68DC.0	fTotalStrokes	 12
U+68DC.0	fCangjie	DYSY
U+68DC.0	fFourCorner	48933
U+68DC.0	fHakka	 ri3; ji3; i5; ji5
U+68DC.0	fCantonese	jyu3; jyu6
U+68DC.0	fMandarin	yu4
U+68DC.0	fEnglish	 branch; fork tray
U+68DD.0	fUTF8	棝
U+68DD.0	fCNS11643	2-3C72
U+68DD.0	fBig5	D9CF
U+68DD.0	fGB	5-443B
U+68DD.0	fR/S	 75.8
U+68DD.0	fTotalStrokes	 12
U+68DD.0	fCangjie	DWJR
U+68DD.0	fFourCorner	46900
U+68DD.0	fCantonese	gu3; gwu3 (also)
U+68DD.0	fMandarin	gu4
U+68DE.0	fUTF8	棞
U+68DE.0	fCNS11643	2-3C73
U+68DE.0	fBig5	D9D0
U+68DE.0	fGB	5-443C
U+68DE.0	fR/S	 75.8
U+68DE.0	fTotalStrokes	 12
U+68DE.0	fCangjie	DWHD
U+68DE.0	fFourCorner	46900
U+68DE.0	fHakka	 kun5
U+68DE.0	fCantonese	gwan6; wan4
U+68DE.0	fMandarin	hun2
U+68DF.0	fUTF8	棟
U+68DF.0	fCNS11643	1-5F6E
U+68DF.0	fBig5	B4C9
U+68DF.0	fGB	1-3630
U+68DF.0	fR/S	 75.8
U+68DF.0	fTotalStrokes	 12
U+68DF.0	fCangjie	DDW
U+68DF.0	fFourCorner	45996
U+68DF.0	fHakka	  dung5; dung1; dung3
U+68DF.0	fCantonese	dung3 (also); dung6
U+68DF.0	fMandarin	dong4
U+68DF.0	fEnglish	the main beams supporting a house; main beam of a house
U+68E0.0	fUTF8	棠
U+68E0.0	fCNS11643	1-5F6A
U+68E0.0	fBig5	B4C5
U+68E0.0	fGB	0-4C44
U+68E0.0	fR/S	 75.8
U+68E0.0	fTotalStrokes	 12
U+68E0.0	fCangjie	FBRD
U+68E0.0	fFourCorner	90904
U+68E0.0	fHakka	  tong2; tong3
U+68E0.0	fCantonese	tong4; tong2 (also)
U+68E0.0	fMandarin	tang2
U+68E0.0	fEnglish	 sweet pear tree; wild plum
U+68E1.0	fUTF8	棡
U+68E1.0	fCNS11643	2-3C5E
U+68E1.0	fBig5	D9BB
U+68E1.0	fGB	3-405E
U+68E1.0	fR/S	 75.8
U+68E1.0	fTotalStrokes	 12
U+68E1.0	fCangjie	DBTU
U+68E1.0	fFourCorner	47920
U+68E1.0	fHakka	 wong3; mong1; vong3
U+68E1.0	fCantonese	gong1; gong3 (also)
U+68E1.0	fMandarin	gang1
U+68E1.0	fEnglish	the felloe of a wheel
U+68E2.0	fUTF8	棢
U+68E2.0	fCNS11643	3-3C3D
U+68E2.0	fGB	E-2E2E
U+68E2.0	fR/S	  75.8
U+68E2.0	fTotalStrokes	 12
U+68E2.0	fCangjie	DBTV
U+68E2.0	fFourCorner	47920
U+68E2.0	fCantonese	mong5; mong1
U+68E2.0	fMandarin	wang3
U+68E2.0	fEnglish	the felloe or rim of a wheel
U+68E3.0	fUTF8	棣
U+68E3.0	fCNS11643	1-5F75
U+68E3.0	fBig5	B4D0
U+68E3.0	fGB	0-6926
U+68E3.0	fR/S	 75.8
U+68E3.0	fTotalStrokes	 12
U+68E3.0	fCangjie	DLE
U+68E3.0	fFourCorner	45932
U+68E3.0	fHakka	 ti5; ti6
U+68E3.0	fCantonese	dai6
U+68E3.0	fMandarin	di4; dai4; ti4; dui4; li4
U+68E3.0	fEnglish	[1] a mountain tree, as a cherry, etc. [2] same as $, kid brother [3] a Chinese family name
U+68E4.0	fUTF8	棤
U+68E4.0	fCNS11643	2-3C59
U+68E4.0	fBig5	D9B6
U+68E4.0	fGB	E-2E2F
U+68E4.0	fR/S	 75.8
U+68E4.0	fTotalStrokes	 12
U+68E4.0	fCangjie	DTA
U+68E4.0	fFourCorner	44961
U+68E4.0	fHakka	 cok7;  sit7
U+68E4.0	fCantonese	co3; sik1; coek3
U+68E4.0	fMandarin	xi2
U+68E4.0	fEnglish	the rough bark of a tree
U+68E5.0	fUTF8	棥
U+68E5.0	fCNS11643	3-3C32
U+68E5.0	fGB	E-2E30
U+68E5.0	fR/S	  75.8
U+68E5.0	fTotalStrokes	 12
U+68E5.0	fCangjie	DKKD
U+68E5.0	fFourCorner	44990
U+68E5.0	fCantonese	faan4
U+68E5.0	fMandarin	fan2
U+68E5.0	fEnglish	 railing a fence; an enclosed place
U+68E6.0	fUTF8	棦
U+68E6.0	fCNS11643	2-3C74
U+68E6.0	fBig5	D9D1
U+68E6.0	fGB	5-4370
U+68E6.0	fR/S	 75.8
U+68E6.0	fTotalStrokes	 12
U+68E6.0	fCangjie	DBSD
U+68E6.0	fFourCorner	52057
U+68E6.0	fHakka	 zang1;  zen1
U+68E6.0	fCantonese	caang1; zaang1; zang1
U+68E6.0	fMandarin	cheng1
U+68E6.0	fEnglish	 thorn
U+68E7.0	fUTF8	棧
U+68E7.0	fCNS11643	1-5F71
U+68E7.0	fBig5	B4CC
U+68E7.0	fGB	1-553B
U+68E7.0	fR/S	 75.8
U+68E7.0	fTotalStrokes	 12
U+68E7.0	fCangjie	DII
U+68E7.0	fFourCorner	43953
U+68E7.0	fHakka	 zan5;  zan3; can3; {can5}; can5; can6
U+68E7.0	fCantonese	zaan6; zaan2 (also); zaan3
U+68E7.0	fMandarin	zhan4; chen2
U+68E7.0	fEnglish	[1] warehouse; storehouse [2] tavern; inn [3] a road made along a cliff [4] pen; stable [5] a Chinese family name
U+68E8.0	fUTF8	棨
U+68E8.0	fCNS11643	2-3C6C
U+68E8.0	fBig5	D9C9
U+68E8.0	fGB	3-422C
U+68E8.0	fR/S	 75.8
U+68E8.0	fTotalStrokes	 12
U+68E8.0	fCangjie	HKD
U+68E8.0	fFourCorner	78904
U+68E8.0	fHakka	  ki3
U+68E8.0	fCantonese	kai2
U+68E8.0	fMandarin	qi3
U+68E8.0	fEnglish	 tally or wooden pass used by a messenger as his credentials in passing through a guarded gate, checkpoint, etc.
U+68E9.0	fUTF8	棩
U+68E9.0	fCNS11643	2-3C79
U+68E9.0	fBig5	D9D6
U+68E9.0	fGB	5-4461
U+68E9.0	fR/S	 75.8
U+68E9.0	fTotalStrokes	 12
U+68E9.0	fCangjie	DLXL
U+68E9.0	fFourCorner	42900
U+68E9.0	fCantonese	jyun1
U+68E9.0	fMandarin	yuan1; yung4
U+68EA.0	fUTF8	棪
U+68EA.0	fCNS11643	2-3C53
U+68EA.0	fBig5	D9B0
U+68EA.0	fGB	3-4228
U+68EA.0	fR/S	 75.8
U+68EA.0	fTotalStrokes	 12
U+68EA.0	fCangjie	DFF
U+68EA.0	fFourCorner	49989
U+68EA.0	fCantonese	jim5
U+68EA.0	fMandarin	yan3; yan2
U+68EA.0	fEnglish	tree
U+68EB.0	fUTF8	棫
U+68EB.0	fCNS11643	2-3C58
U+68EB.0	fBig5	D9B5
U+68EB.0	fGB	3-416F
U+68EB.0	fR/S	 75.8
U+68EB.0	fTotalStrokes	 12
U+68EB.0	fCangjie	DIRM
U+68EB.0	fFourCorner	43950
U+68EB.0	fHakka	 wet7; vet7
U+68EB.0	fCantonese	wik6
U+68EB.0	fMandarin	yu4
U+68EB.0	fEnglish	thorny shrub with yellow flowers and dark fruit
U+68EC.0	fUTF8	棬
U+68EC.0	fCNS11643	2-3C52
U+68EC.0	fBig5	D9AF
U+68EC.0	fGB	3-4225
U+68EC.0	fR/S	 75.8
U+68EC.0	fTotalStrokes	 12
U+68EC.0	fCangjie	DFQU
U+68EC.0	fFourCorner	49927
U+68EC.0	fHakka	 gien3; kien1; ken1; kian1; kien5
U+68EC.0	fCantonese	hyun1; gyun3
U+68EC.0	fMandarin	quan1; quan4
U+68EC.0	fEnglish	 wooden bowl; crooked wood
U+68ED.0	fUTF8	棭
U+68ED.0	fCNS11643	3-3C2A
U+68ED.0	fGB	E-2E31
U+68ED.0	fR/S	  75.8
U+68ED.0	fTotalStrokes	 12
U+68ED.0	fCangjie	DYOK
U+68ED.0	fFourCorner	40940
U+68ED.0	fCantonese	jik6
U+68ED.0	fMandarin	yi4
U+68EE.0	fUTF8	森
U+68EE.0	fCNS11643	1-5F70
U+68EE.0	fBig5	B4CB
U+68EE.0	fGB	0-492D
U+68EE.0	fR/S	 75.8
U+68EE.0	fTotalStrokes	 12
U+68EE.0	fCangjie	DDD
U+68EE.0	fFourCorner	40994
U+68EE.0	fHakka	  sem1; sen1; siem1
U+68EE.0	fCantonese	sam1
U+68EE.0	fMandarin	sen1
U+68EE.0	fEnglish	[1] forest; luxuriant vegetation; luxuriant growth of trees [2] dark and obscure; severe [3] serene; majestic
U+68EF.0	fUTF8	棯
U+68EF.0	fCNS11643	2-3C65
U+68EF.0	fBig5	D9C2
U+68EF.0	fGB	3-4179
U+68EF.0	fR/S	 75.8
U+68EF.0	fTotalStrokes	 12
U+68EF.0	fCangjie	DOIP
U+68EF.0	fFourCorner	48932
U+68EF.0	fHakka	  ngim3
U+68EF.0	fCantonese	nim1; nim6; lim1 (also); lim6 (also); nin2; nim3
U+68EF.0	fMandarin	ren3; nian4; shen3
U+68EF.0	fEnglish	jujube tree
U+68F0.0	fUTF8	棰
U+68F0.0	fCNS11643	2-4364
U+68F0.0	fBig5	DDDE
U+68F0.0	fGB	0-6922
U+68F0.0	fR/S	 75.8
U+68F0.0	fTotalStrokes	 12
U+68F0.0	fCangjie	DHJM
U+68F0.0	fFourCorner	42914
U+68F0.0	fHakka	 ch'ui3; cui3
U+68F0.0	fCantonese	ceoi4; zeoi1
U+68F0.0	fMandarin	chui2; duo3
U+68F0.0	fEnglish	 cudgel; beat
U+68F1.0	fUTF8	棱
U+68F1.0	fCNS11643	2-3C54
U+68F1.0	fBig5	D9B1
U+68F1.0	fGB	0-4062
U+68F1.0	fR/S	 75.8
U+68F1.0	fTotalStrokes	 12
U+68F1.0	fCangjie	DGCE
U+68F1.0	fFourCorner	54047
U+68F1.0	fHakka	 lin2
U+68F1.0	fCantonese	ling4
U+68F1.0	fMandarin	leng2; ling2; leng1; cheng1; leng4
U+68F1.0	fEnglish	 squared timber angle; edge
U+68F2.0	fUTF8	棲
U+68F2.0	fCNS11643	1-5F74
U+68F2.0	fBig5	B4CF
U+68F2.0	fGB	1-465C
U+68F2.0	fR/S	 75.8
U+68F2.0	fTotalStrokes	 12
U+68F2.0	fCangjie	DJLV
U+68F2.0	fFourCorner	45944
U+68F2.0	fHakka	 ci1; zi1; zi5
U+68F2.0	fCantonese	cai1; sai1
U+68F2.0	fMandarin	qi1; xi1
U+68F2.0	fEnglish	[1] perch; rest; stay; settle [2] the place one stays [3] bed
U+68F3.0	fUTF8	棳
U+68F3.0	fCNS11643	2-3C5D
U+68F3.0	fBig5	D9BA
U+68F3.0	fGB	3-422F
U+68F3.0	fR/S	 75.8
U+68F3.0	fTotalStrokes	 12
U+68F3.0	fCangjie	DEEE
U+68F3.0	fFourCorner	47947
U+68F3.0	fHakka	 chot7; zot7
U+68F3.0	fCantonese	zyut3
U+68F3.0	fMandarin	zhuo2
U+68F3.0	fEnglish	small pillars that support the roof; those which rest on the main bean
U+68F4.0	fUTF8	棴
U+68F4.0	fCNS11643	2-3C75
U+68F4.0	fBig5	D9D2
U+68F4.0	fGB	5-4442
U+68F4.0	fR/S	 75.8
U+68F4.0	fTotalStrokes	 12
U+68F4.0	fCangjie	DBSE
U+68F4.0	fFourCorner	47947
U+68F4.0	fCantonese	fuk6; suk1
U+68F4.0	fMandarin	fu2; su4
U+68F5.0	fUTF8	棵
U+68F5.0	fCNS11643	1-5F6F
U+68F5.0	fBig5	B4CA
U+68F5.0	fGB	0-3F43
U+68F5.0	fR/S	 75.8
U+68F5.0	fTotalStrokes	 12
U+68F5.0	fCangjie	DWD
U+68F5.0	fFourCorner	46994
U+68F5.0	fHakka	 po1; ko3
U+68F5.0	fCantonese	fo2; fo1 (also); po1 (also)
U+68F5.0	fMandarin	ke1; kuan3; ke3
U+68F5.0	fEnglish	numerical classifier for trees
U+68F6.0	fUTF8	棶
U+68F6.0	fCNS11643	2-3C5A
U+68F6.0	fBig5	D9B7
U+68F6.0	fGB	3-4150
U+68F6.0	fR/S	 75.8
U+68F6.0	fTotalStrokes	 12
U+68F6.0	fCangjie	DDOO
U+68F6.0	fFourCorner	44998
U+68F6.0	fHakka	  loi2
U+68F6.0	fCantonese	loi4
U+68F6.0	fMandarin	lai2
U+68F6.0	fEnglish	the name of a tree that grows in Kiangsi
U+68F7.0	fUTF8	棷
U+68F7.0	fCNS11643	2-3C57
U+68F7.0	fBig5	D9B4
U+68F7.0	fGB	3-416B
U+68F7.0	fR/S	 75.8
U+68F7.0	fTotalStrokes	 12
U+68F7.0	fCangjie	DSJE
U+68F7.0	fFourCorner	47940
U+68F7.0	fHakka	  zeu1
U+68F7.0	fCantonese	zau1; sau2
U+68F7.0	fMandarin	zou1; sou3; zhou3; zui4
U+68F7.0	fEnglish	[1] name of a tree [2] rattle
U+68F8.0	fUTF8	棸
U+68F8.0	fCNS11643	2-3C68
U+68F8.0	fBig5	D9C5
U+68F8.0	fGB	5-4434
U+68F8.0	fR/S	 75.8
U+68F8.0	fTotalStrokes	 12
U+68F8.0	fCangjie	SED
U+68F8.0	fFourCorner	17904
U+68F8.0	fHakka	  zeu1
U+68F8.0	fCantonese	zau1
U+68F8.0	fMandarin	zou1; chou2; zhou4
U+68F8.0	fEnglish	a Chinese family name
U+68F9.0	fUTF8	棹
U+68F9.0	fCNS11643	1-5F72
U+68F9.0	fBig5	B4CD
U+68F9.0	fGB	0-687E
U+68F9.0	fR/S	 75.8
U+68F9.0	fTotalStrokes	 12
U+68F9.0	fCangjie	DYAJ
U+68F9.0	fFourCorner	41946
U+68F9.0	fHakka	  zok7; cau5; cau6
U+68F9.0	fCantonese	zaau6; zoek3; coek3 (also)
U+68F9.0	fMandarin	zhuo1
U+68F9.0	fEnglish	 table
U+68F9.1	fUTF8	棹
U+68F9.1	fCNS11643	1-5F72
U+68F9.1	fBig5	B4CD
U+68F9.1	fGB	0-687E
U+68F9.1	fR/S	 75.8
U+68F9.1	fTotalStrokes	 12
U+68F9.1	fCangjie	DYAJ
U+68F9.1	fFourCorner	41946
U+68F9.1	fMandarin	zhao4; zhuo1}
U+68F9.1	fEnglish	[1] oar; scull [2] boat
U+68FA.0	fUTF8	棺
U+68FA.0	fCNS11643	1-5F68
U+68FA.0	fBig5	B4C3
U+68FA.0	fGB	0-3957
U+68FA.0	fR/S	 75.8
U+68FA.0	fTotalStrokes	 12
U+68FA.0	fCangjie	DJRR
U+68FA.0	fFourCorner	43977
U+68FA.0	fHakka	  gon1;  kwan1; gwon1
U+68FA.0	fCantonese	gun1; gwun1 (also); gun3
U+68FA.0	fMandarin	guan1; guan4
U+68FA.0	fEnglish	 coffin (usually made of wood)
U+68FB.0	fUTF8	棻
U+68FB.0	fCNS11643	1-5F7E
U+68FB.0	fBig5	B4D9
U+68FB.0	fGB	3-414A
U+68FB.0	fR/S	 75.8
U+68FB.0	fTotalStrokes	 12
U+68FB.0	fCangjie	TCSD
U+68FB.0	fFourCorner	44904
U+68FB.0	fHakka	 fun1
U+68FB.0	fCantonese	fan1
U+68FB.0	fMandarin	fen1; fen2
U+68FB.0	fEnglish	a kind of wood from which perfume is made; a kind of fragrant tree used to produce perfume by burning it
U+68FC.0	fUTF8	棼
U+68FC.0	fCNS11643	2-3C6B
U+68FC.0	fBig5	D9C8
U+68FC.0	fGB	0-687B
U+68FC.0	fR/S	 75.8
U+68FC.0	fTotalStrokes	 12
U+68FC.0	fCangjie	DDCSH
U+68FC.0	fFourCorner	44227
U+68FC.0	fHakka	  fun1
U+68FC.0	fCantonese	fan4; fan1
U+68FC.0	fMandarin	fen2; fen1; fen4
U+68FC.0	fEnglish	[1] beams in the roof of a house [2] confused; disordered; disarrayed [3] a kind of linen
U+68FD.0	fUTF8	棽
U+68FD.0	fCNS11643	2-3C6A
U+68FD.0	fBig5	D9C7
U+68FD.0	fGB	3-416E
U+68FD.0	fR/S	 75.8
U+68FD.0	fTotalStrokes	 12
U+68FD.0	fCangjie	DDOIN
U+68FD.0	fFourCorner	44207
U+68FD.0	fCantonese	cam1; cam4 (also); sam1
U+68FD.0	fMandarin	chen1; sen1; shen1
U+68FE.0	fUTF8	棾
U+68FE.0	fCNS11643	3-3C31
U+68FE.0	fGB	E-2E32
U+68FE.0	fR/S	  75.8
U+68FE.0	fTotalStrokes	 12
U+68FE.0	fCangjie	DDBHN
U+68FE.0	fFourCorner	44210
U+68FE.0	fCantonese	king4
U+68FE.0	fMandarin	qiong2
U+68FF.0	fUTF8	棿
U+68FF.0	fCNS11643	3-3C42
U+68FF.0	fGB	E-2E33
U+68FF.0	fR/S	  75.8
U+68FF.0	fTotalStrokes	 12
U+68FF.0	fCangjie	DHXU
U+68FF.0	fFourCorner	47910
U+68FF.0	fHakka	 ngi2
U+68FF.0	fCantonese	ngai4; ai4 (also)
U+68FF.0	fMandarin	nie4
U+6900.0	fUTF8	椀
U+6900.0	fCNS11643	3-3C39
U+6900.0	fGB	E-2E34
U+6900.0	fR/S	  75.8
U+6900.0	fTotalStrokes	 12
U+6900.0	fCangjie	DJNU
U+6900.0	fFourCorner	43910
U+6900.0	fHakka	 won3; wan3; von3
U+6900.0	fCantonese	wun2
U+6900.0	fMandarin	wan3
U+6900.0	fEnglish	 bowl cup dish
U+6901.0	fUTF8	椁
U+6901.0	fCNS11643	3-3C3B
U+6901.0	fGB	0-6924
U+6901.0	fR/S	  75.8
U+6901.0	fTotalStrokes	 12
U+6901.0	fCangjie	DYRD
U+6901.0	fFourCorner	40940
U+6901.0	fHakka	 gok7; gwok7
U+6901.0	fCantonese	gwok3; gok3 (also)
U+6901.0	fMandarin	guo3
U+6901.0	fEnglish	 outer coffin
U+6902.0	fUTF8	椂
U+6902.0	fCNS11643	3-3C34
U+6902.0	fGB	5-4449
U+6902.0	fR/S	  75.8
U+6902.0	fTotalStrokes	 12
U+6902.0	fCangjie	DVNE
U+6902.0	fFourCorner	47930
U+6902.0	fCantonese	luk6
U+6902.0	fMandarin	lu4
U+6903.0	fUTF8	椃
U+6903.0	fCNS11643	3-3C2F
U+6903.0	fGB	5-4438
U+6903.0	fR/S	  75.8
U+6903.0	fTotalStrokes	 12
U+6903.0	fCangjie	DYPU
U+6903.0	fFourCorner	41910
U+6903.0	fMandarin	hao2
U+6903.0	fEnglish	same as &#x2362B;
U+6904.0	fUTF8	椄
U+6904.0	fCNS11643	2-3C4F
U+6904.0	fBig5	D9AC
U+6904.0	fGB	3-4223
U+6904.0	fR/S	 75.8
U+6904.0	fTotalStrokes	 12
U+6904.0	fCangjie	DYTV
U+6904.0	fFourCorner	40944
U+6904.0	fHakka	 ziap7
U+6904.0	fCantonese	zip3
U+6904.0	fMandarin	jie1; jie2
U+6904.0	fEnglish	 graft
U+6905.0	fUTF8	椅
U+6905.0	fCNS11643	1-5F6D
U+6905.0	fBig5	B4C8
U+6905.0	fGB	0-524E
U+6905.0	fR/S	 75.8
U+6905.0	fTotalStrokes	 12
U+6905.0	fCangjie	DKMR
U+6905.0	fFourCorner	44921
U+6905.0	fHakka	  ji3; ri3; i3
U+6905.0	fCantonese	ji2; ji1
U+6905.0	fMandarin	yi3; yi1; yi4
U+6905.0	fEnglish	[1] chair; seat; bench [2] Idesia polycarpa
U+6906.0	fUTF8	椆
U+6906.0	fCNS11643	2-3C77
U+6906.0	fBig5	D9D4
U+6906.0	fGB	3-417A
U+6906.0	fR/S	 75.8
U+6906.0	fTotalStrokes	 12
U+6906.0	fCangjie	DBGR
U+6906.0	fFourCorner	47920
U+6906.0	fHakka	 chiu2; cu2
U+6906.0	fCantonese	cau4; zau3; diu1
U+6906.0	fMandarin	chou2; jiu4; mian3; zhou4
U+6906.0	fEnglish	Quercus glauca
U+6907.0	fUTF8	椇
U+6907.0	fCNS11643	2-3C5F
U+6907.0	fBig5	D9BC
U+6907.0	fGB	3-4173
U+6907.0	fR/S	 75.8
U+6907.0	fTotalStrokes	 12
U+6907.0	fCangjie	DBMC
U+6907.0	fFourCorner	46981
U+6907.0	fHakka	 gi3; ki
U+6907.0	fCantonese	geoi2
U+6907.0	fMandarin	ju3
U+6907.0	fEnglish	the edible fruit of the Hovenia dulcis
U+6908.0	fUTF8	椈
U+6908.0	fCNS11643	2-3C61
U+6908.0	fBig5	D9BE
U+6908.0	fGB	3-417C
U+6908.0	fR/S	 75.8
U+6908.0	fTotalStrokes	 12
U+6908.0	fCangjie	DPFD
U+6908.0	fFourCorner	47920
U+6908.0	fCantonese	guk1
U+6908.0	fMandarin	ju2
U+6908.0	fEnglish	cedar
U+6909.0	fUTF8	椉
U+6909.0	fCNS11643	3-3C37
U+6909.0	fGB	E-2E35
U+6909.0	fR/S	 75.8
U+6909.0	fTotalStrokes	 12
U+6909.0	fCangjie	YNQD
U+6909.0	fFourCorner	00900
U+6909.0	fHakka	  shin5; sin2; sin5; shin1; shin2; sen1; sin1; sen2; sen5; sin3
U+6909.0	fCantonese	sing4 sing6
U+6909.0	fMandarin	cheng2 sheng4
U+6909.0	fEnglish	standard form of &#x4E58;
U+690A.0	fUTF8	椊
U+690A.0	fCNS11643	2-3C6E
U+690A.0	fBig5	D9CB
U+690A.0	fGB	3-417E
U+690A.0	fR/S	 75.8
U+690A.0	fTotalStrokes	 12
U+690A.0	fCangjie	DYOJ
U+690A.0	fFourCorner	40948
U+690A.0	fHakka	 zut7; zut8; cut7
U+690A.0	fCantonese	zyut6; cyut3; zeot1
U+690A.0	fMandarin	zuo2
U+690A.0	fEnglish	 fit a handle into a socket plug; cork
U+690B.0	fUTF8	椋
U+690B.0	fCNS11643	2-3C6D
U+690B.0	fBig5	D9CA
U+690B.0	fGB	0-6923
U+690B.0	fR/S	 75.8
U+690B.0	fTotalStrokes	 12
U+690B.0	fCangjie	DYRF
U+690B.0	fFourCorner	40996
U+690B.0	fHakka	  liong2; liong3
U+690B.0	fCantonese	loeng4
U+690B.0	fMandarin	liang2
U+690B.0	fEnglish	fruit
U+690C.0	fUTF8	椌
U+690C.0	fCNS11643	2-3C4D
U+690C.0	fBig5	D9AA
U+690C.0	fGB	3-422A
U+690C.0	fR/S	 75.8
U+690C.0	fTotalStrokes	 12
U+690C.0	fCangjie	DJCM
U+690C.0	fFourCorner	43911
U+690C.0	fHakka	  kiong1; kung1
U+690C.0	fCantonese	hong1
U+690C.0	fMandarin	qiang1; kong1
U+690C.0	fEnglish	instrument
U+690D.0	fUTF8	植
U+690D.0	fCNS11643	1-5F78
U+690D.0	fBig5	B4D3
U+690D.0	fGB	0-5632
U+690D.0	fR/S	 75.8
U+690D.0	fTotalStrokes	 12
U+690D.0	fCangjie	DJBM
U+690D.0	fFourCorner	44916
U+690D.0	fHakka	  chit8; cit8; ts'iit8; tsik8
U+690D.0	fCantonese	zik6
U+690D.0	fMandarin	zhi2; shi4; zhi4
U+690D.0	fEnglish	[1] plant; set up; erect [2] lean on [3] trees; plants; vegetation
U+690E.0	fUTF8	椎
U+690E.0	fCNS11643	1-5F7A
U+690E.0	fBig5	B4D5
U+690E.0	fGB	0-5735
U+690E.0	fR/S	 75.8
U+690E.0	fTotalStrokes	 12
U+690E.0	fCangjie	DOG
U+690E.0	fFourCorner	40914
U+690E.0	fHakka	 cui2; ch'ui2; chui2
U+690E.0	fCantonese	ceoi4; zeoi1
U+690E.0	fMandarin	zhui1; chui2
U+690E.0	fEnglish	[1] hammer; mallet; bludgeon; mace [2] beat; hammer; hit [3] vertebra
U+690F.0	fUTF8	椏
U+690F.0	fCNS11643	2-3C55
U+690F.0	fBig5	D9B2
U+690F.0	fGB	1-6862
U+690F.0	fR/S	 75.8
U+690F.0	fTotalStrokes	 12
U+690F.0	fCangjie	DMLM
U+690F.0	fFourCorner	41917
U+690F.0	fHakka	  a1;  a3;  wa1
U+690F.0	fCantonese	aa1; ngaa1 (also); ngaa4
U+690F.0	fMandarin	ya1; e3; ya2; e4
U+690F.0	fEnglish	the forking branch of a tree
U+6910.0	fUTF8	椐
U+6910.0	fCNS11643	2-3C5C
U+6910.0	fBig5	D9B9
U+6910.0	fGB	0-6927
U+6910.0	fR/S	 75.8
U+6910.0	fTotalStrokes	 12
U+6910.0	fCangjie	DSJR
U+6910.0	fFourCorner	47964
U+6910.0	fHakka	  gi1
U+6910.0	fCantonese	geoi1
U+6910.0	fMandarin	ju1
U+6910.0	fEnglish	a knotty tree from which are made walking sticks for old people
U+6911.0	fUTF8	椑
U+6911.0	fCNS11643	2-3C64
U+6911.0	fBig5	D9C1
U+6911.0	fGB	3-4177
U+6911.0	fR/S	 75.8
U+6911.0	fTotalStrokes	 12
U+6911.0	fCangjie	DHWJ
U+6911.0	fFourCorner	46940
U+6911.0	fHakka	 bi1
U+6911.0	fCantonese	bei1; pei4; pik1
U+6911.0	fMandarin	bei1; pi2; pi4; bai4; pai2
U+6911.0	fEnglish	oval
U+6912.0	fUTF8	椒
U+6912.0	fCNS11643	1-5F79
U+6912.0	fBig5	B4D4
U+6912.0	fGB	0-3D37
U+6912.0	fR/S	 75.8
U+6912.0	fTotalStrokes	 12
U+6912.0	fCangjie	DYFE
U+6912.0	fFourCorner	47940
U+6912.0	fHakka	  ziau1; zeu1
U+6912.0	fCantonese	ziu1
U+6912.0	fMandarin	jiao1; jiao4
U+6912.0	fEnglish	[1] pepper [2] mountaintops
U+6913.0	fUTF8	椓
U+6913.0	fCNS11643	2-3C5B
U+6913.0	fBig5	D9B8
U+6913.0	fGB	3-4171
U+6913.0	fR/S	 75.8
U+6913.0	fTotalStrokes	 12
U+6913.0	fCangjie	DMSO
U+6913.0	fFourCorner	41923
U+6913.0	fHakka	  dok7
U+6913.0	fCantonese	doek3
U+6913.0	fMandarin	zhuo2
U+6913.0	fEnglish	[1] strike; hit; beat; hammer [2] castrate; castration [3] slander; injury
U+6914.0	fUTF8	椔
U+6914.0	fCNS11643	2-3C67
U+6914.0	fBig5	D9C4
U+6914.0	fGB	3-4230
U+6914.0	fR/S	 75.8
U+6914.0	fTotalStrokes	 12
U+6914.0	fCangjie	DVVW
U+6914.0	fFourCorner	42960
U+6914.0	fCantonese	zi1
U+6914.0	fMandarin	zi1; zi4
U+6915.0	fUTF8	椕
U+6915.0	fCNS11643	2-3C7A
U+6915.0	fBig5	D9D7
U+6915.0	fGB	5-4441
U+6915.0	fR/S	 75.8
U+6915.0	fTotalStrokes	 12
U+6915.0	fCangjie	DCHD
U+6915.0	fFourCorner	44990
U+6915.0	fCantonese	ban1
U+6915.0	fMandarin	bin1
U+6916.0	fUTF8	椖
U+6916.0	fCNS11643	F-3B70
U+6916.0	fGB	E-2E36
U+6916.0	fR/S	 75.8
U+6916.0	fTotalStrokes	 12
U+6916.0	fCangjie	DISS
U+6916.0	fFourCorner	4292
U+6916.0	fCantonese	paang4
U+6916.0	fMandarin	peng2
U+6917.0	fUTF8	椗
U+6917.0	fCNS11643	2-3C6F
U+6917.0	fBig5	D9CC
U+6917.0	fGB	E-2E37
U+6917.0	fR/S	 75.8
U+6917.0	fTotalStrokes	 12
U+6917.0	fCangjie	DJMO
U+6917.0	fFourCorner	43981
U+6917.0	fCantonese	ding3; ding6
U+6917.0	fMandarin	ding4
U+6918.0	fUTF8	椘
U+6918.0	fCNS11643	3-3C3C
U+6918.0	fGB	E-2E38
U+6918.0	fR/S	 75.8
U+6918.0	fTotalStrokes	 12
U+6918.0	fCangjie	DDINO
U+6918.0	fFourCorner	44300
U+6918.0	fHakka	  co3; cu3; cih3; ze3; chui3
U+6918.0	fCantonese	co2
U+6918.0	fMandarin	chu3; chu4
U+6918.0	fEnglish	same as &#x695A;
U+6919.0	fUTF8	椙
U+6919.0	fGB	E-2E39
U+6919.0	fR/S	 75.8
U+6919.0	fTotalStrokes	 12
U+6919.0	fCangjie	DAA
U+6919.0	fFourCorner	4696
U+6919.0	fCantonese	coeng1
U+6919.0	fMandarin	shan1
U+691A.0	fUTF8	椚
U+691A.0	fGB	E-2E3A
U+691A.0	fR/S	 75.8
U+691A.0	fTotalStrokes	 12
U+691A.0	fCangjie	DAN
U+691A.0	fFourCorner	4792
U+691A.0	fCantonese	haan4
U+691A.0	fEnglish	a kind of oak used for charcoal
U+691B.0	fUTF8	椛
U+691B.0	fCNS11643	F-3C21
U+691B.0	fGB	E-2E3B
U+691B.0	fR/S	 75.8
U+691B.0	fTotalStrokes	 12
U+691B.0	fCangjie	DTOP
U+691B.0	fCantonese	waa4
U+691B.0	fEnglish	type of birch
U+691C.0	fUTF8	検
U+691C.0	fCNS11643	3-3C44
U+691C.0	fGB	E-2E3C
U+691C.0	fR/S	 75.8
U+691C.0	fTotalStrokes	 12
U+691C.0	fCangjie	DOMO
U+691C.0	fFourCorner	48930
U+691C.0	fHakka	 giam3
U+691C.0	fCantonese	gim2
U+691C.0	fMandarin	jian3
U+691C.0	fEnglish	same as &#x6AA2;
U+691D.0	fUTF8	椝
U+691D.0	fGB	8-2D6D
U+691D.0	fR/S	 75.11; 75.8
U+691D.0	fTotalStrokes	 12
U+691D.0	fCangjie	XQUD
U+691D.0	fCantonese	kwai1
U+691D.0	fMandarin	gui1
U+691E.0	fUTF8	椞
U+691E.0	fCNS11643	3-3C33
U+691E.0	fGB	E-2E3D
U+691E.0	fR/S	 75.8
U+691E.0	fTotalStrokes	 12
U+691E.0	fCangjie	DLD
U+691E.0	fFourCorner	42900
U+691E.0	fCantonese	sap1
U+691E.0	fMandarin	xi4
U+691F.0	fUTF8	椟
U+691F.0	fGB	0-687C
U+691F.0	fR/S	 75.8
U+691F.0	fTotalStrokes	 12
U+691F.0	fCangjie	DJNK
U+691F.0	fHakka	 tuk8
U+691F.0	fCantonese	duk6
U+691F.0	fMandarin	du2
U+691F.0	fEnglish	simplified form of &#x6ADD;
U+6920.0	fUTF8	椠
U+6920.0	fGB	0-687D
U+6920.0	fR/S	 75.8
U+6920.0	fTotalStrokes	 12
U+6920.0	fCangjie	KLD
U+6920.0	fHakka	 ciam5
U+6920.0	fCantonese	cim3
U+6920.0	fMandarin	qian4; jian4; qian1
U+6920.0	fEnglish	simplified form of &#x69E7;
U+6921.0	fUTF8	椡
U+6921.0	fGB	E-2E3E
U+6921.0	fR/S	 75.8
U+6921.0	fTotalStrokes	 12
U+6921.0	fCangjie	DMGN
U+6921.0	fCantonese	dou3
U+6922.0	fUTF8	椢
U+6922.0	fGB	E-2E3F
U+6922.0	fR/S	 75.8
U+6922.0	fTotalStrokes	 12
U+6922.0	fCangjie	DWMI
U+6922.0	fCantonese	gui3; gwui3 (also)
U+6922.0	fMandarin	kui4
U+6923.0	fUTF8	椣
U+6923.0	fGB	E-2E40
U+6923.0	fR/S	 75.8
U+6923.0	fTotalStrokes	 12
U+6923.0	fCangjie	DTBC
U+6923.0	fCantonese	din2
U+6924.0	fUTF8	椤
U+6924.0	fGB	0-6921
U+6924.0	fR/S	 75.8
U+6924.0	fTotalStrokes	 12
U+6924.0	fCangjie	DWLN
U+6924.0	fHakka	 lo2
U+6924.0	fCantonese	lo4
U+6924.0	fMandarin	luo2; luo4; luo3
U+6924.0	fEnglish	simplified form of &#x6B0F;
U+6925.0	fUTF8	椥
U+6925.0	fCNS11643	2-3C7B
U+6925.0	fBig5	D9D8
U+6925.0	fGB	E-2E41
U+6925.0	fR/S	 75.8
U+6925.0	fTotalStrokes	 12
U+6925.0	fCangjie	DOKR
U+6925.0	fFourCorner	46900
U+6925.0	fCantonese	zi1
U+6925.0	fMandarin	zhi1
U+6926.0	fUTF8	椦
U+6926.0	fGB	E-2E42
U+6926.0	fR/S	 75.8
U+6926.0	fTotalStrokes	 12
U+6926.0	fCangjie	DFQS
U+6926.0	fCantonese	hyun1
U+6927.0	fUTF8	椧
U+6927.0	fCNS11643	F-3C29
U+6927.0	fGB	1-7D3E
U+6927.0	fR/S	 75.8
U+6927.0	fTotalStrokes	 12
U+6927.0	fCangjie	DOML
U+6927.0	fCantonese	ming6
U+6927.0	fEnglish	 gutter
U+6928.0	fUTF8	椨
U+6928.0	fGB	E-2E43
U+6928.0	fR/S	 75.8
U+6928.0	fTotalStrokes	 12
U+6928.0	fCangjie	DIOI
U+6928.0	fCantonese	fu2
U+6929.0	fUTF8	椩
U+6929.0	fCNS11643	3-652F
U+6929.0	fGB	E-2E44
U+6929.0	fR/S	 75.8
U+6929.0	fTotalStrokes	 12
U+6929.0	fCangjie	DILO
U+6929.0	fFourCorner	4093
U+6929.0	fCantonese	gang1
U+692A.0	fUTF8	椪
U+692A.0	fCNS11643	2-3C51
U+692A.0	fBig5	D9AE
U+692A.0	fGB	3-4226
U+692A.0	fR/S	 75.8
U+692A.0	fTotalStrokes	 12
U+692A.0	fCangjie	DTTC
U+692A.0	fFourCorner	48912
U+692A.0	fCantonese	pung3
U+692A.0	fMandarin	peng4
U+692A.0	fEnglish	machilus nanmu; variety of everg
U+692B.0	fUTF8	椫
U+692B.0	fCNS11643	F-3B6E
U+692B.0	fGB	8-7B78
U+692B.0	fR/S	 75.12; 75.8
U+692B.0	fTotalStrokes	 12
U+692B.0	fCangjie	DCWJ
U+692B.0	fCantonese	sin6; zin2 (also)
U+692B.0	fMandarin	zhan3; shan4
U+692C.0	fUTF8	椬
U+692C.0	fCNS11643	3-652E
U+692C.0	fGB	E-2E45
U+692C.0	fR/S	 75.8
U+692C.0	fTotalStrokes	 12
U+692C.0	fCangjie	DJBM
U+692C.0	fFourCorner	4391
U+692C.0	fCantonese	ji4
U+692D.0	fUTF8	椭
U+692D.0	fGB	0-4D56
U+692D.0	fR/S	 75.8
U+692D.0	fTotalStrokes	 12
U+692D.0	fCangjie	XDNLB
U+692D.0	fHakka	 to3
U+692D.0	fCantonese	to5
U+692D.0	fMandarin	tuo3; duo3; tuo2
U+692D.0	fEnglish	simplified form of &#x6A62;
U+692E.0	fUTF8	椮
U+692E.0	fCNS11643	3-3C40
U+692E.0	fGB	E-2E46
U+692E.0	fR/S	  75.8
U+692E.0	fTotalStrokes	 12
U+692E.0	fCangjie	DIKH
U+692E.0	fFourCorner	43920
U+692E.0	fCantonese	sam1
U+692E.0	fMandarin	sen1; shen1
U+692F.0	fUTF8	椯
U+692F.0	fCNS11643	2-4378
U+692F.0	fBig5	DDF2
U+692F.0	fGB	3-423E
U+692F.0	fR/S	 75.9
U+692F.0	fTotalStrokes	 13
U+692F.0	fCangjie	DUMB
U+692F.0	fFourCorner	42927
U+692F.0	fCantonese	do2; cyun4; syun4
U+692F.0	fMandarin	duo2; chuai3; chuan2
U+6930.0	fUTF8	椰
U+6930.0	fCNS11643	1-644C
U+6930.0	fBig5	B7A6
U+6930.0	fGB	0-522C
U+6930.0	fR/S	 75.9
U+6930.0	fTotalStrokes	 13
U+6930.0	fCangjie	DSJL
U+6930.0	fFourCorner	47927
U+6930.0	fHakka	  ja2; jai2; ra2; jai3; zai2; ye2
U+6930.0	fCantonese	je4; je2 (also)
U+6930.0	fMandarin	ye1; ye2
U+6930.0	fEnglish	 coconut; coconut palm; coconut tree
U+6931.0	fUTF8	椱
U+6931.0	fCNS11643	3-4165
U+6931.0	fGB	3-4242
U+6931.0	fR/S	  75.9
U+6931.0	fTotalStrokes	 13
U+6931.0	fCangjie	DOAE
U+6931.0	fFourCorner	48940
U+6931.0	fCantonese	fuk6
U+6931.0	fMandarin	fou4; fu4
U+6932.0	fUTF8	椲
U+6932.0	fCNS11643	2-4376
U+6932.0	fBig5	DDF0
U+6932.0	fGB	5-4340
U+6932.0	fR/S	 75.9
U+6932.0	fTotalStrokes	 13
U+6932.0	fCangjie	DDMQ
U+6932.0	fFourCorner	45956
U+6932.0	fCantonese	wai5; fai1
U+6932.0	fMandarin	wei3; hui1; wei1; wei2
U+6933.0	fUTF8	椳
U+6933.0	fCNS11643	2-4361
U+6933.0	fBig5	DDDB
U+6933.0	fGB	3-423C
U+6933.0	fR/S	 75.9
U+6933.0	fTotalStrokes	 13
U+6933.0	fCangjie	DWMV
U+6933.0	fFourCorner	46932
U+6933.0	fHakka	 woi1; voe1; voi1
U+6933.0	fCantonese	wui1
U+6933.0	fMandarin	wei1; wei3
U+6933.0	fEnglish	the pivots at the top and bottom of a Chinese door on which the door turns; sockets which hold doors in place
U+6934.0	fUTF8	椴
U+6934.0	fCNS11643	2-4366
U+6934.0	fBig5	DDE0
U+6934.0	fGB	0-6932
U+6934.0	fR/S	 75.9
U+6934.0	fTotalStrokes	 13
U+6934.0	fCangjie	DHJE
U+6934.0	fFourCorner	47947
U+6934.0	fHakka	 ton5; ton3; ton6; fuk8
U+6934.0	fCantonese	dyun6
U+6934.0	fMandarin	duan4; xia2
U+6934.0	fEnglish	poplar; aspen
U+6935.0	fUTF8	椵
U+6935.0	fCNS11643	2-435F
U+6935.0	fBig5	DDD9
U+6935.0	fGB	3-424E
U+6935.0	fR/S	 75.9
U+6935.0	fTotalStrokes	 13
U+6935.0	fCangjie	DRSE
U+6935.0	fFourCorner	47947
U+6935.0	fHakka	  ga5
U+6935.0	fCantonese	gaa2; gaa1
U+6935.0	fMandarin	jia3; jia1; jia4; xia2
U+6935.0	fEnglish	 pompelmoose canque
U+6936.0	fUTF8	椶
U+6936.0	fCNS11643	3-4177
U+6936.0	fGB	E-2E47
U+6936.0	fR/S	  75.9
U+6936.0	fTotalStrokes	 13
U+6936.0	fCangjie	DUCE
U+6936.0	fFourCorner	42940
U+6936.0	fHakka	  zung1
U+6936.0	fCantonese	zung1
U+6936.0	fMandarin	zong1
U+6936.0	fEnglish	palm tree
U+6937.0	fUTF8	椷
U+6937.0	fCNS11643	2-4372
U+6937.0	fBig5	DDEC
U+6937.0	fGB	E-2E48
U+6937.0	fR/S	 75.9
U+6937.0	fTotalStrokes	 13
U+6937.0	fCangjie	DIHR
U+6937.0	fFourCorner	43950
U+6937.0	fHakka	  kiam2; ham2
U+6937.0	fCantonese	haam4; gaam1; zin1 (also)
U+6937.0	fMandarin	jian1; han2
U+6937.0	fEnglish	[1] wooden box; casket; chest [2] same as $ ( letter; envelope)
U+6937.1	fUTF8	椷
U+6937.1	fCNS11643	2-4372
U+6937.1	fBig5	DDEC
U+6937.1	fGB	E-2E48
U+6937.1	fR/S	 75.9
U+6937.1	fTotalStrokes	 13
U+6937.1	fCangjie	DIHR
U+6937.1	fFourCorner	43950
U+6937.1	fMandarin	xian2
U+6937.1	fEnglish	 goblet
U+6938.0	fUTF8	椸
U+6938.0	fCNS11643	2-4351
U+6938.0	fBig5	DDCB
U+6938.0	fGB	3-424B
U+6938.0	fR/S	 75.9
U+6938.0	fTotalStrokes	 13
U+6938.0	fCangjie	DYSD
U+6938.0	fFourCorner	48912
U+6938.0	fHakka	 ji2; ri2; i2
U+6938.0	fCantonese	ji4
U+6938.0	fMandarin	yi2; yi4
U+6938.0	fEnglish	rack
U+6939.0	fUTF8	椹
U+6939.0	fCNS11643	2-4358
U+6939.0	fBig5	DDD2
U+6939.0	fGB	0-6929
U+6939.0	fR/S	 75.9
U+6939.0	fTotalStrokes	 13
U+6939.0	fCangjie	DTMV
U+6939.0	fFourCorner	44911
U+6939.0	fHakka	 zhim1; zihm1; zem1; shim3; siim3; sim3
U+6939.0	fCantonese	zam1; sam6
U+6939.0	fMandarin	shen4; zhen1
U+6939.0	fEnglish	 chopping board; block
U+693A.0	fUTF8	椺
U+693A.0	fCNS11643	4-3D71
U+693A.0	fGB	1-7D42
U+693A.0	fR/S	 75.9
U+693A.0	fTotalStrokes	 13
U+693A.0	fCangjie	DORD
U+693A.0	fFourCorner	46990
U+693A.0	fCantonese	hat6
U+693A.0	fMandarin	xi2
U+693A.0	fEnglish	beam of a house
U+693B.0	fUTF8	椻
U+693B.0	fCNS11643	2-4370
U+693B.0	fBig5	DDEA
U+693B.0	fGB	5-4453
U+693B.0	fR/S	 75.9
U+693B.0	fTotalStrokes	 13
U+693B.0	fCangjie	DSAV
U+693B.0	fFourCorner	41914
U+693B.0	fCantonese	jin3; jin2
U+693B.0	fMandarin	yan4
U+693C.0	fUTF8	椼
U+693C.0	fCNS11643	2-437A
U+693C.0	fBig5	DDF4
U+693C.0	fGB	5-4464
U+693C.0	fR/S	 75.9
U+693C.0	fTotalStrokes	 13
U+693C.0	fCangjie	DHON
U+693C.0	fFourCorner	41921
U+693C.0	fCantonese	jin5; jin2
U+693C.0	fMandarin	yan3
U+693D.0	fUTF8	椽
U+693D.0	fCNS11643	2-4362
U+693D.0	fBig5	DDDC
U+693D.0	fGB	0-342A
U+693D.0	fR/S	 75.9
U+693D.0	fTotalStrokes	 13
U+693D.0	fCangjie	DVNO
U+693D.0	fFourCorner	47932
U+693D.0	fHakka	 con3; con2; chon2; jen2; chon3; jan2
U+693D.0	fCantonese	cyun4; jyun4
U+693D.0	fMandarin	chuan2; zhuan4
U+693D.0	fEnglish	 beam; rafter; support
U+693E.0	fUTF8	椾
U+693E.0	fCNS11643	3-4168
U+693E.0	fGB	E-2E49
U+693E.0	fR/S	  75.9
U+693E.0	fTotalStrokes	 13
U+693E.0	fCangjie	DTBN
U+693E.0	fFourCorner	48920
U+693E.0	fCantonese	zin1
U+693E.0	fMandarin	zhan4; jian1
U+693F.0	fUTF8	椿
U+693F.0	fCNS11643	2-4355
U+693F.0	fBig5	DDCF
U+693F.0	fGB	0-343B
U+693F.0	fR/S	 75.9
U+693F.0	fTotalStrokes	 13
U+693F.0	fCangjie	DQKA
U+693F.0	fFourCorner	45963
U+693F.0	fHakka	  chun1; cun1; chun3
U+693F.0	fCantonese	ceon1
U+693F.0	fMandarin	chun1
U+693F.0	fEnglish	[1] one's father [2] Cedrela sinensis
U+6940.0	fUTF8	楀
U+6940.0	fCNS11643	2-4368
U+6940.0	fBig5	DDE2
U+6940.0	fGB	3-4244
U+6940.0	fR/S	 75.9
U+6940.0	fTotalStrokes	 13
U+6940.0	fCangjie	DHLB
U+6940.0	fFourCorner	42927
U+6940.0	fHakka	 ji3; ri3
U+6940.0	fCantonese	jyu5; geoi2
U+6940.0	fMandarin	yu3; ju3
U+6940.0	fEnglish	 the name of tree a Chinese family name
U+6941.0	fUTF8	楁
U+6941.0	fCNS11643	2-436D
U+6941.0	fBig5	DDE7
U+6941.0	fGB	5-4470
U+6941.0	fR/S	 75.9
U+6941.0	fTotalStrokes	 13
U+6941.0	fCangjie	DJHR
U+6941.0	fFourCorner	43964
U+6941.0	fCantonese	haak6; hat6
U+6941.0	fMandarin	he2
U+6942.0	fUTF8	楂
U+6942.0	fCNS11643	2-4359
U+6942.0	fBig5	DDD3
U+6942.0	fGB	0-692B
U+6942.0	fR/S	 75.9
U+6942.0	fTotalStrokes	 13
U+6942.0	fCangjie	DDAM
U+6942.0	fFourCorner	44916
U+6942.0	fHakka	 ca2; za1
U+6942.0	fCantonese	zaa1; caa4
U+6942.0	fMandarin	zha1; cha2; cha1
U+6942.0	fEnglish	[1] wooden raft [2] a species of hawthorn
U+6943.0	fUTF8	楃
U+6943.0	fCNS11643	3-416B
U+6943.0	fGB	5-4471
U+6943.0	fR/S	 75.9
U+6943.0	fTotalStrokes	 13
U+6943.0	fCangjie	DSMG
U+6943.0	fFourCorner	47910
U+6943.0	fCantonese	ak1; ngak1 (also)
U+6943.0	fMandarin	wo4
U+6944.0	fUTF8	楄
U+6944.0	fCNS11643	2-436A
U+6944.0	fBig5	DDE4
U+6944.0	fGB	3-424D
U+6944.0	fR/S	 75.9
U+6944.0	fTotalStrokes	 13
U+6944.0	fCangjie	DHSB
U+6944.0	fFourCorner	43927
U+6944.0	fHakka	  pien1
U+6944.0	fCantonese	pin4
U+6944.0	fMandarin	pian2; bian1
U+6944.0	fEnglish	block
U+6945.0	fUTF8	楅
U+6945.0	fCNS11643	2-4356
U+6945.0	fBig5	DDD0
U+6945.0	fGB	3-4235
U+6945.0	fR/S	 75.9
U+6945.0	fTotalStrokes	 13
U+6945.0	fCangjie	DMRW
U+6945.0	fFourCorner	41966
U+6945.0	fHakka	  fuk7;  bet7; bit7
U+6945.0	fCantonese	bik1; fuk1
U+6945.0	fMandarin	bi4; bi1; fu2
U+6945.0	fEnglish	buard
U+6946.0	fUTF8	楆
U+6946.0	fCNS11643	3-415D
U+6946.0	fGB	5-4456
U+6946.0	fR/S	 75.9
U+6946.0	fTotalStrokes	 13
U+6946.0	fCangjie	DMWV
U+6946.0	fFourCorner	41940
U+6946.0	fCantonese	jiu1
U+6946.0	fMandarin	yao1
U+6947.0	fUTF8	楇
U+6947.0	fCNS11643	3-4175
U+6947.0	fGB	3-4159
U+6947.0	fR/S	  75.9
U+6947.0	fTotalStrokes	 13
U+6947.0	fCangjie	DBBR
U+6947.0	fFourCorner	47920
U+6947.0	fHakka	 ko1; kwo3
U+6947.0	fCantonese	gwo2; go2 (also)
U+6947.0	fMandarin	huo4
U+6947.0	fEnglish	a grease-pot, hung at the axles of carts
U+6948.0	fUTF8	楈
U+6948.0	fCNS11643	2-435D
U+6948.0	fBig5	DDD7
U+6948.0	fGB	5-4473
U+6948.0	fR/S	 75.9
U+6948.0	fTotalStrokes	 13
U+6948.0	fCangjie	DNOB
U+6948.0	fFourCorner	47927
U+6948.0	fCantonese	seoi1
U+6948.0	fMandarin	xu1; shan1; xu3; xu4
U+6949.0	fUTF8	楉
U+6949.0	fCNS11643	2-435E
U+6949.0	fBig5	DDD8
U+6949.0	fGB	3-416D
U+6949.0	fR/S	 75.9; 75.8
U+6949.0	fTotalStrokes	 13
U+6949.0	fCangjie	DTKR
U+6949.0	fFourCorner	44964
U+6949.0	fHakka	 jok8; rok8
U+6949.0	fCantonese	joek6
U+6949.0	fMandarin	ruo4
U+6949.0	fEnglish	a kind of plum
U+694A.0	fUTF8	楊
U+694A.0	fCNS11643	1-644E
U+694A.0	fBig5	B7A8
U+694A.0	fGB	1-516E
U+694A.0	fR/S	 75.9
U+694A.0	fTotalStrokes	 13
U+694A.0	fCangjie	DAMH
U+694A.0	fFourCorner	46927
U+694A.0	fHakka	  jong2; rong2; jong3
U+694A.0	fCantonese	joeng4; joeng2 (also)
U+694A.0	fMandarin	yang2
U+694A.0	fEnglish	[1] willow [2] poplar [3] a Chinese family name
U+694B.0	fUTF8	楋
U+694B.0	fCNS11643	2-4371
U+694B.0	fBig5	DDEB
U+694B.0	fGB	5-4454
U+694B.0	fR/S	 75.9
U+694B.0	fTotalStrokes	 13
U+694B.0	fCangjie	DDLN
U+694B.0	fFourCorner	52900
U+694B.0	fCantonese	lip6; laat6
U+694B.0	fMandarin	la4
U+694C.0	fUTF8	楌
U+694C.0	fCNS11643	2-436F
U+694C.0	fBig5	DDE9
U+694C.0	fGB	5-446D
U+694C.0	fR/S	 75.9
U+694C.0	fTotalStrokes	 13
U+694C.0	fCangjie	DYHH
U+694C.0	fFourCorner	40922
U+694C.0	fCantonese	ngaan4; aan4 (also)
U+694C.0	fMandarin	yan2
U+694D.0	fUTF8	楍
U+694D.0	fCNS11643	3-4173
U+694D.0	fGB	E-2E4A
U+694D.0	fR/S	 75.9
U+694D.0	fTotalStrokes	 13
U+694D.0	fCangjie	DRRR
U+694D.0	fFourCorner	40669
U+694D.0	fHakka	  bun3
U+694D.0	fCantonese	bun2
U+694D.0	fMandarin	ben3; ben1
U+694D.0	fEnglish	ancient form of &#x672C;
U+694E.0	fUTF8	楎
U+694E.0	fCNS11643	2-4352
U+694E.0	fBig5	DDCC
U+694E.0	fGB	3-4142
U+694E.0	fR/S	 75.9
U+694E.0	fTotalStrokes	 13
U+694E.0	fCangjie	DBJJ
U+694E.0	fFourCorner	47950
U+694E.0	fHakka	 fui1; fun2; fi1
U+694E.0	fCantonese	wan4; fai1
U+694E.0	fMandarin	hun2; hui1; gun1; jun1
U+694E.0	fEnglish	peg
U+694F.0	fUTF8	楏
U+694F.0	fCNS11643	2-4374
U+694F.0	fBig5	DDEE
U+694F.0	fGB	5-4458
U+694F.0	fR/S	 75.9
U+694F.0	fTotalStrokes	 13
U+694F.0	fCangjie	DKGG
U+694F.0	fFourCorner	44914
U+694F.0	fHakka	 kui2
U+694F.0	fCantonese	fui1; kwai1
U+694F.0	fMandarin	kui2
U+6950.0	fUTF8	楐
U+6950.0	fCNS11643	3-4174
U+6950.0	fGB	5-445D
U+6950.0	fR/S	  75.9
U+6950.0	fTotalStrokes	 13
U+6950.0	fCangjie	DWOL
U+6950.0	fFourCorner	46920
U+6950.0	fCantonese	gaai3
U+6950.0	fMandarin	jie4
U+6951.0	fUTF8	楑
U+6951.0	fCNS11643	2-4375
U+6951.0	fBig5	DDEF
U+6951.0	fGB	5-4474
U+6951.0	fR/S	 75.9
U+6951.0	fTotalStrokes	 13
U+6951.0	fCangjie	DNOK
U+6951.0	fFourCorner	42934
U+6951.0	fCantonese	kwai5; kwai4
U+6951.0	fMandarin	kui2; kui3
U+6952.0	fUTF8	楒
U+6952.0	fCNS11643	2-4377
U+6952.0	fBig5	DDF1
U+6952.0	fGB	5-445E
U+6952.0	fR/S	 75.9
U+6952.0	fTotalStrokes	 13
U+6952.0	fCangjie	DWP
U+6952.0	fFourCorner	46930
U+6952.0	fHakka	 sii1; se1
U+6952.0	fCantonese	si1
U+6952.0	fMandarin	si1
U+6953.0	fUTF8	楓
U+6953.0	fCNS11643	1-6452
U+6953.0	fBig5	B7AC
U+6953.0	fGB	1-3763
U+6953.0	fR/S	 75.9
U+6953.0	fTotalStrokes	 13
U+6953.0	fCangjie	DHNI
U+6953.0	fFourCorner	47910
U+6953.0	fHakka	  fung1; pung1; bung1
U+6953.0	fCantonese	fung1
U+6953.0	fMandarin	feng1; bian1; fan1
U+6953.0	fEnglish	 maple tree; maple
U+6954.0	fUTF8	楔
U+6954.0	fCNS11643	1-644A
U+6954.0	fBig5	B7A4
U+6954.0	fGB	0-5028
U+6954.0	fR/S	 75.9
U+6954.0	fTotalStrokes	 13
U+6954.0	fCangjie	DQHK
U+6954.0	fFourCorner	47934
U+6954.0	fHakka	 siap7;  siet7; set7; siap8
U+6954.0	fCantonese	sit3
U+6954.0	fMandarin	xie1; xie4; jia2; jie2
U+6954.0	fEnglish	[1] wedge [2] gatepost; forward [3] wedge
U+6955.0	fUTF8	楕
U+6955.0	fCNS11643	3-416F
U+6955.0	fGB	3-4236
U+6955.0	fR/S	  75.9
U+6955.0	fTotalStrokes	 13
U+6955.0	fCangjie	DKMB
U+6955.0	fFourCorner	44920
U+6955.0	fHakka	 to5; to3
U+6955.0	fCantonese	to5
U+6955.0	fMandarin	tuo3
U+6955.0	fEnglish	oval-shaped; elliptical; tubular
U+6956.0	fUTF8	楖
U+6956.0	fCNS11643	2-357A
U+6956.0	fBig5	D5B8
U+6956.0	fGB	3-4166
U+6956.0	fR/S	 75.9
U+6956.0	fTotalStrokes	 13
U+6956.0	fCangjie	DAIL
U+6956.0	fFourCorner	47920
U+6956.0	fHakka	 zit7; ziet7
U+6956.0	fCantonese	zit3; zik1; zat1
U+6956.0	fMandarin	ji2; zhi4
U+6956.0	fEnglish	a tree allied to the ash
U+6957.0	fUTF8	楗
U+6957.0	fCNS11643	2-435A
U+6957.0	fBig5	DDD4
U+6957.0	fGB	0-6925
U+6957.0	fR/S	 75.9
U+6957.0	fTotalStrokes	 13
U+6957.0	fCangjie	DNKQ
U+6957.0	fFourCorner	45944
U+6957.0	fHakka	 kien6; kien5; kian5; ken5
U+6957.0	fCantonese	gin6; gin2 (also); kin2
U+6957.0	fMandarin	jian4; juan4; jian3
U+6957.0	fEnglish	bar of door; bolt of lock
U+6958.0	fUTF8	楘
U+6958.0	fCNS11643	2-436C
U+6958.0	fBig5	DDE6
U+6958.0	fGB	3-4250
U+6958.0	fR/S	 75.9
U+6958.0	fTotalStrokes	 13
U+6958.0	fCangjie	NKD
U+6958.0	fFourCorner	18904
U+6958.0	fHakka	 miau3; meu6; meu5; muk7
U+6958.0	fCantonese	mou6; muk6 (also)
U+6958.0	fMandarin	mu4
U+6958.0	fEnglish	band
U+6959.0	fUTF8	楙
U+6959.0	fCNS11643	2-435B
U+6959.0	fBig5	DDD5
U+6959.0	fGB	3-424F
U+6959.0	fR/S	 75.9
U+6959.0	fTotalStrokes	 13
U+6959.0	fCangjie	DNHD
U+6959.0	fFourCorner	44990
U+6959.0	fHakka	 muk8; miu3; meu5; miau3
U+6959.0	fCantonese	mau6
U+6959.0	fMandarin	mao4
U+6959.0	fEnglish	[1] name of plant [2] luxuriant; lush (said of vegetation)
U+695A.0	fUTF8	楚
U+695A.0	fCNS11643	1-6447
U+695A.0	fBig5	B7A1
U+695A.0	fGB	0-337E
U+695A.0	fR/S	 75.9
U+695A.0	fTotalStrokes	 13
U+695A.0	fCangjie	DDNYO
U+695A.0	fFourCorner	44801
U+695A.0	fHakka	  co3; cu3; cih3; ze3; chui3
U+695A.0	fCantonese	co2
U+695A.0	fMandarin	chu3; chu4
U+695A.0	fEnglish	[1] name of a powerful feudal state (740-330 B.C.) [2] a Chinese family name [3] clear; neat [4] suffering; distress
U+695B.0	fUTF8	楛
U+695B.0	fCNS11643	1-6457
U+695B.0	fBig5	B7B1
U+695B.0	fGB	3-416C
U+695B.0	fR/S	 75.9; 75.8
U+695B.0	fTotalStrokes	 13
U+695B.0	fCangjie	DTJR
U+695B.0	fFourCorner	44964
U+695B.0	fHakka	 fu3;  fu5; ku3; fu6
U+695B.0	fCantonese	fu2; wu6
U+695B.0	fMandarin	hu4
U+695B.0	fEnglish	the arrow-thorn
U+695B.1	fUTF8	楛
U+695B.1	fCNS11643	1-6457
U+695B.1	fBig5	B7B1
U+695B.1	fGB	3-416C
U+695B.1	fR/S	 75.9; 75.8
U+695B.1	fTotalStrokes	 13
U+695B.1	fCangjie	DTJR
U+695B.1	fFourCorner	44964
U+695B.1	fMandarin	ku3
U+695B.1	fEnglish	crude and easy to break
U+695C.0	fUTF8	楜
U+695C.0	fCNS11643	2-4373
U+695C.0	fBig5	DDED
U+695C.0	fGB	E-2E4B
U+695C.0	fR/S	 75.9
U+695C.0	fTotalStrokes	 13
U+695C.0	fCangjie	DJRB
U+695C.0	fFourCorner	47920
U+695C.0	fHakka	 fu2
U+695C.0	fCantonese	wu4
U+695C.0	fMandarin	hu2
U+695D.0	fUTF8	楝
U+695D.0	fCNS11643	1-6455
U+695D.0	fBig5	B7AF
U+695D.0	fGB	0-692C
U+695D.0	fR/S	 75.9
U+695D.0	fTotalStrokes	 13
U+695D.0	fCangjie	DDWF
U+695D.0	fFourCorner	45996
U+695D.0	fHakka	 lien5; lien6; len3
U+695D.0	fCantonese	lin6
U+695D.0	fMandarin	lian4
U+695D.0	fEnglish	Melia azedarach; chinaberry
U+695E.0	fUTF8	楞
U+695E.0	fCNS11643	1-6451
U+695E.0	fBig5	B7AB
U+695E.0	fGB	0-4063
U+695E.0	fR/S	 75.9
U+695E.0	fTotalStrokes	 13
U+695E.0	fCangjie	DWLS
U+695E.0	fFourCorner	46927
U+695E.0	fHakka	 lin2; len5
U+695E.0	fCantonese	ling4; ling6
U+695E.0	fMandarin	leng2
U+695E.0	fEnglish	[1] angular [2] edge [3] used for Ceylon in Buddhist text
U+695E.1	fUTF8	楞
U+695E.1	fCNS11643	1-6451
U+695E.1	fBig5	B7AB
U+695E.1	fGB	0-4063
U+695E.1	fR/S	 75.9
U+695E.1	fTotalStrokes	 13
U+695E.1	fCangjie	DWLS
U+695E.1	fFourCorner	46927
U+695E.1	fMandarin	leng4
U+695E.1	fEnglish	stupid; imbecile; rude
U+695F.0	fUTF8	楟
U+695F.0	fCNS11643	2-4350
U+695F.0	fBig5	DDCA
U+695F.0	fGB	3-4248
U+695F.0	fR/S	 75.9
U+695F.0	fTotalStrokes	 13
U+695F.0	fCangjie	DYRN
U+695F.0	fFourCorner	40921
U+695F.0	fCantonese	ting4
U+695F.0	fMandarin	ting2
U+695F.0	fEnglish	tree
U+6960.0	fUTF8	楠
U+6960.0	fCNS11643	1-6449
U+6960.0	fBig5	B7A3
U+6960.0	fGB	0-692A
U+6960.0	fR/S	 75.9
U+6960.0	fTotalStrokes	 13
U+6960.0	fCangjie	DJBJ
U+6960.0	fFourCorner	44927
U+6960.0	fHakka	 nam2; lam2; nem5
U+6960.0	fCantonese	naam4; laam4 (also)
U+6960.0	fMandarin	nan2
U+6960.0	fEnglish	name of tree; machilus nanmu
U+6961.0	fUTF8	楡
U+6961.0	fCNS11643	3-4179
U+6961.0	fGB	E-2E4C
U+6961.0	fR/S	 75.9
U+6961.0	fTotalStrokes	 13
U+6961.0	fCangjie	DOMV
U+6961.0	fFourCorner	48920
U+6961.0	fHakka	 ji2;  ji3; ji5; ri2; ri3; i2
U+6961.0	fCantonese	jyu4
U+6961.0	fMandarin	yu2
U+6961.0	fEnglish	 elm tree; elm
U+6962.0	fUTF8	楢
U+6962.0	fCNS11643	2-4353
U+6962.0	fBig5	DDCD
U+6962.0	fGB	3-424C
U+6962.0	fR/S	 75.9
U+6962.0	fTotalStrokes	 13
U+6962.0	fCangjie	DTCW
U+6962.0	fFourCorner	48961
U+6962.0	fHakka	 ju2; ju3
U+6962.0	fCantonese	jau4
U+6962.0	fMandarin	you2; chu1; qiu1; you3
U+6962.0	fEnglish	Quercus glandulifera
U+6963.0	fUTF8	楣
U+6963.0	fCNS11643	1-6456
U+6963.0	fBig5	B7B0
U+6963.0	fGB	0-6939
U+6963.0	fR/S	 75.9
U+6963.0	fTotalStrokes	 13
U+6963.0	fCangjie	DAHU
U+6963.0	fFourCorner	47967
U+6963.0	fHakka	  mi2
U+6963.0	fCantonese	mei4
U+6963.0	fMandarin	mei2
U+6963.0	fEnglish	the crossbeam above or under gate; lintel (over a door)
U+6964.0	fUTF8	楤
U+6964.0	fCNS11643	3-4178
U+6964.0	fGB	5-4467
U+6964.0	fR/S	 75.9
U+6964.0	fTotalStrokes	 13
U+6964.0	fCangjie	DPKP
U+6964.0	fFourCorner	47930
U+6964.0	fHakka	  cung1
U+6964.0	fCantonese	sung2
U+6964.0	fMandarin	song3
U+6964.0	fEnglish	[1] the pricky ash [2] Aralia spinosa
U+6965.0	fUTF8	楥
U+6965.0	fCNS11643	2-4363
U+6965.0	fBig5	DDDD
U+6965.0	fGB	E-2E4D
U+6965.0	fR/S	 75.9
U+6965.0	fTotalStrokes	 13
U+6965.0	fCangjie	DBME
U+6965.0	fFourCorner	52947
U+6965.0	fHakka	 hien5
U+6965.0	fCantonese	jyun4; hyun3
U+6965.0	fMandarin	xuan4; yuan2
U+6965.0	fEnglish	 a last for making shoes turn on a lathe
U+6966.0	fUTF8	楦
U+6966.0	fCNS11643	2-434F
U+6966.0	fBig5	DDC9
U+6966.0	fGB	0-6938
U+6966.0	fR/S	 75.9
U+6966.0	fTotalStrokes	 13
U+6966.0	fCangjie	DJMM
U+6966.0	fFourCorner	43916
U+6966.0	fHakka	 hien5; hian5; hen5; sen1; sien5
U+6966.0	fCantonese	hyun3
U+6966.0	fMandarin	xuan4; yuan2
U+6966.0	fEnglish	same as &#x6965;
U+6967.0	fUTF8	楧
U+6967.0	fCNS11643	3-4176
U+6967.0	fGB	5-4435
U+6967.0	fR/S	 75.9
U+6967.0	fTotalStrokes	 13
U+6967.0	fCangjie	DTLK
U+6967.0	fFourCorner	44930
U+6967.0	fCantonese	joeng1
U+6967.0	fMandarin	ying1; yang1; ang4; yang4
U+6967.0	fEnglish	corrupted form of &#x67CD;
U+6968.0	fUTF8	楨
U+6968.0	fCNS11643	1-644F
U+6968.0	fBig5	B7A9
U+6968.0	fGB	1-6865
U+6968.0	fR/S	 75.9
U+6968.0	fTotalStrokes	 13
U+6968.0	fCangjie	DYBC
U+6968.0	fFourCorner	41981
U+6968.0	fHakka	 zhin1; zin1; zihn1
U+6968.0	fCantonese	zing1
U+6968.0	fMandarin	zhen1; zheng1
U+6968.0	fEnglish	[1] hardwood; sturdy wood [2] the posts at ends of walls
U+6969.0	fUTF8	楩
U+6969.0	fCNS11643	2-4367
U+6969.0	fBig5	DDE1
U+6969.0	fGB	3-4243
U+6969.0	fR/S	 75.9
U+6969.0	fTotalStrokes	 13
U+6969.0	fCangjie	DOMK
U+6969.0	fFourCorner	41946
U+6969.0	fHakka	  pien1; pen1; pen3
U+6969.0	fCantonese	pin4
U+6969.0	fMandarin	pian2; bian2
U+6969.0	fEnglish	a kind of tree
U+696A.0	fUTF8	楪
U+696A.0	fCNS11643	2-4357
U+696A.0	fBig5	DDD1
U+696A.0	fGB	3-4233
U+696A.0	fR/S	 75.9
U+696A.0	fTotalStrokes	 13
U+696A.0	fCangjie	DPTD
U+696A.0	fFourCorner	44994
U+696A.0	fHakka	 jap8; tiap8; rap8
U+696A.0	fCantonese	dip6; jip6
U+696A.0	fMandarin	die2; ye4; xie4
U+696A.0	fEnglish	 small dish window
U+696B.0	fUTF8	楫
U+696B.0	fCNS11643	1-6450
U+696B.0	fBig5	B7AA
U+696B.0	fGB	0-692E
U+696B.0	fR/S	 75.9
U+696B.0	fTotalStrokes	 13
U+696B.0	fCangjie	DRSJ
U+696B.0	fFourCorner	46941
U+696B.0	fHakka	 ziap7
U+696B.0	fCantonese	zip3; zip6
U+696B.0	fMandarin	ji2; jie1
U+696B.0	fEnglish	[1] oar; paddle [2] boat
U+696C.0	fUTF8	楬
U+696C.0	fCNS11643	2-4360
U+696C.0	fBig5	DDDA
U+696C.0	fGB	3-423B
U+696C.0	fR/S	 75.9
U+696C.0	fTotalStrokes	 13
U+696C.0	fCangjie	DAPV
U+696C.0	fFourCorner	46927
U+696C.0	fHakka	  giet7; get7
U+696C.0	fCantonese	git6; hat6; kit3; hat1; hap1
U+696C.0	fMandarin	jie2; qia4; jie1
U+696C.0	fEnglish	 signpost
U+696D.0	fUTF8	業
U+696D.0	fCNS11643	1-6446
U+696D.0	fBig5	B77E
U+696D.0	fGB	1-5235
U+696D.0	fR/S	 75.9
U+696D.0	fTotalStrokes	 13
U+696D.0	fCangjie	TCTD
U+696D.0	fFourCorner	32904
U+696D.0	fHakka	  ngiap8; niap8; gniap8
U+696D.0	fCantonese	jip6
U+696D.0	fMandarin	ye4
U+696D.0	fEnglish	[1] profession; business; trade; work; occupation; vocation; calling [2] estate; property [3] already
U+696E.0	fUTF8	楮
U+696E.0	fCNS11643	1-5F7D
U+696E.0	fBig5	B4D8
U+696E.0	fGB	0-687A
U+696E.0	fR/S	 75.9
U+696E.0	fTotalStrokes	 13
U+696E.0	fCangjie	DJKA
U+696E.0	fFourCorner	44960
U+696E.0	fHakka	  du3; cu3; chu3; shu3
U+696E.0	fCantonese	cyu5; cyu2
U+696E.0	fMandarin	chu3; du3; zhu1
U+696E.0	fEnglish	paper mulberry
U+696F.0	fUTF8	楯
U+696F.0	fCNS11643	2-4369
U+696F.0	fBig5	DDE3
U+696F.0	fGB	3-4246
U+696F.0	fR/S	 75.9
U+696F.0	fTotalStrokes	 13
U+696F.0	fCangjie	DHJU
U+696F.0	fFourCorner	42964
U+696F.0	fHakka	  tun3; dun3; shun3
U+696F.0	fCantonese	seon5; teon5 (also); ceon4
U+696F.0	fMandarin	shun3; dun4; chun1; chun3; shun4; sun1
U+696F.0	fEnglish	[1] the horizontal bar of a railing [2] same as $ (a shield used in battle)
U+6970.0	fUTF8	楰
U+6970.0	fCNS11643	2-3C62
U+6970.0	fBig5	D9BF
U+6970.0	fGB	3-4176
U+6970.0	fR/S	 75.9
U+6970.0	fTotalStrokes	 13
U+6970.0	fCangjie	DHXO
U+6970.0	fFourCorner	47937
U+6970.0	fHakka	 ji2; ri2
U+6970.0	fCantonese	jyu4
U+6970.0	fMandarin	yu2
U+6970.0	fEnglish	Yu (name of a plant)
U+6971.0	fUTF8	楱
U+6971.0	fCNS11643	2-4354
U+6971.0	fBig5	DDCE
U+6971.0	fGB	0-6928
U+6971.0	fR/S	 75.9
U+6971.0	fTotalStrokes	 13
U+6971.0	fCangjie	DQKK
U+6971.0	fFourCorner	45934
U+6971.0	fHakka	 zong1
U+6971.0	fCantonese	cau3; zeon1
U+6971.0	fMandarin	cou4; zou4; zhuang1
U+6972.0	fUTF8	楲
U+6972.0	fCNS11643	4-3D6C
U+6972.0	fGB	3-4237
U+6972.0	fR/S	 75.9
U+6972.0	fTotalStrokes	 13
U+6972.0	fCangjie	DIHV
U+6972.0	fFourCorner	43950
U+6972.0	fHakka	 wui1; wi1; vui1
U+6972.0	fCantonese	wai1
U+6972.0	fMandarin	wei1
U+6972.0	fEnglish	the young of a tiger
U+6973.0	fUTF8	楳
U+6973.0	fCNS11643	3-416D
U+6973.0	fGB	E-2E4E
U+6973.0	fR/S	  75.9
U+6973.0	fTotalStrokes	 13
U+6973.0	fCangjie	DTMD
U+6973.0	fFourCorner	44990
U+6973.0	fHakka	 moe2; moi2
U+6973.0	fCantonese	mui4
U+6973.0	fMandarin	mei2; mei3
U+6973.0	fEnglish	same as &#x6885;
U+6974.0	fUTF8	楴
U+6974.0	fCNS11643	2-436E
U+6974.0	fBig5	DDE8
U+6974.0	fGB	3-424A
U+6974.0	fR/S	 75.9
U+6974.0	fTotalStrokes	 13
U+6974.0	fCangjie	DYBB
U+6974.0	fFourCorner	40927
U+6974.0	fHakka	  ti5;  di5
U+6974.0	fCantonese	tai3; dai3; sai3; dai6
U+6974.0	fMandarin	di4; shi4
U+6974.0	fEnglish	a long pin for scratching the head
U+6975.0	fUTF8	極
U+6975.0	fCNS11643	1-644B
U+6975.0	fBig5	B7A5
U+6975.0	fGB	1-3C2B
U+6975.0	fR/S	 75.9
U+6975.0	fTotalStrokes	 13
U+6975.0	fCangjie	DMEM
U+6975.0	fFourCorner	41914
U+6975.0	fHakka	  kit8; git8
U+6975.0	fCantonese	gik6
U+6975.0	fMandarin	ji2; ji3
U+6975.0	fEnglish	[1] exhaust [2] extreme; extremely; highest; utmost; furthest; final; farthest; topmost [3] poles [4] reach at; arrive at
U+6976.0	fUTF8	楶
U+6976.0	fCNS11643	2-436B
U+6976.0	fBig5	DDE5
U+6976.0	fGB	5-446C
U+6976.0	fR/S	 75.9
U+6976.0	fTotalStrokes	 13
U+6976.0	fCangjie	IORD
U+6976.0	fFourCorner	37904
U+6976.0	fHakka	 ziet7
U+6976.0	fCantonese	zit3
U+6976.0	fMandarin	jie2
U+6977.0	fUTF8	楷
U+6977.0	fCNS11643	1-6448
U+6977.0	fBig5	B7A2
U+6977.0	fGB	0-3F2C
U+6977.0	fR/S	 75.9
U+6977.0	fTotalStrokes	 13
U+6977.0	fCangjie	DPPA
U+6977.0	fFourCorner	41961
U+6977.0	fHakka	  gai1;  kai3; giai1
U+6977.0	fCantonese	kaai2; gaai1
U+6977.0	fMandarin	kai3; jie1
U+6977.0	fEnglish	[1] regular; standard [2] model; norm [3] style of Chinese writing
U+6978.0	fUTF8	楸
U+6978.0	fCNS11643	2-4365
U+6978.0	fBig5	DDDF
U+6978.0	fGB	0-6931
U+6978.0	fR/S	 75.9
U+6978.0	fTotalStrokes	 13
U+6978.0	fCangjie	DHDF
U+6978.0	fFourCorner	49980
U+6978.0	fHakka	 ciu1
U+6978.0	fCantonese	cau1
U+6978.0	fMandarin	qiu1
U+6978.0	fEnglish	Chinese catalpa (Mallotus japonicus), a kind of hard wood used for making chessboards
U+6979.0	fUTF8	楹
U+6979.0	fCNS11643	1-6453
U+6979.0	fBig5	B7AD
U+6979.0	fGB	0-693A
U+6979.0	fR/S	 75.9
U+6979.0	fTotalStrokes	 13
U+6979.0	fCangjie	DNST
U+6979.0	fFourCorner	47917
U+6979.0	fHakka	  jin2; rin2; jin3
U+6979.0	fCantonese	jing4
U+6979.0	fMandarin	ying2
U+6979.0	fEnglish	 column; pillar numerical classifier
U+697A.0	fUTF8	楺
U+697A.0	fCNS11643	2-435C
U+697A.0	fBig5	DDD6
U+697A.0	fGB	5-4475
U+697A.0	fR/S	 75.9
U+697A.0	fTotalStrokes	 13
U+697A.0	fCangjie	DNHD
U+697A.0	fFourCorner	47994
U+697A.0	fHakka	 ngiu3;  ju2; riu2; nu2
U+697A.0	fCantonese	jau2; jau4
U+697A.0	fMandarin	rou2
U+697A.0	fEnglish	 bend wood by fire or steam
U+697B.0	fUTF8	楻
U+697B.0	fCNS11643	2-4379
U+697B.0	fBig5	DDF3
U+697B.0	fGB	5-4463
U+697B.0	fR/S	 75.9
U+697B.0	fTotalStrokes	 13
U+697B.0	fCangjie	DHAG
U+697B.0	fFourCorner	56914
U+697B.0	fHakka	 fong2
U+697B.0	fCantonese	hang4; wong4
U+697B.0	fMandarin	heng2
U+697C.0	fUTF8	楼
U+697C.0	fCNS11643	3-416A
U+697C.0	fGB	0-4225
U+697C.0	fR/S	 75.9
U+697C.0	fTotalStrokes	 13
U+697C.0	fCangjie	DFDV
U+697C.0	fFourCorner	49940
U+697C.0	fHakka	 leu2
U+697C.0	fCantonese	lau4; lau2 (also)
U+697C.0	fMandarin	lou2
U+697C.0	fEnglish	[1] non-classical form of &#x6A13; [2] simplified form of &#x6A13;
U+697D.0	fUTF8	楽
U+697D.0	fCNS11643	3-417B
U+697D.0	fGB	E-2E4F
U+697D.0	fR/S	 75.9
U+697D.0	fTotalStrokes	 13
U+697D.0	fCangjie	XIOD
U+697D.0	fFourCorner	32900
U+697D.0	fHakka	 ngok8; lok8; ngau5
U+697D.0	fCantonese	lok6; ngok6; lok3; ngau6
U+697D.0	fMandarin	le4; yue4; liao4; lo4
U+697D.0	fEnglish	happy, glad; enjoyable; music
U+697E.0	fUTF8	楾
U+697E.0	fGB	E-2E50
U+697E.0	fR/S	 75.9
U+697E.0	fTotalStrokes	 13
U+697E.0	fCangjie	DHAE
U+697E.0	fFourCorner	4693
U+697E.0	fCantonese	cyun4
U+697F.0	fUTF8	楿
U+697F.0	fCNS11643	F-4249
U+697F.0	fGB	E-2E51
U+697F.0	fR/S	 75.9
U+697F.0	fTotalStrokes	 13
U+697F.0	fCangjie	DHDA
U+697F.0	fFourCorner	4296
U+697F.0	fCantonese	hoeng1
U+697F.0	fMandarin	gui4
U+6980.0	fUTF8	榀
U+6980.0	fCNS11643	F-4247
U+6980.0	fGB	0-692F
U+6980.0	fR/S	 75.9
U+6980.0	fTotalStrokes	 13
U+6980.0	fCangjie	DRRR
U+6980.0	fFourCorner	4696
U+6980.0	fCantonese	ban2
U+6980.0	fMandarin	pin3
U+6981.0	fUTF8	榁
U+6981.0	fGB	E-2E52
U+6981.0	fR/S	 75.9
U+6981.0	fTotalStrokes	 13
U+6981.0	fCangjie	DJMG
U+6981.0	fCantonese	sat1
U+6982.0	fUTF8	概
U+6982.0	fCNS11643	1-644D
U+6982.0	fBig5	B7A7
U+6982.0	fGB	0-3845
U+6982.0	fR/S	 75.9
U+6982.0	fTotalStrokes	 13
U+6982.0	fCangjie	DAIU
U+6982.0	fFourCorner	41914
U+6982.0	fHakka	 koi3
U+6982.0	fCantonese	koi3; koi2 (also); goi3
U+6982.0	fMandarin	gai4; gui4; jie2
U+6982.0	fEnglish	generally; approximately
U+6983.0	fUTF8	榃
U+6983.0	fCNS11643	2-452C
U+6983.0	fBig5	DEC6
U+6983.0	fGB	3-5A6C
U+6983.0	fR/S	 75.9
U+6983.0	fTotalStrokes	 13
U+6983.0	fCangjie	XDDW
U+6983.0	fFourCorner	44609
U+6983.0	fCantonese	dam4; tam4 (also); taan4; taam4
U+6983.0	fMandarin	tan2
U+6983.0	fEnglish	raised path between fields
U+6984.0	fUTF8	榄
U+6984.0	fGB	0-692D
U+6984.0	fR/S	 75.9
U+6984.0	fTotalStrokes	 13
U+6984.0	fCangjie	DLIU
U+6984.0	fHakka	 lam3
U+6984.0	fCantonese	laam5; laam2 (also)
U+6984.0	fMandarin	lan3
U+6984.0	fEnglish	simplified form of &#x6B16;
U+6985.0	fUTF8	榅
U+6985.0	fCNS11643	3-4172
U+6985.0	fGB	3-423A
U+6985.0	fR/S	 75.9
U+6985.0	fTotalStrokes	 13
U+6985.0	fCangjie	DABT
U+6985.0	fFourCorner	46910
U+6985.0	fCantonese	wat1; wan1 (also)
U+6985.0	fMandarin	yun2
U+6985.0	fEnglish	pillar; cryptomeria
U+6986.0	fUTF8	榆
U+6986.0	fCNS11643	1-6454
U+6986.0	fBig5	B7AE
U+6986.0	fGB	0-535C
U+6986.0	fR/S	 75.9
U+6986.0	fTotalStrokes	 13
U+6986.0	fCangjie	DOMN
U+6986.0	fFourCorner	48921
U+6986.0	fHakka	 ji5
U+6986.0	fCantonese	jyu4
U+6986.0	fMandarin	yu2
U+6986.0	fEnglish	elm tree
U+6987.0	fUTF8	榇
U+6987.0	fGB	0-6934
U+6987.0	fR/S	 75.9
U+6987.0	fTotalStrokes	 13
U+6987.0	fCangjie	DYTD
U+6987.0	fHakka	 cang5
U+6987.0	fCantonese	can3
U+6987.0	fMandarin	chen4; qin1; guan4
U+6987.0	fEnglish	simplified form of &#x6AEC;
U+6988.0	fUTF8	榈
U+6988.0	fGB	0-6935
U+6988.0	fR/S	 75.9
U+6988.0	fTotalStrokes	 13
U+6988.0	fCangjie	DLSR
U+6988.0	fHakka	 li1; li2
U+6988.0	fCantonese	leoi4; leoi3 (also)
U+6988.0	fMandarin	luu2; l2
U+6988.0	fEnglish	simplified form of &#x6ADA;
U+6989.0	fUTF8	榉
U+6989.0	fGB	0-6937
U+6989.0	fR/S	 75.9
U+6989.0	fTotalStrokes	 13
U+6989.0	fCangjie	DFCQ
U+6989.0	fHakka	 gi3
U+6989.0	fCantonese	geoi2
U+6989.0	fMandarin	ju3
U+6989.0	fEnglish	simplified form of &#x6AF8;
U+698A.0	fUTF8	榊
U+698A.0	fCNS11643	F-4228
U+698A.0	fGB	E-2E53
U+698A.0	fR/S	 75.10; 75.9
U+698A.0	fTotalStrokes	 14
U+698A.0	fCangjie	DIFL
U+698A.0	fFourCorner	4592
U+698A.0	fCantonese	san4
U+698A.0	fEnglish	a sacred Shinto tree
U+698B.0	fUTF8	榋
U+698B.0	fGB	1-7D5E
U+698B.0	fR/S	 75.9
U+698B.0	fTotalStrokes	 13
U+698B.0	fCangjie	DSMS
U+698B.0	fCantonese	song1
U+698C.0	fUTF8	榌
U+698C.0	fCNS11643	F-423C
U+698C.0	fGB	1-7D46
U+698C.0	fR/S	 75.9
U+698C.0	fTotalStrokes	 13
U+698C.0	fCangjie	DNOO
U+698C.0	fCantonese	fei1
U+698C.0	fEnglish	 wooden ladder
U+698D.0	fUTF8	榍
U+698D.0	fCNS11643	2-4A70
U+698D.0	fBig5	E24A
U+698D.0	fGB	0-6947
U+698D.0	fR/S	 75.10
U+698D.0	fTotalStrokes	 14
U+698D.0	fCangjie	DSFB
U+698D.0	fFourCorner	47927
U+698D.0	fCantonese	sit3
U+698D.0	fMandarin	xie4; jia2; jie2
U+698E.0	fUTF8	榎
U+698E.0	fCNS11643	2-4A6E
U+698E.0	fBig5	E248
U+698E.0	fGB	5-4523
U+698E.0	fR/S	 75.10
U+698E.0	fTotalStrokes	 14
U+698E.0	fCangjie	DMUE
U+698E.0	fFourCorner	41947
U+698E.0	fHakka	 ga3; ga5
U+698E.0	fCantonese	gaa2
U+698E.0	fMandarin	jia3
U+698E.0	fEnglish	small evergreen shrub
U+698F.0	fUTF8	榏
U+698F.0	fCNS11643	3-473A
U+698F.0	fGB	3-426D
U+698F.0	fR/S	  75.10
U+698F.0	fTotalStrokes	 14
U+698F.0	fCangjie	DTCT
U+698F.0	fFourCorner	48910
U+698F.0	fCantonese	jik6
U+698F.0	fMandarin	yi4
U+6990.0	fUTF8	榐
U+6990.0	fCNS11643	2-4B26
U+6990.0	fBig5	E25E
U+6990.0	fGB	3-4272
U+6990.0	fR/S	 75.10
U+6990.0	fTotalStrokes	 14
U+6990.0	fCangjie	DSTV
U+6990.0	fFourCorner	47932
U+6990.0	fCantonese	zin2; zan6; cin2; nin6
U+6990.0	fMandarin	zhan3; nian3; zhen4
U+6991.0	fUTF8	榑
U+6991.0	fCNS11643	2-4A6C
U+6991.0	fBig5	E246
U+6991.0	fGB	5-4521
U+6991.0	fR/S	 75.10
U+6991.0	fTotalStrokes	 14
U+6991.0	fCangjie	DIBI
U+6991.0	fFourCorner	43942
U+6991.0	fHakka	 fu2
U+6991.0	fCantonese	fu4
U+6991.0	fMandarin	fu4; fu2; fu3
U+6991.0	fEnglish	a tree which is said to grow where the sun rises
U+6992.0	fUTF8	榒
U+6992.0	fCNS11643	3-473D
U+6992.0	fGB	E-2E54
U+6992.0	fR/S	  75.10
U+6992.0	fTotalStrokes	 14
U+6992.0	fCangjie	DNMM
U+6992.0	fFourCorner	47920
U+6992.0	fCantonese	nik1; lik1 (also)
U+6992.0	fMandarin	nai4; nuo4
U+6993.0	fUTF8	榓
U+6993.0	fCNS11643	2-4A7E
U+6993.0	fBig5	E258
U+6993.0	fGB	5-4535
U+6993.0	fR/S	 75.10
U+6993.0	fTotalStrokes	 14
U+6993.0	fCangjie	DPHT
U+6993.0	fFourCorner	43917
U+6993.0	fHakka	  met8; mit8
U+6993.0	fCantonese	mat6; mit6
U+6993.0	fMandarin	mi4
U+6993.0	fEnglish	eagle-wood tree or lign-aloes
U+6994.0	fUTF8	榔
U+6994.0	fCNS11643	1-6445
U+6994.0	fBig5	B77D
U+6994.0	fGB	0-4046
U+6994.0	fR/S	 75.9; 75.10
U+6994.0	fTotalStrokes	 13
U+6994.0	fCangjie	DIIL
U+6994.0	fFourCorner	47927
U+6994.0	fHakka	 long2; nong3; nong2
U+6994.0	fCantonese	long4
U+6994.0	fMandarin	lang2; lang3
U+6994.0	fEnglish	[1] a betel palm [2] a betel nut
U+6995.0	fUTF8	榕
U+6995.0	fCNS11643	1-6928
U+6995.0	fBig5	BA5F
U+6995.0	fGB	0-6945
U+6995.0	fR/S	 75.10
U+6995.0	fTotalStrokes	 14
U+6995.0	fCangjie	DJCR
U+6995.0	fFourCorner	43968
U+6995.0	fHakka	  jung2; rung2; jung3
U+6995.0	fCantonese	jung4
U+6995.0	fMandarin	rong2
U+6995.0	fEnglish	a banyan tree
U+6996.0	fUTF8	榖
U+6996.0	fCNS11643	2-4A68
U+6996.0	fBig5	E242
U+6996.0	fGB	3-4F31
U+6996.0	fR/S	 75.10
U+6996.0	fTotalStrokes	 14
U+6996.0	fCangjie	GDHNE
U+6996.0	fFourCorner	47947
U+6996.0	fHakka	 guk7; kwuk7
U+6996.0	fCantonese	guk1
U+6996.0	fMandarin	gu3; gou4
U+6996.0	fEnglish	Broussonetia papyrifera
U+6997.0	fUTF8	榗
U+6997.0	fCNS11643	2-4B25
U+6997.0	fBig5	E25D
U+6997.0	fGB	3-425B
U+6997.0	fR/S	 75.10
U+6997.0	fTotalStrokes	 14
U+6997.0	fCangjie	DMIA
U+6997.0	fFourCorner	41961
U+6997.0	fHakka	 zien5
U+6997.0	fCantonese	zin3; zeon3
U+6997.0	fMandarin	jian4; jin4; jian1
U+6997.0	fEnglish	the name of a kind of wood
U+6998.0	fUTF8	榘
U+6998.0	fCNS11643	3-4751
U+6998.0	fGB	0-6930
U+6998.0	fR/S	 75.9; 75.10
U+6998.0	fTotalStrokes	 15; 14; 13
U+6998.0	fCangjie	OSD
U+6998.0	fFourCorner	81900
U+6998.0	fCantonese	geoi2
U+6998.0	fMandarin	ju3; ju4
U+6998.0	fEnglish	carpenter's square; ruler; rule
U+6999.0	fUTF8	榙
U+6999.0	fCNS11643	2-4A6D
U+6999.0	fBig5	E247
U+6999.0	fGB	5-4450
U+6999.0	fR/S	 75.10
U+6999.0	fTotalStrokes	 14
U+6999.0	fCangjie	DTOR
U+6999.0	fFourCorner	44961
U+6999.0	fCantonese	daap6; daap3; hap6
U+6999.0	fMandarin	ta3; he2
U+699A.0	fUTF8	榚
U+699A.0	fCNS11643	2-4A7B
U+699A.0	fBig5	E255
U+699A.0	fGB	5-452F
U+699A.0	fR/S	 75.10
U+699A.0	fTotalStrokes	 14
U+699A.0	fCangjie	DTGF
U+699A.0	fFourCorner	48931
U+699A.0	fCantonese	jiu5
U+699A.0	fMandarin	yao3
U+699B.0	fUTF8	榛
U+699B.0	fCNS11643	1-692D
U+699B.0	fBig5	BA64
U+699B.0	fGB	0-693B
U+699B.0	fR/S	 75.10
U+699B.0	fTotalStrokes	 14
U+699B.0	fCangjie	DQKD
U+699B.0	fFourCorner	45994
U+699B.0	fHakka	 zin1;  cin2
U+699B.0	fCantonese	zeon1
U+699B.0	fMandarin	zhen1; jian1; qin2
U+699B.0	fEnglish	 hazelnut; hazel
U+699C.0	fUTF8	榜
U+699C.0	fCNS11643	1-6926
U+699C.0	fBig5	BA5D
U+699C.0	fGB	0-3071
U+699C.0	fR/S	 75.10
U+699C.0	fTotalStrokes	 14
U+699C.0	fCangjie	DYBS
U+699C.0	fFourCorner	40927
U+699C.0	fHakka	  bong3
U+699C.0	fCantonese	bong2; pong3; bong6 (also); paang4; pang4; bong3
U+699C.0	fMandarin	bang3; bang4; peng2; beng1; pang2
U+699C.0	fEnglish	 placard; list of successful exam; a publicly posted roll of successful examinees
U+699C.1	fUTF8	榜
U+699C.1	fCNS11643	1-6926
U+699C.1	fBig5	BA5D
U+699C.1	fGB	0-3071
U+699C.1	fR/S	 75.10
U+699C.1	fTotalStrokes	 14
U+699C.1	fCangjie	DYBS
U+699C.1	fFourCorner	40927
U+699C.1	fMandarin	beng4
U+699C.1	fEnglish	[1] oar [2] whip; rod
U+699D.0	fUTF8	榝
U+699D.0	fCNS11643	3-474E
U+699D.0	fGB	3-4138
U+699D.0	fR/S	 75.10
U+699D.0	fTotalStrokes	 14
U+699D.0	fCangjie	DKDE
U+699D.0	fFourCorner	47940
U+699D.0	fCantonese	saat3
U+699D.0	fMandarin	sha1
U+699E.0	fUTF8	榞
U+699E.0	fCNS11643	2-4B23
U+699E.0	fBig5	E25B
U+699E.0	fGB	5-4524
U+699E.0	fR/S	 75.10
U+699E.0	fTotalStrokes	 14
U+699E.0	fCangjie	DMHF
U+699E.0	fFourCorner	41996
U+699E.0	fCantonese	jyun4
U+699E.0	fMandarin	yuan2
U+699F.0	fUTF8	榟
U+699F.0	fCNS11643	3-4746
U+699F.0	fGB	E-2E55
U+699F.0	fR/S	 75.10
U+699F.0	fTotalStrokes	 14
U+699F.0	fCangjie	DJYJ
U+699F.0	fFourCorner	43940
U+699F.0	fHakka	 zih3; zu3; zii3
U+699F.0	fCantonese	zi2
U+699F.0	fMandarin	zi3
U+699F.0	fEnglish	same as &#x6893;
U+69A0.0	fUTF8	榠
U+69A0.0	fCNS11643	2-4A66
U+69A0.0	fBig5	E240
U+69A0.0	fGB	3-4271
U+69A0.0	fR/S	 75.10
U+69A0.0	fTotalStrokes	 14
U+69A0.0	fCangjie	DBAC
U+69A0.0	fFourCorner	47980
U+69A0.0	fCantonese	ming4
U+69A0.0	fMandarin	ming2; ming1; ming3
U+69A1.0	fUTF8	榡
U+69A1.0	fCNS11643	2-4B22
U+69A1.0	fBig5	E25A
U+69A1.0	fGB	5-4479
U+69A1.0	fR/S	 75.10
U+69A1.0	fTotalStrokes	 14
U+69A1.0	fCangjie	DQMF
U+69A1.0	fFourCorner	45993
U+69A1.0	fCantonese	sou3
U+69A1.0	fMandarin	su4
U+69A2.0	fUTF8	榢
U+69A2.0	fCNS11643	3-4747
U+69A2.0	fGB	5-4532
U+69A2.0	fR/S	 75.10
U+69A2.0	fTotalStrokes	 14
U+69A2.0	fCangjie	DJMO
U+69A2.0	fFourCorner	43930
U+69A2.0	fHakka	 ga5
U+69A2.0	fCantonese	gaa3; gaa2
U+69A2.0	fMandarin	jia4
U+69A2.0	fEnglish	same as &#x67B6;
U+69A3.0	fUTF8	榣
U+69A3.0	fCNS11643	1-6938
U+69A3.0	fBig5	BA6F
U+69A3.0	fGB	3-4268
U+69A3.0	fR/S	 75.10
U+69A3.0	fTotalStrokes	 14
U+69A3.0	fCangjie	DBOU
U+69A3.0	fFourCorner	47972
U+69A3.0	fCantonese	jiu4
U+69A3.0	fMandarin	yao2
U+69A4.0	fUTF8	榤
U+69A4.0	fCNS11643	2-4A77
U+69A4.0	fBig5	E251
U+69A4.0	fGB	5-452A
U+69A4.0	fR/S	 75.10
U+69A4.0	fTotalStrokes	 14
U+69A4.0	fCangjie	DNQD
U+69A4.0	fFourCorner	45994
U+69A4.0	fHakka	 kiet8; ket8
U+69A4.0	fCantonese	git6
U+69A4.0	fMandarin	jie2
U+69A4.0	fEnglish	perch for fowls roost on
U+69A5.0	fUTF8	榥
U+69A5.0	fCNS11643	2-4B29
U+69A5.0	fBig5	E261
U+69A5.0	fGB	3-4261
U+69A5.0	fR/S	 75.10
U+69A5.0	fTotalStrokes	 14
U+69A5.0	fCangjie	DAFU
U+69A5.0	fFourCorner	46911
U+69A5.0	fHakka	  fong3
U+69A5.0	fCantonese	fong2
U+69A5.0	fMandarin	huang3; huang4
U+69A5.0	fEnglish	screen
U+69A6.0	fUTF8	榦
U+69A6.0	fCNS11643	1-6936
U+69A6.0	fBig5	BA6D
U+69A6.0	fGB	E-2E56
U+69A6.0	fR/S	 75.10
U+69A6.0	fTotalStrokes	 14
U+69A6.0	fCangjie	JJOD
U+69A6.0	fFourCorner	48494
U+69A6.0	fHakka	  gon5
U+69A6.0	fCantonese	gon3; hon4
U+69A6.0	fMandarin	gan4; gan1
U+69A6.0	fEnglish	[1] tree trunk [2] posts, supports at ends of walls
U+69A6.1	fUTF8	榦
U+69A6.1	fCNS11643	1-6936
U+69A6.1	fBig5	BA6D
U+69A6.1	fGB	E-2E56
U+69A6.1	fR/S	 75.10
U+69A6.1	fTotalStrokes	 14
U+69A6.1	fCangjie	JJOD
U+69A6.1	fFourCorner	48494
U+69A6.1	fMandarin	han2
U+69A6.1	fEnglish	railings around a well
U+69A7.0	fUTF8	榧
U+69A7.0	fCNS11643	2-4A6F
U+69A7.0	fBig5	E249
U+69A7.0	fGB	0-693C
U+69A7.0	fR/S	 75.10
U+69A7.0	fTotalStrokes	 14
U+69A7.0	fCangjie	DSLY
U+69A7.0	fFourCorner	41911
U+69A7.0	fHakka	 fui1; fi1; fui5
U+69A7.0	fCantonese	fei2
U+69A7.0	fMandarin	fei3
U+69A7.0	fEnglish	a species of yew (Torreya nucifera)
U+69A8.0	fUTF8	榨
U+69A8.0	fCNS11643	1-6927
U+69A8.0	fBig5	BA5E
U+69A8.0	fGB	0-5525
U+69A8.0	fR/S	 75.10
U+69A8.0	fTotalStrokes	 14
U+69A8.0	fCangjie	DJCS
U+69A8.0	fFourCorner	43911
U+69A8.0	fHakka	 za5
U+69A8.0	fCantonese	zaa3
U+69A8.0	fMandarin	zha4; zhai4
U+69A8.0	fEnglish	 squeeze; press (for juice); extract
U+69A9.0	fUTF8	榩
U+69A9.0	fCNS11643	2-4A71
U+69A9.0	fBig5	E24B
U+69A9.0	fGB	5-4526
U+69A9.0	fR/S	 75.10
U+69A9.0	fTotalStrokes	 14
U+69A9.0	fCangjie	DYPK
U+69A9.0	fFourCorner	42947
U+69A9.0	fCantonese	kin4
U+69A9.0	fMandarin	qian2
U+69AA.0	fUTF8	榪
U+69AA.0	fCNS11643	2-4B21
U+69AA.0	fBig5	E259
U+69AA.0	fGB	1-683F
U+69AA.0	fR/S	 75.10
U+69AA.0	fTotalStrokes	 14
U+69AA.0	fCangjie	DSQF
U+69AA.0	fFourCorner	41927
U+69AA.0	fHakka	 ma5; ma6
U+69AA.0	fCantonese	maa5; maa6
U+69AA.0	fMandarin	ma4; ma3
U+69AA.0	fEnglish	headboard
U+69AB.0	fUTF8	榫
U+69AB.0	fCNS11643	1-6930
U+69AB.0	fBig5	BA67
U+69AB.0	fGB	0-693E
U+69AB.0	fR/S	 75.10
U+69AB.0	fTotalStrokes	 14
U+69AB.0	fCangjie	DOGJ
U+69AB.0	fFourCorner	4094
U+69AB.0	fHakka	  sun3
U+69AB.0	fCantonese	seon2
U+69AB.0	fMandarin	sun3
U+69AB.0	fEnglish	mortise and tenon
U+69AC.0	fUTF8	榬
U+69AC.0	fCNS11643	2-4A6A
U+69AC.0	fBig5	E244
U+69AC.0	fGB	3-4253
U+69AC.0	fR/S	 75.10
U+69AC.0	fTotalStrokes	 14
U+69AC.0	fCangjie	DGRV
U+69AC.0	fFourCorner	44932
U+69AC.0	fHakka	 jen2; ren2; jan2
U+69AC.0	fCantonese	jyun4
U+69AC.0	fMandarin	yuan2; wei2
U+69AC.0	fEnglish	a Chinese family name
U+69AD.0	fUTF8	榭
U+69AD.0	fCNS11643	1-6934
U+69AD.0	fBig5	BA6B
U+69AD.0	fGB	0-693F
U+69AD.0	fR/S	 75.10
U+69AD.0	fTotalStrokes	 14
U+69AD.0	fCangjie	DHHI
U+69AD.0	fFourCorner	44900
U+69AD.0	fHakka	 cia5; cia3; cia6; sia1
U+69AD.0	fCantonese	ze6
U+69AD.0	fMandarin	xie4
U+69AD.0	fEnglish	 kiosk; pavilion; arbor
U+69AE.0	fUTF8	榮
U+69AE.0	fCNS11643	1-692A
U+69AE.0	fBig5	BA61
U+69AE.0	fGB	1-4859
U+69AE.0	fR/S	 75.10
U+69AE.0	fTotalStrokes	 14
U+69AE.0	fCangjie	FFBD
U+69AE.0	fFourCorner	99904
U+69AE.0	fHakka	  jin2;  jung2; jung3; rung2; rin2
U+69AE.0	fCantonese	wing4
U+69AE.0	fMandarin	rong2; ying2
U+69AE.0	fEnglish	[1] glory; honor [2] luxuriant; lush; teeming; leafy
U+69AF.0	fUTF8	榯
U+69AF.0	fCNS11643	2-4A73
U+69AF.0	fBig5	E24D
U+69AF.0	fGB	3-4154
U+69AF.0	fR/S	 75.10
U+69AF.0	fTotalStrokes	 14
U+69AF.0	fCangjie	DAGI
U+69AF.0	fFourCorner	44941
U+69AF.0	fHakka	 shi2; sii2; she2
U+69AF.0	fCantonese	si4
U+69AF.0	fMandarin	shi2
U+69AF.0	fEnglish	erect; lofty
U+69B0.0	fUTF8	榰
U+69B0.0	fCNS11643	2-4A69
U+69B0.0	fBig5	E243
U+69B0.0	fGB	3-4254
U+69B0.0	fR/S	 75.10
U+69B0.0	fTotalStrokes	 14
U+69B0.0	fCangjie	DJPA
U+69B0.0	fFourCorner	44961
U+69B0.0	fHakka	 zhi1; zih1
U+69B0.0	fCantonese	zi1; zi2
U+69B0.0	fMandarin	zhi1
U+69B0.0	fEnglish	wooden plinth of a pillar
U+69B1.0	fUTF8	榱
U+69B1.0	fCNS11643	2-4A63
U+69B1.0	fBig5	E1FC
U+69B1.0	fGB	0-6941
U+69B1.0	fR/S	 75.10
U+69B1.0	fTotalStrokes	 14
U+69B1.0	fCangjie	DYWV
U+69B1.0	fFourCorner	40932
U+69B1.0	fHakka	  soi1; sui1
U+69B1.0	fCantonese	ceoi1; seoi1
U+69B1.0	fMandarin	cui1; shuai1
U+69B1.0	fEnglish	 rafter
U+69B2.0	fUTF8	榲
U+69B2.0	fCNS11643	3-473E
U+69B2.0	fGB	E-2E57
U+69B2.0	fR/S	 75.10
U+69B2.0	fTotalStrokes	 14
U+69B2.0	fCangjie	DWOT
U+69B2.0	fFourCorner	46910
U+69B2.0	fHakka	 wun1; vun1
U+69B2.0	fCantonese	wat1; wan1 (also)
U+69B2.0	fMandarin	yun2
U+69B2.0	fEnglish	pillar; cryptomeria
U+69B3.0	fUTF8	榳
U+69B3.0	fCNS11643	2-4A7D
U+69B3.0	fBig5	E257
U+69B3.0	fGB	5-446A
U+69B3.0	fR/S	 75.10
U+69B3.0	fTotalStrokes	 14
U+69B3.0	fCangjie	DING
U+69B3.0	fFourCorner	43941
U+69B3.0	fCantonese	ting4
U+69B3.0	fMandarin	ting2
U+69B4.0	fUTF8	榴
U+69B4.0	fCNS11643	1-6931
U+69B4.0	fBig5	BA68
U+69B4.0	fGB	0-4171
U+69B4.0	fR/S	 75.10
U+69B4.0	fTotalStrokes	 14
U+69B4.0	fCangjie	DHHW
U+69B4.0	fFourCorner	47962
U+69B4.0	fHakka	  liu2; liu3
U+69B4.0	fCantonese	lau4; lau2 (also)
U+69B4.0	fMandarin	liu2
U+69B4.0	fEnglish	the pomegranate
U+69B5.0	fUTF8	榵
U+69B5.0	fCNS11643	2-4B28
U+69B5.0	fBig5	E260
U+69B5.0	fGB	5-444C
U+69B5.0	fR/S	 75.10
U+69B5.0	fTotalStrokes	 14
U+69B5.0	fCangjie	DTSJ
U+69B5.0	fFourCorner	44941
U+69B5.0	fCantonese	jung4
U+69B5.0	fMandarin	rong2
U+69B6.0	fUTF8	榶
U+69B6.0	fCNS11643	2-4A64
U+69B6.0	fBig5	E1FD
U+69B6.0	fGB	3-426C
U+69B6.0	fR/S	 75.10
U+69B6.0	fTotalStrokes	 14
U+69B6.0	fCangjie	DILR
U+69B6.0	fFourCorner	40967
U+69B6.0	fCantonese	tong4
U+69B6.0	fMandarin	tang2
U+69B7.0	fUTF8	榷
U+69B7.0	fCNS11643	1-692E
U+69B7.0	fBig5	BA65
U+69B7.0	fGB	0-4836
U+69B7.0	fR/S	 75.10
U+69B7.0	fTotalStrokes	 14
U+69B7.0	fCangjie	DOBG
U+69B7.0	fFourCorner	44914
U+69B7.0	fHakka	  kok7
U+69B7.0	fCantonese	kok3; gok3
U+69B7.0	fMandarin	que4; jiao4; jue2
U+69B7.0	fEnglish	[1] toll; levy taxes [2] monopolise
U+69B8.0	fUTF8	榸
U+69B8.0	fCNS11643	4-4377
U+69B8.0	fGB	3-4252
U+69B8.0	fR/S	  75.10
U+69B8.0	fTotalStrokes	 14
U+69B8.0	fCangjie	DGWG
U+69B8.0	fFourCorner	46910
U+69B8.0	fHakka	 zai1
U+69B8.0	fCantonese	zaai1
U+69B8.0	fMandarin	zhai1
U+69B9.0	fUTF8	榹
U+69B9.0	fCNS11643	2-4A79
U+69B9.0	fBig5	E253
U+69B9.0	fGB	3-4266
U+69B9.0	fR/S	 75.10
U+69B9.0	fTotalStrokes	 14
U+69B9.0	fCangjie	DHYU
U+69B9.0	fFourCorner	42917
U+69B9.0	fHakka	 sii1; se1
U+69B9.0	fCantonese	si1
U+69B9.0	fMandarin	si1
U+69B9.0	fEnglish	the wild peach
U+69BA.0	fUTF8	榺
U+69BA.0	fCNS11643	4-4379
U+69BA.0	fGB	E-2E58
U+69BA.0	fR/S	  75.10
U+69BA.0	fTotalStrokes	 14
U+69BA.0	fCangjie	BFQD
U+69BA.0	fFourCorner	79290
U+69BA.0	fCantonese	sing3
U+69BA.0	fMandarin	sheng4
U+69BB.0	fUTF8	榻
U+69BB.0	fCNS11643	1-692F
U+69BB.0	fBig5	BA66
U+69BB.0	fGB	0-693D
U+69BB.0	fR/S	 75.10
U+69BB.0	fTotalStrokes	 14
U+69BB.0	fCangjie	DASM
U+69BB.0	fFourCorner	46927
U+69BB.0	fHakka	  tap7
U+69BB.0	fCantonese	taap3
U+69BB.0	fMandarin	ta4
U+69BB.0	fEnglish	 cot; couch; bed
U+69BC.0	fUTF8	榼
U+69BC.0	fCNS11643	2-4A6B
U+69BC.0	fBig5	E245
U+69BC.0	fGB	3-4256
U+69BC.0	fR/S	 75.10
U+69BC.0	fTotalStrokes	 14
U+69BC.0	fCangjie	DGIT
U+69BC.0	fFourCorner	44917
U+69BC.0	fHakka	  kap7; hap8
U+69BC.0	fCantonese	hap6; hap1
U+69BC.0	fMandarin	ke4; ke2
U+69BC.0	fEnglish	wine glass
U+69BD.0	fUTF8	榽
U+69BD.0	fCNS11643	2-4A76
U+69BD.0	fBig5	E250
U+69BD.0	fGB	3-426A
U+69BD.0	fR/S	 75.10
U+69BD.0	fTotalStrokes	 14
U+69BD.0	fCangjie	DBVK
U+69BD.0	fFourCorner	42934
U+69BD.0	fCantonese	hai4; haai4
U+69BD.0	fMandarin	xi1; xi4; xie2
U+69BE.0	fUTF8	榾
U+69BE.0	fCNS11643	2-4A72
U+69BE.0	fBig5	E24C
U+69BE.0	fGB	3-4240
U+69BE.0	fR/S	 75.10
U+69BE.0	fTotalStrokes	 14
U+69BE.0	fCangjie	DBBB
U+69BE.0	fFourCorner	47927
U+69BE.0	fHakka	  gut7; kut8; gwut7
U+69BE.0	fCantonese	gwat1; gat1 (also)
U+69BE.0	fMandarin	gu4; gu3
U+69BE.0	fEnglish	chopped pieces of wood
U+69BF.0	fUTF8	榿
U+69BF.0	fCNS11643	2-4A74
U+69BF.0	fBig5	E24E
U+69BF.0	fGB	1-6867
U+69BF.0	fR/S	 75.10
U+69BF.0	fTotalStrokes	 14
U+69BF.0	fCangjie	DUMT
U+69BF.0	fFourCorner	42918
U+69BF.0	fHakka	 ki1
U+69BF.0	fCantonese	kei1 (also); hei1 (also); hei2
U+69BF.0	fMandarin	qi1; ai2; kai3
U+69BF.0	fEnglish	alder
U+69C0.0	fUTF8	槀
U+69C0.0	fCNS11643	3-4743
U+69C0.0	fGB	E-2E59
U+69C0.0	fR/S	 75.10
U+69C0.0	fTotalStrokes	 14
U+69C0.0	fCangjie	YRBD
U+69C0.0	fFourCorner	00900
U+69C0.0	fHakka	 gau3
U+69C0.0	fCantonese	gou2
U+69C0.0	fMandarin	kao3; gao3
U+69C0.0	fEnglish	draft, manuscript
U+69C1.0	fUTF8	槁
U+69C1.0	fCNS11643	1-6929
U+69C1.0	fBig5	BA60
U+69C1.0	fGB	0-6942
U+69C1.0	fR/S	 75.10
U+69C1.0	fTotalStrokes	 14
U+69C1.0	fCangjie	DYRB
U+69C1.0	fFourCorner	40927
U+69C1.0	fHakka	 gau3; gau1
U+69C1.0	fCantonese	gou2
U+69C1.0	fMandarin	gao3; kao4; kao3; gao1
U+69C1.0	fEnglish	wither; withered; rotten; dead
U+69C2.0	fUTF8	槂
U+69C2.0	fCNS11643	2-4B27
U+69C2.0	fBig5	E25F
U+69C2.0	fGB	5-4375
U+69C2.0	fR/S	 75.10
U+69C2.0	fTotalStrokes	 14
U+69C2.0	fCangjie	DNDF
U+69C2.0	fFourCorner	42993
U+69C2.0	fCantonese	syun1; san1
U+69C2.0	fMandarin	sun1
U+69C3.0	fUTF8	槃
U+69C3.0	fCNS11643	1-6937
U+69C3.0	fBig5	BA6E
U+69C3.0	fGB	3-4267
U+69C3.0	fR/S	 75.10
U+69C3.0	fTotalStrokes	 14
U+69C3.0	fCangjie	HED
U+69C3.0	fFourCorner	27904
U+69C3.0	fHakka	 pan2; pan3
U+69C3.0	fCantonese	pun4
U+69C3.0	fMandarin	pan2
U+69C3.0	fEnglish	[1] wooden tray [2] great
U+69C4.0	fUTF8	槄
U+69C4.0	fCNS11643	2-4A75
U+69C4.0	fBig5	E24F
U+69C4.0	fGB	3-4269
U+69C4.0	fR/S	 75.10
U+69C4.0	fTotalStrokes	 14
U+69C4.0	fCangjie	DBHX
U+69C4.0	fFourCorner	42977
U+69C4.0	fHakka	  tau1; do1
U+69C4.0	fCantonese	tou1
U+69C4.0	fMandarin	tao1; tao3
U+69C4.0	fEnglish	name of a tree (identification doubtful)
U+69C5.0	fUTF8	槅
U+69C5.0	fCNS11643	3-473B
U+69C5.0	fGB	3-425E
U+69C5.0	fR/S	  75.10
U+69C5.0	fTotalStrokes	 14
U+69C5.0	fCangjie	DMRB
U+69C5.0	fFourCorner	41920
U+69C5.0	fHakka	  gak7
U+69C5.0	fCantonese	gaak3; hat6
U+69C5.0	fMandarin	ge2; he2
U+69C5.0	fEnglish	[1] a semicircular wooden collar at the end of the shafts of a large carriage; a yoke for harnessing animals [2] screen [3] shelf
U+69C6.0	fUTF8	槆
U+69C6.0	fCNS11643	2-4B2A
U+69C6.0	fBig5	E262
U+69C6.0	fGB	3-4234
U+69C6.0	fR/S	 75.10
U+69C6.0	fTotalStrokes	 14
U+69C6.0	fCangjie	DTPA
U+69C6.0	fFourCorner	44927
U+69C6.0	fCantonese	ceon1; seon1
U+69C6.0	fMandarin	xun2
U+69C7.0	fUTF8	槇
U+69C7.0	fCNS11643	4-4427
U+69C7.0	fGB	E-2E5A
U+69C7.0	fR/S	 75.10
U+69C7.0	fTotalStrokes	 14
U+69C7.0	fCangjie	DPBC
U+69C7.0	fFourCorner	41980
U+69C7.0	fHakka	  dien1; zhin1; zihn1
U+69C7.0	fCantonese	din1
U+69C7.0	fMandarin	zhen1; dian1; zhen3
U+69C7.0	fEnglish	 tip of a tree fallen tree
U+69C8.0	fUTF8	槈
U+69C8.0	fCNS11643	3-473C
U+69C8.0	fGB	5-4522
U+69C8.0	fR/S	  75.10
U+69C8.0	fTotalStrokes	 14
U+69C8.0	fCangjie	DMVI
U+69C8.0	fFourCorner	41940
U+69C8.0	fHakka	 neu3; ngieu3; neu5
U+69C8.0	fCantonese	nau6; lau6 (also)
U+69C8.0	fMandarin	nou4
U+69C8.0	fEnglish	button
U+69C9.0	fUTF8	槉
U+69C9.0	fCNS11643	2-4A65
U+69C9.0	fBig5	E1FE
U+69C9.0	fGB	5-452C
U+69C9.0	fR/S	 75.10
U+69C9.0	fTotalStrokes	 14
U+69C9.0	fCangjie	DKOK
U+69C9.0	fFourCorner	40934
U+69C9.0	fCantonese	zat6
U+69C9.0	fMandarin	ji2
U+69CA.0	fUTF8	槊
U+69CA.0	fCNS11643	2-4A7A
U+69CA.0	fBig5	E254
U+69CA.0	fGB	0-6943
U+69CA.0	fR/S	 75.10
U+69CA.0	fTotalStrokes	 14
U+69CA.0	fCangjie	TBD
U+69CA.0	fFourCorner	87904
U+69CA.0	fHakka	 sok7
U+69CA.0	fCantonese	sok3
U+69CA.0	fMandarin	shuo4
U+69CA.0	fEnglish	 spear; lance
U+69CB.0	fUTF8	構
U+69CB.0	fCNS11643	1-692C
U+69CB.0	fBig5	BA63
U+69CB.0	fGB	1-3939
U+69CB.0	fR/S	 75.10
U+69CB.0	fTotalStrokes	 14
U+69CB.0	fCangjie	DTTB
U+69CB.0	fFourCorner	45947
U+69CB.0	fHakka	 kieu5; giu5; geu5
U+69CB.0	fCantonese	kau3; gau3 (also)
U+69CB.0	fMandarin	gou4; jue2
U+69CB.0	fEnglish	 frame; form; build; establish; constitute
U+69CC.0	fUTF8	槌
U+69CC.0	fCNS11643	1-6935
U+69CC.0	fBig5	BA6C
U+69CC.0	fGB	0-6933
U+69CC.0	fR/S	 75.10
U+69CC.0	fTotalStrokes	 14
U+69CC.0	fCangjie	DYHR
U+69CC.0	fFourCorner	47937
U+69CC.0	fHakka	 cui2; ch'ui2; ch'ui3; chui2
U+69CC.0	fCantonese	ceoi4
U+69CC.0	fMandarin	chui2; zhui4; dui1
U+69CC.0	fEnglish	 hammer; mallet
U+69CD.0	fUTF8	槍
U+69CD.0	fCNS11643	1-6933
U+69CD.0	fBig5	BA6A
U+69CD.0	fGB	1-4739
U+69CD.0	fR/S	 75.10
U+69CD.0	fTotalStrokes	 14
U+69CD.0	fCangjie	DOIR
U+69CD.0	fFourCorner	48967
U+69CD.0	fHakka	  ciong1
U+69CD.0	fCantonese	coeng1; caang1
U+69CD.0	fMandarin	qiang1; cheng1
U+69CD.0	fEnglish	[1] spear; lance; javelin [2] gun; rifle; pistol
U+69CE.0	fUTF8	槎
U+69CE.0	fCNS11643	2-4A67
U+69CE.0	fBig5	E241
U+69CE.0	fGB	0-6936
U+69CE.0	fR/S	 75.10
U+69CE.0	fTotalStrokes	 14
U+69CE.0	fCangjie	DTQM
U+69CE.0	fFourCorner	48911
U+69CE.0	fHakka	  ca1; ca2; za5
U+69CE.0	fCantonese	caa4
U+69CE.0	fMandarin	cha2; zha4; cha1; zha1
U+69CE.0	fEnglish	[1] raft [2] hew; chop; cut
U+69CF.0	fUTF8	槏
U+69CF.0	fCNS11643	2-4A7C
U+69CF.0	fBig5	E256
U+69CF.0	fGB	3-426E
U+69CF.0	fR/S	 75.10
U+69CF.0	fTotalStrokes	 14
U+69CF.0	fCangjie	DTXC
U+69CF.0	fFourCorner	48937
U+69CF.0	fCantonese	haam2; lim4
U+69CF.0	fMandarin	qian3; lian2; xian2
U+69D0.0	fUTF8	槐
U+69D0.0	fCNS11643	1-6932
U+69D0.0	fBig5	BA69
U+69D0.0	fGB	0-3B31
U+69D0.0	fR/S	 75.10
U+69D0.0	fTotalStrokes	 14
U+69D0.0	fCangjie	DHUI
U+69D0.0	fFourCorner	46913
U+69D0.0	fHakka	  fai2; fai3
U+69D0.0	fCantonese	waai4; wai4
U+69D0.0	fMandarin	huai2; hui1
U+69D0.0	fEnglish	locust tree
U+69D1.0	fUTF8	槑
U+69D1.0	fCNS11643	3-4745
U+69D1.0	fGB	E-2E5B
U+69D1.0	fR/S	 75.10
U+69D1.0	fTotalStrokes	 14
U+69D1.0	fCangjie	RDRD
U+69D1.0	fFourCorner	66990
U+69D1.0	fCantonese	mui4
U+69D1.0	fMandarin	mei2
U+69D2.0	fUTF8	槒
U+69D2.0	fCNS11643	3-4748
U+69D2.0	fGB	E-2E5C
U+69D2.0	fR/S	 75.10
U+69D2.0	fTotalStrokes	 14
U+69D2.0	fCangjie	DYVW
U+69D2.0	fFourCorner	40960
U+69D2.0	fCantonese	cuk1
U+69D2.0	fMandarin	xu4; chu4
U+69D3.0	fUTF8	槓
U+69D3.0	fCNS11643	1-692B
U+69D3.0	fBig5	BA62
U+69D3.0	fGB	E-2E5D
U+69D3.0	fR/S	 75.10
U+69D3.0	fTotalStrokes	 14
U+69D3.0	fCangjie	DMBC
U+69D3.0	fFourCorner	41986
U+69D3.0	fHakka	 gong5; gung5; lung3
U+69D3.0	fCantonese	lung5; gung3; gong3
U+69D3.0	fMandarin	gang4
U+69D3.0	fEnglish	[1] lever; carrying pole [2] bar (sports) [3] sharpen (a knife) [4] argue; dispute [5] cross out
U+69D4.0	fUTF8	槔
U+69D4.0	fCNS11643	2-4A78
U+69D4.0	fBig5	E252
U+69D4.0	fGB	0-6940
U+69D4.0	fR/S	 75.10
U+69D4.0	fTotalStrokes	 14
U+69D4.0	fCangjie	DHAJ
U+69D4.0	fFourCorner	46943
U+69D4.0	fCantonese	gou1
U+69D4.0	fMandarin	gao1
U+69D4.0	fEnglish	 well sweep
U+69D5.0	fUTF8	槕
U+69D5.0	fCNS11643	3-474D
U+69D5.0	fGB	E-2E5E
U+69D5.0	fR/S	 75.10
U+69D5.0	fTotalStrokes	 14
U+69D5.0	fCangjie	DYAD
U+69D5.0	fFourCorner	41990
U+69D5.0	fCantonese	coek3; zoek3 (also)
U+69D5.0	fMandarin	zhuo2; zhuo1
U+69D6.0	fUTF8	槖
U+69D6.0	fCNS11643	3-4744
U+69D6.0	fGB	5-3721
U+69D6.0	fR/S	 75.10
U+69D6.0	fTotalStrokes	 14
U+69D6.0	fCangjie	GBMD
U+69D6.0	fFourCorner	40904
U+69D6.0	fCantonese	tok3
U+69D6.0	fMandarin	tuo4; tuo2; du4
U+69D6.0	fEnglish	a sack; a stamping sound
U+69D7.0	fUTF8	槗
U+69D7.0	fCNS11643	4-4428
U+69D7.0	fGB	E-2E5F
U+69D7.0	fR/S	 75.10
U+69D7.0	fTotalStrokes	 14
U+69D7.0	fCangjie	DKRB
U+69D7.0	fFourCorner	44920
U+69D7.0	fCantonese	kiu4
U+69D7.0	fMandarin	qiao2
U+69D8.0	fUTF8	様
U+69D8.0	fCNS11643	3-4C54
U+69D8.0	fGB	E-2E60
U+69D8.0	fR/S	 75.10
U+69D8.0	fTotalStrokes	 14; 15
U+69D8.0	fCangjie	DTGE
U+69D8.0	fFourCorner	48930
U+69D8.0	fCantonese	joeng4
U+69D8.0	fMandarin	yang4; xiang4
U+69D8.0	fEnglish	shape; form; pattern; style
U+69D9.0	fUTF8	槙
U+69D9.0	fCNS11643	2-4B24
U+69D9.0	fBig5	E25C
U+69D9.0	fGB	3-425C
U+69D9.0	fR/S	 75.10
U+69D9.0	fTotalStrokes	 14
U+69D9.0	fCangjie	DJBC
U+69D9.0	fFourCorner	44981
U+69D9.0	fCantonese	din1; can2; zan2
U+69D9.0	fMandarin	dian1; dian4; zhen1; zhen3
U+69D9.0	fEnglish	a twig; an ornamental evergreen
U+69DA.0	fUTF8	槚
U+69DA.0	fGB	8-7C26
U+69DA.0	fR/S	 75.10; 75.13
U+69DA.0	fTotalStrokes	 14
U+69DA.0	fCangjie	DMWO
U+69DA.0	fCantonese	gaa2
U+69DA.0	fMandarin	jia3
U+69DA.0	fEnglish	small evergreen shrub
U+69DB.0	fUTF8	槛
U+69DB.0	fGB	0-3C77
U+69DB.0	fR/S	 75.10
U+69DB.0	fTotalStrokes	 14
U+69DB.0	fCangjie	DLIT
U+69DB.0	fHakka	 gam5
U+69DB.0	fCantonese	laam6; haam5; kaam5 (also)
U+69DB.0	fMandarin	jian4; kan3
U+69DB.0	fEnglish	simplified form of &#x6ABB;
U+69DC.0	fUTF8	槜
U+69DC.0	fGB	3-4264
U+69DC.0	fR/S	 75.10
U+69DC.0	fTotalStrokes	 14
U+69DC.0	fCangjie	XDOGS
U+69DC.0	fCantonese	zeoi3
U+69DC.0	fMandarin	zui4
U+69DD.0	fUTF8	槝
U+69DD.0	fGB	E-2E61
U+69DD.0	fR/S	 75.10
U+69DD.0	fTotalStrokes	 14
U+69DD.0	fCangjie	DHAU
U+69DD.0	fCantonese	dou2
U+69DE.0	fUTF8	槞
U+69DE.0	fGB	E-2E62
U+69DE.0	fR/S	 75.10
U+69DE.0	fTotalStrokes	 14
U+69DE.0	fCangjie	XDYTU
U+69DE.0	fCantonese	lung4
U+69DE.0	fMandarin	long2
U+69DF.0	fUTF8	槟
U+69DF.0	fGB	0-6944
U+69DF.0	fR/S	 75.10
U+69DF.0	fTotalStrokes	 14
U+69DF.0	fCangjie	DJOC
U+69DF.0	fHakka	 bin1
U+69DF.0	fCantonese	ban1
U+69DF.0	fMandarin	bin1; bing1
U+69DF.0	fEnglish	simplified form of &#x6AB3;
U+69E0.0	fUTF8	槠
U+69E0.0	fGB	0-6946
U+69E0.0	fR/S	 75.10
U+69E0.0	fTotalStrokes	 14
U+69E0.0	fCangjie	DIVA
U+69E0.0	fHakka	 zhu1
U+69E0.0	fCantonese	zyu1
U+69E0.0	fMandarin	zhu1
U+69E0.0	fEnglish	simplified form of &#x6AE7;
U+69E1.0	fUTF8	槡
U+69E1.0	fCNS11643	3-474C
U+69E1.0	fGB	E-2E63
U+69E1.0	fR/S	  75.10
U+69E1.0	fTotalStrokes	 14
U+69E1.0	fCangjie	DEED
U+69E1.0	fFourCorner	47990
U+69E1.0	fCantonese	song1
U+69E2.0	fUTF8	槢
U+69E2.0	fCNS11643	2-507C
U+69E2.0	fBig5	E5D5
U+69E2.0	fGB	3-432C
U+69E2.0	fR/S	 75.11
U+69E2.0	fTotalStrokes	 15
U+69E2.0	fCangjie	DSMA
U+69E2.0	fFourCorner	47962
U+69E2.0	fHakka	  sip8
U+69E2.0	fCantonese	zaap6
U+69E2.0	fMandarin	xi2; die2
U+69E2.0	fEnglish	a hard wood
U+69E3.0	fUTF8	槣
U+69E3.0	fCNS11643	3-4742
U+69E3.0	fGB	5-454C
U+69E3.0	fR/S	 75.11
U+69E3.0	fTotalStrokes	 15; 14
U+69E3.0	fCangjie	DJKR
U+69E3.0	fFourCorner	43920
U+69E3.0	fHakka	 ki2
U+69E3.0	fCantonese	gei1
U+69E3.0	fMandarin	ji1; qi3; gui1
U+69E4.0	fUTF8	槤
U+69E4.0	fCNS11643	2-5078
U+69E4.0	fBig5	E5D1
U+69E4.0	fGB	5-437B
U+69E4.0	fR/S	 75.11
U+69E4.0	fTotalStrokes	 15
U+69E4.0	fCangjie	DYJJ
U+69E4.0	fFourCorner	45930
U+69E4.0	fHakka	  lien2; lien3; len2
U+69E4.0	fCantonese	lin4; lin2 (also)
U+69E4.0	fMandarin	lian2; lian3
U+69E4.0	fEnglish	[1] side apartment [2] bar on a door
U+69E5.0	fUTF8	槥
U+69E5.0	fCNS11643	2-5074
U+69E5.0	fBig5	E5CD
U+69E5.0	fGB	3-4275
U+69E5.0	fR/S	 75.11
U+69E5.0	fTotalStrokes	 15
U+69E5.0	fCangjie	DQJM
U+69E5.0	fFourCorner	45971
U+69E5.0	fHakka	 sui5; fi5; fui6; wui6; wi5; sui6; fui5; vui5
U+69E5.0	fCantonese	seoi6; wai6 (also)
U+69E5.0	fMandarin	hui4; sui4; wei4
U+69E5.0	fEnglish	 small coffin
U+69E6.0	fUTF8	槦
U+69E6.0	fCNS11643	2-512A
U+69E6.0	fBig5	E5E1
U+69E6.0	fGB	3-4326
U+69E6.0	fR/S	 75.11
U+69E6.0	fTotalStrokes	 15
U+69E6.0	fCangjie	DILB
U+69E6.0	fFourCorner	40927
U+69E6.0	fCantonese	jung4
U+69E6.0	fMandarin	yong2; tong3
U+69E7.0	fUTF8	槧
U+69E7.0	fCNS11643	2-5127
U+69E7.0	fBig5	E5DE
U+69E7.0	fGB	1-687D
U+69E7.0	fR/S	 75.11
U+69E7.0	fTotalStrokes	 15
U+69E7.0	fCangjie	JLD
U+69E7.0	fFourCorner	52904
U+69E7.0	fHakka	 ciam3;  ciam5; ciam6
U+69E7.0	fCantonese	cim3; cim5
U+69E7.0	fMandarin	qian4; jian4; qian1
U+69E7.0	fEnglish	[1] wooden tablet for writing [2] version; edition of a book
U+69E8.0	fUTF8	槨
U+69E8.0	fCNS11643	1-6D37
U+69E8.0	fBig5	BCCD
U+69E8.0	fGB	E-2E64
U+69E8.0	fR/S	 75.11
U+69E8.0	fTotalStrokes	 15
U+69E8.0	fCangjie	DYDL
U+69E8.0	fFourCorner	47927
U+69E8.0	fHakka	  gok7; gwok7
U+69E8.0	fCantonese	gwok3; gok3 (also)
U+69E8.0	fMandarin	guo3
U+69E8.0	fEnglish	outer-coffin
U+69E9.0	fUTF8	槩
U+69E9.0	fCNS11643	3-4C64
U+69E9.0	fGB	E-2E65
U+69E9.0	fR/S	 75.10; 75.11
U+69E9.0	fTotalStrokes	 15; 14
U+69E9.0	fCangjie	HUD
U+69E9.0	fFourCorner	21900
U+69E9.0	fCantonese	koi3
U+69E9.0	fMandarin	gai4
U+69E9.0	fEnglish	generally; approximately
U+69EA.0	fUTF8	槪
U+69EA.0	fCNS11643	3-4C60
U+69EA.0	fGB	E-2E66
U+69EA.0	fR/S	 75.11
U+69EA.0	fTotalStrokes	 15
U+69EA.0	fCangjie	DHPU
U+69EA.0	fFourCorner	41910
U+69EA.0	fHakka	  kai3;  koi3; ge5; gi5
U+69EA.0	fCantonese	koi3; goi3
U+69EA.0	fMandarin	gai4
U+69EA.0	fEnglish	[1] general; generally; overall; roughly [2] without exception; categorically [3] the manner of carrying oneself; deportment
U+69EB.0	fUTF8	槫
U+69EB.0	fCNS11643	2-512E
U+69EB.0	fBig5	E5E5
U+69EB.0	fGB	3-4059
U+69EB.0	fR/S	 75.11
U+69EB.0	fTotalStrokes	 15
U+69EB.0	fCangjie	DJII
U+69EB.0	fFourCorner	45943
U+69EB.0	fCantonese	tyun4
U+69EB.0	fMandarin	tuan2; chuan2
U+69EC.0	fUTF8	槬
U+69EC.0	fCNS11643	2-507B
U+69EC.0	fBig5	E5D4
U+69EC.0	fGB	3-427C
U+69EC.0	fR/S	 75.11
U+69EC.0	fTotalStrokes	 15
U+69EC.0	fCangjie	DKSO
U+69EC.0	fFourCorner	42930
U+69EC.0	fCantonese	waa6
U+69EC.0	fMandarin	hua4
U+69ED.0	fUTF8	槭
U+69ED.0	fCNS11643	1-6D42
U+69ED.0	fBig5	BCD8
U+69ED.0	fGB	0-694A
U+69ED.0	fR/S	 75.11
U+69ED.0	fTotalStrokes	 15
U+69ED.0	fCangjie	DIHF
U+69ED.0	fFourCorner	43950
U+69ED.0	fHakka	 cuk7; cuk8
U+69ED.0	fCantonese	cik1; zuk1; zik1
U+69ED.0	fMandarin	qi1; cu4; se4; zu2; zhou1
U+69ED.0	fEnglish	a species of maple (Acer palmatum)
U+69EE.0	fUTF8	槮
U+69EE.0	fCNS11643	2-5124
U+69EE.0	fBig5	E5DB
U+69EE.0	fGB	3-422E
U+69EE.0	fR/S	 75.11
U+69EE.0	fTotalStrokes	 15
U+69EE.0	fCangjie	DIIH
U+69EE.0	fFourCorner	43922
U+69EE.0	fHakka	  sem1
U+69EE.0	fCantonese	sam1; sam2; sam6
U+69EE.0	fMandarin	sen1; san3; shen1; sao1; shan1; shen3; shen4
U+69EE.0	fEnglish	[1] leafless, as trees [2] fish-trap
U+69EF.0	fUTF8	槯
U+69EF.0	fCNS11643	3-4C4F
U+69EF.0	fGB	3-4322
U+69EF.0	fR/S	  75.11
U+69EF.0	fTotalStrokes	 15
U+69EF.0	fCangjie	DUOG
U+69EF.0	fFourCorner	42910
U+69EF.0	fCantonese	ceoi4
U+69EF.0	fMandarin	cui1
U+69F0.0	fUTF8	槰
U+69F0.0	fCNS11643	3-4C51
U+69F0.0	fGB	5-4529
U+69F0.0	fR/S	  75.11
U+69F0.0	fTotalStrokes	 15
U+69F0.0	fCangjie	DYHJ
U+69F0.0	fFourCorner	47930
U+69F0.0	fCantonese	bung6
U+69F0.0	fMandarin	beng4
U+69F1.0	fUTF8	槱
U+69F1.0	fCNS11643	2-5077
U+69F1.0	fBig5	E5D0
U+69F1.0	fGB	3-427B
U+69F1.0	fR/S	 75.11
U+69F1.0	fTotalStrokes	 15
U+69F1.0	fCangjie	DMWF
U+69F1.0	fFourCorner	41989
U+69F1.0	fHakka	 ju3; riu3
U+69F1.0	fCantonese	jau5
U+69F1.0	fMandarin	you3; you2; you4
U+69F1.0	fEnglish	[1] an ancient rite of building fires in worship [2] supply firewood for building fires; firewood for sacrifice; burn piled firewood
U+69F2.0	fUTF8	槲
U+69F2.0	fCNS11643	2-5123
U+69F2.0	fBig5	E5DA
U+69F2.0	fGB	0-694E
U+69F2.0	fR/S	 75.11
U+69F2.0	fTotalStrokes	 15
U+69F2.0	fCangjie	DNBJ
U+69F2.0	fFourCorner	44900
U+69F2.0	fHakka	 fuk8
U+69F2.0	fCantonese	huk6
U+69F2.0	fMandarin	hu2
U+69F2.0	fEnglish	a species of oak (Quercus dentata)
U+69F3.0	fUTF8	槳
U+69F3.0	fCNS11643	1-6D3F
U+69F3.0	fBig5	BCD5
U+69F3.0	fGB	1-3D30
U+69F3.0	fR/S	 75.11
U+69F3.0	fTotalStrokes	 15
U+69F3.0	fCangjie	VID
U+69F3.0	fFourCorner	27904
U+69F3.0	fHakka	  ziong3
U+69F3.0	fCantonese	zoeng2
U+69F3.0	fMandarin	jiang3; jiang1
U+69F3.0	fEnglish	 oar; paddle
U+69F4.0	fUTF8	槴
U+69F4.0	fCNS11643	2-5137
U+69F4.0	fBig5	E5EE
U+69F4.0	fGB	5-454E
U+69F4.0	fR/S	 75.11
U+69F4.0	fTotalStrokes	 15
U+69F4.0	fCangjie	DHSU
U+69F4.0	fFourCorner	42917
U+69F4.0	fHakka	 fu5
U+69F4.0	fCantonese	wu6
U+69F4.0	fMandarin	hu4
U+69F5.0	fUTF8	槵
U+69F5.0	fCNS11643	4-4A3D
U+69F5.0	fGB	3-4321
U+69F5.0	fR/S	  75.11
U+69F5.0	fTotalStrokes	 15
U+69F5.0	fCangjie	DLLP
U+69F5.0	fFourCorner	45930
U+69F5.0	fHakka	 fam6;  fam5
U+69F5.0	fCantonese	waan6
U+69F5.0	fMandarin	huan4
U+69F5.0	fEnglish	a kind of soap-berry (Sapindus)
U+69F6.0	fUTF8	槶
U+69F6.0	fCNS11643	2-5134
U+69F6.0	fBig5	E5EB
U+69F6.0	fGB	5-443A
U+69F6.0	fR/S	 75.11
U+69F6.0	fTotalStrokes	 15
U+69F6.0	fCangjie	DWIM
U+69F6.0	fFourCorner	46900
U+69F6.0	fCantonese	gui3; gwui3 (also); kui3
U+69F6.0	fMandarin	kui4; gui1
U+69F7.0	fUTF8	槷
U+69F7.0	fCNS11643	2-5126
U+69F7.0	fBig5	E5DD
U+69F7.0	fGB	3-4277
U+69F7.0	fR/S	 75.11
U+69F7.0	fTotalStrokes	 15
U+69F7.0	fCangjie	GID
U+69F7.0	fFourCorner	55904
U+69F7.0	fHakka	 ngiet7; nget7
U+69F7.0	fCantonese	jit6; nip6 (also); lip6 (also); sit3
U+69F7.0	fMandarin	yi4; nie4; xie4
U+69F7.0	fEnglish	[1] stake; post [2] dangerous
U+69F8.0	fUTF8	槸
U+69F8.0	fCNS11643	2-5075
U+69F8.0	fBig5	E5CE
U+69F8.0	fGB	5-4538
U+69F8.0	fR/S	 75.11
U+69F8.0	fTotalStrokes	 15
U+69F8.0	fCangjie	DGGI
U+69F8.0	fFourCorner	45917
U+69F8.0	fHakka	 ngi5; ngi6
U+69F8.0	fCantonese	ngai6; ai6 (also); ji6
U+69F8.0	fMandarin	nie4; yi4
U+69F8.0	fEnglish	branches rubbing against each other in the wind
U+69F9.0	fUTF8	槹
U+69F9.0	fCNS11643	3-4C5F
U+69F9.0	fGB	E-2E67
U+69F9.0	fR/S	  75.11
U+69F9.0	fTotalStrokes	 15
U+69F9.0	fCangjie	DHAJ
U+69F9.0	fFourCorner	46940
U+69F9.0	fCantonese	gou1
U+69F9.0	fMandarin	gao1
U+69F9.0	fEnglish	 spar well-sweep
U+69FA.0	fUTF8	槺
U+69FA.0	fCNS11643	3-4C57
U+69FA.0	fGB	5-4549
U+69FA.0	fR/S	  75.11
U+69FA.0	fTotalStrokes	 15
U+69FA.0	fCangjie	DILE
U+69FA.0	fFourCorner	40932
U+69FA.0	fCantonese	hong1
U+69FA.0	fMandarin	kang1
U+69FB.0	fUTF8	槻
U+69FB.0	fCNS11643	2-512B
U+69FB.0	fBig5	E5E2
U+69FB.0	fGB	3-416A
U+69FB.0	fR/S	 75.11
U+69FB.0	fTotalStrokes	 15
U+69FB.0	fCangjie	DQOU
U+69FB.0	fFourCorner	46910
U+69FB.0	fHakka	 gui1; kwui1
U+69FB.0	fCantonese	kwai1
U+69FB.0	fMandarin	gui1
U+69FB.0	fEnglish	Zelkova tree
U+69FC.0	fUTF8	槼
U+69FC.0	fCNS11643	2-512D
U+69FC.0	fBig5	E5E4
U+69FC.0	fGB	E-2E68
U+69FC.0	fR/S	 75.11
U+69FC.0	fTotalStrokes	 15
U+69FC.0	fCangjie	QUD
U+69FC.0	fFourCorner	56904
U+69FC.0	fCantonese	kwai1
U+69FC.0	fMandarin	gui1
U+69FC.0	fEnglish	rules; regulations; customs; law
U+69FD.0	fUTF8	槽
U+69FD.0	fCNS11643	1-6D3B
U+69FD.0	fBig5	BCD1
U+69FD.0	fGB	0-325B
U+69FD.0	fR/S	 75.11
U+69FD.0	fTotalStrokes	 15
U+69FD.0	fCangjie	DTWA
U+69FD.0	fFourCorner	45966
U+69FD.0	fHakka	  cau2;  co2
U+69FD.0	fCantonese	cou4
U+69FD.0	fMandarin	cao2; zao1
U+69FD.0	fEnglish	[1] trough; flume; chute [2] manger [3] used in combination
U+69FE.0	fUTF8	槾
U+69FE.0	fCNS11643	2-5121
U+69FE.0	fBig5	E5D8
U+69FE.0	fGB	5-4540
U+69FE.0	fR/S	 75.11
U+69FE.0	fTotalStrokes	 15
U+69FE.0	fCangjie	DAWE
U+69FE.0	fFourCorner	46947
U+69FE.0	fHakka	 man2; man5; man6
U+69FE.0	fCantonese	maan6; maan4; maan3
U+69FE.0	fMandarin	man2; man4; wan4
U+69FE.0	fEnglish	paste wall with dirt
U+69FF.0	fUTF8	槿
U+69FF.0	fCNS11643	2-507A
U+69FF.0	fBig5	E5D3
U+69FF.0	fGB	0-6948
U+69FF.0	fR/S	 75.11
U+69FF.0	fTotalStrokes	 15
U+69FF.0	fCangjie	DTLM
U+69FF.0	fFourCorner	44914
U+69FF.0	fHakka	 giun3
U+69FF.0	fCantonese	gan2
U+69FF.0	fMandarin	jin3; jin1; qin2
U+69FF.0	fEnglish	[1] hibiscus [2] used in combination
U+6A00.0	fUTF8	樀
U+6A00.0	fCNS11643	2-5071
U+6A00.0	fBig5	E5CA
U+6A00.0	fGB	3-4327
U+6A00.0	fR/S	 75.11
U+6A00.0	fTotalStrokes	 15
U+6A00.0	fCangjie	DYCB
U+6A00.0	fFourCorner	40927
U+6A00.0	fHakka	  dit7
U+6A00.0	fCantonese	dik1
U+6A00.0	fMandarin	di4; zhe2
U+6A00.0	fEnglish	short rafters which support projecting eaves
U+6A01.0	fUTF8	樁
U+6A01.0	fCNS11643	1-6D38
U+6A01.0	fBig5	BCCE
U+6A01.0	fGB	1-572E
U+6A01.0	fR/S	 75.11
U+6A01.0	fTotalStrokes	 15
U+6A01.0	fCangjie	DQKX
U+6A01.0	fFourCorner	45977
U+6A01.0	fHakka	 zhung1; zung1; zong1
U+6A01.0	fCantonese	zong1
U+6A01.0	fMandarin	zhuang1; chong1
U+6A01.0	fEnglish	[1] stake; post; pile [2] numerical classifier for affairs or matters
U+6A02.0	fUTF8	樂
U+6A02.0	fCNS11643	1-6D40
U+6A02.0	fBig5	BCD6
U+6A02.0	fGB	1-4056
U+6A02.0	fR/S	 75.11
U+6A02.0	fTotalStrokes	 15
U+6A02.0	fCangjie	VID
U+6A02.0	fFourCorner	22904
U+6A02.0	fHakka	  lok8;  ngok8;  ngau5; lok7; ngau6; ngau3; liau2
U+6A02.0	fCantonese	lok6; ngok6; ngaau6; ok6 (also); aau6 (also); ngaau4 (also); aau4 (also); lok3; ngau6
U+6A02.0	fMandarin	le4; liao4; lo4; luo4; liao2
U+6A02.0	fEnglish	[1] music [2] a Chinese family name
U+6A02.1	fUTF8	樂
U+6A02.1	fCNS11643	1-6D40
U+6A02.1	fBig5	BCD6
U+6A02.1	fGB	1-4056
U+6A02.1	fR/S	 75.11
U+6A02.1	fTotalStrokes	 15
U+6A02.1	fCangjie	VID
U+6A02.1	fFourCorner	22904
U+6A02.1	fMandarin	yue4
U+6A02.1	fEnglish	[1] happy; glad; joyful; joyous; cheerful; elated; content; delighted; pleased [2] pleasant; agreeable; enjoyable; pleasing; comfortable
U+6A02.2	fUTF8	樂
U+6A02.2	fCNS11643	1-6D40
U+6A02.2	fBig5	BCD6
U+6A02.2	fGB	1-4056
U+6A02.2	fR/S	 75.11
U+6A02.2	fTotalStrokes	 15
U+6A02.2	fCangjie	VID
U+6A02.2	fFourCorner	22904
U+6A02.2	fMandarin	yao4
U+6A02.2	fEnglish	 love; be fond of; delight in
U+6A03.0	fUTF8	樃
U+6A03.0	fCNS11643	4-4A48
U+6A03.0	fGB	E-2E69
U+6A03.0	fR/S	 75.11
U+6A03.0	fTotalStrokes	 15
U+6A03.0	fCangjie	DIIB
U+6A03.0	fFourCorner	47920
U+6A03.0	fCantonese	long5
U+6A03.0	fMandarin	lang2; lang3
U+6A04.0	fUTF8	樄
U+6A04.0	fCNS11643	2-5130
U+6A04.0	fBig5	E5E7
U+6A04.0	fGB	5-442F
U+6A04.0	fR/S	 75.11
U+6A04.0	fTotalStrokes	 15
U+6A04.0	fCangjie	DNLW
U+6A04.0	fFourCorner	45996
U+6A04.0	fCantonese	can4
U+6A04.0	fMandarin	chen2
U+6A05.0	fUTF8	樅
U+6A05.0	fCNS11643	1-6D41
U+6A05.0	fBig5	BCD7
U+6A05.0	fGB	1-6848
U+6A05.0	fR/S	 75.11
U+6A05.0	fTotalStrokes	 15
U+6A05.0	fCangjie	DHOO
U+6A05.0	fFourCorner	48981
U+6A05.0	fHakka	  ciung2;  ziung1
U+6A05.0	fCantonese	cung1; zung1
U+6A05.0	fMandarin	cong1; zong1
U+6A05.0	fEnglish	 fir; fir tree (Abies firma)
U+6A06.0	fUTF8	樆
U+6A06.0	fCNS11643	2-5072
U+6A06.0	fBig5	E5CB
U+6A06.0	fGB	3-426B
U+6A06.0	fR/S	 75.11
U+6A06.0	fTotalStrokes	 15
U+6A06.0	fCangjie	DYUB
U+6A06.0	fFourCorner	50027
U+6A06.0	fHakka	  li2
U+6A06.0	fCantonese	lei4; ci1
U+6A06.0	fMandarin	li2; chi1
U+6A06.0	fEnglish	wild pears (Pyrus sinensis)
U+6A07.0	fUTF8	樇
U+6A07.0	fCNS11643	2-5136
U+6A07.0	fBig5	E5ED
U+6A07.0	fGB	5-4527
U+6A07.0	fR/S	 75.11
U+6A07.0	fTotalStrokes	 15
U+6A07.0	fCangjie	DOLB
U+6A07.0	fFourCorner	47927
U+6A07.0	fCantonese	sau1
U+6A07.0	fMandarin	xiu1
U+6A08.0	fUTF8	樈
U+6A08.0	fCNS11643	2-5129
U+6A08.0	fBig5	E5E0
U+6A08.0	fGB	5-454B
U+6A08.0	fR/S	 75.11
U+6A08.0	fTotalStrokes	 15
U+6A08.0	fCangjie	DYTU
U+6A08.0	fFourCorner	40916
U+6A08.0	fCantonese	king4
U+6A08.0	fMandarin	qing2
U+6A09.0	fUTF8	樉
U+6A09.0	fCNS11643	2-512F
U+6A09.0	fBig5	E5E6
U+6A09.0	fGB	3-427D
U+6A09.0	fR/S	 75.11
U+6A09.0	fTotalStrokes	 15
U+6A09.0	fCangjie	DKKK
U+6A09.0	fFourCorner	44934
U+6A09.0	fCantonese	song2
U+6A09.0	fMandarin	shuang3
U+6A0A.0	fUTF8	樊
U+6A0A.0	fCNS11643	1-6D3E
U+6A0A.0	fBig5	BCD4
U+6A0A.0	fGB	0-372E
U+6A0A.0	fR/S	 75.11
U+6A0A.0	fTotalStrokes	 15
U+6A0A.0	fCangjie	DDK
U+6A0A.0	fFourCorner	44430
U+6A0A.0	fHakka	  fan2; fan3
U+6A0A.0	fCantonese	faan4; faan6
U+6A0A.0	fMandarin	fan2; fan4; pan2
U+6A0A.0	fEnglish	[1] bird cage [2] disorderly; confused; messy [3] a Chinese family name
U+6A0B.0	fUTF8	樋
U+6A0B.0	fCNS11643	3-4C4D
U+6A0B.0	fGB	3-4274
U+6A0B.0	fR/S	  75.11
U+6A0B.0	fTotalStrokes	 15
U+6A0B.0	fCangjie	DYNB
U+6A0B.0	fFourCorner	47930
U+6A0B.0	fCantonese	tung1
U+6A0B.0	fMandarin	tong1
U+6A0B.0	fEnglish	tree name
U+6A0C.0	fUTF8	樌
U+6A0C.0	fCNS11643	3-4C50
U+6A0C.0	fGB	5-444A
U+6A0C.0	fR/S	 75.11
U+6A0C.0	fTotalStrokes	 15
U+6A0C.0	fCangjie	DWJC
U+6A0C.0	fFourCorner	47980
U+6A0C.0	fCantonese	gun3; gwun3 (also)
U+6A0C.0	fMandarin	guan4
U+6A0D.0	fUTF8	樍
U+6A0D.0	fCNS11643	2-512C
U+6A0D.0	fBig5	E5E3
U+6A0D.0	fGB	5-4433
U+6A0D.0	fR/S	 75.11
U+6A0D.0	fTotalStrokes	 15
U+6A0D.0	fCangjie	DQMC
U+6A0D.0	fFourCorner	45986
U+6A0D.0	fCantonese	zik1; zaak3
U+6A0D.0	fMandarin	ji1; ze2
U+6A0E.0	fUTF8	樎
U+6A0E.0	fCNS11643	3-4C47
U+6A0E.0	fGB	3-432A
U+6A0E.0	fR/S	  75.11
U+6A0E.0	fTotalStrokes	 15
U+6A0E.0	fCangjie	DJOA
U+6A0E.0	fFourCorner	43960
U+6A0E.0	fCantonese	suk1
U+6A0E.0	fMandarin	suo1
U+6A0F.0	fUTF8	樏
U+6A0F.0	fCNS11643	2-5133
U+6A0F.0	fBig5	E5EA
U+6A0F.0	fGB	3-427E
U+6A0F.0	fR/S	 75.11
U+6A0F.0	fTotalStrokes	 15
U+6A0F.0	fCangjie	DWVF
U+6A0F.0	fFourCorner	46993
U+6A0F.0	fHakka	  lui3
U+6A0F.0	fCantonese	leoi4; lai5
U+6A0F.0	fMandarin	lei3; lei2; luo3
U+6A0F.0	fEnglish	sedan
U+6A10.0	fUTF8	樐
U+6A10.0	fCNS11643	3-4C5B
U+6A10.0	fGB	E-2E6A
U+6A10.0	fR/S	 75.11
U+6A10.0	fTotalStrokes	 15
U+6A10.0	fCangjie	DYWI
U+6A10.0	fFourCorner	41960
U+6A10.0	fHakka	 lu1
U+6A10.0	fCantonese	lou5
U+6A10.0	fMandarin	lu3
U+6A10.0	fEnglish	 a lookout turret on a city wall; moveable wooden tower for archers a scull a sweep; anoar
U+6A11.0	fUTF8	樑
U+6A11.0	fCNS11643	1-6D43
U+6A11.0	fBig5	BCD9
U+6A11.0	fGB	8-2D6E
U+6A11.0	fR/S	 75.11
U+6A11.0	fTotalStrokes	 15
U+6A11.0	fCangjie	DEID
U+6A11.0	fFourCorner	47994
U+6A11.0	fHakka	  liong2; liong3
U+6A11.0	fCantonese	loeng4; loeng2 (also)
U+6A11.0	fMandarin	liang2
U+6A11.0	fEnglish	 bridge beam
U+6A12.0	fUTF8	樒
U+6A12.0	fCNS11643	3-4C56
U+6A12.0	fGB	5-454D
U+6A12.0	fR/S	  75.11
U+6A12.0	fTotalStrokes	 15
U+6A12.0	fCangjie	DJPU
U+6A12.0	fFourCorner	43970
U+6A12.0	fHakka	 mit8; met8
U+6A12.0	fCantonese	mat6
U+6A12.0	fMandarin	mi4
U+6A12.0	fEnglish	the eagle-wood tree, or lign-aloes Aquilaria or Aloexylon found in Cambodia. It's heart is called garoo wood
U+6A13.0	fUTF8	樓
U+6A13.0	fCNS11643	1-6D3D
U+6A13.0	fBig5	BCD3
U+6A13.0	fGB	1-4225
U+6A13.0	fR/S	 75.11
U+6A13.0	fTotalStrokes	 15
U+6A13.0	fCangjie	DLLV
U+6A13.0	fFourCorner	45944
U+6A13.0	fHakka	 leu2; lieu2; liu2; leu3
U+6A13.0	fCantonese	lau4; lau2 (also)
U+6A13.0	fMandarin	lou2; l2
U+6A13.0	fEnglish	a building of two or more stories
U+6A14.0	fUTF8	樔
U+6A14.0	fCNS11643	2-5125
U+6A14.0	fBig5	E5DC
U+6A14.0	fGB	5-4551
U+6A14.0	fR/S	 75.11
U+6A14.0	fTotalStrokes	 15
U+6A14.0	fCangjie	DVVD
U+6A14.0	fFourCorner	42994
U+6A14.0	fHakka	  cau2; ziau3
U+6A14.0	fCantonese	caau4; ziu2
U+6A14.0	fMandarin	chao2; jiao3
U+6A14.0	fEnglish	[1] hut [2] basket for catching fish
U+6A15.0	fUTF8	樕
U+6A15.0	fCNS11643	2-5076
U+6A15.0	fBig5	E5CF
U+6A15.0	fGB	3-427A
U+6A15.0	fR/S	 75.11
U+6A15.0	fTotalStrokes	 15
U+6A15.0	fCangjie	DDLO
U+6A15.0	fFourCorner	47982
U+6A15.0	fHakka	  suk7
U+6A15.0	fCantonese	cuk1; sok3
U+6A15.0	fMandarin	su4
U+6A15.0	fEnglish	shrub
U+6A16.0	fUTF8	樖
U+6A16.0	fCNS11643	2-5138
U+6A16.0	fBig5	E5EF
U+6A16.0	fGB	5-4546
U+6A16.0	fR/S	 75.11
U+6A16.0	fTotalStrokes	 15
U+6A16.0	fCangjie	ORDMR
U+6A16.0	fFourCorner	80921
U+6A16.0	fHakka	 po1
U+6A16.0	fCantonese	po1
U+6A16.0	fMandarin	ke1
U+6A16.0	fEnglish	classifier for plants or trees; a tree-trunk
U+6A17.0	fUTF8	樗
U+6A17.0	fCNS11643	2-5073
U+6A17.0	fBig5	E5CC
U+6A17.0	fGB	0-694B
U+6A17.0	fR/S	 75.11
U+6A17.0	fTotalStrokes	 15
U+6A17.0	fCangjie	DMBS
U+6A17.0	fFourCorner	41927
U+6A17.0	fHakka	 ri2; ji2; shu1
U+6A17.0	fCantonese	syu1
U+6A17.0	fMandarin	chu1; shu1; tu2
U+6A17.0	fEnglish	Ailanthus altissima; kind of tree with useless timber
U+6A18.0	fUTF8	樘
U+6A18.0	fCNS11643	2-5131
U+6A18.0	fBig5	E5E8
U+6A18.0	fGB	0-694C
U+6A18.0	fR/S	 75.11
U+6A18.0	fTotalStrokes	 15
U+6A18.0	fCangjie	DFBG
U+6A18.0	fFourCorner	49914
U+6A18.0	fHakka	 cang5
U+6A18.0	fCantonese	tong4
U+6A18.0	fMandarin	tang2; cheng1
U+6A19.0	fUTF8	標
U+6A19.0	fCNS11643	1-6D3A
U+6A19.0	fBig5	BCD0
U+6A19.0	fGB	1-316A
U+6A19.0	fR/S	 75.11
U+6A19.0	fTotalStrokes	 15
U+6A19.0	fCangjie	DMWF
U+6A19.0	fFourCorner	41991
U+6A19.0	fHakka	  biau1;  piau1; peu1; beu1
U+6A19.0	fCantonese	biu1
U+6A19.0	fMandarin	biao1; biao3
U+6A19.0	fEnglish	[1] show; mark; symbolise; label; tag; indicate [2] sign; mark; symbol; indication; label [3] model; paragon [4] bid; tender
U+6A1A.0	fUTF8	樚
U+6A1A.0	fCNS11643	3-4C4A
U+6A1A.0	fGB	5-454A
U+6A1A.0	fR/S	 75.11
U+6A1A.0	fTotalStrokes	 15
U+6A1A.0	fCangjie	DIXP
U+6A1A.0	fFourCorner	40911
U+6A1A.0	fHakka	  luk8; luk7
U+6A1A.0	fCantonese	luk6
U+6A1A.0	fMandarin	lu4
U+6A1A.0	fEnglish	a pulley; a windlass; a wheel; a block
U+6A1B.0	fUTF8	樛
U+6A1B.0	fCNS11643	2-507D
U+6A1B.0	fBig5	E5D6
U+6A1B.0	fGB	3-432D
U+6A1B.0	fR/S	 75.11
U+6A1B.0	fTotalStrokes	 15
U+6A1B.0	fCangjie	DSMH
U+6A1B.0	fFourCorner	47922
U+6A1B.0	fHakka	 kieu1; geu1
U+6A1B.0	fCantonese	gau1; kau1
U+6A1B.0	fMandarin	jiu1; liao2; miu2
U+6A1B.0	fEnglish	[1] bending branches; drooping branches [2] twisted; distorted
U+6A1C.0	fUTF8	樜
U+6A1C.0	fCNS11643	3-4C49
U+6A1C.0	fGB	3-4324
U+6A1C.0	fR/S	 75.11
U+6A1C.0	fTotalStrokes	 15
U+6A1C.0	fCangjie	DITF
U+6A1C.0	fFourCorner	40937
U+6A1C.0	fHakka	 zha5; za5
U+6A1C.0	fCantonese	ze3
U+6A1C.0	fMandarin	shu4
U+6A1D.0	fUTF8	樝
U+6A1D.0	fCNS11643	2-507E
U+6A1D.0	fBig5	E5D7
U+6A1D.0	fGB	5-453F
U+6A1D.0	fR/S	 75.11
U+6A1D.0	fTotalStrokes	 15
U+6A1D.0	fCangjie	DYPM
U+6A1D.0	fFourCorner	41917
U+6A1D.0	fCantonese	zaa1
U+6A1D.0	fMandarin	zha1
U+6A1D.0	fEnglish	a sour red fruit of the size of a cherry; a species of hawthorn (Cratagus cuneata or C. pinnatifida)
U+6A1E.0	fUTF8	樞
U+6A1E.0	fCNS11643	1-6D39
U+6A1E.0	fBig5	BCCF
U+6A1E.0	fGB	1-4A60
U+6A1E.0	fR/S	 75.11
U+6A1E.0	fTotalStrokes	 15
U+6A1E.0	fCangjie	DSRR
U+6A1E.0	fFourCorner	41916
U+6A1E.0	fHakka	  ki1
U+6A1E.0	fCantonese	syu1
U+6A1E.0	fMandarin	shu1; chu1; ou1
U+6A1E.0	fEnglish	[1] door hinge; pivot [2] a kind of tree
U+6A1F.0	fUTF8	樟
U+6A1F.0	fCNS11643	1-6D36
U+6A1F.0	fBig5	BCCC
U+6A1F.0	fGB	0-5541
U+6A1F.0	fR/S	 75.11
U+6A1F.0	fTotalStrokes	 15
U+6A1F.0	fCangjie	DYTJ
U+6A1F.0	fFourCorner	40946
U+6A1F.0	fHakka	  zong1; zhong1
U+6A1F.0	fCantonese	zoeng1
U+6A1F.0	fMandarin	zhang1
U+6A1F.0	fEnglish	the camphor tree
U+6A20.0	fUTF8	樠
U+6A20.0	fCNS11643	2-5079
U+6A20.0	fBig5	E5D2
U+6A20.0	fGB	3-4258
U+6A20.0	fR/S	 75.11
U+6A20.0	fTotalStrokes	 15
U+6A20.0	fCangjie	DTLB
U+6A20.0	fFourCorner	54027
U+6A20.0	fHakka	  man2; mun2
U+6A20.0	fCantonese	mun4
U+6A20.0	fMandarin	men2; man2; lang3
U+6A20.0	fEnglish	a kind of tree
U+6A21.0	fUTF8	模
U+6A21.0	fCNS11643	1-6D3C
U+6A21.0	fBig5	BCD2
U+6A21.0	fGB	0-4423
U+6A21.0	fR/S	 75.11
U+6A21.0	fTotalStrokes	 15
U+6A21.0	fCangjie	DTAK
U+6A21.0	fFourCorner	44934
U+6A21.0	fCantonese	mou4; mou2 (also)
U+6A21.0	fMandarin	mo2
U+6A21.0	fEnglish	[1] model; standard; norm [2] imitate; copy
U+6A21.1	fUTF8	模
U+6A21.1	fCNS11643	1-6D3C
U+6A21.1	fBig5	BCD2
U+6A21.1	fGB	0-4423
U+6A21.1	fR/S	 75.11
U+6A21.1	fTotalStrokes	 15
U+6A21.1	fCangjie	DTAK
U+6A21.1	fFourCorner	44934
U+6A21.1	fHakka	 mu2
U+6A21.1	fMandarin	mu2
U+6A21.1	fEnglish	 mold; matrix; form
U+6A22.0	fUTF8	樢
U+6A22.0	fCNS11643	3-4C52
U+6A22.0	fGB	3-4077
U+6A22.0	fR/S	  75.11
U+6A22.0	fTotalStrokes	 15
U+6A22.0	fCangjie	DHAF
U+6A22.0	fFourCorner	47920
U+6A22.0	fCantonese	niu5; liu5 (also)
U+6A22.0	fMandarin	mu4; niao3
U+6A23.0	fUTF8	樣
U+6A23.0	fCNS11643	1-6D35
U+6A23.0	fBig5	BCCB
U+6A23.0	fGB	1-5179
U+6A23.0	fR/S	 75.11
U+6A23.0	fTotalStrokes	 15
U+6A23.0	fCangjie	DTGE
U+6A23.0	fFourCorner	48932
U+6A23.0	fHakka	  jong5; jong3; ngiong3; rong6; jong6; ngiong5; ngiong6; niong5
U+6A23.0	fCantonese	joeng6; joeng2 (also)
U+6A23.0	fMandarin	yang4; xiang4
U+6A23.0	fEnglish	[1] appearances; looks; shape; form [2] pattern; style; mode; form [3] sort; kind; variety [4] sample
U+6A24.0	fUTF8	樤
U+6A24.0	fCNS11643	3-4C62
U+6A24.0	fGB	5-4429
U+6A24.0	fR/S	  75.11
U+6A24.0	fTotalStrokes	 15
U+6A24.0	fCangjie	DOLD
U+6A24.0	fFourCorner	48990
U+6A24.0	fCantonese	tiu4
U+6A24.0	fMandarin	tiao2
U+6A25.0	fUTF8	樥
U+6A25.0	fCNS11643	2-5132
U+6A25.0	fBig5	E5E9
U+6A25.0	fGB	5-447D
U+6A25.0	fR/S	 75.11
U+6A25.0	fTotalStrokes	 15
U+6A25.0	fCangjie	DTHJ
U+6A25.0	fFourCorner	44958
U+6A25.0	fCantonese	pung4
U+6A25.0	fMandarin	peng2
U+6A26.0	fUTF8	樦
U+6A26.0	fCNS11643	2-5135
U+6A26.0	fBig5	E5EC
U+6A26.0	fGB	5-4544
U+6A26.0	fR/S	 75.11
U+6A26.0	fTotalStrokes	 15
U+6A26.0	fCangjie	DHYG
U+6A26.0	fFourCorner	48914
U+6A26.0	fCantonese	cyu5
U+6A26.0	fMandarin	zhu4
U+6A27.0	fUTF8	樧
U+6A27.0	fCNS11643	2-5122
U+6A27.0	fBig5	E5D9
U+6A27.0	fGB	E-2E6B
U+6A27.0	fR/S	 75.11
U+6A27.0	fTotalStrokes	 15
U+6A27.0	fCangjie	DKCE
U+6A27.0	fFourCorner	47947
U+6A27.0	fCantonese	saat3
U+6A27.0	fMandarin	sha1; xie4
U+6A28.0	fUTF8	樨
U+6A28.0	fCNS11643	2-5755
U+6A28.0	fBig5	E9CA
U+6A28.0	fGB	0-6958
U+6A28.0	fR/S	 75.11
U+6A28.0	fTotalStrokes	 15
U+6A28.0	fCangjie	DSEQ
U+6A28.0	fFourCorner	47953
U+6A28.0	fHakka	 si1; chi2; cih2
U+6A28.0	fCantonese	sai1
U+6A28.0	fMandarin	xi1
U+6A28.0	fEnglish	a cassia tree
U+6A29.0	fUTF8	権
U+6A29.0	fCNS11643	3-4C5E
U+6A29.0	fGB	E-2E6C
U+6A29.0	fR/S	 75.11
U+6A29.0	fTotalStrokes	 15
U+6A29.0	fCangjie	DOKG
U+6A29.0	fFourCorner	48910
U+6A29.0	fCantonese	kyun4
U+6A29.0	fMandarin	quan2
U+6A29.0	fEnglish	power; right; authority
U+6A2A.0	fUTF8	横
U+6A2A.0	fCNS11643	3-4C59
U+6A2A.0	fGB	0-3A61
U+6A2A.0	fR/S	 75.11
U+6A2A.0	fTotalStrokes	 15
U+6A2A.0	fCangjie	DTLC
U+6A2A.0	fFourCorner	44980
U+6A2A.0	fCantonese	waang4; waang6
U+6A2A.0	fMandarin	heng2; heng4; guang4; guang1; huang2; huang4
U+6A2A.0	fEnglish	[1] across [2] [3] ly
U+6A2B.0	fUTF8	樫
U+6A2B.0	fCNS11643	F-4F42
U+6A2B.0	fGB	E-2E6D
U+6A2B.0	fR/S	 75.11
U+6A2B.0	fTotalStrokes	 15
U+6A2B.0	fCangjie	DSEG
U+6A2B.0	fFourCorner	4791
U+6A2B.0	fCantonese	gin1
U+6A2B.0	fMandarin	jian1
U+6A2B.0	fEnglish	evergreen oak
U+6A2C.0	fUTF8	樬
U+6A2C.0	fCNS11643	3-4C61
U+6A2C.0	fGB	3-4323
U+6A2C.0	fR/S	 75.11
U+6A2C.0	fTotalStrokes	 15
U+6A2C.0	fCangjie	DHWP
U+6A2C.0	fFourCorner	46930
U+6A2C.0	fCantonese	sung2
U+6A2C.0	fMandarin	song3; cong1; song1
U+6A2D.0	fUTF8	樭
U+6A2D.0	fCNS11643	3-6530
U+6A2D.0	fGB	E-2E6E
U+6A2D.0	fR/S	 75.11
U+6A2D.0	fTotalStrokes	 15
U+6A2D.0	fCangjie	DTCG
U+6A2D.0	fFourCorner	4491
U+6A2D.0	fCantonese	kei4
U+6A2E.0	fUTF8	樮
U+6A2E.0	fGB	E-2E6F
U+6A2E.0	fR/S	 75.10
U+6A2E.0	fTotalStrokes	 14
U+6A2E.0	fCangjie	XXDMW
U+6A2E.0	fCantonese	jim5
U+6A2F.0	fUTF8	樯
U+6A2F.0	fGB	0-6949
U+6A2F.0	fR/S	 75.11
U+6A2F.0	fTotalStrokes	 15
U+6A2F.0	fCangjie	DGCW
U+6A2F.0	fHakka	 siong2
U+6A2F.0	fCantonese	coeng4
U+6A2F.0	fMandarin	qiang2
U+6A2F.0	fEnglish	simplified form of &#x6AA3;
U+6A30.0	fUTF8	樰
U+6A30.0	fGB	E-2E70
U+6A30.0	fR/S	 75.11
U+6A30.0	fTotalStrokes	 15
U+6A30.0	fCangjie	XDMBS
U+6A30.0	fCantonese	syut3
U+6A31.0	fUTF8	樱
U+6A31.0	fGB	0-5323
U+6A31.0	fR/S	 75.11
U+6A31.0	fTotalStrokes	 15
U+6A31.0	fCangjie	DBOV
U+6A31.0	fHakka	 jin1
U+6A31.0	fCantonese	jing1
U+6A31.0	fMandarin	ying1
U+6A31.0	fEnglish	simplified form of &#x6AFB;
U+6A32.0	fUTF8	樲
U+6A32.0	fCNS11643	2-574D
U+6A32.0	fBig5	E9C2
U+6A32.0	fGB	3-4231
U+6A32.0	fR/S	 75.12
U+6A32.0	fTotalStrokes	 16
U+6A32.0	fCangjie	DIPC
U+6A32.0	fFourCorner	53940
U+6A32.0	fHakka	 ngi5; ngi6
U+6A32.0	fCantonese	ji6
U+6A32.0	fMandarin	er4; zhi4
U+6A32.0	fEnglish	jujube
U+6A33.0	fUTF8	樳
U+6A33.0	fCNS11643	3-512C
U+6A33.0	fGB	3-4143
U+6A33.0	fR/S	 75.12
U+6A33.0	fTotalStrokes	 16
U+6A33.0	fCangjie	DSMI
U+6A33.0	fFourCorner	47940
U+6A33.0	fCantonese	cam4
U+6A33.0	fMandarin	xin2
U+6A33.0	fEnglish	name of a large tree
U+6A34.0	fUTF8	樴
U+6A34.0	fCNS11643	2-5749
U+6A34.0	fBig5	E9BE
U+6A34.0	fGB	E-2E71
U+6A34.0	fR/S	 75.12
U+6A34.0	fTotalStrokes	 16
U+6A34.0	fCangjie	DYIA
U+6A34.0	fFourCorner	43950
U+6A34.0	fCantonese	zik1
U+6A34.0	fMandarin	zhi2; yi4
U+6A35.0	fUTF8	樵
U+6A35.0	fCNS11643	1-7121
U+6A35.0	fBig5	BEF6
U+6A35.0	fGB	0-6954
U+6A35.0	fR/S	 75.12
U+6A35.0	fTotalStrokes	 16
U+6A35.0	fCangjie	DOGF
U+6A35.0	fFourCorner	40931
U+6A35.0	fHakka	  ziau1; zeu1
U+6A35.0	fCantonese	ciu4
U+6A35.0	fMandarin	qiao2
U+6A35.0	fEnglish	[1] woodcutter [2] firewood; fuel [3] gather firewood or fuel [4] burn [5] tower; lookout
U+6A36.0	fUTF8	樶
U+6A36.0	fCNS11643	3-513C
U+6A36.0	fGB	5-455B
U+6A36.0	fR/S	 75.12
U+6A36.0	fTotalStrokes	 16
U+6A36.0	fCangjie	DASE
U+6A36.0	fFourCorner	46940
U+6A36.0	fCantonese	zeoi1
U+6A36.0	fMandarin	zui1
U+6A36.0	fEnglish	same as &#x235B1;
U+6A37.0	fUTF8	樷
U+6A37.0	fCNS11643	3-5138
U+6A37.0	fGB	E-2E72
U+6A37.0	fR/S	  75.12
U+6A37.0	fTotalStrokes	 16
U+6A37.0	fCangjie	DDSJE
U+6A37.0	fFourCorner	44140
U+6A37.0	fHakka	 cung2
U+6A37.0	fCantonese	cung4
U+6A37.0	fMandarin	cong2; cong1
U+6A38.0	fUTF8	樸
U+6A38.0	fCNS11643	1-7074
U+6A38.0	fBig5	BEEB
U+6A38.0	fGB	1-4653
U+6A38.0	fR/S	 75.12
U+6A38.0	fTotalStrokes	 16
U+6A38.0	fCangjie	DTCO
U+6A38.0	fFourCorner	42934
U+6A38.0	fHakka	 pok7;  puk7
U+6A38.0	fCantonese	buk6; pok3; po1; piu4; pok2 (also); buk1
U+6A38.0	fMandarin	pu3; pu2; po4; bu2
U+6A38.0	fEnglish	[1] plain; simple (said of dress, clothing, literary) [2] the substance of things; things in the rough [3] honest; sincere; simple [4] Aphananthe asprera; a tree of the elm family
U+6A39.0	fUTF8	樹
U+6A39.0	fCNS11643	1-7079
U+6A39.0	fBig5	BEF0
U+6A39.0	fGB	1-4A77
U+6A39.0	fR/S	 75.12
U+6A39.0	fTotalStrokes	 16
U+6A39.0	fCangjie	DGTI
U+6A39.0	fFourCorner	44900
U+6A39.0	fHakka	 su3; shu6; su5; shu5
U+6A39.0	fCantonese	syu6
U+6A39.0	fMandarin	shu4
U+6A39.0	fEnglish	[1] tree [2] plant; cultivate [3] set up; establish; erect [4] door screen
U+6A3A.0	fUTF8	樺
U+6A3A.0	fCNS11643	1-7075
U+6A3A.0	fBig5	BEEC
U+6A3A.0	fGB	1-686B
U+6A3A.0	fR/S	 75.12
U+6A3A.0	fTotalStrokes	 16
U+6A3A.0	fCangjie	DTMJ
U+6A3A.0	fFourCorner	44954
U+6A3A.0	fHakka	  fa2; fa5
U+6A3A.0	fCantonese	waa6 (also); waa4
U+6A3A.0	fMandarin	hua4; hua2; jia2
U+6A3A.0	fEnglish	 birch
U+6A3B.0	fUTF8	樻
U+6A3B.0	fCNS11643	2-5757
U+6A3B.0	fBig5	E9CC
U+6A3B.0	fGB	5-445C
U+6A3B.0	fR/S	 75.12
U+6A3B.0	fTotalStrokes	 16
U+6A3B.0	fCangjie	DLMC
U+6A3B.0	fFourCorner	47986
U+6A3B.0	fHakka	 kui6; kui5; k'wui5
U+6A3B.0	fCantonese	gwai6
U+6A3B.0	fMandarin	kui4; kui3
U+6A3B.0	fEnglish	a small knotty tree
U+6A3C.0	fUTF8	樼
U+6A3C.0	fCNS11643	2-5762
U+6A3C.0	fBig5	E9D7
U+6A3C.0	fGB	5-456A
U+6A3C.0	fR/S	 75.12
U+6A3C.0	fTotalStrokes	 16
U+6A3C.0	fCangjie	DSND
U+6A3C.0	fFourCorner	47947
U+6A3C.0	fCantonese	zeon1
U+6A3C.0	fMandarin	zhen1; jian1; qin2
U+6A3D.0	fUTF8	樽
U+6A3D.0	fCNS11643	1-7073
U+6A3D.0	fBig5	BEEA
U+6A3D.0	fGB	0-6957
U+6A3D.0	fR/S	 75.12
U+6A3D.0	fTotalStrokes	 16
U+6A3D.0	fCangjie	DTWI
U+6A3D.0	fFourCorner	48946
U+6A3D.0	fHakka	  zun1
U+6A3D.0	fCantonese	zeon1
U+6A3D.0	fMandarin	zun1; zun3
U+6A3D.0	fEnglish	[1] wine vessel; goblet; bottle; wine jar; jug [2] luxuriant (said of vegetation)
U+6A3E.0	fUTF8	樾
U+6A3E.0	fCNS11643	2-574F
U+6A3E.0	fBig5	E9C4
U+6A3E.0	fGB	0-6950
U+6A3E.0	fR/S	 75.12
U+6A3E.0	fTotalStrokes	 16
U+6A3E.0	fCangjie	DGOV
U+6A3E.0	fFourCorner	43985
U+6A3E.0	fHakka	 jet8; zat8
U+6A3E.0	fCantonese	jyut6
U+6A3E.0	fMandarin	yue4
U+6A3E.0	fEnglish	the shade of trees
U+6A3F.0	fUTF8	樿
U+6A3F.0	fCNS11643	2-5758
U+6A3F.0	fBig5	E9CD
U+6A3F.0	fGB	3-4227
U+6A3F.0	fR/S	 75.12
U+6A3F.0	fTotalStrokes	 16
U+6A3F.0	fCangjie	DRRJ
U+6A3F.0	fFourCorner	46956
U+6A3F.0	fHakka	 zhan5; zan5; zhen5
U+6A3F.0	fCantonese	sin6; zin2 (also)
U+6A3F.0	fMandarin	zhan3; shan4
U+6A3F.0	fEnglish	coffin
U+6A40.0	fUTF8	橀
U+6A40.0	fCNS11643	2-5128
U+6A40.0	fBig5	E5DF
U+6A40.0	fGB	5-4566
U+6A40.0	fR/S	 75.12
U+6A40.0	fTotalStrokes	 16
U+6A40.0	fCangjie	DYIT
U+6A40.0	fFourCorner	50917
U+6A40.0	fCantonese	hei1; hoi2
U+6A40.0	fMandarin	xi1; hai3
U+6A40.0	fEnglish	name of a tree
U+6A41.0	fUTF8	橁
U+6A41.0	fCNS11643	2-5759
U+6A41.0	fBig5	E9CE
U+6A41.0	fGB	5-4561
U+6A41.0	fR/S	 75.12
U+6A41.0	fTotalStrokes	 16
U+6A41.0	fCangjie	DHPA
U+6A41.0	fFourCorner	48927
U+6A41.0	fCantonese	ceon1; ceon4
U+6A41.0	fMandarin	xun2; chun1
U+6A41.0	fEnglish	name of a tree
U+6A42.0	fUTF8	橂
U+6A42.0	fCNS11643	3-5128
U+6A42.0	fGB	E-2E73
U+6A42.0	fR/S	  75.12
U+6A42.0	fTotalStrokes	 16
U+6A42.0	fCangjie	DTWK
U+6A42.0	fFourCorner	48930
U+6A42.0	fCantonese	din6
U+6A42.0	fMandarin	dian4
U+6A43.0	fUTF8	橃
U+6A43.0	fCNS11643	3-512B
U+6A43.0	fGB	5-435E
U+6A43.0	fR/S	  75.12
U+6A43.0	fTotalStrokes	 16
U+6A43.0	fCangjie	DNOE
U+6A43.0	fFourCorner	42940
U+6A43.0	fHakka	 fat8; fat7
U+6A43.0	fCantonese	fat6
U+6A43.0	fMandarin	fa1; bo1; fei4
U+6A43.0	fEnglish	large vessel; raft
U+6A44.0	fUTF8	橄
U+6A44.0	fCNS11643	1-707A
U+6A44.0	fBig5	BEF1
U+6A44.0	fGB	0-694F
U+6A44.0	fR/S	 75.12
U+6A44.0	fTotalStrokes	 16
U+6A44.0	fCangjie	DMJK
U+6A44.0	fFourCorner	48940
U+6A44.0	fHakka	  gam3; gam1; ga1
U+6A44.0	fCantonese	gam2; gaam3 (also); gam3 (also)
U+6A44.0	fMandarin	gan3; qian3
U+6A44.0	fEnglish	[1] olive [2] a Chinese family name
U+6A45.0	fUTF8	橅
U+6A45.0	fCNS11643	3-5140
U+6A45.0	fGB	5-433F
U+6A45.0	fR/S	 75.12
U+6A45.0	fTotalStrokes	 16
U+6A45.0	fCangjie	DOTF
U+6A45.0	fFourCorner	48930
U+6A45.0	fCantonese	mou4
U+6A45.0	fMandarin	mo2; mu2
U+6A45.0	fEnglish	a beech tree
U+6A46.0	fUTF8	橆
U+6A46.0	fCNS11643	2-5768
U+6A46.0	fBig5	E9DD
U+6A46.0	fGB	E-2E74
U+6A46.0	fR/S	 75.12
U+6A46.0	fTotalStrokes	 16
U+6A46.0	fCangjie	OTDD
U+6A46.0	fFourCorner	80994
U+6A46.0	fHakka	 wu3; vu2
U+6A46.0	fCantonese	mou4
U+6A46.0	fMandarin	wu3
U+6A46.0	fEnglish	 without; apart from; none a negative
U+6A47.0	fUTF8	橇
U+6A47.0	fCNS11643	1-707E
U+6A47.0	fBig5	BEF5
U+6A47.0	fGB	0-4741
U+6A47.0	fR/S	 75.12
U+6A47.0	fTotalStrokes	 16
U+6A47.0	fCangjie	DHUU
U+6A47.0	fFourCorner	42914
U+6A47.0	fHakka	 kiau1; k'ieu1
U+6A47.0	fCantonese	hiu1; ceoi3 (also); zyut3
U+6A47.0	fMandarin	qiao1; qiao2; qiao4; jue2; cui4; zui4
U+6A47.0	fEnglish	a sledge for transportation over mud or snow; sleigh
U+6A48.0	fUTF8	橈
U+6A48.0	fCNS11643	1-7123
U+6A48.0	fBig5	BEF8
U+6A48.0	fGB	1-6863
U+6A48.0	fR/S	 75.12
U+6A48.0	fTotalStrokes	 16
U+6A48.0	fCangjie	DGGU
U+6A48.0	fFourCorner	44911
U+6A48.0	fHakka	  ngiau2; jau3; ngieu2; nau2; nau5
U+6A48.0	fCantonese	naau4; jiu4; laau4 (also); naau6; ling4; lau4
U+6A48.0	fMandarin	nao2; rao2; nao4; nao3; rao3
U+6A48.0	fEnglish	[1] bent wood; twisted piece of wood; crooked wood [2] enfeeble; weaken; sap; devitalize [3] disperse; scatter [4] wrong; be wronged
U+6A49.0	fUTF8	橉
U+6A49.0	fCNS11643	2-574B
U+6A49.0	fBig5	E9C0
U+6A49.0	fGB	3-433D
U+6A49.0	fR/S	 75.12
U+6A49.0	fTotalStrokes	 16
U+6A49.0	fCangjie	DFDQ
U+6A49.0	fFourCorner	49959
U+6A49.0	fHakka	 nin3; lin3; lin5; nin5
U+6A49.0	fCantonese	leon6; leon4
U+6A49.0	fMandarin	lin4; lin3
U+6A49.0	fEnglish	a tree found in Kangnan, the bark of which is used for dye
U+6A4A.0	fUTF8	橊
U+6A4A.0	fCNS11643	3-513B
U+6A4A.0	fGB	E-2E75
U+6A4A.0	fR/S	 75.12
U+6A4A.0	fTotalStrokes	 16
U+6A4A.0	fCangjie	DMLW
U+6A4A.0	fFourCorner	41960
U+6A4A.0	fHakka	 liu2
U+6A4A.0	fCantonese	lau4
U+6A4A.0	fMandarin	liu2
U+6A4A.0	fEnglish	$name of a fruit tree, same as &#x69B4;
U+6A4B.0	fUTF8	橋
U+6A4B.0	fCNS11643	1-707D
U+6A4B.0	fBig5	BEF4
U+6A4B.0	fGB	1-4745
U+6A4B.0	fR/S	 75.12
U+6A4B.0	fTotalStrokes	 16
U+6A4B.0	fCangjie	DHKB
U+6A4B.0	fFourCorner	42927
U+6A4B.0	fHakka	  kiau2; k'ieu2; kiau3
U+6A4B.0	fCantonese	kiu4; kiu2 (also)
U+6A4B.0	fMandarin	qiao2; qiao1; jiao4; gao1jiao3
U+6A4B.0	fEnglish	[1] bridge; any bridge-like structure [2] beams of a structure [3] ; cross-grained [4] tall; high; elevated [5] a Chinese family name
U+6A4C.0	fUTF8	橌
U+6A4C.0	fCNS11643	3-5131
U+6A4C.0	fGB	5-442B
U+6A4C.0	fR/S	 75.12
U+6A4C.0	fTotalStrokes	 16
U+6A4C.0	fCangjie	DANB
U+6A4C.0	fFourCorner	47920
U+6A4C.0	fHakka	 han3
U+6A4C.0	fCantonese	haan5
U+6A4C.0	fMandarin	xian4
U+6A4D.0	fUTF8	橍
U+6A4D.0	fCNS11643	2-5766
U+6A4D.0	fBig5	E9DB
U+6A4D.0	fGB	5-442A
U+6A4D.0	fR/S	 75.12
U+6A4D.0	fTotalStrokes	 16
U+6A4D.0	fCangjie	DANG
U+6A4D.0	fFourCorner	47920
U+6A4D.0	fCantonese	jeon6
U+6A4D.0	fMandarin	run4
U+6A4D.0	fEnglish	name of a tree
U+6A4E.0	fUTF8	橎
U+6A4E.0	fCNS11643	2-5767
U+6A4E.0	fBig5	E9DC
U+6A4E.0	fGB	5-4562
U+6A4E.0	fR/S	 75.12
U+6A4E.0	fTotalStrokes	 16
U+6A4E.0	fCangjie	DHDW
U+6A4E.0	fFourCorner	42969
U+6A4E.0	fCantonese	faan2
U+6A4E.0	fMandarin	fan2; fan3
U+6A4E.0	fEnglish	name of tree
U+6A4F.0	fUTF8	橏
U+6A4F.0	fCNS11643	2-575D
U+6A4F.0	fBig5	E9D2
U+6A4F.0	fGB	3-433B
U+6A4F.0	fR/S	 75.12
U+6A4F.0	fTotalStrokes	 16
U+6A4F.0	fCangjie	DTGR
U+6A4F.0	fFourCorner	48961
U+6A4F.0	fCantonese	zin2; gin2
U+6A4F.0	fMandarin	zhan3; jian3; shan3
U+6A4F.0	fEnglish	[1] name of a tree
U+6A50.0	fUTF8	橐
U+6A50.0	fCNS11643	2-575C
U+6A50.0	fBig5	E9D1
U+6A50.0	fGB	0-6952
U+6A50.0	fR/S	 75.12
U+6A50.0	fTotalStrokes	 16
U+6A50.0	fCangjie	JBMRD
U+6A50.0	fFourCorner	40904
U+6A50.0	fHakka	  tok7
U+6A50.0	fCantonese	tok3; lok6
U+6A50.0	fMandarin	tuo4; du4; tuo2; zhe4; luo4
U+6A50.0	fEnglish	 sack; bag
U+6A51.0	fUTF8	橑
U+6A51.0	fCNS11643	2-5754
U+6A51.0	fBig5	E9C9
U+6A51.0	fGB	3-4333
U+6A51.0	fR/S	 75.12
U+6A51.0	fTotalStrokes	 16
U+6A51.0	fCangjie	DKCF
U+6A51.0	fFourCorner	43996
U+6A51.0	fHakka	 lau3
U+6A51.0	fCantonese	liu4 (also); lou5
U+6A51.0	fMandarin	lao3; liao2
U+6A51.0	fEnglish	short beams as of a verandah
U+6A52.0	fUTF8	橒
U+6A52.0	fCNS11643	3-5129
U+6A52.0	fGB	E-2E76
U+6A52.0	fR/S	 75.12
U+6A52.0	fTotalStrokes	 16
U+6A52.0	fCangjie	DMBI
U+6A52.0	fFourCorner	41930
U+6A52.0	fCantonese	wan4
U+6A52.0	fMandarin	yun2
U+6A52.0	fEnglish	[1] name of a tree [2] wood streaks
U+6A53.0	fUTF8	橓
U+6A53.0	fCNS11643	3-513E
U+6A53.0	fGB	5-4563
U+6A53.0	fR/S	 75.12
U+6A53.0	fTotalStrokes	 16
U+6A53.0	fCangjie	DBBQ
U+6A53.0	fFourCorner	42950
U+6A53.0	fHakka	 shun5; sun5
U+6A53.0	fCantonese	seon3
U+6A53.0	fMandarin	shun4
U+6A53.0	fEnglish	the Hibiscus Syyriacus - a type of transitoriness
U+6A54.0	fUTF8	橔
U+6A54.0	fCNS11643	2-575E
U+6A54.0	fBig5	E9D3
U+6A54.0	fGB	3-4338
U+6A54.0	fR/S	 75.12
U+6A54.0	fTotalStrokes	 16
U+6A54.0	fCangjie	DYDK
U+6A54.0	fFourCorner	48940
U+6A54.0	fHakka	  dun1;  tui1; dun3
U+6A54.0	fCantonese	deon1
U+6A54.0	fMandarin	tui2; dui1; dun1
U+6A54.0	fEnglish	a wooden chopping-block
U+6A55.0	fUTF8	橕
U+6A55.0	fCNS11643	2-5765
U+6A55.0	fBig5	E9DA
U+6A55.0	fGB	5-455A
U+6A55.0	fR/S	 75.12
U+6A55.0	fTotalStrokes	 16
U+6A55.0	fCangjie	DFBH
U+6A55.0	fFourCorner	49941
U+6A55.0	fHakka	 cang1;  cang5
U+6A55.0	fCantonese	caang1
U+6A55.0	fMandarin	cheng1
U+6A55.0	fEnglish	 prop shore
U+6A56.0	fUTF8	橖
U+6A56.0	fCNS11643	2-5764
U+6A56.0	fBig5	E9D9
U+6A56.0	fGB	5-4559
U+6A56.0	fR/S	 75.12
U+6A56.0	fTotalStrokes	 16
U+6A56.0	fCangjie	DFBD
U+6A56.0	fFourCorner	49996
U+6A56.0	fCantonese	tong4
U+6A56.0	fMandarin	tang2; cheng4; cheng1
U+6A57.0	fUTF8	橗
U+6A57.0	fCNS11643	3-512F
U+6A57.0	fGB	3-4278
U+6A57.0	fR/S	 75.12
U+6A57.0	fTotalStrokes	 16
U+6A57.0	fCangjie	DTAB
U+6A57.0	fFourCorner	44920
U+6A57.0	fHakka	 men2
U+6A57.0	fCantonese	mang4
U+6A57.0	fMandarin	meng2
U+6A57.0	fEnglish	[1] name of a tree [2] wood from the middle of a tree
U+6A58.0	fUTF8	橘
U+6A58.0	fCNS11643	1-7078
U+6A58.0	fBig5	BEEF
U+6A58.0	fGB	0-6959
U+6A58.0	fR/S	 75.12
U+6A58.0	fTotalStrokes	 16
U+6A58.0	fCangjie	DNHB
U+6A58.0	fFourCorner	47927
U+6A58.0	fHakka	  git7
U+6A58.0	fCantonese	gwat1; gat1
U+6A58.0	fMandarin	ju2
U+6A58.0	fEnglish	the mandarin orange; tangerine; Chinese orange
U+6A59.0	fUTF8	橙
U+6A59.0	fCNS11643	1-7076
U+6A59.0	fBig5	BEED
U+6A59.0	fGB	0-3348
U+6A59.0	fR/S	 75.12
U+6A59.0	fTotalStrokes	 16
U+6A59.0	fCangjie	DNOT
U+6A59.0	fFourCorner	42918
U+6A59.0	fHakka	 cang2; cen2; ch'ang2; can2; chang2; chin2
U+6A59.0	fCantonese	caang2; caang4 (also); dang3
U+6A59.0	fMandarin	cheng2; chen2; deng4
U+6A59.0	fEnglish	the orange
U+6A5A.0	fUTF8	橚
U+6A5A.0	fCNS11643	2-5756
U+6A5A.0	fBig5	E9CB
U+6A5A.0	fGB	3-422D
U+6A5A.0	fR/S	 75.12
U+6A5A.0	fTotalStrokes	 16
U+6A5A.0	fCangjie	DLX
U+6A5A.0	fFourCorner	45927
U+6A5A.0	fHakka	  siau1;  suk7; seu1
U+6A5A.0	fCantonese	suk1; siu1 (also)
U+6A5A.0	fMandarin	su4; shao3; qiu1; xiao1
U+6A5A.0	fEnglish	tall tree
U+6A5B.0	fUTF8	橛
U+6A5B.0	fCNS11643	2-5753
U+6A5B.0	fBig5	E9C8
U+6A5B.0	fGB	0-6953
U+6A5B.0	fR/S	 75.12
U+6A5B.0	fTotalStrokes	 16
U+6A5B.0	fCangjie	DMTO
U+6A5B.0	fFourCorner	41982
U+6A5B.0	fHakka	 kiet7; ket7
U+6A5B.0	fCantonese	kyut3; gyut6; kyut6 (also)
U+6A5B.0	fMandarin	jue2; gui4
U+6A5B.0	fEnglish	 portion of a long and slender object
U+6A5C.0	fUTF8	橜
U+6A5C.0	fCNS11643	3-5144
U+6A5C.0	fGB	E-2E77
U+6A5C.0	fR/S	 75.12
U+6A5C.0	fTotalStrokes	 16
U+6A5C.0	fCangjie	XMOD
U+6A5C.0	fFourCorner	71290
U+6A5C.0	fHakka	 kiet7; ket7
U+6A5C.0	fCantonese	kyut3; gyut6; kyut6 (also); gyut3
U+6A5C.0	fMandarin	jue2; gui4
U+6A5C.0	fEnglish	same as &#x6A5B;
U+6A5D.0	fUTF8	橝
U+6A5D.0	fCNS11643	2-5750
U+6A5D.0	fBig5	E9C5
U+6A5D.0	fGB	3-4332
U+6A5D.0	fR/S	 75.12
U+6A5D.0	fTotalStrokes	 16
U+6A5D.0	fCangjie	DMWJ
U+6A5D.0	fFourCorner	41946
U+6A5D.0	fHakka	  tam2
U+6A5D.0	fCantonese	taam4; taam2
U+6A5D.0	fMandarin	tan2; tan1; zhen4; dian4; xin2
U+6A5D.0	fEnglish	[1] the ends of rafters on which the eaves rest [2] ashes used in dyeing
U+6A5E.0	fUTF8	橞
U+6A5E.0	fCNS11643	2-5763
U+6A5E.0	fBig5	E9D8
U+6A5E.0	fGB	5-4557
U+6A5E.0	fR/S	 75.12
U+6A5E.0	fTotalStrokes	 16
U+6A5E.0	fCangjie	DJIP
U+6A5E.0	fFourCorner	45933
U+6A5E.0	fCantonese	wai6
U+6A5E.0	fMandarin	hui4
U+6A5E.0	fEnglish	name of a tree
U+6A5F.0	fUTF8	機
U+6A5F.0	fCNS11643	1-7122
U+6A5F.0	fBig5	BEF7
U+6A5F.0	fGB	1-3B7A
U+6A5F.0	fR/S	 75.12
U+6A5F.0	fTotalStrokes	 16
U+6A5F.0	fCangjie	DVII
U+6A5F.0	fFourCorner	42953
U+6A5F.0	fHakka	  gi1; gia2
U+6A5F.0	fCantonese	gei1
U+6A5F.0	fMandarin	ji1
U+6A5F.0	fEnglish	[1] machine; machinery; mechanics [2] opportune; chance; opportunity [3] crucial point; pivot [4] tricky; cunning [5] aircraft; plane; airplane
U+6A60.0	fUTF8	橠
U+6A60.0	fCNS11643	2-5761
U+6A60.0	fBig5	E9D6
U+6A60.0	fGB	5-4564
U+6A60.0	fR/S	 75.12
U+6A60.0	fTotalStrokes	 16
U+6A60.0	fCangjie	DYNV
U+6A60.0	fFourCorner	40932
U+6A60.0	fCantonese	no5; lo5 (also); no4
U+6A60.0	fMandarin	nuo3; luo3; nuo2
U+6A61.0	fUTF8	橡
U+6A61.0	fCNS11643	1-707C
U+6A61.0	fBig5	BEF3
U+6A61.0	fGB	0-4F70
U+6A61.0	fR/S	 75.12
U+6A61.0	fTotalStrokes	 16
U+6A61.0	fCangjie	DNAO
U+6A61.0	fFourCorner	47932
U+6A61.0	fHakka	  siong5; siong3; siong6
U+6A61.0	fCantonese	zoeng6
U+6A61.0	fMandarin	xiang4
U+6A61.0	fEnglish	the acorn
U+6A62.0	fUTF8	橢
U+6A62.0	fCNS11643	1-707B
U+6A62.0	fBig5	BEF2
U+6A62.0	fGB	1-4D56
U+6A62.0	fR/S	 75.12
U+6A62.0	fTotalStrokes	 16
U+6A62.0	fCangjie	DNLB
U+6A62.0	fFourCorner	44927
U+6A62.0	fHakka	  to3
U+6A62.0	fCantonese	to5; to2
U+6A62.0	fMandarin	tuo3; duo3; tuo2
U+6A62.0	fEnglish	oval; oval-shaped; elliptical; oblong
U+6A63.0	fUTF8	橣
U+6A63.0	fCNS11643	3-5127
U+6A63.0	fGB	5-4568
U+6A63.0	fR/S	  75.12
U+6A63.0	fTotalStrokes	 16
U+6A63.0	fCangjie	DJPB
U+6A63.0	fFourCorner	43920
U+6A63.0	fCantonese	ning5; ling5 (also)
U+6A63.0	fMandarin	ning3
U+6A63.0	fEnglish	name of a tree
U+6A64.0	fUTF8	橤
U+6A64.0	fCNS11643	2-575B
U+6A64.0	fBig5	E9D0
U+6A64.0	fGB	E-2E78
U+6A64.0	fR/S	 75.12
U+6A64.0	fTotalStrokes	 16
U+6A64.0	fCangjie	PPPD
U+6A64.0	fFourCorner	33904
U+6A64.0	fHakka	 lui3
U+6A64.0	fCantonese	jeoi5; jeoi6 (also)
U+6A64.0	fMandarin	rui3
U+6A64.0	fEnglish	stamen
U+6A65.0	fUTF8	橥
U+6A65.0	fCNS11643	4-4A4B
U+6A65.0	fGB	0-694D
U+6A65.0	fR/S	 75.11; 75.12
U+6A65.0	fTotalStrokes	 15
U+6A65.0	fCangjie	KAD
U+6A65.0	fFourCorner	44900
U+6A65.0	fCantonese	zyu1
U+6A65.0	fMandarin	zhu1
U+6A65.0	fEnglish	wooden peg; post or stick
U+6A66.0	fUTF8	橦
U+6A66.0	fCNS11643	2-574A
U+6A66.0	fBig5	E9BF
U+6A66.0	fGB	3-4339
U+6A66.0	fR/S	 75.12
U+6A66.0	fTotalStrokes	 16
U+6A66.0	fCangjie	DYTG
U+6A66.0	fFourCorner	40914
U+6A66.0	fHakka	 tung2
U+6A66.0	fCantonese	tung4; cong4
U+6A66.0	fMandarin	tong2; chong1; chuang2; chong2; dong3; zhong1
U+6A66.0	fEnglish	a species of tree in Yunnan whose flowers are used to make a kind of cloth
U+6A67.0	fUTF8	橧
U+6A67.0	fCNS11643	2-574C
U+6A67.0	fBig5	E9C1
U+6A67.0	fGB	3-433F
U+6A67.0	fR/S	 75.12
U+6A67.0	fTotalStrokes	 16
U+6A67.0	fCangjie	DCWA
U+6A67.0	fFourCorner	48966
U+6A67.0	fHakka	  zen1
U+6A67.0	fCantonese	zang1
U+6A67.0	fMandarin	zeng1; ceng2
U+6A67.0	fEnglish	hut
U+6A68.0	fUTF8	橨
U+6A68.0	fCNS11643	2-574E
U+6A68.0	fBig5	E9C3
U+6A68.0	fGB	3-4232
U+6A68.0	fR/S	 75.12
U+6A68.0	fTotalStrokes	 16
U+6A68.0	fCangjie	DJTC
U+6A68.0	fFourCorner	44986
U+6A68.0	fCantonese	fan4; fan5
U+6A68.0	fMandarin	fen2; fei3; fen4
U+6A68.0	fEnglish	[1] name of a tree [2] Xiongnu carriage
U+6A69.0	fUTF8	橩
U+6A69.0	fCNS11643	2-5760
U+6A69.0	fBig5	E9D5
U+6A69.0	fGB	E-4933
U+6A69.0	fR/S	 75.12
U+6A69.0	fTotalStrokes	 16
U+6A69.0	fCangjie	DFFN
U+6A69.0	fFourCorner	49917
U+6A69.0	fCantonese	king4
U+6A69.0	fMandarin	qiong2
U+6A69.0	fEnglish	[1] name of a tree
U+6A6A.0	fUTF8	橪
U+6A6A.0	fCNS11643	2-575A
U+6A6A.0	fBig5	E9CF
U+6A6A.0	fGB	3-4337
U+6A6A.0	fR/S	 75.12
U+6A6A.0	fTotalStrokes	 16
U+6A6A.0	fCangjie	DBKF
U+6A6A.0	fFourCorner	43933
U+6A6A.0	fCantonese	jim5; jin4
U+6A6A.0	fMandarin	ran3; yan1
U+6A6A.0	fEnglish	name of a tree
U+6A6B.0	fUTF8	橫
U+6A6B.0	fCNS11643	1-7077
U+6A6B.0	fBig5	BEEE
U+6A6B.0	fGB	E-2E79
U+6A6B.0	fR/S	 75.12; 75.11
U+6A6B.0	fTotalStrokes	 16
U+6A6B.0	fCangjie	DTMC
U+6A6B.0	fFourCorner	44986
U+6A6B.0	fHakka	 wang2;  wang5; wang3; wang6; vang5; wong2
U+6A6B.0	fCantonese	waang4; waang6; waang1 (also); gwong1
U+6A6B.0	fMandarin	heng2; guang1; guang4; huang4
U+6A6B.0	fEnglish	[1] crosswise; horizontal; lateral [2] east to west or vice versa [3] by the side; sideways [4] move crosswise; traverse [5] unrestrainedly; turbulently [6] a Chinese family name
U+6A6B.1	fUTF8	橫
U+6A6B.1	fCNS11643	1-7077
U+6A6B.1	fBig5	BEEE
U+6A6B.1	fGB	E-2E79
U+6A6B.1	fR/S	 75.12; 75.11
U+6A6B.1	fTotalStrokes	 16
U+6A6B.1	fCangjie	DTMC
U+6A6B.1	fFourCorner	44986
U+6A6B.1	fHakka	 wang2
U+6A6B.1	fMandarin	heng4
U+6A6B.1	fEnglish	[1] cross-grained; perverse [2] presumptuous and unreasonable [3] unexpected; uncalled for [4] violent; cross
U+6A6C.0	fUTF8	橬
U+6A6C.0	fCNS11643	3-512E
U+6A6C.0	fGB	3-4334
U+6A6C.0	fR/S	 75.12
U+6A6C.0	fTotalStrokes	 16
U+6A6C.0	fCangjie	DMUA
U+6A6C.0	fFourCorner	41960
U+6A6C.0	fCantonese	cim4
U+6A6C.0	fMandarin	cen2; chen2; chan2; qian2
U+6A6D.0	fUTF8	橭
U+6A6D.0	fCNS11643	2-5751
U+6A6D.0	fBig5	E9C6
U+6A6D.0	fGB	5-4555
U+6A6D.0	fR/S	 75.12
U+6A6D.0	fTotalStrokes	 16
U+6A6D.0	fCangjie	DJRJ
U+6A6D.0	fFourCorner	44941
U+6A6D.0	fCantonese	gu1; gwu1 (also); fu1
U+6A6D.0	fMandarin	gu1; ku1; hu4
U+6A6E.0	fUTF8	橮
U+6A6E.0	fCNS11643	3-5132
U+6A6E.0	fGB	5-4468
U+6A6E.0	fR/S	 75.12
U+6A6E.0	fTotalStrokes	 16
U+6A6E.0	fCangjie	DHHC
U+6A6E.0	fFourCorner	47980
U+6A6E.0	fHakka	  liu3; liu1; lieu3; leu1
U+6A6E.0	fCantonese	lau5
U+6A6E.0	fMandarin	liu3; ping2
U+6A6E.0	fEnglish	same as &#x67F3;
U+6A6F.0	fUTF8	橯
U+6A6F.0	fCNS11643	2-575F
U+6A6F.0	fBig5	E9D4
U+6A6F.0	fGB	5-4377
U+6A6F.0	fR/S	 75.12
U+6A6F.0	fTotalStrokes	 16
U+6A6F.0	fCangjie	DFFS
U+6A6F.0	fFourCorner	49927
U+6A6F.0	fCantonese	lou6
U+6A6F.0	fMandarin	lao4
U+6A70.0	fUTF8	橰
U+6A70.0	fCNS11643	3-5141
U+6A70.0	fGB	E-2E7A
U+6A70.0	fR/S	 75.12
U+6A70.0	fTotalStrokes	 16
U+6A70.0	fCangjie	DHUJ
U+6A70.0	fFourCorner	46940
U+6A70.0	fHakka	  gau1
U+6A70.0	fCantonese	gou1
U+6A70.0	fMandarin	gao1
U+6A70.0	fEnglish	a spar
U+6A71.0	fUTF8	橱
U+6A71.0	fCNS11643	3-513A
U+6A71.0	fGB	0-3377
U+6A71.0	fR/S	 75.12
U+6A71.0	fTotalStrokes	 16
U+6A71.0	fCangjie	DMMI
U+6A71.0	fFourCorner	41940
U+6A71.0	fHakka	 cu2; chu2
U+6A71.0	fCantonese	ceoi4; cyu4 (also)
U+6A71.0	fMandarin	chu2
U+6A71.0	fEnglish	cabinet, wardrobe, cupboard
U+6A72.0	fUTF8	橲
U+6A72.0	fCNS11643	F-547B
U+6A72.0	fGB	E-2E7B
U+6A72.0	fR/S	 75.12
U+6A72.0	fTotalStrokes	 16
U+6A72.0	fCangjie	DGRR
U+6A72.0	fFourCorner	4496
U+6A72.0	fCantonese	hei2
U+6A73.0	fUTF8	橳
U+6A73.0	fGB	E-2E7C
U+6A73.0	fR/S	 75.12
U+6A73.0	fTotalStrokes	 16
U+6A73.0	fCangjie	DBFS
U+6A73.0	fCantonese	sing3
U+6A74.0	fUTF8	橴
U+6A74.0	fGB	1-7D53
U+6A74.0	fR/S	 75.11
U+6A74.0	fTotalStrokes	 15
U+6A74.0	fCangjie	DYPF
U+6A74.0	fCantonese	zi2
U+6A74.0	fEnglish	 place name
U+6A75.0	fUTF8	橵
U+6A75.0	fGB	1-7D49
U+6A75.0	fR/S	 75.12
U+6A75.0	fTotalStrokes	 16
U+6A75.0	fCangjie	DTBK
U+6A75.0	fCantonese	saan3
U+6A75.0	fEnglish	 wood placed under roof tiles
U+6A76.0	fUTF8	橶
U+6A76.0	fCNS11643	2-5752
U+6A76.0	fBig5	E9C7
U+6A76.0	fGB	E-2E7D
U+6A76.0	fR/S	 75.12
U+6A76.0	fTotalStrokes	 16
U+6A76.0	fCangjie	DJJI
U+6A76.0	fFourCorner	43950
U+6A76.0	fCantonese	gik1
U+6A76.0	fMandarin	ji3; ji2
U+6A77.0	fUTF8	橷
U+6A77.0	fCNS11643	3-6531
U+6A77.0	fGB	E-2E7E
U+6A77.0	fR/S	 75.12
U+6A77.0	fTotalStrokes	 16
U+6A77.0	fCangjie	DLPN
U+6A77.0	fFourCorner	4191
U+6A77.0	fCantonese	dau1
U+6A77.0	fMandarin	dou1
U+6A78.0	fUTF8	橸
U+6A78.0	fCNS11643	F-5529
U+6A78.0	fGB	E-2F21
U+6A78.0	fR/S	 75.12
U+6A78.0	fTotalStrokes	 16
U+6A78.0	fCangjie	DAAA
U+6A78.0	fCantonese	zing1
U+6A79.0	fUTF8	橹
U+6A79.0	fGB	0-6956
U+6A79.0	fR/S	 75.12
U+6A79.0	fTotalStrokes	 16
U+6A79.0	fCangjie	XDNWA
U+6A79.0	fHakka	 lu1; lu3
U+6A79.0	fCantonese	lou5
U+6A79.0	fMandarin	lu3
U+6A79.0	fEnglish	simplified form of &#x6AD3;
U+6A7A.0	fUTF8	橺
U+6A7A.0	fCNS11643	3-6532
U+6A7A.0	fGB	E-2F22
U+6A7A.0	fR/S	 75.12
U+6A7A.0	fTotalStrokes	 16
U+6A7A.0	fCangjie	DANA
U+6A7A.0	fFourCorner	4792
U+6A7A.0	fHakka	 han3; gan3; kian1
U+6A7A.0	fCantonese	gaan3
U+6A7A.0	fEnglish	a partition; to separate
U+6A7B.0	fUTF8	橻
U+6A7B.0	fGB	1-7D5F
U+6A7B.0	fR/S	 75.12
U+6A7B.0	fTotalStrokes	 16
U+6A7B.0	fCangjie	DNGG
U+6A7B.0	fCantonese	cau2
U+6A7C.0	fUTF8	橼
U+6A7C.0	fGB	0-695A
U+6A7C.0	fR/S	 75.12
U+6A7C.0	fTotalStrokes	 16
U+6A7C.0	fCangjie	DVMO
U+6A7C.0	fHakka	 jen2
U+6A7C.0	fCantonese	jyun4
U+6A7C.0	fMandarin	yuan2
U+6A7C.0	fEnglish	simplified form of &#x6ADE;
U+6A7D.0	fUTF8	橽
U+6A7D.0	fCNS11643	3-5475
U+6A7D.0	fGB	5-4366
U+6A7D.0	fR/S	  75.13
U+6A7D.0	fTotalStrokes	 17
U+6A7D.0	fCangjie	DYGQ
U+6A7D.0	fFourCorner	44930
U+6A7D.0	fCantonese	taat3
U+6A7D.0	fMandarin	ta4
U+6A7E.0	fUTF8	橾
U+6A7E.0	fCNS11643	1-7378
U+6A7E.0	fBig5	C0CF
U+6A7E.0	fGB	3-4349
U+6A7E.0	fR/S	 75.13
U+6A7E.0	fTotalStrokes	 17
U+6A7E.0	fCangjie	DRRD
U+6A7E.0	fFourCorner	46994
U+6A7E.0	fCantonese	syu1; ciu1; sou1
U+6A7E.0	fMandarin	shu1; sou3; qiao1; sao1
U+6A7F.0	fUTF8	橿
U+6A7F.0	fCNS11643	2-5D31
U+6A7F.0	fBig5	ED45
U+6A7F.0	fGB	3-4345
U+6A7F.0	fR/S	 75.13
U+6A7F.0	fTotalStrokes	 17
U+6A7F.0	fCangjie	DMWM
U+6A7F.0	fFourCorner	41916
U+6A7F.0	fHakka	 giong1
U+6A7F.0	fCantonese	goeng1
U+6A7F.0	fMandarin	jiang1
U+6A7F.0	fEnglish	[1] handle of a hoe [2] a name for the everlasting wood used for certain parts of carts
U+6A80.0	fUTF8	檀
U+6A80.0	fCNS11643	1-7371
U+6A80.0	fBig5	C0C8
U+6A80.0	fGB	0-4C34
U+6A80.0	fR/S	 75.13
U+6A80.0	fTotalStrokes	 17
U+6A80.0	fCangjie	DYWM
U+6A80.0	fFourCorner	40916
U+6A80.0	fHakka	  tan2; tan3
U+6A80.0	fCantonese	taan4; taan2 (also)
U+6A80.0	fMandarin	tan2; shan4
U+6A80.0	fEnglish	[1] sandalwood; hardwood [2] a Chinese family name
U+6A81.0	fUTF8	檁
U+6A81.0	fCNS11643	2-5D22
U+6A81.0	fBig5	ECF5
U+6A81.0	fGB	E-2F23
U+6A81.0	fR/S	 75.13
U+6A81.0	fTotalStrokes	 17
U+6A81.0	fCangjie	DYWD
U+6A81.0	fFourCorner	40991
U+6A81.0	fHakka	  lim3
U+6A81.0	fCantonese	lam5
U+6A81.0	fMandarin	lin3
U+6A81.0	fEnglish	 the bole of a tree cross beam the combing round the hatches of a ship
U+6A82.0	fUTF8	檂
U+6A82.0	fCNS11643	3-547B
U+6A82.0	fGB	5-4372
U+6A82.0	fR/S	 75.13
U+6A82.0	fTotalStrokes	 17
U+6A82.0	fCangjie	DTWV
U+6A82.0	fFourCorner	45930
U+6A82.0	fCantonese	nung4; lung4 (also)
U+6A82.0	fMandarin	nong2
U+6A82.0	fEnglish	name of a tree
U+6A83.0	fUTF8	檃
U+6A83.0	fCNS11643	2-5D2D
U+6A83.0	fBig5	ED41
U+6A83.0	fGB	E-2F24
U+6A83.0	fR/S	 75.13
U+6A83.0	fTotalStrokes	 17
U+6A83.0	fCangjie	XNMD
U+6A83.0	fFourCorner	72904
U+6A83.0	fHakka	 jun3; run3; rin1; jin1
U+6A83.0	fCantonese	jan2
U+6A83.0	fMandarin	yin3; yin4
U+6A84.0	fUTF8	檄
U+6A84.0	fCNS11643	1-7373
U+6A84.0	fBig5	C0CA
U+6A84.0	fGB	0-4F2D
U+6A84.0	fR/S	 75.13
U+6A84.0	fTotalStrokes	 17
U+6A84.0	fCangjie	DHSK
U+6A84.0	fFourCorner	48940
U+6A84.0	fHakka	  git7; hit7
U+6A84.0	fCantonese	hat6
U+6A84.0	fMandarin	xi2; xiao4
U+6A84.0	fEnglish	 call arms; summons to arms
U+6A85.0	fUTF8	檅
U+6A85.0	fCNS11643	2-5D34
U+6A85.0	fBig5	ED48
U+6A85.0	fGB	5-436A
U+6A85.0	fR/S	 75.13
U+6A85.0	fTotalStrokes	 17
U+6A85.0	fCangjie	DYMH
U+6A85.0	fFourCorner	41953
U+6A85.0	fCantonese	seoi6; wai6
U+6A85.0	fMandarin	sui4
U+6A86.0	fUTF8	檆
U+6A86.0	fCNS11643	3-5521
U+6A86.0	fGB	5-457B
U+6A86.0	fR/S	  75.13
U+6A86.0	fTotalStrokes	 17
U+6A86.0	fCangjie	DFFR
U+6A86.0	fFourCorner	41960
U+6A86.0	fHakka	  cam5;  cam1; cam6; cam3
U+6A86.0	fCantonese	caam3; saam1
U+6A86.0	fMandarin	shan1; sha1
U+6A86.0	fEnglish	same as &#x6749;
U+6A87.0	fUTF8	檇
U+6A87.0	fCNS11643	2-5D29
U+6A87.0	fBig5	ECFC
U+6A87.0	fGB	E-2F25
U+6A87.0	fR/S	 75.13
U+6A87.0	fTotalStrokes	 17
U+6A87.0	fCangjie	DOGS
U+6A87.0	fFourCorner	40927
U+6A87.0	fHakka	 zui1; zui5; zui6
U+6A87.0	fCantonese	zeoi3
U+6A87.0	fMandarin	zui4; zui1; cui2
U+6A87.0	fEnglish	 wooden rammer or pestle place
U+6A88.0	fUTF8	檈
U+6A88.0	fCNS11643	3-547C
U+6A88.0	fGB	3-434A
U+6A88.0	fR/S	 75.13
U+6A88.0	fTotalStrokes	 17
U+6A88.0	fCangjie	DWLV
U+6A88.0	fFourCorner	46930
U+6A88.0	fCantonese	syun4
U+6A88.0	fMandarin	xuan2; xun2; xuan4
U+6A89.0	fUTF8	檉
U+6A89.0	fCNS11643	2-5D24
U+6A89.0	fBig5	ECF7
U+6A89.0	fGB	1-685F
U+6A89.0	fR/S	 75.13
U+6A89.0	fTotalStrokes	 17
U+6A89.0	fCangjie	DSRG
U+6A89.0	fFourCorner	46914
U+6A89.0	fHakka	 shin5; siin5
U+6A89.0	fCantonese	cing1
U+6A89.0	fMandarin	cheng1
U+6A89.0	fEnglish	the tamarisk
U+6A8A.0	fUTF8	檊
U+6A8A.0	fCNS11643	3-5476
U+6A8A.0	fGB	3-4344
U+6A8A.0	fR/S	  75.13
U+6A8A.0	fTotalStrokes	 17
U+6A8A.0	fCangjie	DJJJ
U+6A8A.0	fFourCorner	48940
U+6A8A.0	fCantonese	gon1
U+6A8A.0	fMandarin	gan4
U+6A8A.0	fEnglish	name of a tree
U+6A8B.0	fUTF8	檋
U+6A8B.0	fCNS11643	4-5645
U+6A8B.0	fGB	3-4259
U+6A8B.0	fR/S	 75.13
U+6A8B.0	fTotalStrokes	 17; 16
U+6A8B.0	fCangjie	DTCJ
U+6A8B.0	fFourCorner	44950
U+6A8B.0	fCantonese	guk6
U+6A8B.0	fMandarin	ju1; ju2
U+6A8C.0	fUTF8	檌
U+6A8C.0	fCNS11643	2-5D35
U+6A8C.0	fBig5	ED49
U+6A8C.0	fGB	5-4575
U+6A8C.0	fR/S	 75.13
U+6A8C.0	fTotalStrokes	 17
U+6A8C.0	fCangjie	DWLY
U+6A8C.0	fFourCorner	46911
U+6A8C.0	fCantonese	seoi4; zeoi6
U+6A8C.0	fMandarin	zui4
U+6A8D.0	fUTF8	檍
U+6A8D.0	fCNS11643	2-5C7E
U+6A8D.0	fBig5	ECF3
U+6A8D.0	fGB	3-434D
U+6A8D.0	fR/S	 75.13
U+6A8D.0	fTotalStrokes	 17
U+6A8D.0	fCangjie	DYTP
U+6A8D.0	fFourCorner	40936
U+6A8D.0	fHakka	 jit7; rit7
U+6A8D.0	fCantonese	jik1
U+6A8D.0	fMandarin	yi4
U+6A8D.0	fEnglish	ilex
U+6A8E.0	fUTF8	檎
U+6A8E.0	fCNS11643	2-5D2B
U+6A8E.0	fBig5	ECFE
U+6A8E.0	fGB	0-6955
U+6A8E.0	fR/S	 75.13
U+6A8E.0	fTotalStrokes	 17
U+6A8E.0	fCangjie	DOYB
U+6A8E.0	fFourCorner	48927
U+6A8E.0	fHakka	  kim2
U+6A8E.0	fCantonese	kam4
U+6A8E.0	fMandarin	qin2
U+6A8E.0	fEnglish	a small red apple (Malus asiatica)
U+6A8F.0	fUTF8	檏
U+6A8F.0	fCNS11643	3-5527
U+6A8F.0	fGB	E-2F26
U+6A8F.0	fR/S	 75.13
U+6A8F.0	fTotalStrokes	 17
U+6A8F.0	fCangjie	DTCD
U+6A8F.0	fFourCorner	42990
U+6A8F.0	fHakka	 pok7;  puk7
U+6A8F.0	fCantonese	buk6; pok3; po1; piu4; pok2 (also); buk1
U+6A8F.0	fMandarin	pu3; pu2; po4; bu2
U+6A8F.0	fEnglish	same as &#x6A38;
U+6A90.0	fUTF8	檐
U+6A90.0	fCNS11643	1-737A
U+6A90.0	fBig5	C0D1
U+6A90.0	fGB	0-695C
U+6A90.0	fR/S	 75.13
U+6A90.0	fTotalStrokes	 17
U+6A90.0	fCangjie	DNCR
U+6A90.0	fFourCorner	47961
U+6A90.0	fHakka	 jam2;  dam5; ram2; jang2
U+6A90.0	fCantonese	jim4; jam4 (also); sim4 (also); jam2 (also); gim2
U+6A90.0	fMandarin	yan2; dan4; dan1
U+6A90.0	fEnglish	[1] eaves of a house [2] brim
U+6A91.0	fUTF8	檑
U+6A91.0	fCNS11643	2-5D30
U+6A91.0	fBig5	ED44
U+6A91.0	fGB	0-695B
U+6A91.0	fR/S	 75.13
U+6A91.0	fTotalStrokes	 17
U+6A91.0	fCangjie	DMBW
U+6A91.0	fFourCorner	41962
U+6A91.0	fHakka	 lui2
U+6A91.0	fCantonese	leoi4; leoi6
U+6A91.0	fMandarin	lei2; lei4
U+6A91.0	fEnglish	name of a tree
U+6A92.0	fUTF8	檒
U+6A92.0	fCNS11643	2-5D36
U+6A92.0	fBig5	ED4A
U+6A92.0	fGB	5-5050
U+6A92.0	fR/S	 75.13
U+6A92.0	fTotalStrokes	 17
U+6A92.0	fCangjie	DDHNI
U+6A92.0	fFourCorner	44217
U+6A92.0	fHakka	 fung1
U+6A92.0	fCantonese	fung1
U+6A92.0	fMandarin	feng1
U+6A93.0	fUTF8	檓
U+6A93.0	fCNS11643	2-5D2A
U+6A93.0	fBig5	ECFD
U+6A93.0	fGB	5-4577
U+6A93.0	fR/S	 75.13
U+6A93.0	fTotalStrokes	 17
U+6A93.0	fCangjie	DHGE
U+6A93.0	fFourCorner	47947
U+6A93.0	fCantonese	wai2
U+6A93.0	fMandarin	hui3
U+6A93.0	fEnglish	name of a tree
U+6A94.0	fUTF8	檔
U+6A94.0	fCNS11643	1-7372
U+6A94.0	fBig5	C0C9
U+6A94.0	fGB	1-3535
U+6A94.0	fR/S	 75.13
U+6A94.0	fTotalStrokes	 17
U+6A94.0	fCangjie	DFBW
U+6A94.0	fFourCorner	49977
U+6A94.0	fHakka	 dong5;  dong1
U+6A94.0	fCantonese	dong3 (also); dong2
U+6A94.0	fMandarin	dang3; dang1; dang4
U+6A94.0	fEnglish	[1] shelf; pigeonholes [2] a wooden crosspiece (as the rung of a ladder, etc.) [3] wooden chair [4] an abbreviation for files
U+6A95.0	fUTF8	檕
U+6A95.0	fCNS11643	2-5D2C
U+6A95.0	fBig5	ED40
U+6A95.0	fGB	E-2F27
U+6A95.0	fR/S	 75.13
U+6A95.0	fTotalStrokes	 17
U+6A95.0	fCangjie	JED
U+6A95.0	fFourCorner	57904
U+6A95.0	fHakka	 hi5; he5
U+6A95.0	fCantonese	hai6
U+6A95.0	fMandarin	ji4; qi1; xi4
U+6A95.0	fEnglish	[1] the pivot in a well-sweep [2] a fruit like a plum
U+6A96.0	fUTF8	檖
U+6A96.0	fCNS11643	2-5D21
U+6A96.0	fBig5	ECF4
U+6A96.0	fGB	3-433E
U+6A96.0	fR/S	 75.13
U+6A96.0	fTotalStrokes	 17
U+6A96.0	fCangjie	DYTO
U+6A96.0	fFourCorner	48933
U+6A96.0	fHakka	 sui5; sui6
U+6A96.0	fCantonese	seoi6
U+6A96.0	fMandarin	sui4
U+6A96.0	fEnglish	wild pear tree
U+6A97.0	fUTF8	檗
U+6A97.0	fCNS11643	1-7379
U+6A97.0	fBig5	C0D0
U+6A97.0	fGB	0-695E
U+6A97.0	fR/S	 75.13
U+6A97.0	fTotalStrokes	 17
U+6A97.0	fCangjie	SJD
U+6A97.0	fFourCorner	70904
U+6A97.0	fHakka	  bak7;  pit7
U+6A97.0	fCantonese	paak3; baak3 (also)
U+6A97.0	fMandarin	bo4; bi4
U+6A97.0	fEnglish	tree
U+6A98.0	fUTF8	檘
U+6A98.0	fCNS11643	3-5523
U+6A98.0	fGB	E-2F28
U+6A98.0	fR/S	 75.13
U+6A98.0	fTotalStrokes	 17
U+6A98.0	fCangjie	DSRJ
U+6A98.0	fFourCorner	40940
U+6A98.0	fHakka	 bak7; pin2
U+6A98.0	fCantonese	ping4
U+6A98.0	fMandarin	bo4; ping2; bi4
U+6A98.0	fEnglish	[1] same as &#x6A97; [2] same as &#x67B0;
U+6A99.0	fUTF8	檙
U+6A99.0	fCNS11643	3-547A
U+6A99.0	fGB	5-455C
U+6A99.0	fR/S	 75.13
U+6A99.0	fTotalStrokes	 17
U+6A99.0	fCangjie	DBUL
U+6A99.0	fFourCorner	42927
U+6A99.0	fCantonese	caang4
U+6A99.0	fMandarin	ding3
U+6A9A.0	fUTF8	檚
U+6A9A.0	fCNS11643	2-5D33
U+6A9A.0	fBig5	ED47
U+6A9A.0	fGB	5-456F
U+6A9A.0	fR/S	 75.13
U+6A9A.0	fTotalStrokes	 17
U+6A9A.0	fCangjie	DDDO
U+6A9A.0	fFourCorner	44981
U+6A9A.0	fHakka	 cih3
U+6A9A.0	fCantonese	co2
U+6A9A.0	fMandarin	chu3; chu4
U+6A9A.0	fEnglish	[1] name of a tree [2] non-classical form of &#x695A;
U+6A9B.0	fUTF8	檛
U+6A9B.0	fCNS11643	2-5D26
U+6A9B.0	fBig5	ECF9
U+6A9B.0	fGB	3-4130
U+6A9B.0	fR/S	 75.13
U+6A9B.0	fTotalStrokes	 17
U+6A9B.0	fCangjie	DYBB
U+6A9B.0	fFourCorner	47932
U+6A9B.0	fHakka	  ko1
U+6A9B.0	fCantonese	zaa1
U+6A9B.0	fMandarin	zhua1
U+6A9B.0	fEnglish	[1] switch [2] riding whip
U+6A9C.0	fUTF8	檜
U+6A9C.0	fCNS11643	1-7375
U+6A9C.0	fBig5	C0CC
U+6A9C.0	fGB	1-686D
U+6A9C.0	fR/S	 75.13
U+6A9C.0	fTotalStrokes	 17
U+6A9C.0	fCangjie	DOMA
U+6A9C.0	fFourCorner	48966
U+6A9C.0	fHakka	  kwai5; kai5
U+6A9C.0	fCantonese	kui2; kwui2 (also); kui3
U+6A9C.0	fMandarin	gui4; kuai4; gua1; hui4
U+6A9C.0	fEnglish	the Chinese cypress; the Chinese juniper
U+6A9D.0	fUTF8	檝
U+6A9D.0	fCNS11643	3-5524
U+6A9D.0	fGB	E-2F29
U+6A9D.0	fR/S	  75.13
U+6A9D.0	fTotalStrokes	 17
U+6A9D.0	fCangjie	DRJI
U+6A9D.0	fFourCorner	43950
U+6A9D.0	fCantonese	zip3
U+6A9D.0	fMandarin	ji2
U+6A9D.0	fEnglish	oar
U+6A9E.0	fUTF8	檞
U+6A9E.0	fCNS11643	2-5D28
U+6A9E.0	fBig5	ECFB
U+6A9E.0	fGB	3-434C
U+6A9E.0	fR/S	 75.13
U+6A9E.0	fTotalStrokes	 17
U+6A9E.0	fCangjie	DNBQ
U+6A9E.0	fFourCorner	47952
U+6A9E.0	fCantonese	gaai2; haai5 (also)
U+6A9E.0	fMandarin	jie3; xie4
U+6A9E.0	fEnglish	a kind of oak
U+6A9F.0	fUTF8	檟
U+6A9F.0	fCNS11643	2-5D25
U+6A9F.0	fBig5	ECF8
U+6A9F.0	fGB	3-425F
U+6A9F.0	fR/S	 75.13
U+6A9F.0	fTotalStrokes	 17
U+6A9F.0	fCangjie	DMWC
U+6A9F.0	fFourCorner	41986
U+6A9F.0	fHakka	 ga3; ga5
U+6A9F.0	fCantonese	gaa2
U+6A9F.0	fMandarin	jia3
U+6A9F.0	fEnglish	[1] a small evergreen shrub (Mallotus japonicus) [2] ancient form of @ ( tea)
U+6AA0.0	fUTF8	檠
U+6AA0.0	fCNS11643	1-737B
U+6AA0.0	fBig5	C0D2
U+6AA0.0	fGB	0-6951
U+6AA0.0	fR/S	 75.13
U+6AA0.0	fTotalStrokes	 17
U+6AA0.0	fCangjie	TKD
U+6AA0.0	fFourCorner	48904
U+6AA0.0	fHakka	 kiang2; kiong1; gin3
U+6AA0.0	fCantonese	king4
U+6AA0.0	fMandarin	qing2; jing4; jing3
U+6AA0.0	fEnglish	 stand for lamp frame for bow
U+6AA1.0	fUTF8	檡
U+6AA1.0	fCNS11643	2-5D27
U+6AA1.0	fBig5	ECFA
U+6AA1.0	fGB	3-4123
U+6AA1.0	fR/S	 75.13
U+6AA1.0	fTotalStrokes	 17
U+6AA1.0	fCangjie	DWLJ
U+6AA1.0	fFourCorner	46941
U+6AA1.0	fCantonese	sik1; zaak6; tou4
U+6AA1.0	fMandarin	zhe4; shi4; zhai2; duo2; yi4; tu2
U+6AA1.0	fEnglish	[1] name of a tree [2] name of a tree [3] name of a fruit tree
U+6AA2.0	fUTF8	檢
U+6AA2.0	fCNS11643	1-7374
U+6AA2.0	fBig5	C0CB
U+6AA2.0	fGB	1-3C6C
U+6AA2.0	fR/S	 75.13
U+6AA2.0	fTotalStrokes	 17
U+6AA2.0	fCangjie	DOMO
U+6AA2.0	fFourCorner	48986
U+6AA2.0	fHakka	  giam3
U+6AA2.0	fCantonese	gim2
U+6AA2.0	fMandarin	jian3
U+6AA2.0	fEnglish	[1] book label [2] sort; gather [3] inspect; check up; examine; collate [4] discuss thoroughly [5] form; pattern [6] restrict; regulate
U+6AA3.0	fUTF8	檣
U+6AA3.0	fCNS11643	1-7377
U+6AA3.0	fBig5	C0CE
U+6AA3.0	fGB	1-6949
U+6AA3.0	fR/S	 75.13
U+6AA3.0	fTotalStrokes	 17
U+6AA3.0	fCangjie	DGOW
U+6AA3.0	fFourCorner	44961
U+6AA3.0	fHakka	 siong2;  ciong2
U+6AA3.0	fCantonese	coeng4
U+6AA3.0	fMandarin	qiang2
U+6AA3.0	fEnglish	 mast of a ship
U+6AA4.0	fUTF8	檤
U+6AA4.0	fCNS11643	2-5D2F
U+6AA4.0	fBig5	ED43
U+6AA4.0	fGB	5-4567
U+6AA4.0	fR/S	 75.13
U+6AA4.0	fTotalStrokes	 17
U+6AA4.0	fCangjie	DYTU
U+6AA4.0	fFourCorner	48936
U+6AA4.0	fCantonese	dou6
U+6AA4.0	fMandarin	dao4
U+6AA4.0	fEnglish	name of a tree
U+6AA5.0	fUTF8	檥
U+6AA5.0	fCNS11643	2-5D23
U+6AA5.0	fBig5	ECF6
U+6AA5.0	fGB	5-433D
U+6AA5.0	fR/S	 75.13
U+6AA5.0	fTotalStrokes	 17
U+6AA5.0	fCangjie	DTGI
U+6AA5.0	fFourCorner	48953
U+6AA5.0	fHakka	 ngi5; ngi6; ngi5ngi3
U+6AA5.0	fCantonese	ngai5; ai5 (also)
U+6AA5.0	fMandarin	yi3; e2; yi2
U+6AA5.0	fEnglish	 moor a boat to the bank
U+6AA6.0	fUTF8	檦
U+6AA6.0	fCNS11643	2-5D32
U+6AA6.0	fBig5	ED46
U+6AA6.0	fGB	5-4572
U+6AA6.0	fR/S	 75.13
U+6AA6.0	fTotalStrokes	 17
U+6AA6.0	fCangjie	DMFN
U+6AA6.0	fFourCorner	41991
U+6AA6.0	fCantonese	biu2
U+6AA6.0	fMandarin	biao3
U+6AA7.0	fUTF8	檧
U+6AA7.0	fCNS11643	3-5478
U+6AA7.0	fGB	3-4331
U+6AA7.0	fR/S	 75.13
U+6AA7.0	fTotalStrokes	 17
U+6AA7.0	fCangjie	DTPP
U+6AA7.0	fFourCorner	44930
U+6AA7.0	fCantonese	sung1
U+6AA7.0	fMandarin	song1; song3
U+6AA8.0	fUTF8	檨
U+6AA8.0	fCNS11643	2-5D2E
U+6AA8.0	fBig5	ED42
U+6AA8.0	fGB	3-433C
U+6AA8.0	fR/S	 75.13
U+6AA8.0	fTotalStrokes	 17
U+6AA8.0	fCangjie	DTGO
U+6AA8.0	fFourCorner	48982
U+6AA8.0	fCantonese	ce1; se1
U+6AA8.0	fMandarin	she1
U+6AA8.0	fEnglish	name of a fruit tree
U+6AA9.0	fUTF8	檩
U+6AA9.0	fGB	0-695D
U+6AA9.0	fR/S	 75.13
U+6AA9.0	fTotalStrokes	 17
U+6AA9.0	fCangjie	DYWF
U+6AA9.0	fHakka	 lin3
U+6AA9.0	fCantonese	lam5
U+6AA9.0	fMandarin	lin3
U+6AA9.0	fEnglish	a cross beam in a house
U+6AAA.0	fUTF8	檪
U+6AAA.0	fCNS11643	3-5529
U+6AAA.0	fGB	E-2F2A
U+6AAA.0	fR/S	 75.13
U+6AAA.0	fTotalStrokes	 17
U+6AAA.0	fCangjie	DIOD
U+6AAA.0	fFourCorner	42990
U+6AAA.0	fCantonese	lik1
U+6AAA.0	fMandarin	li4; yue4
U+6AAA.0	fEnglish	 chestnut-leaved oak oak
U+6AAB.0	fUTF8	檫
U+6AAB.0	fCNS11643	3-5774
U+6AAB.0	fGB	0-695F
U+6AAB.0	fR/S	 75.14
U+6AAB.0	fTotalStrokes	 18
U+6AAB.0	fCangjie	DJBF
U+6AAB.0	fFourCorner	43990
U+6AAB.0	fCantonese	caat3
U+6AAB.0	fMandarin	cha2; sa4
U+6AAB.0	fEnglish	sassafras tzumu
U+6AAC.0	fUTF8	檬
U+6AAC.0	fCNS11643	1-764F
U+6AAC.0	fBig5	C263
U+6AAC.0	fGB	0-434A
U+6AAC.0	fR/S	 75.14
U+6AAC.0	fTotalStrokes	 18
U+6AAC.0	fCangjie	DTBO
U+6AAC.0	fFourCorner	44932
U+6AAC.0	fHakka	  mung2; mung3
U+6AAC.0	fCantonese	mung1; mung4 (also)
U+6AAC.0	fMandarin	meng2
U+6AAC.0	fEnglish	[1] a species of locust or acacia [2] the lemon
U+6AAD.0	fUTF8	檭
U+6AAD.0	fCNS11643	2-6173
U+6AAD.0	fBig5	EFE7
U+6AAD.0	fGB	5-4541
U+6AAD.0	fR/S	 75.14
U+6AAD.0	fTotalStrokes	 18
U+6AAD.0	fCangjie	DCAV
U+6AAD.0	fFourCorner	57932
U+6AAD.0	fCantonese	ngan4; an4 (also)
U+6AAD.0	fMandarin	yin2
U+6AAD.0	fEnglish	name of a tree
U+6AAE.0	fUTF8	檮
U+6AAE.0	fCNS11643	1-7654
U+6AAE.0	fBig5	C268
U+6AAE.0	fGB	3-4145
U+6AAE.0	fR/S	 75.14
U+6AAE.0	fTotalStrokes	 18
U+6AAE.0	fCangjie	DGNI
U+6AAE.0	fFourCorner	44941
U+6AAE.0	fHakka	 tau2; to2
U+6AAE.0	fCantonese	tou4; to4
U+6AAE.0	fMandarin	tao2; dao3; chou2; dao4
U+6AAE.0	fEnglish	blockhead; ignorant and stupid
U+6AAF.0	fUTF8	檯
U+6AAF.0	fCNS11643	1-7655
U+6AAF.0	fBig5	C269
U+6AAF.0	fGB	1-786D
U+6AAF.0	fR/S	 75.14
U+6AAF.0	fTotalStrokes	 18
U+6AAF.0	fCangjie	DGRG
U+6AAF.0	fFourCorner	44914
U+6AAF.0	fHakka	 toi2; toi3
U+6AAF.0	fCantonese	toi4 (also); toi2
U+6AAF.0	fMandarin	tai2
U+6AAF.0	fEnglish	 table
U+6AB0.0	fUTF8	檰
U+6AB0.0	fCNS11643	3-5777
U+6AB0.0	fGB	3-432E
U+6AB0.0	fR/S	  75.14
U+6AB0.0	fTotalStrokes	 18
U+6AB0.0	fCangjie	DVFB
U+6AB0.0	fFourCorner	46920
U+6AB0.0	fCantonese	min4
U+6AB0.0	fMandarin	mian2
U+6AB0.0	fEnglish	name of a tree
U+6AB1.0	fUTF8	檱
U+6AB1.0	fCNS11643	3-5821
U+6AB1.0	fGB	E-2F2B
U+6AB1.0	fR/S	 75.14
U+6AB1.0	fTotalStrokes	 18
U+6AB1.0	fCangjie	DHTC
U+6AB1.0	fFourCorner	48980
U+6AB1.0	fHakka	  ki2; ki3
U+6AB1.0	fCantonese	kei4; kei2 (also)
U+6AB1.0	fMandarin	qi2; ji1
U+6AB1.0	fEnglish	same as &#x68CB;
U+6AB2.0	fUTF8	檲
U+6AB2.0	fCNS11643	3-5776
U+6AB2.0	fGB	E-2F2C
U+6AB2.0	fR/S	 75.14
U+6AB2.0	fTotalStrokes	 18
U+6AB2.0	fCangjie	DWJI
U+6AB2.0	fFourCorner	46900
U+6AB2.0	fCantonese	tyun4
U+6AB2.0	fMandarin	tuan2
U+6AB2.0	fEnglish	name of a large tree
U+6AB3.0	fUTF8	檳
U+6AB3.0	fCNS11643	1-764E
U+6AB3.0	fBig5	C262
U+6AB3.0	fGB	1-6944
U+6AB3.0	fR/S	 75.14
U+6AB3.0	fTotalStrokes	 18
U+6AB3.0	fCangjie	DJMC
U+6AB3.0	fFourCorner	43986
U+6AB3.0	fHakka	  bin1
U+6AB3.0	fCantonese	ban1; bing1
U+6AB3.0	fMandarin	bin1; bing1
U+6AB3.0	fEnglish	the betelnut; areca nut; areca; betel
U+6AB4.0	fUTF8	檴
U+6AB4.0	fCNS11643	2-6172
U+6AB4.0	fBig5	EFE6
U+6AB4.0	fGB	3-4343
U+6AB4.0	fR/S	 75.14; 75.13
U+6AB4.0	fTotalStrokes	 18
U+6AB4.0	fCangjie	DTOE
U+6AB4.0	fFourCorner	44947
U+6AB4.0	fHakka	 wok7; vok7
U+6AB4.0	fCantonese	wok6
U+6AB4.0	fMandarin	huo4; hua4
U+6AB4.0	fEnglish	tree
U+6AB5.0	fUTF8	檵
U+6AB5.0	fCNS11643	3-5778
U+6AB5.0	fGB	3-4355
U+6AB5.0	fR/S	 75.14
U+6AB5.0	fTotalStrokes	 18
U+6AB5.0	fCangjie	DVVI
U+6AB5.0	fFourCorner	42910
U+6AB5.0	fHakka	  gi5
U+6AB5.0	fCantonese	gai3
U+6AB5.0	fMandarin	ji4; qi3
U+6AB6.0	fUTF8	檶
U+6AB6.0	fCNS11643	2-616F
U+6AB6.0	fBig5	EFE3
U+6AB6.0	fGB	5-437A
U+6AB6.0	fR/S	 75.14
U+6AB6.0	fTotalStrokes	 18
U+6AB6.0	fCangjie	DKSR
U+6AB6.0	fFourCorner	44916
U+6AB6.0	fCantonese	him1
U+6AB6.0	fMandarin	qian1
U+6AB7.0	fUTF8	檷
U+6AB7.0	fCNS11643	2-6170
U+6AB7.0	fBig5	EFE4
U+6AB7.0	fGB	3-407A
U+6AB7.0	fR/S	 75.14
U+6AB7.0	fTotalStrokes	 18
U+6AB7.0	fCangjie	DMFB
U+6AB7.0	fFourCorner	41927
U+6AB7.0	fCantonese	nai5; lai5 (also); nei5
U+6AB7.0	fMandarin	mi2; ni3
U+6AB8.0	fUTF8	檸
U+6AB8.0	fCNS11643	1-7652
U+6AB8.0	fBig5	C266
U+6AB8.0	fGB	1-447B
U+6AB8.0	fR/S	 75.14
U+6AB8.0	fTotalStrokes	 18
U+6AB8.0	fCangjie	DJPN
U+6AB8.0	fFourCorner	43921
U+6AB8.0	fHakka	  nen2; nen3; lin2; len2
U+6AB8.0	fCantonese	ning4; ling4 (also)
U+6AB8.0	fMandarin	ning2; neng3
U+6AB8.0	fEnglish	the lemon
U+6AB9.0	fUTF8	檹
U+6AB9.0	fCNS11643	2-616A
U+6AB9.0	fBig5	EFDE
U+6AB9.0	fGB	5-4625
U+6AB9.0	fR/S	 75.14
U+6AB9.0	fTotalStrokes	 18
U+6AB9.0	fCangjie	DYSR
U+6AB9.0	fFourCorner	48921
U+6AB9.0	fCantonese	ji1
U+6AB9.0	fMandarin	yi1; yi3
U+6ABA.0	fUTF8	檺
U+6ABA.0	fCNS11643	2-616E
U+6ABA.0	fBig5	EFE2
U+6ABA.0	fGB	E-2F2D
U+6ABA.0	fR/S	 75.14
U+6ABA.0	fTotalStrokes	 18
U+6ABA.0	fCangjie	DYRO
U+6ABA.0	fFourCorner	40932
U+6ABA.0	fCantonese	gou2
U+6ABA.0	fMandarin	gao3
U+6ABA.0	fEnglish	name of a tree
U+6ABB.0	fUTF8	檻
U+6ABB.0	fCNS11643	1-7651
U+6ABB.0	fBig5	C265
U+6ABB.0	fGB	1-3C77
U+6ABB.0	fR/S	 75.14
U+6ABB.0	fTotalStrokes	 18
U+6ABB.0	fCangjie	DSIT
U+6ABB.0	fFourCorner	48917
U+6ABB.0	fHakka	 gam5;  kiam1;  lam3; kiam2
U+6ABB.0	fCantonese	laam6; haam5; kaam5 (also)
U+6ABB.0	fMandarin	jian4; xian3
U+6ABB.0	fEnglish	railings; bars (as window or door bars)
U+6ABB.1	fUTF8	檻
U+6ABB.1	fCNS11643	1-7651
U+6ABB.1	fBig5	C265
U+6ABB.1	fGB	1-3C77
U+6ABB.1	fR/S	 75.14
U+6ABB.1	fTotalStrokes	 18
U+6ABB.1	fCangjie	DSIT
U+6ABB.1	fFourCorner	48917
U+6ABB.1	fMandarin	kan3
U+6ABB.1	fEnglish	 threshold; door-sill
U+6ABC.0	fUTF8	檼
U+6ABC.0	fCNS11643	3-577E
U+6ABC.0	fGB	5-4623
U+6ABC.0	fR/S	  75.14
U+6ABC.0	fTotalStrokes	 18
U+6ABC.0	fCangjie	DBMP
U+6ABC.0	fFourCorner	42930
U+6ABC.0	fHakka	 jun3
U+6ABC.0	fCantonese	jan2
U+6ABC.0	fMandarin	yin3; yin4
U+6ABC.0	fEnglish	 ridge pole shape wood by use of heat tool for shaping bent wood
U+6ABD.0	fUTF8	檽
U+6ABD.0	fCNS11643	2-616B
U+6ABD.0	fBig5	EFDF
U+6ABD.0	fGB	3-4351
U+6ABD.0	fR/S	 75.14
U+6ABD.0	fTotalStrokes	 18
U+6ABD.0	fCangjie	DMBB
U+6ABD.0	fFourCorner	41927
U+6ABD.0	fHakka	 ngieu3;  neu3; neu5
U+6ABD.0	fCantonese	nau6; lau6 (also); nau2
U+6ABD.0	fMandarin	er2; ruan3; nou4; ru2; nuu3
U+6ABD.0	fEnglish	kind of pine, used for coffins
U+6ABE.0	fUTF8	檾
U+6ABE.0	fCNS11643	3-5779
U+6ABE.0	fGB	5-6950
U+6ABE.0	fR/S	 75.14
U+6ABE.0	fTotalStrokes	 18
U+6ABE.0	fCangjie	FFBDD
U+6ABE.0	fFourCorner	99990
U+6ABE.0	fHakka	 ken3;  kwen3
U+6ABE.0	fCantonese	king2; gwing2 (also)
U+6ABE.0	fMandarin	jiong3; qing3
U+6ABE.0	fEnglish	grassy plant with fibres from which cloth is made
U+6ABF.0	fUTF8	檿
U+6ABF.0	fCNS11643	3-5823
U+6ABF.0	fGB	3-303B
U+6ABF.0	fR/S	  75.14
U+6ABF.0	fTotalStrokes	 18
U+6ABF.0	fCangjie	MKD
U+6ABF.0	fFourCorner	71900
U+6ABF.0	fHakka	 ram6; jam5; jam3
U+6ABF.0	fCantonese	jim2
U+6ABF.0	fMandarin	yan3
U+6ABF.0	fEnglish	mulberry
U+6AC0.0	fUTF8	櫀
U+6AC0.0	fCNS11643	3-577D
U+6AC0.0	fGB	5-457D
U+6AC0.0	fR/S	 75.14
U+6AC0.0	fTotalStrokes	 18
U+6AC0.0	fCangjie	DTCF
U+6AC0.0	fFourCorner	44990
U+6AC0.0	fHakka	 ki2
U+6AC0.0	fCantonese	kei4
U+6AC0.0	fMandarin	qi2
U+6AC1.0	fUTF8	櫁
U+6AC1.0	fCNS11643	3-577A
U+6AC1.0	fGB	5-4626
U+6AC1.0	fR/S	  75.14
U+6AC1.0	fTotalStrokes	 18
U+6AC1.0	fCangjie	DJPI
U+6AC1.0	fFourCorner	43930
U+6AC1.0	fHakka	 mit8; met8
U+6AC1.0	fCantonese	mat6
U+6AC1.0	fMandarin	mi4
U+6AC1.0	fEnglish	same as &#x6A12;
U+6AC2.0	fUTF8	櫂
U+6AC2.0	fCNS11643	1-7653
U+6AC2.0	fBig5	C267
U+6AC2.0	fGB	E-2F2E
U+6AC2.0	fR/S	 75.14
U+6AC2.0	fTotalStrokes	 18
U+6AC2.0	fCangjie	DSMG
U+6AC2.0	fFourCorner	47914
U+6AC2.0	fHakka	  cau5; tau5; cau6; tau6; zok7
U+6AC2.0	fCantonese	zaau6
U+6AC2.0	fMandarin	zhao4; zhuo2; zhuo1; zhe4; zhai2; di2
U+6AC2.0	fEnglish	[1] oar; paddle [2] row [3] general name of boat
U+6AC3.0	fUTF8	櫃
U+6AC3.0	fCNS11643	1-7650
U+6AC3.0	fBig5	C264
U+6AC3.0	fGB	1-3971
U+6AC3.0	fR/S	 75.14
U+6AC3.0	fTotalStrokes	 18
U+6AC3.0	fCangjie	DSLC
U+6AC3.0	fFourCorner	41918
U+6AC3.0	fHakka	 kui5; kui6; kui3; kwi5; k'wui5
U+6AC3.0	fCantonese	gwai6; gwai2 (also)
U+6AC3.0	fMandarin	gui4
U+6AC3.0	fEnglish	[1] cupboard; wardrobe; cabinet [2] shop counter
U+6AC4.0	fUTF8	櫄
U+6AC4.0	fCNS11643	3-5822
U+6AC4.0	fGB	5-4621
U+6AC4.0	fR/S	  75.14
U+6AC4.0	fTotalStrokes	 18
U+6AC4.0	fCangjie	DHGF
U+6AC4.0	fFourCorner	42930
U+6AC4.0	fHakka	 chun1; cun1
U+6AC4.0	fCantonese	ceon1
U+6AC4.0	fMandarin	chun1
U+6AC4.0	fEnglish	the wild varnish tree, wood used for making musical instruments
U+6AC5.0	fUTF8	櫅
U+6AC5.0	fCNS11643	2-6169
U+6AC5.0	fBig5	EFDD
U+6AC5.0	fGB	3-413F
U+6AC5.0	fR/S	 75.14
U+6AC5.0	fTotalStrokes	 18
U+6AC5.0	fCangjie	DYX
U+6AC5.0	fFourCorner	40923
U+6AC5.0	fHakka	 zi5; zi1
U+6AC5.0	fCantonese	zai1; zai3
U+6AC5.0	fMandarin	ji1; qi2; ji4
U+6AC5.0	fEnglish	[1] a fruit tree, called the white date [2] the jujube
U+6AC6.0	fUTF8	櫆
U+6AC6.0	fCNS11643	2-616D
U+6AC6.0	fBig5	EFE1
U+6AC6.0	fGB	3-434B
U+6AC6.0	fR/S	 75.14
U+6AC6.0	fTotalStrokes	 18
U+6AC6.0	fCangjie	DHIJ
U+6AC6.0	fFourCorner	44910
U+6AC6.0	fHakka	 kui2
U+6AC6.0	fCantonese	fui1
U+6AC6.0	fMandarin	kui2
U+6AC7.0	fUTF8	櫇
U+6AC7.0	fCNS11643	2-6171
U+6AC7.0	fBig5	EFE5
U+6AC7.0	fGB	5-4550
U+6AC7.0	fR/S	 75.14
U+6AC7.0	fTotalStrokes	 18
U+6AC7.0	fCangjie	DDEC
U+6AC7.0	fFourCorner	41986
U+6AC7.0	fCantonese	po4
U+6AC7.0	fMandarin	po2
U+6AC7.0	fEnglish	name of a fruit
U+6AC8.0	fUTF8	櫈
U+6AC8.0	fCNS11643	3-577B
U+6AC8.0	fGB	E-2F2F
U+6AC8.0	fR/S	  75.14
U+6AC8.0	fTotalStrokes	 18
U+6AC8.0	fCangjie	DNTN
U+6AC8.0	fFourCorner	42910
U+6AC8.0	fHakka	 den5
U+6AC8.0	fCantonese	dang3
U+6AC8.0	fMandarin	deng4
U+6AC8.0	fEnglish	same as &#x51F3;
U+6AC9.0	fUTF8	櫉
U+6AC9.0	fGB	E-2F30
U+6AC9.0	fR/S	 75.14
U+6AC9.0	fTotalStrokes	 18
U+6AC9.0	fCangjie	DMGI
U+6AC9.0	fCantonese	ceoi4
U+6AC9.0	fMandarin	chu2
U+6ACA.0	fUTF8	櫊
U+6ACA.0	fCNS11643	3-6533
U+6ACA.0	fGB	E-2F31
U+6ACA.0	fR/S	 75.14
U+6ACA.0	fTotalStrokes	 18
U+6ACA.0	fCangjie	DANR
U+6ACA.0	fFourCorner	4792
U+6ACA.0	fCantonese	gok3
U+6ACB.0	fUTF8	櫋
U+6ACB.0	fCNS11643	2-655C
U+6ACB.0	fBig5	F251
U+6ACB.0	fGB	3-4358
U+6ACB.0	fR/S	 75.15
U+6ACB.0	fTotalStrokes	 19
U+6ACB.0	fCangjie	DHUS
U+6ACB.0	fFourCorner	46927
U+6ACB.0	fCantonese	min4
U+6ACB.0	fMandarin	mian2
U+6ACC.0	fUTF8	櫌
U+6ACC.0	fCNS11643	2-6559
U+6ACC.0	fBig5	F24E
U+6ACC.0	fGB	5-4629
U+6ACC.0	fR/S	 75.15
U+6ACC.0	fTotalStrokes	 19
U+6ACC.0	fCangjie	DMBE
U+6ACC.0	fFourCorner	41947
U+6ACC.0	fHakka	 ju2; riu2
U+6ACC.0	fCantonese	jau1; jau4
U+6ACC.0	fMandarin	you1
U+6ACC.0	fEnglish	a kind of hoe
U+6ACD.0	fUTF8	櫍
U+6ACD.0	fCNS11643	2-6562
U+6ACD.0	fBig5	F257
U+6ACD.0	fGB	3-4178
U+6ACD.0	fR/S	 75.15
U+6ACD.0	fTotalStrokes	 19
U+6ACD.0	fCangjie	DHLC
U+6ACD.0	fFourCorner	42986
U+6ACD.0	fCantonese	zat1
U+6ACD.0	fMandarin	zhi4
U+6ACE.0	fUTF8	櫎
U+6ACE.0	fCNS11643	3-5A44
U+6ACE.0	fGB	3-4050
U+6ACE.0	fR/S	  75.15
U+6ACE.0	fTotalStrokes	 19
U+6ACE.0	fCangjie	DITC
U+6ACE.0	fFourCorner	40980
U+6ACE.0	fHakka	 fong3
U+6ACE.0	fCantonese	fong2
U+6ACE.0	fMandarin	guang4; huang4; huang3; guo1
U+6ACE.0	fEnglish	 screen, porch, verandah
U+6ACF.0	fUTF8	櫏
U+6ACF.0	fCNS11643	2-6561
U+6ACF.0	fBig5	F256
U+6ACF.0	fGB	E-2F32
U+6ACF.0	fR/S	 75.15
U+6ACF.0	fTotalStrokes	 19
U+6ACF.0	fCangjie	DYMU
U+6ACF.0	fFourCorner	41931
U+6ACF.0	fCantonese	cin1
U+6ACF.0	fMandarin	qian1
U+6AD0.0	fUTF8	櫐
U+6AD0.0	fCNS11643	2-655F
U+6AD0.0	fBig5	F254
U+6AD0.0	fGB	E-2F33
U+6AD0.0	fR/S	 75.15
U+6AD0.0	fTotalStrokes	 19
U+6AD0.0	fCangjie	WWWD
U+6AD0.0	fFourCorner	60904
U+6AD0.0	fCantonese	leoi5; leoi4
U+6AD0.0	fMandarin	lei3
U+6AD1.0	fUTF8	櫑
U+6AD1.0	fCNS11643	2-655A
U+6AD1.0	fBig5	F24F
U+6AD1.0	fGB	5-462A
U+6AD1.0	fR/S	 75.15
U+6AD1.0	fTotalStrokes	 19
U+6AD1.0	fCangjie	DWWW
U+6AD1.0	fFourCorner	46960
U+6AD1.0	fHakka	 lui2
U+6AD1.0	fCantonese	leoi4; leoi5
U+6AD1.0	fMandarin	lei3; lei2; lei4
U+6AD1.0	fEnglish	an ornamental drinking-cup, made of carved wood
U+6AD2.0	fUTF8	櫒
U+6AD2.0	fCNS11643	3-5A4F
U+6AD2.0	fGB	E-2F34
U+6AD2.0	fR/S	  75.15
U+6AD2.0	fTotalStrokes	 19
U+6AD2.0	fCangjie	DTBF
U+6AD2.0	fFourCorner	44990
U+6AD2.0	fCantonese	saat3
U+6AD2.0	fMandarin	sa4
U+6AD3.0	fUTF8	櫓
U+6AD3.0	fCNS11643	1-783F
U+6AD3.0	fBig5	C372
U+6AD3.0	fGB	1-6956
U+6AD3.0	fR/S	 75.15
U+6AD3.0	fTotalStrokes	 19
U+6AD3.0	fCangjie	DNWA
U+6AD3.0	fFourCorner	47963
U+6AD3.0	fHakka	  lu3; lu1
U+6AD3.0	fCantonese	lou5
U+6AD3.0	fMandarin	lu3
U+6AD3.0	fEnglish	[1] oar; scull; sweep [2] big shield; long spear [3] lookout over on a city wall
U+6AD4.0	fUTF8	櫔
U+6AD4.0	fCNS11643	3-5A45
U+6AD4.0	fGB	5-4353
U+6AD4.0	fR/S	 75.15
U+6AD4.0	fTotalStrokes	 19
U+6AD4.0	fCangjie	DMTB
U+6AD4.0	fFourCorner	41920
U+6AD4.0	fCantonese	lai6
U+6AD4.0	fMandarin	li4
U+6AD4.0	fEnglish	name of a tree
U+6AD5.0	fUTF8	櫕
U+6AD5.0	fCNS11643	3-5A4E
U+6AD5.0	fGB	5-4552
U+6AD5.0	fR/S	  75.15
U+6AD5.0	fTotalStrokes	 19
U+6AD5.0	fCangjie	DQOC
U+6AD5.0	fFourCorner	45980
U+6AD5.0	fHakka	 zon1
U+6AD5.0	fCantonese	cyun4
U+6AD5.0	fMandarin	cuan2
U+6AD6.0	fUTF8	櫖
U+6AD6.0	fCNS11643	4-5F71
U+6AD6.0	fGB	E-2F35
U+6AD6.0	fR/S	 75.15
U+6AD6.0	fTotalStrokes	 19
U+6AD6.0	fCangjie	DYPP
U+6AD6.0	fFourCorner	41930
U+6AD6.0	fCantonese	leoi6
U+6AD6.0	fMandarin	luu2; l2; luu4
U+6AD7.0	fUTF8	櫗
U+6AD7.0	fCNS11643	3-5A47
U+6AD7.0	fGB	3-434E
U+6AD7.0	fR/S	  75.15
U+6AD7.0	fTotalStrokes	 19
U+6AD7.0	fCangjie	DTWI
U+6AD7.0	fFourCorner	44950
U+6AD7.0	fCantonese	mit6
U+6AD7.0	fMandarin	mie4
U+6AD8.0	fUTF8	櫘
U+6AD8.0	fCNS11643	3-5A4D
U+6AD8.0	fGB	5-4627
U+6AD8.0	fR/S	 75.15
U+6AD8.0	fTotalStrokes	 19
U+6AD8.0	fCangjie	DQJP
U+6AD8.0	fFourCorner	45930
U+6AD8.0	fHakka	 sui5; fi5; fui6; wui6; wi5; sui6; fui5; vui5
U+6AD8.0	fCantonese	seoi6; wai6 (also)
U+6AD8.0	fMandarin	hui4; sui4; wei4
U+6AD8.0	fEnglish	same as &#x69E5;
U+6AD9.0	fUTF8	櫙
U+6AD9.0	fCNS11643	2-655B
U+6AD9.0	fBig5	F250
U+6AD9.0	fGB	3-4149
U+6AD9.0	fR/S	 75.15
U+6AD9.0	fTotalStrokes	 19
U+6AD9.0	fCangjie	DTSR
U+6AD9.0	fFourCorner	44916
U+6AD9.0	fCantonese	au1; ngau1 (also)
U+6AD9.0	fMandarin	ou1
U+6ADA.0	fUTF8	櫚
U+6ADA.0	fCNS11643	1-783E
U+6ADA.0	fBig5	C371
U+6ADA.0	fGB	1-6935
U+6ADA.0	fR/S	 75.15
U+6ADA.0	fTotalStrokes	 19
U+6ADA.0	fCangjie	DANR
U+6ADA.0	fFourCorner	47920
U+6ADA.0	fHakka	 li1; li2
U+6ADA.0	fCantonese	leoi4; leoi3 (also)
U+6ADA.0	fMandarin	luu2; l2
U+6ADA.0	fEnglish	name of a tree
U+6ADB.0	fUTF8	櫛
U+6ADB.0	fCNS11643	1-7376
U+6ADB.0	fBig5	C0CD
U+6ADB.0	fGB	1-684E
U+6ADB.0	fR/S	 75.15
U+6ADB.0	fTotalStrokes	 19
U+6ADB.0	fCangjie	DHAL
U+6ADB.0	fFourCorner	48927
U+6ADB.0	fHakka	 ziet7; zet7
U+6ADB.0	fCantonese	zit3; zit1
U+6ADB.0	fMandarin	zhi4; jie2
U+6ADB.0	fEnglish	[1] comb; comb of many fine teeth [2] comb the hair [3] weed out; eliminate; delete
U+6ADC.0	fUTF8	櫜
U+6ADC.0	fCNS11643	2-655E
U+6ADC.0	fBig5	F253
U+6ADC.0	fGB	3-4356
U+6ADC.0	fR/S	 75.15
U+6ADC.0	fTotalStrokes	 19
U+6ADC.0	fCangjie	JBHOD
U+6ADC.0	fFourCorner	50904
U+6ADC.0	fHakka	  gau1
U+6ADC.0	fCantonese	gou1
U+6ADC.0	fMandarin	gao1; gao4
U+6ADC.0	fEnglish	quiver (case for arrows)
U+6ADD.0	fUTF8	櫝
U+6ADD.0	fCNS11643	1-783D
U+6ADD.0	fBig5	C370
U+6ADD.0	fGB	1-687C
U+6ADD.0	fR/S	 75.15
U+6ADD.0	fTotalStrokes	 19
U+6ADD.0	fCangjie	DGWC
U+6ADD.0	fFourCorner	44986
U+6ADD.0	fHakka	 tuk8
U+6ADD.0	fCantonese	duk6
U+6ADD.0	fMandarin	du2
U+6ADD.0	fEnglish	[1] cabinet; wardrobe; closet; sideboard or cupboard [2] coffin [3] scabbard [4] hide; conceal
U+6ADE.0	fUTF8	櫞
U+6ADE.0	fCNS11643	2-6563
U+6ADE.0	fBig5	F258
U+6ADE.0	fGB	1-695A
U+6ADE.0	fR/S	 75.15
U+6ADE.0	fTotalStrokes	 19
U+6ADE.0	fCangjie	DVFO
U+6ADE.0	fFourCorner	47932
U+6ADE.0	fHakka	 jen2; ren2; jan2
U+6ADE.0	fCantonese	jyun4
U+6ADE.0	fMandarin	yuan2
U+6ADE.0	fEnglish	citrus
U+6ADF.0	fUTF8	櫟
U+6ADF.0	fCNS11643	2-655D
U+6ADF.0	fBig5	F252
U+6ADF.0	fGB	1-685D
U+6ADF.0	fR/S	 75.15
U+6ADF.0	fTotalStrokes	 19
U+6ADF.0	fCangjie	DVID
U+6ADF.0	fFourCorner	42994
U+6ADF.0	fHakka	 lit8; sok7; jok8; lep7
U+6ADF.0	fCantonese	lik1; joek6; lik6; ngok6
U+6ADF.0	fMandarin	li4; lao2; yue4; luo4; yao4; shuo4
U+6ADF.0	fEnglish	the chestnut-leafed oak (Quercus chinensis and serrata)
U+6AE0.0	fUTF8	櫠
U+6AE0.0	fCNS11643	2-6558
U+6AE0.0	fBig5	F24D
U+6AE0.0	fGB	3-4221
U+6AE0.0	fR/S	 75.15
U+6AE0.0	fTotalStrokes	 19
U+6AE0.0	fCangjie	DINE
U+6AE0.0	fFourCorner	40947
U+6AE0.0	fCantonese	fai3
U+6AE0.0	fMandarin	fei4
U+6AE0.0	fEnglish	name of a fruit
U+6AE1.0	fUTF8	櫡
U+6AE1.0	fCNS11643	2-616C
U+6AE1.0	fBig5	EFE0
U+6AE1.0	fGB	3-4352
U+6AE1.0	fR/S	 75.15
U+6AE1.0	fTotalStrokes	 19
U+6AE1.0	fCangjie	DHJA
U+6AE1.0	fFourCorner	48964
U+6AE1.0	fCantonese	zoek3; zyu6
U+6AE1.0	fMandarin	zhuo2; zhu4
U+6AE2.0	fUTF8	櫢
U+6AE2.0	fCNS11643	3-5A48
U+6AE2.0	fGB	5-457A
U+6AE2.0	fR/S	  75.15
U+6AE2.0	fTotalStrokes	 19
U+6AE2.0	fCangjie	DLVK
U+6AE2.0	fFourCorner	48940
U+6AE2.0	fCantonese	sau2
U+6AE2.0	fMandarin	sou3
U+6AE3.0	fUTF8	櫣
U+6AE3.0	fCNS11643	3-5A46
U+6AE3.0	fGB	3-425A
U+6AE3.0	fR/S	  75.15
U+6AE3.0	fTotalStrokes	 19
U+6AE3.0	fCangjie	XXDYT
U+6AE3.0	fFourCorner	44930
U+6AE3.0	fCantonese	lin4
U+6AE3.0	fMandarin	lian2
U+6AE4.0	fUTF8	櫤
U+6AE4.0	fGB	E-2F36
U+6AE4.0	fR/S	 75.15
U+6AE4.0	fTotalStrokes	 19
U+6AE4.0	fCangjie	DHTN
U+6AE4.0	fCantonese	zin3
U+6AE5.0	fUTF8	櫥
U+6AE5.0	fCNS11643	1-783C
U+6AE5.0	fBig5	C36F
U+6AE5.0	fGB	E-2F37
U+6AE5.0	fR/S	 75.15
U+6AE5.0	fTotalStrokes	 19
U+6AE5.0	fCangjie	DIGI
U+6AE5.0	fFourCorner	40940
U+6AE5.0	fHakka	 cu2
U+6AE5.0	fCantonese	cyu4 (also); ceoi4; cyu1
U+6AE5.0	fMandarin	chu2
U+6AE5.0	fEnglish	 cabinet; wardrobe; cupboard; closet; sideboard
U+6AE6.0	fUTF8	櫦
U+6AE6.0	fCNS11643	3-6534
U+6AE6.0	fGB	E-2F38
U+6AE6.0	fR/S	 75.15
U+6AE6.0	fTotalStrokes	 19
U+6AE6.0	fCangjie	DIXE
U+6AE6.0	fFourCorner	4094
U+6AE6.0	fCantonese	hing3
U+6AE7.0	fUTF8	櫧
U+6AE7.0	fCNS11643	2-6557
U+6AE7.0	fBig5	F24C
U+6AE7.0	fGB	1-6946
U+6AE7.0	fR/S	 75.16
U+6AE7.0	fTotalStrokes	 20
U+6AE7.0	fCangjie	DYRA
U+6AE7.0	fFourCorner	44960
U+6AE7.0	fHakka	 zhu1; zu1
U+6AE7.0	fCantonese	zyu1
U+6AE7.0	fMandarin	zhu1
U+6AE7.0	fEnglish	Oak
U+6AE8.0	fUTF8	櫨
U+6AE8.0	fCNS11643	2-6924
U+6AE8.0	fBig5	F456
U+6AE8.0	fGB	1-6853
U+6AE8.0	fR/S	 75.16
U+6AE8.0	fTotalStrokes	 20
U+6AE8.0	fCangjie	DYPT
U+6AE8.0	fFourCorner	41917
U+6AE8.0	fHakka	  lu2
U+6AE8.0	fCantonese	lou4; leoi4
U+6AE8.0	fMandarin	lu2
U+6AE8.0	fEnglish	[1] the square peck-shaped box halfway up a staff or mast [2] name of a plant
U+6AE9.0	fUTF8	櫩
U+6AE9.0	fCNS11643	3-5C49
U+6AE9.0	fGB	3-4329
U+6AE9.0	fR/S	  75.16
U+6AE9.0	fTotalStrokes	 20
U+6AE9.0	fCangjie	DANX
U+6AE9.0	fFourCorner	47920
U+6AE9.0	fCantonese	jim4
U+6AE9.0	fMandarin	yan2; yan3
U+6AEA.0	fUTF8	櫪
U+6AEA.0	fCNS11643	2-6923
U+6AEA.0	fBig5	F455
U+6AEA.0	fGB	1-6840
U+6AEA.0	fR/S	 75.16
U+6AEA.0	fTotalStrokes	 20
U+6AEA.0	fCangjie	DMHM
U+6AEA.0	fFourCorner	41911
U+6AEA.0	fHakka	 lit8;  let8
U+6AEA.0	fCantonese	lik1; lik6
U+6AEA.0	fMandarin	li4
U+6AEA.0	fEnglish	[1] Quercus serrata [2] manger; stable [3] a wooden device used to torture a criminal by pressing his fingers
U+6AEB.0	fUTF8	櫫
U+6AEB.0	fCNS11643	2-6560
U+6AEB.0	fBig5	F255
U+6AEB.0	fGB	E-2F39
U+6AEB.0	fR/S	 75.15; 75.16
U+6AEB.0	fTotalStrokes	 19
U+6AEB.0	fCangjie	MAD
U+6AEB.0	fFourCorner	14904
U+6AEB.0	fHakka	 zu1; zhu1
U+6AEB.0	fCantonese	zyu1
U+6AEB.0	fMandarin	zhu1
U+6AEB.0	fEnglish	 wooden peg; post; stalk; stick
U+6AEC.0	fUTF8	櫬
U+6AEC.0	fCNS11643	1-7973
U+6AEC.0	fBig5	C468
U+6AEC.0	fGB	1-6934
U+6AEC.0	fR/S	 75.16
U+6AEC.0	fTotalStrokes	 20
U+6AEC.0	fCangjie	DYDU
U+6AEC.0	fFourCorner	46910
U+6AEC.0	fHakka	  cang5; chin5; cihn5
U+6AEC.0	fCantonese	can3
U+6AEC.0	fMandarin	chen4; guan4; qin4; qin1
U+6AEC.0	fEnglish	[1] coffin [2] a tung tree
U+6AED.0	fUTF8	櫭
U+6AED.0	fCNS11643	3-5775
U+6AED.0	fGB	3-4354
U+6AED.0	fR/S	 75.15; 75.14
U+6AED.0	fTotalStrokes	 18; 19
U+6AED.0	fCangjie	DAUM
U+6AED.0	fFourCorner	41910
U+6AED.0	fCantonese	git6
U+6AED.0	fMandarin	jie2; ji4; jiao4
U+6AEE.0	fUTF8	櫮
U+6AEE.0	fCNS11643	2-6927
U+6AEE.0	fBig5	F459
U+6AEE.0	fGB	5-462D
U+6AEE.0	fR/S	 75.16
U+6AEE.0	fTotalStrokes	 20
U+6AEE.0	fCangjie	DMGR
U+6AEE.0	fFourCorner	41916
U+6AEE.0	fCantonese	ngok6; ok6 (also)
U+6AEE.0	fMandarin	e4
U+6AEF.0	fUTF8	櫯
U+6AEF.0	fCNS11643	2-6928
U+6AEF.0	fBig5	F45A
U+6AEF.0	fGB	E-2F3A
U+6AEF.0	fR/S	 75.16
U+6AEF.0	fTotalStrokes	 20
U+6AEF.0	fCangjie	DNFD
U+6AEF.0	fFourCorner	42994
U+6AEF.0	fHakka	 su1; se1; sii1
U+6AEF.0	fCantonese	sou1; si1
U+6AEF.0	fMandarin	su1
U+6AEF.0	fEnglish	name of a tree
U+6AF0.0	fUTF8	櫰
U+6AF0.0	fCNS11643	2-6922
U+6AF0.0	fBig5	F454
U+6AF0.0	fGB	3-435C
U+6AF0.0	fR/S	 75.16
U+6AF0.0	fTotalStrokes	 20
U+6AF0.0	fCangjie	DYWV
U+6AF0.0	fFourCorner	40932
U+6AF0.0	fHakka	 fai2
U+6AF0.0	fCantonese	waai4
U+6AF0.0	fMandarin	huai2; hui4; gui1
U+6AF1.0	fUTF8	櫱
U+6AF1.0	fCNS11643	2-6926
U+6AF1.0	fBig5	F458
U+6AF1.0	fGB	E-2F3B
U+6AF1.0	fR/S	 75.16
U+6AF1.0	fTotalStrokes	 20
U+6AF1.0	fCangjie	UJD
U+6AF1.0	fFourCorner	44904
U+6AF1.0	fHakka	  ngiet7
U+6AF1.0	fCantonese	jit6; jip6 (also); nip6 (also); lip6 (also); baak3; paak3
U+6AF1.0	fMandarin	nie4; e4
U+6AF1.0	fEnglish	sprouts grown from the stump of a tree
U+6AF2.0	fUTF8	櫲
U+6AF2.0	fCNS11643	4-6350
U+6AF2.0	fGB	5-462C
U+6AF2.0	fR/S	  75.16
U+6AF2.0	fTotalStrokes	 20
U+6AF2.0	fCangjie	DNNO
U+6AF2.0	fFourCorner	47930
U+6AF2.0	fCantonese	jyu6
U+6AF2.0	fMandarin	yu4
U+6AF3.0	fUTF8	櫳
U+6AF3.0	fCNS11643	2-6921
U+6AF3.0	fBig5	F453
U+6AF3.0	fGB	1-6850
U+6AF3.0	fR/S	 75.16
U+6AF3.0	fTotalStrokes	 20
U+6AF3.0	fCangjie	DYBP
U+6AF3.0	fFourCorner	41911
U+6AF3.0	fHakka	 lung2; lung3; lun3
U+6AF3.0	fCantonese	lung4
U+6AF3.0	fMandarin	long2
U+6AF3.0	fEnglish	[1] cage; pen for animals [2] windows with bars
U+6AF4.0	fUTF8	櫴
U+6AF4.0	fCNS11643	3-5C47
U+6AF4.0	fGB	5-4571
U+6AF4.0	fR/S	  75.16
U+6AF4.0	fTotalStrokes	 20
U+6AF4.0	fCangjie	DDLC
U+6AF4.0	fFourCorner	47980
U+6AF4.0	fCantonese	laai6
U+6AF4.0	fMandarin	lai4
U+6AF5.0	fUTF8	櫵
U+6AF5.0	fCNS11643	3-6535
U+6AF5.0	fGB	E-2F3C
U+6AF5.0	fR/S	 75.16
U+6AF5.0	fTotalStrokes	 20
U+6AF5.0	fCangjie	DTOF
U+6AF5.0	fFourCorner	4493
U+6AF5.0	fCantonese	ciu4
U+6AF6.0	fUTF8	櫶
U+6AF6.0	fCNS11643	4-634E
U+6AF6.0	fGB	5-4632
U+6AF6.0	fR/S	 75.16
U+6AF6.0	fTotalStrokes	 20
U+6AF6.0	fCangjie	DJQP
U+6AF6.0	fFourCorner	43930
U+6AF6.0	fCantonese	hin2
U+6AF6.0	fMandarin	xian3
U+6AF7.0	fUTF8	櫷
U+6AF7.0	fGB	1-7D2B
U+6AF7.0	fR/S	 75.18
U+6AF7.0	fTotalStrokes	 22
U+6AF7.0	fCangjie	DNXU
U+6AF7.0	fCantonese	gwai1
U+6AF8.0	fUTF8	櫸
U+6AF8.0	fCNS11643	2-6B5F
U+6AF8.0	fBig5	F5D1
U+6AF8.0	fGB	1-6937
U+6AF8.0	fR/S	 75.17
U+6AF8.0	fTotalStrokes	 21
U+6AF8.0	fCangjie	DHCQ
U+6AF8.0	fFourCorner	47952
U+6AF8.0	fHakka	  gi3
U+6AF8.0	fCantonese	geoi2
U+6AF8.0	fMandarin	ju3
U+6AF8.0	fEnglish	type of elm
U+6AF9.0	fUTF8	櫹
U+6AF9.0	fCNS11643	2-6925
U+6AF9.0	fBig5	F457
U+6AF9.0	fGB	3-4279
U+6AF9.0	fR/S	 75.17
U+6AF9.0	fTotalStrokes	 21
U+6AF9.0	fCangjie	DTLX
U+6AF9.0	fFourCorner	44927
U+6AF9.0	fCantonese	siu1 (also); suk1
U+6AF9.0	fMandarin	xiao1
U+6AFA.0	fUTF8	櫺
U+6AFA.0	fCNS11643	1-7A72
U+6AFA.0	fBig5	C4E7
U+6AFA.0	fGB	5-4633
U+6AFA.0	fR/S	 75.17
U+6AFA.0	fTotalStrokes	 21
U+6AFA.0	fCangjie	DMBR
U+6AFA.0	fFourCorner	41963
U+6AFA.0	fHakka	  lin2
U+6AFA.0	fCantonese	ling4
U+6AFA.0	fMandarin	ling2
U+6AFA.0	fEnglish	[1] carved or patterned window railings, sills [2] the wooden planks which join eaves with a house
U+6AFB.0	fUTF8	櫻
U+6AFB.0	fCNS11643	1-7A70
U+6AFB.0	fBig5	C4E5
U+6AFB.0	fGB	1-5323
U+6AFB.0	fR/S	 75.17
U+6AFB.0	fTotalStrokes	 21
U+6AFB.0	fCangjie	DBCV
U+6AFB.0	fFourCorner	46944
U+6AFB.0	fHakka	 jin1; rin1
U+6AFB.0	fCantonese	jing1
U+6AFB.0	fMandarin	ying1
U+6AFB.0	fEnglish	the cherry; cherry blossom
U+6AFC.0	fUTF8	櫼
U+6AFC.0	fCNS11643	2-6B5D
U+6AFC.0	fBig5	F5CF
U+6AFC.0	fGB	3-435D
U+6AFC.0	fR/S	 75.17
U+6AFC.0	fTotalStrokes	 21
U+6AFC.0	fCangjie	DOIM
U+6AFC.0	fFourCorner	53950
U+6AFC.0	fHakka	 siam1
U+6AFC.0	fCantonese	cim1; zim1 (also)
U+6AFC.0	fMandarin	jian1; jin1; shan1
U+6AFD.0	fUTF8	櫽
U+6AFD.0	fCNS11643	3-5E35
U+6AFD.0	fGB	3-432B
U+6AFD.0	fR/S	  75.17
U+6AFD.0	fTotalStrokes	 21
U+6AFD.0	fCangjie	NPD
U+6AFD.0	fFourCorner	72900
U+6AFD.0	fHakka	 jun3; run3
U+6AFD.0	fCantonese	jan2
U+6AFD.0	fMandarin	yin3
U+6AFD.0	fEnglish	[1] straighten bent wood by use of heat [2] correct; adjust
U+6AFE.0	fUTF8	櫾
U+6AFE.0	fCNS11643	3-5E34
U+6AFE.0	fGB	5-4634
U+6AFE.0	fR/S	  75.17
U+6AFE.0	fTotalStrokes	 21
U+6AFE.0	fCangjie	DBUF
U+6AFE.0	fFourCorner	42990
U+6AFE.0	fCantonese	jau2; jau6; jau4 (also)
U+6AFE.0	fMandarin	you2; you4
U+6AFE.0	fEnglish	the pumelo
U+6AFF.0	fUTF8	櫿
U+6AFF.0	fCNS11643	3-5E33
U+6AFF.0	fGB	E-2F3D
U+6AFF.0	fR/S	  75.17
U+6AFF.0	fTotalStrokes	 21
U+6AFF.0	fCangjie	DFFR
U+6AFF.0	fFourCorner	49960
U+6AFF.0	fCantonese	jing4
U+6AFF.0	fMandarin	ying2
U+6AFF.0	fEnglish	name of a tree
U+6B00.0	fUTF8	欀
U+6B00.0	fCNS11643	2-6B60
U+6B00.0	fBig5	F5D2
U+6B00.0	fGB	3-435F
U+6B00.0	fR/S	 75.17
U+6B00.0	fTotalStrokes	 21
U+6B00.0	fCangjie	DYRV
U+6B00.0	fFourCorner	40932
U+6B00.0	fHakka	  siong1
U+6B00.0	fCantonese	soeng1
U+6B00.0	fMandarin	xiang1; xiang3; rang4
U+6B00.0	fEnglish	a tree which furnishes a white grain like sago. It is found in Annam
U+6B01.0	fUTF8	欁
U+6B01.0	fCNS11643	3-5E37
U+6B01.0	fGB	5-4534
U+6B01.0	fR/S	  75.17
U+6B01.0	fTotalStrokes	 21
U+6B01.0	fCangjie	DTVD
U+6B01.0	fFourCorner	44990
U+6B01.0	fCantonese	nung4; lung4 (also)
U+6B01.0	fMandarin	nong2
U+6B02.0	fUTF8	欂
U+6B02.0	fCNS11643	2-6B5C
U+6B02.0	fBig5	F5CE
U+6B02.0	fGB	3-435B
U+6B02.0	fR/S	 75.17; 75.16
U+6B02.0	fTotalStrokes	 21
U+6B02.0	fCangjie	DTEI
U+6B02.0	fFourCorner	44942
U+6B02.0	fCantonese	bok3; bok6 (also)
U+6B02.0	fMandarin	bo2; bi4
U+6B02.0	fEnglish	a square wooden block at the top of a column
U+6B03.0	fUTF8	欃
U+6B03.0	fCNS11643	2-6B5E
U+6B03.0	fBig5	F5D0
U+6B03.0	fGB	3-435E
U+6B03.0	fR/S	 75.17
U+6B03.0	fTotalStrokes	 21
U+6B03.0	fCangjie	DNRI
U+6B03.0	fFourCorner	47913
U+6B03.0	fHakka	  cam2
U+6B03.0	fCantonese	caam4
U+6B03.0	fMandarin	chan2
U+6B03.0	fEnglish	[1] sandalwood [2] comet
U+6B04.0	fUTF8	欄
U+6B04.0	fCNS11643	1-7A71
U+6B04.0	fBig5	C4E6
U+6B04.0	fGB	1-4038
U+6B04.0	fR/S	 75.17
U+6B04.0	fTotalStrokes	 21
U+6B04.0	fCangjie	DANW
U+6B04.0	fFourCorner	47920
U+6B04.0	fHakka	  lan2; lan3
U+6B04.0	fCantonese	laan4; laan1 (also)
U+6B04.0	fMandarin	lan2; lian4
U+6B04.0	fEnglish	[1] railing; balustrade [2] pan for domesticated animals
U+6B05.0	fUTF8	欅
U+6B05.0	fGB	E-2F3E
U+6B05.0	fR/S	 75.10
U+6B05.0	fTotalStrokes	 14
U+6B05.0	fCangjie	DHCQ
U+6B05.0	fCantonese	geoi2
U+6B05.0	fMandarin	ju3
U+6B05.0	fEnglish	a species of elm with fine-grained wood (Zelkova serrata)
U+6B06.0	fUTF8	欆
U+6B06.0	fCNS11643	3-5F4F
U+6B06.0	fGB	5-4350
U+6B06.0	fR/S	  75.18
U+6B06.0	fTotalStrokes	 22
U+6B06.0	fCangjie	DOGE
U+6B06.0	fFourCorner	40940
U+6B06.0	fCantonese	soeng1
U+6B06.0	fMandarin	shuang1
U+6B07.0	fUTF8	欇
U+6B07.0	fCNS11643	3-5F4E
U+6B07.0	fGB	3-4257
U+6B07.0	fR/S	  75.18
U+6B07.0	fTotalStrokes	 22
U+6B07.0	fCangjie	DSJJ
U+6B07.0	fFourCorner	41940
U+6B07.0	fHakka	 ngiap7
U+6B07.0	fCantonese	sip3
U+6B07.0	fMandarin	she4
U+6B07.0	fEnglish	[1] branches waving in the wind [2] a creeping plant
U+6B08.0	fUTF8	欈
U+6B08.0	fCNS11643	2-6D53
U+6B08.0	fBig5	F6E5
U+6B08.0	fGB	5-4636
U+6B08.0	fR/S	 75.18
U+6B08.0	fTotalStrokes	 22
U+6B08.0	fCangjie	DUOB
U+6B08.0	fFourCorner	42927
U+6B08.0	fCantonese	wai4; zeoi3
U+6B08.0	fMandarin	wei2; zui1; zui4
U+6B09.0	fUTF8	欉
U+6B09.0	fCNS11643	2-6D54
U+6B09.0	fBig5	F6E6
U+6B09.0	fGB	E-2F3F
U+6B09.0	fR/S	 75.18
U+6B09.0	fTotalStrokes	 22
U+6B09.0	fCangjie	DTCE
U+6B09.0	fFourCorner	43947
U+6B09.0	fCantonese	cung4; cung3
U+6B09.0	fMandarin	cong4
U+6B0A.0	fUTF8	權
U+6B0A.0	fCNS11643	1-7B62
U+6B0A.0	fBig5	C576
U+6B0A.0	fGB	1-4828
U+6B0A.0	fR/S	 75.18
U+6B0A.0	fTotalStrokes	 22
U+6B0A.0	fCangjie	DTRG
U+6B0A.0	fFourCorner	44914
U+6B0A.0	fHakka	  kien2;  ken2; k'ian3; kian2
U+6B0A.0	fCantonese	kyun4
U+6B0A.0	fMandarin	quan2; guan4
U+6B0A.0	fEnglish	[1] power; right; authority; inherent rights; jurisdiction; influence [2] weigh (the significance, etc.); assess [3] expedient way; alternative; expediency [4] temporarily; for the time being [5] a Chinese family name
U+6B0B.0	fUTF8	欋
U+6B0B.0	fCNS11643	2-6D52
U+6B0B.0	fBig5	F6E4
U+6B0B.0	fGB	3-4360
U+6B0B.0	fR/S	 75.18
U+6B0B.0	fTotalStrokes	 22
U+6B0B.0	fCangjie	DBUG
U+6B0B.0	fFourCorner	46914
U+6B0B.0	fHakka	  ki1
U+6B0B.0	fCantonese	keoi4; geoi6
U+6B0B.0	fMandarin	qu2; chu1
U+6B0B.0	fEnglish	[1] rake with four teeth [2] twisted roots of trees
U+6B0C.0	fUTF8	欌
U+6B0C.0	fCNS11643	F-694C
U+6B0C.0	fGB	1-7D56
U+6B0C.0	fR/S	 75.17
U+6B0C.0	fTotalStrokes	 21
U+6B0C.0	fCangjie	DTIS
U+6B0C.0	fCantonese	cong4
U+6B0D.0	fUTF8	欍
U+6B0D.0	fCNS11643	3-6536
U+6B0D.0	fGB	E-2F40
U+6B0D.0	fR/S	 75.18
U+6B0D.0	fTotalStrokes	 22
U+6B0D.0	fCangjie	DTOX
U+6B0D.0	fFourCorner	4497
U+6B0D.0	fCantonese	kau5
U+6B0E.0	fUTF8	欎
U+6B0E.0	fCNS11643	3-6537
U+6B0E.0	fGB	E-2F41
U+6B0E.0	fR/S	 75.18
U+6B0E.0	fTotalStrokes	 22
U+6B0E.0	fCangjie	DDBAI
U+6B0E.0	fFourCorner	4474
U+6B0E.0	fCantonese	wat1
U+6B0E.0	fMandarin	yu4
U+6B0F.0	fUTF8	欏
U+6B0F.0	fCNS11643	2-6F31
U+6B0F.0	fBig5	F7E2
U+6B0F.0	fGB	1-6921
U+6B0F.0	fR/S	 75.19
U+6B0F.0	fTotalStrokes	 23
U+6B0F.0	fCangjie	DWLG
U+6B0F.0	fFourCorner	46914
U+6B0F.0	fHakka	  lo2
U+6B0F.0	fCantonese	lo4
U+6B0F.0	fMandarin	luo2; luo3; luo4
U+6B0F.0	fEnglish	tree the horse-chestnut
U+6B10.0	fUTF8	欐
U+6B10.0	fCNS11643	1-7C3B
U+6B10.0	fBig5	C5CF
U+6B10.0	fGB	3-414E
U+6B10.0	fR/S	 75.19
U+6B10.0	fTotalStrokes	 23
U+6B10.0	fCangjie	DMMP
U+6B10.0	fFourCorner	41911
U+6B10.0	fCantonese	lai6; lai1
U+6B10.0	fMandarin	li3; li4; li2; si1
U+6B10.0	fEnglish	beam
U+6B11.0	fUTF8	欑
U+6B11.0	fCNS11643	2-6F2F
U+6B11.0	fBig5	F7E0
U+6B11.0	fGB	5-4630
U+6B11.0	fR/S	 75.19
U+6B11.0	fTotalStrokes	 23
U+6B11.0	fCangjie	DHUC
U+6B11.0	fFourCorner	44986
U+6B11.0	fHakka	  zon1; zan3
U+6B11.0	fCantonese	cyun4; zaan2
U+6B11.0	fMandarin	zan4; zuan4; cuan2
U+6B11.0	fEnglish	gather
U+6B12.0	fUTF8	欒
U+6B12.0	fCNS11643	2-6F30
U+6B12.0	fBig5	F7E1
U+6B12.0	fGB	1-686F
U+6B12.0	fR/S	 75.19
U+6B12.0	fTotalStrokes	 23
U+6B12.0	fCangjie	VFD
U+6B12.0	fFourCorner	22904
U+6B12.0	fHakka	 lan2
U+6B12.0	fCantonese	lyun4
U+6B12.0	fMandarin	luan2
U+6B12.0	fEnglish	[1] name of tree (Koelreuteria paniculata) [2] the two corners at the moth of the Chinese bell [3] a Chinese family name)
U+6B13.0	fUTF8	欓
U+6B13.0	fCNS11643	2-703A
U+6B13.0	fBig5	F8AC
U+6B13.0	fGB	3-4260
U+6B13.0	fR/S	 75.20
U+6B13.0	fTotalStrokes	 24
U+6B13.0	fCangjie	DFBF
U+6B13.0	fFourCorner	49931
U+6B13.0	fHakka	 dong3; dong5
U+6B13.0	fCantonese	dong2; dong1
U+6B13.0	fMandarin	dang3; tang3
U+6B13.0	fEnglish	 a cross-piece (as the round of a ladder); the rail of a chair shelves pigeonholes
U+6B14.0	fUTF8	欔
U+6B14.0	fCNS11643	3-6074
U+6B14.0	fGB	3-4362
U+6B14.0	fR/S	  75.20
U+6B14.0	fTotalStrokes	 24; 22
U+6B14.0	fCangjie	DBUE
U+6B14.0	fFourCorner	46940
U+6B14.0	fCantonese	fok3
U+6B14.0	fMandarin	jue2
U+6B15.0	fUTF8	欕
U+6B15.0	fCNS11643	F-6B51
U+6B15.0	fGB	1-7D75
U+6B15.0	fR/S	 75.20
U+6B15.0	fTotalStrokes	 24
U+6B15.0	fCangjie	DRRK
U+6B15.0	fFourCorner	4694
U+6B15.0	fCantonese	jim4
U+6B15.0	fEnglish	 tree name
U+6B16.0	fUTF8	欖
U+6B16.0	fCNS11643	1-7D23
U+6B16.0	fBig5	C656
U+6B16.0	fGB	1-692D
U+6B16.0	fR/S	 75.21
U+6B16.0	fTotalStrokes	 25
U+6B16.0	fCangjie	DSWU
U+6B16.0	fFourCorner	48916
U+6B16.0	fHakka	  lam3
U+6B16.0	fCantonese	laam5; laam2 (also)
U+6B16.0	fMandarin	lan3
U+6B16.0	fEnglish	the olive
U+6B17.0	fUTF8	欗
U+6B17.0	fCNS11643	2-7123
U+6B17.0	fBig5	F8F3
U+6B17.0	fGB	E-2F42
U+6B17.0	fR/S	 75.21; 75.20
U+6B17.0	fTotalStrokes	 25
U+6B17.0	fCangjie	DTAW
U+6B17.0	fFourCorner	44927
U+6B17.0	fHakka	 lan2
U+6B17.0	fCantonese	laan4
U+6B17.0	fMandarin	lan2
U+6B17.0	fEnglish	name of a tree
U+6B18.0	fUTF8	欘
U+6B18.0	fCNS11643	2-7121
U+6B18.0	fBig5	F8F1
U+6B18.0	fGB	3-4341
U+6B18.0	fR/S	 75.21
U+6B18.0	fTotalStrokes	 25
U+6B18.0	fCangjie	DSEI
U+6B18.0	fFourCorner	47927
U+6B18.0	fCantonese	zuk1
U+6B18.0	fMandarin	zhu3; zhuo2; zhu2
U+6B19.0	fUTF8	欙
U+6B19.0	fCNS11643	2-7122
U+6B19.0	fBig5	F8F2
U+6B19.0	fGB	E-2F43
U+6B19.0	fR/S	 75.21
U+6B19.0	fTotalStrokes	 25
U+6B19.0	fCangjie	DWWF
U+6B19.0	fFourCorner	46993
U+6B19.0	fCantonese	leoi4; leoi5
U+6B19.0	fMandarin	lei2; luo3
U+6B19.0	fEnglish	sedan
U+6B1A.0	fUTF8	欚
U+6B1A.0	fCNS11643	2-7124
U+6B1A.0	fBig5	F8F4
U+6B1A.0	fGB	3-4363
U+6B1A.0	fR/S	 75.21
U+6B1A.0	fTotalStrokes	 25
U+6B1A.0	fCangjie	DVOI
U+6B1A.0	fFourCorner	47936
U+6B1A.0	fHakka	 li1
U+6B1A.0	fCantonese	lai5
U+6B1A.0	fMandarin	li3
U+6B1B.0	fUTF8	欛
U+6B1B.0	fCNS11643	3-6140
U+6B1B.0	fGB	5-4638
U+6B1B.0	fR/S	  75.21
U+6B1B.0	fTotalStrokes	 25
U+6B1B.0	fCangjie	DMBB
U+6B1B.0	fFourCorner	41927
U+6B1B.0	fHakka	  ba5
U+6B1B.0	fCantonese	baa3
U+6B1B.0	fMandarin	ba4
U+6B1B.0	fEnglish	a handle
U+6B1C.0	fUTF8	欜
U+6B1C.0	fCNS11643	3-6165
U+6B1C.0	fGB	3-4364
U+6B1C.0	fR/S	  75.22
U+6B1C.0	fTotalStrokes	 26
U+6B1C.0	fCangjie	DJBV
U+6B1C.0	fFourCorner	45930
U+6B1C.0	fCantonese	nong4; long4 (also)
U+6B1C.0	fMandarin	nang2
U+6B1D.0	fUTF8	欝
U+6B1D.0	fCNS11643	3-6141
U+6B1D.0	fGB	E-2F44
U+6B1D.0	fR/S	 75.22; 75.21
U+6B1D.0	fTotalStrokes	 25; 26
U+6B1D.0	fCangjie	DDWLI
U+6B1D.0	fFourCorner	44740
U+6B1D.0	fHakka	 jut7; ngut7; rut7
U+6B1D.0	fCantonese	wat1
U+6B1D.0	fMandarin	yu4
U+6B1D.0	fEnglish	luxuriant; dense, thick; moody
U+6B1E.0	fUTF8	欞
U+6B1E.0	fCNS11643	2-722B
U+6B1E.0	fBig5	F9BB
U+6B1E.0	fGB	1-6879
U+6B1E.0	fR/S	 75.24
U+6B1E.0	fTotalStrokes	 28
U+6B1E.0	fCangjie	DMBO
U+6B1E.0	fFourCorner	41914
U+6B1E.0	fHakka	  lin2
U+6B1E.0	fCantonese	ling4
U+6B1E.0	fMandarin	ling2
U+6B1E.0	fEnglish	the lattice of a window a sill; a lintel
U+6B1F.0	fUTF8	欟
U+6B1F.0	fGB	E-2F45
U+6B1F.0	fR/S	 75.24
U+6B1F.0	fTotalStrokes	 28
U+6B1F.0	fCangjie	DTGU
U+6B1F.0	fCantonese	gun1; gwun1 (also)
U+6B20.0	fUTF8	欠
U+6B20.0	fCNS11643	1-454E
U+6B20.0	fBig5	A4ED
U+6B20.0	fGB	0-4737
U+6B20.0	fR/S	 76.0
U+6B20.0	fTotalStrokes	 4
U+6B20.0	fCangjie	NO
U+6B20.0	fFourCorner	27802
U+6B20.0	fHakka	  kiam5
U+6B20.0	fCantonese	him3
U+6B20.0	fMandarin	qian4; que1
U+6B20.0	fEnglish	[1] owe; owe money [2] lack; be deficient; short of; deficient; lacking; not enough [3] raise slightly (a part of the body) [4] yawn [5] Kangxi radical 76
U+6B20.1	fUTF8	欠
U+6B20.1	fCNS11643	1-454E
U+6B20.1	fBig5	A4ED
U+6B20.1	fGB	0-4737
U+6B20.1	fR/S	 76.0
U+6B20.1	fTotalStrokes	 4
U+6B20.1	fCangjie	NO
U+6B20.1	fFourCorner	27802
U+6B20.1	fMandarin	qian5
U+6B21.0	fUTF8	次
U+6B21.0	fCNS11643	1-4839
U+6B21.0	fBig5	A6B8
U+6B21.0	fGB	0-344E
U+6B21.0	fR/S	 76.2
U+6B21.0	fTotalStrokes	 6
U+6B21.0	fCangjie	IMNO
U+6B21.0	fFourCorner	37182
U+6B21.0	fHakka	 cu5;  cih5; cih6; ze5
U+6B21.0	fCantonese	ci3; zi1; ci1
U+6B21.0	fMandarin	ci4; zi1zi4; ci2
U+6B21.0	fEnglish	[1] the next in order; secondary [2] inferior; lower [3] vice; deputy (ministers, etc.) [4] a place where one stops for rest on a trip; place [5] stop at a place [6] by; at (the feast, table, etc.); in the midst [7] grade; grading; order; sequence [8] hypo (chemistry) [9] time (each occasion of a recurring action or event)
U+6B22.0	fUTF8	欢
U+6B22.0	fCNS11643	3-2379
U+6B22.0	fGB	0-3B36
U+6B22.0	fR/S	 76.2
U+6B22.0	fTotalStrokes	 6
U+6B22.0	fCangjie	ENO
U+6B22.0	fFourCorner	77480
U+6B22.0	fHakka	 fon1; fan1
U+6B22.0	fCantonese	fun1
U+6B22.0	fMandarin	huan1
U+6B22.0	fEnglish	simplified form of &#x6B61;
U+6B22.1	fUTF8	欢
U+6B22.1	fCNS11643	3-2379
U+6B22.1	fGB	0-3B36
U+6B22.1	fR/S	 76.2
U+6B22.1	fTotalStrokes	 6
U+6B22.1	fCangjie	ENO
U+6B22.1	fFourCorner	77480
U+6B22.1	fMandarin	huan5
U+6B23.0	fUTF8	欣
U+6B23.0	fCNS11643	1-4E3C
U+6B23.0	fBig5	AA59
U+6B23.0	fGB	0-5040
U+6B23.0	fR/S	 76.4
U+6B23.0	fTotalStrokes	 8
U+6B23.0	fCangjie	HLNO
U+6B23.0	fFourCorner	77282
U+6B23.0	fHakka	  hiun1; hin1; him1
U+6B23.0	fCantonese	jan1
U+6B23.0	fMandarin	xin1
U+6B23.0	fEnglish	happy; joyous; delighted; glad; gladly; joyful; joyfully
U+6B24.0	fUTF8	欤
U+6B24.0	fCNS11643	4-2671
U+6B24.0	fGB	0-6C23
U+6B24.0	fR/S	 76.3
U+6B24.0	fTotalStrokes	 8; 7
U+6B24.0	fCangjie	YMNO
U+6B24.0	fFourCorner	27180
U+6B24.0	fHakka	 ji2
U+6B24.0	fCantonese	jyu4
U+6B24.0	fMandarin	yu2; yu3; yu4
U+6B24.0	fEnglish	simplified form of &#x6B5F;
U+6B25.0	fUTF8	欥
U+6B25.0	fCNS11643	2-274A
U+6B25.0	fBig5	CCE9
U+6B25.0	fGB	5-4A39
U+6B25.0	fR/S	 76.4
U+6B25.0	fTotalStrokes	 8
U+6B25.0	fCangjie	ANO
U+6B25.0	fFourCorner	67082
U+6B25.0	fCantonese	him3; jat6; jyut6
U+6B25.0	fMandarin	yu4; huan1
U+6B26.0	fUTF8	欦
U+6B26.0	fCNS11643	3-2953
U+6B26.0	fGB	3-4E32
U+6B26.0	fR/S	 76.4
U+6B26.0	fTotalStrokes	 8
U+6B26.0	fCangjie	ONNO
U+6B26.0	fFourCorner	87280
U+6B26.0	fCantonese	him1
U+6B26.0	fMandarin	qian1; han1; xian1; qian3; han3
U+6B27.0	fUTF8	欧
U+6B27.0	fCNS11643	3-2954
U+6B27.0	fGB	0-4537
U+6B27.0	fR/S	 76.4
U+6B27.0	fTotalStrokes	 8
U+6B27.0	fCangjie	SKNO
U+6B27.0	fFourCorner	77780
U+6B27.0	fHakka	 eu1
U+6B27.0	fCantonese	au1; ngau1 (also); au2; ngau2 (also)
U+6B27.0	fMandarin	ou1; ou3
U+6B27.0	fEnglish	simplified form of &#x6B50;
U+6B28.0	fUTF8	欨
U+6B28.0	fCNS11643	2-2B46
U+6B28.0	fBig5	CF64
U+6B28.0	fGB	5-5027
U+6B28.0	fR/S	 76.5
U+6B28.0	fTotalStrokes	 9
U+6B28.0	fCangjie	PRNO
U+6B28.0	fFourCorner	27682
U+6B28.0	fHakka	  hi1
U+6B28.0	fCantonese	heoi2; heoi1 (also); hei1
U+6B28.0	fMandarin	xu1; xu3; xu4
U+6B28.0	fEnglish	 blow or breathe upon; smile
U+6B29.0	fUTF8	欩
U+6B29.0	fCNS11643	4-2972
U+6B29.0	fGB	3-4E38
U+6B29.0	fR/S	 76.5
U+6B29.0	fTotalStrokes	 9; 13
U+6B29.0	fCangjie	SRNO
U+6B29.0	fFourCorner	17680
U+6B29.0	fCantonese	ciu1
U+6B29.0	fMandarin	chao1
U+6B2A.0	fUTF8	欪
U+6B2A.0	fCNS11643	3-2D3C
U+6B2A.0	fGB	3-4E37
U+6B2A.0	fR/S	  76.5
U+6B2A.0	fTotalStrokes	 9
U+6B2A.0	fCangjie	UUNO
U+6B2A.0	fFourCorner	27782
U+6B2A.0	fCantonese	ceot1
U+6B2A.0	fMandarin	chu4; chi4
U+6B2B.0	fUTF8	欫
U+6B2B.0	fCNS11643	3-3166
U+6B2B.0	fGB	E-2F46
U+6B2B.0	fR/S	 76.6
U+6B2B.0	fTotalStrokes	 10
U+6B2B.0	fCangjie	OMNO
U+6B2B.0	fFourCorner	87780
U+6B2B.0	fCantonese	kik1
U+6B2B.0	fMandarin	chi1; qi4
U+6B2C.0	fUTF8	欬
U+6B2C.0	fCNS11643	2-2F77
U+6B2C.0	fBig5	D1F5
U+6B2C.0	fGB	E-2F47
U+6B2C.0	fR/S	 76.6
U+6B2C.0	fTotalStrokes	 10
U+6B2C.0	fCangjie	YONO
U+6B2C.0	fFourCorner	07882
U+6B2C.0	fHakka	 koi3; kai3; ket7
U+6B2C.0	fCantonese	kat1; haai1; koi3; hoi4
U+6B2C.0	fMandarin	ke2; qi4; ai4
U+6B2C.0	fEnglish	[1] cough [2] sound of laughter
U+6B2C.1	fUTF8	欬
U+6B2C.1	fCNS11643	2-2F77
U+6B2C.1	fBig5	D1F5
U+6B2C.1	fGB	E-2F47
U+6B2C.1	fR/S	 76.6
U+6B2C.1	fTotalStrokes	 10
U+6B2C.1	fCangjie	YONO
U+6B2C.1	fFourCorner	07882
U+6B2C.1	fMandarin	kai4
U+6B2C.1	fEnglish	asthma and coughing
U+6B2D.0	fUTF8	欭
U+6B2D.0	fCNS11643	2-2F79
U+6B2D.0	fBig5	D1F7
U+6B2D.0	fGB	3-4E40
U+6B2D.0	fR/S	 76.6
U+6B2D.0	fTotalStrokes	 10
U+6B2D.0	fCangjie	WKNO
U+6B2D.0	fFourCorner	67082
U+6B2D.0	fHakka	 ji5; ri5; rin3; jin3
U+6B2D.0	fCantonese	ji3; jan1; ji6
U+6B2D.0	fMandarin	yi4; yin1
U+6B2D.0	fEnglish	the '-er-er", of a hesitating speaker
U+6B2E.0	fUTF8	欮
U+6B2E.0	fCNS11643	3-3165
U+6B2E.0	fGB	5-502B
U+6B2E.0	fR/S	  76.6
U+6B2E.0	fTotalStrokes	 10
U+6B2E.0	fCangjie	TUNO
U+6B2E.0	fFourCorner	87480
U+6B2E.0	fHakka	 kiet7; ket7
U+6B2E.0	fCantonese	kyut3
U+6B2E.0	fMandarin	jue2
U+6B2E.0	fEnglish	 hiccough dig out to expand
U+6B2F.0	fUTF8	欯
U+6B2F.0	fCNS11643	2-2F78
U+6B2F.0	fBig5	D1F6
U+6B2F.0	fGB	3-4E3B
U+6B2F.0	fR/S	 76.6
U+6B2F.0	fTotalStrokes	 10
U+6B2F.0	fCangjie	GRNO
U+6B2F.0	fFourCorner	47682
U+6B2F.0	fHakka	 git7
U+6B2F.0	fCantonese	hat1; koi3; kat1
U+6B2F.0	fMandarin	xi2; kai4
U+6B2F.0	fEnglish	[1] rejoice [2] smile
U+6B30.0	fUTF8	欰
U+6B30.0	fCNS11643	3-3167
U+6B30.0	fGB	5-6466
U+6B30.0	fR/S	 76.6
U+6B30.0	fTotalStrokes	 10
U+6B30.0	fCangjie	HTNO
U+6B30.0	fFourCorner	27180
U+6B30.0	fCantonese	seot1
U+6B30.0	fMandarin	xu1
U+6B31.0	fUTF8	欱
U+6B31.0	fCNS11643	2-2F7A
U+6B31.0	fBig5	D1F8
U+6B31.0	fGB	3-4E42
U+6B31.0	fR/S	 76.6
U+6B31.0	fTotalStrokes	 10
U+6B31.0	fCangjie	ORNO
U+6B31.0	fFourCorner	87682
U+6B31.0	fHakka	  hap8
U+6B31.0	fCantonese	hap6; hot3; hap1
U+6B31.0	fMandarin	xia4; he2; he1
U+6B31.0	fEnglish	 drink; to sip
U+6B32.0	fUTF8	欲
U+6B32.0	fCNS11643	1-5B43
U+6B32.0	fBig5	B1FD
U+6B32.0	fGB	0-537B
U+6B32.0	fR/S	 76.7
U+6B32.0	fTotalStrokes	 11
U+6B32.0	fCangjie	CRNO
U+6B32.0	fFourCorner	87682
U+6B32.0	fHakka	  juk8; ruk8
U+6B32.0	fCantonese	juk6
U+6B32.0	fMandarin	yu4
U+6B32.0	fEnglish	desire; want; long for; intend; wish; expectation; longing
U+6B33.0	fUTF8	欳
U+6B33.0	fCNS11643	2-363B
U+6B33.0	fBig5	D5D7
U+6B33.0	fGB	E-2F48
U+6B33.0	fR/S	 76.7
U+6B33.0	fTotalStrokes	 11
U+6B33.0	fCangjie	YBNO
U+6B33.0	fFourCorner	27282
U+6B33.0	fCantonese	wai2; faai3
U+6B33.0	fMandarin	kuai4
U+6B34.0	fUTF8	欴
U+6B34.0	fCNS11643	2-2F7B
U+6B34.0	fBig5	D1F9
U+6B34.0	fGB	E-2F49
U+6B34.0	fR/S	 76.7
U+6B34.0	fTotalStrokes	 11
U+6B34.0	fCangjie	IINO
U+6B34.0	fFourCorner	37782
U+6B34.0	fCantonese	long4
U+6B34.0	fMandarin	lang2
U+6B35.0	fUTF8	欵
U+6B35.0	fCNS11643	3-3662
U+6B35.0	fGB	E-2F4A
U+6B35.0	fR/S	  76.7
U+6B35.0	fTotalStrokes	 11
U+6B35.0	fCangjie	PKNO
U+6B35.0	fFourCorner	27480
U+6B35.0	fHakka	 k'wan3;  kan3; kwan3; kwan5
U+6B35.0	fCantonese	fun2
U+6B35.0	fMandarin	kuan3
U+6B35.0	fEnglish	 treat well detain
U+6B36.0	fUTF8	欶
U+6B36.0	fCNS11643	2-363A
U+6B36.0	fBig5	D5D6
U+6B36.0	fGB	3-4E44
U+6B36.0	fR/S	 76.7
U+6B36.0	fTotalStrokes	 11
U+6B36.0	fCangjie	DLNO
U+6B36.0	fFourCorner	57982
U+6B36.0	fHakka	 suk7; sok7
U+6B36.0	fCantonese	sok3; cuk1
U+6B36.0	fMandarin	shuo4; suo1; su4; sou4
U+6B36.0	fEnglish	 suck in; imbibe
U+6B37.0	fUTF8	欷
U+6B37.0	fCNS11643	2-363C
U+6B37.0	fBig5	D5D8
U+6B37.0	fGB	0-6C24
U+6B37.0	fR/S	 76.7
U+6B37.0	fTotalStrokes	 11
U+6B37.0	fCangjie	KBNO
U+6B37.0	fFourCorner	47282
U+6B37.0	fHakka	 hi1
U+6B37.0	fCantonese	hei1
U+6B37.0	fMandarin	xi1; xi4
U+6B37.0	fEnglish	 sob
U+6B38.0	fUTF8	欸
U+6B38.0	fCNS11643	2-363D
U+6B38.0	fBig5	D5D9
U+6B38.0	fGB	3-4E46
U+6B38.0	fR/S	 76.7
U+6B38.0	fTotalStrokes	 11
U+6B38.0	fCangjie	IKNO
U+6B38.0	fFourCorner	27482
U+6B38.0	fHakka	 ai1; kan3; ai3
U+6B38.0	fCantonese	ei6; ngei6 (also); ei1 (also); oi1; ngei1 (also); aai2; ei3 (also); ngei3 (also); ngoi1 (also); ai2; e6; oi2; ngaai2 (also); ngoi2
U+6B38.0	fMandarin	ai1; ai3; ei1; yi1; ye4
U+6B38.0	fEnglish	[1] the sound of answering [2] sighs [3] the sound of rowing a boat
U+6B38.1	fUTF8	欸
U+6B38.1	fCNS11643	2-363D
U+6B38.1	fBig5	D5D9
U+6B38.1	fGB	3-4E46
U+6B38.1	fR/S	 76.7
U+6B38.1	fTotalStrokes	 11
U+6B38.1	fCangjie	IKNO
U+6B38.1	fFourCorner	27482
U+6B38.1	fMandarin	ei4
U+6B38.1	fEnglish	an exclamatory sound; an exclamation indicating promise or affirmation
U+6B39.0	fUTF8	欹
U+6B39.0	fCNS11643	2-3C7D
U+6B39.0	fBig5	D9DA
U+6B39.0	fGB	0-6C25
U+6B39.0	fR/S	 76.8
U+6B39.0	fTotalStrokes	 12
U+6B39.0	fCangjie	KRNO
U+6B39.0	fFourCorner	47682
U+6B39.0	fHakka	 ji1; ri2; ji2; ki1; i1
U+6B39.0	fCantonese	ji1; kei1
U+6B39.0	fMandarin	yi1; qi1
U+6B39.0	fEnglish	an interjection of pleasure; Bravo!; Good!
U+6B3A.0	fUTF8	欺
U+6B3A.0	fCNS11643	1-6022
U+6B3A.0	fBig5	B4DB
U+6B3A.0	fGB	0-465B
U+6B3A.0	fR/S	 76.8
U+6B3A.0	fTotalStrokes	 12
U+6B3A.0	fCangjie	TCNO
U+6B3A.0	fFourCorner	47882
U+6B3A.0	fHakka	  ki1
U+6B3A.0	fCantonese	hei1
U+6B3A.0	fMandarin	qi1
U+6B3A.0	fEnglish	[1] cheat; deceive; swindle; impose on; take advantage of [2] disregard the dictates of one's own conscience [3] insult; bully
U+6B3B.0	fUTF8	欻
U+6B3B.0	fCNS11643	2-3C7E
U+6B3B.0	fBig5	D9DB
U+6B3B.0	fGB	3-4E4C
U+6B3B.0	fR/S	 76.8
U+6B3B.0	fTotalStrokes	 12
U+6B3B.0	fCangjie	FFNO
U+6B3B.0	fFourCorner	97882
U+6B3B.0	fHakka	 fut7
U+6B3B.0	fCantonese	fat1; caat3; caa3; cuk1; caa1
U+6B3B.0	fMandarin	hu1; xu1; xu4
U+6B3B.0	fEnglish	sudden; suddenly; abrupt; abruptly
U+6B3B.1	fUTF8	欻
U+6B3B.1	fCNS11643	2-3C7E
U+6B3B.1	fBig5	D9DB
U+6B3B.1	fGB	3-4E4C
U+6B3B.1	fR/S	 76.8
U+6B3B.1	fTotalStrokes	 12
U+6B3B.1	fCangjie	FFNO
U+6B3B.1	fFourCorner	97882
U+6B3B.1	fMandarin	chua1
U+6B3B.1	fEnglish	descriptive of the sound
U+6B3C.0	fUTF8	欼
U+6B3C.0	fCNS11643	2-3D22
U+6B3C.0	fBig5	D9DD
U+6B3C.0	fGB	5-502F
U+6B3C.0	fR/S	 76.8
U+6B3C.0	fTotalStrokes	 12
U+6B3C.0	fCangjie	EEEEO
U+6B3C.0	fFourCorner	17482
U+6B3C.0	fHakka	 chot7; zot7; ch'ot7; cot7; {ch'ot7}
U+6B3C.0	fCantonese	zyut3; syut3; ci2
U+6B3C.0	fMandarin	chi3; chuai4
U+6B3C.0	fEnglish	drink
U+6B3D.0	fUTF8	欽
U+6B3D.0	fCNS11643	1-6023
U+6B3D.0	fBig5	B4DC
U+6B3D.0	fGB	1-4755
U+6B3D.0	fR/S	 76.8
U+6B3D.0	fTotalStrokes	 12
U+6B3D.0	fCangjie	CNO
U+6B3D.0	fFourCorner	87182
U+6B3D.0	fHakka	  kim1
U+6B3D.0	fCantonese	jam1
U+6B3D.0	fMandarin	qin1; qin4; yin2
U+6B3D.0	fEnglish	[1] respect; respectful; admire [2] a term used to address a monarch; your Majesty [3] a Chinese family name
U+6B3E.0	fUTF8	款
U+6B3E.0	fCNS11643	1-6021
U+6B3E.0	fBig5	B4DA
U+6B3E.0	fGB	0-3F6E
U+6B3E.0	fR/S	 76.8
U+6B3E.0	fTotalStrokes	 12
U+6B3E.0	fCangjie	GFNO
U+6B3E.0	fFourCorner	47982
U+6B3E.0	fHakka	  kan3;  k'wan3; k'uan3; kwan3
U+6B3E.0	fCantonese	fun2
U+6B3E.0	fMandarin	kuan3; xin1
U+6B3E.0	fEnglish	[1] item; article [2] fund; sum of money; money [3] entertain; treat well [4] sincerity; sincere; sincerely [5] slow; slowly [6] knock (at a door) [7] empty (words, etc.)
U+6B3F.0	fUTF8	欿
U+6B3F.0	fCNS11643	2-3D21
U+6B3F.0	fBig5	D9DC
U+6B3F.0	fGB	3-4E4A
U+6B3F.0	fR/S	 76.8
U+6B3F.0	fTotalStrokes	 12
U+6B3F.0	fCangjie	NXNO
U+6B3F.0	fFourCorner	27782
U+6B3F.0	fHakka	 kam3; kam1
U+6B3F.0	fCantonese	ham2; daam6
U+6B3F.0	fMandarin	kan3; tan1; kan1; han4
U+6B3F.0	fEnglish	[1] discontented with oneself [2] sad; gloomy
U+6B40.0	fUTF8	歀
U+6B40.0	fCNS11643	3-417D
U+6B40.0	fGB	5-5031
U+6B40.0	fR/S	  76.9
U+6B40.0	fTotalStrokes	 13
U+6B40.0	fCangjie	DFNO
U+6B40.0	fFourCorner	47980
U+6B40.0	fCantonese	fun2
U+6B40.0	fMandarin	kuan3
U+6B40.0	fEnglish	same as &#x6B3E;
U+6B41.0	fUTF8	歁
U+6B41.0	fCNS11643	2-4422
U+6B41.0	fBig5	DDFA
U+6B41.0	fGB	3-4E4E
U+6B41.0	fR/S	 76.9
U+6B41.0	fTotalStrokes	 13
U+6B41.0	fCangjie	TVNO
U+6B41.0	fFourCorner	47782
U+6B41.0	fHakka	  kam3
U+6B41.0	fCantonese	ham2; ham3; zam3
U+6B41.0	fMandarin	kan3; kan1; qian1; qian3; ke4
U+6B41.0	fEnglish	 eat and not be satisfied
U+6B42.0	fUTF8	歂
U+6B42.0	fCNS11643	2-437E
U+6B42.0	fBig5	DDF8
U+6B42.0	fGB	5-5033
U+6B42.0	fR/S	 76.9
U+6B42.0	fTotalStrokes	 13
U+6B42.0	fCangjie	UBNO
U+6B42.0	fFourCorner	27282
U+6B42.0	fHakka	 chon3; zhon1; zon1; con3
U+6B42.0	fCantonese	cyun2; cyun4; syun4
U+6B42.0	fMandarin	chuan2; chuan3
U+6B43.0	fUTF8	歃
U+6B43.0	fCNS11643	2-437D
U+6B43.0	fBig5	DDF7
U+6B43.0	fGB	0-6C26
U+6B43.0	fR/S	 76.9
U+6B43.0	fTotalStrokes	 13
U+6B43.0	fCangjie	HXNO
U+6B43.0	fFourCorner	27782
U+6B43.0	fHakka	 cap7
U+6B43.0	fCantonese	saap3; sip3
U+6B43.0	fMandarin	sha4; xia2
U+6B43.0	fEnglish	 smear one's mouth with blood of a victim when taking an oath; drink blood
U+6B44.0	fUTF8	歄
U+6B44.0	fCNS11643	3-417C
U+6B44.0	fGB	3-4E45
U+6B44.0	fR/S	  76.9
U+6B44.0	fTotalStrokes	 13
U+6B44.0	fCangjie	BBNO
U+6B44.0	fFourCorner	77280
U+6B44.0	fCantonese	gwaa1
U+6B44.0	fMandarin	gua1
U+6B45.0	fUTF8	歅
U+6B45.0	fCNS11643	2-437C
U+6B45.0	fBig5	DDF6
U+6B45.0	fGB	5-5032
U+6B45.0	fR/S	 76.9
U+6B45.0	fTotalStrokes	 13
U+6B45.0	fCangjie	MGNO
U+6B45.0	fFourCorner	17182
U+6B45.0	fCantonese	jan1
U+6B45.0	fMandarin	yin1; yan1; yi1
U+6B46.0	fUTF8	歆
U+6B46.0	fCNS11643	2-437B
U+6B46.0	fBig5	DDF5
U+6B46.0	fGB	0-6C27
U+6B46.0	fR/S	 76.9
U+6B46.0	fTotalStrokes	 13
U+6B46.0	fCangjie	YANO
U+6B46.0	fFourCorner	07682
U+6B46.0	fHakka	 him1; him5; kim1
U+6B46.0	fCantonese	jam1
U+6B46.0	fMandarin	xin1
U+6B46.0	fEnglish	[1] like; admire; submit to willingly [2] accept offerings (said of gods, etc.) [3] move
U+6B47.0	fUTF8	歇
U+6B47.0	fCNS11643	1-6458
U+6B47.0	fBig5	B7B2
U+6B47.0	fGB	0-502A
U+6B47.0	fR/S	 76.9
U+6B47.0	fTotalStrokes	 13
U+6B47.0	fCangjie	AVNO
U+6B47.0	fFourCorner	67782
U+6B47.0	fHakka	  hiet7;  het7; hiat7; ket7; het8
U+6B47.0	fCantonese	hit3
U+6B47.0	fMandarin	xie1; ya4; he1
U+6B47.0	fEnglish	[1] rest [2] sleep [3] come to an end; stop [4] lodge [5] in; over (period) [6] passed
U+6B48.0	fUTF8	歈
U+6B48.0	fCNS11643	2-4421
U+6B48.0	fBig5	DDF9
U+6B48.0	fGB	3-4E52
U+6B48.0	fR/S	 76.9
U+6B48.0	fTotalStrokes	 13
U+6B48.0	fCangjie	ONNO
U+6B48.0	fFourCorner	87280
U+6B48.0	fHakka	 ji2; ri2
U+6B48.0	fCantonese	jyu4
U+6B48.0	fMandarin	yu2; tou2
U+6B48.0	fEnglish	songs
U+6B49.0	fUTF8	歉
U+6B49.0	fCNS11643	1-6939
U+6B49.0	fBig5	BA70
U+6B49.0	fGB	0-4738
U+6B49.0	fR/S	 76.10
U+6B49.0	fTotalStrokes	 14
U+6B49.0	fCangjie	TCNO
U+6B49.0	fFourCorner	87282
U+6B49.0	fHakka	 kiap7; kiam1; kiam5; hiap7
U+6B49.0	fCantonese	hip3; him3
U+6B49.0	fMandarin	qian4; qian3
U+6B49.0	fEnglish	[1] deficient; lacking; insufficient; deficiency [2] poor crop; poor harvest [3] regret; sorry [4] apology; apologetic
U+6B4A.0	fUTF8	歊
U+6B4A.0	fCNS11643	2-4B2B
U+6B4A.0	fBig5	E263
U+6B4A.0	fGB	3-7B70
U+6B4A.0	fR/S	 76.10
U+6B4A.0	fTotalStrokes	 14
U+6B4A.0	fCangjie	YBNO
U+6B4A.0	fFourCorner	07282
U+6B4A.0	fHakka	 hiau1; hieu1; hau2
U+6B4A.0	fCantonese	hiu1
U+6B4A.0	fMandarin	xiao1
U+6B4A.0	fEnglish	sigh
U+6B4B.0	fUTF8	歋
U+6B4B.0	fCNS11643	2-4B2D
U+6B4B.0	fBig5	E265
U+6B4B.0	fGB	5-5035
U+6B4B.0	fR/S	 76.10
U+6B4B.0	fTotalStrokes	 14
U+6B4B.0	fCangjie	HUNO
U+6B4B.0	fFourCorner	77282
U+6B4B.0	fCantonese	ji4; je4
U+6B4B.0	fMandarin	yi2; ye2
U+6B4C.0	fUTF8	歌
U+6B4C.0	fCNS11643	1-693A
U+6B4C.0	fBig5	BA71
U+6B4C.0	fGB	0-3868
U+6B4C.0	fR/S	 76.10
U+6B4C.0	fTotalStrokes	 14
U+6B4C.0	fCangjie	MRNO
U+6B4C.0	fFourCorner	17682
U+6B4C.0	fHakka	  go1; go2
U+6B4C.0	fCantonese	go1
U+6B4C.0	fMandarin	ge1
U+6B4C.0	fEnglish	[1] song; chant [2] praise [3] song [4] poems with rhythms and rhymes suitable for use as lyrics in songs
U+6B4C.1	fUTF8	歌
U+6B4C.1	fCNS11643	1-693A
U+6B4C.1	fBig5	BA71
U+6B4C.1	fGB	0-3868
U+6B4C.1	fR/S	 76.10
U+6B4C.1	fTotalStrokes	 14
U+6B4C.1	fCangjie	MRNO
U+6B4C.1	fFourCorner	17682
U+6B4C.1	fMandarin	ge5
U+6B4D.0	fUTF8	歍
U+6B4D.0	fCNS11643	2-4B2C
U+6B4D.0	fBig5	E264
U+6B4D.0	fGB	3-4E33
U+6B4D.0	fR/S	 76.10
U+6B4D.0	fTotalStrokes	 14
U+6B4D.0	fCangjie	HFNO
U+6B4D.0	fFourCorner	27382
U+6B4D.0	fHakka	 wu1; vu1
U+6B4D.0	fCantonese	wu1
U+6B4D.0	fMandarin	wu1; yang1
U+6B4D.0	fEnglish	[1] vomit [2] feed with the mouth, as birds do
U+6B4E.0	fUTF8	歎
U+6B4E.0	fCNS11643	1-6D45
U+6B4E.0	fBig5	BCDB
U+6B4E.0	fGB	E-2F4B
U+6B4E.0	fR/S	 76.11
U+6B4E.0	fTotalStrokes	 15
U+6B4E.0	fCangjie	TONO
U+6B4E.0	fFourCorner	47582
U+6B4E.0	fHakka	  tan5
U+6B4E.0	fCantonese	taan3
U+6B4E.0	fMandarin	tan4; tan1
U+6B4E.0	fEnglish	 sigh in wonderment of lamentatuion; exclaim
U+6B4F.0	fUTF8	歏
U+6B4F.0	fCNS11643	3-4C66
U+6B4F.0	fGB	3-4E56
U+6B4F.0	fR/S	  76.11
U+6B4F.0	fTotalStrokes	 15
U+6B4F.0	fCangjie	TMNO
U+6B4F.0	fFourCorner	47180
U+6B4F.0	fCantonese	gan6
U+6B4F.0	fMandarin	jin4
U+6B50.0	fUTF8	歐
U+6B50.0	fCNS11643	1-6D44
U+6B50.0	fBig5	BCDA
U+6B50.0	fGB	1-4537
U+6B50.0	fR/S	 76.11
U+6B50.0	fTotalStrokes	 15
U+6B50.0	fCangjie	SRNO
U+6B50.0	fFourCorner	77782
U+6B50.0	fHakka	  eu1; iu1
U+6B50.0	fCantonese	au1; ngau1 (also); au2; ngau2 (also)
U+6B50.0	fMandarin	ou1; ou3
U+6B50.0	fEnglish	[1] Europe; European [2] same as $; [3] same as $; vomit [4] same as $; beat [5] same as $; sing [6] ohm [7] a Chinese family name
U+6B51.0	fUTF8	歑
U+6B51.0	fCNS11643	2-5139
U+6B51.0	fBig5	E5F0
U+6B51.0	fGB	5-5039
U+6B51.0	fR/S	 76.11
U+6B51.0	fTotalStrokes	 15
U+6B51.0	fCangjie	YDNO
U+6B51.0	fFourCorner	27282
U+6B51.0	fHakka	 fu1
U+6B51.0	fCantonese	fu1
U+6B51.0	fMandarin	hu1
U+6B52.0	fUTF8	歒
U+6B52.0	fCNS11643	3-4C65
U+6B52.0	fGB	E-2F4C
U+6B52.0	fR/S	 76.11
U+6B52.0	fTotalStrokes	 15
U+6B52.0	fCangjie	YBNO
U+6B52.0	fFourCorner	07280
U+6B52.0	fCantonese	tik1
U+6B52.0	fMandarin	ti4
U+6B53.0	fUTF8	歓
U+6B53.0	fCNS11643	3-4C67
U+6B53.0	fGB	E-2F4D
U+6B53.0	fR/S	 76.11
U+6B53.0	fTotalStrokes	 15
U+6B53.0	fCangjie	OGNO
U+6B53.0	fFourCorner	87280
U+6B53.0	fCantonese	fun1
U+6B53.0	fMandarin	huan1
U+6B53.0	fEnglish	 happy; pleased; glad joy enjoy
U+6B54.0	fUTF8	歔
U+6B54.0	fCNS11643	2-576A
U+6B54.0	fBig5	E9DF
U+6B54.0	fGB	5-5038
U+6B54.0	fR/S	 76.12
U+6B54.0	fTotalStrokes	 16
U+6B54.0	fCangjie	YMNO
U+6B54.0	fFourCorner	27282
U+6B54.0	fHakka	  hi1
U+6B54.0	fCantonese	heoi1
U+6B54.0	fMandarin	xu1
U+6B54.0	fEnglish	 blow; exhale
U+6B55.0	fUTF8	歕
U+6B55.0	fCNS11643	2-5769
U+6B55.0	fBig5	E9DE
U+6B55.0	fGB	5-5030
U+6B55.0	fR/S	 76.12; 76.13
U+6B55.0	fTotalStrokes	 16
U+6B55.0	fCangjie	JCNO
U+6B55.0	fFourCorner	07882
U+6B55.0	fHakka	  pun2
U+6B55.0	fCantonese	pan3; fan5 (also); pan1 (also)
U+6B55.0	fMandarin	pen1; pen4; fen4
U+6B55.0	fEnglish	spurt; blow out; puff out
U+6B56.0	fUTF8	歖
U+6B56.0	fCNS11643	2-576B
U+6B56.0	fBig5	E9E0
U+6B56.0	fGB	5-503C
U+6B56.0	fR/S	 76.12
U+6B56.0	fTotalStrokes	 16
U+6B56.0	fCangjie	GRNO
U+6B56.0	fFourCorner	47682
U+6B56.0	fHakka	 hi3; hi1
U+6B56.0	fCantonese	hei2
U+6B56.0	fMandarin	xi1; yi3
U+6B56.0	fEnglish	joyful
U+6B57.0	fUTF8	歗
U+6B57.0	fCNS11643	3-5146
U+6B57.0	fGB	5-502E
U+6B57.0	fR/S	 76.12
U+6B57.0	fTotalStrokes	 16; 17
U+6B57.0	fCangjie	LXNO
U+6B57.0	fFourCorner	57280
U+6B57.0	fHakka	  siau5; seu5
U+6B57.0	fCantonese	siu3
U+6B57.0	fMandarin	xiao4
U+6B57.0	fEnglish	[1] scream; whistle [2] sigh; groan
U+6B58.0	fUTF8	歘
U+6B58.0	fCNS11643	3-5145
U+6B58.0	fGB	5-503D
U+6B58.0	fR/S	 76.12
U+6B58.0	fTotalStrokes	 16
U+6B58.0	fCangjie	FFNO
U+6B58.0	fFourCorner	97880
U+6B58.0	fHakka	 fut7
U+6B58.0	fCantonese	fat1; caat3 (also)
U+6B58.0	fMandarin	hu1; xu1; chua1
U+6B58.0	fEnglish	sudden; abrupt; to sniff
U+6B59.0	fUTF8	歙
U+6B59.0	fCNS11643	1-7124
U+6B59.0	fBig5	BEF9
U+6B59.0	fGB	0-6C28
U+6B59.0	fR/S	 76.12
U+6B59.0	fTotalStrokes	 16
U+6B59.0	fCangjie	OMNO
U+6B59.0	fFourCorner	87182
U+6B59.0	fHakka	 kip7;  hip7; hiak8; hiap8
U+6B59.0	fCantonese	kap1; sip3; hap1; jap1
U+6B59.0	fMandarin	xi1; xi4; xie2
U+6B59.0	fEnglish	 suck; inhale
U+6B59.1	fUTF8	歙
U+6B59.1	fCNS11643	1-7124
U+6B59.1	fBig5	BEF9
U+6B59.1	fGB	0-6C28
U+6B59.1	fR/S	 76.12
U+6B59.1	fTotalStrokes	 16
U+6B59.1	fCangjie	OMNO
U+6B59.1	fFourCorner	87182
U+6B59.1	fMandarin	she4
U+6B59.1	fEnglish	a name of a county in Anhui province
U+6B5A.0	fUTF8	歚
U+6B5A.0	fCNS11643	F-5539
U+6B5A.0	fGB	E-2F4E
U+6B5A.0	fR/S	 76.12
U+6B5A.0	fTotalStrokes	 16
U+6B5A.0	fCangjie	TRNO
U+6B5A.0	fCantonese	sin6
U+6B5A.0	fEnglish	 used in name of famous Korean painter Ceng Sen
U+6B5B.0	fUTF8	歛
U+6B5B.0	fCNS11643	2-5D37
U+6B5B.0	fBig5	ED4B
U+6B5B.0	fGB	E-2F4F
U+6B5B.0	fR/S	 76.13
U+6B5B.0	fTotalStrokes	 17
U+6B5B.0	fCangjie	OONO
U+6B5B.0	fFourCorner	87882
U+6B5B.0	fHakka	  ham5;  ham1; liam3; liam1; lien1
U+6B5B.0	fCantonese	lim5; lim6 (also)
U+6B5B.0	fMandarin	han1; han4; lian3; qian1; lian4
U+6B5B.0	fEnglish	[1] collect; gather [2] hold together [3] deduct; subtract [4] desire; ask for something
U+6B5C.0	fUTF8	歜
U+6B5C.0	fCNS11643	1-737C
U+6B5C.0	fBig5	C0D3
U+6B5C.0	fGB	3-4E58
U+6B5C.0	fR/S	 76.13
U+6B5C.0	fTotalStrokes	 17
U+6B5C.0	fCangjie	WINO
U+6B5C.0	fFourCorner	67180
U+6B5C.0	fHakka	 ch'uk7; cuk7
U+6B5C.0	fCantonese	cuk1
U+6B5C.0	fMandarin	chu4; can3
U+6B5C.0	fEnglish	furious; wrathful
U+6B5D.0	fUTF8	歝
U+6B5D.0	fCNS11643	3-552B
U+6B5D.0	fGB	5-5028
U+6B5D.0	fR/S	  76.13
U+6B5D.0	fTotalStrokes	 17
U+6B5D.0	fCangjie	WJNO
U+6B5D.0	fFourCorner	67480
U+6B5D.0	fCantonese	jik6
U+6B5D.0	fMandarin	yi4
U+6B5E.0	fUTF8	歞
U+6B5E.0	fCNS11643	2-6174
U+6B5E.0	fBig5	EFE8
U+6B5E.0	fGB	3-4E59
U+6B5E.0	fR/S	 76.14
U+6B5E.0	fTotalStrokes	 18
U+6B5E.0	fCangjie	AFNO
U+6B5E.0	fFourCorner	67382
U+6B5E.0	fCantonese	ngap6; ap6 (also)
U+6B5E.0	fMandarin	kan3
U+6B5F.0	fUTF8	歟
U+6B5F.0	fCNS11643	1-7656
U+6B5F.0	fBig5	C26A
U+6B5F.0	fGB	1-6C23
U+6B5F.0	fR/S	 76.14
U+6B5F.0	fTotalStrokes	 18
U+6B5F.0	fCangjie	HCNO
U+6B5F.0	fFourCorner	77882
U+6B5F.0	fHakka	 ji2; ri2; i2
U+6B5F.0	fCantonese	jyu4
U+6B5F.0	fMandarin	yu2; yu3; yu4
U+6B5F.0	fEnglish	a final particle indicating doubt, surprise, exclamation, etc.
U+6B60.0	fUTF8	歠
U+6B60.0	fCNS11643	2-6564
U+6B60.0	fBig5	F259
U+6B60.0	fGB	3-4E5B
U+6B60.0	fR/S	 76.15
U+6B60.0	fTotalStrokes	 19
U+6B60.0	fCangjie	EWNO
U+6B60.0	fFourCorner	77682
U+6B60.0	fHakka	 ch'ot7; cot7
U+6B60.0	fCantonese	zyut3; cyut3
U+6B60.0	fMandarin	chuo4
U+6B60.0	fEnglish	 sip; drink; suck
U+6B61.0	fUTF8	歡
U+6B61.0	fCNS11643	1-7B63
U+6B61.0	fBig5	C577
U+6B61.0	fGB	1-3B36
U+6B61.0	fR/S	 76.18
U+6B61.0	fTotalStrokes	 22
U+6B61.0	fCangjie	TGNO
U+6B61.0	fFourCorner	47282
U+6B61.0	fHakka	  fon1; fan1; kon1
U+6B61.0	fCantonese	fun1
U+6B61.0	fMandarin	huan1
U+6B61.0	fEnglish	[1] happy; pleased; glad; joy; jubilant [2] joys; pleasures [3] lover [4] active and energetic; quick; in full swing [5] a Chinese family name
U+6B61.1	fUTF8	歡
U+6B61.1	fCNS11643	1-7B63
U+6B61.1	fBig5	C577
U+6B61.1	fGB	1-3B36
U+6B61.1	fR/S	 76.18
U+6B61.1	fTotalStrokes	 22
U+6B61.1	fCangjie	TGNO
U+6B61.1	fFourCorner	47282
U+6B61.1	fMandarin	huan5
U+6B62.0	fUTF8	止
U+6B62.0	fCNS11643	1-454F
U+6B62.0	fBig5	A4EE
U+6B62.0	fGB	0-5639
U+6B62.0	fR/S	 77.0
U+6B62.0	fTotalStrokes	 4
U+6B62.0	fCangjie	YLM
U+6B62.0	fFourCorner	21100
U+6B62.0	fHakka	 zi3;  zhi3; zih3; tje3
U+6B62.0	fCantonese	zi2
U+6B62.0	fMandarin	zhi3
U+6B62.0	fEnglish	[1] stop; halt; desist; still [2] rest in; stay [3] detain [4] deportment [5] prohibit [6] come to; arrive at [7] still; calm; stagnant [8] only
U+6B63.0	fUTF8	正
U+6B63.0	fCNS11643	1-465F
U+6B63.0	fBig5	A5BF
U+6B63.0	fGB	0-557D
U+6B63.0	fR/S	 77.1
U+6B63.0	fTotalStrokes	 5
U+6B63.0	fCangjie	MYLM
U+6B63.0	fFourCorner	10101
U+6B63.0	fHakka	  zang1;  zin5; zhin5; zang5;  chang1; chang5; zihn5; ziang5; zhang1; zhang5
U+6B63.0	fCantonese	zing3; zing1; zeng3; zing2
U+6B63.0	fMandarin	zheng4; zheng3
U+6B63.0	fEnglish	[1] right; proper; correct; appropriate [2] the obverse side; right side [3] formal [4] rectify; correct [5] pure; not contaminated [6] straightforward and unbending; honestand virtuous [7] person in charge; person in command; principal [8] mete out punishment for a criminal [9] original (texts, etc.) [10] positively [11] main; principal [12] exactly; just; right [13] sharp; punctually [14] just; unbiased [15] a Chinese family name
U+6B63.1	fUTF8	正
U+6B63.1	fCNS11643	1-465F
U+6B63.1	fBig5	A5BF
U+6B63.1	fGB	0-557D
U+6B63.1	fR/S	 77.1
U+6B63.1	fTotalStrokes	 5
U+6B63.1	fCangjie	MYLM
U+6B63.1	fFourCorner	10101
U+6B63.1	fHakka	 zen5
U+6B63.1	fMandarin	zheng1
U+6B63.1	fEnglish	the first in the lunar calendar
U+6B63.2	fUTF8	正
U+6B63.2	fCNS11643	1-465F
U+6B63.2	fBig5	A5BF
U+6B63.2	fGB	0-557D
U+6B63.2	fR/S	 77.1
U+6B63.2	fTotalStrokes	 5
U+6B63.2	fCangjie	MYLM
U+6B63.2	fFourCorner	10101
U+6B63.2	fMandarin	zheng5
U+6B64.0	fUTF8	此
U+6B64.0	fCNS11643	1-483A
U+6B64.0	fBig5	A6B9
U+6B64.0	fGB	0-344B
U+6B64.0	fR/S	 77.2
U+6B64.0	fTotalStrokes	 6
U+6B64.0	fCangjie	YMP
U+6B64.0	fFourCorner	21110
U+6B64.0	fHakka	  cih3; cu3; cu5; ze3
U+6B64.0	fCantonese	ci2
U+6B64.0	fMandarin	ci3
U+6B64.0	fEnglish	[1] this; these [2] if so; in this case [3] here [4] such; thus
U+6B65.0	fUTF8	步
U+6B65.0	fCNS11643	1-4A63
U+6B65.0	fBig5	A842
U+6B65.0	fGB	0-323D
U+6B65.0	fR/S	 77.3
U+6B65.0	fTotalStrokes	 7
U+6B65.0	fCangjie	YLMH
U+6B65.0	fFourCorner	21209
U+6B65.0	fHakka	  pu5; pu6; pu3
U+6B65.0	fCantonese	bou6; fau6
U+6B65.0	fMandarin	bu4
U+6B65.0	fEnglish	[1] step; pace [2] walk; stroll; on foot [3] situation; state; degree [4] banks of rivers, ponds, etc. [5] fortune; doom [6] a unit in length in ancient China of about 5.5 feet [7] a Chinese family name
U+6B66.0	fUTF8	武
U+6B66.0	fCNS11643	1-4E3D
U+6B66.0	fBig5	AA5A
U+6B66.0	fGB	0-4E64
U+6B66.0	fR/S	 77.4
U+6B66.0	fTotalStrokes	 8
U+6B66.0	fCangjie	MPYLM
U+6B66.0	fFourCorner	13140
U+6B66.0	fHakka	  wu1;  wu3; vu3; {vu1}
U+6B66.0	fCantonese	mou5
U+6B66.0	fMandarin	wu3; wu2
U+6B66.0	fEnglish	[1] force; military [2] martial; warlike [3] footprints; steps [4] the length of a half pace [5] the string of an ancient hat [6] inherit [7] a Chinese family name
U+6B67.0	fUTF8	歧
U+6B67.0	fCNS11643	1-4E3E
U+6B67.0	fBig5	AA5B
U+6B67.0	fGB	0-4667
U+6B67.0	fR/S	 77.4
U+6B67.0	fTotalStrokes	 8
U+6B67.0	fCangjie	YMJE
U+6B67.0	fFourCorner	24147
U+6B67.0	fHakka	  ki1; kia5; ki3; ki2
U+6B67.0	fCantonese	kei4
U+6B67.0	fMandarin	qi2
U+6B67.0	fEnglish	[1] forked road; path branching out from the main road [2] forked; divergent; strayed [3] anything that goes astray
U+6B68.0	fUTF8	歨
U+6B68.0	fCNS11643	3-2955
U+6B68.0	fGB	5-4860
U+6B68.0	fR/S	  77.4
U+6B68.0	fTotalStrokes	 8
U+6B68.0	fCangjie	YLMO
U+6B68.0	fFourCorner	21800
U+6B68.0	fCantonese	bou6
U+6B68.0	fMandarin	bu4
U+6B69.0	fUTF8	歩
U+6B69.0	fCNS11643	3-2956
U+6B69.0	fGB	E-2F50
U+6B69.0	fR/S	 77.4
U+6B69.0	fTotalStrokes	 8
U+6B69.0	fCangjie	YLMH
U+6B69.0	fFourCorner	21200
U+6B69.0	fCantonese	bou6
U+6B69.0	fMandarin	bu4
U+6B69.0	fEnglish	step
U+6B6A.0	fUTF8	歪
U+6B6A.0	fCNS11643	1-5171
U+6B6A.0	fBig5	AC6E
U+6B6A.0	fGB	0-4D61
U+6B6A.0	fR/S	 77.5
U+6B6A.0	fTotalStrokes	 9
U+6B6A.0	fCangjie	MFMYM
U+6B6A.0	fFourCorner	10101
U+6B6A.0	fHakka	  wai1; {uai1}; fe3; vai1
U+6B6A.0	fCantonese	waai1; me2 (also); waai2
U+6B6A.0	fMandarin	wai1; wai3
U+6B6A.0	fEnglish	[1] slant; inclined; askew; awry; aslant; crooked; tilted; out of the perpendicular [2] depraved; evil [3] lie down on one side for a brief nap [4] shirk one's responsibility and try to involve others
U+6B6B.0	fUTF8	歫
U+6B6B.0	fCNS11643	3-2D3D
U+6B6B.0	fGB	5-485F
U+6B6B.0	fR/S	 77.5
U+6B6B.0	fTotalStrokes	 9
U+6B6B.0	fCangjie	YMSS
U+6B6B.0	fFourCorner	21110
U+6B6B.0	fCantonese	keoi5; geoi6 (also)
U+6B6B.0	fMandarin	ju4
U+6B6C.0	fUTF8	歬
U+6B6C.0	fCNS11643	3-3168
U+6B6C.0	fGB	5-4863
U+6B6C.0	fR/S	 77.6
U+6B6C.0	fTotalStrokes	 10
U+6B6C.0	fCangjie	YMHBY
U+6B6C.0	fFourCorner	21440
U+6B6C.0	fCantonese	cin4
U+6B6C.0	fMandarin	qian2
U+6B6D.0	fUTF8	歭
U+6B6D.0	fCNS11643	2-2F7C
U+6B6D.0	fBig5	D1FA
U+6B6D.0	fGB	5-4862
U+6B6D.0	fR/S	 77.6
U+6B6D.0	fTotalStrokes	 10
U+6B6D.0	fCangjie	YMGDI
U+6B6D.0	fFourCorner	24141
U+6B6D.0	fCantonese	ci4; zi6
U+6B6D.0	fMandarin	chi2; zhi4
U+6B6E.0	fUTF8	歮
U+6B6E.0	fCNS11643	3-3C49
U+6B6E.0	fGB	5-4865
U+6B6E.0	fR/S	  77.8
U+6B6E.0	fTotalStrokes	 12
U+6B6E.0	fCangjie	YMYMM
U+6B6E.0	fFourCorner	21110
U+6B6E.0	fHakka	 sep7
U+6B6E.0	fCantonese	gip3
U+6B6E.0	fMandarin	se4
U+6B6F.0	fUTF8	歯
U+6B6F.0	fCNS11643	3-3C47
U+6B6F.0	fGB	E-2F51
U+6B6F.0	fR/S	 211.0; 77.8
U+6B6F.0	fTotalStrokes	 12
U+6B6F.0	fCangjie	YMUFD
U+6B6F.0	fFourCorner	21770
U+6B6F.0	fCantonese	ci2
U+6B6F.0	fMandarin	chi3
U+6B6F.0	fEnglish	 teeth gears; cogs age Kangxi radical number 211
U+6B70.0	fUTF8	歰
U+6B70.0	fCNS11643	4-442D
U+6B70.0	fGB	5-486B
U+6B70.0	fR/S	 77.9; 77.10
U+6B70.0	fTotalStrokes	 14; 13
U+6B70.0	fCangjie	SIYMM
U+6B70.0	fFourCorner	17110
U+6B70.0	fHakka	 sep7
U+6B70.0	fCantonese	saap3 (also); sap1
U+6B70.0	fMandarin	se4; sha4
U+6B70.0	fEnglish	rough
U+6B71.0	fUTF8	歱
U+6B71.0	fCNS11643	3-417E
U+6B71.0	fGB	5-486A
U+6B71.0	fR/S	 77.9
U+6B71.0	fTotalStrokes	 13
U+6B71.0	fCangjie	YMHJG
U+6B71.0	fFourCorner	22110
U+6B71.0	fHakka	 zhung3; zung3
U+6B71.0	fCantonese	zung2; dung2 (also)
U+6B71.0	fMandarin	zhong3
U+6B71.0	fEnglish	[1] heel [2] imitate
U+6B72.0	fUTF8	歲
U+6B72.0	fCNS11643	1-6459
U+6B72.0	fBig5	B7B3
U+6B72.0	fGB	1-4B6A
U+6B72.0	fR/S	 77.9
U+6B72.0	fTotalStrokes	 13
U+6B72.0	fCangjie	YMIHH
U+6B72.0	fFourCorner	21253
U+6B72.0	fHakka	  soi5; sui5;  se5
U+6B72.0	fCantonese	seoi3
U+6B72.0	fMandarin	sui4; xue3; suo4
U+6B72.0	fEnglish	[1] year; age (of a person) [2] harvest
U+6B73.0	fUTF8	歳
U+6B73.0	fCNS11643	3-4221
U+6B73.0	fGB	E-2F52
U+6B73.0	fR/S	 77.9
U+6B73.0	fTotalStrokes	 13
U+6B73.0	fCangjie	YMIHF
U+6B73.0	fFourCorner	21250
U+6B73.0	fCantonese	seoi3
U+6B73.0	fMandarin	sui4
U+6B73.0	fEnglish	 year age harvest
U+6B74.0	fUTF8	歴
U+6B74.0	fCNS11643	3-4753
U+6B74.0	fGB	E-2F53
U+6B74.0	fR/S	 77.10
U+6B74.0	fTotalStrokes	 14
U+6B74.0	fCangjie	MDDM
U+6B74.0	fFourCorner	71210
U+6B74.0	fCantonese	lik6
U+6B74.0	fMandarin	li4
U+6B74.0	fEnglish	take place; past; history
U+6B75.0	fUTF8	歵
U+6B75.0	fCNS11643	4-4A4E
U+6B75.0	fGB	3-4678
U+6B75.0	fR/S	  77.11
U+6B75.0	fTotalStrokes	 15
U+6B75.0	fCangjie	YMQMC
U+6B75.0	fFourCorner	25180
U+6B75.0	fCantonese	zaak6
U+6B75.0	fMandarin	cuo4
U+6B76.0	fUTF8	歶
U+6B76.0	fCNS11643	2-5259
U+6B76.0	fBig5	E6D1
U+6B76.0	fGB	E-2F54
U+6B76.0	fR/S	 77.11; 114.9
U+6B76.0	fTotalStrokes	 15
U+6B76.0	fCangjie	YPWLB
U+6B76.0	fFourCorner	21227
U+6B76.0	fCantonese	jyu4
U+6B76.0	fMandarin	yu2
U+6B77.0	fUTF8	歷
U+6B77.0	fCNS11643	1-7125
U+6B77.0	fBig5	BEFA
U+6B77.0	fGB	1-407A
U+6B77.0	fR/S	 77.12
U+6B77.0	fTotalStrokes	 16
U+6B77.0	fCangjie	MHDM
U+6B77.0	fFourCorner	71211
U+6B77.0	fHakka	  lit8;  lak8; let8; liak8; liet8
U+6B77.0	fCantonese	lik6
U+6B77.0	fMandarin	li4
U+6B77.0	fEnglish	[1] take place; pass; elapse [2] undergo; go through; experience [3] things or duration that had to come to pass [4] all previous (occasions, sessions, etc.); what has taken place [5] through; throughout; successive [6] last (a certain period of time)
U+6B78.0	fUTF8	歸
U+6B78.0	fCNS11643	1-7657
U+6B78.0	fBig5	C26B
U+6B78.0	fGB	1-3969
U+6B78.0	fR/S	 77.14
U+6B78.0	fTotalStrokes	 18
U+6B78.0	fCangjie	HMSMB
U+6B78.0	fFourCorner	27127
U+6B78.0	fHakka	  gui1; gwi1; kwui1
U+6B78.0	fCantonese	gwai1; gwai6; kwai3; kwai5
U+6B78.0	fMandarin	gui1; kui4
U+6B78.0	fEnglish	[1] come back; return [2] return (something to its owner); return to [3] marry (said of a woman) [4] pledge allegiance to [5] belong to; attribute [6] turn over to; put in somebody's charge [7] a Chinese family name
U+6B79.0	fUTF8	歹
U+6B79.0	fCNS11643	1-4550
U+6B79.0	fBig5	A4EF
U+6B79.0	fGB	0-3475
U+6B79.0	fR/S	 78.0
U+6B79.0	fTotalStrokes	 4
U+6B79.0	fCangjie	MNI
U+6B79.0	fFourCorner	10207
U+6B79.0	fHakka	 dai3; fai6; fai5
U+6B79.0	fCantonese	daai2; aat3; ngaat6; at3
U+6B79.0	fMandarin	dai3; e4; dai1
U+6B79.0	fEnglish	bad; vicious; depraved; wicked; evil; crooked
U+6B79.1	fUTF8	歹
U+6B79.1	fCNS11643	1-4550
U+6B79.1	fBig5	A4EF
U+6B79.1	fGB	0-3475
U+6B79.1	fR/S	 78.0
U+6B79.1	fTotalStrokes	 4
U+6B79.1	fCangjie	MNI
U+6B79.1	fFourCorner	10207
U+6B79.1	fMandarin	e4dai3}
U+6B79.1	fEnglish	the remains of a person
U+6B7A.0	fUTF8	歺
U+6B7A.0	fCNS11643	3-2253
U+6B7A.0	fGB	E-2F55
U+6B7A.0	fR/S	 78.1
U+6B7A.0	fTotalStrokes	 5
U+6B7A.0	fCangjie	YNI
U+6B7A.0	fFourCorner	21200
U+6B7A.0	fHakka	  dai3
U+6B7A.0	fCantonese	aat3; ngaat3 (also); daai2
U+6B7A.0	fMandarin	can1; e4; dai3
U+6B7A.0	fEnglish	vicious; depraved; bad
U+6B7B.0	fUTF8	死
U+6B7B.0	fCNS11643	1-483B
U+6B7B.0	fBig5	A6BA
U+6B7B.0	fGB	0-4B40
U+6B7B.0	fR/S	 78.2
U+6B7B.0	fTotalStrokes	 6
U+6B7B.0	fCangjie	MNP
U+6B7B.0	fFourCorner	10212
U+6B7B.0	fHakka	  si3; shi1
U+6B7B.0	fCantonese	sei2; si2 (also)
U+6B7B.0	fMandarin	si3
U+6B7B.0	fEnglish	[1] die; die for; dead; death [2] used as an intensive or superlative; very; extremely [3] condemned (persons whose lives are numbered, as criminals on the death row) [4] inanimate; dull and stupid; persevering; resolute; resolutely [5] rigid; fixed or unchangeable (regulations, etc.); immovable (drawers, etc.) [6] impassable; closed
U+6B7C.0	fUTF8	歼
U+6B7C.0	fGB	0-3C5F
U+6B7C.0	fR/S	 78.3
U+6B7C.0	fTotalStrokes	 7
U+6B7C.0	fCangjie	XMNHJ
U+6B7C.0	fCantonese	cim1
U+6B7C.0	fMandarin	jian1
U+6B7C.0	fEnglish	simplified form of &#x6BB2;
U+6B7D.0	fUTF8	歽
U+6B7D.0	fCNS11643	4-2676
U+6B7D.0	fGB	E-2F56
U+6B7D.0	fR/S	 78.4
U+6B7D.0	fTotalStrokes	 8
U+6B7D.0	fCangjie	MNHML
U+6B7D.0	fFourCorner	12220
U+6B7D.0	fCantonese	zit3
U+6B7D.0	fMandarin	zhe2
U+6B7E.0	fUTF8	歾
U+6B7E.0	fCNS11643	2-274C
U+6B7E.0	fBig5	CCEB
U+6B7E.0	fGB	5-473B
U+6B7E.0	fR/S	 78.4
U+6B7E.0	fTotalStrokes	 8
U+6B7E.0	fCangjie	MNPHH
U+6B7E.0	fFourCorner	17220
U+6B7E.0	fHakka	  mut8
U+6B7E.0	fCantonese	mut6; man5; mut1
U+6B7E.0	fMandarin	mo4; wen3
U+6B7E.0	fEnglish	drown
U+6B7F.0	fUTF8	歿
U+6B7F.0	fCNS11643	1-4E3F
U+6B7F.0	fBig5	AA5C
U+6B7F.0	fGB	E-2F57
U+6B7F.0	fR/S	 78.4
U+6B7F.0	fTotalStrokes	 8
U+6B7F.0	fCangjie	MNNE
U+6B7F.0	fFourCorner	17247
U+6B7F.0	fHakka	 mut8; mut7
U+6B7F.0	fCantonese	mut6
U+6B7F.0	fMandarin	mo4
U+6B7F.0	fEnglish	 die; death; dead
U+6B80.0	fUTF8	殀
U+6B80.0	fCNS11643	2-274B
U+6B80.0	fBig5	CCEA
U+6B80.0	fGB	E-2F58
U+6B80.0	fR/S	 78.4
U+6B80.0	fTotalStrokes	 8
U+6B80.0	fCangjie	MNHK
U+6B80.0	fFourCorner	12234
U+6B80.0	fHakka	 jau1; rau1; jeu1; jau3
U+6B80.0	fCantonese	jiu2
U+6B80.0	fMandarin	yao3; yao1
U+6B80.0	fEnglish	[1] die young; die prematurely; die untimely [2] be wronged; be aggrieved
U+6B81.0	fUTF8	殁
U+6B81.0	fCNS11643	3-2958
U+6B81.0	fGB	0-6962
U+6B81.0	fR/S	 78.4
U+6B81.0	fTotalStrokes	 8
U+6B81.0	fCangjie	MNHNE
U+6B81.0	fFourCorner	17240
U+6B81.0	fHakka	 mut8; mut7
U+6B81.0	fCantonese	mut6
U+6B81.0	fMandarin	mei2; mo4; wen3
U+6B81.0	fEnglish	 die death dead
U+6B82.0	fUTF8	殂
U+6B82.0	fCNS11643	2-2B47
U+6B82.0	fBig5	CF65
U+6B82.0	fGB	0-6963
U+6B82.0	fR/S	 78.5
U+6B82.0	fTotalStrokes	 9
U+6B82.0	fCangjie	MNBM
U+6B82.0	fFourCorner	17210
U+6B82.0	fHakka	 zu1; cu2; zii1; zih1
U+6B82.0	fCantonese	cou4
U+6B82.0	fMandarin	cu2
U+6B82.0	fEnglish	 die; dead; death
U+6B83.0	fUTF8	殃
U+6B83.0	fCNS11643	1-5172
U+6B83.0	fBig5	AC6F
U+6B83.0	fGB	0-516A
U+6B83.0	fR/S	 78.5
U+6B83.0	fTotalStrokes	 9
U+6B83.0	fCangjie	MNLBK
U+6B83.0	fFourCorner	15230
U+6B83.0	fHakka	  jong1; rong1
U+6B83.0	fCantonese	joeng1
U+6B83.0	fMandarin	yang1
U+6B83.0	fEnglish	[1] misfortune; disaster; calamity [2] return of the spirit of the deceased
U+6B84.0	fUTF8	殄
U+6B84.0	fCNS11643	2-2B48
U+6B84.0	fBig5	CF66
U+6B84.0	fGB	0-6965
U+6B84.0	fR/S	 78.5
U+6B84.0	fTotalStrokes	 9
U+6B84.0	fCangjie	MNOHH
U+6B84.0	fFourCorner	18222
U+6B84.0	fHakka	 tien5; ten3; tien3
U+6B84.0	fCantonese	tin5; tim5 (also); can2
U+6B84.0	fMandarin	tian3
U+6B84.0	fEnglish	[1] end; terminate [2] exterminate; extirpate [3] waste
U+6B85.0	fUTF8	殅
U+6B85.0	fCNS11643	5-2969
U+6B85.0	fGB	5-473F
U+6B85.0	fR/S	  78.5
U+6B85.0	fTotalStrokes	 9
U+6B85.0	fCangjie	MNHQM
U+6B85.0	fFourCorner	15210
U+6B85.0	fCantonese	sang1
U+6B85.0	fMandarin	sheng1
U+6B86.0	fUTF8	殆
U+6B86.0	fCNS11643	1-5173
U+6B86.0	fBig5	AC70
U+6B86.0	fGB	0-3479
U+6B86.0	fR/S	 78.5
U+6B86.0	fTotalStrokes	 9
U+6B86.0	fCangjie	MNIR
U+6B86.0	fFourCorner	13260
U+6B86.0	fHakka	 tai5; tai6; toi3
U+6B86.0	fCantonese	toi5; doi6 (also)
U+6B86.0	fMandarin	dai4
U+6B86.0	fEnglish	[1] dangerous; perilous; endanger; danger; precarious [2] tired [3] afraid [4] nearly; almost
U+6B87.0	fUTF8	殇
U+6B87.0	fGB	0-6964
U+6B87.0	fR/S	 78.5
U+6B87.0	fTotalStrokes	 9
U+6B87.0	fCangjie	MNONH
U+6B87.0	fHakka	 shong1
U+6B87.0	fCantonese	soeng1
U+6B87.0	fMandarin	shang1
U+6B87.0	fEnglish	simplified form of &#x6BA4;
U+6B88.0	fUTF8	殈
U+6B88.0	fCNS11643	2-2F7E
U+6B88.0	fBig5	D1FC
U+6B88.0	fGB	3-4476
U+6B88.0	fR/S	 78.6
U+6B88.0	fTotalStrokes	 10
U+6B88.0	fCangjie	MNHBT
U+6B88.0	fFourCorner	17210
U+6B88.0	fCantonese	gwik1
U+6B88.0	fMandarin	xu4; yi4
U+6B88.0	fEnglish	infertile
U+6B89.0	fUTF8	殉
U+6B89.0	fCNS11643	1-5632
U+6B89.0	fBig5	AEEE
U+6B89.0	fGB	0-5133
U+6B89.0	fR/S	 78.6
U+6B89.0	fTotalStrokes	 10
U+6B89.0	fCangjie	MNPA
U+6B89.0	fFourCorner	17220
U+6B89.0	fHakka	 sun2; sun5
U+6B89.0	fCantonese	seon1; seon6
U+6B89.0	fMandarin	xun4; xun2; yuan4
U+6B89.0	fEnglish	[1] die for a cause; be martyr for [2] be buried with the dead (usually said of slaves, loyal servants, concubines, etc.)
U+6B8A.0	fUTF8	殊
U+6B8A.0	fCNS11643	1-5631
U+6B8A.0	fBig5	AEED
U+6B8A.0	fGB	0-4A62
U+6B8A.0	fR/S	 78.6
U+6B8A.0	fTotalStrokes	 10
U+6B8A.0	fCangjie	MNHJD
U+6B8A.0	fFourCorner	15290
U+6B8A.0	fHakka	 su2;  shu2; shu3; chu2
U+6B8A.0	fCantonese	syu4
U+6B8A.0	fMandarin	shu1
U+6B8A.0	fEnglish	[1] different; special; unusual; strange [2] distinguished; outstanding [3] extremely; very [4] really; indeed [5] still; yet [6] exceed; over
U+6B8B.0	fUTF8	残
U+6B8B.0	fCNS11643	3-316A
U+6B8B.0	fGB	0-3250
U+6B8B.0	fR/S	 78.5; 78.6
U+6B8B.0	fTotalStrokes	 10
U+6B8B.0	fCangjie	MNIJ
U+6B8B.0	fFourCorner	13250
U+6B8B.0	fHakka	 can2
U+6B8B.0	fCantonese	caan4; caan1
U+6B8B.0	fMandarin	can2
U+6B8B.0	fEnglish	simplified form of &#x6B98;
U+6B8C.0	fUTF8	殌
U+6B8C.0	fCNS11643	2-3642
U+6B8C.0	fBig5	D5DE
U+6B8C.0	fGB	E-2F59
U+6B8C.0	fR/S	 78.7
U+6B8C.0	fTotalStrokes	 11
U+6B8C.0	fCangjie	MNMVM
U+6B8C.0	fFourCorner	11211
U+6B8C.0	fCantonese	kyut3
U+6B8C.0	fMandarin	jue2; qing2
U+6B8D.0	fUTF8	殍
U+6B8D.0	fCNS11643	2-3640
U+6B8D.0	fBig5	D5DC
U+6B8D.0	fGB	0-6968
U+6B8D.0	fR/S	 78.7
U+6B8D.0	fTotalStrokes	 11
U+6B8D.0	fCangjie	MNBND
U+6B8D.0	fFourCorner	12247
U+6B8D.0	fHakka	 piau3; fu1; peu3
U+6B8D.0	fCantonese	piu5; fu1 (also)
U+6B8D.0	fMandarin	piao3; fu1; bi4; pi3
U+6B8D.0	fEnglish	[1] starve to death [2] a person who died from starvation
U+6B8E.0	fUTF8	殎
U+6B8E.0	fCNS11643	2-3641
U+6B8E.0	fBig5	D5DD
U+6B8E.0	fGB	E-2F5A
U+6B8E.0	fR/S	 78.7
U+6B8E.0	fTotalStrokes	 11
U+6B8E.0	fCangjie	MNKOO
U+6B8E.0	fFourCorner	14288
U+6B8E.0	fCantonese	haap3; hap1
U+6B8E.0	fMandarin	qia4
U+6B8F.0	fUTF8	殏
U+6B8F.0	fCNS11643	2-363F
U+6B8F.0	fBig5	D5DB
U+6B8F.0	fGB	E-2F5B
U+6B8F.0	fR/S	 78.7
U+6B8F.0	fTotalStrokes	 11
U+6B8F.0	fCangjie	MNIJE
U+6B8F.0	fFourCorner	13232
U+6B8F.0	fCantonese	kau4
U+6B8F.0	fMandarin	qiu2; qiu4
U+6B90.0	fUTF8	殐
U+6B90.0	fCNS11643	3-3664
U+6B90.0	fGB	3-4479
U+6B90.0	fR/S	  78.7
U+6B90.0	fTotalStrokes	 11
U+6B90.0	fCangjie	MNDL
U+6B90.0	fFourCorner	14290
U+6B90.0	fCantonese	suk1
U+6B90.0	fMandarin	su4; shuo4
U+6B91.0	fUTF8	殑
U+6B91.0	fCNS11643	2-363E
U+6B91.0	fBig5	D5DA
U+6B91.0	fGB	3-4478
U+6B91.0	fR/S	 78.7
U+6B91.0	fTotalStrokes	 11
U+6B91.0	fCangjie	MNJRU
U+6B91.0	fFourCorner	14216
U+6B91.0	fCantonese	king4; ging6; cing4
U+6B91.0	fMandarin	qing2; jing4; jing1
U+6B91.0	fEnglish	swoon
U+6B92.0	fUTF8	殒
U+6B92.0	fGB	0-6966
U+6B92.0	fR/S	 78.7
U+6B92.0	fTotalStrokes	 11
U+6B92.0	fCangjie	MNRBO
U+6B92.0	fHakka	 jun3
U+6B92.0	fCantonese	wan5
U+6B92.0	fMandarin	yun3
U+6B92.0	fEnglish	simplified form of &#x6B9E;
U+6B93.0	fUTF8	殓
U+6B93.0	fGB	0-6967
U+6B93.0	fR/S	 78.7
U+6B93.0	fTotalStrokes	 11
U+6B93.0	fCangjie	MNOMM
U+6B93.0	fHakka	 liam5
U+6B93.0	fCantonese	lim5; lim6
U+6B93.0	fMandarin	lian4
U+6B93.0	fEnglish	simplified form of &#x6BAE;
U+6B94.0	fUTF8	殔
U+6B94.0	fCNS11643	2-3D23
U+6B94.0	fBig5	D9DE
U+6B94.0	fGB	3-4526
U+6B94.0	fR/S	 78.8
U+6B94.0	fTotalStrokes	 12
U+6B94.0	fCangjie	MNLE
U+6B94.0	fFourCorner	15232
U+6B94.0	fHakka	 ji5; ri6; i5
U+6B94.0	fCantonese	ji6; sei3
U+6B94.0	fMandarin	yi4
U+6B94.0	fEnglish	 dig a grave
U+6B95.0	fUTF8	殕
U+6B95.0	fCNS11643	2-3D26
U+6B95.0	fBig5	D9E1
U+6B95.0	fGB	3-4524
U+6B95.0	fR/S	 78.8
U+6B95.0	fTotalStrokes	 12
U+6B95.0	fCangjie	MNYTR
U+6B95.0	fFourCorner	10261
U+6B95.0	fCantonese	fau2; baak6; ak1; fu2
U+6B95.0	fMandarin	fou3; fu4; e4; bo2
U+6B96.0	fUTF8	殖
U+6B96.0	fCNS11643	1-6025
U+6B96.0	fBig5	B4DE
U+6B96.0	fGB	0-5633
U+6B96.0	fR/S	 78.8
U+6B96.0	fTotalStrokes	 12
U+6B96.0	fCangjie	MNJBM
U+6B96.0	fFourCorner	14216
U+6B96.0	fHakka	 chit8; cit8; ts'iit8
U+6B96.0	fCantonese	zik6
U+6B96.0	fMandarin	zhi2; shi1; shi4
U+6B96.0	fEnglish	[1] grow in abundance; prosper; reproduce; propagate [2] plant [3] become wealthy [4] colonize; colonization
U+6B97.0	fUTF8	殗
U+6B97.0	fCNS11643	2-3D24
U+6B97.0	fBig5	D9DF
U+6B97.0	fGB	3-4522
U+6B97.0	fR/S	 78.8
U+6B97.0	fTotalStrokes	 12
U+6B97.0	fCangjie	MNKLU
U+6B97.0	fFourCorner	14216
U+6B97.0	fHakka	 ngiem5
U+6B97.0	fCantonese	jip3
U+6B97.0	fMandarin	ye4; yan1; yan4
U+6B97.0	fEnglish	 sickness repeated
U+6B98.0	fUTF8	殘
U+6B98.0	fCNS11643	1-6024
U+6B98.0	fBig5	B4DD
U+6B98.0	fGB	1-3250
U+6B98.0	fR/S	 78.8
U+6B98.0	fTotalStrokes	 12
U+6B98.0	fCangjie	MNII
U+6B98.0	fFourCorner	13253
U+6B98.0	fHakka	  can2; ts'ian3
U+6B98.0	fCantonese	caan4; caan1
U+6B98.0	fMandarin	can2
U+6B98.0	fEnglish	[1] destroy; injure; damage; spoil [2] wither [3] crippled; disfigured [4] cruel and fierce; heartless and relentless [5] remnants; residues; the little amount of something left [6] incomplete [7] kill
U+6B99.0	fUTF8	殙
U+6B99.0	fCNS11643	2-3D25
U+6B99.0	fBig5	D9E0
U+6B99.0	fGB	3-4523
U+6B99.0	fR/S	 78.8
U+6B99.0	fTotalStrokes	 12
U+6B99.0	fCangjie	MNHPA
U+6B99.0	fFourCorner	12264
U+6B99.0	fHakka	  fun1
U+6B99.0	fCantonese	fan1
U+6B99.0	fMandarin	hun1; men4
U+6B99.0	fEnglish	 die by taking poison
U+6B9A.0	fUTF8	殚
U+6B9A.0	fGB	0-6969
U+6B9A.0	fR/S	 78.8
U+6B9A.0	fTotalStrokes	 12
U+6B9A.0	fCangjie	MNCWJ
U+6B9A.0	fHakka	 dan1
U+6B9A.0	fCantonese	daan1
U+6B9A.0	fMandarin	dan1
U+6B9A.0	fEnglish	simplified form of &#x6BAB;
U+6B9B.0	fUTF8	殛
U+6B9B.0	fCNS11643	2-4423
U+6B9B.0	fBig5	DDFB
U+6B9B.0	fGB	0-696A
U+6B9B.0	fR/S	 78.9
U+6B9B.0	fTotalStrokes	 13
U+6B9B.0	fCangjie	MNMEM
U+6B9B.0	fFourCorner	17214
U+6B9B.0	fHakka	  git7
U+6B9B.0	fCantonese	gik1
U+6B9B.0	fMandarin	ji2
U+6B9B.0	fEnglish	 put to death
U+6B9C.0	fUTF8	殜
U+6B9C.0	fCNS11643	3-4222
U+6B9C.0	fGB	3-4528
U+6B9C.0	fR/S	  78.9
U+6B9C.0	fTotalStrokes	 13
U+6B9C.0	fCangjie	MNPTD
U+6B9C.0	fFourCorner	14290
U+6B9C.0	fCantonese	jip6; dip6 (also)
U+6B9C.0	fMandarin	ye4; die2
U+6B9C.0	fEnglish	half-sitting; half-reclining
U+6B9D.0	fUTF8	殝
U+6B9D.0	fCNS11643	4-4431
U+6B9D.0	fGB	3-452D
U+6B9D.0	fR/S	 78.10
U+6B9D.0	fTotalStrokes	 14
U+6B9D.0	fCangjie	MNQKD
U+6B9D.0	fFourCorner	15294
U+6B9D.0	fCantonese	zeon1
U+6B9D.0	fMandarin	zhen1
U+6B9E.0	fUTF8	殞
U+6B9E.0	fCNS11643	2-4B2E
U+6B9E.0	fBig5	E266
U+6B9E.0	fGB	1-6966
U+6B9E.0	fR/S	 78.10
U+6B9E.0	fTotalStrokes	 14
U+6B9E.0	fCangjie	MNRBC
U+6B9E.0	fFourCorner	16286
U+6B9E.0	fHakka	 jun3; run5; jun5
U+6B9E.0	fCantonese	wan5
U+6B9E.0	fMandarin	yun3
U+6B9E.0	fEnglish	[1] die; perish; vanish [2] same as $ ( fall)
U+6B9F.0	fUTF8	殟
U+6B9F.0	fCNS11643	2-4B2F
U+6B9F.0	fBig5	E267
U+6B9F.0	fGB	3-452A
U+6B9F.0	fR/S	 78.10
U+6B9F.0	fTotalStrokes	 14
U+6B9F.0	fCangjie	MNWOT
U+6B9F.0	fFourCorner	16217
U+6B9F.0	fHakka	 wut7
U+6B9F.0	fCantonese	wan1
U+6B9F.0	fMandarin	wen1; wa4
U+6BA0.0	fUTF8	殠
U+6BA0.0	fCNS11643	2-4B30
U+6BA0.0	fBig5	E268
U+6BA0.0	fGB	5-4756
U+6BA0.0	fR/S	 78.10
U+6BA0.0	fTotalStrokes	 14
U+6BA0.0	fCangjie	MNHUK
U+6BA0.0	fFourCorner	16234
U+6BA0.0	fCantonese	cau3
U+6BA0.0	fMandarin	chou4; xiu4; xiu3
U+6BA1.0	fUTF8	殡
U+6BA1.0	fCNS11643	3-4754
U+6BA1.0	fGB	0-696B
U+6BA1.0	fR/S	 78.10
U+6BA1.0	fTotalStrokes	 14
U+6BA1.0	fCangjie	MNJOC
U+6BA1.0	fFourCorner	13280
U+6BA1.0	fHakka	 bin5
U+6BA1.0	fCantonese	ban3
U+6BA1.0	fMandarin	bin4
U+6BA1.0	fEnglish	simplified form of &#x6BAF;
U+6BA2.0	fUTF8	殢
U+6BA2.0	fCNS11643	2-513C
U+6BA2.0	fBig5	E5F3
U+6BA2.0	fGB	3-4529
U+6BA2.0	fR/S	 78.11
U+6BA2.0	fTotalStrokes	 15
U+6BA2.0	fCangjie	MNKPB
U+6BA2.0	fFourCorner	14227
U+6BA2.0	fHakka	 tiam3
U+6BA2.0	fCantonese	tai3; taai3
U+6BA2.0	fMandarin	ti4; di4; ni4; xi4
U+6BA2.0	fEnglish	greatly distressed
U+6BA3.0	fUTF8	殣
U+6BA3.0	fCNS11643	2-513B
U+6BA3.0	fBig5	E5F2
U+6BA3.0	fGB	3-452F
U+6BA3.0	fR/S	 78.11
U+6BA3.0	fTotalStrokes	 15
U+6BA3.0	fCangjie	MNTLM
U+6BA3.0	fFourCorner	14214
U+6BA3.0	fHakka	  giun3
U+6BA3.0	fCantonese	gan2; gan6
U+6BA3.0	fMandarin	jin3; jin4
U+6BA3.0	fEnglish	[1] starve to death [2] bury
U+6BA4.0	fUTF8	殤
U+6BA4.0	fCNS11643	1-6D46
U+6BA4.0	fBig5	BCDC
U+6BA4.0	fGB	1-6964
U+6BA4.0	fR/S	 78.11
U+6BA4.0	fTotalStrokes	 15
U+6BA4.0	fCangjie	MNOAH
U+6BA4.0	fFourCorner	18227
U+6BA4.0	fHakka	 song1;  shong1
U+6BA4.0	fCantonese	soeng1
U+6BA4.0	fMandarin	shang1
U+6BA4.0	fEnglish	[1] die young; die prematurely [2] national mourning
U+6BA5.0	fUTF8	殥
U+6BA5.0	fCNS11643	2-513A
U+6BA5.0	fBig5	E5F1
U+6BA5.0	fGB	3-4530
U+6BA5.0	fR/S	 78.11
U+6BA5.0	fTotalStrokes	 15
U+6BA5.0	fCangjie	MNJMC
U+6BA5.0	fFourCorner	13286
U+6BA5.0	fCantonese	jan4
U+6BA5.0	fMandarin	yin2
U+6BA6.0	fUTF8	殦
U+6BA6.0	fCNS11643	2-513D
U+6BA6.0	fBig5	E5F4
U+6BA6.0	fGB	E-2F5C
U+6BA6.0	fR/S	 78.11
U+6BA6.0	fTotalStrokes	 15
U+6BA6.0	fCangjie	MNHAF
U+6BA6.0	fFourCorner	17227
U+6BA6.0	fCantonese	diu1
U+6BA6.0	fMandarin	diao1
U+6BA7.0	fUTF8	殧
U+6BA7.0	fCNS11643	2-576C
U+6BA7.0	fBig5	E9E1
U+6BA7.0	fGB	3-4532
U+6BA7.0	fR/S	 78.12
U+6BA7.0	fTotalStrokes	 16
U+6BA7.0	fCangjie	MNYFU
U+6BA7.0	fFourCorner	13214
U+6BA7.0	fCantonese	zau6; zuk1
U+6BA7.0	fMandarin	cu4; za1; jiu4
U+6BA8.0	fUTF8	殨
U+6BA8.0	fCNS11643	4-506D
U+6BA8.0	fGB	5-474E
U+6BA8.0	fR/S	 78.12
U+6BA8.0	fTotalStrokes	 16
U+6BA8.0	fCangjie	MNLMC
U+6BA8.0	fFourCorner	15280
U+6BA8.0	fHakka	 kui3; kui6; kui5; kwi5; k'wui5
U+6BA8.0	fCantonese	kui2; kwui2 (also)
U+6BA8.0	fMandarin	hui4; hu2
U+6BA8.0	fEnglish	 open as an ulcer or sore
U+6BA9.0	fUTF8	殩
U+6BA9.0	fCNS11643	5-5B72
U+6BA9.0	fGB	E-2F5D
U+6BA9.0	fR/S	  78.12
U+6BA9.0	fTotalStrokes	 17; 16
U+6BA9.0	fCangjie	MNYED
U+6BA9.0	fFourCorner	17290
U+6BA9.0	fCantonese	cyun3
U+6BA9.0	fMandarin	cuan4
U+6BAA.0	fUTF8	殪
U+6BAA.0	fCNS11643	2-576D
U+6BAA.0	fBig5	E9E2
U+6BAA.0	fGB	0-696C
U+6BAA.0	fR/S	 78.12
U+6BAA.0	fTotalStrokes	 16
U+6BAA.0	fCangjie	MNGBT
U+6BAA.0	fFourCorner	14218
U+6BAA.0	fHakka	 ji5; ri5; rit7; jit7; i5
U+6BAA.0	fCantonese	ji3; ai3; ngai3
U+6BAA.0	fMandarin	yi4
U+6BAA.0	fEnglish	[1] die [2] kill; exterminate
U+6BAB.0	fUTF8	殫
U+6BAB.0	fCNS11643	2-576E
U+6BAB.0	fBig5	E9E3
U+6BAB.0	fGB	1-6969
U+6BAB.0	fR/S	 78.12
U+6BAB.0	fTotalStrokes	 16
U+6BAB.0	fCangjie	MNRRJ
U+6BAB.0	fFourCorner	16256
U+6BAB.0	fHakka	 dan1
U+6BAB.0	fCantonese	daan1
U+6BAB.0	fMandarin	dan1
U+6BAB.0	fEnglish	 use up; exhaust
U+6BAC.0	fUTF8	殬
U+6BAC.0	fCNS11643	4-5652
U+6BAC.0	fGB	E-2F5E
U+6BAC.0	fR/S	  78.13
U+6BAC.0	fTotalStrokes	 17; 16
U+6BAC.0	fCangjie	MNWLJ
U+6BAC.0	fFourCorner	16240
U+6BAC.0	fHakka	 du5
U+6BAC.0	fCantonese	dou3
U+6BAC.0	fMandarin	du4
U+6BAD.0	fUTF8	殭
U+6BAD.0	fCNS11643	2-5D38
U+6BAD.0	fBig5	ED4C
U+6BAD.0	fGB	E-2F5F
U+6BAD.0	fR/S	 78.13
U+6BAD.0	fTotalStrokes	 17
U+6BAD.0	fCangjie	MNMWM
U+6BAD.0	fFourCorner	11216
U+6BAD.0	fHakka	  giong1
U+6BAD.0	fCantonese	goeng1
U+6BAD.0	fMandarin	jiang1; jiang4
U+6BAD.0	fEnglish	 stiff corpse; dead and stiff
U+6BAE.0	fUTF8	殮
U+6BAE.0	fCNS11643	1-737D
U+6BAE.0	fBig5	C0D4
U+6BAE.0	fGB	1-6967
U+6BAE.0	fR/S	 78.13
U+6BAE.0	fTotalStrokes	 17
U+6BAE.0	fCangjie	MNOMO
U+6BAE.0	fFourCorner	18286
U+6BAE.0	fHakka	  liam5; liam3; liam6
U+6BAE.0	fCantonese	lim6 (also); lim5
U+6BAE.0	fMandarin	lian4
U+6BAE.0	fEnglish	 dress a corpse for burial; prepare a body for the coffin; encoffin
U+6BAF.0	fUTF8	殯
U+6BAF.0	fCNS11643	1-7658
U+6BAF.0	fBig5	C26C
U+6BAF.0	fGB	1-696B
U+6BAF.0	fR/S	 78.14
U+6BAF.0	fTotalStrokes	 18
U+6BAF.0	fCangjie	MNJMC
U+6BAF.0	fFourCorner	13286
U+6BAF.0	fHakka	  bin5; bin1
U+6BAF.0	fCantonese	ban3
U+6BAF.0	fMandarin	bin4
U+6BAF.0	fEnglish	[1] lay a coffin n a memorial hall [2] carry to the grave
U+6BB0.0	fUTF8	殰
U+6BB0.0	fCNS11643	2-6565
U+6BB0.0	fBig5	F25A
U+6BB0.0	fGB	3-447E
U+6BB0.0	fR/S	 78.15
U+6BB0.0	fTotalStrokes	 19
U+6BB0.0	fCangjie	MNGWC
U+6BB0.0	fFourCorner	14286
U+6BB0.0	fHakka	  tuk8
U+6BB0.0	fCantonese	duk6
U+6BB0.0	fMandarin	du2
U+6BB0.0	fEnglish	 abortion stillborn
U+6BB1.0	fUTF8	殱
U+6BB1.0	fGB	E-2F60
U+6BB1.0	fR/S	 78.15
U+6BB1.0	fTotalStrokes	 19
U+6BB1.0	fCangjie	MNJIM
U+6BB1.0	fHakka	 ciam2;  siam2
U+6BB1.0	fCantonese	cim1
U+6BB1.0	fMandarin	jian1
U+6BB1.0	fEnglish	 annihilate, wipe out, kill off
U+6BB2.0	fUTF8	殲
U+6BB2.0	fCNS11643	1-7A73
U+6BB2.0	fBig5	C4E8
U+6BB2.0	fGB	1-3C5F
U+6BB2.0	fR/S	 78.17
U+6BB2.0	fTotalStrokes	 21
U+6BB2.0	fCangjie	MNOIM
U+6BB2.0	fFourCorner	13250
U+6BB2.0	fHakka	 ciam1; siam2
U+6BB2.0	fCantonese	cim1
U+6BB2.0	fMandarin	jian1
U+6BB2.0	fEnglish	 annihilate; wipe out; kill off; exterminate; destroy
U+6BB3.0	fUTF8	殳
U+6BB3.0	fCNS11643	2-213F
U+6BB3.0	fBig5	C95F
U+6BB3.0	fGB	0-6C2F
U+6BB3.0	fR/S	 79.0
U+6BB3.0	fTotalStrokes	 4
U+6BB3.0	fCangjie	HNE
U+6BB3.0	fFourCorner	77407
U+6BB3.0	fHakka	 su2;  shu2
U+6BB3.0	fCantonese	syu4
U+6BB3.0	fMandarin	shu1
U+6BB3.0	fEnglish	[1] halberd; an old weapon made of bamboo [2] Kangxi radical 79
U+6BB4.0	fUTF8	殴
U+6BB4.0	fCNS11643	3-2959
U+6BB4.0	fGB	0-4539
U+6BB4.0	fR/S	 79.4
U+6BB4.0	fTotalStrokes	 8
U+6BB4.0	fCangjie	SKHNE
U+6BB4.0	fFourCorner	77740
U+6BB4.0	fHakka	 eu3
U+6BB4.0	fCantonese	au2; ngau2 (also); au1 (also); ngau1 (also); keoi1
U+6BB4.0	fMandarin	ou1; ou3; kou1; qu1
U+6BB4.0	fEnglish	simplified form of &#x6BC6;
U+6BB5.0	fUTF8	段
U+6BB5.0	fCNS11643	1-5174
U+6BB5.0	fBig5	AC71
U+6BB5.0	fGB	0-364E
U+6BB5.0	fR/S	 79.5
U+6BB5.0	fTotalStrokes	 9
U+6BB5.0	fCangjie	HJHNE
U+6BB5.0	fFourCorner	77447
U+6BB5.0	fHakka	 ton3; {don5}; ton5; don5; ton6
U+6BB5.0	fCantonese	dyun6
U+6BB5.0	fMandarin	duan4
U+6BB5.0	fEnglish	[1] section; piece; division; part; paragraph [2] stage [3] a Chinese family name
U+6BB6.0	fUTF8	殶
U+6BB6.0	fCNS11643	2-2B49
U+6BB6.0	fBig5	CF67
U+6BB6.0	fGB	3-4F29
U+6BB6.0	fR/S	 79.5
U+6BB6.0	fTotalStrokes	 9
U+6BB6.0	fCangjie	YGHNE
U+6BB6.0	fFourCorner	07147
U+6BB6.0	fCantonese	zyu3
U+6BB6.0	fMandarin	zhu4
U+6BB7.0	fUTF8	殷
U+6BB7.0	fCNS11643	1-5633
U+6BB7.0	fBig5	AEEF
U+6BB7.0	fGB	0-5273
U+6BB7.0	fR/S	 79.6
U+6BB7.0	fTotalStrokes	 10
U+6BB7.0	fCangjie	HSHNE
U+6BB7.0	fFourCorner	27247
U+6BB7.0	fHakka	  jin1; rin1; ren1; jen1
U+6BB7.0	fCantonese	jan1; jan2; jin1; aan1
U+6BB7.0	fMandarin	yin1; yin3; yin4
U+6BB7.0	fEnglish	abundant; many; flourishing; thriving; prosperous; rich
U+6BB7.1	fUTF8	殷
U+6BB7.1	fCNS11643	1-5633
U+6BB7.1	fBig5	AEEF
U+6BB7.1	fGB	0-5273
U+6BB7.1	fR/S	 79.6
U+6BB7.1	fTotalStrokes	 10
U+6BB7.1	fCangjie	HSHNE
U+6BB7.1	fFourCorner	27247
U+6BB7.1	fMandarin	yan1
U+6BB7.1	fEnglish	dark red
U+6BB8.0	fUTF8	殸
U+6BB8.0	fCNS11643	4-324E
U+6BB8.0	fGB	5-5065
U+6BB8.0	fR/S	 79.7
U+6BB8.0	fTotalStrokes	 11
U+6BB8.0	fCangjie	GHHNE
U+6BB8.0	fFourCorner	47240
U+6BB8.0	fCantonese	hing3
U+6BB8.0	fMandarin	qing4
U+6BB9.0	fUTF8	殹
U+6BB9.0	fCNS11643	4-3250
U+6BB9.0	fGB	3-4F2B
U+6BB9.0	fR/S	  79.7
U+6BB9.0	fTotalStrokes	 11
U+6BB9.0	fCangjie	XSKHN
U+6BB9.0	fFourCorner	77740
U+6BB9.0	fCantonese	ai3; ngai3 (also); ji1
U+6BB9.0	fMandarin	yi4; yi1
U+6BB9.0	fEnglish	an echo
U+6BBA.0	fUTF8	殺
U+6BBA.0	fCNS11643	1-5B44
U+6BBA.0	fBig5	B1FE
U+6BBA.0	fGB	1-4931
U+6BBA.0	fR/S	 79.7; 79.6
U+6BBA.0	fTotalStrokes	 11
U+6BBA.0	fCangjie	KCHNE
U+6BBA.0	fFourCorner	47947
U+6BBA.0	fHakka	  sat7;  sai5; sat8
U+6BBA.0	fCantonese	saat3; saai3
U+6BBA.0	fMandarin	sha1; sa4; xue1; shi4; she4
U+6BBA.0	fEnglish	[1] kill; slaughter; murder; put to death [2] exceedingly; extremely [3] weaken; deflate [4] fight
U+6BBA.1	fUTF8	殺
U+6BBA.1	fCNS11643	1-5B44
U+6BBA.1	fBig5	B1FE
U+6BBA.1	fGB	1-4931
U+6BBA.1	fR/S	 79.7; 79.6
U+6BBA.1	fTotalStrokes	 11
U+6BBA.1	fCangjie	KCHNE
U+6BBA.1	fFourCorner	47947
U+6BBA.1	fMandarin	shai4
U+6BBA.1	fEnglish	 degrade; decline; diminish; abate
U+6BBB.0	fUTF8	殻
U+6BBB.0	fCNS11643	3-3665
U+6BBB.0	fGB	1-3F47
U+6BBB.0	fR/S	 79.7
U+6BBB.0	fTotalStrokes	 11
U+6BBB.0	fCangjie	GNHNE
U+6BBB.0	fFourCorner	47240
U+6BBB.0	fHakka	 kok7
U+6BBB.0	fCantonese	hok3
U+6BBB.0	fMandarin	ke2; qiao4; que4
U+6BBB.0	fEnglish	casing; shell; husk; hull; skin
U+6BBC.0	fUTF8	殼
U+6BBC.0	fCNS11643	1-6026
U+6BBC.0	fBig5	B4DF
U+6BBC.0	fGB	5-5068
U+6BBC.0	fR/S	 79.8
U+6BBC.0	fTotalStrokes	 12
U+6BBC.0	fCangjie	GNHNE
U+6BBC.0	fFourCorner	47247
U+6BBC.0	fHakka	 hok7;  kok7
U+6BBC.0	fCantonese	hok3; hok2 (also)
U+6BBC.0	fMandarin	ke2; que4; ku1
U+6BBC.0	fEnglish	 casing; shell; husk; hull; skin; coverings
U+6BBC.1	fUTF8	殼
U+6BBC.1	fCNS11643	1-6026
U+6BBC.1	fBig5	B4DF
U+6BBC.1	fGB	5-5068
U+6BBC.1	fR/S	 79.8
U+6BBC.1	fTotalStrokes	 12
U+6BBC.1	fCangjie	GNHNE
U+6BBC.1	fFourCorner	47247
U+6BBC.1	fMandarin	qiao4
U+6BBD.0	fUTF8	殽
U+6BBD.0	fCNS11643	2-3D27
U+6BBD.0	fBig5	D9E2
U+6BBD.0	fGB	E-2F61
U+6BBD.0	fR/S	 79.8
U+6BBD.0	fTotalStrokes	 12
U+6BBD.0	fCangjie	KBHNE
U+6BBD.0	fFourCorner	47247
U+6BBD.0	fHakka	 ngau2; ngau3
U+6BBD.0	fCantonese	ngaau4; aau4 (also); haau6
U+6BBD.0	fMandarin	yao2; xiao4; xiao2; hao2
U+6BBD.0	fEnglish	[1] confusion; disorder; mess [2] dishes
U+6BBE.0	fUTF8	殾
U+6BBE.0	fCNS11643	3-3C4B
U+6BBE.0	fGB	E-2F62
U+6BBE.0	fR/S	 79.9; 79.8
U+6BBE.0	fTotalStrokes	 13; 12
U+6BBE.0	fCangjie	JAHNE
U+6BBE.0	fFourCorner	47640
U+6BBE.0	fCantonese	seon3
U+6BBE.0	fMandarin	jun4
U+6BBF.0	fUTF8	殿
U+6BBF.0	fCNS11643	1-645B
U+6BBF.0	fBig5	B7B5
U+6BBF.0	fGB	0-356E
U+6BBF.0	fR/S	 79.9
U+6BBF.0	fTotalStrokes	 13
U+6BBF.0	fCangjie	SCHNE
U+6BBF.0	fFourCorner	77247
U+6BBF.0	fHakka	  tien5; ten3; tien6; t'ian6
U+6BBF.0	fCantonese	din6; din3
U+6BBF.0	fMandarin	dian4
U+6BBF.0	fEnglish	[1] palace; palace hall; temple; sanctuary [2] the rear; the rear guard
U+6BC0.0	fUTF8	毀
U+6BC0.0	fCNS11643	1-645A
U+6BC0.0	fBig5	B7B4
U+6BC0.0	fGB	E-2F63
U+6BC0.0	fR/S	 79.9
U+6BC0.0	fTotalStrokes	 13
U+6BC0.0	fCangjie	HGHNE
U+6BC0.0	fFourCorner	77147
U+6BC0.0	fHakka	 kui3; fui5; fui3; kwi3; fui1
U+6BC0.0	fCantonese	wai2
U+6BC0.0	fMandarin	hui3; hui4
U+6BC0.0	fEnglish	[1] destroy; ruin; injure; damage [2] libel; slander; abuse; revile; defame; blame excessively
U+6BC1.0	fUTF8	毁
U+6BC1.0	fCNS11643	3-4226
U+6BC1.0	fGB	0-3B59
U+6BC1.0	fR/S	 79.9
U+6BC1.0	fTotalStrokes	 13
U+6BC1.0	fCangjie	HMHNE
U+6BC1.0	fFourCorner	77140
U+6BC1.0	fHakka	 kui3; fui5; fi5; fui3; fi3; k'wui3
U+6BC1.0	fCantonese	wai2
U+6BC1.0	fMandarin	hui3
U+6BC1.0	fEnglish	destroy
U+6BC2.0	fUTF8	毂
U+6BC2.0	fGB	0-6C31
U+6BC2.0	fR/S	 79.9
U+6BC2.0	fTotalStrokes	 13
U+6BC2.0	fCangjie	GQHNE
U+6BC2.0	fHakka	 kwuk7
U+6BC2.0	fCantonese	guk1; gu1; gwu1 (also)
U+6BC2.0	fMandarin	gu3
U+6BC2.0	fEnglish	simplified form of &#x8F42;
U+6BC3.0	fUTF8	毃
U+6BC3.0	fCNS11643	2-4B31
U+6BC3.0	fBig5	E269
U+6BC3.0	fGB	5-7968
U+6BC3.0	fR/S	 79.10
U+6BC3.0	fTotalStrokes	 14
U+6BC3.0	fCangjie	YBHNE
U+6BC3.0	fFourCorner	07247
U+6BC3.0	fHakka	 kok7
U+6BC3.0	fCantonese	kok3; haau1; hok3
U+6BC3.0	fMandarin	que4; qiao1; qiao4
U+6BC3.0	fEnglish	[1] cross [2] ass a thing across
U+6BC4.0	fUTF8	毄
U+6BC4.0	fCNS11643	2-4B32
U+6BC4.0	fBig5	E26A
U+6BC4.0	fGB	E-2F64
U+6BC4.0	fR/S	 79.10
U+6BC4.0	fTotalStrokes	 14
U+6BC4.0	fCangjie	JRHNE
U+6BC4.0	fFourCorner	57647
U+6BC4.0	fHakka	  git7
U+6BC4.0	fCantonese	gik1; hek3
U+6BC4.0	fMandarin	ji1; ji4
U+6BC4.0	fEnglish	[1] jostle and hit each other [2] rub; brush
U+6BC5.0	fUTF8	毅
U+6BC5.0	fCNS11643	1-6D47
U+6BC5.0	fBig5	BCDD
U+6BC5.0	fGB	0-5263
U+6BC5.0	fR/S	 79.11
U+6BC5.0	fTotalStrokes	 15
U+6BC5.0	fCangjie	YOHNE
U+6BC5.0	fFourCorner	07247
U+6BC5.0	fHakka	 ngi5; ngi6; ji3; ngi3
U+6BC5.0	fCantonese	ngai6; ai6 (also)
U+6BC5.0	fMandarin	yi4
U+6BC5.0	fEnglish	resolute; decisive; firm; endurance; fortitude
U+6BC6.0	fUTF8	毆
U+6BC6.0	fCNS11643	1-6D48
U+6BC6.0	fBig5	BCDE
U+6BC6.0	fGB	1-4539
U+6BC6.0	fR/S	 79.11
U+6BC6.0	fTotalStrokes	 15
U+6BC6.0	fCangjie	SRHNE
U+6BC6.0	fFourCorner	77747
U+6BC6.0	fHakka	 eu3;  eu1; iu1; eu2
U+6BC6.0	fCantonese	au2; ngau2 (also); au1 (also); ngau1 (also); keoi1
U+6BC6.0	fMandarin	ou1; kou1; qu1; ou3
U+6BC6.0	fEnglish	 beat; blow; hit
U+6BC7.0	fUTF8	毇
U+6BC7.0	fCNS11643	2-5770
U+6BC7.0	fBig5	E9E5
U+6BC7.0	fGB	3-4F3B
U+6BC7.0	fR/S	 79.12
U+6BC7.0	fTotalStrokes	 16
U+6BC7.0	fCangjie	HDHNE
U+6BC7.0	fFourCorner	77947
U+6BC7.0	fCantonese	wai2
U+6BC7.0	fMandarin	hui3; zuo4
U+6BC8.0	fUTF8	毈
U+6BC8.0	fCNS11643	2-576F
U+6BC8.0	fBig5	E9E4
U+6BC8.0	fGB	5-5070
U+6BC8.0	fR/S	 79.12
U+6BC8.0	fTotalStrokes	 16
U+6BC8.0	fCangjie	HHSLE
U+6BC8.0	fFourCorner	77247
U+6BC8.0	fCantonese	dyun6; wo5
U+6BC8.0	fMandarin	duan4; duo4
U+6BC8.0	fEnglish	fertile eggs
U+6BC9.0	fUTF8	毉
U+6BC9.0	fCNS11643	2-6175
U+6BC9.0	fBig5	EFE9
U+6BC9.0	fGB	5-5074
U+6BC9.0	fR/S	 79.15
U+6BC9.0	fTotalStrokes	 19
U+6BC9.0	fCangjie	SEMOO
U+6BC9.0	fFourCorner	77108
U+6BC9.0	fHakka	 i1
U+6BC9.0	fCantonese	ji1
U+6BC9.0	fMandarin	yi1
U+6BCA.0	fUTF8	毊
U+6BCA.0	fCNS11643	2-6F32
U+6BCA.0	fBig5	F7E3
U+6BCA.0	fGB	5-5072
U+6BCA.0	fR/S	 79.19
U+6BCA.0	fTotalStrokes	 23
U+6BCA.0	fCangjie	GEHKB
U+6BCA.0	fFourCorner	47227
U+6BCA.0	fCantonese	hiu1; kiu4
U+6BCA.0	fMandarin	xiao1; qiao2
U+6BCB.0	fUTF8	毋
U+6BCB.0	fCNS11643	1-4551
U+6BCB.0	fBig5	A4F0
U+6BCB.0	fGB	0-4E63
U+6BCB.0	fR/S	 80.0
U+6BCB.0	fTotalStrokes	 4
U+6BCB.0	fCangjie	WJH
U+6BCB.0	fFourCorner	77500
U+6BCB.0	fHakka	  wu1;  mu1; m2; wu3; vu1
U+6BCB.0	fCantonese	mou4
U+6BCB.0	fMandarin	wu2; mou2; wu3
U+6BCB.0	fEnglish	[1] do not; not; no (imperative) [2] a Chinese family name [3] Kangxi radical 80
U+6BCC.0	fUTF8	毌
U+6BCC.0	fCNS11643	2-2140
U+6BCC.0	fBig5	C960
U+6BCC.0	fGB	3-5324
U+6BCC.0	fR/S	 80.0
U+6BCC.0	fTotalStrokes	 4
U+6BCC.0	fCangjie	WJ
U+6BCC.0	fFourCorner	77750
U+6BCC.0	fHakka	 gon5
U+6BCC.0	fCantonese	gun3; gwun3 (also); kwun3
U+6BCC.0	fMandarin	guan4; guan1
U+6BCD.0	fUTF8	母
U+6BCD.0	fCNS11643	1-4660
U+6BCD.0	fBig5	A5C0
U+6BCD.0	fGB	0-4438
U+6BCD.0	fR/S	 80.1
U+6BCD.0	fTotalStrokes	 5
U+6BCD.0	fCangjie	WYI
U+6BCD.0	fFourCorner	77500
U+6BCD.0	fHakka	  mu1;  mu3; me1
U+6BCD.0	fCantonese	mou5; mou2 (also); mau4
U+6BCD.0	fMandarin	mu3; mu2; mou3; wu3; wu2
U+6BCD.0	fEnglish	[1] one's mother; one's female parent [2] mother (figuratively); origin
U+6BCE.0	fUTF8	毎
U+6BCE.0	fCNS11643	3-237A
U+6BCE.0	fGB	E-2F65
U+6BCE.0	fR/S	 80.2
U+6BCE.0	fTotalStrokes	 6
U+6BCE.0	fCangjie	OWJ
U+6BCE.0	fFourCorner	80500
U+6BCE.0	fCantonese	mui5
U+6BCE.0	fMandarin	mei3; mei4
U+6BCE.0	fEnglish	every
U+6BCF.0	fUTF8	每
U+6BCF.0	fCNS11643	1-4A64
U+6BCF.0	fBig5	A843
U+6BCF.0	fGB	0-433F
U+6BCF.0	fR/S	 80.3
U+6BCF.0	fTotalStrokes	 7
U+6BCF.0	fCangjie	OWYI
U+6BCF.0	fFourCorner	80507
U+6BCF.0	fHakka	  mui1; mi1; mi3; mui3; moi3
U+6BCF.0	fCantonese	mui5
U+6BCF.0	fMandarin	mei3; mei4; mei2
U+6BCF.0	fEnglish	every; each; per
U+6BD0.0	fUTF8	毐
U+6BD0.0	fCNS11643	2-2448
U+6BD0.0	fBig5	CB48
U+6BD0.0	fGB	3-347E
U+6BD0.0	fR/S	 80.3
U+6BD0.0	fTotalStrokes	 7
U+6BD0.0	fCangjie	GWJ
U+6BD0.0	fFourCorner	40557
U+6BD0.0	fHakka	 ai3; ai1
U+6BD0.0	fCantonese	oi2; ngoi2 (also); aai2; ai4
U+6BD0.0	fMandarin	ai3; 1
U+6BD0.0	fEnglish	unprincipled; adulterous; given up to lust
U+6BD1.0	fUTF8	毑
U+6BD1.0	fCNS11643	3-295A
U+6BD1.0	fGB	3-5325
U+6BD1.0	fR/S	 80.4; 80.3
U+6BD1.0	fTotalStrokes	 8
U+6BD1.0	fCangjie	WYPD
U+6BD1.0	fFourCorner	74510
U+6BD1.0	fCantonese	ze2
U+6BD1.0	fMandarin	zuo3; jie3
U+6BD2.0	fUTF8	毒
U+6BD2.0	fCNS11643	1-5175
U+6BD2.0	fBig5	AC72
U+6BD2.0	fGB	0-363E
U+6BD2.0	fR/S	 80.4
U+6BD2.0	fTotalStrokes	 8
U+6BD2.0	fCangjie	QMWYI
U+6BD2.0	fFourCorner	50507
U+6BD2.0	fHakka	  tuk8; teu6; teu5
U+6BD2.0	fCantonese	duk6; dou6 (also); doi6; duk1
U+6BD2.0	fMandarin	du2; dai4
U+6BD2.0	fEnglish	[1] poison; venom; toxin; poisonous [2] bright sun
U+6BD3.0	fUTF8	毓
U+6BD3.0	fCNS11643	1-645C
U+6BD3.0	fBig5	B7B6
U+6BD3.0	fGB	0-5839
U+6BD3.0	fR/S	 80.9
U+6BD3.0	fTotalStrokes	 13
U+6BD3.0	fCangjie	OYYIU
U+6BD3.0	fFourCorner	80513
U+6BD3.0	fHakka	  juk7; ruk7
U+6BD3.0	fCantonese	juk1; juk6
U+6BD3.0	fMandarin	yu4
U+6BD3.0	fEnglish	[1] bring up; rear; nurture; nurse [2] grow
U+6BD4.0	fUTF8	比
U+6BD4.0	fCNS11643	1-4552
U+6BD4.0	fBig5	A4F1
U+6BD4.0	fGB	0-3148
U+6BD4.0	fR/S	 81.0
U+6BD4.0	fTotalStrokes	 4
U+6BD4.0	fCangjie	PP
U+6BD4.0	fFourCorner	21710
U+6BD4.0	fHakka	  bi3
U+6BD4.0	fCantonese	bei2; bei6; pei4; bei3
U+6BD4.0	fMandarin	bi3; pi2; pi3
U+6BD4.0	fEnglish	[1] compare with; be comparable to [2] liken; compare [3] compete [4] than [5] to
U+6BD4.1	fUTF8	比
U+6BD4.1	fCNS11643	1-4552
U+6BD4.1	fBig5	A4F1
U+6BD4.1	fGB	0-3148
U+6BD4.1	fR/S	 81.0
U+6BD4.1	fTotalStrokes	 4
U+6BD4.1	fCangjie	PP
U+6BD4.1	fFourCorner	21710
U+6BD4.1	fMandarin	bi4
U+6BD4.1	fEnglish	[1] close; near; neighboring [2] stand side by side
U+6BD5.0	fUTF8	毕
U+6BD5.0	fGB	0-314F
U+6BD5.0	fR/S	 81.2
U+6BD5.0	fTotalStrokes	 6
U+6BD5.0	fCangjie	PPJ
U+6BD5.0	fHakka	 bit7
U+6BD5.0	fCantonese	bat1
U+6BD5.0	fMandarin	bi4
U+6BD5.0	fEnglish	simplified form of &#x7562;
U+6BD6.0	fUTF8	毖
U+6BD6.0	fCNS11643	2-2B4A
U+6BD6.0	fBig5	CF68
U+6BD6.0	fGB	0-3151
U+6BD6.0	fR/S	 81.5
U+6BD6.0	fTotalStrokes	 9
U+6BD6.0	fCangjie	PPPH
U+6BD6.0	fFourCorner	21331
U+6BD6.0	fHakka	 bi5; bit7
U+6BD6.0	fCantonese	bei3
U+6BD6.0	fMandarin	bi4
U+6BD6.0	fEnglish	[1] cautious; caution; take care; guard against; judicious [2] flowing (springs); gushing
U+6BD7.0	fUTF8	毗
U+6BD7.0	fCNS11643	1-5176
U+6BD7.0	fBig5	AC73
U+6BD7.0	fGB	0-457E
U+6BD7.0	fR/S	 81.5; 102.4
U+6BD7.0	fTotalStrokes	 9
U+6BD7.0	fCangjie	WPP
U+6BD7.0	fFourCorner	61010
U+6BD7.0	fHakka	 pi2
U+6BD7.0	fCantonese	pei4; bei2 (also)
U+6BD7.0	fMandarin	pi2
U+6BD7.0	fEnglish	[1] help; assist [2] connect; adjoin
U+6BD8.0	fUTF8	毘
U+6BD8.0	fCNS11643	2-2B4B
U+6BD8.0	fBig5	CF69
U+6BD8.0	fGB	E-2F66
U+6BD8.0	fR/S	 81.5
U+6BD8.0	fTotalStrokes	 9
U+6BD8.0	fCangjie	WPP
U+6BD8.0	fFourCorner	60711
U+6BD8.0	fHakka	  pi2
U+6BD8.0	fCantonese	pei4; bei2 (also); pei3
U+6BD8.0	fMandarin	pi2
U+6BD8.0	fEnglish	 help; assist connect; adjoin
U+6BD9.0	fUTF8	毙
U+6BD9.0	fGB	0-3150
U+6BD9.0	fR/S	 81.6
U+6BD9.0	fTotalStrokes	 10
U+6BD9.0	fCangjie	PPMNP
U+6BD9.0	fHakka	 bi5
U+6BD9.0	fCantonese	bai6
U+6BD9.0	fMandarin	bi4
U+6BD9.0	fEnglish	simplified form of &#x6583;
U+6BDA.0	fUTF8	毚
U+6BDA.0	fCNS11643	1-737E
U+6BDA.0	fBig5	C0D5
U+6BDA.0	fGB	3-3270
U+6BDA.0	fR/S	 81.13
U+6BDA.0	fTotalStrokes	 17
U+6BDA.0	fCangjie	NRPPI
U+6BDA.0	fFourCorner	27213
U+6BDA.0	fHakka	  cam2
U+6BDA.0	fCantonese	caam4
U+6BDA.0	fMandarin	chan2; chan1
U+6BDA.0	fEnglish	[1] cunning; crafty [2] greedy
U+6BDB.0	fUTF8	毛
U+6BDB.0	fCNS11643	1-4553
U+6BDB.0	fBig5	A4F2
U+6BDB.0	fGB	0-432B
U+6BDB.0	fR/S	 82.0
U+6BDB.0	fTotalStrokes	 4
U+6BDB.0	fCangjie	HQU
U+6BDB.0	fFourCorner	20714
U+6BDB.0	fHakka	  mau1;  mo1
U+6BDB.0	fCantonese	mou4; mou1 (also); maau4
U+6BDB.0	fMandarin	mao2; mao4; mu2
U+6BDB.0	fEnglish	[1] hair; fur; feathers; down [2] vegetation [3] ten cents; dime [4] gross; untouched; unpolished [5] panic-stricken; scared; flurried [6] very young; little [7] a Chinese family name
U+6BDB.1	fUTF8	毛
U+6BDB.1	fCNS11643	1-4553
U+6BDB.1	fBig5	A4F2
U+6BDB.1	fGB	0-432B
U+6BDB.1	fR/S	 82.0
U+6BDB.1	fTotalStrokes	 4
U+6BDB.1	fCangjie	HQU
U+6BDB.1	fFourCorner	20714
U+6BDB.1	fMandarin	mao5
U+6BDC.0	fUTF8	毜
U+6BDC.0	fCNS11643	3-6538
U+6BDC.0	fGB	E-2F67
U+6BDC.0	fR/S	 82.3
U+6BDC.0	fTotalStrokes	 7
U+6BDC.0	fCangjie	HUF
U+6BDC.0	fFourCorner	2971
U+6BDC.0	fCantonese	hou4
U+6BDD.0	fUTF8	毝
U+6BDD.0	fCNS11643	3-6539
U+6BDD.0	fGB	E-2F68
U+6BDD.0	fR/S	 82.3
U+6BDD.0	fTotalStrokes	 7
U+6BDD.0	fCangjie	HUHHH
U+6BDD.0	fFourCorner	2271
U+6BDD.0	fCantonese	coi2
U+6BDE.0	fUTF8	毞
U+6BDE.0	fCNS11643	2-274D
U+6BDE.0	fBig5	CCEC
U+6BDE.0	fGB	5-4842
U+6BDE.0	fR/S	 82.4
U+6BDE.0	fTotalStrokes	 8
U+6BDE.0	fCangjie	PPHQU
U+6BDE.0	fFourCorner	21714
U+6BDE.0	fCantonese	bei4; pei4
U+6BDE.0	fMandarin	pi2; pu2
U+6BDF.0	fUTF8	毟
U+6BDF.0	fGB	E-2F69
U+6BDF.0	fR/S	 82.4
U+6BDF.0	fTotalStrokes	 8
U+6BDF.0	fCangjie	FHHQU
U+6BDF.0	fFourCorner	9071
U+6BDF.0	fCantonese	siu2
U+6BDF.0	fEnglish	pluck; pick; tear
U+6BE0.0	fUTF8	毠
U+6BE0.0	fCNS11643	2-2B4C
U+6BE0.0	fBig5	CF6A
U+6BE0.0	fGB	E-2F6A
U+6BE0.0	fR/S	 82.5
U+6BE0.0	fTotalStrokes	 9
U+6BE0.0	fCangjie	KRHQU
U+6BE0.0	fFourCorner	46714
U+6BE0.0	fCantonese	gaa1
U+6BE0.0	fMandarin	jia1
U+6BE1.0	fUTF8	毡
U+6BE1.0	fCNS11643	3-2D42
U+6BE1.0	fGB	0-5531
U+6BE1.0	fR/S	  82.5
U+6BE1.0	fTotalStrokes	 9
U+6BE1.0	fCangjie	HUYR
U+6BE1.0	fFourCorner	21710
U+6BE1.0	fHakka	 zhan1; zan1; zhen1
U+6BE1.0	fCantonese	zin1
U+6BE1.0	fMandarin	zhan1
U+6BE1.0	fEnglish	simplified form of &#x6C08;
U+6BE2.0	fUTF8	毢
U+6BE2.0	fCNS11643	2-3025
U+6BE2.0	fBig5	D242
U+6BE2.0	fGB	5-4E2C
U+6BE2.0	fR/S	 82.6
U+6BE2.0	fTotalStrokes	 10
U+6BE2.0	fCangjie	HUMCW
U+6BE2.0	fFourCorner	21716
U+6BE2.0	fCantonese	soi1
U+6BE2.0	fMandarin	sai1
U+6BE3.0	fUTF8	毣
U+6BE3.0	fCNS11643	2-3024
U+6BE3.0	fBig5	D241
U+6BE3.0	fGB	3-6B65
U+6BE3.0	fR/S	 82.6
U+6BE3.0	fTotalStrokes	 10
U+6BE3.0	fCangjie	SMHQU
U+6BE3.0	fFourCorner	17714
U+6BE3.0	fHakka	  muk7
U+6BE3.0	fCantonese	muk6
U+6BE3.0	fMandarin	mu4; mao4
U+6BE3.0	fEnglish	rushing of wind
U+6BE4.0	fUTF8	毤
U+6BE4.0	fCNS11643	2-3022
U+6BE4.0	fBig5	D1FE
U+6BE4.0	fGB	E-2F6B
U+6BE4.0	fR/S	 82.6
U+6BE4.0	fTotalStrokes	 10
U+6BE4.0	fCangjie	CUHQU
U+6BE4.0	fFourCorner	82211
U+6BE4.0	fHakka	 tui5; tot7; to5; toi5
U+6BE4.0	fCantonese	teoi3; to3
U+6BE4.0	fMandarin	tuo4; yue4
U+6BE4.0	fEnglish	 moult
U+6BE5.0	fUTF8	毥
U+6BE5.0	fCNS11643	3-316E
U+6BE5.0	fGB	3-4B62
U+6BE5.0	fR/S	 82.6
U+6BE5.0	fTotalStrokes	 10
U+6BE5.0	fCangjie	HUPA
U+6BE5.0	fFourCorner	27710
U+6BE5.0	fCantonese	seon1
U+6BE5.0	fMandarin	xun2; xun4
U+6BE6.0	fUTF8	毦
U+6BE6.0	fCNS11643	2-3021
U+6BE6.0	fBig5	D1FD
U+6BE6.0	fGB	3-6048
U+6BE6.0	fR/S	 82.6
U+6BE6.0	fTotalStrokes	 10
U+6BE6.0	fCangjie	SJHQU
U+6BE6.0	fFourCorner	12414
U+6BE6.0	fHakka	 ngi3; ngi5
U+6BE6.0	fCantonese	nei6; lei6 (also); ji6
U+6BE6.0	fMandarin	er4; er3
U+6BE6.0	fEnglish	[1] tail or hair of the yak [2] embroidery of feathers
U+6BE7.0	fUTF8	毧
U+6BE7.0	fCNS11643	2-3026
U+6BE7.0	fBig5	D243
U+6BE7.0	fGB	E-2F6C
U+6BE7.0	fR/S	 82.6
U+6BE7.0	fTotalStrokes	 10
U+6BE7.0	fCangjie	HUIJ
U+6BE7.0	fFourCorner	23715
U+6BE7.0	fHakka	 jung2; rung2; rung1; jung1
U+6BE7.0	fCantonese	jung4 (also); jung2
U+6BE7.0	fMandarin	rong2
U+6BE7.0	fEnglish	[1] fine hair; down [2] felt
U+6BE8.0	fUTF8	毨
U+6BE8.0	fCNS11643	2-3023
U+6BE8.0	fBig5	D240
U+6BE8.0	fGB	3-4B61
U+6BE8.0	fR/S	 82.6
U+6BE8.0	fTotalStrokes	 10
U+6BE8.0	fCangjie	HUHGU
U+6BE8.0	fFourCorner	24711
U+6BE8.0	fHakka	  sien3
U+6BE8.0	fCantonese	sin2
U+6BE8.0	fMandarin	xian3
U+6BE8.0	fEnglish	 moult; shed hair
U+6BE9.0	fUTF8	毩
U+6BE9.0	fCNS11643	3-316C
U+6BE9.0	fGB	E-2F6D
U+6BE9.0	fR/S	 82.6
U+6BE9.0	fTotalStrokes	 10
U+6BE9.0	fCangjie	HUFD
U+6BE9.0	fFourCorner	29710
U+6BE9.0	fCantonese	guk1
U+6BE9.0	fMandarin	ju2
U+6BE9.0	fEnglish	a leather ball for kicking
U+6BEA.0	fUTF8	毪
U+6BEA.0	fCNS11643	3-316D
U+6BEA.0	fGB	0-6B24
U+6BEA.0	fR/S	 82.6
U+6BEA.0	fTotalStrokes	 10
U+6BEA.0	fCangjie	HUIHQ
U+6BEA.0	fFourCorner	23710
U+6BEA.0	fCantonese	mou4; mau4 (also)
U+6BEA.0	fMandarin	mu2
U+6BEA.0	fEnglish	serge from tibet
U+6BEB.0	fUTF8	毫
U+6BEB.0	fCNS11643	1-5B45
U+6BEB.0	fBig5	B240
U+6BEB.0	fGB	0-3A41
U+6BEB.0	fR/S	 82.7
U+6BEB.0	fTotalStrokes	 11
U+6BEB.0	fCangjie	YRBU
U+6BEB.0	fFourCorner	00714
U+6BEB.0	fHakka	  hau2; ho2; hau3
U+6BEB.0	fCantonese	hou4; hou2 (also)
U+6BEB.0	fMandarin	hao2
U+6BEB.0	fEnglish	[1] fine hair [2] a measure of length [3] writing brush [4] dime [5] a measure of weight [6] a Chinese family name
U+6BEC.0	fUTF8	毬
U+6BEC.0	fCNS11643	1-5B46
U+6BEC.0	fBig5	B241
U+6BEC.0	fGB	E-2F6E
U+6BEC.0	fR/S	 82.7
U+6BEC.0	fTotalStrokes	 11
U+6BEC.0	fCangjie	HUIJE
U+6BEC.0	fFourCorner	23713
U+6BEC.0	fHakka	 kiu2; kiu3
U+6BEC.0	fCantonese	kau4
U+6BEC.0	fMandarin	qiu2
U+6BEC.0	fEnglish	 ball; sphere
U+6BED.0	fUTF8	毭
U+6BED.0	fCNS11643	3-3666
U+6BED.0	fGB	3-6E43
U+6BED.0	fR/S	  82.7
U+6BED.0	fTotalStrokes	 11
U+6BED.0	fCangjie	MTHQU
U+6BED.0	fFourCorner	12110
U+6BED.0	fCantonese	dau6
U+6BED.0	fMandarin	dou4; nuo4
U+6BEE.0	fUTF8	毮
U+6BEE.0	fGB	E-2F6F
U+6BEE.0	fR/S	 82.7
U+6BEE.0	fTotalStrokes	 11
U+6BEE.0	fCangjie	QFHU
U+6BEE.0	fCantonese	caau1
U+6BEE.0	fEnglish	 to pull out hair, etc.
U+6BEF.0	fUTF8	毯
U+6BEF.0	fCNS11643	1-6027
U+6BEF.0	fBig5	B4E0
U+6BEF.0	fGB	0-4C3A
U+6BEF.0	fR/S	 82.8
U+6BEF.0	fTotalStrokes	 12
U+6BEF.0	fCangjie	HUFF
U+6BEF.0	fFourCorner	29718
U+6BEF.0	fHakka	 tan3; tam3
U+6BEF.0	fCantonese	taam2 (also); taan2
U+6BEF.0	fMandarin	tan3
U+6BEF.0	fEnglish	 rug; carpet; blanket
U+6BF0.0	fUTF8	毰
U+6BF0.0	fCNS11643	2-3D28
U+6BF0.0	fBig5	D9E3
U+6BF0.0	fGB	3-4B6A
U+6BF0.0	fR/S	 82.8
U+6BF0.0	fTotalStrokes	 12
U+6BF0.0	fCangjie	HUYTR
U+6BF0.0	fFourCorner	20716
U+6BF0.0	fHakka	 pui2; pi2
U+6BF0.0	fCantonese	pui4
U+6BF0.0	fMandarin	pei2
U+6BF0.0	fEnglish	the wing feathers spread out
U+6BF1.0	fUTF8	毱
U+6BF1.0	fCNS11643	4-384E
U+6BF1.0	fGB	5-4E33
U+6BF1.0	fR/S	 82.8
U+6BF1.0	fTotalStrokes	 12
U+6BF1.0	fCangjie	HUPFD
U+6BF1.0	fFourCorner	27710
U+6BF1.0	fHakka	  kiuk7
U+6BF1.0	fCantonese	guk1
U+6BF1.0	fMandarin	ju2
U+6BF1.0	fEnglish	football
U+6BF2.0	fUTF8	毲
U+6BF2.0	fCNS11643	2-3D29
U+6BF2.0	fBig5	D9E4
U+6BF2.0	fGB	3-4B6B
U+6BF2.0	fR/S	 82.8
U+6BF2.0	fTotalStrokes	 12
U+6BF2.0	fCangjie	EEEEU
U+6BF2.0	fFourCorner	12414
U+6BF2.0	fCantonese	zyut3
U+6BF2.0	fMandarin	duo2
U+6BF3.0	fUTF8	毳
U+6BF3.0	fCNS11643	2-3D2A
U+6BF3.0	fBig5	D9E5
U+6BF3.0	fGB	0-6B25
U+6BF3.0	fR/S	 82.8
U+6BF3.0	fTotalStrokes	 12
U+6BF3.0	fCangjie	HUHUU
U+6BF3.0	fFourCorner	20714
U+6BF3.0	fHakka	  cui5
U+6BF3.0	fCantonese	ceoi3; zyut3
U+6BF3.0	fMandarin	cui4; rui4; qiao1; chui4; xia1; chuo4; jue2
U+6BF3.0	fEnglish	 fine feathers; fine hair or fur on animals
U+6BF4.0	fUTF8	毴
U+6BF4.0	fCNS11643	3-3C4D
U+6BF4.0	fGB	5-4E31
U+6BF4.0	fR/S	  82.8
U+6BF4.0	fTotalStrokes	 12
U+6BF4.0	fCangjie	HULMY
U+6BF4.0	fFourCorner	21710
U+6BF4.0	fCantonese	bei1
U+6BF4.0	fMandarin	bi1
U+6BF5.0	fUTF8	毵
U+6BF5.0	fCNS11643	F-3C3C
U+6BF5.0	fGB	0-6B27
U+6BF5.0	fR/S	 82.8
U+6BF5.0	fTotalStrokes	 12
U+6BF5.0	fCangjie	XIHHQ
U+6BF5.0	fFourCorner	2221
U+6BF5.0	fHakka	 cam1; sem1; sam1
U+6BF5.0	fCantonese	saam1
U+6BF5.0	fMandarin	san1
U+6BF5.0	fEnglish	simplified form of &#x6BFF;
U+6BF6.0	fUTF8	毶
U+6BF6.0	fCNS11643	3-653A
U+6BF6.0	fGB	E-2F70
U+6BF6.0	fR/S	 82.8
U+6BF6.0	fTotalStrokes	 12
U+6BF6.0	fCangjie	HUIKH
U+6BF6.0	fFourCorner	2371
U+6BF6.0	fCantonese	saam1
U+6BF7.0	fUTF8	毷
U+6BF7.0	fCNS11643	2-4427
U+6BF7.0	fBig5	DE41
U+6BF7.0	fGB	5-4E37
U+6BF7.0	fR/S	 82.9
U+6BF7.0	fTotalStrokes	 13
U+6BF7.0	fCangjie	AUHQU
U+6BF7.0	fFourCorner	62614
U+6BF7.0	fCantonese	mou6
U+6BF7.0	fMandarin	mao4
U+6BF7.0	fEnglish	restless
U+6BF8.0	fUTF8	毸
U+6BF8.0	fCNS11643	2-4428
U+6BF8.0	fBig5	DE42
U+6BF8.0	fGB	3-4B6F
U+6BF8.0	fR/S	 82.9
U+6BF8.0	fTotalStrokes	 13
U+6BF8.0	fCangjie	HUWP
U+6BF8.0	fFourCorner	26713
U+6BF8.0	fHakka	  soi1; sui1
U+6BF8.0	fCantonese	soi1; seoi1
U+6BF8.0	fMandarin	sui1; sai1
U+6BF8.0	fEnglish	[1] spread the wings [2] wings fluttering
U+6BF9.0	fUTF8	毹
U+6BF9.0	fCNS11643	2-4426
U+6BF9.0	fBig5	DE40
U+6BF9.0	fGB	0-6B28
U+6BF9.0	fR/S	 82.9
U+6BF9.0	fTotalStrokes	 13
U+6BF9.0	fCangjie	ONHQU
U+6BF9.0	fFourCorner	82214
U+6BF9.0	fHakka	 shu1; ri2; ji2; su1; ji1
U+6BF9.0	fCantonese	jyu4 (also); syu1
U+6BF9.0	fMandarin	shu1; yu2; yu1
U+6BF9.0	fEnglish	a mat for kneeling on
U+6BFA.0	fUTF8	毺
U+6BFA.0	fCNS11643	3-4228
U+6BFA.0	fGB	5-4E38
U+6BFA.0	fR/S	 82.9
U+6BFA.0	fTotalStrokes	 13
U+6BFA.0	fCangjie	HUOMN
U+6BFA.0	fFourCorner	28710
U+6BFA.0	fCantonese	jyu4 (also); syu1
U+6BFA.0	fMandarin	yu1
U+6BFB.0	fUTF8	毻
U+6BFB.0	fCNS11643	2-4424
U+6BFB.0	fBig5	DDFD
U+6BFB.0	fGB	5-4E36
U+6BFB.0	fR/S	 82.9
U+6BFB.0	fTotalStrokes	 13
U+6BFB.0	fCangjie	KBHQU
U+6BFB.0	fFourCorner	42214
U+6BFB.0	fHakka	 to5; to3
U+6BFB.0	fCantonese	teoi3; to3
U+6BFB.0	fMandarin	tuo4; tuo3; tui4
U+6BFB.0	fEnglish	 moult change the coat (of animals)
U+6BFC.0	fUTF8	毼
U+6BFC.0	fCNS11643	2-4425
U+6BFC.0	fBig5	DDFE
U+6BFC.0	fGB	3-4B6E
U+6BFC.0	fR/S	 82.9
U+6BFC.0	fTotalStrokes	 13
U+6BFC.0	fCangjie	AVHQU
U+6BFC.0	fFourCorner	62715
U+6BFC.0	fHakka	 hot7; hot8
U+6BFC.0	fCantonese	hot3; hot6
U+6BFC.0	fMandarin	he2; ke3; da2
U+6BFC.0	fEnglish	serge
U+6BFD.0	fUTF8	毽
U+6BFD.0	fCNS11643	1-645D
U+6BFD.0	fBig5	B7B7
U+6BFD.0	fGB	0-6B26
U+6BFD.0	fR/S	 82.9
U+6BFD.0	fTotalStrokes	 13
U+6BFD.0	fCangjie	HUNKQ
U+6BFD.0	fFourCorner	25714
U+6BFD.0	fHakka	 gien5; jen5; gian3; gen3
U+6BFD.0	fCantonese	gin3; jin2 (also)
U+6BFD.0	fMandarin	jian4
U+6BFD.0	fEnglish	 shuttlecock
U+6BFE.0	fUTF8	毾
U+6BFE.0	fCNS11643	2-4B33
U+6BFE.0	fBig5	E26B
U+6BFE.0	fGB	3-4B74
U+6BFE.0	fR/S	 82.10
U+6BFE.0	fTotalStrokes	 14
U+6BFE.0	fCangjie	AMHQU
U+6BFE.0	fFourCorner	62114
U+6BFE.0	fHakka	  tap7
U+6BFE.0	fCantonese	taap3
U+6BFE.0	fMandarin	ta4
U+6BFE.0	fEnglish	a coarse; woollen serge
U+6BFF.0	fUTF8	毿
U+6BFF.0	fCNS11643	2-5140
U+6BFF.0	fBig5	E5F7
U+6BFF.0	fGB	1-6B27
U+6BFF.0	fR/S	 82.11
U+6BFF.0	fTotalStrokes	 15
U+6BFF.0	fCangjie	IHHQU
U+6BFF.0	fFourCorner	22214
U+6BFF.0	fHakka	 sam1;  cam1; sem1
U+6BFF.0	fCantonese	saam1
U+6BFF.0	fMandarin	san1
U+6BFF.0	fEnglish	 long feathers scraggy
U+6C00.0	fUTF8	氀
U+6C00.0	fCNS11643	2-513F
U+6C00.0	fBig5	E5F6
U+6C00.0	fGB	3-4B71
U+6C00.0	fR/S	 82.11
U+6C00.0	fTotalStrokes	 15
U+6C00.0	fCangjie	LVHQU
U+6C00.0	fFourCorner	52414
U+6C00.0	fCantonese	leoi4
U+6C00.0	fMandarin	l2; luu2; shu1; dou1
U+6C01.0	fUTF8	氁
U+6C01.0	fCNS11643	2-513E
U+6C01.0	fBig5	E5F5
U+6C01.0	fGB	E-2F71
U+6C01.0	fR/S	 82.11
U+6C01.0	fTotalStrokes	 15
U+6C01.0	fCangjie	HUTAK
U+6C01.0	fFourCorner	24713
U+6C01.0	fCantonese	mou4
U+6C01.0	fMandarin	mu2
U+6C02.0	fUTF8	氂
U+6C02.0	fCNS11643	2-5141
U+6C02.0	fBig5	E5F8
U+6C02.0	fGB	E-2F72
U+6C02.0	fR/S	 82.11
U+6C02.0	fTotalStrokes	 15
U+6C02.0	fCangjie	JKMHU
U+6C02.0	fFourCorner	47714
U+6C02.0	fHakka	  li2
U+6C02.0	fCantonese	mou4; lei4; moi4
U+6C02.0	fMandarin	mao2; li2; lai2
U+6C02.0	fEnglish	[1] horse tail [2] long hair; thick hair [3] the yak
U+6C03.0	fUTF8	氃
U+6C03.0	fCNS11643	2-5772
U+6C03.0	fBig5	E9E7
U+6C03.0	fGB	3-4B79
U+6C03.0	fR/S	 82.12
U+6C03.0	fTotalStrokes	 16
U+6C03.0	fCangjie	YGHQU
U+6C03.0	fFourCorner	02114
U+6C03.0	fHakka	  tung2
U+6C03.0	fCantonese	tung4
U+6C03.0	fMandarin	tong2
U+6C03.0	fEnglish	the feathers in disorder
U+6C04.0	fUTF8	氄
U+6C04.0	fCNS11643	2-5771
U+6C04.0	fBig5	E9E6
U+6C04.0	fGB	3-4B7B
U+6C04.0	fR/S	 82.12
U+6C04.0	fTotalStrokes	 16
U+6C04.0	fCangjie	NBHQU
U+6C04.0	fFourCorner	12214
U+6C04.0	fHakka	 jung3; jung2; rung1; jung1; rung3
U+6C04.0	fCantonese	jung5 (also); jung2
U+6C04.0	fMandarin	rong3; run3; rong2
U+6C04.0	fEnglish	[1] fine hair [2] fine soft
U+6C05.0	fUTF8	氅
U+6C05.0	fCNS11643	1-7126
U+6C05.0	fBig5	BEFB
U+6C05.0	fGB	0-6B29
U+6C05.0	fR/S	 82.12
U+6C05.0	fTotalStrokes	 16
U+6C05.0	fCangjie	FKHQU
U+6C05.0	fFourCorner	98710
U+6C05.0	fHakka	 chong3; cong3
U+6C05.0	fCantonese	cong2
U+6C05.0	fMandarin	chang3
U+6C05.0	fEnglish	a garment woven of down
U+6C06.0	fUTF8	氆
U+6C06.0	fCNS11643	2-5773
U+6C06.0	fBig5	E9E8
U+6C06.0	fGB	0-6B2B
U+6C06.0	fR/S	 82.12
U+6C06.0	fTotalStrokes	 16
U+6C06.0	fCangjie	HUTCA
U+6C06.0	fFourCorner	28716
U+6C06.0	fHakka	 pu3; bong3
U+6C06.0	fCantonese	pou2
U+6C06.0	fMandarin	pu3; bang3
U+6C06.0	fEnglish	thick; rough serge from Tibet
U+6C07.0	fUTF8	氇
U+6C07.0	fGB	0-6B2A
U+6C07.0	fR/S	 82.12
U+6C07.0	fTotalStrokes	 16
U+6C07.0	fCangjie	XHUNW
U+6C07.0	fHakka	 lo2; lu3
U+6C07.0	fCantonese	lou5
U+6C07.0	fMandarin	luo2; lu3
U+6C07.0	fEnglish	simplified form of &#x6C0C;
U+6C08.0	fUTF8	氈
U+6C08.0	fCNS11643	1-7421
U+6C08.0	fBig5	C0D6
U+6C08.0	fGB	1-5531
U+6C08.0	fR/S	 82.13
U+6C08.0	fTotalStrokes	 17
U+6C08.0	fCangjie	YMHQU
U+6C08.0	fFourCorner	02114
U+6C08.0	fHakka	 zhan1; zan1; zen1
U+6C08.0	fCantonese	zin1
U+6C08.0	fMandarin	zhan1
U+6C08.0	fEnglish	[1] felt [2] blanket
U+6C09.0	fUTF8	氉
U+6C09.0	fCNS11643	2-5D39
U+6C09.0	fBig5	ED4D
U+6C09.0	fGB	5-4E43
U+6C09.0	fR/S	 82.13
U+6C09.0	fTotalStrokes	 17
U+6C09.0	fCangjie	RDHQU
U+6C09.0	fFourCorner	62914
U+6C09.0	fCantonese	cou3; sou3 (also); so3
U+6C09.0	fMandarin	sao4
U+6C09.0	fEnglish	restless; melancholy
U+6C0A.0	fUTF8	氊
U+6C0A.0	fCNS11643	3-552D
U+6C0A.0	fGB	E-2F73
U+6C0A.0	fR/S	 82.13
U+6C0A.0	fTotalStrokes	 17
U+6C0A.0	fCangjie	HUYWM
U+6C0A.0	fFourCorner	20710
U+6C0A.0	fHakka	 zen1;  zhan1; zan1; zhen1
U+6C0A.0	fCantonese	zin1
U+6C0A.0	fMandarin	zhan1
U+6C0A.0	fEnglish	 felt coarse fabrics use for rugs;carpets; wrappers etc.
U+6C0B.0	fUTF8	氋
U+6C0B.0	fCNS11643	2-6176
U+6C0B.0	fBig5	EFEA
U+6C0B.0	fGB	3-4B7C
U+6C0B.0	fR/S	 82.14
U+6C0B.0	fTotalStrokes	 18
U+6C0B.0	fCangjie	TOHQU
U+6C0B.0	fFourCorner	42214
U+6C0B.0	fCantonese	mung4
U+6C0B.0	fMandarin	meng2
U+6C0C.0	fUTF8	氌
U+6C0C.0	fCNS11643	2-6566
U+6C0C.0	fBig5	F25B
U+6C0C.0	fGB	1-6B2A
U+6C0C.0	fR/S	 82.15
U+6C0C.0	fTotalStrokes	 19
U+6C0C.0	fCangjie	HUNWA
U+6C0C.0	fFourCorner	27716
U+6C0C.0	fHakka	  lo2;  lu3
U+6C0C.0	fCantonese	lou5
U+6C0C.0	fMandarin	luo2; lu3
U+6C0C.0	fEnglish	thick rough serge from tibet
U+6C0D.0	fUTF8	氍
U+6C0D.0	fCNS11643	2-6D55
U+6C0D.0	fBig5	F6E7
U+6C0D.0	fGB	0-6B2C
U+6C0D.0	fR/S	 82.18
U+6C0D.0	fTotalStrokes	 22
U+6C0D.0	fCangjie	BGHQU
U+6C0D.0	fFourCorner	62214
U+6C0D.0	fHakka	 ki1
U+6C0D.0	fCantonese	keoi4
U+6C0D.0	fMandarin	qu2
U+6C0D.0	fEnglish	 fine woollen cloth a mat used by the emperor in worshipping god
U+6C0E.0	fUTF8	氎
U+6C0E.0	fGB	E-2F74
U+6C0E.0	fR/S	 82.22
U+6C0E.0	fTotalStrokes	 26
U+6C0E.0	fCangjie	WMHQU
U+6C0E.0	fHakka	  tiap8
U+6C0E.0	fCantonese	dip6; dip3
U+6C0E.0	fMandarin	die2
U+6C0E.0	fEnglish	a soft whitish cloth like cashmere
U+6C0F.0	fUTF8	氏
U+6C0F.0	fCNS11643	1-4554
U+6C0F.0	fBig5	A4F3
U+6C0F.0	fGB	0-4A4F
U+6C0F.0	fR/S	 83.0
U+6C0F.0	fTotalStrokes	 4
U+6C0F.0	fCangjie	HVP
U+6C0F.0	fFourCorner	72740
U+6C0F.0	fHakka	 si3; shi6; shi5; si5; sii5; zi1; she5
U+6C0F.0	fCantonese	si6; zi1
U+6C0F.0	fMandarin	shi4; zhi3; jing1
U+6C0F.0	fEnglish	[1] clan; family; family name; surname [2] a character placed after a married woman's maiden name [3] the title of a government position in former times
U+6C0F.1	fUTF8	氏
U+6C0F.1	fCNS11643	1-4554
U+6C0F.1	fBig5	A4F3
U+6C0F.1	fGB	0-4A4F
U+6C0F.1	fR/S	 83.0
U+6C0F.1	fTotalStrokes	 4
U+6C0F.1	fCangjie	HVP
U+6C0F.1	fFourCorner	72740
U+6C0F.1	fMandarin	zhi1
U+6C0F.1	fEnglish	used in combination for the name of an ancient barbarian tribe
U+6C10.0	fUTF8	氐
U+6C10.0	fCNS11643	1-4662
U+6C10.0	fBig5	A5C2
U+6C10.0	fGB	0-5835
U+6C10.0	fR/S	 83.1
U+6C10.0	fTotalStrokes	 5
U+6C10.0	fCangjie	HPM
U+6C10.0	fFourCorner	72740
U+6C10.0	fHakka	  di3; di1
U+6C10.0	fCantonese	dai1; dai2
U+6C10.0	fMandarin	zhi1; zhi3; di1; di4
U+6C10.0	fEnglish	[1] the name of an ancient tribe to the west [2] same as $; [3] inside; inner
U+6C10.1	fUTF8	氐
U+6C10.1	fCNS11643	1-4662
U+6C10.1	fBig5	A5C2
U+6C10.1	fGB	0-5835
U+6C10.1	fR/S	 83.1
U+6C10.1	fTotalStrokes	 5
U+6C10.1	fCangjie	HPM
U+6C10.1	fFourCorner	72740
U+6C10.1	fMandarin	di3
U+6C10.1	fEnglish	[1] foundation [2] same as $
U+6C11.0	fUTF8	民
U+6C11.0	fCNS11643	1-4661
U+6C11.0	fBig5	A5C1
U+6C11.0	fGB	0-4371
U+6C11.0	fR/S	 83.1
U+6C11.0	fTotalStrokes	 5
U+6C11.0	fCangjie	RVP
U+6C11.0	fFourCorner	77747
U+6C11.0	fHakka	  min2; min3
U+6C11.0	fCantonese	man4
U+6C11.0	fMandarin	min2
U+6C11.0	fEnglish	[1] the people; the subjects; the populace; the public [2] civilians
U+6C12.0	fUTF8	氒
U+6C12.0	fCNS11643	4-2321
U+6C12.0	fGB	5-4F7C
U+6C12.0	fR/S	  83.2
U+6C12.0	fTotalStrokes	 6
U+6C12.0	fCangjie	HPJ
U+6C12.0	fFourCorner	72400
U+6C12.0	fCantonese	kyut3
U+6C12.0	fMandarin	jue2
U+6C13.0	fUTF8	氓
U+6C13.0	fCNS11643	1-4E40
U+6C13.0	fBig5	AA5D
U+6C13.0	fGB	0-4325
U+6C13.0	fR/S	 83.4
U+6C13.0	fTotalStrokes	 8
U+6C13.0	fCangjie	YVRVP
U+6C13.0	fFourCorner	07747
U+6C13.0	fHakka	 men2;  min2; {mong2}; mong2
U+6C13.0	fCantonese	maang4; man4 (also); mang4 (also); mong4; mong5
U+6C13.0	fMandarin	meng2
U+6C13.0	fEnglish	the people; vassals; populace
U+6C13.1	fUTF8	氓
U+6C13.1	fCNS11643	1-4E40
U+6C13.1	fBig5	AA5D
U+6C13.1	fGB	0-4325
U+6C13.1	fR/S	 83.4
U+6C13.1	fTotalStrokes	 8
U+6C13.1	fCangjie	YVRVP
U+6C13.1	fFourCorner	07747
U+6C13.1	fMandarin	mang2
U+6C13.1	fEnglish	 rascal; vagabond
U+6C14.0	fUTF8	气
U+6C14.0	fCNS11643	2-2141
U+6C14.0	fBig5	C961
U+6C14.0	fGB	0-4678
U+6C14.0	fR/S	 84.0
U+6C14.0	fTotalStrokes	 4
U+6C14.0	fCangjie	OMN
U+6C14.0	fFourCorner	80017
U+6C14.0	fHakka	 hi5; ki5
U+6C14.0	fCantonese	hei3
U+6C14.0	fMandarin	qi4; xi4
U+6C14.0	fEnglish	[1] Kangxi radical 84 [2] simplified form of &#x6C23;
U+6C14.1	fUTF8	气
U+6C14.1	fCNS11643	2-2141
U+6C14.1	fBig5	C961
U+6C14.1	fGB	0-4678
U+6C14.1	fR/S	 84.0
U+6C14.1	fTotalStrokes	 4
U+6C14.1	fCangjie	OMN
U+6C14.1	fFourCorner	80017
U+6C14.1	fMandarin	qi5
U+6C15.0	fUTF8	氕
U+6C15.0	fCNS11643	2-215F
U+6C15.0	fBig5	C97E
U+6C15.0	fGB	0-6B2D
U+6C15.0	fR/S	 84.1
U+6C15.0	fTotalStrokes	 5
U+6C15.0	fCangjie	ONL
U+6C15.0	fFourCorner	80217
U+6C15.0	fCantonese	pit3
U+6C15.0	fMandarin	pie1
U+6C16.0	fUTF8	氖
U+6C16.0	fCNS11643	1-483C
U+6C16.0	fBig5	A6BB
U+6C16.0	fGB	0-444A
U+6C16.0	fR/S	 84.2
U+6C16.0	fTotalStrokes	 6
U+6C16.0	fCangjie	ONNHS
U+6C16.0	fFourCorner	80217
U+6C16.0	fHakka	 lai1
U+6C16.0	fCantonese	naai5; laai5 (also)
U+6C16.0	fMandarin	nai3
U+6C16.0	fEnglish	neon
U+6C17.0	fUTF8	気
U+6C17.0	fGB	E-2F75
U+6C17.0	fR/S	 84.2
U+6C17.0	fTotalStrokes	 6
U+6C17.0	fCangjie	ONK
U+6C17.0	fCantonese	hei3
U+6C17.0	fMandarin	qi4
U+6C17.0	fEnglish	 air; gas; steam; vapor spirit
U+6C18.0	fUTF8	氘
U+6C18.0	fCNS11643	2-2258
U+6C18.0	fBig5	C9F7
U+6C18.0	fGB	0-6B2E
U+6C18.0	fR/S	 84.2
U+6C18.0	fTotalStrokes	 6
U+6C18.0	fCangjie	ONLL
U+6C18.0	fFourCorner	80217
U+6C18.0	fCantonese	dou1
U+6C18.0	fMandarin	dao1
U+6C19.0	fUTF8	氙
U+6C19.0	fCNS11643	2-2449
U+6C19.0	fBig5	CB49
U+6C19.0	fGB	0-6B2F
U+6C19.0	fR/S	 84.3
U+6C19.0	fTotalStrokes	 7
U+6C19.0	fCangjie	ONU
U+6C19.0	fFourCorner	80717
U+6C19.0	fCantonese	sin1; saan3 (also); saan1
U+6C19.0	fMandarin	xian1
U+6C1A.0	fUTF8	氚
U+6C1A.0	fCNS11643	2-244A
U+6C1A.0	fBig5	CB4A
U+6C1A.0	fGB	0-6B30
U+6C1A.0	fR/S	 84.3
U+6C1A.0	fTotalStrokes	 7
U+6C1A.0	fCangjie	ONLLL
U+6C1A.0	fFourCorner	80217
U+6C1A.0	fCantonese	cyun1
U+6C1A.0	fMandarin	chuan1
U+6C1B.0	fUTF8	氛
U+6C1B.0	fCNS11643	1-4E41
U+6C1B.0	fBig5	AA5E
U+6C1B.0	fGB	0-3755
U+6C1B.0	fR/S	 84.4
U+6C1B.0	fTotalStrokes	 8
U+6C1B.0	fCangjie	ONCSH
U+6C1B.0	fFourCorner	80217
U+6C1B.0	fHakka	 fun1
U+6C1B.0	fCantonese	fan1
U+6C1B.0	fMandarin	fen1; fen2
U+6C1B.0	fEnglish	 gas; vapor; air; atmosphere; prevailing mood
U+6C1C.0	fUTF8	氜
U+6C1C.0	fCNS11643	3-295D
U+6C1C.0	fGB	5-4E48
U+6C1C.0	fR/S	 84.4
U+6C1C.0	fTotalStrokes	 8
U+6C1C.0	fCangjie	ONA
U+6C1C.0	fFourCorner	80610
U+6C1C.0	fHakka	 jong2; rong2
U+6C1C.0	fCantonese	joeng4
U+6C1C.0	fMandarin	ri4
U+6C1C.0	fEnglish	 clear; bright the sun heat pertaining to this world superior upper front
U+6C1D.0	fUTF8	氝
U+6C1D.0	fCNS11643	2-274E
U+6C1D.0	fBig5	CCED
U+6C1D.0	fGB	5-4E49
U+6C1D.0	fR/S	 84.4
U+6C1D.0	fTotalStrokes	 8
U+6C1D.0	fCangjie	ONOB
U+6C1D.0	fFourCorner	80217
U+6C1D.0	fCantonese	noi6; loi6 (also)
U+6C1D.0	fMandarin	nei4
U+6C1D.0	fEnglish	neon
U+6C1E.0	fUTF8	氞
U+6C1E.0	fCNS11643	3-653B
U+6C1E.0	fGB	E-2F76
U+6C1E.0	fR/S	 84.5
U+6C1E.0	fTotalStrokes	 9
U+6C1E.0	fCangjie	ONMOB
U+6C1E.0	fFourCorner	8021
U+6C1E.0	fCantonese	bing2
U+6C1F.0	fUTF8	氟
U+6C1F.0	fCNS11643	1-5177
U+6C1F.0	fBig5	AC74
U+6C1F.0	fGB	0-377A
U+6C1F.0	fR/S	 84.5
U+6C1F.0	fTotalStrokes	 9
U+6C1F.0	fCangjie	ONLLN
U+6C1F.0	fFourCorner	80510
U+6C1F.0	fHakka	 fut7
U+6C1F.0	fCantonese	fat1
U+6C1F.0	fMandarin	fu2
U+6C1F.0	fEnglish	fluorine
U+6C20.0	fUTF8	氠
U+6C20.0	fCNS11643	2-2B4D
U+6C20.0	fBig5	CF6B
U+6C20.0	fGB	E-2F77
U+6C20.0	fR/S	 84.5
U+6C20.0	fTotalStrokes	 9
U+6C20.0	fCangjie	ONLWL
U+6C20.0	fFourCorner	80517
U+6C20.0	fCantonese	san1
U+6C20.0	fMandarin	shen1; shan1
U+6C21.0	fUTF8	氡
U+6C21.0	fCNS11643	2-2B4E
U+6C21.0	fBig5	CF6C
U+6C21.0	fGB	0-6B31
U+6C21.0	fR/S	 84.5
U+6C21.0	fTotalStrokes	 9
U+6C21.0	fCangjie	ONHEY
U+6C21.0	fFourCorner	80317
U+6C21.0	fCantonese	dung1
U+6C21.0	fMandarin	dong1
U+6C22.0	fUTF8	氢
U+6C22.0	fGB	0-4762
U+6C22.0	fR/S	 84.5
U+6C22.0	fTotalStrokes	 9
U+6C22.0	fCangjie	ONNOM
U+6C22.0	fCantonese	hing1
U+6C22.0	fMandarin	qing1
U+6C22.0	fEnglish	simplified form of &#x6C2B;
U+6C23.0	fUTF8	氣
U+6C23.0	fCNS11643	1-5634
U+6C23.0	fBig5	AEF0
U+6C23.0	fGB	1-4678
U+6C23.0	fR/S	 84.6
U+6C23.0	fTotalStrokes	 10
U+6C23.0	fCangjie	ONFD
U+6C23.0	fFourCorner	80917
U+6C23.0	fHakka	  hi5;  ki5
U+6C23.0	fCantonese	hei3
U+6C23.0	fMandarin	qi4; xi4
U+6C23.0	fEnglish	[1] air; gas; steam; vapor [2] spirit; morale [3] breath [4] influence [5] smells; odors [6] be angry; be indignant; rage; anger [7] bearing; manner [8] provoke; goad; make angry; annoy [9] weather
U+6C23.1	fUTF8	氣
U+6C23.1	fCNS11643	1-5634
U+6C23.1	fBig5	AEF0
U+6C23.1	fGB	1-4678
U+6C23.1	fR/S	 84.6
U+6C23.1	fTotalStrokes	 10
U+6C23.1	fCangjie	ONFD
U+6C23.1	fFourCorner	80917
U+6C23.1	fMandarin	qi5
U+6C24.0	fUTF8	氤
U+6C24.0	fCNS11643	1-5638
U+6C24.0	fBig5	AEF4
U+6C24.0	fGB	0-6B33
U+6C24.0	fR/S	 84.6
U+6C24.0	fTotalStrokes	 10
U+6C24.0	fCangjie	ONWK
U+6C24.0	fFourCorner	80617
U+6C24.0	fHakka	 jin1; rin1
U+6C24.0	fCantonese	jan1
U+6C24.0	fMandarin	yin1; yan2
U+6C24.0	fEnglish	the spirit of harmony (between heaven and earth)
U+6C25.0	fUTF8	氥
U+6C25.0	fCNS11643	2-3027
U+6C25.0	fBig5	D244
U+6C25.0	fGB	8-2D73
U+6C25.0	fR/S	 84.6
U+6C25.0	fTotalStrokes	 10
U+6C25.0	fCangjie	ONMCW
U+6C25.0	fFourCorner	80617
U+6C25.0	fCantonese	sai1; saan1; sin1
U+6C25.0	fMandarin	xi1
U+6C25.0	fEnglish	xenon
U+6C26.0	fUTF8	氦
U+6C26.0	fCNS11643	1-5637
U+6C26.0	fBig5	AEF3
U+6C26.0	fGB	0-3A24
U+6C26.0	fR/S	 84.6
U+6C26.0	fTotalStrokes	 10
U+6C26.0	fCangjie	ONYVO
U+6C26.0	fFourCorner	80817
U+6C26.0	fHakka	 hoi3
U+6C26.0	fCantonese	hoi6
U+6C26.0	fMandarin	hai4
U+6C26.0	fEnglish	helium
U+6C27.0	fUTF8	氧
U+6C27.0	fCNS11643	1-5635
U+6C27.0	fBig5	AEF1
U+6C27.0	fGB	0-5175
U+6C27.0	fR/S	 84.6
U+6C27.0	fTotalStrokes	 10
U+6C27.0	fCangjie	ONTQ
U+6C27.0	fFourCorner	80517
U+6C27.0	fHakka	 jong1
U+6C27.0	fCantonese	joeng5
U+6C27.0	fMandarin	yang3
U+6C27.0	fEnglish	oxygen
U+6C28.0	fUTF8	氨
U+6C28.0	fCNS11643	1-5636
U+6C28.0	fBig5	AEF2
U+6C28.0	fGB	0-3031
U+6C28.0	fR/S	 84.6
U+6C28.0	fTotalStrokes	 10
U+6C28.0	fCangjie	ONJV
U+6C28.0	fFourCorner	80427
U+6C28.0	fHakka	 on1
U+6C28.0	fCantonese	on1; ngon1 (also); ngoi1
U+6C28.0	fMandarin	an1
U+6C28.0	fEnglish	 ammonia hydrogen nitride
U+6C29.0	fUTF8	氩
U+6C29.0	fGB	0-6B32
U+6C29.0	fR/S	 84.6
U+6C29.0	fTotalStrokes	 10
U+6C29.0	fCangjie	ONMTC
U+6C29.0	fCantonese	aa3; ngaa3 (also)
U+6C29.0	fMandarin	ya4
U+6C29.0	fEnglish	simplified form of &#x6C2C;
U+6C2A.0	fUTF8	氪
U+6C2A.0	fCNS11643	2-3643
U+6C2A.0	fBig5	D5DF
U+6C2A.0	fGB	0-6B34
U+6C2A.0	fR/S	 84.7
U+6C2A.0	fTotalStrokes	 11
U+6C2A.0	fCangjie	ONJRU
U+6C2A.0	fFourCorner	80217
U+6C2A.0	fHakka	 ket7
U+6C2A.0	fCantonese	hak1
U+6C2A.0	fMandarin	ke4
U+6C2B.0	fUTF8	氫
U+6C2B.0	fCNS11643	1-5B47
U+6C2B.0	fBig5	B242
U+6C2B.0	fGB	1-4762
U+6C2B.0	fR/S	 84.7
U+6C2B.0	fTotalStrokes	 11
U+6C2B.0	fCangjie	ONMVM
U+6C2B.0	fFourCorner	80117
U+6C2B.0	fHakka	 kiang1
U+6C2B.0	fCantonese	hing1
U+6C2B.0	fMandarin	qing1
U+6C2B.0	fEnglish	hydrogen
U+6C2C.0	fUTF8	氬
U+6C2C.0	fCNS11643	1-602A
U+6C2C.0	fBig5	B4E3
U+6C2C.0	fGB	1-6B32
U+6C2C.0	fR/S	 84.8
U+6C2C.0	fTotalStrokes	 12
U+6C2C.0	fCangjie	ONMLM
U+6C2C.0	fFourCorner	80117
U+6C2C.0	fHakka	 a1
U+6C2C.0	fCantonese	aa3; ngaa3 (also)
U+6C2C.0	fMandarin	ya4
U+6C2C.0	fEnglish	argonium
U+6C2D.0	fUTF8	氭
U+6C2D.0	fCNS11643	3-3C4E
U+6C2D.0	fGB	5-4E4A
U+6C2D.0	fR/S	  84.8
U+6C2D.0	fTotalStrokes	 12
U+6C2D.0	fCangjie	ONDW
U+6C2D.0	fFourCorner	80910
U+6C2D.0	fCantonese	dung1
U+6C2D.0	fMandarin	dong1
U+6C2E.0	fUTF8	氮
U+6C2E.0	fCNS11643	1-6028
U+6C2E.0	fBig5	B4E1
U+6C2E.0	fGB	0-352A
U+6C2E.0	fR/S	 84.8
U+6C2E.0	fTotalStrokes	 12
U+6C2E.0	fCangjie	ONFF
U+6C2E.0	fFourCorner	80817
U+6C2E.0	fHakka	 tam3
U+6C2E.0	fCantonese	daam6
U+6C2E.0	fMandarin	dan4
U+6C2E.0	fEnglish	nitrogen
U+6C2F.0	fUTF8	氯
U+6C2F.0	fCNS11643	1-6029
U+6C2F.0	fBig5	B4E2
U+6C2F.0	fGB	0-4248
U+6C2F.0	fR/S	 84.8
U+6C2F.0	fTotalStrokes	 12
U+6C2F.0	fCangjie	ONVNE
U+6C2F.0	fFourCorner	80917
U+6C2F.0	fHakka	 luk8
U+6C2F.0	fCantonese	luk6
U+6C2F.0	fMandarin	l4; luu4; lu4
U+6C2F.0	fEnglish	chlorine
U+6C30.0	fUTF8	氰
U+6C30.0	fCNS11643	2-3D2B
U+6C30.0	fBig5	D9E6
U+6C30.0	fGB	0-4768
U+6C30.0	fR/S	 84.8
U+6C30.0	fTotalStrokes	 12
U+6C30.0	fCangjie	ONQMB
U+6C30.0	fFourCorner	80217
U+6C30.0	fHakka	 ciang1
U+6C30.0	fCantonese	cing1; cing4
U+6C30.0	fMandarin	qing1; qing2
U+6C30.0	fEnglish	 cyanogen ethane dinitrile
U+6C31.0	fUTF8	氱
U+6C31.0	fCNS11643	3-4229
U+6C31.0	fGB	5-4E46
U+6C31.0	fR/S	 84.9
U+6C31.0	fTotalStrokes	 13
U+6C31.0	fCangjie	ONAMH
U+6C31.0	fFourCorner	80210
U+6C31.0	fCantonese	joeng4
U+6C31.0	fMandarin	yang3
U+6C32.0	fUTF8	氲
U+6C32.0	fCNS11643	3-422A
U+6C32.0	fGB	0-6B35
U+6C32.0	fR/S	 84.10
U+6C32.0	fTotalStrokes	 13; 14
U+6C32.0	fCangjie	ONABT
U+6C32.0	fFourCorner	80110
U+6C32.0	fCantonese	wan1
U+6C32.0	fMandarin	yun1
U+6C32.0	fEnglish	 life giving influences of nature spirit of harmony prosperity
U+6C33.0	fUTF8	氳
U+6C33.0	fCNS11643	1-693B
U+6C33.0	fBig5	BA72
U+6C33.0	fGB	E-2F78
U+6C33.0	fR/S	 84.10
U+6C33.0	fTotalStrokes	 14
U+6C33.0	fCangjie	ONWOT
U+6C33.0	fFourCorner	80117
U+6C33.0	fHakka	 wun1; vun1
U+6C33.0	fCantonese	wan1
U+6C33.0	fMandarin	yun1
U+6C33.0	fEnglish	the atmosphere of harmony; the spirit of prosperity or vigor
U+6C34.0	fUTF8	水
U+6C34.0	fCNS11643	1-4555
U+6C34.0	fBig5	A4F4
U+6C34.0	fGB	0-4B2E
U+6C34.0	fR/S	 85.0
U+6C34.0	fTotalStrokes	 4
U+6C34.0	fCangjie	E
U+6C34.0	fFourCorner	12230
U+6C34.0	fHakka	  sui3; shui3
U+6C34.0	fCantonese	seoi2
U+6C34.0	fMandarin	shui3
U+6C34.0	fEnglish	[1] water [2] a general term for seas [3] liquid; juice [4] flood; flood disaster [5] a Chinese family name [6] Kangxi radical 85
U+6C35.0	fUTF8	氵
U+6C35.0	fGB	0-635F
U+6C35.0	fR/S	 85.0
U+6C35.0	fTotalStrokes	 3
U+6C35.0	fCangjie	YM
U+6C35.0	fFourCorner	3010
U+6C35.0	fCantonese	seoi2
U+6C35.0	fMandarin	san1dian3shui3
U+6C35.0	fEnglish	 water Kangxi radical number 85
U+6C36.0	fUTF8	氶
U+6C36.0	fCNS11643	2-2160
U+6C36.0	fBig5	C9A1
U+6C36.0	fGB	5-3038
U+6C36.0	fR/S	 85.1
U+6C36.0	fTotalStrokes	 5
U+6C36.0	fCangjie	NE
U+6C36.0	fFourCorner	17232
U+6C36.0	fCantonese	cing2; sing4; zing3
U+6C36.0	fMandarin	zheng3; cheng2; zheng4
U+6C37.0	fUTF8	氷
U+6C37.0	fCNS11643	3-2257
U+6C37.0	fGB	E-2F79
U+6C37.0	fR/S	  85.1
U+6C37.0	fTotalStrokes	 5
U+6C37.0	fCangjie	IE
U+6C37.0	fFourCorner	32230
U+6C37.0	fHakka	 ben1
U+6C37.0	fCantonese	bing1; jing4
U+6C37.0	fMandarin	bing1
U+6C37.0	fEnglish	 ice; frost; icicles cold
U+6C38.0	fUTF8	永
U+6C38.0	fCNS11643	1-4663
U+6C38.0	fBig5	A5C3
U+6C38.0	fGB	0-5340
U+6C38.0	fR/S	 85.1
U+6C38.0	fTotalStrokes	 5
U+6C38.0	fCangjie	INE
U+6C38.0	fFourCorner	30232
U+6C38.0	fHakka	  jun1; jun3; run3
U+6C38.0	fCantonese	wing5
U+6C38.0	fMandarin	yong3; yong4
U+6C38.0	fEnglish	long in time; perpetual; eternal; forever; everlasting; permanent
U+6C39.0	fUTF8	氹
U+6C39.0	fCNS11643	3-2254
U+6C39.0	fGB	5-5460
U+6C39.0	fR/S	 85.1; 5.5; 85.4; 5.4
U+6C39.0	fTotalStrokes	 5
U+6C39.0	fCangjie	NE
U+6C39.0	fFourCorner	12710
U+6C39.0	fHakka	 tun3; tam3
U+6C39.0	fCantonese	tam3; dam4; tam5
U+6C39.0	fMandarin	gan1; dang4
U+6C39.0	fEnglish	 ditch; pool ; puddle; cesspool; hole
U+6C3A.0	fUTF8	氺
U+6C3A.0	fGB	E-2F7A
U+6C3A.0	fR/S	 85.1
U+6C3A.0	fTotalStrokes	 5
U+6C3A.0	fCangjie	XXFF
U+6C3A.0	fFourCorner	3213
U+6C3A.0	fCantonese	seoi2
U+6C3A.0	fMandarin	shui3
U+6C3B.0	fUTF8	氻
U+6C3B.0	fCNS11643	2-2163
U+6C3B.0	fBig5	C9A4
U+6C3B.0	fGB	5-546E
U+6C3B.0	fR/S	 85.2
U+6C3B.0	fTotalStrokes	 5
U+6C3B.0	fCangjie	EKS
U+6C3B.0	fFourCorner	34127
U+6C3B.0	fCantonese	lak6
U+6C3B.0	fMandarin	le4
U+6C3C.0	fUTF8	氼
U+6C3C.0	fCNS11643	3-2421
U+6C3C.0	fGB	5-5461
U+6C3C.0	fR/S	  85.2
U+6C3C.0	fTotalStrokes	 6
U+6C3C.0	fCangjie	EO
U+6C3C.0	fFourCorner	32800
U+6C3C.0	fHakka	  nit8; ni5; ni6
U+6C3C.0	fCantonese	nik6; lik6 (also)
U+6C3C.0	fMandarin	ni4
U+6C3C.0	fEnglish	 sink drown to be given over to
U+6C3D.0	fUTF8	氽
U+6C3D.0	fGB	0-595B
U+6C3D.0	fR/S	 9.4; 85.2
U+6C3D.0	fTotalStrokes	 6
U+6C3D.0	fCangjie	OE
U+6C3D.0	fFourCorner	8023
U+6C3D.0	fHakka	 ciu2;  tun3; siu2
U+6C3D.0	fCantonese	dan2; tan1 (also); tan2 (also)
U+6C3D.0	fMandarin	tun3; niao4; qiu2
U+6C3D.0	fEnglish	the action of water moving or pushing an object
U+6C3D.1	fUTF8	氽
U+6C3D.1	fGB	0-595B
U+6C3D.1	fR/S	 9.4; 85.2
U+6C3D.1	fTotalStrokes	 6
U+6C3D.1	fCangjie	OE
U+6C3D.1	fFourCorner	8023
U+6C3D.1	fMandarin	cuan1
U+6C3D.1	fEnglish	 boil (food, cooking)
U+6C3E.0	fUTF8	氾
U+6C3E.0	fCNS11643	1-4666
U+6C3E.0	fBig5	A5C6
U+6C3E.0	fGB	8-2F6A
U+6C3E.0	fR/S	 85.2
U+6C3E.0	fTotalStrokes	 5
U+6C3E.0	fCangjie	ESU
U+6C3E.0	fFourCorner	37112
U+6C3E.0	fHakka	  fam5; fam6
U+6C3E.0	fCantonese	faan4; faan3; faan6 (also)
U+6C3E.0	fMandarin	fan4; fan2; feng3
U+6C3E.0	fEnglish	[1] spread; fill everywhere [2] extensive; vast; boundless [3] floating
U+6C3F.0	fUTF8	氿
U+6C3F.0	fCNS11643	2-2162
U+6C3F.0	fBig5	C9A3
U+6C3F.0	fGB	3-532F
U+6C3F.0	fR/S	 85.2
U+6C3F.0	fTotalStrokes	 5
U+6C3F.0	fCangjie	EKN
U+6C3F.0	fFourCorner	34117
U+6C3F.0	fHakka	 gui3; gwi3; kwui3
U+6C3F.0	fCantonese	gwai2; gau2; gwai1
U+6C3F.0	fMandarin	gui3; qiu2
U+6C3F.0	fEnglish	spring
U+6C40.0	fUTF8	汀
U+6C40.0	fCNS11643	1-4665
U+6C40.0	fBig5	A5C5
U+6C40.0	fGB	0-4D21
U+6C40.0	fR/S	 85.2
U+6C40.0	fTotalStrokes	 5
U+6C40.0	fCangjie	EMN
U+6C40.0	fFourCorner	31120
U+6C40.0	fHakka	 tin1;  ten1;  din5; din2; tin3; tin5
U+6C40.0	fCantonese	ting1; ding1 (also)
U+6C40.0	fMandarin	ting1; ting4; ding4; ting3
U+6C40.0	fEnglish	[1] low; level land along a river; beach; bank; shore [2] shoal
U+6C41.0	fUTF8	汁
U+6C41.0	fCNS11643	1-4664
U+6C41.0	fBig5	A5C4
U+6C41.0	fGB	0-562D
U+6C41.0	fR/S	 85.2
U+6C41.0	fTotalStrokes	 5
U+6C41.0	fCangjie	EJ
U+6C41.0	fFourCorner	34100
U+6C41.0	fHakka	 zip7;  chip7; tsiip7; zhip7
U+6C41.0	fCantonese	zap1
U+6C41.0	fMandarin	zhi1; xie2; shi2; shi1
U+6C41.0	fEnglish	juice; natural fluid; sap; gravy
U+6C42.0	fUTF8	求
U+6C42.0	fCNS11643	1-4A65
U+6C42.0	fBig5	A844
U+6C42.0	fGB	0-4773
U+6C42.0	fR/S	 85.2
U+6C42.0	fTotalStrokes	 6
U+6C42.0	fCangjie	IJE
U+6C42.0	fFourCorner	43132
U+6C42.0	fHakka	  kiu2; k'ieu2; kiu3
U+6C42.0	fCantonese	kau4
U+6C42.0	fMandarin	qiu2; ju1
U+6C42.0	fEnglish	[1] solicit; ask for [2] seek [3] demand; request; beseech [4] covet; desire
U+6C43.0	fUTF8	汃
U+6C43.0	fCNS11643	2-2161
U+6C43.0	fBig5	C9A2
U+6C43.0	fGB	3-532E
U+6C43.0	fR/S	 85.2
U+6C43.0	fTotalStrokes	 5
U+6C43.0	fCangjie	EC
U+6C43.0	fFourCorner	38100
U+6C43.0	fCantonese	ban1; paak3; paat3 (also)
U+6C43.0	fMandarin	bin1; pa4
U+6C44.0	fUTF8	汄
U+6C44.0	fCNS11643	3-2258
U+6C44.0	fGB	E-2F7B
U+6C44.0	fR/S	 85.2
U+6C44.0	fTotalStrokes	 5
U+6C44.0	fCangjie	EO
U+6C44.0	fFourCorner	38100
U+6C44.0	fCantonese	zak1
U+6C44.0	fMandarin	ze4
U+6C45.0	fUTF8	汅
U+6C45.0	fCNS11643	3-2259
U+6C45.0	fGB	E-2F7C
U+6C45.0	fR/S	  85.2
U+6C45.0	fTotalStrokes	 5
U+6C45.0	fCangjie	EMVS
U+6C45.0	fFourCorner	31120
U+6C45.0	fCantonese	min5
U+6C45.0	fMandarin	mian3
U+6C46.0	fUTF8	汆
U+6C46.0	fCNS11643	2-2259
U+6C46.0	fBig5	C9F8
U+6C46.0	fGB	0-5960
U+6C46.0	fR/S	 85.2
U+6C46.0	fTotalStrokes	 6
U+6C46.0	fCangjie	OE
U+6C46.0	fFourCorner	80232
U+6C46.0	fHakka	 tun3; con1
U+6C46.0	fCantonese	cyun1; mei6; tan2
U+6C46.0	fMandarin	cuan1; tun3
U+6C46.0	fEnglish	 parboil hot water kettle boil
U+6C47.0	fUTF8	汇
U+6C47.0	fGB	0-3B63
U+6C47.0	fR/S	 85.2
U+6C47.0	fTotalStrokes	 5
U+6C47.0	fCangjie	ES
U+6C47.0	fHakka	 fui5
U+6C47.0	fCantonese	wui6
U+6C47.0	fMandarin	hui4; hui3; kuai1; ma1
U+6C47.0	fEnglish	simplified form of &#x532F;
U+6C48.0	fUTF8	汈
U+6C48.0	fGB	3-5330
U+6C48.0	fR/S	 85.2
U+6C48.0	fTotalStrokes	 5
U+6C48.0	fCangjie	ESM
U+6C48.0	fCantonese	diu1
U+6C48.0	fMandarin	diao1
U+6C49.0	fUTF8	汉
U+6C49.0	fCNS11643	F-2166
U+6C49.0	fGB	0-3A3A
U+6C49.0	fR/S	 85.2
U+6C49.0	fTotalStrokes	 5
U+6C49.0	fCangjie	EE
U+6C49.0	fFourCorner	3714
U+6C49.0	fHakka	 hon5
U+6C49.0	fCantonese	hon3
U+6C49.0	fMandarin	han4; yi4; tan1
U+6C49.0	fEnglish	simplified form of &#x6F22;
U+6C4A.0	fUTF8	汊
U+6C4A.0	fCNS11643	2-225D
U+6C4A.0	fBig5	C9FC
U+6C4A.0	fGB	0-6362
U+6C4A.0	fR/S	 85.3
U+6C4A.0	fTotalStrokes	 6
U+6C4A.0	fCangjie	EEI
U+6C4A.0	fFourCorner	37140
U+6C4A.0	fHakka	 ca5; ca1
U+6C4A.0	fCantonese	caa3; caa1
U+6C4A.0	fMandarin	cha4
U+6C4A.0	fEnglish	 branching stream
U+6C4B.0	fUTF8	汋
U+6C4B.0	fCNS11643	2-225F
U+6C4B.0	fBig5	C9FE
U+6C4B.0	fGB	3-5336
U+6C4B.0	fR/S	 85.3
U+6C4B.0	fTotalStrokes	 6
U+6C4B.0	fCangjie	EPI
U+6C4B.0	fFourCorner	37120
U+6C4B.0	fHakka	 sok7; ch'ok7; cok7; shok7; chok7; zhok7
U+6C4B.0	fCantonese	joek6; zoek3; coek3; soek3
U+6C4B.0	fMandarin	zhuo2; yue4; shuo4; yao4; chuo4
U+6C4B.0	fEnglish	 pour
U+6C4C.0	fUTF8	汌
U+6C4C.0	fCNS11643	2-2260
U+6C4C.0	fBig5	CA40
U+6C4C.0	fGB	3-5333
U+6C4C.0	fR/S	 85.3
U+6C4C.0	fTotalStrokes	 6
U+6C4C.0	fCangjie	ELLL
U+6C4C.0	fFourCorner	32100
U+6C4C.0	fCantonese	cyun3; zyun6
U+6C4C.0	fMandarin	chuan4
U+6C4D.0	fUTF8	汍
U+6C4D.0	fCNS11643	1-4846
U+6C4D.0	fBig5	A6C5
U+6C4D.0	fGB	3-5335
U+6C4D.0	fR/S	 85.3
U+6C4D.0	fTotalStrokes	 6
U+6C4D.0	fCangjie	EKNI
U+6C4D.0	fFourCorner	35117
U+6C4D.0	fHakka	 fan2; jen2; ren2
U+6C4D.0	fCantonese	jyun4
U+6C4D.0	fMandarin	wan2
U+6C4D.0	fEnglish	weep
U+6C4E.0	fUTF8	汎
U+6C4E.0	fCNS11643	1-4847
U+6C4E.0	fBig5	A6C6
U+6C4E.0	fGB	E-2F7D
U+6C4E.0	fR/S	 85.3
U+6C4E.0	fTotalStrokes	 6
U+6C4E.0	fCangjie	EHNI
U+6C4E.0	fFourCorner	37110
U+6C4E.0	fHakka	  fam5; fam6
U+6C4E.0	fCantonese	faan4; faan3; faan6 (also)
U+6C4E.0	fMandarin	fan4; feng2; feng3; fa2
U+6C4E.0	fEnglish	[1] float; drift; afloat [2] extensive; widespread
U+6C4F.0	fUTF8	汏
U+6C4F.0	fCNS11643	2-225C
U+6C4F.0	fBig5	C9FB
U+6C4F.0	fGB	3-5331
U+6C4F.0	fR/S	 85.3
U+6C4F.0	fTotalStrokes	 6
U+6C4F.0	fCangjie	EK
U+6C4F.0	fFourCorner	34130
U+6C4F.0	fCantonese	daai6; taai3
U+6C4F.0	fMandarin	dai4; ta4; tuo2; tai4
U+6C4F.1	fUTF8	汏
U+6C4F.1	fCNS11643	2-225C
U+6C4F.1	fBig5	C9FB
U+6C4F.1	fGB	3-5331
U+6C4F.1	fR/S	 85.3
U+6C4F.1	fTotalStrokes	 6
U+6C4F.1	fCangjie	EK
U+6C4F.1	fFourCorner	34130
U+6C4F.1	fMandarin	da4
U+6C4F.1	fEnglish	 wash
U+6C50.0	fUTF8	汐
U+6C50.0	fCNS11643	1-4842
U+6C50.0	fBig5	A6C1
U+6C50.0	fGB	0-4F2B
U+6C50.0	fR/S	 85.3
U+6C50.0	fTotalStrokes	 6
U+6C50.0	fCangjie	ENI
U+6C50.0	fFourCorner	37120
U+6C50.0	fHakka	 sip8
U+6C50.0	fCantonese	zik6
U+6C50.0	fMandarin	xi1; xi4
U+6C50.0	fEnglish	the flow of the tides at night; evening ebbtide
U+6C51.0	fUTF8	汑
U+6C51.0	fCNS11643	3-237D
U+6C51.0	fGB	E-2F7E
U+6C51.0	fR/S	  85.3
U+6C51.0	fTotalStrokes	 6
U+6C51.0	fCangjie	EHP
U+6C51.0	fFourCorner	32110
U+6C51.0	fCantonese	tok3
U+6C51.0	fMandarin	tuo1
U+6C52.0	fUTF8	汒
U+6C52.0	fCNS11643	2-225A
U+6C52.0	fBig5	C9F9
U+6C52.0	fGB	5-5470
U+6C52.0	fR/S	 85.3
U+6C52.0	fTotalStrokes	 6
U+6C52.0	fCangjie	EYV
U+6C52.0	fFourCorner	30110
U+6C52.0	fHakka	  mong2
U+6C52.0	fCantonese	mong4
U+6C52.0	fMandarin	mang2; mang3; mang4
U+6C52.0	fEnglish	wide; expansive
U+6C53.0	fUTF8	汓
U+6C53.0	fCNS11643	3-2425
U+6C53.0	fGB	5-5471
U+6C53.0	fR/S	 85.3
U+6C53.0	fTotalStrokes	 6
U+6C53.0	fCangjie	END
U+6C53.0	fFourCorner	37140
U+6C53.0	fCantonese	cau4
U+6C53.0	fMandarin	qiu2; you2
U+6C54.0	fUTF8	汔
U+6C54.0	fCNS11643	2-225E
U+6C54.0	fBig5	C9FD
U+6C54.0	fGB	0-6360
U+6C54.0	fR/S	 85.3
U+6C54.0	fTotalStrokes	 6
U+6C54.0	fCangjie	EON
U+6C54.0	fFourCorner	38117
U+6C54.0	fHakka	  hit7;  ngiet7; hi5; nget7
U+6C54.0	fCantonese	ngat6; at6 (also); hat1
U+6C54.0	fMandarin	qi4
U+6C54.0	fEnglish	[1] water dried up [2] shed tears [3] dangerous
U+6C55.0	fUTF8	汕
U+6C55.0	fCNS11643	1-4843
U+6C55.0	fBig5	A6C2
U+6C55.0	fGB	0-4947
U+6C55.0	fR/S	 85.3
U+6C55.0	fTotalStrokes	 6
U+6C55.0	fCangjie	EU
U+6C55.0	fFourCorner	32170
U+6C55.0	fHakka	 san5;  san1
U+6C55.0	fCantonese	saan3
U+6C55.0	fMandarin	shan4; chan3; shuan4; shan1
U+6C55.0	fEnglish	a basket for catching fish
U+6C56.0	fUTF8	汖
U+6C56.0	fCNS11643	3-2655
U+6C56.0	fGB	E-3021
U+6C56.0	fR/S	 85.3
U+6C56.0	fTotalStrokes	 7
U+6C56.0	fCangjie	UE
U+6C56.0	fFourCorner	22230
U+6C56.0	fCantonese	pan3
U+6C56.0	fMandarin	pai4; chi2
U+6C57.0	fUTF8	汗
U+6C57.0	fCNS11643	1-483E
U+6C57.0	fBig5	A6BD
U+6C57.0	fGB	0-3A39
U+6C57.0	fR/S	 85.3
U+6C57.0	fTotalStrokes	 6
U+6C57.0	fCangjie	EMJ
U+6C57.0	fFourCorner	31140
U+6C57.0	fHakka	 hon5; hon6; hon3
U+6C57.0	fCantonese	hon6; hon4; hong6
U+6C57.0	fMandarin	han4; gan1
U+6C57.0	fEnglish	 perspiration; sweat
U+6C57.1	fUTF8	汗
U+6C57.1	fCNS11643	1-483E
U+6C57.1	fBig5	A6BD
U+6C57.1	fGB	0-3A39
U+6C57.1	fR/S	 85.3
U+6C57.1	fTotalStrokes	 6
U+6C57.1	fCangjie	EMJ
U+6C57.1	fFourCorner	31140
U+6C57.1	fMandarin	han2
U+6C57.1	fEnglish	used in combination for khan
U+6C58.0	fUTF8	汘
U+6C58.0	fCNS11643	3-237C
U+6C58.0	fGB	3-5332
U+6C58.0	fR/S	 85.3
U+6C58.0	fTotalStrokes	 6
U+6C58.0	fCangjie	EHJ
U+6C58.0	fFourCorner	32140
U+6C58.0	fCantonese	cim4
U+6C58.0	fMandarin	qian1
U+6C59.0	fUTF8	汙
U+6C59.0	fCNS11643	1-483F
U+6C59.0	fBig5	A6BE
U+6C59.0	fGB	E-3022
U+6C59.0	fR/S	 85.3
U+6C59.0	fTotalStrokes	 6
U+6C59.0	fCangjie	EMD
U+6C59.0	fFourCorner	31140
U+6C59.0	fHakka	  wu1; hon6; vu1
U+6C59.0	fCantonese	waa1; wu1; wui1; jyu1
U+6C59.0	fMandarin	wu1; wu4; wa1; yu2; yu1
U+6C59.0	fEnglish	filthy; dirty; impure; polluted
U+6C5A.0	fUTF8	汚
U+6C5A.0	fCNS11643	3-2423
U+6C5A.0	fGB	E-3023
U+6C5A.0	fR/S	 85.3
U+6C5A.0	fTotalStrokes	 6
U+6C5A.0	fCangjie	EMJS
U+6C5A.0	fFourCorner	31120
U+6C5A.0	fHakka	 wu1; vu1
U+6C5A.0	fCantonese	wu1; wu3; waa1; jyu1
U+6C5A.0	fMandarin	wu1; wu4; wa1; yu2; yu1
U+6C5A.0	fEnglish	filthy; dirty; impure; polluted
U+6C5B.0	fUTF8	汛
U+6C5B.0	fCNS11643	1-4845
U+6C5B.0	fBig5	A6C4
U+6C5B.0	fGB	0-5134
U+6C5B.0	fR/S	 85.3
U+6C5B.0	fTotalStrokes	 6
U+6C5B.0	fCangjie	ENJ
U+6C5B.0	fFourCorner	37110
U+6C5B.0	fHakka	 sin3; sin5; jan2
U+6C5B.0	fCantonese	seon3
U+6C5B.0	fMandarin	xun4; shai4; xian4; sa3
U+6C5B.0	fEnglish	[1] sprinkle [2] abundant water; high water; flood [3] menses; menstruation
U+6C5C.0	fUTF8	汜
U+6C5C.0	fCNS11643	2-225B
U+6C5C.0	fBig5	C9FA
U+6C5C.0	fGB	0-6361
U+6C5C.0	fR/S	 85.3
U+6C5C.0	fTotalStrokes	 6
U+6C5C.0	fCangjie	ERU
U+6C5C.0	fFourCorner	37117
U+6C5C.0	fHakka	 sii5; sii6
U+6C5C.0	fCantonese	ci5
U+6C5C.0	fMandarin	si4; yi3
U+6C5C.0	fEnglish	a stream which leaves the main branch then later returns; a stream that branches and afterwards merges again
U+6C5D.0	fUTF8	汝
U+6C5D.0	fCNS11643	1-483D
U+6C5D.0	fBig5	A6BC
U+6C5D.0	fGB	0-486A
U+6C5D.0	fR/S	 85.3
U+6C5D.0	fTotalStrokes	 6
U+6C5D.0	fCangjie	EV
U+6C5D.0	fFourCorner	34140
U+6C5D.0	fHakka	  ni3; ji3; ni1
U+6C5D.0	fCantonese	jyu5
U+6C5D.0	fMandarin	ru3
U+6C5D.0	fEnglish	[1] you; thou; thee; thy [2] you
U+6C5D.1	fUTF8	汝
U+6C5D.1	fCNS11643	1-483D
U+6C5D.1	fBig5	A6BC
U+6C5D.1	fGB	0-486A
U+6C5D.1	fR/S	 85.3
U+6C5D.1	fTotalStrokes	 6
U+6C5D.1	fCangjie	EV
U+6C5D.1	fFourCorner	34140
U+6C5E.0	fUTF8	汞
U+6C5E.0	fCNS11643	1-4A66
U+6C5E.0	fBig5	A845
U+6C5E.0	fGB	0-392F
U+6C5E.0	fR/S	 85.3
U+6C5E.0	fTotalStrokes	 7
U+6C5E.0	fCangjie	ME
U+6C5E.0	fFourCorner	10232
U+6C5E.0	fHakka	  fung3; gung5
U+6C5E.0	fCantonese	hung3; hung6
U+6C5E.0	fMandarin	gong3; hong4
U+6C5E.0	fEnglish	mercury (chemistry)
U+6C5F.0	fUTF8	江
U+6C5F.0	fCNS11643	1-4840
U+6C5F.0	fBig5	A6BF
U+6C5F.0	fGB	0-3D2D
U+6C5F.0	fR/S	 85.3
U+6C5F.0	fTotalStrokes	 6
U+6C5F.0	fCangjie	EM
U+6C5F.0	fFourCorner	31110
U+6C5F.0	fHakka	  gong1
U+6C5F.0	fCantonese	gong1
U+6C5F.0	fMandarin	jiang1
U+6C5F.0	fEnglish	[1] large river [2] the Yangtze river [3] a Chinese family name
U+6C60.0	fUTF8	池
U+6C60.0	fCNS11643	1-4841
U+6C60.0	fBig5	A6C0
U+6C60.0	fGB	0-3358
U+6C60.0	fR/S	 85.3
U+6C60.0	fTotalStrokes	 6
U+6C60.0	fCangjie	EPD
U+6C60.0	fFourCorner	34112
U+6C60.0	fHakka	 ci2;  chi2; cih2; chi3; chii2
U+6C60.0	fCantonese	ci4; to4
U+6C60.0	fMandarin	chi2; tuo2; zhe2; che4
U+6C60.0	fEnglish	[1] pool; pond; moat [2] an enclosed space with raised sides [3] a Chinese family name
U+6C61.0	fUTF8	污
U+6C61.0	fCNS11643	1-4844
U+6C61.0	fBig5	A6C3
U+6C61.0	fGB	0-4E5B
U+6C61.0	fR/S	 85.3
U+6C61.0	fTotalStrokes	 6
U+6C61.0	fCangjie	EMMS
U+6C61.0	fFourCorner	31127
U+6C61.0	fHakka	 wu1; wu5
U+6C61.0	fCantonese	wu1; wu3; waa1; hyu1
U+6C61.0	fMandarin	wu1; wu4; wa1; yu2; yu1
U+6C61.0	fEnglish	[1] filthy; dirty; impure; polluted [2] stain; mar; insult; slander [3] corrupt
U+6C62.0	fUTF8	汢
U+6C62.0	fCNS11643	F-225D
U+6C62.0	fGB	E-3024
U+6C62.0	fR/S	 85.3
U+6C62.0	fTotalStrokes	 6
U+6C62.0	fCangjie	XXEG
U+6C62.0	fFourCorner	3411
U+6C62.0	fCantonese	tou2
U+6C63.0	fUTF8	汣
U+6C63.0	fCNS11643	3-653C
U+6C63.0	fGB	E-3025
U+6C63.0	fR/S	 85.3
U+6C63.0	fTotalStrokes	 6
U+6C63.0	fCangjie	XENO
U+6C63.0	fFourCorner	3718
U+6C63.0	fCantonese	gwai2
U+6C64.0	fUTF8	汤
U+6C64.0	fGB	0-4C40
U+6C64.0	fR/S	 85.3
U+6C64.0	fTotalStrokes	 6
U+6C64.0	fCangjie	ENSH
U+6C64.0	fHakka	 tong1; tong5
U+6C64.0	fCantonese	tong1; soeng1; tong3; joeng4
U+6C64.0	fMandarin	tang1; yang2; tang3; tang4; shang1
U+6C64.0	fEnglish	simplified form of &#x6E6F;
U+6C64.1	fUTF8	汤
U+6C64.1	fGB	0-4C40
U+6C64.1	fR/S	 85.3
U+6C64.1	fTotalStrokes	 6
U+6C64.1	fCangjie	ENSH
U+6C64.1	fMandarin	tang5
U+6C65.0	fUTF8	汥
U+6C65.0	fCNS11643	2-245B
U+6C65.0	fBig5	CB5B
U+6C65.0	fGB	3-5340
U+6C65.0	fR/S	 85.4
U+6C65.0	fTotalStrokes	 7
U+6C65.0	fCangjie	EJE
U+6C65.0	fFourCorner	34147
U+6C65.0	fCantonese	zi1; kei3
U+6C65.0	fMandarin	zhi1; qi2; ji4; bei4
U+6C66.0	fUTF8	汦
U+6C66.0	fCNS11643	2-2459
U+6C66.0	fBig5	CB59
U+6C66.0	fGB	3-5351
U+6C66.0	fR/S	 85.4
U+6C66.0	fTotalStrokes	 7
U+6C66.0	fCangjie	EHVP
U+6C66.0	fFourCorner	32140
U+6C66.0	fCantonese	zi2
U+6C66.0	fMandarin	chi2; zhi3; zhi1
U+6C67.0	fUTF8	汧
U+6C67.0	fCNS11643	2-244C
U+6C67.0	fBig5	CB4C
U+6C67.0	fGB	3-5339
U+6C67.0	fR/S	 85.4
U+6C67.0	fTotalStrokes	 7
U+6C67.0	fCangjie	EMT
U+6C67.0	fFourCorner	31140
U+6C67.0	fCantonese	hin1
U+6C67.0	fMandarin	qian1
U+6C67.0	fEnglish	 name of a river in Shangtung marsh float
U+6C68.0	fUTF8	汨
U+6C68.0	fCNS11643	1-4A72
U+6C68.0	fBig5	A851
U+6C68.0	fGB	0-6368
U+6C68.0	fR/S	 85.4
U+6C68.0	fTotalStrokes	 7
U+6C68.0	fCangjie	EA
U+6C68.0	fFourCorner	36100
U+6C68.0	fHakka	 met7; gut7; mi5; mi6; gwut7
U+6C68.0	fCantonese	mik6; jat6
U+6C68.0	fMandarin	mi4; gu3
U+6C68.0	fEnglish	Mi, a river in Hunan province where Qu Yuan drowned himself
U+6C69.0	fUTF8	汩
U+6C69.0	fCNS11643	2-2453
U+6C69.0	fBig5	CB53
U+6C69.0	fGB	0-6369
U+6C69.0	fR/S	 85.4
U+6C69.0	fTotalStrokes	 7
U+6C69.0	fCangjie	EA
U+6C69.0	fFourCorner	36100
U+6C69.0	fHakka	 gut7; zat7; jat7; ze1; je1; jet7
U+6C69.0	fCantonese	gwat1; wat6; mik6; jat6
U+6C69.0	fMandarin	gu3; hu2
U+6C69.0	fEnglish	[1] dredge (channel, etc.) [2] confused; disorderly [3] the sound of waves [4] a Chinese family name
U+6C69.1	fUTF8	汩
U+6C69.1	fCNS11643	2-2453
U+6C69.1	fBig5	CB53
U+6C69.1	fGB	0-6369
U+6C69.1	fR/S	 85.4
U+6C69.1	fTotalStrokes	 7
U+6C69.1	fCangjie	EA
U+6C69.1	fFourCorner	36100
U+6C69.1	fMandarin	yu4
U+6C69.1	fEnglish	rapid; fleeting
U+6C6A.0	fUTF8	汪
U+6C6A.0	fCNS11643	1-4A6D
U+6C6A.0	fBig5	A84C
U+6C6A.0	fGB	0-4D74
U+6C6A.0	fR/S	 85.4
U+6C6A.0	fTotalStrokes	 7
U+6C6A.0	fCangjie	EMG
U+6C6A.0	fFourCorner	31114
U+6C6A.0	fHakka	  wong1; wong5; vong1; mong1
U+6C6A.0	fCantonese	wong1
U+6C6A.0	fMandarin	wang1; wang3; hong2; wang4
U+6C6A.0	fEnglish	[1] deep and extensive (water) [2] puddle [3] soak (liquid); collect; accumulate [4] a Chinese family name
U+6C6B.0	fUTF8	汫
U+6C6B.0	fCNS11643	2-244D
U+6C6B.0	fBig5	CB4D
U+6C6B.0	fGB	3-533A
U+6C6B.0	fR/S	 85.4
U+6C6B.0	fTotalStrokes	 7
U+6C6B.0	fCangjie	ETT
U+6C6B.0	fFourCorner	35100
U+6C6B.0	fCantonese	zeng2
U+6C6B.0	fMandarin	qing4; jing4
U+6C6C.0	fUTF8	汬
U+6C6C.0	fCNS11643	3-2964
U+6C6C.0	fGB	5-5463
U+6C6C.0	fR/S	  85.4
U+6C6C.0	fTotalStrokes	 8; 7
U+6C6C.0	fCangjie	TTE
U+6C6C.0	fFourCorner	55230
U+6C6C.0	fCantonese	zing6
U+6C6C.0	fMandarin	jing3
U+6C6D.0	fUTF8	汭
U+6C6D.0	fCNS11643	2-2455
U+6C6D.0	fBig5	CB55
U+6C6D.0	fGB	3-5349
U+6C6D.0	fR/S	 85.4
U+6C6D.0	fTotalStrokes	 7
U+6C6D.0	fCangjie	EOB
U+6C6D.0	fFourCorner	34127
U+6C6D.0	fHakka	 nui5; nui6; jui5; rui6; jui1
U+6C6D.0	fCantonese	jeoi6
U+6C6D.0	fMandarin	rui4; ruo4; tun1
U+6C6D.0	fEnglish	 bend in a stream
U+6C6E.0	fUTF8	汮
U+6C6E.0	fCNS11643	3-2650
U+6C6E.0	fGB	5-547B
U+6C6E.0	fR/S	  85.4
U+6C6E.0	fTotalStrokes	 7
U+6C6E.0	fCangjie	EPIM
U+6C6E.0	fFourCorner	37120
U+6C6E.0	fCantonese	gwan1
U+6C6E.0	fMandarin	jun1; lun2; yun2; ying2; jun4
U+6C6F.0	fUTF8	汯
U+6C6F.0	fCNS11643	2-2452
U+6C6F.0	fBig5	CB52
U+6C6F.0	fGB	3-5344
U+6C6F.0	fR/S	 85.4
U+6C6F.0	fTotalStrokes	 7
U+6C6F.0	fCangjie	EKI
U+6C6F.0	fFourCorner	34130
U+6C6F.0	fCantonese	wang4
U+6C6F.0	fMandarin	hong2
U+6C70.0	fUTF8	汰
U+6C70.0	fCNS11643	1-4A70
U+6C70.0	fBig5	A84F
U+6C70.0	fGB	0-4C2D
U+6C70.0	fR/S	 85.4
U+6C70.0	fTotalStrokes	 7
U+6C70.0	fCangjie	EKI
U+6C70.0	fFourCorner	34130
U+6C70.0	fHakka	 tai5; tai1; toi1
U+6C70.0	fCantonese	taai3
U+6C70.0	fMandarin	tai4; ti4; yi4; ta4
U+6C70.0	fEnglish	[1] excessive [2] sift; eliminate; remove
U+6C71.0	fUTF8	汱
U+6C71.0	fCNS11643	2-2451
U+6C71.0	fBig5	CB51
U+6C71.0	fGB	3-5342
U+6C71.0	fR/S	 85.4
U+6C71.0	fTotalStrokes	 7
U+6C71.0	fCangjie	EIK
U+6C71.0	fFourCorner	33134
U+6C71.0	fCantonese	gyun2
U+6C71.0	fMandarin	quan3
U+6C72.0	fUTF8	汲
U+6C72.0	fCNS11643	1-4A77
U+6C72.0	fBig5	A856
U+6C72.0	fGB	0-3C33
U+6C72.0	fR/S	 85.4
U+6C72.0	fTotalStrokes	 7
U+6C72.0	fCangjie	ENHE
U+6C72.0	fFourCorner	37147
U+6C72.0	fHakka	 kip7; gep7; kip8; gip7
U+6C72.0	fCantonese	kap1
U+6C72.0	fMandarin	ji1; ji2
U+6C72.0	fEnglish	[1] draw water or liquid [2] a Chinese family name
U+6C73.0	fUTF8	汳
U+6C73.0	fCNS11643	2-245A
U+6C73.0	fBig5	CB5A
U+6C73.0	fGB	3-534D
U+6C73.0	fR/S	 85.4
U+6C73.0	fTotalStrokes	 7
U+6C73.0	fCangjie	EHE
U+6C73.0	fFourCorner	31147
U+6C73.0	fCantonese	bin6
U+6C73.0	fMandarin	bian4; fan3
U+6C74.0	fUTF8	汴
U+6C74.0	fCNS11643	1-4A79
U+6C74.0	fBig5	A858
U+6C74.0	fGB	0-636A
U+6C74.0	fR/S	 85.4
U+6C74.0	fTotalStrokes	 7
U+6C74.0	fCangjie	EYY
U+6C74.0	fFourCorner	30130
U+6C74.0	fHakka	 pien3; pen3; ben1
U+6C74.0	fCantonese	bin6
U+6C74.0	fMandarin	bian4; fan4
U+6C74.0	fEnglish	[1] the name of a river in Honan [2] another name for Honan
U+6C75.0	fUTF8	汵
U+6C75.0	fCNS11643	3-2653
U+6C75.0	fGB	3-5350
U+6C75.0	fR/S	 85.4
U+6C75.0	fTotalStrokes	 7
U+6C75.0	fCangjie	EOIN
U+6C75.0	fFourCorner	38120
U+6C75.0	fCantonese	gam3
U+6C75.0	fMandarin	gan4; han2; cen2
U+6C76.0	fUTF8	汶
U+6C76.0	fCNS11643	1-4A7B
U+6C76.0	fBig5	A85A
U+6C76.0	fGB	0-636B
U+6C76.0	fR/S	 85.4
U+6C76.0	fTotalStrokes	 7
U+6C76.0	fCangjie	EYK
U+6C76.0	fFourCorner	30140
U+6C76.0	fHakka	 wun3; wun2; wun6; vun3; wun5; lun3
U+6C76.0	fCantonese	man6; man4
U+6C76.0	fMandarin	wen4; wen2; men2; min2; wen3
U+6C76.0	fEnglish	the name of a river in Shantung province
U+6C77.0	fUTF8	汷
U+6C77.0	fCNS11643	3-237E
U+6C77.0	fGB	3-5334
U+6C77.0	fR/S	 85.4; 85.3
U+6C77.0	fTotalStrokes	 6; 7
U+6C77.0	fCangjie	EHE
U+6C77.0	fFourCorner	37140
U+6C77.0	fCantonese	zung1
U+6C77.0	fMandarin	zhong1
U+6C78.0	fUTF8	汸
U+6C78.0	fCNS11643	2-244B
U+6C78.0	fBig5	CB4B
U+6C78.0	fGB	3-5356
U+6C78.0	fR/S	 85.4
U+6C78.0	fTotalStrokes	 7
U+6C78.0	fCangjie	EYHS
U+6C78.0	fFourCorner	30127
U+6C78.0	fHakka	 pong2; fong1
U+6C78.0	fCantonese	fong1; pong4 (also); pong1
U+6C78.0	fMandarin	fang1
U+6C79.0	fUTF8	汹
U+6C79.0	fCNS11643	3-2657
U+6C79.0	fGB	0-505A
U+6C79.0	fR/S	  85.4
U+6C79.0	fTotalStrokes	 7
U+6C79.0	fCangjie	EUK
U+6C79.0	fFourCorner	32170
U+6C79.0	fHakka	 hiung1
U+6C79.0	fCantonese	hung1
U+6C79.0	fMandarin	xiong1
U+6C79.0	fEnglish	 turbulent noisy; restless
U+6C7A.0	fUTF8	決
U+6C7A.0	fCNS11643	1-4A6E
U+6C7A.0	fBig5	A84D
U+6C7A.0	fGB	E-3026
U+6C7A.0	fR/S	 85.4
U+6C7A.0	fTotalStrokes	 7
U+6C7A.0	fCangjie	EDK
U+6C7A.0	fFourCorner	35130
U+6C7A.0	fHakka	 giet7; get7; kiat7
U+6C7A.0	fCantonese	kyut3; hyut3
U+6C7A.0	fMandarin	jue2; yue4; gui4
U+6C7A.0	fEnglish	[1] decide; determine; conclude; judge [2] brust; break (dike) [3] certain; sure; definite [4] put someone to death; execute a person
U+6C7B.0	fUTF8	汻
U+6C7B.0	fCNS11643	2-245C
U+6C7B.0	fBig5	CB5C
U+6C7B.0	fGB	3-534B
U+6C7B.0	fR/S	 85.4
U+6C7B.0	fTotalStrokes	 7
U+6C7B.0	fCangjie	EOJ
U+6C7B.0	fFourCorner	38140
U+6C7B.0	fCantonese	wu2; gon1; hong5; fu2
U+6C7B.0	fMandarin	hang3; hu3
U+6C7C.0	fUTF8	汼
U+6C7C.0	fCNS11643	3-2651
U+6C7C.0	fGB	3-534C
U+6C7C.0	fR/S	 85.4
U+6C7C.0	fTotalStrokes	 7
U+6C7C.0	fCangjie	EHQ
U+6C7C.0	fFourCorner	35100
U+6C7C.0	fCantonese	ngau4; au4 (also)
U+6C7C.0	fMandarin	niu2
U+6C7D.0	fUTF8	汽
U+6C7D.0	fCNS11643	1-4A75
U+6C7D.0	fBig5	A854
U+6C7D.0	fGB	0-467B
U+6C7D.0	fR/S	 85.4
U+6C7D.0	fTotalStrokes	 7
U+6C7D.0	fCangjie	EOMN
U+6C7D.0	fFourCorner	38117
U+6C7D.0	fHakka	  hi5;  ki5
U+6C7D.0	fCantonese	hei3
U+6C7D.0	fMandarin	qi4; yi3; gai1; gai4
U+6C7D.0	fEnglish	 steam; vapor; gas
U+6C7E.0	fUTF8	汾
U+6C7E.0	fCNS11643	1-4A78
U+6C7E.0	fBig5	A857
U+6C7E.0	fGB	0-375A
U+6C7E.0	fR/S	 85.4
U+6C7E.0	fTotalStrokes	 7
U+6C7E.0	fCangjie	ECSH
U+6C7E.0	fFourCorner	38127
U+6C7E.0	fHakka	  fun1; fun2
U+6C7E.0	fCantonese	fan4
U+6C7E.0	fMandarin	fen2; pen2; fen1
U+6C7E.0	fEnglish	the name of a tributary of the Yellow river in Shanxi province
U+6C7F.0	fUTF8	汿
U+6C7F.0	fCNS11643	3-264C
U+6C7F.0	fGB	5-5521
U+6C7F.0	fR/S	 85.4
U+6C7F.0	fTotalStrokes	 7
U+6C7F.0	fCangjie	ENIN
U+6C7F.0	fFourCorner	37120
U+6C7F.0	fCantonese	zeoi6
U+6C7F.0	fMandarin	xu4
U+6C80.0	fUTF8	沀
U+6C80.0	fCNS11643	2-2765
U+6C80.0	fBig5	CD45
U+6C80.0	fGB	3-5327
U+6C80.0	fR/S	 85.4
U+6C80.0	fTotalStrokes	 8
U+6C80.0	fCangjie	ENIN
U+6C80.0	fFourCorner	17222
U+6C80.0	fHakka	 si5
U+6C80.0	fCantonese	zeoi6; si3
U+6C80.0	fMandarin	xu3; xu4
U+6C80.0	fEnglish	a Chinese family name
U+6C81.0	fUTF8	沁
U+6C81.0	fCNS11643	1-4A68
U+6C81.0	fBig5	A847
U+6C81.0	fGB	0-475F
U+6C81.0	fR/S	 85.4
U+6C81.0	fTotalStrokes	 7
U+6C81.0	fCangjie	EP
U+6C81.0	fFourCorner	33100
U+6C81.0	fHakka	  sim5;  sim3; cim5; sim1
U+6C81.0	fCantonese	sam3
U+6C81.0	fMandarin	qin4; shen3
U+6C81.0	fEnglish	 soak; seep; percolate; permeate; ooze; exude
U+6C82.0	fUTF8	沂
U+6C82.0	fCNS11643	1-4B21
U+6C82.0	fBig5	A85E
U+6C82.0	fGB	0-524A
U+6C82.0	fR/S	 85.4
U+6C82.0	fTotalStrokes	 7
U+6C82.0	fCangjie	EHML
U+6C82.0	fFourCorner	32121
U+6C82.0	fHakka	 ki2; ngi2; ngiun2
U+6C82.0	fCantonese	ji4; ngan4; an4 (also); ji6
U+6C82.0	fMandarin	yi2; yin2
U+6C82.0	fEnglish	names of four rivers originating in Shantung
U+6C83.0	fUTF8	沃
U+6C83.0	fCNS11643	1-4A76
U+6C83.0	fBig5	A855
U+6C83.0	fGB	0-4E56
U+6C83.0	fR/S	 85.4
U+6C83.0	fTotalStrokes	 7
U+6C83.0	fCangjie	EHK
U+6C83.0	fFourCorner	32134
U+6C83.0	fHakka	 wok7; rau1; jeu1; rau5; jeu5; vok7; jau2
U+6C83.0	fCantonese	juk1
U+6C83.0	fMandarin	wo4; wu4; ao4
U+6C83.0	fEnglish	[1] water; irrigate [2] fertile (land)
U+6C84.0	fUTF8	沄
U+6C84.0	fCNS11643	2-244E
U+6C84.0	fBig5	CB4E
U+6C84.0	fGB	3-533D
U+6C84.0	fR/S	 85.4
U+6C84.0	fTotalStrokes	 7
U+6C84.0	fCangjie	EMMI
U+6C84.0	fFourCorner	31131
U+6C84.0	fHakka	 jun2
U+6C84.0	fCantonese	wan4
U+6C84.0	fMandarin	yun2; hun2; hun4
U+6C84.0	fEnglish	billows
U+6C85.0	fUTF8	沅
U+6C85.0	fCNS11643	1-4A6B
U+6C85.0	fBig5	A84A
U+6C85.0	fGB	0-6364
U+6C85.0	fR/S	 85.4
U+6C85.0	fTotalStrokes	 7
U+6C85.0	fCangjie	EMMU
U+6C85.0	fFourCorner	31111
U+6C85.0	fHakka	  ngien2; jen2
U+6C85.0	fCantonese	jyun4
U+6C85.0	fMandarin	yuan2; ruan3
U+6C85.0	fEnglish	the name of a river in western Honan that flows into Dongting lake
U+6C86.0	fUTF8	沆
U+6C86.0	fCNS11643	1-4A7A
U+6C86.0	fBig5	A859
U+6C86.0	fGB	0-636C
U+6C86.0	fR/S	 85.4
U+6C86.0	fTotalStrokes	 7
U+6C86.0	fCangjie	EYHN
U+6C86.0	fFourCorner	30117
U+6C86.0	fHakka	  kong5; hong2; hong3
U+6C86.0	fCantonese	hong4; hong5 (also); hong6
U+6C86.0	fMandarin	hang2; kang4
U+6C86.0	fEnglish	 ferry
U+6C86.1	fUTF8	沆
U+6C86.1	fCNS11643	1-4A7A
U+6C86.1	fBig5	A859
U+6C86.1	fGB	0-636C
U+6C86.1	fR/S	 85.4
U+6C86.1	fTotalStrokes	 7
U+6C86.1	fCangjie	EYHN
U+6C86.1	fFourCorner	30117
U+6C86.1	fMandarin	hang4
U+6C86.1	fEnglish	[1] a vast expanse of water [2] mist; fog [3] flowing
U+6C87.0	fUTF8	沇
U+6C87.0	fCNS11643	2-2456
U+6C87.0	fBig5	CB56
U+6C87.0	fGB	5-547E
U+6C87.0	fR/S	 85.4
U+6C87.0	fTotalStrokes	 7
U+6C87.0	fCangjie	EIHU
U+6C87.0	fFourCorner	33111
U+6C87.0	fHakka	 ren2; jen2; jan3; jen3
U+6C87.0	fCantonese	jin5; jin2 (also); wai5
U+6C87.0	fMandarin	yan3; yun3; wei3
U+6C87.0	fEnglish	flowing and engulfing; overflowing and brimming
U+6C88.0	fUTF8	沈
U+6C88.0	fCNS11643	1-4A69
U+6C88.0	fBig5	A848
U+6C88.0	fGB	0-4972
U+6C88.0	fR/S	 85.4
U+6C88.0	fTotalStrokes	 7
U+6C88.0	fCangjie	ELBU
U+6C88.0	fFourCorner	34112
U+6C88.0	fHakka	 sim3;  chim2; shim3; siim3; cihm2; cim2; chim5
U+6C88.0	fCantonese	sam2; cam4; zam3
U+6C88.0	fMandarin	chen2; tan2; zhen4; chen3
U+6C88.0	fEnglish	[1] sink; submerge; drawn deep into; set [2] indulge in; addicted to [3] deep; sound; fast (sleep) [4] persistent and lasting; for a long time [5] delaying; postponement [6] heavy (weight) [7] latent; hidden [8] straighten (one's face); put on a grave expression
U+6C88.1	fUTF8	沈
U+6C88.1	fCNS11643	1-4A69
U+6C88.1	fBig5	A848
U+6C88.1	fGB	0-4972
U+6C88.1	fR/S	 85.4
U+6C88.1	fTotalStrokes	 7
U+6C88.1	fCangjie	ELBU
U+6C88.1	fFourCorner	34112
U+6C88.1	fMandarin	shen3
U+6C88.1	fEnglish	a Chinese family name
U+6C89.0	fUTF8	沉
U+6C89.0	fCNS11643	1-4A6A
U+6C89.0	fBig5	A849
U+6C89.0	fGB	0-3341
U+6C89.0	fR/S	 85.4
U+6C89.0	fTotalStrokes	 7
U+6C89.0	fCangjie	EBHN
U+6C89.0	fFourCorner	37110
U+6C89.0	fHakka	 cim2;  chim2; dem5; cin2; chim3; cihm2
U+6C89.0	fCantonese	cam4; zam6
U+6C89.0	fMandarin	chen2; chen3; shen3; zhen4
U+6C89.0	fEnglish	 sink; submerge addicted to
U+6C89.1	fUTF8	沉
U+6C89.1	fCNS11643	1-4A6A
U+6C89.1	fBig5	A849
U+6C89.1	fGB	0-3341
U+6C89.1	fR/S	 85.4
U+6C89.1	fTotalStrokes	 7
U+6C89.1	fCangjie	EBHN
U+6C89.1	fFourCorner	37110
U+6C89.1	fMandarin	chen1
U+6C8A.0	fUTF8	沊
U+6C8A.0	fCNS11643	2-2763
U+6C8A.0	fBig5	CD43
U+6C8A.0	fGB	5-5464
U+6C8A.0	fR/S	 85.4
U+6C8A.0	fTotalStrokes	 8
U+6C8A.0	fCangjie	LUE
U+6C8A.0	fFourCorner	42013
U+6C8A.0	fCantonese	dam2
U+6C8A.0	fMandarin	dan4
U+6C8A.0	fEnglish	onomatopoetic
U+6C8B.0	fUTF8	沋
U+6C8B.0	fCNS11643	2-244F
U+6C8B.0	fBig5	CB4F
U+6C8B.0	fGB	3-5343
U+6C8B.0	fR/S	 85.4
U+6C8B.0	fTotalStrokes	 7
U+6C8B.0	fCangjie	EIKU
U+6C8B.0	fFourCorner	33114
U+6C8B.0	fCantonese	jau4
U+6C8B.0	fMandarin	you2
U+6C8C.0	fUTF8	沌
U+6C8C.0	fCNS11643	1-4A71
U+6C8C.0	fBig5	A850
U+6C8C.0	fGB	0-6367
U+6C8C.0	fR/S	 85.4
U+6C8C.0	fTotalStrokes	 7
U+6C8C.0	fCangjie	EPU
U+6C8C.0	fFourCorner	35117
U+6C8C.0	fHakka	  tun3; tun2; tun5
U+6C8C.0	fCantonese	deon6; cyun5
U+6C8C.0	fMandarin	dun4; tun2; zhuan4; chun2
U+6C8C.0	fEnglish	 chaotic; confused turbid; murky
U+6C8D.0	fUTF8	沍
U+6C8D.0	fCNS11643	1-4A7C
U+6C8D.0	fBig5	A85B
U+6C8D.0	fGB	3-5346
U+6C8D.0	fR/S	 85.4
U+6C8D.0	fTotalStrokes	 7
U+6C8D.0	fCangjie	EMVM
U+6C8D.0	fFourCorner	31117
U+6C8D.0	fHakka	 fu5; fu6
U+6C8D.0	fCantonese	wu6
U+6C8D.0	fMandarin	hu4; he4; he2; hu2
U+6C8D.0	fEnglish	 freezing stopped up; closed off
U+6C8E.0	fUTF8	沎
U+6C8E.0	fCNS11643	2-245D
U+6C8E.0	fBig5	CB5D
U+6C8E.0	fGB	5-5479
U+6C8E.0	fR/S	 85.4
U+6C8E.0	fTotalStrokes	 7
U+6C8E.0	fCangjie	EOP
U+6C8E.0	fFourCorner	31110
U+6C8E.0	fCantonese	fo3
U+6C8E.0	fMandarin	huo4
U+6C8F.0	fUTF8	沏
U+6C8F.0	fCNS11643	2-2450
U+6C8F.0	fBig5	CB50
U+6C8F.0	fGB	0-4663
U+6C8F.0	fR/S	 85.4
U+6C8F.0	fTotalStrokes	 7
U+6C8F.0	fCangjie	EPSH
U+6C8F.0	fFourCorner	37120
U+6C8F.0	fHakka	  ciet7
U+6C8F.0	fCantonese	cai3; cit3; cai1; keoi1
U+6C8F.0	fMandarin	qi1; qie1; qie4
U+6C8F.0	fEnglish	[1] infuse [2] rapidly; turbulently (said of flower water)
U+6C8F.1	fUTF8	沏
U+6C8F.1	fCNS11643	2-2450
U+6C8F.1	fBig5	CB50
U+6C8F.1	fGB	0-4663
U+6C8F.1	fR/S	 85.4
U+6C8F.1	fTotalStrokes	 7
U+6C8F.1	fCangjie	EPSH
U+6C8F.1	fFourCorner	37120
U+6C8F.1	fMandarin	qu1
U+6C8F.1	fEnglish	 drench with water
U+6C90.0	fUTF8	沐
U+6C90.0	fCNS11643	1-4A6F
U+6C90.0	fBig5	A84E
U+6C90.0	fGB	0-6365
U+6C90.0	fR/S	 85.4
U+6C90.0	fTotalStrokes	 7
U+6C90.0	fCangjie	ED
U+6C90.0	fFourCorner	34190
U+6C90.0	fHakka	 muk8; muk7
U+6C90.0	fCantonese	muk6
U+6C90.0	fMandarin	mu4
U+6C90.0	fEnglish	[1] bathe; cleanse; wash; shampoo [2] holiday; leave; take a leave [3] receive favor [4] a Chinese family name
U+6C91.0	fUTF8	沑
U+6C91.0	fCNS11643	3-264D
U+6C91.0	fGB	3-5357
U+6C91.0	fR/S	 85.4
U+6C91.0	fTotalStrokes	 7
U+6C91.0	fCangjie	ENG
U+6C91.0	fFourCorner	37110
U+6C91.0	fCantonese	nuk6; luk6 (also)
U+6C91.0	fMandarin	rou2; niu3; nuu4
U+6C92.0	fUTF8	沒
U+6C92.0	fCNS11643	1-4A74
U+6C92.0	fBig5	A853
U+6C92.0	fGB	E-3027
U+6C92.0	fR/S	 85.4
U+6C92.0	fTotalStrokes	 7
U+6C92.0	fCangjie	ENE
U+6C92.0	fFourCorner	37147
U+6C92.0	fHakka	  mut8
U+6C92.0	fCantonese	mut6; mei6
U+6C92.0	fMandarin	mei2
U+6C92.0	fEnglish	[1] none; nothing; no [2] not yet; negative
U+6C92.1	fUTF8	沒
U+6C92.1	fCNS11643	1-4A74
U+6C92.1	fBig5	A853
U+6C92.1	fGB	E-3027
U+6C92.1	fR/S	 85.4
U+6C92.1	fTotalStrokes	 7
U+6C92.1	fCangjie	ENE
U+6C92.1	fFourCorner	37147
U+6C92.1	fMandarin	mo4; mo3
U+6C92.1	fEnglish	[1] sink; submerge [2] overflow; rise beyond [3] disappear [4] go into oblivion [5] none; exhausted [6] eliminated [7] finished; completed [8] take property away from another; confiscate [9] die; dead
U+6C93.0	fUTF8	沓
U+6C93.0	fCNS11643	2-274F
U+6C93.0	fBig5	CCEE
U+6C93.0	fGB	0-6D33
U+6C93.0	fR/S	 85.4
U+6C93.0	fTotalStrokes	 8
U+6C93.0	fCangjie	EA
U+6C93.0	fFourCorner	12603
U+6C93.0	fHakka	 tap8; tiap8
U+6C93.0	fCantonese	daap6
U+6C93.0	fMandarin	ta4; da2; da4
U+6C93.0	fEnglish	[1] reiterated; repeated [2] crowded together; joined; connected; piled up
U+6C94.0	fUTF8	沔
U+6C94.0	fCNS11643	1-4A7D
U+6C94.0	fBig5	A85C
U+6C94.0	fGB	0-6366
U+6C94.0	fR/S	 85.4
U+6C94.0	fTotalStrokes	 7
U+6C94.0	fCangjie	EMLS
U+6C94.0	fFourCorner	31127
U+6C94.0	fHakka	  mien3; men3
U+6C94.0	fCantonese	min5
U+6C94.0	fMandarin	mian3; min3; mi3
U+6C94.0	fEnglish	[1] flood; overflowing [2] the name of a river
U+6C95.0	fUTF8	沕
U+6C95.0	fCNS11643	2-2457
U+6C95.0	fBig5	CB57
U+6C95.0	fGB	3-5353
U+6C95.0	fR/S	 85.4
U+6C95.0	fTotalStrokes	 7
U+6C95.0	fCangjie	EPHH
U+6C95.0	fFourCorner	37120
U+6C95.0	fHakka	 wut7; wut8; mit8; mat8; vut7
U+6C95.0	fCantonese	mat6; mei6
U+6C95.0	fMandarin	wu4; mi4; mei4
U+6C95.0	fEnglish	deep
U+6C96.0	fUTF8	沖
U+6C96.0	fCNS11643	1-4A73
U+6C96.0	fBig5	A852
U+6C96.0	fGB	8-2D54
U+6C96.0	fR/S	 85.4
U+6C96.0	fTotalStrokes	 7
U+6C96.0	fCangjie	EL
U+6C96.0	fFourCorner	35106
U+6C96.0	fHakka	  chung1; cung1; cung2; chung5
U+6C96.0	fCantonese	cung1
U+6C96.0	fMandarin	chong1; dong4
U+6C96.0	fEnglish	[1] pour water (to powder, etc.); infuse; make beverages [2] wash away; wash with running water; rinse [3] soar; rise rapidly or shoot up [4] empty; void [5] dash against; clash with [6] childhood [7] neutralise; make void
U+6C96.1	fUTF8	沖
U+6C96.1	fCNS11643	1-4A73
U+6C96.1	fBig5	A852
U+6C96.1	fGB	8-2D54
U+6C96.1	fR/S	 85.4
U+6C96.1	fTotalStrokes	 7
U+6C96.1	fCangjie	EL
U+6C96.1	fFourCorner	35106
U+6C96.1	fMandarin	chong4
U+6C97.0	fUTF8	沗
U+6C97.0	fCNS11643	3-2963
U+6C97.0	fGB	E-3028
U+6C97.0	fR/S	 85.4; 85.5; 37.5
U+6C97.0	fTotalStrokes	 8
U+6C97.0	fCangjie	MKE
U+6C97.0	fFourCorner	10230
U+6C97.0	fHakka	 tiam1
U+6C97.0	fCantonese	tim1
U+6C97.0	fMandarin	pang1; tian1
U+6C98.0	fUTF8	沘
U+6C98.0	fCNS11643	1-4A7E
U+6C98.0	fBig5	A85D
U+6C98.0	fGB	3-5345
U+6C98.0	fR/S	 85.4
U+6C98.0	fTotalStrokes	 7
U+6C98.0	fCangjie	EPP
U+6C98.0	fFourCorner	31110
U+6C98.0	fCantonese	bei2
U+6C98.0	fMandarin	bi3; pi2
U+6C98.0	fEnglish	name of a river
U+6C99.0	fUTF8	沙
U+6C99.0	fCNS11643	1-4A67
U+6C99.0	fBig5	A846
U+6C99.0	fGB	0-4933
U+6C99.0	fR/S	 85.4
U+6C99.0	fTotalStrokes	 7
U+6C99.0	fCangjie	EFH
U+6C99.0	fFourCorner	39120
U+6C99.0	fHakka	  sa1
U+6C99.0	fCantonese	saa1
U+6C99.0	fMandarin	sha1; suo1; sha4; si1
U+6C99.0	fEnglish	[1] sand; tiny gravel; tiny pebbles [2] the land around water; beach; sandbank; desert [3] pick; select; sort; sift [4] overripe (fruit, especially melons) [5] hoars (human voice) [6] sandy; granular [7] a kind of clay for making utensils, vessels, etc. [8] a Chinese family name
U+6C9A.0	fUTF8	沚
U+6C9A.0	fCNS11643	2-2454
U+6C9A.0	fBig5	CB54
U+6C9A.0	fGB	3-5347
U+6C9A.0	fR/S	 85.4
U+6C9A.0	fTotalStrokes	 7
U+6C9A.0	fCangjie	EYLM
U+6C9A.0	fFourCorner	31110
U+6C9A.0	fHakka	 zhi3; zih3; tje3
U+6C9A.0	fCantonese	zi2
U+6C9A.0	fMandarin	zhi3; zhi4
U+6C9A.0	fEnglish	 sandy islet in a stream; small sandbank
U+6C9B.0	fUTF8	沛
U+6C9B.0	fCNS11643	1-4A6C
U+6C9B.0	fBig5	A84B
U+6C9B.0	fGB	0-4566
U+6C9B.0	fR/S	 85.4
U+6C9B.0	fTotalStrokes	 7
U+6C9B.0	fCangjie	EJB
U+6C9B.0	fFourCorner	35127
U+6C9B.0	fHakka	 pai5; pui5; pi5
U+6C9B.0	fCantonese	pui3
U+6C9B.0	fMandarin	pei4; bei4
U+6C9B.0	fEnglish	[1] abundance; full of; copious; much [2] quickly; rapidly; sudden [3] fall prostrate [4] reserve water for irrigation [5] tall; high; great
U+6C9C.0	fUTF8	沜
U+6C9C.0	fCNS11643	2-2458
U+6C9C.0	fBig5	CB58
U+6C9C.0	fGB	5-5478
U+6C9C.0	fR/S	 85.4
U+6C9C.0	fTotalStrokes	 7
U+6C9C.0	fCangjie	ELLL
U+6C9C.0	fFourCorner	32127
U+6C9C.0	fHakka	 pan2; pan5
U+6C9C.0	fCantonese	pun3
U+6C9C.0	fMandarin	pan4
U+6C9C.0	fEnglish	banks of a stream
U+6C9D.0	fUTF8	沝
U+6C9D.0	fCNS11643	2-2764
U+6C9D.0	fBig5	CD44
U+6C9D.0	fGB	5-5465
U+6C9D.0	fR/S	 85.4
U+6C9D.0	fTotalStrokes	 8
U+6C9D.0	fCangjie	EE
U+6C9D.0	fFourCorner	12230
U+6C9D.0	fCantonese	zeoi2; ceoi4
U+6C9D.0	fMandarin	zhui3; zi1
U+6C9E.0	fUTF8	沞
U+6C9E.0	fCNS11643	3-264E
U+6C9E.0	fGB	3-533F
U+6C9E.0	fR/S	  85.4
U+6C9E.0	fTotalStrokes	 7
U+6C9E.0	fCangjie	EMLB
U+6C9E.0	fFourCorner	31120
U+6C9E.0	fCantonese	zaap3
U+6C9E.0	fMandarin	za1
U+6C9F.0	fUTF8	沟
U+6C9F.0	fCNS11643	3-2652
U+6C9F.0	fGB	0-3935
U+6C9F.0	fR/S	 85.4
U+6C9F.0	fTotalStrokes	 7
U+6C9F.0	fCangjie	EPI
U+6C9F.0	fFourCorner	37120
U+6C9F.0	fHakka	 geu1
U+6C9F.0	fCantonese	gau1 (also); kau1
U+6C9F.0	fMandarin	gou1; gang3; kou4
U+6C9F.0	fEnglish	simplified form of &#x6E9D;
U+6CA0.0	fUTF8	沠
U+6CA0.0	fCNS11643	3-265C
U+6CA0.0	fGB	E-3029
U+6CA0.0	fR/S	 85.4
U+6CA0.0	fTotalStrokes	 7
U+6CA0.0	fCangjie	EHLO
U+6CA0.0	fFourCorner	32130
U+6CA0.0	fCantonese	paai3
U+6CA0.0	fMandarin	liu2
U+6CA1.0	fUTF8	没
U+6CA1.0	fCNS11643	3-2659
U+6CA1.0	fGB	0-433B
U+6CA1.0	fR/S	 85.4
U+6CA1.0	fTotalStrokes	 7
U+6CA1.0	fCangjie	EHNE
U+6CA1.0	fFourCorner	37140
U+6CA1.0	fHakka	 mut8
U+6CA1.0	fCantonese	mut6
U+6CA1.0	fMandarin	mei2; me5
U+6CA1.0	fEnglish	 not; have not; none drown; sink
U+6CA1.1	fUTF8	没
U+6CA1.1	fCNS11643	3-2659
U+6CA1.1	fGB	0-433B
U+6CA1.1	fR/S	 85.4
U+6CA1.1	fTotalStrokes	 7
U+6CA1.1	fCangjie	EHNE
U+6CA1.1	fFourCorner	37140
U+6CA1.1	fMandarin	mo4
U+6CA2.0	fUTF8	沢
U+6CA2.0	fCNS11643	3-2656
U+6CA2.0	fGB	E-302A
U+6CA2.0	fR/S	 85.4
U+6CA2.0	fTotalStrokes	 7
U+6CA2.0	fCangjie	ESO
U+6CA2.0	fFourCorner	37130
U+6CA2.0	fCantonese	zaak6
U+6CA2.0	fMandarin	ze2
U+6CA2.0	fEnglish	 marsh; swamp grace; brilliance
U+6CA3.0	fUTF8	沣
U+6CA3.0	fCNS11643	F-2468
U+6CA3.0	fGB	0-6363
U+6CA3.0	fR/S	 85.4
U+6CA3.0	fTotalStrokes	 7
U+6CA3.0	fCangjie	EQJ
U+6CA3.0	fHakka	 fung1
U+6CA3.0	fCantonese	fung1
U+6CA3.0	fMandarin	feng1
U+6CA3.0	fEnglish	simplified form of &#x7043;
U+6CA4.0	fUTF8	沤
U+6CA4.0	fGB	0-453D
U+6CA4.0	fR/S	 85.4
U+6CA4.0	fTotalStrokes	 7
U+6CA4.0	fCangjie	ESK
U+6CA4.0	fHakka	 eu5
U+6CA4.0	fCantonese	au3; au1; ngau3 (also); ngau1 (also)
U+6CA4.0	fMandarin	ou1; ou4
U+6CA4.0	fEnglish	simplified form of &#x6F1A;
U+6CA5.0	fUTF8	沥
U+6CA5.0	fGB	0-4124
U+6CA5.0	fR/S	 85.4
U+6CA5.0	fTotalStrokes	 7
U+6CA5.0	fCangjie	EMKS
U+6CA5.0	fHakka	 lit8
U+6CA5.0	fCantonese	lik6; lik1 (also); lek6
U+6CA5.0	fMandarin	li4
U+6CA5.0	fEnglish	simplified form of &#x701D;
U+6CA6.0	fUTF8	沦
U+6CA6.0	fGB	0-4259
U+6CA6.0	fR/S	 85.4
U+6CA6.0	fTotalStrokes	 7
U+6CA6.0	fCangjie	EOP
U+6CA6.0	fHakka	 lun2
U+6CA6.0	fCantonese	leon4
U+6CA6.0	fMandarin	lun2; guan1; lun3; lun4
U+6CA6.0	fEnglish	simplified form of &#x6DEA;
U+6CA7.0	fUTF8	沧
U+6CA7.0	fGB	0-3257
U+6CA7.0	fR/S	 85.4
U+6CA7.0	fTotalStrokes	 7
U+6CA7.0	fCangjie	EOSU
U+6CA7.0	fHakka	 cong1
U+6CA7.0	fCantonese	cong1
U+6CA7.0	fMandarin	cang1; chuang4
U+6CA7.0	fEnglish	simplified form of &#x6EC4;
U+6CA8.0	fUTF8	沨
U+6CA8.0	fGB	8-7C70
U+6CA8.0	fR/S	 85.4
U+6CA8.0	fTotalStrokes	 7
U+6CA8.0	fCangjie	EHNK
U+6CA8.0	fCantonese	fung1; fung4 (also)
U+6CA8.0	fMandarin	feng2; feng1
U+6CA9.0	fUTF8	沩
U+6CA9.0	fGB	0-636D
U+6CA9.0	fR/S	 85.4
U+6CA9.0	fTotalStrokes	 7
U+6CA9.0	fCangjie	EIKS
U+6CA9.0	fCantonese	gwai1
U+6CA9.0	fMandarin	wei2; gui1
U+6CA9.0	fEnglish	simplified form of &#x6F59;
U+6CAA.0	fUTF8	沪
U+6CAA.0	fCNS11643	3-265B
U+6CAA.0	fGB	0-3B26
U+6CAA.0	fR/S	 85.4
U+6CAA.0	fTotalStrokes	 7
U+6CAA.0	fCangjie	EIS
U+6CAA.0	fFourCorner	33170
U+6CAA.0	fHakka	 fu5; fu3
U+6CAA.0	fCantonese	wu6
U+6CAA.0	fMandarin	hu4
U+6CAA.0	fEnglish	simplified form of &#x6EEC;
U+6CAB.0	fUTF8	沫
U+6CAB.0	fCNS11643	1-4E4D
U+6CAB.0	fBig5	AA6A
U+6CAB.0	fGB	0-442D
U+6CAB.0	fR/S	 85.5
U+6CAB.0	fTotalStrokes	 8
U+6CAB.0	fCangjie	EDJ
U+6CAB.0	fFourCorner	35190
U+6CAB.0	fHakka	 mat8; mat7
U+6CAB.0	fCantonese	mut6; mut3
U+6CAB.0	fMandarin	mo4
U+6CAB.0	fEnglish	[1] tiny bubbles on the surface of water [2] saliva [3] end; finish
U+6CAB.1	fUTF8	沫
U+6CAB.1	fCNS11643	1-4E4D
U+6CAB.1	fBig5	AA6A
U+6CAB.1	fGB	0-442D
U+6CAB.1	fR/S	 85.5
U+6CAB.1	fTotalStrokes	 8
U+6CAB.1	fCangjie	EDJ
U+6CAB.1	fFourCorner	35190
U+6CAB.1	fMandarin	mo5
U+6CAC.0	fUTF8	沬
U+6CAC.0	fCNS11643	1-4E5D
U+6CAC.0	fBig5	AA7A
U+6CAC.0	fGB	3-5358
U+6CAC.0	fR/S	 85.5
U+6CAC.0	fTotalStrokes	 8
U+6CAC.0	fCangjie	EJD
U+6CAC.0	fFourCorner	35190
U+6CAC.0	fHakka	 moi5; moi6; moe5; mi2
U+6CAC.0	fCantonese	mui6; mut3
U+6CAC.0	fMandarin	mei4; hui4; wei4
U+6CAC.0	fEnglish	[1] dusk [2] the Mei, name of a river [3] the name of a town in ancient China in the state of Wei, in today's Honan province
U+6CAD.0	fUTF8	沭
U+6CAD.0	fCNS11643	2-2756
U+6CAD.0	fBig5	CCF5
U+6CAD.0	fGB	0-6370
U+6CAD.0	fR/S	 85.5
U+6CAD.0	fTotalStrokes	 8
U+6CAD.0	fCangjie	EIJC
U+6CAD.0	fFourCorner	33134
U+6CAD.0	fHakka	 shut8; sut8
U+6CAD.0	fCantonese	seot6
U+6CAD.0	fMandarin	shu4; yu4
U+6CAD.0	fEnglish	short for a river in Shantung province
U+6CAE.0	fUTF8	沮
U+6CAE.0	fCNS11643	1-4E54
U+6CAE.0	fBig5	AA71
U+6CAE.0	fGB	0-3E5A
U+6CAE.0	fR/S	 85.5
U+6CAE.0	fTotalStrokes	 8
U+6CAE.0	fCangjie	EBM
U+6CAE.0	fFourCorner	37110
U+6CAE.0	fHakka	 zu3;  zu1; zih3; zii5; zih1
U+6CAE.0	fCantonese	zeoi1; zeoi2; zeoi3; ceoi1; zeoi6
U+6CAE.0	fMandarin	ju1; jian1; zhu1; qu1; zu3
U+6CAE.0	fEnglish	[1] name of a river in Shantung province, name of a rive in Shansi province [2] a Chinese family name
U+6CAE.1	fUTF8	沮
U+6CAE.1	fCNS11643	1-4E54
U+6CAE.1	fBig5	AA71
U+6CAE.1	fGB	0-3E5A
U+6CAE.1	fR/S	 85.5
U+6CAE.1	fTotalStrokes	 8
U+6CAE.1	fCangjie	EBM
U+6CAE.1	fFourCorner	37110
U+6CAE.1	fMandarin	ju3
U+6CAE.1	fEnglish	[1] stop; abate [2] lose; defeated [3] spoil; injure; destroy; damage
U+6CAE.2	fUTF8	沮
U+6CAE.2	fCNS11643	1-4E54
U+6CAE.2	fBig5	AA71
U+6CAE.2	fGB	0-3E5A
U+6CAE.2	fR/S	 85.5
U+6CAE.2	fTotalStrokes	 8
U+6CAE.2	fCangjie	EBM
U+6CAE.2	fFourCorner	37110
U+6CAE.2	fMandarin	ju4
U+6CAE.2	fEnglish	damp; low-lying land; marshy
U+6CAF.0	fUTF8	沯
U+6CAF.0	fCNS11643	3-2D4E
U+6CAF.0	fGB	5-5466
U+6CAF.0	fR/S	 85.5
U+6CAF.0	fTotalStrokes	 9; 8
U+6CAF.0	fCangjie	EMR
U+6CAF.0	fFourCorner	12600
U+6CAF.0	fCantonese	zaat3
U+6CAF.0	fMandarin	zan3
U+6CB0.0	fUTF8	沰
U+6CB0.0	fCNS11643	2-276B
U+6CB0.0	fBig5	CD4B
U+6CB0.0	fGB	3-535E
U+6CB0.0	fR/S	 85.5
U+6CB0.0	fTotalStrokes	 8
U+6CB0.0	fCangjie	EMR
U+6CB0.0	fFourCorner	31160
U+6CB0.0	fHakka	 tok7; duk8
U+6CB0.0	fCantonese	tok3
U+6CB0.0	fMandarin	tuo1
U+6CB0.0	fEnglish	[1] let drop [2] mass; lump [3] drip
U+6CB1.0	fUTF8	沱
U+6CB1.0	fCNS11643	1-4E45
U+6CB1.0	fBig5	AA62
U+6CB1.0	fGB	0-637B
U+6CB1.0	fR/S	 85.5
U+6CB1.0	fTotalStrokes	 8
U+6CB1.0	fCangjie	EJP
U+6CB1.0	fFourCorner	33111
U+6CB1.0	fHakka	  to2
U+6CB1.0	fCantonese	to4
U+6CB1.0	fMandarin	tuo2; chi2; duo4; duo3
U+6CB1.0	fEnglish	[1] rivers; streams; waterways; flow [2] name of a river [3] continuous heavy rains
U+6CB2.0	fUTF8	沲
U+6CB2.0	fCNS11643	3-296B
U+6CB2.0	fGB	0-6375
U+6CB2.0	fR/S	 85.5
U+6CB2.0	fTotalStrokes	 8
U+6CB2.0	fCangjie	EOPD
U+6CB2.0	fFourCorner	38110
U+6CB2.0	fHakka	 to2
U+6CB2.0	fCantonese	to4
U+6CB2.0	fMandarin	tuo2; duo4
U+6CB2.0	fEnglish	float
U+6CB3.0	fUTF8	河
U+6CB3.0	fCNS11643	1-4E48
U+6CB3.0	fBig5	AA65
U+6CB3.0	fGB	0-3A53
U+6CB3.0	fR/S	 85.5
U+6CB3.0	fTotalStrokes	 8
U+6CB3.0	fCangjie	EMNR
U+6CB3.0	fFourCorner	31120
U+6CB3.0	fHakka	  ho2; ho3
U+6CB3.0	fCantonese	ho4; ho2 (also)
U+6CB3.0	fMandarin	he2
U+6CB3.0	fEnglish	[1] a general name for rivers, streams, and waterways [2] Ho, the Yellow river in northern China, 2,700 miles long
U+6CB4.0	fUTF8	沴
U+6CB4.0	fCNS11643	2-2762
U+6CB4.0	fBig5	CD42
U+6CB4.0	fGB	3-5368
U+6CB4.0	fR/S	 85.5
U+6CB4.0	fTotalStrokes	 8
U+6CB4.0	fCangjie	EOHH
U+6CB4.0	fFourCorner	38122
U+6CB4.0	fHakka	  tien5; li1; li5
U+6CB4.0	fCantonese	leoi6; lai6
U+6CB4.0	fMandarin	li4; dian3; nie4
U+6CB4.0	fEnglish	[1] miasmas; foul and poisonous air [2] dislocation
U+6CB5.0	fUTF8	沵
U+6CB5.0	fCNS11643	4-2726
U+6CB5.0	fGB	8-7C72
U+6CB5.0	fR/S	 85.5
U+6CB5.0	fTotalStrokes	 8
U+6CB5.0	fCangjie	ENF
U+6CB5.0	fFourCorner	37190
U+6CB5.0	fCantonese	leoi6; mei4; mei5 (also); nei4 (also); lei4 (also)
U+6CB5.0	fMandarin	li4; mi3; zhen3; mi2; ni3
U+6CB6.0	fUTF8	沶
U+6CB6.0	fCNS11643	2-2754
U+6CB6.0	fBig5	CCF3
U+6CB6.0	fGB	3-5359
U+6CB6.0	fR/S	 85.5
U+6CB6.0	fTotalStrokes	 8
U+6CB6.0	fCangjie	EMMF
U+6CB6.0	fFourCorner	31191
U+6CB6.0	fHakka	 ji2; ri2; chi1; cih1
U+6CB6.0	fCantonese	ji4; ci4; si6
U+6CB6.0	fMandarin	yi2; chi2; shi4; zhi4
U+6CB6.0	fEnglish	a small tributary of the Han river
U+6CB7.0	fUTF8	沷
U+6CB7.0	fCNS11643	2-2758
U+6CB7.0	fBig5	CCF7
U+6CB7.0	fGB	3-535F
U+6CB7.0	fR/S	 85.5
U+6CB7.0	fTotalStrokes	 8
U+6CB7.0	fCangjie	EIKK
U+6CB7.0	fFourCorner	33147
U+6CB7.0	fHakka	  fut7;  pat8; wat8; fat7
U+6CB7.0	fCantonese	faat3; fat3; put3
U+6CB7.0	fMandarin	fa1; fu2
U+6CB7.0	fEnglish	 remove evil cleanse wash away
U+6CB8.0	fUTF8	沸
U+6CB8.0	fCNS11643	1-4E50
U+6CB8.0	fBig5	AA6D
U+6CB8.0	fGB	0-3750
U+6CB8.0	fR/S	 85.5
U+6CB8.0	fTotalStrokes	 8
U+6CB8.0	fCangjie	ELLN
U+6CB8.0	fFourCorner	35127
U+6CB8.0	fHakka	  bui5; fut7; bi5; fut8
U+6CB8.0	fCantonese	fai3; faak1
U+6CB8.0	fMandarin	fei4; fu2; pei4
U+6CB8.0	fEnglish	[1] boiling (water, etc.) [2] gush; bubble up
U+6CB9.0	fUTF8	油
U+6CB9.0	fCNS11643	1-4E52
U+6CB9.0	fBig5	AA6F
U+6CB9.0	fGB	0-534D
U+6CB9.0	fR/S	 85.5
U+6CB9.0	fTotalStrokes	 8
U+6CB9.0	fCangjie	ELW
U+6CB9.0	fFourCorner	35160
U+6CB9.0	fHakka	  ju2; jeu2; riu2; jiu3; jiu2
U+6CB9.0	fCantonese	jau4; jau2 (also)
U+6CB9.0	fMandarin	you2; you4
U+6CB9.0	fEnglish	[1] a general name for oil; fat; grease (either animal or vegetable) [2] anything in liquid form which is inflammable (petroleum, gasoline, etc.) [3] oil [4] varnish; paint [5] greasy [6] polished and over-experienced; sly; sleeky [7] luxuriant; prospering; flourishing
U+6CBA.0	fUTF8	沺
U+6CBA.0	fCNS11643	2-275B
U+6CBA.0	fBig5	CCFA
U+6CBA.0	fGB	3-5363
U+6CBA.0	fR/S	 85.5
U+6CBA.0	fTotalStrokes	 8
U+6CBA.0	fCangjie	EW
U+6CBA.0	fFourCorner	36100
U+6CBA.0	fCantonese	tin4
U+6CBA.0	fMandarin	tian2
U+6CBA.0	fEnglish	turbulent
U+6CBB.0	fUTF8	治
U+6CBB.0	fCNS11643	1-4E59
U+6CBB.0	fBig5	AA76
U+6CBB.0	fGB	0-564E
U+6CBB.0	fR/S	 85.5
U+6CBB.0	fTotalStrokes	 8
U+6CBB.0	fCangjie	EIR
U+6CBB.0	fFourCorner	33160
U+6CBB.0	fHakka	 ci3; cih2; ci5; chi6; cih5; chi5; ci2; chii5
U+6CBB.0	fCantonese	zi6; ci4
U+6CBB.0	fMandarin	chi2; zhi4; yi2; tai1
U+6CBB.0	fEnglish	[1] govern; regulate; administer; control; manage; rule [2] regulate; harness (river) [3] the seat of the local government [4] treat (disease); cure [5] study; research [6] punish [7] peaceful and orderly [8] a Chinese family name
U+6CBC.0	fUTF8	沼
U+6CBC.0	fCNS11643	1-4E4B
U+6CBC.0	fBig5	AA68
U+6CBC.0	fGB	0-5553
U+6CBC.0	fR/S	 85.5
U+6CBC.0	fTotalStrokes	 8
U+6CBC.0	fCangjie	ESHR
U+6CBC.0	fFourCorner	37162
U+6CBC.0	fHakka	  zhau5; zau5; sau3; zau3; zhau1; cheu5
U+6CBC.0	fCantonese	ziu2
U+6CBC.0	fMandarin	zhao3; zhao4
U+6CBC.0	fEnglish	 lake; pond; pool; marsh; swamp
U+6CBD.0	fUTF8	沽
U+6CBD.0	fCNS11643	1-4E49
U+6CBD.0	fBig5	AA66
U+6CBD.0	fGB	0-3941
U+6CBD.0	fR/S	 85.5
U+6CBD.0	fTotalStrokes	 8
U+6CBD.0	fCangjie	EJR
U+6CBD.0	fFourCorner	34160
U+6CBD.0	fHakka	  gu1
U+6CBD.0	fCantonese	gu1; gu2; gwu1 (also); gwu2 (also); gu3
U+6CBD.0	fMandarin	gu1; gu3; gu4
U+6CBD.0	fEnglish	[1] buy [2] sell [3] crude; inferior (quality) [4] negligent
U+6CBE.0	fUTF8	沾
U+6CBE.0	fCNS11643	1-4E4A
U+6CBE.0	fBig5	AA67
U+6CBE.0	fGB	0-5534
U+6CBE.0	fR/S	 85.5
U+6CBE.0	fTotalStrokes	 8
U+6CBE.0	fCangjie	EYR
U+6CBE.0	fFourCorner	31160
U+6CBE.0	fHakka	  zam1; zham1;  dem1; dep7; tiam1; diam1
U+6CBE.0	fCantonese	zim1; tip3
U+6CBE.0	fMandarin	zhan1; tian1; dian4; tie1; die2; chan1
U+6CBE.0	fEnglish	[1] moisten; wet; soak [2] tinge; stain; contaminate [3] be imbued; with; be infected with [4] benefit from [5] touch
U+6CBF.0	fUTF8	沿
U+6CBF.0	fCNS11643	1-4E58
U+6CBF.0	fBig5	AA75
U+6CBF.0	fGB	0-5158
U+6CBF.0	fR/S	 85.5
U+6CBF.0	fTotalStrokes	 8
U+6CBF.0	fCangjie	ECR
U+6CBF.0	fFourCorner	38160
U+6CBF.0	fHakka	 jen2; ren2; jan2
U+6CBF.0	fCantonese	jyun4
U+6CBF.0	fMandarin	yan2; yan1; yan3
U+6CBF.0	fEnglish	[1] follow; go along; along [2] hand down; continue [3] successive; continuous
U+6CBF.1	fUTF8	沿
U+6CBF.1	fCNS11643	1-4E58
U+6CBF.1	fBig5	AA75
U+6CBF.1	fGB	0-5158
U+6CBF.1	fR/S	 85.5
U+6CBF.1	fTotalStrokes	 8
U+6CBF.1	fCangjie	ECR
U+6CBF.1	fFourCorner	38160
U+6CBF.1	fMandarin	yan4
U+6CBF.1	fEnglish	the edge of something; brim
U+6CC0.0	fUTF8	泀
U+6CC0.0	fCNS11643	2-2767
U+6CC0.0	fBig5	CD47
U+6CC0.0	fGB	5-5530
U+6CC0.0	fR/S	 85.5
U+6CC0.0	fTotalStrokes	 8
U+6CC0.0	fCangjie	ESMR
U+6CC0.0	fFourCorner	37120
U+6CC0.0	fCantonese	si1
U+6CC0.0	fMandarin	si1
U+6CC1.0	fUTF8	況
U+6CC1.0	fCNS11643	1-4E53
U+6CC1.0	fBig5	AA70
U+6CC1.0	fGB	E-302B
U+6CC1.0	fR/S	 85.5
U+6CC1.0	fTotalStrokes	 8
U+6CC1.0	fCangjie	ERHU
U+6CC1.0	fFourCorner	36110
U+6CC1.0	fHakka	 kong3; k'wat7; kong5; k'wong3; kwong3
U+6CC1.0	fCantonese	fong3; fong2 (also)
U+6CC1.0	fMandarin	kuang4
U+6CC1.0	fEnglish	[1] moreover; in addition; not to mention [2] compare; comparative [3] situation; condition; circumstances [4] visit; call on [5] a Chinese family name
U+6CC2.0	fUTF8	泂
U+6CC2.0	fCNS11643	2-275A
U+6CC2.0	fBig5	CCF9
U+6CC2.0	fGB	3-5364
U+6CC2.0	fR/S	 85.5
U+6CC2.0	fTotalStrokes	 8
U+6CC2.0	fCangjie	EBR
U+6CC2.0	fFourCorner	37120
U+6CC2.0	fHakka	 kwen3; gwen3
U+6CC2.0	fCantonese	gwing2
U+6CC2.0	fMandarin	jiong3; ying2; jiong1
U+6CC2.0	fEnglish	[1] clear and deep (water) [2] far and wide
U+6CC3.0	fUTF8	泃
U+6CC3.0	fCNS11643	2-275C
U+6CC3.0	fBig5	CCFB
U+6CC3.0	fGB	3-536A
U+6CC3.0	fR/S	 85.5
U+6CC3.0	fTotalStrokes	 8
U+6CC3.0	fCangjie	EPR
U+6CC3.0	fFourCorner	37120
U+6CC3.0	fHakka	 gi5; kieu1; geu1
U+6CC3.0	fCantonese	keoi1; gau1; geoi3
U+6CC3.0	fMandarin	ju4; gou1; ju1; qu2
U+6CC3.0	fEnglish	river in Henan
U+6CC4.0	fUTF8	泄
U+6CC4.0	fCNS11643	1-4E51
U+6CC4.0	fBig5	AA6E
U+6CC4.0	fGB	0-5039
U+6CC4.0	fR/S	 85.5
U+6CC4.0	fTotalStrokes	 8
U+6CC4.0	fCangjie	EPT
U+6CC4.0	fFourCorner	34117
U+6CC4.0	fHakka	 siet7; set7; ji3; ri3; ji5; i5
U+6CC4.0	fCantonese	sit3; jai6
U+6CC4.0	fMandarin	xie4
U+6CC4.0	fEnglish	[1] leak out; reveal [2] vent [3] scatter; disperse [4] a Chinese family name
U+6CC4.1	fUTF8	泄
U+6CC4.1	fCNS11643	1-4E51
U+6CC4.1	fBig5	AA6E
U+6CC4.1	fGB	0-5039
U+6CC4.1	fR/S	 85.5
U+6CC4.1	fTotalStrokes	 8
U+6CC4.1	fCangjie	EPT
U+6CC4.1	fFourCorner	34117
U+6CC4.1	fMandarin	yi4
U+6CC4.1	fEnglish	[1] mild and easy [2] many; crowded
U+6CC5.0	fUTF8	泅
U+6CC5.0	fCNS11643	1-4E56
U+6CC5.0	fBig5	AA73
U+6CC5.0	fGB	0-4776
U+6CC5.0	fR/S	 85.5
U+6CC5.0	fTotalStrokes	 8
U+6CC5.0	fCangjie	EWO
U+6CC5.0	fFourCorner	36100
U+6CC5.0	fHakka	  siu2;  ciu2
U+6CC5.0	fCantonese	cau4; jau4
U+6CC5.0	fMandarin	qiu2; you1
U+6CC5.0	fEnglish	 swim; float; wade
U+6CC6.0	fUTF8	泆
U+6CC6.0	fCNS11643	2-275D
U+6CC6.0	fBig5	CCFC
U+6CC6.0	fGB	3-5365
U+6CC6.0	fR/S	 85.5
U+6CC6.0	fTotalStrokes	 8
U+6CC6.0	fCangjie	EHQO
U+6CC6.0	fFourCorner	35130
U+6CC6.0	fHakka	 jit8; rit8
U+6CC6.0	fCantonese	jat6
U+6CC6.0	fMandarin	yi1; die2; yi4
U+6CC6.0	fEnglish	[1] licentious; libertine; dissipated [2] flooding; overflowing
U+6CC7.0	fUTF8	泇
U+6CC7.0	fCNS11643	2-276A
U+6CC7.0	fBig5	CD4A
U+6CC7.0	fGB	3-536F
U+6CC7.0	fR/S	 85.5
U+6CC7.0	fTotalStrokes	 8
U+6CC7.0	fCangjie	EKSR
U+6CC7.0	fFourCorner	36100
U+6CC7.0	fCantonese	gaa1
U+6CC7.0	fMandarin	jia1
U+6CC8.0	fUTF8	泈
U+6CC8.0	fCNS11643	3-296D
U+6CC8.0	fGB	5-552E
U+6CC8.0	fR/S	 85.5
U+6CC8.0	fTotalStrokes	 8
U+6CC8.0	fCangjie	EHEY
U+6CC8.0	fFourCorner	37130
U+6CC8.0	fCantonese	zung1
U+6CC8.0	fMandarin	zhong1
U+6CC9.0	fUTF8	泉
U+6CC9.0	fCNS11643	1-5178
U+6CC9.0	fBig5	AC75
U+6CC9.0	fGB	0-482A
U+6CC9.0	fR/S	 85.5
U+6CC9.0	fTotalStrokes	 9
U+6CC9.0	fCangjie	HAE
U+6CC9.0	fFourCorner	26232
U+6CC9.0	fHakka	  can2; cien2; can3
U+6CC9.0	fCantonese	cyun4
U+6CC9.0	fMandarin	quan2; juan4
U+6CC9.0	fEnglish	[1] spring; fountain [2] money (archaic) [3] a Chinese family name
U+6CCA.0	fUTF8	泊
U+6CCA.0	fCNS11643	1-4E5C
U+6CCA.0	fBig5	AA79
U+6CCA.0	fGB	0-3234
U+6CCA.0	fR/S	 85.5
U+6CCA.0	fTotalStrokes	 8
U+6CCA.0	fCangjie	EHA
U+6CCA.0	fFourCorner	36100
U+6CCA.0	fHakka	  pok7;  pak7; pok8; bok7
U+6CCA.0	fCantonese	bok6; paak3 (also); baak3
U+6CCA.0	fMandarin	po2; po1
U+6CCA.0	fEnglish	[1] stay; anchor a vessel; lie at anchor; moor [2] drift [3] tranquil and quiet [4] lake; body of water [5] park (cars)
U+6CCA.1	fUTF8	泊
U+6CCA.1	fCNS11643	1-4E5C
U+6CCA.1	fBig5	AA79
U+6CCA.1	fGB	0-3234
U+6CCA.1	fR/S	 85.5
U+6CCA.1	fTotalStrokes	 8
U+6CCA.1	fCangjie	EHA
U+6CCA.1	fFourCorner	36100
U+6CCA.1	fMandarin	bo2; po4
U+6CCB.0	fUTF8	泋
U+6CCB.0	fCNS11643	4-267D
U+6CCB.0	fGB	3-535A
U+6CCB.0	fR/S	  85.5
U+6CCB.0	fTotalStrokes	 8
U+6CCB.0	fCangjie	EJT
U+6CCB.0	fFourCorner	34140
U+6CCB.0	fCantonese	wai3
U+6CCB.0	fMandarin	hui4
U+6CCC.0	fUTF8	泌
U+6CCC.0	fCNS11643	1-4E46
U+6CCC.0	fBig5	AA63
U+6CCC.0	fGB	0-435A
U+6CCC.0	fR/S	 85.5
U+6CCC.0	fTotalStrokes	 8
U+6CCC.0	fCangjie	EPH
U+6CCC.0	fFourCorner	33100
U+6CCC.0	fHakka	  bi5; bit7
U+6CCC.0	fCantonese	bei3; bat6; bai3; bat1
U+6CCC.0	fMandarin	mi4
U+6CCC.0	fEnglish	[1] swift and easy gushing of water [2] the name of a rive in Honan province
U+6CCC.1	fUTF8	泌
U+6CCC.1	fCNS11643	1-4E46
U+6CCC.1	fBig5	AA63
U+6CCC.1	fGB	0-435A
U+6CCC.1	fR/S	 85.5
U+6CCC.1	fTotalStrokes	 8
U+6CCC.1	fCangjie	EPH
U+6CCC.1	fFourCorner	33100
U+6CCC.1	fMandarin	bi4
U+6CCC.1	fEnglish	 seep out; excrete; secrete
U+6CCD.0	fUTF8	泍
U+6CCD.0	fCNS11643	2-2769
U+6CCD.0	fBig5	CD49
U+6CCD.0	fGB	3-535C
U+6CCD.0	fR/S	 85.5
U+6CCD.0	fTotalStrokes	 8
U+6CCD.0	fCangjie	EDM
U+6CCD.0	fFourCorner	35130
U+6CCD.0	fCantonese	ban1; ban6
U+6CCD.0	fMandarin	ben1; ben4
U+6CCE.0	fUTF8	泎
U+6CCE.0	fCNS11643	3-296C
U+6CCE.0	fGB	5-552A
U+6CCE.0	fR/S	 85.5
U+6CCE.0	fTotalStrokes	 8
U+6CCE.0	fCangjie	EOS
U+6CCE.0	fFourCorner	38110
U+6CCE.0	fCantonese	zaak6
U+6CCE.0	fMandarin	zhuo2
U+6CCF.0	fUTF8	泏
U+6CCF.0	fCNS11643	2-276D
U+6CCF.0	fBig5	CD4D
U+6CCF.0	fGB	3-536E
U+6CCF.0	fR/S	 85.5
U+6CCF.0	fTotalStrokes	 8
U+6CCF.0	fCangjie	EUU
U+6CCF.0	fFourCorner	32177
U+6CCF.0	fCantonese	zeot1; fat1; zyut3
U+6CCF.0	fMandarin	chu4; ku1
U+6CD0.0	fUTF8	泐
U+6CD0.0	fCNS11643	2-2759
U+6CD0.0	fBig5	CCF8
U+6CD0.0	fGB	0-636E
U+6CD0.0	fR/S	 85.5
U+6CD0.0	fTotalStrokes	 8
U+6CD0.0	fCangjie	ENLS
U+6CD0.0	fFourCorner	34127
U+6CD0.0	fHakka	 lit8; let7; let8
U+6CD0.0	fCantonese	lak6; laak6 (also)
U+6CD0.0	fMandarin	le4; li4
U+6CD0.0	fEnglish	[1] rocks splitting [2] write letters [3] carve [4] condense; coagulate
U+6CD1.0	fUTF8	泑
U+6CD1.0	fCNS11643	2-276F
U+6CD1.0	fBig5	CD4F
U+6CD1.0	fGB	3-5371
U+6CD1.0	fR/S	 85.5
U+6CD1.0	fTotalStrokes	 8
U+6CD1.0	fCangjie	EVIS
U+6CD1.0	fFourCorner	34127
U+6CD1.0	fHakka	 ju5
U+6CD1.0	fCantonese	jau2; jau1 (also); jau3
U+6CD1.0	fMandarin	you3; you1; ao1
U+6CD1.0	fEnglish	the vitreous glaze on china; porcelain; etc.
U+6CD2.0	fUTF8	泒
U+6CD2.0	fCNS11643	2-2760
U+6CD2.0	fBig5	CD40
U+6CD2.0	fGB	3-5367
U+6CD2.0	fR/S	 85.5; 85.6
U+6CD2.0	fTotalStrokes	 8
U+6CD2.0	fCangjie	EHVO
U+6CD2.0	fFourCorner	32130
U+6CD2.0	fHakka	 pai5
U+6CD2.0	fCantonese	gu1; gwu1 (also)
U+6CD2.0	fMandarin	gu1
U+6CD2.0	fEnglish	a river in Hubei province
U+6CD3.0	fUTF8	泓
U+6CD3.0	fCNS11643	1-4E4F
U+6CD3.0	fBig5	AA6C
U+6CD3.0	fGB	0-637C
U+6CD3.0	fR/S	 85.5
U+6CD3.0	fTotalStrokes	 8
U+6CD3.0	fCangjie	ENI
U+6CD3.0	fFourCorner	32130
U+6CD3.0	fHakka	 fen2; fen1
U+6CD3.0	fCantonese	wang4
U+6CD3.0	fMandarin	hong2; weng1
U+6CD3.0	fEnglish	[1] clear; deep water; limpid water [2] the ancient name of a stream in Honan province
U+6CD4.0	fUTF8	泔
U+6CD4.0	fCNS11643	2-2755
U+6CD4.0	fBig5	CCF4
U+6CD4.0	fGB	0-636F
U+6CD4.0	fR/S	 85.5
U+6CD4.0	fTotalStrokes	 8
U+6CD4.0	fCangjie	ETM
U+6CD4.0	fFourCorner	34170
U+6CD4.0	fHakka	  gam1
U+6CD4.0	fCantonese	gam1
U+6CD4.0	fMandarin	gan1; han4
U+6CD4.0	fEnglish	[1] the water from washing rice [2] a way of cooking [3] stale (food)
U+6CD5.0	fUTF8	法
U+6CD5.0	fCNS11643	1-4E4E
U+6CD5.0	fBig5	AA6B
U+6CD5.0	fGB	0-3728
U+6CD5.0	fR/S	 85.5
U+6CD5.0	fTotalStrokes	 8
U+6CD5.0	fCangjie	EGI
U+6CD5.0	fFourCorner	34131
U+6CD5.0	fHakka	  fap7
U+6CD5.0	fCantonese	faat3; faat2 (also); fat3
U+6CD5.0	fMandarin	fa3
U+6CD5.0	fEnglish	[1] institution [2] law; rules; regulations; statute; legal [3] methods; ways of doing things [4] pattern; model after [5] the way; doctrines (Buddhism) [6] tricks; magic arts [7] expert or standard (calligraphy, painting, etc.) [8] penalty; punishment [9] a Chinese family name
U+6CD5.1	fUTF8	法
U+6CD5.1	fCNS11643	1-4E4E
U+6CD5.1	fBig5	AA6B
U+6CD5.1	fGB	0-3728
U+6CD5.1	fR/S	 85.5
U+6CD5.1	fTotalStrokes	 8
U+6CD5.1	fCangjie	EGI
U+6CD5.1	fFourCorner	34131
U+6CD5.1	fMandarin	fa2
U+6CD5.1	fEnglish	used in combination for weight of a scale
U+6CD5.2	fUTF8	法
U+6CD5.2	fCNS11643	1-4E4E
U+6CD5.2	fBig5	AA6B
U+6CD5.2	fGB	0-3728
U+6CD5.2	fR/S	 85.5
U+6CD5.2	fTotalStrokes	 8
U+6CD5.2	fCangjie	EGI
U+6CD5.2	fFourCorner	34131
U+6CD5.2	fMandarin	fa1
U+6CD5.2	fEnglish	a way, especially used in "no way out"
U+6CD5.3	fUTF8	法
U+6CD5.3	fCNS11643	1-4E4E
U+6CD5.3	fBig5	AA6B
U+6CD5.3	fGB	0-3728
U+6CD5.3	fR/S	 85.5
U+6CD5.3	fTotalStrokes	 8
U+6CD5.3	fCangjie	EGI
U+6CD5.3	fFourCorner	34131
U+6CD5.3	fMandarin	fa4
U+6CD5.3	fEnglish	[1] the Frank [2] France; French
U+6CD5.4	fUTF8	法
U+6CD5.4	fCNS11643	1-4E4E
U+6CD5.4	fBig5	AA6B
U+6CD5.4	fGB	0-3728
U+6CD5.4	fR/S	 85.5
U+6CD5.4	fTotalStrokes	 8
U+6CD5.4	fCangjie	EGI
U+6CD5.4	fFourCorner	34131
U+6CD5.4	fMandarin	fa5
U+6CD6.0	fUTF8	泖
U+6CD6.0	fCNS11643	1-4E60
U+6CD6.0	fBig5	AA7D
U+6CD6.0	fGB	0-6377
U+6CD6.0	fR/S	 85.5
U+6CD6.0	fTotalStrokes	 8
U+6CD6.0	fCangjie	EHHL
U+6CD6.0	fFourCorner	37120
U+6CD6.0	fHakka	 mau3
U+6CD6.0	fCantonese	maau5
U+6CD6.0	fMandarin	mao3; liu3; mao2
U+6CD6.0	fEnglish	[1] still waters [2] a river in Kiangsu province
U+6CD7.0	fUTF8	泗
U+6CD7.0	fCNS11643	1-4E55
U+6CD7.0	fBig5	AA72
U+6CD7.0	fGB	0-6374
U+6CD7.0	fR/S	 85.5
U+6CD7.0	fTotalStrokes	 8
U+6CD7.0	fCangjie	EWC
U+6CD7.0	fFourCorner	36100
U+6CD7.0	fHakka	  si5
U+6CD7.0	fCantonese	si3; sei3
U+6CD7.0	fMandarin	si4
U+6CD7.0	fEnglish	[1] nasal mucous; snivel [2] the name of a river in Shantung [3] the name of a county
U+6CD8.0	fUTF8	泘
U+6CD8.0	fCNS11643	3-296A
U+6CD8.0	fGB	5-552D
U+6CD8.0	fR/S	  85.5
U+6CD8.0	fTotalStrokes	 8
U+6CD8.0	fCangjie	EHFD
U+6CD8.0	fFourCorner	32140
U+6CD8.0	fCantonese	fu1
U+6CD8.0	fMandarin	hu1
U+6CD9.0	fUTF8	泙
U+6CD9.0	fCNS11643	2-2753
U+6CD9.0	fBig5	CCF2
U+6CD9.0	fGB	3-5360
U+6CD9.0	fR/S	 85.5
U+6CD9.0	fTotalStrokes	 8
U+6CD9.0	fCangjie	EMFJ
U+6CD9.0	fFourCorner	30247
U+6CD9.0	fHakka	 pin2; fin2
U+6CD9.0	fCantonese	paang1; ping4
U+6CD9.0	fMandarin	ping2; peng1
U+6CD9.0	fEnglish	roar
U+6CDA.0	fUTF8	泚
U+6CDA.0	fCNS11643	2-2B57
U+6CDA.0	fBig5	CF75
U+6CDA.0	fGB	3-537E
U+6CDA.0	fR/S	 85.6; 85.5
U+6CDA.0	fTotalStrokes	 9
U+6CDA.0	fCangjie	EYMP
U+6CDA.0	fFourCorner	31110
U+6CDA.0	fHakka	 cih3; ze3
U+6CDA.0	fCantonese	ci2; cai2
U+6CDA.0	fMandarin	ci3
U+6CDA.0	fEnglish	[1] clear waters [2] bright and brilliant [3] sweating
U+6CDB.0	fUTF8	泛
U+6CDB.0	fCNS11643	1-4E5B
U+6CDB.0	fBig5	AA78
U+6CDB.0	fGB	0-373A
U+6CDB.0	fR/S	 85.5
U+6CDB.0	fTotalStrokes	 8
U+6CDB.0	fCangjie	EHIO
U+6CDB.0	fFourCorner	32137
U+6CDB.0	fHakka	 fam5; fam6
U+6CDB.0	fCantonese	faan3; faan6 (also); faan4; fung2; fung3
U+6CDB.0	fMandarin	fan4; feng3; fa2; fan2
U+6CDB.0	fEnglish	[1] drift; float [2] not exact or precise; not practical [3] not sincere; not intimate [4] generally (speaking); as a whole; pan- [5] be suffused with
U+6CDC.0	fUTF8	泜
U+6CDC.0	fCNS11643	1-4E5F
U+6CDC.0	fBig5	AA7C
U+6CDC.0	fGB	3-5369
U+6CDC.0	fR/S	 85.5
U+6CDC.0	fTotalStrokes	 8
U+6CDC.0	fCangjie	EHPM
U+6CDC.0	fFourCorner	32140
U+6CDC.0	fHakka	 di3
U+6CDC.0	fCantonese	ci4; zi6; zi1
U+6CDC.0	fMandarin	zhi1; di4; chi2; zhi4
U+6CDC.0	fEnglish	a river in Hebei province
U+6CDD.0	fUTF8	泝
U+6CDD.0	fCNS11643	2-2761
U+6CDD.0	fBig5	CD41
U+6CDD.0	fGB	E-302C
U+6CDD.0	fR/S	 85.5
U+6CDD.0	fTotalStrokes	 8
U+6CDD.0	fCangjie	EHMY
U+6CDD.0	fFourCorner	32141
U+6CDD.0	fHakka	 sok7; su5
U+6CDD.0	fCantonese	sou3
U+6CDD.0	fMandarin	su4
U+6CDD.0	fEnglish	same as $ (go upstream; trace source; formerly)
U+6CDE.0	fUTF8	泞
U+6CDE.0	fCNS11643	2-2766
U+6CDE.0	fBig5	CD46
U+6CDE.0	fGB	0-4522
U+6CDE.0	fR/S	 85.5
U+6CDE.0	fTotalStrokes	 8
U+6CDE.0	fCangjie	EJMN
U+6CDE.0	fFourCorner	33121
U+6CDE.0	fHakka	 du3; chu3; cu3; nin5; nen5
U+6CDE.0	fCantonese	ning6; ling6 (also); ning4 (also); ling4 (also)
U+6CDE.0	fMandarin	ning4; ning3; zhu3; ning2; zhu4; ni4
U+6CDE.0	fEnglish	simplified form of &#x6FD8;
U+6CDF.0	fUTF8	泟
U+6CDF.0	fCNS11643	3-2965
U+6CDF.0	fGB	5-5522
U+6CDF.0	fR/S	 85.5
U+6CDF.0	fTotalStrokes	 8
U+6CDF.0	fCangjie	EMYM
U+6CDF.0	fFourCorner	31110
U+6CDF.0	fCantonese	zaa1
U+6CDF.0	fMandarin	cheng1
U+6CE0.0	fUTF8	泠
U+6CE0.0	fCNS11643	1-4E61
U+6CE0.0	fBig5	AA7E
U+6CE0.0	fGB	0-6376
U+6CE0.0	fR/S	 85.5
U+6CE0.0	fTotalStrokes	 8
U+6CE0.0	fCangjie	EOII
U+6CE0.0	fFourCorner	38137
U+6CE0.0	fHakka	 lang1; lin2; lang2
U+6CE0.0	fCantonese	ling4
U+6CE0.0	fMandarin	ling2; ling3; lian2
U+6CE0.0	fEnglish	[1] clear sounds [2] mild and comfortable [3] same as $ [4] a Chinese family name
U+6CE1.0	fUTF8	泡
U+6CE1.0	fCNS11643	1-4E5A
U+6CE1.0	fBig5	AA77
U+6CE1.0	fGB	0-455D
U+6CE1.0	fR/S	 85.5
U+6CE1.0	fTotalStrokes	 8
U+6CE1.0	fCangjie	EPRU
U+6CE1.0	fFourCorner	37112
U+6CE1.0	fHakka	  pau5;  pau1; po1; {pau1}; po2
U+6CE1.0	fCantonese	paau1; pou5; paau3; paau2 (also); pau3; paau4
U+6CE1.0	fMandarin	pao4; pao2; bao1
U+6CE1.0	fEnglish	[1] bubbles; suds; froth; foam [2] blister [3] steep; dip; infuse (tea, etc.); soak [4] dawdle; fool around (especially with women) (slang)
U+6CE1.1	fUTF8	泡
U+6CE1.1	fCNS11643	1-4E5A
U+6CE1.1	fBig5	AA77
U+6CE1.1	fGB	0-455D
U+6CE1.1	fR/S	 85.5
U+6CE1.1	fTotalStrokes	 8
U+6CE1.1	fCangjie	EPRU
U+6CE1.1	fFourCorner	37112
U+6CE1.1	fMandarin	pao1
U+6CE1.1	fEnglish	[1] loose and soft; spongy [2] an amount of excrement or urine
U+6CE2.0	fUTF8	波
U+6CE2.0	fCNS11643	1-4E4C
U+6CE2.0	fBig5	AA69
U+6CE2.0	fGB	0-3228
U+6CE2.0	fR/S	 85.5
U+6CE2.0	fTotalStrokes	 8
U+6CE2.0	fCangjie	EDHE
U+6CE2.0	fFourCorner	34147
U+6CE2.0	fHakka	  bo1; bi1; bo5
U+6CE2.0	fCantonese	bo1
U+6CE2.0	fMandarin	bo1; bei1; bi4; po1
U+6CE2.0	fEnglish	[1] waves; breakers [2] undulate; undulation; fluctuate; fluctuation [3] affect; involve; implicate; entangle
U+6CE3.0	fUTF8	泣
U+6CE3.0	fCNS11643	1-4E42
U+6CE3.0	fBig5	AA5F
U+6CE3.0	fGB	0-467C
U+6CE3.0	fR/S	 85.5
U+6CE3.0	fTotalStrokes	 8
U+6CE3.0	fCangjie	EYT
U+6CE3.0	fFourCorner	30118
U+6CE3.0	fHakka	 kip7;  hip7; lip8; lip7
U+6CE3.0	fCantonese	jap1
U+6CE3.0	fMandarin	qi4; li4; se4
U+6CE3.0	fEnglish	 come to tears without crying; weep
U+6CE4.0	fUTF8	泤
U+6CE4.0	fCNS11643	3-2969
U+6CE4.0	fGB	5-547D
U+6CE4.0	fR/S	 85.5
U+6CE4.0	fTotalStrokes	 8
U+6CE4.0	fCangjie	EVIO
U+6CE4.0	fFourCorner	38100
U+6CE4.0	fCantonese	ci5
U+6CE4.0	fMandarin	si4
U+6CE5.0	fUTF8	泥
U+6CE5.0	fCNS11643	1-4E47
U+6CE5.0	fBig5	AA64
U+6CE5.0	fGB	0-4460
U+6CE5.0	fR/S	 85.5
U+6CE5.0	fTotalStrokes	 8
U+6CE5.0	fCangjie	ESP
U+6CE5.0	fFourCorner	37111
U+6CE5.0	fHakka	 lai2;  ni2;  nai2; nai3; {nai2}; ni5; ne2
U+6CE5.0	fCantonese	nai4; lai4 (also); nei6; nai2 (also); lai2 (also); lei6 (also); nai5; nai6; nei5; leu5
U+6CE5.0	fMandarin	ni2; ni3; nie4; ning4
U+6CE5.0	fEnglish	[1] mud; mire; earth; clay; soil [2] plaster; paste [3] mashed vegetables or fruit; paste
U+6CE5.1	fUTF8	泥
U+6CE5.1	fCNS11643	1-4E47
U+6CE5.1	fBig5	AA64
U+6CE5.1	fGB	0-4460
U+6CE5.1	fR/S	 85.5
U+6CE5.1	fTotalStrokes	 8
U+6CE5.1	fCangjie	ESP
U+6CE5.1	fFourCorner	37111
U+6CE5.1	fMandarin	ni4
U+6CE5.1	fEnglish	[1] be tied down by conventions, old practices; very conservative [2] request with sweet words [3] inapplicable
U+6CE6.0	fUTF8	泦
U+6CE6.0	fCNS11643	4-267B
U+6CE6.0	fGB	3-536D
U+6CE6.0	fR/S	 85.5
U+6CE6.0	fTotalStrokes	 8
U+6CE6.0	fCangjie	ESKN
U+6CE6.0	fFourCorner	37110
U+6CE6.0	fCantonese	guk1
U+6CE6.0	fMandarin	ju2
U+6CE7.0	fUTF8	泧
U+6CE7.0	fCNS11643	2-2757
U+6CE7.0	fBig5	CCF6
U+6CE7.0	fGB	3-5361
U+6CE7.0	fR/S	 85.5
U+6CE7.0	fTotalStrokes	 8
U+6CE7.0	fCangjie	EIV
U+6CE7.0	fFourCorner	33150
U+6CE7.0	fHakka	 wat7; jet8; sat7
U+6CE7.0	fCantonese	kut3; kwut3 (also); jyut6; wut6
U+6CE7.0	fMandarin	yue4; huo4; sa4
U+6CE8.0	fUTF8	注
U+6CE8.0	fCNS11643	1-4E43
U+6CE8.0	fBig5	AA60
U+6CE8.0	fGB	0-5722
U+6CE8.0	fR/S	 85.5
U+6CE8.0	fTotalStrokes	 8
U+6CE8.0	fCangjie	EYG
U+6CE8.0	fFourCorner	30114
U+6CE8.0	fHakka	 zu5;  zhu5; zhu6
U+6CE8.0	fCantonese	zyu3
U+6CE8.0	fMandarin	zhu4; zhou4
U+6CE8.0	fEnglish	[1] pour (liquid) [2] concentrate; focus; engross; preoccupation [3] same as $ [4] stakes (in gambling)
U+6CE9.0	fUTF8	泩
U+6CE9.0	fCNS11643	2-276E
U+6CE9.0	fBig5	CD4E
U+6CE9.0	fGB	5-5529
U+6CE9.0	fR/S	 85.5
U+6CE9.0	fTotalStrokes	 8
U+6CE9.0	fCangjie	EHQM
U+6CE9.0	fFourCorner	35110
U+6CE9.0	fCantonese	sang1
U+6CE9.0	fMandarin	sheng1
U+6CEA.0	fUTF8	泪
U+6CEA.0	fCNS11643	3-2967
U+6CEA.0	fGB	0-4061
U+6CEA.0	fR/S	 85.5
U+6CEA.0	fTotalStrokes	 8
U+6CEA.0	fCangjie	EBU
U+6CEA.0	fFourCorner	36100
U+6CEA.0	fHakka	  lui5; lui6; lui3
U+6CEA.0	fCantonese	leoi6
U+6CEA.0	fMandarin	lei4; li4
U+6CEA.0	fEnglish	 tears weep; cry
U+6CEB.0	fUTF8	泫
U+6CEB.0	fCNS11643	2-2751
U+6CEB.0	fBig5	CCF0
U+6CEB.0	fGB	0-6379
U+6CEB.0	fR/S	 85.5
U+6CEB.0	fTotalStrokes	 8
U+6CEB.0	fCangjie	EYVI
U+6CEB.0	fFourCorner	30132
U+6CEB.0	fHakka	  hien2; hien3
U+6CEB.0	fCantonese	jyun5
U+6CEB.0	fMandarin	xuan4; xuan2; xian4; juan1
U+6CEB.0	fEnglish	 weep cry shine; glisten
U+6CEC.0	fUTF8	泬
U+6CEC.0	fCNS11643	2-2750
U+6CEC.0	fBig5	CCEF
U+6CEC.0	fGB	3-536C
U+6CEC.0	fR/S	 85.5
U+6CEC.0	fTotalStrokes	 8
U+6CEC.0	fCangjie	EJC
U+6CEC.0	fFourCorner	33187
U+6CEC.0	fHakka	  hiet8; giet7
U+6CEC.0	fCantonese	kyut3; hyut3; jyut6
U+6CEC.0	fMandarin	xue4; jue2
U+6CEC.0	fEnglish	stream of water
U+6CED.0	fUTF8	泭
U+6CED.0	fCNS11643	2-275E
U+6CED.0	fBig5	CCFD
U+6CED.0	fGB	3-5366
U+6CED.0	fR/S	 85.5
U+6CED.0	fTotalStrokes	 8
U+6CED.0	fCangjie	EODI
U+6CED.0	fFourCorner	34100
U+6CED.0	fHakka	 fu1; fu2
U+6CED.0	fCantonese	fu1; fu4
U+6CED.0	fMandarin	fu1; fu2
U+6CED.0	fEnglish	a raft
U+6CEE.0	fUTF8	泮
U+6CEE.0	fCNS11643	2-2752
U+6CEE.0	fBig5	CCF1
U+6CEE.0	fGB	0-637A
U+6CEE.0	fR/S	 85.5
U+6CEE.0	fTotalStrokes	 8
U+6CEE.0	fCangjie	EFQ
U+6CEE.0	fFourCorner	39150
U+6CEE.0	fHakka	  pan5; pan3
U+6CEE.0	fCantonese	pun3; bun6
U+6CEE.0	fMandarin	pan4
U+6CEE.0	fEnglish	[1] Zhou dynasty school; an institution of higher learning [2] dissolve; melt
U+6CEF.0	fUTF8	泯
U+6CEF.0	fCNS11643	1-4E5E
U+6CEF.0	fBig5	AA7B
U+6CEF.0	fGB	0-637D
U+6CEF.0	fR/S	 85.5
U+6CEF.0	fTotalStrokes	 8
U+6CEF.0	fCangjie	ERVP
U+6CEF.0	fFourCorner	37147
U+6CEF.0	fHakka	  min3; min2
U+6CEF.0	fCantonese	man5
U+6CEF.0	fMandarin	min3; mian4; mian2
U+6CEF.0	fEnglish	 destroy; eliminate; perish; vanish; put an end to
U+6CF0.0	fUTF8	泰
U+6CF0.0	fCNS11643	1-5639
U+6CF0.0	fBig5	AEF5
U+6CF0.0	fGB	0-4C29
U+6CF0.0	fR/S	 85.5
U+6CF0.0	fTotalStrokes	 9
U+6CF0.0	fCangjie	QKE
U+6CF0.0	fFourCorner	50132
U+6CF0.0	fHakka	  tai5
U+6CF0.0	fCantonese	taai3
U+6CF0.0	fMandarin	tai4
U+6CF0.0	fEnglish	 great; exaulted; superior big
U+6CF1.0	fUTF8	泱
U+6CF1.0	fCNS11643	1-4E57
U+6CF1.0	fBig5	AA74
U+6CF1.0	fGB	0-6373
U+6CF1.0	fR/S	 85.5
U+6CF1.0	fTotalStrokes	 8
U+6CF1.0	fCangjie	ELBK
U+6CF1.0	fFourCorner	35130
U+6CF1.0	fHakka	  jong1; rong1
U+6CF1.0	fCantonese	joeng1; ong1; ngong1
U+6CF1.0	fMandarin	yang1; ang3; yang3; ying1; ang4
U+6CF1.0	fEnglish	[1] great; big; expansive [2] quiet; calm; peace; ease
U+6CF2.0	fUTF8	泲
U+6CF2.0	fCNS11643	2-275F
U+6CF2.0	fBig5	CCFE
U+6CF2.0	fGB	3-5352
U+6CF2.0	fR/S	 85.5; 85.4
U+6CF2.0	fTotalStrokes	 8
U+6CF2.0	fCangjie	ELXH
U+6CF2.0	fFourCorner	35127
U+6CF2.0	fHakka	  zi5
U+6CF2.0	fCantonese	zai3; zi2 (also); zai2 (also)
U+6CF2.0	fMandarin	ji3
U+6CF2.0	fEnglish	 kind of white wine a river in heb
U+6CF3.0	fUTF8	泳
U+6CF3.0	fCNS11643	1-4E44
U+6CF3.0	fBig5	AA61
U+6CF3.0	fGB	0-533E
U+6CF3.0	fR/S	 85.5
U+6CF3.0	fTotalStrokes	 8
U+6CF3.0	fCangjie	EINE
U+6CF3.0	fFourCorner	33132
U+6CF3.0	fHakka	  jun3; run3; jun1
U+6CF3.0	fCantonese	wing6
U+6CF3.0	fMandarin	yong3
U+6CF3.0	fEnglish	 dive swim
U+6CF4.0	fUTF8	泴
U+6CF4.0	fCNS11643	3-2D4F
U+6CF4.0	fGB	5-5468
U+6CF4.0	fR/S	  85.5
U+6CF4.0	fTotalStrokes	 9; 8
U+6CF4.0	fCangjie	EBT
U+6CF4.0	fFourCorner	12100
U+6CF4.0	fCantonese	gun3; gwun3 (also)
U+6CF4.0	fMandarin	guan4
U+6CF5.0	fUTF8	泵
U+6CF5.0	fCNS11643	1-5229
U+6CF5.0	fBig5	ACA6
U+6CF5.0	fGB	0-3143
U+6CF5.0	fR/S	 85.5; 112.4
U+6CF5.0	fTotalStrokes	 9
U+6CF5.0	fCangjie	MRE
U+6CF5.0	fFourCorner	10232
U+6CF5.0	fHakka	 dun5; dun6; bang1
U+6CF5.0	fCantonese	bam1
U+6CF5.0	fMandarin	liu2; ping4; beng4; pin4
U+6CF5.0	fEnglish	[1] pump [2] prolong (time); delay (the completion of work)
U+6CF6.0	fUTF8	泶
U+6CF6.0	fGB	0-6D34
U+6CF6.0	fR/S	 85.5
U+6CF6.0	fTotalStrokes	 9
U+6CF6.0	fCangjie	FBE
U+6CF6.0	fHakka	 hok8; hok7
U+6CF6.0	fCantonese	hok6
U+6CF6.0	fMandarin	xue2; zhuo2; xiao4
U+6CF6.0	fEnglish	simplified form of &#x6FA9;
U+6CF7.0	fUTF8	泷
U+6CF7.0	fGB	0-6371
U+6CF7.0	fR/S	 85.5
U+6CF7.0	fTotalStrokes	 8
U+6CF7.0	fCangjie	EIKP
U+6CF7.0	fHakka	 lung2
U+6CF7.0	fCantonese	lung4; soeng1
U+6CF7.0	fMandarin	long2; long4; shuang1
U+6CF7.0	fEnglish	simplified form of &#x7027;
U+6CF8.0	fUTF8	泸
U+6CF8.0	fGB	0-6372
U+6CF8.0	fR/S	 85.5
U+6CF8.0	fTotalStrokes	 8
U+6CF8.0	fCangjie	EYS
U+6CF8.0	fHakka	 lu2
U+6CF8.0	fCantonese	lou4
U+6CF8.0	fMandarin	lu2
U+6CF8.0	fEnglish	simplified form of &#x7018;
U+6CF9.0	fUTF8	泹
U+6CF9.0	fCNS11643	2-276C
U+6CF9.0	fBig5	CD4C
U+6CF9.0	fGB	E-302D
U+6CF9.0	fR/S	 85.5
U+6CF9.0	fTotalStrokes	 8
U+6CF9.0	fCangjie	EAM
U+6CF9.0	fFourCorner	36112
U+6CF9.0	fCantonese	daan3; daan6
U+6CF9.0	fMandarin	dan4
U+6CFA.0	fUTF8	泺
U+6CFA.0	fGB	0-6378
U+6CFA.0	fR/S	 85.5
U+6CFA.0	fTotalStrokes	 8; 9
U+6CFA.0	fCangjie	EHVD
U+6CFA.0	fHakka	 lok8
U+6CFA.0	fCantonese	lok6; bok6; lik1
U+6CFA.0	fMandarin	luo4; lu4; bo2; pu2; po1; po4; li4; yue4; yao4; shuo4
U+6CFA.0	fEnglish	simplified form of &#x6FFC;
U+6CFB.0	fUTF8	泻
U+6CFB.0	fGB	0-503A
U+6CFB.0	fR/S	 85.5
U+6CFB.0	fTotalStrokes	 8
U+6CFB.0	fCangjie	EBYM
U+6CFB.0	fHakka	 sia5
U+6CFB.0	fCantonese	se3; se2 (also)
U+6CFB.0	fMandarin	xie4; xie3
U+6CFB.0	fEnglish	simplified form of &#x7009;
U+6CFC.0	fUTF8	泼
U+6CFC.0	fGB	0-4643
U+6CFC.0	fR/S	 85.5
U+6CFC.0	fTotalStrokes	 8
U+6CFC.0	fCangjie	EIVE
U+6CFC.0	fHakka	 pat7
U+6CFC.0	fCantonese	put3; but3
U+6CFC.0	fMandarin	po1; pai4; bo1
U+6CFC.0	fEnglish	simplified form of &#x6F51;
U+6CFC.1	fUTF8	泼
U+6CFC.1	fGB	0-4643
U+6CFC.1	fR/S	 85.5
U+6CFC.1	fTotalStrokes	 8
U+6CFC.1	fCangjie	EIVE
U+6CFC.1	fMandarin	po5
U+6CFD.0	fUTF8	泽
U+6CFD.0	fGB	0-5473
U+6CFD.0	fR/S	 85.5
U+6CFD.0	fTotalStrokes	 8
U+6CFD.0	fCangjie	EEQ
U+6CFD.0	fHakka	 cet8
U+6CFD.0	fCantonese	zaak6; sik1
U+6CFD.0	fMandarin	ze2; duo2; shi4; yi4
U+6CFD.0	fEnglish	simplified form of &#x6FA4;
U+6CFE.0	fUTF8	泾
U+6CFE.0	fGB	0-637E
U+6CFE.0	fR/S	 85.5
U+6CFE.0	fTotalStrokes	 8
U+6CFE.0	fCangjie	ENOM
U+6CFE.0	fHakka	 gin1
U+6CFE.0	fCantonese	ging1
U+6CFE.0	fMandarin	jing1; jing4
U+6CFE.0	fEnglish	simplified form of &#x6D87;
U+6CFF.0	fUTF8	泿
U+6CFF.0	fCNS11643	3-2D46
U+6CFF.0	fGB	3-5430
U+6CFF.0	fR/S	  85.6
U+6CFF.0	fTotalStrokes	 9
U+6CFF.0	fCangjie	EAV
U+6CFF.0	fFourCorner	37130
U+6CFF.0	fHakka	 gem2; gem5
U+6CFF.0	fCantonese	ngan4; an4 (also)
U+6CFF.0	fMandarin	yin2; yin4
U+6D00.0	fUTF8	洀
U+6D00.0	fCNS11643	2-2B5E
U+6D00.0	fBig5	CF7C
U+6D00.0	fGB	5-5543
U+6D00.0	fR/S	 85.6
U+6D00.0	fTotalStrokes	 9
U+6D00.0	fCangjie	EHBY
U+6D00.0	fFourCorner	37140
U+6D00.0	fCantonese	zau1; pun4
U+6D00.0	fMandarin	zhou1; pan2
U+6D01.0	fUTF8	洁
U+6D01.0	fCNS11643	2-2B61
U+6D01.0	fBig5	CFA1
U+6D01.0	fGB	0-3D60
U+6D01.0	fR/S	 85.6
U+6D01.0	fTotalStrokes	 9
U+6D01.0	fCangjie	EGR
U+6D01.0	fFourCorner	34161
U+6D01.0	fHakka	 get7; giet7
U+6D01.0	fCantonese	git3
U+6D01.0	fMandarin	jie2; ji2
U+6D01.0	fEnglish	simplified form of &#x6F54;
U+6D02.0	fUTF8	洂
U+6D02.0	fCNS11643	3-2D50
U+6D02.0	fGB	5-5547
U+6D02.0	fR/S	 85.6
U+6D02.0	fTotalStrokes	 9
U+6D02.0	fCangjie	EYLC
U+6D02.0	fFourCorner	30130
U+6D02.0	fHakka	 jit8; rit8
U+6D02.0	fCantonese	jik6
U+6D02.0	fMandarin	yi4
U+6D02.0	fEnglish	a Chinese family name
U+6D03.0	fUTF8	洃
U+6D03.0	fCNS11643	2-2B64
U+6D03.0	fBig5	CFA4
U+6D03.0	fGB	5-5537
U+6D03.0	fR/S	 85.6
U+6D03.0	fTotalStrokes	 9
U+6D03.0	fCangjie	EKF
U+6D03.0	fFourCorner	34189
U+6D03.0	fHakka	 foi1
U+6D03.0	fCantonese	fui1
U+6D03.0	fMandarin	hui1
U+6D04.0	fUTF8	洄
U+6D04.0	fCNS11643	2-2B59
U+6D04.0	fBig5	CF77
U+6D04.0	fGB	0-6427
U+6D04.0	fR/S	 85.6
U+6D04.0	fTotalStrokes	 9
U+6D04.0	fCangjie	EWR
U+6D04.0	fFourCorner	36100
U+6D04.0	fHakka	  fui2; fi2
U+6D04.0	fCantonese	wui4
U+6D04.0	fMandarin	hui2; kui4; hui4
U+6D04.0	fEnglish	whirling (water)
U+6D05.0	fUTF8	洅
U+6D05.0	fCNS11643	3-2D48
U+6D05.0	fGB	5-5534
U+6D05.0	fR/S	 85.6
U+6D05.0	fTotalStrokes	 9
U+6D05.0	fCangjie	EMGB
U+6D05.0	fFourCorner	31150
U+6D05.0	fCantonese	zoi3
U+6D05.0	fMandarin	zui3; zai4
U+6D06.0	fUTF8	洆
U+6D06.0	fCNS11643	3-2D4A
U+6D06.0	fGB	E-302E
U+6D06.0	fR/S	 85.6
U+6D06.0	fTotalStrokes	 9
U+6D06.0	fCangjie	ENEM
U+6D06.0	fFourCorner	37110
U+6D06.0	fCantonese	sing4
U+6D06.0	fMandarin	cheng2
U+6D07.0	fUTF8	洇
U+6D07.0	fCNS11643	2-2B67
U+6D07.0	fBig5	CFA7
U+6D07.0	fGB	0-6426
U+6D07.0	fR/S	 85.6
U+6D07.0	fTotalStrokes	 9
U+6D07.0	fCangjie	EWK
U+6D07.0	fFourCorner	36100
U+6D07.0	fHakka	 jin1; jen1
U+6D07.0	fCantonese	jan1
U+6D07.0	fMandarin	yin1; ye1; yan1
U+6D08.0	fUTF8	洈
U+6D08.0	fCNS11643	2-2B6A
U+6D08.0	fBig5	CFAA
U+6D08.0	fGB	5-5546
U+6D08.0	fR/S	 85.6
U+6D08.0	fTotalStrokes	 9
U+6D08.0	fCangjie	ENMU
U+6D08.0	fFourCorner	37112
U+6D08.0	fHakka	 ngui2
U+6D08.0	fCantonese	ngai4; ai4 (also); gwai2
U+6D08.0	fMandarin	wei2; gui3
U+6D08.0	fEnglish	the name of a small river in Hupeh
U+6D09.0	fUTF8	洉
U+6D09.0	fCNS11643	2-2B6C
U+6D09.0	fBig5	CFAC
U+6D09.0	fGB	5-5541
U+6D09.0	fR/S	 85.6
U+6D09.0	fTotalStrokes	 9
U+6D09.0	fCangjie	EHMR
U+6D09.0	fFourCorner	32161
U+6D09.0	fCantonese	hau6
U+6D09.0	fMandarin	hou4
U+6D0A.0	fUTF8	洊
U+6D0A.0	fCNS11643	2-2B56
U+6D0A.0	fBig5	CF74
U+6D0A.0	fGB	3-5378
U+6D0A.0	fR/S	 85.6
U+6D0A.0	fTotalStrokes	 9
U+6D0A.0	fCangjie	EKLD
U+6D0A.0	fFourCorner	34147
U+6D0A.0	fHakka	  zien5
U+6D0A.0	fCantonese	zin3; zin6
U+6D0A.0	fMandarin	jian4; cun4; cun1; cun2
U+6D0A.0	fEnglish	flowing water
U+6D0B.0	fUTF8	洋
U+6D0B.0	fCNS11643	1-5179
U+6D0B.0	fBig5	AC76
U+6D0B.0	fGB	0-5173
U+6D0B.0	fR/S	 85.6
U+6D0B.0	fTotalStrokes	 9
U+6D0B.0	fCangjie	ETQ
U+6D0B.0	fFourCorner	38151
U+6D0B.0	fHakka	  jong2; rong2; jong3
U+6D0B.0	fCantonese	joeng4; joeng2 (also)
U+6D0B.0	fMandarin	yang2; xiang2; yang3
U+6D0B.0	fEnglish	[1] ocean; sea [2] foreign; Western; Occidental [3] imported
U+6D0C.0	fUTF8	洌
U+6D0C.0	fCNS11643	1-517E
U+6D0C.0	fBig5	AC7B
U+6D0C.0	fGB	0-6423
U+6D0C.0	fR/S	 85.6
U+6D0C.0	fTotalStrokes	 9
U+6D0C.0	fCangjie	EMNN
U+6D0C.0	fFourCorner	32100
U+6D0C.0	fHakka	  liet8; let8
U+6D0C.0	fCantonese	lit6
U+6D0C.0	fMandarin	lie4; li4
U+6D0C.0	fEnglish	clear and transparent (liquid)
U+6D0D.0	fUTF8	洍
U+6D0D.0	fCNS11643	2-302C
U+6D0D.0	fBig5	D249
U+6D0D.0	fGB	5-5551
U+6D0D.0	fR/S	 85.7
U+6D0D.0	fTotalStrokes	 10
U+6D0D.0	fCangjie	ESLL
U+6D0D.0	fFourCorner	31117
U+6D0D.0	fCantonese	ci5; ji4
U+6D0D.0	fMandarin	si4
U+6D0E.0	fUTF8	洎
U+6D0E.0	fCNS11643	1-5230
U+6D0E.0	fBig5	ACAD
U+6D0E.0	fGB	0-6429
U+6D0E.0	fR/S	 85.6
U+6D0E.0	fTotalStrokes	 9
U+6D0E.0	fCangjie	EHBU
U+6D0E.0	fFourCorner	36100
U+6D0E.0	fHakka	 ki5; koi3
U+6D0E.0	fCantonese	gei3; gei6 (also); kei3
U+6D0E.0	fMandarin	ji4
U+6D0E.0	fEnglish	[1] until; till; up to [2] soup; meat broth [3] soak; drench
U+6D0F.0	fUTF8	洏
U+6D0F.0	fCNS11643	2-2B65
U+6D0F.0	fBig5	CFA5
U+6D0F.0	fGB	3-5377
U+6D0F.0	fR/S	 85.6
U+6D0F.0	fTotalStrokes	 9
U+6D0F.0	fCangjie	EMBL
U+6D0F.0	fFourCorner	31127
U+6D0F.0	fHakka	 ji2; ri2; i2
U+6D0F.0	fCantonese	ji4
U+6D0F.0	fMandarin	er2
U+6D0F.0	fEnglish	[1] tearful; weeping [2] cook food thoroughly
U+6D10.0	fUTF8	洐
U+6D10.0	fCNS11643	2-2B6D
U+6D10.0	fBig5	CFAD
U+6D10.0	fGB	5-5542
U+6D10.0	fR/S	 85.6
U+6D10.0	fTotalStrokes	 9
U+6D10.0	fCangjie	EHON
U+6D10.0	fFourCorner	32112
U+6D10.0	fHakka	 hang2
U+6D10.0	fCantonese	hang4
U+6D10.0	fMandarin	xing2
U+6D11.0	fUTF8	洑
U+6D11.0	fCNS11643	2-2B5D
U+6D11.0	fBig5	CF7B
U+6D11.0	fGB	3-5425
U+6D11.0	fR/S	 85.6
U+6D11.0	fTotalStrokes	 9
U+6D11.0	fCangjie	EOIK
U+6D11.0	fFourCorner	33134
U+6D11.0	fHakka	  fuk8
U+6D11.0	fCantonese	fuk6; fu4; fu3
U+6D11.0	fMandarin	fu2; fu4
U+6D11.0	fEnglish	[1] whirlpool; vortex [2] undercurrent
U+6D12.0	fUTF8	洒
U+6D12.0	fCNS11643	2-2B55
U+6D12.0	fBig5	CF73
U+6D12.0	fGB	0-4877
U+6D12.0	fR/S	 85.6
U+6D12.0	fTotalStrokes	 9
U+6D12.0	fCangjie	EMCW
U+6D12.0	fFourCorner	31160
U+6D12.0	fHakka	  sa3; sa5; sai5; sha3
U+6D12.0	fCantonese	saa2; ceoi1
U+6D12.0	fMandarin	sa3; sha3; cui3; shai4; sen3; xun4
U+6D12.0	fEnglish	[1] simplified form of &#x7051; [2] I; me
U+6D12.1	fUTF8	洒
U+6D12.1	fCNS11643	2-2B55
U+6D12.1	fBig5	CF73
U+6D12.1	fGB	0-4877
U+6D12.1	fR/S	 85.6
U+6D12.1	fTotalStrokes	 9
U+6D12.1	fCangjie	EMCW
U+6D12.1	fFourCorner	31160
U+6D12.1	fMandarin	xian3
U+6D12.1	fEnglish	[1] respectful [2] deep [3] alarmed; surprised
U+6D12.2	fUTF8	洒
U+6D12.2	fCNS11643	2-2B55
U+6D12.2	fBig5	CF73
U+6D12.2	fGB	0-4877
U+6D12.2	fR/S	 85.6
U+6D12.2	fTotalStrokes	 9
U+6D12.2	fCangjie	EMCW
U+6D12.2	fFourCorner	31160
U+6D12.2	fHakka	 se3
U+6D12.2	fCantonese	sai2; sin2
U+6D12.2	fMandarin	xi3
U+6D12.2	fEnglish	same as &#x6D17;
U+6D13.0	fUTF8	洓
U+6D13.0	fCNS11643	3-2D49
U+6D13.0	fGB	3-5376
U+6D13.0	fR/S	 85.6
U+6D13.0	fTotalStrokes	 9
U+6D13.0	fCangjie	EDB
U+6D13.0	fFourCorner	35190
U+6D13.0	fCantonese	saak3
U+6D13.0	fMandarin	suo3; qi4; zi4
U+6D14.0	fUTF8	洔
U+6D14.0	fCNS11643	3-2D47
U+6D14.0	fGB	3-5374
U+6D14.0	fR/S	 85.6
U+6D14.0	fTotalStrokes	 9
U+6D14.0	fCangjie	EGDI
U+6D14.0	fFourCorner	34140
U+6D14.0	fHakka	 zhi3; shi5; zih3; sii5; tje3
U+6D14.0	fCantonese	zi2
U+6D14.0	fMandarin	zhi3; zhi4; shi4
U+6D14.0	fEnglish	an islet in a river
U+6D15.0	fUTF8	洕
U+6D15.0	fCNS11643	3-2D4D
U+6D15.0	fGB	3-5426
U+6D15.0	fR/S	  85.6
U+6D15.0	fTotalStrokes	 9
U+6D15.0	fCangjie	ECB
U+6D15.0	fFourCorner	38120
U+6D15.0	fCantonese	jan5
U+6D15.0	fMandarin	yin1
U+6D16.0	fUTF8	洖
U+6D16.0	fCNS11643	2-3047
U+6D16.0	fBig5	D264
U+6D16.0	fGB	E-302F
U+6D16.0	fR/S	 85.7
U+6D16.0	fTotalStrokes	 10
U+6D16.0	fCangjie	ERVK
U+6D16.0	fFourCorner	36134
U+6D16.0	fHakka	 ng3
U+6D16.0	fCantonese	ng4
U+6D16.0	fMandarin	wu2
U+6D17.0	fUTF8	洗
U+6D17.0	fCNS11643	1-5223
U+6D17.0	fBig5	AC7E
U+6D17.0	fGB	0-4F34
U+6D17.0	fR/S	 85.6
U+6D17.0	fTotalStrokes	 9
U+6D17.0	fCangjie	EHGU
U+6D17.0	fFourCorner	34111
U+6D17.0	fHakka	  se3; sien3
U+6D17.0	fCantonese	sai2; sin2
U+6D17.0	fMandarin	xi3
U+6D17.0	fEnglish	[1] wash; rinse; clean; cleanse; purify; clear [2] baptise
U+6D17.1	fUTF8	洗
U+6D17.1	fCNS11643	1-5223
U+6D17.1	fBig5	AC7E
U+6D17.1	fGB	0-4F34
U+6D17.1	fR/S	 85.6
U+6D17.1	fTotalStrokes	 9
U+6D17.1	fCangjie	EHGU
U+6D17.1	fFourCorner	34111
U+6D17.1	fMandarin	xian3
U+6D17.1	fEnglish	a Chinese family name
U+6D18.0	fUTF8	洘
U+6D18.0	fCNS11643	2-2B62
U+6D18.0	fBig5	CFA2
U+6D18.0	fGB	3-5375
U+6D18.0	fR/S	 85.6
U+6D18.0	fTotalStrokes	 9
U+6D18.0	fCangjie	EJKS
U+6D18.0	fFourCorner	34127
U+6D18.0	fCantonese	haau2
U+6D18.0	fMandarin	kao3
U+6D19.0	fUTF8	洙
U+6D19.0	fCNS11643	2-2B5A
U+6D19.0	fBig5	CF78
U+6D19.0	fGB	0-6428
U+6D19.0	fR/S	 85.6
U+6D19.0	fTotalStrokes	 9
U+6D19.0	fCangjie	EHJD
U+6D19.0	fFourCorner	35190
U+6D19.0	fHakka	 zhu1; zu1; ki1
U+6D19.0	fCantonese	zyu1; syu4
U+6D19.0	fMandarin	zhu1
U+6D19.0	fEnglish	the name of a river in Shantung
U+6D1A.0	fUTF8	洚
U+6D1A.0	fCNS11643	2-2B5C
U+6D1A.0	fBig5	CF7A
U+6D1A.0	fGB	0-642E
U+6D1A.0	fR/S	 85.6
U+6D1A.0	fTotalStrokes	 9
U+6D1A.0	fCangjie	EHEQ
U+6D1A.0	fFourCorner	37154
U+6D1A.0	fHakka	 gong5
U+6D1A.0	fCantonese	gong3; hung4 (also); hong4
U+6D1A.0	fMandarin	jiang4; hong2; xiang2; hong4; zong1
U+6D1A.0	fEnglish	 flood
U+6D1B.0	fUTF8	洛
U+6D1B.0	fCNS11643	1-5228
U+6D1B.0	fBig5	ACA5
U+6D1B.0	fGB	0-4265
U+6D1B.0	fR/S	 85.6
U+6D1B.0	fTotalStrokes	 9
U+6D1B.0	fCangjie	EHER
U+6D1B.0	fFourCorner	37164
U+6D1B.0	fHakka	 lok7; lok8
U+6D1B.0	fCantonese	lok3; lok6 (also)
U+6D1B.0	fMandarin	luo4
U+6D1B.0	fEnglish	the name of a river in Shaanxi province
U+6D1C.0	fUTF8	洜
U+6D1C.0	fCNS11643	3-317A
U+6D1C.0	fGB	E-3030
U+6D1C.0	fR/S	 85.6
U+6D1C.0	fTotalStrokes	 10
U+6D1C.0	fCangjie	HRE
U+6D1C.0	fFourCorner	27230
U+6D1C.0	fCantonese	lok3
U+6D1C.0	fMandarin	luo4
U+6D1D.0	fUTF8	洝
U+6D1D.0	fCNS11643	2-2B5F
U+6D1D.0	fBig5	CF7D
U+6D1D.0	fGB	3-542F
U+6D1D.0	fR/S	 85.6
U+6D1D.0	fTotalStrokes	 9
U+6D1D.0	fCangjie	EJV
U+6D1D.0	fFourCorner	33144
U+6D1D.0	fCantonese	on3; ngon3 (also); aan3; aat3
U+6D1D.0	fMandarin	an4; yan4; e4
U+6D1E.0	fUTF8	洞
U+6D1E.0	fCNS11643	1-5222
U+6D1E.0	fBig5	AC7D
U+6D1E.0	fGB	0-3634
U+6D1E.0	fR/S	 85.6
U+6D1E.0	fTotalStrokes	 9
U+6D1E.0	fCangjie	EBMR
U+6D1E.0	fFourCorner	37120
U+6D1E.0	fHakka	  tung5;  tung3; tung6
U+6D1E.0	fCantonese	dung6; tung4; dung2 (also)
U+6D1E.0	fMandarin	dong4; tong2
U+6D1E.0	fEnglish	[1] cave; hole [2] penetrate; see through
U+6D1F.0	fUTF8	洟
U+6D1F.0	fCNS11643	2-2B52
U+6D1F.0	fBig5	CF70
U+6D1F.0	fGB	3-537B
U+6D1F.0	fR/S	 85.6
U+6D1F.0	fTotalStrokes	 9
U+6D1F.0	fCangjie	EKN
U+6D1F.0	fFourCorner	35132
U+6D1F.0	fHakka	 ji2; ri2; ze2; je2; i2
U+6D1F.0	fCantonese	tai3 (also); ji4
U+6D1F.0	fMandarin	yi2; ti34; ti4; ti3
U+6D1F.0	fEnglish	 snivel; mucus from nose; nasal mucus
U+6D20.0	fUTF8	洠
U+6D20.0	fCNS11643	2-2B68
U+6D20.0	fBig5	CFA8
U+6D20.0	fGB	5-5549
U+6D20.0	fR/S	 85.6
U+6D20.0	fTotalStrokes	 9
U+6D20.0	fCangjie	EIHQ
U+6D20.0	fFourCorner	33150
U+6D20.0	fCantonese	mau4; maau4
U+6D20.0	fMandarin	mou2
U+6D21.0	fUTF8	洡
U+6D21.0	fCNS11643	3-2D4B
U+6D21.0	fGB	3-5373
U+6D21.0	fR/S	  85.6
U+6D21.0	fTotalStrokes	 9
U+6D21.0	fCangjie	EQD
U+6D21.0	fFourCorner	35190
U+6D21.0	fCantonese	leoi6
U+6D21.0	fMandarin	lei3; lei4
U+6D22.0	fUTF8	洢
U+6D22.0	fCNS11643	2-2B6B
U+6D22.0	fBig5	CFAB
U+6D22.0	fGB	5-5540
U+6D22.0	fR/S	 85.6
U+6D22.0	fTotalStrokes	 9
U+6D22.0	fCangjie	EOSK
U+6D22.0	fFourCorner	37157
U+6D22.0	fHakka	 ji1; ri1
U+6D22.0	fCantonese	ji1
U+6D22.0	fMandarin	yi1
U+6D22.0	fEnglish	a river in Honan
U+6D23.0	fUTF8	洣
U+6D23.0	fCNS11643	3-2D45
U+6D23.0	fGB	3-542E
U+6D23.0	fR/S	  85.6
U+6D23.0	fTotalStrokes	 9
U+6D23.0	fCangjie	EFD
U+6D23.0	fFourCorner	39194
U+6D23.0	fHakka	 mi3
U+6D23.0	fCantonese	mai5
U+6D23.0	fMandarin	mi3
U+6D23.0	fEnglish	a river in Hunan
U+6D24.0	fUTF8	洤
U+6D24.0	fCNS11643	3-2D57
U+6D24.0	fGB	5-5544
U+6D24.0	fR/S	 85.6
U+6D24.0	fTotalStrokes	 9
U+6D24.0	fCangjie	EOMG
U+6D24.0	fFourCorner	38110
U+6D24.0	fHakka	  can2
U+6D24.0	fCantonese	cyun4
U+6D24.0	fMandarin	quan2
U+6D24.0	fEnglish	a fountain or spring
U+6D25.0	fUTF8	津
U+6D25.0	fCNS11643	1-517D
U+6D25.0	fBig5	AC7A
U+6D25.0	fGB	0-3D72
U+6D25.0	fR/S	 85.6
U+6D25.0	fTotalStrokes	 9
U+6D25.0	fCangjie	ELQ
U+6D25.0	fFourCorner	35107
U+6D25.0	fHakka	  zin1; ngiap7
U+6D25.0	fCantonese	zeon1
U+6D25.0	fMandarin	jin1
U+6D25.0	fEnglish	[1] ferry [2] juicy; tasty [3] saliva [4] sweat; perspire [5] used in combination [6] short for Tientsin
U+6D26.0	fUTF8	洦
U+6D26.0	fCNS11643	3-2D53
U+6D26.0	fGB	5-5536
U+6D26.0	fR/S	 85.6
U+6D26.0	fTotalStrokes	 9
U+6D26.0	fCangjie	EMA
U+6D26.0	fFourCorner	31160
U+6D26.0	fCantonese	paak3
U+6D26.0	fMandarin	mo4; po4; bo2
U+6D27.0	fUTF8	洧
U+6D27.0	fCNS11643	1-522B
U+6D27.0	fBig5	ACA8
U+6D27.0	fGB	0-6422
U+6D27.0	fR/S	 85.6
U+6D27.0	fTotalStrokes	 9
U+6D27.0	fCangjie	EKB
U+6D27.0	fFourCorner	34127
U+6D27.0	fHakka	 ji3; rui3; wui3; wi3; vui3
U+6D27.0	fCantonese	fui2; wai5
U+6D27.0	fMandarin	wei3
U+6D27.0	fEnglish	the name of a river in Honan
U+6D28.0	fUTF8	洨
U+6D28.0	fCNS11643	2-2B4F
U+6D28.0	fBig5	CF6D
U+6D28.0	fGB	3-5429
U+6D28.0	fR/S	 85.6
U+6D28.0	fTotalStrokes	 9
U+6D28.0	fCangjie	EYCK
U+6D28.0	fFourCorner	30148
U+6D28.0	fCantonese	ngaau4; aau4 (also); haau4
U+6D28.0	fMandarin	xiao2; xiao4; jiao1
U+6D28.0	fEnglish	river in Hebei province
U+6D29.0	fUTF8	洩
U+6D29.0	fCNS11643	1-522D
U+6D29.0	fBig5	ACAA
U+6D29.0	fGB	8-2F6B
U+6D29.0	fR/S	 85.6
U+6D29.0	fTotalStrokes	 9
U+6D29.0	fCangjie	ELWP
U+6D29.0	fFourCorner	35106
U+6D29.0	fHakka	  siet7; siap7; siat7; set7; ji3; ri5; ji5; siet8; zit8
U+6D29.0	fCantonese	jai6; sit3
U+6D29.0	fMandarin	xie4; yi4
U+6D29.0	fEnglish	 drain; leak; drip; vent; release; leak out; dissipate
U+6D2A.0	fUTF8	洪
U+6D2A.0	fCNS11643	1-517B
U+6D2A.0	fBig5	AC78
U+6D2A.0	fGB	0-3A69
U+6D2A.0	fR/S	 85.6
U+6D2A.0	fTotalStrokes	 9
U+6D2A.0	fCangjie	ETC
U+6D2A.0	fFourCorner	34181
U+6D2A.0	fHakka	  fung2; fung3
U+6D2A.0	fCantonese	hung4
U+6D2A.0	fMandarin	hong2; jiang4
U+6D2A.0	fEnglish	[1] great; vast; immense; magnificent [2] flood; turbulent waters; torrents [3] a Chinese family name
U+6D2B.0	fUTF8	洫
U+6D2B.0	fCNS11643	1-5231
U+6D2B.0	fBig5	ACAE
U+6D2B.0	fGB	0-642A
U+6D2B.0	fR/S	 85.6
U+6D2B.0	fTotalStrokes	 9
U+6D2B.0	fCangjie	EHBT
U+6D2B.0	fFourCorner	37110
U+6D2B.0	fHakka	  sit7; sut7
U+6D2B.0	fCantonese	gwik1; kwik1
U+6D2B.0	fMandarin	xu4; yi4
U+6D2B.0	fEnglish	 ditch; moat
U+6D2C.0	fUTF8	洬
U+6D2C.0	fCNS11643	2-2B69
U+6D2C.0	fBig5	CFA9
U+6D2C.0	fGB	3-5427
U+6D2C.0	fR/S	 85.6
U+6D2C.0	fTotalStrokes	 9
U+6D2C.0	fCangjie	EHNN
U+6D2C.0	fFourCorner	37110
U+6D2C.0	fHakka	 siuk7
U+6D2C.0	fCantonese	suk1; sok3
U+6D2C.0	fMandarin	shuo4; du4
U+6D2C.0	fEnglish	the noise of wind and rain
U+6D2D.0	fUTF8	洭
U+6D2D.0	fCNS11643	2-2B51
U+6D2D.0	fBig5	CF6F
U+6D2D.0	fGB	3-5372
U+6D2D.0	fR/S	 85.6
U+6D2D.0	fTotalStrokes	 9
U+6D2D.0	fCangjie	ESMG
U+6D2D.0	fFourCorner	31111
U+6D2D.0	fHakka	 kiong1
U+6D2D.0	fCantonese	hong1
U+6D2D.0	fMandarin	kuang1
U+6D2D.0	fEnglish	a river in Hunan
U+6D2E.0	fUTF8	洮
U+6D2E.0	fCNS11643	1-522E
U+6D2E.0	fBig5	ACAB
U+6D2E.0	fGB	0-642C
U+6D2E.0	fR/S	 85.6
U+6D2E.0	fTotalStrokes	 9
U+6D2E.0	fCangjie	ELMO
U+6D2E.0	fFourCorner	32113
U+6D2E.0	fHakka	 tau2; to2
U+6D2E.0	fCantonese	tou4; jiu4; tou1
U+6D2E.0	fMandarin	dao4; tao1; tao2; zhao4
U+6D2E.0	fEnglish	the name of a river in Kansu province
U+6D2E.1	fUTF8	洮
U+6D2E.1	fCNS11643	1-522E
U+6D2E.1	fBig5	ACAB
U+6D2E.1	fGB	0-642C
U+6D2E.1	fR/S	 85.6
U+6D2E.1	fTotalStrokes	 9
U+6D2E.1	fCangjie	ELMO
U+6D2E.1	fFourCorner	32113
U+6D2E.1	fMandarin	yao2
U+6D2E.1	fEnglish	the name of a lake in Kansu province
U+6D2F.0	fUTF8	洯
U+6D2F.0	fCNS11643	2-3041
U+6D2F.0	fBig5	D25E
U+6D2F.0	fGB	5-5469
U+6D2F.0	fR/S	 85.6
U+6D2F.0	fTotalStrokes	 10; 9
U+6D2F.0	fCangjie	QHE
U+6D2F.0	fFourCorner	57232
U+6D2F.0	fHakka	 giet7; get7
U+6D2F.0	fCantonese	kit3; git3; kit1
U+6D2F.0	fMandarin	qie4; jie2
U+6D2F.0	fEnglish	name of a river in Yung-chiu
U+6D30.0	fUTF8	洰
U+6D30.0	fCNS11643	2-2768
U+6D30.0	fBig5	CD48
U+6D30.0	fGB	5-5474
U+6D30.0	fR/S	 85.6; 85.5
U+6D30.0	fTotalStrokes	 9
U+6D30.0	fCangjie	ESS
U+6D30.0	fFourCorner	31117
U+6D30.0	fCantonese	keoi4; geoi6
U+6D30.0	fMandarin	ju4
U+6D31.0	fUTF8	洱
U+6D31.0	fCNS11643	1-5221
U+6D31.0	fBig5	AC7C
U+6D31.0	fGB	0-367D
U+6D31.0	fR/S	 85.6
U+6D31.0	fTotalStrokes	 9
U+6D31.0	fCangjie	ESJ
U+6D31.0	fFourCorner	31140
U+6D31.0	fHakka	  ngi3
U+6D31.0	fCantonese	ji5; nei2 (also); nei5; lei5 (also); lei2 (also)
U+6D31.0	fMandarin	er3; mi3; er4
U+6D31.0	fEnglish	[1] the name of a lake in Yunnan [2] the name of a river in Yunnan
U+6D32.0	fUTF8	洲
U+6D32.0	fCNS11643	1-517A
U+6D32.0	fBig5	AC77
U+6D32.0	fGB	0-565E
U+6D32.0	fR/S	 85.6
U+6D32.0	fTotalStrokes	 9
U+6D32.0	fCangjie	EILL
U+6D32.0	fFourCorner	32100
U+6D32.0	fHakka	  zhiu1; zu1; ziu1; zhu1
U+6D32.0	fCantonese	zau1
U+6D32.0	fMandarin	zhou1
U+6D32.0	fEnglish	[1] continent [2] island; islet
U+6D33.0	fUTF8	洳
U+6D33.0	fCNS11643	2-2B58
U+6D33.0	fBig5	CF76
U+6D33.0	fGB	0-6432
U+6D33.0	fR/S	 85.6
U+6D33.0	fTotalStrokes	 9
U+6D33.0	fCangjie	EVR
U+6D33.0	fFourCorner	36100
U+6D33.0	fHakka	 ji2; ri2
U+6D33.0	fCantonese	jyu6; jyu5
U+6D33.0	fMandarin	ru2; ru4
U+6D33.0	fEnglish	damp; moist; oozy
U+6D34.0	fUTF8	洴
U+6D34.0	fCNS11643	2-2B50
U+6D34.0	fBig5	CF6E
U+6D34.0	fGB	3-542D
U+6D34.0	fR/S	 85.6
U+6D34.0	fTotalStrokes	 9
U+6D34.0	fCangjie	ETT
U+6D34.0	fFourCorner	38141
U+6D34.0	fHakka	  pin2; bin5
U+6D34.0	fCantonese	ping4; paang1
U+6D34.0	fMandarin	ping2; beng4
U+6D34.0	fEnglish	[1] the sound of silf, etc. floating in the wind [2] wash; bleach
U+6D35.0	fUTF8	洵
U+6D35.0	fCNS11643	1-522F
U+6D35.0	fBig5	ACAC
U+6D35.0	fGB	0-642D
U+6D35.0	fR/S	 85.6
U+6D35.0	fTotalStrokes	 9
U+6D35.0	fCangjie	EPA
U+6D35.0	fFourCorner	37120
U+6D35.0	fHakka	 sun2
U+6D35.0	fCantonese	seon1; hyun3
U+6D35.0	fMandarin	xun2; xuan4
U+6D35.0	fEnglish	true; real; truly; really; certainly
U+6D36.0	fUTF8	洶
U+6D36.0	fCNS11643	1-5227
U+6D36.0	fBig5	ACA4
U+6D36.0	fGB	E-3031
U+6D36.0	fR/S	 85.6
U+6D36.0	fTotalStrokes	 9
U+6D36.0	fCangjie	EPUK
U+6D36.0	fFourCorner	37120
U+6D36.0	fHakka	 hiung1; hiung3
U+6D36.0	fCantonese	hung1
U+6D36.0	fMandarin	xiong1; song3
U+6D36.0	fEnglish	[1] unquiet; restless; disturbed; tumultuous [2] noisy; uproarious; clamorous
U+6D37.0	fUTF8	洷
U+6D37.0	fCNS11643	2-2B63
U+6D37.0	fBig5	CFA3
U+6D37.0	fGB	3-537D
U+6D37.0	fR/S	 85.6
U+6D37.0	fTotalStrokes	 9
U+6D37.0	fCangjie	EMIG
U+6D37.0	fFourCorner	31114
U+6D37.0	fHakka	 zit7; zi5
U+6D37.0	fCantonese	zat6; zi3; dit6
U+6D37.0	fMandarin	zhi4
U+6D38.0	fUTF8	洸
U+6D38.0	fCNS11643	1-522C
U+6D38.0	fBig5	ACA9
U+6D38.0	fGB	3-5422
U+6D38.0	fR/S	 85.6
U+6D38.0	fTotalStrokes	 9
U+6D38.0	fCangjie	EFMU
U+6D38.0	fFourCorner	39111
U+6D38.0	fHakka	 gong1; wong1; fong3; kwong1; gwong1
U+6D38.0	fCantonese	gwong1; gong1 (also)
U+6D38.0	fMandarin	guang1; wang1; huang2; huang3
U+6D38.0	fEnglish	[1] sparkle; glitter (water) [2] name of a river in Shantung [3] cavalierly; bravely; be martial-looking
U+6D39.0	fUTF8	洹
U+6D39.0	fCNS11643	1-522A
U+6D39.0	fBig5	ACA7
U+6D39.0	fGB	0-6421
U+6D39.0	fR/S	 85.6
U+6D39.0	fTotalStrokes	 9
U+6D39.0	fCangjie	EMAM
U+6D39.0	fFourCorner	31116
U+6D39.0	fHakka	 jen2; ren2; fan2; jan2
U+6D39.0	fCantonese	wun4; jyun4 (also)
U+6D39.0	fMandarin	huan2; yuan2
U+6D39.0	fEnglish	a river in Henan province
U+6D3A.0	fUTF8	洺
U+6D3A.0	fCNS11643	2-2B5B
U+6D3A.0	fBig5	CF79
U+6D3A.0	fGB	3-5428
U+6D3A.0	fR/S	 85.6
U+6D3A.0	fTotalStrokes	 9
U+6D3A.0	fCangjie	ENIR
U+6D3A.0	fFourCorner	37162
U+6D3A.0	fHakka	 miang2; min2
U+6D3A.0	fCantonese	ming4
U+6D3A.0	fMandarin	ming2
U+6D3A.0	fEnglish	river in Hebei province
U+6D3B.0	fUTF8	活
U+6D3B.0	fCNS11643	1-5224
U+6D3B.0	fBig5	ACA1
U+6D3B.0	fGB	0-3B6E
U+6D3B.0	fR/S	 85.6
U+6D3B.0	fTotalStrokes	 9
U+6D3B.0	fCangjie	EHJR
U+6D3B.0	fFourCorner	32164
U+6D3B.0	fHakka	  fat8
U+6D3B.0	fCantonese	wut6; gut3; gwut3 (also); kut3
U+6D3B.0	fMandarin	huo2; kuo4; guo1
U+6D3B.0	fEnglish	[1] live; exist; survive; be alive [2] save the life of [3] active; lively; vivacious [4] movable; mobile; flexible [5] work
U+6D3B.1	fUTF8	活
U+6D3B.1	fCNS11643	1-5224
U+6D3B.1	fBig5	ACA1
U+6D3B.1	fGB	0-3B6E
U+6D3B.1	fR/S	 85.6
U+6D3B.1	fTotalStrokes	 9
U+6D3B.1	fCangjie	EHJR
U+6D3B.1	fFourCorner	32164
U+6D3B.1	fMandarin	huo5
U+6D3C.0	fUTF8	洼
U+6D3C.0	fCNS11643	2-2B53
U+6D3C.0	fBig5	CF71
U+6D3C.0	fGB	0-4D5D
U+6D3C.0	fR/S	 85.6
U+6D3C.0	fTotalStrokes	 9
U+6D3C.0	fCangjie	EGG
U+6D3C.0	fFourCorner	34114
U+6D3C.0	fHakka	  wa1; va1
U+6D3C.0	fCantonese	gwai1; waa1
U+6D3C.0	fMandarin	wa1; ke3; gui1
U+6D3C.0	fEnglish	simplified form of &#x7AAA;
U+6D3D.0	fUTF8	洽
U+6D3D.0	fCNS11643	1-5225
U+6D3D.0	fBig5	ACA2
U+6D3D.0	fGB	0-4722
U+6D3D.0	fR/S	 85.6
U+6D3D.0	fTotalStrokes	 9
U+6D3D.0	fCangjie	EOMR
U+6D3D.0	fFourCorner	38161
U+6D3D.0	fHakka	 giap7; hiap7; hap7; hap8
U+6D3D.0	fCantonese	hap1; hap6 (also); haap6; ap1
U+6D3D.0	fMandarin	qia4; xia2; he2; ge2
U+6D3D.0	fEnglish	[1] spread; diffuse [2] harmony; agreement [3] negotiate; consult
U+6D3E.0	fUTF8	派
U+6D3E.0	fCNS11643	1-5226
U+6D3E.0	fBig5	ACA3
U+6D3E.0	fGB	0-4549
U+6D3E.0	fR/S	 85.6
U+6D3E.0	fTotalStrokes	 9
U+6D3E.0	fCangjie	EHHV
U+6D3E.0	fFourCorner	32132
U+6D3E.0	fHakka	  pai5
U+6D3E.0	fCantonese	paai3; paai1 (also); paai2 (also)
U+6D3E.0	fMandarin	pai4; mai4; pa4; bai4; mo4; pa1
U+6D3E.0	fEnglish	[1] tributary; branch [2] division; school (of thought, philosophy, art, etc.); party [3] faction [4] assign; dispatch; send
U+6D3F.0	fUTF8	洿
U+6D3F.0	fCNS11643	2-2B54
U+6D3F.0	fBig5	CF72
U+6D3F.0	fGB	3-5379
U+6D3F.0	fR/S	 85.6
U+6D3F.0	fTotalStrokes	 9
U+6D3F.0	fCangjie	EKMS
U+6D3F.0	fFourCorner	34127
U+6D3F.0	fHakka	 wu1; vu1
U+6D3F.0	fCantonese	wu1
U+6D3F.0	fMandarin	wu1; hu4; wu4
U+6D3F.0	fEnglish	 stagnant water impure; filthy
U+6D40.0	fUTF8	浀
U+6D40.0	fCNS11643	2-2B66
U+6D40.0	fBig5	CFA6
U+6D40.0	fGB	5-553B
U+6D40.0	fR/S	 85.6
U+6D40.0	fTotalStrokes	 9
U+6D40.0	fCangjie	ETW
U+6D40.0	fFourCorner	35160
U+6D40.0	fCantonese	kuk1
U+6D40.0	fMandarin	qu3
U+6D41.0	fUTF8	流
U+6D41.0	fCNS11643	1-517C
U+6D41.0	fBig5	AC79
U+6D41.0	fGB	0-4177
U+6D41.0	fR/S	 85.6
U+6D41.0	fTotalStrokes	 9
U+6D41.0	fCangjie	EYIU
U+6D41.0	fFourCorner	34113
U+6D41.0	fHakka	  liu2; lau2; liu3; lieu2
U+6D41.0	fCantonese	lau4; lau2 (also)
U+6D41.0	fMandarin	liu2
U+6D41.0	fEnglish	[1] flow; discharge [2] wander; stray [3] branch; division [4] class; rank [5] unsettled; unfixed; mobile
U+6D42.0	fUTF8	浂
U+6D42.0	fCNS11643	2-2B60
U+6D42.0	fBig5	CF7E
U+6D42.0	fGB	5-5548
U+6D42.0	fR/S	 85.6
U+6D42.0	fTotalStrokes	 9
U+6D42.0	fCangjie	ETK
U+6D42.0	fFourCorner	38134
U+6D42.0	fCantonese	gun3; gwun3 (also); jat6
U+6D42.0	fMandarin	yi4
U+6D43.0	fUTF8	浃
U+6D43.0	fCNS11643	4-2A21
U+6D43.0	fGB	0-6424
U+6D43.0	fR/S	 85.6
U+6D43.0	fTotalStrokes	 9
U+6D43.0	fCangjie	EKT
U+6D43.0	fFourCorner	35139
U+6D43.0	fHakka	 hiap8; giap7
U+6D43.0	fCantonese	zip3; gaap3
U+6D43.0	fMandarin	jia2; za1; jia1; xia2
U+6D43.0	fEnglish	simplified form of &#x6D79;
U+6D44.0	fUTF8	浄
U+6D44.0	fCNS11643	3-2D55
U+6D44.0	fGB	8-2D55
U+6D44.0	fR/S	 85.6
U+6D44.0	fTotalStrokes	 9
U+6D44.0	fCangjie	ENSD
U+6D44.0	fFourCorner	37150
U+6D44.0	fCantonese	zing6; zeng6 (also)
U+6D44.0	fMandarin	jing4
U+6D44.0	fEnglish	pure; clean; unsoiled
U+6D45.0	fUTF8	浅
U+6D45.0	fCNS11643	3-2D51
U+6D45.0	fGB	0-4733
U+6D45.0	fR/S	 85.5; 85.6
U+6D45.0	fTotalStrokes	 9
U+6D45.0	fCangjie	EIJ
U+6D45.0	fFourCorner	33150
U+6D45.0	fHakka	 cien3
U+6D45.0	fCantonese	cin2; zin1
U+6D45.0	fMandarin	qian3; jian1; can2; jian4; zan4
U+6D45.0	fEnglish	simplified form of &#x6DFA;
U+6D46.0	fUTF8	浆
U+6D46.0	fGB	0-3D2C
U+6D46.0	fR/S	 85.6
U+6D46.0	fTotalStrokes	 10
U+6D46.0	fCangjie	LNE
U+6D46.0	fHakka	 ziong1
U+6D46.0	fCantonese	zoeng1
U+6D46.0	fMandarin	jiang1; jiang4
U+6D46.0	fEnglish	simplified form of &#x6F3F;
U+6D47.0	fUTF8	浇
U+6D47.0	fGB	0-3D3D
U+6D47.0	fR/S	 85.6
U+6D47.0	fTotalStrokes	 9
U+6D47.0	fCangjie	EJPU
U+6D47.0	fHakka	 hiau1
U+6D47.0	fCantonese	hiu1; giu1 (also); ngou6; ou6 (also)
U+6D47.0	fMandarin	jiao1; ao4; ao2; liao2; nao4; yao4
U+6D47.0	fEnglish	simplified form of &#x6F86;
U+6D48.0	fUTF8	浈
U+6D48.0	fGB	0-6425
U+6D48.0	fR/S	 85.6
U+6D48.0	fTotalStrokes	 9
U+6D48.0	fCangjie	EYBO
U+6D48.0	fHakka	 zhin1
U+6D48.0	fCantonese	zing1
U+6D48.0	fMandarin	zhen1; cheng2; cheng1
U+6D48.0	fEnglish	simplified form of &#x6E5E;
U+6D49.0	fUTF8	浉
U+6D49.0	fGB	8-7C73
U+6D49.0	fR/S	 85.10; 85.6
U+6D49.0	fTotalStrokes	 9
U+6D49.0	fCangjie	ELLB
U+6D49.0	fCantonese	si1
U+6D49.0	fMandarin	shi1
U+6D4A.0	fUTF8	浊
U+6D4A.0	fGB	0-5747
U+6D4A.0	fR/S	 85.6
U+6D4A.0	fTotalStrokes	 9
U+6D4A.0	fCangjie	ELMI
U+6D4A.0	fFourCorner	3513
U+6D4A.0	fHakka	 cuk8
U+6D4A.0	fCantonese	zuk6
U+6D4A.0	fMandarin	zhuo2
U+6D4A.0	fEnglish	simplified form of &#x6FC1;
U+6D4B.0	fUTF8	测
U+6D4B.0	fGB	0-3262
U+6D4B.0	fR/S	 85.6
U+6D4B.0	fTotalStrokes	 9
U+6D4B.0	fCangjie	EBON
U+6D4B.0	fHakka	 cet7
U+6D4B.0	fCantonese	cak1; caak1 (also)
U+6D4B.0	fMandarin	ce4
U+6D4B.0	fEnglish	simplified form of &#x6E2C;
U+6D4C.0	fUTF8	浌
U+6D4C.0	fGB	1-7D41
U+6D4C.0	fR/S	 85.6
U+6D4C.0	fTotalStrokes	 9
U+6D4C.0	fCangjie	EOI
U+6D4C.0	fCantonese	fat6
U+6D4D.0	fUTF8	浍
U+6D4D.0	fGB	0-642B
U+6D4D.0	fR/S	 85.6
U+6D4D.0	fTotalStrokes	 9
U+6D4D.0	fCangjie	EOMI
U+6D4D.0	fHakka	 kwai5
U+6D4D.0	fCantonese	kui2; kwui2 (also)
U+6D4D.0	fMandarin	kuai4; hui4; guai4; hua2
U+6D4D.0	fEnglish	simplified form of &#x6FAE;
U+6D4E.0	fUTF8	济
U+6D4E.0	fGB	0-3C43
U+6D4E.0	fR/S	 85.6
U+6D4E.0	fTotalStrokes	 9
U+6D4E.0	fCangjie	EYKL
U+6D4E.0	fHakka	 zi5
U+6D4E.0	fCantonese	zai3; zai2
U+6D4E.0	fMandarin	ji4; qi2; ji3
U+6D4E.0	fEnglish	simplified form of &#x6FDF;
U+6D4F.0	fUTF8	浏
U+6D4F.0	fGB	0-642F
U+6D4F.0	fR/S	 85.6
U+6D4F.0	fTotalStrokes	 9
U+6D4F.0	fCangjie	EYKN
U+6D4F.0	fHakka	 liu2
U+6D4F.0	fCantonese	lau4
U+6D4F.0	fMandarin	liu2; liu3
U+6D4F.0	fEnglish	simplified form of &#x700F;
U+6D50.0	fUTF8	浐
U+6D50.0	fGB	8-7C78
U+6D50.0	fR/S	 85.6; 85.11
U+6D50.0	fTotalStrokes	 9
U+6D50.0	fCangjie	EYTH
U+6D50.0	fCantonese	caan2
U+6D50.0	fMandarin	chan3
U+6D51.0	fUTF8	浑
U+6D51.0	fGB	0-3B6B
U+6D51.0	fR/S	 85.6
U+6D51.0	fTotalStrokes	 9
U+6D51.0	fCangjie	EBKQ
U+6D51.0	fHakka	 fun2
U+6D51.0	fCantonese	wan4; wan6; gwan2
U+6D51.0	fMandarin	hun2; hun4; gun3
U+6D51.0	fEnglish	simplified form of &#x6E3E;
U+6D52.0	fUTF8	浒
U+6D52.0	fGB	0-6430
U+6D52.0	fR/S	 85.6
U+6D52.0	fTotalStrokes	 9
U+6D52.0	fCangjie	EIVJ
U+6D52.0	fHakka	 fu3
U+6D52.0	fCantonese	wu2; heoi2
U+6D52.0	fMandarin	hu3; xu3
U+6D52.0	fEnglish	simplified form of &#x6EF8;
U+6D53.0	fUTF8	浓
U+6D53.0	fGB	0-4528
U+6D53.0	fR/S	 85.6
U+6D53.0	fTotalStrokes	 9
U+6D53.0	fCangjie	EHBV
U+6D53.0	fHakka	 ngiung2
U+6D53.0	fCantonese	nung4; lung4 (also)
U+6D53.0	fMandarin	nong2
U+6D53.0	fEnglish	simplified form of &#x6FC3;
U+6D54.0	fUTF8	浔
U+6D54.0	fGB	0-6431
U+6D54.0	fR/S	 85.6
U+6D54.0	fTotalStrokes	 9
U+6D54.0	fCangjie	XESMI
U+6D54.0	fHakka	 cim2
U+6D54.0	fCantonese	cam4
U+6D54.0	fMandarin	xun2; tan2; yin2
U+6D54.0	fEnglish	simplified form of &#x6F6F;
U+6D55.0	fUTF8	浕
U+6D55.0	fGB	8-7C7A
U+6D55.0	fR/S	 85.6; 85.14
U+6D55.0	fTotalStrokes	 9
U+6D55.0	fCangjie	ESOY
U+6D55.0	fCantonese	zeon6
U+6D55.0	fMandarin	jin4
U+6D56.0	fUTF8	浖
U+6D56.0	fCNS11643	3-3177
U+6D56.0	fGB	5-555C
U+6D56.0	fR/S	  85.7
U+6D56.0	fTotalStrokes	 10
U+6D56.0	fCangjie	EBDI
U+6D56.0	fFourCorner	32140
U+6D56.0	fHakka	 lot7
U+6D56.0	fCantonese	lyut6; lyut1
U+6D56.0	fMandarin	lie4
U+6D56.0	fEnglish	intensifier
U+6D57.0	fUTF8	浗
U+6D57.0	fCNS11643	3-3173
U+6D57.0	fGB	E-3032
U+6D57.0	fR/S	 85.7
U+6D57.0	fTotalStrokes	 10
U+6D57.0	fCangjie	EIJE
U+6D57.0	fFourCorner	33130
U+6D57.0	fCantonese	kau4
U+6D57.0	fMandarin	qiu2
U+6D58.0	fUTF8	浘
U+6D58.0	fCNS11643	2-302F
U+6D58.0	fBig5	D24C
U+6D58.0	fGB	3-544A
U+6D58.0	fR/S	 85.7
U+6D58.0	fTotalStrokes	 10
U+6D58.0	fCangjie	ESHU
U+6D58.0	fFourCorner	37114
U+6D58.0	fCantonese	mei5
U+6D58.0	fMandarin	wei2; wei3
U+6D59.0	fUTF8	浙
U+6D59.0	fCNS11643	1-5641
U+6D59.0	fBig5	AEFD
U+6D59.0	fGB	0-5563
U+6D59.0	fR/S	 85.7
U+6D59.0	fTotalStrokes	 10
U+6D59.0	fCangjie	EQHL
U+6D59.0	fFourCorner	32121
U+6D59.0	fHakka	 zet7; zat7; ziet7; zhat7; zhap8; zap8; zhet7
U+6D59.0	fCantonese	zit3
U+6D59.0	fMandarin	zhe4; zhi4
U+6D59.0	fEnglish	[1] Chekiang province [2] the name of a river
U+6D5A.0	fUTF8	浚
U+6D5A.0	fCNS11643	1-5646
U+6D5A.0	fBig5	AF43
U+6D5A.0	fGB	0-3F23
U+6D5A.0	fR/S	 85.7
U+6D5A.0	fTotalStrokes	 10
U+6D5A.0	fCangjie	EICE
U+6D5A.0	fFourCorner	33147
U+6D5A.0	fHakka	 zun5
U+6D5A.0	fCantonese	zeon3; seon3
U+6D5A.0	fMandarin	jun4; qun1; xun4; cun2
U+6D5A.0	fEnglish	[1] dredge [2] deep
U+6D5B.0	fUTF8	浛
U+6D5B.0	fCNS11643	3-3179
U+6D5B.0	fGB	3-5442
U+6D5B.0	fR/S	 85.7
U+6D5B.0	fTotalStrokes	 10
U+6D5B.0	fCangjie	EOIR
U+6D5B.0	fFourCorner	38160
U+6D5B.0	fCantonese	ham4
U+6D5B.0	fMandarin	han4; han2
U+6D5C.0	fUTF8	浜
U+6D5C.0	fCNS11643	3-3226
U+6D5C.0	fGB	0-643A
U+6D5C.0	fR/S	  85.7
U+6D5C.0	fTotalStrokes	 10
U+6D5C.0	fCangjie	EOMC
U+6D5C.0	fFourCorner	32180
U+6D5C.0	fHakka	  bin1; bang1
U+6D5C.0	fCantonese	bong1; ban1
U+6D5C.0	fMandarin	bang1; bin1; beng3
U+6D5C.0	fEnglish	[1] a small stream [2] a small stream
U+6D5D.0	fUTF8	浝
U+6D5D.0	fCNS11643	3-3176
U+6D5D.0	fGB	5-5555
U+6D5D.0	fR/S	  85.7
U+6D5D.0	fTotalStrokes	 10
U+6D5D.0	fCangjie	EIUH
U+6D5D.0	fFourCorner	33110
U+6D5D.0	fHakka	 mang2
U+6D5D.0	fCantonese	mong4
U+6D5D.0	fMandarin	mang2
U+6D5D.0	fEnglish	water flowing
U+6D5E.0	fUTF8	浞
U+6D5E.0	fCNS11643	2-3038
U+6D5E.0	fBig5	D255
U+6D5E.0	fGB	0-6437
U+6D5E.0	fR/S	 85.7
U+6D5E.0	fTotalStrokes	 10
U+6D5E.0	fCangjie	ERYO
U+6D5E.0	fFourCorner	36181
U+6D5E.0	fHakka	  zok7; zuk7
U+6D5E.0	fCantonese	zuk1; zok6 (also)
U+6D5E.0	fMandarin	zhuo2
U+6D5E.0	fEnglish	 soak
U+6D5F.0	fUTF8	浟
U+6D5F.0	fCNS11643	2-303E
U+6D5F.0	fBig5	D25B
U+6D5F.0	fGB	3-543F
U+6D5F.0	fR/S	 85.7
U+6D5F.0	fTotalStrokes	 10
U+6D5F.0	fCangjie	EOLK
U+6D5F.0	fFourCorner	38140
U+6D5F.0	fCantonese	jau4; dik6
U+6D5F.0	fMandarin	you2; di2
U+6D60.0	fUTF8	浠
U+6D60.0	fCNS11643	2-303A
U+6D60.0	fBig5	D257
U+6D60.0	fGB	0-643B
U+6D60.0	fR/S	 85.7
U+6D60.0	fTotalStrokes	 10
U+6D60.0	fCangjie	EKKB
U+6D60.0	fFourCorner	34127
U+6D60.0	fHakka	 hi1
U+6D60.0	fCantonese	hei1
U+6D60.0	fMandarin	xi1
U+6D60.0	fEnglish	name of a small river in Hupeh
U+6D61.0	fUTF8	浡
U+6D61.0	fCNS11643	2-302D
U+6D61.0	fBig5	D24A
U+6D61.0	fGB	3-5435
U+6D61.0	fR/S	 85.7
U+6D61.0	fTotalStrokes	 10
U+6D61.0	fCangjie	EJBD
U+6D61.0	fFourCorner	34147
U+6D61.0	fHakka	 put8;  but8; but7
U+6D61.0	fCantonese	but6
U+6D61.0	fMandarin	bo2
U+6D61.0	fEnglish	 rise; excited
U+6D62.0	fUTF8	浢
U+6D62.0	fCNS11643	2-3030
U+6D62.0	fBig5	D24D
U+6D62.0	fGB	3-5437
U+6D62.0	fR/S	 85.7
U+6D62.0	fTotalStrokes	 10
U+6D62.0	fCangjie	EMRT
U+6D62.0	fFourCorner	31118
U+6D62.0	fHakka	 teu5; teu6
U+6D62.0	fCantonese	dau6
U+6D62.0	fMandarin	dou4
U+6D63.0	fUTF8	浣
U+6D63.0	fCNS11643	2-3029
U+6D63.0	fBig5	D246
U+6D63.0	fGB	0-643D
U+6D63.0	fR/S	 85.7
U+6D63.0	fTotalStrokes	 10
U+6D63.0	fCangjie	EJMU
U+6D63.0	fFourCorner	33111
U+6D63.0	fHakka	  fon5; jen5; ren5; fon3; won3; fan5; wan2
U+6D63.0	fCantonese	wun5; wun2 (also)
U+6D63.0	fMandarin	huan4; wan3; huan3; guan3
U+6D63.0	fEnglish	[1] wash; rinse [2] ten days; any of the three ten-day divisions of a month
U+6D64.0	fUTF8	浤
U+6D64.0	fCNS11643	2-302A
U+6D64.0	fBig5	D247
U+6D64.0	fGB	3-5446
U+6D64.0	fR/S	 85.7
U+6D64.0	fTotalStrokes	 10
U+6D64.0	fCangjie	EJKI
U+6D64.0	fFourCorner	33132
U+6D64.0	fCantonese	wang4
U+6D64.0	fMandarin	hong2
U+6D64.0	fEnglish	 beating of ocean surging of wate
U+6D65.0	fUTF8	浥
U+6D65.0	fCNS11643	1-564D
U+6D65.0	fBig5	AF4A
U+6D65.0	fGB	3-543C
U+6D65.0	fR/S	 85.7
U+6D65.0	fTotalStrokes	 10
U+6D65.0	fCangjie	ERAU
U+6D65.0	fFourCorner	36117
U+6D65.0	fHakka	  jip7; rip7
U+6D65.0	fCantonese	jap1; aap3; ngaap3 (also)
U+6D65.0	fMandarin	yi4; yan1; ya4; ya1
U+6D65.0	fEnglish	moist; wet; dampen; moisten
U+6D66.0	fUTF8	浦
U+6D66.0	fCNS11643	1-563E
U+6D66.0	fBig5	AEFA
U+6D66.0	fGB	0-4656
U+6D66.0	fR/S	 85.7
U+6D66.0	fTotalStrokes	 10
U+6D66.0	fCangjie	EIJB
U+6D66.0	fFourCorner	33127
U+6D66.0	fHakka	 pu3; pu1
U+6D66.0	fCantonese	pou2; pou4 (also)
U+6D66.0	fMandarin	pu3
U+6D66.0	fEnglish	[1] riverside; bank of river; shore; beach [2] a Chinese family name
U+6D67.0	fUTF8	浧
U+6D67.0	fCNS11643	2-3039
U+6D67.0	fBig5	D256
U+6D67.0	fGB	3-543A
U+6D67.0	fR/S	 85.7
U+6D67.0	fTotalStrokes	 10
U+6D67.0	fCangjie	ERHG
U+6D67.0	fFourCorner	36114
U+6D67.0	fCantonese	jing5; cing2
U+6D67.0	fMandarin	ying3; cheng3; ying4; cheng2
U+6D68.0	fUTF8	浨
U+6D68.0	fCNS11643	2-3042
U+6D68.0	fBig5	D25F
U+6D68.0	fGB	E-3033
U+6D68.0	fR/S	 85.7
U+6D68.0	fTotalStrokes	 10
U+6D68.0	fCangjie	EJD
U+6D68.0	fFourCorner	33194
U+6D68.0	fCantonese	lam5
U+6D68.0	fMandarin	lan3
U+6D69.0	fUTF8	浩
U+6D69.0	fCNS11643	1-5648
U+6D69.0	fBig5	AF45
U+6D69.0	fGB	0-3A46
U+6D69.0	fR/S	 85.7
U+6D69.0	fTotalStrokes	 10
U+6D69.0	fCangjie	EHGR
U+6D69.0	fFourCorner	34161
U+6D69.0	fHakka	  hau5;  hau1;  hau3
U+6D69.0	fCantonese	hou6; hou5
U+6D69.0	fMandarin	hao4; gao3; ge2; gao4
U+6D69.0	fEnglish	[1] massive; great; vast [2] numerous; many; much [3] a Chinese family name
U+6D6A.0	fUTF8	浪
U+6D6A.0	fCNS11643	1-563A
U+6D6A.0	fBig5	AEF6
U+6D6A.0	fGB	0-404B
U+6D6A.0	fR/S	 85.7
U+6D6A.0	fTotalStrokes	 10
U+6D6A.0	fCangjie	EIAV
U+6D6A.0	fFourCorner	37132
U+6D6A.0	fHakka	  long5; long6; long3; long2; long1
U+6D6A.0	fCantonese	long6; long4
U+6D6A.0	fMandarin	lang4; lang3; lan4
U+6D6A.0	fEnglish	[1] wave; billows; breakers [2] dissolute; debauched; unrestrained; rash
U+6D6A.1	fUTF8	浪
U+6D6A.1	fCNS11643	1-563A
U+6D6A.1	fBig5	AEF6
U+6D6A.1	fGB	0-404B
U+6D6A.1	fR/S	 85.7
U+6D6A.1	fTotalStrokes	 10
U+6D6A.1	fCangjie	EIAV
U+6D6A.1	fFourCorner	37132
U+6D6A.1	fMandarin	lang2
U+6D6A.1	fEnglish	flowing; fluent
U+6D6B.0	fUTF8	浫
U+6D6B.0	fCNS11643	3-316F
U+6D6B.0	fGB	3-5448
U+6D6B.0	fR/S	  85.7
U+6D6B.0	fTotalStrokes	 10
U+6D6B.0	fCangjie	EBCJ
U+6D6B.0	fFourCorner	37140
U+6D6B.0	fCantonese	hon2
U+6D6B.0	fMandarin	han3
U+6D6C.0	fUTF8	浬
U+6D6C.0	fCNS11643	1-5643
U+6D6C.0	fBig5	AF40
U+6D6C.0	fGB	8-2876
U+6D6C.0	fR/S	 85.7
U+6D6C.0	fTotalStrokes	 10
U+6D6C.0	fCangjie	EWG
U+6D6C.0	fFourCorner	36114
U+6D6C.0	fHakka	 li1
U+6D6C.0	fCantonese	lei5; hoi2 (also)
U+6D6C.0	fMandarin	li3
U+6D6C.0	fEnglish	nautical mile; sea mile; geographic mile (unit of distance)
U+6D6D.0	fUTF8	浭
U+6D6D.0	fCNS11643	2-3031
U+6D6D.0	fBig5	D24E
U+6D6D.0	fGB	3-5436
U+6D6D.0	fR/S	 85.7
U+6D6D.0	fTotalStrokes	 10
U+6D6D.0	fCangjie	EMLK
U+6D6D.0	fFourCorner	31146
U+6D6D.0	fCantonese	gang1; gaang3
U+6D6D.0	fMandarin	geng1
U+6D6D.0	fEnglish	the name of a river in Hopeh province
U+6D6E.0	fUTF8	浮
U+6D6E.0	fCNS11643	1-5645
U+6D6E.0	fBig5	AF42
U+6D6E.0	fGB	0-3821
U+6D6E.0	fR/S	 85.7
U+6D6E.0	fTotalStrokes	 10
U+6D6E.0	fCangjie	EBND
U+6D6E.0	fFourCorner	32147
U+6D6E.0	fHakka	  feu2; peu2; po2; feu3; fiu2
U+6D6E.0	fCantonese	fau4
U+6D6E.0	fMandarin	fu2; fou2; pou1
U+6D6E.0	fEnglish	[1] float; drift; waft [2] exceed; overflow [3] empty; superficial; unsubstantial; unfounded; groundless
U+6D6F.0	fUTF8	浯
U+6D6F.0	fCNS11643	2-3032
U+6D6F.0	fBig5	D24F
U+6D6F.0	fGB	0-6434
U+6D6F.0	fR/S	 85.7
U+6D6F.0	fTotalStrokes	 10
U+6D6F.0	fCangjie	EMMR
U+6D6F.0	fFourCorner	31161
U+6D6F.0	fHakka	 ng2; ngu2
U+6D6F.0	fCantonese	ng4
U+6D6F.0	fMandarin	wu2
U+6D6F.0	fEnglish	[1] the name of a river in Shantung [2] a river in Hunan
U+6D70.0	fUTF8	浰
U+6D70.0	fCNS11643	2-303C
U+6D70.0	fBig5	D259
U+6D70.0	fGB	3-543E
U+6D70.0	fR/S	 85.7
U+6D70.0	fTotalStrokes	 10
U+6D70.0	fCangjie	EHDN
U+6D70.0	fFourCorner	32100
U+6D70.0	fCantonese	lei6 (also); lin6
U+6D70.0	fMandarin	lian4; li4
U+6D71.0	fUTF8	浱
U+6D71.0	fCNS11643	3-3222
U+6D71.0	fGB	5-5553
U+6D71.0	fR/S	  85.7
U+6D71.0	fTotalStrokes	 10
U+6D71.0	fCangjie	EMMV
U+6D71.0	fFourCorner	31130
U+6D71.0	fCantonese	seon4
U+6D71.0	fMandarin	chun2
U+6D72.0	fUTF8	浲
U+6D72.0	fCNS11643	3-3225
U+6D72.0	fGB	5-555F
U+6D72.0	fR/S	 85.7
U+6D72.0	fTotalStrokes	 10
U+6D72.0	fCangjie	EHEJ
U+6D72.0	fFourCorner	37150
U+6D72.0	fCantonese	fung4
U+6D72.0	fMandarin	feng2
U+6D73.0	fUTF8	浳
U+6D73.0	fCNS11643	3-3172
U+6D73.0	fGB	E-3034
U+6D73.0	fR/S	  85.7
U+6D73.0	fTotalStrokes	 10
U+6D73.0	fCangjie	EIPB
U+6D73.0	fFourCorner	33120
U+6D73.0	fCantonese	jik6
U+6D73.0	fMandarin	yi4
U+6D74.0	fUTF8	浴
U+6D74.0	fCNS11643	1-5647
U+6D74.0	fBig5	AF44
U+6D74.0	fGB	0-5421
U+6D74.0	fR/S	 85.7
U+6D74.0	fTotalStrokes	 10
U+6D74.0	fCangjie	ECOR
U+6D74.0	fFourCorner	38168
U+6D74.0	fHakka	  juk8; ruk8; rok7; jok7; niok7; jok8; liuk7
U+6D74.0	fCantonese	juk6
U+6D74.0	fMandarin	yu4
U+6D74.0	fEnglish	[1] bathe; bath [2] wash
U+6D75.0	fUTF8	浵
U+6D75.0	fCNS11643	2-304B
U+6D75.0	fBig5	D268
U+6D75.0	fGB	3-5443
U+6D75.0	fR/S	 85.7
U+6D75.0	fTotalStrokes	 10
U+6D75.0	fCangjie	EBYH
U+6D75.0	fFourCorner	32022
U+6D75.0	fCantonese	tung4
U+6D75.0	fMandarin	tong2
U+6D76.0	fUTF8	浶
U+6D76.0	fCNS11643	2-302B
U+6D76.0	fBig5	D248
U+6D76.0	fGB	3-5447
U+6D76.0	fR/S	 85.7
U+6D76.0	fTotalStrokes	 10
U+6D76.0	fCangjie	EJHQ
U+6D76.0	fFourCorner	33152
U+6D76.0	fHakka	 lau2
U+6D76.0	fCantonese	lou4
U+6D76.0	fMandarin	lao2
U+6D77.0	fUTF8	海
U+6D77.0	fCNS11643	1-5640
U+6D77.0	fBig5	AEFC
U+6D77.0	fGB	0-3A23
U+6D77.0	fR/S	 85.7
U+6D77.0	fTotalStrokes	 10
U+6D77.0	fCangjie	EOWY
U+6D77.0	fFourCorner	38157
U+6D77.0	fHakka	  hoi3
U+6D77.0	fCantonese	hoi2
U+6D77.0	fMandarin	hai3
U+6D77.0	fEnglish	[1] sea; ocean; maritime [2] a great number of people or things coming together; huge gathering [3] area; field [4] great; unlimited [5] a Chinese family name [6] direction; side (strictly used after North, South, etc.)
U+6D78.0	fUTF8	浸
U+6D78.0	fCNS11643	1-563F
U+6D78.0	fBig5	AEFB
U+6D78.0	fGB	0-3D7E
U+6D78.0	fR/S	 85.7
U+6D78.0	fTotalStrokes	 10
U+6D78.0	fCangjie	ESME
U+6D78.0	fFourCorner	37147
U+6D78.0	fHakka	  zim5
U+6D78.0	fCantonese	zam3; cam1
U+6D78.0	fMandarin	jin4; jin1; qin1
U+6D78.0	fEnglish	[1] soak; immerse; dip; percolate; permeate [2] gradual; gradually
U+6D79.0	fUTF8	浹
U+6D79.0	fCNS11643	1-564B
U+6D79.0	fBig5	AF48
U+6D79.0	fGB	1-6424
U+6D79.0	fR/S	 85.7
U+6D79.0	fTotalStrokes	 10
U+6D79.0	fCangjie	EKOO
U+6D79.0	fFourCorner	34138
U+6D79.0	fHakka	 giap7; hiap8
U+6D79.0	fCantonese	zip3; gaap3
U+6D79.0	fMandarin	jia2; za1; jia1; xia2
U+6D79.0	fEnglish	 saturate; drench damp; wet
U+6D7A.0	fUTF8	浺
U+6D7A.0	fCNS11643	2-3028
U+6D7A.0	fBig5	D245
U+6D7A.0	fGB	3-5445
U+6D7A.0	fR/S	 85.7
U+6D7A.0	fTotalStrokes	 10
U+6D7A.0	fCangjie	EPL
U+6D7A.0	fFourCorner	35106
U+6D7A.0	fHakka	 chung1; cung1
U+6D7A.0	fCantonese	cung1
U+6D7A.0	fMandarin	chong1; chong2
U+6D7A.0	fEnglish	wide expanse of water
U+6D7B.0	fUTF8	浻
U+6D7B.0	fCNS11643	2-3049
U+6D7B.0	fBig5	D266
U+6D7B.0	fGB	5-555A
U+6D7B.0	fR/S	 85.7
U+6D7B.0	fTotalStrokes	 10
U+6D7B.0	fCangjie	EBCR
U+6D7B.0	fFourCorner	37120
U+6D7B.0	fCantonese	gwing2
U+6D7B.0	fMandarin	weng3; jiong3
U+6D7C.0	fUTF8	浼
U+6D7C.0	fCNS11643	2-303D
U+6D7C.0	fBig5	D25A
U+6D7C.0	fGB	0-643C
U+6D7C.0	fR/S	 85.7
U+6D7C.0	fTotalStrokes	 10
U+6D7C.0	fCangjie	ENAU
U+6D7C.0	fFourCorner	37116
U+6D7C.0	fHakka	  mui3; mi3; mien3
U+6D7C.0	fCantonese	mui5
U+6D7C.0	fMandarin	mei3; man2; man3; man4; mian3
U+6D7C.0	fEnglish	[1] stain; soil; contaminate; defile [2] full of water [3] entrust
U+6D7D.0	fUTF8	浽
U+6D7D.0	fCNS11643	2-304A
U+6D7D.0	fBig5	D267
U+6D7D.0	fGB	3-5441
U+6D7D.0	fR/S	 85.7
U+6D7D.0	fTotalStrokes	 10
U+6D7D.0	fCangjie	EBV
U+6D7D.0	fFourCorner	32144
U+6D7D.0	fHakka	 sui1;  nui1
U+6D7D.0	fCantonese	seoi1; neoi5
U+6D7D.0	fMandarin	sui1; nei3
U+6D7D.0	fEnglish	drizzling; misty
U+6D7E.0	fUTF8	浾
U+6D7E.0	fCNS11643	2-3044
U+6D7E.0	fBig5	D261
U+6D7E.0	fGB	5-554D
U+6D7E.0	fR/S	 85.7
U+6D7E.0	fTotalStrokes	 10
U+6D7E.0	fCangjie	EGLC
U+6D7E.0	fFourCorner	34131
U+6D7E.0	fCantonese	zaa1
U+6D7E.0	fMandarin	cheng1; jin1; zha1
U+6D7F.0	fUTF8	浿
U+6D7F.0	fCNS11643	2-3036
U+6D7F.0	fBig5	D253
U+6D7F.0	fGB	3-5348
U+6D7F.0	fR/S	 85.7
U+6D7F.0	fTotalStrokes	 10
U+6D7F.0	fCangjie	EBUC
U+6D7F.0	fFourCorner	36180
U+6D7F.0	fCantonese	bui3
U+6D7F.0	fMandarin	pei4; pai4; bei4
U+6D80.0	fUTF8	涀
U+6D80.0	fCNS11643	2-3045
U+6D80.0	fBig5	D262
U+6D80.0	fGB	3-534A
U+6D80.0	fR/S	 85.7
U+6D80.0	fTotalStrokes	 10
U+6D80.0	fCangjie	EBUU
U+6D80.0	fFourCorner	36110
U+6D80.0	fCantonese	jin6; gaan2; gin2
U+6D80.0	fMandarin	xian4; xi4; jian3
U+6D81.0	fUTF8	涁
U+6D81.0	fCNS11643	3-3221
U+6D81.0	fGB	5-5552
U+6D81.0	fR/S	 85.7
U+6D81.0	fTotalStrokes	 10
U+6D81.0	fCangjie	EDHH
U+6D81.0	fFourCorner	32120
U+6D81.0	fCantonese	sam3
U+6D81.0	fMandarin	shen4
U+6D82.0	fUTF8	涂
U+6D82.0	fCNS11643	2-303F
U+6D82.0	fBig5	D25C
U+6D82.0	fGB	0-4D3F
U+6D82.0	fR/S	 85.7
U+6D82.0	fTotalStrokes	 10
U+6D82.0	fCangjie	EOMD
U+6D82.0	fFourCorner	38194
U+6D82.0	fHakka	  tu2; tu3
U+6D82.0	fCantonese	tou4; ceoi4 (also)
U+6D82.0	fMandarin	tu2; cha2; du1; du4
U+6D82.0	fEnglish	simplified form of &#x5857;
U+6D82.1	fUTF8	涂
U+6D82.1	fCNS11643	2-303F
U+6D82.1	fBig5	D25C
U+6D82.1	fGB	0-4D3F
U+6D82.1	fR/S	 85.7
U+6D82.1	fTotalStrokes	 10
U+6D82.1	fCangjie	EOMD
U+6D82.1	fFourCorner	38194
U+6D82.1	fMandarin	tu5
U+6D83.0	fUTF8	涃
U+6D83.0	fCNS11643	2-3048
U+6D83.0	fBig5	D265
U+6D83.0	fGB	5-5559
U+6D83.0	fR/S	 85.7
U+6D83.0	fTotalStrokes	 10
U+6D83.0	fCangjie	EWD
U+6D83.0	fFourCorner	36100
U+6D83.0	fCantonese	kwan3
U+6D83.0	fMandarin	kun4
U+6D84.0	fUTF8	涄
U+6D84.0	fCNS11643	2-3046
U+6D84.0	fBig5	D263
U+6D84.0	fGB	5-5558
U+6D84.0	fR/S	 85.7
U+6D84.0	fTotalStrokes	 10
U+6D84.0	fCangjie	ELWS
U+6D84.0	fFourCorner	35127
U+6D84.0	fCantonese	ping1
U+6D84.0	fMandarin	pin1
U+6D85.0	fUTF8	涅
U+6D85.0	fCNS11643	1-564C
U+6D85.0	fBig5	AF49
U+6D85.0	fGB	0-4479
U+6D85.0	fR/S	 85.7
U+6D85.0	fTotalStrokes	 10
U+6D85.0	fCangjie	EAG
U+6D85.0	fFourCorner	36114
U+6D85.0	fHakka	 ngiap7; let7
U+6D85.0	fCantonese	nip6; lip6 (also)
U+6D85.0	fMandarin	nie4; qian2
U+6D85.0	fEnglish	[1] blacken; dye black [2] block up
U+6D86.0	fUTF8	涆
U+6D86.0	fCNS11643	2-3037
U+6D86.0	fBig5	D254
U+6D86.0	fGB	5-5556
U+6D86.0	fR/S	 85.7
U+6D86.0	fTotalStrokes	 10
U+6D86.0	fCangjie	EAMJ
U+6D86.0	fFourCorner	31140
U+6D86.0	fHakka	 hon1
U+6D86.0	fCantonese	hon6
U+6D86.0	fMandarin	han4; gan4
U+6D87.0	fUTF8	涇
U+6D87.0	fCNS11643	1-563D
U+6D87.0	fBig5	AEF9
U+6D87.0	fGB	1-637E
U+6D87.0	fR/S	 85.7
U+6D87.0	fTotalStrokes	 10
U+6D87.0	fCangjie	EMVM
U+6D87.0	fFourCorner	31111
U+6D87.0	fHakka	 gin1
U+6D87.0	fCantonese	ging1
U+6D87.0	fMandarin	jing1; jing4; qing3
U+6D87.0	fEnglish	the name f a river in Shensi
U+6D88.0	fUTF8	消
U+6D88.0	fCNS11643	1-563C
U+6D88.0	fBig5	AEF8
U+6D88.0	fGB	0-4F7B
U+6D88.0	fR/S	 85.7
U+6D88.0	fTotalStrokes	 10
U+6D88.0	fCangjie	EFB
U+6D88.0	fFourCorner	39127
U+6D88.0	fHakka	  siau1
U+6D88.0	fCantonese	siu1
U+6D88.0	fMandarin	xiao1
U+6D88.0	fEnglish	[1] vanish; die out; disappear [2] disperse; eliminate; remove; alleviate; allay; extinguish; quench [3] need; take
U+6D89.0	fUTF8	涉
U+6D89.0	fCNS11643	1-5644
U+6D89.0	fBig5	AF41
U+6D89.0	fGB	0-4966
U+6D89.0	fR/S	 85.7
U+6D89.0	fTotalStrokes	 10
U+6D89.0	fCangjie	EYLH
U+6D89.0	fFourCorner	31129
U+6D89.0	fHakka	  hiap8; siap8;  shap8; shap7; sap8
U+6D89.0	fCantonese	sip3; dip6
U+6D89.0	fMandarin	she4; die2
U+6D89.0	fEnglish	[1] wade [2] cross [3] experience [4] involve; entangle; implicate [5] a Chinese family name
U+6D8A.0	fUTF8	涊
U+6D8A.0	fCNS11643	1-564A
U+6D8A.0	fBig5	AF47
U+6D8A.0	fGB	3-544B
U+6D8A.0	fR/S	 85.7
U+6D8A.0	fTotalStrokes	 10
U+6D8A.0	fCangjie	ESIP
U+6D8A.0	fFourCorner	37132
U+6D8A.0	fHakka	 ngiun3;  ngien3
U+6D8A.0	fCantonese	nin5; lin5 (also); nan2
U+6D8A.0	fMandarin	nian3; ren3; nian4; lian3
U+6D8A.0	fEnglish	dirt
U+6D8B.0	fUTF8	涋
U+6D8B.0	fCNS11643	2-3043
U+6D8B.0	fBig5	D260
U+6D8B.0	fGB	E-3035
U+6D8B.0	fR/S	 85.7
U+6D8B.0	fTotalStrokes	 10
U+6D8B.0	fCangjie	EJIK
U+6D8B.0	fFourCorner	33134
U+6D8B.0	fCantonese	teot1; tuk1
U+6D8B.0	fMandarin	tu1
U+6D8C.0	fUTF8	涌
U+6D8C.0	fCNS11643	1-5649
U+6D8C.0	fBig5	AF46
U+6D8C.0	fGB	0-533F
U+6D8C.0	fR/S	 85.7
U+6D8C.0	fTotalStrokes	 10
U+6D8C.0	fCangjie	ENIB
U+6D8C.0	fFourCorner	37127
U+6D8C.0	fHakka	  jung3; rung3; jung5
U+6D8C.0	fCantonese	jung2; cung1; jung5
U+6D8C.0	fMandarin	yong3; chong1
U+6D8C.0	fEnglish	[1] surge; gush out; rise; spring; pour out [2] stream; brook; very small water
U+6D8D.0	fUTF8	涍
U+6D8D.0	fCNS11643	2-3034
U+6D8D.0	fBig5	D251
U+6D8D.0	fGB	3-5433
U+6D8D.0	fR/S	 85.7
U+6D8D.0	fTotalStrokes	 10
U+6D8D.0	fCangjie	EJKD
U+6D8D.0	fFourCorner	34147
U+6D8D.0	fCantonese	haau3; haau1
U+6D8D.0	fMandarin	xiao4; xiao1
U+6D8E.0	fUTF8	涎
U+6D8E.0	fCNS11643	1-5B48
U+6D8E.0	fBig5	B243
U+6D8E.0	fGB	0-4F51
U+6D8E.0	fR/S	 85.7
U+6D8E.0	fTotalStrokes	 10
U+6D8E.0	fCangjie	ENKM
U+6D8E.0	fFourCorner	32141
U+6D8E.0	fHakka	  jen2;  lan1; ren2; jan2
U+6D8E.0	fCantonese	jin4
U+6D8E.0	fMandarin	xian2; yan4; dian4
U+6D8E.0	fEnglish	saliva
U+6D8F.0	fUTF8	涏
U+6D8F.0	fCNS11643	3-3178
U+6D8F.0	fGB	3-5424
U+6D8F.0	fR/S	  85.7
U+6D8F.0	fTotalStrokes	 10
U+6D8F.0	fCangjie	ENKG
U+6D8F.0	fFourCorner	32140
U+6D8F.0	fCantonese	ting5
U+6D8F.0	fMandarin	ting3; dian4
U+6D90.0	fUTF8	涐
U+6D90.0	fCNS11643	2-304C
U+6D90.0	fBig5	D269
U+6D90.0	fGB	E-3036
U+6D90.0	fR/S	 85.7
U+6D90.0	fTotalStrokes	 10
U+6D90.0	fCangjie	EHQI
U+6D90.0	fFourCorner	33150
U+6D90.0	fCantonese	ngo4
U+6D90.0	fMandarin	e2; e3
U+6D91.0	fUTF8	涑
U+6D91.0	fCNS11643	2-3033
U+6D91.0	fBig5	D250
U+6D91.0	fGB	0-6433
U+6D91.0	fR/S	 85.7
U+6D91.0	fTotalStrokes	 10
U+6D91.0	fCangjie	EDL
U+6D91.0	fFourCorner	35196
U+6D91.0	fHakka	 suk7; seu1
U+6D91.0	fCantonese	cuk1; suk1
U+6D91.0	fMandarin	sou1; su4; shou4; shu4
U+6D91.0	fEnglish	the name of a tributary of the Yellow river in Shansi province
U+6D92.0	fUTF8	涒
U+6D92.0	fCNS11643	2-302E
U+6D92.0	fBig5	D24B
U+6D92.0	fGB	3-5449
U+6D92.0	fR/S	 85.7
U+6D92.0	fTotalStrokes	 10
U+6D92.0	fCangjie	ESKR
U+6D92.0	fFourCorner	37167
U+6D92.0	fHakka	 dun1; tun1
U+6D92.0	fCantonese	tan1; wan1
U+6D92.0	fMandarin	tun1; yun1; jun1; yun2
U+6D92.0	fEnglish	meander
U+6D93.0	fUTF8	涓
U+6D93.0	fCNS11643	1-5642
U+6D93.0	fBig5	AEFE
U+6D93.0	fGB	0-6438
U+6D93.0	fR/S	 85.7
U+6D93.0	fTotalStrokes	 10
U+6D93.0	fCangjie	ERB
U+6D93.0	fFourCorner	36127
U+6D93.0	fHakka	 gien1; gen1; gian1
U+6D93.0	fCantonese	gyun1
U+6D93.0	fMandarin	juan1; yuan4; xuan4
U+6D93.0	fEnglish	 brook; small stream; rivulet
U+6D94.0	fUTF8	涔
U+6D94.0	fCNS11643	1-564E
U+6D94.0	fBig5	AF4B
U+6D94.0	fGB	0-6439
U+6D94.0	fR/S	 85.7
U+6D94.0	fTotalStrokes	 10
U+6D94.0	fCangjie	EUON
U+6D94.0	fFourCorner	32127
U+6D94.0	fHakka	 cem2;  cim2
U+6D94.0	fCantonese	sam4
U+6D94.0	fMandarin	cen2; qian2; zhan4; zan4
U+6D94.0	fEnglish	[1] puddle [2] tearful
U+6D95.0	fUTF8	涕
U+6D95.0	fCNS11643	1-563B
U+6D95.0	fBig5	AEF7
U+6D95.0	fGB	0-4C69
U+6D95.0	fR/S	 85.7
U+6D95.0	fTotalStrokes	 10
U+6D95.0	fCangjie	ECNH
U+6D95.0	fFourCorner	38127
U+6D95.0	fHakka	  ti5; tai1; ti6; ti3; pi6; pi5
U+6D95.0	fCantonese	tai3
U+6D95.0	fMandarin	ti4; ti3; di4; yi4
U+6D95.0	fEnglish	[1] tears [2] snivel; nasal mucus
U+6D96.0	fUTF8	涖
U+6D96.0	fCNS11643	3-3227
U+6D96.0	fGB	E-3037
U+6D96.0	fR/S	  85.7
U+6D96.0	fTotalStrokes	 10
U+6D96.0	fCangjie	EOYT
U+6D96.0	fFourCorner	30110
U+6D96.0	fHakka	 li5; li6
U+6D96.0	fCantonese	lei6
U+6D96.0	fMandarin	li4
U+6D96.0	fEnglish	[1] arrive [2] the murmur of flowing water
U+6D97.0	fUTF8	涗
U+6D97.0	fCNS11643	2-303B
U+6D97.0	fBig5	D258
U+6D97.0	fGB	E-3038
U+6D97.0	fR/S	 85.7
U+6D97.0	fTotalStrokes	 10
U+6D97.0	fCangjie	ECRU
U+6D97.0	fFourCorner	38116
U+6D97.0	fCantonese	seoi3
U+6D97.0	fMandarin	shui4; sui4; xue3
U+6D98.0	fUTF8	涘
U+6D98.0	fCNS11643	2-3040
U+6D98.0	fBig5	D25D
U+6D98.0	fGB	3-544C
U+6D98.0	fR/S	 85.7
U+6D98.0	fTotalStrokes	 10
U+6D98.0	fCangjie	EIOK
U+6D98.0	fFourCorner	33134
U+6D98.0	fHakka	 sii5; sii6; se5
U+6D98.0	fCantonese	zi6
U+6D98.0	fMandarin	si4
U+6D98.0	fEnglish	 river bank water's edge
U+6D99.0	fUTF8	涙
U+6D99.0	fCNS11643	3-317D
U+6D99.0	fGB	E-3039
U+6D99.0	fR/S	 85.8
U+6D99.0	fTotalStrokes	 10; 11
U+6D99.0	fCangjie	EMSK
U+6D99.0	fFourCorner	33130
U+6D99.0	fCantonese	leoi6
U+6D99.0	fMandarin	lei4; li4; lie4
U+6D99.0	fEnglish	 tears weep
U+6D9A.0	fUTF8	涚
U+6D9A.0	fCNS11643	3-317B
U+6D9A.0	fGB	3-5444
U+6D9A.0	fR/S	 85.7
U+6D9A.0	fTotalStrokes	 10
U+6D9A.0	fCangjie	ECRU
U+6D9A.0	fFourCorner	38110
U+6D9A.0	fCantonese	seoi3
U+6D9A.0	fMandarin	shui4; sui4; xue2
U+6D9B.0	fUTF8	涛
U+6D9B.0	fCNS11643	3-317C
U+6D9B.0	fGB	0-4C4E
U+6D9B.0	fR/S	 85.7
U+6D9B.0	fTotalStrokes	 10
U+6D9B.0	fCangjie	EQKI
U+6D9B.0	fFourCorner	34140
U+6D9B.0	fHakka	 tau2
U+6D9B.0	fCantonese	tou4
U+6D9B.0	fMandarin	tao2; tao1; chuo2; shou4; dao4
U+6D9B.0	fEnglish	simplified form of &#x6FE4;
U+6D9C.0	fUTF8	涜
U+6D9C.0	fGB	E-303A
U+6D9C.0	fR/S	 85.7
U+6D9C.0	fTotalStrokes	 10
U+6D9C.0	fCangjie	EGBU
U+6D9C.0	fCantonese	duk6
U+6D9C.0	fMandarin	du2; dou4
U+6D9C.0	fEnglish	ditch; sluice; gutter; drain
U+6D9D.0	fUTF8	涝
U+6D9D.0	fGB	0-4054
U+6D9D.0	fR/S	 85.7
U+6D9D.0	fTotalStrokes	 10
U+6D9D.0	fCangjie	ETBS
U+6D9D.0	fHakka	 lau5
U+6D9D.0	fCantonese	lou6; lou4
U+6D9D.0	fMandarin	lao4; lao2; lao3
U+6D9D.0	fEnglish	simplified form of &#x6F87;
U+6D9E.0	fUTF8	涞
U+6D9E.0	fGB	0-6435
U+6D9E.0	fR/S	 85.7
U+6D9E.0	fTotalStrokes	 10
U+6D9E.0	fCangjie	EDT
U+6D9E.0	fHakka	 loi2
U+6D9E.0	fCantonese	loi4
U+6D9E.0	fMandarin	lai2
U+6D9E.0	fEnglish	simplified form of &#x6DF6;
U+6D9F.0	fUTF8	涟
U+6D9F.0	fGB	0-4130
U+6D9F.0	fR/S	 85.7
U+6D9F.0	fTotalStrokes	 10
U+6D9F.0	fCangjie	EYKQ
U+6D9F.0	fHakka	 lien2
U+6D9F.0	fCantonese	lin4
U+6D9F.0	fMandarin	lian2; lan2
U+6D9F.0	fEnglish	simplified form of &#x6F23;
U+6DA0.0	fUTF8	涠
U+6DA0.0	fGB	0-6436
U+6DA0.0	fR/S	 85.7
U+6DA0.0	fTotalStrokes	 10
U+6DA0.0	fCangjie	EWQS
U+6DA0.0	fCantonese	wai4
U+6DA0.0	fMandarin	wei2
U+6DA0.0	fEnglish	simplified form of &#x6F7F;
U+6DA1.0	fUTF8	涡
U+6DA1.0	fGB	0-4E50
U+6DA1.0	fR/S	 85.7
U+6DA1.0	fTotalStrokes	 10
U+6DA1.0	fCangjie	EROB
U+6DA1.0	fHakka	 vo1
U+6DA1.0	fCantonese	wo1; gwo1; go1 (also)
U+6DA1.0	fMandarin	wo1; gua1; guo1
U+6DA1.0	fEnglish	simplified form of &#x6E26;
U+6DA2.0	fUTF8	涢
U+6DA2.0	fGB	8-7C7C
U+6DA2.0	fR/S	 85.7; 85.10
U+6DA2.0	fTotalStrokes	 10
U+6DA2.0	fCangjie	ERBO
U+6DA2.0	fCantonese	wan4
U+6DA2.0	fMandarin	yun2
U+6DA3.0	fUTF8	涣
U+6DA3.0	fGB	0-3B41
U+6DA3.0	fR/S	 85.7
U+6DA3.0	fTotalStrokes	 10
U+6DA3.0	fCangjie	XENBK
U+6DA3.0	fHakka	 fon5; fan5
U+6DA3.0	fCantonese	wun6
U+6DA3.0	fMandarin	huan4
U+6DA3.0	fEnglish	simplified form of &#x6E19;
U+6DA4.0	fUTF8	涤
U+6DA4.0	fGB	0-3553
U+6DA4.0	fR/S	 85.7
U+6DA4.0	fTotalStrokes	 10
U+6DA4.0	fCangjie	EHED
U+6DA4.0	fFourCorner	3819
U+6DA4.0	fHakka	 tit7
U+6DA4.0	fCantonese	dik6
U+6DA4.0	fMandarin	di2; diao4
U+6DA4.0	fEnglish	simplified form of &#x6ECC;
U+6DA5.0	fUTF8	涥
U+6DA5.0	fCNS11643	3-653D
U+6DA5.0	fGB	E-303B
U+6DA5.0	fR/S	 85.7
U+6DA5.0	fTotalStrokes	 10
U+6DA5.0	fCangjie	EYRN
U+6DA5.0	fFourCorner	3012
U+6DA5.0	fCantonese	hang1
U+6DA6.0	fUTF8	润
U+6DA6.0	fGB	0-4873
U+6DA6.0	fR/S	 85.7
U+6DA6.0	fTotalStrokes	 10
U+6DA6.0	fCangjie	ELSG
U+6DA6.0	fHakka	 jun5
U+6DA6.0	fCantonese	jeon6; jeon2 (also)
U+6DA6.0	fMandarin	run4
U+6DA6.0	fEnglish	simplified form of &#x6F64;
U+6DA7.0	fUTF8	涧
U+6DA7.0	fGB	0-3D27
U+6DA7.0	fR/S	 85.7
U+6DA7.0	fTotalStrokes	 10
U+6DA7.0	fCangjie	ELSA
U+6DA7.0	fHakka	 gan3
U+6DA7.0	fCantonese	gaan3; gaan2 (also)
U+6DA7.0	fMandarin	jian4
U+6DA7.0	fEnglish	simplified form of &#x6F97;
U+6DA8.0	fUTF8	涨
U+6DA8.0	fGB	0-5547
U+6DA8.0	fR/S	 85.7
U+6DA8.0	fTotalStrokes	 10
U+6DA8.0	fCangjie	ENPO
U+6DA8.0	fHakka	 zhong5
U+6DA8.0	fCantonese	zoeng2; zoeng3
U+6DA8.0	fMandarin	zhang3; zhang1
U+6DA8.0	fEnglish	simplified form of &#x6F32;
U+6DA8.1	fUTF8	涨
U+6DA8.1	fGB	0-5547
U+6DA8.1	fR/S	 85.7
U+6DA8.1	fTotalStrokes	 10
U+6DA8.1	fCangjie	ENPO
U+6DA8.1	fMandarin	zhang4
U+6DA9.0	fUTF8	涩
U+6DA9.0	fGB	0-492C
U+6DA9.0	fR/S	 85.7
U+6DA9.0	fTotalStrokes	 10
U+6DA9.0	fCangjie	XESIM
U+6DA9.0	fHakka	 sep7
U+6DA9.0	fCantonese	gip3; sik1 (also); sap1 (also); sap3
U+6DA9.0	fMandarin	se4
U+6DA9.0	fEnglish	simplified form of &#x6F80;
U+6DAA.0	fUTF8	涪
U+6DAA.0	fCNS11643	1-5B6A
U+6DAA.0	fBig5	B265
U+6DAA.0	fGB	0-3822
U+6DAA.0	fR/S	 85.8
U+6DAA.0	fTotalStrokes	 11
U+6DAA.0	fCangjie	EYTR
U+6DAA.0	fFourCorner	30161
U+6DAA.0	fHakka	 feu2; fu1; fu2; fiu2
U+6DAA.0	fCantonese	fau4
U+6DAA.0	fMandarin	fu2; pou2
U+6DAA.0	fEnglish	[1] the name of a river in Szechwan province [2] the name of an old administrative district
U+6DAB.0	fUTF8	涫
U+6DAB.0	fCNS11643	2-3645
U+6DAB.0	fBig5	D5E1
U+6DAB.0	fGB	0-644A
U+6DAB.0	fR/S	 85.8
U+6DAB.0	fTotalStrokes	 11
U+6DAB.0	fCangjie	EJRR
U+6DAB.0	fFourCorner	30177
U+6DAB.0	fCantonese	gun3; gwun3 (also); gun1
U+6DAB.0	fMandarin	guan4; guan1; huan4
U+6DAC.0	fUTF8	涬
U+6DAC.0	fCNS11643	2-3649
U+6DAC.0	fBig5	D5E5
U+6DAC.0	fGB	3-5450
U+6DAC.0	fR/S	 85.8
U+6DAC.0	fTotalStrokes	 11
U+6DAC.0	fCangjie	EGTJ
U+6DAC.0	fFourCorner	34141
U+6DAC.0	fHakka	 hen5; hen6
U+6DAC.0	fCantonese	hang6; hang5
U+6DAC.0	fMandarin	xing4
U+6DAC.0	fEnglish	watery expanse
U+6DAD.0	fUTF8	涭
U+6DAD.0	fCNS11643	3-366F
U+6DAD.0	fGB	5-5577
U+6DAD.0	fR/S	 85.8
U+6DAD.0	fTotalStrokes	 11
U+6DAD.0	fCangjie	EBBE
U+6DAD.0	fFourCorner	32140
U+6DAD.0	fCantonese	sau6
U+6DAD.0	fMandarin	shou4
U+6DAE.0	fUTF8	涮
U+6DAE.0	fCNS11643	1-5B57
U+6DAE.0	fBig5	B252
U+6DAE.0	fGB	0-644C
U+6DAE.0	fR/S	 85.8
U+6DAE.0	fTotalStrokes	 11
U+6DAE.0	fCangjie	ESBN
U+6DAE.0	fFourCorner	32100
U+6DAE.0	fHakka	  sot7; san5; cat7
U+6DAE.0	fCantonese	saan3; hyun1; syun3
U+6DAE.0	fMandarin	shuan4; shua1
U+6DAE.0	fEnglish	[1] rinse (a container, etc.) [2] boil in a chafing pot [3] cheat with lies
U+6DAF.0	fUTF8	涯
U+6DAF.0	fCNS11643	1-5B55
U+6DAF.0	fBig5	B250
U+6DAF.0	fGB	0-5144
U+6DAF.0	fR/S	 85.8
U+6DAF.0	fTotalStrokes	 11
U+6DAF.0	fCangjie	EMGG
U+6DAF.0	fFourCorner	31114
U+6DAF.0	fHakka	  ngai2; ngai3
U+6DAF.0	fCantonese	ngaai4; aai4 (also); ji4
U+6DAF.0	fMandarin	ya2; yi2
U+6DAF.0	fEnglish	[1] shore; bank; water's edge [2] limit [3] faraway places
U+6DB0.0	fUTF8	涰
U+6DB0.0	fCNS11643	3-367A
U+6DB0.0	fGB	E-303C
U+6DB0.0	fR/S	  85.8
U+6DB0.0	fTotalStrokes	 11
U+6DB0.0	fCangjie	EEEE
U+6DB0.0	fFourCorner	37140
U+6DB0.0	fCantonese	zyut3
U+6DB0.0	fMandarin	chuo4
U+6DB1.0	fUTF8	涱
U+6DB1.0	fCNS11643	3-3721
U+6DB1.0	fGB	5-5477
U+6DB1.0	fR/S	 85.8
U+6DB1.0	fTotalStrokes	 11
U+6DB1.0	fCangjie	ESMV
U+6DB1.0	fFourCorner	31130
U+6DB1.0	fCantonese	zoeng2
U+6DB1.0	fMandarin	zhang4
U+6DB2.0	fUTF8	液
U+6DB2.0	fCNS11643	1-5B4C
U+6DB2.0	fBig5	B247
U+6DB2.0	fGB	0-523A
U+6DB2.0	fR/S	 85.8
U+6DB2.0	fTotalStrokes	 11
U+6DB2.0	fCangjie	EYOK
U+6DB2.0	fFourCorner	30147
U+6DB2.0	fHakka	 jit8; rit8; ye2
U+6DB2.0	fCantonese	jik6; jat6
U+6DB2.0	fMandarin	ye4; shi4; yi4
U+6DB2.0	fEnglish	 sap; juice; liquid; fluid; secretion
U+6DB3.0	fUTF8	涳
U+6DB3.0	fCNS11643	2-3647
U+6DB3.0	fBig5	D5E3
U+6DB3.0	fGB	5-557C
U+6DB3.0	fR/S	 85.8
U+6DB3.0	fTotalStrokes	 11
U+6DB3.0	fCangjie	EJCM
U+6DB3.0	fFourCorner	33111
U+6DB3.0	fHakka	 kung1
U+6DB3.0	fCantonese	hung1; gung1
U+6DB3.0	fMandarin	kong1; qiang1; nong2; kong4
U+6DB4.0	fUTF8	涴
U+6DB4.0	fCNS11643	2-3646
U+6DB4.0	fBig5	D5E2
U+6DB4.0	fGB	3-5471
U+6DB4.0	fR/S	 85.8
U+6DB4.0	fTotalStrokes	 11
U+6DB4.0	fCangjie	EJNU
U+6DB4.0	fFourCorner	33112
U+6DB4.0	fHakka	 wan3; won3; van3; von3
U+6DB4.0	fCantonese	wo3; jyun2; jyun1
U+6DB4.0	fMandarin	wan3; wo4; yuan4; yuan1
U+6DB4.0	fEnglish	 stain; soil
U+6DB5.0	fUTF8	涵
U+6DB5.0	fCNS11643	1-5B60
U+6DB5.0	fBig5	B25B
U+6DB5.0	fGB	0-3A2D
U+6DB5.0	fR/S	 85.8
U+6DB5.0	fTotalStrokes	 11
U+6DB5.0	fCangjie	EUNE
U+6DB5.0	fFourCorner	31172
U+6DB5.0	fHakka	 ham2
U+6DB5.0	fCantonese	haam4
U+6DB5.0	fMandarin	han2; han4; xian2
U+6DB5.0	fEnglish	[1] damp and marshy; wet [2] contain [3] lenient and broad-minded
U+6DB6.0	fUTF8	涶
U+6DB6.0	fCNS11643	3-3C50
U+6DB6.0	fGB	3-5460
U+6DB6.0	fR/S	 85.8; 85.9
U+6DB6.0	fTotalStrokes	 12; 11
U+6DB6.0	fCangjie	EHJM
U+6DB6.0	fFourCorner	32110
U+6DB6.0	fHakka	 to5; tio5; tui5
U+6DB6.0	fCantonese	to1
U+6DB6.0	fMandarin	tuo1
U+6DB6.0	fEnglish	 spit saliva
U+6DB7.0	fUTF8	涷
U+6DB7.0	fCNS11643	2-364C
U+6DB7.0	fBig5	D5E8
U+6DB7.0	fGB	3-5362
U+6DB7.0	fR/S	 85.8
U+6DB7.0	fTotalStrokes	 11
U+6DB7.0	fCangjie	EDW
U+6DB7.0	fFourCorner	35196
U+6DB7.0	fHakka	 dung5; dung1
U+6DB7.0	fCantonese	dung1; dung3 (also)
U+6DB7.0	fMandarin	dong1; dong4
U+6DB7.0	fEnglish	rainstorm
U+6DB8.0	fUTF8	涸
U+6DB8.0	fCNS11643	1-5B5A
U+6DB8.0	fBig5	B255
U+6DB8.0	fGB	0-3A54
U+6DB8.0	fR/S	 85.8
U+6DB8.0	fTotalStrokes	 11
U+6DB8.0	fCangjie	EWJR
U+6DB8.0	fFourCorner	36100
U+6DB8.0	fHakka	  kok7; kok8; hok7
U+6DB8.0	fCantonese	kok3; hok6
U+6DB8.0	fMandarin	he2; hao4; hu4
U+6DB8.0	fEnglish	drying up; dried up; exhausted; dry
U+6DB9.0	fUTF8	涹
U+6DB9.0	fCNS11643	3-3672
U+6DB9.0	fGB	3-5461
U+6DB9.0	fR/S	 85.8
U+6DB9.0	fTotalStrokes	 11
U+6DB9.0	fCangjie	EHDV
U+6DB9.0	fFourCorner	32140
U+6DB9.0	fHakka	 wo1; wi1; wui1; vui1
U+6DB9.0	fCantonese	wo1
U+6DB9.0	fMandarin	wo1
U+6DB9.0	fEnglish	muddy
U+6DBA.0	fUTF8	涺
U+6DBA.0	fCNS11643	2-365E
U+6DBA.0	fBig5	D5FA
U+6DBA.0	fGB	5-5621
U+6DBA.0	fR/S	 85.8
U+6DBA.0	fTotalStrokes	 11
U+6DBA.0	fCangjie	ESJR
U+6DBA.0	fFourCorner	37164
U+6DBA.0	fCantonese	geoi1
U+6DBA.0	fMandarin	ju1
U+6DBB.0	fUTF8	涻
U+6DBB.0	fCNS11643	2-366A
U+6DBB.0	fBig5	D647
U+6DBB.0	fGB	5-5575
U+6DBB.0	fR/S	 85.8
U+6DBB.0	fTotalStrokes	 11
U+6DBB.0	fCangjie	EOMR
U+6DBB.0	fFourCorner	38164
U+6DBB.0	fCantonese	se3
U+6DBB.0	fMandarin	gan4; she4
U+6DBC.0	fUTF8	涼
U+6DBC.0	fCNS11643	1-5B49
U+6DBC.0	fBig5	B244
U+6DBC.0	fGB	E-303D
U+6DBC.0	fR/S	 85.8
U+6DBC.0	fTotalStrokes	 11
U+6DBC.0	fCangjie	EYRF
U+6DBC.0	fFourCorner	30196
U+6DBC.0	fHakka	 liong2
U+6DBC.0	fCantonese	loeng4; loeng6; loeng2 (also)
U+6DBC.0	fMandarin	liang2; liang4
U+6DBC.0	fEnglish	[1] cool; cold; chilly [2] thin [3] discouraged; disappointed
U+6DBD.0	fUTF8	涽
U+6DBD.0	fCNS11643	2-365B
U+6DBD.0	fBig5	D5F7
U+6DBD.0	fGB	3-5468
U+6DBD.0	fR/S	 85.8
U+6DBD.0	fTotalStrokes	 11
U+6DBD.0	fCangjie	EHPA
U+6DBD.0	fFourCorner	32164
U+6DBD.0	fHakka	  fun1; men3
U+6DBD.0	fCantonese	fan1
U+6DBD.0	fMandarin	hun1; hun4
U+6DBD.0	fEnglish	unstable, muddy
U+6DBE.0	fUTF8	涾
U+6DBE.0	fCNS11643	2-3654
U+6DBE.0	fBig5	D5F0
U+6DBE.0	fGB	3-5474
U+6DBE.0	fR/S	 85.8
U+6DBE.0	fTotalStrokes	 11
U+6DBE.0	fCangjie	EEA
U+6DBE.0	fFourCorner	32163
U+6DBE.0	fCantonese	taap3; daap6
U+6DBE.0	fMandarin	ta4
U+6DBF.0	fUTF8	涿
U+6DBF.0	fCNS11643	1-5B6C
U+6DBF.0	fBig5	B267
U+6DBF.0	fGB	0-6443
U+6DBF.0	fR/S	 85.8
U+6DBF.0	fTotalStrokes	 11
U+6DBF.0	fCangjie	EMSO
U+6DBF.0	fFourCorner	31132
U+6DBF.0	fHakka	 duk7;  dok7; tuk7
U+6DBF.0	fCantonese	doek3
U+6DBF.0	fMandarin	zhuo1; zhuo2; tan1
U+6DBF.0	fEnglish	[1] drip; dribble; trickle [2] old and current names of counties, rivers, mountains in various places
U+6DC0.0	fUTF8	淀
U+6DC0.0	fCNS11643	2-3644
U+6DC0.0	fBig5	D5E0
U+6DC0.0	fGB	0-356D
U+6DC0.0	fR/S	 85.8
U+6DC0.0	fTotalStrokes	 11
U+6DC0.0	fCangjie	EJMO
U+6DC0.0	fFourCorner	33181
U+6DC0.0	fHakka	 tien5; tin2
U+6DC0.0	fCantonese	din6
U+6DC0.0	fMandarin	dian4
U+6DC0.0	fEnglish	[1] shallow water [2] simplified form of &#x6FB1;
U+6DC1.0	fUTF8	淁
U+6DC1.0	fCNS11643	3-366A
U+6DC1.0	fGB	3-546C
U+6DC1.0	fR/S	 85.8
U+6DC1.0	fTotalStrokes	 11
U+6DC1.0	fCangjie	EYTV
U+6DC1.0	fFourCorner	30140
U+6DC1.0	fCantonese	cip3
U+6DC1.0	fMandarin	qie4; ji2
U+6DC2.0	fUTF8	淂
U+6DC2.0	fCNS11643	2-3660
U+6DC2.0	fBig5	D5FC
U+6DC2.0	fGB	3-545B
U+6DC2.0	fR/S	 85.8
U+6DC2.0	fTotalStrokes	 11
U+6DC2.0	fCangjie	EAMI
U+6DC2.0	fFourCorner	36141
U+6DC2.0	fHakka	 det7
U+6DC2.0	fCantonese	dak1
U+6DC2.0	fMandarin	de2; di1
U+6DC2.0	fEnglish	watery
U+6DC3.0	fUTF8	淃
U+6DC3.0	fCNS11643	3-366B
U+6DC3.0	fGB	3-546E
U+6DC3.0	fR/S	  85.8
U+6DC3.0	fTotalStrokes	 11
U+6DC3.0	fCangjie	EFQU
U+6DC3.0	fFourCorner	39110
U+6DC3.0	fCantonese	gyun6
U+6DC3.0	fMandarin	juan4
U+6DC4.0	fUTF8	淄
U+6DC4.0	fCNS11643	1-5B69
U+6DC4.0	fBig5	B264
U+6DC4.0	fGB	0-574D
U+6DC4.0	fR/S	 85.8
U+6DC4.0	fTotalStrokes	 11
U+6DC4.0	fCangjie	EVVW
U+6DC4.0	fFourCorner	32160
U+6DC4.0	fHakka	 zih1; zu1
U+6DC4.0	fCantonese	zi1
U+6DC4.0	fMandarin	zi1
U+6DC4.0	fEnglish	[1] black [2] the name of a river in Shantung province
U+6DC5.0	fUTF8	淅
U+6DC5.0	fCNS11643	1-5B5D
U+6DC5.0	fBig5	B258
U+6DC5.0	fGB	0-6440
U+6DC5.0	fR/S	 85.8
U+6DC5.0	fTotalStrokes	 11
U+6DC5.0	fCangjie	EDHL
U+6DC5.0	fFourCorner	32121
U+6DC5.0	fHakka	 sit7; zhat7
U+6DC5.0	fCantonese	sik1
U+6DC5.0	fMandarin	xi1
U+6DC5.0	fEnglish	[1] water used to wash rice; water for washing rice wash rice [2] a river in Honan province
U+6DC6.0	fUTF8	淆
U+6DC6.0	fCNS11643	1-5B68
U+6DC6.0	fBig5	B263
U+6DC6.0	fGB	0-4F7D
U+6DC6.0	fR/S	 85.8
U+6DC6.0	fTotalStrokes	 11
U+6DC6.0	fCangjie	EKKB
U+6DC6.0	fFourCorner	34127
U+6DC6.0	fHakka	  ngau2; ngau3
U+6DC6.0	fCantonese	ngaau4; aau4 (also); au4
U+6DC6.0	fMandarin	xiao2; yao2
U+6DC6.0	fEnglish	confused and disorderly; mixed
U+6DC7.0	fUTF8	淇
U+6DC7.0	fCNS11643	1-5B53
U+6DC7.0	fBig5	B24E
U+6DC7.0	fGB	0-643F
U+6DC7.0	fR/S	 85.8
U+6DC7.0	fTotalStrokes	 11
U+6DC7.0	fCangjie	ETMC
U+6DC7.0	fFourCorner	34181
U+6DC7.0	fHakka	 ki2
U+6DC7.0	fCantonese	kei4
U+6DC7.0	fMandarin	qi2
U+6DC7.0	fEnglish	a river in Honan province
U+6DC8.0	fUTF8	淈
U+6DC8.0	fCNS11643	2-3650
U+6DC8.0	fBig5	D5EC
U+6DC8.0	fGB	3-5473
U+6DC8.0	fR/S	 85.8
U+6DC8.0	fTotalStrokes	 11
U+6DC8.0	fCangjie	ESUU
U+6DC8.0	fFourCorner	37177
U+6DC8.0	fHakka	 gut7; kut7; kut8; gwut7
U+6DC8.0	fCantonese	gwat1
U+6DC8.0	fMandarin	gu3; he2
U+6DC8.0	fEnglish	[1] dirty water [2] confused [3] drain off
U+6DC9.0	fUTF8	淉
U+6DC9.0	fCNS11643	2-3662
U+6DC9.0	fBig5	D5FE
U+6DC9.0	fGB	3-545C
U+6DC9.0	fR/S	 85.8
U+6DC9.0	fTotalStrokes	 11
U+6DC9.0	fCangjie	EWD
U+6DC9.0	fFourCorner	36194
U+6DC9.0	fHakka	 go3
U+6DC9.0	fCantonese	gwo2; go2 (also); gun3
U+6DC9.0	fMandarin	guo3; guan4
U+6DCA.0	fUTF8	淊
U+6DCA.0	fCNS11643	2-365A
U+6DCA.0	fBig5	D5F6
U+6DCA.0	fGB	3-5469
U+6DCA.0	fR/S	 85.8
U+6DCA.0	fTotalStrokes	 11
U+6DCA.0	fCangjie	ENHX
U+6DCA.0	fFourCorner	37177
U+6DCA.0	fCantonese	jim1
U+6DCA.0	fMandarin	han4; yan1; han2; yan3; yan4
U+6DCB.0	fUTF8	淋
U+6DCB.0	fCNS11643	1-5B54
U+6DCB.0	fBig5	B24F
U+6DCB.0	fGB	0-415C
U+6DCB.0	fR/S	 85.8
U+6DCB.0	fTotalStrokes	 11
U+6DCB.0	fCangjie	EDD
U+6DCB.0	fFourCorner	34190
U+6DCB.0	fHakka	  lim2; lim3
U+6DCB.0	fCantonese	lam4; lam2 (also); lam6
U+6DCB.0	fMandarin	lin2; lun2
U+6DCB.0	fEnglish	[1] drip; soak with water [2] gonorrhea
U+6DCB.1	fUTF8	淋
U+6DCB.1	fCNS11643	1-5B54
U+6DCB.1	fBig5	B24F
U+6DCB.1	fGB	0-415C
U+6DCB.1	fR/S	 85.8
U+6DCB.1	fTotalStrokes	 11
U+6DCB.1	fCangjie	EDD
U+6DCB.1	fFourCorner	34190
U+6DCB.1	fMandarin	lin4
U+6DCB.1	fEnglish	 filter; strain
U+6DCC.0	fUTF8	淌
U+6DCC.0	fCNS11643	1-5B4E
U+6DCC.0	fBig5	B249
U+6DCC.0	fGB	0-4C4A
U+6DCC.0	fR/S	 85.8
U+6DCC.0	fTotalStrokes	 11
U+6DCC.0	fCangjie	EFBR
U+6DCC.0	fFourCorner	39127
U+6DCC.0	fHakka	 chong3; cong3; tong3
U+6DCC.0	fCantonese	tong2
U+6DCC.0	fMandarin	tang3; chang3; chang4
U+6DCC.0	fEnglish	 trickle; flow down; drip
U+6DCD.0	fUTF8	淍
U+6DCD.0	fCNS11643	2-3668
U+6DCD.0	fBig5	D645
U+6DCD.0	fGB	3-5467
U+6DCD.0	fR/S	 85.8
U+6DCD.0	fTotalStrokes	 11
U+6DCD.0	fCangjie	EBGR
U+6DCD.0	fFourCorner	37120
U+6DCD.0	fCantonese	zau1
U+6DCD.0	fMandarin	zhou1
U+6DCE.0	fUTF8	淎
U+6DCE.0	fCNS11643	3-366C
U+6DCE.0	fGB	5-5567
U+6DCE.0	fR/S	  85.8
U+6DCE.0	fTotalStrokes	 11
U+6DCE.0	fCangjie	EQKQ
U+6DCE.0	fFourCorner	35150
U+6DCE.0	fCantonese	fung2
U+6DCE.0	fMandarin	peng3
U+6DCF.0	fUTF8	淏
U+6DCF.0	fCNS11643	2-3661
U+6DCF.0	fBig5	D5FD
U+6DCF.0	fGB	3-545A
U+6DCF.0	fR/S	 85.8
U+6DCF.0	fTotalStrokes	 11
U+6DCF.0	fCangjie	EAMK
U+6DCF.0	fFourCorner	36134
U+6DCF.0	fHakka	 hau1
U+6DCF.0	fCantonese	hou6
U+6DCF.0	fMandarin	hao4
U+6DD0.0	fUTF8	淐
U+6DD0.0	fCNS11643	2-3663
U+6DD0.0	fBig5	D640
U+6DD0.0	fGB	3-545E
U+6DD0.0	fR/S	 85.8
U+6DD0.0	fTotalStrokes	 11
U+6DD0.0	fCangjie	EAA
U+6DD0.0	fFourCorner	36160
U+6DD0.0	fCantonese	coeng1
U+6DD0.0	fMandarin	chang1
U+6DD1.0	fUTF8	淑
U+6DD1.0	fCNS11643	1-5B56
U+6DD1.0	fBig5	B251
U+6DD1.0	fGB	0-4A67
U+6DD1.0	fR/S	 85.8
U+6DD1.0	fTotalStrokes	 11
U+6DD1.0	fCangjie	EYFE
U+6DD1.0	fFourCorner	37140
U+6DD1.0	fHakka	 suk7;  shuk7; shuk8; siuk7
U+6DD1.0	fCantonese	suk6; suk1
U+6DD1.0	fMandarin	shu1; shu2; chu4
U+6DD1.0	fEnglish	[1] good; pure; virtuous [2] beautiful; charming (women) [3] clear
U+6DD2.0	fUTF8	淒
U+6DD2.0	fCNS11643	1-5B5E
U+6DD2.0	fBig5	B259
U+6DD2.0	fGB	E-303E
U+6DD2.0	fR/S	 85.8
U+6DD2.0	fTotalStrokes	 11
U+6DD2.0	fCangjie	EJLV
U+6DD2.0	fFourCorner	35144
U+6DD2.0	fHakka	  ci1; zi1
U+6DD2.0	fCantonese	cai1
U+6DD2.0	fMandarin	qi1; ci3; qian4
U+6DD2.0	fEnglish	[1] cold and chilly [2] cloudy and rainy [3] sorrow; sorrowful; miserable; desolate
U+6DD3.0	fUTF8	淓
U+6DD3.0	fCNS11643	2-3665
U+6DD3.0	fBig5	D642
U+6DD3.0	fGB	5-5550
U+6DD3.0	fR/S	 85.8
U+6DD3.0	fTotalStrokes	 11
U+6DD3.0	fCangjie	ETYS
U+6DD3.0	fFourCorner	34127
U+6DD3.0	fCantonese	fong1
U+6DD3.0	fMandarin	fang1
U+6DD4.0	fUTF8	淔
U+6DD4.0	fCNS11643	2-364E
U+6DD4.0	fBig5	D5EA
U+6DD4.0	fGB	3-5452
U+6DD4.0	fR/S	 85.8
U+6DD4.0	fTotalStrokes	 11
U+6DD4.0	fCangjie	EJBM
U+6DD4.0	fFourCorner	34116
U+6DD4.0	fCantonese	zik6; cik1
U+6DD4.0	fMandarin	chi4; zhi2
U+6DD5.0	fUTF8	淕
U+6DD5.0	fCNS11643	2-365F
U+6DD5.0	fBig5	D5FB
U+6DD5.0	fGB	3-544E
U+6DD5.0	fR/S	 85.8
U+6DD5.0	fTotalStrokes	 11
U+6DD5.0	fCangjie	EGCG
U+6DD5.0	fFourCorner	34114
U+6DD5.0	fCantonese	luk6
U+6DD5.0	fMandarin	lu4
U+6DD6.0	fUTF8	淖
U+6DD6.0	fCNS11643	2-3653
U+6DD6.0	fBig5	D5EF
U+6DD6.0	fGB	0-4457
U+6DD6.0	fR/S	 85.8
U+6DD6.0	fTotalStrokes	 11
U+6DD6.0	fCangjie	EYAJ
U+6DD6.0	fFourCorner	31147
U+6DD6.0	fHakka	 cau5;  nau5; neu2; nau6; lau5
U+6DD6.0	fCantonese	naau6; zoek3; laau6 (also); coek3
U+6DD6.0	fMandarin	nao4; chuo4; zhao4; zhuo1; shu3; zhuo2
U+6DD6.0	fEnglish	slush; mud
U+6DD7.0	fUTF8	淗
U+6DD7.0	fCNS11643	2-3667
U+6DD7.0	fBig5	D644
U+6DD7.0	fGB	5-5578
U+6DD7.0	fR/S	 85.8
U+6DD7.0	fTotalStrokes	 11
U+6DD7.0	fCangjie	EPFD
U+6DD7.0	fFourCorner	37120
U+6DD7.0	fCantonese	guk1
U+6DD7.0	fMandarin	ju2
U+6DD8.0	fUTF8	淘
U+6DD8.0	fCNS11643	1-5B63
U+6DD8.0	fBig5	B25E
U+6DD8.0	fGB	0-4C54
U+6DD8.0	fR/S	 85.8
U+6DD8.0	fTotalStrokes	 11
U+6DD8.0	fCangjie	EPOU
U+6DD8.0	fFourCorner	37120
U+6DD8.0	fHakka	  tau2; to2; to3; jau3; rau2; jau2
U+6DD8.0	fCantonese	tou4
U+6DD8.0	fMandarin	tao2
U+6DD8.0	fEnglish	[1] wash in sieve; wash (especially rice) [2] dredge; scour
U+6DD9.0	fUTF8	淙
U+6DD9.0	fCNS11643	1-5B4B
U+6DD9.0	fBig5	B246
U+6DD9.0	fGB	0-6448
U+6DD9.0	fR/S	 85.8
U+6DD9.0	fTotalStrokes	 11
U+6DD9.0	fCangjie	EJMF
U+6DD9.0	fFourCorner	33191
U+6DD9.0	fHakka	 zung1; cung1; cung2
U+6DD9.0	fCantonese	cung4; zung1 (also); cung6
U+6DD9.0	fMandarin	shuang4; cong1; cong2; zong1; zhuang4
U+6DD9.0	fEnglish	gurgling sound of water; water flowing; sound of flowing water
U+6DDA.0	fUTF8	淚
U+6DDA.0	fCNS11643	1-5B61
U+6DDA.0	fBig5	B25C
U+6DDA.0	fGB	E-303F
U+6DDA.0	fR/S	 85.8; 85.7
U+6DDA.0	fTotalStrokes	 11
U+6DDA.0	fCangjie	EHSK
U+6DDA.0	fFourCorner	33134
U+6DDA.0	fHakka	  lui5; lui6; lui3
U+6DDA.0	fCantonese	lai6; leoi6
U+6DDA.0	fMandarin	lei4; li4; lie4
U+6DDA.0	fEnglish	tears
U+6DDB.0	fUTF8	淛
U+6DDB.0	fCNS11643	2-3658
U+6DDB.0	fBig5	D5F4
U+6DDB.0	fGB	E-3040
U+6DDB.0	fR/S	 85.8
U+6DDB.0	fTotalStrokes	 11
U+6DDB.0	fCangjie	EHBN
U+6DDB.0	fFourCorner	32100
U+6DDB.0	fCantonese	zit3
U+6DDB.0	fMandarin	zhi4; zhe2; zhe4
U+6DDC.0	fUTF8	淜
U+6DDC.0	fCNS11643	2-3656
U+6DDC.0	fBig5	D5F2
U+6DDC.0	fGB	3-5466
U+6DDC.0	fR/S	 85.8
U+6DDC.0	fTotalStrokes	 11
U+6DDC.0	fCangjie	EBB
U+6DDC.0	fFourCorner	37120
U+6DDC.0	fHakka	  pen2;  pang2; bang2
U+6DDC.0	fCantonese	pang4; ping1; paang1; paang4
U+6DDC.0	fMandarin	peng2; feng2; ping2; pan2; peng1
U+6DDC.0	fEnglish	roar of dashing waves
U+6DDD.0	fUTF8	淝
U+6DDD.0	fCNS11643	2-3657
U+6DDD.0	fBig5	D5F3
U+6DDD.0	fGB	0-6447
U+6DDD.0	fR/S	 85.8
U+6DDD.0	fTotalStrokes	 11
U+6DDD.0	fCangjie	EBAU
U+6DDD.0	fFourCorner	37117
U+6DDD.0	fHakka	 pui2; pi2; fui2
U+6DDD.0	fCantonese	fei4; fai6
U+6DDD.0	fMandarin	fei2
U+6DDD.0	fEnglish	a river in Anhwei province
U+6DDE.0	fUTF8	淞
U+6DDE.0	fCNS11643	1-5B58
U+6DDE.0	fBig5	B253
U+6DDE.0	fGB	0-6441
U+6DDE.0	fR/S	 85.8
U+6DDE.0	fTotalStrokes	 11
U+6DDE.0	fCangjie	EDCI
U+6DDE.0	fFourCorner	38132
U+6DDE.0	fHakka	  ciung2; ziung2; siung1
U+6DDE.0	fCantonese	sung1
U+6DDE.0	fMandarin	song1; song4
U+6DDE.0	fEnglish	a river in Kiangsu province
U+6DDF.0	fUTF8	淟
U+6DDF.0	fCNS11643	2-3652
U+6DDF.0	fBig5	D5EE
U+6DDF.0	fGB	3-545F
U+6DDF.0	fR/S	 85.8
U+6DDF.0	fTotalStrokes	 11
U+6DDF.0	fCangjie	ETBC
U+6DDF.0	fFourCorner	35181
U+6DDF.0	fHakka	  tien3
U+6DDF.0	fCantonese	tin2
U+6DDF.0	fMandarin	tian3; dian3
U+6DDF.0	fEnglish	turbid; muddy
U+6DE0.0	fUTF8	淠
U+6DE0.0	fCNS11643	2-3651
U+6DE0.0	fBig5	D5ED
U+6DE0.0	fGB	0-6444
U+6DE0.0	fR/S	 85.8
U+6DE0.0	fTotalStrokes	 11
U+6DE0.0	fCangjie	EWML
U+6DE0.0	fFourCorner	36121
U+6DE0.0	fHakka	  pi5
U+6DE0.0	fCantonese	pei3; pui3
U+6DE0.0	fMandarin	pi4; pei4; bei1
U+6DE0.0	fEnglish	a branch of the Huai river
U+6DE1.0	fUTF8	淡
U+6DE1.0	fCNS11643	1-5B4D
U+6DE1.0	fBig5	B248
U+6DE1.0	fGB	0-352D
U+6DE1.0	fR/S	 85.8
U+6DE1.0	fTotalStrokes	 11
U+6DE1.0	fCangjie	EFF
U+6DE1.0	fFourCorner	39189
U+6DE1.0	fHakka	  tam1; tam3;  tam5; tam6
U+6DE1.0	fCantonese	daam6; taam5 (also); taam4; jim5
U+6DE1.0	fMandarin	dan4; tan2; yan3; yan4
U+6DE1.0	fEnglish	[1] weak; thin (tea, coffee, etc.) [2] tasteless; without enough salt; insipid [3] off-season; when business is poor; dull; slack [4] light (color); slight [5] without worldly desires [6] same as $ (nitrogen)
U+6DE2.0	fUTF8	淢
U+6DE2.0	fCNS11643	2-364B
U+6DE2.0	fBig5	D5E7
U+6DE2.0	fGB	3-5454
U+6DE2.0	fR/S	 85.8
U+6DE2.0	fTotalStrokes	 11
U+6DE2.0	fCangjie	EIRM
U+6DE2.0	fFourCorner	33150
U+6DE2.0	fHakka	 wet7; vet7
U+6DE2.0	fCantonese	gwik1; wik6; kwik1
U+6DE2.0	fMandarin	yu4; xu4
U+6DE2.0	fEnglish	ditch
U+6DE3.0	fUTF8	淣
U+6DE3.0	fCNS11643	2-3669
U+6DE3.0	fBig5	D646
U+6DE3.0	fGB	5-5572
U+6DE3.0	fR/S	 85.8
U+6DE3.0	fTotalStrokes	 11
U+6DE3.0	fCangjie	EHXU
U+6DE3.0	fFourCorner	37117
U+6DE3.0	fCantonese	ngai4; ai4 (also); ngaai4
U+6DE3.0	fMandarin	ni2; yai2; yi4
U+6DE4.0	fUTF8	淤
U+6DE4.0	fCNS11643	1-5B4F
U+6DE4.0	fBig5	B24A
U+6DE4.0	fGB	0-5359
U+6DE4.0	fR/S	 85.8
U+6DE4.0	fTotalStrokes	 11
U+6DE4.0	fCangjie	EYSY
U+6DE4.0	fFourCorner	38133
U+6DE4.0	fHakka	 ji1; wu1; ri2; ji2; i1; vu1
U+6DE4.0	fCantonese	jyu1; jyu2 (also); ji1 (also); jyu3
U+6DE4.0	fMandarin	yu1
U+6DE4.0	fEnglish	[1] muddy sediment; sediment; mud [2] blocked; silt up; stalemated
U+6DE5.0	fUTF8	淥
U+6DE5.0	fCNS11643	2-3655
U+6DE5.0	fBig5	D5F1
U+6DE5.0	fGB	E-3041
U+6DE5.0	fR/S	 85.8
U+6DE5.0	fTotalStrokes	 11
U+6DE5.0	fCangjie	EVNE
U+6DE5.0	fFourCorner	37132
U+6DE5.0	fHakka	  luk8
U+6DE5.0	fCantonese	luk6
U+6DE5.0	fMandarin	lu4
U+6DE5.0	fEnglish	[1] clear water [2] drip; strain
U+6DE6.0	fUTF8	淦
U+6DE6.0	fCNS11643	1-5B6D
U+6DE6.0	fBig5	B268
U+6DE6.0	fGB	0-6446
U+6DE6.0	fR/S	 85.8
U+6DE6.0	fTotalStrokes	 11
U+6DE6.0	fCangjie	EC
U+6DE6.0	fFourCorner	38119
U+6DE6.0	fHakka	 gam5
U+6DE6.0	fCantonese	gam3; gaam3 (also)
U+6DE6.0	fMandarin	gan4; gan1; han2
U+6DE6.0	fEnglish	[1] a river in Kiangsi province [2] water leaking into a boat
U+6DE7.0	fUTF8	淧
U+6DE7.0	fCNS11643	3-3674
U+6DE7.0	fGB	E-3042
U+6DE7.0	fR/S	  85.8
U+6DE7.0	fTotalStrokes	 11
U+6DE7.0	fCangjie	EJPH
U+6DE7.0	fFourCorner	33130
U+6DE7.0	fCantonese	mat6
U+6DE7.0	fMandarin	mi4
U+6DE8.0	fUTF8	淨
U+6DE8.0	fCNS11643	1-5B67
U+6DE8.0	fBig5	B262
U+6DE8.0	fGB	E-3043
U+6DE8.0	fR/S	 85.8
U+6DE8.0	fTotalStrokes	 11
U+6DE8.0	fCangjie	EBSD
U+6DE8.0	fFourCorner	32157
U+6DE8.0	fHakka	  ciang5; ciang6; ciang3
U+6DE8.0	fCantonese	zing6; zeng6 (also)
U+6DE8.0	fMandarin	jing4; zhen1; zheng1
U+6DE8.0	fEnglish	[1] pure; clean; cleanse; purify [2] empty; vain [3] a role in Chinese opera with a heavily painted face [4] completely; totally [5] only; merely; nothing but [6] net (income, profit, etc.)
U+6DE9.0	fUTF8	淩
U+6DE9.0	fCNS11643	2-364A
U+6DE9.0	fBig5	D5E6
U+6DE9.0	fGB	3-544F
U+6DE9.0	fR/S	 85.8
U+6DE9.0	fTotalStrokes	 11
U+6DE9.0	fCangjie	EGCE
U+6DE9.0	fFourCorner	34147
U+6DE9.0	fHakka	 lin2
U+6DE9.0	fCantonese	ling4
U+6DE9.0	fMandarin	ling2
U+6DE9.0	fEnglish	[1] pass; cross; traverse [2] intrude; insult; bully [3] a Chinese family name
U+6DEA.0	fUTF8	淪
U+6DEA.0	fCNS11643	1-5B64
U+6DEA.0	fBig5	B25F
U+6DEA.0	fGB	1-4259
U+6DEA.0	fR/S	 85.8
U+6DEA.0	fTotalStrokes	 11
U+6DEA.0	fCangjie	EOMB
U+6DEA.0	fFourCorner	38127
U+6DEA.0	fHakka	 lun2; lun3
U+6DEA.0	fCantonese	leon4
U+6DEA.0	fMandarin	lun2; guan1; lun3; lun4
U+6DEA.0	fEnglish	[1] sink into oblivion; ruin, etc.; fall; engulfed or lost; submerged [2] ripples; eddying water
U+6DEB.0	fUTF8	淫
U+6DEB.0	fCNS11643	1-5B62
U+6DEB.0	fBig5	B25D
U+6DEB.0	fGB	0-5279
U+6DEB.0	fR/S	 85.8
U+6DEB.0	fTotalStrokes	 11
U+6DEB.0	fCangjie	EBHG
U+6DEB.0	fFourCorner	32114
U+6DEB.0	fHakka	  jim2; rim2; jim3
U+6DEB.0	fCantonese	jam4
U+6DEB.0	fMandarin	yin2; yan4; yao2
U+6DEB.0	fEnglish	[1] obscene; pornographic [2] licentious; lewd; lascivious; libidinous; dissolute [3] seduce; debauch; tempt; temptation [4] things related to sexual desire and behavior
U+6DEC.0	fUTF8	淬
U+6DEC.0	fCNS11643	1-5B6B
U+6DEC.0	fBig5	B266
U+6DEC.0	fGB	0-3463
U+6DEC.0	fR/S	 85.8
U+6DEC.0	fTotalStrokes	 11
U+6DEC.0	fCangjie	EYOJ
U+6DEC.0	fFourCorner	30148
U+6DEC.0	fHakka	 cui5; ci5
U+6DEC.0	fCantonese	ceoi3; seoi6 (also)
U+6DEC.0	fMandarin	cui4; zu2
U+6DEC.0	fEnglish	[1] temper iron or steel for making swords, etc. (also used figuratively) [2] dye; soak; dip into water
U+6DED.0	fUTF8	淭
U+6DED.0	fCNS11643	2-365C
U+6DED.0	fBig5	D5F8
U+6DED.0	fGB	E-3044
U+6DED.0	fR/S	 85.8
U+6DED.0	fTotalStrokes	 11
U+6DED.0	fCangjie	EHSD
U+6DED.0	fFourCorner	33194
U+6DED.0	fCantonese	keoi4
U+6DED.0	fMandarin	qu2
U+6DEE.0	fUTF8	淮
U+6DEE.0	fCNS11643	1-5B66
U+6DEE.0	fBig5	B261
U+6DEE.0	fGB	0-3B34
U+6DEE.0	fR/S	 85.8
U+6DEE.0	fTotalStrokes	 11
U+6DEE.0	fCangjie	EOG
U+6DEE.0	fFourCorner	30114
U+6DEE.0	fHakka	 fai2; fai3; foi2
U+6DEE.0	fCantonese	waai4
U+6DEE.0	fMandarin	huai2
U+6DEE.0	fEnglish	the name of a river flowing from West China into the Gulf of Pohai
U+6DEF.0	fUTF8	淯
U+6DEF.0	fCNS11643	2-3035
U+6DEF.0	fBig5	D252
U+6DEF.0	fGB	5-557A
U+6DEF.0	fR/S	 85.8
U+6DEF.0	fTotalStrokes	 11
U+6DEF.0	fCangjie	EYIB
U+6DEF.0	fFourCorner	30127
U+6DEF.0	fHakka	 juk7; ruk7; juk8
U+6DEF.0	fCantonese	juk6
U+6DEF.0	fMandarin	yu4
U+6DEF.0	fEnglish	a river in Honan
U+6DF0.0	fUTF8	淰
U+6DF0.0	fCNS11643	2-365D
U+6DF0.0	fBig5	D5F9
U+6DF0.0	fGB	3-5465
U+6DF0.0	fR/S	 85.8
U+6DF0.0	fTotalStrokes	 11
U+6DF0.0	fCangjie	EOIP
U+6DF0.0	fFourCorner	38132
U+6DF0.0	fHakka	 ngiam5; nam5; ngiam3; ngiam6
U+6DF0.0	fCantonese	naam5; nam4 (also); nam6; laam5 (also); sam2; lam6 (also); lam4 (also); nim2
U+6DF0.0	fMandarin	nian3; nian4; shen3; an3; na4; nan3; lian3
U+6DF0.0	fEnglish	 fish still
U+6DF1.0	fUTF8	深
U+6DF1.0	fCNS11643	1-5B65
U+6DF1.0	fBig5	B260
U+6DF1.0	fGB	0-496E
U+6DF1.0	fR/S	 85.8
U+6DF1.0	fTotalStrokes	 11
U+6DF1.0	fCangjie	EBCD
U+6DF1.0	fFourCorner	37194
U+6DF1.0	fHakka	  chim1;  cim1; cihm1
U+6DF1.0	fCantonese	sam1
U+6DF1.0	fMandarin	shen1; shen4
U+6DF1.0	fEnglish	[1] deep; depth [2] profound; mysterious; difficult; abstruse [3] close; intimate [4] very; extremely
U+6DF2.0	fUTF8	淲
U+6DF2.0	fCNS11643	2-3664
U+6DF2.0	fBig5	D641
U+6DF2.0	fGB	3-5459
U+6DF2.0	fR/S	 85.8
U+6DF2.0	fTotalStrokes	 11
U+6DF2.0	fCangjie	EYPU
U+6DF2.0	fFourCorner	31114
U+6DF2.0	fHakka	 biau1
U+6DF2.0	fCantonese	piu4
U+6DF2.0	fMandarin	piao2; hu1
U+6DF3.0	fUTF8	淳
U+6DF3.0	fCNS11643	1-5B4A
U+6DF3.0	fBig5	B245
U+6DF3.0	fGB	0-343E
U+6DF3.0	fR/S	 85.8
U+6DF3.0	fTotalStrokes	 11
U+6DF3.0	fCangjie	EYRD
U+6DF3.0	fFourCorner	30147
U+6DF3.0	fHakka	 sun2;  shun2
U+6DF3.0	fCantonese	seon4
U+6DF3.0	fMandarin	chun2; zhun1; zhun3
U+6DF3.0	fEnglish	[1] pure; clean; simple; sincere; honest [2] couple; pair (of chariots) [3] big; great
U+6DF4.0	fUTF8	淴
U+6DF4.0	fCNS11643	2-3659
U+6DF4.0	fBig5	D5F5
U+6DF4.0	fGB	3-546B
U+6DF4.0	fR/S	 85.8
U+6DF4.0	fTotalStrokes	 11
U+6DF4.0	fCangjie	EPHP
U+6DF4.0	fFourCorner	37132
U+6DF4.0	fHakka	 fut7
U+6DF4.0	fCantonese	fat1
U+6DF4.0	fMandarin	hu1; wa4; hui4
U+6DF4.0	fEnglish	the sound of flowing waters
U+6DF5.0	fUTF8	淵
U+6DF5.0	fCNS11643	1-5B5C
U+6DF5.0	fBig5	B257
U+6DF5.0	fGB	1-5428
U+6DF5.0	fR/S	 85.8
U+6DF5.0	fTotalStrokes	 11
U+6DF5.0	fCangjie	ELXL
U+6DF5.0	fFourCorner	32100
U+6DF5.0	fHakka	 jen1; jan1; ren1
U+6DF5.0	fCantonese	jyun1
U+6DF5.0	fMandarin	yuan1; yun1
U+6DF5.0	fEnglish	[1] deep waters; gulf; abyss [2] profound (learning); depth; profundity; erudition; extensive
U+6DF6.0	fUTF8	淶
U+6DF6.0	fCNS11643	2-364D
U+6DF6.0	fBig5	D5E9
U+6DF6.0	fGB	1-6435
U+6DF6.0	fR/S	 85.8
U+6DF6.0	fTotalStrokes	 11
U+6DF6.0	fCangjie	EDOO
U+6DF6.0	fFourCorner	34198
U+6DF6.0	fHakka	  loi2
U+6DF6.0	fCantonese	loi4
U+6DF6.0	fMandarin	lai2
U+6DF6.0	fEnglish	a river in Hopeh province
U+6DF7.0	fUTF8	混
U+6DF7.0	fCNS11643	1-5B5B
U+6DF7.0	fBig5	B256
U+6DF7.0	fGB	0-3B6C
U+6DF7.0	fR/S	 85.8
U+6DF7.0	fTotalStrokes	 11
U+6DF7.0	fCangjie	EAPP
U+6DF7.0	fFourCorner	36111
U+6DF7.0	fHakka	 fun3; fun5; gun3; kun3; kun2; gwun3
U+6DF7.0	fCantonese	wan6; wan4; gwan2; kwan1
U+6DF7.0	fMandarin	hun4; gun3; kun1
U+6DF7.0	fEnglish	[1] disorderly; confused [2] mix; blend; mingle; mixed [3] fool around; just to get along [4] do things at random or without purpose
U+6DF7.1	fUTF8	混
U+6DF7.1	fCNS11643	1-5B5B
U+6DF7.1	fBig5	B256
U+6DF7.1	fGB	0-3B6C
U+6DF7.1	fR/S	 85.8
U+6DF7.1	fTotalStrokes	 11
U+6DF7.1	fCangjie	EAPP
U+6DF7.1	fFourCorner	36111
U+6DF7.1	fMandarin	hun2
U+6DF7.1	fEnglish	turbid; muddy; not clear
U+6DF7.2	fUTF8	混
U+6DF7.2	fCNS11643	1-5B5B
U+6DF7.2	fBig5	B256
U+6DF7.2	fGB	0-3B6C
U+6DF7.2	fR/S	 85.8
U+6DF7.2	fTotalStrokes	 11
U+6DF7.2	fCangjie	EAPP
U+6DF7.2	fFourCorner	36111
U+6DF7.2	fMandarin	hun3
U+6DF7.2	fEnglish	confused and not distinguishable
U+6DF8.0	fUTF8	淸
U+6DF8.0	fCNS11643	3-367C
U+6DF8.0	fGB	E-3045
U+6DF8.0	fR/S	 85.8
U+6DF8.0	fTotalStrokes	 11
U+6DF8.0	fCangjie	EQMB
U+6DF8.0	fFourCorner	35120
U+6DF8.0	fHakka	 cin1
U+6DF8.0	fCantonese	cing1
U+6DF8.0	fMandarin	qing1
U+6DF8.0	fEnglish	clear
U+6DF9.0	fUTF8	淹
U+6DF9.0	fCNS11643	1-5B59
U+6DF9.0	fBig5	B254
U+6DF9.0	fGB	0-514D
U+6DF9.0	fR/S	 85.8
U+6DF9.0	fTotalStrokes	 11
U+6DF9.0	fCangjie	EKLU
U+6DF9.0	fFourCorner	34116
U+6DF9.0	fHakka	  jam1; jam3; ram1; jam5
U+6DF9.0	fCantonese	jim1
U+6DF9.0	fMandarin	yan1; yan3; yan4; yi4
U+6DF9.0	fEnglish	[1] drown; cover with liquid; steep; flood; submerge [2] delay; procrastinate [3] stay; be stranded [4] deep; wellversed; erudite; profound [5] discomfort; caused by perspiration or liquids on skin
U+6DFA.0	fUTF8	淺
U+6DFA.0	fCNS11643	1-5B51
U+6DFA.0	fBig5	B24C
U+6DFA.0	fGB	1-4733
U+6DFA.0	fR/S	 85.8
U+6DFA.0	fTotalStrokes	 11
U+6DFA.0	fCangjie	EII
U+6DFA.0	fFourCorner	33153
U+6DFA.0	fHakka	 cien3; cen3; ts'ian3
U+6DFA.0	fCantonese	cin2; zin1
U+6DFA.0	fMandarin	qian3; jian1; can2; jian4; zan4
U+6DFA.0	fEnglish	[1] shallow; not deep; superficial [2] easy; simple [3] light (color) [4] narrow and small (land)
U+6DFB.0	fUTF8	添
U+6DFB.0	fCNS11643	1-5B50
U+6DFB.0	fBig5	B24B
U+6DFB.0	fGB	0-4C6D
U+6DFB.0	fR/S	 85.8
U+6DFB.0	fTotalStrokes	 11
U+6DFB.0	fCangjie	EHKP
U+6DFB.0	fFourCorner	32133
U+6DFB.0	fHakka	  tiam1
U+6DFB.0	fCantonese	tim1
U+6DFB.0	fMandarin	tian1; tian4
U+6DFB.0	fEnglish	[1] append; add to; increase; replenish (stock, etc.) [2] have a baby
U+6DFC.0	fUTF8	淼
U+6DFC.0	fCNS11643	2-3D2C
U+6DFC.0	fBig5	D9E7
U+6DFC.0	fGB	0-6D35
U+6DFC.0	fR/S	 85.8
U+6DFC.0	fTotalStrokes	 12
U+6DFC.0	fCangjie	EEE
U+6DFC.0	fFourCorner	12232
U+6DFC.0	fHakka	 miau3; meu3; miau1
U+6DFC.0	fCantonese	miu5
U+6DFC.0	fMandarin	miao3
U+6DFC.0	fEnglish	extensive; overwhelming (water)
U+6DFD.0	fUTF8	淽
U+6DFD.0	fCNS11643	2-3666
U+6DFD.0	fBig5	D643
U+6DFD.0	fGB	5-554F
U+6DFD.0	fR/S	 85.8
U+6DFD.0	fTotalStrokes	 11
U+6DFD.0	fCangjie	ETYM
U+6DFD.0	fFourCorner	34111
U+6DFD.0	fCantonese	zi2
U+6DFD.0	fMandarin	zhi3
U+6DFE.0	fUTF8	淾
U+6DFE.0	fCNS11643	3-3C5D
U+6DFE.0	fGB	5-7530
U+6DFE.0	fR/S	  85.8
U+6DFE.0	fTotalStrokes	 12
U+6DFE.0	fCangjie	CE
U+6DFE.0	fFourCorner	80230
U+6DFE.0	fCantonese	jam2
U+6DFE.0	fMandarin	yin3
U+6DFF.0	fUTF8	淿
U+6DFF.0	fCNS11643	3-3728
U+6DFF.0	fGB	E-3046
U+6DFF.0	fR/S	  85.8
U+6DFF.0	fTotalStrokes	 11
U+6DFF.0	fCangjie	EHAB
U+6DFF.0	fFourCorner	36120
U+6DFF.0	fCantonese	bok6
U+6DFF.0	fMandarin	mi4
U+6E00.0	fUTF8	渀
U+6E00.0	fCNS11643	2-364F
U+6E00.0	fBig5	D5EB
U+6E00.0	fGB	3-5457
U+6E00.0	fR/S	 85.8
U+6E00.0	fTotalStrokes	 11
U+6E00.0	fCangjie	EKJT
U+6E00.0	fFourCorner	34144
U+6E00.0	fCantonese	ban6; ban1; ban3
U+6E00.0	fMandarin	ben1
U+6E01.0	fUTF8	渁
U+6E01.0	fCNS11643	3-3726
U+6E01.0	fGB	E-3047
U+6E01.0	fR/S	  85.8
U+6E01.0	fTotalStrokes	 11
U+6E01.0	fCangjie	EEE
U+6E01.0	fFourCorner	32130
U+6E01.0	fCantonese	jyun1
U+6E01.0	fMandarin	yuan1
U+6E02.0	fUTF8	渂
U+6E02.0	fCNS11643	3-3723
U+6E02.0	fGB	5-556E
U+6E02.0	fR/S	 85.8
U+6E02.0	fTotalStrokes	 11
U+6E02.0	fCangjie	EAYK
U+6E02.0	fFourCorner	36140
U+6E02.0	fCantonese	man6
U+6E02.0	fMandarin	wen4
U+6E03.0	fUTF8	渃
U+6E03.0	fCNS11643	2-3D41
U+6E03.0	fBig5	D9FC
U+6E03.0	fGB	3-5451
U+6E03.0	fR/S	 85.9
U+6E03.0	fTotalStrokes	 12
U+6E03.0	fCangjie	ETKR
U+6E03.0	fFourCorner	34164
U+6E03.0	fCantonese	joek6; je6
U+6E03.0	fMandarin	re4; ruo4
U+6E04.0	fUTF8	渄
U+6E04.0	fCNS11643	3-3670
U+6E04.0	fGB	E-3048
U+6E04.0	fR/S	 85.8
U+6E04.0	fTotalStrokes	 11
U+6E04.0	fCangjie	ELMY
U+6E04.0	fFourCorner	31110
U+6E04.0	fCantonese	fei1
U+6E04.0	fMandarin	fei1
U+6E05.0	fUTF8	清
U+6E05.0	fCNS11643	1-5B52
U+6E05.0	fBig5	B24D
U+6E05.0	fGB	0-4765
U+6E05.0	fR/S	 85.8
U+6E05.0	fTotalStrokes	 11
U+6E05.0	fCangjie	EQMB
U+6E05.0	fFourCorner	35127
U+6E05.0	fHakka	 cin1; ciang1
U+6E05.0	fCantonese	cing1; ceng1
U+6E05.0	fMandarin	qing1; qing4; jing4
U+6E05.0	fEnglish	[1] clear; pure; clean [2] brief; scarce [3] virtuous; honest [4] arrange; place in order [5] conclude; terminate; repay (debts); settle [6] clear; simple and easily understandable [7] Ching dynasty (1644-1911) [8] clean
U+6E06.0	fUTF8	渆
U+6E06.0	fCNS11643	3-3677
U+6E06.0	fGB	E-3049
U+6E06.0	fR/S	  85.8
U+6E06.0	fTotalStrokes	 11
U+6E06.0	fCangjie	ETTN
U+6E06.0	fFourCorner	32100
U+6E06.0	fCantonese	jyun1
U+6E06.0	fMandarin	yuan1
U+6E07.0	fUTF8	渇
U+6E07.0	fCNS11643	3-3724
U+6E07.0	fGB	E-304A
U+6E07.0	fR/S	 85.8
U+6E07.0	fTotalStrokes	 11
U+6E07.0	fCangjie	EAPP
U+6E07.0	fFourCorner	36120
U+6E07.0	fCantonese	hot3
U+6E07.0	fMandarin	ke3
U+6E07.0	fEnglish	thirsty
U+6E08.0	fUTF8	済
U+6E08.0	fCNS11643	3-3676
U+6E08.0	fGB	E-304B
U+6E08.0	fR/S	 85.8
U+6E08.0	fTotalStrokes	 11
U+6E08.0	fCangjie	EYKL
U+6E08.0	fFourCorner	30120
U+6E08.0	fCantonese	zai3
U+6E08.0	fMandarin	ji4; ji3
U+6E08.0	fEnglish	 help; aid; relieve ferry; cross
U+6E09.0	fUTF8	渉
U+6E09.0	fCNS11643	3-3725
U+6E09.0	fGB	E-304C
U+6E09.0	fR/S	 85.8
U+6E09.0	fTotalStrokes	 11
U+6E09.0	fCangjie	EYLH
U+6E09.0	fFourCorner	31120
U+6E09.0	fCantonese	sip3
U+6E09.0	fMandarin	she4; die2
U+6E09.0	fEnglish	ford stream
U+6E0A.0	fUTF8	渊
U+6E0A.0	fCNS11643	3-3679
U+6E0A.0	fGB	0-5428
U+6E0A.0	fR/S	 85.8
U+6E0A.0	fTotalStrokes	 11
U+6E0A.0	fCangjie	ELFL
U+6E0A.0	fFourCorner	32100
U+6E0A.0	fHakka	 jen1
U+6E0A.0	fCantonese	jyun1
U+6E0A.0	fMandarin	yuan1; yun1
U+6E0A.0	fEnglish	simplified form of &#x6DF5;
U+6E0B.0	fUTF8	渋
U+6E0B.0	fCNS11643	4-326D
U+6E0B.0	fGB	E-304D
U+6E0B.0	fR/S	 85.8
U+6E0B.0	fTotalStrokes	 11
U+6E0B.0	fCangjie	EYMO
U+6E0B.0	fFourCorner	31130
U+6E0B.0	fCantonese	gip3
U+6E0B.0	fMandarin	se4
U+6E0B.0	fEnglish	 astringent; harsh; uneven, rough
U+6E0C.0	fUTF8	渌
U+6E0C.0	fCNS11643	3-367D
U+6E0C.0	fGB	0-644B
U+6E0C.0	fR/S	 85.8
U+6E0C.0	fTotalStrokes	 11
U+6E0C.0	fCangjie	ENME
U+6E0C.0	fFourCorner	37130
U+6E0C.0	fCantonese	luk6
U+6E0C.0	fMandarin	lu4
U+6E0C.0	fEnglish	strain
U+6E0D.0	fUTF8	渍
U+6E0D.0	fGB	0-5755
U+6E0D.0	fR/S	 85.8
U+6E0D.0	fTotalStrokes	 11
U+6E0D.0	fCangjie	EQMO
U+6E0D.0	fHakka	 zit7
U+6E0D.0	fCantonese	zi3; zik1 (also)
U+6E0D.0	fMandarin	zi4
U+6E0D.0	fEnglish	simplified form of &#x6F2C;
U+6E0E.0	fUTF8	渎
U+6E0E.0	fGB	0-6442
U+6E0E.0	fR/S	 85.8
U+6E0E.0	fTotalStrokes	 11
U+6E0E.0	fCangjie	EJNK
U+6E0E.0	fHakka	 tuk8
U+6E0E.0	fCantonese	duk6; dau6
U+6E0E.0	fMandarin	du2; dou4
U+6E0E.0	fEnglish	simplified form of &#x7006;
U+6E0F.0	fUTF8	渏
U+6E0F.0	fCNS11643	3-653E
U+6E0F.0	fGB	E-304E
U+6E0F.0	fR/S	 85.8
U+6E0F.0	fTotalStrokes	 11
U+6E0F.0	fCangjie	EKMR
U+6E0F.0	fFourCorner	3412
U+6E0F.0	fCantonese	ji1
U+6E10.0	fUTF8	渐
U+6E10.0	fGB	0-3D25
U+6E10.0	fR/S	 85.8
U+6E10.0	fTotalStrokes	 11
U+6E10.0	fCangjie	EKQL
U+6E10.0	fHakka	 ciam5
U+6E10.0	fCantonese	zim6; zim2 (also); zim1
U+6E10.0	fMandarin	jian4; chan2; jian1; zhan1; zhe2; qian2
U+6E10.0	fEnglish	simplified form of &#x6F38;
U+6E11.0	fUTF8	渑
U+6E11.0	fGB	0-6445
U+6E11.0	fR/S	 85.8
U+6E11.0	fTotalStrokes	 11
U+6E11.0	fCangjie	ERLU
U+6E11.0	fHakka	 shun2; mien3
U+6E11.0	fCantonese	man5; sing4
U+6E11.0	fMandarin	sheng2; mian3; min3
U+6E11.0	fEnglish	simplified form of &#x6FA0;
U+6E12.0	fUTF8	渒
U+6E12.0	fCNS11643	3-3729
U+6E12.0	fGB	5-5573
U+6E12.0	fR/S	  85.8
U+6E12.0	fTotalStrokes	 11
U+6E12.0	fCangjie	EHWJ
U+6E12.0	fFourCorner	36140
U+6E12.0	fCantonese	bei1
U+6E12.0	fMandarin	pi4
U+6E13.0	fUTF8	渓
U+6E13.0	fGB	E-304F
U+6E13.0	fR/S	 85.8
U+6E13.0	fTotalStrokes	 11
U+6E13.0	fCangjie	EBQO
U+6E13.0	fCantonese	kai1
U+6E13.0	fMandarin	xi1; qi1
U+6E13.0	fEnglish	 mountain stream, creek
U+6E14.0	fUTF8	渔
U+6E14.0	fGB	0-5366
U+6E14.0	fR/S	 85.8
U+6E14.0	fTotalStrokes	 11
U+6E14.0	fCangjie	ENWM
U+6E14.0	fHakka	 ng2
U+6E14.0	fCantonese	jyu4
U+6E14.0	fMandarin	yu2; yu4
U+6E14.0	fEnglish	simplified form of &#x6F01;
U+6E15.0	fUTF8	渕
U+6E15.0	fCNS11643	F-3629
U+6E15.0	fGB	E-3050
U+6E15.0	fR/S	 85.8
U+6E15.0	fTotalStrokes	 11
U+6E15.0	fCangjie	ETKN
U+6E15.0	fFourCorner	3210
U+6E15.0	fCantonese	jyun1
U+6E15.0	fMandarin	yuan1
U+6E15.0	fEnglish	surge up; bubble up; gush forth
U+6E16.0	fUTF8	渖
U+6E16.0	fGB	0-6449
U+6E16.0	fR/S	 85.8
U+6E16.0	fTotalStrokes	 11
U+6E16.0	fCangjie	EJLL
U+6E16.0	fHakka	 shim3
U+6E16.0	fCantonese	sam2
U+6E16.0	fMandarin	shen3; chen4
U+6E16.0	fEnglish	simplified form of &#x700B;
U+6E17.0	fUTF8	渗
U+6E17.0	fCNS11643	3-3727
U+6E17.0	fGB	0-4978
U+6E17.0	fR/S	 85.8
U+6E17.0	fTotalStrokes	 11
U+6E17.0	fCangjie	EIKH
U+6E17.0	fFourCorner	33120
U+6E17.0	fHakka	 sem1; cam1; sim5
U+6E17.0	fCantonese	sam3
U+6E17.0	fMandarin	shen4; lin2; qin1; qin4; sen1
U+6E17.0	fEnglish	simplified form of &#x6EF2;
U+6E18.0	fUTF8	渘
U+6E18.0	fCNS11643	3-3C54
U+6E18.0	fGB	3-5546
U+6E18.0	fR/S	 85.9
U+6E18.0	fTotalStrokes	 12
U+6E18.0	fCangjie	ENHD
U+6E18.0	fFourCorner	37190
U+6E18.0	fCantonese	jau4
U+6E18.0	fMandarin	rou2
U+6E19.0	fUTF8	渙
U+6E19.0	fCNS11643	1-6047
U+6E19.0	fBig5	B541
U+6E19.0	fGB	1-3B41
U+6E19.0	fR/S	 85.9
U+6E19.0	fTotalStrokes	 12
U+6E19.0	fCangjie	ENBK
U+6E19.0	fFourCorner	37134
U+6E19.0	fHakka	 fon5;  fan5; won3; fon3
U+6E19.0	fCantonese	wun6
U+6E19.0	fMandarin	huan4; hui4
U+6E19.0	fEnglish	[1] scatter; scattered; dispersed [2] the name of a river
U+6E1A.0	fUTF8	渚
U+6E1A.0	fCNS11643	1-5B5F
U+6E1A.0	fBig5	B25A
U+6E1A.0	fGB	0-643E
U+6E1A.0	fR/S	 85.8; 85.9
U+6E1A.0	fTotalStrokes	 11
U+6E1A.0	fCangjie	EJKA
U+6E1A.0	fFourCorner	34160
U+6E1A.0	fHakka	 du3
U+6E1A.0	fCantonese	zyu2
U+6E1A.0	fMandarin	zhu3
U+6E1A.0	fEnglish	 small sand bank in a river; islet; sand bar in a river
U+6E1B.0	fUTF8	減
U+6E1B.0	fCNS11643	1-6035
U+6E1B.0	fBig5	B4EE
U+6E1B.0	fGB	8-2D56
U+6E1B.0	fR/S	 85.9
U+6E1B.0	fTotalStrokes	 12
U+6E1B.0	fCangjie	EIHR
U+6E1B.0	fFourCorner	33150
U+6E1B.0	fHakka	  gam3
U+6E1B.0	fCantonese	gaam2
U+6E1B.0	fMandarin	jian3; jian4; xian4
U+6E1B.0	fEnglish	[1] decrease; subtract; diminish; reduce; deduct; cut [2] a Chinese family name
U+6E1C.0	fUTF8	渜
U+6E1C.0	fCNS11643	2-3D3B
U+6E1C.0	fBig5	D9F6
U+6E1C.0	fGB	3-547D
U+6E1C.0	fR/S	 85.9
U+6E1C.0	fTotalStrokes	 12
U+6E1C.0	fCangjie	EMBK
U+6E1C.0	fFourCorner	31134
U+6E1C.0	fHakka	  non3; ngion1
U+6E1C.0	fCantonese	nyun5; lyun5 (also); nyun6
U+6E1C.0	fMandarin	nuan3; nuan2; nuan4; ruan3
U+6E1C.0	fEnglish	warm water
U+6E1D.0	fUTF8	渝
U+6E1D.0	fCNS11643	1-6043
U+6E1D.0	fBig5	B4FC
U+6E1D.0	fGB	0-5365
U+6E1D.0	fR/S	 85.9
U+6E1D.0	fTotalStrokes	 12
U+6E1D.0	fCangjie	EOMN
U+6E1D.0	fFourCorner	38121
U+6E1D.0	fHakka	 ji2; ri2; i2
U+6E1D.0	fCantonese	jyu4
U+6E1D.0	fMandarin	yu2; dou4; yu4
U+6E1D.0	fEnglish	[1] change one's mind [2] another name of Chungking [3] another name of the Chialing river in Szechwan
U+6E1E.0	fUTF8	渞
U+6E1E.0	fCNS11643	3-3C52
U+6E1E.0	fGB	E-3051
U+6E1E.0	fR/S	 85.9
U+6E1E.0	fTotalStrokes	 12
U+6E1E.0	fCangjie	ETHU
U+6E1E.0	fFourCorner	38160
U+6E1E.0	fCantonese	cau4
U+6E1E.0	fMandarin	qiu2
U+6E1F.0	fUTF8	渟
U+6E1F.0	fCNS11643	2-3D2F
U+6E1F.0	fBig5	D9EA
U+6E1F.0	fGB	3-5534
U+6E1F.0	fR/S	 85.9
U+6E1F.0	fTotalStrokes	 12
U+6E1F.0	fCangjie	EYRN
U+6E1F.0	fFourCorner	30121
U+6E1F.0	fHakka	 tin2
U+6E1F.0	fCantonese	ting4
U+6E1F.0	fMandarin	ting2
U+6E1F.0	fEnglish	[1] not flowing (water) [2] clear (water)
U+6E20.0	fUTF8	渠
U+6E20.0	fCNS11643	1-6032
U+6E20.0	fBig5	B4EB
U+6E20.0	fGB	0-477E
U+6E20.0	fR/S	 85.9
U+6E20.0	fTotalStrokes	 12
U+6E20.0	fCangjie	ESD
U+6E20.0	fFourCorner	31904
U+6E20.0	fHakka	  ki2; gi2
U+6E20.0	fCantonese	keoi4; keoi5; heoi5 (also)
U+6E20.0	fMandarin	qu2; ju4
U+6E20.0	fEnglish	[1] ditch; drain; canal; channel [2] great; deep [3] he; she [4] a Chinese family name
U+6E21.0	fUTF8	渡
U+6E21.0	fCNS11643	1-602E
U+6E21.0	fBig5	B4E7
U+6E21.0	fGB	0-3649
U+6E21.0	fR/S	 85.9
U+6E21.0	fTotalStrokes	 12
U+6E21.0	fCangjie	EITE
U+6E21.0	fFourCorner	30147
U+6E21.0	fHakka	  tu5; tu6
U+6E21.0	fCantonese	dou6; dou2 (also)
U+6E21.0	fMandarin	du4
U+6E21.0	fEnglish	[1] cross (river, ocean); ferry over [2] ferry
U+6E22.0	fUTF8	渢
U+6E22.0	fCNS11643	2-3D4D
U+6E22.0	fBig5	DA49
U+6E22.0	fGB	3-5354
U+6E22.0	fR/S	 85.9
U+6E22.0	fTotalStrokes	 12
U+6E22.0	fCangjie	EHNI
U+6E22.0	fFourCorner	37110
U+6E22.0	fHakka	  fung1;  fam2
U+6E22.0	fCantonese	fung1; fung4 (also)
U+6E22.0	fMandarin	feng2; fan2; feng1; fan4
U+6E22.0	fEnglish	pleasant sound
U+6E23.0	fUTF8	渣
U+6E23.0	fCNS11643	1-6034
U+6E23.0	fBig5	B4ED
U+6E23.0	fGB	0-547C
U+6E23.0	fR/S	 85.9
U+6E23.0	fTotalStrokes	 12
U+6E23.0	fCangjie	EDAM
U+6E23.0	fFourCorner	34116
U+6E23.0	fHakka	  za1
U+6E23.0	fCantonese	zaa1; zaa2
U+6E23.0	fMandarin	zha1; zha3; zha4
U+6E23.0	fEnglish	dregs; lees; sediment; grounds
U+6E24.0	fUTF8	渤
U+6E24.0	fCNS11643	1-6038
U+6E24.0	fBig5	B4F1
U+6E24.0	fGB	0-3233
U+6E24.0	fR/S	 85.9
U+6E24.0	fTotalStrokes	 12
U+6E24.0	fCangjie	EJDS
U+6E24.0	fFourCorner	34127
U+6E24.0	fHakka	 put8
U+6E24.0	fCantonese	but6
U+6E24.0	fMandarin	bo2
U+6E24.0	fEnglish	swelling or rising (water)
U+6E25.0	fUTF8	渥
U+6E25.0	fCNS11643	1-6033
U+6E25.0	fBig5	B4EC
U+6E25.0	fGB	0-6457
U+6E25.0	fR/S	 85.9
U+6E25.0	fTotalStrokes	 12
U+6E25.0	fCangjie	ESMG
U+6E25.0	fFourCorner	37114
U+6E25.0	fHakka	  wok7; vok7
U+6E25.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+6E25.0	fMandarin	wo4; ou4; wu1
U+6E25.0	fEnglish	[1] great (kindness) [2] dye
U+6E26.0	fUTF8	渦
U+6E26.0	fCNS11643	1-603C
U+6E26.0	fBig5	B4F5
U+6E26.0	fGB	1-4E50
U+6E26.0	fR/S	 85.9
U+6E26.0	fTotalStrokes	 12
U+6E26.0	fCangjie	EBBR
U+6E26.0	fFourCorner	37127
U+6E26.0	fHakka	 wo1; vo1
U+6E26.0	fCantonese	wo1; gwo1; go1 (also)
U+6E26.0	fMandarin	wo1; gua1
U+6E26.0	fEnglish	swirl; whirlpool; eddy
U+6E26.1	fUTF8	渦
U+6E26.1	fCNS11643	1-603C
U+6E26.1	fBig5	B4F5
U+6E26.1	fGB	1-4E50
U+6E26.1	fR/S	 85.9
U+6E26.1	fTotalStrokes	 12
U+6E26.1	fCangjie	EBBR
U+6E26.1	fFourCorner	37127
U+6E26.1	fMandarin	guo1
U+6E26.1	fEnglish	the name of a river
U+6E27.0	fUTF8	渧
U+6E27.0	fCNS11643	2-3D51
U+6E27.0	fBig5	DA4D
U+6E27.0	fGB	3-5539
U+6E27.0	fR/S	 85.9
U+6E27.0	fTotalStrokes	 12
U+6E27.0	fCangjie	EYBB
U+6E27.0	fFourCorner	30127
U+6E27.0	fHakka	  di5
U+6E27.0	fCantonese	dai3; dik6
U+6E27.0	fMandarin	di4; di1; ti2
U+6E27.0	fEnglish	 drop (liquids) drop
U+6E28.0	fUTF8	渨
U+6E28.0	fCNS11643	2-3D48
U+6E28.0	fBig5	DA44
U+6E28.0	fGB	3-5529
U+6E28.0	fR/S	 85.9
U+6E28.0	fTotalStrokes	 12
U+6E28.0	fCangjie	EWMV
U+6E28.0	fFourCorner	36132
U+6E28.0	fHakka	 woi1; wui3; wi3; voi1; voe1
U+6E28.0	fCantonese	wui1; wai2
U+6E28.0	fMandarin	wei1; wei3
U+6E28.0	fEnglish	a bend on a river
U+6E29.0	fUTF8	温
U+6E29.0	fCNS11643	3-3C6B
U+6E29.0	fGB	0-4E42
U+6E29.0	fR/S	 85.9
U+6E29.0	fTotalStrokes	 12
U+6E29.0	fCangjie	EABT
U+6E29.0	fFourCorner	36110
U+6E29.0	fHakka	  wun1; vun1
U+6E29.0	fCantonese	wan1
U+6E29.0	fMandarin	wen1; yun4
U+6E29.0	fEnglish	lukewarm
U+6E2A.0	fUTF8	渪
U+6E2A.0	fCNS11643	3-3C6D
U+6E2A.0	fGB	5-5631
U+6E2A.0	fR/S	  85.9
U+6E2A.0	fTotalStrokes	 12
U+6E2A.0	fCangjie	EHLB
U+6E2A.0	fFourCorner	32120
U+6E2A.0	fCantonese	jyu4
U+6E2A.0	fMandarin	ru2
U+6E2B.0	fUTF8	渫
U+6E2B.0	fCNS11643	2-3D36
U+6E2B.0	fBig5	D9F1
U+6E2B.0	fGB	0-644D
U+6E2B.0	fR/S	 85.9
U+6E2B.0	fTotalStrokes	 12
U+6E2B.0	fCangjie	EPTD
U+6E2B.0	fFourCorner	34194
U+6E2B.0	fHakka	  tiap8; siet7; set7
U+6E2B.0	fCantonese	sit3; dip6
U+6E2B.0	fMandarin	xie4; die2; yi4; zha2; qie4
U+6E2B.0	fEnglish	 remove; eliminate
U+6E2C.0	fUTF8	測
U+6E2C.0	fCNS11643	1-6041
U+6E2C.0	fBig5	B4FA
U+6E2C.0	fGB	1-3262
U+6E2C.0	fR/S	 85.9
U+6E2C.0	fTotalStrokes	 12
U+6E2C.0	fCangjie	EBCN
U+6E2C.0	fFourCorner	32100
U+6E2C.0	fHakka	 cet7; ts'ek7
U+6E2C.0	fCantonese	cak1; caak1 (also)
U+6E2C.0	fMandarin	ce4
U+6E2C.0	fEnglish	 measure; survey
U+6E2D.0	fUTF8	渭
U+6E2D.0	fCNS11643	1-603B
U+6E2D.0	fBig5	B4F4
U+6E2D.0	fGB	0-4E3C
U+6E2D.0	fR/S	 85.9
U+6E2D.0	fTotalStrokes	 12
U+6E2D.0	fCangjie	EWB
U+6E2D.0	fFourCorner	36127
U+6E2D.0	fHakka	 wi5; wui6; wui3; vui5; wui5
U+6E2D.0	fCantonese	wai6
U+6E2D.0	fMandarin	wei4
U+6E2D.0	fEnglish	a river in Shaanxi
U+6E2E.0	fUTF8	渮
U+6E2E.0	fCNS11643	2-3D42
U+6E2E.0	fBig5	D9FD
U+6E2E.0	fGB	5-5569
U+6E2E.0	fR/S	 85.9
U+6E2E.0	fTotalStrokes	 12
U+6E2E.0	fCangjie	ETMR
U+6E2E.0	fFourCorner	34121
U+6E2E.0	fHakka	  ho2; go1
U+6E2E.0	fCantonese	ho4; go1
U+6E2E.0	fMandarin	ge1; ge3; he2
U+6E2E.0	fEnglish	a river in Shandong province
U+6E2F.0	fUTF8	港
U+6E2F.0	fCNS11643	1-602B
U+6E2F.0	fBig5	B4E4
U+6E2F.0	fGB	0-385B
U+6E2F.0	fR/S	 85.9
U+6E2F.0	fTotalStrokes	 12
U+6E2F.0	fCangjie	ETCU
U+6E2F.0	fFourCorner	34117
U+6E2F.0	fHakka	  gong3
U+6E2F.0	fCantonese	gong2
U+6E2F.0	fMandarin	gang3; hong1; hong4; xiang4
U+6E2F.0	fEnglish	[1] seaport; harbour [2] bay; gulf [3] short for Hongkong
U+6E30.0	fUTF8	渰
U+6E30.0	fCNS11643	2-3D4E
U+6E30.0	fBig5	DA4A
U+6E30.0	fGB	3-552F
U+6E30.0	fR/S	 85.9
U+6E30.0	fTotalStrokes	 12
U+6E30.0	fCangjie	EORT
U+6E30.0	fFourCorner	38146
U+6E30.0	fHakka	 jam1; ram1; jam3
U+6E30.0	fCantonese	jim2; jim1
U+6E30.0	fMandarin	yan3; yan1
U+6E30.0	fEnglish	forming or rising (clouds)
U+6E31.0	fUTF8	渱
U+6E31.0	fCNS11643	2-3D47
U+6E31.0	fBig5	DA43
U+6E31.0	fGB	3-552A
U+6E31.0	fR/S	 85.9
U+6E31.0	fTotalStrokes	 12
U+6E31.0	fCangjie	ELIM
U+6E31.0	fFourCorner	31110
U+6E31.0	fCantonese	hung4; gung3
U+6E31.0	fMandarin	hong2; gong4
U+6E32.0	fUTF8	渲
U+6E32.0	fCNS11643	1-602F
U+6E32.0	fBig5	B4E8
U+6E32.0	fGB	0-6456
U+6E32.0	fR/S	 85.9
U+6E32.0	fTotalStrokes	 12
U+6E32.0	fCangjie	EJMM
U+6E32.0	fFourCorner	33116
U+6E32.0	fHakka	 sien1; sen3
U+6E32.0	fCantonese	hyun1; syun3 (also); syun1 (also)
U+6E32.0	fMandarin	xuan4; xuan1
U+6E32.0	fEnglish	 color with paint
U+6E33.0	fUTF8	渳
U+6E33.0	fCNS11643	2-3D3C
U+6E33.0	fBig5	D9F7
U+6E33.0	fGB	3-5543
U+6E33.0	fR/S	 85.9
U+6E33.0	fTotalStrokes	 12
U+6E33.0	fCangjie	ENSJ
U+6E33.0	fFourCorner	31140
U+6E33.0	fCantonese	mai5
U+6E33.0	fMandarin	mi3; min3; er3
U+6E34.0	fUTF8	渴
U+6E34.0	fCNS11643	1-603E
U+6E34.0	fBig5	B4F7
U+6E34.0	fGB	0-3F4A
U+6E34.0	fR/S	 85.9
U+6E34.0	fTotalStrokes	 12
U+6E34.0	fCangjie	EAPV
U+6E34.0	fFourCorner	36127
U+6E34.0	fHakka	  hot7; k'ot7; hat8
U+6E34.0	fCantonese	hot3; kit3
U+6E34.0	fMandarin	ke3; jie2; he2; kai4
U+6E34.0	fEnglish	[1] thirsty [2] long; crave; pine
U+6E35.0	fUTF8	渵
U+6E35.0	fCNS11643	2-3D59
U+6E35.0	fBig5	DA55
U+6E35.0	fGB	E-3052
U+6E35.0	fR/S	 85.9
U+6E35.0	fTotalStrokes	 12
U+6E35.0	fCangjie	ETW
U+6E35.0	fFourCorner	34160
U+6E35.0	fCantonese	maau4
U+6E35.0	fMandarin	mao2; mao1
U+6E36.0	fUTF8	渶
U+6E36.0	fCNS11643	2-3D5A
U+6E36.0	fBig5	DA56
U+6E36.0	fGB	5-556A
U+6E36.0	fR/S	 85.9; 85.8
U+6E36.0	fTotalStrokes	 12
U+6E36.0	fCangjie	ETLK
U+6E36.0	fFourCorner	34135
U+6E36.0	fCantonese	jing1
U+6E36.0	fMandarin	ying1; qi4
U+6E37.0	fUTF8	渷
U+6E37.0	fCNS11643	4-385B
U+6E37.0	fGB	E-3053
U+6E37.0	fR/S	  85.9
U+6E37.0	fTotalStrokes	 12
U+6E37.0	fCangjie	EYCU
U+6E37.0	fFourCorner	30110
U+6E37.0	fCantonese	jin5
U+6E37.0	fMandarin	yan3
U+6E38.0	fUTF8	游
U+6E38.0	fCNS11643	1-602C
U+6E38.0	fBig5	B4E5
U+6E38.0	fGB	0-534E
U+6E38.0	fR/S	 85.9
U+6E38.0	fTotalStrokes	 12
U+6E38.0	fCangjie	EYSD
U+6E38.0	fFourCorner	38147
U+6E38.0	fHakka	  ju2; jiu2; riu2; jiu3
U+6E38.0	fCantonese	jau4; jau6
U+6E38.0	fMandarin	you2; liu2; qiu2
U+6E38.0	fEnglish	[1] swim; float; drift; waft [2] same as $ [3] part of a river [4] a Chinese family name
U+6E39.0	fUTF8	渹
U+6E39.0	fCNS11643	2-3D4C
U+6E39.0	fBig5	DA48
U+6E39.0	fGB	3-5531
U+6E39.0	fR/S	 85.9
U+6E39.0	fTotalStrokes	 12
U+6E39.0	fCangjie	EPYR
U+6E39.0	fFourCorner	37120
U+6E39.0	fHakka	 wang2; vang1; wang1
U+6E39.0	fCantonese	gwang1
U+6E39.0	fMandarin	hong1; qing4
U+6E39.0	fEnglish	the roar of water; the sound of a cataract
U+6E3A.0	fUTF8	渺
U+6E3A.0	fCNS11643	1-6040
U+6E3A.0	fBig5	B4F9
U+6E3A.0	fGB	0-436C
U+6E3A.0	fR/S	 85.9
U+6E3A.0	fTotalStrokes	 12
U+6E3A.0	fCangjie	EBUH
U+6E3A.0	fFourCorner	39120
U+6E3A.0	fHakka	 miau3; miau1; meu3
U+6E3A.0	fCantonese	miu5
U+6E3A.0	fMandarin	miao3
U+6E3A.0	fEnglish	[1] endlessly long or vast; boundless; far [2] tiny; infinitesimal [3] indistinct; blurred
U+6E3B.0	fUTF8	渻
U+6E3B.0	fCNS11643	2-3D40
U+6E3B.0	fBig5	D9FB
U+6E3B.0	fGB	5-562E
U+6E3B.0	fR/S	 85.9
U+6E3B.0	fTotalStrokes	 12
U+6E3B.0	fCangjie	EFHU
U+6E3B.0	fFourCorner	39162
U+6E3B.0	fHakka	 sen3
U+6E3B.0	fCantonese	saang2; sing2 (also); sang2
U+6E3B.0	fMandarin	xing3; sheng3; sheng4
U+6E3B.0	fEnglish	rare; very little of it
U+6E3C.0	fUTF8	渼
U+6E3C.0	fCNS11643	2-3D32
U+6E3C.0	fBig5	D9ED
U+6E3C.0	fGB	3-553D
U+6E3C.0	fR/S	 85.9
U+6E3C.0	fTotalStrokes	 12
U+6E3C.0	fCangjie	ETGK
U+6E3C.0	fFourCorner	38135
U+6E3C.0	fCantonese	mei5
U+6E3C.0	fMandarin	mei3; bian4
U+6E3C.0	fEnglish	ripples
U+6E3D.0	fUTF8	渽
U+6E3D.0	fCNS11643	2-3D33
U+6E3D.0	fBig5	D9EE
U+6E3D.0	fGB	3-5477
U+6E3D.0	fR/S	 85.9
U+6E3D.0	fTotalStrokes	 12
U+6E3D.0	fCangjie	EJIR
U+6E3D.0	fFourCorner	33150
U+6E3D.0	fCantonese	zoi1
U+6E3D.0	fMandarin	zai1
U+6E3E.0	fUTF8	渾
U+6E3E.0	fCNS11643	1-6044
U+6E3E.0	fBig5	B4FD
U+6E3E.0	fGB	1-3B6B
U+6E3E.0	fR/S	 85.9
U+6E3E.0	fTotalStrokes	 12
U+6E3E.0	fCangjie	EBJJ
U+6E3E.0	fFourCorner	37156
U+6E3E.0	fHakka	  fun2; wun2; won2; fun3
U+6E3E.0	fCantonese	wan4; wan6; gwan2
U+6E3E.0	fMandarin	hun2; gun3
U+6E3E.0	fEnglish	[1] entire; complete [2] muddy; turbid [3] blend; merge; mix
U+6E3E.1	fUTF8	渾
U+6E3E.1	fCNS11643	1-6044
U+6E3E.1	fBig5	B4FD
U+6E3E.1	fGB	1-3B6B
U+6E3E.1	fR/S	 85.9
U+6E3E.1	fTotalStrokes	 12
U+6E3E.1	fCangjie	EBJJ
U+6E3E.1	fFourCorner	37156
U+6E3E.1	fMandarin	hun4
U+6E3E.1	fEnglish	chaotic; confused; messy
U+6E3F.0	fUTF8	渿
U+6E3F.0	fCNS11643	2-3D37
U+6E3F.0	fBig5	D9F2
U+6E3F.0	fGB	5-5629
U+6E3F.0	fR/S	 85.9
U+6E3F.0	fTotalStrokes	 12
U+6E3F.0	fCangjie	EDMF
U+6E3F.0	fFourCorner	34191
U+6E3F.0	fCantonese	noi6; loi6 (also)
U+6E3F.0	fMandarin	nai4
U+6E40.0	fUTF8	湀
U+6E40.0	fCNS11643	2-3D3E
U+6E40.0	fBig5	D9F9
U+6E40.0	fGB	5-563B
U+6E40.0	fR/S	 85.9
U+6E40.0	fTotalStrokes	 12
U+6E40.0	fCangjie	ENOK
U+6E40.0	fFourCorner	32134
U+6E40.0	fHakka	 kui2
U+6E40.0	fCantonese	gwai2; kwai1; kwai5
U+6E40.0	fMandarin	kui2; gui3; que4
U+6E41.0	fUTF8	湁
U+6E41.0	fCNS11643	2-3D38
U+6E41.0	fBig5	D9F3
U+6E41.0	fGB	3-547E
U+6E41.0	fR/S	 85.9
U+6E41.0	fTotalStrokes	 12
U+6E41.0	fCangjie	EQOR
U+6E41.0	fFourCorner	38161
U+6E41.0	fCantonese	cap1
U+6E41.0	fMandarin	shi2
U+6E42.0	fUTF8	湂
U+6E42.0	fCNS11643	3-3C5A
U+6E42.0	fGB	E-3054
U+6E42.0	fR/S	  85.9
U+6E42.0	fTotalStrokes	 12
U+6E42.0	fCangjie	ERRS
U+6E42.0	fFourCorner	36120
U+6E42.0	fCantonese	ngok6; ok6 (also)
U+6E42.0	fMandarin	e4
U+6E43.0	fUTF8	湃
U+6E43.0	fCNS11643	1-6042
U+6E43.0	fBig5	B4FB
U+6E43.0	fGB	0-4548
U+6E43.0	fR/S	 85.9
U+6E43.0	fTotalStrokes	 12
U+6E43.0	fCangjie	EHQJ
U+6E43.0	fFourCorner	31150
U+6E43.0	fHakka	 pai5; bai5; bai3
U+6E43.0	fCantonese	baai3; paai3 (also)
U+6E43.0	fMandarin	pai4; ba2
U+6E43.0	fEnglish	turbulent; billowy
U+6E44.0	fUTF8	湄
U+6E44.0	fCNS11643	1-604A
U+6E44.0	fBig5	B544
U+6E44.0	fGB	0-6458
U+6E44.0	fR/S	 85.9
U+6E44.0	fTotalStrokes	 12
U+6E44.0	fCangjie	EAHU
U+6E44.0	fFourCorner	37167
U+6E44.0	fHakka	 mi2
U+6E44.0	fCantonese	mei4
U+6E44.0	fMandarin	mei2; nuan3
U+6E44.0	fEnglish	 water's edge; shore; bank; margin of the water
U+6E45.0	fUTF8	湅
U+6E45.0	fCNS11643	2-3D34
U+6E45.0	fBig5	D9EF
U+6E45.0	fGB	3-547C
U+6E45.0	fR/S	 85.9
U+6E45.0	fTotalStrokes	 12
U+6E45.0	fCangjie	EDWF
U+6E45.0	fFourCorner	35196
U+6E45.0	fHakka	 lien5; lien6; len3
U+6E45.0	fCantonese	lin6
U+6E45.0	fMandarin	lian4; feng1
U+6E45.0	fEnglish	boil silk
U+6E46.0	fUTF8	湆
U+6E46.0	fCNS11643	2-3D2D
U+6E46.0	fBig5	D9E8
U+6E46.0	fGB	3-5538
U+6E46.0	fR/S	 85.9
U+6E46.0	fTotalStrokes	 12
U+6E46.0	fCangjie	EYTA
U+6E46.0	fFourCorner	30161
U+6E46.0	fHakka	 kip7; hip7
U+6E46.0	fCantonese	jap1; nap6
U+6E46.0	fMandarin	qi4
U+6E46.0	fEnglish	[1] damp [2] juicy [3] dark
U+6E47.0	fUTF8	湇
U+6E47.0	fCNS11643	2-3D2E
U+6E47.0	fBig5	D9E9
U+6E47.0	fGB	5-5635
U+6E47.0	fR/S	 85.9
U+6E47.0	fTotalStrokes	 12
U+6E47.0	fCangjie	EYTB
U+6E47.0	fFourCorner	30127
U+6E47.0	fCantonese	jap1
U+6E47.0	fMandarin	qi4; qia4; qie4
U+6E48.0	fUTF8	湈
U+6E48.0	fCNS11643	3-3C56
U+6E48.0	fGB	E-3055
U+6E48.0	fR/S	  85.9
U+6E48.0	fTotalStrokes	 12
U+6E48.0	fCangjie	ETMD
U+6E48.0	fFourCorner	34190
U+6E48.0	fCantonese	mui4
U+6E48.0	fMandarin	mei2
U+6E49.0	fUTF8	湉
U+6E49.0	fCNS11643	2-3D30
U+6E49.0	fBig5	D9EB
U+6E49.0	fGB	3-5540
U+6E49.0	fR/S	 85.9
U+6E49.0	fTotalStrokes	 12
U+6E49.0	fCangjie	EPHR
U+6E49.0	fFourCorner	32164
U+6E49.0	fHakka	  tiam2; tin2
U+6E49.0	fCantonese	tim4
U+6E49.0	fMandarin	tian2
U+6E4A.0	fUTF8	湊
U+6E4A.0	fCNS11643	1-6031
U+6E4A.0	fBig5	B4EA
U+6E4A.0	fGB	E-3056
U+6E4A.0	fR/S	 85.9
U+6E4A.0	fTotalStrokes	 12
U+6E4A.0	fCangjie	EQKK
U+6E4A.0	fFourCorner	35134
U+6E4A.0	fHakka	 ceu5; deu5; ciu5
U+6E4A.0	fCantonese	cau3
U+6E4A.0	fMandarin	cou4
U+6E4A.0	fEnglish	[1] piece together; put together; assemble [2] raise (fund) [3] happen by chance [4] move close to; press near
U+6E4B.0	fUTF8	湋
U+6E4B.0	fCNS11643	2-3D3D
U+6E4B.0	fBig5	D9F8
U+6E4B.0	fGB	3-533C
U+6E4B.0	fR/S	 85.9
U+6E4B.0	fTotalStrokes	 12
U+6E4B.0	fCangjie	EDMQ
U+6E4B.0	fFourCorner	34156
U+6E4B.0	fHakka	 wui2; wi2; vui2
U+6E4B.0	fCantonese	wai4; wai5 (also)
U+6E4B.0	fMandarin	wei2
U+6E4B.0	fEnglish	a river in Shanxi province
U+6E4C.0	fUTF8	湌
U+6E4C.0	fCNS11643	3-3C6E
U+6E4C.0	fGB	5-5633
U+6E4C.0	fR/S	  85.9
U+6E4C.0	fTotalStrokes	 12
U+6E4C.0	fCangjie	EOIV
U+6E4C.0	fFourCorner	38130
U+6E4C.0	fCantonese	caan1; can1
U+6E4C.0	fMandarin	can1
U+6E4D.0	fUTF8	湍
U+6E4D.0	fCNS11643	1-603F
U+6E4D.0	fBig5	B4F8
U+6E4D.0	fGB	0-4D44
U+6E4D.0	fR/S	 85.9
U+6E4D.0	fTotalStrokes	 12
U+6E4D.0	fCangjie	EUMB
U+6E4D.0	fFourCorner	32127
U+6E4D.0	fHakka	  ton2; zon1; zhon1; tam1
U+6E4D.0	fCantonese	teon1; cyun2 (also); zyun1 (also); tyun1; zyun2
U+6E4D.0	fMandarin	tuan1; zhuan1; tuan4
U+6E4D.0	fEnglish	rapid water current; rapid flowing
U+6E4E.0	fUTF8	湎
U+6E4E.0	fCNS11643	1-6048
U+6E4E.0	fBig5	B542
U+6E4E.0	fGB	0-644F
U+6E4E.0	fR/S	 85.9
U+6E4E.0	fTotalStrokes	 12
U+6E4E.0	fCangjie	EMWL
U+6E4E.0	fFourCorner	31160
U+6E4E.0	fHakka	  mien3; mien6; mien5
U+6E4E.0	fCantonese	min5
U+6E4E.0	fMandarin	mian3
U+6E4E.0	fEnglish	[1] flushed with drink; drunk [2] unaware [3] changing
U+6E4F.0	fUTF8	湏
U+6E4F.0	fCNS11643	3-3C58
U+6E4F.0	fGB	E-3057
U+6E4F.0	fR/S	  85.9
U+6E4F.0	fTotalStrokes	 12
U+6E4F.0	fCangjie	EMBC
U+6E4F.0	fFourCorner	31180
U+6E4F.0	fHakka	 moi5;  si1; fui3; fi3; moe5
U+6E4F.0	fCantonese	mui6
U+6E4F.0	fMandarin	hui4; min3
U+6E4F.0	fEnglish	 wash face erroneously borrowed for "necessary; must"
U+6E50.0	fUTF8	湐
U+6E50.0	fCNS11643	3-3C69
U+6E50.0	fGB	5-562A
U+6E50.0	fR/S	  85.9
U+6E50.0	fTotalStrokes	 12
U+6E50.0	fCangjie	EDHA
U+6E50.0	fFourCorner	36100
U+6E50.0	fCantonese	paak3
U+6E50.0	fMandarin	mo4
U+6E51.0	fUTF8	湑
U+6E51.0	fCNS11643	2-3D3F
U+6E51.0	fBig5	D9FA
U+6E51.0	fGB	3-5545
U+6E51.0	fR/S	 85.9
U+6E51.0	fTotalStrokes	 12
U+6E51.0	fCangjie	ENOB
U+6E51.0	fFourCorner	37127
U+6E51.0	fHakka	  si1
U+6E51.0	fCantonese	seoi2; seoi1 (also)
U+6E51.0	fMandarin	xu3; xu1
U+6E51.0	fEnglish	luxuriant; rich
U+6E52.0	fUTF8	湒
U+6E52.0	fCNS11643	2-3D57
U+6E52.0	fBig5	DA53
U+6E52.0	fGB	3-5525
U+6E52.0	fR/S	 85.9
U+6E52.0	fTotalStrokes	 12
U+6E52.0	fCangjie	ERSJ
U+6E52.0	fFourCorner	36141
U+6E52.0	fCantonese	cap1; zap1
U+6E52.0	fMandarin	ji2; qi4
U+6E53.0	fUTF8	湓
U+6E53.0	fCNS11643	2-3D4F
U+6E53.0	fBig5	DA4B
U+6E53.0	fGB	0-6454
U+6E53.0	fR/S	 85.9
U+6E53.0	fTotalStrokes	 12
U+6E53.0	fCangjie	ECST
U+6E53.0	fFourCorner	38117
U+6E53.0	fHakka	 pun2; pun1; put8
U+6E53.0	fCantonese	pun4; pun6
U+6E53.0	fMandarin	pen2; pen4; ben4; fen4
U+6E53.0	fEnglish	an affluent of the river Yangtze near Kiukiang
U+6E54.0	fUTF8	湔
U+6E54.0	fCNS11643	1-602D
U+6E54.0	fBig5	B4E6
U+6E54.0	fGB	0-6455
U+6E54.0	fR/S	 85.9
U+6E54.0	fTotalStrokes	 12
U+6E54.0	fCangjie	ETBN
U+6E54.0	fFourCorner	38121
U+6E54.0	fHakka	  zien1; zen1
U+6E54.0	fCantonese	zin1; zin3
U+6E54.0	fMandarin	jian1; jian4; zan4; qian2; zhan3
U+6E54.0	fEnglish	 wash; cleanse; purge
U+6E55.0	fUTF8	湕
U+6E55.0	fCNS11643	2-3D55
U+6E55.0	fBig5	DA51
U+6E55.0	fGB	5-557E
U+6E55.0	fR/S	 85.9
U+6E55.0	fTotalStrokes	 12
U+6E55.0	fCangjie	ENKQ
U+6E55.0	fFourCorner	35144
U+6E55.0	fCantonese	gin2
U+6E55.0	fMandarin	jian3
U+6E56.0	fUTF8	湖
U+6E56.0	fCNS11643	1-6039
U+6E56.0	fBig5	B4F2
U+6E56.0	fGB	0-3A7E
U+6E56.0	fR/S	 85.9
U+6E56.0	fTotalStrokes	 12
U+6E56.0	fCangjie	EJRB
U+6E56.0	fFourCorner	37120
U+6E56.0	fHakka	  fu2; fu3
U+6E56.0	fCantonese	wu4
U+6E56.0	fMandarin	hu2
U+6E56.0	fEnglish	[1] lake [2] a Chinese family name
U+6E57.0	fUTF8	湗
U+6E57.0	fCNS11643	3-3C66
U+6E57.0	fGB	5-5623
U+6E57.0	fR/S	 85.9
U+6E57.0	fTotalStrokes	 12
U+6E57.0	fCangjie	EGGI
U+6E57.0	fFourCorner	34100
U+6E57.0	fCantonese	fung6
U+6E57.0	fMandarin	feng4
U+6E58.0	fUTF8	湘
U+6E58.0	fCNS11643	1-6037
U+6E58.0	fBig5	B4F0
U+6E58.0	fGB	0-4F66
U+6E58.0	fR/S	 85.9
U+6E58.0	fTotalStrokes	 12
U+6E58.0	fCangjie	EDBU
U+6E58.0	fFourCorner	36100
U+6E58.0	fHakka	  siong1
U+6E58.0	fCantonese	soeng1
U+6E58.0	fMandarin	xiang1; shuang1
U+6E58.0	fEnglish	[1] a river flowing through Honan province [2] another name for Hunan province
U+6E59.0	fUTF8	湙
U+6E59.0	fCNS11643	3-3C64
U+6E59.0	fGB	3-5536
U+6E59.0	fR/S	  85.9
U+6E59.0	fTotalStrokes	 12
U+6E59.0	fCangjie	EYCK
U+6E59.0	fFourCorner	30130
U+6E59.0	fCantonese	jik6
U+6E59.0	fMandarin	yi4
U+6E5A.0	fUTF8	湚
U+6E5A.0	fCNS11643	2-3D5B
U+6E5A.0	fBig5	DA57
U+6E5A.0	fGB	E-3058
U+6E5A.0	fR/S	 85.9
U+6E5A.0	fTotalStrokes	 12
U+6E5A.0	fCangjie	ELVU
U+6E5A.0	fFourCorner	32110
U+6E5A.0	fCantonese	jan6
U+6E5A.0	fMandarin	yin4; yin3; zhen4
U+6E5B.0	fUTF8	湛
U+6E5B.0	fCNS11643	1-6036
U+6E5B.0	fBig5	B4EF
U+6E5B.0	fGB	0-553F
U+6E5B.0	fR/S	 85.9
U+6E5B.0	fTotalStrokes	 12
U+6E5B.0	fCangjie	ETMV
U+6E5B.0	fFourCorner	34111
U+6E5B.0	fHakka	 zam5;  tam3;  cam3; dam1
U+6E5B.0	fCantonese	zaam3; daam1; cam4; taam4
U+6E5B.0	fMandarin	zhan4; chen2; dan1; jian1; jin4; tan2; yin2; zhen4
U+6E5B.0	fEnglish	[1] dewy [2] deep; profound [3] same as $ [4] a Chinese family name
U+6E5C.0	fUTF8	湜
U+6E5C.0	fCNS11643	2-3D45
U+6E5C.0	fBig5	DA41
U+6E5C.0	fGB	3-5524
U+6E5C.0	fR/S	 85.9
U+6E5C.0	fTotalStrokes	 12
U+6E5C.0	fCangjie	EAMO
U+6E5C.0	fFourCorner	36181
U+6E5C.0	fHakka	 chit8; ts'iit8
U+6E5C.0	fCantonese	zik6; sat6 (also)
U+6E5C.0	fMandarin	shi2
U+6E5C.0	fEnglish	transparent; clear (water)
U+6E5D.0	fUTF8	湝
U+6E5D.0	fCNS11643	2-3D39
U+6E5D.0	fBig5	D9F4
U+6E5D.0	fGB	3-5521
U+6E5D.0	fR/S	 85.9
U+6E5D.0	fTotalStrokes	 12
U+6E5D.0	fCangjie	EPPA
U+6E5D.0	fFourCorner	31161
U+6E5D.0	fHakka	 gai1; kie1
U+6E5D.0	fCantonese	gaai1
U+6E5D.0	fMandarin	jie1; xie2
U+6E5D.0	fEnglish	flow
U+6E5E.0	fUTF8	湞
U+6E5E.0	fCNS11643	2-3D43
U+6E5E.0	fBig5	D9FE
U+6E5E.0	fGB	1-6425
U+6E5E.0	fR/S	 85.9
U+6E5E.0	fTotalStrokes	 12
U+6E5E.0	fCangjie	EYBC
U+6E5E.0	fFourCorner	31186
U+6E5E.0	fHakka	 zhin1; zihn1; zin1
U+6E5E.0	fCantonese	zing1
U+6E5E.0	fMandarin	zhen1; cheng2; cheng1
U+6E5E.0	fEnglish	a river in Guangdong province
U+6E5F.0	fUTF8	湟
U+6E5F.0	fCNS11643	1-604D
U+6E5F.0	fBig5	B547
U+6E5F.0	fGB	0-6452
U+6E5F.0	fR/S	 85.9
U+6E5F.0	fTotalStrokes	 12
U+6E5F.0	fCangjie	EHAG
U+6E5F.0	fFourCorner	36114
U+6E5F.0	fHakka	  fong2
U+6E5F.0	fCantonese	wong4
U+6E5F.0	fMandarin	huang2; kuang4
U+6E5F.0	fEnglish	[1] a mean, dirty place [2] a river in Qinghai province
U+6E60.0	fUTF8	湠
U+6E60.0	fCNS11643	2-3D49
U+6E60.0	fBig5	DA45
U+6E60.0	fGB	3-552B
U+6E60.0	fR/S	 85.9
U+6E60.0	fTotalStrokes	 12
U+6E60.0	fCangjie	EUMF
U+6E60.0	fFourCorner	32189
U+6E60.0	fCantonese	taan3
U+6E60.0	fMandarin	tan4
U+6E61.0	fUTF8	湡
U+6E61.0	fCNS11643	2-3D46
U+6E61.0	fBig5	DA42
U+6E61.0	fGB	3-5527
U+6E61.0	fR/S	 85.9
U+6E61.0	fTotalStrokes	 12
U+6E61.0	fCangjie	EWLB
U+6E61.0	fFourCorner	36127
U+6E61.0	fCantonese	jyu4
U+6E61.0	fMandarin	yu2; ou3; weng3
U+6E62.0	fUTF8	湢
U+6E62.0	fCNS11643	2-3D35
U+6E62.0	fBig5	D9F0
U+6E62.0	fGB	3-547B
U+6E62.0	fR/S	 85.9
U+6E62.0	fTotalStrokes	 12
U+6E62.0	fCangjie	EMRW
U+6E62.0	fFourCorner	31166
U+6E62.0	fHakka	  bit7;  bit8; bet7
U+6E62.0	fCantonese	bik1; seon4
U+6E62.0	fMandarin	bi4
U+6E62.0	fEnglish	[1] bathroom [2] neat; orderly
U+6E63.0	fUTF8	湣
U+6E63.0	fCNS11643	1-6049
U+6E63.0	fBig5	B543
U+6E63.0	fGB	5-5639
U+6E63.0	fR/S	 85.9
U+6E63.0	fTotalStrokes	 12
U+6E63.0	fCangjie	ERPA
U+6E63.0	fFourCorner	37164
U+6E63.0	fHakka	 men3; fun1
U+6E63.0	fCantonese	man5; fan1
U+6E63.0	fMandarin	min3; hun1; mian4; min2
U+6E63.0	fEnglish	mixed; confused
U+6E64.0	fUTF8	湤
U+6E64.0	fCNS11643	2-3D53
U+6E64.0	fBig5	DA4F
U+6E64.0	fGB	3-553B
U+6E64.0	fR/S	 85.9
U+6E64.0	fTotalStrokes	 12
U+6E64.0	fCangjie	EYSD
U+6E64.0	fFourCorner	38112
U+6E64.0	fCantonese	si1
U+6E64.0	fMandarin	shi1
U+6E65.0	fUTF8	湥
U+6E65.0	fCNS11643	2-3D50
U+6E65.0	fBig5	DA4C
U+6E65.0	fGB	5-5638
U+6E65.0	fR/S	 85.9
U+6E65.0	fTotalStrokes	 12
U+6E65.0	fCangjie	EJCK
U+6E65.0	fFourCorner	33143
U+6E65.0	fCantonese	teot1; dat6
U+6E65.0	fMandarin	tu2; mo4
U+6E66.0	fUTF8	湦
U+6E66.0	fCNS11643	2-3D58
U+6E66.0	fBig5	DA54
U+6E66.0	fGB	3-5528
U+6E66.0	fR/S	 85.9
U+6E66.0	fTotalStrokes	 12
U+6E66.0	fCangjie	EAHM
U+6E66.0	fFourCorner	36114
U+6E66.0	fCantonese	sang1
U+6E66.0	fMandarin	sheng1
U+6E67.0	fUTF8	湧
U+6E67.0	fCNS11643	1-6030
U+6E67.0	fBig5	B4E9
U+6E67.0	fGB	8-2D57
U+6E67.0	fR/S	 85.9
U+6E67.0	fTotalStrokes	 12
U+6E67.0	fCangjie	ENBS
U+6E67.0	fFourCorner	37127
U+6E67.0	fHakka	  jung3; rung3; jung5
U+6E67.0	fCantonese	jung2; cung1; jung5
U+6E67.0	fMandarin	yong3; chong1
U+6E67.0	fEnglish	[1] pour; gush [2] rise
U+6E68.0	fUTF8	湨
U+6E68.0	fCNS11643	2-3D44
U+6E68.0	fBig5	DA40
U+6E68.0	fGB	3-5523
U+6E68.0	fR/S	 85.9
U+6E68.0	fTotalStrokes	 12
U+6E68.0	fCangjie	EBUK
U+6E68.0	fFourCorner	36134
U+6E68.0	fCantonese	gwik1; gwik6
U+6E68.0	fMandarin	ju2; qu4
U+6E69.0	fUTF8	湩
U+6E69.0	fCNS11643	1-604C
U+6E69.0	fBig5	B546
U+6E69.0	fGB	3-552D
U+6E69.0	fR/S	 85.9
U+6E69.0	fTotalStrokes	 12
U+6E69.0	fCangjie	EHJG
U+6E69.0	fFourCorner	32114
U+6E69.0	fHakka	 dung5; dung3; zhung3; zung3
U+6E69.0	fCantonese	dung3 (also); dung6; zung3
U+6E69.0	fMandarin	zhong4; dong4; tong2; zhong3
U+6E69.0	fEnglish	 muddy; turbid milk
U+6E6A.0	fUTF8	湪
U+6E6A.0	fCNS11643	3-3C5B
U+6E6A.0	fGB	3-5547
U+6E6A.0	fR/S	 85.9
U+6E6A.0	fTotalStrokes	 12
U+6E6A.0	fCangjie	EVNO
U+6E6A.0	fFourCorner	37130
U+6E6A.0	fCantonese	teon3
U+6E6A.0	fMandarin	sui4
U+6E6B.0	fUTF8	湫
U+6E6B.0	fCNS11643	2-3D4B
U+6E6B.0	fBig5	DA47
U+6E6B.0	fGB	0-6450
U+6E6B.0	fR/S	 85.9
U+6E6B.0	fTotalStrokes	 12
U+6E6B.0	fCangjie	EHDF
U+6E6B.0	fFourCorner	39180
U+6E6B.0	fHakka	  ziu1;  ciu1
U+6E6B.0	fCantonese	zau1; ziu2; cau1 (also)
U+6E6B.0	fMandarin	jiu1; qiu1; jiu4; qiu4
U+6E6B.0	fEnglish	[1] small pond [2] a river in Kansu province
U+6E6B.1	fUTF8	湫
U+6E6B.1	fCNS11643	2-3D4B
U+6E6B.1	fBig5	DA47
U+6E6B.1	fGB	0-6450
U+6E6B.1	fR/S	 85.9
U+6E6B.1	fTotalStrokes	 12
U+6E6B.1	fCangjie	EHDF
U+6E6B.1	fFourCorner	39180
U+6E6B.1	fMandarin	jiao3
U+6E6B.1	fEnglish	damp and narrow
U+6E6C.0	fUTF8	湬
U+6E6C.0	fCNS11643	3-423A
U+6E6C.0	fGB	5-546C
U+6E6C.0	fR/S	  85.9
U+6E6C.0	fTotalStrokes	 13; 12
U+6E6C.0	fCangjie	HFE
U+6E6C.0	fFourCorner	29230
U+6E6C.0	fCantonese	zau1
U+6E6C.0	fMandarin	jiao3
U+6E6D.0	fUTF8	湭
U+6E6D.0	fCNS11643	3-3C60
U+6E6D.0	fGB	3-553F
U+6E6D.0	fR/S	 85.9
U+6E6D.0	fTotalStrokes	 12
U+6E6D.0	fCangjie	ETCW
U+6E6D.0	fFourCorner	38160
U+6E6D.0	fHakka	 ciu2
U+6E6D.0	fCantonese	cau4
U+6E6D.0	fMandarin	qiu2
U+6E6E.0	fUTF8	湮
U+6E6E.0	fCNS11643	1-603A
U+6E6E.0	fBig5	B4F3
U+6E6E.0	fGB	0-644E
U+6E6E.0	fR/S	 85.9
U+6E6E.0	fTotalStrokes	 12
U+6E6E.0	fCangjie	EMWG
U+6E6E.0	fFourCorner	31114
U+6E6E.0	fHakka	  jin1;  jen1; rin1; jim1; ren1
U+6E6E.0	fCantonese	jin1; jan1
U+6E6E.0	fMandarin	yin1; yan1; yan4; ye1; yi4
U+6E6E.0	fEnglish	[1] bury [2] block [3] long (time)
U+6E6F.0	fUTF8	湯
U+6E6F.0	fCNS11643	1-603D
U+6E6F.0	fBig5	B4F6
U+6E6F.0	fGB	1-4C40
U+6E6F.0	fR/S	 85.9
U+6E6F.0	fTotalStrokes	 12
U+6E6F.0	fCangjie	EAMH
U+6E6F.0	fFourCorner	36127
U+6E6F.0	fHakka	  tong1;  tong5
U+6E6F.0	fCantonese	tong1; soeng1; tong3; joeng4
U+6E6F.0	fMandarin	tang1; yang2; tang4; tang3
U+6E6F.0	fEnglish	[1] hot water [2] soup; broth [3] a Chinese family name
U+6E6F.1	fUTF8	湯
U+6E6F.1	fCNS11643	1-603D
U+6E6F.1	fBig5	B4F6
U+6E6F.1	fGB	1-4C40
U+6E6F.1	fR/S	 85.9
U+6E6F.1	fTotalStrokes	 12
U+6E6F.1	fCangjie	EAMH
U+6E6F.1	fFourCorner	36127
U+6E6F.1	fMandarin	shang1
U+6E6F.1	fEnglish	said of water flowing
U+6E6F.2	fUTF8	湯
U+6E6F.2	fCNS11643	1-603D
U+6E6F.2	fBig5	B4F6
U+6E6F.2	fGB	1-4C40
U+6E6F.2	fR/S	 85.9
U+6E6F.2	fTotalStrokes	 12
U+6E6F.2	fCangjie	EAMH
U+6E6F.2	fFourCorner	36127
U+6E6F.2	fMandarin	tang5
U+6E70.0	fUTF8	湰
U+6E70.0	fCNS11643	3-3C5C
U+6E70.0	fGB	3-5532
U+6E70.0	fR/S	  85.9
U+6E70.0	fTotalStrokes	 12
U+6E70.0	fCangjie	EHEM
U+6E70.0	fFourCorner	37110
U+6E70.0	fCantonese	lung4
U+6E70.0	fMandarin	long2
U+6E71.0	fUTF8	湱
U+6E71.0	fCNS11643	2-3D4A
U+6E71.0	fBig5	DA46
U+6E71.0	fGB	3-5475
U+6E71.0	fR/S	 85.9
U+6E71.0	fTotalStrokes	 12
U+6E71.0	fCangjie	EQJR
U+6E71.0	fFourCorner	35161
U+6E71.0	fHakka	 wak8; vak8
U+6E71.0	fCantonese	waak6
U+6E71.0	fMandarin	huo4
U+6E71.0	fEnglish	sound of water
U+6E72.0	fUTF8	湲
U+6E72.0	fCNS11643	1-604B
U+6E72.0	fBig5	B545
U+6E72.0	fGB	3-5530
U+6E72.0	fR/S	 85.9
U+6E72.0	fTotalStrokes	 12
U+6E72.0	fCangjie	EBME
U+6E72.0	fFourCorner	32147
U+6E72.0	fHakka	 jen2; ren2; jan2; jen1
U+6E72.0	fCantonese	jyun4; wun4 (also)
U+6E72.0	fMandarin	yuan2; huan2
U+6E72.0	fEnglish	flowing (water)
U+6E73.0	fUTF8	湳
U+6E73.0	fCNS11643	2-3D3A
U+6E73.0	fBig5	D9F5
U+6E73.0	fGB	3-547A
U+6E73.0	fR/S	 85.9
U+6E73.0	fTotalStrokes	 12
U+6E73.0	fCangjie	EJBJ
U+6E73.0	fFourCorner	34127
U+6E73.0	fCantonese	naam5; laam5 (also)
U+6E73.0	fMandarin	nan3
U+6E74.0	fUTF8	湴
U+6E74.0	fCNS11643	2-3648
U+6E74.0	fBig5	D5E4
U+6E74.0	fGB	3-546F
U+6E74.0	fR/S	 85.8
U+6E74.0	fTotalStrokes	 11
U+6E74.0	fCangjie	ETTC
U+6E74.0	fFourCorner	38117
U+6E74.0	fHakka	 pan5; pan6
U+6E74.0	fCantonese	baan6
U+6E74.0	fMandarin	ban4; pan1
U+6E74.0	fEnglish	[1] mud [2] mire
U+6E75.0	fUTF8	湵
U+6E75.0	fCNS11643	3-3C51
U+6E75.0	fGB	E-3059
U+6E75.0	fR/S	  85.9
U+6E75.0	fTotalStrokes	 12
U+6E75.0	fCangjie	ETGO
U+6E75.0	fFourCorner	38180
U+6E75.0	fCantonese	jau5
U+6E75.0	fMandarin	you3
U+6E76.0	fUTF8	湶
U+6E76.0	fCNS11643	3-3C6C
U+6E76.0	fGB	5-5630
U+6E76.0	fR/S	 85.9
U+6E76.0	fTotalStrokes	 12
U+6E76.0	fCangjie	EHAE
U+6E76.0	fFourCorner	36130
U+6E76.0	fCantonese	cyun4
U+6E76.0	fMandarin	quan2
U+6E77.0	fUTF8	湷
U+6E77.0	fCNS11643	2-3D54
U+6E77.0	fBig5	DA50
U+6E77.0	fGB	E-305A
U+6E77.0	fR/S	 85.9
U+6E77.0	fTotalStrokes	 12
U+6E77.0	fCangjie	EQKA
U+6E77.0	fFourCorner	35163
U+6E77.0	fCantonese	zong1; ceoi4
U+6E77.0	fMandarin	chui2
U+6E78.0	fUTF8	湸
U+6E78.0	fCNS11643	2-3D52
U+6E78.0	fBig5	DA4E
U+6E78.0	fGB	5-5634
U+6E78.0	fR/S	 85.9
U+6E78.0	fTotalStrokes	 12
U+6E78.0	fCangjie	EYRU
U+6E78.0	fFourCorner	30112
U+6E78.0	fCantonese	loeng6
U+6E78.0	fMandarin	liang4
U+6E79.0	fUTF8	湹
U+6E79.0	fCNS11643	2-3D56
U+6E79.0	fBig5	DA52
U+6E79.0	fGB	5-562B
U+6E79.0	fR/S	 85.9
U+6E79.0	fTotalStrokes	 12
U+6E79.0	fCangjie	EMWG
U+6E79.0	fFourCorner	31114
U+6E79.0	fCantonese	cin4
U+6E79.0	fMandarin	chan2
U+6E7A.0	fUTF8	湺
U+6E7A.0	fCNS11643	F-3C5D
U+6E7A.0	fGB	E-305B
U+6E7A.0	fR/S	 85.9
U+6E7A.0	fTotalStrokes	 12
U+6E7A.0	fCangjie	EORD
U+6E7A.0	fFourCorner	3619
U+6E7A.0	fCantonese	jin4
U+6E7A.0	fMandarin	yan2
U+6E7B.0	fUTF8	湻
U+6E7B.0	fCNS11643	3-3C62
U+6E7B.0	fGB	E-305C
U+6E7B.0	fR/S	  85.9
U+6E7B.0	fTotalStrokes	 12
U+6E7B.0	fCangjie	EYRA
U+6E7B.0	fFourCorner	30160
U+6E7B.0	fHakka	 shun2; sun2
U+6E7B.0	fCantonese	seon4
U+6E7B.0	fMandarin	chun2; zhun1
U+6E7B.0	fEnglish	 pure; unmixed; genuine honest
U+6E7C.0	fUTF8	湼
U+6E7C.0	fCNS11643	3-6542
U+6E7C.0	fGB	E-305D
U+6E7C.0	fR/S	 85.9
U+6E7C.0	fTotalStrokes	 12
U+6E7C.0	fCangjie	EHXM
U+6E7C.0	fFourCorner	3711
U+6E7C.0	fHakka	 ngiap7; lit7
U+6E7C.0	fCantonese	nip6; lip6 (also)
U+6E7C.0	fMandarin	nie4
U+6E7C.0	fEnglish	 black mud slime
U+6E7D.0	fUTF8	湽
U+6E7D.0	fCNS11643	4-385E
U+6E7D.0	fGB	E-305E
U+6E7D.0	fR/S	 85.9
U+6E7D.0	fTotalStrokes	 12
U+6E7D.0	fCangjie	EVMW
U+6E7D.0	fFourCorner	32160
U+6E7D.0	fCantonese	zi1
U+6E7D.0	fMandarin	zi1
U+6E7E.0	fUTF8	湾
U+6E7E.0	fCNS11643	3-3C63
U+6E7E.0	fGB	0-4D65
U+6E7E.0	fR/S	 85.9
U+6E7E.0	fTotalStrokes	 12
U+6E7E.0	fCangjie	EYCN
U+6E7E.0	fFourCorner	30120
U+6E7E.0	fHakka	 van1
U+6E7E.0	fCantonese	waan1; wan1 (also)
U+6E7E.0	fMandarin	wan1
U+6E7E.0	fEnglish	simplified form of &#x7063;
U+6E7F.0	fUTF8	湿
U+6E7F.0	fGB	0-4A2A
U+6E7F.0	fR/S	 85.9
U+6E7F.0	fTotalStrokes	 12
U+6E7F.0	fCangjie	EATC
U+6E7F.0	fHakka	 ship7
U+6E7F.0	fCantonese	sap1
U+6E7F.0	fMandarin	shi1; ta4; qi4; xie2; xi2
U+6E7F.0	fEnglish	simplified form of &#x6FD5;
U+6E80.0	fUTF8	満
U+6E80.0	fCNS11643	3-3C68
U+6E80.0	fGB	E-305F
U+6E80.0	fR/S	 85.9
U+6E80.0	fTotalStrokes	 12
U+6E80.0	fCangjie	XETUB
U+6E80.0	fFourCorner	34120
U+6E80.0	fCantonese	mun5
U+6E80.0	fMandarin	man3
U+6E80.0	fEnglish	fullness, enough; pride
U+6E81.0	fUTF8	溁
U+6E81.0	fGB	8-2E5D
U+6E81.0	fR/S	 85.9; 85.14
U+6E81.0	fTotalStrokes	 12
U+6E81.0	fCangjie	ETBD
U+6E81.0	fCantonese	jing4
U+6E81.0	fMandarin	ying2
U+6E82.0	fUTF8	溂
U+6E82.0	fCNS11643	F-3C47
U+6E82.0	fGB	E-3060
U+6E82.0	fR/S	 85.9
U+6E82.0	fTotalStrokes	 12
U+6E82.0	fCangjie	EDLN
U+6E82.0	fCantonese	laat6
U+6E82.0	fMandarin	la4
U+6E83.0	fUTF8	溃
U+6E83.0	fGB	0-4023
U+6E83.0	fR/S	 85.9
U+6E83.0	fTotalStrokes	 12
U+6E83.0	fCangjie	XELMO
U+6E83.0	fHakka	 k'wui5
U+6E83.0	fCantonese	kui2; kwui2 (also)
U+6E83.0	fMandarin	kui4; hui4; hu2; hui2
U+6E83.0	fEnglish	simplified form of &#x6F70;
U+6E84.0	fUTF8	溄
U+6E84.0	fCNS11643	3-6540
U+6E84.0	fGB	E-3061
U+6E84.0	fR/S	 85.9
U+6E84.0	fTotalStrokes	 12
U+6E84.0	fCangjie	EUHQ
U+6E84.0	fFourCorner	3215
U+6E84.0	fCantonese	fung4
U+6E85.0	fUTF8	溅
U+6E85.0	fGB	0-3D26
U+6E85.0	fR/S	 85.9
U+6E85.0	fTotalStrokes	 12
U+6E85.0	fCangjie	EBOJ
U+6E85.0	fHakka	 cien5; zien5
U+6E85.0	fCantonese	zin3; zin1
U+6E85.0	fMandarin	jian4; jian1; zan4
U+6E85.0	fEnglish	simplified form of &#x6FFA;
U+6E86.0	fUTF8	溆
U+6E86.0	fCNS11643	4-385F
U+6E86.0	fGB	0-6453
U+6E86.0	fR/S	 85.9
U+6E86.0	fTotalStrokes	 12
U+6E86.0	fCangjie	EODE
U+6E86.0	fFourCorner	37140
U+6E86.0	fCantonese	zeoi6
U+6E86.0	fMandarin	xu4
U+6E86.0	fEnglish	a river in Hunan
U+6E87.0	fUTF8	溇
U+6E87.0	fGB	8-7D27
U+6E87.0	fR/S	 85.11; 85.9
U+6E87.0	fTotalStrokes	 12
U+6E87.0	fCangjie	EFDV
U+6E87.0	fCantonese	lau4
U+6E87.0	fMandarin	lou2; luu3; l3
U+6E88.0	fUTF8	溈
U+6E88.0	fCNS11643	2-3D31
U+6E88.0	fBig5	D9EC
U+6E88.0	fGB	E-3062
U+6E88.0	fR/S	 85.9
U+6E88.0	fTotalStrokes	 12
U+6E88.0	fCangjie	EIKF
U+6E88.0	fFourCorner	32127
U+6E88.0	fCantonese	gwai1
U+6E88.0	fMandarin	gui1; wei2
U+6E88.0	fEnglish	a river in Yung-chi county in Shansi province
U+6E89.0	fUTF8	溉
U+6E89.0	fCNS11643	1-6046
U+6E89.0	fBig5	B540
U+6E89.0	fGB	0-3848
U+6E89.0	fR/S	 85.9
U+6E89.0	fTotalStrokes	 12
U+6E89.0	fCangjie	EAIU
U+6E89.0	fFourCorner	31114
U+6E89.0	fHakka	 kai3
U+6E89.0	fCantonese	koi3; goi3
U+6E89.0	fMandarin	gai4; xie4
U+6E89.0	fEnglish	[1] water; irrigate [2] wash
U+6E8A.0	fUTF8	溊
U+6E8A.0	fCNS11643	3-653F
U+6E8A.0	fGB	E-3063
U+6E8A.0	fR/S	 85.9
U+6E8A.0	fTotalStrokes	 12
U+6E8A.0	fCangjie	ERSE
U+6E8A.0	fFourCorner	3714
U+6E8A.0	fCantonese	bo1
U+6E8B.0	fUTF8	溋
U+6E8B.0	fCNS11643	3-6541
U+6E8B.0	fGB	E-3064
U+6E8B.0	fR/S	 85.9
U+6E8B.0	fTotalStrokes	 12
U+6E8B.0	fCangjie	ENST
U+6E8B.0	fFourCorner	3711
U+6E8B.0	fCantonese	jing4
U+6E8C.0	fUTF8	溌
U+6E8C.0	fGB	E-3065
U+6E8C.0	fR/S	 85.9
U+6E8C.0	fTotalStrokes	 12
U+6E8C.0	fCangjie	ENOP
U+6E8C.0	fCantonese	put3
U+6E8C.0	fMandarin	po1
U+6E8C.0	fEnglish	pour; splash; water; sprinkle
U+6E8D.0	fUTF8	溍
U+6E8D.0	fCNS11643	2-4447
U+6E8D.0	fBig5	DE61
U+6E8D.0	fGB	3-554B
U+6E8D.0	fR/S	 85.10
U+6E8D.0	fTotalStrokes	 13
U+6E8D.0	fCangjie	EMIA
U+6E8D.0	fFourCorner	31161
U+6E8D.0	fCantonese	zeon3
U+6E8D.0	fMandarin	jin4
U+6E8D.0	fEnglish	water
U+6E8E.0	fUTF8	溎
U+6E8E.0	fCNS11643	2-4446
U+6E8E.0	fBig5	DE60
U+6E8E.0	fGB	3-554C
U+6E8E.0	fR/S	 85.10
U+6E8E.0	fTotalStrokes	 13
U+6E8E.0	fCangjie	EDGG
U+6E8E.0	fFourCorner	34114
U+6E8E.0	fCantonese	gwai3
U+6E8E.0	fMandarin	gui4; yan4
U+6E8E.0	fEnglish	the name of a river
U+6E8F.0	fUTF8	溏
U+6E8F.0	fCNS11643	2-442C
U+6E8F.0	fBig5	DE46
U+6E8F.0	fGB	0-6467
U+6E8F.0	fR/S	 85.10
U+6E8F.0	fTotalStrokes	 13
U+6E8F.0	fCangjie	EILR
U+6E8F.0	fFourCorner	30167
U+6E8F.0	fCantonese	tong4
U+6E8F.0	fMandarin	tang2
U+6E8F.0	fEnglish	 pool not hardened; semi-soft
U+6E90.0	fUTF8	源
U+6E90.0	fCNS11643	1-6463
U+6E90.0	fBig5	B7BD
U+6E90.0	fGB	0-5434
U+6E90.0	fR/S	 85.10
U+6E90.0	fTotalStrokes	 13
U+6E90.0	fCangjie	EMHF
U+6E90.0	fFourCorner	31196
U+6E90.0	fHakka	  ngien2; nien2; gnian3; ngen2; ngian2
U+6E90.0	fCantonese	jyun4
U+6E90.0	fMandarin	yuan2
U+6E90.0	fEnglish	 spring; source; head (stream)
U+6E91.0	fUTF8	溑
U+6E91.0	fCNS11643	3-4230
U+6E91.0	fGB	3-5439
U+6E91.0	fR/S	  85.10
U+6E91.0	fTotalStrokes	 13
U+6E91.0	fCangjie	EFBC
U+6E91.0	fFourCorner	39180
U+6E91.0	fCantonese	so2
U+6E91.0	fMandarin	suo3
U+6E92.0	fUTF8	溒
U+6E92.0	fCNS11643	2-4445
U+6E92.0	fBig5	DE5F
U+6E92.0	fGB	3-554A
U+6E92.0	fR/S	 85.10
U+6E92.0	fTotalStrokes	 13
U+6E92.0	fCangjie	EGRV
U+6E92.0	fFourCorner	34132
U+6E92.0	fCantonese	jyun4
U+6E92.0	fMandarin	yuan2
U+6E93.0	fUTF8	溓
U+6E93.0	fCNS11643	2-442F
U+6E93.0	fBig5	DE49
U+6E93.0	fGB	3-555F
U+6E93.0	fR/S	 85.10
U+6E93.0	fTotalStrokes	 13
U+6E93.0	fCangjie	ETXC
U+6E93.0	fFourCorner	38137
U+6E93.0	fHakka	 liam2; liam3; liam1
U+6E93.0	fCantonese	lim5; lim4
U+6E93.0	fMandarin	lian2; lian3; nian2; lian4; lin2; xian2; xian4
U+6E93.0	fEnglish	[1] waterfall [2] cascade
U+6E94.0	fUTF8	溔
U+6E94.0	fCNS11643	2-4430
U+6E94.0	fBig5	DE4A
U+6E94.0	fGB	3-555E
U+6E94.0	fR/S	 85.10
U+6E94.0	fTotalStrokes	 13
U+6E94.0	fCangjie	ETGF
U+6E94.0	fFourCorner	38131
U+6E94.0	fHakka	 jau5; rau2; jeu2; jeu5; rau5; jau3
U+6E94.0	fCantonese	jiu5; ngaau5; jiu6
U+6E94.0	fMandarin	yao3
U+6E94.0	fEnglish	boundless
U+6E95.0	fUTF8	溕
U+6E95.0	fCNS11643	4-3E3F
U+6E95.0	fGB	E-3066
U+6E95.0	fR/S	  85.10
U+6E95.0	fTotalStrokes	 13
U+6E95.0	fCangjie	EBMO
U+6E95.0	fFourCorner	37130
U+6E95.0	fCantonese	mung4
U+6E95.0	fMandarin	meng4
U+6E96.0	fUTF8	準
U+6E96.0	fCNS11643	1-646D
U+6E96.0	fBig5	B7C7
U+6E96.0	fGB	1-573C
U+6E96.0	fR/S	 85.10
U+6E96.0	fTotalStrokes	 13
U+6E96.0	fCangjie	EGJ
U+6E96.0	fFourCorner	30401
U+6E96.0	fHakka	  zun3; zhun3
U+6E96.0	fCantonese	zeon2; zyut3
U+6E96.0	fMandarin	zhun3; shui3; zhuo1; zhuo2
U+6E96.0	fEnglish	[1] level; even [2] rule; guideline; standard; criterion; accurate; accuracy [3] aim; sight [4] quasi- (law) [5] would-be (bride, son-in-law, etc.); to- be [6] certainly
U+6E97.0	fUTF8	溗
U+6E97.0	fCNS11643	2-444E
U+6E97.0	fBig5	DE68
U+6E97.0	fGB	3-5554
U+6E97.0	fR/S	 85.10
U+6E97.0	fTotalStrokes	 13
U+6E97.0	fCangjie	EHDP
U+6E97.0	fFourCorner	32191
U+6E97.0	fCantonese	sing4
U+6E97.0	fMandarin	sheng2
U+6E98.0	fUTF8	溘
U+6E98.0	fCNS11643	1-6468
U+6E98.0	fBig5	B7C2
U+6E98.0	fGB	0-645B
U+6E98.0	fR/S	 85.10
U+6E98.0	fTotalStrokes	 13
U+6E98.0	fCangjie	EGIT
U+6E98.0	fFourCorner	34117
U+6E98.0	fHakka	  kap7;  hap8
U+6E98.0	fCantonese	hap6; hap1
U+6E98.0	fMandarin	ke4; kai4
U+6E98.0	fEnglish	abruptly; suddenly; unexpectedly; sudden; abrupt; unexpected
U+6E99.0	fUTF8	溙
U+6E99.0	fCNS11643	2-4444
U+6E99.0	fBig5	DE5E
U+6E99.0	fGB	5-563D
U+6E99.0	fR/S	 85.10
U+6E99.0	fTotalStrokes	 13
U+6E99.0	fCangjie	EQKE
U+6E99.0	fFourCorner	35132
U+6E99.0	fHakka	 tai5
U+6E99.0	fCantonese	taai3
U+6E99.0	fMandarin	tai4; tai3
U+6E9A.0	fUTF8	溚
U+6E9A.0	fCNS11643	3-4233
U+6E9A.0	fGB	3-5478
U+6E9A.0	fR/S	 85.10
U+6E9A.0	fTotalStrokes	 13
U+6E9A.0	fCangjie	ETOR
U+6E9A.0	fFourCorner	34160
U+6E9A.0	fHakka	 dap7
U+6E9A.0	fCantonese	taap3; daap3 (also); dap6
U+6E9A.0	fMandarin	da2; ta3
U+6E9A.0	fEnglish	to get wet by rain; to drip
U+6E9B.0	fUTF8	溛
U+6E9B.0	fCNS11643	2-4429
U+6E9B.0	fBig5	DE43
U+6E9B.0	fGB	5-5652
U+6E9B.0	fR/S	 85.10
U+6E9B.0	fTotalStrokes	 13
U+6E9B.0	fCangjie	EJCO
U+6E9B.0	fFourCorner	33132
U+6E9B.0	fHakka	 wa1
U+6E9B.0	fCantonese	waa1
U+6E9B.0	fMandarin	wa1
U+6E9C.0	fUTF8	溜
U+6E9C.0	fCNS11643	1-646E
U+6E9C.0	fBig5	B7C8
U+6E9C.0	fGB	0-416F
U+6E9C.0	fR/S	 85.10
U+6E9C.0	fTotalStrokes	 13
U+6E9C.0	fCangjie	EHHW
U+6E9C.0	fFourCorner	37162
U+6E9C.0	fHakka	 liu5; liu1;  tiu5; liu2; liu3
U+6E9C.0	fCantonese	lau6; liu1; lau1 (also); lau4 (also)
U+6E9C.0	fMandarin	liu4; liu2
U+6E9C.0	fEnglish	[1] rapids [2] row; column
U+6E9C.1	fUTF8	溜
U+6E9C.1	fCNS11643	1-646E
U+6E9C.1	fBig5	B7C8
U+6E9C.1	fGB	0-416F
U+6E9C.1	fR/S	 85.10
U+6E9C.1	fTotalStrokes	 13
U+6E9C.1	fCangjie	EHHW
U+6E9C.1	fFourCorner	37162
U+6E9C.1	fMandarin	liu1
U+6E9C.1	fEnglish	[1] slide; glide; slip; slippery [2] go secretly and quietly
U+6E9D.0	fUTF8	溝
U+6E9D.0	fCNS11643	1-6464
U+6E9D.0	fBig5	B7BE
U+6E9D.0	fGB	1-3935
U+6E9D.0	fR/S	 85.10
U+6E9D.0	fTotalStrokes	 13
U+6E9D.0	fCangjie	ETTB
U+6E9D.0	fFourCorner	35147
U+6E9D.0	fHakka	 geu1; kieu1; giu1
U+6E9D.0	fCantonese	gau1 (also); kau1
U+6E9D.0	fMandarin	gou1; gang3; kou4
U+6E9D.0	fEnglish	[1] ditch; drain; waterway; moat [2] groove; rut
U+6E9E.0	fUTF8	溞
U+6E9E.0	fCNS11643	2-4438
U+6E9E.0	fBig5	DE52
U+6E9E.0	fGB	5-563C
U+6E9E.0	fR/S	 85.10
U+6E9E.0	fTotalStrokes	 13
U+6E9E.0	fCangjie	EEII
U+6E9E.0	fFourCorner	37136
U+6E9E.0	fHakka	  sau1; zau1
U+6E9E.0	fCantonese	sou1
U+6E9E.0	fMandarin	sao1; sou1
U+6E9E.0	fEnglish	the sound of washing rice
U+6E9F.0	fUTF8	溟
U+6E9F.0	fCNS11643	2-442E
U+6E9F.0	fBig5	DE48
U+6E9F.0	fGB	0-6469
U+6E9F.0	fR/S	 85.10
U+6E9F.0	fTotalStrokes	 13
U+6E9F.0	fCangjie	EBAC
U+6E9F.0	fFourCorner	37180
U+6E9F.0	fHakka	 men2
U+6E9F.0	fCantonese	ming4; ming5
U+6E9F.0	fMandarin	ming2; mi4; ming3
U+6E9F.0	fEnglish	[1] drizzle vast; boundless [2] sea; ocean
U+6EA0.0	fUTF8	溠
U+6EA0.0	fCNS11643	2-4431
U+6EA0.0	fBig5	DE4B
U+6EA0.0	fGB	3-553C
U+6EA0.0	fR/S	 85.10
U+6EA0.0	fTotalStrokes	 13
U+6EA0.0	fCangjie	ETQM
U+6EA0.0	fFourCorner	38112
U+6EA0.0	fHakka	 za5
U+6EA0.0	fCantonese	zaa3
U+6EA0.0	fMandarin	zha4; zha1; chai1; cuo1
U+6EA0.0	fEnglish	a river in Hubei province
U+6EA1.0	fUTF8	溡
U+6EA1.0	fCNS11643	2-4449
U+6EA1.0	fBig5	DE63
U+6EA1.0	fGB	5-5557
U+6EA1.0	fR/S	 85.10
U+6EA1.0	fTotalStrokes	 13
U+6EA1.0	fCangjie	EAGI
U+6EA1.0	fFourCorner	34141
U+6EA1.0	fHakka	 shi2; sii2
U+6EA1.0	fCantonese	si4
U+6EA1.0	fMandarin	shi2
U+6EA1.0	fEnglish	a small stream in Shantung
U+6EA2.0	fUTF8	溢
U+6EA2.0	fCNS11643	1-645E
U+6EA2.0	fBig5	B7B8
U+6EA2.0	fGB	0-5267
U+6EA2.0	fR/S	 85.10
U+6EA2.0	fTotalStrokes	 13
U+6EA2.0	fCangjie	ETCT
U+6EA2.0	fFourCorner	38117
U+6EA2.0	fHakka	  jit7; rit7; jit8
U+6EA2.0	fCantonese	jat6
U+6EA2.0	fMandarin	yi4; shi2; shi4
U+6EA2.0	fEnglish	[1] flow over; brim over [2] excessive
U+6EA3.0	fUTF8	溣
U+6EA3.0	fCNS11643	2-4450
U+6EA3.0	fBig5	DE6A
U+6EA3.0	fGB	5-553E
U+6EA3.0	fR/S	 85.10
U+6EA3.0	fTotalStrokes	 13
U+6EA3.0	fCangjie	EOOB
U+6EA3.0	fFourCorner	38127
U+6EA3.0	fCantonese	leon6
U+6EA3.0	fMandarin	lun2; lun4
U+6EA4.0	fUTF8	溤
U+6EA4.0	fCNS11643	2-4448
U+6EA4.0	fBig5	DE62
U+6EA4.0	fGB	5-5472
U+6EA4.0	fR/S	 85.10
U+6EA4.0	fTotalStrokes	 13
U+6EA4.0	fCangjie	ESQF
U+6EA4.0	fFourCorner	31127
U+6EA4.0	fCantonese	maa5
U+6EA4.0	fMandarin	ma3
U+6EA5.0	fUTF8	溥
U+6EA5.0	fCNS11643	1-6467
U+6EA5.0	fBig5	B7C1
U+6EA5.0	fGB	0-645F
U+6EA5.0	fR/S	 85.10
U+6EA5.0	fTotalStrokes	 13
U+6EA5.0	fCangjie	EIBI
U+6EA5.0	fFourCorner	33142
U+6EA5.0	fHakka	 pu3
U+6EA5.0	fCantonese	pou2
U+6EA5.0	fMandarin	pu3; bu4; bo2; fu1; po4
U+6EA5.0	fEnglish	[1] big; great; vast; wide [2] universal
U+6EA6.0	fUTF8	溦
U+6EA6.0	fCNS11643	2-443D
U+6EA6.0	fBig5	DE57
U+6EA6.0	fGB	3-5552
U+6EA6.0	fR/S	 85.10
U+6EA6.0	fTotalStrokes	 13
U+6EA6.0	fCangjie	EUUK
U+6EA6.0	fFourCorner	38140
U+6EA6.0	fHakka	  mi2; mui2
U+6EA6.0	fCantonese	mei4; mei1 (also)
U+6EA6.0	fMandarin	wei2; wei1; mei2
U+6EA6.0	fEnglish	 drizzle (variant) valley (variant) mold
U+6EA7.0	fUTF8	溧
U+6EA7.0	fCNS11643	1-6472
U+6EA7.0	fBig5	B7CC
U+6EA7.0	fGB	0-6460
U+6EA7.0	fR/S	 85.10
U+6EA7.0	fTotalStrokes	 13
U+6EA7.0	fCangjie	EMWD
U+6EA7.0	fFourCorner	31194
U+6EA7.0	fHakka	  lit8
U+6EA7.0	fCantonese	leot6
U+6EA7.0	fMandarin	li4
U+6EA7.0	fEnglish	[1] a river in Anhwei and Kiangsu province [2] a county in Kiangsu
U+6EA8.0	fUTF8	溨
U+6EA8.0	fCNS11643	3-422E
U+6EA8.0	fGB	5-563F
U+6EA8.0	fR/S	  85.10
U+6EA8.0	fTotalStrokes	 13
U+6EA8.0	fCangjie	EJID
U+6EA8.0	fFourCorner	33150
U+6EA8.0	fCantonese	zoi1
U+6EA8.0	fMandarin	cai2
U+6EA9.0	fUTF8	溩
U+6EA9.0	fCNS11643	3-4236
U+6EA9.0	fGB	3-5355
U+6EA9.0	fR/S	  85.10
U+6EA9.0	fTotalStrokes	 13
U+6EA9.0	fCangjie	EHRF
U+6EA9.0	fFourCorner	37120
U+6EA9.0	fCantonese	wu2
U+6EA9.0	fMandarin	wu4
U+6EAA.0	fUTF8	溪
U+6EAA.0	fCNS11643	1-6471
U+6EAA.0	fBig5	B7CB
U+6EAA.0	fGB	0-4F2A
U+6EAA.0	fR/S	 85.10
U+6EAA.0	fTotalStrokes	 13
U+6EAA.0	fCangjie	EBVK
U+6EAA.0	fFourCorner	32134
U+6EAA.0	fHakka	  hai1; kai1; k'ie1; ke1; k'e1
U+6EAA.0	fCantonese	kai1
U+6EAA.0	fMandarin	xi1; qi1
U+6EAA.0	fEnglish	 mountain stream; creek
U+6EAB.0	fUTF8	溫
U+6EAB.0	fCNS11643	1-646B
U+6EAB.0	fBig5	B7C5
U+6EAB.0	fGB	E-3067
U+6EAB.0	fR/S	 85.10; 85.9
U+6EAB.0	fTotalStrokes	 13
U+6EAB.0	fCangjie	EWOT
U+6EAB.0	fFourCorner	36117
U+6EAB.0	fHakka	 wun1; vun1; wun2; mun1
U+6EAB.0	fCantonese	wan1
U+6EAB.0	fMandarin	wen1; yun4
U+6EAB.0	fEnglish	[1] lukewarm; warm; mild; warm [2] review; revise [3] temperature [4] a Chinese family name
U+6EAC.0	fUTF8	溬
U+6EAC.0	fCNS11643	3-4240
U+6EAC.0	fGB	E-3068
U+6EAC.0	fR/S	 85.10
U+6EAC.0	fTotalStrokes	 13
U+6EAC.0	fCangjie	ETUI
U+6EAC.0	fFourCorner	38110
U+6EAC.0	fCantonese	goeng1
U+6EAC.0	fMandarin	qiang1
U+6EAD.0	fUTF8	溭
U+6EAD.0	fCNS11643	4-3E36
U+6EAD.0	fGB	3-5551
U+6EAD.0	fR/S	  85.10
U+6EAD.0	fTotalStrokes	 13
U+6EAD.0	fCangjie	EWCE
U+6EAD.0	fFourCorner	36140
U+6EAD.0	fHakka	 zet7
U+6EAD.0	fCantonese	zik1
U+6EAD.0	fMandarin	ze2; ce4
U+6EAD.0	fEnglish	waves rolling one after another
U+6EAE.0	fUTF8	溮
U+6EAE.0	fCNS11643	2-444F
U+6EAE.0	fBig5	DE69
U+6EAE.0	fGB	3-5421
U+6EAE.0	fR/S	 85.10
U+6EAE.0	fTotalStrokes	 13
U+6EAE.0	fCangjie	EHRB
U+6EAE.0	fFourCorner	31127
U+6EAE.0	fHakka	 sii1
U+6EAE.0	fCantonese	si1
U+6EAE.0	fMandarin	shi1
U+6EAE.0	fEnglish	a river in Henan province
U+6EAF.0	fUTF8	溯
U+6EAF.0	fCNS11643	1-645F
U+6EAF.0	fBig5	B7B9
U+6EAF.0	fGB	0-4B5D
U+6EAF.0	fR/S	 85.10
U+6EAF.0	fTotalStrokes	 13
U+6EAF.0	fCangjie	ETUB
U+6EAF.0	fFourCorner	37120
U+6EAF.0	fHakka	 sok7; shok7
U+6EAF.0	fCantonese	sou3
U+6EAF.0	fMandarin	su4; shuo4
U+6EAF.0	fEnglish	[1] go upstream; go against current; go against a stream [2] trace; recall
U+6EB0.0	fUTF8	溰
U+6EB0.0	fCNS11643	2-443B
U+6EB0.0	fBig5	DE55
U+6EB0.0	fGB	5-553C
U+6EB0.0	fR/S	 85.10
U+6EB0.0	fTotalStrokes	 13
U+6EB0.0	fCangjie	EUMT
U+6EB0.0	fFourCorner	32119
U+6EB0.0	fHakka	  ki2; wui2; wi2
U+6EB0.0	fCantonese	ji4; kei2
U+6EB0.0	fMandarin	yi1
U+6EB0.0	fEnglish	the appearance of frost and snow
U+6EB1.0	fUTF8	溱
U+6EB1.0	fCNS11643	2-4432
U+6EB1.0	fBig5	DE4C
U+6EB1.0	fGB	0-645A
U+6EB1.0	fR/S	 85.10
U+6EB1.0	fTotalStrokes	 13
U+6EB1.0	fCangjie	EQKD
U+6EB1.0	fFourCorner	35194
U+6EB1.0	fHakka	  zin1
U+6EB1.0	fCantonese	zeon1; ceon4
U+6EB1.0	fMandarin	zhen1; qin2
U+6EB1.0	fEnglish	a river in Henan
U+6EB2.0	fUTF8	溲
U+6EB2.0	fCNS11643	2-443F
U+6EB2.0	fBig5	DE59
U+6EB2.0	fGB	0-6451
U+6EB2.0	fR/S	 85.10
U+6EB2.0	fTotalStrokes	 13
U+6EB2.0	fCangjie	EHXE
U+6EB2.0	fFourCorner	37147
U+6EB2.0	fHakka	 seu3; seu1; seu5
U+6EB2.0	fCantonese	sau1; sau2
U+6EB2.0	fMandarin	sou1; sou3; sao1; shao1
U+6EB2.0	fEnglish	[1] urinate [2] soak; drench; immerse
U+6EB3.0	fUTF8	溳
U+6EB3.0	fCNS11643	2-444B
U+6EB3.0	fBig5	DE65
U+6EB3.0	fGB	3-543B
U+6EB3.0	fR/S	 85.10
U+6EB3.0	fTotalStrokes	 13
U+6EB3.0	fCangjie	ERBC
U+6EB3.0	fFourCorner	36186
U+6EB3.0	fHakka	 jun2; run2; jun3
U+6EB3.0	fCantonese	wan4
U+6EB3.0	fMandarin	yun2; yun3
U+6EB3.0	fEnglish	a small branch of the river Han in Hupeh
U+6EB4.0	fUTF8	溴
U+6EB4.0	fCNS11643	1-6473
U+6EB4.0	fBig5	B7CD
U+6EB4.0	fGB	0-6465
U+6EB4.0	fR/S	 85.10
U+6EB4.0	fTotalStrokes	 13
U+6EB4.0	fCangjie	EHUK
U+6EB4.0	fFourCorner	36134
U+6EB4.0	fHakka	 hiu3; cu5; chiu5; ciu5
U+6EB4.0	fCantonese	cau3
U+6EB4.0	fMandarin	xiu4; chou4
U+6EB4.0	fEnglish	bromine
U+6EB5.0	fUTF8	溵
U+6EB5.0	fCNS11643	3-4239
U+6EB5.0	fGB	5-564D
U+6EB5.0	fR/S	  85.10
U+6EB5.0	fTotalStrokes	 13
U+6EB5.0	fCangjie	EHSE
U+6EB5.0	fFourCorner	37140
U+6EB5.0	fCantonese	jan1; jan2 (also)
U+6EB5.0	fMandarin	yin1
U+6EB6.0	fUTF8	溶
U+6EB6.0	fCNS11643	1-6461
U+6EB6.0	fBig5	B7BB
U+6EB6.0	fGB	0-485C
U+6EB6.0	fR/S	 85.10
U+6EB6.0	fTotalStrokes	 13
U+6EB6.0	fCangjie	EJCR
U+6EB6.0	fFourCorner	33168
U+6EB6.0	fHakka	  jung2; rung2
U+6EB6.0	fCantonese	jung4
U+6EB6.0	fMandarin	rong2; yong3
U+6EB6.0	fEnglish	[1] melt; dissolve [2] overflowing with water; having much water (rivers)
U+6EB7.0	fUTF8	溷
U+6EB7.0	fCNS11643	2-443A
U+6EB7.0	fBig5	DE54
U+6EB7.0	fGB	0-6463
U+6EB7.0	fR/S	 85.10
U+6EB7.0	fTotalStrokes	 13
U+6EB7.0	fCangjie	EWMO
U+6EB7.0	fFourCorner	36100
U+6EB7.0	fHakka	  fun5
U+6EB7.0	fCantonese	wan6
U+6EB7.0	fMandarin	hun4; hun2; huan4
U+6EB7.0	fEnglish	[1] dirty [2] messy
U+6EB8.0	fUTF8	溸
U+6EB8.0	fCNS11643	3-422D
U+6EB8.0	fGB	5-563E
U+6EB8.0	fR/S	 85.10
U+6EB8.0	fTotalStrokes	 13
U+6EB8.0	fCangjie	EQMF
U+6EB8.0	fFourCorner	35190
U+6EB8.0	fCantonese	sou3
U+6EB8.0	fMandarin	su4
U+6EB9.0	fUTF8	溹
U+6EB9.0	fCNS11643	2-4433
U+6EB9.0	fBig5	DE4D
U+6EB9.0	fGB	5-5642
U+6EB9.0	fR/S	 85.10
U+6EB9.0	fTotalStrokes	 13
U+6EB9.0	fCangjie	EJBF
U+6EB9.0	fFourCorner	34193
U+6EB9.0	fCantonese	sok3
U+6EB9.0	fMandarin	su4; suo4
U+6EBA.0	fUTF8	溺
U+6EBA.0	fCNS11643	1-646A
U+6EBA.0	fBig5	B7C4
U+6EBA.0	fGB	0-4467
U+6EBA.0	fR/S	 85.10
U+6EBA.0	fTotalStrokes	 13
U+6EBA.0	fCangjie	ENMM
U+6EBA.0	fFourCorner	37127
U+6EBA.0	fHakka	  nit8;  ngiau5; lek8; lit8; lit7; ngiau3; net8
U+6EBA.0	fCantonese	nik6; nik1 (also); niu6; lik6 (also); lik1 (also); liu6 (also)
U+6EBA.0	fMandarin	ni4; nuo4; ruo4
U+6EBA.0	fEnglish	[1] drown [2] indulge
U+6EBA.1	fUTF8	溺
U+6EBA.1	fCNS11643	1-646A
U+6EBA.1	fBig5	B7C4
U+6EBA.1	fGB	0-4467
U+6EBA.1	fR/S	 85.10
U+6EBA.1	fTotalStrokes	 13
U+6EBA.1	fCangjie	ENMM
U+6EBA.1	fFourCorner	37127
U+6EBA.1	fMandarin	niao4
U+6EBA.1	fEnglish	 urinate
U+6EBB.0	fUTF8	溻
U+6EBB.0	fCNS11643	3-4231
U+6EBB.0	fGB	0-6462
U+6EBB.0	fR/S	 85.10
U+6EBB.0	fTotalStrokes	 13
U+6EBB.0	fCangjie	EASM
U+6EBB.0	fFourCorner	36127
U+6EBB.0	fHakka	  tap7
U+6EBB.0	fCantonese	taap3
U+6EBB.0	fMandarin	ta4; ta1
U+6EBB.0	fEnglish	wet
U+6EBC.0	fUTF8	溼
U+6EBC.0	fCNS11643	1-6469
U+6EBC.0	fBig5	B7C3
U+6EBC.0	fGB	E-3069
U+6EBC.0	fR/S	 85.10
U+6EBC.0	fTotalStrokes	 13
U+6EBC.0	fCangjie	EMVG
U+6EBC.0	fFourCorner	31114
U+6EBC.0	fHakka	 ship7; siip7; sip7
U+6EBC.0	fCantonese	sap1
U+6EBC.0	fMandarin	shi1; qi4
U+6EBC.0	fEnglish	 damp; moist dampness; moisture
U+6EBD.0	fUTF8	溽
U+6EBD.0	fCNS11643	2-4436
U+6EBD.0	fBig5	DE50
U+6EBD.0	fGB	0-6461
U+6EBD.0	fR/S	 85.10
U+6EBD.0	fTotalStrokes	 13
U+6EBD.0	fCangjie	EMVI
U+6EBD.0	fFourCorner	31143
U+6EBD.0	fHakka	  juk8; ruk8
U+6EBD.0	fCantonese	juk6
U+6EBD.0	fMandarin	ru4; ru2
U+6EBD.0	fEnglish	moist; humid
U+6EBE.0	fUTF8	溾
U+6EBE.0	fCNS11643	2-4440
U+6EBE.0	fBig5	DE5A
U+6EBE.0	fGB	3-552E
U+6EBE.0	fR/S	 85.10
U+6EBE.0	fTotalStrokes	 13
U+6EBE.0	fCangjie	EHUI
U+6EBE.0	fFourCorner	36113
U+6EBE.0	fCantonese	aai1; ngaai1 (also); waai1; wui6
U+6EBE.0	fMandarin	wei1; hui4
U+6EBF.0	fUTF8	溿
U+6EBF.0	fCNS11643	2-444A
U+6EBF.0	fBig5	DE64
U+6EBF.0	fGB	5-5645
U+6EBF.0	fR/S	 85.10
U+6EBF.0	fTotalStrokes	 13
U+6EBF.0	fCangjie	EWFQ
U+6EBF.0	fFourCorner	39150
U+6EBF.0	fCantonese	pun3
U+6EBF.0	fMandarin	pan4
U+6EC0.0	fUTF8	滀
U+6EC0.0	fCNS11643	2-442D
U+6EC0.0	fBig5	DE47
U+6EC0.0	fGB	3-555D
U+6EC0.0	fR/S	 85.10
U+6EC0.0	fTotalStrokes	 13
U+6EC0.0	fCangjie	EYVW
U+6EC0.0	fFourCorner	30163
U+6EC0.0	fHakka	 hiuk7
U+6EC0.0	fCantonese	cuk1
U+6EC0.0	fMandarin	chu4; xu4
U+6EC0.0	fEnglish	[1] water flowing [2] foster [3] tolerate
U+6EC1.0	fUTF8	滁
U+6EC1.0	fCNS11643	2-4437
U+6EC1.0	fBig5	DE51
U+6EC1.0	fGB	0-337C
U+6EC1.0	fR/S	 85.10
U+6EC1.0	fTotalStrokes	 13
U+6EC1.0	fCangjie	ENLD
U+6EC1.0	fFourCorner	38194
U+6EC1.0	fHakka	 chu2; cu2
U+6EC1.0	fCantonese	cyu4; ceoi4; cyu1
U+6EC1.0	fMandarin	chu2
U+6EC1.0	fEnglish	the name of a tributary of the Yangtze river
U+6EC2.0	fUTF8	滂
U+6EC2.0	fCNS11643	1-6462
U+6EC2.0	fBig5	B7BC
U+6EC2.0	fGB	0-6468
U+6EC2.0	fR/S	 85.10
U+6EC2.0	fTotalStrokes	 13
U+6EC2.0	fCangjie	EYBS
U+6EC2.0	fFourCorner	30127
U+6EC2.0	fHakka	  pong2; pong1
U+6EC2.0	fCantonese	pong4; pong1
U+6EC2.0	fMandarin	pang1; peng1
U+6EC2.0	fEnglish	torrential; overwhelming
U+6EC3.0	fUTF8	滃
U+6EC3.0	fCNS11643	2-4441
U+6EC3.0	fBig5	DE5B
U+6EC3.0	fGB	3-5557
U+6EC3.0	fR/S	 85.10
U+6EC3.0	fTotalStrokes	 13
U+6EC3.0	fCangjie	ECIM
U+6EC3.0	fFourCorner	38127
U+6EC3.0	fHakka	 wung3; vung3
U+6EC3.0	fCantonese	jung2; jung1
U+6EC3.0	fMandarin	weng3; weng1
U+6EC3.0	fEnglish	swelling and rising (rivers, clouds)
U+6EC4.0	fUTF8	滄
U+6EC4.0	fCNS11643	1-646F
U+6EC4.0	fBig5	B7C9
U+6EC4.0	fGB	1-3257
U+6EC4.0	fR/S	 85.10
U+6EC4.0	fTotalStrokes	 13
U+6EC4.0	fCangjie	EOIR
U+6EC4.0	fFourCorner	38167
U+6EC4.0	fHakka	  cong1
U+6EC4.0	fCantonese	cong1
U+6EC4.0	fMandarin	cang1; chuang4
U+6EC4.0	fEnglish	blue; azure; green
U+6EC5.0	fUTF8	滅
U+6EC5.0	fCNS11643	1-6466
U+6EC5.0	fBig5	B7C0
U+6EC5.0	fGB	1-4370
U+6EC5.0	fR/S	 85.10
U+6EC5.0	fTotalStrokes	 13
U+6EC5.0	fCangjie	EIHF
U+6EC5.0	fFourCorner	33150
U+6EC5.0	fHakka	 met8;  miet8
U+6EC5.0	fCantonese	mit6
U+6EC5.0	fMandarin	mie4
U+6EC5.0	fEnglish	[1] destroy; ruin; extinguish; exterminate [2] put out; extinguish; go out
U+6EC6.0	fUTF8	滆
U+6EC6.0	fCNS11643	2-4434
U+6EC6.0	fBig5	DE4E
U+6EC6.0	fGB	3-554E
U+6EC6.0	fR/S	 85.10
U+6EC6.0	fTotalStrokes	 13
U+6EC6.0	fCangjie	EMRB
U+6EC6.0	fFourCorner	31127
U+6EC6.0	fHakka	 het7
U+6EC6.0	fCantonese	gaak3; gak3
U+6EC6.0	fMandarin	he2; ge2
U+6EC6.0	fEnglish	a small lake in the East of Kiangsi
U+6EC7.0	fUTF8	滇
U+6EC7.0	fCNS11643	1-6465
U+6EC7.0	fBig5	B7BF
U+6EC7.0	fGB	0-3561
U+6EC7.0	fR/S	 85.10
U+6EC7.0	fTotalStrokes	 13
U+6EC7.0	fCangjie	EJBC
U+6EC7.0	fFourCorner	34181
U+6EC7.0	fHakka	  dien1;  tien2; ten2
U+6EC7.0	fCantonese	tin4; din1
U+6EC7.0	fMandarin	dian1; tian2; zhen1; tian4; zhen4
U+6EC7.0	fEnglish	another name for Yunnan province
U+6EC8.0	fUTF8	滈
U+6EC8.0	fCNS11643	2-442B
U+6EC8.0	fBig5	DE45
U+6EC8.0	fGB	3-555A
U+6EC8.0	fR/S	 85.10
U+6EC8.0	fTotalStrokes	 13
U+6EC8.0	fCangjie	EYRB
U+6EC8.0	fFourCorner	30127
U+6EC8.0	fHakka	 hau3
U+6EC8.0	fCantonese	hou6
U+6EC8.0	fMandarin	hao4; hou4; huo4; xiao1
U+6EC8.0	fEnglish	[1] rippling or bubbling of water [2] a river in Shensi
U+6EC9.0	fUTF8	滉
U+6EC9.0	fCNS11643	2-4439
U+6EC9.0	fBig5	DE53
U+6EC9.0	fGB	3-5550
U+6EC9.0	fR/S	 85.10
U+6EC9.0	fTotalStrokes	 13
U+6EC9.0	fCangjie	EAFU
U+6EC9.0	fFourCorner	36111
U+6EC9.0	fHakka	 fong3
U+6EC9.0	fCantonese	fong2
U+6EC9.0	fMandarin	huang3; huang4
U+6EC9.0	fEnglish	deep
U+6ECA.0	fUTF8	滊
U+6ECA.0	fCNS11643	2-444D
U+6ECA.0	fBig5	DE67
U+6ECA.0	fGB	E-306A
U+6ECA.0	fR/S	 85.10
U+6ECA.0	fTotalStrokes	 13
U+6ECA.0	fCangjie	EOND
U+6ECA.0	fFourCorner	38117
U+6ECA.0	fCantonese	hei3; hit3; kit3
U+6ECA.0	fMandarin	xi4; xie5
U+6ECA.0	fEnglish	[assumed variant]
U+6ECB.0	fUTF8	滋
U+6ECB.0	fCNS11643	1-6045
U+6ECB.0	fBig5	B4FE
U+6ECB.0	fGB	0-574C
U+6ECB.0	fR/S	 85.10; 85.9
U+6ECB.0	fTotalStrokes	 12
U+6ECB.0	fCangjie	ETVI
U+6ECB.0	fFourCorner	38132
U+6ECB.0	fHakka	 zih1; zu1; zii1
U+6ECB.0	fCantonese	zi1
U+6ECB.0	fMandarin	zi1; ci2
U+6ECB.0	fEnglish	[1] grow [2] multiply; increase [3] nourish [4] give rise to [5] spurt out [6] juice; sap
U+6ECC.0	fUTF8	滌
U+6ECC.0	fCNS11643	1-6957
U+6ECC.0	fBig5	BAB0
U+6ECC.0	fGB	1-3553
U+6ECC.0	fR/S	 85.11
U+6ECC.0	fTotalStrokes	 14
U+6ECC.0	fCangjie	EOLD
U+6ECC.0	fFourCorner	38194
U+6ECC.0	fHakka	 tit8; tit7
U+6ECC.0	fCantonese	dik6
U+6ECC.0	fMandarin	di2; diao4
U+6ECC.0	fEnglish	[1] wash; cleanse [2] sweep
U+6ECD.0	fUTF8	滍
U+6ECD.0	fCNS11643	2-443C
U+6ECD.0	fBig5	DE56
U+6ECD.0	fGB	3-5564
U+6ECD.0	fR/S	 85.10
U+6ECD.0	fTotalStrokes	 13
U+6ECD.0	fCangjie	EUMI
U+6ECD.0	fFourCorner	32136
U+6ECD.0	fCantonese	zi6; zi3; ci5
U+6ECD.0	fMandarin	zhi4; zhi3
U+6ECE.0	fUTF8	滎
U+6ECE.0	fCNS11643	2-4B34
U+6ECE.0	fBig5	E26C
U+6ECE.0	fGB	1-5C7E
U+6ECE.0	fR/S	 85.10
U+6ECE.0	fTotalStrokes	 14; 13
U+6ECE.0	fCangjie	FFBE
U+6ECE.0	fFourCorner	99232
U+6ECE.0	fHakka	 jin2; rin2
U+6ECE.0	fCantonese	jing4
U+6ECE.0	fMandarin	xing2; ying2; ying1; ying3; ying4
U+6ECE.0	fEnglish	rising (waves)
U+6ECF.0	fUTF8	滏
U+6ECF.0	fCNS11643	2-443E
U+6ECF.0	fBig5	DE58
U+6ECF.0	fGB	0-6466
U+6ECF.0	fR/S	 85.10
U+6ECF.0	fTotalStrokes	 13
U+6ECF.0	fCangjie	ECKC
U+6ECF.0	fFourCorner	38119
U+6ECF.0	fCantonese	fu2
U+6ECF.0	fMandarin	fu3
U+6ED0.0	fUTF8	滐
U+6ED0.0	fCNS11643	2-444C
U+6ED0.0	fBig5	DE66
U+6ED0.0	fGB	5-564E
U+6ED0.0	fR/S	 85.10
U+6ED0.0	fTotalStrokes	 13
U+6ED0.0	fCangjie	ENQD
U+6ED0.0	fFourCorner	35194
U+6ED0.0	fCantonese	git6
U+6ED0.0	fMandarin	jie2
U+6ED0.0	fEnglish	dense, thick, viscous
U+6ED1.0	fUTF8	滑
U+6ED1.0	fCNS11643	1-646C
U+6ED1.0	fBig5	B7C6
U+6ED1.0	fGB	0-3B2C
U+6ED1.0	fR/S	 85.10
U+6ED1.0	fTotalStrokes	 13
U+6ED1.0	fCangjie	EBBB
U+6ED1.0	fFourCorner	37127
U+6ED1.0	fHakka	  wat8;  gut7; gwut7; vat8
U+6ED1.0	fCantonese	waat6; gwat1; waat2 (also)
U+6ED1.0	fMandarin	hua2; gu3
U+6ED1.0	fEnglish	[1] slip; slide; glide [2] smooth; slippery [3] insincere; dishonest; cunning
U+6ED1.1	fUTF8	滑
U+6ED1.1	fCNS11643	1-646C
U+6ED1.1	fBig5	B7C6
U+6ED1.1	fGB	0-3B2C
U+6ED1.1	fR/S	 85.10
U+6ED1.1	fTotalStrokes	 13
U+6ED1.1	fCangjie	EBBB
U+6ED1.1	fFourCorner	37127
U+6ED1.1	fMandarin	hua5
U+6ED2.0	fUTF8	滒
U+6ED2.0	fCNS11643	2-4435
U+6ED2.0	fBig5	DE4F
U+6ED2.0	fGB	3-554D
U+6ED2.0	fR/S	 85.10
U+6ED2.0	fTotalStrokes	 13
U+6ED2.0	fCangjie	EMRR
U+6ED2.0	fFourCorner	31121
U+6ED2.0	fCantonese	go1
U+6ED2.0	fMandarin	ge1
U+6ED3.0	fUTF8	滓
U+6ED3.0	fCNS11643	1-6460
U+6ED3.0	fBig5	B7BA
U+6ED3.0	fGB	0-5752
U+6ED3.0	fR/S	 85.10
U+6ED3.0	fTotalStrokes	 13
U+6ED3.0	fCangjie	EJYJ
U+6ED3.0	fFourCorner	33141
U+6ED3.0	fHakka	  zai3
U+6ED3.0	fCantonese	zi2
U+6ED3.0	fMandarin	zi3
U+6ED3.0	fEnglish	 sediment; lees; dregs
U+6ED4.0	fUTF8	滔
U+6ED4.0	fCNS11643	1-6470
U+6ED4.0	fBig5	B7CA
U+6ED4.0	fGB	0-4C4F
U+6ED4.0	fR/S	 85.10
U+6ED4.0	fTotalStrokes	 13
U+6ED4.0	fCangjie	EBHX
U+6ED4.0	fFourCorner	32177
U+6ED4.0	fHakka	  tau1;  to1
U+6ED4.0	fCantonese	tou1
U+6ED4.0	fMandarin	tao1; tao2
U+6ED4.0	fEnglish	[1] fluent [2] fill; prevail
U+6ED5.0	fUTF8	滕
U+6ED5.0	fCNS11643	1-6D5A
U+6ED5.0	fBig5	BCF0
U+6ED5.0	fGB	0-6B78
U+6ED5.0	fR/S	 85.10
U+6ED5.0	fTotalStrokes	 15; 13
U+6ED5.0	fCangjie	BFQE
U+6ED5.0	fFourCorner	79232
U+6ED5.0	fHakka	 ten2;  tin2
U+6ED5.0	fCantonese	tang4
U+6ED5.0	fMandarin	teng2
U+6ED5.0	fEnglish	[1] name of a state in the Spring and Autumn period in today's Shantung province [2] a Chinese family name
U+6ED6.0	fUTF8	滖
U+6ED6.0	fCNS11643	2-442A
U+6ED6.0	fBig5	DE44
U+6ED6.0	fGB	E-306B
U+6ED6.0	fR/S	 85.10
U+6ED6.0	fTotalStrokes	 13
U+6ED6.0	fCangjie	EYWV
U+6ED6.0	fFourCorner	30132
U+6ED6.0	fHakka	 se5
U+6ED6.0	fCantonese	seoi1
U+6ED6.0	fMandarin	sui1; nei2
U+6ED7.0	fUTF8	滗
U+6ED7.0	fCNS11643	6-544E
U+6ED7.0	fGB	0-6464
U+6ED7.0	fR/S	  85.10
U+6ED7.0	fTotalStrokes	 13
U+6ED7.0	fCangjie	EHHU
U+6ED7.0	fFourCorner	38110
U+6ED7.0	fHakka	 bit7; {bi5}
U+6ED7.0	fCantonese	bei3; bat1
U+6ED7.0	fMandarin	bi3; bi4
U+6ED7.0	fEnglish	simplified form of &#x6F77;
U+6ED8.0	fUTF8	滘
U+6ED8.0	fCNS11643	2-4443
U+6ED8.0	fBig5	DE5D
U+6ED8.0	fGB	3-5563
U+6ED8.0	fR/S	 85.10
U+6ED8.0	fTotalStrokes	 13
U+6ED8.0	fCangjie	XEBCR
U+6ED8.0	fFourCorner	37161
U+6ED8.0	fHakka	 gau5
U+6ED8.0	fCantonese	gaau3; gau3
U+6ED8.0	fMandarin	jiao4; jiao1
U+6ED8.0	fEnglish	character used in place names
U+6ED9.0	fUTF8	滙
U+6ED9.0	fCNS11643	3-4244
U+6ED9.0	fGB	E-306C
U+6ED9.0	fR/S	  85.10
U+6ED9.0	fTotalStrokes	 13
U+6ED9.0	fCangjie	ESOG
U+6ED9.0	fFourCorner	31110
U+6ED9.0	fHakka	  fui3; fi5; fui5; fi3; wui3
U+6ED9.0	fCantonese	wui6
U+6ED9.0	fMandarin	hui4
U+6ED9.0	fEnglish	 waters coverging to one spot whirling waters
U+6EDA.0	fUTF8	滚
U+6EDA.0	fCNS11643	3-423E
U+6EDA.0	fGB	0-3976
U+6EDA.0	fR/S	 85.11
U+6EDA.0	fTotalStrokes	 13; 14
U+6EDA.0	fCangjie	EYCV
U+6EDA.0	fFourCorner	30130
U+6EDA.0	fCantonese	gwan2; kwan2
U+6EDA.0	fMandarin	gun3
U+6EDA.0	fEnglish	turn
U+6EDB.0	fUTF8	滛
U+6EDB.0	fCNS11643	3-4247
U+6EDB.0	fGB	E-306D
U+6EDB.0	fR/S	 85.10
U+6EDB.0	fTotalStrokes	 13
U+6EDB.0	fCangjie	EBOU
U+6EDB.0	fFourCorner	37170
U+6EDB.0	fCantonese	jam4
U+6EDB.0	fMandarin	yin2; yan4; yao2
U+6EDC.0	fUTF8	滜
U+6EDC.0	fCNS11643	2-4442
U+6EDC.0	fBig5	DE5C
U+6EDC.0	fGB	5-564A
U+6EDC.0	fR/S	 85.10
U+6EDC.0	fTotalStrokes	 13
U+6EDC.0	fCangjie	EHAJ
U+6EDC.0	fFourCorner	36143
U+6EDC.0	fCantonese	gou1
U+6EDC.0	fMandarin	gao1; hao4; ze4; ze2; hao2
U+6EDD.0	fUTF8	滝
U+6EDD.0	fCNS11643	3-423D
U+6EDD.0	fGB	E-306E
U+6EDD.0	fR/S	 85.10
U+6EDD.0	fTotalStrokes	 13
U+6EDD.0	fCangjie	EYTU
U+6EDD.0	fFourCorner	30110
U+6EDD.0	fCantonese	lung4
U+6EDD.0	fMandarin	long2; shuang1; shuan1
U+6EDD.0	fEnglish	 raining wet soaked waterful
U+6EDE.0	fUTF8	滞
U+6EDE.0	fGB	0-564D
U+6EDE.0	fR/S	 85.9; 85.10
U+6EDE.0	fTotalStrokes	 12
U+6EDE.0	fCangjie	ETJB
U+6EDE.0	fHakka	 che5
U+6EDE.0	fCantonese	zai6
U+6EDE.0	fMandarin	zhi4; chi4
U+6EDE.0	fEnglish	simplified form of &#x6EEF;
U+6EDF.0	fUTF8	滟
U+6EDF.0	fGB	0-6459
U+6EDF.0	fR/S	 85.10
U+6EDF.0	fTotalStrokes	 13
U+6EDF.0	fCangjie	EQJU
U+6EDF.0	fCantonese	jim6
U+6EDF.0	fMandarin	yan4
U+6EDF.0	fEnglish	simplified form of &#x7067;
U+6EE0.0	fUTF8	滠
U+6EE0.0	fGB	0-645C
U+6EE0.0	fR/S	 85.10
U+6EE0.0	fTotalStrokes	 13
U+6EE0.0	fCangjie	ESJE
U+6EE0.0	fHakka	 ngiap7
U+6EE0.0	fCantonese	sip3
U+6EE0.0	fMandarin	she4; nie4
U+6EE0.0	fEnglish	simplified form of &#x7044;
U+6EE1.0	fUTF8	满
U+6EE1.0	fGB	0-427A
U+6EE1.0	fR/S	 85.10
U+6EE1.0	fTotalStrokes	 13
U+6EE1.0	fCangjie	ETMB
U+6EE1.0	fHakka	 man1
U+6EE1.0	fCantonese	mun5; mun2 (also)
U+6EE1.0	fMandarin	man3; men4
U+6EE1.0	fEnglish	simplified form of &#x6EFF;
U+6EE2.0	fUTF8	滢
U+6EE2.0	fGB	0-645E
U+6EE2.0	fR/S	 85.10
U+6EE2.0	fTotalStrokes	 13
U+6EE2.0	fCangjie	ETBI
U+6EE2.0	fCantonese	jing4; jing6 (also)
U+6EE2.0	fMandarin	ying4; ying3; ying2; jiong1; jing3
U+6EE2.0	fEnglish	simplified form of &#x7005;
U+6EE3.0	fUTF8	滣
U+6EE3.0	fCNS11643	3-4245
U+6EE3.0	fGB	5-5643
U+6EE3.0	fR/S	  85.10
U+6EE3.0	fTotalStrokes	 13
U+6EE3.0	fCangjie	EMVR
U+6EE3.0	fFourCorner	31160
U+6EE3.0	fCantonese	seon4
U+6EE3.0	fMandarin	chun2
U+6EE4.0	fUTF8	滤
U+6EE4.0	fGB	0-424B
U+6EE4.0	fR/S	 85.10
U+6EE4.0	fTotalStrokes	 13
U+6EE4.0	fCangjie	XEYPP
U+6EE4.0	fHakka	 li5
U+6EE4.0	fCantonese	leoi6
U+6EE4.0	fMandarin	luu4; l4
U+6EE4.0	fEnglish	simplified form of &#x6FFE;
U+6EE5.0	fUTF8	滥
U+6EE5.0	fGB	0-4044
U+6EE5.0	fR/S	 85.10
U+6EE5.0	fTotalStrokes	 13
U+6EE5.0	fCangjie	ELIT
U+6EE5.0	fHakka	 lam5; lam3
U+6EE5.0	fCantonese	laam6; laam5 (also)
U+6EE5.0	fMandarin	lan4; dan4; jian4; lan3; lan2; xian4
U+6EE5.0	fEnglish	simplified form of &#x6FEB;
U+6EE6.0	fUTF8	滦
U+6EE6.0	fCNS11643	3-423F
U+6EE6.0	fGB	0-4250
U+6EE6.0	fR/S	 85.10
U+6EE6.0	fTotalStrokes	 13
U+6EE6.0	fCangjie	EYCD
U+6EE6.0	fFourCorner	30190
U+6EE6.0	fHakka	 lan2
U+6EE6.0	fCantonese	lyun4
U+6EE6.0	fMandarin	luan2
U+6EE6.0	fEnglish	simplified form of &#x7064;
U+6EE7.0	fUTF8	滧
U+6EE7.0	fGB	8-2F6C
U+6EE7.0	fR/S	 85.10
U+6EE7.0	fTotalStrokes	 13
U+6EE7.0	fCangjie	EYKK
U+6EE7.0	fCantonese	haau6
U+6EE7.0	fMandarin	xiao4
U+6EE8.0	fUTF8	滨
U+6EE8.0	fCNS11643	3-423B
U+6EE8.0	fGB	0-3175
U+6EE8.0	fR/S	 85.10
U+6EE8.0	fTotalStrokes	 13
U+6EE8.0	fCangjie	EJOC
U+6EE8.0	fFourCorner	33180
U+6EE8.0	fCantonese	ban1
U+6EE8.0	fMandarin	bin1; pin2
U+6EE8.0	fEnglish	simplified form of &#x6FF1;
U+6EE9.0	fUTF8	滩
U+6EE9.0	fCNS11643	3-4246
U+6EE9.0	fGB	0-4C32
U+6EE9.0	fR/S	 85.10
U+6EE9.0	fTotalStrokes	 13
U+6EE9.0	fCangjie	EEOG
U+6EE9.0	fFourCorner	30110
U+6EE9.0	fHakka	 tan1
U+6EE9.0	fCantonese	taan1
U+6EE9.0	fMandarin	tan1; han3; nan4; tan4
U+6EE9.0	fEnglish	simplified form of &#x7058;
U+6EEA.0	fUTF8	滪
U+6EEA.0	fGB	8-7D2D
U+6EEA.0	fR/S	 85.10; 85.13
U+6EEA.0	fTotalStrokes	 13
U+6EEA.0	fCangjie	ENNO
U+6EEA.0	fCantonese	jyu6
U+6EEA.0	fMandarin	yu4
U+6EEA.0	fEnglish	a place in Sichuan
U+6EEB.0	fUTF8	滫
U+6EEB.0	fCNS11643	2-4B50
U+6EEB.0	fBig5	E2AA
U+6EEB.0	fGB	3-5555
U+6EEB.0	fR/S	 85.11
U+6EEB.0	fTotalStrokes	 14
U+6EEB.0	fCangjie	EOLB
U+6EEB.0	fFourCorner	37127
U+6EEB.0	fHakka	  siu1; siu3
U+6EEB.0	fCantonese	sau1; sau2 (also); sau3 (also)
U+6EEB.0	fMandarin	xiu1; xiu3; sou3; xiu4
U+6EEB.0	fEnglish	water in which rice has been boiled
U+6EEC.0	fUTF8	滬
U+6EEC.0	fCNS11643	1-6954
U+6EEC.0	fBig5	BAAD
U+6EEC.0	fGB	1-3B26
U+6EEC.0	fR/S	 85.11
U+6EEC.0	fTotalStrokes	 14
U+6EEC.0	fCangjie	EHSU
U+6EEC.0	fFourCorner	33117
U+6EEC.0	fHakka	  fu5; fu3; fu6
U+6EEC.0	fCantonese	wu6
U+6EEC.0	fMandarin	hu4
U+6EEC.0	fEnglish	another name for Shanghai
U+6EED.0	fUTF8	滭
U+6EED.0	fCNS11643	2-4B45
U+6EED.0	fBig5	E27D
U+6EED.0	fGB	3-537C
U+6EED.0	fR/S	 85.11
U+6EED.0	fTotalStrokes	 14
U+6EED.0	fCangjie	EWTJ
U+6EED.0	fFourCorner	36154
U+6EED.0	fHakka	  bit7
U+6EED.0	fCantonese	bat1
U+6EED.0	fMandarin	bi4
U+6EED.0	fEnglish	bubbling water
U+6EEE.0	fUTF8	滮
U+6EEE.0	fCNS11643	2-4B4A
U+6EEE.0	fBig5	E2A4
U+6EEE.0	fGB	3-556C
U+6EEE.0	fR/S	 85.11
U+6EEE.0	fTotalStrokes	 14
U+6EEE.0	fCangjie	EYUH
U+6EEE.0	fFourCorner	32112
U+6EEE.0	fHakka	 piu3;  biau1; piu2; beu1
U+6EEE.0	fCantonese	biu1; piu4
U+6EEE.0	fMandarin	biao1
U+6EEE.0	fEnglish	flow
U+6EEF.0	fUTF8	滯
U+6EEF.0	fCNS11643	1-6949
U+6EEF.0	fBig5	BAA2
U+6EEF.0	fGB	1-564D
U+6EEF.0	fR/S	 85.11
U+6EEF.0	fTotalStrokes	 14
U+6EEF.0	fCangjie	EKPB
U+6EEF.0	fFourCorner	34127
U+6EEF.0	fHakka	 ce5; che5; ts'ie5; tch'e5
U+6EEF.0	fCantonese	zai6
U+6EEF.0	fMandarin	zhi4; chi4
U+6EEF.0	fEnglish	at a standstill; blocked; obstructed; stagnant; impeded; stationary
U+6EF0.0	fUTF8	滰
U+6EF0.0	fCNS11643	3-4756
U+6EF0.0	fGB	3-5579
U+6EF0.0	fR/S	  85.11
U+6EF0.0	fTotalStrokes	 14
U+6EF0.0	fCangjie	EYTU
U+6EF0.0	fFourCorner	30110
U+6EF0.0	fCantonese	koeng5
U+6EF0.0	fMandarin	jiang3
U+6EF1.0	fUTF8	滱
U+6EF1.0	fCNS11643	2-4B36
U+6EF1.0	fBig5	E26E
U+6EF1.0	fGB	3-557D
U+6EF1.0	fR/S	 85.11
U+6EF1.0	fTotalStrokes	 14
U+6EF1.0	fCangjie	EJME
U+6EF1.0	fFourCorner	33114
U+6EF1.0	fHakka	 kiu5
U+6EF1.0	fCantonese	kau3
U+6EF1.0	fMandarin	kou4; kou1
U+6EF2.0	fUTF8	滲
U+6EF2.0	fCNS11643	1-6956
U+6EF2.0	fBig5	BAAF
U+6EF2.0	fGB	1-4978
U+6EF2.0	fR/S	 85.11
U+6EF2.0	fTotalStrokes	 14
U+6EF2.0	fCangjie	EIIH
U+6EF2.0	fFourCorner	33122
U+6EF2.0	fHakka	  cam1; zam5;  sim5; sem1; sip8; sep8; cam3
U+6EF2.0	fCantonese	sam3
U+6EF2.0	fMandarin	shen4; lin2; sen1; qin1; qin4
U+6EF2.0	fEnglish	 permeate; percolate; seep; ooze; soak; infiltrate
U+6EF3.0	fUTF8	滳
U+6EF3.0	fCNS11643	3-4757
U+6EF3.0	fGB	3-557A
U+6EF3.0	fR/S	  85.11
U+6EF3.0	fTotalStrokes	 14
U+6EF3.0	fCangjie	EYCB
U+6EF3.0	fFourCorner	30120
U+6EF3.0	fCantonese	soeng1
U+6EF3.0	fMandarin	shang1
U+6EF4.0	fUTF8	滴
U+6EF4.0	fCNS11643	1-6940
U+6EF4.0	fBig5	BA77
U+6EF4.0	fGB	0-354E
U+6EF4.0	fR/S	 85.11
U+6EF4.0	fTotalStrokes	 14
U+6EF4.0	fCangjie	EYCB
U+6EF4.0	fFourCorner	30127
U+6EF4.0	fHakka	  dit7; din5; dit8; sit7
U+6EF4.0	fCantonese	dik1 (also); dik6
U+6EF4.0	fMandarin	di1
U+6EF4.0	fEnglish	[1] drip [2] drop of water; water drops
U+6EF5.0	fUTF8	滵
U+6EF5.0	fCNS11643	2-4B35
U+6EF5.0	fBig5	E26D
U+6EF5.0	fGB	3-5622
U+6EF5.0	fR/S	 85.11
U+6EF5.0	fTotalStrokes	 14
U+6EF5.0	fCangjie	EJPU
U+6EF5.0	fFourCorner	33173
U+6EF5.0	fCantonese	mat6
U+6EF5.0	fMandarin	mi4
U+6EF6.0	fUTF8	滶
U+6EF6.0	fCNS11643	2-4B56
U+6EF6.0	fBig5	E2B0
U+6EF6.0	fGB	3-5548
U+6EF6.0	fR/S	 85.11
U+6EF6.0	fTotalStrokes	 14
U+6EF6.0	fCangjie	EGSK
U+6EF6.0	fFourCorner	38140
U+6EF6.0	fCantonese	ngou4; ou4 (also)
U+6EF6.0	fMandarin	ao2
U+6EF7.0	fUTF8	滷
U+6EF7.0	fCNS11643	1-6958
U+6EF7.0	fBig5	BAB1
U+6EF7.0	fGB	1-4231
U+6EF7.0	fR/S	 85.11
U+6EF7.0	fTotalStrokes	 14
U+6EF7.0	fCangjie	EYWI
U+6EF7.0	fFourCorner	31160
U+6EF7.0	fHakka	  lu1;  lu3
U+6EF7.0	fCantonese	lou5
U+6EF7.0	fMandarin	lu3; chi3; di2
U+6EF7.0	fEnglish	[1] gravy; sauce; broth [2] salty; salted
U+6EF8.0	fUTF8	滸
U+6EF8.0	fCNS11643	2-4B39
U+6EF8.0	fBig5	E271
U+6EF8.0	fGB	1-6430
U+6EF8.0	fR/S	 85.11
U+6EF8.0	fTotalStrokes	 14
U+6EF8.0	fCangjie	EYRJ
U+6EF8.0	fFourCorner	38140
U+6EF8.0	fHakka	  fu3
U+6EF8.0	fCantonese	wu2; heoi2; fu2
U+6EF8.0	fMandarin	hu3; xu3
U+6EF8.0	fEnglish	 riverbank; shore; water side
U+6EF9.0	fUTF8	滹
U+6EF9.0	fCNS11643	2-4B49
U+6EF9.0	fBig5	E2A3
U+6EF9.0	fGB	0-646F
U+6EF9.0	fR/S	 85.11
U+6EF9.0	fTotalStrokes	 14
U+6EF9.0	fCangjie	EYPD
U+6EF9.0	fFourCorner	31149
U+6EF9.0	fHakka	 fu1
U+6EF9.0	fCantonese	fu1
U+6EF9.0	fMandarin	hu1; hu3
U+6EF9.0	fEnglish	a river flowing through Hopeh
U+6EFA.0	fUTF8	滺
U+6EFA.0	fCNS11643	3-4765
U+6EFA.0	fGB	5-5660
U+6EFA.0	fR/S	  85.11
U+6EFA.0	fTotalStrokes	 14
U+6EFA.0	fCangjie	EOKP
U+6EFA.0	fFourCorner	38130
U+6EFA.0	fHakka	 ju2; riu2
U+6EFA.0	fCantonese	jau4; dik6 (also)
U+6EFA.0	fMandarin	you2; di2
U+6EFA.0	fEnglish	flow
U+6EFB.0	fUTF8	滻
U+6EFB.0	fCNS11643	2-4B3B
U+6EFB.0	fBig5	E273
U+6EFB.0	fGB	3-542A
U+6EFB.0	fR/S	 85.11
U+6EFB.0	fTotalStrokes	 14
U+6EFB.0	fCangjie	EYHM
U+6EFB.0	fFourCorner	30115
U+6EFB.0	fHakka	  san3
U+6EFB.0	fCantonese	caan2
U+6EFB.0	fMandarin	chan3
U+6EFB.0	fEnglish	a small stream in Shensi
U+6EFC.0	fUTF8	滼
U+6EFC.0	fCNS11643	2-4B59
U+6EFC.0	fBig5	E2B3
U+6EFC.0	fGB	3-5569
U+6EFC.0	fR/S	 85.11
U+6EFC.0	fTotalStrokes	 14
U+6EFC.0	fCangjie	EDDN
U+6EFC.0	fFourCorner	34117
U+6EFC.0	fCantonese	paan3; faan6
U+6EFC.0	fMandarin	fan4
U+6EFD.0	fUTF8	滽
U+6EFD.0	fCNS11643	2-4B55
U+6EFD.0	fBig5	E2AF
U+6EFD.0	fGB	3-5577
U+6EFD.0	fR/S	 85.11
U+6EFD.0	fTotalStrokes	 14
U+6EFD.0	fCangjie	EILB
U+6EFD.0	fFourCorner	30127
U+6EFD.0	fCantonese	jung4
U+6EFD.0	fMandarin	yong2; yong1; rong2
U+6EFE.0	fUTF8	滾
U+6EFE.0	fCNS11643	1-693E
U+6EFE.0	fBig5	BA75
U+6EFE.0	fGB	E-306F
U+6EFE.0	fR/S	 85.11
U+6EFE.0	fTotalStrokes	 14
U+6EFE.0	fCangjie	EYCV
U+6EFE.0	fFourCorner	30132
U+6EFE.0	fHakka	  kun3; gun3; kun2; gwun3
U+6EFE.0	fCantonese	gwan2; kwan2
U+6EFE.0	fMandarin	gun3; gun4
U+6EFE.0	fEnglish	[1] turn round and round; roll; rotate [2] boiling
U+6EFF.0	fUTF8	滿
U+6EFF.0	fCNS11643	1-6948
U+6EFF.0	fBig5	BAA1
U+6EFF.0	fGB	1-427A
U+6EFF.0	fR/S	 85.11
U+6EFF.0	fTotalStrokes	 14
U+6EFF.0	fCangjie	ETLB
U+6EFF.0	fFourCorner	34127
U+6EFF.0	fHakka	  man1; man2; man3; mang1; nem1
U+6EFF.0	fCantonese	mun5; mun2 (also); mun6
U+6EFF.0	fMandarin	man3; men4
U+6EFF.0	fEnglish	[1] filled; full [2] plentiful; abundant [3] proud; haughty [4] expire [5] completely; entirely; perfectly [6] Manchu [7] a Chinese family name
U+6F00.0	fUTF8	漀
U+6F00.0	fCNS11643	2-515B
U+6F00.0	fBig5	E653
U+6F00.0	fGB	3-532B
U+6F00.0	fR/S	 85.11
U+6F00.0	fTotalStrokes	 15; 14
U+6F00.0	fCangjie	GEE
U+6F00.0	fFourCorner	47232
U+6F00.0	fHakka	 kin5
U+6F00.0	fCantonese	hing2; hing1; king1; king2
U+6F00.0	fMandarin	qing4; qing1
U+6F00.0	fEnglish	[1] stream flowing from the side of a hill [2] pour out
U+6F01.0	fUTF8	漁
U+6F01.0	fCNS11643	1-6955
U+6F01.0	fBig5	BAAE
U+6F01.0	fGB	1-5366
U+6F01.0	fR/S	 85.11
U+6F01.0	fTotalStrokes	 14
U+6F01.0	fCangjie	ENWF
U+6F01.0	fFourCorner	37136
U+6F01.0	fHakka	  ng2; ng3
U+6F01.0	fCantonese	jyu4
U+6F01.0	fMandarin	yu2; yu4
U+6F01.0	fEnglish	[1] fish [2] seek; pursue [3] seize; acquire forcibly
U+6F02.0	fUTF8	漂
U+6F02.0	fCNS11643	1-6946
U+6F02.0	fBig5	BA7D
U+6F02.0	fGB	0-462F
U+6F02.0	fR/S	 85.11
U+6F02.0	fTotalStrokes	 14
U+6F02.0	fCangjie	EMWF
U+6F02.0	fFourCorner	31191
U+6F02.0	fHakka	  piau5; piu5; piau2; peu5; piau3; peu1; peu2
U+6F02.0	fCantonese	piu1; piu3
U+6F02.0	fMandarin	piao1; biao1
U+6F02.0	fEnglish	 float; drift; be tossed about
U+6F02.1	fUTF8	漂
U+6F02.1	fCNS11643	1-6946
U+6F02.1	fBig5	BA7D
U+6F02.1	fGB	0-462F
U+6F02.1	fR/S	 85.11
U+6F02.1	fTotalStrokes	 14
U+6F02.1	fCangjie	EMWF
U+6F02.1	fFourCorner	31191
U+6F02.1	fHakka	 biau1
U+6F02.1	fMandarin	piao3
U+6F02.1	fEnglish	 bleach
U+6F02.2	fUTF8	漂
U+6F02.2	fCNS11643	1-6946
U+6F02.2	fBig5	BA7D
U+6F02.2	fGB	0-462F
U+6F02.2	fR/S	 85.11
U+6F02.2	fTotalStrokes	 14
U+6F02.2	fCangjie	EMWF
U+6F02.2	fFourCorner	31191
U+6F02.2	fMandarin	piao4
U+6F02.2	fEnglish	pretty; nice; sleek
U+6F03.0	fUTF8	漃
U+6F03.0	fCNS11643	2-4B37
U+6F03.0	fBig5	E26F
U+6F03.0	fGB	3-557E
U+6F03.0	fR/S	 85.11
U+6F03.0	fTotalStrokes	 14
U+6F03.0	fCangjie	EJYE
U+6F03.0	fFourCorner	33147
U+6F03.0	fCantonese	zik6
U+6F03.0	fMandarin	ji2
U+6F04.0	fUTF8	漄
U+6F04.0	fCNS11643	3-4763
U+6F04.0	fGB	E-3070
U+6F04.0	fR/S	  85.11
U+6F04.0	fTotalStrokes	 14
U+6F04.0	fCangjie	EUMG
U+6F04.0	fFourCorner	32110
U+6F04.0	fHakka	 ngai2
U+6F04.0	fCantonese	ngaai4; aai4 (also)
U+6F04.0	fMandarin	ya2
U+6F05.0	fUTF8	漅
U+6F05.0	fCNS11643	2-4B54
U+6F05.0	fBig5	E2AE
U+6F05.0	fGB	5-5665
U+6F05.0	fR/S	 85.11
U+6F05.0	fTotalStrokes	 14
U+6F05.0	fCangjie	EVVD
U+6F05.0	fFourCorner	32194
U+6F05.0	fHakka	 cau2; ziau3
U+6F05.0	fCantonese	ziu2; caau4
U+6F05.0	fMandarin	jiao3; chao2; lou3; suo3
U+6F05.0	fEnglish	a lake in Anhui which produces gold-fish
U+6F06.0	fUTF8	漆
U+6F06.0	fCNS11643	1-694A
U+6F06.0	fBig5	BAA3
U+6F06.0	fGB	0-4661
U+6F06.0	fR/S	 85.11
U+6F06.0	fTotalStrokes	 14
U+6F06.0	fCangjie	EDOE
U+6F06.0	fFourCorner	34132
U+6F06.0	fHakka	  cit7; zhat7
U+6F06.0	fCantonese	cat1; cit3
U+6F06.0	fMandarin	qi1; ci4; qie1
U+6F06.0	fEnglish	[1] the varnish tree; lacquer tree [2] varnish; lacquer
U+6F06.1	fUTF8	漆
U+6F06.1	fCNS11643	1-694A
U+6F06.1	fBig5	BAA3
U+6F06.1	fGB	0-4661
U+6F06.1	fR/S	 85.11
U+6F06.1	fTotalStrokes	 14
U+6F06.1	fCangjie	EDOE
U+6F06.1	fFourCorner	34132
U+6F06.1	fMandarin	qu4
U+6F06.1	fEnglish	pitch-black
U+6F07.0	fUTF8	漇
U+6F07.0	fCNS11643	2-4B51
U+6F07.0	fBig5	E2AB
U+6F07.0	fGB	3-5574
U+6F07.0	fR/S	 85.11
U+6F07.0	fTotalStrokes	 14
U+6F07.0	fCangjie	EHOO
U+6F07.0	fFourCorner	31181
U+6F07.0	fCantonese	saai2; saai1
U+6F07.0	fMandarin	xi3
U+6F07.0	fEnglish	waste
U+6F08.0	fUTF8	漈
U+6F08.0	fCNS11643	2-4B5E
U+6F08.0	fBig5	E2B8
U+6F08.0	fGB	3-5576
U+6F08.0	fR/S	 85.11
U+6F08.0	fTotalStrokes	 14
U+6F08.0	fCangjie	EBOF
U+6F08.0	fFourCorner	37191
U+6F08.0	fCantonese	zai3
U+6F08.0	fMandarin	ji4
U+6F08.0	fEnglish	shore; waterside
U+6F09.0	fUTF8	漉
U+6F09.0	fCNS11643	2-4B3D
U+6F09.0	fBig5	E275
U+6F09.0	fGB	0-6475
U+6F09.0	fR/S	 85.11
U+6F09.0	fTotalStrokes	 14
U+6F09.0	fCangjie	EIXP
U+6F09.0	fFourCorner	30111
U+6F09.0	fHakka	 luk7;  luk8
U+6F09.0	fCantonese	luk6
U+6F09.0	fMandarin	lu4
U+6F09.0	fEnglish	[1] remove sediments by filtering [2] wet; dripping
U+6F0A.0	fUTF8	漊
U+6F0A.0	fCNS11643	2-4B46
U+6F0A.0	fBig5	E27E
U+6F0A.0	fGB	3-553E
U+6F0A.0	fR/S	 85.11
U+6F0A.0	fTotalStrokes	 14
U+6F0A.0	fCangjie	ELLV
U+6F0A.0	fFourCorner	35144
U+6F0A.0	fHakka	  leu3; liu3; liu2
U+6F0A.0	fCantonese	lau4; lou4
U+6F0A.0	fMandarin	lou2; l3; luu3; lu3; qiang2
U+6F0A.0	fEnglish	a river in Hubei province
U+6F0B.0	fUTF8	漋
U+6F0B.0	fCNS11643	3-4C78
U+6F0B.0	fGB	5-5664
U+6F0B.0	fR/S	 85.12
U+6F0B.0	fTotalStrokes	 15
U+6F0B.0	fCangjie	ENLM
U+6F0B.0	fFourCorner	37110
U+6F0B.0	fCantonese	lung4
U+6F0B.0	fMandarin	long2
U+6F0C.0	fUTF8	漌
U+6F0C.0	fCNS11643	3-475B
U+6F0C.0	fGB	5-565A
U+6F0C.0	fR/S	 85.11
U+6F0C.0	fTotalStrokes	 14
U+6F0C.0	fCangjie	ETLM
U+6F0C.0	fFourCorner	34110
U+6F0C.0	fCantonese	gan2
U+6F0C.0	fMandarin	jin3
U+6F0D.0	fUTF8	漍
U+6F0D.0	fCNS11643	2-4B5C
U+6F0D.0	fBig5	E2B6
U+6F0D.0	fGB	3-545D
U+6F0D.0	fR/S	 85.11
U+6F0D.0	fTotalStrokes	 14
U+6F0D.0	fCangjie	EWIM
U+6F0D.0	fFourCorner	36100
U+6F0D.0	fCantonese	gwok3; gok3 (also)
U+6F0D.0	fMandarin	guo2
U+6F0E.0	fUTF8	漎
U+6F0E.0	fCNS11643	2-4B52
U+6F0E.0	fBig5	E2AC
U+6F0E.0	fGB	3-534E
U+6F0E.0	fR/S	 85.11
U+6F0E.0	fTotalStrokes	 14
U+6F0E.0	fCangjie	EHOO
U+6F0E.0	fFourCorner	38181
U+6F0E.0	fCantonese	cung4; sung2
U+6F0E.0	fMandarin	cong2; song3; chuang2
U+6F0E.0	fEnglish	a place where small streams flow into a large one
U+6F0F.0	fUTF8	漏
U+6F0F.0	fCNS11643	1-6945
U+6F0F.0	fBig5	BA7C
U+6F0F.0	fGB	0-4229
U+6F0F.0	fR/S	 85.11
U+6F0F.0	fTotalStrokes	 14
U+6F0F.0	fCangjie	ESMB
U+6F0F.0	fFourCorner	37127
U+6F0F.0	fHakka	 leu6;  leu5; lieu5; liu3; leu3
U+6F0F.0	fCantonese	lau6; lau2 (also)
U+6F0F.0	fMandarin	lou4; lou2
U+6F0F.0	fEnglish	[1] leak; drip [2] hour glass; water clock [3] divulge; disclose [4] slip; omit unintentionally; neglect; forget; be missing
U+6F10.0	fUTF8	漐
U+6F10.0	fCNS11643	3-4C6A
U+6F10.0	fGB	3-5329
U+6F10.0	fR/S	 85.11
U+6F10.0	fTotalStrokes	 15; 14
U+6F10.0	fCangjie	GIE
U+6F10.0	fFourCorner	44230
U+6F10.0	fCantonese	zap6
U+6F10.0	fMandarin	zhi2
U+6F11.0	fUTF8	漑
U+6F11.0	fCNS11643	3-4764
U+6F11.0	fGB	E-3071
U+6F11.0	fR/S	 85.11
U+6F11.0	fTotalStrokes	 14
U+6F11.0	fCangjie	EHPU
U+6F11.0	fFourCorner	31110
U+6F11.0	fHakka	 kai3; koi3
U+6F11.0	fCantonese	koi3
U+6F11.0	fMandarin	gai4
U+6F11.0	fEnglish	water
U+6F12.0	fUTF8	漒
U+6F12.0	fCNS11643	2-4B44
U+6F12.0	fBig5	E27C
U+6F12.0	fGB	3-563E
U+6F12.0	fR/S	 85.11
U+6F12.0	fTotalStrokes	 14
U+6F12.0	fCangjie	ENII
U+6F12.0	fFourCorner	32137
U+6F12.0	fCantonese	koeng4
U+6F12.0	fMandarin	qiang2
U+6F13.0	fUTF8	漓
U+6F13.0	fCNS11643	1-693F
U+6F13.0	fBig5	BA76
U+6F13.0	fGB	0-406C
U+6F13.0	fR/S	 85.10; 85.11
U+6F13.0	fTotalStrokes	 13
U+6F13.0	fCangjie	EYUB
U+6F13.0	fFourCorner	30127
U+6F13.0	fHakka	 li2; li3
U+6F13.0	fCantonese	lei4
U+6F13.0	fMandarin	li2
U+6F13.0	fEnglish	[1] dripping wet [2] thin [3] simplified form of &#x7055;
U+6F14.0	fUTF8	演
U+6F14.0	fCNS11643	1-693D
U+6F14.0	fBig5	BA74
U+6F14.0	fGB	0-515D
U+6F14.0	fR/S	 85.11
U+6F14.0	fTotalStrokes	 14
U+6F14.0	fCangjie	EJMC
U+6F14.0	fFourCorner	33186
U+6F14.0	fHakka	  jen1; jen3; ren1; jan1; jen5; ren3; rin3; jin3; jan3
U+6F14.0	fCantonese	jin2; jin5 (also)
U+6F14.0	fMandarin	yan3; yan4; yin3
U+6F14.0	fEnglish	[1] perform for entertainment; act; play [2] exercise; practice [3] expound [4] evolve; develop
U+6F15.0	fUTF8	漕
U+6F15.0	fCNS11643	1-694F
U+6F15.0	fBig5	BAA8
U+6F15.0	fGB	0-646E
U+6F15.0	fR/S	 85.11
U+6F15.0	fTotalStrokes	 14
U+6F15.0	fCangjie	ETWA
U+6F15.0	fFourCorner	35166
U+6F15.0	fHakka	 cau2; co2; zau5; zo5
U+6F15.0	fCantonese	cou4
U+6F15.0	fMandarin	cao2; zao4; cao4; zou1
U+6F15.0	fEnglish	 transport grain by water
U+6F16.0	fUTF8	漖
U+6F16.0	fCNS11643	4-4443
U+6F16.0	fGB	5-5658
U+6F16.0	fR/S	  85.11
U+6F16.0	fTotalStrokes	 14
U+6F16.0	fCangjie	EJDK
U+6F16.0	fFourCorner	38140
U+6F16.0	fCantonese	gaau3
U+6F16.0	fMandarin	jiao4
U+6F16.0	fEnglish	a branching river (used in toponyms)
U+6F17.0	fUTF8	漗
U+6F17.0	fCNS11643	3-475F
U+6F17.0	fGB	E-3072
U+6F17.0	fR/S	 85.11
U+6F17.0	fTotalStrokes	 14
U+6F17.0	fCangjie	EHWP
U+6F17.0	fFourCorner	36130
U+6F17.0	fCantonese	cung1
U+6F17.0	fMandarin	cong1
U+6F18.0	fUTF8	漘
U+6F18.0	fCNS11643	2-4B42
U+6F18.0	fBig5	E27A
U+6F18.0	fGB	3-556A
U+6F18.0	fR/S	 85.11
U+6F18.0	fTotalStrokes	 14
U+6F18.0	fCangjie	EMVB
U+6F18.0	fFourCorner	31127
U+6F18.0	fHakka	 shun2; sun2; shun3; fin2
U+6F18.0	fCantonese	seon4
U+6F18.0	fMandarin	chun2; qun2
U+6F18.0	fEnglish	bank
U+6F19.0	fUTF8	漙
U+6F19.0	fCNS11643	2-4B3F
U+6F19.0	fBig5	E277
U+6F19.0	fGB	3-533E
U+6F19.0	fR/S	 85.11
U+6F19.0	fTotalStrokes	 14
U+6F19.0	fCangjie	EJII
U+6F19.0	fFourCorner	35143
U+6F19.0	fHakka	  ton2
U+6F19.0	fCantonese	tyun4
U+6F19.0	fMandarin	tuan2; chuan3; zhuan1
U+6F19.0	fEnglish	dewy
U+6F1A.0	fUTF8	漚
U+6F1A.0	fCNS11643	2-4B40
U+6F1A.0	fBig5	E278
U+6F1A.0	fGB	1-453D
U+6F1A.0	fR/S	 85.11
U+6F1A.0	fTotalStrokes	 14
U+6F1A.0	fCangjie	ESRR
U+6F1A.0	fFourCorner	31116
U+6F1A.0	fHakka	 eu5; eu1; iu5
U+6F1A.0	fCantonese	au3; au1; ngau3 (also); ngau1 (also)
U+6F1A.0	fMandarin	ou1
U+6F1A.0	fEnglish	foam; bubbles; froth
U+6F1A.1	fUTF8	漚
U+6F1A.1	fCNS11643	2-4B40
U+6F1A.1	fBig5	E278
U+6F1A.1	fGB	1-453D
U+6F1A.1	fR/S	 85.11
U+6F1A.1	fTotalStrokes	 14
U+6F1A.1	fCangjie	ESRR
U+6F1A.1	fFourCorner	31116
U+6F1A.1	fMandarin	ou4
U+6F1A.1	fEnglish	 soak
U+6F1B.0	fUTF8	漛
U+6F1B.0	fCNS11643	3-475A
U+6F1B.0	fGB	E-3073
U+6F1B.0	fR/S	  85.11
U+6F1B.0	fTotalStrokes	 14
U+6F1B.0	fCangjie	EFQE
U+6F1B.0	fFourCorner	39130
U+6F1B.0	fCantonese	tang4
U+6F1B.0	fMandarin	teng2
U+6F1C.0	fUTF8	漜
U+6F1C.0	fCNS11643	2-4B58
U+6F1C.0	fBig5	E2B2
U+6F1C.0	fGB	3-5568
U+6F1C.0	fR/S	 85.11
U+6F1C.0	fTotalStrokes	 14
U+6F1C.0	fCangjie	EDDG
U+6F1C.0	fFourCorner	34114
U+6F1C.0	fCantonese	je5
U+6F1C.0	fMandarin	ye3
U+6F1D.0	fUTF8	漝
U+6F1D.0	fCNS11643	3-475C
U+6F1D.0	fGB	3-5625
U+6F1D.0	fR/S	 85.11
U+6F1D.0	fTotalStrokes	 14
U+6F1D.0	fCangjie	ESMA
U+6F1D.0	fFourCorner	37162
U+6F1D.0	fCantonese	zaap6
U+6F1D.0	fMandarin	xi2
U+6F1E.0	fUTF8	漞
U+6F1E.0	fCNS11643	2-4B5D
U+6F1E.0	fBig5	E2B7
U+6F1E.0	fGB	5-5576
U+6F1E.0	fR/S	 85.11
U+6F1E.0	fTotalStrokes	 14
U+6F1E.0	fCangjie	EBBU
U+6F1E.0	fFourCorner	32116
U+6F1E.0	fCantonese	mik6
U+6F1E.0	fMandarin	mi4
U+6F1F.0	fUTF8	漟
U+6F1F.0	fCNS11643	2-4B5B
U+6F1F.0	fBig5	E2B5
U+6F1F.0	fGB	5-565D
U+6F1F.0	fR/S	 85.11
U+6F1F.0	fTotalStrokes	 14
U+6F1F.0	fCangjie	EFBG
U+6F1F.0	fFourCorner	39114
U+6F1F.0	fCantonese	tong4
U+6F1F.0	fMandarin	tang2
U+6F20.0	fUTF8	漠
U+6F20.0	fCNS11643	1-6943
U+6F20.0	fBig5	BA7A
U+6F20.0	fGB	0-442E
U+6F20.0	fR/S	 85.11
U+6F20.0	fTotalStrokes	 14
U+6F20.0	fCangjie	ETAK
U+6F20.0	fFourCorner	34134
U+6F20.0	fHakka	  mok7; mok8
U+6F20.0	fCantonese	mok6
U+6F20.0	fMandarin	mo4
U+6F20.0	fEnglish	[1] desert [2] unconcerned; indifferent [3] quiet; silent
U+6F21.0	fUTF8	漡
U+6F21.0	fCNS11643	2-4B5F
U+6F21.0	fBig5	E2B9
U+6F21.0	fGB	5-552C
U+6F21.0	fR/S	 85.11
U+6F21.0	fTotalStrokes	 14
U+6F21.0	fCangjie	EOAH
U+6F21.0	fFourCorner	38127
U+6F21.0	fCantonese	tong1; soeng1
U+6F21.0	fMandarin	shang1
U+6F22.0	fUTF8	漢
U+6F22.0	fCNS11643	1-6947
U+6F22.0	fBig5	BA7E
U+6F22.0	fGB	1-3A3A
U+6F22.0	fR/S	 85.11; 85.10
U+6F22.0	fTotalStrokes	 14
U+6F22.0	fCangjie	ETLO
U+6F22.0	fFourCorner	34134
U+6F22.0	fHakka	  hon5
U+6F22.0	fCantonese	hon3
U+6F22.0	fMandarin	han4; tan1
U+6F22.0	fEnglish	[1] of the Han dynasty (206 B.C.-220 A.D.) [2] of the Chinese people or language [3] fellow; man [4] a tributary of the Yangtze river
U+6F23.0	fUTF8	漣
U+6F23.0	fCNS11643	1-694E
U+6F23.0	fBig5	BAA7
U+6F23.0	fGB	1-4130
U+6F23.0	fR/S	 85.11
U+6F23.0	fTotalStrokes	 14
U+6F23.0	fCangjie	EYJJ
U+6F23.0	fFourCorner	35130
U+6F23.0	fHakka	  lien2; lian3; len2
U+6F23.0	fCantonese	lin4
U+6F23.0	fMandarin	lian2; lan2
U+6F23.0	fEnglish	[1] ripples [2] weeping
U+6F24.0	fUTF8	漤
U+6F24.0	fCNS11643	4-4444
U+6F24.0	fGB	0-646D
U+6F24.0	fR/S	 85.11
U+6F24.0	fTotalStrokes	 14
U+6F24.0	fCangjie	EDDV
U+6F24.0	fFourCorner	34140
U+6F24.0	fHakka	  lam3; len3
U+6F24.0	fCantonese	laam5; laam6 (also)
U+6F24.0	fMandarin	lan3
U+6F24.0	fEnglish	marinate in salt
U+6F25.0	fUTF8	漥
U+6F25.0	fCNS11643	2-4B38
U+6F25.0	fBig5	E270
U+6F25.0	fGB	3-5621
U+6F25.0	fR/S	 85.11
U+6F25.0	fTotalStrokes	 14
U+6F25.0	fCangjie	XEJCG
U+6F25.0	fFourCorner	33114
U+6F25.0	fHakka	  wa1; va1
U+6F25.0	fCantonese	waa1
U+6F25.0	fMandarin	wa1; ying3
U+6F25.0	fEnglish	[1] puddle; hollow [2] swamp
U+6F26.0	fUTF8	漦
U+6F26.0	fCNS11643	2-5143
U+6F26.0	fBig5	E5FA
U+6F26.0	fGB	3-4745
U+6F26.0	fR/S	 85.11
U+6F26.0	fTotalStrokes	 15; 14
U+6F26.0	fCangjie	JKME
U+6F26.0	fFourCorner	58232
U+6F26.0	fHakka	  li2; lan1; li3
U+6F26.0	fCantonese	lei4; ci4; toi1
U+6F26.0	fMandarin	li2; chan2; chi1; chi2; lai2; tai1; yi2
U+6F26.0	fEnglish	[1] saliva; spittle [2] flowing downstream
U+6F27.0	fUTF8	漧
U+6F27.0	fCNS11643	2-4B41
U+6F27.0	fBig5	E279
U+6F27.0	fGB	E-3074
U+6F27.0	fR/S	 85.11
U+6F27.0	fTotalStrokes	 14
U+6F27.0	fCangjie	EJJN
U+6F27.0	fFourCorner	38117
U+6F27.0	fCantonese	gon1
U+6F27.0	fMandarin	qian2; gan1
U+6F28.0	fUTF8	漨
U+6F28.0	fCNS11643	3-475E
U+6F28.0	fGB	3-5559
U+6F28.0	fR/S	 85.11
U+6F28.0	fTotalStrokes	 14
U+6F28.0	fCangjie	EYHJ
U+6F28.0	fFourCorner	37130
U+6F28.0	fHakka	  fung2
U+6F28.0	fCantonese	fung4
U+6F28.0	fMandarin	feng2
U+6F29.0	fUTF8	漩
U+6F29.0	fCNS11643	1-6941
U+6F29.0	fBig5	BA78
U+6F29.0	fGB	0-6476
U+6F29.0	fR/S	 85.11
U+6F29.0	fTotalStrokes	 14
U+6F29.0	fCangjie	EYSO
U+6F29.0	fFourCorner	38181
U+6F29.0	fHakka	  sien2; sion2
U+6F29.0	fCantonese	syun4
U+6F29.0	fMandarin	xuan2; xuan4
U+6F29.0	fEnglish	whirlpool
U+6F2A.0	fUTF8	漪
U+6F2A.0	fCNS11643	1-6953
U+6F2A.0	fBig5	BAAC
U+6F2A.0	fGB	0-6474
U+6F2A.0	fR/S	 85.11
U+6F2A.0	fTotalStrokes	 14
U+6F2A.0	fCangjie	EKHR
U+6F2A.0	fFourCorner	34121
U+6F2A.0	fHakka	 ji2; ji1; ri2; i1
U+6F2A.0	fCantonese	ji1; ji2 (also)
U+6F2A.0	fMandarin	yi1
U+6F2A.0	fEnglish	ripples
U+6F2B.0	fUTF8	漫
U+6F2B.0	fCNS11643	1-6950
U+6F2B.0	fBig5	BAA9
U+6F2B.0	fGB	0-427E
U+6F2B.0	fR/S	 85.11
U+6F2B.0	fTotalStrokes	 14
U+6F2B.0	fCangjie	EAWE
U+6F2B.0	fFourCorner	36147
U+6F2B.0	fHakka	 man5; man3; nem1; man6
U+6F2B.0	fCantonese	maan6; maan4
U+6F2B.0	fMandarin	man4
U+6F2B.0	fEnglish	[1] overflowing [2] uncontrolled; uninhibited [3] reckless; wild [4] unsystematic; aimless [5] spread over; extend over [6] all over the place; everywhere [7] a Chinese family name
U+6F2B.1	fUTF8	漫
U+6F2B.1	fCNS11643	1-6950
U+6F2B.1	fBig5	BAA9
U+6F2B.1	fGB	0-427E
U+6F2B.1	fR/S	 85.11
U+6F2B.1	fTotalStrokes	 14
U+6F2B.1	fCangjie	EAWE
U+6F2B.1	fFourCorner	36147
U+6F2B.1	fMandarin	man2
U+6F2B.1	fEnglish	vast or endless (expanse of water)
U+6F2C.0	fUTF8	漬
U+6F2C.0	fCNS11643	1-6944
U+6F2C.0	fBig5	BA7B
U+6F2C.0	fGB	1-5755
U+6F2C.0	fR/S	 85.11
U+6F2C.0	fTotalStrokes	 14
U+6F2C.0	fCangjie	EQMC
U+6F2C.0	fFourCorner	35186
U+6F2C.0	fHakka	  zit7;  zi5; zit8; zii5
U+6F2C.0	fCantonese	zi3; zik1 (also); zi6
U+6F2C.0	fMandarin	zi4; se4; qi4
U+6F2C.0	fEnglish	[1] soaked [2] dye [3] be caked with
U+6F2D.0	fUTF8	漭
U+6F2D.0	fCNS11643	2-4B4B
U+6F2D.0	fBig5	E2A5
U+6F2D.0	fGB	0-645D
U+6F2D.0	fR/S	 85.11; 85.10
U+6F2D.0	fTotalStrokes	 14
U+6F2D.0	fCangjie	ETIT
U+6F2D.0	fFourCorner	34143
U+6F2D.0	fHakka	  mong3;  mang3; mong5
U+6F2D.0	fCantonese	mong5
U+6F2D.0	fMandarin	mang3; mang4
U+6F2D.0	fEnglish	vast; expansive
U+6F2E.0	fUTF8	漮
U+6F2E.0	fCNS11643	2-4B3C
U+6F2E.0	fBig5	E274
U+6F2E.0	fGB	3-5578
U+6F2E.0	fR/S	 85.11
U+6F2E.0	fTotalStrokes	 14
U+6F2E.0	fCangjie	EILE
U+6F2E.0	fFourCorner	30132
U+6F2E.0	fHakka	 kong1
U+6F2E.0	fCantonese	hong1
U+6F2E.0	fMandarin	kang1
U+6F2F.0	fUTF8	漯
U+6F2F.0	fCNS11643	1-6951
U+6F2F.0	fBig5	BAAA
U+6F2F.0	fGB	0-6470
U+6F2F.0	fR/S	 85.11
U+6F2F.0	fTotalStrokes	 14
U+6F2F.0	fCangjie	EWVF
U+6F2F.0	fFourCorner	36193
U+6F2F.0	fHakka	  tap7
U+6F2F.0	fCantonese	taap3; leoi5; lok3 (also); leoi6; lo6 (also); lo4
U+6F2F.0	fMandarin	ta4; lei3; luo4
U+6F2F.0	fEnglish	a river in northern Shantung
U+6F30.0	fUTF8	漰
U+6F30.0	fCNS11643	2-4B4D
U+6F30.0	fBig5	E2A7
U+6F30.0	fGB	3-5571
U+6F30.0	fR/S	 85.11
U+6F30.0	fTotalStrokes	 14
U+6F30.0	fCangjie	EUBB
U+6F30.0	fFourCorner	32127
U+6F30.0	fHakka	  pen2;  pang2; bang2
U+6F30.0	fCantonese	paang1; peng1; paang4
U+6F30.0	fMandarin	peng1
U+6F30.0	fEnglish	roar
U+6F31.0	fUTF8	漱
U+6F31.0	fCNS11643	1-694B
U+6F31.0	fBig5	BAA4
U+6F31.0	fGB	0-4A7E
U+6F31.0	fR/S	 85.11
U+6F31.0	fTotalStrokes	 14
U+6F31.0	fCangjie	EDLO
U+6F31.0	fFourCorner	37182
U+6F31.0	fHakka	  seu5; siu5; ch'uk8; cuk8
U+6F31.0	fCantonese	sau3; sou3 (also)
U+6F31.0	fMandarin	shu4; sou4; shou4
U+6F31.0	fEnglish	[1] gargle; rinse [2] wash
U+6F32.0	fUTF8	漲
U+6F32.0	fCNS11643	1-694D
U+6F32.0	fBig5	BAA6
U+6F32.0	fGB	1-5547
U+6F32.0	fR/S	 85.11
U+6F32.0	fTotalStrokes	 14
U+6F32.0	fCangjie	ENSV
U+6F32.0	fFourCorner	31132
U+6F32.0	fHakka	  zhong5
U+6F32.0	fCantonese	zoeng2; zoeng3
U+6F32.0	fMandarin	zhang3; zhang1
U+6F32.0	fEnglish	 rise (water, price); go up in price
U+6F32.1	fUTF8	漲
U+6F32.1	fCNS11643	1-694D
U+6F32.1	fBig5	BAA6
U+6F32.1	fGB	1-5547
U+6F32.1	fR/S	 85.11
U+6F32.1	fTotalStrokes	 14
U+6F32.1	fCangjie	ENSV
U+6F32.1	fFourCorner	31132
U+6F32.1	fHakka	 zong5
U+6F32.1	fMandarin	zhang4
U+6F32.1	fEnglish	 swell; expand
U+6F33.0	fUTF8	漳
U+6F33.0	fCNS11643	1-693C
U+6F33.0	fBig5	BA73
U+6F33.0	fGB	0-5544
U+6F33.0	fR/S	 85.11
U+6F33.0	fTotalStrokes	 14
U+6F33.0	fCangjie	EYTJ
U+6F33.0	fFourCorner	30146
U+6F33.0	fHakka	 zong1; zhong1
U+6F33.0	fCantonese	zoeng1
U+6F33.0	fMandarin	zhang1
U+6F33.0	fEnglish	[1] a river in Honan [2] a river in Fukien
U+6F34.0	fUTF8	漴
U+6F34.0	fCNS11643	3-475D
U+6F34.0	fGB	3-5572
U+6F34.0	fR/S	 85.11
U+6F34.0	fTotalStrokes	 14
U+6F34.0	fCangjie	EUJF
U+6F34.0	fFourCorner	32190
U+6F34.0	fHakka	 cung1
U+6F34.0	fCantonese	zong6
U+6F34.0	fMandarin	chong2
U+6F35.0	fUTF8	漵
U+6F35.0	fCNS11643	2-4B4F
U+6F35.0	fBig5	E2A9
U+6F35.0	fGB	E-3075
U+6F35.0	fR/S	 85.11
U+6F35.0	fTotalStrokes	 14
U+6F35.0	fCangjie	EODK
U+6F35.0	fFourCorner	38140
U+6F35.0	fHakka	 si5
U+6F35.0	fCantonese	zeoi6
U+6F35.0	fMandarin	xu4
U+6F35.0	fEnglish	a river in Honan province
U+6F36.0	fUTF8	漶
U+6F36.0	fCNS11643	2-4B47
U+6F36.0	fBig5	E2A1
U+6F36.0	fGB	0-6471
U+6F36.0	fR/S	 85.11
U+6F36.0	fTotalStrokes	 14
U+6F36.0	fCangjie	ELLP
U+6F36.0	fFourCorner	35130
U+6F36.0	fCantonese	waan6; wun6
U+6F36.0	fMandarin	huan4; huan3
U+6F36.0	fEnglish	 wear out beyond recognition
U+6F37.0	fUTF8	漷
U+6F37.0	fCNS11643	2-4B3A
U+6F37.0	fBig5	E272
U+6F37.0	fGB	3-555B
U+6F37.0	fR/S	 85.11
U+6F37.0	fTotalStrokes	 14
U+6F37.0	fCangjie	EYDL
U+6F37.0	fFourCorner	37127
U+6F37.0	fHakka	 gok7; kok7; gwok7
U+6F37.0	fCantonese	gwok3; kwok3 (also); gok3 (also); kok3 (also); fok3
U+6F37.0	fMandarin	kuo4; huo3; huo4
U+6F37.0	fEnglish	a river in Hebei province
U+6F38.0	fUTF8	漸
U+6F38.0	fCNS11643	1-694C
U+6F38.0	fBig5	BAA5
U+6F38.0	fGB	1-3D25
U+6F38.0	fR/S	 85.11
U+6F38.0	fTotalStrokes	 14
U+6F38.0	fCangjie	EJJL
U+6F38.0	fFourCorner	32121
U+6F38.0	fHakka	  ciam5; ciam3; ciam6
U+6F38.0	fCantonese	zim1; zim6; zim2 (also); caam4
U+6F38.0	fMandarin	jian4; chan2; zhan1; zhe2; qian2
U+6F38.0	fEnglish	gradually; little by little; by degrees
U+6F38.1	fUTF8	漸
U+6F38.1	fCNS11643	1-694C
U+6F38.1	fBig5	BAA5
U+6F38.1	fGB	1-3D25
U+6F38.1	fR/S	 85.11
U+6F38.1	fTotalStrokes	 14
U+6F38.1	fCangjie	EJJL
U+6F38.1	fFourCorner	32121
U+6F38.1	fMandarin	jian1
U+6F38.1	fEnglish	[1] soak; permeate [2] reach (said of the territory)
U+6F39.0	fUTF8	漹
U+6F39.0	fCNS11643	2-4B57
U+6F39.0	fBig5	E2B1
U+6F39.0	fGB	3-5566
U+6F39.0	fR/S	 85.11
U+6F39.0	fTotalStrokes	 14
U+6F39.0	fCangjie	EMYF
U+6F39.0	fFourCorner	31127
U+6F39.0	fCantonese	jin4; jin1
U+6F39.0	fMandarin	yan1; yan2; yan4
U+6F3A.0	fUTF8	漺
U+6F3A.0	fCNS11643	2-4B5A
U+6F3A.0	fBig5	E2B4
U+6F3A.0	fGB	3-556B
U+6F3A.0	fR/S	 85.11
U+6F3A.0	fTotalStrokes	 14
U+6F3A.0	fCangjie	EKKK
U+6F3A.0	fFourCorner	34134
U+6F3A.0	fCantonese	song2
U+6F3A.0	fMandarin	chuang3; shuang3
U+6F3B.0	fUTF8	漻
U+6F3B.0	fCNS11643	2-4B43
U+6F3B.0	fBig5	E27B
U+6F3B.0	fGB	3-5626
U+6F3B.0	fR/S	 85.11
U+6F3B.0	fTotalStrokes	 14
U+6F3B.0	fCangjie	ESMH
U+6F3B.0	fFourCorner	37122
U+6F3B.0	fHakka	  liau2
U+6F3B.0	fCantonese	liu4; lau4
U+6F3B.0	fMandarin	liao2; liu2; li4; liao3; xiao4
U+6F3B.0	fEnglish	deep and clear
U+6F3C.0	fUTF8	漼
U+6F3C.0	fCNS11643	2-4B4E
U+6F3C.0	fBig5	E2A8
U+6F3C.0	fGB	3-5570
U+6F3C.0	fR/S	 85.11
U+6F3C.0	fTotalStrokes	 14
U+6F3C.0	fCangjie	EUOG
U+6F3C.0	fFourCorner	32114
U+6F3C.0	fHakka	  cui1
U+6F3C.0	fCantonese	ceoi2; ceoi1; ceoi4
U+6F3C.0	fMandarin	cui3; cui1
U+6F3C.0	fEnglish	having the appearance of depth
U+6F3D.0	fUTF8	漽
U+6F3D.0	fCNS11643	3-4C74
U+6F3D.0	fGB	5-567B
U+6F3D.0	fR/S	  85.12
U+6F3D.0	fTotalStrokes	 15
U+6F3D.0	fCangjie	ESEQ
U+6F3D.0	fFourCorner	37150
U+6F3D.0	fCantonese	tai4
U+6F3D.0	fMandarin	ti2
U+6F3E.0	fUTF8	漾
U+6F3E.0	fCNS11643	1-6942
U+6F3E.0	fBig5	BA79
U+6F3E.0	fGB	0-517A
U+6F3E.0	fR/S	 85.11
U+6F3E.0	fTotalStrokes	 14
U+6F3E.0	fCangjie	ETGE
U+6F3E.0	fFourCorner	38132
U+6F3E.0	fHakka	 jong5; rong6; jong3
U+6F3E.0	fCantonese	joeng6
U+6F3E.0	fMandarin	yang4; yang3; yang2
U+6F3E.0	fEnglish	[1] ripple [2] brim over [3] throw up; vomit
U+6F3F.0	fUTF8	漿
U+6F3F.0	fCNS11643	1-6D49
U+6F3F.0	fBig5	BCDF
U+6F3F.0	fGB	1-3D2C
U+6F3F.0	fR/S	 85.11
U+6F3F.0	fTotalStrokes	 15
U+6F3F.0	fCangjie	VIE
U+6F3F.0	fFourCorner	27232
U+6F3F.0	fHakka	  ziong1; ziong3
U+6F3F.0	fCantonese	zoeng1
U+6F3F.0	fMandarin	jiang1; jiang4
U+6F3F.0	fEnglish	[1] thick fluid; starch [2] starch
U+6F40.0	fUTF8	潀
U+6F40.0	fCNS11643	2-4B4C
U+6F40.0	fBig5	E2A6
U+6F40.0	fGB	E-3076
U+6F40.0	fR/S	 85.11
U+6F40.0	fTotalStrokes	 14
U+6F40.0	fCangjie	EWLO
U+6F40.0	fFourCorner	36182
U+6F40.0	fHakka	 cung2
U+6F40.0	fCantonese	cung4; zung1
U+6F40.0	fMandarin	cong2; zhong1; zhuang4; zong1
U+6F40.0	fEnglish	gather; flow into (water); sound of waters flowing together
U+6F41.0	fUTF8	潁
U+6F41.0	fCNS11643	2-5142
U+6F41.0	fBig5	E5F9
U+6F41.0	fGB	1-7223
U+6F41.0	fR/S	 85.11
U+6F41.0	fTotalStrokes	 15
U+6F41.0	fCangjie	PEMBC
U+6F41.0	fFourCorner	21286
U+6F41.0	fHakka	  jin3; rin3
U+6F41.0	fCantonese	wing6
U+6F41.0	fMandarin	ying3
U+6F41.0	fEnglish	a river and place in Anhwei
U+6F42.0	fUTF8	潂
U+6F42.0	fCNS11643	3-4762
U+6F42.0	fGB	E-3077
U+6F42.0	fR/S	 85.11
U+6F42.0	fTotalStrokes	 14
U+6F42.0	fCangjie	EUTC
U+6F42.0	fFourCorner	32110
U+6F42.0	fCantonese	hung4
U+6F42.0	fMandarin	hong2
U+6F43.0	fUTF8	潃
U+6F43.0	fCNS11643	2-4B53
U+6F43.0	fBig5	E2AD
U+6F43.0	fGB	E-3078
U+6F43.0	fR/S	 85.11
U+6F43.0	fTotalStrokes	 14
U+6F43.0	fCangjie	EHOB
U+6F43.0	fFourCorner	38127
U+6F43.0	fCantonese	sau1; sau2; sau3
U+6F43.0	fMandarin	xun2; xiu1
U+6F44.0	fUTF8	潄
U+6F44.0	fCNS11643	3-4761
U+6F44.0	fGB	E-3079
U+6F44.0	fR/S	 85.11
U+6F44.0	fTotalStrokes	 14
U+6F44.0	fCangjie	EDLK
U+6F44.0	fFourCorner	38140
U+6F44.0	fCantonese	sau3; sou3 (also)
U+6F44.0	fMandarin	shu4; sou4
U+6F45.0	fUTF8	潅
U+6F45.0	fCNS11643	4-4452
U+6F45.0	fGB	E-307A
U+6F45.0	fR/S	 85.11
U+6F45.0	fTotalStrokes	 14
U+6F45.0	fCangjie	EOKG
U+6F45.0	fFourCorner	38110
U+6F45.0	fCantonese	gun3; gwun3 (also)
U+6F45.0	fMandarin	guan4; huan4
U+6F45.0	fEnglish	 pour water irrigate; flood
U+6F46.0	fUTF8	潆
U+6F46.0	fCNS11643	F-4F79
U+6F46.0	fGB	0-646B
U+6F46.0	fR/S	 85.11
U+6F46.0	fTotalStrokes	 14
U+6F46.0	fCangjie	ETBF
U+6F46.0	fFourCorner	3419
U+6F46.0	fCantonese	jing4
U+6F46.0	fMandarin	ying2; ying3
U+6F46.0	fEnglish	simplified form of &#x7020;
U+6F47.0	fUTF8	潇
U+6F47.0	fGB	0-646C
U+6F47.0	fR/S	 85.11
U+6F47.0	fTotalStrokes	 14
U+6F47.0	fCangjie	XETLX
U+6F47.0	fHakka	 siau1
U+6F47.0	fCantonese	siu1
U+6F47.0	fMandarin	xiao1
U+6F47.0	fEnglish	simplified form of &#x701F;
U+6F48.0	fUTF8	潈
U+6F48.0	fCNS11643	3-6544
U+6F48.0	fGB	E-307B
U+6F48.0	fR/S	 85.11
U+6F48.0	fTotalStrokes	 14
U+6F48.0	fCangjie	XEWLO
U+6F48.0	fFourCorner	3713
U+6F48.0	fCantonese	zung1
U+6F49.0	fUTF8	潉
U+6F49.0	fCNS11643	3-6545
U+6F49.0	fGB	E-307C
U+6F49.0	fR/S	 85.11
U+6F49.0	fTotalStrokes	 14
U+6F49.0	fCangjie	EUAP
U+6F49.0	fFourCorner	3211
U+6F49.0	fCantonese	wan6
U+6F4A.0	fUTF8	潊
U+6F4A.0	fCNS11643	3-4766
U+6F4A.0	fGB	E-307D
U+6F4A.0	fR/S	 85.11
U+6F4A.0	fTotalStrokes	 14
U+6F4A.0	fCangjie	EODE
U+6F4A.0	fFourCorner	31140
U+6F4A.0	fCantonese	zeoi6
U+6F4A.0	fMandarin	xu4
U+6F4B.0	fUTF8	潋
U+6F4B.0	fGB	0-6472
U+6F4B.0	fR/S	 85.11
U+6F4B.0	fTotalStrokes	 14
U+6F4B.0	fCangjie	EOMK
U+6F4B.0	fHakka	 liam5
U+6F4B.0	fCantonese	lim6
U+6F4B.0	fMandarin	lian4
U+6F4B.0	fEnglish	simplified form of &#x7032;
U+6F4C.0	fUTF8	潌
U+6F4C.0	fCNS11643	3-6543
U+6F4C.0	fGB	E-307E
U+6F4C.0	fR/S	 85.11
U+6F4C.0	fTotalStrokes	 14
U+6F4C.0	fCangjie	EJCG
U+6F4C.0	fFourCorner	3311
U+6F4C.0	fCantonese	zat6
U+6F4C.0	fMandarin	zhi4
U+6F4D.0	fUTF8	潍
U+6F4D.0	fGB	0-4E2B
U+6F4D.0	fR/S	 85.11
U+6F4D.0	fTotalStrokes	 14
U+6F4D.0	fCangjie	EVMG
U+6F4D.0	fHakka	 vui2
U+6F4D.0	fCantonese	wai4
U+6F4D.0	fMandarin	wei2
U+6F4D.0	fEnglish	simplified form of &#x6FF0;
U+6F4E.0	fUTF8	潎
U+6F4E.0	fCNS11643	2-4B3E
U+6F4E.0	fBig5	E276
U+6F4E.0	fGB	3-556D
U+6F4E.0	fR/S	 85.12
U+6F4E.0	fTotalStrokes	 15
U+6F4E.0	fCangjie	EFBK
U+6F4E.0	fFourCorner	38140
U+6F4E.0	fHakka	 pi5; bi5; piak7
U+6F4E.0	fCantonese	pit3; pik1; piu3
U+6F4E.0	fMandarin	pi4; pie1; bi3; piao4
U+6F4E.0	fEnglish	 rippling pour
U+6F4F.0	fUTF8	潏
U+6F4F.0	fCNS11643	2-514C
U+6F4F.0	fBig5	E644
U+6F4F.0	fGB	3-563F
U+6F4F.0	fR/S	 85.12
U+6F4F.0	fTotalStrokes	 15
U+6F4F.0	fCangjie	ENHB
U+6F4F.0	fFourCorner	37127
U+6F4F.0	fHakka	  giet7; get7
U+6F4F.0	fCantonese	kyut3; wat6; seot6; jyut6; gyut3
U+6F4F.0	fMandarin	jue2; yu4; shu4
U+6F4F.0	fEnglish	fareshare reclaimed from river
U+6F50.0	fUTF8	潐
U+6F50.0	fCNS11643	2-5156
U+6F50.0	fBig5	E64E
U+6F50.0	fGB	3-5633
U+6F50.0	fR/S	 85.12
U+6F50.0	fTotalStrokes	 15
U+6F50.0	fCangjie	EOGF
U+6F50.0	fFourCorner	30131
U+6F50.0	fCantonese	ziu3; ciu4; ziu2
U+6F50.0	fMandarin	jiao4; jiao3; qiao2
U+6F51.0	fUTF8	潑
U+6F51.0	fCNS11643	1-6D4C
U+6F51.0	fBig5	BCE2
U+6F51.0	fGB	1-4643
U+6F51.0	fR/S	 85.12
U+6F51.0	fTotalStrokes	 15
U+6F51.0	fCangjie	ENOE
U+6F51.0	fFourCorner	32147
U+6F51.0	fHakka	 pat7; pat8; biak8
U+6F51.0	fCantonese	put3; but3
U+6F51.0	fMandarin	po1; pai4; bo1
U+6F51.0	fEnglish	[1] pour; sprinkle [2] ferocious; fierce; spiteful; villainous
U+6F51.1	fUTF8	潑
U+6F51.1	fCNS11643	1-6D4C
U+6F51.1	fBig5	BCE2
U+6F51.1	fGB	1-4643
U+6F51.1	fR/S	 85.12
U+6F51.1	fTotalStrokes	 15
U+6F51.1	fCangjie	ENOE
U+6F51.1	fFourCorner	32147
U+6F51.1	fMandarin	po5
U+6F52.0	fUTF8	潒
U+6F52.0	fCNS11643	2-5155
U+6F52.0	fBig5	E64D
U+6F52.0	fGB	5-5661
U+6F52.0	fR/S	 85.12
U+6F52.0	fTotalStrokes	 15
U+6F52.0	fCangjie	ENAO
U+6F52.0	fFourCorner	37132
U+6F52.0	fHakka	  tong3;  siong5; jong5
U+6F52.0	fCantonese	dong6; tong3
U+6F52.0	fMandarin	dang4; xiang4; yang3
U+6F52.0	fEnglish	a torrent of water rushing along
U+6F53.0	fUTF8	潓
U+6F53.0	fCNS11643	2-5161
U+6F53.0	fBig5	E659
U+6F53.0	fGB	3-562B
U+6F53.0	fR/S	 85.12
U+6F53.0	fTotalStrokes	 15
U+6F53.0	fCangjie	EJIP
U+6F53.0	fFourCorner	35133
U+6F53.0	fCantonese	wai6
U+6F53.0	fMandarin	hui4
U+6F54.0	fUTF8	潔
U+6F54.0	fCNS11643	1-6D4E
U+6F54.0	fBig5	BCE4
U+6F54.0	fGB	1-3D60
U+6F54.0	fR/S	 85.12
U+6F54.0	fTotalStrokes	 15
U+6F54.0	fCangjie	EQHF
U+6F54.0	fFourCorner	37193
U+6F54.0	fHakka	  giet7; get7; kiat7
U+6F54.0	fCantonese	git3
U+6F54.0	fMandarin	jie2
U+6F54.0	fEnglish	[1] clean; purify; pure; spotless; immaculate [2] clean; keep clean
U+6F55.0	fUTF8	潕
U+6F55.0	fCNS11643	2-5153
U+6F55.0	fBig5	E64B
U+6F55.0	fGB	3-533B
U+6F55.0	fR/S	 85.12
U+6F55.0	fTotalStrokes	 15
U+6F55.0	fCangjie	EOTF
U+6F55.0	fFourCorner	38131
U+6F55.0	fHakka	 wu3; vu3
U+6F55.0	fCantonese	mou5
U+6F55.0	fMandarin	wu3
U+6F55.0	fEnglish	a river in Henan
U+6F56.0	fUTF8	潖
U+6F56.0	fCNS11643	4-4A58
U+6F56.0	fGB	3-5627
U+6F56.0	fR/S	 85.12
U+6F56.0	fTotalStrokes	 15
U+6F56.0	fCangjie	EMGU
U+6F56.0	fFourCorner	31110
U+6F56.0	fCantonese	paa4
U+6F56.0	fMandarin	pa2
U+6F56.0	fEnglish	the name of a river
U+6F57.0	fUTF8	潗
U+6F57.0	fCNS11643	2-5157
U+6F57.0	fBig5	E64F
U+6F57.0	fGB	5-5672
U+6F57.0	fR/S	 85.12
U+6F57.0	fTotalStrokes	 15
U+6F57.0	fCangjie	EOGD
U+6F57.0	fFourCorner	30194
U+6F57.0	fHakka	  sip8;  cip8
U+6F57.0	fCantonese	cap1; zap1; zap6
U+6F57.0	fMandarin	ji2; qi4
U+6F57.0	fEnglish	[1] a fountain bubbling up [2] the noise of boiling water
U+6F58.0	fUTF8	潘
U+6F58.0	fCNS11643	1-6D59
U+6F58.0	fBig5	BCEF
U+6F58.0	fGB	0-454B
U+6F58.0	fR/S	 85.12
U+6F58.0	fTotalStrokes	 15
U+6F58.0	fCangjie	EHDW
U+6F58.0	fFourCorner	32169
U+6F58.0	fHakka	 pan1; pan2
U+6F58.0	fCantonese	pun1
U+6F58.0	fMandarin	pan1; pan2; fan1; bo1; pan4
U+6F58.0	fEnglish	[1] water from rinsing rice; water in which rice has been washed [2] a Chinese family name
U+6F59.0	fUTF8	潙
U+6F59.0	fCNS11643	3-4C7D
U+6F59.0	fGB	1-636D
U+6F59.0	fR/S	 85.12
U+6F59.0	fTotalStrokes	 15
U+6F59.0	fCangjie	EBHF
U+6F59.0	fFourCorner	32120
U+6F59.0	fCantonese	gwai1
U+6F59.0	fMandarin	gui1; wei2; gui2
U+6F59.0	fEnglish	a river in Shanxi
U+6F5A.0	fUTF8	潚
U+6F5A.0	fCNS11643	2-514E
U+6F5A.0	fBig5	E646
U+6F5A.0	fGB	3-5472
U+6F5A.0	fR/S	 85.12; 85.13
U+6F5A.0	fTotalStrokes	 15
U+6F5A.0	fCangjie	ELX
U+6F5A.0	fFourCorner	35127
U+6F5A.0	fHakka	 suk7; siau1; siuk7
U+6F5A.0	fCantonese	suk1; siu1
U+6F5A.0	fMandarin	xiao1; su4; cu4
U+6F5A.0	fEnglish	 gay deep
U+6F5B.0	fUTF8	潛
U+6F5B.0	fCNS11643	1-6D51
U+6F5B.0	fBig5	BCE7
U+6F5B.0	fGB	1-4731
U+6F5B.0	fR/S	 85.12
U+6F5B.0	fTotalStrokes	 15
U+6F5B.0	fCangjie	EMUA
U+6F5B.0	fFourCorner	35161
U+6F5B.0	fHakka	  siam2; siam1; ciam2
U+6F5B.0	fCantonese	cim4
U+6F5B.0	fMandarin	qian2; jian4
U+6F5B.0	fEnglish	[1] hide; conceal [2] hidden; secret; latent [3] dive
U+6F5C.0	fUTF8	潜
U+6F5C.0	fCNS11643	3-4C73
U+6F5C.0	fGB	0-4731
U+6F5C.0	fR/S	  85.12
U+6F5C.0	fTotalStrokes	 15
U+6F5C.0	fCangjie	EQOA
U+6F5C.0	fFourCorner	35160
U+6F5C.0	fHakka	  siam2; siam1
U+6F5C.0	fCantonese	cim4
U+6F5C.0	fMandarin	qian2; jian4
U+6F5C.0	fEnglish	simplified form of &#x6F5B;
U+6F5D.0	fUTF8	潝
U+6F5D.0	fCNS11643	2-515A
U+6F5D.0	fBig5	E652
U+6F5D.0	fGB	3-5637
U+6F5D.0	fR/S	 85.12
U+6F5D.0	fTotalStrokes	 15
U+6F5D.0	fCangjie	EORM
U+6F5D.0	fFourCorner	38127
U+6F5D.0	fHakka	  hip7
U+6F5D.0	fCantonese	kap1; jap1; hap1
U+6F5D.0	fMandarin	xi1; xi4
U+6F5D.0	fEnglish	agree
U+6F5E.0	fUTF8	潞
U+6F5E.0	fCNS11643	2-577B
U+6F5E.0	fBig5	E9F0
U+6F5E.0	fGB	0-423A
U+6F5E.0	fR/S	 85.12; 85.13
U+6F5E.0	fTotalStrokes	 15
U+6F5E.0	fCangjie	ERMR
U+6F5E.0	fFourCorner	37164
U+6F5E.0	fHakka	 lu5; lu6
U+6F5E.0	fCantonese	lou6; lou2 (also)
U+6F5E.0	fMandarin	lu4
U+6F5E.0	fEnglish	names of several rivers in northern China
U+6F5F.0	fUTF8	潟
U+6F5F.0	fCNS11643	1-6D5D
U+6F5F.0	fBig5	BCF3
U+6F5F.0	fGB	3-5632
U+6F5F.0	fR/S	 85.12
U+6F5F.0	fTotalStrokes	 15
U+6F5F.0	fCangjie	EHXF
U+6F5F.0	fFourCorner	37127
U+6F5F.0	fHakka	  sit7
U+6F5F.0	fCantonese	sik1; coek3
U+6F5F.0	fMandarin	xi4; chi3; di2; que4
U+6F5F.0	fEnglish	saline land
U+6F60.0	fUTF8	潠
U+6F60.0	fCNS11643	1-6D5C
U+6F60.0	fBig5	BCF2
U+6F60.0	fGB	5-567C
U+6F60.0	fR/S	 85.12
U+6F60.0	fTotalStrokes	 15
U+6F60.0	fCangjie	ERUC
U+6F60.0	fFourCorner	37181
U+6F60.0	fHakka	 sun2; sun5
U+6F60.0	fCantonese	seon3; seon1
U+6F60.0	fMandarin	sun4; xun4; xuan4
U+6F60.0	fEnglish	 blow out puff out
U+6F61.0	fUTF8	潡
U+6F61.0	fCNS11643	2-515C
U+6F61.0	fBig5	E654
U+6F61.0	fGB	5-5675
U+6F61.0	fR/S	 85.12
U+6F61.0	fTotalStrokes	 15
U+6F61.0	fCangjie	EYDK
U+6F61.0	fFourCorner	38140
U+6F61.0	fCantonese	dan6; deon6
U+6F61.0	fMandarin	dun4
U+6F62.0	fUTF8	潢
U+6F62.0	fCNS11643	2-514B
U+6F62.0	fBig5	E643
U+6F62.0	fGB	0-646A
U+6F62.0	fR/S	 85.12
U+6F62.0	fTotalStrokes	 15
U+6F62.0	fCangjie	ETLC
U+6F62.0	fFourCorner	34186
U+6F62.0	fHakka	 wong2; wang2; won2; wong3; fong3; vong2; vang2
U+6F62.0	fCantonese	wong4
U+6F62.0	fMandarin	huang2; huang3; huang4; guang1
U+6F62.0	fEnglish	 lake; pond
U+6F63.0	fUTF8	潣
U+6F63.0	fCNS11643	2-5166
U+6F63.0	fBig5	E65E
U+6F63.0	fGB	5-5562
U+6F63.0	fR/S	 85.12
U+6F63.0	fTotalStrokes	 15
U+6F63.0	fCangjie	EANK
U+6F63.0	fFourCorner	37120
U+6F63.0	fHakka	 men3
U+6F63.0	fCantonese	man5; mui5
U+6F63.0	fMandarin	min3; hui3; mei3; mian3
U+6F63.0	fEnglish	water flowing
U+6F64.0	fUTF8	潤
U+6F64.0	fCNS11643	1-6D57
U+6F64.0	fBig5	BCED
U+6F64.0	fGB	1-4873
U+6F64.0	fR/S	 85.12
U+6F64.0	fTotalStrokes	 15
U+6F64.0	fCangjie	EANG
U+6F64.0	fFourCorner	37120
U+6F64.0	fHakka	  jun5; run6; jun6; jun3; ngiun6; ngiun5
U+6F64.0	fCantonese	jeon6; jeon2 (also)
U+6F64.0	fMandarin	run4
U+6F64.0	fEnglish	[1] moist; glossy; fresh [2] moisten; freshen [3] enrich; benefit [4] embellish
U+6F65.0	fUTF8	潥
U+6F65.0	fCNS11643	3-4C6B
U+6F65.0	fGB	3-562C
U+6F65.0	fR/S	  85.12
U+6F65.0	fTotalStrokes	 15
U+6F65.0	fCangjie	EMWD
U+6F65.0	fFourCorner	31190
U+6F65.0	fCantonese	suk1
U+6F65.0	fMandarin	su4
U+6F66.0	fUTF8	潦
U+6F66.0	fCNS11643	1-6D4D
U+6F66.0	fBig5	BCE3
U+6F66.0	fGB	0-414A
U+6F66.0	fR/S	 85.12
U+6F66.0	fTotalStrokes	 15
U+6F66.0	fCangjie	EKCF
U+6F66.0	fFourCorner	34196
U+6F66.0	fCantonese	liu2 (also)
U+6F66.0	fMandarin	liao2
U+6F66.0	fEnglish	[1] disheartened; disappointed [2] without care
U+6F66.1	fUTF8	潦
U+6F66.1	fCNS11643	1-6D4D
U+6F66.1	fBig5	BCE3
U+6F66.1	fGB	0-414A
U+6F66.1	fR/S	 85.12
U+6F66.1	fTotalStrokes	 15
U+6F66.1	fCangjie	EKCF
U+6F66.1	fFourCorner	34196
U+6F66.1	fHakka	  liau2; lau2; lo3; lau5
U+6F66.1	fCantonese	liu4; lou5; lou6; liu5
U+6F66.1	fMandarin	lao4; lao2; liao3
U+6F66.1	fEnglish	 flood; floods
U+6F66.2	fUTF8	潦
U+6F66.2	fCNS11643	1-6D4D
U+6F66.2	fBig5	BCE3
U+6F66.2	fGB	0-414A
U+6F66.2	fR/S	 85.12
U+6F66.2	fTotalStrokes	 15
U+6F66.2	fCangjie	EKCF
U+6F66.2	fFourCorner	34196
U+6F66.2	fMandarin	lao3
U+6F66.2	fEnglish	 puddle
U+6F67.0	fUTF8	潧
U+6F67.0	fCNS11643	2-515F
U+6F67.0	fBig5	E657
U+6F67.0	fGB	5-567A
U+6F67.0	fR/S	 85.12
U+6F67.0	fTotalStrokes	 15
U+6F67.0	fCangjie	ECWA
U+6F67.0	fFourCorner	38166
U+6F67.0	fCantonese	zeon1
U+6F67.0	fMandarin	zhen1
U+6F68.0	fUTF8	潨
U+6F68.0	fCNS11643	4-4A69
U+6F68.0	fGB	3-5634
U+6F68.0	fR/S	 85.12
U+6F68.0	fTotalStrokes	 15
U+6F68.0	fCangjie	EHWO
U+6F68.0	fFourCorner	37180
U+6F68.0	fCantonese	zung1
U+6F68.0	fMandarin	zhong1
U+6F68.0	fEnglish	 gather flow into (water) sound of waters flowing together
U+6F69.0	fUTF8	潩
U+6F69.0	fCNS11643	2-5163
U+6F69.0	fBig5	E65B
U+6F69.0	fGB	3-556E
U+6F69.0	fR/S	 85.12
U+6F69.0	fTotalStrokes	 15
U+6F69.0	fCangjie	EWTC
U+6F69.0	fFourCorner	36181
U+6F69.0	fCantonese	ji6
U+6F69.0	fMandarin	yi4; chi4
U+6F6A.0	fUTF8	潪
U+6F6A.0	fCNS11643	2-5168
U+6F6A.0	fBig5	E660
U+6F6A.0	fGB	3-5631
U+6F6A.0	fR/S	 85.12
U+6F6A.0	fTotalStrokes	 15
U+6F6A.0	fCangjie	EORA
U+6F6A.0	fFourCorner	36160
U+6F6A.0	fCantonese	zik6; zi3
U+6F6A.0	fMandarin	di2; zhe2; zhi4
U+6F6B.0	fUTF8	潫
U+6F6B.0	fCNS11643	2-515D
U+6F6B.0	fBig5	E655
U+6F6B.0	fGB	3-5638
U+6F6B.0	fR/S	 85.12
U+6F6B.0	fTotalStrokes	 15
U+6F6B.0	fCangjie	EFQF
U+6F6B.0	fFourCorner	39193
U+6F6B.0	fCantonese	waan1
U+6F6B.0	fMandarin	wan1; yuan1
U+6F6C.0	fUTF8	潬
U+6F6C.0	fCNS11643	2-5151
U+6F6C.0	fBig5	E649
U+6F6C.0	fGB	3-5470
U+6F6C.0	fR/S	 85.12
U+6F6C.0	fTotalStrokes	 15
U+6F6C.0	fCangjie	ERRJ
U+6F6C.0	fFourCorner	36146
U+6F6C.0	fHakka	  tan1; tan3
U+6F6C.0	fCantonese	taan1; sin5
U+6F6C.0	fMandarin	dan4; shan4; tan1
U+6F6C.0	fEnglish	 a rapid sand-bank foreshore
U+6F6D.0	fUTF8	潭
U+6F6D.0	fCNS11643	1-6D50
U+6F6D.0	fBig5	BCE6
U+6F6D.0	fGB	0-4C36
U+6F6D.0	fR/S	 85.12
U+6F6D.0	fTotalStrokes	 15
U+6F6D.0	fCangjie	EMWJ
U+6F6D.0	fFourCorner	31146
U+6F6D.0	fHakka	  tam2; tam3
U+6F6D.0	fCantonese	taam4
U+6F6D.0	fMandarin	tan2; dan4; xun2; yin3; ren3; xin2; yin2
U+6F6D.0	fEnglish	[1] deep water; deep pool [2] deep; profound [3] hole in the ground
U+6F6E.0	fUTF8	潮
U+6F6E.0	fCNS11643	1-6D53
U+6F6E.0	fBig5	BCE9
U+6F6E.0	fGB	0-3331
U+6F6E.0	fR/S	 85.12
U+6F6E.0	fTotalStrokes	 15
U+6F6E.0	fCangjie	EJJB
U+6F6E.0	fFourCorner	37120
U+6F6E.0	fHakka	 chau2; ceu2; chau3; cau2; ch'eu2
U+6F6E.0	fCantonese	ciu4
U+6F6E.0	fMandarin	chao2
U+6F6E.0	fEnglish	[1] tide [2] moist; wet; damp [3] inferior in skill or fineness (gold, silver, etc.)
U+6F6F.0	fUTF8	潯
U+6F6F.0	fCNS11643	1-6D5B
U+6F6F.0	fBig5	BCF1
U+6F6F.0	fGB	1-6431
U+6F6F.0	fR/S	 85.12
U+6F6F.0	fTotalStrokes	 15
U+6F6F.0	fCangjie	ESMI
U+6F6F.0	fFourCorner	37146
U+6F6F.0	fHakka	 cim2
U+6F6F.0	fCantonese	cam4
U+6F6F.0	fMandarin	xun2; yin2; tan2
U+6F6F.0	fEnglish	[1] a steep bank by the stream [2] another name for Chiuchiang in Kiangsi
U+6F70.0	fUTF8	潰
U+6F70.0	fCNS11643	1-6D56
U+6F70.0	fBig5	BCEC
U+6F70.0	fGB	1-4023
U+6F70.0	fR/S	 85.12
U+6F70.0	fTotalStrokes	 15
U+6F70.0	fCangjie	ELMC
U+6F70.0	fFourCorner	35186
U+6F70.0	fHakka	 kui5; kui3; kwi5; k'wui5
U+6F70.0	fCantonese	kui2; kwui2 (also)
U+6F70.0	fMandarin	kui4; hui4; xie4; hu2; hui2
U+6F70.0	fEnglish	[1] a river overflowing its banks; flooding river [2] broken up; scattered [3] defeat (militarily) [4] burst (dam, dike)
U+6F71.0	fUTF8	潱
U+6F71.0	fCNS11643	3-4C75
U+6F71.0	fGB	E-3121
U+6F71.0	fR/S	  85.12
U+6F71.0	fTotalStrokes	 15
U+6F71.0	fCangjie	EGBT
U+6F71.0	fFourCorner	34110
U+6F71.0	fCantonese	jan1
U+6F71.0	fMandarin	ye1
U+6F72.0	fUTF8	潲
U+6F72.0	fCNS11643	2-5154
U+6F72.0	fBig5	E64C
U+6F72.0	fGB	0-647B
U+6F72.0	fR/S	 85.12
U+6F72.0	fTotalStrokes	 15
U+6F72.0	fCangjie	EHDB
U+6F72.0	fFourCorner	39127
U+6F72.0	fHakka	  sau5; sau1; sam3; sau6; cau5
U+6F72.0	fCantonese	saau3; siu6; sau3
U+6F72.0	fMandarin	shao4; shao3
U+6F72.0	fEnglish	 fall slantwise (rain)
U+6F73.0	fUTF8	潳
U+6F73.0	fCNS11643	2-4B48
U+6F73.0	fBig5	E2A2
U+6F73.0	fGB	3-5623
U+6F73.0	fR/S	 85.12
U+6F73.0	fTotalStrokes	 15
U+6F73.0	fCangjie	ESJA
U+6F73.0	fFourCorner	37164
U+6F73.0	fCantonese	tou4; zaa1
U+6F73.0	fMandarin	tu2; zha1
U+6F74.0	fUTF8	潴
U+6F74.0	fCNS11643	3-4C7E
U+6F74.0	fGB	0-6473
U+6F74.0	fR/S	 85.12; 85.11
U+6F74.0	fTotalStrokes	 15; 14
U+6F74.0	fCangjie	EKHA
U+6F74.0	fFourCorner	34160
U+6F74.0	fHakka	 zhu1; zu1
U+6F74.0	fCantonese	zyu1
U+6F74.0	fMandarin	zhu1
U+6F74.0	fEnglish	 pond pool
U+6F75.0	fUTF8	潵
U+6F75.0	fCNS11643	3-4C6C
U+6F75.0	fGB	3-5629
U+6F75.0	fR/S	 85.12
U+6F75.0	fTotalStrokes	 15
U+6F75.0	fCangjie	ETBK
U+6F75.0	fFourCorner	38140
U+6F75.0	fCantonese	saat3; saan3
U+6F75.0	fMandarin	san4; sa3
U+6F76.0	fUTF8	潶
U+6F76.0	fCNS11643	2-5150
U+6F76.0	fBig5	E648
U+6F76.0	fGB	E-3122
U+6F76.0	fR/S	 85.12
U+6F76.0	fTotalStrokes	 15
U+6F76.0	fCangjie	EWGF
U+6F76.0	fFourCorner	36131
U+6F76.0	fCantonese	hak1
U+6F76.0	fMandarin	hei1; he4
U+6F77.0	fUTF8	潷
U+6F77.0	fCNS11643	2-5167
U+6F77.0	fBig5	E65F
U+6F77.0	fGB	1-6464
U+6F77.0	fR/S	 85.12
U+6F77.0	fTotalStrokes	 15
U+6F77.0	fCangjie	EHLQ
U+6F77.0	fFourCorner	38157
U+6F77.0	fHakka	  bit7; bi5; {bi5}
U+6F77.0	fCantonese	bei3; bat1
U+6F77.0	fMandarin	bi4; bi3
U+6F77.0	fEnglish	drain
U+6F78.0	fUTF8	潸
U+6F78.0	fCNS11643	1-6D52
U+6F78.0	fBig5	BCE8
U+6F78.0	fGB	0-647A
U+6F78.0	fR/S	 85.12
U+6F78.0	fTotalStrokes	 15
U+6F78.0	fCangjie	EDDB
U+6F78.0	fFourCorner	34127
U+6F78.0	fHakka	 san3; san1
U+6F78.0	fCantonese	saan1; saan2; sim2
U+6F78.0	fMandarin	shan1; shan4
U+6F78.0	fEnglish	 weep; tears flowing
U+6F79.0	fUTF8	潹
U+6F79.0	fCNS11643	3-4C79
U+6F79.0	fGB	5-5669
U+6F79.0	fR/S	 85.12
U+6F79.0	fTotalStrokes	 15
U+6F79.0	fCangjie	EDDD
U+6F79.0	fFourCorner	34190
U+6F79.0	fHakka	 zan1
U+6F79.0	fCantonese	saan4
U+6F79.0	fMandarin	chan2
U+6F7A.0	fUTF8	潺
U+6F7A.0	fCNS11643	1-6D55
U+6F7A.0	fBig5	BCEB
U+6F7A.0	fGB	0-647D
U+6F7A.0	fR/S	 85.12
U+6F7A.0	fTotalStrokes	 15
U+6F7A.0	fCangjie	ESND
U+6F7A.0	fFourCorner	37147
U+6F7A.0	fHakka	 zan1
U+6F7A.0	fCantonese	saan4
U+6F7A.0	fMandarin	chan2
U+6F7A.0	fEnglish	the sound of flowing water
U+6F7B.0	fUTF8	潻
U+6F7B.0	fCNS11643	2-5169
U+6F7B.0	fBig5	E661
U+6F7B.0	fGB	E-3123
U+6F7B.0	fR/S	 85.12
U+6F7B.0	fTotalStrokes	 15
U+6F7B.0	fCangjie	EHDE
U+6F7B.0	fFourCorner	32123
U+6F7B.0	fCantonese	syu2
U+6F7B.0	fMandarin	shu3
U+6F7C.0	fUTF8	潼
U+6F7C.0	fCNS11643	1-6D4A
U+6F7C.0	fBig5	BCE0
U+6F7C.0	fGB	0-647C
U+6F7C.0	fR/S	 85.12
U+6F7C.0	fTotalStrokes	 15
U+6F7C.0	fCangjie	EYTG
U+6F7C.0	fFourCorner	30114
U+6F7C.0	fHakka	 tung2; tung3; kwi5
U+6F7C.0	fCantonese	tung4
U+6F7C.0	fMandarin	tong2; chong1; zhong1
U+6F7C.0	fEnglish	[1] high; lofty [2] a tributary of the Yellow river [3] a county in Shensi province
U+6F7D.0	fUTF8	潽
U+6F7D.0	fCNS11643	2-515E
U+6F7D.0	fBig5	E656
U+6F7D.0	fGB	3-5639
U+6F7D.0	fR/S	 85.12
U+6F7D.0	fTotalStrokes	 15
U+6F7D.0	fCangjie	ETCA
U+6F7D.0	fFourCorner	38161
U+6F7D.0	fHakka	 pu1
U+6F7D.0	fCantonese	pou1; pou2
U+6F7D.0	fMandarin	pu3; pu1
U+6F7E.0	fUTF8	潾
U+6F7E.0	fCNS11643	2-5144
U+6F7E.0	fBig5	E5FB
U+6F7E.0	fGB	3-563A
U+6F7E.0	fR/S	 85.12; 85.13
U+6F7E.0	fTotalStrokes	 15
U+6F7E.0	fCangjie	EFDQ
U+6F7E.0	fFourCorner	39151
U+6F7E.0	fHakka	 lin2; {lan1}
U+6F7E.0	fCantonese	leon4
U+6F7E.0	fMandarin	lin2; lan2; lin4
U+6F7E.0	fEnglish	clear water
U+6F7F.0	fUTF8	潿
U+6F7F.0	fCNS11643	2-5164
U+6F7F.0	fBig5	E65C
U+6F7F.0	fGB	1-6436
U+6F7F.0	fR/S	 85.12
U+6F7F.0	fTotalStrokes	 15
U+6F7F.0	fCangjie	EWDQ
U+6F7F.0	fFourCorner	36100
U+6F7F.0	fCantonese	wai4
U+6F7F.0	fMandarin	wei2
U+6F7F.0	fEnglish	still water
U+6F80.0	fUTF8	澀
U+6F80.0	fCNS11643	1-742A
U+6F80.0	fBig5	C0DF
U+6F80.0	fGB	1-492C
U+6F80.0	fR/S	 85.14; 85.12
U+6F80.0	fTotalStrokes	 17
U+6F80.0	fCangjie	ESIM
U+6F80.0	fFourCorner	37111
U+6F80.0	fHakka	  sep7
U+6F80.0	fCantonese	gip3; sik1 (also); sap1 (also); saap3; sap3
U+6F80.0	fMandarin	se4
U+6F80.0	fEnglish	[1] harsh; uneven; rough; not smooth [2] a slightly bitter taste that numbs the tongue (unripened fruits); puckery [3] difficult; jolting (writing, reading, etc.) [4] slow of tongue
U+6F81.0	fUTF8	澁
U+6F81.0	fCNS11643	3-4C7A
U+6F81.0	fGB	E-3124
U+6F81.0	fR/S	 85.12
U+6F81.0	fTotalStrokes	 15
U+6F81.0	fCangjie	EYMM
U+6F81.0	fFourCorner	31110
U+6F81.0	fHakka	  sep7
U+6F81.0	fCantonese	gip3; sik1 (also); sap1 (also); kip3
U+6F81.0	fMandarin	se4
U+6F81.0	fEnglish	 astringent harsh uneven; rough
U+6F82.0	fUTF8	澂
U+6F82.0	fCNS11643	2-5152
U+6F82.0	fBig5	E64A
U+6F82.0	fGB	8-2D58
U+6F82.0	fR/S	 85.12
U+6F82.0	fTotalStrokes	 15
U+6F82.0	fCangjie	EUGK
U+6F82.0	fFourCorner	38140
U+6F82.0	fHakka	 chin2; zhin1; zihn1; cihn2
U+6F82.0	fCantonese	cing4
U+6F82.0	fMandarin	cheng2; deng4
U+6F82.0	fEnglish	same as $ (clear and still water)
U+6F83.0	fUTF8	澃
U+6F83.0	fCNS11643	4-5075
U+6F83.0	fGB	E-3125
U+6F83.0	fR/S	  85.12
U+6F83.0	fTotalStrokes	 16
U+6F83.0	fCangjie	HCE
U+6F83.0	fFourCorner	21230
U+6F83.0	fCantonese	gwing3
U+6F83.0	fMandarin	jiong4; jiong3
U+6F84.0	fUTF8	澄
U+6F84.0	fCNS11643	1-6D4B
U+6F84.0	fBig5	BCE1
U+6F84.0	fGB	0-334E
U+6F84.0	fR/S	 85.12
U+6F84.0	fTotalStrokes	 15
U+6F84.0	fCangjie	ENOT
U+6F84.0	fFourCorner	32118
U+6F84.0	fHakka	 chin2; cihn2; chin3; cin2; cang2; can2
U+6F84.0	fCantonese	cing4; dang6
U+6F84.0	fMandarin	cheng2; deng4; zheng4
U+6F84.0	fEnglish	[1] clear and still water [2] purify water by allowing sedimentation; purify water by letting the impurities settle down to the bottom [3] pacify
U+6F85.0	fUTF8	澅
U+6F85.0	fCNS11643	2-514D
U+6F85.0	fBig5	E645
U+6F85.0	fGB	3-5455
U+6F85.0	fR/S	 85.12
U+6F85.0	fTotalStrokes	 15
U+6F85.0	fCangjie	ELGM
U+6F85.0	fFourCorner	35116
U+6F85.0	fCantonese	waa6
U+6F85.0	fMandarin	hua4
U+6F86.0	fUTF8	澆
U+6F86.0	fCNS11643	1-6D4F
U+6F86.0	fBig5	BCE5
U+6F86.0	fGB	1-3D3D
U+6F86.0	fR/S	 85.12
U+6F86.0	fTotalStrokes	 15
U+6F86.0	fCangjie	EGGU
U+6F86.0	fFourCorner	34111
U+6F86.0	fHakka	  hiau1; ngiau2; {giau1}; shau1; sau1; hieu1
U+6F86.0	fCantonese	hiu1; giu1; jiu6; ngou6
U+6F86.0	fMandarin	jiao1; ao4; nao4; ao2; liao2; yao4
U+6F86.0	fEnglish	[1] water (plants, flowers, etc.) [2] spray water; sprinkle water on [3] perfidious; faithless; ungrateful
U+6F87.0	fUTF8	澇
U+6F87.0	fCNS11643	2-5145
U+6F87.0	fBig5	E5FC
U+6F87.0	fGB	1-4054
U+6F87.0	fR/S	 85.12
U+6F87.0	fTotalStrokes	 15
U+6F87.0	fCangjie	EFFS
U+6F87.0	fFourCorner	39127
U+6F87.0	fHakka	 lau5; lau2; lau3; lan2
U+6F87.0	fCantonese	lou6; lou4
U+6F87.0	fMandarin	lao4; lao2; lao3
U+6F87.0	fEnglish	[1] rot in the field owing to floods; waterlogged [2] flood; torrent
U+6F88.0	fUTF8	澈
U+6F88.0	fCNS11643	1-6952
U+6F88.0	fBig5	BAAB
U+6F88.0	fGB	0-333A
U+6F88.0	fR/S	 85.12
U+6F88.0	fTotalStrokes	 15
U+6F88.0	fCangjie	EYBK
U+6F88.0	fFourCorner	38140
U+6F88.0	fHakka	 cet8; chat8; chet8
U+6F88.0	fCantonese	cit3
U+6F88.0	fMandarin	che4
U+6F88.0	fEnglish	[1] thoroughly; completely [2] clear water [3] understand
U+6F89.0	fUTF8	澉
U+6F89.0	fCNS11643	2-5149
U+6F89.0	fBig5	E641
U+6F89.0	fGB	0-6477
U+6F89.0	fR/S	 85.12
U+6F89.0	fTotalStrokes	 15
U+6F89.0	fCangjie	EMJK
U+6F89.0	fFourCorner	38140
U+6F89.0	fHakka	 gam3; gam5
U+6F89.0	fCantonese	gam2
U+6F89.0	fMandarin	gan3; tan4; han4
U+6F89.0	fEnglish	 wash
U+6F8A.0	fUTF8	澊
U+6F8A.0	fCNS11643	4-4A57
U+6F8A.0	fGB	3-563B
U+6F8A.0	fR/S	  85.12
U+6F8A.0	fTotalStrokes	 15
U+6F8A.0	fCangjie	ETWI
U+6F8A.0	fFourCorner	38140
U+6F8A.0	fMandarin	cun1
U+6F8B.0	fUTF8	澋
U+6F8B.0	fCNS11643	2-5162
U+6F8B.0	fBig5	E65A
U+6F8B.0	fGB	5-566C
U+6F8B.0	fR/S	 85.12
U+6F8B.0	fTotalStrokes	 15
U+6F8B.0	fCangjie	EAYF
U+6F8B.0	fFourCorner	36196
U+6F8B.0	fCantonese	ging2
U+6F8B.0	fMandarin	heng4; jing3
U+6F8C.0	fUTF8	澌
U+6F8C.0	fCNS11643	2-514A
U+6F8C.0	fBig5	E642
U+6F8C.0	fGB	0-6479
U+6F8C.0	fR/S	 85.12
U+6F8C.0	fTotalStrokes	 15
U+6F8C.0	fCangjie	ETCL
U+6F8C.0	fFourCorner	32121
U+6F8C.0	fHakka	 sii1; se1
U+6F8C.0	fCantonese	si1; si3
U+6F8C.0	fMandarin	si1; xi1; si4
U+6F8C.0	fEnglish	[1] exhaust; drain out [2] the sound of breaking or scattering
U+6F8D.0	fUTF8	澍
U+6F8D.0	fCNS11643	2-5148
U+6F8D.0	fBig5	E640
U+6F8D.0	fGB	0-6478
U+6F8D.0	fR/S	 85.12
U+6F8D.0	fTotalStrokes	 15
U+6F8D.0	fCangjie	EGTI
U+6F8D.0	fFourCorner	34100
U+6F8D.0	fHakka	 zhu5; zu5; shu5; shu6; su5
U+6F8D.0	fCantonese	syu6
U+6F8D.0	fMandarin	shu4; zhu4
U+6F8D.0	fEnglish	[1] seasonal rains; timely rain [2] saturated with rain water (plants, etc.)
U+6F8E.0	fUTF8	澎
U+6F8E.0	fCNS11643	1-6D54
U+6F8E.0	fBig5	BCEA
U+6F8E.0	fGB	0-456C
U+6F8E.0	fR/S	 85.12
U+6F8E.0	fTotalStrokes	 15
U+6F8E.0	fCangjie	EGTH
U+6F8E.0	fFourCorner	32122
U+6F8E.0	fHakka	 pang2; pen2
U+6F8E.0	fCantonese	paang1; paang4
U+6F8E.0	fMandarin	peng1
U+6F8E.0	fEnglish	the roaring of colliding billows
U+6F8E.1	fUTF8	澎
U+6F8E.1	fCNS11643	1-6D54
U+6F8E.1	fBig5	BCEA
U+6F8E.1	fGB	0-456C
U+6F8E.1	fR/S	 85.12
U+6F8E.1	fTotalStrokes	 15
U+6F8E.1	fCangjie	EGTH
U+6F8E.1	fFourCorner	32122
U+6F8E.1	fMandarin	peng2
U+6F8E.1	fEnglish	the Pescadores
U+6F8F.0	fUTF8	澏
U+6F8F.0	fCNS11643	3-4C7B
U+6F8F.0	fGB	5-566E
U+6F8F.0	fR/S	  85.12
U+6F8F.0	fTotalStrokes	 15
U+6F8F.0	fCangjie	ERRU
U+6F8F.0	fFourCorner	36170
U+6F8F.0	fMandarin	han4
U+6F90.0	fUTF8	澐
U+6F90.0	fCNS11643	2-5160
U+6F90.0	fBig5	E658
U+6F90.0	fGB	E-3126
U+6F90.0	fR/S	 85.12
U+6F90.0	fTotalStrokes	 15
U+6F90.0	fCangjie	EMBI
U+6F90.0	fFourCorner	31131
U+6F90.0	fHakka	 jun2; run2
U+6F90.0	fCantonese	wan4
U+6F90.0	fMandarin	yun2
U+6F90.0	fEnglish	billows
U+6F91.0	fUTF8	澑
U+6F91.0	fCNS11643	3-4C77
U+6F91.0	fGB	E-3127
U+6F91.0	fR/S	 85.12
U+6F91.0	fTotalStrokes	 15
U+6F91.0	fCangjie	EMLW
U+6F91.0	fFourCorner	31160
U+6F91.0	fHakka	 liu5; liu1
U+6F91.0	fMandarin	liu1; liu4; liu2
U+6F91.0	fEnglish	 slide; glide; slip slippery
U+6F92.0	fUTF8	澒
U+6F92.0	fCNS11643	2-5147
U+6F92.0	fBig5	E5FE
U+6F92.0	fGB	5-5624
U+6F92.0	fR/S	 85.12
U+6F92.0	fTotalStrokes	 15
U+6F92.0	fCangjie	EMMC
U+6F92.0	fFourCorner	31186
U+6F92.0	fHakka	  fung3; hong6; hong5
U+6F92.0	fCantonese	hung6; hung3
U+6F92.0	fMandarin	hong4; hong3; xiang4; gong3
U+6F92.0	fEnglish	vast
U+6F93.0	fUTF8	澓
U+6F93.0	fCNS11643	2-5159
U+6F93.0	fBig5	E651
U+6F93.0	fGB	3-5635
U+6F93.0	fR/S	 85.12
U+6F93.0	fTotalStrokes	 15
U+6F93.0	fCangjie	EHOE
U+6F93.0	fFourCorner	38147
U+6F93.0	fCantonese	fuk6
U+6F93.0	fMandarin	fu2; fu4
U+6F94.0	fUTF8	澔
U+6F94.0	fCNS11643	2-5158
U+6F94.0	fBig5	E650
U+6F94.0	fGB	5-5673
U+6F94.0	fR/S	 85.12
U+6F94.0	fTotalStrokes	 15
U+6F94.0	fCangjie	EHAR
U+6F94.0	fFourCorner	34161
U+6F94.0	fCantonese	hou6; hou4
U+6F94.0	fMandarin	hao4
U+6F94.0	fEnglish	radiance of gems
U+6F95.0	fUTF8	澕
U+6F95.0	fCNS11643	2-5165
U+6F95.0	fBig5	E65D
U+6F95.0	fGB	5-553F
U+6F95.0	fR/S	 85.12
U+6F95.0	fTotalStrokes	 15
U+6F95.0	fCangjie	ETMJ
U+6F95.0	fFourCorner	34154
U+6F95.0	fCantonese	wo4
U+6F95.0	fMandarin	he2
U+6F96.0	fUTF8	澖
U+6F96.0	fCNS11643	2-514F
U+6F96.0	fBig5	E647
U+6F96.0	fGB	E-3128
U+6F96.0	fR/S	 85.12
U+6F96.0	fTotalStrokes	 15
U+6F96.0	fCangjie	EAND
U+6F96.0	fFourCorner	37120
U+6F96.0	fCantonese	haan4
U+6F96.0	fMandarin	xian1
U+6F97.0	fUTF8	澗
U+6F97.0	fCNS11643	1-6D58
U+6F97.0	fBig5	BCEE
U+6F97.0	fGB	1-3D27
U+6F97.0	fR/S	 85.12
U+6F97.0	fTotalStrokes	 15
U+6F97.0	fCangjie	EANA
U+6F97.0	fFourCorner	37120
U+6F97.0	fHakka	  gan3; gan1; gian3
U+6F97.0	fCantonese	gaan3; gaan2
U+6F97.0	fMandarin	jian4
U+6F97.0	fEnglish	 mountain stream; brook
U+6F98.0	fUTF8	澘
U+6F98.0	fCNS11643	4-4A64
U+6F98.0	fGB	5-566A
U+6F98.0	fR/S	  85.12
U+6F98.0	fTotalStrokes	 15
U+6F98.0	fCangjie	EDDA
U+6F98.0	fFourCorner	34160
U+6F98.0	fMandarin	shan1
U+6F99.0	fUTF8	澙
U+6F99.0	fCNS11643	3-6547
U+6F99.0	fGB	E-3129
U+6F99.0	fR/S	 85.13
U+6F99.0	fTotalStrokes	 16
U+6F99.0	fCangjie	XEHXF
U+6F99.0	fMandarin	xi4
U+6F9A.0	fUTF8	澚
U+6F9A.0	fCNS11643	F-4F7D
U+6F9A.0	fGB	E-312A
U+6F9A.0	fR/S	 85.12
U+6F9A.0	fTotalStrokes	 15
U+6F9A.0	fCangjie	EHBS
U+6F9A.0	fFourCorner	3712
U+6F9A.0	fMandarin	ao4
U+6F9B.0	fUTF8	澛
U+6F9B.0	fGB	8-2E60
U+6F9B.0	fR/S	 85.12; 85.15
U+6F9B.0	fTotalStrokes	 15
U+6F9B.0	fCangjie	XENWA
U+6F9B.0	fMandarin	lu3
U+6F9C.0	fUTF8	澜
U+6F9C.0	fGB	0-403D
U+6F9C.0	fR/S	 85.12
U+6F9C.0	fTotalStrokes	 15
U+6F9C.0	fCangjie	ELSW
U+6F9C.0	fHakka	 lan2
U+6F9C.0	fCantonese	laan4; laan6
U+6F9C.0	fMandarin	lan2
U+6F9C.0	fEnglish	simplified form of &#x703E;
U+6F9D.0	fUTF8	澝
U+6F9D.0	fCNS11643	3-6546
U+6F9D.0	fGB	E-312B
U+6F9D.0	fR/S	 85.12
U+6F9D.0	fTotalStrokes	 15
U+6F9D.0	fCangjie	EJPB
U+6F9D.0	fFourCorner	3312
U+6F9D.0	fEnglish	 mud miry; muddy; stagnant
U+6F9E.0	fUTF8	澞
U+6F9E.0	fCNS11643	2-577E
U+6F9E.0	fBig5	E9F3
U+6F9E.0	fGB	5-5727
U+6F9E.0	fR/S	 85.13
U+6F9E.0	fTotalStrokes	 16
U+6F9E.0	fCangjie	EYPK
U+6F9E.0	fFourCorner	31134
U+6F9E.0	fHakka	 ngi2; ngu2
U+6F9E.0	fCantonese	jyu4; ji4
U+6F9E.0	fMandarin	yu2
U+6F9E.0	fEnglish	rivulet running between two high banks
U+6F9F.0	fUTF8	澟
U+6F9F.0	fCNS11643	3-5156
U+6F9F.0	fGB	E-312C
U+6F9F.0	fR/S	  85.13
U+6F9F.0	fTotalStrokes	 16
U+6F9F.0	fCangjie	EYWD
U+6F9F.0	fFourCorner	30190
U+6F9F.0	fMandarin	lin3
U+6F9F.0	fEnglish	[1] desolate; deserted [2] cold; chilly
U+6FA0.0	fUTF8	澠
U+6FA0.0	fCNS11643	1-7133
U+6FA0.0	fBig5	BF49
U+6FA0.0	fGB	1-6445
U+6FA0.0	fR/S	 85.13
U+6FA0.0	fTotalStrokes	 16
U+6FA0.0	fCangjie	ERXU
U+6FA0.0	fFourCorner	37117
U+6FA0.0	fHakka	 shun2; sin2; siin2; men3; sun2; mien3
U+6FA0.0	fCantonese	man5; sing4; min5
U+6FA0.0	fMandarin	sheng2; mian3
U+6FA0.0	fEnglish	a river in Shantung province
U+6FA0.1	fUTF8	澠
U+6FA0.1	fCNS11643	1-7133
U+6FA0.1	fBig5	BF49
U+6FA0.1	fGB	1-6445
U+6FA0.1	fR/S	 85.13
U+6FA0.1	fTotalStrokes	 16
U+6FA0.1	fCangjie	ERXU
U+6FA0.1	fFourCorner	37117
U+6FA0.1	fMandarin	min3
U+6FA0.1	fEnglish	[1] the ancient name of a place in Honan province where the kings of Chin and Chao of the Warring States period held their summit conference [2] a river in Honan province [3] a county in Honan province
U+6FA1.0	fUTF8	澡
U+6FA1.0	fCNS11643	1-7129
U+6FA1.0	fBig5	BEFE
U+6FA1.0	fGB	0-5468
U+6FA1.0	fR/S	 85.13
U+6FA1.0	fTotalStrokes	 16
U+6FA1.0	fCangjie	ERRD
U+6FA1.0	fFourCorner	36194
U+6FA1.0	fHakka	  cau5; co5; zau3
U+6FA1.0	fCantonese	cou3; zou2
U+6FA1.0	fMandarin	zao3; cao1
U+6FA1.0	fEnglish	 wash; bathe
U+6FA2.0	fUTF8	澢
U+6FA2.0	fCNS11643	2-582C
U+6FA2.0	fBig5	EA40
U+6FA2.0	fGB	5-553A
U+6FA2.0	fR/S	 85.13
U+6FA2.0	fTotalStrokes	 16
U+6FA2.0	fCangjie	EFBW
U+6FA2.0	fFourCorner	39166
U+6FA2.0	fCantonese	dong1
U+6FA2.0	fMandarin	dang1
U+6FA3.0	fUTF8	澣
U+6FA3.0	fCNS11643	2-5776
U+6FA3.0	fBig5	E9EB
U+6FA3.0	fGB	E-312D
U+6FA3.0	fR/S	 85.13
U+6FA3.0	fTotalStrokes	 16
U+6FA3.0	fCangjie	EJJJ
U+6FA3.0	fFourCorner	38141
U+6FA3.0	fHakka	  fon5;  fon3; ren2; jen2; jen5; fan5
U+6FA3.0	fCantonese	wun5; wun2 (also)
U+6FA3.0	fMandarin	huan3; han4; wan3; huan4; guan3
U+6FA3.0	fEnglish	[1] cleanse; wash [2] ten days (Tang dynasty)
U+6FA4.0	fUTF8	澤
U+6FA4.0	fCNS11643	1-712B
U+6FA4.0	fBig5	BF41
U+6FA4.0	fGB	1-5473
U+6FA4.0	fR/S	 85.13
U+6FA4.0	fTotalStrokes	 16
U+6FA4.0	fCangjie	EWLJ
U+6FA4.0	fFourCorner	36141
U+6FA4.0	fHakka	  cet8; cet7; cak8
U+6FA4.0	fCantonese	zaak6; sik1
U+6FA4.0	fMandarin	ze2; duo2; shi4; yi4
U+6FA4.0	fEnglish	[1] marsh; swamp; the place where water gathers [2] grace; favors; kindness [3] luster; bright; brilliance; radiance; glossy; smooth [4] benefit; enrich
U+6FA5.0	fUTF8	澥
U+6FA5.0	fCNS11643	2-5824
U+6FA5.0	fBig5	E9F7
U+6FA5.0	fGB	3-564A
U+6FA5.0	fR/S	 85.13
U+6FA5.0	fTotalStrokes	 16
U+6FA5.0	fCangjie	ENBQ
U+6FA5.0	fFourCorner	37152
U+6FA5.0	fHakka	 hai3; gai3; hai5
U+6FA5.0	fCantonese	haai5; haai6
U+6FA5.0	fMandarin	xie4
U+6FA5.0	fEnglish	[1] blocked stream [2] stop the flow of water [3] another name of Pohai
U+6FA6.0	fUTF8	澦
U+6FA6.0	fCNS11643	1-7132
U+6FA6.0	fBig5	BF48
U+6FA6.0	fGB	3-5565
U+6FA6.0	fR/S	 85.13
U+6FA6.0	fTotalStrokes	 16
U+6FA6.0	fCangjie	ENNC
U+6FA6.0	fFourCorner	31186
U+6FA6.0	fHakka	 ji5; ri6
U+6FA6.0	fCantonese	jyu6
U+6FA6.0	fMandarin	yu4
U+6FA6.0	fEnglish	a place in Sichuan
U+6FA7.0	fUTF8	澧
U+6FA7.0	fCNS11643	1-712D
U+6FA7.0	fBig5	BF43
U+6FA7.0	fGB	0-6522
U+6FA7.0	fR/S	 85.13
U+6FA7.0	fTotalStrokes	 16
U+6FA7.0	fCangjie	ETWT
U+6FA7.0	fFourCorner	35118
U+6FA7.0	fHakka	 li3; li1
U+6FA7.0	fCantonese	lai5
U+6FA7.0	fMandarin	li3
U+6FA7.0	fEnglish	[1] fountain; sprig [2] a river in northern Honan province
U+6FA8.0	fUTF8	澨
U+6FA8.0	fCNS11643	2-5822
U+6FA8.0	fBig5	E9F5
U+6FA8.0	fGB	3-5648
U+6FA8.0	fR/S	 85.13
U+6FA8.0	fTotalStrokes	 16
U+6FA8.0	fCangjie	EHMO
U+6FA8.0	fFourCorner	38118
U+6FA8.0	fHakka	 shi6; sii5; chi5; ci5; chi6; chii5
U+6FA8.0	fCantonese	sai6
U+6FA8.0	fMandarin	shi4; yi4
U+6FA8.0	fEnglish	[1] water side; water front; shore [2] a river in Hopeh province
U+6FA9.0	fUTF8	澩
U+6FA9.0	fCNS11643	2-5D3B
U+6FA9.0	fBig5	ED4F
U+6FA9.0	fGB	1-6D34
U+6FA9.0	fR/S	 85.13
U+6FA9.0	fTotalStrokes	 17
U+6FA9.0	fCangjie	HBE
U+6FA9.0	fFourCorner	77232
U+6FA9.0	fHakka	 hok8; hok7
U+6FA9.0	fCantonese	hok6
U+6FA9.0	fMandarin	xue2; zhuo2; xiao4
U+6FA9.0	fEnglish	 mountain current
U+6FAA.0	fUTF8	澪
U+6FAA.0	fCNS11643	2-5828
U+6FAA.0	fBig5	E9FB
U+6FAA.0	fGB	3-5642
U+6FAA.0	fR/S	 85.13
U+6FAA.0	fTotalStrokes	 16
U+6FAA.0	fCangjie	EMBI
U+6FAA.0	fFourCorner	31137
U+6FAA.0	fCantonese	ling4
U+6FAA.0	fMandarin	ling2
U+6FAB.0	fUTF8	澫
U+6FAB.0	fCNS11643	2-582E
U+6FAB.0	fBig5	EA42
U+6FAB.0	fGB	5-546F
U+6FAB.0	fR/S	 85.13
U+6FAB.0	fTotalStrokes	 16
U+6FAB.0	fCangjie	ETWB
U+6FAB.0	fFourCorner	34127
U+6FAB.0	fCantonese	maan6; maan4; ngaau5
U+6FAB.0	fMandarin	man4
U+6FAC.0	fUTF8	澬
U+6FAC.0	fCNS11643	2-5827
U+6FAC.0	fBig5	E9FA
U+6FAC.0	fGB	3-555C
U+6FAC.0	fR/S	 85.13
U+6FAC.0	fTotalStrokes	 16
U+6FAC.0	fCangjie	EIOC
U+6FAC.0	fFourCorner	37186
U+6FAC.0	fCantonese	zi1; ci4
U+6FAC.0	fMandarin	zi1; ci2
U+6FAD.0	fUTF8	澭
U+6FAD.0	fCNS11643	2-5774
U+6FAD.0	fBig5	E9E9
U+6FAD.0	fGB	5-572B
U+6FAD.0	fR/S	 85.13
U+6FAD.0	fTotalStrokes	 16
U+6FAD.0	fCangjie	EYVG
U+6FAD.0	fFourCorner	30114
U+6FAD.0	fHakka	 jung1; rung1
U+6FAD.0	fCantonese	jung1
U+6FAD.0	fMandarin	yong1; yong3; yong4
U+6FAD.0	fEnglish	 sluice for diverting the water of a river
U+6FAE.0	fUTF8	澮
U+6FAE.0	fCNS11643	2-5825
U+6FAE.0	fBig5	E9F8
U+6FAE.0	fGB	1-642B
U+6FAE.0	fR/S	 85.13
U+6FAE.0	fTotalStrokes	 16
U+6FAE.0	fCangjie	EOMA
U+6FAE.0	fFourCorner	38166
U+6FAE.0	fHakka	  kwai5
U+6FAE.0	fCantonese	kui2; kwui2 (also); kui3
U+6FAE.0	fMandarin	kuai4; hui4; hua2; guai4
U+6FAE.0	fEnglish	[1] ditches on farmland; irrigation ditch [2] a river in Shansi and Honan provinces
U+6FAF.0	fUTF8	澯
U+6FAF.0	fCNS11643	2-5830
U+6FAF.0	fBig5	EA44
U+6FAF.0	fGB	5-5726
U+6FAF.0	fR/S	 85.13
U+6FAF.0	fTotalStrokes	 16
U+6FAF.0	fCangjie	EYED
U+6FAF.0	fFourCorner	37194
U+6FAF.0	fCantonese	caan3
U+6FAF.0	fMandarin	can4
U+6FB0.0	fUTF8	澰
U+6FB0.0	fCNS11643	2-5832
U+6FB0.0	fBig5	EA46
U+6FB0.0	fGB	3-5440
U+6FB0.0	fR/S	 85.13
U+6FB0.0	fTotalStrokes	 16
U+6FB0.0	fCangjie	EOMO
U+6FB0.0	fFourCorner	38186
U+6FB0.0	fHakka	 liam3
U+6FB0.0	fCantonese	lim6
U+6FB0.0	fMandarin	lian4; lan3
U+6FB0.0	fEnglish	overflowing
U+6FB1.0	fUTF8	澱
U+6FB1.0	fCNS11643	1-7128
U+6FB1.0	fBig5	BEFD
U+6FB1.0	fGB	1-356D
U+6FB1.0	fR/S	 85.13
U+6FB1.0	fTotalStrokes	 16
U+6FB1.0	fCangjie	ESCE
U+6FB1.0	fFourCorner	37147
U+6FB1.0	fHakka	 tin2; tien5; ten3; tien6
U+6FB1.0	fCantonese	din6
U+6FB1.0	fMandarin	dian4
U+6FB1.0	fEnglish	[1] sediment; dregs; precipitate [2] indigo
U+6FB2.0	fUTF8	澲
U+6FB2.0	fCNS11643	2-5831
U+6FB2.0	fBig5	EA45
U+6FB2.0	fGB	5-5525
U+6FB2.0	fR/S	 85.13
U+6FB2.0	fTotalStrokes	 16
U+6FB2.0	fCangjie	ETCD
U+6FB2.0	fFourCorner	32194
U+6FB2.0	fCantonese	jip6
U+6FB2.0	fMandarin	ye4
U+6FB3.0	fUTF8	澳
U+6FB3.0	fCNS11643	1-712E
U+6FB3.0	fBig5	BF44
U+6FB3.0	fGB	0-3044
U+6FB3.0	fR/S	 85.13; 85.12
U+6FB3.0	fTotalStrokes	 16
U+6FB3.0	fCangjie	EHBK
U+6FB3.0	fFourCorner	37134
U+6FB3.0	fHakka	 au5;  o5; juk7; ruk7
U+6FB3.0	fCantonese	ou3; ngou3 (also); juk1
U+6FB3.0	fMandarin	ao4; yu4
U+6FB3.0	fEnglish	[1] deep waters, where sea-going vessels cannot moor [2] the name of various places
U+6FB4.0	fUTF8	澴
U+6FB4.0	fCNS11643	1-7134
U+6FB4.0	fBig5	BF4A
U+6FB4.0	fGB	3-5646
U+6FB4.0	fR/S	 85.13
U+6FB4.0	fTotalStrokes	 16
U+6FB4.0	fCangjie	EWLV
U+6FB4.0	fFourCorner	36132
U+6FB4.0	fCantonese	waan4
U+6FB4.0	fMandarin	huan2; xuan1; xuan4; yung4
U+6FB4.0	fEnglish	a river in Hubei province
U+6FB5.0	fUTF8	澵
U+6FB5.0	fCNS11643	3-5154
U+6FB5.0	fGB	E-312E
U+6FB5.0	fR/S	 85.13
U+6FB5.0	fTotalStrokes	 16
U+6FB5.0	fCangjie	EYDL
U+6FB5.0	fFourCorner	32120
U+6FB5.0	fCantonese	zeon1
U+6FB5.0	fMandarin	zhen1; qin1
U+6FB6.0	fUTF8	澶
U+6FB6.0	fCNS11643	1-7131
U+6FB6.0	fBig5	BF47
U+6FB6.0	fGB	0-6524
U+6FB6.0	fR/S	 85.13
U+6FB6.0	fTotalStrokes	 16
U+6FB6.0	fCangjie	EYWM
U+6FB6.0	fFourCorner	30116
U+6FB6.0	fHakka	 tan5; sham2; sam2; tan3
U+6FB6.0	fCantonese	sin4; sim4 (also); daan3; daan6
U+6FB6.0	fMandarin	chan2; dan4; zhan1; dun4; tan4
U+6FB6.0	fEnglish	[1] still water; placid; tranquil; calm (water) [2] the name of a river
U+6FB7.0	fUTF8	澷
U+6FB7.0	fCNS11643	3-5158
U+6FB7.0	fGB	E-312F
U+6FB7.0	fR/S	 85.13
U+6FB7.0	fTotalStrokes	 16
U+6FB7.0	fCangjie	EAWS
U+6FB7.0	fFourCorner	36120
U+6FB7.0	fCantonese	maan6
U+6FB7.0	fMandarin	man4
U+6FB7.0	fEnglish	 water overflowing diffused spreading
U+6FB8.0	fUTF8	澸
U+6FB8.0	fCNS11643	2-582B
U+6FB8.0	fBig5	E9FE
U+6FB8.0	fGB	E-3130
U+6FB8.0	fR/S	 85.13
U+6FB8.0	fTotalStrokes	 16
U+6FB8.0	fCangjie	EIRP
U+6FB8.0	fFourCorner	33150
U+6FB8.0	fCantonese	dam2
U+6FB8.0	fMandarin	dan3
U+6FB9.0	fUTF8	澹
U+6FB9.0	fCNS11643	1-7130
U+6FB9.0	fBig5	BF46
U+6FB9.0	fGB	0-6523
U+6FB9.0	fR/S	 85.13
U+6FB9.0	fTotalStrokes	 16
U+6FB9.0	fCangjie	ENCR
U+6FB9.0	fFourCorner	37161
U+6FB9.0	fHakka	 tam3; tam2; dam2; dam1; {tam6}; {tam5}
U+6FB9.0	fCantonese	daam6; taam4; taam5
U+6FB9.0	fMandarin	dan4; shan4; dan1; dan2
U+6FB9.0	fEnglish	calm; quiet; tranquil
U+6FB9.1	fUTF8	澹
U+6FB9.1	fCNS11643	1-7130
U+6FB9.1	fBig5	BF46
U+6FB9.1	fGB	0-6523
U+6FB9.1	fR/S	 85.13
U+6FB9.1	fTotalStrokes	 16
U+6FB9.1	fCangjie	ENCR
U+6FB9.1	fFourCorner	37161
U+6FB9.1	fMandarin	tan2
U+6FB9.1	fEnglish	[1] used in combination for a double surname [2] used in combination for a lake in Kiangsu
U+6FBA.0	fUTF8	澺
U+6FBA.0	fCNS11643	2-5826
U+6FBA.0	fBig5	E9F9
U+6FBA.0	fGB	3-564C
U+6FBA.0	fR/S	 85.13
U+6FBA.0	fTotalStrokes	 16
U+6FBA.0	fCangjie	EYTP
U+6FBA.0	fFourCorner	30136
U+6FBA.0	fCantonese	jik1
U+6FBA.0	fMandarin	yi4
U+6FBB.0	fUTF8	澻
U+6FBB.0	fCNS11643	3-514D
U+6FBB.0	fGB	3-563C
U+6FBB.0	fR/S	 85.13
U+6FBB.0	fTotalStrokes	 16
U+6FBB.0	fCangjie	EYTO
U+6FBB.0	fFourCorner	38130
U+6FBB.0	fHakka	 sui5; sui6
U+6FBB.0	fCantonese	seoi6
U+6FBB.0	fMandarin	sui4
U+6FBB.0	fEnglish	 ditch; drain
U+6FBC.0	fUTF8	澼
U+6FBC.0	fCNS11643	2-5778
U+6FBC.0	fBig5	E9ED
U+6FBC.0	fGB	3-564E
U+6FBC.0	fR/S	 85.13
U+6FBC.0	fTotalStrokes	 16
U+6FBC.0	fCangjie	ESRJ
U+6FBC.0	fFourCorner	30141
U+6FBC.0	fHakka	  pit7
U+6FBC.0	fCantonese	pik1
U+6FBC.0	fMandarin	pi4; pi1
U+6FBC.0	fEnglish	launder; wash; clean
U+6FBD.0	fUTF8	澽
U+6FBD.0	fCNS11643	2-577D
U+6FBD.0	fBig5	E9F2
U+6FBD.0	fGB	3-5643
U+6FBD.0	fR/S	 85.13
U+6FBD.0	fTotalStrokes	 16
U+6FBD.0	fCangjie	EYPO
U+6FBD.0	fFourCorner	31132
U+6FBD.0	fCantonese	geoi6; geoi3 (also)
U+6FBD.0	fMandarin	ju4; ju3; qu2
U+6FBE.0	fUTF8	澾
U+6FBE.0	fCNS11643	3-5150
U+6FBE.0	fGB	3-537A
U+6FBE.0	fR/S	 85.13
U+6FBE.0	fTotalStrokes	 16
U+6FBE.0	fCangjie	EYGQ
U+6FBE.0	fFourCorner	34130
U+6FBE.0	fHakka	  dat7;  tat8
U+6FBE.0	fCantonese	taat3; tat3
U+6FBE.0	fMandarin	ta4
U+6FBE.0	fEnglish	slippery; smooth
U+6FBF.0	fUTF8	澿
U+6FBF.0	fCNS11643	2-582A
U+6FBF.0	fBig5	E9FD
U+6FBF.0	fGB	5-5723
U+6FBF.0	fR/S	 85.13
U+6FBF.0	fTotalStrokes	 16
U+6FBF.0	fCangjie	EDDF
U+6FBF.0	fFourCorner	34191
U+6FBF.0	fCantonese	kam4
U+6FBF.0	fMandarin	qin2
U+6FC0.0	fUTF8	激
U+6FC0.0	fCNS11643	1-712F
U+6FC0.0	fBig5	BF45
U+6FC0.0	fGB	0-3C24
U+6FC0.0	fR/S	 85.13
U+6FC0.0	fTotalStrokes	 16
U+6FC0.0	fCangjie	EHSK
U+6FC0.0	fFourCorner	38140
U+6FC0.0	fHakka	  git7; giep8
U+6FC0.0	fCantonese	gik1
U+6FC0.0	fMandarin	ji1; jiao1; jiao4; ji2
U+6FC0.0	fEnglish	[1] stir up; arouse; excite; incite; urge [2] sudden; great; very [3] heated (debate, battle, etc.); fierce; angry; vexed [4] abnormal; unusual; drastic [5] turn back the current (dike)
U+6FC0.1	fUTF8	激
U+6FC0.1	fCNS11643	1-712F
U+6FC0.1	fBig5	BF45
U+6FC0.1	fGB	0-3C24
U+6FC0.1	fR/S	 85.13
U+6FC0.1	fTotalStrokes	 16
U+6FC0.1	fCangjie	EHSK
U+6FC0.1	fFourCorner	38140
U+6FC0.1	fMandarin	ji5
U+6FC1.0	fUTF8	濁
U+6FC1.0	fCNS11643	1-712C
U+6FC1.0	fBig5	BF42
U+6FC1.0	fGB	1-5747
U+6FC1.0	fR/S	 85.13
U+6FC1.0	fTotalStrokes	 16
U+6FC1.0	fCangjie	EWLI
U+6FC1.0	fFourCorner	36127
U+6FC1.0	fHakka	  cuk8; wun2; ch'uk8; ch'ok8; cok8; chuk8; chok8
U+6FC1.0	fCantonese	zuk6
U+6FC1.0	fMandarin	zhuo2
U+6FC1.0	fEnglish	[1] muddy; turbid; dirty; filthy (water) [2] tumultuous; evil; corrupt (world) [3] stupid and idiotic (person)
U+6FC2.0	fUTF8	濂
U+6FC2.0	fCNS11643	1-7127
U+6FC2.0	fBig5	BEFC
U+6FC2.0	fGB	0-6525
U+6FC2.0	fR/S	 85.13
U+6FC2.0	fTotalStrokes	 16
U+6FC2.0	fCangjie	EITC
U+6FC2.0	fFourCorner	30137
U+6FC2.0	fHakka	 liam2
U+6FC2.0	fCantonese	lim4
U+6FC2.0	fMandarin	lian2; xian3
U+6FC2.0	fEnglish	a river in Honan province
U+6FC3.0	fUTF8	濃
U+6FC3.0	fCNS11643	1-712A
U+6FC3.0	fBig5	BF40
U+6FC3.0	fGB	1-4528
U+6FC3.0	fR/S	 85.13
U+6FC3.0	fTotalStrokes	 16
U+6FC3.0	fCangjie	ETWV
U+6FC3.0	fFourCorner	35132
U+6FC3.0	fHakka	 ngiung2; nung2; niung2; nung1; niung5
U+6FC3.0	fCantonese	nung4; lung4 (also); jung4
U+6FC3.0	fMandarin	nong2
U+6FC3.0	fEnglish	[1] thick; strong; concentrated; heavy (drinks, liquids, etc.) [2] deep; dark (colors) [3] dense [4] strong; heavy (smell)
U+6FC4.0	fUTF8	濄
U+6FC4.0	fCNS11643	2-577C
U+6FC4.0	fBig5	E9F1
U+6FC4.0	fGB	5-5539
U+6FC4.0	fR/S	 85.13
U+6FC4.0	fTotalStrokes	 16
U+6FC4.0	fCangjie	EYBB
U+6FC4.0	fFourCorner	37132
U+6FC4.0	fCantonese	wo1
U+6FC4.0	fMandarin	guo1; wo1
U+6FC5.0	fUTF8	濅
U+6FC5.0	fCNS11643	4-5122
U+6FC5.0	fGB	5-572E
U+6FC5.0	fR/S	  85.13
U+6FC5.0	fTotalStrokes	 16
U+6FC5.0	fCangjie	EJSE
U+6FC5.0	fFourCorner	33140
U+6FC5.0	fCantonese	zam3
U+6FC5.0	fMandarin	jin4
U+6FC6.0	fUTF8	濆
U+6FC6.0	fCNS11643	2-5146
U+6FC6.0	fBig5	E5FD
U+6FC6.0	fGB	3-5476
U+6FC6.0	fR/S	 85.13
U+6FC6.0	fTotalStrokes	 16
U+6FC6.0	fCangjie	EJTC
U+6FC6.0	fFourCorner	34186
U+6FC6.0	fHakka	  fun2
U+6FC6.0	fCantonese	fan4; pan3
U+6FC6.0	fMandarin	fen2; fen4; pen1
U+6FC6.0	fEnglish	 river bank a river in Hunan province gush
U+6FC7.0	fUTF8	濇
U+6FC7.0	fCNS11643	2-5777
U+6FC7.0	fBig5	E9EC
U+6FC7.0	fGB	E-3131
U+6FC7.0	fR/S	 85.13
U+6FC7.0	fTotalStrokes	 16
U+6FC7.0	fCangjie	EGOW
U+6FC7.0	fFourCorner	34161
U+6FC7.0	fHakka	 sep7
U+6FC7.0	fCantonese	sik1 (also); sap1 (also); gip3; saap3
U+6FC7.0	fMandarin	se4
U+6FC7.0	fEnglish	 astringent harsh; rough uneven
U+6FC8.0	fUTF8	濈
U+6FC8.0	fCNS11643	2-577A
U+6FC8.0	fBig5	E9EF
U+6FC8.0	fGB	3-562F
U+6FC8.0	fR/S	 85.13
U+6FC8.0	fTotalStrokes	 16
U+6FC8.0	fCangjie	ERJI
U+6FC8.0	fFourCorner	33150
U+6FC8.0	fHakka	  sip8; si5; cip8
U+6FC8.0	fCantonese	cap1; zap1
U+6FC8.0	fMandarin	ji2; chuang2; sha4
U+6FC8.0	fEnglish	many
U+6FC9.0	fUTF8	濉
U+6FC9.0	fCNS11643	2-582D
U+6FC9.0	fBig5	EA41
U+6FC9.0	fGB	0-6521
U+6FC9.0	fR/S	 85.13
U+6FC9.0	fTotalStrokes	 16
U+6FC9.0	fCangjie	EBUG
U+6FC9.0	fFourCorner	30114
U+6FC9.0	fCantonese	seoi1
U+6FC9.0	fMandarin	sui1; hui1; hui4
U+6FCA.0	fUTF8	濊
U+6FCA.0	fCNS11643	2-5821
U+6FCA.0	fBig5	E9F4
U+6FCA.0	fGB	3-5423
U+6FCA.0	fR/S	 85.13
U+6FCA.0	fTotalStrokes	 16
U+6FCA.0	fCangjie	EYMH
U+6FCA.0	fFourCorner	31153
U+6FCA.0	fHakka	 woi5; we5; ve5; voi5
U+6FCA.0	fCantonese	wai3; kut3; kwut3 (also)
U+6FCA.0	fMandarin	hui4; huo4
U+6FCA.0	fEnglish	water plentiful
U+6FCA.1	fUTF8	濊
U+6FCA.1	fCNS11643	2-5821
U+6FCA.1	fBig5	E9F4
U+6FCA.1	fGB	3-5423
U+6FCA.1	fR/S	 85.13
U+6FCA.1	fTotalStrokes	 16
U+6FCA.1	fCangjie	EYMH
U+6FCA.1	fFourCorner	31153
U+6FCA.1	fMandarin	wei4
U+6FCA.1	fEnglish	[1] expansive; deep [2] dirty [3] the name of a river
U+6FCB.0	fUTF8	濋
U+6FCB.0	fCNS11643	2-5775
U+6FCB.0	fBig5	E9EA
U+6FCB.0	fGB	5-5724
U+6FCB.0	fR/S	 85.13
U+6FCB.0	fTotalStrokes	 16
U+6FCB.0	fCangjie	EDDO
U+6FCB.0	fFourCorner	34181
U+6FCB.0	fHakka	 cu3; cih3
U+6FCB.0	fCantonese	co2
U+6FCB.0	fMandarin	chu3
U+6FCB.0	fEnglish	streamlets
U+6FCC.0	fUTF8	濌
U+6FCC.0	fCNS11643	2-5D3A
U+6FCC.0	fBig5	ED4E
U+6FCC.0	fGB	3-532C
U+6FCC.0	fR/S	 85.13
U+6FCC.0	fTotalStrokes	 17
U+6FCC.0	fCangjie	HGEA
U+6FCC.0	fFourCorner	22163
U+6FCC.0	fCantonese	taap3; daap6
U+6FCC.0	fMandarin	ta4
U+6FCD.0	fUTF8	濍
U+6FCD.0	fCNS11643	2-582F
U+6FCD.0	fBig5	EA43
U+6FCD.0	fGB	5-5668
U+6FCD.0	fR/S	 85.13
U+6FCD.0	fTotalStrokes	 16
U+6FCD.0	fCangjie	ETPP
U+6FCD.0	fFourCorner	34132
U+6FCD.0	fCantonese	sung1
U+6FCD.0	fMandarin	song1
U+6FCE.0	fUTF8	濎
U+6FCE.0	fCNS11643	2-5779
U+6FCE.0	fBig5	E9EE
U+6FCE.0	fGB	3-562E
U+6FCE.0	fR/S	 85.13
U+6FCE.0	fTotalStrokes	 16
U+6FCE.0	fCangjie	EBUL
U+6FCE.0	fFourCorner	36127
U+6FCE.0	fHakka	 din3; den3
U+6FCE.0	fCantonese	ding2; ting5; din3
U+6FCE.0	fMandarin	ding3; ting4
U+6FCE.0	fEnglish	streams of water
U+6FCF.0	fUTF8	濏
U+6FCF.0	fCNS11643	2-5829
U+6FCF.0	fBig5	E9FC
U+6FCF.0	fGB	5-567E
U+6FCF.0	fR/S	 85.13
U+6FCF.0	fTotalStrokes	 16
U+6FCF.0	fCangjie	EMGH
U+6FCF.0	fFourCorner	31131
U+6FCF.0	fCantonese	sat1
U+6FCF.0	fMandarin	se4
U+6FD0.0	fUTF8	濐
U+6FD0.0	fCNS11643	3-4C71
U+6FD0.0	fGB	E-3132
U+6FD0.0	fR/S	 85.12
U+6FD0.0	fTotalStrokes	 15
U+6FD0.0	fCangjie	EAJA
U+6FD0.0	fFourCorner	36160
U+6FD0.0	fCantonese	zyu2
U+6FD0.0	fMandarin	zhu3
U+6FD1.0	fUTF8	濑
U+6FD1.0	fGB	0-647E
U+6FD1.0	fR/S	 85.13
U+6FD1.0	fTotalStrokes	 16
U+6FD1.0	fCangjie	EDLO
U+6FD1.0	fHakka	 lai5
U+6FD1.0	fCantonese	laai6
U+6FD1.0	fMandarin	lai4
U+6FD1.0	fEnglish	simplified form of &#x7028;
U+6FD2.0	fUTF8	濒
U+6FD2.0	fGB	0-3174
U+6FD2.0	fR/S	 85.13
U+6FD2.0	fTotalStrokes	 16
U+6FD2.0	fCangjie	EYHO
U+6FD2.0	fHakka	 pin2
U+6FD2.0	fCantonese	ban1; pan4 (also)
U+6FD2.0	fMandarin	bin1; pin2
U+6FD2.0	fEnglish	simplified form of &#x7015;
U+6FD3.0	fUTF8	濓
U+6FD3.0	fCNS11643	3-5155
U+6FD3.0	fGB	E-3133
U+6FD3.0	fR/S	  85.13
U+6FD3.0	fTotalStrokes	 16
U+6FD3.0	fCangjie	EYTC
U+6FD3.0	fFourCorner	30130
U+6FD3.0	fCantonese	lim4
U+6FD3.0	fMandarin	lian2
U+6FD4.0	fUTF8	濔
U+6FD4.0	fCNS11643	2-5D3D
U+6FD4.0	fBig5	ED51
U+6FD4.0	fGB	3-536B
U+6FD4.0	fR/S	 85.14
U+6FD4.0	fTotalStrokes	 17
U+6FD4.0	fCangjie	EMFB
U+6FD4.0	fFourCorner	31127
U+6FD4.0	fHakka	 ni3; ni2
U+6FD4.0	fCantonese	mei4; mei5 (also); nei4 (also); lei4 (also); nei5
U+6FD4.0	fMandarin	mi3; ni3; mi2
U+6FD4.0	fEnglish	many
U+6FD5.0	fUTF8	濕
U+6FD5.0	fCNS11643	1-742E
U+6FD5.0	fBig5	C0E3
U+6FD5.0	fGB	1-4A2A
U+6FD5.0	fR/S	 85.14
U+6FD5.0	fTotalStrokes	 17
U+6FD5.0	fCangjie	EAVF
U+6FD5.0	fFourCorner	36133
U+6FD5.0	fHakka	  ship7;  sip7; siip7
U+6FD5.0	fCantonese	sap1
U+6FD5.0	fMandarin	shi1; ta4; xi2; qi4; xie2
U+6FD5.0	fEnglish	[1] wet; moist; humid; damp; get wet [2] ailments caused by high humidity (Chinese medicine)
U+6FD6.0	fUTF8	濖
U+6FD6.0	fCNS11643	3-5530
U+6FD6.0	fGB	3-5645
U+6FD6.0	fR/S	  85.13
U+6FD6.0	fTotalStrokes	 16; 17
U+6FD6.0	fCangjie	EWLA
U+6FD6.0	fFourCorner	36160
U+6FD6.0	fCantonese	syu6
U+6FD6.0	fMandarin	shu4
U+6FD7.0	fUTF8	濗
U+6FD7.0	fCNS11643	3-552F
U+6FD7.0	fGB	E-3134
U+6FD7.0	fR/S	  85.14
U+6FD7.0	fTotalStrokes	 17
U+6FD7.0	fCangjie	ETAB
U+6FD7.0	fFourCorner	34120
U+6FD7.0	fCantonese	mik6
U+6FD7.0	fMandarin	mi4
U+6FD8.0	fUTF8	濘
U+6FD8.0	fCNS11643	1-7422
U+6FD8.0	fBig5	C0D7
U+6FD8.0	fGB	1-4522
U+6FD8.0	fR/S	 85.14
U+6FD8.0	fTotalStrokes	 17
U+6FD8.0	fCangjie	EJPN
U+6FD8.0	fFourCorner	33121
U+6FD8.0	fHakka	  nen5; nen2; nem2; len2; nin5
U+6FD8.0	fCantonese	ning6; ling6 (also); ning4 (also); ling4 (also)
U+6FD8.0	fMandarin	ning4; ning2; ni4; ning3
U+6FD8.0	fEnglish	miry; muddy
U+6FD8.1	fUTF8	濘
U+6FD8.1	fCNS11643	1-7422
U+6FD8.1	fBig5	C0D7
U+6FD8.1	fGB	1-4522
U+6FD8.1	fR/S	 85.14
U+6FD8.1	fTotalStrokes	 17
U+6FD8.1	fCangjie	EJPN
U+6FD8.1	fFourCorner	33121
U+6FD8.1	fMandarin	neng4
U+6FD8.1	fEnglish	pasty; soft and mashy
U+6FD9.0	fUTF8	濙
U+6FD9.0	fCNS11643	3-5535
U+6FD9.0	fGB	3-5479
U+6FD9.0	fR/S	 85.14
U+6FD9.0	fTotalStrokes	 17
U+6FD9.0	fCangjie	EFFF
U+6FD9.0	fFourCorner	39189
U+6FD9.0	fHakka	 jung2
U+6FD9.0	fCantonese	jing4
U+6FD9.0	fMandarin	ying2; ying4
U+6FD9.0	fEnglish	 revolve to curl in eddies; as rushing water
U+6FDA.0	fUTF8	濚
U+6FDA.0	fCNS11643	3-5536
U+6FDA.0	fGB	5-5628
U+6FDA.0	fR/S	 85.14
U+6FDA.0	fTotalStrokes	 17
U+6FDA.0	fCangjie	EFFD
U+6FDA.0	fFourCorner	39190
U+6FDA.0	fCantonese	jing4
U+6FDA.0	fMandarin	ying2; xing2
U+6FDB.0	fUTF8	濛
U+6FDB.0	fCNS11643	1-7426
U+6FDB.0	fBig5	C0DB
U+6FDB.0	fGB	1-7855
U+6FDB.0	fR/S	 85.14
U+6FDB.0	fTotalStrokes	 17
U+6FDB.0	fCangjie	ETBO
U+6FDB.0	fFourCorner	34132
U+6FDB.0	fHakka	 mung2
U+6FDB.0	fCantonese	mung4
U+6FDB.0	fMandarin	meng2; meng3
U+6FDB.0	fEnglish	drizzly; misty
U+6FDC.0	fUTF8	濜
U+6FDC.0	fCNS11643	2-5D3F
U+6FDC.0	fBig5	ED53
U+6FDC.0	fGB	3-5431
U+6FDC.0	fR/S	 85.14
U+6FDC.0	fTotalStrokes	 17
U+6FDC.0	fCangjie	ELMT
U+6FDC.0	fFourCorner	35117
U+6FDC.0	fHakka	 cin5; cin6
U+6FDC.0	fCantonese	zeon6
U+6FDC.0	fMandarin	jin4; jin1
U+6FDC.0	fEnglish	a river in Hubei province
U+6FDD.0	fUTF8	濝
U+6FDD.0	fCNS11643	2-5D45
U+6FDD.0	fBig5	ED59
U+6FDD.0	fGB	E-3135
U+6FDD.0	fR/S	 85.14
U+6FDD.0	fTotalStrokes	 17
U+6FDD.0	fCangjie	ETCF
U+6FDD.0	fFourCorner	34194
U+6FDD.0	fCantonese	kei4
U+6FDD.0	fMandarin	qi2
U+6FDE.0	fUTF8	濞
U+6FDE.0	fCNS11643	2-5D43
U+6FDE.0	fBig5	ED57
U+6FDE.0	fGB	0-6528
U+6FDE.0	fR/S	 85.14
U+6FDE.0	fTotalStrokes	 17
U+6FDE.0	fCangjie	EHUL
U+6FDE.0	fFourCorner	36146
U+6FDE.0	fHakka	 pi5; pi6; pi3; pui5
U+6FDE.0	fCantonese	pei3; bai3
U+6FDE.0	fMandarin	pi4; bi4
U+6FDF.0	fUTF8	濟
U+6FDF.0	fCNS11643	1-7424
U+6FDF.0	fBig5	C0D9
U+6FDF.0	fGB	1-3C43
U+6FDF.0	fR/S	 85.14
U+6FDF.0	fTotalStrokes	 17
U+6FDF.0	fCangjie	EYX
U+6FDF.0	fFourCorner	30123
U+6FDF.0	fHakka	  zi5
U+6FDF.0	fCantonese	zai3; zai2
U+6FDF.0	fMandarin	ji4; qi2
U+6FDF.0	fEnglish	[1] help; aid; relieve [2] ferry; cross a stream [3] succeed; be up to standard [4] benefit [5] ford
U+6FDF.1	fUTF8	濟
U+6FDF.1	fCNS11643	1-7424
U+6FDF.1	fBig5	C0D9
U+6FDF.1	fGB	1-3C43
U+6FDF.1	fR/S	 85.14
U+6FDF.1	fTotalStrokes	 17
U+6FDF.1	fCangjie	EYX
U+6FDF.1	fFourCorner	30123
U+6FDF.1	fMandarin	ji3
U+6FDF.1	fEnglish	[1] various; varied; numerous [2] elegant and dignified [3] the name of various counties and a river
U+6FE0.0	fUTF8	濠
U+6FE0.0	fCNS11643	1-7425
U+6FE0.0	fBig5	C0DA
U+6FE0.0	fGB	0-6529
U+6FE0.0	fR/S	 85.14
U+6FE0.0	fTotalStrokes	 17
U+6FE0.0	fCangjie	EYRO
U+6FE0.0	fFourCorner	30132
U+6FE0.0	fHakka	 hau2; ho2
U+6FE0.0	fCantonese	hou4
U+6FE0.0	fMandarin	hao2
U+6FE0.0	fEnglish	 moat; trench; ditch
U+6FE1.0	fUTF8	濡
U+6FE1.0	fCNS11643	1-742C
U+6FE1.0	fBig5	C0E1
U+6FE1.0	fGB	0-6526
U+6FE1.0	fR/S	 85.14
U+6FE1.0	fTotalStrokes	 17
U+6FE1.0	fCangjie	EMBB
U+6FE1.0	fFourCorner	31127
U+6FE1.0	fHakka	  si1; ji2; ri2
U+6FE1.0	fCantonese	jyu4; nyun5
U+6FE1.0	fMandarin	ru2; nuan2; er2; ruan3; ru4; nuo4; rou2; nuan4
U+6FE1.0	fEnglish	[1] immerse; moisten; wet [2] smooth; glossy [3] linger; procrastinate [4] tolerate; endure
U+6FE2.0	fUTF8	濢
U+6FE2.0	fCNS11643	2-5D46
U+6FE2.0	fBig5	ED5A
U+6FE2.0	fGB	3-5657
U+6FE2.0	fR/S	 85.14
U+6FE2.0	fTotalStrokes	 17
U+6FE2.0	fCangjie	ESMJ
U+6FE2.0	fFourCorner	37148
U+6FE2.0	fCantonese	ceoi3; zeoi2
U+6FE2.0	fMandarin	zui3; cui4
U+6FE3.0	fUTF8	濣
U+6FE3.0	fCNS11643	2-5D3E
U+6FE3.0	fBig5	ED52
U+6FE3.0	fGB	5-5732
U+6FE3.0	fR/S	 85.14
U+6FE3.0	fTotalStrokes	 17
U+6FE3.0	fCangjie	EJJJ
U+6FE3.0	fFourCorner	38140
U+6FE3.0	fCantonese	waat3
U+6FE3.0	fMandarin	wo4
U+6FE4.0	fUTF8	濤
U+6FE4.0	fCNS11643	1-7427
U+6FE4.0	fBig5	C0DC
U+6FE4.0	fGB	1-4C4E
U+6FE4.0	fR/S	 85.14
U+6FE4.0	fTotalStrokes	 17
U+6FE4.0	fCangjie	EGNI
U+6FE4.0	fFourCorner	34141
U+6FE4.0	fHakka	  tau2; to2
U+6FE4.0	fCantonese	tou4
U+6FE4.0	fMandarin	tao1; tao2; shou4; dao4; chao2; chuo2
U+6FE4.0	fEnglish	 big wave; large wave; billow; heavy swell
U+6FE5.0	fUTF8	濥
U+6FE5.0	fCNS11643	3-5531
U+6FE5.0	fGB	3-5654
U+6FE5.0	fR/S	  85.14
U+6FE5.0	fTotalStrokes	 17
U+6FE5.0	fCangjie	ENIC
U+6FE5.0	fFourCorner	37180
U+6FE5.0	fHakka	 jin3; rin3
U+6FE5.0	fCantonese	jan5
U+6FE5.0	fMandarin	yin4; yin3
U+6FE5.0	fEnglish	water-courses running underground
U+6FE6.0	fUTF8	濦
U+6FE6.0	fCNS11643	2-5D42
U+6FE6.0	fBig5	ED56
U+6FE6.0	fGB	5-5738
U+6FE6.0	fR/S	 85.14
U+6FE6.0	fTotalStrokes	 17
U+6FE6.0	fCangjie	EBMP
U+6FE6.0	fFourCorner	32137
U+6FE6.0	fCantonese	jan2 (also); jan1
U+6FE6.0	fMandarin	yin3; yin1; xi1; yin4
U+6FE7.0	fUTF8	濧
U+6FE7.0	fCNS11643	2-5D41
U+6FE7.0	fBig5	ED55
U+6FE7.0	fGB	3-5370
U+6FE7.0	fR/S	 85.14
U+6FE7.0	fTotalStrokes	 17
U+6FE7.0	fCangjie	ETGI
U+6FE7.0	fFourCorner	34100
U+6FE7.0	fCantonese	deoi6
U+6FE7.0	fMandarin	dui4
U+6FE8.0	fUTF8	濨
U+6FE8.0	fCNS11643	2-5D47
U+6FE8.0	fBig5	ED5B
U+6FE8.0	fGB	5-572D
U+6FE8.0	fR/S	 85.14
U+6FE8.0	fTotalStrokes	 17
U+6FE8.0	fCangjie	ETVP
U+6FE8.0	fFourCorner	38133
U+6FE8.0	fCantonese	ci4
U+6FE8.0	fMandarin	ci2
U+6FE9.0	fUTF8	濩
U+6FE9.0	fCNS11643	1-742D
U+6FE9.0	fBig5	C0E2
U+6FE9.0	fGB	3-5641
U+6FE9.0	fR/S	 85.14
U+6FE9.0	fTotalStrokes	 17
U+6FE9.0	fCangjie	ETOE
U+6FE9.0	fFourCorner	34147
U+6FE9.0	fHakka	 wok8; fu5; fu6; wok7; vok7
U+6FE9.0	fCantonese	wok6; wu6
U+6FE9.0	fMandarin	huo4; hu4; huo1
U+6FE9.0	fEnglish	[1] turbulent water [2] cook; boil
U+6FEA.0	fUTF8	濪
U+6FEA.0	fCNS11643	3-5537
U+6FEA.0	fGB	E-3136
U+6FEA.0	fR/S	  85.14
U+6FEA.0	fTotalStrokes	 17
U+6FEA.0	fCangjie	EQBU
U+6FEA.0	fFourCorner	37110
U+6FEA.0	fCantonese	cing3
U+6FEA.0	fMandarin	jing4
U+6FEB.0	fUTF8	濫
U+6FEB.0	fCNS11643	1-7428
U+6FEB.0	fBig5	C0DD
U+6FEB.0	fGB	1-4044
U+6FEB.0	fR/S	 85.14
U+6FEB.0	fTotalStrokes	 17
U+6FEB.0	fCangjie	ESIT
U+6FEB.0	fFourCorner	38117
U+6FEB.0	fHakka	  lam3; lam5; lam1; nam5; lam6; lam2
U+6FEB.0	fCantonese	laam6; laam5 (also); haam5
U+6FEB.0	fMandarin	lan4; jian4; lan3; lan2; dan4; xian4
U+6FEB.0	fEnglish	[1] flood; overflow; inundate; inundation [2] do things without plans; wreckless [3] practice no self-restrains; give way to unbridled license [4] abuse (one's power, influence, etc.) [5] false; not true [6] superfluous words or expressions
U+6FEC.0	fUTF8	濬
U+6FEC.0	fCNS11643	1-742B
U+6FEC.0	fBig5	C0E0
U+6FEC.0	fGB	8-2D59
U+6FEC.0	fR/S	 85.14
U+6FEC.0	fTotalStrokes	 17
U+6FEC.0	fCangjie	EYBU
U+6FEC.0	fFourCorner	31168
U+6FEC.0	fHakka	  zun5
U+6FEC.0	fCantonese	zeon3; seon3; zeon6
U+6FEC.0	fMandarin	jun4; xun4
U+6FEC.0	fEnglish	[1] dredge a waterway; dig or wash (well, etc.) [2] profound; deep
U+6FED.0	fUTF8	濭
U+6FED.0	fCNS11643	2-5D40
U+6FED.0	fBig5	ED54
U+6FED.0	fGB	3-5640
U+6FED.0	fR/S	 85.14
U+6FED.0	fTotalStrokes	 17
U+6FED.0	fCangjie	ETGT
U+6FED.0	fFourCorner	34117
U+6FED.0	fHakka	 ai5
U+6FED.0	fCantonese	oi3; ngoi3 (also); hap1; koi3
U+6FED.0	fMandarin	ai4; ke1
U+6FEE.0	fUTF8	濮
U+6FEE.0	fCNS11643	1-742F
U+6FEE.0	fBig5	C0E4
U+6FEE.0	fGB	0-6527
U+6FEE.0	fR/S	 85.14
U+6FEE.0	fTotalStrokes	 17
U+6FEE.0	fCangjie	EOTO
U+6FEE.0	fFourCorner	32134
U+6FEE.0	fHakka	 puk8
U+6FEE.0	fCantonese	buk6; buk1
U+6FEE.0	fMandarin	pu1; pu2
U+6FEE.0	fEnglish	[1] an ancient river in today's Honan province [2] an ancient barbarian tribe [3] a Chinese family name
U+6FEF.0	fUTF8	濯
U+6FEF.0	fCNS11643	1-7429
U+6FEF.0	fBig5	C0DE
U+6FEF.0	fGB	0-652A
U+6FEF.0	fR/S	 85.14
U+6FEF.0	fTotalStrokes	 17
U+6FEF.0	fCangjie	ESMG
U+6FEF.0	fFourCorner	37114
U+6FEF.0	fHakka	 cok8; cok7
U+6FEF.0	fCantonese	zok6
U+6FEF.0	fMandarin	zhuo2; shuo4; zhao4
U+6FEF.0	fEnglish	[1] wash; rinse; cleanse [2] eliminate vices [3] grand; magnificent [4] a Chinese family name
U+6FF0.0	fUTF8	濰
U+6FF0.0	fCNS11643	1-7430
U+6FF0.0	fBig5	C0E5
U+6FF0.0	fGB	1-4E2B
U+6FF0.0	fR/S	 85.14
U+6FF0.0	fTotalStrokes	 17
U+6FF0.0	fCangjie	EVFG
U+6FF0.0	fFourCorner	30114
U+6FF0.0	fHakka	 wui2; wi2; vui2
U+6FF0.0	fCantonese	wai4
U+6FF0.0	fMandarin	wei2
U+6FF0.0	fEnglish	the name of a river in Shantung province
U+6FF1.0	fUTF8	濱
U+6FF1.0	fCNS11643	1-7423
U+6FF1.0	fBig5	C0D8
U+6FF1.0	fGB	1-3175
U+6FF1.0	fR/S	 85.14
U+6FF1.0	fTotalStrokes	 17
U+6FF1.0	fCangjie	EJMC
U+6FF1.0	fFourCorner	33186
U+6FF1.0	fHakka	  bin1
U+6FF1.0	fCantonese	ban1
U+6FF1.0	fMandarin	bin1; pin2
U+6FF1.0	fEnglish	[1] beach; sea coast; water's edge; border on [2] same as $ [3] low, level seacoast (military)
U+6FF2.0	fUTF8	濲
U+6FF2.0	fCNS11643	2-5D44
U+6FF2.0	fBig5	ED58
U+6FF2.0	fGB	E-3137
U+6FF2.0	fR/S	 85.14
U+6FF2.0	fTotalStrokes	 17
U+6FF2.0	fCangjie	EGDE
U+6FF2.0	fFourCorner	37147
U+6FF2.0	fCantonese	guk1
U+6FF2.0	fMandarin	gu3
U+6FF3.0	fUTF8	濳
U+6FF3.0	fCNS11643	3-5A55
U+6FF3.0	fGB	5-573D
U+6FF3.0	fR/S	 85.16
U+6FF3.0	fTotalStrokes	 19
U+6FF3.0	fCangjie	EHUA
U+6FF3.0	fFourCorner	34160
U+6FF3.0	fHakka	 siam2
U+6FF3.0	fCantonese	cim4
U+6FF3.0	fMandarin	qian2
U+6FF3.0	fEnglish	 hide; conceal; secrete hidden
U+6FF4.0	fUTF8	濴
U+6FF4.0	fCNS11643	2-5D3C
U+6FF4.0	fBig5	ED50
U+6FF4.0	fGB	E-3138
U+6FF4.0	fR/S	 85.14
U+6FF4.0	fTotalStrokes	 17
U+6FF4.0	fCangjie	EFFE
U+6FF4.0	fFourCorner	39132
U+6FF4.0	fHakka	 jin2; rin2; rung2; jung2
U+6FF4.0	fCantonese	jing4
U+6FF4.0	fMandarin	xing2; ying2
U+6FF5.0	fUTF8	濵
U+6FF5.0	fCNS11643	3-5159
U+6FF5.0	fGB	E-3139
U+6FF5.0	fR/S	 85.14
U+6FF5.0	fTotalStrokes	 17; 16
U+6FF5.0	fCangjie	EJAC
U+6FF5.0	fFourCorner	33180
U+6FF5.0	fHakka	  bin1
U+6FF5.0	fCantonese	ban1
U+6FF5.0	fMandarin	bin1
U+6FF5.0	fEnglish	 bank; brink; shore; beach
U+6FF6.0	fUTF8	濶
U+6FF6.0	fCNS11643	3-5539
U+6FF6.0	fGB	E-313A
U+6FF6.0	fR/S	 85.14
U+6FF6.0	fTotalStrokes	 17
U+6FF6.0	fCangjie	EANR
U+6FF6.0	fFourCorner	37120
U+6FF6.0	fHakka	  fat7;  k'wat7
U+6FF6.0	fCantonese	fut3
U+6FF6.0	fMandarin	kuo4
U+6FF6.0	fEnglish	 broad; ample; wide be apart
U+6FF7.0	fUTF8	濷
U+6FF7.0	fCNS11643	2-6225
U+6FF7.0	fBig5	EFF7
U+6FF7.0	fGB	E-313B
U+6FF7.0	fR/S	 85.14; 85.15
U+6FF7.0	fTotalStrokes	 18
U+6FF7.0	fCangjie	EETB
U+6FF7.0	fFourCorner	12127
U+6FF7.0	fCantonese	fai3
U+6FF7.0	fMandarin	fei4
U+6FF8.0	fUTF8	濸
U+6FF8.0	fCNS11643	3-6548
U+6FF8.0	fGB	E-313C
U+6FF8.0	fR/S	 85.14
U+6FF8.0	fTotalStrokes	 17
U+6FF8.0	fCangjie	ETOR
U+6FF8.0	fFourCorner	3416
U+6FF8.0	fCantonese	cong1
U+6FF9.0	fUTF8	濹
U+6FF9.0	fGB	E-313D
U+6FF9.0	fR/S	 85.15; 85.14
U+6FF9.0	fTotalStrokes	 18
U+6FF9.0	fCangjie	EWGG
U+6FF9.0	fCantonese	mak6
U+6FF9.0	fMandarin	bin1
U+6FFA.0	fUTF8	濺
U+6FFA.0	fCNS11643	1-765D
U+6FFA.0	fBig5	C271
U+6FFA.0	fGB	1-3D26
U+6FFA.0	fR/S	 85.15
U+6FFA.0	fTotalStrokes	 18
U+6FFA.0	fCangjie	EBCI
U+6FFA.0	fFourCorner	33153
U+6FFA.0	fHakka	 cien5; cen3; zien1; cien6; zen1; zien5
U+6FFA.0	fCantonese	zin3; zin1
U+6FFA.0	fMandarin	jian4; jian1; zan4
U+6FFA.0	fEnglish	 sprinkle; spray; spill; splash
U+6FFB.0	fUTF8	濻
U+6FFB.0	fCNS11643	2-6222
U+6FFB.0	fBig5	EFF4
U+6FFB.0	fGB	E-313E
U+6FFB.0	fR/S	 85.15
U+6FFB.0	fTotalStrokes	 18
U+6FFB.0	fCangjie	ENLC
U+6FFB.0	fFourCorner	35186
U+6FFB.0	fCantonese	wai5
U+6FFB.0	fMandarin	dui4; wei2; wei3
U+6FFC.0	fUTF8	濼
U+6FFC.0	fCNS11643	2-6224
U+6FFC.0	fBig5	EFF6
U+6FFC.0	fGB	1-6378
U+6FFC.0	fR/S	 85.15
U+6FFC.0	fTotalStrokes	 18
U+6FFC.0	fCangjie	EVID
U+6FFC.0	fFourCorner	32194
U+6FFC.0	fHakka	 lok8
U+6FFC.0	fCantonese	lok6; bok6; lik1; baak3; pok3
U+6FFC.0	fMandarin	luo4; li4; lu4; pu2; po1; po4; yue4; yao4; shuo4
U+6FFC.0	fEnglish	a river in Shantung province
U+6FFC.1	fUTF8	濼
U+6FFC.1	fCNS11643	2-6224
U+6FFC.1	fBig5	EFF6
U+6FFC.1	fGB	1-6378
U+6FFC.1	fR/S	 85.15
U+6FFC.1	fTotalStrokes	 18
U+6FFC.1	fCangjie	EVID
U+6FFC.1	fFourCorner	32194
U+6FFC.1	fMandarin	bo2
U+6FFC.1	fEnglish	a lake
U+6FFD.0	fUTF8	濽
U+6FFD.0	fCNS11643	3-5828
U+6FFD.0	fGB	E-313F
U+6FFD.0	fR/S	 85.15
U+6FFD.0	fTotalStrokes	 18
U+6FFD.0	fCangjie	EQOC
U+6FFD.0	fFourCorner	35180
U+6FFD.0	fHakka	 can5
U+6FFD.0	fCantonese	zaan3
U+6FFD.0	fMandarin	zan4; cuan2; qian2; za1
U+6FFE.0	fUTF8	濾
U+6FFE.0	fCNS11643	1-765B
U+6FFE.0	fBig5	C26F
U+6FFE.0	fGB	1-424B
U+6FFE.0	fR/S	 85.15
U+6FFE.0	fTotalStrokes	 18
U+6FFE.0	fCangjie	EYPP
U+6FFE.0	fFourCorner	31136
U+6FFE.0	fHakka	  li5; li6; li3
U+6FFE.0	fCantonese	leoi6
U+6FFE.0	fMandarin	luu4; l4
U+6FFE.0	fEnglish	 strain out; filter
U+6FFF.0	fUTF8	濿
U+6FFF.0	fCNS11643	2-617E
U+6FFF.0	fBig5	EFF2
U+6FFF.0	fGB	5-5523
U+6FFF.0	fR/S	 85.15
U+6FFF.0	fTotalStrokes	 18
U+6FFF.0	fCangjie	EMTB
U+6FFF.0	fFourCorner	31127
U+6FFF.0	fHakka	 li5; li6
U+6FFF.0	fCantonese	lai6
U+6FFF.0	fMandarin	li4
U+6FFF.0	fEnglish	 cross a stream on stepping stones
U+7000.0	fUTF8	瀀
U+7000.0	fCNS11643	2-6221
U+7000.0	fBig5	EFF3
U+7000.0	fGB	3-5659
U+7000.0	fR/S	 85.15
U+7000.0	fTotalStrokes	 18
U+7000.0	fCangjie	EMBE
U+7000.0	fFourCorner	31147
U+7000.0	fCantonese	jau1
U+7000.0	fMandarin	you1
U+7001.0	fUTF8	瀁
U+7001.0	fCNS11643	2-617A
U+7001.0	fBig5	EFEE
U+7001.0	fGB	3-5655
U+7001.0	fR/S	 85.15
U+7001.0	fTotalStrokes	 18
U+7001.0	fCangjie	ETOV
U+7001.0	fFourCorner	38132
U+7001.0	fHakka	 rong3; jong3; jong5
U+7001.0	fCantonese	joeng6; joeng5
U+7001.0	fMandarin	yang4; yang3
U+7001.0	fEnglish	[1] water in motion [2] move a boat or something in the water [3] waves; ripples; rapids [4] overflow; too much; too many [5] the name of a stream in Shensi province
U+7002.0	fUTF8	瀂
U+7002.0	fCNS11643	3-582A
U+7002.0	fGB	5-5674
U+7002.0	fR/S	  85.15
U+7002.0	fTotalStrokes	 18
U+7002.0	fCangjie	ENWA
U+7002.0	fFourCorner	37160
U+7002.0	fHakka	 lu1
U+7002.0	fCantonese	lou5
U+7002.0	fMandarin	lu3
U+7003.0	fUTF8	瀃
U+7003.0	fCNS11643	3-5827
U+7003.0	fGB	5-566F
U+7003.0	fR/S	 85.15
U+7003.0	fTotalStrokes	 18
U+7003.0	fCangjie	EBCH
U+7003.0	fFourCorner	36120
U+7003.0	fCantonese	ci3
U+7003.0	fMandarin	si4
U+7004.0	fUTF8	瀄
U+7004.0	fCNS11643	2-5823
U+7004.0	fBig5	E9F6
U+7004.0	fGB	3-535B
U+7004.0	fR/S	 85.15
U+7004.0	fTotalStrokes	 18
U+7004.0	fCangjie	EHAL
U+7004.0	fFourCorner	38127
U+7004.0	fCantonese	zit1; zit3
U+7004.0	fMandarin	jie2; ji2
U+7004.0	fEnglish	to squeeze; to eject; to spray
U+7005.0	fUTF8	瀅
U+7005.0	fCNS11643	2-617B
U+7005.0	fBig5	EFEF
U+7005.0	fGB	1-645E
U+7005.0	fR/S	 85.15
U+7005.0	fTotalStrokes	 18
U+7005.0	fCangjie	EFFI
U+7005.0	fFourCorner	39113
U+7005.0	fHakka	 jin2; rin2; rung2; jung2
U+7005.0	fCantonese	jing4; jing6 (also); jing3
U+7005.0	fMandarin	ying4; jiong1; ying3; ying2; jing3
U+7005.0	fEnglish	clear; bright and clear (water); glossy
U+7006.0	fUTF8	瀆
U+7006.0	fCNS11643	1-765C
U+7006.0	fBig5	C270
U+7006.0	fGB	1-6442
U+7006.0	fR/S	 85.15
U+7006.0	fTotalStrokes	 18
U+7006.0	fCangjie	EGWC
U+7006.0	fFourCorner	34186
U+7006.0	fHakka	 tuk8
U+7006.0	fCantonese	duk6; dau6
U+7006.0	fMandarin	du2; dou4
U+7006.0	fEnglish	[1] ditch [2] river [3] desecrate; profane; blaspheme; be rude and disrespectful [4] annoy
U+7007.0	fUTF8	瀇
U+7007.0	fCNS11643	2-6177
U+7007.0	fBig5	EFEB
U+7007.0	fGB	3-5337
U+7007.0	fR/S	 85.15
U+7007.0	fTotalStrokes	 18
U+7007.0	fCangjie	EITC
U+7007.0	fFourCorner	30186
U+7007.0	fCantonese	wong2
U+7007.0	fMandarin	wang3; huang3; wang4
U+7008.0	fUTF8	瀈
U+7008.0	fCNS11643	3-5826
U+7008.0	fGB	5-567D
U+7008.0	fR/S	 85.15
U+7008.0	fTotalStrokes	 18
U+7008.0	fCangjie	XESMJ
U+7008.0	fFourCorner	37150
U+7008.0	fCantonese	fai1
U+7008.0	fMandarin	hui1
U+7009.0	fUTF8	瀉
U+7009.0	fCNS11643	1-7659
U+7009.0	fBig5	C26D
U+7009.0	fGB	1-503A
U+7009.0	fR/S	 85.15
U+7009.0	fTotalStrokes	 18
U+7009.0	fCangjie	EJHF
U+7009.0	fFourCorner	33127
U+7009.0	fHakka	  sia5
U+7009.0	fCantonese	se3; se2 (also)
U+7009.0	fMandarin	xie4; xie3
U+7009.0	fEnglish	[1] drain; water flowing down [2] diarrhea; have loose bowels
U+700A.0	fUTF8	瀊
U+700A.0	fCNS11643	2-6226
U+700A.0	fBig5	EFF8
U+700A.0	fGB	E-3140
U+700A.0	fR/S	 85.15
U+700A.0	fTotalStrokes	 18
U+700A.0	fCangjie	EHET
U+700A.0	fFourCorner	37117
U+700A.0	fCantonese	pun4
U+700A.0	fMandarin	pan2
U+700B.0	fUTF8	瀋
U+700B.0	fCNS11643	1-765A
U+700B.0	fBig5	C26E
U+700B.0	fGB	1-6449
U+700B.0	fR/S	 85.15
U+700B.0	fTotalStrokes	 18
U+700B.0	fCangjie	EJHW
U+700B.0	fFourCorner	33169
U+700B.0	fHakka	 shim3; siim3; sim3
U+700B.0	fCantonese	sam2
U+700B.0	fMandarin	shen3; chen4; pan2
U+700B.0	fEnglish	[1] juice; liquid; water; fluid [2] short for Shenyang (Mukden), capital of Liaoning province
U+700C.0	fUTF8	瀌
U+700C.0	fCNS11643	2-6178
U+700C.0	fBig5	EFEC
U+700C.0	fGB	3-565F
U+700C.0	fR/S	 85.15
U+700C.0	fTotalStrokes	 18
U+700C.0	fCangjie	EIPF
U+700C.0	fFourCorner	30131
U+700C.0	fHakka	 piau1; peu1
U+700C.0	fCantonese	biu1; piu4
U+700C.0	fMandarin	biao1; biao4; piao2; piao4
U+700C.0	fEnglish	plenty
U+700D.0	fUTF8	瀍
U+700D.0	fCNS11643	2-6179
U+700D.0	fBig5	EFED
U+700D.0	fGB	3-565E
U+700D.0	fR/S	 85.15
U+700D.0	fTotalStrokes	 18
U+700D.0	fCangjie	EIWG
U+700D.0	fFourCorner	30114
U+700D.0	fHakka	 chan1; can1; chen2
U+700D.0	fCantonese	cin4
U+700D.0	fMandarin	chan2
U+700D.0	fEnglish	a river in Henan province
U+700E.0	fUTF8	瀎
U+700E.0	fCNS11643	2-617D
U+700E.0	fBig5	EFF1
U+700E.0	fGB	3-5650
U+700E.0	fR/S	 85.15
U+700E.0	fTotalStrokes	 18
U+700E.0	fCangjie	ETWI
U+700E.0	fFourCorner	34153
U+700E.0	fCantonese	mut3; mut6; mit6
U+700E.0	fMandarin	mo4; mie4
U+700F.0	fUTF8	瀏
U+700F.0	fCNS11643	1-765F
U+700F.0	fBig5	C273
U+700F.0	fGB	1-642F
U+700F.0	fR/S	 85.15
U+700F.0	fTotalStrokes	 18
U+700F.0	fCangjie	EHCN
U+700F.0	fFourCorner	32100
U+700F.0	fHakka	 liu2
U+700F.0	fCantonese	lau4
U+700F.0	fMandarin	liu2; liu3; liu1
U+700F.0	fEnglish	[1] clear; bright and clear (water); the appearance of a clear stream [2] fast-blowing wind; cool wind [3] get away secretly; take French leave
U+7010.0	fUTF8	瀐
U+7010.0	fCNS11643	3-5829
U+7010.0	fGB	5-573A
U+7010.0	fR/S	  85.15
U+7010.0	fTotalStrokes	 18
U+7010.0	fCangjie	EJIM
U+7010.0	fFourCorner	33150
U+7010.0	fCantonese	zim1
U+7010.0	fMandarin	jian1
U+7011.0	fUTF8	瀑
U+7011.0	fCNS11643	1-765E
U+7011.0	fBig5	C272
U+7011.0	fGB	0-4659
U+7011.0	fR/S	 85.15
U+7011.0	fTotalStrokes	 18
U+7011.0	fCangjie	EATE
U+7011.0	fFourCorner	36132
U+7011.0	fHakka	 pau5;  pok7; pau6
U+7011.0	fCantonese	buk6; bou6; puk6 (also)
U+7011.0	fMandarin	pu4; bao4; bo2; bao2
U+7011.0	fEnglish	 waterfall; cascade; cataract
U+7012.0	fUTF8	瀒
U+7012.0	fCNS11643	4-5B5E
U+7012.0	fGB	E-3141
U+7012.0	fR/S	 85.15
U+7012.0	fTotalStrokes	 18
U+7012.0	fCangjie	EDOW
U+7012.0	fFourCorner	34160
U+7012.0	fCantonese	gip3
U+7012.0	fMandarin	se4
U+7012.0	fEnglish	 coarse; rough astringent
U+7013.0	fUTF8	瀓
U+7013.0	fCNS11643	3-582B
U+7013.0	fGB	E-3142
U+7013.0	fR/S	  85.15
U+7013.0	fTotalStrokes	 18
U+7013.0	fCangjie	EHOK
U+7013.0	fFourCorner	38140
U+7013.0	fHakka	 chim2; cim2
U+7013.0	fCantonese	cing4
U+7013.0	fMandarin	cheng2
U+7014.0	fUTF8	瀔
U+7014.0	fCNS11643	2-617C
U+7014.0	fBig5	EFF0
U+7014.0	fGB	3-5658
U+7014.0	fR/S	 85.15
U+7014.0	fTotalStrokes	 18
U+7014.0	fCangjie	EGDE
U+7014.0	fFourCorner	37147
U+7014.0	fCantonese	guk1; juk6; luk6
U+7014.0	fMandarin	gu3
U+7014.0	fEnglish	a river in Henan province
U+7015.0	fUTF8	瀕
U+7015.0	fCNS11643	1-7845
U+7015.0	fBig5	C378
U+7015.0	fGB	1-3174
U+7015.0	fR/S	 85.16
U+7015.0	fTotalStrokes	 19
U+7015.0	fCangjie	EYHC
U+7015.0	fFourCorner	31186
U+7015.0	fHakka	  pin2; bin1
U+7015.0	fCantonese	ban1; pan4 (also)
U+7015.0	fMandarin	bin1; pin2
U+7015.0	fEnglish	[1] close; near; border [2] water's edge
U+7016.0	fUTF8	瀖
U+7016.0	fCNS11643	2-656A
U+7016.0	fBig5	F25F
U+7016.0	fGB	3-5662
U+7016.0	fR/S	 85.16
U+7016.0	fTotalStrokes	 19
U+7016.0	fCangjie	EMBG
U+7016.0	fFourCorner	31114
U+7016.0	fCantonese	fok3
U+7016.0	fMandarin	huo4
U+7017.0	fUTF8	瀗
U+7017.0	fCNS11643	2-6570
U+7017.0	fBig5	F265
U+7017.0	fGB	3-5666
U+7017.0	fR/S	 85.16
U+7017.0	fTotalStrokes	 19
U+7017.0	fCangjie	EJQP
U+7017.0	fFourCorner	33136
U+7017.0	fCantonese	hin3
U+7017.0	fMandarin	xian4
U+7018.0	fUTF8	瀘
U+7018.0	fCNS11643	1-7846
U+7018.0	fBig5	C379
U+7018.0	fGB	1-6372
U+7018.0	fR/S	 85.16
U+7018.0	fTotalStrokes	 19
U+7018.0	fCangjie	EYPT
U+7018.0	fFourCorner	31117
U+7018.0	fHakka	  lu2
U+7018.0	fCantonese	lou4
U+7018.0	fMandarin	lu2
U+7018.0	fEnglish	[1] a river in Szechwan province [2] a river in Yunnan province
U+7019.0	fUTF8	瀙
U+7019.0	fCNS11643	2-6567
U+7019.0	fBig5	F25C
U+7019.0	fGB	3-5537
U+7019.0	fR/S	 85.16
U+7019.0	fTotalStrokes	 19
U+7019.0	fCangjie	EYDU
U+7019.0	fFourCorner	36110
U+7019.0	fCantonese	can3
U+7019.0	fMandarin	qin1; qin4; chen4; zhen1
U+701A.0	fUTF8	瀚
U+701A.0	fCNS11643	1-7843
U+701A.0	fBig5	C376
U+701A.0	fGB	0-652B
U+701A.0	fR/S	 85.16
U+701A.0	fTotalStrokes	 19
U+701A.0	fCangjie	EJJM
U+701A.0	fFourCorner	38127
U+701A.0	fHakka	 hon5; hon6
U+701A.0	fCantonese	hon6
U+701A.0	fMandarin	han4
U+701A.0	fEnglish	vast; wide; extensive; expansive
U+701B.0	fUTF8	瀛
U+701B.0	fCNS11643	1-7840
U+701B.0	fBig5	C373
U+701B.0	fGB	0-652D
U+701B.0	fR/S	 85.16
U+701B.0	fTotalStrokes	 19
U+701B.0	fCangjie	EYNV
U+701B.0	fFourCorner	30117
U+701B.0	fHakka	  jin2; rin2
U+701B.0	fCantonese	jing4
U+701B.0	fMandarin	ying2
U+701B.0	fEnglish	[1] sea; ocean [2] within the lake
U+701C.0	fUTF8	瀜
U+701C.0	fCNS11643	2-6572
U+701C.0	fBig5	F267
U+701C.0	fGB	3-5661
U+701C.0	fR/S	 85.16
U+701C.0	fTotalStrokes	 19
U+701C.0	fCangjie	EMBI
U+701C.0	fFourCorner	35136
U+701C.0	fHakka	 jung2; rung2
U+701C.0	fCantonese	jung4
U+701C.0	fMandarin	rong2; yong1
U+701C.0	fEnglish	wide; expansive, as a sheet of water
U+701D.0	fUTF8	瀝
U+701D.0	fCNS11643	1-7844
U+701D.0	fBig5	C377
U+701D.0	fGB	1-4124
U+701D.0	fR/S	 85.16
U+701D.0	fTotalStrokes	 19
U+701D.0	fCangjie	EMHM
U+701D.0	fFourCorner	31111
U+701D.0	fHakka	  lak8;  lit8; lak7; let8
U+701D.0	fCantonese	lik6; lik1 (also); lek6
U+701D.0	fMandarin	li4
U+701D.0	fEnglish	[1] trickle; drip; fall down drop by drop [2] strain water or liquid [3] remaining drops of wine; dregs
U+701E.0	fUTF8	瀞
U+701E.0	fCNS11643	3-5A52
U+701E.0	fGB	5-572F
U+701E.0	fR/S	 85.16; 84.14
U+701E.0	fTotalStrokes	 19
U+701E.0	fCangjie	EQBD
U+701E.0	fFourCorner	32150
U+701E.0	fCantonese	zing6
U+701E.0	fMandarin	jing4
U+701E.0	fEnglish	pool in a river
U+701F.0	fUTF8	瀟
U+701F.0	fCNS11643	1-7841
U+701F.0	fBig5	C374
U+701F.0	fGB	1-646C
U+701F.0	fR/S	 85.16
U+701F.0	fTotalStrokes	 19
U+701F.0	fCangjie	ETLX
U+701F.0	fFourCorner	34127
U+701F.0	fHakka	 siau1
U+701F.0	fCantonese	siu1
U+701F.0	fMandarin	xiao1
U+701F.0	fEnglish	[1] the sound of beating rain and whistling wind; the roar of strong wind [2] a stream in Hunan
U+7020.0	fUTF8	瀠
U+7020.0	fCNS11643	2-6569
U+7020.0	fBig5	F25E
U+7020.0	fGB	1-646B
U+7020.0	fR/S	 85.16
U+7020.0	fTotalStrokes	 19
U+7020.0	fCangjie	EFFF
U+7020.0	fFourCorner	39193
U+7020.0	fHakka	 jin2; rin2; rung2; jung2
U+7020.0	fCantonese	jing4
U+7020.0	fMandarin	ying2; ying3
U+7020.0	fEnglish	the whirling of water
U+7021.0	fUTF8	瀡
U+7021.0	fCNS11643	2-656C
U+7021.0	fBig5	F261
U+7021.0	fGB	3-5656
U+7021.0	fR/S	 85.16
U+7021.0	fTotalStrokes	 19
U+7021.0	fCangjie	ENLB
U+7021.0	fFourCorner	34132
U+7021.0	fHakka	  sui3
U+7021.0	fCantonese	seoi5
U+7021.0	fMandarin	sui3; sui1
U+7021.0	fEnglish	slippery; smooth
U+7022.0	fUTF8	瀢
U+7022.0	fCNS11643	2-656D
U+7022.0	fBig5	F262
U+7022.0	fGB	3-5630
U+7022.0	fR/S	 85.16
U+7022.0	fTotalStrokes	 19
U+7022.0	fCangjie	EYLC
U+7022.0	fFourCorner	35138
U+7022.0	fCantonese	wai5
U+7022.0	fMandarin	wei2; dui4
U+7023.0	fUTF8	瀣
U+7023.0	fCNS11643	2-656E
U+7023.0	fBig5	F263
U+7023.0	fGB	0-652C
U+7023.0	fR/S	 85.16
U+7023.0	fTotalStrokes	 19
U+7023.0	fCangjie	EYEM
U+7023.0	fFourCorner	37111
U+7023.0	fHakka	 hai5; hai6
U+7023.0	fCantonese	haai6
U+7023.0	fMandarin	xie4; hai4; hui4
U+7023.0	fEnglish	 mist; vapor
U+7024.0	fUTF8	瀤
U+7024.0	fCNS11643	2-6571
U+7024.0	fBig5	F266
U+7024.0	fGB	3-5665
U+7024.0	fR/S	 85.16
U+7024.0	fTotalStrokes	 19
U+7024.0	fCangjie	EYWV
U+7024.0	fFourCorner	30132
U+7024.0	fCantonese	waai4; waai1
U+7024.0	fMandarin	huai2; wei1
U+7025.0	fUTF8	瀥
U+7025.0	fCNS11643	3-5A54
U+7025.0	fGB	5-573F
U+7025.0	fR/S	  85.16
U+7025.0	fTotalStrokes	 19
U+7025.0	fCangjie	XESMB
U+7025.0	fFourCorner	37120
U+7025.0	fCantonese	hou6
U+7025.0	fMandarin	hao4
U+7026.0	fUTF8	瀦
U+7026.0	fCNS11643	2-6223
U+7026.0	fBig5	EFF5
U+7026.0	fGB	E-3143
U+7026.0	fR/S	 85.16
U+7026.0	fTotalStrokes	 19
U+7026.0	fCangjie	EMOA
U+7026.0	fFourCorner	34160
U+7026.0	fCantonese	zyu1
U+7026.0	fMandarin	zhu1; du1
U+7026.0	fEnglish	 pond; pool
U+7027.0	fUTF8	瀧
U+7027.0	fCNS11643	2-6568
U+7027.0	fBig5	F25D
U+7027.0	fGB	1-6371
U+7027.0	fR/S	 85.16
U+7027.0	fTotalStrokes	 19
U+7027.0	fCangjie	EYBP
U+7027.0	fFourCorner	31111
U+7027.0	fHakka	 lung2
U+7027.0	fCantonese	lung4; soeng1; long4
U+7027.0	fMandarin	long2; long4
U+7027.0	fEnglish	[1] rainy; raining [2] wet; soaked; saturated [3] swift current [4] waterfall
U+7027.1	fUTF8	瀧
U+7027.1	fCNS11643	2-6568
U+7027.1	fBig5	F25D
U+7027.1	fGB	1-6371
U+7027.1	fR/S	 85.16
U+7027.1	fTotalStrokes	 19
U+7027.1	fCangjie	EYBP
U+7027.1	fFourCorner	31111
U+7027.1	fMandarin	shuang1
U+7027.1	fEnglish	a river in Hunan
U+7028.0	fUTF8	瀨
U+7028.0	fCNS11643	1-7842
U+7028.0	fBig5	C375
U+7028.0	fGB	1-647E
U+7028.0	fR/S	 85.16
U+7028.0	fTotalStrokes	 19
U+7028.0	fCangjie	EDLC
U+7028.0	fFourCorner	31186
U+7028.0	fHakka	 lai5; lai2; lai3; lai6
U+7028.0	fCantonese	laai6
U+7028.0	fMandarin	lai4
U+7028.0	fEnglish	[1] swift current; torrent [2] water flowing over shallows [3] Li, a stream in Kwangsi [4] a stream in Kiangsu
U+7029.0	fUTF8	瀩
U+7029.0	fCNS11643	2-656F
U+7029.0	fBig5	F264
U+7029.0	fGB	E-3144
U+7029.0	fR/S	 85.16
U+7029.0	fTotalStrokes	 19
U+7029.0	fCangjie	EHUC
U+7029.0	fFourCorner	31186
U+7029.0	fCantonese	deoi6
U+7029.0	fMandarin	dui4
U+702A.0	fUTF8	瀪
U+702A.0	fCNS11643	2-6934
U+702A.0	fBig5	F268
U+702A.0	fGB	E-3145
U+702A.0	fR/S	 85.16
U+702A.0	fTotalStrokes	 20; 19
U+702A.0	fCangjie	OKHAE
U+702A.0	fFourCorner	88904
U+702A.0	fCantonese	faan6; faan4
U+702A.0	fMandarin	fan2
U+702B.0	fUTF8	瀫
U+702B.0	fCNS11643	2-656B
U+702B.0	fBig5	F260
U+702B.0	fGB	3-5660
U+702B.0	fR/S	 85.16
U+702B.0	fTotalStrokes	 19
U+702B.0	fCangjie	EGFE
U+702B.0	fFourCorner	37147
U+702B.0	fCantonese	huk6; huk1
U+702B.0	fMandarin	hu2
U+702C.0	fUTF8	瀬
U+702C.0	fCNS11643	3-5A53
U+702C.0	fGB	E-3146
U+702C.0	fR/S	 85.16
U+702C.0	fTotalStrokes	 19
U+702C.0	fCangjie	EDLC
U+702C.0	fFourCorner	31180
U+702C.0	fCantonese	laai6
U+702C.0	fMandarin	lai4
U+702C.0	fEnglish	 swift current rapids
U+702D.0	fUTF8	瀭
U+702D.0	fCNS11643	3-654A
U+702D.0	fGB	E-3147
U+702D.0	fR/S	 85.16
U+702D.0	fTotalStrokes	 19
U+702D.0	fCangjie	XEJJN
U+702D.0	fFourCorner	3812
U+702D.0	fCantonese	syu1
U+702E.0	fUTF8	瀮
U+702E.0	fCNS11643	3-6549
U+702E.0	fGB	E-3148
U+702E.0	fR/S	 85.16
U+702E.0	fTotalStrokes	 19
U+702E.0	fCangjie	EMBD
U+702E.0	fFourCorner	3119
U+702E.0	fCantonese	lam4
U+702F.0	fUTF8	瀯
U+702F.0	fCNS11643	2-692B
U+702F.0	fBig5	F45D
U+702F.0	fGB	3-5567
U+702F.0	fR/S	 85.17
U+702F.0	fTotalStrokes	 20
U+702F.0	fCangjie	EFFR
U+702F.0	fFourCorner	39166
U+702F.0	fHakka	 jin2; rin2
U+702F.0	fCantonese	jing4
U+702F.0	fMandarin	ying2
U+702F.0	fEnglish	 brook; rivulet
U+7030.0	fUTF8	瀰
U+7030.0	fCNS11643	1-7975
U+7030.0	fBig5	C46A
U+7030.0	fGB	1-7858
U+7030.0	fR/S	 85.17
U+7030.0	fTotalStrokes	 20
U+7030.0	fCangjie	ENMB
U+7030.0	fFourCorner	31127
U+7030.0	fHakka	 mi2; ni2; li2
U+7030.0	fCantonese	nei4; mei4 (also); lei4 (also); mei5
U+7030.0	fMandarin	mi2; mi3; ni3
U+7030.0	fEnglish	overflowing; brimming (water)
U+7031.0	fUTF8	瀱
U+7031.0	fCNS11643	2-692E
U+7031.0	fBig5	F460
U+7031.0	fGB	3-5667
U+7031.0	fR/S	 85.17
U+7031.0	fTotalStrokes	 20
U+7031.0	fCangjie	EWLN
U+7031.0	fFourCorner	36121
U+7031.0	fCantonese	gai3
U+7031.0	fMandarin	ji4; chan2
U+7032.0	fUTF8	瀲
U+7032.0	fCNS11643	1-7976
U+7032.0	fBig5	C46B
U+7032.0	fGB	1-6472
U+7032.0	fR/S	 85.17
U+7032.0	fTotalStrokes	 20
U+7032.0	fCangjie	EOOK
U+7032.0	fFourCorner	38140
U+7032.0	fHakka	 liam5; liam3
U+7032.0	fCantonese	lim6; lim5
U+7032.0	fMandarin	lian4
U+7032.0	fEnglish	[1] overflowing (water) [2] the edge of a large body of water
U+7033.0	fUTF8	瀳
U+7033.0	fCNS11643	2-6937
U+7033.0	fBig5	F468
U+7033.0	fGB	5-5626
U+7033.0	fR/S	 85.17
U+7033.0	fTotalStrokes	 20
U+7033.0	fCangjie	ETIF
U+7033.0	fFourCorner	35127
U+7033.0	fCantonese	zin6; zin1
U+7033.0	fMandarin	jian4; zun1; jian1; zun4
U+7034.0	fUTF8	瀴
U+7034.0	fCNS11643	2-692D
U+7034.0	fBig5	F45F
U+7034.0	fGB	3-5573
U+7034.0	fR/S	 85.17
U+7034.0	fTotalStrokes	 20
U+7034.0	fCangjie	EBCV
U+7034.0	fFourCorner	36144
U+7034.0	fCantonese	jing1; jing2 (also)
U+7034.0	fMandarin	ying3; ying1; ying4; ming3
U+7035.0	fUTF8	瀵
U+7035.0	fCNS11643	2-692A
U+7035.0	fBig5	F45C
U+7035.0	fGB	0-652F
U+7035.0	fR/S	 85.17
U+7035.0	fTotalStrokes	 20
U+7035.0	fCangjie	EFDC
U+7035.0	fFourCorner	39181
U+7035.0	fHakka	  bun5
U+7035.0	fCantonese	fan3
U+7035.0	fMandarin	fen4; pen4
U+7035.0	fEnglish	a river in Shantung
U+7036.0	fUTF8	瀶
U+7036.0	fCNS11643	3-5C51
U+7036.0	fGB	E-3149
U+7036.0	fR/S	  85.17
U+7036.0	fTotalStrokes	 20
U+7036.0	fCangjie	ESLR
U+7036.0	fFourCorner	38160
U+7036.0	fCantonese	lam4
U+7036.0	fMandarin	lin2
U+7037.0	fUTF8	瀷
U+7037.0	fCNS11643	2-692C
U+7037.0	fBig5	F45E
U+7037.0	fGB	3-566D
U+7037.0	fR/S	 85.17
U+7037.0	fTotalStrokes	 20
U+7037.0	fCangjie	ESMC
U+7037.0	fFourCorner	37181
U+7037.0	fHakka	 jit8; rit8
U+7037.0	fCantonese	jik6
U+7037.0	fMandarin	yi4; chi4
U+7037.0	fEnglish	a small river in Honan
U+7038.0	fUTF8	瀸
U+7038.0	fCNS11643	2-6930
U+7038.0	fBig5	F462
U+7038.0	fGB	3-5668
U+7038.0	fR/S	 85.17
U+7038.0	fTotalStrokes	 20
U+7038.0	fCangjie	EOIM
U+7038.0	fFourCorner	33150
U+7038.0	fHakka	  siam2;  ziam1
U+7038.0	fCantonese	zim1; cim1
U+7038.0	fMandarin	jian1
U+7038.0	fEnglish	[1] moisten [2] destroy
U+7039.0	fUTF8	瀹
U+7039.0	fCNS11643	2-6933
U+7039.0	fBig5	F465
U+7039.0	fGB	0-652E
U+7039.0	fR/S	 85.17
U+7039.0	fTotalStrokes	 20
U+7039.0	fCangjie	EOMB
U+7039.0	fFourCorner	38127
U+7039.0	fHakka	 jok7; rok7; jok8
U+7039.0	fCantonese	joek6
U+7039.0	fMandarin	yue4; yao4
U+7039.0	fEnglish	[1] boil with soup; cook with soup [2] soak [3] clear the courses of rivers
U+703A.0	fUTF8	瀺
U+703A.0	fCNS11643	2-6932
U+703A.0	fBig5	F464
U+703A.0	fGB	3-566A
U+703A.0	fR/S	 85.17
U+703A.0	fTotalStrokes	 20
U+703A.0	fCangjie	ENRI
U+703A.0	fFourCorner	37113
U+703A.0	fHakka	 cam2
U+703A.0	fCantonese	caam4; zaam6
U+703A.0	fMandarin	chan2; zhan3
U+703A.0	fEnglish	sound
U+703B.0	fUTF8	瀻
U+703B.0	fCNS11643	2-6936
U+703B.0	fBig5	F467
U+703B.0	fGB	5-5740
U+703B.0	fR/S	 85.17
U+703B.0	fTotalStrokes	 20
U+703B.0	fCangjie	EJIC
U+703B.0	fFourCorner	33150
U+703B.0	fCantonese	daai3
U+703B.0	fMandarin	dai4
U+703C.0	fUTF8	瀼
U+703C.0	fCNS11643	2-6929
U+703C.0	fBig5	F45B
U+703C.0	fGB	3-566B
U+703C.0	fR/S	 85.17
U+703C.0	fTotalStrokes	 20
U+703C.0	fCangjie	EYRV
U+703C.0	fFourCorner	30132
U+703C.0	fHakka	  siong1; rong2; jong2
U+703C.0	fCantonese	joeng4; joeng5; joeng6
U+703C.0	fMandarin	rang2; rang4; nang2; rang3; nang3; xiang4
U+703C.0	fEnglish	 a river in Hunan province flowing
U+703D.0	fUTF8	瀽
U+703D.0	fCNS11643	3-5C4F
U+703D.0	fGB	3-566C
U+703D.0	fR/S	  85.17
U+703D.0	fTotalStrokes	 20
U+703D.0	fCangjie	EJTO
U+703D.0	fFourCorner	33180
U+703D.0	fCantonese	gin2 (also); zin2
U+703D.0	fMandarin	jian3
U+703E.0	fUTF8	瀾
U+703E.0	fCNS11643	1-7974
U+703E.0	fBig5	C469
U+703E.0	fGB	1-403D
U+703E.0	fR/S	 85.17
U+703E.0	fTotalStrokes	 20
U+703E.0	fCangjie	EANW
U+703E.0	fFourCorner	37120
U+703E.0	fHakka	 lan2; lan3
U+703E.0	fCantonese	laan4; laan6
U+703E.0	fMandarin	lan2
U+703E.0	fEnglish	great wave; huge billow
U+703E.1	fUTF8	瀾
U+703E.1	fCNS11643	1-7974
U+703E.1	fBig5	C469
U+703E.1	fGB	1-403D
U+703E.1	fR/S	 85.17
U+703E.1	fTotalStrokes	 20
U+703E.1	fCangjie	EANW
U+703E.1	fFourCorner	37120
U+703E.1	fMandarin	lan4
U+703E.1	fEnglish	[1] overflowing; dripping wet; vast expanse of water [2] thin rice paste
U+703F.0	fUTF8	瀿
U+703F.0	fCNS11643	2-6931
U+703F.0	fBig5	F463
U+703F.0	fGB	5-5744
U+703F.0	fR/S	 85.17
U+703F.0	fTotalStrokes	 20
U+703F.0	fCangjie	EOKF
U+703F.0	fFourCorner	38193
U+703F.0	fHakka	 fan2
U+703F.0	fCantonese	faan4
U+703F.0	fMandarin	fan2
U+703F.0	fEnglish	 water plants
U+7040.0	fUTF8	灀
U+7040.0	fCNS11643	2-6935
U+7040.0	fBig5	F466
U+7040.0	fGB	5-5742
U+7040.0	fR/S	 85.17
U+7040.0	fTotalStrokes	 20
U+7040.0	fCangjie	EMBU
U+7040.0	fFourCorner	31162
U+7040.0	fCantonese	soeng3; soeng1
U+7040.0	fMandarin	shuang4
U+7041.0	fUTF8	灁
U+7041.0	fCNS11643	2-6938
U+7041.0	fBig5	F469
U+7041.0	fGB	E-314A
U+7041.0	fR/S	 85.17
U+7041.0	fTotalStrokes	 20
U+7041.0	fCangjie	EANE
U+7041.0	fFourCorner	37120
U+7041.0	fCantonese	jyun1
U+7041.0	fMandarin	yuan1
U+7042.0	fUTF8	灂
U+7042.0	fCNS11643	2-692F
U+7042.0	fBig5	F461
U+7042.0	fGB	3-5669
U+7042.0	fR/S	 85.18
U+7042.0	fTotalStrokes	 21
U+7042.0	fCangjie	EBWI
U+7042.0	fFourCorner	32146
U+7042.0	fHakka	 ziok7; zok7; ziok8;  ziau5; tsio5; zio5
U+7042.0	fCantonese	zok6; ziu3; coek3; zuk1
U+7042.0	fMandarin	zhuo2; jiao4
U+7042.0	fEnglish	the rippling of water
U+7043.0	fUTF8	灃
U+7043.0	fCNS11643	2-6B61
U+7043.0	fBig5	F5D3
U+7043.0	fGB	1-6363
U+7043.0	fR/S	 85.18
U+7043.0	fTotalStrokes	 21
U+7043.0	fCangjie	EUJT
U+7043.0	fFourCorner	35118
U+7043.0	fHakka	  fung1
U+7043.0	fCantonese	fung1
U+7043.0	fMandarin	feng1
U+7043.0	fEnglish	a river in Shensi province
U+7044.0	fUTF8	灄
U+7044.0	fCNS11643	2-6B62
U+7044.0	fBig5	F5D4
U+7044.0	fGB	1-645C
U+7044.0	fR/S	 85.18
U+7044.0	fTotalStrokes	 21
U+7044.0	fCangjie	ESJJ
U+7044.0	fFourCorner	31141
U+7044.0	fHakka	 ngiap7
U+7044.0	fCantonese	sip3
U+7044.0	fMandarin	she4; nie4; ni4
U+7044.0	fEnglish	a river in Hubei province
U+7045.0	fUTF8	灅
U+7045.0	fCNS11643	2-6B66
U+7045.0	fBig5	F5D8
U+7045.0	fGB	3-5670
U+7045.0	fR/S	 85.18
U+7045.0	fTotalStrokes	 21
U+7045.0	fCangjie	EWWG
U+7045.0	fFourCorner	36114
U+7045.0	fCantonese	leoi5
U+7045.0	fMandarin	lei3
U+7046.0	fUTF8	灆
U+7046.0	fCNS11643	2-6B67
U+7046.0	fBig5	F5D9
U+7046.0	fGB	5-5722
U+7046.0	fR/S	 85.18
U+7046.0	fTotalStrokes	 21
U+7046.0	fCangjie	ETST
U+7046.0	fFourCorner	34111
U+7046.0	fHakka	 lam2; lam5
U+7046.0	fCantonese	laam4
U+7046.0	fMandarin	lan2; han4; lan4
U+7047.0	fUTF8	灇
U+7047.0	fCNS11643	3-5E3C
U+7047.0	fGB	E-314B
U+7047.0	fR/S	  85.18
U+7047.0	fTotalStrokes	 21
U+7047.0	fCangjie	ETCE
U+7047.0	fFourCorner	32140
U+7047.0	fCantonese	cung4
U+7047.0	fMandarin	cong2
U+7048.0	fUTF8	灈
U+7048.0	fCNS11643	2-6B64
U+7048.0	fBig5	F5D6
U+7048.0	fGB	3-566F
U+7048.0	fR/S	 85.18
U+7048.0	fTotalStrokes	 21
U+7048.0	fCangjie	EBUG
U+7048.0	fFourCorner	36114
U+7048.0	fCantonese	keoi4
U+7048.0	fMandarin	qu2
U+7049.0	fUTF8	灉
U+7049.0	fCNS11643	2-6B65
U+7049.0	fBig5	F5D7
U+7049.0	fGB	3-5673
U+7049.0	fR/S	 85.18
U+7049.0	fTotalStrokes	 21
U+7049.0	fCangjie	EVUG
U+7049.0	fFourCorner	30114
U+7049.0	fHakka	 jung1; rung1
U+7049.0	fCantonese	jung1
U+7049.0	fMandarin	yong1; yong4
U+7049.0	fEnglish	[1] the flowing back of flooding waters [2] a river in Shantung province
U+704A.0	fUTF8	灊
U+704A.0	fCNS11643	2-6B63
U+704A.0	fBig5	F5D5
U+704A.0	fGB	3-566E
U+704A.0	fR/S	 85.18
U+704A.0	fTotalStrokes	 21
U+704A.0	fCangjie	EMUB
U+704A.0	fFourCorner	31127
U+704A.0	fHakka	 siam2;  cim2
U+704A.0	fCantonese	cim4; cam4
U+704A.0	fMandarin	qian2; xun2; cen2
U+704A.0	fEnglish	a tributary of the Yangtsze river in Anhui
U+704B.0	fUTF8	灋
U+704B.0	fCNS11643	3-5E3A
U+704B.0	fGB	E-314C
U+704B.0	fR/S	  85.18
U+704B.0	fTotalStrokes	 21
U+704B.0	fCangjie	EIFI
U+704B.0	fFourCorner	30130
U+704B.0	fHakka	 fap7
U+704B.0	fCantonese	faat3
U+704B.0	fMandarin	fa3
U+704B.0	fEnglish	law
U+704C.0	fUTF8	灌
U+704C.0	fCNS11643	1-7A74
U+704C.0	fBig5	C4E9
U+704C.0	fGB	0-3960
U+704C.0	fR/S	 85.18
U+704C.0	fTotalStrokes	 21
U+704C.0	fCangjie	ETRG
U+704C.0	fFourCorner	34114
U+704C.0	fHakka	 gon5;  kwan5; gwon5
U+704C.0	fCantonese	gun3; gwun3 (also)
U+704C.0	fMandarin	guan4; huan4; guan3
U+704C.0	fEnglish	[1] water; irrigate; fill; pour (on, into, at) [2] offer a libation [3] shrubs; bushy clumps
U+704D.0	fUTF8	灍
U+704D.0	fCNS11643	3-5E3B
U+704D.0	fGB	5-572C
U+704D.0	fR/S	  85.18
U+704D.0	fTotalStrokes	 21
U+704D.0	fCangjie	EANO
U+704D.0	fFourCorner	37120
U+704D.0	fCantonese	kyut3
U+704D.0	fMandarin	que4
U+704E.0	fUTF8	灎
U+704E.0	fGB	E-314D
U+704E.0	fR/S	 85.23
U+704E.0	fTotalStrokes	 26
U+704E.0	fCangjie	ETTT
U+704E.0	fFourCorner	3411
U+704E.0	fCantonese	jim6
U+704E.0	fMandarin	yan4
U+704E.0	fEnglish	 movement of water overflowing; billowing wavy
U+704F.0	fUTF8	灏
U+704F.0	fGB	0-6530
U+704F.0	fR/S	 85.18
U+704F.0	fTotalStrokes	 21
U+704F.0	fCangjie	EAFO
U+704F.0	fHakka	 hau3; hau1
U+704F.0	fCantonese	hou6
U+704F.0	fMandarin	hao4; gan3; gao3
U+704F.0	fEnglish	simplified form of &#x705D;
U+7050.0	fUTF8	灐
U+7050.0	fGB	E-314E
U+7050.0	fR/S	 85.18
U+7050.0	fTotalStrokes	 21
U+7050.0	fCangjie	EFFC
U+7050.0	fCantonese	jing4
U+7051.0	fUTF8	灑
U+7051.0	fCNS11643	1-7B64
U+7051.0	fBig5	C578
U+7051.0	fGB	1-4877
U+7051.0	fR/S	 85.19
U+7051.0	fTotalStrokes	 22
U+7051.0	fCangjie	EMMP
U+7051.0	fFourCorner	31111
U+7051.0	fHakka	  sa3; sha3
U+7051.0	fCantonese	saa2
U+7051.0	fMandarin	sa3; xi3; li2; sha3; xian3; shai4; shi1
U+7051.0	fEnglish	[1] sprinkle (liquids); splash; spill [2] wash
U+7052.0	fUTF8	灒
U+7052.0	fCNS11643	2-6D59
U+7052.0	fBig5	F6EB
U+7052.0	fGB	3-5664
U+7052.0	fR/S	 85.19
U+7052.0	fTotalStrokes	 22
U+7052.0	fCangjie	EHUC
U+7052.0	fFourCorner	34186
U+7052.0	fHakka	 zan5; can5
U+7052.0	fCantonese	zaan3; zaan6
U+7052.0	fMandarin	zan4; cuan2; qian2
U+7052.0	fEnglish	 spatter; splash; scatter
U+7053.0	fUTF8	灓
U+7053.0	fCNS11643	3-6036
U+7053.0	fGB	3-532A
U+7053.0	fR/S	  85.19
U+7053.0	fTotalStrokes	 23
U+7053.0	fCangjie	VFE
U+7053.0	fFourCorner	22230
U+7053.0	fCantonese	lyun4
U+7053.0	fMandarin	luan2; lian4; luan4
U+7054.0	fUTF8	灔
U+7054.0	fCNS11643	3-5F52
U+7054.0	fGB	E-314F
U+7054.0	fR/S	 85.19
U+7054.0	fTotalStrokes	 22
U+7054.0	fCangjie	ETTU
U+7054.0	fFourCorner	37110
U+7054.0	fCantonese	jim6
U+7054.0	fMandarin	yan4
U+7054.0	fEnglish	waves; billows
U+7055.0	fUTF8	灕
U+7055.0	fCNS11643	2-6D56
U+7055.0	fBig5	F6E8
U+7055.0	fGB	1-406C
U+7055.0	fR/S	 85.19
U+7055.0	fTotalStrokes	 22
U+7055.0	fCangjie	EYBG
U+7055.0	fFourCorner	30114
U+7055.0	fHakka	 li2
U+7055.0	fCantonese	lei4
U+7055.0	fMandarin	li2
U+7055.0	fEnglish	a short name for Guangxi province
U+7056.0	fUTF8	灖
U+7056.0	fCNS11643	2-6D57
U+7056.0	fBig5	F6E9
U+7056.0	fGB	3-5674
U+7056.0	fR/S	 85.19
U+7056.0	fTotalStrokes	 22
U+7056.0	fCangjie	EIDY
U+7056.0	fFourCorner	30111
U+7056.0	fCantonese	mei5
U+7056.0	fMandarin	mi3
U+7057.0	fUTF8	灗
U+7057.0	fCNS11643	2-6D58
U+7057.0	fBig5	F6EA
U+7057.0	fGB	E-3150
U+7057.0	fR/S	 85.19
U+7057.0	fTotalStrokes	 22
U+7057.0	fCangjie	ELIM
U+7057.0	fFourCorner	30117
U+7057.0	fCantonese	sin5
U+7057.0	fMandarin	shan4
U+7058.0	fUTF8	灘
U+7058.0	fCNS11643	1-7B65
U+7058.0	fBig5	C579
U+7058.0	fGB	1-4C32
U+7058.0	fR/S	 85.19
U+7058.0	fTotalStrokes	 22
U+7058.0	fCangjie	ETOG
U+7058.0	fFourCorner	30114
U+7058.0	fHakka	  tan1
U+7058.0	fCantonese	taan1; taan5
U+7058.0	fMandarin	tan1; nan4; han3; han4; tan4
U+7058.0	fEnglish	 bank; beach; shoal; sandback
U+7059.0	fUTF8	灙
U+7059.0	fCNS11643	3-6037
U+7059.0	fGB	3-554F
U+7059.0	fR/S	  85.20
U+7059.0	fTotalStrokes	 23
U+7059.0	fCangjie	EFBF
U+7059.0	fFourCorner	39130
U+7059.0	fCantonese	dong2
U+7059.0	fMandarin	dang3
U+705A.0	fUTF8	灚
U+705A.0	fCNS11643	2-6F34
U+705A.0	fBig5	F7E5
U+705A.0	fGB	5-5637
U+705A.0	fR/S	 85.20
U+705A.0	fTotalStrokes	 23
U+705A.0	fCangjie	EHBU
U+705A.0	fFourCorner	37116
U+705A.0	fCantonese	gaau2
U+705A.0	fMandarin	jiao3
U+705B.0	fUTF8	灛
U+705B.0	fCNS11643	2-6F33
U+705B.0	fBig5	F7E4
U+705B.0	fGB	3-557C
U+705B.0	fR/S	 85.20
U+705B.0	fTotalStrokes	 23
U+705B.0	fCangjie	EANJ
U+705B.0	fFourCorner	37120
U+705B.0	fCantonese	cin2; zin2
U+705B.0	fMandarin	chan3
U+705C.0	fUTF8	灜
U+705C.0	fCNS11643	3-654B
U+705C.0	fGB	E-3151
U+705C.0	fR/S	 85.20
U+705C.0	fTotalStrokes	 23
U+705C.0	fCangjie	EYNC
U+705C.0	fFourCorner	3011
U+705C.0	fCantonese	jing4
U+705D.0	fUTF8	灝
U+705D.0	fCNS11643	2-703D
U+705D.0	fBig5	F8AF
U+705D.0	fGB	1-6530
U+705D.0	fR/S	 85.21
U+705D.0	fTotalStrokes	 24
U+705D.0	fCangjie	EAFC
U+705D.0	fFourCorner	31186
U+705D.0	fHakka	  hau1;  hau3; hau6
U+705D.0	fCantonese	hou6
U+705D.0	fMandarin	hao4; gan3; gao3
U+705D.0	fEnglish	bean soup
U+705E.0	fUTF8	灞
U+705E.0	fCNS11643	1-7C60
U+705E.0	fBig5	C5F4
U+705E.0	fGB	0-6531
U+705E.0	fR/S	 85.21
U+705E.0	fTotalStrokes	 24
U+705E.0	fCangjie	EMBB
U+705E.0	fFourCorner	31127
U+705E.0	fHakka	  ba5
U+705E.0	fCantonese	baa3
U+705E.0	fMandarin	ba4
U+705E.0	fEnglish	a river in Shensi province
U+705F.0	fUTF8	灟
U+705F.0	fCNS11643	2-703B
U+705F.0	fBig5	F8AD
U+705F.0	fGB	3-563D
U+705F.0	fR/S	 85.21
U+705F.0	fTotalStrokes	 24
U+705F.0	fCangjie	ESEI
U+705F.0	fFourCorner	37127
U+705F.0	fCantonese	zuk1
U+705F.0	fMandarin	zhu2
U+7060.0	fUTF8	灠
U+7060.0	fCNS11643	2-703E
U+7060.0	fBig5	F8B0
U+7060.0	fGB	5-562D
U+7060.0	fR/S	 85.21
U+7060.0	fTotalStrokes	 24
U+7060.0	fCangjie	ESWU
U+7060.0	fFourCorner	38116
U+7060.0	fHakka	 lam5
U+7060.0	fCantonese	laam5; laam6 (also)
U+7060.0	fMandarin	lan3; lan4
U+7060.0	fEnglish	marinate in salt
U+7061.0	fUTF8	灡
U+7061.0	fCNS11643	2-703C
U+7061.0	fBig5	F8AE
U+7061.0	fGB	E-3152
U+7061.0	fR/S	 85.21
U+7061.0	fTotalStrokes	 24
U+7061.0	fCangjie	ETAW
U+7061.0	fFourCorner	34127
U+7061.0	fHakka	 lan2
U+7061.0	fCantonese	laan4
U+7061.0	fMandarin	lan2; lan3; lan4
U+7062.0	fUTF8	灢
U+7062.0	fCNS11643	2-7125
U+7062.0	fBig5	F8F5
U+7062.0	fGB	E-3153
U+7062.0	fR/S	 85.22
U+7062.0	fTotalStrokes	 25
U+7062.0	fCangjie	EJBV
U+7062.0	fFourCorner	30132
U+7062.0	fHakka	  nong5; nong2
U+7062.0	fCantonese	nong5; long5 (also); nong6
U+7062.0	fMandarin	nang3; nang4
U+7062.0	fEnglish	 muddy thick; muddy water
U+7063.0	fUTF8	灣
U+7063.0	fCNS11643	1-7D24
U+7063.0	fBig5	C657
U+7063.0	fGB	1-4D65
U+7063.0	fR/S	 85.22
U+7063.0	fTotalStrokes	 25
U+7063.0	fCangjie	EVFN
U+7063.0	fFourCorner	32127
U+7063.0	fHakka	  wan1; wan2; wan3; van1
U+7063.0	fCantonese	waan4 (also); waan1; wan1 (also)
U+7063.0	fMandarin	wan1
U+7063.0	fEnglish	[1] bay; cove; inlet; gulf [2] the bend of stream [3] anchor; moor
U+7064.0	fUTF8	灤
U+7064.0	fCNS11643	1-7D32
U+7064.0	fBig5	C665
U+7064.0	fGB	1-4250
U+7064.0	fR/S	 85.23
U+7064.0	fTotalStrokes	 26
U+7064.0	fCangjie	EVFD
U+7064.0	fFourCorner	32194
U+7064.0	fHakka	 lan2
U+7064.0	fCantonese	lyun4
U+7064.0	fMandarin	luan2
U+7064.0	fEnglish	a river in northern China
U+7065.0	fUTF8	灥
U+7065.0	fCNS11643	2-7171
U+7065.0	fBig5	F9A3
U+7065.0	fGB	5-5B68
U+7065.0	fR/S	 85.23
U+7065.0	fTotalStrokes	 27
U+7065.0	fCangjie	HEHEE
U+7065.0	fFourCorner	26232
U+7065.0	fCantonese	ceon4; cyun1; cyun4
U+7065.0	fMandarin	xun2; chuan1; qun2
U+7066.0	fUTF8	灦
U+7066.0	fCNS11643	2-715B
U+7066.0	fBig5	F96C
U+7066.0	fGB	E-3154
U+7066.0	fR/S	 85.23
U+7066.0	fTotalStrokes	 26
U+7066.0	fCangjie	EAFC
U+7066.0	fFourCorner	36186
U+7066.0	fCantonese	hin2
U+7066.0	fMandarin	xian3; xian4
U+7067.0	fUTF8	灧
U+7067.0	fCNS11643	3-6179
U+7067.0	fGB	1-6459
U+7067.0	fR/S	 85.24
U+7067.0	fTotalStrokes	 27
U+7067.0	fCangjie	EUTU
U+7067.0	fFourCorner	37117
U+7067.0	fCantonese	jim6
U+7067.0	fMandarin	yan4
U+7067.0	fEnglish	 overflowing; billowing wavy
U+7068.0	fUTF8	灨
U+7068.0	fCNS11643	2-7170
U+7068.0	fBig5	F9A2
U+7068.0	fGB	E-3155
U+7068.0	fR/S	 85.24
U+7068.0	fTotalStrokes	 27
U+7068.0	fCangjie	EYJC
U+7068.0	fFourCorner	37186
U+7068.0	fHakka	  gam5
U+7068.0	fCantonese	gam3
U+7068.0	fMandarin	gan3; gan4; gong4
U+7068.0	fEnglish	the Kan river in Kiangsi province
U+7069.0	fUTF8	灩
U+7069.0	fCNS11643	2-723E
U+7069.0	fBig5	F9D0
U+7069.0	fGB	E-3156
U+7069.0	fR/S	 85.28
U+7069.0	fTotalStrokes	 31
U+7069.0	fCangjie	EUTT
U+7069.0	fFourCorner	34117
U+7069.0	fHakka	 ram6; jam5; jam3
U+7069.0	fCantonese	jim6
U+7069.0	fMandarin	yan4
U+7069.0	fEnglish	overflowing; inundating
U+706A.0	fUTF8	灪
U+706A.0	fCNS11643	2-723F
U+706A.0	fBig5	F9D1
U+706A.0	fGB	3-5676
U+706A.0	fR/S	 85.29
U+706A.0	fTotalStrokes	 32
U+706A.0	fCangjie	EDDH
U+706A.0	fFourCorner	34122
U+706A.0	fCantonese	wat1
U+706A.0	fMandarin	yu4
U+706B.0	fUTF8	火
U+706B.0	fCNS11643	1-4556
U+706B.0	fBig5	A4F5
U+706B.0	fGB	0-3B70
U+706B.0	fR/S	 86.0
U+706B.0	fTotalStrokes	 4
U+706B.0	fCangjie	F
U+706B.0	fFourCorner	90800
U+706B.0	fHakka	  fo3
U+706B.0	fCantonese	fo2
U+706B.0	fMandarin	huo3; huo1
U+706B.0	fEnglish	[1] fire; flame; burn with fire [2] anger; rage; anger; temper [3] urgency; urgent; imminent; pressing [4] the latent heat in a human body (Chinese medicine) [5] a group (people) [6] a Chinese family name [7] Kangxi radical 86
U+706C.0	fUTF8	灬
U+706C.0	fCNS11643	4-2150
U+706C.0	fGB	0-6C61
U+706C.0	fR/S	 86.0
U+706C.0	fTotalStrokes	 4
U+706C.0	fCangjie	HF
U+706C.0	fFourCorner	33000
U+706C.0	fHakka	 fo3
U+706C.0	fCantonese	biu1
U+706C.0	fMandarin	biao1; huo3; hou3
U+706C.0	fEnglish	 fire Kangxi radical number 86
U+706D.0	fUTF8	灭
U+706D.0	fGB	0-4370
U+706D.0	fR/S	 86.1
U+706D.0	fTotalStrokes	 5
U+706D.0	fCangjie	XMF
U+706D.0	fHakka	 met8
U+706D.0	fCantonese	mit6
U+706D.0	fMandarin	mie4
U+706D.0	fEnglish	simplified form of &#x6EC5;
U+706E.0	fUTF8	灮
U+706E.0	fCNS11643	3-2428
U+706E.0	fGB	E-3157
U+706E.0	fR/S	  86.2
U+706E.0	fTotalStrokes	 6
U+706E.0	fCangjie	FHU
U+706E.0	fFourCorner	90210
U+706E.0	fCantonese	gwong1; gong1 (also)
U+706E.0	fMandarin	guang1; guang2
U+706F.0	fUTF8	灯
U+706F.0	fCNS11643	3-2426
U+706F.0	fGB	0-3546
U+706F.0	fR/S	 86.2
U+706F.0	fTotalStrokes	 6
U+706F.0	fCangjie	FMN
U+706F.0	fFourCorner	91820
U+706F.0	fHakka	  den1
U+706F.0	fCantonese	dang1
U+706F.0	fMandarin	deng1
U+706F.0	fEnglish	simplified form of &#x71C8;
U+7070.0	fUTF8	灰
U+7070.0	fCNS11643	1-4848
U+7070.0	fBig5	A6C7
U+7070.0	fGB	0-3B52
U+7070.0	fR/S	 86.2
U+7070.0	fTotalStrokes	 6
U+7070.0	fCangjie	KF
U+7070.0	fFourCorner	71289
U+7070.0	fHakka	  foi1; foe1
U+7070.0	fCantonese	fui1
U+7070.0	fMandarin	hui1
U+7070.0	fEnglish	[1] ashes; dust [2] lime [3] gray (color) [4] break into tiny pieces or particles [5] disheartened disappointedly; discouraged
U+7071.0	fUTF8	灱
U+7071.0	fCNS11643	2-2261
U+7071.0	fBig5	CA41
U+7071.0	fGB	5-512F
U+7071.0	fR/S	 86.2
U+7071.0	fTotalStrokes	 6
U+7071.0	fCangjie	FSH
U+7071.0	fFourCorner	97820
U+7071.0	fCantonese	haau1
U+7071.0	fMandarin	xiao1
U+7072.0	fUTF8	灲
U+7072.0	fCNS11643	3-2427
U+7072.0	fGB	E-3158
U+7072.0	fR/S	 86.2
U+7072.0	fTotalStrokes	 6
U+7072.0	fCangjie	FLN
U+7072.0	fFourCorner	92800
U+7072.0	fCantonese	haau1
U+7072.0	fMandarin	xiao1
U+7073.0	fUTF8	灳
U+7073.0	fCNS11643	3-242B
U+7073.0	fGB	E-3159
U+7073.0	fR/S	 86.2; 86.4; 20.4
U+7073.0	fTotalStrokes	 6
U+7073.0	fCangjie	PF
U+7073.0	fFourCorner	27820
U+7073.0	fCantonese	fai1
U+7073.0	fMandarin	hui1
U+7074.0	fUTF8	灴
U+7074.0	fCNS11643	2-245E
U+7074.0	fBig5	CB5E
U+7074.0	fGB	5-5131
U+7074.0	fR/S	 86.3
U+7074.0	fTotalStrokes	 7
U+7074.0	fCangjie	FM
U+7074.0	fFourCorner	91810
U+7074.0	fHakka	 fung2
U+7074.0	fCantonese	hung4; hung3
U+7074.0	fMandarin	hong2
U+7074.0	fEnglish	 bake; roast dry at a fire
U+7075.0	fUTF8	灵
U+7075.0	fCNS11643	3-2662
U+7075.0	fGB	0-4169
U+7075.0	fR/S	 86.3; 58.4
U+7075.0	fTotalStrokes	 7
U+7075.0	fCangjie	SMF
U+7075.0	fFourCorner	17800
U+7075.0	fHakka	  lin2
U+7075.0	fCantonese	ling4; ling2 (also); leng4 (also)
U+7075.0	fMandarin	ling2; dai4; ling4
U+7075.0	fEnglish	simplified form of &#x9748;
U+7075.1	fUTF8	灵
U+7075.1	fCNS11643	3-2662
U+7075.1	fGB	0-4169
U+7075.1	fR/S	 86.3; 58.4
U+7075.1	fTotalStrokes	 7
U+7075.1	fCangjie	SMF
U+7075.1	fFourCorner	17800
U+7075.1	fMandarin	ling5
U+7076.0	fUTF8	灶
U+7076.0	fCNS11643	1-4B22
U+7076.0	fBig5	A85F
U+7076.0	fGB	0-546E
U+7076.0	fR/S	 86.3
U+7076.0	fTotalStrokes	 7
U+7076.0	fCangjie	FG
U+7076.0	fFourCorner	30717
U+7076.0	fHakka	  zau5;  zo5
U+7076.0	fCantonese	zou3
U+7076.0	fMandarin	zao4
U+7076.0	fEnglish	simplified form of &#x7AC8;
U+7077.0	fUTF8	灷
U+7077.0	fCNS11643	3-265F
U+7077.0	fGB	3-4F4C
U+7077.0	fR/S	  86.3
U+7077.0	fTotalStrokes	 7
U+7077.0	fCangjie	FT
U+7077.0	fFourCorner	90440
U+7077.0	fCantonese	zyun6
U+7077.0	fMandarin	zhuan4
U+7078.0	fUTF8	灸
U+7078.0	fCNS11643	1-4B25
U+7078.0	fBig5	A862
U+7078.0	fGB	0-3E44
U+7078.0	fR/S	 86.3
U+7078.0	fTotalStrokes	 7
U+7078.0	fCangjie	NOF
U+7078.0	fFourCorner	27809
U+7078.0	fHakka	  giu5; giu3; gui3
U+7078.0	fCantonese	gau3
U+7078.0	fMandarin	jiu3; jiu4
U+7078.0	fEnglish	 cauterize by burning moxa; moxibustion; moxa cautery
U+7079.0	fUTF8	灹
U+7079.0	fCNS11643	3-265E
U+7079.0	fGB	3-4F4E
U+7079.0	fR/S	  86.3
U+7079.0	fTotalStrokes	 7
U+7079.0	fCangjie	FHP
U+7079.0	fFourCorner	92810
U+7079.0	fHakka	 za5
U+7079.0	fCantonese	zaa3
U+7079.0	fMandarin	zha4; yu4
U+7079.0	fEnglish	 crack (as of a sharp clap of thunder); report of a gun
U+707A.0	fUTF8	灺
U+707A.0	fCNS11643	2-245F
U+707A.0	fBig5	CB5F
U+707A.0	fGB	5-5132
U+707A.0	fR/S	 86.3
U+707A.0	fTotalStrokes	 7
U+707A.0	fCangjie	FPD
U+707A.0	fFourCorner	95817
U+707A.0	fHakka	  sia3
U+707A.0	fCantonese	se3; ce5 (also); ce2 (also); se2; ce6
U+707A.0	fMandarin	xie4; duo4
U+707A.0	fEnglish	candle-end
U+707B.0	fUTF8	灻
U+707B.0	fCNS11643	3-2663
U+707B.0	fGB	E-315A
U+707B.0	fR/S	 86.3
U+707B.0	fTotalStrokes	 7
U+707B.0	fCangjie	GF
U+707B.0	fFourCorner	40800
U+707B.0	fCantonese	cek3
U+707B.0	fMandarin	chi4
U+707C.0	fUTF8	灼
U+707C.0	fCNS11643	1-4B23
U+707C.0	fBig5	A860
U+707C.0	fGB	0-5746
U+707C.0	fR/S	 86.3
U+707C.0	fTotalStrokes	 7
U+707C.0	fCangjie	FPI
U+707C.0	fFourCorner	97820
U+707C.0	fHakka	 zok7; chok7; shok7; sok7; shok8
U+707C.0	fCantonese	coek3; zoek3 (also)
U+707C.0	fMandarin	zhuo2
U+707C.0	fEnglish	[1] burn; cauterize [2] bright; clear; luminous; brilliant [3] flowers in full bloom [4] cook by soaking in boiling water
U+707D.0	fUTF8	災
U+707D.0	fCNS11643	1-4B24
U+707D.0	fBig5	A861
U+707D.0	fGB	E-315B
U+707D.0	fR/S	 86.3
U+707D.0	fTotalStrokes	 7
U+707D.0	fCangjie	VVF
U+707D.0	fFourCorner	22809
U+707D.0	fHakka	  zai1; coi1
U+707D.0	fCantonese	zoi1
U+707D.0	fMandarin	zai1
U+707D.0	fEnglish	 calamity; disaster; catastrophe
U+707E.0	fUTF8	灾
U+707E.0	fCNS11643	3-2660
U+707E.0	fGB	0-5456
U+707E.0	fR/S	  86.3
U+707E.0	fTotalStrokes	 7
U+707E.0	fCangjie	JF
U+707E.0	fFourCorner	30800
U+707E.0	fHakka	 zai1
U+707E.0	fCantonese	zoi1
U+707E.0	fMandarin	zai1
U+707E.0	fEnglish	calamity; disaster; catastrophe
U+707F.0	fUTF8	灿
U+707F.0	fCNS11643	F-2471
U+707F.0	fGB	0-3253
U+707F.0	fR/S	 86.3
U+707F.0	fTotalStrokes	 7
U+707F.0	fCangjie	FU
U+707F.0	fFourCorner	9287
U+707F.0	fHakka	 can3
U+707F.0	fCantonese	caan3; can3
U+707F.0	fMandarin	can4
U+707F.0	fEnglish	simplified form of &#x71E6;
U+7080.0	fUTF8	炀
U+7080.0	fGB	0-6C3E
U+7080.0	fR/S	 86.3
U+7080.0	fTotalStrokes	 7
U+7080.0	fCangjie	FNSH
U+7080.0	fHakka	 jit8; jong2
U+7080.0	fCantonese	joeng4; joeng6
U+7080.0	fMandarin	yang2; shang1; yang4; xiang3
U+7080.0	fEnglish	simplified form of &#x716C;
U+7081.0	fUTF8	炁
U+7081.0	fCNS11643	3-2977
U+7081.0	fGB	5-427B
U+7081.0	fR/S	  86.4; 86.5
U+7081.0	fTotalStrokes	 8
U+7081.0	fCangjie	MUF
U+7081.0	fFourCorner	10330
U+7081.0	fHakka	  hi5
U+7081.0	fCantonese	hei3
U+7081.0	fMandarin	qi4
U+7081.0	fEnglish	breath; air; steam; gas; weather; used in Taoist charms
U+7082.0	fUTF8	炂
U+7082.0	fCNS11643	2-2778
U+7082.0	fBig5	CD58
U+7082.0	fGB	3-4F59
U+7082.0	fR/S	 86.4
U+7082.0	fTotalStrokes	 8
U+7082.0	fCangjie	FCI
U+7082.0	fFourCorner	98832
U+7082.0	fHakka	 zhung1
U+7082.0	fCantonese	zung1
U+7082.0	fMandarin	zhong1
U+7083.0	fUTF8	炃
U+7083.0	fCNS11643	2-277A
U+7083.0	fBig5	CD5A
U+7083.0	fGB	5-5139
U+7083.0	fR/S	 86.4
U+7083.0	fTotalStrokes	 8
U+7083.0	fCangjie	CSHF
U+7083.0	fFourCorner	80809
U+7083.0	fHakka	 fun2
U+7083.0	fCantonese	fan4; ban6
U+7083.0	fMandarin	fen2; ben4
U+7084.0	fUTF8	炄
U+7084.0	fCNS11643	2-2775
U+7084.0	fBig5	CD55
U+7084.0	fGB	5-513E
U+7084.0	fR/S	 86.4
U+7084.0	fTotalStrokes	 8
U+7084.0	fCangjie	FNG
U+7084.0	fFourCorner	97815
U+7084.0	fCantonese	nau5; lau5 (also)
U+7084.0	fMandarin	niu3
U+7085.0	fUTF8	炅
U+7085.0	fCNS11643	2-2772
U+7085.0	fBig5	CD52
U+7085.0	fGB	0-6A41
U+7085.0	fR/S	 86.4
U+7085.0	fTotalStrokes	 8
U+7085.0	fCangjie	AF
U+7085.0	fFourCorner	60809
U+7085.0	fHakka	 gui6; gui5; kwui5
U+7085.0	fCantonese	gwing2; gwai3; gwai6 (also)
U+7085.0	fMandarin	jiong3; gui4; jing3; geng3; ying3
U+7085.0	fEnglish	brilliance
U+7086.0	fUTF8	炆
U+7086.0	fCNS11643	2-2774
U+7086.0	fBig5	CD54
U+7086.0	fGB	3-4F5B
U+7086.0	fR/S	 86.4
U+7086.0	fTotalStrokes	 8
U+7086.0	fCangjie	FYK
U+7086.0	fFourCorner	90841
U+7086.0	fHakka	 mun1
U+7086.0	fCantonese	man1
U+7086.0	fMandarin	wen2
U+7086.0	fEnglish	 cook on a small fire
U+7087.0	fUTF8	炇
U+7087.0	fCNS11643	3-2974
U+7087.0	fGB	5-5137
U+7087.0	fR/S	 86.2; 86.4
U+7087.0	fTotalStrokes	 8; 6
U+7087.0	fCangjie	FOK
U+7087.0	fFourCorner	98840
U+7087.0	fCantonese	pok3
U+7087.0	fMandarin	po4
U+7088.0	fUTF8	炈
U+7088.0	fCNS11643	3-2970
U+7088.0	fGB	3-4F5A
U+7088.0	fR/S	  86.4
U+7088.0	fTotalStrokes	 8
U+7088.0	fCangjie	FHNE
U+7088.0	fFourCorner	97840
U+7088.0	fCantonese	jik6
U+7088.0	fMandarin	yi4
U+7089.0	fUTF8	炉
U+7089.0	fCNS11643	3-2975
U+7089.0	fGB	0-422F
U+7089.0	fR/S	 86.4
U+7089.0	fTotalStrokes	 8
U+7089.0	fCangjie	FHS
U+7089.0	fFourCorner	93870
U+7089.0	fHakka	  lu2
U+7089.0	fCantonese	lou4; lou2 (also)
U+7089.0	fMandarin	lu2
U+7089.0	fEnglish	simplified form of &#x7210;
U+708A.0	fUTF8	炊
U+708A.0	fCNS11643	1-4E65
U+708A.0	fBig5	AAA4
U+708A.0	fGB	0-3436
U+708A.0	fR/S	 86.4
U+708A.0	fTotalStrokes	 8
U+708A.0	fCangjie	FNO
U+708A.0	fFourCorner	97882
U+708A.0	fHakka	 cui1;  ch'oi1; ch'ui1; coi1; chui1; choi1
U+708A.0	fCantonese	ceoi1; ceoi3
U+708A.0	fMandarin	chui1; chui4
U+708A.0	fEnglish	 cook
U+708B.0	fUTF8	炋
U+708B.0	fCNS11643	3-296F
U+708B.0	fGB	E-315C
U+708B.0	fR/S	  86.4
U+708B.0	fTotalStrokes	 8
U+708B.0	fCangjie	FMF
U+708B.0	fFourCorner	91890
U+708B.0	fCantonese	pui1
U+708B.0	fMandarin	pi1
U+708C.0	fUTF8	炌
U+708C.0	fCNS11643	3-2979
U+708C.0	fGB	3-4F57
U+708C.0	fR/S	  86.4
U+708C.0	fTotalStrokes	 8
U+708C.0	fCangjie	FOLL
U+708C.0	fFourCorner	98820
U+708C.0	fCantonese	kaai3
U+708C.0	fMandarin	kai4
U+708D.0	fUTF8	炍
U+708D.0	fCNS11643	3-2976
U+708D.0	fGB	E-315D
U+708D.0	fR/S	  86.4
U+708D.0	fTotalStrokes	 8
U+708D.0	fCangjie	FHE
U+708D.0	fFourCorner	91840
U+708D.0	fCantonese	pun3
U+708D.0	fMandarin	pan4
U+708E.0	fUTF8	炎
U+708E.0	fCNS11643	1-4E63
U+708E.0	fBig5	AAA2
U+708E.0	fGB	0-5157
U+708E.0	fR/S	 86.4
U+708E.0	fTotalStrokes	 8
U+708E.0	fCangjie	FF
U+708E.0	fFourCorner	90809
U+708E.0	fHakka	 jam2; ram2; jam3; ram6; jam5
U+708E.0	fCantonese	jim4
U+708E.0	fMandarin	yan2; tan2; yan4
U+708E.0	fEnglish	[1] burning; hot; sultry [2] blaze; flame; flare up [3] inflammation
U+708F.0	fUTF8	炏
U+708F.0	fCNS11643	3-2972
U+708F.0	fGB	5-513C
U+708F.0	fR/S	 86.4
U+708F.0	fTotalStrokes	 8
U+708F.0	fCangjie	FF
U+708F.0	fFourCorner	99880
U+708F.0	fCantonese	kaai3
U+708F.0	fMandarin	kai4
U+7090.0	fUTF8	炐
U+7090.0	fCNS11643	3-2971
U+7090.0	fGB	3-4F50
U+7090.0	fR/S	  86.4
U+7090.0	fTotalStrokes	 8
U+7090.0	fCangjie	FQJ
U+7090.0	fFourCorner	95800
U+7090.0	fCantonese	pong3
U+7090.0	fMandarin	pang4
U+7091.0	fUTF8	炑
U+7091.0	fCNS11643	2-2776
U+7091.0	fBig5	CD56
U+7091.0	fGB	5-5135
U+7091.0	fR/S	 86.4
U+7091.0	fTotalStrokes	 8
U+7091.0	fCangjie	FD
U+7091.0	fFourCorner	94890
U+7091.0	fCantonese	muk6
U+7091.0	fMandarin	mu4
U+7092.0	fUTF8	炒
U+7092.0	fCNS11643	1-4E64
U+7092.0	fBig5	AAA3
U+7092.0	fGB	0-3334
U+7092.0	fR/S	 86.4
U+7092.0	fTotalStrokes	 8
U+7092.0	fCangjie	FFH
U+7092.0	fFourCorner	99820
U+7092.0	fHakka	  cau3; ciau3
U+7092.0	fCantonese	caau2
U+7092.0	fMandarin	chao3
U+7092.0	fEnglish	 fry; stir-fry
U+7093.0	fUTF8	炓
U+7093.0	fCNS11643	2-2773
U+7093.0	fBig5	CD53
U+7093.0	fGB	5-513D
U+7093.0	fR/S	 86.4
U+7093.0	fTotalStrokes	 8
U+7093.0	fCangjie	FYJ
U+7093.0	fFourCorner	94800
U+7093.0	fCantonese	liu6; tau3
U+7093.0	fMandarin	liao4
U+7094.0	fUTF8	炔
U+7094.0	fCNS11643	2-2770
U+7094.0	fBig5	CD50
U+7094.0	fGB	0-4832
U+7094.0	fR/S	 86.4
U+7094.0	fTotalStrokes	 8
U+7094.0	fCangjie	FDK
U+7094.0	fFourCorner	95830
U+7094.0	fHakka	 gui5; ket7; kwui5
U+7094.0	fCantonese	kyut3; gai3; gwai3 (also)
U+7094.0	fMandarin	gui4; jue2; xue4; que1; geng3
U+7094.0	fEnglish	acetylene
U+7095.0	fUTF8	炕
U+7095.0	fCNS11643	1-4E62
U+7095.0	fBig5	AAA1
U+7095.0	fGB	0-3F3B
U+7095.0	fR/S	 86.4
U+7095.0	fTotalStrokes	 8
U+7095.0	fCangjie	FYHN
U+7095.0	fFourCorner	90817
U+7095.0	fHakka	  kong5
U+7095.0	fCantonese	kong3; hong3
U+7095.0	fMandarin	kang4; hang1; kang1; kang3
U+7095.0	fEnglish	[1] dry; dry [2] hot [3] kang, a brick bed warmed by a fire underneath (in northern China)
U+7096.0	fUTF8	炖
U+7096.0	fCNS11643	2-2777
U+7096.0	fBig5	CD57
U+7096.0	fGB	0-6C40
U+7096.0	fR/S	 86.4
U+7096.0	fTotalStrokes	 8
U+7096.0	fCangjie	FPU
U+7096.0	fFourCorner	95817
U+7096.0	fHakka	 dun1
U+7096.0	fCantonese	dan6; deon6 (also); dan3; deon1
U+7096.0	fMandarin	dun4; tun1; tun2
U+7096.0	fEnglish	[1] fire burning intensely [2] cook on fire
U+7097.0	fUTF8	炗
U+7097.0	fCNS11643	4-2729
U+7097.0	fGB	5-5134
U+7097.0	fR/S	 86.4
U+7097.0	fTotalStrokes	 8
U+7097.0	fCangjie	TF
U+7097.0	fFourCorner	44800
U+7097.0	fCantonese	gwong1; gong1 (also)
U+7097.0	fMandarin	guang1
U+7098.0	fUTF8	炘
U+7098.0	fCNS11643	2-2771
U+7098.0	fBig5	CD51
U+7098.0	fGB	3-4F56
U+7098.0	fR/S	 86.4
U+7098.0	fTotalStrokes	 8
U+7098.0	fCangjie	FHML
U+7098.0	fFourCorner	92821
U+7098.0	fHakka	 hiun1
U+7098.0	fCantonese	jan1; jan3
U+7098.0	fMandarin	xin1; xin3; xin4
U+7098.0	fEnglish	[1] brilliant; shining; bright [2] scorch; scorching hot
U+7099.0	fUTF8	炙
U+7099.0	fCNS11643	1-4E66
U+7099.0	fBig5	AAA5
U+7099.0	fGB	0-564B
U+7099.0	fR/S	 86.4
U+7099.0	fTotalStrokes	 8
U+7099.0	fCangjie	BF
U+7099.0	fFourCorner	27809
U+7099.0	fHakka	 chak7; zak7; zhak7
U+7099.0	fCantonese	zek3; zik3 (also)
U+7099.0	fMandarin	zhi4; zhe4
U+7099.0	fEnglish	 roast; broil; heat; cauterize; burn
U+709A.0	fUTF8	炚
U+709A.0	fCNS11643	2-2779
U+709A.0	fBig5	CD59
U+709A.0	fGB	5-4959
U+709A.0	fR/S	 86.4
U+709A.0	fTotalStrokes	 8
U+709A.0	fCangjie	AF
U+709A.0	fFourCorner	69080
U+709A.0	fCantonese	gwong1; gong1 (also)
U+709A.0	fMandarin	guang1
U+709B.0	fUTF8	炛
U+709B.0	fCNS11643	4-272B
U+709B.0	fGB	5-5138
U+709B.0	fR/S	 86.4
U+709B.0	fTotalStrokes	 8
U+709B.0	fCangjie	FOP
U+709B.0	fFourCorner	90210
U+709B.0	fCantonese	gwong1; gong1 (also)
U+709B.0	fMandarin	guang1; guang2; guang4
U+709C.0	fUTF8	炜
U+709C.0	fGB	0-6C3F
U+709C.0	fR/S	 86.4
U+709C.0	fTotalStrokes	 8
U+709C.0	fCangjie	FQS
U+709C.0	fHakka	 vui3
U+709C.0	fCantonese	wai5
U+709C.0	fMandarin	wei3; hui1
U+709C.0	fEnglish	simplified form of &#x7152;
U+709D.0	fUTF8	炝
U+709D.0	fGB	0-6C41
U+709D.0	fR/S	 86.4
U+709D.0	fTotalStrokes	 8
U+709D.0	fCangjie	FOSU
U+709D.0	fCantonese	coeng3
U+709D.0	fMandarin	qiang4
U+709D.0	fEnglish	simplified form of &#x7197;
U+709E.0	fUTF8	炞
U+709E.0	fGB	8-2D6A
U+709E.0	fR/S	 86.4
U+709E.0	fTotalStrokes	 8
U+709E.0	fCangjie	FYY
U+709E.0	fCantonese	bin6
U+709E.0	fMandarin	bian4
U+709F.0	fUTF8	炟
U+709F.0	fCNS11643	2-2B6F
U+709F.0	fBig5	CFAF
U+709F.0	fGB	3-4F60
U+709F.0	fR/S	 86.5
U+709F.0	fTotalStrokes	 9
U+709F.0	fCangjie	FAM
U+709F.0	fFourCorner	96810
U+709F.0	fCantonese	daat3
U+709F.0	fMandarin	da2
U+70A0.0	fUTF8	炠
U+70A0.0	fCNS11643	3-2D5E
U+70A0.0	fGB	3-4F61
U+70A0.0	fR/S	  86.5
U+70A0.0	fTotalStrokes	 9
U+70A0.0	fCangjie	FWL
U+70A0.0	fFourCorner	96850
U+70A0.0	fCantonese	saap6
U+70A0.0	fMandarin	xia2
U+70A0.0	fEnglish	cook in boiling water
U+70A1.0	fUTF8	炡
U+70A1.0	fCNS11643	2-2B73
U+70A1.0	fBig5	CFB3
U+70A1.0	fGB	5-5141
U+70A1.0	fR/S	 86.5
U+70A1.0	fTotalStrokes	 9
U+70A1.0	fCangjie	FMYM
U+70A1.0	fFourCorner	91811
U+70A1.0	fCantonese	zing1
U+70A1.0	fMandarin	zheng1
U+70A2.0	fUTF8	炢
U+70A2.0	fCNS11643	3-2D5C
U+70A2.0	fGB	3-4F5C
U+70A2.0	fR/S	 86.5
U+70A2.0	fTotalStrokes	 9
U+70A2.0	fCangjie	FID
U+70A2.0	fFourCorner	93890
U+70A2.0	fCantonese	seot6
U+70A2.0	fMandarin	zhu2
U+70A3.0	fUTF8	炣
U+70A3.0	fCNS11643	3-2D5A
U+70A3.0	fGB	3-4F5D
U+70A3.0	fR/S	 86.5
U+70A3.0	fTotalStrokes	 9
U+70A3.0	fCangjie	FMNR
U+70A3.0	fFourCorner	91820
U+70A3.0	fHakka	 ho1; ho5
U+70A3.0	fCantonese	ho2
U+70A3.0	fMandarin	ke3
U+70A4.0	fUTF8	炤
U+70A4.0	fCNS11643	1-523A
U+70A4.0	fBig5	ACB7
U+70A4.0	fGB	E-315E
U+70A4.0	fR/S	 86.5
U+70A4.0	fTotalStrokes	 9
U+70A4.0	fCangjie	FSHR
U+70A4.0	fFourCorner	97862
U+70A4.0	fHakka	 zhau5; zau5; zhau1
U+70A4.0	fCantonese	ciu1; ziu1 (also); ziu3
U+70A4.0	fMandarin	zhao4; zhao1; zhao3; zhuo2
U+70A4.0	fEnglish	 illumine; light up; reflect
U+70A5.0	fUTF8	炥
U+70A5.0	fCNS11643	3-2D58
U+70A5.0	fGB	3-4F66
U+70A5.0	fR/S	 86.5
U+70A5.0	fTotalStrokes	 9
U+70A5.0	fCangjie	FLLN
U+70A5.0	fFourCorner	95820
U+70A5.0	fCantonese	fat6
U+70A5.0	fMandarin	fu2; po1; fei4
U+70A6.0	fUTF8	炦
U+70A6.0	fCNS11643	4-2A26
U+70A6.0	fGB	3-4F5F
U+70A6.0	fR/S	 86.5
U+70A6.0	fTotalStrokes	 9
U+70A6.0	fCangjie	FIKK
U+70A6.0	fFourCorner	93840
U+70A6.0	fCantonese	bat6
U+70A6.0	fMandarin	ba2
U+70A7.0	fUTF8	炧
U+70A7.0	fCNS11643	3-2D65
U+70A7.0	fGB	3-4F63
U+70A7.0	fR/S	  86.5
U+70A7.0	fTotalStrokes	 9
U+70A7.0	fCangjie	FOPD
U+70A7.0	fFourCorner	98810
U+70A7.0	fHakka	 to3
U+70A7.0	fCantonese	se3; ce5 (also); ce2 (also)
U+70A7.0	fMandarin	duo4; xie4
U+70A7.0	fEnglish	ashes
U+70A8.0	fUTF8	炨
U+70A8.0	fCNS11643	3-2D62
U+70A8.0	fGB	5-5147
U+70A8.0	fR/S	  86.5
U+70A8.0	fTotalStrokes	 9
U+70A8.0	fCangjie	FJP
U+70A8.0	fFourCorner	93810
U+70A8.0	fHakka	 to3
U+70A8.0	fCantonese	se3
U+70A8.0	fMandarin	duo4
U+70A9.0	fUTF8	炩
U+70A9.0	fCNS11643	2-2B76
U+70A9.0	fBig5	CFB6
U+70A9.0	fGB	5-5145
U+70A9.0	fR/S	 86.5
U+70A9.0	fTotalStrokes	 9
U+70A9.0	fCangjie	FOII
U+70A9.0	fFourCorner	98827
U+70A9.0	fCantonese	ling6
U+70A9.0	fMandarin	ling4
U+70AA.0	fUTF8	炪
U+70AA.0	fCNS11643	3-2D5F
U+70AA.0	fGB	3-4F67
U+70AA.0	fR/S	 86.5
U+70AA.0	fTotalStrokes	 9
U+70AA.0	fCangjie	FUU
U+70AA.0	fFourCorner	92870
U+70AA.0	fCantonese	dyut3
U+70AA.0	fMandarin	zhuo2; chu4
U+70AB.0	fUTF8	炫
U+70AB.0	fCNS11643	1-5232
U+70AB.0	fBig5	ACAF
U+70AB.0	fGB	0-6C45
U+70AB.0	fR/S	 86.5
U+70AB.0	fTotalStrokes	 9
U+70AB.0	fCangjie	FYVI
U+70AB.0	fFourCorner	90832
U+70AB.0	fHakka	 hien2;  hien5; hen2; hien6
U+70AB.0	fCantonese	jyun6; jyun4 (also)
U+70AB.0	fMandarin	xuan4
U+70AB.0	fEnglish	[1] show off; flaunt; display [2] dazzling; bright; shining
U+70AC.0	fUTF8	炬
U+70AC.0	fCNS11643	1-5235
U+70AC.0	fBig5	ACB2
U+70AC.0	fGB	0-3E66
U+70AC.0	fR/S	 86.5
U+70AC.0	fTotalStrokes	 9
U+70AC.0	fCangjie	FSS
U+70AC.0	fFourCorner	91817
U+70AC.0	fHakka	  gi3;  ki3; ki1; gi5
U+70AC.0	fCantonese	geoi6
U+70AC.0	fMandarin	ju4
U+70AC.0	fEnglish	[1] torch [2] fire
U+70AD.0	fUTF8	炭
U+70AD.0	fCNS11643	1-5237
U+70AD.0	fBig5	ACB4
U+70AD.0	fGB	0-4C3F
U+70AD.0	fR/S	 86.5
U+70AD.0	fTotalStrokes	 9
U+70AD.0	fCangjie	UMF
U+70AD.0	fFourCorner	22289
U+70AD.0	fHakka	  tan5
U+70AD.0	fCantonese	taan3
U+70AD.0	fMandarin	tan4
U+70AD.0	fEnglish	[1] charcoal [2] coal [3] carbon (chemistry)
U+70AE.0	fUTF8	炮
U+70AE.0	fCNS11643	1-5239
U+70AE.0	fBig5	ACB6
U+70AE.0	fGB	0-455A
U+70AE.0	fR/S	 86.5
U+70AE.0	fTotalStrokes	 9
U+70AE.0	fCangjie	FPRU
U+70AE.0	fFourCorner	97812
U+70AE.0	fHakka	 pau5
U+70AE.0	fCantonese	paau4; baau1
U+70AE.0	fMandarin	pao4
U+70AE.0	fEnglish	 large gun; cannon; artillery, etc.
U+70AE.1	fUTF8	炮
U+70AE.1	fCNS11643	1-5239
U+70AE.1	fBig5	ACB6
U+70AE.1	fGB	0-455A
U+70AE.1	fR/S	 86.5
U+70AE.1	fTotalStrokes	 9
U+70AE.1	fCangjie	FPRU
U+70AE.1	fFourCorner	97812
U+70AE.1	fHakka	  pau5;  pau2; pau3
U+70AE.1	fCantonese	paau3; baau3
U+70AE.1	fMandarin	pao2
U+70AE.1	fEnglish	 refine medicinal herb
U+70AE.2	fUTF8	炮
U+70AE.2	fCNS11643	1-5239
U+70AE.2	fBig5	ACB6
U+70AE.2	fGB	0-455A
U+70AE.2	fR/S	 86.5
U+70AE.2	fTotalStrokes	 9
U+70AE.2	fCangjie	FPRU
U+70AE.2	fFourCorner	97812
U+70AE.2	fEnglish	[1] roast; bake [2] dry by heat
U+70AF.0	fUTF8	炯
U+70AF.0	fCNS11643	1-5236
U+70AF.0	fBig5	ACB3
U+70AF.0	fGB	0-3E3C
U+70AF.0	fR/S	 86.5
U+70AF.0	fTotalStrokes	 9
U+70AF.0	fCangjie	FBR
U+70AF.0	fFourCorner	97820
U+70AF.0	fHakka	 kwen3; gen3; gwen3
U+70AF.0	fCantonese	gwing2
U+70AF.0	fMandarin	jiong3
U+70AF.0	fEnglish	bright; brightness; clear
U+70B0.0	fUTF8	炰
U+70B0.0	fCNS11643	2-2B72
U+70B0.0	fBig5	CFB2
U+70B0.0	fGB	3-5060
U+70B0.0	fR/S	 86.5
U+70B0.0	fTotalStrokes	 9
U+70B0.0	fCangjie	PUF
U+70B0.0	fFourCorner	27337
U+70B0.0	fHakka	 pau2; bau1; pau5
U+70B0.0	fCantonese	paau4; fau2; paau3
U+70B0.0	fMandarin	pao2; fou3; pao4
U+70B0.0	fEnglish	 roast; broil bake
U+70B1.0	fUTF8	炱
U+70B1.0	fCNS11643	2-2B71
U+70B1.0	fBig5	CFB1
U+70B1.0	fGB	0-6C46
U+70B1.0	fR/S	 86.5
U+70B1.0	fTotalStrokes	 9
U+70B1.0	fCangjie	IRF
U+70B1.0	fFourCorner	23809
U+70B1.0	fHakka	  toi2
U+70B1.0	fCantonese	toi4
U+70B1.0	fMandarin	tai2
U+70B1.0	fEnglish	soot
U+70B2.0	fUTF8	炲
U+70B2.0	fCNS11643	3-2D64
U+70B2.0	fGB	5-5148
U+70B2.0	fR/S	  86.5
U+70B2.0	fTotalStrokes	 9
U+70B2.0	fCangjie	FIR
U+70B2.0	fFourCorner	93860
U+70B2.0	fCantonese	toi4
U+70B2.0	fMandarin	tai2
U+70B3.0	fUTF8	炳
U+70B3.0	fCNS11643	1-5234
U+70B3.0	fBig5	ACB1
U+70B3.0	fGB	0-317E
U+70B3.0	fR/S	 86.5
U+70B3.0	fTotalStrokes	 9
U+70B3.0	fCangjie	FMOB
U+70B3.0	fFourCorner	91827
U+70B3.0	fHakka	  biang3
U+70B3.0	fCantonese	bing2
U+70B3.0	fMandarin	bing3; beng3; bing4
U+70B3.0	fEnglish	bright; luminous
U+70B4.0	fUTF8	炴
U+70B4.0	fCNS11643	2-2B74
U+70B4.0	fBig5	CFB4
U+70B4.0	fGB	3-4F62
U+70B4.0	fR/S	 86.5
U+70B4.0	fTotalStrokes	 9
U+70B4.0	fCangjie	FLBK
U+70B4.0	fFourCorner	95830
U+70B4.0	fCantonese	joeng2
U+70B4.0	fMandarin	yang3; ying3
U+70B5.0	fUTF8	炵
U+70B5.0	fCNS11643	2-2B75
U+70B5.0	fBig5	CFB5
U+70B5.0	fGB	3-4F65
U+70B5.0	fR/S	 86.5
U+70B5.0	fTotalStrokes	 9
U+70B5.0	fCangjie	FHEY
U+70B5.0	fFourCorner	97833
U+70B5.0	fHakka	  tung1
U+70B5.0	fCantonese	tung1; dung1
U+70B5.0	fMandarin	tong1
U+70B5.0	fEnglish	blazing (as a fire)
U+70B6.0	fUTF8	炶
U+70B6.0	fCNS11643	3-2D63
U+70B6.0	fGB	5-5144
U+70B6.0	fR/S	  86.5
U+70B6.0	fTotalStrokes	 9
U+70B6.0	fCangjie	FYR
U+70B6.0	fFourCorner	91860
U+70B6.0	fCantonese	sim2
U+70B6.0	fMandarin	han1; shan4
U+70B7.0	fUTF8	炷
U+70B7.0	fCNS11643	2-2B6E
U+70B7.0	fBig5	CFAE
U+70B7.0	fGB	0-6C44
U+70B7.0	fR/S	 86.5
U+70B7.0	fTotalStrokes	 9
U+70B7.0	fCangjie	FYG
U+70B7.0	fFourCorner	90814
U+70B7.0	fHakka	 zhu5; zu5; zu1
U+70B7.0	fCantonese	zyu3
U+70B7.0	fMandarin	zhu4; zhu3
U+70B7.0	fEnglish	[1] wick (candle, lamp, etc.); [2] stick (incense, etc.) [3] bur; cauterize
U+70B8.0	fUTF8	炸
U+70B8.0	fCNS11643	1-5238
U+70B8.0	fBig5	ACB5
U+70B8.0	fGB	0-5528
U+70B8.0	fR/S	 86.5
U+70B8.0	fTotalStrokes	 9
U+70B8.0	fCangjie	FOS
U+70B8.0	fFourCorner	98811
U+70B8.0	fHakka	 cap7
U+70B8.0	fCantonese	zaa3
U+70B8.0	fMandarin	zha4
U+70B8.0	fEnglish	[1] explode; burst; bomb [2] get mad [3] disperse boisterously; flee in terror
U+70B8.1	fUTF8	炸
U+70B8.1	fCNS11643	1-5238
U+70B8.1	fBig5	ACB5
U+70B8.1	fGB	0-5528
U+70B8.1	fR/S	 86.5
U+70B8.1	fTotalStrokes	 9
U+70B8.1	fCangjie	FOS
U+70B8.1	fFourCorner	98811
U+70B8.1	fHakka	 za5
U+70B8.1	fMandarin	zha2
U+70B8.1	fEnglish	 fry in oil or fat; deep-fry
U+70B9.0	fUTF8	点
U+70B9.0	fCNS11643	3-2D68
U+70B9.0	fGB	0-3563
U+70B9.0	fR/S	 86.5
U+70B9.0	fTotalStrokes	 9
U+70B9.0	fCangjie	YRF
U+70B9.0	fFourCorner	21330
U+70B9.0	fHakka	  diam3
U+70B9.0	fCantonese	dim2
U+70B9.0	fMandarin	dian3; dian4; zhan1
U+70B9.0	fEnglish	simplified form of &#x9EDE;
U+70BA.0	fUTF8	為
U+70BA.0	fCNS11643	1-5233
U+70BA.0	fBig5	ACB0
U+70BA.0	fGB	1-7D66
U+70BA.0	fR/S	 86.5
U+70BA.0	fTotalStrokes	 9
U+70BA.0	fCangjie	IKNF
U+70BA.0	fFourCorner	20227
U+70BA.0	fHakka	 wui2; wui3; wi2; wi5; wui5
U+70BA.0	fCantonese	wai4; wai6 (also); wai2 (also)
U+70BA.0	fMandarin	wei2
U+70BA.0	fEnglish	[1] do; act [2] manage; handle; govern; exercise; administer [3] serve as [4] become [5] be
U+70BA.1	fUTF8	為
U+70BA.1	fCNS11643	1-5233
U+70BA.1	fBig5	ACB0
U+70BA.1	fGB	1-7D66
U+70BA.1	fR/S	 86.5
U+70BA.1	fTotalStrokes	 9
U+70BA.1	fCangjie	IKNF
U+70BA.1	fFourCorner	20227
U+70BA.1	fMandarin	wei4
U+70BA.1	fEnglish	for; for the good; for the sake of
U+70BB.0	fUTF8	炻
U+70BB.0	fCNS11643	3-2D5D
U+70BB.0	fGB	0-6C42
U+70BB.0	fR/S	  86.5
U+70BB.0	fTotalStrokes	 9
U+70BB.0	fCangjie	FMR
U+70BB.0	fFourCorner	91860
U+70BB.0	fCantonese	sek6
U+70BB.0	fMandarin	shi2
U+70BB.0	fEnglish	a kind of china
U+70BC.0	fUTF8	炼
U+70BC.0	fGB	0-4136
U+70BC.0	fR/S	 86.5
U+70BC.0	fTotalStrokes	 9
U+70BC.0	fCangjie	FKVC
U+70BC.0	fHakka	 lien5
U+70BC.0	fCantonese	lin6; lin2 (also)
U+70BC.0	fMandarin	lian4; lan4
U+70BC.0	fEnglish	simplified form of &#x7149;
U+70BD.0	fUTF8	炽
U+70BD.0	fGB	0-3363
U+70BD.0	fR/S	 86.5
U+70BD.0	fTotalStrokes	 9
U+70BD.0	fCangjie	FRC
U+70BD.0	fHakka	 zhi5; zhit7
U+70BD.0	fCantonese	ci3
U+70BD.0	fMandarin	chi4
U+70BD.0	fEnglish	simplified form of &#x71BE;
U+70BE.0	fUTF8	炾
U+70BE.0	fCNS11643	2-2B70
U+70BE.0	fBig5	CFB0
U+70BE.0	fGB	E-315F
U+70BE.0	fR/S	 86.5
U+70BE.0	fTotalStrokes	 9
U+70BE.0	fCangjie	FRHU
U+70BE.0	fFourCorner	96812
U+70BE.0	fCantonese	fong2
U+70BE.0	fMandarin	ping2; huang3
U+70BF.0	fUTF8	炿
U+70BF.0	fCNS11643	3-654D
U+70BF.0	fGB	E-3160
U+70BF.0	fR/S	 86.5
U+70BF.0	fTotalStrokes	 9
U+70BF.0	fCangjie	XFHBY
U+70BF.0	fFourCorner	9784
U+70BF.0	fCantonese	zau1
U+70C0.0	fUTF8	烀
U+70C0.0	fCNS11643	4-2A28
U+70C0.0	fGB	0-6C43
U+70C0.0	fR/S	  86.5
U+70C0.0	fTotalStrokes	 9
U+70C0.0	fCangjie	FHFD
U+70C0.0	fFourCorner	92840
U+70C0.0	fCantonese	fu1
U+70C0.0	fMandarin	hu1
U+70C0.0	fEnglish	simmer
U+70C1.0	fUTF8	烁
U+70C1.0	fGB	0-4B38
U+70C1.0	fR/S	 86.5
U+70C1.0	fTotalStrokes	 9
U+70C1.0	fCangjie	FHVD
U+70C1.0	fHakka	 shok7
U+70C1.0	fCantonese	soek3; lik1 (also)
U+70C1.0	fMandarin	shuo4; luo4; le4; yao4; yue4
U+70C1.0	fEnglish	simplified form of &#x720D;
U+70C2.0	fUTF8	烂
U+70C2.0	fGB	0-4043
U+70C2.0	fR/S	 86.5
U+70C2.0	fTotalStrokes	 9
U+70C2.0	fCangjie	FTMM
U+70C2.0	fHakka	 lan5
U+70C2.0	fCantonese	laan6; laan2 (also)
U+70C2.0	fMandarin	lan4; lan2
U+70C2.0	fEnglish	simplified form of &#x721B;
U+70C3.0	fUTF8	烃
U+70C3.0	fGB	0-4C7E
U+70C3.0	fR/S	 86.5
U+70C3.0	fTotalStrokes	 9
U+70C3.0	fCangjie	FNOM
U+70C3.0	fCantonese	ting1
U+70C3.0	fMandarin	jing3; ting1
U+70C3.0	fEnglish	simplified form of &#x70F4;
U+70C4.0	fUTF8	烄
U+70C4.0	fCNS11643	3-3228
U+70C4.0	fGB	3-4F71
U+70C4.0	fR/S	 86.6
U+70C4.0	fTotalStrokes	 10
U+70C4.0	fCangjie	FYCK
U+70C4.0	fFourCorner	90840
U+70C4.0	fCantonese	gaau2
U+70C4.0	fMandarin	jiao3; gao4; yao4
U+70C5.0	fUTF8	烅
U+70C5.0	fCNS11643	2-305A
U+70C5.0	fBig5	D277
U+70C5.0	fGB	5-5156
U+70C5.0	fR/S	 86.6
U+70C5.0	fTotalStrokes	 10
U+70C5.0	fCangjie	FHBT
U+70C5.0	fFourCorner	97810
U+70C5.0	fCantonese	gwik1; kwik1
U+70C5.0	fMandarin	xu4
U+70C6.0	fUTF8	烆
U+70C6.0	fCNS11643	2-305B
U+70C6.0	fBig5	D278
U+70C6.0	fGB	5-5157
U+70C6.0	fR/S	 86.6
U+70C6.0	fTotalStrokes	 10
U+70C6.0	fCangjie	FHON
U+70C6.0	fFourCorner	91821
U+70C6.0	fCantonese	hang4
U+70C6.0	fMandarin	xing2
U+70C7.0	fUTF8	烇
U+70C7.0	fCNS11643	2-305C
U+70C7.0	fBig5	D279
U+70C7.0	fGB	3-4F6E
U+70C7.0	fR/S	 86.6
U+70C7.0	fTotalStrokes	 10
U+70C7.0	fCangjie	FOMG
U+70C7.0	fFourCorner	98814
U+70C7.0	fCantonese	cyun2; cyun3
U+70C7.0	fMandarin	quan4; quan3
U+70C8.0	fUTF8	烈
U+70C8.0	fCNS11643	1-5653
U+70C8.0	fBig5	AF50
U+70C8.0	fGB	0-4152
U+70C8.0	fR/S	 86.6
U+70C8.0	fTotalStrokes	 10
U+70C8.0	fCangjie	MNF
U+70C8.0	fFourCorner	12330
U+70C8.0	fHakka	  liet8; let8; liat8; liap8
U+70C8.0	fCantonese	lit6
U+70C8.0	fMandarin	lie4
U+70C8.0	fEnglish	[1] fiery; acute; violent; vehement; ardent; fierce; strong [2] honest and virtuous; just and straightforward; chaste [3] merits; achievements [4] a Chinese family name
U+70C9.0	fUTF8	烉
U+70C9.0	fCNS11643	4-2D75
U+70C9.0	fGB	E-3161
U+70C9.0	fR/S	 86.6
U+70C9.0	fTotalStrokes	 10
U+70C9.0	fCangjie	NBF
U+70C9.0	fFourCorner	27800
U+70C9.0	fCantonese	wun6
U+70C9.0	fMandarin	huan4
U+70C9.0	fEnglish	shining; brilliant; lustrous
U+70CA.0	fUTF8	烊
U+70CA.0	fCNS11643	1-564F
U+70CA.0	fBig5	AF4C
U+70CA.0	fGB	0-6C48
U+70CA.0	fR/S	 86.6
U+70CA.0	fTotalStrokes	 10
U+70CA.0	fCangjie	FTQ
U+70CA.0	fFourCorner	98851
U+70CA.0	fHakka	  jong2; rong2; jong5
U+70CA.0	fCantonese	joeng4; joeng2; joeng6
U+70CA.0	fMandarin	yang2; yang4
U+70CA.0	fEnglish	 smelt; melt
U+70CB.0	fUTF8	烋
U+70CB.0	fCNS11643	2-3051
U+70CB.0	fBig5	D26E
U+70CB.0	fGB	5-5257
U+70CB.0	fR/S	 86.6
U+70CB.0	fTotalStrokes	 10
U+70CB.0	fCangjie	ODF
U+70CB.0	fFourCorner	24339
U+70CB.0	fHakka	  hau1;  hiu1
U+70CB.0	fCantonese	haau1; jau1 (also)
U+70CB.0	fMandarin	xiao1; xiu1
U+70CB.0	fEnglish	boast
U+70CC.0	fUTF8	烌
U+70CC.0	fCNS11643	3-322A
U+70CC.0	fGB	5-5155
U+70CC.0	fR/S	  86.6
U+70CC.0	fTotalStrokes	 10
U+70CC.0	fCangjie	FOD
U+70CC.0	fFourCorner	94890
U+70CC.0	fCantonese	jau1
U+70CC.0	fMandarin	xiu1
U+70CD.0	fUTF8	烍
U+70CD.0	fCNS11643	2-3059
U+70CD.0	fBig5	D276
U+70CD.0	fGB	5-5153
U+70CD.0	fR/S	 86.6
U+70CD.0	fTotalStrokes	 10
U+70CD.0	fCangjie	FHGU
U+70CD.0	fFourCorner	94811
U+70CD.0	fCantonese	sin2
U+70CD.0	fMandarin	xian3
U+70CE.0	fUTF8	烎
U+70CE.0	fCNS11643	2-305E
U+70CE.0	fBig5	D27B
U+70CE.0	fGB	E-3162
U+70CE.0	fR/S	 86.6
U+70CE.0	fTotalStrokes	 10
U+70CE.0	fCangjie	MJF
U+70CE.0	fFourCorner	11809
U+70CE.0	fCantonese	cim4; jam4
U+70CE.0	fMandarin	yin2
U+70CF.0	fUTF8	烏
U+70CF.0	fCNS11643	1-5654
U+70CF.0	fBig5	AF51
U+70CF.0	fGB	1-4E5A
U+70CF.0	fR/S	 86.6
U+70CF.0	fTotalStrokes	 10
U+70CF.0	fCangjie	HRYF
U+70CF.0	fFourCorner	27327
U+70CF.0	fHakka	  wu1; vu1
U+70CF.0	fCantonese	wu1
U+70CF.0	fMandarin	wu1; ya1; wu4; yan1
U+70CF.0	fEnglish	[1] crow; rook; raven [2] black; dark color [3] how; what; when [4] Alas! [5] sun [6] a Chinese family name
U+70D0.0	fUTF8	烐
U+70D0.0	fCNS11643	3-322B
U+70D0.0	fGB	5-5158
U+70D0.0	fR/S	 86.6
U+70D0.0	fTotalStrokes	 10
U+70D0.0	fCangjie	FHBY
U+70D0.0	fFourCorner	97840
U+70D0.0	fCantonese	zau1
U+70D0.0	fMandarin	zhou1
U+70D1.0	fUTF8	烑
U+70D1.0	fCNS11643	2-304F
U+70D1.0	fBig5	D26C
U+70D1.0	fGB	5-5159
U+70D1.0	fR/S	 86.6
U+70D1.0	fTotalStrokes	 10
U+70D1.0	fCangjie	FLMO
U+70D1.0	fFourCorner	92813
U+70D1.0	fCantonese	jiu4
U+70D1.0	fMandarin	yao2
U+70D2.0	fUTF8	烒
U+70D2.0	fCNS11643	2-3055
U+70D2.0	fBig5	D272
U+70D2.0	fGB	5-514A
U+70D2.0	fR/S	 86.6
U+70D2.0	fTotalStrokes	 10
U+70D2.0	fCangjie	FIPM
U+70D2.0	fFourCorner	93850
U+70D2.0	fCantonese	sik1
U+70D2.0	fMandarin	shi4
U+70D3.0	fUTF8	烓
U+70D3.0	fCNS11643	2-304E
U+70D3.0	fBig5	D26B
U+70D3.0	fGB	5-514B
U+70D3.0	fR/S	 86.6
U+70D3.0	fTotalStrokes	 10
U+70D3.0	fCangjie	FGG
U+70D3.0	fFourCorner	94814
U+70D3.0	fCantonese	gwing1; gwing2
U+70D3.0	fMandarin	wei1; qing3; jiong3; kui3; hui4; gui4; jue2
U+70D3.0	fEnglish	stove
U+70D4.0	fUTF8	烔
U+70D4.0	fCNS11643	2-3058
U+70D4.0	fBig5	D275
U+70D4.0	fGB	5-5151
U+70D4.0	fR/S	 86.6
U+70D4.0	fTotalStrokes	 10
U+70D4.0	fCangjie	FBMR
U+70D4.0	fFourCorner	97820
U+70D4.0	fHakka	 tung2
U+70D4.0	fCantonese	cung4 (also); tung4
U+70D4.0	fMandarin	tong2; dong4
U+70D4.0	fEnglish	heat
U+70D5.0	fUTF8	烕
U+70D5.0	fCNS11643	3-3231
U+70D5.0	fGB	3-4632
U+70D5.0	fR/S	  86.6
U+70D5.0	fTotalStrokes	 10
U+70D5.0	fCangjie	IHMF
U+70D5.0	fFourCorner	53200
U+70D5.0	fHakka	 met8; miet8
U+70D5.0	fCantonese	hyut3; kyut3 (also); mit6 (also)
U+70D5.0	fMandarin	xue4; mie4
U+70D5.0	fEnglish	 destroy exterminate to extinguish
U+70D6.0	fUTF8	烖
U+70D6.0	fCNS11643	3-322F
U+70D6.0	fGB	E-3163
U+70D6.0	fR/S	  86.6
U+70D6.0	fTotalStrokes	 10
U+70D6.0	fCangjie	JIF
U+70D6.0	fFourCorner	43850
U+70D6.0	fHakka	 zai1
U+70D6.0	fCantonese	zoi1
U+70D6.0	fMandarin	zai1
U+70D6.0	fEnglish	 calamities from Heaven (as floods; famines; pestilence; etc.) misery
U+70D7.0	fUTF8	烗
U+70D7.0	fCNS11643	2-3054
U+70D7.0	fBig5	D271
U+70D7.0	fGB	5-515A
U+70D7.0	fR/S	 86.6
U+70D7.0	fTotalStrokes	 10
U+70D7.0	fCangjie	FYVO
U+70D7.0	fFourCorner	90882
U+70D7.0	fCantonese	kaai3; hoi1
U+70D7.0	fMandarin	kai4; kai1
U+70D8.0	fUTF8	烘
U+70D8.0	fCNS11643	1-5650
U+70D8.0	fBig5	AF4D
U+70D8.0	fGB	0-3A66
U+70D8.0	fR/S	 86.6
U+70D8.0	fTotalStrokes	 10
U+70D8.0	fCangjie	FTC
U+70D8.0	fFourCorner	94881
U+70D8.0	fHakka	  fung2; fung1; fung6; kung1; fung5; fung3
U+70D8.0	fCantonese	hung1 (also); hung3; hong3 (also); hung4 (also)
U+70D8.0	fMandarin	hong1; gong1; hong4
U+70D8.0	fEnglish	[1] bake; roast [2] dry or warm near a fire
U+70D9.0	fUTF8	烙
U+70D9.0	fCNS11643	1-5652
U+70D9.0	fBig5	AF4F
U+70D9.0	fGB	0-4053
U+70D9.0	fR/S	 86.6
U+70D9.0	fTotalStrokes	 10
U+70D9.0	fCangjie	FHER
U+70D9.0	fFourCorner	97864
U+70D9.0	fHakka	  lok8; lok7
U+70D9.0	fCantonese	lok3; lok6
U+70D9.0	fMandarin	luo4; lao4; ge2
U+70D9.0	fEnglish	[1] brand; iron [2] burn [3] bake in a pan
U+70DA.0	fUTF8	烚
U+70DA.0	fCNS11643	2-305D
U+70DA.0	fBig5	D27A
U+70DA.0	fGB	3-4F6F
U+70DA.0	fR/S	 86.6
U+70DA.0	fTotalStrokes	 10
U+70DA.0	fCangjie	FOMR
U+70DA.0	fFourCorner	98861
U+70DA.0	fHakka	  giap7; hip7
U+70DA.0	fCantonese	hap6; saap3
U+70DA.0	fMandarin	xia2
U+70DA.0	fEnglish	[1] fiery; blazing [2] rail at
U+70DB.0	fUTF8	烛
U+70DB.0	fCNS11643	3-3229
U+70DB.0	fGB	0-5672
U+70DB.0	fR/S	  86.6
U+70DB.0	fTotalStrokes	 10
U+70DB.0	fCangjie	FLMI
U+70DB.0	fFourCorner	95830
U+70DB.0	fHakka	 chuk7; zuk7; tung1
U+70DB.0	fCantonese	zuk1
U+70DB.0	fMandarin	chong2; zhu2; zhu4
U+70DB.0	fEnglish	simplified form of &#x71ED;
U+70DC.0	fUTF8	烜
U+70DC.0	fCNS11643	2-304D
U+70DC.0	fBig5	D26A
U+70DC.0	fGB	3-4F69
U+70DC.0	fR/S	 86.6
U+70DC.0	fTotalStrokes	 10
U+70DC.0	fCangjie	FMAM
U+70DC.0	fFourCorner	91816
U+70DC.0	fHakka	  sien1
U+70DC.0	fCantonese	hyun1 (also); hyun2
U+70DC.0	fMandarin	xuan3; hui3; guan4; xuan1
U+70DC.0	fEnglish	 light of the sun dry in the sun
U+70DD.0	fUTF8	烝
U+70DD.0	fCNS11643	2-3050
U+70DD.0	fBig5	D26D
U+70DD.0	fGB	3-5061
U+70DD.0	fR/S	 86.6
U+70DD.0	fTotalStrokes	 10
U+70DD.0	fCangjie	NEMF
U+70DD.0	fFourCorner	17331
U+70DD.0	fHakka	  zhin1; zihn1; zin1
U+70DD.0	fCantonese	zing1
U+70DD.0	fMandarin	zheng1; zheng4; zheng3
U+70DD.0	fEnglish	[1] rise (steam) [2] many; numerous
U+70DE.0	fUTF8	烞
U+70DE.0	fCNS11643	2-3056
U+70DE.0	fBig5	D273
U+70DE.0	fGB	5-514D
U+70DE.0	fR/S	 86.6
U+70DE.0	fTotalStrokes	 10
U+70DE.0	fCangjie	FDY
U+70DE.0	fFourCorner	93800
U+70DE.0	fCantonese	pok3
U+70DE.0	fMandarin	po4
U+70DF.0	fUTF8	烟
U+70DF.0	fCNS11643	3-322D
U+70DF.0	fGB	0-514C
U+70DF.0	fR/S	  86.6
U+70DF.0	fTotalStrokes	 10
U+70DF.0	fCangjie	FWK
U+70DF.0	fFourCorner	96800
U+70DF.0	fHakka	 jen1; jan1; ren1
U+70DF.0	fCantonese	jin1; jan1
U+70DF.0	fMandarin	yan1; yin1
U+70DF.0	fEnglish	 smoke; soot tobacco; cigarettes
U+70E0.0	fUTF8	烠
U+70E0.0	fCNS11643	2-3057
U+70E0.0	fBig5	D274
U+70E0.0	fGB	3-4F6A
U+70E0.0	fR/S	 86.6
U+70E0.0	fTotalStrokes	 10
U+70E0.0	fCangjie	FKB
U+70E0.0	fFourCorner	94827
U+70E0.0	fCantonese	wui4
U+70E0.0	fMandarin	hui3; hui4; ai3; hui2; dui1
U+70E1.0	fUTF8	烡
U+70E1.0	fCNS11643	2-305F
U+70E1.0	fBig5	D27C
U+70E1.0	fGB	E-3164
U+70E1.0	fR/S	 86.6
U+70E1.0	fTotalStrokes	 10
U+70E1.0	fCangjie	FTC
U+70E1.0	fFourCorner	90801
U+70E1.0	fCantonese	gwong1; gong1 (also)
U+70E1.0	fMandarin	guang1
U+70E2.0	fUTF8	烢
U+70E2.0	fCNS11643	2-3053
U+70E2.0	fBig5	D270
U+70E2.0	fGB	5-515B
U+70E2.0	fR/S	 86.6
U+70E2.0	fTotalStrokes	 10
U+70E2.0	fCangjie	FJHP
U+70E2.0	fFourCorner	93814
U+70E2.0	fCantonese	caak3; zaak6
U+70E2.0	fMandarin	zhe4; zhai2
U+70E3.0	fUTF8	烣
U+70E3.0	fCNS11643	3-322C
U+70E3.0	fGB	5-5150
U+70E3.0	fR/S	 86.6
U+70E3.0	fTotalStrokes	 10
U+70E3.0	fCangjie	FKF
U+70E3.0	fFourCorner	91889
U+70E3.0	fCantonese	fui1
U+70E3.0	fMandarin	hui1
U+70E4.0	fUTF8	烤
U+70E4.0	fCNS11643	1-5651
U+70E4.0	fBig5	AF4E
U+70E4.0	fGB	0-3F3E
U+70E4.0	fR/S	 86.6
U+70E4.0	fTotalStrokes	 10
U+70E4.0	fCangjie	FJKS
U+70E4.0	fFourCorner	94827
U+70E4.0	fHakka	  kau3; ko1
U+70E4.0	fCantonese	haau1; haau2 (also)
U+70E4.0	fMandarin	kao3
U+70E4.0	fEnglish	[1] bake; roast; toast [2] warm by a fire [3] scorching
U+70E5.0	fUTF8	烥
U+70E5.0	fCNS11643	3-654F
U+70E5.0	fGB	E-3165
U+70E5.0	fR/S	 86.6
U+70E5.0	fTotalStrokes	 10
U+70E5.0	fCangjie	FSLL
U+70E5.0	fFourCorner	9181
U+70E5.0	fCantonese	geoi6
U+70E6.0	fUTF8	烦
U+70E6.0	fGB	0-3733
U+70E6.0	fR/S	 86.6
U+70E6.0	fTotalStrokes	 10
U+70E6.0	fCangjie	FMBO
U+70E6.0	fHakka	 fan2
U+70E6.0	fCantonese	faan4
U+70E6.0	fMandarin	fan2
U+70E6.0	fEnglish	simplified form of &#x7169;
U+70E6.1	fUTF8	烦
U+70E6.1	fGB	0-3733
U+70E6.1	fR/S	 86.6
U+70E6.1	fTotalStrokes	 10
U+70E6.1	fCangjie	FMBO
U+70E6.1	fMandarin	fan5
U+70E7.0	fUTF8	烧
U+70E7.0	fGB	0-4955
U+70E7.0	fR/S	 86.6
U+70E7.0	fTotalStrokes	 10
U+70E7.0	fCangjie	FJPU
U+70E7.0	fHakka	 shau1; s'eu1
U+70E7.0	fCantonese	siu1
U+70E7.0	fMandarin	shao1; shao4
U+70E7.0	fEnglish	simplified form of &#x71D2;
U+70E8.0	fUTF8	烨
U+70E8.0	fGB	0-6C47
U+70E8.0	fR/S	 86.6
U+70E8.0	fTotalStrokes	 10
U+70E8.0	fCangjie	FOPJ
U+70E8.0	fCantonese	jip6
U+70E8.0	fMandarin	ye4; yi4
U+70E8.0	fEnglish	simplified form of &#x71C1;
U+70E9.0	fUTF8	烩
U+70E9.0	fGB	0-3B62
U+70E9.0	fR/S	 86.6
U+70E9.0	fTotalStrokes	 10
U+70E9.0	fCangjie	FOMI
U+70E9.0	fCantonese	wui6
U+70E9.0	fMandarin	hui4
U+70E9.0	fEnglish	simplified form of &#x71F4;
U+70EA.0	fUTF8	烪
U+70EA.0	fCNS11643	3-654E
U+70EA.0	fGB	E-3166
U+70EA.0	fR/S	 86.6
U+70EA.0	fTotalStrokes	 10
U+70EA.0	fCangjie	FTK
U+70EA.0	fFourCorner	9883
U+70EA.0	fCantonese	gwaan1
U+70EB.0	fUTF8	烫
U+70EB.0	fGB	0-4C4C
U+70EB.0	fR/S	 86.6
U+70EB.0	fTotalStrokes	 10
U+70EB.0	fCangjie	XEHF
U+70EB.0	fHakka	 tong5
U+70EB.0	fCantonese	tong3
U+70EB.0	fMandarin	tang4; tang2
U+70EB.0	fEnglish	simplified form of &#x71D9;
U+70EC.0	fUTF8	烬
U+70EC.0	fGB	0-3D7D
U+70EC.0	fR/S	 86.6
U+70EC.0	fTotalStrokes	 10
U+70EC.0	fCangjie	FSOY
U+70EC.0	fHakka	 zin5
U+70EC.0	fCantonese	zeon6; zeon2 (also)
U+70EC.0	fMandarin	jin4
U+70EC.0	fEnglish	simplified form of &#x71FC;
U+70ED.0	fUTF8	热
U+70ED.0	fGB	0-4848
U+70ED.0	fR/S	 86.6
U+70ED.0	fTotalStrokes	 10
U+70ED.0	fCangjie	QIF
U+70ED.0	fHakka	 ngiet8
U+70ED.0	fCantonese	jit6; jit2 (also)
U+70ED.0	fMandarin	re4
U+70ED.0	fEnglish	simplified form of &#x71B1;
U+70EE.0	fUTF8	烮
U+70EE.0	fCNS11643	3-3230
U+70EE.0	fGB	E-3167
U+70EE.0	fR/S	  86.6
U+70EE.0	fTotalStrokes	 10
U+70EE.0	fCangjie	MNF
U+70EE.0	fFourCorner	12800
U+70EE.0	fCantonese	lit6
U+70EF.0	fUTF8	烯
U+70EF.0	fCNS11643	1-5B72
U+70EF.0	fBig5	B26D
U+70EF.0	fGB	0-4F29
U+70EF.0	fR/S	 86.7
U+70EF.0	fTotalStrokes	 11
U+70EF.0	fCangjie	FKKB
U+70EF.0	fFourCorner	94827
U+70EF.0	fHakka	 hi1
U+70EF.0	fCantonese	hei1
U+70EF.0	fMandarin	xi1
U+70EF.0	fEnglish	[1] the color of fire [2] alkene
U+70F0.0	fUTF8	烰
U+70F0.0	fCNS11643	2-3671
U+70F0.0	fBig5	D64E
U+70F0.0	fGB	5-5165
U+70F0.0	fR/S	 86.7
U+70F0.0	fTotalStrokes	 11
U+70F0.0	fCangjie	FBND
U+70F0.0	fFourCorner	92847
U+70F0.0	fHakka	  feu2
U+70F0.0	fCantonese	fau4
U+70F0.0	fMandarin	fu2
U+70F0.0	fEnglish	steam
U+70F1.0	fUTF8	烱
U+70F1.0	fCNS11643	3-3730
U+70F1.0	fGB	8-2E58
U+70F1.0	fR/S	 86.7
U+70F1.0	fTotalStrokes	 11
U+70F1.0	fCangjie	FBCR
U+70F1.0	fFourCorner	97820
U+70F1.0	fHakka	 kwen3; gen3
U+70F1.0	fCantonese	gwing2
U+70F1.0	fMandarin	jiong3
U+70F1.0	fEnglish	 bright; brilliant; clear hot
U+70F2.0	fUTF8	烲
U+70F2.0	fCNS11643	3-3732
U+70F2.0	fGB	3-4F76
U+70F2.0	fR/S	  86.7
U+70F2.0	fTotalStrokes	 11
U+70F2.0	fCangjie	QLF
U+70F2.0	fFourCorner	52800
U+70F2.0	fCantonese	hit3
U+70F2.0	fMandarin	che4
U+70F3.0	fUTF8	烳
U+70F3.0	fCNS11643	2-3673
U+70F3.0	fBig5	D650
U+70F3.0	fGB	3-4F74
U+70F3.0	fR/S	 86.7
U+70F3.0	fTotalStrokes	 11
U+70F3.0	fCangjie	FIJB
U+70F3.0	fFourCorner	93827
U+70F3.0	fHakka	  pu2
U+70F3.0	fCantonese	pou2; pou4
U+70F3.0	fMandarin	pu3
U+70F3.0	fEnglish	 boil; cook; steam
U+70F4.0	fUTF8	烴
U+70F4.0	fCNS11643	2-366F
U+70F4.0	fBig5	D64C
U+70F4.0	fGB	1-4C7E
U+70F4.0	fR/S	 86.7
U+70F4.0	fTotalStrokes	 11
U+70F4.0	fCangjie	FMVM
U+70F4.0	fFourCorner	91811
U+70F4.0	fHakka	 gin3
U+70F4.0	fCantonese	ting1; ging2
U+70F4.0	fMandarin	jing3; ting1
U+70F4.0	fEnglish	hydrocarbon
U+70F5.0	fUTF8	烵
U+70F5.0	fCNS11643	3-372F
U+70F5.0	fGB	5-514C
U+70F5.0	fR/S	  86.7
U+70F5.0	fTotalStrokes	 11
U+70F5.0	fCangjie	FTPI
U+70F5.0	fFourCorner	94820
U+70F5.0	fCantonese	zoek3
U+70F5.0	fMandarin	zhuo2
U+70F6.0	fUTF8	烶
U+70F6.0	fCNS11643	2-367B
U+70F6.0	fBig5	D658
U+70F6.0	fGB	5-5154
U+70F6.0	fR/S	 86.7
U+70F6.0	fTotalStrokes	 11
U+70F6.0	fCangjie	FNKG
U+70F6.0	fFourCorner	92841
U+70F6.0	fCantonese	ting5
U+70F6.0	fMandarin	ting3
U+70F7.0	fUTF8	烷
U+70F7.0	fCNS11643	2-366D
U+70F7.0	fBig5	D64A
U+70F7.0	fGB	0-4D69
U+70F7.0	fR/S	 86.7
U+70F7.0	fTotalStrokes	 11
U+70F7.0	fCangjie	FJMU
U+70F7.0	fFourCorner	93811
U+70F7.0	fHakka	 wan2
U+70F7.0	fCantonese	jyun2; jyun4 (also)
U+70F7.0	fMandarin	wan2
U+70F7.0	fEnglish	[1] fire [2] alkane
U+70F8.0	fUTF8	烸
U+70F8.0	fCNS11643	2-367A
U+70F8.0	fBig5	D657
U+70F8.0	fGB	5-5162
U+70F8.0	fR/S	 86.7
U+70F8.0	fTotalStrokes	 11
U+70F8.0	fCangjie	FOWY
U+70F8.0	fFourCorner	98857
U+70F8.0	fCantonese	hoi2
U+70F8.0	fMandarin	hai3
U+70F9.0	fUTF8	烹
U+70F9.0	fCNS11643	1-5B6E
U+70F9.0	fBig5	B269
U+70F9.0	fGB	0-456B
U+70F9.0	fR/S	 86.7
U+70F9.0	fTotalStrokes	 11
U+70F9.0	fCangjie	YRNF
U+70F9.0	fFourCorner	00332
U+70F9.0	fHakka	 pen1; pen3; pang1
U+70F9.0	fCantonese	paang1
U+70F9.0	fMandarin	peng1
U+70F9.0	fEnglish	[1] boil; cook; decoct [2] add bean sauce and dressing after frying (cooking) [3] frighten (away) (slang)
U+70FA.0	fUTF8	烺
U+70FA.0	fCNS11643	2-366B
U+70FA.0	fBig5	D648
U+70FA.0	fGB	3-4F7D
U+70FA.0	fR/S	 86.7
U+70FA.0	fTotalStrokes	 11
U+70FA.0	fCangjie	FIAV
U+70FA.0	fFourCorner	93832
U+70FA.0	fHakka	 long5; lang5; long6; lang6
U+70FA.0	fCantonese	long5
U+70FA.0	fMandarin	lang3
U+70FA.0	fEnglish	bright (fire)
U+70FB.0	fUTF8	烻
U+70FB.0	fCNS11643	2-3D5F
U+70FB.0	fBig5	DA5B
U+70FB.0	fGB	3-4F6D
U+70FB.0	fR/S	 86.7
U+70FB.0	fTotalStrokes	 11
U+70FB.0	fCangjie	FNKM
U+70FB.0	fFourCorner	92841
U+70FB.0	fCantonese	jin3
U+70FB.0	fMandarin	shan1; chan1; shan2; yan2; yan4
U+70FC.0	fUTF8	烼
U+70FC.0	fCNS11643	2-3675
U+70FC.0	fBig5	D652
U+70FC.0	fGB	3-4F75
U+70FC.0	fR/S	 86.7
U+70FC.0	fTotalStrokes	 11
U+70FC.0	fCangjie	FMSO
U+70FC.0	fFourCorner	91832
U+70FC.0	fCantonese	fat1
U+70FC.0	fMandarin	hu1
U+70FD.0	fUTF8	烽
U+70FD.0	fCNS11643	1-5B71
U+70FD.0	fBig5	B26C
U+70FD.0	fGB	0-3769
U+70FD.0	fR/S	 86.7
U+70FD.0	fTotalStrokes	 11
U+70FD.0	fCangjie	FHEJ
U+70FD.0	fFourCorner	97854
U+70FD.0	fHakka	  fung1
U+70FD.0	fCantonese	fung1
U+70FD.0	fMandarin	feng1; hong2
U+70FD.0	fEnglish	 signal fire; a tall structure (on a city wall, etc.) where fire was made to signal enemy invasion or presence of bandits
U+70FE.0	fUTF8	烾
U+70FE.0	fCNS11643	3-3735
U+70FE.0	fGB	E-3168
U+70FE.0	fR/S	 32.8; 86.7; 86.8
U+70FE.0	fTotalStrokes	 11
U+70FE.0	fCangjie	FFG
U+70FE.0	fFourCorner	90100
U+70FE.0	fCantonese	cek3
U+70FE.0	fMandarin	chi4
U+70FF.0	fUTF8	烿
U+70FF.0	fCNS11643	2-3676
U+70FF.0	fBig5	D653
U+70FF.0	fGB	5-5166
U+70FF.0	fR/S	 86.7
U+70FF.0	fTotalStrokes	 11
U+70FF.0	fCangjie	FBYH
U+70FF.0	fFourCorner	92822
U+70FF.0	fCantonese	jung4
U+70FF.0	fMandarin	rong2
U+7100.0	fUTF8	焀
U+7100.0	fCNS11643	2-3679
U+7100.0	fBig5	D656
U+7100.0	fGB	5-5164
U+7100.0	fR/S	 86.7
U+7100.0	fTotalStrokes	 11
U+7100.0	fCangjie	FCOR
U+7100.0	fFourCorner	98868
U+7100.0	fHakka	 fuk8
U+7100.0	fCantonese	huk6; fuk6
U+7100.0	fMandarin	hu2
U+7100.0	fEnglish	flame; blaze of a fire
U+7101.0	fUTF8	焁
U+7101.0	fCNS11643	3-372E
U+7101.0	fGB	3-4F7A
U+7101.0	fR/S	 86.7
U+7101.0	fTotalStrokes	 11
U+7101.0	fCangjie	ROF
U+7101.0	fFourCorner	67800
U+7101.0	fCantonese	hei1
U+7101.0	fMandarin	xi1
U+7102.0	fUTF8	焂
U+7102.0	fCNS11643	2-367D
U+7102.0	fBig5	D65A
U+7102.0	fGB	5-5163
U+7102.0	fR/S	 86.7
U+7102.0	fTotalStrokes	 11
U+7102.0	fCangjie	OKF
U+7102.0	fFourCorner	28809
U+7102.0	fCantonese	suk1
U+7102.0	fMandarin	shu2
U+7103.0	fUTF8	焃
U+7103.0	fCNS11643	3-372C
U+7103.0	fGB	5-515E
U+7103.0	fR/S	 86.7
U+7103.0	fTotalStrokes	 11
U+7103.0	fCangjie	FGLC
U+7103.0	fFourCorner	94830
U+7103.0	fCantonese	haak1
U+7103.0	fMandarin	he4; xi4; huo4
U+7104.0	fUTF8	焄
U+7104.0	fCNS11643	2-3672
U+7104.0	fBig5	D64F
U+7104.0	fGB	5-525A
U+7104.0	fR/S	 86.7
U+7104.0	fTotalStrokes	 11
U+7104.0	fCangjie	SRF
U+7104.0	fFourCorner	17336
U+7104.0	fHakka	  hiun1
U+7104.0	fCantonese	fan1
U+7104.0	fMandarin	xun1; hun1
U+7104.0	fEnglish	[1] rising flames or fumes [2] aroma
U+7105.0	fUTF8	焅
U+7105.0	fCNS11643	4-3272
U+7105.0	fGB	5-5161
U+7105.0	fR/S	 86.7
U+7105.0	fTotalStrokes	 11
U+7105.0	fCangjie	FHGR
U+7105.0	fFourCorner	94860
U+7105.0	fHakka	  kau5; kuk7; k'wuk7
U+7105.0	fCantonese	guk6
U+7105.0	fMandarin	ku4; kao4
U+7105.0	fEnglish	[1] hot air [2] dry burning atmosphere
U+7106.0	fUTF8	焆
U+7106.0	fCNS11643	2-3677
U+7106.0	fBig5	D654
U+7106.0	fGB	3-4F78
U+7106.0	fR/S	 86.7
U+7106.0	fTotalStrokes	 11
U+7106.0	fCangjie	FRB
U+7106.0	fFourCorner	96827
U+7106.0	fCantonese	gyun1
U+7106.0	fMandarin	jue2; juan1; yuan1
U+7107.0	fUTF8	焇
U+7107.0	fCNS11643	3-372D
U+7107.0	fGB	3-4F77
U+7107.0	fR/S	 86.7
U+7107.0	fTotalStrokes	 11
U+7107.0	fCangjie	FFB
U+7107.0	fFourCorner	99820
U+7107.0	fHakka	 siau1
U+7107.0	fCantonese	siu1
U+7107.0	fMandarin	xiao1; shao1
U+7108.0	fUTF8	焈
U+7108.0	fCNS11643	3-3731
U+7108.0	fGB	E-3169
U+7108.0	fR/S	 86.7
U+7108.0	fTotalStrokes	 11
U+7108.0	fCangjie	HUF
U+7108.0	fFourCorner	37809
U+7108.0	fCantonese	hei1
U+7108.0	fMandarin	xi1; yi2
U+7109.0	fUTF8	焉
U+7109.0	fCNS11643	1-5B6F
U+7109.0	fBig5	B26A
U+7109.0	fGB	0-5149
U+7109.0	fR/S	 86.7
U+7109.0	fTotalStrokes	 11
U+7109.0	fCangjie	MYLF
U+7109.0	fFourCorner	10327
U+7109.0	fHakka	  jen2; jan1; ren2; jan3; jen1; jan2
U+7109.0	fCantonese	jin1; jin4
U+7109.0	fMandarin	yan1; yi2; yan2
U+7109.0	fEnglish	[1] it (pronoun) [2] there; then (adverb) [3] how? why? when, etc. (interrogative) [4] and so; so that (conjunctive) [5] a final particle indicating numerous senses
U+710A.0	fUTF8	焊
U+710A.0	fCNS11643	1-5B70
U+710A.0	fBig5	B26B
U+710A.0	fGB	0-3A38
U+710A.0	fR/S	 86.7
U+710A.0	fTotalStrokes	 11
U+710A.0	fCangjie	FAMJ
U+710A.0	fFourCorner	96841
U+710A.0	fHakka	 hon5; hon3
U+710A.0	fCantonese	hon6; hon2
U+710A.0	fMandarin	han3; han4
U+710A.0	fEnglish	weld; solder
U+710B.0	fUTF8	焋
U+710B.0	fCNS11643	2-367C
U+710B.0	fBig5	D659
U+710B.0	fGB	E-316A
U+710B.0	fR/S	 86.7
U+710B.0	fTotalStrokes	 11
U+710B.0	fCangjie	VGF
U+710B.0	fFourCorner	24809
U+710B.0	fCantonese	zong3
U+710B.0	fMandarin	zhuang4
U+710C.0	fUTF8	焌
U+710C.0	fCNS11643	2-3670
U+710C.0	fBig5	D64D
U+710C.0	fGB	3-5022
U+710C.0	fR/S	 86.7
U+710C.0	fTotalStrokes	 11
U+710C.0	fCangjie	FICE
U+710C.0	fFourCorner	93847
U+710C.0	fHakka	  zun5
U+710C.0	fCantonese	zeon3; zeot1; ceot1; keoi1
U+710C.0	fMandarin	jun4; zun4; zuan3
U+710C.0	fEnglish	 light
U+710D.0	fUTF8	焍
U+710D.0	fCNS11643	2-366C
U+710D.0	fBig5	D649
U+710D.0	fGB	3-4F7C
U+710D.0	fR/S	 86.7
U+710D.0	fTotalStrokes	 11
U+710D.0	fCangjie	FCNH
U+710D.0	fFourCorner	98827
U+710D.0	fCantonese	dai6
U+710D.0	fMandarin	di4; ti1
U+710E.0	fUTF8	焎
U+710E.0	fCNS11643	2-367E
U+710E.0	fBig5	D65B
U+710E.0	fGB	5-5259
U+710E.0	fR/S	 86.7
U+710E.0	fTotalStrokes	 11
U+710E.0	fCangjie	QLF
U+710E.0	fFourCorner	52332
U+710E.0	fCantonese	hit3; cit3
U+710E.0	fMandarin	xie4; che4
U+710F.0	fUTF8	焏
U+710F.0	fCNS11643	3-3733
U+710F.0	fGB	E-316B
U+710F.0	fR/S	 86.7
U+710F.0	fTotalStrokes	 11
U+710F.0	fCangjie	NEF
U+710F.0	fFourCorner	17330
U+710F.0	fCantonese	gik1
U+710F.0	fMandarin	ji2; qi4
U+7110.0	fUTF8	焐
U+7110.0	fCNS11643	2-3674
U+7110.0	fBig5	D651
U+7110.0	fGB	0-6C49
U+7110.0	fR/S	 86.7
U+7110.0	fTotalStrokes	 11
U+7110.0	fCangjie	FMMR
U+7110.0	fFourCorner	91861
U+7110.0	fCantonese	ng6
U+7110.0	fMandarin	wu4
U+7111.0	fUTF8	焑
U+7111.0	fCNS11643	3-6550
U+7111.0	fGB	E-316C
U+7111.0	fR/S	 86.7
U+7111.0	fTotalStrokes	 11
U+7111.0	fCangjie	XFWD
U+7111.0	fFourCorner	9680
U+7111.0	fCantonese	jin1
U+7112.0	fUTF8	焒
U+7112.0	fCNS11643	3-6551
U+7112.0	fGB	E-316D
U+7112.0	fR/S	 86.7
U+7112.0	fTotalStrokes	 11
U+7112.0	fCangjie	FRR
U+7112.0	fFourCorner	9686
U+7112.0	fCantonese	leoi5
U+7113.0	fUTF8	焓
U+7113.0	fCNS11643	2-3678
U+7113.0	fBig5	D655
U+7113.0	fGB	0-6C4A
U+7113.0	fR/S	 86.7
U+7113.0	fTotalStrokes	 11
U+7113.0	fCangjie	FOIR
U+7113.0	fFourCorner	98862
U+7113.0	fCantonese	ham4
U+7113.0	fMandarin	han2
U+7113.0	fEnglish	onomatopoetic, the sound of a fiercely-burning fire
U+7114.0	fUTF8	焔
U+7114.0	fGB	E-316E
U+7114.0	fR/S	 86.8
U+7114.0	fTotalStrokes	 12
U+7114.0	fCangjie	FNLA
U+7114.0	fCantonese	jim6
U+7114.0	fMandarin	yan4; yan2
U+7114.0	fEnglish	 flame; blaze glowing; blazing
U+7115.0	fUTF8	焕
U+7115.0	fGB	0-3B40
U+7115.0	fR/S	 86.7
U+7115.0	fTotalStrokes	 11
U+7115.0	fCangjie	XFNBK
U+7115.0	fHakka	 fon5; fan5
U+7115.0	fCantonese	wun6
U+7115.0	fMandarin	huan4
U+7115.0	fEnglish	simplified form of &#x7165;
U+7116.0	fUTF8	焖
U+7116.0	fGB	0-6C4B
U+7116.0	fR/S	 86.7
U+7116.0	fTotalStrokes	 11
U+7116.0	fCangjie	FLSP
U+7116.0	fCantonese	mun6
U+7116.0	fMandarin	men4; men1
U+7116.0	fEnglish	simplified form of &#x71DC;
U+7117.0	fUTF8	焗
U+7117.0	fCNS11643	2-366E
U+7117.0	fBig5	D64B
U+7117.0	fGB	3-4F7E
U+7117.0	fR/S	 86.7
U+7117.0	fTotalStrokes	 11
U+7117.0	fCangjie	FSSR
U+7117.0	fFourCorner	97827
U+7117.0	fHakka	 giuk7; guk8; kuk8
U+7117.0	fCantonese	guk6
U+7117.0	fMandarin	ju2
U+7117.0	fEnglish	 bake
U+7118.0	fUTF8	焘
U+7118.0	fGB	0-6C62
U+7118.0	fR/S	 86.7
U+7118.0	fTotalStrokes	 11
U+7118.0	fCangjie	XQIF
U+7118.0	fHakka	 tau2; tau5
U+7118.0	fCantonese	tou4; dou6 (also)
U+7118.0	fMandarin	chou2; dao4; tao2; tao1
U+7118.0	fEnglish	simplified form of &#x71FE;
U+7119.0	fUTF8	焙
U+7119.0	fCNS11643	1-604E
U+7119.0	fBig5	B548
U+7119.0	fGB	0-313A
U+7119.0	fR/S	 86.8
U+7119.0	fTotalStrokes	 12
U+7119.0	fCangjie	FYTR
U+7119.0	fFourCorner	90861
U+7119.0	fHakka	 poi3; poi5; poi6
U+7119.0	fCantonese	bui6; pui6 (also)
U+7119.0	fMandarin	bei4
U+7119.0	fEnglish	 dry over slow fire; dry or heat near a fire; bake; roast
U+711A.0	fUTF8	焚
U+711A.0	fCNS11643	1-604F
U+711A.0	fBig5	B549
U+711A.0	fGB	0-3759
U+711A.0	fR/S	 86.8
U+711A.0	fTotalStrokes	 12
U+711A.0	fCangjie	DDF
U+711A.0	fFourCorner	44809
U+711A.0	fHakka	  fun2; fun3
U+711A.0	fCantonese	fan4; fan3; fan5
U+711A.0	fMandarin	fen4; fen1; fen2
U+711A.0	fEnglish	 burn; set fire to
U+711B.0	fUTF8	焛
U+711B.0	fCNS11643	2-3D69
U+711B.0	fBig5	DA65
U+711B.0	fGB	E-316F
U+711B.0	fR/S	 86.8
U+711B.0	fTotalStrokes	 12
U+711B.0	fCangjie	ANF
U+711B.0	fFourCorner	77809
U+711B.0	fCantonese	leon6
U+711B.0	fMandarin	lin4
U+711C.0	fUTF8	焜
U+711C.0	fCNS11643	1-6055
U+711C.0	fBig5	B54F
U+711C.0	fGB	3-5027
U+711C.0	fR/S	 86.8
U+711C.0	fTotalStrokes	 12
U+711C.0	fCangjie	FAPP
U+711C.0	fFourCorner	96811
U+711C.0	fHakka	 kun1;  fun3; fun5
U+711C.0	fCantonese	kwan1; gwan1; wan6
U+711C.0	fMandarin	kun1; hun4; hun3
U+711C.0	fEnglish	 fire; flames bright; shining
U+711D.0	fUTF8	焝
U+711D.0	fCNS11643	3-3C74
U+711D.0	fGB	3-502A
U+711D.0	fR/S	 86.8
U+711D.0	fTotalStrokes	 12
U+711D.0	fCangjie	FHPA
U+711D.0	fFourCorner	92860
U+711D.0	fCantonese	fan3
U+711D.0	fMandarin	hun4
U+711E.0	fUTF8	焞
U+711E.0	fCNS11643	2-3D5D
U+711E.0	fBig5	DA59
U+711E.0	fGB	3-502B
U+711E.0	fR/S	 86.8
U+711E.0	fTotalStrokes	 12
U+711E.0	fCangjie	FYRD
U+711E.0	fFourCorner	90843
U+711E.0	fHakka	  dun1; sun2; shun2; tun2
U+711E.0	fCantonese	seon4; tan1 (also); teoi1; teon1
U+711E.0	fMandarin	tun1; tui1; zuan3; zun4; chun1
U+711E.0	fEnglish	dim
U+711F.0	fUTF8	焟
U+711F.0	fCNS11643	2-3D66
U+711F.0	fBig5	DA62
U+711F.0	fGB	3-5023
U+711F.0	fR/S	 86.8
U+711F.0	fTotalStrokes	 12
U+711F.0	fCangjie	FTA
U+711F.0	fFourCorner	91861
U+711F.0	fCantonese	sik1
U+711F.0	fMandarin	xi2
U+7120.0	fUTF8	焠
U+7120.0	fCNS11643	2-3D5C
U+7120.0	fBig5	DA58
U+7120.0	fGB	5-5171
U+7120.0	fR/S	 86.8
U+7120.0	fTotalStrokes	 12
U+7120.0	fCangjie	FYOJ
U+7120.0	fFourCorner	90848
U+7120.0	fHakka	  cui5
U+7120.0	fCantonese	ceoi3; seoi6 (also)
U+7120.0	fMandarin	cui4
U+7120.0	fEnglish	 temper burn
U+7121.0	fUTF8	無
U+7121.0	fCNS11643	1-6052
U+7121.0	fBig5	B54C
U+7121.0	fGB	1-4E5E
U+7121.0	fR/S	 86.8
U+7121.0	fTotalStrokes	 12
U+7121.0	fCangjie	OTF
U+7121.0	fFourCorner	80331
U+7121.0	fHakka	  wu2;  mo2; wu3; mau2; vu2
U+7121.0	fCantonese	mou4; mo4
U+7121.0	fMandarin	wu2; mo2
U+7121.0	fEnglish	[1] negative; no; not; none [2] without; destitute of; wanting; lack; have not [3] no matter what (or how); not yet [4] a Chinese family name
U+7122.0	fUTF8	焢
U+7122.0	fCNS11643	2-3D64
U+7122.0	fBig5	DA60
U+7122.0	fGB	E-3170
U+7122.0	fR/S	 86.8
U+7122.0	fTotalStrokes	 12
U+7122.0	fCangjie	FJCM
U+7122.0	fFourCorner	93811
U+7122.0	fCantonese	hung1
U+7122.0	fMandarin	hong1
U+7123.0	fUTF8	焣
U+7123.0	fCNS11643	2-3D62
U+7123.0	fBig5	DA5E
U+7123.0	fGB	E-3171
U+7123.0	fR/S	 86.8
U+7123.0	fTotalStrokes	 12
U+7123.0	fCangjie	SEF
U+7123.0	fFourCorner	17334
U+7123.0	fCantonese	caau2; zeoi6
U+7123.0	fMandarin	ju4
U+7124.0	fUTF8	焤
U+7124.0	fCNS11643	3-3C7D
U+7124.0	fGB	E-3172
U+7124.0	fR/S	  86.8
U+7124.0	fTotalStrokes	 12
U+7124.0	fCangjie	IOIF
U+7124.0	fFourCorner	00280
U+7124.0	fCantonese	fu6
U+7124.0	fMandarin	fu3
U+7125.0	fUTF8	焥
U+7125.0	fCNS11643	2-3D63
U+7125.0	fBig5	DA5F
U+7125.0	fGB	3-502E
U+7125.0	fR/S	 86.8
U+7125.0	fTotalStrokes	 12
U+7125.0	fCangjie	FJNU
U+7125.0	fFourCorner	93811
U+7125.0	fCantonese	waat3; oi3
U+7125.0	fMandarin	wo4; ai4
U+7126.0	fUTF8	焦
U+7126.0	fCNS11643	1-6050
U+7126.0	fBig5	B54A
U+7126.0	fGB	0-3D39
U+7126.0	fR/S	 86.8
U+7126.0	fTotalStrokes	 12
U+7126.0	fCangjie	OGF
U+7126.0	fFourCorner	20331
U+7126.0	fHakka	  ziau1; zeu1
U+7126.0	fCantonese	ziu1
U+7126.0	fMandarin	jiao1; qiao2; jiu1
U+7126.0	fEnglish	[1] burned; scorched; charred [2] the smell or stench of things burned [3] worried and anxious [4] a Chinese family name
U+7127.0	fUTF8	焧
U+7127.0	fCNS11643	3-3C7C
U+7127.0	fGB	5-516E
U+7127.0	fR/S	  86.8
U+7127.0	fTotalStrokes	 12
U+7127.0	fCangjie	FCIP
U+7127.0	fFourCorner	98830
U+7127.0	fCantonese	cung1
U+7127.0	fMandarin	cong1
U+7128.0	fUTF8	焨
U+7128.0	fCNS11643	2-3D67
U+7128.0	fBig5	DA63
U+7128.0	fGB	E-3173
U+7128.0	fR/S	 86.8
U+7128.0	fTotalStrokes	 12
U+7128.0	fCangjie	FAB
U+7128.0	fFourCorner	96827
U+7128.0	fCantonese	fung3
U+7128.0	fMandarin	feng4
U+7129.0	fUTF8	焩
U+7129.0	fCNS11643	F-3C7E
U+7129.0	fGB	5-516F
U+7129.0	fR/S	 86.8
U+7129.0	fTotalStrokes	 12
U+7129.0	fCangjie	FBB
U+7129.0	fFourCorner	9782
U+7129.0	fCantonese	pang4
U+7129.0	fMandarin	ping1
U+712A.0	fUTF8	焪
U+712A.0	fCNS11643	3-3C6F
U+712A.0	fGB	3-502D
U+712A.0	fR/S	 86.8
U+712A.0	fTotalStrokes	 12
U+712A.0	fCangjie	FJCN
U+712A.0	fFourCorner	93820
U+712A.0	fCantonese	hung3
U+712A.0	fMandarin	qiong1; qiong4
U+712B.0	fUTF8	焫
U+712B.0	fCNS11643	3-3C7A
U+712B.0	fGB	5-515F
U+712B.0	fR/S	  86.8
U+712B.0	fTotalStrokes	 12
U+712B.0	fCangjie	FTOB
U+712B.0	fFourCorner	94820
U+712B.0	fCantonese	jyut3; jyut6 (also); naat3; laat3 (also)
U+712B.0	fMandarin	ruo4; re4
U+712B.0	fEnglish	hot; to scald, burn
U+712C.0	fUTF8	焬
U+712C.0	fCNS11643	3-3C72
U+712C.0	fGB	E-3174
U+712C.0	fR/S	  86.8
U+712C.0	fTotalStrokes	 12
U+712C.0	fCangjie	FAPH
U+712C.0	fFourCorner	96820
U+712C.0	fHakka	 sit7;  jit8; rit8
U+712C.0	fCantonese	sik1
U+712C.0	fMandarin	xi2; yi4
U+712D.0	fUTF8	焭
U+712D.0	fCNS11643	3-3D21
U+712D.0	fGB	5-3344
U+712D.0	fR/S	  86.8
U+712D.0	fTotalStrokes	 12
U+712D.0	fCangjie	FFBHN
U+712D.0	fFourCorner	99210
U+712D.0	fHakka	 kiun2
U+712D.0	fCantonese	king4
U+712D.0	fMandarin	qiong2
U+712D.0	fEnglish	orphan; alone, desolate
U+712E.0	fUTF8	焮
U+712E.0	fCNS11643	2-3D60
U+712E.0	fBig5	DA5C
U+712E.0	fGB	5-516D
U+712E.0	fR/S	 86.8
U+712E.0	fTotalStrokes	 12
U+712E.0	fCangjie	FHLO
U+712E.0	fFourCorner	97880
U+712E.0	fCantonese	jan1; jan3
U+712E.0	fMandarin	xin4; xin1
U+712E.0	fEnglish	[1] burn; heat; cauterize [2] heat (Chinese medicine)
U+712F.0	fUTF8	焯
U+712F.0	fCNS11643	2-3D5E
U+712F.0	fBig5	DA5A
U+712F.0	fGB	0-6C4C
U+712F.0	fR/S	 86.8
U+712F.0	fTotalStrokes	 12
U+712F.0	fCangjie	FYAJ
U+712F.0	fFourCorner	91846
U+712F.0	fHakka	 zok7; chok7; shok7; sok7
U+712F.0	fCantonese	zoek3; coek3 (also); ceoi3
U+712F.0	fMandarin	zhuo2; du3; zhuo1; chuo4; chao1
U+712F.0	fEnglish	[1] same as $ [2] bright and brilliant
U+7130.0	fUTF8	焰
U+7130.0	fCNS11643	1-6051
U+7130.0	fBig5	B54B
U+7130.0	fGB	0-5166
U+7130.0	fR/S	 86.8
U+7130.0	fTotalStrokes	 12
U+7130.0	fCangjie	FNHX
U+7130.0	fFourCorner	97877
U+7130.0	fHakka	 jam5; jam3; ram6; liam5
U+7130.0	fCantonese	jim6; jim4 (also)
U+7130.0	fMandarin	yan4; yan2
U+7130.0	fEnglish	[1] flame; blaze [2] glowing; brilliant
U+7131.0	fUTF8	焱
U+7131.0	fCNS11643	2-3D61
U+7131.0	fBig5	DA5D
U+7131.0	fGB	0-6C4D
U+7131.0	fR/S	 86.8
U+7131.0	fTotalStrokes	 12
U+7131.0	fCangjie	FFF
U+7131.0	fFourCorner	90889
U+7131.0	fHakka	  jam5; ram6; jam3
U+7131.0	fCantonese	jim6
U+7131.0	fMandarin	yan4; yi4; yan3; biao1; xi4; huo4
U+7131.0	fEnglish	flames
U+7132.0	fUTF8	焲
U+7132.0	fCNS11643	2-3D65
U+7132.0	fBig5	DA61
U+7132.0	fGB	5-5170
U+7132.0	fR/S	 86.8
U+7132.0	fTotalStrokes	 12
U+7132.0	fCangjie	FYOK
U+7132.0	fFourCorner	90847
U+7132.0	fHakka	 jit8; rit8
U+7132.0	fCantonese	jik6
U+7132.0	fMandarin	yi4; yue4; yao4
U+7132.0	fEnglish	[1] blaze [2] withered
U+7133.0	fUTF8	焳
U+7133.0	fCNS11643	3-3C7B
U+7133.0	fGB	E-3175
U+7133.0	fR/S	 86.8
U+7133.0	fTotalStrokes	 12
U+7133.0	fCangjie	FOG
U+7133.0	fFourCorner	90810
U+7133.0	fCantonese	zoek3
U+7133.0	fMandarin	jue2; jiao1
U+7134.0	fUTF8	焴
U+7134.0	fCNS11643	3-3C79
U+7134.0	fGB	5-5172
U+7134.0	fR/S	 86.8; 86.7
U+7134.0	fTotalStrokes	 12; 11
U+7134.0	fCangjie	FYIB
U+7134.0	fFourCorner	90820
U+7134.0	fHakka	 juk7
U+7134.0	fCantonese	juk1
U+7134.0	fMandarin	yu4
U+7134.0	fEnglish	 bright flame
U+7135.0	fUTF8	焵
U+7135.0	fCNS11643	3-3C71
U+7135.0	fGB	3-4F52
U+7135.0	fR/S	 86.8
U+7135.0	fTotalStrokes	 12
U+7135.0	fCangjie	FBTU
U+7135.0	fFourCorner	97820
U+7135.0	fHakka	 gong5
U+7135.0	fCantonese	gong3
U+7135.0	fMandarin	gang4
U+7135.0	fEnglish	 temper steel
U+7136.0	fUTF8	然
U+7136.0	fCNS11643	1-6053
U+7136.0	fBig5	B54D
U+7136.0	fGB	0-483B
U+7136.0	fR/S	 86.8
U+7136.0	fTotalStrokes	 12
U+7136.0	fCangjie	BKF
U+7136.0	fFourCorner	23333
U+7136.0	fHakka	 jen2; jan3; ren2; jan2
U+7136.0	fCantonese	jin4; jin2 (also)
U+7136.0	fMandarin	ran2
U+7136.0	fEnglish	[1] yes; most certainly; permission; right; correct [2] however; but; still; nevertheless; on the other hand [3] really; if so [4] same as $ (burn) [5] a Chinese family name
U+7136.1	fUTF8	然
U+7136.1	fCNS11643	1-6053
U+7136.1	fBig5	B54D
U+7136.1	fGB	0-483B
U+7136.1	fR/S	 86.8
U+7136.1	fTotalStrokes	 12
U+7136.1	fCangjie	BKF
U+7136.1	fFourCorner	23333
U+7136.1	fMandarin	ran5
U+7137.0	fUTF8	焷
U+7137.0	fCNS11643	3-3C73
U+7137.0	fGB	3-5029
U+7137.0	fR/S	  86.8
U+7137.0	fTotalStrokes	 12
U+7137.0	fCangjie	FHWJ
U+7137.0	fFourCorner	96840
U+7137.0	fCantonese	pei4
U+7137.0	fMandarin	pi2; pi4
U+7138.0	fUTF8	焸
U+7138.0	fCNS11643	3-3C77
U+7138.0	fGB	3-5028
U+7138.0	fR/S	 86.8
U+7138.0	fTotalStrokes	 12
U+7138.0	fCangjie	AAF
U+7138.0	fFourCorner	66800
U+7138.0	fCantonese	hing2
U+7138.0	fMandarin	gu3; xiong4; ying1
U+7139.0	fUTF8	焹
U+7139.0	fCNS11643	3-6553
U+7139.0	fGB	E-3176
U+7139.0	fR/S	 86.8
U+7139.0	fTotalStrokes	 12
U+7139.0	fCangjie	FBTV
U+7139.0	fFourCorner	9782
U+7139.0	fCantonese	gong3
U+713A.0	fUTF8	焺
U+713A.0	fCNS11643	2-3D68
U+713A.0	fBig5	DA64
U+713A.0	fGB	5-516C
U+713A.0	fR/S	 86.8
U+713A.0	fTotalStrokes	 12
U+713A.0	fCangjie	FAHT
U+713A.0	fFourCorner	96840
U+713A.0	fCantonese	sing1
U+713A.0	fMandarin	sheng1
U+713B.0	fUTF8	焻
U+713B.0	fCNS11643	3-6554
U+713B.0	fGB	5-516B
U+713B.0	fR/S	 86.8
U+713B.0	fTotalStrokes	 12
U+713B.0	fCangjie	FAA
U+713B.0	fFourCorner	9686
U+713B.0	fCantonese	coeng3
U+713B.0	fMandarin	chang4
U+713B.0	fEnglish	breath
U+713C.0	fUTF8	焼
U+713C.0	fGB	E-3177
U+713C.0	fR/S	 86.8
U+713C.0	fTotalStrokes	 12
U+713C.0	fCangjie	FJTU
U+713C.0	fCantonese	siu1
U+713C.0	fMandarin	shao1; shao4
U+713C.0	fEnglish	 burn bake heat roast
U+713D.0	fUTF8	焽
U+713D.0	fCNS11643	3-6555
U+713D.0	fGB	E-3178
U+713D.0	fR/S	 86.8
U+713D.0	fTotalStrokes	 12
U+713D.0	fCangjie	ABF
U+713D.0	fFourCorner	6780
U+713D.0	fCantonese	ming4
U+713D.0	fMandarin	gu3; xiong4
U+713E.0	fUTF8	焾
U+713E.0	fGB	1-7C41
U+713E.0	fR/S	 86.8
U+713E.0	fTotalStrokes	 12
U+713E.0	fCangjie	FOIP
U+713E.0	fCantonese	nam4; lam4 (also)
U+713E.0	fEnglish	soft
U+713F.0	fUTF8	焿
U+713F.0	fCNS11643	3-6552
U+713F.0	fGB	E-3179
U+713F.0	fR/S	 86.8
U+713F.0	fTotalStrokes	 12
U+713F.0	fCangjie	FILO
U+713F.0	fFourCorner	9088
U+713F.0	fCantonese	gang1
U+713F.0	fEnglish	
U+7140.0	fUTF8	煀
U+7140.0	fGB	1-7C42
U+7140.0	fR/S	 86.8
U+7140.0	fTotalStrokes	 12
U+7140.0	fCangjie	FSUU
U+7140.0	fCantonese	wat1
U+7140.0	fEnglish	to smoke; to fumigate
U+7141.0	fUTF8	煁
U+7141.0	fCNS11643	2-4456
U+7141.0	fBig5	DE70
U+7141.0	fGB	3-5030
U+7141.0	fR/S	 86.9
U+7141.0	fTotalStrokes	 13
U+7141.0	fCangjie	FTMV
U+7141.0	fFourCorner	94811
U+7141.0	fHakka	 shim3; siim3; shim1
U+7141.0	fCantonese	sam4; nam4
U+7141.0	fMandarin	chen2; shen2
U+7141.0	fEnglish	hearth
U+7142.0	fUTF8	煂
U+7142.0	fCNS11643	2-445D
U+7142.0	fBig5	DE77
U+7142.0	fGB	E-317A
U+7142.0	fR/S	 86.9
U+7142.0	fTotalStrokes	 13
U+7142.0	fCangjie	FTLJ
U+7142.0	fFourCorner	94856
U+7142.0	fCantonese	haak3; haak1
U+7142.0	fMandarin	he4
U+7143.0	fUTF8	煃
U+7143.0	fCNS11643	2-445F
U+7143.0	fBig5	DE79
U+7143.0	fGB	3-5032
U+7143.0	fR/S	 86.9
U+7143.0	fTotalStrokes	 13
U+7143.0	fCangjie	FKGG
U+7143.0	fFourCorner	94814
U+7143.0	fCantonese	kwai2
U+7143.0	fMandarin	kui3
U+7143.0	fEnglish	 remove; get rid of scatter
U+7144.0	fUTF8	煄
U+7144.0	fCNS11643	2-4465
U+7144.0	fBig5	DEA1
U+7144.0	fGB	5-517B
U+7144.0	fR/S	 86.9
U+7144.0	fTotalStrokes	 13
U+7144.0	fCangjie	FHJG
U+7144.0	fFourCorner	92814
U+7144.0	fHakka	 zhung3; zung3
U+7144.0	fCantonese	zung2
U+7144.0	fMandarin	zhong1; zhong3
U+7144.0	fEnglish	[1] fire flaming up [2] kindle
U+7145.0	fUTF8	煅
U+7145.0	fCNS11643	3-4257
U+7145.0	fGB	0-6C51
U+7145.0	fR/S	  86.9
U+7145.0	fTotalStrokes	 13
U+7145.0	fCangjie	FHJE
U+7145.0	fFourCorner	97840
U+7145.0	fHakka	 ton5; ton3; ton6; don5
U+7145.0	fCantonese	dyun3; dyun6 (also)
U+7145.0	fMandarin	duan4
U+7145.0	fEnglish	 forge metal perfect one's skill
U+7146.0	fUTF8	煆
U+7146.0	fCNS11643	1-6522
U+7146.0	fBig5	B7DA
U+7146.0	fGB	3-503A
U+7146.0	fR/S	 86.9
U+7146.0	fTotalStrokes	 13
U+7146.0	fCangjie	FRSE
U+7146.0	fFourCorner	97847
U+7146.0	fHakka	 ha3
U+7146.0	fCantonese	haa6; haa1; dyun6; dyun3
U+7146.0	fMandarin	xia1; xia4; ya1; duan4; xia3
U+7146.0	fEnglish	raging fire
U+7147.0	fUTF8	煇
U+7147.0	fCNS11643	2-4451
U+7147.0	fBig5	DE6B
U+7147.0	fGB	E-317B
U+7147.0	fR/S	 86.9
U+7147.0	fTotalStrokes	 13
U+7147.0	fCangjie	FBJJ
U+7147.0	fFourCorner	97850
U+7147.0	fHakka	 fui1; fi1
U+7147.0	fCantonese	wai1; wan6; fai1; fan1
U+7147.0	fMandarin	hui1; xun1; yun4; hun2; hun4; xuan1; xuan4
U+7147.0	fEnglish	weld; solder
U+7148.0	fUTF8	煈
U+7148.0	fCNS11643	3-424C
U+7148.0	fGB	5-513B
U+7148.0	fR/S	 86.9
U+7148.0	fTotalStrokes	 13
U+7148.0	fCangjie	FHNI
U+7148.0	fFourCorner	97810
U+7148.0	fCantonese	fung3
U+7148.0	fMandarin	feng4
U+7149.0	fUTF8	煉
U+7149.0	fCNS11643	1-6478
U+7149.0	fBig5	B7D2
U+7149.0	fGB	1-4136
U+7149.0	fR/S	 86.9
U+7149.0	fTotalStrokes	 13
U+7149.0	fCangjie	FDWF
U+7149.0	fFourCorner	95896
U+7149.0	fHakka	  lien5; len3; lien6; lian6
U+7149.0	fCantonese	lin6
U+7149.0	fMandarin	lian4; lan4
U+7149.0	fEnglish	[1] smelt; refine; distill; condense (milk); temper (metal) with fire [2] keep herbs, etc. boiling for a long time (Chinese medicine) [3] train; form character by hardship
U+714A.0	fUTF8	煊
U+714A.0	fCNS11643	3-424E
U+714A.0	fGB	0-6C53
U+714A.0	fR/S	  86.9
U+714A.0	fTotalStrokes	 13
U+714A.0	fCangjie	FJMM
U+714A.0	fFourCorner	93810
U+714A.0	fCantonese	hyun1
U+714A.0	fMandarin	xuan1
U+714A.0	fEnglish	warm
U+714B.0	fUTF8	煋
U+714B.0	fCNS11643	2-4460
U+714B.0	fBig5	DE7A
U+714B.0	fGB	3-5035
U+714B.0	fR/S	 86.9
U+714B.0	fTotalStrokes	 13
U+714B.0	fCangjie	FAHM
U+714B.0	fFourCorner	96815
U+714B.0	fHakka	  sen1
U+714B.0	fCantonese	sing1; sing3
U+714B.0	fMandarin	xing1
U+714B.0	fEnglish	a raging fire
U+714C.0	fUTF8	煌
U+714C.0	fCNS11643	1-647D
U+714C.0	fBig5	B7D7
U+714C.0	fGB	0-3B4D
U+714C.0	fR/S	 86.9
U+714C.0	fTotalStrokes	 13
U+714C.0	fCangjie	FHAG
U+714C.0	fFourCorner	96814
U+714C.0	fHakka	 fong2
U+714C.0	fCantonese	wong4
U+714C.0	fMandarin	huang2; heng2; huang3; huang4
U+714C.0	fEnglish	bright and brilliant; shining
U+714D.0	fUTF8	煍
U+714D.0	fCNS11643	2-4466
U+714D.0	fBig5	DEA2
U+714D.0	fGB	E-317C
U+714D.0	fR/S	 86.9
U+714D.0	fTotalStrokes	 13
U+714D.0	fCangjie	FHDF
U+714D.0	fFourCorner	99880
U+714D.0	fHakka	 ziau3;  ciau3
U+714D.0	fCantonese	ziu2; ciu2; cau1
U+714D.0	fMandarin	jiao3
U+714D.0	fEnglish	 colour by smoke
U+714E.0	fUTF8	煎
U+714E.0	fCNS11643	1-6474
U+714E.0	fBig5	B7CE
U+714E.0	fGB	0-3C65
U+714E.0	fR/S	 86.9
U+714E.0	fTotalStrokes	 13
U+714E.0	fCangjie	TBNF
U+714E.0	fFourCorner	80332
U+714E.0	fHakka	 zien1; zen1; tsian1; zien5
U+714E.0	fCantonese	zin1
U+714E.0	fMandarin	jian1; jian3; jian4
U+714E.0	fEnglish	[1] fry in fat or oil [2] decoct [3] torment; kill (figuratively)
U+714F.0	fUTF8	煏
U+714F.0	fCNS11643	3-4251
U+714F.0	fGB	3-5031
U+714F.0	fR/S	  86.9
U+714F.0	fTotalStrokes	 13
U+714F.0	fCangjie	FMRW
U+714F.0	fFourCorner	91860
U+714F.0	fHakka	  bet7; bit7; bit8; kong5
U+714F.0	fCantonese	bik1; bik6
U+714F.0	fMandarin	bi4
U+714F.0	fEnglish	 dry by fire iron
U+7150.0	fUTF8	煐
U+7150.0	fCNS11643	2-4463
U+7150.0	fBig5	DE7D
U+7150.0	fGB	3-5024
U+7150.0	fR/S	 86.9; 86.8
U+7150.0	fTotalStrokes	 13
U+7150.0	fCangjie	FTLK
U+7150.0	fFourCorner	94835
U+7150.0	fCantonese	jing1
U+7150.0	fMandarin	ying1
U+7151.0	fUTF8	煑
U+7151.0	fCNS11643	3-4258
U+7151.0	fGB	E-317D
U+7151.0	fR/S	 86.9
U+7151.0	fTotalStrokes	 13
U+7151.0	fCangjie	JAF
U+7151.0	fFourCorner	44800
U+7151.0	fCantonese	zyu2
U+7151.0	fMandarin	zhu3
U+7151.0	fEnglish	cook
U+7152.0	fUTF8	煒
U+7152.0	fCNS11643	2-4453
U+7152.0	fBig5	DE6D
U+7152.0	fGB	1-6C3F
U+7152.0	fR/S	 86.9
U+7152.0	fTotalStrokes	 13
U+7152.0	fCangjie	FDMQ
U+7152.0	fFourCorner	95856
U+7152.0	fHakka	 wi3; wui3; vui3
U+7152.0	fCantonese	wai5
U+7152.0	fMandarin	wei3; hui1
U+7152.0	fEnglish	[1] dark red [2] glowing; bright
U+7153.0	fUTF8	煓
U+7153.0	fCNS11643	2-4464
U+7153.0	fBig5	DE7E
U+7153.0	fGB	3-5036
U+7153.0	fR/S	 86.9
U+7153.0	fTotalStrokes	 13
U+7153.0	fCangjie	FUMB
U+7153.0	fFourCorner	92827
U+7153.0	fHakka	 ton2
U+7153.0	fCantonese	tyun1; tyun4
U+7153.0	fMandarin	tuan1
U+7153.0	fEnglish	the red blaze of fire
U+7154.0	fUTF8	煔
U+7154.0	fCNS11643	2-4452
U+7154.0	fBig5	DE6C
U+7154.0	fGB	3-5039
U+7154.0	fR/S	 86.9
U+7154.0	fTotalStrokes	 13
U+7154.0	fCangjie	FFYR
U+7154.0	fFourCorner	91860
U+7154.0	fCantonese	sim2
U+7154.0	fMandarin	tian4; qian2; shan1; han1; shan3; shan4; yan4
U+7155.0	fUTF8	煕
U+7155.0	fGB	E-317E
U+7155.0	fR/S	 86.9
U+7155.0	fTotalStrokes	 13
U+7155.0	fCangjie	SUF
U+7155.0	fCantonese	hei1
U+7155.0	fMandarin	xi1
U+7155.0	fEnglish	bright; splendid; glorious
U+7156.0	fUTF8	煖
U+7156.0	fCNS11643	1-6524
U+7156.0	fBig5	B7DC
U+7156.0	fGB	E-3221
U+7156.0	fR/S	 86.9
U+7156.0	fTotalStrokes	 13
U+7156.0	fCangjie	FBME
U+7156.0	fFourCorner	92847
U+7156.0	fHakka	  non1;  non3
U+7156.0	fCantonese	hyun1; nyun5; lyun5 (also)
U+7156.0	fMandarin	nuan3; xuan1
U+7156.0	fEnglish	warm; genial; warm
U+7157.0	fUTF8	煗
U+7157.0	fCNS11643	3-4252
U+7157.0	fGB	E-3222
U+7157.0	fR/S	  86.9
U+7157.0	fTotalStrokes	 13
U+7157.0	fCangjie	FMBK
U+7157.0	fFourCorner	91830
U+7157.0	fHakka	 nun1
U+7157.0	fCantonese	nyun5; lyun5 (also); nyn5
U+7157.0	fMandarin	nuan3
U+7158.0	fUTF8	煘
U+7158.0	fCNS11643	2-445E
U+7158.0	fBig5	DE78
U+7158.0	fGB	5-5177
U+7158.0	fR/S	 86.9
U+7158.0	fTotalStrokes	 13
U+7158.0	fCangjie	FIHR
U+7158.0	fFourCorner	93850
U+7158.0	fCantonese	cim4; gaam2; tim4
U+7158.0	fMandarin	chan2
U+7159.0	fUTF8	煙
U+7159.0	fCNS11643	1-6475
U+7159.0	fBig5	B7CF
U+7159.0	fGB	E-3223
U+7159.0	fR/S	 86.9
U+7159.0	fTotalStrokes	 13
U+7159.0	fCangjie	FMWG
U+7159.0	fFourCorner	91814
U+7159.0	fHakka	  jen1; ren1; jan1
U+7159.0	fCantonese	jin1
U+7159.0	fMandarin	yan1; yin1
U+7159.0	fEnglish	[1] smoke; fumes [2] soot; tobacco; cigarettes [3] mist; vapor [4] opium [5] a Chinese family name
U+715A.0	fUTF8	煚
U+715A.0	fCNS11643	2-4467
U+715A.0	fBig5	DEA3
U+715A.0	fGB	3-5026
U+715A.0	fR/S	 86.9
U+715A.0	fTotalStrokes	 13
U+715A.0	fCangjie	ASF
U+715A.0	fFourCorner	61809
U+715A.0	fCantonese	gwing2
U+715A.0	fMandarin	jiong3
U+715B.0	fUTF8	煛
U+715B.0	fCNS11643	3-483C
U+715B.0	fGB	3-503F
U+715B.0	fR/S	 109.9; 86.9
U+715B.0	fTotalStrokes	 14; 13
U+715B.0	fCangjie	BUF
U+715B.0	fFourCorner	66800
U+715B.0	fCantonese	gwing2
U+715B.0	fMandarin	jiong3; jiong1
U+715C.0	fUTF8	煜
U+715C.0	fCNS11643	1-647A
U+715C.0	fBig5	B7D4
U+715C.0	fGB	0-6C4F
U+715C.0	fR/S	 86.9
U+715C.0	fTotalStrokes	 13
U+715C.0	fCangjie	FAYT
U+715C.0	fFourCorner	96818
U+715C.0	fHakka	  juk7;  hiuk7; ruk7
U+715C.0	fCantonese	juk1; jap6
U+715C.0	fMandarin	yu4; kao4; yi4
U+715C.0	fEnglish	[1] bright and brilliant [2] flames or blazes [3] illuminate; shine
U+715D.0	fUTF8	煝
U+715D.0	fCNS11643	2-4457
U+715D.0	fBig5	DE71
U+715D.0	fGB	5-5222
U+715D.0	fR/S	 86.9
U+715D.0	fTotalStrokes	 13
U+715D.0	fCangjie	FAHU
U+715D.0	fFourCorner	97867
U+715D.0	fCantonese	mei6
U+715D.0	fMandarin	mei4; mei2
U+715D.0	fEnglish	twisted paper for lighting cigar
U+715E.0	fUTF8	煞
U+715E.0	fCNS11643	1-6521
U+715E.0	fBig5	B7D9
U+715E.0	fGB	0-4937
U+715E.0	fR/S	 86.9
U+715E.0	fTotalStrokes	 13
U+715E.0	fCangjie	NKF
U+715E.0	fFourCorner	28334
U+715E.0	fHakka	  sat7; sot7; sot8
U+715E.0	fCantonese	saat3
U+715E.0	fMandarin	sha4; shai4
U+715E.0	fEnglish	[1] fierce god; malignant deity; evil spirit; goblin [2] very much; extremely [3] bring to an end; conclude
U+715E.1	fUTF8	煞
U+715E.1	fCNS11643	1-6521
U+715E.1	fBig5	B7D9
U+715E.1	fGB	0-4937
U+715E.1	fR/S	 86.9
U+715E.1	fTotalStrokes	 13
U+715E.1	fCangjie	NKF
U+715E.1	fFourCorner	28334
U+715E.1	fMandarin	sha1
U+715E.1	fEnglish	[1] tighten; bind [2] offset; reduce; mitigate [3] an auxiliary particle in old usage [4] brake; stop; bring to a close
U+715F.0	fUTF8	煟
U+715F.0	fCNS11643	2-4462
U+715F.0	fBig5	DE7C
U+715F.0	fGB	5-5179
U+715F.0	fR/S	 86.9
U+715F.0	fTotalStrokes	 13
U+715F.0	fCangjie	FWB
U+715F.0	fFourCorner	96827
U+715F.0	fCantonese	wai6
U+715F.0	fMandarin	wei4; wu4
U+715F.0	fEnglish	bright room
U+7160.0	fUTF8	煠
U+7160.0	fCNS11643	2-4455
U+7160.0	fBig5	DE6F
U+7160.0	fGB	5-5174
U+7160.0	fR/S	 86.9
U+7160.0	fTotalStrokes	 13
U+7160.0	fCangjie	FPTD
U+7160.0	fFourCorner	94894
U+7160.0	fHakka	 cap7; sap8; jap7; rap7
U+7160.0	fCantonese	zaa3; saap6
U+7160.0	fMandarin	ye4; zha2; cha2
U+7160.0	fEnglish	to fry in fat or oil. to scald
U+7161.0	fUTF8	煡
U+7161.0	fCNS11643	2-445C
U+7161.0	fBig5	DE76
U+7161.0	fGB	E-3224
U+7161.0	fR/S	 86.9
U+7161.0	fTotalStrokes	 13
U+7161.0	fCangjie	FNKQ
U+7161.0	fFourCorner	95840
U+7161.0	fCantonese	zeon6
U+7161.0	fMandarin	xin4; xin1
U+7162.0	fUTF8	煢
U+7162.0	fCNS11643	2-4458
U+7162.0	fBig5	DE72
U+7162.0	fGB	1-5C64
U+7162.0	fR/S	 86.9
U+7162.0	fTotalStrokes	 13
U+7162.0	fCangjie	FFBNJ
U+7162.0	fFourCorner	99417
U+7162.0	fHakka	  kiun2
U+7162.0	fCantonese	king4
U+7162.0	fMandarin	qiong2
U+7162.0	fEnglish	 be all alone; without friends or relatives
U+7163.0	fUTF8	煣
U+7163.0	fCNS11643	2-4454
U+7163.0	fBig5	DE6E
U+7163.0	fGB	3-503B
U+7163.0	fR/S	 86.9
U+7163.0	fTotalStrokes	 13
U+7163.0	fCangjie	FNHD
U+7163.0	fFourCorner	97894
U+7163.0	fHakka	 ju2; ngiu3; riu2
U+7163.0	fCantonese	jau2; jau5; jau4
U+7163.0	fMandarin	rou3; rou2; rou4
U+7163.0	fEnglish	 bend wood by fire or steam
U+7164.0	fUTF8	煤
U+7164.0	fCNS11643	1-6477
U+7164.0	fBig5	B7D1
U+7164.0	fGB	0-433A
U+7164.0	fR/S	 86.9
U+7164.0	fTotalStrokes	 13
U+7164.0	fCangjie	FTMD
U+7164.0	fFourCorner	94894
U+7164.0	fHakka	  moi2; moi3; moe2
U+7164.0	fCantonese	mui4
U+7164.0	fMandarin	mei2
U+7164.0	fEnglish	[1] coal; coke; charcoal [2] carbon; soot
U+7165.0	fUTF8	煥
U+7165.0	fCNS11643	1-647E
U+7165.0	fBig5	B7D8
U+7165.0	fGB	1-3B40
U+7165.0	fR/S	 86.9
U+7165.0	fTotalStrokes	 13
U+7165.0	fCangjie	FNBK
U+7165.0	fFourCorner	97834
U+7165.0	fHakka	  fon5;  fan5; fon6; fon3; won3
U+7165.0	fCantonese	wun6
U+7165.0	fMandarin	huan4
U+7165.0	fEnglish	[1] shining; vigorous and elegant (appearance) [2] brilliant; lustrous; bright; luminous
U+7166.0	fUTF8	煦
U+7166.0	fCNS11643	1-647C
U+7166.0	fBig5	B7D6
U+7166.0	fGB	0-6C63
U+7166.0	fR/S	 86.9
U+7166.0	fTotalStrokes	 13
U+7166.0	fCangjie	ARF
U+7166.0	fFourCorner	67332
U+7166.0	fHakka	 hi1; hi5; hi3
U+7166.0	fCantonese	heoi2; heoi3 (also); jyu3; jyu2
U+7166.0	fMandarin	xu4; xu3; xiu1; xiong3; xu1
U+7166.0	fEnglish	[1] warm and cozy [2] favors; kindness; good graces; kind and gracious; genial
U+7167.0	fUTF8	照
U+7167.0	fCNS11643	1-6479
U+7167.0	fBig5	B7D3
U+7167.0	fGB	0-5555
U+7167.0	fR/S	 86.9
U+7167.0	fTotalStrokes	 13
U+7167.0	fCangjie	ARF
U+7167.0	fFourCorner	67336
U+7167.0	fHakka	  zau5; zhau5; zeu5
U+7167.0	fCantonese	ziu3
U+7167.0	fMandarin	zhao4
U+7167.0	fEnglish	[1] shine; illumine; reflect; light [2] certificate; license [3] in accordance with; according to; pattern on or after [4] compare; collate; survey [5] photograph; take a picture; shoot [6] look after; take care of [7] notify; proclaim [8] sunshine [9] picture [10] a Chinese family name
U+7168.0	fUTF8	煨
U+7168.0	fCNS11643	1-6523
U+7168.0	fBig5	B7DB
U+7168.0	fGB	0-6C50
U+7168.0	fR/S	 86.9
U+7168.0	fTotalStrokes	 13
U+7168.0	fCangjie	FWMV
U+7168.0	fFourCorner	96832
U+7168.0	fHakka	  woi1; wui5; voe1; voi1
U+7168.0	fCantonese	wui1
U+7168.0	fMandarin	wei1; yu4
U+7168.0	fEnglish	 bake; stew; simmer; burn in ashes; roast (sweet potatoes, etc.) in fresh cinders
U+7169.0	fUTF8	煩
U+7169.0	fCNS11643	1-6476
U+7169.0	fBig5	B7D0
U+7169.0	fGB	1-3733
U+7169.0	fR/S	 86.9
U+7169.0	fTotalStrokes	 13
U+7169.0	fCangjie	FMBC
U+7169.0	fFourCorner	91886
U+7169.0	fHakka	  fan2; fan3
U+7169.0	fCantonese	faan4
U+7169.0	fMandarin	fan2
U+7169.0	fEnglish	[1] bother; vex; trouble; annoy; worry [2] annoying [3] trouble [4] superfluous and confusing
U+7169.1	fUTF8	煩
U+7169.1	fCNS11643	1-6476
U+7169.1	fBig5	B7D0
U+7169.1	fGB	1-3733
U+7169.1	fR/S	 86.9
U+7169.1	fTotalStrokes	 13
U+7169.1	fCangjie	FMBC
U+7169.1	fFourCorner	91886
U+7169.1	fMandarin	fan5
U+716A.0	fUTF8	煪
U+716A.0	fCNS11643	2-445B
U+716A.0	fBig5	DE75
U+716A.0	fGB	3-5038
U+716A.0	fR/S	 86.9
U+716A.0	fTotalStrokes	 13
U+716A.0	fCangjie	FTCW
U+716A.0	fFourCorner	98861
U+716A.0	fHakka	 shiu2; riu2; ju2; ciu2
U+716A.0	fCantonese	cau4
U+716A.0	fMandarin	qiu2; you3; zhou1
U+716A.0	fEnglish	 scorch; roast; dry
U+716B.0	fUTF8	煫
U+716B.0	fCNS11643	3-424F
U+716B.0	fGB	5-5221
U+716B.0	fR/S	 86.9
U+716B.0	fTotalStrokes	 13
U+716B.0	fCangjie	FTPO
U+716B.0	fFourCorner	98830
U+716B.0	fCantonese	seoi6
U+716B.0	fMandarin	sui4
U+716C.0	fUTF8	煬
U+716C.0	fCNS11643	1-647B
U+716C.0	fBig5	B7D5
U+716C.0	fGB	1-6C3E
U+716C.0	fR/S	 86.9
U+716C.0	fTotalStrokes	 13
U+716C.0	fCangjie	FAMH
U+716C.0	fFourCorner	96827
U+716C.0	fHakka	 jong2; jong3; rong2; jong5
U+716C.0	fCantonese	joeng4; joeng6
U+716C.0	fMandarin	yang4
U+716C.0	fEnglish	same as &#x70CA;
U+716C.1	fUTF8	煬
U+716C.1	fCNS11643	1-647B
U+716C.1	fBig5	B7D5
U+716C.1	fGB	1-6C3E
U+716C.1	fR/S	 86.9
U+716C.1	fTotalStrokes	 13
U+716C.1	fCangjie	FAMH
U+716C.1	fFourCorner	96827
U+716C.1	fMandarin	yang2
U+716C.1	fEnglish	[1] roaring; blazing (fire) [2] put before the fire
U+716D.0	fUTF8	煭
U+716D.0	fCNS11643	3-4259
U+716D.0	fGB	E-3225
U+716D.0	fR/S	 86.9
U+716D.0	fTotalStrokes	 13
U+716D.0	fCangjie	VNF
U+716D.0	fFourCorner	22330
U+716D.0	fCantonese	lit6
U+716D.0	fMandarin	lie4
U+716E.0	fUTF8	煮
U+716E.0	fCNS11643	1-6054
U+716E.0	fBig5	B54E
U+716E.0	fGB	0-5673
U+716E.0	fR/S	 86.8; 86.9
U+716E.0	fTotalStrokes	 12
U+716E.0	fCangjie	JAF
U+716E.0	fFourCorner	44336
U+716E.0	fHakka	  zu3;  zhu3
U+716E.0	fCantonese	zyu2
U+716E.0	fMandarin	zhu3
U+716E.0	fEnglish	 cook; boil; stew; decoct
U+716F.0	fUTF8	煯
U+716F.0	fCNS11643	3-4249
U+716F.0	fGB	E-3226
U+716F.0	fR/S	  86.9
U+716F.0	fTotalStrokes	 13
U+716F.0	fCangjie	FPPA
U+716F.0	fFourCorner	91860
U+716F.0	fCantonese	gaai1
U+716F.0	fMandarin	jie1
U+7170.0	fUTF8	煰
U+7170.0	fCNS11643	2-4461
U+7170.0	fBig5	DE7B
U+7170.0	fGB	E-3227
U+7170.0	fR/S	 86.9
U+7170.0	fTotalStrokes	 13
U+7170.0	fCangjie	FRRR
U+7170.0	fFourCorner	96860
U+7170.0	fCantonese	cou3
U+7170.0	fMandarin	gao4
U+7171.0	fUTF8	煱
U+7171.0	fCNS11643	3-6559
U+7171.0	fGB	E-3228
U+7171.0	fR/S	 86.9
U+7171.0	fTotalStrokes	 13
U+7171.0	fCangjie	FBBR
U+7171.0	fFourCorner	9782
U+7171.0	fCantonese	wo1
U+7171.0	fMandarin	gua1
U+7171.0	fEnglish	火煱, to instant-boil thin slices of meat and vegetables in a chafing dish at the dining table
U+7172.0	fUTF8	煲
U+7172.0	fCNS11643	2-4459
U+7172.0	fBig5	DE73
U+7172.0	fGB	0-6C52
U+7172.0	fR/S	 86.9
U+7172.0	fTotalStrokes	 13
U+7172.0	fCangjie	ODF
U+7172.0	fFourCorner	26809
U+7172.0	fHakka	  bau1; bo1
U+7172.0	fCantonese	bou1
U+7172.0	fMandarin	bao1; bao4
U+7172.0	fEnglish	[1] heat; boil a saucepan [2] kettle
U+7173.0	fUTF8	煳
U+7173.0	fGB	0-6C4E
U+7173.0	fR/S	 86.9
U+7173.0	fTotalStrokes	 13
U+7173.0	fCangjie	FJRB
U+7173.0	fFourCorner	9782
U+7173.0	fHakka	  fu2; fu3
U+7173.0	fCantonese	wu4
U+7173.0	fMandarin	hu2
U+7173.0	fEnglish	be burned; char (in cooking)
U+7174.0	fUTF8	煴
U+7174.0	fCNS11643	3-4253
U+7174.0	fGB	3-5034
U+7174.0	fR/S	 86.9
U+7174.0	fTotalStrokes	 13
U+7174.0	fCangjie	FABT
U+7174.0	fFourCorner	96810
U+7174.0	fCantonese	wan1; wan3; wan6
U+7174.0	fMandarin	yun2; yun1; yun4; wen3
U+7174.0	fEnglish	sultriness, stuffiness
U+7175.0	fUTF8	煵
U+7175.0	fCNS11643	3-6557
U+7175.0	fGB	E-3229
U+7175.0	fR/S	 86.9
U+7175.0	fTotalStrokes	 13
U+7175.0	fCangjie	FJBJ
U+7175.0	fFourCorner	9482
U+7175.0	fCantonese	naam4; laam4 (also)
U+7175.0	fMandarin	xia1
U+7176.0	fUTF8	煶
U+7176.0	fCNS11643	3-6558
U+7176.0	fGB	E-322A
U+7176.0	fR/S	 86.9
U+7176.0	fTotalStrokes	 13
U+7176.0	fCangjie	FAMO
U+7176.0	fFourCorner	9688
U+7176.0	fCantonese	zik6
U+7177.0	fUTF8	煷
U+7177.0	fCNS11643	3-6556
U+7177.0	fGB	E-322B
U+7177.0	fR/S	 86.9
U+7177.0	fTotalStrokes	 13
U+7177.0	fCangjie	FYRU
U+7177.0	fFourCorner	9081
U+7177.0	fCantonese	fo2
U+7178.0	fUTF8	煸
U+7178.0	fCNS11643	2-445A
U+7178.0	fBig5	DE74
U+7178.0	fGB	0-6C54
U+7178.0	fR/S	 86.9
U+7178.0	fTotalStrokes	 13
U+7178.0	fCangjie	FISB
U+7178.0	fFourCorner	92827
U+7178.0	fCantonese	bin1 (also); pin1
U+7178.0	fMandarin	bian1
U+7179.0	fUTF8	煹
U+7179.0	fCNS11643	4-4457
U+7179.0	fGB	E-322C
U+7179.0	fR/S	  86.10
U+7179.0	fTotalStrokes	 14
U+7179.0	fCangjie	FTTB
U+7179.0	fFourCorner	95840
U+7179.0	fCantonese	gau3
U+7179.0	fMandarin	gou4
U+717A.0	fUTF8	煺
U+717A.0	fCNS11643	4-445B
U+717A.0	fGB	0-6C55
U+717A.0	fR/S	 86.9
U+717A.0	fTotalStrokes	 14; 13
U+717A.0	fCangjie	FYAV
U+717A.0	fFourCorner	97830
U+717A.0	fHakka	 tui5
U+717A.0	fCantonese	teoi3
U+717A.0	fMandarin	tui4; tui1
U+717B.0	fUTF8	煻
U+717B.0	fCNS11643	2-4B67
U+717B.0	fBig5	E2C1
U+717B.0	fGB	3-5044
U+717B.0	fR/S	 86.10
U+717B.0	fTotalStrokes	 14
U+717B.0	fCangjie	FILR
U+717B.0	fFourCorner	90867
U+717B.0	fHakka	  tong2; tong3
U+717B.0	fCantonese	tong4
U+717B.0	fMandarin	tang2
U+717B.0	fEnglish	 warm toast
U+717C.0	fUTF8	煼
U+717C.0	fCNS11643	3-476F
U+717C.0	fGB	5-5146
U+717C.0	fR/S	 86.10
U+717C.0	fTotalStrokes	 14
U+717C.0	fCangjie	FPUU
U+717C.0	fFourCorner	97820
U+717C.0	fHakka	  cau3
U+717C.0	fCantonese	caau2
U+717C.0	fMandarin	chao3
U+717C.0	fEnglish	[1] roast [2] fry
U+717D.0	fUTF8	煽
U+717D.0	fCNS11643	1-695B
U+717D.0	fBig5	BAB4
U+717D.0	fGB	0-493F
U+717D.0	fR/S	 86.10
U+717D.0	fTotalStrokes	 14
U+717D.0	fCangjie	FHSM
U+717D.0	fFourCorner	93827
U+717D.0	fHakka	 sen5; shan5; san5; shen5
U+717D.0	fCantonese	sin3
U+717D.0	fMandarin	shan4; shan1
U+717D.0	fEnglish	[1] stir up; incite; agitate; provoke; instigate; fan [2] flaming; blazing; flame
U+717E.0	fUTF8	煾
U+717E.0	fCNS11643	3-476E
U+717E.0	fGB	E-322D
U+717E.0	fR/S	 86.10
U+717E.0	fTotalStrokes	 14
U+717E.0	fCangjie	FWKP
U+717E.0	fFourCorner	96830
U+717E.0	fCantonese	jan1
U+717E.0	fMandarin	en1; yun1
U+717F.0	fUTF8	煿
U+717F.0	fCNS11643	3-476C
U+717F.0	fGB	5-5225
U+717F.0	fR/S	  86.10
U+717F.0	fTotalStrokes	 14
U+717F.0	fCangjie	FIBI
U+717F.0	fFourCorner	93840
U+717F.0	fCantonese	bok3
U+717F.0	fMandarin	bo2
U+7180.0	fUTF8	熀
U+7180.0	fCNS11643	2-4B63
U+7180.0	fBig5	E2BD
U+7180.0	fGB	3-5040
U+7180.0	fR/S	 86.10
U+7180.0	fTotalStrokes	 14
U+7180.0	fCangjie	FAFU
U+7180.0	fFourCorner	96811
U+7180.0	fHakka	 ngiap7;  fong3
U+7180.0	fCantonese	fong2; nip6
U+7180.0	fMandarin	huang3
U+7180.0	fEnglish	 the blaze of fire dazzling
U+7181.0	fUTF8	熁
U+7181.0	fCNS11643	2-4B69
U+7181.0	fBig5	E2C3
U+7181.0	fGB	3-5046
U+7181.0	fR/S	 86.10
U+7181.0	fTotalStrokes	 14
U+7181.0	fCangjie	FKSB
U+7181.0	fFourCorner	94827
U+7181.0	fHakka	 hiap7
U+7181.0	fCantonese	hip3
U+7181.0	fMandarin	xie2
U+7182.0	fUTF8	熂
U+7182.0	fCNS11643	2-4B65
U+7182.0	fBig5	E2BF
U+7182.0	fGB	3-4F54
U+7182.0	fR/S	 86.10
U+7182.0	fTotalStrokes	 14
U+7182.0	fCangjie	FOND
U+7182.0	fFourCorner	98817
U+7182.0	fCantonese	hei3
U+7182.0	fMandarin	xi4
U+7183.0	fUTF8	熃
U+7183.0	fCNS11643	3-4770
U+7183.0	fGB	5-522D
U+7183.0	fR/S	 86.10; 86.11
U+7183.0	fTotalStrokes	 14; 15
U+7183.0	fCangjie	NSF
U+7183.0	fFourCorner	18800
U+7183.0	fCantonese	mou6
U+7183.0	fMandarin	wu4
U+7184.0	fUTF8	熄
U+7184.0	fCNS11643	1-695D
U+7184.0	fBig5	BAB6
U+7184.0	fGB	0-4F28
U+7184.0	fR/S	 86.10
U+7184.0	fTotalStrokes	 14
U+7184.0	fCangjie	FHUP
U+7184.0	fFourCorner	96830
U+7184.0	fHakka	  sit7
U+7184.0	fCantonese	sik1
U+7184.0	fMandarin	xi1; xi2
U+7184.0	fEnglish	[1] put out (light); extinguish (fire) [2] quash; destroy; obliterate
U+7185.0	fUTF8	熅
U+7185.0	fCNS11643	2-4B64
U+7185.0	fBig5	E2BE
U+7185.0	fGB	E-322E
U+7185.0	fR/S	 86.10
U+7185.0	fTotalStrokes	 14
U+7185.0	fCangjie	FWOT
U+7185.0	fFourCorner	96817
U+7185.0	fHakka	 wun1; wun6; vun1; vut7; vun5
U+7185.0	fCantonese	wan1; wan2; wat1; wan3; wan6
U+7185.0	fMandarin	yun2; yun4; wen1; wen3; yun1
U+7185.0	fEnglish	smooth out; iron out
U+7186.0	fUTF8	熆
U+7186.0	fCNS11643	2-4B68
U+7186.0	fBig5	E2C2
U+7186.0	fGB	5-5223
U+7186.0	fR/S	 86.10
U+7186.0	fTotalStrokes	 14
U+7186.0	fCangjie	FGIT
U+7186.0	fFourCorner	94817
U+7186.0	fCantonese	hap6
U+7186.0	fMandarin	he2
U+7187.0	fUTF8	熇
U+7187.0	fCNS11643	2-4B60
U+7187.0	fBig5	E2BA
U+7187.0	fGB	3-5042
U+7187.0	fR/S	 86.10
U+7187.0	fTotalStrokes	 14
U+7187.0	fCangjie	FYRB
U+7187.0	fFourCorner	90827
U+7187.0	fHakka	  hok7; kok7;  hau1; hok8; kau3
U+7187.0	fCantonese	huk6; hok3 (also); kok3 (also); haau2; hok6; hiu1
U+7187.0	fMandarin	he4; hu4; kao3; kao4; xiao1
U+7187.0	fEnglish	bake
U+7188.0	fUTF8	熈
U+7188.0	fCNS11643	3-4772
U+7188.0	fGB	E-322F
U+7188.0	fR/S	 86.10
U+7188.0	fTotalStrokes	 14
U+7188.0	fCangjie	HUF
U+7188.0	fFourCorner	27330
U+7188.0	fHakka	 hi1
U+7188.0	fCantonese	hei1
U+7188.0	fMandarin	xi1
U+7188.0	fEnglish	bright; splendid; glorious
U+7189.0	fUTF8	熉
U+7189.0	fCNS11643	2-4B62
U+7189.0	fBig5	E2BC
U+7189.0	fGB	3-4F79
U+7189.0	fR/S	 86.10
U+7189.0	fTotalStrokes	 14
U+7189.0	fCangjie	FRBC
U+7189.0	fFourCorner	96886
U+7189.0	fCantonese	jyun4; wan4
U+7189.0	fMandarin	yun2; yun3
U+718A.0	fUTF8	熊
U+718A.0	fCNS11643	1-695C
U+718A.0	fBig5	BAB5
U+718A.0	fGB	0-505C
U+718A.0	fR/S	 86.10
U+718A.0	fTotalStrokes	 14
U+718A.0	fCangjie	IPF
U+718A.0	fFourCorner	21331
U+718A.0	fHakka	  jung2; ngiung2; jung3; rung2
U+718A.0	fCantonese	hung4
U+718A.0	fMandarin	xiong2
U+718A.0	fEnglish	[1] the bear [2] brilliant; shining bright [3] a Chinese family name
U+718B.0	fUTF8	熋
U+718B.0	fCNS11643	3-476A
U+718B.0	fGB	5-522C
U+718B.0	fR/S	  86.10
U+718B.0	fTotalStrokes	 14
U+718B.0	fCangjie	IPF
U+718B.0	fFourCorner	21800
U+718B.0	fCantonese	noi4; loi4 (also)
U+718B.0	fMandarin	nai2
U+718C.0	fUTF8	熌
U+718C.0	fCNS11643	3-476D
U+718C.0	fGB	E-3230
U+718C.0	fR/S	 86.10
U+718C.0	fTotalStrokes	 14
U+718C.0	fCangjie	FANO
U+718C.0	fFourCorner	97820
U+718C.0	fHakka	 sham3; jam3
U+718C.0	fCantonese	sim3
U+718C.0	fMandarin	shan4; shan3
U+718D.0	fUTF8	熍
U+718D.0	fCNS11643	3-476B
U+718D.0	fGB	E-3231
U+718D.0	fR/S	 86.10
U+718D.0	fTotalStrokes	 14
U+718D.0	fCangjie	FJRR
U+718D.0	fFourCorner	93860
U+718D.0	fCantonese	hung3
U+718D.0	fMandarin	qiong1
U+718E.0	fUTF8	熎
U+718E.0	fCNS11643	3-4768
U+718E.0	fGB	E-3232
U+718E.0	fR/S	 86.10
U+718E.0	fTotalStrokes	 14
U+718E.0	fCangjie	FBOU
U+718E.0	fFourCorner	97870
U+718E.0	fCantonese	jiu3
U+718E.0	fMandarin	yao4
U+718F.0	fUTF8	熏
U+718F.0	fCNS11643	2-4B66
U+718F.0	fBig5	E2C0
U+718F.0	fGB	0-512C
U+718F.0	fR/S	 86.10
U+718F.0	fTotalStrokes	 14
U+718F.0	fCangjie	HGF
U+718F.0	fFourCorner	20331
U+718F.0	fHakka	 hiun1; hun1; fun1; fun5
U+718F.0	fCantonese	fan1; fan3
U+718F.0	fMandarin	xun1; xun4
U+718F.0	fEnglish	[1] smoke; burn; smoked (meat, fish, etc.) [2] assail nostrils [3] warm; mild [4] move; touch [5] same as $ [6] scent; fumigate
U+7190.0	fUTF8	熐
U+7190.0	fCNS11643	2-4B61
U+7190.0	fBig5	E2BB
U+7190.0	fGB	E-3233
U+7190.0	fR/S	 86.10
U+7190.0	fTotalStrokes	 14
U+7190.0	fCangjie	FBAC
U+7190.0	fFourCorner	97880
U+7190.0	fCantonese	mik6
U+7190.0	fMandarin	mi4; ming2
U+7191.0	fUTF8	熑
U+7191.0	fCNS11643	3-4767
U+7191.0	fGB	3-5045
U+7191.0	fR/S	  86.10
U+7191.0	fTotalStrokes	 14
U+7191.0	fCangjie	FTXC
U+7191.0	fFourCorner	98830
U+7191.0	fCantonese	lim4
U+7191.0	fMandarin	lian2
U+7192.0	fUTF8	熒
U+7192.0	fCNS11643	1-695E
U+7192.0	fBig5	BAB7
U+7192.0	fGB	1-532B
U+7192.0	fR/S	 86.10
U+7192.0	fTotalStrokes	 14
U+7192.0	fCangjie	FFBF
U+7192.0	fFourCorner	99809
U+7192.0	fHakka	 jin2; rin2
U+7192.0	fCantonese	jing4
U+7192.0	fMandarin	ying2; hong2; xing2; jyong3; jiong3; xing1; ying3; ying4
U+7192.0	fEnglish	[1] bright; shining; luminous [2] dazzling; glittering; sparkling; glimmering [3] doubt; suspect
U+7193.0	fUTF8	熓
U+7193.0	fCNS11643	3-655B
U+7193.0	fGB	E-3234
U+7193.0	fR/S	 86.10
U+7193.0	fTotalStrokes	 14
U+7193.0	fCangjie	FHRF
U+7193.0	fFourCorner	9782
U+7193.0	fCantonese	wat1
U+7193.0	fMandarin	wen4; wu3
U+7193.0	fEnglish	the bank a fire
U+7194.0	fUTF8	熔
U+7194.0	fCNS11643	1-6959
U+7194.0	fBig5	BAB2
U+7194.0	fGB	0-485B
U+7194.0	fR/S	 86.10
U+7194.0	fTotalStrokes	 14
U+7194.0	fCangjie	FJCR
U+7194.0	fFourCorner	93868
U+7194.0	fHakka	 jung2
U+7194.0	fCantonese	jung4
U+7194.0	fMandarin	rong2
U+7194.0	fEnglish	[1] melt; smelt; weld metals; fuse metals [2] mold; die [3] spear-like weapon
U+7195.0	fUTF8	熕
U+7195.0	fCNS11643	F-494B
U+7195.0	fGB	E-3235
U+7195.0	fR/S	 86.10
U+7195.0	fTotalStrokes	 14
U+7195.0	fCangjie	XFMBC
U+7195.0	fCantonese	gung3
U+7196.0	fUTF8	熖
U+7196.0	fCNS11643	3-655A
U+7196.0	fGB	E-3236
U+7196.0	fR/S	 86.10
U+7196.0	fTotalStrokes	 14
U+7196.0	fCangjie	FBHX
U+7196.0	fFourCorner	9287
U+7196.0	fCantonese	jim6
U+7196.0	fEnglish	same as &#x71C4;
U+7197.0	fUTF8	熗
U+7197.0	fCNS11643	2-4B6A
U+7197.0	fBig5	E2C4
U+7197.0	fGB	1-6C41
U+7197.0	fR/S	 86.10
U+7197.0	fTotalStrokes	 14
U+7197.0	fCangjie	FOIR
U+7197.0	fFourCorner	98861
U+7197.0	fCantonese	coeng3
U+7197.0	fMandarin	qiang4
U+7197.0	fEnglish	[1] a way of cooking in which foods (especially bivalves and some vegetables) are eaten right after being brought to a boil [2] fry something quickly in hot oil, then cook it with sauce and water [3] same as $
U+7198.0	fUTF8	熘
U+7198.0	fCNS11643	F-495F
U+7198.0	fGB	0-6C56
U+7198.0	fR/S	 86.10
U+7198.0	fTotalStrokes	 14
U+7198.0	fCangjie	FHHW
U+7198.0	fFourCorner	9786
U+7198.0	fCantonese	lau6; liu1 (also); lau4 (also)
U+7198.0	fMandarin	liu1; liu4
U+7198.0	fEnglish	 steam
U+7199.0	fUTF8	熙
U+7199.0	fCNS11643	1-695A
U+7199.0	fBig5	BAB3
U+7199.0	fGB	0-4E75
U+7199.0	fR/S	 86.10; 86.9
U+7199.0	fTotalStrokes	 14
U+7199.0	fCangjie	SUF
U+7199.0	fFourCorner	77331
U+7199.0	fHakka	  hi1; gi1
U+7199.0	fCantonese	hei1
U+7199.0	fMandarin	xi1; yi2
U+7199.0	fEnglish	[1] bright and brilliant; splendid; glorious [2] expansive; spacious [3] flourishing; prosperous; booming [4] peaceful and happy
U+719A.0	fUTF8	熚
U+719A.0	fCNS11643	2-516F
U+719A.0	fBig5	E667
U+719A.0	fGB	3-4F6B
U+719A.0	fR/S	 86.11
U+719A.0	fTotalStrokes	 15
U+719A.0	fCangjie	FWTJ
U+719A.0	fFourCorner	96054
U+719A.0	fHakka	 bit8; bit7
U+719A.0	fCantonese	bat1; bit1
U+719A.0	fMandarin	bi4
U+719A.0	fEnglish	fiery; blazing; scorching
U+719B.0	fUTF8	熛
U+719B.0	fCNS11643	2-516C
U+719B.0	fBig5	E664
U+719B.0	fGB	3-504B
U+719B.0	fR/S	 86.11
U+719B.0	fTotalStrokes	 15
U+719B.0	fCangjie	FMWF
U+719B.0	fFourCorner	91891
U+719B.0	fHakka	 biau1
U+719B.0	fCantonese	biu1
U+719B.0	fMandarin	biao1; piao1
U+719B.0	fEnglish	blaze
U+719C.0	fUTF8	熜
U+719C.0	fCNS11643	2-5178
U+719C.0	fBig5	E670
U+719C.0	fGB	3-504D
U+719C.0	fR/S	 86.11
U+719C.0	fTotalStrokes	 15
U+719C.0	fCangjie	FHWP
U+719C.0	fFourCorner	96030
U+719C.0	fCantonese	zung2; cung1
U+719C.0	fMandarin	zong3; cong1
U+719D.0	fUTF8	熝
U+719D.0	fCNS11643	2-5172
U+719D.0	fBig5	E66A
U+719D.0	fGB	5-5237
U+719D.0	fR/S	 86.11
U+719D.0	fTotalStrokes	 15
U+719D.0	fCangjie	FIXP
U+719D.0	fFourCorner	90811
U+719D.0	fHakka	  luk8
U+719D.0	fCantonese	luk6
U+719D.0	fMandarin	lu4
U+719D.0	fEnglish	 scald (by steam)
U+719E.0	fUTF8	熞
U+719E.0	fCNS11643	2-5174
U+719E.0	fBig5	E66C
U+719E.0	fGB	5-5160
U+719E.0	fR/S	 86.11
U+719E.0	fTotalStrokes	 15
U+719E.0	fCangjie	FSEG
U+719E.0	fFourCorner	97014
U+719E.0	fCantonese	gin1
U+719E.0	fMandarin	jian1
U+719F.0	fUTF8	熟
U+719F.0	fCNS11643	1-6D5E
U+719F.0	fBig5	BCF4
U+719F.0	fGB	0-4A6C
U+719F.0	fR/S	 86.11
U+719F.0	fTotalStrokes	 15
U+719F.0	fCangjie	YIF
U+719F.0	fFourCorner	05331
U+719F.0	fHakka	  shuk8;  suk8;  siuk7
U+719F.0	fCantonese	suk6
U+719F.0	fMandarin	shu2; shou2
U+719F.0	fEnglish	[1] well done; cooked (as opposed to raw); prepared; processed [2] ripe (fruit); ripen [3] very familiar with; well versed; experienced; conversant [4] careful or painstaking survey, study, inspection, etc. [5] deep sleep; sound sleep
U+71A0.0	fUTF8	熠
U+71A0.0	fCNS11643	2-516E
U+71A0.0	fBig5	E666
U+71A0.0	fGB	0-6C5A
U+71A0.0	fR/S	 86.11
U+71A0.0	fTotalStrokes	 15
U+71A0.0	fCangjie	FSMA
U+71A0.0	fFourCorner	97862
U+71A0.0	fHakka	 sip8; jip8
U+71A0.0	fCantonese	jap1; saap6; jap6
U+71A0.0	fMandarin	yi4; xi2; yu4
U+71A0.0	fEnglish	bright and brilliant; luminous
U+71A1.0	fUTF8	熡
U+71A1.0	fCNS11643	2-5176
U+71A1.0	fBig5	E66E
U+71A1.0	fGB	E-3237
U+71A1.0	fR/S	 86.11
U+71A1.0	fTotalStrokes	 15
U+71A1.0	fCangjie	FLLV
U+71A1.0	fFourCorner	95844
U+71A1.0	fHakka	 liu2
U+71A1.0	fCantonese	lau4
U+71A1.0	fMandarin	lou2
U+71A2.0	fUTF8	熢
U+71A2.0	fCNS11643	3-4D2A
U+71A2.0	fGB	E-3238
U+71A2.0	fR/S	 86.11
U+71A2.0	fTotalStrokes	 15
U+71A2.0	fCangjie	FYHJ
U+71A2.0	fFourCorner	97830
U+71A2.0	fCantonese	pung4
U+71A2.0	fMandarin	peng2; feng1; beng4
U+71A3.0	fUTF8	熣
U+71A3.0	fCNS11643	3-4D24
U+71A3.0	fGB	5-5233
U+71A3.0	fR/S	 86.11
U+71A3.0	fTotalStrokes	 15
U+71A3.0	fCangjie	FUOG
U+71A3.0	fFourCorner	92810
U+71A3.0	fCantonese	seoi1
U+71A3.0	fMandarin	sui1
U+71A4.0	fUTF8	熤
U+71A4.0	fCNS11643	2-5175
U+71A4.0	fBig5	E66D
U+71A4.0	fGB	3-504F
U+71A4.0	fR/S	 86.11
U+71A4.0	fTotalStrokes	 15
U+71A4.0	fCangjie	FSMT
U+71A4.0	fFourCorner	97818
U+71A4.0	fCantonese	jik6; jap1
U+71A4.0	fMandarin	yi4
U+71A5.0	fUTF8	熥
U+71A5.0	fCNS11643	2-5173
U+71A5.0	fBig5	E66B
U+71A5.0	fGB	3-5047
U+71A5.0	fR/S	 86.11
U+71A5.0	fTotalStrokes	 15
U+71A5.0	fCangjie	FYNB
U+71A5.0	fFourCorner	97832
U+71A5.0	fCantonese	tung1
U+71A5.0	fMandarin	tong1; teng1
U+71A6.0	fUTF8	熦
U+71A6.0	fCNS11643	3-4D29
U+71A6.0	fGB	5-5232
U+71A6.0	fR/S	 86.10; 86.11
U+71A6.0	fTotalStrokes	 15; 14
U+71A6.0	fCangjie	FFOG
U+71A6.0	fFourCorner	99810
U+71A6.0	fCantonese	zoek3
U+71A6.0	fMandarin	jue2
U+71A7.0	fUTF8	熧
U+71A7.0	fCNS11643	2-5179
U+71A7.0	fBig5	E671
U+71A7.0	fGB	3-4F58
U+71A7.0	fR/S	 86.11
U+71A7.0	fTotalStrokes	 15
U+71A7.0	fCangjie	HOF
U+71A7.0	fFourCorner	28808
U+71A7.0	fCantonese	zung1
U+71A7.0	fMandarin	zong1
U+71A8.0	fUTF8	熨
U+71A8.0	fCNS11643	1-6D61
U+71A8.0	fBig5	BCF7
U+71A8.0	fGB	0-6C59
U+71A8.0	fR/S	 86.11
U+71A8.0	fTotalStrokes	 15
U+71A8.0	fCangjie	SIF
U+71A8.0	fFourCorner	74809
U+71A8.0	fHakka	 tong5; tung5; jun5; wui3; wi3; run5; vui5; wut7
U+71A8.0	fCantonese	tong3; wan6 (also); wat1; wai3
U+71A8.0	fMandarin	yun4; wei4
U+71A8.0	fEnglish	 iron (clothes or cloth)
U+71A8.1	fUTF8	熨
U+71A8.1	fCNS11643	1-6D61
U+71A8.1	fBig5	BCF7
U+71A8.1	fGB	0-6C59
U+71A8.1	fR/S	 86.11
U+71A8.1	fTotalStrokes	 15
U+71A8.1	fCangjie	SIF
U+71A8.1	fFourCorner	74809
U+71A8.1	fMandarin	yu4
U+71A8.1	fEnglish	 settle (matters)
U+71A9.0	fUTF8	熩
U+71A9.0	fCNS11643	2-5170
U+71A9.0	fBig5	E668
U+71A9.0	fGB	5-5239
U+71A9.0	fR/S	 86.11
U+71A9.0	fTotalStrokes	 15
U+71A9.0	fCangjie	FHSU
U+71A9.0	fFourCorner	92817
U+71A9.0	fCantonese	wu6
U+71A9.0	fMandarin	hu4
U+71AA.0	fUTF8	熪
U+71AA.0	fCNS11643	2-5177
U+71AA.0	fBig5	E66F
U+71AA.0	fGB	5-5234
U+71AA.0	fR/S	 86.11
U+71AA.0	fTotalStrokes	 15
U+71AA.0	fCangjie	FHDN
U+71AA.0	fFourCorner	97827
U+71AA.0	fCantonese	ji4
U+71AA.0	fMandarin	yi2
U+71AB.0	fUTF8	熫
U+71AB.0	fCNS11643	3-4D28
U+71AB.0	fGB	E-3239
U+71AB.0	fR/S	 86.11; 86.10
U+71AB.0	fTotalStrokes	 15; 14
U+71AB.0	fCangjie	FITF
U+71AB.0	fFourCorner	90830
U+71AB.0	fCantonese	zek3
U+71AB.0	fMandarin	zhi4
U+71AC.0	fUTF8	熬
U+71AC.0	fCNS11643	1-6D5F
U+71AC.0	fBig5	BCF5
U+71AC.0	fGB	0-303E
U+71AC.0	fR/S	 86.11
U+71AC.0	fTotalStrokes	 15
U+71AC.0	fCangjie	GKF
U+71AC.0	fFourCorner	58334
U+71AC.0	fHakka	 ngau3; ngau5
U+71AC.0	fCantonese	ngaau4; ngou4 (also); aau4 (also); ou4 (also); ngau4; au4
U+71AC.0	fMandarin	ao2
U+71AC.0	fEnglish	[1] extract (oil, etc.) by applying heat [2] cook; stew; boil; simmer; decoct [3] endure with perseverance; suffer with patience (an ordeal, etc.); sustain [4] endure; bear; suffer; stand
U+71AC.1	fUTF8	熬
U+71AC.1	fCNS11643	1-6D5F
U+71AC.1	fBig5	BCF5
U+71AC.1	fGB	0-303E
U+71AC.1	fR/S	 86.11
U+71AC.1	fTotalStrokes	 15
U+71AC.1	fCangjie	GKF
U+71AC.1	fFourCorner	58334
U+71AC.1	fHakka	 ngau2
U+71AC.1	fMandarin	ao1
U+71AC.1	fEnglish	[1] cook; stew [2] be worn down by worries, cares; discouraged or despondent; dejected
U+71AD.0	fUTF8	熭
U+71AD.0	fCNS11643	3-4D25
U+71AD.0	fGB	3-5048
U+71AD.0	fR/S	 86.11
U+71AD.0	fTotalStrokes	 15
U+71AD.0	fCangjie	QJSMF
U+71AD.0	fFourCorner	55800
U+71AD.0	fCantonese	wai6
U+71AD.0	fMandarin	wei4
U+71AE.0	fUTF8	熮
U+71AE.0	fCNS11643	3-4D22
U+71AE.0	fGB	3-504E
U+71AE.0	fR/S	  86.11
U+71AE.0	fTotalStrokes	 15
U+71AE.0	fCangjie	FSMH
U+71AE.0	fFourCorner	97820
U+71AE.0	fHakka	 liu3;  liau2
U+71AE.0	fCantonese	lau5
U+71AE.0	fMandarin	liao2
U+71AE.0	fEnglish	[1] singe [2] burn; scorch
U+71AF.0	fUTF8	熯
U+71AF.0	fCNS11643	2-516B
U+71AF.0	fBig5	E663
U+71AF.0	fGB	3-5049
U+71AF.0	fR/S	 86.11
U+71AF.0	fTotalStrokes	 15
U+71AF.0	fCangjie	FTLO
U+71AF.0	fFourCorner	94034
U+71AF.0	fHakka	  hon5
U+71AF.0	fCantonese	hon3; naan5; jin5; hon2; hon5
U+71AF.0	fMandarin	han4; han3; ran3
U+71AF.0	fEnglish	by fire
U+71B0.0	fUTF8	熰
U+71B0.0	fCNS11643	2-516D
U+71B0.0	fBig5	E665
U+71B0.0	fGB	3-4F51
U+71B0.0	fR/S	 86.11
U+71B0.0	fTotalStrokes	 15
U+71B0.0	fCangjie	FSRR
U+71B0.0	fFourCorner	91816
U+71B0.0	fHakka	 eu5;  eu1; iu5
U+71B0.0	fCantonese	au1; ngau1 (also)
U+71B0.0	fMandarin	ou1; ou4
U+71B0.0	fEnglish	 great drough heat
U+71B1.0	fUTF8	熱
U+71B1.0	fCNS11643	1-6D60
U+71B1.0	fBig5	BCF6
U+71B1.0	fGB	1-4848
U+71B1.0	fR/S	 86.11
U+71B1.0	fTotalStrokes	 15
U+71B1.0	fCangjie	GIF
U+71B1.0	fFourCorner	45331
U+71B1.0	fHakka	 ngiet8; nget8; gniat8
U+71B1.0	fCantonese	jit6; jit2 (also)
U+71B1.0	fMandarin	re4
U+71B1.0	fEnglish	[1] hot; heat; heated; burning [2] fever [3] earnest; ardent; zealous; enthusiastic; passionate
U+71B2.0	fUTF8	熲
U+71B2.0	fCNS11643	2-516A
U+71B2.0	fBig5	E662
U+71B2.0	fGB	3-6075
U+71B2.0	fR/S	 86.11
U+71B2.0	fTotalStrokes	 15
U+71B2.0	fCangjie	PFMBC
U+71B2.0	fFourCorner	21886
U+71B2.0	fHakka	 kwen3
U+71B2.0	fCantonese	gwing2; wing6; gang2
U+71B2.0	fMandarin	jiong3; jiong1
U+71B2.0	fEnglish	bright
U+71B3.0	fUTF8	熳
U+71B3.0	fCNS11643	2-517A
U+71B3.0	fBig5	E672
U+71B3.0	fGB	0-6C57
U+71B3.0	fR/S	 86.11
U+71B3.0	fTotalStrokes	 15
U+71B3.0	fCangjie	FAWE
U+71B3.0	fFourCorner	96847
U+71B3.0	fCantonese	maan6; maan4
U+71B3.0	fMandarin	man4
U+71B4.0	fUTF8	熴
U+71B4.0	fCNS11643	3-655C
U+71B4.0	fGB	E-323A
U+71B4.0	fR/S	 86.11
U+71B4.0	fTotalStrokes	 15
U+71B4.0	fCangjie	FUAP
U+71B4.0	fFourCorner	9281
U+71B4.0	fCantonese	kwan1
U+71B5.0	fUTF8	熵
U+71B5.0	fCNS11643	2-5171
U+71B5.0	fBig5	E669
U+71B5.0	fGB	0-6C58
U+71B5.0	fR/S	 86.11
U+71B5.0	fTotalStrokes	 15
U+71B5.0	fCangjie	FYCB
U+71B5.0	fFourCorner	90827
U+71B5.0	fCantonese	soeng1
U+71B5.0	fMandarin	shang1; di1
U+71B6.0	fUTF8	熶
U+71B6.0	fCNS11643	3-5162
U+71B6.0	fGB	5-523F
U+71B6.0	fR/S	  86.12
U+71B6.0	fTotalStrokes	 16
U+71B6.0	fCangjie	FASE
U+71B6.0	fFourCorner	96840
U+71B6.0	fHakka	 con5
U+71B6.0	fCantonese	cyun3
U+71B6.0	fMandarin	cuan4
U+71B7.0	fUTF8	熷
U+71B7.0	fCNS11643	3-515A
U+71B7.0	fGB	5-5242
U+71B7.0	fR/S	 86.12
U+71B7.0	fTotalStrokes	 16
U+71B7.0	fCangjie	FCWA
U+71B7.0	fFourCorner	98860
U+71B7.0	fCantonese	zang1
U+71B7.0	fMandarin	zeng1
U+71B8.0	fUTF8	熸
U+71B8.0	fCNS11643	2-5836
U+71B8.0	fBig5	EA4A
U+71B8.0	fGB	3-5052
U+71B8.0	fR/S	 86.12
U+71B8.0	fTotalStrokes	 16
U+71B8.0	fCangjie	FMUA
U+71B8.0	fFourCorner	91861
U+71B8.0	fCantonese	zim1
U+71B8.0	fMandarin	jian1
U+71B8.0	fEnglish	put out
U+71B9.0	fUTF8	熹
U+71B9.0	fCNS11643	1-713B
U+71B9.0	fBig5	BF51
U+71B9.0	fGB	0-6C64
U+71B9.0	fR/S	 86.12
U+71B9.0	fTotalStrokes	 16
U+71B9.0	fCangjie	GRTF
U+71B9.0	fFourCorner	40336
U+71B9.0	fHakka	 hi1
U+71B9.0	fCantonese	hei1
U+71B9.0	fMandarin	xi1
U+71B9.0	fEnglish	[1] faint sunlight; dim light dawn [2] give out faint light
U+71BA.0	fUTF8	熺
U+71BA.0	fCNS11643	3-5160
U+71BA.0	fGB	5-523B
U+71BA.0	fR/S	  86.12
U+71BA.0	fTotalStrokes	 16
U+71BA.0	fCangjie	FGRR
U+71BA.0	fFourCorner	94860
U+71BA.0	fHakka	  hi1
U+71BA.0	fCantonese	hei1
U+71BA.0	fMandarin	xi1
U+71BA.0	fEnglish	 dim light; glimmer warm
U+71BB.0	fUTF8	熻
U+71BB.0	fCNS11643	3-515C
U+71BB.0	fGB	3-5054
U+71BB.0	fR/S	 86.12
U+71BB.0	fTotalStrokes	 16
U+71BB.0	fCangjie	FORM
U+71BB.0	fFourCorner	98820
U+71BB.0	fHakka	  hip7
U+71BB.0	fCantonese	jap1
U+71BB.0	fMandarin	xi1
U+71BB.0	fEnglish	 heat roast burn
U+71BC.0	fUTF8	熼
U+71BC.0	fCNS11643	2-5841
U+71BC.0	fBig5	EA55
U+71BC.0	fGB	3-504C
U+71BC.0	fR/S	 86.12
U+71BC.0	fTotalStrokes	 16
U+71BC.0	fCangjie	FWTC
U+71BC.0	fFourCorner	96881
U+71BC.0	fCantonese	jik6
U+71BC.0	fMandarin	yi4
U+71BD.0	fUTF8	熽
U+71BD.0	fCNS11643	2-583F
U+71BD.0	fBig5	EA53
U+71BD.0	fGB	3-502F
U+71BD.0	fR/S	 86.12
U+71BD.0	fTotalStrokes	 16
U+71BD.0	fCangjie	FLX
U+71BD.0	fFourCorner	95827
U+71BD.0	fCantonese	siu3
U+71BD.0	fMandarin	xiao4
U+71BE.0	fUTF8	熾
U+71BE.0	fCNS11643	1-7135
U+71BE.0	fBig5	BF4B
U+71BE.0	fGB	1-3363
U+71BE.0	fR/S	 86.12
U+71BE.0	fTotalStrokes	 16
U+71BE.0	fCangjie	FYIA
U+71BE.0	fFourCorner	93850
U+71BE.0	fHakka	 zhit7; zhi5; tsiit7; ci5; zit7; zih5; tje5
U+71BE.0	fCantonese	ci3
U+71BE.0	fMandarin	chi4
U+71BE.0	fEnglish	[1] vigorous; energetic; intense [2] burning hot; flaming
U+71BF.0	fUTF8	熿
U+71BF.0	fCNS11643	2-5835
U+71BF.0	fBig5	EA49
U+71BF.0	fGB	5-522F
U+71BF.0	fR/S	 86.12
U+71BF.0	fTotalStrokes	 16
U+71BF.0	fCangjie	FTLC
U+71BF.0	fFourCorner	94886
U+71BF.0	fCantonese	wong4; fong2
U+71BF.0	fMandarin	huang2; huang3
U+71C0.0	fUTF8	燀
U+71C0.0	fCNS11643	2-5838
U+71C0.0	fBig5	EA4C
U+71C0.0	fGB	3-502C
U+71C0.0	fR/S	 86.12
U+71C0.0	fTotalStrokes	 16
U+71C0.0	fCangjie	FRRJ
U+71C0.0	fFourCorner	96856
U+71C0.0	fHakka	 shan5; sen5; san5; shen5
U+71C0.0	fCantonese	cin2; daan6; zin2
U+71C0.0	fMandarin	chan3; dan3; chan1
U+71C0.0	fEnglish	 make a fire blaze
U+71C1.0	fUTF8	燁
U+71C1.0	fCNS11643	2-5839
U+71C1.0	fBig5	EA4D
U+71C1.0	fGB	1-6C47
U+71C1.0	fR/S	 86.12; 86.10
U+71C1.0	fTotalStrokes	 16
U+71C1.0	fCangjie	FTMJ
U+71C1.0	fFourCorner	94854
U+71C1.0	fHakka	 jap7
U+71C1.0	fCantonese	jip6
U+71C1.0	fMandarin	ye4; yi4
U+71C1.0	fEnglish	blazing; glorious; splendid
U+71C2.0	fUTF8	燂
U+71C2.0	fCNS11643	2-5834
U+71C2.0	fBig5	EA48
U+71C2.0	fGB	3-5051
U+71C2.0	fR/S	 86.12
U+71C2.0	fTotalStrokes	 16
U+71C2.0	fCangjie	FMWJ
U+71C2.0	fFourCorner	91846
U+71C2.0	fHakka	  tam2; tam3
U+71C2.0	fCantonese	taam4; cim4; cam4
U+71C2.0	fMandarin	qian2; xun2; tan2
U+71C2.0	fEnglish	 smoke; fumes tobacco; opium
U+71C3.0	fUTF8	燃
U+71C3.0	fCNS11643	1-713F
U+71C3.0	fBig5	BF55
U+71C3.0	fGB	0-483C
U+71C3.0	fR/S	 86.12
U+71C3.0	fTotalStrokes	 16
U+71C3.0	fCangjie	FBKF
U+71C3.0	fFourCorner	93833
U+71C3.0	fHakka	  jen2; ren2; jan2
U+71C3.0	fCantonese	jin4; jin6
U+71C3.0	fMandarin	ran2
U+71C3.0	fEnglish	 burn; light; ignite
U+71C4.0	fUTF8	燄
U+71C4.0	fCNS11643	1-7140
U+71C4.0	fBig5	BF56
U+71C4.0	fGB	E-323B
U+71C4.0	fR/S	 86.12
U+71C4.0	fTotalStrokes	 16
U+71C4.0	fCangjie	NXFF
U+71C4.0	fFourCorner	29789
U+71C4.0	fHakka	 jam5; ram6; jam3; jam6
U+71C4.0	fCantonese	jim6; jim4 (also)
U+71C4.0	fMandarin	yan4; jian3; yan2
U+71C4.0	fEnglish	 flame blazing; brilliant
U+71C5.0	fUTF8	燅
U+71C5.0	fCNS11643	2-5833
U+71C5.0	fBig5	EA47
U+71C5.0	fGB	5-523C
U+71C5.0	fR/S	 86.12
U+71C5.0	fTotalStrokes	 16
U+71C5.0	fCangjie	GGFF
U+71C5.0	fFourCorner	49189
U+71C5.0	fCantonese	cam4; cim4 (also)
U+71C5.0	fMandarin	xian2; xun2; qian2
U+71C6.0	fUTF8	燆
U+71C6.0	fCNS11643	2-5842
U+71C6.0	fBig5	EA56
U+71C6.0	fGB	3-4F6C
U+71C6.0	fR/S	 86.12
U+71C6.0	fTotalStrokes	 16
U+71C6.0	fCangjie	FHKB
U+71C6.0	fFourCorner	92827
U+71C6.0	fCantonese	hiu4; hiu1
U+71C6.0	fMandarin	qiao2; xiao1
U+71C7.0	fUTF8	燇
U+71C7.0	fCNS11643	2-583D
U+71C7.0	fBig5	EA51
U+71C7.0	fGB	5-5241
U+71C7.0	fR/S	 86.12
U+71C7.0	fTotalStrokes	 16
U+71C7.0	fCangjie	FTWI
U+71C7.0	fFourCorner	98846
U+71C7.0	fCantonese	zeon3
U+71C7.0	fMandarin	zun4; jun4
U+71C8.0	fUTF8	燈
U+71C8.0	fCNS11643	1-7139
U+71C8.0	fBig5	BF4F
U+71C8.0	fGB	1-3546
U+71C8.0	fR/S	 86.12
U+71C8.0	fTotalStrokes	 16
U+71C8.0	fCangjie	FNOT
U+71C8.0	fFourCorner	92818
U+71C8.0	fHakka	  den1; dien1
U+71C8.0	fCantonese	dang1
U+71C8.0	fMandarin	deng1
U+71C8.0	fEnglish	[1] lantern; lamp; burner [2] Buddha dharma; the Buddhist doctrine [3] valve; tube
U+71C9.0	fUTF8	燉
U+71C9.0	fCNS11643	1-7136
U+71C9.0	fBig5	BF4C
U+71C9.0	fGB	3-5055
U+71C9.0	fR/S	 86.12
U+71C9.0	fTotalStrokes	 16
U+71C9.0	fCangjie	FYDK
U+71C9.0	fFourCorner	98840
U+71C9.0	fHakka	  dun1
U+71C9.0	fCantonese	dan6; deon6 (also)
U+71C9.0	fMandarin	dun4; tun2; tun1
U+71C9.0	fEnglish	[1] warm; heat with fire [2] stew; simmer
U+71C9.1	fUTF8	燉
U+71C9.1	fCNS11643	1-7136
U+71C9.1	fBig5	BF4C
U+71C9.1	fGB	3-5055
U+71C9.1	fR/S	 86.12
U+71C9.1	fTotalStrokes	 16
U+71C9.1	fCangjie	FYDK
U+71C9.1	fFourCorner	98840
U+71C9.1	fMandarin	dun1
U+71C9.1	fEnglish	used in combination for Tunhwang, Kansu
U+71CA.0	fUTF8	燊
U+71CA.0	fCNS11643	2-583C
U+71CA.0	fBig5	EA50
U+71CA.0	fGB	3-4340
U+71CA.0	fR/S	 86.12
U+71CA.0	fTotalStrokes	 16
U+71CA.0	fCangjie	FFFD
U+71CA.0	fFourCorner	90904
U+71CA.0	fHakka	 sin1
U+71CA.0	fCantonese	san1
U+71CA.0	fMandarin	shen1; hua2; sui4
U+71CA.0	fEnglish	vigorous
U+71CB.0	fUTF8	燋
U+71CB.0	fCNS11643	2-583A
U+71CB.0	fBig5	EA4E
U+71CB.0	fGB	3-5053
U+71CB.0	fR/S	 86.12
U+71CB.0	fTotalStrokes	 16
U+71CB.0	fCangjie	FOGF
U+71CB.0	fFourCorner	90831
U+71CB.0	fHakka	 ziau1; zeu1; ziau5
U+71CB.0	fCantonese	ziu1; ciu4
U+71CB.0	fMandarin	jiao1; qiao3; ji1; jiao4; jue2; zhuo1
U+71CB.0	fEnglish	[1] same as $ [2] torch
U+71CB.1	fUTF8	燋
U+71CB.1	fCNS11643	2-583A
U+71CB.1	fBig5	EA4E
U+71CB.1	fGB	3-5053
U+71CB.1	fR/S	 86.12
U+71CB.1	fTotalStrokes	 16
U+71CB.1	fCangjie	FOGF
U+71CB.1	fFourCorner	90831
U+71CB.1	fMandarin	qiao2
U+71CB.1	fEnglish	haggard; worn; emaciated
U+71CC.0	fUTF8	燌
U+71CC.0	fCNS11643	3-5163
U+71CC.0	fGB	5-5173
U+71CC.0	fR/S	  86.12
U+71CC.0	fTotalStrokes	 16
U+71CC.0	fCangjie	FJTC
U+71CC.0	fFourCorner	94880
U+71CC.0	fCantonese	fan4
U+71CC.0	fMandarin	fen1; fen2; ben4
U+71CD.0	fUTF8	燍
U+71CD.0	fCNS11643	4-5123
U+71CD.0	fGB	3-5050
U+71CD.0	fR/S	 86.12
U+71CD.0	fTotalStrokes	 16
U+71CD.0	fCangjie	FTCL
U+71CD.0	fFourCorner	92820
U+71CD.0	fCantonese	si1
U+71CD.0	fMandarin	si1
U+71CE.0	fUTF8	燎
U+71CE.0	fCNS11643	1-713C
U+71CE.0	fBig5	BF52
U+71CE.0	fGB	0-4147
U+71CE.0	fR/S	 86.12
U+71CE.0	fTotalStrokes	 16
U+71CE.0	fCangjie	FKCF
U+71CE.0	fFourCorner	94896
U+71CE.0	fHakka	  liau2; liau5; liau3
U+71CE.0	fCantonese	liu4; liu6; liu5
U+71CE.0	fMandarin	liao2; liao4
U+71CE.0	fEnglish	[1] burn over a wider and wider area [2] set fire to [3] be brilliant
U+71CE.1	fUTF8	燎
U+71CE.1	fCNS11643	1-713C
U+71CE.1	fBig5	BF52
U+71CE.1	fGB	0-4147
U+71CE.1	fR/S	 86.12
U+71CE.1	fTotalStrokes	 16
U+71CE.1	fCangjie	FKCF
U+71CE.1	fFourCorner	94896
U+71CE.1	fMandarin	liao3
U+71CE.1	fEnglish	 singe
U+71CF.0	fUTF8	燏
U+71CF.0	fCNS11643	2-583E
U+71CF.0	fBig5	EA52
U+71CF.0	fGB	3-5057
U+71CF.0	fR/S	 86.12
U+71CF.0	fTotalStrokes	 16
U+71CF.0	fCangjie	FNHB
U+71CF.0	fFourCorner	97827
U+71CF.0	fCantonese	wat6; leot6; jyut6
U+71CF.0	fMandarin	yu4
U+71CF.0	fEnglish	blaze
U+71D0.0	fUTF8	燐
U+71D0.0	fCNS11643	1-7137
U+71D0.0	fBig5	BF4D
U+71D0.0	fGB	E-323C
U+71D0.0	fR/S	 86.12
U+71D0.0	fTotalStrokes	 16
U+71D0.0	fCangjie	FFDQ
U+71D0.0	fFourCorner	99859
U+71D0.0	fHakka	 lin2
U+71D0.0	fCantonese	leon4
U+71D0.0	fMandarin	lin2; lin3; lin4
U+71D0.0	fEnglish	phosphorus
U+71D1.0	fUTF8	燑
U+71D1.0	fCNS11643	3-515E
U+71D1.0	fGB	5-5240
U+71D1.0	fR/S	 86.12
U+71D1.0	fTotalStrokes	 16
U+71D1.0	fCangjie	FYTG
U+71D1.0	fFourCorner	90810
U+71D1.0	fHakka	 tung2
U+71D1.0	fCantonese	tung4
U+71D1.0	fMandarin	tong2
U+71D2.0	fUTF8	燒
U+71D2.0	fCNS11643	1-7138
U+71D2.0	fBig5	BF4E
U+71D2.0	fGB	1-4955
U+71D2.0	fR/S	 86.12
U+71D2.0	fTotalStrokes	 16
U+71D2.0	fCangjie	FGGU
U+71D2.0	fFourCorner	94811
U+71D2.0	fHakka	 sau1;  shau1; seu1; s'eu1
U+71D2.0	fCantonese	siu1
U+71D2.0	fMandarin	shao1; shao4
U+71D2.0	fEnglish	[1] burn [2] stew; roast [3] boil; heat [4] run a fever; have a temperature [5] fever
U+71D3.0	fUTF8	燓
U+71D3.0	fCNS11643	3-5166
U+71D3.0	fGB	5-523E
U+71D3.0	fR/S	  86.12
U+71D3.0	fTotalStrokes	 16
U+71D3.0	fCangjie	DDF
U+71D3.0	fFourCorner	44800
U+71D3.0	fCantonese	fan4
U+71D3.0	fMandarin	fen1; fen2; fen4
U+71D4.0	fUTF8	燔
U+71D4.0	fCNS11643	2-583B
U+71D4.0	fBig5	EA4F
U+71D4.0	fGB	0-6C5C
U+71D4.0	fR/S	 86.12
U+71D4.0	fTotalStrokes	 16
U+71D4.0	fCangjie	FHDW
U+71D4.0	fFourCorner	92869
U+71D4.0	fHakka	  fan1;  pan1
U+71D4.0	fCantonese	faan4
U+71D4.0	fMandarin	fan2; fen2
U+71D4.0	fEnglish	[1] roast meat for offering [2] burn; roast
U+71D5.0	fUTF8	燕
U+71D5.0	fCNS11643	1-713A
U+71D5.0	fBig5	BF50
U+71D5.0	fGB	0-5160
U+71D5.0	fR/S	 86.12
U+71D5.0	fTotalStrokes	 16
U+71D5.0	fCangjie	TLPF
U+71D5.0	fFourCorner	44331
U+71D5.0	fHakka	 jen5; jen1; jan5; jiin5; ren5; ren1
U+71D5.0	fCantonese	jin3; jin1; jin2 (also)
U+71D5.0	fMandarin	yan4; yan3
U+71D5.0	fEnglish	[1] the swallow (bird) [2] comfort; ease [3] feast; enjoy
U+71D5.1	fUTF8	燕
U+71D5.1	fCNS11643	1-713A
U+71D5.1	fBig5	BF50
U+71D5.1	fGB	0-5160
U+71D5.1	fR/S	 86.12
U+71D5.1	fTotalStrokes	 16
U+71D5.1	fCangjie	TLPF
U+71D5.1	fFourCorner	44331
U+71D5.1	fMandarin	yan1
U+71D5.1	fEnglish	a state in what is Hopeh today during the Warring States period
U+71D6.0	fUTF8	燖
U+71D6.0	fCNS11643	2-5837
U+71D6.0	fBig5	EA4B
U+71D6.0	fGB	3-4F72
U+71D6.0	fR/S	 86.12
U+71D6.0	fTotalStrokes	 16
U+71D6.0	fCangjie	FSMI
U+71D6.0	fFourCorner	97846
U+71D6.0	fHakka	  cim2; ciam2
U+71D6.0	fCantonese	cam4
U+71D6.0	fMandarin	xun2; qian2; xian2
U+71D6.0	fEnglish	reheat food
U+71D7.0	fUTF8	燗
U+71D7.0	fCNS11643	7-2F4F
U+71D7.0	fGB	E-323D
U+71D7.0	fR/S	  86.12
U+71D7.0	fTotalStrokes	 16
U+71D7.0	fCangjie	FANB
U+71D7.0	fFourCorner	97820
U+71D7.0	fCantonese	laan6
U+71D7.0	fMandarin	lan4
U+71D7.0	fEnglish	warming sake, to heat sake
U+71D8.0	fUTF8	燘
U+71D8.0	fCNS11643	2-5840
U+71D8.0	fBig5	EA54
U+71D8.0	fGB	5-5167
U+71D8.0	fR/S	 86.12
U+71D8.0	fTotalStrokes	 16
U+71D8.0	fCangjie	FANK
U+71D8.0	fFourCorner	97820
U+71D8.0	fCantonese	mui5; fui2; mui2
U+71D8.0	fMandarin	mei3; hui1
U+71D8.0	fEnglish	to suck or chew without using the teeth
U+71D9.0	fUTF8	燙
U+71D9.0	fCNS11643	1-713D
U+71D9.0	fBig5	BF53
U+71D9.0	fGB	1-4C4C
U+71D9.0	fR/S	 86.12
U+71D9.0	fTotalStrokes	 16
U+71D9.0	fCangjie	EHF
U+71D9.0	fFourCorner	36809
U+71D9.0	fHakka	  tong5; luk8
U+71D9.0	fCantonese	tong3
U+71D9.0	fMandarin	tang4; tang2; dang4
U+71D9.0	fEnglish	[1] scald; burn [2] heat; warm [3] very hot [4] iron (clothes)
U+71DA.0	fUTF8	燚
U+71DA.0	fCNS11643	2-5843
U+71DA.0	fBig5	EA57
U+71DA.0	fGB	3-5056
U+71DA.0	fR/S	 86.12
U+71DA.0	fTotalStrokes	 16
U+71DA.0	fCangjie	FFFF
U+71DA.0	fFourCorner	99889
U+71DA.0	fCantonese	jik6; ngai6
U+71DA.0	fMandarin	yi1
U+71DB.0	fUTF8	燛
U+71DB.0	fCNS11643	2-5844
U+71DB.0	fBig5	EA58
U+71DB.0	fGB	E-323E
U+71DB.0	fR/S	 86.12
U+71DB.0	fTotalStrokes	 16
U+71DB.0	fCangjie	SSF
U+71DB.0	fFourCorner	11809
U+71DB.0	fCantonese	gwing2
U+71DB.0	fMandarin	jing3
U+71DC.0	fUTF8	燜
U+71DC.0	fCNS11643	1-713E
U+71DC.0	fBig5	BF54
U+71DC.0	fGB	1-6C4B
U+71DC.0	fR/S	 86.12
U+71DC.0	fTotalStrokes	 16
U+71DC.0	fCangjie	FANP
U+71DC.0	fFourCorner	97820
U+71DC.0	fHakka	 mun1
U+71DC.0	fCantonese	mun6; man1
U+71DC.0	fMandarin	men4; men1
U+71DC.0	fEnglish	 cook with mild heat in a closed vessel; cook in a casserole; cook over slow fire
U+71DD.0	fUTF8	燝
U+71DD.0	fCNS11643	3-655D
U+71DD.0	fGB	E-323F
U+71DD.0	fR/S	 86.12
U+71DD.0	fTotalStrokes	 16
U+71DD.0	fCangjie	FAYF
U+71DD.0	fFourCorner	9689
U+71DD.0	fCantonese	ging2
U+71DE.0	fUTF8	燞
U+71DE.0	fCNS11643	3-655E
U+71DE.0	fGB	E-3240
U+71DE.0	fR/S	 86.12
U+71DE.0	fTotalStrokes	 16
U+71DE.0	fCangjie	XOOF
U+71DE.0	fFourCorner	2733
U+71DE.0	fCantonese	ziu3
U+71DF.0	fUTF8	營
U+71DF.0	fCNS11643	1-7432
U+71DF.0	fBig5	C0E7
U+71DF.0	fGB	1-532A
U+71DF.0	fR/S	 86.13
U+71DF.0	fTotalStrokes	 17
U+71DF.0	fCangjie	FFBRR
U+71DF.0	fFourCorner	99606
U+71DF.0	fHakka	  jang2; jin2; rang2; jang3; rin2; jin3
U+71DF.0	fCantonese	jing4
U+71DF.0	fMandarin	ying2; xing2; cuo1
U+71DF.0	fEnglish	[1] encampment; military barracks [2] battalion [3] manage; administer; handle; operate; run
U+71E0.0	fUTF8	燠
U+71E0.0	fCNS11643	1-7439
U+71E0.0	fBig5	C0EE
U+71E0.0	fGB	0-6C5B
U+71E0.0	fR/S	 86.13
U+71E0.0	fTotalStrokes	 17
U+71E0.0	fCangjie	FHBK
U+71E0.0	fFourCorner	97834
U+71E0.0	fHakka	  juk7;  au5; o5; ruk7; au3
U+71E0.0	fCantonese	juk1; ou2 (also); ngou2 (also); jyu3; ou3; jau1
U+71E0.0	fMandarin	yu4; ao3; ao4
U+71E0.0	fEnglish	warm
U+71E0.2	fUTF8	燠
U+71E0.2	fCNS11643	1-7439
U+71E0.2	fBig5	C0EE
U+71E0.2	fGB	0-6C5B
U+71E0.2	fR/S	 86.13
U+71E0.2	fTotalStrokes	 17
U+71E0.2	fCangjie	FHBK
U+71E0.2	fFourCorner	97834
U+71E0.2	fMandarin	you1
U+71E0.2	fEnglish	used in combination for to comfort; soothe
U+71E1.0	fUTF8	燡
U+71E1.0	fCNS11643	2-5D48
U+71E1.0	fBig5	ED5C
U+71E1.0	fGB	3-4F68
U+71E1.0	fR/S	 86.13
U+71E1.0	fTotalStrokes	 17
U+71E1.0	fCangjie	FWLJ
U+71E1.0	fFourCorner	96841
U+71E1.0	fCantonese	jik6
U+71E1.0	fMandarin	yi4
U+71E1.0	fEnglish	bright
U+71E2.0	fUTF8	燢
U+71E2.0	fCNS11643	2-5D4E
U+71E2.0	fBig5	ED62
U+71E2.0	fGB	E-3241
U+71E2.0	fR/S	 86.13
U+71E2.0	fTotalStrokes	 17
U+71E2.0	fCangjie	HBF
U+71E2.0	fFourCorner	77809
U+71E2.0	fHakka	 gok7
U+71E2.0	fCantonese	hok6
U+71E2.0	fMandarin	xue2; xiao2
U+71E3.0	fUTF8	燣
U+71E3.0	fCNS11643	3-553C
U+71E3.0	fGB	E-3242
U+71E3.0	fR/S	  86.13
U+71E3.0	fTotalStrokes	 17
U+71E3.0	fCangjie	FYWD
U+71E3.0	fFourCorner	90890
U+71E3.0	fHakka	  lam3; lim1
U+71E3.0	fCantonese	laam4
U+71E3.0	fMandarin	lan2; lan3
U+71E3.0	fEnglish	disappointed
U+71E4.0	fUTF8	燤
U+71E4.0	fCNS11643	2-5D4C
U+71E4.0	fBig5	ED60
U+71E4.0	fGB	3-4F4D
U+71E4.0	fR/S	 86.13
U+71E4.0	fTotalStrokes	 17
U+71E4.0	fCangjie	FTWB
U+71E4.0	fFourCorner	94827
U+71E4.0	fCantonese	taai3; lit6
U+71E4.0	fMandarin	tai4; ta4; lie4
U+71E5.0	fUTF8	燥
U+71E5.0	fCNS11643	1-7435
U+71E5.0	fBig5	C0EA
U+71E5.0	fGB	0-546F
U+71E5.0	fR/S	 86.13
U+71E5.0	fTotalStrokes	 17
U+71E5.0	fCangjie	FRRD
U+71E5.0	fFourCorner	96894
U+71E5.0	fHakka	  zau1; cau5
U+71E5.0	fCantonese	cou3
U+71E5.0	fMandarin	zao4; sao3; sao4
U+71E5.0	fEnglish	[1] dry; parched; arid [2] impatient; restless
U+71E6.0	fUTF8	燦
U+71E6.0	fCNS11643	1-7434
U+71E6.0	fBig5	C0E9
U+71E6.0	fGB	1-3253
U+71E6.0	fR/S	 86.13
U+71E6.0	fTotalStrokes	 17
U+71E6.0	fCangjie	FYED
U+71E6.0	fFourCorner	97894
U+71E6.0	fHakka	 can3;  can5; can6
U+71E6.0	fCantonese	caan3; can3
U+71E6.0	fMandarin	can4
U+71E6.0	fEnglish	illuminating; bright; brilliant; resplendent
U+71E7.0	fUTF8	燧
U+71E7.0	fCNS11643	1-7431
U+71E7.0	fBig5	C0E6
U+71E7.0	fGB	0-6C5D
U+71E7.0	fR/S	 86.13
U+71E7.0	fTotalStrokes	 17
U+71E7.0	fCangjie	FYTO
U+71E7.0	fFourCorner	98833
U+71E7.0	fHakka	 sui5; sui6; sui3
U+71E7.0	fCantonese	seoi6
U+71E7.0	fMandarin	sui4
U+71E7.0	fEnglish	[1] flint [2] beacon; signal fire
U+71E8.0	fUTF8	燨
U+71E8.0	fCNS11643	2-5D4A
U+71E8.0	fBig5	ED5E
U+71E8.0	fGB	E-3243
U+71E8.0	fR/S	 86.13
U+71E8.0	fTotalStrokes	 17
U+71E8.0	fCangjie	FTGI
U+71E8.0	fFourCorner	98853
U+71E8.0	fCantonese	hei1
U+71E8.0	fMandarin	xi1
U+71E9.0	fUTF8	燩
U+71E9.0	fCNS11643	3-553E
U+71E9.0	fGB	3-5059
U+71E9.0	fR/S	 86.13
U+71E9.0	fTotalStrokes	 17
U+71E9.0	fCangjie	FHSK
U+71E9.0	fFourCorner	98840
U+71E9.0	fHakka	 kok7; fok8; fak8
U+71E9.0	fCantonese	kok3
U+71E9.0	fMandarin	que4
U+71E9.0	fEnglish	[1] dry at the fire [2] roast; fry
U+71EA.0	fUTF8	燪
U+71EA.0	fCNS11643	3-5543
U+71EA.0	fGB	5-523D
U+71EA.0	fR/S	  86.13
U+71EA.0	fTotalStrokes	 17
U+71EA.0	fCangjie	FTPP
U+71EA.0	fFourCorner	94830
U+71EA.0	fCantonese	zung2
U+71EA.0	fMandarin	cong1
U+71EB.0	fUTF8	燫
U+71EB.0	fCNS11643	3-5540
U+71EB.0	fGB	3-505A
U+71EB.0	fR/S	 86.13
U+71EB.0	fTotalStrokes	 17
U+71EB.0	fCangjie	FITC
U+71EB.0	fFourCorner	90830
U+71EB.0	fCantonese	lim4
U+71EB.0	fMandarin	lian2
U+71EC.0	fUTF8	燬
U+71EC.0	fCNS11643	1-7437
U+71EC.0	fBig5	C0EC
U+71EC.0	fGB	E-3244
U+71EC.0	fR/S	 86.13
U+71EC.0	fTotalStrokes	 17
U+71EC.0	fCangjie	FHGE
U+71EC.0	fFourCorner	97847
U+71EC.0	fHakka	 kui3;  fui5; fi5; fui3; fi3; kwi3; k'wui3; fui1
U+71EC.0	fCantonese	wai2
U+71EC.0	fMandarin	hui3; huo4
U+71EC.0	fEnglish	[1] blaze; fire [2] destroy by fire; burn away; burn down
U+71ED.0	fUTF8	燭
U+71ED.0	fCNS11643	1-7436
U+71ED.0	fBig5	C0EB
U+71ED.0	fGB	1-5672
U+71ED.0	fR/S	 86.13
U+71ED.0	fTotalStrokes	 17
U+71ED.0	fCangjie	FWLI
U+71ED.0	fFourCorner	96827
U+71ED.0	fHakka	  zuk7;  chuk7; zhuk7
U+71ED.0	fCantonese	zuk1
U+71ED.0	fMandarin	zhu2; zhu4
U+71ED.0	fEnglish	 candle; taper shine; illuminate
U+71EE.0	fUTF8	燮
U+71EE.0	fCNS11643	1-7433
U+71EE.0	fBig5	C0E8
U+71EE.0	fGB	0-5B46
U+71EE.0	fR/S	 86.13
U+71EE.0	fTotalStrokes	 17
U+71EE.0	fCangjie	FFE
U+71EE.0	fFourCorner	99407
U+71EE.0	fHakka	  siet7; set7
U+71EE.0	fCantonese	sit3; sip3
U+71EE.0	fMandarin	xie4
U+71EE.0	fEnglish	 harmonise; blend; harmonious
U+71EF.0	fUTF8	燯
U+71EF.0	fCNS11643	3-5541
U+71EF.0	fGB	3-5058
U+71EF.0	fR/S	  86.13
U+71EF.0	fTotalStrokes	 17
U+71EF.0	fCangjie	FMBI
U+71EF.0	fFourCorner	91830
U+71EF.0	fCantonese	ling4
U+71EF.0	fMandarin	ling2
U+71F0.0	fUTF8	燰
U+71F0.0	fCNS11643	2-5D4D
U+71F0.0	fBig5	ED61
U+71F0.0	fGB	E-3245
U+71F0.0	fR/S	 86.13
U+71F0.0	fTotalStrokes	 17
U+71F0.0	fCangjie	FBBE
U+71F0.0	fFourCorner	92847
U+71F0.0	fCantonese	wui1
U+71F0.0	fMandarin	wei1
U+71F1.0	fUTF8	燱
U+71F1.0	fCNS11643	2-5D49
U+71F1.0	fBig5	ED5D
U+71F1.0	fGB	5-5248
U+71F1.0	fR/S	 86.13
U+71F1.0	fTotalStrokes	 17
U+71F1.0	fCangjie	FYTP
U+71F1.0	fFourCorner	90836
U+71F1.0	fCantonese	ji3; jik1
U+71F1.0	fMandarin	yi4
U+71F2.0	fUTF8	燲
U+71F2.0	fCNS11643	2-5D4B
U+71F2.0	fBig5	ED5F
U+71F2.0	fGB	E-3246
U+71F2.0	fR/S	 86.13
U+71F2.0	fTotalStrokes	 17
U+71F2.0	fCangjie	FKSC
U+71F2.0	fFourCorner	94886
U+71F2.0	fCantonese	hip3
U+71F2.0	fMandarin	xie2
U+71F3.0	fUTF8	燳
U+71F3.0	fCNS11643	3-5542
U+71F3.0	fGB	E-3247
U+71F3.0	fR/S	 86.13
U+71F3.0	fTotalStrokes	 17
U+71F3.0	fCangjie	FARF
U+71F3.0	fFourCorner	97830
U+71F3.0	fCantonese	ziu3
U+71F3.0	fMandarin	zhao4
U+71F4.0	fUTF8	燴
U+71F4.0	fCNS11643	1-7438
U+71F4.0	fBig5	C0ED
U+71F4.0	fGB	1-3B62
U+71F4.0	fR/S	 86.13
U+71F4.0	fTotalStrokes	 17
U+71F4.0	fCangjie	FOMA
U+71F4.0	fFourCorner	98866
U+71F4.0	fHakka	 fui5; fi5; fui3
U+71F4.0	fCantonese	wui6
U+71F4.0	fMandarin	hui4
U+71F4.0	fEnglish	[1] put (a variety of materials) together and cook; braise [2] serve (noodles, rice, etc.) with a topping of meat, vegetables, etc. in gravy
U+71F5.0	fUTF8	燵
U+71F5.0	fCNS11643	F-5A73
U+71F5.0	fGB	E-3248
U+71F5.0	fR/S	 86.13
U+71F5.0	fTotalStrokes	 17
U+71F5.0	fCangjie	FYGQ
U+71F5.0	fFourCorner	9483
U+71F5.0	fCantonese	daat6
U+71F5.0	fEnglish	a foot-warmer
U+71F6.0	fUTF8	燶
U+71F6.0	fGB	1-7C43
U+71F6.0	fR/S	 86.13
U+71F6.0	fTotalStrokes	 17
U+71F6.0	fCangjie	FTWV
U+71F6.0	fFourCorner	9583
U+71F6.0	fCantonese	nung1; lung1 (also)
U+71F6.0	fEnglish	 scorch; burn
U+71F7.0	fUTF8	燷
U+71F7.0	fGB	5-5246
U+71F7.0	fR/S	 86.13
U+71F7.0	fTotalStrokes	 17
U+71F7.0	fCangjie	FYWF
U+71F7.0	fFourCorner	9089
U+71F7.0	fCantonese	laam4
U+71F7.0	fMandarin	lan2
U+71F7.0	fEnglish	disappointed
U+71F8.0	fUTF8	燸
U+71F8.0	fCNS11643	1-7663
U+71F8.0	fBig5	C277
U+71F8.0	fGB	3-505B
U+71F8.0	fR/S	 86.14
U+71F8.0	fTotalStrokes	 18
U+71F8.0	fCangjie	FMBB
U+71F8.0	fFourCorner	91827
U+71F8.0	fCantonese	jyu4; nyun6
U+71F8.0	fMandarin	ru2
U+71F9.0	fUTF8	燹
U+71F9.0	fCNS11643	2-6229
U+71F9.0	fBig5	EFFB
U+71F9.0	fGB	0-6C5E
U+71F9.0	fR/S	 86.14
U+71F9.0	fTotalStrokes	 18
U+71F9.0	fCangjie	MOF
U+71F9.0	fFourCorner	11809
U+71F9.0	fHakka	  sien3; sen3
U+71F9.0	fCantonese	sin2
U+71F9.0	fMandarin	xian3; hui4; bing4; fen1
U+71F9.0	fEnglish	[1] fire set of by troops or shells [2] outdoor fire; wild fire
U+71FA.0	fUTF8	燺
U+71FA.0	fCNS11643	3-582D
U+71FA.0	fGB	5-524C
U+71FA.0	fR/S	  86.14
U+71FA.0	fTotalStrokes	 18
U+71FA.0	fCangjie	FYRD
U+71FA.0	fFourCorner	90890
U+71FA.0	fHakka	 kau3
U+71FA.0	fCantonese	huk6
U+71FA.0	fMandarin	kao3
U+71FA.0	fEnglish	the dry at the fire to roast
U+71FB.0	fUTF8	燻
U+71FB.0	fCNS11643	1-7660
U+71FB.0	fBig5	C274
U+71FB.0	fGB	E-3249
U+71FB.0	fR/S	 86.14
U+71FB.0	fTotalStrokes	 18
U+71FB.0	fCangjie	FHGF
U+71FB.0	fFourCorner	92831
U+71FB.0	fHakka	 hiun1; fun1
U+71FB.0	fCantonese	fan1
U+71FB.0	fMandarin	xun1; xun2
U+71FB.0	fEnglish	 smoke; fog; vapor smoke; cure
U+71FC.0	fUTF8	燼
U+71FC.0	fCNS11643	1-7661
U+71FC.0	fBig5	C275
U+71FC.0	fGB	1-3D7D
U+71FC.0	fR/S	 86.14
U+71FC.0	fTotalStrokes	 18
U+71FC.0	fCangjie	FLMT
U+71FC.0	fFourCorner	95817
U+71FC.0	fHakka	  zin5;  zin3
U+71FC.0	fCantonese	zeon6; zeon2 (also)
U+71FC.0	fMandarin	jin4
U+71FC.0	fEnglish	[1] cinders; ashes; embers; remnants [2] victims of disasters
U+71FD.0	fUTF8	燽
U+71FD.0	fCNS11643	2-622B
U+71FD.0	fBig5	EFFD
U+71FD.0	fGB	5-515D
U+71FD.0	fR/S	 86.14
U+71FD.0	fTotalStrokes	 18
U+71FD.0	fCangjie	FGNI
U+71FD.0	fFourCorner	94841
U+71FD.0	fCantonese	cau4
U+71FD.0	fMandarin	chou2
U+71FE.0	fUTF8	燾
U+71FE.0	fCNS11643	1-7662
U+71FE.0	fBig5	C276
U+71FE.0	fGB	1-6C62
U+71FE.0	fR/S	 86.14
U+71FE.0	fTotalStrokes	 18
U+71FE.0	fCangjie	GNMF
U+71FE.0	fFourCorner	40334
U+71FE.0	fHakka	  tau2;  tau5; to2; tau6; to6; to5
U+71FE.0	fCantonese	tou4; dou6 (also)
U+71FE.0	fMandarin	dao4; chou2; tao1; tao2
U+71FE.0	fEnglish	[1] shine; illuminate extensively [2] cover like a canopy
U+71FF.0	fUTF8	燿
U+71FF.0	fCNS11643	2-6228
U+71FF.0	fBig5	EFFA
U+71FF.0	fGB	8-2D6C
U+71FF.0	fR/S	 86.14
U+71FF.0	fTotalStrokes	 18
U+71FF.0	fCangjie	FSMG
U+71FF.0	fFourCorner	97814
U+71FF.0	fHakka	 jau5; rau6; jau3; jeu5
U+71FF.0	fCantonese	soek3; jiu6
U+71FF.0	fMandarin	yao4; shao4; shuo4; yao2; yue4
U+71FF.0	fEnglish	same as $
U+7200.0	fUTF8	爀
U+7200.0	fCNS11643	3-582E
U+7200.0	fGB	5-5249
U+7200.0	fR/S	  86.14
U+7200.0	fTotalStrokes	 18
U+7200.0	fCangjie	FGCC
U+7200.0	fFourCorner	94830
U+7200.0	fCantonese	haak1
U+7200.0	fMandarin	he4
U+7201.0	fUTF8	爁
U+7201.0	fCNS11643	2-6227
U+7201.0	fBig5	EFF9
U+7201.0	fGB	3-503D
U+7201.0	fR/S	 86.14
U+7201.0	fTotalStrokes	 18
U+7201.0	fCangjie	FSIT
U+7201.0	fFourCorner	98817
U+7201.0	fHakka	  lam3; lat7
U+7201.0	fCantonese	laam6; laam5 (also); laam4
U+7201.0	fMandarin	lan4; lian4; lan2; lan3
U+7201.0	fEnglish	[1] heat [2] scorch [3] singe
U+7202.0	fUTF8	爂
U+7202.0	fCNS11643	2-6576
U+7202.0	fBig5	F26C
U+7202.0	fGB	E-324A
U+7202.0	fR/S	 86.14
U+7202.0	fTotalStrokes	 18
U+7202.0	fCangjie	HBF
U+7202.0	fFourCorner	77809
U+7202.0	fCantonese	biu1
U+7202.0	fMandarin	biao1; miao2; piao4; zui4
U+7203.0	fUTF8	爃
U+7203.0	fCNS11643	2-622A
U+7203.0	fBig5	EFFC
U+7203.0	fGB	5-5175
U+7203.0	fR/S	 86.14
U+7203.0	fTotalStrokes	 18
U+7203.0	fCangjie	FFFD
U+7203.0	fFourCorner	99094
U+7203.0	fCantonese	wing4
U+7203.0	fMandarin	rong2
U+7204.0	fUTF8	爄
U+7204.0	fCNS11643	3-5A56
U+7204.0	fGB	3-4F5E
U+7204.0	fR/S	  86.15
U+7204.0	fTotalStrokes	 19
U+7204.0	fCangjie	FMTB
U+7204.0	fFourCorner	91820
U+7204.0	fCantonese	lai6
U+7204.0	fMandarin	li4; lie4
U+7205.0	fUTF8	爅
U+7205.0	fCNS11643	2-6577
U+7205.0	fBig5	F26D
U+7205.0	fGB	5-5250
U+7205.0	fR/S	 86.15
U+7205.0	fTotalStrokes	 19
U+7205.0	fCangjie	FWGG
U+7205.0	fFourCorner	96814
U+7205.0	fCantonese	mak6
U+7205.0	fMandarin	mo4
U+7206.0	fUTF8	爆
U+7206.0	fCNS11643	1-7847
U+7206.0	fBig5	C37A
U+7206.0	fGB	0-312C
U+7206.0	fR/S	 86.15
U+7206.0	fTotalStrokes	 19
U+7206.0	fCangjie	FATE
U+7206.0	fFourCorner	96832
U+7206.0	fHakka	  bau5;  pau5; bok7
U+7206.0	fCantonese	baau3; bok3; paau3
U+7206.0	fMandarin	bao4; bo2; pu4; bao2; pao4; bo1
U+7206.0	fEnglish	[1] crackle; pop; burst; explode [2] quick-boil; quick-fry
U+7207.0	fUTF8	爇
U+7207.0	fCNS11643	2-6575
U+7207.0	fBig5	F26B
U+7207.0	fGB	3-6B42
U+7207.0	fR/S	 86.15
U+7207.0	fTotalStrokes	 19
U+7207.0	fCangjie	TGIF
U+7207.0	fFourCorner	44331
U+7207.0	fHakka	 ngiet8; nget8
U+7207.0	fCantonese	jyut3; jyut6 (also); joek6
U+7207.0	fMandarin	ruo4; re4
U+7207.0	fEnglish	 burn
U+7208.0	fUTF8	爈
U+7208.0	fCNS11643	3-5A57
U+7208.0	fGB	3-503C
U+7208.0	fR/S	 86.15
U+7208.0	fTotalStrokes	 19
U+7208.0	fCangjie	FYPP
U+7208.0	fFourCorner	91830
U+7208.0	fCantonese	leoi4
U+7208.0	fMandarin	luu2; l2; luu4
U+7209.0	fUTF8	爉
U+7209.0	fCNS11643	4-5F7E
U+7209.0	fGB	3-505C
U+7209.0	fR/S	  86.15
U+7209.0	fTotalStrokes	 19
U+7209.0	fCangjie	FVVV
U+7209.0	fFourCorner	92810
U+7209.0	fHakka	 liap8
U+7209.0	fCantonese	laap6
U+7209.0	fMandarin	la4; lie4
U+720A.0	fUTF8	爊
U+720A.0	fCNS11643	2-6574
U+720A.0	fBig5	F26A
U+720A.0	fGB	5-5252
U+720A.0	fR/S	 86.15
U+720A.0	fTotalStrokes	 19
U+720A.0	fCangjie	FIPF
U+720A.0	fFourCorner	90831
U+720A.0	fHakka	 biau1; beu1; piau1; au1
U+720A.0	fCantonese	ngaau4; ngou4 (also); aau4 (also); ou4 (also); ou1; ngou1
U+720A.0	fMandarin	ao2; ao1
U+720A.0	fEnglish	 boil; stew
U+720B.0	fUTF8	爋
U+720B.0	fCNS11643	3-5C59
U+720B.0	fGB	E-324B
U+720B.0	fR/S	 86.16
U+720B.0	fTotalStrokes	 20
U+720B.0	fCangjie	FHFS
U+720B.0	fFourCorner	94820
U+720B.0	fCantonese	fan1
U+720B.0	fMandarin	xun4; xun1
U+720C.0	fUTF8	爌
U+720C.0	fCNS11643	2-6573
U+720C.0	fBig5	F269
U+720C.0	fGB	3-4F4F
U+720C.0	fR/S	 86.15
U+720C.0	fTotalStrokes	 19
U+720C.0	fCangjie	FITC
U+720C.0	fFourCorner	90886
U+720C.0	fCantonese	kong3; kwong3 (also)
U+720C.0	fMandarin	kuang4; huang3; huang4; kuang3
U+720D.0	fUTF8	爍
U+720D.0	fCNS11643	1-7848
U+720D.0	fBig5	C37B
U+720D.0	fGB	1-4B38
U+720D.0	fR/S	 86.15
U+720D.0	fTotalStrokes	 19
U+720D.0	fCangjie	FVID
U+720D.0	fFourCorner	92894
U+720D.0	fHakka	 sok7; shok7; shok8
U+720D.0	fCantonese	soek3; lik1 (also); lok6
U+720D.0	fMandarin	shuo4; luo4; yue4; le4; yao4
U+720D.0	fEnglish	[1] shine; glitter; sparkle [2] melt
U+720E.0	fUTF8	爎
U+720E.0	fCNS11643	3-655F
U+720E.0	fGB	E-324C
U+720E.0	fR/S	 86.15
U+720E.0	fTotalStrokes	 19
U+720E.0	fCangjie	FJKF
U+720E.0	fFourCorner	9389
U+720E.0	fCantonese	liu4
U+720F.0	fUTF8	爏
U+720F.0	fCNS11643	3-5C56
U+720F.0	fGB	5-5136
U+720F.0	fR/S	 86.16
U+720F.0	fTotalStrokes	 20
U+720F.0	fCangjie	FMHM
U+720F.0	fFourCorner	91810
U+720F.0	fCantonese	lik6
U+720F.0	fMandarin	li4
U+7210.0	fUTF8	爐
U+7210.0	fCNS11643	1-7977
U+7210.0	fBig5	C46C
U+7210.0	fGB	1-422F
U+7210.0	fR/S	 86.16
U+7210.0	fTotalStrokes	 20
U+7210.0	fCangjie	FYPT
U+7210.0	fFourCorner	91817
U+7210.0	fHakka	  lu2; lu3
U+7210.0	fCantonese	lou4; lou2 (also)
U+7210.0	fMandarin	lu2
U+7210.0	fEnglish	 fireplace; stove; oven; furnace; hearth
U+7211.0	fUTF8	爑
U+7211.0	fCNS11643	3-5C58
U+7211.0	fGB	5-524F
U+7211.0	fR/S	  86.16
U+7211.0	fTotalStrokes	 20
U+7211.0	fCangjie	FTOF
U+7211.0	fFourCorner	94830
U+7211.0	fCantonese	zoek3
U+7211.0	fMandarin	jue2
U+7212.0	fUTF8	爒
U+7212.0	fCNS11643	3-5C55
U+7212.0	fGB	E-324D
U+7212.0	fR/S	  86.16
U+7212.0	fTotalStrokes	 20
U+7212.0	fCangjie	BFKCF
U+7212.0	fFourCorner	24890
U+7212.0	fCantonese	liu5
U+7212.0	fMandarin	liao4; liao3
U+7213.0	fUTF8	爓
U+7213.0	fCNS11643	2-6939
U+7213.0	fBig5	F46A
U+7213.0	fGB	5-5238
U+7213.0	fR/S	 86.16
U+7213.0	fTotalStrokes	 20
U+7213.0	fCangjie	FANX
U+7213.0	fFourCorner	97820
U+7213.0	fHakka	 ram2; jam2; jam5
U+7213.0	fCantonese	jim6; jim4 (also); cam4; cim4 (also)
U+7213.0	fMandarin	yan4; qian2; xun2; yan2; xian2
U+7213.0	fEnglish	 flame brilliant; blazing
U+7214.0	fUTF8	爔
U+7214.0	fCNS11643	2-693A
U+7214.0	fBig5	F46B
U+7214.0	fGB	3-505D
U+7214.0	fR/S	 86.16
U+7214.0	fTotalStrokes	 20
U+7214.0	fCangjie	FTGS
U+7214.0	fFourCorner	98853
U+7214.0	fCantonese	hei1
U+7214.0	fMandarin	xi1
U+7215.0	fUTF8	爕
U+7215.0	fCNS11643	3-5A58
U+7215.0	fGB	E-324E
U+7215.0	fR/S	  86.15
U+7215.0	fTotalStrokes	 19
U+7215.0	fCangjie	FFF
U+7215.0	fFourCorner	44800
U+7215.0	fCantonese	sit3
U+7215.0	fMandarin	xie4
U+7216.0	fUTF8	爖
U+7216.0	fCNS11643	3-5C53
U+7216.0	fGB	5-5143
U+7216.0	fR/S	 86.16
U+7216.0	fTotalStrokes	 20
U+7216.0	fCangjie	FYBP
U+7216.0	fFourCorner	91810
U+7216.0	fCantonese	lung4
U+7216.0	fMandarin	long2
U+7216.0	fEnglish	fire
U+7217.0	fUTF8	爗
U+7217.0	fCNS11643	3-5C57
U+7217.0	fGB	E-324F
U+7217.0	fR/S	 86.16; 86.14
U+7217.0	fTotalStrokes	 20
U+7217.0	fCangjie	FATJ
U+7217.0	fFourCorner	96850
U+7217.0	fCantonese	jip6
U+7217.0	fMandarin	ye4; yi4
U+7218.0	fUTF8	爘
U+7218.0	fCNS11643	3-6561
U+7218.0	fGB	E-3250
U+7218.0	fR/S	 86.16
U+7218.0	fTotalStrokes	 20
U+7218.0	fCangjie	FYEV
U+7218.0	fFourCorner	9783
U+7218.0	fCantonese	caan1
U+7219.0	fUTF8	爙
U+7219.0	fCNS11643	2-6B6A
U+7219.0	fBig5	F5DC
U+7219.0	fGB	5-5255
U+7219.0	fR/S	 86.17
U+7219.0	fTotalStrokes	 21
U+7219.0	fCangjie	FYRV
U+7219.0	fFourCorner	90832
U+7219.0	fCantonese	joeng5
U+7219.0	fMandarin	rang3
U+721A.0	fUTF8	爚
U+721A.0	fCNS11643	2-6B69
U+721A.0	fBig5	F5DB
U+721A.0	fGB	5-5254
U+721A.0	fR/S	 86.17
U+721A.0	fTotalStrokes	 21
U+721A.0	fCangjie	FOMB
U+721A.0	fFourCorner	98827
U+721A.0	fHakka	  jok7; rok7
U+721A.0	fCantonese	joek6
U+721A.0	fMandarin	yue4; shuo4; yao4
U+721A.0	fEnglish	 bright fiery
U+721B.0	fUTF8	爛
U+721B.0	fCNS11643	1-7A75
U+721B.0	fBig5	C4EA
U+721B.0	fGB	1-4043
U+721B.0	fR/S	 86.17
U+721B.0	fTotalStrokes	 21
U+721B.0	fCangjie	FANW
U+721B.0	fFourCorner	97820
U+721B.0	fHakka	  lan5; lan6; lan3
U+721B.0	fCantonese	laan6; laan2 (also)
U+721B.0	fMandarin	lan4; lan2
U+721B.0	fEnglish	[1] overripe; rotten; spoiled; decayed rot; fester [2] cooked; soft; well cooked [3] bright and brilliant [4] scald; burn; scorch [5] worn-out [6] dissolute; messy
U+721C.0	fUTF8	爜
U+721C.0	fCNS11643	3-5F53
U+721C.0	fGB	E-3251
U+721C.0	fR/S	  86.18
U+721C.0	fTotalStrokes	 22
U+721C.0	fCangjie	FTCE
U+721C.0	fFourCorner	92840
U+721C.0	fCantonese	cung4
U+721C.0	fMandarin	cong2
U+721D.0	fUTF8	爝
U+721D.0	fCNS11643	2-6B68
U+721D.0	fBig5	F5DA
U+721D.0	fGB	0-6C5F
U+721D.0	fR/S	 86.18
U+721D.0	fTotalStrokes	 22
U+721D.0	fCangjie	FBWI
U+721D.0	fFourCorner	92846
U+721D.0	fHakka	  ziok7
U+721D.0	fCantonese	zoek3; ziu3 (also); zoek6
U+721D.0	fMandarin	jue2; jiao4; jiao2
U+721D.0	fEnglish	 torch
U+721E.0	fUTF8	爞
U+721E.0	fCNS11643	2-6D5A
U+721E.0	fBig5	F6EC
U+721E.0	fGB	E-3252
U+721E.0	fR/S	 86.18
U+721E.0	fTotalStrokes	 22
U+721E.0	fCangjie	FLII
U+721E.0	fFourCorner	95036
U+721E.0	fHakka	 tung1
U+721E.0	fCantonese	tung4; cung4
U+721E.0	fMandarin	tong2; chong2
U+721F.0	fUTF8	爟
U+721F.0	fCNS11643	2-6D5B
U+721F.0	fBig5	F6ED
U+721F.0	fGB	3-505E
U+721F.0	fR/S	 86.18
U+721F.0	fTotalStrokes	 22
U+721F.0	fCangjie	FTRG
U+721F.0	fFourCorner	94814
U+721F.0	fHakka	 kwan5; gwon5
U+721F.0	fCantonese	gun3; gwun3 (also)
U+721F.0	fMandarin	guan4; quan2
U+721F.0	fEnglish	fire
U+7220.0	fUTF8	爠
U+7220.0	fCNS11643	3-5F54
U+7220.0	fGB	E-3253
U+7220.0	fR/S	 86.18
U+7220.0	fTotalStrokes	 22
U+7220.0	fCangjie	FBUG
U+7220.0	fFourCorner	96810
U+7220.0	fCantonese	keoi4
U+7220.0	fMandarin	qu2
U+7221.0	fUTF8	爡
U+7221.0	fCNS11643	3-6039
U+7221.0	fGB	3-5033
U+7221.0	fR/S	 86.19; 85.17; 142.17
U+7221.0	fTotalStrokes	 23
U+7221.0	fCangjie	FTWI
U+7221.0	fFourCorner	94830
U+7221.0	fCantonese	cit3
U+7221.0	fMandarin	che4; ta4
U+7222.0	fUTF8	爢
U+7222.0	fCNS11643	2-6F35
U+7222.0	fBig5	F7E6
U+7222.0	fGB	E-3254
U+7222.0	fR/S	 86.19
U+7222.0	fTotalStrokes	 23
U+7222.0	fCangjie	IYF
U+7222.0	fFourCorner	00211
U+7222.0	fCantonese	mei4
U+7222.0	fMandarin	mi2
U+7223.0	fUTF8	爣
U+7223.0	fCNS11643	2-703F
U+7223.0	fBig5	F8B1
U+7223.0	fGB	3-503E
U+7223.0	fR/S	 86.20
U+7223.0	fTotalStrokes	 24
U+7223.0	fCangjie	FFBF
U+7223.0	fFourCorner	99831
U+7223.0	fCantonese	tong2
U+7223.0	fMandarin	tang3
U+7224.0	fUTF8	爤
U+7224.0	fCNS11643	3-6143
U+7224.0	fGB	E-3255
U+7224.0	fR/S	  86.21
U+7224.0	fTotalStrokes	 25
U+7224.0	fCangjie	FTAW
U+7224.0	fFourCorner	94820
U+7224.0	fCantonese	laan6
U+7224.0	fMandarin	lan4
U+7225.0	fUTF8	爥
U+7225.0	fCNS11643	3-6142
U+7225.0	fGB	5-5244
U+7225.0	fR/S	 86.21
U+7225.0	fTotalStrokes	 25
U+7225.0	fCangjie	FSEI
U+7225.0	fFourCorner	97820
U+7225.0	fHakka	 zhuk7
U+7225.0	fCantonese	zuk1
U+7225.0	fMandarin	zhu2
U+7225.0	fEnglish	simmer; cook over slow fire
U+7226.0	fUTF8	爦
U+7226.0	fCNS11643	2-7126
U+7226.0	fBig5	F8F6
U+7226.0	fGB	E-3256
U+7226.0	fR/S	 86.21
U+7226.0	fTotalStrokes	 25
U+7226.0	fCangjie	FSWU
U+7226.0	fFourCorner	98816
U+7226.0	fCantonese	laam5
U+7226.0	fMandarin	lan3
U+7227.0	fUTF8	爧
U+7227.0	fCNS11643	2-722C
U+7227.0	fBig5	F9BC
U+7227.0	fGB	E-3257
U+7227.0	fR/S	 86.24
U+7227.0	fTotalStrokes	 28
U+7227.0	fCangjie	XFMBO
U+7227.0	fFourCorner	91814
U+7227.0	fHakka	 lang2; lin2
U+7227.0	fCantonese	ling4
U+7227.0	fMandarin	ling2
U+7228.0	fUTF8	爨
U+7228.0	fCNS11643	1-7D46
U+7228.0	fBig5	C679
U+7228.0	fGB	0-6C60
U+7228.0	fR/S	 86.25
U+7228.0	fTotalStrokes	 29
U+7228.0	fCangjie	HBDDF
U+7228.0	fFourCorner	77809
U+7228.0	fHakka	 cion2; con3
U+7228.0	fCantonese	cyun3
U+7228.0	fMandarin	cuan4; cuan1; quan4
U+7228.0	fEnglish	oven; cooking stove; cook
U+7229.0	fUTF8	爩
U+7229.0	fCNS11643	2-7240
U+7229.0	fBig5	F9C6
U+7229.0	fGB	E-3258
U+7229.0	fR/S	 86.29
U+7229.0	fTotalStrokes	 33
U+7229.0	fCangjie	FDDH
U+7229.0	fFourCorner	94822
U+7229.0	fCantonese	wat1
U+7229.0	fMandarin	yu4
U+7229.0	fEnglish	to smoke, fumigate; to bleach with burning sulfur
U+722A.0	fUTF8	爪
U+722A.0	fCNS11643	1-4557
U+722A.0	fBig5	A4F6
U+722A.0	fGB	0-5726
U+722A.0	fR/S	 87.0
U+722A.0	fTotalStrokes	 4
U+722A.0	fCangjie	HLO
U+722A.0	fFourCorner	72230
U+722A.0	fHakka	  zau3
U+722A.0	fCantonese	zaau2
U+722A.0	fMandarin	zhao3; zhao4
U+722A.0	fEnglish	[1] claw; talon; animal feet [2] nail [3] Kangxi radical 87
U+722A.1	fUTF8	爪
U+722A.1	fCNS11643	1-4557
U+722A.1	fBig5	A4F6
U+722A.1	fGB	0-5726
U+722A.1	fR/S	 87.0
U+722A.1	fTotalStrokes	 4
U+722A.1	fCangjie	HLO
U+722A.1	fFourCorner	72230
U+722A.1	fHakka	 zau1
U+722A.1	fMandarin	zhua3
U+722A.1	fEnglish	 claw
U+722B.0	fUTF8	爫
U+722B.0	fCNS11643	4-2151
U+722B.0	fGB	E-3259
U+722B.0	fR/S	 87.0
U+722B.0	fTotalStrokes	 4
U+722B.0	fCangjie	XHF
U+722B.0	fFourCorner	20230
U+722B.0	fCantonese	zaau2
U+722B.0	fMandarin	lan4
U+722B.0	fEnglish	a variant of Kangxi radical 87
U+722C.0	fUTF8	爬
U+722C.0	fCNS11643	1-4E67
U+722C.0	fBig5	AAA6
U+722C.0	fGB	0-4540
U+722C.0	fR/S	 87.4
U+722C.0	fTotalStrokes	 8
U+722C.0	fCangjie	HOAU
U+722C.0	fFourCorner	77231
U+722C.0	fHakka	  pa2; pa3; pa1
U+722C.0	fCantonese	paa4
U+722C.0	fMandarin	pa2
U+722C.0	fEnglish	[1] crawl; creep [2] climb; clamber [3] scratch [4] lie face downwards
U+722D.0	fUTF8	爭
U+722D.0	fCNS11643	1-4E68
U+722D.0	fBig5	AAA7
U+722D.0	fGB	1-5579
U+722D.0	fR/S	 87.4
U+722D.0	fTotalStrokes	 8
U+722D.0	fCangjie	BSD
U+722D.0	fFourCorner	20507
U+722D.0	fHakka	  zang1; zen1
U+722D.0	fCantonese	zang1; zaang1 (also); caang1; zang3; zaang3 (also)
U+722D.0	fMandarin	zheng1; zheng4
U+722D.0	fEnglish	[1] dispute; fight; argue; quarrel [2] contend; strive; struggle [3] short of; lack; be deficient in [4] same as $
U+722E.0	fUTF8	爮
U+722E.0	fCNS11643	3-2D69
U+722E.0	fGB	3-4C37
U+722E.0	fR/S	 87.5
U+722E.0	fTotalStrokes	 9
U+722E.0	fCangjie	HOPRU
U+722E.0	fFourCorner	77230
U+722E.0	fCantonese	paau4
U+722E.0	fMandarin	pao2
U+722F.0	fUTF8	爯
U+722F.0	fCNS11643	4-2A2A
U+722F.0	fGB	5-4E6A
U+722F.0	fR/S	  87.5
U+722F.0	fTotalStrokes	 9
U+722F.0	fCangjie	BGB
U+722F.0	fFourCorner	20550
U+722F.0	fCantonese	cing1
U+722F.0	fMandarin	cheng1; cheng4
U+7230.0	fUTF8	爰
U+7230.0	fCNS11643	1-523B
U+7230.0	fBig5	ACB8
U+7230.0	fGB	0-6B3C
U+7230.0	fR/S	 87.5
U+7230.0	fTotalStrokes	 9
U+7230.0	fCangjie	BMKE
U+7230.0	fFourCorner	20447
U+7230.0	fHakka	 jen1; jen2; ren2; ren1; jan1
U+7230.0	fCantonese	wun4; jyun4 (also)
U+7230.0	fMandarin	yuan2
U+7230.0	fEnglish	thereupon; therefore; accordingly
U+7231.0	fUTF8	爱
U+7231.0	fGB	0-302E
U+7231.0	fR/S	 87.6
U+7231.0	fTotalStrokes	 10
U+7231.0	fCangjie	BBKE
U+7231.0	fHakka	 oi5
U+7231.0	fCantonese	oi3; ngoi3 (also)
U+7231.0	fMandarin	ai4
U+7231.0	fEnglish	simplified form of &#x611B;
U+7232.0	fUTF8	爲
U+7232.0	fCNS11643	3-3D22
U+7232.0	fGB	1-4E2A
U+7232.0	fR/S	 87.8
U+7232.0	fTotalStrokes	 12
U+7232.0	fCangjie	BHNF
U+7232.0	fFourCorner	20220
U+7232.0	fHakka	 wui2;  wui5; wui3; wi2; wui6; wi5; vui5; vui2
U+7232.0	fCantonese	wai4; wai6 (also)
U+7232.0	fMandarin	wei2; wei4
U+7232.0	fEnglish	 do; handle; govern; act be
U+7233.0	fUTF8	爳
U+7233.0	fGB	1-7D68
U+7233.0	fR/S	 87.10; 6.13
U+7233.0	fTotalStrokes	 14
U+7233.0	fCangjie	BHFN
U+7233.0	fCantonese	wai4
U+7234.0	fUTF8	爴
U+7234.0	fCNS11643	4-4A6C
U+7234.0	fGB	E-325A
U+7234.0	fR/S	 87.11
U+7234.0	fTotalStrokes	 15
U+7234.0	fCangjie	WMHLO
U+7234.0	fFourCorner	62010
U+7234.0	fCantonese	fok3
U+7234.0	fMandarin	jue2
U+7235.0	fUTF8	爵
U+7235.0	fCNS11643	1-743A
U+7235.0	fBig5	C0EF
U+7235.0	fGB	0-3E74
U+7235.0	fR/S	 87.14
U+7235.0	fTotalStrokes	 18
U+7235.0	fCangjie	BWLI
U+7235.0	fFourCorner	20746
U+7235.0	fHakka	  ziok7
U+7235.0	fCantonese	zoek3
U+7235.0	fMandarin	jue2
U+7235.0	fEnglish	[1] a degree or title of nobility; peerage; the rank of dignity of a peer [2] an ancient wine pitcher with three legs and a loop handle
U+7236.0	fUTF8	父
U+7236.0	fCNS11643	1-4558
U+7236.0	fBig5	A4F7
U+7236.0	fGB	0-3838
U+7236.0	fR/S	 88.0
U+7236.0	fTotalStrokes	 4
U+7236.0	fCangjie	CK
U+7236.0	fFourCorner	80400
U+7236.0	fHakka	  fu5; fu3
U+7236.0	fCantonese	fu6; fu2
U+7236.0	fMandarin	fu4
U+7236.0	fEnglish	[1] father [2] a male relative of an elder generation [3] do father's duties [4] Kangxi radical 88
U+7236.1	fUTF8	父
U+7236.1	fCNS11643	1-4558
U+7236.1	fBig5	A4F7
U+7236.1	fGB	0-3838
U+7236.1	fR/S	 88.0
U+7236.1	fTotalStrokes	 4
U+7236.1	fCangjie	CK
U+7236.1	fFourCorner	80400
U+7236.1	fMandarin	fu3
U+7236.1	fEnglish	a respectful term for an elderly man in ancient times
U+7237.0	fUTF8	爷
U+7237.0	fGB	0-522F
U+7237.0	fR/S	 88.2
U+7237.0	fTotalStrokes	 6
U+7237.0	fCangjie	CKSL
U+7237.0	fHakka	 ja2
U+7237.0	fCantonese	je4; je1 (also); je2 (also)
U+7237.0	fMandarin	ye2
U+7237.0	fEnglish	simplified form of &#x723A;
U+7237.1	fUTF8	爷
U+7237.1	fGB	0-522F
U+7237.1	fR/S	 88.2
U+7237.1	fTotalStrokes	 6
U+7237.1	fCangjie	CKSL
U+7237.1	fMandarin	ye5
U+7238.0	fUTF8	爸
U+7238.0	fCNS11643	1-4E69
U+7238.0	fBig5	AAA8
U+7238.0	fGB	0-3056
U+7238.0	fR/S	 88.4
U+7238.0	fTotalStrokes	 8
U+7238.0	fCangjie	CKAU
U+7238.0	fFourCorner	80717
U+7238.0	fHakka	  ba1; ba3
U+7238.0	fCantonese	baa1; baa4 (also)
U+7238.0	fMandarin	ba4; bo4
U+7238.0	fEnglish	 father; papa; daddy
U+7238.1	fUTF8	爸
U+7238.1	fCNS11643	1-4E69
U+7238.1	fBig5	AAA8
U+7238.1	fGB	0-3056
U+7238.1	fR/S	 88.4
U+7238.1	fTotalStrokes	 8
U+7238.1	fCangjie	CKAU
U+7238.1	fFourCorner	80717
U+7238.1	fMandarin	ba5
U+7239.0	fUTF8	爹
U+7239.0	fCNS11643	1-5655
U+7239.0	fBig5	AF52
U+7239.0	fGB	0-3579
U+7239.0	fR/S	 88.6
U+7239.0	fTotalStrokes	 10
U+7239.0	fCangjie	CKNIN
U+7239.0	fFourCorner	80207
U+7239.0	fHakka	  dia1
U+7239.0	fCantonese	de1
U+7239.0	fMandarin	die1; duo4
U+7239.0	fEnglish	 father; daddy; papa
U+723A.0	fUTF8	爺
U+723A.0	fCNS11643	1-6525
U+723A.0	fBig5	B7DD
U+723A.0	fGB	1-522F
U+723A.0	fR/S	 88.9
U+723A.0	fTotalStrokes	 13
U+723A.0	fCangjie	CKSJL
U+723A.0	fFourCorner	80127
U+723A.0	fHakka	  ja2; ra2; ja3
U+723A.0	fCantonese	je4; je1 (also); je2 (also)
U+723A.0	fMandarin	ye2
U+723A.0	fEnglish	[1] father; grandfather [2] master; sir [3] god
U+723A.1	fUTF8	爺
U+723A.1	fCNS11643	1-6525
U+723A.1	fBig5	B7DD
U+723A.1	fGB	1-522F
U+723A.1	fR/S	 88.9
U+723A.1	fTotalStrokes	 13
U+723A.1	fCangjie	CKSJL
U+723A.1	fFourCorner	80127
U+723A.1	fMandarin	ye5
U+723B.0	fUTF8	爻
U+723B.0	fCNS11643	1-4559
U+723B.0	fBig5	A4F8
U+723B.0	fGB	0-5833
U+723B.0	fR/S	 89.0
U+723B.0	fTotalStrokes	 4
U+723B.0	fCangjie	KK
U+723B.0	fFourCorner	40400
U+723B.0	fHakka	 ngau2; ngau3
U+723B.0	fCantonese	ngaau4; aau4 (also); au4
U+723B.0	fMandarin	yao2; xiao4; xiao2
U+723B.0	fEnglish	single and divided lines in eight groups of three lines each as specified in the Book of changes
U+723C.0	fUTF8	爼
U+723C.0	fCNS11643	3-2D6B
U+723C.0	fGB	E-325B
U+723C.0	fR/S	  89.5
U+723C.0	fTotalStrokes	 9
U+723C.0	fCangjie	KKBM
U+723C.0	fFourCorner	47410
U+723C.0	fHakka	 zu3; cu3; zih3; cih1
U+723C.0	fCantonese	zo2
U+723C.0	fMandarin	zu3
U+723C.0	fEnglish	same as &#x4FCE;
U+723D.0	fUTF8	爽
U+723D.0	fCNS11643	1-5B73
U+723D.0	fBig5	B26E
U+723D.0	fGB	0-4B2C
U+723D.0	fR/S	 89.7
U+723D.0	fTotalStrokes	 11
U+723D.0	fCangjie	KKKK
U+723D.0	fFourCorner	40034
U+723D.0	fHakka	  song3; song1
U+723D.0	fCantonese	song2
U+723D.0	fMandarin	shuang3; shuang1
U+723D.0	fEnglish	[1] refreshing; bracing; crisp; agreeable; pleasant; brisk [2] feel well [3] straightforward; frank; open-hearted [4] fail; miss; lose [5] be in error
U+723E.0	fUTF8	爾
U+723E.0	fCNS11643	1-695F
U+723E.0	fBig5	BAB8
U+723E.0	fGB	1-367B
U+723E.0	fR/S	 89.10
U+723E.0	fTotalStrokes	 14
U+723E.0	fCangjie	MFBK
U+723E.0	fFourCorner	10227
U+723E.0	fHakka	  ni3; li1; ngi2; ni1; li3; {ngi2}; {ni1}
U+723E.0	fCantonese	ji5
U+723E.0	fMandarin	er3; ni3
U+723E.0	fEnglish	[1] you; thou [2] that; this; those; these [3] after adjectives [4] only
U+723F.0	fUTF8	爿
U+723F.0	fCNS11643	2-2142
U+723F.0	fBig5	C962
U+723F.0	fGB	0-635D
U+723F.0	fR/S	 90.0
U+723F.0	fTotalStrokes	 4
U+723F.0	fCangjie	VLM
U+723F.0	fFourCorner	22200
U+723F.0	fHakka	 cong2; siong2; ciong2; cong3; pan5
U+723F.0	fCantonese	baan6; coeng4; baan2
U+723F.0	fMandarin	qiang2; pan2
U+723F.0	fEnglish	[1] half of a tree trunk [2] Kangxi radical 90 [3] [4] measure word for shops, stores, factories
U+7240.0	fUTF8	牀
U+7240.0	fCNS11643	3-297C
U+7240.0	fGB	E-325C
U+7240.0	fR/S	  90.4
U+7240.0	fTotalStrokes	 8
U+7240.0	fCangjie	VMD
U+7240.0	fFourCorner	24290
U+7240.0	fHakka	 cong2
U+7240.0	fCantonese	cong4
U+7240.0	fMandarin	chuang2
U+7240.0	fEnglish	[1] bed; couch [2] the ground under a body of water
U+7241.0	fUTF8	牁
U+7241.0	fCNS11643	2-2B77
U+7241.0	fBig5	CFB7
U+7241.0	fGB	3-3A3A
U+7241.0	fR/S	 90.5
U+7241.0	fTotalStrokes	 9
U+7241.0	fCangjie	VMMNR
U+7241.0	fFourCorner	27220
U+7241.0	fHakka	 ko1; go1
U+7241.0	fCantonese	go1 (also); ngo1 (also); o1; ho1
U+7241.0	fMandarin	ge1
U+7241.0	fEnglish	a stake to which a boat may be moored; mooring stake
U+7242.0	fUTF8	牂
U+7242.0	fCNS11643	2-3060
U+7242.0	fBig5	D27D
U+7242.0	fGB	3-3A3C
U+7242.0	fR/S	 90.6
U+7242.0	fTotalStrokes	 10
U+7242.0	fCangjie	VMTQ
U+7242.0	fFourCorner	28251
U+7242.0	fHakka	  zong1
U+7242.0	fCantonese	zong1
U+7242.0	fMandarin	zang1
U+7242.0	fEnglish	 female sheep; ewe
U+7243.0	fUTF8	牃
U+7243.0	fCNS11643	4-3E4C
U+7243.0	fGB	E-325D
U+7243.0	fR/S	  90.9
U+7243.0	fTotalStrokes	 13
U+7243.0	fCangjie	VMPTD
U+7243.0	fFourCorner	24290
U+7243.0	fCantonese	dip6
U+7243.0	fMandarin	die2
U+7244.0	fUTF8	牄
U+7244.0	fCNS11643	2-4B6B
U+7244.0	fBig5	E2C5
U+7244.0	fGB	5-3C70
U+7244.0	fR/S	 90.10
U+7244.0	fTotalStrokes	 14
U+7244.0	fCangjie	VMOIR
U+7244.0	fFourCorner	28267
U+7244.0	fHakka	 cong1
U+7244.0	fCantonese	coeng1; cong1
U+7244.0	fMandarin	qiang1
U+7244.0	fEnglish	 walk rapidly
U+7245.0	fUTF8	牅
U+7245.0	fCNS11643	4-4A6D
U+7245.0	fGB	E-325E
U+7245.0	fR/S	  90.11
U+7245.0	fTotalStrokes	 15
U+7245.0	fCangjie	VMILB
U+7245.0	fFourCorner	20220
U+7245.0	fCantonese	jung4
U+7245.0	fMandarin	yong2; yong1
U+7246.0	fUTF8	牆
U+7246.0	fCNS11643	1-743B
U+7246.0	fBig5	C0F0
U+7246.0	fGB	E-325F
U+7246.0	fR/S	 90.13
U+7246.0	fTotalStrokes	 17
U+7246.0	fCangjie	VMGOW
U+7246.0	fFourCorner	24261
U+7246.0	fHakka	  siong2;  ciong2; ciong3
U+7246.0	fCantonese	coeng4
U+7246.0	fMandarin	qiang2
U+7246.0	fEnglish	 wall; fence
U+7247.0	fUTF8	片
U+7247.0	fCNS11643	1-455A
U+7247.0	fBig5	A4F9
U+7247.0	fGB	0-462C
U+7247.0	fR/S	 91.0
U+7247.0	fTotalStrokes	 4
U+7247.0	fCangjie	LLML
U+7247.0	fFourCorner	22027
U+7247.0	fHakka	  pien3;  pien5; p'ian5; pen3; pen5
U+7247.0	fCantonese	pin3; pin2
U+7247.0	fMandarin	pian4; pan4
U+7247.0	fEnglish	[1] piece; slice; splinter; chip; fragment [2] Kangxi radical 91
U+7247.1	fUTF8	片
U+7247.1	fCNS11643	1-455A
U+7247.1	fBig5	A4F9
U+7247.1	fGB	0-462C
U+7247.1	fR/S	 91.0
U+7247.1	fTotalStrokes	 4
U+7247.1	fCangjie	LLML
U+7247.1	fFourCorner	22027
U+7247.1	fMandarin	pian1
U+7247.1	fEnglish	[1] photograph [2] phonograph record
U+7248.0	fUTF8	版
U+7248.0	fCNS11643	1-4E6A
U+7248.0	fBig5	AAA9
U+7248.0	fGB	0-3066
U+7248.0	fR/S	 91.4
U+7248.0	fTotalStrokes	 8
U+7248.0	fCangjie	LLHE
U+7248.0	fFourCorner	21047
U+7248.0	fHakka	  ban3
U+7248.0	fCantonese	baan2
U+7248.0	fMandarin	ban3
U+7248.0	fEnglish	[1] household registers [2] printing plate; printing blocks [3] edition [4] supporting boards used in building walls
U+7249.0	fUTF8	牉
U+7249.0	fCNS11643	2-2B78
U+7249.0	fBig5	CFB8
U+7249.0	fGB	3-4C2C
U+7249.0	fR/S	 91.5
U+7249.0	fTotalStrokes	 9
U+7249.0	fCangjie	LLFQ
U+7249.0	fFourCorner	29250
U+7249.0	fHakka	  pan5
U+7249.0	fCantonese	pun3; bun3
U+7249.0	fMandarin	pan4
U+7249.0	fEnglish	[1] fat [2] obese [3] swollen
U+724A.0	fUTF8	牊
U+724A.0	fCNS11643	2-2B79
U+724A.0	fBig5	CFB9
U+724A.0	fGB	5-4E55
U+724A.0	fR/S	 91.5
U+724A.0	fTotalStrokes	 9
U+724A.0	fCangjie	LLSHR
U+724A.0	fFourCorner	27062
U+724A.0	fCantonese	ciu4
U+724A.0	fMandarin	shao2
U+724B.0	fUTF8	牋
U+724B.0	fCNS11643	2-3D6A
U+724B.0	fBig5	DA66
U+724B.0	fGB	E-3260
U+724B.0	fR/S	 91.8
U+724B.0	fTotalStrokes	 12
U+724B.0	fCangjie	LLII
U+724B.0	fFourCorner	23053
U+724B.0	fHakka	  zien1
U+724B.0	fCantonese	zin1
U+724B.0	fMandarin	jian1
U+724B.0	fEnglish	memorandum; official note
U+724C.0	fUTF8	牌
U+724C.0	fCNS11643	1-6056
U+724C.0	fBig5	B550
U+724C.0	fGB	0-4546
U+724C.0	fR/S	 91.8
U+724C.0	fTotalStrokes	 12
U+724C.0	fCangjie	LNHWJ
U+724C.0	fFourCorner	26040
U+724C.0	fHakka	  pai2; pai3
U+724C.0	fCantonese	paai4; paai2 (also)
U+724C.0	fMandarin	pai2
U+724C.0	fEnglish	[1] bulletin board [2] tablet; signboard; placard; plate [3] card; tag; label [4] trademark; brand
U+724D.0	fUTF8	牍
U+724D.0	fGB	0-6B39
U+724D.0	fR/S	 91.8
U+724D.0	fTotalStrokes	 12
U+724D.0	fCangjie	LNJNK
U+724D.0	fHakka	 tuk8
U+724D.0	fCantonese	duk6; duk2 (also)
U+724D.0	fMandarin	du2
U+724D.0	fEnglish	simplified form of &#x7258;
U+724E.0	fUTF8	牎
U+724E.0	fCNS11643	3-425D
U+724E.0	fGB	E-3261
U+724E.0	fR/S	  91.9
U+724E.0	fTotalStrokes	 13
U+724E.0	fCangjie	LLPKP
U+724E.0	fFourCorner	27030
U+724E.0	fCantonese	coeng1
U+724E.0	fMandarin	chuang1; cong1
U+724F.0	fUTF8	牏
U+724F.0	fCNS11643	2-4468
U+724F.0	fBig5	DEA4
U+724F.0	fGB	3-4C2F
U+724F.0	fR/S	 91.9
U+724F.0	fTotalStrokes	 13
U+724F.0	fCangjie	LNOMN
U+724F.0	fFourCorner	28221
U+724F.0	fHakka	 chu5; cu5
U+724F.0	fCantonese	jyu4; tau4 (also)
U+724F.0	fMandarin	tou2; zhu4; yu2; yu4; dou4
U+724F.0	fEnglish	privy hole
U+7250.0	fUTF8	牐
U+7250.0	fCNS11643	3-425C
U+7250.0	fGB	E-3262
U+7250.0	fR/S	 91.9
U+7250.0	fTotalStrokes	 13
U+7250.0	fCangjie	LLHJX
U+7250.0	fFourCorner	22070
U+7250.0	fHakka	  cap7
U+7250.0	fCantonese	zaap6
U+7250.0	fMandarin	zha2
U+7250.0	fEnglish	 sluice flood gate; canal lock
U+7251.0	fUTF8	牑
U+7251.0	fCNS11643	3-425B
U+7251.0	fGB	3-4C30
U+7251.0	fR/S	  91.9
U+7251.0	fTotalStrokes	 13
U+7251.0	fCangjie	LLHSB
U+7251.0	fFourCorner	23020
U+7251.0	fCantonese	bin1
U+7251.0	fMandarin	bian1; pian2; bian4; mian4
U+7252.0	fUTF8	牒
U+7252.0	fCNS11643	1-6526
U+7252.0	fBig5	B7DE
U+7252.0	fGB	0-6B3A
U+7252.0	fR/S	 91.9
U+7252.0	fTotalStrokes	 13
U+7252.0	fCangjie	LLPTD
U+7252.0	fFourCorner	24094
U+7252.0	fHakka	  tiap8; tiap7
U+7252.0	fCantonese	dip6
U+7252.0	fMandarin	die2
U+7252.0	fEnglish	[1] official documents [2] certificates [3] records of a family pedigree
U+7253.0	fUTF8	牓
U+7253.0	fCNS11643	2-4B6C
U+7253.0	fBig5	E2C6
U+7253.0	fGB	E-3263
U+7253.0	fR/S	 91.10
U+7253.0	fTotalStrokes	 14
U+7253.0	fCangjie	LLYBS
U+7253.0	fFourCorner	20027
U+7253.0	fHakka	  bong3
U+7253.0	fCantonese	bong2
U+7253.0	fMandarin	bang3; pang1
U+7253.0	fEnglish	[1] tablet; plaque [2] public notice
U+7254.0	fUTF8	牔
U+7254.0	fCNS11643	4-445E
U+7254.0	fGB	E-3264
U+7254.0	fR/S	  91.10
U+7254.0	fTotalStrokes	 14
U+7254.0	fCangjie	LLIBI
U+7254.0	fFourCorner	23040
U+7254.0	fCantonese	bok3
U+7254.0	fMandarin	bo2
U+7254.0	fEnglish	gable
U+7255.0	fUTF8	牕
U+7255.0	fCNS11643	3-4D2E
U+7255.0	fGB	E-3265
U+7255.0	fR/S	  91.11
U+7255.0	fTotalStrokes	 15
U+7255.0	fCangjie	LLHWP
U+7255.0	fFourCorner	26030
U+7255.0	fHakka	  cung1
U+7255.0	fCantonese	coeng1
U+7255.0	fMandarin	chuang1; cong1
U+7255.0	fEnglish	window
U+7256.0	fUTF8	牖
U+7256.0	fCNS11643	1-6D62
U+7256.0	fBig5	BCF8
U+7256.0	fGB	0-6B3B
U+7256.0	fR/S	 91.11
U+7256.0	fTotalStrokes	 15
U+7256.0	fCangjie	LLHSB
U+7256.0	fFourCorner	27027
U+7256.0	fHakka	  ju3; riu3; jiu3
U+7256.0	fCantonese	jau5
U+7256.0	fMandarin	you3
U+7256.0	fEnglish	[1] window [2] guide; educate; enlighten
U+7257.0	fUTF8	牗
U+7257.0	fCNS11643	3-4D2D
U+7257.0	fGB	E-3266
U+7257.0	fR/S	 91.11
U+7257.0	fTotalStrokes	 15
U+7257.0	fCangjie	LNILB
U+7257.0	fFourCorner	20020
U+7257.0	fCantonese	jau5
U+7257.0	fMandarin	you3
U+7258.0	fUTF8	牘
U+7258.0	fCNS11643	1-7849
U+7258.0	fBig5	C37C
U+7258.0	fGB	1-6B39
U+7258.0	fR/S	 91.15
U+7258.0	fTotalStrokes	 19
U+7258.0	fCangjie	LLGWC
U+7258.0	fFourCorner	24086
U+7258.0	fHakka	  tuk8
U+7258.0	fCantonese	duk6; duk2 (also)
U+7258.0	fMandarin	du2
U+7258.0	fEnglish	[1] wooden writing tablet [2] documents; books; archive; letters [3] a hollow pole used to strike the ground to mark the beat of music in ancient times
U+7259.0	fUTF8	牙
U+7259.0	fCNS11643	1-455B
U+7259.0	fBig5	A4FA
U+7259.0	fGB	0-5140
U+7259.0	fR/S	 92.0
U+7259.0	fTotalStrokes	 4
U+7259.0	fCangjie	MVDH
U+7259.0	fFourCorner	10240
U+7259.0	fHakka	  nga2; nga3
U+7259.0	fCantonese	ngaa4; aa4 (also); ngaa2 (also); aa2 (also)
U+7259.0	fMandarin	ya2; ya4; ya3
U+7259.0	fEnglish	[1] teeth; tooth [2] bite [3] ivory articles [4] broker [5] Kangxi radical 92
U+725A.0	fUTF8	牚
U+725A.0	fCNS11643	2-3D6B
U+725A.0	fBig5	DA67
U+725A.0	fGB	3-463B
U+725A.0	fR/S	 92.8
U+725A.0	fTotalStrokes	 12
U+725A.0	fCangjie	FBRMH
U+725A.0	fFourCorner	90240
U+725A.0	fHakka	 cang1;  cang5;  tong3
U+725A.0	fCantonese	caang1; caang3; cang1
U+725A.0	fMandarin	cheng4; cheng1; teng4
U+725A.0	fEnglish	 prop; shore
U+725B.0	fUTF8	牛
U+725B.0	fCNS11643	1-455C
U+725B.0	fBig5	A4FB
U+725B.0	fGB	0-4523
U+725B.0	fR/S	 93.0
U+725B.0	fTotalStrokes	 4
U+725B.0	fCangjie	HQ
U+725B.0	fFourCorner	25000
U+725B.0	fHakka	 ngiu2; ngieu2; gniu3; niu2; ngeu2
U+725B.0	fCantonese	ngau4; au4 (also); ngau2 (also); au2 (also)
U+725B.0	fMandarin	niu2
U+725B.0	fEnglish	[1] cow; ox; bull; cattle [2] a Chinese family name [3] stubborn; headstrong (person) [4] Kangxi radical 93
U+725C.0	fUTF8	牜
U+725C.0	fGB	E-3267
U+725C.0	fR/S	 93.0
U+725C.0	fTotalStrokes	 4
U+725C.0	fCangjie	HQ
U+725C.0	fMandarin	cheng1
U+725C.0	fEnglish	[1] ox [2] variant of Kangxi radical 93
U+725D.0	fUTF8	牝
U+725D.0	fCNS11643	1-484A
U+725D.0	fBig5	A6C9
U+725D.0	fGB	0-6A72
U+725D.0	fR/S	 93.2
U+725D.0	fTotalStrokes	 6
U+725D.0	fCangjie	HQP
U+725D.0	fFourCorner	21510
U+725D.0	fHakka	  pi3; pin5; pin6; pin3
U+725D.0	fCantonese	pan5; fan5
U+725D.0	fMandarin	pin4; bin4; bi4
U+725D.0	fEnglish	female of species; female of an animal
U+725E.0	fUTF8	牞
U+725E.0	fCNS11643	2-2262
U+725E.0	fBig5	CA42
U+725E.0	fGB	5-4A7B
U+725E.0	fR/S	 93.2
U+725E.0	fTotalStrokes	 6
U+725E.0	fCangjie	HQKS
U+725E.0	fFourCorner	24527
U+725E.0	fCantonese	kau1; gau1
U+725E.0	fMandarin	jiu1
U+725F.0	fUTF8	牟
U+725F.0	fCNS11643	1-4849
U+725F.0	fBig5	A6C8
U+725F.0	fGB	0-4432
U+725F.0	fR/S	 93.2
U+725F.0	fTotalStrokes	 6
U+725F.0	fCangjie	IHQ
U+725F.0	fFourCorner	23500
U+725F.0	fHakka	  meu2; miu2
U+725F.0	fCantonese	mau4; muk6
U+725F.0	fMandarin	mou2; mao4; mu4; mu3
U+725F.0	fEnglish	[1] seek [2] bellow; low [3] a Chinese family name
U+7260.0	fUTF8	牠
U+7260.0	fCNS11643	1-4B28
U+7260.0	fBig5	A865
U+7260.0	fGB	E-3268
U+7260.0	fR/S	 93.3
U+7260.0	fTotalStrokes	 7
U+7260.0	fCangjie	HQPD
U+7260.0	fFourCorner	24512
U+7260.0	fHakka	 ta1
U+7260.0	fCantonese	taa1; to1 (also)
U+7260.0	fMandarin	ta1; tuo1; tuo2
U+7260.0	fEnglish	[1] it; third pronoun for animals [2] a cow without horns
U+7261.0	fUTF8	牡
U+7261.0	fCNS11643	1-4B27
U+7261.0	fBig5	A864
U+7261.0	fGB	0-4435
U+7261.0	fR/S	 93.3
U+7261.0	fTotalStrokes	 7
U+7261.0	fCangjie	HQG
U+7261.0	fFourCorner	24510
U+7261.0	fHakka	 meu3; meu5; miu3; miau3
U+7261.0	fCantonese	maau5; mau5 (also)
U+7261.0	fMandarin	mu3; mou3
U+7261.0	fEnglish	a male of animals
U+7262.0	fUTF8	牢
U+7262.0	fCNS11643	1-4B26
U+7262.0	fBig5	A863
U+7262.0	fGB	0-404E
U+7262.0	fR/S	 93.3
U+7262.0	fTotalStrokes	 7
U+7262.0	fCangjie	JHQ
U+7262.0	fFourCorner	30502
U+7262.0	fHakka	  lau2; no2; lo3
U+7262.0	fCantonese	lou4; lou6
U+7262.0	fMandarin	lao2; lou2; lao4
U+7262.0	fEnglish	[1] prison; jail [2] stable; pen; cage [3] secure; stable; firm; fast [4] worried; concerned [5] sacrifice
U+7263.0	fUTF8	牣
U+7263.0	fCNS11643	2-2460
U+7263.0	fBig5	CB60
U+7263.0	fGB	3-4877
U+7263.0	fR/S	 93.3
U+7263.0	fTotalStrokes	 7
U+7263.0	fCangjie	HQSHI
U+7263.0	fFourCorner	27520
U+7263.0	fHakka	 ngiun3; ngiun5; ngiun6
U+7263.0	fCantonese	jan6
U+7263.0	fMandarin	ren4
U+7263.0	fEnglish	 fill; stuff full
U+7264.0	fUTF8	牤
U+7264.0	fCNS11643	3-2664
U+7264.0	fGB	3-4876
U+7264.0	fR/S	  93.3
U+7264.0	fTotalStrokes	 7
U+7264.0	fCangjie	HQYV
U+7264.0	fFourCorner	20510
U+7264.0	fCantonese	mong1; mong4 (also)
U+7264.0	fMandarin	mang2; mang1
U+7265.0	fUTF8	牥
U+7265.0	fCNS11643	3-2A21
U+7265.0	fGB	3-487D
U+7265.0	fR/S	  93.4
U+7265.0	fTotalStrokes	 8
U+7265.0	fCangjie	HQYHS
U+7265.0	fFourCorner	20520
U+7265.0	fCantonese	fong1
U+7265.0	fMandarin	fang1
U+7266.0	fUTF8	牦
U+7266.0	fCNS11643	3-2A22
U+7266.0	fGB	0-6A73
U+7266.0	fR/S	 93.4
U+7266.0	fTotalStrokes	 8
U+7266.0	fCangjie	HQHQU
U+7266.0	fFourCorner	22510
U+7266.0	fHakka	 mau2
U+7266.0	fCantonese	mou4; lei4 (also); moi4
U+7266.0	fMandarin	mao2; li2
U+7266.0	fEnglish	 tail hair yak
U+7267.0	fUTF8	牧
U+7267.0	fCNS11643	1-4E6B
U+7267.0	fBig5	AAAA
U+7267.0	fGB	0-4441
U+7267.0	fR/S	 93.4
U+7267.0	fTotalStrokes	 8
U+7267.0	fCangjie	HQOK
U+7267.0	fFourCorner	28540
U+7267.0	fHakka	 muk8; muk7
U+7267.0	fCantonese	muk6
U+7267.0	fMandarin	mu4; mao4
U+7267.0	fEnglish	[1] pasture; shepherd; tend cattle [2] pasture [3] govern [4] magistrate; public administrator
U+7268.0	fUTF8	牨
U+7268.0	fCNS11643	3-2A23
U+7268.0	fGB	5-4B22
U+7268.0	fR/S	  93.4
U+7268.0	fTotalStrokes	 8
U+7268.0	fCangjie	HQYHN
U+7268.0	fFourCorner	20510
U+7268.0	fCantonese	gong1
U+7268.0	fMandarin	gang1
U+7269.0	fUTF8	物
U+7269.0	fCNS11643	1-4E6C
U+7269.0	fBig5	AAAB
U+7269.0	fGB	0-4E6F
U+7269.0	fR/S	 93.4
U+7269.0	fTotalStrokes	 8
U+7269.0	fCangjie	HQPHH
U+7269.0	fFourCorner	27520
U+7269.0	fHakka	  wut8; vut8
U+7269.0	fCantonese	mat6; mat2 (also)
U+7269.0	fMandarin	wu4
U+7269.0	fEnglish	[1] thing; substance; creature; being; matter [2] content; substance [3] the physical world; nature [4] other people
U+7269.1	fUTF8	物
U+7269.1	fCNS11643	1-4E6C
U+7269.1	fBig5	AAAB
U+7269.1	fGB	0-4E6F
U+7269.1	fR/S	 93.4
U+7269.1	fTotalStrokes	 8
U+7269.1	fCangjie	HQPHH
U+7269.1	fFourCorner	27520
U+7269.1	fMandarin	wu5
U+726A.0	fUTF8	牪
U+726A.0	fCNS11643	2-277B
U+726A.0	fBig5	CD5B
U+726A.0	fGB	5-4A7E
U+726A.0	fR/S	 93.4
U+726A.0	fTotalStrokes	 8
U+726A.0	fCangjie	HQHQ
U+726A.0	fFourCorner	25500
U+726A.0	fCantonese	jyun6; jin6
U+726A.0	fMandarin	yan4
U+726B.0	fUTF8	牫
U+726B.0	fCNS11643	4-2731
U+726B.0	fGB	E-3269
U+726B.0	fR/S	  93.4
U+726B.0	fTotalStrokes	 8
U+726B.0	fCangjie	HQI
U+726B.0	fFourCorner	23550
U+726B.0	fCantonese	go1
U+726B.0	fMandarin	ge1
U+726C.0	fUTF8	牬
U+726C.0	fCNS11643	2-2B7A
U+726C.0	fBig5	CFBA
U+726C.0	fGB	E-326A
U+726C.0	fR/S	 93.4
U+726C.0	fTotalStrokes	 8
U+726C.0	fCangjie	HQLXH
U+726C.0	fFourCorner	25527
U+726C.0	fCantonese	bui3
U+726C.0	fMandarin	bei4
U+726D.0	fUTF8	牭
U+726D.0	fCNS11643	3-2D6D
U+726D.0	fGB	3-4922
U+726D.0	fR/S	  93.5
U+726D.0	fTotalStrokes	 9
U+726D.0	fCangjie	HQWC
U+726D.0	fFourCorner	26500
U+726D.0	fCantonese	sei3
U+726D.0	fMandarin	si4
U+726E.0	fUTF8	牮
U+726E.0	fCNS11643	2-2B7D
U+726E.0	fBig5	CFBD
U+726E.0	fGB	0-6A70
U+726E.0	fR/S	 93.5
U+726E.0	fTotalStrokes	 9
U+726E.0	fCangjie	OPHQ
U+726E.0	fFourCorner	23504
U+726E.0	fHakka	  zien5
U+726E.0	fCantonese	zin3
U+726E.0	fMandarin	jian4
U+726E.0	fEnglish	 prop; shore
U+726F.0	fUTF8	牯
U+726F.0	fCNS11643	1-523D
U+726F.0	fBig5	ACBA
U+726F.0	fGB	0-6A74
U+726F.0	fR/S	 93.5
U+726F.0	fTotalStrokes	 9
U+726F.0	fCangjie	HQJR
U+726F.0	fFourCorner	24560
U+726F.0	fHakka	  gu3
U+726F.0	fCantonese	gu2; gwu2 (also)
U+726F.0	fMandarin	gu3
U+726F.0	fEnglish	[1] cow; bull; ox [2] castrated bull
U+7270.0	fUTF8	牰
U+7270.0	fCNS11643	2-2B7B
U+7270.0	fBig5	CFBB
U+7270.0	fGB	E-326B
U+7270.0	fR/S	 93.5
U+7270.0	fTotalStrokes	 9
U+7270.0	fCangjie	HQLW
U+7270.0	fFourCorner	25060
U+7270.0	fCantonese	jau6; cau1; zau6
U+7270.0	fMandarin	you4; xiu4; chou1; jiu4
U+7271.0	fUTF8	牱
U+7271.0	fCNS11643	4-2A2F
U+7271.0	fGB	E-326C
U+7271.0	fR/S	  93.5
U+7271.0	fTotalStrokes	 9
U+7271.0	fCangjie	HQMNR
U+7271.0	fFourCorner	21520
U+7271.0	fCantonese	ngo1 (also)
U+7271.0	fMandarin	ge1
U+7272.0	fUTF8	牲
U+7272.0	fCNS11643	1-523C
U+7272.0	fBig5	ACB9
U+7272.0	fGB	0-497C
U+7272.0	fR/S	 93.5
U+7272.0	fTotalStrokes	 9
U+7272.0	fCangjie	HQHQM
U+7272.0	fFourCorner	25510
U+7272.0	fHakka	  sang1;  sen1
U+7272.0	fCantonese	saang1 (also); sang1
U+7272.0	fMandarin	sheng1
U+7272.0	fEnglish	 livestock; sacrificial animal
U+7273.0	fUTF8	牳
U+7273.0	fCNS11643	2-2B7C
U+7273.0	fBig5	CFBC
U+7273.0	fGB	3-4924
U+7273.0	fR/S	 93.5
U+7273.0	fTotalStrokes	 9
U+7273.0	fCangjie	HQWYI
U+7273.0	fFourCorner	27550
U+7273.0	fHakka	 mu1; muk8
U+7273.0	fCantonese	mau5
U+7273.0	fMandarin	mu3
U+7274.0	fUTF8	牴
U+7274.0	fCNS11643	1-523E
U+7274.0	fBig5	ACBB
U+7274.0	fGB	E-326D
U+7274.0	fR/S	 93.5
U+7274.0	fTotalStrokes	 9
U+7274.0	fCangjie	HQHPM
U+7274.0	fFourCorner	22540
U+7274.0	fHakka	 di3
U+7274.0	fCantonese	dai2
U+7274.0	fMandarin	di3; di1; chai1
U+7274.0	fEnglish	 gore
U+7275.0	fUTF8	牵
U+7275.0	fGB	0-4723
U+7275.0	fR/S	 37.6; 93.5
U+7275.0	fTotalStrokes	 9
U+7275.0	fCangjie	KBHQ
U+7275.0	fHakka	 ken1; kien1
U+7275.0	fCantonese	hin1
U+7275.0	fMandarin	qian1; qian4
U+7275.0	fEnglish	simplified form of &#x727D;
U+7276.0	fUTF8	牶
U+7276.0	fCNS11643	2-3063
U+7276.0	fBig5	D2A2
U+7276.0	fGB	5-4B2B
U+7276.0	fR/S	 93.6
U+7276.0	fTotalStrokes	 10
U+7276.0	fCangjie	FQHQ
U+7276.0	fFourCorner	90500
U+7276.0	fHakka	 kien5; ken5; k'ian5; kian5
U+7276.0	fCantonese	hyun3; kin6
U+7276.0	fMandarin	quan4
U+7276.0	fEnglish	the ring put through an animal's nose to lead it by
U+7277.0	fUTF8	牷
U+7277.0	fCNS11643	2-3062
U+7277.0	fBig5	D2A1
U+7277.0	fGB	3-4927
U+7277.0	fR/S	 93.6
U+7277.0	fTotalStrokes	 10
U+7277.0	fCangjie	HQOMG
U+7277.0	fFourCorner	28514
U+7277.0	fHakka	  cien2; cion2
U+7277.0	fCantonese	cyun4
U+7277.0	fMandarin	quan2
U+7277.0	fEnglish	[1] cow or ox of one color [2] intact (sacrifice)
U+7278.0	fUTF8	牸
U+7278.0	fCNS11643	2-3061
U+7278.0	fBig5	D27E
U+7278.0	fGB	3-4929
U+7278.0	fR/S	 93.6
U+7278.0	fTotalStrokes	 10
U+7278.0	fCangjie	HQJND
U+7278.0	fFourCorner	23547
U+7278.0	fHakka	  ci5; cih6; cih5; ci3; ci6; ze5
U+7278.0	fCantonese	zi6; zi3
U+7278.0	fMandarin	zi4
U+7278.0	fEnglish	[1] the female of certain animals [2] cow
U+7279.0	fUTF8	特
U+7279.0	fCNS11643	1-5656
U+7279.0	fBig5	AF53
U+7279.0	fGB	0-4C58
U+7279.0	fR/S	 93.6
U+7279.0	fTotalStrokes	 10
U+7279.0	fCangjie	HQGDI
U+7279.0	fFourCorner	24541
U+7279.0	fHakka	  tit8
U+7279.0	fCantonese	dak6
U+7279.0	fMandarin	te4
U+7279.0	fEnglish	[1] special; unique; distinguished; peculiar; particular; outstanding; unusual; extraordinary; exclusive [2] just; merely; only [3] bull
U+727A.0	fUTF8	牺
U+727A.0	fCNS11643	3-3236
U+727A.0	fGB	0-4E7E
U+727A.0	fR/S	 93.6
U+727A.0	fTotalStrokes	 10
U+727A.0	fCangjie	HQMCW
U+727A.0	fFourCorner	21560
U+727A.0	fHakka	 hi1
U+727A.0	fCantonese	hei1
U+727A.0	fMandarin	xi1; suo1
U+727A.0	fEnglish	simplified form of &#x72A7;
U+727B.0	fUTF8	牻
U+727B.0	fCNS11643	2-3722
U+727B.0	fBig5	D65D
U+727B.0	fGB	5-4B2D
U+727B.0	fR/S	 93.7
U+727B.0	fTotalStrokes	 11
U+727B.0	fCangjie	HQIUH
U+727B.0	fFourCorner	23512
U+727B.0	fHakka	 mang2
U+727B.0	fCantonese	mong4
U+727B.0	fMandarin	mang2
U+727C.0	fUTF8	牼
U+727C.0	fCNS11643	2-3723
U+727C.0	fBig5	D65E
U+727C.0	fGB	3-4923
U+727C.0	fR/S	 93.7
U+727C.0	fTotalStrokes	 11
U+727C.0	fCangjie	HQMVM
U+727C.0	fFourCorner	21511
U+727C.0	fHakka	 gin1; gen1
U+727C.0	fCantonese	hang1
U+727C.0	fMandarin	keng1; qian1; jing4; jing1
U+727C.0	fEnglish	man
U+727D.0	fUTF8	牽
U+727D.0	fCNS11643	1-5B74
U+727D.0	fBig5	B26F
U+727D.0	fGB	1-4723
U+727D.0	fR/S	 93.7
U+727D.0	fTotalStrokes	 11
U+727D.0	fCangjie	YVBQ
U+727D.0	fFourCorner	00503
U+727D.0	fHakka	  kien1;  ken1; k'ian1; kian1; kian5
U+727D.0	fCantonese	hin1
U+727D.0	fMandarin	qian1; qian4
U+727D.0	fEnglish	[1] drag; pull; lead by hand; lead along; tug; haul [2] involve; affect [3] control; restrain
U+727E.0	fUTF8	牾
U+727E.0	fCNS11643	2-3721
U+727E.0	fBig5	D65C
U+727E.0	fGB	0-6A75
U+727E.0	fR/S	 93.7
U+727E.0	fTotalStrokes	 11
U+727E.0	fCangjie	HQMMR
U+727E.0	fFourCorner	21561
U+727E.0	fHakka	 ngu2; ngu6; ng3; ngu5
U+727E.0	fCantonese	ng5; ng6
U+727E.0	fMandarin	wu2; wu3
U+727E.0	fEnglish	[1] oppose [2] gore
U+727F.0	fUTF8	牿
U+727F.0	fCNS11643	2-3724
U+727F.0	fBig5	D65F
U+727F.0	fGB	0-6A76
U+727F.0	fR/S	 93.7
U+727F.0	fTotalStrokes	 11
U+727F.0	fCangjie	HQHGR
U+727F.0	fFourCorner	24561
U+727F.0	fHakka	 guk7; kuk7; kwuk7
U+727F.0	fCantonese	guk1
U+727F.0	fMandarin	gu4
U+727F.0	fEnglish	a shed or pen for cattle
U+7280.0	fUTF8	犀
U+7280.0	fCNS11643	1-6058
U+7280.0	fBig5	B552
U+7280.0	fGB	0-4F2C
U+7280.0	fR/S	 93.8
U+7280.0	fTotalStrokes	 12
U+7280.0	fCangjie	SEHQ
U+7280.0	fFourCorner	77251
U+7280.0	fHakka	  si1; ci1; chi1; cih1; shi1
U+7280.0	fCantonese	sai1
U+7280.0	fMandarin	xi1
U+7280.0	fEnglish	[1] the rhinoceros [2] sharp-edged and hard (weapons, armor, etc.)
U+7281.0	fUTF8	犁
U+7281.0	fCNS11643	1-5B75
U+7281.0	fBig5	B270
U+7281.0	fGB	0-4067
U+7281.0	fR/S	 93.7
U+7281.0	fTotalStrokes	 11
U+7281.0	fCangjie	HNHQ
U+7281.0	fFourCorner	22500
U+7281.0	fHakka	  lai2;  li2
U+7281.0	fCantonese	lai4
U+7281.0	fMandarin	li2; liu2
U+7281.0	fEnglish	[1] plow; till [2] plough
U+7282.0	fUTF8	犂
U+7282.0	fCNS11643	3-3D24
U+7282.0	fGB	E-326E
U+7282.0	fR/S	  93.8
U+7282.0	fTotalStrokes	 12
U+7282.0	fCangjie	HHHQ
U+7282.0	fFourCorner	27500
U+7282.0	fHakka	 lai2;  li2; lai3
U+7282.0	fCantonese	lai4
U+7282.0	fMandarin	li2; liu2
U+7282.0	fEnglish	plow
U+7283.0	fUTF8	犃
U+7283.0	fCNS11643	3-3D23
U+7283.0	fGB	3-4930
U+7283.0	fR/S	  93.8
U+7283.0	fTotalStrokes	 12
U+7283.0	fCangjie	HQYTR
U+7283.0	fFourCorner	20560
U+7283.0	fCantonese	bau6
U+7283.0	fMandarin	pou3
U+7284.0	fUTF8	犄
U+7284.0	fCNS11643	1-6057
U+7284.0	fBig5	B551
U+7284.0	fGB	0-6A77
U+7284.0	fR/S	 93.8
U+7284.0	fTotalStrokes	 12
U+7284.0	fCangjie	HQKMR
U+7284.0	fFourCorner	24521
U+7284.0	fHakka	 ki3; ji1; ri1
U+7284.0	fCantonese	gei1; ji2; gei2
U+7284.0	fMandarin	yi1; ji1
U+7284.0	fEnglish	 animal horns; horns
U+7285.0	fUTF8	犅
U+7285.0	fCNS11643	2-3D6F
U+7285.0	fBig5	DA6B
U+7285.0	fGB	3-487A
U+7285.0	fR/S	 93.8
U+7285.0	fTotalStrokes	 12
U+7285.0	fCangjie	HQBTU
U+7285.0	fFourCorner	27520
U+7285.0	fHakka	  gong1
U+7285.0	fCantonese	gong1
U+7285.0	fMandarin	gang1
U+7285.0	fEnglish	[1] red bull [2] sacrificial ox
U+7286.0	fUTF8	犆
U+7286.0	fCNS11643	2-3D6E
U+7286.0	fBig5	DA6A
U+7286.0	fGB	3-492E
U+7286.0	fR/S	 93.8
U+7286.0	fTotalStrokes	 12
U+7286.0	fCangjie	HQJBM
U+7286.0	fFourCorner	24512
U+7286.0	fHakka	 tit8
U+7286.0	fCantonese	dat6; zik6; dak6 (also)
U+7286.0	fMandarin	zhi2; te4
U+7286.0	fEnglish	special
U+7287.0	fUTF8	犇
U+7287.0	fCNS11643	3-3D25
U+7287.0	fGB	8-2F73
U+7287.0	fR/S	 93.8
U+7287.0	fTotalStrokes	 12
U+7287.0	fCangjie	HQHQQ
U+7287.0	fFourCorner	25550
U+7287.0	fHakka	 bun1; pun1
U+7287.0	fCantonese	ban1
U+7287.0	fMandarin	ben1
U+7287.0	fEnglish	variant of @
U+7288.0	fUTF8	犈
U+7288.0	fCNS11643	2-3D6C
U+7288.0	fBig5	DA68
U+7288.0	fGB	5-4B34
U+7288.0	fR/S	 93.8
U+7288.0	fTotalStrokes	 12
U+7288.0	fCangjie	HQFQU
U+7288.0	fFourCorner	29512
U+7288.0	fCantonese	kyun4; gyun3
U+7288.0	fMandarin	quan2
U+7289.0	fUTF8	犉
U+7289.0	fCNS11643	2-3D6D
U+7289.0	fBig5	DA69
U+7289.0	fGB	3-492F
U+7289.0	fR/S	 93.8
U+7289.0	fTotalStrokes	 12
U+7289.0	fCangjie	HQYRD
U+7289.0	fFourCorner	20547
U+7289.0	fHakka	 shun2; sun2
U+7289.0	fCantonese	seon4; jyun4
U+7289.0	fMandarin	run2; ruan2
U+7289.0	fEnglish	[1] an ox with yellow hair and a black mouth [2] an ox seven feet tall
U+728A.0	fUTF8	犊
U+728A.0	fGB	0-363F
U+728A.0	fR/S	 93.8
U+728A.0	fTotalStrokes	 12
U+728A.0	fCangjie	HQJNK
U+728A.0	fHakka	 tuk8
U+728A.0	fCantonese	duk6
U+728A.0	fMandarin	du2
U+728A.0	fEnglish	simplified form of &#x72A2;
U+728B.0	fUTF8	犋
U+728B.0	fCNS11643	2-3D70
U+728B.0	fBig5	DA6C
U+728B.0	fGB	0-6A78
U+728B.0	fR/S	 93.8
U+728B.0	fTotalStrokes	 12
U+728B.0	fCangjie	HQBMC
U+728B.0	fFourCorner	27581
U+728B.0	fCantonese	geoi6
U+728B.0	fMandarin	ju4
U+728C.0	fUTF8	犌
U+728C.0	fCNS11643	2-446A
U+728C.0	fBig5	DEA6
U+728C.0	fGB	5-4B37
U+728C.0	fR/S	 93.9
U+728C.0	fTotalStrokes	 13
U+728C.0	fCangjie	HQRSE
U+728C.0	fFourCorner	27547
U+728C.0	fCantonese	gaa1
U+728C.0	fMandarin	jia1; xia2
U+728D.0	fUTF8	犍
U+728D.0	fCNS11643	2-4469
U+728D.0	fBig5	DEA5
U+728D.0	fGB	0-6A79
U+728D.0	fR/S	 93.9
U+728D.0	fTotalStrokes	 13
U+728D.0	fCangjie	HQNKQ
U+728D.0	fFourCorner	25544
U+728D.0	fHakka	 gien1; kien5; ken5; ken2; gian1; gen1
U+728D.0	fCantonese	gin1; kin4
U+728D.0	fMandarin	jian1; jian3; qian2
U+728D.0	fEnglish	 castrated bull
U+728E.0	fUTF8	犎
U+728E.0	fCNS11643	2-446D
U+728E.0	fBig5	DEA9
U+728E.0	fGB	3-4933
U+728E.0	fR/S	 93.9
U+728E.0	fTotalStrokes	 13
U+728E.0	fCangjie	GIHQ
U+728E.0	fFourCorner	44500
U+728E.0	fHakka	  fung1
U+728E.0	fCantonese	fung1
U+728E.0	fMandarin	feng1
U+728E.0	fEnglish	the zebu; or humped-ox
U+728F.0	fUTF8	犏
U+728F.0	fCNS11643	4-3E4F
U+728F.0	fGB	0-6A7A
U+728F.0	fR/S	 93.9
U+728F.0	fTotalStrokes	 13
U+728F.0	fCangjie	HQISB
U+728F.0	fFourCorner	23520
U+728F.0	fHakka	 pien1
U+728F.0	fCantonese	pin1
U+728F.0	fMandarin	pian1
U+728F.0	fEnglish	yak-ox
U+7290.0	fUTF8	犐
U+7290.0	fCNS11643	2-446C
U+7290.0	fBig5	DEA8
U+7290.0	fGB	3-4934
U+7290.0	fR/S	 93.9
U+7290.0	fTotalStrokes	 13
U+7290.0	fCangjie	HQHDJ
U+7290.0	fFourCorner	24500
U+7290.0	fCantonese	fo1
U+7290.0	fMandarin	ke1
U+7291.0	fUTF8	犑
U+7291.0	fCNS11643	2-446B
U+7291.0	fBig5	DEA7
U+7291.0	fGB	5-4B35
U+7291.0	fR/S	 93.9
U+7291.0	fTotalStrokes	 13
U+7291.0	fCangjie	HQBUK
U+7291.0	fFourCorner	26534
U+7291.0	fCantonese	gwik1
U+7291.0	fMandarin	ju2
U+7292.0	fUTF8	犒
U+7292.0	fCNS11643	1-6960
U+7292.0	fBig5	BAB9
U+7292.0	fGB	0-6A7B
U+7292.0	fR/S	 93.10
U+7292.0	fTotalStrokes	 14
U+7292.0	fCangjie	HQYRB
U+7292.0	fFourCorner	20527
U+7292.0	fHakka	 kau5;  kau3; gau3; kau1
U+7292.0	fCantonese	hou3
U+7292.0	fMandarin	kao4
U+7292.0	fEnglish	 reward (soldiers, laborers, etc.)
U+7293.0	fUTF8	犓
U+7293.0	fCNS11643	2-4B6F
U+7293.0	fBig5	E2C9
U+7293.0	fGB	5-4B25
U+7293.0	fR/S	 93.10
U+7293.0	fTotalStrokes	 14
U+7293.0	fCangjie	HQPUU
U+7293.0	fFourCorner	27527
U+7293.0	fCantonese	co1
U+7293.0	fMandarin	chu2
U+7294.0	fUTF8	犔
U+7294.0	fCNS11643	3-4775
U+7294.0	fGB	3-487B
U+7294.0	fR/S	 93.10
U+7294.0	fTotalStrokes	 14
U+7294.0	fCangjie	HQOND
U+7294.0	fFourCorner	28510
U+7294.0	fHakka	 hi5; ki5
U+7294.0	fCantonese	hei3
U+7294.0	fMandarin	xi4
U+7294.0	fEnglish	[1] cattle starving for want of food [2] fodder
U+7295.0	fUTF8	犕
U+7295.0	fCNS11643	2-4B6E
U+7295.0	fBig5	E2C8
U+7295.0	fGB	3-4936
U+7295.0	fR/S	 93.10
U+7295.0	fTotalStrokes	 14
U+7295.0	fCangjie	HQTHB
U+7295.0	fFourCorner	24527
U+7295.0	fCantonese	bei6
U+7295.0	fMandarin	bei4; fu2
U+7296.0	fUTF8	犖
U+7296.0	fCNS11643	1-6961
U+7296.0	fBig5	BABA
U+7296.0	fGB	1-5C7D
U+7296.0	fR/S	 93.10
U+7296.0	fTotalStrokes	 14
U+7296.0	fCangjie	FFBHQ
U+7296.0	fFourCorner	99502
U+7296.0	fHakka	  lok7
U+7296.0	fCantonese	lok3; lok6 (also)
U+7296.0	fMandarin	luo4
U+7296.0	fEnglish	[1] spotted ox [2] of many colors
U+7297.0	fUTF8	犗
U+7297.0	fCNS11643	2-4B6D
U+7297.0	fBig5	E2C7
U+7297.0	fGB	3-493B
U+7297.0	fR/S	 93.10
U+7297.0	fTotalStrokes	 14
U+7297.0	fCangjie	HQJQR
U+7297.0	fFourCorner	23561
U+7297.0	fHakka	 gai5; giai5
U+7297.0	fCantonese	gaai3
U+7297.0	fMandarin	jie4
U+7297.0	fEnglish	[1] bullock [2] strong
U+7298.0	fUTF8	犘
U+7298.0	fCNS11643	2-517B
U+7298.0	fBig5	E673
U+7298.0	fGB	5-7972
U+7298.0	fR/S	 93.11
U+7298.0	fTotalStrokes	 15
U+7298.0	fCangjie	IDHQ
U+7298.0	fFourCorner	00259
U+7298.0	fHakka	 ma2
U+7298.0	fCantonese	maa4
U+7298.0	fMandarin	ma2
U+7298.0	fEnglish	old name for the yak
U+7299.0	fUTF8	犙
U+7299.0	fCNS11643	3-4D2F
U+7299.0	fGB	3-4931
U+7299.0	fR/S	  93.11
U+7299.0	fTotalStrokes	 15; 14
U+7299.0	fCangjie	HQIIH
U+7299.0	fFourCorner	22520
U+7299.0	fCantonese	saam1
U+7299.0	fMandarin	san1; sou1
U+729A.0	fUTF8	犚
U+729A.0	fCNS11643	2-517C
U+729A.0	fBig5	E674
U+729A.0	fGB	5-4B3B
U+729A.0	fR/S	 93.11
U+729A.0	fTotalStrokes	 15
U+729A.0	fCangjie	SIHQ
U+729A.0	fFourCorner	74500
U+729A.0	fCantonese	wai3
U+729A.0	fMandarin	wei4
U+729B.0	fUTF8	犛
U+729B.0	fCNS11643	1-6D63
U+729B.0	fBig5	BCF9
U+729B.0	fGB	E-326F
U+729B.0	fR/S	 93.11
U+729B.0	fTotalStrokes	 15
U+729B.0	fCangjie	JKMHQ
U+729B.0	fFourCorner	58251
U+729B.0	fHakka	  li2;  mau2
U+729B.0	fCantonese	lei4 (also); mou4; maau4
U+729B.0	fMandarin	li2; mao2; lai2
U+729B.0	fEnglish	[1] black ox [2] yak
U+729C.0	fUTF8	犜
U+729C.0	fCNS11643	4-512D
U+729C.0	fGB	1-7D31
U+729C.0	fR/S	  93.12
U+729C.0	fTotalStrokes	 16
U+729C.0	fCangjie	HQYDK
U+729C.0	fFourCorner	28540
U+729C.0	fCantonese	deon1
U+729C.0	fMandarin	dun1
U+729D.0	fUTF8	犝
U+729D.0	fCNS11643	2-5845
U+729D.0	fBig5	EA59
U+729D.0	fGB	5-4B3D
U+729D.0	fR/S	 93.12
U+729D.0	fTotalStrokes	 16
U+729D.0	fCangjie	HQYTG
U+729D.0	fFourCorner	20514
U+729D.0	fHakka	  tung2; tung3
U+729D.0	fCantonese	tung4
U+729D.0	fMandarin	tong2
U+729D.0	fEnglish	a calf without horns
U+729E.0	fUTF8	犞
U+729E.0	fCNS11643	2-5846
U+729E.0	fBig5	EA5A
U+729E.0	fGB	E-3270
U+729E.0	fR/S	 93.12
U+729E.0	fTotalStrokes	 16
U+729E.0	fCangjie	HQHKB
U+729E.0	fFourCorner	22527
U+729E.0	fCantonese	kiu4
U+729E.0	fMandarin	se4
U+729F.0	fUTF8	犟
U+729F.0	fCNS11643	5-4E37
U+729F.0	fGB	0-6A71
U+729F.0	fR/S	 93.12; 93.11
U+729F.0	fTotalStrokes	 16; 15
U+729F.0	fCangjie	NIHQ
U+729F.0	fFourCorner	12500
U+729F.0	fCantonese	goeng6
U+729F.0	fMandarin	jiang4
U+729F.0	fEnglish	stubborn
U+72A0.0	fUTF8	犠
U+72A0.0	fGB	E-3271
U+72A0.0	fR/S	 93.13
U+72A0.0	fTotalStrokes	 17
U+72A0.0	fCangjie	HQTGI
U+72A0.0	fCantonese	ji6
U+72A0.0	fMandarin	xi1
U+72A0.0	fEnglish	 sacrifice, give up; sacrificial
U+72A1.0	fUTF8	犡
U+72A1.0	fCNS11643	2-657C
U+72A1.0	fBig5	F272
U+72A1.0	fGB	3-4921
U+72A1.0	fR/S	 93.15
U+72A1.0	fTotalStrokes	 19
U+72A1.0	fCangjie	HQMTB
U+72A1.0	fFourCorner	21527
U+72A1.0	fCantonese	lai6; laai6
U+72A1.0	fMandarin	li4; lai4
U+72A2.0	fUTF8	犢
U+72A2.0	fCNS11643	1-784A
U+72A2.0	fBig5	C37D
U+72A2.0	fGB	1-363F
U+72A2.0	fR/S	 93.15
U+72A2.0	fTotalStrokes	 19
U+72A2.0	fCangjie	HQGWC
U+72A2.0	fFourCorner	24586
U+72A2.0	fHakka	  tuk8
U+72A2.0	fCantonese	duk6
U+72A2.0	fMandarin	du2
U+72A2.0	fEnglish	 calf
U+72A3.0	fUTF8	犣
U+72A3.0	fCNS11643	2-657B
U+72A3.0	fBig5	F271
U+72A3.0	fGB	3-4944
U+72A3.0	fR/S	 93.15
U+72A3.0	fTotalStrokes	 19
U+72A3.0	fCangjie	HQVVV
U+72A3.0	fFourCorner	22516
U+72A3.0	fCantonese	lip6
U+72A3.0	fMandarin	lie4
U+72A4.0	fUTF8	犤
U+72A4.0	fCNS11643	2-657A
U+72A4.0	fBig5	F270
U+72A4.0	fGB	3-493A
U+72A4.0	fR/S	 93.15
U+72A4.0	fTotalStrokes	 19
U+72A4.0	fCangjie	HQWLP
U+72A4.0	fFourCorner	26511
U+72A4.0	fCantonese	bei1
U+72A4.0	fMandarin	pi2; pai2; bei1; ba4
U+72A5.0	fUTF8	犥
U+72A5.0	fCNS11643	2-6578
U+72A5.0	fBig5	F26E
U+72A5.0	fGB	3-4943
U+72A5.0	fR/S	 93.15
U+72A5.0	fTotalStrokes	 19
U+72A5.0	fCangjie	HQIPF
U+72A5.0	fFourCorner	20531
U+72A5.0	fCantonese	piu1
U+72A5.0	fMandarin	piao3; piao1; pao1; pao3; pao4
U+72A6.0	fUTF8	犦
U+72A6.0	fCNS11643	2-6579
U+72A6.0	fBig5	F26F
U+72A6.0	fGB	3-4942
U+72A6.0	fR/S	 93.15
U+72A6.0	fTotalStrokes	 19
U+72A6.0	fCangjie	HQATE
U+72A6.0	fFourCorner	26532
U+72A6.0	fCantonese	bok6; baau3 (also)
U+72A6.0	fMandarin	bo2; bao4
U+72A7.0	fUTF8	犧
U+72A7.0	fCNS11643	1-7A76
U+72A7.0	fBig5	C4EB
U+72A7.0	fGB	1-4E7E
U+72A7.0	fR/S	 93.16
U+72A7.0	fTotalStrokes	 20
U+72A7.0	fCangjie	HQTGS
U+72A7.0	fFourCorner	28553
U+72A7.0	fHakka	 hi1
U+72A7.0	fCantonese	hei1
U+72A7.0	fMandarin	xi1; suo1
U+72A7.0	fEnglish	[1] sacrifice (as homage to a deity); a beast of a uniform color for sacrifice [2] give up (for the sake of something valued for the sake of something else)
U+72A8.0	fUTF8	犨
U+72A8.0	fCNS11643	2-693B
U+72A8.0	fBig5	F46C
U+72A8.0	fGB	3-4945
U+72A8.0	fR/S	 93.16
U+72A8.0	fTotalStrokes	 20
U+72A8.0	fCangjie	OGHQ
U+72A8.0	fFourCorner	20501
U+72A8.0	fHakka	 chiu2; cu2; chu2
U+72A8.0	fCantonese	cau4; cau1 (also); sau4
U+72A8.0	fMandarin	chou1
U+72A8.0	fEnglish	the panting of a cow or ox
U+72A9.0	fUTF8	犩
U+72A9.0	fCNS11643	2-6D5C
U+72A9.0	fBig5	F6EE
U+72A9.0	fGB	3-4946
U+72A9.0	fR/S	 93.18
U+72A9.0	fTotalStrokes	 22
U+72A9.0	fCangjie	HIHQ
U+72A9.0	fFourCorner	26501
U+72A9.0	fCantonese	ngai4; ai4 (also)
U+72A9.0	fMandarin	wei2; wei4
U+72AA.0	fUTF8	犪
U+72AA.0	fCNS11643	2-7127
U+72AA.0	fBig5	F8F7
U+72AA.0	fGB	3-4947
U+72AA.0	fR/S	 93.20
U+72AA.0	fTotalStrokes	 24
U+72AA.0	fCangjie	HQTCE
U+72AA.0	fFourCorner	28547
U+72AA.0	fCantonese	kwai4
U+72AA.0	fMandarin	kui2
U+72AB.0	fUTF8	犫
U+72AB.0	fCNS11643	3-617B
U+72AB.0	fGB	5-4B3F
U+72AB.0	fR/S	  93.23
U+72AB.0	fTotalStrokes	 27
U+72AB.0	fCangjie	OGHQ
U+72AB.0	fFourCorner	20500
U+72AB.0	fHakka	 chu2; chiu2
U+72AB.0	fCantonese	cau1; cau4 (also)
U+72AB.0	fMandarin	chou1
U+72AB.0	fEnglish	protrude
U+72AC.0	fUTF8	犬
U+72AC.0	fCNS11643	1-455D
U+72AC.0	fBig5	A4FC
U+72AC.0	fGB	0-482E
U+72AC.0	fR/S	 94.0
U+72AC.0	fTotalStrokes	 4
U+72AC.0	fCangjie	IK
U+72AC.0	fFourCorner	43030
U+72AC.0	fHakka	 kien3;  ken3; k'ian3; kian3
U+72AC.0	fCantonese	hyun2
U+72AC.0	fMandarin	quan3
U+72AC.0	fEnglish	[1] dog; canine [2] Kangxi radical 94
U+72AD.0	fUTF8	犭
U+72AD.0	fCNS11643	4-2138
U+72AD.0	fGB	0-616B
U+72AD.0	fR/S	 94.0
U+72AD.0	fTotalStrokes	 3
U+72AD.0	fCangjie	KSH
U+72AD.0	fFourCorner	40220
U+72AD.0	fCantonese	hyun2
U+72AD.0	fMandarin	quan3
U+72AD.0	fEnglish	 dog Kangxi radical number 94
U+72AE.0	fUTF8	犮
U+72AE.0	fCNS11643	2-2164
U+72AE.0	fBig5	C9A5
U+72AE.0	fGB	E-3272
U+72AE.0	fR/S	 94.1
U+72AE.0	fTotalStrokes	 5; 4
U+72AE.0	fCangjie	IKK
U+72AE.0	fFourCorner	43040
U+72AE.0	fHakka	  pat8
U+72AE.0	fCantonese	bat6
U+72AE.0	fMandarin	ba2
U+72AE.0	fEnglish	 prick a dog to make him go
U+72AF.0	fUTF8	犯
U+72AF.0	fCNS11643	1-4667
U+72AF.0	fBig5	A5C7
U+72AF.0	fGB	0-3738
U+72AF.0	fR/S	 94.2
U+72AF.0	fTotalStrokes	 5
U+72AF.0	fCangjie	KHSU
U+72AF.0	fFourCorner	47212
U+72AF.0	fHakka	 fam5;  fam1; fam6; fam3; {fam1}
U+72AF.0	fCantonese	faan6; faan2 (also); fan6 (also)
U+72AF.0	fMandarin	fan4
U+72AF.0	fEnglish	[1] commit crime; violate; offend; break (regulations, laws) [2] commit (crimes, etc.) [3] invade; attack; work against [4] criminal [5] have a recurrence of; revert to
U+72B0.0	fUTF8	犰
U+72B0.0	fCNS11643	2-2165
U+72B0.0	fBig5	C9A6
U+72B0.0	fGB	0-616C
U+72B0.0	fR/S	 94.2
U+72B0.0	fTotalStrokes	 5
U+72B0.0	fCangjie	KHKN
U+72B0.0	fFourCorner	44217
U+72B0.0	fCantonese	kau4
U+72B0.0	fMandarin	qiu2
U+72B1.0	fUTF8	犱
U+72B1.0	fCNS11643	3-242C
U+72B1.0	fGB	E-3273
U+72B1.0	fR/S	  94.3
U+72B1.0	fTotalStrokes	 6
U+72B1.0	fCangjie	KHKNI
U+72B1.0	fFourCorner	45217
U+72B1.0	fCantonese	gik1
U+72B1.0	fMandarin	ji3
U+72B2.0	fUTF8	犲
U+72B2.0	fCNS11643	3-242D
U+72B2.0	fGB	5-464E
U+72B2.0	fR/S	 94.3
U+72B2.0	fTotalStrokes	 6
U+72B2.0	fCangjie	KHDH
U+72B2.0	fFourCorner	44200
U+72B2.0	fHakka	  sai2
U+72B2.0	fCantonese	caai4
U+72B2.0	fMandarin	cai2; chai2
U+72B2.0	fEnglish	 wolf cruel; wicked; mean
U+72B3.0	fUTF8	犳
U+72B3.0	fCNS11643	4-2327
U+72B3.0	fGB	3-436C
U+72B3.0	fR/S	 94.3
U+72B3.0	fTotalStrokes	 6
U+72B3.0	fCangjie	KHPI
U+72B3.0	fFourCorner	47220
U+72B3.0	fCantonese	zoek3
U+72B3.0	fMandarin	zhuo2; chuo2
U+72B4.0	fUTF8	犴
U+72B4.0	fCNS11643	2-2263
U+72B4.0	fBig5	CA43
U+72B4.0	fGB	0-616D
U+72B4.0	fR/S	 94.3
U+72B4.0	fTotalStrokes	 6
U+72B4.0	fCangjie	KHMJ
U+72B4.0	fFourCorner	41440
U+72B4.0	fHakka	  gon1; ngan1
U+72B4.0	fCantonese	ngon6; hon4 (also); hon6; on6 (also)
U+72B4.0	fMandarin	an2; an4; jian4; han2; han1; han4; yan4
U+72B4.0	fEnglish	[1] a species of dog with black mouth and nose [2] prison; jail
U+72B5.0	fUTF8	犵
U+72B5.0	fCNS11643	2-2264
U+72B5.0	fBig5	CA44
U+72B5.0	fGB	5-4650
U+72B5.0	fR/S	 94.3
U+72B5.0	fTotalStrokes	 6
U+72B5.0	fCangjie	KHON
U+72B5.0	fFourCorner	48217
U+72B5.0	fHakka	 ngiet7;  giet7; get7
U+72B5.0	fCantonese	hat1; go1
U+72B5.0	fMandarin	jie2; ge1; qi4; he2
U+72B5.0	fEnglish	resolute
U+72B6.0	fUTF8	状
U+72B6.0	fCNS11643	3-266B
U+72B6.0	fGB	0-5734
U+72B6.0	fR/S	 94.3
U+72B6.0	fTotalStrokes	 7; 6
U+72B6.0	fCangjie	LMIK
U+72B6.0	fFourCorner	33130
U+72B6.0	fHakka	 cong5; cong3
U+72B6.0	fCantonese	zong6; zong2 (also)
U+72B6.0	fMandarin	zhuang4
U+72B6.0	fEnglish	simplified form of &#x72C0;
U+72B7.0	fUTF8	犷
U+72B7.0	fGB	0-616E
U+72B7.0	fR/S	 94.3
U+72B7.0	fTotalStrokes	 6
U+72B7.0	fCangjie	KHI
U+72B7.0	fHakka	 kwang3
U+72B7.0	fCantonese	gwong2; gong2 (also); kwong3 (also); kong3 (also)
U+72B7.0	fMandarin	guang3; gong3; jing3
U+72B7.0	fEnglish	simplified form of &#x7377;
U+72B8.0	fUTF8	犸
U+72B8.0	fGB	0-616F
U+72B8.0	fR/S	 94.3
U+72B8.0	fTotalStrokes	 6
U+72B8.0	fCangjie	KHNVM
U+72B8.0	fHakka	 ma5
U+72B8.0	fCantonese	maa5; maa6 (also)
U+72B8.0	fMandarin	ma4; ma3
U+72B8.0	fEnglish	simplified form of &#x7341;
U+72B9.0	fUTF8	犹
U+72B9.0	fCNS11643	3-2669
U+72B9.0	fGB	0-534C
U+72B9.0	fR/S	  94.4
U+72B9.0	fTotalStrokes	 7
U+72B9.0	fCangjie	KHIKU
U+72B9.0	fFourCorner	43210
U+72B9.0	fHakka	 ju2
U+72B9.0	fCantonese	jau4
U+72B9.0	fMandarin	you2
U+72B9.0	fEnglish	simplified form of &#x7336;
U+72BA.0	fUTF8	犺
U+72BA.0	fCNS11643	2-2466
U+72BA.0	fBig5	CB66
U+72BA.0	fGB	3-4374
U+72BA.0	fR/S	 94.4
U+72BA.0	fTotalStrokes	 7
U+72BA.0	fCangjie	KHYHN
U+72BA.0	fFourCorner	40217
U+72BA.0	fHakka	 kong5; kong3
U+72BA.0	fCantonese	kong3
U+72BA.0	fMandarin	kang4; gang3
U+72BA.0	fEnglish	animal
U+72BB.0	fUTF8	犻
U+72BB.0	fCNS11643	3-2666
U+72BB.0	fGB	3-436D
U+72BB.0	fR/S	 94.4; 94.5
U+72BB.0	fTotalStrokes	 7; 8
U+72BB.0	fCangjie	KHJB
U+72BB.0	fFourCorner	45200
U+72BB.0	fCantonese	but6
U+72BB.0	fMandarin	bo2; pei4
U+72BC.0	fUTF8	犼
U+72BC.0	fCNS11643	3-2667
U+72BC.0	fGB	5-4656
U+72BC.0	fR/S	  94.4
U+72BC.0	fTotalStrokes	 7
U+72BC.0	fCangjie	KHNDU
U+72BC.0	fFourCorner	42210
U+72BC.0	fHakka	 k'ieu5; keu5; keu3
U+72BC.0	fCantonese	hau3
U+72BC.0	fMandarin	hou3
U+72BC.0	fEnglish	a fierce Mongolian wolf dragon's head on roofs
U+72BD.0	fUTF8	犽
U+72BD.0	fCNS11643	2-2462
U+72BD.0	fBig5	CB62
U+72BD.0	fGB	3-436F
U+72BD.0	fR/S	 94.4
U+72BD.0	fTotalStrokes	 7
U+72BD.0	fCangjie	KHMVH
U+72BD.0	fFourCorner	41240
U+72BD.0	fHakka	 nga2
U+72BD.0	fCantonese	ngaa6; aa6 (also); ngaa4
U+72BD.0	fMandarin	ya2; ya4
U+72BE.0	fUTF8	犾
U+72BE.0	fCNS11643	3-266A
U+72BE.0	fGB	E-3274
U+72BE.0	fR/S	  94.4
U+72BE.0	fTotalStrokes	 7
U+72BE.0	fCangjie	KHIK
U+72BE.0	fFourCorner	43230
U+72BE.0	fCantonese	ngan4; an4 (also)
U+72BE.0	fMandarin	yin2
U+72BF.0	fUTF8	犿
U+72BF.0	fCNS11643	2-2461
U+72BF.0	fBig5	CB61
U+72BF.0	fGB	3-4373
U+72BF.0	fR/S	 94.4
U+72BF.0	fTotalStrokes	 7
U+72BF.0	fCangjie	KHYY
U+72BF.0	fFourCorner	44230
U+72BF.0	fCantonese	fun1; faan1 (also); bin6
U+72BF.0	fMandarin	huan1; fan1; fan2; fan3
U+72C0.0	fUTF8	狀
U+72C0.0	fCNS11643	1-4E6D
U+72C0.0	fBig5	AAAC
U+72C0.0	fGB	1-5734
U+72C0.0	fR/S	 94.4
U+72C0.0	fTotalStrokes	 8
U+72C0.0	fCangjie	VMIK
U+72C0.0	fFourCorner	33134
U+72C0.0	fHakka	  cong3;  cong5; cong6; chong6
U+72C0.0	fCantonese	zong6; zong2 (also)
U+72C0.0	fMandarin	zhuang4
U+72C0.0	fEnglish	[1] form; appearance; shape; look [2] condition; state; situation [3] written appeal [4] certificate [5] describe; narrate; description
U+72C1.0	fUTF8	狁
U+72C1.0	fCNS11643	2-2465
U+72C1.0	fBig5	CB65
U+72C1.0	fGB	0-6171
U+72C1.0	fR/S	 94.4
U+72C1.0	fTotalStrokes	 7
U+72C1.0	fCangjie	KHIHU
U+72C1.0	fFourCorner	43210
U+72C1.0	fHakka	 jun3; run3
U+72C1.0	fCantonese	wan5
U+72C1.0	fMandarin	yun3
U+72C1.0	fEnglish	the name of a tribe of Scythian nomads to the north of ancient China
U+72C2.0	fUTF8	狂
U+72C2.0	fCNS11643	1-4B2A
U+72C2.0	fBig5	A867
U+72C2.0	fGB	0-3F71
U+72C2.0	fR/S	 94.4
U+72C2.0	fTotalStrokes	 7
U+72C2.0	fCangjie	KHMG
U+72C2.0	fFourCorner	41214
U+72C2.0	fHakka	  kong2; kong3; k'wong2
U+72C2.0	fCantonese	kong4; kwong4 (also)
U+72C2.0	fMandarin	kuang2; kuang1; jue2
U+72C2.0	fEnglish	[1] insane; mad; crazy; mentally deranged [2] violent [3] wild; unrestrained; uninhibited [4] haughty
U+72C3.0	fUTF8	狃
U+72C3.0	fCNS11643	2-2463
U+72C3.0	fBig5	CB63
U+72C3.0	fGB	0-6170
U+72C3.0	fR/S	 94.4
U+72C3.0	fTotalStrokes	 7
U+72C3.0	fCangjie	KHNG
U+72C3.0	fFourCorner	47215
U+72C3.0	fHakka	  ngiu3; niu3
U+72C3.0	fCantonese	nau2; lau2 (also); nau5
U+72C3.0	fMandarin	niu3; rou3; n4; nuu4
U+72C3.0	fEnglish	[1] covet [2] be accustomed to [3] hold the post of [4] avaricious; greedy
U+72C4.0	fUTF8	狄
U+72C4.0	fCNS11643	1-4B29
U+72C4.0	fBig5	A866
U+72C4.0	fGB	0-3552
U+72C4.0	fR/S	 94.4
U+72C4.0	fTotalStrokes	 7
U+72C4.0	fCangjie	KHF
U+72C4.0	fFourCorner	49280
U+72C4.0	fHakka	  tit8
U+72C4.0	fCantonese	dik6; tik1
U+72C4.0	fMandarin	di2; ti4
U+72C4.0	fEnglish	[1] name of a barbarian tribe from northern China [2] a Chinese family name
U+72C5.0	fUTF8	狅
U+72C5.0	fCNS11643	2-2467
U+72C5.0	fBig5	CB67
U+72C5.0	fGB	3-4370
U+72C5.0	fR/S	 94.4
U+72C5.0	fTotalStrokes	 7
U+72C5.0	fCangjie	KHHG
U+72C5.0	fFourCorner	42214
U+72C5.0	fCantonese	kong4; kwong4
U+72C5.0	fMandarin	qing1
U+72C6.0	fUTF8	狆
U+72C6.0	fCNS11643	2-2464
U+72C6.0	fBig5	CB64
U+72C6.0	fGB	E-3275
U+72C6.0	fR/S	 94.4
U+72C6.0	fTotalStrokes	 7
U+72C6.0	fCangjie	KHL
U+72C6.0	fFourCorner	45206
U+72C6.0	fCantonese	zung1; zung6
U+72C6.0	fMandarin	zhong4
U+72C6.0	fEnglish	 Pekinese dog; lap dog; pug pup
U+72C7.0	fUTF8	狇
U+72C7.0	fCNS11643	3-2668
U+72C7.0	fGB	E-3276
U+72C7.0	fR/S	  94.4
U+72C7.0	fTotalStrokes	 7
U+72C7.0	fCangjie	KHD
U+72C7.0	fFourCorner	44290
U+72C7.0	fCantonese	muk6
U+72C7.0	fMandarin	mu4
U+72C8.0	fUTF8	狈
U+72C8.0	fGB	0-3137
U+72C8.0	fR/S	 94.4
U+72C8.0	fTotalStrokes	 7
U+72C8.0	fCangjie	KHBO
U+72C8.0	fHakka	 bui5
U+72C8.0	fCantonese	bui3
U+72C8.0	fMandarin	bei4
U+72C8.0	fEnglish	simplified form of &#x72FD;
U+72C9.0	fUTF8	狉
U+72C9.0	fCNS11643	2-2821
U+72C9.0	fBig5	CD5F
U+72C9.0	fGB	3-4379
U+72C9.0	fR/S	 94.5
U+72C9.0	fTotalStrokes	 8
U+72C9.0	fCangjie	KHMFM
U+72C9.0	fFourCorner	41219
U+72C9.0	fHakka	  pi1
U+72C9.0	fCantonese	pei1
U+72C9.0	fMandarin	pi1
U+72C9.0	fEnglish	fox-cub
U+72CA.0	fUTF8	狊
U+72CA.0	fCNS11643	2-2B7E
U+72CA.0	fBig5	CFBE
U+72CA.0	fGB	3-592D
U+72CA.0	fR/S	 94.5
U+72CA.0	fTotalStrokes	 9
U+72CA.0	fCangjie	BUIK
U+72CA.0	fFourCorner	60433
U+72CA.0	fCantonese	gwik1
U+72CA.0	fMandarin	ju2
U+72CB.0	fUTF8	狋
U+72CB.0	fCNS11643	2-277D
U+72CB.0	fBig5	CD5D
U+72CB.0	fGB	3-4377
U+72CB.0	fR/S	 94.5
U+72CB.0	fTotalStrokes	 8
U+72CB.0	fCangjie	KHMMF
U+72CB.0	fFourCorner	41291
U+72CB.0	fCantonese	ji4; ngan4 (also); an4 (also)
U+72CB.0	fMandarin	ni2; zui1; yi2; shi2; yin2; quan2
U+72CC.0	fUTF8	狌
U+72CC.0	fCNS11643	2-2826
U+72CC.0	fBig5	CD64
U+72CC.0	fGB	3-437D
U+72CC.0	fR/S	 94.5
U+72CC.0	fTotalStrokes	 8
U+72CC.0	fCangjie	KHHQM
U+72CC.0	fFourCorner	45210
U+72CC.0	fHakka	 sin5; sin3; sen3
U+72CC.0	fCantonese	sing1; sang1; saang1; sing3
U+72CC.0	fMandarin	sheng1; xing1; xing4
U+72CC.0	fEnglish	of the fox tribe
U+72CD.0	fUTF8	狍
U+72CD.0	fCNS11643	3-2A26
U+72CD.0	fGB	0-6173
U+72CD.0	fR/S	 94.5
U+72CD.0	fTotalStrokes	 8
U+72CD.0	fCangjie	KHPRU
U+72CD.0	fFourCorner	47210
U+72CD.0	fCantonese	paau4
U+72CD.0	fMandarin	pao2
U+72CD.0	fEnglish	species of deer found in north c
U+72CE.0	fUTF8	狎
U+72CE.0	fCNS11643	1-4E6E
U+72CE.0	fBig5	AAAD
U+72CE.0	fGB	0-6172
U+72CE.0	fR/S	 94.5
U+72CE.0	fTotalStrokes	 8
U+72CE.0	fCangjie	KHWL
U+72CE.0	fFourCorner	46250
U+72CE.0	fHakka	  ap7
U+72CE.0	fCantonese	haap6
U+72CE.0	fMandarin	xia2
U+72CE.0	fEnglish	 show familiarity, intimacy, or disrespect
U+72CF.0	fUTF8	狏
U+72CF.0	fCNS11643	3-2A2A
U+72CF.0	fGB	5-465A
U+72CF.0	fR/S	  94.5
U+72CF.0	fTotalStrokes	 8
U+72CF.0	fCangjie	KHOPD
U+72CF.0	fFourCorner	48210
U+72CF.0	fCantonese	to4
U+72CF.0	fMandarin	tuo2; yi2
U+72D0.0	fUTF8	狐
U+72D0.0	fCNS11643	1-4E71
U+72D0.0	fBig5	AAB0
U+72D0.0	fGB	0-3A7C
U+72D0.0	fR/S	 94.5
U+72D0.0	fTotalStrokes	 8
U+72D0.0	fCangjie	KHHVO
U+72D0.0	fFourCorner	42230
U+72D0.0	fHakka	  fu2; fu3
U+72D0.0	fCantonese	wu4
U+72D0.0	fMandarin	hu2
U+72D0.0	fEnglish	the fox
U+72D1.0	fUTF8	狑
U+72D1.0	fCNS11643	2-2827
U+72D1.0	fBig5	CD65
U+72D1.0	fGB	5-465B
U+72D1.0	fR/S	 94.5
U+72D1.0	fTotalStrokes	 8
U+72D1.0	fCangjie	KHOII
U+72D1.0	fFourCorner	48227
U+72D1.0	fCantonese	ling4
U+72D1.0	fMandarin	ling2
U+72D2.0	fUTF8	狒
U+72D2.0	fCNS11643	2-2823
U+72D2.0	fBig5	CD61
U+72D2.0	fGB	0-6174
U+72D2.0	fR/S	 94.5
U+72D2.0	fTotalStrokes	 8
U+72D2.0	fCangjie	KHLLN
U+72D2.0	fFourCorner	45227
U+72D2.0	fHakka	 fui5; fi5
U+72D2.0	fCantonese	fai3; fei6 (also); fat1 (also); fai6
U+72D2.0	fMandarin	fei4
U+72D2.0	fEnglish	the baboon
U+72D3.0	fUTF8	狓
U+72D3.0	fCNS11643	3-2A27
U+72D3.0	fGB	3-4425
U+72D3.0	fR/S	  94.5
U+72D3.0	fTotalStrokes	 8
U+72D3.0	fCangjie	KHDHE
U+72D3.0	fFourCorner	44240
U+72D3.0	fCantonese	pei1
U+72D3.0	fMandarin	pi1
U+72D4.0	fUTF8	狔
U+72D4.0	fCNS11643	2-2824
U+72D4.0	fBig5	CD62
U+72D4.0	fGB	5-465F
U+72D4.0	fR/S	 94.5
U+72D4.0	fTotalStrokes	 8
U+72D4.0	fCangjie	KHSP
U+72D4.0	fFourCorner	47211
U+72D4.0	fCantonese	nei5; lei5 (also)
U+72D4.0	fMandarin	ni3; ni2
U+72D5.0	fUTF8	狕
U+72D5.0	fCNS11643	4-2736
U+72D5.0	fGB	5-4661
U+72D5.0	fR/S	  94.5
U+72D5.0	fTotalStrokes	 8
U+72D5.0	fCangjie	KHVIS
U+72D5.0	fFourCorner	44220
U+72D5.0	fCantonese	maa1
U+72D5.0	fMandarin	ao3; yao1
U+72D6.0	fUTF8	狖
U+72D6.0	fCNS11643	2-277C
U+72D6.0	fBig5	CD5C
U+72D6.0	fGB	3-4423
U+72D6.0	fR/S	 94.5
U+72D6.0	fTotalStrokes	 8
U+72D6.0	fCangjie	KHJC
U+72D6.0	fFourCorner	43282
U+72D6.0	fHakka	 ju5; riu6
U+72D6.0	fCantonese	jau6
U+72D6.0	fMandarin	you4
U+72D6.0	fEnglish	the name of an animal
U+72D7.0	fUTF8	狗
U+72D7.0	fCNS11643	1-4E70
U+72D7.0	fBig5	AAAF
U+72D7.0	fGB	0-3937
U+72D7.0	fR/S	 94.5
U+72D7.0	fTotalStrokes	 8
U+72D7.0	fCangjie	KHPR
U+72D7.0	fFourCorner	47220
U+72D7.0	fHakka	  geu3;  kieu3; giu3
U+72D7.0	fCantonese	gau2
U+72D7.0	fMandarin	gou3; hou4
U+72D7.0	fEnglish	[1] dog (Canis familiaris) [2] lackey; footman; servile person; follower [3] damned; cursed
U+72D8.0	fUTF8	狘
U+72D8.0	fCNS11643	2-277E
U+72D8.0	fBig5	CD5E
U+72D8.0	fGB	3-437A
U+72D8.0	fR/S	 94.5
U+72D8.0	fTotalStrokes	 8
U+72D8.0	fCangjie	KHIV
U+72D8.0	fFourCorner	43250
U+72D8.0	fHakka	 jet8; zat8
U+72D8.0	fCantonese	hyut3; jyut6
U+72D8.0	fMandarin	yue4; xue4
U+72D8.0	fEnglish	jump
U+72D9.0	fUTF8	狙
U+72D9.0	fCNS11643	1-4E6F
U+72D9.0	fBig5	AAAE
U+72D9.0	fGB	0-3E51
U+72D9.0	fR/S	 94.5
U+72D9.0	fTotalStrokes	 8
U+72D9.0	fCangjie	KHBM
U+72D9.0	fFourCorner	47210
U+72D9.0	fHakka	 zu1; zii1; zih1
U+72D9.0	fCantonese	zeoi1
U+72D9.0	fMandarin	ju1; qu4
U+72D9.0	fEnglish	[1] ape; monkey [2] lie in ambush
U+72DA.0	fUTF8	狚
U+72DA.0	fCNS11643	2-2825
U+72DA.0	fBig5	CD63
U+72DA.0	fGB	3-437B
U+72DA.0	fR/S	 94.5
U+72DA.0	fTotalStrokes	 8
U+72DA.0	fCangjie	KHAM
U+72DA.0	fFourCorner	46210
U+72DA.0	fCantonese	daan3; daam3; taan2; daat3
U+72DA.0	fMandarin	dan4; dan3; da2
U+72DB.0	fUTF8	狛
U+72DB.0	fCNS11643	3-2A28
U+72DB.0	fGB	3-437E
U+72DB.0	fR/S	  94.5
U+72DB.0	fTotalStrokes	 8
U+72DB.0	fCangjie	KHHA
U+72DB.0	fFourCorner	46200
U+72DB.0	fCantonese	baak3
U+72DB.0	fMandarin	po4; bi4; ba4
U+72DB.0	fEnglish	lion-dog shrine guardians
U+72DC.0	fUTF8	狜
U+72DC.0	fCNS11643	2-2822
U+72DC.0	fBig5	CD60
U+72DC.0	fGB	E-3277
U+72DC.0	fR/S	 94.5
U+72DC.0	fTotalStrokes	 8
U+72DC.0	fCangjie	KHJR
U+72DC.0	fFourCorner	44260
U+72DC.0	fCantonese	gu2; gwu2 (also)
U+72DC.0	fMandarin	gu3
U+72DD.0	fUTF8	狝
U+72DD.0	fCNS11643	6-2C7E
U+72DD.0	fGB	8-7C2D
U+72DD.0	fR/S	 94.5
U+72DD.0	fTotalStrokes	 8
U+72DD.0	fCangjie	KHNF
U+72DD.0	fFourCorner	48290
U+72DD.0	fCantonese	sin2
U+72DD.0	fMandarin	mi2; xian3
U+72DD.0	fEnglish	hunt; autumn hunting; to capture with a fine net
U+72DE.0	fUTF8	狞
U+72DE.0	fGB	0-447C
U+72DE.0	fR/S	 94.5
U+72DE.0	fTotalStrokes	 8
U+72DE.0	fCangjie	KHJMN
U+72DE.0	fHakka	 nen2
U+72DE.0	fCantonese	ning4; ling4 (also)
U+72DE.0	fMandarin	ning2
U+72DE.0	fEnglish	simplified form of &#x7370;
U+72DF.0	fUTF8	狟
U+72DF.0	fCNS11643	2-2C24
U+72DF.0	fBig5	CFC2
U+72DF.0	fGB	5-4662
U+72DF.0	fR/S	 94.6
U+72DF.0	fTotalStrokes	 9
U+72DF.0	fCangjie	KHMAM
U+72DF.0	fFourCorner	41216
U+72DF.0	fCantonese	wun4; hyun1
U+72DF.0	fMandarin	huan2; xuan1
U+72E0.0	fUTF8	狠
U+72E0.0	fCNS11643	1-5240
U+72E0.0	fBig5	ACBD
U+72E0.0	fGB	0-3A5D
U+72E0.0	fR/S	 94.6
U+72E0.0	fTotalStrokes	 9
U+72E0.0	fCangjie	KHAV
U+72E0.0	fFourCorner	47232
U+72E0.0	fHakka	  hen3;  ken3; han5; hen6; hen2
U+72E0.0	fCantonese	han2
U+72E0.0	fMandarin	yin2; yan2; wan2; hen3; hen2; ken3; hang3
U+72E0.0	fEnglish	[1] vicious; cruel; atrocious [2] severe(ly); extreme(ly)
U+72E1.0	fUTF8	狡
U+72E1.0	fCNS11643	1-5241
U+72E1.0	fBig5	ACBE
U+72E1.0	fGB	0-3D46
U+72E1.0	fR/S	 94.6
U+72E1.0	fTotalStrokes	 9
U+72E1.0	fCangjie	KHYCK
U+72E1.0	fFourCorner	40248
U+72E1.0	fHakka	  gau3; git7
U+72E1.0	fCantonese	gaau2
U+72E1.0	fMandarin	jiao3; xiao4
U+72E1.0	fEnglish	[1] cunning; crafty; sly; wily; deceitful; artful; shrewd [2] suspicion; suspect
U+72E2.0	fUTF8	狢
U+72E2.0	fCNS11643	3-2D75
U+72E2.0	fGB	5-4668
U+72E2.0	fR/S	 94.6
U+72E2.0	fTotalStrokes	 9
U+72E2.0	fCangjie	KHHER
U+72E2.0	fFourCorner	47260
U+72E2.0	fCantonese	hok6
U+72E2.0	fMandarin	he2
U+72E2.0	fEnglish	animal name
U+72E3.0	fUTF8	狣
U+72E3.0	fCNS11643	2-2C27
U+72E3.0	fBig5	CFC5
U+72E3.0	fGB	5-4667
U+72E3.0	fR/S	 94.6
U+72E3.0	fTotalStrokes	 9
U+72E3.0	fCangjie	KHLMO
U+72E3.0	fFourCorner	42213
U+72E3.0	fCantonese	ziu6; siu6
U+72E3.0	fMandarin	zhao4; dao4; chao2
U+72E4.0	fUTF8	狤
U+72E4.0	fCNS11643	2-2C21
U+72E4.0	fBig5	CFBF
U+72E4.0	fGB	3-4426
U+72E4.0	fR/S	 94.6
U+72E4.0	fTotalStrokes	 9
U+72E4.0	fCangjie	KHGR
U+72E4.0	fFourCorner	44261
U+72E4.0	fHakka	 giet7; git7; get7
U+72E4.0	fCantonese	gat1; git3; gwaai3; git6
U+72E4.0	fMandarin	ji2; jie2; guai4
U+72E4.0	fEnglish	an animal like a monkey with a short tail and black stripe down the back
U+72E5.0	fUTF8	狥
U+72E5.0	fCNS11643	3-2D74
U+72E5.0	fGB	E-3278
U+72E5.0	fR/S	 94.6
U+72E5.0	fTotalStrokes	 9
U+72E5.0	fCangjie	KHPA
U+72E5.0	fFourCorner	47220
U+72E5.0	fHakka	  sun2
U+72E5.0	fCantonese	seon1; seon6; ceon4
U+72E5.0	fMandarin	xun4; xun2
U+72E5.0	fEnglish	follow
U+72E6.0	fUTF8	狦
U+72E6.0	fCNS11643	2-2C26
U+72E6.0	fBig5	CFC4
U+72E6.0	fGB	E-3279
U+72E6.0	fR/S	 94.6
U+72E6.0	fTotalStrokes	 9
U+72E6.0	fCangjie	KHBTT
U+72E6.0	fFourCorner	47250
U+72E6.0	fCantonese	saan1; saan3
U+72E6.0	fMandarin	shan1; shan4
U+72E7.0	fUTF8	狧
U+72E7.0	fCNS11643	3-2D70
U+72E7.0	fGB	3-442D
U+72E7.0	fR/S	 94.6
U+72E7.0	fTotalStrokes	 9
U+72E7.0	fCangjie	KHHJR
U+72E7.0	fFourCorner	42260
U+72E7.0	fCantonese	daat3
U+72E7.0	fMandarin	ta4; shi4
U+72E8.0	fUTF8	狨
U+72E8.0	fCNS11643	2-2C22
U+72E8.0	fBig5	CFC0
U+72E8.0	fGB	0-6175
U+72E8.0	fR/S	 94.6
U+72E8.0	fTotalStrokes	 9
U+72E8.0	fCangjie	KHIJ
U+72E8.0	fFourCorner	43250
U+72E8.0	fHakka	 jung2; rung2
U+72E8.0	fCantonese	jung4
U+72E8.0	fMandarin	rong2
U+72E8.0	fEnglish	[1] large ape [2] violent; fierce
U+72E9.0	fUTF8	狩
U+72E9.0	fCNS11643	1-523F
U+72E9.0	fBig5	ACBC
U+72E9.0	fGB	0-6177
U+72E9.0	fR/S	 94.6
U+72E9.0	fTotalStrokes	 9
U+72E9.0	fCangjie	KHJDI
U+72E9.0	fFourCorner	43242
U+72E9.0	fHakka	 su3;  shiu3; siu3; shu3
U+72E9.0	fCantonese	sau3; sau2 (also)
U+72E9.0	fMandarin	shou4; shou3
U+72E9.0	fEnglish	[1] hunt in winter [2] imperial tour
U+72EA.0	fUTF8	狪
U+72EA.0	fCNS11643	2-2C25
U+72EA.0	fBig5	CFC3
U+72EA.0	fGB	3-442B
U+72EA.0	fR/S	 94.6
U+72EA.0	fTotalStrokes	 9
U+72EA.0	fCangjie	KHBMR
U+72EA.0	fFourCorner	47220
U+72EA.0	fCantonese	tung4; dung6
U+72EA.0	fMandarin	tong1; tong2
U+72EB.0	fUTF8	狫
U+72EB.0	fCNS11643	2-2C23
U+72EB.0	fBig5	CFC1
U+72EB.0	fGB	3-4427
U+72EB.0	fR/S	 94.6
U+72EB.0	fTotalStrokes	 9
U+72EB.0	fCangjie	KHJKP
U+72EB.0	fFourCorner	44211
U+72EB.0	fHakka	 lau3
U+72EB.0	fCantonese	lou5
U+72EB.0	fMandarin	lao3
U+72EB.0	fEnglish	the name of certain savage tribes known as Laos still found in Kuei-chou
U+72EC.0	fUTF8	独
U+72EC.0	fCNS11643	3-2D73
U+72EC.0	fGB	0-3640
U+72EC.0	fR/S	 94.6
U+72EC.0	fTotalStrokes	 9
U+72EC.0	fCangjie	KHLMI
U+72EC.0	fFourCorner	45230
U+72EC.0	fHakka	 tuk8
U+72EC.0	fCantonese	duk6
U+72EC.0	fMandarin	du2
U+72EC.0	fEnglish	simplified form of &#x7368;
U+72ED.0	fUTF8	狭
U+72ED.0	fCNS11643	3-2D71
U+72ED.0	fGB	0-4F41
U+72ED.0	fR/S	 94.6
U+72ED.0	fTotalStrokes	 9
U+72ED.0	fCangjie	KHKT
U+72ED.0	fFourCorner	45230
U+72ED.0	fHakka	 hiap8; kiap8
U+72ED.0	fCantonese	haap6; haap3 (also); hap6 (also); gip6 (also)
U+72ED.0	fMandarin	xia2
U+72ED.0	fEnglish	simplified form of &#x72F9;
U+72EE.0	fUTF8	狮
U+72EE.0	fGB	0-4A28
U+72EE.0	fR/S	 94.6
U+72EE.0	fTotalStrokes	 9
U+72EE.0	fCangjie	KHLLB
U+72EE.0	fHakka	 sii1
U+72EE.0	fCantonese	si1
U+72EE.0	fMandarin	shi1
U+72EE.0	fEnglish	simplified form of &#x7345;
U+72EF.0	fUTF8	狯
U+72EF.0	fGB	0-6176
U+72EF.0	fR/S	 94.6
U+72EF.0	fTotalStrokes	 9
U+72EF.0	fCangjie	KHOMI
U+72EF.0	fHakka	 kwai5
U+72EF.0	fCantonese	kui2; kwui2 (also)
U+72EF.0	fMandarin	kuai4; hua2; guai4
U+72EF.0	fEnglish	simplified form of &#x736A;
U+72F0.0	fUTF8	狰
U+72F0.0	fGB	0-5578
U+72F0.0	fR/S	 94.6
U+72F0.0	fTotalStrokes	 9
U+72F0.0	fCangjie	KHNSD
U+72F0.0	fHakka	 zen1
U+72F0.0	fCantonese	zang1; zaang1 (also)
U+72F0.0	fMandarin	zheng1; jing4
U+72F0.0	fEnglish	simplified form of &#x7319;
U+72F1.0	fUTF8	狱
U+72F1.0	fGB	0-537C
U+72F1.0	fR/S	 94.6
U+72F1.0	fTotalStrokes	 9
U+72F1.0	fCangjie	KHIVK
U+72F1.0	fHakka	 ngiuk8
U+72F1.0	fCantonese	juk6
U+72F1.0	fMandarin	yu4
U+72F1.0	fEnglish	simplified form of &#x7344;
U+72F2.0	fUTF8	狲
U+72F2.0	fGB	0-6178
U+72F2.0	fR/S	 94.6
U+72F2.0	fTotalStrokes	 9
U+72F2.0	fCangjie	XKHND
U+72F2.0	fHakka	 sun1
U+72F2.0	fCantonese	syun1
U+72F2.0	fMandarin	sun1
U+72F2.0	fEnglish	simplified form of &#x733B;
U+72F3.0	fUTF8	狳
U+72F3.0	fCNS11643	2-3069
U+72F3.0	fBig5	D2A8
U+72F3.0	fGB	0-617C
U+72F3.0	fR/S	 94.7
U+72F3.0	fTotalStrokes	 10
U+72F3.0	fCangjie	KHOMD
U+72F3.0	fFourCorner	48294
U+72F3.0	fCantonese	jyu4
U+72F3.0	fMandarin	yu2
U+72F4.0	fUTF8	狴
U+72F4.0	fCNS11643	2-3066
U+72F4.0	fBig5	D2A5
U+72F4.0	fGB	0-6179
U+72F4.0	fR/S	 94.7
U+72F4.0	fTotalStrokes	 10
U+72F4.0	fCangjie	KHPPG
U+72F4.0	fFourCorner	41214
U+72F4.0	fHakka	 pi5; pi1
U+72F4.0	fCantonese	bai6; bai1
U+72F4.0	fMandarin	bi4; bi1
U+72F4.0	fEnglish	[1] prison [2] a legendary beast looking like a tiger
U+72F5.0	fUTF8	狵
U+72F5.0	fCNS11643	3-3238
U+72F5.0	fGB	5-466E
U+72F5.0	fR/S	 94.7
U+72F5.0	fTotalStrokes	 10
U+72F5.0	fCangjie	KHIUH
U+72F5.0	fFourCorner	42212
U+72F5.0	fCantonese	mong4
U+72F5.0	fMandarin	mang2; zhuo2
U+72F5.0	fEnglish	 blend variegated striped
U+72F6.0	fUTF8	狶
U+72F6.0	fCNS11643	2-3068
U+72F6.0	fBig5	D2A7
U+72F6.0	fGB	5-4672
U+72F6.0	fR/S	 94.7
U+72F6.0	fTotalStrokes	 10
U+72F6.0	fCangjie	KHKKB
U+72F6.0	fFourCorner	44227
U+72F6.0	fHakka	  hi1; hi3
U+72F6.0	fCantonese	hei1; hei2
U+72F6.0	fMandarin	xi3; xi1; shi3; chi1; yi2
U+72F6.0	fEnglish	swine
U+72F7.0	fUTF8	狷
U+72F7.0	fCNS11643	1-565B
U+72F7.0	fBig5	AF58
U+72F7.0	fGB	0-617A
U+72F7.0	fR/S	 94.7
U+72F7.0	fTotalStrokes	 10
U+72F7.0	fCangjie	KHRB
U+72F7.0	fFourCorner	46227
U+72F7.0	fHakka	 gien5; gen3; gian5; gen5
U+72F7.0	fCantonese	gyun3; gyun1 (also)
U+72F7.0	fMandarin	juan4; juan3
U+72F7.0	fEnglish	[1] rash; quick-tempered; impulsive [2] honest and straightforward
U+72F8.0	fUTF8	狸
U+72F8.0	fCNS11643	1-565A
U+72F8.0	fBig5	AF57
U+72F8.0	fGB	0-406A
U+72F8.0	fR/S	 94.7
U+72F8.0	fTotalStrokes	 10
U+72F8.0	fCangjie	KHWG
U+72F8.0	fFourCorner	46214
U+72F8.0	fHakka	  li2; li3
U+72F8.0	fCantonese	lei4; lei2 (also)
U+72F8.0	fMandarin	li2; mai2
U+72F8.0	fEnglish	[1] fox [2] raccoon dog
U+72F9.0	fUTF8	狹
U+72F9.0	fCNS11643	1-5658
U+72F9.0	fBig5	AF55
U+72F9.0	fGB	1-4F41
U+72F9.0	fR/S	 94.7
U+72F9.0	fTotalStrokes	 10
U+72F9.0	fCangjie	KHKOO
U+72F9.0	fFourCorner	44238
U+72F9.0	fHakka	  kiap8;  hiap8;  hap8
U+72F9.0	fCantonese	haap6; gip6 (also); haap3 (also); hap6 (also)
U+72F9.0	fMandarin	xia2
U+72F9.0	fEnglish	narrow; limited; narrow-minded
U+72FA.0	fUTF8	狺
U+72FA.0	fCNS11643	2-3065
U+72FA.0	fBig5	D2A4
U+72FA.0	fGB	0-617E
U+72FA.0	fR/S	 94.7
U+72FA.0	fTotalStrokes	 10
U+72FA.0	fCangjie	KHYMR
U+72FA.0	fFourCorner	40261
U+72FA.0	fHakka	  ngiun2; niun2
U+72FA.0	fCantonese	ngan4; an4 (also)
U+72FA.0	fMandarin	yin2
U+72FA.0	fEnglish	the snarling of dogs
U+72FB.0	fUTF8	狻
U+72FB.0	fCNS11643	2-306A
U+72FB.0	fBig5	D2A9
U+72FB.0	fGB	0-6221
U+72FB.0	fR/S	 94.7
U+72FB.0	fTotalStrokes	 10
U+72FB.0	fCangjie	KHICE
U+72FB.0	fFourCorner	43247
U+72FB.0	fHakka	  son1; so1
U+72FB.0	fCantonese	syun1
U+72FB.0	fMandarin	suan1; xun4; jun4
U+72FB.0	fEnglish	a fabulous beast
U+72FC.0	fUTF8	狼
U+72FC.0	fCNS11643	1-5657
U+72FC.0	fBig5	AF54
U+72FC.0	fGB	0-4047
U+72FC.0	fR/S	 94.7
U+72FC.0	fTotalStrokes	 10
U+72FC.0	fCangjie	KHIAV
U+72FC.0	fFourCorner	43232
U+72FC.0	fHakka	  long2; long3
U+72FC.0	fCantonese	long4; long2 (also)
U+72FC.0	fMandarin	lang2; lang3; lang4; hang3
U+72FC.0	fEnglish	[1] the wolf [2] heartless, cruel person; cruel and heartless; cunning and crafty [3] the name of a constellation [4] a Chinese family name
U+72FD.0	fUTF8	狽
U+72FD.0	fCNS11643	1-5659
U+72FD.0	fBig5	AF56
U+72FD.0	fGB	1-3137
U+72FD.0	fR/S	 94.7
U+72FD.0	fTotalStrokes	 10
U+72FD.0	fCangjie	KHBUC
U+72FD.0	fFourCorner	46280
U+72FD.0	fHakka	  bui5; bi5
U+72FD.0	fCantonese	bui3
U+72FD.0	fMandarin	bei4
U+72FD.0	fEnglish	a kind of wolf with shorter forelegs
U+72FE.0	fUTF8	狾
U+72FE.0	fCNS11643	2-3067
U+72FE.0	fBig5	D2A6
U+72FE.0	fGB	5-466F
U+72FE.0	fR/S	 94.7
U+72FE.0	fTotalStrokes	 10
U+72FE.0	fCangjie	KHQHL
U+72FE.0	fFourCorner	42221
U+72FE.0	fCantonese	zai3
U+72FE.0	fMandarin	zhi4; ji4; chi4
U+72FF.0	fUTF8	狿
U+72FF.0	fCNS11643	2-372C
U+72FF.0	fBig5	D667
U+72FF.0	fGB	3-442F
U+72FF.0	fR/S	 94.7
U+72FF.0	fTotalStrokes	 10
U+72FF.0	fCangjie	KHNKM
U+72FF.0	fFourCorner	42241
U+72FF.0	fCantonese	jin4
U+72FF.0	fMandarin	yan2; chan1; yan4
U+7300.0	fUTF8	猀
U+7300.0	fCNS11643	2-3064
U+7300.0	fBig5	D2A3
U+7300.0	fGB	E-327A
U+7300.0	fR/S	 94.7
U+7300.0	fTotalStrokes	 10
U+7300.0	fCangjie	KHEFH
U+7300.0	fFourCorner	49220
U+7300.0	fCantonese	saa1
U+7300.0	fMandarin	sha1
U+7301.0	fUTF8	猁
U+7301.0	fCNS11643	2-306B
U+7301.0	fBig5	D2AA
U+7301.0	fGB	0-617B
U+7301.0	fR/S	 94.7
U+7301.0	fTotalStrokes	 10
U+7301.0	fCangjie	KHHDN
U+7301.0	fFourCorner	42200
U+7301.0	fCantonese	lei6
U+7301.0	fMandarin	li4
U+7301.0	fEnglish	a kind of monkey
U+7302.0	fUTF8	猂
U+7302.0	fCNS11643	3-3239
U+7302.0	fGB	E-327B
U+7302.0	fR/S	 94.7
U+7302.0	fTotalStrokes	 10
U+7302.0	fCangjie	KHAMJ
U+7302.0	fFourCorner	46240
U+7302.0	fCantonese	hon5; hon6 (also)
U+7302.0	fMandarin	han4
U+7303.0	fUTF8	猃
U+7303.0	fGB	0-617D
U+7303.0	fR/S	 94.7
U+7303.0	fTotalStrokes	 10
U+7303.0	fCangjie	KHOMM
U+7303.0	fHakka	 hiam3
U+7303.0	fCantonese	him2
U+7303.0	fMandarin	xian3; lian3; lian4; lian2
U+7303.0	fEnglish	simplified form of &#x736B;
U+7304.0	fUTF8	猄
U+7304.0	fCNS11643	4-3323
U+7304.0	fGB	3-4441
U+7304.0	fR/S	 94.8
U+7304.0	fTotalStrokes	 11
U+7304.0	fCangjie	KHYRF
U+7304.0	fFourCorner	40290
U+7304.0	fCantonese	geng1
U+7304.0	fMandarin	jing1
U+7305.0	fUTF8	猅
U+7305.0	fCNS11643	4-332A
U+7305.0	fGB	E-327C
U+7305.0	fR/S	  94.8
U+7305.0	fTotalStrokes	 11
U+7305.0	fCangjie	KHLMY
U+7305.0	fFourCorner	41210
U+7305.0	fCantonese	paai4
U+7305.0	fMandarin	pai2
U+7306.0	fUTF8	猆
U+7306.0	fCNS11643	3-3D26
U+7306.0	fGB	3-745B
U+7306.0	fR/S	  94.8
U+7306.0	fTotalStrokes	 12; 11
U+7306.0	fCangjie	LYIK
U+7306.0	fFourCorner	11430
U+7306.0	fCantonese	fei1
U+7306.0	fMandarin	fei1
U+7307.0	fUTF8	猇
U+7307.0	fCNS11643	2-3727
U+7307.0	fBig5	D662
U+7307.0	fGB	3-4439
U+7307.0	fR/S	 94.8
U+7307.0	fTotalStrokes	 11
U+7307.0	fCangjie	KHYPU
U+7307.0	fFourCorner	41217
U+7307.0	fHakka	 hau1; ho1
U+7307.0	fCantonese	haau1
U+7307.0	fMandarin	yao2; xiao1; chi2; you2; xiao2
U+7307.0	fEnglish	 the scream or roar of a tiger intimidate scare
U+7308.0	fUTF8	猈
U+7308.0	fCNS11643	2-372B
U+7308.0	fBig5	D666
U+7308.0	fGB	3-443E
U+7308.0	fR/S	 94.8
U+7308.0	fTotalStrokes	 11
U+7308.0	fCangjie	KHHWJ
U+7308.0	fFourCorner	46240
U+7308.0	fCantonese	baa6; paai4; pei4
U+7308.0	fMandarin	ba4; pai2; pi2; pian2
U+7309.0	fUTF8	猉
U+7309.0	fCNS11643	5-3354
U+7309.0	fGB	E-327D
U+7309.0	fR/S	 94.8
U+7309.0	fTotalStrokes	 11
U+7309.0	fCangjie	KHTMC
U+7309.0	fFourCorner	44280
U+7309.0	fCantonese	kei4
U+7309.0	fMandarin	qi2
U+730A.0	fUTF8	猊
U+730A.0	fCNS11643	2-372A
U+730A.0	fBig5	D665
U+730A.0	fGB	0-6225
U+730A.0	fR/S	 94.8
U+730A.0	fTotalStrokes	 11
U+730A.0	fCangjie	KHHXU
U+730A.0	fFourCorner	47217
U+730A.0	fHakka	  ngi2; ji2; ri2
U+730A.0	fCantonese	ngai4; ai4 (also)
U+730A.0	fMandarin	ni2
U+730A.0	fEnglish	[1] the lion [2] wild beast; wild horse
U+730B.0	fUTF8	猋
U+730B.0	fCNS11643	2-3D72
U+730B.0	fBig5	DA6E
U+730B.0	fGB	3-436A
U+730B.0	fR/S	 94.8
U+730B.0	fTotalStrokes	 12
U+730B.0	fCangjie	IKIKK
U+730B.0	fFourCorner	43434
U+730B.0	fHakka	 biau1;  piau1; beu1; peu1
U+730B.0	fCantonese	biu1
U+730B.0	fMandarin	biao1; piao1
U+730B.0	fEnglish	[1] same as $ [2] running fast (dogs) [3] swift; quick; rapid
U+730C.0	fUTF8	猌
U+730C.0	fCNS11643	2-3D7D
U+730C.0	fBig5	DA79
U+730C.0	fGB	E-327E
U+730C.0	fR/S	 94.8
U+730C.0	fTotalStrokes	 12
U+730C.0	fCangjie	DOIK
U+730C.0	fFourCorner	43934
U+730C.0	fCantonese	ngan6; an6 (also); jan6
U+730C.0	fMandarin	yin4
U+730D.0	fUTF8	猍
U+730D.0	fCNS11643	4-332D
U+730D.0	fGB	3-4434
U+730D.0	fR/S	  94.8
U+730D.0	fTotalStrokes	 11
U+730D.0	fCangjie	KHDOO
U+730D.0	fFourCorner	44290
U+730D.0	fCantonese	loi4
U+730D.0	fMandarin	lai2
U+730E.0	fUTF8	猎
U+730E.0	fCNS11643	3-373B
U+730E.0	fGB	0-4154
U+730E.0	fR/S	  94.8
U+730E.0	fTotalStrokes	 11
U+730E.0	fCangjie	KHTA
U+730E.0	fFourCorner	44260
U+730E.0	fHakka	 sit7; cok7; liap8
U+730E.0	fCantonese	lip6
U+730E.0	fMandarin	xi2; lie4; que4
U+730E.0	fEnglish	simplified form of &#x7375;
U+730F.0	fUTF8	猏
U+730F.0	fCNS11643	2-372D
U+730F.0	fBig5	D668
U+730F.0	fGB	E-3321
U+730F.0	fR/S	 94.8
U+730F.0	fTotalStrokes	 11
U+730F.0	fCangjie	KHHSB
U+730F.0	fFourCorner	43227
U+730F.0	fCantonese	gin1
U+730F.0	fMandarin	jian1
U+7310.0	fUTF8	猐
U+7310.0	fCNS11643	3-373A
U+7310.0	fGB	E-3322
U+7310.0	fR/S	 94.8
U+7310.0	fTotalStrokes	 11
U+7310.0	fCangjie	KHTGU
U+7310.0	fFourCorner	48210
U+7310.0	fCantonese	goeng1
U+7310.0	fMandarin	qiang1
U+7311.0	fUTF8	猑
U+7311.0	fCNS11643	2-3728
U+7311.0	fBig5	D663
U+7311.0	fGB	3-443A
U+7311.0	fR/S	 94.8
U+7311.0	fTotalStrokes	 11
U+7311.0	fCangjie	KHAPP
U+7311.0	fFourCorner	46211
U+7311.0	fHakka	 kun1
U+7311.0	fCantonese	kwan1
U+7311.0	fMandarin	kun1
U+7312.0	fUTF8	猒
U+7312.0	fCNS11643	2-3D71
U+7312.0	fBig5	DA6D
U+7312.0	fGB	5-4648
U+7312.0	fR/S	 94.8
U+7312.0	fTotalStrokes	 12
U+7312.0	fCangjie	ABIK
U+7312.0	fFourCorner	63234
U+7312.0	fCantonese	jim3; aat3; ngaat3 (also); jim1; jim2
U+7312.0	fMandarin	yan1; yan4; ya1; xia1; ye1
U+7313.0	fUTF8	猓
U+7313.0	fCNS11643	1-5B79
U+7313.0	fBig5	B274
U+7313.0	fGB	0-6223
U+7313.0	fR/S	 94.8
U+7313.0	fTotalStrokes	 11
U+7313.0	fCangjie	KHWD
U+7313.0	fFourCorner	46294
U+7313.0	fHakka	 go3; kwo3
U+7313.0	fCantonese	gwo2; go2 (also)
U+7313.0	fMandarin	guo3; luo3
U+7313.0	fEnglish	the name of a tribe spread over Yunnan, Kweichow and Szechwan provinces
U+7314.0	fUTF8	猔
U+7314.0	fCNS11643	4-3322
U+7314.0	fGB	3-4443
U+7314.0	fR/S	  94.8
U+7314.0	fTotalStrokes	 11
U+7314.0	fCangjie	KHJMF
U+7314.0	fFourCorner	43290
U+7314.0	fHakka	  zung5
U+7314.0	fCantonese	zung3
U+7314.0	fMandarin	zong4
U+7314.0	fEnglish	the name of an ancient tribe
U+7315.0	fUTF8	猕
U+7315.0	fCNS11643	4-332C
U+7315.0	fGB	0-6228
U+7315.0	fR/S	 94.8
U+7315.0	fTotalStrokes	 11
U+7315.0	fCangjie	KHNNF
U+7315.0	fFourCorner	47290
U+7315.0	fHakka	 ni2
U+7315.0	fCantonese	mei4; lei4 (also); nei4 (also)
U+7315.0	fMandarin	mi2
U+7315.0	fEnglish	simplified form of &#x737C;
U+7316.0	fUTF8	猖
U+7316.0	fCNS11643	1-5B78
U+7316.0	fBig5	B273
U+7316.0	fGB	0-327E
U+7316.0	fR/S	 94.8
U+7316.0	fTotalStrokes	 11
U+7316.0	fCangjie	KHAA
U+7316.0	fFourCorner	46260
U+7316.0	fHakka	  chong1; cong1
U+7316.0	fCantonese	coeng1
U+7316.0	fMandarin	chang1
U+7316.0	fEnglish	mad; wild; reckless; unruly; imprudent
U+7317.0	fUTF8	猗
U+7317.0	fCNS11643	2-3726
U+7317.0	fBig5	D661
U+7317.0	fGB	0-6222
U+7317.0	fR/S	 94.8
U+7317.0	fTotalStrokes	 11
U+7317.0	fCangjie	KHKMR
U+7317.0	fFourCorner	44221
U+7317.0	fHakka	 ji2; ji1; ri2; i1
U+7317.0	fCantonese	ji1; ji2; ngo5; o2; ngo2; o5
U+7317.0	fMandarin	yi1; yi3; e1; yi4; wei1; ji4
U+7317.0	fEnglish	[1] an exclamation of admiration [2] an adverbial particle
U+7317.1	fUTF8	猗
U+7317.1	fCNS11643	2-3726
U+7317.1	fBig5	D661
U+7317.1	fGB	0-6222
U+7317.1	fR/S	 94.8
U+7317.1	fTotalStrokes	 11
U+7317.1	fCangjie	KHKMR
U+7317.1	fFourCorner	44221
U+7317.1	fMandarin	e3
U+7317.1	fEnglish	gentle, soft, and pliant
U+7318.0	fUTF8	猘
U+7318.0	fCNS11643	2-3729
U+7318.0	fBig5	D664
U+7318.0	fGB	3-443B
U+7318.0	fR/S	 94.8
U+7318.0	fTotalStrokes	 11
U+7318.0	fCangjie	KHHBN
U+7318.0	fFourCorner	42200
U+7318.0	fCantonese	zai3; gai3
U+7318.0	fMandarin	zhi4; ji4
U+7318.0	fEnglish	 fierce dog furious; frenzied
U+7319.0	fUTF8	猙
U+7319.0	fCNS11643	1-5B7A
U+7319.0	fBig5	B275
U+7319.0	fGB	1-5578
U+7319.0	fR/S	 94.8
U+7319.0	fTotalStrokes	 11
U+7319.0	fCangjie	KHBSD
U+7319.0	fFourCorner	42257
U+7319.0	fHakka	  zen1; zang1
U+7319.0	fCantonese	zang1; zaang1 (also)
U+7319.0	fMandarin	zheng1; jing4
U+7319.0	fEnglish	fierce-looking; ferocious; hideous; repulsive
U+731A.0	fUTF8	猚
U+731A.0	fCNS11643	3-3739
U+731A.0	fGB	3-443D
U+731A.0	fR/S	  94.8
U+731A.0	fTotalStrokes	 11
U+731A.0	fCangjie	KHOG
U+731A.0	fFourCorner	40210
U+731A.0	fCantonese	ngaai4; aai4 (also)
U+731A.0	fMandarin	ya2; yi2; lei3; you4
U+731B.0	fUTF8	猛
U+731B.0	fCNS11643	1-5B77
U+731B.0	fBig5	B272
U+731B.0	fGB	0-434D
U+731B.0	fR/S	 94.8
U+731B.0	fTotalStrokes	 11
U+731B.0	fCangjie	KHNDT
U+731B.0	fFourCorner	47217
U+731B.0	fHakka	  mang1;  mang3
U+731B.0	fCantonese	maang5
U+731B.0	fMandarin	meng3
U+731B.0	fEnglish	[1] violent; savage; cruel; bold; fierce; brave [2] sudden and quick (strikes, thrusts, etc.) [3] severe; strict; stringent [4] a Chinese family name
U+731C.0	fUTF8	猜
U+731C.0	fCNS11643	1-5B76
U+731C.0	fBig5	B271
U+731C.0	fGB	0-3242
U+731C.0	fR/S	 94.8
U+731C.0	fTotalStrokes	 11
U+731C.0	fCangjie	KHQMB
U+731C.0	fFourCorner	45227
U+731C.0	fHakka	  cai1
U+731C.0	fCantonese	caai1
U+731C.0	fMandarin	cai1
U+731C.0	fEnglish	[1] guess; suspect; suppose; doubt [2] cruel and suspicious
U+731D.0	fUTF8	猝
U+731D.0	fCNS11643	2-3725
U+731D.0	fBig5	D660
U+731D.0	fGB	0-6227
U+731D.0	fR/S	 94.8
U+731D.0	fTotalStrokes	 11
U+731D.0	fCangjie	KHYOJ
U+731D.0	fFourCorner	40248
U+731D.0	fHakka	 cut7; sut7; cuk7
U+731D.0	fCantonese	cyut3
U+731D.0	fMandarin	cu4
U+731D.0	fEnglish	abrupt; sudden; hurried; unexpected
U+731E.0	fUTF8	猞
U+731E.0	fCNS11643	2-372E
U+731E.0	fBig5	D669
U+731E.0	fGB	0-6226
U+731E.0	fR/S	 94.8
U+731E.0	fTotalStrokes	 11
U+731E.0	fCangjie	KHOMR
U+731E.0	fFourCorner	48264
U+731E.0	fHakka	 sa5
U+731E.0	fCantonese	se3
U+731E.0	fMandarin	she4; she1
U+731E.0	fEnglish	a wild cat; 猞猁, a lynx
U+731F.0	fUTF8	猟
U+731F.0	fGB	E-3323
U+731F.0	fR/S	 94.8
U+731F.0	fTotalStrokes	 11
U+731F.0	fCangjie	KHFHN
U+731F.0	fCantonese	lip6
U+731F.0	fMandarin	lie4
U+731F.0	fEnglish	 hunt; field sports
U+7320.0	fUTF8	猠
U+7320.0	fGB	1-7D59
U+7320.0	fR/S	 94.8
U+7320.0	fTotalStrokes	 11
U+7320.0	fCangjie	KHTBC
U+7320.0	fCantonese	din2
U+7321.0	fUTF8	猡
U+7321.0	fGB	0-6224
U+7321.0	fR/S	 94.8
U+7321.0	fTotalStrokes	 11
U+7321.0	fCangjie	KHWLN
U+7321.0	fCantonese	lo4
U+7321.0	fMandarin	luo2
U+7321.0	fEnglish	simplified form of &#x7380;
U+7322.0	fUTF8	猢
U+7322.0	fCNS11643	2-3D74
U+7322.0	fBig5	DA70
U+7322.0	fGB	0-6229
U+7322.0	fR/S	 94.9
U+7322.0	fTotalStrokes	 12
U+7322.0	fCangjie	KHJRB
U+7322.0	fFourCorner	47220
U+7322.0	fHakka	 fu2
U+7322.0	fCantonese	wu4
U+7322.0	fMandarin	hu2; hu1
U+7322.0	fEnglish	a kind of monkey found in the western of China
U+7323.0	fUTF8	猣
U+7323.0	fCNS11643	2-3D7B
U+7323.0	fBig5	DA77
U+7323.0	fGB	5-4721
U+7323.0	fR/S	 94.9
U+7323.0	fTotalStrokes	 12
U+7323.0	fCangjie	KHUCE
U+7323.0	fFourCorner	42247
U+7323.0	fCantonese	zung1; zung2
U+7323.0	fMandarin	zong1; zong3
U+7324.0	fUTF8	猤
U+7324.0	fCNS11643	3-3D27
U+7324.0	fGB	3-444D
U+7324.0	fR/S	 94.9
U+7324.0	fTotalStrokes	 12
U+7324.0	fCangjie	KHNOK
U+7324.0	fFourCorner	42230
U+7324.0	fCantonese	gwai3
U+7324.0	fMandarin	ji4; gui4
U+7325.0	fUTF8	猥
U+7325.0	fCNS11643	1-605A
U+7325.0	fBig5	B554
U+7325.0	fGB	0-622B
U+7325.0	fR/S	 94.9
U+7325.0	fTotalStrokes	 12
U+7325.0	fCangjie	KHWMV
U+7325.0	fFourCorner	46232
U+7325.0	fHakka	 woi1; voe1; voi1
U+7325.0	fCantonese	wai2; wui1 (also); wui2
U+7325.0	fMandarin	wei3; wei4
U+7325.0	fEnglish	[1] vulgar; low; cheap; wanton; obscene; lewd and licentious [2] many; numerous; varied; multitudinous [3] the bark of a dog
U+7326.0	fUTF8	猦
U+7326.0	fCNS11643	2-3D7A
U+7326.0	fBig5	DA76
U+7326.0	fGB	E-3324
U+7326.0	fR/S	 94.9
U+7326.0	fTotalStrokes	 12
U+7326.0	fCangjie	KHHNI
U+7326.0	fFourCorner	47210
U+7326.0	fCantonese	fung1
U+7326.0	fMandarin	feng1
U+7327.0	fUTF8	猧
U+7327.0	fCNS11643	2-3D77
U+7327.0	fBig5	DA73
U+7327.0	fGB	3-4435
U+7327.0	fR/S	 94.9
U+7327.0	fTotalStrokes	 12
U+7327.0	fCangjie	KHBBR
U+7327.0	fFourCorner	47227
U+7327.0	fHakka	 wo1; vo1
U+7327.0	fCantonese	wo1
U+7327.0	fMandarin	wo1
U+7327.0	fEnglish	a lap-dog
U+7328.0	fUTF8	猨
U+7328.0	fCNS11643	3-3D2E
U+7328.0	fGB	E-3325
U+7328.0	fR/S	  94.9
U+7328.0	fTotalStrokes	 12
U+7328.0	fCangjie	KHBME
U+7328.0	fFourCorner	42240
U+7328.0	fHakka	 jen2; ren2
U+7328.0	fCantonese	jyun4
U+7328.0	fMandarin	yuan2
U+7328.0	fEnglish	ape
U+7329.0	fUTF8	猩
U+7329.0	fCNS11643	1-605C
U+7329.0	fBig5	B556
U+7329.0	fGB	0-5049
U+7329.0	fR/S	 94.9
U+7329.0	fTotalStrokes	 12
U+7329.0	fCangjie	KHAHM
U+7329.0	fFourCorner	46214
U+7329.0	fHakka	  sin1; sen1; siang1
U+7329.0	fCantonese	sing1
U+7329.0	fMandarin	xing1; fan1; sheng1
U+7329.0	fEnglish	[1] a yellow-haired ape [2] scarlet; red
U+7329.1	fUTF8	猩
U+7329.1	fCNS11643	1-605C
U+7329.1	fBig5	B556
U+7329.1	fGB	0-5049
U+7329.1	fR/S	 94.9
U+7329.1	fTotalStrokes	 12
U+7329.1	fCangjie	KHAHM
U+7329.1	fFourCorner	46214
U+7329.1	fMandarin	xing5
U+732A.0	fUTF8	猪
U+732A.0	fCNS11643	3-3D29
U+732A.0	fGB	0-566D
U+732A.0	fR/S	  94.8; 93.9
U+732A.0	fTotalStrokes	 12; 11
U+732A.0	fCangjie	KHJKA
U+732A.0	fFourCorner	44260
U+732A.0	fHakka	 zu1; zhu1
U+732A.0	fCantonese	zyu1
U+732A.0	fMandarin	zhu1
U+732A.0	fEnglish	pig; hog; wild boar
U+732B.0	fUTF8	猫
U+732B.0	fCNS11643	3-3D2B
U+732B.0	fGB	0-4328
U+732B.0	fR/S	  94.9
U+732B.0	fTotalStrokes	 12
U+732B.0	fCangjie	KHTW
U+732B.0	fFourCorner	44260
U+732B.0	fHakka	  miau5; miau2; meu2; meu5; ngiau5
U+732B.0	fCantonese	maau1; mau1
U+732B.0	fMandarin	mao1; mao2; miao2
U+732B.0	fEnglish	cat
U+732C.0	fUTF8	猬
U+732C.0	fCNS11643	3-3D2A
U+732C.0	fGB	0-622C
U+732C.0	fR/S	 94.9
U+732C.0	fTotalStrokes	 12
U+732C.0	fCangjie	KHWB
U+732C.0	fFourCorner	46220
U+732C.0	fHakka	 wi5; wui5; wui3; vui5
U+732C.0	fCantonese	wai6
U+732C.0	fMandarin	wei4
U+732C.0	fEnglish	 vulgar wanton low many varied
U+732D.0	fUTF8	猭
U+732D.0	fCNS11643	2-3D79
U+732D.0	fBig5	DA75
U+732D.0	fGB	3-444E
U+732D.0	fR/S	 94.9
U+732D.0	fTotalStrokes	 12
U+732D.0	fCangjie	KHVNO
U+732D.0	fFourCorner	47232
U+732D.0	fHakka	 chon5; con5; chon2; con2
U+732D.0	fCantonese	cyun1; sin1; syun6
U+732D.0	fMandarin	yuan2; chuan1; chuan2; chuan4; shan1
U+732D.0	fEnglish	haze scampering away
U+732E.0	fUTF8	献
U+732E.0	fCNS11643	3-4261
U+732E.0	fGB	0-4F57
U+732E.0	fR/S	  94.9
U+732E.0	fTotalStrokes	 13; 12
U+732E.0	fCangjie	JBIK
U+732E.0	fFourCorner	43230
U+732E.0	fHakka	 hen5; hien5
U+732E.0	fCantonese	hin3
U+732E.0	fMandarin	xian4; suo1; yi2
U+732E.0	fEnglish	simplified form of &#x737B;
U+732F.0	fUTF8	猯
U+732F.0	fCNS11643	3-3D2D
U+732F.0	fGB	5-467E
U+732F.0	fR/S	  94.9
U+732F.0	fTotalStrokes	 12
U+732F.0	fCangjie	KHUMB
U+732F.0	fFourCorner	42220
U+732F.0	fCantonese	teon1
U+732F.0	fMandarin	tuan1
U+7330.0	fUTF8	猰
U+7330.0	fCNS11643	2-3D73
U+7330.0	fBig5	DA6F
U+7330.0	fGB	3-4446
U+7330.0	fR/S	 94.9
U+7330.0	fTotalStrokes	 12
U+7330.0	fCangjie	KHQHK
U+7330.0	fFourCorner	47234
U+7330.0	fHakka	 kat7;  zap7; zhap7
U+7330.0	fCantonese	aat3; jaak3 (also); ngaat3 (also); zaat3; jat3
U+7330.0	fMandarin	ya4
U+7330.0	fEnglish	evil-disposed
U+7331.0	fUTF8	猱
U+7331.0	fCNS11643	2-3D75
U+7331.0	fBig5	DA71
U+7331.0	fGB	0-622E
U+7331.0	fR/S	 94.9
U+7331.0	fTotalStrokes	 12
U+7331.0	fCangjie	KHNHD
U+7331.0	fFourCorner	47294
U+7331.0	fHakka	 nau2; ju2; riu2
U+7331.0	fCantonese	naau4; laau4 (also); nou4
U+7331.0	fMandarin	nao2; niu4; rou2
U+7331.0	fEnglish	[1] a yellow-haired monkey [2] scratch
U+7332.0	fUTF8	猲
U+7332.0	fCNS11643	2-3D78
U+7332.0	fBig5	DA74
U+7332.0	fGB	3-4449
U+7332.0	fR/S	 94.9
U+7332.0	fTotalStrokes	 12
U+7332.0	fCangjie	KHAPV
U+7332.0	fFourCorner	46227
U+7332.0	fHakka	  hiet7
U+7332.0	fCantonese	hit3; hot3
U+7332.0	fMandarin	xie1; he4; xie2; ge2; hai4; he1; xie5
U+7332.0	fEnglish	 smoke or flames from fire roast
U+7333.0	fUTF8	猳
U+7333.0	fCNS11643	2-3D76
U+7333.0	fBig5	DA72
U+7333.0	fGB	5-4724
U+7333.0	fR/S	 94.9
U+7333.0	fTotalStrokes	 12
U+7333.0	fCangjie	KHRSE
U+7333.0	fFourCorner	47247
U+7333.0	fHakka	 ga1
U+7333.0	fCantonese	gaa1
U+7333.0	fMandarin	jia1
U+7334.0	fUTF8	猴
U+7334.0	fCNS11643	1-605B
U+7334.0	fBig5	B555
U+7334.0	fGB	0-3A6F
U+7334.0	fR/S	 94.9
U+7334.0	fTotalStrokes	 12
U+7334.0	fCangjie	KHONK
U+7334.0	fFourCorner	47234
U+7334.0	fHakka	  heu2; heu3; hiu2
U+7334.0	fCantonese	hau4
U+7334.0	fMandarin	hou2
U+7334.0	fEnglish	[1] the monkey; ape [2] naughty; impish (child)
U+7335.0	fUTF8	猵
U+7335.0	fCNS11643	2-3D7C
U+7335.0	fBig5	DA78
U+7335.0	fGB	3-444C
U+7335.0	fR/S	 94.9
U+7335.0	fTotalStrokes	 12
U+7335.0	fCangjie	KHHSB
U+7335.0	fFourCorner	43227
U+7335.0	fCantonese	bin1
U+7335.0	fMandarin	bian1; pian4; fei4; pian1; pin2; pin4
U+7336.0	fUTF8	猶
U+7336.0	fCNS11643	1-6059
U+7336.0	fBig5	B553
U+7336.0	fGB	1-534C
U+7336.0	fR/S	 94.9
U+7336.0	fTotalStrokes	 12
U+7336.0	fCangjie	KHTCW
U+7336.0	fFourCorner	48261
U+7336.0	fHakka	  ju2; jiu2; riu2; jiu3
U+7336.0	fCantonese	jau4
U+7336.0	fMandarin	you2; yao2
U+7336.0	fEnglish	[1] like; similar to; just like; as if; tantamount to [2] still; yet; even; especially; while [3] a kind of monkey [4] a Chinese family name [5] strategy; scheme; plot
U+7337.0	fUTF8	猷
U+7337.0	fCNS11643	1-6527
U+7337.0	fBig5	B7DF
U+7337.0	fGB	0-6960
U+7337.0	fR/S	 94.9
U+7337.0	fTotalStrokes	 13; 12
U+7337.0	fCangjie	TWIK
U+7337.0	fFourCorner	83634
U+7337.0	fHakka	  ju2; riu2; jiu2
U+7337.0	fCantonese	jau4
U+7337.0	fMandarin	you2; jiu4; yao2; you4
U+7337.0	fEnglish	[1] plan; scheme; program; plot [2] way; path [3] draw; paint [4] like; similar [5] an opening particle
U+7338.0	fUTF8	猸
U+7338.0	fCNS11643	3-3D28
U+7338.0	fGB	0-622D
U+7338.0	fR/S	 94.9
U+7338.0	fTotalStrokes	 12
U+7338.0	fCangjie	KHAHU
U+7338.0	fFourCorner	47260
U+7338.0	fCantonese	mei4
U+7338.0	fMandarin	mei2
U+7338.0	fEnglish	a kind of animal
U+7339.0	fUTF8	猹
U+7339.0	fGB	0-622A
U+7339.0	fR/S	 94.9
U+7339.0	fTotalStrokes	 12
U+7339.0	fCangjie	KHDAM
U+7339.0	fCantonese	zaa1
U+7339.0	fMandarin	cha2; zha1
U+7339.0	fEnglish	wild animal mentioned in short story by Lu Xun
U+733A.0	fUTF8	猺
U+733A.0	fCNS11643	2-4471
U+733A.0	fBig5	DEAD
U+733A.0	fGB	3-4454
U+733A.0	fR/S	 94.10
U+733A.0	fTotalStrokes	 13
U+733A.0	fCangjie	KHBOU
U+733A.0	fFourCorner	47272
U+733A.0	fHakka	 jau2; rau2; jeu2
U+733A.0	fCantonese	jiu4
U+733A.0	fMandarin	yao2
U+733A.0	fEnglish	[1] jackal [2] the name of a tribe in Kwangtung, Hunan, Kwangsi, and Yunnan provinces
U+733B.0	fUTF8	猻
U+733B.0	fCNS11643	2-4470
U+733B.0	fBig5	DEAC
U+733B.0	fGB	1-6178
U+733B.0	fR/S	 94.10
U+733B.0	fTotalStrokes	 13
U+733B.0	fCangjie	KHNDF
U+733B.0	fFourCorner	42293
U+733B.0	fHakka	 sun1
U+733B.0	fCantonese	syun1
U+733B.0	fMandarin	sun1
U+733B.0	fEnglish	 monkey
U+733C.0	fUTF8	猼
U+733C.0	fCNS11643	2-446E
U+733C.0	fBig5	DEAA
U+733C.0	fGB	3-4452
U+733C.0	fR/S	 94.10
U+733C.0	fTotalStrokes	 13
U+733C.0	fCangjie	KHIBI
U+733C.0	fFourCorner	43246
U+733C.0	fCantonese	bok3; pok3
U+733C.0	fMandarin	bo2; pao2; po4
U+733D.0	fUTF8	猽
U+733D.0	fCNS11643	6-547C
U+733D.0	fGB	5-472B
U+733D.0	fR/S	  94.10
U+733D.0	fTotalStrokes	 13
U+733D.0	fCangjie	KHBAC
U+733D.0	fFourCorner	47280
U+733D.0	fCantonese	ming4
U+733D.0	fMandarin	ming2
U+733E.0	fUTF8	猾
U+733E.0	fCNS11643	1-652A
U+733E.0	fBig5	B7E2
U+733E.0	fGB	0-3B2B
U+733E.0	fR/S	 94.10
U+733E.0	fTotalStrokes	 13
U+733E.0	fCangjie	KHBBB
U+733E.0	fFourCorner	47227
U+733E.0	fHakka	  wat8; wat7; vat8
U+733E.0	fCantonese	waat6
U+733E.0	fMandarin	hua2
U+733E.0	fEnglish	crafty; cunning; shrewd
U+733F.0	fUTF8	猿
U+733F.0	fCNS11643	1-6529
U+733F.0	fBig5	B7E1
U+733F.0	fGB	0-5433
U+733F.0	fR/S	 94.10
U+733F.0	fTotalStrokes	 13
U+733F.0	fCangjie	KHGRV
U+733F.0	fFourCorner	44232
U+733F.0	fHakka	 jen2; ren2; jan3
U+733F.0	fCantonese	jyun4
U+733F.0	fMandarin	yuan2
U+733F.0	fEnglish	 ape; gibbon
U+7340.0	fUTF8	獀
U+7340.0	fCNS11643	2-4472
U+7340.0	fBig5	DEAE
U+7340.0	fGB	3-444A
U+7340.0	fR/S	 94.10
U+7340.0	fTotalStrokes	 13
U+7340.0	fCangjie	KHHXE
U+7340.0	fFourCorner	47247
U+7340.0	fCantonese	sau1; sau2 (also)
U+7340.0	fMandarin	sou1; sou3
U+7340.0	fEnglish	 gather hunt; search for
U+7341.0	fUTF8	獁
U+7341.0	fCNS11643	3-425F
U+7341.0	fGB	1-616F
U+7341.0	fR/S	 94.9; 94.10
U+7341.0	fTotalStrokes	 13; 12
U+7341.0	fCangjie	KHSQF
U+7341.0	fFourCorner	41220
U+7341.0	fHakka	 ma3; ma5
U+7341.0	fCantonese	maa5; maa6 (also)
U+7341.0	fMandarin	ma4; ma3
U+7341.0	fEnglish	[1] monkey [2] wolf
U+7342.0	fUTF8	獂
U+7342.0	fCNS11643	2-446F
U+7342.0	fBig5	DEAB
U+7342.0	fGB	3-4453
U+7342.0	fR/S	 94.10
U+7342.0	fTotalStrokes	 13
U+7342.0	fCangjie	KHMHF
U+7342.0	fFourCorner	41296
U+7342.0	fCantonese	wun4; jyun4 (also)
U+7342.0	fMandarin	yuan2; huan2
U+7343.0	fUTF8	獃
U+7343.0	fCNS11643	2-4B70
U+7343.0	fBig5	E2CA
U+7343.0	fGB	E-3326
U+7343.0	fR/S	 94.10
U+7343.0	fTotalStrokes	 14; 13
U+7343.0	fCangjie	UTIK
U+7343.0	fFourCorner	23134
U+7343.0	fHakka	  ngoi2; ngoi3
U+7343.0	fCantonese	ngoi4; daai1; oi4 (also); daai4
U+7343.0	fMandarin	dai1; ai2
U+7343.0	fEnglish	[1] stupid; silly; foolish; idiotic [2] maladroit; awkward; clumsy; bungling [3] stay
U+7344.0	fUTF8	獄
U+7344.0	fCNS11643	1-6962
U+7344.0	fBig5	BABB
U+7344.0	fGB	1-537C
U+7344.0	fR/S	 94.10; 94.11
U+7344.0	fTotalStrokes	 13
U+7344.0	fCangjie	KHYRK
U+7344.0	fFourCorner	43234
U+7344.0	fHakka	 ngiuk8; niuk8; gniuk8
U+7344.0	fCantonese	juk6
U+7344.0	fMandarin	yu4
U+7344.0	fEnglish	[1] prison; jail [2] case ; lawsuit
U+7345.0	fUTF8	獅
U+7345.0	fCNS11643	1-6528
U+7345.0	fBig5	B7E0
U+7345.0	fGB	1-4A28
U+7345.0	fR/S	 94.10
U+7345.0	fTotalStrokes	 13
U+7345.0	fCangjie	KHHRB
U+7345.0	fFourCorner	41227
U+7345.0	fHakka	  sii1;  su1; si1; se1
U+7345.0	fCantonese	si1
U+7345.0	fMandarin	shi1
U+7345.0	fEnglish	the lion
U+7346.0	fUTF8	獆
U+7346.0	fCNS11643	3-4262
U+7346.0	fGB	5-4727
U+7346.0	fR/S	  94.10
U+7346.0	fTotalStrokes	 13
U+7346.0	fCangjie	KHHAJ
U+7346.0	fFourCorner	46243
U+7346.0	fCantonese	hou4
U+7346.0	fMandarin	hao2
U+7347.0	fUTF8	獇
U+7347.0	fCNS11643	3-6562
U+7347.0	fGB	E-3327
U+7347.0	fR/S	 94.10
U+7347.0	fTotalStrokes	 13
U+7347.0	fCangjie	KHTUI
U+7347.0	fFourCorner	4821
U+7347.0	fCantonese	goeng1
U+7348.0	fUTF8	獈
U+7348.0	fCNS11643	3-6563
U+7348.0	fGB	E-3328
U+7348.0	fR/S	 94.10
U+7348.0	fTotalStrokes	 13
U+7348.0	fCangjie	KHTCT
U+7348.0	fFourCorner	4821
U+7348.0	fCantonese	jik6
U+7348.0	fMandarin	yi4
U+7349.0	fUTF8	獉
U+7349.0	fCNS11643	2-4474
U+7349.0	fBig5	DEB0
U+7349.0	fGB	5-4725
U+7349.0	fR/S	 94.10
U+7349.0	fTotalStrokes	 13
U+7349.0	fCangjie	KHQKD
U+7349.0	fFourCorner	45294
U+7349.0	fCantonese	zeon1
U+7349.0	fMandarin	zhen1; jian1; qin2
U+7349.0	fEnglish	jungle
U+734A.0	fUTF8	獊
U+734A.0	fCNS11643	2-4473
U+734A.0	fBig5	DEAF
U+734A.0	fGB	3-4371
U+734A.0	fR/S	 94.10
U+734A.0	fTotalStrokes	 13
U+734A.0	fCangjie	KHOIR
U+734A.0	fFourCorner	48267
U+734A.0	fCantonese	cong1
U+734A.0	fMandarin	chuang4; chuang1; cang1; chuang3
U+734B.0	fUTF8	獋
U+734B.0	fCNS11643	3-4D30
U+734B.0	fGB	E-3329
U+734B.0	fR/S	  94.12
U+734B.0	fTotalStrokes	 15
U+734B.0	fCangjie	KHHUJ
U+734B.0	fFourCorner	46240
U+734B.0	fCantonese	hou4
U+734B.0	fMandarin	hao2
U+734B.0	fEnglish	 roar cry
U+734C.0	fUTF8	獌
U+734C.0	fCNS11643	2-4B73
U+734C.0	fBig5	E2CD
U+734C.0	fGB	3-445A
U+734C.0	fR/S	 94.11
U+734C.0	fTotalStrokes	 14
U+734C.0	fCangjie	KHAWE
U+734C.0	fFourCorner	46247
U+734C.0	fHakka	 man2
U+734C.0	fCantonese	maan6
U+734C.0	fMandarin	man4; man2; wan4
U+734D.0	fUTF8	獍
U+734D.0	fCNS11643	2-4B71
U+734D.0	fBig5	E2CB
U+734D.0	fGB	0-6230
U+734D.0	fR/S	 94.11
U+734D.0	fTotalStrokes	 14
U+734D.0	fCangjie	KHYTU
U+734D.0	fFourCorner	40216
U+734D.0	fHakka	 gin5
U+734D.0	fCantonese	ging3
U+734D.0	fMandarin	jing4
U+734D.0	fEnglish	an animal which on being born eats its mother
U+734E.0	fUTF8	獎
U+734E.0	fCNS11643	1-6D64
U+734E.0	fBig5	BCFA
U+734E.0	fGB	E-332A
U+734E.0	fR/S	 94.11
U+734E.0	fTotalStrokes	 15; 14
U+734E.0	fCangjie	VIIK
U+734E.0	fFourCorner	27430
U+734E.0	fHakka	 ziong3
U+734E.0	fCantonese	zoeng2
U+734E.0	fMandarin	jiang3
U+734E.0	fEnglish	 prize; reward give award to
U+734F.0	fUTF8	獏
U+734F.0	fCNS11643	3-4779
U+734F.0	fGB	3-4451
U+734F.0	fR/S	 94.11
U+734F.0	fTotalStrokes	 14
U+734F.0	fCangjie	KHTAK
U+734F.0	fFourCorner	44230
U+734F.0	fHakka	 mu2
U+734F.0	fCantonese	mok6
U+734F.0	fMandarin	mu2; mo4
U+734F.0	fEnglish	Mu, the name of a tribe in Hopu, Kwangtung province
U+7350.0	fUTF8	獐
U+7350.0	fCNS11643	1-6963
U+7350.0	fBig5	BABC
U+7350.0	fGB	0-622F
U+7350.0	fR/S	 94.11
U+7350.0	fTotalStrokes	 14
U+7350.0	fCangjie	KHYTJ
U+7350.0	fFourCorner	40246
U+7350.0	fHakka	 zong1
U+7350.0	fCantonese	zoeng1
U+7350.0	fMandarin	zhang1
U+7350.0	fEnglish	the roe deer; hornless river deer
U+7351.0	fUTF8	獑
U+7351.0	fCNS11643	2-4B72
U+7351.0	fBig5	E2CC
U+7351.0	fGB	3-4437
U+7351.0	fR/S	 94.11
U+7351.0	fTotalStrokes	 14
U+7351.0	fCangjie	KHJJL
U+7351.0	fFourCorner	42221
U+7351.0	fHakka	  cam2
U+7351.0	fCantonese	caam4
U+7351.0	fMandarin	chan2
U+7352.0	fUTF8	獒
U+7352.0	fCNS11643	2-517E
U+7352.0	fBig5	E676
U+7352.0	fGB	0-6961
U+7352.0	fR/S	 94.11
U+7352.0	fTotalStrokes	 15; 14
U+7352.0	fCangjie	GKIK
U+7352.0	fFourCorner	58430
U+7352.0	fHakka	  ngau2
U+7352.0	fCantonese	ngou4; ou4 (also); ngou1
U+7352.0	fMandarin	ao2
U+7352.0	fEnglish	a large dog, described as four feet high, and fierce but tractable. Probably the mastiff of Tibetan shepherds
U+7353.0	fUTF8	獓
U+7353.0	fCNS11643	3-4778
U+7353.0	fGB	3-4450
U+7353.0	fR/S	 94.9; 94.11
U+7353.0	fTotalStrokes	 14; 12
U+7353.0	fCangjie	KHGSK
U+7353.0	fFourCorner	48240
U+7353.0	fCantonese	ngou4; ou4 (also)
U+7353.0	fMandarin	ao2
U+7354.0	fUTF8	獔
U+7354.0	fCNS11643	3-477A
U+7354.0	fGB	E-332B
U+7354.0	fR/S	  94.11
U+7354.0	fTotalStrokes	 14
U+7354.0	fCangjie	KHHAJ
U+7354.0	fFourCorner	46240
U+7354.0	fCantonese	hou4
U+7354.0	fMandarin	hao2
U+7355.0	fUTF8	獕
U+7355.0	fCNS11643	3-4777
U+7355.0	fGB	3-445B
U+7355.0	fR/S	 94.11
U+7355.0	fTotalStrokes	 14
U+7355.0	fCangjie	KHUOG
U+7355.0	fFourCorner	42210
U+7355.0	fCantonese	ceoi1
U+7355.0	fMandarin	cui1
U+7356.0	fUTF8	獖
U+7356.0	fCNS11643	4-4A77
U+7356.0	fGB	3-4447
U+7356.0	fR/S	  94.12
U+7356.0	fTotalStrokes	 15
U+7356.0	fCangjie	KHJTC
U+7356.0	fFourCorner	44286
U+7356.0	fCantonese	ban6
U+7356.0	fMandarin	fen2; ben4; fen4
U+7357.0	fUTF8	獗
U+7357.0	fCNS11643	1-6D65
U+7357.0	fBig5	BCFB
U+7357.0	fGB	0-6231
U+7357.0	fR/S	 94.12
U+7357.0	fTotalStrokes	 15
U+7357.0	fCangjie	KHMTO
U+7357.0	fFourCorner	51082
U+7357.0	fHakka	 kiet7; giet7; ket7
U+7357.0	fCantonese	kyut3
U+7357.0	fMandarin	jue2; jue1
U+7357.0	fEnglish	unruly; wild; violent; lawless; rampant
U+7358.0	fUTF8	獘
U+7358.0	fCNS11643	2-517D
U+7358.0	fBig5	E675
U+7358.0	fGB	E-332C
U+7358.0	fR/S	 94.12
U+7358.0	fTotalStrokes	 16
U+7358.0	fCangjie	FKIK
U+7358.0	fFourCorner	97434
U+7358.0	fHakka	  bi5
U+7358.0	fCantonese	bai6
U+7358.0	fMandarin	bi4; bie2
U+7358.0	fEnglish	collapse
U+7359.0	fUTF8	獙
U+7359.0	fCNS11643	2-5228
U+7359.0	fBig5	E67E
U+7359.0	fGB	E-332D
U+7359.0	fR/S	 94.12
U+7359.0	fTotalStrokes	 15
U+7359.0	fCangjie	KHFBK
U+7359.0	fFourCorner	48240
U+7359.0	fCantonese	bai6
U+7359.0	fMandarin	bi4
U+735A.0	fUTF8	獚
U+735A.0	fCNS11643	2-5227
U+735A.0	fBig5	E67D
U+735A.0	fGB	3-4458
U+735A.0	fR/S	 94.12
U+735A.0	fTotalStrokes	 15
U+735A.0	fCangjie	KHTLC
U+735A.0	fFourCorner	44286
U+735A.0	fCantonese	wong4
U+735A.0	fMandarin	huang2
U+735B.0	fUTF8	獛
U+735B.0	fCNS11643	2-5225
U+735B.0	fBig5	E67B
U+735B.0	fGB	E-332E
U+735B.0	fR/S	 94.12
U+735B.0	fTotalStrokes	 15
U+735B.0	fCangjie	KHTCO
U+735B.0	fFourCorner	42234
U+735B.0	fCantonese	buk1
U+735B.0	fMandarin	bu3; pu2
U+735C.0	fUTF8	獜
U+735C.0	fCNS11643	4-4A75
U+735C.0	fGB	3-4464
U+735C.0	fR/S	  94.12
U+735C.0	fTotalStrokes	 15
U+735C.0	fCangjie	KHFDQ
U+735C.0	fFourCorner	49250
U+735C.0	fCantonese	leon4
U+735C.0	fMandarin	lin2
U+735D.0	fUTF8	獝
U+735D.0	fCNS11643	2-5224
U+735D.0	fBig5	E67A
U+735D.0	fGB	3-4465
U+735D.0	fR/S	 94.12
U+735D.0	fTotalStrokes	 15
U+735D.0	fCangjie	KHNHB
U+735D.0	fFourCorner	47227
U+735D.0	fHakka	 git7;  lut8
U+735D.0	fCantonese	wat6; leot6
U+735D.0	fMandarin	yu4; xu4
U+735D.0	fEnglish	devil
U+735E.0	fUTF8	獞
U+735E.0	fCNS11643	2-5221
U+735E.0	fBig5	E677
U+735E.0	fGB	5-472E
U+735E.0	fR/S	 94.12
U+735E.0	fTotalStrokes	 15
U+735E.0	fCangjie	KHYTG
U+735E.0	fFourCorner	40214
U+735E.0	fHakka	 tung2; tung3
U+735E.0	fCantonese	tung4; zong6; zong3
U+735E.0	fMandarin	tong2
U+735E.0	fEnglish	Tung, name of a primitive tribe in Kwangtung and Kwangsi provinces
U+735F.0	fUTF8	獟
U+735F.0	fCNS11643	2-5222
U+735F.0	fBig5	E678
U+735F.0	fGB	5-4664
U+735F.0	fR/S	 94.12
U+735F.0	fTotalStrokes	 15
U+735F.0	fCangjie	KHGGU
U+735F.0	fFourCorner	44211
U+735F.0	fCantonese	hiu1; hiu3
U+735F.0	fMandarin	yao4; xiao1; qiao4; xiao4
U+7360.0	fUTF8	獠
U+7360.0	fCNS11643	2-5223
U+7360.0	fBig5	E679
U+7360.0	fGB	0-6232
U+7360.0	fR/S	 94.12
U+7360.0	fTotalStrokes	 15
U+7360.0	fCangjie	KHKCF
U+7360.0	fFourCorner	44296
U+7360.0	fHakka	  liau2; liau5
U+7360.0	fCantonese	liu4; lou5
U+7360.0	fMandarin	liao2; lao3; liao4; zhao3
U+7360.0	fEnglish	[1] hunt at night by torches; nocturnal hunting [2] also known as @, a primitive tribe in Southwest China [3] fierce (one's looks) [4] monster; wicked person
U+7361.0	fUTF8	獡
U+7361.0	fCNS11643	2-5226
U+7361.0	fBig5	E67C
U+7361.0	fGB	3-4461
U+7361.0	fR/S	 94.12
U+7361.0	fTotalStrokes	 15
U+7361.0	fCangjie	KHHXF
U+7361.0	fFourCorner	47227
U+7361.0	fCantonese	soek3; sik1; coek3
U+7361.0	fMandarin	shuo4; xi1; que4
U+7362.0	fUTF8	獢
U+7362.0	fCNS11643	2-5229
U+7362.0	fBig5	E6A1
U+7362.0	fGB	3-442E
U+7362.0	fR/S	 94.12
U+7362.0	fTotalStrokes	 15
U+7362.0	fCangjie	KHHKB
U+7362.0	fFourCorner	42227
U+7362.0	fCantonese	hiu1
U+7362.0	fMandarin	xiao1
U+7362.0	fEnglish	dog
U+7363.0	fUTF8	獣
U+7363.0	fGB	E-332F
U+7363.0	fR/S	 94.12
U+7363.0	fTotalStrokes	 16; 15
U+7363.0	fCangjie	FRIK
U+7363.0	fCantonese	sau3
U+7363.0	fMandarin	shou4
U+7363.0	fEnglish	 beast, animal; bestial
U+7364.0	fUTF8	獤
U+7364.0	fGB	1-7D30
U+7364.0	fR/S	 94.12
U+7364.0	fTotalStrokes	 15
U+7364.0	fCangjie	KHYDK
U+7364.0	fCantonese	deon1
U+7365.0	fUTF8	獥
U+7365.0	fCNS11643	2-584B
U+7365.0	fBig5	EA5F
U+7365.0	fGB	3-4467
U+7365.0	fR/S	 94.13
U+7365.0	fTotalStrokes	 16
U+7365.0	fCangjie	KHHSK
U+7365.0	fFourCorner	48240
U+7365.0	fCantonese	giu3; hat6; gik1
U+7365.0	fMandarin	xi2; xiao1; jiao4; ji1
U+7366.0	fUTF8	獦
U+7366.0	fCNS11643	2-5848
U+7366.0	fBig5	EA5C
U+7366.0	fGB	3-4460
U+7366.0	fR/S	 94.13
U+7366.0	fTotalStrokes	 16
U+7366.0	fCangjie	KHTAV
U+7366.0	fFourCorner	44227
U+7366.0	fCantonese	got3; lip6; hot3
U+7366.0	fMandarin	ge2; lie4
U+7367.0	fUTF8	獧
U+7367.0	fCNS11643	2-5849
U+7367.0	fBig5	EA5D
U+7367.0	fGB	E-3330
U+7367.0	fR/S	 94.13
U+7367.0	fTotalStrokes	 16
U+7367.0	fCangjie	KHWLV
U+7367.0	fFourCorner	46232
U+7367.0	fHakka	 gien3; gen5
U+7367.0	fCantonese	gyun3; gyun1 (also); gyun6
U+7367.0	fMandarin	juan4; xuan1; xuan2; quan3
U+7367.0	fEnglish	 rash honest and straightforward
U+7368.0	fUTF8	獨
U+7368.0	fCNS11643	1-7141
U+7368.0	fBig5	BF57
U+7368.0	fGB	1-3640
U+7368.0	fR/S	 94.13
U+7368.0	fTotalStrokes	 16
U+7368.0	fCangjie	KHWLI
U+7368.0	fFourCorner	46227
U+7368.0	fHakka	  tuk8
U+7368.0	fCantonese	duk6
U+7368.0	fMandarin	du2
U+7368.0	fEnglish	[1] alone; single; solitary [2] only [3] monopolize [4] be old without a son [5] how; Is it possible?
U+7369.0	fUTF8	獩
U+7369.0	fCNS11643	2-5847
U+7369.0	fBig5	EA5B
U+7369.0	fGB	5-4665
U+7369.0	fR/S	 94.13
U+7369.0	fTotalStrokes	 16
U+7369.0	fCangjie	KHYMH
U+7369.0	fFourCorner	41253
U+7369.0	fCantonese	wai3
U+7369.0	fMandarin	hui4
U+736A.0	fUTF8	獪
U+736A.0	fCNS11643	2-584D
U+736A.0	fBig5	EA61
U+736A.0	fGB	1-6176
U+736A.0	fR/S	 94.13
U+736A.0	fTotalStrokes	 16
U+736A.0	fCangjie	KHOMA
U+736A.0	fFourCorner	48266
U+736A.0	fHakka	  kwai5; kai5
U+736A.0	fCantonese	kui2; kwui2 (also)
U+736A.0	fMandarin	kuai4; hua2; guai4
U+736A.0	fEnglish	sly; cunning; crafty; cunning
U+736B.0	fUTF8	獫
U+736B.0	fCNS11643	2-584C
U+736B.0	fBig5	EA60
U+736B.0	fGB	1-617D
U+736B.0	fR/S	 94.13
U+736B.0	fTotalStrokes	 16
U+736B.0	fCangjie	KHOMO
U+736B.0	fFourCorner	48286
U+736B.0	fHakka	  hiam3
U+736B.0	fCantonese	him2
U+736B.0	fMandarin	xian3; lian3; lian4; lian2
U+736B.0	fEnglish	dog
U+736C.0	fUTF8	獬
U+736C.0	fCNS11643	2-584A
U+736C.0	fBig5	EA5E
U+736C.0	fGB	0-6233
U+736C.0	fR/S	 94.13
U+736C.0	fTotalStrokes	 16
U+736C.0	fCangjie	KHNBQ
U+736C.0	fFourCorner	47252
U+736C.0	fHakka	 hai5; hai6; hai3
U+736C.0	fCantonese	haai5
U+736C.0	fMandarin	xie4; ha3; jie3
U+736C.0	fEnglish	[1] a Pekinese dog [2] a mythical animal which was supposed to know the difference between right and wrong
U+736D.0	fUTF8	獭
U+736D.0	fGB	0-4C21
U+736D.0	fR/S	 94.13
U+736D.0	fTotalStrokes	 16
U+736D.0	fCangjie	KHDLO
U+736D.0	fHakka	 cat7
U+736D.0	fCantonese	caat3
U+736D.0	fMandarin	ta3; ta4
U+736D.0	fEnglish	simplified form of &#x737A;
U+736E.0	fUTF8	獮
U+736E.0	fCNS11643	2-5D50
U+736E.0	fBig5	ED64
U+736E.0	fGB	3-4422
U+736E.0	fR/S	 94.14
U+736E.0	fTotalStrokes	 17
U+736E.0	fCangjie	KHMFB
U+736E.0	fFourCorner	41227
U+736E.0	fHakka	  sien3
U+736E.0	fCantonese	sin2
U+736E.0	fMandarin	xian3; mi2
U+736E.0	fEnglish	[1] hunt; kill [2] autumn hunting by emperors
U+736F.0	fUTF8	獯
U+736F.0	fCNS11643	2-5D51
U+736F.0	fBig5	ED65
U+736F.0	fGB	0-6234
U+736F.0	fR/S	 94.14
U+736F.0	fTotalStrokes	 17
U+736F.0	fCangjie	KHHGF
U+736F.0	fFourCorner	42231
U+736F.0	fHakka	  hiun1
U+736F.0	fCantonese	fan1
U+736F.0	fMandarin	xun1
U+736F.0	fEnglish	a tribe of Scythians which invaded China during the Hsia dynasty; the name of a northern barbarian tribe in ancient China known as the Huns during the Han dynasty
U+7370.0	fUTF8	獰
U+7370.0	fCNS11643	1-743C
U+7370.0	fBig5	C0F1
U+7370.0	fGB	1-447C
U+7370.0	fR/S	 94.14
U+7370.0	fTotalStrokes	 17
U+7370.0	fCangjie	KHJPN
U+7370.0	fFourCorner	43221
U+7370.0	fHakka	  nen2; len2
U+7370.0	fCantonese	ning4; ling4 (also)
U+7370.0	fMandarin	ning2
U+7370.0	fEnglish	ferocious appearance; fierce appearance; awe-inspiring look
U+7371.0	fUTF8	獱
U+7371.0	fCNS11643	3-5549
U+7371.0	fGB	3-4456
U+7371.0	fR/S	 94.14
U+7371.0	fTotalStrokes	 17
U+7371.0	fCangjie	KHJMC
U+7371.0	fFourCorner	43280
U+7371.0	fHakka	 bin1
U+7371.0	fCantonese	ban1; bin1 (also)
U+7371.0	fMandarin	pin2; bian1; bin1
U+7371.0	fEnglish	a kind of otter
U+7372.0	fUTF8	獲
U+7372.0	fCNS11643	1-743D
U+7372.0	fBig5	C0F2
U+7372.0	fGB	1-3B71
U+7372.0	fR/S	 94.14
U+7372.0	fTotalStrokes	 17
U+7372.0	fCangjie	KHTOE
U+7372.0	fFourCorner	44247
U+7372.0	fHakka	 fet8; wet7; fiet8; vet7
U+7372.0	fCantonese	wok6; waai4; wai4
U+7372.0	fMandarin	huo4; hu4; hua4
U+7372.0	fEnglish	[1] obtain; get; receive; seize; incur; capture; catch; reap [2] can; able [3] slave-girl
U+7373.0	fUTF8	獳
U+7373.0	fCNS11643	2-5D4F
U+7373.0	fBig5	ED63
U+7373.0	fGB	3-4468
U+7373.0	fR/S	 94.14
U+7373.0	fTotalStrokes	 17
U+7373.0	fCangjie	KHMBB
U+7373.0	fFourCorner	41027
U+7373.0	fHakka	 ngieu3; neu5; neu3
U+7373.0	fCantonese	nau6; lau6 (also); nau1; nau2
U+7373.0	fMandarin	nou4; nou2; ru2
U+7373.0	fEnglish	fierce dog looking
U+7374.0	fUTF8	獴
U+7374.0	fCNS11643	4-566C
U+7374.0	fGB	3-4466
U+7374.0	fR/S	 94.14
U+7374.0	fTotalStrokes	 17; 16
U+7374.0	fCangjie	KHTBO
U+7374.0	fFourCorner	44230
U+7374.0	fCantonese	mung4
U+7374.0	fMandarin	meng2; meng3
U+7375.0	fUTF8	獵
U+7375.0	fCNS11643	1-7665
U+7375.0	fBig5	C279
U+7375.0	fGB	1-4154
U+7375.0	fR/S	 94.15
U+7375.0	fTotalStrokes	 18
U+7375.0	fCangjie	KHVVV
U+7375.0	fFourCorner	42216
U+7375.0	fHakka	  liap8
U+7375.0	fCantonese	lip6
U+7375.0	fMandarin	lie4
U+7375.0	fEnglish	 hunt; chase; field sports
U+7376.0	fUTF8	獶
U+7376.0	fCNS11643	2-622C
U+7376.0	fBig5	EFFE
U+7376.0	fGB	3-446A
U+7376.0	fR/S	 94.15
U+7376.0	fTotalStrokes	 18
U+7376.0	fCangjie	KHMBE
U+7376.0	fFourCorner	41247
U+7376.0	fCantonese	naau4; nou4 (also); laau4 (also); lou4 (also)
U+7376.0	fMandarin	nao2; you1; nou2
U+7377.0	fUTF8	獷
U+7377.0	fCNS11643	1-7664
U+7377.0	fBig5	C278
U+7377.0	fGB	1-616E
U+7377.0	fR/S	 94.15
U+7377.0	fTotalStrokes	 18
U+7377.0	fCangjie	KHITC
U+7377.0	fFourCorner	40281
U+7377.0	fHakka	 kong3;  kwang3; kong5; gwang3
U+7377.0	fCantonese	gwong2; gong2 (also); kwong3 (also); kong3 (also)
U+7377.0	fMandarin	guang3; jing3; gong3
U+7377.0	fEnglish	fierce and rude; uncivilized
U+7378.0	fUTF8	獸
U+7378.0	fCNS11643	1-784B
U+7378.0	fBig5	C37E
U+7378.0	fGB	1-4A5E
U+7378.0	fR/S	 94.15
U+7378.0	fTotalStrokes	 19
U+7378.0	fCangjie	RRIK
U+7378.0	fFourCorner	63634
U+7378.0	fHakka	 cu5; chiu5; ciu5; chu5
U+7378.0	fCantonese	sau3
U+7378.0	fMandarin	shou4
U+7378.0	fEnglish	[1] a general name for quadruped; beast; animal [2] bestial; beastly
U+7379.0	fUTF8	獹
U+7379.0	fCNS11643	4-6025
U+7379.0	fGB	5-4659
U+7379.0	fR/S	  94.16
U+7379.0	fTotalStrokes	 19
U+7379.0	fCangjie	KHYPT
U+7379.0	fFourCorner	41210
U+7379.0	fHakka	 lu2; lu3
U+7379.0	fCantonese	lou4
U+7379.0	fMandarin	lu2
U+7379.0	fEnglish	a hound
U+737A.0	fUTF8	獺
U+737A.0	fCNS11643	1-784C
U+737A.0	fBig5	C3A1
U+737A.0	fGB	1-4C21
U+737A.0	fR/S	 94.16
U+737A.0	fTotalStrokes	 19
U+737A.0	fCangjie	KHDLC
U+737A.0	fFourCorner	47286
U+737A.0	fHakka	  cat7
U+737A.0	fCantonese	caat3; taat3
U+737A.0	fMandarin	ta3; ta4
U+737A.0	fEnglish	the otter
U+737B.0	fUTF8	獻
U+737B.0	fCNS11643	1-7978
U+737B.0	fBig5	C46D
U+737B.0	fGB	1-4F57
U+737B.0	fR/S	 94.16
U+737B.0	fTotalStrokes	 20; 19
U+737B.0	fCangjie	YBIK
U+737B.0	fFourCorner	23234
U+737B.0	fHakka	  hien5; hian5; hen5
U+737B.0	fCantonese	hin3
U+737B.0	fMandarin	xian4; suo1; yi2; xi1
U+737B.0	fEnglish	[1] offer; present; forward; dedicate; donate [2] show; display; stage [3] curry (favor, etc.); flatter or cater to
U+737C.0	fUTF8	獼
U+737C.0	fCNS11643	2-693D
U+737C.0	fBig5	F46E
U+737C.0	fGB	1-6228
U+737C.0	fR/S	 94.17
U+737C.0	fTotalStrokes	 20
U+737C.0	fCangjie	KHNMB
U+737C.0	fFourCorner	41227
U+737C.0	fHakka	  ni2
U+737C.0	fCantonese	mei4; nei4 (also); lei4 (also)
U+737C.0	fMandarin	mi2
U+737C.0	fEnglish	a rhesus monkey
U+737D.0	fUTF8	獽
U+737D.0	fCNS11643	2-693C
U+737D.0	fBig5	F46D
U+737D.0	fGB	3-4471
U+737D.0	fR/S	 94.17
U+737D.0	fTotalStrokes	 20
U+737D.0	fCangjie	KHYRV
U+737D.0	fFourCorner	40232
U+737D.0	fCantonese	joeng4
U+737D.0	fMandarin	rang2
U+737E.0	fUTF8	獾
U+737E.0	fCNS11643	2-6B6B
U+737E.0	fBig5	F5DD
U+737E.0	fGB	0-6235
U+737E.0	fR/S	 94.18
U+737E.0	fTotalStrokes	 21
U+737E.0	fCangjie	KHTRG
U+737E.0	fFourCorner	44214
U+737E.0	fHakka	  fon1;  fan1
U+737E.0	fCantonese	fun1
U+737E.0	fMandarin	huan1; quan2
U+737E.0	fEnglish	[1] he-wolf [2] wild boar [3] badger
U+737F.0	fUTF8	獿
U+737F.0	fCNS11643	2-6D5D
U+737F.0	fBig5	F6EF
U+737F.0	fGB	5-4734
U+737F.0	fR/S	 94.18
U+737F.0	fTotalStrokes	 21
U+737F.0	fCangjie	KHMCE
U+737F.0	fFourCorner	41247
U+737F.0	fCantonese	naau4; laau4 (also); nou4 (also); lou4 (also)
U+737F.0	fMandarin	nao2; nei2; nao3; rao3
U+7380.0	fUTF8	玀
U+7380.0	fCNS11643	1-7B66
U+7380.0	fBig5	C57A
U+7380.0	fGB	1-6224
U+7380.0	fR/S	 94.19
U+7380.0	fTotalStrokes	 22
U+7380.0	fCangjie	KHWLG
U+7380.0	fFourCorner	46214
U+7380.0	fHakka	 lo2
U+7380.0	fCantonese	lo4
U+7380.0	fMandarin	luo2; e3
U+7380.0	fEnglish	used in combination for Lolo, an aboriginal tribe
U+7381.0	fUTF8	玁
U+7381.0	fCNS11643	2-6F37
U+7381.0	fBig5	F7E8
U+7381.0	fGB	3-4432
U+7381.0	fR/S	 94.20
U+7381.0	fTotalStrokes	 23
U+7381.0	fCangjie	KHRRK
U+7381.0	fFourCorner	46248
U+7381.0	fHakka	  hiam3
U+7381.0	fCantonese	him2
U+7381.0	fMandarin	xian3
U+7381.0	fEnglish	a dog with a long snout or muzzle
U+7382.0	fUTF8	玂
U+7382.0	fCNS11643	2-6F36
U+7382.0	fBig5	F7E7
U+7382.0	fGB	5-4732
U+7382.0	fR/S	 94.20
U+7382.0	fTotalStrokes	 23
U+7382.0	fCangjie	KHTJL
U+7382.0	fFourCorner	42221
U+7382.0	fCantonese	kei4
U+7382.0	fMandarin	qi2
U+7383.0	fUTF8	玃
U+7383.0	fCNS11643	2-6F38
U+7383.0	fBig5	F7E9
U+7383.0	fGB	3-4473
U+7383.0	fR/S	 94.20
U+7383.0	fTotalStrokes	 23
U+7383.0	fCangjie	KHBUE
U+7383.0	fFourCorner	46247
U+7383.0	fHakka	  kiok7
U+7383.0	fCantonese	fok3; gwok3
U+7383.0	fMandarin	jue2
U+7383.0	fEnglish	a large ape found in Western China
U+7384.0	fUTF8	玄
U+7384.0	fCNS11643	1-4668
U+7384.0	fBig5	A5C8
U+7384.0	fGB	0-507E
U+7384.0	fR/S	 95.0
U+7384.0	fTotalStrokes	 5
U+7384.0	fCangjie	YVI
U+7384.0	fFourCorner	00732
U+7384.0	fHakka	  hien2;  hen2; hian3
U+7384.0	fCantonese	jyun4
U+7384.0	fMandarin	xuan2; xuan4
U+7384.0	fEnglish	[1] far and obscure; occult; mystic [2] dark; black [3] deep and profound; abstruse and subtle [4] silent and meditative [5] pretending [6] a Chinese family name [7] Kangxi radical 95
U+7385.0	fUTF8	玅
U+7385.0	fCNS11643	2-2C28
U+7385.0	fBig5	CFC6
U+7385.0	fGB	E-3331
U+7385.0	fR/S	 95.4
U+7385.0	fTotalStrokes	 9
U+7385.0	fCangjie	YIFH
U+7385.0	fFourCorner	09720
U+7385.0	fHakka	 miau3; miau6; meu5; meu3
U+7385.0	fCantonese	miu6
U+7385.0	fMandarin	miao4
U+7385.0	fEnglish	 mysterious; subtle exquisite
U+7386.0	fUTF8	玆
U+7386.0	fCNS11643	1-565C
U+7386.0	fBig5	AF59
U+7386.0	fGB	3-3254
U+7386.0	fR/S	 95.5
U+7386.0	fTotalStrokes	 10
U+7386.0	fCangjie	YIYVI
U+7386.0	fFourCorner	0073
U+7386.0	fHakka	 zih1
U+7386.0	fCantonese	zi1; jyun4 (also); ci4
U+7386.0	fMandarin	zi1; xuan2; ci1
U+7386.0	fEnglish	 now; here this time; year
U+7387.0	fUTF8	率
U+7387.0	fCNS11643	1-5B7B
U+7387.0	fBig5	B276
U+7387.0	fGB	0-424A
U+7387.0	fR/S	 95.6
U+7387.0	fTotalStrokes	 11
U+7387.0	fCangjie	YIOJ
U+7387.0	fFourCorner	00403
U+7387.0	fHakka	  sut7; lit8; luk8; lut8
U+7387.0	fCantonese	seot1; leot6; leot2 (also); seoi3
U+7387.0	fMandarin	l4; luu4; le4
U+7387.0	fEnglish	[1] ratio (mathematics) [2] a suffix used to indicate a measure or rate [3] calculate
U+7387.1	fUTF8	率
U+7387.1	fCNS11643	1-5B7B
U+7387.1	fBig5	B276
U+7387.1	fGB	0-424A
U+7387.1	fR/S	 95.6
U+7387.1	fTotalStrokes	 11
U+7387.1	fCangjie	YIOJ
U+7387.1	fFourCorner	00403
U+7387.1	fMandarin	shuai4; shuo4; lei4; shua1
U+7387.1	fEnglish	[1] lead (troops, team, etc.); command [2] follow; act in accordance with [3] rash and hasty [4] generally; in general; usually [5] simple and candid; frank; straightforward; to the point [6] dashing (men)
U+7388.0	fUTF8	玈
U+7388.0	fCNS11643	2-372F
U+7388.0	fBig5	D66A
U+7388.0	fGB	5-3353
U+7388.0	fR/S	 95.6
U+7388.0	fTotalStrokes	 11
U+7388.0	fCangjie	YIOHV
U+7388.0	fFourCorner	08732
U+7388.0	fHakka	  lu2
U+7388.0	fCantonese	lou4
U+7388.0	fMandarin	lu2
U+7388.0	fEnglish	black
U+7389.0	fUTF8	玉
U+7389.0	fCNS11643	1-4669
U+7389.0	fBig5	A5C9
U+7389.0	fGB	0-5371
U+7389.0	fR/S	 96.0
U+7389.0	fTotalStrokes	 5
U+7389.0	fCangjie	MGI
U+7389.0	fFourCorner	10103
U+7389.0	fHakka	 ngiuk8; niuk8; gniuk8
U+7389.0	fCantonese	juk6; juk2 (also)
U+7389.0	fMandarin	yu4
U+7389.0	fEnglish	[1] jade; precious stone; gem [2] a polite expression for "your" [3] a designation of things belong to a girl or young woman [4] pure; fair; beautiful (person, woman) [5] a Chinese family name [6] Kangxi radical 96
U+738A.0	fUTF8	玊
U+738A.0	fCNS11643	2-2166
U+738A.0	fBig5	C9A7
U+738A.0	fGB	3-3E75
U+738A.0	fR/S	 96.0; 96.1
U+738A.0	fTotalStrokes	 5
U+738A.0	fCangjie	MGI
U+738A.0	fFourCorner	10104
U+738A.0	fCantonese	suk1
U+738A.0	fMandarin	su4; xiu4
U+738B.0	fUTF8	王
U+738B.0	fCNS11643	1-455E
U+738B.0	fBig5	A4FD
U+738B.0	fGB	0-4D75
U+738B.0	fR/S	 96.0
U+738B.0	fTotalStrokes	 4
U+738B.0	fCangjie	MG
U+738B.0	fFourCorner	10104
U+738B.0	fHakka	  wong2; wong3; vong2; wong5
U+738B.0	fCantonese	wong4; wong6; wong2 (also)
U+738B.0	fMandarin	wang2; wang3; yu4
U+738B.0	fEnglish	[1] king; ruler; royal [2] prince, the highest rank of nobility [3] great; of a tremendous size [4] the strongest or most powerful [5] a salutation of respect [6] an audience with the ruler or emperor [7] a Chinese family name
U+738B.1	fUTF8	王
U+738B.1	fCNS11643	1-455E
U+738B.1	fBig5	A4FD
U+738B.1	fGB	0-4D75
U+738B.1	fR/S	 96.0
U+738B.1	fTotalStrokes	 4
U+738B.1	fCangjie	MG
U+738B.1	fFourCorner	10104
U+738B.1	fMandarin	wang4
U+738B.1	fEnglish	 rule; govern
U+738C.0	fUTF8	玌
U+738C.0	fCNS11643	3-225A
U+738C.0	fGB	5-416B
U+738C.0	fR/S	  96.1
U+738C.0	fTotalStrokes	 5
U+738C.0	fCangjie	MGU
U+738C.0	fFourCorner	12110
U+738C.0	fCantonese	kau4
U+738C.0	fMandarin	qiu2
U+738D.0	fUTF8	玍
U+738D.0	fCNS11643	3-216B
U+738D.0	fGB	3-3E74
U+738D.0	fR/S	 4.4; 96.1
U+738D.0	fTotalStrokes	 5
U+738D.0	fCangjie	OG
U+738D.0	fFourCorner	80100
U+738D.0	fCantonese	gaa2; gaat3
U+738D.0	fMandarin	ga3
U+738D.0	fEnglish	 ill tempered naughty
U+738E.0	fUTF8	玎
U+738E.0	fCNS11643	2-2265
U+738E.0	fBig5	CA45
U+738E.0	fGB	0-6760
U+738E.0	fR/S	 96.2
U+738E.0	fTotalStrokes	 6
U+738E.0	fCangjie	MGMN
U+738E.0	fFourCorner	11120
U+738E.0	fHakka	  den1
U+738E.0	fCantonese	ding1
U+738E.0	fMandarin	ding1; zheng1
U+738E.0	fEnglish	jingling; tinkling sound
U+738F.0	fUTF8	玏
U+738F.0	fCNS11643	3-242F
U+738F.0	fGB	3-3E76
U+738F.0	fR/S	 96.2
U+738F.0	fTotalStrokes	 6
U+738F.0	fCangjie	MGKS
U+738F.0	fFourCorner	14120
U+738F.0	fCantonese	lak6
U+738F.0	fMandarin	le4
U+7390.0	fUTF8	玐
U+7390.0	fCNS11643	4-232A
U+7390.0	fGB	5-416D
U+7390.0	fR/S	  96.2
U+7390.0	fTotalStrokes	 6
U+7390.0	fCangjie	MGC
U+7390.0	fFourCorner	18100
U+7390.0	fCantonese	baat3
U+7390.0	fMandarin	ba1
U+7391.0	fUTF8	玑
U+7391.0	fCNS11643	F-2266
U+7391.0	fGB	0-6761
U+7391.0	fR/S	 96.2
U+7391.0	fTotalStrokes	 6
U+7391.0	fCangjie	MGHN
U+7391.0	fHakka	 gi1
U+7391.0	fCantonese	gei1
U+7391.0	fMandarin	ji1; ji4
U+7391.0	fEnglish	simplified form of &#x74A3;
U+7392.0	fUTF8	玒
U+7392.0	fCNS11643	2-246C
U+7392.0	fBig5	CB6C
U+7392.0	fGB	3-3E79
U+7392.0	fR/S	 96.3
U+7392.0	fTotalStrokes	 7
U+7392.0	fCangjie	MGM
U+7392.0	fFourCorner	11110
U+7392.0	fCantonese	gung1; gong1; hung4
U+7392.0	fMandarin	hong2; jiang1
U+7393.0	fUTF8	玓
U+7393.0	fCNS11643	2-246A
U+7393.0	fBig5	CB6A
U+7393.0	fGB	5-4171
U+7393.0	fR/S	 96.3
U+7393.0	fTotalStrokes	 7
U+7393.0	fCangjie	MGPI
U+7393.0	fFourCorner	17120
U+7393.0	fHakka	  dit7
U+7393.0	fCantonese	dik1
U+7393.0	fMandarin	di4
U+7393.0	fEnglish	pearly
U+7394.0	fUTF8	玔
U+7394.0	fCNS11643	2-246B
U+7394.0	fBig5	CB6B
U+7394.0	fGB	5-4170
U+7394.0	fR/S	 96.3
U+7394.0	fTotalStrokes	 7
U+7394.0	fCangjie	MGLLL
U+7394.0	fFourCorner	12100
U+7394.0	fHakka	 chon5; con5
U+7394.0	fCantonese	cyun3; cyun1 (also)
U+7394.0	fMandarin	chuan4; quan4
U+7394.0	fEnglish	green jade
U+7395.0	fUTF8	玕
U+7395.0	fCNS11643	2-2468
U+7395.0	fBig5	CB68
U+7395.0	fGB	3-3E77
U+7395.0	fR/S	 96.3
U+7395.0	fTotalStrokes	 7
U+7395.0	fCangjie	MGMJ
U+7395.0	fFourCorner	11140
U+7395.0	fHakka	 gon1
U+7395.0	fCantonese	gon1
U+7395.0	fMandarin	gan1
U+7395.0	fEnglish	an inferior variety of gem; a stone slightly inferior in quality to jade
U+7396.0	fUTF8	玖
U+7396.0	fCNS11643	1-4B2B
U+7396.0	fBig5	A868
U+7396.0	fGB	0-3E41
U+7396.0	fR/S	 96.3
U+7396.0	fTotalStrokes	 7
U+7396.0	fCangjie	MGNO
U+7396.0	fFourCorner	17180
U+7396.0	fHakka	 giu3
U+7396.0	fCantonese	gau2
U+7396.0	fMandarin	jiu3; jiu4
U+7396.0	fEnglish	[1] a black jade stone [2] nine used in accountancy
U+7397.0	fUTF8	玗
U+7397.0	fCNS11643	2-2469
U+7397.0	fBig5	CB69
U+7397.0	fGB	3-3E78
U+7397.0	fR/S	 96.3
U+7397.0	fTotalStrokes	 7
U+7397.0	fCangjie	MGMD
U+7397.0	fFourCorner	11140
U+7397.0	fCantonese	jyu1; jyu4
U+7397.0	fMandarin	yu2
U+7397.0	fEnglish	1/2 pr stone
U+7398.0	fUTF8	玘
U+7398.0	fCNS11643	3-266E
U+7398.0	fGB	3-3E7B
U+7398.0	fR/S	 96.3
U+7398.0	fTotalStrokes	 7
U+7398.0	fCangjie	MGSU
U+7398.0	fFourCorner	17110
U+7398.0	fHakka	 hi3;  gi3
U+7398.0	fCantonese	hei2
U+7398.0	fMandarin	ji3; qi3
U+7398.0	fEnglish	court-dress jewels
U+7399.0	fUTF8	玙
U+7399.0	fGB	8-7B48
U+7399.0	fR/S	 96.3
U+7399.0	fTotalStrokes	 7
U+7399.0	fCangjie	MGYSM
U+7399.0	fCantonese	jyu4
U+7399.0	fMandarin	yu2
U+739A.0	fUTF8	玚
U+739A.0	fGB	8-2C4A
U+739A.0	fR/S	 96.3
U+739A.0	fTotalStrokes	 7
U+739A.0	fCangjie	MGNSH
U+739A.0	fCantonese	coeng3; dong6; joeng4
U+739A.0	fMandarin	yang2; chang4; dang4
U+739B.0	fUTF8	玛
U+739B.0	fGB	0-426A
U+739B.0	fR/S	 96.3
U+739B.0	fTotalStrokes	 7
U+739B.0	fCangjie	MGNVM
U+739B.0	fHakka	 ma1; ma3
U+739B.0	fCantonese	maa5
U+739B.0	fMandarin	ma3
U+739B.0	fEnglish	simplified form of &#x746A;
U+739C.0	fUTF8	玜
U+739C.0	fCNS11643	3-2A33
U+739C.0	fGB	5-417A
U+739C.0	fR/S	  96.4
U+739C.0	fTotalStrokes	 8
U+739C.0	fCangjie	MGCI
U+739C.0	fFourCorner	18130
U+739C.0	fCantonese	gung1
U+739C.0	fMandarin	gong1; jiang1
U+739D.0	fUTF8	玝
U+739D.0	fCNS11643	2-282F
U+739D.0	fBig5	CD6D
U+739D.0	fGB	5-4178
U+739D.0	fR/S	 96.4
U+739D.0	fTotalStrokes	 8
U+739D.0	fCangjie	MGOJ
U+739D.0	fFourCorner	18140
U+739D.0	fHakka	 ng3
U+739D.0	fCantonese	ng5
U+739D.0	fMandarin	wu3
U+739E.0	fUTF8	玞
U+739E.0	fCNS11643	3-2A2D
U+739E.0	fGB	5-4173
U+739E.0	fR/S	 96.4
U+739E.0	fTotalStrokes	 8
U+739E.0	fCangjie	MGQO
U+739E.0	fFourCorner	15130
U+739E.0	fHakka	 fu1
U+739E.0	fCantonese	fu1
U+739E.0	fMandarin	fu1
U+739E.0	fEnglish	an inferior agate
U+739F.0	fUTF8	玟
U+739F.0	fCNS11643	1-4E74
U+739F.0	fBig5	AAB3
U+739F.0	fGB	0-6764
U+739F.0	fR/S	 96.4
U+739F.0	fTotalStrokes	 8
U+739F.0	fCangjie	MGYK
U+739F.0	fFourCorner	10140
U+739F.0	fHakka	 wun2;  men2; min2; vun2
U+739F.0	fCantonese	man4
U+739F.0	fMandarin	min2; wen2
U+739F.0	fEnglish	 streaks in jade gem
U+73A0.0	fUTF8	玠
U+73A0.0	fCNS11643	2-282D
U+73A0.0	fBig5	CD6B
U+73A0.0	fGB	3-3F23
U+73A0.0	fR/S	 96.4
U+73A0.0	fTotalStrokes	 8
U+73A0.0	fCangjie	MGOLL
U+73A0.0	fFourCorner	18120
U+73A0.0	fHakka	 gai5; kie5; giai5
U+73A0.0	fCantonese	gaai3
U+73A0.0	fMandarin	jie4
U+73A0.0	fEnglish	 large jade tablet
U+73A1.0	fUTF8	玡
U+73A1.0	fCNS11643	2-2829
U+73A1.0	fBig5	CD67
U+73A1.0	fGB	3-3F22
U+73A1.0	fR/S	 96.4
U+73A1.0	fTotalStrokes	 8
U+73A1.0	fCangjie	MGMVH
U+73A1.0	fFourCorner	11140
U+73A1.0	fHakka	 nga3
U+73A1.0	fCantonese	ngaa4; je4 (also); aa4 (also)
U+73A1.0	fMandarin	ya4
U+73A2.0	fUTF8	玢
U+73A2.0	fCNS11643	2-282C
U+73A2.0	fBig5	CD6A
U+73A2.0	fGB	0-6763
U+73A2.0	fR/S	 96.4
U+73A2.0	fTotalStrokes	 8
U+73A2.0	fCangjie	MGCSH
U+73A2.0	fFourCorner	18127
U+73A2.0	fHakka	 bin1; fun2
U+73A2.0	fCantonese	ban1; fan1 (also); fan4
U+73A2.0	fMandarin	bin1; fen1; bian1; fen2
U+73A3.0	fUTF8	玣
U+73A3.0	fCNS11643	3-2A2C
U+73A3.0	fGB	5-417E
U+73A3.0	fR/S	  96.4
U+73A3.0	fTotalStrokes	 8
U+73A3.0	fCangjie	MGYY
U+73A3.0	fFourCorner	40230
U+73A3.0	fCantonese	bin6
U+73A3.0	fMandarin	bian4
U+73A4.0	fUTF8	玤
U+73A4.0	fCNS11643	2-2828
U+73A4.0	fBig5	CD66
U+73A4.0	fGB	3-3E7D
U+73A4.0	fR/S	 96.4
U+73A4.0	fTotalStrokes	 8
U+73A4.0	fCangjie	MGQJ
U+73A4.0	fFourCorner	15100
U+73A4.0	fCantonese	bung2; pong5
U+73A4.0	fMandarin	beng3; bang4
U+73A4.0	fEnglish	a kind of gem inferior to jade
U+73A5.0	fUTF8	玥
U+73A5.0	fCNS11643	1-4E76
U+73A5.0	fBig5	AAB5
U+73A5.0	fGB	3-3F26
U+73A5.0	fR/S	 96.4
U+73A5.0	fTotalStrokes	 8
U+73A5.0	fCangjie	MGB
U+73A5.0	fFourCorner	17120
U+73A5.0	fCantonese	jyut6
U+73A5.0	fMandarin	yue4
U+73A5.0	fEnglish	 mysterious gem pearl
U+73A6.0	fUTF8	玦
U+73A6.0	fCNS11643	2-282B
U+73A6.0	fBig5	CD69
U+73A6.0	fGB	3-3F27
U+73A6.0	fR/S	 96.4
U+73A6.0	fTotalStrokes	 8
U+73A6.0	fCangjie	MGDK
U+73A6.0	fFourCorner	15130
U+73A6.0	fHakka	 giet7; get7
U+73A6.0	fCantonese	kyut3
U+73A6.0	fMandarin	jue2
U+73A6.0	fEnglish	a jade ring with a small segment cut of
U+73A7.0	fUTF8	玧
U+73A7.0	fCNS11643	3-2A2B
U+73A7.0	fGB	5-4221
U+73A7.0	fR/S	  96.4
U+73A7.0	fTotalStrokes	 8
U+73A7.0	fCangjie	MGIHU
U+73A7.0	fFourCorner	42213
U+73A7.0	fCantonese	mun4
U+73A7.0	fMandarin	yun3; yan2; men2
U+73A7.0	fEnglish	reddish
U+73A8.0	fUTF8	玨
U+73A8.0	fCNS11643	1-4E73
U+73A8.0	fBig5	AAB2
U+73A8.0	fGB	5-4172
U+73A8.0	fR/S	 96.4
U+73A8.0	fTotalStrokes	 8
U+73A8.0	fCangjie	MGMG
U+73A8.0	fFourCorner	11114
U+73A8.0	fHakka	 gok7
U+73A8.0	fCantonese	gok3
U+73A8.0	fMandarin	jue2
U+73A8.0	fEnglish	two pieces of jade joined together
U+73A9.0	fUTF8	玩
U+73A9.0	fCNS11643	1-4E72
U+73A9.0	fBig5	AAB1
U+73A9.0	fGB	0-4D66
U+73A9.0	fR/S	 96.4
U+73A9.0	fTotalStrokes	 8
U+73A9.0	fCangjie	MGMMU
U+73A9.0	fFourCorner	11111
U+73A9.0	fHakka	  ngan3; ngan5;  ngan2; nguan3; ngan6
U+73A9.0	fCantonese	wun6; waan4; wun2 (also); waan2; wan4
U+73A9.0	fMandarin	wan2
U+73A9.0	fEnglish	[1] play with; toy with [2] enjoy oneself; amuse oneself; find pleasure in
U+73A9.1	fUTF8	玩
U+73A9.1	fCNS11643	1-4E72
U+73A9.1	fBig5	AAB1
U+73A9.1	fGB	0-4D66
U+73A9.1	fR/S	 96.4
U+73A9.1	fTotalStrokes	 8
U+73A9.1	fCangjie	MGMMU
U+73A9.1	fFourCorner	11111
U+73A9.1	fMandarin	wan4
U+73A9.1	fEnglish	[1] joke; take things lightly; toy with [2] something to amuse oneself (as antiques, etc.) [3] learn
U+73AA.0	fUTF8	玪
U+73AA.0	fCNS11643	3-2A32
U+73AA.0	fGB	5-4179
U+73AA.0	fR/S	  96.4
U+73AA.0	fTotalStrokes	 8
U+73AA.0	fCangjie	MGOIN
U+73AA.0	fFourCorner	18120
U+73AA.0	fCantonese	gaam1
U+73AA.0	fMandarin	jian1; qian2; yin2; lin2
U+73AB.0	fUTF8	玫
U+73AB.0	fCNS11643	1-4E75
U+73AB.0	fBig5	AAB4
U+73AB.0	fGB	0-4335
U+73AB.0	fR/S	 96.4
U+73AB.0	fTotalStrokes	 8
U+73AB.0	fCangjie	MGOK
U+73AB.0	fFourCorner	18140
U+73AB.0	fHakka	  moi2; mi2; mui2; moe2
U+73AB.0	fCantonese	mui4
U+73AB.0	fMandarin	mei2
U+73AB.0	fEnglish	[1] the rose [2] another name of black mica, a sparkling red gem
U+73AC.0	fUTF8	玬
U+73AC.0	fCNS11643	2-282E
U+73AC.0	fBig5	CD6C
U+73AC.0	fGB	5-417B
U+73AC.0	fR/S	 96.4
U+73AC.0	fTotalStrokes	 8
U+73AC.0	fCangjie	MGBY
U+73AC.0	fFourCorner	17140
U+73AC.0	fCantonese	taan2
U+73AC.0	fMandarin	dan3
U+73AD.0	fUTF8	玭
U+73AD.0	fCNS11643	2-282A
U+73AD.0	fBig5	CD68
U+73AD.0	fGB	5-4176
U+73AD.0	fR/S	 96.4
U+73AD.0	fTotalStrokes	 8
U+73AD.0	fCangjie	MGPP
U+73AD.0	fFourCorner	11110
U+73AD.0	fCantonese	pan4; pin4 (also)
U+73AD.0	fMandarin	pian2; pin2; pi2
U+73AE.0	fUTF8	玮
U+73AE.0	fGB	0-6762
U+73AE.0	fR/S	 96.4
U+73AE.0	fTotalStrokes	 8
U+73AE.0	fCangjie	MGQS
U+73AE.0	fHakka	 vui3
U+73AE.0	fCantonese	wai5; wai4
U+73AE.0	fMandarin	wei3; wei4
U+73AE.0	fEnglish	simplified form of &#x744B;
U+73AF.0	fUTF8	环
U+73AF.0	fCNS11643	3-2A2F
U+73AF.0	fGB	0-3B37
U+73AF.0	fR/S	 96.4
U+73AF.0	fTotalStrokes	 8
U+73AF.0	fCangjie	MGMF
U+73AF.0	fFourCorner	11190
U+73AF.0	fHakka	 fan2
U+73AF.0	fCantonese	waan4; waan2 (also)
U+73AF.0	fMandarin	huan2; huan4
U+73AF.0	fEnglish	simplified form of &#x74B0;
U+73B0.0	fUTF8	现
U+73B0.0	fGB	0-4F56
U+73B0.0	fR/S	 96.4
U+73B0.0	fTotalStrokes	 8
U+73B0.0	fCangjie	MGBHU
U+73B0.0	fHakka	 hien5
U+73B0.0	fCantonese	jin6
U+73B0.0	fMandarin	xian4
U+73B0.0	fEnglish	simplified form of &#x73FE;
U+73B1.0	fUTF8	玱
U+73B1.0	fGB	8-7B4A
U+73B1.0	fR/S	 96.4
U+73B1.0	fTotalStrokes	 8
U+73B1.0	fCangjie	MGOSU
U+73B1.0	fCantonese	coeng1
U+73B1.0	fMandarin	qiang1
U+73B1.0	fEnglish	tinkling sound tinkling of pendant gems
U+73B2.0	fUTF8	玲
U+73B2.0	fCNS11643	1-5245
U+73B2.0	fBig5	ACC2
U+73B2.0	fGB	0-4161
U+73B2.0	fR/S	 96.5
U+73B2.0	fTotalStrokes	 9
U+73B2.0	fCangjie	MGOII
U+73B2.0	fFourCorner	18137
U+73B2.0	fHakka	 lang2;  lin2
U+73B2.0	fCantonese	ling4; ling2 (also)
U+73B2.0	fMandarin	ling2; lin2
U+73B2.0	fEnglish	the tinkling of jade pendants
U+73B3.0	fUTF8	玳
U+73B3.0	fCNS11643	1-5248
U+73B3.0	fBig5	ACC5
U+73B3.0	fGB	0-6769
U+73B3.0	fR/S	 96.5
U+73B3.0	fTotalStrokes	 9
U+73B3.0	fCangjie	MGOIP
U+73B3.0	fFourCorner	13140
U+73B3.0	fHakka	 toi5; toi6; toi3
U+73B3.0	fCantonese	doi6
U+73B3.0	fMandarin	dai4
U+73B3.0	fEnglish	the tortoise shell
U+73B4.0	fUTF8	玴
U+73B4.0	fCNS11643	2-2C30
U+73B4.0	fBig5	CFCE
U+73B4.0	fGB	5-4223
U+73B4.0	fR/S	 96.5
U+73B4.0	fTotalStrokes	 9
U+73B4.0	fCangjie	MGPT
U+73B4.0	fFourCorner	14117
U+73B4.0	fCantonese	jai6
U+73B4.0	fMandarin	yi4
U+73B5.0	fUTF8	玵
U+73B5.0	fCNS11643	2-2C2F
U+73B5.0	fBig5	CFCD
U+73B5.0	fGB	5-4222
U+73B5.0	fR/S	 96.5
U+73B5.0	fTotalStrokes	 9
U+73B5.0	fCangjie	MGTM
U+73B5.0	fFourCorner	14170
U+73B5.0	fCantonese	ngam4; am4 (also)
U+73B5.0	fMandarin	an2
U+73B6.0	fUTF8	玶
U+73B6.0	fCNS11643	2-2C2E
U+73B6.0	fBig5	CFCC
U+73B6.0	fGB	3-3F28
U+73B6.0	fR/S	 96.5
U+73B6.0	fTotalStrokes	 9
U+73B6.0	fCangjie	MGMFJ
U+73B6.0	fFourCorner	11149
U+73B6.0	fCantonese	ping4
U+73B6.0	fMandarin	ping2
U+73B6.0	fEnglish	name of one kind of jade
U+73B7.0	fUTF8	玷
U+73B7.0	fCNS11643	1-5242
U+73B7.0	fBig5	ACBF
U+73B7.0	fGB	0-6768
U+73B7.0	fR/S	 96.5
U+73B7.0	fTotalStrokes	 9
U+73B7.0	fCangjie	MGYR
U+73B7.0	fFourCorner	11160
U+73B7.0	fHakka	 diam5
U+73B7.0	fCantonese	dim3; dim2
U+73B7.0	fMandarin	dian4; dian1; dian3
U+73B7.0	fEnglish	[1] flaw or blemish in a jade; stain; defect; spot [2] stain; blemish; disgrace
U+73B8.0	fUTF8	玸
U+73B8.0	fCNS11643	2-2C37
U+73B8.0	fBig5	CFD5
U+73B8.0	fGB	3-3F2D
U+73B8.0	fR/S	 96.5
U+73B8.0	fTotalStrokes	 9
U+73B8.0	fCangjie	MGPRU
U+73B8.0	fFourCorner	17112
U+73B8.0	fCantonese	fu4
U+73B8.0	fMandarin	fu2
U+73B9.0	fUTF8	玹
U+73B9.0	fCNS11643	2-2C2D
U+73B9.0	fBig5	CFCB
U+73B9.0	fGB	3-3F2E
U+73B9.0	fR/S	 96.5
U+73B9.0	fTotalStrokes	 9
U+73B9.0	fCangjie	MGYVI
U+73B9.0	fFourCorner	10132
U+73B9.0	fHakka	 hien2
U+73B9.0	fCantonese	jyun4; jin4; jyun6
U+73B9.0	fMandarin	xuan2
U+73BA.0	fUTF8	玺
U+73BA.0	fCNS11643	4-2E2A
U+73BA.0	fGB	0-6774
U+73BA.0	fR/S	 96.5
U+73BA.0	fTotalStrokes	 10
U+73BA.0	fCangjie	NFMGI
U+73BA.0	fFourCorner	27100
U+73BA.0	fHakka	 sai3
U+73BA.0	fCantonese	saai2
U+73BA.0	fMandarin	xi3
U+73BA.0	fEnglish	simplified form of &#x74BD;
U+73BB.0	fUTF8	玻
U+73BB.0	fCNS11643	1-5244
U+73BB.0	fBig5	ACC1
U+73BB.0	fGB	0-3223
U+73BB.0	fR/S	 96.5
U+73BB.0	fTotalStrokes	 9
U+73BB.0	fCangjie	MGDHE
U+73BB.0	fFourCorner	14147
U+73BB.0	fHakka	  bo1
U+73BB.0	fCantonese	bo1
U+73BB.0	fMandarin	bo1
U+73BB.0	fEnglish	glass
U+73BC.0	fUTF8	玼
U+73BC.0	fCNS11643	2-3070
U+73BC.0	fBig5	D2AF
U+73BC.0	fGB	3-3F35
U+73BC.0	fR/S	 96.5
U+73BC.0	fTotalStrokes	 9
U+73BC.0	fCangjie	MGYMP
U+73BC.0	fFourCorner	11110
U+73BC.0	fHakka	 cih3; ze3
U+73BC.0	fCantonese	ci2; ci1; cai2
U+73BC.0	fMandarin	ci3; cuo3; ci1; ci2
U+73BC.0	fEnglish	[1] flaw; blemish [2] brilliant
U+73BD.0	fUTF8	玽
U+73BD.0	fCNS11643	3-2D78
U+73BD.0	fGB	3-3F2B
U+73BD.0	fR/S	  96.5
U+73BD.0	fTotalStrokes	 9
U+73BD.0	fCangjie	MGPR
U+73BD.0	fFourCorner	17120
U+73BD.0	fCantonese	gau2
U+73BD.0	fMandarin	gou3
U+73BE.0	fUTF8	玾
U+73BE.0	fCNS11643	2-2C34
U+73BE.0	fBig5	CFD2
U+73BE.0	fGB	5-4224
U+73BE.0	fR/S	 96.5
U+73BE.0	fTotalStrokes	 9
U+73BE.0	fCangjie	MGWL
U+73BE.0	fFourCorner	16150
U+73BE.0	fCantonese	gaap3
U+73BE.0	fMandarin	jia3
U+73BF.0	fUTF8	玿
U+73BF.0	fCNS11643	2-2C32
U+73BF.0	fBig5	CFD0
U+73BF.0	fGB	5-4228
U+73BF.0	fR/S	 96.5
U+73BF.0	fTotalStrokes	 9
U+73BF.0	fCangjie	MGSHR
U+73BF.0	fFourCorner	17162
U+73BF.0	fCantonese	siu4
U+73BF.0	fMandarin	shao2
U+73C0.0	fUTF8	珀
U+73C0.0	fCNS11643	1-5247
U+73C0.0	fBig5	ACC4
U+73C0.0	fGB	0-676A
U+73C0.0	fR/S	 96.5
U+73C0.0	fTotalStrokes	 9
U+73C0.0	fCangjie	MGHA
U+73C0.0	fFourCorner	16100
U+73C0.0	fHakka	 pak7; pet7
U+73C0.0	fCantonese	paak3
U+73C0.0	fMandarin	po4
U+73C0.0	fEnglish	amber
U+73C1.0	fUTF8	珁
U+73C1.0	fCNS11643	3-2D7B
U+73C1.0	fGB	E-3332
U+73C1.0	fR/S	 96.5
U+73C1.0	fTotalStrokes	 9
U+73C1.0	fCangjie	MGMVN
U+73C1.0	fFourCorner	11110
U+73C1.0	fCantonese	ci4
U+73C1.0	fMandarin	ci2
U+73C2.0	fUTF8	珂
U+73C2.0	fCNS11643	2-2C2A
U+73C2.0	fBig5	CFC8
U+73C2.0	fGB	0-6766
U+73C2.0	fR/S	 96.5
U+73C2.0	fTotalStrokes	 9
U+73C2.0	fCangjie	MGMNR
U+73C2.0	fFourCorner	11120
U+73C2.0	fHakka	 ko1
U+73C2.0	fCantonese	ngo1 (also); o1
U+73C2.0	fMandarin	ke1
U+73C2.0	fEnglish	[1] a stone resembling jade [2] a decorative article on a bridle [3] a kind of cowry
U+73C3.0	fUTF8	珃
U+73C3.0	fCNS11643	2-2C35
U+73C3.0	fBig5	CFD3
U+73C3.0	fGB	5-4225
U+73C3.0	fR/S	 96.5
U+73C3.0	fTotalStrokes	 9
U+73C3.0	fCangjie	MGGB
U+73C3.0	fFourCorner	15157
U+73C3.0	fCantonese	jim5
U+73C3.0	fMandarin	ran3
U+73C4.0	fUTF8	珄
U+73C4.0	fCNS11643	3-2D77
U+73C4.0	fGB	E-3333
U+73C4.0	fR/S	  96.5
U+73C4.0	fTotalStrokes	 9
U+73C4.0	fCangjie	MGHQM
U+73C4.0	fFourCorner	15110
U+73C4.0	fCantonese	sang1
U+73C4.0	fMandarin	sheng1
U+73C5.0	fUTF8	珅
U+73C5.0	fCNS11643	2-2C2C
U+73C5.0	fBig5	CFCA
U+73C5.0	fGB	3-3F2A
U+73C5.0	fR/S	 96.5
U+73C5.0	fTotalStrokes	 9
U+73C5.0	fCangjie	MGLWL
U+73C5.0	fFourCorner	15106
U+73C5.0	fCantonese	san1
U+73C5.0	fMandarin	shen1
U+73C6.0	fUTF8	珆
U+73C6.0	fCNS11643	2-2C36
U+73C6.0	fBig5	CFD4
U+73C6.0	fGB	3-3F31
U+73C6.0	fR/S	 96.5
U+73C6.0	fTotalStrokes	 9
U+73C6.0	fCangjie	MGIR
U+73C6.0	fFourCorner	13160
U+73C6.0	fCantonese	ji4; toi1
U+73C6.0	fMandarin	yi2; tai1
U+73C7.0	fUTF8	珇
U+73C7.0	fCNS11643	2-2C33
U+73C7.0	fBig5	CFD1
U+73C7.0	fGB	3-3F29
U+73C7.0	fR/S	 96.5
U+73C7.0	fTotalStrokes	 9
U+73C7.0	fCangjie	MGBM
U+73C7.0	fFourCorner	17110
U+73C7.0	fCantonese	zou2; zeoi2
U+73C7.0	fMandarin	zu3; ju3
U+73C8.0	fUTF8	珈
U+73C8.0	fCNS11643	2-2C2B
U+73C8.0	fBig5	CFC9
U+73C8.0	fGB	0-676C
U+73C8.0	fR/S	 96.5
U+73C8.0	fTotalStrokes	 9
U+73C8.0	fCangjie	MGKSR
U+73C8.0	fFourCorner	16100
U+73C8.0	fHakka	  ga1
U+73C8.0	fCantonese	gaa1
U+73C8.0	fMandarin	jia1
U+73C8.0	fEnglish	a kind of jewelry; ornament attached woman's hairpin
U+73C9.0	fUTF8	珉
U+73C9.0	fCNS11643	3-2D7A
U+73C9.0	fGB	0-676B
U+73C9.0	fR/S	  96.5
U+73C9.0	fTotalStrokes	 9
U+73C9.0	fCangjie	MGRVP
U+73C9.0	fFourCorner	17140
U+73C9.0	fHakka	  min2
U+73C9.0	fCantonese	man4
U+73C9.0	fMandarin	min2; wen2
U+73C9.0	fEnglish	a stone resembling jade
U+73CA.0	fUTF8	珊
U+73CA.0	fCNS11643	1-5243
U+73CA.0	fBig5	ACC0
U+73CA.0	fGB	0-493A
U+73CA.0	fR/S	 96.5
U+73CA.0	fTotalStrokes	 9
U+73CA.0	fCangjie	MGBT
U+73CA.0	fFourCorner	17140
U+73CA.0	fHakka	  san1
U+73CA.0	fCantonese	saan1
U+73CA.0	fMandarin	shan1; sa4
U+73CA.0	fEnglish	[1] coral [2] the tinkling of pendants
U+73CB.0	fUTF8	珋
U+73CB.0	fCNS11643	2-2C38
U+73CB.0	fBig5	CFD6
U+73CB.0	fGB	5-4226
U+73CB.0	fR/S	 96.5
U+73CB.0	fTotalStrokes	 9
U+73CB.0	fCangjie	MGHHL
U+73CB.0	fFourCorner	17120
U+73CB.0	fCantonese	lau5
U+73CB.0	fMandarin	liu3
U+73CC.0	fUTF8	珌
U+73CC.0	fCNS11643	2-2C29
U+73CC.0	fBig5	CFC7
U+73CC.0	fGB	3-3F2F
U+73CC.0	fR/S	 96.5
U+73CC.0	fTotalStrokes	 9
U+73CC.0	fCangjie	MGPH
U+73CC.0	fFourCorner	13100
U+73CC.0	fHakka	 bit7
U+73CC.0	fCantonese	bit1; bat1 (also); bei3
U+73CC.0	fMandarin	bi4; bie4
U+73CC.0	fEnglish	ornament
U+73CD.0	fUTF8	珍
U+73CD.0	fCNS11643	1-5246
U+73CD.0	fBig5	ACC3
U+73CD.0	fGB	0-5564
U+73CD.0	fR/S	 96.5
U+73CD.0	fTotalStrokes	 9
U+73CD.0	fCangjie	MGOHH
U+73CD.0	fFourCorner	18122
U+73CD.0	fHakka	  zhin1;  zin1; zihn1; zin3
U+73CD.0	fCantonese	zan1
U+73CD.0	fMandarin	zhen1
U+73CD.0	fEnglish	[1] precious; valuable; rare [2] valuables; treasure [3] delicacies; dainties
U+73CE.0	fUTF8	珎
U+73CE.0	fCNS11643	3-2D7E
U+73CE.0	fGB	E-3334
U+73CE.0	fR/S	  96.5
U+73CE.0	fTotalStrokes	 9
U+73CE.0	fCangjie	MGNF
U+73CE.0	fFourCorner	17190
U+73CE.0	fHakka	 zhin1; zihn1
U+73CE.0	fCantonese	zan1
U+73CE.0	fMandarin	zhen1
U+73CE.0	fEnglish	precious; valuable; rare
U+73CF.0	fUTF8	珏
U+73CF.0	fCNS11643	3-2D79
U+73CF.0	fGB	0-6765
U+73CF.0	fR/S	  96.5
U+73CF.0	fTotalStrokes	 9
U+73CF.0	fCangjie	MGMGI
U+73CF.0	fFourCorner	11110
U+73CF.0	fHakka	  gok7
U+73CF.0	fCantonese	gok3
U+73CF.0	fMandarin	jue2; gu3
U+73CF.0	fEnglish	two pieces of jade joined together
U+73D0.0	fUTF8	珐
U+73D0.0	fCNS11643	3-2D7C
U+73D0.0	fGB	0-3729
U+73D0.0	fR/S	 96.5
U+73D0.0	fTotalStrokes	 9
U+73D0.0	fCangjie	MGGI
U+73D0.0	fFourCorner	14130
U+73D0.0	fHakka	 fap7
U+73D0.0	fCantonese	faat3
U+73D0.0	fMandarin	fa4
U+73D0.0	fEnglish	 enamel enamelware; cloissonewar
U+73D1.0	fUTF8	珑
U+73D1.0	fGB	0-6767
U+73D1.0	fR/S	 96.5
U+73D1.0	fTotalStrokes	 9
U+73D1.0	fCangjie	MGIKP
U+73D1.0	fHakka	 lung2
U+73D1.0	fCantonese	lung4
U+73D1.0	fMandarin	long2
U+73D1.0	fEnglish	simplified form of &#x74CF;
U+73D2.0	fUTF8	珒
U+73D2.0	fCNS11643	2-3075
U+73D2.0	fBig5	D2B4
U+73D2.0	fGB	3-3F42
U+73D2.0	fR/S	 96.6
U+73D2.0	fTotalStrokes	 10
U+73D2.0	fCangjie	MGLQ
U+73D2.0	fFourCorner	15107
U+73D2.0	fCantonese	zeon1
U+73D2.0	fMandarin	jin1
U+73D3.0	fUTF8	珓
U+73D3.0	fCNS11643	2-306C
U+73D3.0	fBig5	D2AB
U+73D3.0	fGB	3-3F40
U+73D3.0	fR/S	 96.6
U+73D3.0	fTotalStrokes	 10
U+73D3.0	fCangjie	MGYCK
U+73D3.0	fFourCorner	10148
U+73D3.0	fHakka	  gau5; kau5
U+73D3.0	fCantonese	gaau3
U+73D3.0	fMandarin	jiao4
U+73D3.0	fEnglish	two hemispherical objects used i
U+73D4.0	fUTF8	珔
U+73D4.0	fCNS11643	2-3077
U+73D4.0	fBig5	D2B6
U+73D4.0	fGB	5-422A
U+73D4.0	fR/S	 96.6
U+73D4.0	fTotalStrokes	 10
U+73D4.0	fCangjie	MGKLD
U+73D4.0	fFourCorner	14147
U+73D4.0	fCantonese	zin6
U+73D4.0	fMandarin	jian4
U+73D5.0	fUTF8	珕
U+73D5.0	fCNS11643	3-323E
U+73D5.0	fGB	3-3F45
U+73D5.0	fR/S	  96.6
U+73D5.0	fTotalStrokes	 10
U+73D5.0	fCangjie	MGKSS
U+73D5.0	fFourCorner	14120
U+73D5.0	fCantonese	lai6
U+73D5.0	fMandarin	li4
U+73D6.0	fUTF8	珖
U+73D6.0	fCNS11643	2-306F
U+73D6.0	fBig5	D2AE
U+73D6.0	fGB	3-3F36
U+73D6.0	fR/S	 96.6
U+73D6.0	fTotalStrokes	 10
U+73D6.0	fCangjie	MGFMU
U+73D6.0	fFourCorner	19111
U+73D6.0	fCantonese	gwong1; gong1 (also)
U+73D6.0	fMandarin	guang1
U+73D6.0	fEnglish	carat
U+73D7.0	fUTF8	珗
U+73D7.0	fCNS11643	2-307A
U+73D7.0	fBig5	D2B9
U+73D7.0	fGB	E-3335
U+73D7.0	fR/S	 96.6
U+73D7.0	fTotalStrokes	 10
U+73D7.0	fCangjie	MGHGU
U+73D7.0	fFourCorner	14111
U+73D7.0	fCantonese	sin1
U+73D7.0	fMandarin	xian1
U+73D8.0	fUTF8	珘
U+73D8.0	fCNS11643	2-307B
U+73D8.0	fBig5	D2BA
U+73D8.0	fGB	5-422F
U+73D8.0	fR/S	 96.6
U+73D8.0	fTotalStrokes	 10
U+73D8.0	fCangjie	MGHBY
U+73D8.0	fFourCorner	17140
U+73D8.0	fCantonese	zau1
U+73D8.0	fMandarin	zhou1
U+73D9.0	fUTF8	珙
U+73D9.0	fCNS11643	2-306D
U+73D9.0	fBig5	D2AC
U+73D9.0	fGB	0-676E
U+73D9.0	fR/S	 96.6
U+73D9.0	fTotalStrokes	 10
U+73D9.0	fCangjie	MGTC
U+73D9.0	fFourCorner	14181
U+73D9.0	fHakka	  giung3
U+73D9.0	fCantonese	gung2
U+73D9.0	fMandarin	gong3; gong1; hong2
U+73D9.0	fEnglish	a large piece of jade stone
U+73DA.0	fUTF8	珚
U+73DA.0	fCNS11643	2-3079
U+73DA.0	fBig5	D2B8
U+73DA.0	fGB	5-422D
U+73DA.0	fR/S	 96.6
U+73DA.0	fTotalStrokes	 10
U+73DA.0	fCangjie	MGWK
U+73DA.0	fFourCorner	16100
U+73DA.0	fCantonese	jin1
U+73DA.0	fMandarin	yan1
U+73DB.0	fUTF8	珛
U+73DB.0	fCNS11643	2-3076
U+73DB.0	fBig5	D2B5
U+73DB.0	fGB	3-3F33
U+73DB.0	fR/S	 96.6
U+73DB.0	fTotalStrokes	 10
U+73DB.0	fCangjie	MGKB
U+73DB.0	fFourCorner	14127
U+73DB.0	fCantonese	jau3; cau3
U+73DB.0	fMandarin	xiu4; xu4
U+73DC.0	fUTF8	珜
U+73DC.0	fCNS11643	2-3074
U+73DC.0	fBig5	D2B3
U+73DC.0	fGB	5-4232
U+73DC.0	fR/S	 96.6
U+73DC.0	fTotalStrokes	 10
U+73DC.0	fCangjie	MGTQ
U+73DC.0	fFourCorner	18150
U+73DC.0	fCantonese	joeng4
U+73DC.0	fMandarin	yang2
U+73DD.0	fUTF8	珝
U+73DD.0	fCNS11643	2-3078
U+73DD.0	fBig5	D2B7
U+73DD.0	fGB	3-3F46
U+73DD.0	fR/S	 96.6
U+73DD.0	fTotalStrokes	 10
U+73DD.0	fCangjie	MGSMM
U+73DD.0	fFourCorner	17120
U+73DD.0	fHakka	  hi3
U+73DD.0	fCantonese	heoi2
U+73DD.0	fMandarin	xu3
U+73DD.0	fEnglish	a precious stone
U+73DE.0	fUTF8	珞
U+73DE.0	fCNS11643	1-5662
U+73DE.0	fBig5	AF5F
U+73DE.0	fGB	0-6773
U+73DE.0	fR/S	 96.6
U+73DE.0	fTotalStrokes	 10
U+73DE.0	fCangjie	MGHER
U+73DE.0	fFourCorner	17164
U+73DE.0	fHakka	 lok8; lok7
U+73DE.0	fCantonese	lok3; lok6
U+73DE.0	fMandarin	luo4; li4
U+73DE.0	fEnglish	jade ornaments for the neck
U+73DF.0	fUTF8	珟
U+73DF.0	fCNS11643	3-3240
U+73DF.0	fGB	3-3F3C
U+73DF.0	fR/S	  96.6
U+73DF.0	fTotalStrokes	 10
U+73DF.0	fCangjie	MGHNN
U+73DF.0	fFourCorner	17110
U+73DF.0	fHakka	 siuk7
U+73DF.0	fCantonese	suk1
U+73DF.0	fMandarin	su4
U+73E0.0	fUTF8	珠
U+73E0.0	fCNS11643	1-5660
U+73E0.0	fBig5	AF5D
U+73E0.0	fGB	0-5669
U+73E0.0	fR/S	 96.6
U+73E0.0	fTotalStrokes	 10
U+73E0.0	fCangjie	MGHJD
U+73E0.0	fFourCorner	15190
U+73E0.0	fHakka	 zu1;  zhu1
U+73E0.0	fCantonese	zyu1
U+73E0.0	fMandarin	zhu1
U+73E0.0	fEnglish	[1] pearl [2] bead; drop [3] pupil of the eye [4] a Chinese family name
U+73E1.0	fUTF8	珡
U+73E1.0	fCNS11643	3-3247
U+73E1.0	fGB	E-3336
U+73E1.0	fR/S	 96.6
U+73E1.0	fTotalStrokes	 10
U+73E1.0	fCangjie	MGO
U+73E1.0	fFourCorner	11800
U+73E1.0	fCantonese	kam4
U+73E1.0	fMandarin	qin2
U+73E2.0	fUTF8	珢
U+73E2.0	fCNS11643	3-323C
U+73E2.0	fGB	3-3F44
U+73E2.0	fR/S	 96.6
U+73E2.0	fTotalStrokes	 10
U+73E2.0	fCangjie	MGAV
U+73E2.0	fFourCorner	17130
U+73E2.0	fHakka	 ngiun2
U+73E2.0	fCantonese	ngan4; an4 (also)
U+73E2.0	fMandarin	yin2; ken4; gen1; gen4
U+73E2.0	fEnglish	precious stone
U+73E3.0	fUTF8	珣
U+73E3.0	fCNS11643	2-3072
U+73E3.0	fBig5	D2B1
U+73E3.0	fGB	3-3F3D
U+73E3.0	fR/S	 96.6
U+73E3.0	fTotalStrokes	 10
U+73E3.0	fCangjie	MGPA
U+73E3.0	fFourCorner	17120
U+73E3.0	fHakka	  sun2
U+73E3.0	fCantonese	seon1
U+73E3.0	fMandarin	xun2; xun4; xuan1
U+73E3.0	fEnglish	the name of a kind of jade
U+73E4.0	fUTF8	珤
U+73E4.0	fCNS11643	3-3246
U+73E4.0	fGB	5-422E
U+73E4.0	fR/S	  96.6
U+73E4.0	fTotalStrokes	 10
U+73E4.0	fCangjie	MGOJU
U+73E4.0	fFourCorner	18174
U+73E4.0	fCantonese	bou2
U+73E4.0	fMandarin	bao3
U+73E4.0	fEnglish	 treasure; jewel precious; rare
U+73E5.0	fUTF8	珥
U+73E5.0	fCNS11643	2-306E
U+73E5.0	fBig5	D2AD
U+73E5.0	fGB	0-676D
U+73E5.0	fR/S	 96.6
U+73E5.0	fTotalStrokes	 10
U+73E5.0	fCangjie	MGSJ
U+73E5.0	fFourCorner	11140
U+73E5.0	fHakka	  ngi3
U+73E5.0	fCantonese	nei6; lei6 (also); ji5 (also); ji6
U+73E5.0	fMandarin	er3; er4; reng2
U+73E5.0	fEnglish	 ear ornament stick; insert
U+73E6.0	fUTF8	珦
U+73E6.0	fCNS11643	3-3241
U+73E6.0	fGB	3-3F39
U+73E6.0	fR/S	 96.6
U+73E6.0	fTotalStrokes	 10
U+73E6.0	fCangjie	MGHBR
U+73E6.0	fFourCorner	17120
U+73E6.0	fHakka	 hiong5
U+73E6.0	fCantonese	hoeng3
U+73E6.0	fMandarin	xiang4; xiang3
U+73E6.0	fEnglish	a kind of jade
U+73E7.0	fUTF8	珧
U+73E7.0	fCNS11643	2-3071
U+73E7.0	fBig5	D2B0
U+73E7.0	fGB	0-6772
U+73E7.0	fR/S	 96.6
U+73E7.0	fTotalStrokes	 10
U+73E7.0	fCangjie	MGLMO
U+73E7.0	fFourCorner	12113
U+73E7.0	fHakka	 rau2; jeu2; jau2
U+73E7.0	fCantonese	jiu4
U+73E7.0	fMandarin	yao2
U+73E7.0	fEnglish	mother-of-pearl
U+73E8.0	fUTF8	珨
U+73E8.0	fCNS11643	2-307C
U+73E8.0	fBig5	D2BB
U+73E8.0	fGB	3-3F3B
U+73E8.0	fR/S	 96.6
U+73E8.0	fTotalStrokes	 10
U+73E8.0	fCangjie	MGOMR
U+73E8.0	fFourCorner	18161
U+73E8.0	fCantonese	hap6; haap6
U+73E8.0	fMandarin	xia2; ya1
U+73E9.0	fUTF8	珩
U+73E9.0	fCNS11643	2-3073
U+73E9.0	fBig5	D2B2
U+73E9.0	fGB	0-6771
U+73E9.0	fR/S	 96.6
U+73E9.0	fTotalStrokes	 10
U+73E9.0	fCangjie	MGHON
U+73E9.0	fFourCorner	11121
U+73E9.0	fHakka	  hang2; hong2
U+73E9.0	fCantonese	hang4
U+73E9.0	fMandarin	heng2
U+73E9.0	fEnglish	the top gem of the pendants from a girdle
U+73EA.0	fUTF8	珪
U+73EA.0	fCNS11643	1-5661
U+73EA.0	fBig5	AF5E
U+73EA.0	fGB	5-4229
U+73EA.0	fR/S	 96.6
U+73EA.0	fTotalStrokes	 10
U+73EA.0	fCangjie	MGGG
U+73EA.0	fFourCorner	14114
U+73EA.0	fHakka	 gui1; kwui1
U+73EA.0	fCantonese	gwai1
U+73EA.0	fMandarin	gui1
U+73EA.0	fEnglish	a jade table worn by feudal princes as a symbol of authority
U+73EB.0	fUTF8	珫
U+73EB.0	fCNS11643	2-2C31
U+73EB.0	fBig5	CFCF
U+73EB.0	fGB	3-3F41
U+73EB.0	fR/S	 96.6
U+73EB.0	fTotalStrokes	 10
U+73EB.0	fCangjie	MGYIU
U+73EB.0	fFourCorner	10113
U+73EB.0	fCantonese	cung1
U+73EB.0	fMandarin	chong1
U+73EC.0	fUTF8	珬
U+73EC.0	fCNS11643	3-323D
U+73EC.0	fGB	3-3F32
U+73EC.0	fR/S	 96.6
U+73EC.0	fTotalStrokes	 10
U+73EC.0	fCangjie	MGIHM
U+73EC.0	fFourCorner	13150
U+73EC.0	fHakka	 sut7
U+73EC.0	fCantonese	seot1
U+73EC.0	fMandarin	xu4
U+73EC.0	fEnglish	another name for the cone shell
U+73ED.0	fUTF8	班
U+73ED.0	fCNS11643	1-565D
U+73ED.0	fBig5	AF5A
U+73ED.0	fGB	0-3060
U+73ED.0	fR/S	 96.6
U+73ED.0	fTotalStrokes	 10
U+73ED.0	fCangjie	MGILG
U+73ED.0	fFourCorner	11114
U+73ED.0	fHakka	  ban1
U+73ED.0	fCantonese	baan1
U+73ED.0	fMandarin	ban1
U+73ED.0	fEnglish	[1] class; group; company; set [2] grade; seat; position [3] squad (soldiers) [4] distribute [5] return [6] same as $ [7] of equal rank; same generation, etc. [8] shift; duty [9] scheduled runs (bus, train, etc.)
U+73EE.0	fUTF8	珮
U+73EE.0	fCNS11643	1-565F
U+73EE.0	fBig5	AF5C
U+73EE.0	fGB	5-4231
U+73EE.0	fR/S	 96.6
U+73EE.0	fTotalStrokes	 10
U+73EE.0	fCangjie	MGHNB
U+73EE.0	fFourCorner	17110
U+73EE.0	fHakka	 pui5; poi3; pi3; pi5
U+73EE.0	fCantonese	pui3
U+73EE.0	fMandarin	pei4
U+73EE.0	fEnglish	jade ornament; jade pendants
U+73EF.0	fUTF8	珯
U+73EF.0	fCNS11643	3-6564
U+73EF.0	fGB	E-3337
U+73EF.0	fR/S	 96.6
U+73EF.0	fTotalStrokes	 10
U+73EF.0	fCangjie	MGJKP
U+73EF.0	fFourCorner	1411
U+73EF.0	fCantonese	lou5
U+73F0.0	fUTF8	珰
U+73F0.0	fGB	8-7B4E
U+73F0.0	fR/S	 96.6
U+73F0.0	fTotalStrokes	 10
U+73F0.0	fCangjie	MGFSM
U+73F0.0	fCantonese	dong1
U+73F0.0	fMandarin	dang1
U+73F0.0	fEnglish	pendant ornaments, earrings
U+73F1.0	fUTF8	珱
U+73F1.0	fGB	E-3338
U+73F1.0	fR/S	 96.6
U+73F1.0	fTotalStrokes	 10
U+73F1.0	fCangjie	MGFV
U+73F1.0	fCantonese	jing1
U+73F1.0	fMandarin	ying1
U+73F1.0	fEnglish	 necklace made of precious stones
U+73F2.0	fUTF8	珲
U+73F2.0	fGB	0-6775
U+73F2.0	fR/S	 96.6
U+73F2.0	fTotalStrokes	 10
U+73F2.0	fCangjie	MGBKQ
U+73F2.0	fHakka	 fun2
U+73F2.0	fCantonese	wan4; fai1
U+73F2.0	fMandarin	hun2; hui1
U+73F2.0	fEnglish	simplified form of &#x743F;
U+73F3.0	fUTF8	珳
U+73F3.0	fCNS11643	3-373D
U+73F3.0	fGB	E-3339
U+73F3.0	fR/S	 96.7
U+73F3.0	fTotalStrokes	 11
U+73F3.0	fCangjie	MGYKH
U+73F3.0	fFourCorner	12140
U+73F3.0	fCantonese	man4
U+73F3.0	fMandarin	wen2
U+73F4.0	fUTF8	珴
U+73F4.0	fCNS11643	2-373D
U+73F4.0	fBig5	D678
U+73F4.0	fGB	5-423A
U+73F4.0	fR/S	 96.7
U+73F4.0	fTotalStrokes	 11
U+73F4.0	fCangjie	MGHQI
U+73F4.0	fFourCorner	13150
U+73F4.0	fCantonese	ngo4
U+73F4.0	fMandarin	e2
U+73F5.0	fUTF8	珵
U+73F5.0	fCNS11643	2-3732
U+73F5.0	fBig5	D66D
U+73F5.0	fGB	3-3F4A
U+73F5.0	fR/S	 96.7
U+73F5.0	fTotalStrokes	 11
U+73F5.0	fCangjie	MGRHG
U+73F5.0	fFourCorner	16114
U+73F5.0	fHakka	 chin2; cihn2
U+73F5.0	fCantonese	cing4
U+73F5.0	fMandarin	cheng2; ting3
U+73F5.0	fEnglish	brilliant stone worn at the girdle
U+73F6.0	fUTF8	珶
U+73F6.0	fCNS11643	2-3730
U+73F6.0	fBig5	D66B
U+73F6.0	fGB	3-3F50
U+73F6.0	fR/S	 96.7
U+73F6.0	fTotalStrokes	 11
U+73F6.0	fCangjie	MGCNH
U+73F6.0	fFourCorner	18127
U+73F6.0	fHakka	 ti5; ti6
U+73F6.0	fCantonese	dai6; tai4
U+73F6.0	fMandarin	ti2; di4
U+73F6.0	fEnglish	a piece of white jade, formerly worn on the girdle
U+73F7.0	fUTF8	珷
U+73F7.0	fCNS11643	3-3D38
U+73F7.0	fGB	5-4241
U+73F7.0	fR/S	  96.8
U+73F7.0	fTotalStrokes	 12
U+73F7.0	fCangjie	MGMPM
U+73F7.0	fFourCorner	13140
U+73F7.0	fHakka	 wu3; vu3
U+73F7.0	fCantonese	mou5
U+73F7.0	fMandarin	wu3
U+73F7.0	fEnglish	1/2 pr st
U+73F8.0	fUTF8	珸
U+73F8.0	fCNS11643	2-3731
U+73F8.0	fBig5	D66C
U+73F8.0	fGB	3-3F49
U+73F8.0	fR/S	 96.7
U+73F8.0	fTotalStrokes	 11
U+73F8.0	fCangjie	MGMMR
U+73F8.0	fFourCorner	11161
U+73F8.0	fCantonese	ng4
U+73F8.0	fMandarin	wu2
U+73F9.0	fUTF8	珹
U+73F9.0	fCNS11643	3-323F
U+73F9.0	fGB	3-3F34
U+73F9.0	fR/S	 96.7; 96.6
U+73F9.0	fTotalStrokes	 10; 11
U+73F9.0	fCangjie	MGIHS
U+73F9.0	fFourCorner	13150
U+73F9.0	fCantonese	sing4
U+73F9.0	fMandarin	cheng2
U+73F9.0	fEnglish	 type of jade pearl
U+73FA.0	fUTF8	珺
U+73FA.0	fCNS11643	2-3738
U+73FA.0	fBig5	D673
U+73FA.0	fGB	3-3F51
U+73FA.0	fR/S	 96.7
U+73FA.0	fTotalStrokes	 11
U+73FA.0	fCangjie	MGSKR
U+73FA.0	fFourCorner	17167
U+73FA.0	fHakka	 kiun3
U+73FA.0	fCantonese	gwan6; kwan2
U+73FA.0	fMandarin	jun4
U+73FA.0	fEnglish	beautiful jade
U+73FB.0	fUTF8	珻
U+73FB.0	fCNS11643	3-3740
U+73FB.0	fGB	E-333A
U+73FB.0	fR/S	 96.7
U+73FB.0	fTotalStrokes	 11
U+73FB.0	fCangjie	MGOWY
U+73FB.0	fFourCorner	18150
U+73FB.0	fHakka	 moe2
U+73FB.0	fCantonese	mui4
U+73FB.0	fMandarin	mei2
U+73FC.0	fUTF8	珼
U+73FC.0	fCNS11643	2-3739
U+73FC.0	fBig5	D674
U+73FC.0	fGB	5-4177
U+73FC.0	fR/S	 96.7
U+73FC.0	fTotalStrokes	 11
U+73FC.0	fCangjie	MGBUC
U+73FC.0	fFourCorner	16180
U+73FC.0	fHakka	 bui5; pi5; poi5; bi5
U+73FC.0	fCantonese	bui3
U+73FC.0	fMandarin	bei4
U+73FC.0	fEnglish	tortoiseshell ornaments
U+73FD.0	fUTF8	珽
U+73FD.0	fCNS11643	2-3735
U+73FD.0	fBig5	D670
U+73FD.0	fGB	3-3F38
U+73FD.0	fR/S	 96.7
U+73FD.0	fTotalStrokes	 11
U+73FD.0	fCangjie	MGNKG
U+73FD.0	fFourCorner	12141
U+73FD.0	fHakka	  tin3
U+73FD.0	fCantonese	ting5; ting2 (also)
U+73FD.0	fMandarin	ting3; ting1
U+73FD.0	fEnglish	jade tablet
U+73FE.0	fUTF8	現
U+73FE.0	fCNS11643	1-5C22
U+73FE.0	fBig5	B27B
U+73FE.0	fGB	1-4F56
U+73FE.0	fR/S	 96.7
U+73FE.0	fTotalStrokes	 11
U+73FE.0	fCangjie	MGBUU
U+73FE.0	fFourCorner	16110
U+73FE.0	fHakka	 hien5; hen3; hen5; hian6; hien6
U+73FE.0	fCantonese	jin6
U+73FE.0	fMandarin	xian4
U+73FE.0	fEnglish	[1] appear; emerge; reveal; manifest; become visible [2] current; now; present; modern [3] in time of need; extempore [4] cash [5] ready; available [6] actual
U+73FF.0	fUTF8	珿
U+73FF.0	fCNS11643	2-373A
U+73FF.0	fBig5	D675
U+73FF.0	fGB	3-3F4B
U+73FF.0	fR/S	 96.7
U+73FF.0	fTotalStrokes	 11
U+73FF.0	fCangjie	MGRYO
U+73FF.0	fFourCorner	16181
U+73FF.0	fCantonese	cuk1
U+73FF.0	fMandarin	chuo4
U+7400.0	fUTF8	琀
U+7400.0	fCNS11643	2-3737
U+7400.0	fBig5	D672
U+7400.0	fGB	3-3F4F
U+7400.0	fR/S	 96.7
U+7400.0	fTotalStrokes	 11
U+7400.0	fCangjie	MGOIR
U+7400.0	fFourCorner	18162
U+7400.0	fHakka	 ham2; ham5
U+7400.0	fCantonese	ham3; ham6
U+7400.0	fMandarin	han4; han2
U+7400.0	fEnglish	gems of pearls formerly put into the mouth of a corpse
U+7401.0	fUTF8	琁
U+7401.0	fCNS11643	2-3734
U+7401.0	fBig5	D66F
U+7401.0	fGB	5-423B
U+7401.0	fR/S	 96.7
U+7401.0	fTotalStrokes	 11
U+7401.0	fCangjie	MGONO
U+7401.0	fFourCorner	18181
U+7401.0	fCantonese	syun4
U+7401.0	fMandarin	xuan2; qiong2
U+7401.0	fEnglish	 beautiful jade star
U+7402.0	fUTF8	琂
U+7402.0	fCNS11643	3-373C
U+7402.0	fGB	5-423F
U+7402.0	fR/S	  96.7
U+7402.0	fTotalStrokes	 11
U+7402.0	fCangjie	MGYMR
U+7402.0	fFourCorner	10160
U+7402.0	fCantonese	jin4
U+7402.0	fMandarin	yan2
U+7403.0	fUTF8	球
U+7403.0	fCNS11643	1-5B7E
U+7403.0	fBig5	B279
U+7403.0	fGB	0-4772
U+7403.0	fR/S	 96.7
U+7403.0	fTotalStrokes	 11
U+7403.0	fCangjie	MGIJE
U+7403.0	fFourCorner	13132
U+7403.0	fHakka	  kiu2; k'ieu2; kiu3
U+7403.0	fCantonese	kau4
U+7403.0	fMandarin	qiu2
U+7403.0	fEnglish	[1] ball; sphere; globe; round [2] the globe; the earth
U+7404.0	fUTF8	琄
U+7404.0	fCNS11643	2-3733
U+7404.0	fBig5	D66E
U+7404.0	fGB	5-4239
U+7404.0	fR/S	 96.7
U+7404.0	fTotalStrokes	 11
U+7404.0	fCangjie	MGRB
U+7404.0	fFourCorner	16127
U+7404.0	fHakka	 gien1;  hien2
U+7404.0	fCantonese	jyun5; jyun6; gyun1
U+7404.0	fMandarin	quan3; jiong3; juan1; xuan4
U+7405.0	fUTF8	琅
U+7405.0	fCNS11643	1-5B7C
U+7405.0	fBig5	B277
U+7405.0	fGB	0-4045
U+7405.0	fR/S	 96.7
U+7405.0	fTotalStrokes	 11
U+7405.0	fCangjie	MGIAV
U+7405.0	fFourCorner	13132
U+7405.0	fHakka	 long2
U+7405.0	fCantonese	long4
U+7405.0	fMandarin	lang2; lang4
U+7405.0	fEnglish	[1] a kind of stone resembling jade [2] clean and white; pure; spotless [3] a Chinese family name
U+7406.0	fUTF8	理
U+7406.0	fCNS11643	1-5C21
U+7406.0	fBig5	B27A
U+7406.0	fGB	0-406D
U+7406.0	fR/S	 96.7
U+7406.0	fTotalStrokes	 11
U+7406.0	fCangjie	MGWG
U+7406.0	fFourCorner	16114
U+7406.0	fHakka	  li1; li3
U+7406.0	fCantonese	lei5; lei1 (also)
U+7406.0	fMandarin	li3
U+7406.0	fEnglish	[1] reason; logic; cause; truth; right; righteousness [2] law; principles; doctrine; theory; science [3] arrange [4] administer; govern; manage; operate; regulate; run [5] reply; answer; respond [6] texture; grain (in wood, skin, etc.) [7] name of a religious sect [8] a Chinese family name
U+7406.1	fUTF8	理
U+7406.1	fCNS11643	1-5C21
U+7406.1	fBig5	B27A
U+7406.1	fGB	0-406D
U+7406.1	fR/S	 96.7
U+7406.1	fTotalStrokes	 11
U+7406.1	fCangjie	MGWG
U+7406.1	fFourCorner	16114
U+7406.1	fMandarin	li5
U+7407.0	fUTF8	琇
U+7407.0	fCNS11643	2-3736
U+7407.0	fBig5	D671
U+7407.0	fGB	3-3F4C
U+7407.0	fR/S	 96.7
U+7407.0	fTotalStrokes	 11
U+7407.0	fCangjie	MGHDS
U+7407.0	fFourCorner	12127
U+7407.0	fHakka	  siu5
U+7407.0	fCantonese	sau3
U+7407.0	fMandarin	xiu4; you3
U+7407.0	fEnglish	coarse variety of jasper or jade
U+7408.0	fUTF8	琈
U+7408.0	fCNS11643	2-373E
U+7408.0	fBig5	D679
U+7408.0	fGB	3-3F4E
U+7408.0	fR/S	 96.7
U+7408.0	fTotalStrokes	 11
U+7408.0	fCangjie	MGBND
U+7408.0	fFourCorner	12147
U+7408.0	fCantonese	fau4
U+7408.0	fMandarin	fu2; fou2; fu1
U+7409.0	fUTF8	琉
U+7409.0	fCNS11643	1-565E
U+7409.0	fBig5	AF5B
U+7409.0	fGB	0-4170
U+7409.0	fR/S	 96.7; 96.6
U+7409.0	fTotalStrokes	 10
U+7409.0	fCangjie	MGYIU
U+7409.0	fFourCorner	10113
U+7409.0	fHakka	  liu2; lieu2
U+7409.0	fCantonese	lau4
U+7409.0	fMandarin	liu2
U+7409.0	fEnglish	[1] sparkling stone; glossy and bright stone [2] glazed
U+740A.0	fUTF8	琊
U+740A.0	fCNS11643	1-5B7D
U+740A.0	fBig5	B278
U+740A.0	fGB	0-6770
U+740A.0	fR/S	 96.7; 96.8
U+740A.0	fTotalStrokes	 11
U+740A.0	fCangjie	MGMHL
U+740A.0	fFourCorner	17127
U+740A.0	fHakka	 ja2; ra2
U+740A.0	fCantonese	ngaa4; je4 (also); aa4 (also)
U+740A.0	fMandarin	ye2; ya2
U+740A.0	fEnglish	[1] used in the ancient name of the eastern portion of Shantung [2] a mountain in Eastern Shantung
U+740B.0	fUTF8	琋
U+740B.0	fCNS11643	2-373C
U+740B.0	fBig5	D677
U+740B.0	fGB	5-423C
U+740B.0	fR/S	 96.7
U+740B.0	fTotalStrokes	 11
U+740B.0	fCangjie	MGKKB
U+740B.0	fFourCorner	14127
U+740B.0	fCantonese	hei1
U+740B.0	fMandarin	xi1
U+740C.0	fUTF8	琌
U+740C.0	fCNS11643	2-373B
U+740C.0	fBig5	D676
U+740C.0	fGB	E-333B
U+740C.0	fR/S	 96.7
U+740C.0	fTotalStrokes	 11
U+740C.0	fCangjie	MGUON
U+740C.0	fFourCorner	12127
U+740C.0	fCantonese	ling4
U+740C.0	fMandarin	ling2
U+740D.0	fUTF8	琍
U+740D.0	fCNS11643	1-5C23
U+740D.0	fBig5	B27C
U+740D.0	fGB	E-333C
U+740D.0	fR/S	 96.7
U+740D.0	fTotalStrokes	 11
U+740D.0	fCangjie	MGHDN
U+740D.0	fFourCorner	12100
U+740D.0	fCantonese	lei4; lei1 (also)
U+740D.0	fMandarin	li4; li2
U+740E.0	fUTF8	琎
U+740E.0	fGB	8-7B52
U+740E.0	fR/S	 96.7
U+740E.0	fTotalStrokes	 11
U+740E.0	fCangjie	MGYTT
U+740E.0	fCantonese	zeon3; zeon1 (also)
U+740E.0	fMandarin	jin4; jin1
U+740F.0	fUTF8	琏
U+740F.0	fGB	0-6776
U+740F.0	fR/S	 96.7
U+740F.0	fTotalStrokes	 11
U+740F.0	fCangjie	MGYKQ
U+740F.0	fHakka	 lien3
U+740F.0	fCantonese	lin5; lin4
U+740F.0	fMandarin	lian2; lian3
U+740F.0	fEnglish	simplified form of &#x7489;
U+7410.0	fUTF8	琐
U+7410.0	fGB	0-4B76
U+7410.0	fR/S	 96.7
U+7410.0	fTotalStrokes	 11
U+7410.0	fCangjie	MGFBO
U+7410.0	fHakka	 so3
U+7410.0	fCantonese	so2
U+7410.0	fMandarin	suo3
U+7410.0	fEnglish	simplified form of &#x7463;
U+7411.0	fUTF8	琑
U+7411.0	fGB	E-333D
U+7411.0	fR/S	 96.7
U+7411.0	fTotalStrokes	 11
U+7411.0	fCangjie	MGFB
U+7411.0	fFourCorner	1912
U+7411.0	fCantonese	ciu3
U+7411.0	fMandarin	suo3
U+7412.0	fUTF8	琒
U+7412.0	fCNS11643	3-6565
U+7412.0	fGB	E-333E
U+7412.0	fR/S	 96.7
U+7412.0	fTotalStrokes	 11
U+7412.0	fCangjie	MGHEJ
U+7412.0	fFourCorner	1715
U+7412.0	fCantonese	fung1
U+7413.0	fUTF8	琓
U+7413.0	fCNS11643	F-3657
U+7413.0	fGB	E-333F
U+7413.0	fR/S	 96.7
U+7413.0	fTotalStrokes	 11
U+7413.0	fCangjie	MGJMU
U+7413.0	fFourCorner	1311
U+7413.0	fCantonese	jyun4
U+7413.0	fMandarin	wan2
U+7414.0	fUTF8	琔
U+7414.0	fCNS11643	3-3D36
U+7414.0	fGB	5-424B
U+7414.0	fR/S	 96.8
U+7414.0	fTotalStrokes	 12
U+7414.0	fCangjie	MGJMO
U+7414.0	fFourCorner	13180
U+7414.0	fCantonese	din6
U+7414.0	fMandarin	dian4
U+7415.0	fUTF8	琕
U+7415.0	fCNS11643	3-3D3A
U+7415.0	fGB	5-4247
U+7415.0	fR/S	  96.8
U+7415.0	fTotalStrokes	 12
U+7415.0	fCangjie	MGHWJ
U+7415.0	fFourCorner	16140
U+7415.0	fCantonese	pan4
U+7415.0	fMandarin	pian2; pin2; bing3
U+7415.0	fEnglish	 type of pearl scabbard
U+7416.0	fUTF8	琖
U+7416.0	fCNS11643	2-3E24
U+7416.0	fBig5	DA7E
U+7416.0	fGB	E-3340
U+7416.0	fR/S	 96.8
U+7416.0	fTotalStrokes	 12
U+7416.0	fCangjie	MGII
U+7416.0	fFourCorner	13153
U+7416.0	fHakka	  zan3
U+7416.0	fCantonese	zaan2; zaan1
U+7416.0	fMandarin	zhan3
U+7416.0	fEnglish	a carved jade wine cup or chalice
U+7417.0	fUTF8	琗
U+7417.0	fCNS11643	3-3D2F
U+7417.0	fGB	5-4249
U+7417.0	fR/S	  96.8
U+7417.0	fTotalStrokes	 12
U+7417.0	fCangjie	MGYOJ
U+7417.0	fFourCorner	10140
U+7417.0	fCantonese	ceoi3
U+7417.0	fMandarin	cui4
U+7418.0	fUTF8	琘
U+7418.0	fCNS11643	3-3D3B
U+7418.0	fGB	5-4248
U+7418.0	fR/S	  96.8
U+7418.0	fTotalStrokes	 12
U+7418.0	fCangjie	MGHPA
U+7418.0	fFourCorner	12160
U+7418.0	fCantonese	man4
U+7418.0	fMandarin	min2
U+7419.0	fUTF8	琙
U+7419.0	fCNS11643	3-3D32
U+7419.0	fGB	3-3F53
U+7419.0	fR/S	 96.8
U+7419.0	fTotalStrokes	 12
U+7419.0	fCangjie	MGIRM
U+7419.0	fFourCorner	13150
U+7419.0	fCantonese	wik6
U+7419.0	fMandarin	yu4
U+741A.0	fUTF8	琚
U+741A.0	fCNS11643	2-3E25
U+741A.0	fBig5	DAA1
U+741A.0	fGB	0-6822
U+741A.0	fR/S	 96.8
U+741A.0	fTotalStrokes	 12
U+741A.0	fCangjie	MGSJR
U+741A.0	fFourCorner	17164
U+741A.0	fHakka	 gi1
U+741A.0	fCantonese	geoi1
U+741A.0	fMandarin	ju1
U+741A.0	fEnglish	a germ worn at the girdle-pendant
U+741B.0	fUTF8	琛
U+741B.0	fCNS11643	1-6066
U+741B.0	fBig5	B560
U+741B.0	fGB	0-6821
U+741B.0	fR/S	 96.8
U+741B.0	fTotalStrokes	 12
U+741B.0	fCangjie	MGBCD
U+741B.0	fFourCorner	17194
U+741B.0	fHakka	 chim1; cihm1; cim1
U+741B.0	fCantonese	sam1; cam1
U+741B.0	fMandarin	chen1
U+741B.0	fEnglish	treasure; valuables; jewelry
U+741C.0	fUTF8	琜
U+741C.0	fCNS11643	3-3D39
U+741C.0	fGB	5-4236
U+741C.0	fR/S	 96.8
U+741C.0	fTotalStrokes	 12
U+741C.0	fCangjie	MGDOO
U+741C.0	fFourCorner	14190
U+741C.0	fCantonese	loi4
U+741C.0	fMandarin	lai2
U+741D.0	fUTF8	琝
U+741D.0	fCNS11643	2-3E2B
U+741D.0	fBig5	DAA7
U+741D.0	fGB	5-4245
U+741D.0	fR/S	 96.8
U+741D.0	fTotalStrokes	 12
U+741D.0	fCangjie	MGAYK
U+741D.0	fFourCorner	16141
U+741D.0	fCantonese	man4
U+741D.0	fMandarin	wen2; min2
U+741E.0	fUTF8	琞
U+741E.0	fCNS11643	3-426D
U+741E.0	fGB	E-3341
U+741E.0	fR/S	 96.9
U+741E.0	fTotalStrokes	 13
U+741E.0	fCangjie	ABMGI
U+741E.0	fFourCorner	67100
U+741E.0	fCantonese	sing3
U+741E.0	fMandarin	sheng4; wang4
U+741F.0	fUTF8	琟
U+741F.0	fCNS11643	3-3D35
U+741F.0	fGB	3-3F59
U+741F.0	fR/S	 96.8
U+741F.0	fTotalStrokes	 12
U+741F.0	fCangjie	MGOG
U+741F.0	fFourCorner	10110
U+741F.0	fCantonese	wai4
U+741F.0	fMandarin	wei2
U+7420.0	fUTF8	琠
U+7420.0	fCNS11643	2-3E2D
U+7420.0	fBig5	DAA9
U+7420.0	fGB	5-4246
U+7420.0	fR/S	 96.8
U+7420.0	fTotalStrokes	 12
U+7420.0	fCangjie	MGTBC
U+7420.0	fFourCorner	15181
U+7420.0	fHakka	 dien3; tien3; tien5
U+7420.0	fCantonese	tin2; tin3; din6
U+7420.0	fMandarin	dian3; tian3
U+7420.0	fEnglish	 gem used as ear plug jade earring
U+7421.0	fUTF8	琡
U+7421.0	fCNS11643	2-3E26
U+7421.0	fBig5	DAA2
U+7421.0	fGB	3-3F56
U+7421.0	fR/S	 96.8
U+7421.0	fTotalStrokes	 12
U+7421.0	fCangjie	MGYFE
U+7421.0	fFourCorner	17140
U+7421.0	fCantonese	cuk1; zyut3
U+7421.0	fMandarin	chu4; zhou1; zhu4
U+7422.0	fUTF8	琢
U+7422.0	fCNS11643	1-6060
U+7422.0	fBig5	B55A
U+7422.0	fGB	0-5741
U+7422.0	fR/S	 96.8; 96.7
U+7422.0	fTotalStrokes	 12
U+7422.0	fCangjie	MGMSO
U+7422.0	fFourCorner	11132
U+7422.0	fHakka	  dok7; dok8; duk8; duk7
U+7422.0	fCantonese	doek3
U+7422.0	fMandarin	zhuo2; zuo2
U+7422.0	fEnglish	[1] polish jade; cut jade; chisel jade [2] improve (literary works); polish; refine
U+7423.0	fUTF8	琣
U+7423.0	fCNS11643	2-3E2A
U+7423.0	fBig5	DAA6
U+7423.0	fGB	5-424A
U+7423.0	fR/S	 96.8
U+7423.0	fTotalStrokes	 12
U+7423.0	fCangjie	MGYTR
U+7423.0	fFourCorner	10161
U+7423.0	fCantonese	bung2
U+7423.0	fMandarin	pei3; beng3
U+7424.0	fUTF8	琤
U+7424.0	fCNS11643	2-3E29
U+7424.0	fBig5	DAA5
U+7424.0	fGB	3-3F3E
U+7424.0	fR/S	 96.8
U+7424.0	fTotalStrokes	 12
U+7424.0	fCangjie	MGBSD
U+7424.0	fFourCorner	12157
U+7424.0	fHakka	  zen1
U+7424.0	fCantonese	zang1; zaang1 (also); caang1; saang1
U+7424.0	fMandarin	cheng1
U+7424.0	fEnglish	a tinkling sound, descriptive of jangling, twanging, or gurgling (flowing water)
U+7425.0	fUTF8	琥
U+7425.0	fCNS11643	1-6061
U+7425.0	fBig5	B55B
U+7425.0	fGB	0-677A
U+7425.0	fR/S	 96.8
U+7425.0	fTotalStrokes	 12
U+7425.0	fCangjie	MGYPU
U+7425.0	fFourCorner	11117
U+7425.0	fHakka	  fu3
U+7425.0	fCantonese	fu2
U+7425.0	fMandarin	hu3
U+7425.0	fEnglish	[1] jewel or ornament in the shape of a tiger [2] amber
U+7426.0	fUTF8	琦
U+7426.0	fCNS11643	1-6067
U+7426.0	fBig5	B561
U+7426.0	fGB	0-6779
U+7426.0	fR/S	 96.8
U+7426.0	fTotalStrokes	 12
U+7426.0	fCangjie	MGKMR
U+7426.0	fFourCorner	14121
U+7426.0	fHakka	  ki2; ki3
U+7426.0	fCantonese	kei4; kei2 (also)
U+7426.0	fMandarin	qi2
U+7426.0	fEnglish	[1] a kind of jade [2] same as $ [3] outstanding; extraordinary; distinguished; admirable
U+7427.0	fUTF8	琧
U+7427.0	fCNS11643	3-4264
U+7427.0	fGB	5-4272
U+7427.0	fR/S	  96.8
U+7427.0	fTotalStrokes	 13
U+7427.0	fCangjie	MMMGI
U+7427.0	fFourCorner	10100
U+7427.0	fCantonese	ok3; ngok3 (also)
U+7427.0	fMandarin	e4
U+7428.0	fUTF8	琨
U+7428.0	fCNS11643	1-6068
U+7428.0	fBig5	B562
U+7428.0	fGB	0-677B
U+7428.0	fR/S	 96.8
U+7428.0	fTotalStrokes	 12
U+7428.0	fCangjie	MGAPP
U+7428.0	fFourCorner	16111
U+7428.0	fHakka	 kun1; kwun1
U+7428.0	fCantonese	gwan1 (also); kwan1
U+7428.0	fMandarin	kun1
U+7428.0	fEnglish	beautiful jade; precious stones
U+7429.0	fUTF8	琩
U+7429.0	fCNS11643	2-3E2C
U+7429.0	fBig5	DAA8
U+7429.0	fGB	5-4244
U+7429.0	fR/S	 96.8
U+7429.0	fTotalStrokes	 12
U+7429.0	fCangjie	MGAA
U+7429.0	fFourCorner	16160
U+7429.0	fCantonese	coeng1
U+7429.0	fMandarin	chang1
U+742A.0	fUTF8	琪
U+742A.0	fCNS11643	1-605E
U+742A.0	fBig5	B558
U+742A.0	fGB	0-6777
U+742A.0	fR/S	 96.8
U+742A.0	fTotalStrokes	 12
U+742A.0	fCangjie	MGTMC
U+742A.0	fFourCorner	14181
U+742A.0	fHakka	  ki2; ki3
U+742A.0	fCantonese	kei4
U+742A.0	fMandarin	qi2
U+742A.0	fEnglish	a piece of jade; a jade-like precious stone; white gem
U+742B.0	fUTF8	琫
U+742B.0	fCNS11643	2-3E23
U+742B.0	fBig5	DA7D
U+742B.0	fGB	3-3F52
U+742B.0	fR/S	 96.8
U+742B.0	fTotalStrokes	 12
U+742B.0	fCangjie	MGQKQ
U+742B.0	fFourCorner	15153
U+742B.0	fHakka	 fung3;  bung3
U+742B.0	fCantonese	bung2; fung2; pung2
U+742B.0	fMandarin	beng3
U+742B.0	fEnglish	ornament
U+742C.0	fUTF8	琬
U+742C.0	fCNS11643	2-3E21
U+742C.0	fBig5	DA7B
U+742C.0	fGB	0-677E
U+742C.0	fR/S	 96.8
U+742C.0	fTotalStrokes	 12
U+742C.0	fCangjie	MGJNU
U+742C.0	fFourCorner	13112
U+742C.0	fHakka	 won3; wan3; van3
U+742C.0	fCantonese	jyun2
U+742C.0	fMandarin	wan3; wan4
U+742C.0	fEnglish	[1] the virtue of a gentleman [2] a kind of jade tablet slightly tapering at the top
U+742D.0	fUTF8	琭
U+742D.0	fCNS11643	2-3E27
U+742D.0	fBig5	DAA3
U+742D.0	fGB	3-3F5C
U+742D.0	fR/S	 96.8
U+742D.0	fTotalStrokes	 12
U+742D.0	fCangjie	MGVNE
U+742D.0	fFourCorner	17132
U+742D.0	fCantonese	luk6
U+742D.0	fMandarin	lu4
U+742D.0	fEnglish	jade like stone
U+742E.0	fUTF8	琮
U+742E.0	fCNS11643	2-3D7E
U+742E.0	fBig5	DA7A
U+742E.0	fGB	0-677D
U+742E.0	fR/S	 96.8
U+742E.0	fTotalStrokes	 12
U+742E.0	fCangjie	MGJMF
U+742E.0	fFourCorner	13191
U+742E.0	fHakka	 zung1;  cung1
U+742E.0	fCantonese	cung4; cung6
U+742E.0	fMandarin	cong2; zong4
U+742E.0	fEnglish	an octagonal piece of jade with hole in the middle
U+742F.0	fUTF8	琯
U+742F.0	fCNS11643	1-6065
U+742F.0	fBig5	B55F
U+742F.0	fGB	3-3F5A
U+742F.0	fR/S	 96.8
U+742F.0	fTotalStrokes	 12
U+742F.0	fCangjie	MGJRR
U+742F.0	fFourCorner	13177
U+742F.0	fHakka	 gon3; kwan3; gwon3
U+742F.0	fCantonese	gun2; gwun2 (also)
U+742F.0	fMandarin	guan3; gun3
U+742F.0	fEnglish	[1] a jade tube used as an instrument [2] polish precious metals or stones
U+7430.0	fUTF8	琰
U+7430.0	fCNS11643	2-3E22
U+7430.0	fBig5	DA7C
U+7430.0	fGB	0-677C
U+7430.0	fR/S	 96.8
U+7430.0	fTotalStrokes	 12
U+7430.0	fCangjie	MGFF
U+7430.0	fFourCorner	19189
U+7430.0	fHakka	  jam3; ram3
U+7430.0	fCantonese	jim5
U+7430.0	fMandarin	yan3
U+7430.0	fEnglish	 jewel; gem glitter of gems
U+7431.0	fUTF8	琱
U+7431.0	fCNS11643	2-3E28
U+7431.0	fBig5	DAA4
U+7431.0	fGB	E-3342
U+7431.0	fR/S	 96.8
U+7431.0	fTotalStrokes	 12
U+7431.0	fCangjie	MGBGR
U+7431.0	fFourCorner	17120
U+7431.0	fHakka	  diau1
U+7431.0	fCantonese	diu1
U+7431.0	fMandarin	diao1
U+7431.0	fEnglish	 engrave; inlay; carve exhaust
U+7432.0	fUTF8	琲
U+7432.0	fCNS11643	2-3E2E
U+7432.0	fBig5	DAAA
U+7432.0	fGB	3-3F55
U+7432.0	fR/S	 96.8
U+7432.0	fTotalStrokes	 12
U+7432.0	fCangjie	MGLMY
U+7432.0	fFourCorner	11111
U+7432.0	fHakka	 pi5; pui6; pui5
U+7432.0	fCantonese	pui3; bei3; pui5
U+7432.0	fMandarin	bei4
U+7432.0	fEnglish	necklace
U+7433.0	fUTF8	琳
U+7433.0	fCNS11643	1-605F
U+7433.0	fBig5	B559
U+7433.0	fGB	0-4155
U+7433.0	fR/S	 96.8
U+7433.0	fTotalStrokes	 12
U+7433.0	fCangjie	MGDD
U+7433.0	fFourCorner	14190
U+7433.0	fHakka	 lim2; lim3
U+7433.0	fCantonese	lam4
U+7433.0	fMandarin	lin2
U+7433.0	fEnglish	a fine piece of jade; beautiful jade; gem
U+7434.0	fUTF8	琴
U+7434.0	fCNS11643	1-6064
U+7434.0	fBig5	B55E
U+7434.0	fGB	0-4759
U+7434.0	fR/S	 96.8
U+7434.0	fTotalStrokes	 12
U+7434.0	fCangjie	MGOIN
U+7434.0	fFourCorner	11207
U+7434.0	fHakka	  kim2; kim3
U+7434.0	fCantonese	kam4; kam2 (also)
U+7434.0	fMandarin	qin2
U+7434.0	fEnglish	[1] chin, a Chinese fretted instrument with five or seven strings somewhat similar to the zither [2] a musical instrument, especially stringed, as the piano, violin [3] a Chinese family name
U+7435.0	fUTF8	琵
U+7435.0	fCNS11643	1-6062
U+7435.0	fBig5	B55C
U+7435.0	fGB	0-457D
U+7435.0	fR/S	 96.8
U+7435.0	fTotalStrokes	 12
U+7435.0	fCangjie	MGPP
U+7435.0	fFourCorner	11711
U+7435.0	fHakka	  pi2; pi3
U+7435.0	fCantonese	pei4
U+7435.0	fMandarin	pi2
U+7435.0	fEnglish	a four-stringed guitar-like instrument; balloon-guitar
U+7436.0	fUTF8	琶
U+7436.0	fCNS11643	1-6063
U+7436.0	fBig5	B55D
U+7436.0	fGB	0-4543
U+7436.0	fR/S	 96.8
U+7436.0	fTotalStrokes	 12
U+7436.0	fCangjie	MGAU
U+7436.0	fFourCorner	11717
U+7436.0	fHakka	  pa2; pa3
U+7436.0	fCantonese	paa4; paa2 (also)
U+7436.0	fMandarin	pa2; ba5; pa5
U+7436.0	fEnglish	a four-stringed guitar-like instrument; balloon-guitar
U+7437.0	fUTF8	琷
U+7437.0	fGB	5-4240
U+7437.0	fR/S	 96.8
U+7437.0	fTotalStrokes	 12
U+7437.0	fCangjie	MGTGU
U+7437.0	fFourCorner	1811
U+7437.0	fCantonese	kyut3
U+7437.0	fMandarin	que4
U+7438.0	fUTF8	琸
U+7438.0	fCNS11643	3-3D33
U+7438.0	fGB	E-3343
U+7438.0	fR/S	 96.8
U+7438.0	fTotalStrokes	 12
U+7438.0	fCangjie	MGYAJ
U+7438.0	fFourCorner	11146
U+7438.0	fCantonese	zoek3
U+7438.0	fMandarin	zhuo2
U+7439.0	fUTF8	琹
U+7439.0	fCNS11643	3-3D3C
U+7439.0	fGB	E-3344
U+7439.0	fR/S	 96.8; 75.8
U+7439.0	fTotalStrokes	 12
U+7439.0	fCangjie	MGD
U+7439.0	fFourCorner	11900
U+7439.0	fCantonese	kam4
U+7439.0	fMandarin	qin2
U+743A.0	fUTF8	琺
U+743A.0	fCNS11643	1-605D
U+743A.0	fBig5	B557
U+743A.0	fGB	E-3345
U+743A.0	fR/S	 96.8
U+743A.0	fTotalStrokes	 12
U+743A.0	fCangjie	MGEGI
U+743A.0	fFourCorner	14131
U+743A.0	fHakka	  fap7
U+743A.0	fCantonese	faat3; fat3
U+743A.0	fMandarin	fa4
U+743A.0	fEnglish	enamel; enamelware
U+743B.0	fUTF8	琻
U+743B.0	fCNS11643	3-6566
U+743B.0	fGB	E-3346
U+743B.0	fR/S	 96.8
U+743B.0	fTotalStrokes	 12
U+743B.0	fCangjie	MGC
U+743B.0	fFourCorner	1811
U+743B.0	fCantonese	cyun4
U+743C.0	fUTF8	琼
U+743C.0	fCNS11643	3-3D37
U+743C.0	fGB	0-476D
U+743C.0	fR/S	 96.8
U+743C.0	fTotalStrokes	 12
U+743C.0	fCangjie	MGYRF
U+743C.0	fFourCorner	10190
U+743C.0	fHakka	 kiun2
U+743C.0	fCantonese	king4
U+743C.0	fMandarin	qiong2; xuan2
U+743C.0	fEnglish	simplified form of &#x74CA;
U+743D.0	fUTF8	琽
U+743D.0	fCNS11643	3-4268
U+743D.0	fGB	E-3347
U+743D.0	fR/S	 96.9; 96.8
U+743D.0	fTotalStrokes	 13; 12
U+743D.0	fCangjie	MGJKA
U+743D.0	fFourCorner	14160
U+743D.0	fCantonese	dou2
U+743D.0	fMandarin	du3
U+743E.0	fUTF8	琾
U+743E.0	fCNS11643	3-4275
U+743E.0	fGB	5-4252
U+743E.0	fR/S	  96.9
U+743E.0	fTotalStrokes	 13
U+743E.0	fCangjie	MGWOL
U+743E.0	fFourCorner	16120
U+743E.0	fCantonese	gaai3
U+743E.0	fMandarin	jie4
U+743F.0	fUTF8	琿
U+743F.0	fCNS11643	1-6531
U+743F.0	fBig5	B7E9
U+743F.0	fGB	1-6775
U+743F.0	fR/S	 96.9
U+743F.0	fTotalStrokes	 13
U+743F.0	fCangjie	MGBJJ
U+743F.0	fFourCorner	17156
U+743F.0	fHakka	 fun2
U+743F.0	fCantonese	wan4; fai1
U+743F.0	fMandarin	hun2; hui1
U+743F.0	fEnglish	a kind of precious jade
U+7440.0	fUTF8	瑀
U+7440.0	fCNS11643	2-447B
U+7440.0	fBig5	DEB7
U+7440.0	fGB	3-3F62
U+7440.0	fR/S	 96.9
U+7440.0	fTotalStrokes	 13
U+7440.0	fCangjie	MGHLB
U+7440.0	fFourCorner	12127
U+7440.0	fHakka	 ji3; ri3
U+7440.0	fCantonese	jyu5
U+7440.0	fMandarin	yu3
U+7440.0	fEnglish	[1] the luster of jade [2] a jadelike pebble [3] smooth and glossy; clean and shining; bright and brilliant [4] transparent; pure [5] bright and clever (person)
U+7441.0	fUTF8	瑁
U+7441.0	fCNS11643	1-6530
U+7441.0	fBig5	B7E8
U+7441.0	fGB	0-6823
U+7441.0	fR/S	 96.9
U+7441.0	fTotalStrokes	 13
U+7441.0	fCangjie	MGABU
U+7441.0	fFourCorner	16160
U+7441.0	fHakka	 mau5; mau6; mau3; mo6; mo5
U+7441.0	fCantonese	mou6; mui6
U+7441.0	fMandarin	mei4
U+7441.0	fEnglish	 tortoise shell
U+7441.1	fUTF8	瑁
U+7441.1	fCNS11643	1-6530
U+7441.1	fBig5	B7E8
U+7441.1	fGB	0-6823
U+7441.1	fR/S	 96.9
U+7441.1	fTotalStrokes	 13
U+7441.1	fCangjie	MGABU
U+7441.1	fFourCorner	16160
U+7441.1	fMandarin	mao4
U+7441.1	fEnglish	a very precious piece of jade worn by emperors to match tablets borne by the nobles
U+7442.0	fUTF8	瑂
U+7442.0	fCNS11643	2-4521
U+7442.0	fBig5	DEBB
U+7442.0	fGB	3-3F65
U+7442.0	fR/S	 96.9
U+7442.0	fTotalStrokes	 13
U+7442.0	fCangjie	MGAHU
U+7442.0	fFourCorner	17167
U+7442.0	fCantonese	mei4
U+7442.0	fMandarin	mei2
U+7443.0	fUTF8	瑃
U+7443.0	fCNS11643	3-426F
U+7443.0	fGB	3-3F5D
U+7443.0	fR/S	 96.9
U+7443.0	fTotalStrokes	 13
U+7443.0	fCangjie	MGQKA
U+7443.0	fFourCorner	15160
U+7443.0	fCantonese	ceon1
U+7443.0	fMandarin	chun1
U+7444.0	fUTF8	瑄
U+7444.0	fCNS11643	2-4475
U+7444.0	fBig5	DEB1
U+7444.0	fGB	3-3F64
U+7444.0	fR/S	 96.9
U+7444.0	fTotalStrokes	 13
U+7444.0	fCangjie	MGJMM
U+7444.0	fFourCorner	13116
U+7444.0	fHakka	  sien1
U+7444.0	fCantonese	syun1
U+7444.0	fMandarin	xuan1
U+7444.0	fEnglish	an ornamental piece of jade about 6.5 inches in diameter
U+7445.0	fUTF8	瑅
U+7445.0	fCNS11643	3-4274
U+7445.0	fGB	5-424F
U+7445.0	fR/S	 96.9
U+7445.0	fTotalStrokes	 13
U+7445.0	fCangjie	MGAMO
U+7445.0	fFourCorner	16180
U+7445.0	fCantonese	tai4
U+7445.0	fMandarin	ti2
U+7446.0	fUTF8	瑆
U+7446.0	fCNS11643	2-4522
U+7446.0	fBig5	DEBC
U+7446.0	fGB	3-3F60
U+7446.0	fR/S	 96.9
U+7446.0	fTotalStrokes	 13
U+7446.0	fCangjie	MGAHM
U+7446.0	fFourCorner	16115
U+7446.0	fCantonese	sing1
U+7446.0	fMandarin	xing1
U+7447.0	fUTF8	瑇
U+7447.0	fCNS11643	3-426E
U+7447.0	fGB	E-3348
U+7447.0	fR/S	 96.9
U+7447.0	fTotalStrokes	 13
U+7447.0	fCangjie	MGQMY
U+7447.0	fFourCorner	16150
U+7447.0	fHakka	 toi5; toi6
U+7447.0	fCantonese	doi6
U+7447.0	fMandarin	dai4; du2
U+7447.0	fEnglish	tortoise shell
U+7448.0	fUTF8	瑈
U+7448.0	fCNS11643	3-4266
U+7448.0	fGB	3-3F67
U+7448.0	fR/S	 96.9
U+7448.0	fTotalStrokes	 13
U+7448.0	fCangjie	MGNHD
U+7448.0	fFourCorner	17190
U+7448.0	fCantonese	jau4
U+7448.0	fMandarin	rou2
U+7449.0	fUTF8	瑉
U+7449.0	fCNS11643	3-4271
U+7449.0	fGB	5-4257
U+7449.0	fR/S	  96.9
U+7449.0	fTotalStrokes	 13
U+7449.0	fCangjie	MGRPA
U+7449.0	fFourCorner	17160
U+7449.0	fCantonese	man4
U+7449.0	fMandarin	min2
U+744A.0	fUTF8	瑊
U+744A.0	fCNS11643	2-4476
U+744A.0	fBig5	DEB2
U+744A.0	fGB	3-3F5F
U+744A.0	fR/S	 96.9
U+744A.0	fTotalStrokes	 13
U+744A.0	fCangjie	MGIHR
U+744A.0	fFourCorner	13150
U+744A.0	fCantonese	zam1
U+744A.0	fMandarin	zhen1; jian1; xian2
U+744B.0	fUTF8	瑋
U+744B.0	fCNS11643	2-4477
U+744B.0	fBig5	DEB3
U+744B.0	fGB	1-6762
U+744B.0	fR/S	 96.9; 96.10
U+744B.0	fTotalStrokes	 13
U+744B.0	fCangjie	MGDMQ
U+744B.0	fFourCorner	14156
U+744B.0	fHakka	 wui3; wi3; vui3
U+744B.0	fCantonese	wai5; wai4
U+744B.0	fMandarin	wei3; wei4
U+744B.0	fEnglish	[1] a kind of jade [2] rare; valuable; precious; splendorous
U+744C.0	fUTF8	瑌
U+744C.0	fCNS11643	3-4273
U+744C.0	fGB	5-424D
U+744C.0	fR/S	  96.9
U+744C.0	fTotalStrokes	 13
U+744C.0	fCangjie	MGMBK
U+744C.0	fFourCorner	11130
U+744C.0	fHakka	 ngion3; nion1
U+744C.0	fCantonese	jyun5
U+744C.0	fMandarin	ruan3
U+744C.0	fEnglish	a white opaque quartz used for ornaments
U+744D.0	fUTF8	瑍
U+744D.0	fCNS11643	2-4523
U+744D.0	fBig5	DEBD
U+744D.0	fGB	5-423D
U+744D.0	fR/S	 96.9
U+744D.0	fTotalStrokes	 13
U+744D.0	fCangjie	MGNBK
U+744D.0	fFourCorner	17134
U+744D.0	fCantonese	wun6
U+744D.0	fMandarin	huan4
U+744E.0	fUTF8	瑎
U+744E.0	fCNS11643	2-447E
U+744E.0	fBig5	DEBA
U+744E.0	fGB	5-424E
U+744E.0	fR/S	 96.9
U+744E.0	fTotalStrokes	 13
U+744E.0	fCangjie	MGPPA
U+744E.0	fFourCorner	11161
U+744E.0	fCantonese	haai4
U+744E.0	fMandarin	jie1; xie2
U+744F.0	fUTF8	瑏
U+744F.0	fCNS11643	2-447C
U+744F.0	fBig5	DEB8
U+744F.0	fGB	5-4255
U+744F.0	fR/S	 96.9
U+744F.0	fTotalStrokes	 13
U+744F.0	fCangjie	MGJCH
U+744F.0	fFourCorner	13232
U+744F.0	fHakka	 con1
U+744F.0	fCantonese	cyun1
U+744F.0	fMandarin	chuan1
U+7450.0	fUTF8	瑐
U+7450.0	fCNS11643	2-447D
U+7450.0	fBig5	DEB9
U+7450.0	fGB	E-3349
U+7450.0	fR/S	 96.9
U+7450.0	fTotalStrokes	 13
U+7450.0	fCangjie	MGTBN
U+7450.0	fFourCorner	18121
U+7450.0	fCantonese	zin2
U+7450.0	fMandarin	jian3
U+7451.0	fUTF8	瑑
U+7451.0	fCNS11643	2-4479
U+7451.0	fBig5	DEB5
U+7451.0	fGB	3-3F68
U+7451.0	fR/S	 96.9
U+7451.0	fTotalStrokes	 13
U+7451.0	fCangjie	MGVNO
U+7451.0	fFourCorner	17132
U+7451.0	fHakka	 chon5; con5
U+7451.0	fCantonese	syun6; cyun5
U+7451.0	fMandarin	zhuan4; chi2; zhun4
U+7451.0	fEnglish	engraving on a jade tablet
U+7452.0	fUTF8	瑒
U+7452.0	fCNS11643	2-4478
U+7452.0	fBig5	DEB4
U+7452.0	fGB	3-3E7C
U+7452.0	fR/S	 96.9
U+7452.0	fTotalStrokes	 13
U+7452.0	fCangjie	MGAMH
U+7452.0	fFourCorner	16127
U+7452.0	fCantonese	coeng3; dong6; joeng4
U+7452.0	fMandarin	yang2; chang4; dang4; ting3
U+7453.0	fUTF8	瑓
U+7453.0	fCNS11643	3-4267
U+7453.0	fGB	E-334A
U+7453.0	fR/S	  96.9
U+7453.0	fTotalStrokes	 13
U+7453.0	fCangjie	MGDWF
U+7453.0	fFourCorner	15190
U+7453.0	fCantonese	lin6
U+7453.0	fMandarin	lian4
U+7454.0	fUTF8	瑔
U+7454.0	fCNS11643	2-4524
U+7454.0	fBig5	DEBE
U+7454.0	fGB	5-4254
U+7454.0	fR/S	 96.9
U+7454.0	fTotalStrokes	 13
U+7454.0	fCangjie	MGHAE
U+7454.0	fFourCorner	16132
U+7454.0	fCantonese	cyun4
U+7454.0	fMandarin	quan2
U+7455.0	fUTF8	瑕
U+7455.0	fCNS11643	1-652D
U+7455.0	fBig5	B7E5
U+7455.0	fGB	0-6826
U+7455.0	fR/S	 96.9
U+7455.0	fTotalStrokes	 13
U+7455.0	fCangjie	MGRSE
U+7455.0	fFourCorner	17147
U+7455.0	fHakka	  ha2; ha3
U+7455.0	fCantonese	haa4
U+7455.0	fMandarin	xia2
U+7455.0	fEnglish	[1] flaw, spot, or blemish in a piece of jade [2] fault; error; blemish; flaw
U+7456.0	fUTF8	瑖
U+7456.0	fCNS11643	3-426B
U+7456.0	fGB	5-4253
U+7456.0	fR/S	 96.9
U+7456.0	fTotalStrokes	 13
U+7456.0	fCangjie	MGHJE
U+7456.0	fFourCorner	17140
U+7456.0	fCantonese	dyun3
U+7456.0	fMandarin	duan4
U+7457.0	fUTF8	瑗
U+7457.0	fCNS11643	2-447A
U+7457.0	fBig5	DEB6
U+7457.0	fGB	0-6825
U+7457.0	fR/S	 96.9
U+7457.0	fTotalStrokes	 13
U+7457.0	fCangjie	MGBME
U+7457.0	fFourCorner	12147
U+7457.0	fHakka	 jen5; ren6; jan5
U+7457.0	fCantonese	jyun6
U+7457.0	fMandarin	yuan4; huan2; yung4
U+7457.0	fEnglish	[1] large ring of fine jade [2] the name of a kind of jade
U+7458.0	fUTF8	瑘
U+7458.0	fCNS11643	3-4270
U+7458.0	fGB	5-4242
U+7458.0	fR/S	  96.9
U+7458.0	fTotalStrokes	 13
U+7458.0	fCangjie	MGSJL
U+7458.0	fFourCorner	17120
U+7458.0	fCantonese	ngaa4; aa4 (also); je4 (also)
U+7458.0	fMandarin	ye2
U+7459.0	fUTF8	瑙
U+7459.0	fCNS11643	1-6532
U+7459.0	fBig5	B7EA
U+7459.0	fGB	0-6827
U+7459.0	fR/S	 96.9
U+7459.0	fTotalStrokes	 13
U+7459.0	fCangjie	MGVVW
U+7459.0	fFourCorner	12163
U+7459.0	fHakka	  no3; nau3; lau3
U+7459.0	fCantonese	nou5; lou5 (also)
U+7459.0	fMandarin	nao3
U+7459.0	fEnglish	agate; cornelian
U+745A.0	fUTF8	瑚
U+745A.0	fCNS11643	1-652C
U+745A.0	fBig5	B7E4
U+745A.0	fGB	0-3A77
U+745A.0	fR/S	 96.9
U+745A.0	fTotalStrokes	 13
U+745A.0	fCangjie	MGJRB
U+745A.0	fFourCorner	17120
U+745A.0	fHakka	 fu2
U+745A.0	fCantonese	wu4
U+745A.0	fMandarin	hu2
U+745A.0	fEnglish	coral
U+745B.0	fUTF8	瑛
U+745B.0	fCNS11643	1-6533
U+745B.0	fBig5	B7EB
U+745B.0	fGB	0-6778
U+745B.0	fR/S	 96.9
U+745B.0	fTotalStrokes	 13
U+745B.0	fCangjie	MGTLK
U+745B.0	fFourCorner	14134
U+745B.0	fHakka	  jin1; rin1
U+745B.0	fCantonese	jing1
U+745B.0	fMandarin	ying1
U+745B.0	fEnglish	[1] the glitter or sheen of jade [2] a transparent piece of jade; crystal
U+745C.0	fUTF8	瑜
U+745C.0	fCNS11643	1-6534
U+745C.0	fBig5	B7EC
U+745C.0	fGB	0-6824
U+745C.0	fR/S	 96.9
U+745C.0	fTotalStrokes	 13
U+745C.0	fCangjie	MGOMN
U+745C.0	fFourCorner	18121
U+745C.0	fHakka	 ji2; ri2
U+745C.0	fCantonese	jyu4
U+745C.0	fMandarin	yu2
U+745C.0	fEnglish	[1] a fine and flawless piece of jade; flawless gem or jewel; perfect gem [2] the brilliancy of jade; luster of gems [3] excellencies; virtues [4] yoga, a mystic and ascetic practice in Hindu philosophy
U+745D.0	fUTF8	瑝
U+745D.0	fCNS11643	3-426A
U+745D.0	fGB	3-3F61
U+745D.0	fR/S	  96.9
U+745D.0	fTotalStrokes	 13
U+745D.0	fCangjie	MGHAG
U+745D.0	fFourCorner	16110
U+745D.0	fCantonese	waang4
U+745D.0	fMandarin	huang2
U+745E.0	fUTF8	瑞
U+745E.0	fCNS11643	1-652F
U+745E.0	fBig5	B7E7
U+745E.0	fGB	0-4870
U+745E.0	fR/S	 96.9
U+745E.0	fTotalStrokes	 13
U+745E.0	fCangjie	MGUMB
U+745E.0	fFourCorner	12127
U+745E.0	fHakka	 sui5;  shui5; sui6; sui3
U+745E.0	fCantonese	seoi6
U+745E.0	fMandarin	rui4
U+745E.0	fEnglish	[1] felicitous omen; something portending good luck or fortune; good omen [2] lucky; auspicious; fortunate [3] a jade tablet given to feudal princes on their investiture, as a sign of authority and rank [4] a Chinese family name
U+745F.0	fUTF8	瑟
U+745F.0	fCNS11643	1-652E
U+745F.0	fBig5	B7E6
U+745F.0	fGB	0-492A
U+745F.0	fR/S	 96.9
U+745F.0	fTotalStrokes	 13
U+745F.0	fCangjie	MGPH
U+745F.0	fFourCorner	11331
U+745F.0	fHakka	  sit7
U+745F.0	fCantonese	sat1
U+745F.0	fMandarin	se4; shi4
U+745F.0	fEnglish	[1] se, a large stringed horizontal musical, about nine feet long, originally with 50 strings which was later reduced to 25 with movable bridges for tuning [2] varied and many [3] elegant and stately; majestic [4] bright and clean; pure and clean [5] same as $
U+7460.0	fUTF8	瑠
U+7460.0	fCNS11643	3-4825
U+7460.0	fGB	E-334B
U+7460.0	fR/S	  96.10
U+7460.0	fTotalStrokes	 14
U+7460.0	fCangjie	MGHHW
U+7460.0	fFourCorner	17160
U+7460.0	fHakka	  liu2; liu3
U+7460.0	fCantonese	lau4
U+7460.0	fMandarin	liu2
U+7460.0	fEnglish	precious stone
U+7461.0	fUTF8	瑡
U+7461.0	fCNS11643	3-477E
U+7461.0	fGB	E-334C
U+7461.0	fR/S	 96.10
U+7461.0	fTotalStrokes	 14
U+7461.0	fCangjie	MGHRB
U+7461.0	fFourCorner	11120
U+7461.0	fCantonese	si1
U+7461.0	fMandarin	shi1
U+7462.0	fUTF8	瑢
U+7462.0	fCNS11643	2-4B74
U+7462.0	fBig5	E2CE
U+7462.0	fGB	3-3F72
U+7462.0	fR/S	 96.10
U+7462.0	fTotalStrokes	 14
U+7462.0	fCangjie	MGJCR
U+7462.0	fFourCorner	13168
U+7462.0	fHakka	 jung2; rung2
U+7462.0	fCantonese	jung4
U+7462.0	fMandarin	rong2
U+7462.0	fEnglish	gems or ornaments hanging from the girdle
U+7463.0	fUTF8	瑣
U+7463.0	fCNS11643	1-6965
U+7463.0	fBig5	BABE
U+7463.0	fGB	1-4B76
U+7463.0	fR/S	 96.10
U+7463.0	fTotalStrokes	 14
U+7463.0	fCangjie	MGFBC
U+7463.0	fFourCorner	19186
U+7463.0	fHakka	  so3
U+7463.0	fCantonese	so2
U+7463.0	fMandarin	suo3
U+7463.0	fEnglish	[1] trifles; petty; frivolous; trifling [2] troublesome; annoying [3] a jade chain [4] palace gate
U+7464.0	fUTF8	瑤
U+7464.0	fCNS11643	1-6964
U+7464.0	fBig5	BABD
U+7464.0	fGB	E-334D
U+7464.0	fR/S	 96.10
U+7464.0	fTotalStrokes	 14
U+7464.0	fCangjie	MGBOU
U+7464.0	fFourCorner	17172
U+7464.0	fHakka	 jau2; rau2; jau3; jeu2
U+7464.0	fCantonese	jiu4; jiu2 (also)
U+7464.0	fMandarin	yao2
U+7464.0	fEnglish	[1] a precious jade or stone [2] clean, pure and white [3] treasurable; valuable; precious
U+7465.0	fUTF8	瑥
U+7465.0	fCNS11643	3-477C
U+7465.0	fGB	5-4250
U+7465.0	fR/S	  96.10
U+7465.0	fTotalStrokes	 14
U+7465.0	fCangjie	MGABT
U+7465.0	fFourCorner	16110
U+7465.0	fCantonese	wan1
U+7465.0	fMandarin	wen1
U+7466.0	fUTF8	瑦
U+7466.0	fCNS11643	3-477D
U+7466.0	fGB	5-417C
U+7466.0	fR/S	  96.10
U+7466.0	fTotalStrokes	 14
U+7466.0	fCangjie	MGHRF
U+7466.0	fFourCorner	17120
U+7466.0	fCantonese	wu2
U+7466.0	fMandarin	wu1; wu3
U+7467.0	fUTF8	瑧
U+7467.0	fCNS11643	2-4B79
U+7467.0	fBig5	E2D3
U+7467.0	fGB	5-4258
U+7467.0	fR/S	 96.10
U+7467.0	fTotalStrokes	 14
U+7467.0	fCangjie	MGQKD
U+7467.0	fFourCorner	15194
U+7467.0	fCantonese	zeon1
U+7467.0	fMandarin	jin1
U+7468.0	fUTF8	瑨
U+7468.0	fCNS11643	3-4822
U+7468.0	fGB	3-3F6B
U+7468.0	fR/S	 96.10
U+7468.0	fTotalStrokes	 14
U+7468.0	fCangjie	MGMIA
U+7468.0	fFourCorner	11160
U+7468.0	fHakka	 zin3; zin5
U+7468.0	fCantonese	zeon3
U+7468.0	fMandarin	jin4
U+7468.0	fEnglish	a fine piece of jade
U+7469.0	fUTF8	瑩
U+7469.0	fCNS11643	1-6D66
U+7469.0	fBig5	BCFC
U+7469.0	fGB	1-5328
U+7469.0	fR/S	 96.10
U+7469.0	fTotalStrokes	 15
U+7469.0	fCangjie	FFBMI
U+7469.0	fFourCorner	99103
U+7469.0	fHakka	  jin2; jin3; rin2; jung2
U+7469.0	fCantonese	jing4; jing2 (also)
U+7469.0	fMandarin	ying2; ying3; ying4
U+7469.0	fEnglish	 lustre of gems bright; lustrous
U+746A.0	fUTF8	瑪
U+746A.0	fCNS11643	1-6966
U+746A.0	fBig5	BABF
U+746A.0	fGB	1-426A
U+746A.0	fR/S	 96.10
U+746A.0	fTotalStrokes	 14
U+746A.0	fCangjie	MGSQF
U+746A.0	fFourCorner	11127
U+746A.0	fHakka	  ma1;  ma3
U+746A.0	fCantonese	maa5
U+746A.0	fMandarin	ma3
U+746A.0	fEnglish	agate; cornelian
U+746B.0	fUTF8	瑫
U+746B.0	fCNS11643	3-4821
U+746B.0	fGB	3-3F70
U+746B.0	fR/S	 96.10
U+746B.0	fTotalStrokes	 14
U+746B.0	fCangjie	MGBHX
U+746B.0	fFourCorner	12170
U+746B.0	fCantonese	tou1
U+746B.0	fMandarin	tao1
U+746C.0	fUTF8	瑬
U+746C.0	fCNS11643	3-477B
U+746C.0	fGB	3-402A
U+746C.0	fR/S	 96.10; 96.9
U+746C.0	fTotalStrokes	 14; 15
U+746C.0	fCangjie	EUMGI
U+746C.0	fFourCorner	30100
U+746C.0	fCantonese	lau4
U+746C.0	fMandarin	liu2
U+746D.0	fUTF8	瑭
U+746D.0	fCNS11643	1-6968
U+746D.0	fBig5	BAC1
U+746D.0	fGB	0-6829
U+746D.0	fR/S	 96.10
U+746D.0	fTotalStrokes	 14
U+746D.0	fCangjie	MGILR
U+746D.0	fFourCorner	10167
U+746D.0	fHakka	 tong2
U+746D.0	fCantonese	tong4
U+746D.0	fMandarin	tang2
U+746D.0	fEnglish	a kind of jade
U+746E.0	fUTF8	瑮
U+746E.0	fCNS11643	2-4B7A
U+746E.0	fBig5	E2D4
U+746E.0	fGB	3-3F6E
U+746E.0	fR/S	 96.10
U+746E.0	fTotalStrokes	 14
U+746E.0	fCangjie	MGMWD
U+746E.0	fFourCorner	11194
U+746E.0	fCantonese	leot6
U+746E.0	fMandarin	li4
U+746F.0	fUTF8	瑯
U+746F.0	fCNS11643	1-652B
U+746F.0	fBig5	B7E3
U+746F.0	fGB	E-334E
U+746F.0	fR/S	 96.10
U+746F.0	fTotalStrokes	 14
U+746F.0	fCangjie	MGIIL
U+746F.0	fFourCorner	17127
U+746F.0	fHakka	 long2
U+746F.0	fCantonese	long4
U+746F.0	fMandarin	lang2; lan2
U+746F.0	fEnglish	a stone resembling jade, pure and white
U+7470.0	fUTF8	瑰
U+7470.0	fCNS11643	1-6967
U+7470.0	fBig5	BAC0
U+7470.0	fGB	0-3965
U+7470.0	fR/S	 96.10
U+7470.0	fTotalStrokes	 14
U+7470.0	fCangjie	MGHUI
U+7470.0	fFourCorner	16113
U+7470.0	fHakka	 gui5; kui2; kwui1; gwi5; kwui5
U+7470.0	fCantonese	gwai1; gwai3
U+7470.0	fMandarin	gui1; gui4; hui2
U+7470.0	fEnglish	[1] extraordinary; fabulous; great [2] a stone which is a little less valuable than jade; a kind of jasper
U+7471.0	fUTF8	瑱
U+7471.0	fCNS11643	2-4B76
U+7471.0	fBig5	E2D0
U+7471.0	fGB	3-3F6D
U+7471.0	fR/S	 96.10
U+7471.0	fTotalStrokes	 14
U+7471.0	fCangjie	MGJBC
U+7471.0	fFourCorner	14186
U+7471.0	fHakka	  tien5
U+7471.0	fCantonese	tin3; zan3; din3
U+7471.0	fMandarin	tian4; tian3
U+7471.0	fEnglish	gem used as ear plug; earplugs of jade
U+7471.1	fUTF8	瑱
U+7471.1	fCNS11643	2-4B76
U+7471.1	fBig5	E2D0
U+7471.1	fGB	3-3F6D
U+7471.1	fR/S	 96.10
U+7471.1	fTotalStrokes	 14
U+7471.1	fCangjie	MGJBC
U+7471.1	fFourCorner	14186
U+7471.1	fMandarin	zhen4; zhen1
U+7471.1	fEnglish	 weight; press
U+7472.0	fUTF8	瑲
U+7472.0	fCNS11643	2-4B78
U+7472.0	fBig5	E2D2
U+7472.0	fGB	3-3F25
U+7472.0	fR/S	 96.10
U+7472.0	fTotalStrokes	 14
U+7472.0	fCangjie	MGOIR
U+7472.0	fFourCorner	18167
U+7472.0	fHakka	  cong1
U+7472.0	fCantonese	coeng1
U+7472.0	fMandarin	qiang1; cang1; cheng1
U+7472.0	fEnglish	the tinkling sound tinkling of pendant gems or jade
U+7473.0	fUTF8	瑳
U+7473.0	fCNS11643	2-4B75
U+7473.0	fBig5	E2CF
U+7473.0	fGB	3-3F63
U+7473.0	fR/S	 96.10
U+7473.0	fTotalStrokes	 14
U+7473.0	fCangjie	MGTQM
U+7473.0	fFourCorner	18111
U+7473.0	fHakka	 co2; co3; co1
U+7473.0	fCantonese	co1; co2
U+7473.0	fMandarin	cuo3; cuo1
U+7473.0	fEnglish	[1] the luster and purity of jade [2] lustrous; bright and flourishing [3] the brilliant white appearance of the teeth
U+7474.0	fUTF8	瑴
U+7474.0	fCNS11643	3-4827
U+7474.0	fGB	5-506C
U+7474.0	fR/S	  96.10
U+7474.0	fTotalStrokes	 14
U+7474.0	fCangjie	GGHNE
U+7474.0	fFourCorner	47140
U+7474.0	fCantonese	gok3
U+7474.0	fMandarin	jue2
U+7474.0	fEnglish	twin gems
U+7475.0	fUTF8	瑵
U+7475.0	fCNS11643	2-4B77
U+7475.0	fBig5	E2D1
U+7475.0	fGB	3-3F66
U+7475.0	fR/S	 96.10
U+7475.0	fTotalStrokes	 14
U+7475.0	fCangjie	MGEII
U+7475.0	fFourCorner	17113
U+7475.0	fCantonese	zaau2; zou2
U+7475.0	fMandarin	zhao3
U+7476.0	fUTF8	瑶
U+7476.0	fCNS11643	3-4823
U+7476.0	fGB	0-517E
U+7476.0	fR/S	 96.10; 96.9
U+7476.0	fTotalStrokes	 14
U+7476.0	fCangjie	MGBOU
U+7476.0	fFourCorner	12174
U+7476.0	fHakka	 jau2
U+7476.0	fCantonese	jiu4
U+7476.0	fMandarin	yao2
U+7476.0	fEnglish	precious jade
U+7477.0	fUTF8	瑷
U+7477.0	fGB	0-6828
U+7477.0	fR/S	 96.10
U+7477.0	fTotalStrokes	 14
U+7477.0	fCangjie	XMGBB
U+7477.0	fCantonese	oi3; ngoi3 (also)
U+7477.0	fMandarin	ai4
U+7477.0	fEnglish	simplified form of &#x74A6;
U+7478.0	fUTF8	瑸
U+7478.0	fCNS11643	F-4976
U+7478.0	fGB	8-7B5B
U+7478.0	fR/S	 96.10
U+7478.0	fTotalStrokes	 14
U+7478.0	fCangjie	MGJOC
U+7478.0	fCantonese	ban1
U+7478.0	fMandarin	bin1
U+7479.0	fUTF8	瑹
U+7479.0	fCNS11643	2-5233
U+7479.0	fBig5	E6AB
U+7479.0	fGB	5-425A
U+7479.0	fR/S	 96.11
U+7479.0	fTotalStrokes	 15
U+7479.0	fCangjie	MGTOD
U+7479.0	fFourCorner	14194
U+7479.0	fHakka	 shu1; su1
U+7479.0	fCantonese	syu1
U+7479.0	fMandarin	tu2; shu1
U+7479.0	fEnglish	[1] a gem [2] the ivory tablet used at audiences by feudal princes
U+747A.0	fUTF8	瑺
U+747A.0	fCNS11643	3-4D31
U+747A.0	fGB	3-3F75
U+747A.0	fR/S	 96.11
U+747A.0	fTotalStrokes	 15
U+747A.0	fCangjie	MGFBB
U+747A.0	fFourCorner	19120
U+747A.0	fCantonese	soeng4
U+747A.0	fMandarin	chang2
U+747B.0	fUTF8	瑻
U+747B.0	fCNS11643	3-4D3B
U+747B.0	fGB	5-424C
U+747B.0	fR/S	  96.11
U+747B.0	fTotalStrokes	 15
U+747B.0	fCangjie	MGWJC
U+747B.0	fFourCorner	17180
U+747B.0	fCantonese	kwan1
U+747B.0	fMandarin	kun1
U+747C.0	fUTF8	瑼
U+747C.0	fCNS11643	2-5232
U+747C.0	fBig5	E6AA
U+747C.0	fGB	5-4174
U+747C.0	fR/S	 96.11
U+747C.0	fTotalStrokes	 15
U+747C.0	fCangjie	MGJII
U+747C.0	fFourCorner	15143
U+747C.0	fCantonese	zyun1
U+747C.0	fMandarin	zhuan1
U+747D.0	fUTF8	瑽
U+747D.0	fCNS11643	2-522F
U+747D.0	fBig5	E6A7
U+747D.0	fGB	3-3F24
U+747D.0	fR/S	 96.11
U+747D.0	fTotalStrokes	 15
U+747D.0	fCangjie	MGHOO
U+747D.0	fFourCorner	18181
U+747D.0	fHakka	 ciung2; cung5
U+747D.0	fCantonese	cung1; cung4 (also)
U+747D.0	fMandarin	cong1
U+747D.0	fEnglish	the tinkling of jade pendants or metal ornaments
U+747E.0	fUTF8	瑾
U+747E.0	fCNS11643	1-6D69
U+747E.0	fBig5	BD40
U+747E.0	fGB	0-682A
U+747E.0	fR/S	 96.11
U+747E.0	fTotalStrokes	 15
U+747E.0	fCangjie	MGTLM
U+747E.0	fFourCorner	14114
U+747E.0	fHakka	  giun3
U+747E.0	fCantonese	gan2; gan6
U+747E.0	fMandarin	jin3; jin4
U+747E.0	fEnglish	fine jade
U+747F.0	fUTF8	瑿
U+747F.0	fCNS11643	2-584E
U+747F.0	fBig5	EA62
U+747F.0	fGB	3-402B
U+747F.0	fR/S	 96.11
U+747F.0	fTotalStrokes	 16
U+747F.0	fCangjie	SEMGI
U+747F.0	fFourCorner	77103
U+747F.0	fHakka	 hi1; ri1; ji1
U+747F.0	fCantonese	ji1; hei1
U+747F.0	fMandarin	yi1
U+747F.0	fEnglish	 a black stone like jade jet
U+7480.0	fUTF8	璀
U+7480.0	fCNS11643	1-6D6A
U+7480.0	fBig5	BD41
U+7480.0	fGB	0-682D
U+7480.0	fR/S	 96.11
U+7480.0	fTotalStrokes	 15
U+7480.0	fCangjie	MGUOG
U+7480.0	fFourCorner	12114
U+7480.0	fHakka	 cui3; cui1; cui5
U+7480.0	fCantonese	ceoi1; ceoi2 (also)
U+7480.0	fMandarin	cui3
U+7480.0	fEnglish	the luster or glitter of jade or gems
U+7481.0	fUTF8	璁
U+7481.0	fCNS11643	2-522E
U+7481.0	fBig5	E6A6
U+7481.0	fGB	0-682E
U+7481.0	fR/S	 96.11
U+7481.0	fTotalStrokes	 15
U+7481.0	fCangjie	MGHWP
U+7481.0	fFourCorner	16130
U+7481.0	fHakka	 cung1
U+7481.0	fCantonese	cung1
U+7481.0	fMandarin	cong1; zong3
U+7481.0	fEnglish	a jadelike stone of a greenish blue color
U+7482.0	fUTF8	璂
U+7482.0	fCNS11643	3-4D39
U+7482.0	fGB	3-3F73
U+7482.0	fR/S	  96.11
U+7482.0	fTotalStrokes	 15
U+7482.0	fCangjie	MGTCG
U+7482.0	fFourCorner	14110
U+7482.0	fHakka	 ki2
U+7482.0	fCantonese	kei4
U+7482.0	fMandarin	qi2
U+7482.0	fEnglish	jade ornament
U+7483.0	fUTF8	璃
U+7483.0	fCNS11643	1-6D68
U+7483.0	fBig5	BCFE
U+7483.0	fGB	0-4127
U+7483.0	fR/S	 96.11
U+7483.0	fTotalStrokes	 15
U+7483.0	fCangjie	MGYUB
U+7483.0	fFourCorner	10127
U+7483.0	fHakka	  li2
U+7483.0	fCantonese	lei4; lei1 (also)
U+7483.0	fMandarin	li2; li5
U+7483.0	fEnglish	glass
U+7484.0	fUTF8	璄
U+7484.0	fCNS11643	3-4D36
U+7484.0	fGB	5-4260
U+7484.0	fR/S	  96.11
U+7484.0	fTotalStrokes	 15
U+7484.0	fCangjie	MGYTU
U+7484.0	fFourCorner	10110
U+7484.0	fHakka	  gin3; rang3; jang3
U+7484.0	fCantonese	ging2
U+7484.0	fMandarin	ying3
U+7484.0	fEnglish	the lustre of gems
U+7485.0	fUTF8	璅
U+7485.0	fCNS11643	2-5230
U+7485.0	fBig5	E6A8
U+7485.0	fGB	5-4261
U+7485.0	fR/S	 96.11
U+7485.0	fTotalStrokes	 15
U+7485.0	fCangjie	MGVVD
U+7485.0	fFourCorner	12194
U+7485.0	fHakka	 zau3; so3
U+7485.0	fCantonese	so2; ziu2; zou2
U+7485.0	fMandarin	suo3; zao3
U+7485.0	fEnglish	gem
U+7486.0	fUTF8	璆
U+7486.0	fCNS11643	2-522D
U+7486.0	fBig5	E6A5
U+7486.0	fGB	3-3F76
U+7486.0	fR/S	 96.11
U+7486.0	fTotalStrokes	 15
U+7486.0	fCangjie	MGSMH
U+7486.0	fFourCorner	17122
U+7486.0	fHakka	  kiu2
U+7486.0	fCantonese	kau4
U+7486.0	fMandarin	qiu2; liu4; you2; zhou1
U+7486.0	fEnglish	 beautiful jade tinkle
U+7487.0	fUTF8	璇
U+7487.0	fCNS11643	2-522A
U+7487.0	fBig5	E6A2
U+7487.0	fGB	0-682F
U+7487.0	fR/S	 96.11
U+7487.0	fTotalStrokes	 15
U+7487.0	fCangjie	MGYSO
U+7487.0	fFourCorner	18181
U+7487.0	fHakka	  sien2; sion2
U+7487.0	fCantonese	syun4
U+7487.0	fMandarin	xuan2
U+7487.0	fEnglish	[1] beautiful jade; fine jade [2] the name of a constellation
U+7488.0	fUTF8	璈
U+7488.0	fCNS11643	2-5231
U+7488.0	fBig5	E6A9
U+7488.0	fGB	3-3F69
U+7488.0	fR/S	 96.11
U+7488.0	fTotalStrokes	 15
U+7488.0	fCangjie	MGGSK
U+7488.0	fFourCorner	18140
U+7488.0	fHakka	  ngau2
U+7488.0	fCantonese	ngou4; ou4 (also); ngou6
U+7488.0	fMandarin	ao2
U+7488.0	fEnglish	an stringed musical instrument
U+7489.0	fUTF8	璉
U+7489.0	fCNS11643	2-522B
U+7489.0	fBig5	E6A3
U+7489.0	fGB	1-6776
U+7489.0	fR/S	 96.11
U+7489.0	fTotalStrokes	 15
U+7489.0	fCangjie	MGYJJ
U+7489.0	fFourCorner	15130
U+7489.0	fHakka	  lien3; len3; len2
U+7489.0	fCantonese	lin5; lin4
U+7489.0	fMandarin	lian2; lian3
U+7489.0	fEnglish	a vessel used to hold grain offerings for the imperial sacrifice
U+748A.0	fUTF8	璊
U+748A.0	fCNS11643	2-522C
U+748A.0	fBig5	E6A4
U+748A.0	fGB	3-3F6A
U+748A.0	fR/S	 96.11
U+748A.0	fTotalStrokes	 15
U+748A.0	fCangjie	MGTLB
U+748A.0	fFourCorner	14127
U+748A.0	fHakka	  mun2;  man1
U+748A.0	fCantonese	mun4
U+748A.0	fMandarin	man2; men2
U+748A.0	fEnglish	reddish
U+748B.0	fUTF8	璋
U+748B.0	fCNS11643	1-6D67
U+748B.0	fBig5	BCFD
U+748B.0	fGB	0-6830
U+748B.0	fR/S	 96.11
U+748B.0	fTotalStrokes	 15
U+748B.0	fCangjie	MGYTJ
U+748B.0	fFourCorner	10146
U+748B.0	fHakka	 zong1;  zhong1
U+748B.0	fCantonese	zoeng1
U+748B.0	fMandarin	zhang1
U+748B.0	fEnglish	a jade ornament used in state ceremonies; jade tablet
U+748C.0	fUTF8	璌
U+748C.0	fCNS11643	3-4D35
U+748C.0	fGB	E-334F
U+748C.0	fR/S	 96.11
U+748C.0	fTotalStrokes	 15
U+748C.0	fCangjie	MGJMC
U+748C.0	fFourCorner	13180
U+748C.0	fCantonese	jan4
U+748C.0	fMandarin	yin2
U+748D.0	fUTF8	璍
U+748D.0	fCNS11643	3-6568
U+748D.0	fGB	E-3350
U+748D.0	fR/S	 96.12
U+748D.0	fTotalStrokes	 16
U+748D.0	fCangjie	MGTMJ
U+748D.0	fFourCorner	1415
U+748D.0	fCantonese	jip6
U+748E.0	fUTF8	璎
U+748E.0	fGB	0-682C
U+748E.0	fR/S	 96.11
U+748E.0	fTotalStrokes	 15
U+748E.0	fCangjie	MGBOV
U+748E.0	fHakka	 jin1
U+748E.0	fCantonese	jing1
U+748E.0	fMandarin	ying1
U+748E.0	fEnglish	simplified form of &#x74D4;
U+748F.0	fUTF8	璏
U+748F.0	fCNS11643	4-5139
U+748F.0	fGB	3-3F7E
U+748F.0	fR/S	 96.12
U+748F.0	fTotalStrokes	 16
U+748F.0	fCangjie	MGVMP
U+748F.0	fFourCorner	17110
U+748F.0	fCantonese	wai6
U+748F.0	fMandarin	zhi4; wei4
U+7490.0	fUTF8	璐
U+7490.0	fCNS11643	2-5D55
U+7490.0	fBig5	ED69
U+7490.0	fGB	0-6834
U+7490.0	fR/S	 96.12; 96.13
U+7490.0	fTotalStrokes	 16
U+7490.0	fCangjie	MGRMR
U+7490.0	fFourCorner	17164
U+7490.0	fHakka	 lu5; lu6; lu3
U+7490.0	fCantonese	lou6
U+7490.0	fMandarin	lu4
U+7490.0	fEnglish	fine jade
U+7491.0	fUTF8	璑
U+7491.0	fCNS11643	4-513A
U+7491.0	fGB	3-3E7E
U+7491.0	fR/S	  96.12
U+7491.0	fTotalStrokes	 16
U+7491.0	fCangjie	MGOTF
U+7491.0	fFourCorner	18130
U+7491.0	fCantonese	mou4
U+7491.0	fMandarin	wu2
U+7492.0	fUTF8	璒
U+7492.0	fCNS11643	2-5852
U+7492.0	fBig5	EA66
U+7492.0	fGB	5-4265
U+7492.0	fR/S	 96.12
U+7492.0	fTotalStrokes	 16
U+7492.0	fCangjie	MGNOT
U+7492.0	fFourCorner	17118
U+7492.0	fCantonese	dang1
U+7492.0	fMandarin	deng1
U+7493.0	fUTF8	璓
U+7493.0	fCNS11643	3-4D3A
U+7493.0	fGB	5-4259
U+7493.0	fR/S	 96.11; 96.12
U+7493.0	fTotalStrokes	 15
U+7493.0	fCangjie	MGTHS
U+7493.0	fFourCorner	14120
U+7493.0	fCantonese	sau3
U+7493.0	fMandarin	xiu4
U+7494.0	fUTF8	璔
U+7494.0	fCNS11643	2-5851
U+7494.0	fBig5	EA65
U+7494.0	fGB	E-3351
U+7494.0	fR/S	 96.12
U+7494.0	fTotalStrokes	 16
U+7494.0	fCangjie	MGCWA
U+7494.0	fFourCorner	18166
U+7494.0	fCantonese	zang1
U+7494.0	fMandarin	zeng1
U+7495.0	fUTF8	璕
U+7495.0	fCNS11643	2-5853
U+7495.0	fBig5	EA67
U+7495.0	fGB	3-3F43
U+7495.0	fR/S	 96.12
U+7495.0	fTotalStrokes	 16
U+7495.0	fCangjie	MGSMI
U+7495.0	fFourCorner	17146
U+7495.0	fCantonese	cam4
U+7495.0	fMandarin	xun2
U+7496.0	fUTF8	璖
U+7496.0	fCNS11643	3-516A
U+7496.0	fGB	E-3352
U+7496.0	fR/S	 96.12
U+7496.0	fTotalStrokes	 16
U+7496.0	fCangjie	MGESD
U+7496.0	fFourCorner	11190
U+7496.0	fHakka	 ki2
U+7496.0	fCantonese	keoi4
U+7496.0	fMandarin	qu2
U+7497.0	fUTF8	璗
U+7497.0	fCNS11643	2-5D52
U+7497.0	fBig5	ED66
U+7497.0	fGB	3-4028
U+7497.0	fR/S	 96.12
U+7497.0	fTotalStrokes	 17
U+7497.0	fCangjie	EHMGI
U+7497.0	fFourCorner	36103
U+7497.0	fHakka	 tong3
U+7497.0	fCantonese	dong6
U+7497.0	fMandarin	dang4
U+7497.0	fEnglish	pure (gold, gems
U+7498.0	fUTF8	璘
U+7498.0	fCNS11643	1-7144
U+7498.0	fBig5	BF5A
U+7498.0	fGB	3-3F7C
U+7498.0	fR/S	 96.12; 96.13
U+7498.0	fTotalStrokes	 16
U+7498.0	fCangjie	MGFDQ
U+7498.0	fFourCorner	19159
U+7498.0	fHakka	 lin2
U+7498.0	fCantonese	leon4
U+7498.0	fMandarin	lin2; lin3
U+7498.0	fEnglish	the brilliance of jade
U+7499.0	fUTF8	璙
U+7499.0	fCNS11643	4-5138
U+7499.0	fGB	3-3F78
U+7499.0	fR/S	 96.12
U+7499.0	fTotalStrokes	 16
U+7499.0	fCangjie	MGKCF
U+7499.0	fFourCorner	14190
U+7499.0	fCantonese	liu4
U+7499.0	fMandarin	liao2
U+749A.0	fUTF8	璚
U+749A.0	fCNS11643	2-584F
U+749A.0	fBig5	EA63
U+749A.0	fGB	5-4266
U+749A.0	fR/S	 96.12
U+749A.0	fTotalStrokes	 16
U+749A.0	fCangjie	MGNHB
U+749A.0	fFourCorner	17127
U+749A.0	fCantonese	king4; kyut3; ging4
U+749A.0	fMandarin	qiong2; jue2
U+749A.0	fEnglish	splendid
U+749B.0	fUTF8	璛
U+749B.0	fCNS11643	3-5167
U+749B.0	fGB	3-3F5B
U+749B.0	fR/S	 96.12
U+749B.0	fTotalStrokes	 16
U+749B.0	fCangjie	MGLX
U+749B.0	fFourCorner	15120
U+749B.0	fCantonese	suk1
U+749B.0	fMandarin	su4
U+749C.0	fUTF8	璜
U+749C.0	fCNS11643	1-7142
U+749C.0	fBig5	BF58
U+749C.0	fGB	0-682B
U+749C.0	fR/S	 96.12
U+749C.0	fTotalStrokes	 16
U+749C.0	fCangjie	MGTLC
U+749C.0	fFourCorner	14186
U+749C.0	fHakka	 wong2; wang2; vong2; vang2
U+749C.0	fCantonese	wong4
U+749C.0	fMandarin	huang2; heng2
U+749C.0	fEnglish	jade ornament semicircular in shape
U+749D.0	fUTF8	璝
U+749D.0	fCNS11643	3-516C
U+749D.0	fGB	5-4251
U+749D.0	fR/S	 96.12
U+749D.0	fTotalStrokes	 16
U+749D.0	fCangjie	MGLMC
U+749D.0	fFourCorner	15180
U+749D.0	fCantonese	gwai1
U+749D.0	fMandarin	gui1
U+749E.0	fUTF8	璞
U+749E.0	fCNS11643	1-7146
U+749E.0	fBig5	BF5C
U+749E.0	fGB	0-6831
U+749E.0	fR/S	 96.12
U+749E.0	fTotalStrokes	 16
U+749E.0	fCangjie	MGTCO
U+749E.0	fFourCorner	12134
U+749E.0	fHakka	  pok7; puk7; puk8
U+749E.0	fCantonese	pok3
U+749E.0	fMandarin	pu2
U+749E.0	fEnglish	an unpolished jade or gem; uncarved jade or gem
U+749F.0	fUTF8	璟
U+749F.0	fCNS11643	1-7145
U+749F.0	fBig5	BF5B
U+749F.0	fGB	3-3F79
U+749F.0	fR/S	 96.12
U+749F.0	fTotalStrokes	 16
U+749F.0	fCangjie	MGAYF
U+749F.0	fFourCorner	16196
U+749F.0	fHakka	  gin3; rang3; jang3
U+749F.0	fCantonese	ging2
U+749F.0	fMandarin	jing3; ying3
U+749F.0	fEnglish	the luster of jade
U+74A0.0	fUTF8	璠
U+74A0.0	fCNS11643	2-5850
U+74A0.0	fBig5	EA64
U+74A0.0	fGB	3-3F7A
U+74A0.0	fR/S	 96.12
U+74A0.0	fTotalStrokes	 16
U+74A0.0	fCangjie	MGHDW
U+74A0.0	fFourCorner	12169
U+74A0.0	fHakka	 fan1
U+74A0.0	fCantonese	faan4
U+74A0.0	fMandarin	fan2; fan1
U+74A0.0	fEnglish	a piece of precious jade
U+74A1.0	fUTF8	璡
U+74A1.0	fCNS11643	2-5854
U+74A1.0	fBig5	EA68
U+74A1.0	fGB	3-3F48
U+74A1.0	fR/S	 96.12
U+74A1.0	fTotalStrokes	 16
U+74A1.0	fCangjie	MGYOG
U+74A1.0	fFourCorner	10131
U+74A1.0	fHakka	 zin1; zin5
U+74A1.0	fCantonese	zeon3; zeon1 (also)
U+74A1.0	fMandarin	jin4; jin1
U+74A1.0	fEnglish	jade look alike stone
U+74A2.0	fUTF8	璢
U+74A2.0	fCNS11643	3-516B
U+74A2.0	fGB	E-3353
U+74A2.0	fR/S	 96.13; 96.12
U+74A2.0	fTotalStrokes	 16; 17
U+74A2.0	fCangjie	MGMLW
U+74A2.0	fFourCorner	11160
U+74A2.0	fCantonese	lau4
U+74A2.0	fMandarin	liu2
U+74A2.0	fEnglish	precious stone
U+74A3.0	fUTF8	璣
U+74A3.0	fCNS11643	1-7143
U+74A3.0	fBig5	BF59
U+74A3.0	fGB	1-6761
U+74A3.0	fR/S	 96.12
U+74A3.0	fTotalStrokes	 16
U+74A3.0	fCangjie	MGVII
U+74A3.0	fFourCorner	12153
U+74A3.0	fHakka	  gi1
U+74A3.0	fCantonese	gei1
U+74A3.0	fMandarin	ji1; ji4
U+74A3.0	fEnglish	[1] pearls, jade that is not quite round [2] used in combination for an ancient astronomical instrument [3] the name of a constellation
U+74A4.0	fUTF8	璤
U+74A4.0	fCNS11643	3-6567
U+74A4.0	fGB	E-3354
U+74A4.0	fR/S	 96.12
U+74A4.0	fTotalStrokes	 16
U+74A4.0	fCangjie	MGJIP
U+74A4.0	fFourCorner	1513
U+74A4.0	fCantonese	wai6
U+74A5.0	fUTF8	璥
U+74A5.0	fCNS11643	2-5D59
U+74A5.0	fBig5	ED6D
U+74A5.0	fGB	3-3F77
U+74A5.0	fR/S	 96.13
U+74A5.0	fTotalStrokes	 17
U+74A5.0	fCangjie	MGTRK
U+74A5.0	fFourCorner	18140
U+74A5.0	fHakka	 gin3
U+74A5.0	fCantonese	ging2
U+74A5.0	fMandarin	jing3; jing4
U+74A5.0	fEnglish	a kind of jade
U+74A6.0	fUTF8	璦
U+74A6.0	fCNS11643	1-7440
U+74A6.0	fBig5	C0F5
U+74A6.0	fGB	1-6828
U+74A6.0	fR/S	 96.13
U+74A6.0	fTotalStrokes	 17
U+74A6.0	fCangjie	MGBBE
U+74A6.0	fFourCorner	12147
U+74A6.0	fCantonese	oi3; ngoi3 (also)
U+74A6.0	fMandarin	ai4
U+74A6.0	fEnglish	fine jade
U+74A7.0	fUTF8	璧
U+74A7.0	fCNS11643	1-7666
U+74A7.0	fBig5	C27A
U+74A7.0	fGB	0-6835
U+74A7.0	fR/S	 96.13
U+74A7.0	fTotalStrokes	 17
U+74A7.0	fCangjie	SJMGI
U+74A7.0	fFourCorner	70103
U+74A7.0	fHakka	 biak7
U+74A7.0	fCantonese	bik1; bik3 (also)
U+74A7.0	fMandarin	bi4
U+74A7.0	fEnglish	[1] a round and flat piece of jade with a circular hole in it [2] a general name of all kinds of jade, jade-wares and ornaments
U+74A8.0	fUTF8	璨
U+74A8.0	fCNS11643	1-7441
U+74A8.0	fBig5	C0F6
U+74A8.0	fGB	0-6832
U+74A8.0	fR/S	 96.13
U+74A8.0	fTotalStrokes	 17
U+74A8.0	fCangjie	MGYED
U+74A8.0	fFourCorner	17194
U+74A8.0	fHakka	  can5; can3
U+74A8.0	fCantonese	caan3
U+74A8.0	fMandarin	can4
U+74A8.0	fEnglish	bright and brilliant; lustrous and luminous
U+74A9.0	fUTF8	璩
U+74A9.0	fCNS11643	1-743E
U+74A9.0	fBig5	C0F3
U+74A9.0	fGB	0-6833
U+74A9.0	fR/S	 96.13
U+74A9.0	fTotalStrokes	 17
U+74A9.0	fCangjie	MGYPO
U+74A9.0	fFourCorner	11132
U+74A9.0	fHakka	  ki2; gi1
U+74A9.0	fCantonese	keoi4
U+74A9.0	fMandarin	qu2; ju1
U+74A9.0	fEnglish	[1] a ringed ornament (earrings, rings for fingers) [2] a Chinese family name
U+74AA.0	fUTF8	璪
U+74AA.0	fCNS11643	2-5D56
U+74AA.0	fBig5	ED6A
U+74AA.0	fGB	3-4023
U+74AA.0	fR/S	 96.13
U+74AA.0	fTotalStrokes	 17
U+74AA.0	fCangjie	MGRRD
U+74AA.0	fFourCorner	16194
U+74AA.0	fHakka	 cau5;  zau3
U+74AA.0	fCantonese	zou2
U+74AA.0	fMandarin	zao3
U+74AA.0	fEnglish	pearl pendants on coronet
U+74AB.0	fUTF8	璫
U+74AB.0	fCNS11643	2-5D54
U+74AB.0	fBig5	ED68
U+74AB.0	fGB	3-3F37
U+74AB.0	fR/S	 96.13
U+74AB.0	fTotalStrokes	 17
U+74AB.0	fCangjie	MGFBW
U+74AB.0	fFourCorner	19166
U+74AB.0	fHakka	  dong1
U+74AB.0	fCantonese	dong1
U+74AB.0	fMandarin	dang1
U+74AB.0	fEnglish	[1] rich ornamented [2] pearls for filling up ear punctures to prevent the holes from closing
U+74AC.0	fUTF8	璬
U+74AC.0	fCNS11643	3-554F
U+74AC.0	fGB	3-4024
U+74AC.0	fR/S	  96.13
U+74AC.0	fTotalStrokes	 17
U+74AC.0	fCangjie	MGHSK
U+74AC.0	fFourCorner	18140
U+74AC.0	fCantonese	giu2
U+74AC.0	fMandarin	jiao3
U+74AD.0	fUTF8	璭
U+74AD.0	fCNS11643	2-5D57
U+74AD.0	fBig5	ED6B
U+74AD.0	fGB	E-3355
U+74AD.0	fR/S	 96.13
U+74AD.0	fTotalStrokes	 17
U+74AD.0	fCangjie	MGYBJ
U+74AD.0	fFourCorner	17135
U+74AD.0	fCantonese	gun2; gwun2 (also)
U+74AD.0	fMandarin	gun4
U+74AE.0	fUTF8	璮
U+74AE.0	fCNS11643	3-554B
U+74AE.0	fGB	E-3356
U+74AE.0	fR/S	  96.13
U+74AE.0	fTotalStrokes	 17
U+74AE.0	fCangjie	MGYWM
U+74AE.0	fFourCorner	10110
U+74AE.0	fCantonese	taan2
U+74AE.0	fMandarin	tan3
U+74AF.0	fUTF8	璯
U+74AF.0	fCNS11643	2-5D5A
U+74AF.0	fBig5	ED6E
U+74AF.0	fGB	3-3F3A
U+74AF.0	fR/S	 96.13
U+74AF.0	fTotalStrokes	 17
U+74AF.0	fCangjie	MGOMA
U+74AF.0	fFourCorner	18166
U+74AF.0	fCantonese	wui6; faai3
U+74AF.0	fMandarin	hui4; kuai4
U+74B0.0	fUTF8	環
U+74B0.0	fCNS11643	1-743F
U+74B0.0	fBig5	C0F4
U+74B0.0	fGB	1-3B37
U+74B0.0	fR/S	 96.13
U+74B0.0	fTotalStrokes	 17
U+74B0.0	fCangjie	MGWLV
U+74B0.0	fFourCorner	16132
U+74B0.0	fHakka	  wan2; fan2; k'wan2; k'wan1; wan3; pan2
U+74B0.0	fCantonese	waan4; waan2 (also)
U+74B0.0	fMandarin	huan2; huan4
U+74B0.0	fEnglish	[1] jade ring or bracelet; ring; bracelet [2] earrings for women; ear-ornaments [3] around; round; surround; surrounding [4] link [5] ring (sports) [6] a Chinese family name
U+74B1.0	fUTF8	璱
U+74B1.0	fCNS11643	2-5D58
U+74B1.0	fBig5	ED6C
U+74B1.0	fGB	5-4267
U+74B1.0	fR/S	 96.13
U+74B1.0	fTotalStrokes	 17
U+74B1.0	fCangjie	MGMGH
U+74B1.0	fFourCorner	11131
U+74B1.0	fHakka	 sit7
U+74B1.0	fCantonese	sat1
U+74B1.0	fMandarin	se4
U+74B1.0	fEnglish	the lustre of a gem
U+74B2.0	fUTF8	璲
U+74B2.0	fCNS11643	2-5D53
U+74B2.0	fBig5	ED67
U+74B2.0	fGB	3-3F7D
U+74B2.0	fR/S	 96.13
U+74B2.0	fTotalStrokes	 17
U+74B2.0	fCangjie	MGYTO
U+74B2.0	fFourCorner	18133
U+74B2.0	fHakka	 sui5; sui6; sui3
U+74B2.0	fCantonese	seoi6
U+74B2.0	fMandarin	sui4
U+74B2.0	fEnglish	pendant girdle ornament
U+74B3.0	fUTF8	璳
U+74B3.0	fCNS11643	3-554D
U+74B3.0	fGB	E-3357
U+74B3.0	fR/S	  96.13
U+74B3.0	fTotalStrokes	 17
U+74B3.0	fCangjie	MGHDC
U+74B3.0	fFourCorner	15180
U+74B3.0	fCantonese	tin4
U+74B3.0	fMandarin	tian2
U+74B4.0	fUTF8	璴
U+74B4.0	fCNS11643	3-6569
U+74B4.0	fGB	E-3358
U+74B4.0	fR/S	 96.13
U+74B4.0	fTotalStrokes	 17
U+74B4.0	fCangjie	MGDDO
U+74B4.0	fFourCorner	1418
U+74B4.0	fCantonese	co2
U+74B5.0	fUTF8	璵
U+74B5.0	fCNS11643	2-622F
U+74B5.0	fBig5	F042
U+74B5.0	fGB	3-3E7A
U+74B5.0	fR/S	 96.14
U+74B5.0	fTotalStrokes	 18
U+74B5.0	fCangjie	MGHXC
U+74B5.0	fFourCorner	17181
U+74B5.0	fHakka	 ji2; ri2
U+74B5.0	fCantonese	jyu4
U+74B5.0	fMandarin	yu2
U+74B5.0	fEnglish	fine jade
U+74B6.0	fUTF8	璶
U+74B6.0	fCNS11643	2-6232
U+74B6.0	fBig5	F045
U+74B6.0	fGB	5-4234
U+74B6.0	fR/S	 96.14
U+74B6.0	fTotalStrokes	 18
U+74B6.0	fCangjie	MGLMT
U+74B6.0	fFourCorner	15117
U+74B6.0	fHakka	 zin5
U+74B6.0	fCantonese	zeon6
U+74B6.0	fMandarin	jin4
U+74B7.0	fUTF8	璷
U+74B7.0	fCNS11643	2-6621
U+74B7.0	fBig5	F275
U+74B7.0	fGB	3-4026
U+74B7.0	fR/S	 96.14; 96.15
U+74B7.0	fTotalStrokes	 18
U+74B7.0	fCangjie	MGISK
U+74B7.0	fFourCorner	18140
U+74B7.0	fCantonese	leoi4; lou4; fu1
U+74B7.0	fMandarin	lu2; fu1
U+74B8.0	fUTF8	璸
U+74B8.0	fCNS11643	2-622D
U+74B8.0	fBig5	F040
U+74B8.0	fGB	3-3F71
U+74B8.0	fR/S	 96.14
U+74B8.0	fTotalStrokes	 18
U+74B8.0	fCangjie	MGJMC
U+74B8.0	fFourCorner	13186
U+74B8.0	fCantonese	ban1
U+74B8.0	fMandarin	bin1; pian2
U+74B8.0	fEnglish	jade's streaks
U+74B9.0	fUTF8	璹
U+74B9.0	fCNS11643	3-582F
U+74B9.0	fGB	3-3F47
U+74B9.0	fR/S	  96.14
U+74B9.0	fTotalStrokes	 18
U+74B9.0	fCangjie	MGGNI
U+74B9.0	fFourCorner	14140
U+74B9.0	fCantonese	sau6; suk6
U+74B9.0	fMandarin	shou4; shu2; shu4; dao4
U+74BA.0	fUTF8	璺
U+74BA.0	fCNS11643	2-693E
U+74BA.0	fBig5	F46F
U+74BA.0	fGB	0-6837
U+74BA.0	fR/S	 96.14
U+74BA.0	fTotalStrokes	 19
U+74BA.0	fCangjie	HBMGI
U+74BA.0	fFourCorner	77103
U+74BA.0	fHakka	  mun5; wun5
U+74BA.0	fCantonese	man6; man4
U+74BA.0	fMandarin	wen4; xin4
U+74BA.0	fEnglish	a crack in jade, porcelain, earthenware, or glassware
U+74BB.0	fUTF8	璻
U+74BB.0	fCNS11643	2-6233
U+74BB.0	fBig5	F046
U+74BB.0	fGB	5-426B
U+74BB.0	fR/S	 96.14
U+74BB.0	fTotalStrokes	 18
U+74BB.0	fCangjie	MGSMJ
U+74BB.0	fFourCorner	17148
U+74BB.0	fCantonese	zeoi2
U+74BB.0	fMandarin	zui3
U+74BC.0	fUTF8	璼
U+74BC.0	fCNS11643	3-5830
U+74BC.0	fGB	5-425B
U+74BC.0	fR/S	  96.14
U+74BC.0	fTotalStrokes	 18
U+74BC.0	fCangjie	MGSIT
U+74BC.0	fFourCorner	18110
U+74BC.0	fCantonese	laam4
U+74BC.0	fMandarin	lan2
U+74BD.0	fUTF8	璽
U+74BD.0	fCNS11643	1-784D
U+74BD.0	fBig5	C3A2
U+74BD.0	fGB	1-6774
U+74BD.0	fR/S	 96.14
U+74BD.0	fTotalStrokes	 19
U+74BD.0	fCangjie	MBMGI
U+74BD.0	fFourCorner	10103
U+74BD.0	fHakka	  sai3; se3
U+74BD.0	fCantonese	saai2
U+74BD.0	fMandarin	xi3
U+74BD.0	fEnglish	[1] the seal of an emperor or king [2] the formal seal of a state; national emblem
U+74BE.0	fUTF8	璾
U+74BE.0	fCNS11643	2-6231
U+74BE.0	fBig5	F044
U+74BE.0	fGB	3-3F3F
U+74BE.0	fR/S	 96.14
U+74BE.0	fTotalStrokes	 18
U+74BE.0	fCangjie	MGYX
U+74BE.0	fFourCorner	10123
U+74BE.0	fCantonese	zai6; cai5; zi1
U+74BE.0	fMandarin	ji4; zi1
U+74BF.0	fUTF8	璿
U+74BF.0	fCNS11643	1-7667
U+74BF.0	fBig5	C27B
U+74BF.0	fGB	E-3359
U+74BF.0	fR/S	 96.14
U+74BF.0	fTotalStrokes	 18
U+74BF.0	fCangjie	MGYBU
U+74BF.0	fFourCorner	11163
U+74BF.0	fHakka	  sien2; sion2
U+74BF.0	fCantonese	syun4
U+74BF.0	fMandarin	xuan2; gui4; rui4
U+74BF.0	fEnglish	fine jade
U+74C0.0	fUTF8	瓀
U+74C0.0	fCNS11643	2-622E
U+74C0.0	fBig5	F041
U+74C0.0	fGB	3-4025
U+74C0.0	fR/S	 96.14
U+74C0.0	fTotalStrokes	 18
U+74C0.0	fCangjie	MGMBB
U+74C0.0	fFourCorner	11127
U+74C0.0	fHakka	 ngion3; nion1
U+74C0.0	fCantonese	jyun5
U+74C0.0	fMandarin	ruan3; ruan2
U+74C0.0	fEnglish	gem
U+74C1.0	fUTF8	瓁
U+74C1.0	fCNS11643	2-6230
U+74C1.0	fBig5	F043
U+74C1.0	fGB	3-4021
U+74C1.0	fR/S	 96.14
U+74C1.0	fTotalStrokes	 18
U+74C1.0	fCangjie	MGTOE
U+74C1.0	fFourCorner	14147
U+74C1.0	fCantonese	wok6
U+74C1.0	fMandarin	huo4
U+74C2.0	fUTF8	瓂
U+74C2.0	fCNS11643	2-6234
U+74C2.0	fBig5	F047
U+74C2.0	fGB	E-335A
U+74C2.0	fR/S	 96.14
U+74C2.0	fTotalStrokes	 18
U+74C2.0	fCangjie	MGTGT
U+74C2.0	fFourCorner	14117
U+74C2.0	fCantonese	koi3; goi3
U+74C2.0	fMandarin	gai4
U+74C3.0	fUTF8	瓃
U+74C3.0	fCNS11643	2-6622
U+74C3.0	fBig5	F276
U+74C3.0	fGB	5-426C
U+74C3.0	fR/S	 96.15
U+74C3.0	fTotalStrokes	 19
U+74C3.0	fCangjie	MGWWW
U+74C3.0	fFourCorner	16160
U+74C3.0	fCantonese	leoi4; leoi6
U+74C3.0	fMandarin	lei2; lei4
U+74C4.0	fUTF8	瓄
U+74C4.0	fCNS11643	3-5A5A
U+74C4.0	fGB	3-3F54
U+74C4.0	fR/S	  96.15
U+74C4.0	fTotalStrokes	 19
U+74C4.0	fCangjie	MGGWC
U+74C4.0	fFourCorner	14180
U+74C4.0	fHakka	 tuk8
U+74C4.0	fCantonese	duk6
U+74C4.0	fMandarin	du2
U+74C4.0	fEnglish	a white jade fromm the Khun-lun mountains
U+74C5.0	fUTF8	瓅
U+74C5.0	fCNS11643	2-657E
U+74C5.0	fBig5	F274
U+74C5.0	fGB	5-4227
U+74C5.0	fR/S	 96.15
U+74C5.0	fTotalStrokes	 19
U+74C5.0	fCangjie	MGVID
U+74C5.0	fFourCorner	12194
U+74C5.0	fHakka	  lit8
U+74C5.0	fCantonese	lik1; lik6
U+74C5.0	fMandarin	li4
U+74C5.0	fEnglish	the lustre of pearls
U+74C6.0	fUTF8	瓆
U+74C6.0	fCNS11643	4-6027
U+74C6.0	fGB	E-335B
U+74C6.0	fR/S	  96.15
U+74C6.0	fTotalStrokes	 19
U+74C6.0	fCangjie	MGHLC
U+74C6.0	fFourCorner	12180
U+74C6.0	fCantonese	zat1
U+74C6.0	fMandarin	zhi2
U+74C7.0	fUTF8	瓇
U+74C7.0	fCNS11643	3-5A5C
U+74C7.0	fGB	E-335C
U+74C7.0	fR/S	  96.15
U+74C7.0	fTotalStrokes	 19
U+74C7.0	fCangjie	MGMBE
U+74C7.0	fFourCorner	11140
U+74C7.0	fCantonese	jau4
U+74C7.0	fMandarin	rou2
U+74C8.0	fUTF8	瓈
U+74C8.0	fCNS11643	3-5A5D
U+74C8.0	fGB	E-335D
U+74C8.0	fR/S	  96.15
U+74C8.0	fTotalStrokes	 19
U+74C8.0	fCangjie	MGHHE
U+74C8.0	fFourCorner	17130
U+74C8.0	fHakka	 li2; li3
U+74C8.0	fCantonese	lei4; lei1 (also)
U+74C8.0	fMandarin	li2
U+74C8.0	fEnglish	glass
U+74C9.0	fUTF8	瓉
U+74C9.0	fCNS11643	3-5A5B
U+74C9.0	fGB	5-4262
U+74C9.0	fR/S	  96.15
U+74C9.0	fTotalStrokes	 19
U+74C9.0	fCangjie	MGQOC
U+74C9.0	fFourCorner	15180
U+74C9.0	fHakka	 zan5
U+74C9.0	fCantonese	zaan3
U+74C9.0	fMandarin	zan4
U+74CA.0	fUTF8	瓊
U+74CA.0	fCNS11643	1-784E
U+74CA.0	fBig5	C3A3
U+74CA.0	fGB	1-476D
U+74CA.0	fR/S	 96.15
U+74CA.0	fTotalStrokes	 19
U+74CA.0	fCangjie	MGNBE
U+74CA.0	fFourCorner	17147
U+74CA.0	fHakka	  kiun2; kiun3
U+74CA.0	fCantonese	king4
U+74CA.0	fMandarin	qiong2; xuan2
U+74CA.0	fEnglish	[1] fine jade; agate [2] excellent; beautiful; elegant; fabulous
U+74CB.0	fUTF8	瓋
U+74CB.0	fCNS11643	2-657D
U+74CB.0	fBig5	F273
U+74CB.0	fGB	E-335E
U+74CB.0	fR/S	 96.15
U+74CB.0	fTotalStrokes	 19
U+74CB.0	fCangjie	MGYYB
U+74CB.0	fFourCorner	10132
U+74CB.0	fCantonese	tik1
U+74CB.0	fMandarin	zhe2; zhai1
U+74CC.0	fUTF8	瓌
U+74CC.0	fCNS11643	3-5C5E
U+74CC.0	fGB	E-335F
U+74CC.0	fR/S	 96.16
U+74CC.0	fTotalStrokes	 20
U+74CC.0	fCangjie	MGYWV
U+74CC.0	fFourCorner	10130
U+74CC.0	fCantonese	gwai1
U+74CC.0	fMandarin	gui1
U+74CC.0	fEnglish	extraordinary; fabulous
U+74CD.0	fUTF8	瓍
U+74CD.0	fCNS11643	3-5C5C
U+74CD.0	fGB	E-3360
U+74CD.0	fR/S	 96.16
U+74CD.0	fTotalStrokes	 20
U+74CD.0	fCangjie	MGNLB
U+74CD.0	fFourCorner	14130
U+74CD.0	fCantonese	ceoi4
U+74CD.0	fMandarin	sui4
U+74CE.0	fUTF8	瓎
U+74CE.0	fCNS11643	3-5C5F
U+74CE.0	fGB	E-3361
U+74CE.0	fR/S	  96.16
U+74CE.0	fTotalStrokes	 20
U+74CE.0	fCangjie	MGDLC
U+74CE.0	fFourCorner	17180
U+74CE.0	fCantonese	laat6
U+74CE.0	fMandarin	la4
U+74CF.0	fUTF8	瓏
U+74CF.0	fCNS11643	1-7979
U+74CF.0	fBig5	C46E
U+74CF.0	fGB	1-6767
U+74CF.0	fR/S	 96.16
U+74CF.0	fTotalStrokes	 20
U+74CF.0	fCangjie	MGYBP
U+74CF.0	fFourCorner	11111
U+74CF.0	fHakka	 lung2; lun3
U+74CF.0	fCantonese	lung4
U+74CF.0	fMandarin	long2
U+74CF.0	fEnglish	[1] a clear and crisp sound [2] dry; parched [3] the rumbling of a cart [4] tinkling of metals or jades [5] dusky
U+74D0.0	fUTF8	瓐
U+74D0.0	fCNS11643	3-5C5D
U+74D0.0	fGB	E-3362
U+74D0.0	fR/S	 96.16
U+74D0.0	fTotalStrokes	 20
U+74D0.0	fCangjie	MGYPT
U+74D0.0	fFourCorner	11110
U+74D0.0	fHakka	 lu2
U+74D0.0	fCantonese	lou4
U+74D0.0	fMandarin	lu2
U+74D1.0	fUTF8	瓑
U+74D1.0	fCNS11643	3-5C60
U+74D1.0	fGB	5-4175
U+74D1.0	fR/S	  96.16
U+74D1.0	fTotalStrokes	 20
U+74D1.0	fCangjie	MGMHM
U+74D1.0	fFourCorner	11110
U+74D1.0	fCantonese	lik6
U+74D1.0	fMandarin	li4
U+74D2.0	fUTF8	瓒
U+74D2.0	fGB	0-6836
U+74D2.0	fR/S	 96.16
U+74D2.0	fTotalStrokes	 20
U+74D2.0	fCangjie	MGHUO
U+74D2.0	fHakka	 zan5
U+74D2.0	fCantonese	zaan3
U+74D2.0	fMandarin	zan4
U+74D2.0	fEnglish	simplified form of &#x74DA;
U+74D3.0	fUTF8	瓓
U+74D3.0	fCNS11643	3-5E3D
U+74D3.0	fGB	3-3F7B
U+74D3.0	fR/S	 96.17
U+74D3.0	fTotalStrokes	 21
U+74D3.0	fCangjie	MGANW
U+74D3.0	fFourCorner	17120
U+74D3.0	fHakka	 lan5; lan6
U+74D3.0	fCantonese	laan6
U+74D3.0	fMandarin	lan4
U+74D3.0	fEnglish	the lustre of a gem
U+74D4.0	fUTF8	瓔
U+74D4.0	fCNS11643	1-7A78
U+74D4.0	fBig5	C4ED
U+74D4.0	fGB	1-682C
U+74D4.0	fR/S	 96.17
U+74D4.0	fTotalStrokes	 21
U+74D4.0	fCangjie	MGBCV
U+74D4.0	fFourCorner	16144
U+74D4.0	fHakka	 jin1; rin1
U+74D4.0	fCantonese	jing1
U+74D4.0	fMandarin	ying1
U+74D4.0	fEnglish	a necklace made of precious stones
U+74D5.0	fUTF8	瓕
U+74D5.0	fCNS11643	2-6D5F
U+74D5.0	fBig5	F6F1
U+74D5.0	fGB	3-4029
U+74D5.0	fR/S	 96.17
U+74D5.0	fTotalStrokes	 22
U+74D5.0	fCangjie	NBMGI
U+74D5.0	fFourCorner	11107
U+74D5.0	fCantonese	mei4; saai2
U+74D5.0	fMandarin	mi2; xian3; xi3
U+74D6.0	fUTF8	瓖
U+74D6.0	fCNS11643	1-7A77
U+74D6.0	fBig5	C4EC
U+74D6.0	fGB	5-426F
U+74D6.0	fR/S	 96.17
U+74D6.0	fTotalStrokes	 21
U+74D6.0	fCangjie	MGYRV
U+74D6.0	fFourCorner	10132
U+74D6.0	fHakka	 siong1
U+74D6.0	fCantonese	soeng1
U+74D6.0	fMandarin	xiang1
U+74D6.0	fEnglish	[1] same as $ [2] extraordinary; fabulous; admirable
U+74D7.0	fUTF8	瓗
U+74D7.0	fCNS11643	2-6D61
U+74D7.0	fBig5	F6F3
U+74D7.0	fGB	5-4270
U+74D7.0	fR/S	 96.18
U+74D7.0	fTotalStrokes	 22
U+74D7.0	fCangjie	MGUOB
U+74D7.0	fFourCorner	12127
U+74D7.0	fCantonese	king4; kwai4
U+74D7.0	fMandarin	xi1; qiong2; wei4; gui4; zui1
U+74D8.0	fUTF8	瓘
U+74D8.0	fCNS11643	2-6D5E
U+74D8.0	fBig5	F6F0
U+74D8.0	fGB	3-4027
U+74D8.0	fR/S	 96.18
U+74D8.0	fTotalStrokes	 22
U+74D8.0	fCangjie	MGTRG
U+74D8.0	fFourCorner	14114
U+74D8.0	fHakka	 kwan5; gon5; gwon5
U+74D8.0	fCantonese	gun3; gwun3 (also)
U+74D8.0	fMandarin	guan4
U+74D8.0	fEnglish	jade
U+74D9.0	fUTF8	瓙
U+74D9.0	fCNS11643	2-6D60
U+74D9.0	fBig5	F6F2
U+74D9.0	fGB	E-3363
U+74D9.0	fR/S	 96.18
U+74D9.0	fTotalStrokes	 22
U+74D9.0	fCangjie	MGGNF
U+74D9.0	fFourCorner	14141
U+74D9.0	fCantonese	dou6
U+74D9.0	fMandarin	dao4
U+74DA.0	fUTF8	瓚
U+74DA.0	fCNS11643	1-7C3C
U+74DA.0	fBig5	C5D0
U+74DA.0	fGB	1-6836
U+74DA.0	fR/S	 96.19
U+74DA.0	fTotalStrokes	 23
U+74DA.0	fCangjie	MGHUC
U+74DA.0	fFourCorner	14186
U+74DA.0	fHakka	  zan5
U+74DA.0	fCantonese	zaan3; zaan6
U+74DA.0	fMandarin	zan4
U+74DA.0	fEnglish	ceremonial libation cup
U+74DB.0	fUTF8	瓛
U+74DB.0	fCNS11643	2-7040
U+74DB.0	fBig5	F8B2
U+74DB.0	fGB	3-4022
U+74DB.0	fR/S	 96.20
U+74DB.0	fTotalStrokes	 24
U+74DB.0	fCangjie	MGYBK
U+74DB.0	fFourCorner	13134
U+74DB.0	fHakka	 fan2
U+74DB.0	fCantonese	wun4
U+74DB.0	fMandarin	huan2; xian4
U+74DB.0	fEnglish	scaptre
U+74DC.0	fUTF8	瓜
U+74DC.0	fCNS11643	1-466A
U+74DC.0	fBig5	A5CA
U+74DC.0	fGB	0-394F
U+74DC.0	fR/S	 97.0
U+74DC.0	fTotalStrokes	 5
U+74DC.0	fCangjie	HVIO
U+74DC.0	fFourCorner	72230
U+74DC.0	fHakka	  gwa1;  ga1; kua1
U+74DC.0	fCantonese	gwaa1
U+74DC.0	fMandarin	gua1
U+74DC.0	fEnglish	[1] melon; gourd; cucumber [2] dead [3] Kangxi radical 97
U+74DD.0	fUTF8	瓝
U+74DD.0	fCNS11643	2-2830
U+74DD.0	fBig5	CD6E
U+74DD.0	fGB	3-5C67
U+74DD.0	fR/S	 97.3
U+74DD.0	fTotalStrokes	 8
U+74DD.0	fCangjie	HOPI
U+74DD.0	fFourCorner	77232
U+74DD.0	fCantonese	bok6; paau4; bok3
U+74DD.0	fMandarin	bo2; bao2; pu2
U+74DE.0	fUTF8	瓞
U+74DE.0	fCNS11643	2-307D
U+74DE.0	fBig5	D2BC
U+74DE.0	fGB	0-702C
U+74DE.0	fR/S	 97.5
U+74DE.0	fTotalStrokes	 10
U+74DE.0	fCangjie	HOHQO
U+74DE.0	fFourCorner	75233
U+74DE.0	fHakka	  diet7; det7
U+74DE.0	fCantonese	dit6
U+74DE.0	fMandarin	die2
U+74DE.0	fEnglish	a kind of small melons
U+74DF.0	fUTF8	瓟
U+74DF.0	fCNS11643	2-307E
U+74DF.0	fBig5	D2BD
U+74DF.0	fGB	5-5B6B
U+74DF.0	fR/S	 97.5
U+74DF.0	fTotalStrokes	 10
U+74DF.0	fCangjie	HOPRU
U+74DF.0	fFourCorner	77230
U+74DF.0	fHakka	 pau2; pok8; pau3
U+74DF.0	fCantonese	bok6; paau4
U+74DF.0	fMandarin	bao2; bo2; pao2
U+74DF.0	fEnglish	the bottle-gourd
U+74E0.0	fUTF8	瓠
U+74E0.0	fCNS11643	1-5C24
U+74E0.0	fBig5	B27D
U+74E0.0	fGB	0-702D
U+74E0.0	fR/S	 97.6
U+74E0.0	fTotalStrokes	 11
U+74E0.0	fCangjie	KSHVO
U+74E0.0	fFourCorner	42230
U+74E0.0	fHakka	 pu2; fu2; pu3
U+74E0.0	fCantonese	wu6; wu4 (also); wok6
U+74E0.0	fMandarin	hu4; hu2; huo4; gu1
U+74E0.0	fEnglish	bottle gourd; calabash
U+74E1.0	fUTF8	瓡
U+74E1.0	fCNS11643	2-4525
U+74E1.0	fBig5	DEBF
U+74E1.0	fGB	5-5B6D
U+74E1.0	fR/S	 97.8
U+74E1.0	fTotalStrokes	 13
U+74E1.0	fCangjie	GJHVO
U+74E1.0	fFourCorner	42430
U+74E1.0	fCantonese	zap1
U+74E1.0	fMandarin	zhi2; gu1
U+74E2.0	fUTF8	瓢
U+74E2.0	fCNS11643	1-7147
U+74E2.0	fBig5	BF5D
U+74E2.0	fGB	0-4630
U+74E2.0	fR/S	 97.11
U+74E2.0	fTotalStrokes	 16
U+74E2.0	fCangjie	MFHVO
U+74E2.0	fFourCorner	12930
U+74E2.0	fHakka	  piau1; piau2; peu1; piau3
U+74E2.0	fCantonese	piu4
U+74E2.0	fMandarin	piao2
U+74E2.0	fEnglish	ladle made from dried gourd or calabash
U+74E3.0	fUTF8	瓣
U+74E3.0	fCNS11643	1-784F
U+74E3.0	fBig5	C3A4
U+74E3.0	fGB	0-306A
U+74E3.0	fR/S	 97.14
U+74E3.0	fTotalStrokes	 19
U+74E3.0	fCangjie	YJHOJ
U+74E3.0	fFourCorner	00441
U+74E3.0	fHakka	 pan5; pan3; ngiam5; pan6; pien6; pien5
U+74E3.0	fCantonese	faan6; baan6 (also); faan2 (also)
U+74E3.0	fMandarin	ban4; pan2; pian4
U+74E3.0	fEnglish	[1] petal (flower) [2] section; segment (as oranges) [3] valve; lamella [4] fragments; pieces
U+74E4.0	fUTF8	瓤
U+74E4.0	fCNS11643	1-7B67
U+74E4.0	fBig5	C57B
U+74E4.0	fGB	0-483F
U+74E4.0	fR/S	 97.17
U+74E4.0	fTotalStrokes	 22
U+74E4.0	fCangjie	YVHVO
U+74E4.0	fFourCorner	02710
U+74E4.0	fHakka	 nong2;  nong1; long2
U+74E4.0	fCantonese	nong4; long4 (also)
U+74E4.0	fMandarin	rang2; nang2
U+74E4.0	fEnglish	[1] pulp of a fruit; section of an orange, tanggerine, etc. [2] the interior part of certain things; core [3] inferior
U+74E5.0	fUTF8	瓥
U+74E5.0	fCNS11643	2-7041
U+74E5.0	fBig5	F8B3
U+74E5.0	fGB	E-3364
U+74E5.0	fR/S	 97.19
U+74E5.0	fTotalStrokes	 24
U+74E5.0	fCangjie	HOLII
U+74E5.0	fFourCorner	71136
U+74E5.0	fCantonese	lai6
U+74E5.0	fMandarin	li4
U+74E6.0	fUTF8	瓦
U+74E6.0	fCNS11643	1-466B
U+74E6.0	fBig5	A5CB
U+74E6.0	fGB	0-4D5F
U+74E6.0	fR/S	 98.0
U+74E6.0	fTotalStrokes	 5
U+74E6.0	fCangjie	MVNI
U+74E6.0	fFourCorner	10717
U+74E6.0	fHakka	  nga3; ngua3; ngwa3
U+74E6.0	fCantonese	ngaa5; aa5 (also); ngaa2 (also); aa2 (also); ngaa6
U+74E6.0	fMandarin	wa3; wei3
U+74E6.0	fEnglish	[1] tile [2] earthenware; pottery [3] Watt, the unit of power [4] a Chinese family name [5] Kangxi radical 98
U+74E6.1	fUTF8	瓦
U+74E6.1	fCNS11643	1-466B
U+74E6.1	fBig5	A5CB
U+74E6.1	fGB	0-4D5F
U+74E6.1	fR/S	 98.0
U+74E6.1	fTotalStrokes	 5
U+74E6.1	fCangjie	MVNI
U+74E6.1	fFourCorner	10717
U+74E6.1	fMandarin	wa4
U+74E6.1	fEnglish	 cover a roof with tiles; tile
U+74E7.0	fUTF8	瓧
U+74E7.0	fGB	E-3365
U+74E7.0	fR/S	 98.2
U+74E7.0	fTotalStrokes	 7
U+74E7.0	fCangjie	MNJ
U+74E7.0	fFourCorner	1471
U+74E7.0	fCantonese	sap6
U+74E7.0	fEnglish	decagram
U+74E8.0	fUTF8	瓨
U+74E8.0	fCNS11643	2-2831
U+74E8.0	fBig5	CD6F
U+74E8.0	fGB	5-354B
U+74E8.0	fR/S	 98.3
U+74E8.0	fTotalStrokes	 8
U+74E8.0	fCangjie	MMVN
U+74E8.0	fFourCorner	11117
U+74E8.0	fCantonese	gong1
U+74E8.0	fMandarin	jiang1; hong2; hang2
U+74E9.0	fUTF8	瓩
U+74E9.0	fCNS11643	1-2322
U+74E9.0	fBig5	A260
U+74E9.0	fGB	8-2877
U+74E9.0	fR/S	 98.3
U+74E9.0	fTotalStrokes	 8
U+74E9.0	fCangjie	MNHJ
U+74E9.0	fFourCorner	1271
U+74E9.0	fCantonese	cin1; ngaa5
U+74E9.0	fMandarin	qian1wa3
U+74E9.0	fEnglish	kilowatt
U+74EA.0	fUTF8	瓪
U+74EA.0	fCNS11643	4-2A3C
U+74EA.0	fGB	3-463E
U+74EA.0	fR/S	  98.4
U+74EA.0	fTotalStrokes	 9
U+74EA.0	fCangjie	MNHE
U+74EA.0	fFourCorner	11710
U+74EA.0	fHakka	  ban3
U+74EA.0	fCantonese	baan2
U+74EA.0	fMandarin	fan3; ban3
U+74EA.0	fEnglish	flat tiles
U+74EB.0	fUTF8	瓫
U+74EB.0	fCNS11643	3-2E22
U+74EB.0	fGB	5-4848
U+74EB.0	fR/S	  98.4
U+74EB.0	fTotalStrokes	 9
U+74EB.0	fCangjie	CSHN
U+74EB.0	fFourCorner	80710
U+74EB.0	fHakka	 wung3; pun2
U+74EB.0	fCantonese	pun4
U+74EB.0	fMandarin	pen2
U+74EC.0	fUTF8	瓬
U+74EC.0	fCNS11643	2-2C39
U+74EC.0	fBig5	CFD7
U+74EC.0	fGB	5-507E
U+74EC.0	fR/S	 98.4
U+74EC.0	fTotalStrokes	 9
U+74EC.0	fCangjie	YSMVN
U+74EC.0	fFourCorner	01217
U+74EC.0	fHakka	  fong3
U+74EC.0	fCantonese	fong2; ling4
U+74EC.0	fMandarin	fang3
U+74ED.0	fUTF8	瓭
U+74ED.0	fCNS11643	3-2E21
U+74ED.0	fGB	3-4640
U+74ED.0	fR/S	  98.4
U+74ED.0	fTotalStrokes	 9
U+74ED.0	fCangjie	LUMVN
U+74ED.0	fFourCorner	41010
U+74ED.0	fCantonese	dam2
U+74ED.0	fMandarin	dan3; chen2; dan1
U+74EE.0	fUTF8	瓮
U+74EE.0	fCNS11643	2-2C3A
U+74EE.0	fBig5	CFD8
U+74EE.0	fGB	0-4E4D
U+74EE.0	fR/S	 98.4
U+74EE.0	fTotalStrokes	 9
U+74EE.0	fCangjie	CIMN
U+74EE.0	fFourCorner	80717
U+74EE.0	fHakka	 wung5; vung5
U+74EE.0	fCantonese	ung3; ngung3 (also)
U+74EE.0	fMandarin	weng4
U+74EE.0	fEnglish	earthen jar
U+74EF.0	fUTF8	瓯
U+74EF.0	fCNS11643	3-2E24
U+74EF.0	fGB	0-6A31
U+74EF.0	fR/S	 98.4
U+74EF.0	fTotalStrokes	 9
U+74EF.0	fCangjie	SKMVN
U+74EF.0	fFourCorner	71710
U+74EF.0	fHakka	 eu1
U+74EF.0	fCantonese	au1; ngau1 (also)
U+74EF.0	fMandarin	ou1; ou3
U+74EF.0	fEnglish	simplified form of &#x750C;
U+74F0.0	fUTF8	瓰
U+74F0.0	fGB	E-3366
U+74F0.0	fR/S	 98.4
U+74F0.0	fTotalStrokes	 9
U+74F0.0	fCangjie	MNCSH
U+74F0.0	fFourCorner	1871
U+74F0.0	fCantonese	fan1
U+74F0.0	fEnglish	a decigram, one-tenth of a gram
U+74F1.0	fUTF8	瓱
U+74F1.0	fGB	E-3367
U+74F1.0	fR/S	 98.4
U+74F1.0	fTotalStrokes	 9
U+74F1.0	fCangjie	MNHQU
U+74F1.0	fFourCorner	1271
U+74F1.0	fCantonese	hou4
U+74F1.0	fEnglish	milligram
U+74F2.0	fUTF8	瓲
U+74F2.0	fGB	E-3368
U+74F2.0	fR/S	 98.4
U+74F2.0	fTotalStrokes	 9
U+74F2.0	fCangjie	MNPU
U+74F2.0	fCantonese	tyun4
U+74F3.0	fUTF8	瓳
U+74F3.0	fCNS11643	3-3248
U+74F3.0	fGB	3-4641
U+74F3.0	fR/S	  98.5
U+74F3.0	fTotalStrokes	 10
U+74F3.0	fCangjie	JRMVN
U+74F3.0	fFourCorner	41610
U+74F3.0	fCantonese	wu4
U+74F3.0	fMandarin	hu2
U+74F4.0	fUTF8	瓴
U+74F4.0	fCNS11643	2-3121
U+74F4.0	fBig5	D2BE
U+74F4.0	fGB	0-6A32
U+74F4.0	fR/S	 98.5
U+74F4.0	fTotalStrokes	 10
U+74F4.0	fCangjie	OIMVN
U+74F4.0	fFourCorner	81217
U+74F4.0	fHakka	  lin2
U+74F4.0	fCantonese	ling4
U+74F4.0	fMandarin	ling2
U+74F4.0	fEnglish	[1] the concave channels of tiling [2] a jar with a handle
U+74F5.0	fUTF8	瓵
U+74F5.0	fCNS11643	2-3122
U+74F5.0	fBig5	D2BF
U+74F5.0	fGB	5-484E
U+74F5.0	fR/S	 98.5
U+74F5.0	fTotalStrokes	 10
U+74F5.0	fCangjie	IRMVN
U+74F5.0	fFourCorner	21617
U+74F5.0	fCantonese	ji4
U+74F5.0	fMandarin	yi2
U+74F6.0	fUTF8	瓶
U+74F6.0	fCNS11643	1-5C25
U+74F6.0	fBig5	B27E
U+74F6.0	fGB	0-463F
U+74F6.0	fR/S	 98.6
U+74F6.0	fTotalStrokes	 11
U+74F6.0	fCangjie	TTMVN
U+74F6.0	fFourCorner	81417
U+74F6.0	fHakka	  piang2;  pin2;  pen2; piang3
U+74F6.0	fCantonese	ping4; ping2 (also); peng4
U+74F6.0	fMandarin	ping2
U+74F6.0	fEnglish	 jug; pitcher; vase; jar; bottle
U+74F7.0	fUTF8	瓷
U+74F7.0	fCNS11643	1-5C26
U+74F7.0	fBig5	B2A1
U+74F7.0	fGB	0-3449
U+74F7.0	fR/S	 98.6
U+74F7.0	fTotalStrokes	 11
U+74F7.0	fCangjie	IOMVN
U+74F7.0	fFourCorner	37717
U+74F7.0	fHakka	 cih2; cu2; ze2
U+74F7.0	fCantonese	ci4
U+74F7.0	fMandarin	ci2
U+74F7.0	fEnglish	porcelain; chinaware
U+74F8.0	fUTF8	瓸
U+74F8.0	fCNS11643	F-3667
U+74F8.0	fGB	E-3369
U+74F8.0	fR/S	 98.6
U+74F8.0	fTotalStrokes	 11
U+74F8.0	fCangjie	MNMA
U+74F8.0	fFourCorner	1171
U+74F8.0	fCantonese	baak3
U+74F8.0	fEnglish	hectogram
U+74F9.0	fUTF8	瓹
U+74F9.0	fCNS11643	3-3D3D
U+74F9.0	fGB	3-4650
U+74F9.0	fR/S	  98.7
U+74F9.0	fTotalStrokes	 12
U+74F9.0	fCangjie	RBMVN
U+74F9.0	fFourCorner	61210
U+74F9.0	fCantonese	gyun1
U+74F9.0	fMandarin	juan4
U+74F9.0	fEnglish	to hide quickly; to dash
U+74FA.0	fUTF8	瓺
U+74FA.0	fCNS11643	4-3925
U+74FA.0	fGB	3-4C24
U+74FA.0	fR/S	 98.7; 98.5; 168.5
U+74FA.0	fTotalStrokes	 12; 13
U+74FA.0	fCangjie	SIMVN
U+74FA.0	fFourCorner	71710
U+74FA.0	fCantonese	coeng4
U+74FA.0	fMandarin	chang2; zhang4; gan1
U+74FB.0	fUTF8	瓻
U+74FB.0	fCNS11643	2-3E2F
U+74FB.0	fBig5	DAAB
U+74FB.0	fGB	3-4651
U+74FB.0	fR/S	 98.7
U+74FB.0	fTotalStrokes	 12
U+74FB.0	fCangjie	KBMVN
U+74FB.0	fFourCorner	41217
U+74FB.0	fHakka	 hi1; di2
U+74FB.0	fCantonese	ci1
U+74FB.0	fMandarin	chi1; chi4
U+74FB.0	fEnglish	jar
U+74FC.0	fUTF8	瓼
U+74FC.0	fGB	E-336A
U+74FC.0	fR/S	 98.7
U+74FC.0	fTotalStrokes	 12
U+74FC.0	fCangjie	MNWG
U+74FC.0	fCantonese	lei5
U+74FD.0	fUTF8	瓽
U+74FD.0	fCNS11643	2-4528
U+74FD.0	fBig5	DEC2
U+74FD.0	fGB	3-4656
U+74FD.0	fR/S	 98.8
U+74FD.0	fTotalStrokes	 13
U+74FD.0	fCangjie	FBRMN
U+74FD.0	fFourCorner	90717
U+74FD.0	fHakka	 dong3; dong5
U+74FD.0	fCantonese	dong6; dong2; dong3
U+74FD.0	fMandarin	dang4; dang3
U+74FD.0	fEnglish	 large earthenware bowl
U+74FE.0	fUTF8	瓾
U+74FE.0	fCNS11643	2-4527
U+74FE.0	fBig5	DEC1
U+74FE.0	fGB	3-4659
U+74FE.0	fR/S	 98.8
U+74FE.0	fTotalStrokes	 13
U+74FE.0	fCangjie	HVMVN
U+74FE.0	fFourCorner	21417
U+74FE.0	fCantonese	waa1; maang5
U+74FE.0	fMandarin	meng3
U+74FF.0	fUTF8	瓿
U+74FF.0	fCNS11643	2-4526
U+74FF.0	fBig5	DEC0
U+74FF.0	fGB	0-6A33
U+74FF.0	fR/S	 98.8
U+74FF.0	fTotalStrokes	 13
U+74FF.0	fCangjie	YRMVN
U+74FF.0	fFourCorner	01617
U+74FF.0	fHakka	  piau3;  pu5; peu3
U+74FF.0	fCantonese	pau2; bau6; bou6
U+74FF.0	fMandarin	pou3; fu2; bu4; pou2; pu2
U+74FF.0	fEnglish	 jar; pot
U+7500.0	fUTF8	甀
U+7500.0	fCNS11643	2-4B7B
U+7500.0	fBig5	E2D5
U+7500.0	fGB	3-4658
U+7500.0	fR/S	 98.8
U+7500.0	fTotalStrokes	 13
U+7500.0	fCangjie	HMMVN
U+7500.0	fFourCorner	21117
U+7500.0	fCantonese	zeoi6
U+7500.0	fMandarin	zhui4; chui2
U+7500.0	fEnglish	jar
U+7501.0	fUTF8	甁
U+7501.0	fCNS11643	3-4277
U+7501.0	fGB	E-336B
U+7501.0	fR/S	 98.8
U+7501.0	fTotalStrokes	 13
U+7501.0	fCangjie	YJYJN
U+7501.0	fFourCorner	81410
U+7501.0	fCantonese	ping4
U+7501.0	fMandarin	ping2
U+7501.0	fEnglish	jug
U+7502.0	fUTF8	甂
U+7502.0	fCNS11643	2-4B7C
U+7502.0	fBig5	E2D6
U+7502.0	fGB	3-4665
U+7502.0	fR/S	 98.9
U+7502.0	fTotalStrokes	 14
U+7502.0	fCangjie	HBMVN
U+7502.0	fFourCorner	71237
U+7502.0	fCantonese	pin1; bin1
U+7502.0	fMandarin	bian1; pian1
U+7502.0	fEnglish	打甂爐, to instant-boil thin slices of meat and vegetables in a chafing dish at a dining table
U+7503.0	fUTF8	甃
U+7503.0	fCNS11643	2-4B7D
U+7503.0	fBig5	E2D7
U+7503.0	fGB	3-4662
U+7503.0	fR/S	 98.9
U+7503.0	fTotalStrokes	 14
U+7503.0	fCangjie	HFMVN
U+7503.0	fFourCorner	29717
U+7503.0	fHakka	  ziu5
U+7503.0	fCantonese	zau3
U+7503.0	fMandarin	zhou4
U+7503.0	fEnglish	[1] brick wall of a well; tile [2] build a wall [3] contract with bricks
U+7504.0	fUTF8	甄
U+7504.0	fCNS11643	1-6969
U+7504.0	fBig5	BAC2
U+7504.0	fGB	0-5567
U+7504.0	fR/S	 98.9
U+7504.0	fTotalStrokes	 14
U+7504.0	fCangjie	MGMVN
U+7504.0	fFourCorner	11117
U+7504.0	fHakka	 jen1; zhin1; ren1; zihn1; zin1
U+7504.0	fCantonese	jan1; zan1 (also)
U+7504.0	fMandarin	zhen1; zhen4; juan4; jian1; zhan1; quan4; ji1
U+7504.0	fEnglish	[1] potter's wheel [2] make pottery ware [3] examine; discern [4] grade (competence, etc.); by examination [5] make clear [6] a Chinese family name
U+7505.0	fUTF8	甅
U+7505.0	fGB	E-336C
U+7505.0	fR/S	 98.9
U+7505.0	fTotalStrokes	 14
U+7505.0	fCangjie	MNMWG
U+7505.0	fFourCorner	1171
U+7505.0	fCantonese	lei4
U+7505.0	fEnglish	centigram
U+7506.0	fUTF8	甆
U+7506.0	fCNS11643	3-4828
U+7506.0	fGB	5-4855
U+7506.0	fR/S	 98.9; 98.10
U+7506.0	fTotalStrokes	 14; 15
U+7506.0	fCangjie	TVIN
U+7506.0	fFourCorner	80717
U+7506.0	fCantonese	ci4
U+7506.0	fMandarin	ci2
U+7507.0	fUTF8	甇
U+7507.0	fCNS11643	2-5235
U+7507.0	fBig5	E6AD
U+7507.0	fGB	5-484A
U+7507.0	fR/S	 98.10
U+7507.0	fTotalStrokes	 15
U+7507.0	fCangjie	FFBMN
U+7507.0	fFourCorner	99717
U+7507.0	fCantonese	aang1; ngaang1 (also); ang1
U+7507.0	fMandarin	ying1
U+7507.0	fEnglish	jar
U+7508.0	fUTF8	甈
U+7508.0	fCNS11643	2-5234
U+7508.0	fBig5	E6AC
U+7508.0	fGB	3-4666
U+7508.0	fR/S	 98.10
U+7508.0	fTotalStrokes	 15
U+7508.0	fCangjie	HDMVN
U+7508.0	fFourCorner	21917
U+7508.0	fHakka	  ngi5;  hi5
U+7508.0	fCantonese	hei3
U+7508.0	fMandarin	qi4; yi4; qie4; gui4; nie4
U+7508.0	fEnglish	[1] large jar [2] burn crockery [3] crack
U+7509.0	fUTF8	甉
U+7509.0	fCNS11643	3-4D3D
U+7509.0	fGB	3-4668
U+7509.0	fR/S	 98.10
U+7509.0	fTotalStrokes	 15
U+7509.0	fCangjie	TCMVN
U+7509.0	fFourCorner	81310
U+7509.0	fCantonese	haam4
U+7509.0	fMandarin	xian2; xian4; zhan4; chan4
U+750A.0	fUTF8	甊
U+750A.0	fCNS11643	3-516D
U+750A.0	fGB	3-4664
U+750A.0	fR/S	  98.11
U+750A.0	fTotalStrokes	 16
U+750A.0	fCangjie	LVMVN
U+750A.0	fFourCorner	51410
U+750A.0	fHakka	  leu3
U+750A.0	fCantonese	lau5
U+750A.0	fMandarin	lou3
U+750A.0	fEnglish	long-necked jar
U+750B.0	fUTF8	甋
U+750B.0	fCNS11643	2-5855
U+750B.0	fBig5	EA69
U+750B.0	fGB	3-466B
U+750B.0	fR/S	 98.11
U+750B.0	fTotalStrokes	 16
U+750B.0	fCangjie	YBMVN
U+750B.0	fFourCorner	01217
U+750B.0	fHakka	  dit7
U+750B.0	fCantonese	dik1
U+750B.0	fMandarin	di4
U+750B.0	fEnglish	a jar without ears
U+750C.0	fUTF8	甌
U+750C.0	fCNS11643	1-7148
U+750C.0	fBig5	BF5E
U+750C.0	fGB	1-6A31
U+750C.0	fR/S	 98.11
U+750C.0	fTotalStrokes	 16
U+750C.0	fCangjie	SRMVN
U+750C.0	fFourCorner	71717
U+750C.0	fHakka	  eu1; iu1
U+750C.0	fCantonese	au1; ngau1 (also)
U+750C.0	fMandarin	ou1; ou3
U+750C.0	fEnglish	[1] bowl; cup [2] a Chinese family name
U+750D.0	fUTF8	甍
U+750D.0	fCNS11643	1-7149
U+750D.0	fBig5	BF5F
U+750D.0	fGB	0-5D79
U+750D.0	fR/S	 98.11
U+750D.0	fTotalStrokes	 16
U+750D.0	fCangjie	TWLN
U+750D.0	fFourCorner	44717
U+750D.0	fHakka	  men2
U+750D.0	fCantonese	mang4
U+750D.0	fMandarin	meng2; meng4; ming2
U+750D.0	fEnglish	rafters supporting roof tiles
U+750E.0	fUTF8	甎
U+750E.0	fCNS11643	3-516E
U+750E.0	fGB	E-336D
U+750E.0	fR/S	  98.11
U+750E.0	fTotalStrokes	 16
U+750E.0	fCangjie	JIMVN
U+750E.0	fFourCorner	51310
U+750E.0	fHakka	  zhon1; zon1
U+750E.0	fCantonese	zyun1
U+750E.0	fMandarin	zhuan1; chuan2
U+750E.0	fEnglish	brick
U+750F.0	fUTF8	甏
U+750F.0	fCNS11643	2-5D5E
U+750F.0	fBig5	ED72
U+750F.0	fGB	0-6A34
U+750F.0	fR/S	 98.12
U+750F.0	fTotalStrokes	 17
U+750F.0	fCangjie	GHMVN
U+750F.0	fFourCorner	42717
U+750F.0	fHakka	  pang5
U+750F.0	fCantonese	bong3; paang6 (also); bong6 (also); paang1
U+750F.0	fMandarin	beng4; peng4
U+750F.0	fEnglish	[1] a squat jar for holding wine; sauces etc. [2] earthen jar; altar
U+7510.0	fUTF8	甐
U+7510.0	fCNS11643	2-5D5B
U+7510.0	fBig5	ED6F
U+7510.0	fGB	3-466F
U+7510.0	fR/S	 98.12
U+7510.0	fTotalStrokes	 17
U+7510.0	fCangjie	FQMVN
U+7510.0	fFourCorner	91217
U+7510.0	fHakka	 lin2
U+7510.0	fCantonese	leon6; leon4
U+7510.0	fMandarin	lin2; lin4
U+7511.0	fUTF8	甑
U+7511.0	fCNS11643	2-5D5C
U+7511.0	fBig5	ED70
U+7511.0	fGB	0-6A35
U+7511.0	fR/S	 98.12
U+7511.0	fTotalStrokes	 17
U+7511.0	fCangjie	CAMVN
U+7511.0	fFourCorner	81617
U+7511.0	fHakka	  zen5
U+7511.0	fCantonese	zang6; zang3
U+7511.0	fMandarin	zeng4; ceng2
U+7511.0	fEnglish	a boiler for steaming rice; earthenware for cooking
U+7512.0	fUTF8	甒
U+7512.0	fCNS11643	2-5D5D
U+7512.0	fBig5	ED71
U+7512.0	fGB	3-402E
U+7512.0	fR/S	 98.12
U+7512.0	fTotalStrokes	 17
U+7512.0	fCangjie	OFMVN
U+7512.0	fFourCorner	81317
U+7512.0	fHakka	 wu3; vu3
U+7512.0	fCantonese	mou5
U+7512.0	fMandarin	wu3
U+7512.0	fEnglish	jar
U+7513.0	fUTF8	甓
U+7513.0	fCNS11643	2-6236
U+7513.0	fBig5	F049
U+7513.0	fGB	0-6A36
U+7513.0	fR/S	 98.13
U+7513.0	fTotalStrokes	 18
U+7513.0	fCangjie	SJMVN
U+7513.0	fFourCorner	70717
U+7513.0	fHakka	  pit7; piak7
U+7513.0	fCantonese	pik1; bik1
U+7513.0	fMandarin	pi4; bo4
U+7513.0	fEnglish	 tiles; brick
U+7514.0	fUTF8	甔
U+7514.0	fCNS11643	2-6235
U+7514.0	fBig5	F048
U+7514.0	fGB	3-4672
U+7514.0	fR/S	 98.13
U+7514.0	fTotalStrokes	 18
U+7514.0	fCangjie	NRMVN
U+7514.0	fFourCorner	21617
U+7514.0	fHakka	  dam1
U+7514.0	fCantonese	daam1; daam3
U+7514.0	fMandarin	dan1; dan4; dan3; chan4
U+7514.0	fEnglish	big jar
U+7515.0	fUTF8	甕
U+7515.0	fCNS11643	1-7668
U+7515.0	fBig5	C27C
U+7515.0	fGB	E-336E
U+7515.0	fR/S	 98.13
U+7515.0	fTotalStrokes	 18
U+7515.0	fCangjie	YVGN
U+7515.0	fFourCorner	00717
U+7515.0	fHakka	 wung5; vung5
U+7515.0	fCantonese	ung3; ngung3 (also); ong3
U+7515.0	fMandarin	weng4; yong1; yong3; yong4
U+7515.0	fEnglish	 earthen jar; jug; pot; urn
U+7516.0	fUTF8	甖
U+7516.0	fCNS11643	2-6623
U+7516.0	fBig5	F277
U+7516.0	fGB	E-336F
U+7516.0	fR/S	 98.14
U+7516.0	fTotalStrokes	 19
U+7516.0	fCangjie	BCMVN
U+7516.0	fFourCorner	66717
U+7516.0	fHakka	 ang1
U+7516.0	fCantonese	aang1; ngaang1 (also)
U+7516.0	fMandarin	ying1
U+7516.0	fEnglish	 pot; pitcher small-mouthed eart
U+7517.0	fUTF8	甗
U+7517.0	fCNS11643	2-6B6C
U+7517.0	fBig5	F5DE
U+7517.0	fGB	3-4674
U+7517.0	fR/S	 98.16
U+7517.0	fTotalStrokes	 21
U+7517.0	fCangjie	YBMVN
U+7517.0	fFourCorner	21217
U+7517.0	fHakka	 hien3; hien5
U+7517.0	fCantonese	jin5; jin2 (also)
U+7517.0	fMandarin	yan3; yan2; yan4
U+7517.0	fEnglish	an earthenware vessel in two parts, of which one is for steaming and the other is for boiling; ancient double boiler
U+7518.0	fUTF8	甘
U+7518.0	fCNS11643	1-466C
U+7518.0	fBig5	A5CC
U+7518.0	fGB	0-384A
U+7518.0	fR/S	 99.0
U+7518.0	fTotalStrokes	 5
U+7518.0	fCangjie	TM
U+7518.0	fFourCorner	44770
U+7518.0	fHakka	  gam1
U+7518.0	fCantonese	gam1
U+7518.0	fMandarin	gan1; gan4
U+7518.0	fEnglish	[1] sweetness; sweet [2] tasty; luscious; delicious [3] willing [4] enjoy [5] pleasant; pleasing [6] a Chinese family name [7] Kangxi radical 99
U+7519.0	fUTF8	甙
U+7519.0	fGB	0-5F30
U+7519.0	fR/S	 56.5; 99.3
U+7519.0	fTotalStrokes	 8
U+7519.0	fCangjie	IPTM
U+7519.0	fCantonese	doi6
U+7519.0	fMandarin	dai4
U+7519.0	fEnglish	sugar product
U+751A.0	fUTF8	甚
U+751A.0	fCNS11643	1-5249
U+751A.0	fBig5	ACC6
U+751A.0	fGB	0-4975
U+751A.0	fR/S	 99.4
U+751A.0	fTotalStrokes	 9
U+751A.0	fCangjie	TMMV
U+751A.0	fFourCorner	44711
U+751A.0	fHakka	 sim3; shim6; shim3; sim5; siim5; siim3; shim5
U+751A.0	fCantonese	sam6; sap6
U+751A.0	fMandarin	shen4; shi2
U+751A.0	fEnglish	[1] to a great extent; considerably; very; exceedingly; to a high degree [2] more than
U+751A.1	fUTF8	甚
U+751A.1	fCNS11643	1-5249
U+751A.1	fBig5	ACC6
U+751A.1	fGB	0-4975
U+751A.1	fR/S	 99.4
U+751A.1	fTotalStrokes	 9
U+751A.1	fCangjie	TMMV
U+751A.1	fFourCorner	44711
U+751A.1	fMandarin	she2; shen2
U+751A.1	fEnglish	what
U+751B.0	fUTF8	甛
U+751B.0	fCNS11643	3-3744
U+751B.0	fGB	5-576E
U+751B.0	fR/S	 99.6; .6
U+751B.0	fTotalStrokes	 11
U+751B.0	fCangjie	TMHJR
U+751B.0	fFourCorner	42760
U+751B.0	fCantonese	tim4
U+751B.0	fMandarin	tian2
U+751B.0	fEnglish	sweet; sweetness
U+751C.0	fUTF8	甜
U+751C.0	fCNS11643	1-5C27
U+751C.0	fBig5	B2A2
U+751C.0	fGB	0-4C70
U+751C.0	fR/S	 99.6
U+751C.0	fTotalStrokes	 11
U+751C.0	fCangjie	HRTM
U+751C.0	fFourCorner	24670
U+751C.0	fHakka	  tiam2; tiam3
U+751C.0	fCantonese	tim4; tim2 (also)
U+751C.0	fMandarin	tian2
U+751C.0	fEnglish	[1] sweet; sweetness; luscious [2] agreeable; pleasant
U+751D.0	fUTF8	甝
U+751D.0	fCNS11643	2-4529
U+751D.0	fBig5	DEC3
U+751D.0	fGB	3-614D
U+751D.0	fR/S	 99.8
U+751D.0	fTotalStrokes	 13
U+751D.0	fCangjie	YUTM
U+751D.0	fFourCorner	24217
U+751D.0	fCantonese	hon4
U+751D.0	fMandarin	han1; han2
U+751E.0	fUTF8	甞
U+751E.0	fCNS11643	3-4278
U+751E.0	fGB	E-3370
U+751E.0	fR/S	 99.8; .8
U+751E.0	fTotalStrokes	 13
U+751E.0	fCangjie	FBRTM
U+751E.0	fFourCorner	90770
U+751E.0	fHakka	 song2;  shong2
U+751E.0	fCantonese	soeng4
U+751E.0	fMandarin	chang2
U+751E.0	fEnglish	 taste experience; experiment
U+751F.0	fUTF8	生
U+751F.0	fCNS11643	1-466D
U+751F.0	fBig5	A5CD
U+751F.0	fGB	0-497A
U+751F.0	fR/S	 100.0
U+751F.0	fTotalStrokes	 5
U+751F.0	fCangjie	HQM
U+751F.0	fFourCorner	25100
U+751F.0	fHakka	  sang1;  sen1; sien1
U+751F.0	fCantonese	sang1; saang1 (also)
U+751F.0	fMandarin	sheng1; sheng4; sheng3
U+751F.0	fEnglish	[1] live; life; living; lifetime; livelihood; alive [2] be born; come into being; come into existence; birth [3] breed; bear; produce; beget; create; give rise to; cause [4] uncooked; raw; unripe; crude [5] unfamiliar; strange; unknown [6] save; untamed; barbarian; uncultured [7] pupil; student [8] the male character type in a Chinese opera [9] creatures [10] a Chinese family name [11] Kangxi radical 100
U+751F.1	fUTF8	生
U+751F.1	fCNS11643	1-466D
U+751F.1	fBig5	A5CD
U+751F.1	fGB	0-497A
U+751F.1	fR/S	 100.0
U+751F.1	fTotalStrokes	 5
U+751F.1	fCangjie	HQM
U+751F.1	fFourCorner	25100
U+751F.1	fMandarin	sheng5
U+7520.0	fUTF8	甠
U+7520.0	fCNS11643	4-2A40
U+7520.0	fGB	E-3371
U+7520.0	fR/S	 100.4
U+7520.0	fTotalStrokes	 9
U+7520.0	fCangjie	AHQM
U+7520.0	fFourCorner	65010
U+7520.0	fCantonese	cing4
U+7520.0	fMandarin	qing2
U+7521.0	fUTF8	甡
U+7521.0	fCNS11643	2-3123
U+7521.0	fBig5	D2C0
U+7521.0	fGB	3-5B48
U+7521.0	fR/S	 100.5
U+7521.0	fTotalStrokes	 10
U+7521.0	fCangjie	HMHQM
U+7521.0	fFourCorner	25110
U+7521.0	fHakka	  sin1
U+7521.0	fCantonese	san1
U+7521.0	fMandarin	shen1; sheng1
U+7521.0	fEnglish	[1] abundant; numerous; many [2] crowded
U+7522.0	fUTF8	產
U+7522.0	fCNS11643	1-5C28
U+7522.0	fBig5	B2A3
U+7522.0	fGB	E-3372
U+7522.0	fR/S	 100.6
U+7522.0	fTotalStrokes	 11
U+7522.0	fCangjie	YKMHM
U+7522.0	fFourCorner	00214
U+7522.0	fHakka	  san3; can3
U+7522.0	fCantonese	caan2
U+7522.0	fMandarin	chan3; shan4
U+7522.0	fEnglish	[1] bear (offspring); lay (eggs) [2] bring forth; produce; bring about
U+7523.0	fUTF8	産
U+7523.0	fCNS11643	3-3745
U+7523.0	fGB	1-327A
U+7523.0	fR/S	 100.6
U+7523.0	fTotalStrokes	 11
U+7523.0	fCangjie	YHHQM
U+7523.0	fFourCorner	00210
U+7523.0	fCantonese	caan2
U+7523.0	fMandarin	chan3
U+7523.0	fEnglish	give birth
U+7524.0	fUTF8	甤
U+7524.0	fCNS11643	3-3D3E
U+7524.0	fGB	5-6E41
U+7524.0	fR/S	 100.7
U+7524.0	fTotalStrokes	 12
U+7524.0	fCangjie	MOHQM
U+7524.0	fFourCorner	15230
U+7524.0	fHakka	  sui1; rui1; jui1
U+7524.0	fCantonese	jeoi6; jeoi4 (also)
U+7524.0	fMandarin	rui2; rui3
U+7524.0	fEnglish	[1] hat sashes hanging down [2] luxuriant vegetation
U+7525.0	fUTF8	甥
U+7525.0	fCNS11643	1-6069
U+7525.0	fBig5	B563
U+7525.0	fGB	0-497B
U+7525.0	fR/S	 100.7
U+7525.0	fTotalStrokes	 12
U+7525.0	fCangjie	HMWKS
U+7525.0	fFourCorner	26127
U+7525.0	fHakka	  sang1; sen1
U+7525.0	fCantonese	saang1 (also); sang1
U+7525.0	fMandarin	sheng1
U+7525.0	fEnglish	[1] sister's child; nephew [2] a son-in-law who assumes one's own name and live under one's own roof
U+7526.0	fUTF8	甦
U+7526.0	fCNS11643	1-606A
U+7526.0	fBig5	B564
U+7526.0	fGB	8-2C75
U+7526.0	fR/S	 100.7
U+7526.0	fTotalStrokes	 12
U+7526.0	fCangjie	MKHQM
U+7526.0	fFourCorner	15501
U+7526.0	fHakka	 su1; se1; sii1
U+7526.0	fCantonese	sou1
U+7526.0	fMandarin	su1
U+7526.0	fEnglish	 be reborn; come back to life; rise from the deadly resuscitate; revive; resurrect; regain consciousness
U+7527.0	fUTF8	甧
U+7527.0	fCNS11643	5-4749
U+7527.0	fGB	E-3373
U+7527.0	fR/S	  100.9
U+7527.0	fTotalStrokes	 14
U+7527.0	fCangjie	HMB
U+7527.0	fFourCorner	25220
U+7527.0	fCantonese	sam1
U+7527.0	fMandarin	sen1
U+7528.0	fUTF8	用
U+7528.0	fCNS11643	1-466E
U+7528.0	fBig5	A5CE
U+7528.0	fGB	0-5343
U+7528.0	fR/S	 101.0
U+7528.0	fTotalStrokes	 5
U+7528.0	fCangjie	BQ
U+7528.0	fFourCorner	77220
U+7528.0	fHakka	  jung5; jung3; rung6; jung6
U+7528.0	fCantonese	jung6; jung2 (also)
U+7528.0	fMandarin	yong4
U+7528.0	fEnglish	[1] use; employ; apply; operate; use [2] exert [3] use [4] effect [5] finance [6] need [7] eat; drink [8] Kangxi radical 101
U+7528.1	fUTF8	用
U+7528.1	fCNS11643	1-466E
U+7528.1	fBig5	A5CE
U+7528.1	fGB	0-5343
U+7528.1	fR/S	 101.0
U+7528.1	fTotalStrokes	 5
U+7528.1	fCangjie	BQ
U+7528.1	fFourCorner	77220
U+7528.1	fMandarin	yong5
U+7529.0	fUTF8	甩
U+7529.0	fCNS11643	1-466F
U+7529.0	fBig5	A5CF
U+7529.0	fGB	0-4B26
U+7529.0	fR/S	 101.0
U+7529.0	fTotalStrokes	 5
U+7529.0	fCangjie	BQU
U+7529.0	fFourCorner	77212
U+7529.0	fHakka	 lot7; sut7
U+7529.0	fCantonese	lat1; seot1
U+7529.0	fMandarin	shuai3
U+7529.0	fEnglish	[1] throw away; discard; cast away [2] leave (somebody) behind [3] swing
U+752A.0	fUTF8	甪
U+752A.0	fCNS11643	2-2266
U+752A.0	fBig5	CA46
U+752A.0	fGB	3-302C
U+752A.0	fR/S	 101.1
U+752A.0	fTotalStrokes	 6
U+752A.0	fCangjie	HBQ
U+752A.0	fFourCorner	27220
U+752A.0	fHakka	 luk8
U+752A.0	fCantonese	luk6
U+752A.0	fMandarin	lu4
U+752A.0	fEnglish	to loose, take off, get rid of
U+752B.0	fUTF8	甫
U+752B.0	fCNS11643	1-4B2D
U+752B.0	fBig5	A86A
U+752B.0	fGB	0-3826
U+752B.0	fR/S	 101.2
U+752B.0	fTotalStrokes	 7
U+752B.0	fCangjie	IJB
U+752B.0	fFourCorner	53227
U+752B.0	fHakka	  pu3
U+752B.0	fCantonese	fu2; pou2; pou3; pou1 (also)
U+752B.0	fMandarin	fu3; bu3; pu4; pu3
U+752B.0	fEnglish	[1] man (euphemism) [2] father (euphemism)
U+752C.0	fUTF8	甬
U+752C.0	fCNS11643	1-4B2C
U+752C.0	fBig5	A869
U+752C.0	fGB	0-702E
U+752C.0	fR/S	 101.2
U+752C.0	fTotalStrokes	 7
U+752C.0	fCangjie	NIBQ
U+752C.0	fFourCorner	17227
U+752C.0	fHakka	 jung3; rung3; jung1
U+752C.0	fCantonese	jung2; jung5
U+752C.0	fMandarin	yong3; dong4
U+752C.0	fEnglish	[1] another name for Ningbo, Chekiang [2] a measure of capacity
U+752D.0	fUTF8	甭
U+752D.0	fCNS11643	1-524A
U+752D.0	fBig5	ACC7
U+752D.0	fGB	0-3142
U+752D.0	fR/S	 101.4
U+752D.0	fTotalStrokes	 9
U+752D.0	fCangjie	MFBQ
U+752D.0	fFourCorner	10227
U+752D.0	fCantonese	bung2; bung3 (also); bang2; bat1
U+752D.0	fMandarin	beng2
U+752D.0	fEnglish	unnecessary; there is no need; do not have to
U+752E.0	fUTF8	甮
U+752E.0	fCNS11643	2-2C3B
U+752E.0	fBig5	CFD9
U+752E.0	fGB	3-302D
U+752E.0	fR/S	 20.7; 101.4
U+752E.0	fTotalStrokes	 9
U+752E.0	fCangjie	PHBQ
U+752E.0	fFourCorner	27227
U+752E.0	fCantonese	fung6; mung6 (also)
U+752E.0	fMandarin	feng4; fong4; fung4
U+752F.0	fUTF8	甯
U+752F.0	fCNS11643	2-3E30
U+752F.0	fBig5	DAAC
U+752F.0	fGB	0-6538
U+752F.0	fR/S	 101.7
U+752F.0	fTotalStrokes	 12
U+752F.0	fCangjie	JPBQ
U+752F.0	fFourCorner	30227
U+752F.0	fHakka	 nin5; nen6; len2; nin6
U+752F.0	fCantonese	ning6; ling4 (also); ling6 (also)
U+752F.0	fEnglish	a Chinese family name
U+752F.1	fUTF8	甯
U+752F.1	fCNS11643	2-3E30
U+752F.1	fBig5	DAAC
U+752F.1	fGB	0-6538
U+752F.1	fR/S	 101.7
U+752F.1	fTotalStrokes	 12
U+752F.1	fCangjie	JPBQ
U+752F.1	fFourCorner	30227
U+752F.1	fHakka	 nen2
U+752F.1	fCantonese	ning4
U+752F.1	fMandarin	ning2; ning4
U+752F.1	fEnglish	same as &#x5BE7;
U+7530.0	fUTF8	田
U+7530.0	fCNS11643	1-4670
U+7530.0	fBig5	A5D0
U+7530.0	fGB	0-4C6F
U+7530.0	fR/S	 102.0
U+7530.0	fTotalStrokes	 5
U+7530.0	fCangjie	W
U+7530.0	fFourCorner	60400
U+7530.0	fHakka	 tien2; ten2; t'ian3
U+7530.0	fCantonese	tin4; tin2 (also)
U+7530.0	fMandarin	tian2; tin2; dian4
U+7530.0	fEnglish	[1] agricultural land; field; cultivated land; arable land; rice field [2] hunt game [3] a Chinese family name [4] Kangxi radical 102
U+7531.0	fUTF8	由
U+7531.0	fCNS11643	1-4671
U+7531.0	fBig5	A5D1
U+7531.0	fGB	0-5349
U+7531.0	fR/S	 102.0
U+7531.0	fTotalStrokes	 5
U+7531.0	fCangjie	LW
U+7531.0	fFourCorner	50600
U+7531.0	fHakka	  ju2; jiu2; jeu2; jiu3; riu2
U+7531.0	fCantonese	jau4; jau2 (also)
U+7531.0	fMandarin	you2; yao1
U+7531.0	fEnglish	[1] cause; reason; source; derivation [2] from [3] up to [4] by; through [5] a Chinese family name
U+7532.0	fUTF8	甲
U+7532.0	fCNS11643	1-4672
U+7532.0	fBig5	A5D2
U+7532.0	fGB	0-3C57
U+7532.0	fR/S	 102.0
U+7532.0	fTotalStrokes	 5
U+7532.0	fCangjie	WL
U+7532.0	fFourCorner	60500
U+7532.0	fHakka	  gap7
U+7532.0	fCantonese	gaap3
U+7532.0	fMandarin	jia3; jia4
U+7532.0	fEnglish	[1] the first of the ten Celestial Stems [2] armor; shell; crust [3] most outstanding [4] a measure of lad in Taiwan (0.97 hectare) [5] first [6] a tithing, a group of ten families under the Tithing System introduced by Wang An-shih of the Sung dynasty
U+7532.1	fUTF8	甲
U+7532.1	fCNS11643	1-4672
U+7532.1	fBig5	A5D2
U+7532.1	fGB	0-3C57
U+7532.1	fR/S	 102.0
U+7532.1	fTotalStrokes	 5
U+7532.1	fCangjie	WL
U+7532.1	fFourCorner	60500
U+7532.1	fMandarin	jia5
U+7533.0	fUTF8	申
U+7533.0	fCNS11643	1-4673
U+7533.0	fBig5	A5D3
U+7533.0	fGB	0-496A
U+7533.0	fR/S	 102.0
U+7533.0	fTotalStrokes	 5
U+7533.0	fCangjie	LWL
U+7533.0	fFourCorner	50006
U+7533.0	fHakka	  shin1;  sin1; siin1
U+7533.0	fCantonese	san1
U+7533.0	fMandarin	shen1; xin4
U+7533.0	fEnglish	[1] the ninth of the twelve Terrestrial Branches [2] appeal; plead [3] state; report; set forth; explain; explicate [4] extend; expand [5] inculcate (especially repeatedly) [6] a brief name of Shanghai [7] a Chinese family name
U+7534.0	fUTF8	甴
U+7534.0	fGB	1-7C4D
U+7534.0	fR/S	 102.0
U+7534.0	fTotalStrokes	 5
U+7534.0	fCangjie	LWLM
U+7534.0	fFourCorner	4060
U+7534.0	fHakka	 zat7
U+7534.0	fCantonese	zaat6; zaat2 (also); zat6 (also); gaat6
U+7534.0	fMandarin	zha2
U+7534.0	fEnglish	 cockroach
U+7535.0	fUTF8	电
U+7535.0	fGB	0-3567
U+7535.0	fR/S	 73.1; 102.0
U+7535.0	fTotalStrokes	 5
U+7535.0	fCangjie	LWU
U+7535.0	fHakka	 tien5
U+7535.0	fCantonese	din6
U+7535.0	fMandarin	dian4
U+7535.0	fEnglish	simplified form of &#x96FB;
U+7536.0	fUTF8	甶
U+7536.0	fCNS11643	4-232B
U+7536.0	fGB	5-597C
U+7536.0	fR/S	  102.1
U+7536.0	fTotalStrokes	 6
U+7536.0	fCangjie	HW
U+7536.0	fFourCorner	26000
U+7536.0	fHakka	 fut8; fut7
U+7536.0	fCantonese	fat6
U+7536.0	fMandarin	fu2; fei4
U+7536.0	fEnglish	devil's head used by Buddhists
U+7537.0	fUTF8	男
U+7537.0	fCNS11643	1-4B2E
U+7537.0	fBig5	A86B
U+7537.0	fGB	0-4450
U+7537.0	fR/S	 102.2
U+7537.0	fTotalStrokes	 7
U+7537.0	fCangjie	WKS
U+7537.0	fFourCorner	60427
U+7537.0	fHakka	  nam2;  lam2; nam3
U+7537.0	fCantonese	naam4; laam4 (also)
U+7537.0	fMandarin	nan2
U+7537.0	fEnglish	[1] human male; man; boy; son [2] baron
U+7538.0	fUTF8	甸
U+7538.0	fCNS11643	1-4B2F
U+7538.0	fBig5	A86C
U+7538.0	fGB	0-3569
U+7538.0	fR/S	 102.2
U+7538.0	fTotalStrokes	 7
U+7538.0	fCangjie	PW
U+7538.0	fFourCorner	27620
U+7538.0	fHakka	 tien5; tien6; ten3; tian5
U+7538.0	fCantonese	din6; tin4; din1
U+7538.0	fMandarin	dian4; tian2; sheng4; ying4
U+7538.0	fEnglish	[1] suburbs or outskirts of the capital [2] govern; rule [3] farm crops
U+7539.0	fUTF8	甹
U+7539.0	fCNS11643	2-246E
U+7539.0	fBig5	CB6E
U+7539.0	fGB	3-3027
U+7539.0	fR/S	 102.2
U+7539.0	fTotalStrokes	 7
U+7539.0	fCangjie	LWMVS
U+7539.0	fFourCorner	50727
U+7539.0	fCantonese	ping1
U+7539.0	fMandarin	ping2; ping1
U+753A.0	fUTF8	町
U+753A.0	fCNS11643	2-246D
U+753A.0	fBig5	CB6D
U+753A.0	fGB	0-6E2E
U+753A.0	fR/S	 102.2
U+753A.0	fTotalStrokes	 7
U+753A.0	fCangjie	WMN
U+753A.0	fFourCorner	61020
U+753A.0	fHakka	  tin3; tang3; den1; tin1; tang2
U+753A.0	fCantonese	ting5; ding1; ting1; ting2; tin2
U+753A.0	fMandarin	ting3; ting1; zheng4; tian3
U+753A.0	fEnglish	[1] raised path between fields; boundary between agricultural land [2] [3] elevated surface; elevated area; platform
U+753A.1	fUTF8	町
U+753A.1	fCNS11643	2-246D
U+753A.1	fBig5	CB6D
U+753A.1	fGB	0-6E2E
U+753A.1	fR/S	 102.2
U+753A.1	fTotalStrokes	 7
U+753A.1	fCangjie	WMN
U+753A.1	fFourCorner	61020
U+753A.1	fMandarin	ding1
U+753A.1	fEnglish	 street; city block (Japan)
U+753B.0	fUTF8	画
U+753B.0	fCNS11643	3-2A37
U+753B.0	fGB	0-3B2D
U+753B.0	fR/S	  102.3
U+753B.0	fTotalStrokes	 8
U+753B.0	fCangjie	MUW
U+753B.0	fFourCorner	10770
U+753B.0	fHakka	 fa5; vak8
U+753B.0	fCantonese	waak6; waak2 (also); waa2 (also); waa6
U+753B.0	fMandarin	hua4; huo4
U+753B.0	fEnglish	simplified form of &#x756B;
U+753C.0	fUTF8	甼
U+753C.0	fGB	E-3374
U+753C.0	fR/S	 102.2
U+753C.0	fTotalStrokes	 7
U+753C.0	fCangjie	XWMN
U+753C.0	fCantonese	ting5
U+753C.0	fMandarin	ting3; ding1
U+753C.0	fEnglish	raised path between fields
U+753D.0	fUTF8	甽
U+753D.0	fCNS11643	1-4E77
U+753D.0	fBig5	AAB6
U+753D.0	fGB	5-5A23
U+753D.0	fR/S	 102.3
U+753D.0	fTotalStrokes	 8
U+753D.0	fCangjie	WLLL
U+753D.0	fFourCorner	62000
U+753D.0	fHakka	 kien3
U+753D.0	fCantonese	hyun2; zan3
U+753D.0	fMandarin	quan3; xun2; zhen4; zhun4
U+753D.0	fEnglish	drain between fields; irrigation
U+753E.0	fUTF8	甾
U+753E.0	fCNS11643	2-2834
U+753E.0	fBig5	CD72
U+753E.0	fGB	0-675E
U+753E.0	fR/S	 102.3
U+753E.0	fTotalStrokes	 8
U+753E.0	fCangjie	VVW
U+753E.0	fFourCorner	22600
U+753E.0	fHakka	 zih1; zii1
U+753E.0	fCantonese	zi1; zoi1
U+753E.0	fMandarin	zi1; zai1
U+753E.0	fEnglish	 ground that has been under cultivation for one year evil; calamity
U+753F.0	fUTF8	甿
U+753F.0	fCNS11643	2-2832
U+753F.0	fBig5	CD70
U+753F.0	fGB	3-5A60
U+753F.0	fR/S	 102.3
U+753F.0	fTotalStrokes	 8
U+753F.0	fCangjie	WYV
U+753F.0	fFourCorner	60010
U+753F.0	fHakka	  men2; mang2
U+753F.0	fCantonese	man4 (also); mang4 (also); maang4; mong4
U+753F.0	fMandarin	meng2; meng4; mang2
U+753F.0	fEnglish	farming population
U+7540.0	fUTF8	畀
U+7540.0	fCNS11643	2-2833
U+7540.0	fBig5	CD71
U+7540.0	fGB	0-6E2F
U+7540.0	fR/S	 102.3
U+7540.0	fTotalStrokes	 8
U+7540.0	fCangjie	WML
U+7540.0	fFourCorner	60221
U+7540.0	fHakka	  bi3; bun1; bin1; bi5
U+7540.0	fCantonese	bei2; bei3 (also)
U+7540.0	fMandarin	bi4
U+7540.0	fEnglish	 give to; confer; bestow
U+7541.0	fUTF8	畁
U+7541.0	fCNS11643	3-2A38
U+7541.0	fGB	5-302E
U+7541.0	fR/S	 102.3
U+7541.0	fTotalStrokes	 8
U+7541.0	fCangjie	LWML
U+7541.0	fFourCorner	50220
U+7541.0	fCantonese	bei2
U+7541.0	fMandarin	bi4; qi4; qi2; ji1
U+7541.0	fEnglish	 give to confer on
U+7542.0	fUTF8	畂
U+7542.0	fCNS11643	3-2A35
U+7542.0	fGB	E-3375
U+7542.0	fR/S	 102.3
U+7542.0	fTotalStrokes	 8
U+7542.0	fCangjie	WNO
U+7542.0	fFourCorner	67080
U+7542.0	fCantonese	mau5
U+7542.0	fMandarin	liu4; jiu4; mu3
U+7543.0	fUTF8	畃
U+7543.0	fCNS11643	4-2740
U+7543.0	fGB	E-3376
U+7543.0	fR/S	 102.3
U+7543.0	fTotalStrokes	 8
U+7543.0	fCangjie	LLPI
U+7543.0	fFourCorner	57020
U+7543.0	fCantonese	seon4
U+7543.0	fMandarin	xun2
U+7544.0	fUTF8	畄
U+7544.0	fCNS11643	3-2A36
U+7544.0	fGB	E-3377
U+7544.0	fR/S	 102.3
U+7544.0	fTotalStrokes	 8
U+7544.0	fCangjie	FW
U+7544.0	fFourCorner	23600
U+7544.0	fCantonese	lau4
U+7544.0	fMandarin	liu2
U+7544.0	fEnglish	 stop; halt stay; detain; keep
U+7545.0	fUTF8	畅
U+7545.0	fGB	0-3329
U+7545.0	fR/S	 72.4; 102.3
U+7545.0	fTotalStrokes	 8
U+7545.0	fCangjie	LLNSH
U+7545.0	fHakka	 chong3
U+7545.0	fCantonese	coeng3
U+7545.0	fMandarin	chang4
U+7545.0	fEnglish	simplified form of &#x66A2;
U+7546.0	fUTF8	畆
U+7546.0	fCNS11643	3-2E27
U+7546.0	fGB	E-3378
U+7546.0	fR/S	 102.4
U+7546.0	fTotalStrokes	 9
U+7546.0	fCangjie	YWI
U+7546.0	fFourCorner	02630
U+7546.0	fHakka	 meu3; meu1
U+7546.0	fCantonese	mau5
U+7546.0	fMandarin	mu3
U+7546.0	fEnglish	 Chinese land measure fields
U+7547.0	fUTF8	畇
U+7547.0	fCNS11643	2-2C3C
U+7547.0	fBig5	CFDA
U+7547.0	fGB	3-5A63
U+7547.0	fR/S	 102.4
U+7547.0	fTotalStrokes	 9
U+7547.0	fCangjie	WPMM
U+7547.0	fFourCorner	67020
U+7547.0	fHakka	 jun2; run2
U+7547.0	fCantonese	wan4
U+7547.0	fMandarin	yun2; xun2; jun1; xuan4; tian2; dian4
U+7547.0	fEnglish	land
U+7548.0	fUTF8	畈
U+7548.0	fCNS11643	2-2C3D
U+7548.0	fBig5	CFDB
U+7548.0	fGB	0-6E32
U+7548.0	fR/S	 102.4
U+7548.0	fTotalStrokes	 9
U+7548.0	fCangjie	WHE
U+7548.0	fFourCorner	61047
U+7548.0	fHakka	  fan5
U+7548.0	fCantonese	faan3
U+7548.0	fMandarin	fan4
U+7548.0	fEnglish	[1] plain; field; hamlet [2] field [3] used in place names
U+7549.0	fUTF8	畉
U+7549.0	fCNS11643	3-2E25
U+7549.0	fGB	5-5A27
U+7549.0	fR/S	  102.4
U+7549.0	fTotalStrokes	 9
U+7549.0	fCangjie	WQO
U+7549.0	fFourCorner	65030
U+7549.0	fCantonese	fu4
U+7549.0	fMandarin	fu2
U+754A.0	fUTF8	畊
U+754A.0	fCNS11643	3-2E28
U+754A.0	fGB	E-3379
U+754A.0	fR/S	  102.4
U+754A.0	fTotalStrokes	 9
U+754A.0	fCangjie	WTT
U+754A.0	fFourCorner	65000
U+754A.0	fHakka	 gang1
U+754A.0	fCantonese	gaang1; gang1 (also)
U+754A.0	fMandarin	geng1; jing1
U+754A.0	fEnglish	plow; cultivate
U+754B.0	fUTF8	畋
U+754B.0	fCNS11643	1-524E
U+754B.0	fBig5	ACCB
U+754B.0	fGB	0-6E31
U+754B.0	fR/S	 102.4
U+754B.0	fTotalStrokes	 9
U+754B.0	fCangjie	WOK
U+754B.0	fFourCorner	68040
U+754B.0	fHakka	  tien2; ten2
U+754B.0	fCantonese	tin4
U+754B.0	fMandarin	tian2; dian4
U+754B.0	fEnglish	[1] till land; cultivated land [2] hunt game
U+754C.0	fUTF8	界
U+754C.0	fCNS11643	1-524C
U+754C.0	fBig5	ACC9
U+754C.0	fGB	0-3D67
U+754C.0	fR/S	 102.4
U+754C.0	fTotalStrokes	 9
U+754C.0	fCangjie	WOLL
U+754C.0	fFourCorner	46128
U+754C.0	fHakka	  gai5; kie5; giai5
U+754C.0	fCantonese	gaai3
U+754C.0	fMandarin	jie4
U+754C.0	fEnglish	[1] boundary; limit [2] limit; demarcate; define; delimit [3] world
U+754D.0	fUTF8	畍
U+754D.0	fCNS11643	3-2E2A
U+754D.0	fGB	E-337A
U+754D.0	fR/S	 102.4
U+754D.0	fTotalStrokes	 9
U+754D.0	fCangjie	WOLL
U+754D.0	fFourCorner	68020
U+754D.0	fHakka	 giai5
U+754D.0	fCantonese	gaai3
U+754D.0	fMandarin	jie4
U+754E.0	fUTF8	畎
U+754E.0	fCNS11643	1-524D
U+754E.0	fBig5	ACCA
U+754E.0	fGB	0-6E30
U+754E.0	fR/S	 102.4
U+754E.0	fTotalStrokes	 9
U+754E.0	fCangjie	WIK
U+754E.0	fFourCorner	63034
U+754E.0	fHakka	 kien3; ken3; kian3
U+754E.0	fCantonese	hyun2; gyun2
U+754E.0	fMandarin	quan3; juan3
U+754E.0	fEnglish	[1] irrigation ditch; drain between fields; irrigation [2] valley; dale; canyon
U+754F.0	fUTF8	畏
U+754F.0	fCNS11643	1-524B
U+754F.0	fBig5	ACC8
U+754F.0	fGB	0-4E37
U+754F.0	fR/S	 102.4
U+754F.0	fTotalStrokes	 9
U+754F.0	fCangjie	WMV
U+754F.0	fFourCorner	60732
U+754F.0	fHakka	  wui5; wi5; vui5
U+754F.0	fCantonese	wai3
U+754F.0	fMandarin	wei4; wei1; wei3
U+754F.0	fEnglish	[1] fear; dread; stand in awe of; be afraid of; be scared of [2] revere; respect
U+7550.0	fUTF8	畐
U+7550.0	fCNS11643	3-2E26
U+7550.0	fGB	3-5A61
U+7550.0	fR/S	  102.4
U+7550.0	fTotalStrokes	 9
U+7550.0	fCangjie	MRW
U+7550.0	fFourCorner	10600
U+7550.0	fHakka	  fuk7
U+7550.0	fCantonese	fuk6
U+7550.0	fMandarin	fu2; bi4; fu4
U+7550.0	fEnglish	 fill a foll of cloth
U+7551.0	fUTF8	畑
U+7551.0	fCNS11643	F-2B7A
U+7551.0	fGB	8-2D6B
U+7551.0	fR/S	 86.5; 102.4
U+7551.0	fTotalStrokes	 9
U+7551.0	fCangjie	FW
U+7551.0	fFourCorner	9680
U+7551.0	fCantonese	tin4
U+7551.0	fMandarin	tian2
U+7551.0	fEnglish	 dry field (as opposed to rice field) used in Japanese names
U+7552.0	fUTF8	畒
U+7552.0	fGB	E-337B
U+7552.0	fR/S	 102.4
U+7552.0	fTotalStrokes	 9
U+7552.0	fCangjie	YWO
U+7552.0	fCantonese	mau5
U+7552.0	fMandarin	mu3
U+7552.0	fEnglish	 Chinese land measure fields
U+7553.0	fUTF8	畓
U+7553.0	fCNS11643	F-2C4A
U+7553.0	fGB	1-7D2F
U+7553.0	fR/S	 85.5; 102.4
U+7553.0	fTotalStrokes	 9
U+7553.0	fCangjie	EW
U+7553.0	fFourCorner	1260
U+7553.0	fCantonese	cyun4
U+7553.0	fEnglish	 rice field
U+7554.0	fUTF8	畔
U+7554.0	fCNS11643	1-5663
U+7554.0	fBig5	AF60
U+7554.0	fGB	0-454F
U+7554.0	fR/S	 102.5
U+7554.0	fTotalStrokes	 10
U+7554.0	fCangjie	WFQ
U+7554.0	fFourCorner	69050
U+7554.0	fHakka	  ban3; pan3
U+7554.0	fCantonese	bun6; bun5 (also)
U+7554.0	fMandarin	pan4
U+7554.0	fEnglish	[1] boundary path dividing fields; boundary between fields [2] side; bank [3] same as $
U+7555.0	fUTF8	畕
U+7555.0	fCNS11643	3-324E
U+7555.0	fGB	5-5A2A
U+7555.0	fR/S	 102.5
U+7555.0	fTotalStrokes	 10
U+7555.0	fCangjie	WW
U+7555.0	fFourCorner	60600
U+7555.0	fCantonese	goeng1
U+7555.0	fMandarin	jiang1
U+7556.0	fUTF8	畖
U+7556.0	fCNS11643	3-324D
U+7556.0	fGB	3-5A66
U+7556.0	fR/S	 102.5; 102.4
U+7556.0	fTotalStrokes	 10
U+7556.0	fCangjie	WHVO
U+7556.0	fFourCorner	62030
U+7556.0	fCantonese	waa1
U+7556.0	fMandarin	wa1
U+7557.0	fUTF8	畗
U+7557.0	fCNS11643	3-3250
U+7557.0	fGB	5-334E
U+7557.0	fR/S	 102.5
U+7557.0	fTotalStrokes	 10
U+7557.0	fCangjie	YRW
U+7557.0	fFourCorner	00600
U+7557.0	fCantonese	daap3
U+7557.0	fMandarin	fu2; da2
U+7558.0	fUTF8	畘
U+7558.0	fCNS11643	3-324F
U+7558.0	fGB	3-5A65
U+7558.0	fR/S	  102.5
U+7558.0	fTotalStrokes	 10
U+7558.0	fCangjie	WGB
U+7558.0	fFourCorner	65040
U+7558.0	fCantonese	naam4; laam4 (also)
U+7558.0	fMandarin	nan2
U+7559.0	fUTF8	留
U+7559.0	fCNS11643	1-5667
U+7559.0	fBig5	AF64
U+7559.0	fGB	0-4174
U+7559.0	fR/S	 102.5
U+7559.0	fTotalStrokes	 10
U+7559.0	fCangjie	HHW
U+7559.0	fFourCorner	77602
U+7559.0	fHakka	  liu2; lieu2; liu3
U+7559.0	fCantonese	lau4; lau1 (also)
U+7559.0	fMandarin	liu2; liu3; liu4
U+7559.0	fEnglish	[1] remain; stay; be at a standstill [2] ask somebody to stay [3] delay; detain; keep; obstruct [4] leave [5] preserve; reserve
U+755A.0	fUTF8	畚
U+755A.0	fCNS11643	1-5666
U+755A.0	fBig5	AF63
U+755A.0	fGB	0-5B4E
U+755A.0	fR/S	 102.5
U+755A.0	fTotalStrokes	 10
U+755A.0	fCangjie	IKW
U+755A.0	fFourCorner	23603
U+755A.0	fHakka	 bun5
U+755A.0	fCantonese	bun2
U+755A.0	fMandarin	ben3
U+755A.0	fEnglish	 bamboo basket or hod for carrying earth
U+755B.0	fUTF8	畛
U+755B.0	fCNS11643	2-3124
U+755B.0	fBig5	D2C1
U+755B.0	fGB	0-6E33
U+755B.0	fR/S	 102.5
U+755B.0	fTotalStrokes	 10
U+755B.0	fCangjie	WOHH
U+755B.0	fFourCorner	68022
U+755B.0	fHakka	 zhin3; zihn3; zin1; zin3
U+755B.0	fCantonese	zan2; can2 (also)
U+755B.0	fMandarin	zhen3; zhen1
U+755B.0	fEnglish	[1] border; boundary [2] raised path between fields; dikes along watercourses in fields
U+755C.0	fUTF8	畜
U+755C.0	fCNS11643	1-5665
U+755C.0	fBig5	AF62
U+755C.0	fGB	0-5073
U+755C.0	fR/S	 102.5
U+755C.0	fTotalStrokes	 10
U+755C.0	fCangjie	YVIW
U+755C.0	fFourCorner	00603
U+755C.0	fHakka	  hiuk7;  kiuk7; ch'uk7; cuk7; chuk7
U+755C.0	fCantonese	cuk1; cuk6 (also)
U+755C.0	fMandarin	chu4; xiu4; chou4
U+755C.0	fEnglish	[1] livestock; domestic animals [2] dumb creature; animal
U+755C.1	fUTF8	畜
U+755C.1	fCNS11643	1-5665
U+755C.1	fBig5	AF62
U+755C.1	fGB	0-5073
U+755C.1	fR/S	 102.5
U+755C.1	fTotalStrokes	 10
U+755C.1	fCangjie	YVIW
U+755C.1	fFourCorner	00603
U+755C.1	fHakka	 hit7
U+755C.1	fMandarin	xu4
U+755C.1	fEnglish	[1] rear or raise (livestock, children) [2] a Chinese family name
U+755D.0	fUTF8	畝
U+755D.0	fCNS11643	1-5664
U+755D.0	fBig5	AF61
U+755D.0	fGB	1-4436
U+755D.0	fR/S	 102.5
U+755D.0	fTotalStrokes	 10
U+755D.0	fCangjie	YWNO
U+755D.0	fFourCorner	07680
U+755D.0	fHakka	  meu1; miu1; miau1
U+755D.0	fCantonese	mau5
U+755D.0	fMandarin	mou3; mu3
U+755D.0	fEnglish	mu, Chinese land measure (733.5 square yards)
U+755E.0	fUTF8	畞
U+755E.0	fCNS11643	3-3251
U+755E.0	fGB	E-337C
U+755E.0	fR/S	 102.5
U+755E.0	fTotalStrokes	 10
U+755E.0	fCangjie	JWNO
U+755E.0	fFourCorner	40680
U+755E.0	fCantonese	mau5
U+755E.0	fMandarin	mu3
U+755F.0	fUTF8	畟
U+755F.0	fCNS11643	2-3125
U+755F.0	fBig5	D2C2
U+755F.0	fGB	5-5A2C
U+755F.0	fR/S	 102.5
U+755F.0	fTotalStrokes	 10
U+755F.0	fCangjie	WCHE
U+755F.0	fFourCorner	60407
U+755F.0	fHakka	 zit7;  cet7
U+755F.0	fCantonese	cak1; zik1; cik1
U+755F.0	fMandarin	ce4
U+755F.0	fEnglish	plough
U+7560.0	fUTF8	畠
U+7560.0	fGB	E-337D
U+7560.0	fR/S	 106.5
U+7560.0	fTotalStrokes	 10
U+7560.0	fCangjie	HAW
U+7560.0	fFourCorner	2660
U+7560.0	fCantonese	tin4; zi1
U+7560.0	fEnglish	garden; field; farm; plantation
U+7561.0	fUTF8	畡
U+7561.0	fCNS11643	3-3746
U+7561.0	fGB	5-5A31
U+7561.0	fR/S	  102.6
U+7561.0	fTotalStrokes	 11
U+7561.0	fCangjie	WYVO
U+7561.0	fFourCorner	60080
U+7561.0	fHakka	 goi1
U+7561.0	fCantonese	goi1
U+7561.0	fMandarin	gai1
U+7562.0	fUTF8	畢
U+7562.0	fCNS11643	1-5C2B
U+7562.0	fBig5	B2A6
U+7562.0	fGB	1-314F
U+7562.0	fR/S	 102.6
U+7562.0	fTotalStrokes	 11
U+7562.0	fCangjie	WTJ
U+7562.0	fFourCorner	60507
U+7562.0	fHakka	  bit7
U+7562.0	fCantonese	bat1
U+7562.0	fMandarin	bi4
U+7562.0	fEnglish	[1] end; finish; conclude; complete [2] whole; total; complete [3] a Chinese family name
U+7563.0	fUTF8	畣
U+7563.0	fCNS11643	2-3740
U+7563.0	fBig5	D67B
U+7563.0	fGB	5-5A2F
U+7563.0	fR/S	 102.6
U+7563.0	fTotalStrokes	 11
U+7563.0	fCangjie	ORW
U+7563.0	fFourCorner	80606
U+7563.0	fCantonese	daap3
U+7563.0	fMandarin	da2
U+7564.0	fUTF8	畤
U+7564.0	fCNS11643	2-373F
U+7564.0	fBig5	D67A
U+7564.0	fGB	3-5A68
U+7564.0	fR/S	 102.6
U+7564.0	fTotalStrokes	 11
U+7564.0	fCangjie	WGDI
U+7564.0	fFourCorner	64041
U+7564.0	fHakka	 zhi3; sii5; zih3; shi6; tje3; shi5
U+7564.0	fCantonese	zi6; si5; zi2
U+7564.0	fMandarin	zhi4; zhi3
U+7564.0	fEnglish	a place for worshipping the Heaven and Earth, and the five sage kings in ancient times
U+7565.0	fUTF8	略
U+7565.0	fCNS11643	1-5C29
U+7565.0	fBig5	B2A4
U+7565.0	fGB	0-4254
U+7565.0	fR/S	 102.6
U+7565.0	fTotalStrokes	 11
U+7565.0	fCangjie	WHER
U+7565.0	fFourCorner	67064
U+7565.0	fHakka	  liok8
U+7565.0	fCantonese	loek6; loek2 (also)
U+7565.0	fMandarin	luue4; le4
U+7565.0	fEnglish	[1] approximately; roughly; outline; rough; brief [2] slight; small in extent [3] scheme; plan [4] strategy [5] invade; seize [6] omit; leave out [7] survey the boundaries [8] a Chinese family name
U+7566.0	fUTF8	畦
U+7566.0	fCNS11643	1-5C2A
U+7566.0	fBig5	B2A5
U+7566.0	fGB	0-4668
U+7566.0	fR/S	 102.6
U+7566.0	fTotalStrokes	 11
U+7566.0	fCangjie	WGG
U+7566.0	fFourCorner	64014
U+7566.0	fHakka	  hi1;  gui1; kui2; hi2; kwui1
U+7566.0	fCantonese	kwai4
U+7566.0	fMandarin	qi2; xi1; xi2; wei2; sui1
U+7566.0	fEnglish	[1] land of 50 mu [2] plot, piece, or parcel of land [3] farmer; laborer
U+7567.0	fUTF8	畧
U+7567.0	fCNS11643	3-3749
U+7567.0	fGB	E-337E
U+7567.0	fR/S	  102.6
U+7567.0	fTotalStrokes	 11
U+7567.0	fCangjie	WHER
U+7567.0	fFourCorner	60600
U+7567.0	fHakka	 liok8
U+7567.0	fCantonese	loek6
U+7567.0	fMandarin	luue4; le4
U+7567.0	fEnglish	 approximately; roughly outline
U+7568.0	fUTF8	畨
U+7568.0	fCNS11643	3-3747
U+7568.0	fGB	E-3421
U+7568.0	fR/S	 119.5; 102.6
U+7568.0	fTotalStrokes	 11
U+7568.0	fCangjie	FDW
U+7568.0	fFourCorner	90600
U+7568.0	fCantonese	faan1
U+7568.0	fMandarin	fan1; pan1
U+7569.0	fUTF8	畩
U+7569.0	fGB	E-3422
U+7569.0	fR/S	 102.6
U+7569.0	fTotalStrokes	 11
U+7569.0	fCangjie	WYHV
U+7569.0	fCantonese	ji1
U+756A.0	fUTF8	番
U+756A.0	fCNS11643	1-606C
U+756A.0	fBig5	B566
U+756A.0	fGB	0-372C
U+756A.0	fR/S	 102.7
U+756A.0	fTotalStrokes	 12
U+756A.0	fCangjie	HDW
U+756A.0	fFourCorner	20609
U+756A.0	fHakka	  fan1; pan1; po2
U+756A.0	fCantonese	faan1; pun1; po4; bo1
U+756A.0	fMandarin	fan1; fan2; po2; pi2; bo4; fan4; pan2; pan4
U+756A.0	fEnglish	[1] take turns; turn [2] order in series [3] a time [4] a kind of; a sort of [5] barbarians [6] foreign
U+756A.1	fUTF8	番
U+756A.1	fCNS11643	1-606C
U+756A.1	fBig5	B566
U+756A.1	fGB	0-372C
U+756A.1	fR/S	 102.7
U+756A.1	fTotalStrokes	 12
U+756A.1	fCangjie	HDW
U+756A.1	fFourCorner	20609
U+756A.1	fMandarin	pan1
U+756A.1	fEnglish	[1] a county in Kwangtung [2] a Chinese family name
U+756A.2	fUTF8	番
U+756A.2	fCNS11643	1-606C
U+756A.2	fBig5	B566
U+756A.2	fGB	0-372C
U+756A.2	fR/S	 102.7
U+756A.2	fTotalStrokes	 12
U+756A.2	fCangjie	HDW
U+756A.2	fFourCorner	20609
U+756A.2	fMandarin	bo1
U+756A.2	fEnglish	martial-like
U+756B.0	fUTF8	畫
U+756B.0	fCNS11643	1-606B
U+756B.0	fBig5	B565
U+756B.0	fGB	1-3B2D
U+756B.0	fR/S	 102.7
U+756B.0	fTotalStrokes	 12
U+756B.0	fCangjie	LGWM
U+756B.0	fFourCorner	50106
U+756B.0	fHakka	  wak8; fa5; fa6; fa3; wa3; fa2; vak8
U+756B.0	fCantonese	waak6; waa6; waa2 (also); waak2 (also)
U+756B.0	fMandarin	hua4; huo4
U+756B.0	fEnglish	[1] delineate; mark off; delimit [2] paint; draw; painting; picture; drawing [3] plan; design; plan [4] a stroke in Chinese writing
U+756C.0	fUTF8	畬
U+756C.0	fCNS11643	2-3E32
U+756C.0	fBig5	DAAE
U+756C.0	fGB	3-5A6A
U+756C.0	fR/S	 102.7
U+756C.0	fTotalStrokes	 12
U+756C.0	fCangjie	OMDW
U+756C.0	fFourCorner	68094
U+756C.0	fHakka	  cia2;  ji2; ri2; ca1
U+756C.0	fCantonese	jyu4; se1; se4
U+756C.0	fMandarin	yu2; she1; yu4
U+756C.0	fEnglish	[1] name of a backward people in southern China [2] cultivate land by first setting fire to it
U+756D.0	fUTF8	畭
U+756D.0	fCNS11643	3-3D43
U+756D.0	fGB	E-3423
U+756D.0	fR/S	 102.7
U+756D.0	fTotalStrokes	 12
U+756D.0	fCangjie	WOMD
U+756D.0	fFourCorner	68090
U+756D.0	fCantonese	jyu4
U+756D.0	fMandarin	yu2
U+756E.0	fUTF8	畮
U+756E.0	fCNS11643	3-3D42
U+756E.0	fGB	E-3424
U+756E.0	fR/S	  102.7
U+756E.0	fTotalStrokes	 12
U+756E.0	fCangjie	WOWY
U+756E.0	fFourCorner	68050
U+756E.0	fCantonese	mau5
U+756E.0	fMandarin	mu3; mou3
U+756F.0	fUTF8	畯
U+756F.0	fCNS11643	2-3E31
U+756F.0	fBig5	DAAD
U+756F.0	fGB	3-5A6B
U+756F.0	fR/S	 102.7
U+756F.0	fTotalStrokes	 12
U+756F.0	fCangjie	WICE
U+756F.0	fFourCorner	63047
U+756F.0	fHakka	  zun5
U+756F.0	fCantonese	zeon3
U+756F.0	fMandarin	jun4
U+756F.0	fEnglish	[1] the official in charge of farmland in ancient times [2] rustic; crude
U+7570.0	fUTF8	異
U+7570.0	fCNS11643	1-5C2C
U+7570.0	fBig5	B2A7
U+7570.0	fGB	E-3425
U+7570.0	fR/S	 102.6; 102.7
U+7570.0	fTotalStrokes	 12
U+7570.0	fCangjie	WTC
U+7570.0	fFourCorner	60801
U+7570.0	fHakka	  ji5; ji3; ri5; i5
U+7570.0	fCantonese	ji6
U+7570.0	fMandarin	yi4
U+7570.0	fEnglish	[1] different; difference [2] unusual; strange; peculiar; extraordinary; uncommon [3] foreign; unfamiliar; unknown [4] marvel; wonder [5] separate
U+7571.0	fUTF8	畱
U+7571.0	fCNS11643	3-3D44
U+7571.0	fGB	E-3426
U+7571.0	fR/S	 102.9; 102.7
U+7571.0	fTotalStrokes	 12; 14
U+7571.0	fCangjie	XMLLW
U+7571.0	fFourCorner	10600
U+7571.0	fHakka	 liu2
U+7571.0	fCantonese	lau4
U+7571.0	fMandarin	liu2; liu3; liu4
U+7571.0	fEnglish	 detain entertain to keep put by
U+7572.0	fUTF8	畲
U+7572.0	fCNS11643	4-392C
U+7572.0	fGB	0-6E34
U+7572.0	fR/S	 102.7
U+7572.0	fTotalStrokes	 12
U+7572.0	fCangjie	OMMW
U+7572.0	fFourCorner	80600
U+7572.0	fCantonese	se1; se4
U+7572.0	fMandarin	yu2; she1
U+7572.0	fEnglish	to cultivate land by first setting fire to it
U+7573.0	fUTF8	畳
U+7573.0	fGB	E-3427
U+7573.0	fR/S	 102.7
U+7573.0	fTotalStrokes	 12
U+7573.0	fCangjie	WBBM
U+7573.0	fCantonese	dip6
U+7573.0	fMandarin	die2
U+7573.0	fEnglish	 repeat; duplicate repetitious
U+7574.0	fUTF8	畴
U+7574.0	fGB	0-336B
U+7574.0	fR/S	 102.7
U+7574.0	fTotalStrokes	 12
U+7574.0	fCangjie	WQKI
U+7574.0	fHakka	 chiu2
U+7574.0	fCantonese	cau4
U+7574.0	fMandarin	chou2
U+7574.0	fEnglish	simplified form of &#x7587;
U+7575.0	fUTF8	畵
U+7575.0	fCNS11643	4-3E6B
U+7575.0	fGB	E-3428
U+7575.0	fR/S	 102.8
U+7575.0	fTotalStrokes	 13
U+7575.0	fCangjie	LGUW
U+7575.0	fFourCorner	50770
U+7575.0	fHakka	 fa3; wak8
U+7575.0	fCantonese	waak6
U+7575.0	fMandarin	hua4
U+7575.0	fEnglish	 delineate; painting; picture; drawing draw
U+7576.0	fUTF8	當
U+7576.0	fCNS11643	1-6535
U+7576.0	fBig5	B7ED
U+7576.0	fGB	1-3531
U+7576.0	fR/S	 102.8
U+7576.0	fTotalStrokes	 13
U+7576.0	fCangjie	FBRW
U+7576.0	fFourCorner	90606
U+7576.0	fHakka	  dong1; dang5
U+7576.0	fCantonese	dong1; dong3; dong2
U+7576.0	fMandarin	dang1; dang2
U+7576.0	fEnglish	[1] undertake; assume (responsibilities, etc.) [2] face [3] equal; well-matched [4] in the very same (place, year, day, etc.) [5] ought to; should; must [6] just at (at time or place) [7] work as; serve as
U+7576.1	fUTF8	當
U+7576.1	fCNS11643	1-6535
U+7576.1	fBig5	B7ED
U+7576.1	fGB	1-3531
U+7576.1	fR/S	 102.8
U+7576.1	fTotalStrokes	 13
U+7576.1	fCangjie	FBRW
U+7576.1	fFourCorner	90606
U+7576.1	fHakka	 dong5
U+7576.1	fMandarin	dang4
U+7576.1	fEnglish	[1] proper; appropriate [2] pawn; mortgage; pledge [3] take as; regard as; consider as
U+7576.2	fUTF8	當
U+7576.2	fCNS11643	1-6535
U+7576.2	fBig5	B7ED
U+7576.2	fGB	1-3531
U+7576.2	fR/S	 102.8
U+7576.2	fTotalStrokes	 13
U+7576.2	fCangjie	FBRW
U+7576.2	fFourCorner	90606
U+7576.2	fMandarin	dang3
U+7576.2	fEnglish	 mistake something for another
U+7576.3	fUTF8	當
U+7576.3	fCNS11643	1-6535
U+7576.3	fBig5	B7ED
U+7576.3	fGB	1-3531
U+7576.3	fR/S	 102.8
U+7576.3	fTotalStrokes	 13
U+7576.3	fCangjie	FBRW
U+7576.3	fFourCorner	90606
U+7577.0	fUTF8	畷
U+7577.0	fCNS11643	2-452B
U+7577.0	fBig5	DEC5
U+7577.0	fGB	3-5A6F
U+7577.0	fR/S	 102.8
U+7577.0	fTotalStrokes	 13
U+7577.0	fCangjie	WEEE
U+7577.0	fFourCorner	67047
U+7577.0	fHakka	  dot7; chot7; zot7
U+7577.0	fCantonese	zyut3; zeoi3 (also)
U+7577.0	fMandarin	chuo4; zhui4; zhuo2; chui4
U+7577.0	fEnglish	raised path between fields
U+7578.0	fUTF8	畸
U+7578.0	fCNS11643	1-6536
U+7578.0	fBig5	B7EE
U+7578.0	fGB	0-3B7B
U+7578.0	fR/S	 102.8
U+7578.0	fTotalStrokes	 13
U+7578.0	fCangjie	WKMR
U+7578.0	fFourCorner	64021
U+7578.0	fHakka	 ki2
U+7578.0	fCantonese	kei1; gei1 (also)
U+7578.0	fMandarin	ji1; qi2
U+7578.0	fEnglish	[1] fractional remainder [2] fields with irregular boundaries [3] malformed; misshapen; deformity
U+7579.0	fUTF8	畹
U+7579.0	fCNS11643	2-452A
U+7579.0	fBig5	DEC4
U+7579.0	fGB	0-6E35
U+7579.0	fR/S	 102.8
U+7579.0	fTotalStrokes	 13
U+7579.0	fCangjie	WJNU
U+7579.0	fFourCorner	63012
U+7579.0	fHakka	 wan3; van3
U+7579.0	fCantonese	jyun2
U+7579.0	fMandarin	wan3; yuan4
U+7579.0	fEnglish	[1] a measure of land equaling 12 or 30 mu [2] a large field
U+757A.0	fUTF8	畺
U+757A.0	fCNS11643	3-427B
U+757A.0	fGB	5-302B
U+757A.0	fR/S	 102.8
U+757A.0	fTotalStrokes	 13
U+757A.0	fCangjie	MWMM
U+757A.0	fFourCorner	10100
U+757A.0	fHakka	 giong1
U+757A.0	fCantonese	goeng1
U+757A.0	fMandarin	jiang1; jiang4
U+757A.0	fEnglish	boundary; border
U+757B.0	fUTF8	畻
U+757B.0	fCNS11643	4-447C
U+757B.0	fGB	E-3429
U+757B.0	fR/S	  102.9
U+757B.0	fTotalStrokes	 14
U+757B.0	fCangjie	WFQG
U+757B.0	fFourCorner	69010
U+757B.0	fHakka	 shin5
U+757B.0	fCantonese	sing4
U+757B.0	fMandarin	sheng2; cheng2
U+757C.0	fUTF8	畼
U+757C.0	fCNS11643	4-447B
U+757C.0	fGB	5-5A25
U+757C.0	fR/S	 102.9
U+757C.0	fTotalStrokes	 14
U+757C.0	fCangjie	WAMH
U+757C.0	fFourCorner	66020
U+757C.0	fHakka	 chong3; cong3
U+757C.0	fCantonese	coeng4
U+757C.0	fMandarin	chang4; zhang4; chang2
U+757C.0	fEnglish	[1] broad barren piece of ground [2] a place in the old feudal State of Wei
U+757D.0	fUTF8	畽
U+757D.0	fCNS11643	2-4B7E
U+757D.0	fBig5	E2D8
U+757D.0	fGB	5-5A36
U+757D.0	fR/S	 102.9
U+757D.0	fTotalStrokes	 14
U+757D.0	fCangjie	WHJG
U+757D.0	fFourCorner	62014
U+757D.0	fHakka	  ton3; tung1
U+757D.0	fCantonese	teon2; dung2
U+757D.0	fMandarin	tuan3; tun3; tong1
U+757D.0	fEnglish	ground
U+757E.0	fUTF8	畾
U+757E.0	fCNS11643	2-5236
U+757E.0	fBig5	E6AE
U+757E.0	fGB	3-5A73
U+757E.0	fR/S	 102.10
U+757E.0	fTotalStrokes	 15
U+757E.0	fCangjie	WWW
U+757E.0	fFourCorner	60660
U+757E.0	fHakka	 lui2; lui3
U+757E.0	fCantonese	leoi4; leoi5
U+757E.0	fMandarin	lei2; lei3
U+757E.0	fEnglish	fields divided by dikes
U+757F.0	fUTF8	畿
U+757F.0	fCNS11643	1-6D6B
U+757F.0	fBig5	BD42
U+757F.0	fGB	0-675C
U+757F.0	fR/S	 102.10
U+757F.0	fTotalStrokes	 15
U+757F.0	fCangjie	VIW
U+757F.0	fFourCorner	22653
U+757F.0	fHakka	  gi1; ki2
U+757F.0	fCantonese	gei1; kei4
U+757F.0	fMandarin	ji1; qi2
U+757F.0	fEnglish	imperial domain; areas near the capital; royal domain
U+7580.0	fUTF8	疀
U+7580.0	fCNS11643	2-5856
U+7580.0	fBig5	EA6A
U+7580.0	fGB	5-4167
U+7580.0	fR/S	 102.11
U+7580.0	fTotalStrokes	 16
U+7580.0	fCangjie	VWJLO
U+7580.0	fFourCorner	25681
U+7580.0	fCantonese	caap3
U+7580.0	fMandarin	cha1; qie4
U+7581.0	fUTF8	疁
U+7581.0	fCNS11643	3-5170
U+7581.0	fGB	3-5A74
U+7581.0	fR/S	 102.11
U+7581.0	fTotalStrokes	 16
U+7581.0	fCangjie	WSMH
U+7581.0	fFourCorner	67020
U+7581.0	fHakka	  liu2
U+7581.0	fCantonese	lau4
U+7581.0	fMandarin	liu2; xu4
U+7581.0	fEnglish	 burn grass or stubble on a field
U+7582.0	fUTF8	疂
U+7582.0	fGB	E-342A
U+7582.0	fR/S	 102.11
U+7582.0	fTotalStrokes	 16
U+7582.0	fCangjie	WIOM
U+7582.0	fCantonese	dip6
U+7582.0	fMandarin	die2
U+7582.0	fEnglish	 repeat, duplicate; repititious
U+7583.0	fUTF8	疃
U+7583.0	fCNS11643	3-5552
U+7583.0	fGB	0-6E36
U+7583.0	fR/S	 102.12
U+7583.0	fTotalStrokes	 17
U+7583.0	fCangjie	WYTG
U+7583.0	fFourCorner	60010
U+7583.0	fHakka	  ton3; tung1
U+7583.0	fCantonese	teon2
U+7583.0	fMandarin	tuan3
U+7583.0	fEnglish	 hamlet area outside city
U+7584.0	fUTF8	疄
U+7584.0	fCNS11643	2-5D5F
U+7584.0	fBig5	ED73
U+7584.0	fGB	3-5A75
U+7584.0	fR/S	 102.12
U+7584.0	fTotalStrokes	 17
U+7584.0	fCangjie	WFDQ
U+7584.0	fFourCorner	69059
U+7584.0	fHakka	  lin2
U+7584.0	fCantonese	leon6; leon4
U+7584.0	fMandarin	lin2; lin4; lin3
U+7584.0	fEnglish	raised path
U+7585.0	fUTF8	疅
U+7585.0	fCNS11643	4-5B74
U+7585.0	fGB	E-342B
U+7585.0	fR/S	  102.13
U+7585.0	fTotalStrokes	 18
U+7585.0	fCangjie	WMWM
U+7585.0	fFourCorner	61010
U+7585.0	fCantonese	goeng1
U+7585.0	fMandarin	jiang1
U+7586.0	fUTF8	疆
U+7586.0	fCNS11643	1-7851
U+7586.0	fBig5	C3A6
U+7586.0	fGB	0-3D2E
U+7586.0	fR/S	 102.14
U+7586.0	fTotalStrokes	 19
U+7586.0	fCangjie	NGMWM
U+7586.0	fFourCorner	11116
U+7586.0	fHakka	  giong1;  kiong1
U+7586.0	fCantonese	goeng1
U+7586.0	fMandarin	jiang1; meng4; qiang2; qiang3; jiang4
U+7586.0	fEnglish	[1] boundary; border; frontier [2] limit
U+7587.0	fUTF8	疇
U+7587.0	fCNS11643	1-7850
U+7587.0	fBig5	C3A5
U+7587.0	fGB	1-336B
U+7587.0	fR/S	 102.14
U+7587.0	fTotalStrokes	 19
U+7587.0	fCangjie	WGNI
U+7587.0	fFourCorner	64041
U+7587.0	fHakka	 chiu2; cu2; ciu2
U+7587.0	fCantonese	cau4
U+7587.0	fMandarin	chou2
U+7587.0	fEnglish	[1] agricultural field; farmland; arable land [2] who [3] category; class; rank
U+7588.0	fUTF8	疈
U+7588.0	fCNS11643	4-6368
U+7588.0	fGB	3-5A79
U+7588.0	fR/S	  102.15
U+7588.0	fTotalStrokes	 20
U+7588.0	fCangjie	MWILW
U+7588.0	fFourCorner	11660
U+7588.0	fCantonese	bik1
U+7588.0	fMandarin	bo4; pi4
U+7589.0	fUTF8	疉
U+7589.0	fCNS11643	3-5C62
U+7589.0	fGB	E-342C
U+7589.0	fR/S	 102.17
U+7589.0	fTotalStrokes	 20; 22
U+7589.0	fCangjie	XWWWM
U+7589.0	fFourCorner	60100
U+7589.0	fHakka	  tiap8
U+7589.0	fCantonese	dip6; daap6 (also)
U+7589.0	fMandarin	die2
U+7589.0	fEnglish	 repeat; duplicate repititious
U+758A.0	fUTF8	疊
U+758A.0	fCNS11643	1-7B68
U+758A.0	fBig5	C57C
U+758A.0	fGB	1-357E
U+758A.0	fR/S	 102.17
U+758A.0	fTotalStrokes	 22
U+758A.0	fCangjie	WWWM
U+758A.0	fFourCorner	60107
U+758A.0	fHakka	  tiap8
U+758A.0	fCantonese	dip6; daap6 (also)
U+758A.0	fMandarin	die2
U+758A.0	fEnglish	[1] repeat; duplicate; repetitious [2] fold up [3] pile up [4] a stack of (bank notes)
U+758B.0	fUTF8	疋
U+758B.0	fCNS11643	1-4674
U+758B.0	fBig5	A5D4
U+758B.0	fGB	0-7162
U+758B.0	fR/S	 103.0
U+758B.0	fTotalStrokes	 5
U+758B.0	fCangjie	NYO
U+758B.0	fFourCorner	17801
U+758B.0	fHakka	  pit7
U+758B.0	fCantonese	ngaa5; so1; aa5 (also); pat1
U+758B.0	fMandarin	shu1; pi3; ya3; xu3; pi1; shu4; zu2
U+758B.0	fEnglish	[1] roll (cloth); bolt (cloth) [2] Kangxi radical 103
U+758C.0	fUTF8	疌
U+758C.0	fCNS11643	2-2835
U+758C.0	fBig5	CD73
U+758C.0	fGB	5-3026
U+758C.0	fR/S	 103.3
U+758C.0	fTotalStrokes	 8
U+758C.0	fCangjie	JLYO
U+758C.0	fFourCorner	50801
U+758C.0	fCantonese	zit3; zit6
U+758C.0	fMandarin	nie4
U+758D.0	fUTF8	疍
U+758D.0	fGB	3-5F72
U+758D.0	fR/S	 103.5
U+758D.0	fTotalStrokes	 10
U+758D.0	fCangjie	NYOM
U+758D.0	fCantonese	daan6; daan2 (also); dan6
U+758D.0	fMandarin	dan4
U+758E.0	fUTF8	疎
U+758E.0	fCNS11643	3-3D45
U+758E.0	fGB	E-342D
U+758E.0	fR/S	  103.7
U+758E.0	fTotalStrokes	 12
U+758E.0	fCangjie	NMDL
U+758E.0	fFourCorner	15190
U+758E.0	fHakka	  so1; su1; se1; sii1
U+758E.0	fCantonese	so1; so3
U+758E.0	fMandarin	shu1; su1; shu4
U+758E.0	fEnglish	 neglect careless; lax
U+758F.0	fUTF8	疏
U+758F.0	fCNS11643	1-5C2D
U+758F.0	fBig5	B2A8
U+758F.0	fGB	0-4A68
U+758F.0	fR/S	 103.7
U+758F.0	fTotalStrokes	 12
U+758F.0	fCangjie	NMYIU
U+758F.0	fFourCorner	10113
U+758F.0	fHakka	  so1; su1; se1; sii1
U+758F.0	fCantonese	so1; so3
U+758F.0	fMandarin	shu1; su1
U+758F.0	fEnglish	[1] thin; sparse; few [2] unfamiliar; distant; unfriendly [3] neglect; careless [4] channel; remove obstructions [5] coarse
U+758F.1	fUTF8	疏
U+758F.1	fCNS11643	1-5C2D
U+758F.1	fBig5	B2A8
U+758F.1	fGB	0-4A68
U+758F.1	fR/S	 103.7
U+758F.1	fTotalStrokes	 12
U+758F.1	fCangjie	NMYIU
U+758F.1	fFourCorner	10113
U+758F.1	fMandarin	shu4
U+758F.1	fEnglish	[1] present point by point [2] explicate; annotate
U+7590.0	fUTF8	疐
U+7590.0	fCNS11643	2-4C21
U+7590.0	fBig5	E2D9
U+7590.0	fGB	5-304E
U+7590.0	fR/S	 103.9
U+7590.0	fTotalStrokes	 14
U+7590.0	fCangjie	JBWNO
U+7590.0	fFourCorner	40801
U+7590.0	fHakka	 zhi5; zih5; di1
U+7590.0	fCantonese	zi3; dai3
U+7590.0	fMandarin	zhi4; di4
U+7590.0	fEnglish	 suffer a fall; stumble; fall; triple
U+7591.0	fUTF8	疑
U+7591.0	fCNS11643	1-696A
U+7591.0	fBig5	BAC3
U+7591.0	fGB	0-5249
U+7591.0	fR/S	 103.9
U+7591.0	fTotalStrokes	 14
U+7591.0	fCangjie	PKNIO
U+7591.0	fFourCorner	27481
U+7591.0	fHakka	  ngi2; gni2; gni3; ni2
U+7591.0	fCantonese	ji4
U+7591.0	fMandarin	yi2; ning2; ni3; yi4; ni4
U+7591.0	fEnglish	[1] doubt; question; doubtful; dubious; skeptical; doubt [2] suspicious; suspect [3] strange; incomprehensible; mysterious; questionable [4] sham; dummy; false
U+7592.0	fUTF8	疒
U+7592.0	fCNS11643	3-225B
U+7592.0	fGB	0-705A
U+7592.0	fR/S	 104.0
U+7592.0	fTotalStrokes	 5
U+7592.0	fCangjie	IIM
U+7592.0	fFourCorner	00100
U+7592.0	fHakka	 cong2
U+7592.0	fCantonese	nik6; lik6 (also); cong4 (also)
U+7592.0	fMandarin	chuang2
U+7592.0	fEnglish	 sick Kangxi radical number 104
U+7593.0	fUTF8	疓
U+7593.0	fCNS11643	3-2673
U+7593.0	fGB	5-5C60
U+7593.0	fR/S	  104.2
U+7593.0	fTotalStrokes	 7
U+7593.0	fCangjie	KNHS
U+7593.0	fFourCorner	00120
U+7593.0	fHakka	  nai3
U+7593.0	fCantonese	naai5; laai5 (also); naai3
U+7593.0	fMandarin	nai3
U+7593.0	fEnglish	exhausted; worn-out
U+7594.0	fUTF8	疔
U+7594.0	fCNS11643	2-246F
U+7594.0	fBig5	CB6F
U+7594.0	fGB	0-705B
U+7594.0	fR/S	 104.2
U+7594.0	fTotalStrokes	 7
U+7594.0	fCangjie	KMN
U+7594.0	fFourCorner	00121
U+7594.0	fHakka	  dang1;  den1
U+7594.0	fCantonese	ding1; deng1 (also)
U+7594.0	fMandarin	ding1; ne4
U+7594.0	fEnglish	 carbuncle; boil
U+7595.0	fUTF8	疕
U+7595.0	fCNS11643	2-2470
U+7595.0	fBig5	CB70
U+7595.0	fGB	3-5E47
U+7595.0	fR/S	 104.2
U+7595.0	fTotalStrokes	 7
U+7595.0	fCangjie	KP
U+7595.0	fFourCorner	00111
U+7595.0	fHakka	 to2; pi3; bi3
U+7595.0	fCantonese	bei2; pei2
U+7595.0	fMandarin	bi3; pi3; pi1
U+7595.0	fEnglish	head sores
U+7596.0	fUTF8	疖
U+7596.0	fGB	0-705C
U+7596.0	fR/S	 104.2
U+7596.0	fTotalStrokes	 7
U+7596.0	fCangjie	KSL
U+7596.0	fHakka	 ziet7
U+7596.0	fCantonese	zit3
U+7596.0	fMandarin	jie2
U+7596.0	fEnglish	simplified form of &#x7664;
U+7597.0	fUTF8	疗
U+7597.0	fGB	0-4146
U+7597.0	fR/S	 104.2
U+7597.0	fTotalStrokes	 7
U+7597.0	fCangjie	KNN
U+7597.0	fHakka	 liau2; liau3
U+7597.0	fCantonese	liu4
U+7597.0	fMandarin	liao2; liao4; shuo4
U+7597.0	fEnglish	simplified form of &#x7642;
U+7598.0	fUTF8	疘
U+7598.0	fCNS11643	2-2836
U+7598.0	fBig5	CD74
U+7598.0	fGB	E-342E
U+7598.0	fR/S	 104.3
U+7598.0	fTotalStrokes	 8
U+7598.0	fCangjie	KM
U+7598.0	fFourCorner	00111
U+7598.0	fHakka	  gung1
U+7598.0	fCantonese	gong1
U+7598.0	fMandarin	gong1; gang1
U+7598.0	fEnglish	prolapsus of the rectum
U+7599.0	fUTF8	疙
U+7599.0	fCNS11643	1-4E79
U+7599.0	fBig5	AAB8
U+7599.0	fGB	0-386D
U+7599.0	fR/S	 104.3
U+7599.0	fTotalStrokes	 8
U+7599.0	fCangjie	KON
U+7599.0	fFourCorner	00117
U+7599.0	fHakka	  ngiet7; kit8
U+7599.0	fCantonese	gat1; ngat6 (also); at6 (also); gat6
U+7599.0	fMandarin	yi4; ge1; ji4
U+7599.0	fEnglish	 pimple; wart; pustule
U+759A.0	fUTF8	疚
U+759A.0	fCNS11643	1-4E7A
U+759A.0	fBig5	AAB9
U+759A.0	fGB	0-3E4E
U+759A.0	fR/S	 104.3
U+759A.0	fTotalStrokes	 8
U+759A.0	fCangjie	KNO
U+759A.0	fFourCorner	00187
U+759A.0	fHakka	 giu5
U+759A.0	fCantonese	gau3
U+759A.0	fMandarin	jiu4
U+759A.0	fEnglish	[1] chronic disease; chronic illness; prolonged illness [2] mental discomfort [3] guilty conscience
U+759B.0	fUTF8	疛
U+759B.0	fCNS11643	3-2A3A
U+759B.0	fGB	3-5E48
U+759B.0	fR/S	 104.3
U+759B.0	fTotalStrokes	 8
U+759B.0	fCangjie	XKDI
U+759B.0	fFourCorner	00140
U+759B.0	fCantonese	zau2
U+759B.0	fMandarin	zhou3; zhou4; zhu2
U+759C.0	fUTF8	疜
U+759C.0	fCNS11643	4-2743
U+759C.0	fGB	E-342F
U+759C.0	fR/S	 104.3
U+759C.0	fTotalStrokes	 8
U+759C.0	fCangjie	KMY
U+759C.0	fFourCorner	00130
U+759C.0	fCantonese	haa6
U+759C.0	fMandarin	xia4
U+759D.0	fUTF8	疝
U+759D.0	fCNS11643	1-4E78
U+759D.0	fBig5	AAB7
U+759D.0	fGB	0-705E
U+759D.0	fR/S	 104.3
U+759D.0	fTotalStrokes	 8
U+759D.0	fCangjie	KU
U+759D.0	fFourCorner	00172
U+759D.0	fHakka	 san5;  san3; sen1; san1
U+759D.0	fCantonese	saan3
U+759D.0	fMandarin	shan4; shan1
U+759D.0	fEnglish	hernia
U+759E.0	fUTF8	疞
U+759E.0	fCNS11643	3-2A3B
U+759E.0	fGB	E-3430
U+759E.0	fR/S	 104.3
U+759E.0	fTotalStrokes	 8
U+759E.0	fCangjie	KMMS
U+759E.0	fFourCorner	00120
U+759E.0	fCantonese	heoi1
U+759E.0	fMandarin	xu1
U+759F.0	fUTF8	疟
U+759F.0	fGB	0-4531
U+759F.0	fR/S	 104.3
U+759F.0	fTotalStrokes	 8
U+759F.0	fCangjie	KSM
U+759F.0	fHakka	 ngiok7
U+759F.0	fCantonese	joek6
U+759F.0	fMandarin	yao4; nuue4; ne4; yao2
U+759F.0	fEnglish	simplified form of &#x7627;
U+75A0.0	fUTF8	疠
U+75A0.0	fGB	0-705D
U+75A0.0	fR/S	 104.3
U+75A0.0	fTotalStrokes	 8
U+75A0.0	fCangjie	KMS
U+75A0.0	fHakka	 li5
U+75A0.0	fCantonese	lai6
U+75A0.0	fMandarin	li4; ji1; lai4
U+75A0.0	fEnglish	simplified form of &#x7658;
U+75A1.0	fUTF8	疡
U+75A1.0	fGB	0-5171
U+75A1.0	fR/S	 104.4
U+75A1.0	fTotalStrokes	 9
U+75A1.0	fCangjie	KNSH
U+75A1.0	fHakka	 jong2
U+75A1.0	fCantonese	joeng4
U+75A1.0	fMandarin	yang2; dang4
U+75A1.0	fEnglish	simplified form of &#x760D;
U+75A2.0	fUTF8	疢
U+75A2.0	fCNS11643	1-5252
U+75A2.0	fBig5	ACCF
U+75A2.0	fGB	3-5E50
U+75A2.0	fR/S	 104.4
U+75A2.0	fTotalStrokes	 9
U+75A2.0	fCangjie	KF
U+75A2.0	fFourCorner	00189
U+75A2.0	fHakka	 zhin3; zihn3; chin3; cihn3
U+75A2.0	fCantonese	can3; can2 (also)
U+75A2.0	fMandarin	chen4; chen3
U+75A2.0	fEnglish	aching of limbs; muscular pains
U+75A3.0	fUTF8	疣
U+75A3.0	fCNS11643	1-5253
U+75A3.0	fBig5	ACD0
U+75A3.0	fGB	0-7060
U+75A3.0	fR/S	 104.4
U+75A3.0	fTotalStrokes	 9
U+75A3.0	fCangjie	KIKU
U+75A3.0	fFourCorner	00114
U+75A3.0	fHakka	  ju2; riu2; jiu2
U+75A3.0	fCantonese	jau4
U+75A3.0	fMandarin	you2; you4
U+75A3.0	fEnglish	 wart; papule
U+75A4.0	fUTF8	疤
U+75A4.0	fCNS11643	1-5250
U+75A4.0	fBig5	ACCD
U+75A4.0	fGB	0-304C
U+75A4.0	fR/S	 104.4
U+75A4.0	fTotalStrokes	 9
U+75A4.0	fCangjie	KAU
U+75A4.0	fFourCorner	00117
U+75A4.0	fHakka	  ba1
U+75A4.0	fCantonese	baa1; baan1
U+75A4.0	fMandarin	ba1
U+75A4.0	fEnglish	[1] scar [2] birthmark
U+75A5.0	fUTF8	疥
U+75A5.0	fCNS11643	1-5251
U+75A5.0	fBig5	ACCE
U+75A5.0	fGB	0-3D6A
U+75A5.0	fR/S	 104.4
U+75A5.0	fTotalStrokes	 9
U+75A5.0	fCangjie	KOLL
U+75A5.0	fFourCorner	00128
U+75A5.0	fHakka	  gai5; gai3; kie5; giai5
U+75A5.0	fCantonese	gaai3
U+75A5.0	fMandarin	jie4
U+75A5.0	fEnglish	scabies
U+75A6.0	fUTF8	疦
U+75A6.0	fCNS11643	3-2E2B
U+75A6.0	fGB	3-5E52
U+75A6.0	fR/S	 104.4
U+75A6.0	fTotalStrokes	 9
U+75A6.0	fCangjie	KDK
U+75A6.0	fFourCorner	00130
U+75A6.0	fCantonese	kyut3
U+75A6.0	fMandarin	jue2; xue4
U+75A7.0	fUTF8	疧
U+75A7.0	fCNS11643	2-2C3E
U+75A7.0	fBig5	CFDC
U+75A7.0	fGB	3-5E4E
U+75A7.0	fR/S	 104.4
U+75A7.0	fTotalStrokes	 9
U+75A7.0	fCangjie	KHVP
U+75A7.0	fFourCorner	00142
U+75A7.0	fHakka	 ki2; di3; ti3
U+75A7.0	fCantonese	kei4; zi1
U+75A7.0	fMandarin	zhi1; qi2; di3
U+75A7.0	fEnglish	sick
U+75A8.0	fUTF8	疨
U+75A8.0	fCNS11643	3-2E2F
U+75A8.0	fGB	3-5E4A
U+75A8.0	fR/S	 104.4
U+75A8.0	fTotalStrokes	 9
U+75A8.0	fCangjie	KMVH
U+75A8.0	fFourCorner	00120
U+75A8.0	fCantonese	haa1; ngaa4; aa4 (also)
U+75A8.0	fMandarin	xia1; ya2; ya3
U+75A9.0	fUTF8	疩
U+75A9.0	fCNS11643	4-2A4A
U+75A9.0	fGB	E-3431
U+75A9.0	fR/S	 104.4
U+75A9.0	fTotalStrokes	 9
U+75A9.0	fCangjie	KKNJ
U+75A9.0	fFourCorner	00140
U+75A9.0	fCantonese	seoi6
U+75A9.0	fMandarin	cui4
U+75AA.0	fUTF8	疪
U+75AA.0	fCNS11643	2-2C3F
U+75AA.0	fBig5	CFDD
U+75AA.0	fGB	5-5C68
U+75AA.0	fR/S	 104.4
U+75AA.0	fTotalStrokes	 9
U+75AA.0	fCangjie	KPP
U+75AA.0	fFourCorner	00111
U+75AA.0	fHakka	 bi5
U+75AA.0	fCantonese	bei3
U+75AA.0	fMandarin	bi4
U+75AB.0	fUTF8	疫
U+75AB.0	fCNS11643	1-524F
U+75AB.0	fBig5	ACCC
U+75AB.0	fGB	0-525F
U+75AB.0	fR/S	 104.4
U+75AB.0	fTotalStrokes	 9
U+75AB.0	fCangjie	KHNE
U+75AB.0	fFourCorner	00147
U+75AB.0	fHakka	 jit8; mut8; rit8; {mut7}
U+75AB.0	fCantonese	jik6
U+75AB.0	fMandarin	yi4; wei4
U+75AB.0	fEnglish	 epidemic; plague; pestilence
U+75AC.0	fUTF8	疬
U+75AC.0	fGB	0-705F
U+75AC.0	fR/S	 104.4
U+75AC.0	fTotalStrokes	 9
U+75AC.0	fCangjie	KMKS
U+75AC.0	fHakka	 let8
U+75AC.0	fCantonese	lik6; lek6 (also)
U+75AC.0	fMandarin	li4
U+75AC.0	fEnglish	simplified form of &#x7667;
U+75AD.0	fUTF8	疭
U+75AD.0	fGB	7-212E
U+75AD.0	fR/S	 104.4
U+75AD.0	fTotalStrokes	 9
U+75AD.0	fCangjie	XKOO
U+75AD.0	fCantonese	zung3
U+75AD.0	fMandarin	zong4
U+75AD.0	fEnglish	spasms or convulsions in young children, caused by indigestion
U+75AE.0	fUTF8	疮
U+75AE.0	fGB	0-342F
U+75AE.0	fR/S	 104.4
U+75AE.0	fTotalStrokes	 9
U+75AE.0	fCangjie	KOSU
U+75AE.0	fHakka	 cong1
U+75AE.0	fCantonese	cong1
U+75AE.0	fMandarin	chuang1; chuan1
U+75AE.0	fEnglish	simplified form of &#x7621;
U+75AF.0	fUTF8	疯
U+75AF.0	fGB	0-3768
U+75AF.0	fR/S	 104.4
U+75AF.0	fTotalStrokes	 9
U+75AF.0	fCangjie	KHNK
U+75AF.0	fHakka	 fung1
U+75AF.0	fCantonese	fung1
U+75AF.0	fMandarin	feng1
U+75AF.0	fEnglish	simplified form of &#x760B;
U+75B0.0	fUTF8	疰
U+75B0.0	fCNS11643	2-3126
U+75B0.0	fBig5	D2C3
U+75B0.0	fGB	0-7066
U+75B0.0	fR/S	 104.5
U+75B0.0	fTotalStrokes	 10
U+75B0.0	fCangjie	KYG
U+75B0.0	fFourCorner	00114
U+75B0.0	fHakka	 zhu5; zu5
U+75B0.0	fCantonese	zyu3
U+75B0.0	fMandarin	zhu4
U+75B1.0	fUTF8	疱
U+75B1.0	fCNS11643	3-3257
U+75B1.0	fGB	0-7065
U+75B1.0	fR/S	 104.5
U+75B1.0	fTotalStrokes	 10
U+75B1.0	fCangjie	KPRU
U+75B1.0	fFourCorner	00110
U+75B1.0	fHakka	 pau5
U+75B1.0	fCantonese	paau3
U+75B1.0	fMandarin	pao4
U+75B1.0	fEnglish	acne
U+75B2.0	fUTF8	疲
U+75B2.0	fCNS11643	1-566B
U+75B2.0	fBig5	AF68
U+75B2.0	fGB	0-4623
U+75B2.0	fR/S	 104.5
U+75B2.0	fTotalStrokes	 10
U+75B2.0	fCangjie	KDHE
U+75B2.0	fFourCorner	00147
U+75B2.0	fHakka	  pi2; pi1
U+75B2.0	fCantonese	pei4
U+75B2.0	fMandarin	pi2; zhi1
U+75B2.0	fEnglish	feel tired; be exhausted; weary; fatigued
U+75B3.0	fUTF8	疳
U+75B3.0	fCNS11643	1-566C
U+75B3.0	fBig5	AF69
U+75B3.0	fGB	0-7061
U+75B3.0	fR/S	 104.5
U+75B3.0	fTotalStrokes	 10
U+75B3.0	fCangjie	KTM
U+75B3.0	fFourCorner	00175
U+75B3.0	fHakka	  gam1
U+75B3.0	fCantonese	gam1
U+75B3.0	fMandarin	gan1
U+75B3.0	fEnglish	[1] childhood diseases; a kind of infantile disease caused by digestive trouble or malnutrition [2] a kind of infectious venereal disease
U+75B4.0	fUTF8	疴
U+75B4.0	fCNS11643	3-3255
U+75B4.0	fGB	0-7062
U+75B4.0	fR/S	  104.5
U+75B4.0	fTotalStrokes	 10
U+75B4.0	fCangjie	KMNR
U+75B4.0	fFourCorner	00120
U+75B4.0	fHakka	  o1; ko1
U+75B4.0	fCantonese	ngo1 (also); o1
U+75B4.0	fMandarin	ke1; e1; qia4
U+75B4.0	fEnglish	sickness; disease
U+75B5.0	fUTF8	疵
U+75B5.0	fCNS11643	1-5C30
U+75B5.0	fBig5	B2AB
U+75B5.0	fGB	0-3443
U+75B5.0	fR/S	 104.5
U+75B5.0	fTotalStrokes	 10
U+75B5.0	fCangjie	KYMP
U+75B5.0	fFourCorner	00111
U+75B5.0	fHakka	 zih1; cih2; zu1; ci1; zii1
U+75B5.0	fCantonese	ci1; ci4 (also)
U+75B5.0	fMandarin	ci1; ji4; ci2; zi1; zhai4; zi3
U+75B5.0	fEnglish	 flaw; fault; defect disease
U+75B6.0	fUTF8	疶
U+75B6.0	fCNS11643	2-312C
U+75B6.0	fBig5	D2C9
U+75B6.0	fGB	5-5C72
U+75B6.0	fR/S	 104.5
U+75B6.0	fTotalStrokes	 10
U+75B6.0	fCangjie	KPT
U+75B6.0	fFourCorner	00117
U+75B6.0	fHakka	  siet7
U+75B6.0	fCantonese	sit3
U+75B6.0	fMandarin	xie4; yi4
U+75B6.0	fEnglish	dysentery
U+75B7.0	fUTF8	疷
U+75B7.0	fCNS11643	4-2E40
U+75B7.0	fGB	5-5C78
U+75B7.0	fR/S	 104.5
U+75B7.0	fTotalStrokes	 10
U+75B7.0	fCangjie	KHVI
U+75B7.0	fFourCorner	00140
U+75B7.0	fHakka	 di3;  ki2
U+75B7.0	fCantonese	zi1
U+75B7.0	fMandarin	qi2
U+75B7.0	fEnglish	disease; sickness
U+75B8.0	fUTF8	疸
U+75B8.0	fCNS11643	1-5671
U+75B8.0	fBig5	AF6E
U+75B8.0	fGB	0-7063
U+75B8.0	fR/S	 104.5
U+75B8.0	fTotalStrokes	 10
U+75B8.0	fCangjie	KAM
U+75B8.0	fFourCorner	00116
U+75B8.0	fHakka	 tan3;  dan5; tan5
U+75B8.0	fCantonese	taan2; daap3; daam2; daan3
U+75B8.0	fMandarin	dan3; da5; dan4
U+75B8.0	fEnglish	jaundice
U+75B9.0	fUTF8	疹
U+75B9.0	fCNS11643	1-566F
U+75B9.0	fBig5	AF6C
U+75B9.0	fGB	0-556E
U+75B9.0	fR/S	 104.5
U+75B9.0	fTotalStrokes	 10
U+75B9.0	fCangjie	KOHH
U+75B9.0	fFourCorner	00122
U+75B9.0	fHakka	 zhin3; zin3; zihn3; zin1
U+75B9.0	fCantonese	can2; zan2 (also)
U+75B9.0	fMandarin	zhen3; chen4; jin3; nie4
U+75B9.0	fEnglish	 measles rash fever
U+75BA.0	fUTF8	疺
U+75BA.0	fCNS11643	2-312D
U+75BA.0	fBig5	D2CA
U+75BA.0	fGB	3-5E4D
U+75BA.0	fR/S	 104.4
U+75BA.0	fTotalStrokes	 9
U+75BA.0	fCangjie	KHIO
U+75BA.0	fFourCorner	00137
U+75BA.0	fHakka	  fat8
U+75BA.0	fCantonese	fat6
U+75BA.0	fMandarin	fa2; bian3
U+75BA.0	fEnglish	weary; exhausted
U+75BB.0	fUTF8	疻
U+75BB.0	fCNS11643	2-3128
U+75BB.0	fBig5	D2C5
U+75BB.0	fGB	3-5E58
U+75BB.0	fR/S	 104.5
U+75BB.0	fTotalStrokes	 10
U+75BB.0	fCangjie	KRC
U+75BB.0	fFourCorner	00186
U+75BB.0	fHakka	 zhi3; zih3; tje3
U+75BB.0	fCantonese	zi2; zi1
U+75BB.0	fMandarin	zhi3; shi1; zhi1
U+75BB.0	fEnglish	bruise; bump
U+75BC.0	fUTF8	疼
U+75BC.0	fCNS11643	1-566E
U+75BC.0	fBig5	AF6B
U+75BC.0	fGB	0-4C5B
U+75BC.0	fR/S	 104.5
U+75BC.0	fTotalStrokes	 10
U+75BC.0	fCangjie	KHEY
U+75BC.0	fFourCorner	00133
U+75BC.0	fHakka	 dung1; tung1; tung5; ten2; dung5
U+75BC.0	fCantonese	tang4; tung3 (also); tung4
U+75BC.0	fMandarin	teng2; tong2
U+75BC.0	fEnglish	[1] ache; hurt; pain; sore [2] dote on; be fond of (a child); love
U+75BD.0	fUTF8	疽
U+75BD.0	fCNS11643	1-566D
U+75BD.0	fBig5	AF6A
U+75BD.0	fGB	0-3E52
U+75BD.0	fR/S	 104.5
U+75BD.0	fTotalStrokes	 10
U+75BD.0	fCangjie	KBM
U+75BD.0	fFourCorner	00117
U+75BD.0	fHakka	 zu1; zii1; zih1
U+75BD.0	fCantonese	zeoi1; ceoi1
U+75BD.0	fMandarin	ju1; ju3
U+75BD.0	fEnglish	ulcer; abscess
U+75BE.0	fUTF8	疾
U+75BE.0	fCNS11643	1-5668
U+75BE.0	fBig5	AF65
U+75BE.0	fGB	0-3C32
U+75BE.0	fR/S	 104.5
U+75BE.0	fTotalStrokes	 10
U+75BE.0	fCangjie	KOK
U+75BE.0	fFourCorner	00134
U+75BE.0	fHakka	  cit8; cit7
U+75BE.0	fCantonese	zat6; zaat6
U+75BE.0	fMandarin	ji2
U+75BE.0	fEnglish	[1] illness; disease; sickness; suffering [2] hate; detest [3] swift; rapid; quick; fast
U+75BF.0	fUTF8	疿
U+75BF.0	fCNS11643	2-312B
U+75BF.0	fBig5	D2C8
U+75BF.0	fGB	E-3432
U+75BF.0	fR/S	 104.5
U+75BF.0	fTotalStrokes	 10
U+75BF.0	fCangjie	KLLN
U+75BF.0	fFourCorner	00127
U+75BF.0	fHakka	 bi5; bui5; fui5; fi5
U+75BF.0	fCantonese	fai2; fai3 (also); fai6 (also); fei2 (also)
U+75BF.0	fMandarin	fei4; fei2
U+75BF.0	fEnglish	prickly heat; rash
U+75C0.0	fUTF8	痀
U+75C0.0	fCNS11643	2-312A
U+75C0.0	fBig5	D2C7
U+75C0.0	fGB	5-5C79
U+75C0.0	fR/S	 104.5
U+75C0.0	fTotalStrokes	 10
U+75C0.0	fCangjie	KPR
U+75C0.0	fFourCorner	00127
U+75C0.0	fHakka	  ki1;  gi1; gu1; ku1; gu2; ku2
U+75C0.0	fCantonese	keoi1; keoi4; jyu2
U+75C0.0	fMandarin	qu2; ju1; yu3; gou1; ju4
U+75C0.0	fEnglish	 humpback; hunchback
U+75C1.0	fUTF8	痁
U+75C1.0	fCNS11643	2-3127
U+75C1.0	fBig5	D2C4
U+75C1.0	fGB	3-5E56
U+75C1.0	fR/S	 104.5
U+75C1.0	fTotalStrokes	 10
U+75C1.0	fCangjie	KYR
U+75C1.0	fFourCorner	00162
U+75C1.0	fHakka	  diam1; diam5
U+75C1.0	fCantonese	dim3; sim1 (also); zim1
U+75C1.0	fMandarin	dian4; shan1; shan4
U+75C1.0	fEnglish	chronic malaria
U+75C2.0	fUTF8	痂
U+75C2.0	fCNS11643	1-5670
U+75C2.0	fBig5	AF6D
U+75C2.0	fGB	0-7068
U+75C2.0	fR/S	 104.5
U+75C2.0	fTotalStrokes	 10
U+75C2.0	fCangjie	KKSR
U+75C2.0	fFourCorner	00160
U+75C2.0	fHakka	  ga1; kio2
U+75C2.0	fCantonese	gaa1; ge1
U+75C2.0	fMandarin	jia1
U+75C2.0	fEnglish	scab over a sore
U+75C3.0	fUTF8	痃
U+75C3.0	fCNS11643	3-3252
U+75C3.0	fGB	0-7067
U+75C3.0	fR/S	 104.5
U+75C3.0	fTotalStrokes	 10
U+75C3.0	fCangjie	KYVI
U+75C3.0	fFourCorner	00130
U+75C3.0	fHakka	  hien2
U+75C3.0	fCantonese	jin4; jyun4 (also)
U+75C3.0	fMandarin	xian2; xuan2
U+75C3.0	fEnglish	bubo (pathology)
U+75C4.0	fUTF8	痄
U+75C4.0	fCNS11643	2-3129
U+75C4.0	fBig5	D2C6
U+75C4.0	fGB	0-7064
U+75C4.0	fR/S	 104.5
U+75C4.0	fTotalStrokes	 10
U+75C4.0	fCangjie	KOS
U+75C4.0	fFourCorner	00111
U+75C4.0	fHakka	 ca2;  za5
U+75C4.0	fCantonese	zaa3
U+75C4.0	fMandarin	zha4; cha2; zha3; chi2
U+75C4.0	fEnglish	scrofulous swellings and sores
U+75C5.0	fUTF8	病
U+75C5.0	fCNS11643	1-5669
U+75C5.0	fBig5	AF66
U+75C5.0	fGB	0-3221
U+75C5.0	fR/S	 104.5
U+75C5.0	fTotalStrokes	 10
U+75C5.0	fCangjie	KMOB
U+75C5.0	fFourCorner	00127
U+75C5.0	fHakka	 piang5; piang6; piang3
U+75C5.0	fCantonese	beng6; bing6 (also)
U+75C5.0	fMandarin	bing4
U+75C5.0	fEnglish	[1] illness; sickness; disease; ailment [2] be ill [3] blemish; fault [4] injure; harm [5] worry [6] hate [7] insult
U+75C6.0	fUTF8	痆
U+75C6.0	fCNS11643	4-2E34
U+75C6.0	fGB	3-5E5E
U+75C6.0	fR/S	  104.5
U+75C6.0	fTotalStrokes	 10
U+75C6.0	fCangjie	KSP
U+75C6.0	fFourCorner	00110
U+75C6.0	fCantonese	naat6; laat6 (also)
U+75C6.0	fMandarin	ni4; nie4; na4
U+75C7.0	fUTF8	症
U+75C7.0	fCNS11643	1-566A
U+75C7.0	fBig5	AF67
U+75C7.0	fGB	0-5622
U+75C7.0	fR/S	 104.5
U+75C7.0	fTotalStrokes	 10
U+75C7.0	fCangjie	KMYM
U+75C7.0	fFourCorner	00111
U+75C7.0	fHakka	 zin5;  zhin5; zihn5; zhin1
U+75C7.0	fCantonese	zing3; zing1
U+75C7.0	fMandarin	zheng1; zheng4
U+75C7.0	fEnglish	[1] disease; illness; ailment [2] symptoms or manifestations of a disease [3] simplified form of &#x7665;
U+75C8.0	fUTF8	痈
U+75C8.0	fGB	0-5338
U+75C8.0	fR/S	 104.5
U+75C8.0	fTotalStrokes	 10
U+75C8.0	fCangjie	KBQ
U+75C8.0	fHakka	 jung1
U+75C8.0	fCantonese	jung1
U+75C8.0	fMandarin	yong1
U+75C8.0	fEnglish	simplified form of &#x7670;
U+75C9.0	fUTF8	痉
U+75C9.0	fGB	0-3E37
U+75C9.0	fR/S	 104.5
U+75C9.0	fTotalStrokes	 10
U+75C9.0	fCangjie	KNOM
U+75C9.0	fCantonese	ging6 (also); ging3
U+75C9.0	fMandarin	jing4
U+75C9.0	fEnglish	simplified form of &#x75D9;
U+75CA.0	fUTF8	痊
U+75CA.0	fCNS11643	1-5C31
U+75CA.0	fBig5	B2AC
U+75CA.0	fGB	0-482C
U+75CA.0	fR/S	 104.6
U+75CA.0	fTotalStrokes	 11
U+75CA.0	fCangjie	KOMG
U+75CA.0	fFourCorner	00114
U+75CA.0	fHakka	  cien2; cion2; cen2
U+75CA.0	fCantonese	cyun4
U+75CA.0	fMandarin	quan2
U+75CA.0	fEnglish	healed; cured; recovery
U+75CB.0	fUTF8	痋
U+75CB.0	fCNS11643	2-3744
U+75CB.0	fBig5	D6A1
U+75CB.0	fGB	5-5D23
U+75CB.0	fR/S	 104.6
U+75CB.0	fTotalStrokes	 11
U+75CB.0	fCangjie	KLMI
U+75CB.0	fFourCorner	00136
U+75CB.0	fCantonese	tang4; cung4; tung4
U+75CB.0	fMandarin	chong2; tong2
U+75CC.0	fUTF8	痌
U+75CC.0	fCNS11643	2-3745
U+75CC.0	fBig5	D6A2
U+75CC.0	fGB	5-5D24
U+75CC.0	fR/S	 104.6
U+75CC.0	fTotalStrokes	 11
U+75CC.0	fCangjie	KBMR
U+75CC.0	fFourCorner	00127
U+75CC.0	fHakka	  tung2
U+75CC.0	fCantonese	tung1
U+75CC.0	fMandarin	tong1; tong2
U+75CC.0	fEnglish	aching; in pain; painful
U+75CD.0	fUTF8	痍
U+75CD.0	fCNS11643	1-5C32
U+75CD.0	fBig5	B2AD
U+75CD.0	fGB	0-706A
U+75CD.0	fR/S	 104.6
U+75CD.0	fTotalStrokes	 11
U+75CD.0	fCangjie	KKN
U+75CD.0	fFourCorner	00132
U+75CD.0	fHakka	 ji2; ri2; i2
U+75CD.0	fCantonese	ji4
U+75CD.0	fMandarin	yi2
U+75CD.0	fEnglish	 wound; bruise; sore
U+75CE.0	fUTF8	痎
U+75CE.0	fCNS11643	2-3741
U+75CE.0	fBig5	D67C
U+75CE.0	fGB	3-5E6A
U+75CE.0	fR/S	 104.6
U+75CE.0	fTotalStrokes	 11
U+75CE.0	fCangjie	KYVO
U+75CE.0	fFourCorner	00182
U+75CE.0	fHakka	  gai1;  goi1
U+75CE.0	fCantonese	gaai1
U+75CE.0	fMandarin	kai1; jie4; lie1; hai4; jie1
U+75CE.0	fEnglish	 intermittent fever
U+75CF.0	fUTF8	痏
U+75CF.0	fCNS11643	2-3743
U+75CF.0	fBig5	D67E
U+75CF.0	fGB	3-5E62
U+75CF.0	fR/S	 104.6
U+75CF.0	fTotalStrokes	 11
U+75CF.0	fCangjie	KKB
U+75CF.0	fFourCorner	00127
U+75CF.0	fHakka	 ju3; riu3; wui6; wi5; vui3; wi3; wui3
U+75CF.0	fCantonese	fui2; wai5 (also)
U+75CF.0	fMandarin	wei3; you4; yu4
U+75CF.0	fEnglish	 mark on the skin; scar
U+75D0.0	fUTF8	痐
U+75D0.0	fCNS11643	2-3747
U+75D0.0	fBig5	D6A4
U+75D0.0	fGB	E-3433
U+75D0.0	fR/S	 104.6
U+75D0.0	fTotalStrokes	 11
U+75D0.0	fCangjie	KWR
U+75D0.0	fFourCorner	00160
U+75D0.0	fHakka	 fui2; fi2
U+75D0.0	fCantonese	wui4
U+75D0.0	fMandarin	hui2
U+75D0.0	fEnglish	whirlwind; cyclone; storm
U+75D1.0	fUTF8	痑
U+75D1.0	fCNS11643	2-3746
U+75D1.0	fBig5	D6A3
U+75D1.0	fGB	3-5E69
U+75D1.0	fR/S	 104.6
U+75D1.0	fTotalStrokes	 11
U+75D1.0	fCangjie	KNIN
U+75D1.0	fFourCorner	00127
U+75D1.0	fHakka	 tan3; to3
U+75D1.0	fCantonese	taan1; taan2
U+75D1.0	fMandarin	duo3; duo4; tan1; tan4; tuo1
U+75D1.0	fEnglish	worn out; jaded
U+75D2.0	fUTF8	痒
U+75D2.0	fCNS11643	2-3742
U+75D2.0	fBig5	D67D
U+75D2.0	fGB	0-5177
U+75D2.0	fR/S	 104.6
U+75D2.0	fTotalStrokes	 11
U+75D2.0	fCangjie	KTQ
U+75D2.0	fFourCorner	00151
U+75D2.0	fHakka	 jong2; siong3; siong2; rong2; jong3; {jong1}
U+75D2.0	fCantonese	joeng4; joeng5
U+75D2.0	fMandarin	yang3; yang2
U+75D2.0	fEnglish	simplified form of &#x7662;
U+75D3.0	fUTF8	痓
U+75D3.0	fCNS11643	3-374A
U+75D3.0	fGB	3-5E65
U+75D3.0	fR/S	 104.6
U+75D3.0	fTotalStrokes	 11
U+75D3.0	fCangjie	KMIG
U+75D3.0	fFourCorner	00110
U+75D3.0	fCantonese	ci3
U+75D3.0	fMandarin	chi4
U+75D4.0	fUTF8	痔
U+75D4.0	fCNS11643	1-5C2E
U+75D4.0	fBig5	B2A9
U+75D4.0	fGB	0-564C
U+75D4.0	fR/S	 104.6
U+75D4.0	fTotalStrokes	 11
U+75D4.0	fCangjie	KGDI
U+75D4.0	fFourCorner	00141
U+75D4.0	fHakka	 cih5; chi6; ci5; zi3; chi5; chii5
U+75D4.0	fCantonese	zi6
U+75D4.0	fMandarin	zhi4
U+75D4.0	fEnglish	hemorrhoids; piles
U+75D5.0	fUTF8	痕
U+75D5.0	fCNS11643	1-5C2F
U+75D5.0	fBig5	B2AA
U+75D5.0	fGB	0-3A5B
U+75D5.0	fR/S	 104.6
U+75D5.0	fTotalStrokes	 11
U+75D5.0	fCangjie	KAV
U+75D5.0	fFourCorner	00132
U+75D5.0	fHakka	  hen2; hien2;  hen1; hiun2
U+75D5.0	fCantonese	han4; han2 (also)
U+75D5.0	fMandarin	hen2; gen4; yin2
U+75D5.0	fEnglish	 scar; mark; trace
U+75D6.0	fUTF8	痖
U+75D6.0	fGB	0-7069
U+75D6.0	fR/S	 104.6
U+75D6.0	fTotalStrokes	 11
U+75D6.0	fCangjie	KMTC
U+75D6.0	fHakka	 a3
U+75D6.0	fCantonese	aa2; ngaa2 (also)
U+75D6.0	fMandarin	ya3
U+75D6.0	fEnglish	simplified form of &#x7602;
U+75D7.0	fUTF8	痗
U+75D7.0	fCNS11643	2-3E3A
U+75D7.0	fBig5	DAB6
U+75D7.0	fGB	3-5E75
U+75D7.0	fR/S	 104.7
U+75D7.0	fTotalStrokes	 12
U+75D7.0	fCangjie	KOWY
U+75D7.0	fFourCorner	00157
U+75D7.0	fHakka	 mui3; mi5; mui5
U+75D7.0	fCantonese	mui6; fui3
U+75D7.0	fMandarin	mei4; hui3; hui4; mei3
U+75D7.0	fEnglish	ill
U+75D8.0	fUTF8	痘
U+75D8.0	fCNS11643	1-6071
U+75D8.0	fBig5	B56B
U+75D8.0	fGB	0-363B
U+75D8.0	fR/S	 104.7
U+75D8.0	fTotalStrokes	 12
U+75D8.0	fCangjie	KMRT
U+75D8.0	fFourCorner	00118
U+75D8.0	fHakka	 teu5; teu6; tiu3
U+75D8.0	fCantonese	dau6; dau2 (also)
U+75D8.0	fMandarin	dou4
U+75D8.0	fEnglish	smallpox
U+75D9.0	fUTF8	痙
U+75D9.0	fCNS11643	1-6070
U+75D9.0	fBig5	B56A
U+75D9.0	fGB	1-3E37
U+75D9.0	fR/S	 104.7
U+75D9.0	fTotalStrokes	 12
U+75D9.0	fCangjie	KMVM
U+75D9.0	fFourCorner	00111
U+75D9.0	fHakka	 gin5
U+75D9.0	fCantonese	ging6 (also); ging3
U+75D9.0	fMandarin	jing4
U+75D9.0	fEnglish	convulsions; spasm
U+75DA.0	fUTF8	痚
U+75DA.0	fCNS11643	2-3E34
U+75DA.0	fBig5	DAB0
U+75DA.0	fGB	5-5D2B
U+75DA.0	fR/S	 104.7
U+75DA.0	fTotalStrokes	 12
U+75DA.0	fCangjie	KJKD
U+75DA.0	fFourCorner	00147
U+75DA.0	fHakka	  hiau1; hau1; hieu1
U+75DA.0	fCantonese	haau1
U+75DA.0	fMandarin	xiao1
U+75DA.0	fEnglish	asthma
U+75DB.0	fUTF8	痛
U+75DB.0	fCNS11643	1-606E
U+75DB.0	fBig5	B568
U+75DB.0	fGB	0-4D34
U+75DB.0	fR/S	 104.7
U+75DB.0	fTotalStrokes	 12
U+75DB.0	fCangjie	KNIB
U+75DB.0	fFourCorner	00127
U+75DB.0	fHakka	  tung5
U+75DB.0	fCantonese	tung3
U+75DB.0	fMandarin	tong4
U+75DB.0	fEnglish	[1] painful; aching [2] sorrowful; sad; bitter; poignant; bitterly [3] heartily; to one's heart's content
U+75DC.0	fUTF8	痜
U+75DC.0	fCNS11643	4-3934
U+75DC.0	fGB	E-3434
U+75DC.0	fR/S	 104.7
U+75DC.0	fTotalStrokes	 12
U+75DC.0	fCangjie	XKHDN
U+75DC.0	fFourCorner	00110
U+75DC.0	fHakka	  tut7
U+75DC.0	fCantonese	tuk1
U+75DC.0	fMandarin	tu1
U+75DC.0	fEnglish	a scabby bald head
U+75DD.0	fUTF8	痝
U+75DD.0	fCNS11643	2-3E37
U+75DD.0	fBig5	DAB3
U+75DD.0	fGB	3-5E71
U+75DD.0	fR/S	 104.7
U+75DD.0	fTotalStrokes	 12
U+75DD.0	fCangjie	KIUH
U+75DD.0	fFourCorner	00112
U+75DD.0	fCantonese	mong4
U+75DD.0	fMandarin	mang2
U+75DE.0	fUTF8	痞
U+75DE.0	fCNS11643	1-6072
U+75DE.0	fBig5	B56C
U+75DE.0	fGB	0-4626
U+75DE.0	fR/S	 104.7
U+75DE.0	fTotalStrokes	 12
U+75DE.0	fCangjie	KMFR
U+75DE.0	fFourCorner	00169
U+75DE.0	fHakka	  pi3; peu3; pi1
U+75DE.0	fCantonese	pei2; mau1; fau2; pei5
U+75DE.0	fMandarin	pi3
U+75DE.0	fEnglish	[1] dyspepsia; spleen infection [2] ruffian; scoundrel
U+75DF.0	fUTF8	痟
U+75DF.0	fCNS11643	2-3E38
U+75DF.0	fBig5	DAB4
U+75DF.0	fGB	3-5E72
U+75DF.0	fR/S	 104.7
U+75DF.0	fTotalStrokes	 12
U+75DF.0	fCangjie	KFB
U+75DF.0	fFourCorner	00127
U+75DF.0	fHakka	  siau1; seu1
U+75DF.0	fCantonese	siu1
U+75DF.0	fMandarin	xiao1
U+75DF.0	fEnglish	excessive thirst
U+75E0.0	fUTF8	痠
U+75E0.0	fCNS11643	1-6073
U+75E0.0	fBig5	B56D
U+75E0.0	fGB	5-5D38
U+75E0.0	fR/S	 104.7
U+75E0.0	fTotalStrokes	 12
U+75E0.0	fCangjie	KICE
U+75E0.0	fFourCorner	00147
U+75E0.0	fHakka	  son1
U+75E0.0	fCantonese	syun1
U+75E0.0	fMandarin	suan1
U+75E0.0	fEnglish	aching of limbs; muscular pains
U+75E1.0	fUTF8	痡
U+75E1.0	fCNS11643	2-3E35
U+75E1.0	fBig5	DAB1
U+75E1.0	fGB	3-5E6F
U+75E1.0	fR/S	 104.7
U+75E1.0	fTotalStrokes	 12
U+75E1.0	fCangjie	KIJB
U+75E1.0	fFourCorner	00127
U+75E1.0	fHakka	 pu2; pu1; pu3
U+75E1.0	fCantonese	pou1; fu1 (also)
U+75E1.0	fMandarin	pu1; bu1; fu1; fu4; pu4
U+75E1.0	fEnglish	 disease; ailment
U+75E2.0	fUTF8	痢
U+75E2.0	fCNS11643	1-606D
U+75E2.0	fBig5	B567
U+75E2.0	fGB	0-4121
U+75E2.0	fR/S	 104.7
U+75E2.0	fTotalStrokes	 12
U+75E2.0	fCangjie	KHDN
U+75E2.0	fFourCorner	00120
U+75E2.0	fHakka	  li5; li6; li3
U+75E2.0	fCantonese	lei6; lei1
U+75E2.0	fMandarin	li4
U+75E2.0	fEnglish	dysentery; diarrhea
U+75E3.0	fUTF8	痣
U+75E3.0	fCNS11643	1-606F
U+75E3.0	fBig5	B569
U+75E3.0	fGB	0-706B
U+75E3.0	fR/S	 104.7
U+75E3.0	fTotalStrokes	 12
U+75E3.0	fCangjie	KGP
U+75E3.0	fFourCorner	00131
U+75E3.0	fHakka	 zi5;  zhi5; zih5; tje5
U+75E3.0	fCantonese	zi3
U+75E3.0	fMandarin	zhi4
U+75E3.0	fEnglish	spots; moles; nevus
U+75E4.0	fUTF8	痤
U+75E4.0	fCNS11643	2-3E39
U+75E4.0	fBig5	DAB5
U+75E4.0	fGB	0-706E
U+75E4.0	fR/S	 104.7
U+75E4.0	fTotalStrokes	 12
U+75E4.0	fCangjie	KOOG
U+75E4.0	fFourCorner	00114
U+75E4.0	fHakka	 co2
U+75E4.0	fCantonese	co4
U+75E4.0	fMandarin	cuo2
U+75E4.0	fEnglish	[1] a boil or carbuncle on the face [2] minor swelling; abscess
U+75E5.0	fUTF8	痥
U+75E5.0	fCNS11643	3-3D47
U+75E5.0	fGB	3-5E78
U+75E5.0	fR/S	  104.7
U+75E5.0	fTotalStrokes	 12
U+75E5.0	fCangjie	KCRU
U+75E5.0	fFourCorner	00110
U+75E5.0	fCantonese	dyut6
U+75E5.0	fMandarin	duo2
U+75E6.0	fUTF8	痦
U+75E6.0	fCNS11643	2-3E36
U+75E6.0	fBig5	DAB2
U+75E6.0	fGB	0-706D
U+75E6.0	fR/S	 104.7
U+75E6.0	fTotalStrokes	 12
U+75E6.0	fCangjie	KMMR
U+75E6.0	fFourCorner	00161
U+75E6.0	fCantonese	ng6; ng4
U+75E6.0	fMandarin	wu4; pi1
U+75E7.0	fUTF8	痧
U+75E7.0	fCNS11643	2-3E33
U+75E7.0	fBig5	DAAF
U+75E7.0	fGB	0-7070
U+75E7.0	fR/S	 104.7
U+75E7.0	fTotalStrokes	 12
U+75E7.0	fCangjie	KEFH
U+75E7.0	fFourCorner	00129
U+75E7.0	fHakka	  sa1
U+75E7.0	fCantonese	saa1
U+75E7.0	fMandarin	sha1
U+75E7.0	fEnglish	[1] cholera [2] measles
U+75E8.0	fUTF8	痨
U+75E8.0	fGB	0-706C
U+75E8.0	fR/S	 104.7
U+75E8.0	fTotalStrokes	 12
U+75E8.0	fCangjie	KTBS
U+75E8.0	fHakka	 lau2
U+75E8.0	fCantonese	lou4
U+75E8.0	fMandarin	lao2; lao4; liao2
U+75E8.0	fEnglish	simplified form of &#x7646;
U+75E9.0	fUTF8	痩
U+75E9.0	fGB	E-3435
U+75E9.0	fR/S	 104.7
U+75E9.0	fTotalStrokes	 12
U+75E9.0	fCangjie	KLWE
U+75E9.0	fCantonese	sau3
U+75E9.0	fMandarin	shou4
U+75E9.0	fEnglish	 thin; emaciated; lean meager
U+75EA.0	fUTF8	痪
U+75EA.0	fGB	0-3B3E
U+75EA.0	fR/S	 104.7
U+75EA.0	fTotalStrokes	 12
U+75EA.0	fCangjie	XKNBK
U+75EA.0	fHakka	 fon5
U+75EA.0	fCantonese	wun6
U+75EA.0	fMandarin	huan4; tuan3
U+75EA.0	fEnglish	simplified form of &#x7613;
U+75EB.0	fUTF8	痫
U+75EB.0	fGB	0-706F
U+75EB.0	fR/S	 104.7
U+75EB.0	fTotalStrokes	 12
U+75EB.0	fCangjie	KLSD
U+75EB.0	fCantonese	haan4 (also); gaan2
U+75EB.0	fMandarin	xian2
U+75EB.0	fEnglish	simplified form of &#x7647;
U+75EC.0	fUTF8	痬
U+75EC.0	fCNS11643	4-3E77
U+75EC.0	fGB	3-5F22
U+75EC.0	fR/S	  104.8
U+75EC.0	fTotalStrokes	 13
U+75EC.0	fCangjie	KAPH
U+75EC.0	fFourCorner	00120
U+75EC.0	fCantonese	jik6
U+75EC.0	fMandarin	yi4
U+75ED.0	fUTF8	痭
U+75ED.0	fCNS11643	2-4538
U+75ED.0	fBig5	DED2
U+75ED.0	fGB	3-5F27
U+75ED.0	fR/S	 104.8
U+75ED.0	fTotalStrokes	 13
U+75ED.0	fCangjie	KBB
U+75ED.0	fFourCorner	00127
U+75ED.0	fHakka	 ben1
U+75ED.0	fCantonese	bang1
U+75ED.0	fMandarin	peng2; beng1; ping4
U+75ED.0	fEnglish	menorrhagia
U+75EE.0	fUTF8	痮
U+75EE.0	fCNS11643	3-427C
U+75EE.0	fGB	5-5C6B
U+75EE.0	fR/S	 104.8
U+75EE.0	fTotalStrokes	 13
U+75EE.0	fCangjie	KSMV
U+75EE.0	fFourCorner	00130
U+75EE.0	fHakka	 zhong5; zong5
U+75EE.0	fCantonese	zoeng3
U+75EE.0	fMandarin	zhang4
U+75EE.0	fEnglish	 a swelled belly dropsical; swell inflated
U+75EF.0	fUTF8	痯
U+75EF.0	fCNS11643	2-452D
U+75EF.0	fBig5	DEC7
U+75EF.0	fGB	3-5F29
U+75EF.0	fR/S	 104.8
U+75EF.0	fTotalStrokes	 13
U+75EF.0	fCangjie	KJRR
U+75EF.0	fFourCorner	00177
U+75EF.0	fHakka	 gon3;  kwan3; gwon3
U+75EF.0	fCantonese	gun2; gwun2 (also)
U+75EF.0	fMandarin	guan3; guan4
U+75EF.0	fEnglish	sick
U+75F0.0	fUTF8	痰
U+75F0.0	fCNS11643	1-6538
U+75F0.0	fBig5	B7F0
U+75F0.0	fGB	0-4C35
U+75F0.0	fR/S	 104.8
U+75F0.0	fTotalStrokes	 13
U+75F0.0	fCangjie	KFF
U+75F0.0	fFourCorner	00189
U+75F0.0	fHakka	  tam2; tam3
U+75F0.0	fCantonese	taam4
U+75F0.0	fMandarin	tan2
U+75F0.0	fEnglish	phlegm; mucus; spittle; sputum; expectoration
U+75F1.0	fUTF8	痱
U+75F1.0	fCNS11643	1-653B
U+75F1.0	fBig5	B7F3
U+75F1.0	fGB	0-7072
U+75F1.0	fR/S	 104.8
U+75F1.0	fTotalStrokes	 13
U+75F1.0	fCangjie	KLMY
U+75F1.0	fFourCorner	00111
U+75F1.0	fHakka	  fui1; fi1; pui1; fui5; bui5; bi5
U+75F1.0	fCantonese	fai2; fai6 (also); fei2 (also); fei4; fai3 (also); fei1 (also)
U+75F1.0	fMandarin	fei4; fei2; fei3; bei4
U+75F1.0	fEnglish	heat rashes; heat spots; prickly heat
U+75F2.0	fUTF8	痲
U+75F2.0	fCNS11643	1-653A
U+75F2.0	fBig5	B7F2
U+75F2.0	fGB	E-3436
U+75F2.0	fR/S	 104.8
U+75F2.0	fTotalStrokes	 13
U+75F2.0	fCangjie	KJCC
U+75F2.0	fFourCorner	00114
U+75F2.0	fHakka	 ma2; ma3
U+75F2.0	fCantonese	maa4; lam4
U+75F2.0	fMandarin	ma2
U+75F2.0	fEnglish	[1] leprosy [2] measles [3] stupefy; benumb; anesthetize; paralyze [4] pockmark
U+75F3.0	fUTF8	痳
U+75F3.0	fCNS11643	1-653F
U+75F3.0	fBig5	B7F7
U+75F3.0	fGB	8-2F78
U+75F3.0	fR/S	 104.8
U+75F3.0	fTotalStrokes	 13
U+75F3.0	fCangjie	KDD
U+75F3.0	fFourCorner	00194
U+75F3.0	fHakka	 lim2; lim3
U+75F3.0	fCantonese	maa4; lam4
U+75F3.0	fMandarin	lin2; lin4
U+75F3.0	fEnglish	gonorrhea
U+75F4.0	fUTF8	痴
U+75F4.0	fCNS11643	1-653E
U+75F4.0	fBig5	B7F6
U+75F4.0	fGB	0-3355
U+75F4.0	fR/S	 104.8
U+75F4.0	fTotalStrokes	 13
U+75F4.0	fCangjie	KOKR
U+75F4.0	fFourCorner	00160
U+75F4.0	fHakka	 chi1; ci1; cih1; chii1
U+75F4.0	fCantonese	ci1
U+75F4.0	fMandarin	chi1; chi2
U+75F4.0	fEnglish	simplified form of @
U+75F5.0	fUTF8	痵
U+75F5.0	fCNS11643	2-4539
U+75F5.0	fBig5	DED3
U+75F5.0	fGB	5-5D3E
U+75F5.0	fR/S	 104.8
U+75F5.0	fTotalStrokes	 13
U+75F5.0	fCangjie	KHDD
U+75F5.0	fFourCorner	00149
U+75F5.0	fHakka	  gi5; gwai5; kui3; kwui5
U+75F5.0	fCantonese	gwai3
U+75F5.0	fMandarin	ji4
U+75F5.0	fEnglish	a heart symptom
U+75F6.0	fUTF8	痶
U+75F6.0	fCNS11643	2-4537
U+75F6.0	fBig5	DED1
U+75F6.0	fGB	3-5F23
U+75F6.0	fR/S	 104.8
U+75F6.0	fTotalStrokes	 13
U+75F6.0	fCangjie	KTBC
U+75F6.0	fFourCorner	00181
U+75F6.0	fHakka	 tien3; {tiam3}; diam3; ten3
U+75F6.0	fCantonese	din2
U+75F6.0	fMandarin	dian3; tian3
U+75F7.0	fUTF8	痷
U+75F7.0	fCNS11643	2-4530
U+75F7.0	fBig5	DECA
U+75F7.0	fGB	3-5E7D
U+75F7.0	fR/S	 104.8
U+75F7.0	fTotalStrokes	 13
U+75F7.0	fCangjie	KKLU
U+75F7.0	fFourCorner	00216
U+75F7.0	fCantonese	am1; ngam1 (also); jip3
U+75F7.0	fMandarin	an1; ye4
U+75F8.0	fUTF8	痸
U+75F8.0	fCNS11643	2-4534
U+75F8.0	fBig5	DECE
U+75F8.0	fGB	3-5F26
U+75F8.0	fR/S	 104.8
U+75F8.0	fTotalStrokes	 13
U+75F8.0	fCangjie	KHBN
U+75F8.0	fFourCorner	00120
U+75F8.0	fHakka	 chat7
U+75F8.0	fCantonese	cai3
U+75F8.0	fMandarin	chi4
U+75F9.0	fUTF8	痹
U+75F9.0	fCNS11643	2-4533
U+75F9.0	fBig5	DECD
U+75F9.0	fGB	0-3154
U+75F9.0	fR/S	 104.8
U+75F9.0	fTotalStrokes	 13
U+75F9.0	fCangjie	KWML
U+75F9.0	fFourCorner	00121
U+75F9.0	fHakka	  bi5; bi3
U+75F9.0	fCantonese	bei3
U+75F9.0	fMandarin	bi4
U+75F9.0	fEnglish	paralysis; numbness
U+75FA.0	fUTF8	痺
U+75FA.0	fCNS11643	1-653C
U+75FA.0	fBig5	B7F4
U+75FA.0	fGB	E-3437
U+75FA.0	fR/S	 104.8
U+75FA.0	fTotalStrokes	 13
U+75FA.0	fCangjie	KHWJ
U+75FA.0	fFourCorner	00146
U+75FA.0	fHakka	  bi1; bi5
U+75FA.0	fCantonese	bei3; bei1
U+75FA.0	fMandarin	bi4; qing3; bei1
U+75FA.0	fEnglish	paralysis; numbness
U+75FB.0	fUTF8	痻
U+75FB.0	fCNS11643	2-4536
U+75FB.0	fBig5	DED0
U+75FB.0	fGB	5-5D44
U+75FB.0	fR/S	 104.8
U+75FB.0	fTotalStrokes	 13
U+75FB.0	fCangjie	KHPA
U+75FB.0	fFourCorner	00164
U+75FB.0	fCantonese	man4; fan1
U+75FB.0	fMandarin	min2; hun1
U+75FB.0	fEnglish	suffer
U+75FC.0	fUTF8	痼
U+75FC.0	fCNS11643	2-4532
U+75FC.0	fBig5	DECC
U+75FC.0	fGB	0-7073
U+75FC.0	fR/S	 104.8
U+75FC.0	fTotalStrokes	 13
U+75FC.0	fCangjie	KWJR
U+75FC.0	fFourCorner	00160
U+75FC.0	fHakka	 gu5; go1
U+75FC.0	fCantonese	gu3; gwu3 (also)
U+75FC.0	fMandarin	gu4; gu1
U+75FC.0	fEnglish	chronic disease
U+75FD.0	fUTF8	痽
U+75FD.0	fCNS11643	2-453A
U+75FD.0	fBig5	DED4
U+75FD.0	fGB	5-5D40
U+75FD.0	fR/S	 104.8
U+75FD.0	fTotalStrokes	 13
U+75FD.0	fCangjie	KOG
U+75FD.0	fFourCorner	00114
U+75FD.0	fCantonese	deoi1
U+75FD.0	fMandarin	dui1
U+75FE.0	fUTF8	痾
U+75FE.0	fCNS11643	2-4531
U+75FE.0	fBig5	DECB
U+75FE.0	fGB	E-3438
U+75FE.0	fR/S	 104.8
U+75FE.0	fTotalStrokes	 13
U+75FE.0	fCangjie	KNLR
U+75FE.0	fFourCorner	00121
U+75FE.0	fHakka	  o1
U+75FE.0	fCantonese	ngo1 (also); o1
U+75FE.0	fMandarin	e1; e4; ke1
U+75FE.0	fEnglish	 chronic illness dysentery
U+75FF.0	fUTF8	痿
U+75FF.0	fCNS11643	1-653D
U+75FF.0	fBig5	B7F5
U+75FF.0	fGB	0-7074
U+75FF.0	fR/S	 104.8
U+75FF.0	fTotalStrokes	 13
U+75FF.0	fCangjie	KHDV
U+75FF.0	fFourCorner	00144
U+75FF.0	fHakka	 wui1; wui3; wi1; vui3; wi3
U+75FF.0	fCantonese	wai2; wai1
U+75FF.0	fMandarin	wei3; rui2; wei1; wei4
U+75FF.0	fEnglish	paralysis; impotent
U+7600.0	fUTF8	瘀
U+7600.0	fCNS11643	1-6537
U+7600.0	fBig5	B7EF
U+7600.0	fGB	0-7076
U+7600.0	fR/S	 104.8
U+7600.0	fTotalStrokes	 13
U+7600.0	fCangjie	KYSY
U+7600.0	fFourCorner	00133
U+7600.0	fHakka	 ji3; ri3; i1
U+7600.0	fCantonese	jyu1 (also); jyu2; jyu3
U+7600.0	fMandarin	yu1; se4; yu4
U+7600.0	fEnglish	hematoma
U+7601.0	fUTF8	瘁
U+7601.0	fCNS11643	1-6539
U+7601.0	fBig5	B7F1
U+7601.0	fGB	0-3461
U+7601.0	fR/S	 104.8
U+7601.0	fTotalStrokes	 13
U+7601.0	fCangjie	KYOJ
U+7601.0	fFourCorner	00148
U+7601.0	fHakka	  cui5
U+7601.0	fCantonese	seoi6; seoi5 (also)
U+7601.0	fMandarin	cui4
U+7601.0	fEnglish	[1] feel tired; be weary; be worn out [2] disease; illness [3] painful; aching (headache)
U+7602.0	fUTF8	瘂
U+7602.0	fCNS11643	3-427D
U+7602.0	fGB	1-7069
U+7602.0	fR/S	  104.8
U+7602.0	fTotalStrokes	 13
U+7602.0	fCangjie	KMLM
U+7602.0	fFourCorner	00110
U+7602.0	fHakka	 a3; at7
U+7602.0	fCantonese	aa2; ngaa2 (also)
U+7602.0	fMandarin	ya3
U+7602.0	fEnglish	dumb
U+7603.0	fUTF8	瘃
U+7603.0	fCNS11643	2-452F
U+7603.0	fBig5	DEC9
U+7603.0	fGB	0-7071
U+7603.0	fR/S	 104.8
U+7603.0	fTotalStrokes	 13
U+7603.0	fCangjie	KMSO
U+7603.0	fFourCorner	00132
U+7603.0	fHakka	 dok7; chuk7; zuk7
U+7603.0	fCantonese	zuk1 (also); zuk6; zuk3 (also)
U+7603.0	fMandarin	zhu2; zhu3
U+7603.0	fEnglish	sores from cold
U+7604.0	fUTF8	瘄
U+7604.0	fGB	3-5E7B
U+7604.0	fR/S	 104.8
U+7604.0	fTotalStrokes	 13
U+7604.0	fCangjie	KTA
U+7604.0	fFourCorner	0016
U+7604.0	fCantonese	cou3
U+7604.0	fMandarin	cu4
U+7605.0	fUTF8	瘅
U+7605.0	fGB	0-7077
U+7605.0	fR/S	 104.8
U+7605.0	fTotalStrokes	 13
U+7605.0	fCangjie	KCWJ
U+7605.0	fHakka	 tan3; dan5
U+7605.0	fCantonese	daan3; daan1
U+7605.0	fMandarin	dan4; dan1; dan3; duo3; duo4; tan2
U+7605.0	fEnglish	simplified form of &#x7649;
U+7606.0	fUTF8	瘆
U+7606.0	fGB	7-212F
U+7606.0	fR/S	 104.8
U+7606.0	fTotalStrokes	 13
U+7606.0	fCangjie	KIKH
U+7606.0	fCantonese	sam3
U+7606.0	fMandarin	shen4
U+7607.0	fUTF8	瘇
U+7607.0	fCNS11643	3-482B
U+7607.0	fGB	3-5F34
U+7607.0	fR/S	 104.9
U+7607.0	fTotalStrokes	 14
U+7607.0	fCangjie	KHJG
U+7607.0	fFourCorner	00110
U+7607.0	fHakka	  zhung3; zung3
U+7607.0	fCantonese	zung2
U+7607.0	fMandarin	zhong3
U+7607.0	fEnglish	swelling of the legs
U+7608.0	fUTF8	瘈
U+7608.0	fCNS11643	2-4C23
U+7608.0	fBig5	E2DB
U+7608.0	fGB	3-5F2C
U+7608.0	fR/S	 104.9
U+7608.0	fTotalStrokes	 14
U+7608.0	fCangjie	KQHK
U+7608.0	fFourCorner	00134
U+7608.0	fHakka	 gi5; k'ie5; ki1
U+7608.0	fCantonese	zai3; kai3; gai3 (also)
U+7608.0	fMandarin	ji4; zhi4; chi4; qi4
U+7608.0	fEnglish	 prolonged fever chronic malaria
U+7609.0	fUTF8	瘉
U+7609.0	fCNS11643	1-696E
U+7609.0	fBig5	BAC7
U+7609.0	fGB	E-3439
U+7609.0	fR/S	 104.9
U+7609.0	fTotalStrokes	 14
U+7609.0	fCangjie	KOMN
U+7609.0	fFourCorner	00121
U+7609.0	fHakka	 ji5; ri5; ji3
U+7609.0	fCantonese	jyu5; jyu6
U+7609.0	fMandarin	yu4; yu2; yu3
U+7609.0	fEnglish	same as $ get well; recover
U+760A.0	fUTF8	瘊
U+760A.0	fCNS11643	2-4C27
U+760A.0	fBig5	E2DF
U+760A.0	fGB	0-707A
U+760A.0	fR/S	 104.9
U+760A.0	fTotalStrokes	 14
U+760A.0	fCangjie	KONK
U+760A.0	fFourCorner	00134
U+760A.0	fHakka	 heu2; hiu2
U+760A.0	fCantonese	hau4
U+760A.0	fMandarin	hou2
U+760A.0	fEnglish	warts; pimples
U+760B.0	fUTF8	瘋
U+760B.0	fCNS11643	1-696D
U+760B.0	fBig5	BAC6
U+760B.0	fGB	1-3768
U+760B.0	fR/S	 104.9
U+760B.0	fTotalStrokes	 14
U+760B.0	fCangjie	KHNI
U+760B.0	fFourCorner	00117
U+760B.0	fHakka	  fung1
U+760B.0	fCantonese	fung1
U+760B.0	fMandarin	feng1
U+760B.0	fEnglish	crazy; insane; mentally ill; mentally deranged; lunatic; wild
U+760C.0	fUTF8	瘌
U+760C.0	fCNS11643	2-4C24
U+760C.0	fBig5	E2DC
U+760C.0	fGB	0-7078
U+760C.0	fR/S	 104.9
U+760C.0	fTotalStrokes	 14
U+760C.0	fCangjie	KDLN
U+760C.0	fFourCorner	00120
U+760C.0	fHakka	 lat8; lat7
U+760C.0	fCantonese	laat6 (also); laat3
U+760C.0	fMandarin	la4; lai4
U+760C.0	fEnglish	favus
U+760D.0	fUTF8	瘍
U+760D.0	fCNS11643	1-696C
U+760D.0	fBig5	BAC5
U+760D.0	fGB	1-5171
U+760D.0	fR/S	 104.9
U+760D.0	fTotalStrokes	 14
U+760D.0	fCangjie	KAMH
U+760D.0	fFourCorner	00127
U+760D.0	fHakka	  jong2; rong2
U+760D.0	fCantonese	joeng4
U+760D.0	fMandarin	yang2; dang4
U+760D.0	fEnglish	skin disease or infections; sore; ulcer
U+760E.0	fUTF8	瘎
U+760E.0	fCNS11643	3-4829
U+760E.0	fGB	3-5F2E
U+760E.0	fR/S	  104.9
U+760E.0	fTotalStrokes	 14
U+760E.0	fCangjie	KTMV
U+760E.0	fFourCorner	00110
U+760E.0	fCantonese	sam4
U+760E.0	fMandarin	shen4
U+760F.0	fUTF8	瘏
U+760F.0	fCNS11643	2-452E
U+760F.0	fBig5	DEC8
U+760F.0	fGB	3-5E7A
U+760F.0	fR/S	 104.9
U+760F.0	fTotalStrokes	 14
U+760F.0	fCangjie	KJKA
U+760F.0	fFourCorner	00164
U+760F.0	fHakka	 tu2; tu1
U+760F.0	fCantonese	tou4
U+760F.0	fMandarin	tu2
U+760F.0	fEnglish	ill
U+7610.0	fUTF8	瘐
U+7610.0	fCNS11643	2-4535
U+7610.0	fBig5	DECF
U+7610.0	fGB	0-7075
U+7610.0	fR/S	 104.9
U+7610.0	fTotalStrokes	 14
U+7610.0	fCangjie	KHXO
U+7610.0	fFourCorner	30137
U+7610.0	fHakka	 ji2; ri2
U+7610.0	fCantonese	jyu5
U+7610.0	fMandarin	yu3; yu2
U+7610.0	fEnglish	 die in prison from cold and hunger treat with cruelty
U+7611.0	fUTF8	瘑
U+7611.0	fCNS11643	2-4C26
U+7611.0	fBig5	E2DE
U+7611.0	fGB	3-5E74
U+7611.0	fR/S	 104.9
U+7611.0	fTotalStrokes	 14
U+7611.0	fCangjie	KBBR
U+7611.0	fFourCorner	00127
U+7611.0	fCantonese	gwo1; go1 (also)
U+7611.0	fMandarin	gua1; ge1
U+7612.0	fUTF8	瘒
U+7612.0	fCNS11643	4-447E
U+7612.0	fGB	E-343A
U+7612.0	fR/S	 104.9
U+7612.0	fTotalStrokes	 14
U+7612.0	fCangjie	KBJJ
U+7612.0	fFourCorner	00150
U+7612.0	fCantonese	wan4
U+7612.0	fMandarin	wen2
U+7613.0	fUTF8	瘓
U+7613.0	fCNS11643	1-696F
U+7613.0	fBig5	BAC8
U+7613.0	fGB	1-3B3E
U+7613.0	fR/S	 104.9
U+7613.0	fTotalStrokes	 14
U+7613.0	fCangjie	KNBK
U+7613.0	fFourCorner	00181
U+7613.0	fHakka	  fon5; fon3
U+7613.0	fCantonese	wun6
U+7613.0	fMandarin	huan4; tuan3
U+7613.0	fEnglish	paralysis
U+7614.0	fUTF8	瘔
U+7614.0	fCNS11643	2-4C28
U+7614.0	fBig5	E2E0
U+7614.0	fGB	5-5D3A
U+7614.0	fR/S	 104.9
U+7614.0	fTotalStrokes	 14
U+7614.0	fCangjie	KTJR
U+7614.0	fFourCorner	00164
U+7614.0	fCantonese	fu2; fu3
U+7614.0	fMandarin	ku4
U+7615.0	fUTF8	瘕
U+7615.0	fCNS11643	2-4C25
U+7615.0	fBig5	E2DD
U+7615.0	fGB	0-707D
U+7615.0	fR/S	 104.9
U+7615.0	fTotalStrokes	 14
U+7615.0	fCangjie	KRSE
U+7615.0	fFourCorner	00147
U+7615.0	fHakka	 ga1; ga2; ha2; ga5
U+7615.0	fCantonese	gaa2; haa4; gaa1; gaa3
U+7615.0	fMandarin	jia3; xia2; jia1; jia4; xia1
U+7615.0	fEnglish	ashma. disese of the bowels
U+7616.0	fUTF8	瘖
U+7616.0	fCNS11643	2-4C22
U+7616.0	fBig5	E2DA
U+7616.0	fGB	E-343B
U+7616.0	fR/S	 104.9
U+7616.0	fTotalStrokes	 14
U+7616.0	fCangjie	KYTA
U+7616.0	fFourCorner	00161
U+7616.0	fHakka	  jim1; rim1; am1; am5
U+7616.0	fCantonese	jam1; jam3; am1; ngam1 (also)
U+7616.0	fMandarin	yin1; yin4
U+7616.0	fEnglish	dumb; mute; unable speak
U+7617.0	fUTF8	瘗
U+7617.0	fGB	0-7079
U+7617.0	fR/S	 104.9
U+7617.0	fTotalStrokes	 14
U+7617.0	fCangjie	KKTG
U+7617.0	fHakka	 ji5
U+7617.0	fCantonese	ji3
U+7617.0	fMandarin	yi4
U+7617.0	fEnglish	simplified form of &#x761E;
U+7618.0	fUTF8	瘘
U+7618.0	fGB	0-707C
U+7618.0	fR/S	 104.9
U+7618.0	fTotalStrokes	 14
U+7618.0	fCangjie	KFDV
U+7618.0	fHakka	 leu5
U+7618.0	fCantonese	lau6; lau4
U+7618.0	fMandarin	l2; luu2; lou4
U+7618.0	fEnglish	simplified form of &#x763B;
U+7619.0	fUTF8	瘙
U+7619.0	fCNS11643	2-5239
U+7619.0	fBig5	E6B1
U+7619.0	fGB	0-707E
U+7619.0	fR/S	 104.10
U+7619.0	fTotalStrokes	 15
U+7619.0	fCangjie	KEII
U+7619.0	fFourCorner	00136
U+7619.0	fHakka	  sau5; zau3; sau1
U+7619.0	fCantonese	sou3; sou1
U+7619.0	fMandarin	sao4; sao1
U+7619.0	fEnglish	itch
U+761A.0	fUTF8	瘚
U+761A.0	fCNS11643	2-523D
U+761A.0	fBig5	E6B5
U+761A.0	fGB	5-5D5D
U+761A.0	fR/S	 104.10
U+761A.0	fTotalStrokes	 15
U+761A.0	fCangjie	KTUO
U+761A.0	fFourCorner	00187
U+761A.0	fHakka	 kiet7; ket7
U+761A.0	fCantonese	kyut3; gyut3
U+761A.0	fMandarin	jue2
U+761A.0	fEnglish	 hiccough the humours of the body
U+761B.0	fUTF8	瘛
U+761B.0	fCNS11643	2-523F
U+761B.0	fBig5	E6B7
U+761B.0	fGB	0-7121
U+761B.0	fR/S	 104.10
U+761B.0	fTotalStrokes	 15
U+761B.0	fCangjie	KQHP
U+761B.0	fFourCorner	00132
U+761B.0	fCantonese	kai3; zai3; gai3 (also); cai3
U+761B.0	fMandarin	chi4; che4; xi4
U+761C.0	fUTF8	瘜
U+761C.0	fCNS11643	2-523B
U+761C.0	fBig5	E6B3
U+761C.0	fGB	5-5D5A
U+761C.0	fR/S	 104.10
U+761C.0	fTotalStrokes	 15
U+761C.0	fCangjie	KHUP
U+761C.0	fFourCorner	00136
U+761C.0	fHakka	 sit7
U+761C.0	fCantonese	sik1
U+761C.0	fMandarin	xi2
U+761C.0	fEnglish	a polypus
U+761D.0	fUTF8	瘝
U+761D.0	fCNS11643	2-523A
U+761D.0	fBig5	E6B2
U+761D.0	fGB	3-5F3A
U+761D.0	fR/S	 104.10
U+761D.0	fTotalStrokes	 15
U+761D.0	fCangjie	KWLE
U+761D.0	fFourCorner	00199
U+761D.0	fHakka	 kwan1; gwan1
U+761D.0	fCantonese	gwaan1
U+761D.0	fMandarin	guan1
U+761D.0	fEnglish	illness; sick; ill; ailing
U+761E.0	fUTF8	瘞
U+761E.0	fCNS11643	2-5238
U+761E.0	fBig5	E6B0
U+761E.0	fGB	1-7079
U+761E.0	fR/S	 104.10
U+761E.0	fTotalStrokes	 15
U+761E.0	fCangjie	KKOG
U+761E.0	fFourCorner	00114
U+761E.0	fHakka	 ji5; ri5; i5
U+761E.0	fCantonese	ji3; ai3; ngai3
U+761E.0	fMandarin	yi4
U+761E.0	fEnglish	 bury
U+761F.0	fUTF8	瘟
U+761F.0	fCNS11643	1-6D6E
U+761F.0	fBig5	BD45
U+761F.0	fGB	0-4E41
U+761F.0	fR/S	 104.10; 104.9
U+761F.0	fTotalStrokes	 15
U+761F.0	fCangjie	KWOT
U+761F.0	fFourCorner	00117
U+761F.0	fHakka	  wun1; vun1
U+761F.0	fCantonese	wan1
U+761F.0	fMandarin	wen1; yun1; wo4
U+761F.0	fEnglish	 epidemic; plague; pestilence
U+7620.0	fUTF8	瘠
U+7620.0	fCNS11643	1-6D6C
U+7620.0	fBig5	BD43
U+7620.0	fGB	0-7124
U+7620.0	fR/S	 104.10
U+7620.0	fTotalStrokes	 15
U+7620.0	fCangjie	KFCB
U+7620.0	fFourCorner	00127
U+7620.0	fHakka	 zit7
U+7620.0	fCantonese	zek3; zik3 (also); zik6
U+7620.0	fMandarin	ji2; zi4
U+7620.0	fEnglish	[1] thin; emaciated; lean; meager [2] sterile; infertile; unproductive; barren (land)
U+7621.0	fUTF8	瘡
U+7621.0	fCNS11643	1-6D71
U+7621.0	fBig5	BD48
U+7621.0	fGB	1-342F
U+7621.0	fR/S	 104.10
U+7621.0	fTotalStrokes	 15
U+7621.0	fCangjie	KOIR
U+7621.0	fFourCorner	00167
U+7621.0	fHakka	  cong1
U+7621.0	fCantonese	cong1
U+7621.0	fMandarin	chuang1; chuan1
U+7621.0	fEnglish	[1] tumor; boil; sore [2] wound
U+7622.0	fUTF8	瘢
U+7622.0	fCNS11643	1-6D72
U+7622.0	fBig5	BD49
U+7622.0	fGB	0-7123
U+7622.0	fR/S	 104.10
U+7622.0	fTotalStrokes	 15
U+7622.0	fCangjie	KHYE
U+7622.0	fFourCorner	00147
U+7622.0	fHakka	 ban1; pan2
U+7622.0	fCantonese	baan1; pun4
U+7622.0	fMandarin	ban1
U+7622.0	fEnglish	 scar; mole; freckle
U+7623.0	fUTF8	瘣
U+7623.0	fCNS11643	2-523C
U+7623.0	fBig5	E6B4
U+7623.0	fGB	3-5F35
U+7623.0	fR/S	 104.10
U+7623.0	fTotalStrokes	 15
U+7623.0	fCangjie	KHUI
U+7623.0	fFourCorner	00113
U+7623.0	fHakka	 fui5; fi5
U+7623.0	fCantonese	wui6; waai6; fui3
U+7623.0	fMandarin	lei3; gui1; hui2; hui4; kuai4
U+7623.0	fEnglish	 knobs or excrescencies on trees
U+7624.0	fUTF8	瘤
U+7624.0	fCNS11643	1-6D6F
U+7624.0	fBig5	BD46
U+7624.0	fGB	0-4176
U+7624.0	fR/S	 104.10
U+7624.0	fTotalStrokes	 15
U+7624.0	fCangjie	KHHW
U+7624.0	fFourCorner	00162
U+7624.0	fHakka	 liu2; lieu2
U+7624.0	fCantonese	lau4; lau2 (also)
U+7624.0	fMandarin	liu2; liu4
U+7624.0	fEnglish	[1] tumor [2] lump; bump; swelling
U+7625.0	fUTF8	瘥
U+7625.0	fCNS11643	2-5237
U+7625.0	fBig5	E6AF
U+7625.0	fGB	0-707B
U+7625.0	fR/S	 104.10
U+7625.0	fTotalStrokes	 15
U+7625.0	fCangjie	KTQM
U+7625.0	fFourCorner	00112
U+7625.0	fHakka	  co1; ca1
U+7625.0	fCantonese	caai3; co4; co1
U+7625.0	fMandarin	chai4; cha4; jie1
U+7625.0	fEnglish	recovered from any disease; cured; healed
U+7625.1	fUTF8	瘥
U+7625.1	fCNS11643	2-5237
U+7625.1	fBig5	E6AF
U+7625.1	fGB	0-707B
U+7625.1	fR/S	 104.10
U+7625.1	fTotalStrokes	 15
U+7625.1	fCangjie	KTQM
U+7625.1	fFourCorner	00112
U+7625.1	fMandarin	cuo2
U+7625.1	fEnglish	sick; ill; affliction; ailing
U+7626.0	fUTF8	瘦
U+7626.0	fCNS11643	1-6D70
U+7626.0	fBig5	BD47
U+7626.0	fGB	0-4A5D
U+7626.0	fR/S	 104.10
U+7626.0	fTotalStrokes	 15
U+7626.0	fCangjie	KHXE
U+7626.0	fFourCorner	00147
U+7626.0	fHakka	  seu5; ceu5; sieu5; siu5
U+7626.0	fCantonese	sau3
U+7626.0	fMandarin	shou4; sou1
U+7626.0	fEnglish	thin; emaciated; lean; meager; slim; scrawny
U+7627.0	fUTF8	瘧
U+7627.0	fCNS11643	1-696B
U+7627.0	fBig5	BAC4
U+7627.0	fGB	1-4531
U+7627.0	fR/S	 104.10; 104.9
U+7627.0	fTotalStrokes	 15
U+7627.0	fCangjie	KYPM
U+7627.0	fFourCorner	00114
U+7627.0	fHakka	 ngiok7; ngiok8; niok7
U+7627.0	fCantonese	joek6
U+7627.0	fMandarin	yao4; nuue4; ne4; yao2
U+7627.0	fEnglish	malaria
U+7628.0	fUTF8	瘨
U+7628.0	fCNS11643	2-523E
U+7628.0	fBig5	E6B6
U+7628.0	fGB	3-5F38
U+7628.0	fR/S	 104.10
U+7628.0	fTotalStrokes	 15
U+7628.0	fCangjie	KJBC
U+7628.0	fFourCorner	00181
U+7628.0	fHakka	  dien1
U+7628.0	fCantonese	din1
U+7628.0	fMandarin	dian1; chen1; dian4; tian2; tian3; zhen4
U+7628.0	fEnglish	 mad ill
U+7629.0	fUTF8	瘩
U+7629.0	fCNS11643	1-6D6D
U+7629.0	fBig5	BD44
U+7629.0	fGB	0-3471
U+7629.0	fR/S	 104.10; 104.9
U+7629.0	fTotalStrokes	 15
U+7629.0	fCangjie	KTOR
U+7629.0	fFourCorner	00161
U+7629.0	fHakka	  dap7
U+7629.0	fCantonese	daap3; daat6
U+7629.0	fMandarin	da5; da2; da1
U+7629.0	fEnglish	 boil pimple; wart
U+762A.0	fUTF8	瘪
U+762A.0	fGB	0-3171
U+762A.0	fR/S	 104.10
U+762A.0	fTotalStrokes	 15
U+762A.0	fCangjie	XKHUP
U+762A.0	fHakka	 piet8
U+762A.0	fCantonese	bit6
U+762A.0	fMandarin	pie1; bie3; bie1
U+762A.0	fEnglish	simplified form of &#x765F;
U+762B.0	fUTF8	瘫
U+762B.0	fGB	0-4C31
U+762B.0	fR/S	 104.10
U+762B.0	fTotalStrokes	 15
U+762B.0	fCangjie	KEOG
U+762B.0	fHakka	 tan1
U+762B.0	fCantonese	taan1; taan2 (also)
U+762B.0	fMandarin	tan1
U+762B.0	fEnglish	simplified form of &#x7671;
U+762C.0	fUTF8	瘬
U+762C.0	fCNS11643	3-5171
U+762C.0	fGB	5-5D37
U+762C.0	fR/S	 104.11
U+762C.0	fTotalStrokes	 16
U+762C.0	fCangjie	KNSV
U+762C.0	fFourCorner	00130
U+762C.0	fHakka	 zhong5; zong5
U+762C.0	fCantonese	zoeng3
U+762C.0	fMandarin	zhang4
U+762D.0	fUTF8	瘭
U+762D.0	fCNS11643	2-5858
U+762D.0	fBig5	EA6C
U+762D.0	fGB	0-7126
U+762D.0	fR/S	 104.11
U+762D.0	fTotalStrokes	 16
U+762D.0	fCangjie	KMWF
U+762D.0	fFourCorner	00191
U+762D.0	fCantonese	biu1
U+762D.0	fMandarin	biao1; piao4
U+762D.0	fEnglish	a whitlow
U+762E.0	fUTF8	瘮
U+762E.0	fCNS11643	4-514F
U+762E.0	fGB	3-5F2B
U+762E.0	fR/S	  104.11
U+762E.0	fTotalStrokes	 16
U+762E.0	fCangjie	KIIH
U+762E.0	fFourCorner	00120
U+762E.0	fCantonese	sam3
U+762E.0	fMandarin	shen4; shen3
U+762F.0	fUTF8	瘯
U+762F.0	fCNS11643	2-5857
U+762F.0	fBig5	EA6B
U+762F.0	fGB	5-5D66
U+762F.0	fR/S	 104.11
U+762F.0	fTotalStrokes	 16
U+762F.0	fCangjie	KYSK
U+762F.0	fFourCorner	00134
U+762F.0	fCantonese	cuk1; zuk6
U+762F.0	fMandarin	cu4; zu2
U+762F.0	fEnglish	itch
U+7630.0	fUTF8	瘰
U+7630.0	fCNS11643	2-585F
U+7630.0	fBig5	EA73
U+7630.0	fGB	0-7127
U+7630.0	fR/S	 104.11
U+7630.0	fTotalStrokes	 16
U+7630.0	fCangjie	KWVF
U+7630.0	fFourCorner	00193
U+7630.0	fHakka	 lo2;  lo3; lui5; lui6; lui1
U+7630.0	fCantonese	lo2; leoi4; lo5
U+7630.0	fMandarin	luo3; lei3; luo2
U+7630.0	fEnglish	scrofula; swellings
U+7631.0	fUTF8	瘱
U+7631.0	fCNS11643	2-5859
U+7631.0	fBig5	EA6D
U+7631.0	fGB	5-5D57
U+7631.0	fR/S	 104.11
U+7631.0	fTotalStrokes	 16
U+7631.0	fCangjie	KKOP
U+7631.0	fFourCorner	00133
U+7631.0	fCantonese	ai3; ngai3 (also); ji3 (also)
U+7631.0	fMandarin	yi4
U+7632.0	fUTF8	瘲
U+7632.0	fCNS11643	2-585E
U+7632.0	fBig5	EA72
U+7632.0	fGB	3-5E4C
U+7632.0	fR/S	 104.11
U+7632.0	fTotalStrokes	 16
U+7632.0	fCangjie	KHOO
U+7632.0	fFourCorner	00181
U+7632.0	fHakka	  ziung3
U+7632.0	fCantonese	zung3
U+7632.0	fMandarin	zong4; zong1
U+7632.0	fEnglish	spasms or convulsions in young children ; caused by indigestion
U+7633.0	fUTF8	瘳
U+7633.0	fCNS11643	2-585B
U+7633.0	fBig5	EA6F
U+7633.0	fGB	0-712C
U+7633.0	fR/S	 104.11
U+7633.0	fTotalStrokes	 16
U+7633.0	fCangjie	KSMH
U+7633.0	fFourCorner	00122
U+7633.0	fHakka	 ch'ui2; liau3; cui2; liau2; liau6; ciu2; liau5
U+7633.0	fCantonese	cau1
U+7633.0	fMandarin	chou1; liao2; lu4
U+7633.0	fEnglish	[1] cured; healed [2] harm; hurt
U+7634.0	fUTF8	瘴
U+7634.0	fCNS11643	1-714A
U+7634.0	fBig5	BF60
U+7634.0	fGB	0-554E
U+7634.0	fR/S	 104.11
U+7634.0	fTotalStrokes	 16
U+7634.0	fCangjie	KYTJ
U+7634.0	fFourCorner	00146
U+7634.0	fHakka	 zong5; zhong5
U+7634.0	fCantonese	zoeng3
U+7634.0	fMandarin	zhang4
U+7634.0	fEnglish	miasma; swap vapor
U+7635.0	fUTF8	瘵
U+7635.0	fCNS11643	2-585D
U+7635.0	fBig5	EA71
U+7635.0	fGB	0-7129
U+7635.0	fR/S	 104.11
U+7635.0	fTotalStrokes	 16
U+7635.0	fCangjie	KBOF
U+7635.0	fFourCorner	00191
U+7635.0	fHakka	  zai5; zi5; cat7
U+7635.0	fCantonese	zaai3
U+7635.0	fMandarin	zhai4; ji4; zhi4
U+7635.0	fEnglish	[1] tuberculosis [2] distress
U+7636.0	fUTF8	瘶
U+7636.0	fCNS11643	3-5173
U+7636.0	fGB	5-5D62
U+7636.0	fR/S	  104.11
U+7636.0	fTotalStrokes	 16
U+7636.0	fCangjie	KDLO
U+7636.0	fFourCorner	00180
U+7636.0	fHakka	 seu5; {seu5}; cuk7
U+7636.0	fCantonese	sau3
U+7636.0	fMandarin	sou4
U+7636.0	fEnglish	 cough expectorate
U+7637.0	fUTF8	瘷
U+7637.0	fCNS11643	3-5172
U+7637.0	fGB	5-5D61
U+7637.0	fR/S	  104.11
U+7637.0	fTotalStrokes	 16
U+7637.0	fCangjie	KDLK
U+7637.0	fFourCorner	00140
U+7637.0	fCantonese	zek3
U+7637.0	fMandarin	suo3
U+7638.0	fUTF8	瘸
U+7638.0	fCNS11643	1-714B
U+7638.0	fBig5	BF61
U+7638.0	fGB	0-4833
U+7638.0	fR/S	 104.11
U+7638.0	fTotalStrokes	 16
U+7638.0	fCangjie	KKRB
U+7638.0	fFourCorner	00127
U+7638.0	fHakka	 kio2
U+7638.0	fCantonese	ke4; koe4
U+7638.0	fMandarin	que2
U+7638.0	fEnglish	[1] cripple [2] be lame
U+7639.0	fUTF8	瘹
U+7639.0	fCNS11643	4-514D
U+7639.0	fGB	3-5F25
U+7639.0	fR/S	  104.11
U+7639.0	fTotalStrokes	 16
U+7639.0	fCangjie	KCPI
U+7639.0	fFourCorner	00120
U+7639.0	fCantonese	diu3
U+7639.0	fMandarin	diao4
U+763A.0	fUTF8	瘺
U+763A.0	fCNS11643	1-714C
U+763A.0	fBig5	BF62
U+763A.0	fGB	5-5D69
U+763A.0	fR/S	 104.11
U+763A.0	fTotalStrokes	 16
U+763A.0	fCangjie	KSMB
U+763A.0	fFourCorner	00127
U+763A.0	fHakka	 leu5; leu6; liu3
U+763A.0	fCantonese	lau6
U+763A.0	fMandarin	lou4
U+763A.0	fEnglish	 anal fistula tumor; sore
U+763B.0	fUTF8	瘻
U+763B.0	fCNS11643	3-5174
U+763B.0	fGB	1-707C
U+763B.0	fR/S	 104.11
U+763B.0	fTotalStrokes	 16
U+763B.0	fCangjie	KLLV
U+763B.0	fFourCorner	00140
U+763B.0	fHakka	 leu2; leu5; leu6
U+763B.0	fCantonese	lau6; lau4
U+763B.0	fMandarin	l2
U+763B.0	fEnglish	 hunchback; humpback
U+763B.1	fUTF8	瘻
U+763B.1	fCNS11643	3-5174
U+763B.1	fGB	1-707C
U+763B.1	fR/S	 104.11
U+763B.1	fTotalStrokes	 16
U+763B.1	fCangjie	KLLV
U+763B.1	fFourCorner	00140
U+763B.1	fMandarin	lou4; luu2
U+763B.1	fEnglish	goiter; scrofula
U+763C.0	fUTF8	瘼
U+763C.0	fCNS11643	2-585C
U+763C.0	fBig5	EA70
U+763C.0	fGB	0-7122
U+763C.0	fR/S	 104.11; 104.10
U+763C.0	fTotalStrokes	 16
U+763C.0	fCangjie	KTAK
U+763C.0	fFourCorner	00134
U+763C.0	fHakka	 mok8; mok7
U+763C.0	fCantonese	mok6; mok2 (also)
U+763C.0	fMandarin	mo4
U+763C.0	fEnglish	[1] sickness; disease; ailment [2] suffering; hardship; privation
U+763D.0	fUTF8	瘽
U+763D.0	fCNS11643	2-585A
U+763D.0	fBig5	EA6E
U+763D.0	fGB	5-5D60
U+763D.0	fR/S	 104.11
U+763D.0	fTotalStrokes	 16
U+763D.0	fCangjie	KTLM
U+763D.0	fFourCorner	00114
U+763D.0	fCantonese	kan4; gan2
U+763D.0	fMandarin	jin4; qin2
U+763E.0	fUTF8	瘾
U+763E.0	fGB	0-712B
U+763E.0	fR/S	 104.11
U+763E.0	fTotalStrokes	 16
U+763E.0	fCangjie	XKNLP
U+763E.0	fHakka	 jin3
U+763E.0	fCantonese	jan5
U+763E.0	fMandarin	yin3
U+763E.0	fEnglish	simplified form of &#x766E;
U+763F.0	fUTF8	瘿
U+763F.0	fGB	0-7128
U+763F.0	fR/S	 104.11
U+763F.0	fTotalStrokes	 16
U+763F.0	fCangjie	KBOV
U+763F.0	fHakka	 jin1
U+763F.0	fCantonese	jing2
U+763F.0	fMandarin	ying3; ying1
U+763F.0	fEnglish	simplified form of &#x766D;
U+7640.0	fUTF8	癀
U+7640.0	fCNS11643	4-5721
U+7640.0	fGB	0-7125
U+7640.0	fR/S	 104.12; 104.11
U+7640.0	fTotalStrokes	 16; 17
U+7640.0	fCangjie	KTLC
U+7640.0	fFourCorner	00180
U+7640.0	fHakka	 wong2; wang2; vong2
U+7640.0	fCantonese	wong4
U+7640.0	fMandarin	huang2
U+7640.0	fEnglish	jaundice
U+7641.0	fUTF8	癁
U+7641.0	fCNS11643	4-5724
U+7641.0	fGB	E-343C
U+7641.0	fR/S	  104.12
U+7641.0	fTotalStrokes	 17; 16
U+7641.0	fCangjie	KHOE
U+7641.0	fFourCorner	00147
U+7641.0	fCantonese	fuk6
U+7641.0	fMandarin	fu2
U+7642.0	fUTF8	療
U+7642.0	fCNS11643	1-7443
U+7642.0	fBig5	C0F8
U+7642.0	fGB	1-4146
U+7642.0	fR/S	 104.12
U+7642.0	fTotalStrokes	 17
U+7642.0	fCangjie	KKCF
U+7642.0	fFourCorner	00196
U+7642.0	fHakka	  liau2; liau5; liau6; liau3
U+7642.0	fCantonese	liu4; liu6
U+7642.0	fMandarin	liao2; shuo4; liao4
U+7642.0	fEnglish	 treat (disease); relieve; heal
U+7643.0	fUTF8	癃
U+7643.0	fCNS11643	2-5D60
U+7643.0	fBig5	ED74
U+7643.0	fGB	0-712A
U+7643.0	fR/S	 104.12
U+7643.0	fTotalStrokes	 17
U+7643.0	fCangjie	KNLM
U+7643.0	fFourCorner	00114
U+7643.0	fHakka	  lung2
U+7643.0	fCantonese	lung4
U+7643.0	fMandarin	long2
U+7643.0	fEnglish	[1] humping of the back in old age [2] anuria
U+7644.0	fUTF8	癄
U+7644.0	fCNS11643	3-5555
U+7644.0	fGB	E-343D
U+7644.0	fR/S	 104.12
U+7644.0	fTotalStrokes	 17
U+7644.0	fCangjie	KOGF
U+7644.0	fFourCorner	00130
U+7644.0	fHakka	  ziau1; zeu1
U+7644.0	fCantonese	ciu4
U+7644.0	fMandarin	qiao2
U+7644.0	fEnglish	thin, shrivelled; lean
U+7645.0	fUTF8	癅
U+7645.0	fCNS11643	3-5553
U+7645.0	fGB	E-343E
U+7645.0	fR/S	 104.12
U+7645.0	fTotalStrokes	 17
U+7645.0	fCangjie	KMLW
U+7645.0	fFourCorner	00160
U+7645.0	fCantonese	lau4
U+7645.0	fMandarin	liu2
U+7645.0	fEnglish	a swelling, tumor
U+7646.0	fUTF8	癆
U+7646.0	fCNS11643	1-7442
U+7646.0	fBig5	C0F7
U+7646.0	fGB	1-706C
U+7646.0	fR/S	 104.12
U+7646.0	fTotalStrokes	 17
U+7646.0	fCangjie	KFFS
U+7646.0	fFourCorner	00127
U+7646.0	fHakka	 lau2; lo2; lo5; lau5; lau3
U+7646.0	fCantonese	lou4
U+7646.0	fMandarin	lao2; lao4; liao2
U+7646.0	fEnglish	consumption; tuberculosis
U+7647.0	fUTF8	癇
U+7647.0	fCNS11643	2-5D63
U+7647.0	fBig5	ED77
U+7647.0	fGB	1-706F
U+7647.0	fR/S	 104.12
U+7647.0	fTotalStrokes	 17
U+7647.0	fCangjie	KANB
U+7647.0	fFourCorner	00127
U+7647.0	fHakka	 han2
U+7647.0	fCantonese	haan4 (also); gaan2
U+7647.0	fMandarin	xian2
U+7647.0	fEnglish	epilepsy; convulsions
U+7648.0	fUTF8	癈
U+7648.0	fCNS11643	2-5D61
U+7648.0	fBig5	ED75
U+7648.0	fGB	E-343F
U+7648.0	fR/S	 104.12; 104.9
U+7648.0	fTotalStrokes	 17
U+7648.0	fCangjie	KNOE
U+7648.0	fFourCorner	00147
U+7648.0	fHakka	 fui5; fi5
U+7648.0	fCantonese	fai3
U+7648.0	fMandarin	fei4
U+7648.0	fEnglish	abrogate; terminate; discard
U+7649.0	fUTF8	癉
U+7649.0	fCNS11643	2-5D62
U+7649.0	fBig5	ED76
U+7649.0	fGB	1-7077
U+7649.0	fR/S	 104.12
U+7649.0	fTotalStrokes	 17
U+7649.0	fCangjie	KRRJ
U+7649.0	fFourCorner	00146
U+7649.0	fHakka	  dan1; tan3;  dan3; dan5
U+7649.0	fCantonese	daan3; daan1; taan1; do3; taan2
U+7649.0	fMandarin	dan4; dan1; dan3; tan2; duo3; duo4
U+7649.0	fEnglish	[1] hate bitterly; detest [2] illness caused by overexhaustion; distress
U+764A.0	fUTF8	癊
U+764A.0	fCNS11643	5-5569
U+764A.0	fGB	3-5E6B
U+764A.0	fR/S	  104.11
U+764A.0	fTotalStrokes	 16
U+764A.0	fCangjie	KNLI
U+764A.0	fFourCorner	00131
U+764A.0	fHakka	 jim5; rim5; ngim5
U+764A.0	fCantonese	jam3; jan3
U+764A.0	fMandarin	yin4
U+764A.0	fEnglish	a disease of the heart
U+764B.0	fUTF8	癋
U+764B.0	fCNS11643	5-5C36
U+764B.0	fGB	E-3440
U+764B.0	fR/S	  104.12
U+764B.0	fTotalStrokes	 17
U+764B.0	fCangjie	KMMP
U+764B.0	fFourCorner	00130
U+764B.0	fCantonese	hok3
U+764B.0	fMandarin	he4
U+764C.0	fUTF8	癌
U+764C.0	fCNS11643	1-7444
U+764C.0	fBig5	C0F9
U+764C.0	fGB	0-3029
U+764C.0	fR/S	 104.12
U+764C.0	fTotalStrokes	 17
U+764C.0	fCangjie	KRRU
U+764C.0	fFourCorner	00172
U+764C.0	fHakka	 ngam2
U+764C.0	fCantonese	ngaam4; aam4 (also)
U+764C.0	fMandarin	ai2; yan2; obs
U+764C.0	fEnglish	cancer
U+764D.0	fUTF8	癍
U+764D.0	fGB	0-712D
U+764D.0	fR/S	 104.12
U+764D.0	fTotalStrokes	 17
U+764D.0	fCangjie	KMGG
U+764D.0	fFourCorner	0011
U+764D.0	fCantonese	baan1
U+764D.0	fMandarin	ban1
U+764D.0	fEnglish	unhealthy marks on the skin
U+764E.0	fUTF8	癎
U+764E.0	fCNS11643	3-5554
U+764E.0	fGB	5-5D35
U+764E.0	fR/S	 104.12
U+764E.0	fTotalStrokes	 17
U+764E.0	fCangjie	KANA
U+764E.0	fFourCorner	00120
U+764E.0	fHakka	 han2;  gan3; gien3
U+764E.0	fCantonese	gaan2; haan4 (also)
U+764E.0	fMandarin	xian2
U+764E.0	fEnglish	epilepsy; convulsions
U+764F.0	fUTF8	癏
U+764F.0	fCNS11643	3-5834
U+764F.0	fGB	5-5D6E
U+764F.0	fR/S	 104.13
U+764F.0	fTotalStrokes	 18
U+764F.0	fCangjie	KWLV
U+764F.0	fFourCorner	00130
U+764F.0	fCantonese	gwaan1
U+764F.0	fMandarin	guan1
U+7650.0	fUTF8	癐
U+7650.0	fCNS11643	2-623A
U+7650.0	fBig5	F04D
U+7650.0	fGB	3-5E68
U+7650.0	fR/S	 104.13
U+7650.0	fTotalStrokes	 18
U+7650.0	fCangjie	KOMA
U+7650.0	fFourCorner	00166
U+7650.0	fHakka	  kwai5
U+7650.0	fCantonese	gui6; gwui6 (also); wai1
U+7650.0	fMandarin	guai4; wei1
U+7650.0	fEnglish	[1] dangerously ill [2] tired; exhausted
U+7651.0	fUTF8	癑
U+7651.0	fCNS11643	3-5833
U+7651.0	fGB	5-5D2A
U+7651.0	fR/S	 104.13
U+7651.0	fTotalStrokes	 18
U+7651.0	fCangjie	KTWV
U+7651.0	fFourCorner	00130
U+7651.0	fCantonese	nung6; lung6 (also)
U+7651.0	fMandarin	nong2
U+7652.0	fUTF8	癒
U+7652.0	fCNS11643	1-766B
U+7652.0	fBig5	C2A1
U+7652.0	fGB	8-2D74
U+7652.0	fR/S	 104.13
U+7652.0	fTotalStrokes	 18
U+7652.0	fCangjie	KOMP
U+7652.0	fFourCorner	00132
U+7652.0	fHakka	 ji3
U+7652.0	fCantonese	jyu6
U+7652.0	fMandarin	yu4
U+7652.0	fEnglish	get well; recovered; healed; cured
U+7653.0	fUTF8	癓
U+7653.0	fCNS11643	2-623B
U+7653.0	fBig5	F04E
U+7653.0	fGB	5-5D70
U+7653.0	fR/S	 104.13
U+7653.0	fTotalStrokes	 18
U+7653.0	fCangjie	KHOK
U+7653.0	fFourCorner	00148
U+7653.0	fCantonese	mei4
U+7653.0	fMandarin	wei2
U+7654.0	fUTF8	癔
U+7654.0	fCNS11643	4-5B75
U+7654.0	fGB	0-712F
U+7654.0	fR/S	 104.13
U+7654.0	fTotalStrokes	 18
U+7654.0	fCangjie	KYTP
U+7654.0	fFourCorner	00130
U+7654.0	fCantonese	ji3
U+7654.0	fMandarin	yi4
U+7654.0	fEnglish	hysterical
U+7655.0	fUTF8	癕
U+7655.0	fCNS11643	4-5B79
U+7655.0	fGB	5-5D73
U+7655.0	fR/S	  104.13
U+7655.0	fTotalStrokes	 18
U+7655.0	fCangjie	KYVG
U+7655.0	fFourCorner	00110
U+7655.0	fCantonese	jung1
U+7655.0	fMandarin	yong1
U+7655.0	fEnglish	loss of the sense of smell
U+7656.0	fUTF8	癖
U+7656.0	fCNS11643	1-7669
U+7656.0	fBig5	C27D
U+7656.0	fGB	0-7131
U+7656.0	fR/S	 104.13
U+7656.0	fTotalStrokes	 18
U+7656.0	fCangjie	KSRJ
U+7656.0	fFourCorner	00141
U+7656.0	fHakka	 pit7; biak7
U+7656.0	fCantonese	pik1
U+7656.0	fMandarin	pi3; pi4
U+7656.0	fEnglish	[1] chronic swelling of the spleen [2] addiction; a habitual inclination
U+7657.0	fUTF8	癗
U+7657.0	fCNS11643	2-623C
U+7657.0	fBig5	F04F
U+7657.0	fGB	5-5D6D
U+7657.0	fR/S	 104.13
U+7657.0	fTotalStrokes	 18
U+7657.0	fCangjie	KMBW
U+7657.0	fFourCorner	00162
U+7657.0	fHakka	 lui3
U+7657.0	fCantonese	leoi5
U+7657.0	fMandarin	lei3
U+7657.0	fEnglish	[1] pimples [2] small blisters
U+7658.0	fUTF8	癘
U+7658.0	fCNS11643	1-766A
U+7658.0	fBig5	C27E
U+7658.0	fGB	1-705D
U+7658.0	fR/S	 104.13
U+7658.0	fTotalStrokes	 18
U+7658.0	fCangjie	KTWB
U+7658.0	fFourCorner	00127
U+7658.0	fHakka	 li5; li6; li3
U+7658.0	fCantonese	lai6; laai3; laai6
U+7658.0	fMandarin	li4; ji1; lai4
U+7658.0	fEnglish	[1] sore; ulcer [2] pestilence; pestilential vapor
U+7659.0	fUTF8	癙
U+7659.0	fCNS11643	2-6239
U+7659.0	fBig5	F04C
U+7659.0	fGB	3-5F44
U+7659.0	fR/S	 104.13
U+7659.0	fTotalStrokes	 18
U+7659.0	fCangjie	KHXV
U+7659.0	fFourCorner	00117
U+7659.0	fHakka	 chu3; cu3; shu3; su3
U+7659.0	fCantonese	syu2
U+7659.0	fMandarin	shu3; shu4
U+7659.0	fEnglish	ill
U+765A.0	fUTF8	癚
U+765A.0	fCNS11643	2-623D
U+765A.0	fBig5	F050
U+765A.0	fGB	5-5D71
U+765A.0	fR/S	 104.13
U+765A.0	fTotalStrokes	 18
U+765A.0	fCangjie	KNCR
U+765A.0	fFourCorner	00161
U+765A.0	fCantonese	daam3
U+765A.0	fMandarin	dan4
U+765B.0	fUTF8	癛
U+765B.0	fCNS11643	3-5832
U+765B.0	fGB	E-3441
U+765B.0	fR/S	  104.13
U+765B.0	fTotalStrokes	 18
U+765B.0	fCangjie	KYWD
U+765B.0	fFourCorner	00190
U+765B.0	fCantonese	lam5
U+765B.0	fMandarin	lin3; bing3
U+765C.0	fUTF8	癜
U+765C.0	fCNS11643	2-6237
U+765C.0	fBig5	F04A
U+765C.0	fGB	0-7130
U+765C.0	fR/S	 104.13
U+765C.0	fTotalStrokes	 18
U+765C.0	fCangjie	KSCE
U+765C.0	fFourCorner	00147
U+765C.0	fHakka	 tien5
U+765C.0	fCantonese	din6
U+765C.0	fMandarin	dian4
U+765C.0	fEnglish	erythema
U+765D.0	fUTF8	癝
U+765D.0	fGB	3-5F45
U+765D.0	fR/S	 104.13
U+765D.0	fTotalStrokes	 18
U+765D.0	fCangjie	KYWF
U+765D.0	fFourCorner	0019
U+765D.0	fCantonese	lam5
U+765D.0	fMandarin	lin3; bing3
U+765E.0	fUTF8	癞
U+765E.0	fGB	0-712E
U+765E.0	fR/S	 104.13
U+765E.0	fTotalStrokes	 18
U+765E.0	fCangjie	XKDLO
U+765E.0	fHakka	 lai5
U+765E.0	fCantonese	laai3
U+765E.0	fMandarin	lai4; la4
U+765E.0	fEnglish	simplified form of &#x7669;
U+765F.0	fUTF8	癟
U+765F.0	fCNS11643	1-7852
U+765F.0	fBig5	C3A7
U+765F.0	fGB	1-3171
U+765F.0	fR/S	 104.14
U+765F.0	fTotalStrokes	 19
U+765F.0	fCangjie	KHUB
U+765F.0	fFourCorner	00127
U+765F.0	fHakka	 biet7; piet7; piet8; pet7; bet7
U+765F.0	fCantonese	bit6
U+765F.0	fMandarin	bie3; pie1; bie1
U+765F.0	fEnglish	flat; sunken; not full
U+7660.0	fUTF8	癠
U+7660.0	fCNS11643	2-6624
U+7660.0	fBig5	F278
U+7660.0	fGB	5-5D28
U+7660.0	fR/S	 104.14
U+7660.0	fTotalStrokes	 19
U+7660.0	fCangjie	KYX
U+7660.0	fFourCorner	00123
U+7660.0	fHakka	 zi5; ci5; zi1; zi2; ci2; ci1
U+7660.0	fCantonese	zik3 (also); zek3; cai4; zai6; zai3
U+7660.0	fMandarin	ji4; qi2; ji2; ji3
U+7660.0	fEnglish	sick
U+7661.0	fUTF8	癡
U+7661.0	fCNS11643	1-7853
U+7661.0	fBig5	C3A8
U+7661.0	fGB	E-3442
U+7661.0	fR/S	 104.14
U+7661.0	fTotalStrokes	 19
U+7661.0	fCangjie	KPKO
U+7661.0	fFourCorner	00181
U+7661.0	fHakka	  chi1; ci1; cih1; chii1
U+7661.0	fCantonese	ci1
U+7661.0	fMandarin	chi1; chi2
U+7661.0	fEnglish	[1] silly; foolish; idiotic; stupid; senseless; crazy; insane [2] besot
U+7662.0	fUTF8	癢
U+7662.0	fCNS11643	1-797A
U+7662.0	fBig5	C46F
U+7662.0	fGB	1-5177
U+7662.0	fR/S	 104.15
U+7662.0	fTotalStrokes	 20
U+7662.0	fCangjie	KTOV
U+7662.0	fFourCorner	00132
U+7662.0	fHakka	 jong1;  jong3; hoi2; rong2; jong2; rong3; {jong1}
U+7662.0	fCantonese	joeng5
U+7662.0	fMandarin	yang3
U+7662.0	fEnglish	 itch; tickle
U+7663.0	fUTF8	癣
U+7663.0	fGB	0-5122
U+7663.0	fR/S	 104.14
U+7663.0	fTotalStrokes	 19
U+7663.0	fCangjie	KNMQ
U+7663.0	fHakka	 sien3; cien3
U+7663.0	fCantonese	sin2
U+7663.0	fMandarin	xian3; xuan3; xian1
U+7663.0	fEnglish	simplified form of &#x766C;
U+7664.0	fUTF8	癤
U+7664.0	fCNS11643	2-6238
U+7664.0	fBig5	F04B
U+7664.0	fGB	1-705C
U+7664.0	fR/S	 104.15
U+7664.0	fTotalStrokes	 20
U+7664.0	fCangjie	KHAL
U+7664.0	fFourCorner	00127
U+7664.0	fHakka	  ziet7; tsiat7; zet7
U+7664.0	fCantonese	zit3
U+7664.0	fMandarin	jie2
U+7664.0	fEnglish	 pimple; small sore
U+7664.1	fUTF8	癤
U+7664.1	fCNS11643	2-6238
U+7664.1	fBig5	F04B
U+7664.1	fGB	1-705C
U+7664.1	fR/S	 104.15
U+7664.1	fTotalStrokes	 20
U+7664.1	fCangjie	KHAL
U+7664.1	fFourCorner	00127
U+7664.1	fMandarin	jie1
U+7664.1	fEnglish	 gnarl
U+7665.0	fUTF8	癥
U+7665.0	fCNS11643	1-797B
U+7665.0	fBig5	C470
U+7665.0	fGB	1-5622
U+7665.0	fR/S	 104.15
U+7665.0	fTotalStrokes	 20
U+7665.0	fCangjie	KHOK
U+7665.0	fFourCorner	00148
U+7665.0	fHakka	 zhin1; zihn1; zin1
U+7665.0	fCantonese	zing1; zing3
U+7665.0	fMandarin	zheng1
U+7665.0	fEnglish	obstruction of bowels
U+7666.0	fUTF8	癦
U+7666.0	fGB	1-7C44
U+7666.0	fR/S	 104.15
U+7666.0	fTotalStrokes	 20
U+7666.0	fCangjie	KWGG
U+7666.0	fCantonese	mak6; mak2 (also)
U+7666.0	fEnglish	a mole (on the skin)
U+7667.0	fUTF8	癧
U+7667.0	fCNS11643	3-5E3F
U+7667.0	fGB	1-705F
U+7667.0	fR/S	 104.16
U+7667.0	fTotalStrokes	 21
U+7667.0	fCangjie	KMHM
U+7667.0	fFourCorner	00110
U+7667.0	fHakka	 let8;  lak8; lit8
U+7667.0	fCantonese	lik6; lek6 (also)
U+7667.0	fMandarin	li4
U+7667.0	fEnglish	scrofulous lumps or swellings
U+7668.0	fUTF8	癨
U+7668.0	fCNS11643	3-5E3E
U+7668.0	fGB	E-3443
U+7668.0	fR/S	  104.16
U+7668.0	fTotalStrokes	 21
U+7668.0	fCangjie	KMBG
U+7668.0	fFourCorner	00110
U+7668.0	fHakka	  kok7
U+7668.0	fCantonese	fok3
U+7668.0	fMandarin	huo4
U+7668.0	fEnglish	 quickly; suddenly a Chinese family name
U+7669.0	fUTF8	癩
U+7669.0	fCNS11643	1-7A79
U+7669.0	fBig5	C4EE
U+7669.0	fGB	1-712E
U+7669.0	fR/S	 104.16
U+7669.0	fTotalStrokes	 21
U+7669.0	fCangjie	KDLC
U+7669.0	fFourCorner	00186
U+7669.0	fHakka	  lai5; lau3; lai3; lat7; tai5
U+7669.0	fCantonese	laai3; laai6; laat6
U+7669.0	fMandarin	lai4; la4
U+7669.0	fEnglish	[1] leprosy [2] favus; scabies [3] bad
U+766A.0	fUTF8	癪
U+766A.0	fCNS11643	2-6B6D
U+766A.0	fBig5	F5DF
U+766A.0	fGB	E-3444
U+766A.0	fR/S	 104.16
U+766A.0	fTotalStrokes	 21
U+766A.0	fCangjie	KHDC
U+766A.0	fFourCorner	00186
U+766A.0	fCantonese	zik1
U+766A.0	fMandarin	ji1
U+766A.0	fEnglish	spasms; convulsions; hysteria
U+766B.0	fUTF8	癫
U+766B.0	fGB	0-7132
U+766B.0	fR/S	 104.16
U+766B.0	fTotalStrokes	 21
U+766B.0	fCangjie	KJCO
U+766B.0	fHakka	 dien1
U+766B.0	fCantonese	din1
U+766B.0	fMandarin	dian1
U+766B.0	fEnglish	simplified form of &#x7672;
U+766C.0	fUTF8	癬
U+766C.0	fCNS11643	1-7B6A
U+766C.0	fBig5	C57E
U+766C.0	fGB	1-5122
U+766C.0	fR/S	 104.17
U+766C.0	fTotalStrokes	 22
U+766C.0	fCangjie	KNFQ
U+766C.0	fFourCorner	00151
U+766C.0	fHakka	 sien3; sen3; cien3
U+766C.0	fCantonese	sin2
U+766C.0	fMandarin	xian3; xuan3; xian1
U+766C.0	fEnglish	ringworm; tetter
U+766D.0	fUTF8	癭
U+766D.0	fCNS11643	2-6D62
U+766D.0	fBig5	F6F4
U+766D.0	fGB	1-7128
U+766D.0	fR/S	 104.17
U+766D.0	fTotalStrokes	 22
U+766D.0	fCangjie	KBCV
U+766D.0	fFourCorner	00144
U+766D.0	fHakka	 rin3; jin3; jin2; jin1
U+766D.0	fCantonese	jing2
U+766D.0	fMandarin	ying3; ying1
U+766D.0	fEnglish	[1] a reddish swelling on the neck [2] gnarl
U+766E.0	fUTF8	癮
U+766E.0	fCNS11643	1-7B69
U+766E.0	fBig5	C57D
U+766E.0	fGB	1-712B
U+766E.0	fR/S	 104.17
U+766E.0	fTotalStrokes	 22
U+766E.0	fCangjie	KNLP
U+766E.0	fFourCorner	00136
U+766E.0	fHakka	  jin3; rin3; jin1
U+766E.0	fCantonese	jan5; jan2
U+766E.0	fMandarin	yin3
U+766E.0	fEnglish	[1] addiction; habitual craving [2] strong interest (in a sport or pastime)
U+766F.0	fUTF8	癯
U+766F.0	fCNS11643	3-603B
U+766F.0	fGB	0-7133
U+766F.0	fR/S	  104.18
U+766F.0	fTotalStrokes	 23
U+766F.0	fCangjie	KBUG
U+766F.0	fFourCorner	00110
U+766F.0	fHakka	 ki1; ki2; ki3
U+766F.0	fCantonese	keoi4
U+766F.0	fMandarin	qu2
U+766F.0	fEnglish	emaciated; thin; lean
U+7670.0	fUTF8	癰
U+7670.0	fCNS11643	2-6F39
U+7670.0	fBig5	F7EA
U+7670.0	fGB	1-5338
U+7670.0	fR/S	 104.18
U+7670.0	fTotalStrokes	 23
U+7670.0	fCangjie	KVUG
U+7670.0	fFourCorner	00114
U+7670.0	fHakka	 jung1; rung1
U+7670.0	fCantonese	jung1; jung2
U+7670.0	fMandarin	yong1
U+7670.0	fEnglish	 carbuncle
U+7671.0	fUTF8	癱
U+7671.0	fCNS11643	1-7C61
U+7671.0	fBig5	C5F5
U+7671.0	fGB	1-4C31
U+7671.0	fR/S	 104.19
U+7671.0	fTotalStrokes	 24
U+7671.0	fCangjie	KTOG
U+7671.0	fFourCorner	00114
U+7671.0	fHakka	  tan1
U+7671.0	fCantonese	taan1; taan2 (also)
U+7671.0	fMandarin	tan1
U+7671.0	fEnglish	paralysis; palsy; numbness
U+7672.0	fUTF8	癲
U+7672.0	fCNS11643	1-7C62
U+7672.0	fBig5	C5F6
U+7672.0	fGB	1-7132
U+7672.0	fR/S	 104.19
U+7672.0	fTotalStrokes	 24
U+7672.0	fCangjie	KJCC
U+7672.0	fFourCorner	00186
U+7672.0	fHakka	  dien1; tian1; den1
U+7672.0	fCantonese	din1
U+7672.0	fMandarin	dian1
U+7672.0	fEnglish	crazy; mad; madness; insane; mentally deranged; lunatic
U+7673.0	fUTF8	癳
U+7673.0	fCNS11643	3-656A
U+7673.0	fGB	E-3445
U+7673.0	fR/S	 104.21
U+7673.0	fTotalStrokes	 26
U+7673.0	fCangjie	KWWF
U+7673.0	fFourCorner	0019
U+7673.0	fCantonese	lo2
U+7673.0	fMandarin	luo3
U+7674.0	fUTF8	癴
U+7674.0	fCNS11643	7-6452
U+7674.0	fGB	E-3446
U+7674.0	fR/S	 104.23
U+7674.0	fTotalStrokes	 28
U+7674.0	fCangjie	KVFQ
U+7674.0	fFourCorner	00150
U+7674.0	fCantonese	lyun4
U+7674.0	fMandarin	luan2
U+7675.0	fUTF8	癵
U+7675.0	fCNS11643	2-723A
U+7675.0	fBig5	F9CC
U+7675.0	fGB	E-3447
U+7675.0	fR/S	 104.25
U+7675.0	fTotalStrokes	 30
U+7675.0	fCangjie	KVFB
U+7675.0	fFourCorner	00127
U+7675.0	fHakka	 lan2
U+7675.0	fCantonese	lyun4; lyun1
U+7675.0	fMandarin	luan2
U+7675.0	fEnglish	muscular contractions, as from rheumatism
U+7676.0	fUTF8	癶
U+7676.0	fCNS11643	3-225C
U+7676.0	fGB	5-5E76
U+7676.0	fR/S	 105.0
U+7676.0	fTotalStrokes	 5
U+7676.0	fCangjie	NIYO
U+7676.0	fFourCorner	12230
U+7676.0	fHakka	 pat7; bat7
U+7676.0	fCantonese	but6
U+7676.0	fMandarin	bo1
U+7676.0	fEnglish	 legs Kangxi radical number 105
U+7677.0	fUTF8	癷
U+7677.0	fCNS11643	3-656B
U+7677.0	fGB	E-3448
U+7677.0	fR/S	 105.3
U+7677.0	fTotalStrokes	 8
U+7677.0	fCangjie	NOMJ
U+7677.0	fFourCorner	1240
U+7677.0	fCantonese	gon1
U+7678.0	fUTF8	癸
U+7678.0	fCNS11643	1-5254
U+7678.0	fBig5	ACD1
U+7678.0	fGB	0-396F
U+7678.0	fR/S	 105.4
U+7678.0	fTotalStrokes	 9
U+7678.0	fCangjie	NOMK
U+7678.0	fFourCorner	12430
U+7678.0	fHakka	  gui5; kui3; kwui5
U+7678.0	fCantonese	gwai3
U+7678.0	fMandarin	gui3
U+7678.0	fEnglish	the last of ten Celestial stems
U+7679.0	fUTF8	癹
U+7679.0	fCNS11643	2-2C40
U+7679.0	fBig5	CFDE
U+7679.0	fGB	3-6025
U+7679.0	fR/S	 105.4
U+7679.0	fTotalStrokes	 9
U+7679.0	fCangjie	NOHNE
U+7679.0	fFourCorner	12407
U+7679.0	fCantonese	bat6
U+7679.0	fMandarin	po1; ba2
U+767A.0	fUTF8	発
U+767A.0	fGB	E-3449
U+767A.0	fR/S	 105.4
U+767A.0	fTotalStrokes	 9
U+767A.0	fCangjie	NOMKP
U+767A.0	fCantonese	faat3
U+767A.0	fMandarin	fa1; fa4
U+767A.0	fEnglish	issue; dispatch; send out; emit
U+767B.0	fUTF8	登
U+767B.0	fCNS11643	1-6074
U+767B.0	fBig5	B56E
U+767B.0	fGB	0-3547
U+767B.0	fR/S	 105.7
U+767B.0	fTotalStrokes	 12
U+767B.0	fCangjie	NOMRT
U+767B.0	fFourCorner	12108
U+767B.0	fHakka	  den1; dien1; dang1
U+767B.0	fCantonese	dang1
U+767B.0	fMandarin	deng1; deng4; de2
U+767B.0	fEnglish	[1] rise; mount; board; climb; ascent [2] record; register; enter [3] take; employ [4] board [5] step on; tread
U+767C.0	fUTF8	發
U+767C.0	fCNS11643	1-6075
U+767C.0	fBig5	B56F
U+767C.0	fGB	1-3722
U+767C.0	fR/S	 105.7
U+767C.0	fTotalStrokes	 12
U+767C.0	fCangjie	NONHE
U+767C.0	fFourCorner	12247
U+767C.0	fHakka	  fat7;  bot7; fak7
U+767C.0	fCantonese	faat3; fat3; but3; but6
U+767C.0	fMandarin	fa1; bo1
U+767C.0	fEnglish	[1] issue; publish; dispatch; send out; emit [2] shoot; launch [3] begin; start; initiate; originate [4] reveal; disclose; uncover [5] become; come to be [6] utter; express; speak [7] set off; set out [8] illuminate; help out
U+767C.1	fUTF8	發
U+767C.1	fCNS11643	1-6075
U+767C.1	fBig5	B56F
U+767C.1	fGB	1-3722
U+767C.1	fR/S	 105.7
U+767C.1	fTotalStrokes	 12
U+767C.1	fCangjie	NONHE
U+767C.1	fFourCorner	12247
U+767C.1	fMandarin	fa4
U+767C.2	fUTF8	發
U+767C.2	fCNS11643	1-6075
U+767C.2	fBig5	B56F
U+767C.2	fGB	1-3722
U+767C.2	fR/S	 105.7
U+767C.2	fTotalStrokes	 12
U+767C.2	fCangjie	NONHE
U+767C.2	fFourCorner	12247
U+767C.2	fMandarin	fa5
U+767D.0	fUTF8	白
U+767D.0	fCNS11643	1-4675
U+767D.0	fBig5	A5D5
U+767D.0	fGB	0-3057
U+767D.0	fR/S	 106.0
U+767D.0	fTotalStrokes	 5
U+767D.0	fCangjie	HA
U+767D.0	fFourCorner	26000
U+767D.0	fHakka	  pak8
U+767D.0	fCantonese	baak6; baak2 (also)
U+767D.0	fMandarin	bo2; bai2; ba4; bai3; zi4
U+767D.0	fEnglish	[1] white; clear; pure; unblemished; bright; clean; plain [2] empty; blank [3] in vain; for nothing [4] free of charge; gratis [5] the spoken part in an opera, etc. [6] state; explain [7] a Chinese family name
U+767D.1	fUTF8	白
U+767D.1	fCNS11643	1-4675
U+767D.1	fBig5	A5D5
U+767D.1	fGB	0-3057
U+767D.1	fR/S	 106.0
U+767D.1	fTotalStrokes	 5
U+767D.1	fCangjie	HA
U+767D.1	fFourCorner	26000
U+767D.1	fMandarin	bo5
U+767E.0	fUTF8	百
U+767E.0	fCNS11643	1-484B
U+767E.0	fBig5	A6CA
U+767E.0	fGB	0-3059
U+767E.0	fR/S	 106.1
U+767E.0	fTotalStrokes	 6
U+767E.0	fCangjie	MA
U+767E.0	fFourCorner	10600
U+767E.0	fHakka	  bak7
U+767E.0	fCantonese	baak3; mak6
U+767E.0	fMandarin	bai3; bo2; mo4
U+767E.0	fEnglish	[1] hundred [2] numerous; many [3] all
U+767F.0	fUTF8	癿
U+767F.0	fCNS11643	2-2267
U+767F.0	fBig5	CA47
U+767F.0	fGB	3-5C53
U+767F.0	fR/S	 106.1
U+767F.0	fTotalStrokes	 6
U+767F.0	fCangjie	HAU
U+767F.0	fFourCorner	22610
U+767F.0	fHakka	 ga1
U+767F.0	fCantonese	ke4; gaa1
U+767F.0	fMandarin	qie2
U+767F.0	fEnglish	a Chinese family name
U+7680.0	fUTF8	皀
U+7680.0	fCNS11643	3-2674
U+7680.0	fGB	3-5C54
U+7680.0	fR/S	 106.2
U+7680.0	fTotalStrokes	 7
U+7680.0	fCangjie	HAP
U+7680.0	fFourCorner	26710
U+7680.0	fCantonese	bik1
U+7680.0	fMandarin	bi1; ji2; xiang1
U+7680.0	fEnglish	 kernel; seed
U+7681.0	fUTF8	皁
U+7681.0	fCNS11643	2-2471
U+7681.0	fBig5	CB71
U+7681.0	fGB	E-344A
U+7681.0	fR/S	 106.2
U+7681.0	fTotalStrokes	 7
U+7681.0	fCangjie	HAJ
U+7681.0	fFourCorner	26400
U+7681.0	fHakka	  cau5; cau6
U+7681.0	fCantonese	zou6; zou3 (also)
U+7681.0	fMandarin	zao4
U+7681.0	fEnglish	[1] black [2] menial labor [3] menial; lictor
U+7682.0	fUTF8	皂
U+7682.0	fCNS11643	1-4B30
U+7682.0	fBig5	A86D
U+7682.0	fGB	0-546D
U+7682.0	fR/S	 106.2
U+7682.0	fTotalStrokes	 7
U+7682.0	fCangjie	HAP
U+7682.0	fFourCorner	26711
U+7682.0	fHakka	  cau5; cau6; co6; co5
U+7682.0	fCantonese	zou6; zou3 (also)
U+7682.0	fMandarin	zao4
U+7682.0	fEnglish	soap
U+7683.0	fUTF8	皃
U+7683.0	fCNS11643	3-2676
U+7683.0	fGB	5-5B4F
U+7683.0	fR/S	  106.2
U+7683.0	fTotalStrokes	 7
U+7683.0	fCangjie	HAHU
U+7683.0	fFourCorner	26210
U+7683.0	fHakka	 mau5; mau6
U+7683.0	fCantonese	maau6
U+7683.0	fMandarin	mao4
U+7683.0	fEnglish	countenance; appearance
U+7684.0	fUTF8	的
U+7684.0	fCNS11643	1-4E7B
U+7684.0	fBig5	AABA
U+7684.0	fGB	0-3544
U+7684.0	fR/S	 106.3
U+7684.0	fTotalStrokes	 8
U+7684.0	fCangjie	HAPI
U+7684.0	fFourCorner	27620
U+7684.0	fHakka	  dit7; tak8; tiak8; tit7; {gai5}; {ge5}
U+7684.0	fCantonese	dik1; di1
U+7684.0	fMandarin	di4; xiao4
U+7684.0	fEnglish	[1] possessive; adjectival suffix [2] "-ly" [3] adjectival ending, a prepositional phrase, or a relative [4] accurate; exact; proper ???
U+7684.1	fUTF8	的
U+7684.1	fCNS11643	1-4E7B
U+7684.1	fBig5	AABA
U+7684.1	fGB	0-3544
U+7684.1	fR/S	 106.3
U+7684.1	fTotalStrokes	 8
U+7684.1	fCangjie	HAPI
U+7684.1	fFourCorner	27620
U+7684.1	fMandarin	di2
U+7684.2	fUTF8	的
U+7684.2	fCNS11643	1-4E7B
U+7684.2	fBig5	AABA
U+7684.2	fGB	0-3544
U+7684.2	fR/S	 106.3
U+7684.2	fTotalStrokes	 8
U+7684.2	fCangjie	HAPI
U+7684.2	fFourCorner	27620
U+7684.2	fMandarin	de5
U+7685.0	fUTF8	皅
U+7685.0	fCNS11643	4-2A4C
U+7685.0	fGB	3-5C56
U+7685.0	fR/S	  106.4
U+7685.0	fTotalStrokes	 9
U+7685.0	fCangjie	HAAU
U+7685.0	fFourCorner	27610
U+7685.0	fCantonese	paa1
U+7685.0	fMandarin	pa1
U+7686.0	fUTF8	皆
U+7686.0	fCNS11643	1-5255
U+7686.0	fBig5	ACD2
U+7686.0	fGB	0-3D54
U+7686.0	fR/S	 106.4
U+7686.0	fTotalStrokes	 9
U+7686.0	fCangjie	PPHA
U+7686.0	fFourCorner	21601
U+7686.0	fHakka	  gai1; giai1
U+7686.0	fCantonese	gaai1
U+7686.0	fMandarin	jie1; ji1
U+7686.0	fEnglish	all; every; entire
U+7687.0	fUTF8	皇
U+7687.0	fCNS11643	1-5256
U+7687.0	fBig5	ACD3
U+7687.0	fGB	0-3B4A
U+7687.0	fR/S	 106.4
U+7687.0	fTotalStrokes	 9
U+7687.0	fCangjie	HAMG
U+7687.0	fFourCorner	26104
U+7687.0	fHakka	  fong2;  wong2; fong3
U+7687.0	fCantonese	wong4
U+7687.0	fMandarin	huang2; wang3
U+7687.0	fEnglish	[1] royal; imperial [2] emperor; ruler [3] beautiful; brilliant [4] uneasy; anxious [5] a term of respect for an ancestor
U+7688.0	fUTF8	皈
U+7688.0	fCNS11643	1-5257
U+7688.0	fBig5	ACD4
U+7688.0	fGB	0-7027
U+7688.0	fR/S	 106.4
U+7688.0	fTotalStrokes	 9
U+7688.0	fCangjie	HAHE
U+7688.0	fFourCorner	21647
U+7688.0	fHakka	 gui1; gwi1; kwui1
U+7688.0	fCantonese	gwai1
U+7688.0	fMandarin	gui1
U+7688.0	fEnglish	 follow; comply with
U+7689.0	fUTF8	皉
U+7689.0	fCNS11643	2-3749
U+7689.0	fBig5	D6A6
U+7689.0	fGB	5-5B59
U+7689.0	fR/S	 106.5
U+7689.0	fTotalStrokes	 10
U+7689.0	fCangjie	HAYMP
U+7689.0	fFourCorner	21610
U+7689.0	fCantonese	cai2; ci2
U+7689.0	fMandarin	ci3
U+768A.0	fUTF8	皊
U+768A.0	fCNS11643	2-312E
U+768A.0	fBig5	D2CB
U+768A.0	fGB	5-5B56
U+768A.0	fR/S	 106.5
U+768A.0	fTotalStrokes	 10
U+768A.0	fCangjie	HAOII
U+768A.0	fFourCorner	28627
U+768A.0	fCantonese	ling4
U+768A.0	fMandarin	ling2
U+768B.0	fUTF8	皋
U+768B.0	fCNS11643	1-5672
U+768B.0	fBig5	AF6F
U+768B.0	fGB	0-385E
U+768B.0	fR/S	 106.5
U+768B.0	fTotalStrokes	 10
U+768B.0	fCangjie	HAKJ
U+768B.0	fFourCorner	26403
U+768B.0	fHakka	 gau1
U+768B.0	fCantonese	gou1
U+768B.0	fMandarin	gao1; hao2; gu1; hao4
U+768B.0	fEnglish	[1] marsh; swamp [2] shore
U+768C.0	fUTF8	皌
U+768C.0	fCNS11643	4-2E41
U+768C.0	fGB	E-344B
U+768C.0	fR/S	  106.5
U+768C.0	fTotalStrokes	 10
U+768C.0	fCangjie	HADJ
U+768C.0	fFourCorner	25690
U+768C.0	fCantonese	mut6
U+768C.0	fMandarin	mo4
U+768D.0	fUTF8	皍
U+768D.0	fCNS11643	4-2E43
U+768D.0	fGB	E-344C
U+768D.0	fR/S	 106.5
U+768D.0	fTotalStrokes	 10
U+768D.0	fCangjie	HAPSL
U+768D.0	fFourCorner	26720
U+768D.0	fCantonese	zik1
U+768D.0	fMandarin	ji2
U+768E.0	fUTF8	皎
U+768E.0	fCNS11643	1-5C33
U+768E.0	fBig5	B2AE
U+768E.0	fGB	0-7028
U+768E.0	fR/S	 106.6
U+768E.0	fTotalStrokes	 11
U+768E.0	fCangjie	HAYCK
U+768E.0	fFourCorner	20648
U+768E.0	fHakka	 gau3; giau3
U+768E.0	fCantonese	gaau2; giu2
U+768E.0	fMandarin	jiao3
U+768E.0	fEnglish	[1] white; clean [2] bright; brilliant; lustrous
U+768F.0	fUTF8	皏
U+768F.0	fCNS11643	2-3748
U+768F.0	fBig5	D6A5
U+768F.0	fGB	3-5C57
U+768F.0	fR/S	 106.6
U+768F.0	fTotalStrokes	 11
U+768F.0	fCangjie	HATT
U+768F.0	fFourCorner	28641
U+768F.0	fCantonese	pang2
U+768F.0	fMandarin	peng3
U+7690.0	fUTF8	皐
U+7690.0	fCNS11643	3-374D
U+7690.0	fGB	E-344D
U+7690.0	fR/S	  106.6
U+7690.0	fTotalStrokes	 11
U+7690.0	fCangjie	HAEJ
U+7690.0	fFourCorner	26400
U+7690.0	fHakka	 gau1
U+7690.0	fCantonese	gou1
U+7690.0	fMandarin	gao1
U+7690.0	fEnglish	the high land along a river
U+7691.0	fUTF8	皑
U+7691.0	fGB	0-3028
U+7691.0	fR/S	 106.6
U+7691.0	fTotalStrokes	 11
U+7691.0	fCangjie	HAUSU
U+7691.0	fHakka	 ngoi2
U+7691.0	fCantonese	ngoi2; oi2 (also); ngoi4; ji4; oi4
U+7691.0	fMandarin	ai2; yi2
U+7691.0	fEnglish	simplified form of &#x769A;
U+7692.0	fUTF8	皒
U+7692.0	fCNS11643	2-3E3C
U+7692.0	fBig5	DAB8
U+7692.0	fGB	5-5B5F
U+7692.0	fR/S	 106.7
U+7692.0	fTotalStrokes	 12
U+7692.0	fCangjie	HAHQI
U+7692.0	fFourCorner	23650
U+7692.0	fCantonese	ngo4
U+7692.0	fMandarin	e2
U+7693.0	fUTF8	皓
U+7693.0	fCNS11643	1-6077
U+7693.0	fBig5	B571
U+7693.0	fGB	0-7029
U+7693.0	fR/S	 106.7
U+7693.0	fTotalStrokes	 12
U+7693.0	fCangjie	HAHGR
U+7693.0	fFourCorner	24661
U+7693.0	fHakka	  hau3;  gau3; hau1; hau6; hau2
U+7693.0	fCantonese	hou6; gou2
U+7693.0	fMandarin	hao4; hui1; gao3
U+7693.0	fEnglish	white and bright
U+7694.0	fUTF8	皔
U+7694.0	fCNS11643	3-3D4C
U+7694.0	fGB	3-5C59
U+7694.0	fR/S	  106.7
U+7694.0	fTotalStrokes	 12
U+7694.0	fCangjie	HAAMJ
U+7694.0	fFourCorner	26640
U+7694.0	fCantonese	hon5
U+7694.0	fMandarin	han4
U+7695.0	fUTF8	皕
U+7695.0	fCNS11643	2-3E3B
U+7695.0	fBig5	DAB7
U+7695.0	fGB	3-5C58
U+7695.0	fR/S	 106.7
U+7695.0	fTotalStrokes	 12
U+7695.0	fCangjie	MAMA
U+7695.0	fFourCorner	11660
U+7695.0	fCantonese	bik1; bat6
U+7695.0	fMandarin	bi1; bi4
U+7695.0	fEnglish	two hundred
U+7696.0	fUTF8	皖
U+7696.0	fCNS11643	1-6076
U+7696.0	fBig5	B570
U+7696.0	fGB	0-4D6E
U+7696.0	fR/S	 106.7
U+7696.0	fTotalStrokes	 12
U+7696.0	fCangjie	HAJMU
U+7696.0	fFourCorner	23611
U+7696.0	fHakka	  fan3; won3; fon3; wan3; van3; jen3
U+7696.0	fCantonese	wun5 (also); wun2
U+7696.0	fMandarin	wan3; huan3; huan4; huan2; hun4
U+7696.0	fEnglish	[1] name of an ancient state in what is today's Anhwei province [2] another name for Anhwei province
U+7697.0	fUTF8	皗
U+7697.0	fCNS11643	3-427E
U+7697.0	fGB	3-5C5C
U+7697.0	fR/S	  106.8
U+7697.0	fTotalStrokes	 13
U+7697.0	fCangjie	HABGR
U+7697.0	fFourCorner	27620
U+7697.0	fCantonese	cau4
U+7697.0	fMandarin	chou2
U+7698.0	fUTF8	皘
U+7698.0	fCNS11643	3-4321
U+7698.0	fGB	3-5C5A
U+7698.0	fR/S	 106.8
U+7698.0	fTotalStrokes	 13
U+7698.0	fCangjie	HAQMB
U+7698.0	fFourCorner	25620
U+7698.0	fCantonese	sin3
U+7698.0	fMandarin	qian4
U+7699.0	fUTF8	皙
U+7699.0	fCNS11643	2-453B
U+7699.0	fBig5	DED5
U+7699.0	fGB	0-702A
U+7699.0	fR/S	 106.8
U+7699.0	fTotalStrokes	 13
U+7699.0	fCangjie	DLHA
U+7699.0	fFourCorner	42602
U+7699.0	fHakka	  sit7; siak7
U+7699.0	fCantonese	sik1
U+7699.0	fMandarin	xi1; xi2
U+7699.0	fEnglish	[1] white [2] a kind of date
U+769A.0	fUTF8	皚
U+769A.0	fCNS11643	1-6D73
U+769A.0	fBig5	BD4A
U+769A.0	fGB	1-3028
U+769A.0	fR/S	 106.10
U+769A.0	fTotalStrokes	 15
U+769A.0	fCangjie	HAUMT
U+769A.0	fFourCorner	22618
U+769A.0	fHakka	  ngoi2
U+769A.0	fCantonese	ngoi4 (also); ji4 (also); oi4 (also); ngoi2; oi2 (also)
U+769A.0	fMandarin	ai2; yi2
U+769A.0	fEnglish	whiteness; pure white; white and clean; brightly white
U+769B.0	fUTF8	皛
U+769B.0	fCNS11643	2-5243
U+769B.0	fBig5	E6BB
U+769B.0	fGB	3-5C5F
U+769B.0	fR/S	 106.10
U+769B.0	fTotalStrokes	 15
U+769B.0	fCangjie	HAHAA
U+769B.0	fFourCorner	26660
U+769B.0	fHakka	  hiau3; hieu3
U+769B.0	fCantonese	kiu5; jiu5; miu5
U+769B.0	fMandarin	jiong3; xiao4; jiao3; po4; xiao3
U+769B.0	fEnglish	[1] bright [2] white
U+769C.0	fUTF8	皜
U+769C.0	fCNS11643	2-5240
U+769C.0	fBig5	E6B8
U+769C.0	fGB	E-344E
U+769C.0	fR/S	 106.10
U+769C.0	fTotalStrokes	 15
U+769C.0	fCangjie	HAYRB
U+769C.0	fFourCorner	20627
U+769C.0	fHakka	  hau3; gau3; hau2
U+769C.0	fCantonese	hou6
U+769C.0	fMandarin	hao4; gao3
U+769C.0	fEnglish	white; bright
U+769D.0	fUTF8	皝
U+769D.0	fCNS11643	2-5241
U+769D.0	fBig5	E6B9
U+769D.0	fGB	3-5C5D
U+769D.0	fR/S	 106.10
U+769D.0	fTotalStrokes	 15
U+769D.0	fCangjie	HGFMU
U+769D.0	fFourCorner	29111
U+769D.0	fCantonese	fong2
U+769D.0	fMandarin	huang3
U+769D.0	fEnglish	 luminous bright hoary; white
U+769E.0	fUTF8	皞
U+769E.0	fCNS11643	2-5242
U+769E.0	fBig5	E6BA
U+769E.0	fGB	3-5C5E
U+769E.0	fR/S	 106.10
U+769E.0	fTotalStrokes	 15
U+769E.0	fCangjie	HAHAJ
U+769E.0	fFourCorner	26653
U+769E.0	fHakka	  hau1; hau6; hau3
U+769E.0	fCantonese	hou6
U+769E.0	fMandarin	hao4
U+769E.0	fEnglish	bright; brilliant
U+769F.0	fUTF8	皟
U+769F.0	fCNS11643	4-5154
U+769F.0	fGB	3-5C5B
U+769F.0	fR/S	  106.11
U+769F.0	fTotalStrokes	 16
U+769F.0	fCangjie	HAQMC
U+769F.0	fFourCorner	25680
U+769F.0	fCantonese	caak3
U+769F.0	fMandarin	ze2; ce4
U+76A0.0	fUTF8	皠
U+76A0.0	fCNS11643	4-5155
U+76A0.0	fGB	3-5C63
U+76A0.0	fR/S	 106.11
U+76A0.0	fTotalStrokes	 16
U+76A0.0	fCangjie	HAUOG
U+76A0.0	fFourCorner	22610
U+76A0.0	fCantonese	ceoi3
U+76A0.0	fMandarin	cui3; cui4
U+76A1.0	fUTF8	皡
U+76A1.0	fCNS11643	3-5176
U+76A1.0	fGB	E-344F
U+76A1.0	fR/S	 106.11
U+76A1.0	fTotalStrokes	 16
U+76A1.0	fCangjie	HAHAJ
U+76A1.0	fFourCorner	26641
U+76A1.0	fCantonese	hou6
U+76A1.0	fMandarin	hao4
U+76A1.0	fEnglish	 bright brilliant
U+76A2.0	fUTF8	皢
U+76A2.0	fCNS11643	5-5C43
U+76A2.0	fGB	5-5B58
U+76A2.0	fR/S	 106.12
U+76A2.0	fTotalStrokes	 17
U+76A2.0	fCangjie	HAGGU
U+76A2.0	fFourCorner	24610
U+76A2.0	fCantonese	hiu2
U+76A2.0	fMandarin	xiao3
U+76A3.0	fUTF8	皣
U+76A3.0	fCNS11643	3-5556
U+76A3.0	fGB	5-5B5B
U+76A3.0	fR/S	 106.12
U+76A3.0	fTotalStrokes	 17
U+76A3.0	fCangjie	HATMJ
U+76A3.0	fFourCorner	24650
U+76A3.0	fCantonese	jip6
U+76A3.0	fMandarin	ye4
U+76A4.0	fUTF8	皤
U+76A4.0	fCNS11643	2-5D64
U+76A4.0	fBig5	ED78
U+76A4.0	fGB	0-702B
U+76A4.0	fR/S	 106.12
U+76A4.0	fTotalStrokes	 17
U+76A4.0	fCangjie	HAHDW
U+76A4.0	fFourCorner	22669
U+76A4.0	fHakka	 po2
U+76A4.0	fCantonese	po4
U+76A4.0	fMandarin	po2; pan2; bo1
U+76A4.0	fEnglish	[1] white [2] corpulent; vast belly [3] hoary-headed (old man)
U+76A5.0	fUTF8	皥
U+76A5.0	fCNS11643	3-5557
U+76A5.0	fGB	E-3450
U+76A5.0	fR/S	  106.12
U+76A5.0	fTotalStrokes	 17
U+76A5.0	fCangjie	HAHUJ
U+76A5.0	fFourCorner	26640
U+76A5.0	fHakka	  hau1;  hau3
U+76A5.0	fCantonese	hou6
U+76A5.0	fMandarin	hao4
U+76A5.0	fEnglish	bright; brilliant
U+76A6.0	fUTF8	皦
U+76A6.0	fCNS11643	2-623E
U+76A6.0	fBig5	F051
U+76A6.0	fGB	3-5C64
U+76A6.0	fR/S	 106.13
U+76A6.0	fTotalStrokes	 18
U+76A6.0	fCangjie	HAHSK
U+76A6.0	fFourCorner	28640
U+76A6.0	fHakka	  giau3; kieu3
U+76A6.0	fCantonese	giu2; gaau2
U+76A6.0	fMandarin	jiao3
U+76A6.0	fEnglish	 bright white bright; clear
U+76A7.0	fUTF8	皧
U+76A7.0	fCNS11643	4-5B7C
U+76A7.0	fGB	3-5C60
U+76A7.0	fR/S	  106.13
U+76A7.0	fTotalStrokes	 18
U+76A7.0	fCangjie	HABBE
U+76A7.0	fFourCorner	22640
U+76A7.0	fCantonese	oi3; ngoi3 (also)
U+76A7.0	fMandarin	ai4
U+76A8.0	fUTF8	皨
U+76A8.0	fCNS11643	4-5B7D
U+76A8.0	fGB	E-3451
U+76A8.0	fR/S	 106.13
U+76A8.0	fTotalStrokes	 18
U+76A8.0	fCangjie	HAHAG
U+76A8.0	fFourCorner	26100
U+76A8.0	fCantonese	sing1
U+76A8.0	fMandarin	xing1
U+76A9.0	fUTF8	皩
U+76A9.0	fCNS11643	3-5A5E
U+76A9.0	fGB	E-3452
U+76A9.0	fR/S	 106.14
U+76A9.0	fTotalStrokes	 19
U+76A9.0	fCangjie	HGAFU
U+76A9.0	fFourCorner	26110
U+76A9.0	fCantonese	fong2
U+76A9.0	fMandarin	huang4; huang3
U+76AA.0	fUTF8	皪
U+76AA.0	fCNS11643	2-6940
U+76AA.0	fBig5	F471
U+76AA.0	fGB	5-5B57
U+76AA.0	fR/S	 106.15
U+76AA.0	fTotalStrokes	 20
U+76AA.0	fCangjie	HAVID
U+76AA.0	fFourCorner	22694
U+76AA.0	fHakka	 lit8; lok8
U+76AA.0	fCantonese	lik6; lik1
U+76AA.0	fMandarin	li4; luo4; bo1
U+76AA.0	fEnglish	the lustre of pearls
U+76AB.0	fUTF8	皫
U+76AB.0	fCNS11643	2-693F
U+76AB.0	fBig5	F470
U+76AB.0	fGB	3-5C65
U+76AB.0	fR/S	 106.15
U+76AB.0	fTotalStrokes	 20
U+76AB.0	fCangjie	HAIPF
U+76AB.0	fFourCorner	20631
U+76AB.0	fHakka	 piau1; piau3; p'ieu3
U+76AB.0	fCantonese	piu2; piu5 (also); piu3
U+76AB.0	fMandarin	piao3; pao3
U+76AB.0	fEnglish	white
U+76AC.0	fUTF8	皬
U+76AC.0	fCNS11643	4-665A
U+76AC.0	fGB	5-5B66
U+76AC.0	fR/S	  106.16
U+76AC.0	fTotalStrokes	 21
U+76AC.0	fCangjie	HAMBG
U+76AC.0	fFourCorner	21610
U+76AC.0	fCantonese	hok6
U+76AC.0	fMandarin	he4; he2
U+76AD.0	fUTF8	皭
U+76AD.0	fCNS11643	2-6D63
U+76AD.0	fBig5	F6F5
U+76AD.0	fGB	3-5C66
U+76AD.0	fR/S	 106.18
U+76AD.0	fTotalStrokes	 23
U+76AD.0	fCangjie	HABWI
U+76AD.0	fFourCorner	22646
U+76AD.0	fHakka	  ziok7
U+76AD.0	fCantonese	zoek3; ziu3 (also); ziu6 (also); zoek6
U+76AD.0	fMandarin	jiao4; jue2
U+76AD.0	fEnglish	 white bright clear clean
U+76AE.0	fUTF8	皮
U+76AE.0	fCNS11643	1-4676
U+76AE.0	fBig5	A5D6
U+76AE.0	fGB	0-4624
U+76AE.0	fR/S	 107.0
U+76AE.0	fTotalStrokes	 5
U+76AE.0	fCangjie	DHE
U+76AE.0	fFourCorner	40247
U+76AE.0	fHakka	  pi2; pi3; cak7; pi5
U+76AE.0	fCantonese	pei4; pei2 (also)
U+76AE.0	fMandarin	pi2
U+76AE.0	fEnglish	[1] skin; hide; fur; leather; rind; peltry bark [2] thin sheet [3] naughty [4] a Chinese family name
U+76AF.0	fUTF8	皯
U+76AF.0	fCNS11643	2-2837
U+76AF.0	fBig5	CD75
U+76AF.0	fGB	3-5F75
U+76AF.0	fR/S	 107.3
U+76AF.0	fTotalStrokes	 8
U+76AF.0	fCangjie	DEMJ
U+76AF.0	fFourCorner	41240
U+76AF.0	fCantonese	gon2
U+76AF.0	fMandarin	gan3; gan4
U+76B0.0	fUTF8	皰
U+76B0.0	fCNS11643	1-5673
U+76B0.0	fBig5	AF70
U+76B0.0	fGB	E-3453
U+76B0.0	fR/S	 107.5
U+76B0.0	fTotalStrokes	 10
U+76B0.0	fCangjie	DEPRU
U+76B0.0	fFourCorner	47212
U+76B0.0	fHakka	 pau5;  piau5
U+76B0.0	fCantonese	paau3
U+76B0.0	fMandarin	pao4
U+76B0.0	fEnglish	pimples
U+76B1.0	fUTF8	皱
U+76B1.0	fGB	0-5665
U+76B1.0	fR/S	 107.5
U+76B1.0	fTotalStrokes	 10
U+76B1.0	fCangjie	NSDHE
U+76B1.0	fHakka	 ziu5
U+76B1.0	fCantonese	zau3; caau4 (also)
U+76B1.0	fMandarin	zhou4; zhou1
U+76B1.0	fEnglish	simplified form of &#x76BA;
U+76B2.0	fUTF8	皲
U+76B2.0	fGB	0-7164
U+76B2.0	fR/S	 107.6
U+76B2.0	fTotalStrokes	 11
U+76B2.0	fCangjie	BQDHE
U+76B2.0	fHakka	 giun1
U+76B2.0	fCantonese	gwan1
U+76B2.0	fMandarin	jun1; jun4; qun1
U+76B2.0	fEnglish	simplified form of &#x76B8;
U+76B3.0	fUTF8	皳
U+76B3.0	fCNS11643	3-3D4D
U+76B3.0	fGB	E-3454
U+76B3.0	fR/S	  107.7
U+76B3.0	fTotalStrokes	 12
U+76B3.0	fCangjie	IEDHE
U+76B3.0	fFourCorner	44140
U+76B3.0	fCantonese	kau4
U+76B3.0	fMandarin	qiu2
U+76B4.0	fUTF8	皴
U+76B4.0	fCNS11643	1-6078
U+76B4.0	fBig5	B572
U+76B4.0	fGB	0-7165
U+76B4.0	fR/S	 107.7
U+76B4.0	fTotalStrokes	 12
U+76B4.0	fCangjie	IEDHE
U+76B4.0	fFourCorner	24447
U+76B4.0	fHakka	  zun1
U+76B4.0	fCantonese	seon1; ceon1 (also)
U+76B4.0	fMandarin	cun1
U+76B4.0	fEnglish	[1] chapped or cracked from cold (skin) [2] a technique of representing irregular surfaces
U+76B5.0	fUTF8	皵
U+76B5.0	fCNS11643	2-453C
U+76B5.0	fBig5	DED6
U+76B5.0	fGB	3-5F7D
U+76B5.0	fR/S	 107.8
U+76B5.0	fTotalStrokes	 13
U+76B5.0	fCangjie	TADHE
U+76B5.0	fFourCorner	44647
U+76B5.0	fHakka	 siak7; cok7
U+76B5.0	fCantonese	coek3 (also); zoek3; cik1
U+76B5.0	fMandarin	que4; cuo4; qi4
U+76B5.0	fEnglish	rough, wrinkled, as bark
U+76B6.0	fUTF8	皶
U+76B6.0	fCNS11643	4-4534
U+76B6.0	fGB	E-3455
U+76B6.0	fR/S	 107.9
U+76B6.0	fTotalStrokes	 14
U+76B6.0	fCangjie	DMDHE
U+76B6.0	fFourCorner	44140
U+76B6.0	fCantonese	zaa1
U+76B6.0	fMandarin	zha1; cu3
U+76B6.0	fEnglish	discolourations on the skin; pimples; blotches
U+76B7.0	fUTF8	皷
U+76B7.0	fCNS11643	3-4830
U+76B7.0	fGB	E-3456
U+76B7.0	fR/S	  107.9
U+76B7.0	fTotalStrokes	 14
U+76B7.0	fCangjie	GTDHE
U+76B7.0	fFourCorner	44140
U+76B7.0	fHakka	 gu3
U+76B7.0	fCantonese	gu2; gwu2 (also)
U+76B7.0	fMandarin	gu3
U+76B7.0	fEnglish	 drum excite; arouse
U+76B8.0	fUTF8	皸
U+76B8.0	fCNS11643	2-4C29
U+76B8.0	fBig5	E2E1
U+76B8.0	fGB	1-7164
U+76B8.0	fR/S	 107.9
U+76B8.0	fTotalStrokes	 14
U+76B8.0	fCangjie	BJDHE
U+76B8.0	fFourCorner	34547
U+76B8.0	fHakka	  giun1
U+76B8.0	fCantonese	gwan1
U+76B8.0	fMandarin	jun1; jun4; qun1
U+76B8.0	fEnglish	 crack or chap from cold or dryness (skin)
U+76B9.0	fUTF8	皹
U+76B9.0	fCNS11643	3-482E
U+76B9.0	fGB	E-3457
U+76B9.0	fR/S	 107.9
U+76B9.0	fTotalStrokes	 14
U+76B9.0	fCangjie	DEBJJ
U+76B9.0	fFourCorner	47250
U+76B9.0	fCantonese	gwan1
U+76B9.0	fMandarin	jun1; jun4; qun1
U+76B9.0	fEnglish	crack; chap
U+76BA.0	fUTF8	皺
U+76BA.0	fCNS11643	1-6D74
U+76BA.0	fBig5	BD4B
U+76BA.0	fGB	1-5665
U+76BA.0	fR/S	 107.10
U+76BA.0	fTotalStrokes	 15
U+76BA.0	fCangjie	PUDHE
U+76BA.0	fFourCorner	24447
U+76BA.0	fHakka	  ziu5; tsieu5
U+76BA.0	fCantonese	zau3; caau4 (also)
U+76BA.0	fMandarin	zhou4; zhou1
U+76BA.0	fEnglish	[1] rumples; wrinkles; creases; folds [2] wrinkle; fold; contract; crease; crumple
U+76BB.0	fUTF8	皻
U+76BB.0	fCNS11643	2-5860
U+76BB.0	fBig5	EA74
U+76BB.0	fGB	5-5E72
U+76BB.0	fR/S	 107.11
U+76BB.0	fTotalStrokes	 16
U+76BB.0	fCangjie	YMDHE
U+76BB.0	fFourCorner	24247
U+76BB.0	fHakka	  za1
U+76BB.0	fCantonese	zaa1
U+76BB.0	fMandarin	zha1; zhu3
U+76BB.0	fEnglish	pimples; blotches on the skin
U+76BC.0	fUTF8	皼
U+76BC.0	fCNS11643	3-656C
U+76BC.0	fGB	E-3458
U+76BC.0	fR/S	 107.12
U+76BC.0	fTotalStrokes	 17
U+76BC.0	fCangjie	XGTDH
U+76BC.0	fCantonese	gu2; gwu2 (also)
U+76BC.0	fMandarin	gu3
U+76BD.0	fUTF8	皽
U+76BD.0	fCNS11643	2-623F
U+76BD.0	fBig5	F052
U+76BD.0	fGB	3-6024
U+76BD.0	fR/S	 107.13
U+76BD.0	fTotalStrokes	 18
U+76BD.0	fCangjie	YMDHE
U+76BD.0	fFourCorner	04147
U+76BD.0	fHakka	 zhan3; zen3; zan3; zhan1; zhen3; zhen1
U+76BD.0	fCantonese	zin2
U+76BD.0	fMandarin	zhan3; zhao1; dan3
U+76BD.0	fEnglish	scurf
U+76BE.0	fUTF8	皾
U+76BE.0	fCNS11643	2-6941
U+76BE.0	fBig5	F472
U+76BE.0	fGB	5-5E6E
U+76BE.0	fR/S	 107.15
U+76BE.0	fTotalStrokes	 20
U+76BE.0	fCangjie	GCDHE
U+76BE.0	fFourCorner	44847
U+76BE.0	fCantonese	duk6
U+76BE.0	fMandarin	du2
U+76BF.0	fUTF8	皿
U+76BF.0	fCNS11643	1-4677
U+76BF.0	fBig5	A5D7
U+76BF.0	fGB	0-4373
U+76BF.0	fR/S	 108.0
U+76BF.0	fTotalStrokes	 5
U+76BF.0	fCangjie	BT
U+76BF.0	fFourCorner	77100
U+76BF.0	fHakka	 men3; men1; min3
U+76BF.0	fCantonese	ming5; ming2 (also)
U+76BF.0	fMandarin	min3; ming3; meng3
U+76BF.0	fEnglish	[1] a shallow container (such as a dish, plate, saucer, etc.) [2] Kangxi radical 108
U+76C0.0	fUTF8	盀
U+76C0.0	fCNS11643	3-2677
U+76C0.0	fGB	3-3053
U+76C0.0	fR/S	 108.2
U+76C0.0	fTotalStrokes	 7
U+76C0.0	fCangjie	YBT
U+76C0.0	fFourCorner	23100
U+76C0.0	fCantonese	hei2
U+76C0.0	fMandarin	qi3
U+76C1.0	fUTF8	盁
U+76C1.0	fCNS11643	3-2678
U+76C1.0	fGB	E-3459
U+76C1.0	fR/S	 108.2
U+76C1.0	fTotalStrokes	 7
U+76C1.0	fCangjie	NSBT
U+76C1.0	fFourCorner	17100
U+76C1.0	fCantonese	jing4
U+76C1.0	fMandarin	ying2
U+76C2.0	fUTF8	盂
U+76C2.0	fCNS11643	1-4E7C
U+76C2.0	fBig5	AABB
U+76C2.0	fGB	0-535B
U+76C2.0	fR/S	 108.3
U+76C2.0	fTotalStrokes	 8
U+76C2.0	fCangjie	MDBT
U+76C2.0	fFourCorner	10107
U+76C2.0	fHakka	 ji2; ri2; ji3; ji1; i2
U+76C2.0	fCantonese	jyu4; jyu1 (also)
U+76C2.0	fMandarin	yu2
U+76C2.0	fEnglish	 basin; broad-mouthed receptacle for holding liquid; jar
U+76C3.0	fUTF8	盃
U+76C3.0	fCNS11643	1-525A
U+76C3.0	fBig5	ACD7
U+76C3.0	fGB	E-345A
U+76C3.0	fR/S	 108.4
U+76C3.0	fTotalStrokes	 9
U+76C3.0	fCangjie	MFBT
U+76C3.0	fFourCorner	10107
U+76C3.0	fHakka	  bui1; bi1
U+76C3.0	fCantonese	bui1
U+76C3.0	fMandarin	bei1
U+76C3.0	fEnglish	 tumbler; cup
U+76C4.0	fUTF8	盄
U+76C4.0	fCNS11643	2-2C41
U+76C4.0	fBig5	CFDF
U+76C4.0	fGB	5-5A5C
U+76C4.0	fR/S	 108.4
U+76C4.0	fTotalStrokes	 9
U+76C4.0	fCangjie	NLBT
U+76C4.0	fFourCorner	17107
U+76C4.0	fCantonese	diu3
U+76C4.0	fMandarin	zhao1; diao4
U+76C5.0	fUTF8	盅
U+76C5.0	fCNS11643	1-525B
U+76C5.0	fBig5	ACD8
U+76C5.0	fGB	0-5651
U+76C5.0	fR/S	 108.4
U+76C5.0	fTotalStrokes	 9
U+76C5.0	fCangjie	LBT
U+76C5.0	fFourCorner	50107
U+76C5.0	fHakka	 zung1;  zhung1
U+76C5.0	fCantonese	zung1
U+76C5.0	fMandarin	zhong1; chong1; chong2
U+76C5.0	fEnglish	 small cup or bowl
U+76C6.0	fUTF8	盆
U+76C6.0	fCNS11643	1-5259
U+76C6.0	fBig5	ACD6
U+76C6.0	fGB	0-4568
U+76C6.0	fR/S	 108.4
U+76C6.0	fTotalStrokes	 9
U+76C6.0	fCangjie	CSHT
U+76C6.0	fFourCorner	80107
U+76C6.0	fHakka	  pun2; pun3
U+76C6.0	fCantonese	pun4; pun2 (also)
U+76C6.0	fMandarin	pen2; ben4
U+76C6.0	fEnglish	 basin; tub; bowl
U+76C7.0	fUTF8	盇
U+76C7.0	fCNS11643	3-2E32
U+76C7.0	fGB	E-345B
U+76C7.0	fR/S	 108.4
U+76C7.0	fTotalStrokes	 9
U+76C7.0	fCangjie	KIBT
U+76C7.0	fFourCorner	40100
U+76C7.0	fHakka	 hap8
U+76C7.0	fCantonese	hap6; jing4; jing2 (also); wo4; jik1; but3
U+76C7.0	fMandarin	he2
U+76C7.0	fEnglish	why not ? would it not be better to?
U+76C8.0	fUTF8	盈
U+76C8.0	fCNS11643	1-5258
U+76C8.0	fBig5	ACD5
U+76C8.0	fGB	0-532F
U+76C8.0	fR/S	 108.4
U+76C8.0	fTotalStrokes	 9
U+76C8.0	fCangjie	NSBT
U+76C8.0	fFourCorner	17107
U+76C8.0	fHakka	  jin2; rin2; jin3
U+76C8.0	fCantonese	jing4
U+76C8.0	fMandarin	ying2
U+76C8.0	fEnglish	 fill; become full
U+76C9.0	fUTF8	盉
U+76C9.0	fCNS11643	2-312F
U+76C9.0	fBig5	D2CC
U+76C9.0	fGB	3-5B60
U+76C9.0	fR/S	 108.5
U+76C9.0	fTotalStrokes	 10
U+76C9.0	fCangjie	HDBT
U+76C9.0	fFourCorner	20101
U+76C9.0	fHakka	 wo1; fo2; fo3; fo5
U+76C9.0	fCantonese	wo4; wo6
U+76C9.0	fMandarin	he2; he4
U+76C9.0	fEnglish	 mix and season dishes as a cook
U+76CA.0	fUTF8	益
U+76CA.0	fCNS11643	1-5674
U+76CA.0	fBig5	AF71
U+76CA.0	fGB	0-5266
U+76CA.0	fR/S	 108.5
U+76CA.0	fTotalStrokes	 10
U+76CA.0	fCangjie	TCBT
U+76CA.0	fFourCorner	80107
U+76CA.0	fHakka	  jit7; rit7; jit8; {jak7}; jak7
U+76CA.0	fCantonese	jik1
U+76CA.0	fMandarin	yi4; yi2
U+76CA.0	fEnglish	[1] increase; add to; augment [2] in a higher degree; to a greater extent; more [3] profit; benefit; advantage
U+76CB.0	fUTF8	盋
U+76CB.0	fCNS11643	3-325A
U+76CB.0	fGB	E-345C
U+76CB.0	fR/S	  108.5
U+76CB.0	fTotalStrokes	 10
U+76CB.0	fCangjie	IKBT
U+76CB.0	fFourCorner	43100
U+76CB.0	fHakka	 bat7
U+76CB.0	fMandarin	bo1
U+76CB.0	fEnglish	 disk; platter
U+76CC.0	fUTF8	盌
U+76CC.0	fCNS11643	3-325B
U+76CC.0	fGB	E-345D
U+76CC.0	fR/S	  108.5
U+76CC.0	fTotalStrokes	 10
U+76CC.0	fCangjie	NUBT
U+76CC.0	fFourCorner	27100
U+76CC.0	fHakka	 won3; wan3; von3
U+76CC.0	fCantonese	wun2
U+76CC.0	fMandarin	wan3
U+76CC.0	fEnglish	bowl; basin; cup
U+76CD.0	fUTF8	盍
U+76CD.0	fCNS11643	1-5675
U+76CD.0	fBig5	AF72
U+76CD.0	fGB	0-6E41
U+76CD.0	fR/S	 108.5
U+76CD.0	fTotalStrokes	 10
U+76CD.0	fCangjie	GIBT
U+76CD.0	fFourCorner	40107
U+76CD.0	fHakka	 hap8
U+76CD.0	fCantonese	hap6
U+76CD.0	fMandarin	he2; ke3; kai4
U+76CD.0	fEnglish	[1] what? [2] why not? [3] get together
U+76CE.0	fUTF8	盎
U+76CE.0	fCNS11643	1-5676
U+76CE.0	fBig5	AF73
U+76CE.0	fGB	0-303B
U+76CE.0	fR/S	 108.5
U+76CE.0	fTotalStrokes	 10
U+76CE.0	fCangjie	LKBT
U+76CE.0	fFourCorner	50107
U+76CE.0	fHakka	 ong5; ang1
U+76CE.0	fCantonese	ong3; ngong3 (also); on1 (also); ong2 (also); ngong2 (also); ngon1 (also)
U+76CE.0	fMandarin	ang4; ang3
U+76CE.0	fEnglish	[1] basin; cup; pot; bowl; tureen [2] abundant; plentiful; rich [3] sleek [4] an instrument of music which yields an earthy timbre when struck
U+76CF.0	fUTF8	盏
U+76CF.0	fGB	0-5535
U+76CF.0	fR/S	 108.5
U+76CF.0	fTotalStrokes	 10
U+76CF.0	fCangjie	IJBT
U+76CF.0	fHakka	 zan3
U+76CF.0	fCantonese	zaan2
U+76CF.0	fMandarin	zhan3
U+76CF.0	fEnglish	simplified form of &#x76DE;
U+76D0.0	fUTF8	盐
U+76D0.0	fGB	0-514E
U+76D0.0	fR/S	 108.5
U+76D0.0	fTotalStrokes	 10
U+76D0.0	fCangjie	GYBT
U+76D0.0	fHakka	 jam2
U+76D0.0	fCantonese	jim4; jim1 (also); jim6
U+76D0.0	fMandarin	yan2; yan4
U+76D0.0	fEnglish	simplified form of &#x9E7D;
U+76D1.0	fUTF8	监
U+76D1.0	fGB	0-3C60
U+76D1.0	fR/S	 108.5
U+76D1.0	fTotalStrokes	 10
U+76D1.0	fCangjie	LIBT
U+76D1.0	fHakka	 gam5; gam1
U+76D1.0	fCantonese	gaam1; gaam3
U+76D1.0	fMandarin	jian1; jian4; kan4
U+76D1.0	fEnglish	simplified form of &#x76E3;
U+76D2.0	fUTF8	盒
U+76D2.0	fCNS11643	1-5C35
U+76D2.0	fBig5	B2B0
U+76D2.0	fGB	0-3A50
U+76D2.0	fR/S	 108.6
U+76D2.0	fTotalStrokes	 11
U+76D2.0	fCangjie	ORBT
U+76D2.0	fFourCorner	80107
U+76D2.0	fHakka	  hap8
U+76D2.0	fCantonese	hap6; haap6 (also); hap2 (also)
U+76D2.0	fMandarin	he2; an1
U+76D2.0	fEnglish	 small box; case; casket
U+76D3.0	fUTF8	盓
U+76D3.0	fCNS11643	2-374A
U+76D3.0	fBig5	D6A7
U+76D3.0	fGB	3-5B3D
U+76D3.0	fR/S	 108.6
U+76D3.0	fTotalStrokes	 11
U+76D3.0	fCangjie	EDBT
U+76D3.0	fFourCorner	31107
U+76D3.0	fCantonese	jyu1; wu1
U+76D3.0	fMandarin	yu1; wu1; wu4
U+76D4.0	fUTF8	盔
U+76D4.0	fCNS11643	1-5C34
U+76D4.0	fBig5	B2AF
U+76D4.0	fGB	0-3F78
U+76D4.0	fR/S	 108.6
U+76D4.0	fTotalStrokes	 11
U+76D4.0	fCangjie	KFBT
U+76D4.0	fFourCorner	40107
U+76D4.0	fHakka	  kui1; foi1; kwi1; k'wui1
U+76D4.0	fCantonese	kwai1
U+76D4.0	fMandarin	kui1
U+76D4.0	fEnglish	[1] helmet [2] bowl; basin; pot
U+76D5.0	fUTF8	盕
U+76D5.0	fCNS11643	3-3750
U+76D5.0	fGB	3-5B3E
U+76D5.0	fR/S	 108.6
U+76D5.0	fTotalStrokes	 11
U+76D5.0	fCangjie	ENBT
U+76D5.0	fFourCorner	37100
U+76D5.0	fCantonese	faan3
U+76D5.0	fMandarin	fan4
U+76D6.0	fUTF8	盖
U+76D6.0	fCNS11643	3-3752
U+76D6.0	fGB	0-3847
U+76D6.0	fR/S	 108.6
U+76D6.0	fTotalStrokes	 11
U+76D6.0	fCangjie	TGBT
U+76D6.0	fFourCorner	80100
U+76D6.0	fHakka	  goi5
U+76D6.0	fCantonese	goi3; koi3 (also); hap1; hap6 (also); gap3; kap6
U+76D6.0	fMandarin	gai4; ge3; he2
U+76D6.0	fEnglish	simplified form of &#x84CB;
U+76D6.1	fUTF8	盖
U+76D6.1	fCNS11643	3-3752
U+76D6.1	fGB	0-3847
U+76D6.1	fR/S	 108.6
U+76D6.1	fTotalStrokes	 11
U+76D6.1	fCangjie	TGBT
U+76D6.1	fFourCorner	80100
U+76D6.1	fMandarin	gai5
U+76D7.0	fUTF8	盗
U+76D7.0	fCNS11643	3-3751
U+76D7.0	fGB	0-3541
U+76D7.0	fR/S	 108.6
U+76D7.0	fTotalStrokes	 11
U+76D7.0	fCangjie	IOBT
U+76D7.0	fFourCorner	37100
U+76D7.0	fHakka	 to6; tau5
U+76D7.0	fCantonese	dou6
U+76D7.0	fMandarin	dao4
U+76D7.0	fEnglish	 rob; steal thief; bandit
U+76D8.0	fUTF8	盘
U+76D8.0	fCNS11643	F-3723
U+76D8.0	fGB	0-454C
U+76D8.0	fR/S	 108.6
U+76D8.0	fTotalStrokes	 11
U+76D8.0	fCangjie	HYBT
U+76D8.0	fHakka	 pan2
U+76D8.0	fCantonese	pun4; pun2 (also)
U+76D8.0	fMandarin	pan2
U+76D8.0	fEnglish	simplified form of &#x76E4;
U+76D8.1	fUTF8	盘
U+76D8.1	fCNS11643	F-3723
U+76D8.1	fGB	0-454C
U+76D8.1	fR/S	 108.6
U+76D8.1	fTotalStrokes	 11
U+76D8.1	fCangjie	HYBT
U+76D8.1	fMandarin	pan5
U+76D9.0	fUTF8	盙
U+76D9.0	fCNS11643	3-3D4F
U+76D9.0	fGB	E-345E
U+76D9.0	fR/S	  108.7
U+76D9.0	fTotalStrokes	 12
U+76D9.0	fCangjie	IBBT
U+76D9.0	fFourCorner	53100
U+76D9.0	fHakka	  pu3
U+76D9.0	fCantonese	fu2
U+76D9.0	fMandarin	fu3
U+76D9.0	fEnglish	a basket;square outside and round inside; used to hold boiled grain in State worship
U+76DA.0	fUTF8	盚
U+76DA.0	fCNS11643	2-3E3D
U+76DA.0	fBig5	DAB9
U+76DA.0	fGB	3-5B3F
U+76DA.0	fR/S	 108.7
U+76DA.0	fTotalStrokes	 12
U+76DA.0	fCangjie	IEBT
U+76DA.0	fFourCorner	43107
U+76DA.0	fHakka	 kiu2
U+76DA.0	fCantonese	kau4
U+76DA.0	fMandarin	qiu2
U+76DB.0	fUTF8	盛
U+76DB.0	fCNS11643	1-5C36
U+76DB.0	fBig5	B2B1
U+76DB.0	fGB	0-4A22
U+76DB.0	fR/S	 108.7; 108.6
U+76DB.0	fTotalStrokes	 12
U+76DB.0	fCangjie	ISBT
U+76DB.0	fFourCorner	53107
U+76DB.0	fHakka	  shin5;  sin5; siin5; sin2; shin2
U+76DB.0	fCantonese	sing6; sing4; sing2 (also); dou6; luk6; zaan2; mang4; ming4
U+76DB.0	fMandarin	sheng4
U+76DB.0	fEnglish	 take (loose material) into a bowl or basin; hold; contain
U+76DB.1	fUTF8	盛
U+76DB.1	fCNS11643	1-5C36
U+76DB.1	fBig5	B2B1
U+76DB.1	fGB	0-4A22
U+76DB.1	fR/S	 108.7; 108.6
U+76DB.1	fTotalStrokes	 12
U+76DB.1	fCangjie	ISBT
U+76DB.1	fFourCorner	53107
U+76DB.1	fMandarin	cheng2
U+76DC.0	fUTF8	盜
U+76DC.0	fCNS11643	1-6079
U+76DC.0	fBig5	B573
U+76DC.0	fGB	E-345F
U+76DC.0	fR/S	 108.7
U+76DC.0	fTotalStrokes	 12
U+76DC.0	fCangjie	EOBT
U+76DC.0	fFourCorner	37107
U+76DC.0	fHakka	  tau5; to6; tau3; to5
U+76DC.0	fCantonese	dou6
U+76DC.0	fMandarin	dao4
U+76DC.0	fEnglish	 rob; steal; misappropriate
U+76DD.0	fUTF8	盝
U+76DD.0	fCNS11643	2-453D
U+76DD.0	fBig5	DED7
U+76DD.0	fGB	3-5B40
U+76DD.0	fR/S	 108.8
U+76DD.0	fTotalStrokes	 13
U+76DD.0	fCangjie	VEBT
U+76DD.0	fFourCorner	27107
U+76DD.0	fHakka	  luk7
U+76DD.0	fCantonese	luk6
U+76DD.0	fMandarin	lu4
U+76DD.0	fEnglish	drip
U+76DE.0	fUTF8	盞
U+76DE.0	fCNS11643	1-6540
U+76DE.0	fBig5	B7F8
U+76DE.0	fGB	1-5535
U+76DE.0	fR/S	 108.8
U+76DE.0	fTotalStrokes	 13
U+76DE.0	fCangjie	IIBT
U+76DE.0	fFourCorner	53107
U+76DE.0	fHakka	  zan3
U+76DE.0	fCantonese	zaan2
U+76DE.0	fMandarin	zhan3
U+76DE.0	fEnglish	small cup or container
U+76DF.0	fUTF8	盟
U+76DF.0	fCNS11643	1-6541
U+76DF.0	fBig5	B7F9
U+76DF.0	fGB	0-434B
U+76DF.0	fR/S	 108.8
U+76DF.0	fTotalStrokes	 13
U+76DF.0	fCangjie	ABBT
U+76DF.0	fFourCorner	67107
U+76DF.0	fHakka	  men2; men3
U+76DF.0	fCantonese	mang4; maang6
U+76DF.0	fMandarin	meng2; meng4; ming3; ming2; ming4
U+76DF.0	fEnglish	 swear oath; covenant; alliance
U+76E0.0	fUTF8	盠
U+76E0.0	fCNS11643	4-4535
U+76E0.0	fGB	5-5A65
U+76E0.0	fR/S	  108.9
U+76E0.0	fTotalStrokes	 14
U+76E0.0	fCangjie	VOBT
U+76E0.0	fFourCorner	27100
U+76E0.0	fCantonese	lai4
U+76E0.0	fMandarin	li3; li2
U+76E1.0	fUTF8	盡
U+76E1.0	fCNS11643	1-6970
U+76E1.0	fBig5	BAC9
U+76E1.0	fGB	1-3E21
U+76E1.0	fR/S	 108.9
U+76E1.0	fTotalStrokes	 14
U+76E1.0	fCangjie	LMFBT
U+76E1.0	fFourCorner	50107
U+76E1.0	fHakka	 cin3;  cin5; t's'in6; t'sin5; cin6; cin1; ziang5
U+76E1.0	fCantonese	zeon6; zeon2
U+76E1.0	fMandarin	jin3
U+76E1.0	fEnglish	 exhaust; use up deplete
U+76E1.1	fUTF8	盡
U+76E1.1	fCNS11643	1-6970
U+76E1.1	fBig5	BAC9
U+76E1.1	fGB	1-3E21
U+76E1.1	fR/S	 108.9
U+76E1.1	fTotalStrokes	 14
U+76E1.1	fCangjie	LMFBT
U+76E1.1	fFourCorner	50107
U+76E1.1	fMandarin	jin4
U+76E2.0	fUTF8	盢
U+76E2.0	fCNS11643	3-4831
U+76E2.0	fGB	E-3460
U+76E2.0	fR/S	 108.9
U+76E2.0	fTotalStrokes	 14
U+76E2.0	fCangjie	WKBT
U+76E2.0	fFourCorner	63100
U+76E2.0	fCantonese	gwik1
U+76E2.0	fMandarin	xu4
U+76E3.0	fUTF8	監
U+76E3.0	fCNS11643	1-6971
U+76E3.0	fBig5	BACA
U+76E3.0	fGB	1-3C60
U+76E3.0	fR/S	 108.9
U+76E3.0	fTotalStrokes	 14
U+76E3.0	fCangjie	SIBT
U+76E3.0	fFourCorner	78107
U+76E3.0	fHakka	  gam5;  gam1; gan1
U+76E3.0	fCantonese	gaam1; gaam3
U+76E3.0	fMandarin	jian1; jian4; kan4
U+76E3.0	fEnglish	supervise; control; direct
U+76E4.0	fUTF8	盤
U+76E4.0	fCNS11643	1-6D75
U+76E4.0	fBig5	BD4C
U+76E4.0	fGB	1-454C
U+76E4.0	fR/S	 108.10
U+76E4.0	fTotalStrokes	 15
U+76E4.0	fCangjie	HEBT
U+76E4.0	fFourCorner	27107
U+76E4.0	fHakka	  pan2; pan3
U+76E4.0	fCantonese	pun4; pun2 (also)
U+76E4.0	fMandarin	pan2; xuan2
U+76E4.0	fEnglish	 tray; plate; dish examine
U+76E4.1	fUTF8	盤
U+76E4.1	fCNS11643	1-6D75
U+76E4.1	fBig5	BD4C
U+76E4.1	fGB	1-454C
U+76E4.1	fR/S	 108.10
U+76E4.1	fTotalStrokes	 15
U+76E4.1	fCangjie	HEBT
U+76E4.1	fFourCorner	27107
U+76E4.1	fMandarin	pan5
U+76E5.0	fUTF8	盥
U+76E5.0	fCNS11643	1-714E
U+76E5.0	fBig5	BF64
U+76E5.0	fGB	0-6E42
U+76E5.0	fR/S	 108.11
U+76E5.0	fTotalStrokes	 16
U+76E5.0	fCangjie	HXBT
U+76E5.0	fFourCorner	77107
U+76E5.0	fHakka	 gon5;  kwan5; kuan1; gwon5
U+76E5.0	fCantonese	gun3; fun2 (also); gwun3 (also); gun2
U+76E5.0	fMandarin	guan4; guan3
U+76E5.0	fEnglish	wash
U+76E6.0	fUTF8	盦
U+76E6.0	fCNS11643	2-5861
U+76E6.0	fBig5	EA75
U+76E6.0	fGB	3-5B44
U+76E6.0	fR/S	 108.11
U+76E6.0	fTotalStrokes	 16
U+76E6.0	fCangjie	OINT
U+76E6.0	fFourCorner	80107
U+76E6.0	fHakka	 am1; am3
U+76E6.0	fCantonese	am1; ngam1 (also)
U+76E6.0	fMandarin	an1; e4; an3
U+76E6.0	fEnglish	 cover of a dish or tripod
U+76E7.0	fUTF8	盧
U+76E7.0	fCNS11643	1-714D
U+76E7.0	fBig5	BF63
U+76E7.0	fGB	1-422C
U+76E7.0	fR/S	 108.11
U+76E7.0	fTotalStrokes	 16
U+76E7.0	fCangjie	YPWBT
U+76E7.0	fFourCorner	21217
U+76E7.0	fHakka	  lu2; lu3
U+76E7.0	fCantonese	lou4; lou2 (also); leoi4
U+76E7.0	fMandarin	lu2; luu2; lei2
U+76E7.0	fEnglish	 cottage; hut a Chinese family name black
U+76E8.0	fUTF8	盨
U+76E8.0	fCNS11643	4-572D
U+76E8.0	fGB	3-5B41
U+76E8.0	fR/S	  108.12
U+76E8.0	fTotalStrokes	 17; 16
U+76E8.0	fCangjie	HCBT
U+76E8.0	fFourCorner	21100
U+76E8.0	fCantonese	seoi5; so2 (also); seoi2 (also)
U+76E8.0	fMandarin	shu3; suo3; xu3
U+76E9.0	fUTF8	盩
U+76E9.0	fCNS11643	2-5D65
U+76E9.0	fBig5	ED79
U+76E9.0	fGB	3-5B45
U+76E9.0	fR/S	 108.12
U+76E9.0	fTotalStrokes	 17
U+76E9.0	fCangjie	GKBT
U+76E9.0	fFourCorner	48107
U+76E9.0	fCantonese	zau1
U+76E9.0	fMandarin	zhou1; zhou4; chou1; chou2
U+76E9.0	fEnglish	callus
U+76EA.0	fUTF8	盪
U+76EA.0	fCNS11643	1-7445
U+76EA.0	fBig5	C0FA
U+76EA.0	fGB	8-2D76
U+76EA.0	fR/S	 108.12
U+76EA.0	fTotalStrokes	 17
U+76EA.0	fCangjie	EHBT
U+76EA.0	fFourCorner	36107
U+76EA.0	fHakka	 tong5; tong3; tong6
U+76EA.0	fCantonese	dong6; tong3
U+76EA.0	fMandarin	dang4; tang2; tang1; tang4
U+76EA.0	fEnglish	 toss about swing; rock
U+76EB.0	fUTF8	盫
U+76EB.0	fCNS11643	3-5835
U+76EB.0	fGB	5-5A69
U+76EB.0	fR/S	  108.13
U+76EB.0	fTotalStrokes	 18
U+76EB.0	fCangjie	ORMWT
U+76EB.0	fFourCorner	80107
U+76EB.0	fCantonese	am1; ngam1 (also)
U+76EB.0	fMandarin	an1
U+76EC.0	fUTF8	盬
U+76EC.0	fCNS11643	2-6240
U+76EC.0	fBig5	F053
U+76EC.0	fGB	3-5B46
U+76EC.0	fR/S	 108.13
U+76EC.0	fTotalStrokes	 18
U+76EC.0	fCangjie	SRBT
U+76EC.0	fFourCorner	78107
U+76EC.0	fHakka	  gu3
U+76EC.0	fCantonese	gu2; gwu2 (also)
U+76EC.0	fMandarin	gu3; gu4; gu1
U+76EC.0	fEnglish	 salt pit leisure pot drink
U+76ED.0	fUTF8	盭
U+76ED.0	fCNS11643	2-6942
U+76ED.0	fBig5	F473
U+76ED.0	fGB	5-5A6A
U+76ED.0	fR/S	 108.15
U+76ED.0	fTotalStrokes	 20
U+76ED.0	fCangjie	VKBT
U+76ED.0	fFourCorner	28107
U+76ED.0	fHakka	 lui65; lui55; lui5; li5
U+76ED.0	fCantonese	leoi6; lai6 (also)
U+76ED.0	fMandarin	li4; lie4
U+76ED.0	fEnglish	cruel
U+76EE.0	fUTF8	目
U+76EE.0	fCNS11643	1-4678
U+76EE.0	fBig5	A5D8
U+76EE.0	fGB	0-443F
U+76EE.0	fR/S	 109.0
U+76EE.0	fTotalStrokes	 5
U+76EE.0	fCangjie	BU
U+76EE.0	fFourCorner	60101
U+76EE.0	fHakka	  muk7;  muk8
U+76EE.0	fCantonese	muk6
U+76EE.0	fMandarin	mu4
U+76EE.0	fEnglish	[1] eye [2] look; regard; see [3] table of contents; category
U+76EF.0	fUTF8	盯
U+76EF.0	fCNS11643	1-4B31
U+76EF.0	fBig5	A86E
U+76EF.0	fGB	0-3622
U+76EF.0	fR/S	 109.2
U+76EF.0	fTotalStrokes	 7
U+76EF.0	fCangjie	BUMN
U+76EF.0	fFourCorner	61020
U+76EF.0	fHakka	 dang1; den1; cin2
U+76EF.0	fCantonese	ding1; deng1 (also)
U+76EF.0	fMandarin	cheng2; ding1; cheng1; zheng3; zheng4
U+76EF.0	fEnglish	 stare at; gaze at; fix one's eyes on; keep a close watch
U+76F0.0	fUTF8	盰
U+76F0.0	fCNS11643	2-283A
U+76F0.0	fBig5	CD78
U+76F0.0	fGB	5-5874
U+76F0.0	fR/S	 109.3
U+76F0.0	fTotalStrokes	 8
U+76F0.0	fCangjie	BUMJ
U+76F0.0	fFourCorner	61040
U+76F0.0	fHakka	 gon6; gon5; gon3
U+76F0.0	fCantonese	gon3; gon2
U+76F0.0	fMandarin	gan3; gan4
U+76F0.0	fEnglish	evening; sunset; dusk
U+76F1.0	fUTF8	盱
U+76F1.0	fCNS11643	2-2839
U+76F1.0	fBig5	CD77
U+76F1.0	fGB	0-6D6C
U+76F1.0	fR/S	 109.3
U+76F1.0	fTotalStrokes	 8
U+76F1.0	fCangjie	BUMD
U+76F1.0	fFourCorner	61040
U+76F1.0	fHakka	  hi1; vu1
U+76F1.0	fCantonese	heoi1
U+76F1.0	fMandarin	xu1; yu2; hu1
U+76F1.0	fEnglish	[1] open the eyes wide; gaze in astonishment [2] worried; uneasy; anxious
U+76F2.0	fUTF8	盲
U+76F2.0	fCNS11643	1-4E7D
U+76F2.0	fBig5	AABC
U+76F2.0	fGB	0-4324
U+76F2.0	fR/S	 109.3
U+76F2.0	fTotalStrokes	 8
U+76F2.0	fCangjie	YVBU
U+76F2.0	fFourCorner	00601
U+76F2.0	fHakka	  mang2;  miang1; {mong2}; mo1; mang3
U+76F2.0	fCantonese	maang4; maang1 (also); maang2 (also)
U+76F2.0	fMandarin	mang2; wang4
U+76F2.0	fEnglish	[1] blind; blind [2] deluded
U+76F3.0	fUTF8	盳
U+76F3.0	fCNS11643	2-2838
U+76F3.0	fBig5	CD76
U+76F3.0	fGB	5-5877
U+76F3.0	fR/S	 109.3
U+76F3.0	fTotalStrokes	 8
U+76F3.0	fCangjie	BUYV
U+76F3.0	fFourCorner	60010
U+76F3.0	fCantonese	maang4; mong6
U+76F3.0	fMandarin	mang2; wang4; pang2
U+76F4.0	fUTF8	直
U+76F4.0	fCNS11643	1-4E7E
U+76F4.0	fBig5	AABD
U+76F4.0	fGB	0-5631
U+76F4.0	fR/S	 109.3
U+76F4.0	fTotalStrokes	 8
U+76F4.0	fCangjie	JBMM
U+76F4.0	fFourCorner	40716
U+76F4.0	fHakka	  chit8; cit8; ts'iit8; ts'ik8
U+76F4.0	fCantonese	zik6; zik2 (also)
U+76F4.0	fMandarin	zhi2; zhi4
U+76F4.0	fEnglish	[1] straight; straighten [2] upright and honest; fair; unbiased [3] vertical; longitudinal; from bottom to bottom [4] outspoken; frank; straightforward [5] directly; firsthand [6] continuous; uninterrupted [7] stiff; numb [8] just; simply; only; merely [9] vertical stroke
U+76F5.0	fUTF8	盵
U+76F5.0	fCNS11643	2-283B
U+76F5.0	fBig5	CD79
U+76F5.0	fGB	3-5922
U+76F5.0	fR/S	 109.3
U+76F5.0	fTotalStrokes	 8
U+76F5.0	fCangjie	BUON
U+76F5.0	fFourCorner	68017
U+76F5.0	fHakka	 hi5; hi6
U+76F5.0	fCantonese	hei3
U+76F5.0	fMandarin	qi4
U+76F6.0	fUTF8	盶
U+76F6.0	fCNS11643	4-2A55
U+76F6.0	fGB	3-592A
U+76F6.0	fR/S	  109.4
U+76F6.0	fTotalStrokes	 9
U+76F6.0	fCangjie	BUMMU
U+76F6.0	fFourCorner	61010
U+76F6.0	fCantonese	jyun5
U+76F6.0	fMandarin	ruan3; yuan3
U+76F7.0	fUTF8	盷
U+76F7.0	fCNS11643	2-2C47
U+76F7.0	fBig5	CFE5
U+76F7.0	fGB	3-5938
U+76F7.0	fR/S	 109.4
U+76F7.0	fTotalStrokes	 9
U+76F7.0	fCangjie	BUPIM
U+76F7.0	fFourCorner	67020
U+76F7.0	fCantonese	jyun4; tin4
U+76F7.0	fMandarin	tian2; xuan2; min2; xian2
U+76F8.0	fUTF8	相
U+76F8.0	fCNS11643	1-525E
U+76F8.0	fBig5	ACDB
U+76F8.0	fGB	0-4F60
U+76F8.0	fR/S	 109.4
U+76F8.0	fTotalStrokes	 9
U+76F8.0	fCangjie	DBU
U+76F8.0	fFourCorner	46900
U+76F8.0	fCantonese	soeng2 (also)
U+76F8.0	fMandarin	xiang1; rang2
U+76F8.0	fEnglish	[1] each other; one another; mutually; reciprocal [2] substance
U+76F8.1	fUTF8	相
U+76F8.1	fCNS11643	1-525E
U+76F8.1	fBig5	ACDB
U+76F8.1	fGB	0-4F60
U+76F8.1	fR/S	 109.4
U+76F8.1	fTotalStrokes	 9
U+76F8.1	fCangjie	DBU
U+76F8.1	fFourCorner	46900
U+76F8.1	fHakka	  siong1;  siong5
U+76F8.1	fCantonese	soeng1; soeng3
U+76F8.1	fMandarin	xiang4
U+76F8.1	fEnglish	[1] examine; study; read; [2] countenance; appearance; facial features; looks; bearing; posture [3] prime minister (in feudal times) [4] assist; help
U+76F9.0	fUTF8	盹
U+76F9.0	fCNS11643	1-525D
U+76F9.0	fBig5	ACDA
U+76F9.0	fGB	0-6D6F
U+76F9.0	fR/S	 109.4
U+76F9.0	fTotalStrokes	 9
U+76F9.0	fCangjie	BUPU
U+76F9.0	fFourCorner	65017
U+76F9.0	fHakka	 dun1; zun1; zhun1; dun3
U+76F9.0	fCantonese	deon6; zeon3
U+76F9.0	fMandarin	dun3; dun4; zhun1
U+76F9.0	fEnglish	 doze; nod doze; nap; nod
U+76FA.0	fUTF8	盺
U+76FA.0	fCNS11643	2-2C49
U+76FA.0	fBig5	CFE7
U+76FA.0	fGB	3-5932
U+76FA.0	fR/S	 109.4
U+76FA.0	fTotalStrokes	 9
U+76FA.0	fCangjie	BUHML
U+76FA.0	fFourCorner	62021
U+76FA.0	fHakka	 hiun1
U+76FA.0	fCantonese	jan1
U+76FA.0	fMandarin	xin1
U+76FA.0	fEnglish	[1] indistinct vision [2] fixed look [3] joyful
U+76FB.0	fUTF8	盻
U+76FB.0	fCNS11643	2-2C48
U+76FB.0	fBig5	CFE6
U+76FB.0	fGB	3-5934
U+76FB.0	fR/S	 109.4
U+76FB.0	fTotalStrokes	 9
U+76FB.0	fCangjie	BUCMS
U+76FB.0	fFourCorner	68027
U+76FB.0	fHakka	 hi5; he5
U+76FB.0	fCantonese	hai6; paan3
U+76FB.0	fMandarin	xi4; xi1; yi3; yi4
U+76FB.0	fEnglish	 look in anger
U+76FC.0	fUTF8	盼
U+76FC.0	fCNS11643	1-5262
U+76FC.0	fBig5	ACDF
U+76FC.0	fGB	0-454E
U+76FC.0	fR/S	 109.4
U+76FC.0	fTotalStrokes	 9
U+76FC.0	fCangjie	BUCSH
U+76FC.0	fFourCorner	68027
U+76FC.0	fHakka	 pan3; pan5
U+76FC.0	fCantonese	paan3
U+76FC.0	fMandarin	pan4; fen2; pan1
U+76FC.0	fEnglish	[1] look; [2] (descriptive of the black and white of the eyes) well defined [3] hope; look for; expect
U+76FD.0	fUTF8	盽
U+76FD.0	fCNS11643	3-2E35
U+76FD.0	fGB	E-3461
U+76FD.0	fR/S	 109.4
U+76FD.0	fTotalStrokes	 9
U+76FD.0	fCangjie	BUQJ
U+76FD.0	fFourCorner	65000
U+76FD.0	fCantonese	fung1
U+76FD.0	fMandarin	feng1
U+76FE.0	fUTF8	盾
U+76FE.0	fCNS11643	1-5261
U+76FE.0	fBig5	ACDE
U+76FE.0	fGB	0-365C
U+76FE.0	fR/S	 109.4
U+76FE.0	fTotalStrokes	 9
U+76FE.0	fCangjie	HJBU
U+76FE.0	fFourCorner	72264
U+76FE.0	fHakka	 tun3; dun3; tun6; tun5
U+76FE.0	fCantonese	teon5; seon5
U+76FE.0	fMandarin	dun4; shun3; yun3
U+76FE.0	fEnglish	[1] shield; buckler [2] guilder; former monetary unit in the Netherlands
U+76FF.0	fUTF8	盿
U+76FF.0	fCNS11643	3-2E33
U+76FF.0	fGB	E-3462
U+76FF.0	fR/S	  109.4
U+76FF.0	fTotalStrokes	 9
U+76FF.0	fCangjie	BUYK
U+76FF.0	fFourCorner	60040
U+76FF.0	fCantonese	man4
U+76FF.0	fMandarin	min2
U+7700.0	fUTF8	眀
U+7700.0	fCNS11643	4-2A59
U+7700.0	fGB	5-587C
U+7700.0	fR/S	  109.4
U+7700.0	fTotalStrokes	 9
U+7700.0	fCangjie	BUB
U+7700.0	fFourCorner	67020
U+7700.0	fCantonese	ming4
U+7700.0	fMandarin	ming2
U+7701.0	fUTF8	省
U+7701.0	fCNS11643	1-525C
U+7701.0	fBig5	ACD9
U+7701.0	fGB	0-4A21
U+7701.0	fR/S	 109.4
U+7701.0	fTotalStrokes	 9
U+7701.0	fCangjie	FHBU
U+7701.0	fFourCorner	90602
U+7701.0	fHakka	 sen1;  sem3;  sang3; sien3
U+7701.0	fCantonese	saang2; sing2
U+7701.0	fMandarin	sheng3; xian3
U+7701.0	fEnglish	[1] province; provincial [2] economical; frugal; to economise [3] save; omit; reduce; avoid; abridge ???
U+7701.1	fUTF8	省
U+7701.1	fCNS11643	1-525C
U+7701.1	fBig5	ACD9
U+7701.1	fGB	0-4A21
U+7701.1	fR/S	 109.4
U+7701.1	fTotalStrokes	 9
U+7701.1	fCangjie	FHBU
U+7701.1	fFourCorner	90602
U+7701.1	fHakka	 sen3
U+7701.1	fMandarin	xing3
U+7701.1	fEnglish	[1] examine (oneself, etc.); reflect; introspect; consider [2] understand; know [3] visit (one's seniors, etc.) [4] test; examination; [5] memory ???
U+7702.0	fUTF8	眂
U+7702.0	fCNS11643	4-2A5D
U+7702.0	fGB	5-587D
U+7702.0	fR/S	 109.4
U+7702.0	fTotalStrokes	 9
U+7702.0	fCangjie	BUHVP
U+7702.0	fFourCorner	62040
U+7702.0	fHakka	 shi6; sii5; shi5; she5; shi3
U+7702.0	fCantonese	si6
U+7702.0	fMandarin	shi4
U+7702.0	fEnglish	 see; look at; regard
U+7703.0	fUTF8	眃
U+7703.0	fCNS11643	2-2C43
U+7703.0	fBig5	CFE1
U+7703.0	fGB	3-592C
U+7703.0	fR/S	 109.4
U+7703.0	fTotalStrokes	 9
U+7703.0	fCangjie	BUMMI
U+7703.0	fFourCorner	61031
U+7703.0	fCantonese	wan4; wan6
U+7703.0	fMandarin	yun2; hun4
U+7704.0	fUTF8	眄
U+7704.0	fCNS11643	2-2C44
U+7704.0	fBig5	CFE2
U+7704.0	fGB	0-6D6D
U+7704.0	fR/S	 109.4
U+7704.0	fTotalStrokes	 9
U+7704.0	fCangjie	BUMLS
U+7704.0	fFourCorner	61027
U+7704.0	fHakka	 men3
U+7704.0	fCantonese	min5; min6
U+7704.0	fMandarin	mian3; mian4
U+7704.0	fEnglish	[1] look askance; ogle [2] look
U+7705.0	fUTF8	眅
U+7705.0	fCNS11643	2-2C45
U+7705.0	fBig5	CFE3
U+7705.0	fGB	3-5933
U+7705.0	fR/S	 109.4
U+7705.0	fTotalStrokes	 9
U+7705.0	fCangjie	BUHE
U+7705.0	fFourCorner	61047
U+7705.0	fHakka	  ban1
U+7705.0	fCantonese	paan1
U+7705.0	fMandarin	pan1; pan3; pan4
U+7705.0	fEnglish	 inlay border
U+7706.0	fUTF8	眆
U+7706.0	fCNS11643	4-2A52
U+7706.0	fGB	5-587E
U+7706.0	fR/S	  109.4
U+7706.0	fTotalStrokes	 9
U+7706.0	fCangjie	BUYHS
U+7706.0	fFourCorner	60020
U+7706.0	fHakka	  fong3
U+7706.0	fCantonese	fong2
U+7706.0	fMandarin	fang3
U+7706.0	fEnglish	indistinct
U+7707.0	fUTF8	眇
U+7707.0	fCNS11643	1-5263
U+7707.0	fBig5	ACE0
U+7707.0	fGB	0-6D70
U+7707.0	fR/S	 109.4
U+7707.0	fTotalStrokes	 9
U+7707.0	fCangjie	BUFH
U+7707.0	fFourCorner	69020
U+7707.0	fHakka	 miau3; miau1; meu3; miau2
U+7707.0	fCantonese	miu5
U+7707.0	fMandarin	miao3; miao4
U+7707.0	fEnglish	[1] tiny; fine; small; unimportant [2] blind in one eye
U+7708.0	fUTF8	眈
U+7708.0	fCNS11643	2-2C42
U+7708.0	fBig5	CFE0
U+7708.0	fGB	0-6D71
U+7708.0	fR/S	 109.4
U+7708.0	fTotalStrokes	 9
U+7708.0	fCangjie	BULBU
U+7708.0	fFourCorner	64012
U+7708.0	fHakka	  dam1; dang1
U+7708.0	fCantonese	daam1
U+7708.0	fMandarin	dan1; chen3; tan2; dan4
U+7708.0	fEnglish	 look downward
U+7709.0	fUTF8	眉
U+7709.0	fCNS11643	1-525F
U+7709.0	fBig5	ACDC
U+7709.0	fGB	0-433C
U+7709.0	fR/S	 109.4
U+7709.0	fTotalStrokes	 9
U+7709.0	fCangjie	AHBU
U+7709.0	fFourCorner	77267
U+7709.0	fHakka	  mi2; mi3
U+7709.0	fCantonese	mei4; mei2 (also)
U+7709.0	fMandarin	mei2
U+7709.0	fEnglish	[1] eyebrows [2] side [3] top margin of a printed page in a book [4] rare Chinese family name
U+770A.0	fUTF8	眊
U+770A.0	fCNS11643	2-2C46
U+770A.0	fBig5	CFE4
U+770A.0	fGB	3-5931
U+770A.0	fR/S	 109.4
U+770A.0	fTotalStrokes	 9
U+770A.0	fCangjie	BUHQU
U+770A.0	fFourCorner	62014
U+770A.0	fHakka	 mau3;  mau5; mo1; mau6; mau1; mo5; mo6
U+770A.0	fCantonese	mou6
U+770A.0	fMandarin	mao4; mei4; mo4
U+770A.0	fEnglish	[1] dim-sighted; eyes not seeing very clearly; dull and mixed-up [2] same as
U+770B.0	fUTF8	看
U+770B.0	fCNS11643	1-5260
U+770B.0	fBig5	ACDD
U+770B.0	fGB	0-3F34
U+770B.0	fR/S	 109.4
U+770B.0	fTotalStrokes	 9
U+770B.0	fCangjie	HQBU
U+770B.0	fFourCorner	20604
U+770B.0	fHakka	  kon5; kon1; kan1
U+770B.0	fCantonese	hon3; hon1
U+770B.0	fMandarin	kan4
U+770B.0	fEnglish	[1] see; look at; observe; watch; look in the direction of; read [2] examine; consider; think [3] visit; call on [4] present (tea, wine, etc.) [5] depend on
U+770B.1	fUTF8	看
U+770B.1	fCNS11643	1-5260
U+770B.1	fBig5	ACDD
U+770B.1	fGB	0-3F34
U+770B.1	fR/S	 109.4
U+770B.1	fTotalStrokes	 9
U+770B.1	fCangjie	HQBU
U+770B.1	fFourCorner	20604
U+770B.1	fMandarin	kan1
U+770B.1	fEnglish	[1] watch; mind; look after [2] guard; keep under surveillance
U+770C.0	fUTF8	県
U+770C.0	fGB	E-3463
U+770C.0	fR/S	 109.4
U+770C.0	fTotalStrokes	 9
U+770C.0	fCangjie	BUVF
U+770C.0	fCantonese	jyun6
U+770C.0	fMandarin	xian4; xuan2
U+770C.0	fEnglish	county; district; subdivision
U+770D.0	fUTF8	眍
U+770D.0	fGB	0-6D6E
U+770D.0	fR/S	 109.4
U+770D.0	fTotalStrokes	 9
U+770D.0	fCangjie	BUSK
U+770D.0	fHakka	 eu2
U+770D.0	fCantonese	kau1
U+770D.0	fMandarin	kou1; ou1
U+770D.0	fEnglish	simplified form of &#x7798;
U+770E.0	fUTF8	眎
U+770E.0	fCNS11643	3-3264
U+770E.0	fGB	E-3464
U+770E.0	fR/S	 109.5
U+770E.0	fTotalStrokes	 10
U+770E.0	fCangjie	BUMMF
U+770E.0	fFourCorner	61090
U+770E.0	fHakka	 shi6; sii5; shi5; she5; shi3
U+770E.0	fCantonese	si6
U+770E.0	fMandarin	shi4
U+770E.0	fEnglish	look at; inspect; observe; see
U+770F.0	fUTF8	眏
U+770F.0	fCNS11643	3-325F
U+770F.0	fGB	3-5945
U+770F.0	fR/S	 109.5
U+770F.0	fTotalStrokes	 10
U+770F.0	fCangjie	BULBK
U+770F.0	fFourCorner	65030
U+770F.0	fCantonese	ong1; ngong1 (also)
U+770F.0	fMandarin	yang1
U+7710.0	fUTF8	眐
U+7710.0	fCNS11643	2-3132
U+7710.0	fBig5	D2CF
U+7710.0	fGB	3-593D
U+7710.0	fR/S	 109.5
U+7710.0	fTotalStrokes	 10
U+7710.0	fCangjie	BUMYM
U+7710.0	fFourCorner	61011
U+7710.0	fCantonese	zing1
U+7710.0	fMandarin	zheng1; ci2; zheng4
U+7711.0	fUTF8	眑
U+7711.0	fCNS11643	2-3136
U+7711.0	fBig5	D2D3
U+7711.0	fGB	3-5950
U+7711.0	fR/S	 109.5
U+7711.0	fTotalStrokes	 10
U+7711.0	fCangjie	BUVIS
U+7711.0	fFourCorner	64027
U+7711.0	fHakka	  miau3; meu3; jau3
U+7711.0	fCantonese	jiu2; miu5
U+7711.0	fMandarin	yao3; ao1; you3
U+7711.0	fEnglish	 sunken eyes deep abstruse
U+7712.0	fUTF8	眒
U+7712.0	fCNS11643	2-3134
U+7712.0	fBig5	D2D1
U+7712.0	fGB	3-5944
U+7712.0	fR/S	 109.5
U+7712.0	fTotalStrokes	 10
U+7712.0	fCangjie	BULWL
U+7712.0	fFourCorner	65006
U+7712.0	fCantonese	san1
U+7712.0	fMandarin	shen4; shen1
U+7713.0	fUTF8	眓
U+7713.0	fCNS11643	2-3133
U+7713.0	fBig5	D2D0
U+7713.0	fGB	3-5942
U+7713.0	fR/S	 109.5
U+7713.0	fTotalStrokes	 10
U+7713.0	fCangjie	BUIV
U+7713.0	fFourCorner	63050
U+7713.0	fCantonese	kut3; kwut3 (also)
U+7713.0	fMandarin	huo4; xue4
U+7714.0	fUTF8	眔
U+7714.0	fCNS11643	4-2E55
U+7714.0	fGB	3-5A7D
U+7714.0	fR/S	 109.5
U+7714.0	fTotalStrokes	 10
U+7714.0	fCangjie	WLFF
U+7714.0	fFourCorner	60230
U+7714.0	fCantonese	daap6
U+7714.0	fMandarin	da4
U+7715.0	fUTF8	眕
U+7715.0	fCNS11643	2-3137
U+7715.0	fBig5	D2D4
U+7715.0	fGB	3-5947
U+7715.0	fR/S	 109.5
U+7715.0	fTotalStrokes	 10
U+7715.0	fCangjie	BUOHH
U+7715.0	fFourCorner	68022
U+7715.0	fHakka	 zhin3; zihn3
U+7715.0	fCantonese	zan2; can2
U+7715.0	fMandarin	zhen3
U+7715.0	fEnglish	restraint
U+7716.0	fUTF8	眖
U+7716.0	fCNS11643	3-325E
U+7716.0	fGB	3-5946
U+7716.0	fR/S	  109.5
U+7716.0	fTotalStrokes	 10
U+7716.0	fCangjie	BURHU
U+7716.0	fFourCorner	66010
U+7716.0	fHakka	 kong3; k'wong3
U+7716.0	fCantonese	fong3
U+7716.0	fMandarin	kuang4
U+7716.0	fEnglish	 look at
U+7717.0	fUTF8	眗
U+7717.0	fCNS11643	4-2E4F
U+7717.0	fGB	3-5948
U+7717.0	fR/S	 109.5
U+7717.0	fTotalStrokes	 10
U+7717.0	fCangjie	BUPR
U+7717.0	fFourCorner	67020
U+7717.0	fCantonese	geoi1
U+7717.0	fMandarin	ju1
U+7718.0	fUTF8	眘
U+7718.0	fCNS11643	3-3268
U+7718.0	fGB	E-3465
U+7718.0	fR/S	  109.5
U+7718.0	fTotalStrokes	 10
U+7718.0	fCangjie	KCBU
U+7718.0	fFourCorner	40600
U+7718.0	fHakka	 shim3; siim3
U+7718.0	fCantonese	san6
U+7718.0	fMandarin	shen4
U+7718.0	fEnglish	 be careful; be cautious
U+7719.0	fUTF8	眙
U+7719.0	fCNS11643	2-3138
U+7719.0	fBig5	D2D5
U+7719.0	fGB	0-6D74
U+7719.0	fR/S	 109.5
U+7719.0	fTotalStrokes	 10
U+7719.0	fCangjie	BUIR
U+7719.0	fFourCorner	63060
U+7719.0	fHakka	 chi5; ji3; cih5; ri3
U+7719.0	fCantonese	ci3; ji4
U+7719.0	fMandarin	chi4; cheng4; deng4
U+7719.0	fEnglish	[1] stare; look in the face; [2] look in astonishment; look with the eyes wide open
U+7719.1	fUTF8	眙
U+7719.1	fCNS11643	2-3138
U+7719.1	fBig5	D2D5
U+7719.1	fGB	0-6D74
U+7719.1	fR/S	 109.5
U+7719.1	fTotalStrokes	 10
U+7719.1	fCangjie	BUIR
U+7719.1	fFourCorner	63060
U+7719.1	fMandarin	yi2
U+7719.1	fEnglish	used in combination
U+771A.0	fUTF8	眚
U+771A.0	fCNS11643	2-3139
U+771A.0	fBig5	D2D6
U+771A.0	fGB	0-6D72
U+771A.0	fR/S	 109.5
U+771A.0	fTotalStrokes	 10
U+771A.0	fCangjie	HMBU
U+771A.0	fFourCorner	25601
U+771A.0	fHakka	  sen3
U+771A.0	fCantonese	saang2
U+771A.0	fMandarin	sheng3
U+771A.0	fEnglish	[1] disease of the eyes; eye-ailment [2] fault; blunder; mistake; error [3] corruption [4] reduce; save (the trouble, etc.) [5] calamity; disaster
U+771B.0	fUTF8	眛
U+771B.0	fCNS11643	2-3131
U+771B.0	fBig5	D2CE
U+771B.0	fGB	3-593B
U+771B.0	fR/S	 109.5
U+771B.0	fTotalStrokes	 10
U+771B.0	fCangjie	BUJD
U+771B.0	fFourCorner	65090
U+771B.0	fHakka	 mui3; mi3; mi5; mui5
U+771B.0	fCantonese	mui6
U+771B.0	fMandarin	mei4
U+771B.0	fEnglish	dim-sighted; poor visioned
U+771C.0	fUTF8	眜
U+771C.0	fCNS11643	4-2E51
U+771C.0	fGB	3-593C
U+771C.0	fR/S	  109.5
U+771C.0	fTotalStrokes	 10
U+771C.0	fCangjie	BUDJ
U+771C.0	fFourCorner	65090
U+771C.0	fCantonese	mut6; mit6
U+771C.0	fMandarin	mo4; mie4
U+771D.0	fUTF8	眝
U+771D.0	fCNS11643	2-3130
U+771D.0	fBig5	D2CD
U+771D.0	fGB	5-5921
U+771D.0	fR/S	 109.5
U+771D.0	fTotalStrokes	 10
U+771D.0	fCangjie	BUJMN
U+771D.0	fFourCorner	63021
U+771D.0	fHakka	 du3
U+771D.0	fCantonese	cyu5; ning5; ling5 (also); dou2
U+771D.0	fMandarin	zhu4; zhu3
U+771D.0	fEnglish	 store up; hoard
U+771E.0	fUTF8	眞
U+771E.0	fCNS11643	3-3759
U+771E.0	fGB	E-3466
U+771E.0	fR/S	 109.5
U+771E.0	fTotalStrokes	 11; 10
U+771E.0	fCangjie	PBUC
U+771E.0	fFourCorner	21800
U+771E.0	fHakka	 zhin1; zihn1
U+771E.0	fCantonese	zan1
U+771E.0	fMandarin	zhen1
U+771E.0	fEnglish	[1] true; real; factual; genuine; actual; substantial; really; truly [2] the highest sincerity one is capable of [3] a Chinese family name
U+771F.0	fUTF8	真
U+771F.0	fCNS11643	1-5678
U+771F.0	fBig5	AF75
U+771F.0	fGB	0-5566
U+771F.0	fR/S	 109.5
U+771F.0	fTotalStrokes	 10
U+771F.0	fCangjie	JBMC
U+771F.0	fFourCorner	40801
U+771F.0	fHakka	 zin1; zhin1
U+771F.0	fCantonese	zan1
U+771F.0	fMandarin	zhen1
U+771F.0	fEnglish	[1] true; real; factual; genuine; actual; substantial; really; truly [2] the highest sincerity one is capable of [3] a Chinese family name
U+7720.0	fUTF8	眠
U+7720.0	fCNS11643	1-5679
U+7720.0	fBig5	AF76
U+7720.0	fGB	0-435F
U+7720.0	fR/S	 109.5
U+7720.0	fTotalStrokes	 10
U+7720.0	fCangjie	BURVP
U+7720.0	fFourCorner	67047
U+7720.0	fHakka	  min2;  min3; min1
U+7720.0	fCantonese	min4
U+7720.0	fMandarin	mian2; min3; mian3; mian4
U+7720.0	fEnglish	[1] sleep [2] hibernate; hibernation
U+7721.0	fUTF8	眡
U+7721.0	fCNS11643	3-3267
U+7721.0	fGB	E-3467
U+7721.0	fR/S	  109.5
U+7721.0	fTotalStrokes	 10
U+7721.0	fCangjie	BUHPM
U+7721.0	fFourCorner	62040
U+7721.0	fCantonese	si6; ci1
U+7721.0	fMandarin	di1
U+7721.0	fEnglish	blurred sight
U+7722.0	fUTF8	眢
U+7722.0	fCNS11643	2-313A
U+7722.0	fBig5	D2D7
U+7722.0	fGB	0-6D73
U+7722.0	fR/S	 109.5
U+7722.0	fTotalStrokes	 10
U+7722.0	fCangjie	NUBU
U+7722.0	fFourCorner	27601
U+7722.0	fHakka	 ren1; jen1; wan1; van1
U+7722.0	fCantonese	jyun1
U+7722.0	fMandarin	yuan1; wan1; wei1
U+7722.0	fEnglish	eyes without brightness
U+7723.0	fUTF8	眣
U+7723.0	fCNS11643	2-3135
U+7723.0	fBig5	D2D2
U+7723.0	fGB	5-5928
U+7723.0	fR/S	 109.5
U+7723.0	fTotalStrokes	 10
U+7723.0	fCangjie	BUHQO
U+7723.0	fFourCorner	65030
U+7723.0	fHakka	 diet7; det7
U+7723.0	fCantonese	dit6; seon3
U+7723.0	fMandarin	die2; chi4
U+7723.0	fEnglish	squinting eyes
U+7724.0	fUTF8	眤
U+7724.0	fCNS11643	3-3265
U+7724.0	fGB	E-3468
U+7724.0	fR/S	 109.5
U+7724.0	fTotalStrokes	 10
U+7724.0	fCangjie	BUSP
U+7724.0	fFourCorner	67010
U+7724.0	fHakka	  nit8; ne1
U+7724.0	fCantonese	nik1; lik1 (also)
U+7724.0	fMandarin	yi2
U+7724.0	fEnglish	near; familiar
U+7725.0	fUTF8	眥
U+7725.0	fCNS11643	2-3753
U+7725.0	fBig5	D6B0
U+7725.0	fGB	E-3469
U+7725.0	fR/S	 109.5
U+7725.0	fTotalStrokes	 10
U+7725.0	fCangjie	YPBU
U+7725.0	fFourCorner	21601
U+7725.0	fHakka	 zih1; zii1
U+7725.0	fCantonese	zi6; zaai6 (also); zai6
U+7725.0	fMandarin	zi4; chai2; ji4; zhai4
U+7725.0	fEnglish	eye sockets
U+7726.0	fUTF8	眦
U+7726.0	fCNS11643	3-3755
U+7726.0	fGB	0-6D76
U+7726.0	fR/S	 109.5; 109.6
U+7726.0	fTotalStrokes	 11; 10
U+7726.0	fCangjie	BUYMP
U+7726.0	fFourCorner	61010
U+7726.0	fHakka	 zii1
U+7726.0	fCantonese	zi6; zaai6 (also)
U+7726.0	fMandarin	zi4
U+7726.0	fEnglish	corner of the eyes; eyesockets
U+7727.0	fUTF8	眧
U+7727.0	fCNS11643	2-313B
U+7727.0	fBig5	D2D8
U+7727.0	fGB	5-592D
U+7727.0	fR/S	 109.5
U+7727.0	fTotalStrokes	 10
U+7727.0	fCangjie	BUSHR
U+7727.0	fFourCorner	67062
U+7727.0	fCantonese	ciu2; ciu1
U+7727.0	fMandarin	chao3
U+7728.0	fUTF8	眨
U+7728.0	fCNS11643	1-567A
U+7728.0	fBig5	AF77
U+7728.0	fGB	0-5523
U+7728.0	fR/S	 109.5; 109.4
U+7728.0	fTotalStrokes	 10
U+7728.0	fCangjie	BUHIO
U+7728.0	fFourCorner	62037
U+7728.0	fHakka	 ngiap7; sap7;  shap7; zam3
U+7728.0	fCantonese	zaap3; zaam2 (also)
U+7728.0	fMandarin	zha3
U+7728.0	fEnglish	 wink; blink
U+7729.0	fUTF8	眩
U+7729.0	fCNS11643	1-5677
U+7729.0	fBig5	AF74
U+7729.0	fGB	0-5123
U+7729.0	fR/S	 109.5
U+7729.0	fTotalStrokes	 10
U+7729.0	fCangjie	BUYVI
U+7729.0	fFourCorner	60032
U+7729.0	fHakka	  hien2; hen2; hen3; hien5
U+7729.0	fCantonese	jyun6; jyun4 (also); waan6
U+7729.0	fMandarin	xuan4; huan4; juan4; xuan2
U+7729.0	fEnglish	[1] confuse; dazzle [2] dizzy; giddy; confused vision
U+772A.0	fUTF8	眪
U+772A.0	fCNS11643	3-325C
U+772A.0	fGB	3-5940
U+772A.0	fR/S	  109.5
U+772A.0	fTotalStrokes	 10
U+772A.0	fCangjie	BUMOB
U+772A.0	fFourCorner	61020
U+772A.0	fCantonese	bing2
U+772A.0	fMandarin	bing3
U+772B.0	fUTF8	眫
U+772B.0	fCNS11643	3-3263
U+772B.0	fGB	E-346A
U+772B.0	fR/S	 109.5
U+772B.0	fTotalStrokes	 10
U+772B.0	fCangjie	BUFQ
U+772B.0	fFourCorner	69050
U+772B.0	fCantonese	mai5
U+772B.0	fMandarin	mi3
U+772C.0	fUTF8	眬
U+772C.0	fGB	8-7D4C
U+772C.0	fR/S	 109.5
U+772C.0	fTotalStrokes	 10
U+772C.0	fCangjie	BUIKP
U+772C.0	fCantonese	lung4
U+772C.0	fMandarin	long2
U+772C.0	fEnglish	faint, fuzzy, blurred
U+772D.0	fUTF8	眭
U+772D.0	fCNS11643	2-374D
U+772D.0	fBig5	D6AA
U+772D.0	fGB	0-6D75
U+772D.0	fR/S	 109.6
U+772D.0	fTotalStrokes	 11
U+772D.0	fCangjie	BUGG
U+772D.0	fFourCorner	64014
U+772D.0	fHakka	 gui5; kwui5
U+772D.0	fCantonese	seoi1; kwai4
U+772D.0	fMandarin	sui1; gui4; hui1; hui4; xie2; xi1; wei4
U+772E.0	fUTF8	眮
U+772E.0	fCNS11643	4-334F
U+772E.0	fGB	3-5955
U+772E.0	fR/S	  109.6
U+772E.0	fTotalStrokes	 11
U+772E.0	fCangjie	BUBMR
U+772E.0	fFourCorner	67020
U+772E.0	fCantonese	tung4
U+772E.0	fMandarin	dong4; tong2
U+772F.0	fUTF8	眯
U+772F.0	fCNS11643	2-374C
U+772F.0	fBig5	D6A9
U+772F.0	fGB	0-4350
U+772F.0	fR/S	 109.6
U+772F.0	fTotalStrokes	 11
U+772F.0	fCangjie	BUFD
U+772F.0	fFourCorner	69094
U+772F.0	fHakka	 mi3; mi2
U+772F.0	fCantonese	mai5; mei1; mai4 (also); mi1 (also); mei6
U+772F.0	fMandarin	mi3; mi1; mi2; mi4; mei3; mei4
U+772F.0	fEnglish	[1] a foreign body getting into the eye [2] close one's eyes into narrow slits
U+7730.0	fUTF8	眰
U+7730.0	fCNS11643	3-3753
U+7730.0	fGB	3-5954
U+7730.0	fR/S	  109.6
U+7730.0	fTotalStrokes	 11
U+7730.0	fCangjie	BUMIG
U+7730.0	fFourCorner	61010
U+7730.0	fCantonese	dit6
U+7730.0	fMandarin	die2
U+7731.0	fUTF8	眱
U+7731.0	fCNS11643	2-374E
U+7731.0	fBig5	D6AB
U+7731.0	fGB	3-5953
U+7731.0	fR/S	 109.6
U+7731.0	fTotalStrokes	 11
U+7731.0	fCangjie	BUKN
U+7731.0	fFourCorner	65032
U+7731.0	fCantonese	ji4
U+7731.0	fMandarin	yi2; ti2
U+7732.0	fUTF8	眲
U+7732.0	fCNS11643	2-374F
U+7732.0	fBig5	D6AC
U+7732.0	fGB	3-5951
U+7732.0	fR/S	 109.6
U+7732.0	fTotalStrokes	 11
U+7732.0	fCangjie	BUSJ
U+7732.0	fFourCorner	61040
U+7732.0	fCantonese	nei6; ngak1; ngaak1 (also); lei6 (also); ak1 (also); aak1 (also); nik6
U+7732.0	fMandarin	er4; zhi4; ne4
U+7733.0	fUTF8	眳
U+7733.0	fCNS11643	2-3751
U+7733.0	fBig5	D6AE
U+7733.0	fGB	3-595C
U+7733.0	fR/S	 109.6
U+7733.0	fTotalStrokes	 11
U+7733.0	fCangjie	BUNIR
U+7733.0	fFourCorner	67062
U+7733.0	fCantonese	ming4
U+7733.0	fMandarin	ming3; ming2
U+7734.0	fUTF8	眴
U+7734.0	fCNS11643	2-3750
U+7734.0	fBig5	D6AD
U+7734.0	fGB	3-595B
U+7734.0	fR/S	 109.6
U+7734.0	fTotalStrokes	 11
U+7734.0	fCangjie	BUPA
U+7734.0	fFourCorner	67020
U+7734.0	fHakka	 shun5; sun5; jen5; ren5; hien5
U+7734.0	fCantonese	seon1; hyun3 (also); jyun6; seon3
U+7734.0	fMandarin	xuan4; shun4; run2; xun1
U+7734.0	fEnglish	dazzled
U+7735.0	fUTF8	眵
U+7735.0	fCNS11643	2-3755
U+7735.0	fBig5	D6B2
U+7735.0	fGB	0-6D77
U+7735.0	fR/S	 109.6
U+7735.0	fTotalStrokes	 11
U+7735.0	fCangjie	BUNIN
U+7735.0	fFourCorner	67027
U+7735.0	fHakka	 chi1; cih1; ci1; zi1; chii1
U+7735.0	fCantonese	ci1
U+7735.0	fMandarin	chi1; zhi1
U+7735.0	fEnglish	caking of eye secretions; secretions
U+7736.0	fUTF8	眶
U+7736.0	fCNS11643	1-5C3A
U+7736.0	fBig5	B2B5
U+7736.0	fGB	0-3F74
U+7736.0	fR/S	 109.6
U+7736.0	fTotalStrokes	 11
U+7736.0	fCangjie	BUSMG
U+7736.0	fFourCorner	61011
U+7736.0	fHakka	  kiong1
U+7736.0	fCantonese	kwaang1; hong1 (also)
U+7736.0	fMandarin	kuang4; kuang1
U+7736.0	fEnglish	 socket of the eye; rim of the eye
U+7737.0	fUTF8	眷
U+7737.0	fCNS11643	1-5C37
U+7737.0	fBig5	B2B2
U+7737.0	fGB	0-3E6C
U+7737.0	fR/S	 109.6
U+7737.0	fTotalStrokes	 11
U+7737.0	fCangjie	FQBU
U+7737.0	fFourCorner	90603
U+7737.0	fHakka	  gien5; gen5; kian5; ken5; gian5
U+7737.0	fCantonese	gyun3
U+7737.0	fMandarin	juan4
U+7737.0	fEnglish	[1] look back; regard; care for; concern [2] admire; love [3] relatives; dependents
U+7738.0	fUTF8	眸
U+7738.0	fCNS11643	1-5C3B
U+7738.0	fBig5	B2B6
U+7738.0	fGB	0-6D78
U+7738.0	fR/S	 109.6
U+7738.0	fTotalStrokes	 11
U+7738.0	fCangjie	BUIHQ
U+7738.0	fFourCorner	63050
U+7738.0	fHakka	  meu2; miu2
U+7738.0	fCantonese	mau4
U+7738.0	fMandarin	mou2
U+7738.0	fEnglish	 pupil of the eye; eyes
U+7739.0	fUTF8	眹
U+7739.0	fCNS11643	2-374B
U+7739.0	fBig5	D6A8
U+7739.0	fGB	3-5961
U+7739.0	fR/S	 109.6
U+7739.0	fTotalStrokes	 11
U+7739.0	fCangjie	BUTK
U+7739.0	fFourCorner	68035
U+7739.0	fHakka	 zhin3; zhim3; zihn3; zihm3; cihn3; chin3; chim3; cihm3
U+7739.0	fCantonese	zan2; zam6; zan6
U+7739.0	fMandarin	zhen4; jie1
U+7739.0	fEnglish	pupil
U+773A.0	fUTF8	眺
U+773A.0	fCNS11643	1-5C3C
U+773A.0	fBig5	B2B7
U+773A.0	fGB	0-4C77
U+773A.0	fR/S	 109.6
U+773A.0	fTotalStrokes	 11
U+773A.0	fCangjie	BULMO
U+773A.0	fFourCorner	62013
U+773A.0	fHakka	 diau5; tiau5
U+773A.0	fCantonese	tiu3
U+773A.0	fMandarin	tiao4; tiao3
U+773A.0	fEnglish	 look far away; take a look at faraway things; look far into the distance
U+773B.0	fUTF8	眻
U+773B.0	fCNS11643	2-3754
U+773B.0	fBig5	D6B1
U+773B.0	fGB	5-5936
U+773B.0	fR/S	 109.6
U+773B.0	fTotalStrokes	 11
U+773B.0	fCangjie	BUTQ
U+773B.0	fFourCorner	68051
U+773B.0	fCantonese	joeng4
U+773B.0	fMandarin	yang2; yang4
U+773C.0	fUTF8	眼
U+773C.0	fCNS11643	1-5C39
U+773C.0	fBig5	B2B4
U+773C.0	fGB	0-515B
U+773C.0	fR/S	 109.6
U+773C.0	fTotalStrokes	 11
U+773C.0	fCangjie	BUAV
U+773C.0	fFourCorner	67032
U+773C.0	fHakka	  ngan3;  ngien3; ngian3
U+773C.0	fCantonese	ngaan5; aan5 (also); ngaan2 (also); aan2 (also)
U+773C.0	fMandarin	yan3; en3; wen3
U+773C.0	fEnglish	[1] eye [2] look; glance [3] tiny hole; opening; orifice; aperture; [4] key point
U+773D.0	fUTF8	眽
U+773D.0	fCNS11643	2-3752
U+773D.0	fBig5	D6AF
U+773D.0	fGB	3-5959
U+773D.0	fR/S	 109.6
U+773D.0	fTotalStrokes	 11
U+773D.0	fCangjie	BUHHV
U+773D.0	fFourCorner	62032
U+773D.0	fHakka	  mak7; mak8
U+773D.0	fCantonese	mak6; maak6
U+773D.0	fMandarin	mo4; mai4; mi4
U+773D.0	fEnglish	 gaze ogle; look at
U+773E.0	fUTF8	眾
U+773E.0	fCNS11643	1-5C38
U+773E.0	fBig5	B2B3
U+773E.0	fGB	E-346B
U+773E.0	fR/S	 109.6
U+773E.0	fTotalStrokes	 11
U+773E.0	fCangjie	WLOOO
U+773E.0	fFourCorner	27232
U+773E.0	fHakka	 zung5; zhung5
U+773E.0	fCantonese	zung3; zung1
U+773E.0	fMandarin	zhong4; zhong1
U+773E.0	fEnglish	[1] many; numerous [2] crowd; multitude; all; masses [3] public or popular (opinion, views, etc.)
U+773F.0	fUTF8	眿
U+773F.0	fCNS11643	3-3262
U+773F.0	fGB	E-346C
U+773F.0	fR/S	  109.5
U+773F.0	fTotalStrokes	 10
U+773F.0	fCangjie	BUINE
U+773F.0	fFourCorner	63030
U+773F.0	fHakka	 mak7
U+773F.0	fCantonese	mak6
U+773F.0	fMandarin	mai4
U+7740.0	fUTF8	着
U+7740.0	fCNS11643	3-3757
U+7740.0	fGB	0-5745
U+7740.0	fR/S	 109.6; 123.5; 109.7
U+7740.0	fTotalStrokes	 11
U+7740.0	fCangjie	TQBU
U+7740.0	fFourCorner	80600
U+7740.0	fHakka	  cok8;  ch'ok8; tiok8; do3; ziok7; zok7; chok7; zhok7; chok8
U+7740.0	fCantonese	zoek6; zoek3; zau1; zau2
U+7740.0	fMandarin	zhao2
U+7740.0	fEnglish	make move; take action
U+7740.1	fUTF8	着
U+7740.1	fCNS11643	3-3757
U+7740.1	fGB	0-5745
U+7740.1	fR/S	 109.6; 123.5; 109.7
U+7740.1	fTotalStrokes	 11
U+7740.1	fCangjie	TQBU
U+7740.1	fFourCorner	80600
U+7740.1	fMandarin	zhuo2
U+7740.2	fUTF8	着
U+7740.2	fCNS11643	3-3757
U+7740.2	fGB	0-5745
U+7740.2	fR/S	 109.6; 123.5; 109.7
U+7740.2	fTotalStrokes	 11
U+7740.2	fCangjie	TQBU
U+7740.2	fFourCorner	80600
U+7740.2	fMandarin	zhao1
U+7740.3	fUTF8	着
U+7740.3	fCNS11643	3-3757
U+7740.3	fGB	0-5745
U+7740.3	fR/S	 109.6; 123.5; 109.7
U+7740.3	fTotalStrokes	 11
U+7740.3	fCangjie	TQBU
U+7740.3	fFourCorner	80600
U+7740.3	fMandarin	zhe5
U+7741.0	fUTF8	睁
U+7741.0	fCNS11643	4-3356
U+7741.0	fGB	0-5576
U+7741.0	fR/S	 109.6
U+7741.0	fTotalStrokes	 11
U+7741.0	fCangjie	BUNSD
U+7741.0	fFourCorner	67050
U+7741.0	fHakka	 zang1
U+7741.0	fCantonese	zang1; zaang1 (also)
U+7741.0	fMandarin	zheng1; jing3
U+7741.0	fEnglish	simplified form of &#x775C;
U+7742.0	fUTF8	睂
U+7742.0	fCNS11643	3-3D56
U+7742.0	fGB	E-346D
U+7742.0	fR/S	 109.7
U+7742.0	fTotalStrokes	 12
U+7742.0	fCangjie	OLMBU
U+7742.0	fFourCorner	82600
U+7742.0	fHakka	 mi2
U+7742.0	fCantonese	mei4
U+7742.0	fMandarin	mei2
U+7742.0	fEnglish	[1] eyebrows [2] eyelids
U+7743.0	fUTF8	睃
U+7743.0	fCNS11643	3-3D53
U+7743.0	fGB	0-6D7C
U+7743.0	fR/S	 109.7
U+7743.0	fTotalStrokes	 12
U+7743.0	fCangjie	BUICE
U+7743.0	fFourCorner	63040
U+7743.0	fHakka	 zun5
U+7743.0	fCantonese	so1; zeon3 (also)
U+7743.0	fMandarin	jun4; suo1; juan1
U+7743.0	fEnglish	 look at carefully
U+7744.0	fUTF8	睄
U+7744.0	fCNS11643	2-3E40
U+7744.0	fBig5	DABC
U+7744.0	fGB	3-5968
U+7744.0	fR/S	 109.7
U+7744.0	fTotalStrokes	 12
U+7744.0	fCangjie	BUFB
U+7744.0	fFourCorner	69027
U+7744.0	fCantonese	ciu4; saau3; saau4
U+7744.0	fMandarin	shao4; qiao2
U+7744.0	fEnglish	to glance
U+7745.0	fUTF8	睅
U+7745.0	fCNS11643	2-3E42
U+7745.0	fBig5	DABE
U+7745.0	fGB	3-5969
U+7745.0	fR/S	 109.7
U+7745.0	fTotalStrokes	 12
U+7745.0	fCangjie	BUAMJ
U+7745.0	fFourCorner	66041
U+7745.0	fHakka	 hon3; gon3
U+7745.0	fCantonese	hon6; hon5 (also); wun5
U+7745.0	fMandarin	han4; huan4
U+7745.0	fEnglish	big-eyed
U+7746.0	fUTF8	睆
U+7746.0	fCNS11643	2-3E3E
U+7746.0	fBig5	DABA
U+7746.0	fGB	3-5978
U+7746.0	fR/S	 109.7
U+7746.0	fTotalStrokes	 12
U+7746.0	fCangjie	BUJMU
U+7746.0	fFourCorner	63011
U+7746.0	fHakka	 gon3; gwon3; fon5; fon6; fon3; kwan3; van3
U+7746.0	fCantonese	wun2; wun5
U+7746.0	fMandarin	huan3; wan3
U+7746.0	fEnglish	bright
U+7747.0	fUTF8	睇
U+7747.0	fCNS11643	2-3E3F
U+7747.0	fBig5	DABB
U+7747.0	fGB	0-6D7B
U+7747.0	fR/S	 109.7
U+7747.0	fTotalStrokes	 12
U+7747.0	fCangjie	BUCNH
U+7747.0	fFourCorner	68027
U+7747.0	fHakka	 ti2;  ti5; tai3; ti3; tai1; ti6
U+7747.0	fCantonese	tai2; dai6 (also)
U+7747.0	fMandarin	di4; ti1; ti2
U+7747.0	fEnglish	[1] take a casual look at [2] look sideways; look askance; cast a sidelong glance
U+7748.0	fUTF8	睈
U+7748.0	fCNS11643	3-3D51
U+7748.0	fGB	3-596A
U+7748.0	fR/S	  109.7
U+7748.0	fTotalStrokes	 12
U+7748.0	fCangjie	BURMG
U+7748.0	fFourCorner	66010
U+7748.0	fCantonese	cing2
U+7748.0	fMandarin	cheng3
U+7749.0	fUTF8	睉
U+7749.0	fCNS11643	3-3D52
U+7749.0	fGB	3-5972
U+7749.0	fR/S	 109.7
U+7749.0	fTotalStrokes	 12
U+7749.0	fCangjie	BUOOG
U+7749.0	fFourCorner	68010
U+7749.0	fHakka	 doi1
U+7749.0	fCantonese	co4
U+7749.0	fMandarin	cuo2; cuo1; zhuai4
U+7749.0	fEnglish	 squat
U+774A.0	fUTF8	睊
U+774A.0	fCNS11643	2-3E43
U+774A.0	fBig5	DABF
U+774A.0	fGB	3-596C
U+774A.0	fR/S	 109.7
U+774A.0	fTotalStrokes	 12
U+774A.0	fCangjie	BURB
U+774A.0	fFourCorner	66027
U+774A.0	fHakka	 gien1; gian1; gen1
U+774A.0	fCantonese	gyun3; gyun1
U+774A.0	fMandarin	juan4; juan1; yung4
U+774A.0	fEnglish	 see; look; gaze at askance
U+774B.0	fUTF8	睋
U+774B.0	fCNS11643	2-3E45
U+774B.0	fBig5	DAC1
U+774B.0	fGB	3-596E
U+774B.0	fR/S	 109.7
U+774B.0	fTotalStrokes	 12
U+774B.0	fCangjie	BUHQI
U+774B.0	fFourCorner	63050
U+774B.0	fCantonese	ngo4
U+774B.0	fMandarin	e2
U+774C.0	fUTF8	睌
U+774C.0	fCNS11643	2-3E46
U+774C.0	fBig5	DAC2
U+774C.0	fGB	3-5974
U+774C.0	fR/S	 109.7
U+774C.0	fTotalStrokes	 12
U+774C.0	fCangjie	BUNAU
U+774C.0	fFourCorner	67017
U+774C.0	fCantonese	maan5
U+774C.0	fMandarin	wan3
U+774D.0	fUTF8	睍
U+774D.0	fCNS11643	2-3E41
U+774D.0	fBig5	DABD
U+774D.0	fGB	3-5930
U+774D.0	fR/S	 109.7
U+774D.0	fTotalStrokes	 12
U+774D.0	fCangjie	BUBUU
U+774D.0	fFourCorner	66010
U+774D.0	fHakka	 hien5; hien6; sien5; sien6; hien3
U+774D.0	fCantonese	jin5 (also); jin2; jin6
U+774D.0	fMandarin	xian4
U+774D.0	fEnglish	 look at with fear over-cautious
U+774E.0	fUTF8	睎
U+774E.0	fCNS11643	2-3E44
U+774E.0	fBig5	DAC0
U+774E.0	fGB	3-5971
U+774E.0	fR/S	 109.7
U+774E.0	fTotalStrokes	 12
U+774E.0	fCangjie	BUKKB
U+774E.0	fFourCorner	64027
U+774E.0	fHakka	  hi1
U+774E.0	fCantonese	hei1
U+774E.0	fMandarin	xi1
U+774E.0	fEnglish	 long for gaze at
U+774F.0	fUTF8	睏
U+774F.0	fCNS11643	1-607A
U+774F.0	fBig5	B574
U+774F.0	fGB	1-784A
U+774F.0	fR/S	 109.7
U+774F.0	fTotalStrokes	 12
U+774F.0	fCangjie	BUWD
U+774F.0	fFourCorner	66000
U+774F.0	fHakka	 kun5; kun3; kwun3; kwun5
U+774F.0	fCantonese	kwan3
U+774F.0	fMandarin	kun4
U+774F.0	fEnglish	drowsy; sleepy
U+7750.0	fUTF8	睐
U+7750.0	fGB	0-6D79
U+7750.0	fR/S	 109.7
U+7750.0	fTotalStrokes	 12
U+7750.0	fCangjie	BUDT
U+7750.0	fHakka	 loi3
U+7750.0	fCantonese	loi6
U+7750.0	fMandarin	lai4; lai2
U+7750.0	fEnglish	simplified form of &#x775E;
U+7751.0	fUTF8	睑
U+7751.0	fGB	0-6D7A
U+7751.0	fR/S	 109.7
U+7751.0	fTotalStrokes	 12
U+7751.0	fCangjie	BUOMM
U+7751.0	fCantonese	gim2; lim4 (also)
U+7751.0	fMandarin	jian3
U+7751.0	fEnglish	simplified form of &#x77BC;
U+7752.0	fUTF8	睒
U+7752.0	fCNS11643	2-4541
U+7752.0	fBig5	DEDB
U+7752.0	fGB	3-5A24
U+7752.0	fR/S	 109.8
U+7752.0	fTotalStrokes	 13
U+7752.0	fCangjie	BUFF
U+7752.0	fFourCorner	69089
U+7752.0	fHakka	 sham3; sam3
U+7752.0	fCantonese	sim2; sip3 (also)
U+7752.0	fMandarin	shan3; shan4; tan4
U+7752.0	fEnglish	 glance at; peep glittering shine
U+7753.0	fUTF8	睓
U+7753.0	fCNS11643	3-4327
U+7753.0	fGB	5-5944
U+7753.0	fR/S	  109.8
U+7753.0	fTotalStrokes	 13
U+7753.0	fCangjie	BUTBC
U+7753.0	fFourCorner	65080
U+7753.0	fCantonese	tin5
U+7753.0	fMandarin	tian3
U+7754.0	fUTF8	睔
U+7754.0	fCNS11643	2-4546
U+7754.0	fBig5	DEE0
U+7754.0	fGB	3-5936
U+7754.0	fR/S	 109.8
U+7754.0	fTotalStrokes	 13
U+7754.0	fCangjie	BUOMB
U+7754.0	fFourCorner	68027
U+7754.0	fCantonese	gwan3
U+7754.0	fMandarin	hun3; guan4; gun3; gun4; huan2; lun3
U+7755.0	fUTF8	睕
U+7755.0	fCNS11643	2-453E
U+7755.0	fBig5	DED8
U+7755.0	fGB	3-5A25
U+7755.0	fR/S	 109.8
U+7755.0	fTotalStrokes	 13
U+7755.0	fCangjie	BUJNU
U+7755.0	fFourCorner	63012
U+7755.0	fHakka	 wan3
U+7755.0	fCantonese	jyun2; waan1; wun1 (also)
U+7755.0	fMandarin	wan3; wan1; wan4; wo4
U+7756.0	fUTF8	睖
U+7756.0	fCNS11643	2-4542
U+7756.0	fBig5	DEDC
U+7756.0	fGB	3-597A
U+7756.0	fR/S	 109.8
U+7756.0	fTotalStrokes	 13
U+7756.0	fCangjie	BUGCE
U+7756.0	fFourCorner	64047
U+7756.0	fHakka	 lin2
U+7756.0	fCantonese	ling6
U+7756.0	fMandarin	ling2; leng4; cheng1
U+7756.0	fEnglish	 stare stright ahead
U+7757.0	fUTF8	睗
U+7757.0	fCNS11643	4-3F29
U+7757.0	fGB	3-597E
U+7757.0	fR/S	 109.8
U+7757.0	fTotalStrokes	 13
U+7757.0	fCangjie	BUAPH
U+7757.0	fFourCorner	66020
U+7757.0	fCantonese	sik1
U+7757.0	fMandarin	shi4
U+7757.0	fEnglish	a flash
U+7758.0	fUTF8	睘
U+7758.0	fCNS11643	3-4328
U+7758.0	fGB	5-5A4D
U+7758.0	fR/S	  109.8
U+7758.0	fTotalStrokes	 13
U+7758.0	fCangjie	WLMRV
U+7758.0	fFourCorner	60730
U+7758.0	fHakka	  kiun2; sien2; sien1
U+7758.0	fCantonese	king4
U+7758.0	fMandarin	qiong2
U+7758.0	fEnglish	 round stare
U+7759.0	fUTF8	睙
U+7759.0	fCNS11643	2-4547
U+7759.0	fBig5	DEE1
U+7759.0	fGB	E-346E
U+7759.0	fR/S	 109.8
U+7759.0	fTotalStrokes	 13
U+7759.0	fCangjie	BUHSK
U+7759.0	fFourCorner	63034
U+7759.0	fCantonese	lit6
U+7759.0	fMandarin	lie4
U+775A.0	fUTF8	睚
U+775A.0	fCNS11643	2-4543
U+775A.0	fBig5	DEDD
U+775A.0	fGB	0-6D7D
U+775A.0	fR/S	 109.8
U+775A.0	fTotalStrokes	 13
U+775A.0	fCangjie	BUMGG
U+775A.0	fFourCorner	61014
U+775A.0	fHakka	 ngai2
U+775A.0	fCantonese	ngaai4; aai4 (also); ngaai6
U+775A.0	fMandarin	yai2; ya2; ya4; yai4
U+775A.0	fEnglish	[1] corner of eye [2] stare
U+775B.0	fUTF8	睛
U+775B.0	fCNS11643	1-6542
U+775B.0	fBig5	B7FA
U+775B.0	fGB	0-3E26
U+775B.0	fR/S	 109.8
U+775B.0	fTotalStrokes	 13
U+775B.0	fCangjie	BUQMB
U+775B.0	fFourCorner	65027
U+775B.0	fHakka	  zin1; ciang1; ziang1
U+775B.0	fCantonese	zing1
U+775B.0	fMandarin	jing1; jing3
U+775B.0	fEnglish	[1] pupil of the eye [2] eyes
U+775B.1	fUTF8	睛
U+775B.1	fCNS11643	1-6542
U+775B.1	fBig5	B7FA
U+775B.1	fGB	0-3E26
U+775B.1	fR/S	 109.8
U+775B.1	fTotalStrokes	 13
U+775B.1	fCangjie	BUQMB
U+775B.1	fFourCorner	65027
U+775B.1	fMandarin	jing5
U+775C.0	fUTF8	睜
U+775C.0	fCNS11643	1-654A
U+775C.0	fBig5	B843
U+775C.0	fGB	1-5576
U+775C.0	fR/S	 109.8
U+775C.0	fTotalStrokes	 13
U+775C.0	fCangjie	BUBSD
U+775C.0	fFourCorner	62057
U+775C.0	fHakka	 zang1;  zen1; nang5; cang2
U+775C.0	fCantonese	zang1; zaang1 (also)
U+775C.0	fMandarin	zheng1; jing3
U+775C.0	fEnglish	 open the eyes
U+775D.0	fUTF8	睝
U+775D.0	fCNS11643	3-4325
U+775D.0	fGB	E-346F
U+775D.0	fR/S	  109.8
U+775D.0	fTotalStrokes	 13
U+775D.0	fCangjie	HHBU
U+775D.0	fFourCorner	27600
U+775D.0	fCantonese	lei4
U+775D.0	fMandarin	li2
U+775E.0	fUTF8	睞
U+775E.0	fCNS11643	1-6545
U+775E.0	fBig5	B7FD
U+775E.0	fGB	1-6D79
U+775E.0	fR/S	 109.8
U+775E.0	fTotalStrokes	 13
U+775E.0	fCangjie	BUDOO
U+775E.0	fFourCorner	64098
U+775E.0	fHakka	 lai3; loi1; lai2; loi3; loi5
U+775E.0	fCantonese	loi6
U+775E.0	fMandarin	lai4; lai2
U+775E.0	fEnglish	[1] look at; glance [2] squint; cockeyed
U+775F.0	fUTF8	睟
U+775F.0	fCNS11643	2-453F
U+775F.0	fBig5	DED9
U+775F.0	fGB	3-5A23
U+775F.0	fR/S	 109.8
U+775F.0	fTotalStrokes	 13
U+775F.0	fCangjie	BUYOJ
U+775F.0	fFourCorner	60048
U+775F.0	fHakka	 cui5
U+775F.0	fCantonese	seoi6; seoi3
U+775F.0	fMandarin	sui4; zui4
U+775F.0	fEnglish	clear-eyed
U+7760.0	fUTF8	睠
U+7760.0	fCNS11643	2-4540
U+7760.0	fBig5	DEDA
U+7760.0	fGB	E-3470
U+7760.0	fR/S	 109.8
U+7760.0	fTotalStrokes	 13
U+7760.0	fCangjie	BUFQU
U+7760.0	fFourCorner	69012
U+7760.0	fHakka	 gien3; gien5; kien2; gen3; gian5
U+7760.0	fCantonese	gyun3
U+7760.0	fMandarin	juan4; quan2
U+7760.0	fEnglish	[1] look back [2] care for
U+7761.0	fUTF8	睡
U+7761.0	fCNS11643	1-6975
U+7761.0	fBig5	BACE
U+7761.0	fGB	0-4B2F
U+7761.0	fR/S	 109.8; 109.9
U+7761.0	fTotalStrokes	 13
U+7761.0	fCangjie	BUHJM
U+7761.0	fFourCorner	62014
U+7761.0	fHakka	 soi5; s'oi6; soi3; s'oi5; choe5
U+7761.0	fCantonese	seoi6
U+7761.0	fMandarin	shui4
U+7761.0	fEnglish	 sleep; rest with eyes closed
U+7762.0	fUTF8	睢
U+7762.0	fCNS11643	1-654D
U+7762.0	fBig5	B846
U+7762.0	fGB	0-6E21
U+7762.0	fR/S	 109.8
U+7762.0	fTotalStrokes	 13
U+7762.0	fCangjie	BUOG
U+7762.0	fFourCorner	60014
U+7762.0	fHakka	 cui1; zih1; zui1
U+7762.0	fCantonese	seoi1; fai1
U+7762.0	fMandarin	sui1; hui1; hui4
U+7762.0	fEnglish	[1] raise one's eyes; [2] freely or without thought of others; at random [3] a Chinese family name
U+7763.0	fUTF8	督
U+7763.0	fCNS11643	1-6546
U+7763.0	fBig5	B7FE
U+7763.0	fGB	0-363D
U+7763.0	fR/S	 109.8
U+7763.0	fTotalStrokes	 13
U+7763.0	fCangjie	YEBU
U+7763.0	fFourCorner	27604
U+7763.0	fHakka	  duk7; duk8
U+7763.0	fCantonese	duk1
U+7763.0	fMandarin	du1
U+7763.0	fEnglish	[1] oversee; superintend; supervise [2] reprove; censure [3] marshal; general [4] viceroy or governor-general
U+7764.0	fUTF8	睤
U+7764.0	fCNS11643	3-4326
U+7764.0	fGB	5-5943
U+7764.0	fR/S	  109.8
U+7764.0	fTotalStrokes	 13
U+7764.0	fCangjie	BUWML
U+7764.0	fFourCorner	66020
U+7764.0	fHakka	 pi5
U+7764.0	fCantonese	pai5
U+7764.0	fMandarin	pi4; bi4
U+7764.0	fEnglish	to look around, look askance
U+7765.0	fUTF8	睥
U+7765.0	fCNS11643	1-654B
U+7765.0	fBig5	B844
U+7765.0	fGB	0-6E22
U+7765.0	fR/S	 109.8
U+7765.0	fTotalStrokes	 13
U+7765.0	fCangjie	BUHWJ
U+7765.0	fFourCorner	66040
U+7765.0	fHakka	 pi3; pi5
U+7765.0	fCantonese	pai5; pei5 (also); pai6 (also); pai3
U+7765.0	fMandarin	bi4; pi4; pi3
U+7765.0	fEnglish	[1] look askance; scornful look [2] battlements atop city walls
U+7766.0	fUTF8	睦
U+7766.0	fCNS11643	1-6544
U+7766.0	fBig5	B7FC
U+7766.0	fGB	0-4440
U+7766.0	fR/S	 109.8
U+7766.0	fTotalStrokes	 13
U+7766.0	fCangjie	BUGCG
U+7766.0	fFourCorner	64014
U+7766.0	fHakka	  muk8
U+7766.0	fCantonese	muk6
U+7766.0	fMandarin	mu4
U+7766.0	fEnglish	[1] friendly; befriend; be on friendly terms [2] a Chinese family name
U+7767.0	fUTF8	睧
U+7767.0	fCNS11643	2-4545
U+7767.0	fBig5	DEDF
U+7767.0	fGB	5-5947
U+7767.0	fR/S	 109.8
U+7767.0	fTotalStrokes	 13
U+7767.0	fCangjie	BUHPA
U+7767.0	fFourCorner	62064
U+7767.0	fHakka	  fun1
U+7767.0	fCantonese	fan1
U+7767.0	fMandarin	hun1
U+7767.0	fEnglish	dimness of vision
U+7768.0	fUTF8	睨
U+7768.0	fCNS11643	1-654C
U+7768.0	fBig5	B845
U+7768.0	fGB	0-6D7E
U+7768.0	fR/S	 109.8
U+7768.0	fTotalStrokes	 13
U+7768.0	fCangjie	BUHXU
U+7768.0	fFourCorner	67017
U+7768.0	fHakka	 ngi5; ngi6; ngi3
U+7768.0	fCantonese	ngai6; ai6 (also)
U+7768.0	fMandarin	ni4; yi3; yi4
U+7768.0	fEnglish	[1] look askance; expression of disdain or arrogance [2] slant
U+7769.0	fUTF8	睩
U+7769.0	fCNS11643	2-4544
U+7769.0	fBig5	DEDE
U+7769.0	fGB	3-5A29
U+7769.0	fR/S	 109.8
U+7769.0	fTotalStrokes	 13
U+7769.0	fCangjie	BUVNE
U+7769.0	fFourCorner	67032
U+7769.0	fHakka	 luk8
U+7769.0	fCantonese	luk1; luk6 (also)
U+7769.0	fMandarin	lu4
U+776A.0	fUTF8	睪
U+776A.0	fCNS11643	1-6548
U+776A.0	fBig5	B841
U+776A.0	fGB	E-3471
U+776A.0	fR/S	 109.8
U+776A.0	fTotalStrokes	 13
U+776A.0	fCangjie	WLGTJ
U+776A.0	fFourCorner	26401
U+776A.0	fHakka	 jit8; gau1
U+776A.0	fCantonese	jik6; zaak6; gou1
U+776A.0	fMandarin	gao1; yi4; ze2; die4; hao4; du4
U+776A.0	fEnglish	[1] marsh [2] high; lofty [3] testicle; testis
U+776B.0	fUTF8	睫
U+776B.0	fCNS11643	1-6543
U+776B.0	fBig5	B7FB
U+776B.0	fGB	0-3D5E
U+776B.0	fR/S	 109.8
U+776B.0	fTotalStrokes	 13
U+776B.0	fCangjie	BUJLO
U+776B.0	fFourCorner	65081
U+776B.0	fHakka	  ciap8; ci5; ciap7
U+776B.0	fCantonese	zit6; coi2; zau2; jyu4; fan1; maat3; haa6; zip3; zit3
U+776B.0	fMandarin	jie2; she4; qie4
U+776B.0	fEnglish	[1] eyelashes [2] blink; wink
U+776C.0	fUTF8	睬
U+776C.0	fCNS11643	1-6549
U+776C.0	fBig5	B842
U+776C.0	fGB	0-3247
U+776C.0	fR/S	 109.8
U+776C.0	fTotalStrokes	 13
U+776C.0	fCangjie	BUBD
U+776C.0	fFourCorner	62094
U+776C.0	fHakka	  cai3; coi3; cioi3
U+776C.0	fCantonese	coi2
U+776C.0	fMandarin	cai3
U+776C.0	fEnglish	[1] look; watch [2] notice; pay attention to
U+776D.0	fUTF8	睭
U+776D.0	fCNS11643	2-4548
U+776D.0	fBig5	DEE2
U+776D.0	fGB	3-5A21
U+776D.0	fR/S	 109.8
U+776D.0	fTotalStrokes	 13
U+776D.0	fCangjie	BUBGR
U+776D.0	fFourCorner	67020
U+776D.0	fCantonese	zau2
U+776D.0	fMandarin	zhou3
U+776E.0	fUTF8	睮
U+776E.0	fCNS11643	2-4C2E
U+776E.0	fBig5	E2E6
U+776E.0	fGB	3-5A38
U+776E.0	fR/S	 109.9
U+776E.0	fTotalStrokes	 14
U+776E.0	fCangjie	BUOMN
U+776E.0	fFourCorner	68021
U+776E.0	fCantonese	jyu4
U+776E.0	fMandarin	yu2
U+776F.0	fUTF8	睯
U+776F.0	fCNS11643	2-4C30
U+776F.0	fBig5	E2E8
U+776F.0	fGB	3-5A3E
U+776F.0	fR/S	 109.9
U+776F.0	fTotalStrokes	 14
U+776F.0	fCangjie	RKBU
U+776F.0	fFourCorner	78604
U+776F.0	fHakka	 men3
U+776F.0	fCantonese	fan1; mau6
U+776F.0	fMandarin	hun1
U+776F.0	fEnglish	robust; vigorous
U+7770.0	fUTF8	睰
U+7770.0	fCNS11643	4-4542
U+7770.0	fGB	E-3472
U+7770.0	fR/S	  109.9
U+7770.0	fTotalStrokes	 14
U+7770.0	fCangjie	BUTKR
U+7770.0	fFourCorner	64060
U+7770.0	fMandarin	ma4
U+7771.0	fUTF8	睱
U+7771.0	fCNS11643	3-4839
U+7771.0	fGB	3-5A3C
U+7771.0	fR/S	 109.9
U+7771.0	fTotalStrokes	 14
U+7771.0	fCangjie	BURSE
U+7771.0	fFourCorner	67040
U+7771.0	fHakka	 ha5; ha6
U+7771.0	fMandarin	xia4; jia3
U+7771.0	fEnglish	 gaze slowly white appearance of the eyes
U+7772.0	fUTF8	睲
U+7772.0	fCNS11643	4-4540
U+7772.0	fGB	3-5A33
U+7772.0	fR/S	 109.9
U+7772.0	fTotalStrokes	 14
U+7772.0	fCangjie	BUAHM
U+7772.0	fFourCorner	66010
U+7772.0	fCantonese	sing2
U+7772.0	fMandarin	xing3
U+7773.0	fUTF8	睳
U+7773.0	fCNS11643	4-453D
U+7773.0	fGB	5-594F
U+7773.0	fR/S	 109.9
U+7773.0	fTotalStrokes	 14
U+7773.0	fCangjie	BUKGG
U+7773.0	fFourCorner	64010
U+7773.0	fCantonese	fui1
U+7773.0	fMandarin	xi1; hui1
U+7774.0	fUTF8	睴
U+7774.0	fCNS11643	3-4833
U+7774.0	fGB	3-5962
U+7774.0	fR/S	  109.9
U+7774.0	fTotalStrokes	 14
U+7774.0	fCangjie	BUBJJ
U+7774.0	fFourCorner	67050
U+7774.0	fCantonese	gwan3
U+7774.0	fMandarin	gun4
U+7775.0	fUTF8	睵
U+7775.0	fCNS11643	3-4836
U+7775.0	fGB	3-5A2E
U+7775.0	fR/S	  109.9
U+7775.0	fTotalStrokes	 14
U+7775.0	fCangjie	BUJIR
U+7775.0	fFourCorner	63050
U+7775.0	fCantonese	zoi1
U+7775.0	fMandarin	cai1
U+7776.0	fUTF8	睶
U+7776.0	fCNS11643	3-4835
U+7776.0	fGB	3-5A2C
U+7776.0	fR/S	  109.9
U+7776.0	fTotalStrokes	 14
U+7776.0	fCangjie	BUQKA
U+7776.0	fFourCorner	65060
U+7776.0	fCantonese	ceon2
U+7776.0	fMandarin	chun3
U+7777.0	fUTF8	睷
U+7777.0	fCNS11643	3-4834
U+7777.0	fGB	3-5A28
U+7777.0	fR/S	 109.9
U+7777.0	fTotalStrokes	 14; 24
U+7777.0	fCangjie	BUNKQ
U+7777.0	fFourCorner	65040
U+7777.0	fCantonese	gin1
U+7777.0	fMandarin	jian1
U+7778.0	fUTF8	睸
U+7778.0	fCNS11643	3-4838
U+7778.0	fGB	E-3473
U+7778.0	fR/S	 109.9
U+7778.0	fTotalStrokes	 14
U+7778.0	fCangjie	BUAHU
U+7778.0	fFourCorner	67060
U+7778.0	fCantonese	mei1
U+7778.0	fMandarin	mei4
U+7778.0	fEnglish	close; shut
U+7779.0	fUTF8	睹
U+7779.0	fCNS11643	1-6547
U+7779.0	fBig5	B840
U+7779.0	fGB	0-3643
U+7779.0	fR/S	 109.9
U+7779.0	fTotalStrokes	 14
U+7779.0	fCangjie	BUJKA
U+7779.0	fFourCorner	64060
U+7779.0	fHakka	 du3;  du5
U+7779.0	fCantonese	dou2
U+7779.0	fMandarin	du3
U+7779.0	fEnglish	 witness; see; look at; observe; gaze at
U+777A.0	fUTF8	睺
U+777A.0	fCNS11643	3-483B
U+777A.0	fGB	3-5A37
U+777A.0	fR/S	  109.9
U+777A.0	fTotalStrokes	 14
U+777A.0	fCangjie	BUONK
U+777A.0	fFourCorner	67030
U+777A.0	fHakka	 heu2
U+777A.0	fCantonese	hau4; hau1
U+777A.0	fMandarin	hou2
U+777A.0	fEnglish	dim of sight
U+777B.0	fUTF8	睻
U+777B.0	fCNS11643	3-483D
U+777B.0	fGB	5-5954
U+777B.0	fR/S	 109.9
U+777B.0	fTotalStrokes	 14
U+777B.0	fCangjie	BUJMM
U+777B.0	fFourCorner	63010
U+777B.0	fCantonese	hyun1
U+777B.0	fMandarin	xuan1
U+777C.0	fUTF8	睼
U+777C.0	fCNS11643	2-4C2B
U+777C.0	fBig5	E2E3
U+777C.0	fGB	5-5950
U+777C.0	fR/S	 109.9
U+777C.0	fTotalStrokes	 14
U+777C.0	fCangjie	BUAMO
U+777C.0	fFourCorner	66081
U+777C.0	fHakka	  ti5
U+777C.0	fCantonese	tin3; dai6; tai4
U+777C.0	fMandarin	ti4; ti2; tian4
U+777C.0	fEnglish	cooked
U+777D.0	fUTF8	睽
U+777D.0	fCNS11643	1-6973
U+777D.0	fBig5	BACC
U+777D.0	fGB	0-6E25
U+777D.0	fR/S	 109.9
U+777D.0	fTotalStrokes	 14
U+777D.0	fCangjie	BUNOK
U+777D.0	fFourCorner	62034
U+777D.0	fHakka	 kui2; kwi2; k'wui2
U+777D.0	fCantonese	kwai4; kwai1
U+777D.0	fMandarin	kui2; ji4
U+777D.0	fEnglish	[1] separated [2] in opposition [3] squint [4] stare at [5] unusual; strange
U+777E.0	fUTF8	睾
U+777E.0	fCNS11643	2-4C31
U+777E.0	fBig5	E2E9
U+777E.0	fGB	0-583A
U+777E.0	fR/S	 109.9
U+777E.0	fTotalStrokes	 14
U+777E.0	fCangjie	HWGTJ
U+777E.0	fFourCorner	26401
U+777E.0	fHakka	 jit8; rit8
U+777E.0	fCantonese	gou1
U+777E.0	fMandarin	gao1; hao4
U+777E.0	fEnglish	[1] marsh [2] high; lofty [3] testicle; testis
U+777F.0	fUTF8	睿
U+777F.0	fCNS11643	1-6974
U+777F.0	fBig5	BACD
U+777F.0	fGB	0-6E23
U+777F.0	fR/S	 109.9
U+777F.0	fTotalStrokes	 14
U+777F.0	fCangjie	YBMCU
U+777F.0	fFourCorner	21608
U+777F.0	fHakka	 jui3; rui3; jui5
U+777F.0	fCantonese	jeoi6
U+777F.0	fMandarin	rui4
U+777F.0	fEnglish	[1] understand thoroughly; quick or keen of perception [2] wise and clever; astute and discreet; perspicacious [3] profoundest (learning); divine sagacity of sages
U+7780.0	fUTF8	瞀
U+7780.0	fCNS11643	2-4C2F
U+7780.0	fBig5	E2E7
U+7780.0	fGB	0-6E26
U+7780.0	fR/S	 109.9
U+7780.0	fTotalStrokes	 14
U+7780.0	fCangjie	NKBU
U+7780.0	fFourCorner	18604
U+7780.0	fHakka	 meu5; meu6; miu3
U+7780.0	fCantonese	mau6
U+7780.0	fMandarin	mao4; mou4; wu2; mou2; mu4; wu4
U+7780.0	fEnglish	[1] dim-sighted; nearsighted; indistinct vision [2] illiterate; ignorant; feebleminded [3] confused; confusion; dazzled
U+7781.0	fUTF8	瞁
U+7781.0	fCNS11643	2-4C2A
U+7781.0	fBig5	E2E2
U+7781.0	fGB	3-5A31
U+7781.0	fR/S	 109.9
U+7781.0	fTotalStrokes	 14
U+7781.0	fCangjie	BUBUK
U+7781.0	fFourCorner	66033
U+7781.0	fCantonese	gwik1; hik1
U+7781.0	fMandarin	xu4
U+7782.0	fUTF8	瞂
U+7782.0	fCNS11643	2-4C2D
U+7782.0	fBig5	E2E5
U+7782.0	fGB	5-3078
U+7782.0	fR/S	 109.9
U+7782.0	fTotalStrokes	 14
U+7782.0	fCangjie	HUIKK
U+7782.0	fFourCorner	73247
U+7782.0	fCantonese	fat6
U+7782.0	fMandarin	fa1; fa2
U+7783.0	fUTF8	瞃
U+7783.0	fCNS11643	2-4C32
U+7783.0	fBig5	E2EA
U+7783.0	fGB	E-3474
U+7783.0	fR/S	 109.9
U+7783.0	fTotalStrokes	 14
U+7783.0	fCangjie	BUHJU
U+7783.0	fFourCorner	62064
U+7783.0	fCantonese	wat1
U+7783.0	fMandarin	wen1
U+7784.0	fUTF8	瞄
U+7784.0	fCNS11643	1-6972
U+7784.0	fBig5	BACB
U+7784.0	fGB	0-4369
U+7784.0	fR/S	 109.9; 109.8
U+7784.0	fTotalStrokes	 14
U+7784.0	fCangjie	BUTW
U+7784.0	fFourCorner	64060
U+7784.0	fHakka	 miau2; miau3; meu3; meu2
U+7784.0	fCantonese	miu4
U+7784.0	fMandarin	miao2
U+7784.0	fEnglish	 aim at; take aim ; look at attentively
U+7785.0	fUTF8	瞅
U+7785.0	fCNS11643	2-4C2C
U+7785.0	fBig5	E2E4
U+7785.0	fGB	0-3372
U+7785.0	fR/S	 109.9
U+7785.0	fTotalStrokes	 14
U+7785.0	fCangjie	BUHDF
U+7785.0	fFourCorner	69080
U+7785.0	fHakka	  ceu3; seu3; ciu3
U+7785.0	fCantonese	cau2
U+7785.0	fMandarin	chou3
U+7785.0	fEnglish	 look at; see; gaze
U+7786.0	fUTF8	瞆
U+7786.0	fGB	8-7D4D
U+7786.0	fR/S	 109.12; 109.9
U+7786.0	fTotalStrokes	 14
U+7786.0	fCangjie	XBULM
U+7786.0	fCantonese	gwai3; kui2 (also); kwui2 (also)
U+7786.0	fMandarin	kui4; gui4
U+7786.0	fEnglish	dim
U+7787.0	fUTF8	瞇
U+7787.0	fCNS11643	1-6D77
U+7787.0	fBig5	BD4E
U+7787.0	fGB	E-3475
U+7787.0	fR/S	 109.10
U+7787.0	fTotalStrokes	 15
U+7787.0	fCangjie	BUYFD
U+7787.0	fFourCorner	69039
U+7787.0	fHakka	 mi1; mi2
U+7787.0	fCantonese	mai5; mei1; mai4 (also); mi1 (also)
U+7787.0	fMandarin	mi1; mi3; mi4
U+7787.0	fEnglish	[1] keep one's eyes half closed; narrow the eyes [2] take a nap
U+7788.0	fUTF8	瞈
U+7788.0	fCNS11643	2-5247
U+7788.0	fBig5	E6BF
U+7788.0	fGB	5-5961
U+7788.0	fR/S	 109.10
U+7788.0	fTotalStrokes	 15
U+7788.0	fCangjie	BUCIM
U+7788.0	fFourCorner	68027
U+7788.0	fCantonese	jung2
U+7788.0	fMandarin	weng3
U+7789.0	fUTF8	瞉
U+7789.0	fCNS11643	2-5246
U+7789.0	fBig5	E6BE
U+7789.0	fGB	3-4F37
U+7789.0	fR/S	 109.10
U+7789.0	fTotalStrokes	 15
U+7789.0	fCangjie	GUHNE
U+7789.0	fFourCorner	47647
U+7789.0	fCantonese	kau3; gai3
U+7789.0	fMandarin	kou4; gou4; ji4
U+778A.0	fUTF8	瞊
U+778A.0	fCNS11643	3-4D44
U+778A.0	fGB	5-5963
U+778A.0	fR/S	  109.10
U+778A.0	fTotalStrokes	 15
U+778A.0	fCangjie	BUILR
U+778A.0	fFourCorner	60060
U+778A.0	fCantonese	dong6
U+778A.0	fMandarin	dang4
U+778B.0	fUTF8	瞋
U+778B.0	fCNS11643	1-6D7A
U+778B.0	fBig5	BD51
U+778B.0	fGB	8-2D75
U+778B.0	fR/S	 109.10
U+778B.0	fTotalStrokes	 15
U+778B.0	fCangjie	BUJBC
U+778B.0	fFourCorner	61081
U+778B.0	fHakka	 zhin1;  chin1; zihn1; zin1; cihn1
U+778B.0	fCantonese	can1; zan1 (also); hap6; hap1 (also); sau2; hat6
U+778B.0	fMandarin	chen1; chi1; tian2; tian4; zhen4
U+778B.0	fEnglish	[1] angry; anger [2] open the eyes
U+778C.0	fUTF8	瞌
U+778C.0	fCNS11643	1-6D78
U+778C.0	fBig5	BD4F
U+778C.0	fGB	0-6E27
U+778C.0	fR/S	 109.10
U+778C.0	fTotalStrokes	 15
U+778C.0	fCangjie	BUGIT
U+778C.0	fFourCorner	64017
U+778C.0	fHakka	  kap8; hap8; kap7
U+778C.0	fCantonese	hap6; hap1
U+778C.0	fMandarin	ke1
U+778C.0	fEnglish	 be tired and to doze off
U+778D.0	fUTF8	瞍
U+778D.0	fCNS11643	2-5244
U+778D.0	fBig5	E6BC
U+778D.0	fGB	0-6E24
U+778D.0	fR/S	 109.10
U+778D.0	fTotalStrokes	 15
U+778D.0	fCangjie	BUHXE
U+778D.0	fFourCorner	67047
U+778D.0	fHakka	 seu3
U+778D.0	fCantonese	sau2
U+778D.0	fMandarin	sou3; sou1; xiao1
U+778D.0	fEnglish	 have eyes without pupils; blind
U+778E.0	fUTF8	瞎
U+778E.0	fCNS11643	1-6D76
U+778E.0	fBig5	BD4D
U+778E.0	fGB	0-4F39
U+778E.0	fR/S	 109.10
U+778E.0	fTotalStrokes	 15
U+778E.0	fCangjie	BUJQR
U+778E.0	fFourCorner	63061
U+778E.0	fHakka	  hat7; hat8
U+778E.0	fCantonese	hat6
U+778E.0	fMandarin	xia1
U+778E.0	fEnglish	[1] blind; blindly [2] reckless; heedless; rash; (do things, etc.) without purpose or reason; at random; groundlessly [3] become tangled (said of thread, etc.)
U+778F.0	fUTF8	瞏
U+778F.0	fCNS11643	2-5245
U+778F.0	fBig5	E6BD
U+778F.0	fGB	5-5A51
U+778F.0	fR/S	 109.10
U+778F.0	fTotalStrokes	 15
U+778F.0	fCangjie	WLGRV
U+778F.0	fFourCorner	60732
U+778F.0	fHakka	 kiun2
U+778F.0	fCantonese	king4
U+778F.0	fMandarin	qiong2; xuan2
U+778F.0	fEnglish	round
U+7790.0	fUTF8	瞐
U+7790.0	fCNS11643	3-4D46
U+7790.0	fGB	3-5A42
U+7790.0	fR/S	  109.10
U+7790.0	fTotalStrokes	 15
U+7790.0	fCangjie	BUBUU
U+7790.0	fFourCorner	60660
U+7790.0	fCantonese	mok6
U+7790.0	fMandarin	mao4
U+7791.0	fUTF8	瞑
U+7791.0	fCNS11643	1-6D79
U+7791.0	fBig5	BD50
U+7791.0	fGB	0-6E28
U+7791.0	fR/S	 109.10
U+7791.0	fTotalStrokes	 15
U+7791.0	fCangjie	BUBAC
U+7791.0	fFourCorner	67080
U+7791.0	fHakka	 men2; men3
U+7791.0	fCantonese	ming4; ming5 (also); ming2 (also); min6
U+7791.0	fMandarin	ming2; ming3; mian2; meng2; ming4
U+7791.0	fEnglish	 close the eyes
U+7791.1	fUTF8	瞑
U+7791.1	fCNS11643	1-6D79
U+7791.1	fBig5	BD50
U+7791.1	fGB	0-6E28
U+7791.1	fR/S	 109.10
U+7791.1	fTotalStrokes	 15
U+7791.1	fCangjie	BUBAC
U+7791.1	fFourCorner	67080
U+7791.1	fMandarin	mian4
U+7791.1	fEnglish	 throw into a state of confusion
U+7792.0	fUTF8	瞒
U+7792.0	fGB	0-4277
U+7792.0	fR/S	 109.10
U+7792.0	fTotalStrokes	 15
U+7792.0	fCangjie	BUTMB
U+7792.0	fHakka	 man2
U+7792.0	fCantonese	mun4
U+7792.0	fMandarin	man2; men2; man3; men4; men3
U+7792.0	fEnglish	simplified form of &#x779E;
U+7793.0	fUTF8	瞓
U+7793.0	fGB	1-7C45
U+7793.0	fR/S	 109.10
U+7793.0	fTotalStrokes	 15
U+7793.0	fCangjie	BUYRL
U+7793.0	fHakka	  fun5
U+7793.0	fCantonese	fan3
U+7793.0	fMandarin	shui4
U+7793.0	fEnglish	 sleep; go to bed; lie down
U+7794.0	fUTF8	瞔
U+7794.0	fCNS11643	4-515F
U+7794.0	fGB	E-3476
U+7794.0	fR/S	  109.11
U+7794.0	fTotalStrokes	 16
U+7794.0	fCangjie	BUQMC
U+7794.0	fFourCorner	65080
U+7794.0	fCantonese	zaak3
U+7794.0	fMandarin	ze2
U+7795.0	fUTF8	瞕
U+7795.0	fCNS11643	2-5869
U+7795.0	fBig5	EA7D
U+7795.0	fGB	5-596C
U+7795.0	fR/S	 109.11
U+7795.0	fTotalStrokes	 16
U+7795.0	fCangjie	BUYTJ
U+7795.0	fFourCorner	60046
U+7795.0	fHakka	 zhong5; zong5
U+7795.0	fCantonese	zoeng3
U+7795.0	fMandarin	zhang4
U+7795.0	fEnglish	cataract in the eye
U+7796.0	fUTF8	瞖
U+7796.0	fCNS11643	3-517B
U+7796.0	fGB	E-3477
U+7796.0	fR/S	  109.11
U+7796.0	fTotalStrokes	 16
U+7796.0	fCangjie	SEBU
U+7796.0	fFourCorner	77600
U+7796.0	fHakka	 ji5; ri5
U+7796.0	fCantonese	ai3; ngai3 (also)
U+7796.0	fMandarin	yi4
U+7796.0	fEnglish	a cataract or film over the eye
U+7797.0	fUTF8	瞗
U+7797.0	fCNS11643	2-586B
U+7797.0	fBig5	EAA1
U+7797.0	fGB	3-5949
U+7797.0	fR/S	 109.11
U+7797.0	fTotalStrokes	 16
U+7797.0	fCangjie	BUHAF
U+7797.0	fFourCorner	67027
U+7797.0	fCantonese	diu1; dau1
U+7797.0	fMandarin	diao1; dou1
U+7798.0	fUTF8	瞘
U+7798.0	fCNS11643	3-5179
U+7798.0	fGB	1-6D6E
U+7798.0	fR/S	 109.11
U+7798.0	fTotalStrokes	 16
U+7798.0	fCangjie	BUSRR
U+7798.0	fFourCorner	61010
U+7798.0	fHakka	 eu1; eu2
U+7798.0	fCantonese	kau1
U+7798.0	fMandarin	ou1; kou1
U+7798.0	fEnglish	deeply-sunk eyes
U+7799.0	fUTF8	瞙
U+7799.0	fCNS11643	2-586A
U+7799.0	fBig5	EA7E
U+7799.0	fGB	5-5958
U+7799.0	fR/S	 109.11
U+7799.0	fTotalStrokes	 16
U+7799.0	fCangjie	BUTAK
U+7799.0	fFourCorner	64034
U+7799.0	fHakka	 mok8; mo1; mok7
U+7799.0	fCantonese	mok6
U+7799.0	fMandarin	mo4
U+7799.0	fEnglish	eyesight dimmed, as by thickening of the cornea
U+779A.0	fUTF8	瞚
U+779A.0	fCNS11643	2-5862
U+779A.0	fBig5	EA76
U+779A.0	fGB	5-596E
U+779A.0	fR/S	 109.11
U+779A.0	fTotalStrokes	 16
U+779A.0	fCangjie	BUJMC
U+779A.0	fFourCorner	60086
U+779A.0	fHakka	 shun5; sun5
U+779A.0	fCantonese	seon3; seon6
U+779A.0	fMandarin	shun4
U+779A.0	fEnglish	 blink; glance
U+779B.0	fUTF8	瞛
U+779B.0	fCNS11643	2-5866
U+779B.0	fBig5	EA7A
U+779B.0	fGB	3-5935
U+779B.0	fR/S	 109.11
U+779B.0	fTotalStrokes	 16
U+779B.0	fCangjie	BUHOO
U+779B.0	fFourCorner	68681
U+779B.0	fCantonese	cung1
U+779B.0	fMandarin	cong1
U+779C.0	fUTF8	瞜
U+779C.0	fCNS11643	2-5865
U+779C.0	fBig5	EA79
U+779C.0	fGB	3-5A3B
U+779C.0	fR/S	 109.11
U+779C.0	fTotalStrokes	 16
U+779C.0	fCangjie	BULLV
U+779C.0	fFourCorner	65044
U+779C.0	fCantonese	lau1; lau4 (also)
U+779C.0	fMandarin	lou1; lou2
U+779D.0	fUTF8	瞝
U+779D.0	fCNS11643	2-5863
U+779D.0	fBig5	EA77
U+779D.0	fGB	E-3478
U+779D.0	fR/S	 109.11
U+779D.0	fTotalStrokes	 16
U+779D.0	fCangjie	BUYUB
U+779D.0	fFourCorner	60027
U+779D.0	fHakka	 li2
U+779D.0	fCantonese	ci1
U+779D.0	fMandarin	chi1
U+779E.0	fUTF8	瞞
U+779E.0	fCNS11643	1-7150
U+779E.0	fBig5	BF66
U+779E.0	fGB	1-4277
U+779E.0	fR/S	 109.11
U+779E.0	fTotalStrokes	 16
U+779E.0	fCangjie	BUTLB
U+779E.0	fFourCorner	64027
U+779E.0	fHakka	  man2; man3
U+779E.0	fCantonese	mun4
U+779E.0	fMandarin	man2; men2; man3; men4; men3
U+779E.0	fEnglish	[1] hide the truth; fool others by lying; deceive [2] dim-sighted; poor vision
U+779F.0	fUTF8	瞟
U+779F.0	fCNS11643	1-7151
U+779F.0	fBig5	BF67
U+779F.0	fGB	0-6E29
U+779F.0	fR/S	 109.11
U+779F.0	fTotalStrokes	 16
U+779F.0	fCangjie	BUMWF
U+779F.0	fFourCorner	61091
U+779F.0	fHakka	 piau3; peu3; piau1; piau5
U+779F.0	fCantonese	piu5; piu2
U+779F.0	fMandarin	piao3; piao1; piao4
U+779F.0	fEnglish	 look askance at
U+77A0.0	fUTF8	瞠
U+77A0.0	fCNS11643	1-714F
U+77A0.0	fBig5	BF65
U+77A0.0	fGB	0-6E2A
U+77A0.0	fR/S	 109.11
U+77A0.0	fTotalStrokes	 16
U+77A0.0	fCangjie	BUFBG
U+77A0.0	fFourCorner	69014
U+77A0.0	fHakka	  cang1; cang2; ch'ang2
U+77A0.0	fCantonese	caang1; tong4 (also); zaang1
U+77A0.0	fMandarin	cheng1; zheng4; chang1; cheng4
U+77A0.0	fEnglish	 look straight at; stare at
U+77A1.0	fUTF8	瞡
U+77A1.0	fCNS11643	2-5864
U+77A1.0	fBig5	EA78
U+77A1.0	fGB	5-5941
U+77A1.0	fR/S	 109.11
U+77A1.0	fTotalStrokes	 16
U+77A1.0	fCangjie	BUQOU
U+77A1.0	fFourCorner	66010
U+77A1.0	fCantonese	kwai1
U+77A1.0	fMandarin	ji4; gui1
U+77A2.0	fUTF8	瞢
U+77A2.0	fCNS11643	2-5867
U+77A2.0	fBig5	EA7B
U+77A2.0	fGB	0-5E2B
U+77A2.0	fR/S	 109.11
U+77A2.0	fTotalStrokes	 16
U+77A2.0	fCangjie	TWLU
U+77A2.0	fFourCorner	44602
U+77A2.0	fHakka	 mung5; mung6; mung3; mu5
U+77A2.0	fCantonese	mung4; mung1 (also); mang4; mung6
U+77A2.0	fMandarin	meng2; mang2; meng4; meng3; mie4
U+77A2.0	fEnglish	[1] dim-sighted; poor vision [2] obscure; not bright [3] ashamed [4] grief
U+77A3.0	fUTF8	瞣
U+77A3.0	fCNS11643	2-5868
U+77A3.0	fBig5	EA7C
U+77A3.0	fGB	E-3479
U+77A3.0	fR/S	 109.11
U+77A3.0	fTotalStrokes	 16
U+77A3.0	fCangjie	BULLP
U+77A3.0	fFourCorner	65030
U+77A3.0	fCantonese	waan6
U+77A3.0	fMandarin	huan4
U+77A4.0	fUTF8	瞤
U+77A4.0	fCNS11643	3-5559
U+77A4.0	fGB	3-5975
U+77A4.0	fR/S	  109.12
U+77A4.0	fTotalStrokes	 17
U+77A4.0	fCangjie	BUANG
U+77A4.0	fFourCorner	67020
U+77A4.0	fHakka	 shun5; sun5
U+77A4.0	fCantonese	seon4; seon3 (also)
U+77A4.0	fMandarin	run2
U+77A4.0	fEnglish	 blink or twitch the eyes
U+77A5.0	fUTF8	瞥
U+77A5.0	fCNS11643	1-7152
U+77A5.0	fBig5	BF68
U+77A5.0	fGB	0-4633
U+77A5.0	fR/S	 109.12
U+77A5.0	fTotalStrokes	 17
U+77A5.0	fCangjie	FKBU
U+77A5.0	fFourCorner	98604
U+77A5.0	fHakka	 piet7; pet7; piak7; pit7
U+77A5.0	fCantonese	pit3
U+77A5.0	fMandarin	pie1; bi4
U+77A5.0	fEnglish	 have a casual and short glance; catch a glimpse of
U+77A6.0	fUTF8	瞦
U+77A6.0	fCNS11643	3-5558
U+77A6.0	fGB	3-5A48
U+77A6.0	fR/S	 109.12
U+77A6.0	fTotalStrokes	 17
U+77A6.0	fCangjie	BUGRR
U+77A6.0	fFourCorner	64060
U+77A6.0	fCantonese	hei1
U+77A6.0	fMandarin	xi1
U+77A7.0	fUTF8	瞧
U+77A7.0	fCNS11643	1-744A
U+77A7.0	fBig5	C140
U+77A7.0	fGB	0-4746
U+77A7.0	fR/S	 109.12
U+77A7.0	fTotalStrokes	 17
U+77A7.0	fCangjie	BUOGF
U+77A7.0	fFourCorner	60031
U+77A7.0	fHakka	  ziau1; ciau2; zeu1
U+77A7.0	fCantonese	ciu4; pok3; zuk1; dang1; dang6 (also); sam2; seon3
U+77A7.0	fMandarin	qiao2
U+77A7.0	fEnglish	[1] see; look at [2] steal a glance; glance quickly
U+77A8.0	fUTF8	瞨
U+77A8.0	fCNS11643	2-5D6D
U+77A8.0	fBig5	EDA3
U+77A8.0	fGB	E-347A
U+77A8.0	fR/S	 109.12
U+77A8.0	fTotalStrokes	 17
U+77A8.0	fCangjie	BUTCO
U+77A8.0	fFourCorner	62035
U+77A8.0	fHakka	 puk7; pu3; puk8
U+77A8.0	fCantonese	pok3; buk6
U+77A8.0	fMandarin	pu2; pu1
U+77A8.0	fEnglish	near sight
U+77A9.0	fUTF8	瞩
U+77A9.0	fCNS11643	3-555B
U+77A9.0	fGB	0-5675
U+77A9.0	fR/S	 109.12
U+77A9.0	fTotalStrokes	 17
U+77A9.0	fCangjie	BUSHB
U+77A9.0	fFourCorner	67020
U+77A9.0	fHakka	 chuk7
U+77A9.0	fCantonese	zuk1
U+77A9.0	fMandarin	zhu3
U+77A9.0	fEnglish	simplified form of &#x77DA;
U+77AA.0	fUTF8	瞪
U+77AA.0	fCNS11643	1-7447
U+77AA.0	fBig5	C0FC
U+77AA.0	fGB	0-3549
U+77AA.0	fR/S	 109.12
U+77AA.0	fTotalStrokes	 17
U+77AA.0	fCangjie	BUNOT
U+77AA.0	fFourCorner	62018
U+77AA.0	fHakka	 cang5; dang1; chin2; cihn2; cang2; den1; den5; chin1
U+77AA.0	fCantonese	dang6; cing4; caang4; dang1
U+77AA.0	fMandarin	deng4; cheng2
U+77AA.0	fEnglish	 stare at; open (one's eyes) wide; glare at
U+77AB.0	fUTF8	瞫
U+77AB.0	fCNS11643	2-5D67
U+77AB.0	fBig5	ED7B
U+77AB.0	fGB	3-5A49
U+77AB.0	fR/S	 109.12
U+77AB.0	fTotalStrokes	 17
U+77AB.0	fCangjie	BUMWJ
U+77AB.0	fFourCorner	61046
U+77AB.0	fHakka	 shim3; siim3
U+77AB.0	fCantonese	sam2
U+77AB.0	fMandarin	shen3; dan4; shen4; tan2
U+77AB.0	fEnglish	look
U+77AC.0	fUTF8	瞬
U+77AC.0	fCNS11643	1-7449
U+77AC.0	fBig5	C0FE
U+77AC.0	fGB	0-4B32
U+77AC.0	fR/S	 109.12
U+77AC.0	fTotalStrokes	 17
U+77AC.0	fCangjie	BUBBQ
U+77AC.0	fFourCorner	62052
U+77AC.0	fHakka	 sun5;  shun5
U+77AC.0	fCantonese	seon3
U+77AC.0	fMandarin	shun4
U+77AC.0	fEnglish	[1] blink; wink or twinkle [2] very short time; in the twinkling of an eye
U+77AD.0	fUTF8	瞭
U+77AD.0	fCNS11643	1-744B
U+77AD.0	fBig5	C141
U+77AD.0	fGB	1-7851
U+77AD.0	fR/S	 109.12
U+77AD.0	fTotalStrokes	 17
U+77AD.0	fCangjie	BUKCF
U+77AD.0	fFourCorner	64096
U+77AD.0	fHakka	  liau2; liau3; liau6; liau5
U+77AD.0	fCantonese	liu5; liu4; liu2 (also)
U+77AD.0	fMandarin	liao3; liao2; yao3
U+77AD.0	fEnglish	[1] understand [2] clear and bright
U+77AD.1	fUTF8	瞭
U+77AD.1	fCNS11643	1-744B
U+77AD.1	fBig5	C141
U+77AD.1	fGB	1-7851
U+77AD.1	fR/S	 109.12
U+77AD.1	fTotalStrokes	 17
U+77AD.1	fCangjie	BUKCF
U+77AD.1	fFourCorner	64096
U+77AD.1	fMandarin	liao4
U+77AD.1	fEnglish	 look down from a higher place
U+77AE.0	fUTF8	瞮
U+77AE.0	fCNS11643	4-5161
U+77AE.0	fGB	E-347B
U+77AE.0	fR/S	 109.12; 109.11
U+77AE.0	fTotalStrokes	 17; 16
U+77AE.0	fCangjie	BUYBK
U+77AE.0	fFourCorner	68040
U+77AE.0	fCantonese	cit3
U+77AE.0	fMandarin	che4
U+77AF.0	fUTF8	瞯
U+77AF.0	fCNS11643	3-555C
U+77AF.0	fGB	3-5976
U+77AF.0	fR/S	  109.12
U+77AF.0	fTotalStrokes	 17
U+77AF.0	fCangjie	BUANB
U+77AF.0	fFourCorner	67020
U+77AF.0	fHakka	  han2; gan5
U+77AF.0	fCantonese	gaan3; haan4; haan2
U+77AF.0	fMandarin	xian2; jian4
U+77AF.0	fEnglish	 peep spy
U+77B0.0	fUTF8	瞰
U+77B0.0	fCNS11643	1-7448
U+77B0.0	fBig5	C0FD
U+77B0.0	fGB	0-6E2B
U+77B0.0	fR/S	 109.12
U+77B0.0	fTotalStrokes	 17
U+77B0.0	fCangjie	BUMJK
U+77B0.0	fFourCorner	68040
U+77B0.0	fHakka	  kam3;  kam5
U+77B0.0	fCantonese	ham3; ham6
U+77B0.0	fMandarin	kan4
U+77B0.0	fEnglish	[1] watch; look far away [2] overlook (from a high ground)
U+77B1.0	fUTF8	瞱
U+77B1.0	fCNS11643	2-5D6C
U+77B1.0	fBig5	EDA2
U+77B1.0	fGB	3-5958
U+77B1.0	fR/S	 109.12; 109.10
U+77B1.0	fTotalStrokes	 17
U+77B1.0	fCangjie	BUTMJ
U+77B1.0	fFourCorner	64054
U+77B1.0	fCantonese	jip6
U+77B1.0	fMandarin	ye4
U+77B2.0	fUTF8	瞲
U+77B2.0	fCNS11643	2-5D68
U+77B2.0	fBig5	ED7C
U+77B2.0	fGB	5-5971
U+77B2.0	fR/S	 109.12
U+77B2.0	fTotalStrokes	 17
U+77B2.0	fCangjie	BUNHB
U+77B2.0	fFourCorner	67027
U+77B2.0	fCantonese	hyut3; wat6; leot6; jyut6
U+77B2.0	fMandarin	xu4; xue4
U+77B3.0	fUTF8	瞳
U+77B3.0	fCNS11643	1-7446
U+77B3.0	fBig5	C0FB
U+77B3.0	fGB	0-4D2B
U+77B3.0	fR/S	 109.12
U+77B3.0	fTotalStrokes	 17
U+77B3.0	fCangjie	BUYTG
U+77B3.0	fFourCorner	60014
U+77B3.0	fHakka	  tung2; tung3
U+77B3.0	fCantonese	tung4
U+77B3.0	fMandarin	tong2; chuang4
U+77B3.0	fEnglish	[1] pupil of the eye [2] stupid; ignorant
U+77B4.0	fUTF8	瞴
U+77B4.0	fCNS11643	2-5D6B
U+77B4.0	fBig5	EDA1
U+77B4.0	fGB	3-592B
U+77B4.0	fR/S	 109.12
U+77B4.0	fTotalStrokes	 17
U+77B4.0	fCangjie	BUOTF
U+77B4.0	fFourCorner	68031
U+77B4.0	fCantonese	mou4; mei4; mou5
U+77B4.0	fMandarin	mou2; wu2; mi2; wu3
U+77B5.0	fUTF8	瞵
U+77B5.0	fCNS11643	2-5D66
U+77B5.0	fBig5	ED7A
U+77B5.0	fGB	0-6E2C
U+77B5.0	fR/S	 109.12
U+77B5.0	fTotalStrokes	 17
U+77B5.0	fCangjie	BUFDQ
U+77B5.0	fFourCorner	69059
U+77B5.0	fHakka	 lin2
U+77B5.0	fCantonese	leon4
U+77B5.0	fMandarin	lin2; lian2; lin4; lin3
U+77B5.0	fEnglish	[1] look at; stare at [2] (said of eyes) clear and bright
U+77B6.0	fUTF8	瞶
U+77B6.0	fCNS11643	2-5D6A
U+77B6.0	fBig5	ED7E
U+77B6.0	fGB	3-5A35
U+77B6.0	fR/S	 109.12
U+77B6.0	fTotalStrokes	 17
U+77B6.0	fCangjie	BULMC
U+77B6.0	fFourCorner	65086
U+77B6.0	fHakka	 kui3; kwi5; k'wui3
U+77B6.0	fCantonese	gwai3; kui2 (also); kwui2 (also)
U+77B6.0	fMandarin	kui4; gui4; wei4
U+77B6.0	fEnglish	[1] blind man [2] stupid; ignorant
U+77B7.0	fUTF8	瞷
U+77B7.0	fCNS11643	2-5D69
U+77B7.0	fBig5	ED7D
U+77B7.0	fGB	3-5977
U+77B7.0	fR/S	 109.12
U+77B7.0	fTotalStrokes	 17
U+77B7.0	fCangjie	BUANA
U+77B7.0	fFourCorner	67020
U+77B7.0	fHakka	 gan5; gan3
U+77B7.0	fCantonese	gaan3
U+77B7.0	fMandarin	jian4; xian2
U+77B7.0	fEnglish	 water; peep at; spy; mix up
U+77B8.0	fUTF8	瞸
U+77B8.0	fCNS11643	3-5836
U+77B8.0	fGB	E-347C
U+77B8.0	fR/S	  109.13
U+77B8.0	fTotalStrokes	 18
U+77B8.0	fCangjie	BUTPD
U+77B8.0	fFourCorner	64090
U+77B8.0	fCantonese	jip6
U+77B8.0	fMandarin	ye4
U+77B9.0	fUTF8	瞹
U+77B9.0	fCNS11643	3-5838
U+77B9.0	fGB	5-5960
U+77B9.0	fR/S	 109.13
U+77B9.0	fTotalStrokes	 18
U+77B9.0	fCangjie	BUBBE
U+77B9.0	fFourCorner	62040
U+77B9.0	fHakka	 oi5
U+77B9.0	fCantonese	oi3; ngoi3 (also); oi2
U+77B9.0	fMandarin	ai4
U+77B9.0	fEnglish	[1] dull [2] obscure; hidden
U+77BA.0	fUTF8	瞺
U+77BA.0	fCNS11643	2-6242
U+77BA.0	fBig5	F055
U+77BA.0	fGB	5-5933
U+77BA.0	fR/S	 109.13
U+77BA.0	fTotalStrokes	 18
U+77BA.0	fCangjie	BUOMA
U+77BA.0	fFourCorner	68066
U+77BA.0	fCantonese	wai3; wui3
U+77BA.0	fMandarin	hui4
U+77BB.0	fUTF8	瞻
U+77BB.0	fCNS11643	1-766E
U+77BB.0	fBig5	C2A4
U+77BB.0	fGB	0-5530
U+77BB.0	fR/S	 109.13
U+77BB.0	fTotalStrokes	 18
U+77BB.0	fCangjie	BUNCR
U+77BB.0	fFourCorner	67061
U+77BB.0	fHakka	 zam1; zham1
U+77BB.0	fCantonese	zim1
U+77BB.0	fMandarin	zhan1; zhan4
U+77BB.0	fEnglish	[1] look; look up [2] regard respectfully; reverence
U+77BC.0	fUTF8	瞼
U+77BC.0	fCNS11643	1-766F
U+77BC.0	fBig5	C2A5
U+77BC.0	fGB	1-6D7A
U+77BC.0	fR/S	 109.13
U+77BC.0	fTotalStrokes	 18
U+77BC.0	fCangjie	BUOMO
U+77BC.0	fFourCorner	68086
U+77BC.0	fHakka	 ngiam1
U+77BC.0	fCantonese	gim2; lim4 (also)
U+77BC.0	fMandarin	jian3
U+77BC.0	fEnglish	 eyelids
U+77BD.0	fUTF8	瞽
U+77BD.0	fCNS11643	1-766C
U+77BD.0	fBig5	C2A2
U+77BD.0	fGB	0-6E2D
U+77BD.0	fR/S	 109.13
U+77BD.0	fTotalStrokes	 18
U+77BD.0	fCangjie	GEBU
U+77BD.0	fFourCorner	44604
U+77BD.0	fHakka	 gu3
U+77BD.0	fCantonese	gu2; gwu2 (also)
U+77BD.0	fMandarin	gu3
U+77BD.0	fEnglish	[1] blind [2] (in ancient China) blind musicians [3] having no discerning ability; lacking the power of judgment
U+77BE.0	fUTF8	瞾
U+77BE.0	fCNS11643	3-583B
U+77BE.0	fGB	E-347D
U+77BE.0	fR/S	 109.13
U+77BE.0	fTotalStrokes	 18
U+77BE.0	fCangjie	BUJCM
U+77BE.0	fFourCorner	66100
U+77BE.0	fCantonese	ziu3
U+77BE.0	fMandarin	zhao4
U+77BF.0	fUTF8	瞿
U+77BF.0	fCNS11643	1-766D
U+77BF.0	fBig5	C2A3
U+77BF.0	fGB	0-7644
U+77BF.0	fR/S	 109.13
U+77BF.0	fTotalStrokes	 18
U+77BF.0	fCangjie	BUOG
U+77BF.0	fFourCorner	66214
U+77BF.0	fHakka	 ki1; gi5; ki3; ki2
U+77BF.0	fCantonese	keoi4; geoi3; geoi6; geoi2
U+77BF.0	fMandarin	qu2; qu1; ji2; ji3
U+77BF.0	fEnglish	a Chinese family name
U+77BF.1	fUTF8	瞿
U+77BF.1	fCNS11643	1-766D
U+77BF.1	fBig5	C2A3
U+77BF.1	fGB	0-7644
U+77BF.1	fR/S	 109.13
U+77BF.1	fTotalStrokes	 18
U+77BF.1	fCangjie	BUOG
U+77BF.1	fFourCorner	66214
U+77BF.1	fMandarin	ju4
U+77BF.1	fEnglish	[1] shocked or scared [2] look around (said of hawks and falcons)
U+77C0.0	fUTF8	矀
U+77C0.0	fCNS11643	3-583A
U+77C0.0	fGB	5-5974
U+77C0.0	fR/S	  109.13
U+77C0.0	fTotalStrokes	 18
U+77C0.0	fCangjie	BUHOK
U+77C0.0	fFourCorner	68040
U+77C0.0	fCantonese	mei4
U+77C0.0	fMandarin	wei2
U+77C1.0	fUTF8	矁
U+77C1.0	fCNS11643	3-5839
U+77C1.0	fGB	E-347E
U+77C1.0	fR/S	 109.13
U+77C1.0	fTotalStrokes	 18
U+77C1.0	fCangjie	BUHFP
U+77C1.0	fFourCorner	69030
U+77C1.0	fHakka	 ceu3
U+77C1.0	fCantonese	cau2
U+77C1.0	fMandarin	chou3
U+77C1.0	fEnglish	 look at gaze
U+77C2.0	fUTF8	矂
U+77C2.0	fCNS11643	2-6241
U+77C2.0	fBig5	F054
U+77C2.0	fGB	3-5A50
U+77C2.0	fR/S	 109.13
U+77C2.0	fTotalStrokes	 18
U+77C2.0	fCangjie	BURRD
U+77C2.0	fFourCorner	66094
U+77C2.0	fCantonese	cou3 (also); sou3
U+77C2.0	fMandarin	sao4; zao4
U+77C3.0	fUTF8	矃
U+77C3.0	fCNS11643	4-6039
U+77C3.0	fGB	E-3521
U+77C3.0	fR/S	 109.14
U+77C3.0	fTotalStrokes	 19
U+77C3.0	fCangjie	BUJPN
U+77C3.0	fFourCorner	63020
U+77C3.0	fHakka	 nen3
U+77C3.0	fCantonese	ning5; ling5 (also)
U+77C3.0	fMandarin	ning3
U+77C3.0	fEnglish	wax in the ear
U+77C4.0	fUTF8	矄
U+77C4.0	fCNS11643	2-6627
U+77C4.0	fBig5	F27B
U+77C4.0	fGB	3-5A55
U+77C4.0	fR/S	 109.14
U+77C4.0	fTotalStrokes	 19
U+77C4.0	fCangjie	BUHGF
U+77C4.0	fFourCorner	62031
U+77C4.0	fCantonese	fan1
U+77C4.0	fMandarin	xun1
U+77C5.0	fUTF8	矅
U+77C5.0	fCNS11643	3-5A60
U+77C5.0	fGB	3-5A57
U+77C5.0	fR/S	  109.14
U+77C5.0	fTotalStrokes	 19
U+77C5.0	fCangjie	BUSMG
U+77C5.0	fFourCorner	67010
U+77C5.0	fCantonese	jiu6
U+77C5.0	fMandarin	yao4
U+77C6.0	fUTF8	矆
U+77C6.0	fCNS11643	3-5A5F
U+77C6.0	fGB	3-5A4F
U+77C6.0	fR/S	  109.14
U+77C6.0	fTotalStrokes	 19
U+77C6.0	fCangjie	BUTOE
U+77C6.0	fFourCorner	64040
U+77C6.0	fCantonese	fok3
U+77C6.0	fMandarin	huo4; xue1
U+77C7.0	fUTF8	矇
U+77C7.0	fCNS11643	1-7854
U+77C7.0	fBig5	C3A9
U+77C7.0	fGB	1-7857
U+77C7.0	fR/S	 109.14
U+77C7.0	fTotalStrokes	 19
U+77C7.0	fCangjie	BUTBO
U+77C7.0	fFourCorner	64032
U+77C7.0	fHakka	 mung2; mung3
U+77C7.0	fCantonese	mung4; mung1 (also)
U+77C7.0	fMandarin	meng2; meng3
U+77C7.0	fEnglish	[1] blind [2] (figuratively) ignorant; stupid and obstinate ???
U+77C7.1	fUTF8	矇
U+77C7.1	fCNS11643	1-7854
U+77C7.1	fBig5	C3A9
U+77C7.1	fGB	1-7857
U+77C7.1	fR/S	 109.14
U+77C7.1	fTotalStrokes	 19
U+77C7.1	fCangjie	BUTBO
U+77C7.1	fFourCorner	64032
U+77C7.1	fMandarin	meng1
U+77C7.1	fEnglish	[1] deceive; cheat [2] lucky ???
U+77C8.0	fUTF8	矈
U+77C8.0	fGB	5-5976
U+77C8.0	fR/S	 109.14
U+77C8.0	fTotalStrokes	 19
U+77C8.0	fCangjie	BUHAS
U+77C8.0	fCantonese	min4
U+77C8.0	fMandarin	mian2; mian3
U+77C8.0	fEnglish	dense; detailed; fine
U+77C9.0	fUTF8	矉
U+77C9.0	fCNS11643	2-6625
U+77C9.0	fBig5	F279
U+77C9.0	fGB	5-5964
U+77C9.0	fR/S	 109.14
U+77C9.0	fTotalStrokes	 19
U+77C9.0	fCangjie	BUJMC
U+77C9.0	fFourCorner	63086
U+77C9.0	fCantonese	pan4
U+77C9.0	fMandarin	pin2; bin1
U+77C9.0	fEnglish	 angry glare open the eyes with anger
U+77CA.0	fUTF8	矊
U+77CA.0	fCNS11643	2-6626
U+77CA.0	fBig5	F27A
U+77CA.0	fGB	3-5A47
U+77CA.0	fR/S	 109.14
U+77CA.0	fTotalStrokes	 19
U+77CA.0	fCangjie	BUVFB
U+77CA.0	fFourCorner	66027
U+77CA.0	fCantonese	min4
U+77CA.0	fMandarin	mian2; mian3
U+77CB.0	fUTF8	矋
U+77CB.0	fGB	1-7C46
U+77CB.0	fR/S	 109.15
U+77CB.0	fTotalStrokes	 20
U+77CB.0	fCangjie	BUMTB
U+77CB.0	fCantonese	lai6; lai3
U+77CB.0	fMandarin	li4
U+77CB.0	fEnglish	 stare
U+77CC.0	fUTF8	矌
U+77CC.0	fCNS11643	2-6943
U+77CC.0	fBig5	F474
U+77CC.0	fGB	3-5925
U+77CC.0	fR/S	 109.15
U+77CC.0	fTotalStrokes	 20
U+77CC.0	fCangjie	BUITC
U+77CC.0	fFourCorner	60086
U+77CC.0	fCantonese	kwong3; kong3 (also); gwong3
U+77CC.0	fMandarin	kuang4; guang4; guo1
U+77CD.0	fUTF8	矍
U+77CD.0	fCNS11643	2-6946
U+77CD.0	fBig5	F477
U+77CD.0	fGB	0-5B47
U+77CD.0	fR/S	 109.15
U+77CD.0	fTotalStrokes	 20
U+77CD.0	fCangjie	BUOGE
U+77CD.0	fFourCorner	66407
U+77CD.0	fHakka	 kiok7; giok7
U+77CD.0	fCantonese	fok3; gwok3
U+77CD.0	fMandarin	jue2
U+77CD.0	fEnglish	[1] watch in fright; scared [2] old but healthy
U+77CE.0	fUTF8	矎
U+77CE.0	fCNS11643	2-6944
U+77CE.0	fBig5	F475
U+77CE.0	fGB	3-5A56
U+77CE.0	fR/S	 109.15
U+77CE.0	fTotalStrokes	 20
U+77CE.0	fCangjie	BUNBE
U+77CE.0	fFourCorner	67047
U+77CE.0	fCantonese	hing3; hyun3
U+77CE.0	fMandarin	xuan1; xing4; xuan4
U+77CF.0	fUTF8	矏
U+77CF.0	fCNS11643	2-6945
U+77CF.0	fBig5	F476
U+77CF.0	fGB	E-3522
U+77CF.0	fR/S	 109.15
U+77CF.0	fTotalStrokes	 20
U+77CF.0	fCangjie	BUHUS
U+77CF.0	fFourCorner	66027
U+77CF.0	fCantonese	min4
U+77CF.0	fMandarin	mian2; mian3
U+77D0.0	fUTF8	矐
U+77D0.0	fCNS11643	2-6B6E
U+77D0.0	fBig5	F5E0
U+77D0.0	fGB	3-5A5A
U+77D0.0	fR/S	 109.16
U+77D0.0	fTotalStrokes	 21
U+77D0.0	fCangjie	BUMBG
U+77D0.0	fFourCorner	61014
U+77D0.0	fCantonese	hok6; kok3 (also); fok3; hok3
U+77D0.0	fMandarin	huo4; he4
U+77D1.0	fUTF8	矑
U+77D1.0	fCNS11643	3-5E42
U+77D1.0	fGB	3-5943
U+77D1.0	fR/S	  109.16
U+77D1.0	fTotalStrokes	 21
U+77D1.0	fCangjie	BUYPT
U+77D1.0	fFourCorner	61010
U+77D1.0	fHakka	  lu2
U+77D1.0	fCantonese	lou4
U+77D1.0	fMandarin	lu2
U+77D1.0	fEnglish	 pupil of the eye see
U+77D2.0	fUTF8	矒
U+77D2.0	fCNS11643	4-665E
U+77D2.0	fGB	E-3523
U+77D2.0	fR/S	 109.16
U+77D2.0	fTotalStrokes	 21
U+77D2.0	fCangjie	BUTWU
U+77D2.0	fFourCorner	64060
U+77D2.0	fCantonese	mung4
U+77D2.0	fMandarin	meng2; mang2; meng4
U+77D3.0	fUTF8	矓
U+77D3.0	fCNS11643	1-7A7A
U+77D3.0	fBig5	C4EF
U+77D3.0	fGB	3-5941
U+77D3.0	fR/S	 109.16
U+77D3.0	fTotalStrokes	 21
U+77D3.0	fCangjie	BUYBP
U+77D3.0	fFourCorner	61011
U+77D3.0	fHakka	 lung2
U+77D3.0	fCantonese	lung4
U+77D3.0	fMandarin	long2
U+77D3.0	fEnglish	hazy; blurred
U+77D4.0	fUTF8	矔
U+77D4.0	fCNS11643	2-6F3A
U+77D4.0	fBig5	F7EB
U+77D4.0	fGB	3-5A5C
U+77D4.0	fR/S	 109.18
U+77D4.0	fTotalStrokes	 23
U+77D4.0	fCangjie	BUTRG
U+77D4.0	fFourCorner	64014
U+77D4.0	fCantonese	gun3; gwun3 (also)
U+77D4.0	fMandarin	guan4; quan2
U+77D5.0	fUTF8	矕
U+77D5.0	fCNS11643	2-7042
U+77D5.0	fBig5	F8B4
U+77D5.0	fGB	3-595E
U+77D5.0	fR/S	 109.19
U+77D5.0	fTotalStrokes	 24
U+77D5.0	fCangjie	VFBU
U+77D5.0	fFourCorner	22609
U+77D5.0	fCantonese	maan5
U+77D5.0	fMandarin	man3; man2; luan2
U+77D5.0	fEnglish	see; witness; inspect
U+77D6.0	fUTF8	矖
U+77D6.0	fCNS11643	4-6B74
U+77D6.0	fGB	3-5966
U+77D6.0	fR/S	 109.21; 109.19
U+77D6.0	fTotalStrokes	 24; 26
U+77D6.0	fCangjie	BUMMP
U+77D6.0	fFourCorner	61010
U+77D6.0	fHakka	  sai5;  sa3;  li2
U+77D6.0	fCantonese	saai2; lai6
U+77D6.0	fMandarin	xi3
U+77D6.0	fEnglish	 stare at
U+77D7.0	fUTF8	矗
U+77D7.0	fCNS11643	1-7C63
U+77D7.0	fBig5	C5F7
U+77D7.0	fGB	0-3423
U+77D7.0	fR/S	 109.19
U+77D7.0	fTotalStrokes	 24
U+77D7.0	fCangjie	JMJMM
U+77D7.0	fFourCorner	40716
U+77D7.0	fHakka	 ch'uk8; cuk7; cuk8; chuk8
U+77D7.0	fCantonese	cuk1
U+77D7.0	fMandarin	chu4; chong4
U+77D7.0	fEnglish	[1] rising sharply; steep [2] lofty; upright; straight [3] luxuriant growth
U+77D8.0	fUTF8	矘
U+77D8.0	fCNS11643	2-7128
U+77D8.0	fBig5	F8F8
U+77D8.0	fGB	3-5A41
U+77D8.0	fR/S	 109.20
U+77D8.0	fTotalStrokes	 25
U+77D8.0	fCangjie	BUFBF
U+77D8.0	fFourCorner	69031
U+77D8.0	fCantonese	tong2
U+77D8.0	fMandarin	tang3
U+77D9.0	fUTF8	矙
U+77D9.0	fCNS11643	2-7129
U+77D9.0	fBig5	F8F9
U+77D9.0	fGB	E-3524
U+77D9.0	fR/S	 109.20
U+77D9.0	fTotalStrokes	 25
U+77D9.0	fCangjie	BUANK
U+77D9.0	fFourCorner	67077
U+77D9.0	fHakka	 kam3; kam5
U+77D9.0	fCantonese	ham3; ham6
U+77D9.0	fMandarin	kan4
U+77D9.0	fEnglish	[1] watch; spy [2] overlook; look downward
U+77DA.0	fUTF8	矚
U+77DA.0	fCNS11643	1-7D33
U+77DA.0	fBig5	C666
U+77DA.0	fGB	1-5675
U+77DA.0	fR/S	 109.21
U+77DA.0	fTotalStrokes	 26
U+77DA.0	fCangjie	BUSEI
U+77DA.0	fFourCorner	67027
U+77DA.0	fHakka	 zuk7; chuk7
U+77DA.0	fCantonese	zuk1
U+77DA.0	fMandarin	zhu3
U+77DA.0	fEnglish	 watch; observe or gaze at carefully or steadily; pay attention
U+77DB.0	fUTF8	矛
U+77DB.0	fCNS11643	1-4679
U+77DB.0	fBig5	A5D9
U+77DB.0	fGB	0-432C
U+77DB.0	fR/S	 110.0
U+77DB.0	fTotalStrokes	 5
U+77DB.0	fCangjie	NINH
U+77DB.0	fFourCorner	17222
U+77DB.0	fHakka	  mau2; mau3
U+77DB.0	fCantonese	maau4
U+77DB.0	fMandarin	mao2
U+77DB.0	fEnglish	[1] spear; lance [2] Kangxi radical 110
U+77DC.0	fUTF8	矜
U+77DC.0	fCNS11643	1-5264
U+77DC.0	fBig5	ACE1
U+77DC.0	fGB	0-7166
U+77DC.0	fR/S	 110.4
U+77DC.0	fTotalStrokes	 9
U+77DC.0	fCangjie	NHOIN
U+77DC.0	fFourCorner	18227
U+77DC.0	fHakka	 kim1
U+77DC.0	fCantonese	ging1; gwaan1; kan4; kam4; ging6; gwan1
U+77DC.0	fMandarin	jin1; qin2; qing2
U+77DC.0	fEnglish	[1] pity; feel sorry for; be sympathetic with; commiserate with; compassionate [2] brag; boast; arrogant; conceited [3] self-esteem; self-control; self-discipline; dignified [4] emulate
U+77DC.1	fUTF8	矜
U+77DC.1	fCNS11643	1-5264
U+77DC.1	fBig5	ACE1
U+77DC.1	fGB	0-7166
U+77DC.1	fR/S	 110.4
U+77DC.1	fTotalStrokes	 9
U+77DC.1	fCangjie	NHOIN
U+77DC.1	fFourCorner	18227
U+77DC.1	fMandarin	guan1
U+77DC.1	fEnglish	 widower
U+77DD.0	fUTF8	矝
U+77DD.0	fCNS11643	3-3269
U+77DD.0	fGB	5-5F24
U+77DD.0	fR/S	  110.5
U+77DD.0	fTotalStrokes	 10
U+77DD.0	fCangjie	NHOII
U+77DD.0	fFourCorner	18237
U+77DD.0	fCantonese	ging1
U+77DD.0	fMandarin	lin2
U+77DE.0	fUTF8	矞
U+77DE.0	fCNS11643	2-3E47
U+77DE.0	fBig5	DAC3
U+77DE.0	fGB	3-6028
U+77DE.0	fR/S	 110.7
U+77DE.0	fTotalStrokes	 12
U+77DE.0	fCangjie	NHBCR
U+77DE.0	fFourCorner	20227
U+77DE.0	fHakka	  lut8; jut8; jut7
U+77DE.0	fCantonese	wat6; leot6; jyut6
U+77DE.0	fMandarin	yu4; jue2; shu4; xi4; xu4
U+77DE.0	fEnglish	[1] bright and brilliant; charming [2] nature bursting into life [3] clouds of many hues
U+77DF.0	fUTF8	矟
U+77DF.0	fCNS11643	3-3D57
U+77DF.0	fGB	5-5F27
U+77DF.0	fR/S	  110.7
U+77DF.0	fTotalStrokes	 12
U+77DF.0	fCangjie	NHFB
U+77DF.0	fFourCorner	19220
U+77DF.0	fHakka	  sok7
U+77DF.0	fCantonese	sok3
U+77DF.0	fMandarin	shuo4
U+77DF.0	fEnglish	8' lance
U+77E0.0	fUTF8	矠
U+77E0.0	fCNS11643	2-4549
U+77E0.0	fBig5	DEE3
U+77E0.0	fGB	3-6029
U+77E0.0	fR/S	 110.8
U+77E0.0	fTotalStrokes	 13
U+77E0.0	fCangjie	NHTA
U+77E0.0	fFourCorner	14261
U+77E0.0	fCantonese	zaak6; caak3
U+77E0.0	fMandarin	ce4; ze2; zhuo2
U+77E1.0	fUTF8	矡
U+77E1.0	fCNS11643	4-6C76
U+77E1.0	fGB	3-602E
U+77E1.0	fR/S	  110.20
U+77E1.0	fTotalStrokes	 25
U+77E1.0	fCangjie	NHBUE
U+77E1.0	fFourCorner	16240
U+77E1.0	fCantonese	fok3
U+77E1.0	fMandarin	jue2
U+77E2.0	fUTF8	矢
U+77E2.0	fCNS11643	1-467A
U+77E2.0	fBig5	A5DA
U+77E2.0	fGB	0-4A38
U+77E2.0	fR/S	 111.0
U+77E2.0	fTotalStrokes	 5
U+77E2.0	fCangjie	OK
U+77E2.0	fFourCorner	80430
U+77E2.0	fHakka	 si3; chi3; chi1; cih1; ci1; ci3; she5; shi5
U+77E2.0	fCantonese	ci2; si2
U+77E2.0	fMandarin	shi3
U+77E2.0	fEnglish	[1] arrow; dart [2] vow; swear; take an oath; pledge [3] display [4] straightforward
U+77E3.0	fUTF8	矣
U+77E3.0	fCNS11643	1-4B32
U+77E3.0	fBig5	A86F
U+77E3.0	fGB	0-5253
U+77E3.0	fR/S	 111.2
U+77E3.0	fTotalStrokes	 7
U+77E3.0	fCangjie	IOK
U+77E3.0	fFourCorner	23430
U+77E3.0	fHakka	  ji1; ji3; ri1; i3
U+77E3.0	fCantonese	ji5
U+77E3.0	fMandarin	yi3; xian2
U+77E3.0	fEnglish	[1] a final particle denoting the perfect tense [2] an auxiliary denoting determination [3] both particles or auxiliaries indicating exclamations or questions
U+77E4.0	fUTF8	矤
U+77E4.0	fCNS11643	4-2752
U+77E4.0	fGB	5-3E78
U+77E4.0	fR/S	  111.3
U+77E4.0	fTotalStrokes	 8
U+77E4.0	fCangjie	NOK
U+77E4.0	fFourCorner	18230
U+77E4.0	fCantonese	can2
U+77E4.0	fMandarin	shen3
U+77E5.0	fUTF8	知
U+77E5.0	fCNS11643	1-4F21
U+77E5.0	fBig5	AABE
U+77E5.0	fGB	0-562A
U+77E5.0	fR/S	 111.3
U+77E5.0	fTotalStrokes	 8
U+77E5.0	fCangjie	OKR
U+77E5.0	fFourCorner	86400
U+77E5.0	fHakka	  di1; zhi1; zi1; zi5
U+77E5.0	fCantonese	zi1; zi3
U+77E5.0	fMandarin	zhi1
U+77E5.0	fEnglish	[1] know; understand; comprehend; feel; recognize; be aware of [2] knowledge [3] acquaint; be familiar with; befriend [4] control; operate; direct [5] wait on
U+77E5.1	fUTF8	知
U+77E5.1	fCNS11643	1-4F21
U+77E5.1	fBig5	AABE
U+77E5.1	fGB	0-562A
U+77E5.1	fR/S	 111.3
U+77E5.1	fTotalStrokes	 8
U+77E5.1	fCangjie	OKR
U+77E5.1	fFourCorner	86400
U+77E5.1	fMandarin	zhi4
U+77E5.1	fEnglish	the learned; the wise; brains
U+77E6.0	fUTF8	矦
U+77E6.0	fCNS11643	3-2E36
U+77E6.0	fGB	5-342B
U+77E6.0	fR/S	  111.4
U+77E6.0	fTotalStrokes	 9
U+77E6.0	fCangjie	NMOK
U+77E6.0	fFourCorner	27230
U+77E6.0	fCantonese	hau4; hau6
U+77E6.0	fMandarin	hou2
U+77E6.0	fEnglish	[1] the second of the five grades of the nobility; marquis [2] bull's eye [3] beautiful [4] an opening particle; but [5] why [6] a Chinese family name
U+77E7.0	fUTF8	矧
U+77E7.0	fCNS11643	2-2C4A
U+77E7.0	fBig5	CFE8
U+77E7.0	fGB	0-6F72
U+77E7.0	fR/S	 111.4
U+77E7.0	fTotalStrokes	 9
U+77E7.0	fCangjie	OKNL
U+77E7.0	fFourCorner	22400
U+77E7.0	fHakka	 jin3; sin3; rin3; shin3
U+77E7.0	fCantonese	can2
U+77E7.0	fMandarin	shen3
U+77E7.0	fEnglish	[1] much more; still more [2] also [3] gums
U+77E8.0	fUTF8	矨
U+77E8.0	fCNS11643	2-2C4B
U+77E8.0	fBig5	CFE9
U+77E8.0	fGB	E-3525
U+77E8.0	fR/S	 111.4
U+77E8.0	fTotalStrokes	 9
U+77E8.0	fCangjie	OKHK
U+77E8.0	fFourCorner	82434
U+77E8.0	fCantonese	jing5
U+77E8.0	fMandarin	ying3
U+77E9.0	fUTF8	矩
U+77E9.0	fCNS11643	1-567B
U+77E9.0	fBig5	AF78
U+77E9.0	fGB	0-3E58
U+77E9.0	fR/S	 111.5
U+77E9.0	fTotalStrokes	 10
U+77E9.0	fCangjie	OKSS
U+77E9.0	fFourCorner	81417
U+77E9.0	fHakka	  gi3
U+77E9.0	fCantonese	geoi2
U+77E9.0	fMandarin	ju3
U+77E9.0	fEnglish	[1] carpenter's square [2] rule; regulation; pattern [3] carve
U+77EA.0	fUTF8	矪
U+77EA.0	fCNS11643	4-335B
U+77EA.0	fGB	E-3526
U+77EA.0	fR/S	 111.6
U+77EA.0	fTotalStrokes	 11
U+77EA.0	fCangjie	OKHBY
U+77EA.0	fFourCorner	87440
U+77EA.0	fCantonese	zau1
U+77EA.0	fMandarin	zhou1
U+77EB.0	fUTF8	矫
U+77EB.0	fGB	0-3D43
U+77EB.0	fR/S	 111.6
U+77EB.0	fTotalStrokes	 11
U+77EB.0	fCangjie	OKHKL
U+77EB.0	fHakka	 giau3
U+77EB.0	fCantonese	giu2; kiu2 (also)
U+77EB.0	fMandarin	jiao3; jiao1; jiao2; jiao4
U+77EB.0	fEnglish	simplified form of &#x77EF;
U+77EC.0	fUTF8	矬
U+77EC.0	fCNS11643	2-3E48
U+77EC.0	fBig5	DAC4
U+77EC.0	fGB	0-6F73
U+77EC.0	fR/S	 111.7
U+77EC.0	fTotalStrokes	 12
U+77EC.0	fCangjie	OKOOG
U+77EC.0	fFourCorner	88414
U+77EC.0	fHakka	 co2
U+77EC.0	fCantonese	co4
U+77EC.0	fMandarin	cuo2
U+77EC.0	fEnglish	short; dwarf
U+77ED.0	fUTF8	短
U+77ED.0	fCNS11643	1-607B
U+77ED.0	fBig5	B575
U+77ED.0	fGB	0-364C
U+77ED.0	fR/S	 111.7
U+77ED.0	fTotalStrokes	 12
U+77ED.0	fCangjie	OKMRT
U+77ED.0	fFourCorner	81418
U+77ED.0	fHakka	  don3
U+77ED.0	fCantonese	dyun2
U+77ED.0	fMandarin	duan3
U+77ED.0	fEnglish	[1] short; brief [2] be deficient; want; lack; owe; lacking [3] shortcomings; faults; mistakes
U+77EE.0	fUTF8	矮
U+77EE.0	fCNS11643	1-654E
U+77EE.0	fBig5	B847
U+77EE.0	fGB	0-302B
U+77EE.0	fR/S	 111.8
U+77EE.0	fTotalStrokes	 13
U+77EE.0	fCangjie	OKHDV
U+77EE.0	fFourCorner	82444
U+77EE.0	fHakka	  ai3; e3
U+77EE.0	fCantonese	ai2; aai2 (also); ngai2 (also); ngaai2 (also)
U+77EE.0	fMandarin	ai3
U+77EE.0	fEnglish	[1] short person; dwarf [2] short; low; short of stature; low in height [3] lower
U+77EF.0	fUTF8	矯
U+77EF.0	fCNS11643	1-744C
U+77EF.0	fBig5	C142
U+77EF.0	fGB	1-3D43
U+77EF.0	fR/S	 111.12
U+77EF.0	fTotalStrokes	 17
U+77EF.0	fCangjie	OKHKB
U+77EF.0	fFourCorner	82427
U+77EF.0	fHakka	  giau3; kieu3
U+77EF.0	fCantonese	giu2; kiu2 (also)
U+77EF.0	fMandarin	jiao3; jiao1; jiao4
U+77EF.0	fEnglish	[1] correct; rectify; straighten [2] falsify; forge; fake; pretend [3] strong and powerful; vigorous; brave [4] raise (one's head) high [5] a Chinese family name
U+77F0.0	fUTF8	矰
U+77F0.0	fCNS11643	2-5D6E
U+77F0.0	fBig5	EDA4
U+77F0.0	fGB	3-5B4F
U+77F0.0	fR/S	 111.12
U+77F0.0	fTotalStrokes	 17
U+77F0.0	fCangjie	OKCWA
U+77F0.0	fFourCorner	88486
U+77F0.0	fHakka	  zen1
U+77F0.0	fCantonese	zang1
U+77F0.0	fMandarin	zeng1
U+77F0.0	fEnglish	 arrow with attached silk cord for shooting birds, etc.
U+77F1.0	fUTF8	矱
U+77F1.0	fCNS11643	2-6628
U+77F1.0	fBig5	F27C
U+77F1.0	fGB	3-5B50
U+77F1.0	fR/S	 111.14
U+77F1.0	fTotalStrokes	 19
U+77F1.0	fCangjie	OKTOE
U+77F1.0	fFourCorner	24447
U+77F1.0	fHakka	 wok7; vok7
U+77F1.0	fCantonese	wok3 (also); wok6
U+77F1.0	fMandarin	huo4; wo4; yue1
U+77F1.0	fEnglish	 measure; criterion
U+77F2.0	fUTF8	矲
U+77F2.0	fCNS11643	2-6947
U+77F2.0	fBig5	F478
U+77F2.0	fGB	5-5A74
U+77F2.0	fR/S	 111.15
U+77F2.0	fTotalStrokes	 20
U+77F2.0	fCangjie	OKWLP
U+77F2.0	fFourCorner	86411
U+77F2.0	fCantonese	baa6
U+77F2.0	fMandarin	bai3; bai4; ba4
U+77F3.0	fUTF8	石
U+77F3.0	fCNS11643	1-467B
U+77F3.0	fBig5	A5DB
U+77F3.0	fGB	0-4A2F
U+77F3.0	fR/S	 112.0
U+77F3.0	fTotalStrokes	 5
U+77F3.0	fCangjie	MR
U+77F3.0	fFourCorner	10600
U+77F3.0	fHakka	  sak8; shak8
U+77F3.0	fCantonese	sek6; daam3; sek2 (also)
U+77F3.0	fMandarin	shi2
U+77F3.0	fEnglish	[1] stone; rock; mineral [2] calculus (as a kidney calculus; kidney stone) [3] stone tablets [4] medicines [5] barren (as a barren woman) [6] name of an ancient musical instrument [7] a Chinese family name [8] Kangxi radical 112
U+77F3.1	fUTF8	石
U+77F3.1	fCNS11643	1-467B
U+77F3.1	fBig5	A5DB
U+77F3.1	fGB	0-4A2F
U+77F3.1	fR/S	 112.0
U+77F3.1	fTotalStrokes	 5
U+77F3.1	fCangjie	MR
U+77F3.1	fFourCorner	10600
U+77F3.1	fMandarin	dan4
U+77F3.1	fEnglish	[1] a dry measure for grains roughly equivalent to 120-160 pounds; picul [2] a weight measure equivalent to about 110 pounds
U+77F4.0	fUTF8	矴
U+77F4.0	fCNS11643	3-267B
U+77F4.0	fGB	E-3527
U+77F4.0	fR/S	  112.2
U+77F4.0	fTotalStrokes	 7
U+77F4.0	fCangjie	MRMN
U+77F4.0	fFourCorner	11620
U+77F4.0	fHakka	 den1; dang5; tin5; den5
U+77F4.0	fCantonese	deng6; ding3; deng3; ding6
U+77F4.0	fMandarin	ding4
U+77F4.0	fEnglish	[1] anchor [2] place (generally preceded by &#x6709; or &#x5187;)
U+77F5.0	fUTF8	矵
U+77F5.0	fCNS11643	3-267C
U+77F5.0	fGB	E-3528
U+77F5.0	fR/S	 112.2
U+77F5.0	fTotalStrokes	 7
U+77F5.0	fCangjie	MRLN
U+77F5.0	fFourCorner	12600
U+77F5.0	fCantonese	cai3
U+77F5.0	fMandarin	qi4; diao1
U+77F6.0	fUTF8	矶
U+77F6.0	fCNS11643	F-2523
U+77F6.0	fGB	0-6D36
U+77F6.0	fR/S	 112.2
U+77F6.0	fTotalStrokes	 7
U+77F6.0	fCangjie	MRHN
U+77F6.0	fHakka	 gi1
U+77F6.0	fCantonese	gei1
U+77F6.0	fMandarin	ji1; gai4
U+77F6.0	fEnglish	simplified form of &#x78EF;
U+77F7.0	fUTF8	矷
U+77F7.0	fCNS11643	2-2841
U+77F7.0	fBig5	CDA1
U+77F7.0	fGB	5-5774
U+77F7.0	fR/S	 112.3
U+77F7.0	fTotalStrokes	 8
U+77F7.0	fCangjie	MRND
U+77F7.0	fFourCorner	17647
U+77F7.0	fCantonese	zi2
U+77F7.0	fMandarin	zi3
U+77F8.0	fUTF8	矸
U+77F8.0	fCNS11643	2-283C
U+77F8.0	fBig5	CD7A
U+77F8.0	fGB	0-6D37
U+77F8.0	fR/S	 112.3
U+77F8.0	fTotalStrokes	 8
U+77F8.0	fCangjie	MRMJ
U+77F8.0	fFourCorner	11640
U+77F8.0	fHakka	 gon1; gon5
U+77F8.0	fCantonese	gon1; gon3; hon6
U+77F8.0	fMandarin	gan1; gan4; an4; han4; gan3
U+77F8.0	fEnglish	a rock or cliff
U+77F9.0	fUTF8	矹
U+77F9.0	fCNS11643	2-283E
U+77F9.0	fBig5	CD7C
U+77F9.0	fGB	5-5771
U+77F9.0	fR/S	 112.3
U+77F9.0	fTotalStrokes	 8
U+77F9.0	fCangjie	MRMU
U+77F9.0	fFourCorner	11610
U+77F9.0	fHakka	 ngut7
U+77F9.0	fCantonese	ngat6; at6 (also)
U+77F9.0	fMandarin	wu4
U+77F9.0	fEnglish	stones; gravel
U+77FA.0	fUTF8	矺
U+77FA.0	fCNS11643	2-2840
U+77FA.0	fBig5	CD7E
U+77FA.0	fGB	3-5748
U+77FA.0	fR/S	 112.3
U+77FA.0	fTotalStrokes	 8
U+77FA.0	fCangjie	MRHP
U+77FA.0	fFourCorner	12614
U+77FA.0	fHakka	 zak7
U+77FA.0	fCantonese	zaak3; zaak6
U+77FA.0	fMandarin	tuo1; da1; zha4
U+77FA.0	fEnglish	press steadily
U+77FB.0	fUTF8	矻
U+77FB.0	fCNS11643	2-283F
U+77FB.0	fBig5	CD7D
U+77FB.0	fGB	3-5749
U+77FB.0	fR/S	 112.3
U+77FB.0	fTotalStrokes	 8
U+77FB.0	fCangjie	MRON
U+77FB.0	fFourCorner	18617
U+77FB.0	fHakka	 kut8; kit7; kwut8
U+77FB.0	fCantonese	ngat6; fat1 (also); at6 (also)
U+77FB.0	fMandarin	ku4; wu4; ku1
U+77FB.0	fEnglish	[1] stone [2] diligent; industrious [3] very tired
U+77FC.0	fUTF8	矼
U+77FC.0	fCNS11643	2-283D
U+77FC.0	fBig5	CD7B
U+77FC.0	fGB	3-5747
U+77FC.0	fR/S	 112.3
U+77FC.0	fTotalStrokes	 8
U+77FC.0	fCangjie	MRM
U+77FC.0	fFourCorner	11610
U+77FC.0	fHakka	 gong1;  kiong1
U+77FC.0	fCantonese	gong1; hong1; kung4
U+77FC.0	fMandarin	qiang1; kong4; jiang1; gang1
U+77FC.0	fEnglish	 stone bridge stepping stones
U+77FD.0	fUTF8	矽
U+77FD.0	fCNS11643	1-4F22
U+77FD.0	fBig5	AABF
U+77FD.0	fGB	0-4E79
U+77FD.0	fR/S	 112.3
U+77FD.0	fTotalStrokes	 8
U+77FD.0	fCangjie	MRNI
U+77FD.0	fFourCorner	17620
U+77FD.0	fHakka	 sip8
U+77FD.0	fCantonese	zik6
U+77FD.0	fMandarin	xi4; xi1
U+77FD.0	fEnglish	silicon (Si)
U+77FE.0	fUTF8	矾
U+77FE.0	fCNS11643	F-2834
U+77FE.0	fGB	0-372F
U+77FE.0	fR/S	 112.3
U+77FE.0	fTotalStrokes	 8
U+77FE.0	fCangjie	MRHNI
U+77FE.0	fFourCorner	1761
U+77FE.0	fHakka	 fan2
U+77FE.0	fCantonese	faan4
U+77FE.0	fMandarin	fan2
U+77FE.0	fEnglish	simplified form of &#x792C;
U+77FF.0	fUTF8	矿
U+77FF.0	fGB	0-3F73
U+77FF.0	fR/S	 112.3
U+77FF.0	fTotalStrokes	 8
U+77FF.0	fCangjie	MRI
U+77FF.0	fHakka	 kwang3
U+77FF.0	fCantonese	kong3; kwong3 (also)
U+77FF.0	fMandarin	kuang4; gong3
U+77FF.0	fEnglish	simplified form of &#x7926;
U+7800.0	fUTF8	砀
U+7800.0	fGB	0-6D38
U+7800.0	fR/S	 112.3; 112.4
U+7800.0	fTotalStrokes	 8
U+7800.0	fCangjie	MRNSH
U+7800.0	fHakka	 tong5
U+7800.0	fCantonese	dong6
U+7800.0	fMandarin	dang4; tang2; yang2
U+7800.0	fEnglish	simplified form of &#x78AD;
U+7801.0	fUTF8	码
U+7801.0	fGB	0-426B
U+7801.0	fR/S	 112.3
U+7801.0	fTotalStrokes	 8
U+7801.0	fCangjie	MRNVM
U+7801.0	fHakka	 ma1
U+7801.0	fCantonese	maa5; maa2 (also)
U+7801.0	fMandarin	ma3
U+7801.0	fEnglish	simplified form of &#x78BC;
U+7802.0	fUTF8	砂
U+7802.0	fCNS11643	1-5265
U+7802.0	fBig5	ACE2
U+7802.0	fGB	0-4930
U+7802.0	fR/S	 112.4
U+7802.0	fTotalStrokes	 9
U+7802.0	fCangjie	MRFH
U+7802.0	fFourCorner	19620
U+7802.0	fHakka	  sa1
U+7802.0	fCantonese	saa1
U+7802.0	fMandarin	sha1
U+7802.0	fEnglish	[1] sand; coarse sand; gravel [2] coarse; not smooth [3] infinitesimal
U+7803.0	fUTF8	砃
U+7803.0	fCNS11643	2-2C54
U+7803.0	fBig5	CFF2
U+7803.0	fGB	3-5757
U+7803.0	fR/S	 112.4
U+7803.0	fTotalStrokes	 9
U+7803.0	fCangjie	MRBY
U+7803.0	fFourCorner	17640
U+7803.0	fCantonese	daan1
U+7803.0	fMandarin	dan1
U+7804.0	fUTF8	砄
U+7804.0	fCNS11643	3-2E39
U+7804.0	fGB	3-5759
U+7804.0	fR/S	  112.4
U+7804.0	fTotalStrokes	 9
U+7804.0	fCangjie	MRDK
U+7804.0	fFourCorner	15630
U+7804.0	fCantonese	kyut3
U+7804.0	fMandarin	jue2
U+7805.0	fUTF8	砅
U+7805.0	fCNS11643	2-2C4F
U+7805.0	fBig5	CFED
U+7805.0	fGB	5-577D
U+7805.0	fR/S	 112.4
U+7805.0	fTotalStrokes	 9
U+7805.0	fCangjie	MRE
U+7805.0	fFourCorner	12630
U+7805.0	fHakka	 li5; li6; pin2
U+7805.0	fCantonese	lai6
U+7805.0	fMandarin	li4; li2; ping1
U+7805.0	fEnglish	 cross a stream on stepping stones
U+7806.0	fUTF8	砆
U+7806.0	fCNS11643	2-2C4C
U+7806.0	fBig5	CFEA
U+7806.0	fGB	5-5775
U+7806.0	fR/S	 112.4
U+7806.0	fTotalStrokes	 9
U+7806.0	fCangjie	MRQO
U+7806.0	fFourCorner	15630
U+7806.0	fHakka	 fu1
U+7806.0	fCantonese	fu1
U+7806.0	fMandarin	fu1
U+7806.0	fEnglish	1/2 pr stone
U+7807.0	fUTF8	砇
U+7807.0	fCNS11643	3-2E3B
U+7807.0	fGB	5-577A
U+7807.0	fR/S	  112.4
U+7807.0	fTotalStrokes	 9
U+7807.0	fCangjie	MRYK
U+7807.0	fFourCorner	10640
U+7807.0	fHakka	 min2
U+7807.0	fCantonese	man4
U+7807.0	fMandarin	min2
U+7807.0	fEnglish	alabaster
U+7808.0	fUTF8	砈
U+7808.0	fCNS11643	3-2E3E
U+7808.0	fGB	E-3529
U+7808.0	fR/S	 112.4
U+7808.0	fTotalStrokes	 9
U+7808.0	fCangjie	MRMSU
U+7808.0	fFourCorner	11610
U+7808.0	fCantonese	ngo5
U+7808.0	fMandarin	nuo3
U+7809.0	fUTF8	砉
U+7809.0	fCNS11643	2-2C53
U+7809.0	fBig5	CFF1
U+7809.0	fGB	0-6D39
U+7809.0	fR/S	 112.4
U+7809.0	fTotalStrokes	 9
U+7809.0	fCangjie	QJMR
U+7809.0	fFourCorner	50601
U+7809.0	fHakka	 wak8; vak8
U+7809.0	fCantonese	waak6
U+7809.0	fMandarin	huo4; hua1; xu1
U+7809.0	fEnglish	a splitting sound; a cracking sound; the sound of tearing the skin off an animal
U+780A.0	fUTF8	砊
U+780A.0	fCNS11643	3-2E37
U+780A.0	fGB	3-5758
U+780A.0	fR/S	  112.4
U+780A.0	fTotalStrokes	 9
U+780A.0	fCangjie	MRYHN
U+780A.0	fFourCorner	10610
U+780A.0	fHakka	  kong5; kong1
U+780A.0	fCantonese	kong3; hong1
U+780A.0	fMandarin	kang4; kang1; keng1
U+780A.0	fEnglish	the sound of stones striking together
U+780B.0	fUTF8	砋
U+780B.0	fCNS11643	3-2E3A
U+780B.0	fGB	3-574F
U+780B.0	fR/S	 112.4
U+780B.0	fTotalStrokes	 9
U+780B.0	fCangjie	MRYLM
U+780B.0	fFourCorner	11610
U+780B.0	fCantonese	zi2
U+780B.0	fMandarin	zhi3
U+780C.0	fUTF8	砌
U+780C.0	fCNS11643	1-5267
U+780C.0	fBig5	ACE4
U+780C.0	fGB	0-4676
U+780C.0	fR/S	 112.4
U+780C.0	fTotalStrokes	 9
U+780C.0	fCangjie	MRPSH
U+780C.0	fFourCorner	17620
U+780C.0	fHakka	  ci5; che5; tch'e5; chii5
U+780C.0	fCantonese	cai3; cit3
U+780C.0	fMandarin	qi4; qie4
U+780C.0	fEnglish	[1] lay (bricks, etc.); pave; raise in layers; build [2] steps
U+780D.0	fUTF8	砍
U+780D.0	fCNS11643	1-5268
U+780D.0	fBig5	ACE5
U+780D.0	fGB	0-3F33
U+780D.0	fR/S	 112.4
U+780D.0	fTotalStrokes	 9
U+780D.0	fCangjie	MRNO
U+780D.0	fFourCorner	17682
U+780D.0	fHakka	 kam3;  kam5
U+780D.0	fCantonese	ham2
U+780D.0	fMandarin	kan3
U+780D.0	fEnglish	[1] hack; chop; cut; fell (trees, etc.) [2] throw at
U+780E.0	fUTF8	砎
U+780E.0	fCNS11643	2-2C52
U+780E.0	fBig5	CFF0
U+780E.0	fGB	3-5752
U+780E.0	fR/S	 112.4
U+780E.0	fTotalStrokes	 9
U+780E.0	fCangjie	MROLL
U+780E.0	fFourCorner	18620
U+780E.0	fHakka	 gai5; kie5; giai5
U+780E.0	fCantonese	gaai3
U+780E.0	fMandarin	jie4; xia2; jie2
U+780E.0	fEnglish	[1] hard; rocky [2] obstinate
U+780F.0	fUTF8	砏
U+780F.0	fCNS11643	2-2C51
U+780F.0	fBig5	CFEF
U+780F.0	fGB	3-5756
U+780F.0	fR/S	 112.4
U+780F.0	fTotalStrokes	 9
U+780F.0	fCangjie	MRCSH
U+780F.0	fFourCorner	18627
U+780F.0	fCantonese	fan1; paan1
U+780F.0	fMandarin	fen1; pan1; bin1
U+7810.0	fUTF8	砐
U+7810.0	fCNS11643	2-2C50
U+7810.0	fBig5	CFEE
U+7810.0	fGB	3-574A
U+7810.0	fR/S	 112.4
U+7810.0	fTotalStrokes	 9
U+7810.0	fCangjie	MRNHE
U+7810.0	fFourCorner	17647
U+7810.0	fCantonese	ngap6; ap6 (also)
U+7810.0	fMandarin	e4
U+7810.0	fEnglish	to nod one's head; to shake, sway
U+7811.0	fUTF8	砑
U+7811.0	fCNS11643	2-2C4D
U+7811.0	fBig5	CFEB
U+7811.0	fGB	0-6D3C
U+7811.0	fR/S	 112.4; 112.5
U+7811.0	fTotalStrokes	 9
U+7811.0	fCangjie	MRMVH
U+7811.0	fFourCorner	11640
U+7811.0	fHakka	  nga5
U+7811.0	fCantonese	ngaa6; aa6 (also); nga6
U+7811.0	fMandarin	ya4
U+7811.0	fEnglish	 grind; calender; polish roll with a stone roller
U+7812.0	fUTF8	砒
U+7812.0	fCNS11643	2-2C4E
U+7812.0	fBig5	CFEC
U+7812.0	fGB	0-4578
U+7812.0	fR/S	 112.4
U+7812.0	fTotalStrokes	 9
U+7812.0	fCangjie	MRPP
U+7812.0	fFourCorner	11610
U+7812.0	fHakka	 pai1;  pi1
U+7812.0	fCantonese	pei1
U+7812.0	fMandarin	pi1
U+7812.0	fEnglish	arsenic
U+7813.0	fUTF8	砓
U+7813.0	fCNS11643	2-2C55
U+7813.0	fBig5	CFF3
U+7813.0	fGB	E-352A
U+7813.0	fR/S	 112.4
U+7813.0	fTotalStrokes	 9
U+7813.0	fCangjie	MRHNE
U+7813.0	fFourCorner	17647
U+7813.0	fCantonese	zit3
U+7813.0	fMandarin	zhe2
U+7814.0	fUTF8	研
U+7814.0	fCNS11643	1-5266
U+7814.0	fBig5	ACE3
U+7814.0	fGB	0-5150
U+7814.0	fR/S	 112.6; 112.4
U+7814.0	fTotalStrokes	 11
U+7814.0	fCangjie	MRMT
U+7814.0	fFourCorner	11640
U+7814.0	fHakka	 ngan1;  ngan2; ngien1; ngan5; ngion1; ngien2; ngam5
U+7814.0	fCantonese	jin4; jin6; ngaan4; aan4 (also); gin2; gaan2 (also)
U+7814.0	fMandarin	yan2; yan4; xing2
U+7814.0	fEnglish	[1] grind; powder [2] study; research; go to the very source; investigate; examine; search into carefully
U+7815.0	fUTF8	砕
U+7815.0	fCNS11643	3-2E3D
U+7815.0	fGB	E-352B
U+7815.0	fR/S	 112.4
U+7815.0	fTotalStrokes	 9
U+7815.0	fCangjie	MRKNJ
U+7815.0	fFourCorner	17640
U+7815.0	fCantonese	seoi3
U+7815.0	fMandarin	sui4
U+7815.0	fEnglish	 break; smash broken; busted
U+7816.0	fUTF8	砖
U+7816.0	fGB	0-5729
U+7816.0	fR/S	 112.4
U+7816.0	fTotalStrokes	 9
U+7816.0	fCangjie	MRQNI
U+7816.0	fHakka	 zhon1; ngoi5
U+7816.0	fCantonese	zyun1
U+7816.0	fMandarin	zhuan1; tuan2; tuo2
U+7816.0	fEnglish	simplified form of &#x78DA;
U+7817.0	fUTF8	砗
U+7817.0	fGB	0-6D3A
U+7817.0	fR/S	 112.4
U+7817.0	fTotalStrokes	 9
U+7817.0	fCangjie	MRKQ
U+7817.0	fHakka	 cha1
U+7817.0	fCantonese	ce1
U+7817.0	fMandarin	che1; ju1
U+7817.0	fEnglish	simplified form of &#x7868;
U+7818.0	fUTF8	砘
U+7818.0	fCNS11643	F-2C5C
U+7818.0	fGB	0-6D3B
U+7818.0	fR/S	 112.4
U+7818.0	fTotalStrokes	 9
U+7818.0	fCangjie	MRPU
U+7818.0	fFourCorner	1561
U+7818.0	fCantonese	deon6
U+7818.0	fMandarin	dun4
U+7818.0	fEnglish	a kind of farm tool
U+7819.0	fUTF8	砙
U+7819.0	fCNS11643	3-656D
U+7819.0	fGB	E-352C
U+7819.0	fR/S	 112.4
U+7819.0	fTotalStrokes	 9
U+7819.0	fCangjie	MRMVN
U+7819.0	fFourCorner	1161
U+7819.0	fCantonese	ngaa5; aa5 (also)
U+7819.0	fMandarin	pan1
U+781A.0	fUTF8	砚
U+781A.0	fGB	0-5162
U+781A.0	fR/S	 112.4
U+781A.0	fTotalStrokes	 9
U+781A.0	fCangjie	MRBHU
U+781A.0	fHakka	 ngan5; jen5; jan5
U+781A.0	fCantonese	jin6; jin2 (also)
U+781A.0	fMandarin	yan4
U+781A.0	fEnglish	simplified form of &#x786F;
U+781B.0	fUTF8	砛
U+781B.0	fCNS11643	3-656E
U+781B.0	fGB	E-352D
U+781B.0	fR/S	 112.4
U+781B.0	fTotalStrokes	 9
U+781B.0	fCangjie	MROIN
U+781B.0	fFourCorner	1862
U+781B.0	fCantonese	gaam1
U+781C.0	fUTF8	砜
U+781C.0	fGB	0-6D3F
U+781C.0	fR/S	 112.4
U+781C.0	fTotalStrokes	 9
U+781C.0	fCangjie	MRHNK
U+781C.0	fCantonese	fung1
U+781C.0	fMandarin	feng1
U+781C.0	fEnglish	simplified form of &#x78B8;
U+781D.0	fUTF8	砝
U+781D.0	fCNS11643	1-5721
U+781D.0	fBig5	AF7C
U+781D.0	fGB	0-6D40
U+781D.0	fR/S	 112.5
U+781D.0	fTotalStrokes	 10
U+781D.0	fCangjie	MRGI
U+781D.0	fFourCorner	14631
U+781D.0	fHakka	  fap7
U+781D.0	fCantonese	faat3; fat3
U+781D.0	fMandarin	fa3; fa2; ge2; jie2
U+781D.0	fEnglish	standard balance weights use in scales; steelyard weights
U+781E.0	fUTF8	砞
U+781E.0	fCNS11643	3-667C
U+781E.0	fGB	E-352E
U+781E.0	fR/S	 112.5
U+781E.0	fTotalStrokes	 10
U+781E.0	fCangjie	MRDJ
U+781E.0	fFourCorner	1569
U+781E.0	fCantonese	mut6
U+781E.0	fMandarin	mo4
U+781E.0	fEnglish	 smash rock
U+781F.0	fUTF8	砟
U+781F.0	fCNS11643	1-5727
U+781F.0	fBig5	AFA4
U+781F.0	fGB	0-6D44
U+781F.0	fR/S	 112.5
U+781F.0	fTotalStrokes	 10
U+781F.0	fCangjie	MROS
U+781F.0	fFourCorner	18611
U+781F.0	fCantonese	zok6; zaa3; zaa1 (also); zaa2 (also); zok3
U+781F.0	fMandarin	zha4; zuo2; zha3; zuo4
U+781F.0	fEnglish	 stone tablet monument
U+7820.0	fUTF8	砠
U+7820.0	fCNS11643	1-5726
U+7820.0	fBig5	AFA3
U+7820.0	fGB	3-5760
U+7820.0	fR/S	 112.5
U+7820.0	fTotalStrokes	 10
U+7820.0	fCangjie	MRBM
U+7820.0	fFourCorner	17610
U+7820.0	fHakka	 zu1; zih3
U+7820.0	fCantonese	zeoi1; ceoi1; zo2
U+7820.0	fMandarin	qu1; ju1
U+7820.0	fEnglish	rocky; hilly; uneven
U+7821.0	fUTF8	砡
U+7821.0	fCNS11643	2-3144
U+7821.0	fBig5	D2E1
U+7821.0	fGB	5-577E
U+7821.0	fR/S	 112.5
U+7821.0	fTotalStrokes	 10
U+7821.0	fCangjie	MRMGI
U+7821.0	fFourCorner	11613
U+7821.0	fCantonese	juk6
U+7821.0	fMandarin	yu4
U+7822.0	fUTF8	砢
U+7822.0	fCNS11643	2-313E
U+7822.0	fBig5	D2DB
U+7822.0	fGB	3-575E
U+7822.0	fR/S	 112.5
U+7822.0	fTotalStrokes	 10
U+7822.0	fCangjie	MRMNR
U+7822.0	fFourCorner	11620
U+7822.0	fHakka	 ko3; lo3; ko1
U+7822.0	fCantonese	lo2; ngo1 (also); o1; lo5
U+7822.0	fMandarin	ke1; luo3
U+7822.0	fEnglish	a pile of stones
U+7823.0	fUTF8	砣
U+7823.0	fCNS11643	2-313C
U+7823.0	fBig5	D2D9
U+7823.0	fGB	0-6D48
U+7823.0	fR/S	 112.5
U+7823.0	fTotalStrokes	 10
U+7823.0	fCangjie	MRJP
U+7823.0	fFourCorner	13611
U+7823.0	fHakka	 to2; to3
U+7823.0	fCantonese	to4
U+7823.0	fMandarin	tuo2
U+7823.0	fEnglish	 stone roller heavy stone; weight; plummet
U+7824.0	fUTF8	砤
U+7824.0	fCNS11643	3-326C
U+7824.0	fGB	5-5825
U+7824.0	fR/S	  112.5
U+7824.0	fTotalStrokes	 10
U+7824.0	fCangjie	MROPD
U+7824.0	fFourCorner	18610
U+7824.0	fHakka	 to2
U+7824.0	fCantonese	to4
U+7824.0	fMandarin	tuo2
U+7825.0	fUTF8	砥
U+7825.0	fCNS11643	1-5724
U+7825.0	fBig5	AFA1
U+7825.0	fGB	0-6D46
U+7825.0	fR/S	 112.5
U+7825.0	fTotalStrokes	 10
U+7825.0	fCangjie	MRHPM
U+7825.0	fFourCorner	12640
U+7825.0	fHakka	 di3; zhi3; zih3; zi3; dai3; tje3
U+7825.0	fCantonese	dai2; zi2 (also); zi1; dai1
U+7825.0	fMandarin	di3; zhi3; di1; zhi1
U+7825.0	fEnglish	 whetstone polish and discipline
U+7826.0	fUTF8	砦
U+7826.0	fCNS11643	2-375C
U+7826.0	fBig5	D6B9
U+7826.0	fGB	0-6D4E
U+7826.0	fR/S	 112.5
U+7826.0	fTotalStrokes	 10
U+7826.0	fCangjie	YPMR
U+7826.0	fFourCorner	21601
U+7826.0	fHakka	  cai5; cai6; zai5
U+7826.0	fCantonese	zaai6
U+7826.0	fMandarin	zhai4
U+7826.0	fEnglish	 stockade; fort; military outpost
U+7827.0	fUTF8	砧
U+7827.0	fCNS11643	1-567D
U+7827.0	fBig5	AF7A
U+7827.0	fGB	0-5568
U+7827.0	fR/S	 112.5
U+7827.0	fTotalStrokes	 10
U+7827.0	fCangjie	MRYR
U+7827.0	fFourCorner	11660
U+7827.0	fHakka	 zem1; zem; zem3
U+7827.0	fCantonese	zam1
U+7827.0	fMandarin	zhen1
U+7827.0	fEnglish	[1] a rock with a flat top on which the laundry is beaten and washed; anvil [2] an ancient instrument for torture
U+7828.0	fUTF8	砨
U+7828.0	fCNS11643	2-3141
U+7828.0	fBig5	D2DE
U+7828.0	fGB	E-352F
U+7828.0	fR/S	 112.5
U+7828.0	fTotalStrokes	 10
U+7828.0	fCangjie	MRHSN
U+7828.0	fFourCorner	13617
U+7828.0	fCantonese	ak1; ngak1 (also); aai3
U+7828.0	fMandarin	ai4; e4
U+7829.0	fUTF8	砩
U+7829.0	fCNS11643	2-3145
U+7829.0	fBig5	D2E2
U+7829.0	fGB	0-6D49
U+7829.0	fR/S	 112.5
U+7829.0	fTotalStrokes	 10
U+7829.0	fCangjie	MRLLN
U+7829.0	fFourCorner	15627
U+7829.0	fCantonese	fai3; fat1
U+7829.0	fMandarin	fei4; fu2
U+782A.0	fUTF8	砪
U+782A.0	fCNS11643	2-3147
U+782A.0	fBig5	D2E4
U+782A.0	fGB	5-5826
U+782A.0	fR/S	 112.5
U+782A.0	fTotalStrokes	 10
U+782A.0	fCangjie	MRWYI
U+782A.0	fFourCorner	17650
U+782A.0	fCantonese	mou5
U+782A.0	fMandarin	mu3
U+782B.0	fUTF8	砫
U+782B.0	fCNS11643	2-3143
U+782B.0	fBig5	D2E0
U+782B.0	fGB	3-5766
U+782B.0	fR/S	 112.5
U+782B.0	fTotalStrokes	 10
U+782B.0	fCangjie	MRYG
U+782B.0	fFourCorner	10614
U+782B.0	fHakka	  zhu3; zu3
U+782B.0	fCantonese	zyu2; cyu5; cyu2
U+782B.0	fMandarin	zhu3; zhu4
U+782B.0	fEnglish	an ancestral tablet made of stone
U+782C.0	fUTF8	砬
U+782C.0	fCNS11643	2-313D
U+782C.0	fBig5	D2DA
U+782C.0	fGB	0-6D47
U+782C.0	fR/S	 112.5
U+782C.0	fTotalStrokes	 10
U+782C.0	fCangjie	MRYT
U+782C.0	fFourCorner	10618
U+782C.0	fHakka	 lip8
U+782C.0	fCantonese	laa1 (also); laap6
U+782C.0	fMandarin	li4; la2
U+782C.0	fEnglish	big stone
U+782D.0	fUTF8	砭
U+782D.0	fCNS11643	1-5725
U+782D.0	fBig5	AFA2
U+782D.0	fGB	0-6D3E
U+782D.0	fR/S	 112.5; 112.4
U+782D.0	fTotalStrokes	 10
U+782D.0	fCangjie	MRHIO
U+782D.0	fFourCorner	12637
U+782D.0	fHakka	  bien1; bien3; ben1; ben3; bien5
U+782D.0	fCantonese	bin1; bin2 (also)
U+782D.0	fMandarin	bian1; bian4
U+782D.0	fEnglish	[1] stone probe; pierce; stone needle used in acupuncture [2] advise; exhort; counsel; admonish [3] pierce
U+782E.0	fUTF8	砮
U+782E.0	fCNS11643	2-3142
U+782E.0	fBig5	D2DF
U+782E.0	fGB	3-5768
U+782E.0	fR/S	 112.5
U+782E.0	fTotalStrokes	 10
U+782E.0	fCangjie	VEMR
U+782E.0	fFourCorner	47601
U+782E.0	fHakka	 nu5; nu3
U+782E.0	fCantonese	nou5; lou5 (also); nou4; lou4 (also)
U+782E.0	fMandarin	nu3; nu2; nu4
U+782E.0	fEnglish	flint arrowheads; arrow-tip
U+782F.0	fUTF8	砯
U+782F.0	fCNS11643	2-3140
U+782F.0	fBig5	D2DD
U+782F.0	fGB	3-576C
U+782F.0	fR/S	 112.5
U+782F.0	fTotalStrokes	 10
U+782F.0	fCangjie	MRIE
U+782F.0	fFourCorner	12630
U+782F.0	fHakka	 pin2; pin5
U+782F.0	fCantonese	ping1; ping4
U+782F.0	fMandarin	ping1
U+782F.0	fEnglish	the noise of waves breaking on rocks
U+7830.0	fUTF8	砰
U+7830.0	fCNS11643	1-567C
U+7830.0	fBig5	AF79
U+7830.0	fGB	0-4569
U+7830.0	fR/S	 112.5
U+7830.0	fTotalStrokes	 10
U+7830.0	fCangjie	MRMFJ
U+7830.0	fFourCorner	11649
U+7830.0	fHakka	 pin2; bang3; bang5
U+7830.0	fCantonese	ping1; ping4; paang1
U+7830.0	fMandarin	peng1; ping1; peng4
U+7830.0	fEnglish	[1] the sound of crashing stones [2] bang!; a loud sound; deafening
U+7831.0	fUTF8	砱
U+7831.0	fCNS11643	2-3148
U+7831.0	fBig5	D2E5
U+7831.0	fGB	3-5763
U+7831.0	fR/S	 112.5
U+7831.0	fTotalStrokes	 10
U+7831.0	fCangjie	MROII
U+7831.0	fFourCorner	18627
U+7831.0	fCantonese	leon4; ling4
U+7831.0	fMandarin	ling2
U+7832.0	fUTF8	砲
U+7832.0	fCNS11643	1-5728
U+7832.0	fBig5	AFA5
U+7832.0	fGB	E-3530
U+7832.0	fR/S	 112.5
U+7832.0	fTotalStrokes	 10
U+7832.0	fCangjie	MRPRU
U+7832.0	fFourCorner	17612
U+7832.0	fHakka	  pau5
U+7832.0	fCantonese	paau3; baau3; paau4
U+7832.0	fMandarin	pao4; bao2; bo2
U+7832.0	fEnglish	 gun; cannon; artillery
U+7833.0	fUTF8	砳
U+7833.0	fCNS11643	2-3146
U+7833.0	fBig5	D2E3
U+7833.0	fGB	3-575F
U+7833.0	fR/S	 112.5
U+7833.0	fTotalStrokes	 10
U+7833.0	fCangjie	MRMR
U+7833.0	fFourCorner	11660
U+7833.0	fCantonese	laak6; lak6
U+7833.0	fMandarin	le4; que4
U+7833.0	fEnglish	rocky
U+7834.0	fUTF8	破
U+7834.0	fCNS11643	1-5722
U+7834.0	fBig5	AF7D
U+7834.0	fGB	0-4646
U+7834.0	fR/S	 112.5
U+7834.0	fTotalStrokes	 10
U+7834.0	fCangjie	MRDHE
U+7834.0	fFourCorner	14647
U+7834.0	fHakka	  po5
U+7834.0	fCantonese	po3
U+7834.0	fMandarin	po4; pi4
U+7834.0	fEnglish	[1] break [2] dilapidated; ruined; destroyed [3] defeat; beat (the enemy); capture (enemy territory) [4] expose; exposed; lay bare; see (through) [5] spend (money, etc.) [6] solve; break (a murder case, etc.); analyze [7] come to an end [8] paltry
U+7835.0	fUTF8	砵
U+7835.0	fCNS11643	2-313F
U+7835.0	fBig5	D2DC
U+7835.0	fGB	3-575D
U+7835.0	fR/S	 112.5
U+7835.0	fTotalStrokes	 10
U+7835.0	fCangjie	MRDM
U+7835.0	fFourCorner	14640
U+7835.0	fHakka	 bat7
U+7835.0	fCantonese	but3; but1 (also)
U+7835.0	fMandarin	bo1; e4
U+7835.0	fEnglish	syllable
U+7836.0	fUTF8	砶
U+7836.0	fCNS11643	3-326D
U+7836.0	fGB	E-3531
U+7836.0	fR/S	  112.5
U+7836.0	fTotalStrokes	 10
U+7836.0	fCangjie	MRHA
U+7836.0	fFourCorner	16600
U+7836.0	fCantonese	paak3
U+7836.0	fMandarin	po4
U+7837.0	fUTF8	砷
U+7837.0	fCNS11643	1-5723
U+7837.0	fBig5	AF7E
U+7837.0	fGB	0-4969
U+7837.0	fR/S	 112.5
U+7837.0	fTotalStrokes	 10
U+7837.0	fCangjie	MRLWL
U+7837.0	fFourCorner	15606
U+7837.0	fHakka	 sin1
U+7837.0	fCantonese	san1
U+7837.0	fMandarin	shen1
U+7837.0	fEnglish	arsenic
U+7838.0	fUTF8	砸
U+7838.0	fCNS11643	1-567E
U+7838.0	fBig5	AF7B
U+7838.0	fGB	0-5452
U+7838.0	fR/S	 112.5
U+7838.0	fTotalStrokes	 10
U+7838.0	fCangjie	MRSLB
U+7838.0	fFourCorner	11612
U+7838.0	fHakka	  zap7; sap7
U+7838.0	fCantonese	zaap3
U+7838.0	fMandarin	za2
U+7838.0	fEnglish	[1] smash; crush; crash; break; pound; knock [2] ruin; fail; be bungled [3] mash; beat to a pulp
U+7839.0	fUTF8	砹
U+7839.0	fGB	0-6D41
U+7839.0	fR/S	 112.5
U+7839.0	fTotalStrokes	 10
U+7839.0	fCangjie	MRTK
U+7839.0	fCantonese	ngaai6; aai6 (also)
U+7839.0	fMandarin	nuo3; ai4
U+7839.0	fEnglish	astatine
U+783A.0	fUTF8	砺
U+783A.0	fGB	0-6D42
U+783A.0	fR/S	 112.5
U+783A.0	fTotalStrokes	 10
U+783A.0	fCangjie	MRMMS
U+783A.0	fHakka	 li5
U+783A.0	fCantonese	lai6
U+783A.0	fMandarin	li4
U+783A.0	fEnglish	simplified form of &#x792A;
U+783B.0	fUTF8	砻
U+783B.0	fGB	0-6D43
U+783B.0	fR/S	 112.5; 212.5
U+783B.0	fTotalStrokes	 10
U+783B.0	fCangjie	IPMR
U+783B.0	fHakka	 lung2
U+783B.0	fCantonese	lung4
U+783B.0	fMandarin	long2; long4
U+783B.0	fEnglish	simplified form of &#x7931;
U+783C.0	fUTF8	砼
U+783C.0	fGB	0-6D45
U+783C.0	fR/S	 112.5
U+783C.0	fTotalStrokes	 10
U+783C.0	fCangjie	MROM
U+783C.0	fCantonese	tung4
U+783C.0	fMandarin	tong2
U+783C.0	fEnglish	concrete
U+783D.0	fUTF8	砽
U+783D.0	fCNS11643	3-656F
U+783D.0	fGB	E-3532
U+783D.0	fR/S	 112.5
U+783D.0	fTotalStrokes	 10
U+783D.0	fCangjie	MRBQ
U+783D.0	fFourCorner	1762
U+783D.0	fCantonese	jung6
U+783E.0	fUTF8	砾
U+783E.0	fGB	0-4079
U+783E.0	fR/S	 112.5
U+783E.0	fTotalStrokes	 10
U+783E.0	fCangjie	MRHVD
U+783E.0	fHakka	 lit8
U+783E.0	fCantonese	lik1; lok6; nik1
U+783E.0	fMandarin	li4; luo4
U+783E.0	fEnglish	simplified form of &#x792B;
U+783F.0	fUTF8	砿
U+783F.0	fGB	E-3533
U+783F.0	fR/S	 112.5
U+783F.0	fTotalStrokes	 10
U+783F.0	fCangjie	XMRII
U+783F.0	fCantonese	kong3
U+783F.0	fMandarin	kuang4; gong3
U+783F.0	fEnglish	 mine; mineral, ore
U+7840.0	fUTF8	础
U+7840.0	fGB	0-3421
U+7840.0	fR/S	 112.5
U+7840.0	fTotalStrokes	 10
U+7840.0	fCangjie	MRUU
U+7840.0	fHakka	 cih3
U+7840.0	fCantonese	co2
U+7840.0	fMandarin	chu3
U+7840.0	fEnglish	simplified form of &#x790E;
U+7841.0	fUTF8	硁
U+7841.0	fGB	8-7D43
U+7841.0	fR/S	 112.5
U+7841.0	fTotalStrokes	 10
U+7841.0	fCangjie	MRNOM
U+7841.0	fCantonese	hang1; hing3
U+7841.0	fMandarin	keng1
U+7841.0	fEnglish	the sound of stones knocking together
U+7842.0	fUTF8	硂
U+7842.0	fCNS11643	3-3762
U+7842.0	fGB	5-5829
U+7842.0	fR/S	 112.6
U+7842.0	fTotalStrokes	 11
U+7842.0	fCangjie	MROMG
U+7842.0	fFourCorner	18610
U+7842.0	fCantonese	cyun4
U+7842.0	fMandarin	quan2
U+7843.0	fUTF8	硃
U+7843.0	fCNS11643	1-5C3E
U+7843.0	fBig5	B2B9
U+7843.0	fGB	1-7927
U+7843.0	fR/S	 112.6
U+7843.0	fTotalStrokes	 11
U+7843.0	fCangjie	MRHJD
U+7843.0	fFourCorner	15690
U+7843.0	fHakka	 zu1; zhu1
U+7843.0	fCantonese	zyu1
U+7843.0	fMandarin	zhu1
U+7843.0	fEnglish	[1] vermilion [2] imperial (the signature and instructions of an emperor were written in red)
U+7844.0	fUTF8	硄
U+7844.0	fCNS11643	4-335D
U+7844.0	fGB	3-5772
U+7844.0	fR/S	 112.6
U+7844.0	fTotalStrokes	 11
U+7844.0	fCangjie	MRFMU
U+7844.0	fFourCorner	19610
U+7844.0	fCantonese	hong1
U+7844.0	fMandarin	kuang1
U+7845.0	fUTF8	硅
U+7845.0	fCNS11643	2-375D
U+7845.0	fBig5	D6BA
U+7845.0	fGB	0-3968
U+7845.0	fR/S	 112.6
U+7845.0	fTotalStrokes	 11
U+7845.0	fCangjie	MRGG
U+7845.0	fFourCorner	14614
U+7845.0	fHakka	 gui1
U+7845.0	fCantonese	gwai1
U+7845.0	fMandarin	gui1; he4; huo4
U+7845.0	fEnglish	silicon
U+7846.0	fUTF8	硆
U+7846.0	fCNS11643	3-3763
U+7846.0	fGB	5-582A
U+7846.0	fR/S	  112.6
U+7846.0	fTotalStrokes	 11
U+7846.0	fCangjie	MROMR
U+7846.0	fFourCorner	18660
U+7846.0	fCantonese	ngap6; ap6 (also)
U+7846.0	fMandarin	e4
U+7847.0	fUTF8	硇
U+7847.0	fCNS11643	3-375D
U+7847.0	fGB	0-6D50
U+7847.0	fR/S	 112.6
U+7847.0	fTotalStrokes	 11
U+7847.0	fCangjie	MRHWK
U+7847.0	fFourCorner	16600
U+7847.0	fHakka	  nau2
U+7847.0	fCantonese	naau4; laau4 (also); kit3; leot6; ngai5; ai5 (also); ngoi6; oi6 (also); gok3; lok6; lok3 (also); haan6
U+7847.0	fMandarin	nao2
U+7847.0	fEnglish	a kind of mineral
U+7848.0	fUTF8	硈
U+7848.0	fCNS11643	2-3756
U+7848.0	fBig5	D6B3
U+7848.0	fGB	3-576E
U+7848.0	fR/S	 112.6
U+7848.0	fTotalStrokes	 11
U+7848.0	fCangjie	MRGR
U+7848.0	fFourCorner	14661
U+7848.0	fHakka	  kat7;  kit8;  git8
U+7848.0	fCantonese	kit3; hat1
U+7848.0	fMandarin	jia2; qia2
U+7848.0	fEnglish	firm; solid, abrupt; sudden
U+7849.0	fUTF8	硉
U+7849.0	fCNS11643	2-3758
U+7849.0	fBig5	D6B5
U+7849.0	fGB	3-5777
U+7849.0	fR/S	 112.6
U+7849.0	fTotalStrokes	 11
U+7849.0	fCangjie	MRLQ
U+7849.0	fFourCorner	15607
U+7849.0	fHakka	 lut8; lut7
U+7849.0	fCantonese	leot6
U+7849.0	fMandarin	lu4
U+7849.0	fEnglish	[1] graves [2] stones
U+784A.0	fUTF8	硊
U+784A.0	fCNS11643	2-375A
U+784A.0	fBig5	D6B7
U+784A.0	fGB	3-5776
U+784A.0	fR/S	 112.6
U+784A.0	fTotalStrokes	 11
U+784A.0	fCangjie	MRNMU
U+784A.0	fFourCorner	17612
U+784A.0	fCantonese	kwai3; ngai5
U+784A.0	fMandarin	wei3; kui4
U+784B.0	fUTF8	硋
U+784B.0	fCNS11643	3-375F
U+784B.0	fGB	5-582D
U+784B.0	fR/S	 112.6
U+784B.0	fTotalStrokes	 11
U+784B.0	fCangjie	MRYVO
U+784B.0	fFourCorner	10680
U+784B.0	fMandarin	ai4
U+784C.0	fUTF8	硌
U+784C.0	fCNS11643	2-375B
U+784C.0	fBig5	D6B8
U+784C.0	fGB	0-6D51
U+784C.0	fR/S	 112.6
U+784C.0	fTotalStrokes	 11
U+784C.0	fCangjie	MRHER
U+784C.0	fFourCorner	17664
U+784C.0	fHakka	 lok8; lok7
U+784C.0	fCantonese	gok3; lok6; lok3
U+784C.0	fMandarin	luo4; ge4; li4
U+784C.0	fEnglish	[1] damage or injuries caused by being squeezed or pressed on a rough surface [2] rocks; boulders
U+784D.0	fUTF8	硍
U+784D.0	fCNS11643	2-3759
U+784D.0	fBig5	D6B6
U+784D.0	fGB	3-5778
U+784D.0	fR/S	 112.6
U+784D.0	fTotalStrokes	 11
U+784D.0	fCangjie	MRAV
U+784D.0	fFourCorner	17632
U+784D.0	fHakka	 han5; han6
U+784D.0	fCantonese	haan6; gan1
U+784D.0	fMandarin	ken4; jian1; xian4
U+784D.0	fEnglish	strike
U+784E.0	fUTF8	硎
U+784E.0	fCNS11643	1-5C3F
U+784E.0	fBig5	B2BA
U+784E.0	fGB	0-6D4A
U+784E.0	fR/S	 112.6
U+784E.0	fTotalStrokes	 11
U+784E.0	fCangjie	MRMTN
U+784E.0	fFourCorner	12600
U+784E.0	fHakka	 hin2
U+784E.0	fCantonese	jing4
U+784E.0	fMandarin	xing2; yan2; keng1
U+784E.0	fEnglish	whetstone
U+784F.0	fUTF8	硏
U+784F.0	fCNS11643	3-3761
U+784F.0	fGB	E-3534
U+784F.0	fR/S	 112.6
U+784F.0	fTotalStrokes	 11
U+784F.0	fCangjie	MRMJJ
U+784F.0	fFourCorner	61140
U+784F.0	fHakka	  ngan1; ngan2; ngien1; ngion1; ngan5; jen2; ngien2; ngan3; nion1
U+784F.0	fCantonese	jin4; jin6; ngaan4; aan4 (also); gin2; gaan2 (also)
U+784F.0	fMandarin	jian3; yan2; yan4
U+784F.0	fEnglish	grind
U+7850.0	fUTF8	硐
U+7850.0	fCNS11643	2-375E
U+7850.0	fBig5	D6BB
U+7850.0	fGB	0-6D4F
U+7850.0	fR/S	 112.6
U+7850.0	fTotalStrokes	 11
U+7850.0	fCangjie	MRBMR
U+7850.0	fFourCorner	17620
U+7850.0	fHakka	  tung2
U+7850.0	fCantonese	dung6
U+7850.0	fMandarin	tong2; dong4; dong3
U+7850.0	fEnglish	 plane; smooth away
U+7851.0	fUTF8	硑
U+7851.0	fCNS11643	3-3760
U+7851.0	fGB	5-582E
U+7851.0	fR/S	 112.6
U+7851.0	fTotalStrokes	 11
U+7851.0	fCangjie	MRTT
U+7851.0	fFourCorner	18640
U+7851.0	fCantonese	ping1
U+7851.0	fMandarin	peng1
U+7852.0	fUTF8	硒
U+7852.0	fCNS11643	2-3757
U+7852.0	fBig5	D6B4
U+7852.0	fGB	0-4E78
U+7852.0	fR/S	 112.6
U+7852.0	fTotalStrokes	 11
U+7852.0	fCangjie	MRMCW
U+7852.0	fFourCorner	11660
U+7852.0	fCantonese	sai1
U+7852.0	fMandarin	xi1
U+7852.0	fEnglish	selenium (Se)
U+7853.0	fUTF8	硓
U+7853.0	fCNS11643	3-6570
U+7853.0	fGB	E-3535
U+7853.0	fR/S	 112.6
U+7853.0	fTotalStrokes	 11
U+7853.0	fCangjie	MRJKP
U+7853.0	fFourCorner	1461
U+7853.0	fCantonese	lou5
U+7854.0	fUTF8	硔
U+7854.0	fCNS11643	3-6571
U+7854.0	fGB	E-3536
U+7854.0	fR/S	 112.6
U+7854.0	fTotalStrokes	 11
U+7854.0	fCangjie	MRTC
U+7854.0	fFourCorner	1468
U+7854.0	fCantonese	hung4
U+7854.0	fMandarin	hong2
U+7855.0	fUTF8	硕
U+7855.0	fGB	0-4B36
U+7855.0	fR/S	 112.6
U+7855.0	fTotalStrokes	 11
U+7855.0	fCangjie	MRMBO
U+7855.0	fHakka	 shak8; shok7
U+7855.0	fCantonese	sek6
U+7855.0	fMandarin	shuo4; shi2
U+7855.0	fEnglish	simplified form of &#x78A9;
U+7856.0	fUTF8	硖
U+7856.0	fGB	0-6D4C
U+7856.0	fR/S	 112.6
U+7856.0	fTotalStrokes	 11
U+7856.0	fCangjie	MRKT
U+7856.0	fHakka	 hiap8; giap7
U+7856.0	fCantonese	haap6 (also); hap6; gip3
U+7856.0	fMandarin	xia2
U+7856.0	fEnglish	simplified form of &#x7864;
U+7857.0	fUTF8	硗
U+7857.0	fGB	0-6D4D
U+7857.0	fR/S	 112.6
U+7857.0	fTotalStrokes	 11
U+7857.0	fCangjie	MRJPU
U+7857.0	fHakka	 kau1; ngiau2
U+7857.0	fCantonese	haau1
U+7857.0	fMandarin	qiao1; ao2; qiao3; yao3; yao4; yue4
U+7857.0	fEnglish	simplified form of &#x78FD;
U+7858.0	fUTF8	硘
U+7858.0	fCNS11643	3-6572
U+7858.0	fGB	E-3537
U+7858.0	fR/S	 112.6
U+7858.0	fTotalStrokes	 11
U+7858.0	fCangjie	MRWR
U+7858.0	fFourCorner	1660
U+7858.0	fCantonese	wui4
U+7859.0	fUTF8	硙
U+7859.0	fGB	8-2E64
U+7859.0	fR/S	 112.6
U+7859.0	fTotalStrokes	 11
U+7859.0	fCangjie	MRUSU
U+7859.0	fCantonese	wai3; oi4; ngoi4 (also); hoi2 (also); wui1
U+7859.0	fMandarin	wei4; ai2; wei2
U+7859.0	fEnglish	 stone mill grind break apart
U+785A.0	fUTF8	硚
U+785A.0	fGB	8-7D44
U+785A.0	fR/S	 112.12; 112.6
U+785A.0	fTotalStrokes	 11
U+785A.0	fCangjie	MRHKL
U+785A.0	fCantonese	kiu4
U+785A.0	fMandarin	qiao2
U+785B.0	fUTF8	硛
U+785B.0	fGB	1-7D57
U+785B.0	fR/S	 112.6
U+785B.0	fTotalStrokes	 11
U+785B.0	fCangjie	MRYLC
U+785B.0	fCantonese	zik1
U+785C.0	fUTF8	硜
U+785C.0	fCNS11643	2-3E4C
U+785C.0	fBig5	DAC8
U+785C.0	fGB	3-576B
U+785C.0	fR/S	 112.7
U+785C.0	fTotalStrokes	 12
U+785C.0	fCangjie	MRMVM
U+785C.0	fFourCorner	11611
U+785C.0	fHakka	 gin1; gang1
U+785C.0	fCantonese	hang1; hing3
U+785C.0	fMandarin	keng1; qing4; xing2
U+785C.0	fEnglish	the sound of stones knocking together; sound of pebbles or stones rubbing together
U+785D.0	fUTF8	硝
U+785D.0	fCNS11643	1-607C
U+785D.0	fBig5	B576
U+785D.0	fGB	0-4F75
U+785D.0	fR/S	 112.7
U+785D.0	fTotalStrokes	 12
U+785D.0	fCangjie	MRFB
U+785D.0	fFourCorner	19627
U+785D.0	fHakka	  siau1; seu1
U+785D.0	fCantonese	siu1
U+785D.0	fMandarin	xiao1; qiao4
U+785D.0	fEnglish	[1] saltpeter; niter [2] tan leather
U+785E.0	fUTF8	硞
U+785E.0	fCNS11643	2-3E54
U+785E.0	fBig5	DAD0
U+785E.0	fGB	3-5821
U+785E.0	fR/S	 112.7
U+785E.0	fTotalStrokes	 12
U+785E.0	fCangjie	MRHGR
U+785E.0	fFourCorner	14661
U+785E.0	fHakka	 kok8; kok7; kwat8
U+785E.0	fCantonese	kok3; kuk1; kok6
U+785E.0	fMandarin	que4; ke4; ku4
U+785E.0	fEnglish	[1] craggy [2] rocky
U+785F.0	fUTF8	硟
U+785F.0	fCNS11643	3-3D5C
U+785F.0	fGB	3-5775
U+785F.0	fR/S	 112.7; 112.8
U+785F.0	fTotalStrokes	 12; 13
U+785F.0	fCangjie	MRNKM
U+785F.0	fFourCorner	12640
U+785F.0	fCantonese	cin3; sin3
U+785F.0	fMandarin	chan4
U+785F.0	fEnglish	to slip; to work and polish gems
U+7860.0	fUTF8	硠
U+7860.0	fCNS11643	2-3E49
U+7860.0	fBig5	DAC5
U+7860.0	fGB	3-5824
U+7860.0	fR/S	 112.7
U+7860.0	fTotalStrokes	 12
U+7860.0	fCangjie	MRIAV
U+7860.0	fFourCorner	17632
U+7860.0	fHakka	 long2
U+7860.0	fCantonese	long4
U+7860.0	fMandarin	lang3; dang2; lang2; lang4
U+7860.0	fEnglish	[1] rumbling of stones [2] roll of a drum
U+7861.0	fUTF8	硡
U+7861.0	fCNS11643	6-4C2F
U+7861.0	fGB	3-5823
U+7861.0	fR/S	  112.7
U+7861.0	fTotalStrokes	 12
U+7861.0	fCangjie	MRJKI
U+7861.0	fFourCorner	13630
U+7861.0	fCantonese	wang4; gwang1 (also)
U+7861.0	fMandarin	hong2; hong1
U+7862.0	fUTF8	硢
U+7862.0	fCNS11643	2-3E55
U+7862.0	fBig5	DAD1
U+7862.0	fGB	5-5838
U+7862.0	fR/S	 112.7
U+7862.0	fTotalStrokes	 12
U+7862.0	fCangjie	MROMD
U+7862.0	fFourCorner	18694
U+7862.0	fCantonese	jyu4
U+7862.0	fMandarin	yu2
U+7863.0	fUTF8	硣
U+7863.0	fCNS11643	3-3D59
U+7863.0	fGB	E-3538
U+7863.0	fR/S	 112.7
U+7863.0	fTotalStrokes	 12
U+7863.0	fCangjie	MRJKD
U+7863.0	fFourCorner	14640
U+7863.0	fCantonese	haau1
U+7863.0	fMandarin	xiao1
U+7864.0	fUTF8	硤
U+7864.0	fCNS11643	2-3E4A
U+7864.0	fBig5	DAC6
U+7864.0	fGB	1-6D4C
U+7864.0	fR/S	 112.7
U+7864.0	fTotalStrokes	 12
U+7864.0	fCangjie	MRKOO
U+7864.0	fFourCorner	14638
U+7864.0	fHakka	 giap7;  hiap8; kiap8
U+7864.0	fCantonese	haap6 (also); gip3; hap6
U+7864.0	fMandarin	xia2
U+7864.0	fEnglish	a town in Hebei province
U+7865.0	fUTF8	硥
U+7865.0	fCNS11643	2-3E4B
U+7865.0	fBig5	DAC7
U+7865.0	fGB	3-577C
U+7865.0	fR/S	 112.7
U+7865.0	fTotalStrokes	 12
U+7865.0	fCangjie	MRIUH
U+7865.0	fFourCorner	13612
U+7865.0	fCantonese	mong5; pong5
U+7865.0	fMandarin	mang3
U+7866.0	fUTF8	硦
U+7866.0	fCNS11643	3-3D5D
U+7866.0	fGB	3-5779
U+7866.0	fR/S	 112.7
U+7866.0	fTotalStrokes	 12
U+7866.0	fCangjie	MRMGT
U+7866.0	fFourCorner	11640
U+7866.0	fCantonese	lok6
U+7866.0	fMandarin	long4
U+7867.0	fUTF8	硧
U+7867.0	fCNS11643	4-3954
U+7867.0	fGB	5-583C
U+7867.0	fR/S	  112.7
U+7867.0	fTotalStrokes	 12
U+7867.0	fCangjie	MRNIB
U+7867.0	fFourCorner	17620
U+7867.0	fCantonese	jung5
U+7867.0	fMandarin	yong3; tong2
U+7867.0	fEnglish	[1] a grindstone [2] same as &#x7850;
U+7868.0	fUTF8	硨
U+7868.0	fCNS11643	2-3E53
U+7868.0	fBig5	DACF
U+7868.0	fGB	1-6D3A
U+7868.0	fR/S	 112.7
U+7868.0	fTotalStrokes	 12
U+7868.0	fCangjie	MRJWJ
U+7868.0	fFourCorner	15600
U+7868.0	fHakka	 cha1; ca1
U+7868.0	fCantonese	ce1
U+7868.0	fMandarin	che1; ju1
U+7868.0	fEnglish	giant clam; Tridacna gigas
U+7869.0	fUTF8	硩
U+7869.0	fCNS11643	2-3E52
U+7869.0	fBig5	DACE
U+7869.0	fGB	3-577D
U+7869.0	fR/S	 112.7
U+7869.0	fTotalStrokes	 12
U+7869.0	fCangjie	QLMR
U+7869.0	fFourCorner	52601
U+7869.0	fHakka	 chat8; cat8; chet8
U+7869.0	fCantonese	zit3; cit3
U+7869.0	fMandarin	che4; ti4
U+7869.0	fEnglish	 drive off an ill-omened bird
U+786A.0	fUTF8	硪
U+786A.0	fCNS11643	2-3E4F
U+786A.0	fBig5	DACB
U+786A.0	fGB	0-6D52
U+786A.0	fR/S	 112.7
U+786A.0	fTotalStrokes	 12
U+786A.0	fCangjie	MRHQI
U+786A.0	fFourCorner	13650
U+786A.0	fHakka	  ngo2
U+786A.0	fCantonese	ngo4; ngo6
U+786A.0	fMandarin	e2; yi3; wo4; e3
U+786A.0	fEnglish	rocky cliff
U+786B.0	fUTF8	硫
U+786B.0	fCNS11643	1-5C3D
U+786B.0	fBig5	B2B8
U+786B.0	fGB	0-4172
U+786B.0	fR/S	 112.7
U+786B.0	fTotalStrokes	 12
U+786B.0	fCangjie	MRYIU
U+786B.0	fFourCorner	10613
U+786B.0	fHakka	  liu2; liu3
U+786B.0	fCantonese	lau4
U+786B.0	fMandarin	liu2; chu4
U+786B.0	fEnglish	sulfur; sulfur
U+786C.0	fUTF8	硬
U+786C.0	fCNS11643	1-607D
U+786C.0	fBig5	B577
U+786C.0	fGB	0-5332
U+786C.0	fR/S	 112.7
U+786C.0	fTotalStrokes	 12
U+786C.0	fCangjie	MRMLK
U+786C.0	fFourCorner	11646
U+786C.0	fHakka	 ngang5; ngang3; ngang6; ngiang5; ngiang6; ngiang3
U+786C.0	fCantonese	ngaang6; aang6 (also); ngaang2 (also); aang2 (also)
U+786C.0	fMandarin	ying4; geng3
U+786C.0	fEnglish	[1] hard; stiff; solid; firm [2] stiff; rigid; inflexible; obstinate; very insistent; unyielding [3] harden; solidify; stiffen [4] by force [5] manage to do something in a forced manner [6] good (quality) [7] able (person)
U+786D.0	fUTF8	硭
U+786D.0	fCNS11643	2-3E4D
U+786D.0	fBig5	DAC9
U+786D.0	fGB	0-6D4B
U+786D.0	fR/S	 112.7
U+786D.0	fTotalStrokes	 12
U+786D.0	fCangjie	MRTYV
U+786D.0	fFourCorner	14610
U+786D.0	fHakka	  mong2
U+786D.0	fCantonese	mong4
U+786D.0	fMandarin	mang2
U+786D.0	fEnglish	sodium sulfate
U+786E.0	fUTF8	确
U+786E.0	fCNS11643	2-3E50
U+786E.0	fBig5	DACC
U+786E.0	fGB	0-4837
U+786E.0	fR/S	 112.7
U+786E.0	fTotalStrokes	 12
U+786E.0	fCangjie	MRNBG
U+786E.0	fFourCorner	17627
U+786E.0	fHakka	 kok7; hok7
U+786E.0	fCantonese	kok3
U+786E.0	fMandarin	que4
U+786E.0	fEnglish	[1] hard stone [2] barren; unproductive (land) [3] simplified form of &#x78BA;
U+786F.0	fUTF8	硯
U+786F.0	fCNS11643	1-607E
U+786F.0	fBig5	B578
U+786F.0	fGB	1-5162
U+786F.0	fR/S	 112.7
U+786F.0	fTotalStrokes	 12
U+786F.0	fCangjie	MRBUU
U+786F.0	fFourCorner	16610
U+786F.0	fHakka	 ngan5;  jen5;  jan5; gan3; ngan3; ren5
U+786F.0	fCantonese	jin6; jin2 (also)
U+786F.0	fMandarin	yan4
U+786F.0	fEnglish	 inkstone
U+7870.0	fUTF8	硰
U+7870.0	fCNS11643	2-3E51
U+7870.0	fBig5	DACD
U+7870.0	fGB	E-3539
U+7870.0	fR/S	 112.7
U+7870.0	fTotalStrokes	 12
U+7870.0	fCangjie	EHMR
U+7870.0	fFourCorner	39601
U+7870.0	fCantonese	saa1; co2
U+7870.0	fMandarin	sha1; cuo3; suo4; zuo3; zuo4
U+7871.0	fUTF8	硱
U+7871.0	fCNS11643	2-3E4E
U+7871.0	fBig5	DACA
U+7871.0	fGB	5-5834
U+7871.0	fR/S	 112.7
U+7871.0	fTotalStrokes	 12
U+7871.0	fCangjie	MRWD
U+7871.0	fFourCorner	16600
U+7871.0	fCantonese	kwan2
U+7871.0	fMandarin	kun3
U+7872.0	fUTF8	硲
U+7872.0	fGB	E-353A
U+7872.0	fR/S	 112.7
U+7872.0	fTotalStrokes	 12
U+7872.0	fCangjie	MRCOR
U+7872.0	fCantonese	juk6
U+7872.0	fMandarin	yu4
U+7873.0	fUTF8	硳
U+7873.0	fGB	1-7D58
U+7873.0	fR/S	 112.7
U+7873.0	fTotalStrokes	 12
U+7873.0	fCangjie	MRGLC
U+7873.0	fCantonese	cek3
U+7874.0	fUTF8	硴
U+7874.0	fGB	E-353B
U+7874.0	fR/S	 112.7
U+7874.0	fTotalStrokes	 12
U+7874.0	fCangjie	MRTOP
U+7874.0	fCantonese	faa1
U+7875.0	fUTF8	硵
U+7875.0	fGB	8-7D45
U+7875.0	fR/S	 112.7
U+7875.0	fTotalStrokes	 12
U+7875.0	fCangjie	MRYWK
U+7875.0	fCantonese	lou5
U+7875.0	fMandarin	lu3
U+7876.0	fUTF8	硶
U+7876.0	fCNS11643	3-3D5A
U+7876.0	fGB	5-5837
U+7876.0	fR/S	  112.7
U+7876.0	fTotalStrokes	 12
U+7876.0	fCangjie	MRUON
U+7876.0	fFourCorner	12620
U+7876.0	fCantonese	cam2
U+7876.0	fMandarin	chen3; cen2
U+7877.0	fUTF8	硷
U+7877.0	fGB	0-3C6F
U+7877.0	fR/S	 112.7
U+7877.0	fTotalStrokes	 12
U+7877.0	fCangjie	MROMM
U+7877.0	fHakka	 gan3
U+7877.0	fCantonese	gaan2
U+7877.0	fMandarin	jian3; qian1; yan4; zuan1
U+7877.0	fEnglish	simplified form of &#x9E7C;
U+7878.0	fUTF8	硸
U+7878.0	fCNS11643	3-4331
U+7878.0	fGB	3-5831
U+7878.0	fR/S	  112.8
U+7878.0	fTotalStrokes	 13
U+7878.0	fCangjie	MRUMJ
U+7878.0	fFourCorner	12640
U+7878.0	fCantonese	joek6
U+7878.0	fMandarin	nuue4; ne4
U+7878.0	fEnglish	thick-lipped; a grindstone
U+7879.0	fUTF8	硹
U+7879.0	fCNS11643	2-4554
U+7879.0	fBig5	DEEE
U+7879.0	fGB	5-583E
U+7879.0	fR/S	 112.8
U+7879.0	fTotalStrokes	 13
U+7879.0	fCangjie	MRDCI
U+7879.0	fFourCorner	18632
U+7879.0	fCantonese	sung1
U+7879.0	fMandarin	song1
U+787A.0	fUTF8	硺
U+787A.0	fCNS11643	3-432D
U+787A.0	fGB	E-353C
U+787A.0	fR/S	 112.8
U+787A.0	fTotalStrokes	 13
U+787A.0	fCangjie	MRMSO
U+787A.0	fFourCorner	11630
U+787A.0	fCantonese	zoek3
U+787A.0	fMandarin	zhuo2
U+787B.0	fUTF8	硻
U+787B.0	fCNS11643	2-4558
U+787B.0	fBig5	DEF2
U+787B.0	fGB	5-5778
U+787B.0	fR/S	 112.8
U+787B.0	fTotalStrokes	 13
U+787B.0	fCangjie	SEMR
U+787B.0	fFourCorner	77601
U+787B.0	fCantonese	hang1; paang4; pang4 (also); jim1; zeoi6; hung1; gei1; cang4; kong3; kei4; zung3
U+787B.0	fMandarin	keng1; keng3
U+787C.0	fUTF8	硼
U+787C.0	fCNS11643	1-6555
U+787C.0	fBig5	B84E
U+787C.0	fGB	0-4570
U+787C.0	fR/S	 112.8
U+787C.0	fTotalStrokes	 13
U+787C.0	fCangjie	MRBB
U+787C.0	fFourCorner	17620
U+787C.0	fHakka	 pen2; pin2; pang2
U+787C.0	fCantonese	pang4; paang1; paang4
U+787C.0	fMandarin	peng2
U+787C.0	fEnglish	[1] boron [2] borax
U+787C.1	fUTF8	硼
U+787C.1	fCNS11643	1-6555
U+787C.1	fBig5	B84E
U+787C.1	fGB	0-4570
U+787C.1	fR/S	 112.8
U+787C.1	fTotalStrokes	 13
U+787C.1	fCangjie	MRBB
U+787C.1	fFourCorner	17620
U+787C.1	fMandarin	peng1
U+787C.1	fEnglish	the sound of splashing
U+787D.0	fUTF8	硽
U+787D.0	fCNS11643	3-432E
U+787D.0	fGB	3-582C
U+787D.0	fR/S	  112.8
U+787D.0	fTotalStrokes	 13
U+787D.0	fCangjie	MRKLU
U+787D.0	fFourCorner	14610
U+787D.0	fMandarin	yan3
U+787E.0	fUTF8	硾
U+787E.0	fCNS11643	2-4C38
U+787E.0	fBig5	E2F0
U+787E.0	fGB	3-5833
U+787E.0	fR/S	 112.9; 112.8
U+787E.0	fTotalStrokes	 14
U+787E.0	fCangjie	MRHJM
U+787E.0	fFourCorner	12614
U+787E.0	fHakka	 ch'ui6; ch'ui2; cui2; cui5; ch'ui5; chui2
U+787E.0	fCantonese	zeoi6
U+787E.0	fMandarin	zhui4; duo3
U+787E.0	fEnglish	weight
U+787F.0	fUTF8	硿
U+787F.0	fCNS11643	1-6558
U+787F.0	fBig5	B851
U+787F.0	fGB	3-583B
U+787F.0	fR/S	 112.8
U+787F.0	fTotalStrokes	 13
U+787F.0	fCangjie	MRJCM
U+787F.0	fFourCorner	13611
U+787F.0	fHakka	  kung1; kung5
U+787F.0	fCantonese	hung1
U+787F.0	fMandarin	kong1
U+787F.0	fEnglish	deep pools and cataracts
U+7880.0	fUTF8	碀
U+7880.0	fCNS11643	2-4556
U+7880.0	fBig5	DEF0
U+7880.0	fGB	5-582C
U+7880.0	fR/S	 112.8
U+7880.0	fTotalStrokes	 13
U+7880.0	fCangjie	MRBSD
U+7880.0	fFourCorner	12657
U+7880.0	fCantonese	cang4
U+7880.0	fMandarin	ceng2
U+7881.0	fUTF8	碁
U+7881.0	fCNS11643	3-4337
U+7881.0	fGB	E-353D
U+7881.0	fR/S	  112.8
U+7881.0	fTotalStrokes	 13
U+7881.0	fCangjie	TCMR
U+7881.0	fFourCorner	44601
U+7881.0	fHakka	  ki2
U+7881.0	fCantonese	kei4
U+7881.0	fMandarin	qi2
U+7881.0	fEnglish	[1] an oriental game played on a checkered board, like Chinese chess, go, etc. [2] any piece used in the game of chess
U+7882.0	fUTF8	碂
U+7882.0	fCNS11643	3-4329
U+7882.0	fGB	3-583A
U+7882.0	fR/S	  112.8
U+7882.0	fTotalStrokes	 13
U+7882.0	fCangjie	MRJMF
U+7882.0	fFourCorner	13690
U+7882.0	fMandarin	zong4
U+7883.0	fUTF8	碃
U+7883.0	fCNS11643	2-4553
U+7883.0	fBig5	DEED
U+7883.0	fGB	3-5825
U+7883.0	fR/S	 112.8
U+7883.0	fTotalStrokes	 13
U+7883.0	fCangjie	MRQMB
U+7883.0	fFourCorner	15627
U+7883.0	fCantonese	cing3
U+7883.0	fMandarin	qing4
U+7884.0	fUTF8	碄
U+7884.0	fCNS11643	2-454E
U+7884.0	fBig5	DEE8
U+7884.0	fGB	E-353E
U+7884.0	fR/S	 112.8
U+7884.0	fTotalStrokes	 13
U+7884.0	fCangjie	MRDD
U+7884.0	fFourCorner	14690
U+7884.0	fHakka	 lim2
U+7884.0	fCantonese	lam4
U+7884.0	fMandarin	lin2
U+7885.0	fUTF8	碅
U+7885.0	fCNS11643	2-4550
U+7885.0	fBig5	DEEA
U+7885.0	fGB	3-5832
U+7885.0	fR/S	 112.8
U+7885.0	fTotalStrokes	 13
U+7885.0	fCangjie	MRWHD
U+7885.0	fFourCorner	16600
U+7885.0	fCantonese	kwan1
U+7885.0	fMandarin	jun1
U+7886.0	fUTF8	碆
U+7886.0	fCNS11643	2-4551
U+7886.0	fBig5	DEEB
U+7886.0	fGB	3-5839
U+7886.0	fR/S	 112.8
U+7886.0	fTotalStrokes	 13
U+7886.0	fCangjie	EEMR
U+7886.0	fFourCorner	34601
U+7886.0	fHakka	  bo1;  po2
U+7886.0	fCantonese	bo1
U+7886.0	fMandarin	bo1; pan2; po2
U+7886.0	fEnglish	arrow-tip
U+7887.0	fUTF8	碇
U+7887.0	fCNS11643	2-454A
U+7887.0	fBig5	DEE4
U+7887.0	fGB	0-6D56
U+7887.0	fR/S	 112.8
U+7887.0	fTotalStrokes	 13
U+7887.0	fCangjie	MRJMO
U+7887.0	fFourCorner	13681
U+7887.0	fHakka	  dang5;  den5; tin6; tin3; tin5
U+7887.0	fCantonese	ding3; ding6
U+7887.0	fMandarin	ding4
U+7887.0	fEnglish	 anchor (ship)
U+7888.0	fUTF8	碈
U+7888.0	fCNS11643	3-4335
U+7888.0	fGB	5-5841
U+7888.0	fR/S	 112.8
U+7888.0	fTotalStrokes	 13
U+7888.0	fCangjie	MRHPA
U+7888.0	fFourCorner	12660
U+7888.0	fCantonese	man4
U+7888.0	fMandarin	min2
U+7889.0	fUTF8	碉
U+7889.0	fCNS11643	1-6554
U+7889.0	fBig5	B84D
U+7889.0	fGB	0-356F
U+7889.0	fR/S	 112.8
U+7889.0	fTotalStrokes	 13
U+7889.0	fCangjie	MRBGR
U+7889.0	fFourCorner	17620
U+7889.0	fHakka	  diau1
U+7889.0	fCantonese	diu1
U+7889.0	fMandarin	diao1
U+7889.0	fEnglish	 room made of stone; stone chamber
U+788A.0	fUTF8	碊
U+788A.0	fCNS11643	3-432B
U+788A.0	fGB	3-575B
U+788A.0	fR/S	  112.8
U+788A.0	fTotalStrokes	 13
U+788A.0	fCangjie	MRII
U+788A.0	fFourCorner	13650
U+788A.0	fCantonese	zin1
U+788A.0	fMandarin	jian1
U+788B.0	fUTF8	碋
U+788B.0	fCNS11643	3-4330
U+788B.0	fGB	E-353F
U+788B.0	fR/S	  112.8
U+788B.0	fTotalStrokes	 13
U+788B.0	fCangjie	MRUMR
U+788B.0	fFourCorner	12620
U+788B.0	fCantonese	ho6
U+788B.0	fMandarin	he4
U+788C.0	fUTF8	碌
U+788C.0	fCNS11643	1-6553
U+788C.0	fBig5	B84C
U+788C.0	fGB	0-4235
U+788C.0	fR/S	 112.8
U+788C.0	fTotalStrokes	 13
U+788C.0	fCangjie	MRVNE
U+788C.0	fFourCorner	17632
U+788C.0	fHakka	  luk8; liuk8; luk7
U+788C.0	fCantonese	luk1; luk6
U+788C.0	fMandarin	lu4; liu4; luo4
U+788C.0	fEnglish	[1] mediocre; common [2] busy; occupied [3] a kind of stone roller
U+788D.0	fUTF8	碍
U+788D.0	fCNS11643	3-4334
U+788D.0	fGB	0-302D
U+788D.0	fR/S	 112.8
U+788D.0	fTotalStrokes	 13
U+788D.0	fCangjie	MRAMI
U+788D.0	fFourCorner	16640
U+788D.0	fHakka	  ngoi5; ngoi6; ngoi3; ngai6; ngai5; ngoe3
U+788D.0	fCantonese	ngoi6; oi6 (also)
U+788D.0	fMandarin	ai4; yi2
U+788D.0	fEnglish	simplified form of &#x7919;
U+788E.0	fUTF8	碎
U+788E.0	fCNS11643	1-654F
U+788E.0	fBig5	B848
U+788E.0	fGB	0-4B69
U+788E.0	fR/S	 112.8
U+788E.0	fTotalStrokes	 13
U+788E.0	fCangjie	MRYOJ
U+788E.0	fFourCorner	10648
U+788E.0	fHakka	  sui5
U+788E.0	fCantonese	seoi3; seoi2 (also)
U+788E.0	fMandarin	sui4
U+788E.0	fEnglish	[1] broken; break to pieces; smash; busted; torn [2] trivial; unimportant; trifling [3] garrulous; gabby
U+788F.0	fUTF8	碏
U+788F.0	fCNS11643	2-454D
U+788F.0	fBig5	DEE7
U+788F.0	fGB	3-5827
U+788F.0	fR/S	 112.8
U+788F.0	fTotalStrokes	 13
U+788F.0	fCangjie	MRTA
U+788F.0	fFourCorner	14661
U+788F.0	fHakka	 siak7; sok7; sit7
U+788F.0	fCantonese	coek3; zoek3
U+788F.0	fMandarin	que4; xi2
U+788F.0	fEnglish	colored; many-colored (stone)
U+7890.0	fUTF8	碐
U+7890.0	fCNS11643	3-432A
U+7890.0	fGB	3-5826
U+7890.0	fR/S	  112.8
U+7890.0	fTotalStrokes	 13
U+7890.0	fCangjie	MRGCE
U+7890.0	fFourCorner	14640
U+7890.0	fCantonese	ling4
U+7890.0	fMandarin	ling2; leng2
U+7891.0	fUTF8	碑
U+7891.0	fCNS11643	1-6556
U+7891.0	fBig5	B84F
U+7891.0	fGB	0-312E
U+7891.0	fR/S	 112.8
U+7891.0	fTotalStrokes	 13
U+7891.0	fCangjie	MRHWJ
U+7891.0	fFourCorner	16640
U+7891.0	fHakka	  bi1
U+7891.0	fCantonese	bei1
U+7891.0	fMandarin	bei1
U+7891.0	fEnglish	 stone tablet
U+7892.0	fUTF8	碒
U+7892.0	fCNS11643	3-4336
U+7892.0	fGB	5-5840
U+7892.0	fR/S	  112.8
U+7892.0	fTotalStrokes	 13
U+7892.0	fCangjie	MRC
U+7892.0	fFourCorner	18610
U+7892.0	fHakka	 ngim2
U+7892.0	fCantonese	jam4
U+7892.0	fMandarin	yin2
U+7892.0	fEnglish	high ridges of cliffs
U+7893.0	fUTF8	碓
U+7893.0	fCNS11643	1-6557
U+7893.0	fBig5	B850
U+7893.0	fGB	0-6D54
U+7893.0	fR/S	 112.8
U+7893.0	fTotalStrokes	 13
U+7893.0	fCangjie	MROG
U+7893.0	fFourCorner	10614
U+7893.0	fHakka	  doi5
U+7893.0	fCantonese	deoi3
U+7893.0	fMandarin	dui4; dui1
U+7893.0	fEnglish	 pestle (for husking grain)
U+7894.0	fUTF8	碔
U+7894.0	fCNS11643	2-454C
U+7894.0	fBig5	DEE6
U+7894.0	fGB	5-583D
U+7894.0	fR/S	 112.8
U+7894.0	fTotalStrokes	 13
U+7894.0	fCangjie	MRMPM
U+7894.0	fFourCorner	13640
U+7894.0	fHakka	 wu3; vu3
U+7894.0	fCantonese	mou5
U+7894.0	fMandarin	wu3
U+7894.0	fEnglish	1/2 pr stone
U+7895.0	fUTF8	碕
U+7895.0	fCNS11643	2-454F
U+7895.0	fBig5	DEE9
U+7895.0	fGB	3-582B
U+7895.0	fR/S	 112.8
U+7895.0	fTotalStrokes	 13
U+7895.0	fCangjie	MRKMR
U+7895.0	fFourCorner	14621
U+7895.0	fHakka	  ki2
U+7895.0	fCantonese	kei4
U+7895.0	fMandarin	qi2; qi1; qi3
U+7895.0	fEnglish	[1] craggy shore [2] stone bridge or jetty
U+7896.0	fUTF8	碖
U+7896.0	fCNS11643	2-4557
U+7896.0	fBig5	DEF1
U+7896.0	fGB	3-5755
U+7896.0	fR/S	 112.8
U+7896.0	fTotalStrokes	 13
U+7896.0	fCangjie	MROMB
U+7896.0	fFourCorner	18627
U+7896.0	fHakka	 lun3; lun2
U+7896.0	fCantonese	leon5; leon4; leon6
U+7896.0	fMandarin	lun4; lun2; lun3
U+7896.0	fEnglish	rocky; reefs
U+7897.0	fUTF8	碗
U+7897.0	fCNS11643	1-6551
U+7897.0	fBig5	B84A
U+7897.0	fGB	0-4D6B
U+7897.0	fR/S	 112.8
U+7897.0	fTotalStrokes	 13
U+7897.0	fCangjie	MRJNU
U+7897.0	fFourCorner	13612
U+7897.0	fHakka	  won3; wan3; wun3; von3; van3
U+7897.0	fCantonese	wun2
U+7897.0	fMandarin	wan3
U+7897.0	fEnglish	 bowl (small)
U+7898.0	fUTF8	碘
U+7898.0	fCNS11643	1-6552
U+7898.0	fBig5	B84B
U+7898.0	fGB	0-3562
U+7898.0	fR/S	 112.8
U+7898.0	fTotalStrokes	 13
U+7898.0	fCangjie	MRTBC
U+7898.0	fFourCorner	15681
U+7898.0	fHakka	 den3
U+7898.0	fCantonese	din2; din1
U+7898.0	fMandarin	dian3
U+7898.0	fEnglish	iodine
U+7899.0	fUTF8	碙
U+7899.0	fCNS11643	2-4555
U+7899.0	fBig5	DEEF
U+7899.0	fGB	3-5750
U+7899.0	fR/S	 112.8
U+7899.0	fTotalStrokes	 13
U+7899.0	fCangjie	MRBTU
U+7899.0	fFourCorner	17620
U+7899.0	fCantonese	gong1; naau4; laau4 (also)
U+7899.0	fMandarin	gang1
U+789A.0	fUTF8	碚
U+789A.0	fCNS11643	2-454B
U+789A.0	fBig5	DEE5
U+789A.0	fGB	0-6D55
U+789A.0	fR/S	 112.8
U+789A.0	fTotalStrokes	 13
U+789A.0	fCangjie	MRYTR
U+789A.0	fFourCorner	10661
U+789A.0	fCantonese	pui5; bui3 (also)
U+789A.0	fMandarin	bei4; pei2
U+789A.0	fEnglish	used in combination
U+789B.0	fUTF8	碛
U+789B.0	fGB	0-6D53
U+789B.0	fR/S	 112.8
U+789B.0	fTotalStrokes	 13
U+789B.0	fCangjie	MRQMO
U+789B.0	fHakka	 zit7; zak7
U+789B.0	fCantonese	zik1; zaat3; zaak3
U+789B.0	fMandarin	qi4
U+789B.0	fEnglish	simplified form of &#x78E7;
U+789C.0	fUTF8	碜
U+789C.0	fCNS11643	F-4425
U+789C.0	fGB	0-6D57
U+789C.0	fR/S	 112.8
U+789C.0	fTotalStrokes	 13
U+789C.0	fCangjie	MRIKH
U+789C.0	fHakka	 cam3
U+789C.0	fCantonese	cam2
U+789C.0	fMandarin	chen3
U+789C.0	fEnglish	simplified form of &#x78E3;
U+789D.0	fUTF8	碝
U+789D.0	fCNS11643	4-4553
U+789D.0	fGB	3-583F
U+789D.0	fR/S	  112.9
U+789D.0	fTotalStrokes	 14
U+789D.0	fCangjie	MRMBK
U+789D.0	fFourCorner	11630
U+789D.0	fHakka	 ngion3; ngien3; nion1
U+789D.0	fCantonese	jyun5
U+789D.0	fMandarin	ruan3
U+789D.0	fEnglish	an opaque whitish quartz, with pieces of cornelian in it - used as ornaments
U+789E.0	fUTF8	碞
U+789E.0	fCNS11643	2-4C3A
U+789E.0	fBig5	E2F2
U+789E.0	fGB	5-5846
U+789E.0	fR/S	 112.9
U+789E.0	fTotalStrokes	 14
U+789E.0	fCangjie	RRRR
U+789E.0	fFourCorner	60601
U+789E.0	fHakka	 ngam2
U+789E.0	fCantonese	ngaam4; aam4 (also); jam4
U+789E.0	fMandarin	yan2; yin2; yin3
U+789E.0	fEnglish	 room made of stone watchtower
U+789F.0	fUTF8	碟
U+789F.0	fCNS11643	1-6977
U+789F.0	fBig5	BAD0
U+789F.0	fGB	0-357A
U+789F.0	fR/S	 112.9
U+789F.0	fTotalStrokes	 14
U+789F.0	fCangjie	MRPTD
U+789F.0	fFourCorner	14694
U+789F.0	fHakka	  tiap8; sat8
U+789F.0	fCantonese	dip6; dip2 (also)
U+789F.0	fMandarin	die2; she2
U+789F.0	fEnglish	 small dish; plate
U+78A0.0	fUTF8	碠
U+78A0.0	fCNS11643	2-4C3C
U+78A0.0	fBig5	E2F4
U+78A0.0	fGB	5-584A
U+78A0.0	fR/S	 112.9
U+78A0.0	fTotalStrokes	 14
U+78A0.0	fCangjie	MRYRN
U+78A0.0	fFourCorner	10621
U+78A0.0	fCantonese	ding3
U+78A0.0	fMandarin	ding4
U+78A1.0	fUTF8	碡
U+78A1.0	fCNS11643	2-4552
U+78A1.0	fBig5	DEEC
U+78A1.0	fGB	0-6D58
U+78A1.0	fR/S	 112.9; 112.8
U+78A1.0	fTotalStrokes	 14
U+78A1.0	fCangjie	MRQMY
U+78A1.0	fFourCorner	15657
U+78A1.0	fHakka	 tuk8; cuk8; ch'uk8; chuk8
U+78A1.0	fCantonese	duk6; zuk6
U+78A1.0	fMandarin	zhou2; du2
U+78A1.0	fEnglish	stone roller used to level fields
U+78A2.0	fUTF8	碢
U+78A2.0	fCNS11643	2-4C3E
U+78A2.0	fBig5	E2F6
U+78A2.0	fGB	5-5836
U+78A2.0	fR/S	 112.9
U+78A2.0	fTotalStrokes	 14
U+78A2.0	fCangjie	MRBBR
U+78A2.0	fFourCorner	17627
U+78A2.0	fCantonese	to4; wo1
U+78A2.0	fMandarin	tuo2
U+78A3.0	fUTF8	碣
U+78A3.0	fCNS11643	1-697B
U+78A3.0	fBig5	BAD4
U+78A3.0	fGB	0-6D59
U+78A3.0	fR/S	 112.9
U+78A3.0	fTotalStrokes	 14
U+78A3.0	fCangjie	MRAPV
U+78A3.0	fFourCorner	16627
U+78A3.0	fHakka	  giet7; ket7; get7
U+78A3.0	fCantonese	kit3
U+78A3.0	fMandarin	jie2; ya4; ke4; ji4
U+78A3.0	fEnglish	 stone tablet (especially one with a round outline)
U+78A4.0	fUTF8	碤
U+78A4.0	fCNS11643	2-4C3F
U+78A4.0	fBig5	E2F7
U+78A4.0	fGB	3-5829
U+78A4.0	fR/S	 112.9; 112.8
U+78A4.0	fTotalStrokes	 14
U+78A4.0	fCangjie	MRTLK
U+78A4.0	fFourCorner	14635
U+78A4.0	fCantonese	jing1
U+78A4.0	fMandarin	ying1
U+78A5.0	fUTF8	碥
U+78A5.0	fCNS11643	2-4C3B
U+78A5.0	fBig5	E2F3
U+78A5.0	fGB	0-6D5C
U+78A5.0	fR/S	 112.9
U+78A5.0	fTotalStrokes	 14
U+78A5.0	fCangjie	MRHSB
U+78A5.0	fFourCorner	12627
U+78A5.0	fHakka	  bien3; ben3
U+78A5.0	fCantonese	bin2
U+78A5.0	fMandarin	bian3; bian1
U+78A5.0	fEnglish	a large block of stone, to mount chariots from
U+78A6.0	fUTF8	碦
U+78A6.0	fCNS11643	3-483E
U+78A6.0	fGB	3-584A
U+78A6.0	fR/S	  112.9
U+78A6.0	fTotalStrokes	 14
U+78A6.0	fCangjie	MRJHR
U+78A6.0	fFourCorner	13660
U+78A6.0	fHakka	 kok8
U+78A6.0	fCantonese	haak3
U+78A6.0	fMandarin	ke4
U+78A7.0	fUTF8	碧
U+78A7.0	fCNS11643	1-6978
U+78A7.0	fBig5	BAD1
U+78A7.0	fGB	0-314C
U+78A7.0	fR/S	 112.9
U+78A7.0	fTotalStrokes	 14
U+78A7.0	fCangjie	MAMR
U+78A7.0	fFourCorner	16601
U+78A7.0	fHakka	 bit7; pet7
U+78A7.0	fCantonese	bik1
U+78A7.0	fMandarin	bi4
U+78A7.0	fEnglish	[1] jade; green; blue; verdant; emerald green [2] jasper; emerald
U+78A8.0	fUTF8	碨
U+78A8.0	fCNS11643	2-4C37
U+78A8.0	fBig5	E2EF
U+78A8.0	fGB	3-5843
U+78A8.0	fR/S	 112.9
U+78A8.0	fTotalStrokes	 14
U+78A8.0	fCangjie	MRWMV
U+78A8.0	fFourCorner	16632
U+78A8.0	fHakka	 woi1; wui3; wi3; voi1
U+78A8.0	fCantonese	wai3; wai2; wui2 (also); wui1
U+78A8.0	fMandarin	wei1; wei3
U+78A8.0	fEnglish	rough, stony ground
U+78A9.0	fUTF8	碩
U+78A9.0	fCNS11643	1-697A
U+78A9.0	fBig5	BAD3
U+78A9.0	fGB	1-4B36
U+78A9.0	fR/S	 112.9
U+78A9.0	fTotalStrokes	 14
U+78A9.0	fCangjie	MRMBC
U+78A9.0	fFourCorner	11686
U+78A9.0	fHakka	  shak8; sak8; sok7; shok7
U+78A9.0	fCantonese	sek6
U+78A9.0	fMandarin	shuo4; shi2
U+78A9.0	fEnglish	great; eminent; large, big
U+78AA.0	fUTF8	碪
U+78AA.0	fCNS11643	2-4C34
U+78AA.0	fBig5	E2EC
U+78AA.0	fGB	E-3540
U+78AA.0	fR/S	 112.9
U+78AA.0	fTotalStrokes	 14
U+78AA.0	fCangjie	MRTMV
U+78AA.0	fFourCorner	14611
U+78AA.0	fHakka	  zem1; ham1; zem; ham3
U+78AA.0	fCantonese	zam1; caa4
U+78AA.0	fMandarin	zhen1; an3; kan4
U+78AA.0	fEnglish	stone slab used for washing clot
U+78AB.0	fUTF8	碫
U+78AB.0	fCNS11643	2-4C39
U+78AB.0	fBig5	E2F1
U+78AB.0	fGB	3-5845
U+78AB.0	fR/S	 112.9
U+78AB.0	fTotalStrokes	 14
U+78AB.0	fCangjie	MRHJE
U+78AB.0	fFourCorner	17647
U+78AB.0	fHakka	 ton5; ton3; ton6
U+78AB.0	fCantonese	dyun3; dyun6
U+78AB.0	fMandarin	duan4
U+78AB.0	fEnglish	a coarse stone used for whetstones
U+78AC.0	fUTF8	碬
U+78AC.0	fCNS11643	2-4C3D
U+78AC.0	fBig5	E2F5
U+78AC.0	fGB	3-584B
U+78AC.0	fR/S	 112.9
U+78AC.0	fTotalStrokes	 14
U+78AC.0	fCangjie	MRRSE
U+78AC.0	fFourCorner	17647
U+78AC.0	fHakka	 ha2
U+78AC.0	fCantonese	haa4
U+78AC.0	fMandarin	xia2
U+78AC.0	fEnglish	whetstone
U+78AD.0	fUTF8	碭
U+78AD.0	fCNS11643	2-4C36
U+78AD.0	fBig5	E2EE
U+78AD.0	fGB	1-6D38
U+78AD.0	fR/S	 112.9
U+78AD.0	fTotalStrokes	 14
U+78AD.0	fCangjie	MRAMH
U+78AD.0	fFourCorner	16627
U+78AD.0	fHakka	 tong3; tong5; dong5
U+78AD.0	fCantonese	dong6
U+78AD.0	fMandarin	dang4; tang2; yang2
U+78AD.0	fEnglish	brilliantly colored veined stone
U+78AE.0	fUTF8	碮
U+78AE.0	fCNS11643	3-483F
U+78AE.0	fGB	3-5842
U+78AE.0	fR/S	  112.9
U+78AE.0	fTotalStrokes	 14
U+78AE.0	fCangjie	MRAMO
U+78AE.0	fFourCorner	16680
U+78AE.0	fCantonese	tai4
U+78AE.0	fMandarin	ti2
U+78AF.0	fUTF8	碯
U+78AF.0	fCNS11643	3-4845
U+78AF.0	fGB	5-584D
U+78AF.0	fR/S	  112.9
U+78AF.0	fTotalStrokes	 14
U+78AF.0	fCangjie	MRVVW
U+78AF.0	fFourCorner	12660
U+78AF.0	fHakka	  nau3; no3
U+78AF.0	fCantonese	nou5; lou5 (also)
U+78AF.0	fMandarin	nao3
U+78AF.0	fEnglish	agate
U+78B0.0	fUTF8	碰
U+78B0.0	fCNS11643	1-6550
U+78B0.0	fBig5	B849
U+78B0.0	fGB	0-4576
U+78B0.0	fR/S	 112.8; 112.9
U+78B0.0	fTotalStrokes	 14
U+78B0.0	fCangjie	MRTTC
U+78B0.0	fFourCorner	18612
U+78B0.0	fHakka	  pong1;  pang5; bong1; pung5; pung1
U+78B0.0	fCantonese	pung3
U+78B0.0	fMandarin	peng4
U+78B0.0	fEnglish	[1] collide; hit; touch; bump [2] meet unexpectedly; run into [3] take one's chance
U+78B1.0	fUTF8	碱
U+78B1.0	fCNS11643	3-4844
U+78B1.0	fGB	0-3C6E
U+78B1.0	fR/S	  112.9
U+78B1.0	fTotalStrokes	 14; 13
U+78B1.0	fCangjie	MRIHR
U+78B1.0	fFourCorner	13650
U+78B1.0	fCantonese	gaan2; gan2
U+78B1.0	fMandarin	jian3; xian2
U+78B1.0	fEnglish	alkaline; alkali; lye; salt
U+78B2.0	fUTF8	碲
U+78B2.0	fCNS11643	2-4C33
U+78B2.0	fBig5	E2EB
U+78B2.0	fGB	0-6D5A
U+78B2.0	fR/S	 112.9
U+78B2.0	fTotalStrokes	 14
U+78B2.0	fCangjie	MRYBB
U+78B2.0	fFourCorner	10627
U+78B2.0	fCantonese	dai3
U+78B2.0	fMandarin	di4
U+78B3.0	fUTF8	碳
U+78B3.0	fCNS11643	1-6979
U+78B3.0	fBig5	BAD2
U+78B3.0	fGB	0-4C3C
U+78B3.0	fR/S	 112.9
U+78B3.0	fTotalStrokes	 14
U+78B3.0	fCangjie	MRUMF
U+78B3.0	fFourCorner	12689
U+78B3.0	fHakka	 tan5
U+78B3.0	fCantonese	taan3
U+78B3.0	fMandarin	tan4
U+78B3.0	fEnglish	carbon
U+78B4.0	fUTF8	碴
U+78B4.0	fCNS11643	2-4C35
U+78B4.0	fBig5	E2ED
U+78B4.0	fGB	0-326A
U+78B4.0	fR/S	 112.9
U+78B4.0	fTotalStrokes	 14
U+78B4.0	fCangjie	MRDAM
U+78B4.0	fFourCorner	19616
U+78B4.0	fCantonese	zaa1; caa4 (also); caa1
U+78B4.0	fMandarin	cha2; cha1; zha1; zha3
U+78B4.0	fEnglish	[1] chipped edge or splinter of a container [2] be cut (by broken glass, etc.)
U+78B5.0	fUTF8	碵
U+78B5.0	fGB	E-3541
U+78B5.0	fR/S	 112.9
U+78B5.0	fTotalStrokes	 14
U+78B5.0	fCangjie	MRYBC
U+78B5.0	fFourCorner	1168
U+78B5.0	fCantonese	zing1
U+78B6.0	fUTF8	碶
U+78B6.0	fGB	3-583D
U+78B6.0	fR/S	 112.9
U+78B6.0	fTotalStrokes	 14
U+78B6.0	fCangjie	MRQHK
U+78B6.0	fFourCorner	1763
U+78B6.0	fCantonese	kai3
U+78B6.0	fMandarin	qi4
U+78B6.0	fEnglish	a flood-gate; a water-gate
U+78B7.0	fUTF8	碷
U+78B7.0	fCNS11643	3-6573
U+78B7.0	fGB	E-3542
U+78B7.0	fR/S	 112.9
U+78B7.0	fTotalStrokes	 14
U+78B7.0	fCangjie	MRHJU
U+78B7.0	fFourCorner	1266
U+78B7.0	fCantonese	seon5
U+78B8.0	fUTF8	碸
U+78B8.0	fCNS11643	3-4840
U+78B8.0	fGB	1-6D3F
U+78B8.0	fR/S	 112.9
U+78B8.0	fTotalStrokes	 14
U+78B8.0	fCangjie	MRHNI
U+78B8.0	fFourCorner	17610
U+78B8.0	fCantonese	fung1
U+78B8.0	fMandarin	feng1
U+78B9.0	fUTF8	碹
U+78B9.0	fCNS11643	F-4A3F
U+78B9.0	fGB	0-6D5B
U+78B9.0	fR/S	 112.9
U+78B9.0	fTotalStrokes	 14
U+78B9.0	fCangjie	MRJMM
U+78B9.0	fCantonese	syun4
U+78B9.0	fMandarin	xuan4
U+78BA.0	fUTF8	確
U+78BA.0	fCNS11643	1-6D7D
U+78BA.0	fBig5	BD54
U+78BA.0	fGB	1-4837
U+78BA.0	fR/S	 112.10
U+78BA.0	fTotalStrokes	 15
U+78BA.0	fCangjie	MROBG
U+78BA.0	fFourCorner	14614
U+78BA.0	fHakka	  kok7
U+78BA.0	fCantonese	kok3
U+78BA.0	fMandarin	que4
U+78BA.0	fEnglish	sure; certain; real; true; secure; valid
U+78BB.0	fUTF8	碻
U+78BB.0	fCNS11643	2-5249
U+78BB.0	fBig5	E6C1
U+78BB.0	fGB	3-5853
U+78BB.0	fR/S	 112.10
U+78BB.0	fTotalStrokes	 15
U+78BB.0	fCangjie	MRYRB
U+78BB.0	fFourCorner	10627
U+78BB.0	fCantonese	kok3
U+78BB.0	fMandarin	que4; qiao1
U+78BB.0	fEnglish	sollid
U+78BC.0	fUTF8	碼
U+78BC.0	fCNS11643	1-6E23
U+78BC.0	fBig5	BD58
U+78BC.0	fGB	1-426B
U+78BC.0	fR/S	 112.10
U+78BC.0	fTotalStrokes	 15
U+78BC.0	fCangjie	MRSQF
U+78BC.0	fFourCorner	11627
U+78BC.0	fHakka	  ma1;  ma3
U+78BC.0	fCantonese	maa5; maa2 (also)
U+78BC.0	fMandarin	ma3
U+78BC.0	fEnglish	[1] number; numeral; symbol; code [2] yard (measure of length)
U+78BD.0	fUTF8	碽
U+78BD.0	fCNS11643	4-4B59
U+78BD.0	fGB	3-577A
U+78BD.0	fR/S	  112.10
U+78BD.0	fTotalStrokes	 15
U+78BD.0	fCangjie	MRMBC
U+78BD.0	fFourCorner	11680
U+78BD.0	fCantonese	gung1
U+78BD.0	fMandarin	gong1
U+78BE.0	fUTF8	碾
U+78BE.0	fCNS11643	1-6E21
U+78BE.0	fBig5	BD56
U+78BE.0	fGB	0-446B
U+78BE.0	fR/S	 112.10
U+78BE.0	fTotalStrokes	 15
U+78BE.0	fCangjie	MRSTV
U+78BE.0	fFourCorner	17632
U+78BE.0	fHakka	 ngien5; chan5; len3; can5; zen3; zhan5; zan5; cen5; ngian3; chen5
U+78BE.0	fCantonese	nin5 (also); zin2; lin5 (also); nin6
U+78BE.0	fMandarin	nian3; nian4; lian3
U+78BE.0	fEnglish	[1] stone roller [2] mill; roll; crush
U+78BF.0	fUTF8	碿
U+78BF.0	fCNS11643	4-4B57
U+78BF.0	fGB	3-5859
U+78BF.0	fR/S	  112.10
U+78BF.0	fTotalStrokes	 15
U+78BF.0	fCangjie	MRSFB
U+78BF.0	fFourCorner	17620
U+78BF.0	fCantonese	seot1
U+78BF.0	fMandarin	su4
U+78C0.0	fUTF8	磀
U+78C0.0	fCNS11643	3-4D4A
U+78C0.0	fGB	E-3543
U+78C0.0	fR/S	 112.10
U+78C0.0	fTotalStrokes	 15
U+78C0.0	fCangjie	MRYTU
U+78C0.0	fFourCorner	18630
U+78C0.0	fCantonese	ngaak6; aak6 (also)
U+78C0.0	fMandarin	e2
U+78C1.0	fUTF8	磁
U+78C1.0	fCNS11643	1-6976
U+78C1.0	fBig5	BACF
U+78C1.0	fGB	0-3445
U+78C1.0	fR/S	 112.10; 112.9
U+78C1.0	fTotalStrokes	 14
U+78C1.0	fCangjie	MRTVI
U+78C1.0	fFourCorner	18632
U+78C1.0	fHakka	 cu2;  cih2; cih3; ze2
U+78C1.0	fCantonese	ci4
U+78C1.0	fMandarin	ci2
U+78C1.0	fEnglish	[1] magnetic; magnetism [2] porcelain; china
U+78C2.0	fUTF8	磂
U+78C2.0	fCNS11643	3-4D4D
U+78C2.0	fGB	5-5852
U+78C2.0	fR/S	 112.10
U+78C2.0	fTotalStrokes	 15
U+78C2.0	fCangjie	MRHHW
U+78C2.0	fFourCorner	17660
U+78C2.0	fHakka	 liu2
U+78C2.0	fCantonese	lau4
U+78C2.0	fMandarin	liu4
U+78C3.0	fUTF8	磃
U+78C3.0	fCNS11643	2-5250
U+78C3.0	fBig5	E6C8
U+78C3.0	fGB	3-5851
U+78C3.0	fR/S	 112.10
U+78C3.0	fTotalStrokes	 15
U+78C3.0	fCangjie	MRHYU
U+78C3.0	fFourCorner	12617
U+78C3.0	fCantonese	si1; tai4
U+78C3.0	fMandarin	si1; ti2
U+78C4.0	fUTF8	磄
U+78C4.0	fCNS11643	2-5251
U+78C4.0	fBig5	E6C9
U+78C4.0	fGB	3-5854
U+78C4.0	fR/S	 112.10
U+78C4.0	fTotalStrokes	 15
U+78C4.0	fCangjie	MRILR
U+78C4.0	fFourCorner	10667
U+78C4.0	fHakka	 tong2
U+78C4.0	fCantonese	tong4
U+78C4.0	fMandarin	tang2
U+78C4.0	fEnglish	[1] enchanted stone [2] stone enbankement
U+78C5.0	fUTF8	磅
U+78C5.0	fCNS11643	1-6D7C
U+78C5.0	fBig5	BD53
U+78C5.0	fGB	0-3075
U+78C5.0	fR/S	 112.10
U+78C5.0	fTotalStrokes	 15
U+78C5.0	fCangjie	MRYBS
U+78C5.0	fFourCorner	10627
U+78C5.0	fHakka	 pong2; pong5; bong5; pong3; bong1
U+78C5.0	fCantonese	bong6; bong2 (also); pong4
U+78C5.0	fMandarin	bang4; pang2; beng4; pang4; peng1
U+78C5.0	fEnglish	[1] pound [2] weigh; scales [3] weigh
U+78C5.1	fUTF8	磅
U+78C5.1	fCNS11643	1-6D7C
U+78C5.1	fBig5	BD53
U+78C5.1	fGB	0-3075
U+78C5.1	fR/S	 112.10
U+78C5.1	fTotalStrokes	 15
U+78C5.1	fCangjie	MRYBS
U+78C5.1	fFourCorner	10627
U+78C5.1	fMandarin	pang1
U+78C5.1	fEnglish	the noise of stone crashing
U+78C6.0	fUTF8	磆
U+78C6.0	fCNS11643	3-4D48
U+78C6.0	fGB	5-5847
U+78C6.0	fR/S	  112.10
U+78C6.0	fTotalStrokes	 15
U+78C6.0	fCangjie	MRBBB
U+78C6.0	fFourCorner	17620
U+78C6.0	fHakka	 wat8; vat8
U+78C6.0	fCantonese	waat6
U+78C6.0	fMandarin	hua2
U+78C6.0	fEnglish	soapstone
U+78C7.0	fUTF8	磇
U+78C7.0	fCNS11643	3-4D4C
U+78C7.0	fGB	5-584F
U+78C7.0	fR/S	  112.10
U+78C7.0	fTotalStrokes	 15
U+78C7.0	fCangjie	MRHWP
U+78C7.0	fFourCorner	16610
U+78C7.0	fHakka	  pai1;  pi1
U+78C7.0	fCantonese	pei1; pai1
U+78C7.0	fMandarin	pi1
U+78C7.0	fEnglish	arsenic
U+78C8.0	fUTF8	磈
U+78C8.0	fCNS11643	2-524F
U+78C8.0	fBig5	E6C7
U+78C8.0	fGB	3-5846
U+78C8.0	fR/S	 112.10
U+78C8.0	fTotalStrokes	 15
U+78C8.0	fCangjie	MRHUI
U+78C8.0	fFourCorner	16613
U+78C8.0	fHakka	 kui3; wui3; wi3; vui3
U+78C8.0	fCantonese	faai3; wai5
U+78C8.0	fMandarin	wei3; kui3; gui3; kui4; wai1
U+78C8.0	fEnglish	rough; stony; rocky
U+78C9.0	fUTF8	磉
U+78C9.0	fCNS11643	2-5252
U+78C9.0	fBig5	E6CA
U+78C9.0	fGB	0-6D5F
U+78C9.0	fR/S	 112.10
U+78C9.0	fTotalStrokes	 15
U+78C9.0	fCangjie	MREED
U+78C9.0	fFourCorner	17694
U+78C9.0	fHakka	 song3
U+78C9.0	fCantonese	song2
U+78C9.0	fMandarin	sang3
U+78C9.0	fEnglish	stone base or plinth of a pillar
U+78CA.0	fUTF8	磊
U+78CA.0	fCNS11643	1-6D7E
U+78CA.0	fBig5	BD55
U+78CA.0	fGB	0-405A
U+78CA.0	fR/S	 112.10
U+78CA.0	fTotalStrokes	 15
U+78CA.0	fCangjie	MRMRR
U+78CA.0	fFourCorner	10661
U+78CA.0	fHakka	 lui3;  doi1
U+78CA.0	fCantonese	leoi5
U+78CA.0	fMandarin	lei3
U+78CA.0	fEnglish	[1] pile of rocks or stones [2] great; massive
U+78CB.0	fUTF8	磋
U+78CB.0	fCNS11643	1-6D7B
U+78CB.0	fBig5	BD52
U+78CB.0	fGB	0-3468
U+78CB.0	fR/S	 112.10
U+78CB.0	fTotalStrokes	 15
U+78CB.0	fCangjie	MRTQM
U+78CB.0	fFourCorner	18611
U+78CB.0	fHakka	 co1;  co5
U+78CB.0	fCantonese	co1
U+78CB.0	fMandarin	cuo1; cuo4
U+78CB.0	fEnglish	 file; polish (jade, horn, stone, etc.)
U+78CC.0	fUTF8	磌
U+78CC.0	fCNS11643	2-524B
U+78CC.0	fBig5	E6C3
U+78CC.0	fGB	3-584E
U+78CC.0	fR/S	 112.10
U+78CC.0	fTotalStrokes	 15
U+78CC.0	fCangjie	MRJBC
U+78CC.0	fFourCorner	11686
U+78CC.0	fHakka	  tien2
U+78CC.0	fCantonese	tin4
U+78CC.0	fMandarin	tian2; zhen1
U+78CC.0	fEnglish	[1] a crashing or rumbling sound [2] plinth
U+78CD.0	fUTF8	磍
U+78CD.0	fCNS11643	2-5248
U+78CD.0	fBig5	E6C0
U+78CD.0	fGB	3-5856
U+78CD.0	fR/S	 112.10
U+78CD.0	fTotalStrokes	 15
U+78CD.0	fCangjie	MRJQR
U+78CD.0	fFourCorner	13661
U+78CD.0	fCantonese	git3; daat3; hat1; hat6
U+78CD.0	fMandarin	xia2; jia2
U+78CE.0	fUTF8	磎
U+78CE.0	fCNS11643	2-524D
U+78CE.0	fBig5	E6C5
U+78CE.0	fGB	5-5851
U+78CE.0	fR/S	 112.10
U+78CE.0	fTotalStrokes	 15
U+78CE.0	fCangjie	MRBVK
U+78CE.0	fFourCorner	12635
U+78CE.0	fCantonese	kai1
U+78CE.0	fMandarin	qi1; xi1
U+78CE.0	fEnglish	mountain stream; creek
U+78CF.0	fUTF8	磏
U+78CF.0	fCNS11643	2-524A
U+78CF.0	fBig5	E6C2
U+78CF.0	fGB	3-5855
U+78CF.0	fR/S	 112.10
U+78CF.0	fTotalStrokes	 15
U+78CF.0	fCangjie	MRTXC
U+78CF.0	fFourCorner	18637
U+78CF.0	fHakka	 liam2
U+78CF.0	fCantonese	lim4; him1
U+78CF.0	fMandarin	lian2
U+78CF.0	fEnglish	[1] a coarse red sandstone [2] hypocritical
U+78D0.0	fUTF8	磐
U+78D0.0	fCNS11643	1-6E24
U+78D0.0	fBig5	BD59
U+78D0.0	fGB	0-454D
U+78D0.0	fR/S	 112.10
U+78D0.0	fTotalStrokes	 15
U+78D0.0	fCangjie	HEMR
U+78D0.0	fFourCorner	27601
U+78D0.0	fHakka	 pan2
U+78D0.0	fCantonese	pun4
U+78D0.0	fMandarin	pan2; mian2
U+78D0.0	fEnglish	[1] large rock; massive rock [2] linger around [3] league together
U+78D1.0	fUTF8	磑
U+78D1.0	fCNS11643	2-524C
U+78D1.0	fBig5	E6C4
U+78D1.0	fGB	5-5828
U+78D1.0	fR/S	 112.10
U+78D1.0	fTotalStrokes	 15
U+78D1.0	fCangjie	MRUMT
U+78D1.0	fFourCorner	12618
U+78D1.0	fHakka	  koi3
U+78D1.0	fCantonese	wai3; hoi2 (also); oi4; ngoi4 (also); ngai4; ngoi6; wui1; wai4
U+78D1.0	fMandarin	wei4; ai2; wei2; gai1; gai4; ji1; kai3; mo4; yi2
U+78D1.0	fEnglish	 stone mill grind break apart
U+78D2.0	fUTF8	磒
U+78D2.0	fCNS11643	3-4D4B
U+78D2.0	fGB	5-5835
U+78D2.0	fR/S	 112.10
U+78D2.0	fTotalStrokes	 15
U+78D2.0	fCangjie	MRRBC
U+78D2.0	fFourCorner	16680
U+78D2.0	fHakka	 jun3; run3
U+78D2.0	fCantonese	wan5; wan6
U+78D2.0	fMandarin	yun3
U+78D2.0	fEnglish	 fall down
U+78D3.0	fUTF8	磓
U+78D3.0	fCNS11643	3-4D49
U+78D3.0	fGB	3-5847
U+78D3.0	fR/S	  112.10
U+78D3.0	fTotalStrokes	 15
U+78D3.0	fCangjie	MRYHR
U+78D3.0	fFourCorner	17630
U+78D3.0	fHakka	 dui1; doi1; ch'ui6; cui5
U+78D3.0	fCantonese	deoi1
U+78D3.0	fMandarin	dui1
U+78D3.0	fEnglish	 collect stones together
U+78D4.0	fUTF8	磔
U+78D4.0	fCNS11643	2-524E
U+78D4.0	fBig5	E6C6
U+78D4.0	fGB	0-6D5D
U+78D4.0	fR/S	 112.10
U+78D4.0	fTotalStrokes	 15
U+78D4.0	fCangjie	MRNQD
U+78D4.0	fFourCorner	15694
U+78D4.0	fCantonese	zaak6; zaak3 (also)
U+78D4.0	fMandarin	zhe2
U+78D4.0	fEnglish	[1] dismember a human being [2] downward stroke slanting the right
U+78D5.0	fUTF8	磕
U+78D5.0	fCNS11643	1-6E22
U+78D5.0	fBig5	BD57
U+78D5.0	fGB	0-3F44
U+78D5.0	fR/S	 112.10
U+78D5.0	fTotalStrokes	 15
U+78D5.0	fCangjie	MRGIT
U+78D5.0	fFourCorner	14617
U+78D5.0	fHakka	  kap8; ngap8; koi3; gok8; kok7; kok8
U+78D5.0	fCantonese	hap6; ham2 (also); koi3
U+78D5.0	fMandarin	ke1; ke3; kai4
U+78D5.0	fEnglish	 hit against; collide; knock into; bump against
U+78D6.0	fUTF8	磖
U+78D6.0	fCNS11643	3-5221
U+78D6.0	fGB	E-3544
U+78D6.0	fR/S	 112.10; 112.11
U+78D6.0	fTotalStrokes	 16; 15
U+78D6.0	fCangjie	MRSMA
U+78D6.0	fFourCorner	17660
U+78D6.0	fCantonese	laap6; laa1 (also)
U+78D6.0	fMandarin	la1; la2
U+78D7.0	fUTF8	磗
U+78D7.0	fGB	1-7D40
U+78D7.0	fR/S	 112.10
U+78D7.0	fTotalStrokes	 15
U+78D7.0	fCangjie	MRIBI
U+78D7.0	fCantonese	zyun1
U+78D8.0	fUTF8	磘
U+78D8.0	fCNS11643	3-6574
U+78D8.0	fGB	E-3545
U+78D8.0	fR/S	 112.10
U+78D8.0	fTotalStrokes	 15
U+78D8.0	fCangjie	MRBOU
U+78D8.0	fFourCorner	1267
U+78D8.0	fCantonese	jiu4
U+78D8.0	fMandarin	qing4
U+78D9.0	fUTF8	磙
U+78D9.0	fGB	0-6D5E
U+78D9.0	fR/S	 112.10
U+78D9.0	fTotalStrokes	 15
U+78D9.0	fCangjie	MRYCV
U+78D9.0	fCantonese	gwan2
U+78D9.0	fMandarin	gun3
U+78D9.0	fEnglish	roller
U+78DA.0	fUTF8	磚
U+78DA.0	fCNS11643	1-7154
U+78DA.0	fBig5	BF6A
U+78DA.0	fGB	1-5729
U+78DA.0	fR/S	 112.11
U+78DA.0	fTotalStrokes	 16
U+78DA.0	fCangjie	MRJII
U+78DA.0	fFourCorner	15643
U+78DA.0	fHakka	  zon1;  zhon1; ton2; ngoi5
U+78DA.0	fCantonese	zyun1
U+78DA.0	fMandarin	zhuan1; tuan2; tuo2
U+78DA.0	fEnglish	 brick; tile
U+78DB.0	fUTF8	磛
U+78DB.0	fCNS11643	2-5872
U+78DB.0	fBig5	EAA8
U+78DB.0	fGB	3-582E
U+78DB.0	fR/S	 112.11
U+78DB.0	fTotalStrokes	 16
U+78DB.0	fCangjie	JLMR
U+78DB.0	fFourCorner	52601
U+78DB.0	fHakka	 cam2
U+78DB.0	fCantonese	caam4
U+78DB.0	fMandarin	chan2; jian4; zhan4
U+78DB.0	fEnglish	cliff
U+78DC.0	fUTF8	磜
U+78DC.0	fCNS11643	3-5224
U+78DC.0	fGB	5-5859
U+78DC.0	fR/S	 112.10; 112.11
U+78DC.0	fTotalStrokes	 17; 16; 15
U+78DC.0	fCangjie	MRBOF
U+78DC.0	fFourCorner	17690
U+78DC.0	fCantonese	zai3; cai3 (also)
U+78DC.0	fMandarin	qi4
U+78DD.0	fUTF8	磝
U+78DD.0	fCNS11643	2-586C
U+78DD.0	fBig5	EAA2
U+78DD.0	fGB	3-584C
U+78DD.0	fR/S	 112.11
U+78DD.0	fTotalStrokes	 16
U+78DD.0	fCangjie	MRGSK
U+78DD.0	fFourCorner	18640
U+78DD.0	fHakka	 ngau2
U+78DD.0	fCantonese	ngou4; ou4 (also); ngaau4
U+78DD.0	fMandarin	ao2; qiao1
U+78DD.0	fEnglish	 stony surface; shingle
U+78DE.0	fUTF8	磞
U+78DE.0	fCNS11643	2-5870
U+78DE.0	fBig5	EAA6
U+78DE.0	fGB	3-585E
U+78DE.0	fR/S	 112.11
U+78DE.0	fTotalStrokes	 16
U+78DE.0	fCangjie	MRUBB
U+78DE.0	fFourCorner	12627
U+78DE.0	fCantonese	paang1; pang4
U+78DE.0	fMandarin	peng1
U+78DF.0	fUTF8	磟
U+78DF.0	fCNS11643	2-5876
U+78DF.0	fBig5	EAAC
U+78DF.0	fGB	E-3546
U+78DF.0	fR/S	 112.11
U+78DF.0	fTotalStrokes	 16
U+78DF.0	fCangjie	MRSMH
U+78DF.0	fFourCorner	17622
U+78DF.0	fHakka	 liuk8; luk8
U+78DF.0	fCantonese	luk6; luk1
U+78DF.0	fMandarin	lu4; liu4
U+78DF.0	fEnglish	 mediocre busy occupied roller
U+78E0.0	fUTF8	磠
U+78E0.0	fCNS11643	2-5877
U+78E0.0	fBig5	EAAD
U+78E0.0	fGB	3-577E
U+78E0.0	fR/S	 112.11
U+78E0.0	fTotalStrokes	 16
U+78E0.0	fCangjie	MRYWI
U+78E0.0	fFourCorner	11660
U+78E0.0	fHakka	 lu3; lu1
U+78E0.0	fCantonese	lou5
U+78E0.0	fMandarin	lu3
U+78E0.0	fEnglish	 sal ammoniac sand pebble
U+78E1.0	fUTF8	磡
U+78E1.0	fCNS11643	2-5873
U+78E1.0	fBig5	EAA9
U+78E1.0	fGB	3-585A
U+78E1.0	fR/S	 112.11
U+78E1.0	fTotalStrokes	 16
U+78E1.0	fCangjie	MRTVS
U+78E1.0	fFourCorner	14627
U+78E1.0	fHakka	  ham5;  kam3; kam5; kiam1
U+78E1.0	fCantonese	ham3
U+78E1.0	fMandarin	kan4
U+78E1.0	fEnglish	 cliff; ledge; bank step
U+78E2.0	fUTF8	磢
U+78E2.0	fCNS11643	2-5874
U+78E2.0	fBig5	EAAA
U+78E2.0	fGB	3-585C
U+78E2.0	fR/S	 112.11
U+78E2.0	fTotalStrokes	 16
U+78E2.0	fCangjie	MRKKK
U+78E2.0	fFourCorner	14634
U+78E2.0	fHakka	 song3; ciong3; sang3; cong3
U+78E2.0	fCantonese	cong2; coeng2; saang2
U+78E2.0	fMandarin	qiang3; chuang3
U+78E2.0	fEnglish	 rub clean
U+78E3.0	fUTF8	磣
U+78E3.0	fCNS11643	2-5871
U+78E3.0	fBig5	EAA7
U+78E3.0	fGB	1-6D57
U+78E3.0	fR/S	 112.11
U+78E3.0	fTotalStrokes	 16
U+78E3.0	fCangjie	MRIIH
U+78E3.0	fFourCorner	13622
U+78E3.0	fHakka	  cam3
U+78E3.0	fCantonese	cam2
U+78E3.0	fMandarin	chen3; ca4
U+78E3.0	fEnglish	gritty
U+78E4.0	fUTF8	磤
U+78E4.0	fCNS11643	4-4B5D
U+78E4.0	fGB	3-5852
U+78E4.0	fR/S	  112.10; 112.11
U+78E4.0	fTotalStrokes	 15
U+78E4.0	fCangjie	MRHSE
U+78E4.0	fFourCorner	17640
U+78E4.0	fHakka	 jin1; rin1; jun3; run3; jin3
U+78E4.0	fCantonese	jan2
U+78E4.0	fMandarin	yin3; yin1
U+78E4.0	fEnglish	the sound of thunder
U+78E5.0	fUTF8	磥
U+78E5.0	fCNS11643	2-586E
U+78E5.0	fBig5	EAA4
U+78E5.0	fGB	3-585D
U+78E5.0	fR/S	 112.11
U+78E5.0	fTotalStrokes	 16
U+78E5.0	fCangjie	MRWVF
U+78E5.0	fFourCorner	16693
U+78E5.0	fCantonese	leoi5
U+78E5.0	fMandarin	lei3; lei2
U+78E5.0	fEnglish	 heap of stones boulders
U+78E6.0	fUTF8	磦
U+78E6.0	fCNS11643	4-516E
U+78E6.0	fGB	E-3547
U+78E6.0	fR/S	  112.11
U+78E6.0	fTotalStrokes	 16
U+78E6.0	fCangjie	MRMWF
U+78E6.0	fFourCorner	11690
U+78E6.0	fCantonese	biu1
U+78E6.0	fMandarin	biao1
U+78E7.0	fUTF8	磧
U+78E7.0	fCNS11643	1-7156
U+78E7.0	fBig5	BF6C
U+78E7.0	fGB	1-6D53
U+78E7.0	fR/S	 112.11
U+78E7.0	fTotalStrokes	 16
U+78E7.0	fCangjie	MRQMC
U+78E7.0	fFourCorner	15686
U+78E7.0	fHakka	  zit7;  zak7
U+78E7.0	fCantonese	zik1; zaat3; zaak3; cik1
U+78E7.0	fMandarin	qi4
U+78E7.0	fEnglish	[1] sand and gravel in shallow waters [2] desert [3] press something on; put something on
U+78E8.0	fUTF8	磨
U+78E8.0	fCNS11643	1-7153
U+78E8.0	fBig5	BF69
U+78E8.0	fGB	0-4425
U+78E8.0	fR/S	 112.11
U+78E8.0	fTotalStrokes	 16
U+78E8.0	fCangjie	IDMR
U+78E8.0	fFourCorner	00261
U+78E8.0	fHakka	  mo2; mo3; mo5;  no2; mo1
U+78E8.0	fCantonese	mo4; mo6; mo2 (also)
U+78E8.0	fMandarin	mo2
U+78E8.0	fEnglish	[1] dadle; waste time; while away [2] rub; grind [3] sufferings; obstacles; setbacks
U+78E8.1	fUTF8	磨
U+78E8.1	fCNS11643	1-7153
U+78E8.1	fBig5	BF69
U+78E8.1	fGB	0-4425
U+78E8.1	fR/S	 112.11
U+78E8.1	fTotalStrokes	 16
U+78E8.1	fCangjie	IDMR
U+78E8.1	fFourCorner	00261
U+78E8.1	fHakka	 mo6
U+78E8.1	fMandarin	mo4
U+78E8.1	fEnglish	[1] mill [2] turn around [3] grind (grain, etc.)
U+78E8.2	fUTF8	磨
U+78E8.2	fCNS11643	1-7153
U+78E8.2	fBig5	BF69
U+78E8.2	fGB	0-4425
U+78E8.2	fR/S	 112.11
U+78E8.2	fTotalStrokes	 16
U+78E8.2	fCangjie	IDMR
U+78E8.2	fFourCorner	00261
U+78E8.2	fMandarin	mo5
U+78E9.0	fUTF8	磩
U+78E9.0	fCNS11643	2-586D
U+78E9.0	fBig5	EAA3
U+78E9.0	fGB	3-585B
U+78E9.0	fR/S	 112.11
U+78E9.0	fTotalStrokes	 16
U+78E9.0	fCangjie	MRIHF
U+78E9.0	fFourCorner	13650
U+78E9.0	fCantonese	cik1; cit1
U+78E9.0	fMandarin	qi1; qi4
U+78EA.0	fUTF8	磪
U+78EA.0	fCNS11643	2-586F
U+78EA.0	fBig5	EAA5
U+78EA.0	fGB	5-5858
U+78EA.0	fR/S	 112.11
U+78EA.0	fTotalStrokes	 16
U+78EA.0	fCangjie	MRUOG
U+78EA.0	fFourCorner	12614
U+78EA.0	fHakka	  cui1
U+78EA.0	fCantonese	ceoi4; ceoi1
U+78EA.0	fMandarin	cui1; cui3
U+78EA.0	fEnglish	 a high mountain precipitous
U+78EB.0	fUTF8	磫
U+78EB.0	fCNS11643	3-5222
U+78EB.0	fGB	3-5753
U+78EB.0	fR/S	  112.11
U+78EB.0	fTotalStrokes	 16
U+78EB.0	fCangjie	MRHOO
U+78EB.0	fFourCorner	18680
U+78EB.0	fCantonese	zung1
U+78EB.0	fMandarin	zong1
U+78EC.0	fUTF8	磬
U+78EC.0	fCNS11643	1-7155
U+78EC.0	fBig5	BF6B
U+78EC.0	fGB	0-6D60
U+78EC.0	fR/S	 112.11
U+78EC.0	fTotalStrokes	 16
U+78EC.0	fCangjie	GEMR
U+78EC.0	fFourCorner	47601
U+78EC.0	fHakka	 kin5; kin3; cin3; kin1
U+78EC.0	fCantonese	hing3
U+78EC.0	fMandarin	qing4; qing3
U+78EC.0	fEnglish	a musical instrument made by hollowing out a hard sonorous stone, etc); musical stone
U+78ED.0	fUTF8	磭
U+78ED.0	fCNS11643	2-5875
U+78ED.0	fBig5	EAAB
U+78ED.0	fGB	E-3548
U+78ED.0	fR/S	 112.11
U+78ED.0	fTotalStrokes	 16
U+78ED.0	fCangjie	MRMVB
U+78ED.0	fFourCorner	11627
U+78ED.0	fCantonese	coek3
U+78ED.0	fMandarin	chuo4; yin3
U+78EE.0	fUTF8	磮
U+78EE.0	fCNS11643	3-6575
U+78EE.0	fGB	E-3549
U+78EE.0	fR/S	 112.11
U+78EE.0	fTotalStrokes	 16
U+78EE.0	fCangjie	MRUOB
U+78EE.0	fFourCorner	1262
U+78EE.0	fCantonese	leon4
U+78EF.0	fUTF8	磯
U+78EF.0	fCNS11643	1-7450
U+78EF.0	fBig5	C146
U+78EF.0	fGB	1-6D36
U+78EF.0	fR/S	 112.12
U+78EF.0	fTotalStrokes	 17
U+78EF.0	fCangjie	MRVII
U+78EF.0	fFourCorner	12653
U+78EF.0	fHakka	  gi1
U+78EF.0	fCantonese	gei1
U+78EF.0	fMandarin	ji1; gai4
U+78EF.0	fEnglish	[1] a rocky cliff on the water's edge; water-surrounded rocks [2] rocks that water pounds against; stones in a river, interrupting and fretting the current
U+78F0.0	fUTF8	磰
U+78F0.0	fCNS11643	3-5560
U+78F0.0	fGB	5-5860
U+78F0.0	fR/S	 112.12
U+78F0.0	fTotalStrokes	 17
U+78F0.0	fCangjie	MRTTR
U+78F0.0	fFourCorner	18660
U+78F0.0	fHakka	 sen5; shan5; sen3; shen5; shen3
U+78F0.0	fCantonese	sin6
U+78F0.0	fMandarin	shan4
U+78F0.0	fEnglish	white clay of which porcelain is made
U+78F1.0	fUTF8	磱
U+78F1.0	fCNS11643	3-5562
U+78F1.0	fGB	E-354A
U+78F1.0	fR/S	 112.12
U+78F1.0	fTotalStrokes	 17
U+78F1.0	fCangjie	MRFFS
U+78F1.0	fFourCorner	19620
U+78F1.0	fCantonese	lou4
U+78F1.0	fMandarin	lao2
U+78F2.0	fUTF8	磲
U+78F2.0	fCNS11643	2-5D75
U+78F2.0	fBig5	EDAA
U+78F2.0	fGB	0-6D61
U+78F2.0	fR/S	 112.12
U+78F2.0	fTotalStrokes	 17
U+78F2.0	fCangjie	MRESD
U+78F2.0	fFourCorner	11694
U+78F2.0	fHakka	 ki2; gi2
U+78F2.0	fCantonese	keoi4
U+78F2.0	fMandarin	qu2
U+78F2.0	fEnglish	a white stone used for the opaque white buttons of the sixth rank
U+78F3.0	fUTF8	磳
U+78F3.0	fCNS11643	2-5D6F
U+78F3.0	fBig5	EDA5
U+78F3.0	fGB	3-5868
U+78F3.0	fR/S	 112.12
U+78F3.0	fTotalStrokes	 17
U+78F3.0	fCangjie	MRCWA
U+78F3.0	fFourCorner	18666
U+78F3.0	fHakka	 zen1;  cen2
U+78F3.0	fCantonese	zang1
U+78F3.0	fMandarin	zeng1; cheng2
U+78F3.0	fEnglish	rocky; stony
U+78F4.0	fUTF8	磴
U+78F4.0	fCNS11643	1-744F
U+78F4.0	fBig5	C145
U+78F4.0	fGB	0-6D63
U+78F4.0	fR/S	 112.12
U+78F4.0	fTotalStrokes	 17
U+78F4.0	fCangjie	MRNOT
U+78F4.0	fFourCorner	12618
U+78F4.0	fHakka	 den5
U+78F4.0	fCantonese	dang3
U+78F4.0	fMandarin	deng4; deng1
U+78F4.0	fEnglish	steps on rock
U+78F5.0	fUTF8	磵
U+78F5.0	fGB	5-5839
U+78F5.0	fR/S	 112.12
U+78F5.0	fTotalStrokes	 17
U+78F5.0	fCangjie	MRANA
U+78F5.0	fFourCorner	1762
U+78F5.0	fCantonese	gaan3
U+78F5.0	fMandarin	jian4
U+78F5.0	fEnglish	brook; mountain stream
U+78F6.0	fUTF8	磶
U+78F6.0	fCNS11643	4-573E
U+78F6.0	fGB	3-5864
U+78F6.0	fR/S	 112.12
U+78F6.0	fTotalStrokes	 17; 16
U+78F6.0	fCangjie	MRHXF
U+78F6.0	fFourCorner	17620
U+78F6.0	fHakka	  sit7
U+78F6.0	fCantonese	sik1
U+78F6.0	fMandarin	xi4
U+78F6.0	fEnglish	the stone on which a pillar rests
U+78F7.0	fUTF8	磷
U+78F7.0	fCNS11643	1-744D
U+78F7.0	fBig5	C143
U+78F7.0	fGB	0-4157
U+78F7.0	fR/S	 112.12; 112.13
U+78F7.0	fTotalStrokes	 17
U+78F7.0	fCangjie	MRFDQ
U+78F7.0	fFourCorner	19659
U+78F7.0	fHakka	  lin2
U+78F7.0	fCantonese	leon4; leon6; ling4
U+78F7.0	fMandarin	lin2; lin3; ling2
U+78F7.0	fEnglish	[1] phosphorus [2] water rushing between stones; water flowing between stones
U+78F7.1	fUTF8	磷
U+78F7.1	fCNS11643	1-744D
U+78F7.1	fBig5	C143
U+78F7.1	fGB	0-4157
U+78F7.1	fR/S	 112.12; 112.13
U+78F7.1	fTotalStrokes	 17
U+78F7.1	fCangjie	MRFDQ
U+78F7.1	fFourCorner	19659
U+78F7.1	fMandarin	lin4
U+78F7.1	fEnglish	thin
U+78F8.0	fUTF8	磸
U+78F8.0	fCNS11643	3-5561
U+78F8.0	fGB	E-354B
U+78F8.0	fR/S	 112.12
U+78F8.0	fTotalStrokes	 17
U+78F8.0	fCangjie	MRTWK
U+78F8.0	fFourCorner	18630
U+78F8.0	fCantonese	ding3
U+78F8.0	fMandarin	ding4
U+78F9.0	fUTF8	磹
U+78F9.0	fCNS11643	2-5D77
U+78F9.0	fBig5	EDAC
U+78F9.0	fGB	3-5861
U+78F9.0	fR/S	 112.12
U+78F9.0	fTotalStrokes	 17
U+78F9.0	fCangjie	MRMWJ
U+78F9.0	fFourCorner	11646
U+78F9.0	fCantonese	taam2; dim3
U+78F9.0	fMandarin	dian4; dan4
U+78FA.0	fUTF8	磺
U+78FA.0	fCNS11643	1-744E
U+78FA.0	fBig5	C144
U+78FA.0	fGB	0-3B47
U+78FA.0	fR/S	 112.12
U+78FA.0	fTotalStrokes	 17
U+78FA.0	fCangjie	MRTLC
U+78FA.0	fFourCorner	14686
U+78FA.0	fHakka	  wong2; wong3; kong5; vong2
U+78FA.0	fCantonese	wong4
U+78FA.0	fMandarin	huang2; gong3; kuang4
U+78FA.0	fEnglish	sulfur; brimstone
U+78FB.0	fUTF8	磻
U+78FB.0	fCNS11643	2-5D72
U+78FB.0	fBig5	EDA8
U+78FB.0	fGB	3-5866
U+78FB.0	fR/S	 112.12
U+78FB.0	fTotalStrokes	 17
U+78FB.0	fCangjie	MRHDW
U+78FB.0	fFourCorner	12669
U+78FB.0	fHakka	 pan1; pan2
U+78FB.0	fCantonese	pun4; bo1
U+78FB.0	fMandarin	pan2; bo1; bo4
U+78FB.0	fEnglish	a tributary of Wei river in Shaanxi
U+78FC.0	fUTF8	磼
U+78FC.0	fCNS11643	2-5D73
U+78FC.0	fBig5	EDA9
U+78FC.0	fGB	3-5865
U+78FC.0	fR/S	 112.12
U+78FC.0	fTotalStrokes	 17
U+78FC.0	fCangjie	MROGD
U+78FC.0	fFourCorner	10694
U+78FC.0	fHakka	 cap8
U+78FC.0	fCantonese	zaap6
U+78FC.0	fMandarin	za2
U+78FC.0	fEnglish	 high peak
U+78FD.0	fUTF8	磽
U+78FD.0	fCNS11643	2-5D70
U+78FD.0	fBig5	EDA6
U+78FD.0	fGB	1-6D4D
U+78FD.0	fR/S	 112.12
U+78FD.0	fTotalStrokes	 17
U+78FD.0	fCangjie	MRGGU
U+78FD.0	fFourCorner	14611
U+78FD.0	fHakka	  ngiau2; kau1; ngieu2
U+78FD.0	fCantonese	haau1
U+78FD.0	fMandarin	qiao1; qiao3; ao2; qiao4; yao3; yao4; yue4
U+78FD.0	fEnglish	hard barren land; hard and infertile
U+78FE.0	fUTF8	磾
U+78FE.0	fCNS11643	2-5D78
U+78FE.0	fBig5	EDAD
U+78FE.0	fGB	3-5838
U+78FE.0	fR/S	 112.12
U+78FE.0	fTotalStrokes	 17
U+78FE.0	fCangjie	MRRRJ
U+78FE.0	fFourCorner	16646
U+78FE.0	fHakka	  dai1;  di1
U+78FE.0	fCantonese	dai1
U+78FE.0	fMandarin	di1
U+78FE.0	fEnglish	a mineral used in dyeing silk black
U+78FF.0	fUTF8	磿
U+78FF.0	fCNS11643	2-5D74
U+78FF.0	fBig5	F056
U+78FF.0	fGB	3-3043
U+78FF.0	fR/S	 112.12
U+78FF.0	fTotalStrokes	 17
U+78FF.0	fCangjie	MHDR
U+78FF.0	fFourCorner	71261
U+78FF.0	fCantonese	lik6
U+78FF.0	fMandarin	li4
U+7900.0	fUTF8	礀
U+7900.0	fCNS11643	4-5740
U+7900.0	fGB	E-354C
U+7900.0	fR/S	 112.12
U+7900.0	fTotalStrokes	 17; 16
U+7900.0	fCangjie	MRANB
U+7900.0	fFourCorner	17620
U+7900.0	fCantonese	gaan3
U+7900.0	fMandarin	jian4
U+7901.0	fUTF8	礁
U+7901.0	fCNS11643	1-7451
U+7901.0	fBig5	C147
U+7901.0	fGB	0-3D38
U+7901.0	fR/S	 112.12
U+7901.0	fTotalStrokes	 17
U+7901.0	fCangjie	MROGF
U+7901.0	fFourCorner	10631
U+7901.0	fHakka	 ziau1; zeu1
U+7901.0	fCantonese	ziu1
U+7901.0	fMandarin	jiao1
U+7901.0	fEnglish	 reef
U+7902.0	fUTF8	礂
U+7902.0	fCNS11643	2-5D71
U+7902.0	fBig5	EDA7
U+7902.0	fGB	E-354D
U+7902.0	fR/S	 112.12
U+7902.0	fTotalStrokes	 17
U+7902.0	fCangjie	MRGRR
U+7902.0	fFourCorner	14661
U+7902.0	fCantonese	hei1
U+7902.0	fMandarin	xi1
U+7903.0	fUTF8	礃
U+7903.0	fGB	5-585F
U+7903.0	fR/S	 112.12
U+7903.0	fTotalStrokes	 17
U+7903.0	fCangjie	MRFBQ
U+7903.0	fCantonese	zoeng2
U+7903.0	fMandarin	zhang3
U+7904.0	fUTF8	礄
U+7904.0	fCNS11643	2-5D79
U+7904.0	fBig5	EDAE
U+7904.0	fGB	3-5774
U+7904.0	fR/S	 112.12
U+7904.0	fTotalStrokes	 17
U+7904.0	fCangjie	MRHKB
U+7904.0	fFourCorner	12627
U+7904.0	fCantonese	kiu4
U+7904.0	fMandarin	qiao2
U+7904.0	fEnglish	a place in Sichuan province
U+7905.0	fUTF8	礅
U+7905.0	fCNS11643	2-5D76
U+7905.0	fBig5	EDAB
U+7905.0	fGB	0-6D62
U+7905.0	fR/S	 112.12
U+7905.0	fTotalStrokes	 17
U+7905.0	fCangjie	MRYDK
U+7905.0	fFourCorner	18640
U+7905.0	fCantonese	deon1; dan1
U+7905.0	fMandarin	dun1
U+7906.0	fUTF8	礆
U+7906.0	fCNS11643	3-5843
U+7906.0	fGB	E-354E
U+7906.0	fR/S	 112.13
U+7906.0	fTotalStrokes	 18
U+7906.0	fCangjie	MROMO
U+7906.0	fFourCorner	18680
U+7906.0	fCantonese	him2
U+7906.0	fMandarin	xian3; jian3
U+7907.0	fUTF8	礇
U+7907.0	fCNS11643	3-583E
U+7907.0	fGB	E-354F
U+7907.0	fR/S	  112.13
U+7907.0	fTotalStrokes	 18
U+7907.0	fCangjie	MRHBK
U+7907.0	fFourCorner	17630
U+7907.0	fCantonese	juk1
U+7907.0	fMandarin	yu4
U+7908.0	fUTF8	礈
U+7908.0	fCNS11643	3-5840
U+7908.0	fGB	5-5861
U+7908.0	fR/S	 112.13
U+7908.0	fTotalStrokes	 18
U+7908.0	fCangjie	MRYTO
U+7908.0	fFourCorner	18630
U+7908.0	fHakka	 zhui5
U+7908.0	fCantonese	zeoi6
U+7908.0	fMandarin	zhui4
U+7909.0	fUTF8	礉
U+7909.0	fCNS11643	2-6246
U+7909.0	fBig5	F05A
U+7909.0	fGB	3-586B
U+7909.0	fR/S	 112.13
U+7909.0	fTotalStrokes	 18
U+7909.0	fCangjie	MRHSK
U+7909.0	fFourCorner	18640
U+7909.0	fCantonese	hat6; haau1; hau1
U+7909.0	fMandarin	he2; qiao4; qiao1
U+790A.0	fUTF8	礊
U+790A.0	fCNS11643	3-583F
U+790A.0	fGB	E-3550
U+790A.0	fR/S	  112.13
U+790A.0	fTotalStrokes	 18
U+790A.0	fCangjie	JEMR
U+790A.0	fFourCorner	57600
U+790A.0	fCantonese	hak1
U+790A.0	fMandarin	huo4
U+790B.0	fUTF8	礋
U+790B.0	fCNS11643	3-583C
U+790B.0	fGB	3-576A
U+790B.0	fR/S	 112.13
U+790B.0	fTotalStrokes	 18
U+790B.0	fCangjie	MRWLJ
U+790B.0	fFourCorner	16640
U+790B.0	fCantonese	zaak6
U+790B.0	fMandarin	zhai2; zhe4; ze2
U+790C.0	fUTF8	礌
U+790C.0	fCNS11643	2-6243
U+790C.0	fBig5	F057
U+790C.0	fGB	3-586A
U+790C.0	fR/S	 112.13
U+790C.0	fTotalStrokes	 18
U+790C.0	fCangjie	MRMBW
U+790C.0	fFourCorner	11662
U+790C.0	fCantonese	leoi6; leoi5
U+790C.0	fMandarin	lei4; lei3; lei2
U+790D.0	fUTF8	礍
U+790D.0	fCNS11643	3-5842
U+790D.0	fGB	E-3551
U+790D.0	fR/S	  112.13
U+790D.0	fTotalStrokes	 18
U+790D.0	fCangjie	MRTAV
U+790D.0	fFourCorner	14620
U+790D.0	fCantonese	kit3
U+790D.0	fMandarin	ke3
U+790E.0	fUTF8	礎
U+790E.0	fCNS11643	1-7670
U+790E.0	fBig5	C2A6
U+790E.0	fGB	1-3421
U+790E.0	fR/S	 112.13
U+790E.0	fTotalStrokes	 18
U+790E.0	fCangjie	MRDDO
U+790E.0	fFourCorner	14681
U+790E.0	fHakka	  co3; cu3; ze3; cih3
U+790E.0	fCantonese	co2
U+790E.0	fMandarin	chu3
U+790E.0	fEnglish	 plinth
U+790F.0	fUTF8	礏
U+790F.0	fCNS11643	4-5C32
U+790F.0	fGB	5-5823
U+790F.0	fR/S	  112.13
U+790F.0	fTotalStrokes	 18
U+790F.0	fCangjie	MRTCD
U+790F.0	fFourCorner	12690
U+790F.0	fCantonese	jap6
U+790F.0	fMandarin	ji2
U+7910.0	fUTF8	礐
U+7910.0	fCNS11643	2-6247
U+7910.0	fBig5	F05B
U+7910.0	fGB	3-5767
U+7910.0	fR/S	 112.13
U+7910.0	fTotalStrokes	 18
U+7910.0	fCangjie	HBMR
U+7910.0	fFourCorner	77601
U+7910.0	fHakka	  kok7;  kok8
U+7910.0	fCantonese	kok3
U+7910.0	fMandarin	que4; xue2; xiao2; hu2; le4; wu4; yue4
U+7910.0	fEnglish	[1] craggy [2] rocky
U+7911.0	fUTF8	礑
U+7911.0	fCNS11643	2-6249
U+7911.0	fBig5	F05D
U+7911.0	fGB	E-3552
U+7911.0	fR/S	 112.13
U+7911.0	fTotalStrokes	 18
U+7911.0	fCangjie	MRFBW
U+7911.0	fFourCorner	19666
U+7911.0	fCantonese	dong2; dong3
U+7911.0	fMandarin	dang4
U+7912.0	fUTF8	礒
U+7912.0	fCNS11643	2-6248
U+7912.0	fBig5	F05C
U+7912.0	fGB	3-574B
U+7912.0	fR/S	 112.13
U+7912.0	fTotalStrokes	 18
U+7912.0	fCangjie	MRTGI
U+7912.0	fFourCorner	18653
U+7912.0	fCantonese	ngai5; ai5 (also)
U+7912.0	fMandarin	yi3
U+7913.0	fUTF8	礓
U+7913.0	fCNS11643	2-6244
U+7913.0	fBig5	F058
U+7913.0	fGB	0-6D64
U+7913.0	fR/S	 112.13
U+7913.0	fTotalStrokes	 18
U+7913.0	fCangjie	MRMWM
U+7913.0	fFourCorner	11616
U+7913.0	fHakka	 giong1
U+7913.0	fCantonese	goeng1
U+7913.0	fMandarin	jiang1
U+7913.0	fEnglish	[1] gravel [2] small stones
U+7914.0	fUTF8	礔
U+7914.0	fCNS11643	2-6245
U+7914.0	fBig5	F059
U+7914.0	fGB	5-5865
U+7914.0	fR/S	 112.13
U+7914.0	fTotalStrokes	 18
U+7914.0	fCangjie	MRSRJ
U+7914.0	fFourCorner	10641
U+7914.0	fCantonese	pik1
U+7914.0	fMandarin	pi4
U+7915.0	fUTF8	礕
U+7915.0	fCNS11643	3-5844
U+7915.0	fGB	E-3553
U+7915.0	fR/S	  112.13
U+7915.0	fTotalStrokes	 18
U+7915.0	fCangjie	SJMR
U+7915.0	fFourCorner	70601
U+7915.0	fCantonese	pik1
U+7915.0	fMandarin	pi1
U+7916.0	fUTF8	礖
U+7916.0	fGB	E-3554
U+7916.0	fR/S	 112.13
U+7916.0	fTotalStrokes	 18
U+7916.0	fCangjie	MRHXC
U+7916.0	fCantonese	jyu6
U+7916.0	fMandarin	yu4
U+7917.0	fUTF8	礗
U+7917.0	fCNS11643	2-662D
U+7917.0	fBig5	F2A3
U+7917.0	fGB	3-5857
U+7917.0	fR/S	 112.14
U+7917.0	fTotalStrokes	 19
U+7917.0	fCangjie	MRJMC
U+7917.0	fFourCorner	13686
U+7917.0	fHakka	 bin1
U+7917.0	fCantonese	pan1; ban1
U+7917.0	fMandarin	pin1
U+7918.0	fUTF8	礘
U+7918.0	fCNS11643	3-5A61
U+7918.0	fGB	3-586D
U+7918.0	fR/S	 112.14
U+7918.0	fTotalStrokes	 19
U+7918.0	fCangjie	MRAVF
U+7918.0	fFourCorner	16630
U+7918.0	fCantonese	ngap6; ap6 (also)
U+7918.0	fMandarin	qi4
U+7919.0	fUTF8	礙
U+7919.0	fCNS11643	1-7855
U+7919.0	fBig5	C3AA
U+7919.0	fGB	1-302D
U+7919.0	fR/S	 112.14
U+7919.0	fTotalStrokes	 19
U+7919.0	fCangjie	MRPKO
U+7919.0	fFourCorner	17681
U+7919.0	fHakka	 ngoi5; ngoi3; ngai6; ngai5; ngoe3
U+7919.0	fCantonese	ngoi6; oi6 (also)
U+7919.0	fMandarin	ai4; yi2
U+7919.0	fEnglish	[1] obstruct; hinder; block; be in the way; stop progress [2] harmful; detrimental
U+791A.0	fUTF8	礚
U+791A.0	fCNS11643	3-5A62
U+791A.0	fGB	E-3555
U+791A.0	fR/S	  112.14
U+791A.0	fTotalStrokes	 19
U+791A.0	fCangjie	MRTGT
U+791A.0	fFourCorner	14610
U+791A.0	fCantonese	hap6
U+791A.0	fMandarin	kai4; ke1
U+791B.0	fUTF8	礛
U+791B.0	fCNS11643	2-662A
U+791B.0	fBig5	F27E
U+791B.0	fGB	3-5850
U+791B.0	fR/S	 112.14
U+791B.0	fTotalStrokes	 19
U+791B.0	fCangjie	MRSIT
U+791B.0	fFourCorner	11617
U+791B.0	fCantonese	laam4; gaam1
U+791B.0	fMandarin	jian1; lan2
U+791C.0	fUTF8	礜
U+791C.0	fCNS11643	2-662C
U+791C.0	fBig5	F2A2
U+791C.0	fGB	E-3556
U+791C.0	fR/S	 112.14
U+791C.0	fTotalStrokes	 19
U+791C.0	fCangjie	HCMR
U+791C.0	fFourCorner	77601
U+791C.0	fHakka	 ji5; ri6
U+791C.0	fCantonese	jyu6; jyu5
U+791C.0	fMandarin	yu4; yu2
U+791C.0	fEnglish	a white ore of arsenic found in Hupeh
U+791D.0	fUTF8	礝
U+791D.0	fCNS11643	2-6629
U+791D.0	fBig5	F27D
U+791D.0	fGB	5-5866
U+791D.0	fR/S	 112.14
U+791D.0	fTotalStrokes	 19
U+791D.0	fCangjie	MRMBB
U+791D.0	fFourCorner	11627
U+791D.0	fHakka	 nion1
U+791D.0	fCantonese	jyun5
U+791D.0	fMandarin	ruan3
U+791E.0	fUTF8	礞
U+791E.0	fCNS11643	2-662E
U+791E.0	fBig5	F2A4
U+791E.0	fGB	0-6D66
U+791E.0	fR/S	 112.14
U+791E.0	fTotalStrokes	 19
U+791E.0	fCangjie	MRTBO
U+791E.0	fFourCorner	14632
U+791E.0	fHakka	 mung2
U+791E.0	fCantonese	mung4
U+791E.0	fMandarin	meng2
U+791E.0	fEnglish	[1] a mineral [2] mica
U+791F.0	fUTF8	礟
U+791F.0	fCNS11643	4-6042
U+791F.0	fGB	E-3557
U+791F.0	fR/S	 112.14
U+791F.0	fTotalStrokes	 19
U+791F.0	fCangjie	MRSFK
U+791F.0	fFourCorner	14640
U+791F.0	fCantonese	paau3
U+791F.0	fMandarin	pao4
U+791F.0	fEnglish	ancient ballista for thro- wing heavy stones a cannon; a gun
U+7920.0	fUTF8	礠
U+7920.0	fCNS11643	3-5A63
U+7920.0	fGB	E-3558
U+7920.0	fR/S	 112.14; 112.13
U+7920.0	fTotalStrokes	 19
U+7920.0	fCangjie	MRTVP
U+7920.0	fFourCorner	14633
U+7920.0	fCantonese	ci4
U+7920.0	fMandarin	ci2
U+7921.0	fUTF8	礡
U+7921.0	fCNS11643	2-662B
U+7921.0	fBig5	F2A1
U+7921.0	fGB	E-3559
U+7921.0	fR/S	 112.14
U+7921.0	fTotalStrokes	 19
U+7921.0	fCangjie	MRTII
U+7921.0	fFourCorner	14642
U+7921.0	fHakka	 pok8
U+7921.0	fCantonese	bok6; bok3
U+7921.0	fMandarin	bo2
U+7921.0	fEnglish	fill, extend
U+7922.0	fUTF8	礢
U+7922.0	fCNS11643	3-6576
U+7922.0	fGB	E-355A
U+7922.0	fR/S	 112.15
U+7922.0	fTotalStrokes	 20
U+7922.0	fCangjie	MRTOV
U+7922.0	fFourCorner	1863
U+7922.0	fCantonese	joeng5
U+7923.0	fUTF8	礣
U+7923.0	fCNS11643	2-6949
U+7923.0	fBig5	F47A
U+7923.0	fGB	3-586C
U+7923.0	fR/S	 112.15
U+7923.0	fTotalStrokes	 20
U+7923.0	fCangjie	MRTWI
U+7923.0	fFourCorner	14653
U+7923.0	fHakka	 met8
U+7923.0	fCantonese	maat3; mat6; mit6
U+7923.0	fMandarin	mie4
U+7923.0	fEnglish	[1] pebbly [2] hard
U+7924.0	fUTF8	礤
U+7924.0	fCNS11643	2-694C
U+7924.0	fBig5	F47D
U+7924.0	fGB	0-6D65
U+7924.0	fR/S	 112.15
U+7924.0	fTotalStrokes	 20
U+7924.0	fCangjie	MRTBF
U+7924.0	fFourCorner	14691
U+7924.0	fHakka	 cat7; cai5
U+7924.0	fCantonese	caat3
U+7924.0	fMandarin	ca3
U+7924.0	fEnglish	[1] coarse stone [2] grind
U+7925.0	fUTF8	礥
U+7925.0	fCNS11643	2-6948
U+7925.0	fBig5	F479
U+7925.0	fGB	3-582F
U+7925.0	fR/S	 112.15; 112.16
U+7925.0	fTotalStrokes	 20
U+7925.0	fCangjie	MRSEC
U+7925.0	fFourCorner	17686
U+7925.0	fHakka	 hien2
U+7925.0	fCantonese	jin4
U+7925.0	fMandarin	xian2; xin2
U+7925.0	fEnglish	[1] difficult [2] slow [3] laborious
U+7926.0	fUTF8	礦
U+7926.0	fCNS11643	1-797C
U+7926.0	fBig5	C471
U+7926.0	fGB	1-3F73
U+7926.0	fR/S	 112.15
U+7926.0	fTotalStrokes	 20
U+7926.0	fCangjie	MRITC
U+7926.0	fFourCorner	10686
U+7926.0	fHakka	  kong5;  kwang3; kong3; kuang3; gwang3
U+7926.0	fCantonese	kong3; kwong3 (also); gwong3
U+7926.0	fMandarin	kuang4; gong3
U+7926.0	fEnglish	[1] mine; rich source [2] mineral, ore [3] mining
U+7927.0	fUTF8	礧
U+7927.0	fCNS11643	2-694A
U+7927.0	fBig5	F47B
U+7927.0	fGB	3-586F
U+7927.0	fR/S	 112.15
U+7927.0	fTotalStrokes	 20
U+7927.0	fCangjie	MRWWW
U+7927.0	fFourCorner	16660
U+7927.0	fHakka	 lui3; lui5
U+7927.0	fCantonese	leoi6; leoi5
U+7927.0	fMandarin	lei2; lei3; lei4
U+7927.0	fEnglish	roll
U+7928.0	fUTF8	礨
U+7928.0	fCNS11643	2-694B
U+7928.0	fBig5	F47C
U+7928.0	fGB	3-5870
U+7928.0	fR/S	 112.15
U+7928.0	fTotalStrokes	 20
U+7928.0	fCangjie	WWWR
U+7928.0	fFourCorner	60601
U+7928.0	fCantonese	leoi5
U+7928.0	fMandarin	lei3
U+7929.0	fUTF8	礩
U+7929.0	fCNS11643	2-694D
U+7929.0	fBig5	F47E
U+7929.0	fGB	3-5836
U+7929.0	fR/S	 112.15
U+7929.0	fTotalStrokes	 20
U+7929.0	fCangjie	MRHLC
U+7929.0	fFourCorner	12686
U+7929.0	fCantonese	zat1; zi3 (also); zat6
U+7929.0	fMandarin	zhi4
U+792A.0	fUTF8	礪
U+792A.0	fCNS11643	1-797D
U+792A.0	fBig5	C472
U+792A.0	fGB	1-6D42
U+792A.0	fR/S	 112.15
U+792A.0	fTotalStrokes	 20
U+792A.0	fCangjie	MRMTB
U+792A.0	fFourCorner	11627
U+792A.0	fHakka	 li5; li6; li3
U+792A.0	fCantonese	lai6
U+792A.0	fMandarin	li4
U+792A.0	fEnglish	[1] whetstone [2] sharpen (a knife)
U+792B.0	fUTF8	礫
U+792B.0	fCNS11643	1-7A21
U+792B.0	fBig5	C474
U+792B.0	fGB	1-4079
U+792B.0	fR/S	 112.15
U+792B.0	fTotalStrokes	 20
U+792B.0	fCangjie	MRVID
U+792B.0	fFourCorner	12694
U+792B.0	fHakka	 lit8; shok7; sok7
U+792B.0	fCantonese	lik1; lok6; lik6; nik1
U+792B.0	fMandarin	li4; luo4
U+792B.0	fEnglish	 gravel; pebble; shingle
U+792C.0	fUTF8	礬
U+792C.0	fCNS11643	1-797E
U+792C.0	fBig5	C473
U+792C.0	fGB	1-372F
U+792C.0	fR/S	 112.15
U+792C.0	fTotalStrokes	 20
U+792C.0	fCangjie	DDKMR
U+792C.0	fFourCorner	44601
U+792C.0	fHakka	  fan2; fan3
U+792C.0	fCantonese	faan4; faan6
U+792C.0	fMandarin	fan2
U+792C.0	fEnglish	alum; vitriol
U+792D.0	fUTF8	礭
U+792D.0	fCNS11643	2-6B6F
U+792D.0	fBig5	F5E1
U+792D.0	fGB	5-5868
U+792D.0	fR/S	 112.16
U+792D.0	fTotalStrokes	 21
U+792D.0	fCangjie	MRMBG
U+792D.0	fFourCorner	11614
U+792D.0	fCantonese	kok3
U+792D.0	fMandarin	que4
U+792E.0	fUTF8	礮
U+792E.0	fCNS11643	3-5E45
U+792E.0	fGB	E-355B
U+792E.0	fR/S	  112.16
U+792E.0	fTotalStrokes	 21
U+792E.0	fCangjie	MRSFK
U+792E.0	fFourCorner	10640
U+792E.0	fHakka	 pau5
U+792E.0	fCantonese	paau3
U+792E.0	fMandarin	pao4
U+792E.0	fEnglish	[1] artillery piece; cannon; gun [2] catapult
U+792F.0	fUTF8	礯
U+792F.0	fCNS11643	2-6B71
U+792F.0	fBig5	F5E3
U+792F.0	fGB	5-5854
U+792F.0	fR/S	 112.16
U+792F.0	fTotalStrokes	 21
U+792F.0	fCangjie	MRFFF
U+792F.0	fFourCorner	19693
U+792F.0	fCantonese	jing4; jing1
U+792F.0	fMandarin	ying1
U+7930.0	fUTF8	礰
U+7930.0	fCNS11643	3-5E43
U+7930.0	fGB	3-574E
U+7930.0	fR/S	  112.16
U+7930.0	fTotalStrokes	 21
U+7930.0	fCangjie	MRMHM
U+7930.0	fFourCorner	11610
U+7930.0	fCantonese	lik6
U+7930.0	fMandarin	li4
U+7931.0	fUTF8	礱
U+7931.0	fCNS11643	2-6B70
U+7931.0	fBig5	F5E2
U+7931.0	fGB	1-6D43
U+7931.0	fR/S	 112.16
U+7931.0	fTotalStrokes	 21
U+7931.0	fCangjie	YPMR
U+7931.0	fFourCorner	01601
U+7931.0	fHakka	 lung2; lun3
U+7931.0	fCantonese	lung4
U+7931.0	fMandarin	long2; long4
U+7931.0	fEnglish	[1] grind [2] a kind of mill
U+7932.0	fUTF8	礲
U+7932.0	fCNS11643	3-5E44
U+7932.0	fGB	5-5822
U+7932.0	fR/S	 112.16
U+7932.0	fTotalStrokes	 21
U+7932.0	fCangjie	MRYBP
U+7932.0	fFourCorner	11610
U+7932.0	fCantonese	lung4
U+7932.0	fMandarin	long2
U+7933.0	fUTF8	礳
U+7933.0	fGB	3-5873
U+7933.0	fR/S	 112.16
U+7933.0	fTotalStrokes	 21
U+7933.0	fCangjie	MRIDR
U+7933.0	fFourCorner	1066
U+7933.0	fCantonese	mo6
U+7933.0	fMandarin	mo4
U+7933.0	fEnglish	a place in Shanxi province
U+7934.0	fUTF8	礴
U+7934.0	fGB	0-6D67
U+7934.0	fR/S	 112.16
U+7934.0	fTotalStrokes	 21
U+7934.0	fCangjie	MRTEI
U+7934.0	fFourCorner	1464
U+7934.0	fHakka	 pok7;  pok8
U+7934.0	fCantonese	bok6; bok3
U+7934.0	fMandarin	bo2
U+7934.0	fEnglish	fill; extend
U+7935.0	fUTF8	礵
U+7935.0	fCNS11643	2-6D64
U+7935.0	fBig5	F6F6
U+7935.0	fGB	5-586A
U+7935.0	fR/S	 112.17
U+7935.0	fTotalStrokes	 22
U+7935.0	fCangjie	MRMBU
U+7935.0	fFourCorner	11662
U+7935.0	fHakka	  song1
U+7935.0	fCantonese	soeng1
U+7935.0	fMandarin	shuang1
U+7935.0	fEnglish	arsenic
U+7936.0	fUTF8	礶
U+7936.0	fCNS11643	F-6A66
U+7936.0	fGB	5-5869
U+7936.0	fR/S	 112.18
U+7936.0	fTotalStrokes	 23
U+7936.0	fCangjie	MRTRG
U+7936.0	fFourCorner	1461
U+7936.0	fHakka	 kwan5; gon5; gwon5
U+7936.0	fCantonese	gun3; gwun3 (also)
U+7936.0	fMandarin	guan4
U+7936.0	fEnglish	 jug; pitcher; jar; pot
U+7937.0	fUTF8	礷
U+7937.0	fCNS11643	3-603C
U+7937.0	fGB	5-5862
U+7937.0	fR/S	  112.18
U+7937.0	fTotalStrokes	 23
U+7937.0	fCangjie	MRTST
U+7937.0	fFourCorner	14610
U+7937.0	fCantonese	laam4
U+7937.0	fMandarin	lan2
U+7938.0	fUTF8	礸
U+7938.0	fCNS11643	2-7043
U+7938.0	fBig5	F8B5
U+7938.0	fGB	E-355C
U+7938.0	fR/S	 112.19
U+7938.0	fTotalStrokes	 24
U+7938.0	fCangjie	MRHUC
U+7938.0	fFourCorner	14686
U+7938.0	fCantonese	caat3
U+7938.0	fMandarin	zan3
U+7939.0	fUTF8	礹
U+7939.0	fCNS11643	2-712A
U+7939.0	fBig5	F8FA
U+7939.0	fGB	5-5830
U+7939.0	fR/S	 112.20
U+7939.0	fTotalStrokes	 25
U+7939.0	fCangjie	MRRRK
U+7939.0	fFourCorner	16648
U+7939.0	fHakka	 ngam2
U+7939.0	fCantonese	ngaam4; aam4 (also); jim5
U+7939.0	fMandarin	yan2; yan3
U+793A.0	fUTF8	示
U+793A.0	fCNS11643	1-467C
U+793A.0	fBig5	A5DC
U+793A.0	fGB	0-4A3E
U+793A.0	fR/S	 113.0
U+793A.0	fTotalStrokes	 5
U+793A.0	fCangjie	MMF
U+793A.0	fFourCorner	10901
U+793A.0	fHakka	  si5; shi5; shi6; sii5
U+793A.0	fCantonese	si6; kei4
U+793A.0	fMandarin	shi4; qi2; qi1; zhi4; shi2
U+793A.0	fEnglish	[1] show; indicate [2] make known; notify; notice [3] instruct [4] demonstrate
U+793B.0	fUTF8	礻
U+793B.0	fGB	0-6C6A
U+793B.0	fR/S	 113.0
U+793B.0	fTotalStrokes	 5
U+793B.0	fCangjie	INF
U+793B.0	fHakka	 shi6; sii5; shi5
U+793B.0	fCantonese	si6
U+793B.0	fMandarin	shi4zi4pang2; shi4
U+793B.0	fEnglish	 cult Kangxi radical number 113
U+793C.0	fUTF8	礼
U+793C.0	fCNS11643	3-2433
U+793C.0	fGB	0-4071
U+793C.0	fR/S	  113.1
U+793C.0	fTotalStrokes	 6
U+793C.0	fCangjie	IFU
U+793C.0	fFourCorner	32210
U+793C.0	fHakka	  li1;  li3
U+793C.0	fCantonese	lai5
U+793C.0	fMandarin	li3
U+793C.0	fEnglish	simplified form of &#x79AE;
U+793D.0	fUTF8	礽
U+793D.0	fCNS11643	2-2472
U+793D.0	fBig5	CB72
U+793D.0	fGB	5-574C
U+793D.0	fR/S	 113.2
U+793D.0	fTotalStrokes	 7
U+793D.0	fCangjie	IFNHS
U+793D.0	fFourCorner	37227
U+793D.0	fHakka	 jin2; rin2
U+793D.0	fCantonese	jing4
U+793D.0	fMandarin	reng2
U+793D.0	fEnglish	 blessing; happiness; bliss
U+793E.0	fUTF8	社
U+793E.0	fCNS11643	1-4F23
U+793E.0	fBig5	AAC0
U+793E.0	fGB	0-4967
U+793E.0	fR/S	 113.3
U+793E.0	fTotalStrokes	 8
U+793E.0	fCangjie	IFG
U+793E.0	fFourCorner	34210
U+793E.0	fHakka	 sa5;  sha1; sa1; sha5; sha6; sha3
U+793E.0	fCantonese	se5; se6
U+793E.0	fMandarin	she4
U+793E.0	fEnglish	[1] the god of the soil; god of land [2] society; community [3] association; organisation; corporation; agency
U+793F.0	fUTF8	礿
U+793F.0	fCNS11643	2-2843
U+793F.0	fBig5	CDA3
U+793F.0	fGB	3-567A
U+793F.0	fR/S	 113.3
U+793F.0	fTotalStrokes	 8
U+793F.0	fCangjie	IFPI
U+793F.0	fFourCorner	37220
U+793F.0	fHakka	 jok7; rok7
U+793F.0	fCantonese	joek6
U+793F.0	fMandarin	yue4; yao4
U+793F.0	fEnglish	sacrifice
U+7940.0	fUTF8	祀
U+7940.0	fCNS11643	1-4F24
U+7940.0	fBig5	AAC1
U+7940.0	fGB	0-6C6B
U+7940.0	fR/S	 113.3
U+7940.0	fTotalStrokes	 8
U+7940.0	fCangjie	IFRU
U+7940.0	fFourCorner	37217
U+7940.0	fHakka	 sii5; su3; sii6; cu5; su5; se5
U+7940.0	fCantonese	zi6
U+7940.0	fMandarin	si4
U+7940.0	fEnglish	 sacrifice; worship; offer sacrifices to
U+7941.0	fUTF8	祁
U+7941.0	fCNS11643	1-4F25
U+7941.0	fBig5	AAC2
U+7941.0	fGB	0-466E
U+7941.0	fR/S	 113.3
U+7941.0	fTotalStrokes	 8
U+7941.0	fCangjie	IFNL
U+7941.0	fFourCorner	37227
U+7941.0	fHakka	  ki2; ki3
U+7941.0	fCantonese	kei4
U+7941.0	fMandarin	qi2; zhi3
U+7941.0	fEnglish	vigorous; thriving; be large
U+7942.0	fUTF8	祂
U+7942.0	fCNS11643	2-2842
U+7942.0	fBig5	CDA2
U+7942.0	fGB	E-355D
U+7942.0	fR/S	 113.3
U+7942.0	fTotalStrokes	 8
U+7942.0	fCangjie	IFPD
U+7942.0	fFourCorner	34212
U+7942.0	fHakka	 ta1
U+7942.0	fCantonese	taa1
U+7942.0	fMandarin	ta1
U+7942.0	fEnglish	he
U+7943.0	fUTF8	祃
U+7943.0	fGB	8-7D37
U+7943.0	fR/S	 113.3
U+7943.0	fTotalStrokes	 8
U+7943.0	fCangjie	IFNVM
U+7943.0	fCantonese	maa6
U+7943.0	fMandarin	ma4
U+7943.0	fEnglish	a sacrifice at the beginning of a military campaign
U+7944.0	fUTF8	祄
U+7944.0	fCNS11643	2-2C5A
U+7944.0	fBig5	CFF8
U+7944.0	fGB	5-574E
U+7944.0	fR/S	 113.4
U+7944.0	fTotalStrokes	 9
U+7944.0	fCangjie	IFOLL
U+7944.0	fFourCorner	38320
U+7944.0	fCantonese	gaai3; haai6
U+7944.0	fMandarin	xie4; jie4
U+7945.0	fUTF8	祅
U+7945.0	fCNS11643	2-2C59
U+7945.0	fBig5	CFF7
U+7945.0	fGB	E-355E
U+7945.0	fR/S	 113.4
U+7945.0	fTotalStrokes	 9
U+7945.0	fCangjie	IFHK
U+7945.0	fFourCorner	32234
U+7945.0	fHakka	 jau1; rau1; jeu1
U+7945.0	fCantonese	jiu1; jiu2 (also); hin1
U+7945.0	fMandarin	yao1; xian1
U+7945.0	fEnglish	[1] calamity due to terrestrial disturbances [2] bizarre]
U+7946.0	fUTF8	祆
U+7946.0	fCNS11643	1-5269
U+7946.0	fBig5	ACE6
U+7946.0	fGB	0-6C6C
U+7946.0	fR/S	 113.4
U+7946.0	fTotalStrokes	 9
U+7946.0	fCangjie	IFMK
U+7946.0	fFourCorner	32234
U+7946.0	fHakka	 hien1
U+7946.0	fCantonese	hin1; jiu1
U+7946.0	fMandarin	xian1; tian1; yao1
U+7946.0	fEnglish	Ormazda, god of the Zoroastrians; extended to god of the Manicheans
U+7947.0	fUTF8	祇
U+7947.0	fCNS11643	1-526C
U+7947.0	fBig5	ACE9
U+7947.0	fGB	8-7D38
U+7947.0	fR/S	 113.4
U+7947.0	fTotalStrokes	 9
U+7947.0	fCangjie	IFHVP
U+7947.0	fFourCorner	32240
U+7947.0	fHakka	  ki2; ki3; zi1; zi3; di3
U+7947.0	fCantonese	kei4; zi2
U+7947.0	fMandarin	qi2; chi2
U+7947.0	fEnglish	[1] god of the earth [2] peace; serenity; be at rest
U+7947.1	fUTF8	祇
U+7947.1	fCNS11643	1-526C
U+7947.1	fBig5	ACE9
U+7947.1	fGB	8-7D38
U+7947.1	fR/S	 113.4
U+7947.1	fTotalStrokes	 9
U+7947.1	fCangjie	IFHVP
U+7947.1	fFourCorner	32240
U+7947.1	fMandarin	zhi3; zhi1
U+7947.1	fEnglish	only; merely
U+7948.0	fUTF8	祈
U+7948.0	fCNS11643	1-526B
U+7948.0	fBig5	ACE8
U+7948.0	fGB	0-466D
U+7948.0	fR/S	 113.4
U+7948.0	fTotalStrokes	 9
U+7948.0	fCangjie	IFHML
U+7948.0	fFourCorner	32221
U+7948.0	fHakka	  ki2; ki3; ki1
U+7948.0	fCantonese	kei4
U+7948.0	fMandarin	qi2; gui3
U+7948.0	fEnglish	[1] pray [2] beg; entreat; beseech; supplicate; request respectfully
U+7949.0	fUTF8	祉
U+7949.0	fCNS11643	1-526A
U+7949.0	fBig5	ACE7
U+7949.0	fGB	0-6C6D
U+7949.0	fR/S	 113.4
U+7949.0	fTotalStrokes	 9
U+7949.0	fCangjie	IFYLM
U+7949.0	fFourCorner	31210
U+7949.0	fHakka	 zhi3; zi3; zih3; tje3
U+7949.0	fCantonese	zi2; ci2
U+7949.0	fMandarin	zhi3
U+7949.0	fEnglish	happiness; blessings; blessedness; welfare
U+794A.0	fUTF8	祊
U+794A.0	fCNS11643	2-2C56
U+794A.0	fBig5	CFF4
U+794A.0	fGB	3-5724
U+794A.0	fR/S	 113.4
U+794A.0	fTotalStrokes	 9
U+794A.0	fCangjie	IFYHS
U+794A.0	fFourCorner	30227
U+794A.0	fHakka	 fong1
U+794A.0	fCantonese	bang1; paang1
U+794A.0	fMandarin	beng1
U+794A.0	fEnglish	 a river in Shandong space inside
U+794B.0	fUTF8	祋
U+794B.0	fCNS11643	2-2C58
U+794B.0	fBig5	CFF6
U+794B.0	fGB	3-5723
U+794B.0	fR/S	 113.4
U+794B.0	fTotalStrokes	 9
U+794B.0	fCangjie	IFHNE
U+794B.0	fFourCorner	37247
U+794B.0	fHakka	  dai5; dot7
U+794B.0	fCantonese	doi6; deoi3
U+794B.0	fMandarin	dui4; duo2
U+794B.0	fEnglish	 spear; halberd
U+794C.0	fUTF8	祌
U+794C.0	fCNS11643	2-2C57
U+794C.0	fBig5	CFF5
U+794C.0	fGB	3-5722
U+794C.0	fR/S	 113.4
U+794C.0	fTotalStrokes	 9
U+794C.0	fCangjie	IFL
U+794C.0	fFourCorner	35200
U+794C.0	fCantonese	zung6; cung1
U+794C.0	fMandarin	zhong4; zhong3
U+794D.0	fUTF8	祍
U+794D.0	fCNS11643	3-6577
U+794D.0	fGB	E-355F
U+794D.0	fR/S	 113.4
U+794D.0	fTotalStrokes	 9
U+794D.0	fCangjie	IFHG
U+794D.0	fFourCorner	3221
U+794D.0	fCantonese	jam6
U+794E.0	fUTF8	祎
U+794E.0	fGB	8-2E62
U+794E.0	fR/S	 113.4
U+794E.0	fTotalStrokes	 9
U+794E.0	fCangjie	IFQS
U+794E.0	fCantonese	ji1
U+794E.0	fMandarin	yi1
U+794E.0	fEnglish	excellent
U+794F.0	fUTF8	祏
U+794F.0	fCNS11643	2-314B
U+794F.0	fBig5	D2E8
U+794F.0	fGB	3-5725
U+794F.0	fR/S	 113.5
U+794F.0	fTotalStrokes	 10
U+794F.0	fCangjie	IFMR
U+794F.0	fFourCorner	31260
U+794F.0	fHakka	 shak8; sak8
U+794F.0	fCantonese	sek6; sik6
U+794F.0	fMandarin	shi2
U+794F.0	fEnglish	shrine
U+7950.0	fUTF8	祐
U+7950.0	fCNS11643	1-572A
U+7950.0	fBig5	AFA7
U+7950.0	fGB	3-5726
U+7950.0	fR/S	 113.5
U+7950.0	fTotalStrokes	 10
U+7950.0	fCangjie	IFKR
U+7950.0	fFourCorner	34260
U+7950.0	fHakka	 ju5; riu6; jiu6; ju3; jiu3
U+7950.0	fCantonese	jau6
U+7950.0	fMandarin	you4
U+7950.0	fEnglish	 divine intervention; divine protection; divine help
U+7951.0	fUTF8	祑
U+7951.0	fCNS11643	2-314F
U+7951.0	fBig5	D2EC
U+7951.0	fGB	5-5751
U+7951.0	fR/S	 113.5
U+7951.0	fTotalStrokes	 10
U+7951.0	fCangjie	IFHQO
U+7951.0	fFourCorner	35230
U+7951.0	fCantonese	zat6
U+7951.0	fMandarin	zhi4
U+7952.0	fUTF8	祒
U+7952.0	fCNS11643	2-314E
U+7952.0	fBig5	D2EB
U+7952.0	fGB	E-3560
U+7952.0	fR/S	 113.5
U+7952.0	fTotalStrokes	 10
U+7952.0	fCangjie	IFSHR
U+7952.0	fFourCorner	37262
U+7952.0	fCantonese	tiu4
U+7952.0	fMandarin	tiao2; chao1; shao4
U+7953.0	fUTF8	祓
U+7953.0	fCNS11643	2-314D
U+7953.0	fBig5	D2EA
U+7953.0	fGB	0-6C70
U+7953.0	fR/S	 113.5
U+7953.0	fTotalStrokes	 10
U+7953.0	fCangjie	IFIKK
U+7953.0	fFourCorner	33247
U+7953.0	fHakka	 fut7
U+7953.0	fCantonese	fat1
U+7953.0	fMandarin	fu2; fei4; bai4
U+7953.0	fEnglish	[1] exorcise; remove evil [2] clean; cleanse
U+7954.0	fUTF8	祔
U+7954.0	fCNS11643	2-3149
U+7954.0	fBig5	D2E6
U+7954.0	fGB	3-5727
U+7954.0	fR/S	 113.5
U+7954.0	fTotalStrokes	 10
U+7954.0	fCangjie	IFODI
U+7954.0	fFourCorner	34200
U+7954.0	fHakka	 fu5
U+7954.0	fCantonese	fu6
U+7954.0	fMandarin	fu4
U+7954.0	fEnglish	[1] worship one's ancestors; enshrine in the ancestral temple [2] entomb; bury in the family tomb
U+7955.0	fUTF8	祕
U+7955.0	fCNS11643	1-5729
U+7955.0	fBig5	AFA6
U+7955.0	fGB	E-3561
U+7955.0	fR/S	 113.5
U+7955.0	fTotalStrokes	 10
U+7955.0	fCangjie	IFPH
U+7955.0	fFourCorner	33200
U+7955.0	fHakka	  bi5; cok8
U+7955.0	fCantonese	bei3; bai3
U+7955.0	fMandarin	mi4; bi4
U+7955.0	fEnglish	mysterious; secret; hidden; confidential; unknown
U+7956.0	fUTF8	祖
U+7956.0	fCNS11643	1-572D
U+7956.0	fBig5	AFAA
U+7956.0	fGB	0-5766
U+7956.0	fR/S	 113.5
U+7956.0	fTotalStrokes	 10
U+7956.0	fCangjie	IFBM
U+7956.0	fFourCorner	37210
U+7956.0	fHakka	  zu3; zih3; zii3
U+7956.0	fCantonese	zou2
U+7956.0	fMandarin	zu3; jie1
U+7956.0	fEnglish	[1] ancestor; forebear; forefather [2] founder; originator [3] one's grandfather or grandmother [4] follow the example; imitate
U+7957.0	fUTF8	祗
U+7957.0	fCNS11643	1-5730
U+7957.0	fBig5	AFAD
U+7957.0	fGB	0-6C73
U+7957.0	fR/S	 113.5
U+7957.0	fTotalStrokes	 10
U+7957.0	fCangjie	IFHPM
U+7957.0	fFourCorner	32240
U+7957.0	fHakka	 di3; zhi1; zih1; zi1; di5
U+7957.0	fCantonese	zi1; zi2
U+7957.0	fMandarin	zhi1
U+7957.0	fEnglish	[1] respect; look up to; revere [2] only; but; just
U+7958.0	fUTF8	祘
U+7958.0	fCNS11643	3-326F
U+7958.0	fGB	5-5748
U+7958.0	fR/S	 113.5
U+7958.0	fTotalStrokes	 10
U+7958.0	fCangjie	IFMMF
U+7958.0	fFourCorner	31291
U+7958.0	fCantonese	syun3
U+7958.0	fMandarin	suan4
U+7958.0	fEnglish	 calculate
U+7959.0	fUTF8	祙
U+7959.0	fCNS11643	3-3270
U+7959.0	fGB	5-574F
U+7959.0	fR/S	 113.5
U+7959.0	fTotalStrokes	 10
U+7959.0	fCangjie	IFJD
U+7959.0	fFourCorner	35290
U+7959.0	fCantonese	mei6
U+7959.0	fMandarin	mei4
U+7959.0	fEnglish	demon
U+795A.0	fUTF8	祚
U+795A.0	fCNS11643	1-5731
U+795A.0	fBig5	AFAE
U+795A.0	fGB	0-6C71
U+795A.0	fR/S	 113.5
U+795A.0	fTotalStrokes	 10
U+795A.0	fCangjie	IFOS
U+795A.0	fFourCorner	38211
U+795A.0	fHakka	 cu5; su5; za5; cih5; sii5; ze5; cok8
U+795A.0	fCantonese	zou6
U+795A.0	fMandarin	zuo4; zu4
U+795A.0	fEnglish	[1] throne [2] bless; blessing [3] the year
U+795B.0	fUTF8	祛
U+795B.0	fCNS11643	2-314A
U+795B.0	fBig5	D2E7
U+795B.0	fGB	0-6C6E
U+795B.0	fR/S	 113.5
U+795B.0	fTotalStrokes	 10
U+795B.0	fCangjie	IFGI
U+795B.0	fFourCorner	34231
U+795B.0	fHakka	 ki1; kia5
U+795B.0	fCantonese	keoi1
U+795B.0	fMandarin	qu1
U+795B.0	fEnglish	 dispel; expel; remove
U+795C.0	fUTF8	祜
U+795C.0	fCNS11643	2-314C
U+795C.0	fBig5	D2E9
U+795C.0	fGB	0-6C6F
U+795C.0	fR/S	 113.5
U+795C.0	fTotalStrokes	 10
U+795C.0	fCangjie	IFJR
U+795C.0	fFourCorner	34260
U+795C.0	fHakka	 gu3; fu3; fu5; fu6
U+795C.0	fCantonese	wu2; wu6 (also)
U+795C.0	fMandarin	hu4
U+795C.0	fEnglish	 blessing; happiness; prosperity
U+795D.0	fUTF8	祝
U+795D.0	fCNS11643	1-572F
U+795D.0	fBig5	AFAC
U+795D.0	fGB	0-5723
U+795D.0	fR/S	 113.5
U+795D.0	fTotalStrokes	 10
U+795D.0	fCangjie	IFRHU
U+795D.0	fFourCorner	36210
U+795D.0	fHakka	  zuk7;  chuk7; zu5; ziu5; zhuk7
U+795D.0	fCantonese	zuk1; zau3
U+795D.0	fMandarin	zhu4; zhou4; chu4
U+795D.0	fEnglish	[1] wish some one happiness; pray for happiness or blessings [2] congratulate; felicitate [3] celebrate [4] a Chinese family name
U+795E.0	fUTF8	神
U+795E.0	fCNS11643	1-572E
U+795E.0	fBig5	AFAB
U+795E.0	fGB	0-4971
U+795E.0	fR/S	 113.5
U+795E.0	fTotalStrokes	 10
U+795E.0	fCangjie	IFLWL
U+795E.0	fFourCorner	35206
U+795E.0	fHakka	 sin2;  shin2; siin2; shin3
U+795E.0	fCantonese	san4; san1; san2 (also)
U+795E.0	fMandarin	shen2; shen1
U+795E.0	fEnglish	[1] spirit; god; supernatural being; deity; immortal [2] soul; mind; spirit [3] appearance; looks; expressions; air [4] supernatural; marvelous; wondrous; miraculous; mysterious; mystical [5] smart; clever
U+795E.1	fUTF8	神
U+795E.1	fCNS11643	1-572E
U+795E.1	fBig5	AFAB
U+795E.1	fGB	0-4971
U+795E.1	fR/S	 113.5
U+795E.1	fTotalStrokes	 10
U+795E.1	fCangjie	IFLWL
U+795E.1	fFourCorner	35206
U+795E.1	fMandarin	shen5
U+795F.0	fUTF8	祟
U+795F.0	fCNS11643	1-572C
U+795F.0	fBig5	AFA9
U+795F.0	fGB	0-4B6E
U+795F.0	fR/S	 113.5
U+795F.0	fTotalStrokes	 10
U+795F.0	fCangjie	UUMMF
U+795F.0	fFourCorner	22901
U+795F.0	fHakka	  sui5; sui6
U+795F.0	fCantonese	seoi6
U+795F.0	fMandarin	sui4; xu4
U+795F.0	fEnglish	[1] evil spirit; evil influence of gods or demons [2] haunt; afflict (ghost or evil spirits)
U+7960.0	fUTF8	祠
U+7960.0	fCNS11643	1-572B
U+7960.0	fBig5	AFA8
U+7960.0	fGB	0-6C74
U+7960.0	fR/S	 113.5
U+7960.0	fTotalStrokes	 10
U+7960.0	fCangjie	IFSMR
U+7960.0	fFourCorner	37220
U+7960.0	fHakka	 cih2; cu2; cih3; ze2
U+7960.0	fCantonese	ci4
U+7960.0	fMandarin	ci2; si4
U+7960.0	fEnglish	[1] (ancestral) temple; shrine [2] the spring worship
U+7961.0	fUTF8	祡
U+7961.0	fCNS11643	2-3765
U+7961.0	fBig5	D6C2
U+7961.0	fGB	5-5749
U+7961.0	fR/S	 113.5
U+7961.0	fTotalStrokes	 10
U+7961.0	fCangjie	YPMMF
U+7961.0	fFourCorner	21901
U+7961.0	fHakka	  cai2
U+7961.0	fCantonese	caai4
U+7961.0	fMandarin	chai2
U+7961.0	fEnglish	[1] burn wood in sacrifice [2] the wood thus burnt
U+7962.0	fUTF8	祢
U+7962.0	fCNS11643	3-3272
U+7962.0	fGB	0-6C72
U+7962.0	fR/S	 113.5
U+7962.0	fTotalStrokes	 10; 9
U+7962.0	fCangjie	IFNF
U+7962.0	fFourCorner	37290
U+7962.0	fHakka	 ni3
U+7962.0	fCantonese	nei5; nei4; lei5 (also); lei4 (also)
U+7962.0	fMandarin	ni3; mi2
U+7962.0	fEnglish	simplified form of &#x79B0;
U+7963.0	fUTF8	祣
U+7963.0	fCNS11643	2-3763
U+7963.0	fBig5	D6C0
U+7963.0	fGB	5-5757
U+7963.0	fR/S	 113.6
U+7963.0	fTotalStrokes	 11
U+7963.0	fCangjie	IFOHV
U+7963.0	fFourCorner	38232
U+7963.0	fHakka	  li3
U+7963.0	fCantonese	leoi5
U+7963.0	fMandarin	luu3; l3
U+7963.0	fEnglish	imperial sacrifice
U+7964.0	fUTF8	祤
U+7964.0	fCNS11643	2-375F
U+7964.0	fBig5	D6BC
U+7964.0	fGB	E-3562
U+7964.0	fR/S	 113.6
U+7964.0	fTotalStrokes	 11
U+7964.0	fCangjie	IFSMM
U+7964.0	fFourCorner	37220
U+7964.0	fCantonese	jyu5; heoi2 (also)
U+7964.0	fMandarin	yu3; xu3
U+7965.0	fUTF8	祥
U+7965.0	fCNS11643	1-5C40
U+7965.0	fBig5	B2BB
U+7965.0	fGB	0-4F69
U+7965.0	fR/S	 113.6
U+7965.0	fTotalStrokes	 11
U+7965.0	fCangjie	IFTQ
U+7965.0	fFourCorner	38251
U+7965.0	fHakka	  siong2; siong3
U+7965.0	fCantonese	coeng4; joeng4
U+7965.0	fMandarin	xiang2
U+7965.0	fEnglish	auspicious; propitious; favorable
U+7966.0	fUTF8	祦
U+7966.0	fCNS11643	3-3D5E
U+7966.0	fGB	E-3563
U+7966.0	fR/S	  113.7
U+7966.0	fTotalStrokes	 12
U+7966.0	fCangjie	IFRMK
U+7966.0	fFourCorner	36230
U+7966.0	fCantonese	ng4
U+7966.0	fMandarin	wu2
U+7966.0	fEnglish	 happiness good fortune good luck blessing bliss
U+7967.0	fUTF8	祧
U+7967.0	fCNS11643	2-3760
U+7967.0	fBig5	D6BD
U+7967.0	fGB	0-6C76
U+7967.0	fR/S	 113.6
U+7967.0	fTotalStrokes	 11
U+7967.0	fCangjie	IFLMO
U+7967.0	fFourCorner	32213
U+7967.0	fHakka	 tiau1
U+7967.0	fCantonese	tiu1
U+7967.0	fMandarin	tiao1
U+7967.0	fEnglish	[1] ancestral hall; ancestral temple (especially of distant ancestors) [2] be heir to; become heir to
U+7968.0	fUTF8	票
U+7968.0	fCNS11643	1-5C41
U+7968.0	fBig5	B2BC
U+7968.0	fGB	0-4631
U+7968.0	fR/S	 113.6
U+7968.0	fTotalStrokes	 11
U+7968.0	fCangjie	MWMMF
U+7968.0	fFourCorner	10901
U+7968.0	fHakka	  piau5;  biau1; piau3; peu3; piau1
U+7968.0	fCantonese	piu3; biu1; piu2 (also); piu1; biu6
U+7968.0	fMandarin	piao4; piao1; biao1
U+7968.0	fEnglish	[1] bill; note [2] slip of paper or bamboo; ticket [3] ballot [4] hostage [5] amateur performance
U+7969.0	fUTF8	祩
U+7969.0	fCNS11643	2-3761
U+7969.0	fBig5	D6BE
U+7969.0	fGB	5-5756
U+7969.0	fR/S	 113.6
U+7969.0	fTotalStrokes	 11
U+7969.0	fCangjie	IFHJD
U+7969.0	fFourCorner	35290
U+7969.0	fHakka	 zhu1; zu1
U+7969.0	fCantonese	zyu3; zyu1
U+7969.0	fMandarin	zhu1; zhu4
U+7969.0	fEnglish	 curse
U+796A.0	fUTF8	祪
U+796A.0	fCNS11643	2-3762
U+796A.0	fBig5	D6BF
U+796A.0	fGB	5-5758
U+796A.0	fR/S	 113.6
U+796A.0	fTotalStrokes	 11
U+796A.0	fCangjie	IFNMU
U+796A.0	fFourCorner	37212
U+796A.0	fCantonese	gwai2
U+796A.0	fMandarin	gui3; hui3; gui4
U+796B.0	fUTF8	祫
U+796B.0	fCNS11643	2-3764
U+796B.0	fBig5	D6C1
U+796B.0	fGB	3-572A
U+796B.0	fR/S	 113.6
U+796B.0	fTotalStrokes	 11
U+796B.0	fCangjie	IFOMR
U+796B.0	fFourCorner	38261
U+796B.0	fHakka	 giap7; kap7
U+796B.0	fCantonese	haap6
U+796B.0	fMandarin	xia2
U+796B.0	fEnglish	triennial sacrifice one's ancest
U+796C.0	fUTF8	祬
U+796C.0	fCNS11643	3-3766
U+796C.0	fGB	5-5754
U+796C.0	fR/S	 113.6
U+796C.0	fTotalStrokes	 11
U+796C.0	fCangjie	IFMIG
U+796C.0	fFourCorner	31210
U+796C.0	fCantonese	zi1
U+796C.0	fMandarin	zhi1
U+796D.0	fUTF8	祭
U+796D.0	fCNS11643	1-5C42
U+796D.0	fBig5	B2BD
U+796D.0	fGB	0-3C40
U+796D.0	fR/S	 113.6
U+796D.0	fTotalStrokes	 11
U+796D.0	fCangjie	BOMMF
U+796D.0	fFourCorner	27901
U+796D.0	fHakka	  zi5
U+796D.0	fCantonese	zai3; zaai3
U+796D.0	fMandarin	ji4; zhai4
U+796D.0	fEnglish	[1] sacrifice to; worship; honor by a rite or service [2] wield
U+796E.0	fUTF8	祮
U+796E.0	fCNS11643	3-3765
U+796E.0	fGB	E-3564
U+796E.0	fR/S	 113.6
U+796E.0	fTotalStrokes	 11
U+796E.0	fCangjie	IFGR
U+796E.0	fFourCorner	34260
U+796E.0	fCantonese	haau2
U+796E.0	fMandarin	gao4
U+796F.0	fUTF8	祯
U+796F.0	fGB	0-6C75
U+796F.0	fR/S	 113.6
U+796F.0	fTotalStrokes	 11
U+796F.0	fCangjie	IFYBO
U+796F.0	fHakka	 zhin1
U+796F.0	fCantonese	zing1
U+796F.0	fMandarin	zhen1; zheng1
U+796F.0	fEnglish	simplified form of &#x798E;
U+7970.0	fUTF8	祰
U+7970.0	fCNS11643	2-3E59
U+7970.0	fBig5	DAD5
U+7970.0	fGB	5-575B
U+7970.0	fR/S	 113.7
U+7970.0	fTotalStrokes	 12
U+7970.0	fCangjie	IFHGR
U+7970.0	fFourCorner	34261
U+7970.0	fCantonese	haau2; guk1
U+7970.0	fMandarin	gao4; kao3; gu4
U+7971.0	fUTF8	祱
U+7971.0	fCNS11643	3-3D5F
U+7971.0	fGB	5-575C
U+7971.0	fR/S	  113.7
U+7971.0	fTotalStrokes	 12; 11
U+7971.0	fCangjie	IFCRU
U+7971.0	fFourCorner	38210
U+7971.0	fCantonese	seoi3
U+7971.0	fMandarin	shui4; lei4
U+7972.0	fUTF8	祲
U+7972.0	fCNS11643	2-3E58
U+7972.0	fBig5	DAD4
U+7972.0	fGB	3-572D
U+7972.0	fR/S	 113.7
U+7972.0	fTotalStrokes	 12
U+7972.0	fCangjie	IFSME
U+7972.0	fFourCorner	37247
U+7972.0	fHakka	 cim1;  zim5
U+7972.0	fCantonese	zam1; zam3
U+7972.0	fMandarin	jin4; jin1; qin3
U+7972.0	fEnglish	[1] ominous or sinister spirit [2] vigorous
U+7973.0	fUTF8	祳
U+7973.0	fCNS11643	2-3E57
U+7973.0	fBig5	DAD3
U+7973.0	fGB	E-3565
U+7973.0	fR/S	 113.7
U+7973.0	fTotalStrokes	 12
U+7973.0	fCangjie	IFMMV
U+7973.0	fFourCorner	31232
U+7973.0	fHakka	 shin5; zhin3; zihn3; siin5
U+7973.0	fCantonese	san6; san4
U+7973.0	fMandarin	chen3
U+7973.0	fEnglish	flesh offered in sacrifice by the Emperor
U+7974.0	fUTF8	祴
U+7974.0	fCNS11643	2-3E56
U+7974.0	fBig5	DAD2
U+7974.0	fGB	3-572B
U+7974.0	fR/S	 113.7
U+7974.0	fTotalStrokes	 12
U+7974.0	fCangjie	IFIT
U+7974.0	fFourCorner	33250
U+7974.0	fHakka	 goi1;  gai1; kie1
U+7974.0	fCantonese	goi1
U+7974.0	fMandarin	gai1; jie2; jia1
U+7974.0	fEnglish	the name of an ancient tune warning guests against drinking too much wine
U+7975.0	fUTF8	祵
U+7975.0	fCNS11643	3-3D64
U+7975.0	fGB	E-3566
U+7975.0	fR/S	 113.7
U+7975.0	fTotalStrokes	 12
U+7975.0	fCangjie	XIFWD
U+7975.0	fFourCorner	36200
U+7975.0	fCantonese	kwan2
U+7975.0	fMandarin	kun3; hun2
U+7976.0	fUTF8	祶
U+7976.0	fCNS11643	3-3D62
U+7976.0	fGB	E-3567
U+7976.0	fR/S	 113.7
U+7976.0	fTotalStrokes	 12
U+7976.0	fCangjie	IFCNH
U+7976.0	fFourCorner	38220
U+7976.0	fCantonese	dai3
U+7976.0	fMandarin	di4
U+7977.0	fUTF8	祷
U+7977.0	fCNS11643	4-395A
U+7977.0	fGB	0-353B
U+7977.0	fR/S	 113.7
U+7977.0	fTotalStrokes	 12
U+7977.0	fCangjie	IFQKI
U+7977.0	fFourCorner	34240
U+7977.0	fHakka	 dau3
U+7977.0	fCantonese	tou2; dou2 (also)
U+7977.0	fMandarin	dao3; dao4
U+7977.0	fEnglish	simplified form of &#x79B1;
U+7978.0	fUTF8	祸
U+7978.0	fGB	0-3B76
U+7978.0	fR/S	 113.7
U+7978.0	fTotalStrokes	 12
U+7978.0	fCangjie	IFROB
U+7978.0	fHakka	 fo5
U+7978.0	fCantonese	wo6
U+7978.0	fMandarin	huo4
U+7978.0	fEnglish	simplified form of &#x798D;
U+7979.0	fUTF8	祹
U+7979.0	fCNS11643	2-455C
U+7979.0	fBig5	DEF6
U+7979.0	fGB	3-5732
U+7979.0	fR/S	 113.8
U+7979.0	fTotalStrokes	 13
U+7979.0	fCangjie	IFPOU
U+7979.0	fFourCorner	37227
U+7979.0	fHakka	  tau2; rau2; jeu2
U+7979.0	fCantonese	tou4
U+7979.0	fMandarin	tao2
U+7979.0	fEnglish	[1] happiness [2] divine; spiritual
U+797A.0	fUTF8	祺
U+797A.0	fCNS11643	1-6559
U+797A.0	fBig5	B852
U+797A.0	fGB	0-6C77
U+797A.0	fR/S	 113.8
U+797A.0	fTotalStrokes	 13
U+797A.0	fCangjie	IFTMC
U+797A.0	fFourCorner	34281
U+797A.0	fHakka	 ki2
U+797A.0	fCantonese	kei4
U+797A.0	fMandarin	qi2
U+797A.0	fEnglish	[1] lucky; propitious; auspicious [2] peaceful; serene
U+797B.0	fUTF8	祻
U+797B.0	fCNS11643	4-3F46
U+797B.0	fGB	E-3568
U+797B.0	fR/S	  113.8
U+797B.0	fTotalStrokes	 13; 12
U+797B.0	fCangjie	IFWJR
U+797B.0	fFourCorner	36200
U+797B.0	fCantonese	gu3; gwu3 (also)
U+797B.0	fMandarin	gu4
U+797C.0	fUTF8	祼
U+797C.0	fCNS11643	2-4559
U+797C.0	fBig5	DEF3
U+797C.0	fGB	3-5731
U+797C.0	fR/S	 113.8
U+797C.0	fTotalStrokes	 13
U+797C.0	fCangjie	IFWD
U+797C.0	fFourCorner	36294
U+797C.0	fHakka	 kwan5; gon5; gwon5
U+797C.0	fCantonese	gun3; gwun3 (also)
U+797C.0	fMandarin	guan4
U+797C.0	fEnglish	 pour out a libation
U+797D.0	fUTF8	祽
U+797D.0	fCNS11643	2-455B
U+797D.0	fBig5	DEF5
U+797D.0	fGB	5-5760
U+797D.0	fR/S	 113.8
U+797D.0	fTotalStrokes	 13
U+797D.0	fCangjie	IFYOJ
U+797D.0	fFourCorner	30248
U+797D.0	fCantonese	zeoi3
U+797D.0	fMandarin	zui4; cui4
U+797E.0	fUTF8	祾
U+797E.0	fCNS11643	3-4338
U+797E.0	fGB	3-572E
U+797E.0	fR/S	 113.8
U+797E.0	fTotalStrokes	 13; 12
U+797E.0	fCangjie	IFGCE
U+797E.0	fFourCorner	34240
U+797E.0	fHakka	 lin2
U+797E.0	fCantonese	ling4
U+797E.0	fMandarin	ling2; leng2
U+797F.0	fUTF8	祿
U+797F.0	fCNS11643	1-655A
U+797F.0	fBig5	B853
U+797F.0	fGB	E-3569
U+797F.0	fR/S	 113.8
U+797F.0	fTotalStrokes	 13
U+797F.0	fCangjie	IFVNE
U+797F.0	fFourCorner	37232
U+797F.0	fHakka	  luk7
U+797F.0	fCantonese	luk6
U+797F.0	fMandarin	lu4
U+797F.0	fEnglish	[1] happiness; prosperity [2] official pay; salary
U+7980.0	fUTF8	禀
U+7980.0	fCNS11643	3-433D
U+7980.0	fGB	0-5977
U+7980.0	fR/S	  113.8
U+7980.0	fTotalStrokes	 13
U+7980.0	fCangjie	YWRF
U+7980.0	fFourCorner	00900
U+7980.0	fHakka	  bin3; ben3
U+7980.0	fCantonese	ban2
U+7980.0	fMandarin	bing3; bin3; lin3
U+7980.0	fEnglish	report to; petition
U+7981.0	fUTF8	禁
U+7981.0	fCNS11643	1-655B
U+7981.0	fBig5	B854
U+7981.0	fGB	0-3D7B
U+7981.0	fR/S	 113.8
U+7981.0	fTotalStrokes	 13
U+7981.0	fCangjie	DDMMF
U+7981.0	fFourCorner	44901
U+7981.0	fHakka	 gim1
U+7981.0	fCantonese	gam3; gam1 (also); kam1
U+7981.0	fMandarin	jin4
U+7981.0	fEnglish	[1] restrict; prohibit; forbid; ban [2] confine; imprison; detain [3] secret [4] royal residence
U+7981.1	fUTF8	禁
U+7981.1	fCNS11643	1-655B
U+7981.1	fBig5	B854
U+7981.1	fGB	0-3D7B
U+7981.1	fR/S	 113.8
U+7981.1	fTotalStrokes	 13
U+7981.1	fCangjie	DDMMF
U+7981.1	fFourCorner	44901
U+7981.1	fHakka	 gim5
U+7981.1	fMandarin	jin1
U+7981.1	fEnglish	 endure; bear; withstand; stand
U+7982.0	fUTF8	禂
U+7982.0	fCNS11643	2-455A
U+7982.0	fBig5	DEF4
U+7982.0	fGB	5-575E
U+7982.0	fR/S	 113.8
U+7982.0	fTotalStrokes	 13
U+7982.0	fCangjie	IFBGR
U+7982.0	fFourCorner	37220
U+7982.0	fHakka	  dau3
U+7982.0	fCantonese	dou2; tou2
U+7982.0	fMandarin	dao3; dao4; zhu1
U+7982.0	fEnglish	 pray prayer
U+7983.0	fUTF8	禃
U+7983.0	fCNS11643	3-4339
U+7983.0	fGB	3-572F
U+7983.0	fR/S	  113.8
U+7983.0	fTotalStrokes	 13; 12
U+7983.0	fCangjie	IFJBM
U+7983.0	fFourCorner	34210
U+7983.0	fCantonese	zik6
U+7983.0	fMandarin	zhi2
U+7984.0	fUTF8	禄
U+7984.0	fCNS11643	3-3D63
U+7984.0	fGB	0-423B
U+7984.0	fR/S	 113.7
U+7984.0	fTotalStrokes	 12
U+7984.0	fCangjie	IFNME
U+7984.0	fFourCorner	37230
U+7984.0	fCantonese	luk6
U+7984.0	fMandarin	lu4
U+7984.0	fEnglish	blessing; happiness; prosperity
U+7985.0	fUTF8	禅
U+7985.0	fGB	0-6C78
U+7985.0	fR/S	 113.8; 113.9
U+7985.0	fTotalStrokes	 13
U+7985.0	fCangjie	IFCWJ
U+7985.0	fHakka	 shen5; sham2
U+7985.0	fCantonese	sin4 (also); sim4; sin6; sim3
U+7985.0	fMandarin	chan2; shan4
U+7985.0	fEnglish	simplified form of &#x79AA;
U+7986.0	fUTF8	禆
U+7986.0	fCNS11643	3-6578
U+7986.0	fGB	E-356A
U+7986.0	fR/S	 113.8
U+7986.0	fTotalStrokes	 13
U+7986.0	fCangjie	IFHWJ
U+7986.0	fFourCorner	3624
U+7986.0	fCantonese	pei4
U+7986.0	fMandarin	bei1
U+7987.0	fUTF8	禇
U+7987.0	fCNS11643	3-4848
U+7987.0	fGB	E-356B
U+7987.0	fR/S	  113.9
U+7987.0	fTotalStrokes	 14
U+7987.0	fCangjie	IFJKA
U+7987.0	fFourCorner	34260
U+7987.0	fCantonese	cyu5
U+7987.0	fMandarin	zhe3; zhu3; chu3
U+7988.0	fUTF8	禈
U+7988.0	fCNS11643	2-4C48
U+7988.0	fBig5	E341
U+7988.0	fGB	5-5759
U+7988.0	fR/S	 113.9
U+7988.0	fTotalStrokes	 14
U+7988.0	fCangjie	IFBJJ
U+7988.0	fFourCorner	37250
U+7988.0	fCantonese	fai1
U+7988.0	fMandarin	hui1
U+7989.0	fUTF8	禉
U+7989.0	fCNS11643	3-4847
U+7989.0	fGB	E-356C
U+7989.0	fR/S	  113.9
U+7989.0	fTotalStrokes	 14
U+7989.0	fCangjie	IFTCW
U+7989.0	fFourCorner	38260
U+7989.0	fCantonese	jau5
U+7989.0	fMandarin	you3; chao3
U+798A.0	fUTF8	禊
U+798A.0	fCNS11643	2-4C41
U+798A.0	fBig5	E2F9
U+798A.0	fGB	0-6C79
U+798A.0	fR/S	 113.9
U+798A.0	fTotalStrokes	 14
U+798A.0	fCangjie	IFQHK
U+798A.0	fFourCorner	37234
U+798A.0	fHakka	 he5; k'ie5; ke5
U+798A.0	fCantonese	hai6
U+798A.0	fMandarin	xi4
U+798A.0	fEnglish	semi-annual ceremony of purification or exorcism performed at the water's edge in ancient times
U+798B.0	fUTF8	禋
U+798B.0	fCNS11643	2-4C42
U+798B.0	fBig5	E2FA
U+798B.0	fGB	3-5734
U+798B.0	fR/S	 113.9
U+798B.0	fTotalStrokes	 14
U+798B.0	fCangjie	IFMWG
U+798B.0	fFourCorner	31214
U+798B.0	fHakka	 jen1; jin1; ren1; rin1
U+798B.0	fCantonese	jan1
U+798B.0	fMandarin	yin1; yan1
U+798B.0	fEnglish	[1] offer sacrifices to Heaven [2] worship with sincerity and reverence
U+798C.0	fUTF8	禌
U+798C.0	fCNS11643	5-477E
U+798C.0	fGB	E-356D
U+798C.0	fR/S	 113.9
U+798C.0	fTotalStrokes	 14; 13
U+798C.0	fCangjie	IFTVI
U+798C.0	fFourCorner	38230
U+798C.0	fCantonese	zi1
U+798C.0	fMandarin	zi1
U+798D.0	fUTF8	禍
U+798D.0	fCNS11643	1-697E
U+798D.0	fBig5	BAD7
U+798D.0	fGB	1-3B76
U+798D.0	fR/S	 113.9
U+798D.0	fTotalStrokes	 14
U+798D.0	fCangjie	IFBBR
U+798D.0	fFourCorner	37227
U+798D.0	fHakka	  fo5; fo3; wo3
U+798D.0	fCantonese	wo6; wo5
U+798D.0	fMandarin	huo4
U+798D.0	fEnglish	[1] misfortune; calamity; disaster; evil [2] bring disaster upon; harm; injure; do evil to [3] punish
U+798E.0	fUTF8	禎
U+798E.0	fCNS11643	1-697C
U+798E.0	fBig5	BAD5
U+798E.0	fGB	1-6C75
U+798E.0	fR/S	 113.9
U+798E.0	fTotalStrokes	 14
U+798E.0	fCangjie	IFYBC
U+798E.0	fFourCorner	31286
U+798E.0	fHakka	 zhin1; zihn1; zin1
U+798E.0	fCantonese	zing1
U+798E.0	fMandarin	zhen1; zheng1
U+798E.0	fEnglish	[1] lucky; auspicious; good omen [2] a Chinese family name
U+798F.0	fUTF8	福
U+798F.0	fCNS11643	1-697D
U+798F.0	fBig5	BAD6
U+798F.0	fGB	0-3823
U+798F.0	fR/S	 113.9
U+798F.0	fTotalStrokes	 14
U+798F.0	fCangjie	IFMRW
U+798F.0	fFourCorner	31266
U+798F.0	fHakka	  fuk7
U+798F.0	fCantonese	fuk1; fu3
U+798F.0	fMandarin	fu2; fu4
U+798F.0	fEnglish	 happiness; good fortune; blessing; bliss; good luck
U+7990.0	fUTF8	禐
U+7990.0	fCNS11643	2-4C4A
U+7990.0	fBig5	E343
U+7990.0	fGB	5-5764
U+7990.0	fR/S	 113.9
U+7990.0	fTotalStrokes	 14
U+7990.0	fCangjie	IFBME
U+7990.0	fFourCorner	32247
U+7990.0	fCantonese	jyun6
U+7990.0	fMandarin	yuan4
U+7991.0	fUTF8	禑
U+7991.0	fCNS11643	5-4822
U+7991.0	fGB	3-5735
U+7991.0	fR/S	 113.9
U+7991.0	fTotalStrokes	 14; 13
U+7991.0	fCangjie	IFWLB
U+7991.0	fFourCorner	36220
U+7991.0	fCantonese	ng4
U+7991.0	fMandarin	wu2
U+7992.0	fUTF8	禒
U+7992.0	fCNS11643	2-4C49
U+7992.0	fBig5	E342
U+7992.0	fGB	E-356E
U+7992.0	fR/S	 113.9
U+7992.0	fTotalStrokes	 14
U+7992.0	fCangjie	IFVNO
U+7992.0	fFourCorner	37232
U+7992.0	fHakka	 luk7
U+7992.0	fCantonese	sin2
U+7992.0	fMandarin	xian3
U+7993.0	fUTF8	禓
U+7993.0	fCNS11643	2-4C46
U+7993.0	fBig5	E2FE
U+7993.0	fGB	3-567B
U+7993.0	fR/S	 113.9
U+7993.0	fTotalStrokes	 14
U+7993.0	fCangjie	IFAMH
U+7993.0	fFourCorner	36227
U+7993.0	fHakka	 jong2; rong2
U+7993.0	fCantonese	joeng4; soeng1 (also)
U+7993.0	fMandarin	yang2; shang1
U+7993.0	fEnglish	 drive out demons; etc., as was done at the New Year
U+7994.0	fUTF8	禔
U+7994.0	fCNS11643	2-4C45
U+7994.0	fBig5	E2FD
U+7994.0	fGB	5-5762
U+7994.0	fR/S	 113.9
U+7994.0	fTotalStrokes	 14
U+7994.0	fCangjie	IFAMO
U+7994.0	fFourCorner	36281
U+7994.0	fHakka	  ti2; zhi1; zih1
U+7994.0	fCantonese	tai4
U+7994.0	fMandarin	ti2; zhi1; shi2
U+7994.0	fEnglish	happiness; good fortune; good luck; blessings
U+7995.0	fUTF8	禕
U+7995.0	fCNS11643	2-4C44
U+7995.0	fBig5	E2FC
U+7995.0	fGB	3-567E
U+7995.0	fR/S	 113.9; 113.10
U+7995.0	fTotalStrokes	 14
U+7995.0	fCangjie	IFDMQ
U+7995.0	fFourCorner	34256
U+7995.0	fHakka	 ji1; ri1
U+7995.0	fCantonese	ji1
U+7995.0	fMandarin	yi1
U+7995.0	fEnglish	excellent; fine
U+7996.0	fUTF8	禖
U+7996.0	fCNS11643	2-4C43
U+7996.0	fBig5	E2FB
U+7996.0	fGB	3-5733
U+7996.0	fR/S	 113.9
U+7996.0	fTotalStrokes	 14
U+7996.0	fCangjie	IFTMD
U+7996.0	fFourCorner	34294
U+7996.0	fHakka	 moi2; moe2
U+7996.0	fCantonese	mui4
U+7996.0	fMandarin	mei2
U+7996.0	fEnglish	sacrifice
U+7997.0	fUTF8	禗
U+7997.0	fCNS11643	2-4C47
U+7997.0	fBig5	E340
U+7997.0	fGB	3-5736
U+7997.0	fR/S	 113.9
U+7997.0	fTotalStrokes	 14
U+7997.0	fCangjie	IFWP
U+7997.0	fFourCorner	36230
U+7997.0	fCantonese	si1
U+7997.0	fMandarin	si1; xi3
U+7998.0	fUTF8	禘
U+7998.0	fCNS11643	2-4C40
U+7998.0	fBig5	E2F8
U+7998.0	fGB	3-5737
U+7998.0	fR/S	 113.9
U+7998.0	fTotalStrokes	 14
U+7998.0	fCangjie	IFYBB
U+7998.0	fFourCorner	30227
U+7998.0	fHakka	  di5;  ti5
U+7998.0	fCantonese	dai3
U+7998.0	fMandarin	di4
U+7998.0	fEnglish	imperial ancestor worship; imperial sacrifice made once every five years
U+7999.0	fUTF8	禙
U+7999.0	fCNS11643	3-6579
U+7999.0	fGB	E-356F
U+7999.0	fR/S	 113.9
U+7999.0	fTotalStrokes	 14
U+7999.0	fCangjie	IFLPB
U+7999.0	fFourCorner	3122
U+7999.0	fCantonese	bui3
U+799A.0	fUTF8	禚
U+799A.0	fCNS11643	2-5253
U+799A.0	fBig5	E6CB
U+799A.0	fGB	0-6C7A
U+799A.0	fR/S	 113.10
U+799A.0	fTotalStrokes	 15
U+799A.0	fCangjie	IFTGF
U+799A.0	fFourCorner	38231
U+799A.0	fHakka	 shok7; chok7; sok7; zok7
U+799A.0	fCantonese	zoek3; zuk6
U+799A.0	fMandarin	zhuo2
U+799A.0	fEnglish	the ancient name of Tsi nan-fu, in Shantung
U+799B.0	fUTF8	禛
U+799B.0	fCNS11643	2-5258
U+799B.0	fBig5	E6D0
U+799B.0	fGB	3-5738
U+799B.0	fR/S	 113.10
U+799B.0	fTotalStrokes	 15
U+799B.0	fCangjie	IFJBC
U+799B.0	fFourCorner	34281
U+799B.0	fHakka	 zhin1; zihn1
U+799B.0	fCantonese	zan1
U+799B.0	fMandarin	zhen1
U+799B.0	fEnglish	 be blessed because of one's sincerity
U+799C.0	fUTF8	禜
U+799C.0	fCNS11643	2-5256
U+799C.0	fBig5	E6CE
U+799C.0	fGB	3-5677
U+799C.0	fR/S	 113.10
U+799C.0	fTotalStrokes	 15
U+799C.0	fCangjie	FFBMF
U+799C.0	fFourCorner	99901
U+799C.0	fHakka	 jin5; rin5; jin2; rin2; jun5
U+799C.0	fCantonese	wing6; wing4 (also)
U+799C.0	fMandarin	yong3; ying2
U+799C.0	fEnglish	sacrifice
U+799D.0	fUTF8	禝
U+799D.0	fCNS11643	3-4D50
U+799D.0	fGB	5-5768
U+799D.0	fR/S	  113.10
U+799D.0	fTotalStrokes	 15
U+799D.0	fCangjie	IFWCE
U+799D.0	fFourCorner	36240
U+799D.0	fCantonese	zik1
U+799D.0	fMandarin	ji2
U+799E.0	fUTF8	禞
U+799E.0	fCNS11643	3-4D51
U+799E.0	fGB	E-3570
U+799E.0	fR/S	 113.10
U+799E.0	fTotalStrokes	 15
U+799E.0	fCangjie	IFYRB
U+799E.0	fFourCorner	30220
U+799E.0	fCantonese	haau2
U+799E.0	fMandarin	gao4
U+799F.0	fUTF8	禟
U+799F.0	fCNS11643	3-4D4F
U+799F.0	fGB	3-573B
U+799F.0	fR/S	 113.10
U+799F.0	fTotalStrokes	 15
U+799F.0	fCangjie	IFILR
U+799F.0	fFourCorner	30260
U+799F.0	fCantonese	tong4
U+799F.0	fMandarin	tang2
U+79A0.0	fUTF8	禠
U+79A0.0	fCNS11643	2-5255
U+79A0.0	fBig5	E6CD
U+79A0.0	fGB	3-5739
U+79A0.0	fR/S	 113.10
U+79A0.0	fTotalStrokes	 15
U+79A0.0	fCangjie	IFHYU
U+79A0.0	fFourCorner	32217
U+79A0.0	fCantonese	si1
U+79A0.0	fMandarin	si1; shi2; qi2; yi3
U+79A0.0	fEnglish	happy
U+79A1.0	fUTF8	禡
U+79A1.0	fCNS11643	2-5254
U+79A1.0	fBig5	E6CC
U+79A1.0	fGB	3-567C
U+79A1.0	fR/S	 113.10
U+79A1.0	fTotalStrokes	 15
U+79A1.0	fCangjie	IFSQF
U+79A1.0	fFourCorner	31227
U+79A1.0	fHakka	  ma5; ma6
U+79A1.0	fCantonese	maa6; ngaa2; maa3
U+79A1.0	fMandarin	ma4
U+79A1.0	fEnglish	 sacrifice to the god of war offered by troops on the eve of a battle
U+79A2.0	fUTF8	禢
U+79A2.0	fCNS11643	2-5257
U+79A2.0	fBig5	E6CF
U+79A2.0	fGB	5-5767
U+79A2.0	fR/S	 113.10
U+79A2.0	fTotalStrokes	 15
U+79A2.0	fCangjie	IFASM
U+79A2.0	fFourCorner	36227
U+79A2.0	fCantonese	taap3; daap6; hyun1
U+79A2.0	fMandarin	ta1
U+79A2.0	fEnglish	a Chinese family name
U+79A3.0	fUTF8	禣
U+79A3.0	fCNS11643	3-657A
U+79A3.0	fGB	E-3571
U+79A3.0	fR/S	 113.10
U+79A3.0	fTotalStrokes	 15
U+79A3.0	fCangjie	IFIBI
U+79A3.0	fFourCorner	3324
U+79A3.0	fCantonese	bok3
U+79A4.0	fUTF8	禤
U+79A4.0	fCNS11643	2-5878
U+79A4.0	fBig5	EAAE
U+79A4.0	fGB	3-573E
U+79A4.0	fR/S	 113.11
U+79A4.0	fTotalStrokes	 16
U+79A4.0	fCangjie	IFWLM
U+79A4.0	fFourCorner	36227
U+79A4.0	fHakka	 sien1
U+79A4.0	fCantonese	hyun1; hyun3
U+79A4.0	fMandarin	xuan1
U+79A5.0	fUTF8	禥
U+79A5.0	fCNS11643	3-5226
U+79A5.0	fGB	5-576A
U+79A5.0	fR/S	 113.11
U+79A5.0	fTotalStrokes	 16
U+79A5.0	fCangjie	IFTCG
U+79A5.0	fFourCorner	34210
U+79A5.0	fCantonese	kei4
U+79A5.0	fMandarin	qi2
U+79A6.0	fUTF8	禦
U+79A6.0	fCNS11643	1-7157
U+79A6.0	fBig5	BF6D
U+79A6.0	fGB	1-787A
U+79A6.0	fR/S	 113.11
U+79A6.0	fTotalStrokes	 16
U+79A6.0	fCangjie	HLMMF
U+79A6.0	fFourCorner	27901
U+79A6.0	fHakka	 ngi5; ngi3; ngi6; gni6; gne3
U+79A6.0	fCantonese	jyu6
U+79A6.0	fMandarin	yu4; yu3
U+79A6.0	fEnglish	 guard against; defend; take precautions against
U+79A7.0	fUTF8	禧
U+79A7.0	fCNS11643	1-7452
U+79A7.0	fBig5	C148
U+79A7.0	fGB	0-6C7B
U+79A7.0	fR/S	 113.12
U+79A7.0	fTotalStrokes	 17
U+79A7.0	fCangjie	IFGRR
U+79A7.0	fFourCorner	34265
U+79A7.0	fHakka	 hi1;  hi3
U+79A7.0	fCantonese	hei1; hei2 (also)
U+79A7.0	fMandarin	xi1; xi3
U+79A7.0	fEnglish	happiness; blessings; suspiciousness
U+79A8.0	fUTF8	禨
U+79A8.0	fCNS11643	2-5D7B
U+79A8.0	fBig5	EDB0
U+79A8.0	fGB	3-5679
U+79A8.0	fR/S	 113.12
U+79A8.0	fTotalStrokes	 17
U+79A8.0	fCangjie	IFVII
U+79A8.0	fFourCorner	32253
U+79A8.0	fHakka	 gi1
U+79A8.0	fCantonese	gei1; gei6
U+79A8.0	fMandarin	ji1; ji4
U+79A8.0	fEnglish	seek blessings from ghosts
U+79A9.0	fUTF8	禩
U+79A9.0	fCNS11643	3-5227
U+79A9.0	fGB	E-3572
U+79A9.0	fR/S	 113.12; 113.11
U+79A9.0	fTotalStrokes	 15; 16; 17
U+79A9.0	fCangjie	IFWTC
U+79A9.0	fFourCorner	36280
U+79A9.0	fHakka	 sii5; su3; sii6; cu5; su5; se5
U+79A9.0	fCantonese	zi6
U+79A9.0	fMandarin	si4
U+79A9.0	fEnglish	same as &#x7940;
U+79AA.0	fUTF8	禪
U+79AA.0	fCNS11643	1-7453
U+79AA.0	fBig5	C149
U+79AA.0	fGB	1-6C78
U+79AA.0	fR/S	 113.12
U+79AA.0	fTotalStrokes	 17
U+79AA.0	fCangjie	IFRRJ
U+79AA.0	fFourCorner	36256
U+79AA.0	fHakka	  sam2; san5; sham3; sam3; sham2; sen3; shen5; shan5; dan1
U+79AA.0	fCantonese	sim4; sin4 (also); sin6; sim3
U+79AA.0	fMandarin	chan2; tan2
U+79AA.0	fEnglish	[1] meditation; intense contemplation [2] Zen Buddhism
U+79AA.1	fUTF8	禪
U+79AA.1	fCNS11643	1-7453
U+79AA.1	fBig5	C149
U+79AA.1	fGB	1-6C78
U+79AA.1	fR/S	 113.12
U+79AA.1	fTotalStrokes	 17
U+79AA.1	fCangjie	IFRRJ
U+79AA.1	fFourCorner	36256
U+79AA.1	fMandarin	shan4
U+79AA.1	fEnglish	 abdicate (the throne)
U+79AB.0	fUTF8	禫
U+79AB.0	fCNS11643	2-5D7A
U+79AB.0	fBig5	EDAF
U+79AB.0	fGB	3-573F
U+79AB.0	fR/S	 113.12
U+79AB.0	fTotalStrokes	 17
U+79AB.0	fCangjie	IFMWJ
U+79AB.0	fFourCorner	31246
U+79AB.0	fHakka	 tam3; tam2
U+79AB.0	fCantonese	taam5; daam6
U+79AB.0	fMandarin	tan3; dan4
U+79AB.0	fEnglish	sacrifice offered on anniversary
U+79AC.0	fUTF8	禬
U+79AC.0	fCNS11643	2-624B
U+79AC.0	fBig5	F05F
U+79AC.0	fGB	3-5729
U+79AC.0	fR/S	 113.13
U+79AC.0	fTotalStrokes	 18
U+79AC.0	fCangjie	IFOMA
U+79AC.0	fFourCorner	38266
U+79AC.0	fHakka	  kwai5
U+79AC.0	fCantonese	kui2; wui6 (also); kwui2 (also); kui3; fui2
U+79AC.0	fMandarin	kuai4; hui4; gui4
U+79AC.0	fEnglish	 pray; invoke the gods
U+79AD.0	fUTF8	禭
U+79AD.0	fCNS11643	2-624A
U+79AD.0	fBig5	F05E
U+79AD.0	fGB	3-5740
U+79AD.0	fR/S	 113.13
U+79AD.0	fTotalStrokes	 18
U+79AD.0	fCangjie	IFYTO
U+79AD.0	fFourCorner	38233
U+79AD.0	fCantonese	seoi6
U+79AD.0	fMandarin	sui4
U+79AE.0	fUTF8	禮
U+79AE.0	fCNS11643	1-7671
U+79AE.0	fBig5	C2A7
U+79AE.0	fGB	1-4071
U+79AE.0	fR/S	 113.13
U+79AE.0	fTotalStrokes	 18
U+79AE.0	fCangjie	IFTWT
U+79AE.0	fFourCorner	35218
U+79AE.0	fHakka	  li1; li3
U+79AE.0	fCantonese	lai5
U+79AE.0	fMandarin	li3
U+79AE.0	fEnglish	[1] social custom; rites; ceremony [2] manners; courtesy; propriety; decorum; politeness; civility; etiquette [3] gift; present [4] a Chinese family name
U+79AF.0	fUTF8	禯
U+79AF.0	fCNS11643	3-5845
U+79AF.0	fGB	E-3573
U+79AF.0	fR/S	  113.13
U+79AF.0	fTotalStrokes	 18; 17
U+79AF.0	fCangjie	IFTWV
U+79AF.0	fFourCorner	35230
U+79AF.0	fCantonese	nung4; lung4 (also)
U+79AF.0	fMandarin	nong2
U+79B0.0	fUTF8	禰
U+79B0.0	fCNS11643	2-662F
U+79B0.0	fBig5	F2A5
U+79B0.0	fGB	1-6C72
U+79B0.0	fR/S	 113.14
U+79B0.0	fTotalStrokes	 19
U+79B0.0	fCangjie	IFMFB
U+79B0.0	fFourCorner	31227
U+79B0.0	fHakka	 ni3
U+79B0.0	fCantonese	nei5; nei4; lei5 (also); lei4 (also); nai5
U+79B0.0	fMandarin	ni3; xian3
U+79B0.0	fEnglish	a shrine where one's father's sacred tablet is kept
U+79B0.1	fUTF8	禰
U+79B0.1	fCNS11643	2-662F
U+79B0.1	fBig5	F2A5
U+79B0.1	fGB	1-6C72
U+79B0.1	fR/S	 113.14
U+79B0.1	fTotalStrokes	 19
U+79B0.1	fCangjie	IFMFB
U+79B0.1	fFourCorner	31227
U+79B0.1	fMandarin	mi2
U+79B0.1	fEnglish	a Chinese family name
U+79B1.0	fUTF8	禱
U+79B1.0	fCNS11643	1-7856
U+79B1.0	fBig5	C3AB
U+79B1.0	fGB	1-353B
U+79B1.0	fR/S	 113.14
U+79B1.0	fTotalStrokes	 19
U+79B1.0	fCangjie	IFGNI
U+79B1.0	fFourCorner	34241
U+79B1.0	fHakka	  dau3;  do3; tau2
U+79B1.0	fCantonese	tou2; dou2 (also)
U+79B1.0	fMandarin	dao3; dao4
U+79B1.0	fEnglish	 pray; beseech; entreat; beg; plead
U+79B2.0	fUTF8	禲
U+79B2.0	fCNS11643	2-694E
U+79B2.0	fBig5	F4A1
U+79B2.0	fGB	5-5750
U+79B2.0	fR/S	 113.15
U+79B2.0	fTotalStrokes	 20
U+79B2.0	fCangjie	IFMTB
U+79B2.0	fFourCorner	31227
U+79B2.0	fCantonese	lai6
U+79B2.0	fMandarin	li4
U+79B3.0	fUTF8	禳
U+79B3.0	fCNS11643	1-7B6B
U+79B3.0	fBig5	C5A1
U+79B3.0	fGB	0-6C7C
U+79B3.0	fR/S	 113.17
U+79B3.0	fTotalStrokes	 22
U+79B3.0	fCangjie	IFYRV
U+79B3.0	fFourCorner	30232
U+79B3.0	fHakka	 jong2; rong2; ngiong2; jong5; ngiong5; rong5; jong1
U+79B3.0	fCantonese	joeng4
U+79B3.0	fMandarin	rang2
U+79B3.0	fEnglish	[1] a form of sacrifice performed for exorcism [2] exorcise
U+79B4.0	fUTF8	禴
U+79B4.0	fCNS11643	2-6D65
U+79B4.0	fBig5	F6F7
U+79B4.0	fGB	5-576C
U+79B4.0	fR/S	 113.17
U+79B4.0	fTotalStrokes	 22
U+79B4.0	fCangjie	IFOMB
U+79B4.0	fFourCorner	38227
U+79B4.0	fHakka	 jok7; rok7
U+79B4.0	fCantonese	joek6
U+79B4.0	fMandarin	yue4
U+79B4.0	fEnglish	a kind of annual sacrifice
U+79B5.0	fUTF8	禵
U+79B5.0	fCNS11643	3-603D
U+79B5.0	fGB	3-5742
U+79B5.0	fR/S	  113.18
U+79B5.0	fTotalStrokes	 23; 22
U+79B5.0	fCangjie	IFAOC
U+79B5.0	fFourCorner	31280
U+79B5.0	fCantonese	tai4
U+79B5.0	fMandarin	ti2
U+79B6.0	fUTF8	禶
U+79B6.0	fCNS11643	2-7045
U+79B6.0	fBig5	F8B7
U+79B6.0	fGB	3-5743
U+79B6.0	fR/S	 113.19
U+79B6.0	fTotalStrokes	 24
U+79B6.0	fCangjie	IFHUC
U+79B6.0	fFourCorner	34286
U+79B6.0	fHakka	 zan5
U+79B6.0	fCantonese	zaan6; zaan3
U+79B6.0	fMandarin	zan3; zan4
U+79B6.0	fEnglish	 pray; supplicate
U+79B7.0	fUTF8	禷
U+79B7.0	fCNS11643	2-7044
U+79B7.0	fBig5	F8B6
U+79B7.0	fGB	E-3574
U+79B7.0	fR/S	 113.19
U+79B7.0	fTotalStrokes	 24
U+79B7.0	fCangjie	IFFKC
U+79B7.0	fFourCorner	31286
U+79B7.0	fCantonese	leoi6
U+79B7.0	fMandarin	lei4
U+79B8.0	fUTF8	禸
U+79B8.0	fCNS11643	2-2167
U+79B8.0	fBig5	C9A8
U+79B8.0	fGB	5-3132
U+79B8.0	fR/S	 114.0
U+79B8.0	fTotalStrokes	 5
U+79B8.0	fCangjie	IB
U+79B8.0	fFourCorner	40227
U+79B8.0	fHakka	 ngiu3; ju3
U+79B8.0	fCantonese	jau4; jau5; nau5; jau2; jan4
U+79B8.0	fMandarin	rou2
U+79B8.0	fEnglish	 rump Kangxi radical number 114
U+79B9.0	fUTF8	禹
U+79B9.0	fCNS11643	1-526D
U+79B9.0	fBig5	ACEA
U+79B9.0	fGB	0-536D
U+79B9.0	fR/S	 114.4
U+79B9.0	fTotalStrokes	 9
U+79B9.0	fCangjie	HLBI
U+79B9.0	fFourCorner	20427
U+79B9.0	fHakka	 ji3; ri1; ji1; ri3; i3
U+79B9.0	fCantonese	jyu5
U+79B9.0	fMandarin	yu3
U+79B9.0	fEnglish	[1] Yuu, the legendary founder of the Hsia dynasty (21st-16th B.C) [2] a Chinese family name
U+79BA.0	fUTF8	禺
U+79BA.0	fCNS11643	1-526E
U+79BA.0	fBig5	ACEB
U+79BA.0	fGB	0-582E
U+79BA.0	fR/S	 114.4
U+79BA.0	fTotalStrokes	 9
U+79BA.0	fCangjie	WLBI
U+79BA.0	fFourCorner	60427
U+79BA.0	fHakka	 ngi2; ngi5; gni3
U+79BA.0	fCantonese	jyu4; jyu6
U+79BA.0	fMandarin	yu2; yong2; ou3
U+79BA.0	fEnglish	a mountain in Chekiang
U+79BA.1	fUTF8	禺
U+79BA.1	fCNS11643	1-526E
U+79BA.1	fBig5	ACEB
U+79BA.1	fGB	0-582E
U+79BA.1	fR/S	 114.4
U+79BA.1	fTotalStrokes	 9
U+79BA.1	fCangjie	WLBI
U+79BA.1	fFourCorner	60427
U+79BA.1	fMandarin	yu4
U+79BA.1	fEnglish	 monkey
U+79BB.0	fUTF8	离
U+79BB.0	fCNS11643	2-3766
U+79BB.0	fBig5	D6C3
U+79BB.0	fGB	0-406B
U+79BB.0	fR/S	 114.6
U+79BB.0	fTotalStrokes	 11
U+79BB.0	fCangjie	YUKB
U+79BB.0	fFourCorner	00227
U+79BB.0	fHakka	  li2; li3
U+79BB.0	fCantonese	ci1; lei4; sit3
U+79BB.0	fMandarin	li2; chi1; li4; li3
U+79BB.0	fEnglish	simplified form of &#x96E2;
U+79BC.0	fUTF8	禼
U+79BC.0	fCNS11643	3-657B
U+79BC.0	fGB	E-3575
U+79BC.0	fR/S	 114.7
U+79BC.0	fTotalStrokes	 12
U+79BC.0	fCangjie	YWKB
U+79BC.0	fFourCorner	2142
U+79BC.0	fCantonese	sit3
U+79BC.0	fMandarin	xie4
U+79BD.0	fUTF8	禽
U+79BD.0	fCNS11643	1-655D
U+79BD.0	fBig5	B856
U+79BD.0	fGB	0-475D
U+79BD.0	fR/S	 114.8
U+79BD.0	fTotalStrokes	 13
U+79BD.0	fCangjie	OYUB
U+79BD.0	fFourCorner	80427
U+79BD.0	fHakka	  kim2; kim3
U+79BD.0	fCantonese	kam4
U+79BD.0	fMandarin	qin2
U+79BD.0	fEnglish	[1] birds; fowl [2] same as $ (capture)
U+79BE.0	fUTF8	禾
U+79BE.0	fCNS11643	1-467D
U+79BE.0	fBig5	A5DD
U+79BE.0	fGB	0-3A4C
U+79BE.0	fR/S	 115.0
U+79BE.0	fTotalStrokes	 5
U+79BE.0	fCangjie	HD
U+79BE.0	fFourCorner	20904
U+79BE.0	fHakka	 wo2; wo3; vo2
U+79BE.0	fCantonese	wo4; wo2 (also)
U+79BE.0	fMandarin	he2
U+79BE.0	fEnglish	[1] grain still on stalk [2] rice plant; rice crop
U+79BF.0	fUTF8	禿
U+79BF.0	fCNS11643	1-4B35
U+79BF.0	fBig5	A872
U+79BF.0	fGB	E-3576
U+79BF.0	fR/S	 115.2
U+79BF.0	fTotalStrokes	 7
U+79BF.0	fCangjie	HDHU
U+79BF.0	fFourCorner	20217
U+79BF.0	fHakka	 tut7; tuk7
U+79BF.0	fCantonese	tuk1
U+79BF.0	fMandarin	tu1
U+79BF.0	fEnglish	bald; bare
U+79C0.0	fUTF8	秀
U+79C0.0	fCNS11643	1-4B34
U+79C0.0	fBig5	A871
U+79C0.0	fGB	0-5063
U+79C0.0	fR/S	 115.2
U+79C0.0	fTotalStrokes	 7
U+79C0.0	fCangjie	HDNHS
U+79C0.0	fFourCorner	20227
U+79C0.0	fHakka	  siu5; sieu5; ciu5
U+79C0.0	fCantonese	sau3
U+79C0.0	fMandarin	xiu4
U+79C0.0	fEnglish	[1] put forth new flowers on ears [2] refined; elegant; graceful; beautiful; delicate [3] brilliant; excellent; competent; outstanding
U+79C1.0	fUTF8	私
U+79C1.0	fCNS11643	1-4B33
U+79C1.0	fBig5	A870
U+79C1.0	fGB	0-4B3D
U+79C1.0	fR/S	 115.2
U+79C1.0	fTotalStrokes	 7
U+79C1.0	fCangjie	HDI
U+79C1.0	fFourCorner	22930
U+79C1.0	fHakka	  sii1;  su1; se1
U+79C1.0	fCantonese	si1
U+79C1.0	fMandarin	si1
U+79C1.0	fEnglish	[1] private; personal; person-to-person [2] secret; clandestine [3] have illicit relations or an affair with [4] contraband [5] prejudice; biased; favor [6] selfish; selfishly [7] reproductive organs of both sexes
U+79C2.0	fUTF8	秂
U+79C2.0	fCNS11643	4-2468
U+79C2.0	fGB	E-3577
U+79C2.0	fR/S	  115.2
U+79C2.0	fTotalStrokes	 7
U+79C2.0	fCangjie	HDO
U+79C2.0	fFourCorner	20800
U+79C2.0	fCantonese	jan4
U+79C2.0	fMandarin	ren2
U+79C3.0	fUTF8	秃
U+79C3.0	fCNS11643	3-2A3E
U+79C3.0	fGB	0-4D3A
U+79C3.0	fR/S	 115.2
U+79C3.0	fTotalStrokes	 7; 8
U+79C3.0	fCangjie	HDHN
U+79C3.0	fFourCorner	20210
U+79C3.0	fHakka	 tut7
U+79C3.0	fCantonese	tuk1
U+79C3.0	fMandarin	tu1
U+79C3.0	fEnglish	bald
U+79C4.0	fUTF8	秄
U+79C4.0	fCNS11643	3-2A3D
U+79C4.0	fGB	5-5A7C
U+79C4.0	fR/S	 115.3
U+79C4.0	fTotalStrokes	 8
U+79C4.0	fCangjie	HDND
U+79C4.0	fFourCorner	27940
U+79C4.0	fHakka	 zih3; zii3
U+79C4.0	fCantonese	zi2
U+79C4.0	fMandarin	zi3
U+79C4.0	fEnglish	the hoe earth round plants
U+79C5.0	fUTF8	秅
U+79C5.0	fCNS11643	2-2844
U+79C5.0	fBig5	CDA4
U+79C5.0	fGB	3-5B51
U+79C5.0	fR/S	 115.3
U+79C5.0	fTotalStrokes	 8
U+79C5.0	fCangjie	HDHP
U+79C5.0	fFourCorner	22914
U+79C5.0	fHakka	  ca2; du5
U+79C5.0	fCantonese	caa4; dou3; caa3
U+79C5.0	fMandarin	cha2; du4; na2
U+79C5.0	fEnglish	a method of reckoning grain when reaped
U+79C6.0	fUTF8	秆
U+79C6.0	fCNS11643	3-2A3F
U+79C6.0	fGB	0-3851
U+79C6.0	fR/S	  115.3
U+79C6.0	fTotalStrokes	 8
U+79C6.0	fCangjie	HDMJ
U+79C6.0	fFourCorner	21940
U+79C6.0	fHakka	  gon3
U+79C6.0	fCantonese	gon2
U+79C6.0	fMandarin	gan3
U+79C6.0	fEnglish	 straw stalk of grain
U+79C7.0	fUTF8	秇
U+79C7.0	fCNS11643	3-2A42
U+79C7.0	fGB	E-3578
U+79C7.0	fR/S	 115.3
U+79C7.0	fTotalStrokes	 8
U+79C7.0	fCangjie	HDKNI
U+79C7.0	fFourCorner	25917
U+79C7.0	fHakka	 ngi5; ngi6
U+79C7.0	fCantonese	ngai6; ai6 (also)
U+79C7.0	fMandarin	yi4; zhi2
U+79C7.0	fEnglish	 skill; ability in handicraft; craft; an art; a calling a trade accomplishment
U+79C8.0	fUTF8	秈
U+79C8.0	fCNS11643	1-4F27
U+79C8.0	fBig5	AAC4
U+79C8.0	fGB	E-3579
U+79C8.0	fR/S	 115.3
U+79C8.0	fTotalStrokes	 8
U+79C8.0	fCangjie	HDU
U+79C8.0	fFourCorner	22970
U+79C8.0	fHakka	  sien1
U+79C8.0	fCantonese	sin1
U+79C8.0	fMandarin	xian1
U+79C8.0	fEnglish	non-glutinous long grain rice
U+79C9.0	fUTF8	秉
U+79C9.0	fCNS11643	1-4F26
U+79C9.0	fBig5	AAC3
U+79C9.0	fGB	0-317C
U+79C9.0	fR/S	 115.3
U+79C9.0	fTotalStrokes	 8
U+79C9.0	fCangjie	HDL
U+79C9.0	fFourCorner	20907
U+79C9.0	fHakka	  bin3; biang3; biang1
U+79C9.0	fCantonese	bing2
U+79C9.0	fMandarin	bing3; bing4
U+79C9.0	fEnglish	[1] grasp; hold in hand [2] authority [3] take charge of; rule [4] an ancient grain measure; a measure for liquid
U+79CA.0	fUTF8	秊
U+79CA.0	fCNS11643	3-2A44
U+79CA.0	fGB	E-357A
U+79CA.0	fR/S	  115.3
U+79CA.0	fTotalStrokes	 8
U+79CA.0	fCangjie	HDHJ
U+79CA.0	fFourCorner	20400
U+79CA.0	fCantonese	nin4; lin4 (also)
U+79CA.0	fMandarin	nian2
U+79CA.0	fEnglish	 year new-years person's age
U+79CB.0	fUTF8	秋
U+79CB.0	fCNS11643	1-5271
U+79CB.0	fBig5	ACEE
U+79CB.0	fGB	0-476F
U+79CB.0	fR/S	 115.4
U+79CB.0	fTotalStrokes	 9
U+79CB.0	fCangjie	HDF
U+79CB.0	fFourCorner	29980
U+79CB.0	fHakka	  ciu1; ts'ieu1
U+79CB.0	fCantonese	cau1
U+79CB.0	fMandarin	qiu1
U+79CB.0	fEnglish	[1] autumn; fall [2] year [3] time; period [4] season [5] ripening of grains; harvest [6] a Chinese family name
U+79CC.0	fUTF8	秌
U+79CC.0	fCNS11643	3-654C
U+79CC.0	fGB	E-357B
U+79CC.0	fR/S	 115.4; 86.5
U+79CC.0	fTotalStrokes	 9
U+79CC.0	fCangjie	FHD
U+79CC.0	fFourCorner	9289
U+79CC.0	fHakka	 ciu1
U+79CC.0	fCantonese	cau1
U+79CC.0	fMandarin	qiu1
U+79CC.0	fEnglish	same as &#x79CB;
U+79CD.0	fUTF8	种
U+79CD.0	fCNS11643	2-2C5C
U+79CD.0	fBig5	CFFA
U+79CD.0	fGB	0-5656
U+79CD.0	fR/S	 115.4
U+79CD.0	fTotalStrokes	 9
U+79CD.0	fCangjie	HDL
U+79CD.0	fFourCorner	25906
U+79CD.0	fHakka	 chung1; zhung5; zhung3; cung1; zung5; zung3
U+79CD.0	fCantonese	cung4; zung2; zung3
U+79CD.0	fMandarin	zhong3; chong2
U+79CD.0	fEnglish	[1] naive; naivete [2] a Chinese family name [3] simplified form of &#x7A2E;
U+79CD.1	fUTF8	种
U+79CD.1	fCNS11643	2-2C5C
U+79CD.1	fBig5	CFFA
U+79CD.1	fGB	0-5656
U+79CD.1	fR/S	 115.4
U+79CD.1	fTotalStrokes	 9
U+79CD.1	fCangjie	HDL
U+79CD.1	fFourCorner	25906
U+79CD.1	fMandarin	zhong4
U+79CE.0	fUTF8	秎
U+79CE.0	fCNS11643	2-2C5F
U+79CE.0	fBig5	CFFD
U+79CE.0	fGB	3-5B5B
U+79CE.0	fR/S	 115.4
U+79CE.0	fTotalStrokes	 9
U+79CE.0	fCangjie	HDCSH
U+79CE.0	fFourCorner	28927
U+79CE.0	fHakka	 fun2
U+79CE.0	fCantonese	fan6
U+79CE.0	fMandarin	fen2; fen4
U+79CF.0	fUTF8	秏
U+79CF.0	fCNS11643	2-2C5D
U+79CF.0	fBig5	CFFB
U+79CF.0	fGB	5-5A7D
U+79CF.0	fR/S	 115.4
U+79CF.0	fTotalStrokes	 9
U+79CF.0	fCangjie	HDHQU
U+79CF.0	fFourCorner	22914
U+79CF.0	fHakka	 hau5; fui5; hau1; fi5; mau3
U+79CF.0	fCantonese	hou3
U+79CF.0	fMandarin	hao4
U+79CF.0	fEnglish	 waste; destroy; diminish rat (hence it came to mean from )
U+79D0.0	fUTF8	秐
U+79D0.0	fCNS11643	3-2E42
U+79D0.0	fGB	E-357C
U+79D0.0	fR/S	  115.4
U+79D0.0	fTotalStrokes	 9
U+79D0.0	fCangjie	HDMMI
U+79D0.0	fFourCorner	21930
U+79D0.0	fHakka	 jun2; run2
U+79D0.0	fCantonese	wan4
U+79D0.0	fMandarin	yun2
U+79D0.0	fEnglish	 weed
U+79D1.0	fUTF8	科
U+79D1.0	fCNS11643	1-526F
U+79D1.0	fBig5	ACEC
U+79D1.0	fGB	0-3F46
U+79D1.0	fR/S	 115.4
U+79D1.0	fTotalStrokes	 9
U+79D1.0	fCangjie	HDYJ
U+79D1.0	fFourCorner	24900
U+79D1.0	fHakka	  ko1
U+79D1.0	fCantonese	fo1
U+79D1.0	fMandarin	ke1; ke4
U+79D1.0	fEnglish	[1] section [2] department [3] class; variety; family (of plants or animals) [4] rules; laws [5] the action in Chinese opera [6] subject in the civil service examination of former times [7] branch of academic or vocational studies [8] mete out (prison, terms, etc.); levy (taxes, etc.); fine someone
U+79D2.0	fUTF8	秒
U+79D2.0	fCNS11643	1-5270
U+79D2.0	fBig5	ACED
U+79D2.0	fGB	0-436B
U+79D2.0	fR/S	 115.4
U+79D2.0	fTotalStrokes	 9
U+79D2.0	fCangjie	HDFH
U+79D2.0	fFourCorner	29920
U+79D2.0	fHakka	 miau1;  miau3; meu3; cau1
U+79D2.0	fCantonese	miu5; miu2 (also)
U+79D2.0	fMandarin	miao3
U+79D2.0	fEnglish	[1] beard of grain or corn [2] second (said of time or degree)
U+79D3.0	fUTF8	秓
U+79D3.0	fCNS11643	3-2E40
U+79D3.0	fGB	3-5B57
U+79D3.0	fR/S	 115.4
U+79D3.0	fTotalStrokes	 9
U+79D3.0	fCangjie	HDJE
U+79D3.0	fFourCorner	24940
U+79D3.0	fHakka	 tje1
U+79D3.0	fCantonese	zi1
U+79D3.0	fMandarin	zhi1
U+79D4.0	fUTF8	秔
U+79D4.0	fCNS11643	3-2E41
U+79D4.0	fGB	E-357D
U+79D4.0	fR/S	  115.4
U+79D4.0	fTotalStrokes	 9
U+79D4.0	fCangjie	HDYHN
U+79D4.0	fFourCorner	20910
U+79D4.0	fHakka	 gang1
U+79D4.0	fCantonese	gang1
U+79D4.0	fMandarin	geng1; jing1
U+79D4.0	fEnglish	non-glutinous rice
U+79D5.0	fUTF8	秕
U+79D5.0	fCNS11643	2-2C5B
U+79D5.0	fBig5	CFF9
U+79D5.0	fGB	0-6F75
U+79D5.0	fR/S	 115.4
U+79D5.0	fTotalStrokes	 9
U+79D5.0	fCangjie	HDPP
U+79D5.0	fFourCorner	21910
U+79D5.0	fHakka	 bi3; pi3; pi1
U+79D5.0	fCantonese	bei2
U+79D5.0	fMandarin	bi3
U+79D5.0	fEnglish	[1] empty grain or rice husk; husks; grains not fully grown; blasted grains [2] mean; no good; not qualified or competent
U+79D6.0	fUTF8	秖
U+79D6.0	fCNS11643	2-2C5E
U+79D6.0	fBig5	CFFC
U+79D6.0	fGB	E-357E
U+79D6.0	fR/S	 115.4
U+79D6.0	fTotalStrokes	 9
U+79D6.0	fCangjie	HDHVP
U+79D6.0	fFourCorner	22940
U+79D6.0	fHakka	 zhi1; zih1
U+79D6.0	fCantonese	zi2; zi1; kei4
U+79D6.0	fMandarin	zhi1; zhi3
U+79D7.0	fUTF8	秗
U+79D7.0	fCNS11643	3-2E43
U+79D7.0	fGB	E-3621
U+79D7.0	fR/S	 115.4
U+79D7.0	fTotalStrokes	 9
U+79D7.0	fCangjie	HDHK
U+79D7.0	fFourCorner	22934
U+79D7.0	fCantonese	jyu3
U+79D7.0	fMandarin	yu4
U+79D8.0	fUTF8	秘
U+79D8.0	fCNS11643	1-5738
U+79D8.0	fBig5	AFB5
U+79D8.0	fGB	0-4358
U+79D8.0	fR/S	 115.5
U+79D8.0	fTotalStrokes	 10
U+79D8.0	fCangjie	HDPH
U+79D8.0	fFourCorner	23900
U+79D8.0	fHakka	  bi5
U+79D8.0	fCantonese	bei3; bai3; bit1
U+79D8.0	fMandarin	mi4; bi4; bie2
U+79D8.0	fEnglish	secret; mysterious; abstruce
U+79D9.0	fUTF8	秙
U+79D9.0	fCNS11643	3-3274
U+79D9.0	fGB	3-5B5C
U+79D9.0	fR/S	  115.5
U+79D9.0	fTotalStrokes	 10
U+79D9.0	fCangjie	HDJR
U+79D9.0	fFourCorner	24960
U+79D9.0	fCantonese	fu3
U+79D9.0	fMandarin	ku4
U+79DA.0	fUTF8	秚
U+79DA.0	fCNS11643	3-3273
U+79DA.0	fGB	E-3622
U+79DA.0	fR/S	  115.5
U+79DA.0	fTotalStrokes	 10
U+79DA.0	fCangjie	HDFQ
U+79DA.0	fFourCorner	29950
U+79DA.0	fCantonese	bun6
U+79DA.0	fMandarin	ban4
U+79DB.0	fUTF8	秛
U+79DB.0	fCNS11643	3-3279
U+79DB.0	fGB	3-5B67
U+79DB.0	fR/S	  115.5
U+79DB.0	fTotalStrokes	 10
U+79DB.0	fCangjie	HDDHE
U+79DB.0	fFourCorner	24940
U+79DB.0	fHakka	 pi1
U+79DB.0	fCantonese	pei1
U+79DB.0	fMandarin	pi1; pei1
U+79DC.0	fUTF8	秜
U+79DC.0	fCNS11643	2-3156
U+79DC.0	fBig5	D2F3
U+79DC.0	fGB	3-5B65
U+79DC.0	fR/S	 115.5
U+79DC.0	fTotalStrokes	 10
U+79DC.0	fCangjie	HDSP
U+79DC.0	fFourCorner	27911
U+79DC.0	fHakka	  ni2
U+79DC.0	fCantonese	nei4; lei4 (also)
U+79DC.0	fMandarin	ni2; li2; ni4
U+79DC.0	fEnglish	self-grown rice
U+79DD.0	fUTF8	秝
U+79DD.0	fCNS11643	2-3158
U+79DD.0	fBig5	D2F5
U+79DD.0	fGB	5-5B25
U+79DD.0	fR/S	 115.5
U+79DD.0	fTotalStrokes	 10
U+79DD.0	fCangjie	HDHD
U+79DD.0	fFourCorner	22994
U+79DD.0	fHakka	 lit8; let8
U+79DD.0	fCantonese	lik6; lang3; ling3
U+79DD.0	fMandarin	li4
U+79DD.0	fEnglish	rare; seldom met with
U+79DE.0	fUTF8	秞
U+79DE.0	fCNS11643	2-3157
U+79DE.0	fBig5	D2F4
U+79DE.0	fGB	3-5B5F
U+79DE.0	fR/S	 115.5
U+79DE.0	fTotalStrokes	 10
U+79DE.0	fCangjie	HDLW
U+79DE.0	fFourCorner	25960
U+79DE.0	fCantonese	jau4
U+79DE.0	fMandarin	you2
U+79DF.0	fUTF8	租
U+79DF.0	fCNS11643	1-5735
U+79DF.0	fBig5	AFB2
U+79DF.0	fGB	0-5762
U+79DF.0	fR/S	 115.5
U+79DF.0	fTotalStrokes	 10
U+79DF.0	fCangjie	HDBM
U+79DF.0	fFourCorner	27910
U+79DF.0	fHakka	 zu1; zih1; zii1; zhu1
U+79DF.0	fCantonese	zou1
U+79DF.0	fMandarin	zu1; ju1
U+79DF.0	fEnglish	[1] rent; lease; let [2] rent; rental [3] taxes; tax
U+79E0.0	fUTF8	秠
U+79E0.0	fCNS11643	2-3152
U+79E0.0	fBig5	D2EF
U+79E0.0	fGB	3-5B5D
U+79E0.0	fR/S	 115.5
U+79E0.0	fTotalStrokes	 10
U+79E0.0	fCangjie	HDMFM
U+79E0.0	fFourCorner	21919
U+79E0.0	fHakka	  pi1
U+79E0.0	fCantonese	pei1; pei2
U+79E0.0	fMandarin	pi1; pi3; pei1; fou3; fou2
U+79E0.0	fEnglish	millet
U+79E1.0	fUTF8	秡
U+79E1.0	fCNS11643	3-3276
U+79E1.0	fGB	E-3623
U+79E1.0	fR/S	  115.5
U+79E1.0	fTotalStrokes	 10
U+79E1.0	fCangjie	HDIKK
U+79E1.0	fFourCorner	23940
U+79E1.0	fCantonese	bat3
U+79E1.0	fMandarin	ba2
U+79E2.0	fUTF8	秢
U+79E2.0	fCNS11643	3-327B
U+79E2.0	fGB	3-5B62
U+79E2.0	fR/S	 115.5
U+79E2.0	fTotalStrokes	 10
U+79E2.0	fCangjie	HDOII
U+79E2.0	fFourCorner	28930
U+79E2.0	fCantonese	ling4
U+79E2.0	fMandarin	ling2
U+79E3.0	fUTF8	秣
U+79E3.0	fCNS11643	1-5733
U+79E3.0	fBig5	AFB0
U+79E3.0	fGB	0-6F77
U+79E3.0	fR/S	 115.5
U+79E3.0	fTotalStrokes	 10
U+79E3.0	fCangjie	HDDJ
U+79E3.0	fFourCorner	25990
U+79E3.0	fHakka	 mat7; mat8
U+79E3.0	fCantonese	mut3 (also); mut6
U+79E3.0	fMandarin	mo4; mei4
U+79E3.0	fEnglish	[1] horse feed; fodder [2] feed a horse
U+79E4.0	fUTF8	秤
U+79E4.0	fCNS11643	1-5732
U+79E4.0	fBig5	AFAF
U+79E4.0	fGB	0-3353
U+79E4.0	fR/S	 115.5
U+79E4.0	fTotalStrokes	 10
U+79E4.0	fCangjie	HDMFJ
U+79E4.0	fFourCorner	21949
U+79E4.0	fHakka	  chin5; cin5; cihn5; chin1; cihn1
U+79E4.0	fCantonese	cing3; ping4; cing1
U+79E4.0	fMandarin	cheng4; cheng1; chen4
U+79E4.0	fEnglish	[1] weighing scale; balance; steelyard [2] weight with a scale, etc.
U+79E4.1	fUTF8	秤
U+79E4.1	fCNS11643	1-5732
U+79E4.1	fBig5	AFAF
U+79E4.1	fGB	0-3353
U+79E4.1	fR/S	 115.5
U+79E4.1	fTotalStrokes	 10
U+79E4.1	fCangjie	HDMFJ
U+79E4.1	fFourCorner	21949
U+79E4.1	fMandarin	ping2
U+79E4.1	fEnglish	 scales for measuring weight
U+79E5.0	fUTF8	秥
U+79E5.0	fCNS11643	3-3277
U+79E5.0	fGB	E-3624
U+79E5.0	fR/S	  115.5
U+79E5.0	fTotalStrokes	 10
U+79E5.0	fCangjie	HDYR
U+79E5.0	fFourCorner	21960
U+79E5.0	fHakka	 zham1; zam1
U+79E5.0	fCantonese	nim4; lim4 (also); zim1
U+79E5.0	fMandarin	nian2
U+79E6.0	fUTF8	秦
U+79E6.0	fCNS11643	1-5736
U+79E6.0	fBig5	AFB3
U+79E6.0	fGB	0-4758
U+79E6.0	fR/S	 115.5
U+79E6.0	fTotalStrokes	 10
U+79E6.0	fCangjie	QKHD
U+79E6.0	fFourCorner	50904
U+79E6.0	fHakka	  cin2; cin3
U+79E6.0	fCantonese	ceon4
U+79E6.0	fMandarin	qin2
U+79E6.0	fEnglish	[1] the feudal state of Chin (879-221 B.C.) in the Chou dynasty [2] another name of Shensi province [3] the ancient name of China as known to the people of the Western Regions [4] a Chinese family name
U+79E7.0	fUTF8	秧
U+79E7.0	fCNS11643	1-5734
U+79E7.0	fBig5	AFB1
U+79E7.0	fGB	0-516D
U+79E7.0	fR/S	 115.5
U+79E7.0	fTotalStrokes	 10
U+79E7.0	fCangjie	HDLBK
U+79E7.0	fFourCorner	25930
U+79E7.0	fHakka	  jong1; rong1
U+79E7.0	fCantonese	joeng1
U+79E7.0	fMandarin	yang1
U+79E7.0	fEnglish	[1] rice seedling; young rice plant [2] tree saplings; very young plants for transplanting [3] cultivate; grow
U+79E8.0	fUTF8	秨
U+79E8.0	fCNS11643	3-3278
U+79E8.0	fGB	3-5B61
U+79E8.0	fR/S	  115.5
U+79E8.0	fTotalStrokes	 10
U+79E8.0	fCangjie	HDOS
U+79E8.0	fFourCorner	28910
U+79E8.0	fCantonese	zok6
U+79E8.0	fMandarin	zuo2
U+79E9.0	fUTF8	秩
U+79E9.0	fCNS11643	1-5737
U+79E9.0	fBig5	AFB4
U+79E9.0	fGB	0-5648
U+79E9.0	fR/S	 115.5
U+79E9.0	fTotalStrokes	 10
U+79E9.0	fCangjie	HDHQO
U+79E9.0	fFourCorner	25930
U+79E9.0	fHakka	 zhit7; zit7; tsiit7
U+79E9.0	fCantonese	dit6
U+79E9.0	fMandarin	zhi4
U+79E9.0	fEnglish	[1] order; orderly [2] official ranks [3] official salary [4] decade
U+79EA.0	fUTF8	秪
U+79EA.0	fCNS11643	2-3155
U+79EA.0	fBig5	D2F2
U+79EA.0	fGB	3-5B63
U+79EA.0	fR/S	 115.5
U+79EA.0	fTotalStrokes	 10
U+79EA.0	fCangjie	HDHPM
U+79EA.0	fFourCorner	22940
U+79EA.0	fHakka	 zhi1; zih1; tje1
U+79EA.0	fCantonese	zi1
U+79EA.0	fMandarin	zhi1
U+79EA.0	fEnglish	only; merely
U+79EB.0	fUTF8	秫
U+79EB.0	fCNS11643	2-3150
U+79EB.0	fBig5	D2ED
U+79EB.0	fGB	0-6F78
U+79EB.0	fR/S	 115.5
U+79EB.0	fTotalStrokes	 10
U+79EB.0	fCangjie	HDIJC
U+79EB.0	fFourCorner	23934
U+79EB.0	fHakka	  sut8; shut8
U+79EB.0	fCantonese	seot6
U+79EB.0	fMandarin	shu2; shu4
U+79EB.0	fEnglish	a glutinous variety of millet; kaoliang; sorghum
U+79EC.0	fUTF8	秬
U+79EC.0	fCNS11643	2-3151
U+79EC.0	fBig5	D2EE
U+79EC.0	fGB	3-5B58
U+79EC.0	fR/S	 115.5
U+79EC.0	fTotalStrokes	 10
U+79EC.0	fCangjie	HDSS
U+79EC.0	fFourCorner	21917
U+79EC.0	fHakka	  ki3
U+79EC.0	fCantonese	geoi6
U+79EC.0	fMandarin	ju4
U+79EC.0	fEnglish	the black millet
U+79ED.0	fUTF8	秭
U+79ED.0	fCNS11643	2-3154
U+79ED.0	fBig5	D2F1
U+79ED.0	fGB	0-6F76
U+79ED.0	fR/S	 115.5; 115.4
U+79ED.0	fTotalStrokes	 10
U+79ED.0	fCangjie	HDLXH
U+79ED.0	fFourCorner	25927
U+79ED.0	fHakka	  zi3
U+79ED.0	fCantonese	zi2
U+79ED.0	fMandarin	zi3
U+79ED.0	fEnglish	one trillion
U+79EE.0	fUTF8	秮
U+79EE.0	fCNS11643	2-3153
U+79EE.0	fBig5	D2F0
U+79EE.0	fGB	E-3625
U+79EE.0	fR/S	 115.5
U+79EE.0	fTotalStrokes	 10
U+79EE.0	fCangjie	HDIR
U+79EE.0	fFourCorner	22963
U+79EE.0	fCantonese	wut6
U+79EE.0	fMandarin	huo2; tai2; kuo4
U+79EF.0	fUTF8	积
U+79EF.0	fGB	0-3B7D
U+79EF.0	fR/S	 115.5
U+79EF.0	fTotalStrokes	 10
U+79EF.0	fCangjie	HDRC
U+79EF.0	fFourCorner	2698
U+79EF.0	fHakka	 zit7
U+79EF.0	fCantonese	zik1
U+79EF.0	fMandarin	ji1; zi4
U+79EF.0	fEnglish	simplified form of &#x7A4D;
U+79F0.0	fUTF8	称
U+79F0.0	fCNS11643	3-327C
U+79F0.0	fGB	0-3346
U+79F0.0	fR/S	 115.5
U+79F0.0	fTotalStrokes	 10
U+79F0.0	fCangjie	HDNF
U+79F0.0	fFourCorner	27992
U+79F0.0	fHakka	 chin1
U+79F0.0	fCantonese	cing1; cing3; can3
U+79F0.0	fMandarin	cheng1; cheng4
U+79F0.0	fEnglish	simplified form of &#x7A31;
U+79F0.1	fUTF8	称
U+79F0.1	fCNS11643	3-327C
U+79F0.1	fGB	0-3346
U+79F0.1	fR/S	 115.5
U+79F0.1	fTotalStrokes	 10
U+79F0.1	fCangjie	HDNF
U+79F0.1	fFourCorner	27992
U+79F0.1	fMandarin	chen4
U+79F1.0	fUTF8	秱
U+79F1.0	fCNS11643	3-3769
U+79F1.0	fGB	3-5B70
U+79F1.0	fR/S	 115.6
U+79F1.0	fTotalStrokes	 11
U+79F1.0	fCangjie	HDBMR
U+79F1.0	fFourCorner	27920
U+79F1.0	fCantonese	tung4
U+79F1.0	fMandarin	tong2
U+79F2.0	fUTF8	秲
U+79F2.0	fCNS11643	3-3767
U+79F2.0	fGB	3-5B69
U+79F2.0	fR/S	 115.6
U+79F2.0	fTotalStrokes	 11
U+79F2.0	fCangjie	HDGDI
U+79F2.0	fFourCorner	24940
U+79F2.0	fCantonese	zi6
U+79F2.0	fMandarin	zhi4; shi4
U+79F3.0	fUTF8	秳
U+79F3.0	fCNS11643	3-376B
U+79F3.0	fGB	3-5B72
U+79F3.0	fR/S	  115.6
U+79F3.0	fTotalStrokes	 11
U+79F3.0	fCangjie	HDHJR
U+79F3.0	fFourCorner	22960
U+79F3.0	fCantonese	wut6
U+79F3.0	fMandarin	huo2; hua2
U+79F4.0	fUTF8	秴
U+79F4.0	fGB	5-5B2A
U+79F4.0	fR/S	 115.6
U+79F4.0	fTotalStrokes	 11
U+79F4.0	fCangjie	HDOMR
U+79F4.0	fFourCorner	2896
U+79F4.0	fCantonese	hap6
U+79F4.0	fMandarin	he2; ge2
U+79F5.0	fUTF8	秵
U+79F5.0	fCNS11643	3-376A
U+79F5.0	fGB	3-5B71
U+79F5.0	fR/S	  115.6
U+79F5.0	fTotalStrokes	 11
U+79F5.0	fCangjie	HDWK
U+79F5.0	fFourCorner	26900
U+79F5.0	fCantonese	jan1
U+79F5.0	fMandarin	yin1
U+79F6.0	fUTF8	秶
U+79F6.0	fCNS11643	2-3769
U+79F6.0	fBig5	D6C6
U+79F6.0	fGB	E-3626
U+79F6.0	fR/S	 115.6
U+79F6.0	fTotalStrokes	 11
U+79F6.0	fCangjie	IOHD
U+79F6.0	fFourCorner	37904
U+79F6.0	fCantonese	zi1
U+79F6.0	fMandarin	zi1; ci2; ji4
U+79F7.0	fUTF8	秷
U+79F7.0	fCNS11643	2-376A
U+79F7.0	fBig5	D6C7
U+79F7.0	fGB	3-5B6F
U+79F7.0	fR/S	 115.6
U+79F7.0	fTotalStrokes	 11
U+79F7.0	fCangjie	HDMIG
U+79F7.0	fFourCorner	21914
U+79F7.0	fHakka	 zhit7; tsiit7
U+79F7.0	fCantonese	zat1; zat6
U+79F7.0	fMandarin	zhi2
U+79F7.0	fEnglish	the sound of reaping
U+79F8.0	fUTF8	秸
U+79F8.0	fCNS11643	2-3768
U+79F8.0	fBig5	D6C5
U+79F8.0	fGB	0-3D55
U+79F8.0	fR/S	 115.6
U+79F8.0	fTotalStrokes	 11
U+79F8.0	fCangjie	HDGR
U+79F8.0	fFourCorner	24961
U+79F8.0	fHakka	  kat7;  git7; gai1
U+79F8.0	fCantonese	gaai1
U+79F8.0	fMandarin	jie1; ji2
U+79F8.0	fEnglish	stalks of millet; corn
U+79F9.0	fUTF8	秹
U+79F9.0	fCNS11643	3-376C
U+79F9.0	fGB	E-3627
U+79F9.0	fR/S	  115.6
U+79F9.0	fTotalStrokes	 11
U+79F9.0	fCangjie	HDOHG
U+79F9.0	fFourCorner	22910
U+79F9.0	fCantonese	jam5
U+79F9.0	fMandarin	ren3
U+79FA.0	fUTF8	秺
U+79FA.0	fCNS11643	2-3767
U+79FA.0	fBig5	D6C4
U+79FA.0	fGB	3-5B77
U+79FA.0	fR/S	 115.6
U+79FA.0	fTotalStrokes	 11
U+79FA.0	fCangjie	HDJHP
U+79FA.0	fFourCorner	23914
U+79FA.0	fCantonese	dou6; dou3
U+79FA.0	fMandarin	du4; zha1
U+79FB.0	fUTF8	移
U+79FB.0	fCNS11643	1-5C43
U+79FB.0	fBig5	B2BE
U+79FB.0	fGB	0-5246
U+79FB.0	fR/S	 115.6
U+79FB.0	fTotalStrokes	 11
U+79FB.0	fCangjie	HDNIN
U+79FB.0	fFourCorner	27927
U+79FB.0	fHakka	 ji2; ji3; ri2; i2
U+79FB.0	fCantonese	ji4
U+79FB.0	fMandarin	yi2; yi4; chi3
U+79FB.0	fEnglish	[1] change; alter; influence; affect [2] shift; move [3] forward; transmit; transfer; transplant; convey [4] give; endow
U+79FC.0	fUTF8	秼
U+79FC.0	fCNS11643	F-373F
U+79FC.0	fGB	E-3628
U+79FC.0	fR/S	 115.6
U+79FC.0	fTotalStrokes	 11
U+79FC.0	fCangjie	HDHJD
U+79FC.0	fFourCorner	2599
U+79FC.0	fCantonese	zyu3
U+79FC.0	fMandarin	zhu4; zhu1
U+79FD.0	fUTF8	秽
U+79FD.0	fGB	0-3B60
U+79FD.0	fR/S	 115.6
U+79FD.0	fTotalStrokes	 11
U+79FD.0	fCangjie	HDUNI
U+79FD.0	fHakka	 ve5; voi5; je5
U+79FD.0	fCantonese	wai3
U+79FD.0	fMandarin	hui4; wei4
U+79FD.0	fEnglish	simplified form of &#x7A62;
U+79FE.0	fUTF8	秾
U+79FE.0	fGB	8-7D58
U+79FE.0	fR/S	 115.6; 115.13
U+79FE.0	fTotalStrokes	 11
U+79FE.0	fCangjie	HDHBV
U+79FE.0	fCantonese	nung4; lung4 (also)
U+79FE.0	fMandarin	nong2
U+79FE.0	fEnglish	thick, lush, in clusters
U+79FF.0	fUTF8	秿
U+79FF.0	fCNS11643	3-3D65
U+79FF.0	fGB	3-5B7A
U+79FF.0	fR/S	  115.7
U+79FF.0	fTotalStrokes	 12
U+79FF.0	fCangjie	HDIJB
U+79FF.0	fFourCorner	23920
U+79FF.0	fCantonese	fu6
U+79FF.0	fMandarin	fu3
U+7A00.0	fUTF8	稀
U+7A00.0	fCNS11643	1-6125
U+7A00.0	fBig5	B57D
U+7A00.0	fGB	0-4F21
U+7A00.0	fR/S	 115.7
U+7A00.0	fTotalStrokes	 12
U+7A00.0	fCangjie	HDKKB
U+7A00.0	fFourCorner	24927
U+7A00.0	fHakka	  hi1
U+7A00.0	fCantonese	hei1
U+7A00.0	fMandarin	xi1
U+7A00.0	fEnglish	[1] rare; unusual; scarce; sparse [2] thin (liquids, etc.); watery; diluted
U+7A01.0	fUTF8	稁
U+7A01.0	fCNS11643	6-4C48
U+7A01.0	fGB	E-3629
U+7A01.0	fR/S	 115.7
U+7A01.0	fTotalStrokes	 12
U+7A01.0	fCangjie	YRBD
U+7A01.0	fFourCorner	00900
U+7A01.0	fCantonese	gou2
U+7A01.0	fMandarin	gao3; kao4; kao3; gao4
U+7A02.0	fUTF8	稂
U+7A02.0	fCNS11643	2-3E5A
U+7A02.0	fBig5	DAD6
U+7A02.0	fGB	0-6F7C
U+7A02.0	fR/S	 115.7
U+7A02.0	fTotalStrokes	 12
U+7A02.0	fCangjie	HDIAV
U+7A02.0	fFourCorner	23932
U+7A02.0	fHakka	  long2
U+7A02.0	fCantonese	long4
U+7A02.0	fMandarin	lang2
U+7A02.0	fEnglish	grass or weeds which are particularly harmful to the growth of rice seedlings; wolf's-tail grass
U+7A03.0	fUTF8	稃
U+7A03.0	fCNS11643	2-3E5C
U+7A03.0	fBig5	DAD8
U+7A03.0	fGB	0-6F7B
U+7A03.0	fR/S	 115.7
U+7A03.0	fTotalStrokes	 12
U+7A03.0	fCangjie	HDBND
U+7A03.0	fFourCorner	22947
U+7A03.0	fHakka	  fu1
U+7A03.0	fCantonese	fu1; fau4
U+7A03.0	fMandarin	fu1; fu2
U+7A03.0	fEnglish	bran of rice
U+7A04.0	fUTF8	稄
U+7A04.0	fCNS11643	2-3E5E
U+7A04.0	fBig5	DADA
U+7A04.0	fGB	5-5B34
U+7A04.0	fR/S	 115.7
U+7A04.0	fTotalStrokes	 12
U+7A04.0	fCangjie	HDICE
U+7A04.0	fFourCorner	23947
U+7A04.0	fCantonese	seon3; zak1
U+7A04.0	fMandarin	ze4
U+7A05.0	fUTF8	稅
U+7A05.0	fCNS11643	1-6124
U+7A05.0	fBig5	B57C
U+7A05.0	fGB	E-362A
U+7A05.0	fR/S	 115.7
U+7A05.0	fTotalStrokes	 12
U+7A05.0	fCangjie	HDCRU
U+7A05.0	fFourCorner	28916
U+7A05.0	fHakka	  soi5; s'oi6
U+7A05.0	fCantonese	seoi3; teoi3; tyut3
U+7A05.0	fMandarin	shui4; tui4; tuan4; tuo1
U+7A05.0	fEnglish	[1] taxes; revenue; duties on commodities; tax [2] lay; place [3] present another with
U+7A06.0	fUTF8	稆
U+7A06.0	fCNS11643	6-4C43
U+7A06.0	fGB	0-6F79
U+7A06.0	fR/S	 115.6; 115.7
U+7A06.0	fTotalStrokes	 12; 11
U+7A06.0	fCangjie	HDRR
U+7A06.0	fFourCorner	26960
U+7A06.0	fHakka	 li3
U+7A06.0	fCantonese	leoi5
U+7A06.0	fMandarin	luu3; l3
U+7A06.0	fEnglish	wild grain
U+7A07.0	fUTF8	稇
U+7A07.0	fCNS11643	3-3D66
U+7A07.0	fGB	5-5B30
U+7A07.0	fR/S	  115.7
U+7A07.0	fTotalStrokes	 12
U+7A07.0	fCangjie	HDWD
U+7A07.0	fFourCorner	26900
U+7A07.0	fHakka	 kun3
U+7A07.0	fCantonese	kwan2
U+7A07.0	fMandarin	kun3
U+7A07.0	fEnglish	 bind as faggots of sheaves
U+7A08.0	fUTF8	稈
U+7A08.0	fCNS11643	1-6122
U+7A08.0	fBig5	B57A
U+7A08.0	fGB	E-362B
U+7A08.0	fR/S	 115.7
U+7A08.0	fTotalStrokes	 12
U+7A08.0	fCangjie	HDAMJ
U+7A08.0	fFourCorner	26941
U+7A08.0	fHakka	  gon3
U+7A08.0	fCantonese	gon2
U+7A08.0	fMandarin	gan3
U+7A08.0	fEnglish	the stalk of grain; straw
U+7A09.0	fUTF8	稉
U+7A09.0	fCNS11643	3-3D69
U+7A09.0	fGB	E-362C
U+7A09.0	fR/S	  115.7
U+7A09.0	fTotalStrokes	 12
U+7A09.0	fCangjie	HDMLK
U+7A09.0	fFourCorner	21940
U+7A09.0	fHakka	 gang1
U+7A09.0	fCantonese	gang1
U+7A09.0	fMandarin	geng1; jing1
U+7A09.0	fEnglish	non-glutinous rice
U+7A0A.0	fUTF8	稊
U+7A0A.0	fCNS11643	2-3E5B
U+7A0A.0	fBig5	DAD7
U+7A0A.0	fGB	3-5C21
U+7A0A.0	fR/S	 115.7
U+7A0A.0	fTotalStrokes	 12
U+7A0A.0	fCangjie	HDCNH
U+7A0A.0	fFourCorner	28927
U+7A0A.0	fHakka	  ti2
U+7A0A.0	fCantonese	tai4
U+7A0A.0	fMandarin	ti2
U+7A0A.0	fEnglish	darnels; tares
U+7A0B.0	fUTF8	程
U+7A0B.0	fCNS11643	1-6123
U+7A0B.0	fBig5	B57B
U+7A0B.0	fGB	0-334C
U+7A0B.0	fR/S	 115.7
U+7A0B.0	fTotalStrokes	 12
U+7A0B.0	fCangjie	HDRHG
U+7A0B.0	fFourCorner	26914
U+7A0B.0	fHakka	 cin2; cang2; ch'ang2; chin2; chin3; cihn2; chang2
U+7A0B.0	fCantonese	cing4; cing2 (also)
U+7A0B.0	fMandarin	cheng2
U+7A0B.0	fEnglish	[1] form; pattern [2] degree; extent [3] journey; trip [4] schedule; agenda; order [5] course [6] measure; assess [7] distance [8] a general name of measurements of all kinds [9] a Chinese family name
U+7A0C.0	fUTF8	稌
U+7A0C.0	fCNS11643	2-3E5D
U+7A0C.0	fBig5	DAD9
U+7A0C.0	fGB	3-5B7C
U+7A0C.0	fR/S	 115.7
U+7A0C.0	fTotalStrokes	 12
U+7A0C.0	fCangjie	HDOMD
U+7A0C.0	fFourCorner	28994
U+7A0C.0	fHakka	  tu2
U+7A0C.0	fCantonese	tou4; dou6
U+7A0C.0	fMandarin	tu2
U+7A0C.0	fEnglish	(glutinous) rice
U+7A0D.0	fUTF8	稍
U+7A0D.0	fCNS11643	1-6121
U+7A0D.0	fBig5	B579
U+7A0D.0	fGB	0-4954
U+7A0D.0	fR/S	 115.7
U+7A0D.0	fTotalStrokes	 12
U+7A0D.0	fCangjie	HDFB
U+7A0D.0	fFourCorner	29927
U+7A0D.0	fHakka	  sau1; siau3; siau1; sau5
U+7A0D.0	fCantonese	saau2
U+7A0D.0	fMandarin	shao4; shao1
U+7A0D.0	fEnglish	[1] little; slightly; slight [2] somewhat; rather [3] gradually
U+7A0E.0	fUTF8	税
U+7A0E.0	fCNS11643	3-3D68
U+7A0E.0	fGB	0-4B30
U+7A0E.0	fR/S	 115.7
U+7A0E.0	fTotalStrokes	 12
U+7A0E.0	fCangjie	HDCRU
U+7A0E.0	fFourCorner	28910
U+7A0E.0	fHakka	 s'oi5; soi5; choe5
U+7A0E.0	fCantonese	seoi3
U+7A0E.0	fMandarin	shui4; tuo1; tuan4; yue4; tui4
U+7A0E.0	fEnglish	taxes
U+7A0F.0	fUTF8	稏
U+7A0F.0	fCNS11643	3-4341
U+7A0F.0	fGB	3-5B6B
U+7A0F.0	fR/S	  115.8
U+7A0F.0	fTotalStrokes	 13
U+7A0F.0	fCangjie	HDMLM
U+7A0F.0	fFourCorner	21910
U+7A0F.0	fHakka	  a3
U+7A0F.0	fCantonese	aa3; ngaa3 (also)
U+7A0F.0	fMandarin	ya4
U+7A0F.0	fEnglish	grain in the ear
U+7A10.0	fUTF8	稐
U+7A10.0	fCNS11643	2-4566
U+7A10.0	fBig5	DF41
U+7A10.0	fGB	5-5B21
U+7A10.0	fR/S	 115.8
U+7A10.0	fTotalStrokes	 13
U+7A10.0	fCangjie	HDOMB
U+7A10.0	fFourCorner	28927
U+7A10.0	fHakka	 lun3
U+7A10.0	fCantonese	leon5
U+7A10.0	fMandarin	lun3
U+7A10.0	fEnglish	 bind grain in sheaves
U+7A11.0	fUTF8	稑
U+7A11.0	fCNS11643	2-455D
U+7A11.0	fBig5	DEF7
U+7A11.0	fGB	3-5C22
U+7A11.0	fR/S	 115.8
U+7A11.0	fTotalStrokes	 13
U+7A11.0	fCangjie	HDGCG
U+7A11.0	fFourCorner	24914
U+7A11.0	fHakka	  liuk8
U+7A11.0	fCantonese	luk6; ling4
U+7A11.0	fMandarin	lu4
U+7A11.0	fEnglish	rice
U+7A12.0	fUTF8	稒
U+7A12.0	fCNS11643	2-4560
U+7A12.0	fBig5	DEFA
U+7A12.0	fGB	3-5C27
U+7A12.0	fR/S	 115.8
U+7A12.0	fTotalStrokes	 13
U+7A12.0	fCangjie	HDWJR
U+7A12.0	fFourCorner	26900
U+7A12.0	fCantonese	gu3; gwu3 (also)
U+7A12.0	fMandarin	gu4
U+7A13.0	fUTF8	稓
U+7A13.0	fCNS11643	2-4564
U+7A13.0	fBig5	DEFE
U+7A13.0	fGB	3-5C23
U+7A13.0	fR/S	 115.8
U+7A13.0	fTotalStrokes	 13
U+7A13.0	fCangjie	HDTA
U+7A13.0	fFourCorner	24961
U+7A13.0	fHakka	 cok7
U+7A13.0	fCantonese	zok6; cok3
U+7A13.0	fMandarin	zuo2
U+7A13.0	fEnglish	a Chinese family name
U+7A14.0	fUTF8	稔
U+7A14.0	fCNS11643	1-6561
U+7A14.0	fBig5	B85A
U+7A14.0	fGB	0-6F7E
U+7A14.0	fR/S	 115.8
U+7A14.0	fTotalStrokes	 13
U+7A14.0	fCangjie	HDOIP
U+7A14.0	fFourCorner	28932
U+7A14.0	fHakka	  ngim3;  nem3; shim3
U+7A14.0	fCantonese	nam5; nim1; lam5 (also); lim1 (also); jam5
U+7A14.0	fMandarin	ren3
U+7A14.0	fEnglish	[1] ripening of paddy or rice; harvest [2] know; be familiar with somebody [3] year [4] used to; often [5] accumulate; hoard
U+7A15.0	fUTF8	稕
U+7A15.0	fCNS11643	2-4562
U+7A15.0	fBig5	DEFC
U+7A15.0	fGB	5-5B38
U+7A15.0	fR/S	 115.8
U+7A15.0	fTotalStrokes	 13
U+7A15.0	fCangjie	HDYRD
U+7A15.0	fFourCorner	20947
U+7A15.0	fHakka	 zhun5; zun5; zhun3
U+7A15.0	fCantonese	zeon3
U+7A15.0	fMandarin	zhun4
U+7A15.0	fEnglish	truss of hay or straw
U+7A16.0	fUTF8	稖
U+7A16.0	fCNS11643	4-3F50
U+7A16.0	fGB	5-5B3A
U+7A16.0	fR/S	  115.8
U+7A16.0	fTotalStrokes	 13
U+7A16.0	fCangjie	HDYTR
U+7A16.0	fFourCorner	20960
U+7A16.0	fCantonese	bong6; paang5 (also)
U+7A16.0	fMandarin	bang4
U+7A17.0	fUTF8	稗
U+7A17.0	fCNS11643	2-4561
U+7A17.0	fBig5	DEFB
U+7A17.0	fGB	0-305E
U+7A17.0	fR/S	 115.8
U+7A17.0	fTotalStrokes	 13
U+7A17.0	fCangjie	HDHWJ
U+7A17.0	fFourCorner	26940
U+7A17.0	fHakka	 pai3;  pai5; pai6; bi5; kwai5
U+7A17.0	fCantonese	baai6; bai6 (also); pei4
U+7A17.0	fMandarin	bai4; bi4
U+7A17.0	fEnglish	[1] baynard grass [2] small; little [3] novels; legends
U+7A18.0	fUTF8	稘
U+7A18.0	fCNS11643	2-455E
U+7A18.0	fBig5	DEF8
U+7A18.0	fGB	5-5B35
U+7A18.0	fR/S	 115.8
U+7A18.0	fTotalStrokes	 13
U+7A18.0	fCangjie	HDTMC
U+7A18.0	fFourCorner	24981
U+7A18.0	fHakka	 ki2; hi1; gi1
U+7A18.0	fCantonese	gei1
U+7A18.0	fMandarin	ji1; qi2
U+7A18.0	fEnglish	a full year of twelve or thirteen moons
U+7A19.0	fUTF8	稙
U+7A19.0	fCNS11643	2-455F
U+7A19.0	fBig5	DEF9
U+7A19.0	fGB	3-5C24
U+7A19.0	fR/S	 115.8
U+7A19.0	fTotalStrokes	 13
U+7A19.0	fCangjie	HDJBM
U+7A19.0	fFourCorner	24916
U+7A19.0	fHakka	 chit8; zhit7; ts'iit8; tsiit7
U+7A19.0	fCantonese	zik6; zik1
U+7A19.0	fMandarin	zhi2
U+7A19.0	fEnglish	grain ready for grinding
U+7A1A.0	fUTF8	稚
U+7A1A.0	fCNS11643	1-655F
U+7A1A.0	fBig5	B858
U+7A1A.0	fGB	0-5649
U+7A1A.0	fR/S	 115.8
U+7A1A.0	fTotalStrokes	 13
U+7A1A.0	fCangjie	HDOG
U+7A1A.0	fFourCorner	20914
U+7A1A.0	fHakka	 cih1; ci1; chi1; ci3; chi5; chi6; cih5; {chi1}; chii5
U+7A1A.0	fCantonese	zi6
U+7A1A.0	fMandarin	zhi4
U+7A1A.0	fEnglish	young and tender; immature; childish; small; delicate
U+7A1B.0	fUTF8	稛
U+7A1B.0	fCNS11643	2-4565
U+7A1B.0	fBig5	DF40
U+7A1B.0	fGB	3-5C28
U+7A1B.0	fR/S	 115.8
U+7A1B.0	fTotalStrokes	 13
U+7A1B.0	fCangjie	HDWHD
U+7A1B.0	fFourCorner	26900
U+7A1B.0	fHakka	 kun3
U+7A1B.0	fCantonese	kwan2
U+7A1B.0	fMandarin	kun3; qun3
U+7A1B.0	fEnglish	bind
U+7A1C.0	fUTF8	稜
U+7A1C.0	fCNS11643	1-655E
U+7A1C.0	fBig5	B857
U+7A1C.0	fGB	E-362D
U+7A1C.0	fR/S	 115.8
U+7A1C.0	fTotalStrokes	 13
U+7A1C.0	fCangjie	HDGCE
U+7A1C.0	fFourCorner	24947
U+7A1C.0	fHakka	 lin2
U+7A1C.0	fCantonese	ling4; ling6
U+7A1C.0	fMandarin	leng2; leng4; ling2
U+7A1C.0	fEnglish	[1] corner; edge; angle [2] square piece of wood; square block [3] awe-inspiring air
U+7A1D.0	fUTF8	稝
U+7A1D.0	fCNS11643	3-4344
U+7A1D.0	fGB	3-5C2A
U+7A1D.0	fR/S	  115.8
U+7A1D.0	fTotalStrokes	 13
U+7A1D.0	fCangjie	HDBB
U+7A1D.0	fFourCorner	27920
U+7A1D.0	fCantonese	paang4
U+7A1D.0	fMandarin	peng2
U+7A1E.0	fUTF8	稞
U+7A1E.0	fCNS11643	1-6563
U+7A1E.0	fBig5	B85C
U+7A1E.0	fGB	0-6F7D
U+7A1E.0	fR/S	 115.8
U+7A1E.0	fTotalStrokes	 13
U+7A1E.0	fCangjie	HDWD
U+7A1E.0	fFourCorner	26994
U+7A1E.0	fHakka	 ko1; ko5
U+7A1E.0	fCantonese	fo1; fo3
U+7A1E.0	fMandarin	ke1; hua4
U+7A1E.0	fEnglish	wheat or barley
U+7A1F.0	fUTF8	稟
U+7A1F.0	fCNS11643	1-6562
U+7A1F.0	fBig5	B85B
U+7A1F.0	fGB	E-362E
U+7A1F.0	fR/S	 115.8
U+7A1F.0	fTotalStrokes	 13
U+7A1F.0	fCangjie	YWRD
U+7A1F.0	fFourCorner	00901
U+7A1F.0	fHakka	 lim3; bin3
U+7A1F.0	fCantonese	lam5; ban2
U+7A1F.0	fMandarin	bing3; lin3; bin3; ding3
U+7A1F.0	fEnglish	[1] report to a superior or one's seniors; petition; appeal [2] receive commands; at the behest of [3] one's natural endowments or gifts
U+7A20.0	fUTF8	稠
U+7A20.0	fCNS11643	1-6560
U+7A20.0	fBig5	B859
U+7A20.0	fGB	0-336D
U+7A20.0	fR/S	 115.8
U+7A20.0	fTotalStrokes	 13
U+7A20.0	fCangjie	HDBGR
U+7A20.0	fFourCorner	27920
U+7A20.0	fHakka	 cu2;  chiu2; ciu2; zhiu1; zu1; chu2
U+7A20.0	fCantonese	cau4
U+7A20.0	fMandarin	chou2; tiao4; tiao2; diao4
U+7A20.0	fEnglish	[1] dense; crowded; packed [2] thick; viscous (liquids) [3] a Chinese family name
U+7A21.0	fUTF8	稡
U+7A21.0	fCNS11643	4-3F4A
U+7A21.0	fGB	5-5B39
U+7A21.0	fR/S	 115.8
U+7A21.0	fTotalStrokes	 13
U+7A21.0	fCangjie	HDYOJ
U+7A21.0	fFourCorner	20948
U+7A21.0	fHakka	  cui5;  zut7
U+7A21.0	fCantonese	zeoi3
U+7A21.0	fMandarin	zu2; zui4
U+7A21.0	fEnglish	 collect; bring together
U+7A22.0	fUTF8	稢
U+7A22.0	fCNS11643	2-4563
U+7A22.0	fBig5	DEFD
U+7A22.0	fGB	3-5C25
U+7A22.0	fR/S	 115.8
U+7A22.0	fTotalStrokes	 13
U+7A22.0	fCangjie	HDIRM
U+7A22.0	fFourCorner	23950
U+7A22.0	fCantonese	juk1
U+7A22.0	fMandarin	yu4
U+7A23.0	fUTF8	稣
U+7A23.0	fGB	0-7655
U+7A23.0	fR/S	 115.8; 195.5
U+7A23.0	fTotalStrokes	 13
U+7A23.0	fCangjie	NMHD
U+7A23.0	fHakka	 sii1
U+7A23.0	fCantonese	sou1
U+7A23.0	fMandarin	su1; wei4
U+7A23.0	fEnglish	simplified form of &#x7A4C;
U+7A24.0	fUTF8	稤
U+7A24.0	fGB	1-7D50
U+7A24.0	fR/S	 115.8
U+7A24.0	fTotalStrokes	 13
U+7A24.0	fCangjie	HDYRF
U+7A24.0	fFourCorner	2099
U+7A24.0	fCantonese	loek6
U+7A24.0	fMandarin	le4; luue4; luue3; le3
U+7A25.0	fUTF8	稥
U+7A25.0	fCNS11643	3-6524
U+7A25.0	fGB	E-362F
U+7A25.0	fR/S	 115.8; 72.9
U+7A25.0	fTotalStrokes	 13
U+7A25.0	fCangjie	HDFA
U+7A25.0	fFourCorner	2060
U+7A25.0	fCantonese	hoeng1
U+7A26.0	fUTF8	稦
U+7A26.0	fCNS11643	2-4C50
U+7A26.0	fBig5	E349
U+7A26.0	fGB	3-5B56
U+7A26.0	fR/S	 115.9
U+7A26.0	fTotalStrokes	 14
U+7A26.0	fCangjie	HDDMQ
U+7A26.0	fFourCorner	24956
U+7A26.0	fCantonese	ji1
U+7A26.0	fMandarin	yi1
U+7A27.0	fUTF8	稧
U+7A27.0	fCNS11643	4-455D
U+7A27.0	fGB	3-5C2E
U+7A27.0	fR/S	 115.9
U+7A27.0	fTotalStrokes	 14
U+7A27.0	fCangjie	HDQHK
U+7A27.0	fFourCorner	27930
U+7A27.0	fCantonese	hai6
U+7A27.0	fMandarin	xi4
U+7A28.0	fUTF8	稨
U+7A28.0	fCNS11643	2-4C4F
U+7A28.0	fBig5	E348
U+7A28.0	fGB	5-5B41
U+7A28.0	fR/S	 115.9
U+7A28.0	fTotalStrokes	 14
U+7A28.0	fCangjie	HDHSB
U+7A28.0	fFourCorner	23927
U+7A28.0	fHakka	 bien1
U+7A28.0	fCantonese	bin2
U+7A28.0	fMandarin	bian1; bian3
U+7A28.0	fEnglish	[1] lentil [2] whip; lash [3] whip
U+7A29.0	fUTF8	稩
U+7A29.0	fCNS11643	3-484B
U+7A29.0	fGB	3-5C32
U+7A29.0	fR/S	  115.9
U+7A29.0	fTotalStrokes	 14
U+7A29.0	fCangjie	HDWB
U+7A29.0	fFourCorner	26920
U+7A29.0	fCantonese	gei3
U+7A29.0	fMandarin	ji4
U+7A2A.0	fUTF8	稪
U+7A2A.0	fCNS11643	3-484D
U+7A2A.0	fGB	E-3630
U+7A2A.0	fR/S	  115.9
U+7A2A.0	fTotalStrokes	 14
U+7A2A.0	fCangjie	HDOAE
U+7A2A.0	fFourCorner	28940
U+7A2A.0	fCantonese	fuk1
U+7A2A.0	fMandarin	fu4
U+7A2B.0	fUTF8	稫
U+7A2B.0	fCNS11643	2-4C4B
U+7A2B.0	fBig5	E344
U+7A2B.0	fGB	3-5C30
U+7A2B.0	fR/S	 115.9
U+7A2B.0	fTotalStrokes	 14
U+7A2B.0	fCangjie	HDMRW
U+7A2B.0	fFourCorner	21966
U+7A2B.0	fCantonese	pik1; bik1
U+7A2B.0	fMandarin	bi1; bi4
U+7A2C.0	fUTF8	稬
U+7A2C.0	fCNS11643	3-4850
U+7A2C.0	fGB	E-3631
U+7A2C.0	fR/S	  115.9
U+7A2C.0	fTotalStrokes	 14
U+7A2C.0	fCangjie	HDMBK
U+7A2C.0	fFourCorner	21930
U+7A2C.0	fHakka	 no5; no6
U+7A2C.0	fCantonese	no6; lo6 (also)
U+7A2C.0	fMandarin	nuo4
U+7A2C.0	fEnglish	glutinous; sticky
U+7A2D.0	fUTF8	稭
U+7A2D.0	fCNS11643	3-484F
U+7A2D.0	fGB	E-3632
U+7A2D.0	fR/S	 115.9
U+7A2D.0	fTotalStrokes	 14
U+7A2D.0	fCangjie	HDPPA
U+7A2D.0	fFourCorner	21960
U+7A2D.0	fHakka	  kat7;  gai1
U+7A2D.0	fCantonese	gaai1; gai1
U+7A2D.0	fMandarin	jie1
U+7A2D.0	fEnglish	the stalk of corn, hemp, etc.
U+7A2E.0	fUTF8	種
U+7A2E.0	fCNS11643	1-6A21
U+7A2E.0	fBig5	BAD8
U+7A2E.0	fGB	1-5656
U+7A2E.0	fR/S	 115.9
U+7A2E.0	fTotalStrokes	 14
U+7A2E.0	fCangjie	HDHJG
U+7A2E.0	fFourCorner	22914
U+7A2E.0	fHakka	  zung3; zhung5; zhung3
U+7A2E.0	fCantonese	zung2; zung3
U+7A2E.0	fEnglish	[1] seed of grain [2] race (of human beings) [3] descendants; offspring; posterity [4] species; genus; kind; sort [5] guts; grit
U+7A2E.1	fUTF8	種
U+7A2E.1	fCNS11643	1-6A21
U+7A2E.1	fBig5	BAD8
U+7A2E.1	fGB	1-5656
U+7A2E.1	fR/S	 115.9
U+7A2E.1	fTotalStrokes	 14
U+7A2E.1	fCangjie	HDHJG
U+7A2E.1	fFourCorner	22914
U+7A2E.1	fHakka	 zung5
U+7A2E.1	fMandarin	zhong3; chong2; zhong4
U+7A2E.1	fEnglish	[1] plant; sow; cultivate [2] vaccinate
U+7A2F.0	fUTF8	稯
U+7A2F.0	fCNS11643	2-4C4E
U+7A2F.0	fBig5	E347
U+7A2F.0	fGB	3-5C34
U+7A2F.0	fR/S	 115.9
U+7A2F.0	fTotalStrokes	 14
U+7A2F.0	fCangjie	HDUCE
U+7A2F.0	fFourCorner	22947
U+7A2F.0	fHakka	 zung1
U+7A2F.0	fCantonese	zung1
U+7A2F.0	fMandarin	zong1; zong3
U+7A2F.0	fEnglish	a kind of mushroom
U+7A30.0	fUTF8	稰
U+7A30.0	fCNS11643	2-4C4D
U+7A30.0	fBig5	E346
U+7A30.0	fGB	3-5C37
U+7A30.0	fR/S	 115.9
U+7A30.0	fTotalStrokes	 14
U+7A30.0	fCangjie	HDNOB
U+7A30.0	fFourCorner	27927
U+7A30.0	fCantonese	seoi2; seoi1
U+7A30.0	fMandarin	xu1; xu3
U+7A31.0	fUTF8	稱
U+7A31.0	fCNS11643	1-6A22
U+7A31.0	fBig5	BAD9
U+7A31.0	fGB	1-3346
U+7A31.0	fR/S	 115.9
U+7A31.0	fTotalStrokes	 14
U+7A31.0	fCangjie	HDBGB
U+7A31.0	fFourCorner	22957
U+7A31.0	fHakka	 cin1;  chin1; cin5; chin5; cihn1; cihn5
U+7A31.0	fCantonese	cing1; cing3; can3
U+7A31.0	fMandarin	cheng1
U+7A31.0	fEnglish	[1] call; name; name; appellation [2] weigh; measure weight [3] claim; report; declare [4] say; tell; state [5] offer as an excuse (as illness) [6] speak laudatory words; praise [7] take up (arms, etc.) [8] a Chinese family name
U+7A31.1	fUTF8	稱
U+7A31.1	fCNS11643	1-6A22
U+7A31.1	fBig5	BAD9
U+7A31.1	fGB	1-3346
U+7A31.1	fR/S	 115.9
U+7A31.1	fTotalStrokes	 14
U+7A31.1	fCangjie	HDBGB
U+7A31.1	fFourCorner	22957
U+7A31.1	fHakka	 cem1
U+7A31.1	fMandarin	cheng4
U+7A31.1	fEnglish	[1] steelyard; weighing machine [2] fit; proper; suitable; well-matched [3] symmetrical; be equal to; corresponding to
U+7A31.2	fUTF8	稱
U+7A31.2	fCNS11643	1-6A22
U+7A31.2	fBig5	BAD9
U+7A31.2	fGB	1-3346
U+7A31.2	fR/S	 115.9
U+7A31.2	fTotalStrokes	 14
U+7A31.2	fCangjie	HDBGB
U+7A31.2	fFourCorner	22957
U+7A31.2	fMandarin	chen4
U+7A31.2	fEnglish	fit; suitable; in accordance with
U+7A31.2	fUTF8	稱
U+7A31.2	fCNS11643	1-6A22
U+7A31.2	fBig5	BAD9
U+7A31.2	fGB	1-3346
U+7A31.2	fR/S	 115.9
U+7A31.2	fTotalStrokes	 14
U+7A31.2	fCangjie	HDBGB
U+7A31.2	fFourCorner	22957
U+7A31.2	fMandarin	chen4
U+7A31.2	fEnglish	fit; suitable; in accordance with
U+7A32.0	fUTF8	稲
U+7A32.0	fCNS11643	3-4852
U+7A32.0	fGB	E-3633
U+7A32.0	fR/S	 115.9
U+7A32.0	fTotalStrokes	 14
U+7A32.0	fCangjie	HDBLA
U+7A32.0	fFourCorner	22960
U+7A32.0	fCantonese	dou6
U+7A32.0	fMandarin	dao4
U+7A32.0	fEnglish	rice growing in field; rice plant
U+7A33.0	fUTF8	稳
U+7A33.0	fCNS11643	F-4A64
U+7A33.0	fGB	0-4E48
U+7A33.0	fR/S	 115.9
U+7A33.0	fTotalStrokes	 14
U+7A33.0	fCangjie	HDNSP
U+7A33.0	fHakka	 vun3
U+7A33.0	fCantonese	wan2
U+7A33.0	fMandarin	wen3
U+7A33.0	fEnglish	simplified form of &#x7A69;
U+7A34.0	fUTF8	稴
U+7A34.0	fCNS11643	4-4B63
U+7A34.0	fGB	3-5C41
U+7A34.0	fR/S	 115.10
U+7A34.0	fTotalStrokes	 15
U+7A34.0	fCangjie	HDTXC
U+7A34.0	fFourCorner	28930
U+7A34.0	fHakka	 liam2
U+7A34.0	fCantonese	jim4
U+7A34.0	fMandarin	lian2; xian2; jian1
U+7A34.0	fEnglish	[1] rice [2] green grain
U+7A35.0	fUTF8	稵
U+7A35.0	fCNS11643	4-455E
U+7A35.0	fGB	5-5B40
U+7A35.0	fR/S	 115.10; 115.9
U+7A35.0	fTotalStrokes	 14; 15
U+7A35.0	fCangjie	HDTVI
U+7A35.0	fFourCorner	28930
U+7A35.0	fCantonese	zi1
U+7A35.0	fMandarin	zi1; jiu1
U+7A36.0	fUTF8	稶
U+7A36.0	fCNS11643	4-4B6B
U+7A36.0	fGB	E-3634
U+7A36.0	fR/S	 115.10
U+7A36.0	fTotalStrokes	 15
U+7A36.0	fCangjie	HDIKM
U+7A36.0	fFourCorner	23950
U+7A36.0	fCantonese	juk1
U+7A36.0	fMandarin	yu4
U+7A37.0	fUTF8	稷
U+7A37.0	fCNS11643	1-6E29
U+7A37.0	fBig5	BD5E
U+7A37.0	fGB	0-7022
U+7A37.0	fR/S	 115.10
U+7A37.0	fTotalStrokes	 15
U+7A37.0	fCangjie	HDWCE
U+7A37.0	fFourCorner	26947
U+7A37.0	fHakka	  zit7
U+7A37.0	fCantonese	zik1
U+7A37.0	fMandarin	ji4; ze4
U+7A37.0	fEnglish	panicled millet
U+7A38.0	fUTF8	稸
U+7A38.0	fCNS11643	3-4D55
U+7A38.0	fGB	5-5B44
U+7A38.0	fR/S	 115.10
U+7A38.0	fTotalStrokes	 15
U+7A38.0	fCangjie	HDYVW
U+7A38.0	fFourCorner	20960
U+7A38.0	fCantonese	cuk1
U+7A38.0	fMandarin	xu4
U+7A39.0	fUTF8	稹
U+7A39.0	fCNS11643	2-525A
U+7A39.0	fBig5	E6D2
U+7A39.0	fGB	0-7021
U+7A39.0	fR/S	 115.10
U+7A39.0	fTotalStrokes	 15
U+7A39.0	fCangjie	HDJBC
U+7A39.0	fFourCorner	24986
U+7A39.0	fCantonese	zan2; can2 (also)
U+7A39.0	fMandarin	zhen3; bian1; zhen1; dian4; tian2
U+7A39.0	fEnglish	[1] circumspect [2] entwined (roots)
U+7A3A.0	fUTF8	稺
U+7A3A.0	fCNS11643	3-4D56
U+7A3A.0	fGB	E-3635
U+7A3A.0	fR/S	 115.10
U+7A3A.0	fTotalStrokes	 15
U+7A3A.0	fCangjie	HDSYJ
U+7A3A.0	fFourCorner	27940
U+7A3A.0	fHakka	 chi1; cih1; chi5; cih5; chi6; chii5
U+7A3A.0	fCantonese	zi6
U+7A3A.0	fMandarin	zhi4
U+7A3A.0	fEnglish	variant of &#x7A49;
U+7A3B.0	fUTF8	稻
U+7A3B.0	fCNS11643	1-6E2A
U+7A3B.0	fBig5	BD5F
U+7A3B.0	fGB	0-353E
U+7A3B.0	fR/S	 115.10
U+7A3B.0	fTotalStrokes	 15
U+7A3B.0	fCangjie	HDBHX
U+7A3B.0	fFourCorner	22977
U+7A3B.0	fHakka	 tau1; to5;  tau5; ziau1; to6; tau3; tau6
U+7A3B.0	fCantonese	dou6
U+7A3B.0	fMandarin	dao4; tao3
U+7A3B.0	fEnglish	paddy; rice
U+7A3C.0	fUTF8	稼
U+7A3C.0	fCNS11643	1-6E26
U+7A3C.0	fBig5	BD5B
U+7A3C.0	fGB	0-3C5A
U+7A3C.0	fR/S	 115.10
U+7A3C.0	fTotalStrokes	 15
U+7A3C.0	fCangjie	HDJMO
U+7A3C.0	fFourCorner	23932
U+7A3C.0	fHakka	  ga5
U+7A3C.0	fCantonese	gaa3
U+7A3C.0	fMandarin	jia4
U+7A3C.0	fEnglish	[1] farm; plant; sow; cultivate [2] grain; crops
U+7A3D.0	fUTF8	稽
U+7A3D.0	fCNS11643	1-6E28
U+7A3D.0	fBig5	BD5D
U+7A3D.0	fGB	0-3B7C
U+7A3D.0	fR/S	 115.10
U+7A3D.0	fTotalStrokes	 15
U+7A3D.0	fCangjie	HDIUA
U+7A3D.0	fFourCorner	23961
U+7A3D.0	fHakka	 hi1; kai1
U+7A3D.0	fCantonese	kai1; kai2
U+7A3D.0	fMandarin	ji1
U+7A3D.0	fEnglish	[1] examine, investigate; inspect; verify [2] stay; delay; procrastinate ???
U+7A3D.1	fUTF8	稽
U+7A3D.1	fCNS11643	1-6E28
U+7A3D.1	fBig5	BD5D
U+7A3D.1	fGB	0-3B7C
U+7A3D.1	fR/S	 115.10
U+7A3D.1	fTotalStrokes	 15
U+7A3D.1	fCangjie	HDIUA
U+7A3D.1	fFourCorner	23961
U+7A3D.1	fMandarin	qi3
U+7A3D.1	fEnglish	 kowtow; bow to the ground ???
U+7A3E.0	fUTF8	稾
U+7A3E.0	fCNS11643	3-4D5A
U+7A3E.0	fGB	E-3636
U+7A3E.0	fR/S	 115.10
U+7A3E.0	fTotalStrokes	 15
U+7A3E.0	fCangjie	YRBD
U+7A3E.0	fFourCorner	00900
U+7A3E.0	fHakka	 gau3;  go3
U+7A3E.0	fCantonese	gou2
U+7A3E.0	fMandarin	gao3; kao4
U+7A3E.0	fEnglish	draft; manuscript; rough copy
U+7A3F.0	fUTF8	稿
U+7A3F.0	fCNS11643	1-6E25
U+7A3F.0	fBig5	BD5A
U+7A3F.0	fGB	0-3865
U+7A3F.0	fR/S	 115.10
U+7A3F.0	fTotalStrokes	 15
U+7A3F.0	fCangjie	HDYRB
U+7A3F.0	fFourCorner	20927
U+7A3F.0	fHakka	  gau3;  go3
U+7A3F.0	fCantonese	gou2
U+7A3F.0	fMandarin	gao3; gao4; jiao4
U+7A3F.0	fEnglish	[1] draft; manuscript; rough copy; sketch [2] pattern or copy book for drawing [3] straw; stalk of grain
U+7A40.0	fUTF8	穀
U+7A40.0	fCNS11643	1-6E27
U+7A40.0	fBig5	BD5C
U+7A40.0	fGB	1-7836
U+7A40.0	fR/S	 115.10; 115.9
U+7A40.0	fTotalStrokes	 15
U+7A40.0	fCangjie	GDHNE
U+7A40.0	fFourCorner	47947
U+7A40.0	fHakka	 guk7; guk8; kwuk7
U+7A40.0	fCantonese	guk1; gau3; huk6
U+7A40.0	fMandarin	gu3; gou4; hu2; nou4; yu4
U+7A40.0	fEnglish	[1] corn; grain; cereals [2] lucky; happy; favorable; good [3] live; while alive
U+7A41.0	fUTF8	穁
U+7A41.0	fCNS11643	3-4D53
U+7A41.0	fGB	3-5C2F
U+7A41.0	fR/S	  115.10
U+7A41.0	fTotalStrokes	 15
U+7A41.0	fCangjie	HDTSJ
U+7A41.0	fFourCorner	24940
U+7A41.0	fHakka	 jung2; rung2
U+7A41.0	fCantonese	jung4
U+7A41.0	fMandarin	rong2
U+7A41.0	fEnglish	[1] the aroma of growing grain [2] fragrant
U+7A42.0	fUTF8	穂
U+7A42.0	fCNS11643	3-4D57
U+7A42.0	fGB	E-3637
U+7A42.0	fR/S	 115.10
U+7A42.0	fTotalStrokes	 15
U+7A42.0	fCangjie	HDJWP
U+7A42.0	fFourCorner	25930
U+7A42.0	fCantonese	seoi6
U+7A42.0	fMandarin	sui4
U+7A42.0	fEnglish	 ear of grain tassel Guangzhou
U+7A43.0	fUTF8	穃
U+7A43.0	fGB	E-3638
U+7A43.0	fR/S	 115.10
U+7A43.0	fTotalStrokes	 15
U+7A43.0	fCangjie	HDJCR
U+7A43.0	fCantonese	jung4
U+7A44.0	fUTF8	穄
U+7A44.0	fCNS11643	2-5879
U+7A44.0	fBig5	EAAF
U+7A44.0	fGB	3-5C44
U+7A44.0	fR/S	 115.11
U+7A44.0	fTotalStrokes	 16
U+7A44.0	fCangjie	HDBOF
U+7A44.0	fFourCorner	27991
U+7A44.0	fHakka	  zi5; zi1
U+7A44.0	fCantonese	zai3
U+7A44.0	fMandarin	ji4
U+7A44.0	fEnglish	same as $ (panicled millet)
U+7A45.0	fUTF8	穅
U+7A45.0	fCNS11643	3-522A
U+7A45.0	fGB	E-3639
U+7A45.0	fR/S	  115.11
U+7A45.0	fTotalStrokes	 16
U+7A45.0	fCangjie	HDILE
U+7A45.0	fFourCorner	20930
U+7A45.0	fHakka	  hong1;  kong1
U+7A45.0	fCantonese	hong1
U+7A45.0	fMandarin	kang1
U+7A45.0	fEnglish	chaff; bran; husks of grain; from which comes: --poor; remiss
U+7A46.0	fUTF8	穆
U+7A46.0	fCNS11643	1-715A
U+7A46.0	fBig5	BF70
U+7A46.0	fGB	0-4442
U+7A46.0	fR/S	 115.11
U+7A46.0	fTotalStrokes	 16
U+7A46.0	fCangjie	HDHAH
U+7A46.0	fFourCorner	26922
U+7A46.0	fHakka	  muk8; muk7
U+7A46.0	fCantonese	muk6
U+7A46.0	fMandarin	mu4
U+7A46.0	fEnglish	[1] majestic; solemn [2] peaceful; serene; calm [3] reverent; respectful [4] profound [5] the right side of an ancestral shrine [6] a Chinese family name
U+7A47.0	fUTF8	穇
U+7A47.0	fCNS11643	2-587B
U+7A47.0	fBig5	EAB1
U+7A47.0	fGB	3-5C2C
U+7A47.0	fR/S	 115.11
U+7A47.0	fTotalStrokes	 16
U+7A47.0	fCangjie	HDIIH
U+7A47.0	fFourCorner	23922
U+7A47.0	fHakka	 cam1;  sam1
U+7A47.0	fCantonese	saam1; caam2 (also)
U+7A47.0	fMandarin	shan1; cen1; can3
U+7A47.0	fEnglish	finger millet or Coracan
U+7A48.0	fUTF8	穈
U+7A48.0	fCNS11643	2-587A
U+7A48.0	fBig5	EAB0
U+7A48.0	fGB	3-7C22
U+7A48.0	fR/S	 115.11
U+7A48.0	fTotalStrokes	 16
U+7A48.0	fCangjie	IDHD
U+7A48.0	fFourCorner	00294
U+7A48.0	fHakka	  mun2
U+7A48.0	fCantonese	mun4; mei4
U+7A48.0	fMandarin	men2; mei2
U+7A48.0	fEnglish	large-grained millet with red seedlings
U+7A49.0	fUTF8	穉
U+7A49.0	fCNS11643	3-5564
U+7A49.0	fGB	E-363A
U+7A49.0	fR/S	 115.12; 115.11
U+7A49.0	fTotalStrokes	 17
U+7A49.0	fCangjie	HDSEQ
U+7A49.0	fFourCorner	27950
U+7A49.0	fHakka	 ti2; chi1; cih1; chi5; {chi1}; chii5
U+7A49.0	fCantonese	zi6
U+7A49.0	fMandarin	ti2; zhi4
U+7A49.0	fEnglish	 young grain young; tender
U+7A4A.0	fUTF8	穊
U+7A4A.0	fCNS11643	2-4C4C
U+7A4A.0	fBig5	E345
U+7A4A.0	fGB	3-5C36
U+7A4A.0	fR/S	 115.11; 115.9
U+7A4A.0	fTotalStrokes	 16
U+7A4A.0	fCangjie	HDAIU
U+7A4A.0	fFourCorner	21914
U+7A4A.0	fHakka	 gi5; kai3
U+7A4A.0	fCantonese	gei3; kei3
U+7A4A.0	fMandarin	ji4
U+7A4A.0	fEnglish	 plough deep for sowing
U+7A4B.0	fUTF8	穋
U+7A4B.0	fCNS11643	1-715C
U+7A4B.0	fBig5	BF72
U+7A4B.0	fGB	5-5B48
U+7A4B.0	fR/S	 115.11
U+7A4B.0	fTotalStrokes	 16
U+7A4B.0	fCangjie	HDSMH
U+7A4B.0	fFourCorner	27922
U+7A4B.0	fHakka	 liuk8
U+7A4B.0	fCantonese	luk6
U+7A4B.0	fMandarin	lu4; jiu1; qiu2
U+7A4B.0	fEnglish	rice
U+7A4C.0	fUTF8	穌
U+7A4C.0	fCNS11643	1-715B
U+7A4C.0	fBig5	BF71
U+7A4C.0	fGB	1-7655
U+7A4C.0	fR/S	 115.11
U+7A4C.0	fTotalStrokes	 16
U+7A4C.0	fCangjie	NFHD
U+7A4C.0	fFourCorner	22394
U+7A4C.0	fHakka	 su1; sii1; se1
U+7A4C.0	fCantonese	sou1
U+7A4C.0	fMandarin	su1; wei4
U+7A4C.0	fEnglish	[1] mow grass [2] revive; rise again; come to
U+7A4D.0	fUTF8	積
U+7A4D.0	fCNS11643	1-7158
U+7A4D.0	fBig5	BF6E
U+7A4D.0	fGB	1-3B7D
U+7A4D.0	fR/S	 115.11
U+7A4D.0	fTotalStrokes	 16
U+7A4D.0	fCangjie	HDQMC
U+7A4D.0	fFourCorner	25986
U+7A4D.0	fHakka	  zit7
U+7A4D.0	fCantonese	zik1
U+7A4D.0	fMandarin	ji1; zi4
U+7A4D.0	fEnglish	[1] accumulate; store up; amass [2] long (time); old; deep-rooted; longstanding [3] product (mathematics)
U+7A4E.0	fUTF8	穎
U+7A4E.0	fCNS11643	1-7159
U+7A4E.0	fBig5	BF6F
U+7A4E.0	fGB	1-5331
U+7A4E.0	fR/S	 115.11
U+7A4E.0	fTotalStrokes	 16
U+7A4E.0	fCangjie	PDMBC
U+7A4E.0	fFourCorner	21986
U+7A4E.0	fHakka	  jin3; rin3; jin1
U+7A4E.0	fCantonese	wing6
U+7A4E.0	fMandarin	ying3
U+7A4E.0	fEnglish	[1] sharp point of an awl [2] outstanding; remarkable; talented; distinguished [3] point of a writing brush [4] the ears of grain hanging down with their own weight
U+7A4F.0	fUTF8	穏
U+7A4F.0	fCNS11643	3-522C
U+7A4F.0	fGB	E-363B
U+7A4F.0	fR/S	 115.11
U+7A4F.0	fTotalStrokes	 16
U+7A4F.0	fCangjie	HDBSP
U+7A4F.0	fFourCorner	22930
U+7A4F.0	fCantonese	wan2
U+7A4F.0	fMandarin	wen3
U+7A4F.0	fEnglish	calm, quiet, peace; moderation
U+7A50.0	fUTF8	穐
U+7A50.0	fCNS11643	F-5B59
U+7A50.0	fGB	E-363C
U+7A50.0	fR/S	 115.11
U+7A50.0	fTotalStrokes	 16
U+7A50.0	fCangjie	HDNWU
U+7A50.0	fCantonese	cau1
U+7A50.0	fMandarin	qiu1
U+7A50.0	fEnglish	fall; autumn
U+7A51.0	fUTF8	穑
U+7A51.0	fGB	0-7023
U+7A51.0	fR/S	 115.11
U+7A51.0	fTotalStrokes	 16
U+7A51.0	fCangjie	HDGCW
U+7A51.0	fHakka	 sit7
U+7A51.0	fCantonese	sik1
U+7A51.0	fMandarin	se4
U+7A51.0	fEnglish	simplified form of &#x7A61;
U+7A52.0	fUTF8	穒
U+7A52.0	fGB	1-7D2C
U+7A52.0	fR/S	 115.11
U+7A52.0	fTotalStrokes	 16
U+7A52.0	fCangjie	XXHDH
U+7A52.0	fCantonese	niu5; liu5 (also)
U+7A53.0	fUTF8	穓
U+7A53.0	fCNS11643	3-5229
U+7A53.0	fGB	E-363D
U+7A53.0	fR/S	 115.12; 115.11
U+7A53.0	fTotalStrokes	 16; 17
U+7A53.0	fCangjie	HDWTC
U+7A53.0	fFourCorner	26980
U+7A53.0	fCantonese	jik6
U+7A53.0	fMandarin	yi4
U+7A54.0	fUTF8	穔
U+7A54.0	fCNS11643	2-5E22
U+7A54.0	fBig5	EDB5
U+7A54.0	fGB	5-5B46
U+7A54.0	fR/S	 115.12
U+7A54.0	fTotalStrokes	 17
U+7A54.0	fCangjie	HDTLC
U+7A54.0	fFourCorner	24986
U+7A54.0	fCantonese	wong4
U+7A54.0	fMandarin	huang2
U+7A55.0	fUTF8	穕
U+7A55.0	fCNS11643	4-5749
U+7A55.0	fGB	E-363E
U+7A55.0	fR/S	 115.12
U+7A55.0	fTotalStrokes	 17; 16
U+7A55.0	fCangjie	HDOGD
U+7A55.0	fFourCorner	20990
U+7A55.0	fCantonese	cip3
U+7A55.0	fMandarin	qie4
U+7A56.0	fUTF8	穖
U+7A56.0	fCNS11643	2-5D7E
U+7A56.0	fBig5	EDB3
U+7A56.0	fGB	5-5A75
U+7A56.0	fR/S	 115.12
U+7A56.0	fTotalStrokes	 17
U+7A56.0	fCangjie	HDVII
U+7A56.0	fFourCorner	22953
U+7A56.0	fCantonese	gei1; gei2; gei3
U+7A56.0	fMandarin	ji3; ji4
U+7A57.0	fUTF8	穗
U+7A57.0	fCNS11643	1-7454
U+7A57.0	fBig5	C14A
U+7A57.0	fGB	0-4B6B
U+7A57.0	fR/S	 115.12
U+7A57.0	fTotalStrokes	 17
U+7A57.0	fCangjie	HDJIP
U+7A57.0	fFourCorner	25933
U+7A57.0	fHakka	 sui5; fui5; fi5
U+7A57.0	fCantonese	seoi6; soei2 (also)
U+7A57.0	fMandarin	sui4
U+7A57.0	fEnglish	[1] ear of grain [2] fruits or grains in a cluster grown at the tip of a stem or stalk [3] another name for Guangzhou (Canton) [4] candle snuff; candlewick
U+7A58.0	fUTF8	穘
U+7A58.0	fCNS11643	2-5E21
U+7A58.0	fBig5	EDB4
U+7A58.0	fGB	3-5B6E
U+7A58.0	fR/S	 115.12
U+7A58.0	fTotalStrokes	 17
U+7A58.0	fCangjie	HDGGU
U+7A58.0	fFourCorner	24911
U+7A58.0	fCantonese	hiu1; haau1; jiu4
U+7A58.0	fMandarin	xiao1; rao4
U+7A59.0	fUTF8	穙
U+7A59.0	fCNS11643	4-5748
U+7A59.0	fGB	3-5C46
U+7A59.0	fR/S	  115.12
U+7A59.0	fTotalStrokes	 17; 16
U+7A59.0	fCangjie	HDTCO
U+7A59.0	fFourCorner	22930
U+7A59.0	fCantonese	buk6
U+7A59.0	fMandarin	pu2
U+7A5A.0	fUTF8	穚
U+7A5A.0	fCNS11643	2-5E23
U+7A5A.0	fBig5	EDB6
U+7A5A.0	fGB	5-5B29
U+7A5A.0	fR/S	 115.12
U+7A5A.0	fTotalStrokes	 17
U+7A5A.0	fCangjie	HDHKB
U+7A5A.0	fFourCorner	22927
U+7A5A.0	fHakka	 kiau2;  giau1; k'ieu2
U+7A5A.0	fCantonese	giu1
U+7A5A.0	fMandarin	jiao1
U+7A5B.0	fUTF8	穛
U+7A5B.0	fCNS11643	2-5D7D
U+7A5B.0	fBig5	EDB2
U+7A5B.0	fGB	3-5C48
U+7A5B.0	fR/S	 115.12
U+7A5B.0	fTotalStrokes	 17
U+7A5B.0	fCangjie	HDOGF
U+7A5B.0	fFourCorner	20931
U+7A5B.0	fHakka	 ziau5
U+7A5B.0	fCantonese	zuk1; coek3; zoek3
U+7A5B.0	fMandarin	zhuo1
U+7A5C.0	fUTF8	穜
U+7A5C.0	fCNS11643	2-5D7C
U+7A5C.0	fBig5	EDB1
U+7A5C.0	fGB	3-5C49
U+7A5C.0	fR/S	 115.12
U+7A5C.0	fTotalStrokes	 17
U+7A5C.0	fCangjie	HDYTG
U+7A5C.0	fFourCorner	20914
U+7A5C.0	fHakka	  tung2
U+7A5C.0	fCantonese	tung4; zung3; zung2
U+7A5C.0	fMandarin	tong2; zhong4; chong2
U+7A5C.0	fEnglish	rice
U+7A5D.0	fUTF8	穝
U+7A5D.0	fGB	E-363F
U+7A5D.0	fR/S	 115.12
U+7A5D.0	fTotalStrokes	 17
U+7A5D.0	fCangjie	HDASE
U+7A5D.0	fCantonese	zeoi3
U+7A5E.0	fUTF8	穞
U+7A5E.0	fGB	8-2E69
U+7A5E.0	fR/S	 115.12
U+7A5E.0	fTotalStrokes	 17
U+7A5E.0	fCangjie	XHDNW
U+7A5E.0	fCantonese	leoi5
U+7A5E.0	fMandarin	luu3; l3
U+7A5F.0	fUTF8	穟
U+7A5F.0	fCNS11643	2-624C
U+7A5F.0	fBig5	F060
U+7A5F.0	fGB	3-5C4B
U+7A5F.0	fR/S	 115.13
U+7A5F.0	fTotalStrokes	 18
U+7A5F.0	fCangjie	HDYTO
U+7A5F.0	fFourCorner	28933
U+7A5F.0	fHakka	 sui5; sui6
U+7A5F.0	fCantonese	seoi6
U+7A5F.0	fMandarin	sui4
U+7A5F.0	fEnglish	 ear of grain ripe grain
U+7A60.0	fUTF8	穠
U+7A60.0	fCNS11643	1-7674
U+7A60.0	fBig5	C2AA
U+7A60.0	fGB	3-5B79
U+7A60.0	fR/S	 115.13
U+7A60.0	fTotalStrokes	 18
U+7A60.0	fCangjie	HDTWV
U+7A60.0	fFourCorner	25932
U+7A60.0	fHakka	 ngiung2;  nung2; nung1; niung2
U+7A60.0	fCantonese	nung4; lung4 (also)
U+7A60.0	fMandarin	nong2; rong2
U+7A60.0	fEnglish	luxurious growth of plants
U+7A61.0	fUTF8	穡
U+7A61.0	fCNS11643	1-7672
U+7A61.0	fBig5	C2A8
U+7A61.0	fGB	1-7023
U+7A61.0	fR/S	 115.13
U+7A61.0	fTotalStrokes	 18
U+7A61.0	fCangjie	HDGOW
U+7A61.0	fFourCorner	24961
U+7A61.0	fHakka	 sit7; zit7
U+7A61.0	fCantonese	sik1
U+7A61.0	fMandarin	se4
U+7A61.0	fEnglish	[1] harvest grain; gather in the harvest; reap [2] same as $ (stingy; thrifty)
U+7A62.0	fUTF8	穢
U+7A62.0	fCNS11643	1-7673
U+7A62.0	fBig5	C2A9
U+7A62.0	fGB	1-3B60
U+7A62.0	fR/S	 115.13
U+7A62.0	fTotalStrokes	 18
U+7A62.0	fCangjie	HDYMH
U+7A62.0	fFourCorner	21953
U+7A62.0	fHakka	  woi5;  we5; ve5; ye5; voi5; wai5; je5
U+7A62.0	fCantonese	wai3
U+7A62.0	fMandarin	hui4; wei4
U+7A62.0	fEnglish	[1] vile; wicked [2] dirty; unclean; filthy [3] obscene; wanton (ways or conduct) [4] ugly and abominable [5] weeds on a farm [6] [7] sow; spread
U+7A63.0	fUTF8	穣
U+7A63.0	fGB	E-3640
U+7A63.0	fR/S	 115.13
U+7A63.0	fTotalStrokes	 18
U+7A63.0	fCangjie	HDYCV
U+7A63.0	fCantonese	joeng4
U+7A63.0	fMandarin	rang2
U+7A63.0	fEnglish	 stalks of grain lush; abundant
U+7A64.0	fUTF8	穤
U+7A64.0	fCNS11643	3-5A66
U+7A64.0	fGB	E-3641
U+7A64.0	fR/S	  115.14
U+7A64.0	fTotalStrokes	 19
U+7A64.0	fCangjie	HDMBB
U+7A64.0	fFourCorner	21920
U+7A64.0	fCantonese	no6; lo6 (also)
U+7A64.0	fMandarin	nuo4
U+7A65.0	fUTF8	穥
U+7A65.0	fCNS11643	4-6045
U+7A65.0	fGB	5-5A76
U+7A65.0	fR/S	 115.14
U+7A65.0	fTotalStrokes	 19
U+7A65.0	fCangjie	HDHXC
U+7A65.0	fFourCorner	27980
U+7A65.0	fHakka	 ri2; ji2; ji5; ji1
U+7A65.0	fCantonese	jyu6
U+7A65.0	fMandarin	yu4
U+7A65.0	fEnglish	luxuriant frowth of crops
U+7A66.0	fUTF8	穦
U+7A66.0	fCNS11643	3-5A65
U+7A66.0	fGB	E-3642
U+7A66.0	fR/S	  115.14
U+7A66.0	fTotalStrokes	 19
U+7A66.0	fCangjie	HDJMC
U+7A66.0	fFourCorner	23980
U+7A66.0	fCantonese	pan1
U+7A66.0	fMandarin	bin1
U+7A67.0	fUTF8	穧
U+7A67.0	fCNS11643	2-6630
U+7A67.0	fBig5	F2A6
U+7A67.0	fGB	3-5B76
U+7A67.0	fR/S	 115.14
U+7A67.0	fTotalStrokes	 19
U+7A67.0	fCangjie	HDYX
U+7A67.0	fFourCorner	20923
U+7A67.0	fHakka	  zi5; zi1
U+7A67.0	fCantonese	zai3 (also); zai6
U+7A67.0	fMandarin	ji4; zi4
U+7A67.0	fEnglish	sheaf
U+7A68.0	fUTF8	穨
U+7A68.0	fCNS11643	2-6631
U+7A68.0	fBig5	F2A7
U+7A68.0	fGB	E-3643
U+7A68.0	fR/S	 115.14
U+7A68.0	fTotalStrokes	 19
U+7A68.0	fCangjie	HULMC
U+7A68.0	fFourCorner	25286
U+7A68.0	fCantonese	teoi4
U+7A68.0	fMandarin	tui2
U+7A68.0	fEnglish	 ruined; decayed disintegrate
U+7A69.0	fUTF8	穩
U+7A69.0	fCNS11643	1-7858
U+7A69.0	fBig5	C3AD
U+7A69.0	fGB	1-4E48
U+7A69.0	fR/S	 115.14
U+7A69.0	fTotalStrokes	 19
U+7A69.0	fCangjie	HDBMP
U+7A69.0	fFourCorner	22937
U+7A69.0	fHakka	  wun3; vun3
U+7A69.0	fCantonese	wan2
U+7A69.0	fMandarin	wen3
U+7A69.0	fEnglish	[1] stable; firm; solid; steady; stability [2] sure; certain [3] secure; security
U+7A6A.0	fUTF8	穪
U+7A6A.0	fCNS11643	3-5A67
U+7A6A.0	fGB	E-3644
U+7A6A.0	fR/S	  115.14
U+7A6A.0	fTotalStrokes	 19
U+7A6A.0	fCangjie	HDMFB
U+7A6A.0	fFourCorner	21920
U+7A6A.0	fHakka	 chin1; cihn1
U+7A6A.0	fCantonese	cing1
U+7A6A.0	fMandarin	cheng1; chen4
U+7A6A.0	fEnglish	 praise
U+7A6B.0	fUTF8	穫
U+7A6B.0	fCNS11643	1-7857
U+7A6B.0	fBig5	C3AC
U+7A6B.0	fGB	1-783E
U+7A6B.0	fR/S	 115.14
U+7A6B.0	fTotalStrokes	 19
U+7A6B.0	fCangjie	HDTOE
U+7A6B.0	fFourCorner	24947
U+7A6B.0	fHakka	  wok7; fet8; fu5; fu6; vok7; bet7
U+7A6B.0	fCantonese	wok6; wu6
U+7A6B.0	fMandarin	huo4; hu4
U+7A6B.0	fEnglish	 reap; harvest; cut grain
U+7A6C.0	fUTF8	穬
U+7A6C.0	fCNS11643	2-6950
U+7A6C.0	fBig5	F4A3
U+7A6C.0	fGB	3-5B52
U+7A6C.0	fR/S	 115.15
U+7A6C.0	fTotalStrokes	 20
U+7A6C.0	fCangjie	HDITC
U+7A6C.0	fFourCorner	20986
U+7A6C.0	fHakka	 kwang3; gwang3
U+7A6C.0	fCantonese	kong3; kwong3 (also); gwong2
U+7A6C.0	fMandarin	gong3; kuang4
U+7A6C.0	fEnglish	awn of grain in the ear
U+7A6D.0	fUTF8	穭
U+7A6D.0	fCNS11643	2-6951
U+7A6D.0	fBig5	F4A4
U+7A6D.0	fGB	5-5B4B
U+7A6D.0	fR/S	 115.15
U+7A6D.0	fTotalStrokes	 20
U+7A6D.0	fCangjie	HDNWA
U+7A6D.0	fFourCorner	27963
U+7A6D.0	fCantonese	leoi5
U+7A6D.0	fMandarin	luu3; l3
U+7A6D.0	fEnglish	wild grain
U+7A6E.0	fUTF8	穮
U+7A6E.0	fCNS11643	2-694F
U+7A6E.0	fBig5	F4A2
U+7A6E.0	fGB	3-5C4D
U+7A6E.0	fR/S	 115.15
U+7A6E.0	fTotalStrokes	 20
U+7A6E.0	fCangjie	HDIPF
U+7A6E.0	fFourCorner	20931
U+7A6E.0	fHakka	 biau1;  piau1; beu1; peu1
U+7A6E.0	fCantonese	biu1
U+7A6E.0	fMandarin	biao1; pao1; pao4
U+7A6E.0	fEnglish	weed
U+7A6F.0	fUTF8	穯
U+7A6F.0	fCNS11643	3-5C64
U+7A6F.0	fGB	E-3645
U+7A6F.0	fR/S	  115.15
U+7A6F.0	fTotalStrokes	 20
U+7A6F.0	fCangjie	HDGRW
U+7A6F.0	fFourCorner	24960
U+7A6F.0	fCantonese	sik1
U+7A70.0	fUTF8	穰
U+7A70.0	fCNS11643	2-6D66
U+7A70.0	fBig5	F6F8
U+7A70.0	fGB	0-7026
U+7A70.0	fR/S	 115.17
U+7A70.0	fTotalStrokes	 22
U+7A70.0	fCangjie	HDYRV
U+7A70.0	fFourCorner	20932
U+7A70.0	fHakka	 jong2; rong2; ngiong2; ngiong5
U+7A70.0	fCantonese	joeng4; joeng6; joeng5
U+7A70.0	fMandarin	rang2; rang3; reng2
U+7A70.0	fEnglish	[1] crowded [2] confusing; mixed up; disturbed (mind, etc.) [3] stalks of grain; stalks of cereal grasses [4] lush; luxuriant
U+7A71.0	fUTF8	穱
U+7A71.0	fCNS11643	2-6D67
U+7A71.0	fBig5	F6F9
U+7A71.0	fGB	3-5C51
U+7A71.0	fR/S	 115.18
U+7A71.0	fTotalStrokes	 23
U+7A71.0	fCangjie	HDBWI
U+7A71.0	fFourCorner	22946
U+7A71.0	fHakka	 ziok7
U+7A71.0	fCantonese	zoek3; zuk1
U+7A71.0	fMandarin	zhuo1; jue2
U+7A71.0	fEnglish	 sow wheat between the rice-crops
U+7A72.0	fUTF8	穲
U+7A72.0	fCNS11643	3-6076
U+7A72.0	fGB	E-3646
U+7A72.0	fR/S	  115.19
U+7A72.0	fTotalStrokes	 24
U+7A72.0	fCangjie	HDMMP
U+7A72.0	fFourCorner	21910
U+7A72.0	fHakka	 li2
U+7A72.0	fCantonese	lei4
U+7A72.0	fMandarin	li2
U+7A72.0	fEnglish	grain growing in rows
U+7A73.0	fUTF8	穳
U+7A73.0	fCNS11643	3-6077
U+7A73.0	fGB	3-5C4F
U+7A73.0	fR/S	  115.17
U+7A73.0	fTotalStrokes	 24; 22
U+7A73.0	fCangjie	HDHUC
U+7A73.0	fFourCorner	24980
U+7A73.0	fHakka	  zon1
U+7A73.0	fCantonese	cyun4
U+7A73.0	fMandarin	zan4; cuan2; zuan3
U+7A74.0	fUTF8	穴
U+7A74.0	fCNS11643	1-467E
U+7A74.0	fBig5	A5DE
U+7A74.0	fGB	0-5128
U+7A74.0	fR/S	 116.0
U+7A74.0	fTotalStrokes	 5
U+7A74.0	fCangjie	JC
U+7A74.0	fFourCorner	30802
U+7A74.0	fHakka	  hiet8;  het8; hiat8
U+7A74.0	fCantonese	jyut6
U+7A74.0	fMandarin	xue2; jue2; xue4
U+7A74.0	fEnglish	[1] cave; den; hole [2] points in the human body [3] points in the human body where nerve centers are supposed to be located, a strike at which might cause paralysis or even death [4] Kangxi radical 116
U+7A75.0	fUTF8	穵
U+7A75.0	fCNS11643	2-2268
U+7A75.0	fBig5	CA48
U+7A75.0	fGB	5-5E30
U+7A75.0	fR/S	 116.1
U+7A75.0	fTotalStrokes	 6
U+7A75.0	fCangjie	JCN
U+7A75.0	fFourCorner	30717
U+7A75.0	fHakka	 wat7; wet7; vat7; wa1
U+7A75.0	fCantonese	waat3; waa1 (also)
U+7A75.0	fMandarin	wa1; ya4
U+7A75.0	fEnglish	 deep hollow gouge; dig out
U+7A76.0	fUTF8	究
U+7A76.0	fCNS11643	1-4B36
U+7A76.0	fBig5	A873
U+7A76.0	fGB	0-3E3F
U+7A76.0	fR/S	 116.2
U+7A76.0	fTotalStrokes	 7
U+7A76.0	fCangjie	JCKN
U+7A76.0	fFourCorner	30417
U+7A76.0	fHakka	  giu5; kieu5; kiu6; kiu5
U+7A76.0	fCantonese	gau3; gau2 (also)
U+7A76.0	fMandarin	jiu1; jiu4; obs
U+7A76.0	fEnglish	[1] examine; investigate exhaustively; study; dig into [2] finally; in the end; after all [3] actually; really
U+7A76.1	fUTF8	究
U+7A76.1	fCNS11643	1-4B36
U+7A76.1	fBig5	A873
U+7A76.1	fGB	0-3E3F
U+7A76.1	fR/S	 116.2
U+7A76.1	fTotalStrokes	 7
U+7A76.1	fCangjie	JCKN
U+7A76.1	fFourCorner	30417
U+7A76.1	fMandarin	jiu5
U+7A77.0	fUTF8	穷
U+7A77.0	fGB	0-476E
U+7A77.0	fR/S	 116.2
U+7A77.0	fTotalStrokes	 7
U+7A77.0	fCangjie	JCKS
U+7A77.0	fFourCorner	3042
U+7A77.0	fHakka	 kiung2
U+7A77.0	fCantonese	kung4
U+7A77.0	fMandarin	qiong2; gong1
U+7A77.0	fEnglish	simplified form of &#x7AAE;
U+7A78.0	fUTF8	穸
U+7A78.0	fCNS11643	2-2845
U+7A78.0	fBig5	CDA5
U+7A78.0	fGB	0-7136
U+7A78.0	fR/S	 116.3
U+7A78.0	fTotalStrokes	 8
U+7A78.0	fCangjie	JCNI
U+7A78.0	fFourCorner	30207
U+7A78.0	fHakka	  sip8
U+7A78.0	fCantonese	zik6
U+7A78.0	fMandarin	xi4; xi1
U+7A78.0	fEnglish	[1] tomb; grave [2] the dead of night
U+7A79.0	fUTF8	穹
U+7A79.0	fCNS11643	1-4F29
U+7A79.0	fBig5	AAC6
U+7A79.0	fGB	0-7137
U+7A79.0	fR/S	 116.3
U+7A79.0	fTotalStrokes	 8
U+7A79.0	fCangjie	JCN
U+7A79.0	fFourCorner	30207
U+7A79.0	fHakka	 giung1
U+7A79.0	fCantonese	kung4; hung1; gung1
U+7A79.0	fMandarin	qiong2; qiong1; kong1
U+7A79.0	fEnglish	[1] high and vast [2] vaulted; arched [3] deep and spacious [4] the sky
U+7A7A.0	fUTF8	空
U+7A7A.0	fCNS11643	1-4F28
U+7A7A.0	fBig5	AAC5
U+7A7A.0	fGB	0-3F55
U+7A7A.0	fR/S	 116.3
U+7A7A.0	fTotalStrokes	 8
U+7A7A.0	fCangjie	JCM
U+7A7A.0	fFourCorner	30101
U+7A7A.0	fHakka	  kung1;  kung5; fung1
U+7A7A.0	fCantonese	hung1; hung3
U+7A7A.0	fMandarin	kong1; kong3
U+7A7A.0	fEnglish	[1] empty; hollow; void [2] empty; exhaust; reduce to extremity [3] fictitious; unreal; impractical [4] vain and useless (efforts, etc.); ineffective; fruitless
U+7A7A.1	fUTF8	空
U+7A7A.1	fCNS11643	1-4F28
U+7A7A.1	fBig5	AAC5
U+7A7A.1	fGB	0-3F55
U+7A7A.1	fR/S	 116.3
U+7A7A.1	fTotalStrokes	 8
U+7A7A.1	fCangjie	JCM
U+7A7A.1	fFourCorner	30101
U+7A7A.1	fMandarin	kong4
U+7A7A.1	fEnglish	[1] leisure; free time; spare time [2] blank (space); vacant; vacancy; leave blank or vacant
U+7A7B.0	fUTF8	穻
U+7A7B.0	fCNS11643	2-2846
U+7A7B.0	fBig5	CDA6
U+7A7B.0	fGB	5-5E32
U+7A7B.0	fR/S	 116.3
U+7A7B.0	fTotalStrokes	 8
U+7A7B.0	fCangjie	JCMD
U+7A7B.0	fFourCorner	30401
U+7A7B.0	fCantonese	jyu1; jyu5
U+7A7B.0	fMandarin	yu1
U+7A7C.0	fUTF8	穼
U+7A7C.0	fCNS11643	3-2E44
U+7A7C.0	fGB	3-5F55
U+7A7C.0	fR/S	  116.4
U+7A7C.0	fTotalStrokes	 9
U+7A7C.0	fCangjie	JCD
U+7A7C.0	fFourCorner	30900
U+7A7C.0	fHakka	 sem1
U+7A7C.0	fCantonese	sam1
U+7A7C.0	fMandarin	sen1
U+7A7D.0	fUTF8	穽
U+7A7D.0	fCNS11643	3-2E47
U+7A7D.0	fGB	E-3647
U+7A7D.0	fR/S	 116.4
U+7A7D.0	fTotalStrokes	 9
U+7A7D.0	fCangjie	JCTT
U+7A7D.0	fFourCorner	30550
U+7A7D.0	fHakka	 ziang3
U+7A7D.0	fCantonese	zing6; zeng2 (also); zeng6 (also)
U+7A7D.0	fMandarin	jing3; jing4
U+7A7D.0	fEnglish	 hole pitfall; trap
U+7A7E.0	fUTF8	穾
U+7A7E.0	fCNS11643	2-2C61
U+7A7E.0	fBig5	D040
U+7A7E.0	fGB	5-5E38
U+7A7E.0	fR/S	 116.4
U+7A7E.0	fTotalStrokes	 9
U+7A7E.0	fCangjie	JCHK
U+7A7E.0	fFourCorner	30432
U+7A7E.0	fCantonese	jiu2; jiu3; ngaau5
U+7A7E.0	fMandarin	yao4; yao3; yao1
U+7A7F.0	fUTF8	穿
U+7A7F.0	fCNS11643	1-5272
U+7A7F.0	fBig5	ACEF
U+7A7F.0	fGB	0-3429
U+7A7F.0	fR/S	 116.4
U+7A7F.0	fTotalStrokes	 9
U+7A7F.0	fCangjie	JCMVH
U+7A7F.0	fFourCorner	30241
U+7A7F.0	fHakka	  con1; chon1; chon5
U+7A7F.0	fCantonese	cyun1
U+7A7F.0	fMandarin	chuan1; chuan4; yuan1
U+7A7F.0	fEnglish	[1] wear (clothes, shoes, etc.) [2] penetrate; pierce through; bore through; thread [3] cross (street, etc.) [4] see through; bare (secret, etc.)
U+7A80.0	fUTF8	窀
U+7A80.0	fCNS11643	2-2C60
U+7A80.0	fBig5	CFFE
U+7A80.0	fGB	0-7138
U+7A80.0	fR/S	 116.4
U+7A80.0	fTotalStrokes	 9
U+7A80.0	fCangjie	JCPU
U+7A80.0	fFourCorner	30717
U+7A80.0	fHakka	  tun2; dun3
U+7A80.0	fCantonese	zeon1
U+7A80.0	fMandarin	zhun1; tun2; chuan2
U+7A80.0	fEnglish	 pit for the coffin
U+7A81.0	fUTF8	突
U+7A81.0	fCNS11643	1-5273
U+7A81.0	fBig5	ACF0
U+7A81.0	fGB	0-4D3B
U+7A81.0	fR/S	 116.4; 167.3
U+7A81.0	fTotalStrokes	 9
U+7A81.0	fCangjie	JCIK
U+7A81.0	fFourCorner	30430
U+7A81.0	fHakka	 tut7; tut8
U+7A81.0	fCantonese	dat6; duk1
U+7A81.0	fMandarin	tu1; tuo1; tu2; die2; du1
U+7A81.0	fEnglish	[1] suddenly; abruptly; unexpectedly; unexpected; sudden [2] offend; go against [3] break through (enemy encirclement) [4] project; jut out [5] chimney
U+7A82.0	fUTF8	窂
U+7A82.0	fCNS11643	3-2E49
U+7A82.0	fGB	5-5E37
U+7A82.0	fR/S	  116.4
U+7A82.0	fTotalStrokes	 9
U+7A82.0	fCangjie	JCHQ
U+7A82.0	fFourCorner	30500
U+7A82.0	fCantonese	lou4
U+7A82.0	fMandarin	lao2
U+7A83.0	fUTF8	窃
U+7A83.0	fCNS11643	3-2E48
U+7A83.0	fGB	0-4754
U+7A83.0	fR/S	 116.4
U+7A83.0	fTotalStrokes	 9
U+7A83.0	fCangjie	JCPSH
U+7A83.0	fFourCorner	30720
U+7A83.0	fHakka	  ciap7; ciap8
U+7A83.0	fCantonese	sit3
U+7A83.0	fMandarin	qie4
U+7A83.0	fEnglish	simplified form of &#x7ACA;
U+7A84.0	fUTF8	窄
U+7A84.0	fCNS11643	1-5739
U+7A84.0	fBig5	AFB6
U+7A84.0	fGB	0-552D
U+7A84.0	fR/S	 116.5
U+7A84.0	fTotalStrokes	 10
U+7A84.0	fCangjie	JCOS
U+7A84.0	fFourCorner	30211
U+7A84.0	fHakka	  zak7; zat7; zet7
U+7A84.0	fCantonese	zaak3
U+7A84.0	fMandarin	ze2; zhai3
U+7A84.0	fEnglish	[1] narrow; tight; contracted [2] narrow-minded; mean
U+7A85.0	fUTF8	窅
U+7A85.0	fCNS11643	2-315B
U+7A85.0	fBig5	D2F8
U+7A85.0	fGB	3-5F58
U+7A85.0	fR/S	 116.5
U+7A85.0	fTotalStrokes	 10
U+7A85.0	fCangjie	JCBU
U+7A85.0	fFourCorner	30608
U+7A85.0	fHakka	 miau3; miau1; meu3; jau3
U+7A85.0	fCantonese	jiu2
U+7A85.0	fMandarin	yao3; ao3; yao4; yao1; mian2
U+7A85.0	fEnglish	[1] far; deep; mysterious [2] sad
U+7A86.0	fUTF8	窆
U+7A86.0	fCNS11643	2-3159
U+7A86.0	fBig5	D2F6
U+7A86.0	fGB	0-7139
U+7A86.0	fR/S	 116.5; 116.4
U+7A86.0	fTotalStrokes	 10
U+7A86.0	fCangjie	JCHIO
U+7A86.0	fFourCorner	30307
U+7A86.0	fHakka	  bien3; ben3; bien5; pien1; bien2
U+7A86.0	fCantonese	bin2
U+7A86.0	fMandarin	bian3; bian4; beng4
U+7A86.0	fEnglish	 put the coffin in the grave; bury
U+7A87.0	fUTF8	窇
U+7A87.0	fCNS11643	2-315F
U+7A87.0	fBig5	D2FC
U+7A87.0	fGB	5-5E41
U+7A87.0	fR/S	 116.5
U+7A87.0	fTotalStrokes	 10
U+7A87.0	fCangjie	JCPRU
U+7A87.0	fFourCorner	30712
U+7A87.0	fHakka	 pok7
U+7A87.0	fCantonese	bok6
U+7A87.0	fMandarin	bao2; bo2
U+7A88.0	fUTF8	窈
U+7A88.0	fCNS11643	1-573A
U+7A88.0	fBig5	AFB7
U+7A88.0	fGB	0-713A
U+7A88.0	fR/S	 116.5
U+7A88.0	fTotalStrokes	 10
U+7A88.0	fCangjie	JCVIS
U+7A88.0	fFourCorner	30727
U+7A88.0	fHakka	 miau3; miau1; meu3; jau1; jau3
U+7A88.0	fCantonese	jiu2 (also); miu5
U+7A88.0	fMandarin	yao3; yao4
U+7A88.0	fEnglish	[1] deep; obscure; secluded [2] tranquil
U+7A89.0	fUTF8	窉
U+7A89.0	fCNS11643	2-315A
U+7A89.0	fBig5	D2F7
U+7A89.0	fGB	5-5E3B
U+7A89.0	fR/S	 116.5
U+7A89.0	fTotalStrokes	 10
U+7A89.0	fCangjie	JCMOB
U+7A89.0	fFourCorner	30227
U+7A89.0	fHakka	 biang3
U+7A89.0	fCantonese	bing2; beng3
U+7A89.0	fMandarin	bing3; bing4; xing4
U+7A89.0	fEnglish	to hide things, to conceal
U+7A8A.0	fUTF8	窊
U+7A8A.0	fCNS11643	2-315E
U+7A8A.0	fBig5	D2FB
U+7A8A.0	fGB	5-5E40
U+7A8A.0	fR/S	 116.5
U+7A8A.0	fTotalStrokes	 10
U+7A8A.0	fCangjie	JCHVO
U+7A8A.0	fFourCorner	30232
U+7A8A.0	fHakka	 wa1; va1
U+7A8A.0	fCantonese	waa1
U+7A8A.0	fMandarin	wa1; wa4
U+7A8A.0	fEnglish	 pit; vault store in pit
U+7A8B.0	fUTF8	窋
U+7A8B.0	fCNS11643	2-315C
U+7A8B.0	fBig5	D2F9
U+7A8B.0	fGB	5-5E43
U+7A8B.0	fR/S	 116.5
U+7A8B.0	fTotalStrokes	 10
U+7A8B.0	fCangjie	JCUU
U+7A8B.0	fFourCorner	30777
U+7A8B.0	fHakka	 kut7; chot7; zot7
U+7A8B.0	fCantonese	zyut3; fat1; gwat6
U+7A8B.0	fMandarin	zhu2; ku1
U+7A8B.0	fEnglish	 peep out of a hole; hole
U+7A8C.0	fUTF8	窌
U+7A8C.0	fCNS11643	2-315D
U+7A8C.0	fBig5	D2FA
U+7A8C.0	fGB	3-5F59
U+7A8C.0	fR/S	 116.5
U+7A8C.0	fTotalStrokes	 10
U+7A8C.0	fCangjie	JCHHL
U+7A8C.0	fFourCorner	30227
U+7A8C.0	fHakka	 gau5
U+7A8C.0	fCantonese	gaau3; paau3
U+7A8C.0	fMandarin	jiao4; pao4; liao2; liu4
U+7A8C.0	fEnglish	 pit; vault store in pit
U+7A8D.0	fUTF8	窍
U+7A8D.0	fGB	0-474F
U+7A8D.0	fR/S	 116.5
U+7A8D.0	fTotalStrokes	 10
U+7A8D.0	fCangjie	JCMMS
U+7A8D.0	fHakka	 kiau5
U+7A8D.0	fCantonese	kiu3; hiu3 (also); kiu2 (also)
U+7A8D.0	fMandarin	qiao4
U+7A8D.0	fEnglish	simplified form of &#x7AC5;
U+7A8E.0	fUTF8	窎
U+7A8E.0	fGB	8-7D71
U+7A8E.0	fR/S	 116.5
U+7A8E.0	fTotalStrokes	 10
U+7A8E.0	fCangjie	JCPYM
U+7A8E.0	fCantonese	diu3
U+7A8E.0	fMandarin	diao4
U+7A8E.0	fEnglish	deep; distant
U+7A8F.0	fUTF8	窏
U+7A8F.0	fCNS11643	2-376B
U+7A8F.0	fBig5	D6C8
U+7A8F.0	fGB	3-5F5D
U+7A8F.0	fR/S	 116.6
U+7A8F.0	fTotalStrokes	 11
U+7A8F.0	fCangjie	JCEMD
U+7A8F.0	fFourCorner	30141
U+7A8F.0	fCantonese	wu1
U+7A8F.0	fMandarin	wu1
U+7A90.0	fUTF8	窐
U+7A90.0	fCNS11643	2-376D
U+7A90.0	fBig5	D6CA
U+7A90.0	fGB	5-5E45
U+7A90.0	fR/S	 116.6
U+7A90.0	fTotalStrokes	 11
U+7A90.0	fCangjie	JCGG
U+7A90.0	fFourCorner	30104
U+7A90.0	fCantonese	waa1
U+7A90.0	fMandarin	gui1; wa1; xi1; ya1; ao1; wei1
U+7A91.0	fUTF8	窑
U+7A91.0	fCNS11643	3-3772
U+7A91.0	fGB	0-5224
U+7A91.0	fR/S	 116.6
U+7A91.0	fTotalStrokes	 11
U+7A91.0	fCangjie	JCOJU
U+7A91.0	fFourCorner	30770
U+7A91.0	fHakka	 jau2
U+7A91.0	fCantonese	jiu4
U+7A91.0	fMandarin	yao2
U+7A91.0	fEnglish	 kiln coal mine pit
U+7A92.0	fUTF8	窒
U+7A92.0	fCNS11643	1-5C44
U+7A92.0	fBig5	B2BF
U+7A92.0	fGB	0-564F
U+7A92.0	fR/S	 116.6
U+7A92.0	fTotalStrokes	 11
U+7A92.0	fCangjie	JCMIG
U+7A92.0	fFourCorner	30104
U+7A92.0	fHakka	 zhit7; tsiit7; zit7
U+7A92.0	fCantonese	zat6
U+7A92.0	fMandarin	zhi4; die2; nie4
U+7A92.0	fEnglish	 block; stop up; obstruct; stuff up
U+7A93.0	fUTF8	窓
U+7A93.0	fCNS11643	3-3771
U+7A93.0	fGB	E-3648
U+7A93.0	fR/S	  116.6
U+7A93.0	fTotalStrokes	 11
U+7A93.0	fCangjie	JCIP
U+7A93.0	fFourCorner	30330
U+7A93.0	fHakka	  cung1
U+7A93.0	fCantonese	coeng1
U+7A93.0	fMandarin	chuang1; chuan1; cong1
U+7A93.0	fEnglish	window
U+7A94.0	fUTF8	窔
U+7A94.0	fCNS11643	2-376C
U+7A94.0	fBig5	D6C9
U+7A94.0	fGB	5-5E4A
U+7A94.0	fR/S	 116.6
U+7A94.0	fTotalStrokes	 11
U+7A94.0	fCangjie	JCYCK
U+7A94.0	fFourCorner	30408
U+7A94.0	fCantonese	jiu3 (also); jiu2; ngau5
U+7A94.0	fMandarin	yao4; yao3
U+7A94.0	fEnglish	corner
U+7A95.0	fUTF8	窕
U+7A95.0	fCNS11643	1-5C45
U+7A95.0	fBig5	B2C0
U+7A95.0	fGB	0-713B
U+7A95.0	fR/S	 116.6
U+7A95.0	fTotalStrokes	 11
U+7A95.0	fCangjie	JCLMO
U+7A95.0	fFourCorner	30113
U+7A95.0	fHakka	 tiau2; tiau5; tiau1
U+7A95.0	fCantonese	tiu5; jiu4; diu6
U+7A95.0	fMandarin	tiao3; tiao1; yao2; tiao4
U+7A95.0	fEnglish	[1] slender [2] quiet and modest; charming and attractive [3] beautiful; wonderful; good
U+7A96.0	fUTF8	窖
U+7A96.0	fCNS11643	1-6128
U+7A96.0	fBig5	B5A2
U+7A96.0	fGB	0-3D51
U+7A96.0	fR/S	 116.7
U+7A96.0	fTotalStrokes	 12
U+7A96.0	fCangjie	JCHGR
U+7A96.0	fFourCorner	30601
U+7A96.0	fHakka	  gau5
U+7A96.0	fCantonese	gaau3
U+7A96.0	fMandarin	jiao4; zao4
U+7A96.0	fEnglish	[1] pit; cellar; vault [2] store things in a cellar or pit
U+7A97.0	fUTF8	窗
U+7A97.0	fCNS11643	1-6127
U+7A97.0	fBig5	B5A1
U+7A97.0	fGB	0-3430
U+7A97.0	fR/S	 116.7
U+7A97.0	fTotalStrokes	 12
U+7A97.0	fCangjie	JCHWK
U+7A97.0	fFourCorner	30608
U+7A97.0	fHakka	  cung1; cong1
U+7A97.0	fCantonese	coeng1
U+7A97.0	fMandarin	chuang1; chuan1; cong1; hui4
U+7A97.0	fEnglish	[1] window; skylight [2] a place where one studies
U+7A98.0	fUTF8	窘
U+7A98.0	fCNS11643	1-6126
U+7A98.0	fBig5	B57E
U+7A98.0	fGB	0-3E3D
U+7A98.0	fR/S	 116.7
U+7A98.0	fTotalStrokes	 12
U+7A98.0	fCangjie	JCSKR
U+7A98.0	fFourCorner	30607
U+7A98.0	fHakka	  kiun3; kiun2; kun2; kiung2
U+7A98.0	fCantonese	kwan3; kwan5
U+7A98.0	fMandarin	jiong3; jun3; gui4; jun4
U+7A98.0	fEnglish	[1] embarrass [2] hard-pressed; poverty-stricken [3] afflicted; distressed
U+7A99.0	fUTF8	窙
U+7A99.0	fCNS11643	2-3E5F
U+7A99.0	fBig5	DADB
U+7A99.0	fGB	3-5F5E
U+7A99.0	fR/S	 116.7
U+7A99.0	fTotalStrokes	 12
U+7A99.0	fCangjie	JCJKD
U+7A99.0	fFourCorner	30447
U+7A99.0	fCantonese	haau1
U+7A99.0	fMandarin	xiao1
U+7A9A.0	fUTF8	窚
U+7A9A.0	fCNS11643	3-3770
U+7A9A.0	fGB	5-5E46
U+7A9A.0	fR/S	 116.7; 116.6
U+7A9A.0	fTotalStrokes	 11; 12
U+7A9A.0	fCangjie	JCIHS
U+7A9A.0	fFourCorner	30250
U+7A9A.0	fCantonese	sing4
U+7A9A.0	fMandarin	cheng2
U+7A9B.0	fUTF8	窛
U+7A9B.0	fCNS11643	4-3965
U+7A9B.0	fGB	E-3649
U+7A9B.0	fR/S	  116.7
U+7A9B.0	fTotalStrokes	 12
U+7A9B.0	fCangjie	JCMUV
U+7A9B.0	fFourCorner	30210
U+7A9B.0	fCantonese	kau3
U+7A9B.0	fMandarin	kou4
U+7A9C.0	fUTF8	窜
U+7A9C.0	fGB	0-345C
U+7A9C.0	fR/S	 116.7
U+7A9C.0	fTotalStrokes	 12
U+7A9C.0	fCangjie	JCLL
U+7A9C.0	fHakka	 con5
U+7A9C.0	fCantonese	cyun2; cyun3 (also)
U+7A9C.0	fMandarin	cuan4; cuan1; cui4
U+7A9C.0	fEnglish	simplified form of &#x7AC4;
U+7A9D.0	fUTF8	窝
U+7A9D.0	fGB	0-4E51
U+7A9D.0	fR/S	 116.7
U+7A9D.0	fTotalStrokes	 12
U+7A9D.0	fCangjie	JCROB
U+7A9D.0	fHakka	 vo1
U+7A9D.0	fCantonese	wo1
U+7A9D.0	fMandarin	wo1
U+7A9D.0	fEnglish	simplified form of &#x7AA9;
U+7A9E.0	fUTF8	窞
U+7A9E.0	fCNS11643	2-4569
U+7A9E.0	fBig5	DF44
U+7A9E.0	fGB	3-5F62
U+7A9E.0	fR/S	 116.8
U+7A9E.0	fTotalStrokes	 13
U+7A9E.0	fCangjie	JCNHX
U+7A9E.0	fFourCorner	30777
U+7A9E.0	fHakka	  tam3
U+7A9E.0	fCantonese	daam6; ham2; tam5
U+7A9E.0	fMandarin	dan4; lan3
U+7A9E.0	fEnglish	pit
U+7A9F.0	fUTF8	窟
U+7A9F.0	fCNS11643	1-6564
U+7A9F.0	fBig5	B85D
U+7A9F.0	fGB	0-3F5F
U+7A9F.0	fR/S	 116.8
U+7A9F.0	fTotalStrokes	 13
U+7A9F.0	fCangjie	JCSUU
U+7A9F.0	fFourCorner	30272
U+7A9F.0	fHakka	  fut7;  kut7; kuk7; kwut7
U+7A9F.0	fCantonese	fat1; gwat6 (also)
U+7A9F.0	fMandarin	ku1
U+7A9F.0	fEnglish	[1] hole; cave; pit [2] dig the ground and build underground living quarters [3] a den for wild animals; gambling den; opium-smoking den
U+7AA0.0	fUTF8	窠
U+7AA0.0	fCNS11643	1-6565
U+7AA0.0	fBig5	B85E
U+7AA0.0	fGB	0-713D
U+7AA0.0	fR/S	 116.8
U+7AA0.0	fTotalStrokes	 13
U+7AA0.0	fCangjie	JCWD
U+7AA0.0	fFourCorner	30904
U+7AA0.0	fHakka	 ko1
U+7AA0.0	fCantonese	fo1 (also); wo1
U+7AA0.0	fMandarin	ke1
U+7AA0.0	fEnglish	[1] nest [2] hole [3] den; burrow [4] dwelling for people
U+7AA1.0	fUTF8	窡
U+7AA1.0	fCNS11643	3-4346
U+7AA1.0	fGB	3-5F64
U+7AA1.0	fR/S	 116.8
U+7AA1.0	fTotalStrokes	 13
U+7AA1.0	fCangjie	JCEEE
U+7AA1.0	fFourCorner	30440
U+7AA1.0	fHakka	 tot7; dot7
U+7AA1.0	fCantonese	zeot1
U+7AA1.0	fMandarin	zhui4
U+7AA1.0	fEnglish	looking out froma hole
U+7AA2.0	fUTF8	窢
U+7AA2.0	fCNS11643	2-4568
U+7AA2.0	fBig5	DF43
U+7AA2.0	fGB	3-5F60
U+7AA2.0	fR/S	 116.8
U+7AA2.0	fTotalStrokes	 13
U+7AA2.0	fCangjie	JCIRM
U+7AA2.0	fFourCorner	30153
U+7AA2.0	fCantonese	wik6; wik1
U+7AA2.0	fMandarin	xu4
U+7AA3.0	fUTF8	窣
U+7AA3.0	fCNS11643	2-4567
U+7AA3.0	fBig5	DF42
U+7AA3.0	fGB	3-5F63
U+7AA3.0	fR/S	 116.8
U+7AA3.0	fTotalStrokes	 13
U+7AA3.0	fCangjie	JCYOJ
U+7AA3.0	fFourCorner	30408
U+7AA3.0	fHakka	 cut7; cut8
U+7AA3.0	fCantonese	seot1
U+7AA3.0	fMandarin	su1; su4; cu4
U+7AA3.0	fEnglish	 rush out of a den rustling; wispering
U+7AA4.0	fUTF8	窤
U+7AA4.0	fCNS11643	3-4347
U+7AA4.0	fGB	E-364A
U+7AA4.0	fR/S	  116.8
U+7AA4.0	fTotalStrokes	 13
U+7AA4.0	fCangjie	JCAPP
U+7AA4.0	fFourCorner	30710
U+7AA4.0	fCantonese	gun1; gwun1 (also)
U+7AA4.0	fMandarin	guan1
U+7AA5.0	fUTF8	窥
U+7AA5.0	fGB	0-3F7A
U+7AA5.0	fR/S	 116.8
U+7AA5.0	fTotalStrokes	 13
U+7AA5.0	fCangjie	XJCQO
U+7AA5.0	fHakka	 kwui1
U+7AA5.0	fCantonese	kwai1
U+7AA5.0	fMandarin	kui1; kui3
U+7AA5.0	fEnglish	simplified form of &#x7ABA;
U+7AA6.0	fUTF8	窦
U+7AA6.0	fGB	0-713C
U+7AA6.0	fR/S	 116.8
U+7AA6.0	fTotalStrokes	 13
U+7AA6.0	fCangjie	JCJNK
U+7AA6.0	fHakka	 teu5
U+7AA6.0	fCantonese	dau6; dau2 (also); dau3; duk6
U+7AA6.0	fMandarin	dou4; du2
U+7AA6.0	fEnglish	simplified form of &#x7AC7;
U+7AA7.0	fUTF8	窧
U+7AA7.0	fCNS11643	3-657C
U+7AA7.0	fGB	E-364B
U+7AA7.0	fR/S	 116.8
U+7AA7.0	fTotalStrokes	 13
U+7AA7.0	fCangjie	JCYAJ
U+7AA7.0	fFourCorner	3040
U+7AA7.0	fCantonese	coek3
U+7AA8.0	fUTF8	窨
U+7AA8.0	fCNS11643	2-4C51
U+7AA8.0	fBig5	E34A
U+7AA8.0	fGB	0-713F
U+7AA8.0	fR/S	 116.9
U+7AA8.0	fTotalStrokes	 14
U+7AA8.0	fCangjie	JCYTA
U+7AA8.0	fFourCorner	30601
U+7AA8.0	fHakka	 jim5; rim5
U+7AA8.0	fCantonese	jam3; fan1
U+7AA8.0	fMandarin	yin4; yin1
U+7AA8.0	fEnglish	 a cellar; underground storeroom
U+7AA8.1	fUTF8	窨
U+7AA8.1	fCNS11643	2-4C51
U+7AA8.1	fBig5	E34A
U+7AA8.1	fGB	0-713F
U+7AA8.1	fR/S	 116.9
U+7AA8.1	fTotalStrokes	 14
U+7AA8.1	fCangjie	JCYTA
U+7AA8.1	fFourCorner	30601
U+7AA8.1	fMandarin	xun1
U+7AA8.1	fEnglish	 add aroma to tea by mixing it with jasmine
U+7AA9.0	fUTF8	窩
U+7AA9.0	fCNS11643	1-6A24
U+7AA9.0	fBig5	BADB
U+7AA9.0	fGB	1-4E51
U+7AA9.0	fR/S	 116.9
U+7AA9.0	fTotalStrokes	 14
U+7AA9.0	fCangjie	JCBBR
U+7AA9.0	fFourCorner	30227
U+7AA9.0	fHakka	  wo1; vo1
U+7AA9.0	fCantonese	wo1
U+7AA9.0	fMandarin	wo1
U+7AA9.0	fEnglish	[1] nest; cave; den [2] apartment; living quarters; house [3] hide; harbour (a criminal, etc.) [4] bend; crease [5] litter; brood [6] block; hold in; check [7] hollow part of the human body; pit
U+7AAA.0	fUTF8	窪
U+7AAA.0	fCNS11643	1-6A23
U+7AAA.0	fBig5	BADA
U+7AAA.0	fGB	1-4D5D
U+7AAA.0	fR/S	 116.9
U+7AAA.0	fTotalStrokes	 14
U+7AAA.0	fCangjie	JCEGG
U+7AAA.0	fFourCorner	30114
U+7AAA.0	fHakka	 wa1; wo1
U+7AAA.0	fCantonese	waa1; gwai1
U+7AAA.0	fMandarin	wa1; ke3
U+7AAA.0	fEnglish	[1] deep; hollow; low-lying [2] pit; depression; hollow; hole
U+7AAB.0	fUTF8	窫
U+7AAB.0	fCNS11643	2-4C52
U+7AAB.0	fBig5	E34B
U+7AAB.0	fGB	3-5F65
U+7AAB.0	fR/S	 116.9
U+7AAB.0	fTotalStrokes	 14
U+7AAB.0	fCangjie	JCQHK
U+7AAB.0	fFourCorner	30432
U+7AAB.0	fCantonese	jaak3; aat3 (also); ngaat3 (also); zaat3
U+7AAB.0	fMandarin	ya4
U+7AAC.0	fUTF8	窬
U+7AAC.0	fCNS11643	2-4C53
U+7AAC.0	fBig5	E34C
U+7AAC.0	fGB	0-713E
U+7AAC.0	fR/S	 116.9
U+7AAC.0	fTotalStrokes	 14
U+7AAC.0	fCangjie	JCOMN
U+7AAC.0	fFourCorner	30221
U+7AAC.0	fHakka	 ji2; ri2; i2
U+7AAC.0	fCantonese	jyu4; jyu6
U+7AAC.0	fMandarin	yu2; dou1; dou4; tou2
U+7AAC.0	fEnglish	[1] small door or window; hole in the wall [2] climb over a wall
U+7AAD.0	fUTF8	窭
U+7AAD.0	fGB	0-7140
U+7AAD.0	fR/S	 116.9
U+7AAD.0	fTotalStrokes	 14
U+7AAD.0	fCangjie	JCFDV
U+7AAD.0	fHakka	 ki3
U+7AAD.0	fCantonese	geoi6; lau4
U+7AAD.0	fMandarin	ju4; lou2; ju3
U+7AAD.0	fEnglish	simplified form of &#x7AB6;
U+7AAE.0	fUTF8	窮
U+7AAE.0	fCNS11643	1-6E2C
U+7AAE.0	fBig5	BD61
U+7AAE.0	fGB	1-476E
U+7AAE.0	fR/S	 116.10
U+7AAE.0	fTotalStrokes	 15
U+7AAE.0	fCangjie	JCHHN
U+7AAE.0	fFourCorner	30227
U+7AAE.0	fHakka	  kiung2; kiung3
U+7AAE.0	fCantonese	kung4
U+7AAE.0	fMandarin	qiong2; gong1
U+7AAE.0	fEnglish	[1] poor; destitute; impoverished [2] exhaust; trace to the very source [3] distress; affliction [4] the extreme; farthest; end [5] thoroughly
U+7AAF.0	fUTF8	窯
U+7AAF.0	fCNS11643	1-6E2B
U+7AAF.0	fBig5	BD60
U+7AAF.0	fGB	E-364C
U+7AAF.0	fR/S	 116.10
U+7AAF.0	fTotalStrokes	 15
U+7AAF.0	fCangjie	JCTGF
U+7AAF.0	fFourCorner	30331
U+7AAF.0	fHakka	  jau2; rau2; jeu2
U+7AAF.0	fCantonese	jiu4
U+7AAF.0	fMandarin	yao2; qiao1; yao4
U+7AAF.0	fEnglish	 kiln coal mine pit
U+7AB0.0	fUTF8	窰
U+7AB0.0	fCNS11643	3-4D5B
U+7AB0.0	fGB	E-364D
U+7AB0.0	fR/S	 116.10
U+7AB0.0	fTotalStrokes	 15
U+7AB0.0	fCangjie	JCBOU
U+7AB0.0	fFourCorner	30770
U+7AB0.0	fHakka	 jau2; rau2; jau3; jeu2
U+7AB0.0	fCantonese	jiu4
U+7AB0.0	fMandarin	yao2
U+7AB0.0	fEnglish	[1] kiln; brick furnace [2] pottery [3] pit in a coal mine; coal shaft [4] cave (for human dwelling) [5] brothel
U+7AB1.0	fUTF8	窱
U+7AB1.0	fCNS11643	2-5921
U+7AB1.0	fBig5	EAB5
U+7AB1.0	fGB	5-5E4F
U+7AB1.0	fR/S	 116.10
U+7AB1.0	fTotalStrokes	 15
U+7AB1.0	fCangjie	JCOLD
U+7AB1.0	fFourCorner	30294
U+7AB1.0	fHakka	 tiau6; tiau5; tiau3
U+7AB1.0	fCantonese	tiu5; tiu4
U+7AB1.0	fMandarin	tiao4; tiao3; diao4
U+7AB1.0	fEnglish	deep; profound
U+7AB2.0	fUTF8	窲
U+7AB2.0	fCNS11643	2-525B
U+7AB2.0	fBig5	E6D3
U+7AB2.0	fGB	3-5F69
U+7AB2.0	fR/S	 116.10
U+7AB2.0	fTotalStrokes	 15
U+7AB2.0	fCangjie	JCFDJ
U+7AB2.0	fFourCorner	30948
U+7AB2.0	fCantonese	caau4
U+7AB2.0	fMandarin	chao2; liao4; liao2
U+7AB3.0	fUTF8	窳
U+7AB3.0	fCNS11643	2-525D
U+7AB3.0	fBig5	E6D5
U+7AB3.0	fGB	0-7141
U+7AB3.0	fR/S	 116.10
U+7AB3.0	fTotalStrokes	 15
U+7AB3.0	fCangjie	JCHOO
U+7AB3.0	fFourCorner	30232
U+7AB3.0	fHakka	 ri1; ji1; ji3; i3; ju3
U+7AB3.0	fCantonese	jyu5
U+7AB3.0	fMandarin	yu3; yu2; wa1
U+7AB3.0	fEnglish	[1] coarse; crude; of inferior quality [2] weak; fragile [3] bad; mean
U+7AB4.0	fUTF8	窴
U+7AB4.0	fCNS11643	2-525C
U+7AB4.0	fBig5	E6D4
U+7AB4.0	fGB	3-5F67
U+7AB4.0	fR/S	 116.10
U+7AB4.0	fTotalStrokes	 15
U+7AB4.0	fCangjie	JCJBC
U+7AB4.0	fFourCorner	30801
U+7AB4.0	fHakka	 tien2
U+7AB4.0	fCantonese	zi3; tin4
U+7AB4.0	fMandarin	tian2; chan3; yan3
U+7AB4.0	fEnglish	 glance at; look at wink at
U+7AB5.0	fUTF8	窵
U+7AB5.0	fCNS11643	2-587E
U+7AB5.0	fBig5	EAB4
U+7AB5.0	fGB	3-5F5A
U+7AB5.0	fR/S	 116.11
U+7AB5.0	fTotalStrokes	 16
U+7AB5.0	fCangjie	JCHAF
U+7AB5.0	fFourCorner	30327
U+7AB5.0	fHakka	  diau5
U+7AB5.0	fCantonese	diu3
U+7AB5.0	fMandarin	diao4; diao3
U+7AB5.0	fEnglish	 deep distant
U+7AB6.0	fUTF8	窶
U+7AB6.0	fCNS11643	2-587C
U+7AB6.0	fBig5	EAB2
U+7AB6.0	fGB	1-7140
U+7AB6.0	fR/S	 116.11
U+7AB6.0	fTotalStrokes	 16
U+7AB6.0	fCangjie	JCLLV
U+7AB6.0	fFourCorner	30404
U+7AB6.0	fHakka	 gi3; ki3
U+7AB6.0	fCantonese	geoi6; lau4
U+7AB6.0	fMandarin	ju4; lou2; ju3
U+7AB6.0	fEnglish	poor; impoverished; straitened; destitute
U+7AB7.0	fUTF8	窷
U+7AB7.0	fCNS11643	2-5922
U+7AB7.0	fBig5	EAB6
U+7AB7.0	fGB	E-364E
U+7AB7.0	fR/S	 116.11
U+7AB7.0	fTotalStrokes	 16
U+7AB7.0	fCangjie	JCSJL
U+7AB7.0	fFourCorner	30127
U+7AB7.0	fCantonese	liu6; liu4
U+7AB7.0	fMandarin	liao2
U+7AB8.0	fUTF8	窸
U+7AB8.0	fCNS11643	2-587D
U+7AB8.0	fBig5	EAB3
U+7AB8.0	fGB	3-5F6A
U+7AB8.0	fR/S	 116.11
U+7AB8.0	fTotalStrokes	 16
U+7AB8.0	fCangjie	JCHDP
U+7AB8.0	fFourCorner	30339
U+7AB8.0	fCantonese	sik1
U+7AB8.0	fMandarin	xi1
U+7AB8.0	fEnglish	faint sound; whisper
U+7AB9.0	fUTF8	窹
U+7AB9.0	fCNS11643	5-5652
U+7AB9.0	fGB	E-364F
U+7AB9.0	fR/S	  116.11
U+7AB9.0	fTotalStrokes	 16
U+7AB9.0	fCangjie	JCVMR
U+7AB9.0	fFourCorner	30260
U+7AB9.0	fCantonese	ng6
U+7AB9.0	fMandarin	wu4
U+7ABA.0	fUTF8	窺
U+7ABA.0	fCNS11643	1-715D
U+7ABA.0	fBig5	BF73
U+7ABA.0	fGB	1-3F7A
U+7ABA.0	fR/S	 116.11
U+7ABA.0	fTotalStrokes	 16
U+7ABA.0	fCangjie	JCQOU
U+7ABA.0	fFourCorner	30516
U+7ABA.0	fHakka	  gui1; kui1; gwi1; kwui1
U+7ABA.0	fCantonese	kwai1
U+7ABA.0	fMandarin	kui1; kui3
U+7ABA.0	fEnglish	 peep; watch in secret; spy on; see in secret; pry on
U+7ABB.0	fUTF8	窻
U+7ABB.0	fCNS11643	3-522F
U+7ABB.0	fGB	E-3650
U+7ABB.0	fR/S	 116.11
U+7ABB.0	fTotalStrokes	 16
U+7ABB.0	fCangjie	JCHWP
U+7ABB.0	fFourCorner	30330
U+7ABB.0	fHakka	 cung1
U+7ABB.0	fCantonese	coeng1
U+7ABB.0	fMandarin	chuang1
U+7ABC.0	fUTF8	窼
U+7ABC.0	fCNS11643	4-5227
U+7ABC.0	fGB	E-3651
U+7ABC.0	fR/S	 116.11
U+7ABC.0	fTotalStrokes	 16
U+7ABC.0	fCangjie	JCVVD
U+7ABC.0	fFourCorner	30900
U+7ABC.0	fHakka	 cau2
U+7ABC.0	fCantonese	zaau1
U+7ABC.0	fMandarin	zhao1; ke1; cong1
U+7ABD.0	fUTF8	窽
U+7ABD.0	fCNS11643	3-522E
U+7ABD.0	fGB	E-3652
U+7ABD.0	fR/S	  116.11
U+7ABD.0	fTotalStrokes	 16
U+7ABD.0	fCangjie	JCPKO
U+7ABD.0	fFourCorner	30480
U+7ABD.0	fHakka	 k'wan3
U+7ABD.0	fCantonese	fun2
U+7ABD.0	fEnglish	[1] hollow; empty [2] ignorant
U+7ABE.0	fUTF8	窾
U+7ABE.0	fCNS11643	2-5E24
U+7ABE.0	fBig5	EDB7
U+7ABE.0	fGB	3-5F6D
U+7ABE.0	fR/S	 116.12
U+7ABE.0	fTotalStrokes	 17
U+7ABE.0	fCangjie	JCGFO
U+7ABE.0	fFourCorner	30982
U+7ABE.0	fHakka	 k'wan3; kwan3
U+7ABE.0	fCantonese	fun2; fo1
U+7ABE.0	fMandarin	kuan3; ke1; kong1
U+7ABE.0	fEnglish	[1] opening; hole; hollow [2] hollow; empty
U+7ABF.0	fUTF8	窿
U+7ABF.0	fCNS11643	1-7455
U+7ABF.0	fBig5	C14B
U+7ABF.0	fGB	0-417E
U+7ABF.0	fR/S	 116.12
U+7ABF.0	fTotalStrokes	 17
U+7ABF.0	fCangjie	JCNLM
U+7ABF.0	fFourCorner	30214
U+7ABF.0	fHakka	  lung2;  lung1
U+7ABF.0	fCantonese	lung1; lung4
U+7ABF.0	fMandarin	long2
U+7ABF.0	fEnglish	 cavity; hole
U+7AC0.0	fUTF8	竀
U+7AC0.0	fCNS11643	2-5E25
U+7AC0.0	fBig5	EDB8
U+7AC0.0	fGB	3-5F66
U+7AC0.0	fR/S	 116.12
U+7AC0.0	fTotalStrokes	 17
U+7AC0.0	fCangjie	JCMMU
U+7AC0.0	fFourCorner	30116
U+7AC0.0	fHakka	 cang1; zhin5; zihn5
U+7AC0.0	fCantonese	cang1; cing1 (also)
U+7AC0.0	fMandarin	cheng1
U+7AC0.0	fEnglish	[1] look at [2] red [3] twice dyed
U+7AC1.0	fUTF8	竁
U+7AC1.0	fCNS11643	2-5E26
U+7AC1.0	fBig5	EDB9
U+7AC1.0	fGB	3-5F6E
U+7AC1.0	fR/S	 116.12
U+7AC1.0	fTotalStrokes	 17
U+7AC1.0	fCangjie	JCHUU
U+7AC1.0	fFourCorner	30715
U+7AC1.0	fHakka	 cui5; rui5; jui5
U+7AC1.0	fCantonese	ceoi3
U+7AC1.0	fMandarin	cui4; chuan4
U+7AC1.0	fEnglish	bore
U+7AC2.0	fUTF8	竂
U+7AC2.0	fCNS11643	3-5568
U+7AC2.0	fGB	5-5E59
U+7AC2.0	fR/S	 116.12
U+7AC2.0	fTotalStrokes	 17
U+7AC2.0	fCangjie	JCKCF
U+7AC2.0	fFourCorner	30900
U+7AC2.0	fCantonese	liu4
U+7AC2.0	fMandarin	piao2
U+7AC3.0	fUTF8	竃
U+7AC3.0	fGB	E-3653
U+7AC3.0	fR/S	 116.12
U+7AC3.0	fTotalStrokes	 21
U+7AC3.0	fCangjie	JCGWU
U+7AC3.0	fCantonese	hung1
U+7AC3.0	fMandarin	zao4
U+7AC3.0	fEnglish	 furnace kitchen range
U+7AC4.0	fUTF8	竄
U+7AC4.0	fCNS11643	1-7675
U+7AC4.0	fBig5	C2AB
U+7AC4.0	fGB	1-345C
U+7AC4.0	fR/S	 116.13
U+7AC4.0	fTotalStrokes	 18
U+7AC4.0	fCangjie	JCHXV
U+7AC4.0	fFourCorner	30717
U+7AC4.0	fHakka	 con5; con3; chon3
U+7AC4.0	fCantonese	cyun2; cyun3 (also)
U+7AC4.0	fMandarin	cuan4; cuan1; cui4
U+7AC4.0	fEnglish	[1] escape; run away; flee [2] change; alter; revise; edit [3] banish; execute; expel
U+7AC5.0	fUTF8	竅
U+7AC5.0	fCNS11643	1-7676
U+7AC5.0	fBig5	C2AC
U+7AC5.0	fGB	1-474F
U+7AC5.0	fR/S	 116.13
U+7AC5.0	fTotalStrokes	 18
U+7AC5.0	fCangjie	JCHSK
U+7AC5.0	fFourCorner	30248
U+7AC5.0	fHakka	  kiau5; k'ieu5
U+7AC5.0	fCantonese	kiu3; hiu3 (also); kiu2 (also)
U+7AC5.0	fMandarin	qiao4
U+7AC5.0	fEnglish	[1] hole; opening; cavity [2] apertures [3] crux; key points; gist of a matter; knack
U+7AC6.0	fUTF8	竆
U+7AC6.0	fCNS11643	4-604B
U+7AC6.0	fGB	E-3654
U+7AC6.0	fR/S	 116.14
U+7AC6.0	fTotalStrokes	 19
U+7AC6.0	fCangjie	JCHHR
U+7AC6.0	fFourCorner	30460
U+7AC6.0	fHakka	 kiung2
U+7AC6.0	fCantonese	kung4
U+7AC6.0	fMandarin	qiong2
U+7AC6.0	fEnglish	 exhausted impoverished poor
U+7AC7.0	fUTF8	竇
U+7AC7.0	fCNS11643	1-7A22
U+7AC7.0	fBig5	C475
U+7AC7.0	fGB	1-713C
U+7AC7.0	fR/S	 116.15
U+7AC7.0	fTotalStrokes	 20
U+7AC7.0	fCangjie	JCGWC
U+7AC7.0	fFourCorner	30806
U+7AC7.0	fHakka	  teu5; deu5; tiu5; teu6
U+7AC7.0	fCantonese	dau6; dau3; dau2 (also); duk6
U+7AC7.0	fMandarin	dou4; du2
U+7AC7.0	fEnglish	[1] hole; cavity; burrow [2] corrupt; corrupt practice [3] a Chinese family name
U+7AC8.0	fUTF8	竈
U+7AC8.0	fCNS11643	3-5E49
U+7AC8.0	fGB	1-546E
U+7AC8.0	fR/S	  116.16
U+7AC8.0	fTotalStrokes	 21
U+7AC8.0	fCangjie	JCGRU
U+7AC8.0	fFourCorner	30710
U+7AC8.0	fHakka	  zau5; zo5; za5
U+7AC8.0	fCantonese	zou3
U+7AC8.0	fMandarin	zao4
U+7AC8.0	fEnglish	[1] place for cooking; kitchen [2] cooking stove; furnace
U+7AC9.0	fUTF8	竉
U+7AC9.0	fCNS11643	3-5E48
U+7AC9.0	fGB	5-5E3C
U+7AC9.0	fR/S	 116.16
U+7AC9.0	fTotalStrokes	 21
U+7AC9.0	fCangjie	JCYBP
U+7AC9.0	fFourCorner	30210
U+7AC9.0	fHakka	 lung3; lung1; lung2
U+7AC9.0	fCantonese	lung1
U+7AC9.0	fMandarin	long3
U+7AC9.0	fEnglish	 hole cleft empty
U+7ACA.0	fUTF8	竊
U+7ACA.0	fCNS11643	1-7C3D
U+7ACA.0	fBig5	C5D1
U+7ACA.0	fGB	1-4754
U+7ACA.0	fR/S	 116.17
U+7ACA.0	fTotalStrokes	 22
U+7ACA.0	fCangjie	JCHDB
U+7ACA.0	fFourCorner	30927
U+7ACA.0	fHakka	 ciap7
U+7ACA.0	fCantonese	sit3
U+7ACA.0	fMandarin	qie4
U+7ACA.0	fEnglish	[1] steal; burglarise [2] thief; burglar [3] usurp [4] stealthy [5] a self-derogatory way of saying "I"
U+7ACB.0	fUTF8	立
U+7ACB.0	fCNS11643	1-4721
U+7ACB.0	fBig5	A5DF
U+7ACB.0	fGB	0-4122
U+7ACB.0	fR/S	 117.0
U+7ACB.0	fTotalStrokes	 5
U+7ACB.0	fCangjie	YT
U+7ACB.0	fFourCorner	00108
U+7ACB.0	fHakka	  lip8
U+7ACB.0	fCantonese	laap6 (also); lap6
U+7ACB.0	fMandarin	li4; wei4
U+7ACB.0	fEnglish	[1] stand [2] establish; found; build; erect; create; start [3] stand on one's own feet; live [4] immediately; at once [5] a Chinese family name [6] Kangxi radical 117
U+7ACC.0	fUTF8	竌
U+7ACC.0	fCNS11643	3-267E
U+7ACC.0	fGB	E-3655
U+7ACC.0	fR/S	  117.2
U+7ACC.0	fTotalStrokes	 7
U+7ACC.0	fCangjie	YTHN
U+7ACC.0	fFourCorner	07110
U+7ACC.0	fCantonese	cyu3
U+7ACC.0	fMandarin	chu4
U+7ACD.0	fUTF8	竍
U+7ACD.0	fCNS11643	3-2721
U+7ACD.0	fGB	E-3656
U+7ACD.0	fR/S	  117.2
U+7ACD.0	fTotalStrokes	 7
U+7ACD.0	fCangjie	YTJ
U+7ACD.0	fFourCorner	04100
U+7ACD.0	fCantonese	sap6
U+7ACD.0	fMandarin	shi2gong1sheng1
U+7ACD.0	fEnglish	decaliter
U+7ACE.0	fUTF8	竎
U+7ACE.0	fCNS11643	4-275B
U+7ACE.0	fGB	E-3657
U+7ACE.0	fR/S	  117.3
U+7ACE.0	fTotalStrokes	 8
U+7ACE.0	fCangjie	YTT
U+7ACE.0	fFourCorner	00440
U+7ACE.0	fCantonese	fu3
U+7ACE.0	fMandarin	fou4; fu4
U+7ACF.0	fUTF8	竏
U+7ACF.0	fCNS11643	3-2A47
U+7ACF.0	fGB	E-3658
U+7ACF.0	fR/S	 117.3
U+7ACF.0	fTotalStrokes	 8
U+7ACF.0	fCangjie	YTHJ
U+7ACF.0	fFourCorner	02140
U+7ACF.0	fCantonese	cin1
U+7ACF.0	fMandarin	qian1gong1sheng1
U+7ACF.0	fEnglish	kiloliter
U+7AD0.0	fUTF8	竐
U+7AD0.0	fCNS11643	3-2E4E
U+7AD0.0	fGB	E-3659
U+7AD0.0	fR/S	 117.4
U+7AD0.0	fTotalStrokes	 9
U+7AD0.0	fCangjie	YTHNE
U+7AD0.0	fFourCorner	07140
U+7AD0.0	fCantonese	cuk1
U+7AD0.0	fMandarin	chuo4; chu4; qi4
U+7AD1.0	fUTF8	竑
U+7AD1.0	fCNS11643	2-2C62
U+7AD1.0	fBig5	D041
U+7AD1.0	fGB	3-5F49
U+7AD1.0	fR/S	 117.4
U+7AD1.0	fTotalStrokes	 9
U+7AD1.0	fCangjie	YTKI
U+7AD1.0	fFourCorner	04132
U+7AD1.0	fHakka	 fen2
U+7AD1.0	fCantonese	wang4
U+7AD1.0	fMandarin	hong2
U+7AD1.0	fEnglish	 be vast and endless broad
U+7AD2.0	fUTF8	竒
U+7AD2.0	fCNS11643	3-2E4F
U+7AD2.0	fGB	E-365A
U+7AD2.0	fR/S	 117.4
U+7AD2.0	fTotalStrokes	 9
U+7AD2.0	fCangjie	YTNR
U+7AD2.0	fFourCorner	00620
U+7AD2.0	fHakka	  ki2
U+7AD2.0	fCantonese	kei4; gei1
U+7AD2.0	fMandarin	qi2; ai3; yi3
U+7AD2.0	fEnglish	strange, unusual, uncanny, occult
U+7AD3.0	fUTF8	竓
U+7AD3.0	fCNS11643	3-2E4D
U+7AD3.0	fGB	E-365B
U+7AD3.0	fR/S	 117.4
U+7AD3.0	fTotalStrokes	 9
U+7AD3.0	fCangjie	YTHQU
U+7AD3.0	fFourCorner	02110
U+7AD3.0	fCantonese	hou4
U+7AD3.0	fMandarin	qian1fen1zhi1yi1gong1sheng1
U+7AD3.0	fEnglish	milliliter
U+7AD4.0	fUTF8	竔
U+7AD4.0	fCNS11643	3-2E4B
U+7AD4.0	fGB	E-365C
U+7AD4.0	fR/S	  117.4
U+7AD4.0	fTotalStrokes	 9
U+7AD4.0	fCangjie	YTHT
U+7AD4.0	fFourCorner	02140
U+7AD4.0	fCantonese	sing1
U+7AD4.0	fMandarin	gong1sheng1
U+7AD4.0	fEnglish	litre
U+7AD5.0	fUTF8	竕
U+7AD5.0	fCNS11643	3-2E4C
U+7AD5.0	fGB	E-365D
U+7AD5.0	fR/S	  117.4
U+7AD5.0	fTotalStrokes	 9
U+7AD5.0	fCangjie	YTCSH
U+7AD5.0	fFourCorner	08120
U+7AD5.0	fCantonese	fan1
U+7AD5.0	fMandarin	shi2fen1zhi1yi1gong1sheng1
U+7AD5.0	fEnglish	deciliter
U+7AD6.0	fUTF8	竖
U+7AD6.0	fGB	0-4A7A
U+7AD6.0	fR/S	 117.4
U+7AD6.0	fTotalStrokes	 9
U+7AD6.0	fCangjie	LEYT
U+7AD6.0	fHakka	 shu5
U+7AD6.0	fCantonese	syu6
U+7AD6.0	fMandarin	shu4
U+7AD6.0	fEnglish	simplified form of &#x7AEA;
U+7AD7.0	fUTF8	竗
U+7AD7.0	fCNS11643	F-2C76
U+7AD7.0	fGB	E-365E
U+7AD7.0	fR/S	 117.4
U+7AD7.0	fTotalStrokes	 9
U+7AD7.0	fCangjie	YTFH
U+7AD7.0	fCantonese	miu6
U+7AD7.0	fMandarin	miao4; miao3
U+7AD8.0	fUTF8	竘
U+7AD8.0	fCNS11643	2-3160
U+7AD8.0	fBig5	D2FD
U+7AD8.0	fGB	3-5F4C
U+7AD8.0	fR/S	 117.5
U+7AD8.0	fTotalStrokes	 10
U+7AD8.0	fCangjie	YTPR
U+7AD8.0	fFourCorner	07120
U+7AD8.0	fCantonese	keoi5; geoi2; keoi4
U+7AD8.0	fMandarin	ju3; qu3; kou3; hou3
U+7AD9.0	fUTF8	站
U+7AD9.0	fCNS11643	1-573B
U+7AD9.0	fBig5	AFB8
U+7AD9.0	fGB	0-553E
U+7AD9.0	fR/S	 117.5
U+7AD9.0	fTotalStrokes	 10
U+7AD9.0	fCangjie	YTYR
U+7AD9.0	fFourCorner	01160
U+7AD9.0	fHakka	 cam5; zam5; zham5; zan5; cam6; zan3
U+7AD9.0	fCantonese	zaam6
U+7AD9.0	fMandarin	zhan4; zhan1
U+7AD9.0	fEnglish	[1] stand [2] station; stop; center for rendering certain services
U+7ADA.0	fUTF8	竚
U+7ADA.0	fCNS11643	3-3323
U+7ADA.0	fGB	E-365F
U+7ADA.0	fR/S	 117.5
U+7ADA.0	fTotalStrokes	 10
U+7ADA.0	fCangjie	YTJMN
U+7ADA.0	fFourCorner	03120
U+7ADA.0	fHakka	  du3
U+7ADA.0	fCantonese	cyu5
U+7ADA.0	fMandarin	zhu4
U+7ADA.0	fEnglish	 stand for a long time
U+7ADB.0	fUTF8	竛
U+7ADB.0	fCNS11643	3-3326
U+7ADB.0	fGB	3-5F4B
U+7ADB.0	fR/S	 117.5
U+7ADB.0	fTotalStrokes	 10
U+7ADB.0	fCangjie	YTOII
U+7ADB.0	fFourCorner	08130
U+7ADB.0	fCantonese	ling4
U+7ADB.0	fMandarin	ling2
U+7ADC.0	fUTF8	竜
U+7ADC.0	fCNS11643	3-3327
U+7ADC.0	fGB	5-5D7D
U+7ADC.0	fR/S	 117.5
U+7ADC.0	fTotalStrokes	 10
U+7ADC.0	fCangjie	YTWU
U+7ADC.0	fFourCorner	00710
U+7ADC.0	fHakka	  liung2; lung2
U+7ADC.0	fCantonese	lung4
U+7ADC.0	fMandarin	long2; long3; mang2
U+7ADC.0	fEnglish	 dragon symbolic of emperor
U+7ADD.0	fUTF8	竝
U+7ADD.0	fCNS11643	3-3324
U+7ADD.0	fGB	E-3660
U+7ADD.0	fR/S	 117.5
U+7ADD.0	fTotalStrokes	 10
U+7ADD.0	fCangjie	YTYT
U+7ADD.0	fFourCorner	00110
U+7ADD.0	fHakka	  bin5
U+7ADD.0	fCantonese	bing6; bing3 (also)
U+7ADD.0	fMandarin	bing4; bang4; bian4
U+7ADD.0	fEnglish	[1] also; and [2] combine; join
U+7ADE.0	fUTF8	竞
U+7ADE.0	fGB	0-3E3A
U+7ADE.0	fR/S	 117.5
U+7ADE.0	fTotalStrokes	 10
U+7ADE.0	fCangjie	YTRHU
U+7ADE.0	fHakka	 kin5
U+7ADE.0	fCantonese	ging3 (also); ging6
U+7ADE.0	fMandarin	jing4
U+7ADE.0	fEnglish	simplified form of &#x7AF6;
U+7ADF.0	fUTF8	竟
U+7ADF.0	fCNS11643	1-5D7E
U+7ADF.0	fBig5	B3BA
U+7ADF.0	fGB	0-3E39
U+7ADF.0	fR/S	 117.6; 180.2
U+7ADF.0	fTotalStrokes	 11
U+7ADF.0	fCangjie	YTAHU
U+7ADF.0	fFourCorner	00216
U+7ADF.0	fHakka	  gin3; gin5
U+7ADF.0	fCantonese	ging2
U+7ADF.0	fMandarin	jing4; jing3
U+7ADF.0	fEnglish	[1] come to and end; terminate; go through the whole course; finish; complete [2] rather unexpectedly; somewhat to one's surprise; in a way thought to be rather unlikely
U+7AE0.0	fUTF8	章
U+7AE0.0	fCNS11643	1-5D7D
U+7AE0.0	fBig5	B3B9
U+7AE0.0	fGB	0-5542
U+7AE0.0	fR/S	 117.6; 180.2
U+7AE0.0	fTotalStrokes	 11
U+7AE0.0	fCangjie	YTAJ
U+7AE0.0	fFourCorner	00406
U+7AE0.0	fHakka	  zong1; zhong1
U+7AE0.0	fCantonese	zoeng1
U+7AE0.0	fMandarin	zhang1; zhang4
U+7AE0.0	fEnglish	[1] piece of writing; chapter; main division of a book; section [2] system; statute; organised body [3] emblem; seal; stamp [4] make clear; make known [5] pattern; example [6] a Chinese family name
U+7AE1.0	fUTF8	竡
U+7AE1.0	fCNS11643	3-3773
U+7AE1.0	fGB	E-3661
U+7AE1.0	fR/S	  117.6
U+7AE1.0	fTotalStrokes	 11
U+7AE1.0	fCangjie	YTMA
U+7AE1.0	fFourCorner	01160
U+7AE1.0	fCantonese	baak3
U+7AE1.0	fMandarin	yi1gong1sheng1deyi1bai3bei4
U+7AE1.0	fEnglish	hectoliter
U+7AE2.0	fUTF8	竢
U+7AE2.0	fCNS11643	3-3D71
U+7AE2.0	fGB	E-3662
U+7AE2.0	fR/S	 117.7
U+7AE2.0	fTotalStrokes	 12
U+7AE2.0	fCangjie	YTIOK
U+7AE2.0	fFourCorner	03130
U+7AE2.0	fHakka	 sii5; se5
U+7AE2.0	fCantonese	zi6
U+7AE2.0	fMandarin	si4
U+7AE2.0	fEnglish	wait for; wait until; as soon as
U+7AE3.0	fUTF8	竣
U+7AE3.0	fCNS11643	1-612A
U+7AE3.0	fBig5	B5A4
U+7AE3.0	fGB	0-3F22
U+7AE3.0	fR/S	 117.7
U+7AE3.0	fTotalStrokes	 12
U+7AE3.0	fCangjie	YTICE
U+7AE3.0	fFourCorner	03147
U+7AE3.0	fHakka	 zun5
U+7AE3.0	fCantonese	zeon3; ceon1
U+7AE3.0	fMandarin	jun4; qun1; quan1; zhun1
U+7AE3.0	fEnglish	 terminate; end; finish; accomplished; completed
U+7AE4.0	fUTF8	竤
U+7AE4.0	fCNS11643	2-3E61
U+7AE4.0	fBig5	DADD
U+7AE4.0	fGB	5-5E27
U+7AE4.0	fR/S	 117.7
U+7AE4.0	fTotalStrokes	 12
U+7AE4.0	fCangjie	YTJKI
U+7AE4.0	fFourCorner	03134
U+7AE4.0	fCantonese	wang4
U+7AE4.0	fMandarin	hong2
U+7AE5.0	fUTF8	童
U+7AE5.0	fCNS11643	1-6129
U+7AE5.0	fBig5	B5A3
U+7AE5.0	fGB	0-4D2F
U+7AE5.0	fR/S	 117.7
U+7AE5.0	fTotalStrokes	 12
U+7AE5.0	fCangjie	YTWG
U+7AE5.0	fFourCorner	00104
U+7AE5.0	fHakka	  tung2; tung3
U+7AE5.0	fCantonese	tung4; tung2 (also)
U+7AE5.0	fMandarin	tong2; zhong1
U+7AE5.0	fEnglish	[1] child; minor; virgin [2] bare; barren (land, etc.) [3] a Chinese family name
U+7AE6.0	fUTF8	竦
U+7AE6.0	fCNS11643	2-3E60
U+7AE6.0	fBig5	DADC
U+7AE6.0	fGB	0-7135
U+7AE6.0	fR/S	 117.7
U+7AE6.0	fTotalStrokes	 12
U+7AE6.0	fCangjie	YTDL
U+7AE6.0	fFourCorner	05196
U+7AE6.0	fHakka	 sung3
U+7AE6.0	fCantonese	sung2
U+7AE6.0	fMandarin	song3
U+7AE6.0	fEnglish	[1] respectful [2] awed
U+7AE7.0	fUTF8	竧
U+7AE7.0	fCNS11643	3-3D70
U+7AE7.0	fGB	5-5E26
U+7AE7.0	fR/S	 117.7
U+7AE7.0	fTotalStrokes	 12
U+7AE7.0	fCangjie	YTHXH
U+7AE7.0	fFourCorner	07140
U+7AE7.0	fCantonese	zing6
U+7AE7.0	fMandarin	jing4; zhen4
U+7AE8.0	fUTF8	竨
U+7AE8.0	fCNS11643	4-3F57
U+7AE8.0	fGB	E-3663
U+7AE8.0	fR/S	  117.8
U+7AE8.0	fTotalStrokes	 13
U+7AE8.0	fCangjie	YTYAJ
U+7AE8.0	fFourCorner	01140
U+7AE8.0	fCantonese	diu6
U+7AE8.0	fMandarin	diao4
U+7AE9.0	fUTF8	竩
U+7AE9.0	fCNS11643	3-4349
U+7AE9.0	fGB	E-3664
U+7AE9.0	fR/S	 117.8
U+7AE9.0	fTotalStrokes	 13
U+7AE9.0	fCangjie	YTJBM
U+7AE9.0	fFourCorner	03110
U+7AE9.0	fCantonese	ji4
U+7AE9.0	fMandarin	yi4
U+7AEA.0	fUTF8	竪
U+7AEA.0	fCNS11643	3-434A
U+7AEA.0	fGB	1-4A7A
U+7AEA.0	fR/S	 117.8
U+7AEA.0	fTotalStrokes	 13
U+7AEA.0	fCangjie	SEYT
U+7AEA.0	fFourCorner	77100
U+7AEA.0	fHakka	 su5; shu6; shu5
U+7AEA.0	fCantonese	syu6
U+7AEA.0	fMandarin	shu4
U+7AEA.0	fEnglish	 perpendicular; vertical erect
U+7AEB.0	fUTF8	竫
U+7AEB.0	fCNS11643	2-456A
U+7AEB.0	fBig5	DF45
U+7AEB.0	fGB	5-5E22
U+7AEB.0	fR/S	 117.8
U+7AEB.0	fTotalStrokes	 13
U+7AEB.0	fCangjie	YTBSD
U+7AEB.0	fFourCorner	02157
U+7AEB.0	fHakka	 cin5; cin6
U+7AEB.0	fCantonese	zing6
U+7AEB.0	fMandarin	jing4
U+7AEB.0	fEnglish	[1] quiet; still [2] repose
U+7AEC.0	fUTF8	竬
U+7AEC.0	fCNS11643	3-4854
U+7AEC.0	fGB	E-3665
U+7AEC.0	fR/S	 117.9
U+7AEC.0	fTotalStrokes	 14
U+7AEC.0	fCangjie	YTHLB
U+7AEC.0	fFourCorner	02120
U+7AEC.0	fCantonese	geoi2
U+7AEC.0	fMandarin	qu3
U+7AED.0	fUTF8	竭
U+7AED.0	fCNS11643	1-6A25
U+7AED.0	fBig5	BADC
U+7AED.0	fGB	0-3D5F
U+7AED.0	fR/S	 117.9
U+7AED.0	fTotalStrokes	 14
U+7AED.0	fCangjie	YTAPV
U+7AED.0	fFourCorner	06127
U+7AED.0	fHakka	 kiet8; kiet7; k'iat8; ket7; ket8
U+7AED.0	fCantonese	kit3; git3
U+7AED.0	fMandarin	jie2
U+7AED.0	fEnglish	[1] devote; put forth (effort, etc.) [2] exhaust; use up
U+7AEE.0	fUTF8	竮
U+7AEE.0	fCNS11643	2-4C54
U+7AEE.0	fBig5	E34D
U+7AEE.0	fGB	5-5E2E
U+7AEE.0	fR/S	 117.11
U+7AEE.0	fTotalStrokes	 16
U+7AEE.0	fCangjie	YTSTT
U+7AEE.0	fFourCorner	07141
U+7AEE.0	fCantonese	ping4
U+7AEE.0	fMandarin	ping2; ping1
U+7AEF.0	fUTF8	端
U+7AEF.0	fCNS11643	1-6A26
U+7AEF.0	fBig5	BADD
U+7AEF.0	fGB	0-364B
U+7AEF.0	fR/S	 117.9
U+7AEF.0	fTotalStrokes	 14
U+7AEF.0	fCangjie	YTUMB
U+7AEF.0	fFourCorner	02127
U+7AEF.0	fHakka	  don1
U+7AEF.0	fCantonese	dyun1
U+7AEF.0	fMandarin	duan1; chuan3; mian3
U+7AEF.0	fEnglish	[1] end; extreme [2] head; beginning [3] correct; proper; upright [4] leads; clue [5] carry carefully [6] cause
U+7AF0.0	fUTF8	竰
U+7AF0.0	fCNS11643	3-4855
U+7AF0.0	fGB	E-3666
U+7AF0.0	fR/S	  117.9
U+7AF0.0	fTotalStrokes	 14
U+7AF0.0	fCangjie	YTMWG
U+7AF0.0	fFourCorner	01110
U+7AF0.0	fCantonese	lei4
U+7AF0.0	fMandarin	shao2
U+7AF0.0	fEnglish	centiliter
U+7AF1.0	fUTF8	竱
U+7AF1.0	fCNS11643	3-5230
U+7AF1.0	fGB	3-5F48
U+7AF1.0	fR/S	 117.11
U+7AF1.0	fTotalStrokes	 16
U+7AF1.0	fCangjie	YTJII
U+7AF1.0	fFourCorner	05140
U+7AF1.0	fHakka	 zhon5
U+7AF1.0	fCantonese	zyun2
U+7AF1.0	fMandarin	zhuan3; zhuan4; duan4; duan1
U+7AF2.0	fUTF8	竲
U+7AF2.0	fCNS11643	3-5569
U+7AF2.0	fGB	5-5E2F
U+7AF2.0	fR/S	  117.12
U+7AF2.0	fTotalStrokes	 17
U+7AF2.0	fCangjie	YTCWA
U+7AF2.0	fFourCorner	08160
U+7AF2.0	fHakka	  zen1
U+7AF2.0	fCantonese	cang4
U+7AF2.0	fMandarin	ceng2; zeng1; cheng2
U+7AF2.0	fEnglish	 hut; pigsty
U+7AF3.0	fUTF8	竳
U+7AF3.0	fCNS11643	3-556A
U+7AF3.0	fGB	3-5F53
U+7AF3.0	fR/S	  117.12
U+7AF3.0	fTotalStrokes	 17
U+7AF3.0	fCangjie	YTNOT
U+7AF3.0	fFourCorner	02110
U+7AF3.0	fHakka	 den2; den1; dang1; dang3
U+7AF3.0	fCantonese	dang1
U+7AF3.0	fMandarin	deng1
U+7AF4.0	fUTF8	竴
U+7AF4.0	fCNS11643	4-5750
U+7AF4.0	fGB	E-3667
U+7AF4.0	fR/S	  117.12
U+7AF4.0	fTotalStrokes	 17; 16
U+7AF4.0	fCangjie	YTTWI
U+7AF4.0	fFourCorner	08140
U+7AF4.0	fCantonese	ceon1
U+7AF4.0	fMandarin	cui1; cun1
U+7AF5.0	fUTF8	竵
U+7AF5.0	fCNS11643	3-5849
U+7AF5.0	fGB	E-3668
U+7AF5.0	fR/S	  117.13
U+7AF5.0	fTotalStrokes	 18
U+7AF5.0	fCangjie	YTMRB
U+7AF5.0	fFourCorner	01120
U+7AF5.0	fCantonese	waai1; me2 (also)
U+7AF5.0	fMandarin	huai1; kuai1; wai1
U+7AF6.0	fUTF8	競
U+7AF6.0	fCNS11643	1-7A23
U+7AF6.0	fBig5	C476
U+7AF6.0	fGB	1-3E3A
U+7AF6.0	fR/S	 117.15
U+7AF6.0	fTotalStrokes	 20
U+7AF6.0	fCangjie	YUYTU
U+7AF6.0	fFourCorner	00216
U+7AF6.0	fHakka	  kin5; gin5; kin3; gin3; kin1
U+7AF6.0	fCantonese	ging3 (also); ging6
U+7AF6.0	fMandarin	jing4
U+7AF6.0	fEnglish	 vie; compete
U+7AF7.0	fUTF8	竷
U+7AF7.0	fCNS11643	2-6952
U+7AF7.0	fBig5	F4A5
U+7AF7.0	fGB	E-3669
U+7AF7.0	fR/S	 117.15
U+7AF7.0	fTotalStrokes	 20
U+7AF7.0	fCangjie	YJHEE
U+7AF7.0	fFourCorner	08447
U+7AF7.0	fCantonese	ham2; ham3
U+7AF7.0	fMandarin	kan4; kan3
U+7AF8.0	fUTF8	竸
U+7AF8.0	fCNS11643	3-5F55
U+7AF8.0	fGB	E-366A
U+7AF8.0	fR/S	 117.17
U+7AF8.0	fTotalStrokes	 22
U+7AF8.0	fCangjie	YUYTU
U+7AF8.0	fFourCorner	00210
U+7AF8.0	fHakka	 kin5; kiang5
U+7AF8.0	fCantonese	ging6
U+7AF8.0	fMandarin	jing4
U+7AF8.0	fEnglish	contend; vie; compete
U+7AF9.0	fUTF8	竹
U+7AF9.0	fCNS11643	1-484C
U+7AF9.0	fBig5	A6CB
U+7AF9.0	fGB	0-5671
U+7AF9.0	fR/S	 118.0
U+7AF9.0	fTotalStrokes	 6
U+7AF9.0	fCangjie	H
U+7AF9.0	fFourCorner	88220
U+7AF9.0	fHakka	  zuk7;  chuk7; zhuk7
U+7AF9.0	fCantonese	zuk1
U+7AF9.0	fMandarin	zhu2
U+7AF9.0	fEnglish	[1] bamboo [2] slips of bamboo for writing [3] a Chinese family name [4] Kangxi radical 118
U+7AFA.0	fUTF8	竺
U+7AFA.0	fCNS11643	1-4F2A
U+7AFA.0	fBig5	AAC7
U+7AFA.0	fGB	0-7343
U+7AFA.0	fR/S	 118.2
U+7AFA.0	fTotalStrokes	 8
U+7AFA.0	fCangjie	HMM
U+7AFA.0	fFourCorner	88101
U+7AFA.0	fHakka	 chuk7; zuk7; zhuk7
U+7AFA.0	fCantonese	zuk1
U+7AFA.0	fMandarin	zhu2; du3
U+7AFA.0	fEnglish	[1] India [2] a Chinese family name
U+7AFB.0	fUTF8	竻
U+7AFB.0	fCNS11643	2-2847
U+7AFB.0	fBig5	CDA7
U+7AFB.0	fGB	3-6346
U+7AFB.0	fR/S	 118.2
U+7AFB.0	fTotalStrokes	 8
U+7AFB.0	fCangjie	HKS
U+7AFB.0	fFourCorner	88427
U+7AFB.0	fHakka	 net7;  let8; let7; net8
U+7AFB.0	fCantonese	lak6; lak1
U+7AFB.0	fMandarin	le4; jin1
U+7AFB.0	fEnglish	thorns
U+7AFC.0	fUTF8	竼
U+7AFC.0	fCNS11643	3-2E51
U+7AFC.0	fGB	E-366B
U+7AFC.0	fR/S	  118.3
U+7AFC.0	fTotalStrokes	 9
U+7AFC.0	fCangjie	HHNI
U+7AFC.0	fFourCorner	88210
U+7AFC.0	fCantonese	pung4
U+7AFC.0	fMandarin	peng2
U+7AFC.0	fEnglish	bamboo cover for boat
U+7AFD.0	fUTF8	竽
U+7AFD.0	fCNS11643	1-5275
U+7AFD.0	fBig5	ACF2
U+7AFD.0	fGB	0-7344
U+7AFD.0	fR/S	 118.3
U+7AFD.0	fTotalStrokes	 9
U+7AFD.0	fCangjie	HMD
U+7AFD.0	fFourCorner	88401
U+7AFD.0	fHakka	 ji1; ri1; i2
U+7AFD.0	fCantonese	jyu4; jyu1 (also)
U+7AFD.0	fMandarin	yu2
U+7AFD.0	fEnglish	a woodwind musical instrument with 36 reeds
U+7AFE.0	fUTF8	竾
U+7AFE.0	fCNS11643	3-2E50
U+7AFE.0	fGB	5-6271
U+7AFE.0	fR/S	 118.3
U+7AFE.0	fTotalStrokes	 9
U+7AFE.0	fCangjie	HPD
U+7AFE.0	fFourCorner	88710
U+7AFE.0	fHakka	 chii2; chi2
U+7AFE.0	fCantonese	ci4
U+7AFE.0	fMandarin	chi2
U+7AFE.0	fEnglish	flute
U+7AFF.0	fUTF8	竿
U+7AFF.0	fCNS11643	1-5274
U+7AFF.0	fBig5	ACF1
U+7AFF.0	fGB	0-384D
U+7AFF.0	fR/S	 118.3
U+7AFF.0	fTotalStrokes	 9
U+7AFF.0	fCangjie	HMJ
U+7AFF.0	fFourCorner	88401
U+7AFF.0	fHakka	  gon1; gon3; gwon1
U+7AFF.0	fCantonese	gon1
U+7AFF.0	fMandarin	gan1; gan3; gan4
U+7AFF.0	fEnglish	[1] bamboo pole; bamboo rod [2] slips of bamboo for writing
U+7B00.0	fUTF8	笀
U+7B00.0	fCNS11643	2-2C63
U+7B00.0	fBig5	D042
U+7B00.0	fGB	E-366C
U+7B00.0	fR/S	 118.3
U+7B00.0	fTotalStrokes	 9
U+7B00.0	fCangjie	HYV
U+7B00.0	fFourCorner	88710
U+7B00.0	fCantonese	mong4
U+7B00.0	fMandarin	mang2
U+7B01.0	fUTF8	笁
U+7B01.0	fCNS11643	2-2C64
U+7B01.0	fBig5	D043
U+7B01.0	fGB	E-366D
U+7B01.0	fR/S	 118.3
U+7B01.0	fTotalStrokes	 9
U+7B01.0	fCangjie	HM
U+7B01.0	fFourCorner	88104
U+7B01.0	fCantonese	zuk1
U+7B01.0	fMandarin	zhu2
U+7B02.0	fUTF8	笂
U+7B02.0	fGB	E-366E
U+7B02.0	fR/S	 118.3
U+7B02.0	fTotalStrokes	 9
U+7B02.0	fCangjie	HKNI
U+7B02.0	fCantonese	jyun4
U+7B03.0	fUTF8	笃
U+7B03.0	fGB	0-7346
U+7B03.0	fR/S	 118.3
U+7B03.0	fTotalStrokes	 9
U+7B03.0	fCangjie	HNVM
U+7B03.0	fHakka	 duk7
U+7B03.0	fCantonese	duk1
U+7B03.0	fMandarin	du3
U+7B03.0	fEnglish	simplified form of &#x7BE4;
U+7B04.0	fUTF8	笄
U+7B04.0	fCNS11643	2-3162
U+7B04.0	fBig5	D340
U+7B04.0	fGB	0-7347
U+7B04.0	fR/S	 118.6; 118.4
U+7B04.0	fTotalStrokes	 12
U+7B04.0	fCangjie	HMT
U+7B04.0	fFourCorner	88441
U+7B04.0	fHakka	  gi5;  gi1
U+7B04.0	fCantonese	gai1
U+7B04.0	fMandarin	ji1
U+7B04.0	fEnglish	a hairpin for fastening the hair (used by women in former times)
U+7B05.0	fUTF8	笅
U+7B05.0	fCNS11643	2-3164
U+7B05.0	fBig5	D342
U+7B05.0	fGB	5-6279
U+7B05.0	fR/S	 118.4
U+7B05.0	fTotalStrokes	 10
U+7B05.0	fCangjie	HKK
U+7B05.0	fFourCorner	88408
U+7B05.0	fCantonese	gaau2; haau4; ngaau4
U+7B05.0	fMandarin	xiao2; yao2
U+7B06.0	fUTF8	笆
U+7B06.0	fCNS11643	1-573C
U+7B06.0	fBig5	AFB9
U+7B06.0	fGB	0-304A
U+7B06.0	fR/S	 118.4
U+7B06.0	fTotalStrokes	 10
U+7B06.0	fCangjie	HAU
U+7B06.0	fFourCorner	88717
U+7B06.0	fHakka	 ba1; ba3
U+7B06.0	fCantonese	baa1
U+7B06.0	fMandarin	ba1; ba4; ba3
U+7B06.0	fEnglish	 bamboo fence
U+7B07.0	fUTF8	笇
U+7B07.0	fGB	5-627B
U+7B07.0	fR/S	 118.4
U+7B07.0	fTotalStrokes	 10
U+7B07.0	fCangjie	HYY
U+7B07.0	fFourCorner	8843
U+7B07.0	fCantonese	syun3
U+7B07.0	fMandarin	suan4
U+7B08.0	fUTF8	笈
U+7B08.0	fCNS11643	2-3166
U+7B08.0	fBig5	D344
U+7B08.0	fGB	0-7345
U+7B08.0	fR/S	 118.4
U+7B08.0	fTotalStrokes	 10
U+7B08.0	fCangjie	HNHE
U+7B08.0	fFourCorner	88247
U+7B08.0	fHakka	 kip7
U+7B08.0	fCantonese	kap1; kap6
U+7B08.0	fMandarin	ji2; jie2; cha1
U+7B08.0	fEnglish	 bamboo bookcase; bamboo box used to carry books
U+7B09.0	fUTF8	笉
U+7B09.0	fCNS11643	2-3169
U+7B09.0	fBig5	D347
U+7B09.0	fGB	3-6350
U+7B09.0	fR/S	 118.4
U+7B09.0	fTotalStrokes	 10
U+7B09.0	fCangjie	HPMM
U+7B09.0	fFourCorner	88127
U+7B09.0	fCantonese	can2; wan5
U+7B09.0	fMandarin	zhen3; qin3; yun3
U+7B0A.0	fUTF8	笊
U+7B0A.0	fCNS11643	2-3167
U+7B0A.0	fBig5	D345
U+7B0A.0	fGB	0-7349
U+7B0A.0	fR/S	 118.4
U+7B0A.0	fTotalStrokes	 10
U+7B0A.0	fCangjie	HHLO
U+7B0A.0	fFourCorner	88232
U+7B0A.0	fHakka	  zau5;  zau3; gu1
U+7B0A.0	fCantonese	zaau3
U+7B0A.0	fMandarin	zhao4; zhao3
U+7B0A.0	fEnglish	 bamboo skimmer
U+7B0B.0	fUTF8	笋
U+7B0B.0	fCNS11643	3-332B
U+7B0B.0	fGB	0-4B71
U+7B0B.0	fR/S	 118.4
U+7B0B.0	fTotalStrokes	 10
U+7B0B.0	fCangjie	HSK
U+7B0B.0	fFourCorner	88500
U+7B0B.0	fHakka	  sun3; ca5
U+7B0B.0	fCantonese	seon2
U+7B0B.0	fMandarin	sun3
U+7B0B.0	fEnglish	bamboo shoots
U+7B0C.0	fUTF8	笌
U+7B0C.0	fCNS11643	4-2E71
U+7B0C.0	fGB	5-6277
U+7B0C.0	fR/S	 118.4
U+7B0C.0	fTotalStrokes	 10
U+7B0C.0	fCangjie	HMVH
U+7B0C.0	fFourCorner	88240
U+7B0C.0	fCantonese	ngaa4; aa4 (also)
U+7B0C.0	fMandarin	ya2
U+7B0D.0	fUTF8	笍
U+7B0D.0	fCNS11643	3-332A
U+7B0D.0	fGB	3-634C
U+7B0D.0	fR/S	 118.4
U+7B0D.0	fTotalStrokes	 10
U+7B0D.0	fCangjie	HOB
U+7B0D.0	fFourCorner	88220
U+7B0D.0	fCantonese	zeoi3
U+7B0D.0	fMandarin	zhui4; rui4
U+7B0E.0	fUTF8	笎
U+7B0E.0	fCNS11643	2-3168
U+7B0E.0	fBig5	D346
U+7B0E.0	fGB	3-6347
U+7B0E.0	fR/S	 118.4
U+7B0E.0	fTotalStrokes	 10
U+7B0E.0	fCangjie	HMMU
U+7B0E.0	fFourCorner	88211
U+7B0E.0	fCantonese	jyun4
U+7B0E.0	fMandarin	yuan2
U+7B0F.0	fUTF8	笏
U+7B0F.0	fCNS11643	2-3165
U+7B0F.0	fBig5	D343
U+7B0F.0	fGB	0-734B
U+7B0F.0	fR/S	 118.4
U+7B0F.0	fTotalStrokes	 10
U+7B0F.0	fCangjie	HPHH
U+7B0F.0	fFourCorner	88227
U+7B0F.0	fHakka	 fut7;  wut7; vut7
U+7B0F.0	fCantonese	fat1
U+7B0F.0	fMandarin	hu4; wen3; wu4
U+7B0F.0	fEnglish	a tablet held by a civil official during an audience with the monarch
U+7B10.0	fUTF8	笐
U+7B10.0	fCNS11643	2-3161
U+7B10.0	fBig5	D2FE
U+7B10.0	fGB	3-6351
U+7B10.0	fR/S	 118.4
U+7B10.0	fTotalStrokes	 10
U+7B10.0	fCangjie	HYHN
U+7B10.0	fFourCorner	88217
U+7B10.0	fHakka	 gong1; gong2; hong5
U+7B10.0	fCantonese	hong4; hong3
U+7B10.0	fMandarin	gang1; hang4; xiang4
U+7B10.0	fEnglish	bamboos placed across wooden frames on which grain may be stored in damp climates
U+7B11.0	fUTF8	笑
U+7B11.0	fCNS11643	1-573D
U+7B11.0	fBig5	AFBA
U+7B11.0	fGB	0-5026
U+7B11.0	fR/S	 118.4
U+7B11.0	fTotalStrokes	 10
U+7B11.0	fCangjie	HHK
U+7B11.0	fFourCorner	88432
U+7B11.0	fHakka	  siau5; seu5
U+7B11.0	fCantonese	siu3; siu2 (also)
U+7B11.0	fMandarin	xiao4
U+7B11.0	fEnglish	[1] smile; laugh; grin; giggle; snicker; titter; chuckle [2] ridicule; deride; jeer
U+7B12.0	fUTF8	笒
U+7B12.0	fCNS11643	2-316A
U+7B12.0	fBig5	D348
U+7B12.0	fGB	3-634D
U+7B12.0	fR/S	 118.4
U+7B12.0	fTotalStrokes	 10
U+7B12.0	fCangjie	HOIN
U+7B12.0	fFourCorner	88207
U+7B12.0	fCantonese	sam4; gam6; ham4
U+7B12.0	fMandarin	cen2; jin4; han2
U+7B13.0	fUTF8	笓
U+7B13.0	fCNS11643	2-3163
U+7B13.0	fBig5	D341
U+7B13.0	fGB	5-6275
U+7B13.0	fR/S	 118.4
U+7B13.0	fTotalStrokes	 10
U+7B13.0	fCangjie	HPP
U+7B13.0	fFourCorner	88711
U+7B13.0	fHakka	 bi3;  pi5; bin5; bin1; bi5
U+7B13.0	fCantonese	pei4; pai4; bei6
U+7B13.0	fMandarin	pi2; bi4
U+7B13.0	fEnglish	 comb fine-toothed comb
U+7B14.0	fUTF8	笔
U+7B14.0	fCNS11643	3-332C
U+7B14.0	fGB	0-314A
U+7B14.0	fR/S	 118.4
U+7B14.0	fTotalStrokes	 10
U+7B14.0	fCangjie	HHQU
U+7B14.0	fFourCorner	88710
U+7B14.0	fHakka	 bit7
U+7B14.0	fCantonese	bat1
U+7B14.0	fMandarin	bi3
U+7B14.0	fEnglish	simplified form of &#x7B46;
U+7B15.0	fUTF8	笕
U+7B15.0	fGB	0-7348
U+7B15.0	fR/S	 118.4
U+7B15.0	fTotalStrokes	 10
U+7B15.0	fCangjie	HBHU
U+7B15.0	fHakka	 gan3; gen3
U+7B15.0	fCantonese	gaan2; gan2
U+7B15.0	fMandarin	jian3; xian4
U+7B15.0	fEnglish	simplified form of &#x7B67;
U+7B16.0	fUTF8	笖
U+7B16.0	fCNS11643	3-3778
U+7B16.0	fGB	3-6353
U+7B16.0	fR/S	  118.5
U+7B16.0	fTotalStrokes	 11
U+7B16.0	fCangjie	XHVIO
U+7B16.0	fFourCorner	88180
U+7B16.0	fCantonese	ji5
U+7B16.0	fMandarin	yi3
U+7B17.0	fUTF8	笗
U+7B17.0	fCNS11643	3-3779
U+7B17.0	fGB	5-6326
U+7B17.0	fR/S	  118.5
U+7B17.0	fTotalStrokes	 11
U+7B17.0	fCangjie	HHEY
U+7B17.0	fFourCorner	88300
U+7B17.0	fCantonese	dung1
U+7B17.0	fMandarin	dong1
U+7B18.0	fUTF8	笘
U+7B18.0	fCNS11643	2-3776
U+7B18.0	fBig5	D6D3
U+7B18.0	fGB	3-6357
U+7B18.0	fR/S	 118.5
U+7B18.0	fTotalStrokes	 11
U+7B18.0	fCangjie	HYR
U+7B18.0	fFourCorner	88601
U+7B18.0	fCantonese	sim1; sim3; dip3
U+7B18.0	fMandarin	shan1; can1; chan1; die2; tie1
U+7B19.0	fUTF8	笙
U+7B19.0	fCNS11643	1-5C4B
U+7B19.0	fBig5	B2C6
U+7B19.0	fGB	0-734F
U+7B19.0	fR/S	 118.5
U+7B19.0	fTotalStrokes	 11
U+7B19.0	fCangjie	HHQM
U+7B19.0	fFourCorner	88104
U+7B19.0	fHakka	 sen1; sang1
U+7B19.0	fCantonese	saang1 (also); sang1
U+7B19.0	fMandarin	sheng1; xin1
U+7B19.0	fEnglish	small gourd-shaped musical instrument with 13 reeds
U+7B1A.0	fUTF8	笚
U+7B1A.0	fCNS11643	2-3821
U+7B1A.0	fBig5	D6DC
U+7B1A.0	fGB	3-635A
U+7B1A.0	fR/S	 118.5
U+7B1A.0	fTotalStrokes	 11
U+7B1A.0	fCangjie	HWL
U+7B1A.0	fFourCorner	88500
U+7B1A.0	fHakka	 hap8
U+7B1A.0	fCantonese	daap3; haap6
U+7B1A.0	fMandarin	xia2; da1
U+7B1B.0	fUTF8	笛
U+7B1B.0	fCNS11643	1-5C48
U+7B1B.0	fBig5	B2C3
U+7B1B.0	fGB	0-3551
U+7B1B.0	fR/S	 118.5
U+7B1B.0	fTotalStrokes	 11
U+7B1B.0	fCangjie	HLW
U+7B1B.0	fFourCorner	88603
U+7B1B.0	fHakka	  tak8;  tet8; tit8
U+7B1B.0	fCantonese	dek6; dek2 (also)
U+7B1B.0	fMandarin	di2
U+7B1B.0	fEnglish	 bamboo flute; whistle
U+7B1C.0	fUTF8	笜
U+7B1C.0	fCNS11643	3-3777
U+7B1C.0	fGB	E-366F
U+7B1C.0	fR/S	 118.5
U+7B1C.0	fTotalStrokes	 11
U+7B1C.0	fCangjie	HUU
U+7B1C.0	fFourCorner	88770
U+7B1C.0	fCantonese	zeot1
U+7B1C.0	fMandarin	zhu2
U+7B1D.0	fUTF8	笝
U+7B1D.0	fCNS11643	2-3778
U+7B1D.0	fBig5	D6D5
U+7B1D.0	fGB	5-6321
U+7B1D.0	fR/S	 118.5
U+7B1D.0	fTotalStrokes	 11
U+7B1D.0	fCangjie	HWE
U+7B1D.0	fFourCorner	88603
U+7B1D.0	fHakka	 nap8
U+7B1D.0	fCantonese	naap6; laap6 (also)
U+7B1D.0	fMandarin	na4
U+7B1D.0	fEnglish	rope; hawser
U+7B1E.0	fUTF8	笞
U+7B1E.0	fCNS11643	1-5C4C
U+7B1E.0	fBig5	B2C7
U+7B1E.0	fGB	0-7357
U+7B1E.0	fR/S	 118.5
U+7B1E.0	fTotalStrokes	 11
U+7B1E.0	fCangjie	HIR
U+7B1E.0	fFourCorner	88603
U+7B1E.0	fHakka	 chi1; ci1; cih1; chii1
U+7B1E.0	fCantonese	ci1
U+7B1E.0	fMandarin	chi1
U+7B1E.0	fEnglish	[1] bamboo rod used for beatings; bamboo rod [2] whip; flog
U+7B1F.0	fUTF8	笟
U+7B1F.0	fCNS11643	3-377B
U+7B1F.0	fGB	5-6322
U+7B1F.0	fR/S	 118.5; 118.6
U+7B1F.0	fTotalStrokes	 11
U+7B1F.0	fCangjie	HHVO
U+7B1F.0	fFourCorner	88230
U+7B1F.0	fHakka	 gu1
U+7B1F.0	fCantonese	ku1; kwu1 (also)
U+7B1F.0	fMandarin	gu1
U+7B1F.0	fEnglish	[1] a kind of trumpet [2] whistle
U+7B20.0	fUTF8	笠
U+7B20.0	fCNS11643	1-5C46
U+7B20.0	fBig5	B2C1
U+7B20.0	fGB	0-7352
U+7B20.0	fR/S	 118.5
U+7B20.0	fTotalStrokes	 11
U+7B20.0	fCangjie	HYT
U+7B20.0	fFourCorner	88108
U+7B20.0	fHakka	  lip7;  lep7; lit7; lap7
U+7B20.0	fCantonese	lap1; lap6
U+7B20.0	fMandarin	li4
U+7B20.0	fEnglish	[1] bamboo hat [2] bamboo covering; bamboo shade
U+7B21.0	fUTF8	笡
U+7B21.0	fCNS11643	3-3776
U+7B21.0	fGB	3-6359
U+7B21.0	fR/S	 118.5
U+7B21.0	fTotalStrokes	 11
U+7B21.0	fCangjie	HBM
U+7B21.0	fFourCorner	88100
U+7B21.0	fCantonese	ce3
U+7B21.0	fMandarin	qie4
U+7B22.0	fUTF8	笢
U+7B22.0	fCNS11643	2-3773
U+7B22.0	fBig5	D6D0
U+7B22.0	fGB	3-635F
U+7B22.0	fR/S	 118.5
U+7B22.0	fTotalStrokes	 11
U+7B22.0	fCangjie	HRVP
U+7B22.0	fFourCorner	88747
U+7B22.0	fHakka	 men3; min2
U+7B22.0	fCantonese	man5; man4
U+7B22.0	fMandarin	min3; min2
U+7B22.0	fEnglish	the outer skin of the bamboo
U+7B23.0	fUTF8	笣
U+7B23.0	fCNS11643	2-3822
U+7B23.0	fBig5	D6DD
U+7B23.0	fGB	5-6328
U+7B23.0	fR/S	 118.5
U+7B23.0	fTotalStrokes	 11
U+7B23.0	fCangjie	HPRU
U+7B23.0	fFourCorner	88712
U+7B23.0	fHakka	 bau1
U+7B23.0	fCantonese	baau1
U+7B23.0	fMandarin	bao1; qia4
U+7B24.0	fUTF8	笤
U+7B24.0	fCNS11643	2-3774
U+7B24.0	fBig5	D6D1
U+7B24.0	fGB	0-7354
U+7B24.0	fR/S	 118.5
U+7B24.0	fTotalStrokes	 11
U+7B24.0	fCangjie	HSHR
U+7B24.0	fFourCorner	88602
U+7B24.0	fHakka	  tiau2
U+7B24.0	fCantonese	tiu4
U+7B24.0	fMandarin	tiao2; shao4
U+7B24.0	fEnglish	 bamboo broom
U+7B25.0	fUTF8	笥
U+7B25.0	fCNS11643	2-3771
U+7B25.0	fBig5	D6CE
U+7B25.0	fGB	0-7353
U+7B25.0	fR/S	 118.5
U+7B25.0	fTotalStrokes	 11
U+7B25.0	fCangjie	HSMR
U+7B25.0	fFourCorner	88627
U+7B25.0	fHakka	 sii5; sii1; {shi3}; sii3; se5
U+7B25.0	fCantonese	zi6; si3
U+7B25.0	fMandarin	si4; si1; si3
U+7B25.0	fEnglish	 bamboo basket; bamboo chest; bamboo box
U+7B26.0	fUTF8	符
U+7B26.0	fCNS11643	1-5C4A
U+7B26.0	fBig5	B2C5
U+7B26.0	fGB	0-377B
U+7B26.0	fR/S	 118.5
U+7B26.0	fTotalStrokes	 11
U+7B26.0	fCangjie	HODI
U+7B26.0	fFourCorner	88243
U+7B26.0	fHakka	  pu2;  fu2; pu3
U+7B26.0	fCantonese	fu4
U+7B26.0	fMandarin	fu2
U+7B26.0	fEnglish	[1] tag carried for identification (as a warrant, etc.); identification tag; label [2] match; tally [3] charm; talisman; spell [4] auspicious omen [5] symbol; sign [6] a Chinese family name
U+7B27.0	fUTF8	笧
U+7B27.0	fCNS11643	4-337C
U+7B27.0	fGB	5-6325
U+7B27.0	fR/S	 118.5
U+7B27.0	fTotalStrokes	 11
U+7B27.0	fCangjie	HBT
U+7B27.0	fFourCorner	88440
U+7B27.0	fCantonese	caak3
U+7B27.0	fMandarin	ce4
U+7B27.0	fEnglish	to entwine around; check (a current) with a weir
U+7B28.0	fUTF8	笨
U+7B28.0	fCNS11643	1-5C47
U+7B28.0	fBig5	B2C2
U+7B28.0	fGB	0-313F
U+7B28.0	fR/S	 118.5
U+7B28.0	fTotalStrokes	 11
U+7B28.0	fCangjie	HDM
U+7B28.0	fFourCorner	88234
U+7B28.0	fHakka	  bun5
U+7B28.0	fCantonese	ban6
U+7B28.0	fMandarin	ben4; ben3
U+7B28.0	fEnglish	[1] foolish; stupid; dull [2] clumsy; awkward
U+7B29.0	fUTF8	笩
U+7B29.0	fCNS11643	4-337A
U+7B29.0	fGB	E-3670
U+7B29.0	fR/S	 118.5
U+7B29.0	fTotalStrokes	 11
U+7B29.0	fCangjie	HOIP
U+7B29.0	fFourCorner	88240
U+7B29.0	fCantonese	fat6
U+7B29.0	fMandarin	pei4; fa2
U+7B2A.0	fUTF8	笪
U+7B2A.0	fCNS11643	2-3777
U+7B2A.0	fBig5	D6D4
U+7B2A.0	fGB	0-734E
U+7B2A.0	fR/S	 118.5
U+7B2A.0	fTotalStrokes	 11
U+7B2A.0	fCangjie	HAM
U+7B2A.0	fFourCorner	88106
U+7B2A.0	fHakka	  dat7; tan3
U+7B2A.0	fCantonese	daat3
U+7B2A.0	fMandarin	da2; dan3; dan4
U+7B2A.0	fEnglish	[1] a coarse mat of rushes or bamboo [2] numerary classifier for places and lands
U+7B2B.0	fUTF8	笫
U+7B2B.0	fCNS11643	2-377A
U+7B2B.0	fBig5	D6D7
U+7B2B.0	fGB	0-734A
U+7B2B.0	fR/S	 118.5; 118.4
U+7B2B.0	fTotalStrokes	 11
U+7B2B.0	fCangjie	HLXH
U+7B2B.0	fFourCorner	88227
U+7B2B.0	fHakka	 zi3
U+7B2B.0	fCantonese	zi2; ji2 (also); dai6
U+7B2B.0	fMandarin	zi3; yi2; jie2; zhi4
U+7B2B.0	fEnglish	 bed; the planks of a Chinese bed; a sleeping mat
U+7B2C.0	fUTF8	第
U+7B2C.0	fCNS11643	1-5C49
U+7B2C.0	fBig5	B2C4
U+7B2C.0	fGB	0-355A
U+7B2C.0	fR/S	 118.5
U+7B2C.0	fTotalStrokes	 11
U+7B2C.0	fCangjie	HNLH
U+7B2C.0	fFourCorner	88227
U+7B2C.0	fHakka	  ti5; ti3
U+7B2C.0	fCantonese	dai6; dai2 (also)
U+7B2C.0	fMandarin	di4
U+7B2C.0	fEnglish	 sequence; number grade; degree
U+7B2D.0	fUTF8	笭
U+7B2D.0	fCNS11643	2-377B
U+7B2D.0	fBig5	D6D8
U+7B2D.0	fGB	3-635C
U+7B2D.0	fR/S	 118.5
U+7B2D.0	fTotalStrokes	 11
U+7B2D.0	fCangjie	HOII
U+7B2D.0	fFourCorner	88207
U+7B2D.0	fHakka	  lin2
U+7B2D.0	fCantonese	ling4; ling6
U+7B2D.0	fMandarin	ling2; ling3; ling4
U+7B2D.0	fEnglish	bamboo screen
U+7B2E.0	fUTF8	笮
U+7B2E.0	fCNS11643	1-5C4D
U+7B2E.0	fBig5	B2C8
U+7B2E.0	fGB	0-7350
U+7B2E.0	fR/S	 118.5
U+7B2E.0	fTotalStrokes	 11
U+7B2E.0	fCangjie	HOS
U+7B2E.0	fFourCorner	88211
U+7B2E.0	fHakka	 zak7; cok7; zet7
U+7B2E.0	fCantonese	zaak3; zok6; zaa3; zok3; zaak6
U+7B2E.0	fMandarin	ze2; zuo2; zuo4
U+7B2E.0	fEnglish	[1] pressing [2] boards laid across rafters [3] arrow bag
U+7B2E.1	fUTF8	笮
U+7B2E.1	fCNS11643	1-5C4D
U+7B2E.1	fBig5	B2C8
U+7B2E.1	fGB	0-7350
U+7B2E.1	fR/S	 118.5
U+7B2E.1	fTotalStrokes	 11
U+7B2E.1	fCangjie	HOS
U+7B2E.1	fFourCorner	88211
U+7B2E.1	fMandarin	zha4
U+7B2E.1	fEnglish	[1] a kind of liquid container [2] squeeze; press
U+7B2F.0	fUTF8	笯
U+7B2F.0	fCNS11643	2-377C
U+7B2F.0	fBig5	D6D9
U+7B2F.0	fGB	3-6361
U+7B2F.0	fR/S	 118.5
U+7B2F.0	fTotalStrokes	 11
U+7B2F.0	fCangjie	HVE
U+7B2F.0	fFourCorner	88447
U+7B2F.0	fCantonese	nou4; lou4 (also)
U+7B2F.0	fMandarin	nu2; na2; nu4
U+7B2F.0	fEnglish	 bird cage
U+7B30.0	fUTF8	笰
U+7B30.0	fCNS11643	2-3772
U+7B30.0	fBig5	D6CF
U+7B30.0	fGB	3-6360
U+7B30.0	fR/S	 118.5
U+7B30.0	fTotalStrokes	 11
U+7B30.0	fCangjie	HLLN
U+7B30.0	fFourCorner	88527
U+7B30.0	fHakka	  fut7
U+7B30.0	fCantonese	fat1
U+7B30.0	fMandarin	fu2; fei4
U+7B30.0	fEnglish	curtain
U+7B31.0	fUTF8	笱
U+7B31.0	fCNS11643	2-3779
U+7B31.0	fBig5	D6D6
U+7B31.0	fGB	0-7351
U+7B31.0	fR/S	 118.5
U+7B31.0	fTotalStrokes	 11
U+7B31.0	fCangjie	HPR
U+7B31.0	fFourCorner	88627
U+7B31.0	fHakka	 kieu3; go2; ho2; geu3; giu3
U+7B31.0	fCantonese	gau2
U+7B31.0	fMandarin	gou3
U+7B31.0	fEnglish	a basket trap for fish; placed in the opening of a weir
U+7B32.0	fUTF8	笲
U+7B32.0	fCNS11643	2-377D
U+7B32.0	fBig5	D6DA
U+7B32.0	fGB	3-6362
U+7B32.0	fR/S	 118.5
U+7B32.0	fTotalStrokes	 11
U+7B32.0	fCangjie	HIT
U+7B32.0	fFourCorner	88441
U+7B32.0	fHakka	 fan2; fan1
U+7B32.0	fCantonese	faan4; bin6
U+7B32.0	fMandarin	fan2; fan1; bian4; fan4
U+7B32.0	fEnglish	basket
U+7B33.0	fUTF8	笳
U+7B33.0	fCNS11643	2-3775
U+7B33.0	fBig5	D6D2
U+7B33.0	fGB	0-7355
U+7B33.0	fR/S	 118.5
U+7B33.0	fTotalStrokes	 11
U+7B33.0	fCangjie	HKSR
U+7B33.0	fFourCorner	88463
U+7B33.0	fHakka	 ga1
U+7B33.0	fCantonese	gaa1
U+7B33.0	fMandarin	jia1
U+7B33.0	fEnglish	 reed leaf whistle; the Tartar pipe
U+7B34.0	fUTF8	笴
U+7B34.0	fCNS11643	2-3770
U+7B34.0	fBig5	D6CD
U+7B34.0	fGB	5-627E
U+7B34.0	fR/S	 118.5
U+7B34.0	fTotalStrokes	 11
U+7B34.0	fCangjie	HMNR
U+7B34.0	fFourCorner	88621
U+7B34.0	fHakka	 ko3
U+7B34.0	fCantonese	go2; gon2 (also); gou2
U+7B34.0	fMandarin	ge3; gao3; gan3
U+7B34.0	fEnglish	 stem of an arrow
U+7B35.0	fUTF8	笵
U+7B35.0	fCNS11643	2-376E
U+7B35.0	fBig5	D6CB
U+7B35.0	fGB	5-6329
U+7B35.0	fR/S	 118.5
U+7B35.0	fTotalStrokes	 11
U+7B35.0	fCangjie	HESU
U+7B35.0	fFourCorner	88112
U+7B35.0	fCantonese	faan6
U+7B35.0	fMandarin	fan4
U+7B35.0	fEnglish	[1] bamboo form [2] same as $ (model)
U+7B36.0	fUTF8	笶
U+7B36.0	fCNS11643	3-377A
U+7B36.0	fGB	E-3671
U+7B36.0	fR/S	 118.5
U+7B36.0	fTotalStrokes	 11
U+7B36.0	fCangjie	HOK
U+7B36.0	fFourCorner	88430
U+7B36.0	fCantonese	ci2
U+7B36.0	fMandarin	shi3
U+7B37.0	fUTF8	笷
U+7B37.0	fCNS11643	3-657D
U+7B37.0	fGB	E-3672
U+7B37.0	fR/S	 118.5
U+7B37.0	fTotalStrokes	 11
U+7B37.0	fCangjie	HHHL
U+7B37.0	fFourCorner	8872
U+7B37.0	fCantonese	maau5
U+7B37.0	fMandarin	mao3
U+7B38.0	fUTF8	笸
U+7B38.0	fCNS11643	2-377E
U+7B38.0	fBig5	D6DB
U+7B38.0	fGB	0-734D
U+7B38.0	fR/S	 118.5
U+7B38.0	fTotalStrokes	 11
U+7B38.0	fCangjie	HSR
U+7B38.0	fFourCorner	88716
U+7B38.0	fHakka	 bo1; po3
U+7B38.0	fCantonese	po2
U+7B38.0	fMandarin	po3
U+7B38.0	fEnglish	flat basket for grain
U+7B39.0	fUTF8	笹
U+7B39.0	fCNS11643	F-374D
U+7B39.0	fGB	E-3673
U+7B39.0	fR/S	 118.5
U+7B39.0	fTotalStrokes	 11
U+7B39.0	fCangjie	HPT
U+7B39.0	fFourCorner	8871
U+7B39.0	fCantonese	sai3
U+7B39.0	fEnglish	small bamboo
U+7B3A.0	fUTF8	笺
U+7B3A.0	fGB	0-3C63
U+7B3A.0	fR/S	 118.5
U+7B3A.0	fTotalStrokes	 11
U+7B3A.0	fCangjie	HIJ
U+7B3A.0	fHakka	 zien1
U+7B3A.0	fCantonese	zin1
U+7B3A.0	fMandarin	jian1
U+7B3A.0	fEnglish	simplified form of &#x7B8B;
U+7B3B.0	fUTF8	笻
U+7B3B.0	fCNS11643	2-376F
U+7B3B.0	fBig5	DADF
U+7B3B.0	fGB	5-627C
U+7B3B.0	fR/S	 118.5
U+7B3B.0	fTotalStrokes	 11
U+7B3B.0	fCangjie	HMSL
U+7B3B.0	fFourCorner	88127
U+7B3B.0	fCantonese	kung4; kung1
U+7B3B.0	fMandarin	qiong2; zhu2
U+7B3C.0	fUTF8	笼
U+7B3C.0	fGB	0-417D
U+7B3C.0	fR/S	 118.5
U+7B3C.0	fTotalStrokes	 11
U+7B3C.0	fCangjie	HIKP
U+7B3C.0	fHakka	 lung3; lung1
U+7B3C.0	fCantonese	lung4; lung5; lung2 (also)
U+7B3C.0	fMandarin	long2
U+7B3C.0	fEnglish	simplified form of &#x7C60;
U+7B3C.1	fUTF8	笼
U+7B3C.1	fGB	0-417D
U+7B3C.1	fR/S	 118.5
U+7B3C.1	fTotalStrokes	 11
U+7B3C.1	fCangjie	HIKP
U+7B3C.1	fMandarin	long3
U+7B3C.2	fUTF8	笼
U+7B3C.2	fGB	0-417D
U+7B3C.2	fR/S	 118.5
U+7B3C.2	fTotalStrokes	 11
U+7B3C.2	fCangjie	HIKP
U+7B3C.2	fMandarin	long5
U+7B3D.0	fUTF8	笽
U+7B3D.0	fCNS11643	F-367E
U+7B3D.0	fGB	E-3674
U+7B3D.0	fR/S	 118.5; 108.6
U+7B3D.0	fTotalStrokes	 11
U+7B3D.0	fCangjie	XXHBT
U+7B3D.0	fCantonese	ming5
U+7B3E.0	fUTF8	笾
U+7B3E.0	fGB	0-7356
U+7B3E.0	fR/S	 118.5; 118.6
U+7B3E.0	fTotalStrokes	 11
U+7B3E.0	fCangjie	HYKS
U+7B3E.0	fHakka	 bien1
U+7B3E.0	fCantonese	bin1
U+7B3E.0	fMandarin	bian1
U+7B3E.0	fEnglish	simplified form of &#x7C69;
U+7B3F.0	fUTF8	笿
U+7B3F.0	fCNS11643	5-3B24
U+7B3F.0	fGB	3-6372
U+7B3F.0	fR/S	  118.6
U+7B3F.0	fTotalStrokes	 12
U+7B3F.0	fCangjie	HHER
U+7B3F.0	fFourCorner	88600
U+7B3F.0	fCantonese	lok3
U+7B3F.0	fMandarin	luo4
U+7B40.0	fUTF8	筀
U+7B40.0	fCNS11643	2-3E68
U+7B40.0	fBig5	DAE4
U+7B40.0	fGB	5-632B
U+7B40.0	fR/S	 118.6
U+7B40.0	fTotalStrokes	 12
U+7B40.0	fCangjie	HGG
U+7B40.0	fFourCorner	88104
U+7B40.0	fHakka	 gui5; gwi5; kwui5
U+7B40.0	fCantonese	gai3; gwai3
U+7B40.0	fMandarin	gui4
U+7B40.0	fEnglish	a fine kind of bamboo growing in Kwangtung and in Chehkiang
U+7B41.0	fUTF8	筁
U+7B41.0	fCNS11643	3-3D7B
U+7B41.0	fGB	E-3675
U+7B41.0	fR/S	 118.6
U+7B41.0	fTotalStrokes	 12
U+7B41.0	fCangjie	HTW
U+7B41.0	fFourCorner	88600
U+7B41.0	fHakka	  kiuk7
U+7B41.0	fCantonese	kuk1
U+7B41.0	fMandarin	qu3; qu1
U+7B41.0	fEnglish	a bamboo frame on which silkworms spin their cocoons
U+7B42.0	fUTF8	筂
U+7B42.0	fCNS11643	3-3D78
U+7B42.0	fGB	E-3676
U+7B42.0	fR/S	  118.6
U+7B42.0	fTotalStrokes	 12
U+7B42.0	fCangjie	HEPD
U+7B42.0	fFourCorner	88110
U+7B42.0	fCantonese	ci4
U+7B42.0	fMandarin	chi2
U+7B43.0	fUTF8	筃
U+7B43.0	fCNS11643	3-3D75
U+7B43.0	fGB	3-6368
U+7B43.0	fR/S	 118.6
U+7B43.0	fTotalStrokes	 12
U+7B43.0	fCangjie	HWK
U+7B43.0	fFourCorner	88600
U+7B43.0	fCantonese	jan1
U+7B43.0	fMandarin	yin1
U+7B44.0	fUTF8	筄
U+7B44.0	fCNS11643	2-3E64
U+7B44.0	fBig5	DAE0
U+7B44.0	fGB	3-6370
U+7B44.0	fR/S	 118.6
U+7B44.0	fTotalStrokes	 12
U+7B44.0	fCangjie	HLMO
U+7B44.0	fFourCorner	88113
U+7B44.0	fHakka	 jau5; rau6; jeu5
U+7B44.0	fCantonese	jiu6
U+7B44.0	fMandarin	yao4; yao2; you2
U+7B44.0	fEnglish	the laths over rafters, on which the tiles are laid
U+7B45.0	fUTF8	筅
U+7B45.0	fCNS11643	2-3E6A
U+7B45.0	fBig5	DAE6
U+7B45.0	fGB	0-735A
U+7B45.0	fR/S	 118.6
U+7B45.0	fTotalStrokes	 12
U+7B45.0	fCangjie	HHGU
U+7B45.0	fFourCorner	88211
U+7B45.0	fHakka	  sien3
U+7B45.0	fCantonese	sin2
U+7B45.0	fMandarin	xian3
U+7B45.0	fEnglish	 bamboo brush halberd
U+7B46.0	fUTF8	筆
U+7B46.0	fCNS11643	1-612D
U+7B46.0	fBig5	B5A7
U+7B46.0	fGB	1-314A
U+7B46.0	fR/S	 118.6
U+7B46.0	fTotalStrokes	 12
U+7B46.0	fCangjie	HLQ
U+7B46.0	fFourCorner	88507
U+7B46.0	fHakka	  bit7
U+7B46.0	fCantonese	bat1
U+7B46.0	fMandarin	bi3
U+7B46.0	fEnglish	[1] writing brush; pen; pencil [2] write [3] stroke; touch [4] writer's skill or style [5] a unit of amount [6] prose [7] a Chinese family name
U+7B47.0	fUTF8	筇
U+7B47.0	fCNS11643	2-3E63
U+7B47.0	fBig5	D6CC
U+7B47.0	fGB	0-734C
U+7B47.0	fR/S	 118.6
U+7B47.0	fTotalStrokes	 12
U+7B47.0	fCangjie	HMNL
U+7B47.0	fFourCorner	88127
U+7B47.0	fHakka	  kiung2
U+7B47.0	fCantonese	kung4
U+7B47.0	fMandarin	qiong2
U+7B47.0	fEnglish	a kind of bamboo commonly used to make walking sticks
U+7B48.0	fUTF8	筈
U+7B48.0	fCNS11643	2-3E65
U+7B48.0	fBig5	DAE1
U+7B48.0	fGB	3-636C
U+7B48.0	fR/S	 118.6
U+7B48.0	fTotalStrokes	 12
U+7B48.0	fCangjie	HHJR
U+7B48.0	fFourCorner	88604
U+7B48.0	fHakka	 kwat7; gat7
U+7B48.0	fCantonese	kut3; kwut3 (also)
U+7B48.0	fMandarin	gua1; kuo4
U+7B48.0	fEnglish	 end of an arrow
U+7B49.0	fUTF8	等
U+7B49.0	fCNS11643	1-612B
U+7B49.0	fBig5	B5A5
U+7B49.0	fGB	0-3548
U+7B49.0	fR/S	 118.6
U+7B49.0	fTotalStrokes	 12
U+7B49.0	fCangjie	HGDI
U+7B49.0	fFourCorner	88341
U+7B49.0	fHakka	  den3; den1; dien3
U+7B49.0	fCantonese	dang2
U+7B49.0	fMandarin	deng3
U+7B49.0	fEnglish	[1] rank; grade [2] same; equal; etcetera [3] wait [4] when; till [5] and so on; etcetera; and the like [6] common
U+7B4A.0	fUTF8	筊
U+7B4A.0	fCNS11643	2-3E62
U+7B4A.0	fBig5	DADE
U+7B4A.0	fGB	3-6374
U+7B4A.0	fR/S	 118.6
U+7B4A.0	fTotalStrokes	 12
U+7B4A.0	fCangjie	HYCK
U+7B4A.0	fFourCorner	88408
U+7B4A.0	fHakka	  gau3;  gau5;  gau1; kau5
U+7B4A.0	fCantonese	gaau2; gaau3; ngaau4
U+7B4A.0	fMandarin	jiao3; xiao2
U+7B4A.0	fEnglish	[1] bamboo rope (made of bamboo strips) [2] a kind of bamboo device used in divination
U+7B4B.0	fUTF8	筋
U+7B4B.0	fCNS11643	1-6132
U+7B4B.0	fBig5	B5AC
U+7B4B.0	fGB	0-3D6E
U+7B4B.0	fR/S	 118.6
U+7B4B.0	fTotalStrokes	 12
U+7B4B.0	fCangjie	HBKS
U+7B4B.0	fFourCorner	88227
U+7B4B.0	fHakka	  gin1; gen1
U+7B4B.0	fCantonese	gan1
U+7B4B.0	fMandarin	jin1; qian2
U+7B4B.0	fEnglish	[1] muscles; tendons; sinews [2] veins that stand out under the skin [3] plant fibers resembling tendon
U+7B4C.0	fUTF8	筌
U+7B4C.0	fCNS11643	2-3E66
U+7B4C.0	fBig5	DAE2
U+7B4C.0	fGB	0-735C
U+7B4C.0	fR/S	 118.6
U+7B4C.0	fTotalStrokes	 12
U+7B4C.0	fCangjie	HOMG
U+7B4C.0	fFourCorner	88104
U+7B4C.0	fHakka	  cien2; cion2; cen2
U+7B4C.0	fCantonese	cyun4
U+7B4C.0	fMandarin	quan2
U+7B4C.0	fEnglish	 bamboo fish trap
U+7B4D.0	fUTF8	筍
U+7B4D.0	fCNS11643	1-6131
U+7B4D.0	fBig5	B5AB
U+7B4D.0	fGB	E-3677
U+7B4D.0	fR/S	 118.6
U+7B4D.0	fTotalStrokes	 12
U+7B4D.0	fCangjie	HPA
U+7B4D.0	fFourCorner	88627
U+7B4D.0	fHakka	 sun3
U+7B4D.0	fCantonese	seon2
U+7B4D.0	fMandarin	sun3
U+7B4D.0	fEnglish	bamboo shoot; bamboo sprout
U+7B4E.0	fUTF8	筎
U+7B4E.0	fCNS11643	2-3E67
U+7B4E.0	fBig5	DAE3
U+7B4E.0	fGB	3-6377
U+7B4E.0	fR/S	 118.6
U+7B4E.0	fTotalStrokes	 12
U+7B4E.0	fCangjie	HVR
U+7B4E.0	fFourCorner	88463
U+7B4E.0	fHakka	 ji2; ri2
U+7B4E.0	fCantonese	jyu4
U+7B4E.0	fMandarin	ru2
U+7B4E.0	fEnglish	the thin outside skin of the bamboo
U+7B4F.0	fUTF8	筏
U+7B4F.0	fCNS11643	1-6133
U+7B4F.0	fBig5	B5AD
U+7B4F.0	fGB	0-3724
U+7B4F.0	fR/S	 118.6
U+7B4F.0	fTotalStrokes	 12
U+7B4F.0	fCangjie	HOI
U+7B4F.0	fFourCorner	88253
U+7B4F.0	fHakka	  fat8
U+7B4F.0	fCantonese	fat6
U+7B4F.0	fMandarin	fa2
U+7B4F.0	fEnglish	 raft
U+7B50.0	fUTF8	筐
U+7B50.0	fCNS11643	1-612E
U+7B50.0	fBig5	B5A8
U+7B50.0	fGB	0-3F70
U+7B50.0	fR/S	 118.6
U+7B50.0	fTotalStrokes	 12
U+7B50.0	fCangjie	HSMG
U+7B50.0	fFourCorner	88711
U+7B50.0	fHakka	  kiong1; kian1
U+7B50.0	fCantonese	hong1; kwaang1 (also); kwaang2
U+7B50.0	fMandarin	kuang1
U+7B50.0	fEnglish	 rectangular bamboo box or chest; shallow basket
U+7B51.0	fUTF8	筑
U+7B51.0	fCNS11643	1-6134
U+7B51.0	fBig5	B5AE
U+7B51.0	fGB	0-567E
U+7B51.0	fR/S	 118.6
U+7B51.0	fTotalStrokes	 12
U+7B51.0	fCangjie	HMHN
U+7B51.0	fFourCorner	88117
U+7B51.0	fHakka	 zuk7; chuk7; zhuk7
U+7B51.0	fCantonese	zuk1
U+7B51.0	fMandarin	zhu2; zhu4
U+7B51.0	fEnglish	[1] a kind of lute with five strings, thirteen strings, or twenty one strings [2] simplified form of &#x7BC9;
U+7B52.0	fUTF8	筒
U+7B52.0	fCNS11643	1-612F
U+7B52.0	fBig5	B5A9
U+7B52.0	fGB	0-4D32
U+7B52.0	fR/S	 118.6
U+7B52.0	fTotalStrokes	 12
U+7B52.0	fCangjie	HBMR
U+7B52.0	fFourCorner	88227
U+7B52.0	fHakka	  tung2; tung3
U+7B52.0	fCantonese	tung4; tung2 (also); tung1 (also)
U+7B52.0	fMandarin	tong3; tong2; dong4
U+7B52.0	fEnglish	 tube; pipe; cylinder; thick piece of bamboo
U+7B53.0	fUTF8	筓
U+7B53.0	fCNS11643	3-3D79
U+7B53.0	fGB	E-3678
U+7B53.0	fR/S	 118.6
U+7B53.0	fTotalStrokes	 12
U+7B53.0	fCangjie	HMJJ
U+7B53.0	fFourCorner	88440
U+7B53.0	fHakka	 gi5; gi1
U+7B53.0	fCantonese	gai1
U+7B53.0	fMandarin	ji1
U+7B53.0	fEnglish	a hairpin on which the hair is bound at the back of the head; 15 of age
U+7B54.0	fUTF8	答
U+7B54.0	fCNS11643	1-6130
U+7B54.0	fBig5	B5AA
U+7B54.0	fGB	0-3470
U+7B54.0	fR/S	 118.6
U+7B54.0	fTotalStrokes	 12
U+7B54.0	fCangjie	HOMR
U+7B54.0	fFourCorner	88601
U+7B54.0	fHakka	  dap7
U+7B54.0	fCantonese	daap3
U+7B54.0	fMandarin	da2
U+7B54.0	fEnglish	[1] answer; reply [2] reciprocate; return
U+7B54.1	fUTF8	答
U+7B54.1	fCNS11643	1-6130
U+7B54.1	fBig5	B5AA
U+7B54.1	fGB	0-3470
U+7B54.1	fR/S	 118.6
U+7B54.1	fTotalStrokes	 12
U+7B54.1	fCangjie	HOMR
U+7B54.1	fFourCorner	88601
U+7B54.1	fMandarin	da1
U+7B54.1	fEnglish	a variant sound used only in some phrases
U+7B55.0	fUTF8	筕
U+7B55.0	fCNS11643	3-3D72
U+7B55.0	fGB	3-636F
U+7B55.0	fR/S	 118.6
U+7B55.0	fTotalStrokes	 12
U+7B55.0	fCangjie	HHON
U+7B55.0	fFourCorner	88220
U+7B55.0	fHakka	 hang2
U+7B55.0	fCantonese	hang4; hong4 (also)
U+7B55.0	fMandarin	xing2; hang2
U+7B55.0	fEnglish	coarse bamboo mat to spread on the ground
U+7B56.0	fUTF8	策
U+7B56.0	fCNS11643	1-612C
U+7B56.0	fBig5	B5A6
U+7B56.0	fGB	0-325F
U+7B56.0	fR/S	 118.6
U+7B56.0	fTotalStrokes	 12
U+7B56.0	fCangjie	HDB
U+7B56.0	fFourCorner	88902
U+7B56.0	fHakka	 cak7;  cet7; ciet7
U+7B56.0	fCantonese	caak3
U+7B56.0	fMandarin	ce4
U+7B56.0	fEnglish	[1] scheme; plan [2] whip (for goading horses) [3] orders for appointment [4] expository writings on government affairs [5] whip; spur; impel; urge [6] a Chinese family name
U+7B57.0	fUTF8	筗
U+7B57.0	fCNS11643	3-3D76
U+7B57.0	fGB	5-6333
U+7B57.0	fR/S	  118.6
U+7B57.0	fTotalStrokes	 12
U+7B57.0	fCangjie	HOL
U+7B57.0	fFourCorner	88250
U+7B57.0	fCantonese	zung6
U+7B57.0	fMandarin	zhong4
U+7B58.0	fUTF8	筘
U+7B58.0	fCNS11643	2-3E69
U+7B58.0	fBig5	DAE5
U+7B58.0	fGB	0-7358
U+7B58.0	fR/S	 118.6
U+7B58.0	fTotalStrokes	 12
U+7B58.0	fCangjie	HQR
U+7B58.0	fFourCorner	88563
U+7B58.0	fHakka	 kiu5
U+7B58.0	fCantonese	kau3
U+7B58.0	fMandarin	kou4
U+7B59.0	fUTF8	筙
U+7B59.0	fCNS11643	3-657E
U+7B59.0	fGB	E-3679
U+7B59.0	fR/S	 118.6
U+7B59.0	fTotalStrokes	 12
U+7B59.0	fCangjie	HQD
U+7B59.0	fFourCorner	8890
U+7B59.0	fCantonese	loi4
U+7B59.0	fMandarin	lai2
U+7B5A.0	fUTF8	筚
U+7B5A.0	fGB	0-7359
U+7B5A.0	fR/S	 118.6
U+7B5A.0	fTotalStrokes	 12
U+7B5A.0	fCangjie	HPPJ
U+7B5A.0	fHakka	 bit7
U+7B5A.0	fCantonese	bat1
U+7B5A.0	fMandarin	bi4
U+7B5A.0	fEnglish	simplified form of &#x7BF3;
U+7B5B.0	fUTF8	筛
U+7B5B.0	fGB	0-4938
U+7B5B.0	fR/S	 118.6
U+7B5B.0	fTotalStrokes	 12
U+7B5B.0	fCangjie	HLLB
U+7B5B.0	fHakka	 si1; ce1
U+7B5B.0	fCantonese	sai1
U+7B5B.0	fMandarin	shai1; shi1
U+7B5B.0	fEnglish	simplified form of &#x7BE9;
U+7B5C.0	fUTF8	筜
U+7B5C.0	fGB	8-2C69
U+7B5C.0	fR/S	 118.6
U+7B5C.0	fTotalStrokes	 12
U+7B5C.0	fCangjie	HFSM
U+7B5C.0	fCantonese	dong1
U+7B5C.0	fMandarin	dang1
U+7B5C.0	fEnglish	tall bamboo
U+7B5D.0	fUTF8	筝
U+7B5D.0	fGB	0-735D
U+7B5D.0	fR/S	 118.6
U+7B5D.0	fTotalStrokes	 12
U+7B5D.0	fCangjie	HNSD
U+7B5D.0	fFourCorner	8850
U+7B5D.0	fHakka	 zen1
U+7B5D.0	fCantonese	zang1; zaang1 (also)
U+7B5D.0	fMandarin	zheng1
U+7B5D.0	fEnglish	simplified form of &#x7B8F;
U+7B5E.0	fUTF8	筞
U+7B5E.0	fCNS11643	3-434F
U+7B5E.0	fGB	E-367A
U+7B5E.0	fR/S	  118.7
U+7B5E.0	fTotalStrokes	 13
U+7B5E.0	fCangjie	HJD
U+7B5E.0	fFourCorner	88900
U+7B5E.0	fHakka	 cet7
U+7B5E.0	fCantonese	caak3
U+7B5E.0	fMandarin	ce4
U+7B5F.0	fUTF8	筟
U+7B5F.0	fCNS11643	4-3F67
U+7B5F.0	fGB	3-6427
U+7B5F.0	fR/S	  118.7
U+7B5F.0	fTotalStrokes	 13
U+7B5F.0	fCangjie	HBND
U+7B5F.0	fFourCorner	88400
U+7B5F.0	fCantonese	fu1
U+7B5F.0	fMandarin	fu1
U+7B60.0	fUTF8	筠
U+7B60.0	fCNS11643	1-6568
U+7B60.0	fBig5	B861
U+7B60.0	fGB	0-735E
U+7B60.0	fR/S	 118.7
U+7B60.0	fTotalStrokes	 13
U+7B60.0	fCangjie	HGPM
U+7B60.0	fFourCorner	88127
U+7B60.0	fHakka	  giun1;  jun2; run2; jun1
U+7B60.0	fCantonese	wan4; gwan1
U+7B60.0	fMandarin	yun2; jun1
U+7B60.0	fEnglish	 bamboo skin
U+7B61.0	fUTF8	筡
U+7B61.0	fCNS11643	2-4575
U+7B61.0	fBig5	DF50
U+7B61.0	fGB	5-633F
U+7B61.0	fR/S	 118.7
U+7B61.0	fTotalStrokes	 13
U+7B61.0	fCangjie	HOMD
U+7B61.0	fFourCorner	88904
U+7B61.0	fCantonese	seoi5; tou4; syu1
U+7B61.0	fMandarin	tu2; chi4; chu2; shu1
U+7B62.0	fUTF8	筢
U+7B62.0	fCNS11643	3-434C
U+7B62.0	fGB	0-7361
U+7B62.0	fR/S	 118.7
U+7B62.0	fTotalStrokes	 13
U+7B62.0	fCangjie	HQAU
U+7B62.0	fFourCorner	88510
U+7B62.0	fHakka	  pa2; pa3
U+7B62.0	fCantonese	paa4
U+7B62.0	fMandarin	pa2
U+7B62.0	fEnglish	rake
U+7B63.0	fUTF8	筣
U+7B63.0	fCNS11643	2-4578
U+7B63.0	fBig5	DF53
U+7B63.0	fGB	3-6425
U+7B63.0	fR/S	 118.7
U+7B63.0	fTotalStrokes	 13
U+7B63.0	fCangjie	HHDN
U+7B63.0	fFourCorner	88920
U+7B63.0	fCantonese	lei4
U+7B63.0	fMandarin	li4; li2; li3
U+7B64.0	fUTF8	筤
U+7B64.0	fCNS11643	2-456C
U+7B64.0	fBig5	DF47
U+7B64.0	fGB	3-642B
U+7B64.0	fR/S	 118.7
U+7B64.0	fTotalStrokes	 13
U+7B64.0	fCangjie	HIAV
U+7B64.0	fFourCorner	88732
U+7B64.0	fHakka	  long2
U+7B64.0	fCantonese	long4
U+7B64.0	fMandarin	lang2; lang3; lang4; liang2
U+7B64.0	fEnglish	young bamboo
U+7B65.0	fUTF8	筥
U+7B65.0	fCNS11643	2-4571
U+7B65.0	fBig5	DF4C
U+7B65.0	fGB	3-6367
U+7B65.0	fR/S	 118.7
U+7B65.0	fTotalStrokes	 13
U+7B65.0	fCangjie	HRHR
U+7B65.0	fFourCorner	88606
U+7B65.0	fHakka	 gi3
U+7B65.0	fCantonese	geoi2
U+7B65.0	fMandarin	ju3
U+7B65.0	fEnglish	a round-shaped bamboo basket for holding rice
U+7B66.0	fUTF8	筦
U+7B66.0	fCNS11643	2-456B
U+7B66.0	fBig5	DF46
U+7B66.0	fGB	E-367B
U+7B66.0	fR/S	 118.7
U+7B66.0	fTotalStrokes	 13
U+7B66.0	fCangjie	HJMU
U+7B66.0	fFourCorner	88211
U+7B66.0	fHakka	 gon3;  kwan3; gwon3
U+7B66.0	fCantonese	gun2; gwun2 (also)
U+7B66.0	fMandarin	guan3
U+7B66.0	fEnglish	[1] key [2] be in charge [3] a wind pipe made of bamboo (such as flutes, etc.) [4] a Chinese family name [5] same as $
U+7B67.0	fUTF8	筧
U+7B67.0	fCNS11643	1-656A
U+7B67.0	fBig5	B863
U+7B67.0	fGB	1-7348
U+7B67.0	fR/S	 118.7
U+7B67.0	fTotalStrokes	 13
U+7B67.0	fCangjie	HBUU
U+7B67.0	fFourCorner	88216
U+7B67.0	fHakka	 gan3; kian3; gien6; gien5; gian3; gen3
U+7B67.0	fCantonese	gaan2; gan2
U+7B67.0	fMandarin	jian3; xian4; jian4
U+7B67.0	fEnglish	 bamboo water pipe
U+7B68.0	fUTF8	筨
U+7B68.0	fCNS11643	3-4355
U+7B68.0	fGB	5-6341
U+7B68.0	fR/S	  118.7
U+7B68.0	fTotalStrokes	 13
U+7B68.0	fCangjie	HOIR
U+7B68.0	fFourCorner	88600
U+7B68.0	fCantonese	ham4
U+7B68.0	fMandarin	han2
U+7B69.0	fUTF8	筩
U+7B69.0	fCNS11643	2-456F
U+7B69.0	fBig5	DF4A
U+7B69.0	fGB	E-367C
U+7B69.0	fR/S	 118.7
U+7B69.0	fTotalStrokes	 13
U+7B69.0	fCangjie	HNIB
U+7B69.0	fFourCorner	88227
U+7B69.0	fHakka	  tung2; tung3
U+7B69.0	fCantonese	tung4; tung2 (also)
U+7B69.0	fMandarin	tong3; dong4; yong3; tong2
U+7B69.0	fEnglish	[1] bamboo pipe [2] fish hook
U+7B6A.0	fUTF8	筪
U+7B6A.0	fCNS11643	4-3F5E
U+7B6A.0	fGB	E-367D
U+7B6A.0	fR/S	 118.7
U+7B6A.0	fTotalStrokes	 13
U+7B6A.0	fCangjie	HSWL
U+7B6A.0	fFourCorner	88710
U+7B6A.0	fCantonese	haap6
U+7B6A.0	fMandarin	xia2
U+7B6B.0	fUTF8	筫
U+7B6B.0	fCNS11643	3-434D
U+7B6B.0	fGB	3-634B
U+7B6B.0	fR/S	 118.7
U+7B6B.0	fTotalStrokes	 13
U+7B6B.0	fCangjie	HBUC
U+7B6B.0	fFourCorner	88800
U+7B6B.0	fCantonese	zi3
U+7B6B.0	fMandarin	zhi4
U+7B6C.0	fUTF8	筬
U+7B6C.0	fCNS11643	3-3D77
U+7B6C.0	fGB	3-6364
U+7B6C.0	fR/S	 118.7; 118.6
U+7B6C.0	fTotalStrokes	 12; 13
U+7B6C.0	fCangjie	HIHS
U+7B6C.0	fFourCorner	88250
U+7B6C.0	fCantonese	sing4
U+7B6C.0	fMandarin	cheng2
U+7B6C.0	fEnglish	reed of a loom
U+7B6D.0	fUTF8	筭
U+7B6D.0	fCNS11643	2-456D
U+7B6D.0	fBig5	DF48
U+7B6D.0	fGB	3-637A
U+7B6D.0	fR/S	 118.7
U+7B6D.0	fTotalStrokes	 13
U+7B6D.0	fCangjie	HMGT
U+7B6D.0	fFourCorner	99441
U+7B6D.0	fHakka	  son5
U+7B6D.0	fCantonese	syun3
U+7B6D.0	fMandarin	suan4
U+7B6D.0	fEnglish	 an ancient device for working with numbers count; calculate; figure
U+7B6E.0	fUTF8	筮
U+7B6E.0	fCNS11643	1-6569
U+7B6E.0	fBig5	B862
U+7B6E.0	fGB	0-735F
U+7B6E.0	fR/S	 118.7
U+7B6E.0	fTotalStrokes	 13
U+7B6E.0	fCangjie	HMOO
U+7B6E.0	fFourCorner	88108
U+7B6E.0	fHakka	 chi3; cih3; si5; ci3; chii3
U+7B6E.0	fCantonese	sai6
U+7B6E.0	fMandarin	shi4; yi4
U+7B6E.0	fEnglish	 divine by means of stalks of the milfoil
U+7B6F.0	fUTF8	筯
U+7B6F.0	fCNS11643	3-4354
U+7B6F.0	fGB	E-367E
U+7B6F.0	fR/S	 118.7
U+7B6F.0	fTotalStrokes	 13
U+7B6F.0	fCangjie	HBMS
U+7B6F.0	fFourCorner	88120
U+7B6F.0	fHakka	 chu6; cu5; cu3; chu5
U+7B6F.0	fCantonese	zyu6; zyu3 (also)
U+7B6F.0	fMandarin	zhu4
U+7B6F.0	fEnglish	[1] chopsticks [2] tongs
U+7B70.0	fUTF8	筰
U+7B70.0	fCNS11643	2-4574
U+7B70.0	fBig5	DF4F
U+7B70.0	fGB	3-6426
U+7B70.0	fR/S	 118.7
U+7B70.0	fTotalStrokes	 13
U+7B70.0	fCangjie	HOOS
U+7B70.0	fFourCorner	88211
U+7B70.0	fCantonese	zok6; zok3
U+7B70.0	fMandarin	zuo2; ze2
U+7B70.0	fEnglish	cable
U+7B71.0	fUTF8	筱
U+7B71.0	fCNS11643	2-4573
U+7B71.0	fBig5	DF4E
U+7B71.0	fGB	0-7363
U+7B71.0	fR/S	 118.7
U+7B71.0	fTotalStrokes	 13
U+7B71.0	fCangjie	HOLK
U+7B71.0	fFourCorner	88248
U+7B71.0	fHakka	 siau3; seu3
U+7B71.0	fCantonese	siu2; siu6 (also)
U+7B71.0	fMandarin	xiao3
U+7B71.0	fEnglish	[1] dwarf bamboo; little slender bamboo [2] little; small
U+7B72.0	fUTF8	筲
U+7B72.0	fCNS11643	2-4570
U+7B72.0	fBig5	DF4B
U+7B72.0	fGB	0-7362
U+7B72.0	fR/S	 118.7
U+7B72.0	fTotalStrokes	 13
U+7B72.0	fCangjie	HFB
U+7B72.0	fFourCorner	88227
U+7B72.0	fHakka	  sau1
U+7B72.0	fCantonese	saau1
U+7B72.0	fMandarin	shao1; shuo4
U+7B72.0	fEnglish	 basket for holding cooked rice
U+7B73.0	fUTF8	筳
U+7B73.0	fCNS11643	2-4572
U+7B73.0	fBig5	DF4D
U+7B73.0	fGB	3-636B
U+7B73.0	fR/S	 118.7
U+7B73.0	fTotalStrokes	 13
U+7B73.0	fCangjie	HNKG
U+7B73.0	fFourCorner	88401
U+7B73.0	fCantonese	ting4; jin4
U+7B73.0	fMandarin	ting2; ting3
U+7B73.0	fEnglish	little slender bamboo
U+7B74.0	fUTF8	筴
U+7B74.0	fCNS11643	2-456E
U+7B74.0	fBig5	DF49
U+7B74.0	fGB	E-3721
U+7B74.0	fR/S	 118.7
U+7B74.0	fTotalStrokes	 13
U+7B74.0	fCangjie	HKOO
U+7B74.0	fFourCorner	88438
U+7B74.0	fHakka	  giap7; cet7; gap7; cak7
U+7B74.0	fCantonese	caak3; gaap3; kaap3
U+7B74.0	fMandarin	ce4; jia2; jia1
U+7B74.0	fEnglish	[1] the milfoil used in divination [2] same as $
U+7B75.0	fUTF8	筵
U+7B75.0	fCNS11643	1-6A2A
U+7B75.0	fBig5	BAE1
U+7B75.0	fGB	0-735B
U+7B75.0	fR/S	 118.7
U+7B75.0	fTotalStrokes	 13
U+7B75.0	fCangjie	HNKM
U+7B75.0	fFourCorner	88401
U+7B75.0	fHakka	 jen2; ren2; jan2
U+7B75.0	fCantonese	jin4
U+7B75.0	fMandarin	yan2
U+7B75.0	fEnglish	[1] bamboo mat [2] feast; banquet
U+7B76.0	fUTF8	筶
U+7B76.0	fCNS11643	2-4577
U+7B76.0	fBig5	DF52
U+7B76.0	fGB	E-3722
U+7B76.0	fR/S	 118.7
U+7B76.0	fTotalStrokes	 13
U+7B76.0	fCangjie	HHGR
U+7B76.0	fFourCorner	88601
U+7B76.0	fHakka	  gau5; gau1; kau5
U+7B76.0	fCantonese	gou3; gou2
U+7B76.0	fMandarin	gao3
U+7B76.0	fEnglish	two kidney-shaped pieces of bamboo root, of which one side is flat, and the other convex
U+7B77.0	fUTF8	筷
U+7B77.0	fCNS11643	1-6566
U+7B77.0	fBig5	B85F
U+7B77.0	fGB	0-3F6A
U+7B77.0	fR/S	 118.7
U+7B77.0	fTotalStrokes	 13
U+7B77.0	fCangjie	HPDK
U+7B77.0	fFourCorner	88934
U+7B77.0	fHakka	  kwai5; kai5; k'uai5
U+7B77.0	fCantonese	faai3; faai2 (also)
U+7B77.0	fMandarin	kuai4
U+7B77.0	fEnglish	 chopsticks
U+7B78.0	fUTF8	筸
U+7B78.0	fCNS11643	2-4576
U+7B78.0	fBig5	DF51
U+7B78.0	fGB	3-6422
U+7B78.0	fR/S	 118.7
U+7B78.0	fTotalStrokes	 13
U+7B78.0	fCangjie	HAMJ
U+7B78.0	fFourCorner	88401
U+7B78.0	fCantonese	gon2
U+7B78.0	fMandarin	gan1
U+7B78.0	fEnglish	a place in Honan
U+7B79.0	fUTF8	筹
U+7B79.0	fCNS11643	3-4351
U+7B79.0	fGB	0-336F
U+7B79.0	fR/S	 118.7
U+7B79.0	fTotalStrokes	 13
U+7B79.0	fCangjie	HQKI
U+7B79.0	fFourCorner	88340
U+7B79.0	fHakka	 chiu2
U+7B79.0	fCantonese	cau4; cau2 (also)
U+7B79.0	fMandarin	chou2; tao2
U+7B79.0	fEnglish	simplified form of &#x7C4C;
U+7B7A.0	fUTF8	筺
U+7B7A.0	fGB	E-3723
U+7B7A.0	fR/S	 118.7
U+7B7A.0	fTotalStrokes	 13
U+7B7A.0	fCangjie	HSMI
U+7B7A.0	fCantonese	hong1
U+7B7A.0	fMandarin	kuang1
U+7B7A.0	fEnglish	 bamboo basket or chest
U+7B7B.0	fUTF8	筻
U+7B7B.0	fCNS11643	F-4445
U+7B7B.0	fGB	0-7360
U+7B7B.0	fR/S	 118.7
U+7B7B.0	fTotalStrokes	 13
U+7B7B.0	fCangjie	HMLK
U+7B7B.0	fCantonese	gaang3
U+7B7B.0	fMandarin	gang4
U+7B7B.0	fEnglish	a place in Hunan province
U+7B7C.0	fUTF8	筼
U+7B7C.0	fGB	7-2130
U+7B7C.0	fR/S	 118.7
U+7B7C.0	fTotalStrokes	 13
U+7B7C.0	fCangjie	HRBO
U+7B7C.0	fCantonese	wan4
U+7B7C.0	fMandarin	yun2
U+7B7C.0	fEnglish	tall bamboo
U+7B7D.0	fUTF8	筽
U+7B7D.0	fGB	1-7D77
U+7B7D.0	fR/S	 118.7
U+7B7D.0	fTotalStrokes	 13
U+7B7D.0	fCangjie	HRVK
U+7B7D.0	fCantonese	ng4
U+7B7E.0	fUTF8	签
U+7B7E.0	fGB	0-4729
U+7B7E.0	fR/S	 118.7
U+7B7E.0	fTotalStrokes	 13
U+7B7E.0	fCangjie	HOMM
U+7B7E.0	fHakka	 ciam1
U+7B7E.0	fCantonese	cim1
U+7B7E.0	fMandarin	qian1
U+7B7E.0	fEnglish	simplified form of &#x7C3D;
U+7B7F.0	fUTF8	筿
U+7B7F.0	fGB	8-7E27
U+7B7F.0	fR/S	 118.7
U+7B7F.0	fTotalStrokes	 13
U+7B7F.0	fCangjie	HHED
U+7B7F.0	fCantonese	siu2; siu6 (also)
U+7B7F.0	fMandarin	xiao3
U+7B80.0	fUTF8	简
U+7B80.0	fGB	0-3C72
U+7B80.0	fR/S	 118.7
U+7B80.0	fTotalStrokes	 13
U+7B80.0	fCangjie	HLSA
U+7B80.0	fHakka	 gan3; kian1
U+7B80.0	fCantonese	gaan2
U+7B80.0	fMandarin	jian3
U+7B80.0	fEnglish	simplified form of &#x7C21;
U+7B81.0	fUTF8	箁
U+7B81.0	fCNS11643	3-4856
U+7B81.0	fGB	3-6436
U+7B81.0	fR/S	  118.8
U+7B81.0	fTotalStrokes	 14
U+7B81.0	fCangjie	HYTR
U+7B81.0	fFourCorner	88600
U+7B81.0	fHakka	 piau1
U+7B81.0	fCantonese	pau4
U+7B81.0	fMandarin	pu2
U+7B82.0	fUTF8	箂
U+7B82.0	fCNS11643	2-4C64
U+7B82.0	fBig5	E35D
U+7B82.0	fGB	3-637E
U+7B82.0	fR/S	 118.8
U+7B82.0	fTotalStrokes	 14
U+7B82.0	fCangjie	HDOO
U+7B82.0	fFourCorner	88908
U+7B82.0	fHakka	  loi2
U+7B82.0	fCantonese	loi4
U+7B82.0	fMandarin	lai2
U+7B82.0	fEnglish	a variety of bamboo
U+7B83.0	fUTF8	箃
U+7B83.0	fCNS11643	3-4858
U+7B83.0	fGB	3-642E
U+7B83.0	fR/S	  118.8
U+7B83.0	fTotalStrokes	 14
U+7B83.0	fCangjie	HSJE
U+7B83.0	fFourCorner	88140
U+7B83.0	fCantonese	zau1
U+7B83.0	fMandarin	zou1
U+7B84.0	fUTF8	箄
U+7B84.0	fCNS11643	1-6A31
U+7B84.0	fBig5	BAE8
U+7B84.0	fGB	3-6433
U+7B84.0	fR/S	 118.8
U+7B84.0	fTotalStrokes	 14
U+7B84.0	fCangjie	HHWJ
U+7B84.0	fFourCorner	88406
U+7B84.0	fHakka	 pai2; bi2; bi1; bi5
U+7B84.0	fCantonese	paai4; bei6
U+7B84.0	fMandarin	bi4; bei1; pai2; bi1; bi3
U+7B84.0	fEnglish	a basket-trap for catching fish
U+7B85.0	fUTF8	箅
U+7B85.0	fCNS11643	2-4C5F
U+7B85.0	fBig5	E358
U+7B85.0	fGB	0-736B
U+7B85.0	fR/S	 118.8
U+7B85.0	fTotalStrokes	 14
U+7B85.0	fCangjie	HWML
U+7B85.0	fFourCorner	88221
U+7B85.0	fHakka	  bi5
U+7B85.0	fCantonese	bei3; bai3 (also); bei2
U+7B85.0	fMandarin	bi4; bi3
U+7B85.0	fEnglish	[1] bamboo grid for steaming food [2] a ring on which to stand an iron pan
U+7B86.0	fUTF8	箆
U+7B86.0	fCNS11643	3-4860
U+7B86.0	fGB	E-3724
U+7B86.0	fR/S	  118.8
U+7B86.0	fTotalStrokes	 14
U+7B86.0	fCangjie	HBOP
U+7B86.0	fFourCorner	88710
U+7B86.0	fCantonese	bei6
U+7B86.0	fMandarin	bi4
U+7B86.0	fEnglish	 fine-toothed comb comb hair
U+7B87.0	fUTF8	箇
U+7B87.0	fCNS11643	1-6A30
U+7B87.0	fBig5	BAE7
U+7B87.0	fGB	8-2D7A
U+7B87.0	fR/S	 118.8
U+7B87.0	fTotalStrokes	 14
U+7B87.0	fCangjie	HWJR
U+7B87.0	fFourCorner	88603
U+7B87.0	fHakka	  gai5; go3; go5; ge5; ke5; {ge5}
U+7B87.0	fCantonese	go3
U+7B87.0	fMandarin	ge4; ge5; ge3
U+7B87.0	fEnglish	 numerary adjunct; piece single
U+7B88.0	fUTF8	箈
U+7B88.0	fCNS11643	2-4C55
U+7B88.0	fBig5	E34E
U+7B88.0	fGB	3-643C
U+7B88.0	fR/S	 118.8
U+7B88.0	fTotalStrokes	 14
U+7B88.0	fCangjie	HEIR
U+7B88.0	fFourCorner	88163
U+7B88.0	fCantonese	ci4
U+7B88.0	fMandarin	chi2; tai2; dai4
U+7B89.0	fUTF8	箉
U+7B89.0	fCNS11643	4-456B
U+7B89.0	fGB	E-3725
U+7B89.0	fR/S	  118.8
U+7B89.0	fTotalStrokes	 14
U+7B89.0	fCangjie	HQRS
U+7B89.0	fFourCorner	88520
U+7B89.0	fCantonese	gwaai2
U+7B89.0	fMandarin	guai3
U+7B8A.0	fUTF8	箊
U+7B8A.0	fCNS11643	2-4C57
U+7B8A.0	fBig5	E350
U+7B8A.0	fGB	3-6439
U+7B8A.0	fR/S	 118.8
U+7B8A.0	fTotalStrokes	 14
U+7B8A.0	fCangjie	HYSY
U+7B8A.0	fFourCorner	88233
U+7B8A.0	fCantonese	jyu1
U+7B8A.0	fMandarin	yu1
U+7B8B.0	fUTF8	箋
U+7B8B.0	fCNS11643	1-6A29
U+7B8B.0	fBig5	BAE0
U+7B8B.0	fGB	1-3C63
U+7B8B.0	fR/S	 118.8
U+7B8B.0	fTotalStrokes	 14
U+7B8B.0	fCangjie	HII
U+7B8B.0	fFourCorner	88503
U+7B8B.0	fHakka	  zien1; zen1; tsian1
U+7B8B.0	fCantonese	zin1
U+7B8B.0	fMandarin	jian1
U+7B8B.0	fEnglish	[1] note; memo; commentary [2] fancy note paper; stationery [3] letters; correspondence
U+7B8C.0	fUTF8	箌
U+7B8C.0	fCNS11643	2-4C5C
U+7B8C.0	fBig5	E355
U+7B8C.0	fGB	E-3726
U+7B8C.0	fR/S	 118.8
U+7B8C.0	fTotalStrokes	 14
U+7B8C.0	fCangjie	HMGN
U+7B8C.0	fFourCorner	88120
U+7B8C.0	fHakka	  zau5; dau5
U+7B8C.0	fCantonese	dou3; zaau3; zou3
U+7B8C.0	fMandarin	zhao4; dao4
U+7B8C.0	fEnglish	great; large; rank, as grass
U+7B8D.0	fUTF8	箍
U+7B8D.0	fCNS11643	2-4C5B
U+7B8D.0	fBig5	E354
U+7B8D.0	fGB	0-393F
U+7B8D.0	fR/S	 118.8
U+7B8D.0	fTotalStrokes	 14
U+7B8D.0	fCangjie	HQSB
U+7B8D.0	fFourCorner	88512
U+7B8D.0	fHakka	  ku1; k'ieu1; keu1
U+7B8D.0	fCantonese	ku1; kwu1 (also); fu1
U+7B8D.0	fMandarin	gu1
U+7B8D.0	fEnglish	[1] hoop [2] bind round; surround; hoop
U+7B8E.0	fUTF8	箎
U+7B8E.0	fCNS11643	2-4C5E
U+7B8E.0	fBig5	E357
U+7B8E.0	fGB	5-634D
U+7B8E.0	fR/S	 118.8
U+7B8E.0	fTotalStrokes	 14
U+7B8E.0	fCangjie	HYPU
U+7B8E.0	fFourCorner	88217
U+7B8E.0	fHakka	 chii2; chi2
U+7B8E.0	fCantonese	ci4
U+7B8E.0	fMandarin	chi2; hu3
U+7B8E.0	fEnglish	ancient woodwind instrument
U+7B8F.0	fUTF8	箏
U+7B8F.0	fCNS11643	1-6A2E
U+7B8F.0	fBig5	BAE5
U+7B8F.0	fGB	1-735D
U+7B8F.0	fR/S	 118.8
U+7B8F.0	fTotalStrokes	 14
U+7B8F.0	fCangjie	HBSD
U+7B8F.0	fFourCorner	88507
U+7B8F.0	fHakka	  zen1; zang1
U+7B8F.0	fCantonese	zang1; zaang1 (also)
U+7B8F.0	fMandarin	zheng1
U+7B8F.0	fEnglish	[1] a plucked stringed musical instrument in some ways similar to the zither [2] kite
U+7B90.0	fUTF8	箐
U+7B90.0	fCNS11643	2-4C59
U+7B90.0	fBig5	E352
U+7B90.0	fGB	0-7364
U+7B90.0	fR/S	 118.8
U+7B90.0	fTotalStrokes	 14
U+7B90.0	fCangjie	HQMB
U+7B90.0	fFourCorner	88227
U+7B90.0	fHakka	 zin1
U+7B90.0	fCantonese	zing1; sin6; sin3; coeng1
U+7B90.0	fMandarin	jing1; qing4; qiang1; qian4
U+7B90.0	fEnglish	 draw a bamboo bow or crossbow
U+7B91.0	fUTF8	箑
U+7B91.0	fCNS11643	2-4C58
U+7B91.0	fBig5	E351
U+7B91.0	fGB	3-6431
U+7B91.0	fR/S	 118.8
U+7B91.0	fTotalStrokes	 14
U+7B91.0	fCangjie	HJLO
U+7B91.0	fFourCorner	88801
U+7B91.0	fHakka	 ciap8; sap7; cap7
U+7B91.0	fCantonese	saap3; zit6 (also); sip3; sap3
U+7B91.0	fMandarin	sha4; jie2
U+7B91.0	fEnglish	 fan
U+7B92.0	fUTF8	箒
U+7B92.0	fCNS11643	3-485A
U+7B92.0	fGB	E-3727
U+7B92.0	fR/S	 118.8
U+7B92.0	fTotalStrokes	 14
U+7B92.0	fCangjie	HSMB
U+7B92.0	fFourCorner	88220
U+7B92.0	fHakka	 sau5; so5;  zhiu3; ziu3; zu3; zhu3
U+7B92.0	fCantonese	zau2; zaau2 (also)
U+7B92.0	fMandarin	zhou3
U+7B92.0	fEnglish	broom
U+7B93.0	fUTF8	箓
U+7B93.0	fCNS11643	3-485E
U+7B93.0	fGB	8-7E28
U+7B93.0	fR/S	  118.8
U+7B93.0	fTotalStrokes	 14
U+7B93.0	fCangjie	HNME
U+7B93.0	fFourCorner	88130
U+7B93.0	fCantonese	luk1; luk6
U+7B93.0	fMandarin	lu4; du4
U+7B93.0	fEnglish	book
U+7B94.0	fUTF8	箔
U+7B94.0	fCNS11643	1-6A2D
U+7B94.0	fBig5	BAE4
U+7B94.0	fGB	0-322D
U+7B94.0	fR/S	 118.8
U+7B94.0	fTotalStrokes	 14
U+7B94.0	fCangjie	HEHA
U+7B94.0	fFourCorner	88163
U+7B94.0	fHakka	  pok7; pok8; pak7
U+7B94.0	fCantonese	bok6; bok2 (also)
U+7B94.0	fMandarin	bo2; bao2
U+7B94.0	fEnglish	[1] foil; gillt [2] curtain
U+7B95.0	fUTF8	箕
U+7B95.0	fCNS11643	1-6A28
U+7B95.0	fBig5	BADF
U+7B95.0	fGB	0-3B7E
U+7B95.0	fR/S	 118.8
U+7B95.0	fTotalStrokes	 14
U+7B95.0	fCangjie	HTMC
U+7B95.0	fFourCorner	88801
U+7B95.0	fHakka	  gi1
U+7B95.0	fCantonese	gei1
U+7B95.0	fMandarin	ji1; ji5
U+7B95.0	fEnglish	[1] winnowing basket; sieve [2] dustpan; dust pan; garbage basket [3] nonspiral lines on a fingertip
U+7B96.0	fUTF8	箖
U+7B96.0	fCNS11643	2-4C5A
U+7B96.0	fBig5	E353
U+7B96.0	fGB	3-642F
U+7B96.0	fR/S	 118.8
U+7B96.0	fTotalStrokes	 14
U+7B96.0	fCangjie	HDD
U+7B96.0	fFourCorner	88994
U+7B96.0	fHakka	  lim2
U+7B96.0	fCantonese	lam4; laam4
U+7B96.0	fMandarin	lin2; lin3
U+7B96.0	fEnglish	a variety of slender bamboo
U+7B97.0	fUTF8	算
U+7B97.0	fCNS11643	1-6A2B
U+7B97.0	fBig5	BAE2
U+7B97.0	fGB	0-4B63
U+7B97.0	fR/S	 118.8
U+7B97.0	fTotalStrokes	 14
U+7B97.0	fCangjie	HBUT
U+7B97.0	fFourCorner	88446
U+7B97.0	fHakka	  son5
U+7B97.0	fCantonese	syun3
U+7B97.0	fMandarin	suan4; suan3; xuan3
U+7B97.0	fEnglish	[1] count; calculate; figure; reckon; compute [2] plan; scheme [3] infer; guess; foretell [4] regard; consider; deem; count [5] be of importance; be of consequence; matter
U+7B97.1	fUTF8	算
U+7B97.1	fCNS11643	1-6A2B
U+7B97.1	fBig5	BAE2
U+7B97.1	fGB	0-4B63
U+7B97.1	fR/S	 118.8
U+7B97.1	fTotalStrokes	 14
U+7B97.1	fCangjie	HBUT
U+7B97.1	fFourCorner	88446
U+7B97.1	fMandarin	suan5
U+7B98.0	fUTF8	箘
U+7B98.0	fCNS11643	2-4C60
U+7B98.0	fBig5	E359
U+7B98.0	fGB	3-6432
U+7B98.0	fR/S	 118.8
U+7B98.0	fTotalStrokes	 14
U+7B98.0	fCangjie	HWHD
U+7B98.0	fFourCorner	88603
U+7B98.0	fHakka	  kiun2; kun2; kwun3
U+7B98.0	fCantonese	kwan3; kwan2 (also); kwan5; kwan1
U+7B98.0	fMandarin	jun4; jun1; qun1
U+7B98.0	fEnglish	fine bamboo
U+7B99.0	fUTF8	箙
U+7B99.0	fCNS11643	2-4C62
U+7B99.0	fBig5	E35B
U+7B99.0	fGB	5-6356
U+7B99.0	fR/S	 118.8
U+7B99.0	fTotalStrokes	 14
U+7B99.0	fCangjie	HBSE
U+7B99.0	fFourCorner	88247
U+7B99.0	fHakka	  fuk8
U+7B99.0	fCantonese	fuk6
U+7B99.0	fMandarin	fu2; fu1
U+7B99.0	fEnglish	 quiver
U+7B9A.0	fUTF8	箚
U+7B9A.0	fGB	5-6353
U+7B9A.0	fR/S	 118.8
U+7B9A.0	fTotalStrokes	 14
U+7B9A.0	fCangjie	HORN
U+7B9A.0	fFourCorner	8862
U+7B9A.0	fCantonese	zaap3; zaat3 (also)
U+7B9A.0	fMandarin	zha2; da2
U+7B9A.0	fEnglish	 brief note official communique
U+7B9B.0	fUTF8	箛
U+7B9B.0	fCNS11643	2-4C5D
U+7B9B.0	fBig5	E356
U+7B9B.0	fGB	3-6440
U+7B9B.0	fR/S	 118.8; 118.9
U+7B9B.0	fTotalStrokes	 14
U+7B9B.0	fCangjie	HNDO
U+7B9B.0	fFourCorner	88432
U+7B9B.0	fHakka	 gu1
U+7B9B.0	fCantonese	gu1; gwu1 (also); ku1
U+7B9B.0	fMandarin	gu1
U+7B9B.0	fEnglish	bugle
U+7B9C.0	fUTF8	箜
U+7B9C.0	fCNS11643	2-4C56
U+7B9C.0	fBig5	E34F
U+7B9C.0	fGB	0-736D
U+7B9C.0	fR/S	 118.8
U+7B9C.0	fTotalStrokes	 14
U+7B9C.0	fCangjie	HJCM
U+7B9C.0	fFourCorner	88101
U+7B9C.0	fHakka	  kung1
U+7B9C.0	fCantonese	hung1
U+7B9C.0	fMandarin	kong1
U+7B9C.0	fEnglish	a kind of ancient plucked string music instrument
U+7B9D.0	fUTF8	箝
U+7B9D.0	fCNS11643	1-6A2C
U+7B9D.0	fBig5	BAE3
U+7B9D.0	fGB	0-7369
U+7B9D.0	fR/S	 118.8
U+7B9D.0	fTotalStrokes	 14
U+7B9D.0	fCangjie	HQTM
U+7B9D.0	fFourCorner	88575
U+7B9D.0	fHakka	 kiam2
U+7B9D.0	fCantonese	kim4
U+7B9D.0	fMandarin	qian2
U+7B9D.0	fEnglish	[1] tweezers; pliers; pincers [2] tweezer
U+7B9E.0	fUTF8	箞
U+7B9E.0	fCNS11643	3-4857
U+7B9E.0	fGB	E-3728
U+7B9E.0	fR/S	 118.8
U+7B9E.0	fTotalStrokes	 14
U+7B9E.0	fCangjie	HFQU
U+7B9E.0	fFourCorner	88710
U+7B9E.0	fCantonese	hyun1
U+7B9E.0	fMandarin	quan1; qian1
U+7B9F.0	fUTF8	箟
U+7B9F.0	fCNS11643	3-485D
U+7B9F.0	fGB	5-634E
U+7B9F.0	fR/S	 118.8
U+7B9F.0	fTotalStrokes	 14
U+7B9F.0	fCangjie	HAPP
U+7B9F.0	fFourCorner	88710
U+7B9F.0	fCantonese	kwan2; kwan3 (also)
U+7B9F.0	fMandarin	jun4; jun1
U+7BA0.0	fUTF8	箠
U+7BA0.0	fCNS11643	1-6E34
U+7BA0.0	fBig5	BD69
U+7BA0.0	fGB	E-3729
U+7BA0.0	fR/S	 118.8
U+7BA0.0	fTotalStrokes	 14
U+7BA0.0	fCangjie	HHJM
U+7BA0.0	fFourCorner	88104
U+7BA0.0	fHakka	 ch'ui2; cui2; shui2
U+7BA0.0	fCantonese	seoi4; ceoi4; zeoi1
U+7BA0.0	fMandarin	chui2; zhui1; zhui3
U+7BA0.0	fEnglish	[1] horse whip [2] flog; whip (as a punishment)
U+7BA1.0	fUTF8	管
U+7BA1.0	fCNS11643	1-6A27
U+7BA1.0	fBig5	BADE
U+7BA1.0	fGB	0-395C
U+7BA1.0	fR/S	 118.8
U+7BA1.0	fTotalStrokes	 14
U+7BA1.0	fCangjie	HJRR
U+7BA1.0	fFourCorner	88777
U+7BA1.0	fHakka	  gon3;  kwan3; gwon3
U+7BA1.0	fCantonese	gun2; gwun2 (also)
U+7BA1.0	fMandarin	guan3; guan1
U+7BA1.0	fEnglish	[1] pipe; tube; duct [2] woodwind music instrument [3] control; manage; take care of [4] heed; pay attention to [5] provide [6] guarantee [7] meddle; interfere in; bother about [8] key [9] a Chinese family name
U+7BA2.0	fUTF8	箢
U+7BA2.0	fCNS11643	4-4564
U+7BA2.0	fGB	0-736E
U+7BA2.0	fR/S	 118.8
U+7BA2.0	fTotalStrokes	 14
U+7BA2.0	fCangjie	HJNU
U+7BA2.0	fFourCorner	88210
U+7BA2.0	fCantonese	jyun1
U+7BA2.0	fMandarin	wan3; yuan1
U+7BA3.0	fUTF8	箣
U+7BA3.0	fCNS11643	3-485C
U+7BA3.0	fGB	3-6430
U+7BA3.0	fR/S	 118.8
U+7BA3.0	fTotalStrokes	 14
U+7BA3.0	fCangjie	HDBN
U+7BA3.0	fFourCorner	88920
U+7BA3.0	fCantonese	caak3
U+7BA3.0	fMandarin	ce4
U+7BA4.0	fUTF8	箤
U+7BA4.0	fCNS11643	2-4C63
U+7BA4.0	fBig5	E35C
U+7BA4.0	fGB	E-372A
U+7BA4.0	fR/S	 118.8
U+7BA4.0	fTotalStrokes	 14
U+7BA4.0	fCangjie	HYOJ
U+7BA4.0	fFourCorner	88408
U+7BA4.0	fCantonese	zyut6; ceoi3
U+7BA4.0	fMandarin	ju2; cui4
U+7BA5.0	fUTF8	箥
U+7BA5.0	fCNS11643	4-4570
U+7BA5.0	fGB	E-372B
U+7BA5.0	fR/S	  118.8
U+7BA5.0	fTotalStrokes	 14
U+7BA5.0	fCangjie	HEDE
U+7BA5.0	fFourCorner	88140
U+7BA5.0	fCantonese	bai1
U+7BA5.0	fMandarin	bo3
U+7BA6.0	fUTF8	箦
U+7BA6.0	fGB	0-7365
U+7BA6.0	fR/S	 118.8
U+7BA6.0	fTotalStrokes	 14
U+7BA6.0	fCangjie	HQMO
U+7BA6.0	fHakka	 zit7
U+7BA6.0	fCantonese	zaak3
U+7BA6.0	fMandarin	ze2; kui4; zhai4
U+7BA6.0	fEnglish	simplified form of &#x7C00;
U+7BA7.0	fUTF8	箧
U+7BA7.0	fGB	0-7366
U+7BA7.0	fR/S	 118.8
U+7BA7.0	fTotalStrokes	 14
U+7BA7.0	fCangjie	HSKT
U+7BA7.0	fHakka	 giap7
U+7BA7.0	fCantonese	haap6
U+7BA7.0	fMandarin	qie4
U+7BA7.0	fEnglish	simplified form of &#x7BCB;
U+7BA8.0	fUTF8	箨
U+7BA8.0	fGB	0-736A
U+7BA8.0	fR/S	 118.8
U+7BA8.0	fTotalStrokes	 14
U+7BA8.0	fCangjie	HQEQ
U+7BA8.0	fHakka	 tok8
U+7BA8.0	fCantonese	tok3
U+7BA8.0	fMandarin	tuo4
U+7BA8.0	fEnglish	simplified form of &#x7C5C;
U+7BA9.0	fUTF8	箩
U+7BA9.0	fGB	0-4261
U+7BA9.0	fR/S	 118.8
U+7BA9.0	fTotalStrokes	 14
U+7BA9.0	fCangjie	HWLN
U+7BA9.0	fHakka	 lo2
U+7BA9.0	fCantonese	lo4; lo1 (also); lo2 (also)
U+7BA9.0	fMandarin	luo1; luo2
U+7BA9.0	fEnglish	simplified form of &#x7C6E;
U+7BAA.0	fUTF8	箪
U+7BAA.0	fGB	0-736C
U+7BAA.0	fR/S	 118.8; 118.9
U+7BAA.0	fTotalStrokes	 14
U+7BAA.0	fCangjie	HCWJ
U+7BAA.0	fHakka	 dan1
U+7BAA.0	fCantonese	daan1
U+7BAA.0	fMandarin	dan1
U+7BAA.0	fEnglish	simplified form of &#x7C1E;
U+7BAB.0	fUTF8	箫
U+7BAB.0	fGB	0-736F
U+7BAB.0	fR/S	 118.8
U+7BAB.0	fTotalStrokes	 14
U+7BAB.0	fCangjie	XHLX
U+7BAB.0	fHakka	 siau1
U+7BAB.0	fCantonese	siu1
U+7BAB.0	fMandarin	xiao1; xiao3
U+7BAB.0	fEnglish	simplified form of &#x7C2B;
U+7BAC.0	fUTF8	箬
U+7BAC.0	fCNS11643	2-5261
U+7BAC.0	fBig5	E6D9
U+7BAC.0	fGB	0-7368
U+7BAC.0	fR/S	 118.9
U+7BAC.0	fTotalStrokes	 15
U+7BAC.0	fCangjie	HTKR
U+7BAC.0	fFourCorner	88604
U+7BAC.0	fHakka	 ngiok7; jok8; jok7; rok8; niok7
U+7BAC.0	fCantonese	joek6
U+7BAC.0	fMandarin	ruo4; na4
U+7BAC.0	fEnglish	[1] the cuticle of the bamboo [2] a broad-leaved bamboo of which the leaves are used to make coverings
U+7BAD.0	fUTF8	箭
U+7BAD.0	fCNS11643	1-6E2D
U+7BAD.0	fBig5	BD62
U+7BAD.0	fGB	0-3C7D
U+7BAD.0	fR/S	 118.9
U+7BAD.0	fTotalStrokes	 15
U+7BAD.0	fCangjie	HTBN
U+7BAD.0	fFourCorner	88221
U+7BAD.0	fHakka	 zien5; zen5; tsian5
U+7BAD.0	fCantonese	zin3
U+7BAD.0	fMandarin	jian4
U+7BAD.0	fEnglish	[1] arrow [2] a variety of bamboo (Sinarundinaria nitida) [3] a sign which is like an arrow
U+7BAE.0	fUTF8	箮
U+7BAE.0	fCNS11643	3-4D5C
U+7BAE.0	fGB	E-372C
U+7BAE.0	fR/S	 118.9
U+7BAE.0	fTotalStrokes	 15
U+7BAE.0	fCangjie	HJMM
U+7BAE.0	fFourCorner	88100
U+7BAE.0	fCantonese	hyun1
U+7BAE.0	fMandarin	xuan1
U+7BAF.0	fUTF8	箯
U+7BAF.0	fCNS11643	2-5263
U+7BAF.0	fBig5	E6DB
U+7BAF.0	fGB	3-6452
U+7BAF.0	fR/S	 118.9
U+7BAF.0	fTotalStrokes	 15
U+7BAF.0	fCangjie	HOMK
U+7BAF.0	fFourCorner	88246
U+7BAF.0	fHakka	  bien1; pien1
U+7BAF.0	fCantonese	bin1
U+7BAF.0	fMandarin	bian1; pian2
U+7BAF.0	fEnglish	 bamboo sedan chair
U+7BB0.0	fUTF8	箰
U+7BB0.0	fCNS11643	4-4C2B
U+7BB0.0	fGB	5-6361
U+7BB0.0	fR/S	  118.9
U+7BB0.0	fTotalStrokes	 15
U+7BB0.0	fCangjie	HPAD
U+7BB0.0	fFourCorner	88400
U+7BB0.0	fCantonese	seon2
U+7BB0.0	fMandarin	sun3
U+7BB1.0	fUTF8	箱
U+7BB1.0	fCNS11643	1-6E2E
U+7BB1.0	fBig5	BD63
U+7BB1.0	fGB	0-4F64
U+7BB1.0	fR/S	 118.9
U+7BB1.0	fTotalStrokes	 15
U+7BB1.0	fCangjie	HDBU
U+7BB1.0	fFourCorner	88963
U+7BB1.0	fHakka	  siong1; siong3
U+7BB1.0	fCantonese	soeng1
U+7BB1.0	fMandarin	xiang1
U+7BB1.0	fEnglish	[1] case; box; chest; trunk [2] body or box of a carriage
U+7BB2.0	fUTF8	箲
U+7BB2.0	fCNS11643	3-4D65
U+7BB2.0	fGB	5-6365
U+7BB2.0	fR/S	  118.9
U+7BB2.0	fTotalStrokes	 15
U+7BB2.0	fCangjie	HEHU
U+7BB2.0	fFourCorner	88110
U+7BB2.0	fCantonese	sin2
U+7BB2.0	fMandarin	xian3
U+7BB3.0	fUTF8	箳
U+7BB3.0	fCNS11643	3-4D5D
U+7BB3.0	fGB	5-6369
U+7BB3.0	fR/S	  118.9
U+7BB3.0	fTotalStrokes	 15
U+7BB3.0	fCangjie	HSTT
U+7BB3.0	fFourCorner	88240
U+7BB3.0	fCantonese	ping4
U+7BB3.0	fMandarin	ping2; bing1
U+7BB3.0	fEnglish	bamboo mat used to protect carriage passengers from dust (non-classical form)
U+7BB4.0	fUTF8	箴
U+7BB4.0	fCNS11643	1-6E30
U+7BB4.0	fBig5	BD65
U+7BB4.0	fGB	0-7370
U+7BB4.0	fR/S	 118.9
U+7BB4.0	fTotalStrokes	 15
U+7BB4.0	fCangjie	HIHR
U+7BB4.0	fFourCorner	88253
U+7BB4.0	fHakka	 zhim1; zin1; zim1; zihn1
U+7BB4.0	fCantonese	zam1
U+7BB4.0	fMandarin	zhen1; jian3
U+7BB4.0	fEnglish	[1] needle; probe [2] warn; admonish
U+7BB5.0	fUTF8	箵
U+7BB5.0	fCNS11643	2-5266
U+7BB5.0	fBig5	E6DE
U+7BB5.0	fGB	3-6449
U+7BB5.0	fR/S	 118.9
U+7BB5.0	fTotalStrokes	 15
U+7BB5.0	fCangjie	HFHU
U+7BB5.0	fFourCorner	88602
U+7BB5.0	fHakka	  sen3
U+7BB5.0	fCantonese	sing2; sing1 (also)
U+7BB5.0	fMandarin	sheng3; xing1; xing3
U+7BB5.0	fEnglish	fishing baskets; creels
U+7BB6.0	fUTF8	箶
U+7BB6.0	fCNS11643	4-4B7A
U+7BB6.0	fGB	3-6445
U+7BB6.0	fR/S	  118.9
U+7BB6.0	fTotalStrokes	 15
U+7BB6.0	fCangjie	HJRB
U+7BB6.0	fFourCorner	88620
U+7BB6.0	fHakka	 fu2
U+7BB6.0	fCantonese	wu4
U+7BB6.0	fMandarin	hu2
U+7BB7.0	fUTF8	箷
U+7BB7.0	fCNS11643	2-525E
U+7BB7.0	fBig5	E6D6
U+7BB7.0	fGB	5-6363
U+7BB7.0	fR/S	 118.9
U+7BB7.0	fTotalStrokes	 15
U+7BB7.0	fCangjie	HYSD
U+7BB7.0	fFourCorner	88212
U+7BB7.0	fHakka	 shi1; sii1
U+7BB7.0	fCantonese	ji4; si1
U+7BB7.0	fMandarin	shi1; yi2; chi2
U+7BB7.0	fEnglish	a bamboo clothes-horse
U+7BB8.0	fUTF8	箸
U+7BB8.0	fCNS11643	1-6A2F
U+7BB8.0	fBig5	BAE6
U+7BB8.0	fGB	0-7367
U+7BB8.0	fR/S	 118.9
U+7BB8.0	fTotalStrokes	 15
U+7BB8.0	fCangjie	HJKA
U+7BB8.0	fFourCorner	88604
U+7BB8.0	fHakka	 cu5; chu6; chu3; cu3; zu5; chu5
U+7BB8.0	fCantonese	zyu6; zyu3 (also)
U+7BB8.0	fMandarin	zhu4; zhuo2; zhuo4
U+7BB8.0	fEnglish	 chopsticks
U+7BB9.0	fUTF8	箹
U+7BB9.0	fCNS11643	2-5264
U+7BB9.0	fBig5	E6DC
U+7BB9.0	fGB	3-6378
U+7BB9.0	fR/S	 118.9
U+7BB9.0	fTotalStrokes	 15
U+7BB9.0	fCangjie	HVFI
U+7BB9.0	fFourCorner	88927
U+7BB9.0	fCantonese	ak1; ngak1 (also); aau3; joek3; cuk1
U+7BB9.0	fMandarin	yue1; ao4
U+7BBA.0	fUTF8	箺
U+7BBA.0	fCNS11643	4-4B73
U+7BBA.0	fGB	E-372D
U+7BBA.0	fR/S	  118.9
U+7BBA.0	fTotalStrokes	 15
U+7BBA.0	fCangjie	HQKA
U+7BBA.0	fFourCorner	88600
U+7BBA.0	fCantonese	ceon2
U+7BBA.0	fMandarin	chun3; chun1
U+7BBB.0	fUTF8	箻
U+7BBB.0	fCNS11643	4-4C31
U+7BBB.0	fGB	E-372E
U+7BBB.0	fR/S	 118.9
U+7BBB.0	fTotalStrokes	 15
U+7BBB.0	fCangjie	HHOQ
U+7BBB.0	fFourCorner	88250
U+7BBB.0	fCantonese	leot6
U+7BBB.0	fMandarin	luu4; l4
U+7BBC.0	fUTF8	箼
U+7BBC.0	fCNS11643	4-4B74
U+7BBC.0	fGB	E-372F
U+7BBC.0	fR/S	  118.9
U+7BBC.0	fTotalStrokes	 15
U+7BBC.0	fCangjie	HSMG
U+7BBC.0	fFourCorner	88210
U+7BBC.0	fCantonese	uk1; nguk1 (also)
U+7BBC.0	fMandarin	wu1
U+7BBD.0	fUTF8	箽
U+7BBD.0	fCNS11643	4-4C2A
U+7BBD.0	fGB	3-6451
U+7BBD.0	fR/S	  118.9
U+7BBD.0	fTotalStrokes	 15
U+7BBD.0	fCangjie	HHJG
U+7BBD.0	fFourCorner	88100
U+7BBD.0	fCantonese	dung2
U+7BBD.0	fMandarin	dong3
U+7BBE.0	fUTF8	箾
U+7BBE.0	fCNS11643	2-5260
U+7BBE.0	fBig5	E6D8
U+7BBE.0	fGB	3-644A
U+7BBE.0	fR/S	 118.9
U+7BBE.0	fTotalStrokes	 15
U+7BBE.0	fCangjie	HFBN
U+7BBE.0	fFourCorner	88223
U+7BBE.0	fHakka	 siau1; seu1
U+7BBE.0	fCantonese	sok3; siu1; siu6
U+7BBE.0	fMandarin	shuo4; xiao1; shao1; qiao4
U+7BBE.0	fEnglish	 a musical instrument like pan-pipes; bamboo flute
U+7BBF.0	fUTF8	箿
U+7BBF.0	fCNS11643	3-4D60
U+7BBF.0	fGB	3-644D
U+7BBF.0	fR/S	  118.9
U+7BBF.0	fTotalStrokes	 15
U+7BBF.0	fCangjie	HRSJ
U+7BBF.0	fFourCorner	88400
U+7BBF.0	fCantonese	zaap6
U+7BBF.0	fMandarin	ji2
U+7BC0.0	fUTF8	節
U+7BC0.0	fCNS11643	1-6567
U+7BC0.0	fBig5	B860
U+7BC0.0	fGB	1-3D5A
U+7BC0.0	fR/S	 118.7; 118.8
U+7BC0.0	fTotalStrokes	 13
U+7BC0.0	fCangjie	HAIL
U+7BC0.0	fFourCorner	88727
U+7BC0.0	fHakka	 ziet7; zet7; tsiat7
U+7BC0.0	fCantonese	zit3
U+7BC0.0	fMandarin	jie2; jie1
U+7BC0.0	fEnglish	[1] knot; node; joint [2] passage; paragraph; section [3] principle; integrity; fidelity; constancy; uprightness [4] festival; holiday [5] seasons [6] beats (music); rhythm; time [7] restrain; control; restrict [8] curtail; economise
U+7BC1.0	fUTF8	篁
U+7BC1.0	fCNS11643	1-6E33
U+7BC1.0	fBig5	BD68
U+7BC1.0	fGB	0-7372
U+7BC1.0	fR/S	 118.9
U+7BC1.0	fTotalStrokes	 15
U+7BC1.0	fCangjie	HHAG
U+7BC1.0	fFourCorner	88104
U+7BC1.0	fHakka	 fong2
U+7BC1.0	fCantonese	wong4
U+7BC1.0	fMandarin	huang2
U+7BC1.0	fEnglish	 bamboo grove; bamboo bush; bamboo; clump of bamboos
U+7BC2.0	fUTF8	篂
U+7BC2.0	fCNS11643	4-4C21
U+7BC2.0	fGB	E-3730
U+7BC2.0	fR/S	 118.9
U+7BC2.0	fTotalStrokes	 15
U+7BC2.0	fCangjie	HAHM
U+7BC2.0	fFourCorner	88100
U+7BC2.0	fHakka	  sen1; sin1
U+7BC2.0	fCantonese	sing1
U+7BC2.0	fMandarin	xing1
U+7BC3.0	fUTF8	篃
U+7BC3.0	fCNS11643	4-4B78
U+7BC3.0	fGB	3-645D
U+7BC3.0	fR/S	  118.9
U+7BC3.0	fTotalStrokes	 15
U+7BC3.0	fCangjie	HAHU
U+7BC3.0	fFourCorner	88260
U+7BC3.0	fCantonese	mei6
U+7BC3.0	fMandarin	mei2
U+7BC4.0	fUTF8	範
U+7BC4.0	fCNS11643	1-6E2F
U+7BC4.0	fBig5	BD64
U+7BC4.0	fGB	1-3736
U+7BC4.0	fR/S	 118.9
U+7BC4.0	fTotalStrokes	 15
U+7BC4.0	fCangjie	HJJU
U+7BC4.0	fFourCorner	88512
U+7BC4.0	fHakka	 fam5; fam6
U+7BC4.0	fCantonese	faan6
U+7BC4.0	fMandarin	fan4
U+7BC4.0	fEnglish	[1] pattern; model; form; example [2] range; scope; limits [3] observe the proper rules [4] a Chinese family name
U+7BC5.0	fUTF8	篅
U+7BC5.0	fCNS11643	4-4C26
U+7BC5.0	fGB	3-644F
U+7BC5.0	fR/S	  118.9
U+7BC5.0	fTotalStrokes	 15
U+7BC5.0	fCangjie	HUMB
U+7BC5.0	fFourCorner	88220
U+7BC5.0	fCantonese	cyun4; syun4 (also)
U+7BC5.0	fMandarin	chui2; chuan2; duan1
U+7BC6.0	fUTF8	篆
U+7BC6.0	fCNS11643	1-6E31
U+7BC6.0	fBig5	BD66
U+7BC6.0	fGB	0-572D
U+7BC6.0	fR/S	 118.9
U+7BC6.0	fTotalStrokes	 15
U+7BC6.0	fCangjie	HVNO
U+7BC6.0	fFourCorner	88232
U+7BC6.0	fHakka	 con5;  chon5; con6; zon3
U+7BC6.0	fCantonese	syun6
U+7BC6.0	fMandarin	zhuan4
U+7BC6.0	fEnglish	[1] the seal script [2] seal; official stamp [3] a Chinese family name
U+7BC7.0	fUTF8	篇
U+7BC7.0	fCNS11643	1-6E32
U+7BC7.0	fBig5	BD67
U+7BC7.0	fGB	0-462A
U+7BC7.0	fR/S	 118.9
U+7BC7.0	fTotalStrokes	 15
U+7BC7.0	fCangjie	HHSB
U+7BC7.0	fFourCorner	88227
U+7BC7.0	fHakka	 pien1; pen1; p'ian1
U+7BC7.0	fCantonese	pin1
U+7BC7.0	fMandarin	pian1
U+7BC7.0	fEnglish	[1] numerical classifier for essays, articles, compositions, poems [2] chapter; section; part [3] page [4] books; volumes
U+7BC8.0	fUTF8	篈
U+7BC8.0	fCNS11643	4-4B76
U+7BC8.0	fGB	3-6443
U+7BC8.0	fR/S	 118.9
U+7BC8.0	fTotalStrokes	 15
U+7BC8.0	fCangjie	HGGI
U+7BC8.0	fFourCorner	88140
U+7BC8.0	fCantonese	fung1
U+7BC8.0	fMandarin	feng1
U+7BC9.0	fUTF8	築
U+7BC9.0	fCNS11643	1-7160
U+7BC9.0	fBig5	BF76
U+7BC9.0	fGB	1-567E
U+7BC9.0	fR/S	 118.10
U+7BC9.0	fTotalStrokes	 16
U+7BC9.0	fCangjie	HMND
U+7BC9.0	fFourCorner	88904
U+7BC9.0	fHakka	 zuk7; chuk7; zhuk7
U+7BC9.0	fCantonese	zuk1
U+7BC9.0	fMandarin	zhu4; zhu2
U+7BC9.0	fEnglish	[1] build (out of earth, rock, etc.); erect [2] building; house; room
U+7BCA.0	fUTF8	篊
U+7BCA.0	fCNS11643	2-5265
U+7BCA.0	fBig5	E6DD
U+7BCA.0	fGB	3-645A
U+7BCA.0	fR/S	 118.9
U+7BCA.0	fTotalStrokes	 15
U+7BCA.0	fCangjie	HETC
U+7BCA.0	fFourCorner	88181
U+7BCA.0	fCantonese	hung4
U+7BCA.0	fMandarin	hong2; hong4
U+7BCB.0	fUTF8	篋
U+7BCB.0	fCNS11643	2-525F
U+7BCB.0	fBig5	E6D7
U+7BCB.0	fGB	1-7366
U+7BCB.0	fR/S	 118.9
U+7BCB.0	fTotalStrokes	 15
U+7BCB.0	fCangjie	HSKO
U+7BCB.0	fFourCorner	88713
U+7BCB.0	fHakka	  giap7; kiap7; hiap7
U+7BCB.0	fCantonese	haap6; gip2; hip3
U+7BCB.0	fMandarin	qie4
U+7BCB.0	fEnglish	 chest; box; trunk; case
U+7BCC.0	fUTF8	篌
U+7BCC.0	fCNS11643	1-6E35
U+7BCC.0	fBig5	BD6A
U+7BCC.0	fGB	0-7373
U+7BCC.0	fR/S	 118.9
U+7BCC.0	fTotalStrokes	 15
U+7BCC.0	fCangjie	HONK
U+7BCC.0	fFourCorner	88234
U+7BCC.0	fHakka	  heu2; hiu2
U+7BCC.0	fCantonese	hau4
U+7BCC.0	fMandarin	hou2
U+7BCC.0	fEnglish	a kind of ancient music instrument like a lute
U+7BCD.0	fUTF8	篍
U+7BCD.0	fCNS11643	3-4D63
U+7BCD.0	fGB	3-6450
U+7BCD.0	fR/S	 118.9
U+7BCD.0	fTotalStrokes	 15
U+7BCD.0	fCangjie	HHDF
U+7BCD.0	fFourCorner	88980
U+7BCD.0	fCantonese	cau1
U+7BCD.0	fMandarin	qiu1
U+7BCE.0	fUTF8	篎
U+7BCE.0	fCNS11643	2-5262
U+7BCE.0	fBig5	E6DA
U+7BCE.0	fGB	3-644C
U+7BCE.0	fR/S	 118.9
U+7BCE.0	fTotalStrokes	 15
U+7BCE.0	fCangjie	HBUH
U+7BCE.0	fFourCorner	88629
U+7BCE.0	fCantonese	miu5; miu6
U+7BCE.0	fMandarin	miao3; miao4
U+7BCF.0	fUTF8	篏
U+7BCF.0	fGB	5-635A
U+7BCF.0	fR/S	 118.9
U+7BCF.0	fTotalStrokes	 15
U+7BCF.0	fCangjie	HTMO
U+7BCF.0	fFourCorner	8878
U+7BCF.0	fCantonese	ham3
U+7BCF.0	fMandarin	qian4
U+7BCF.0	fEnglish	 inlay; set in fall into rugged
U+7BD0.0	fUTF8	篐
U+7BD0.0	fCNS11643	3-6621
U+7BD0.0	fGB	E-3731
U+7BD0.0	fR/S	 118.9
U+7BD0.0	fTotalStrokes	 15
U+7BD0.0	fCangjie	HDSB
U+7BD0.0	fFourCorner	8841
U+7BD0.0	fCantonese	ku1; kwu1 (also)
U+7BD1.0	fUTF8	篑
U+7BD1.0	fGB	0-7371
U+7BD1.0	fR/S	 118.9
U+7BD1.0	fTotalStrokes	 15
U+7BD1.0	fCangjie	XHLMO
U+7BD1.0	fHakka	 k'wui5
U+7BD1.0	fCantonese	gwai6
U+7BD1.0	fMandarin	kui4; kuai4
U+7BD1.0	fEnglish	simplified form of &#x7C23;
U+7BD2.0	fUTF8	篒
U+7BD2.0	fGB	E-3732
U+7BD2.0	fR/S	 118.9
U+7BD2.0	fTotalStrokes	 15
U+7BD2.0	fCangjie	HOIV
U+7BD2.0	fCantonese	sik6
U+7BD3.0	fUTF8	篓
U+7BD3.0	fGB	0-4228
U+7BD3.0	fR/S	 118.9
U+7BD3.0	fTotalStrokes	 15
U+7BD3.0	fCangjie	HFDV
U+7BD3.0	fHakka	 leu2; leu3
U+7BD3.0	fCantonese	lau5
U+7BD3.0	fMandarin	lou3; ju4; lou2
U+7BD3.0	fEnglish	simplified form of &#x7C0D;
U+7BD4.0	fUTF8	篔
U+7BD4.0	fCNS11643	2-592C
U+7BD4.0	fBig5	EAC0
U+7BD4.0	fGB	3-6424
U+7BD4.0	fR/S	 118.10
U+7BD4.0	fTotalStrokes	 16
U+7BD4.0	fCangjie	HRBC
U+7BD4.0	fFourCorner	88806
U+7BD4.0	fHakka	 jun2; run2; ren2; jen2
U+7BD4.0	fCantonese	wan4
U+7BD4.0	fMandarin	yun2; yuan2
U+7BD4.0	fEnglish	tall bamboo
U+7BD5.0	fUTF8	篕
U+7BD5.0	fCNS11643	2-5927
U+7BD5.0	fBig5	EABB
U+7BD5.0	fGB	5-636B
U+7BD5.0	fR/S	 118.10
U+7BD5.0	fTotalStrokes	 16
U+7BD5.0	fCangjie	HGIT
U+7BD5.0	fFourCorner	88107
U+7BD5.0	fCantonese	hap6
U+7BD5.0	fMandarin	he2; gai4
U+7BD6.0	fUTF8	篖
U+7BD6.0	fCNS11643	4-522D
U+7BD6.0	fGB	3-646D
U+7BD6.0	fR/S	  118.10
U+7BD6.0	fTotalStrokes	 16
U+7BD6.0	fCangjie	HILR
U+7BD6.0	fFourCorner	88260
U+7BD6.0	fCantonese	tong4
U+7BD6.0	fMandarin	tang2
U+7BD7.0	fUTF8	篗
U+7BD7.0	fCNS11643	5-5669
U+7BD7.0	fGB	3-646A
U+7BD7.0	fR/S	  118.10
U+7BD7.0	fTotalStrokes	 16
U+7BD7.0	fCangjie	HOGE
U+7BD7.0	fFourCorner	88400
U+7BD7.0	fCantonese	wok6; jyut6 (also)
U+7BD7.0	fMandarin	yue4
U+7BD8.0	fUTF8	篘
U+7BD8.0	fCNS11643	2-5931
U+7BD8.0	fBig5	EAC5
U+7BD8.0	fGB	3-635D
U+7BD8.0	fR/S	 118.10
U+7BD8.0	fTotalStrokes	 16
U+7BD8.0	fCangjie	HPUU
U+7BD8.0	fFourCorner	88427
U+7BD8.0	fHakka	 ceu1
U+7BD8.0	fCantonese	cau1
U+7BD8.0	fMandarin	chou1
U+7BD8.0	fEnglish	[1] wine filter [2] filter wine
U+7BD9.0	fUTF8	篙
U+7BD9.0	fCNS11643	1-715E
U+7BD9.0	fBig5	BF74
U+7BD9.0	fGB	0-385D
U+7BD9.0	fR/S	 118.10
U+7BD9.0	fTotalStrokes	 16
U+7BD9.0	fCangjie	HYRB
U+7BD9.0	fFourCorner	88227
U+7BD9.0	fHakka	  gau1; go1
U+7BD9.0	fCantonese	gou1
U+7BD9.0	fMandarin	gao1; gao4
U+7BD9.0	fEnglish	 pole used for punting a boat; boat pole
U+7BDA.0	fUTF8	篚
U+7BDA.0	fCNS11643	2-5929
U+7BDA.0	fBig5	EABD
U+7BDA.0	fGB	0-7375
U+7BDA.0	fR/S	 118.10
U+7BDA.0	fTotalStrokes	 16
U+7BDA.0	fCangjie	HSLY
U+7BDA.0	fFourCorner	88711
U+7BDA.0	fHakka	 fi3; fui3
U+7BDA.0	fCantonese	fei2
U+7BDA.0	fMandarin	fei3
U+7BDA.0	fEnglish	square bamboo basket
U+7BDB.0	fUTF8	篛
U+7BDB.0	fCNS11643	1-7162
U+7BDB.0	fBig5	BF78
U+7BDB.0	fGB	E-3733
U+7BDB.0	fR/S	 118.10
U+7BDB.0	fTotalStrokes	 16
U+7BDB.0	fCangjie	HNMM
U+7BDB.0	fFourCorner	88127
U+7BDB.0	fHakka	 niok7
U+7BDB.0	fCantonese	joek6
U+7BDB.0	fMandarin	ruo4
U+7BDB.0	fEnglish	cuticle of bamboo plant
U+7BDC.0	fUTF8	篜
U+7BDC.0	fCNS11643	2-592F
U+7BDC.0	fBig5	EAC3
U+7BDC.0	fGB	3-6473
U+7BDC.0	fR/S	 118.10
U+7BDC.0	fTotalStrokes	 16
U+7BDC.0	fCangjie	HNEF
U+7BDC.0	fFourCorner	88227
U+7BDC.0	fCantonese	zing1
U+7BDC.0	fMandarin	zheng1
U+7BDD.0	fUTF8	篝
U+7BDD.0	fCNS11643	2-5926
U+7BDD.0	fBig5	EABA
U+7BDD.0	fGB	0-7374
U+7BDD.0	fR/S	 118.10
U+7BDD.0	fTotalStrokes	 16
U+7BDD.0	fCangjie	HTTB
U+7BDD.0	fFourCorner	88557
U+7BDD.0	fHakka	 kieu1; giu1; geu1
U+7BDD.0	fCantonese	gau1; kau1 (also)
U+7BDD.0	fMandarin	gou1; dao3; gou4
U+7BDD.0	fEnglish	 basket; cage
U+7BDE.0	fUTF8	篞
U+7BDE.0	fCNS11643	2-5923
U+7BDE.0	fBig5	EAB7
U+7BDE.0	fGB	5-6379
U+7BDE.0	fR/S	 118.10
U+7BDE.0	fTotalStrokes	 16
U+7BDE.0	fCangjie	HEAG
U+7BDE.0	fFourCorner	88311
U+7BDE.0	fCantonese	nit6; lit6 (also); nip6
U+7BDE.0	fMandarin	nie4
U+7BDF.0	fUTF8	篟
U+7BDF.0	fCNS11643	2-5932
U+7BDF.0	fBig5	EAC6
U+7BDF.0	fGB	3-6469
U+7BDF.0	fR/S	 118.10
U+7BDF.0	fTotalStrokes	 16
U+7BDF.0	fCangjie	HOQB
U+7BDF.0	fFourCorner	88227
U+7BDF.0	fCantonese	sin3
U+7BDF.0	fMandarin	qian4
U+7BE0.0	fUTF8	篠
U+7BE0.0	fCNS11643	1-745B
U+7BE0.0	fBig5	C151
U+7BE0.0	fGB	3-642A
U+7BE0.0	fR/S	 118.10
U+7BE0.0	fTotalStrokes	 16
U+7BE0.0	fCangjie	HOLD
U+7BE0.0	fFourCorner	88293
U+7BE0.0	fHakka	 tiau3;  siau1
U+7BE0.0	fCantonese	siu2; siu6 (also)
U+7BE0.0	fMandarin	xiao3
U+7BE0.0	fEnglish	[1] a variety of bamboo with thin and short stems [2] bamboo basket
U+7BE1.0	fUTF8	篡
U+7BE1.0	fCNS11643	1-7163
U+7BE1.0	fBig5	BF79
U+7BE1.0	fGB	0-345B
U+7BE1.0	fR/S	 118.10
U+7BE1.0	fTotalStrokes	 16
U+7BE1.0	fCangjie	HBUI
U+7BE1.0	fFourCorner	88732
U+7BE1.0	fHakka	  con5; chon5; con3; chon1; con1
U+7BE1.0	fCantonese	saan3
U+7BE1.0	fMandarin	cuan4
U+7BE1.0	fEnglish	 seize (power, throne, etc.); usurp
U+7BE2.0	fUTF8	篢
U+7BE2.0	fCNS11643	2-592E
U+7BE2.0	fBig5	EAC2
U+7BE2.0	fGB	3-637B
U+7BE2.0	fR/S	 118.10
U+7BE2.0	fTotalStrokes	 16
U+7BE2.0	fCangjie	HMBC
U+7BE2.0	fFourCorner	88806
U+7BE2.0	fCantonese	lung5; gung1
U+7BE2.0	fMandarin	gong1; gan3; long3
U+7BE2.0	fEnglish	the name of bamboo hat
U+7BE3.0	fUTF8	篣
U+7BE3.0	fCNS11643	2-5924
U+7BE3.0	fBig5	EAB8
U+7BE3.0	fGB	3-6470
U+7BE3.0	fR/S	 118.10
U+7BE3.0	fTotalStrokes	 16
U+7BE3.0	fCangjie	HYBS
U+7BE3.0	fFourCorner	88227
U+7BE3.0	fHakka	 bong3
U+7BE3.0	fCantonese	pong4; pang4
U+7BE3.0	fMandarin	pang2; peng2
U+7BE4.0	fUTF8	篤
U+7BE4.0	fCNS11643	1-7161
U+7BE4.0	fBig5	BF77
U+7BE4.0	fGB	1-7346
U+7BE4.0	fR/S	 118.10
U+7BE4.0	fTotalStrokes	 16
U+7BE4.0	fCangjie	HSQF
U+7BE4.0	fFourCorner	88327
U+7BE4.0	fHakka	  duk7; duk8
U+7BE4.0	fCantonese	duk1
U+7BE4.0	fMandarin	du3
U+7BE4.0	fEnglish	[1] deep; much; great; profound [2] dangerous; serious [3] generous [4] consolidate; make solid [5] limit
U+7BE5.0	fUTF8	篥
U+7BE5.0	fCNS11643	2-5928
U+7BE5.0	fBig5	EABC
U+7BE5.0	fGB	0-7376
U+7BE5.0	fR/S	 118.10
U+7BE5.0	fTotalStrokes	 16
U+7BE5.0	fCangjie	HMWD
U+7BE5.0	fFourCorner	88904
U+7BE5.0	fHakka	 lit8
U+7BE5.0	fCantonese	leot6
U+7BE5.0	fMandarin	li4
U+7BE5.0	fEnglish	bulgle
U+7BE6.0	fUTF8	篦
U+7BE6.0	fCNS11643	1-7165
U+7BE6.0	fBig5	BF7B
U+7BE6.0	fGB	0-7377
U+7BE6.0	fR/S	 118.10
U+7BE6.0	fTotalStrokes	 16
U+7BE6.0	fCangjie	HHWP
U+7BE6.0	fFourCorner	88711
U+7BE6.0	fHakka	  pi5; bin1;  bi3; bi1; bi5
U+7BE6.0	fCantonese	bei6; bei1 (also); bei2 (also); bai1
U+7BE6.0	fMandarin	bi4; pi2; bi1
U+7BE6.0	fEnglish	 fine-toothed comb
U+7BE7.0	fUTF8	篧
U+7BE7.0	fCNS11643	2-5925
U+7BE7.0	fBig5	EAB9
U+7BE7.0	fGB	5-637B
U+7BE7.0	fR/S	 118.10
U+7BE7.0	fTotalStrokes	 16
U+7BE7.0	fCangjie	HOBG
U+7BE7.0	fFourCorner	88214
U+7BE7.0	fHakka	 kok7; fu5; zok7
U+7BE7.0	fCantonese	zuk1; wok6; zok6; fok3
U+7BE7.0	fMandarin	zhuo2; hu4; kuo4; zhuo1
U+7BE7.0	fEnglish	 fishing creel
U+7BE8.0	fUTF8	篨
U+7BE8.0	fCNS11643	2-592A
U+7BE8.0	fBig5	EABE
U+7BE8.0	fGB	3-645F
U+7BE8.0	fR/S	 118.10
U+7BE8.0	fTotalStrokes	 16
U+7BE8.0	fCangjie	HNLD
U+7BE8.0	fFourCorner	88294
U+7BE8.0	fHakka	 chu2; cu2
U+7BE8.0	fCantonese	ceoi4; cyu4 (also); cyu1
U+7BE8.0	fMandarin	chu2
U+7BE8.0	fEnglish	[1] coarse bamboo mat [2] an ugly bloated mass
U+7BE9.0	fUTF8	篩
U+7BE9.0	fCNS11643	1-7164
U+7BE9.0	fBig5	BF7A
U+7BE9.0	fGB	1-4938
U+7BE9.0	fR/S	 118.10
U+7BE9.0	fTotalStrokes	 16
U+7BE9.0	fCangjie	HHRB
U+7BE9.0	fFourCorner	88727
U+7BE9.0	fHakka	  si1; sai1; chi1; ci1; che2; ce2; ce1
U+7BE9.0	fCantonese	sai1; si1
U+7BE9.0	fMandarin	shai1; shi1
U+7BE9.0	fEnglish	[1] sieve; filter; screen; sifter [2] sieve; screen; sift; strain
U+7BEA.0	fUTF8	篪
U+7BEA.0	fCNS11643	2-592D
U+7BEA.0	fBig5	EAC1
U+7BEA.0	fGB	0-7378
U+7BEA.0	fR/S	 118.10
U+7BEA.0	fTotalStrokes	 16
U+7BEA.0	fCangjie	HHYU
U+7BEA.0	fFourCorner	88217
U+7BEA.0	fHakka	 chi1; cih1; chi2
U+7BEA.0	fCantonese	ci4
U+7BEA.0	fMandarin	chi2
U+7BEA.0	fEnglish	a kind of bamboo flute with seven holes
U+7BEB.0	fUTF8	篫
U+7BEB.0	fCNS11643	2-5930
U+7BEB.0	fBig5	EAC4
U+7BEB.0	fGB	5-636A
U+7BEB.0	fR/S	 118.10
U+7BEB.0	fTotalStrokes	 16
U+7BEB.0	fCangjie	HMNQ
U+7BEB.0	fFourCorner	88502
U+7BEB.0	fCantonese	zuk1
U+7BEB.0	fMandarin	zhu2
U+7BEC.0	fUTF8	篬
U+7BEC.0	fCNS11643	3-5233
U+7BEC.0	fGB	5-627A
U+7BEC.0	fR/S	  118.10
U+7BEC.0	fTotalStrokes	 16
U+7BEC.0	fCangjie	HOIR
U+7BEC.0	fFourCorner	88600
U+7BEC.0	fCantonese	coeng1
U+7BEC.0	fMandarin	qiang1
U+7BED.0	fUTF8	篭
U+7BED.0	fCNS11643	3-5234
U+7BED.0	fGB	E-3734
U+7BED.0	fR/S	 118.10
U+7BED.0	fTotalStrokes	 16
U+7BED.0	fCangjie	HYTU
U+7BED.0	fFourCorner	88710
U+7BED.0	fHakka	 lung1
U+7BED.0	fCantonese	lung4
U+7BED.0	fMandarin	long2; long3
U+7BED.0	fEnglish	 cage cage-like basket
U+7BEE.0	fUTF8	篮
U+7BEE.0	fGB	0-403A
U+7BEE.0	fR/S	 118.10
U+7BEE.0	fTotalStrokes	 16
U+7BEE.0	fCangjie	HLIT
U+7BEE.0	fHakka	 lam2
U+7BEE.0	fCantonese	laam4; laam2 (also)
U+7BEE.0	fMandarin	lan2
U+7BEE.0	fEnglish	simplified form of &#x7C43;
U+7BEF.0	fUTF8	篯
U+7BEF.0	fGB	8-7E29
U+7BEF.0	fR/S	 118.10
U+7BEF.0	fTotalStrokes	 16
U+7BEF.0	fCangjie	HCIJ
U+7BEF.0	fCantonese	zin1
U+7BEF.0	fMandarin	jian1
U+7BF0.0	fUTF8	篰
U+7BF0.0	fCNS11643	2-5E38
U+7BF0.0	fBig5	EDCB
U+7BF0.0	fGB	3-646E
U+7BF0.0	fR/S	 118.11
U+7BF0.0	fTotalStrokes	 17
U+7BF0.0	fCangjie	HYRL
U+7BF0.0	fFourCorner	88627
U+7BF0.0	fCantonese	bou6
U+7BF0.0	fMandarin	bu4; pou3
U+7BF0.0	fEnglish	[1] bamboo basket [2] same as $
U+7BF1.0	fUTF8	篱
U+7BF1.0	fCNS11643	2-5E39
U+7BF1.0	fBig5	EDCC
U+7BF1.0	fGB	0-4069
U+7BF1.0	fR/S	 118.11
U+7BF1.0	fTotalStrokes	 17
U+7BF1.0	fCangjie	HYUB
U+7BF1.0	fFourCorner	88227
U+7BF1.0	fHakka	  li2
U+7BF1.0	fCantonese	lei4; lei1 (also)
U+7BF1.0	fMandarin	li2
U+7BF1.0	fEnglish	simplified form of &#x7C6C;
U+7BF2.0	fUTF8	篲
U+7BF2.0	fCNS11643	2-5E29
U+7BF2.0	fBig5	EDBC
U+7BF2.0	fGB	5-637E
U+7BF2.0	fR/S	 118.11
U+7BF2.0	fTotalStrokes	 17
U+7BF2.0	fCangjie	HQJM
U+7BF2.0	fFourCorner	88171
U+7BF2.0	fHakka	  sui5; fui5; fi5
U+7BF2.0	fCantonese	wai6; seoi6 (also)
U+7BF2.0	fMandarin	hui4; sui4; rui4; wei4
U+7BF2.0	fEnglish	 broom; bamboo broom
U+7BF3.0	fUTF8	篳
U+7BF3.0	fCNS11643	2-5E30
U+7BF3.0	fBig5	EDC3
U+7BF3.0	fGB	1-7359
U+7BF3.0	fR/S	 118.11
U+7BF3.0	fTotalStrokes	 17
U+7BF3.0	fCangjie	HWTJ
U+7BF3.0	fFourCorner	88504
U+7BF3.0	fHakka	 bit7
U+7BF3.0	fCantonese	bat1
U+7BF3.0	fMandarin	bi4
U+7BF3.0	fEnglish	wicker, bamboo, etc. which can be used to make baskets, bags, etc.
U+7BF4.0	fUTF8	篴
U+7BF4.0	fCNS11643	2-5E2E
U+7BF4.0	fBig5	EDC1
U+7BF4.0	fGB	5-6370
U+7BF4.0	fR/S	 118.11
U+7BF4.0	fTotalStrokes	 17
U+7BF4.0	fCangjie	HYMO
U+7BF4.0	fFourCorner	88303
U+7BF4.0	fHakka	 tit8
U+7BF4.0	fCantonese	dek6; dik6
U+7BF4.0	fMandarin	di2; zhu2
U+7BF4.0	fEnglish	 bamboo flute whistle
U+7BF5.0	fUTF8	篵
U+7BF5.0	fCNS11643	4-575F
U+7BF5.0	fGB	E-3735
U+7BF5.0	fR/S	  118.11
U+7BF5.0	fTotalStrokes	 17; 16
U+7BF5.0	fCangjie	HHOO
U+7BF5.0	fFourCorner	88280
U+7BF5.0	fCantonese	cung1
U+7BF5.0	fMandarin	cong1
U+7BF6.0	fUTF8	篶
U+7BF6.0	fCNS11643	3-556D
U+7BF6.0	fGB	E-3736
U+7BF6.0	fR/S	  118.11
U+7BF6.0	fTotalStrokes	 17
U+7BF6.0	fCangjie	HMYF
U+7BF6.0	fFourCorner	88320
U+7BF6.0	fCantonese	jin1
U+7BF6.0	fMandarin	yan1
U+7BF7.0	fUTF8	篷
U+7BF7.0	fCNS11643	1-7459
U+7BF7.0	fBig5	C14F
U+7BF7.0	fGB	0-4571
U+7BF7.0	fR/S	 118.11
U+7BF7.0	fTotalStrokes	 17
U+7BF7.0	fCangjie	HYHJ
U+7BF7.0	fFourCorner	88304
U+7BF7.0	fHakka	  pung2
U+7BF7.0	fCantonese	pung4
U+7BF7.0	fMandarin	peng2
U+7BF7.0	fEnglish	[1] awning; covering; tent [2] sail; boat
U+7BF8.0	fUTF8	篸
U+7BF8.0	fCNS11643	2-5E35
U+7BF8.0	fBig5	EDC8
U+7BF8.0	fGB	5-6359
U+7BF8.0	fR/S	 118.11
U+7BF8.0	fTotalStrokes	 17
U+7BF8.0	fCangjie	HIIH
U+7BF8.0	fFourCorner	88202
U+7BF8.0	fHakka	 sem1; ciam1; cam1
U+7BF8.0	fCantonese	caam2; cam1; zaam1
U+7BF8.0	fMandarin	sen1; zan1; cen1; zan4
U+7BF8.0	fEnglish	the tubes in pan-pipes
U+7BF9.0	fUTF8	篹
U+7BF9.0	fCNS11643	2-592B
U+7BF9.0	fBig5	EABF
U+7BF9.0	fGB	E-3737
U+7BF9.0	fR/S	 118.11
U+7BF9.0	fTotalStrokes	 17
U+7BF9.0	fCangjie	HBUU
U+7BF9.0	fFourCorner	88712
U+7BF9.0	fCantonese	zyun2; syun2
U+7BF9.0	fMandarin	zuan3; zhui4; zuan4
U+7BF9.0	fEnglish	 bamboo basket; splint basket
U+7BF9.1	fUTF8	篹
U+7BF9.1	fCNS11643	2-592B
U+7BF9.1	fBig5	EABF
U+7BF9.1	fGB	E-3737
U+7BF9.1	fR/S	 118.11
U+7BF9.1	fTotalStrokes	 17
U+7BF9.1	fCangjie	HBUU
U+7BF9.1	fFourCorner	88712
U+7BF9.1	fHakka	 sien3; con3; cion5; con5
U+7BF9.1	fCantonese	zaan6
U+7BF9.1	fMandarin	zhuan4; suan3; suan4; xuan3; xuan4; zhuan1
U+7BF9.1	fEnglish	same as &#x64B0;
U+7BFA.0	fUTF8	篺
U+7BFA.0	fCNS11643	3-556B
U+7BFA.0	fGB	5-6422
U+7BFA.0	fR/S	 118.11
U+7BFA.0	fTotalStrokes	 17
U+7BFA.0	fCangjie	HQHJ
U+7BFA.0	fFourCorner	88540
U+7BFA.0	fCantonese	paai4
U+7BFA.0	fMandarin	pai2
U+7BFA.0	fEnglish	 raft; bamboo raft
U+7BFB.0	fUTF8	篻
U+7BFB.0	fCNS11643	2-5E2C
U+7BFB.0	fBig5	EDBF
U+7BFB.0	fGB	3-6478
U+7BFB.0	fR/S	 118.11
U+7BFB.0	fTotalStrokes	 17
U+7BFB.0	fCangjie	HMWF
U+7BFB.0	fFourCorner	88901
U+7BFB.0	fCantonese	piu3
U+7BFB.0	fMandarin	piao4; piao3; miao2
U+7BFC.0	fUTF8	篼
U+7BFC.0	fCNS11643	3-556E
U+7BFC.0	fGB	0-737B
U+7BFC.0	fR/S	 118.11
U+7BFC.0	fTotalStrokes	 17
U+7BFC.0	fCangjie	HHVU
U+7BFC.0	fFourCorner	88210
U+7BFC.0	fHakka	 deu1; diu1
U+7BFC.0	fCantonese	dau1
U+7BFC.0	fMandarin	dou1
U+7BFC.0	fEnglish	a mountain sedan chair made of bamboo
U+7BFD.0	fUTF8	篽
U+7BFD.0	fCNS11643	2-5E36
U+7BFD.0	fBig5	EDC9
U+7BFD.0	fGB	E-3738
U+7BFD.0	fR/S	 118.11
U+7BFD.0	fTotalStrokes	 17
U+7BFD.0	fCangjie	HHOL
U+7BFD.0	fFourCorner	88227
U+7BFD.0	fCantonese	jyu5
U+7BFD.0	fMandarin	yu3
U+7BFE.0	fUTF8	篾
U+7BFE.0	fCNS11643	1-7458
U+7BFE.0	fBig5	C14E
U+7BFE.0	fGB	0-737A
U+7BFE.0	fR/S	 118.11
U+7BFE.0	fTotalStrokes	 17
U+7BFE.0	fCangjie	HWLI
U+7BFE.0	fFourCorner	88253
U+7BFE.0	fHakka	  met8
U+7BFE.0	fCantonese	mit6
U+7BFE.0	fMandarin	mie4
U+7BFE.0	fEnglish	[1] thin and long strip of bamboo for making baskets; thin (bamboo) laths [2] name of a variety of bamboo
U+7BFF.0	fUTF8	篿
U+7BFF.0	fCNS11643	2-5E2B
U+7BFF.0	fBig5	EDBE
U+7BFF.0	fGB	3-6349
U+7BFF.0	fR/S	 118.11
U+7BFF.0	fTotalStrokes	 17
U+7BFF.0	fCangjie	HJII
U+7BFF.0	fFourCorner	88343
U+7BFF.0	fCantonese	tyun4; zyun1
U+7BFF.0	fMandarin	zhuan1; tuan2
U+7C00.0	fUTF8	簀
U+7C00.0	fCNS11643	2-5E2A
U+7C00.0	fBig5	EDBD
U+7C00.0	fGB	1-7365
U+7C00.0	fR/S	 118.11
U+7C00.0	fTotalStrokes	 17
U+7C00.0	fCangjie	HQMC
U+7C00.0	fFourCorner	88806
U+7C00.0	fHakka	  zit7
U+7C00.0	fCantonese	zaak3
U+7C00.0	fMandarin	ze2; zhai4; kui4
U+7C00.0	fEnglish	[1] bamboo bed mat; mat [2] be densely collected together
U+7C01.0	fUTF8	簁
U+7C01.0	fCNS11643	2-5E34
U+7C01.0	fBig5	EDC7
U+7C01.0	fGB	3-647D
U+7C01.0	fR/S	 118.11
U+7C01.0	fTotalStrokes	 17
U+7C01.0	fCangjie	HHOO
U+7C01.0	fFourCorner	88281
U+7C01.0	fHakka	 sii1; sai3; si1; ce1
U+7C01.0	fCantonese	si1; sai1; saai1
U+7C01.0	fMandarin	xi3; shai1; si1
U+7C01.0	fEnglish	 sieve sift; strain
U+7C02.0	fUTF8	簂
U+7C02.0	fCNS11643	2-5E31
U+7C02.0	fBig5	EDC4
U+7C02.0	fGB	5-634F
U+7C02.0	fR/S	 118.11
U+7C02.0	fTotalStrokes	 17
U+7C02.0	fCangjie	HWIM
U+7C02.0	fFourCorner	88600
U+7C02.0	fCantonese	gwok3; gok3 (also)
U+7C02.0	fMandarin	guo2; gui1; gui4
U+7C03.0	fUTF8	簃
U+7C03.0	fCNS11643	2-5E33
U+7C03.0	fBig5	EDC6
U+7C03.0	fGB	3-647C
U+7C03.0	fR/S	 118.11
U+7C03.0	fTotalStrokes	 17
U+7C03.0	fCangjie	HHDN
U+7C03.0	fFourCorner	88927
U+7C03.0	fCantonese	ji4 (also); jyu4
U+7C03.0	fMandarin	yi2; chi2; zhi4
U+7C03.0	fEnglish	a small house attached to a pavilion
U+7C04.0	fUTF8	簄
U+7C04.0	fCNS11643	4-5754
U+7C04.0	fGB	E-3739
U+7C04.0	fR/S	  118.11
U+7C04.0	fTotalStrokes	 17; 16
U+7C04.0	fCangjie	HHSU
U+7C04.0	fFourCorner	88210
U+7C04.0	fHakka	 fu5; fu6
U+7C04.0	fCantonese	wu6
U+7C04.0	fMandarin	hu4
U+7C04.0	fEnglish	fishing-stakes
U+7C05.0	fUTF8	簅
U+7C05.0	fCNS11643	2-5E27
U+7C05.0	fBig5	EDBA
U+7C05.0	fGB	5-6336
U+7C05.0	fR/S	 118.11
U+7C05.0	fTotalStrokes	 17
U+7C05.0	fCangjie	HYHM
U+7C05.0	fFourCorner	88215
U+7C05.0	fCantonese	caan2
U+7C05.0	fMandarin	chan3
U+7C06.0	fUTF8	簆
U+7C06.0	fCNS11643	2-5E37
U+7C06.0	fBig5	EDCA
U+7C06.0	fGB	E-373A
U+7C06.0	fR/S	 118.11
U+7C06.0	fTotalStrokes	 17
U+7C06.0	fCangjie	HJMK
U+7C06.0	fFourCorner	88214
U+7C06.0	fHakka	  keu5; kiu5
U+7C06.0	fCantonese	kau3; kau4
U+7C06.0	fMandarin	kou4
U+7C06.0	fEnglish	reed
U+7C07.0	fUTF8	簇
U+7C07.0	fCNS11643	1-7456
U+7C07.0	fBig5	C14C
U+7C07.0	fGB	0-3458
U+7C07.0	fR/S	 118.11
U+7C07.0	fTotalStrokes	 17
U+7C07.0	fCangjie	HYSK
U+7C07.0	fFourCorner	88234
U+7C07.0	fHakka	 cuk8
U+7C07.0	fCantonese	cuk1; zuk6
U+7C07.0	fMandarin	cu4; chuo4; cou4
U+7C07.0	fEnglish	[1] swarm; crowd; crowded; crowd together; cluster [2] arrowhead [3] a framework on which silkworms spin [4] crowd together; cluster together
U+7C08.0	fUTF8	簈
U+7C08.0	fCNS11643	3-5570
U+7C08.0	fGB	E-373B
U+7C08.0	fR/S	 118.11
U+7C08.0	fTotalStrokes	 17
U+7C08.0	fCangjie	XHSYJ
U+7C08.0	fFourCorner	88240
U+7C08.0	fHakka	  pin2; biang3
U+7C08.0	fCantonese	ping4
U+7C08.0	fMandarin	ping2; bing1
U+7C08.0	fEnglish	bamboo mat used to protect carriage passengers from dust
U+7C09.0	fUTF8	簉
U+7C09.0	fCNS11643	2-5E32
U+7C09.0	fBig5	EDC5
U+7C09.0	fGB	3-6468
U+7C09.0	fR/S	 118.11
U+7C09.0	fTotalStrokes	 17
U+7C09.0	fCangjie	HYHR
U+7C09.0	fFourCorner	88306
U+7C09.0	fHakka	  ceu5;  cau5; co5
U+7C09.0	fCantonese	zou6; cau3; cou3
U+7C09.0	fMandarin	zao4; chou4
U+7C09.0	fEnglish	[1] deputy; subordinate; attendant [2] concubine [3] escorting vehicle
U+7C0A.0	fUTF8	簊
U+7C0A.0	fCNS11643	2-5E3B
U+7C0A.0	fBig5	EDCE
U+7C0A.0	fGB	3-6474
U+7C0A.0	fR/S	 118.11
U+7C0A.0	fTotalStrokes	 17
U+7C0A.0	fCangjie	HTCG
U+7C0A.0	fFourCorner	88104
U+7C0A.0	fHakka	 gi1
U+7C0A.0	fCantonese	gei1
U+7C0A.0	fMandarin	ji1
U+7C0B.0	fUTF8	簋
U+7C0B.0	fCNS11643	2-5E2F
U+7C0B.0	fBig5	EDC2
U+7C0B.0	fGB	0-737E
U+7C0B.0	fR/S	 118.11
U+7C0B.0	fTotalStrokes	 17
U+7C0B.0	fCangjie	HAVT
U+7C0B.0	fFourCorner	88107
U+7C0B.0	fHakka	 kui3; kui1; kwui3; gwi3; k'wui3
U+7C0B.0	fCantonese	gwai2
U+7C0B.0	fMandarin	gui3
U+7C0B.0	fEnglish	a square basket or vessel of bamboo for holding grain used at sacrifices, feast; bronze vessel for holding food
U+7C0C.0	fUTF8	簌
U+7C0C.0	fCNS11643	1-745A
U+7C0C.0	fBig5	C150
U+7C0C.0	fGB	0-7379
U+7C0C.0	fR/S	 118.11
U+7C0C.0	fTotalStrokes	 17
U+7C0C.0	fCangjie	HDLO
U+7C0C.0	fFourCorner	88582
U+7C0C.0	fHakka	  suk7
U+7C0C.0	fCantonese	cuk1; suk1
U+7C0C.0	fMandarin	su4
U+7C0C.0	fEnglish	falling in great quantities (of flower petals)
U+7C0D.0	fUTF8	簍
U+7C0D.0	fCNS11643	1-7457
U+7C0D.0	fBig5	C14D
U+7C0D.0	fGB	1-4228
U+7C0D.0	fR/S	 118.11
U+7C0D.0	fTotalStrokes	 17
U+7C0D.0	fCangjie	HLLV
U+7C0D.0	fFourCorner	88404
U+7C0D.0	fHakka	 leu3; leu2; lam1; lui3; liu3
U+7C0D.0	fCantonese	lau5
U+7C0D.0	fMandarin	lou3; ju4; l3; lou2
U+7C0D.0	fEnglish	a bamboo basket made by weaving bamboo slats, wickers or twigs
U+7C0E.0	fUTF8	簎
U+7C0E.0	fCNS11643	2-5E2D
U+7C0E.0	fBig5	EDC0
U+7C0E.0	fGB	3-6479
U+7C0E.0	fR/S	 118.11
U+7C0E.0	fTotalStrokes	 17
U+7C0E.0	fCangjie	HQTA
U+7C0E.0	fFourCorner	88561
U+7C0E.0	fHakka	  sit8; cok7
U+7C0E.0	fCantonese	caak3; cok3
U+7C0E.0	fMandarin	zha4; ji2; zhuo2
U+7C0E.0	fEnglish	 spear fish
U+7C0F.0	fUTF8	簏
U+7C0F.0	fCNS11643	2-5E28
U+7C0F.0	fBig5	EDBB
U+7C0F.0	fGB	0-737C
U+7C0F.0	fR/S	 118.11
U+7C0F.0	fTotalStrokes	 17
U+7C0F.0	fCangjie	HIXP
U+7C0F.0	fFourCorner	88211
U+7C0F.0	fHakka	  luk8; luk7
U+7C0F.0	fCantonese	luk1; luk6
U+7C0F.0	fMandarin	lu4
U+7C0F.0	fEnglish	 bamboo box; trunk
U+7C10.0	fUTF8	簐
U+7C10.0	fCNS11643	2-5E3A
U+7C10.0	fBig5	EDCD
U+7C10.0	fGB	5-634B
U+7C10.0	fR/S	 118.11
U+7C10.0	fTotalStrokes	 17
U+7C10.0	fCangjie	HJJO
U+7C10.0	fFourCorner	88582
U+7C10.0	fCantonese	nin2; lin2 (also)
U+7C10.0	fMandarin	nian3
U+7C11.0	fUTF8	簑
U+7C11.0	fCNS11643	1-715F
U+7C11.0	fBig5	BF75
U+7C11.0	fGB	E-373C
U+7C11.0	fR/S	 118.10
U+7C11.0	fTotalStrokes	 16
U+7C11.0	fCangjie	HYWV
U+7C11.0	fFourCorner	88732
U+7C11.0	fHakka	 so1
U+7C11.0	fCantonese	so1
U+7C11.0	fMandarin	suo1; cui1; shuai1; sui1; sui2
U+7C11.0	fEnglish	a coir raincoat
U+7C12.0	fUTF8	簒
U+7C12.0	fCNS11643	3-5571
U+7C12.0	fGB	E-373D
U+7C12.0	fR/S	 118.11
U+7C12.0	fTotalStrokes	 17
U+7C12.0	fCangjie	HBUI
U+7C12.0	fFourCorner	88230
U+7C12.0	fHakka	 con5; con3
U+7C12.0	fCantonese	saan3
U+7C12.0	fMandarin	cuan4
U+7C12.0	fEnglish	usurp
U+7C13.0	fUTF8	簓
U+7C13.0	fGB	E-373E
U+7C13.0	fR/S	 118.11
U+7C13.0	fTotalStrokes	 17
U+7C13.0	fCangjie	HBRH
U+7C13.0	fCantonese	diu1
U+7C13.0	fEnglish	a bamboo whisk; the broken end of a bamboo stalk
U+7C14.0	fUTF8	簔
U+7C14.0	fCNS11643	F-5B5F
U+7C14.0	fGB	E-373F
U+7C14.0	fR/S	 118.11
U+7C14.0	fTotalStrokes	 17
U+7C14.0	fCangjie	XHYWV
U+7C14.0	fCantonese	so1
U+7C14.0	fMandarin	suo1
U+7C14.0	fEnglish	a coir raincoat
U+7C15.0	fUTF8	簕
U+7C15.0	fCNS11643	4-5757
U+7C15.0	fGB	3-6476
U+7C15.0	fR/S	 118.10; 118.11
U+7C15.0	fTotalStrokes	 16; 17
U+7C15.0	fCangjie	HTJS
U+7C15.0	fFourCorner	88520
U+7C15.0	fCantonese	lak6
U+7C15.0	fMandarin	le4
U+7C15.0	fEnglish	thorn
U+7C16.0	fUTF8	簖
U+7C16.0	fGB	0-737D
U+7C16.0	fR/S	 118.11
U+7C16.0	fTotalStrokes	 17
U+7C16.0	fCangjie	HVDL
U+7C16.0	fHakka	 ton5
U+7C16.0	fCantonese	dyun6
U+7C16.0	fMandarin	duan4
U+7C16.0	fEnglish	simplified form of &#x7C6A;
U+7C17.0	fUTF8	簗
U+7C17.0	fGB	E-3740
U+7C17.0	fR/S	 118.11
U+7C17.0	fTotalStrokes	 17
U+7C17.0	fCangjie	HECD
U+7C17.0	fCantonese	loeng4
U+7C17.0	fMandarin	liang2
U+7C17.0	fEnglish	a weir, fish trap, fish pond
U+7C18.0	fUTF8	簘
U+7C18.0	fCNS11643	3-556F
U+7C18.0	fGB	E-3741
U+7C18.0	fR/S	 118.12
U+7C18.0	fTotalStrokes	 17; 18
U+7C18.0	fCangjie	HLLL
U+7C18.0	fFourCorner	88220
U+7C18.0	fCantonese	siu1
U+7C18.0	fMandarin	xiao1; xiao3
U+7C18.0	fEnglish	 a musical instrument like pan-pipes; bamboo flute
U+7C19.0	fUTF8	簙
U+7C19.0	fCNS11643	2-624F
U+7C19.0	fBig5	F063
U+7C19.0	fGB	5-642D
U+7C19.0	fR/S	 118.12
U+7C19.0	fTotalStrokes	 18
U+7C19.0	fCangjie	HJII
U+7C19.0	fFourCorner	88442
U+7C19.0	fHakka	  bok7
U+7C19.0	fCantonese	bok3
U+7C19.0	fMandarin	bo2
U+7C19.0	fEnglish	a kind of chess game
U+7C1A.0	fUTF8	簚
U+7C1A.0	fCNS11643	3-5850
U+7C1A.0	fGB	5-6430
U+7C1A.0	fR/S	 118.12
U+7C1A.0	fTotalStrokes	 18
U+7C1A.0	fCangjie	HWLB
U+7C1A.0	fFourCorner	88250
U+7C1A.0	fHakka	 met8
U+7C1A.0	fCantonese	mik6
U+7C1A.0	fMandarin	mi4
U+7C1A.0	fEnglish	screen for the front of a carriage
U+7C1B.0	fUTF8	簛
U+7C1B.0	fCNS11643	3-584E
U+7C1B.0	fGB	3-6523
U+7C1B.0	fR/S	 118.12
U+7C1B.0	fTotalStrokes	 18
U+7C1B.0	fCangjie	HTCL
U+7C1B.0	fFourCorner	88820
U+7C1B.0	fHakka	  si1;  sii1
U+7C1B.0	fCantonese	sai1
U+7C1B.0	fMandarin	si1; shai1
U+7C1B.0	fEnglish	 sieve sift strain
U+7C1C.0	fUTF8	簜
U+7C1C.0	fCNS11643	2-624D
U+7C1C.0	fBig5	F061
U+7C1C.0	fGB	3-6376
U+7C1C.0	fR/S	 118.12
U+7C1C.0	fTotalStrokes	 18
U+7C1C.0	fCangjie	HEAH
U+7C1C.0	fFourCorner	88127
U+7C1C.0	fHakka	  tong3
U+7C1C.0	fCantonese	dong6; dong1
U+7C1C.0	fMandarin	dang4; tang1
U+7C1C.0	fEnglish	bamboo
U+7C1D.0	fUTF8	簝
U+7C1D.0	fCNS11643	2-6253
U+7C1D.0	fBig5	F067
U+7C1D.0	fGB	3-6527
U+7C1D.0	fR/S	 118.12
U+7C1D.0	fTotalStrokes	 18
U+7C1D.0	fCangjie	HKCF
U+7C1D.0	fFourCorner	88906
U+7C1D.0	fHakka	 liau1;  liau2
U+7C1D.0	fCantonese	liu4; lou4
U+7C1D.0	fMandarin	liao2; lao3; lao2
U+7C1D.0	fEnglish	bamboo tray for sacrificial flesh
U+7C1E.0	fUTF8	簞
U+7C1E.0	fCNS11643	1-767A
U+7C1E.0	fBig5	C2B0
U+7C1E.0	fGB	1-736C
U+7C1E.0	fR/S	 118.12
U+7C1E.0	fTotalStrokes	 18
U+7C1E.0	fCangjie	HRRJ
U+7C1E.0	fFourCorner	88506
U+7C1E.0	fHakka	  dan1
U+7C1E.0	fCantonese	daan1
U+7C1E.0	fMandarin	dan1
U+7C1E.0	fEnglish	a small bamboo ware for holding cooked rice
U+7C1F.0	fUTF8	簟
U+7C1F.0	fCNS11643	2-6251
U+7C1F.0	fBig5	F065
U+7C1F.0	fGB	0-7421
U+7C1F.0	fR/S	 118.12
U+7C1F.0	fTotalStrokes	 18
U+7C1F.0	fCangjie	HMWJ
U+7C1F.0	fFourCorner	88406
U+7C1F.0	fHakka	 tiam5; tiam6; diam5; diam6; tiam3
U+7C1F.0	fCantonese	tim5; dim3
U+7C1F.0	fMandarin	dian4
U+7C1F.0	fEnglish	 bamboo mat
U+7C20.0	fUTF8	簠
U+7C20.0	fCNS11643	2-6250
U+7C20.0	fBig5	F064
U+7C20.0	fGB	3-6526
U+7C20.0	fR/S	 118.12
U+7C20.0	fTotalStrokes	 18
U+7C20.0	fCangjie	HIBT
U+7C20.0	fFourCorner	88107
U+7C20.0	fHakka	  pu3
U+7C20.0	fCantonese	fu2
U+7C20.0	fMandarin	fu3; fu1; fu4; pu2
U+7C20.0	fEnglish	a square vessel used for holding the cooked grain at sacrifices or feasts
U+7C21.0	fUTF8	簡
U+7C21.0	fCNS11643	1-767C
U+7C21.0	fBig5	C2B2
U+7C21.0	fGB	1-3C72
U+7C21.0	fR/S	 118.12
U+7C21.0	fTotalStrokes	 18
U+7C21.0	fCangjie	HANA
U+7C21.0	fFourCorner	88227
U+7C21.0	fHakka	  gan3;  gien3; gian3; kian1
U+7C21.0	fCantonese	gaan2
U+7C21.0	fMandarin	jian3
U+7C21.0	fEnglish	[1] simple; terse; succinct; brief [2] a slip or tablet of bamboo for writing [3] letter; note [4] treat coolly or impolitely [5] designate; appoint (an official) [6] select; choose [7] a Chinese family name
U+7C22.0	fUTF8	簢
U+7C22.0	fCNS11643	2-6256
U+7C22.0	fBig5	F06A
U+7C22.0	fGB	5-6345
U+7C22.0	fR/S	 118.12
U+7C22.0	fTotalStrokes	 18
U+7C22.0	fCangjie	HANK
U+7C22.0	fFourCorner	88227
U+7C22.0	fCantonese	man5
U+7C22.0	fMandarin	min3; ao2
U+7C23.0	fUTF8	簣
U+7C23.0	fCNS11643	1-767B
U+7C23.0	fBig5	C2B1
U+7C23.0	fGB	1-7371
U+7C23.0	fR/S	 118.12
U+7C23.0	fTotalStrokes	 18
U+7C23.0	fCangjie	HLMC
U+7C23.0	fFourCorner	88806
U+7C23.0	fHakka	 kui3; kui6; kui5; k'wui5; kwi5
U+7C23.0	fCantonese	gwai6
U+7C23.0	fMandarin	kui4; kuai4
U+7C23.0	fEnglish	a bamboo basket for carrying earth
U+7C24.0	fUTF8	簤
U+7C24.0	fCNS11643	3-5851
U+7C24.0	fGB	5-6338
U+7C24.0	fR/S	  118.12
U+7C24.0	fTotalStrokes	 18
U+7C24.0	fCangjie	HWLC
U+7C24.0	fFourCorner	88800
U+7C24.0	fCantonese	daai6
U+7C24.0	fMandarin	dai4
U+7C25.0	fUTF8	簥
U+7C25.0	fCNS11643	2-6257
U+7C25.0	fBig5	F06B
U+7C25.0	fGB	3-636E
U+7C25.0	fR/S	 118.12
U+7C25.0	fTotalStrokes	 18
U+7C25.0	fCangjie	HHKB
U+7C25.0	fFourCorner	88227
U+7C25.0	fCantonese	giu1
U+7C25.0	fMandarin	qiao2; jiao1
U+7C25.0	fEnglish	a large pipe; a farm tool
U+7C26.0	fUTF8	簦
U+7C26.0	fCNS11643	2-6254
U+7C26.0	fBig5	F068
U+7C26.0	fGB	0-7423
U+7C26.0	fR/S	 118.12
U+7C26.0	fTotalStrokes	 18
U+7C26.0	fCangjie	HNOT
U+7C26.0	fFourCorner	88108
U+7C26.0	fHakka	  den1
U+7C26.0	fCantonese	dang1
U+7C26.0	fMandarin	deng1
U+7C26.0	fEnglish	a large umbrella with a long handle, used for street stalls
U+7C27.0	fUTF8	簧
U+7C27.0	fCNS11643	1-7678
U+7C27.0	fBig5	C2AE
U+7C27.0	fGB	0-3B49
U+7C27.0	fR/S	 118.12
U+7C27.0	fTotalStrokes	 18
U+7C27.0	fCangjie	HTLC
U+7C27.0	fFourCorner	88806
U+7C27.0	fHakka	  wong2; wang2; vong2
U+7C27.0	fCantonese	wong4; wong2 (also)
U+7C27.0	fMandarin	huang2
U+7C27.0	fEnglish	[1] reed; the metal tongue in a reed organ [2] reed organ [3] spring or catch in a machine
U+7C28.0	fUTF8	簨
U+7C28.0	fCNS11643	2-6255
U+7C28.0	fBig5	F069
U+7C28.0	fGB	5-6439
U+7C28.0	fR/S	 118.12
U+7C28.0	fTotalStrokes	 18
U+7C28.0	fCangjie	HRUC
U+7C28.0	fFourCorner	88801
U+7C28.0	fHakka	  sun3
U+7C28.0	fCantonese	seon2; seon3 (also)
U+7C28.0	fMandarin	sun3; zhuan4
U+7C28.0	fEnglish	a beam for hanging bells or chime stone
U+7C29.0	fUTF8	簩
U+7C29.0	fCNS11643	2-624E
U+7C29.0	fBig5	F062
U+7C29.0	fGB	3-637C
U+7C29.0	fR/S	 118.12
U+7C29.0	fTotalStrokes	 18
U+7C29.0	fCangjie	HFFS
U+7C29.0	fFourCorner	88427
U+7C29.0	fHakka	 lau2
U+7C29.0	fCantonese	lou4; lou6
U+7C29.0	fMandarin	lao2; lao4
U+7C2A.0	fUTF8	簪
U+7C2A.0	fCNS11643	1-7679
U+7C2A.0	fBig5	C2AF
U+7C2A.0	fGB	0-7422
U+7C2A.0	fR/S	 118.12
U+7C2A.0	fTotalStrokes	 18
U+7C2A.0	fCangjie	HMUA
U+7C2A.0	fFourCorner	88601
U+7C2A.0	fHakka	  zam1; sun3
U+7C2A.0	fCantonese	zaam1
U+7C2A.0	fMandarin	zan1; zan3; cuan2; zen1
U+7C2A.0	fEnglish	[1] hairpin for women [2] clasp for clipping the cap and hair together [3] wear in the hair; stick in the hair
U+7C2B.0	fUTF8	簫
U+7C2B.0	fCNS11643	1-7677
U+7C2B.0	fBig5	C2AD
U+7C2B.0	fGB	1-736F
U+7C2B.0	fR/S	 118.12
U+7C2B.0	fTotalStrokes	 18
U+7C2B.0	fCangjie	HLX
U+7C2B.0	fFourCorner	88227
U+7C2B.0	fHakka	  siau1; seu1
U+7C2B.0	fCantonese	siu1
U+7C2B.0	fMandarin	xiao1; xiao3
U+7C2B.0	fEnglish	a vertical bamboo flute
U+7C2C.0	fUTF8	簬
U+7C2C.0	fCNS11643	2-6635
U+7C2C.0	fBig5	F2AB
U+7C2C.0	fGB	5-6440
U+7C2C.0	fR/S	 118.12
U+7C2C.0	fTotalStrokes	 18
U+7C2C.0	fCangjie	HRMR
U+7C2C.0	fFourCorner	88164
U+7C2C.0	fCantonese	lou6
U+7C2C.0	fMandarin	lu4; du4
U+7C2D.0	fUTF8	簭
U+7C2D.0	fCNS11643	2-6252
U+7C2D.0	fBig5	F066
U+7C2D.0	fGB	E-3742
U+7C2D.0	fR/S	 118.12
U+7C2D.0	fTotalStrokes	 18
U+7C2D.0	fCangjie	HGCR
U+7C2D.0	fFourCorner	88608
U+7C2D.0	fCantonese	sai6
U+7C2D.0	fMandarin	shi4
U+7C2E.0	fUTF8	簮
U+7C2E.0	fCNS11643	3-584C
U+7C2E.0	fGB	E-3743
U+7C2E.0	fR/S	  118.12
U+7C2E.0	fTotalStrokes	 18
U+7C2E.0	fCangjie	HQOA
U+7C2E.0	fFourCorner	88600
U+7C2E.0	fCantonese	zaam1
U+7C2E.0	fMandarin	zan1; zan3
U+7C2F.0	fUTF8	簯
U+7C2F.0	fCNS11643	3-6622
U+7C2F.0	fGB	E-3744
U+7C2F.0	fR/S	 118.12
U+7C2F.0	fTotalStrokes	 18
U+7C2F.0	fCangjie	HDTC
U+7C2F.0	fFourCorner	8898
U+7C2F.0	fCantonese	kei4
U+7C30.0	fUTF8	簰
U+7C30.0	fCNS11643	2-6258
U+7C30.0	fBig5	F06C
U+7C30.0	fGB	5-6432
U+7C30.0	fR/S	 118.12
U+7C30.0	fTotalStrokes	 18
U+7C30.0	fCangjie	HLLJ
U+7C30.0	fFourCorner	88246
U+7C30.0	fHakka	 pai2
U+7C30.0	fCantonese	paai4; bei6
U+7C30.0	fMandarin	pai2; qiao2; bei1
U+7C30.0	fEnglish	bamboo raft
U+7C31.0	fUTF8	簱
U+7C31.0	fGB	E-3745
U+7C31.0	fR/S	 118.12
U+7C31.0	fTotalStrokes	 18
U+7C31.0	fCangjie	XHYSC
U+7C31.0	fCantonese	kei4
U+7C31.0	fMandarin	qi2
U+7C32.0	fUTF8	簲
U+7C32.0	fCNS11643	3-6623
U+7C32.0	fGB	E-3746
U+7C32.0	fR/S	 118.12
U+7C32.0	fTotalStrokes	 18
U+7C32.0	fCangjie	HBHJ
U+7C32.0	fFourCorner	8824
U+7C32.0	fCantonese	paai4
U+7C32.0	fMandarin	pai2
U+7C33.0	fUTF8	簳
U+7C33.0	fCNS11643	2-6632
U+7C33.0	fBig5	F2A8
U+7C33.0	fGB	3-652F
U+7C33.0	fR/S	 118.13
U+7C33.0	fTotalStrokes	 19
U+7C33.0	fCangjie	HJJJ
U+7C33.0	fFourCorner	88441
U+7C33.0	fHakka	 gon5
U+7C33.0	fCantonese	gon3; gon2 (also)
U+7C33.0	fMandarin	gan4; gan3
U+7C33.0	fEnglish	young bamboo; a slender variety of bamboo suited for arrows
U+7C34.0	fUTF8	簴
U+7C34.0	fCNS11643	3-5A6F
U+7C34.0	fGB	5-643F
U+7C34.0	fR/S	 118.13
U+7C34.0	fTotalStrokes	 19
U+7C34.0	fCangjie	HYPC
U+7C34.0	fFourCorner	88280
U+7C34.0	fHakka	 ki3;  gi3
U+7C34.0	fCantonese	geoi6
U+7C34.0	fMandarin	ju4
U+7C34.0	fEnglish	[1] pillars beside the crossbeam for hanging bells or drums [2] sea anemone
U+7C35.0	fUTF8	簵
U+7C35.0	fCNS11643	3-5A6E
U+7C35.0	fGB	3-6465
U+7C35.0	fR/S	 118.13
U+7C35.0	fTotalStrokes	 19
U+7C35.0	fCangjie	HJJR
U+7C35.0	fFourCorner	88560
U+7C35.0	fHakka	 lu5; lu6
U+7C35.0	fCantonese	lou6
U+7C35.0	fMandarin	du4; lu4
U+7C35.0	fEnglish	fine bamboo
U+7C36.0	fUTF8	簶
U+7C36.0	fCNS11643	4-604E
U+7C36.0	fGB	3-652D
U+7C36.0	fR/S	  118.13
U+7C36.0	fTotalStrokes	 19
U+7C36.0	fCangjie	HIFE
U+7C36.0	fFourCorner	88230
U+7C36.0	fCantonese	luk6
U+7C36.0	fMandarin	lu4
U+7C37.0	fUTF8	簷
U+7C37.0	fCNS11643	1-785D
U+7C37.0	fBig5	C3B2
U+7C37.0	fGB	E-3747
U+7C37.0	fR/S	 118.13
U+7C37.0	fTotalStrokes	 19
U+7C37.0	fCangjie	HNCR
U+7C37.0	fFourCorner	88261
U+7C37.0	fHakka	  jam2; ram2; jam3; dam5; dam3
U+7C37.0	fCantonese	jim4; sim4 (also); jam4 (also)
U+7C37.0	fMandarin	yan2; dan4
U+7C37.0	fEnglish	[1] eaves of a house [2] brim; edge of anything sloping downward
U+7C38.0	fUTF8	簸
U+7C38.0	fCNS11643	1-785B
U+7C38.0	fBig5	C3B0
U+7C38.0	fGB	0-7424
U+7C38.0	fR/S	 118.13
U+7C38.0	fTotalStrokes	 19
U+7C38.0	fCangjie	HTCE
U+7C38.0	fFourCorner	88847
U+7C38.0	fHakka	 boi5; bo5;  bai5; poi5
U+7C38.0	fCantonese	bo3; bo2 (also); bo1 (also)
U+7C38.0	fMandarin	bo3; bo4
U+7C38.0	fEnglish	 winnow; sift (especially rice)
U+7C39.0	fUTF8	簹
U+7C39.0	fCNS11643	2-6634
U+7C39.0	fBig5	F2AA
U+7C39.0	fGB	3-6365
U+7C39.0	fR/S	 118.13
U+7C39.0	fTotalStrokes	 19
U+7C39.0	fCangjie	HFBW
U+7C39.0	fFourCorner	88606
U+7C39.0	fHakka	  dong1
U+7C39.0	fCantonese	dong1
U+7C39.0	fMandarin	dang1; dang4
U+7C39.0	fEnglish	tall bamboo
U+7C3A.0	fUTF8	簺
U+7C3A.0	fCNS11643	4-604C
U+7C3A.0	fGB	3-6533
U+7C3A.0	fR/S	  118.13
U+7C3A.0	fTotalStrokes	 19
U+7C3A.0	fCangjie	HJTG
U+7C3A.0	fFourCorner	88100
U+7C3A.0	fHakka	  soi5; set7
U+7C3A.0	fCantonese	coi3
U+7C3A.0	fMandarin	sai4
U+7C3A.0	fEnglish	fishing-stakes
U+7C3B.0	fUTF8	簻
U+7C3B.0	fCNS11643	2-6636
U+7C3B.0	fBig5	F2AC
U+7C3B.0	fGB	5-6331
U+7C3B.0	fR/S	 118.13
U+7C3B.0	fTotalStrokes	 19
U+7C3B.0	fCangjie	HYBB
U+7C3B.0	fFourCorner	88302
U+7C3B.0	fHakka	 ko1
U+7C3B.0	fCantonese	zaa1; gwo1 (also); go1 (also); gwo6
U+7C3B.0	fMandarin	ke1; zhua1
U+7C3B.0	fEnglish	empty
U+7C3C.0	fUTF8	簼
U+7C3C.0	fCNS11643	2-6633
U+7C3C.0	fBig5	F2A9
U+7C3C.0	fGB	5-643C
U+7C3C.0	fR/S	 118.13
U+7C3C.0	fTotalStrokes	 19
U+7C3C.0	fCangjie	HQTB
U+7C3C.0	fFourCorner	88586
U+7C3C.0	fCantonese	gau1; kau1
U+7C3C.0	fMandarin	long2; gou4; gou1
U+7C3D.0	fUTF8	簽
U+7C3D.0	fCNS11643	1-785C
U+7C3D.0	fBig5	C3B1
U+7C3D.0	fGB	1-4729
U+7C3D.0	fR/S	 118.13
U+7C3D.0	fTotalStrokes	 19
U+7C3D.0	fCangjie	HOMO
U+7C3D.0	fFourCorner	88886
U+7C3D.0	fHakka	  ciam1; ziam1; ciam5
U+7C3D.0	fCantonese	cim1
U+7C3D.0	fMandarin	qian1
U+7C3D.0	fEnglish	[1] sign one's name; put down one's signature; subscribe; endorse [2] bamboo slips used for drawings lots or divinations [3] label
U+7C3E.0	fUTF8	簾
U+7C3E.0	fCNS11643	1-7859
U+7C3E.0	fBig5	C3AE
U+7C3E.0	fGB	1-4131
U+7C3E.0	fR/S	 118.13
U+7C3E.0	fTotalStrokes	 19
U+7C3E.0	fCangjie	HITC
U+7C3E.0	fFourCorner	88237
U+7C3E.0	fHakka	  liam2; liam3
U+7C3E.0	fCantonese	lim4; lim2 (also)
U+7C3E.0	fMandarin	lian2
U+7C3E.0	fEnglish	[1] blind; curtain; loose hanging screen for a door or window, usually made of stringed beads, bamboo slabs, etc. [2] a flag as a shop sign
U+7C3F.0	fUTF8	簿
U+7C3F.0	fCNS11643	1-785A
U+7C3F.0	fBig5	C3AF
U+7C3F.0	fGB	0-323E
U+7C3F.0	fR/S	 118.13
U+7C3F.0	fTotalStrokes	 19
U+7C3F.0	fCangjie	HEII
U+7C3F.0	fFourCorner	88146
U+7C3F.0	fHakka	  pu1;  pu5;  pu3
U+7C3F.0	fCantonese	bou6; bou2 (also); bok6
U+7C3F.0	fMandarin	bu4; bo2; bao2; bi2
U+7C3F.0	fEnglish	[1] register; record [2] books; account book; notebook
U+7C40.0	fUTF8	籀
U+7C40.0	fCNS11643	1-785E
U+7C40.0	fBig5	C3B3
U+7C40.0	fGB	0-7426
U+7C40.0	fR/S	 118.13
U+7C40.0	fTotalStrokes	 19
U+7C40.0	fCangjie	HQHW
U+7C40.0	fFourCorner	88562
U+7C40.0	fHakka	 liu5; ciu3; chiu1; cu1
U+7C40.0	fCantonese	zau6
U+7C40.0	fMandarin	zhou4
U+7C40.0	fEnglish	[1] deduce [2] a style of calligraphy also known as large seal
U+7C41.0	fUTF8	籁
U+7C41.0	fGB	0-7425
U+7C41.0	fR/S	 118.13
U+7C41.0	fTotalStrokes	 19
U+7C41.0	fCangjie	HDLO
U+7C41.0	fHakka	 lai5
U+7C41.0	fCantonese	laai6
U+7C41.0	fMandarin	lai4
U+7C41.0	fEnglish	simplified form of &#x7C5F;
U+7C42.0	fUTF8	籂
U+7C42.0	fCNS11643	3-6624
U+7C42.0	fGB	E-3748
U+7C42.0	fR/S	 118.13
U+7C42.0	fTotalStrokes	 19
U+7C42.0	fCangjie	HOIB
U+7C42.0	fFourCorner	8872
U+7C42.0	fCantonese	sik1
U+7C43.0	fUTF8	籃
U+7C43.0	fCNS11643	1-7A25
U+7C43.0	fBig5	C478
U+7C43.0	fGB	1-403A
U+7C43.0	fR/S	 118.14
U+7C43.0	fTotalStrokes	 20
U+7C43.0	fCangjie	HSIT
U+7C43.0	fFourCorner	88107
U+7C43.0	fHakka	  lam2; lam3; lam1
U+7C43.0	fCantonese	laam4; laam2 (also)
U+7C43.0	fMandarin	lan2
U+7C43.0	fEnglish	 basket
U+7C44.0	fUTF8	籄
U+7C44.0	fCNS11643	3-5C66
U+7C44.0	fGB	5-6421
U+7C44.0	fR/S	 118.14
U+7C44.0	fTotalStrokes	 20
U+7C44.0	fCangjie	HSLC
U+7C44.0	fFourCorner	88710
U+7C44.0	fHakka	 kui3; kui6; kui5; k'wui5
U+7C44.0	fCantonese	gwai6
U+7C44.0	fMandarin	kui4
U+7C44.0	fEnglish	a basket for carrying earth
U+7C45.0	fUTF8	籅
U+7C45.0	fCNS11643	2-6957
U+7C45.0	fBig5	F4AA
U+7C45.0	fGB	5-6270
U+7C45.0	fR/S	 118.14
U+7C45.0	fTotalStrokes	 20
U+7C45.0	fCangjie	HHXC
U+7C45.0	fFourCorner	88801
U+7C45.0	fCantonese	jyu4
U+7C45.0	fMandarin	yu2
U+7C46.0	fUTF8	籆
U+7C46.0	fCNS11643	5-6C79
U+7C46.0	fGB	5-643B
U+7C46.0	fR/S	  118.14
U+7C46.0	fTotalStrokes	 20
U+7C46.0	fCangjie	HTOE
U+7C46.0	fFourCorner	88400
U+7C46.0	fCantonese	wok6; jyut6 (also)
U+7C46.0	fMandarin	yue4
U+7C47.0	fUTF8	籇
U+7C47.0	fCNS11643	2-6956
U+7C47.0	fBig5	F4A9
U+7C47.0	fGB	5-6448
U+7C47.0	fR/S	 118.14
U+7C47.0	fTotalStrokes	 20
U+7C47.0	fCangjie	HYRO
U+7C47.0	fFourCorner	88232
U+7C47.0	fHakka	 hau2
U+7C47.0	fCantonese	gou1; hou4
U+7C47.0	fMandarin	hao2
U+7C48.0	fUTF8	籈
U+7C48.0	fCNS11643	2-6954
U+7C48.0	fBig5	F4A7
U+7C48.0	fGB	3-6530
U+7C48.0	fR/S	 118.14
U+7C48.0	fTotalStrokes	 20
U+7C48.0	fCangjie	HMGN
U+7C48.0	fFourCorner	88117
U+7C48.0	fCantonese	zan1
U+7C48.0	fMandarin	zhen1
U+7C49.0	fUTF8	籉
U+7C49.0	fCNS11643	2-6953
U+7C49.0	fBig5	F4A6
U+7C49.0	fGB	E-3749
U+7C49.0	fR/S	 118.14
U+7C49.0	fTotalStrokes	 20
U+7C49.0	fCangjie	HGRG
U+7C49.0	fFourCorner	88104
U+7C49.0	fCantonese	toi4
U+7C49.0	fMandarin	tai2
U+7C4A.0	fUTF8	籊
U+7C4A.0	fCNS11643	2-6955
U+7C4A.0	fBig5	F4A8
U+7C4A.0	fGB	3-6536
U+7C4A.0	fR/S	 118.14
U+7C4A.0	fTotalStrokes	 20
U+7C4A.0	fCangjie	HSMG
U+7C4A.0	fFourCorner	88214
U+7C4A.0	fHakka	 tit7; tit8
U+7C4A.0	fCantonese	dik6 (also); tik1
U+7C4A.0	fMandarin	ti4; di2
U+7C4A.0	fEnglish	long
U+7C4B.0	fUTF8	籋
U+7C4B.0	fCNS11643	3-5C65
U+7C4B.0	fGB	3-635E
U+7C4B.0	fR/S	  118.14
U+7C4B.0	fTotalStrokes	 20
U+7C4B.0	fCangjie	HMFB
U+7C4B.0	fFourCorner	88220
U+7C4B.0	fHakka	  ngiap7
U+7C4B.0	fCantonese	nip6; lip6 (also)
U+7C4B.0	fMandarin	mi2
U+7C4B.0	fEnglish	basket
U+7C4C.0	fUTF8	籌
U+7C4C.0	fCNS11643	1-7A24
U+7C4C.0	fBig5	C477
U+7C4C.0	fGB	1-336F
U+7C4C.0	fR/S	 118.14
U+7C4C.0	fTotalStrokes	 20
U+7C4C.0	fCangjie	HGNI
U+7C4C.0	fFourCorner	88641
U+7C4C.0	fHakka	 cu2;  chiu2; ciu2; chu2
U+7C4C.0	fCantonese	cau4; cau2 (also)
U+7C4C.0	fMandarin	chou2; tao2
U+7C4C.0	fEnglish	[1] chip, tally for calculating purposes [2] plan; prepare [3] raise money [4] assess; estimate
U+7C4D.0	fUTF8	籍
U+7C4D.0	fCNS11643	1-7A26
U+7C4D.0	fBig5	C479
U+7C4D.0	fGB	0-3C2E
U+7C4D.0	fR/S	 118.14
U+7C4D.0	fTotalStrokes	 20
U+7C4D.0	fCangjie	HQDA
U+7C4D.0	fFourCorner	88961
U+7C4D.0	fHakka	  cit8; sit8; sit7
U+7C4D.0	fCantonese	zik6
U+7C4D.0	fMandarin	ji2; jie4; xie4
U+7C4D.0	fEnglish	[1] books; volumes; reading materials [2] record; register; list; census [3] one's hometown or native place
U+7C4E.0	fUTF8	籎
U+7C4E.0	fCNS11643	3-6625
U+7C4E.0	fGB	E-374A
U+7C4E.0	fR/S	 118.14
U+7C4E.0	fTotalStrokes	 20
U+7C4E.0	fCangjie	HPKO
U+7C4E.0	fFourCorner	8848
U+7C4E.0	fCantonese	ji4
U+7C4F.0	fUTF8	籏
U+7C4F.0	fCNS11643	F-6567
U+7C4F.0	fGB	E-374B
U+7C4F.0	fR/S	 118.14
U+7C4F.0	fTotalStrokes	 20
U+7C4F.0	fCangjie	HYSC
U+7C4F.0	fFourCorner	8828
U+7C4F.0	fCantonese	kei4
U+7C4F.0	fMandarin	qi2
U+7C4F.0	fEnglish	 flag
U+7C50.0	fUTF8	籐
U+7C50.0	fCNS11643	1-7A7B
U+7C50.0	fBig5	C4F0
U+7C50.0	fGB	8-2D7B
U+7C50.0	fR/S	 118.15
U+7C50.0	fTotalStrokes	 21
U+7C50.0	fCangjie	HBFE
U+7C50.0	fFourCorner	88232
U+7C50.0	fHakka	 ten2; ten3
U+7C50.0	fCantonese	tang4
U+7C50.0	fMandarin	teng2
U+7C50.0	fEnglish	climbing plants; vines; cane
U+7C51.0	fUTF8	籑
U+7C51.0	fCNS11643	3-5E4C
U+7C51.0	fGB	E-374C
U+7C51.0	fR/S	 118.15
U+7C51.0	fTotalStrokes	 21
U+7C51.0	fCangjie	HBUV
U+7C51.0	fFourCorner	88730
U+7C51.0	fHakka	 con5
U+7C51.0	fCantonese	zaan6
U+7C51.0	fMandarin	zhuan4; xuan3
U+7C51.0	fEnglish	 feed; to provide for delicacies; dainties
U+7C52.0	fUTF8	籒
U+7C52.0	fCNS11643	4-666F
U+7C52.0	fGB	E-374D
U+7C52.0	fR/S	 118.15
U+7C52.0	fTotalStrokes	 21
U+7C52.0	fCangjie	HQMW
U+7C52.0	fFourCorner	88560
U+7C52.0	fHakka	 liu5; chiu1; cu1; chu5; zhiu5; lui3; chiu5
U+7C52.0	fCantonese	zau6
U+7C52.0	fMandarin	zhou4
U+7C53.0	fUTF8	籓
U+7C53.0	fCNS11643	2-6B73
U+7C53.0	fBig5	F5E5
U+7C53.0	fGB	3-6538
U+7C53.0	fR/S	 118.15
U+7C53.0	fTotalStrokes	 21
U+7C53.0	fCangjie	HEHW
U+7C53.0	fFourCorner	88169
U+7C53.0	fHakka	 fan1; pan1
U+7C53.0	fCantonese	faan1; bun1
U+7C53.0	fMandarin	fan1; fan2
U+7C53.0	fEnglish	sieve
U+7C54.0	fUTF8	籔
U+7C54.0	fCNS11643	2-6B72
U+7C54.0	fBig5	F5E4
U+7C54.0	fGB	3-6532
U+7C54.0	fR/S	 118.15
U+7C54.0	fTotalStrokes	 21
U+7C54.0	fCangjie	HLVK
U+7C54.0	fFourCorner	88448
U+7C54.0	fHakka	 su5; sii5
U+7C54.0	fCantonese	sou2; sau2
U+7C54.0	fMandarin	sou3; shu4; qu4
U+7C54.0	fEnglish	bamboo basket
U+7C55.0	fUTF8	籕
U+7C55.0	fCNS11643	3-5C67
U+7C55.0	fGB	E-374E
U+7C55.0	fR/S	 118.14
U+7C55.0	fTotalStrokes	 20
U+7C55.0	fCangjie	HDHW
U+7C55.0	fFourCorner	88960
U+7C55.0	fCantonese	zau6
U+7C55.0	fMandarin	zhou4
U+7C56.0	fUTF8	籖
U+7C56.0	fCNS11643	3-5E4B
U+7C56.0	fGB	E-374F
U+7C56.0	fR/S	 118.14
U+7C56.0	fTotalStrokes	 21; 20
U+7C56.0	fCangjie	HJIM
U+7C56.0	fFourCorner	88150
U+7C56.0	fCantonese	cim1
U+7C56.0	fMandarin	qian1
U+7C56.0	fEnglish	 tally lot marker
U+7C57.0	fUTF8	籗
U+7C57.0	fCNS11643	2-6D68
U+7C57.0	fBig5	F6FA
U+7C57.0	fGB	5-6450
U+7C57.0	fR/S	 118.16
U+7C57.0	fTotalStrokes	 22
U+7C57.0	fCangjie	HMBG
U+7C57.0	fFourCorner	88214
U+7C57.0	fCantonese	zoek3; doek3; kwok3; zok6
U+7C57.0	fMandarin	zhuo2; kuo4
U+7C58.0	fUTF8	籘
U+7C58.0	fCNS11643	3-5F58
U+7C58.0	fGB	E-3750
U+7C58.0	fR/S	  118.16
U+7C58.0	fTotalStrokes	 22
U+7C58.0	fCangjie	HBFF
U+7C58.0	fFourCorner	88290
U+7C58.0	fCantonese	tang4
U+7C58.0	fMandarin	teng2
U+7C58.0	fEnglish	climbing plants; vines; cane
U+7C59.0	fUTF8	籙
U+7C59.0	fCNS11643	2-6D6A
U+7C59.0	fBig5	F6FC
U+7C59.0	fGB	3-643E
U+7C59.0	fR/S	 118.16
U+7C59.0	fTotalStrokes	 22
U+7C59.0	fCangjie	HCVE
U+7C59.0	fFourCorner	88132
U+7C59.0	fHakka	  liuk8
U+7C59.0	fCantonese	luk6
U+7C59.0	fMandarin	lu4
U+7C59.0	fEnglish	[1] list of registers [2] a Taoist amulet or charm
U+7C5A.0	fUTF8	籚
U+7C5A.0	fCNS11643	2-6D6C
U+7C5A.0	fBig5	F6FE
U+7C5A.0	fGB	3-6358
U+7C5A.0	fR/S	 118.16
U+7C5A.0	fTotalStrokes	 22
U+7C5A.0	fCangjie	HYPT
U+7C5A.0	fFourCorner	88217
U+7C5A.0	fHakka	 lu2
U+7C5A.0	fCantonese	lou4
U+7C5A.0	fMandarin	lu2
U+7C5A.0	fEnglish	 bamboo handle of the spear
U+7C5B.0	fUTF8	籛
U+7C5B.0	fCNS11643	2-6D6B
U+7C5B.0	fBig5	F6FD
U+7C5B.0	fGB	3-6467
U+7C5B.0	fR/S	 118.16
U+7C5B.0	fTotalStrokes	 22
U+7C5B.0	fCangjie	HCII
U+7C5B.0	fFourCorner	88153
U+7C5B.0	fHakka	  cien2; tsian3
U+7C5B.0	fCantonese	zin1
U+7C5B.0	fMandarin	jian1; zan1; jian3; jian4
U+7C5B.0	fEnglish	a Chinese family name
U+7C5C.0	fUTF8	籜
U+7C5C.0	fCNS11643	2-6D69
U+7C5C.0	fBig5	F6FB
U+7C5C.0	fGB	1-736A
U+7C5C.0	fR/S	 118.16
U+7C5C.0	fTotalStrokes	 22
U+7C5C.0	fCangjie	HQWJ
U+7C5C.0	fFourCorner	88541
U+7C5C.0	fHakka	  tok8; hok7
U+7C5C.0	fCantonese	tok3
U+7C5C.0	fMandarin	tuo4
U+7C5C.0	fEnglish	 bamboo sheath; the shell of bamboo shoots
U+7C5D.0	fUTF8	籝
U+7C5D.0	fCNS11643	3-5F57
U+7C5D.0	fGB	5-6452
U+7C5D.0	fR/S	 118.16
U+7C5D.0	fTotalStrokes	 22
U+7C5D.0	fCangjie	HYNV
U+7C5D.0	fFourCorner	88210
U+7C5D.0	fHakka	 jin2; rin2; jin3
U+7C5D.0	fCantonese	jing4
U+7C5D.0	fMandarin	ying2
U+7C5D.0	fEnglish	a tube to hold chopsticks
U+7C5E.0	fUTF8	籞
U+7C5E.0	fCNS11643	3-603F
U+7C5E.0	fGB	3-653D
U+7C5E.0	fR/S	  118.16
U+7C5E.0	fTotalStrokes	 23; 22
U+7C5E.0	fCangjie	HHLF
U+7C5E.0	fFourCorner	88900
U+7C5E.0	fHakka	 ngi5
U+7C5E.0	fCantonese	jyu6
U+7C5E.0	fMandarin	yu4
U+7C5E.0	fEnglish	[1] the forbidden garden in a palace [2] keep the crowds of the Emperor's path [3] a bamboo enclosure in a pond for keeping fish
U+7C5F.0	fUTF8	籟
U+7C5F.0	fCNS11643	1-7B6D
U+7C5F.0	fBig5	C5A3
U+7C5F.0	fGB	1-7425
U+7C5F.0	fR/S	 118.16
U+7C5F.0	fTotalStrokes	 22
U+7C5F.0	fCangjie	HDLC
U+7C5F.0	fFourCorner	88986
U+7C5F.0	fHakka	 lai5; lai6; lai3
U+7C5F.0	fCantonese	laai6
U+7C5F.0	fMandarin	lai4
U+7C5F.0	fEnglish	[1] a kind of bamboo flute; a kind of pipe [2] unspecified sounds
U+7C60.0	fUTF8	籠
U+7C60.0	fCNS11643	1-7B6C
U+7C60.0	fBig5	C5A2
U+7C60.0	fGB	1-417D
U+7C60.0	fR/S	 118.16
U+7C60.0	fTotalStrokes	 22
U+7C60.0	fCangjie	HYBP
U+7C60.0	fFourCorner	88211
U+7C60.0	fHakka	  lung1;  lung3;  lung2
U+7C60.0	fCantonese	lung4; lung5; lung2 (also)
U+7C60.0	fMandarin	long2
U+7C60.0	fEnglish	[1] cage; coop [2] cage-like basket; tight, stored cagelike bamboo ware for steaming dumplings, etc.; bamboo food steamer
U+7C60.1	fUTF8	籠
U+7C60.1	fCNS11643	1-7B6C
U+7C60.1	fBig5	C5A2
U+7C60.1	fGB	1-417D
U+7C60.1	fR/S	 118.16
U+7C60.1	fTotalStrokes	 22
U+7C60.1	fCangjie	HYBP
U+7C60.1	fFourCorner	88211
U+7C60.1	fMandarin	long5; long3
U+7C60.2	fUTF8	籠
U+7C60.2	fCNS11643	1-7B6C
U+7C60.2	fBig5	C5A2
U+7C60.2	fGB	1-417D
U+7C60.2	fR/S	 118.16
U+7C60.2	fTotalStrokes	 22
U+7C60.2	fCangjie	HYBP
U+7C60.2	fFourCorner	88211
U+7C61.0	fUTF8	籡
U+7C61.0	fGB	E-3751
U+7C61.0	fR/S	 118.16
U+7C61.0	fTotalStrokes	 22
U+7C61.0	fCangjie	HQOO
U+7C61.0	fCantonese	gim2
U+7C61.0	fMandarin	qie4
U+7C62.0	fUTF8	籢
U+7C62.0	fCNS11643	3-603E
U+7C62.0	fGB	E-3752
U+7C62.0	fR/S	  118.17
U+7C62.0	fTotalStrokes	 23
U+7C62.0	fCangjie	HOOK
U+7C62.0	fFourCorner	88880
U+7C62.0	fCantonese	lim4
U+7C62.0	fMandarin	lian2
U+7C63.0	fUTF8	籣
U+7C63.0	fCNS11643	1-7C3F
U+7C63.0	fBig5	C5D3
U+7C63.0	fGB	3-652B
U+7C63.0	fR/S	 118.17
U+7C63.0	fTotalStrokes	 23
U+7C63.0	fCangjie	HANW
U+7C63.0	fFourCorner	88227
U+7C63.0	fHakka	 lan2; lan3
U+7C63.0	fCantonese	laan4
U+7C63.0	fMandarin	lan2
U+7C63.0	fEnglish	bow case
U+7C64.0	fUTF8	籤
U+7C64.0	fCNS11643	1-7C3E
U+7C64.0	fBig5	C5D2
U+7C64.0	fGB	1-7862
U+7C64.0	fR/S	 118.17
U+7C64.0	fTotalStrokes	 23
U+7C64.0	fCangjie	HOIM
U+7C64.0	fFourCorner	88153
U+7C64.0	fHakka	  ciam1
U+7C64.0	fCantonese	cim1
U+7C64.0	fMandarin	qian1
U+7C64.0	fEnglish	[1] slip of bamboo engraved with signs to be used in gambling or divinations; lot [2] label [3] a small sharp-pointed stick
U+7C65.0	fUTF8	籥
U+7C65.0	fCNS11643	1-7C40
U+7C65.0	fBig5	C5D4
U+7C65.0	fGB	3-653E
U+7C65.0	fR/S	 118.17
U+7C65.0	fTotalStrokes	 23
U+7C65.0	fCangjie	HOMB
U+7C65.0	fFourCorner	88227
U+7C65.0	fHakka	  jok7; rok7; jok8
U+7C65.0	fCantonese	joek6
U+7C65.0	fMandarin	yue4
U+7C65.0	fEnglish	[1] key [2] a short flute or pipe
U+7C66.0	fUTF8	籦
U+7C66.0	fCNS11643	2-6F3C
U+7C66.0	fBig5	F7ED
U+7C66.0	fGB	3-6534
U+7C66.0	fR/S	 118.17
U+7C66.0	fTotalStrokes	 23
U+7C66.0	fCangjie	HCHG
U+7C66.0	fFourCorner	88114
U+7C66.0	fCantonese	zung1
U+7C66.0	fMandarin	zhong1
U+7C67.0	fUTF8	籧
U+7C67.0	fCNS11643	2-6F3B
U+7C67.0	fBig5	F7EC
U+7C67.0	fGB	3-653A
U+7C67.0	fR/S	 118.17
U+7C67.0	fTotalStrokes	 23
U+7C67.0	fCangjie	HYYO
U+7C67.0	fFourCorner	88303
U+7C67.0	fHakka	 gi2;  ki2
U+7C67.0	fCantonese	keoi4; geoi2
U+7C67.0	fMandarin	qu2
U+7C67.0	fEnglish	 crude bamboo mat
U+7C67.1	fUTF8	籧
U+7C67.1	fCNS11643	2-6F3B
U+7C67.1	fBig5	F7EC
U+7C67.1	fGB	3-653A
U+7C67.1	fR/S	 118.17
U+7C67.1	fTotalStrokes	 23
U+7C67.1	fCangjie	HYYO
U+7C67.1	fFourCorner	88303
U+7C67.1	fMandarin	ju3
U+7C67.1	fEnglish	a vessel for raising silkworms
U+7C68.0	fUTF8	籨
U+7C68.0	fCNS11643	3-6626
U+7C68.0	fGB	E-3753
U+7C68.0	fR/S	 118.17
U+7C68.0	fTotalStrokes	 23
U+7C68.0	fCangjie	HOOO
U+7C68.0	fFourCorner	8888
U+7C68.0	fCantonese	lim4
U+7C68.0	fMandarin	lian2
U+7C68.0	fEnglish	trousseau
U+7C69.0	fUTF8	籩
U+7C69.0	fCNS11643	2-712B
U+7C69.0	fBig5	F8FB
U+7C69.0	fGB	1-7356
U+7C69.0	fR/S	 118.19
U+7C69.0	fTotalStrokes	 25
U+7C69.0	fCangjie	HYHS
U+7C69.0	fFourCorner	88302
U+7C69.0	fHakka	 bien1
U+7C69.0	fCantonese	bin1
U+7C69.0	fMandarin	bian1
U+7C69.0	fEnglish	a bamboo container for food, also used to hold fruit or dried meat, etc. in sacrifices
U+7C6A.0	fUTF8	籪
U+7C6A.0	fCNS11643	2-7046
U+7C6A.0	fBig5	F8B8
U+7C6A.0	fGB	1-737D
U+7C6A.0	fR/S	 118.18
U+7C6A.0	fTotalStrokes	 24
U+7C6A.0	fCangjie	HVIL
U+7C6A.0	fFourCorner	88721
U+7C6A.0	fHakka	 ton5; ton6; don6; don5
U+7C6A.0	fCantonese	dyun6
U+7C6A.0	fMandarin	duan4
U+7C6A.0	fEnglish	a weir of bamboo to catch fish or crabs
U+7C6B.0	fUTF8	籫
U+7C6B.0	fCNS11643	2-712C
U+7C6B.0	fBig5	F8FC
U+7C6B.0	fGB	3-653B
U+7C6B.0	fR/S	 118.19
U+7C6B.0	fTotalStrokes	 25
U+7C6B.0	fCangjie	HHUC
U+7C6B.0	fFourCorner	88806
U+7C6B.0	fCantonese	zyun2
U+7C6B.0	fMandarin	zuan3
U+7C6C.0	fUTF8	籬
U+7C6C.0	fCNS11643	1-7D25
U+7C6C.0	fBig5	C658
U+7C6C.0	fGB	1-4069
U+7C6C.0	fR/S	 118.19
U+7C6C.0	fTotalStrokes	 25
U+7C6C.0	fCangjie	HYBG
U+7C6C.0	fFourCorner	88414
U+7C6C.0	fHakka	  li2; li3
U+7C6C.0	fCantonese	lei4; lei1 (also)
U+7C6C.0	fMandarin	li2
U+7C6C.0	fEnglish	 bamboo or wooden fence; hedge
U+7C6D.0	fUTF8	籭
U+7C6D.0	fCNS11643	3-6144
U+7C6D.0	fGB	5-633D
U+7C6D.0	fR/S	 118.19
U+7C6D.0	fTotalStrokes	 25
U+7C6D.0	fCangjie	HMMP
U+7C6D.0	fFourCorner	88210
U+7C6D.0	fHakka	 si1; ci1; ts'e1
U+7C6D.0	fCantonese	sai1
U+7C6D.0	fMandarin	si1; shi1; xi3; shai1
U+7C6E.0	fUTF8	籮
U+7C6E.0	fCNS11643	1-7D26
U+7C6E.0	fBig5	C659
U+7C6E.0	fGB	1-4261
U+7C6E.0	fR/S	 118.19
U+7C6E.0	fTotalStrokes	 25
U+7C6E.0	fCangjie	HWLG
U+7C6E.0	fFourCorner	88914
U+7C6E.0	fHakka	  lo2; lo3; cung1
U+7C6E.0	fCantonese	lo4; lo1 (also); lo2 (also)
U+7C6E.0	fMandarin	luo1; luo2
U+7C6E.0	fEnglish	[1] bamboo basket with a square or rectangular bottom and a round top [2] a piece of siee-like ware with a broad edge
U+7C6F.0	fUTF8	籯
U+7C6F.0	fCNS11643	2-715C
U+7C6F.0	fBig5	F96D
U+7C6F.0	fGB	3-6540
U+7C6F.0	fR/S	 118.20
U+7C6F.0	fTotalStrokes	 26
U+7C6F.0	fCangjie	HYNC
U+7C6F.0	fFourCorner	88217
U+7C6F.0	fHakka	 jin3; rin2; jin2
U+7C6F.0	fCantonese	jing4
U+7C6F.0	fMandarin	ying2
U+7C6F.0	fEnglish	 bamboo chest tubular bamboo case
U+7C70.0	fUTF8	籰
U+7C70.0	fCNS11643	4-6D5B
U+7C70.0	fGB	5-6458
U+7C70.0	fR/S	 118.20
U+7C70.0	fTotalStrokes	 26
U+7C70.0	fCangjie	HBUE
U+7C70.0	fFourCorner	88400
U+7C70.0	fHakka	 kiok7; wok7; vok7
U+7C70.0	fCantonese	wok6; jyut6 (also); wik6
U+7C70.0	fMandarin	yue4; yu4
U+7C71.0	fUTF8	籱
U+7C71.0	fCNS11643	3-623A
U+7C71.0	fGB	E-3754
U+7C71.0	fR/S	  118.24
U+7C71.0	fTotalStrokes	 30
U+7C71.0	fCangjie	HMBG
U+7C71.0	fFourCorner	88210
U+7C71.0	fCantonese	zoek3
U+7C71.0	fMandarin	zhuo2; kuo4
U+7C72.0	fUTF8	籲
U+7C72.0	fCNS11643	1-7D4B
U+7C72.0	fBig5	C67E
U+7C72.0	fGB	1-787B
U+7C72.0	fR/S	 118.26
U+7C72.0	fTotalStrokes	 32
U+7C72.0	fCangjie	HOBC
U+7C72.0	fFourCorner	81286
U+7C72.0	fHakka	 jok7; ji5; ri5; ji1; ji3; rok7
U+7C72.0	fCantonese	jyu6; joek6 (also)
U+7C72.0	fMandarin	yu4; yao4; xu1; yue4
U+7C72.0	fEnglish	 appeal; request; implore; ask; urge; beseech
U+7C73.0	fUTF8	米
U+7C73.0	fCNS11643	1-484D
U+7C73.0	fBig5	A6CC
U+7C73.0	fGB	0-4357
U+7C73.0	fR/S	 119.0
U+7C73.0	fTotalStrokes	 6
U+7C73.0	fCangjie	FD
U+7C73.0	fFourCorner	90904
U+7C73.0	fHakka	  mi3
U+7C73.0	fCantonese	mai5; mai2 (also); mai1
U+7C73.0	fMandarin	mi3
U+7C73.0	fEnglish	[1] hulled or husked uncooked rice; uncooked rice [2] a shelled or a husked seed [3] meter (unit of length) [4] a Chinese family name
U+7C74.0	fUTF8	籴
U+7C74.0	fCNS11643	3-2A49
U+7C74.0	fGB	0-5961
U+7C74.0	fR/S	 119.2
U+7C74.0	fTotalStrokes	 8
U+7C74.0	fCangjie	OFD
U+7C74.0	fFourCorner	80900
U+7C74.0	fHakka	 tak8; tet8
U+7C74.0	fCantonese	dek6
U+7C74.0	fMandarin	di2; zhuo2; za2; cao2
U+7C74.0	fEnglish	simplified form of &#x7CF4;
U+7C75.0	fUTF8	籵
U+7C75.0	fCNS11643	2-2848
U+7C75.0	fBig5	CDA8
U+7C75.0	fGB	E-3755
U+7C75.0	fR/S	 119.2
U+7C75.0	fTotalStrokes	 8
U+7C75.0	fCangjie	FDJ
U+7C75.0	fFourCorner	94900
U+7C75.0	fCantonese	faan4
U+7C75.0	fMandarin	fan2
U+7C75.0	fEnglish	dm
U+7C76.0	fUTF8	籶
U+7C76.0	fCNS11643	3-2A48
U+7C76.0	fGB	E-3756
U+7C76.0	fR/S	 119.2
U+7C76.0	fTotalStrokes	 8
U+7C76.0	fCangjie	FDHN
U+7C76.0	fFourCorner	97910
U+7C76.0	fCantonese	san1
U+7C76.0	fMandarin	shen1
U+7C77.0	fUTF8	籷
U+7C77.0	fCNS11643	3-2E52
U+7C77.0	fGB	3-676C
U+7C77.0	fR/S	  119.3
U+7C77.0	fTotalStrokes	 9
U+7C77.0	fCangjie	FDHP
U+7C77.0	fFourCorner	92910
U+7C77.0	fCantonese	zaak3
U+7C77.0	fMandarin	zhe2
U+7C78.0	fUTF8	籸
U+7C78.0	fCNS11643	2-2C66
U+7C78.0	fBig5	D045
U+7C78.0	fGB	5-6655
U+7C78.0	fR/S	 119.3
U+7C78.0	fTotalStrokes	 9
U+7C78.0	fCangjie	FDNJ
U+7C78.0	fFourCorner	97910
U+7C78.0	fHakka	  sin1; ku1
U+7C78.0	fCantonese	san1; saam2 (also); sam2; sam3; fan1
U+7C78.0	fMandarin	shen1
U+7C78.0	fEnglish	powdery substance
U+7C79.0	fUTF8	籹
U+7C79.0	fCNS11643	2-2C67
U+7C79.0	fBig5	D046
U+7C79.0	fGB	3-676D
U+7C79.0	fR/S	 119.3
U+7C79.0	fTotalStrokes	 9
U+7C79.0	fCangjie	FDV
U+7C79.0	fFourCorner	94944
U+7C79.0	fHakka	 ni3
U+7C79.0	fCantonese	neoi5; jyu5 (also); leoi5 (also)
U+7C79.0	fMandarin	n3; ru3; nuu3
U+7C79.0	fEnglish	cake
U+7C7A.0	fUTF8	籺
U+7C7A.0	fCNS11643	2-2C65
U+7C7A.0	fBig5	D044
U+7C7A.0	fGB	5-6654
U+7C7A.0	fR/S	 119.3
U+7C7A.0	fTotalStrokes	 9
U+7C7A.0	fCangjie	FDON
U+7C7A.0	fFourCorner	98917
U+7C7A.0	fCantonese	hat6
U+7C7A.0	fMandarin	xie2; xi4; he2
U+7C7B.0	fUTF8	类
U+7C7B.0	fCNS11643	3-2E54
U+7C7B.0	fGB	0-4060
U+7C7B.0	fR/S	 37.6; 119.3
U+7C7B.0	fTotalStrokes	 9
U+7C7B.0	fCangjie	FDK
U+7C7B.0	fFourCorner	90439
U+7C7B.0	fHakka	 lui5
U+7C7B.0	fCantonese	leoi6; leoi2 (also)
U+7C7B.0	fMandarin	lei4; luu4; li4
U+7C7B.0	fEnglish	simplified form of &#x985E;
U+7C7C.0	fUTF8	籼
U+7C7C.0	fCNS11643	3-2E53
U+7C7C.0	fGB	0-744C
U+7C7C.0	fR/S	 119.3
U+7C7C.0	fTotalStrokes	 9
U+7C7C.0	fCangjie	FDU
U+7C7C.0	fFourCorner	92970
U+7C7C.0	fHakka	 sien1
U+7C7C.0	fCantonese	sin1
U+7C7C.0	fMandarin	xian1
U+7C7C.0	fEnglish	non-glutinous long grain rice
U+7C7D.0	fUTF8	籽
U+7C7D.0	fCNS11643	1-5276
U+7C7D.0	fBig5	ACF3
U+7C7D.0	fGB	0-5751
U+7C7D.0	fR/S	 119.3
U+7C7D.0	fTotalStrokes	 9
U+7C7D.0	fCangjie	FDND
U+7C7D.0	fFourCorner	97947
U+7C7D.0	fHakka	 zih3; zu3
U+7C7D.0	fCantonese	zi2
U+7C7D.0	fMandarin	zi3
U+7C7D.0	fEnglish	 seed of plants
U+7C7E.0	fUTF8	籾
U+7C7E.0	fCNS11643	F-2D21
U+7C7E.0	fGB	E-3757
U+7C7E.0	fR/S	 119.3
U+7C7E.0	fTotalStrokes	 9
U+7C7E.0	fCangjie	FDSHI
U+7C7E.0	fFourCorner	9792
U+7C7E.0	fCantonese	nei4; lei4 (also)
U+7C7E.0	fMandarin	ni2
U+7C7E.0	fEnglish	unhulled rice
U+7C7F.0	fUTF8	籿
U+7C7F.0	fCNS11643	2-2C68
U+7C7F.0	fBig5	D047
U+7C7F.0	fGB	E-3758
U+7C7F.0	fR/S	 119.3
U+7C7F.0	fTotalStrokes	 9
U+7C7F.0	fCangjie	FDDI
U+7C7F.0	fFourCorner	94900
U+7C7F.0	fCantonese	fan1
U+7C7F.0	fMandarin	cun4
U+7C80.0	fUTF8	粀
U+7C80.0	fCNS11643	2-2C69
U+7C80.0	fBig5	D048
U+7C80.0	fGB	E-3759
U+7C80.0	fR/S	 119.3
U+7C80.0	fTotalStrokes	 9
U+7C80.0	fCangjie	FDJK
U+7C80.0	fFourCorner	95900
U+7C80.0	fCantonese	sap6
U+7C80.0	fMandarin	zhang4
U+7C81.0	fUTF8	粁
U+7C81.0	fCNS11643	2-2C6A
U+7C81.0	fBig5	D049
U+7C81.0	fGB	E-375A
U+7C81.0	fR/S	 119.3
U+7C81.0	fTotalStrokes	 9
U+7C81.0	fCangjie	FDHJ
U+7C81.0	fFourCorner	92940
U+7C81.0	fCantonese	cin1
U+7C81.0	fMandarin	qian1
U+7C81.0	fEnglish	km
U+7C82.0	fUTF8	粂
U+7C82.0	fGB	E-375B
U+7C82.0	fR/S	 119.3
U+7C82.0	fTotalStrokes	 9
U+7C82.0	fCangjie	NOFD
U+7C82.0	fCantonese	gau2
U+7C82.0	fEnglish	 a family name
U+7C83.0	fUTF8	粃
U+7C83.0	fCNS11643	3-3331
U+7C83.0	fGB	E-375C
U+7C83.0	fR/S	 119.4
U+7C83.0	fTotalStrokes	 10
U+7C83.0	fCangjie	FDPP
U+7C83.0	fFourCorner	91910
U+7C83.0	fHakka	 pi1; bi3; {pi1}
U+7C83.0	fCantonese	bei2; pei4
U+7C83.0	fMandarin	bi3
U+7C83.0	fEnglish	empty husks of grain
U+7C84.0	fUTF8	粄
U+7C84.0	fCNS11643	2-316B
U+7C84.0	fBig5	D349
U+7C84.0	fGB	5-6658
U+7C84.0	fR/S	 119.4
U+7C84.0	fTotalStrokes	 10
U+7C84.0	fCangjie	FDHE
U+7C84.0	fFourCorner	91947
U+7C84.0	fHakka	  ban3
U+7C84.0	fCantonese	bun2; baan2
U+7C84.0	fMandarin	ban3
U+7C84.0	fEnglish	 cake; dumpling
U+7C85.0	fUTF8	粅
U+7C85.0	fCNS11643	2-3171
U+7C85.0	fBig5	D34F
U+7C85.0	fGB	5-6659
U+7C85.0	fR/S	 119.4
U+7C85.0	fTotalStrokes	 10
U+7C85.0	fCangjie	FDPHH
U+7C85.0	fFourCorner	97920
U+7C85.0	fCantonese	mat6
U+7C85.0	fMandarin	wu4
U+7C86.0	fUTF8	粆
U+7C86.0	fCNS11643	3-332E
U+7C86.0	fGB	3-6770
U+7C86.0	fR/S	 119.4
U+7C86.0	fTotalStrokes	 10
U+7C86.0	fCangjie	FDFH
U+7C86.0	fFourCorner	99920
U+7C86.0	fHakka	 sa1
U+7C86.0	fCantonese	saa1
U+7C86.0	fMandarin	sha1; chao3
U+7C86.0	fEnglish	coarse sugar
U+7C87.0	fUTF8	粇
U+7C87.0	fCNS11643	3-3330
U+7C87.0	fGB	E-375D
U+7C87.0	fR/S	  119.4
U+7C87.0	fTotalStrokes	 10
U+7C87.0	fCangjie	FDYHN
U+7C87.0	fFourCorner	90910
U+7C87.0	fCantonese	hong1
U+7C87.0	fMandarin	kang1
U+7C88.0	fUTF8	粈
U+7C88.0	fCNS11643	2-316F
U+7C88.0	fBig5	D34D
U+7C88.0	fGB	3-6772
U+7C88.0	fR/S	 119.4
U+7C88.0	fTotalStrokes	 10
U+7C88.0	fCangjie	FDNG
U+7C88.0	fFourCorner	97915
U+7C88.0	fHakka	 neu2
U+7C88.0	fCantonese	jau2; jau5; nau5
U+7C88.0	fMandarin	rou3; niu4
U+7C89.0	fUTF8	粉
U+7C89.0	fCNS11643	1-573E
U+7C89.0	fBig5	AFBB
U+7C89.0	fGB	0-375B
U+7C89.0	fR/S	 119.4
U+7C89.0	fTotalStrokes	 10
U+7C89.0	fCangjie	FDCSH
U+7C89.0	fFourCorner	98927
U+7C89.0	fHakka	  fun3
U+7C89.0	fCantonese	fan2
U+7C89.0	fMandarin	fen3; fen4
U+7C89.0	fEnglish	[1] flour [2] powder [3] white (color) [4] whitewash; plaster [5] make up; doll up; powder
U+7C8A.0	fUTF8	粊
U+7C8A.0	fCNS11643	2-316D
U+7C8A.0	fBig5	D34B
U+7C8A.0	fGB	E-375E
U+7C8A.0	fR/S	 119.4
U+7C8A.0	fTotalStrokes	 10
U+7C8A.0	fCangjie	PPFD
U+7C8A.0	fFourCorner	21909
U+7C8A.0	fCantonese	bei3
U+7C8A.0	fMandarin	bi4
U+7C8B.0	fUTF8	粋
U+7C8B.0	fCNS11643	3-3332
U+7C8B.0	fGB	E-375F
U+7C8B.0	fR/S	 119.4
U+7C8B.0	fTotalStrokes	 10
U+7C8B.0	fCangjie	FDKNJ
U+7C8B.0	fFourCorner	94940
U+7C8B.0	fCantonese	seoi6
U+7C8B.0	fMandarin	sui4; cui4
U+7C8B.0	fEnglish	 pure unadulterated select
U+7C8C.0	fUTF8	粌
U+7C8C.0	fCNS11643	2-316E
U+7C8C.0	fBig5	D34C
U+7C8C.0	fGB	E-3760
U+7C8C.0	fR/S	 119.4
U+7C8C.0	fTotalStrokes	 10
U+7C8C.0	fCangjie	FDNL
U+7C8C.0	fFourCorner	92900
U+7C8C.0	fCantonese	baak3
U+7C8C.0	fMandarin	yin3
U+7C8D.0	fUTF8	粍
U+7C8D.0	fCNS11643	2-3170
U+7C8D.0	fBig5	D34E
U+7C8D.0	fGB	E-3761
U+7C8D.0	fR/S	 119.4
U+7C8D.0	fTotalStrokes	 10
U+7C8D.0	fCangjie	FDHQU
U+7C8D.0	fFourCorner	92914
U+7C8D.0	fCantonese	hou4
U+7C8D.0	fMandarin	li2
U+7C8D.0	fEnglish	mm
U+7C8E.0	fUTF8	粎
U+7C8E.0	fCNS11643	3-332D
U+7C8E.0	fGB	E-3762
U+7C8E.0	fR/S	 119.4
U+7C8E.0	fTotalStrokes	 10
U+7C8E.0	fCangjie	FDSO
U+7C8E.0	fFourCorner	97980
U+7C8E.0	fCantonese	mai5
U+7C8E.0	fMandarin	chi3
U+7C8E.0	fEnglish	metre
U+7C8F.0	fUTF8	粏
U+7C8F.0	fGB	E-3763
U+7C8F.0	fR/S	 119.4
U+7C8F.0	fTotalStrokes	 10
U+7C8F.0	fCangjie	FDKI
U+7C8F.0	fCantonese	taai3
U+7C8F.0	fMandarin	tai4
U+7C90.0	fUTF8	粐
U+7C90.0	fGB	E-3764
U+7C90.0	fR/S	 119.4
U+7C90.0	fTotalStrokes	 10
U+7C90.0	fCangjie	FDMS
U+7C90.0	fCantonese	wu6
U+7C91.0	fUTF8	粑
U+7C91.0	fCNS11643	2-316C
U+7C91.0	fBig5	D34A
U+7C91.0	fGB	0-744E
U+7C91.0	fR/S	 119.4
U+7C91.0	fTotalStrokes	 10
U+7C91.0	fCangjie	FDAU
U+7C91.0	fFourCorner	97917
U+7C91.0	fHakka	 ba1
U+7C91.0	fCantonese	baa1
U+7C91.0	fMandarin	ba1
U+7C92.0	fUTF8	粒
U+7C92.0	fCNS11643	1-5C4E
U+7C92.0	fBig5	B2C9
U+7C92.0	fGB	0-4123
U+7C92.0	fR/S	 119.5
U+7C92.0	fTotalStrokes	 11
U+7C92.0	fCangjie	FDYT
U+7C92.0	fFourCorner	90918
U+7C92.0	fHakka	  lip7;  liap8; liap7
U+7C92.0	fCantonese	nap1; lap1 (also)
U+7C92.0	fMandarin	li4
U+7C92.0	fEnglish	[1] grain (of rice, etc.) [2] rice food; get grain to eat [3] oill; bead
U+7C93.0	fUTF8	粓
U+7C93.0	fCNS11643	3-377D
U+7C93.0	fGB	5-665B
U+7C93.0	fR/S	  119.5
U+7C93.0	fTotalStrokes	 11
U+7C93.0	fCangjie	FDTM
U+7C93.0	fFourCorner	94970
U+7C93.0	fHakka	 gam1
U+7C93.0	fCantonese	gam1
U+7C93.0	fMandarin	gan1
U+7C94.0	fUTF8	粔
U+7C94.0	fCNS11643	2-3823
U+7C94.0	fBig5	D6DE
U+7C94.0	fGB	3-676F
U+7C94.0	fR/S	 119.5
U+7C94.0	fTotalStrokes	 11
U+7C94.0	fCangjie	FDSS
U+7C94.0	fFourCorner	91917
U+7C94.0	fHakka	 ki3
U+7C94.0	fCantonese	geoi6; geoi2 (also); geoi3
U+7C94.0	fMandarin	ju4
U+7C94.0	fEnglish	cakes made from rice flour twisted into rings
U+7C95.0	fUTF8	粕
U+7C95.0	fCNS11643	1-5C50
U+7C95.0	fBig5	B2CB
U+7C95.0	fGB	0-4649
U+7C95.0	fR/S	 119.5
U+7C95.0	fTotalStrokes	 11
U+7C95.0	fCangjie	FDHA
U+7C95.0	fFourCorner	96900
U+7C95.0	fHakka	 pok7; pak7
U+7C95.0	fCantonese	pok3; paak3 (also)
U+7C95.0	fMandarin	po4; bo2; pai1
U+7C95.0	fEnglish	the lees; dregs; sediments left after distilling liquor
U+7C96.0	fUTF8	粖
U+7C96.0	fCNS11643	2-3825
U+7C96.0	fBig5	D6E0
U+7C96.0	fGB	5-665A
U+7C96.0	fR/S	 119.5
U+7C96.0	fTotalStrokes	 11
U+7C96.0	fCangjie	FDDJ
U+7C96.0	fFourCorner	95990
U+7C96.0	fHakka	 mat8
U+7C96.0	fCantonese	mit6; mut6
U+7C96.0	fMandarin	mo4; mie4
U+7C97.0	fUTF8	粗
U+7C97.0	fCNS11643	1-5C4F
U+7C97.0	fBig5	B2CA
U+7C97.0	fGB	0-3456
U+7C97.0	fR/S	 119.5
U+7C97.0	fTotalStrokes	 11
U+7C97.0	fCangjie	FDBM
U+7C97.0	fFourCorner	97910
U+7C97.0	fHakka	 cu1; cih1; ze1
U+7C97.0	fCantonese	cou1
U+7C97.0	fMandarin	cu1; zu4
U+7C97.0	fEnglish	[1] rough; course; crude [2] thick; bulky; big [3] gruff; husky [4] rude; vulgar [5] brief; sketchy
U+7C98.0	fUTF8	粘
U+7C98.0	fCNS11643	2-3824
U+7C98.0	fBig5	D6DF
U+7C98.0	fGB	0-5533
U+7C98.0	fR/S	 119.5
U+7C98.0	fTotalStrokes	 11
U+7C98.0	fCangjie	FDYR
U+7C98.0	fFourCorner	91960
U+7C98.0	fHakka	 ngiam2;  zam1;  zham1; niam2; fu2
U+7C98.0	fCantonese	zim1; nim4; nim1; lim1 (also); lim4 (also)
U+7C98.0	fMandarin	nian2; zhan1; lian1
U+7C98.0	fEnglish	 paste up; attach; stick up; glue
U+7C99.0	fUTF8	粙
U+7C99.0	fCNS11643	3-377E
U+7C99.0	fGB	5-665C
U+7C99.0	fR/S	 119.5
U+7C99.0	fTotalStrokes	 11
U+7C99.0	fCangjie	FDLW
U+7C99.0	fFourCorner	95960
U+7C99.0	fCantonese	zau6
U+7C99.0	fMandarin	zhou4
U+7C9A.0	fUTF8	粚
U+7C9A.0	fCNS11643	3-3821
U+7C9A.0	fGB	5-665E
U+7C9A.0	fR/S	  119.5
U+7C9A.0	fTotalStrokes	 11
U+7C9A.0	fCangjie	FDOPD
U+7C9A.0	fFourCorner	98910
U+7C9A.0	fCantonese	ci1
U+7C9A.0	fMandarin	li2
U+7C9B.0	fUTF8	粛
U+7C9B.0	fCNS11643	3-383E
U+7C9B.0	fGB	E-3765
U+7C9B.0	fR/S	 129.6; 119.5
U+7C9B.0	fTotalStrokes	 11
U+7C9B.0	fCangjie	LLFL
U+7C9B.0	fFourCorner	50220
U+7C9B.0	fCantonese	suk1
U+7C9B.0	fMandarin	su4
U+7C9B.0	fEnglish	 pay respects reverently
U+7C9C.0	fUTF8	粜
U+7C9C.0	fCNS11643	3-3822
U+7C9C.0	fGB	0-7450
U+7C9C.0	fR/S	 119.5
U+7C9C.0	fTotalStrokes	 11
U+7C9C.0	fCangjie	UUFD
U+7C9C.0	fFourCorner	22900
U+7C9C.0	fHakka	 tiau5
U+7C9C.0	fCantonese	tiu3
U+7C9C.0	fMandarin	tiao4; diao4
U+7C9C.0	fEnglish	simplified form of &#x7CF6;
U+7C9D.0	fUTF8	粝
U+7C9D.0	fCNS11643	4-3422
U+7C9D.0	fGB	0-744F
U+7C9D.0	fR/S	 119.5
U+7C9D.0	fTotalStrokes	 11
U+7C9D.0	fCangjie	FDMMS
U+7C9D.0	fFourCorner	91920
U+7C9D.0	fHakka	 li5
U+7C9D.0	fCantonese	lai6
U+7C9D.0	fMandarin	li4; la4; lai4
U+7C9D.0	fEnglish	simplified form of &#x7CF2;
U+7C9E.0	fUTF8	粞
U+7C9E.0	fCNS11643	2-3E6C
U+7C9E.0	fBig5	DAE8
U+7C9E.0	fGB	0-7451
U+7C9E.0	fR/S	 119.6
U+7C9E.0	fTotalStrokes	 12
U+7C9E.0	fCangjie	FDMCW
U+7C9E.0	fFourCorner	91963
U+7C9E.0	fHakka	 si1
U+7C9E.0	fCantonese	sai1
U+7C9E.0	fMandarin	xi1; sai1; qi1; xi4
U+7C9E.0	fEnglish	broken rice that remains in the mortar after husking
U+7C9F.0	fUTF8	粟
U+7C9F.0	fCNS11643	1-6135
U+7C9F.0	fBig5	B5AF
U+7C9F.0	fGB	0-4B5A
U+7C9F.0	fR/S	 119.6
U+7C9F.0	fTotalStrokes	 12
U+7C9F.0	fCangjie	MWFD
U+7C9F.0	fFourCorner	10904
U+7C9F.0	fHakka	  siuk7; suk7
U+7C9F.0	fCantonese	suk1
U+7C9F.0	fMandarin	su4
U+7C9F.0	fEnglish	[1] grain; paddy [2] millet [3] goose flesh; goose pimples; goose bumps; goose skin [4] a Chinese family name
U+7CA0.0	fUTF8	粠
U+7CA0.0	fCNS11643	3-3E21
U+7CA0.0	fGB	5-6665
U+7CA0.0	fR/S	  119.6
U+7CA0.0	fTotalStrokes	 12
U+7CA0.0	fCangjie	FDTC
U+7CA0.0	fFourCorner	94980
U+7CA0.0	fCantonese	hung4
U+7CA0.0	fMandarin	hong2
U+7CA1.0	fUTF8	粡
U+7CA1.0	fCNS11643	2-3E6E
U+7CA1.0	fBig5	DAEA
U+7CA1.0	fGB	3-6777
U+7CA1.0	fR/S	 119.6
U+7CA1.0	fTotalStrokes	 12
U+7CA1.0	fCangjie	FDBMR
U+7CA1.0	fFourCorner	97920
U+7CA1.0	fCantonese	tung4
U+7CA1.0	fMandarin	tong2
U+7CA2.0	fUTF8	粢
U+7CA2.0	fCNS11643	2-3E6B
U+7CA2.0	fBig5	DAE7
U+7CA2.0	fGB	0-7452
U+7CA2.0	fR/S	 119.6
U+7CA2.0	fTotalStrokes	 12
U+7CA2.0	fCangjie	IOFD
U+7CA2.0	fFourCorner	37904
U+7CA2.0	fHakka	 zih1; zu1; zii1
U+7CA2.0	fCantonese	zi1; ci1
U+7CA2.0	fMandarin	zi1; ji4
U+7CA2.0	fEnglish	[1] the millet to be offered in ritual sacrifice [2] the grain
U+7CA2.1	fUTF8	粢
U+7CA2.1	fCNS11643	2-3E6B
U+7CA2.1	fBig5	DAE7
U+7CA2.1	fGB	0-7452
U+7CA2.1	fR/S	 119.6
U+7CA2.1	fTotalStrokes	 12
U+7CA2.1	fCangjie	IOFD
U+7CA2.1	fFourCorner	37904
U+7CA2.1	fMandarin	ci2
U+7CA2.1	fEnglish	rice cakes
U+7CA3.0	fUTF8	粣
U+7CA3.0	fCNS11643	2-3826
U+7CA3.0	fBig5	D6E1
U+7CA3.0	fGB	3-6774
U+7CA3.0	fR/S	 119.5
U+7CA3.0	fTotalStrokes	 11
U+7CA3.0	fCangjie	FDBT
U+7CA3.0	fFourCorner	97940
U+7CA3.0	fCantonese	saan2
U+7CA3.0	fMandarin	ce4; san3
U+7CA4.0	fUTF8	粤
U+7CA4.0	fCNS11643	3-3E23
U+7CA4.0	fGB	0-5441
U+7CA4.0	fR/S	  119.6
U+7CA4.0	fTotalStrokes	 12
U+7CA4.0	fCangjie	XHWMV
U+7CA4.0	fFourCorner	26200
U+7CA4.0	fHakka	 jet8; ret8
U+7CA4.0	fCantonese	jyut6
U+7CA4.0	fMandarin	yue4
U+7CA4.0	fEnglish	 Guangdong and Guangxi provinces initial particle
U+7CA5.0	fUTF8	粥
U+7CA5.0	fCNS11643	1-6136
U+7CA5.0	fBig5	B5B0
U+7CA5.0	fGB	0-5660
U+7CA5.0	fR/S	 119.6
U+7CA5.0	fTotalStrokes	 12
U+7CA5.0	fCangjie	NFDN
U+7CA5.0	fFourCorner	17227
U+7CA5.0	fHakka	  zuk7;  chuk7; zhuk7
U+7CA5.0	fCantonese	zuk1; juk6
U+7CA5.0	fMandarin	zhou1; zhu4; yu4
U+7CA5.0	fEnglish	 rice gruel; congee
U+7CA6.0	fUTF8	粦
U+7CA6.0	fCNS11643	3-3E22
U+7CA6.0	fGB	8-2F7A
U+7CA6.0	fR/S	 119.6; 119.7
U+7CA6.0	fTotalStrokes	 12
U+7CA6.0	fCangjie	FDNIQ
U+7CA6.0	fFourCorner	90250
U+7CA6.0	fHakka	 lin2;  lin5; lin3
U+7CA6.0	fCantonese	leon4
U+7CA6.0	fMandarin	lin4; lin2
U+7CA6.0	fEnglish	phosphorus
U+7CA7.0	fUTF8	粧
U+7CA7.0	fCNS11643	3-3D7E
U+7CA7.0	fGB	E-3766
U+7CA7.0	fR/S	  119.6
U+7CA7.0	fTotalStrokes	 12
U+7CA7.0	fCangjie	FDIG
U+7CA7.0	fFourCorner	90914
U+7CA7.0	fHakka	 zong1
U+7CA7.0	fCantonese	zong1
U+7CA7.0	fMandarin	zhuang1
U+7CA7.0	fEnglish	[1] toilet; adorn; doll up [2] woman's personal adornments [3] bride's trousseau [4] disguise; pretend
U+7CA8.0	fUTF8	粨
U+7CA8.0	fCNS11643	2-3E6D
U+7CA8.0	fBig5	DAE9
U+7CA8.0	fGB	E-3767
U+7CA8.0	fR/S	 119.6
U+7CA8.0	fTotalStrokes	 12
U+7CA8.0	fCangjie	FDMA
U+7CA8.0	fFourCorner	91960
U+7CA8.0	fCantonese	baak3
U+7CA8.0	fMandarin	bai3; ben3
U+7CA8.0	fEnglish	hm
U+7CA9.0	fUTF8	粩
U+7CA9.0	fCNS11643	3-6627
U+7CA9.0	fGB	E-3768
U+7CA9.0	fR/S	 119.6
U+7CA9.0	fTotalStrokes	 12
U+7CA9.0	fCangjie	FDJKP
U+7CA9.0	fFourCorner	9491
U+7CA9.0	fCantonese	lou5
U+7CAA.0	fUTF8	粪
U+7CAA.0	fGB	0-3760
U+7CAA.0	fR/S	 119.6
U+7CAA.0	fTotalStrokes	 12
U+7CAA.0	fCangjie	FDTC
U+7CAA.0	fHakka	 bun5
U+7CAA.0	fCantonese	fan3
U+7CAA.0	fMandarin	fen4; san1
U+7CAA.0	fEnglish	simplified form of &#x7CDE;
U+7CAB.0	fUTF8	粫
U+7CAB.0	fCNS11643	F-3E21
U+7CAB.0	fGB	E-3769
U+7CAB.0	fR/S	 119.6
U+7CAB.0	fTotalStrokes	 12
U+7CAB.0	fCangjie	FDMBL
U+7CAB.0	fCantonese	ji4
U+7CAB.0	fMandarin	mian4
U+7CAC.0	fUTF8	粬
U+7CAC.0	fCNS11643	F-3E23
U+7CAC.0	fGB	5-6666
U+7CAC.0	fR/S	 119.6
U+7CAC.0	fTotalStrokes	 12
U+7CAC.0	fCangjie	FDTW
U+7CAC.0	fFourCorner	9596
U+7CAC.0	fCantonese	kuk1
U+7CAC.0	fMandarin	qu1
U+7CAD.0	fUTF8	粭
U+7CAD.0	fGB	E-376A
U+7CAD.0	fR/S	 119.6
U+7CAD.0	fTotalStrokes	 12
U+7CAD.0	fCangjie	FDOMR
U+7CAD.0	fCantonese	hap6
U+7CAE.0	fUTF8	粮
U+7CAE.0	fCNS11643	3-4358
U+7CAE.0	fGB	0-4138
U+7CAE.0	fR/S	 119.7
U+7CAE.0	fTotalStrokes	 13
U+7CAE.0	fCangjie	FDIAV
U+7CAE.0	fFourCorner	93930
U+7CAE.0	fHakka	 liong2; liong3
U+7CAE.0	fCantonese	loeng4
U+7CAE.0	fMandarin	liang2
U+7CAE.0	fEnglish	simplified form of &#x7CE7;
U+7CAF.0	fUTF8	粯
U+7CAF.0	fCNS11643	2-457B
U+7CAF.0	fBig5	DF56
U+7CAF.0	fGB	5-6657
U+7CAF.0	fR/S	 119.7
U+7CAF.0	fTotalStrokes	 13
U+7CAF.0	fCangjie	FDBUU
U+7CAF.0	fFourCorner	96910
U+7CAF.0	fCantonese	haan6; jin6
U+7CAF.0	fMandarin	xian4
U+7CB0.0	fUTF8	粰
U+7CB0.0	fCNS11643	3-435C
U+7CB0.0	fGB	E-376B
U+7CB0.0	fR/S	  119.7
U+7CB0.0	fTotalStrokes	 13
U+7CB0.0	fCangjie	FDBND
U+7CB0.0	fFourCorner	92940
U+7CB0.0	fCantonese	fu1
U+7CB0.0	fMandarin	fu2; fu1; fou2
U+7CB1.0	fUTF8	粱
U+7CB1.0	fCNS11643	1-656B
U+7CB1.0	fBig5	B864
U+7CB1.0	fGB	0-413B
U+7CB1.0	fR/S	 119.7
U+7CB1.0	fTotalStrokes	 13
U+7CB1.0	fCangjie	EIFD
U+7CB1.0	fFourCorner	33904
U+7CB1.0	fHakka	  liong2; liong3
U+7CB1.0	fCantonese	loeng4
U+7CB1.0	fMandarin	liang2
U+7CB1.0	fEnglish	maize; grain; sorghums
U+7CB2.0	fUTF8	粲
U+7CB2.0	fCNS11643	2-4579
U+7CB2.0	fBig5	DF54
U+7CB2.0	fGB	0-7453
U+7CB2.0	fR/S	 119.7
U+7CB2.0	fTotalStrokes	 13
U+7CB2.0	fCangjie	YEFD
U+7CB2.0	fFourCorner	27904
U+7CB2.0	fHakka	  can3;  can5
U+7CB2.0	fCantonese	caan3
U+7CB2.0	fMandarin	can4
U+7CB2.0	fEnglish	[1] bright and clear; radiant [2] beautiful; splendid; excellent [3] smiling; laughing [4] well-polished rice
U+7CB3.0	fUTF8	粳
U+7CB3.0	fCNS11643	1-656C
U+7CB3.0	fBig5	B865
U+7CB3.0	fGB	0-3E2C
U+7CB3.0	fR/S	 119.7
U+7CB3.0	fTotalStrokes	 13
U+7CB3.0	fCangjie	FDMLK
U+7CB3.0	fFourCorner	91946
U+7CB3.0	fHakka	 gang1
U+7CB3.0	fCantonese	gang1; gaang1
U+7CB3.0	fMandarin	geng1; jing1
U+7CB3.0	fEnglish	 non-glutinous rice
U+7CB4.0	fUTF8	粴
U+7CB4.0	fCNS11643	2-457A
U+7CB4.0	fBig5	DF55
U+7CB4.0	fGB	E-376C
U+7CB4.0	fR/S	 119.7
U+7CB4.0	fTotalStrokes	 13
U+7CB4.0	fCangjie	FDWG
U+7CB4.0	fFourCorner	96914
U+7CB4.0	fCantonese	gung1
U+7CB4.0	fMandarin	li3
U+7CB5.0	fUTF8	粵
U+7CB5.0	fCNS11643	1-656D
U+7CB5.0	fBig5	B866
U+7CB5.0	fGB	E-376D
U+7CB5.0	fR/S	 119.7
U+7CB5.0	fTotalStrokes	 13
U+7CB5.0	fCangjie	HWMVS
U+7CB5.0	fFourCorner	26207
U+7CB5.0	fHakka	 jet8; jat8
U+7CB5.0	fCantonese	jyut6
U+7CB5.0	fMandarin	yue4
U+7CB5.0	fEnglish	[1] Guangdong province [2] Guangdong and Guangxi provinces [3] a initial particle
U+7CB6.0	fUTF8	粶
U+7CB6.0	fCNS11643	4-4574
U+7CB6.0	fGB	E-376E
U+7CB6.0	fR/S	  119.8
U+7CB6.0	fTotalStrokes	 14
U+7CB6.0	fCangjie	FDVNE
U+7CB6.0	fFourCorner	97930
U+7CB6.0	fCantonese	luk6
U+7CB6.0	fMandarin	lu4
U+7CB7.0	fUTF8	粷
U+7CB7.0	fCNS11643	3-4862
U+7CB7.0	fGB	5-666E
U+7CB7.0	fR/S	 119.8
U+7CB7.0	fTotalStrokes	 14
U+7CB7.0	fCangjie	FDPFD
U+7CB7.0	fFourCorner	97920
U+7CB7.0	fCantonese	guk1
U+7CB7.0	fMandarin	ju2
U+7CB8.0	fUTF8	粸
U+7CB8.0	fCNS11643	3-4861
U+7CB8.0	fGB	E-376F
U+7CB8.0	fR/S	 119.8
U+7CB8.0	fTotalStrokes	 14
U+7CB8.0	fCangjie	FDTMC
U+7CB8.0	fFourCorner	94980
U+7CB8.0	fHakka	  ki2
U+7CB8.0	fCantonese	kei4
U+7CB8.0	fMandarin	qi2
U+7CB8.0	fEnglish	a thin sweet cake
U+7CB9.0	fUTF8	粹
U+7CB9.0	fCNS11643	1-6A32
U+7CB9.0	fBig5	BAE9
U+7CB9.0	fGB	0-3462
U+7CB9.0	fR/S	 119.8
U+7CB9.0	fTotalStrokes	 14
U+7CB9.0	fCangjie	FDYOJ
U+7CB9.0	fFourCorner	90948
U+7CB9.0	fHakka	  sui5; cui5
U+7CB9.0	fCantonese	seoi6; seoi3; seoi5 (also)
U+7CB9.0	fMandarin	sui4; cui4
U+7CB9.0	fEnglish	[1] pure; unmixed; unadulterated; perfect [2] the essence; the best
U+7CBA.0	fUTF8	粺
U+7CBA.0	fCNS11643	2-4C68
U+7CBA.0	fBig5	E361
U+7CBA.0	fGB	E-3770
U+7CBA.0	fR/S	 119.8
U+7CBA.0	fTotalStrokes	 14
U+7CBA.0	fCangjie	FDHWJ
U+7CBA.0	fFourCorner	96940
U+7CBA.0	fHakka	 pai5; pai6
U+7CBA.0	fCantonese	baai6; bai6 (also); pei4
U+7CBA.0	fMandarin	bai4; ba4
U+7CBA.0	fEnglish	polished rice or millet
U+7CBB.0	fUTF8	粻
U+7CBB.0	fCNS11643	2-4C65
U+7CBB.0	fBig5	E35E
U+7CBB.0	fGB	3-6771
U+7CBB.0	fR/S	 119.8
U+7CBB.0	fTotalStrokes	 14
U+7CBB.0	fCangjie	FDSMV
U+7CBB.0	fFourCorner	91932
U+7CBB.0	fHakka	 zhong1; zong1; zhong3
U+7CBB.0	fCantonese	zoeng1
U+7CBB.0	fMandarin	zhang1; zhang4; cheng2
U+7CBB.0	fEnglish	provisions
U+7CBC.0	fUTF8	粼
U+7CBC.0	fCNS11643	2-4C67
U+7CBC.0	fBig5	E360
U+7CBC.0	fGB	0-7454
U+7CBC.0	fR/S	 119.8
U+7CBC.0	fTotalStrokes	 14
U+7CBC.0	fCangjie	FQVV
U+7CBC.0	fFourCorner	92200
U+7CBC.0	fHakka	 lin2; lin3
U+7CBC.0	fCantonese	leon4
U+7CBC.0	fMandarin	lin2; lin3
U+7CBC.0	fEnglish	[1] mountain streams [2] water flowing
U+7CBD.0	fUTF8	粽
U+7CBD.0	fCNS11643	1-6A33
U+7CBD.0	fBig5	BAEA
U+7CBD.0	fGB	0-7455
U+7CBD.0	fR/S	 119.8
U+7CBD.0	fTotalStrokes	 14
U+7CBD.0	fCangjie	FDJMF
U+7CBD.0	fFourCorner	93991
U+7CBD.0	fHakka	  zung5
U+7CBD.0	fCantonese	zung2; zung3 (also)
U+7CBD.0	fMandarin	zong4; zhong4
U+7CBD.0	fEnglish	dumpling made of glutinous rice; glutinous rice tamale (made by wrapping the rice in broad leaves of reeds and boiled for a few hours, usually with other ingredients such as dates, meat, oysters, beans, etc.)
U+7CBE.0	fUTF8	精
U+7CBE.0	fCNS11643	1-6A34
U+7CBE.0	fBig5	BAEB
U+7CBE.0	fGB	0-3E2B
U+7CBE.0	fR/S	 119.8
U+7CBE.0	fTotalStrokes	 14
U+7CBE.0	fCangjie	FDQMB
U+7CBE.0	fFourCorner	95927
U+7CBE.0	fHakka	  zin1; siau3
U+7CBE.0	fCantonese	zing1; zeng1
U+7CBE.0	fMandarin	jing1; jing4; qing2
U+7CBE.0	fEnglish	[1] polished rice; unmixed rice [2] the essence; essentials [3] energy; spirits [4] sperm; semen [5] fine and delicate; exquisite [6] dedicated; intensive [7] very; extremely; completely [8] keen; smart; sharp; clever [9] skilled; specialize in [10] goblin; spirit; demon
U+7CBF.0	fUTF8	粿
U+7CBF.0	fCNS11643	2-4C66
U+7CBF.0	fBig5	E35F
U+7CBF.0	fGB	5-666C
U+7CBF.0	fR/S	 119.8
U+7CBF.0	fTotalStrokes	 14
U+7CBF.0	fCangjie	FDWD
U+7CBF.0	fFourCorner	96994
U+7CBF.0	fHakka	 go3; gwo3; kwo3
U+7CBF.0	fCantonese	gwo2; go2 (also)
U+7CBF.0	fMandarin	guo3; hua4
U+7CBF.0	fEnglish	rice cake
U+7CC0.0	fUTF8	糀
U+7CC0.0	fCNS11643	F-4B27
U+7CC0.0	fGB	E-3771
U+7CC0.0	fR/S	 119.7; 119.8
U+7CC0.0	fTotalStrokes	 13
U+7CC0.0	fCangjie	FDTOP
U+7CC0.0	fFourCorner	9491
U+7CC0.0	fCantonese	faa1
U+7CC1.0	fUTF8	糁
U+7CC1.0	fCNS11643	F-4B2B
U+7CC1.0	fGB	0-7456
U+7CC1.0	fR/S	 119.8
U+7CC1.0	fTotalStrokes	 14
U+7CC1.0	fCangjie	FDIKH
U+7CC1.0	fHakka	 sam3
U+7CC1.0	fCantonese	saam2 (also); sam2
U+7CC1.0	fMandarin	san1; san3; xin3
U+7CC1.0	fEnglish	simplified form of &#x7CDD;
U+7CC2.0	fUTF8	糂
U+7CC2.0	fCNS11643	3-4D6F
U+7CC2.0	fGB	5-6670
U+7CC2.0	fR/S	 119.9
U+7CC2.0	fTotalStrokes	 15
U+7CC2.0	fCangjie	FDTMV
U+7CC2.0	fFourCorner	94910
U+7CC2.0	fHakka	 sam3
U+7CC2.0	fCantonese	saam2 (also); sam2
U+7CC2.0	fMandarin	san3
U+7CC2.0	fEnglish	broken meat
U+7CC3.0	fUTF8	糃
U+7CC3.0	fCNS11643	3-4D71
U+7CC3.0	fGB	3-676E
U+7CC3.0	fR/S	  119.9
U+7CC3.0	fTotalStrokes	 15
U+7CC3.0	fCangjie	FDAMH
U+7CC3.0	fFourCorner	96920
U+7CC3.0	fCantonese	tong4
U+7CC3.0	fMandarin	tang2
U+7CC4.0	fUTF8	糄
U+7CC4.0	fCNS11643	3-4D6D
U+7CC4.0	fGB	3-677C
U+7CC4.0	fR/S	  119.9
U+7CC4.0	fTotalStrokes	 15
U+7CC4.0	fCangjie	FDHSB
U+7CC4.0	fFourCorner	93927
U+7CC4.0	fHakka	 bien1; bien3
U+7CC4.0	fCantonese	bin2
U+7CC4.0	fMandarin	bian1; bian3
U+7CC4.0	fEnglish	 parch rice in order to get off the husk
U+7CC5.0	fUTF8	糅
U+7CC5.0	fCNS11643	2-5267
U+7CC5.0	fBig5	E6DF
U+7CC5.0	fGB	0-745B
U+7CC5.0	fR/S	 119.9
U+7CC5.0	fTotalStrokes	 15
U+7CC5.0	fCangjie	FDNHD
U+7CC5.0	fFourCorner	97994
U+7CC5.0	fHakka	 ngiu3; ju3; ru3; ju2
U+7CC5.0	fCantonese	jau2; nau6 (also); lau6 (also); jau6
U+7CC5.0	fMandarin	rou3; rou2; niu4
U+7CC5.0	fEnglish	[v[ mix; mixed; blend]
U+7CC6.0	fUTF8	糆
U+7CC6.0	fCNS11643	3-4D70
U+7CC6.0	fGB	5-6675
U+7CC6.0	fR/S	  119.9
U+7CC6.0	fTotalStrokes	 15
U+7CC6.0	fCangjie	FDMWL
U+7CC6.0	fFourCorner	91960
U+7CC6.0	fHakka	 mien5
U+7CC6.0	fCantonese	min6
U+7CC6.0	fMandarin	mian4
U+7CC7.0	fUTF8	糇
U+7CC7.0	fCNS11643	3-4D73
U+7CC7.0	fGB	0-7457
U+7CC7.0	fR/S	 119.9
U+7CC7.0	fTotalStrokes	 15
U+7CC7.0	fCangjie	FDONK
U+7CC7.0	fFourCorner	97930
U+7CC7.0	fHakka	  heu2
U+7CC7.0	fCantonese	hau4
U+7CC7.0	fMandarin	hou2
U+7CC7.0	fEnglish	dried rice; dry provisions
U+7CC8.0	fUTF8	糈
U+7CC8.0	fCNS11643	2-5268
U+7CC8.0	fBig5	E6E0
U+7CC8.0	fGB	0-745A
U+7CC8.0	fR/S	 119.9
U+7CC8.0	fTotalStrokes	 15
U+7CC8.0	fCangjie	FDNOB
U+7CC8.0	fFourCorner	97927
U+7CC8.0	fHakka	  si1
U+7CC8.0	fCantonese	seoi2
U+7CC8.0	fMandarin	xu3; xu1; shu3; shu1
U+7CC8.0	fEnglish	 sacrificial rice rations pay
U+7CC9.0	fUTF8	糉
U+7CC9.0	fCNS11643	3-4D72
U+7CC9.0	fGB	E-3772
U+7CC9.0	fR/S	 119.9
U+7CC9.0	fTotalStrokes	 15
U+7CC9.0	fCangjie	FDUCE
U+7CC9.0	fFourCorner	92940
U+7CC9.0	fHakka	  zung5
U+7CC9.0	fCantonese	zung2; zung3 (also)
U+7CC9.0	fMandarin	zong4; zhong4
U+7CC9.0	fEnglish	dumpling made of glutinous rice
U+7CCA.0	fUTF8	糊
U+7CCA.0	fCNS11643	1-6E36
U+7CCA.0	fBig5	BD6B
U+7CCA.0	fGB	0-3A7D
U+7CCA.0	fR/S	 119.9
U+7CCA.0	fTotalStrokes	 15
U+7CCA.0	fCangjie	FDJRB
U+7CCA.0	fFourCorner	97920
U+7CCA.0	fHakka	  fu2; fu3
U+7CCA.0	fCantonese	wu4; wu2 (also)
U+7CCA.0	fMandarin	hu2; hu1; hu4
U+7CCA.0	fEnglish	[1] paste; stick on with paste [2] paste [3] scorched [4] not clear; blurred; confused; ambiguous; unintelligible
U+7CCA.1	fUTF8	糊
U+7CCA.1	fCNS11643	1-6E36
U+7CCA.1	fBig5	BD6B
U+7CCA.1	fGB	0-3A7D
U+7CCA.1	fR/S	 119.9
U+7CCA.1	fTotalStrokes	 15
U+7CCA.1	fCangjie	FDJRB
U+7CCA.1	fFourCorner	97920
U+7CCA.1	fMandarin	hu5
U+7CCB.0	fUTF8	糋
U+7CCB.0	fCNS11643	2-526A
U+7CCB.0	fBig5	E6E2
U+7CCB.0	fGB	5-667A
U+7CCB.0	fR/S	 119.9
U+7CCB.0	fTotalStrokes	 15
U+7CCB.0	fCangjie	FDTBN
U+7CCB.0	fFourCorner	98921
U+7CCB.0	fCantonese	zin3
U+7CCB.0	fMandarin	jian4
U+7CCC.0	fUTF8	糌
U+7CCC.0	fCNS11643	2-5269
U+7CCC.0	fBig5	E6E1
U+7CCC.0	fGB	0-7458
U+7CCC.0	fR/S	 119.9
U+7CCC.0	fTotalStrokes	 15
U+7CCC.0	fCangjie	FDHOA
U+7CCC.0	fFourCorner	98964
U+7CCC.0	fCantonese	zaam1; zaa1 (also); zam1
U+7CCC.0	fMandarin	zan2; zan1
U+7CCD.0	fUTF8	糍
U+7CCD.0	fCNS11643	3-4D6E
U+7CCD.0	fGB	0-7459
U+7CCD.0	fR/S	 119.9
U+7CCD.0	fTotalStrokes	 15
U+7CCD.0	fCangjie	FDTVI
U+7CCD.0	fFourCorner	98930
U+7CCD.0	fHakka	 ci2
U+7CCD.0	fCantonese	ci4
U+7CCD.0	fMandarin	ci2; zi1
U+7CCD.0	fEnglish	food made of crushed and cooked rice
U+7CCE.0	fUTF8	糎
U+7CCE.0	fCNS11643	1-2323
U+7CCE.0	fBig5	A261
U+7CCE.0	fGB	E-3773
U+7CCE.0	fR/S	 119.9
U+7CCE.0	fTotalStrokes	 15
U+7CCE.0	fCangjie	FDMWG
U+7CCE.0	fFourCorner	9191
U+7CCE.0	fCantonese	lei4
U+7CCE.0	fMandarin	li2
U+7CCE.0	fEnglish	mm
U+7CCF.0	fUTF8	糏
U+7CCF.0	fCNS11643	4-5244
U+7CCF.0	fGB	5-6722
U+7CCF.0	fR/S	 119.10
U+7CCF.0	fTotalStrokes	 16
U+7CCF.0	fCangjie	FDSFB
U+7CCF.0	fFourCorner	97920
U+7CCF.0	fHakka	 siet7
U+7CCF.0	fCantonese	sit3
U+7CCF.0	fMandarin	xie4; su4
U+7CCF.0	fEnglish	rice grits left after hulling
U+7CD0.0	fUTF8	糐
U+7CD0.0	fCNS11643	2-5936
U+7CD0.0	fBig5	EACA
U+7CD0.0	fGB	E-3774
U+7CD0.0	fR/S	 119.10
U+7CD0.0	fTotalStrokes	 16
U+7CD0.0	fCangjie	FDIBI
U+7CD0.0	fFourCorner	93942
U+7CD0.0	fCantonese	fu1
U+7CD0.0	fMandarin	fu1
U+7CD1.0	fUTF8	糑
U+7CD1.0	fCNS11643	2-5937
U+7CD1.0	fBig5	EACB
U+7CD1.0	fGB	3-6822
U+7CD1.0	fR/S	 119.10
U+7CD1.0	fTotalStrokes	 16
U+7CD1.0	fCangjie	FDNMM
U+7CD1.0	fFourCorner	97927
U+7CD1.0	fCantonese	nik1; lik1 (also); nik6; nok6
U+7CD1.0	fMandarin	ni4; nuo4
U+7CD2.0	fUTF8	糒
U+7CD2.0	fCNS11643	2-5933
U+7CD2.0	fBig5	EAC7
U+7CD2.0	fGB	3-6821
U+7CD2.0	fR/S	 119.10
U+7CD2.0	fTotalStrokes	 16
U+7CD2.0	fCangjie	FDTHB
U+7CD2.0	fFourCorner	94927
U+7CD2.0	fHakka	 pi5; pi6
U+7CD2.0	fCantonese	bei6
U+7CD2.0	fMandarin	bei4; bi4
U+7CD2.0	fEnglish	parched rice (as a dry food for a journey)
U+7CD3.0	fUTF8	糓
U+7CD3.0	fCNS11643	3-5237
U+7CD3.0	fGB	5-506E
U+7CD3.0	fR/S	 119.10; 119.12; 79.12
U+7CD3.0	fTotalStrokes	 16
U+7CD3.0	fCangjie	GDHNE
U+7CD3.0	fFourCorner	47940
U+7CD3.0	fHakka	 guk7; kwuk7
U+7CD3.0	fCantonese	guk1
U+7CD3.0	fMandarin	gu3; yu4
U+7CD3.0	fEnglish	grain; corn
U+7CD4.0	fUTF8	糔
U+7CD4.0	fCNS11643	2-5934
U+7CD4.0	fBig5	EAC8
U+7CD4.0	fGB	3-677D
U+7CD4.0	fR/S	 119.10
U+7CD4.0	fTotalStrokes	 16
U+7CD4.0	fCangjie	FDEII
U+7CD4.0	fFourCorner	97936
U+7CD4.0	fCantonese	sau1; sau2
U+7CD4.0	fMandarin	xiu3; xiu1; sao3; xiu4
U+7CD4.0	fEnglish	juice
U+7CD5.0	fUTF8	糕
U+7CD5.0	fCNS11643	1-7166
U+7CD5.0	fBig5	BF7C
U+7CD5.0	fGB	0-3862
U+7CD5.0	fR/S	 119.10
U+7CD5.0	fTotalStrokes	 16
U+7CD5.0	fCangjie	FDTGF
U+7CD5.0	fFourCorner	98931
U+7CD5.0	fHakka	  gau1; go1
U+7CD5.0	fCantonese	gou1
U+7CD5.0	fMandarin	gao1
U+7CD5.0	fEnglish	cakes; pastry; steamed dumplings
U+7CD6.0	fUTF8	糖
U+7CD6.0	fCNS11643	1-7167
U+7CD6.0	fBig5	BF7D
U+7CD6.0	fGB	0-4C47
U+7CD6.0	fR/S	 119.10
U+7CD6.0	fTotalStrokes	 16
U+7CD6.0	fCangjie	FDILR
U+7CD6.0	fFourCorner	90967
U+7CD6.0	fHakka	  tong2; tong3
U+7CD6.0	fCantonese	tong4; tong2 (also)
U+7CD6.0	fMandarin	tang2
U+7CD6.0	fEnglish	sugar; candy; sweets
U+7CD7.0	fUTF8	糗
U+7CD7.0	fCNS11643	2-5935
U+7CD7.0	fBig5	EAC9
U+7CD7.0	fGB	0-745C
U+7CD7.0	fR/S	 119.10
U+7CD7.0	fTotalStrokes	 16
U+7CD7.0	fCangjie	FDHUK
U+7CD7.0	fFourCorner	96934
U+7CD7.0	fHakka	 hiu3; chiu5; cu5; chu5
U+7CD7.0	fCantonese	cau3 (also); jau2; hau2; zau2
U+7CD7.0	fMandarin	qiu3; chao3; qiu4
U+7CD7.0	fEnglish	parched grain, wheat or rice; dry food; dry rations
U+7CD8.0	fUTF8	糘
U+7CD8.0	fGB	E-3775
U+7CD8.0	fR/S	 119.10
U+7CD8.0	fTotalStrokes	 16
U+7CD8.0	fCangjie	FDJMO
U+7CD8.0	fCantonese	gaa1
U+7CD9.0	fUTF8	糙
U+7CD9.0	fCNS11643	1-7461
U+7CD9.0	fBig5	C157
U+7CD9.0	fGB	0-325A
U+7CD9.0	fR/S	 119.11
U+7CD9.0	fTotalStrokes	 17
U+7CD9.0	fCangjie	FDYHR
U+7CD9.0	fFourCorner	94936
U+7CD9.0	fHakka	  cau5; co5
U+7CD9.0	fCantonese	cou3
U+7CD9.0	fMandarin	cao1; cao4
U+7CD9.0	fEnglish	[1] coarse; harsh; unpolished [2] rough; rude; rash; desultory; careless
U+7CDA.0	fUTF8	糚
U+7CDA.0	fCNS11643	3-5575
U+7CDA.0	fGB	E-3776
U+7CDA.0	fR/S	 119.11
U+7CDA.0	fTotalStrokes	 17
U+7CDA.0	fCangjie	FDTVG
U+7CDA.0	fFourCorner	94910
U+7CDA.0	fHakka	 zong1
U+7CDA.0	fCantonese	zong1
U+7CDA.0	fMandarin	zhuang1; zhuang4
U+7CDA.0	fEnglish	[1] adorn; paint [2] feign
U+7CDB.0	fUTF8	糛
U+7CDB.0	fCNS11643	3-5574
U+7CDB.0	fGB	5-6723
U+7CDB.0	fR/S	  119.11
U+7CDB.0	fTotalStrokes	 17
U+7CDB.0	fCangjie	FDFBG
U+7CDB.0	fFourCorner	99910
U+7CDB.0	fCantonese	tong4
U+7CDB.0	fMandarin	tang2
U+7CDC.0	fUTF8	糜
U+7CDC.0	fCNS11643	1-745D
U+7CDC.0	fBig5	C153
U+7CDC.0	fGB	0-4353
U+7CDC.0	fR/S	 119.11
U+7CDC.0	fTotalStrokes	 17
U+7CDC.0	fCangjie	IDFD
U+7CDC.0	fFourCorner	00294
U+7CDC.0	fHakka	 mi2;  moi2;  mien2; moi3; men2; fui1; moe2
U+7CDC.0	fCantonese	mei4; mai4
U+7CDC.0	fMandarin	mi2; mei2
U+7CDC.0	fEnglish	[1] rice gruel; congee; porridge [2] mashed; rotten; corrupted [3] waste
U+7CDD.0	fUTF8	糝
U+7CDD.0	fCNS11643	1-7462
U+7CDD.0	fBig5	C158
U+7CDD.0	fGB	1-7456
U+7CDD.0	fR/S	 119.11
U+7CDD.0	fTotalStrokes	 17
U+7CDD.0	fCangjie	FDIIH
U+7CDD.0	fFourCorner	93922
U+7CDD.0	fHakka	 sam3
U+7CDD.0	fCantonese	saam2 (also); sam2; sam3
U+7CDD.0	fMandarin	san3; san1; xin3
U+7CDD.0	fEnglish	[1] a grain of rice [2] mixing rice with broth
U+7CDE.0	fUTF8	糞
U+7CDE.0	fCNS11643	1-745E
U+7CDE.0	fBig5	C154
U+7CDE.0	fGB	1-3760
U+7CDE.0	fR/S	 119.11
U+7CDE.0	fTotalStrokes	 17
U+7CDE.0	fCangjie	FDWTC
U+7CDE.0	fFourCorner	90801
U+7CDE.0	fHakka	  bun5; mun3
U+7CDE.0	fCantonese	fan3
U+7CDE.0	fMandarin	fen4
U+7CDE.0	fEnglish	[1] manure; dung; night soil [2] fertilize the land [3] sweep; wipe out
U+7CDF.0	fUTF8	糟
U+7CDF.0	fCNS11643	1-7460
U+7CDF.0	fBig5	C156
U+7CDF.0	fGB	0-5463
U+7CDF.0	fR/S	 119.11
U+7CDF.0	fTotalStrokes	 17
U+7CDF.0	fCangjie	FDTWA
U+7CDF.0	fFourCorner	95966
U+7CDF.0	fHakka	  zau1;  zo1
U+7CDF.0	fCantonese	zou1
U+7CDF.0	fMandarin	zao1; zao4
U+7CDF.0	fEnglish	[1] sediment of wine; dregs of wine [2] soak food items (as fish, meat, etc.) in wine or wine sediment [3] become a mess (a plan, arrangement); become in bad shape [4] decayed; rotten; spoiled [5] not sturdy; not strong [6] lousy; louse
U+7CE0.0	fUTF8	糠
U+7CE0.0	fCNS11643	1-745C
U+7CE0.0	fBig5	C152
U+7CE0.0	fGB	0-3F37
U+7CE0.0	fR/S	 119.11
U+7CE0.0	fTotalStrokes	 17
U+7CE0.0	fCangjie	FDILE
U+7CE0.0	fFourCorner	90932
U+7CE0.0	fHakka	  hong1; kong1
U+7CE0.0	fCantonese	hong1
U+7CE0.0	fMandarin	kang1
U+7CE0.0	fEnglish	[1] rice chaff; rice bran; husk or rice [2] of inferior quality; not sturdy; empty inside; thing or no value
U+7CE1.0	fUTF8	糡
U+7CE1.0	fCNS11643	3-5239
U+7CE1.0	fGB	5-6727
U+7CE1.0	fR/S	  119.11
U+7CE1.0	fTotalStrokes	 17; 16
U+7CE1.0	fCangjie	FDYTU
U+7CE1.0	fFourCorner	90910
U+7CE1.0	fHakka	 kiong5; kiong6
U+7CE1.0	fCantonese	goeng6
U+7CE1.0	fMandarin	jiang4
U+7CE1.0	fEnglish	 starch paste; starch
U+7CE2.0	fUTF8	糢
U+7CE2.0	fCNS11643	1-745F
U+7CE2.0	fBig5	C155
U+7CE2.0	fGB	5-667C
U+7CE2.0	fR/S	 119.11
U+7CE2.0	fTotalStrokes	 17
U+7CE2.0	fCangjie	FDTAK
U+7CE2.0	fFourCorner	94934
U+7CE2.0	fCantonese	mou4
U+7CE2.0	fMandarin	mo2
U+7CE2.0	fEnglish	blurred; indistinct
U+7CE3.0	fUTF8	糣
U+7CE3.0	fCNS11643	3-5855
U+7CE3.0	fGB	3-6826
U+7CE3.0	fR/S	 119.12
U+7CE3.0	fTotalStrokes	 18
U+7CE3.0	fCangjie	FDMUA
U+7CE3.0	fFourCorner	91960
U+7CE3.0	fCantonese	sam2
U+7CE3.0	fMandarin	san3; san1
U+7CE4.0	fUTF8	糤
U+7CE4.0	fCNS11643	4-5C5A
U+7CE4.0	fGB	5-6729
U+7CE4.0	fR/S	 119.12
U+7CE4.0	fTotalStrokes	 18
U+7CE4.0	fCangjie	FDTBK
U+7CE4.0	fFourCorner	98940
U+7CE4.0	fHakka	  san3
U+7CE4.0	fCantonese	saan2
U+7CE4.0	fMandarin	san3
U+7CE4.0	fEnglish	fried round cakes of wheat flour
U+7CE5.0	fUTF8	糥
U+7CE5.0	fCNS11643	4-5C5B
U+7CE5.0	fGB	5-672A
U+7CE5.0	fR/S	 119.12
U+7CE5.0	fTotalStrokes	 18
U+7CE5.0	fCangjie	FDMBB
U+7CE5.0	fFourCorner	91920
U+7CE5.0	fCantonese	no6; lo6 (also)
U+7CE5.0	fMandarin	nuo4
U+7CE6.0	fUTF8	糦
U+7CE6.0	fCNS11643	3-5854
U+7CE6.0	fGB	5-6728
U+7CE6.0	fR/S	 119.12
U+7CE6.0	fTotalStrokes	 18
U+7CE6.0	fCangjie	FDGRR
U+7CE6.0	fFourCorner	94960
U+7CE6.0	fHakka	 zhi5; zih5; zhit7; tsiit7; tje5
U+7CE6.0	fCantonese	ci3
U+7CE6.0	fMandarin	xi1; chi4
U+7CE6.0	fEnglish	food; cook
U+7CE7.0	fUTF8	糧
U+7CE7.0	fCNS11643	1-767D
U+7CE7.0	fBig5	C2B3
U+7CE7.0	fGB	1-4138
U+7CE7.0	fR/S	 119.12
U+7CE7.0	fTotalStrokes	 18
U+7CE7.0	fCangjie	FDAMG
U+7CE7.0	fFourCorner	96914
U+7CE7.0	fHakka	  liong2
U+7CE7.0	fCantonese	loeng4
U+7CE7.0	fMandarin	liang2
U+7CE7.0	fEnglish	[1] food; grain; provisions [2] farm or land taxes
U+7CE8.0	fUTF8	糨
U+7CE8.0	fCNS11643	2-5E3C
U+7CE8.0	fBig5	EDCF
U+7CE8.0	fGB	0-745D
U+7CE8.0	fR/S	 119.11
U+7CE8.0	fTotalStrokes	 17
U+7CE8.0	fCangjie	FDNII
U+7CE8.0	fFourCorner	93936
U+7CE8.0	fHakka	 kiong5; kiong6
U+7CE8.0	fCantonese	goeng6; zoeng1
U+7CE8.0	fMandarin	jiang4; jiang1
U+7CE8.0	fEnglish	[1] starch; starched; paste; paste together [2] thick
U+7CE9.0	fUTF8	糩
U+7CE9.0	fCNS11643	7-4857
U+7CE9.0	fGB	E-3777
U+7CE9.0	fR/S	  119.13
U+7CE9.0	fTotalStrokes	 19
U+7CE9.0	fCangjie	FDOMA
U+7CE9.0	fFourCorner	98960
U+7CE9.0	fCantonese	kui2; kwui2 (also)
U+7CE9.0	fMandarin	kuai4
U+7CEA.0	fUTF8	糪
U+7CEA.0	fCNS11643	2-6638
U+7CEA.0	fBig5	F2AE
U+7CEA.0	fGB	3-682A
U+7CEA.0	fR/S	 119.13
U+7CEA.0	fTotalStrokes	 19
U+7CEA.0	fCangjie	SJFD
U+7CEA.0	fFourCorner	70904
U+7CEA.0	fCantonese	baak3; bok1
U+7CEA.0	fMandarin	bo2
U+7CEA.0	fEnglish	half-cooked rice; parboiled rice
U+7CEB.0	fUTF8	糫
U+7CEB.0	fCNS11643	4-605A
U+7CEB.0	fGB	3-6827
U+7CEB.0	fR/S	  119.13
U+7CEB.0	fTotalStrokes	 19
U+7CEB.0	fCangjie	FDWLV
U+7CEB.0	fFourCorner	96930
U+7CEB.0	fCantonese	waan4
U+7CEB.0	fMandarin	huan2
U+7CEC.0	fUTF8	糬
U+7CEC.0	fCNS11643	2-6637
U+7CEC.0	fBig5	F2AD
U+7CEC.0	fGB	E-3778
U+7CEC.0	fR/S	 119.13
U+7CEC.0	fTotalStrokes	 19
U+7CEC.0	fCangjie	FDWLA
U+7CEC.0	fFourCorner	96964
U+7CEC.0	fCantonese	syu4
U+7CEC.0	fMandarin	shu3
U+7CED.0	fUTF8	糭
U+7CED.0	fCNS11643	3-6628
U+7CED.0	fGB	E-3779
U+7CED.0	fR/S	 119.13
U+7CED.0	fTotalStrokes	 19
U+7CED.0	fCangjie	FDUCE
U+7CED.0	fFourCorner	9294
U+7CED.0	fCantonese	zung3
U+7CED.0	fMandarin	zong4
U+7CED.0	fEnglish	rice dumpling
U+7CEE.0	fUTF8	糮
U+7CEE.0	fCNS11643	2-6958
U+7CEE.0	fBig5	F4AB
U+7CEE.0	fGB	E-377A
U+7CEE.0	fR/S	 119.14
U+7CEE.0	fTotalStrokes	 20
U+7CEE.0	fCangjie	FDSIT
U+7CEE.0	fFourCorner	91917
U+7CEE.0	fCantonese	haam5; laam6
U+7CEE.0	fMandarin	jian4; xian4
U+7CEF.0	fUTF8	糯
U+7CEF.0	fCNS11643	1-7A27
U+7CEF.0	fBig5	C47A
U+7CEF.0	fGB	0-4534
U+7CEF.0	fR/S	 119.14
U+7CEF.0	fTotalStrokes	 20
U+7CEF.0	fCangjie	FDMBB
U+7CEF.0	fFourCorner	91927
U+7CEF.0	fHakka	 lo5;  no5; no6; lo3; lo6
U+7CEF.0	fCantonese	no6; lo6 (also); no2 (also); lo2 (also)
U+7CEF.0	fMandarin	nuo4; nuan4
U+7CEF.0	fEnglish	glutinous rice
U+7CF0.0	fUTF8	糰
U+7CF0.0	fCNS11643	1-7A28
U+7CF0.0	fBig5	C47B
U+7CF0.0	fGB	1-7870
U+7CF0.0	fR/S	 119.14
U+7CF0.0	fTotalStrokes	 20
U+7CF0.0	fCangjie	FDWJI
U+7CF0.0	fFourCorner	96900
U+7CF0.0	fHakka	  ton2
U+7CF0.0	fCantonese	tyun4
U+7CF0.0	fMandarin	tuan2
U+7CF0.0	fEnglish	round dumplings made from wheat or glutinous rice flour
U+7CF1.0	fUTF8	糱
U+7CF1.0	fCNS11643	2-6D6E
U+7CF1.0	fBig5	F741
U+7CF1.0	fGB	5-6731
U+7CF1.0	fR/S	 119.16
U+7CF1.0	fTotalStrokes	 22
U+7CF1.0	fCangjie	UJFD
U+7CF1.0	fFourCorner	17227
U+7CF1.0	fCantonese	jit6; jip6 (also)
U+7CF1.0	fMandarin	nie4
U+7CF1.0	fEnglish	 fermenting grain yeast
U+7CF2.0	fUTF8	糲
U+7CF2.0	fCNS11643	2-6B74
U+7CF2.0	fBig5	F5E6
U+7CF2.0	fGB	1-744F
U+7CF2.0	fR/S	 119.15
U+7CF2.0	fTotalStrokes	 21
U+7CF2.0	fCangjie	FDMTB
U+7CF2.0	fFourCorner	91927
U+7CF2.0	fHakka	 li5; li6
U+7CF2.0	fCantonese	lai6
U+7CF2.0	fMandarin	li4; la4; lai4
U+7CF2.0	fEnglish	[1] unpolished rice; brown rice [2] coarse; rough
U+7CF3.0	fUTF8	糳
U+7CF3.0	fCNS11643	3-6169
U+7CF3.0	fGB	5-6734
U+7CF3.0	fR/S	 119.16
U+7CF3.0	fTotalStrokes	 26; 22
U+7CF3.0	fCangjie	TEFD
U+7CF3.0	fFourCorner	27900
U+7CF3.0	fCantonese	zok6
U+7CF3.0	fMandarin	zuo4
U+7CF4.0	fUTF8	糴
U+7CF4.0	fCNS11643	2-6D6D
U+7CF4.0	fBig5	F740
U+7CF4.0	fGB	1-5961
U+7CF4.0	fR/S	 119.16
U+7CF4.0	fTotalStrokes	 22
U+7CF4.0	fCangjie	ODSMG
U+7CF4.0	fFourCorner	87914
U+7CF4.0	fHakka	  tak8; tet8
U+7CF4.0	fCantonese	dek6
U+7CF4.0	fMandarin	di2; zhuo2; cao2
U+7CF4.0	fEnglish	 buy grain; purchase grain
U+7CF5.0	fUTF8	糵
U+7CF5.0	fCNS11643	3-6040
U+7CF5.0	fGB	3-682B
U+7CF5.0	fR/S	 119.17
U+7CF5.0	fTotalStrokes	 23
U+7CF5.0	fCangjie	TJFD
U+7CF5.0	fFourCorner	44900
U+7CF5.0	fHakka	 ngiet7; nget7
U+7CF5.0	fCantonese	jip6 (also); jit6
U+7CF5.0	fMandarin	nie4
U+7CF5.0	fEnglish	yeast, leaven or barm for making liquors
U+7CF6.0	fUTF8	糶
U+7CF6.0	fCNS11643	2-712D
U+7CF6.0	fBig5	F8FD
U+7CF6.0	fGB	1-7450
U+7CF6.0	fR/S	 119.19
U+7CF6.0	fTotalStrokes	 25
U+7CF6.0	fCangjie	UDSMG
U+7CF6.0	fFourCorner	27914
U+7CF6.0	fHakka	  tiau5
U+7CF6.0	fCantonese	tiu3
U+7CF6.0	fMandarin	tiao4; diao4
U+7CF6.0	fEnglish	 sell grain
U+7CF7.0	fUTF8	糷
U+7CF7.0	fCNS11643	2-7172
U+7CF7.0	fBig5	F9A4
U+7CF7.0	fGB	E-377B
U+7CF7.0	fR/S	 119.21
U+7CF7.0	fTotalStrokes	 27
U+7CF7.0	fCangjie	FDTAW
U+7CF7.0	fFourCorner	94927
U+7CF7.0	fHakka	 lan6; lan5
U+7CF7.0	fCantonese	laan6; laan4
U+7CF7.0	fMandarin	lan2; lan4; lan3
U+7CF8.0	fUTF8	糸
U+7CF8.0	fCNS11643	1-484E
U+7CF8.0	fBig5	A6CD
U+7CF8.0	fGB	0-7469
U+7CF8.0	fR/S	 120.0
U+7CF8.0	fTotalStrokes	 6
U+7CF8.0	fCangjie	VIF
U+7CF8.0	fFourCorner	22903
U+7CF8.0	fHakka	 met7; ni5; mit8
U+7CF8.0	fCantonese	mik6; si1
U+7CF8.0	fMandarin	mi4; si1
U+7CF8.0	fEnglish	 silk Kangxi radical number 120
U+7CF9.0	fUTF8	糹
U+7CF9.0	fGB	1-6679
U+7CF9.0	fR/S	 120.0
U+7CF9.0	fTotalStrokes	 6
U+7CF9.0	fCangjie	VIF
U+7CF9.0	fCantonese	si1
U+7CF9.0	fMandarin	jiao3si1pang2; si1
U+7CF9.0	fEnglish	[1] silk [2] Kangxi radical 120
U+7CFA.0	fUTF8	糺
U+7CFA.0	fCNS11643	3-2722
U+7CFA.0	fGB	5-6B4D
U+7CFA.0	fR/S	 120.1
U+7CFA.0	fTotalStrokes	 7
U+7CFA.0	fCangjie	VFU
U+7CFA.0	fFourCorner	22910
U+7CFA.0	fHakka	  giu5
U+7CFA.0	fCantonese	gau2; dau2 (also); giu2
U+7CFA.0	fMandarin	jiu1; jiu3; jiao3
U+7CFA.0	fEnglish	 collaborate; band together
U+7CFB.0	fUTF8	系
U+7CFB.0	fCNS11643	1-4B37
U+7CFB.0	fBig5	A874
U+7CFB.0	fGB	0-4F35
U+7CFB.0	fR/S	 120.1
U+7CFB.0	fTotalStrokes	 7
U+7CFB.0	fCangjie	HVIF
U+7CFB.0	fFourCorner	20903
U+7CFB.0	fHakka	  he5;  hi5; ne5; gne5
U+7CFB.0	fCantonese	hai6; hai2 (also); gai3
U+7CFB.0	fMandarin	xi4
U+7CFB.0	fEnglish	[1] system; line; link; connection; connecting link [2] lineage; genealogy [3] clique (politics); theoretic or party line [4] department; school (college or university) [5] relate to; bear on [6] be [7] a Chinese family name [8] tie
U+7CFB.1	fUTF8	系
U+7CFB.1	fCNS11643	1-4B37
U+7CFB.1	fBig5	A874
U+7CFB.1	fGB	0-4F35
U+7CFB.1	fR/S	 120.1
U+7CFB.1	fTotalStrokes	 7
U+7CFB.1	fCangjie	HVIF
U+7CFB.1	fFourCorner	20903
U+7CFB.1	fMandarin	ji4
U+7CFB.2	fUTF8	系
U+7CFB.2	fCNS11643	1-4B37
U+7CFB.2	fBig5	A874
U+7CFB.2	fGB	0-4F35
U+7CFB.2	fR/S	 120.1
U+7CFB.2	fTotalStrokes	 7
U+7CFB.2	fCangjie	HVIF
U+7CFB.2	fFourCorner	20903
U+7CFB.2	fMandarin	xi5
U+7CFC.0	fUTF8	糼
U+7CFC.0	fCNS11643	3-2A4B
U+7CFC.0	fGB	5-6B4F
U+7CFC.0	fR/S	 120.2
U+7CFC.0	fTotalStrokes	 8
U+7CFC.0	fCangjie	VFKS
U+7CFC.0	fFourCorner	24920
U+7CFC.0	fHakka	 gung1
U+7CFC.0	fCantonese	gung1
U+7CFC.0	fMandarin	gong1
U+7CFD.0	fUTF8	糽
U+7CFD.0	fCNS11643	2-2849
U+7CFD.0	fBig5	CDA9
U+7CFD.0	fGB	5-6B4E
U+7CFD.0	fR/S	 120.2
U+7CFD.0	fTotalStrokes	 8
U+7CFD.0	fCangjie	VFMN
U+7CFD.0	fFourCorner	21920
U+7CFD.0	fCantonese	zang2; zang1
U+7CFD.0	fMandarin	zheng3
U+7CFE.0	fUTF8	糾
U+7CFE.0	fCNS11643	1-4F2B
U+7CFE.0	fBig5	AAC8
U+7CFE.0	fGB	1-3E40
U+7CFE.0	fR/S	 120.2
U+7CFE.0	fTotalStrokes	 8
U+7CFE.0	fCangjie	VFVL
U+7CFE.0	fFourCorner	22900
U+7CFE.0	fHakka	  giu5;  giu3; ziu1; giu1
U+7CFE.0	fCantonese	gau2; dau2 (also); giu2
U+7CFE.0	fMandarin	jiu1; jiu3; jiao3
U+7CFE.0	fEnglish	[1] supervise; investigate, inspect [2] correct; censure; impeach; discipline [3] collaborate; band together; entangle; involve; involved
U+7CFF.0	fUTF8	糿
U+7CFF.0	fCNS11643	3-2A4A
U+7CFF.0	fGB	E-377C
U+7CFF.0	fR/S	  120.2
U+7CFF.0	fTotalStrokes	 8
U+7CFF.0	fCangjie	VFSH
U+7CFF.0	fFourCorner	27920
U+7CFF.0	fMandarin	you4
U+7D00.0	fUTF8	紀
U+7D00.0	fCNS11643	1-5279
U+7D00.0	fBig5	ACF6
U+7D00.0	fGB	1-3C4D
U+7D00.0	fR/S	 120.3
U+7D00.0	fTotalStrokes	 9
U+7D00.0	fCangjie	VFSU
U+7D00.0	fFourCorner	27917
U+7D00.0	fHakka	  gi3; gi5; gi1
U+7D00.0	fCantonese	gei2; gei3 (also)
U+7D00.0	fMandarin	ji4; ji1; ji3
U+7D00.0	fEnglish	[1] historical record; annals; chronicles [2] period of 12 years [3] century [4] arrange; put in order [5] institutions; laws and regulations; discipline [6] age of a person [7] geological period [8] a Chinese family name
U+7D01.0	fUTF8	紁
U+7D01.0	fCNS11643	2-2C6D
U+7D01.0	fBig5	D04C
U+7D01.0	fGB	5-6B52
U+7D01.0	fR/S	 120.3
U+7D01.0	fTotalStrokes	 9
U+7D01.0	fCangjie	VFIK
U+7D01.0	fFourCorner	24900
U+7D01.0	fCantonese	caa3
U+7D01.0	fMandarin	cha4
U+7D02.0	fUTF8	紂
U+7D02.0	fCNS11643	1-5277
U+7D02.0	fBig5	ACF4
U+7D02.0	fGB	1-667B
U+7D02.0	fR/S	 120.3
U+7D02.0	fTotalStrokes	 9
U+7D02.0	fCangjie	VFDI
U+7D02.0	fFourCorner	24900
U+7D02.0	fHakka	 cu5;  chiu5; chiu6; ziu5; ciu5; chu5
U+7D02.0	fCantonese	zau6
U+7D02.0	fMandarin	zhou4
U+7D02.0	fEnglish	[1] the last emperor of the Yin dynasty [2] crupper of a saddle
U+7D03.0	fUTF8	紃
U+7D03.0	fCNS11643	2-2C6B
U+7D03.0	fBig5	D04A
U+7D03.0	fGB	3-6C32
U+7D03.0	fR/S	 120.3
U+7D03.0	fTotalStrokes	 9
U+7D03.0	fCangjie	VFLLL
U+7D03.0	fFourCorner	22900
U+7D03.0	fHakka	  sun2
U+7D03.0	fCantonese	ceon4; seon4
U+7D03.0	fMandarin	xun2; chun2; chuan1
U+7D03.0	fEnglish	 cord
U+7D04.0	fUTF8	約
U+7D04.0	fCNS11643	1-527C
U+7D04.0	fBig5	ACF9
U+7D04.0	fGB	1-543C
U+7D04.0	fR/S	 120.3
U+7D04.0	fTotalStrokes	 9
U+7D04.0	fCangjie	VFPI
U+7D04.0	fFourCorner	27920
U+7D04.0	fHakka	  jok7; rok7
U+7D04.0	fCantonese	joek3; wok6
U+7D04.0	fMandarin	yue1; yao4; di4; yao1; ao4; ji1
U+7D04.0	fEnglish	[1] treaty; agreement; covenant; contract [2] brief(ly); simply [3] about; around; approximately [4] date; appointment; engagement; rendezvous; make an appointment [5] poor; poverty; hardship; straitened [6] reduce (mathematics) [7] bind; restrain [8] vague(ly)
U+7D05.0	fUTF8	紅
U+7D05.0	fCNS11643	1-5278
U+7D05.0	fBig5	ACF5
U+7D05.0	fGB	1-3A6C
U+7D05.0	fR/S	 120.3
U+7D05.0	fTotalStrokes	 9
U+7D05.0	fCangjie	VFM
U+7D05.0	fFourCorner	21910
U+7D05.0	fHakka	  fung2; fung3
U+7D05.0	fCantonese	hung4; gung1; hung2 (also)
U+7D05.0	fMandarin	hong2; jiang4
U+7D05.0	fEnglish	[1] red; vermilion; rosy [2] blush; redden [3] eminent; influential (players); very popular [4] specially favored; favorite
U+7D05.1	fUTF8	紅
U+7D05.1	fCNS11643	1-5278
U+7D05.1	fBig5	ACF5
U+7D05.1	fGB	1-3A6C
U+7D05.1	fR/S	 120.3
U+7D05.1	fTotalStrokes	 9
U+7D05.1	fCangjie	VFM
U+7D05.1	fFourCorner	21910
U+7D05.1	fMandarin	gong1
U+7D05.1	fEnglish	work; working
U+7D06.0	fUTF8	紆
U+7D06.0	fCNS11643	1-527D
U+7D06.0	fBig5	ACFA
U+7D06.0	fGB	1-667A
U+7D06.0	fR/S	 120.3
U+7D06.0	fTotalStrokes	 9
U+7D06.0	fCangjie	VFMD
U+7D06.0	fFourCorner	21940
U+7D06.0	fHakka	  hi1; wu1; vu1; i1
U+7D06.0	fCantonese	jyu1
U+7D06.0	fMandarin	yu1; ou1; yu3; xu1
U+7D06.0	fEnglish	[1] bend; turn; twist; distort; wind; spiral; meander [2] a knot in one's heart; melancholy
U+7D07.0	fUTF8	紇
U+7D07.0	fCNS11643	1-527B
U+7D07.0	fBig5	ACF8
U+7D07.0	fGB	1-667C
U+7D07.0	fR/S	 120.3
U+7D07.0	fTotalStrokes	 9
U+7D07.0	fCangjie	VFON
U+7D07.0	fFourCorner	28917
U+7D07.0	fHakka	 ngiet7;  het8; het7; get7
U+7D07.0	fCantonese	hat6; gat1
U+7D07.0	fMandarin	he2; jie2; xie4; xie2; hu2
U+7D07.0	fEnglish	[1] inferior silk [2] name of a barbarian tribe, ancestors of the Huns
U+7D07.1	fUTF8	紇
U+7D07.1	fCNS11643	1-527B
U+7D07.1	fBig5	ACF8
U+7D07.1	fGB	1-667C
U+7D07.1	fR/S	 120.3
U+7D07.1	fTotalStrokes	 9
U+7D07.1	fCangjie	VFON
U+7D07.1	fFourCorner	28917
U+7D07.1	fMandarin	ge1
U+7D07.1	fEnglish	a knot made of a rope or string
U+7D08.0	fUTF8	紈
U+7D08.0	fCNS11643	2-2C6C
U+7D08.0	fBig5	D04B
U+7D08.0	fGB	1-667D
U+7D08.0	fR/S	 120.3
U+7D08.0	fTotalStrokes	 9
U+7D08.0	fCangjie	VFKNI
U+7D08.0	fFourCorner	25917
U+7D08.0	fHakka	  fan2; wan2; jen2; jan2
U+7D08.0	fCantonese	jyun4
U+7D08.0	fMandarin	wan2
U+7D08.0	fEnglish	processed fine and light silk
U+7D09.0	fUTF8	紉
U+7D09.0	fCNS11643	1-527A
U+7D09.0	fBig5	ACF7
U+7D09.0	fGB	1-4852
U+7D09.0	fR/S	 120.3
U+7D09.0	fTotalStrokes	 9
U+7D09.0	fCangjie	VFSHI
U+7D09.0	fFourCorner	27920
U+7D09.0	fHakka	  ngiun3;  ngin2; ngim3; niun5; niun2; ngiun5; nun3; ngiun1
U+7D09.0	fCantonese	jan6; jan4
U+7D09.0	fMandarin	ren4; nin2; ren2; jin4
U+7D09.0	fEnglish	[1] thread a needle [2] sew; stitch [3] tie; wear [4] feel deeply
U+7D0A.0	fUTF8	紊
U+7D0A.0	fCNS11643	1-5742
U+7D0A.0	fBig5	AFBF
U+7D0A.0	fGB	0-4E49
U+7D0A.0	fR/S	 120.4
U+7D0A.0	fTotalStrokes	 10
U+7D0A.0	fCangjie	YKVIF
U+7D0A.0	fFourCorner	00903
U+7D0A.0	fHakka	  fun3;  fun5;  wun3; min5; mun5; fun6; wun5
U+7D0A.0	fCantonese	man6; man5 (also); leon6
U+7D0A.0	fMandarin	wen3; wen4; obs; wen2
U+7D0A.0	fEnglish	confused; disorder
U+7D0B.0	fUTF8	紋
U+7D0B.0	fCNS11643	1-5741
U+7D0B.0	fBig5	AFBE
U+7D0B.0	fGB	1-4E46
U+7D0B.0	fR/S	 120.4
U+7D0B.0	fTotalStrokes	 10
U+7D0B.0	fCangjie	VFYK
U+7D0B.0	fFourCorner	20940
U+7D0B.0	fHakka	 wun2; wun3; vun2
U+7D0B.0	fCantonese	man4; man2 (also); man6
U+7D0B.0	fMandarin	wen2; wen4
U+7D0B.0	fEnglish	[1] line; streak; stripe; vein [2] ripples (water) [3] prints (finger) [4] wrinkles (face) [5] tattoo
U+7D0C.0	fUTF8	紌
U+7D0C.0	fCNS11643	2-317C
U+7D0C.0	fBig5	D35A
U+7D0C.0	fGB	5-6B57
U+7D0C.0	fR/S	 120.4
U+7D0C.0	fTotalStrokes	 10
U+7D0C.0	fCangjie	VFIKU
U+7D0C.0	fFourCorner	23914
U+7D0C.0	fCantonese	kau4
U+7D0C.0	fMandarin	qiu2
U+7D0D.0	fUTF8	納
U+7D0D.0	fCNS11643	1-574A
U+7D0D.0	fBig5	AFC7
U+7D0D.0	fGB	1-4449
U+7D0D.0	fR/S	 120.4
U+7D0D.0	fTotalStrokes	 10
U+7D0D.0	fCangjie	VFOB
U+7D0D.0	fFourCorner	24927
U+7D0D.0	fHakka	 nap8; lap7; lap8
U+7D0D.0	fCantonese	naap6; laap6 (also)
U+7D0D.0	fMandarin	na4
U+7D0D.0	fEnglish	[1] admit; take; receive; accept; adopt [2] offer as tribute [3] enjoy; feel [4] repress; restrain [5] patch old clothes [6] a Chinese family name
U+7D0E.0	fUTF8	紎
U+7D0E.0	fCNS11643	2-3175
U+7D0E.0	fBig5	D353
U+7D0E.0	fGB	5-6B55
U+7D0E.0	fR/S	 120.4
U+7D0E.0	fTotalStrokes	 10
U+7D0E.0	fCangjie	VFIK
U+7D0E.0	fFourCorner	23934
U+7D0E.0	fCantonese	zi1
U+7D0E.0	fMandarin	zi1
U+7D0F.0	fUTF8	紏
U+7D0F.0	fCNS11643	2-317B
U+7D0F.0	fBig5	D359
U+7D0F.0	fGB	5-6B5C
U+7D0F.0	fR/S	 120.4
U+7D0F.0	fTotalStrokes	 10
U+7D0F.0	fCangjie	VFYJ
U+7D0F.0	fFourCorner	24900
U+7D0F.0	fHakka	  deu3; giu5
U+7D0F.0	fCantonese	tau2; dau2
U+7D0F.0	fMandarin	tou3
U+7D0F.0	fEnglish	the colour of yellow silk
U+7D10.0	fUTF8	紐
U+7D10.0	fCNS11643	1-5746
U+7D10.0	fBig5	AFC3
U+7D10.0	fGB	1-4526
U+7D10.0	fR/S	 120.4
U+7D10.0	fTotalStrokes	 10
U+7D10.0	fCangjie	VFNG
U+7D10.0	fFourCorner	27915
U+7D10.0	fHakka	  ngiu3; neu3; ngeu3; gniu5; niu3; {neu3}
U+7D10.0	fCantonese	nau2; lau2 (also); nau5
U+7D10.0	fMandarin	niu3
U+7D10.0	fEnglish	[1] knot; tie; cord [2] button; knob; [3] hold; handle (of a container)
U+7D11.0	fUTF8	紑
U+7D11.0	fCNS11643	2-3174
U+7D11.0	fBig5	D352
U+7D11.0	fGB	3-6C35
U+7D11.0	fR/S	 120.4
U+7D11.0	fTotalStrokes	 10
U+7D11.0	fCangjie	VFMF
U+7D11.0	fFourCorner	21990
U+7D11.0	fHakka	  feu2
U+7D11.0	fCantonese	fau4; fau1; fau2
U+7D11.0	fMandarin	fou2; fou1; fu1; fu2; fu4; pei2
U+7D11.0	fEnglish	fresh
U+7D12.0	fUTF8	紒
U+7D12.0	fCNS11643	2-317A
U+7D12.0	fBig5	D358
U+7D12.0	fGB	3-6C3A
U+7D12.0	fR/S	 120.4
U+7D12.0	fTotalStrokes	 10
U+7D12.0	fCangjie	VFOLL
U+7D12.0	fFourCorner	28920
U+7D12.0	fCantonese	gai3
U+7D12.0	fMandarin	jie4; ji4
U+7D13.0	fUTF8	紓
U+7D13.0	fCNS11643	2-3178
U+7D13.0	fBig5	D356
U+7D13.0	fGB	1-6723
U+7D13.0	fR/S	 120.4
U+7D13.0	fTotalStrokes	 10
U+7D13.0	fCangjie	VFNIN
U+7D13.0	fFourCorner	27922
U+7D13.0	fHakka	 shu1; su1
U+7D13.0	fCantonese	syu1
U+7D13.0	fMandarin	shu1; shu4
U+7D13.0	fEnglish	[1] loosen; relax; slacken; slow down [2] free from; remove from; relieve; extricate from
U+7D14.0	fUTF8	純
U+7D14.0	fCNS11643	1-5745
U+7D14.0	fBig5	AFC2
U+7D14.0	fGB	1-343F
U+7D14.0	fR/S	 120.4
U+7D14.0	fTotalStrokes	 10
U+7D14.0	fCangjie	VFPU
U+7D14.0	fFourCorner	25917
U+7D14.0	fHakka	  sun2;  shun2; shun3
U+7D14.0	fCantonese	seon4; zeon2; tyun4
U+7D14.0	fMandarin	chun2; quan2; tun2; zhun3; dun4; jun1; zhun1; zi1; zhun4
U+7D14.0	fEnglish	[1] pure; clean; net (profits, etc.) [2] sincere; honest; simple; faithful [3] completely; purely; entirely [4] great; large [5] silk; silk strands or threads
U+7D15.0	fUTF8	紕
U+7D15.0	fCNS11643	1-5747
U+7D15.0	fBig5	AFC4
U+7D15.0	fGB	1-6722
U+7D15.0	fR/S	 120.4
U+7D15.0	fTotalStrokes	 10
U+7D15.0	fCangjie	VFPP
U+7D15.0	fFourCorner	21910
U+7D15.0	fHakka	 pi1;  bi3; pi2
U+7D15.0	fCantonese	pei1; pei4; pai1
U+7D15.0	fMandarin	pi1; bi3; bi4; bei1; bei4; bi1; pian2; chi3
U+7D15.0	fEnglish	[1] error; mistake; blunder [2] become unwoven or untwisted (cloth, thread, etc.)
U+7D15.1	fUTF8	紕
U+7D15.1	fCNS11643	1-5747
U+7D15.1	fBig5	AFC4
U+7D15.1	fGB	1-6722
U+7D15.1	fR/S	 120.4
U+7D15.1	fTotalStrokes	 10
U+7D15.1	fCangjie	VFPP
U+7D15.1	fFourCorner	21910
U+7D15.1	fMandarin	pi2
U+7D15.1	fEnglish	hem ornaments
U+7D16.0	fUTF8	紖
U+7D16.0	fCNS11643	2-3177
U+7D16.0	fBig5	D355
U+7D16.0	fGB	3-6C3F
U+7D16.0	fR/S	 120.4
U+7D16.0	fTotalStrokes	 10
U+7D16.0	fCangjie	VFNL
U+7D16.0	fFourCorner	22900
U+7D16.0	fHakka	 jin3; rin3
U+7D16.0	fCantonese	zan3 (also); zan5; zan6; can5; jan5
U+7D16.0	fMandarin	yin3; zhen4
U+7D16.0	fEnglish	rope for leading cattle or horse
U+7D17.0	fUTF8	紗
U+7D17.0	fCNS11643	1-5740
U+7D17.0	fBig5	AFBD
U+7D17.0	fGB	1-4934
U+7D17.0	fR/S	 120.4
U+7D17.0	fTotalStrokes	 10
U+7D17.0	fCangjie	VFFH
U+7D17.0	fFourCorner	29920
U+7D17.0	fHakka	  sa1
U+7D17.0	fCantonese	saa1
U+7D17.0	fMandarin	sha1; miao3
U+7D17.0	fEnglish	[1] gauze; thin silk or cloth [2] yarn [3] a thin, netty or meshy sheet of any material
U+7D18.0	fUTF8	紘
U+7D18.0	fCNS11643	2-3176
U+7D18.0	fBig5	D354
U+7D18.0	fGB	3-6C36
U+7D18.0	fR/S	 120.4
U+7D18.0	fTotalStrokes	 10
U+7D18.0	fCangjie	VFKI
U+7D18.0	fFourCorner	24932
U+7D18.0	fHakka	  fen2
U+7D18.0	fCantonese	wang4
U+7D18.0	fMandarin	hong2
U+7D18.0	fEnglish	[1] a hat string fastened under the chin [2] vast; spacious; expansive
U+7D19.0	fUTF8	紙
U+7D19.0	fCNS11643	1-574B
U+7D19.0	fBig5	AFC8
U+7D19.0	fGB	1-563D
U+7D19.0	fR/S	 120.4
U+7D19.0	fTotalStrokes	 10
U+7D19.0	fCangjie	VFHVP
U+7D19.0	fFourCorner	22940
U+7D19.0	fHakka	 zi3;  zhi3; zih3; tje3
U+7D19.0	fCantonese	zi2
U+7D19.0	fMandarin	zhi3
U+7D19.0	fEnglish	paper
U+7D1A.0	fUTF8	級
U+7D1A.0	fCNS11643	1-5748
U+7D1A.0	fBig5	AFC5
U+7D1A.0	fGB	1-3C36
U+7D1A.0	fR/S	 120.4
U+7D1A.0	fTotalStrokes	 10
U+7D1A.0	fCangjie	VFNHE
U+7D1A.0	fFourCorner	27947
U+7D1A.0	fHakka	  kip7; kip8
U+7D1A.0	fCantonese	kap1
U+7D1A.0	fMandarin	ji2
U+7D1A.0	fEnglish	[1] level; degree; rank [2] class; grade [3] step (flight of steps) [4] decapitated head
U+7D1B.0	fUTF8	紛
U+7D1B.0	fCNS11643	1-574C
U+7D1B.0	fBig5	AFC9
U+7D1B.0	fGB	1-3757
U+7D1B.0	fR/S	 120.4
U+7D1B.0	fTotalStrokes	 10
U+7D1B.0	fCangjie	VFCSH
U+7D1B.0	fFourCorner	28927
U+7D1B.0	fHakka	  fun1; fun5
U+7D1B.0	fCantonese	fan1
U+7D1B.0	fMandarin	fen1; fen2
U+7D1B.0	fEnglish	[1] confused; in disorder; disorderly [2] numerous; many; varied
U+7D1C.0	fUTF8	紜
U+7D1C.0	fCNS11643	1-5749
U+7D1C.0	fBig5	AFC6
U+7D1C.0	fGB	1-6721
U+7D1C.0	fR/S	 120.4
U+7D1C.0	fTotalStrokes	 10
U+7D1C.0	fCangjie	VFMMI
U+7D1C.0	fFourCorner	21931
U+7D1C.0	fHakka	  jun2; run2; jun3
U+7D1C.0	fCantonese	wan4
U+7D1C.0	fMandarin	yun2
U+7D1C.0	fEnglish	confusing; disorderly
U+7D1D.0	fUTF8	紝
U+7D1D.0	fCNS11643	2-3173
U+7D1D.0	fBig5	D351
U+7D1D.0	fGB	3-6C39
U+7D1D.0	fR/S	 120.4
U+7D1D.0	fTotalStrokes	 10
U+7D1D.0	fCangjie	VFHG
U+7D1D.0	fFourCorner	22914
U+7D1D.0	fHakka	 ngim6; ngim5; ngim3
U+7D1D.0	fCantonese	jam6; jam4 (also)
U+7D1D.0	fMandarin	ren4; ren2
U+7D1D.0	fEnglish	 weave; lay the warp
U+7D1E.0	fUTF8	紞
U+7D1E.0	fCNS11643	2-3172
U+7D1E.0	fBig5	D350
U+7D1E.0	fGB	3-6C3E
U+7D1E.0	fR/S	 120.4
U+7D1E.0	fTotalStrokes	 10
U+7D1E.0	fCangjie	VFLBU
U+7D1E.0	fFourCorner	24912
U+7D1E.0	fHakka	  dam3
U+7D1E.0	fCantonese	daam2 (also); daam5; dam2; dam3; daam1
U+7D1E.0	fMandarin	dan3
U+7D1E.0	fEnglish	a fringe
U+7D1F.0	fUTF8	紟
U+7D1F.0	fCNS11643	2-3179
U+7D1F.0	fBig5	D357
U+7D1F.0	fGB	3-6C3B
U+7D1F.0	fR/S	 120.4
U+7D1F.0	fTotalStrokes	 10
U+7D1F.0	fCangjie	VFOIN
U+7D1F.0	fFourCorner	28927
U+7D1F.0	fHakka	 gim1; kim1
U+7D1F.0	fCantonese	gam1; gam3; kam1
U+7D1F.0	fMandarin	jin1; jin4; qian2
U+7D1F.0	fEnglish	 sash tie
U+7D20.0	fUTF8	素
U+7D20.0	fCNS11643	1-5743
U+7D20.0	fBig5	AFC0
U+7D20.0	fGB	0-4B58
U+7D20.0	fR/S	 120.4
U+7D20.0	fTotalStrokes	 10
U+7D20.0	fCangjie	QMVIF
U+7D20.0	fFourCorner	50903
U+7D20.0	fHakka	 su5; sii5; se5
U+7D20.0	fCantonese	sou3
U+7D20.0	fMandarin	su4
U+7D20.0	fEnglish	[1] pure white silk [2] white (color) [3] plain; simple [4] mourning [5] vegetarian; vegetable food; vegetarian diet [6] heretofore; up to the present [7] usually; generally; normally [8] the original constitution of things; matter; elements
U+7D21.0	fUTF8	紡
U+7D21.0	fCNS11643	1-573F
U+7D21.0	fBig5	AFBC
U+7D21.0	fGB	1-3744
U+7D21.0	fR/S	 120.4
U+7D21.0	fTotalStrokes	 10
U+7D21.0	fCangjie	VFYHS
U+7D21.0	fFourCorner	20927
U+7D21.0	fHakka	  fong3;  p'iong3; {p'iong3}; piong3
U+7D21.0	fCantonese	fong2
U+7D21.0	fMandarin	fang3; bang3; fang4
U+7D21.0	fEnglish	[1] spin; reel; weave [2] reeled pongee (a kind of thin silk)
U+7D22.0	fUTF8	索
U+7D22.0	fCNS11643	1-5744
U+7D22.0	fBig5	AFC1
U+7D22.0	fGB	0-4B77
U+7D22.0	fR/S	 120.4
U+7D22.0	fTotalStrokes	 10
U+7D22.0	fCangjie	JBVIF
U+7D22.0	fFourCorner	40903
U+7D22.0	fHakka	  sok7; so1
U+7D22.0	fCantonese	sok3; saak3; suk3 (also)
U+7D22.0	fMandarin	suo3; sai4; su4
U+7D22.0	fEnglish	[1] thick rope; cable [2] alone; lonely; solitary [3] rules; laws; laws and regulations [4] demand; ask [5] search into; inquire into [6] exhaust [7] tighten; squeeze; twist [8] need [9] a Chinese family name
U+7D22.1	fUTF8	索
U+7D22.1	fCNS11643	1-5744
U+7D22.1	fBig5	AFC1
U+7D22.1	fGB	0-4B77
U+7D22.1	fR/S	 120.4
U+7D22.1	fTotalStrokes	 10
U+7D22.1	fCangjie	JBVIF
U+7D22.1	fFourCorner	40903
U+7D22.1	fMandarin	suo2
U+7D22.1	fEnglish	 decide to go ahead and do something without any more consideration; may as well; simply
U+7D22.2	fUTF8	索
U+7D22.2	fCNS11643	1-5744
U+7D22.2	fBig5	AFC1
U+7D22.2	fGB	0-4B77
U+7D22.2	fR/S	 120.4
U+7D22.2	fTotalStrokes	 10
U+7D22.2	fCangjie	JBVIF
U+7D22.2	fFourCorner	40903
U+7D22.2	fMandarin	suo5
U+7D23.0	fUTF8	紣
U+7D23.0	fCNS11643	4-2F25
U+7D23.0	fGB	E-377D
U+7D23.0	fR/S	 120.4
U+7D23.0	fTotalStrokes	 10
U+7D23.0	fCangjie	VFKNJ
U+7D23.0	fFourCorner	24940
U+7D23.0	fCantonese	ceoi3
U+7D23.0	fMandarin	cui4; zu2
U+7D24.0	fUTF8	紤
U+7D24.0	fCNS11643	3-3337
U+7D24.0	fGB	E-377E
U+7D24.0	fR/S	 120.4
U+7D24.0	fTotalStrokes	 10
U+7D24.0	fCangjie	VFHML
U+7D24.0	fFourCorner	22920
U+7D24.0	fCantonese	gau2
U+7D24.0	fMandarin	jiu3
U+7D25.0	fUTF8	紥
U+7D25.0	fCNS11643	3-333B
U+7D25.0	fGB	E-3821
U+7D25.0	fR/S	 120.4
U+7D25.0	fTotalStrokes	 10
U+7D25.0	fCangjie	QUVIF
U+7D25.0	fFourCorner	52900
U+7D25.0	fHakka	 zat7
U+7D25.0	fCantonese	zaat3
U+7D25.0	fMandarin	zha2; za1
U+7D25.0	fEnglish	tie, fasten, bind
U+7D26.0	fUTF8	紦
U+7D26.0	fGB	E-3822
U+7D26.0	fR/S	 120.4
U+7D26.0	fTotalStrokes	 10
U+7D26.0	fCangjie	VFAU
U+7D26.0	fCantonese	baa1
U+7D26.0	fMandarin	ba1
U+7D27.0	fUTF8	紧
U+7D27.0	fGB	0-3D74
U+7D27.0	fR/S	 120.4
U+7D27.0	fTotalStrokes	 10
U+7D27.0	fCangjie	LEVIF
U+7D27.0	fHakka	 gin3
U+7D27.0	fCantonese	gan2
U+7D27.0	fMandarin	jin3
U+7D27.0	fEnglish	simplified form of &#x7DCA;
U+7D28.0	fUTF8	紨
U+7D28.0	fCNS11643	2-3835
U+7D28.0	fBig5	D6F0
U+7D28.0	fGB	3-6C48
U+7D28.0	fR/S	 120.5
U+7D28.0	fTotalStrokes	 11
U+7D28.0	fCangjie	VFODI
U+7D28.0	fFourCorner	24900
U+7D28.0	fCantonese	fu1; fu6
U+7D28.0	fMandarin	fu4; fu1; fu2
U+7D29.0	fUTF8	紩
U+7D29.0	fCNS11643	2-382E
U+7D29.0	fBig5	D6E9
U+7D29.0	fGB	3-6C45
U+7D29.0	fR/S	 120.5
U+7D29.0	fTotalStrokes	 11
U+7D29.0	fCangjie	VFHQO
U+7D29.0	fFourCorner	25930
U+7D29.0	fHakka	 zhit7; tsiit7
U+7D29.0	fCantonese	dit6; zat6
U+7D29.0	fMandarin	zhi4; chi1
U+7D29.0	fEnglish	 sew
U+7D2A.0	fUTF8	紪
U+7D2A.0	fCNS11643	4-3A26
U+7D2A.0	fGB	E-3823
U+7D2A.0	fR/S	 120.5; 120.6
U+7D2A.0	fTotalStrokes	 12; 11
U+7D2A.0	fCangjie	VFYMP
U+7D2A.0	fFourCorner	21910
U+7D2A.0	fCantonese	cai1
U+7D2A.0	fMandarin	ci3; qi1
U+7D2B.0	fUTF8	紫
U+7D2B.0	fCNS11643	1-613B
U+7D2B.0	fBig5	B5B5
U+7D2B.0	fGB	0-574F
U+7D2B.0	fR/S	 120.5
U+7D2B.0	fTotalStrokes	 11
U+7D2B.0	fCangjie	YPVIF
U+7D2B.0	fFourCorner	21903
U+7D2B.0	fHakka	  zih3;  zu3; zii3
U+7D2B.0	fCantonese	zi2
U+7D2B.0	fMandarin	zi3
U+7D2B.0	fEnglish	[1] purple; violet; amethyst [2] a Chinese family name
U+7D2C.0	fUTF8	紬
U+7D2C.0	fCNS11643	2-382D
U+7D2C.0	fBig5	D6E8
U+7D2C.0	fGB	3-6C44
U+7D2C.0	fR/S	 120.5
U+7D2C.0	fTotalStrokes	 11
U+7D2C.0	fCangjie	VFLW
U+7D2C.0	fFourCorner	25960
U+7D2C.0	fHakka	 chiu2; cu2; chu2
U+7D2C.0	fCantonese	cau4; cau1
U+7D2C.0	fMandarin	chou2; qu2; xiu4; zhou4
U+7D2C.0	fEnglish	a kind of thin silk; a kind of silk fabric
U+7D2C.1	fUTF8	紬
U+7D2C.1	fCNS11643	2-382D
U+7D2C.1	fBig5	D6E8
U+7D2C.1	fGB	3-6C44
U+7D2C.1	fR/S	 120.5
U+7D2C.1	fTotalStrokes	 11
U+7D2C.1	fCangjie	VFLW
U+7D2C.1	fFourCorner	25960
U+7D2C.1	fMandarin	chou1
U+7D2C.1	fEnglish	[1] draw out [2] collect and edit
U+7D2D.0	fUTF8	紭
U+7D2D.0	fCNS11643	3-3829
U+7D2D.0	fGB	5-6B6A
U+7D2D.0	fR/S	  120.5
U+7D2D.0	fTotalStrokes	 11
U+7D2D.0	fCangjie	VFNI
U+7D2D.0	fFourCorner	22330
U+7D2D.0	fHakka	  fen2
U+7D2D.0	fCantonese	wang4
U+7D2D.0	fMandarin	hong2
U+7D2E.0	fUTF8	紮
U+7D2E.0	fCNS11643	1-5C54
U+7D2E.0	fBig5	B2CF
U+7D2E.0	fGB	E-3824
U+7D2E.0	fR/S	 120.5
U+7D2E.0	fTotalStrokes	 11
U+7D2E.0	fCangjie	DUVIF
U+7D2E.0	fFourCorner	42903
U+7D2E.0	fHakka	  zat7;  zap7
U+7D2E.0	fCantonese	zaat3; zaap3
U+7D2E.0	fMandarin	zha2; zha1
U+7D2E.0	fEnglish	[1] tie; fasten; bind; make a bundle; bundle [2] stop; station; post
U+7D2E.1	fUTF8	紮
U+7D2E.1	fCNS11643	1-5C54
U+7D2E.1	fBig5	B2CF
U+7D2E.1	fGB	E-3824
U+7D2E.1	fR/S	 120.5
U+7D2E.1	fTotalStrokes	 11
U+7D2E.1	fCangjie	DUVIF
U+7D2E.1	fFourCorner	42903
U+7D2E.1	fMandarin	za1
U+7D2E.1	fEnglish	 tie; bind; fasten
U+7D2F.0	fUTF8	累
U+7D2F.0	fCNS11643	1-5C5B
U+7D2F.0	fBig5	B2D6
U+7D2F.0	fGB	0-405B
U+7D2F.0	fR/S	 120.5
U+7D2F.0	fTotalStrokes	 11
U+7D2F.0	fCangjie	WVIF
U+7D2F.0	fFourCorner	60903
U+7D2F.0	fHakka	 lui1; lui6; lui3
U+7D2F.0	fCantonese	leoi6; leoi5; leoi4
U+7D2F.0	fMandarin	lei3; lie4; luo3; l4
U+7D2F.0	fEnglish	[1] accumulate through a length of time [2] pile up [3] repeat; repeatedly; successively
U+7D2F.1	fUTF8	累
U+7D2F.1	fCNS11643	1-5C5B
U+7D2F.1	fBig5	B2D6
U+7D2F.1	fGB	0-405B
U+7D2F.1	fR/S	 120.5
U+7D2F.1	fTotalStrokes	 11
U+7D2F.1	fCangjie	WVIF
U+7D2F.1	fFourCorner	60903
U+7D2F.1	fHakka	 lui5
U+7D2F.1	fMandarin	lei4
U+7D2F.1	fEnglish	[1] involve; involvement; implicate [2] owe; be in debt [3] tired; weary; fatigue [4] strain (eyes) [5] family burden
U+7D2F.2	fUTF8	累
U+7D2F.2	fCNS11643	1-5C5B
U+7D2F.2	fBig5	B2D6
U+7D2F.2	fGB	0-405B
U+7D2F.2	fR/S	 120.5
U+7D2F.2	fTotalStrokes	 11
U+7D2F.2	fCangjie	WVIF
U+7D2F.2	fFourCorner	60903
U+7D2F.2	fMandarin	lei2
U+7D2F.2	fEnglish	[1] nuisance [2] fasten; bind
U+7D30.0	fUTF8	細
U+7D30.0	fCNS11643	1-5C58
U+7D30.0	fBig5	B2D3
U+7D30.0	fGB	1-4F38
U+7D30.0	fR/S	 120.5
U+7D30.0	fTotalStrokes	 11
U+7D30.0	fCangjie	VFW
U+7D30.0	fFourCorner	26900
U+7D30.0	fHakka	  se5
U+7D30.0	fCantonese	sai3; sai2 (also)
U+7D30.0	fMandarin	xi4
U+7D30.0	fEnglish	[1] fine [2] tiny; small; little [3] slender; thin; tall but lean; slim [4] petty; trifling; detailed [5] precise; exquisite; delicate (workmanship, etc.)
U+7D31.0	fUTF8	紱
U+7D31.0	fCNS11643	1-5C5E
U+7D31.0	fBig5	B2D9
U+7D31.0	fGB	1-6726
U+7D31.0	fR/S	 120.5
U+7D31.0	fTotalStrokes	 11
U+7D31.0	fCangjie	VFIKK
U+7D31.0	fFourCorner	23944
U+7D31.0	fHakka	  fut7
U+7D31.0	fCantonese	fat1
U+7D31.0	fMandarin	fu2
U+7D31.0	fEnglish	[1] ribbon; strand; sashes [2] ceremonial dress worn during sacrificial rituals
U+7D32.0	fUTF8	紲
U+7D32.0	fCNS11643	1-5C5D
U+7D32.0	fBig5	B2D8
U+7D32.0	fGB	1-6725
U+7D32.0	fR/S	 120.5
U+7D32.0	fTotalStrokes	 11
U+7D32.0	fCangjie	VFPT
U+7D32.0	fFourCorner	24917
U+7D32.0	fHakka	  siet7
U+7D32.0	fCantonese	sit3; jai6
U+7D32.0	fMandarin	xie4; yi4
U+7D32.0	fEnglish	 bridle halter
U+7D33.0	fUTF8	紳
U+7D33.0	fCNS11643	1-5C59
U+7D33.0	fBig5	B2D4
U+7D33.0	fGB	1-4970
U+7D33.0	fR/S	 120.5
U+7D33.0	fTotalStrokes	 11
U+7D33.0	fCangjie	VFLWL
U+7D33.0	fFourCorner	25906
U+7D33.0	fHakka	  shin1; sin1; siin1
U+7D33.0	fCantonese	san1
U+7D33.0	fMandarin	shen1
U+7D33.0	fEnglish	[1] the middle class as a group or individuals; gentry; gentleman [2] sash; girdle [3] tie; bind
U+7D34.0	fUTF8	紴
U+7D34.0	fCNS11643	3-3826
U+7D34.0	fGB	3-6C50
U+7D34.0	fR/S	  120.5
U+7D34.0	fTotalStrokes	 11
U+7D34.0	fCangjie	VFDHE
U+7D34.0	fFourCorner	24940
U+7D34.0	fCantonese	bo1
U+7D34.0	fMandarin	bei4
U+7D35.0	fUTF8	紵
U+7D35.0	fCNS11643	2-3827
U+7D35.0	fBig5	D6E2
U+7D35.0	fGB	3-6C3D
U+7D35.0	fR/S	 120.5
U+7D35.0	fTotalStrokes	 11
U+7D35.0	fCangjie	VFJMN
U+7D35.0	fFourCorner	23921
U+7D35.0	fHakka	 du3; chu3; cu3
U+7D35.0	fCantonese	cyu5
U+7D35.0	fMandarin	zhu4; zhu3
U+7D35.0	fEnglish	bohemia; ramie; sackcloth; linen
U+7D36.0	fUTF8	紶
U+7D36.0	fCNS11643	2-382A
U+7D36.0	fBig5	D6E5
U+7D36.0	fGB	5-6B60
U+7D36.0	fR/S	 120.5
U+7D36.0	fTotalStrokes	 11
U+7D36.0	fCangjie	VFGI
U+7D36.0	fFourCorner	24931
U+7D36.0	fCantonese	keoi1; heoi2
U+7D36.0	fMandarin	qu3; qu1
U+7D37.0	fUTF8	紷
U+7D37.0	fCNS11643	3-3827
U+7D37.0	fGB	3-6C4A
U+7D37.0	fR/S	  120.5
U+7D37.0	fTotalStrokes	 11
U+7D37.0	fCangjie	VFOII
U+7D37.0	fFourCorner	28930
U+7D37.0	fCantonese	ling4
U+7D37.0	fMandarin	ling2
U+7D38.0	fUTF8	紸
U+7D38.0	fCNS11643	2-3829
U+7D38.0	fBig5	D6E4
U+7D38.0	fGB	3-6C4D
U+7D38.0	fR/S	 120.5
U+7D38.0	fTotalStrokes	 11
U+7D38.0	fCangjie	VFYG
U+7D38.0	fFourCorner	20914
U+7D38.0	fCantonese	zyu3
U+7D38.0	fMandarin	zhu4
U+7D39.0	fUTF8	紹
U+7D39.0	fCNS11643	1-5C55
U+7D39.0	fBig5	B2D0
U+7D39.0	fGB	1-495C
U+7D39.0	fR/S	 120.5
U+7D39.0	fTotalStrokes	 11
U+7D39.0	fCangjie	VFSHR
U+7D39.0	fFourCorner	27962
U+7D39.0	fHakka	 sau2; shau5; sau5; seu5; shau2
U+7D39.0	fCantonese	siu6
U+7D39.0	fMandarin	shao4; chao1
U+7D39.0	fEnglish	[1] hand down; continue; carry on [2] bring together; connect; join
U+7D3A.0	fUTF8	紺
U+7D3A.0	fCNS11643	2-382B
U+7D3A.0	fBig5	D6E6
U+7D3A.0	fGB	1-6724
U+7D3A.0	fR/S	 120.5
U+7D3A.0	fTotalStrokes	 11
U+7D3A.0	fCangjie	VFTM
U+7D3A.0	fFourCorner	24970
U+7D3A.0	fHakka	  gam5
U+7D3A.0	fCantonese	gam3
U+7D3A.0	fMandarin	gan4
U+7D3A.0	fEnglish	a reddish dark color; a reddish dark color
U+7D3B.0	fUTF8	紻
U+7D3B.0	fCNS11643	2-3834
U+7D3B.0	fBig5	D6EF
U+7D3B.0	fGB	5-6B63
U+7D3B.0	fR/S	 120.5
U+7D3B.0	fTotalStrokes	 11
U+7D3B.0	fCangjie	VFLBK
U+7D3B.0	fFourCorner	25930
U+7D3B.0	fCantonese	joeng2
U+7D3B.0	fMandarin	yang1; yang3
U+7D3C.0	fUTF8	紼
U+7D3C.0	fCNS11643	1-5C56
U+7D3C.0	fBig5	B2D1
U+7D3C.0	fGB	1-6728
U+7D3C.0	fR/S	 120.5
U+7D3C.0	fTotalStrokes	 11
U+7D3C.0	fCangjie	VFLLN
U+7D3C.0	fFourCorner	25927
U+7D3C.0	fHakka	 fut7
U+7D3C.0	fCantonese	fat1
U+7D3C.0	fMandarin	fu2; fei4
U+7D3C.0	fEnglish	[1] large rope [2] the cord or rope attached to a bier or coffin
U+7D3D.0	fUTF8	紽
U+7D3D.0	fCNS11643	2-3828
U+7D3D.0	fBig5	D6E3
U+7D3D.0	fGB	3-6C4F
U+7D3D.0	fR/S	 120.5
U+7D3D.0	fTotalStrokes	 11
U+7D3D.0	fCangjie	VFJP
U+7D3D.0	fFourCorner	23911
U+7D3D.0	fHakka	 to2
U+7D3D.0	fCantonese	to4
U+7D3D.0	fMandarin	tuo2
U+7D3D.0	fEnglish	 strand of silk; braiding
U+7D3E.0	fUTF8	紾
U+7D3E.0	fCNS11643	2-3831
U+7D3E.0	fBig5	D6EC
U+7D3E.0	fGB	3-6C49
U+7D3E.0	fR/S	 120.5
U+7D3E.0	fTotalStrokes	 11
U+7D3E.0	fCangjie	VFOHH
U+7D3E.0	fFourCorner	28922
U+7D3E.0	fHakka	 zhin3; zihn3
U+7D3E.0	fCantonese	can2 (also); zan2
U+7D3E.0	fMandarin	zhen3; tian3; gu3; jin3; shan4; zhan3; zhan4
U+7D3E.0	fEnglish	 turn; twist; turn; twist
U+7D3F.0	fUTF8	紿
U+7D3F.0	fCNS11643	2-3832
U+7D3F.0	fBig5	D6ED
U+7D3F.0	fGB	1-672A
U+7D3F.0	fR/S	 120.5
U+7D3F.0	fTotalStrokes	 11
U+7D3F.0	fCangjie	VFIR
U+7D3F.0	fFourCorner	23960
U+7D3F.0	fHakka	  toi5; tai5; tui5
U+7D3F.0	fCantonese	doi6; toi5 (also)
U+7D3F.0	fMandarin	dai4
U+7D3F.0	fEnglish	cheat; fool; pretend
U+7D40.0	fUTF8	絀
U+7D40.0	fCNS11643	1-5C57
U+7D40.0	fBig5	B2D2
U+7D40.0	fGB	1-6729
U+7D40.0	fR/S	 120.5
U+7D40.0	fTotalStrokes	 11
U+7D40.0	fCangjie	VFUU
U+7D40.0	fFourCorner	22977
U+7D40.0	fHakka	 zot7; chot7; zhot7; tut7
U+7D40.0	fCantonese	zyut3; zyut6 (also); zeot1; ceot1; zeot3
U+7D40.0	fMandarin	chu4; zhuo2; lei2
U+7D40.0	fEnglish	[1] sew; stitch [2] bend [3] degrade [4] deficient; insufficient; wanting; inadequate [5] red; deep red color
U+7D41.0	fUTF8	絁
U+7D41.0	fCNS11643	2-382F
U+7D41.0	fBig5	D6EA
U+7D41.0	fGB	3-6C47
U+7D41.0	fR/S	 120.5
U+7D41.0	fTotalStrokes	 11
U+7D41.0	fCangjie	VFOPD
U+7D41.0	fFourCorner	28912
U+7D41.0	fHakka	 shi1; sii1
U+7D41.0	fCantonese	si1
U+7D41.0	fMandarin	shi1; chi1
U+7D41.0	fEnglish	a kind of coarse silk; indelicate silk
U+7D42.0	fUTF8	終
U+7D42.0	fCNS11643	1-5C5C
U+7D42.0	fBig5	B2D7
U+7D42.0	fGB	1-5655
U+7D42.0	fR/S	 120.5
U+7D42.0	fTotalStrokes	 11
U+7D42.0	fCangjie	VFHEY
U+7D42.0	fFourCorner	27933
U+7D42.0	fHakka	  zung1; zhung1
U+7D42.0	fCantonese	zung1
U+7D42.0	fMandarin	zhong1
U+7D42.0	fEnglish	[1] end; conclusion; come to the end [2] death; die; pass away [3] finally; at last; in the end; after all; in the long run [4] whole; all; entire
U+7D43.0	fUTF8	絃
U+7D43.0	fCNS11643	1-5C52
U+7D43.0	fBig5	B2CD
U+7D43.0	fGB	E-3825
U+7D43.0	fR/S	 120.5
U+7D43.0	fTotalStrokes	 11
U+7D43.0	fCangjie	VFYVI
U+7D43.0	fFourCorner	20932
U+7D43.0	fHakka	  hien2; hian3; hen2
U+7D43.0	fCantonese	jin4; jyun4 (also)
U+7D43.0	fMandarin	xian2; xuan4
U+7D43.0	fEnglish	[1] string on a musical instrument; cord [2] first and last quarters of the moon
U+7D44.0	fUTF8	組
U+7D44.0	fCNS11643	1-5C5A
U+7D44.0	fBig5	B2D5
U+7D44.0	fGB	1-5769
U+7D44.0	fR/S	 120.5
U+7D44.0	fTotalStrokes	 11
U+7D44.0	fCangjie	VFBM
U+7D44.0	fFourCorner	27910
U+7D44.0	fHakka	  zu3; zih3; zu1; zii3
U+7D44.0	fCantonese	zou2
U+7D44.0	fMandarin	zu3; qu1
U+7D44.0	fEnglish	[1] group; team; section; department; organization [2] organize; arrange; unite; form [3] tassels; fringe; girdle; tape
U+7D45.0	fUTF8	絅
U+7D45.0	fCNS11643	2-382C
U+7D45.0	fBig5	D6E7
U+7D45.0	fGB	5-6B65
U+7D45.0	fR/S	 120.5
U+7D45.0	fTotalStrokes	 11
U+7D45.0	fCangjie	VFBR
U+7D45.0	fFourCorner	27920
U+7D45.0	fHakka	 kwen3;  kwen1
U+7D45.0	fCantonese	gwing2
U+7D45.0	fMandarin	jiong3; jiong1; qing4; qiong1
U+7D45.0	fEnglish	unlined garment; dust coat; an overall with no lining
U+7D46.0	fUTF8	絆
U+7D46.0	fCNS11643	1-5C51
U+7D46.0	fBig5	B2CC
U+7D46.0	fGB	1-306D
U+7D46.0	fR/S	 120.5
U+7D46.0	fTotalStrokes	 11
U+7D46.0	fCangjie	VFFQ
U+7D46.0	fFourCorner	29950
U+7D46.0	fHakka	 ban5; pan3; pan5
U+7D46.0	fCantonese	bun6; bun3
U+7D46.0	fMandarin	ban4
U+7D46.0	fEnglish	[1] fetters; shackles [2] stumble; trip over; trip
U+7D47.0	fUTF8	絇
U+7D47.0	fCNS11643	2-3830
U+7D47.0	fBig5	D6EB
U+7D47.0	fGB	3-6C4B
U+7D47.0	fR/S	 120.5
U+7D47.0	fTotalStrokes	 11
U+7D47.0	fCangjie	VFPR
U+7D47.0	fFourCorner	27920
U+7D47.0	fHakka	  ki1;  gi1
U+7D47.0	fCantonese	keoi4
U+7D47.0	fMandarin	ju4; qu2
U+7D47.0	fEnglish	[1] ornaments for the frontal parts of shoes; thread used to embroider shoes [2] shoe toes
U+7D48.0	fUTF8	絈
U+7D48.0	fCNS11643	3-382E
U+7D48.0	fGB	E-3826
U+7D48.0	fR/S	  120.5
U+7D48.0	fTotalStrokes	 11
U+7D48.0	fCangjie	VFHA
U+7D48.0	fFourCorner	26900
U+7D48.0	fCantonese	mak6
U+7D48.0	fMandarin	mo4
U+7D49.0	fUTF8	絉
U+7D49.0	fCNS11643	3-3824
U+7D49.0	fGB	E-3827
U+7D49.0	fR/S	  120.5
U+7D49.0	fTotalStrokes	 11
U+7D49.0	fCangjie	VFID
U+7D49.0	fFourCorner	23990
U+7D49.0	fCantonese	seot6
U+7D49.0	fMandarin	shu4
U+7D4A.0	fUTF8	絊
U+7D4A.0	fCNS11643	2-3833
U+7D4A.0	fBig5	D6EE
U+7D4A.0	fGB	E-3828
U+7D4A.0	fR/S	 120.5
U+7D4A.0	fTotalStrokes	 11
U+7D4A.0	fCangjie	VFDM
U+7D4A.0	fFourCorner	25933
U+7D4A.0	fCantonese	zeoi3
U+7D4A.0	fMandarin	zui4
U+7D4B.0	fUTF8	絋
U+7D4B.0	fCNS11643	F-3754
U+7D4B.0	fGB	E-3829
U+7D4B.0	fR/S	 120.5
U+7D4B.0	fTotalStrokes	 11
U+7D4B.0	fCangjie	XVFII
U+7D4B.0	fCantonese	kwong3; kong3 (also)
U+7D4B.0	fMandarin	kuang4
U+7D4C.0	fUTF8	経
U+7D4C.0	fCNS11643	3-382B
U+7D4C.0	fGB	E-382A
U+7D4C.0	fR/S	 120.5
U+7D4C.0	fTotalStrokes	 11
U+7D4C.0	fCangjie	VFEG
U+7D4C.0	fFourCorner	27910
U+7D4C.0	fCantonese	ging1
U+7D4C.0	fMandarin	jing1; jing4
U+7D4C.0	fEnglish	 classic works pass through
U+7D4D.0	fUTF8	絍
U+7D4D.0	fCNS11643	3-3E34
U+7D4D.0	fGB	5-6B77
U+7D4D.0	fR/S	 120.6
U+7D4D.0	fTotalStrokes	 12
U+7D4D.0	fCangjie	VFOHG
U+7D4D.0	fFourCorner	22910
U+7D4D.0	fHakka	 ngim5; ngim6; ngim3; rim6; jim5
U+7D4D.0	fCantonese	jam6; jam4 (also)
U+7D4D.0	fMandarin	ren2; ren4
U+7D4D.0	fEnglish	 lay the warp weave
U+7D4E.0	fUTF8	絎
U+7D4E.0	fCNS11643	2-3F21
U+7D4E.0	fBig5	DAFB
U+7D4E.0	fGB	1-672C
U+7D4E.0	fR/S	 120.6
U+7D4E.0	fTotalStrokes	 12
U+7D4E.0	fCangjie	VFHON
U+7D4E.0	fFourCorner	21921
U+7D4E.0	fCantonese	hong4; han6
U+7D4E.0	fMandarin	heng4; hang2; xing4
U+7D4E.0	fEnglish	baste
U+7D4F.0	fUTF8	絏
U+7D4F.0	fCNS11643	2-3E76
U+7D4F.0	fBig5	DAF2
U+7D4F.0	fGB	E-382B
U+7D4F.0	fR/S	 120.6
U+7D4F.0	fTotalStrokes	 12
U+7D4F.0	fCangjie	VFLWP
U+7D4F.0	fFourCorner	25906
U+7D4F.0	fHakka	  siet7; ri3; ji3
U+7D4F.0	fCantonese	jai6; sit3
U+7D4F.0	fMandarin	xie4; yi4
U+7D4F.0	fEnglish	[1] reins; rope for leading animals [2] tie up; bind; fetters; bonds
U+7D50.0	fUTF8	結
U+7D50.0	fCNS11643	1-6138
U+7D50.0	fBig5	B5B2
U+7D50.0	fGB	1-3D61
U+7D50.0	fR/S	 120.6
U+7D50.0	fTotalStrokes	 12
U+7D50.0	fCangjie	VFGR
U+7D50.0	fFourCorner	24961
U+7D50.0	fHakka	  giet7; get7; git7; kie1; gat7; kiat7
U+7D50.0	fCantonese	git3; lit3 (also); gai3
U+7D50.0	fMandarin	jie2; ji4
U+7D50.0	fEnglish	[1] knot; tie; weave [2] unite; join; connect [3] knot [4] congeal [5] form; found; constitute [6] bear fruit; result [7] pay; settle (account, etc.) [8] node
U+7D50.1	fUTF8	結
U+7D50.1	fCNS11643	1-6138
U+7D50.1	fBig5	B5B2
U+7D50.1	fGB	1-3D61
U+7D50.1	fR/S	 120.6
U+7D50.1	fTotalStrokes	 12
U+7D50.1	fCangjie	VFGR
U+7D50.1	fFourCorner	24961
U+7D50.1	fMandarin	jie1
U+7D50.1	fEnglish	[1] stutter; stammer [2] tough; strong; durable [3] bear (fruit); form (seed)
U+7D51.0	fUTF8	絑
U+7D51.0	fCNS11643	2-3E7D
U+7D51.0	fBig5	DAF9
U+7D51.0	fGB	3-6C5A
U+7D51.0	fR/S	 120.6
U+7D51.0	fTotalStrokes	 12
U+7D51.0	fCangjie	VFHJD
U+7D51.0	fFourCorner	25990
U+7D51.0	fCantonese	zyu1
U+7D51.0	fMandarin	zhu1
U+7D52.0	fUTF8	絒
U+7D52.0	fCNS11643	2-3E7A
U+7D52.0	fBig5	DAF6
U+7D52.0	fGB	E-382C
U+7D52.0	fR/S	 120.6
U+7D52.0	fTotalStrokes	 12
U+7D52.0	fCangjie	VFILL
U+7D52.0	fFourCorner	22900
U+7D52.0	fCantonese	cau4
U+7D52.0	fMandarin	chou2
U+7D53.0	fUTF8	絓
U+7D53.0	fCNS11643	2-3E72
U+7D53.0	fBig5	DAEE
U+7D53.0	fGB	3-6C51
U+7D53.0	fR/S	 120.6
U+7D53.0	fTotalStrokes	 12
U+7D53.0	fCangjie	VFGG
U+7D53.0	fFourCorner	24914
U+7D53.0	fHakka	  gwa5; kua5
U+7D53.0	fCantonese	gwaa3; waa6
U+7D53.0	fMandarin	gua4; guai1; hua4; kua1
U+7D53.0	fEnglish	obstructed; hindered
U+7D54.0	fUTF8	絔
U+7D54.0	fCNS11643	2-3E7B
U+7D54.0	fBig5	DAF7
U+7D54.0	fGB	5-6B6F
U+7D54.0	fR/S	 120.6
U+7D54.0	fTotalStrokes	 12
U+7D54.0	fCangjie	VFMA
U+7D54.0	fFourCorner	21960
U+7D54.0	fCantonese	baak3; mak6
U+7D54.0	fMandarin	bai3; bo2; mo4
U+7D55.0	fUTF8	絕
U+7D55.0	fCNS11643	1-613A
U+7D55.0	fBig5	B5B4
U+7D55.0	fGB	E-382D
U+7D55.0	fR/S	 120.6
U+7D55.0	fTotalStrokes	 12
U+7D55.0	fCangjie	VFSHU
U+7D55.0	fFourCorner	27917
U+7D55.0	fHakka	 ciet8; cet8; ts'iat8
U+7D55.0	fCantonese	zyut6; zyut2 (also); zyut3
U+7D55.0	fMandarin	jue2
U+7D55.0	fEnglish	[1] cut; sever; break off [2] renounce; decline [3] run out of; exhausted; used up; finished [4] without match; peerless; unparalleled; utmost [5] isolated; get out of circulation; separate [6] discontinue; terminate; cease [7] without posterity; heirless [8] extremely; utmost; most; absolutely [9] destroy [10] leaving with no leeway; making no allowance; uncompromising [11] poem of four lines
U+7D56.0	fUTF8	絖
U+7D56.0	fCNS11643	2-3E73
U+7D56.0	fBig5	DAEF
U+7D56.0	fGB	5-6B73
U+7D56.0	fR/S	 120.6
U+7D56.0	fTotalStrokes	 12
U+7D56.0	fCangjie	VFFMU
U+7D56.0	fFourCorner	29911
U+7D56.0	fHakka	 fong3; kong3; gong5; kwong3; k'wong3
U+7D56.0	fCantonese	kwong3; kong3 (also); fong3
U+7D56.0	fMandarin	kuang4
U+7D56.0	fEnglish	 fine silks floss
U+7D57.0	fUTF8	絗
U+7D57.0	fCNS11643	3-3E29
U+7D57.0	fGB	E-382E
U+7D57.0	fR/S	  120.6
U+7D57.0	fTotalStrokes	 12
U+7D57.0	fCangjie	VFWR
U+7D57.0	fFourCorner	26900
U+7D57.0	fCantonese	wat6
U+7D57.0	fMandarin	hu2
U+7D58.0	fUTF8	絘
U+7D58.0	fCNS11643	2-3E6F
U+7D58.0	fBig5	DAEB
U+7D58.0	fGB	3-6C62
U+7D58.0	fR/S	 120.6
U+7D58.0	fTotalStrokes	 12
U+7D58.0	fCangjie	VFIMO
U+7D58.0	fFourCorner	27982
U+7D58.0	fCantonese	ci3
U+7D58.0	fMandarin	ci4; zi1; zi4
U+7D59.0	fUTF8	絙
U+7D59.0	fCNS11643	3-3E26
U+7D59.0	fGB	3-6C52
U+7D59.0	fR/S	  120.6
U+7D59.0	fTotalStrokes	 12
U+7D59.0	fCangjie	VFMAM
U+7D59.0	fFourCorner	21910
U+7D59.0	fCantonese	gang1; gang2 (also)
U+7D59.0	fMandarin	huan2; geng1
U+7D5A.0	fUTF8	絚
U+7D5A.0	fCNS11643	3-3E2E
U+7D5A.0	fGB	E-382F
U+7D5A.0	fR/S	 120.6
U+7D5A.0	fTotalStrokes	 12
U+7D5A.0	fCangjie	VFMBM
U+7D5A.0	fFourCorner	21910
U+7D5A.0	fHakka	 hen2; kwen3
U+7D5A.0	fCantonese	gang1; gang2 (also)
U+7D5A.0	fMandarin	geng1; geng4
U+7D5A.0	fEnglish	 rope
U+7D5B.0	fUTF8	絛
U+7D5B.0	fCNS11643	1-6573
U+7D5B.0	fBig5	B86C
U+7D5B.0	fGB	E-3830
U+7D5B.0	fR/S	 120.7
U+7D5B.0	fTotalStrokes	 13
U+7D5B.0	fCangjie	OLOF
U+7D5B.0	fFourCorner	28293
U+7D5B.0	fHakka	 tau1; to1
U+7D5B.0	fCantonese	tou1; tou3
U+7D5B.0	fMandarin	tao1
U+7D5B.0	fEnglish	 silk band; silk sash; silk ribbon
U+7D5C.0	fUTF8	絜
U+7D5C.0	fCNS11643	2-3E78
U+7D5C.0	fBig5	DAF4
U+7D5C.0	fGB	E-3831
U+7D5C.0	fR/S	 120.6
U+7D5C.0	fTotalStrokes	 12
U+7D5C.0	fCangjie	QHVIF
U+7D5C.0	fFourCorner	57903
U+7D5C.0	fHakka	  giet7; get7
U+7D5C.0	fCantonese	git3; kit3
U+7D5C.0	fMandarin	jie2; guo1; jia2; qi4
U+7D5C.0	fEnglish	pure; clean
U+7D5C.1	fUTF8	絜
U+7D5C.1	fCNS11643	2-3E78
U+7D5C.1	fBig5	DAF4
U+7D5C.1	fGB	E-3831
U+7D5C.1	fR/S	 120.6
U+7D5C.1	fTotalStrokes	 12
U+7D5C.1	fCangjie	QHVIF
U+7D5C.1	fFourCorner	57903
U+7D5C.1	fMandarin	xie2
U+7D5C.1	fEnglish	[1] ascertain; assess; measure [2] restrain; regulate
U+7D5D.0	fUTF8	絝
U+7D5D.0	fCNS11643	3-3E2F
U+7D5D.0	fGB	1-672B
U+7D5D.0	fR/S	 120.6
U+7D5D.0	fTotalStrokes	 12
U+7D5D.0	fCangjie	VFKMS
U+7D5D.0	fFourCorner	24920
U+7D5D.0	fHakka	 ku5
U+7D5D.0	fCantonese	fu3
U+7D5D.0	fMandarin	ku4
U+7D5D.0	fEnglish	 trousers breeches leggings drawers
U+7D5E.0	fUTF8	絞
U+7D5E.0	fCNS11643	1-6137
U+7D5E.0	fBig5	B5B1
U+7D5E.0	fGB	1-3D4A
U+7D5E.0	fR/S	 120.6
U+7D5E.0	fTotalStrokes	 12
U+7D5E.0	fCangjie	VFYCK
U+7D5E.0	fFourCorner	20948
U+7D5E.0	fHakka	  gau3
U+7D5E.0	fCantonese	gaau2; haau4
U+7D5E.0	fMandarin	jiao3; jia3; xiao2; yao2; jiao4
U+7D5E.0	fEnglish	[1] twist; wring; twine [2] hang (criminal) [3] rudeness [4] mix up
U+7D5F.0	fUTF8	絟
U+7D5F.0	fCNS11643	2-3E7E
U+7D5F.0	fBig5	DAFA
U+7D5F.0	fGB	3-6C5F
U+7D5F.0	fR/S	 120.6
U+7D5F.0	fTotalStrokes	 12
U+7D5F.0	fCangjie	VFOMG
U+7D5F.0	fFourCorner	28914
U+7D5F.0	fCantonese	cyun1; cyun4
U+7D5F.0	fMandarin	quan1; ji2
U+7D60.0	fUTF8	絠
U+7D60.0	fCNS11643	3-3E28
U+7D60.0	fGB	3-6C53
U+7D60.0	fR/S	  120.6
U+7D60.0	fTotalStrokes	 12
U+7D60.0	fCangjie	VFKB
U+7D60.0	fFourCorner	24920
U+7D60.0	fCantonese	goi2
U+7D60.0	fMandarin	gai3
U+7D61.0	fUTF8	絡
U+7D61.0	fCNS11643	1-613E
U+7D61.0	fBig5	B5B8
U+7D61.0	fGB	1-4267
U+7D61.0	fR/S	 120.6
U+7D61.0	fTotalStrokes	 12
U+7D61.0	fCangjie	VFHER
U+7D61.0	fFourCorner	27964
U+7D61.0	fHakka	  lok8; lok7; lu6
U+7D61.0	fCantonese	lok3; lok6 (also); lok2 (also)
U+7D61.0	fMandarin	luo4
U+7D61.0	fEnglish	[1] wrap around; encompass [2] net; web [3] a cellulose structure in fruits (melons, etc.) [4] associate; unite; connected [5] halter [6] capillaries (blood vessels) [7] unreel silk [8] cotton fiber [9] hemp [10] a Chinese family name
U+7D61.1	fUTF8	絡
U+7D61.1	fCNS11643	1-613E
U+7D61.1	fBig5	B5B8
U+7D61.1	fGB	1-4267
U+7D61.1	fR/S	 120.6
U+7D61.1	fTotalStrokes	 12
U+7D61.1	fCangjie	VFHER
U+7D61.1	fFourCorner	27964
U+7D61.1	fMandarin	lao4
U+7D61.1	fEnglish	 web; net
U+7D62.0	fUTF8	絢
U+7D62.0	fCNS11643	1-6140
U+7D62.0	fBig5	B5BA
U+7D62.0	fGB	1-5124
U+7D62.0	fR/S	 120.6
U+7D62.0	fTotalStrokes	 12
U+7D62.0	fCangjie	VFPA
U+7D62.0	fFourCorner	27920
U+7D62.0	fHakka	 sien3; sen3; hien5
U+7D62.0	fCantonese	hyun3; seon1 (also)
U+7D62.0	fMandarin	xuan4; xun2
U+7D62.0	fEnglish	adorned and stylish; brilliant and bright
U+7D63.0	fUTF8	絣
U+7D63.0	fCNS11643	2-3E71
U+7D63.0	fBig5	DAED
U+7D63.0	fGB	3-6C64
U+7D63.0	fR/S	 120.6
U+7D63.0	fTotalStrokes	 12
U+7D63.0	fCangjie	VFTT
U+7D63.0	fFourCorner	28941
U+7D63.0	fHakka	  ben1; bin1
U+7D63.0	fCantonese	bang1; maang1; bing3; bing1
U+7D63.0	fMandarin	beng1; bing1; ping3; beng3; peng1
U+7D63.0	fEnglish	 baste for sewing connect
U+7D64.0	fUTF8	絤
U+7D64.0	fCNS11643	3-3E2C
U+7D64.0	fGB	5-6B6E
U+7D64.0	fR/S	 120.6
U+7D64.0	fTotalStrokes	 12
U+7D64.0	fCangjie	VFMCW
U+7D64.0	fFourCorner	21960
U+7D64.0	fCantonese	sin3
U+7D64.0	fMandarin	xian4
U+7D65.0	fUTF8	絥
U+7D65.0	fCNS11643	4-3A27
U+7D65.0	fGB	5-6B76
U+7D65.0	fR/S	  120.6
U+7D65.0	fTotalStrokes	 12
U+7D65.0	fCangjie	VFOIK
U+7D65.0	fFourCorner	23930
U+7D65.0	fHakka	 fuk8
U+7D65.0	fCantonese	fuk6; bei6 (also)
U+7D65.0	fMandarin	bei4; fu2
U+7D65.0	fEnglish	 harness a horse a board in front of a cariage for the driver to lean on
U+7D66.0	fUTF8	給
U+7D66.0	fCNS11643	1-613F
U+7D66.0	fBig5	B5B9
U+7D66.0	fGB	1-3878
U+7D66.0	fR/S	 120.6
U+7D66.0	fTotalStrokes	 12
U+7D66.0	fCangjie	VFOMR
U+7D66.0	fFourCorner	28961
U+7D66.0	fHakka	 giap7
U+7D66.0	fCantonese	kap1
U+7D66.0	fMandarin	gei3; xia2
U+7D66.0	fEnglish	[1] give [2] for; for the benefit of [3] let; allow [4] used to indicate a passive meaning of voice [5] used after a verb to indicate "giving"
U+7D66.1	fUTF8	給
U+7D66.1	fCNS11643	1-613F
U+7D66.1	fBig5	B5B9
U+7D66.1	fGB	1-3878
U+7D66.1	fR/S	 120.6
U+7D66.1	fTotalStrokes	 12
U+7D66.1	fCangjie	VFOMR
U+7D66.1	fFourCorner	28961
U+7D66.1	fHakka	 gip7
U+7D66.1	fMandarin	ji3
U+7D66.1	fEnglish	[1] provide; provisions; supply; supplies [2] award; approve; grant [3] sufficiency; affluence [4] glib; eloquent
U+7D67.0	fUTF8	絧
U+7D67.0	fCNS11643	2-3E74
U+7D67.0	fBig5	DAF0
U+7D67.0	fGB	3-6C58
U+7D67.0	fR/S	 120.6
U+7D67.0	fTotalStrokes	 12
U+7D67.0	fCangjie	VFBMR
U+7D67.0	fFourCorner	27920
U+7D67.0	fHakka	  tung2
U+7D67.0	fCantonese	dung6; tung4; tung1
U+7D67.0	fMandarin	tong2; dong4; tong1
U+7D67.0	fEnglish	a kind of cloth
U+7D68.0	fUTF8	絨
U+7D68.0	fCNS11643	1-6139
U+7D68.0	fBig5	B5B3
U+7D68.0	fGB	1-485E
U+7D68.0	fR/S	 120.6
U+7D68.0	fTotalStrokes	 12
U+7D68.0	fCangjie	VFIJ
U+7D68.0	fFourCorner	23950
U+7D68.0	fHakka	 jung2; rung2; jung3
U+7D68.0	fCantonese	jung4 (also); jung2
U+7D68.0	fMandarin	rong2
U+7D68.0	fEnglish	[1] fine wool; woolen; velvety; velvet [2] any kind of woolen fabric or goods with a felt-like surface [3] fine; furry; flossy
U+7D69.0	fUTF8	絩
U+7D69.0	fCNS11643	2-3E7C
U+7D69.0	fBig5	DAF8
U+7D69.0	fGB	3-6C60
U+7D69.0	fR/S	 120.6
U+7D69.0	fTotalStrokes	 12
U+7D69.0	fCangjie	VFLMO
U+7D69.0	fFourCorner	22913
U+7D69.0	fCantonese	tiu3; tiu5
U+7D69.0	fMandarin	tiao4; dao4; tiao3; zhao4
U+7D6A.0	fUTF8	絪
U+7D6A.0	fCNS11643	2-3E75
U+7D6A.0	fBig5	DAF1
U+7D6A.0	fGB	5-6B74
U+7D6A.0	fR/S	 120.6
U+7D6A.0	fTotalStrokes	 12
U+7D6A.0	fCangjie	VFWK
U+7D6A.0	fFourCorner	26900
U+7D6A.0	fHakka	 jin1; rin1
U+7D6A.0	fCantonese	jan1
U+7D6A.0	fMandarin	yin1
U+7D6A.0	fEnglish	misty; foggy; cloudy
U+7D6B.0	fUTF8	絫
U+7D6B.0	fCNS11643	2-3E79
U+7D6B.0	fBig5	DAF5
U+7D6B.0	fGB	5-6B46
U+7D6B.0	fR/S	 120.6
U+7D6B.0	fTotalStrokes	 12
U+7D6B.0	fCangjie	IIIF
U+7D6B.0	fFourCorner	23903
U+7D6B.0	fHakka	 lui3; li2
U+7D6B.0	fCantonese	leoi5; leoi4
U+7D6B.0	fMandarin	lei3; lei2; luo2
U+7D6B.0	fEnglish	a weight equal to ten millet seeds
U+7D6C.0	fUTF8	絬
U+7D6C.0	fCNS11643	3-3E2A
U+7D6C.0	fGB	3-6C5C
U+7D6C.0	fR/S	  120.6
U+7D6C.0	fTotalStrokes	 12
U+7D6C.0	fCangjie	VFHJR
U+7D6C.0	fFourCorner	22960
U+7D6C.0	fCantonese	sit3
U+7D6C.0	fMandarin	xie4
U+7D6D.0	fUTF8	絭
U+7D6D.0	fCNS11643	2-3E77
U+7D6D.0	fBig5	DAF3
U+7D6D.0	fGB	3-6C25
U+7D6D.0	fR/S	 120.6
U+7D6D.0	fTotalStrokes	 12
U+7D6D.0	fCangjie	FQVIF
U+7D6D.0	fFourCorner	90903
U+7D6D.0	fHakka	 kien5; gien5
U+7D6D.0	fCantonese	gyun3; hyun3 (also)
U+7D6D.0	fMandarin	quan4; juan4; ju2; quan1
U+7D6E.0	fUTF8	絮
U+7D6E.0	fCNS11643	1-613C
U+7D6E.0	fBig5	B5B6
U+7D6E.0	fGB	0-5075
U+7D6E.0	fR/S	 120.6
U+7D6E.0	fTotalStrokes	 12
U+7D6E.0	fCangjie	VRVIF
U+7D6E.0	fFourCorner	46903
U+7D6E.0	fHakka	  si5; cih1
U+7D6E.0	fCantonese	seoi5; seoi6 (also); seoi3
U+7D6E.0	fMandarin	xu4; n4; qu4; chu4; na4
U+7D6E.0	fEnglish	[1] raw, coarse, old, waste cotton or silk [2] wooly; fluffy [3] catkins and similar blossoms [4] padding; cushioning [5] windy (chatter, writing, etc,) [6] was with cotton
U+7D6F.0	fUTF8	絯
U+7D6F.0	fCNS11643	2-3E70
U+7D6F.0	fBig5	DAEC
U+7D6F.0	fGB	3-6C63
U+7D6F.0	fR/S	 120.6
U+7D6F.0	fTotalStrokes	 12
U+7D6F.0	fCangjie	VFYVO
U+7D6F.0	fFourCorner	20982
U+7D6F.0	fHakka	 gai1; ke1
U+7D6F.0	fCantonese	goi1; haai5; haai6
U+7D6F.0	fMandarin	gai1; hai4
U+7D70.0	fUTF8	絰
U+7D70.0	fCNS11643	1-6141
U+7D70.0	fBig5	B5BB
U+7D70.0	fGB	3-6C57
U+7D70.0	fR/S	 120.6
U+7D70.0	fTotalStrokes	 12
U+7D70.0	fCangjie	VFMIG
U+7D70.0	fFourCorner	21914
U+7D70.0	fHakka	  diet7; zhi5; det7; zih5
U+7D70.0	fCantonese	dit6
U+7D70.0	fMandarin	die2
U+7D70.0	fEnglish	a white hemp hat worn in mourning for one's parents
U+7D71.0	fUTF8	統
U+7D71.0	fCNS11643	1-5C53
U+7D71.0	fBig5	B2CE
U+7D71.0	fGB	1-4D33
U+7D71.0	fR/S	 120.6
U+7D71.0	fTotalStrokes	 12
U+7D71.0	fCangjie	VFYIU
U+7D71.0	fFourCorner	20913
U+7D71.0	fHakka	  tung3
U+7D71.0	fCantonese	tung2
U+7D71.0	fMandarin	tong3; tong4
U+7D71.0	fEnglish	[1] govern; command; control; rule [2] unify; unite [3] wholly; totally; completely; generally; all [4] succession; consecutive generations; from generation to generation
U+7D72.0	fUTF8	絲
U+7D72.0	fCNS11643	1-613D
U+7D72.0	fBig5	B5B7
U+7D72.0	fGB	1-4B3F
U+7D72.0	fR/S	 120.6
U+7D72.0	fTotalStrokes	 12
U+7D72.0	fCangjie	VFVIF
U+7D72.0	fFourCorner	22993
U+7D72.0	fHakka	  sii1;  su1; si1; se1
U+7D72.0	fCantonese	si1
U+7D72.0	fMandarin	si1
U+7D72.0	fEnglish	[1] silk [2] very fine thread, fiber, etc. [3] general name of silk fabrics or goods [4] strings of a musical instrument [5] infinitesimal; trace; thread; tiny bit [6] 1/100,00 of a tael
U+7D73.0	fUTF8	絳
U+7D73.0	fCNS11643	1-6142
U+7D73.0	fBig5	B5BC
U+7D73.0	fGB	1-672D
U+7D73.0	fR/S	 120.6
U+7D73.0	fTotalStrokes	 12
U+7D73.0	fCangjie	VFHEQ
U+7D73.0	fFourCorner	27954
U+7D73.0	fHakka	 gong5
U+7D73.0	fCantonese	gong3
U+7D73.0	fMandarin	jiang4
U+7D73.0	fEnglish	crimson; a deep red color
U+7D74.0	fUTF8	絴
U+7D74.0	fCNS11643	3-3E24
U+7D74.0	fGB	E-3832
U+7D74.0	fR/S	 120.6
U+7D74.0	fTotalStrokes	 12
U+7D74.0	fCangjie	VFTQ
U+7D74.0	fFourCorner	28950
U+7D74.0	fCantonese	coeng4
U+7D74.0	fMandarin	xiang2
U+7D75.0	fUTF8	絵
U+7D75.0	fCNS11643	3-3E35
U+7D75.0	fGB	E-3833
U+7D75.0	fR/S	 120.6
U+7D75.0	fTotalStrokes	 12
U+7D75.0	fCangjie	VFOMI
U+7D75.0	fFourCorner	88930
U+7D75.0	fCantonese	kui2; kwui2 (also)
U+7D75.0	fMandarin	hui4
U+7D75.0	fEnglish	draw; sketch; paint
U+7D76.0	fUTF8	絶
U+7D76.0	fCNS11643	3-3E33
U+7D76.0	fGB	1-3E78
U+7D76.0	fR/S	 120.6
U+7D76.0	fTotalStrokes	 12
U+7D76.0	fCangjie	VFNAU
U+7D76.0	fFourCorner	27910
U+7D76.0	fCantonese	zyut6
U+7D76.0	fMandarin	jue2
U+7D76.0	fEnglish	cut
U+7D77.0	fUTF8	絷
U+7D77.0	fGB	0-746A
U+7D77.0	fR/S	 120.6
U+7D77.0	fTotalStrokes	 12
U+7D77.0	fCangjie	QIVIF
U+7D77.0	fHakka	 zhi5; chip7
U+7D77.0	fCantonese	zap1
U+7D77.0	fMandarin	zhi2; zhe2
U+7D77.0	fEnglish	simplified form of &#x7E36;
U+7D78.0	fUTF8	絸
U+7D78.0	fCNS11643	3-4367
U+7D78.0	fGB	5-6B59
U+7D78.0	fR/S	  120.7
U+7D78.0	fTotalStrokes	 13
U+7D78.0	fCangjie	VFBUU
U+7D78.0	fFourCorner	26910
U+7D78.0	fHakka	 gian3
U+7D78.0	fCantonese	gaan2
U+7D78.0	fMandarin	jian3
U+7D79.0	fUTF8	絹
U+7D79.0	fCNS11643	1-656F
U+7D79.0	fBig5	B868
U+7D79.0	fGB	1-3E6E
U+7D79.0	fR/S	 120.7
U+7D79.0	fTotalStrokes	 13
U+7D79.0	fCangjie	VFRB
U+7D79.0	fFourCorner	26927
U+7D79.0	fHakka	 gien3; gen1; gien5; gen5; gien1; kian3; gian5; gen3
U+7D79.0	fCantonese	gyun3
U+7D79.0	fMandarin	juan4; quan3; xuan4
U+7D79.0	fEnglish	[1] a kind of thick, loosely woven raw silk fabric [2] handkerchief
U+7D7A.0	fUTF8	絺
U+7D7A.0	fCNS11643	2-4624
U+7D7A.0	fBig5	DF5D
U+7D7A.0	fGB	3-6C6C
U+7D7A.0	fR/S	 120.7
U+7D7A.0	fTotalStrokes	 13
U+7D7A.0	fCangjie	VFKKB
U+7D7A.0	fFourCorner	24927
U+7D7A.0	fHakka	  hi1
U+7D7A.0	fCantonese	ci1
U+7D7A.0	fMandarin	chi1; zhi3
U+7D7A.0	fEnglish	[1] a kind of fine hemp cloth; linen [2] a Chinese family name
U+7D7B.0	fUTF8	絻
U+7D7B.0	fCNS11643	2-4626
U+7D7B.0	fBig5	DF5F
U+7D7B.0	fGB	3-6C70
U+7D7B.0	fR/S	 120.7
U+7D7B.0	fTotalStrokes	 13
U+7D7B.0	fCangjie	VFNAU
U+7D7B.0	fFourCorner	27916
U+7D7B.0	fHakka	 mien1;  mun5; mun6
U+7D7B.0	fCantonese	man6; min5; man5
U+7D7B.0	fMandarin	mian3; wen4; man2; wan4
U+7D7B.0	fEnglish	mourning
U+7D7C.0	fUTF8	絼
U+7D7C.0	fCNS11643	2-4628
U+7D7C.0	fBig5	DF61
U+7D7C.0	fGB	3-6C6F
U+7D7C.0	fR/S	 120.7
U+7D7C.0	fTotalStrokes	 13
U+7D7C.0	fCangjie	VFBSH
U+7D7C.0	fFourCorner	22922
U+7D7C.0	fCantonese	zi6; zam2
U+7D7C.0	fMandarin	zhen3; zhi4
U+7D7D.0	fUTF8	絽
U+7D7D.0	fCNS11643	2-462C
U+7D7D.0	fBig5	DF65
U+7D7D.0	fGB	3-6C59
U+7D7D.0	fR/S	 120.7
U+7D7D.0	fTotalStrokes	 13
U+7D7D.0	fCangjie	VFRHR
U+7D7D.0	fFourCorner	26960
U+7D7D.0	fCantonese	leoi5
U+7D7D.0	fMandarin	l3; luu3; lu3
U+7D7D.0	fEnglish	a silk gauze
U+7D7E.0	fUTF8	絾
U+7D7E.0	fCNS11643	3-3E2B
U+7D7E.0	fGB	5-6B71
U+7D7E.0	fR/S	 120.7; 120.6
U+7D7E.0	fTotalStrokes	 12; 13
U+7D7E.0	fCangjie	VFIHS
U+7D7E.0	fFourCorner	23950
U+7D7E.0	fCantonese	sing4
U+7D7E.0	fMandarin	cheng2
U+7D7F.0	fUTF8	絿
U+7D7F.0	fCNS11643	2-4622
U+7D7F.0	fBig5	DF5B
U+7D7F.0	fGB	3-6C69
U+7D7F.0	fR/S	 120.7
U+7D7F.0	fTotalStrokes	 13
U+7D7F.0	fCangjie	VFIJE
U+7D7F.0	fFourCorner	23932
U+7D7F.0	fHakka	  kiu2
U+7D7F.0	fCantonese	kau4; kau6
U+7D7F.0	fMandarin	qiu2
U+7D7F.0	fEnglish	urgent
U+7D80.0	fUTF8	綀
U+7D80.0	fCNS11643	2-457E
U+7D80.0	fBig5	DF59
U+7D80.0	fGB	5-6C25
U+7D80.0	fR/S	 120.7
U+7D80.0	fTotalStrokes	 13
U+7D80.0	fCangjie	VFDL
U+7D80.0	fFourCorner	25996
U+7D80.0	fHakka	 su1; so1; sii1
U+7D80.0	fCantonese	so1; si1
U+7D80.0	fMandarin	shu1
U+7D80.0	fEnglish	a kind of sackcloth
U+7D81.0	fUTF8	綁
U+7D81.0	fCNS11643	1-6571
U+7D81.0	fBig5	B86A
U+7D81.0	fGB	1-3073
U+7D81.0	fR/S	 120.7
U+7D81.0	fTotalStrokes	 13
U+7D81.0	fCangjie	VFQJL
U+7D81.0	fFourCorner	27927
U+7D81.0	fHakka	  bong3
U+7D81.0	fCantonese	bong2
U+7D81.0	fMandarin	bang3
U+7D81.0	fEnglish	 bind; tie; fasten
U+7D82.0	fUTF8	綂
U+7D82.0	fCNS11643	4-3F71
U+7D82.0	fGB	E-3834
U+7D82.0	fR/S	 120.7
U+7D82.0	fTotalStrokes	 13
U+7D82.0	fCangjie	VFYRU
U+7D82.0	fFourCorner	20910
U+7D82.0	fCantonese	tung2
U+7D82.0	fMandarin	tong3
U+7D82.0	fEnglish	govern, command, control; unite
U+7D83.0	fUTF8	綃
U+7D83.0	fCNS11643	2-4627
U+7D83.0	fBig5	DF60
U+7D83.0	fGB	1-672F
U+7D83.0	fR/S	 120.7
U+7D83.0	fTotalStrokes	 13
U+7D83.0	fCangjie	VFFB
U+7D83.0	fFourCorner	29927
U+7D83.0	fHakka	  siau1; seu1
U+7D83.0	fCantonese	siu1
U+7D83.0	fMandarin	xiao1; shao1
U+7D83.0	fEnglish	[1] a fabric made of raw silk [2] raw silk
U+7D84.0	fUTF8	綄
U+7D84.0	fCNS11643	2-462B
U+7D84.0	fBig5	DF64
U+7D84.0	fGB	3-6C72
U+7D84.0	fR/S	 120.7
U+7D84.0	fTotalStrokes	 13
U+7D84.0	fCangjie	VFJMU
U+7D84.0	fFourCorner	23911
U+7D84.0	fHakka	 fan2; fon5; gwon3; fon6
U+7D84.0	fCantonese	jyun4; wun4; wun5; waan4
U+7D84.0	fMandarin	wan4; huan3; huan2
U+7D84.0	fEnglish	[1] hate [2] crimson [3] run through; string
U+7D85.0	fUTF8	綅
U+7D85.0	fCNS11643	2-4623
U+7D85.0	fBig5	DF5C
U+7D85.0	fGB	3-6C73
U+7D85.0	fR/S	 120.7
U+7D85.0	fTotalStrokes	 13
U+7D85.0	fCangjie	VFSME
U+7D85.0	fFourCorner	27947
U+7D85.0	fHakka	  cim1
U+7D85.0	fCantonese	cam1; cim1
U+7D85.0	fMandarin	qin1; xian1; jin1
U+7D85.0	fEnglish	thread
U+7D86.0	fUTF8	綆
U+7D86.0	fCNS11643	2-457D
U+7D86.0	fBig5	DF58
U+7D86.0	fGB	1-672E
U+7D86.0	fR/S	 120.7
U+7D86.0	fTotalStrokes	 13
U+7D86.0	fCangjie	VFMLK
U+7D86.0	fFourCorner	21946
U+7D86.0	fHakka	 gien3; gang3; gen3
U+7D86.0	fCantonese	gang2
U+7D86.0	fMandarin	geng3; bing3; ban3
U+7D86.0	fEnglish	 well rope; rope for drawing water (from a well, stream, etc.)
U+7D87.0	fUTF8	綇
U+7D87.0	fCNS11643	3-4366
U+7D87.0	fGB	E-3835
U+7D87.0	fR/S	 120.7
U+7D87.0	fTotalStrokes	 13
U+7D87.0	fCangjie	VFMCW
U+7D87.0	fFourCorner	21960
U+7D87.0	fCantonese	sau2
U+7D87.0	fMandarin	xiu3
U+7D88.0	fUTF8	綈
U+7D88.0	fCNS11643	2-457C
U+7D88.0	fBig5	DF57
U+7D88.0	fGB	1-6730
U+7D88.0	fR/S	 120.7
U+7D88.0	fTotalStrokes	 13
U+7D88.0	fCangjie	VFCNH
U+7D88.0	fFourCorner	28927
U+7D88.0	fHakka	 ti2
U+7D88.0	fCantonese	tai4; tai3
U+7D88.0	fMandarin	ti2; ti4; ti3
U+7D88.0	fEnglish	a glossy thick silk fabric
U+7D89.0	fUTF8	綉
U+7D89.0	fCNS11643	3-4369
U+7D89.0	fGB	1-5065
U+7D89.0	fR/S	 120.7
U+7D89.0	fTotalStrokes	 13
U+7D89.0	fCangjie	VFHDS
U+7D89.0	fFourCorner	22920
U+7D89.0	fHakka	  siu5
U+7D89.0	fCantonese	sau3
U+7D89.0	fMandarin	xiu4; tou4
U+7D89.0	fEnglish	 embroider embroidery ornament
U+7D8A.0	fUTF8	綊
U+7D8A.0	fCNS11643	3-4360
U+7D8A.0	fGB	3-6C55
U+7D8A.0	fR/S	 120.7
U+7D8A.0	fTotalStrokes	 13
U+7D8A.0	fCangjie	VFKOO
U+7D8A.0	fFourCorner	24930
U+7D8A.0	fCantonese	haap6
U+7D8A.0	fMandarin	xie2
U+7D8B.0	fUTF8	綋
U+7D8B.0	fCNS11643	3-4362
U+7D8B.0	fGB	5-6C2B
U+7D8B.0	fR/S	  120.7
U+7D8B.0	fTotalStrokes	 13
U+7D8B.0	fCangjie	VFJKI
U+7D8B.0	fFourCorner	23930
U+7D8B.0	fCantonese	wang4
U+7D8B.0	fMandarin	hong2
U+7D8C.0	fUTF8	綌
U+7D8C.0	fCNS11643	2-4629
U+7D8C.0	fBig5	DF62
U+7D8C.0	fGB	3-6C6E
U+7D8C.0	fR/S	 120.7
U+7D8C.0	fTotalStrokes	 13
U+7D8C.0	fCangjie	VFCOR
U+7D8C.0	fFourCorner	28968
U+7D8C.0	fHakka	  hit7
U+7D8C.0	fCantonese	gwik1; kwik1
U+7D8C.0	fMandarin	xi4
U+7D8C.0	fEnglish	cloth
U+7D8D.0	fUTF8	綍
U+7D8D.0	fCNS11643	2-4621
U+7D8D.0	fBig5	DF5A
U+7D8D.0	fGB	5-6C24
U+7D8D.0	fR/S	 120.7
U+7D8D.0	fTotalStrokes	 13
U+7D8D.0	fCangjie	VFJBD
U+7D8D.0	fFourCorner	24947
U+7D8D.0	fHakka	 fut7; put8
U+7D8D.0	fCantonese	fat1; but6
U+7D8D.0	fMandarin	fu2; fei4
U+7D8D.0	fEnglish	rope
U+7D8E.0	fUTF8	綎
U+7D8E.0	fCNS11643	2-4625
U+7D8E.0	fBig5	DF5E
U+7D8E.0	fGB	3-6C5B
U+7D8E.0	fR/S	 120.7
U+7D8E.0	fTotalStrokes	 13
U+7D8E.0	fCangjie	VFNKG
U+7D8E.0	fFourCorner	22941
U+7D8E.0	fHakka	 jen2; ren2
U+7D8E.0	fCantonese	ting1; ting4 (also); jin4
U+7D8E.0	fMandarin	ting2; ting1
U+7D8E.0	fEnglish	tassels (with gems attached) on ancient coronets
U+7D8F.0	fUTF8	綏
U+7D8F.0	fCNS11643	1-6572
U+7D8F.0	fBig5	B86B
U+7D8F.0	fGB	1-4B67
U+7D8F.0	fR/S	 120.7
U+7D8F.0	fTotalStrokes	 13
U+7D8F.0	fCangjie	VFBV
U+7D8F.0	fFourCorner	22944
U+7D8F.0	fHakka	  sui1
U+7D8F.0	fCantonese	seoi1
U+7D8F.0	fMandarin	sui2; rui2; tuo3; sui1; shuai1; shui1
U+7D8F.0	fEnglish	[1] soothe; appease; pacify; repose; tranquilise [2] retreat [3] a strap to help one mount a carriage, etc. [4] banner; flag
U+7D90.0	fUTF8	綐
U+7D90.0	fCNS11643	3-435E
U+7D90.0	fGB	3-6C71
U+7D90.0	fR/S	 120.7
U+7D90.0	fTotalStrokes	 13
U+7D90.0	fCangjie	VFCRU
U+7D90.0	fFourCorner	28910
U+7D90.0	fCantonese	deoi6
U+7D90.0	fMandarin	dui4
U+7D91.0	fUTF8	綑
U+7D91.0	fCNS11643	1-6570
U+7D91.0	fBig5	B869
U+7D91.0	fGB	E-3836
U+7D91.0	fR/S	 120.7
U+7D91.0	fTotalStrokes	 13
U+7D91.0	fCangjie	VFWD
U+7D91.0	fFourCorner	26900
U+7D91.0	fHakka	 kun3; kwun3
U+7D91.0	fCantonese	kwan2
U+7D91.0	fMandarin	kun3
U+7D91.0	fEnglish	[1] roll; bundle; tie up; bundle up [2] weave
U+7D92.0	fUTF8	綒
U+7D92.0	fCNS11643	2-462D
U+7D92.0	fBig5	DF66
U+7D92.0	fGB	5-6C28
U+7D92.0	fR/S	 120.7
U+7D92.0	fTotalStrokes	 13
U+7D92.0	fCangjie	VFBND
U+7D92.0	fFourCorner	22947
U+7D92.0	fCantonese	fu1
U+7D92.0	fMandarin	fu1
U+7D93.0	fUTF8	經
U+7D93.0	fCNS11643	1-656E
U+7D93.0	fBig5	B867
U+7D93.0	fGB	1-3E2D
U+7D93.0	fR/S	 120.7
U+7D93.0	fTotalStrokes	 13
U+7D93.0	fCangjie	VFMVM
U+7D93.0	fFourCorner	21911
U+7D93.0	fHakka	  gin1; gang1; gen1
U+7D93.0	fCantonese	ging1
U+7D93.0	fMandarin	jing1; jing4
U+7D93.0	fEnglish	[1] classic works; religious scriptures; books of significant value [2] warp of a fabric; things running lengthwise [3] common or customary ways; rules; regulations [4] plan; arrange; regulate; rule; manage; deal in; engage [5] menses [6] human arteries, etc. [7] as a result; after [8] pass through; pass by [9] stand; bear; endure [10] longitude [11] a Chinese family name
U+7D93.1	fUTF8	經
U+7D93.1	fCNS11643	1-656E
U+7D93.1	fBig5	B867
U+7D93.1	fGB	1-3E2D
U+7D93.1	fR/S	 120.7
U+7D93.1	fTotalStrokes	 13
U+7D93.1	fCangjie	VFMVM
U+7D93.1	fFourCorner	21911
U+7D93.1	fMandarin	jing5
U+7D94.0	fUTF8	綔
U+7D94.0	fCNS11643	2-462A
U+7D94.0	fBig5	DF63
U+7D94.0	fGB	E-3837
U+7D94.0	fR/S	 120.7
U+7D94.0	fTotalStrokes	 13
U+7D94.0	fCangjie	KSHVF
U+7D94.0	fFourCorner	42293
U+7D94.0	fCantonese	wu6
U+7D94.0	fMandarin	hu4
U+7D95.0	fUTF8	綕
U+7D95.0	fCNS11643	3-4364
U+7D95.0	fGB	5-6C22
U+7D95.0	fR/S	 120.7
U+7D95.0	fTotalStrokes	 13
U+7D95.0	fCangjie	VFGP
U+7D95.0	fFourCorner	24930
U+7D95.0	fCantonese	zik1
U+7D95.0	fMandarin	zhi1
U+7D96.0	fUTF8	綖
U+7D96.0	fCNS11643	2-4C79
U+7D96.0	fBig5	E372
U+7D96.0	fGB	3-6C5E
U+7D96.0	fR/S	 120.7
U+7D96.0	fTotalStrokes	 13
U+7D96.0	fCangjie	VFNKM
U+7D96.0	fFourCorner	21911
U+7D96.0	fCantonese	jin4; jin1 (also)
U+7D96.0	fMandarin	yan2; xian4; yan3; yan4
U+7D96.0	fEnglish	the hanging flap in front and at the back of a hat
U+7D97.0	fUTF8	綗
U+7D97.0	fCNS11643	3-4368
U+7D97.0	fGB	E-3838
U+7D97.0	fR/S	 120.7
U+7D97.0	fTotalStrokes	 13
U+7D97.0	fCangjie	VFBCR
U+7D97.0	fFourCorner	27920
U+7D97.0	fCantonese	gwing2
U+7D97.0	fMandarin	jiong3
U+7D98.0	fUTF8	綘
U+7D98.0	fCNS11643	3-436A
U+7D98.0	fGB	5-6C29
U+7D98.0	fR/S	 120.7
U+7D98.0	fTotalStrokes	 13
U+7D98.0	fCangjie	VFHEJ
U+7D98.0	fFourCorner	27950
U+7D98.0	fHakka	 fun2; pung2
U+7D98.0	fCantonese	fung4
U+7D98.0	fMandarin	feng2
U+7D99.0	fUTF8	継
U+7D99.0	fCNS11643	3-4363
U+7D99.0	fGB	E-3839
U+7D99.0	fR/S	 120.7
U+7D99.0	fTotalStrokes	 13
U+7D99.0	fCangjie	VFVFD
U+7D99.0	fFourCorner	29910
U+7D99.0	fCantonese	gai3
U+7D99.0	fMandarin	ji4
U+7D99.0	fEnglish	continue; maintain; carry on
U+7D9A.0	fUTF8	続
U+7D9A.0	fGB	E-383A
U+7D9A.0	fR/S	 120.7
U+7D9A.0	fTotalStrokes	 13
U+7D9A.0	fCangjie	VFGBU
U+7D9A.0	fCantonese	zuk6
U+7D9A.0	fMandarin	xu4
U+7D9A.0	fEnglish	 continue, carry on; succeed
U+7D9B.0	fUTF8	綛
U+7D9B.0	fGB	E-383B
U+7D9B.0	fR/S	 120.7
U+7D9B.0	fTotalStrokes	 13
U+7D9B.0	fCangjie	VFSIP
U+7D9B.0	fFourCorner	2793
U+7D9B.0	fCantonese	jan2
U+7D9C.0	fUTF8	綜
U+7D9C.0	fCNS11643	1-6A37
U+7D9C.0	fBig5	BAEE
U+7D9C.0	fGB	1-575B
U+7D9C.0	fR/S	 120.8
U+7D9C.0	fTotalStrokes	 14
U+7D9C.0	fCangjie	VFJMF
U+7D9C.0	fFourCorner	23991
U+7D9C.0	fHakka	  zung1; zung5
U+7D9C.0	fCantonese	zung1; zung3
U+7D9C.0	fMandarin	zong1; zeng4; zong4
U+7D9C.0	fEnglish	[1] sum up; in a nutshell [2] in view of; take account of [3] arrange [4] synthesis; synthetic [5] inquire into; examine into
U+7D9D.0	fUTF8	綝
U+7D9D.0	fCNS11643	2-4C71
U+7D9D.0	fBig5	E36A
U+7D9D.0	fGB	3-6C79
U+7D9D.0	fR/S	 120.8
U+7D9D.0	fTotalStrokes	 14
U+7D9D.0	fCangjie	VFDD
U+7D9D.0	fFourCorner	24990
U+7D9D.0	fHakka	 chim1; cihm1
U+7D9D.0	fCantonese	sam1; lam4 (also)
U+7D9D.0	fMandarin	lin2; chen1; lin3; shen1; sen1
U+7D9D.0	fEnglish	 stop; properly adjusted
U+7D9E.0	fUTF8	綞
U+7D9E.0	fCNS11643	1-6E43
U+7D9E.0	fBig5	BD78
U+7D9E.0	fGB	1-6736
U+7D9E.0	fR/S	 120.8
U+7D9E.0	fTotalStrokes	 14
U+7D9E.0	fCangjie	VFHJM
U+7D9E.0	fFourCorner	22914
U+7D9E.0	fCantonese	do2
U+7D9E.0	fMandarin	duo3; chui2; nuo3
U+7D9F.0	fUTF8	綟
U+7D9F.0	fCNS11643	2-4C7B
U+7D9F.0	fBig5	E374
U+7D9F.0	fGB	3-6D28
U+7D9F.0	fR/S	 120.8
U+7D9F.0	fTotalStrokes	 14
U+7D9F.0	fCangjie	VFHSK
U+7D9F.0	fFourCorner	23934
U+7D9F.0	fHakka	 li5; lui5; li6; lui6
U+7D9F.0	fCantonese	lit6; lai6 (also); leoi6
U+7D9F.0	fMandarin	li4; ji4; lie4
U+7D9F.0	fEnglish	a dark, dull green
U+7DA0.0	fUTF8	綠
U+7DA0.0	fCNS11643	1-6A3A
U+7DA0.0	fBig5	BAF1
U+7DA0.0	fGB	E-383C
U+7DA0.0	fR/S	 120.8
U+7DA0.0	fTotalStrokes	 14
U+7DA0.0	fCangjie	VFVNE
U+7DA0.0	fFourCorner	27932
U+7DA0.0	fHakka	 liuk8;  luk8
U+7DA0.0	fCantonese	luk6; luk2 (also)
U+7DA0.0	fMandarin	luu4; l4
U+7DA0.0	fEnglish	[1] green (color) [2] old character for chlorine, now written as &@x6C2F;
U+7DA0.1	fUTF8	綠
U+7DA0.1	fCNS11643	1-6A3A
U+7DA0.1	fBig5	BAF1
U+7DA0.1	fGB	E-383C
U+7DA0.1	fR/S	 120.8
U+7DA0.1	fTotalStrokes	 14
U+7DA0.1	fCangjie	VFVNE
U+7DA0.1	fFourCorner	27932
U+7DA0.1	fMandarin	lu4
U+7DA0.1	fEnglish	used in combination
U+7DA1.0	fUTF8	綡
U+7DA1.0	fCNS11643	2-4D21
U+7DA1.0	fBig5	E378
U+7DA1.0	fGB	5-6C3A
U+7DA1.0	fR/S	 120.8
U+7DA1.0	fTotalStrokes	 14
U+7DA1.0	fCangjie	VFYRF
U+7DA1.0	fFourCorner	20996
U+7DA1.0	fHakka	  liong2
U+7DA1.0	fCantonese	loeng4
U+7DA1.0	fMandarin	liang2
U+7DA1.0	fEnglish	strings of an official cap
U+7DA2.0	fUTF8	綢
U+7DA2.0	fCNS11643	1-6A40
U+7DA2.0	fBig5	BAF7
U+7DA2.0	fGB	1-3371
U+7DA2.0	fR/S	 120.8
U+7DA2.0	fTotalStrokes	 14
U+7DA2.0	fCangjie	VFBGR
U+7DA2.0	fFourCorner	27920
U+7DA2.0	fHakka	 cu2;  chiu2; ciu2; chiu3; chu2
U+7DA2.0	fCantonese	cau4; cau2 (also)
U+7DA2.0	fMandarin	chou2; diao4; tao1
U+7DA2.0	fEnglish	[1] a general name of all silk fabrics [2] fine and delicate [3] twine and tangle
U+7DA3.0	fUTF8	綣
U+7DA3.0	fCNS11643	2-4C6C
U+7DA3.0	fBig5	E365
U+7DA3.0	fGB	1-6739
U+7DA3.0	fR/S	 120.8
U+7DA3.0	fTotalStrokes	 14
U+7DA3.0	fCangjie	VFFQU
U+7DA3.0	fFourCorner	29912
U+7DA3.0	fHakka	 gien3; gen3; gian3
U+7DA3.0	fCantonese	hyun3; hyun2
U+7DA3.0	fMandarin	quan3; quan4
U+7DA3.0	fEnglish	 make tender love
U+7DA4.0	fUTF8	綤
U+7DA4.0	fCNS11643	3-4874
U+7DA4.0	fGB	5-6B48
U+7DA4.0	fR/S	 120.8
U+7DA4.0	fTotalStrokes	 14
U+7DA4.0	fCangjie	SLVIF
U+7DA4.0	fFourCorner	17900
U+7DA4.0	fCantonese	siu6
U+7DA4.0	fMandarin	shao4
U+7DA5.0	fUTF8	綥
U+7DA5.0	fCNS11643	4-457B
U+7DA5.0	fGB	5-6C34
U+7DA5.0	fR/S	 120.8
U+7DA5.0	fTotalStrokes	 14
U+7DA5.0	fCangjie	VFWML
U+7DA5.0	fFourCorner	26920
U+7DA5.0	fCantonese	kei4
U+7DA5.0	fMandarin	qi2; qi4
U+7DA6.0	fUTF8	綦
U+7DA6.0	fCNS11643	2-4C7C
U+7DA6.0	fBig5	E375
U+7DA6.0	fGB	0-746B
U+7DA6.0	fR/S	 120.8
U+7DA6.0	fTotalStrokes	 14
U+7DA6.0	fCangjie	TCVIF
U+7DA6.0	fFourCorner	44900
U+7DA6.0	fHakka	  ki2; gi1
U+7DA6.0	fCantonese	kei4
U+7DA6.0	fMandarin	qi2; ji4
U+7DA6.0	fEnglish	[1] very; exceedingly [2] dark gray [3] shoelaces
U+7DA7.0	fUTF8	綧
U+7DA7.0	fCNS11643	2-4C69
U+7DA7.0	fBig5	E362
U+7DA7.0	fGB	5-6C3B
U+7DA7.0	fR/S	 120.8
U+7DA7.0	fTotalStrokes	 14
U+7DA7.0	fCangjie	VFYRD
U+7DA7.0	fFourCorner	20947
U+7DA7.0	fHakka	 zhun3
U+7DA7.0	fCantonese	zeon2
U+7DA7.0	fMandarin	zhun3
U+7DA8.0	fUTF8	綨
U+7DA8.0	fCNS11643	3-486D
U+7DA8.0	fGB	5-6C2E
U+7DA8.0	fR/S	 120.8
U+7DA8.0	fTotalStrokes	 14
U+7DA8.0	fCangjie	VFTMC
U+7DA8.0	fFourCorner	24980
U+7DA8.0	fCantonese	kei4
U+7DA8.0	fMandarin	qi2
U+7DA9.0	fUTF8	綩
U+7DA9.0	fCNS11643	2-4C7E
U+7DA9.0	fBig5	E377
U+7DA9.0	fGB	3-6D27
U+7DA9.0	fR/S	 120.8
U+7DA9.0	fTotalStrokes	 14
U+7DA9.0	fCangjie	VFJNU
U+7DA9.0	fFourCorner	23912
U+7DA9.0	fCantonese	jyun2
U+7DA9.0	fMandarin	wan3
U+7DAA.0	fUTF8	綪
U+7DAA.0	fCNS11643	2-4C6D
U+7DAA.0	fBig5	E366
U+7DAA.0	fGB	3-6C75
U+7DAA.0	fR/S	 120.8
U+7DAA.0	fTotalStrokes	 14
U+7DAA.0	fCangjie	VFQMB
U+7DAA.0	fFourCorner	25927
U+7DAA.0	fHakka	  cien5;  zen1
U+7DAA.0	fCantonese	sin6; zang1; sin3
U+7DAA.0	fMandarin	qian4; zheng1; qing1
U+7DAA.0	fEnglish	dark red
U+7DAB.0	fUTF8	綫
U+7DAB.0	fCNS11643	3-486E
U+7DAB.0	fGB	1-4F5F
U+7DAB.0	fR/S	 120.8
U+7DAB.0	fTotalStrokes	 14
U+7DAB.0	fCangjie	VFII
U+7DAB.0	fFourCorner	23950
U+7DAB.0	fHakka	 sien5;  sen5; sien3
U+7DAB.0	fCantonese	sin3
U+7DAB.0	fMandarin	xian4
U+7DAB.0	fEnglish	 line; thread; wire clue
U+7DAC.0	fUTF8	綬
U+7DAC.0	fCNS11643	1-6A47
U+7DAC.0	fBig5	BAFE
U+7DAC.0	fGB	1-6737
U+7DAC.0	fR/S	 120.8
U+7DAC.0	fTotalStrokes	 14
U+7DAC.0	fCangjie	VFBBE
U+7DAC.0	fFourCorner	32947
U+7DAC.0	fHakka	 shiu6; su5; su3; shiu5; siu3; shu5
U+7DAC.0	fCantonese	sau6
U+7DAC.0	fMandarin	shou4
U+7DAC.0	fEnglish	silk ribbons attached to an official seal or medal
U+7DAD.0	fUTF8	維
U+7DAD.0	fCNS11643	1-6A44
U+7DAD.0	fBig5	BAFB
U+7DAD.0	fGB	1-4E2C
U+7DAD.0	fR/S	 120.8
U+7DAD.0	fTotalStrokes	 14
U+7DAD.0	fCangjie	VFOG
U+7DAD.0	fFourCorner	20914
U+7DAD.0	fHakka	  wui2; wi2; wui3; vui2
U+7DAD.0	fCantonese	wai4; jung1
U+7DAD.0	fMandarin	wei2; yong1; yi2
U+7DAD.0	fEnglish	[1] hold fast; tie; secure [2] maintain, preserve; safeguard [3] unite; hold together [4] long and slender (fibers) [5] an initial particle (only, but, etc.) [6] pattern; rule
U+7DAE.0	fUTF8	綮
U+7DAE.0	fCNS11643	2-4C7D
U+7DAE.0	fBig5	E376
U+7DAE.0	fGB	0-746C
U+7DAE.0	fR/S	 120.8
U+7DAE.0	fTotalStrokes	 14
U+7DAE.0	fCangjie	HKVIF
U+7DAE.0	fFourCorner	38903
U+7DAE.0	fHakka	  ki3
U+7DAE.0	fCantonese	hing3; kai2
U+7DAE.0	fMandarin	qi3; qing2; qing3; yi4
U+7DAE.0	fEnglish	a sheat for a lancehead
U+7DAE.1	fUTF8	綮
U+7DAE.1	fCNS11643	2-4C7D
U+7DAE.1	fBig5	E376
U+7DAE.1	fGB	0-746C
U+7DAE.1	fR/S	 120.8
U+7DAE.1	fTotalStrokes	 14
U+7DAE.1	fCangjie	HKVIF
U+7DAE.1	fFourCorner	38903
U+7DAE.1	fMandarin	qing4
U+7DAE.1	fEnglish	used in combination
U+7DAF.0	fUTF8	綯
U+7DAF.0	fCNS11643	2-4C77
U+7DAF.0	fBig5	E370
U+7DAF.0	fGB	3-6D23
U+7DAF.0	fR/S	 120.8
U+7DAF.0	fTotalStrokes	 14
U+7DAF.0	fCangjie	VFPOU
U+7DAF.0	fFourCorner	27920
U+7DAF.0	fHakka	 tau2;  to2
U+7DAF.0	fCantonese	tou4
U+7DAF.0	fMandarin	tao2; ku4
U+7DAF.0	fEnglish	 braid; twist twisted or braided
U+7DB0.0	fUTF8	綰
U+7DB0.0	fCNS11643	1-6A36
U+7DB0.0	fBig5	BAED
U+7DB0.0	fGB	1-673A
U+7DB0.0	fR/S	 120.8
U+7DB0.0	fTotalStrokes	 14
U+7DB0.0	fCangjie	VFJRR
U+7DB0.0	fFourCorner	23977
U+7DB0.0	fHakka	 kwan3; gon3; gwon3
U+7DB0.0	fCantonese	waan2; waan5
U+7DB0.0	fMandarin	wan3; wan4
U+7DB0.0	fEnglish	 string together; bind up (hair, etc.); coil up
U+7DB1.0	fUTF8	綱
U+7DB1.0	fCNS11643	1-6A3E
U+7DB1.0	fBig5	BAF5
U+7DB1.0	fGB	1-3859
U+7DB1.0	fR/S	 120.8
U+7DB1.0	fTotalStrokes	 14
U+7DB1.0	fCangjie	VFBTU
U+7DB1.0	fFourCorner	27920
U+7DB1.0	fHakka	  gong1
U+7DB1.0	fCantonese	gong1
U+7DB1.0	fMandarin	gang1
U+7DB1.0	fEnglish	[1] the large rope of a net, round which it is netted, and by which it is drawn [2] outline; main points [3] principle; discipline
U+7DB2.0	fUTF8	網
U+7DB2.0	fCNS11643	1-6A3D
U+7DB2.0	fBig5	BAF4
U+7DB2.0	fGB	1-4D78
U+7DB2.0	fR/S	 120.8
U+7DB2.0	fTotalStrokes	 14
U+7DB2.0	fCangjie	VFBTV
U+7DB2.0	fFourCorner	27920
U+7DB2.0	fHakka	  miong3; miong5
U+7DB2.0	fCantonese	mong5; mong1 (also)
U+7DB2.0	fMandarin	wang3
U+7DB2.0	fEnglish	[1] net; web; network [2] dragnet (figuratively); arms of the law [3] bring together; collect [4] the World Wide Web
U+7DB3.0	fUTF8	綳
U+7DB3.0	fCNS11643	3-4870
U+7DB3.0	fGB	1-3141
U+7DB3.0	fR/S	 120.8
U+7DB3.0	fTotalStrokes	 14
U+7DB3.0	fCangjie	VFBB
U+7DB3.0	fFourCorner	27920
U+7DB3.0	fHakka	  ben1; pen2; bong5
U+7DB3.0	fCantonese	bang1; mang2; maang1 (also)
U+7DB3.0	fMandarin	beng1; beng3
U+7DB3.0	fEnglish	 a cloth to carry an infant on the back tie bind
U+7DB4.0	fUTF8	綴
U+7DB4.0	fCNS11643	1-6A3C
U+7DB4.0	fBig5	BAF3
U+7DB4.0	fGB	1-573A
U+7DB4.0	fR/S	 120.8
U+7DB4.0	fTotalStrokes	 14
U+7DB4.0	fCangjie	VFEEE
U+7DB4.0	fFourCorner	27947
U+7DB4.0	fHakka	  dot7; zui5; zot7; zhui5
U+7DB4.0	fCantonese	zeoi3; zeoi6 (also); zyut3; zyut6 (also)
U+7DB4.0	fMandarin	zhui4; chuo4; dui4; zhuo2
U+7DB4.0	fEnglish	[1] put together; patch together; combine; compose [2] mend clothes; patch up; sew; stitch [3] decorate; stud
U+7DB5.0	fUTF8	綵
U+7DB5.0	fCNS11643	1-6A42
U+7DB5.0	fBig5	BAF9
U+7DB5.0	fGB	E-383D
U+7DB5.0	fR/S	 120.8
U+7DB5.0	fTotalStrokes	 14
U+7DB5.0	fCangjie	VFBD
U+7DB5.0	fFourCorner	22994
U+7DB5.0	fHakka	 cai3
U+7DB5.0	fCantonese	coi2
U+7DB5.0	fMandarin	cai3
U+7DB5.0	fEnglish	[1] varicolored silk; silf festoon [2] motley; varicolored
U+7DB6.0	fUTF8	綶
U+7DB6.0	fCNS11643	3-4869
U+7DB6.0	fGB	E-383E
U+7DB6.0	fR/S	  120.8
U+7DB6.0	fTotalStrokes	 14
U+7DB6.0	fCangjie	VFWD
U+7DB6.0	fFourCorner	26990
U+7DB6.0	fHakka	 go3; gwo3; kwo3
U+7DB6.0	fCantonese	gwo2; go2 (also)
U+7DB6.0	fMandarin	guo3
U+7DB6.0	fEnglish	 bind; bandage
U+7DB7.0	fUTF8	綷
U+7DB7.0	fCNS11643	2-4C6A
U+7DB7.0	fBig5	E363
U+7DB7.0	fGB	3-6D24
U+7DB7.0	fR/S	 120.8
U+7DB7.0	fTotalStrokes	 14
U+7DB7.0	fCangjie	VFYOJ
U+7DB7.0	fFourCorner	20948
U+7DB7.0	fHakka	 cui5
U+7DB7.0	fCantonese	ceoi3; zeoi3
U+7DB7.0	fMandarin	cui4; cui3; sui1; zu2; zui4
U+7DB8.0	fUTF8	綸
U+7DB8.0	fCNS11643	1-6A43
U+7DB8.0	fBig5	BAFA
U+7DB8.0	fGB	1-425A
U+7DB8.0	fR/S	 120.8
U+7DB8.0	fTotalStrokes	 14
U+7DB8.0	fCangjie	VFOMB
U+7DB8.0	fFourCorner	28927
U+7DB8.0	fHakka	 lun2; lun3
U+7DB8.0	fCantonese	leon4; gwaan1; gwan1
U+7DB8.0	fMandarin	lun2
U+7DB8.0	fEnglish	[1] green silk thread or cord [2] fishing line
U+7DB8.1	fUTF8	綸
U+7DB8.1	fCNS11643	1-6A43
U+7DB8.1	fBig5	BAFA
U+7DB8.1	fGB	1-425A
U+7DB8.1	fR/S	 120.8
U+7DB8.1	fTotalStrokes	 14
U+7DB8.1	fCangjie	VFOMB
U+7DB8.1	fFourCorner	28927
U+7DB8.1	fMandarin	guan1
U+7DB8.1	fEnglish	a kind of ancient cap
U+7DB9.0	fUTF8	綹
U+7DB9.0	fCNS11643	2-4C78
U+7DB9.0	fBig5	E371
U+7DB9.0	fGB	1-6738
U+7DB9.0	fR/S	 120.8
U+7DB9.0	fTotalStrokes	 14
U+7DB9.0	fCangjie	VFHOR
U+7DB9.0	fFourCorner	28964
U+7DB9.0	fHakka	  liu3
U+7DB9.0	fCantonese	lau5
U+7DB9.0	fMandarin	liu3
U+7DB9.0	fEnglish	 skein (of yarn); tuft; lock
U+7DBA.0	fUTF8	綺
U+7DBA.0	fCNS11643	1-6A3F
U+7DBA.0	fBig5	BAF6
U+7DBA.0	fGB	1-6732
U+7DBA.0	fR/S	 120.8
U+7DBA.0	fTotalStrokes	 14
U+7DBA.0	fCangjie	VFKMR
U+7DBA.0	fFourCorner	24921
U+7DBA.0	fHakka	 ji3; ki3; ri3
U+7DBA.0	fCantonese	ji2
U+7DBA.0	fMandarin	qi3; yi3
U+7DBA.0	fEnglish	[1] elegant; beautiful; magnificent; fine; fair; gorgeous; resplendent [2] twilled silk cloth; figured woven silk material
U+7DBB.0	fUTF8	綻
U+7DBB.0	fCNS11643	1-6A35
U+7DBB.0	fBig5	BAEC
U+7DBB.0	fGB	1-5540
U+7DBB.0	fR/S	 120.8
U+7DBB.0	fTotalStrokes	 14
U+7DBB.0	fCangjie	VFJMO
U+7DBB.0	fFourCorner	23981
U+7DBB.0	fHakka	 can5;  tien5; zan3; tien6; can3; ten3
U+7DBB.0	fCantonese	zaan6
U+7DBB.0	fMandarin	zhan4; dian4
U+7DBB.0	fEnglish	[1] ripped seam; crack [2] flaw; defect (in a scheme, etc.)
U+7DBC.0	fUTF8	綼
U+7DBC.0	fCNS11643	2-4C7A
U+7DBC.0	fBig5	E373
U+7DBC.0	fGB	3-6D21
U+7DBC.0	fR/S	 120.8
U+7DBC.0	fTotalStrokes	 14
U+7DBC.0	fCangjie	VFHWJ
U+7DBC.0	fFourCorner	26940
U+7DBC.0	fCantonese	bik1
U+7DBC.0	fMandarin	bei1; bi4; pi2
U+7DBD.0	fUTF8	綽
U+7DBD.0	fCNS11643	1-6A38
U+7DBD.0	fBig5	BAEF
U+7DBD.0	fGB	1-3442
U+7DBD.0	fR/S	 120.8
U+7DBD.0	fTotalStrokes	 14
U+7DBD.0	fCangjie	VFYAJ
U+7DBD.0	fFourCorner	21946
U+7DBD.0	fHakka	 shok7; sok7; cok7; tsek7; zok7; zhok7
U+7DBD.0	fCantonese	coek3; caau1
U+7DBD.0	fMandarin	chuo4; chao1
U+7DBD.0	fEnglish	[1] delicate [2] roomy; spacious
U+7DBE.0	fUTF8	綾
U+7DBE.0	fCNS11643	1-6A39
U+7DBE.0	fBig5	BAF0
U+7DBE.0	fGB	1-6731
U+7DBE.0	fR/S	 120.8
U+7DBE.0	fTotalStrokes	 14
U+7DBE.0	fCangjie	VFGCE
U+7DBE.0	fFourCorner	24947
U+7DBE.0	fHakka	  lin2; lin3
U+7DBE.0	fCantonese	ling4
U+7DBE.0	fMandarin	ling2
U+7DBE.0	fEnglish	a very fine silk cloth; damask silk
U+7DBF.0	fUTF8	綿
U+7DBF.0	fCNS11643	1-6A41
U+7DBF.0	fBig5	BAF8
U+7DBF.0	fGB	1-4360
U+7DBF.0	fR/S	 120.8
U+7DBF.0	fTotalStrokes	 14
U+7DBF.0	fCangjie	VFHAB
U+7DBF.0	fFourCorner	26927
U+7DBF.0	fHakka	 mien2; men2; mian3
U+7DBF.0	fCantonese	min4
U+7DBF.0	fMandarin	mian2
U+7DBF.0	fEnglish	[1] cotton [2] everlasting; endless [3] weak
U+7DC0.0	fUTF8	緀
U+7DC0.0	fCNS11643	2-4C6F
U+7DC0.0	fBig5	E368
U+7DC0.0	fGB	5-6C32
U+7DC0.0	fR/S	 120.8
U+7DC0.0	fTotalStrokes	 14
U+7DC0.0	fCangjie	VFJLV
U+7DC0.0	fFourCorner	25944
U+7DC0.0	fHakka	  ci1
U+7DC0.0	fCantonese	cai1; cai2
U+7DC0.0	fMandarin	qi1; qie4; ci3
U+7DC0.0	fEnglish	[1] the stripes and shades in silken fabrics [2] elegant
U+7DC1.0	fUTF8	緁
U+7DC1.0	fCNS11643	2-4C6E
U+7DC1.0	fBig5	E367
U+7DC1.0	fGB	5-6C31
U+7DC1.0	fR/S	 120.8
U+7DC1.0	fTotalStrokes	 14
U+7DC1.0	fCangjie	VFJLO
U+7DC1.0	fFourCorner	25981
U+7DC1.0	fCantonese	cap1 (also); cip3; zip3 (also); zit6
U+7DC1.0	fMandarin	qie4; qi1; jie2; ji1; qi4
U+7DC1.0	fEnglish	 join; splice; braid
U+7DC2.0	fUTF8	緂
U+7DC2.0	fCNS11643	2-4C6B
U+7DC2.0	fBig5	E364
U+7DC2.0	fGB	3-6D26
U+7DC2.0	fR/S	 120.8
U+7DC2.0	fTotalStrokes	 14
U+7DC2.0	fCangjie	VFFF
U+7DC2.0	fFourCorner	29989
U+7DC2.0	fCantonese	taam1
U+7DC2.0	fMandarin	tan1; chan1; tian2; tan3; zui4
U+7DC3.0	fUTF8	緃
U+7DC3.0	fCNS11643	4-457C
U+7DC3.0	fGB	5-6C36
U+7DC3.0	fR/S	  120.8
U+7DC3.0	fTotalStrokes	 14
U+7DC3.0	fCangjie	VFOOO
U+7DC3.0	fFourCorner	28980
U+7DC3.0	fCantonese	zung1
U+7DC3.0	fMandarin	zong1
U+7DC4.0	fUTF8	緄
U+7DC4.0	fCNS11643	2-4C73
U+7DC4.0	fBig5	E36C
U+7DC4.0	fGB	1-6735
U+7DC4.0	fR/S	 120.8
U+7DC4.0	fTotalStrokes	 14
U+7DC4.0	fCangjie	VFAPP
U+7DC4.0	fFourCorner	26911
U+7DC4.0	fHakka	 gun3; gwun3
U+7DC4.0	fCantonese	gwan2; kwan2
U+7DC4.0	fMandarin	gun3; hun4; hun3; hun2
U+7DC4.0	fEnglish	 cord woven belt hem hemming
U+7DC5.0	fUTF8	緅
U+7DC5.0	fCNS11643	2-4C70
U+7DC5.0	fBig5	E369
U+7DC5.0	fGB	3-6C77
U+7DC5.0	fR/S	 120.8
U+7DC5.0	fTotalStrokes	 14
U+7DC5.0	fCangjie	VFSJE
U+7DC5.0	fFourCorner	27940
U+7DC5.0	fHakka	  zeu1
U+7DC5.0	fCantonese	zau1
U+7DC5.0	fMandarin	zou1; ju1; ju4
U+7DC5.0	fEnglish	[1] bluish red; a puce color [2] light red
U+7DC6.0	fUTF8	緆
U+7DC6.0	fCNS11643	2-4C74
U+7DC6.0	fBig5	E36D
U+7DC6.0	fGB	3-6C7C
U+7DC6.0	fR/S	 120.8
U+7DC6.0	fTotalStrokes	 14
U+7DC6.0	fCangjie	VFAPH
U+7DC6.0	fFourCorner	26927
U+7DC6.0	fHakka	  sit7;  siak7
U+7DC6.0	fCantonese	sik1; sik3
U+7DC6.0	fMandarin	yi4; xi4
U+7DC6.0	fEnglish	[1] fine clothe [2] shirt
U+7DC7.0	fUTF8	緇
U+7DC7.0	fCNS11643	1-6A46
U+7DC7.0	fBig5	BAFD
U+7DC7.0	fGB	1-673B
U+7DC7.0	fR/S	 120.8
U+7DC7.0	fTotalStrokes	 14
U+7DC7.0	fCangjie	VFVVW
U+7DC7.0	fFourCorner	22963
U+7DC7.0	fHakka	 zih1; zu1; zii1
U+7DC7.0	fCantonese	zi1
U+7DC7.0	fMandarin	zi1; zhi1; zi3; zi4
U+7DC7.0	fEnglish	[1] black [2] black silk
U+7DC8.0	fUTF8	緈
U+7DC8.0	fCNS11643	3-4868
U+7DC8.0	fGB	3-6C76
U+7DC8.0	fR/S	 120.8
U+7DC8.0	fTotalStrokes	 14
U+7DC8.0	fCangjie	VFGTJ
U+7DC8.0	fFourCorner	24940
U+7DC8.0	fCantonese	hang6
U+7DC8.0	fMandarin	xing4
U+7DC9.0	fUTF8	緉
U+7DC9.0	fCNS11643	2-4D22
U+7DC9.0	fBig5	E379
U+7DC9.0	fGB	3-6C66
U+7DC9.0	fR/S	 120.8
U+7DC9.0	fTotalStrokes	 14
U+7DC9.0	fCangjie	VFMLB
U+7DC9.0	fFourCorner	21927
U+7DC9.0	fHakka	  liong3
U+7DC9.0	fCantonese	loeng5; loeng6
U+7DC9.0	fMandarin	liang3; liang4
U+7DC9.0	fEnglish	pair of shoes, shoe-strings
U+7DCA.0	fUTF8	緊
U+7DCA.0	fCNS11643	1-6A3B
U+7DCA.0	fBig5	BAF2
U+7DCA.0	fGB	1-3D74
U+7DCA.0	fR/S	 120.8
U+7DCA.0	fTotalStrokes	 14
U+7DCA.0	fCangjie	SEVIF
U+7DCA.0	fFourCorner	77903
U+7DCA.0	fHakka	  gin3
U+7DCA.0	fCantonese	gan2
U+7DCA.0	fMandarin	jin3
U+7DCA.0	fEnglish	[1] tense; tight; taut; firm; secure; fast; close [2] urgent; critical; pressing [3] modifier for ongoing actions
U+7DCB.0	fUTF8	緋
U+7DCB.0	fCNS11643	2-4C75
U+7DCB.0	fBig5	E36E
U+7DCB.0	fGB	1-6733
U+7DCB.0	fR/S	 120.8
U+7DCB.0	fTotalStrokes	 14
U+7DCB.0	fCangjie	VFLMY
U+7DCB.0	fFourCorner	21911
U+7DCB.0	fHakka	 fui1; fi1
U+7DCB.0	fCantonese	fei1
U+7DCB.0	fMandarin	fei1
U+7DCB.0	fEnglish	scarlet; crimson
U+7DCC.0	fUTF8	緌
U+7DCC.0	fCNS11643	2-4C76
U+7DCC.0	fBig5	E36F
U+7DCC.0	fGB	3-6C7D
U+7DCC.0	fR/S	 120.8
U+7DCC.0	fTotalStrokes	 14
U+7DCC.0	fCangjie	VFHDV
U+7DCC.0	fFourCorner	22944
U+7DCC.0	fHakka	  sui1
U+7DCC.0	fCantonese	jeoi4; jeoi5; jeoi6
U+7DCC.0	fMandarin	rui2
U+7DCC.0	fEnglish	ornamental stringson a hat
U+7DCD.0	fUTF8	緍
U+7DCD.0	fCNS11643	3-4871
U+7DCD.0	fGB	5-6C38
U+7DCD.0	fR/S	 120.8
U+7DCD.0	fTotalStrokes	 14
U+7DCD.0	fCangjie	VFHPA
U+7DCD.0	fFourCorner	22960
U+7DCD.0	fHakka	 min2
U+7DCD.0	fCantonese	man4
U+7DCD.0	fMandarin	min2
U+7DCD.0	fEnglish	variant of &#x7DE1;
U+7DCE.0	fUTF8	緎
U+7DCE.0	fCNS11643	2-4C72
U+7DCE.0	fBig5	E36B
U+7DCE.0	fGB	3-6C7A
U+7DCE.0	fR/S	 120.8
U+7DCE.0	fTotalStrokes	 14
U+7DCE.0	fCangjie	VFIRM
U+7DCE.0	fFourCorner	23950
U+7DCE.0	fHakka	 wet7; vet7
U+7DCE.0	fCantonese	wik6
U+7DCE.0	fMandarin	yu4; xu4
U+7DCE.0	fEnglish	seam
U+7DCF.0	fUTF8	総
U+7DCF.0	fCNS11643	3-4873
U+7DCF.0	fGB	E-383F
U+7DCF.0	fR/S	  120.8
U+7DCF.0	fTotalStrokes	 14
U+7DCF.0	fCangjie	VFCIP
U+7DCF.0	fFourCorner	28930
U+7DCF.0	fHakka	 zung3
U+7DCF.0	fCantonese	zung2
U+7DCF.0	fMandarin	cong1; zong3
U+7DCF.0	fEnglish	 collect overall; altogether
U+7DD0.0	fUTF8	緐
U+7DD0.0	fCNS11643	3-6629
U+7DD0.0	fGB	E-3840
U+7DD0.0	fR/S	 120.7; 120.8
U+7DD0.0	fTotalStrokes	 13
U+7DD0.0	fCangjie	OYHVF
U+7DD0.0	fCantonese	po4; faan4
U+7DD0.0	fMandarin	fan2; pan2; po2
U+7DD1.0	fUTF8	緑
U+7DD1.0	fCNS11643	3-486B
U+7DD1.0	fGB	1-424C
U+7DD1.0	fR/S	 120.8
U+7DD1.0	fTotalStrokes	 14
U+7DD1.0	fCangjie	VFNME
U+7DD1.0	fFourCorner	27930
U+7DD1.0	fCantonese	luk6
U+7DD1.0	fMandarin	luu4; l4; lu4
U+7DD1.0	fEnglish	 green chlorine
U+7DD2.0	fUTF8	緒
U+7DD2.0	fCNS11643	1-6A45
U+7DD2.0	fBig5	BAFC
U+7DD2.0	fGB	1-5077
U+7DD2.0	fR/S	 120.8
U+7DD2.0	fTotalStrokes	 14
U+7DD2.0	fCangjie	VFJKA
U+7DD2.0	fFourCorner	24960
U+7DD2.0	fHakka	  si5
U+7DD2.0	fCantonese	seoi5
U+7DD2.0	fMandarin	xu4
U+7DD2.0	fEnglish	[1] end of a thread or string [2] clue [3] beginning [4] task; cause; enterprise [5] mental or emotional state; mood [6] remnants; remains; leftovers
U+7DD3.0	fUTF8	緓
U+7DD3.0	fCNS11643	3-662B
U+7DD3.0	fGB	E-3841
U+7DD3.0	fR/S	 120.9
U+7DD3.0	fTotalStrokes	 15
U+7DD3.0	fCangjie	VFTLK
U+7DD3.0	fFourCorner	2493
U+7DD3.0	fCantonese	jing1
U+7DD3.0	fMandarin	ying1
U+7DD4.0	fUTF8	緔
U+7DD4.0	fGB	1-6734
U+7DD4.0	fR/S	 120.8
U+7DD4.0	fTotalStrokes	 14
U+7DD4.0	fCangjie	VFFBR
U+7DD4.0	fCantonese	soeng5
U+7DD4.0	fMandarin	shang4; zhang4
U+7DD5.0	fUTF8	緕
U+7DD5.0	fGB	E-3842
U+7DD5.0	fR/S	 120.8
U+7DD5.0	fTotalStrokes	 14
U+7DD5.0	fCangjie	VFYKL
U+7DD5.0	fCantonese	cai4
U+7DD6.0	fUTF8	緖
U+7DD6.0	fCNS11643	3-4E22
U+7DD6.0	fGB	E-3843
U+7DD6.0	fR/S	 120.9
U+7DD6.0	fTotalStrokes	 15
U+7DD6.0	fCangjie	VFJKA
U+7DD6.0	fFourCorner	24960
U+7DD6.0	fHakka	 si5
U+7DD6.0	fCantonese	seoi5
U+7DD6.0	fMandarin	xu4
U+7DD6.0	fEnglish	 end of thread thread; clue
U+7DD7.0	fUTF8	緗
U+7DD7.0	fCNS11643	2-526F
U+7DD7.0	fBig5	E6E7
U+7DD7.0	fGB	1-673D
U+7DD7.0	fR/S	 120.9
U+7DD7.0	fTotalStrokes	 15
U+7DD7.0	fCangjie	VFDBU
U+7DD7.0	fFourCorner	26900
U+7DD7.0	fHakka	  siong1
U+7DD7.0	fCantonese	soeng1
U+7DD7.0	fMandarin	xiang1; shuang1
U+7DD7.0	fEnglish	light-yellow silk
U+7DD8.0	fUTF8	緘
U+7DD8.0	fCNS11643	1-6E3B
U+7DD8.0	fBig5	BD70
U+7DD8.0	fGB	1-3C6A
U+7DD8.0	fR/S	 120.9
U+7DD8.0	fTotalStrokes	 15
U+7DD8.0	fCangjie	VFIHR
U+7DD8.0	fFourCorner	23950
U+7DD8.0	fHakka	 kiam2; ham1
U+7DD8.0	fCantonese	zin1 (also); gaam1
U+7DD8.0	fMandarin	jian1; jian4
U+7DD8.0	fEnglish	[1] seal; close [2] letter
U+7DD9.0	fUTF8	緙
U+7DD9.0	fCNS11643	1-6E44
U+7DD9.0	fBig5	BD79
U+7DD9.0	fGB	1-673C
U+7DD9.0	fR/S	 120.9
U+7DD9.0	fTotalStrokes	 15
U+7DD9.0	fCangjie	VFTLJ
U+7DD9.0	fFourCorner	24956
U+7DD9.0	fHakka	 giet7; get7
U+7DD9.0	fCantonese	kaak1; kaak3 (also); hak1
U+7DD9.0	fMandarin	ke4
U+7DD9.0	fEnglish	 woof of a woven item
U+7DDA.0	fUTF8	線
U+7DDA.0	fCNS11643	1-6E40
U+7DDA.0	fBig5	BD75
U+7DDA.0	fGB	E-3844
U+7DDA.0	fR/S	 120.9
U+7DDA.0	fTotalStrokes	 15
U+7DDA.0	fCangjie	VFHAE
U+7DDA.0	fFourCorner	26932
U+7DDA.0	fHakka	  sien5; sen5; sian5; sien3
U+7DDA.0	fCantonese	sin3; sin2 (also)
U+7DDA.0	fMandarin	xian4
U+7DDA.0	fEnglish	[1] thread; wire [2] line [3] wires
U+7DDB.0	fUTF8	緛
U+7DDB.0	fCNS11643	2-526C
U+7DDB.0	fBig5	E6E4
U+7DDB.0	fGB	3-6D2C
U+7DDB.0	fR/S	 120.9
U+7DDB.0	fTotalStrokes	 15
U+7DDB.0	fCangjie	VFMBK
U+7DDB.0	fFourCorner	21934
U+7DDB.0	fCantonese	jyun5
U+7DDB.0	fMandarin	ruan3; ruan4
U+7DDC.0	fUTF8	緜
U+7DDC.0	fCNS11643	3-4D7C
U+7DDC.0	fGB	E-3845
U+7DDC.0	fR/S	 120.9
U+7DDC.0	fTotalStrokes	 15
U+7DDC.0	fCangjie	HBHVF
U+7DDC.0	fFourCorner	22290
U+7DDC.0	fHakka	 men2
U+7DDC.0	fCantonese	min4
U+7DDC.0	fMandarin	mian2
U+7DDC.0	fEnglish	[1] soft; downy [2] enduring
U+7DDD.0	fUTF8	緝
U+7DDD.0	fCNS11643	1-6E3D
U+7DDD.0	fBig5	BD72
U+7DDD.0	fGB	1-3C29
U+7DDD.0	fR/S	 120.9
U+7DDD.0	fTotalStrokes	 15
U+7DDD.0	fCangjie	VFRSJ
U+7DDD.0	fFourCorner	26941
U+7DDD.0	fHakka	  cip8; sip8; cap8
U+7DDD.0	fCantonese	cap1
U+7DDD.0	fMandarin	ji1; ji2; qi4
U+7DDD.0	fEnglish	[1] order the arrest of; arrest; capture [2] twist and join (cords) [3] continue [4] hem clothing
U+7DDD.1	fUTF8	緝
U+7DDD.1	fCNS11643	1-6E3D
U+7DDD.1	fBig5	BD72
U+7DDD.1	fGB	1-3C29
U+7DDD.1	fR/S	 120.9
U+7DDD.1	fTotalStrokes	 15
U+7DDD.1	fCangjie	VFRSJ
U+7DDD.1	fFourCorner	26941
U+7DDD.1	fMandarin	qi1
U+7DDD.1	fEnglish	 sew in close stitches and straight stitches
U+7DDE.0	fUTF8	緞
U+7DDE.0	fCNS11643	1-6E41
U+7DDE.0	fBig5	BD76
U+7DDE.0	fGB	1-3650
U+7DDE.0	fR/S	 120.9
U+7DDE.0	fTotalStrokes	 15
U+7DDE.0	fCangjie	VFHJE
U+7DDE.0	fFourCorner	27947
U+7DDE.0	fHakka	 ton5; ton6; ton3; don5
U+7DDE.0	fCantonese	dyun6; dyun2 (also)
U+7DDE.0	fMandarin	duan4
U+7DDE.0	fEnglish	satin
U+7DDF.0	fUTF8	緟
U+7DDF.0	fCNS11643	2-5278
U+7DDF.0	fBig5	E6F0
U+7DDF.0	fGB	5-6C48
U+7DDF.0	fR/S	 120.9
U+7DDF.0	fTotalStrokes	 15
U+7DDF.0	fCangjie	VFHJG
U+7DDF.0	fFourCorner	22914
U+7DDF.0	fCantonese	cung5; cung4; zung6
U+7DDF.0	fMandarin	zhong4; chong2
U+7DE0.0	fUTF8	締
U+7DE0.0	fCNS11643	1-6E37
U+7DE0.0	fBig5	BD6C
U+7DE0.0	fGB	1-355E
U+7DE0.0	fR/S	 120.9
U+7DE0.0	fTotalStrokes	 15
U+7DE0.0	fCangjie	VFYBB
U+7DE0.0	fFourCorner	20927
U+7DE0.0	fHakka	 ti5;  di5
U+7DE0.0	fCantonese	dai3; tai3 (also); dai6
U+7DE0.0	fMandarin	di4; ti2; zhi4
U+7DE0.0	fEnglish	 tie; join; connect; connect; unite
U+7DE1.0	fUTF8	緡
U+7DE1.0	fCNS11643	2-5270
U+7DE1.0	fBig5	E6E8
U+7DE1.0	fGB	1-6745
U+7DE1.0	fR/S	 120.9
U+7DE1.0	fTotalStrokes	 15
U+7DE1.0	fCangjie	VFRPA
U+7DE1.0	fFourCorner	27964
U+7DE1.0	fHakka	  min2;  men2
U+7DE1.0	fCantonese	man4; fan1
U+7DE1.0	fMandarin	min2; min3; mian2; hun1; hun2
U+7DE1.0	fEnglish	[1] fishing-line [2] cords for stringing up coins
U+7DE2.0	fUTF8	緢
U+7DE2.0	fCNS11643	3-4D78
U+7DE2.0	fGB	3-6C78
U+7DE2.0	fR/S	  120.9; 120.8
U+7DE2.0	fTotalStrokes	 15
U+7DE2.0	fCangjie	VFTW
U+7DE2.0	fFourCorner	24960
U+7DE2.0	fHakka	 miau2; meu2; miau3
U+7DE2.0	fCantonese	miu4
U+7DE2.0	fMandarin	miao2
U+7DE2.0	fEnglish	[1] wind off silk [2] fringe on a flag
U+7DE3.0	fUTF8	緣
U+7DE3.0	fCNS11643	1-6E3F
U+7DE3.0	fBig5	BD74
U+7DE3.0	fGB	1-5435
U+7DE3.0	fR/S	 120.9
U+7DE3.0	fTotalStrokes	 15
U+7DE3.0	fCangjie	VFVNO
U+7DE3.0	fFourCorner	27932
U+7DE3.0	fHakka	 jen2; ren2; jan3; jan2; jan5
U+7DE3.0	fCantonese	jyun4; jyun6
U+7DE3.0	fMandarin	yuan2; yung4; yuan4
U+7DE3.0	fEnglish	[1] cause; reason [2] go along; follow [3] hem; margin; edge; fringe [4] relationship by fate; predestined relationship
U+7DE4.0	fUTF8	緤
U+7DE4.0	fCNS11643	3-4E24
U+7DE4.0	fGB	5-6C40
U+7DE4.0	fR/S	 120.9
U+7DE4.0	fTotalStrokes	 15
U+7DE4.0	fCangjie	VFPTD
U+7DE4.0	fFourCorner	24990
U+7DE4.0	fHakka	  siet7
U+7DE4.0	fCantonese	sit3
U+7DE4.0	fMandarin	xie4
U+7DE4.0	fEnglish	cord
U+7DE5.0	fUTF8	緥
U+7DE5.0	fCNS11643	3-4E2F
U+7DE5.0	fGB	E-3846
U+7DE5.0	fR/S	 120.9
U+7DE5.0	fTotalStrokes	 15
U+7DE5.0	fCangjie	VFORD
U+7DE5.0	fFourCorner	26990
U+7DE5.0	fHakka	  bau3; bo3
U+7DE5.0	fCantonese	bou2
U+7DE5.0	fMandarin	bao3
U+7DE5.0	fEnglish	swaddling
U+7DE6.0	fUTF8	緦
U+7DE6.0	fCNS11643	2-5273
U+7DE6.0	fBig5	E6EB
U+7DE6.0	fGB	1-6741
U+7DE6.0	fR/S	 120.9
U+7DE6.0	fTotalStrokes	 15
U+7DE6.0	fCangjie	VFWP
U+7DE6.0	fFourCorner	26930
U+7DE6.0	fHakka	 sii1; se1
U+7DE6.0	fCantonese	si1
U+7DE6.0	fMandarin	si1
U+7DE6.0	fEnglish	[1] fine jute cloth [2] spin jute thread or yarn
U+7DE7.0	fUTF8	緧
U+7DE7.0	fCNS11643	2-526E
U+7DE7.0	fBig5	E6E6
U+7DE7.0	fGB	5-6C4E
U+7DE7.0	fR/S	 120.9
U+7DE7.0	fTotalStrokes	 15
U+7DE7.0	fCangjie	VFTCW
U+7DE7.0	fFourCorner	28961
U+7DE7.0	fHakka	 ciu1; ciu2
U+7DE7.0	fCantonese	cau1
U+7DE7.0	fMandarin	qiu1
U+7DE7.0	fEnglish	 put on a crupper
U+7DE8.0	fUTF8	編
U+7DE8.0	fCNS11643	1-6E3E
U+7DE8.0	fBig5	BD73
U+7DE8.0	fGB	1-3160
U+7DE8.0	fR/S	 120.9
U+7DE8.0	fTotalStrokes	 15
U+7DE8.0	fCangjie	VFHSB
U+7DE8.0	fFourCorner	23927
U+7DE8.0	fHakka	  pien1;  pin1; pen1; pian1
U+7DE8.0	fCantonese	pin1
U+7DE8.0	fMandarin	bian1; bian3; bian4
U+7DE8.0	fEnglish	[1] knit; weave [2] put together; organise [3] fabricate; make up; invent [4] arrange; compile; edit [5] (part of) a book; volume
U+7DE9.0	fUTF8	緩
U+7DE9.0	fCNS11643	1-6E42
U+7DE9.0	fBig5	BD77
U+7DE9.0	fGB	1-3B3A
U+7DE9.0	fR/S	 120.9
U+7DE9.0	fTotalStrokes	 15
U+7DE9.0	fCangjie	VFBME
U+7DE9.0	fFourCorner	22947
U+7DE9.0	fHakka	 fon5; fon3; man5; non1; won3; fan6; fan5; man6; mon3; man3
U+7DE9.0	fCantonese	wun6; wun4 (also)
U+7DE9.0	fMandarin	huan3; kuan3; xuan3
U+7DE9.0	fEnglish	[1] slow; gradual; tardy; leisurely; unhurried [2] postpone; delay; slacken; put off; defer [3] revive; refresh
U+7DEA.0	fUTF8	緪
U+7DEA.0	fCNS11643	2-526D
U+7DEA.0	fBig5	E6E5
U+7DEA.0	fGB	E-3847
U+7DEA.0	fR/S	 120.9
U+7DEA.0	fTotalStrokes	 15
U+7DEA.0	fCangjie	VFPMM
U+7DEA.0	fFourCorner	21912
U+7DEA.0	fHakka	 hen2
U+7DEA.0	fCantonese	gang1; gang2 (also); gang3
U+7DEA.0	fMandarin	geng1; geng4
U+7DEA.0	fEnglish	 rope
U+7DEB.0	fUTF8	緫
U+7DEB.0	fCNS11643	3-4E2E
U+7DEB.0	fGB	E-3848
U+7DEB.0	fR/S	  120.9
U+7DEB.0	fTotalStrokes	 15
U+7DEB.0	fCangjie	VFPKP
U+7DEB.0	fFourCorner	27930
U+7DEB.0	fHakka	 cung1
U+7DEB.0	fCantonese	zung2
U+7DEB.0	fMandarin	cong1; zong1; zong3
U+7DEB.0	fEnglish	dark green
U+7DEC.0	fUTF8	緬
U+7DEC.0	fCNS11643	1-6E3C
U+7DEC.0	fBig5	BD71
U+7DEC.0	fGB	1-4365
U+7DEC.0	fR/S	 120.9
U+7DEC.0	fTotalStrokes	 15
U+7DEC.0	fCangjie	VFMWL
U+7DEC.0	fFourCorner	21960
U+7DEC.0	fHakka	  mien3; men3; mian3
U+7DEC.0	fCantonese	min5
U+7DEC.0	fMandarin	mian3
U+7DEC.0	fEnglish	[1] distant; far; remote [2] think of something or somebody in the past
U+7DED.0	fUTF8	緭
U+7DED.0	fCNS11643	3-4D77
U+7DED.0	fGB	3-6D31
U+7DED.0	fR/S	  120.9
U+7DED.0	fTotalStrokes	 15
U+7DED.0	fCangjie	VFWB
U+7DED.0	fFourCorner	26920
U+7DED.0	fCantonese	wai6
U+7DED.0	fMandarin	wei4
U+7DEE.0	fUTF8	緮
U+7DEE.0	fCNS11643	2-5277
U+7DEE.0	fBig5	E6EF
U+7DEE.0	fGB	5-6C49
U+7DEE.0	fR/S	 120.9
U+7DEE.0	fTotalStrokes	 15
U+7DEE.0	fCangjie	VFOAE
U+7DEE.0	fFourCorner	28947
U+7DEE.0	fCantonese	fuk6; fau6
U+7DEE.0	fMandarin	fu4
U+7DEF.0	fUTF8	緯
U+7DEF.0	fCNS11643	1-6E39
U+7DEF.0	fBig5	BD6E
U+7DEF.0	fGB	1-4E33
U+7DEF.0	fR/S	 120.9
U+7DEF.0	fTotalStrokes	 15
U+7DEF.0	fCangjie	VFDMQ
U+7DEF.0	fFourCorner	24956
U+7DEF.0	fHakka	  wui3; wi3; we5; we6; vui3; ve5; voi5
U+7DEF.0	fCantonese	wai5; wai6
U+7DEF.0	fMandarin	wei3; wei4
U+7DEF.0	fEnglish	[1] woof [2] parallels showing latitude on a map [3] books about charms, omens, etc. circulated as appendices to the classics
U+7DF0.0	fUTF8	緰
U+7DF0.0	fCNS11643	2-5276
U+7DF0.0	fBig5	E6EE
U+7DF0.0	fGB	3-6D33
U+7DF0.0	fR/S	 120.9
U+7DF0.0	fTotalStrokes	 15
U+7DF0.0	fCangjie	VFOMN
U+7DF0.0	fFourCorner	28921
U+7DF0.0	fCantonese	tau4; seoi1; syu1
U+7DF0.0	fMandarin	yu2; tou2; xu1; yao2
U+7DF0.0	fEnglish	net
U+7DF1.0	fUTF8	緱
U+7DF1.0	fCNS11643	2-5275
U+7DF1.0	fBig5	E6ED
U+7DF1.0	fGB	1-6743
U+7DF1.0	fR/S	 120.9
U+7DF1.0	fTotalStrokes	 15
U+7DF1.0	fCangjie	VFONK
U+7DF1.0	fFourCorner	27934
U+7DF1.0	fHakka	 kieu1; geu1
U+7DF1.0	fCantonese	gau1; kau1
U+7DF1.0	fMandarin	gou1; kou1
U+7DF1.0	fEnglish	cord binding on the hilt of a sword
U+7DF2.0	fUTF8	緲
U+7DF2.0	fCNS11643	1-6E45
U+7DF2.0	fBig5	BD7A
U+7DF2.0	fGB	1-673F
U+7DF2.0	fR/S	 120.9
U+7DF2.0	fTotalStrokes	 15
U+7DF2.0	fCangjie	VFBUH
U+7DF2.0	fFourCorner	29920
U+7DF2.0	fHakka	 miau3; meu3; miau1
U+7DF2.0	fCantonese	miu5
U+7DF2.0	fMandarin	miao3
U+7DF2.0	fEnglish	distant; far and dim; unsubstantial
U+7DF3.0	fUTF8	緳
U+7DF3.0	fCNS11643	2-503B
U+7DF3.0	fBig5	E572
U+7DF3.0	fGB	E-3849
U+7DF3.0	fR/S	 120.9
U+7DF3.0	fTotalStrokes	 15
U+7DF3.0	fCangjie	IQHF
U+7DF3.0	fFourCorner	00293
U+7DF3.0	fCantonese	kit3
U+7DF3.0	fMandarin	xie2; jie2
U+7DF4.0	fUTF8	練
U+7DF4.0	fCNS11643	1-6E38
U+7DF4.0	fBig5	BD6D
U+7DF4.0	fGB	1-4137
U+7DF4.0	fR/S	 120.9; 120.8
U+7DF4.0	fTotalStrokes	 15
U+7DF4.0	fCangjie	VFDWF
U+7DF4.0	fFourCorner	25996
U+7DF4.0	fHakka	  lien5; len3; lien6; lian6
U+7DF4.0	fCantonese	lin6
U+7DF4.0	fMandarin	lian4
U+7DF4.0	fEnglish	[1] practice; drill; exercise; train [2] skilled; experienced [3] soften and whiten raw silk by boiling
U+7DF5.0	fUTF8	緵
U+7DF5.0	fCNS11643	3-4E2B
U+7DF5.0	fGB	3-6D34
U+7DF5.0	fR/S	 120.9
U+7DF5.0	fTotalStrokes	 15
U+7DF5.0	fCangjie	VFUCE
U+7DF5.0	fFourCorner	22940
U+7DF5.0	fCantonese	zung1
U+7DF5.0	fMandarin	zong1; zong4
U+7DF5.0	fEnglish	net
U+7DF6.0	fUTF8	緶
U+7DF6.0	fCNS11643	2-5274
U+7DF6.0	fBig5	E6EC
U+7DF6.0	fGB	1-6742
U+7DF6.0	fR/S	 120.9
U+7DF6.0	fTotalStrokes	 15
U+7DF6.0	fCangjie	VFOMK
U+7DF6.0	fFourCorner	21946
U+7DF6.0	fHakka	 pien1;  bien1
U+7DF6.0	fCantonese	pin4; bin1
U+7DF6.0	fMandarin	bian4; pian2; bian3; bian1
U+7DF6.0	fEnglish	[1] narrow strip of woven material; (such as the hem of a straw hat) [2] sew up a hem
U+7DF7.0	fUTF8	緷
U+7DF7.0	fCNS11643	2-526B
U+7DF7.0	fBig5	E6E3
U+7DF7.0	fGB	3-6C65
U+7DF7.0	fR/S	 120.9
U+7DF7.0	fTotalStrokes	 15
U+7DF7.0	fCangjie	VFBJJ
U+7DF7.0	fFourCorner	27950
U+7DF7.0	fHakka	 gun3
U+7DF7.0	fCantonese	wan6; gwan2; kwan2
U+7DF7.0	fMandarin	yun4; gun3; hun2; hun4; juan4
U+7DF8.0	fUTF8	緸
U+7DF8.0	fCNS11643	3-4D75
U+7DF8.0	fGB	5-6C42
U+7DF8.0	fR/S	  120.9
U+7DF8.0	fTotalStrokes	 15
U+7DF8.0	fCangjie	VFMWG
U+7DF8.0	fFourCorner	21910
U+7DF8.0	fCantonese	jan1
U+7DF8.0	fMandarin	yin1
U+7DF9.0	fUTF8	緹
U+7DF9.0	fCNS11643	1-6E46
U+7DF9.0	fBig5	BD7B
U+7DF9.0	fGB	1-673E
U+7DF9.0	fR/S	 120.9
U+7DF9.0	fTotalStrokes	 15
U+7DF9.0	fCangjie	VFAMO
U+7DF9.0	fFourCorner	24981
U+7DF9.0	fHakka	 ti2; ti3
U+7DF9.0	fCantonese	tai4
U+7DF9.0	fMandarin	ti2; di1; ti3
U+7DF9.0	fEnglish	[1] reddish color; red [2] reddish yellow silk [3] reddish yellow soil
U+7DFA.0	fUTF8	緺
U+7DFA.0	fCNS11643	2-5272
U+7DFA.0	fBig5	E6EA
U+7DFA.0	fGB	3-6C6B
U+7DFA.0	fR/S	 120.9
U+7DFA.0	fTotalStrokes	 15
U+7DFA.0	fCangjie	VFBBR
U+7DFA.0	fFourCorner	27927
U+7DFA.0	fCantonese	gwaa1; waa1; gwo1; gwaa3
U+7DFA.0	fMandarin	gua1; guo1; luo2; wo1
U+7DFB.0	fUTF8	緻
U+7DFB.0	fCNS11643	1-6E3A
U+7DFB.0	fBig5	BD6F
U+7DFB.0	fGB	1-7924
U+7DFB.0	fR/S	 120.9
U+7DFB.0	fTotalStrokes	 15
U+7DFB.0	fCangjie	VFMGK
U+7DFB.0	fFourCorner	28940
U+7DFB.0	fHakka	 zhi5; zi5; zih5; tje5
U+7DFB.0	fCantonese	zi3
U+7DFB.0	fMandarin	zhi4
U+7DFB.0	fEnglish	delicate; fine; dense; close
U+7DFC.0	fUTF8	緼
U+7DFC.0	fCNS11643	3-4E28
U+7DFC.0	fGB	3-6D2E
U+7DFC.0	fR/S	 120.9
U+7DFC.0	fTotalStrokes	 15
U+7DFC.0	fCangjie	VFABT
U+7DFC.0	fFourCorner	26910
U+7DFC.0	fHakka	 wun1
U+7DFC.0	fCantonese	wan3; wan1
U+7DFC.0	fMandarin	yun1; yun4; wen1
U+7DFC.0	fEnglish	variant of &#x7E15 confused;
U+7DFD.0	fUTF8	緽
U+7DFD.0	fCNS11643	3-4E29
U+7DFD.0	fGB	5-6B72
U+7DFD.0	fR/S	  120.9
U+7DFD.0	fTotalStrokes	 15
U+7DFD.0	fCangjie	VFYBC
U+7DFD.0	fFourCorner	21980
U+7DFD.0	fCantonese	cing1
U+7DFD.0	fMandarin	cheng1
U+7DFE.0	fUTF8	緾
U+7DFE.0	fCNS11643	3-4E25
U+7DFE.0	fGB	5-6C43
U+7DFE.0	fR/S	 120.9
U+7DFE.0	fTotalStrokes	 15
U+7DFE.0	fCangjie	VFMWG
U+7DFE.0	fFourCorner	21910
U+7DFE.0	fHakka	 chan2
U+7DFE.0	fCantonese	cin4
U+7DFE.0	fMandarin	chan2
U+7DFF.0	fUTF8	緿
U+7DFF.0	fCNS11643	3-4E2C
U+7DFF.0	fGB	5-6C54
U+7DFF.0	fR/S	  120.9
U+7DFF.0	fTotalStrokes	 15
U+7DFF.0	fCangjie	VFIRP
U+7DFF.0	fFourCorner	23930
U+7DFF.0	fCantonese	doi6
U+7DFF.0	fMandarin	dai4
U+7E00.0	fUTF8	縀
U+7E00.0	fCNS11643	3-4E27
U+7E00.0	fGB	5-6C52
U+7E00.0	fR/S	 120.9
U+7E00.0	fTotalStrokes	 15
U+7E00.0	fCangjie	VFRSE
U+7E00.0	fFourCorner	27940
U+7E00.0	fCantonese	dyun6
U+7E00.0	fMandarin	xia2
U+7E01.0	fUTF8	縁
U+7E01.0	fCNS11643	3-4D7E
U+7E01.0	fGB	E-384A
U+7E01.0	fR/S	 120.9
U+7E01.0	fTotalStrokes	 15
U+7E01.0	fCangjie	VFNMO
U+7E01.0	fFourCorner	27930
U+7E01.0	fCantonese	jyun4
U+7E01.0	fMandarin	yuan2
U+7E01.0	fEnglish	hem
U+7E02.0	fUTF8	縂
U+7E02.0	fCNS11643	3-4D7D
U+7E02.0	fGB	E-384B
U+7E02.0	fR/S	 120.9
U+7E02.0	fTotalStrokes	 15
U+7E02.0	fCangjie	VFCRP
U+7E02.0	fFourCorner	28930
U+7E02.0	fCantonese	zung2
U+7E02.0	fMandarin	zong3; zong1
U+7E02.0	fEnglish	 collect overall; altogether
U+7E03.0	fUTF8	縃
U+7E03.0	fCNS11643	2-5271
U+7E03.0	fBig5	E6E9
U+7E03.0	fGB	E-384C
U+7E03.0	fR/S	 120.9
U+7E03.0	fTotalStrokes	 15
U+7E03.0	fCangjie	VFNOB
U+7E03.0	fFourCorner	27927
U+7E03.0	fCantonese	seoi1
U+7E03.0	fMandarin	xu1; shu1; su4; xu3
U+7E04.0	fUTF8	縄
U+7E04.0	fCNS11643	4-4C45
U+7E04.0	fGB	E-384D
U+7E04.0	fR/S	 120.9
U+7E04.0	fTotalStrokes	 15
U+7E04.0	fCangjie	VFWLU
U+7E04.0	fFourCorner	26910
U+7E04.0	fCantonese	sing4
U+7E04.0	fMandarin	sheng2; min3
U+7E04.0	fEnglish	 rope, string, cord; control
U+7E05.0	fUTF8	縅
U+7E05.0	fGB	E-384E
U+7E05.0	fR/S	 120.9
U+7E05.0	fTotalStrokes	 15
U+7E05.0	fCangjie	VFIHV
U+7E05.0	fFourCorner	2395
U+7E05.0	fCantonese	wai1
U+7E05.0	fEnglish	threaten; intimidate
U+7E06.0	fUTF8	縆
U+7E06.0	fGB	5-6C51
U+7E06.0	fR/S	 120.9
U+7E06.0	fTotalStrokes	 15
U+7E06.0	fCangjie	XVFPM
U+7E06.0	fHakka	 hen2; kwen3
U+7E06.0	fCantonese	gang1; gang2 (also)
U+7E06.0	fMandarin	geng1
U+7E06.0	fEnglish	 rope
U+7E07.0	fUTF8	縇
U+7E07.0	fCNS11643	3-662A
U+7E07.0	fGB	1-7D4C
U+7E07.0	fR/S	 120.9
U+7E07.0	fTotalStrokes	 15
U+7E07.0	fCangjie	VFJMM
U+7E07.0	fFourCorner	2391
U+7E07.0	fCantonese	syun1
U+7E07.0	fEnglish	decoration
U+7E08.0	fUTF8	縈
U+7E08.0	fCNS11643	1-716A
U+7E08.0	fBig5	BFA2
U+7E08.0	fGB	1-5D53
U+7E08.0	fR/S	 120.10
U+7E08.0	fTotalStrokes	 16
U+7E08.0	fCangjie	FFBVF
U+7E08.0	fFourCorner	99903
U+7E08.0	fHakka	 jin2; rin2; rang2; jang2; jang1; jung2
U+7E08.0	fCantonese	jing4
U+7E08.0	fMandarin	ying2
U+7E08.0	fEnglish	[1] entwine; coil; wind around and around; round; twine [2] entangle; preoccupy; bog
U+7E09.0	fUTF8	縉
U+7E09.0	fCNS11643	1-716F
U+7E09.0	fBig5	BFA7
U+7E09.0	fGB	1-6746
U+7E09.0	fR/S	 120.10
U+7E09.0	fTotalStrokes	 16
U+7E09.0	fCangjie	VFMIA
U+7E09.0	fFourCorner	21961
U+7E09.0	fHakka	  zin5; zin1; zin6
U+7E09.0	fCantonese	zeon3
U+7E09.0	fMandarin	jin4; zi1
U+7E09.0	fEnglish	red silk
U+7E0A.0	fUTF8	縊
U+7E0A.0	fCNS11643	1-7168
U+7E0A.0	fBig5	BF7E
U+7E0A.0	fGB	1-674B
U+7E0A.0	fR/S	 120.10
U+7E0A.0	fTotalStrokes	 16
U+7E0A.0	fCangjie	VFTCT
U+7E0A.0	fFourCorner	28917
U+7E0A.0	fHakka	 jit7; ai5; ji5; jit8; rit7; jak7
U+7E0A.0	fCantonese	ai3; ngai3 (also)
U+7E0A.0	fMandarin	yi4
U+7E0A.0	fEnglish	 hang; strangle
U+7E0B.0	fUTF8	縋
U+7E0B.0	fCNS11643	2-5944
U+7E0B.0	fBig5	EAD8
U+7E0B.0	fGB	1-6744
U+7E0B.0	fR/S	 120.10
U+7E0B.0	fTotalStrokes	 16
U+7E0B.0	fCangjie	VFYHR
U+7E0B.0	fFourCorner	27937
U+7E0B.0	fHakka	 ch'ui6; cui5; chui5; ch'ui5
U+7E0B.0	fCantonese	zeoi6
U+7E0B.0	fMandarin	zhui4
U+7E0B.0	fEnglish	 hang by a rope; let down by a rope
U+7E0C.0	fUTF8	縌
U+7E0C.0	fCNS11643	2-593B
U+7E0C.0	fBig5	EACF
U+7E0C.0	fGB	5-6C50
U+7E0C.0	fR/S	 120.10
U+7E0C.0	fTotalStrokes	 16
U+7E0C.0	fCangjie	VFYTU
U+7E0C.0	fFourCorner	28934
U+7E0C.0	fCantonese	jik6
U+7E0C.0	fMandarin	ni4
U+7E0D.0	fUTF8	縍
U+7E0D.0	fCNS11643	2-5947
U+7E0D.0	fBig5	EADB
U+7E0D.0	fGB	5-6C5B
U+7E0D.0	fR/S	 120.10
U+7E0D.0	fTotalStrokes	 16
U+7E0D.0	fCangjie	VFYBS
U+7E0D.0	fFourCorner	20927
U+7E0D.0	fHakka	  bong3
U+7E0D.0	fCantonese	bong1; bong2
U+7E0D.0	fMandarin	bang1; bang4
U+7E0D.0	fEnglish	 bind shoes
U+7E0E.0	fUTF8	縎
U+7E0E.0	fCNS11643	2-593F
U+7E0E.0	fBig5	EAD3
U+7E0E.0	fGB	3-6D32
U+7E0E.0	fR/S	 120.10
U+7E0E.0	fTotalStrokes	 16
U+7E0E.0	fCangjie	VFBBB
U+7E0E.0	fFourCorner	27927
U+7E0E.0	fHakka	 gut7; guk7; gwut7
U+7E0E.0	fCantonese	gwat1
U+7E0E.0	fMandarin	gu3; hu2
U+7E0E.0	fEnglish	[1] a kind of silk [2] tangled; knotted
U+7E0F.0	fUTF8	縏
U+7E0F.0	fCNS11643	2-5945
U+7E0F.0	fBig5	EAD9
U+7E0F.0	fGB	3-6C2A
U+7E0F.0	fR/S	 120.10
U+7E0F.0	fTotalStrokes	 16
U+7E0F.0	fCangjie	HEVIF
U+7E0F.0	fFourCorner	27903
U+7E0F.0	fCantonese	pun4
U+7E0F.0	fMandarin	pan2
U+7E0F.0	fEnglish	purse
U+7E10.0	fUTF8	縐
U+7E10.0	fCNS11643	1-7170
U+7E10.0	fBig5	BFA8
U+7E10.0	fGB	1-6727
U+7E10.0	fR/S	 120.10
U+7E10.0	fTotalStrokes	 16
U+7E10.0	fCangjie	VFPUU
U+7E10.0	fFourCorner	27927
U+7E10.0	fHakka	  ziu5; tsieu5
U+7E10.0	fCantonese	zau3
U+7E10.0	fMandarin	zhou4; chao4; zhu4; cu4; zhou1
U+7E10.0	fEnglish	[1] crepe; crape [2] wrinkled; creased; crinkled
U+7E11.0	fUTF8	縑
U+7E11.0	fCNS11643	1-7169
U+7E11.0	fBig5	BFA1
U+7E11.0	fGB	1-674C
U+7E11.0	fR/S	 120.10
U+7E11.0	fTotalStrokes	 16
U+7E11.0	fCangjie	VFTXC
U+7E11.0	fFourCorner	28937
U+7E11.0	fHakka	 giam1
U+7E11.0	fCantonese	gim1
U+7E11.0	fMandarin	jian1
U+7E11.0	fEnglish	a kind of fine silk
U+7E12.0	fUTF8	縒
U+7E12.0	fCNS11643	2-5938
U+7E12.0	fBig5	EACC
U+7E12.0	fGB	5-6C4D
U+7E12.0	fR/S	 120.10
U+7E12.0	fTotalStrokes	 16
U+7E12.0	fCangjie	VFTQM
U+7E12.0	fFourCorner	28911
U+7E12.0	fCantonese	ci1; cok3
U+7E12.0	fMandarin	cuo3; ci1; cuo4; suo3
U+7E12.0	fEnglish	twist
U+7E13.0	fUTF8	縓
U+7E13.0	fCNS11643	2-593E
U+7E13.0	fBig5	EAD2
U+7E13.0	fGB	3-6D37
U+7E13.0	fR/S	 120.10
U+7E13.0	fTotalStrokes	 16
U+7E13.0	fCangjie	VFMHF
U+7E13.0	fFourCorner	21996
U+7E13.0	fHakka	 ngien2; cien5; con5; ngian2; con1; can1
U+7E13.0	fCantonese	cyun4; cyun3 (also); jyun4
U+7E13.0	fMandarin	quan4; quan2; quan3
U+7E13.0	fEnglish	orange or reddish-yellow silk
U+7E14.0	fUTF8	縔
U+7E14.0	fCNS11643	2-5948
U+7E14.0	fBig5	EADC
U+7E14.0	fGB	5-6C5E
U+7E14.0	fR/S	 120.10
U+7E14.0	fTotalStrokes	 16
U+7E14.0	fCangjie	VFEED
U+7E14.0	fFourCorner	27994
U+7E14.0	fCantonese	song2
U+7E14.0	fMandarin	shuang3
U+7E15.0	fUTF8	縕
U+7E15.0	fCNS11643	2-5941
U+7E15.0	fBig5	EAD5
U+7E15.0	fGB	E-384F
U+7E15.0	fR/S	 120.10
U+7E15.0	fTotalStrokes	 16
U+7E15.0	fCangjie	VFWOT
U+7E15.0	fFourCorner	26917
U+7E15.0	fHakka	 wun6; wun1
U+7E15.0	fCantonese	wan3; wan1; wan2
U+7E15.0	fMandarin	yun1; yun4; wen1; yun3
U+7E15.0	fEnglish	[1] loose hemp; old yarn [2] confused; chaotic
U+7E16.0	fUTF8	縖
U+7E16.0	fCNS11643	2-5946
U+7E16.0	fBig5	EADA
U+7E16.0	fGB	5-6C5D
U+7E16.0	fR/S	 120.10
U+7E16.0	fTotalStrokes	 16
U+7E16.0	fCangjie	VFJQR
U+7E16.0	fFourCorner	23961
U+7E16.0	fCantonese	hat6
U+7E16.0	fMandarin	xia2
U+7E17.0	fUTF8	縗
U+7E17.0	fCNS11643	2-593A
U+7E17.0	fBig5	EACE
U+7E17.0	fGB	5-6C5A
U+7E17.0	fR/S	 120.10
U+7E17.0	fTotalStrokes	 16
U+7E17.0	fCangjie	VFYWV
U+7E17.0	fFourCorner	20932
U+7E17.0	fHakka	  cui1;  soi1
U+7E17.0	fCantonese	ceoi1
U+7E17.0	fMandarin	cui1; shuai1; sui1
U+7E17.0	fEnglish	a piece of sackcloth worn on the breast during mourning
U+7E18.0	fUTF8	縘
U+7E18.0	fCNS11643	4-524C
U+7E18.0	fGB	E-3850
U+7E18.0	fR/S	 120.10
U+7E18.0	fTotalStrokes	 16
U+7E18.0	fCangjie	VFBVK
U+7E18.0	fFourCorner	22930
U+7E18.0	fHakka	 he5
U+7E18.0	fCantonese	hai6
U+7E18.0	fMandarin	xi1
U+7E19.0	fUTF8	縙
U+7E19.0	fCNS11643	3-523B
U+7E19.0	fGB	3-6D2B
U+7E19.0	fR/S	  120.10
U+7E19.0	fTotalStrokes	 16
U+7E19.0	fCangjie	VFTSJ
U+7E19.0	fFourCorner	24940
U+7E19.0	fCantonese	jung4
U+7E19.0	fMandarin	rong2
U+7E1A.0	fUTF8	縚
U+7E1A.0	fCNS11643	2-5942
U+7E1A.0	fBig5	EAD6
U+7E1A.0	fGB	E-3851
U+7E1A.0	fR/S	 120.10
U+7E1A.0	fTotalStrokes	 16
U+7E1A.0	fCangjie	VFBHX
U+7E1A.0	fFourCorner	22977
U+7E1A.0	fHakka	  tau1; to1
U+7E1A.0	fCantonese	tou1
U+7E1A.0	fMandarin	tao1
U+7E1A.0	fEnglish	band
U+7E1B.0	fUTF8	縛
U+7E1B.0	fCNS11643	1-716B
U+7E1B.0	fBig5	BFA3
U+7E1B.0	fGB	1-383F
U+7E1B.0	fR/S	 120.10
U+7E1B.0	fTotalStrokes	 16
U+7E1B.0	fCangjie	VFIBI
U+7E1B.0	fFourCorner	23942
U+7E1B.0	fHakka	 p'iok8; bok7; bong3; piok7
U+7E1B.0	fCantonese	bok3; fok3 (also); bok6
U+7E1B.0	fMandarin	fu2; fu4; fo4
U+7E1B.0	fEnglish	 tie; bind
U+7E1C.0	fUTF8	縜
U+7E1C.0	fCNS11643	2-5940
U+7E1C.0	fBig5	EAD4
U+7E1C.0	fGB	3-6C6A
U+7E1C.0	fR/S	 120.10
U+7E1C.0	fTotalStrokes	 16
U+7E1C.0	fCangjie	VFRBC
U+7E1C.0	fFourCorner	26986
U+7E1C.0	fCantonese	wan4; gwan1
U+7E1C.0	fMandarin	yun2; yun3
U+7E1D.0	fUTF8	縝
U+7E1D.0	fCNS11643	1-716E
U+7E1D.0	fBig5	BFA6
U+7E1D.0	fGB	1-6747
U+7E1D.0	fR/S	 120.10
U+7E1D.0	fTotalStrokes	 16
U+7E1D.0	fCangjie	VFJBC
U+7E1D.0	fFourCorner	24981
U+7E1D.0	fHakka	 zhin3; zihn3; zin3
U+7E1D.0	fCantonese	zan2; can2 (also)
U+7E1D.0	fMandarin	chen1; zhen3; zhen1
U+7E1D.0	fEnglish	detailed; fine; close; minute
U+7E1E.0	fUTF8	縞
U+7E1E.0	fCNS11643	1-716D
U+7E1E.0	fBig5	BFA5
U+7E1E.0	fGB	1-6749
U+7E1E.0	fR/S	 120.10
U+7E1E.0	fTotalStrokes	 16
U+7E1E.0	fCangjie	VFYRB
U+7E1E.0	fFourCorner	20927
U+7E1E.0	fHakka	 gau3
U+7E1E.0	fCantonese	gou2
U+7E1E.0	fMandarin	gao3; gao4
U+7E1E.0	fEnglish	plain white raw silk
U+7E1F.0	fUTF8	縟
U+7E1F.0	fCNS11643	2-593C
U+7E1F.0	fBig5	EAD0
U+7E1F.0	fGB	1-6748
U+7E1F.0	fR/S	 120.10
U+7E1F.0	fTotalStrokes	 16
U+7E1F.0	fCangjie	VFMVI
U+7E1F.0	fFourCorner	21943
U+7E1F.0	fHakka	 juk8; ruk8
U+7E1F.0	fCantonese	juk6
U+7E1F.0	fMandarin	ru4; rong3
U+7E1F.0	fEnglish	[1] rich ornament; decorative [2] excessive formality or ceremony
U+7E20.0	fUTF8	縠
U+7E20.0	fCNS11643	2-593D
U+7E20.0	fBig5	EAD1
U+7E20.0	fGB	3-4F3A
U+7E20.0	fR/S	 120.10
U+7E20.0	fTotalStrokes	 16
U+7E20.0	fCangjie	GFHNE
U+7E20.0	fFourCorner	47947
U+7E20.0	fHakka	 guk7; kwuk7
U+7E20.0	fCantonese	huk6
U+7E20.0	fMandarin	hu2
U+7E20.0	fEnglish	crepe
U+7E21.0	fUTF8	縡
U+7E21.0	fCNS11643	2-5939
U+7E21.0	fBig5	EACD
U+7E21.0	fGB	3-6D3A
U+7E21.0	fR/S	 120.10
U+7E21.0	fTotalStrokes	 16
U+7E21.0	fCangjie	VFJYJ
U+7E21.0	fFourCorner	23941
U+7E21.0	fHakka	  zai3;  zai5
U+7E21.0	fCantonese	zoi2; zoi3 (also)
U+7E21.0	fMandarin	zai3; zai4
U+7E21.0	fEnglish	matter; affair
U+7E22.0	fUTF8	縢
U+7E22.0	fCNS11643	2-5943
U+7E22.0	fBig5	EAD7
U+7E22.0	fGB	3-4D78
U+7E22.0	fR/S	 120.10
U+7E22.0	fTotalStrokes	 16
U+7E22.0	fCangjie	BFQF
U+7E22.0	fFourCorner	79293
U+7E22.0	fHakka	  ten2
U+7E22.0	fCantonese	tang4
U+7E22.0	fMandarin	teng2
U+7E22.0	fEnglish	[1] bind; tie; restrict; restrain [2] band
U+7E23.0	fUTF8	縣
U+7E23.0	fCNS11643	1-716C
U+7E23.0	fBig5	BFA4
U+7E23.0	fGB	1-4F58
U+7E23.0	fR/S	 120.10
U+7E23.0	fTotalStrokes	 16
U+7E23.0	fCangjie	BFHVF
U+7E23.0	fFourCorner	62993
U+7E23.0	fHakka	 hien5; jen3; jen5; ren5; jan6; hen2; jan5
U+7E23.0	fCantonese	jyun6; jyun4; jyun2 (also)
U+7E23.0	fMandarin	xuan2
U+7E23.0	fEnglish	 county; prefecture
U+7E23.1	fUTF8	縣
U+7E23.1	fCNS11643	1-716C
U+7E23.1	fBig5	BFA4
U+7E23.1	fGB	1-4F58
U+7E23.1	fR/S	 120.10
U+7E23.1	fTotalStrokes	 16
U+7E23.1	fCangjie	BFHVF
U+7E23.1	fFourCorner	62993
U+7E23.1	fMandarin	xian4
U+7E23.1	fEnglish	 hang
U+7E24.0	fUTF8	縤
U+7E24.0	fCNS11643	2-594A
U+7E24.0	fBig5	EADE
U+7E24.0	fGB	5-6C55
U+7E24.0	fR/S	 120.10
U+7E24.0	fTotalStrokes	 16
U+7E24.0	fCangjie	VFQMF
U+7E24.0	fFourCorner	25993
U+7E24.0	fCantonese	sou3
U+7E24.0	fMandarin	su4
U+7E25.0	fUTF8	縥
U+7E25.0	fCNS11643	2-5949
U+7E25.0	fBig5	EADD
U+7E25.0	fGB	E-3852
U+7E25.0	fR/S	 120.10
U+7E25.0	fTotalStrokes	 16
U+7E25.0	fCangjie	VFQKD
U+7E25.0	fFourCorner	24994
U+7E25.0	fCantonese	zeon2; zan2
U+7E25.0	fMandarin	zhen3
U+7E26.0	fUTF8	縦
U+7E26.0	fCNS11643	3-5240
U+7E26.0	fGB	E-3853
U+7E26.0	fR/S	 120.10
U+7E26.0	fTotalStrokes	 16
U+7E26.0	fCangjie	VFHOO
U+7E26.0	fFourCorner	28980
U+7E26.0	fCantonese	zung3
U+7E26.0	fMandarin	zong4; zong3; cong2; zong1
U+7E26.0	fEnglish	indulge in; give free reign to
U+7E27.0	fUTF8	縧
U+7E27.0	fCNS11643	3-523F
U+7E27.0	fGB	1-4C50
U+7E27.0	fR/S	 120.10; 120.11
U+7E27.0	fTotalStrokes	 16; 17
U+7E27.0	fCangjie	VFOLD
U+7E27.0	fFourCorner	28990
U+7E27.0	fCantonese	tou1
U+7E27.0	fMandarin	tao1
U+7E27.0	fEnglish	ribbon; flat silk cord
U+7E28.0	fUTF8	縨
U+7E28.0	fGB	E-3854
U+7E28.0	fR/S	 120.10
U+7E28.0	fTotalStrokes	 16
U+7E28.0	fCangjie	VFAFU
U+7E28.0	fCantonese	fong2
U+7E28.0	fMandarin	huang3
U+7E29.0	fUTF8	縩
U+7E29.0	fCNS11643	2-5E47
U+7E29.0	fBig5	EDDA
U+7E29.0	fGB	3-6D3E
U+7E29.0	fR/S	 120.11
U+7E29.0	fTotalStrokes	 17
U+7E29.0	fCangjie	VFBOF
U+7E29.0	fFourCorner	27991
U+7E29.0	fHakka	  cai5; ze5
U+7E29.0	fCantonese	coi3; zoi3
U+7E29.0	fMandarin	cai4
U+7E29.0	fEnglish	new of fresh-looking clothes
U+7E2A.0	fUTF8	縪
U+7E2A.0	fCNS11643	2-5E43
U+7E2A.0	fBig5	EDD6
U+7E2A.0	fGB	3-6C56
U+7E2A.0	fR/S	 120.11
U+7E2A.0	fTotalStrokes	 17
U+7E2A.0	fCangjie	VFWTJ
U+7E2A.0	fFourCorner	26944
U+7E2A.0	fHakka	 bit7
U+7E2A.0	fCantonese	bat1
U+7E2A.0	fMandarin	bi4; bie4
U+7E2A.0	fEnglish	[1] fringe [2] knee-pad
U+7E2B.0	fUTF8	縫
U+7E2B.0	fCNS11643	1-7469
U+7E2B.0	fBig5	C15F
U+7E2B.0	fGB	1-376C
U+7E2B.0	fR/S	 120.11
U+7E2B.0	fTotalStrokes	 17
U+7E2B.0	fCangjie	VFYHJ
U+7E2B.0	fFourCorner	27934
U+7E2B.0	fHakka	  fung2; pung2; fung3; pung5; pung6; pung3
U+7E2B.0	fCantonese	fung4; fung6; fung2 (also)
U+7E2B.0	fMandarin	feng4
U+7E2B.0	fEnglish	[1] suture [2] crack; chink; fissure; opening; crevice
U+7E2B.1	fUTF8	縫
U+7E2B.1	fCNS11643	1-7469
U+7E2B.1	fBig5	C15F
U+7E2B.1	fGB	1-376C
U+7E2B.1	fR/S	 120.11
U+7E2B.1	fTotalStrokes	 17
U+7E2B.1	fCangjie	VFYHJ
U+7E2B.1	fFourCorner	27934
U+7E2B.1	fMandarin	feng2
U+7E2B.1	fEnglish	[1] sew; mend; stitch [2] suture
U+7E2C.0	fUTF8	縬
U+7E2C.0	fCNS11643	3-5579
U+7E2C.0	fGB	5-6C61
U+7E2C.0	fR/S	  120.11
U+7E2C.0	fTotalStrokes	 17
U+7E2C.0	fCangjie	VFIHF
U+7E2C.0	fFourCorner	23950
U+7E2C.0	fCantonese	zuk1
U+7E2C.0	fMandarin	cu4
U+7E2D.0	fUTF8	縭
U+7E2D.0	fCNS11643	2-5E3D
U+7E2D.0	fBig5	EDD0
U+7E2D.0	fGB	1-674A
U+7E2D.0	fR/S	 120.11
U+7E2D.0	fTotalStrokes	 17
U+7E2D.0	fCangjie	VFYUB
U+7E2D.0	fFourCorner	20927
U+7E2D.0	fHakka	  li2
U+7E2D.0	fCantonese	lei4
U+7E2D.0	fMandarin	li2; chi1
U+7E2D.0	fEnglish	[1] bridal veil; bride's sash [2] tie; bind
U+7E2E.0	fUTF8	縮
U+7E2E.0	fCNS11643	1-7463
U+7E2E.0	fBig5	C159
U+7E2E.0	fGB	1-4B75
U+7E2E.0	fR/S	 120.11
U+7E2E.0	fTotalStrokes	 17
U+7E2E.0	fCangjie	VFJOA
U+7E2E.0	fFourCorner	23961
U+7E2E.0	fHakka	  suk7; siuk7; su2; sok7; so2; sio2
U+7E2E.0	fCantonese	suk1
U+7E2E.0	fMandarin	suo1; su4
U+7E2E.0	fEnglish	[1] contract; draw in; reduce; decrease; shorten; shrink [2] draw back; recoil; wince
U+7E2F.0	fUTF8	縯
U+7E2F.0	fCNS11643	1-7473
U+7E2F.0	fBig5	C169
U+7E2F.0	fGB	3-6D42
U+7E2F.0	fR/S	 120.11
U+7E2F.0	fTotalStrokes	 17
U+7E2F.0	fCangjie	VFJMC
U+7E2F.0	fFourCorner	23986
U+7E2F.0	fHakka	 jen5; ren5; rin3; jin3; jan5
U+7E2F.0	fCantonese	jin2; jan5; jin5; jin3
U+7E2F.0	fMandarin	yin3; yan3; yan4
U+7E2F.0	fEnglish	long
U+7E30.0	fUTF8	縰
U+7E30.0	fCNS11643	2-5E49
U+7E30.0	fBig5	EDDC
U+7E30.0	fGB	3-6D3D
U+7E30.0	fR/S	 120.11
U+7E30.0	fTotalStrokes	 17
U+7E30.0	fCangjie	VFHOO
U+7E30.0	fFourCorner	21981
U+7E30.0	fHakka	  sai3
U+7E30.0	fCantonese	saai2 (also); si2
U+7E30.0	fMandarin	xi3
U+7E30.0	fEnglish	band for the hair
U+7E31.0	fUTF8	縱
U+7E31.0	fCNS11643	1-746B
U+7E31.0	fBig5	C161
U+7E31.0	fGB	1-575D
U+7E31.0	fR/S	 120.11
U+7E31.0	fTotalStrokes	 17
U+7E31.0	fCangjie	VFHOO
U+7E31.0	fFourCorner	28981
U+7E31.0	fHakka	 ziung1;  zung1; ziung3; zung3; ziung6; ziung5
U+7E31.0	fCantonese	zung3; zung1; zung2
U+7E31.0	fMandarin	zong4; zong3; cong2; song3
U+7E31.0	fEnglish	[1] indulge; indulgence [2] allow to move or work freely; let go; let fly; give free reign to [3] even if; although
U+7E31.1	fUTF8	縱
U+7E31.1	fCNS11643	1-746B
U+7E31.1	fBig5	C161
U+7E31.1	fGB	1-575D
U+7E31.1	fR/S	 120.11
U+7E31.1	fTotalStrokes	 17
U+7E31.1	fCangjie	VFHOO
U+7E31.1	fFourCorner	28981
U+7E31.1	fMandarin	zong1
U+7E31.1	fEnglish	vertical; longitudinal
U+7E32.0	fUTF8	縲
U+7E32.0	fCNS11643	1-7467
U+7E32.0	fBig5	C15D
U+7E32.0	fGB	1-6750
U+7E32.0	fR/S	 120.11
U+7E32.0	fTotalStrokes	 17
U+7E32.0	fCangjie	VFWVF
U+7E32.0	fFourCorner	26993
U+7E32.0	fHakka	  lui3;  lui2;  lui1; lui5
U+7E32.0	fCantonese	leoi4
U+7E32.0	fMandarin	lei2; lei3; lei4; luo2
U+7E32.0	fEnglish	black rope (used to bind a criminal)
U+7E33.0	fUTF8	縳
U+7E33.0	fCNS11643	2-5E40
U+7E33.0	fBig5	EDD3
U+7E33.0	fGB	3-6C34
U+7E33.0	fR/S	 120.11
U+7E33.0	fTotalStrokes	 17
U+7E33.0	fCangjie	VFJII
U+7E33.0	fFourCorner	25943
U+7E33.0	fHakka	 chon5; gien3; p'iok8; con5
U+7E33.0	fCantonese	gyun3; zyun3
U+7E33.0	fMandarin	zhuan4; chuan2
U+7E33.0	fEnglish	wrap
U+7E34.0	fUTF8	縴
U+7E34.0	fCNS11643	1-746E
U+7E34.0	fBig5	C164
U+7E34.0	fGB	1-7863
U+7E34.0	fR/S	 120.11
U+7E34.0	fTotalStrokes	 17
U+7E34.0	fCangjie	VFYVQ
U+7E34.0	fFourCorner	20953
U+7E34.0	fHakka	 kien1; ken1
U+7E34.0	fCantonese	hin3; hin1 (also)
U+7E34.0	fMandarin	qian1; qian4; nian4
U+7E34.0	fEnglish	 tow rope; tow line
U+7E35.0	fUTF8	縵
U+7E35.0	fCNS11643	1-7471
U+7E35.0	fBig5	C167
U+7E35.0	fGB	1-674F
U+7E35.0	fR/S	 120.11
U+7E35.0	fTotalStrokes	 17
U+7E35.0	fCangjie	VFAWE
U+7E35.0	fFourCorner	26947
U+7E35.0	fHakka	 man5; man6; man3
U+7E35.0	fCantonese	maan6
U+7E35.0	fMandarin	man4; man2
U+7E35.0	fEnglish	[1] plain silk [2] simple; plain; unadorned [3] slow
U+7E36.0	fUTF8	縶
U+7E36.0	fCNS11643	2-5E4B
U+7E36.0	fBig5	EDDE
U+7E36.0	fGB	1-746A
U+7E36.0	fR/S	 120.11
U+7E36.0	fTotalStrokes	 17
U+7E36.0	fCangjie	GIVIF
U+7E36.0	fFourCorner	45903
U+7E36.0	fHakka	 chip7; tsiip7; zip7; tje5; zhi5
U+7E36.0	fCantonese	zap1
U+7E36.0	fMandarin	zhi2; zhe2
U+7E36.0	fEnglish	[1] tie; bind; connect [2] imprison [3] bridle
U+7E37.0	fUTF8	縷
U+7E37.0	fCNS11643	1-7466
U+7E37.0	fBig5	C15C
U+7E37.0	fGB	1-4246
U+7E37.0	fR/S	 120.11
U+7E37.0	fTotalStrokes	 17
U+7E37.0	fCangjie	VFLLV
U+7E37.0	fFourCorner	25944
U+7E37.0	fHakka	  leu3; liu3; lui3; liu1
U+7E37.0	fCantonese	leoi5; lau5 (also); lau2 (also)
U+7E37.0	fMandarin	l3; luu3; lou2; lu3
U+7E37.0	fEnglish	[1] thread; yarn [2] wisp; strand; lock [3] detailed; precise
U+7E38.0	fUTF8	縸
U+7E38.0	fCNS11643	2-5E42
U+7E38.0	fBig5	EDD5
U+7E38.0	fGB	3-6D36
U+7E38.0	fR/S	 120.11
U+7E38.0	fTotalStrokes	 17
U+7E38.0	fCangjie	VFTAK
U+7E38.0	fFourCorner	24934
U+7E38.0	fCantonese	mou6; mok6
U+7E38.0	fMandarin	mo4; mu4
U+7E39.0	fUTF8	縹
U+7E39.0	fCNS11643	1-746F
U+7E39.0	fBig5	C165
U+7E39.0	fGB	1-674E
U+7E39.0	fR/S	 120.11
U+7E39.0	fTotalStrokes	 17
U+7E39.0	fCangjie	VFMWF
U+7E39.0	fFourCorner	21991
U+7E39.0	fHakka	 piau5; piau1
U+7E39.0	fCantonese	piu5; piu1; piu2
U+7E39.0	fMandarin	piao3; piao1; piao4
U+7E39.0	fEnglish	[1] light blue silk [2] light blue [3] dim; misty; indistinct
U+7E3A.0	fUTF8	縺
U+7E3A.0	fCNS11643	2-5E4D
U+7E3A.0	fBig5	EDE0
U+7E3A.0	fGB	3-6C68
U+7E3A.0	fR/S	 120.11
U+7E3A.0	fTotalStrokes	 17
U+7E3A.0	fCangjie	VFYJJ
U+7E3A.0	fFourCorner	25930
U+7E3A.0	fHakka	 lien2
U+7E3A.0	fCantonese	lin4
U+7E3A.0	fMandarin	lian2
U+7E3A.0	fEnglish	to tangle, knot, kink, get twisted; to get complicated
U+7E3B.0	fUTF8	縻
U+7E3B.0	fCNS11643	2-5E4A
U+7E3B.0	fBig5	EDDD
U+7E3B.0	fGB	0-7763
U+7E3B.0	fR/S	 120.11
U+7E3B.0	fTotalStrokes	 17
U+7E3B.0	fCangjie	IDVIF
U+7E3B.0	fFourCorner	00293
U+7E3B.0	fHakka	 mi2
U+7E3B.0	fCantonese	mei4
U+7E3B.0	fMandarin	mi2; mei2; mi4
U+7E3B.0	fEnglish	 tie fasten; connect
U+7E3C.0	fUTF8	縼
U+7E3C.0	fCNS11643	2-5E3E
U+7E3C.0	fBig5	EDD1
U+7E3C.0	fGB	3-6D3F
U+7E3C.0	fR/S	 120.11
U+7E3C.0	fTotalStrokes	 17
U+7E3C.0	fCangjie	VFYSO
U+7E3C.0	fFourCorner	28981
U+7E3C.0	fHakka	  sien5
U+7E3C.0	fCantonese	syun6; syun3
U+7E3C.0	fMandarin	xuan4
U+7E3C.0	fEnglish	 rope for tethering cattle
U+7E3D.0	fUTF8	總
U+7E3D.0	fCNS11643	1-746A
U+7E3D.0	fBig5	C160
U+7E3D.0	fGB	1-575C
U+7E3D.0	fR/S	 120.11
U+7E3D.0	fTotalStrokes	 17
U+7E3D.0	fCangjie	VFHWP
U+7E3D.0	fFourCorner	26930
U+7E3D.0	fHakka	  zung3; zung1
U+7E3D.0	fCantonese	zung2
U+7E3D.0	fMandarin	zong3; zong1; cong1
U+7E3D.0	fEnglish	[1] gather; assemble; unite; collect [2] always [3] all; overall; altogether; complete; total; general [4] chief; principal; central [5] at any rate; in any event; anyway; after all; sooner or later
U+7E3E.0	fUTF8	績
U+7E3E.0	fCNS11643	1-7464
U+7E3E.0	fBig5	C15A
U+7E3E.0	fGB	1-3C28
U+7E3E.0	fR/S	 120.11
U+7E3E.0	fTotalStrokes	 17
U+7E3E.0	fCangjie	VFQMC
U+7E3E.0	fFourCorner	25986
U+7E3E.0	fHakka	  ziak7;  zit7; ciak8
U+7E3E.0	fCantonese	zik1; zek3
U+7E3E.0	fMandarin	ji1
U+7E3E.0	fEnglish	[1] spin; twist [2] merit; achievements; exploits; meritorious labor
U+7E3F.0	fUTF8	縿
U+7E3F.0	fCNS11643	1-7472
U+7E3F.0	fBig5	C168
U+7E3F.0	fGB	3-6D29
U+7E3F.0	fR/S	 120.11
U+7E3F.0	fTotalStrokes	 17
U+7E3F.0	fCangjie	VFIIH
U+7E3F.0	fFourCorner	23922
U+7E3F.0	fCantonese	saam1; caam1; siu1
U+7E3F.0	fMandarin	shan1; can3; sao1; xian1; xiao1
U+7E3F.0	fEnglish	streamer
U+7E40.0	fUTF8	繀
U+7E40.0	fCNS11643	2-5E45
U+7E40.0	fBig5	EDD8
U+7E40.0	fGB	3-6D3C
U+7E40.0	fR/S	 120.11
U+7E40.0	fTotalStrokes	 17
U+7E40.0	fCangjie	VFUOG
U+7E40.0	fFourCorner	22914
U+7E40.0	fHakka	  sui5; cui5
U+7E40.0	fCantonese	seoi3; seoi6
U+7E40.0	fMandarin	sui4
U+7E40.0	fEnglish	 spinning-wheel
U+7E41.0	fUTF8	繁
U+7E41.0	fCNS11643	1-746D
U+7E41.0	fBig5	C163
U+7E41.0	fGB	0-3731
U+7E41.0	fR/S	 120.11; 120.10
U+7E41.0	fTotalStrokes	 17
U+7E41.0	fCangjie	OKVIF
U+7E41.0	fFourCorner	88903
U+7E41.0	fHakka	  fan2
U+7E41.0	fCantonese	faan4; po4; pun4
U+7E41.0	fMandarin	fan2; pan2
U+7E41.0	fEnglish	[1] many; numerous; abundant; prolific [2] complicated; complex; intricate
U+7E41.1	fUTF8	繁
U+7E41.1	fCNS11643	1-746D
U+7E41.1	fBig5	C163
U+7E41.1	fGB	0-3731
U+7E41.1	fR/S	 120.11; 120.10
U+7E41.1	fTotalStrokes	 17
U+7E41.1	fCangjie	OKVIF
U+7E41.1	fFourCorner	88903
U+7E41.1	fMandarin	po2
U+7E41.1	fEnglish	a Chinese family name
U+7E42.0	fUTF8	繂
U+7E42.0	fCNS11643	2-5E3F
U+7E42.0	fBig5	EDD2
U+7E42.0	fGB	3-6D40
U+7E42.0	fR/S	 120.11
U+7E42.0	fTotalStrokes	 17
U+7E42.0	fCangjie	VFYIJ
U+7E42.0	fFourCorner	20943
U+7E42.0	fHakka	  lut8
U+7E42.0	fCantonese	leot6
U+7E42.0	fMandarin	shuai4
U+7E42.0	fEnglish	bamboo rope
U+7E43.0	fUTF8	繃
U+7E43.0	fCNS11643	1-7468
U+7E43.0	fBig5	C15E
U+7E43.0	fGB	E-3855
U+7E43.0	fR/S	 120.11
U+7E43.0	fTotalStrokes	 17
U+7E43.0	fCangjie	VFUBB
U+7E43.0	fFourCorner	22927
U+7E43.0	fHakka	 ben1; peu5; bong5
U+7E43.0	fCantonese	bang1; maang1 (also); mang2; baang6; mang1; bang6
U+7E43.0	fMandarin	beng1
U+7E43.0	fEnglish	 bind; strap
U+7E43.1	fUTF8	繃
U+7E43.1	fCNS11643	1-7468
U+7E43.1	fBig5	C15E
U+7E43.1	fGB	E-3855
U+7E43.1	fR/S	 120.11
U+7E43.1	fTotalStrokes	 17
U+7E43.1	fCangjie	VFUBB
U+7E43.1	fFourCorner	22927
U+7E43.1	fMandarin	beng3
U+7E43.1	fEnglish	[1] taut; tense [2] endure; bear
U+7E43.2	fUTF8	繃
U+7E43.2	fCNS11643	1-7468
U+7E43.2	fBig5	C15E
U+7E43.2	fGB	E-3855
U+7E43.2	fR/S	 120.11
U+7E43.2	fTotalStrokes	 17
U+7E43.2	fCangjie	VFUBB
U+7E43.2	fFourCorner	22927
U+7E43.2	fMandarin	beng4
U+7E43.2	fEnglish	 break open; burst open; split open; crack
U+7E44.0	fUTF8	繄
U+7E44.0	fCNS11643	2-5E4C
U+7E44.0	fBig5	EDDF
U+7E44.0	fGB	3-6C2B
U+7E44.0	fR/S	 120.11
U+7E44.0	fTotalStrokes	 17
U+7E44.0	fCangjie	SEVIF
U+7E44.0	fFourCorner	77903
U+7E44.0	fHakka	 ji1; ri1; i1; ji5
U+7E44.0	fCantonese	ji1; ai1; ngai1
U+7E44.0	fMandarin	yi1; yi4
U+7E44.0	fEnglish	[1] a phrase particle in archaic Chinese [2] be (in archaic Chinese)
U+7E45.0	fUTF8	繅
U+7E45.0	fCNS11643	1-746C
U+7E45.0	fBig5	C162
U+7E45.0	fGB	1-6752
U+7E45.0	fR/S	 120.11
U+7E45.0	fTotalStrokes	 17
U+7E45.0	fCangjie	VFVVD
U+7E45.0	fFourCorner	22994
U+7E45.0	fHakka	 sau1;  cau2; cau3
U+7E45.0	fCantonese	sou1
U+7E45.0	fMandarin	sao1; zao3
U+7E45.0	fEnglish	 draw; unwind (silk from cocoons)
U+7E46.0	fUTF8	繆
U+7E46.0	fCNS11643	1-7465
U+7E46.0	fBig5	C15B
U+7E46.0	fGB	1-6751
U+7E46.0	fR/S	 120.11
U+7E46.0	fTotalStrokes	 17
U+7E46.0	fCangjie	VFSMH
U+7E46.0	fFourCorner	27922
U+7E46.0	fHakka	  meu2;  miau3;  muk8; miu2; miau2; meu3; miu3; meu5; muk7
U+7E46.0	fCantonese	mau4; mau6; miu6; muk6; maau6 (also)
U+7E46.0	fMandarin	mou2; jiu1; liao3; liao2; liao4; miao2
U+7E46.0	fEnglish	precautions; preparations
U+7E46.1	fUTF8	繆
U+7E46.1	fCNS11643	1-7465
U+7E46.1	fBig5	C15B
U+7E46.1	fGB	1-6751
U+7E46.1	fR/S	 120.11
U+7E46.1	fTotalStrokes	 17
U+7E46.1	fCangjie	VFSMH
U+7E46.1	fFourCorner	27922
U+7E46.1	fMandarin	miao4
U+7E46.1	fEnglish	a Chinese family name
U+7E46.2	fUTF8	繆
U+7E46.2	fCNS11643	1-7465
U+7E46.2	fBig5	C15B
U+7E46.2	fGB	1-6751
U+7E46.2	fR/S	 120.11
U+7E46.2	fTotalStrokes	 17
U+7E46.2	fCangjie	VFSMH
U+7E46.2	fFourCorner	27922
U+7E46.2	fMandarin	miu4
U+7E46.2	fEnglish	erroneous; preposterous; absurd; false; error; mistake
U+7E46.3	fUTF8	繆
U+7E46.3	fCNS11643	1-7465
U+7E46.3	fBig5	C15B
U+7E46.3	fGB	1-6751
U+7E46.3	fR/S	 120.11
U+7E46.3	fTotalStrokes	 17
U+7E46.3	fCangjie	VFSMH
U+7E46.3	fFourCorner	27922
U+7E46.3	fMandarin	mu4
U+7E46.3	fEnglish	[1] beautiful [2] harmonious [3] silent
U+7E47.0	fUTF8	繇
U+7E47.0	fCNS11643	2-5E46
U+7E47.0	fBig5	EDD9
U+7E47.0	fGB	0-746D
U+7E47.0	fR/S	 120.11
U+7E47.0	fTotalStrokes	 17
U+7E47.0	fCangjie	BUHVF
U+7E47.0	fFourCorner	22793
U+7E47.0	fHakka	  ju2; jau2; rau2; jeu2; riu2; jiu2
U+7E47.0	fCantonese	jau4; jiu4; zau6
U+7E47.0	fMandarin	you2; you1; zhou4; yao1
U+7E47.0	fEnglish	[1] same as &#x7531; [2] same as &#x60A0;
U+7E47.1	fUTF8	繇
U+7E47.1	fCNS11643	2-5E46
U+7E47.1	fBig5	EDD9
U+7E47.1	fGB	0-746D
U+7E47.1	fR/S	 120.11
U+7E47.1	fTotalStrokes	 17
U+7E47.1	fCangjie	BUHVF
U+7E47.1	fFourCorner	22793
U+7E47.1	fMandarin	yao2
U+7E47.1	fEnglish	[1] labor service [2] luxuriant
U+7E48.0	fUTF8	繈
U+7E48.0	fCNS11643	1-7470
U+7E48.0	fBig5	C166
U+7E48.0	fGB	E-3856
U+7E48.0	fR/S	 120.12
U+7E48.0	fTotalStrokes	 18
U+7E48.0	fCangjie	VFNII
U+7E48.0	fFourCorner	23936
U+7E48.0	fHakka	  giong3
U+7E48.0	fCantonese	koeng5
U+7E48.0	fMandarin	qiang3; jiang3
U+7E48.0	fEnglish	 string of copper coins swaddling clothes
U+7E49.0	fUTF8	繉
U+7E49.0	fCNS11643	2-5E44
U+7E49.0	fBig5	EDD7
U+7E49.0	fGB	E-3857
U+7E49.0	fR/S	 120.11
U+7E49.0	fTotalStrokes	 17
U+7E49.0	fCangjie	VFBUO
U+7E49.0	fFourCorner	26913
U+7E49.0	fCantonese	wan4
U+7E49.0	fMandarin	hun2
U+7E4A.0	fUTF8	繊
U+7E4A.0	fGB	E-3858
U+7E4A.0	fR/S	 120.11
U+7E4A.0	fTotalStrokes	 17
U+7E4A.0	fCangjie	VFJIC
U+7E4A.0	fCantonese	cim1
U+7E4A.0	fMandarin	xian1
U+7E4A.0	fEnglish	 fine, delicate; minute; graceful
U+7E4B.0	fUTF8	繋
U+7E4B.0	fGB	E-3859
U+7E4B.0	fR/S	 120.13
U+7E4B.0	fTotalStrokes	 19
U+7E4B.0	fCangjie	XJEVI
U+7E4B.0	fCantonese	hai6
U+7E4B.0	fMandarin	xi4; ji4
U+7E4B.0	fEnglish	attach; connect; unite; fasten
U+7E4C.0	fUTF8	繌
U+7E4C.0	fCNS11643	2-5E48
U+7E4C.0	fBig5	EDDB
U+7E4C.0	fGB	E-385A
U+7E4C.0	fR/S	 120.11
U+7E4C.0	fTotalStrokes	 17
U+7E4C.0	fCangjie	VFHCE
U+7E4C.0	fFourCorner	26943
U+7E4C.0	fCantonese	zung1
U+7E4D.0	fUTF8	繍
U+7E4D.0	fCNS11643	3-557A
U+7E4D.0	fGB	E-385B
U+7E4D.0	fR/S	 120.11
U+7E4D.0	fTotalStrokes	 17
U+7E4D.0	fCangjie	VFLLL
U+7E4D.0	fFourCorner	25920
U+7E4D.0	fCantonese	sau3
U+7E4D.0	fMandarin	xiu4
U+7E4D.0	fEnglish	 embroider embroidery ornament
U+7E4E.0	fUTF8	繎
U+7E4E.0	fCNS11643	4-5C5F
U+7E4E.0	fGB	5-6C69
U+7E4E.0	fR/S	  120.12
U+7E4E.0	fTotalStrokes	 18
U+7E4E.0	fCangjie	VFBKF
U+7E4E.0	fFourCorner	23930
U+7E4E.0	fHakka	 jen2; ren2
U+7E4E.0	fCantonese	jin4
U+7E4E.0	fMandarin	ran2
U+7E4E.0	fEnglish	[1] red silk [2] tangled silk threads
U+7E4F.0	fUTF8	繏
U+7E4F.0	fCNS11643	3-5859
U+7E4F.0	fGB	3-6D49
U+7E4F.0	fR/S	 120.12
U+7E4F.0	fTotalStrokes	 18
U+7E4F.0	fCangjie	VFRUC
U+7E4F.0	fFourCorner	27980
U+7E4F.0	fCantonese	syun3
U+7E4F.0	fMandarin	xuan4
U+7E50.0	fUTF8	繐
U+7E50.0	fCNS11643	2-625A
U+7E50.0	fBig5	F06E
U+7E50.0	fGB	3-6D45
U+7E50.0	fR/S	 120.12
U+7E50.0	fTotalStrokes	 18
U+7E50.0	fCangjie	VFJIP
U+7E50.0	fFourCorner	25933
U+7E50.0	fHakka	  sui5;  fui5; fi5
U+7E50.0	fCantonese	seoi6; seoi2 (also); seoi3
U+7E50.0	fMandarin	hui4; sui4
U+7E50.0	fEnglish	fine cloth
U+7E51.0	fUTF8	繑
U+7E51.0	fCNS11643	2-6260
U+7E51.0	fBig5	F074
U+7E51.0	fGB	3-6C5D
U+7E51.0	fR/S	 120.12
U+7E51.0	fTotalStrokes	 18
U+7E51.0	fCangjie	VFHKB
U+7E51.0	fFourCorner	22927
U+7E51.0	fCantonese	hiu1; kiu5
U+7E51.0	fMandarin	qiao1; jiao3; jue2
U+7E52.0	fUTF8	繒
U+7E52.0	fCNS11643	1-7725
U+7E52.0	fBig5	C2B9
U+7E52.0	fGB	1-6755
U+7E52.0	fR/S	 120.12
U+7E52.0	fTotalStrokes	 18
U+7E52.0	fCangjie	VFCWA
U+7E52.0	fFourCorner	28966
U+7E52.0	fHakka	  zen1
U+7E52.0	fCantonese	zang1; zang6; cang4
U+7E52.0	fMandarin	zeng1; ceng2
U+7E52.0	fEnglish	silk; silk fabrics
U+7E52.1	fUTF8	繒
U+7E52.1	fCNS11643	1-7725
U+7E52.1	fBig5	C2B9
U+7E52.1	fGB	1-6755
U+7E52.1	fR/S	 120.12
U+7E52.1	fTotalStrokes	 18
U+7E52.1	fCangjie	VFCWA
U+7E52.1	fFourCorner	28966
U+7E52.1	fMandarin	zeng4
U+7E52.1	fEnglish	 fasten tight
U+7E53.0	fUTF8	繓
U+7E53.0	fCNS11643	2-6263
U+7E53.0	fBig5	F077
U+7E53.0	fGB	5-6C65
U+7E53.0	fR/S	 120.12
U+7E53.0	fTotalStrokes	 18
U+7E53.0	fCangjie	VFASE
U+7E53.0	fFourCorner	26947
U+7E53.0	fHakka	 cot7
U+7E53.0	fCantonese	dyut3; cyut3
U+7E53.0	fMandarin	zuo3
U+7E53.0	fEnglish	the end of the thread in sewing
U+7E54.0	fUTF8	織
U+7E54.0	fCNS11643	1-767E
U+7E54.0	fBig5	C2B4
U+7E54.0	fGB	1-562F
U+7E54.0	fR/S	 120.12
U+7E54.0	fTotalStrokes	 18
U+7E54.0	fCangjie	VFYIA
U+7E54.0	fFourCorner	23950
U+7E54.0	fHakka	 zhit7; zit7; tsiit7
U+7E54.0	fCantonese	zik1; zi3; ci3
U+7E54.0	fMandarin	zhi1; zhi4; chi4; zhi2
U+7E54.0	fEnglish	 weave; knit
U+7E55.0	fUTF8	繕
U+7E55.0	fCNS11643	1-7721
U+7E55.0	fBig5	C2B5
U+7E55.0	fGB	1-4949
U+7E55.0	fR/S	 120.12
U+7E55.0	fTotalStrokes	 18
U+7E55.0	fCangjie	VFTGR
U+7E55.0	fFourCorner	28961
U+7E55.0	fHakka	 shan5; san5; sen3; sen5; shen3
U+7E55.0	fCantonese	sin6
U+7E55.0	fMandarin	shan4
U+7E55.0	fEnglish	[1] repair; mend [2] copy; transcribe
U+7E56.0	fUTF8	繖
U+7E56.0	fCNS11643	2-625B
U+7E56.0	fBig5	F06F
U+7E56.0	fGB	E-385C
U+7E56.0	fR/S	 120.12
U+7E56.0	fTotalStrokes	 18
U+7E56.0	fCangjie	VFTBK
U+7E56.0	fFourCorner	28940
U+7E56.0	fHakka	 san5; san3; shan3
U+7E56.0	fCantonese	saan3
U+7E56.0	fMandarin	san3; san4
U+7E56.0	fEnglish	 umbrella; parasol; parachute; canopy
U+7E57.0	fUTF8	繗
U+7E57.0	fCNS11643	2-6262
U+7E57.0	fBig5	F076
U+7E57.0	fGB	5-6C6B
U+7E57.0	fR/S	 120.12
U+7E57.0	fTotalStrokes	 18
U+7E57.0	fCangjie	VFFDQ
U+7E57.0	fFourCorner	29959
U+7E57.0	fCantonese	leon4
U+7E57.0	fMandarin	lin2
U+7E58.0	fUTF8	繘
U+7E58.0	fCNS11643	2-625D
U+7E58.0	fBig5	F071
U+7E58.0	fGB	3-6D4A
U+7E58.0	fR/S	 120.12
U+7E58.0	fTotalStrokes	 18
U+7E58.0	fCangjie	VFNHB
U+7E58.0	fFourCorner	27927
U+7E58.0	fHakka	 jut8; lut8; rut8; git7
U+7E58.0	fCantonese	wat6; leot6; gwat1
U+7E58.0	fMandarin	yu4; ju2; jue2; shu4; zhuo2
U+7E58.0	fEnglish	rope
U+7E59.0	fUTF8	繙
U+7E59.0	fCNS11643	1-7726
U+7E59.0	fBig5	C2BA
U+7E59.0	fGB	E-385D
U+7E59.0	fR/S	 120.12
U+7E59.0	fTotalStrokes	 18
U+7E59.0	fCangjie	VFHDW
U+7E59.0	fFourCorner	22969
U+7E59.0	fHakka	 fan1
U+7E59.0	fCantonese	faan4; faan1
U+7E59.0	fMandarin	fan1; fan2; pan2
U+7E59.0	fEnglish	[1] translate; interpret [2] flap; fly; flutter in wind
U+7E5A.0	fUTF8	繚
U+7E5A.0	fCNS11643	1-7723
U+7E5A.0	fBig5	C2B7
U+7E5A.0	fGB	1-6754
U+7E5A.0	fR/S	 120.12
U+7E5A.0	fTotalStrokes	 18
U+7E5A.0	fCangjie	VFKCF
U+7E5A.0	fFourCorner	24996
U+7E5A.0	fHakka	 liau2;  liau3
U+7E5A.0	fCantonese	liu4; liu1 (also)
U+7E5A.0	fMandarin	liao2; rao3; liao3; liao4
U+7E5A.0	fEnglish	 wind round; rap around
U+7E5B.0	fUTF8	繛
U+7E5B.0	fCNS11643	3-585B
U+7E5B.0	fGB	E-385E
U+7E5B.0	fR/S	  120.12
U+7E5B.0	fTotalStrokes	 18
U+7E5B.0	fCangjie	QFYAJ
U+7E5B.0	fFourCorner	51940
U+7E5B.0	fCantonese	coek3
U+7E5B.0	fMandarin	chuo4; chao1
U+7E5C.0	fUTF8	繜
U+7E5C.0	fCNS11643	2-6259
U+7E5C.0	fBig5	F06D
U+7E5C.0	fGB	3-6D46
U+7E5C.0	fR/S	 120.12
U+7E5C.0	fTotalStrokes	 18
U+7E5C.0	fCangjie	VFTWI
U+7E5C.0	fFourCorner	28946
U+7E5C.0	fHakka	  zun1; zun5
U+7E5C.0	fCantonese	zeon1; zeon3
U+7E5C.0	fMandarin	zun1; zun3
U+7E5C.0	fEnglish	drawers worn by women
U+7E5D.0	fUTF8	繝
U+7E5D.0	fCNS11643	3-5858
U+7E5D.0	fGB	E-385F
U+7E5D.0	fR/S	  120.12
U+7E5D.0	fTotalStrokes	 18
U+7E5D.0	fCangjie	VFANB
U+7E5D.0	fFourCorner	27920
U+7E5D.0	fCantonese	gaan3
U+7E5D.0	fMandarin	jian4
U+7E5E.0	fUTF8	繞
U+7E5E.0	fCNS11643	1-7722
U+7E5E.0	fBig5	C2B6
U+7E5E.0	fGB	1-4846
U+7E5E.0	fR/S	 120.12
U+7E5E.0	fTotalStrokes	 18
U+7E5E.0	fCangjie	VFGGU
U+7E5E.0	fFourCorner	24911
U+7E5E.0	fHakka	 ngiau3; {lau5}; ngiau2; niau3; ngieu3
U+7E5E.0	fCantonese	jiu2; jiu5 (also)
U+7E5E.0	fMandarin	rao4
U+7E5E.0	fEnglish	[1] go around; make a detour [2] march round; circle; surround
U+7E5E.1	fUTF8	繞
U+7E5E.1	fCNS11643	1-7722
U+7E5E.1	fBig5	C2B6
U+7E5E.1	fGB	1-4846
U+7E5E.1	fR/S	 120.12
U+7E5E.1	fTotalStrokes	 18
U+7E5E.1	fCangjie	VFGGU
U+7E5E.1	fFourCorner	24911
U+7E5E.1	fMandarin	rao3
U+7E5F.0	fUTF8	繟
U+7E5F.0	fCNS11643	2-625F
U+7E5F.0	fBig5	F073
U+7E5F.0	fGB	3-6D25
U+7E5F.0	fR/S	 120.12
U+7E5F.0	fTotalStrokes	 18
U+7E5F.0	fCangjie	VFRRJ
U+7E5F.0	fFourCorner	26956
U+7E5F.0	fCantonese	taan4; cin2; zin2
U+7E5F.0	fMandarin	chan3; chan2; chan4; dan3; tan2
U+7E60.0	fUTF8	繠
U+7E60.0	fCNS11643	2-6261
U+7E60.0	fBig5	F075
U+7E60.0	fGB	3-6C2E
U+7E60.0	fR/S	 120.12
U+7E60.0	fTotalStrokes	 18
U+7E60.0	fCangjie	PPPF
U+7E60.0	fFourCorner	33903
U+7E60.0	fCantonese	jeoi5
U+7E60.0	fMandarin	rui3
U+7E60.0	fEnglish	hang
U+7E61.0	fUTF8	繡
U+7E61.0	fCNS11643	1-7724
U+7E61.0	fBig5	C2B8
U+7E61.0	fGB	E-3860
U+7E61.0	fR/S	 120.12; 120.13
U+7E61.0	fTotalStrokes	 18
U+7E61.0	fCangjie	VFLX
U+7E61.0	fFourCorner	25927
U+7E61.0	fHakka	 siu5; sieu5
U+7E61.0	fCantonese	sau3
U+7E61.0	fMandarin	xiu4; xiao1
U+7E61.0	fEnglish	[1] embroider [2] embroidery
U+7E62.0	fUTF8	繢
U+7E62.0	fCNS11643	2-625E
U+7E62.0	fBig5	F072
U+7E62.0	fGB	1-6740
U+7E62.0	fR/S	 120.12
U+7E62.0	fTotalStrokes	 18
U+7E62.0	fCangjie	VFLMC
U+7E62.0	fFourCorner	25986
U+7E62.0	fHakka	 kui3; kui5; kui6; fui3; fi3; kwi5; k'wui5; fui5
U+7E62.0	fCantonese	kui2; kwui2 (also)
U+7E62.0	fMandarin	hui4; hui2; hu2; kui2
U+7E62.0	fEnglish	 draw; make a sketch of; sketch; paint
U+7E63.0	fUTF8	繣
U+7E63.0	fCNS11643	2-625C
U+7E63.0	fBig5	F070
U+7E63.0	fGB	3-6C7B
U+7E63.0	fR/S	 120.12
U+7E63.0	fTotalStrokes	 18
U+7E63.0	fCangjie	VFLGM
U+7E63.0	fFourCorner	25916
U+7E63.0	fHakka	 wak8; vak8
U+7E63.0	fCantonese	waak6; waa6; wok6
U+7E63.0	fMandarin	hua4
U+7E63.0	fEnglish	 obstinate perverse
U+7E64.0	fUTF8	繤
U+7E64.0	fCNS11643	3-662C
U+7E64.0	fGB	3-6B38
U+7E64.0	fR/S	 120.12
U+7E64.0	fTotalStrokes	 18
U+7E64.0	fCangjie	XTBUF
U+7E64.0	fFourCorner	4490
U+7E64.0	fCantonese	zyun2
U+7E64.0	fMandarin	zuan3
U+7E64.0	fEnglish	edit
U+7E65.0	fUTF8	繥
U+7E65.0	fCNS11643	4-5C5E
U+7E65.0	fGB	E-3861
U+7E65.0	fR/S	 120.12
U+7E65.0	fTotalStrokes	 18
U+7E65.0	fCangjie	VFGRR
U+7E65.0	fFourCorner	24960
U+7E65.0	fCantonese	hei1
U+7E65.0	fMandarin	xi1
U+7E66.0	fUTF8	繦
U+7E66.0	fCNS11643	3-585A
U+7E66.0	fGB	5-6C6C
U+7E66.0	fR/S	 120.13
U+7E66.0	fTotalStrokes	 18; 19
U+7E66.0	fCangjie	XVFNR
U+7E66.0	fFourCorner	26330
U+7E66.0	fCantonese	koeng5
U+7E66.0	fMandarin	qiang3; jiang3
U+7E66.0	fEnglish	 string of copper coins swaddling
U+7E67.0	fUTF8	繧
U+7E67.0	fGB	E-3862
U+7E67.0	fR/S	 120.12
U+7E67.0	fTotalStrokes	 18
U+7E67.0	fCangjie	VFMBI
U+7E67.0	fFourCorner	2193
U+7E67.0	fCantonese	wan4
U+7E68.0	fUTF8	繨
U+7E68.0	fCNS11643	2-6642
U+7E68.0	fBig5	F2B8
U+7E68.0	fGB	5-6B70
U+7E68.0	fR/S	 120.13
U+7E68.0	fTotalStrokes	 19
U+7E68.0	fCangjie	VFYGQ
U+7E68.0	fFourCorner	24934
U+7E68.0	fHakka	 dat7; tat7; tat8
U+7E68.0	fCantonese	daat6
U+7E68.0	fMandarin	da2; da5
U+7E68.0	fEnglish	 knot (rope)
U+7E69.0	fUTF8	繩
U+7E69.0	fCNS11643	1-7862
U+7E69.0	fBig5	C3B7
U+7E69.0	fGB	1-497E
U+7E69.0	fR/S	 120.13
U+7E69.0	fTotalStrokes	 19
U+7E69.0	fCangjie	VFRXU
U+7E69.0	fFourCorner	27917
U+7E69.0	fHakka	 sin2;  shin2; siin2; shun3; shun2
U+7E69.0	fCantonese	sing4; sing2 (also)
U+7E69.0	fMandarin	sheng2; min3; sheng4; ying4
U+7E69.0	fEnglish	[1] rope; string; cord; line [2] restrain [3] rectify; correct
U+7E69.1	fUTF8	繩
U+7E69.1	fCNS11643	1-7862
U+7E69.1	fBig5	C3B7
U+7E69.1	fGB	1-497E
U+7E69.1	fR/S	 120.13
U+7E69.1	fTotalStrokes	 19
U+7E69.1	fCangjie	VFRXU
U+7E69.1	fFourCorner	27917
U+7E69.1	fMandarin	sheng5
U+7E6A.0	fUTF8	繪
U+7E6A.0	fCNS11643	1-7863
U+7E6A.0	fBig5	C3B8
U+7E6A.0	fGB	1-3B66
U+7E6A.0	fR/S	 120.13
U+7E6A.0	fTotalStrokes	 19
U+7E6A.0	fCangjie	VFOMA
U+7E6A.0	fFourCorner	28966
U+7E6A.0	fHakka	 kwai5; fui3; fi5; fui6; fui5; kui3; kwi5; k'wui5
U+7E6A.0	fCantonese	kui2; kui3 (also); kwui2 (also); kwui3 (also); kui1
U+7E6A.0	fMandarin	hui4; gui4
U+7E6A.0	fEnglish	 draw; sketch; paint (pictures)
U+7E6B.0	fUTF8	繫
U+7E6B.0	fCNS11643	1-785F
U+7E6B.0	fBig5	C3B4
U+7E6B.0	fGB	1-7873
U+7E6B.0	fR/S	 120.13
U+7E6B.0	fTotalStrokes	 19
U+7E6B.0	fCangjie	JEVIF
U+7E6B.0	fFourCorner	57903
U+7E6B.0	fHakka	 he5;  hi5; kie1
U+7E6B.0	fCantonese	hai6; gai3
U+7E6B.0	fMandarin	xi4; qi1
U+7E6B.0	fEnglish	 attach; connect; unite; fasten; link; join
U+7E6B.1	fUTF8	繫
U+7E6B.1	fCNS11643	1-785F
U+7E6B.1	fBig5	C3B4
U+7E6B.1	fGB	1-7873
U+7E6B.1	fR/S	 120.13
U+7E6B.1	fTotalStrokes	 19
U+7E6B.1	fCangjie	JEVIF
U+7E6B.1	fFourCorner	57903
U+7E6B.1	fMandarin	ji4
U+7E6B.1	fEnglish	 bind; tie; hang up
U+7E6C.0	fUTF8	繬
U+7E6C.0	fCNS11643	3-5A72
U+7E6C.0	fGB	3-6D3B
U+7E6C.0	fR/S	  120.13
U+7E6C.0	fTotalStrokes	 19
U+7E6C.0	fCangjie	VFGOW
U+7E6C.0	fFourCorner	24960
U+7E6C.0	fCantonese	sik1
U+7E6C.0	fMandarin	se4
U+7E6D.0	fUTF8	繭
U+7E6D.0	fCNS11643	1-7860
U+7E6D.0	fBig5	C3B5
U+7E6D.0	fGB	1-3C6B
U+7E6D.0	fR/S	 120.13
U+7E6D.0	fTotalStrokes	 19
U+7E6D.0	fCangjie	TBLI
U+7E6D.0	fFourCorner	44227
U+7E6D.0	fHakka	 gan3;  kian3; gian3
U+7E6D.0	fCantonese	gaan2; gin2
U+7E6D.0	fMandarin	jian3
U+7E6D.0	fEnglish	 cocoon callus; blister
U+7E6E.0	fUTF8	繮
U+7E6E.0	fCNS11643	3-5A74
U+7E6E.0	fGB	1-6756
U+7E6E.0	fR/S	 120.13
U+7E6E.0	fTotalStrokes	 19
U+7E6E.0	fCangjie	VFMWM
U+7E6E.0	fFourCorner	21910
U+7E6E.0	fHakka	 giong1
U+7E6E.0	fCantonese	goeng1
U+7E6E.0	fMandarin	jiang1
U+7E6E.0	fEnglish	reins; bridle; halter
U+7E6F.0	fUTF8	繯
U+7E6F.0	fCNS11643	2-663E
U+7E6F.0	fBig5	F2B4
U+7E6F.0	fGB	1-6759
U+7E6F.0	fR/S	 120.13
U+7E6F.0	fTotalStrokes	 19
U+7E6F.0	fCangjie	VFWLV
U+7E6F.0	fFourCorner	26932
U+7E6F.0	fHakka	 fan2; hien3; fan5; fan6
U+7E6F.0	fCantonese	waan4; waan6 (also); jyun6
U+7E6F.0	fMandarin	huan2; huan4; xuan4
U+7E6F.0	fEnglish	[1] noose [2] hang (to death)
U+7E70.0	fUTF8	繰
U+7E70.0	fCNS11643	2-663C
U+7E70.0	fBig5	F2B2
U+7E70.0	fGB	1-6758
U+7E70.0	fR/S	 120.13
U+7E70.0	fTotalStrokes	 19
U+7E70.0	fCangjie	VFRRD
U+7E70.0	fFourCorner	26994
U+7E70.0	fHakka	 zau3; sau1
U+7E70.0	fCantonese	sou1; zou2; ciu1; tiu1
U+7E70.0	fMandarin	zao3; qiao1; qiao3
U+7E70.0	fEnglish	a kind of silk
U+7E70.1	fUTF8	繰
U+7E70.1	fCNS11643	2-663C
U+7E70.1	fBig5	F2B2
U+7E70.1	fGB	1-6758
U+7E70.1	fR/S	 120.13
U+7E70.1	fTotalStrokes	 19
U+7E70.1	fCangjie	VFRRD
U+7E70.1	fFourCorner	26994
U+7E70.1	fMandarin	sao1
U+7E70.1	fEnglish	 draw silk from cocoons
U+7E71.0	fUTF8	繱
U+7E71.0	fCNS11643	3-5A75
U+7E71.0	fGB	3-6D44
U+7E71.0	fR/S	 120.12
U+7E71.0	fTotalStrokes	 19; 18
U+7E71.0	fCangjie	VFTPP
U+7E71.0	fFourCorner	24930
U+7E71.0	fCantonese	cung1
U+7E71.0	fMandarin	cong1
U+7E72.0	fUTF8	繲
U+7E72.0	fCNS11643	2-6640
U+7E72.0	fBig5	F2B6
U+7E72.0	fGB	3-6D4B
U+7E72.0	fR/S	 120.13
U+7E72.0	fTotalStrokes	 19
U+7E72.0	fCangjie	VFNBQ
U+7E72.0	fFourCorner	27952
U+7E72.0	fHakka	 gai5; kie3
U+7E72.0	fCantonese	gaai3; haai5; faai3; haai1
U+7E72.0	fMandarin	jie4
U+7E72.0	fEnglish	 work as a tailor and do washing-for a living
U+7E73.0	fUTF8	繳
U+7E73.0	fCNS11643	1-7864
U+7E73.0	fBig5	C3BA
U+7E73.0	fGB	1-3D49
U+7E73.0	fR/S	 120.13
U+7E73.0	fTotalStrokes	 19
U+7E73.0	fCangjie	VFHSK
U+7E73.0	fFourCorner	28940
U+7E73.0	fHakka	  giau3; kieu3
U+7E73.0	fCantonese	giu2; zoek3
U+7E73.0	fMandarin	jiao3; jiao4; he2; jia3
U+7E73.0	fEnglish	[1] surrender; submit; lay down [2] pay (taxes; tuition, etc.)
U+7E73.1	fUTF8	繳
U+7E73.1	fCNS11643	1-7864
U+7E73.1	fBig5	C3BA
U+7E73.1	fGB	1-3D49
U+7E73.1	fR/S	 120.13
U+7E73.1	fTotalStrokes	 19
U+7E73.1	fCangjie	VFHSK
U+7E73.1	fFourCorner	28940
U+7E73.1	fMandarin	zhuo2
U+7E73.1	fEnglish	a string made of raw silk fastened to the arrow
U+7E74.0	fUTF8	繴
U+7E74.0	fCNS11643	2-6641
U+7E74.0	fBig5	F2B7
U+7E74.0	fGB	3-6C30
U+7E74.0	fR/S	 120.13
U+7E74.0	fTotalStrokes	 19
U+7E74.0	fCangjie	SJVIF
U+7E74.0	fFourCorner	70903
U+7E74.0	fHakka	  pit7; biak7
U+7E74.0	fCantonese	bik1; pik1
U+7E74.0	fMandarin	bo4; bi4; po2
U+7E74.0	fEnglish	a kind of fishing-net
U+7E75.0	fUTF8	繵
U+7E75.0	fCNS11643	2-663A
U+7E75.0	fBig5	F2B0
U+7E75.0	fGB	3-6D4C
U+7E75.0	fR/S	 120.13
U+7E75.0	fTotalStrokes	 19
U+7E75.0	fCangjie	VFYWM
U+7E75.0	fFourCorner	20916
U+7E75.0	fHakka	 chan2; can2; cen2; tan2
U+7E75.0	fCantonese	daan6; daan3; cin4; zin6; taan4
U+7E75.0	fMandarin	chan2; dan3; dan4; tan2
U+7E75.0	fEnglish	[1] cord; ligament [2] rattan for binding
U+7E76.0	fUTF8	繶
U+7E76.0	fCNS11643	2-6639
U+7E76.0	fBig5	F2AF
U+7E76.0	fGB	3-6D4D
U+7E76.0	fR/S	 120.13
U+7E76.0	fTotalStrokes	 19
U+7E76.0	fCangjie	VFYTP
U+7E76.0	fFourCorner	20936
U+7E76.0	fHakka	 jit7; ri5; ji5; rit7
U+7E76.0	fCantonese	jik1
U+7E76.0	fMandarin	yi4
U+7E76.0	fEnglish	the leather binding of a shoe
U+7E77.0	fUTF8	繷
U+7E77.0	fCNS11643	2-663D
U+7E77.0	fBig5	F2B3
U+7E77.0	fGB	5-6B7D
U+7E77.0	fR/S	 120.13
U+7E77.0	fTotalStrokes	 19
U+7E77.0	fCangjie	VFTWV
U+7E77.0	fFourCorner	25932
U+7E77.0	fHakka	 nung2; lung2
U+7E77.0	fCantonese	nung5; lung5 (also); nung4
U+7E77.0	fMandarin	nao2; nong2; nong3
U+7E78.0	fUTF8	繸
U+7E78.0	fCNS11643	2-663B
U+7E78.0	fBig5	F2B1
U+7E78.0	fGB	3-6D47
U+7E78.0	fR/S	 120.13
U+7E78.0	fTotalStrokes	 19
U+7E78.0	fCangjie	VFYTO
U+7E78.0	fFourCorner	29933
U+7E78.0	fHakka	  sui5
U+7E78.0	fCantonese	seoi6; seoi5
U+7E78.0	fMandarin	sui4
U+7E78.0	fEnglish	 hem or border of a garment tassel
U+7E79.0	fUTF8	繹
U+7E79.0	fCNS11643	1-7861
U+7E79.0	fBig5	C3B6
U+7E79.0	fGB	1-526F
U+7E79.0	fR/S	 120.13
U+7E79.0	fTotalStrokes	 19
U+7E79.0	fCangjie	VFWLJ
U+7E79.0	fFourCorner	26941
U+7E79.0	fHakka	 jit8; rit8
U+7E79.0	fCantonese	jik6
U+7E79.0	fMandarin	yi4; shi4
U+7E79.0	fEnglish	[1] draw silk [2] continuous; uninterrupted; [3] infer; deduce
U+7E7A.0	fUTF8	繺
U+7E7A.0	fCNS11643	2-663F
U+7E7A.0	fBig5	F2B5
U+7E7A.0	fGB	E-3863
U+7E7A.0	fR/S	 120.13
U+7E7A.0	fTotalStrokes	 19
U+7E7A.0	fCangjie	VFNKF
U+7E7A.0	fFourCorner	28934
U+7E7A.0	fCantonese	saai2; saat3
U+7E7A.0	fMandarin	shai3; yao3
U+7E7B.0	fUTF8	繻
U+7E7B.0	fCNS11643	2-6959
U+7E7B.0	fBig5	F4AC
U+7E7B.0	fGB	3-6D4F
U+7E7B.0	fR/S	 120.14
U+7E7B.0	fTotalStrokes	 20
U+7E7B.0	fCangjie	VFMBB
U+7E7B.0	fFourCorner	21927
U+7E7B.0	fHakka	  si1; sui1
U+7E7B.0	fCantonese	seoi1; jyu4 (also)
U+7E7B.0	fMandarin	xu1; ru2
U+7E7B.0	fEnglish	[1] fine (silk) gauze [2] torn piece of silk used as credentials
U+7E7C.0	fUTF8	繼
U+7E7C.0	fCNS11643	1-7A2B
U+7E7C.0	fBig5	C47E
U+7E7C.0	fGB	1-3C4C
U+7E7C.0	fR/S	 120.14
U+7E7C.0	fTotalStrokes	 20
U+7E7C.0	fCangjie	VFVVI
U+7E7C.0	fFourCorner	22913
U+7E7C.0	fHakka	  gi5; kie5
U+7E7C.0	fCantonese	gai3
U+7E7C.0	fMandarin	ji4
U+7E7C.0	fEnglish	[1] continue; maintain; carry on [2] then; afterwards [3] follow; inherit; succeed to
U+7E7D.0	fUTF8	繽
U+7E7D.0	fCNS11643	1-7A2A
U+7E7D.0	fBig5	C47D
U+7E7D.0	fGB	1-674D
U+7E7D.0	fR/S	 120.14
U+7E7D.0	fTotalStrokes	 20
U+7E7D.0	fCangjie	VFJMC
U+7E7D.0	fFourCorner	23986
U+7E7D.0	fHakka	 bin1
U+7E7D.0	fCantonese	ban1; pan1
U+7E7D.0	fMandarin	bin1
U+7E7D.0	fEnglish	flourishing; thriving; abundant
U+7E7E.0	fUTF8	繾
U+7E7E.0	fCNS11643	2-695A
U+7E7E.0	fBig5	F4AD
U+7E7E.0	fGB	1-6757
U+7E7E.0	fR/S	 120.14
U+7E7E.0	fTotalStrokes	 20
U+7E7E.0	fCangjie	VFYLR
U+7E7E.0	fFourCorner	25937
U+7E7E.0	fHakka	 kien3; ken3; kian3; kien5
U+7E7E.0	fCantonese	hin2
U+7E7E.0	fMandarin	qian3; qian4
U+7E7E.0	fEnglish	entangled
U+7E7F.0	fUTF8	繿
U+7E7F.0	fCNS11643	3-5C6C
U+7E7F.0	fGB	E-3864
U+7E7F.0	fR/S	 120.14
U+7E7F.0	fTotalStrokes	 20
U+7E7F.0	fCangjie	VFSIT
U+7E7F.0	fFourCorner	28910
U+7E7F.0	fHakka	 lam5
U+7E7F.0	fCantonese	laam4
U+7E7F.0	fMandarin	lan2
U+7E80.0	fUTF8	纀
U+7E80.0	fCNS11643	2-695C
U+7E80.0	fBig5	F4AF
U+7E80.0	fGB	3-6D51
U+7E80.0	fR/S	 120.14
U+7E80.0	fTotalStrokes	 20
U+7E80.0	fCangjie	VFOTO
U+7E80.0	fFourCorner	23934
U+7E80.0	fCantonese	buk6 (also); fuk6
U+7E80.0	fMandarin	pu2; bu3; bu2; fu2
U+7E81.0	fUTF8	纁
U+7E81.0	fCNS11643	2-695B
U+7E81.0	fBig5	F4AE
U+7E81.0	fGB	3-6D50
U+7E81.0	fR/S	 120.14
U+7E81.0	fTotalStrokes	 20
U+7E81.0	fCangjie	VFHGF
U+7E81.0	fFourCorner	22931
U+7E81.0	fHakka	  hiun1
U+7E81.0	fCantonese	fan1
U+7E81.0	fMandarin	xun1; xun4
U+7E81.0	fEnglish	shade of red; light red
U+7E82.0	fUTF8	纂
U+7E82.0	fCNS11643	1-7A2C
U+7E82.0	fBig5	C4A1
U+7E82.0	fGB	0-576B
U+7E82.0	fR/S	 120.14
U+7E82.0	fTotalStrokes	 20
U+7E82.0	fCangjie	HBUF
U+7E82.0	fFourCorner	88903
U+7E82.0	fHakka	 zon3; con3
U+7E82.0	fCantonese	zyun2
U+7E82.0	fMandarin	zuan3; zuan4
U+7E82.0	fEnglish	[1] edit; compile; collect [2] a kind of red cloth
U+7E83.0	fUTF8	纃
U+7E83.0	fGB	E-3865
U+7E83.0	fR/S	 120.14
U+7E83.0	fTotalStrokes	 20
U+7E83.0	fCangjie	VFYX
U+7E83.0	fCantonese	cai4
U+7E83.0	fMandarin	qi2
U+7E84.0	fUTF8	纄
U+7E84.0	fCNS11643	3-5E53
U+7E84.0	fGB	E-3866
U+7E84.0	fR/S	 120.15
U+7E84.0	fTotalStrokes	 21
U+7E84.0	fCangjie	VFTYJ
U+7E84.0	fFourCorner	24930
U+7E84.0	fMandarin	peng2
U+7E85.0	fUTF8	纅
U+7E85.0	fCNS11643	3-5E50
U+7E85.0	fGB	3-6C4C
U+7E85.0	fR/S	  120.15
U+7E85.0	fTotalStrokes	 21
U+7E85.0	fCangjie	VFVID
U+7E85.0	fFourCorner	22990
U+7E85.0	fCantonese	joek6
U+7E85.0	fMandarin	li4; lei2; lei4; lei3
U+7E86.0	fUTF8	纆
U+7E86.0	fCNS11643	2-6B79
U+7E86.0	fBig5	F5EB
U+7E86.0	fGB	3-6D53
U+7E86.0	fR/S	 120.15
U+7E86.0	fTotalStrokes	 21
U+7E86.0	fCangjie	VFWGG
U+7E86.0	fFourCorner	26914
U+7E86.0	fHakka	  met8
U+7E86.0	fCantonese	mak6
U+7E86.0	fMandarin	mo4
U+7E86.0	fEnglish	cord
U+7E87.0	fUTF8	纇
U+7E87.0	fCNS11643	2-6B76
U+7E87.0	fBig5	F5E8
U+7E87.0	fGB	3-613C
U+7E87.0	fR/S	 120.15
U+7E87.0	fTotalStrokes	 21
U+7E87.0	fCangjie	FFMBC
U+7E87.0	fFourCorner	91986
U+7E87.0	fHakka	 lui5; lui6
U+7E87.0	fCantonese	leoi6
U+7E87.0	fMandarin	lei4
U+7E87.0	fEnglish	[1] knot; thread [2] flaw
U+7E88.0	fUTF8	纈
U+7E88.0	fCNS11643	2-6B77
U+7E88.0	fBig5	F5E9
U+7E88.0	fGB	1-6753
U+7E88.0	fR/S	 120.15
U+7E88.0	fTotalStrokes	 21
U+7E88.0	fCangjie	VFGRC
U+7E88.0	fFourCorner	21986
U+7E88.0	fHakka	  giet7; get7
U+7E88.0	fCantonese	kit3; lit3; hit3 (also)
U+7E88.0	fMandarin	xie2
U+7E88.0	fEnglish	[1] patterned silk; silk with patters or designs woven in it [2] tie a knot
U+7E89.0	fUTF8	纉
U+7E89.0	fCNS11643	3-5E51
U+7E89.0	fGB	E-3867
U+7E89.0	fR/S	 120.15
U+7E89.0	fTotalStrokes	 21
U+7E89.0	fCangjie	VFQOC
U+7E89.0	fFourCorner	25980
U+7E89.0	fCantonese	zyun2
U+7E89.0	fMandarin	zuan3
U+7E8A.0	fUTF8	纊
U+7E8A.0	fCNS11643	2-6B75
U+7E8A.0	fBig5	F5E7
U+7E8A.0	fGB	1-667E
U+7E8A.0	fR/S	 120.15
U+7E8A.0	fTotalStrokes	 21
U+7E8A.0	fCangjie	VFITC
U+7E8A.0	fFourCorner	20932
U+7E8A.0	fHakka	 kong3; kwong3; k'wong3; k'wong5
U+7E8A.0	fCantonese	kwong3; kong3 (also)
U+7E8A.0	fMandarin	kuang4; guang4
U+7E8A.0	fEnglish	 cotton silk
U+7E8B.0	fUTF8	纋
U+7E8B.0	fCNS11643	2-6B78
U+7E8B.0	fBig5	F5EA
U+7E8B.0	fGB	3-6D52
U+7E8B.0	fR/S	 120.15
U+7E8B.0	fTotalStrokes	 21
U+7E8B.0	fCangjie	VFMBE
U+7E8B.0	fFourCorner	21947
U+7E8B.0	fCantonese	jau1
U+7E8B.0	fMandarin	you1; ou1
U+7E8C.0	fUTF8	續
U+7E8C.0	fCNS11643	1-7A7D
U+7E8C.0	fBig5	C4F2
U+7E8C.0	fGB	1-5078
U+7E8C.0	fR/S	 120.15
U+7E8C.0	fTotalStrokes	 21
U+7E8C.0	fCangjie	VFGWC
U+7E8C.0	fFourCorner	24986
U+7E8C.0	fHakka	  siuk8; sa5; suk8; suk7
U+7E8C.0	fCantonese	zuk6
U+7E8C.0	fMandarin	xu4; su2
U+7E8C.0	fEnglish	 continue; carry on succeed
U+7E8D.0	fUTF8	纍
U+7E8D.0	fCNS11643	2-6B7A
U+7E8D.0	fBig5	F5EC
U+7E8D.0	fGB	1-784D
U+7E8D.0	fR/S	 120.15
U+7E8D.0	fTotalStrokes	 21
U+7E8D.0	fCangjie	WWWF
U+7E8D.0	fFourCorner	60903
U+7E8D.0	fHakka	 lui2; lui1; lui3; liu3
U+7E8D.0	fCantonese	leoi4; leoi5; leoi6
U+7E8D.0	fMandarin	lei2; lei3; lei4
U+7E8D.0	fEnglish	[1] strung together; [2] heavy rope [3] bind; wind around; twine around; tie
U+7E8E.0	fUTF8	纎
U+7E8E.0	fCNS11643	3-5E52
U+7E8E.0	fGB	E-3868
U+7E8E.0	fR/S	 120.15
U+7E8E.0	fTotalStrokes	 21
U+7E8E.0	fCangjie	VFJIM
U+7E8E.0	fFourCorner	23950
U+7E8E.0	fHakka	 siam2
U+7E8E.0	fCantonese	cim1
U+7E8E.0	fMandarin	xian1
U+7E8E.0	fEnglish	 fine; delicate minute graceful
U+7E8F.0	fUTF8	纏
U+7E8F.0	fCNS11643	1-7A7C
U+7E8F.0	fBig5	C4F1
U+7E8F.0	fGB	1-3278
U+7E8F.0	fR/S	 120.15
U+7E8F.0	fTotalStrokes	 21
U+7E8F.0	fCangjie	VFIWG
U+7E8F.0	fFourCorner	20914
U+7E8F.0	fHakka	 chan2; can2; t'jan3; cen2; chen2
U+7E8F.0	fCantonese	cin4; zin6 (also)
U+7E8F.0	fMandarin	chan2; chan4
U+7E8F.0	fEnglish	[1] wrap; wind around; tie; bind; tangle; twine round [2] bother persistently [3] pester; worry [4] deal with
U+7E90.0	fUTF8	纐
U+7E90.0	fGB	E-3869
U+7E90.0	fR/S	 120.15
U+7E90.0	fTotalStrokes	 21
U+7E90.0	fCangjie	VFYKC
U+7E90.0	fCantonese	haau1
U+7E91.0	fUTF8	纑
U+7E91.0	fCNS11643	2-6D6F
U+7E91.0	fBig5	F742
U+7E91.0	fGB	3-6C42
U+7E91.0	fR/S	 120.16
U+7E91.0	fTotalStrokes	 22
U+7E91.0	fCangjie	VFYPT
U+7E91.0	fFourCorner	21917
U+7E91.0	fHakka	  lu2
U+7E91.0	fCantonese	lou4
U+7E91.0	fMandarin	lu2
U+7E91.0	fEnglish	[1] soften hemp (by boiling) [2] thread
U+7E92.0	fUTF8	纒
U+7E92.0	fCNS11643	3-5F5A
U+7E92.0	fGB	E-386A
U+7E92.0	fR/S	 120.16
U+7E92.0	fTotalStrokes	 22
U+7E92.0	fCangjie	VFMWG
U+7E92.0	fFourCorner	21910
U+7E92.0	fHakka	 cien2; cen2; chan2
U+7E92.0	fCantonese	cin4
U+7E92.0	fMandarin	chan2
U+7E92.0	fEnglish	 wrap; wind around tie; bind
U+7E93.0	fUTF8	纓
U+7E93.0	fCNS11643	1-7C41
U+7E93.0	fBig5	C5D5
U+7E93.0	fGB	1-5327
U+7E93.0	fR/S	 120.17
U+7E93.0	fTotalStrokes	 23
U+7E93.0	fCangjie	VFBCV
U+7E93.0	fFourCorner	26944
U+7E93.0	fHakka	  jin1; rin1
U+7E93.0	fCantonese	jing1
U+7E93.0	fMandarin	ying1; ying4
U+7E93.0	fEnglish	[1] chin strap for holding a hat [2] tassel [3] leaves of turnip
U+7E94.0	fUTF8	纔
U+7E94.0	fCNS11643	1-7C43
U+7E94.0	fBig5	C5D7
U+7E94.0	fGB	1-7826
U+7E94.0	fR/S	 120.17
U+7E94.0	fTotalStrokes	 23
U+7E94.0	fCangjie	VFNRI
U+7E94.0	fFourCorner	27913
U+7E94.0	fHakka	 cai2; cam3
U+7E94.0	fCantonese	coi4
U+7E94.0	fMandarin	cai2; shan1; shan4; chan2; zhan4
U+7E94.0	fEnglish	[1] just now; just then [2] only [3] not until; for the first time; then and only then
U+7E95.0	fUTF8	纕
U+7E95.0	fCNS11643	2-6F3D
U+7E95.0	fBig5	F7EE
U+7E95.0	fGB	3-6D56
U+7E95.0	fR/S	 120.17
U+7E95.0	fTotalStrokes	 23
U+7E95.0	fCangjie	VFYRV
U+7E95.0	fFourCorner	20932
U+7E95.0	fHakka	  siong1
U+7E95.0	fCantonese	soeng1
U+7E95.0	fMandarin	xiang1; sang1; rang2
U+7E95.0	fEnglish	 wear
U+7E95.1	fUTF8	纕
U+7E95.1	fCNS11643	2-6F3D
U+7E95.1	fBig5	F7EE
U+7E95.1	fGB	3-6D56
U+7E95.1	fR/S	 120.17
U+7E95.1	fTotalStrokes	 23
U+7E95.1	fCangjie	VFYRV
U+7E95.1	fFourCorner	20932
U+7E95.1	fHakka	 jong2; jong5
U+7E95.1	fCantonese	joeng4; joeng5
U+7E95.1	fMandarin	rang3; rang4; shang4
U+7E95.1	fEnglish	same as &#x6518;
U+7E96.0	fUTF8	纖
U+7E96.0	fCNS11643	1-7C42
U+7E96.0	fBig5	C5D6
U+7E96.0	fGB	1-4F4B
U+7E96.0	fR/S	 120.17
U+7E96.0	fTotalStrokes	 23
U+7E96.0	fCangjie	VFOIM
U+7E96.0	fFourCorner	23950
U+7E96.0	fHakka	  siam2; siam1; ciam1
U+7E96.0	fCantonese	cim1
U+7E96.0	fMandarin	xian1; jian1
U+7E96.0	fEnglish	fine; delicate; minute; graceful; tiny; slender
U+7E97.0	fUTF8	纗
U+7E97.0	fCNS11643	2-7047
U+7E97.0	fBig5	F8B9
U+7E97.0	fGB	3-6D57
U+7E97.0	fR/S	 120.18
U+7E97.0	fTotalStrokes	 24
U+7E97.0	fCangjie	VFUOB
U+7E97.0	fFourCorner	22927
U+7E97.0	fCantonese	zeoi1; waa6; kwai4
U+7E97.0	fMandarin	zui1; xi1; hua4; zhui4; wei4
U+7E98.0	fUTF8	纘
U+7E98.0	fCNS11643	2-712F
U+7E98.0	fBig5	F940
U+7E98.0	fGB	1-675A
U+7E98.0	fR/S	 120.19
U+7E98.0	fTotalStrokes	 25
U+7E98.0	fCangjie	VFHUC
U+7E98.0	fFourCorner	25986
U+7E98.0	fHakka	 zan5
U+7E98.0	fCantonese	zyun2
U+7E98.0	fMandarin	zuan3
U+7E98.0	fEnglish	 continue; carry on; succeed; keep up
U+7E99.0	fUTF8	纙
U+7E99.0	fCNS11643	2-7131
U+7E99.0	fBig5	F942
U+7E99.0	fGB	5-6C35
U+7E99.0	fR/S	 120.19
U+7E99.0	fTotalStrokes	 25
U+7E99.0	fCangjie	VFWLG
U+7E99.0	fFourCorner	26914
U+7E99.0	fCantonese	lo6
U+7E99.0	fMandarin	luo4
U+7E9A.0	fUTF8	纚
U+7E9A.0	fCNS11643	2-712E
U+7E9A.0	fBig5	F8FE
U+7E9A.0	fGB	3-6C67
U+7E9A.0	fR/S	 120.19
U+7E9A.0	fTotalStrokes	 25
U+7E9A.0	fCangjie	VFMMP
U+7E9A.0	fFourCorner	21911
U+7E9A.0	fHakka	 sai3; li2; li5
U+7E9A.0	fCantonese	saai2; si2 (also); lei4; saa2; lei5; si1
U+7E9A.0	fMandarin	xi3; li2; li3; shi3; sa3; si1; shai3; shi1
U+7E9A.0	fEnglish	 long dangling kerchief rope
U+7E9B.0	fUTF8	纛
U+7E9B.0	fCNS11643	2-7130
U+7E9B.0	fBig5	F941
U+7E9B.0	fGB	0-746E
U+7E9B.0	fR/S	 120.19
U+7E9B.0	fTotalStrokes	 25
U+7E9B.0	fCangjie	QMWYF
U+7E9B.0	fFourCorner	50993
U+7E9B.0	fHakka	  tau5; tuk8; to6; to5; tau6
U+7E9B.0	fCantonese	duk6; dou6 (also)
U+7E9B.0	fMandarin	dao4; du2; tao2
U+7E9B.0	fEnglish	 banner; streamer
U+7E9C.0	fUTF8	纜
U+7E9C.0	fCNS11643	1-7D39
U+7E9C.0	fBig5	C66C
U+7E9C.0	fGB	1-4042
U+7E9C.0	fR/S	 120.21
U+7E9C.0	fTotalStrokes	 27
U+7E9C.0	fCangjie	VFSWU
U+7E9C.0	fFourCorner	28916
U+7E9C.0	fHakka	 lam5; lam2; lam3; lan5
U+7E9C.0	fCantonese	laam6
U+7E9C.0	fMandarin	lan4; lan3
U+7E9C.0	fEnglish	 hawser; heavy-duty rope; cable
U+7E9D.0	fUTF8	纝
U+7E9D.0	fCNS11643	4-6D78
U+7E9D.0	fGB	E-386B
U+7E9D.0	fR/S	  120.21
U+7E9D.0	fTotalStrokes	 27
U+7E9D.0	fCangjie	VFWWF
U+7E9D.0	fFourCorner	26990
U+7E9D.0	fCantonese	leoi4
U+7E9D.0	fMandarin	lei2
U+7E9E.0	fUTF8	纞
U+7E9E.0	fCNS11643	3-6235
U+7E9E.0	fGB	E-386C
U+7E9E.0	fR/S	  120.23
U+7E9E.0	fTotalStrokes	 29
U+7E9E.0	fCangjie	VFVFP
U+7E9E.0	fFourCorner	22930
U+7E9E.0	fCantonese	lyun5
U+7E9E.0	fMandarin	lian4
U+7E9F.0	fUTF8	纟
U+7E9F.0	fGB	0-6679
U+7E9F.0	fR/S	 120.0
U+7E9F.0	fTotalStrokes	 3
U+7E9F.0	fCangjie	VVM
U+7E9F.0	fCantonese	si1
U+7E9F.0	fMandarin	jiao3si1pang2; si1
U+7E9F.0	fEnglish	simplified form of &#x7CF9;
U+7EA0.0	fUTF8	纠
U+7EA0.0	fGB	0-3E40
U+7EA0.0	fR/S	 120.2
U+7EA0.0	fTotalStrokes	 5
U+7EA0.0	fCangjie	VMVL
U+7EA0.0	fHakka	 giu5
U+7EA0.0	fCantonese	gau2; dau2 (also); giu2
U+7EA0.0	fMandarin	jiu1; jiu3; jiao3
U+7EA0.0	fEnglish	simplified form of &#x7CFE;
U+7EA1.0	fUTF8	纡
U+7EA1.0	fGB	0-667A
U+7EA1.0	fR/S	 120.3
U+7EA1.0	fTotalStrokes	 6
U+7EA1.0	fCangjie	VMMD
U+7EA1.0	fHakka	 hi1; vu1
U+7EA1.0	fCantonese	jyu1
U+7EA1.0	fMandarin	yu1; xu1; yu3; ou1
U+7EA1.0	fEnglish	simplified form of &#x7D06;
U+7EA2.0	fUTF8	红
U+7EA2.0	fGB	0-3A6C
U+7EA2.0	fR/S	 120.3
U+7EA2.0	fTotalStrokes	 6
U+7EA2.0	fCangjie	VMM
U+7EA2.0	fHakka	 fung2
U+7EA2.0	fCantonese	hung4; hung2 (also); gung1
U+7EA2.0	fMandarin	hong2; jiang4; hong1; gong1
U+7EA2.0	fEnglish	simplified form of &#x7D05;
U+7EA3.0	fUTF8	纣
U+7EA3.0	fGB	0-667B
U+7EA3.0	fR/S	 120.3
U+7EA3.0	fTotalStrokes	 6
U+7EA3.0	fCangjie	VMDI
U+7EA3.0	fHakka	 chiu5
U+7EA3.0	fCantonese	zau6
U+7EA3.0	fMandarin	zhou4
U+7EA3.0	fEnglish	simplified form of &#x7D02;
U+7EA4.0	fUTF8	纤
U+7EA4.0	fGB	0-4F4B
U+7EA4.0	fR/S	 120.3
U+7EA4.0	fTotalStrokes	 6
U+7EA4.0	fCangjie	VMHJ
U+7EA4.0	fHakka	 siam2
U+7EA4.0	fCantonese	cim1; hin3; hin1 (also)
U+7EA4.0	fMandarin	xian1; jian1; qian4
U+7EA4.0	fEnglish	simplified form of &#x7E96;
U+7EA5.0	fUTF8	纥
U+7EA5.0	fGB	0-667C
U+7EA5.0	fR/S	 120.3
U+7EA5.0	fTotalStrokes	 6
U+7EA5.0	fCangjie	VMON
U+7EA5.0	fHakka	 ngiet7; het8
U+7EA5.0	fCantonese	gat1; hat6
U+7EA5.0	fMandarin	he2; xie4; ge1; xie2; jie2; hu2
U+7EA5.0	fEnglish	simplified form of &#x7D07;
U+7EA6.0	fUTF8	约
U+7EA6.0	fGB	0-543C
U+7EA6.0	fR/S	 120.3
U+7EA6.0	fTotalStrokes	 6
U+7EA6.0	fCangjie	VMPI
U+7EA6.0	fHakka	 jok7
U+7EA6.0	fCantonese	joek3; wok6
U+7EA6.0	fMandarin	yue1; yao4; ao4; ji1; di4
U+7EA6.0	fEnglish	simplified form of &#x7D04;
U+7EA7.0	fUTF8	级
U+7EA7.0	fGB	0-3C36
U+7EA7.0	fR/S	 120.3
U+7EA7.0	fTotalStrokes	 6
U+7EA7.0	fCangjie	VMNHE
U+7EA7.0	fHakka	 kip7
U+7EA7.0	fCantonese	kap1
U+7EA7.0	fMandarin	ji2
U+7EA7.0	fEnglish	simplified form of &#x7D1A;
U+7EA8.0	fUTF8	纨
U+7EA8.0	fGB	0-667D
U+7EA8.0	fR/S	 120.3
U+7EA8.0	fTotalStrokes	 6
U+7EA8.0	fCangjie	VMKNI
U+7EA8.0	fHakka	 fan2
U+7EA8.0	fCantonese	jyun4
U+7EA8.0	fMandarin	wan2
U+7EA8.0	fEnglish	simplified form of &#x7D08;
U+7EA9.0	fUTF8	纩
U+7EA9.0	fGB	0-667E
U+7EA9.0	fR/S	 120.3
U+7EA9.0	fTotalStrokes	 6
U+7EA9.0	fCangjie	VMI
U+7EA9.0	fHakka	 k'wong3; k'wong5
U+7EA9.0	fCantonese	kwong3; kong3 (also)
U+7EA9.0	fMandarin	kuang4; guang4
U+7EA9.0	fEnglish	simplified form of &#x7E8A;
U+7EAA.0	fUTF8	纪
U+7EAA.0	fGB	0-3C4D
U+7EAA.0	fR/S	 120.3
U+7EAA.0	fTotalStrokes	 6
U+7EAA.0	fCangjie	VMSU
U+7EAA.0	fHakka	 gi3
U+7EAA.0	fCantonese	gei2; gei3 (also)
U+7EAA.0	fMandarin	ji4; ji1; ji3
U+7EAA.0	fEnglish	simplified form of &#x7D00;
U+7EAB.0	fUTF8	纫
U+7EAB.0	fGB	0-4852
U+7EAB.0	fR/S	 120.3
U+7EAB.0	fTotalStrokes	 6
U+7EAB.0	fCangjie	VMSHI
U+7EAB.0	fHakka	 ngin2; ngiun5; ngiun3; ngiun1
U+7EAB.0	fCantonese	jan6
U+7EAB.0	fMandarin	ren4; nin2; ren2; jin4
U+7EAB.0	fEnglish	simplified form of &#x7D09;
U+7EAC.0	fUTF8	纬
U+7EAC.0	fGB	0-4E33
U+7EAC.0	fR/S	 120.4
U+7EAC.0	fTotalStrokes	 7
U+7EAC.0	fCangjie	VMQS
U+7EAC.0	fHakka	 vui3; ve5; voi5
U+7EAC.0	fCantonese	wai5
U+7EAC.0	fMandarin	wei3; wei4
U+7EAC.0	fEnglish	simplified form of &#x7DEF;
U+7EAD.0	fUTF8	纭
U+7EAD.0	fGB	0-6721
U+7EAD.0	fR/S	 120.4
U+7EAD.0	fTotalStrokes	 7
U+7EAD.0	fCangjie	VMMMI
U+7EAD.0	fHakka	 jun2
U+7EAD.0	fCantonese	wan4
U+7EAD.0	fMandarin	yun2
U+7EAD.0	fEnglish	simplified form of &#x7D1C;
U+7EAE.0	fUTF8	纮
U+7EAE.0	fGB	8-7E54
U+7EAE.0	fR/S	 120.4
U+7EAE.0	fTotalStrokes	 7
U+7EAE.0	fCangjie	VMKI
U+7EAE.0	fCantonese	wang4
U+7EAE.0	fMandarin	hong2
U+7EAE.0	fEnglish	string; vast, expansive
U+7EAF.0	fUTF8	纯
U+7EAF.0	fGB	0-343F
U+7EAF.0	fR/S	 120.4
U+7EAF.0	fTotalStrokes	 7
U+7EAF.0	fCangjie	VMPU
U+7EAF.0	fHakka	 shun2
U+7EAF.0	fCantonese	seon4; zeon2
U+7EAF.0	fMandarin	chun2; quan2; tun2; zhun3; dun4; jun1; zhun1; zhun4
U+7EAF.0	fEnglish	simplified form of &#x7D14;
U+7EB0.0	fUTF8	纰
U+7EB0.0	fGB	0-6722
U+7EB0.0	fR/S	 120.4
U+7EB0.0	fTotalStrokes	 7
U+7EB0.0	fCangjie	VMPP
U+7EB0.0	fHakka	 pi1; pi2
U+7EB0.0	fCantonese	pei1; pei4
U+7EB0.0	fMandarin	pi1; bi3; pi2; bei1; bei4; bi4; pian2
U+7EB0.0	fEnglish	simplified form of &#x7D15;
U+7EB1.0	fUTF8	纱
U+7EB1.0	fGB	0-4934
U+7EB1.0	fR/S	 120.4
U+7EB1.0	fTotalStrokes	 7
U+7EB1.0	fCangjie	VMFH
U+7EB1.0	fHakka	 sa1
U+7EB1.0	fCantonese	saa1
U+7EB1.0	fMandarin	sha1; miao3
U+7EB1.0	fEnglish	simplified form of &#x7D17;
U+7EB2.0	fUTF8	纲
U+7EB2.0	fGB	0-3859
U+7EB2.0	fR/S	 120.4
U+7EB2.0	fTotalStrokes	 7
U+7EB2.0	fCangjie	VMBK
U+7EB2.0	fHakka	 gong1
U+7EB2.0	fCantonese	gong1; mong5
U+7EB2.0	fMandarin	gang1
U+7EB2.0	fEnglish	simplified form of &#x7DB1;
U+7EB3.0	fUTF8	纳
U+7EB3.0	fGB	0-4449
U+7EB3.0	fR/S	 120.4
U+7EB3.0	fTotalStrokes	 7
U+7EB3.0	fCangjie	VMOB
U+7EB3.0	fHakka	 nap8
U+7EB3.0	fCantonese	naap6; laap6 (also)
U+7EB3.0	fMandarin	na4
U+7EB3.0	fEnglish	simplified form of &#x7D0D;
U+7EB4.0	fUTF8	纴
U+7EB4.0	fGB	8-7E55
U+7EB4.0	fR/S	 120.4
U+7EB4.0	fTotalStrokes	 7
U+7EB4.0	fCangjie	VMHG
U+7EB4.0	fCantonese	jam6; jam4 (also)
U+7EB4.0	fMandarin	ren2; ren4
U+7EB4.0	fEnglish	weave, lay warp
U+7EB5.0	fUTF8	纵
U+7EB5.0	fGB	0-575D
U+7EB5.0	fR/S	 120.4
U+7EB5.0	fTotalStrokes	 7
U+7EB5.0	fCangjie	VMOO
U+7EB5.0	fHakka	 ziung3; ziung1; ziung5
U+7EB5.0	fCantonese	zung3; zung1
U+7EB5.0	fMandarin	zong4; zong3; cong2; song3
U+7EB5.0	fEnglish	simplified form of &#x7E31;
U+7EB6.0	fUTF8	纶
U+7EB6.0	fGB	0-425A
U+7EB6.0	fR/S	 120.4; 120.3
U+7EB6.0	fTotalStrokes	 6
U+7EB6.0	fCangjie	VMOP
U+7EB6.0	fHakka	 lun2
U+7EB6.0	fCantonese	leon4; gwaan1; gwan1
U+7EB6.0	fMandarin	lun2; guan1
U+7EB6.0	fEnglish	simplified form of &#x7DB8;
U+7EB7.0	fUTF8	纷
U+7EB7.0	fGB	0-3757
U+7EB7.0	fR/S	 120.4
U+7EB7.0	fTotalStrokes	 7
U+7EB7.0	fCangjie	VMCSH
U+7EB7.0	fHakka	 fun1
U+7EB7.0	fCantonese	fan1
U+7EB7.0	fMandarin	fen1; fen2
U+7EB7.0	fEnglish	simplified form of &#x7D1B;
U+7EB8.0	fUTF8	纸
U+7EB8.0	fGB	0-563D
U+7EB8.0	fR/S	 120.4
U+7EB8.0	fTotalStrokes	 7
U+7EB8.0	fCangjie	VMHVP
U+7EB8.0	fHakka	 zhi3
U+7EB8.0	fCantonese	zi2
U+7EB8.0	fMandarin	zhi3
U+7EB8.0	fEnglish	simplified form of &#x7D19;
U+7EB9.0	fUTF8	纹
U+7EB9.0	fGB	0-4E46
U+7EB9.0	fR/S	 120.4
U+7EB9.0	fTotalStrokes	 7
U+7EB9.0	fCangjie	VMYK
U+7EB9.0	fHakka	 vun2
U+7EB9.0	fCantonese	man4; man2 (also); man6
U+7EB9.0	fMandarin	wen2; wen4
U+7EB9.0	fEnglish	simplified form of &#x7D0B;
U+7EBA.0	fUTF8	纺
U+7EBA.0	fGB	0-3744
U+7EBA.0	fR/S	 120.4
U+7EBA.0	fTotalStrokes	 7
U+7EBA.0	fCangjie	VMYHS
U+7EBA.0	fHakka	 fong3; {p'iong3}
U+7EBA.0	fCantonese	fong2
U+7EBA.0	fMandarin	fang3; bang3
U+7EBA.0	fEnglish	simplified form of &#x7D21;
U+7EBB.0	fUTF8	纻
U+7EBB.0	fGB	8-2C42
U+7EBB.0	fR/S	 120.4
U+7EBB.0	fTotalStrokes	 7
U+7EBB.0	fCangjie	VMJM
U+7EBB.0	fCantonese	cyu5
U+7EBB.0	fMandarin	zhu4
U+7EBB.0	fEnglish	 ramie sack cloth
U+7EBC.0	fUTF8	纼
U+7EBC.0	fGB	8-7E56
U+7EBC.0	fR/S	 120.4
U+7EBC.0	fTotalStrokes	 7
U+7EBC.0	fCangjie	VMNL
U+7EBC.0	fCantonese	zan5; zan3 (also); can5; jan5
U+7EBC.0	fMandarin	yin3; zhen4
U+7EBC.0	fEnglish	a rope for leading cattle or horse
U+7EBD.0	fUTF8	纽
U+7EBD.0	fGB	0-4526
U+7EBD.0	fR/S	 120.4
U+7EBD.0	fTotalStrokes	 7
U+7EBD.0	fCangjie	VMNG
U+7EBD.0	fHakka	 ngiu3; {neu3}
U+7EBD.0	fCantonese	nau2; lau2 (also)
U+7EBD.0	fMandarin	niu3
U+7EBD.0	fEnglish	simplified form of &#x7D10;
U+7EBE.0	fUTF8	纾
U+7EBE.0	fGB	0-6723
U+7EBE.0	fR/S	 120.4
U+7EBE.0	fTotalStrokes	 7
U+7EBE.0	fCangjie	VMNIN
U+7EBE.0	fHakka	 shu1
U+7EBE.0	fCantonese	syu1
U+7EBE.0	fMandarin	shu1; shu4
U+7EBE.0	fEnglish	simplified form of &#x7D13;
U+7EBF.0	fUTF8	线
U+7EBF.0	fGB	0-4F5F
U+7EBF.0	fR/S	 120.5
U+7EBF.0	fTotalStrokes	 8
U+7EBF.0	fCangjie	VMIJ
U+7EBF.0	fHakka	 sien5; sen5
U+7EBF.0	fCantonese	sin3; sin2 (also)
U+7EBF.0	fMandarin	xian4
U+7EBF.0	fEnglish	simplified form of &#x7DAB;
U+7EC0.0	fUTF8	绀
U+7EC0.0	fGB	0-6724
U+7EC0.0	fR/S	 120.5
U+7EC0.0	fTotalStrokes	 8
U+7EC0.0	fCangjie	VMTM
U+7EC0.0	fHakka	 gam5
U+7EC0.0	fCantonese	gam3
U+7EC0.0	fMandarin	gan4
U+7EC0.0	fEnglish	simplified form of &#x7D3A;
U+7EC1.0	fUTF8	绁
U+7EC1.0	fGB	0-6725
U+7EC1.0	fR/S	 120.5
U+7EC1.0	fTotalStrokes	 8
U+7EC1.0	fCangjie	VMPT
U+7EC1.0	fHakka	 siet7
U+7EC1.0	fCantonese	sit3; jai6
U+7EC1.0	fMandarin	xie4; yi4
U+7EC1.0	fEnglish	simplified form of &#x7D32;
U+7EC2.0	fUTF8	绂
U+7EC2.0	fGB	0-6726
U+7EC2.0	fR/S	 120.5
U+7EC2.0	fTotalStrokes	 8
U+7EC2.0	fCangjie	VMIKE
U+7EC2.0	fHakka	 fut7
U+7EC2.0	fCantonese	fat1
U+7EC2.0	fMandarin	fu2
U+7EC2.0	fEnglish	simplified form of &#x7D31;
U+7EC3.0	fUTF8	练
U+7EC3.0	fGB	0-4137
U+7EC3.0	fR/S	 120.5
U+7EC3.0	fTotalStrokes	 8
U+7EC3.0	fCangjie	VMKVC
U+7EC3.0	fHakka	 lien5
U+7EC3.0	fCantonese	lin6
U+7EC3.0	fMandarin	lian4
U+7EC3.0	fEnglish	simplified form of &#x7DF4;
U+7EC4.0	fUTF8	组
U+7EC4.0	fGB	0-5769
U+7EC4.0	fR/S	 120.5
U+7EC4.0	fTotalStrokes	 8
U+7EC4.0	fCangjie	VMBM
U+7EC4.0	fHakka	 zih3
U+7EC4.0	fCantonese	zou2
U+7EC4.0	fMandarin	zu3; qu1
U+7EC4.0	fEnglish	simplified form of &#x7D44;
U+7EC5.0	fUTF8	绅
U+7EC5.0	fGB	0-4970
U+7EC5.0	fR/S	 120.5
U+7EC5.0	fTotalStrokes	 8
U+7EC5.0	fCangjie	VMLWL
U+7EC5.0	fHakka	 shin1
U+7EC5.0	fCantonese	san1
U+7EC5.0	fMandarin	shen1
U+7EC5.0	fEnglish	simplified form of &#x7D33;
U+7EC6.0	fUTF8	细
U+7EC6.0	fGB	0-4F38
U+7EC6.0	fR/S	 120.5
U+7EC6.0	fTotalStrokes	 8
U+7EC6.0	fCangjie	VMW
U+7EC6.0	fHakka	 se5
U+7EC6.0	fCantonese	sai3; sai2 (also)
U+7EC6.0	fMandarin	xi4
U+7EC6.0	fEnglish	simplified form of &#x7D30;
U+7EC7.0	fUTF8	织
U+7EC7.0	fGB	0-562F
U+7EC7.0	fR/S	 120.5
U+7EC7.0	fTotalStrokes	 8
U+7EC7.0	fCangjie	VMRC
U+7EC7.0	fHakka	 zhit7
U+7EC7.0	fCantonese	zik1; zi3; ci3
U+7EC7.0	fMandarin	zhi1; zhi4; chi4; zhi2
U+7EC7.0	fEnglish	simplified form of &#x7E54;
U+7EC8.0	fUTF8	终
U+7EC8.0	fGB	0-5655
U+7EC8.0	fR/S	 120.5
U+7EC8.0	fTotalStrokes	 8
U+7EC8.0	fCangjie	VMHEY
U+7EC8.0	fHakka	 zhung1
U+7EC8.0	fCantonese	zung1
U+7EC8.0	fMandarin	zhong1
U+7EC8.0	fEnglish	simplified form of &#x7D42;
U+7EC9.0	fUTF8	绉
U+7EC9.0	fGB	0-6727
U+7EC9.0	fR/S	 120.5
U+7EC9.0	fTotalStrokes	 8
U+7EC9.0	fCangjie	VMNSM
U+7EC9.0	fHakka	 ziu5
U+7EC9.0	fCantonese	zau3
U+7EC9.0	fMandarin	zhou4; chao4; zhu4
U+7EC9.0	fEnglish	simplified form of &#x7E10;
U+7ECA.0	fUTF8	绊
U+7ECA.0	fGB	0-306D
U+7ECA.0	fR/S	 120.5
U+7ECA.0	fTotalStrokes	 8
U+7ECA.0	fCangjie	VMFQ
U+7ECA.0	fHakka	 ban5
U+7ECA.0	fCantonese	bun6
U+7ECA.0	fMandarin	ban4
U+7ECA.0	fEnglish	simplified form of &#x7D46;
U+7ECB.0	fUTF8	绋
U+7ECB.0	fGB	0-6728
U+7ECB.0	fR/S	 120.5
U+7ECB.0	fTotalStrokes	 8
U+7ECB.0	fCangjie	VMLLN
U+7ECB.0	fHakka	 fut7
U+7ECB.0	fCantonese	fat1
U+7ECB.0	fMandarin	fu2; fei4
U+7ECB.0	fEnglish	simplified form of &#x7D3C;
U+7ECC.0	fUTF8	绌
U+7ECC.0	fGB	0-6729
U+7ECC.0	fR/S	 120.5
U+7ECC.0	fTotalStrokes	 8
U+7ECC.0	fCangjie	VMUU
U+7ECC.0	fHakka	 tut7; chot7
U+7ECC.0	fCantonese	zyut3; zyut6 (also); zeot1
U+7ECC.0	fMandarin	chu4; zhuo2; lei2
U+7ECC.0	fEnglish	simplified form of &#x7D40;
U+7ECD.0	fUTF8	绍
U+7ECD.0	fGB	0-495C
U+7ECD.0	fR/S	 120.5
U+7ECD.0	fTotalStrokes	 8
U+7ECD.0	fCangjie	VMSHR
U+7ECD.0	fHakka	 shau5; shau2
U+7ECD.0	fCantonese	siu6
U+7ECD.0	fMandarin	shao4; chao1
U+7ECD.0	fEnglish	simplified form of &#x7D39;
U+7ECE.0	fUTF8	绎
U+7ECE.0	fGB	0-526F
U+7ECE.0	fR/S	 120.5
U+7ECE.0	fTotalStrokes	 8
U+7ECE.0	fCangjie	VMEQ
U+7ECE.0	fHakka	 jit8
U+7ECE.0	fCantonese	jik6
U+7ECE.0	fMandarin	yi4; shi4
U+7ECE.0	fEnglish	simplified form of &#x7E79;
U+7ECF.0	fUTF8	经
U+7ECF.0	fGB	0-3E2D
U+7ECF.0	fR/S	 120.5
U+7ECF.0	fTotalStrokes	 8
U+7ECF.0	fCangjie	VMNOM
U+7ECF.0	fHakka	 gang1; gin1
U+7ECF.0	fCantonese	ging1
U+7ECF.0	fMandarin	jing1; jing4
U+7ECF.0	fEnglish	simplified form of &#x7D93;
U+7ECF.1	fUTF8	经
U+7ECF.1	fGB	0-3E2D
U+7ECF.1	fR/S	 120.5
U+7ECF.1	fTotalStrokes	 8
U+7ECF.1	fCangjie	VMNOM
U+7ECF.1	fMandarin	jing5
U+7ED0.0	fUTF8	绐
U+7ED0.0	fGB	0-672A
U+7ED0.0	fR/S	 120.5
U+7ED0.0	fTotalStrokes	 8
U+7ED0.0	fCangjie	VMIR
U+7ED0.0	fHakka	 toi5; tui5
U+7ED0.0	fCantonese	doi6; toi5 (also); bong2
U+7ED0.0	fMandarin	dai4
U+7ED0.0	fEnglish	simplified form of &#x7D3F;
U+7ED1.0	fUTF8	绑
U+7ED1.0	fGB	0-3073
U+7ED1.0	fR/S	 120.6
U+7ED1.0	fTotalStrokes	 9
U+7ED1.0	fCangjie	VMQJL
U+7ED1.0	fHakka	 bong3
U+7ED1.0	fCantonese	bong2
U+7ED1.0	fMandarin	bang3
U+7ED1.0	fEnglish	simplified form of &#x7D81;
U+7ED2.0	fUTF8	绒
U+7ED2.0	fGB	0-485E
U+7ED2.0	fR/S	 120.6
U+7ED2.0	fTotalStrokes	 9
U+7ED2.0	fCangjie	XVMIJ
U+7ED2.0	fHakka	 jung2
U+7ED2.0	fCantonese	jung4 (also); jung2
U+7ED2.0	fMandarin	rong2
U+7ED2.0	fEnglish	simplified form of &#x7D68;
U+7ED3.0	fUTF8	结
U+7ED3.0	fGB	0-3D61
U+7ED3.0	fR/S	 120.6
U+7ED3.0	fTotalStrokes	 9
U+7ED3.0	fCangjie	VMGR
U+7ED3.0	fHakka	 get7; giet7
U+7ED3.0	fCantonese	git3; lit3 (also); gai3
U+7ED3.0	fMandarin	jie2; ji4
U+7ED3.0	fEnglish	simplified form of &#x7D50;
U+7ED3.1	fUTF8	结
U+7ED3.1	fGB	0-3D61
U+7ED3.1	fR/S	 120.6
U+7ED3.1	fTotalStrokes	 9
U+7ED3.1	fCangjie	VMGR
U+7ED3.1	fMandarin	jie1
U+7ED4.0	fUTF8	绔
U+7ED4.0	fGB	0-672B
U+7ED4.0	fR/S	 120.6
U+7ED4.0	fTotalStrokes	 9
U+7ED4.0	fCangjie	VMKMS
U+7ED4.0	fHakka	 ku5
U+7ED4.0	fCantonese	fu3
U+7ED4.0	fMandarin	ku4
U+7ED4.0	fEnglish	simplified form of &#x7D5D;
U+7ED5.0	fUTF8	绕
U+7ED5.0	fGB	0-4846
U+7ED5.0	fR/S	 120.6
U+7ED5.0	fTotalStrokes	 9
U+7ED5.0	fCangjie	VMJPU
U+7ED5.0	fHakka	 ngiau3
U+7ED5.0	fCantonese	jiu5 (also); jiu2
U+7ED5.0	fMandarin	rao4
U+7ED5.0	fEnglish	simplified form of &#x7E5E;
U+7ED5.1	fUTF8	绕
U+7ED5.1	fGB	0-4846
U+7ED5.1	fR/S	 120.6
U+7ED5.1	fTotalStrokes	 9
U+7ED5.1	fCangjie	VMJPU
U+7ED5.1	fMandarin	rao3
U+7ED6.0	fUTF8	绖
U+7ED6.0	fGB	8-7E57
U+7ED6.0	fR/S	 120.6
U+7ED6.0	fTotalStrokes	 9
U+7ED6.0	fCangjie	VMMIG
U+7ED6.0	fCantonese	dit6
U+7ED6.0	fMandarin	die2
U+7ED6.0	fEnglish	white hempen cloth worn by mourners
U+7ED7.0	fUTF8	绗
U+7ED7.0	fGB	0-672C
U+7ED7.0	fR/S	 120.6
U+7ED7.0	fTotalStrokes	 9
U+7ED7.0	fCangjie	VMHON
U+7ED7.0	fCantonese	hong4; han6
U+7ED7.0	fMandarin	heng4; hang2; xing4
U+7ED7.0	fEnglish	simplified form of &#x7D4E;
U+7ED8.0	fUTF8	绘
U+7ED8.0	fGB	0-3B66
U+7ED8.0	fR/S	 120.6
U+7ED8.0	fTotalStrokes	 9
U+7ED8.0	fCangjie	VMOMI
U+7ED8.0	fHakka	 kwai5; fui5; k'wui5
U+7ED8.0	fCantonese	kui2; kwui2 (also); kui3 (also); kwui3 (also); kui1
U+7ED8.0	fMandarin	hui4; gui4
U+7ED8.0	fEnglish	simplified form of &#x7E6A;
U+7ED9.0	fUTF8	给
U+7ED9.0	fGB	0-3878
U+7ED9.0	fR/S	 120.6
U+7ED9.0	fTotalStrokes	 9
U+7ED9.0	fCangjie	VMOMR
U+7ED9.0	fHakka	 gip7
U+7ED9.0	fCantonese	kap1
U+7ED9.0	fMandarin	gei3
U+7ED9.0	fEnglish	simplified form of &#x7D66;
U+7ED9.1	fUTF8	给
U+7ED9.1	fGB	0-3878
U+7ED9.1	fR/S	 120.6
U+7ED9.1	fTotalStrokes	 9
U+7ED9.1	fCangjie	VMOMR
U+7ED9.1	fMandarin	ji3
U+7EDA.0	fUTF8	绚
U+7EDA.0	fGB	0-5124
U+7EDA.0	fR/S	 120.6
U+7EDA.0	fTotalStrokes	 9
U+7EDA.0	fCangjie	VMPA
U+7EDA.0	fHakka	 sien3
U+7EDA.0	fCantonese	hyun3; seon1 (also)
U+7EDA.0	fMandarin	xuan4
U+7EDA.0	fEnglish	simplified form of &#x7D62;
U+7EDB.0	fUTF8	绛
U+7EDB.0	fGB	0-672D
U+7EDB.0	fR/S	 120.6
U+7EDB.0	fTotalStrokes	 9
U+7EDB.0	fCangjie	VMHEQ
U+7EDB.0	fHakka	 gong5
U+7EDB.0	fCantonese	gong3
U+7EDB.0	fMandarin	jiang4
U+7EDB.0	fEnglish	simplified form of &#x7D73;
U+7EDC.0	fUTF8	络
U+7EDC.0	fGB	0-4267
U+7EDC.0	fR/S	 120.6
U+7EDC.0	fTotalStrokes	 9
U+7EDC.0	fCangjie	VMHER
U+7EDC.0	fHakka	 lok8
U+7EDC.0	fCantonese	lok3; lok2 (also); lok6 (also)
U+7EDC.0	fMandarin	luo4; lao4
U+7EDC.0	fEnglish	simplified form of &#x7D61;
U+7EDD.0	fUTF8	绝
U+7EDD.0	fGB	0-3E78
U+7EDD.0	fR/S	 120.6
U+7EDD.0	fTotalStrokes	 9
U+7EDD.0	fCangjie	VMNAU
U+7EDD.0	fCantonese	zyut6; zyut2 (also)
U+7EDD.0	fMandarin	jue2
U+7EDD.0	fEnglish	simplified form of &#x7D76;
U+7EDE.0	fUTF8	绞
U+7EDE.0	fGB	0-3D4A
U+7EDE.0	fR/S	 120.6
U+7EDE.0	fTotalStrokes	 9
U+7EDE.0	fCangjie	VMYCK
U+7EDE.0	fHakka	 gau3
U+7EDE.0	fCantonese	gaau2
U+7EDE.0	fMandarin	jiao3; jia3; yao2
U+7EDE.0	fEnglish	simplified form of &#x7D5E;
U+7EDF.0	fUTF8	统
U+7EDF.0	fGB	0-4D33
U+7EDF.0	fR/S	 120.6
U+7EDF.0	fTotalStrokes	 9
U+7EDF.0	fCangjie	VMYIU
U+7EDF.0	fHakka	 tung3
U+7EDF.0	fCantonese	tung2
U+7EDF.0	fMandarin	tong3; tong4
U+7EDF.0	fEnglish	simplified form of &#x7D71;
U+7EE0.0	fUTF8	绠
U+7EE0.0	fGB	0-672E
U+7EE0.0	fR/S	 120.7
U+7EE0.0	fTotalStrokes	 10
U+7EE0.0	fCangjie	VMMLK
U+7EE0.0	fHakka	 gen3
U+7EE0.0	fCantonese	gang2
U+7EE0.0	fMandarin	geng3; ban3; bing3
U+7EE0.0	fEnglish	simplified form of &#x7D86;
U+7EE1.0	fUTF8	绡
U+7EE1.0	fGB	0-672F
U+7EE1.0	fR/S	 120.7
U+7EE1.0	fTotalStrokes	 10
U+7EE1.0	fCangjie	XVMFB
U+7EE1.0	fHakka	 siau1
U+7EE1.0	fCantonese	siu1
U+7EE1.0	fMandarin	xiao1; shao1
U+7EE1.0	fEnglish	simplified form of &#x7D83;
U+7EE2.0	fUTF8	绢
U+7EE2.0	fGB	0-3E6E
U+7EE2.0	fR/S	 120.7
U+7EE2.0	fTotalStrokes	 10
U+7EE2.0	fCangjie	VMRB
U+7EE2.0	fHakka	 gen3; gen5
U+7EE2.0	fCantonese	gyun3
U+7EE2.0	fMandarin	juan4; quan3
U+7EE2.0	fEnglish	simplified form of &#x7D79;
U+7EE3.0	fUTF8	绣
U+7EE3.0	fGB	0-5065
U+7EE3.0	fR/S	 120.7
U+7EE3.0	fTotalStrokes	 10
U+7EE3.0	fCangjie	VMHDS
U+7EE3.0	fHakka	 siu5
U+7EE3.0	fCantonese	sau3
U+7EE3.0	fMandarin	xiu4
U+7EE3.0	fEnglish	simplified form of &#x7D89;
U+7EE4.0	fUTF8	绤
U+7EE4.0	fGB	8-2C43
U+7EE4.0	fR/S	 120.7
U+7EE4.0	fTotalStrokes	 10
U+7EE4.0	fCangjie	VMCOR
U+7EE4.0	fCantonese	gwik1
U+7EE4.0	fMandarin	xi4
U+7EE4.0	fEnglish	cloth
U+7EE5.0	fUTF8	绥
U+7EE5.0	fGB	0-4B67
U+7EE5.0	fR/S	 120.7
U+7EE5.0	fTotalStrokes	 10
U+7EE5.0	fCangjie	VMBV
U+7EE5.0	fHakka	 sui1
U+7EE5.0	fCantonese	seoi1
U+7EE5.0	fMandarin	sui1; sui2; tuo3; rui2; shui1
U+7EE5.0	fEnglish	simplified form of &#x7D8F;
U+7EE6.0	fUTF8	绦
U+7EE6.0	fGB	0-4C50
U+7EE6.0	fR/S	 120.7
U+7EE6.0	fTotalStrokes	 10
U+7EE6.0	fCangjie	VMHED
U+7EE6.0	fCantonese	tou1; tou3
U+7EE6.0	fMandarin	tao1
U+7EE6.0	fEnglish	simplified form of &#x7E27;
U+7EE7.0	fUTF8	继
U+7EE7.0	fGB	0-3C4C
U+7EE7.0	fR/S	 120.7
U+7EE7.0	fTotalStrokes	 10
U+7EE7.0	fCangjie	VMVFD
U+7EE7.0	fHakka	 gi5
U+7EE7.0	fCantonese	gai3
U+7EE7.0	fMandarin	ji4
U+7EE7.0	fEnglish	simplified form of &#x7E7C;
U+7EE8.0	fUTF8	绨
U+7EE8.0	fGB	0-6730
U+7EE8.0	fR/S	 120.7
U+7EE8.0	fTotalStrokes	 10
U+7EE8.0	fCangjie	VMCNH
U+7EE8.0	fHakka	 ti2
U+7EE8.0	fCantonese	tai4; tai3
U+7EE8.0	fMandarin	ti2; ti4; ti3
U+7EE8.0	fEnglish	simplified form of &#x7D88;
U+7EE9.0	fUTF8	绩
U+7EE9.0	fGB	0-3C28
U+7EE9.0	fR/S	 120.8
U+7EE9.0	fTotalStrokes	 11
U+7EE9.0	fCangjie	VMQMO
U+7EE9.0	fHakka	 ziak7
U+7EE9.0	fCantonese	zik1
U+7EE9.0	fMandarin	ji1
U+7EE9.0	fEnglish	simplified form of &#x7E3E;
U+7EEA.0	fUTF8	绪
U+7EEA.0	fGB	0-5077
U+7EEA.0	fR/S	 120.8
U+7EEA.0	fTotalStrokes	 11
U+7EEA.0	fCangjie	VMJKA
U+7EEA.0	fCantonese	seoi5
U+7EEA.0	fMandarin	xu4
U+7EEA.0	fEnglish	simplified form of &#x7DD2;
U+7EEB.0	fUTF8	绫
U+7EEB.0	fGB	0-6731
U+7EEB.0	fR/S	 120.8
U+7EEB.0	fTotalStrokes	 11
U+7EEB.0	fCangjie	VMGCE
U+7EEB.0	fHakka	 lin2
U+7EEB.0	fCantonese	ling4
U+7EEB.0	fMandarin	ling2
U+7EEB.0	fEnglish	simplified form of &#x7DBE;
U+7EEC.0	fUTF8	绬
U+7EEC.0	fGB	8-2F72
U+7EEC.0	fR/S	 120.8
U+7EEC.0	fTotalStrokes	 11
U+7EEC.0	fCangjie	VMTLK
U+7EEC.0	fCantonese	jing1
U+7EEC.0	fMandarin	yin1; ying1
U+7EED.0	fUTF8	续
U+7EED.0	fGB	0-5078
U+7EED.0	fR/S	 120.8
U+7EED.0	fTotalStrokes	 11
U+7EED.0	fCangjie	VMJNK
U+7EED.0	fHakka	 siuk8
U+7EED.0	fCantonese	zuk6
U+7EED.0	fMandarin	xu4; su2
U+7EED.0	fEnglish	simplified form of &#x7E8C;
U+7EEE.0	fUTF8	绮
U+7EEE.0	fGB	0-6732
U+7EEE.0	fR/S	 120.8
U+7EEE.0	fTotalStrokes	 11
U+7EEE.0	fCangjie	VMKMR
U+7EEE.0	fHakka	 ji3
U+7EEE.0	fCantonese	ji2
U+7EEE.0	fMandarin	qi3; yi3
U+7EEE.0	fEnglish	simplified form of &#x7DBA;
U+7EEF.0	fUTF8	绯
U+7EEF.0	fGB	0-6733
U+7EEF.0	fR/S	 120.8
U+7EEF.0	fTotalStrokes	 11
U+7EEF.0	fCangjie	VMLMY
U+7EEF.0	fHakka	 fui1
U+7EEF.0	fCantonese	fei1
U+7EEF.0	fMandarin	fei1
U+7EEF.0	fEnglish	simplified form of &#x7DCB;
U+7EF0.0	fUTF8	绰
U+7EF0.0	fGB	0-3442
U+7EF0.0	fR/S	 120.8
U+7EF0.0	fTotalStrokes	 11
U+7EF0.0	fCangjie	VMYAJ
U+7EF0.0	fHakka	 shok7
U+7EF0.0	fCantonese	coek3; caau1
U+7EF0.0	fMandarin	chuo4; chao1
U+7EF0.0	fEnglish	simplified form of &#x7DBD;
U+7EF1.0	fUTF8	绱
U+7EF1.0	fGB	0-6734
U+7EF1.0	fR/S	 120.8
U+7EF1.0	fTotalStrokes	 11
U+7EF1.0	fCangjie	VMFBR
U+7EF1.0	fCantonese	soeng5; zoeng2
U+7EF1.0	fMandarin	shang4; zhang3
U+7EF1.0	fEnglish	simplified form of &#x7DD4;
U+7EF2.0	fUTF8	绲
U+7EF2.0	fGB	0-6735
U+7EF2.0	fR/S	 120.8
U+7EF2.0	fTotalStrokes	 11
U+7EF2.0	fCangjie	VMAPP
U+7EF2.0	fHakka	 gwun3
U+7EF2.0	fCantonese	gwan2
U+7EF2.0	fMandarin	gun3; hun3; hun4
U+7EF2.0	fEnglish	simplified form of &#x7DC4;
U+7EF3.0	fUTF8	绳
U+7EF3.0	fGB	0-497E
U+7EF3.0	fR/S	 120.8
U+7EF3.0	fTotalStrokes	 11
U+7EF3.0	fCangjie	VMRLU
U+7EF3.0	fHakka	 shin2; shun2
U+7EF3.0	fCantonese	sing4; sing2 (also)
U+7EF3.0	fMandarin	sheng2; min3; ying4; sheng4
U+7EF3.0	fEnglish	simplified form of &#x7E69;
U+7EF3.1	fUTF8	绳
U+7EF3.1	fGB	0-497E
U+7EF3.1	fR/S	 120.8
U+7EF3.1	fTotalStrokes	 11
U+7EF3.1	fCangjie	VMRLU
U+7EF3.1	fMandarin	sheng5
U+7EF4.0	fUTF8	维
U+7EF4.0	fGB	0-4E2C
U+7EF4.0	fR/S	 120.8
U+7EF4.0	fTotalStrokes	 11
U+7EF4.0	fCangjie	VMOG
U+7EF4.0	fHakka	 vui2
U+7EF4.0	fCantonese	wai4
U+7EF4.0	fMandarin	wei2; yong1
U+7EF4.0	fEnglish	simplified form of &#x7DAD;
U+7EF5.0	fUTF8	绵
U+7EF5.0	fGB	0-4360
U+7EF5.0	fR/S	 120.8
U+7EF5.0	fTotalStrokes	 11
U+7EF5.0	fCangjie	VMHAB
U+7EF5.0	fHakka	 mien2
U+7EF5.0	fCantonese	min4
U+7EF5.0	fMandarin	mian2
U+7EF5.0	fEnglish	simplified form of &#x7DBF;
U+7EF6.0	fUTF8	绶
U+7EF6.0	fGB	0-6737
U+7EF6.0	fR/S	 120.8
U+7EF6.0	fTotalStrokes	 11
U+7EF6.0	fCangjie	VMBBE
U+7EF6.0	fHakka	 shiu5
U+7EF6.0	fCantonese	sau6
U+7EF6.0	fMandarin	shou4
U+7EF6.0	fEnglish	simplified form of &#x7DAC;
U+7EF7.0	fUTF8	绷
U+7EF7.0	fGB	0-3141
U+7EF7.0	fR/S	 120.8
U+7EF7.0	fTotalStrokes	 11
U+7EF7.0	fCangjie	XVMBB
U+7EF7.0	fHakka	 ben1; bong5
U+7EF7.0	fCantonese	bang1; mang2; maang1 (also); mang1
U+7EF7.0	fMandarin	beng1; beng3; beng4
U+7EF7.0	fEnglish	simplified form of &#x7DB3;
U+7EF7.1	fUTF8	绷
U+7EF7.1	fGB	0-3141
U+7EF7.1	fR/S	 120.8
U+7EF7.1	fTotalStrokes	 11
U+7EF7.1	fCangjie	XVMBB
U+7EF8.0	fUTF8	绸
U+7EF8.0	fGB	0-3371
U+7EF8.0	fR/S	 120.8
U+7EF8.0	fTotalStrokes	 11
U+7EF8.0	fCangjie	VMBGR
U+7EF8.0	fHakka	 chiu2
U+7EF8.0	fCantonese	cau4; cau2 (also)
U+7EF8.0	fMandarin	chou2; tao1; diao4
U+7EF8.0	fEnglish	simplified form of &#x7DA2;
U+7EF9.0	fUTF8	绹
U+7EF9.0	fGB	8-2C44
U+7EF9.0	fR/S	 120.8
U+7EF9.0	fTotalStrokes	 11
U+7EF9.0	fCangjie	VMPOU
U+7EF9.0	fCantonese	tou4
U+7EF9.0	fMandarin	tao2; ku4
U+7EF9.0	fEnglish	to braid, twist; twisted or braided
U+7EFA.0	fUTF8	绺
U+7EFA.0	fGB	0-6738
U+7EFA.0	fR/S	 120.8
U+7EFA.0	fTotalStrokes	 11
U+7EFA.0	fCangjie	VMHYR
U+7EFA.0	fHakka	 liu3
U+7EFA.0	fCantonese	lau5
U+7EFA.0	fMandarin	liu3
U+7EFA.0	fEnglish	simplified form of &#x7DB9;
U+7EFB.0	fUTF8	绻
U+7EFB.0	fGB	0-6739
U+7EFB.0	fR/S	 120.8
U+7EFB.0	fTotalStrokes	 11
U+7EFB.0	fCangjie	VMFQU
U+7EFB.0	fHakka	 gen3
U+7EFB.0	fCantonese	hyun3
U+7EFB.0	fMandarin	quan3; quan4
U+7EFB.0	fEnglish	simplified form of &#x7DA3;
U+7EFC.0	fUTF8	综
U+7EFC.0	fGB	0-575B
U+7EFC.0	fR/S	 120.8
U+7EFC.0	fTotalStrokes	 11
U+7EFC.0	fCangjie	VMJMF
U+7EFC.0	fHakka	 zung5
U+7EFC.0	fCantonese	zung3; zung1
U+7EFC.0	fMandarin	zong1; zeng4; zong4
U+7EFC.0	fEnglish	simplified form of &#x7D9C;
U+7EFD.0	fUTF8	绽
U+7EFD.0	fGB	0-5540
U+7EFD.0	fR/S	 120.8
U+7EFD.0	fTotalStrokes	 11
U+7EFD.0	fCangjie	VMJMO
U+7EFD.0	fHakka	 tien5; can5
U+7EFD.0	fCantonese	zaan6
U+7EFD.0	fMandarin	zhan4; dian4
U+7EFD.0	fEnglish	simplified form of &#x7DBB;
U+7EFE.0	fUTF8	绾
U+7EFE.0	fGB	0-673A
U+7EFE.0	fR/S	 120.8
U+7EFE.0	fTotalStrokes	 11
U+7EFE.0	fCangjie	VMJRR
U+7EFE.0	fHakka	 gwon3; kwan3
U+7EFE.0	fCantonese	waan2
U+7EFE.0	fMandarin	wan3; wan4
U+7EFE.0	fEnglish	simplified form of &#x7DB0;
U+7EFF.0	fUTF8	绿
U+7EFF.0	fGB	0-424C
U+7EFF.0	fR/S	 120.8
U+7EFF.0	fTotalStrokes	 11
U+7EFF.0	fCangjie	VMNME
U+7EFF.0	fCantonese	luk6; luk2 (also)
U+7EFF.0	fMandarin	lu4; luu4; l4
U+7EFF.0	fEnglish	simplified form of &#x7DD1;
U+7F00.0	fUTF8	缀
U+7F00.0	fGB	0-573A
U+7F00.0	fR/S	 120.8
U+7F00.0	fTotalStrokes	 11
U+7F00.0	fCangjie	VMEEE
U+7F00.0	fHakka	 dot7
U+7F00.0	fCantonese	zeoi3; zyut3; zeoi6 (also); zyut6 (also)
U+7F00.0	fMandarin	zhui4; chuo4; dui4; zhuo2
U+7F00.0	fEnglish	simplified form of &#x7DB4;
U+7F01.0	fUTF8	缁
U+7F01.0	fGB	0-673B
U+7F01.0	fR/S	 120.8
U+7F01.0	fTotalStrokes	 11
U+7F01.0	fCangjie	VMVVW
U+7F01.0	fHakka	 zih1
U+7F01.0	fCantonese	zi1
U+7F01.0	fMandarin	zi1; zhi1; zi3; zi4
U+7F01.0	fEnglish	simplified form of &#x7DC7;
U+7F02.0	fUTF8	缂
U+7F02.0	fGB	0-673C
U+7F02.0	fR/S	 120.9
U+7F02.0	fTotalStrokes	 12
U+7F02.0	fCangjie	VMTLJ
U+7F02.0	fHakka	 get7; giet7
U+7F02.0	fCantonese	kaak1; kaak3 (also)
U+7F02.0	fMandarin	ke4
U+7F02.0	fEnglish	simplified form of &#x7DD9;
U+7F03.0	fUTF8	缃
U+7F03.0	fGB	0-673D
U+7F03.0	fR/S	 120.9
U+7F03.0	fTotalStrokes	 12
U+7F03.0	fCangjie	VMDBU
U+7F03.0	fHakka	 siong1
U+7F03.0	fCantonese	soeng1
U+7F03.0	fMandarin	xiang1; shuang1
U+7F03.0	fEnglish	simplified form of &#x7DD7;
U+7F04.0	fUTF8	缄
U+7F04.0	fGB	0-3C6A
U+7F04.0	fR/S	 120.9
U+7F04.0	fTotalStrokes	 12
U+7F04.0	fCangjie	VMIHR
U+7F04.0	fHakka	 kiam2
U+7F04.0	fCantonese	gaam1
U+7F04.0	fMandarin	jian1; jian4
U+7F04.0	fEnglish	simplified form of &#x7DD8;
U+7F05.0	fUTF8	缅
U+7F05.0	fGB	0-4365
U+7F05.0	fR/S	 120.9
U+7F05.0	fTotalStrokes	 12
U+7F05.0	fCangjie	VMMWL
U+7F05.0	fHakka	 men3; mien3
U+7F05.0	fCantonese	min5
U+7F05.0	fMandarin	mian3
U+7F05.0	fEnglish	simplified form of &#x7DEC;
U+7F06.0	fUTF8	缆
U+7F06.0	fGB	0-4042
U+7F06.0	fR/S	 120.9
U+7F06.0	fTotalStrokes	 12
U+7F06.0	fCangjie	VMLIU
U+7F06.0	fHakka	 lam5
U+7F06.0	fCantonese	laam6
U+7F06.0	fMandarin	lan4; lan3
U+7F06.0	fEnglish	simplified form of &#x7E9C;
U+7F07.0	fUTF8	缇
U+7F07.0	fGB	0-673E
U+7F07.0	fR/S	 120.9
U+7F07.0	fTotalStrokes	 12
U+7F07.0	fCangjie	VMAMO
U+7F07.0	fCantonese	tai4
U+7F07.0	fMandarin	ti2; di1; ti3
U+7F07.0	fEnglish	simplified form of &#x7DF9;
U+7F08.0	fUTF8	缈
U+7F08.0	fGB	0-673F
U+7F08.0	fR/S	 120.9
U+7F08.0	fTotalStrokes	 12
U+7F08.0	fCangjie	VMBUH
U+7F08.0	fHakka	 miau3
U+7F08.0	fCantonese	miu5
U+7F08.0	fMandarin	miao3
U+7F08.0	fEnglish	simplified form of &#x7DF2;
U+7F09.0	fUTF8	缉
U+7F09.0	fGB	0-3C29
U+7F09.0	fR/S	 120.9
U+7F09.0	fTotalStrokes	 12
U+7F09.0	fCangjie	VMRSJ
U+7F09.0	fHakka	 sip8
U+7F09.0	fCantonese	cap1
U+7F09.0	fMandarin	qi4; ji1; ji2; qi1
U+7F09.0	fEnglish	simplified form of &#x7DDD;
U+7F0A.0	fUTF8	缊
U+7F0A.0	fGB	8-7E58
U+7F0A.0	fR/S	 120.10; 120.9
U+7F0A.0	fTotalStrokes	 12
U+7F0A.0	fCangjie	VMABT
U+7F0A.0	fCantonese	wan3; wan1
U+7F0A.0	fMandarin	yun1; yun4; wen1
U+7F0A.0	fEnglish	tangled hemp, raveled silk; vague, confused
U+7F0B.0	fUTF8	缋
U+7F0B.0	fGB	0-6740
U+7F0B.0	fR/S	 120.9
U+7F0B.0	fTotalStrokes	 12
U+7F0B.0	fCangjie	VMLMO
U+7F0B.0	fHakka	 k'wui5; fui5
U+7F0B.0	fCantonese	kui2; kwui2 (also)
U+7F0B.0	fMandarin	hui4; hu2; hui2; kui2
U+7F0B.0	fEnglish	simplified form of &#x7E62;
U+7F0C.0	fUTF8	缌
U+7F0C.0	fGB	0-6741
U+7F0C.0	fR/S	 120.9
U+7F0C.0	fTotalStrokes	 12
U+7F0C.0	fCangjie	VMWP
U+7F0C.0	fHakka	 sii1
U+7F0C.0	fCantonese	si1
U+7F0C.0	fMandarin	si1
U+7F0C.0	fEnglish	simplified form of &#x7DE6;
U+7F0D.0	fUTF8	缍
U+7F0D.0	fGB	0-6736
U+7F0D.0	fR/S	 120.8; 120.9
U+7F0D.0	fTotalStrokes	 12
U+7F0D.0	fCangjie	VMHJM
U+7F0D.0	fCantonese	do2
U+7F0D.0	fMandarin	duo3; chui2; nuo3
U+7F0D.0	fEnglish	simplified form of &#x7D9E;
U+7F0E.0	fUTF8	缎
U+7F0E.0	fGB	0-3650
U+7F0E.0	fR/S	 120.9
U+7F0E.0	fTotalStrokes	 12
U+7F0E.0	fCangjie	VMHJE
U+7F0E.0	fHakka	 ton5
U+7F0E.0	fCantonese	dyun6; dyun2 (also)
U+7F0E.0	fMandarin	duan4
U+7F0E.0	fEnglish	simplified form of &#x7DDE;
U+7F0F.0	fUTF8	缏
U+7F0F.0	fGB	0-6742
U+7F0F.0	fR/S	 120.9
U+7F0F.0	fTotalStrokes	 12
U+7F0F.0	fCangjie	VMOMK
U+7F0F.0	fHakka	 bien1
U+7F0F.0	fCantonese	pin4; bin1
U+7F0F.0	fMandarin	bian4; pian2; bian1; bian3
U+7F0F.0	fEnglish	simplified form of &#x7DF6;
U+7F10.0	fUTF8	缐
U+7F10.0	fGB	8-2F71
U+7F10.0	fR/S	 120.9
U+7F10.0	fTotalStrokes	 12
U+7F10.0	fCangjie	VMHAE
U+7F10.0	fCantonese	sin3
U+7F10.0	fMandarin	xian4
U+7F11.0	fUTF8	缑
U+7F11.0	fGB	0-6743
U+7F11.0	fR/S	 120.9
U+7F11.0	fTotalStrokes	 12
U+7F11.0	fCangjie	VMONK
U+7F11.0	fHakka	 geu1
U+7F11.0	fCantonese	gau1; kau1
U+7F11.0	fMandarin	gou1; kou1
U+7F11.0	fEnglish	simplified form of &#x7DF1;
U+7F12.0	fUTF8	缒
U+7F12.0	fGB	0-6744
U+7F12.0	fR/S	 120.9
U+7F12.0	fTotalStrokes	 12
U+7F12.0	fCangjie	VMYHR
U+7F12.0	fHakka	 ch'ui5
U+7F12.0	fCantonese	zeoi6
U+7F12.0	fMandarin	zhui4
U+7F12.0	fEnglish	simplified form of &#x7E0B;
U+7F13.0	fUTF8	缓
U+7F13.0	fGB	0-3B3A
U+7F13.0	fR/S	 120.9
U+7F13.0	fTotalStrokes	 12
U+7F13.0	fCangjie	VMBME
U+7F13.0	fHakka	 man5; fon5; fon3
U+7F13.0	fCantonese	wun6; wun4 (also)
U+7F13.0	fMandarin	huan3; kuan3; xuan3
U+7F13.0	fEnglish	simplified form of &#x7DE9;
U+7F14.0	fUTF8	缔
U+7F14.0	fGB	0-355E
U+7F14.0	fR/S	 120.9
U+7F14.0	fTotalStrokes	 12
U+7F14.0	fCangjie	VMYBB
U+7F14.0	fHakka	 ti5
U+7F14.0	fCantonese	dai3; tai3 (also)
U+7F14.0	fMandarin	di4; ti2; zhi4
U+7F14.0	fEnglish	simplified form of &#x7DE0;
U+7F15.0	fUTF8	缕
U+7F15.0	fGB	0-4246
U+7F15.0	fR/S	 120.9
U+7F15.0	fTotalStrokes	 12
U+7F15.0	fCangjie	VMFDV
U+7F15.0	fHakka	 leu3
U+7F15.0	fCantonese	leoi5; lau5 (also); lau2 (also)
U+7F15.0	fMandarin	l3; luu3; lou2; lu3
U+7F15.0	fEnglish	simplified form of &#x7E37;
U+7F16.0	fUTF8	编
U+7F16.0	fGB	0-3160
U+7F16.0	fR/S	 120.9
U+7F16.0	fTotalStrokes	 12
U+7F16.0	fCangjie	VMISB
U+7F16.0	fHakka	 pien1
U+7F16.0	fCantonese	pin1
U+7F16.0	fMandarin	bian1; bian3; bian4
U+7F16.0	fEnglish	simplified form of &#x7DE8;
U+7F17.0	fUTF8	缗
U+7F17.0	fGB	0-6745
U+7F17.0	fR/S	 120.9
U+7F17.0	fTotalStrokes	 12
U+7F17.0	fCangjie	VMRPA
U+7F17.0	fHakka	 min2; men2
U+7F17.0	fCantonese	man4; fan1
U+7F17.0	fMandarin	min2; hun1; mian2; min3
U+7F17.0	fEnglish	simplified form of &#x7DE1;
U+7F18.0	fUTF8	缘
U+7F18.0	fGB	0-5435
U+7F18.0	fR/S	 120.9
U+7F18.0	fTotalStrokes	 12
U+7F18.0	fCangjie	VMVNO
U+7F18.0	fHakka	 jen2
U+7F18.0	fCantonese	jyun4; jyun6
U+7F18.0	fMandarin	yuan2; yung4; yuan4
U+7F18.0	fEnglish	simplified form of &#x7DE3;
U+7F19.0	fUTF8	缙
U+7F19.0	fGB	0-6746
U+7F19.0	fR/S	 120.10
U+7F19.0	fTotalStrokes	 13
U+7F19.0	fCangjie	VMMCA
U+7F19.0	fHakka	 zin5
U+7F19.0	fCantonese	zeon3
U+7F19.0	fMandarin	jin4; zi1
U+7F19.0	fEnglish	simplified form of &#x7E09;
U+7F1A.0	fUTF8	缚
U+7F1A.0	fGB	0-383F
U+7F1A.0	fR/S	 120.10
U+7F1A.0	fTotalStrokes	 13
U+7F1A.0	fCangjie	VMIBI
U+7F1A.0	fHakka	 p'iok8
U+7F1A.0	fCantonese	bok3; fok3 (also)
U+7F1A.0	fMandarin	fu2; fo4; fu4
U+7F1A.0	fEnglish	simplified form of &#x7E1B;
U+7F1B.0	fUTF8	缛
U+7F1B.0	fGB	0-6748
U+7F1B.0	fR/S	 120.10
U+7F1B.0	fTotalStrokes	 13
U+7F1B.0	fCangjie	VMMVI
U+7F1B.0	fHakka	 juk8
U+7F1B.0	fCantonese	juk6
U+7F1B.0	fMandarin	ru4
U+7F1B.0	fEnglish	simplified form of &#x7E1F;
U+7F1C.0	fUTF8	缜
U+7F1C.0	fGB	0-6747
U+7F1C.0	fR/S	 120.10
U+7F1C.0	fTotalStrokes	 13
U+7F1C.0	fCangjie	VMJBC
U+7F1C.0	fHakka	 zhin3
U+7F1C.0	fCantonese	zan2; can2 (also)
U+7F1C.0	fMandarin	zhen3; zhen1; chen1
U+7F1C.0	fEnglish	simplified form of &#x7E1D;
U+7F1D.0	fUTF8	缝
U+7F1D.0	fGB	0-376C
U+7F1D.0	fR/S	 120.10
U+7F1D.0	fTotalStrokes	 13
U+7F1D.0	fCangjie	VMYHJ
U+7F1D.0	fHakka	 fung2; pung2; pung5
U+7F1D.0	fCantonese	fung4; fung2 (also); fung6
U+7F1D.0	fMandarin	feng4
U+7F1D.0	fEnglish	simplified form of &#x7E2B;
U+7F1D.1	fUTF8	缝
U+7F1D.1	fGB	0-376C
U+7F1D.1	fR/S	 120.10
U+7F1D.1	fTotalStrokes	 13
U+7F1D.1	fCangjie	VMYHJ
U+7F1D.1	fMandarin	feng2
U+7F1E.0	fUTF8	缞
U+7F1E.0	fGB	8-2E77
U+7F1E.0	fR/S	 120.10
U+7F1E.0	fTotalStrokes	 13
U+7F1E.0	fCangjie	VMYWV
U+7F1E.0	fCantonese	ceoi1
U+7F1E.0	fMandarin	shuai1; cui1
U+7F1E.0	fEnglish	sackcloth worn on breast during
U+7F1F.0	fUTF8	缟
U+7F1F.0	fGB	0-6749
U+7F1F.0	fR/S	 120.10
U+7F1F.0	fTotalStrokes	 13
U+7F1F.0	fCangjie	VMYRB
U+7F1F.0	fHakka	 gau3
U+7F1F.0	fCantonese	gou2
U+7F1F.0	fMandarin	gao3; gao4
U+7F1F.0	fEnglish	simplified form of &#x7E1E;
U+7F20.0	fUTF8	缠
U+7F20.0	fGB	0-3278
U+7F20.0	fR/S	 120.10
U+7F20.0	fTotalStrokes	 13
U+7F20.0	fCangjie	VMIWG
U+7F20.0	fHakka	 chen2
U+7F20.0	fCantonese	cin4; zin6 (also)
U+7F20.0	fMandarin	chan2; chan4
U+7F20.0	fEnglish	simplified form of &#x7E8F;
U+7F21.0	fUTF8	缡
U+7F21.0	fGB	0-674A
U+7F21.0	fR/S	 120.10
U+7F21.0	fTotalStrokes	 13
U+7F21.0	fCangjie	VMYUB
U+7F21.0	fHakka	 li2
U+7F21.0	fCantonese	lei4
U+7F21.0	fMandarin	li2; chi1
U+7F21.0	fEnglish	simplified form of &#x7E2D;
U+7F22.0	fUTF8	缢
U+7F22.0	fGB	0-674B
U+7F22.0	fR/S	 120.10
U+7F22.0	fTotalStrokes	 13
U+7F22.0	fCangjie	VMTCT
U+7F22.0	fHakka	 ji5
U+7F22.0	fCantonese	ai3; ngai3 (also)
U+7F22.0	fMandarin	yi4
U+7F22.0	fEnglish	simplified form of &#x7E0A;
U+7F23.0	fUTF8	缣
U+7F23.0	fGB	0-674C
U+7F23.0	fR/S	 120.10
U+7F23.0	fTotalStrokes	 13
U+7F23.0	fCangjie	VMTXC
U+7F23.0	fHakka	 giam1
U+7F23.0	fCantonese	gim1
U+7F23.0	fMandarin	jian1
U+7F23.0	fEnglish	simplified form of &#x7E11;
U+7F24.0	fUTF8	缤
U+7F24.0	fGB	0-674D
U+7F24.0	fR/S	 120.10
U+7F24.0	fTotalStrokes	 13
U+7F24.0	fCangjie	VMJOC
U+7F24.0	fHakka	 bin1
U+7F24.0	fCantonese	ban1
U+7F24.0	fMandarin	bin1
U+7F24.0	fEnglish	simplified form of &#x7E7D;
U+7F25.0	fUTF8	缥
U+7F25.0	fGB	0-674E
U+7F25.0	fR/S	 120.11
U+7F25.0	fTotalStrokes	 14
U+7F25.0	fCangjie	VMMWF
U+7F25.0	fHakka	 piau5
U+7F25.0	fCantonese	piu5; piu1
U+7F25.0	fMandarin	piao3; piao1; piao4
U+7F25.0	fEnglish	simplified form of &#x7E39;
U+7F26.0	fUTF8	缦
U+7F26.0	fGB	0-674F
U+7F26.0	fR/S	 120.11
U+7F26.0	fTotalStrokes	 14
U+7F26.0	fCangjie	VMAWE
U+7F26.0	fHakka	 man5
U+7F26.0	fCantonese	maan6
U+7F26.0	fMandarin	man4; man2
U+7F26.0	fEnglish	simplified form of &#x7E35;
U+7F27.0	fUTF8	缧
U+7F27.0	fGB	0-6750
U+7F27.0	fR/S	 120.11
U+7F27.0	fTotalStrokes	 14
U+7F27.0	fCangjie	VMWVF
U+7F27.0	fHakka	 lui3; lui1
U+7F27.0	fCantonese	leoi4
U+7F27.0	fMandarin	lei2; lei3; lei4; luo2
U+7F27.0	fEnglish	simplified form of &#x7E32;
U+7F28.0	fUTF8	缨
U+7F28.0	fGB	0-5327
U+7F28.0	fR/S	 120.11
U+7F28.0	fTotalStrokes	 14
U+7F28.0	fCangjie	VMBOV
U+7F28.0	fHakka	 jin1
U+7F28.0	fCantonese	jing1
U+7F28.0	fMandarin	ying1; ying4
U+7F28.0	fEnglish	simplified form of &#x7E93;
U+7F29.0	fUTF8	缩
U+7F29.0	fGB	0-4B75
U+7F29.0	fR/S	 120.11
U+7F29.0	fTotalStrokes	 14
U+7F29.0	fCangjie	VMJOA
U+7F29.0	fHakka	 suk7
U+7F29.0	fCantonese	suk1
U+7F29.0	fMandarin	suo1; su4
U+7F29.0	fEnglish	simplified form of &#x7E2E;
U+7F2A.0	fUTF8	缪
U+7F2A.0	fGB	0-6751
U+7F2A.0	fR/S	 120.11
U+7F2A.0	fTotalStrokes	 14
U+7F2A.0	fCangjie	VMSMH
U+7F2A.0	fHakka	 miau3; meu5; muk7
U+7F2A.0	fCantonese	mau4; mau6; miu6; maau6 (also); muk6
U+7F2A.0	fMandarin	mou2; jiu1; liao3; liao2; miao4; miu4; liao4; miao2miu4}; mu4
U+7F2A.0	fEnglish	simplified form of &#x7E46;
U+7F2B.0	fUTF8	缫
U+7F2B.0	fGB	0-6752
U+7F2B.0	fR/S	 120.11
U+7F2B.0	fTotalStrokes	 14
U+7F2B.0	fCangjie	VMVVD
U+7F2B.0	fHakka	 sau1; cau2
U+7F2B.0	fCantonese	sou1
U+7F2B.0	fMandarin	sao1; zao3
U+7F2B.0	fEnglish	simplified form of &#x7E45;
U+7F2C.0	fUTF8	缬
U+7F2C.0	fGB	0-6753
U+7F2C.0	fR/S	 120.12
U+7F2C.0	fTotalStrokes	 15
U+7F2C.0	fCangjie	VMGRO
U+7F2C.0	fHakka	 get7; giet7
U+7F2C.0	fCantonese	kit3; lit3; hit3 (also)
U+7F2C.0	fMandarin	xie2
U+7F2C.0	fEnglish	simplified form of &#x7E88;
U+7F2D.0	fUTF8	缭
U+7F2D.0	fGB	0-6754
U+7F2D.0	fR/S	 120.12
U+7F2D.0	fTotalStrokes	 15
U+7F2D.0	fCangjie	VMKCF
U+7F2D.0	fHakka	 liau2
U+7F2D.0	fCantonese	liu4; liu1 (also)
U+7F2D.0	fMandarin	liao2; liao3; liao4; rao3
U+7F2D.0	fEnglish	simplified form of &#x7E5A;
U+7F2E.0	fUTF8	缮
U+7F2E.0	fGB	0-4949
U+7F2E.0	fR/S	 120.12
U+7F2E.0	fTotalStrokes	 15
U+7F2E.0	fCangjie	VMTTR
U+7F2E.0	fHakka	 shen3
U+7F2E.0	fCantonese	sin6
U+7F2E.0	fMandarin	shan4
U+7F2E.0	fEnglish	simplified form of &#x7E55;
U+7F2F.0	fUTF8	缯
U+7F2F.0	fGB	0-6755
U+7F2F.0	fR/S	 120.12
U+7F2F.0	fTotalStrokes	 15
U+7F2F.0	fCangjie	VMCWA
U+7F2F.0	fHakka	 zen1
U+7F2F.0	fCantonese	zang1; zang6
U+7F2F.0	fMandarin	zeng1; ceng2
U+7F2F.0	fEnglish	simplified form of &#x7E52;
U+7F30.0	fUTF8	缰
U+7F30.0	fGB	0-6756
U+7F30.0	fR/S	 120.13
U+7F30.0	fTotalStrokes	 16
U+7F30.0	fCangjie	VMMWM
U+7F30.0	fHakka	 giong1
U+7F30.0	fCantonese	goeng1
U+7F30.0	fMandarin	jiang1
U+7F30.0	fEnglish	simplified form of &#x7E6E;
U+7F31.0	fUTF8	缱
U+7F31.0	fGB	0-6757
U+7F31.0	fR/S	 120.13
U+7F31.0	fTotalStrokes	 16
U+7F31.0	fCangjie	VMYLR
U+7F31.0	fHakka	 ken3; kien5
U+7F31.0	fCantonese	hin2
U+7F31.0	fMandarin	qian3; qian4
U+7F31.0	fEnglish	simplified form of &#x7E7E;
U+7F32.0	fUTF8	缲
U+7F32.0	fGB	0-6758
U+7F32.0	fR/S	 120.13
U+7F32.0	fTotalStrokes	 16
U+7F32.0	fCangjie	VMRRD
U+7F32.0	fHakka	 zau3
U+7F32.0	fCantonese	sou1; ciu1; zou2; tiu1
U+7F32.0	fMandarin	zao3; qiao1; qiao3; sao1
U+7F32.0	fEnglish	simplified form of &#x7E70;
U+7F33.0	fUTF8	缳
U+7F33.0	fGB	0-6759
U+7F33.0	fR/S	 120.13
U+7F33.0	fTotalStrokes	 16
U+7F33.0	fCangjie	VMWLV
U+7F33.0	fHakka	 fan5
U+7F33.0	fCantonese	waan4; waan6 (also)
U+7F33.0	fMandarin	huan2; huan4; xuan4
U+7F33.0	fEnglish	simplified form of &#x7E6F;
U+7F34.0	fUTF8	缴
U+7F34.0	fGB	0-3D49
U+7F34.0	fR/S	 120.13
U+7F34.0	fTotalStrokes	 16
U+7F34.0	fCangjie	VMHSK
U+7F34.0	fHakka	 giau3
U+7F34.0	fCantonese	giu2; zoek3
U+7F34.0	fMandarin	jiao3; jia3; zhuo2; he2; jiao4
U+7F34.0	fEnglish	simplified form of &#x7E73;
U+7F35.0	fUTF8	缵
U+7F35.0	fGB	0-675A
U+7F35.0	fR/S	 120.16
U+7F35.0	fTotalStrokes	 19
U+7F35.0	fCangjie	VMHUO
U+7F35.0	fHakka	 zan5
U+7F35.0	fCantonese	zyun2
U+7F35.0	fMandarin	zuan3
U+7F35.0	fEnglish	simplified form of &#x7E98;
U+7F36.0	fUTF8	缶
U+7F36.0	fCNS11643	1-484F
U+7F36.0	fBig5	A6CE
U+7F36.0	fGB	0-733E
U+7F36.0	fR/S	 121.0
U+7F36.0	fTotalStrokes	 6
U+7F36.0	fCangjie	OJU
U+7F36.0	fFourCorner	80772
U+7F36.0	fHakka	 feu3; fui2; fiu3; fi2
U+7F36.0	fCantonese	fau2
U+7F36.0	fMandarin	fou3; guan4
U+7F36.0	fEnglish	[1] earthen crock or jar with a narrow opening [2] Kangxi radical 121
U+7F37.0	fUTF8	缷
U+7F37.0	fCNS11643	4-2557
U+7F37.0	fGB	E-386D
U+7F37.0	fR/S	 121.2
U+7F37.0	fTotalStrokes	 8; 7
U+7F37.0	fCangjie	OUSL
U+7F37.0	fFourCorner	27720
U+7F37.0	fCantonese	se3
U+7F37.0	fMandarin	xie4
U+7F37.0	fEnglish	 lay down retire from office
U+7F38.0	fUTF8	缸
U+7F38.0	fCNS11643	1-527E
U+7F38.0	fBig5	ACFB
U+7F38.0	fGB	0-3857
U+7F38.0	fR/S	 121.3
U+7F38.0	fTotalStrokes	 9
U+7F38.0	fCangjie	OUM
U+7F38.0	fFourCorner	81710
U+7F38.0	fHakka	 gong1; gong2
U+7F38.0	fCantonese	gong1
U+7F38.0	fMandarin	gang1; xiang2; hong2
U+7F38.0	fEnglish	earthen jug; crock; cistern
U+7F39.0	fUTF8	缹
U+7F39.0	fCNS11643	2-3052
U+7F39.0	fBig5	D26F
U+7F39.0	fGB	5-625D
U+7F39.0	fR/S	 121.4; 86.6
U+7F39.0	fTotalStrokes	 10
U+7F39.0	fCangjie	OUF
U+7F39.0	fFourCorner	80337
U+7F39.0	fCantonese	fau2; bou1
U+7F39.0	fMandarin	fou3
U+7F39.0	fEnglish	cook
U+7F3A.0	fUTF8	缺
U+7F3A.0	fCNS11643	1-574D
U+7F3A.0	fBig5	AFCA
U+7F3A.0	fGB	0-4831
U+7F3A.0	fR/S	 121.4
U+7F3A.0	fTotalStrokes	 10
U+7F3A.0	fCangjie	OUDK
U+7F3A.0	fFourCorner	85730
U+7F3A.0	fHakka	 kiet7; ket7; k'iat7
U+7F3A.0	fCantonese	kyut3
U+7F3A.0	fMandarin	que1; que4; kui3; quan4
U+7F3A.0	fEnglish	[1] be short of; lack; short; gap; deficit; deficient; lacking; incomplete; defective [2] vacancy; opening
U+7F3B.0	fUTF8	缻
U+7F3B.0	fCNS11643	3-3831
U+7F3B.0	fGB	5-625B
U+7F3B.0	fR/S	 121.5
U+7F3B.0	fTotalStrokes	 11
U+7F3B.0	fCangjie	OUMVN
U+7F3B.0	fFourCorner	81710
U+7F3B.0	fHakka	 feu3; fui1; fi1; feu2
U+7F3B.0	fCantonese	fau2
U+7F3B.0	fMandarin	fou3
U+7F3B.0	fEnglish	pottery; earthenware
U+7F3C.0	fUTF8	缼
U+7F3C.0	fCNS11643	F-3178
U+7F3C.0	fGB	E-386E
U+7F3C.0	fR/S	 121.4
U+7F3C.0	fTotalStrokes	 10
U+7F3C.0	fCangjie	OUNO
U+7F3C.0	fHakka	 kim1
U+7F3C.0	fCantonese	kik1
U+7F3C.0	fMandarin	que1
U+7F3D.0	fUTF8	缽
U+7F3D.0	fCNS11643	1-5C5F
U+7F3D.0	fBig5	B2DA
U+7F3D.0	fGB	E-386F
U+7F3D.0	fR/S	 121.5
U+7F3D.0	fTotalStrokes	 11
U+7F3D.0	fCangjie	OUDM
U+7F3D.0	fFourCorner	84749
U+7F3D.0	fHakka	 bat7
U+7F3D.0	fCantonese	but6; but3
U+7F3D.0	fMandarin	bo1
U+7F3D.0	fEnglish	 earthenware basin
U+7F3E.0	fUTF8	缾
U+7F3E.0	fCNS11643	2-3F22
U+7F3E.0	fBig5	DAFC
U+7F3E.0	fGB	E-3870
U+7F3E.0	fR/S	 121.6
U+7F3E.0	fTotalStrokes	 12
U+7F3E.0	fCangjie	OUTT
U+7F3E.0	fFourCorner	28741
U+7F3E.0	fHakka	 pin2;  piang2; pen2
U+7F3E.0	fCantonese	ping4
U+7F3E.0	fMandarin	ping2
U+7F3E.0	fEnglish	 jar; bottle; pitcher
U+7F3F.0	fUTF8	缿
U+7F3F.0	fCNS11643	2-3F23
U+7F3F.0	fBig5	DAFD
U+7F3F.0	fGB	3-633D
U+7F3F.0	fR/S	 121.6
U+7F3F.0	fTotalStrokes	 12
U+7F3F.0	fCangjie	OUHMR
U+7F3F.0	fFourCorner	22761
U+7F3F.0	fCantonese	hong6; hau6 (also); hoeng3
U+7F3F.0	fMandarin	xiang4; hou4; dou4
U+7F40.0	fUTF8	罀
U+7F40.0	fCNS11643	3-662D
U+7F40.0	fGB	E-3871
U+7F40.0	fR/S	 121.6
U+7F40.0	fTotalStrokes	 12
U+7F40.0	fCangjie	OULMO
U+7F40.0	fFourCorner	8271
U+7F40.0	fHakka	 tiau5; tiau6
U+7F40.0	fCantonese	siu6; diu6
U+7F40.0	fEnglish	 earthen jar
U+7F41.0	fUTF8	罁
U+7F41.0	fCNS11643	3-4877
U+7F41.0	fGB	5-625C
U+7F41.0	fR/S	 121.8
U+7F41.0	fTotalStrokes	 14
U+7F41.0	fCangjie	OUBTU
U+7F41.0	fFourCorner	27720
U+7F41.0	fHakka	 gong1
U+7F41.0	fCantonese	gong1
U+7F41.0	fMandarin	gang1
U+7F41.0	fEnglish	 cistern earthenware jar of large dimensions vat crock
U+7F42.0	fUTF8	罂
U+7F42.0	fGB	0-733F
U+7F42.0	fR/S	 121.8
U+7F42.0	fTotalStrokes	 14
U+7F42.0	fCangjie	BOOJU
U+7F42.0	fHakka	 ang1
U+7F42.0	fCantonese	aang1; ngaang1 (also)
U+7F42.0	fMandarin	ying1; ying4
U+7F42.0	fEnglish	simplified form of &#x7F4C;
U+7F43.0	fUTF8	罃
U+7F43.0	fCNS11643	2-594B
U+7F43.0	fBig5	EADF
U+7F43.0	fGB	3-633A
U+7F43.0	fR/S	 121.10
U+7F43.0	fTotalStrokes	 16
U+7F43.0	fCangjie	FFBOU
U+7F43.0	fFourCorner	99772
U+7F43.0	fHakka	 ang1; rin1; jin1
U+7F43.0	fCantonese	aang1; ang1 (also); ngaang1 (also); ngang1 (also)
U+7F43.0	fMandarin	ying1
U+7F43.0	fEnglish	 long-necked jar or bottle
U+7F44.0	fUTF8	罄
U+7F44.0	fCNS11643	1-7474
U+7F44.0	fBig5	C16A
U+7F44.0	fGB	0-7340
U+7F44.0	fR/S	 121.11
U+7F44.0	fTotalStrokes	 17
U+7F44.0	fCangjie	GEOJU
U+7F44.0	fFourCorner	47774
U+7F44.0	fHakka	 kin5;  kin1
U+7F44.0	fCantonese	hing3
U+7F44.0	fMandarin	qing4; qing3
U+7F44.0	fEnglish	 exhaust; run out; use up; empty
U+7F45.0	fUTF8	罅
U+7F45.0	fCNS11643	2-5E4E
U+7F45.0	fBig5	EDE1
U+7F45.0	fGB	0-7341
U+7F45.0	fR/S	 121.11
U+7F45.0	fTotalStrokes	 17
U+7F45.0	fCangjie	OUYPD
U+7F45.0	fFourCorner	81749
U+7F45.0	fHakka	  la5; fu1
U+7F45.0	fCantonese	laa3
U+7F45.0	fMandarin	xia4
U+7F45.0	fEnglish	 crack; fissure; split; rift; cleft; flaw
U+7F46.0	fUTF8	罆
U+7F46.0	fCNS11643	3-557D
U+7F46.0	fGB	5-6265
U+7F46.0	fR/S	  121.11
U+7F46.0	fTotalStrokes	 17
U+7F46.0	fCangjie	OUWJC
U+7F46.0	fFourCorner	87780
U+7F46.0	fCantonese	gun3; gwun3 (also)
U+7F46.0	fMandarin	guan4
U+7F46.0	fEnglish	jar; jug; pitcher; pot
U+7F47.0	fUTF8	罇
U+7F47.0	fCNS11643	3-585C
U+7F47.0	fGB	E-3872
U+7F47.0	fR/S	 121.12
U+7F47.0	fTotalStrokes	 18
U+7F47.0	fCangjie	OUTWI
U+7F47.0	fFourCorner	88740
U+7F47.0	fHakka	  zun1
U+7F47.0	fCantonese	zeon1
U+7F47.0	fMandarin	zun1
U+7F47.0	fEnglish	[1] goblet; bottle [2] wine-jar
U+7F48.0	fUTF8	罈
U+7F48.0	fCNS11643	1-7727
U+7F48.0	fBig5	C2BB
U+7F48.0	fGB	E-3873
U+7F48.0	fR/S	 121.12
U+7F48.0	fTotalStrokes	 18
U+7F48.0	fCangjie	OUMWJ
U+7F48.0	fFourCorner	21746
U+7F48.0	fHakka	 tam2
U+7F48.0	fCantonese	taam4
U+7F48.0	fMandarin	tan2
U+7F48.0	fEnglish	same as $ ( earthenware jar; jug)
U+7F49.0	fUTF8	罉
U+7F49.0	fGB	1-7C47
U+7F49.0	fR/S	 121.12
U+7F49.0	fTotalStrokes	 18
U+7F49.0	fCangjie	OUFBQ
U+7F49.0	fCantonese	caang1; cang1
U+7F49.0	fEnglish	 a pot
U+7F4A.0	fUTF8	罊
U+7F4A.0	fCNS11643	2-6644
U+7F4A.0	fBig5	F2BA
U+7F4A.0	fGB	E-3874
U+7F4A.0	fR/S	 121.13
U+7F4A.0	fTotalStrokes	 19
U+7F4A.0	fCangjie	JEOJU
U+7F4A.0	fFourCorner	07771
U+7F4A.0	fHakka	 ke5; ki5
U+7F4A.0	fCantonese	kai3; hak1; hai4
U+7F4A.0	fMandarin	qi4; ke4
U+7F4A.0	fEnglish	vessel emptied of its contents
U+7F4B.0	fUTF8	罋
U+7F4B.0	fCNS11643	2-6643
U+7F4B.0	fBig5	F2B9
U+7F4B.0	fGB	E-3875
U+7F4B.0	fR/S	 121.13
U+7F4B.0	fTotalStrokes	 19
U+7F4B.0	fCangjie	YVGU
U+7F4B.0	fFourCorner	00772
U+7F4B.0	fHakka	 wung5; vung5
U+7F4B.0	fCantonese	ung3; ngung3 (also); ong3
U+7F4B.0	fMandarin	weng4; yong1
U+7F4B.0	fEnglish	 earthen jar jar for ashes
U+7F4C.0	fUTF8	罌
U+7F4C.0	fCNS11643	1-7A2D
U+7F4C.0	fBig5	C4A2
U+7F4C.0	fGB	1-733F
U+7F4C.0	fR/S	 121.14
U+7F4C.0	fTotalStrokes	 20
U+7F4C.0	fCangjie	BCOJU
U+7F4C.0	fFourCorner	66772
U+7F4C.0	fHakka	  ang1; jin1; rin1
U+7F4C.0	fCantonese	aang1; ngaang1 (also); ang1; ngang1
U+7F4C.0	fMandarin	ying1; ying4
U+7F4C.0	fEnglish	 long necked jar or bottle with a small mouth
U+7F4D.0	fUTF8	罍
U+7F4D.0	fCNS11643	2-6B7B
U+7F4D.0	fBig5	F5ED
U+7F4D.0	fGB	3-6341
U+7F4D.0	fR/S	 121.15
U+7F4D.0	fTotalStrokes	 21
U+7F4D.0	fCangjie	WWWU
U+7F4D.0	fFourCorner	60772
U+7F4D.0	fHakka	  lui2
U+7F4D.0	fCantonese	leoi4
U+7F4D.0	fMandarin	lei2
U+7F4D.0	fEnglish	 large earthenware wine jar
U+7F4E.0	fUTF8	罎
U+7F4E.0	fCNS11643	3-5F5B
U+7F4E.0	fGB	1-786E
U+7F4E.0	fR/S	 121.16
U+7F4E.0	fTotalStrokes	 22
U+7F4E.0	fCangjie	OUAMI
U+7F4E.0	fFourCorner	86730
U+7F4E.0	fHakka	 tam2
U+7F4E.0	fCantonese	taam4
U+7F4E.0	fMandarin	tan2
U+7F4E.0	fEnglish	 earthenware jar or jug for wine
U+7F4F.0	fUTF8	罏
U+7F4F.0	fCNS11643	2-6D70
U+7F4F.0	fBig5	F743
U+7F4F.0	fGB	3-633B
U+7F4F.0	fR/S	 121.16
U+7F4F.0	fTotalStrokes	 22
U+7F4F.0	fCangjie	OUYPT
U+7F4F.0	fFourCorner	21717
U+7F4F.0	fHakka	  lu2
U+7F4F.0	fCantonese	lou4
U+7F4F.0	fMandarin	lu2
U+7F4F.0	fEnglish	 earthen stand for wine jars
U+7F50.0	fUTF8	罐
U+7F50.0	fCNS11643	1-7C64
U+7F50.0	fBig5	C5F8
U+7F50.0	fGB	0-395E
U+7F50.0	fR/S	 121.18
U+7F50.0	fTotalStrokes	 24
U+7F50.0	fCangjie	OUTRG
U+7F50.0	fFourCorner	84714
U+7F50.0	fHakka	 gon5;  kwan5; gwon5
U+7F50.0	fCantonese	gun3; gwun3 (also)
U+7F50.0	fMandarin	guan4
U+7F50.0	fEnglish	 vessel; container; jar; jug; pitcher; pot; can
U+7F51.0	fUTF8	网
U+7F51.0	fCNS11643	2-2269
U+7F51.0	fBig5	CA49
U+7F51.0	fGB	0-4D78
U+7F51.0	fR/S	 122.0
U+7F51.0	fTotalStrokes	 6
U+7F51.0	fCangjie	BKK
U+7F51.0	fFourCorner	77220
U+7F51.0	fHakka	 miong3
U+7F51.0	fCantonese	mong5
U+7F51.0	fMandarin	wang3
U+7F51.0	fEnglish	simplified form of &#x7DB2;
U+7F52.0	fUTF8	罒
U+7F52.0	fCNS11643	4-2221
U+7F52.0	fGB	E-3876
U+7F52.0	fR/S	 122.0
U+7F52.0	fTotalStrokes	 5
U+7F52.0	fCangjie	WLL
U+7F52.0	fFourCorner	60000
U+7F52.0	fCantonese	muk6
U+7F52.0	fMandarin	wang3
U+7F52.0	fEnglish	a variant of Kangxi radical 122
U+7F53.0	fUTF8	罓
U+7F53.0	fGB	E-3877
U+7F53.0	fR/S	 122.0
U+7F53.0	fTotalStrokes	 4
U+7F53.0	fCangjie	BK
U+7F53.0	fMandarin	gang1
U+7F54.0	fUTF8	罔
U+7F54.0	fCNS11643	1-4F2C
U+7F54.0	fBig5	AAC9
U+7F54.0	fGB	0-5868
U+7F54.0	fR/S	 122.3
U+7F54.0	fTotalStrokes	 8
U+7F54.0	fCangjie	BTYV
U+7F54.0	fFourCorner	77220
U+7F54.0	fHakka	 wong3;  mong3; miong3; vong3
U+7F54.0	fCantonese	mong5
U+7F54.0	fMandarin	wang3; wang2
U+7F54.0	fEnglish	[1] slander; deceive; libel [2] not straight; crooked [3] not; negative
U+7F55.0	fUTF8	罕
U+7F55.0	fCNS11643	1-4B38
U+7F55.0	fBig5	A875
U+7F55.0	fGB	0-3A31
U+7F55.0	fR/S	 122.3
U+7F55.0	fTotalStrokes	 8
U+7F55.0	fCangjie	BCMJ
U+7F55.0	fFourCorner	37401
U+7F55.0	fHakka	  hon3; han3
U+7F55.0	fCantonese	hon2
U+7F55.0	fMandarin	han3; han4
U+7F55.0	fEnglish	rare; scarce; few; seldom
U+7F56.0	fUTF8	罖
U+7F56.0	fGB	1-7D38
U+7F56.0	fR/S	 122.3
U+7F56.0	fTotalStrokes	 8
U+7F56.0	fCangjie	WLOI
U+7F56.0	fCantonese	mong5
U+7F56.0	fEnglish	 kwukyel
U+7F57.0	fUTF8	罗
U+7F57.0	fCNS11643	F-284A
U+7F57.0	fGB	0-425E
U+7F57.0	fR/S	 122.3
U+7F57.0	fTotalStrokes	 8
U+7F57.0	fCangjie	WLNI
U+7F57.0	fHakka	 lo2
U+7F57.0	fCantonese	lo4; lo2 (also); lo1
U+7F57.0	fMandarin	luo2; luo1; li2; luo4; luo5
U+7F57.0	fEnglish	simplified form of &#x7F85;
U+7F58.0	fUTF8	罘
U+7F58.0	fCNS11643	2-2C6E
U+7F58.0	fBig5	D04D
U+7F58.0	fGB	0-6E37
U+7F58.0	fR/S	 122.4
U+7F58.0	fTotalStrokes	 9
U+7F58.0	fCangjie	WLMF
U+7F58.0	fFourCorner	60901
U+7F58.0	fHakka	  feu2; fe1
U+7F58.0	fCantonese	fau4
U+7F58.0	fMandarin	fu2; fou2; fu1; pei2; fu3
U+7F58.0	fEnglish	a net for catching rabbits or hares
U+7F59.0	fUTF8	罙
U+7F59.0	fCNS11643	3-2A4E
U+7F59.0	fGB	E-3878
U+7F59.0	fR/S	 122.4; 122.6; 14.6
U+7F59.0	fTotalStrokes	 8; 9
U+7F59.0	fCangjie	BCD
U+7F59.0	fFourCorner	37900
U+7F59.0	fCantonese	sam1
U+7F59.0	fMandarin	mi2; mi3; shen1
U+7F59.0	fEnglish	deep
U+7F5A.0	fUTF8	罚
U+7F5A.0	fGB	0-3723
U+7F5A.0	fR/S	 122.4
U+7F5A.0	fTotalStrokes	 9
U+7F5A.0	fCangjie	WLIVN
U+7F5A.0	fHakka	 fat8
U+7F5A.0	fCantonese	fat6
U+7F5A.0	fMandarin	fa2
U+7F5A.0	fEnglish	simplified form of &#x7F70;
U+7F5B.0	fUTF8	罛
U+7F5B.0	fCNS11643	2-3224
U+7F5B.0	fBig5	D360
U+7F5B.0	fGB	3-5B21
U+7F5B.0	fR/S	 122.5; 122.6
U+7F5B.0	fTotalStrokes	 10
U+7F5B.0	fCangjie	WLHVO
U+7F5B.0	fFourCorner	60232
U+7F5B.0	fHakka	 gu1
U+7F5B.0	fCantonese	gu1; gwu1 (also)
U+7F5B.0	fMandarin	gu1
U+7F5B.0	fEnglish	large fishing net
U+7F5C.0	fUTF8	罜
U+7F5C.0	fCNS11643	2-317D
U+7F5C.0	fBig5	D35B
U+7F5C.0	fGB	3-5B23
U+7F5C.0	fR/S	 122.5
U+7F5C.0	fTotalStrokes	 10
U+7F5C.0	fCangjie	WLYG
U+7F5C.0	fFourCorner	60104
U+7F5C.0	fCantonese	zyu3 (also); zyu2
U+7F5C.0	fMandarin	zhu3; du2; zhu4
U+7F5D.0	fUTF8	罝
U+7F5D.0	fCNS11643	2-3223
U+7F5D.0	fBig5	D35F
U+7F5D.0	fGB	3-5A7E
U+7F5D.0	fR/S	 122.5
U+7F5D.0	fTotalStrokes	 10
U+7F5D.0	fCangjie	WLBM
U+7F5D.0	fFourCorner	60107
U+7F5D.0	fHakka	  zia1
U+7F5D.0	fCantonese	ze1; zeoi1 (also)
U+7F5D.0	fMandarin	jie1; ju1
U+7F5D.0	fEnglish	a net for catching rabbits or hares
U+7F5E.0	fUTF8	罞
U+7F5E.0	fCNS11643	2-3221
U+7F5E.0	fBig5	D35D
U+7F5E.0	fGB	5-5A46
U+7F5E.0	fR/S	 122.5
U+7F5E.0	fTotalStrokes	 10
U+7F5E.0	fCangjie	WLNIH
U+7F5E.0	fFourCorner	60227
U+7F5E.0	fHakka	 mung2; mau2; meu5
U+7F5E.0	fCantonese	mung4
U+7F5E.0	fMandarin	mao2; meng2; mou4
U+7F5E.0	fEnglish	lasso for catching deer
U+7F5F.0	fUTF8	罟
U+7F5F.0	fCNS11643	1-574E
U+7F5F.0	fBig5	AFCB
U+7F5F.0	fGB	0-6E39
U+7F5F.0	fR/S	 122.5
U+7F5F.0	fTotalStrokes	 10
U+7F5F.0	fCangjie	WLJR
U+7F5F.0	fFourCorner	60604
U+7F5F.0	fHakka	 gu3;  ku1; ku3; gu1
U+7F5F.0	fCantonese	gu2; gu1 (also); gwu2 (also); gwu1 (also)
U+7F5F.0	fMandarin	gu3
U+7F5F.0	fEnglish	 net
U+7F60.0	fUTF8	罠
U+7F60.0	fCNS11643	2-3222
U+7F60.0	fBig5	D35E
U+7F60.0	fGB	5-5A45
U+7F60.0	fR/S	 122.5
U+7F60.0	fTotalStrokes	 10
U+7F60.0	fCangjie	WLRVP
U+7F60.0	fFourCorner	60747
U+7F60.0	fHakka	 min2
U+7F60.0	fCantonese	man4
U+7F60.0	fMandarin	min2
U+7F60.0	fEnglish	animal trap
U+7F61.0	fUTF8	罡
U+7F61.0	fCNS11643	2-317E
U+7F61.0	fBig5	D35C
U+7F61.0	fGB	0-6E38
U+7F61.0	fR/S	 122.5
U+7F61.0	fTotalStrokes	 10
U+7F61.0	fCangjie	WLMYM
U+7F61.0	fFourCorner	60101
U+7F61.0	fHakka	 gong1
U+7F61.0	fCantonese	gong1
U+7F61.0	fMandarin	gang1
U+7F61.0	fEnglish	the Taoist name of the Dipper
U+7F62.0	fUTF8	罢
U+7F62.0	fCNS11643	3-333C
U+7F62.0	fGB	0-3055
U+7F62.0	fR/S	 122.5
U+7F62.0	fTotalStrokes	 10
U+7F62.0	fCangjie	WLGI
U+7F62.0	fFourCorner	60730
U+7F62.0	fHakka	 pa5
U+7F62.0	fCantonese	baa6; baa2 (also); pei4
U+7F62.0	fMandarin	ba4; bai4; ba5; bi3; bai3; pa2; pi2; bi1; bei4; pi1
U+7F62.0	fEnglish	simplified form of &#x7F77;
U+7F62.1	fUTF8	罢
U+7F62.1	fCNS11643	3-333C
U+7F62.1	fGB	0-3055
U+7F62.1	fR/S	 122.5
U+7F62.1	fTotalStrokes	 10
U+7F62.1	fCangjie	WLGI
U+7F62.1	fFourCorner	60730
U+7F63.0	fUTF8	罣
U+7F63.0	fCNS11643	2-3836
U+7F63.0	fBig5	D6F1
U+7F63.0	fGB	E-3879
U+7F63.0	fR/S	 122.6
U+7F63.0	fTotalStrokes	 11
U+7F63.0	fCangjie	WLGG
U+7F63.0	fFourCorner	60104
U+7F63.0	fHakka	  gwa5; kua5; ga5
U+7F63.0	fCantonese	gwaa3; waa6
U+7F63.0	fMandarin	gua4; gui4; hua4
U+7F63.0	fEnglish	[1] hindrance; obstruction [2] sieve [3] be concerned; be worried
U+7F64.0	fUTF8	罤
U+7F64.0	fCNS11643	4-3A2B
U+7F64.0	fGB	E-387A
U+7F64.0	fR/S	  122.7
U+7F64.0	fTotalStrokes	 12
U+7F64.0	fCangjie	WLCNH
U+7F64.0	fFourCorner	60220
U+7F64.0	fHakka	  ti2
U+7F64.0	fCantonese	tai4
U+7F64.0	fMandarin	ti2; kun1
U+7F64.0	fEnglish	 trap for catching rabbits
U+7F65.0	fUTF8	罥
U+7F65.0	fCNS11643	2-3F24
U+7F65.0	fBig5	DAFE
U+7F65.0	fGB	3-5B25
U+7F65.0	fR/S	 122.7
U+7F65.0	fTotalStrokes	 12
U+7F65.0	fCangjie	WLRB
U+7F65.0	fFourCorner	60227
U+7F65.0	fHakka	 gien1; kien3; gien3; gian1; gen1
U+7F65.0	fCantonese	gyun3; gyun2
U+7F65.0	fMandarin	juan4; quan3
U+7F65.0	fEnglish	impete
U+7F66.0	fUTF8	罦
U+7F66.0	fCNS11643	2-3F25
U+7F66.0	fBig5	DB40
U+7F66.0	fGB	3-5B27
U+7F66.0	fR/S	 122.7
U+7F66.0	fTotalStrokes	 12
U+7F66.0	fCangjie	WLBND
U+7F66.0	fFourCorner	60407
U+7F66.0	fHakka	  fu2;  fu1
U+7F66.0	fCantonese	fau4; fu1 (also)
U+7F66.0	fMandarin	fu1; fu2; fou2
U+7F66.0	fEnglish	net
U+7F67.0	fUTF8	罧
U+7F67.0	fCNS11643	2-4630
U+7F67.0	fBig5	DF69
U+7F67.0	fGB	3-5B29
U+7F67.0	fR/S	 122.8
U+7F67.0	fTotalStrokes	 13
U+7F67.0	fCangjie	WLDD
U+7F67.0	fFourCorner	60994
U+7F67.0	fHakka	  sem1; sim1; sem5
U+7F67.0	fCantonese	sam1; lam6; sam6
U+7F67.0	fMandarin	lin2; sen1; shen1; shen4; sen3
U+7F67.0	fEnglish	 fish-trap
U+7F68.0	fUTF8	罨
U+7F68.0	fCNS11643	2-4631
U+7F68.0	fBig5	DF6A
U+7F68.0	fGB	0-6E3B
U+7F68.0	fR/S	 122.8
U+7F68.0	fTotalStrokes	 13
U+7F68.0	fCangjie	WLKLU
U+7F68.0	fFourCorner	60716
U+7F68.0	fHakka	 jam3
U+7F68.0	fCantonese	jim2; ngap1; ap1
U+7F68.0	fMandarin	yan3
U+7F68.0	fEnglish	 medical compress fish net
U+7F69.0	fUTF8	罩
U+7F69.0	fCNS11643	1-6575
U+7F69.0	fBig5	B86E
U+7F69.0	fGB	0-5556
U+7F69.0	fR/S	 122.8
U+7F69.0	fTotalStrokes	 13
U+7F69.0	fCangjie	WLYAJ
U+7F69.0	fFourCorner	60406
U+7F69.0	fHakka	  zau5; k'iem2
U+7F69.0	fCantonese	zaau3
U+7F69.0	fMandarin	zhao4; zhuo2
U+7F69.0	fEnglish	[1] a bamboo basket for catching fish [2] cover; shade [3] coop; cover; wrap [4] mantle; cloak
U+7F6A.0	fUTF8	罪
U+7F6A.0	fCNS11643	1-6576
U+7F6A.0	fBig5	B86F
U+7F6A.0	fGB	0-576F
U+7F6A.0	fR/S	 122.8
U+7F6A.0	fTotalStrokes	 13
U+7F6A.0	fCangjie	WLLMY
U+7F6A.0	fFourCorner	60111
U+7F6A.0	fHakka	  cui5; cui6; cui3
U+7F6A.0	fCantonese	zeoi6
U+7F6A.0	fMandarin	zui4
U+7F6A.0	fEnglish	[1] crime; sin; fault; vice; evil; guilt [2] suffering; hardship; pain
U+7F6A.1	fUTF8	罪
U+7F6A.1	fCNS11643	1-6576
U+7F6A.1	fBig5	B86F
U+7F6A.1	fGB	0-576F
U+7F6A.1	fR/S	 122.8
U+7F6A.1	fTotalStrokes	 13
U+7F6A.1	fCangjie	WLLMY
U+7F6A.1	fFourCorner	60111
U+7F6A.1	fMandarin	zui5
U+7F6B.0	fUTF8	罫
U+7F6B.0	fCNS11643	2-462F
U+7F6B.0	fBig5	DF68
U+7F6B.0	fGB	3-5B28
U+7F6B.0	fR/S	 122.8
U+7F6B.0	fTotalStrokes	 13
U+7F6B.0	fCangjie	WLGGY
U+7F6B.0	fFourCorner	60130
U+7F6B.0	fHakka	 ga5; gwa5
U+7F6B.0	fCantonese	waa6; gwaai2
U+7F6B.0	fMandarin	gua4; hua4; guai3
U+7F6B.0	fEnglish	hinder
U+7F6C.0	fUTF8	罬
U+7F6C.0	fCNS11643	2-4632
U+7F6C.0	fBig5	DF6B
U+7F6C.0	fGB	E-387B
U+7F6C.0	fR/S	 122.8
U+7F6C.0	fTotalStrokes	 13
U+7F6C.0	fCangjie	WLEEE
U+7F6C.0	fFourCorner	60447
U+7F6C.0	fCantonese	dyut3; zyut3
U+7F6C.0	fMandarin	zhuo2
U+7F6D.0	fUTF8	罭
U+7F6D.0	fCNS11643	2-462E
U+7F6D.0	fBig5	DF67
U+7F6D.0	fGB	3-5B2A
U+7F6D.0	fR/S	 122.8
U+7F6D.0	fTotalStrokes	 13
U+7F6D.0	fCangjie	WLIRM
U+7F6D.0	fFourCorner	60153
U+7F6D.0	fHakka	 fet8; wet7; vet7
U+7F6D.0	fCantonese	wik6
U+7F6D.0	fMandarin	yu4
U+7F6D.0	fEnglish	drag-net made of fine mesh
U+7F6E.0	fUTF8	置
U+7F6E.0	fCNS11643	1-6574
U+7F6E.0	fBig5	B86D
U+7F6E.0	fGB	0-5643
U+7F6E.0	fR/S	 122.8
U+7F6E.0	fTotalStrokes	 13
U+7F6E.0	fCangjie	WLJBM
U+7F6E.0	fFourCorner	60716
U+7F6E.0	fHakka	  zi5; zih5; tje5; zhi5
U+7F6E.0	fCantonese	zi3
U+7F6E.0	fMandarin	zhi4
U+7F6E.0	fEnglish	[1] place; put; [2] establish; set [3] procure; purchase
U+7F6E.1	fUTF8	置
U+7F6E.1	fCNS11643	1-6574
U+7F6E.1	fBig5	B86D
U+7F6E.1	fGB	0-5643
U+7F6E.1	fR/S	 122.8
U+7F6E.1	fTotalStrokes	 13
U+7F6E.1	fCangjie	WLJBM
U+7F6E.1	fFourCorner	60716
U+7F6E.1	fMandarin	zhi5
U+7F6F.0	fUTF8	罯
U+7F6F.0	fCNS11643	3-4878
U+7F6F.0	fGB	3-5B2D
U+7F6F.0	fR/S	  122.9
U+7F6F.0	fTotalStrokes	 14
U+7F6F.0	fCangjie	WLYTA
U+7F6F.0	fFourCorner	60600
U+7F6F.0	fCantonese	am2; ngam2 (also)
U+7F6F.0	fMandarin	an3
U+7F70.0	fUTF8	罰
U+7F70.0	fCNS11643	1-6A48
U+7F70.0	fBig5	BB40
U+7F70.0	fGB	1-3723
U+7F70.0	fR/S	 122.9
U+7F70.0	fTotalStrokes	 14
U+7F70.0	fCangjie	WLYRN
U+7F70.0	fFourCorner	60620
U+7F70.0	fHakka	 fat8
U+7F70.0	fCantonese	fat6
U+7F70.0	fMandarin	fa2
U+7F70.0	fEnglish	 fine; punish; penalize
U+7F71.0	fUTF8	罱
U+7F71.0	fCNS11643	3-4879
U+7F71.0	fGB	0-6E3D
U+7F71.0	fR/S	 122.9
U+7F71.0	fTotalStrokes	 14
U+7F71.0	fCangjie	WLJBJ
U+7F71.0	fFourCorner	60220
U+7F71.0	fHakka	  lam3
U+7F71.0	fCantonese	laam5; naam5 (also)
U+7F71.0	fMandarin	nan3; lan3
U+7F71.0	fEnglish	fishing net
U+7F72.0	fUTF8	署
U+7F72.0	fCNS11643	1-6577
U+7F72.0	fBig5	B870
U+7F72.0	fGB	0-4A70
U+7F72.0	fR/S	 122.8; 122.9
U+7F72.0	fTotalStrokes	 13
U+7F72.0	fCangjie	WLJKA
U+7F72.0	fFourCorner	60604
U+7F72.0	fHakka	 shu6; shu3; su5; su2; su3; shu5; shu2
U+7F72.0	fCantonese	cyu5; syu5 (also); syu3; syu6
U+7F72.0	fMandarin	shu3
U+7F72.0	fEnglish	 public office
U+7F72.1	fUTF8	署
U+7F72.1	fCNS11643	1-6577
U+7F72.1	fBig5	B870
U+7F72.1	fGB	0-4A70
U+7F72.1	fR/S	 122.8; 122.9
U+7F72.1	fTotalStrokes	 13
U+7F72.1	fCangjie	WLJKA
U+7F72.1	fFourCorner	60604
U+7F72.1	fMandarin	shu4
U+7F72.1	fEnglish	[1] arrange [2] write down; put down [3] be a deputy
U+7F73.0	fUTF8	罳
U+7F73.0	fCNS11643	2-4D23
U+7F73.0	fBig5	E37A
U+7F73.0	fGB	3-5B2C
U+7F73.0	fR/S	 122.9
U+7F73.0	fTotalStrokes	 14
U+7F73.0	fCangjie	WLWP
U+7F73.0	fFourCorner	60336
U+7F73.0	fHakka	 sii1; se1
U+7F73.0	fCantonese	si1
U+7F73.0	fMandarin	si1; sai1
U+7F73.0	fEnglish	a screen with meshes or holes
U+7F74.0	fUTF8	罴
U+7F74.0	fGB	0-6E3C
U+7F74.0	fR/S	 122.9
U+7F74.0	fTotalStrokes	 14
U+7F74.0	fCangjie	WLGIF
U+7F74.0	fHakka	 pi2; pi1
U+7F74.0	fCantonese	bei1
U+7F74.0	fMandarin	pi2
U+7F74.0	fEnglish	simplified form of &#x7F86;
U+7F75.0	fUTF8	罵
U+7F75.0	fCNS11643	1-6E47
U+7F75.0	fBig5	BD7C
U+7F75.0	fGB	E-387C
U+7F75.0	fR/S	 122.10
U+7F75.0	fTotalStrokes	 15
U+7F75.0	fCangjie	WLSQF
U+7F75.0	fFourCorner	60327
U+7F75.0	fHakka	  ma5
U+7F75.0	fCantonese	maa6; maa3 (also)
U+7F75.0	fMandarin	ma4; ma3
U+7F75.0	fEnglish	 call names; swear; curse; scold; revile
U+7F76.0	fUTF8	罶
U+7F76.0	fCNS11643	2-5279
U+7F76.0	fBig5	E6F1
U+7F76.0	fGB	3-5B2E
U+7F76.0	fR/S	 122.10
U+7F76.0	fTotalStrokes	 15
U+7F76.0	fCangjie	WLHHW
U+7F76.0	fFourCorner	60602
U+7F76.0	fHakka	  liu3
U+7F76.0	fCantonese	lau5
U+7F76.0	fMandarin	liu3
U+7F76.0	fEnglish	fishing trap
U+7F77.0	fUTF8	罷
U+7F77.0	fCNS11643	1-6E48
U+7F77.0	fBig5	BD7D
U+7F77.0	fGB	1-3055
U+7F77.0	fR/S	 122.10
U+7F77.0	fTotalStrokes	 15
U+7F77.0	fCangjie	WLIBP
U+7F77.0	fFourCorner	60211
U+7F77.0	fHakka	  pa5; pa3; ba5; ba3
U+7F77.0	fCantonese	baa6; pei4; baa2 (also)
U+7F77.0	fMandarin	ba4; bi3; bai4; bai3; pa2; pi4; bi1; bei4; pi1
U+7F77.0	fEnglish	 cease; finish; stop; be done with
U+7F77.1	fUTF8	罷
U+7F77.1	fCNS11643	1-6E48
U+7F77.1	fBig5	BD7D
U+7F77.1	fGB	1-3055
U+7F77.1	fR/S	 122.10
U+7F77.1	fTotalStrokes	 15
U+7F77.1	fCangjie	WLIBP
U+7F77.1	fFourCorner	60211
U+7F77.1	fMandarin	ba5
U+7F77.1	fEnglish	same as $ (a sentence final particle)
U+7F78.0	fUTF8	罸
U+7F78.0	fCNS11643	3-4E30
U+7F78.0	fGB	E-387D
U+7F78.0	fR/S	 122.10; 109.10
U+7F78.0	fTotalStrokes	 15
U+7F78.0	fCangjie	WLYRI
U+7F78.0	fFourCorner	60640
U+7F78.0	fHakka	  fat8
U+7F78.0	fCantonese	fat6
U+7F78.0	fMandarin	fa2
U+7F78.0	fEnglish	 penalty; fine punish; penalize
U+7F79.0	fUTF8	罹
U+7F79.0	fCNS11643	1-7171
U+7F79.0	fBig5	BFA9
U+7F79.0	fGB	0-6E3E
U+7F79.0	fR/S	 122.11
U+7F79.0	fTotalStrokes	 16
U+7F79.0	fCangjie	WLPOG
U+7F79.0	fFourCorner	60914
U+7F79.0	fHakka	 li2
U+7F79.0	fCantonese	lei4
U+7F79.0	fMandarin	li2
U+7F79.0	fEnglish	[1] sorrow; grief [2] meet (disaster; misfortune, etc.); be stricken by
U+7F7A.0	fUTF8	罺
U+7F7A.0	fCNS11643	2-594E
U+7F7A.0	fBig5	EAE2
U+7F7A.0	fGB	3-5B34
U+7F7A.0	fR/S	 122.11
U+7F7A.0	fTotalStrokes	 16
U+7F7A.0	fCangjie	WLVVD
U+7F7A.0	fFourCorner	60904
U+7F7A.0	fCantonese	caau3; caau1; cau1
U+7F7A.0	fMandarin	chao1; zhao1; chao4
U+7F7B.0	fUTF8	罻
U+7F7B.0	fCNS11643	2-594C
U+7F7B.0	fBig5	EAE0
U+7F7B.0	fGB	3-5B32
U+7F7B.0	fR/S	 122.11
U+7F7B.0	fTotalStrokes	 16
U+7F7B.0	fCangjie	WLSFI
U+7F7B.0	fFourCorner	60240
U+7F7B.0	fHakka	 wui3; wi3; vui5
U+7F7B.0	fCantonese	wai3; wat1 (also)
U+7F7B.0	fMandarin	wei4; yu4
U+7F7B.0	fEnglish	[1] a kind of woolen fabric [2] fishing net
U+7F7C.0	fUTF8	罼
U+7F7C.0	fCNS11643	2-594D
U+7F7C.0	fBig5	EAE1
U+7F7C.0	fGB	5-5A47
U+7F7C.0	fR/S	 122.11
U+7F7C.0	fTotalStrokes	 16
U+7F7C.0	fCangjie	WLWTJ
U+7F7C.0	fFourCorner	60504
U+7F7C.0	fCantonese	bat1
U+7F7C.0	fMandarin	bi4
U+7F7D.0	fUTF8	罽
U+7F7D.0	fCNS11643	2-5E51
U+7F7D.0	fBig5	EDE4
U+7F7D.0	fGB	3-5B35
U+7F7D.0	fR/S	 122.12
U+7F7D.0	fTotalStrokes	 17
U+7F7D.0	fCangjie	WLMFN
U+7F7D.0	fFourCorner	60221
U+7F7D.0	fHakka	 gi2; gi5
U+7F7D.0	fCantonese	gai3
U+7F7D.0	fMandarin	ji4
U+7F7D.0	fEnglish	a kind of woolen fabric
U+7F7E.0	fUTF8	罾
U+7F7E.0	fCNS11643	2-5E50
U+7F7E.0	fBig5	EDE3
U+7F7E.0	fGB	0-6E40
U+7F7E.0	fR/S	 122.12
U+7F7E.0	fTotalStrokes	 17
U+7F7E.0	fCangjie	WLCWA
U+7F7E.0	fFourCorner	60606
U+7F7E.0	fHakka	 zen1
U+7F7E.0	fCantonese	zang1; zang6
U+7F7E.0	fMandarin	zeng1; zeng4
U+7F7E.0	fEnglish	a large square net; lowered and raised from the bank of the river
U+7F7F.0	fUTF8	罿
U+7F7F.0	fCNS11643	2-5E4F
U+7F7F.0	fBig5	EDE2
U+7F7F.0	fGB	3-5B36
U+7F7F.0	fR/S	 122.12
U+7F7F.0	fTotalStrokes	 17
U+7F7F.0	fCangjie	WLYTG
U+7F7F.0	fFourCorner	60104
U+7F7F.0	fHakka	 chung1; tung2; cung1
U+7F7F.0	fCantonese	cung1; tung4 (also)
U+7F7F.0	fMandarin	tong2; chong1; zhong1
U+7F7F.0	fEnglish	net catch birds
U+7F80.0	fUTF8	羀
U+7F80.0	fCNS11643	7-4161
U+7F80.0	fGB	E-387E
U+7F80.0	fR/S	 122.12
U+7F80.0	fTotalStrokes	 18
U+7F80.0	fCangjie	BKMLW
U+7F80.0	fFourCorner	77600
U+7F80.0	fCantonese	lau5
U+7F80.0	fMandarin	liu3
U+7F80.0	fEnglish	fishing trap
U+7F81.0	fUTF8	羁
U+7F81.0	fGB	0-6E3F
U+7F81.0	fR/S	 122.12
U+7F81.0	fTotalStrokes	 17
U+7F81.0	fCangjie	WLTJM
U+7F81.0	fHakka	 gi1
U+7F81.0	fCantonese	gei1
U+7F81.0	fMandarin	ji1
U+7F81.0	fEnglish	simplified form of &#x7F88;
U+7F82.0	fUTF8	羂
U+7F82.0	fCNS11643	4-5C68
U+7F82.0	fGB	3-5B30
U+7F82.0	fR/S	  122.13
U+7F82.0	fTotalStrokes	 18
U+7F82.0	fCangjie	WLVFB
U+7F82.0	fFourCorner	60920
U+7F82.0	fHakka	 gien1; gian1; gen1
U+7F82.0	fCantonese	gyun3
U+7F82.0	fMandarin	juan4; quan3
U+7F82.0	fEnglish	 hang up; entangle; bind
U+7F83.0	fUTF8	羃
U+7F83.0	fCNS11643	2-6645
U+7F83.0	fBig5	F2BB
U+7F83.0	fGB	5-5A56
U+7F83.0	fR/S	 122.14
U+7F83.0	fTotalStrokes	 19
U+7F83.0	fCangjie	WLTAB
U+7F83.0	fFourCorner	60227
U+7F83.0	fCantonese	mik6
U+7F83.0	fMandarin	mi4
U+7F83.0	fEnglish	cover-cloth; cover with cloth
U+7F84.0	fUTF8	羄
U+7F84.0	fCNS11643	4-606F
U+7F84.0	fGB	3-5B33
U+7F84.0	fR/S	  122.14
U+7F84.0	fTotalStrokes	 19
U+7F84.0	fCangjie	WLVFJ
U+7F84.0	fFourCorner	60940
U+7F84.0	fCantonese	zaau3
U+7F84.0	fMandarin	zhao4
U+7F85.0	fUTF8	羅
U+7F85.0	fCNS11643	1-7865
U+7F85.0	fBig5	C3B9
U+7F85.0	fGB	1-425E
U+7F85.0	fR/S	 122.14
U+7F85.0	fTotalStrokes	 19
U+7F85.0	fCangjie	WLVFG
U+7F85.0	fFourCorner	60914
U+7F85.0	fHakka	  lo2; lo3
U+7F85.0	fCantonese	lo4; lo2 (also); lo1
U+7F85.0	fMandarin	luo2; li2; luo1; luo4; luo5
U+7F85.0	fEnglish	[1] thin light silk [2] net for catching birds; net; snare [3] arrange over a wide space [4] a Chinese family name
U+7F86.0	fUTF8	羆
U+7F86.0	fCNS11643	2-6646
U+7F86.0	fBig5	F2BC
U+7F86.0	fGB	1-6E3C
U+7F86.0	fR/S	 122.14
U+7F86.0	fTotalStrokes	 19
U+7F86.0	fCangjie	WLIPF
U+7F86.0	fFourCorner	60331
U+7F86.0	fHakka	  pi2;  pi1
U+7F86.0	fCantonese	bei1
U+7F86.0	fMandarin	pi2
U+7F86.0	fEnglish	the brown bear (Ursus arctos)
U+7F87.0	fUTF8	羇
U+7F87.0	fCNS11643	2-6D71
U+7F87.0	fBig5	F744
U+7F87.0	fGB	3-5B38
U+7F87.0	fR/S	 122.17
U+7F87.0	fTotalStrokes	 22
U+7F87.0	fCangjie	WLTJR
U+7F87.0	fFourCorner	60521
U+7F87.0	fHakka	  gi1
U+7F87.0	fCantonese	gei1
U+7F87.0	fMandarin	ji1
U+7F87.0	fEnglish	 lodge at somebody's house
U+7F88.0	fUTF8	羈
U+7F88.0	fCNS11643	1-7C65
U+7F88.0	fBig5	C5F9
U+7F88.0	fGB	1-6E3F
U+7F88.0	fR/S	 122.19
U+7F88.0	fTotalStrokes	 24
U+7F88.0	fCangjie	WLTJF
U+7F88.0	fFourCorner	60527
U+7F88.0	fHakka	 gi1
U+7F88.0	fCantonese	gei1
U+7F88.0	fMandarin	ji1
U+7F88.0	fEnglish	[1] halter; bridle [2] restriction; restraint; confine; restrain; hold; bind [3] live in another house; stay in a place other than one's hometown
U+7F89.0	fUTF8	羉
U+7F89.0	fCNS11643	2-7048
U+7F89.0	fBig5	F8BA
U+7F89.0	fGB	E-3921
U+7F89.0	fR/S	 122.19
U+7F89.0	fTotalStrokes	 24
U+7F89.0	fCangjie	WLVFF
U+7F89.0	fFourCorner	60993
U+7F89.0	fHakka	  lan2
U+7F89.0	fCantonese	lyun4; laan4
U+7F89.0	fMandarin	luan2; man2
U+7F89.0	fEnglish	a net for catching small animals
U+7F8A.0	fUTF8	羊
U+7F8A.0	fCNS11643	1-4850
U+7F8A.0	fBig5	A6CF
U+7F8A.0	fGB	0-5172
U+7F8A.0	fR/S	 123.0
U+7F8A.0	fTotalStrokes	 6
U+7F8A.0	fCangjie	TQ
U+7F8A.0	fFourCorner	80501
U+7F8A.0	fHakka	  jong2; rong2; jong3
U+7F8A.0	fCantonese	joeng4; coeng4; joeng2 (also)
U+7F8A.0	fMandarin	yang2; xiang2
U+7F8A.0	fEnglish	[1] sheep; goat [2] Kangxi radical 123
U+7F8B.0	fUTF8	羋
U+7F8B.0	fCNS11643	1-4F2E
U+7F8B.0	fBig5	AACB
U+7F8B.0	fGB	5-312D
U+7F8B.0	fR/S	 123.1
U+7F8B.0	fTotalStrokes	 8; 7
U+7F8B.0	fCangjie	XXTQ
U+7F8B.0	fFourCorner	11500
U+7F8B.0	fCantonese	me1; mai5 (also); me5; mei5
U+7F8B.0	fMandarin	mi3; mie1
U+7F8B.0	fEnglish	the bleating of a sheep or goat
U+7F8C.0	fUTF8	羌
U+7F8C.0	fCNS11643	1-4F2D
U+7F8C.0	fBig5	AACA
U+7F8C.0	fGB	0-473C
U+7F8C.0	fR/S	 123.2
U+7F8C.0	fTotalStrokes	 8
U+7F8C.0	fCangjie	TGHU
U+7F8C.0	fFourCorner	80211
U+7F8C.0	fHakka	  giong1
U+7F8C.0	fCantonese	goeng1
U+7F8C.0	fMandarin	qiang1; xiang4
U+7F8C.0	fEnglish	an ancient tribe in West China; Qiang nationality
U+7F8D.0	fUTF8	羍
U+7F8D.0	fCNS11643	2-2C70
U+7F8D.0	fBig5	D04F
U+7F8D.0	fGB	5-373C
U+7F8D.0	fR/S	 123.3
U+7F8D.0	fTotalStrokes	 9
U+7F8D.0	fCangjie	KTQ
U+7F8D.0	fFourCorner	40501
U+7F8D.0	fHakka	  dat7
U+7F8D.0	fCantonese	taat3; daat1
U+7F8D.0	fMandarin	ta4; da2
U+7F8D.0	fEnglish	 lamb
U+7F8E.0	fUTF8	美
U+7F8E.0	fCNS11643	1-5321
U+7F8E.0	fBig5	ACFC
U+7F8E.0	fGB	0-4340
U+7F8E.0	fR/S	 123.3
U+7F8E.0	fTotalStrokes	 9
U+7F8E.0	fCangjie	TGK
U+7F8E.0	fFourCorner	80431
U+7F8E.0	fHakka	  mui1;  mi1; mi3; mui3
U+7F8E.0	fCantonese	mei5
U+7F8E.0	fMandarin	mei3
U+7F8E.0	fEnglish	[1] beautiful; pretty; pleasing; fine; fair [2] good; excellent; exquisite; nice [3] be pleased with oneself [4] praise
U+7F8F.0	fUTF8	羏
U+7F8F.0	fCNS11643	3-2E5D
U+7F8F.0	fGB	5-6639
U+7F8F.0	fR/S	  123.3
U+7F8F.0	fTotalStrokes	 9
U+7F8F.0	fCangjie	TQHHH
U+7F8F.0	fFourCorner	82520
U+7F8F.0	fCantonese	joeng4
U+7F8F.0	fMandarin	yang2
U+7F90.0	fUTF8	羐
U+7F90.0	fCNS11643	4-2F30
U+7F90.0	fGB	E-3922
U+7F90.0	fR/S	 123.3
U+7F90.0	fTotalStrokes	 10; 9
U+7F90.0	fCangjie	TGNO
U+7F90.0	fFourCorner	44800
U+7F90.0	fHakka	 riu3; ju3; ju5
U+7F90.0	fCantonese	jau5
U+7F90.0	fMandarin	you3
U+7F90.0	fEnglish	 lead; guide
U+7F91.0	fUTF8	羑
U+7F91.0	fCNS11643	2-2C6F
U+7F91.0	fBig5	D04E
U+7F91.0	fGB	3-6768
U+7F91.0	fR/S	 123.3
U+7F91.0	fTotalStrokes	 9
U+7F91.0	fCangjie	TGNO
U+7F91.0	fFourCorner	80807
U+7F91.0	fHakka	 ju3; riu3; ju5
U+7F91.0	fCantonese	jau5
U+7F91.0	fMandarin	you3
U+7F91.0	fEnglish	 guide to goodness
U+7F92.0	fUTF8	羒
U+7F92.0	fCNS11643	2-3226
U+7F92.0	fBig5	D362
U+7F92.0	fGB	3-6753
U+7F92.0	fR/S	 123.4
U+7F92.0	fTotalStrokes	 10
U+7F92.0	fCangjie	TQCSH
U+7F92.0	fFourCorner	88527
U+7F92.0	fHakka	  fun1; fun2
U+7F92.0	fCantonese	fan4; fan1
U+7F92.0	fMandarin	fen2
U+7F92.0	fEnglish	 white ram
U+7F93.0	fUTF8	羓
U+7F93.0	fCNS11643	3-333D
U+7F93.0	fGB	3-6756
U+7F93.0	fR/S	  123.4
U+7F93.0	fTotalStrokes	 10
U+7F93.0	fCangjie	TQAU
U+7F93.0	fFourCorner	87510
U+7F93.0	fHakka	 ba1
U+7F93.0	fCantonese	baa1
U+7F93.0	fMandarin	ba1
U+7F93.0	fEnglish	dried salted meat
U+7F94.0	fUTF8	羔
U+7F94.0	fCNS11643	1-574F
U+7F94.0	fBig5	AFCC
U+7F94.0	fGB	0-3861
U+7F94.0	fR/S	 123.4
U+7F94.0	fTotalStrokes	 10
U+7F94.0	fCangjie	TGF
U+7F94.0	fFourCorner	80331
U+7F94.0	fHakka	  gau1;  go1
U+7F94.0	fCantonese	gou1
U+7F94.0	fMandarin	gao1
U+7F94.0	fEnglish	 lamb
U+7F95.0	fUTF8	羕
U+7F95.0	fCNS11643	2-3837
U+7F95.0	fBig5	D6F2
U+7F95.0	fGB	3-6769
U+7F95.0	fR/S	 123.5
U+7F95.0	fTotalStrokes	 11
U+7F95.0	fCangjie	TGINE
U+7F95.0	fFourCorner	80232
U+7F95.0	fHakka	 jong6; jong5
U+7F95.0	fCantonese	joeng6
U+7F95.0	fMandarin	yang4
U+7F96.0	fUTF8	羖
U+7F96.0	fCNS11643	2-3225
U+7F96.0	fBig5	D361
U+7F96.0	fGB	3-6754
U+7F96.0	fR/S	 123.4
U+7F96.0	fTotalStrokes	 10
U+7F96.0	fCangjie	TQHNE
U+7F96.0	fFourCorner	87547
U+7F96.0	fHakka	  gu3
U+7F96.0	fCantonese	gu2; gwu2 (also)
U+7F96.0	fMandarin	gu3
U+7F96.0	fEnglish	 black ram
U+7F97.0	fUTF8	羗
U+7F97.0	fCNS11643	3-333F
U+7F97.0	fGB	E-3923
U+7F97.0	fR/S	 123.4
U+7F97.0	fTotalStrokes	 10
U+7F97.0	fCangjie	TUI
U+7F97.0	fFourCorner	80210
U+7F97.0	fHakka	 giong1; riu3; jiu3; ju3
U+7F97.0	fCantonese	goeng1
U+7F97.0	fMandarin	qiang1
U+7F97.0	fEnglish	Tribes in West China strong. educated. obstinate. a particle
U+7F98.0	fUTF8	羘
U+7F98.0	fCNS11643	4-2F2F
U+7F98.0	fGB	1-7D72
U+7F98.0	fR/S	  123.4
U+7F98.0	fTotalStrokes	 10
U+7F98.0	fCangjie	BTQ
U+7F98.0	fFourCorner	78250
U+7F98.0	fCantonese	zong1; gou1; ling4; ji6; hei3
U+7F98.0	fMandarin	zang1
U+7F98.0	fEnglish	ram
U+7F99.0	fUTF8	羙
U+7F99.0	fCNS11643	3-333E
U+7F99.0	fGB	E-3924
U+7F99.0	fR/S	  123.4
U+7F99.0	fTotalStrokes	 10
U+7F99.0	fCangjie	TGF
U+7F99.0	fFourCorner	80800
U+7F99.0	fCantonese	mei5
U+7F99.0	fMandarin	gao1
U+7F9A.0	fUTF8	羚
U+7F9A.0	fCNS11643	1-5C61
U+7F9A.0	fBig5	B2DC
U+7F9A.0	fGB	0-4167
U+7F9A.0	fR/S	 123.5
U+7F9A.0	fTotalStrokes	 11
U+7F9A.0	fCangjie	TQOII
U+7F9A.0	fFourCorner	88537
U+7F9A.0	fHakka	 lin2
U+7F9A.0	fCantonese	ling4
U+7F9A.0	fMandarin	ling2
U+7F9A.0	fEnglish	an antelope
U+7F9B.0	fUTF8	羛
U+7F9B.0	fCNS11643	2-383A
U+7F9B.0	fBig5	D6F5
U+7F9B.0	fGB	3-676A
U+7F9B.0	fR/S	 123.5
U+7F9B.0	fTotalStrokes	 11
U+7F9B.0	fCangjie	TGLLN
U+7F9B.0	fFourCorner	80527
U+7F9B.0	fCantonese	hei3; ji6
U+7F9B.0	fMandarin	yi4; yi3; xi4; xi1
U+7F9C.0	fUTF8	羜
U+7F9C.0	fCNS11643	2-3838
U+7F9C.0	fBig5	D6F3
U+7F9C.0	fGB	3-6755
U+7F9C.0	fR/S	 123.5
U+7F9C.0	fTotalStrokes	 11
U+7F9C.0	fCangjie	TQJMN
U+7F9C.0	fFourCorner	83521
U+7F9C.0	fHakka	  du3; chu3; cu3
U+7F9C.0	fCantonese	cyu5
U+7F9C.0	fMandarin	zhu4; shu3
U+7F9C.0	fEnglish	lamb
U+7F9D.0	fUTF8	羝
U+7F9D.0	fCNS11643	2-3839
U+7F9D.0	fBig5	D6F4
U+7F9D.0	fGB	0-7446
U+7F9D.0	fR/S	 123.5
U+7F9D.0	fTotalStrokes	 11
U+7F9D.0	fCangjie	TQHPM
U+7F9D.0	fFourCorner	82540
U+7F9D.0	fHakka	  di1;  dai1
U+7F9D.0	fCantonese	dai1
U+7F9D.0	fMandarin	di1; di3
U+7F9D.0	fEnglish	 ram; he-goat
U+7F9E.0	fUTF8	羞
U+7F9E.0	fCNS11643	1-5C60
U+7F9E.0	fBig5	B2DB
U+7F9E.0	fGB	0-505F
U+7F9E.0	fR/S	 123.5
U+7F9E.0	fTotalStrokes	 11
U+7F9E.0	fCangjie	TQNG
U+7F9E.0	fFourCorner	80215
U+7F9E.0	fHakka	  siu1; sieu1
U+7F9E.0	fCantonese	sau1
U+7F9E.0	fMandarin	xiu1
U+7F9E.0	fEnglish	[1] abashed; ashamed [2] bashful; shy [3] disgrace; insult; shame
U+7F9F.0	fUTF8	羟
U+7F9F.0	fGB	0-7447
U+7F9F.0	fR/S	 123.5
U+7F9F.0	fTotalStrokes	 11
U+7F9F.0	fCangjie	TQNOM
U+7F9F.0	fCantonese	koeng5
U+7F9F.0	fMandarin	qian1; keng1; qiang3
U+7F9F.0	fEnglish	simplified form of &#x7FA5;
U+7FA0.0	fUTF8	羠
U+7FA0.0	fCNS11643	2-3F27
U+7FA0.0	fBig5	DB42
U+7FA0.0	fGB	3-6759
U+7FA0.0	fR/S	 123.6
U+7FA0.0	fTotalStrokes	 12
U+7FA0.0	fCangjie	TQKN
U+7FA0.0	fFourCorner	85532
U+7FA0.0	fCantonese	ji4
U+7FA0.0	fMandarin	yi2; si4
U+7FA1.0	fUTF8	羡
U+7FA1.0	fCNS11643	2-3F28
U+7FA1.0	fBig5	DB43
U+7FA1.0	fGB	0-4F5B
U+7FA1.0	fR/S	 123.6
U+7FA1.0	fTotalStrokes	 12
U+7FA1.0	fCangjie	TGIMO
U+7FA1.0	fFourCorner	80182
U+7FA1.0	fHakka	 jen5
U+7FA1.0	fCantonese	sin6; jin4; sin4; ji4
U+7FA1.0	fMandarin	yi2; xian4; yan2
U+7FA1.0	fEnglish	 envy; admire praise covet
U+7FA2.0	fUTF8	羢
U+7FA2.0	fCNS11643	2-3F26
U+7FA2.0	fBig5	DB41
U+7FA2.0	fGB	E-3925
U+7FA2.0	fR/S	 123.6
U+7FA2.0	fTotalStrokes	 12
U+7FA2.0	fCangjie	TQIJ
U+7FA2.0	fFourCorner	83550
U+7FA2.0	fCantonese	jung4 (also); jung2
U+7FA2.0	fMandarin	rong2
U+7FA2.0	fEnglish	fine wool
U+7FA3.0	fUTF8	羣
U+7FA3.0	fCNS11643	3-436E
U+7FA3.0	fGB	E-3926
U+7FA3.0	fR/S	 123.7
U+7FA3.0	fTotalStrokes	 13
U+7FA3.0	fCangjie	SRTQ
U+7FA3.0	fFourCorner	17500
U+7FA3.0	fHakka	 kiun2; kiun3
U+7FA3.0	fCantonese	kwan4
U+7FA3.0	fMandarin	qun2
U+7FA3.0	fEnglish	group; crowd; multitude; mob
U+7FA4.0	fUTF8	群
U+7FA4.0	fCNS11643	1-657A
U+7FA4.0	fBig5	B873
U+7FA4.0	fGB	0-483A
U+7FA4.0	fR/S	 123.7
U+7FA4.0	fTotalStrokes	 13
U+7FA4.0	fCangjie	SRTQ
U+7FA4.0	fFourCorner	18651
U+7FA4.0	fHakka	  kiun2
U+7FA4.0	fCantonese	kwan4
U+7FA4.0	fMandarin	qun2
U+7FA4.0	fEnglish	 group; crowd; multitude; mob; swarm; large number; flock; herd; host
U+7FA5.0	fUTF8	羥
U+7FA5.0	fCNS11643	2-4634
U+7FA5.0	fBig5	DF6D
U+7FA5.0	fGB	1-7447
U+7FA5.0	fR/S	 123.7
U+7FA5.0	fTotalStrokes	 13
U+7FA5.0	fCangjie	TQMVM
U+7FA5.0	fFourCorner	81511
U+7FA5.0	fHakka	 ken1
U+7FA5.0	fCantonese	koeng5; coeng1 (also)
U+7FA5.0	fMandarin	qian1; qiang3; keng1
U+7FA5.0	fEnglish	hydroxide
U+7FA6.0	fUTF8	羦
U+7FA6.0	fCNS11643	2-4633
U+7FA6.0	fBig5	DF6C
U+7FA6.0	fGB	3-675F
U+7FA6.0	fR/S	 123.7
U+7FA6.0	fTotalStrokes	 13
U+7FA6.0	fCangjie	TQJMU
U+7FA6.0	fFourCorner	83511
U+7FA6.0	fCantonese	wun4
U+7FA6.0	fMandarin	huan2
U+7FA7.0	fUTF8	羧
U+7FA7.0	fCNS11643	2-4635
U+7FA7.0	fBig5	DF6E
U+7FA7.0	fGB	0-7448
U+7FA7.0	fR/S	 123.7
U+7FA7.0	fTotalStrokes	 13
U+7FA7.0	fCangjie	TQICE
U+7FA7.0	fFourCorner	83547
U+7FA7.0	fCantonese	so1
U+7FA7.0	fMandarin	suo1; zui1; cui1; sui1
U+7FA7.0	fEnglish	carboxyl group
U+7FA8.0	fUTF8	羨
U+7FA8.0	fCNS11643	1-6579
U+7FA8.0	fBig5	B872
U+7FA8.0	fGB	E-3927
U+7FA8.0	fR/S	 123.7
U+7FA8.0	fTotalStrokes	 13
U+7FA8.0	fCangjie	TGENO
U+7FA8.0	fFourCorner	80182
U+7FA8.0	fHakka	  sien3; sien5; sen3; ren6; jen5
U+7FA8.0	fCantonese	sin6; jin4
U+7FA8.0	fMandarin	xian4; yi2; yan2; yan4; yan3
U+7FA8.0	fEnglish	 envy; covet
U+7FA9.0	fUTF8	義
U+7FA9.0	fCNS11643	1-6578
U+7FA9.0	fBig5	B871
U+7FA9.0	fGB	1-5265
U+7FA9.0	fR/S	 123.7
U+7FA9.0	fTotalStrokes	 13
U+7FA9.0	fCangjie	TGHQI
U+7FA9.0	fFourCorner	80553
U+7FA9.0	fHakka	 ngi5; gni5; ngi6; ngi3; ji3; gni6
U+7FA9.0	fCantonese	ji6
U+7FA9.0	fMandarin	yi4; yi2
U+7FA9.0	fEnglish	[1] generosity; charity; philanthropy; chivalry [2] meaning; connotation [3] unreal; artificial; fake [4] a Chinese family name
U+7FAA.0	fUTF8	羪
U+7FAA.0	fGB	E-3928
U+7FAA.0	fR/S	 123.7
U+7FAA.0	fTotalStrokes	 13
U+7FAA.0	fCangjie	TQIAV
U+7FAA.0	fCantonese	loeng4
U+7FAA.0	fMandarin	yang3
U+7FAB.0	fUTF8	羫
U+7FAB.0	fCNS11643	4-4626
U+7FAB.0	fGB	5-664D
U+7FAB.0	fR/S	  123.8
U+7FAB.0	fTotalStrokes	 14
U+7FAB.0	fCangjie	TQJCM
U+7FAB.0	fFourCorner	83510
U+7FAB.0	fCantonese	hong1
U+7FAB.0	fMandarin	qiang1; kong4
U+7FAB.0	fEnglish	the skeleton of a sheep
U+7FAC.0	fUTF8	羬
U+7FAC.0	fCNS11643	2-527A
U+7FAC.0	fBig5	E6F2
U+7FAC.0	fGB	3-6761
U+7FAC.0	fR/S	 123.9
U+7FAC.0	fTotalStrokes	 15
U+7FAC.0	fCangjie	TQIHR
U+7FAC.0	fFourCorner	83550
U+7FAC.0	fHakka	 gam2; kiam2; ham2
U+7FAC.0	fCantonese	kim4; haam4; ngaam4
U+7FAC.0	fMandarin	xian2; qian2; yan2
U+7FAC.0	fEnglish	the nylghau; an antelope (Portax pictus)
U+7FAD.0	fUTF8	羭
U+7FAD.0	fCNS11643	2-527C
U+7FAD.0	fBig5	E6F4
U+7FAD.0	fGB	3-6762
U+7FAD.0	fR/S	 123.9
U+7FAD.0	fTotalStrokes	 15
U+7FAD.0	fCangjie	TQOMN
U+7FAD.0	fFourCorner	88521
U+7FAD.0	fHakka	 ji2; ri2; i2
U+7FAD.0	fCantonese	jyu4
U+7FAD.0	fMandarin	yu2; yu4
U+7FAD.0	fEnglish	good
U+7FAE.0	fUTF8	羮
U+7FAE.0	fCNS11643	3-4E31
U+7FAE.0	fGB	E-3929
U+7FAE.0	fR/S	 123.9
U+7FAE.0	fTotalStrokes	 15
U+7FAE.0	fCangjie	TGFMO
U+7FAE.0	fFourCorner	80530
U+7FAE.0	fHakka	  gang1; goi2
U+7FAE.0	fCantonese	gang1
U+7FAE.0	fMandarin	geng1; lang2
U+7FAE.0	fEnglish	soup; broth
U+7FAF.0	fUTF8	羯
U+7FAF.0	fCNS11643	1-6E49
U+7FAF.0	fBig5	BD7E
U+7FAF.0	fGB	0-7449
U+7FAF.0	fR/S	 123.9
U+7FAF.0	fTotalStrokes	 15
U+7FAF.0	fCangjie	TQAPV
U+7FAF.0	fFourCorner	86527
U+7FAF.0	fHakka	  giet7; ket7; get7
U+7FAF.0	fCantonese	kit3; git3
U+7FAF.0	fMandarin	jie2
U+7FAF.0	fEnglish	[1] castrated ram [2] an ancient barbarian people
U+7FB0.0	fUTF8	羰
U+7FB0.0	fCNS11643	2-527B
U+7FB0.0	fBig5	E6F3
U+7FB0.0	fGB	0-744A
U+7FB0.0	fR/S	 123.9
U+7FB0.0	fTotalStrokes	 15
U+7FB0.0	fCangjie	TQUMF
U+7FB0.0	fFourCorner	82589
U+7FB0.0	fCantonese	tong1
U+7FB0.0	fMandarin	tang1; tan4
U+7FB1.0	fUTF8	羱
U+7FB1.0	fCNS11643	2-594F
U+7FB1.0	fBig5	EAE3
U+7FB1.0	fGB	3-6764
U+7FB1.0	fR/S	 123.10
U+7FB1.0	fTotalStrokes	 16
U+7FB1.0	fCangjie	TQMHF
U+7FB1.0	fFourCorner	81596
U+7FB1.0	fHakka	 ngien2
U+7FB1.0	fCantonese	jyun4
U+7FB1.0	fMandarin	yuan2; wan2
U+7FB1.0	fEnglish	sheep
U+7FB2.0	fUTF8	羲
U+7FB2.0	fCNS11643	1-7172
U+7FB2.0	fBig5	BFAA
U+7FB2.0	fGB	0-744B
U+7FB2.0	fR/S	 123.10
U+7FB2.0	fTotalStrokes	 16
U+7FB2.0	fCangjie	TGHDS
U+7FB2.0	fFourCorner	80253
U+7FB2.0	fHakka	 hi1
U+7FB2.0	fCantonese	hei1
U+7FB2.0	fMandarin	xi1
U+7FB2.0	fEnglish	Fu Hsi, a legendary ruler who introduced houses
U+7FB3.0	fUTF8	羳
U+7FB3.0	fCNS11643	2-6265
U+7FB3.0	fBig5	F079
U+7FB3.0	fGB	5-6651
U+7FB3.0	fR/S	 123.12
U+7FB3.0	fTotalStrokes	 18
U+7FB3.0	fCangjie	TQHDW
U+7FB3.0	fFourCorner	82569
U+7FB3.0	fHakka	 fan1
U+7FB3.0	fCantonese	faan4; faan1
U+7FB3.0	fMandarin	fan2
U+7FB3.0	fEnglish	a small deer found on the mountains of Koko-nor
U+7FB4.0	fUTF8	羴
U+7FB4.0	fCNS11643	3-585D
U+7FB4.0	fGB	E-392A
U+7FB4.0	fR/S	  123.12
U+7FB4.0	fTotalStrokes	 18
U+7FB4.0	fCangjie	TQTQQ
U+7FB4.0	fFourCorner	80550
U+7FB4.0	fHakka	 shan1; hien5; shen1; zhan1; san1; sen1; zan1; zhen1
U+7FB4.0	fCantonese	zin1; saan1 (also)
U+7FB4.0	fMandarin	shan1; xian1
U+7FB4.0	fEnglish	rank odour of sheep or goats
U+7FB5.0	fUTF8	羵
U+7FB5.0	fCNS11643	2-6264
U+7FB5.0	fBig5	F078
U+7FB5.0	fGB	5-664E
U+7FB5.0	fR/S	 123.12
U+7FB5.0	fTotalStrokes	 18
U+7FB5.0	fCangjie	TQJTC
U+7FB5.0	fFourCorner	84586
U+7FB5.0	fHakka	 fun2
U+7FB5.0	fCantonese	fan4
U+7FB5.0	fMandarin	fen3; fen2; fen4
U+7FB5.0	fEnglish	spirit
U+7FB6.0	fUTF8	羶
U+7FB6.0	fCNS11643	1-7866
U+7FB6.0	fBig5	C3BB
U+7FB6.0	fGB	E-392B
U+7FB6.0	fR/S	 123.13
U+7FB6.0	fTotalStrokes	 19
U+7FB6.0	fCangjie	TQYWM
U+7FB6.0	fFourCorner	80517
U+7FB6.0	fHakka	 shan1; hien5; shen1; zhan1; san1; sen1; zan1; zhen1
U+7FB6.0	fCantonese	zin1; saan1 (also); sin1
U+7FB6.0	fMandarin	shan1; xing1
U+7FB6.0	fEnglish	the odor of a sheep or goat
U+7FB7.0	fUTF8	羷
U+7FB7.0	fCNS11643	2-6647
U+7FB7.0	fBig5	F2BD
U+7FB7.0	fGB	5-6646
U+7FB7.0	fR/S	 123.13
U+7FB7.0	fTotalStrokes	 19
U+7FB7.0	fCangjie	TQOMO
U+7FB7.0	fFourCorner	88586
U+7FB7.0	fHakka	 giam3; liam5
U+7FB7.0	fCantonese	lim5; gim2
U+7FB7.0	fMandarin	lian3; xian3; lian4
U+7FB7.0	fEnglish	twists in the horns of a ram or a deer
U+7FB8.0	fUTF8	羸
U+7FB8.0	fCNS11643	1-7868
U+7FB8.0	fBig5	C3BD
U+7FB8.0	fGB	0-597A
U+7FB8.0	fR/S	 123.13
U+7FB8.0	fTotalStrokes	 19
U+7FB8.0	fCangjie	YNTQ
U+7FB8.0	fFourCorner	00217
U+7FB8.0	fHakka	 lui2
U+7FB8.0	fCantonese	leoi4
U+7FB8.0	fMandarin	lei2; lian2
U+7FB8.0	fEnglish	[1] lean; emaciated [2] weak; feeble [3] entangle; bind
U+7FB9.0	fUTF8	羹
U+7FB9.0	fCNS11643	1-7867
U+7FB9.0	fBig5	C3BC
U+7FB9.0	fGB	0-387E
U+7FB9.0	fR/S	 123.13
U+7FB9.0	fTotalStrokes	 19
U+7FB9.0	fCangjie	TGFTK
U+7FB9.0	fFourCorner	80530
U+7FB9.0	fHakka	 gang1
U+7FB9.0	fCantonese	gang1; long4
U+7FB9.0	fMandarin	geng1; lang2
U+7FB9.0	fEnglish	thick soup; broth
U+7FBA.0	fUTF8	羺
U+7FBA.0	fCNS11643	2-695D
U+7FBA.0	fBig5	F4B0
U+7FBA.0	fGB	5-6652
U+7FBA.0	fR/S	 123.14
U+7FBA.0	fTotalStrokes	 20
U+7FBA.0	fCangjie	TQMBB
U+7FBA.0	fFourCorner	81527
U+7FBA.0	fCantonese	nau4; lau4 (also); nau6
U+7FBA.0	fMandarin	nou2
U+7FBB.0	fUTF8	羻
U+7FBB.0	fCNS11643	2-6B7C
U+7FBB.0	fBig5	F5EE
U+7FBB.0	fGB	E-392C
U+7FBB.0	fR/S	 123.15
U+7FBB.0	fTotalStrokes	 21
U+7FBB.0	fCangjie	TQIXE
U+7FBB.0	fFourCorner	80547
U+7FBB.0	fCantonese	hoeng3; joeng5
U+7FBB.0	fMandarin	qiang4; yang4
U+7FBC.0	fUTF8	羼
U+7FBC.0	fCNS11643	1-7A7E
U+7FBC.0	fBig5	C4F3
U+7FBC.0	fGB	0-6571
U+7FBC.0	fR/S	 123.15
U+7FBC.0	fTotalStrokes	 21
U+7FBC.0	fCangjie	STQQ
U+7FBC.0	fFourCorner	77251
U+7FBC.0	fHakka	  can3
U+7FBC.0	fCantonese	caan3; caan2; can3
U+7FBC.0	fMandarin	chan4; chan3
U+7FBC.0	fEnglish	 mix; interpolate
U+7FBD.0	fUTF8	羽
U+7FBD.0	fCNS11643	1-4851
U+7FBD.0	fBig5	A6D0
U+7FBD.0	fGB	0-5370
U+7FBD.0	fR/S	 124.0
U+7FBD.0	fTotalStrokes	 6
U+7FBD.0	fCangjie	SMSIM
U+7FBD.0	fFourCorner	17120
U+7FBD.0	fHakka	  ji3; ji1; ri3; i3
U+7FBD.0	fCantonese	jyu5; hung4; ngai6; ai6 (also)
U+7FBD.0	fMandarin	yu3; hu4; yu4
U+7FBD.0	fEnglish	[1] feather; plume [2] wings of a bird [3] one of the five notes in the Chinese musical scale [4] Kangxi radical 124
U+7FBE.0	fUTF8	羾
U+7FBE.0	fCNS11643	2-2C71
U+7FBE.0	fBig5	D050
U+7FBE.0	fGB	3-6B63
U+7FBE.0	fR/S	 124.3
U+7FBE.0	fTotalStrokes	 9
U+7FBE.0	fCangjie	SMM
U+7FBE.0	fFourCorner	11110
U+7FBE.0	fHakka	 fung2; gung5
U+7FBE.0	fCantonese	hung4; gung3
U+7FBE.0	fMandarin	gong4
U+7FBE.0	fEnglish	[1] fly [2] reach
U+7FBF.0	fUTF8	羿
U+7FBF.0	fCNS11643	1-5322
U+7FBF.0	fBig5	ACFD
U+7FBF.0	fGB	0-7460
U+7FBF.0	fR/S	 124.3
U+7FBF.0	fTotalStrokes	 9
U+7FBF.0	fCangjie	SMT
U+7FBF.0	fFourCorner	17442
U+7FBF.0	fHakka	 ngi5; ngi6; ngi3
U+7FBF.0	fCantonese	ngai6; ai6
U+7FBF.0	fMandarin	yi4
U+7FBF.0	fEnglish	Yi, name of a legendary archer
U+7FC0.0	fUTF8	翀
U+7FC0.0	fCNS11643	2-3229
U+7FC0.0	fBig5	D365
U+7FC0.0	fGB	5-6B25
U+7FC0.0	fR/S	 124.4
U+7FC0.0	fTotalStrokes	 10
U+7FC0.0	fCangjie	SML
U+7FC0.0	fFourCorner	15106
U+7FC0.0	fHakka	 chung1; cung1
U+7FC0.0	fCantonese	cung1; zung1
U+7FC0.0	fMandarin	chong2; chong1
U+7FC0.0	fEnglish	fly upward
U+7FC1.0	fUTF8	翁
U+7FC1.0	fCNS11643	1-5751
U+7FC1.0	fBig5	AFCE
U+7FC1.0	fGB	0-4E4C
U+7FC1.0	fR/S	 124.4
U+7FC1.0	fTotalStrokes	 10
U+7FC1.0	fCangjie	CISM
U+7FC1.0	fFourCorner	80127
U+7FC1.0	fHakka	  wung1; vung1
U+7FC1.0	fCantonese	jung1
U+7FC1.0	fMandarin	weng1; weng3
U+7FC1.0	fEnglish	[1] old man [2] father [3] father-in-law [4] a title of respect
U+7FC2.0	fUTF8	翂
U+7FC2.0	fCNS11643	2-3228
U+7FC2.0	fBig5	D364
U+7FC2.0	fGB	3-6B66
U+7FC2.0	fR/S	 124.4
U+7FC2.0	fTotalStrokes	 10
U+7FC2.0	fCangjie	SMCSH
U+7FC2.0	fFourCorner	18227
U+7FC2.0	fHakka	 fun1
U+7FC2.0	fCantonese	fan1
U+7FC2.0	fMandarin	fen1
U+7FC3.0	fUTF8	翃
U+7FC3.0	fCNS11643	2-3227
U+7FC3.0	fBig5	D363
U+7FC3.0	fGB	3-6B64
U+7FC3.0	fR/S	 124.4
U+7FC3.0	fTotalStrokes	 10
U+7FC3.0	fCangjie	KISMM
U+7FC3.0	fFourCorner	07720
U+7FC3.0	fHakka	 fen2
U+7FC3.0	fCantonese	wang4
U+7FC3.0	fMandarin	hong2
U+7FC3.0	fEnglish	 fly (said of insects)
U+7FC4.0	fUTF8	翄
U+7FC4.0	fCNS11643	3-3342
U+7FC4.0	fGB	5-6B23
U+7FC4.0	fR/S	  124.4
U+7FC4.0	fTotalStrokes	 10
U+7FC4.0	fCangjie	SMJE
U+7FC4.0	fFourCorner	14140
U+7FC4.0	fHakka	 ti5; chi5; cih5
U+7FC4.0	fCantonese	ci3
U+7FC4.0	fMandarin	chi4
U+7FC4.0	fEnglish	 winds fins
U+7FC5.0	fUTF8	翅
U+7FC5.0	fCNS11643	1-5750
U+7FC5.0	fBig5	AFCD
U+7FC5.0	fGB	0-3361
U+7FC5.0	fR/S	 124.4
U+7FC5.0	fTotalStrokes	 10
U+7FC5.0	fCangjie	JESMM
U+7FC5.0	fFourCorner	47402
U+7FC5.0	fHakka	 ci5;  chi5; ti5; ci2; cih5; chii5
U+7FC5.0	fCantonese	ci3
U+7FC5.0	fMandarin	chi4; shi1; ji4; qi2
U+7FC5.0	fEnglish	[1] wings [2] fins
U+7FC6.0	fUTF8	翆
U+7FC6.0	fCNS11643	3-3343
U+7FC6.0	fGB	E-392D
U+7FC6.0	fR/S	 124.4
U+7FC6.0	fTotalStrokes	 10
U+7FC6.0	fCangjie	SMKNJ
U+7FC6.0	fFourCorner	17400
U+7FC6.0	fCantonese	ceoi3
U+7FC6.0	fMandarin	cui4; cui2
U+7FC6.0	fEnglish	 color green kingfisher
U+7FC7.0	fUTF8	翇
U+7FC7.0	fCNS11643	2-3840
U+7FC7.0	fBig5	D6FB
U+7FC7.0	fGB	5-6B2B
U+7FC7.0	fR/S	 124.5
U+7FC7.0	fTotalStrokes	 11
U+7FC7.0	fCangjie	SMIKK
U+7FC7.0	fFourCorner	17440
U+7FC7.0	fCantonese	fat1
U+7FC7.0	fMandarin	fu2; pei4
U+7FC8.0	fUTF8	翈
U+7FC8.0	fCNS11643	4-343D
U+7FC8.0	fGB	3-6B6B
U+7FC8.0	fR/S	  124.5
U+7FC8.0	fTotalStrokes	 11
U+7FC8.0	fCangjie	WLSMM
U+7FC8.0	fFourCorner	67520
U+7FC8.0	fCantonese	haap6
U+7FC8.0	fMandarin	xia2
U+7FC9.0	fUTF8	翉
U+7FC9.0	fCNS11643	2-3842
U+7FC9.0	fBig5	D6FD
U+7FC9.0	fGB	3-6B69
U+7FC9.0	fR/S	 124.5
U+7FC9.0	fTotalStrokes	 11
U+7FC9.0	fCangjie	DMSMM
U+7FC9.0	fFourCorner	57020
U+7FC9.0	fCantonese	pun2
U+7FC9.0	fMandarin	pen3
U+7FCA.0	fUTF8	翊
U+7FCA.0	fCNS11643	2-383B
U+7FCA.0	fBig5	D6F6
U+7FCA.0	fGB	0-7134
U+7FCA.0	fR/S	 124.5
U+7FCA.0	fTotalStrokes	 11
U+7FCA.0	fCangjie	YTSMM
U+7FCA.0	fFourCorner	07120
U+7FCA.0	fHakka	  jit8; rit8
U+7FCA.0	fCantonese	jik6
U+7FCA.0	fMandarin	yi4
U+7FCA.0	fEnglish	[1] flying [2] assist; help [3] respectful
U+7FCB.0	fUTF8	翋
U+7FCB.0	fCNS11643	2-383C
U+7FCB.0	fBig5	D6F7
U+7FCB.0	fGB	3-6B71
U+7FCB.0	fR/S	 124.5
U+7FCB.0	fTotalStrokes	 11
U+7FCB.0	fCangjie	SMYT
U+7FCB.0	fFourCorner	10118
U+7FCB.0	fCantonese	laap6
U+7FCB.0	fMandarin	la1; la4
U+7FCC.0	fUTF8	翌
U+7FCC.0	fCNS11643	1-5C62
U+7FCC.0	fBig5	B2DD
U+7FCC.0	fGB	0-526E
U+7FCC.0	fR/S	 124.5
U+7FCC.0	fTotalStrokes	 11
U+7FCC.0	fCangjie	SMYT
U+7FCC.0	fFourCorner	17108
U+7FCC.0	fHakka	  jit8; rit8
U+7FCC.0	fCantonese	jik6
U+7FCC.0	fMandarin	yi4; yu4
U+7FCC.0	fEnglish	tomorrow; next day
U+7FCD.0	fUTF8	翍
U+7FCD.0	fCNS11643	2-383D
U+7FCD.0	fBig5	D6F8
U+7FCD.0	fGB	5-6B2C
U+7FCD.0	fR/S	 124.5
U+7FCD.0	fTotalStrokes	 11
U+7FCD.0	fCangjie	SMDHE
U+7FCD.0	fFourCorner	14247
U+7FCD.0	fHakka	  pi1
U+7FCD.0	fCantonese	pei1; bei1
U+7FCD.0	fMandarin	pi1; po1; bi4
U+7FCD.0	fEnglish	 spread out; expand
U+7FCE.0	fUTF8	翎
U+7FCE.0	fCNS11643	1-5C63
U+7FCE.0	fBig5	B2DE
U+7FCE.0	fGB	0-7461
U+7FCE.0	fR/S	 124.5
U+7FCE.0	fTotalStrokes	 11
U+7FCE.0	fCangjie	OISMM
U+7FCE.0	fFourCorner	87320
U+7FCE.0	fHakka	 lin2
U+7FCE.0	fCantonese	ling4
U+7FCE.0	fMandarin	ling2
U+7FCE.0	fEnglish	 feather; plume; wing
U+7FCF.0	fUTF8	翏
U+7FCF.0	fCNS11643	2-3841
U+7FCF.0	fBig5	D6FC
U+7FCF.0	fGB	3-6B6E
U+7FCF.0	fR/S	 124.5
U+7FCF.0	fTotalStrokes	 11
U+7FCF.0	fCangjie	SMOHH
U+7FCF.0	fFourCorner	17202
U+7FCF.0	fHakka	  liu5;  liau2; liau5
U+7FCF.0	fCantonese	lau6; liu4 (also); liu6 (also); luk6
U+7FCF.0	fMandarin	liu4; liu2; liao4; liao2; lu4
U+7FCF.0	fEnglish	 the sound of the wind soar
U+7FD0.0	fUTF8	翐
U+7FD0.0	fCNS11643	2-383E
U+7FD0.0	fBig5	D6F9
U+7FD0.0	fGB	3-6B6D
U+7FD0.0	fR/S	 124.5
U+7FD0.0	fTotalStrokes	 11
U+7FD0.0	fCangjie	SMHQO
U+7FD0.0	fFourCorner	15130
U+7FD0.0	fCantonese	dit6
U+7FD0.0	fMandarin	zhi4
U+7FD1.0	fUTF8	翑
U+7FD1.0	fCNS11643	2-383F
U+7FD1.0	fBig5	D6FA
U+7FD1.0	fGB	3-6B6F
U+7FD1.0	fR/S	 124.5
U+7FD1.0	fTotalStrokes	 11
U+7FD1.0	fCangjie	SMPR
U+7FD1.0	fFourCorner	17120
U+7FD1.0	fHakka	 gi3
U+7FD1.0	fCantonese	keoi4; geoi2; jyu6
U+7FD1.0	fMandarin	qu2; ju3; shu3; yu4
U+7FD1.0	fEnglish	[1] crooked plumes [2] horse with white hind feet [3] feather on an arrow
U+7FD2.0	fUTF8	習
U+7FD2.0	fCNS11643	1-5C64
U+7FD2.0	fBig5	B2DF
U+7FD2.0	fGB	1-4F30
U+7FD2.0	fR/S	 124.5
U+7FD2.0	fTotalStrokes	 11
U+7FD2.0	fCangjie	SMHA
U+7FD2.0	fFourCorner	17602
U+7FD2.0	fHakka	  sip8
U+7FD2.0	fCantonese	zaap6; zap6
U+7FD2.0	fMandarin	xi2
U+7FD2.0	fEnglish	[1] learn; familiarise oneself with; receive training in; practice [2] follow; repeat [3] a Chinese family name
U+7FD3.0	fUTF8	翓
U+7FD3.0	fCNS11643	3-3E37
U+7FD3.0	fGB	5-6B2D
U+7FD3.0	fR/S	  124.6
U+7FD3.0	fTotalStrokes	 12
U+7FD3.0	fCangjie	GRSMM
U+7FD3.0	fFourCorner	47620
U+7FD3.0	fCantonese	kit3
U+7FD3.0	fMandarin	xie2
U+7FD4.0	fUTF8	翔
U+7FD4.0	fCNS11643	1-6144
U+7FD4.0	fBig5	B5BE
U+7FD4.0	fGB	0-4F68
U+7FD4.0	fR/S	 124.6
U+7FD4.0	fTotalStrokes	 12
U+7FD4.0	fCangjie	TQSMM
U+7FD4.0	fFourCorner	87520
U+7FD4.0	fHakka	 siong2; siong3
U+7FD4.0	fCantonese	coeng4
U+7FD4.0	fMandarin	xiang2
U+7FD4.0	fEnglish	[1] soar [2] same as $ (detailed)
U+7FD5.0	fUTF8	翕
U+7FD5.0	fCNS11643	1-6145
U+7FD5.0	fBig5	B5BF
U+7FD5.0	fGB	0-7462
U+7FD5.0	fR/S	 124.6
U+7FD5.0	fTotalStrokes	 12
U+7FD5.0	fCangjie	ORSMM
U+7FD5.0	fFourCorner	80127
U+7FD5.0	fHakka	 hip7; him5
U+7FD5.0	fCantonese	jap1; jap6; hap1
U+7FD5.0	fMandarin	xi4
U+7FD5.0	fEnglish	[1] fold; close [2] draw together; gather
U+7FD6.0	fUTF8	翖
U+7FD6.0	fCNS11643	3-3E38
U+7FD6.0	fGB	E-392E
U+7FD6.0	fR/S	  124.6
U+7FD6.0	fTotalStrokes	 12
U+7FD6.0	fCangjie	XORSM
U+7FD6.0	fFourCorner	87620
U+7FD6.0	fHakka	 lin3
U+7FD6.0	fCantonese	jap1
U+7FD6.0	fMandarin	xi4
U+7FD7.0	fUTF8	翗
U+7FD7.0	fCNS11643	2-3F29
U+7FD7.0	fBig5	DB44
U+7FD7.0	fGB	5-6B30
U+7FD7.0	fR/S	 124.6
U+7FD7.0	fTotalStrokes	 12
U+7FD7.0	fCangjie	NNSMM
U+7FD7.0	fFourCorner	27220
U+7FD7.0	fCantonese	ke4; koe4
U+7FD7.0	fMandarin	qi2
U+7FD8.0	fUTF8	翘
U+7FD8.0	fGB	0-474C
U+7FD8.0	fR/S	 124.6
U+7FD8.0	fTotalStrokes	 12
U+7FD8.0	fCangjie	JUSMM
U+7FD8.0	fHakka	 kiau2
U+7FD8.0	fCantonese	kiu4; kiu3
U+7FD8.0	fMandarin	qiao2; qiao4
U+7FD8.0	fEnglish	simplified form of &#x7FF9;
U+7FD9.0	fUTF8	翙
U+7FD9.0	fGB	8-2C6A
U+7FD9.0	fR/S	 124.6
U+7FD9.0	fTotalStrokes	 12
U+7FD9.0	fCangjie	UNSMM
U+7FD9.0	fCantonese	wai3
U+7FD9.0	fMandarin	hui4
U+7FDA.0	fUTF8	翚
U+7FDA.0	fGB	8-7E53
U+7FDA.0	fR/S	 124.6
U+7FDA.0	fTotalStrokes	 12
U+7FDA.0	fCangjie	SMBKQ
U+7FDA.0	fCantonese	fai1
U+7FDA.0	fMandarin	hui1
U+7FDB.0	fUTF8	翛
U+7FDB.0	fCNS11643	2-4636
U+7FDB.0	fBig5	DF6F
U+7FDB.0	fGB	3-317A
U+7FDB.0	fR/S	 124.7
U+7FDB.0	fTotalStrokes	 13
U+7FDB.0	fCangjie	OLOM
U+7FDB.0	fFourCorner	27227
U+7FDB.0	fHakka	 siau1
U+7FDB.0	fCantonese	siu1; suk1
U+7FDB.0	fMandarin	xiao1; shu4; you2; you3; yu4
U+7FDB.0	fEnglish	 look of rumpled feathers bedraggled hasty rapid flight
U+7FDC.0	fUTF8	翜
U+7FDC.0	fCNS11643	2-4637
U+7FDC.0	fBig5	DF70
U+7FDC.0	fGB	3-6B73
U+7FDC.0	fR/S	 124.7
U+7FDC.0	fTotalStrokes	 13
U+7FDC.0	fCangjie	SMKOO
U+7FDC.0	fFourCorner	17438
U+7FDC.0	fHakka	  sep7
U+7FDC.0	fCantonese	saap3
U+7FDC.0	fMandarin	se4; sha4
U+7FDC.0	fEnglish	 fly fast
U+7FDD.0	fUTF8	翝
U+7FDD.0	fCNS11643	3-4372
U+7FDD.0	fGB	E-392F
U+7FDD.0	fR/S	 124.7
U+7FDD.0	fTotalStrokes	 13
U+7FDD.0	fCangjie	JISMM
U+7FDD.0	fFourCorner	37420
U+7FDD.0	fCantonese	wang4
U+7FDD.0	fMandarin	hong2
U+7FDE.0	fUTF8	翞
U+7FDE.0	fCNS11643	2-4D27
U+7FDE.0	fBig5	E37E
U+7FDE.0	fGB	E-3930
U+7FDE.0	fR/S	 124.8
U+7FDE.0	fTotalStrokes	 14
U+7FDE.0	fCangjie	SMYRF
U+7FDE.0	fFourCorner	10196
U+7FDE.0	fCantonese	goeng1
U+7FDE.0	fMandarin	jiang1
U+7FDF.0	fUTF8	翟
U+7FDF.0	fCNS11643	1-6A4B
U+7FDF.0	fBig5	BB43
U+7FDF.0	fGB	0-3554
U+7FDF.0	fR/S	 124.8
U+7FDF.0	fTotalStrokes	 14
U+7FDF.0	fCangjie	SMOG
U+7FDF.0	fFourCorner	17214
U+7FDF.0	fHakka	 tit8; shak8
U+7FDF.0	fCantonese	dik6; zaak6
U+7FDF.0	fMandarin	zhai2; zhe2; zhuo2
U+7FDF.0	fEnglish	a Chinese family name
U+7FDF.1	fUTF8	翟
U+7FDF.1	fCNS11643	1-6A4B
U+7FDF.1	fBig5	BB43
U+7FDF.1	fGB	0-3554
U+7FDF.1	fR/S	 124.8
U+7FDF.1	fTotalStrokes	 14
U+7FDF.1	fCangjie	SMOG
U+7FDF.1	fFourCorner	17214
U+7FDF.1	fMandarin	di2
U+7FDF.1	fEnglish	a kind of pheasant with long tail feathers
U+7FE0.0	fUTF8	翠
U+7FE0.0	fCNS11643	1-6A49
U+7FE0.0	fBig5	BB41
U+7FE0.0	fGB	0-3464
U+7FE0.0	fR/S	 124.8
U+7FE0.0	fTotalStrokes	 14
U+7FE0.0	fCangjie	SMYOJ
U+7FE0.0	fFourCorner	17408
U+7FE0.0	fHakka	  cui5
U+7FE0.0	fCantonese	ceoi3; cui3
U+7FE0.0	fMandarin	cui4
U+7FE0.0	fEnglish	[1] bluish green [2] kingfisher [3] green jade
U+7FE1.0	fUTF8	翡
U+7FE1.0	fCNS11643	1-6A4A
U+7FE1.0	fBig5	BB42
U+7FE1.0	fGB	0-7464
U+7FE1.0	fR/S	 124.8
U+7FE1.0	fTotalStrokes	 14
U+7FE1.0	fCangjie	LYSMM
U+7FE1.0	fFourCorner	11227
U+7FE1.0	fHakka	 fui3; fui1; fi5; fui6; fui5; fi1
U+7FE1.0	fCantonese	fei2; fei6
U+7FE1.0	fMandarin	fei3; fei4
U+7FE1.0	fEnglish	[1] kingfisher [2] emerald
U+7FE2.0	fUTF8	翢
U+7FE2.0	fCNS11643	2-4D24
U+7FE2.0	fBig5	E37B
U+7FE2.0	fGB	5-6B33
U+7FE2.0	fR/S	 124.8
U+7FE2.0	fTotalStrokes	 14
U+7FE2.0	fCangjie	BRSMM
U+7FE2.0	fFourCorner	77220
U+7FE2.0	fCantonese	duk6; dou6; tou1; tou4
U+7FE2.0	fMandarin	tao1; tao2; dao4
U+7FE3.0	fUTF8	翣
U+7FE3.0	fCNS11643	2-4D25
U+7FE3.0	fBig5	E37C
U+7FE3.0	fGB	3-6B7A
U+7FE3.0	fR/S	 124.8
U+7FE3.0	fTotalStrokes	 14
U+7FE3.0	fCangjie	SMYTV
U+7FE3.0	fFourCorner	17404
U+7FE3.0	fHakka	  ciap7; ciap8
U+7FE3.0	fCantonese	saap3
U+7FE3.0	fMandarin	sha4
U+7FE3.0	fEnglish	feathers used to decorate a coffin; feathers adorning a coffin
U+7FE4.0	fUTF8	翤
U+7FE4.0	fCNS11643	6-6058
U+7FE4.0	fGB	E-3931
U+7FE4.0	fR/S	 124.8
U+7FE4.0	fTotalStrokes	 14
U+7FE4.0	fCangjie	RBSMM
U+7FE4.0	fFourCorner	67420
U+7FE4.0	fCantonese	ci3
U+7FE4.0	fMandarin	chi4
U+7FE5.0	fUTF8	翥
U+7FE5.0	fCNS11643	2-4D26
U+7FE5.0	fBig5	E37D
U+7FE5.0	fGB	0-7463
U+7FE5.0	fR/S	 124.9
U+7FE5.0	fTotalStrokes	 15
U+7FE5.0	fCangjie	JASMM
U+7FE5.0	fFourCorner	44127
U+7FE5.0	fHakka	 zhu5; zu5
U+7FE5.0	fCantonese	zyu3; zyu2
U+7FE5.0	fMandarin	zhu4
U+7FE5.0	fEnglish	 take of; soar
U+7FE6.0	fUTF8	翦
U+7FE6.0	fCNS11643	2-5323
U+7FE6.0	fBig5	E6F9
U+7FE6.0	fGB	0-7465
U+7FE6.0	fR/S	 124.9
U+7FE6.0	fTotalStrokes	 15
U+7FE6.0	fCangjie	TBNM
U+7FE6.0	fFourCorner	80127
U+7FE6.0	fHakka	  zien3; tsian1; zen3
U+7FE6.0	fCantonese	zin2
U+7FE6.0	fMandarin	jian3; jian4
U+7FE6.0	fEnglish	same as $ ( scissors; cut; clip; trim)
U+7FE7.0	fUTF8	翧
U+7FE7.0	fCNS11643	3-4E32
U+7FE7.0	fGB	5-6B38
U+7FE7.0	fR/S	 124.9
U+7FE7.0	fTotalStrokes	 15
U+7FE7.0	fCangjie	JMSMM
U+7FE7.0	fFourCorner	37120
U+7FE7.0	fCantonese	hyun1
U+7FE7.0	fMandarin	xuan1
U+7FE8.0	fUTF8	翨
U+7FE8.0	fCNS11643	2-5324
U+7FE8.0	fBig5	E6FA
U+7FE8.0	fGB	3-6B7D
U+7FE8.0	fR/S	 124.9
U+7FE8.0	fTotalStrokes	 15
U+7FE8.0	fCangjie	SMAMO
U+7FE8.0	fFourCorner	17801
U+7FE8.0	fHakka	 chi5; cih5; chi3
U+7FE8.0	fCantonese	ci3; si6
U+7FE8.0	fMandarin	shi4; ji4; chi4
U+7FE8.0	fEnglish	fins, wings
U+7FE9.0	fUTF8	翩
U+7FE9.0	fCNS11643	1-6E4A
U+7FE9.0	fBig5	BDA1
U+7FE9.0	fGB	0-7466
U+7FE9.0	fR/S	 124.9
U+7FE9.0	fTotalStrokes	 15
U+7FE9.0	fCangjie	HBSMM
U+7FE9.0	fFourCorner	17220
U+7FE9.0	fHakka	  pien1; fan1; pen1
U+7FE9.0	fCantonese	pin1
U+7FE9.0	fMandarin	pian1
U+7FE9.0	fEnglish	 fly swiftly
U+7FEA.0	fUTF8	翪
U+7FEA.0	fCNS11643	2-5321
U+7FEA.0	fBig5	E6F7
U+7FEA.0	fGB	5-6B37
U+7FEA.0	fR/S	 124.9
U+7FEA.0	fTotalStrokes	 15
U+7FEA.0	fCangjie	SMUCE
U+7FEA.0	fFourCorner	12147
U+7FEA.0	fHakka	  zung1
U+7FEA.0	fCantonese	zung1
U+7FEA.0	fMandarin	zong1; zong3; zong4
U+7FEA.0	fEnglish	flutter
U+7FEB.0	fUTF8	翫
U+7FEB.0	fCNS11643	2-527E
U+7FEB.0	fBig5	E6F6
U+7FEB.0	fGB	E-3932
U+7FEB.0	fR/S	 124.9
U+7FEB.0	fTotalStrokes	 15
U+7FEB.0	fCangjie	SAMMU
U+7FEB.0	fFourCorner	11611
U+7FEB.0	fHakka	 ngan3; ngan5
U+7FEB.0	fCantonese	wun6; waan4; waan2
U+7FEB.0	fMandarin	wan4; wan1
U+7FEB.0	fEnglish	[1] careless or casual due to familiarity [2] play
U+7FEC.0	fUTF8	翬
U+7FEC.0	fCNS11643	2-5322
U+7FEC.0	fBig5	E6F8
U+7FEC.0	fGB	3-6B76
U+7FEC.0	fR/S	 124.9
U+7FEC.0	fTotalStrokes	 15
U+7FEC.0	fCangjie	SMBJJ
U+7FEC.0	fFourCorner	17502
U+7FEC.0	fHakka	 fui1; fi1
U+7FEC.0	fCantonese	fai1
U+7FEC.0	fMandarin	hui1
U+7FEC.0	fEnglish	[1] fly [2] a multi-colored pheasant
U+7FED.0	fUTF8	翭
U+7FED.0	fCNS11643	2-527D
U+7FED.0	fBig5	E6F5
U+7FED.0	fGB	3-6B7E
U+7FED.0	fR/S	 124.9
U+7FED.0	fTotalStrokes	 15
U+7FED.0	fCangjie	SMONK
U+7FED.0	fFourCorner	17134
U+7FED.0	fCantonese	hau4
U+7FED.0	fMandarin	hou2; hou4
U+7FEE.0	fUTF8	翮
U+7FEE.0	fCNS11643	1-7175
U+7FEE.0	fBig5	BFAD
U+7FEE.0	fGB	0-7467
U+7FEE.0	fR/S	 124.10
U+7FEE.0	fTotalStrokes	 16
U+7FEE.0	fCangjie	MBSMM
U+7FEE.0	fFourCorner	17220
U+7FEE.0	fHakka	 gak7
U+7FEE.0	fCantonese	hat6; gaak3
U+7FEE.0	fMandarin	he2; li4; ge2
U+7FEE.0	fEnglish	the shaft of a feather
U+7FEF.0	fUTF8	翯
U+7FEF.0	fCNS11643	2-5950
U+7FEF.0	fBig5	EAE4
U+7FEF.0	fGB	3-6C22
U+7FEF.0	fR/S	 124.10
U+7FEF.0	fTotalStrokes	 16
U+7FEF.0	fCangjie	SMYRB
U+7FEF.0	fFourCorner	17227
U+7FEF.0	fHakka	 hok8
U+7FEF.0	fCantonese	hok6
U+7FEF.0	fMandarin	he4; hu2; he2; xue2; hao4
U+7FEF.0	fEnglish	 glistening plumage reflection of the sun on water
U+7FF0.0	fUTF8	翰
U+7FF0.0	fCNS11643	1-7173
U+7FF0.0	fBig5	BFAB
U+7FF0.0	fGB	0-3A32
U+7FF0.0	fR/S	 124.10
U+7FF0.0	fTotalStrokes	 16
U+7FF0.0	fCangjie	JJOSM
U+7FF0.0	fFourCorner	48427
U+7FF0.0	fHakka	  hon5
U+7FF0.0	fCantonese	hon6
U+7FF0.0	fMandarin	han4; han2
U+7FF0.0	fEnglish	[1] white horse [2] a long and hard feather [3] piece of writing
U+7FF1.0	fUTF8	翱
U+7FF1.0	fCNS11643	1-7174
U+7FF1.0	fBig5	BFAC
U+7FF1.0	fGB	0-303F
U+7FF1.0	fR/S	 124.10
U+7FF1.0	fTotalStrokes	 16
U+7FF1.0	fCangjie	HJSMM
U+7FF1.0	fFourCorner	27420
U+7FF1.0	fHakka	  ngau2
U+7FF1.0	fCantonese	ngou4; ou4 (also); ngou6
U+7FF1.0	fMandarin	ao2
U+7FF1.0	fEnglish	[1] soar; fly [2] roam; wander
U+7FF2.0	fUTF8	翲
U+7FF2.0	fCNS11643	2-5E53
U+7FF2.0	fBig5	EDE6
U+7FF2.0	fGB	5-6B3B
U+7FF2.0	fR/S	 124.11
U+7FF2.0	fTotalStrokes	 17
U+7FF2.0	fCangjie	MFSMM
U+7FF2.0	fFourCorner	17920
U+7FF2.0	fCantonese	piu1; piu3
U+7FF2.0	fMandarin	piao1; piao4
U+7FF3.0	fUTF8	翳
U+7FF3.0	fCNS11643	1-7475
U+7FF3.0	fBig5	C16B
U+7FF3.0	fGB	0-7468
U+7FF3.0	fR/S	 124.11
U+7FF3.0	fTotalStrokes	 17
U+7FF3.0	fCangjie	SESMM
U+7FF3.0	fFourCorner	77127
U+7FF3.0	fHakka	 ji5; ji3; ri5; i5
U+7FF3.0	fCantonese	ai3; ngai3 (also); ji1
U+7FF3.0	fMandarin	yi4; yi1; ye4
U+7FF3.0	fEnglish	[1] screen; hide; conceal [2] the haziness of objects due to weakened vision [3] the chariot cover made of feathers [4] the film over a diseased eye
U+7FF4.0	fUTF8	翴
U+7FF4.0	fCNS11643	2-5E52
U+7FF4.0	fBig5	EDE5
U+7FF4.0	fGB	5-6B31
U+7FF4.0	fR/S	 124.11
U+7FF4.0	fTotalStrokes	 17
U+7FF4.0	fCangjie	SMYJJ
U+7FF4.0	fFourCorner	15130
U+7FF4.0	fCantonese	lin4
U+7FF4.0	fMandarin	lian2
U+7FF5.0	fUTF8	翵
U+7FF5.0	fCNS11643	2-6134
U+7FF5.0	fBig5	EFA8
U+7FF5.0	fGB	E-3933
U+7FF5.0	fR/S	 124.11; 196.6
U+7FF5.0	fTotalStrokes	 17
U+7FF5.0	fCangjie	SMHAF
U+7FF5.0	fFourCorner	17127
U+7FF5.0	fCantonese	hau4; keoi4
U+7FF5.0	fMandarin	qu2; hou2
U+7FF6.0	fUTF8	翶
U+7FF6.0	fCNS11643	3-557E
U+7FF6.0	fGB	E-3934
U+7FF6.0	fR/S	 124.11
U+7FF6.0	fTotalStrokes	 17
U+7FF6.0	fCangjie	XXHJS
U+7FF6.0	fFourCorner	27420
U+7FF6.0	fCantonese	ngou4; ou4 (also)
U+7FF6.0	fMandarin	ao2
U+7FF6.0	fEnglish	[1] fly like the common hawk-kite; soar [2] roam about
U+7FF7.0	fUTF8	翷
U+7FF7.0	fCNS11643	2-6266
U+7FF7.0	fBig5	F07A
U+7FF7.0	fGB	5-6B3D
U+7FF7.0	fR/S	 124.12
U+7FF7.0	fTotalStrokes	 18
U+7FF7.0	fCangjie	FQSMM
U+7FF7.0	fFourCorner	97220
U+7FF7.0	fCantonese	leon4
U+7FF7.0	fMandarin	lin2
U+7FF8.0	fUTF8	翸
U+7FF8.0	fCNS11643	2-6267
U+7FF8.0	fBig5	F07B
U+7FF8.0	fGB	5-6B35
U+7FF8.0	fR/S	 124.12
U+7FF8.0	fTotalStrokes	 18
U+7FF8.0	fCangjie	JCSMM
U+7FF8.0	fFourCorner	07820
U+7FF8.0	fCantonese	pun2
U+7FF8.0	fMandarin	pen3
U+7FF9.0	fUTF8	翹
U+7FF9.0	fCNS11643	1-7728
U+7FF9.0	fBig5	C2BC
U+7FF9.0	fGB	1-474C
U+7FF9.0	fR/S	 124.12
U+7FF9.0	fTotalStrokes	 18
U+7FF9.0	fCangjie	GUSMM
U+7FF9.0	fFourCorner	47212
U+7FF9.0	fHakka	 kiau5;  kiau2; hiau5; k'ieu5; hiau1; hieu5; hieu1; kiau3
U+7FF9.0	fCantonese	kiu4; kiu3; giu6
U+7FF9.0	fMandarin	qiao2
U+7FF9.0	fEnglish	[1] lift; elevate; raise [2] long tail of feathers [3] outstanding
U+7FF9.1	fUTF8	翹
U+7FF9.1	fCNS11643	1-7728
U+7FF9.1	fBig5	C2BC
U+7FF9.1	fGB	1-474C
U+7FF9.1	fR/S	 124.12
U+7FF9.1	fTotalStrokes	 18
U+7FF9.1	fCangjie	GUSMM
U+7FF9.1	fFourCorner	47212
U+7FF9.1	fMandarin	qiao4
U+7FF9.1	fEnglish	 project upward; stick up; turn upward
U+7FFA.0	fUTF8	翺
U+7FFA.0	fCNS11643	3-5861
U+7FFA.0	fGB	E-3935
U+7FFA.0	fR/S	 124.12
U+7FFA.0	fTotalStrokes	 18
U+7FFA.0	fCangjie	HJSMM
U+7FFA.0	fFourCorner	27420
U+7FFA.0	fCantonese	ngou4; ou4 (also)
U+7FFA.0	fMandarin	ao2
U+7FFA.0	fEnglish	[1] soar; roam about [2] fly like the common hawk-kite
U+7FFB.0	fUTF8	翻
U+7FFB.0	fCNS11643	1-7729
U+7FFB.0	fBig5	C2BD
U+7FFB.0	fGB	0-372D
U+7FFB.0	fR/S	 124.12
U+7FFB.0	fTotalStrokes	 18
U+7FFB.0	fCangjie	HWSMM
U+7FFB.0	fFourCorner	27620
U+7FFB.0	fHakka	  fan1;  pan1; pon1
U+7FFB.0	fCantonese	faan1
U+7FFB.0	fMandarin	fan1
U+7FFB.0	fEnglish	[1] fly; flutter [2] turn; flip over; upset; capsize [3] rummage [4] translate [5] fall out; break up
U+7FFC.0	fUTF8	翼
U+7FFC.0	fCNS11643	1-7476
U+7FFC.0	fBig5	C16C
U+7FFC.0	fGB	0-526D
U+7FFC.0	fR/S	 124.12; 124.11
U+7FFC.0	fTotalStrokes	 18
U+7FFC.0	fCangjie	SMWTC
U+7FFC.0	fFourCorner	17801
U+7FFC.0	fHakka	  jit8; rit8
U+7FFC.0	fCantonese	jik6; jik2 (also)
U+7FFC.0	fMandarin	yi4
U+7FFC.0	fEnglish	[1] wings [2] fins (fish) [3] shelter; protect; patronise; harbour [4] assist; help
U+7FFD.0	fUTF8	翽
U+7FFD.0	fCNS11643	2-6648
U+7FFD.0	fBig5	F2BE
U+7FFD.0	fGB	3-6B74
U+7FFD.0	fR/S	 124.13
U+7FFD.0	fTotalStrokes	 19
U+7FFD.0	fCangjie	YHSMM
U+7FFD.0	fFourCorner	27220
U+7FFD.0	fHakka	 fui5; fi5; wi5; wui5; vui5
U+7FFD.0	fCantonese	wai3
U+7FFD.0	fMandarin	hui4; kuai4
U+7FFD.0	fEnglish	sounds of winds flapping
U+7FFE.0	fUTF8	翾
U+7FFE.0	fCNS11643	2-6649
U+7FFE.0	fBig5	F2BF
U+7FFE.0	fGB	3-6C23
U+7FFE.0	fR/S	 124.13
U+7FFE.0	fTotalStrokes	 19
U+7FFE.0	fCangjie	WVSMM
U+7FFE.0	fFourCorner	67720
U+7FFE.0	fHakka	  fan2; sien2
U+7FFE.0	fCantonese	hyun1
U+7FFE.0	fMandarin	xuan1
U+7FFE.0	fEnglish	 flit
U+7FFF.0	fUTF8	翿
U+7FFF.0	fCNS11643	2-695E
U+7FFF.0	fBig5	F4B1
U+7FFF.0	fGB	3-6B77
U+7FFF.0	fR/S	 124.14
U+7FFF.0	fTotalStrokes	 20
U+7FFF.0	fCangjie	GISMM
U+7FFF.0	fFourCorner	47620
U+7FFF.0	fHakka	 tau2; to2
U+7FFF.0	fCantonese	dou6; tou4 (also)
U+7FFF.0	fMandarin	dao4; tao2
U+7FFF.0	fEnglish	a kind of streamer or screen adorned with feathers carried by dancers
U+8000.0	fUTF8	耀
U+8000.0	fCNS11643	1-7A2E
U+8000.0	fBig5	C4A3
U+8000.0	fGB	0-522B
U+8000.0	fR/S	 124.14
U+8000.0	fTotalStrokes	 20
U+8000.0	fCangjie	FUSMG
U+8000.0	fFourCorner	97214
U+8000.0	fHakka	  jau5; rau5; jau3; jeu5
U+8000.0	fCantonese	jiu6
U+8000.0	fMandarin	yao4; yue4
U+8000.0	fEnglish	[1] shine; sparkle; dazzle [2] show off
U+8001.0	fUTF8	老
U+8001.0	fCNS11643	1-4852
U+8001.0	fBig5	A6D1
U+8001.0	fGB	0-404F
U+8001.0	fR/S	 125.0
U+8001.0	fTotalStrokes	 6
U+8001.0	fCangjie	JKP
U+8001.0	fFourCorner	44711
U+8001.0	fHakka	  lau3;  lo3; lo6; lo5
U+8001.0	fCantonese	lou5
U+8001.0	fMandarin	lao3
U+8001.0	fEnglish	[1] old; aged; venerable [2] always [3] the youngest [4] very [5] a particle indicating ordinal numbers to designate order of birth [6] parents; [7] a particle used before a man's family name to indicate familiarity and friendship [8] tough (meat); overcooked [9] treat with the reverence [10] dark (color) [11] very
U+8002.0	fUTF8	耂
U+8002.0	fCNS11643	3-667B
U+8002.0	fGB	E-3936
U+8002.0	fR/S	 125.0
U+8002.0	fTotalStrokes	 4; 6
U+8002.0	fCangjie	JK
U+8002.0	fCantonese	lou5
U+8002.0	fEnglish	variant of radical 125
U+8003.0	fUTF8	考
U+8003.0	fCNS11643	1-4853
U+8003.0	fBig5	A6D2
U+8003.0	fGB	0-3F3C
U+8003.0	fR/S	 125.0
U+8003.0	fTotalStrokes	 6
U+8003.0	fCangjie	JKYS
U+8003.0	fFourCorner	44207
U+8003.0	fHakka	  kau3; ko3
U+8003.0	fCantonese	haau2
U+8003.0	fMandarin	kao3
U+8003.0	fEnglish	[1] examine; test [2] investigate; check; study [3] one's deceased father
U+8004.0	fUTF8	耄
U+8004.0	fCNS11643	1-5753
U+8004.0	fBig5	ACFE
U+8004.0	fGB	0-6B23
U+8004.0	fR/S	 125.4
U+8004.0	fTotalStrokes	 10
U+8004.0	fCangjie	JPHQU
U+8004.0	fFourCorner	44714
U+8004.0	fHakka	 mau3; mo3; mau5
U+8004.0	fCantonese	mou6
U+8004.0	fMandarin	mao4
U+8004.0	fEnglish	[1] very old; in an extremely old age; in one's eighties and nineties [2] confused
U+8005.0	fUTF8	者
U+8005.0	fCNS11643	1-4F2F
U+8005.0	fBig5	AACC
U+8005.0	fGB	0-555F
U+8005.0	fR/S	 125.4; 125.5
U+8005.0	fTotalStrokes	 10
U+8005.0	fCangjie	JKA
U+8005.0	fFourCorner	44600
U+8005.0	fHakka	  za3; zha3
U+8005.0	fCantonese	ze2
U+8005.0	fMandarin	zhe3; zhu1
U+8005.0	fEnglish	[1] he who; those who [2] a particle combining with some words to form adverbials
U+8006.0	fUTF8	耆
U+8006.0	fCNS11643	1-5752
U+8006.0	fBig5	AFCF
U+8006.0	fGB	0-6A48
U+8006.0	fR/S	 125.4
U+8006.0	fTotalStrokes	 10
U+8006.0	fCangjie	JPA
U+8006.0	fFourCorner	44601
U+8006.0	fHakka	 ki2; shi5; sii5
U+8006.0	fCantonese	kei4; zi2; si6
U+8006.0	fMandarin	qi2
U+8006.0	fEnglish	 be in one's sixties; aged; old
U+8006.1	fUTF8	耆
U+8006.1	fCNS11643	1-5752
U+8006.1	fBig5	AFCF
U+8006.1	fGB	0-6A48
U+8006.1	fR/S	 125.4
U+8006.1	fTotalStrokes	 10
U+8006.1	fCangjie	JPA
U+8006.1	fFourCorner	44601
U+8006.1	fMandarin	zhi3
U+8006.1	fEnglish	used in combination
U+8006.2	fUTF8	耆
U+8006.2	fCNS11643	1-5752
U+8006.2	fBig5	AFCF
U+8006.2	fGB	0-6A48
U+8006.2	fR/S	 125.4
U+8006.2	fTotalStrokes	 10
U+8006.2	fCangjie	JPA
U+8006.2	fFourCorner	44601
U+8006.2	fMandarin	shi4
U+8006.2	fEnglish	used in combination
U+8007.0	fUTF8	耇
U+8007.0	fCNS11643	2-2C72
U+8007.0	fBig5	D051
U+8007.0	fGB	3-6042
U+8007.0	fR/S	 125.5
U+8007.0	fTotalStrokes	 11
U+8007.0	fCangjie	JKPR
U+8007.0	fFourCorner	44627
U+8007.0	fHakka	 kieu3; geu3
U+8007.0	fCantonese	gau2
U+8007.0	fMandarin	gou3
U+8007.0	fEnglish	old age; wrinkled face of the elderly; the wizened face of age
U+8008.0	fUTF8	耈
U+8008.0	fCNS11643	3-3836
U+8008.0	fGB	5-5F44
U+8008.0	fR/S	 125.5
U+8008.0	fTotalStrokes	 11
U+8008.0	fCangjie	JPPR
U+8008.0	fFourCorner	44620
U+8008.0	fCantonese	gau2
U+8008.0	fMandarin	gou3
U+8008.0	fEnglish	old age
U+8009.0	fUTF8	耉
U+8009.0	fCNS11643	4-2B25
U+8009.0	fGB	E-3937
U+8009.0	fR/S	 125.5
U+8009.0	fTotalStrokes	 9; 11
U+8009.0	fCangjie	JKYR
U+8009.0	fFourCorner	44620
U+8009.0	fCantonese	gau2
U+8009.0	fMandarin	gou3
U+8009.0	fEnglish	old age
U+800A.0	fUTF8	耊
U+800A.0	fCNS11643	3-3344
U+800A.0	fGB	5-5F48
U+800A.0	fR/S	  125.6
U+800A.0	fTotalStrokes	 10; 12
U+800A.0	fCangjie	JKMIG
U+800A.0	fFourCorner	44100
U+800A.0	fCantonese	dit6
U+800A.0	fMandarin	die4
U+800B.0	fUTF8	耋
U+800B.0	fCNS11643	1-6146
U+800B.0	fBig5	B5C0
U+800B.0	fGB	0-7173
U+800B.0	fR/S	 125.6
U+800B.0	fTotalStrokes	 12
U+800B.0	fCangjie	JPMIG
U+800B.0	fFourCorner	44104
U+800B.0	fHakka	  diet7; zhit7; det7; tsiit7
U+800B.0	fCantonese	dit6
U+800B.0	fMandarin	die2; die4
U+800B.0	fEnglish	octogenarians; in one's eighties
U+800C.0	fUTF8	而
U+800C.0	fCNS11643	1-4854
U+800C.0	fBig5	A6D3
U+800C.0	fGB	0-3678
U+800C.0	fR/S	 126.0
U+800C.0	fTotalStrokes	 6
U+800C.0	fCangjie	MBLL
U+800C.0	fFourCorner	10227
U+800C.0	fHakka	 ji2; ri2; ji3; lu5; i2; lu2
U+800C.0	fCantonese	ji4; nang4; ji2 (also); lang4 (also)
U+800C.0	fMandarin	er2; neng2
U+800C.0	fEnglish	[1] accordingly; otherwise [2] and yet; but; nevertheless [3] you [4] on the condition that; supposing that; if [5] and; also
U+800D.0	fUTF8	耍
U+800D.0	fCNS11643	1-5324
U+800D.0	fBig5	AD41
U+800D.0	fGB	0-4B23
U+800D.0	fR/S	 126.3
U+800D.0	fTotalStrokes	 9
U+800D.0	fCangjie	MBV
U+800D.0	fFourCorner	10404
U+800D.0	fHakka	  sa3; sa5; sa2; sa6; sha3
U+800D.0	fCantonese	saa2
U+800D.0	fMandarin	shua3
U+800D.0	fEnglish	 frolic; play; sport
U+800E.0	fUTF8	耎
U+800E.0	fCNS11643	2-2C73
U+800E.0	fBig5	D052
U+800E.0	fGB	5-5F6A
U+800E.0	fR/S	 126.3
U+800E.0	fTotalStrokes	 9
U+800E.0	fCangjie	MBK
U+800E.0	fFourCorner	10432
U+800E.0	fHakka	 ngion3; ngion1; nion1
U+800E.0	fCantonese	jyun5
U+800E.0	fMandarin	ruan3
U+800E.0	fEnglish	weak; pliable; soft; yielding
U+800F.0	fUTF8	耏
U+800F.0	fCNS11643	2-2C74
U+800F.0	fBig5	D053
U+800F.0	fGB	3-6062
U+800F.0	fR/S	 126.3
U+800F.0	fTotalStrokes	 9
U+800F.0	fCangjie	MBHHH
U+800F.0	fFourCorner	12222
U+800F.0	fHakka	 nai5; nai6; i2
U+800F.0	fCantonese	ji4; noi6; loi6 (also)
U+800F.0	fMandarin	nai4; er2
U+800F.0	fEnglish	whiskers
U+8010.0	fUTF8	耐
U+8010.0	fCNS11643	1-5323
U+8010.0	fBig5	AD40
U+8010.0	fGB	0-444D
U+8010.0	fR/S	 126.3
U+8010.0	fTotalStrokes	 9
U+8010.0	fCangjie	MBDI
U+8010.0	fFourCorner	14200
U+8010.0	fHakka	 nai6; lai3;  nai5; lai5
U+8010.0	fCantonese	noi6; loi6 (also); noi1 (also); loi1 (also); noi2 (also); loi2 (also); nang4
U+8010.0	fMandarin	nai4; neng2
U+8010.0	fEnglish	 endure; bear; stand; resist
U+8011.0	fUTF8	耑
U+8011.0	fCNS11643	1-5325
U+8011.0	fBig5	AD42
U+8011.0	fGB	E-3938
U+8011.0	fR/S	 126.3
U+8011.0	fTotalStrokes	 9
U+8011.0	fCangjie	UMBL
U+8011.0	fFourCorner	22227
U+8011.0	fHakka	 don1; zon1; zhon1
U+8011.0	fCantonese	dyun1; zyun1
U+8011.0	fMandarin	zhuan1; duan1; chuan1
U+8011.0	fEnglish	same as $ (specialized; concentrated)
U+8012.0	fUTF8	耒
U+8012.0	fCNS11643	1-4855
U+8012.0	fBig5	A6D4
U+8012.0	fGB	0-7167
U+8012.0	fR/S	 127.0
U+8012.0	fTotalStrokes	 6
U+8012.0	fCangjie	QD
U+8012.0	fFourCorner	50900
U+8012.0	fHakka	 loi3
U+8012.0	fCantonese	loi6; leoi6 (also); leoi5
U+8012.0	fMandarin	lei3; lei4; lei2
U+8012.0	fEnglish	[1] the wooden handle of a plow [2] Kangxi radical 127
U+8013.0	fUTF8	耓
U+8013.0	fCNS11643	4-2761
U+8013.0	fGB	E-3939
U+8013.0	fR/S	  127.2
U+8013.0	fTotalStrokes	 8
U+8013.0	fCangjie	QDMN
U+8013.0	fFourCorner	51920
U+8013.0	fCantonese	ting1
U+8013.0	fMandarin	ting1
U+8014.0	fUTF8	耔
U+8014.0	fCNS11643	2-2C75
U+8014.0	fBig5	D054
U+8014.0	fGB	0-7168
U+8014.0	fR/S	 127.3
U+8014.0	fTotalStrokes	 9
U+8014.0	fCangjie	QDND
U+8014.0	fFourCorner	57947
U+8014.0	fHakka	 zih3; zu3
U+8014.0	fCantonese	zi2
U+8014.0	fMandarin	zi3; zi1
U+8014.0	fEnglish	 hoe up the earth around a plant
U+8015.0	fUTF8	耕
U+8015.0	fCNS11643	1-5755
U+8015.0	fBig5	AFD1
U+8015.0	fGB	0-387B
U+8015.0	fR/S	 127.4
U+8015.0	fTotalStrokes	 10
U+8015.0	fCangjie	QDTT
U+8015.0	fFourCorner	55900
U+8015.0	fHakka	  gang1
U+8015.0	fCantonese	gaang1; gang1 (also)
U+8015.0	fMandarin	geng1; jing1
U+8015.0	fEnglish	 till; plow; cultivate
U+8016.0	fUTF8	耖
U+8016.0	fCNS11643	2-322A
U+8016.0	fBig5	D366
U+8016.0	fGB	0-7169
U+8016.0	fR/S	 127.4
U+8016.0	fTotalStrokes	 10
U+8016.0	fCangjie	QDFH
U+8016.0	fFourCorner	59920
U+8016.0	fHakka	 cau5;  cau1; sa1
U+8016.0	fCantonese	caau3; caau2
U+8016.0	fMandarin	chao4
U+8016.0	fEnglish	[1] harrow ground after ploughing [2] harrow
U+8017.0	fUTF8	耗
U+8017.0	fCNS11643	1-5757
U+8017.0	fBig5	AFD3
U+8017.0	fGB	0-3A44
U+8017.0	fR/S	 127.4
U+8017.0	fTotalStrokes	 10
U+8017.0	fCangjie	QDHQU
U+8017.0	fFourCorner	52914
U+8017.0	fHakka	  hau1; hau5;  ho1; ho5
U+8017.0	fCantonese	hou3
U+8017.0	fMandarin	hao4; mao4; mao2
U+8017.0	fEnglish	[1] expend; consume; use up; waste; squander [2] news; report
U+8018.0	fUTF8	耘
U+8018.0	fCNS11643	1-5754
U+8018.0	fBig5	AFD0
U+8018.0	fGB	0-5445
U+8018.0	fR/S	 127.4
U+8018.0	fTotalStrokes	 10
U+8018.0	fCangjie	QDMMI
U+8018.0	fFourCorner	51931
U+8018.0	fHakka	  jun2; wun2; jun3; run2
U+8018.0	fCantonese	wan4
U+8018.0	fMandarin	yun2; yun4
U+8018.0	fEnglish	 weed
U+8019.0	fUTF8	耙
U+8019.0	fCNS11643	1-5756
U+8019.0	fBig5	AFD2
U+8019.0	fGB	0-3052
U+8019.0	fR/S	 127.4
U+8019.0	fTotalStrokes	 10
U+8019.0	fCangjie	QDAU
U+8019.0	fFourCorner	57917
U+8019.0	fHakka	  pa2
U+8019.0	fCantonese	paa4; baa3
U+8019.0	fMandarin	ba4; pa2; ba3
U+8019.0	fEnglish	[1] harrow; drag [2] rake
U+801A.0	fUTF8	耚
U+801A.0	fCNS11643	3-3837
U+801A.0	fGB	3-6033
U+801A.0	fR/S	  127.5
U+801A.0	fTotalStrokes	 11
U+801A.0	fCangjie	QDDHE
U+801A.0	fFourCorner	54940
U+801A.0	fCantonese	pei1
U+801A.0	fMandarin	pi1
U+801B.0	fUTF8	耛
U+801B.0	fCNS11643	2-3845
U+801B.0	fBig5	D741
U+801B.0	fGB	3-6034
U+801B.0	fR/S	 127.5
U+801B.0	fTotalStrokes	 11
U+801B.0	fCangjie	QDIR
U+801B.0	fFourCorner	53960
U+801B.0	fCantonese	toi2; ji4
U+801B.0	fMandarin	chi2; yi2
U+801C.0	fUTF8	耜
U+801C.0	fCNS11643	1-5C65
U+801C.0	fBig5	B2E0
U+801C.0	fGB	0-716A
U+801C.0	fR/S	 127.5
U+801C.0	fTotalStrokes	 11
U+801C.0	fCangjie	QDRLR
U+801C.0	fFourCorner	57977
U+801C.0	fHakka	 sii5; si5; se5; cih5
U+801C.0	fCantonese	zi6; ci5
U+801C.0	fMandarin	si4
U+801C.0	fEnglish	 spade-shaped tool; plough
U+801D.0	fUTF8	耝
U+801D.0	fCNS11643	3-3839
U+801D.0	fGB	3-6031
U+801D.0	fR/S	 127.5
U+801D.0	fTotalStrokes	 11
U+801D.0	fCangjie	QDBM
U+801D.0	fFourCorner	57910
U+801D.0	fHakka	 ze2
U+801D.0	fCantonese	ceoi3
U+801D.0	fMandarin	chu2; qu4
U+801E.0	fUTF8	耞
U+801E.0	fCNS11643	2-3844
U+801E.0	fBig5	D740
U+801E.0	fGB	5-5F33
U+801E.0	fR/S	 127.5
U+801E.0	fTotalStrokes	 11
U+801E.0	fCangjie	QDKSR
U+801E.0	fFourCorner	56900
U+801E.0	fHakka	  ga1
U+801E.0	fCantonese	gaa1
U+801E.0	fMandarin	jia1
U+801E.0	fEnglish	flail
U+801F.0	fUTF8	耟
U+801F.0	fCNS11643	2-3843
U+801F.0	fBig5	D6FE
U+801F.0	fGB	E-393A
U+801F.0	fR/S	 127.5
U+801F.0	fTotalStrokes	 11
U+801F.0	fCangjie	QDSS
U+801F.0	fFourCorner	57917
U+801F.0	fCantonese	geoi6
U+801F.0	fMandarin	ju4
U+8020.0	fUTF8	耠
U+8020.0	fCNS11643	4-3A38
U+8020.0	fGB	0-716B
U+8020.0	fR/S	  127.6
U+8020.0	fTotalStrokes	 12
U+8020.0	fCangjie	QDOMR
U+8020.0	fFourCorner	58960
U+8020.0	fCantonese	hap6
U+8020.0	fMandarin	he2; huo1
U+8020.0	fEnglish	till; dig
U+8021.0	fUTF8	耡
U+8021.0	fCNS11643	2-4638
U+8021.0	fBig5	DF71
U+8021.0	fGB	E-393B
U+8021.0	fR/S	 127.7
U+8021.0	fTotalStrokes	 13
U+8021.0	fCangjie	QDBMS
U+8021.0	fFourCorner	54927
U+8021.0	fHakka	 cu2; ze2; cih2
U+8021.0	fCantonese	co4
U+8021.0	fMandarin	chu2; zhu4
U+8021.0	fEnglish	same as $ ( hoe)
U+8022.0	fUTF8	耢
U+8022.0	fGB	0-716C
U+8022.0	fR/S	 127.7
U+8022.0	fTotalStrokes	 13
U+8022.0	fCangjie	QDTBS
U+8022.0	fCantonese	lou6; lou4 (also)
U+8022.0	fMandarin	lao4
U+8022.0	fEnglish	simplified form of &#x802E;
U+8023.0	fUTF8	耣
U+8023.0	fCNS11643	3-4922
U+8023.0	fGB	3-6030
U+8023.0	fR/S	  127.8
U+8023.0	fTotalStrokes	 14
U+8023.0	fCangjie	QDOMB
U+8023.0	fFourCorner	58920
U+8023.0	fCantonese	leon5
U+8023.0	fMandarin	lun3; lun2
U+8024.0	fUTF8	耤
U+8024.0	fCNS11643	2-4D28
U+8024.0	fBig5	E3A1
U+8024.0	fGB	5-5F38
U+8024.0	fR/S	 127.8
U+8024.0	fTotalStrokes	 14
U+8024.0	fCangjie	QDTA
U+8024.0	fFourCorner	54961
U+8024.0	fHakka	  sit8; zia5; cit7
U+8024.0	fCantonese	zik6; ze3; ze6
U+8024.0	fMandarin	ji2; jie4
U+8024.0	fEnglish	plough
U+8025.0	fUTF8	耥
U+8025.0	fCNS11643	4-462C
U+8025.0	fGB	0-716D
U+8025.0	fR/S	  127.8
U+8025.0	fTotalStrokes	 14
U+8025.0	fCangjie	QDFBR
U+8025.0	fFourCorner	59920
U+8025.0	fCantonese	tong2
U+8025.0	fMandarin	tang3; tang1
U+8025.0	fEnglish	farm tool
U+8026.0	fUTF8	耦
U+8026.0	fCNS11643	1-6E4B
U+8026.0	fBig5	BDA2
U+8026.0	fGB	0-716E
U+8026.0	fR/S	 127.9
U+8026.0	fTotalStrokes	 15
U+8026.0	fCangjie	QDWLB
U+8026.0	fFourCorner	56927
U+8026.0	fHakka	 ngieu3; ngeu3; ngiu3
U+8026.0	fCantonese	ngau5; au5 (also)
U+8026.0	fMandarin	ou3
U+8026.0	fEnglish	[1] plow side by side [2] even (numbers); in pairs; team of two [3] spouse; mate
U+8027.0	fUTF8	耧
U+8027.0	fGB	0-716F
U+8027.0	fR/S	 127.9
U+8027.0	fTotalStrokes	 15
U+8027.0	fCangjie	QDFDV
U+8027.0	fHakka	 leu2
U+8027.0	fCantonese	lau4
U+8027.0	fMandarin	lou2; lou3
U+8027.0	fEnglish	simplified form of &#x802C;
U+8028.0	fUTF8	耨
U+8028.0	fCNS11643	1-7176
U+8028.0	fBig5	BFAE
U+8028.0	fGB	0-7171
U+8028.0	fR/S	 127.10
U+8028.0	fTotalStrokes	 16
U+8028.0	fCangjie	QDMVI
U+8028.0	fFourCorner	51943
U+8028.0	fHakka	 ngieu3; ngiu3; neu5
U+8028.0	fCantonese	nau6; lau6 (also)
U+8028.0	fMandarin	nou4
U+8028.0	fEnglish	[1] hoe [2] rake; weed
U+8029.0	fUTF8	耩
U+8029.0	fCNS11643	2-5952
U+8029.0	fBig5	EAE6
U+8029.0	fGB	0-7170
U+8029.0	fR/S	 127.10
U+8029.0	fTotalStrokes	 16
U+8029.0	fCangjie	QDTTB
U+8029.0	fFourCorner	55957
U+8029.0	fHakka	  gong3; kieu1
U+8029.0	fCantonese	gong2; kau1
U+8029.0	fMandarin	jiang3; gou1
U+8029.0	fEnglish	 plough; sow
U+802A.0	fUTF8	耪
U+802A.0	fCNS11643	2-5951
U+802A.0	fBig5	EAE5
U+802A.0	fGB	0-4555
U+802A.0	fR/S	 127.10
U+802A.0	fTotalStrokes	 16
U+802A.0	fCangjie	QDYBS
U+802A.0	fFourCorner	50927
U+802A.0	fCantonese	pong5; pong1 (also); pong3
U+802A.0	fMandarin	pang3
U+802A.0	fEnglish	plow; cultivate
U+802B.0	fUTF8	耫
U+802B.0	fCNS11643	3-5621
U+802B.0	fGB	3-6036
U+802B.0	fR/S	  127.11
U+802B.0	fTotalStrokes	 17
U+802B.0	fCangjie	QDQMC
U+802B.0	fFourCorner	55980
U+802B.0	fCantonese	zaa6
U+802B.0	fMandarin	ze2
U+802C.0	fUTF8	耬
U+802C.0	fCNS11643	2-5E54
U+802C.0	fBig5	EDE7
U+802C.0	fGB	1-716F
U+802C.0	fR/S	 127.11
U+802C.0	fTotalStrokes	 17
U+802C.0	fCangjie	QDLLV
U+802C.0	fFourCorner	55944
U+802C.0	fHakka	  leu2
U+802C.0	fCantonese	lau4
U+802C.0	fMandarin	lou2; lou3
U+802C.0	fEnglish	drill for sowing grain
U+802D.0	fUTF8	耭
U+802D.0	fCNS11643	4-5C71
U+802D.0	fGB	E-393C
U+802D.0	fR/S	  127.12
U+802D.0	fTotalStrokes	 18
U+802D.0	fCangjie	QDVII
U+802D.0	fFourCorner	52950
U+802D.0	fCantonese	gei1
U+802D.0	fMandarin	ji1
U+802D.0	fEnglish	plow; cultivate
U+802E.0	fUTF8	耮
U+802E.0	fCNS11643	3-5862
U+802E.0	fGB	1-716C
U+802E.0	fR/S	  127.12
U+802E.0	fTotalStrokes	 18
U+802E.0	fCangjie	QDFFS
U+802E.0	fFourCorner	59920
U+802E.0	fCantonese	lou6; lou4 (also)
U+802E.0	fMandarin	lao4
U+802F.0	fUTF8	耯
U+802F.0	fCNS11643	3-5C6F
U+802F.0	fGB	5-5F41
U+802F.0	fR/S	 127.14
U+802F.0	fTotalStrokes	 20
U+802F.0	fCangjie	QDTOE
U+802F.0	fFourCorner	54940
U+802F.0	fCantonese	wok6
U+802F.0	fMandarin	huo4
U+8030.0	fUTF8	耰
U+8030.0	fCNS11643	2-6B7D
U+8030.0	fBig5	F5EF
U+8030.0	fGB	3-603F
U+8030.0	fR/S	 127.15
U+8030.0	fTotalStrokes	 21
U+8030.0	fCangjie	QDMBE
U+8030.0	fFourCorner	51947
U+8030.0	fHakka	 ju2; riu2
U+8030.0	fCantonese	jau1
U+8030.0	fMandarin	you1
U+8030.0	fEnglish	[1] draw earth over newly-sown grain; cover the seed [2] a kind of hoe
U+8031.0	fUTF8	耱
U+8031.0	fCNS11643	4-692E
U+8031.0	fGB	0-7172
U+8031.0	fR/S	  127.16
U+8031.0	fTotalStrokes	 22; 21
U+8031.0	fCangjie	QDIDR
U+8031.0	fFourCorner	50960
U+8031.0	fCantonese	mo6
U+8031.0	fMandarin	mo4
U+8031.0	fEnglish	a kind of farm tool
U+8032.0	fUTF8	耲
U+8032.0	fCNS11643	4-692D
U+8032.0	fGB	3-6040
U+8032.0	fR/S	  127.16
U+8032.0	fTotalStrokes	 22; 21
U+8032.0	fCangjie	QDYWV
U+8032.0	fFourCorner	50930
U+8032.0	fCantonese	waai4
U+8032.0	fMandarin	huai2
U+8033.0	fUTF8	耳
U+8033.0	fCNS11643	1-4856
U+8033.0	fBig5	A6D5
U+8033.0	fGB	0-367A
U+8033.0	fR/S	 128.0
U+8033.0	fTotalStrokes	 6
U+8033.0	fCangjie	SJ
U+8033.0	fFourCorner	10400
U+8033.0	fHakka	  ngi3; gni3
U+8033.0	fCantonese	ji5
U+8033.0	fMandarin	er3; reng2; reng3
U+8033.0	fEnglish	[1] ear; ears [2] merely; only (a phrase-final particle) [3] a Chinese family name
U+8034.0	fUTF8	耴
U+8034.0	fCNS11643	2-2473
U+8034.0	fBig5	CB73
U+8034.0	fGB	3-6043
U+8034.0	fR/S	 128.1
U+8034.0	fTotalStrokes	 7
U+8034.0	fCangjie	SJU
U+8034.0	fFourCorner	12110
U+8034.0	fHakka	 zhap7; zap7
U+8034.0	fCantonese	ngat6; at6 (also); zip3
U+8034.0	fMandarin	zhe2; nie4
U+8034.0	fEnglish	long ears, sign of wisdom, sign of long life
U+8035.0	fUTF8	耵
U+8035.0	fCNS11643	2-284A
U+8035.0	fBig5	CDAA
U+8035.0	fGB	0-7174
U+8035.0	fR/S	 128.2
U+8035.0	fTotalStrokes	 8
U+8035.0	fCangjie	SJMN
U+8035.0	fFourCorner	11420
U+8035.0	fHakka	  din3; dang3
U+8035.0	fCantonese	ding1; ding2; ting1
U+8035.0	fMandarin	ting1; ding3; ding1
U+8035.0	fEnglish	ear-wax
U+8036.0	fUTF8	耶
U+8036.0	fCNS11643	1-5326
U+8036.0	fBig5	AD43
U+8036.0	fGB	0-522E
U+8036.0	fR/S	 128.3
U+8036.0	fTotalStrokes	 9
U+8036.0	fCangjie	SJNL
U+8036.0	fFourCorner	17127
U+8036.0	fHakka	  ja2;  ja1; ra2; ra1
U+8036.0	fCantonese	je4
U+8036.0	fMandarin	ye2; ye1; xie2
U+8036.0	fEnglish	[1] a phrase-final particle for a question [2] transliteration of English names
U+8037.0	fUTF8	耷
U+8037.0	fCNS11643	2-2C76
U+8037.0	fBig5	D055
U+8037.0	fGB	0-5E47
U+8037.0	fR/S	 128.3
U+8037.0	fTotalStrokes	 9
U+8037.0	fCangjie	KSJ
U+8037.0	fFourCorner	40401
U+8037.0	fHakka	 dap7; tap7
U+8037.0	fCantonese	daap3; dap1
U+8037.0	fMandarin	da1; zhe2
U+8037.0	fEnglish	[1] big ears [2] the given name of a famous Ching artist, Chu Ta
U+8038.0	fUTF8	耸
U+8038.0	fCNS11643	6-3B21
U+8038.0	fGB	0-4B4A
U+8038.0	fR/S	 128.4
U+8038.0	fTotalStrokes	 10; 9
U+8038.0	fCangjie	OOSJ
U+8038.0	fFourCorner	88400
U+8038.0	fHakka	 sung3; ziung3
U+8038.0	fCantonese	sung2
U+8038.0	fMandarin	song3; shuang3
U+8038.0	fEnglish	simplified form of &#x8073;
U+8039.0	fUTF8	耹
U+8039.0	fCNS11643	2-322C
U+8039.0	fBig5	D368
U+8039.0	fGB	5-5F4B
U+8039.0	fR/S	 128.4
U+8039.0	fTotalStrokes	 10
U+8039.0	fCangjie	SJOIN
U+8039.0	fFourCorner	18127
U+8039.0	fCantonese	kam4
U+8039.0	fMandarin	qin2; qian2
U+803A.0	fUTF8	耺
U+803A.0	fCNS11643	4-2F3A
U+803A.0	fGB	3-6046
U+803A.0	fR/S	  128.4
U+803A.0	fTotalStrokes	 10
U+803A.0	fCangjie	SJMMI
U+803A.0	fFourCorner	11130
U+803A.0	fCantonese	wan4
U+803A.0	fMandarin	yun2
U+803A.0	fEnglish	non-classical form of &#x8077;
U+803B.0	fUTF8	耻
U+803B.0	fCNS11643	3-3349
U+803B.0	fGB	0-335C
U+803B.0	fR/S	 128.4
U+803B.0	fTotalStrokes	 10
U+803B.0	fCangjie	SJYLM
U+803B.0	fFourCorner	11110
U+803B.0	fHakka	 ci3; chi3; cih3; chii3
U+803B.0	fCantonese	ci2
U+803B.0	fMandarin	chi3
U+803B.0	fEnglish	 shame; humiliation ashamed
U+803C.0	fUTF8	耼
U+803C.0	fCNS11643	3-3348
U+803C.0	fGB	E-393D
U+803C.0	fR/S	 128.4
U+803C.0	fTotalStrokes	 10
U+803C.0	fCangjie	SJBMM
U+803C.0	fFourCorner	17150
U+803C.0	fHakka	 dam1
U+803C.0	fCantonese	daam1
U+803C.0	fMandarin	tan1; dan1; man2; nan2
U+803C.0	fEnglish	ears without rim
U+803D.0	fUTF8	耽
U+803D.0	fCNS11643	1-5758
U+803D.0	fBig5	AFD4
U+803D.0	fGB	0-3522
U+803D.0	fR/S	 128.4
U+803D.0	fTotalStrokes	 10
U+803D.0	fCangjie	SJLBU
U+803D.0	fFourCorner	14112
U+803D.0	fHakka	  dam1; dam5
U+803D.0	fCantonese	daam1
U+803D.0	fMandarin	dan1; tan2; dan3; tan3
U+803D.0	fEnglish	[1] indulge in; be addicted to [2] delightful and enduring [3] large and drooping (ears) [4] negligent
U+803E.0	fUTF8	耾
U+803E.0	fCNS11643	2-322B
U+803E.0	fBig5	D367
U+803E.0	fGB	3-6047
U+803E.0	fR/S	 128.4
U+803E.0	fTotalStrokes	 10
U+803E.0	fCangjie	SJKI
U+803E.0	fFourCorner	14132
U+803E.0	fHakka	 fen2
U+803E.0	fCantonese	wang4
U+803E.0	fMandarin	hong2; hong1
U+803E.0	fEnglish	deafness
U+803F.0	fUTF8	耿
U+803F.0	fCNS11643	1-5759
U+803F.0	fBig5	AFD5
U+803F.0	fGB	0-3922
U+803F.0	fR/S	 128.4
U+803F.0	fTotalStrokes	 10
U+803F.0	fCangjie	SJF
U+803F.0	fFourCorner	19180
U+803F.0	fHakka	 kwen3; gen3; gwen3
U+803F.0	fCantonese	gang2; gwing2
U+803F.0	fMandarin	geng3; jing3; jiong1
U+803F.0	fEnglish	[1] bright [2] upright; corruptible [3] a Chinese family name
U+8040.0	fUTF8	聀
U+8040.0	fCNS11643	3-3347
U+8040.0	fGB	E-393E
U+8040.0	fR/S	 128.4; 128.6; 62.6
U+8040.0	fTotalStrokes	 10
U+8040.0	fCangjie	SJI
U+8040.0	fFourCorner	13150
U+8040.0	fCantonese	zik1
U+8040.0	fMandarin	zhi2; zhi4
U+8041.0	fUTF8	聁
U+8041.0	fCNS11643	3-662F
U+8041.0	fGB	E-393F
U+8041.0	fR/S	 128.4
U+8041.0	fTotalStrokes	 10
U+8041.0	fCangjie	SJCSH
U+8041.0	fFourCorner	1812
U+8041.0	fCantonese	paan3
U+8042.0	fUTF8	聂
U+8042.0	fGB	0-4474
U+8042.0	fR/S	 128.4
U+8042.0	fTotalStrokes	 10
U+8042.0	fCangjie	SJEE
U+8042.0	fHakka	 ngiap7
U+8042.0	fCantonese	nip6; lip6 (also)
U+8042.0	fMandarin	nie4; ye1; zhe4; ye4
U+8042.0	fEnglish	simplified form of &#x8076;
U+8043.0	fUTF8	聃
U+8043.0	fCNS11643	2-3847
U+8043.0	fBig5	D743
U+8043.0	fGB	0-7175
U+8043.0	fR/S	 128.5
U+8043.0	fTotalStrokes	 11
U+8043.0	fCangjie	SJGB
U+8043.0	fFourCorner	15447
U+8043.0	fHakka	 ngiam1; dam1
U+8043.0	fCantonese	daam1
U+8043.0	fMandarin	dan1
U+8043.0	fEnglish	[1] deformed ear; ears without rims [2] another name for Lao-tzu
U+8044.0	fUTF8	聄
U+8044.0	fCNS11643	3-383D
U+8044.0	fGB	5-5F4F
U+8044.0	fR/S	  128.5
U+8044.0	fTotalStrokes	 11
U+8044.0	fCangjie	SJOHH
U+8044.0	fFourCorner	18120
U+8044.0	fCantonese	can2
U+8044.0	fMandarin	zhen3
U+8045.0	fUTF8	聅
U+8045.0	fCNS11643	3-383C
U+8045.0	fGB	3-6049
U+8045.0	fR/S	  128.5
U+8045.0	fTotalStrokes	 11
U+8045.0	fCangjie	SJOK
U+8045.0	fFourCorner	18130
U+8045.0	fCantonese	cit3
U+8045.0	fMandarin	che4; tan2; ye4
U+8046.0	fUTF8	聆
U+8046.0	fCNS11643	1-5C67
U+8046.0	fBig5	B2E2
U+8046.0	fGB	0-7176
U+8046.0	fR/S	 128.5
U+8046.0	fTotalStrokes	 11
U+8046.0	fCangjie	SJOII
U+8046.0	fFourCorner	18137
U+8046.0	fHakka	  lin2; lang2
U+8046.0	fCantonese	ling4
U+8046.0	fMandarin	ling2
U+8046.0	fEnglish	 listen; hear
U+8047.0	fUTF8	聇
U+8047.0	fCNS11643	2-3846
U+8047.0	fBig5	D742
U+8047.0	fGB	E-3940
U+8047.0	fR/S	 128.5
U+8047.0	fTotalStrokes	 11
U+8047.0	fCangjie	SJMYM
U+8047.0	fFourCorner	11111
U+8047.0	fCantonese	zing1
U+8047.0	fMandarin	zheng1
U+8048.0	fUTF8	聈
U+8048.0	fCNS11643	2-3848
U+8048.0	fBig5	D744
U+8048.0	fGB	E-3941
U+8048.0	fR/S	 128.5
U+8048.0	fTotalStrokes	 11
U+8048.0	fCangjie	SJVIS
U+8048.0	fFourCorner	14420
U+8048.0	fCantonese	jau4; jau2
U+8048.0	fMandarin	you3
U+8049.0	fUTF8	聉
U+8049.0	fCNS11643	3-383B
U+8049.0	fGB	3-604A
U+8049.0	fR/S	  128.5
U+8049.0	fTotalStrokes	 11
U+8049.0	fCangjie	SJUU
U+8049.0	fFourCorner	12477
U+8049.0	fCantonese	waat6
U+8049.0	fMandarin	wa1; yi4; tui3
U+804A.0	fUTF8	聊
U+804A.0	fCNS11643	1-5C66
U+804A.0	fBig5	B2E1
U+804A.0	fGB	0-4144
U+804A.0	fR/S	 128.5
U+804A.0	fTotalStrokes	 11
U+804A.0	fCangjie	SJHHL
U+804A.0	fFourCorner	17120
U+804A.0	fHakka	  liau2
U+804A.0	fCantonese	liu4
U+804A.0	fMandarin	liao2; liu2; liao4
U+804A.0	fEnglish	[1] somehow; somewhat; slightly; a little [2] rely; depend [3] chat [4] interest [5] for the time being
U+804B.0	fUTF8	聋
U+804B.0	fGB	0-417B
U+804B.0	fR/S	 128.5
U+804B.0	fTotalStrokes	 11
U+804B.0	fCangjie	IPSJ
U+804B.0	fHakka	 lung1
U+804B.0	fCantonese	lung4; lung2 (also)
U+804B.0	fMandarin	long2
U+804B.0	fEnglish	simplified form of &#x807E;
U+804C.0	fUTF8	职
U+804C.0	fGB	0-5630
U+804C.0	fR/S	 128.5
U+804C.0	fTotalStrokes	 11
U+804C.0	fCangjie	SJRC
U+804C.0	fHakka	 zhit7
U+804C.0	fCantonese	zik1
U+804C.0	fMandarin	zhi2; te4; zhi4; yi4
U+804C.0	fEnglish	simplified form of &#x8077;
U+804D.0	fUTF8	聍
U+804D.0	fGB	0-7177
U+804D.0	fR/S	 128.5
U+804D.0	fTotalStrokes	 11
U+804D.0	fCangjie	SJJMN
U+804D.0	fHakka	 nen2; nen3
U+804D.0	fCantonese	ning4; ling4 (also)
U+804D.0	fMandarin	ning2; ning3
U+804D.0	fEnglish	simplified form of &#x8079;
U+804E.0	fUTF8	聎
U+804E.0	fCNS11643	3-3E3C
U+804E.0	fGB	3-6050
U+804E.0	fR/S	  128.6
U+804E.0	fTotalStrokes	 12
U+804E.0	fCangjie	XSJLM
U+804E.0	fFourCorner	12110
U+804E.0	fCantonese	tiu1
U+804E.0	fMandarin	tiao1; tao2; tao1
U+804F.0	fUTF8	聏
U+804F.0	fCNS11643	2-3F2B
U+804F.0	fBig5	DB46
U+804F.0	fGB	3-604D
U+804F.0	fR/S	 128.6
U+804F.0	fTotalStrokes	 12
U+804F.0	fCangjie	SJMBL
U+804F.0	fFourCorner	11427
U+804F.0	fCantonese	ji4
U+804F.0	fMandarin	er2; nuu4; er4
U+8050.0	fUTF8	聐
U+8050.0	fCNS11643	2-3F2C
U+8050.0	fBig5	DB47
U+8050.0	fGB	3-604B
U+8050.0	fR/S	 128.6
U+8050.0	fTotalStrokes	 12
U+8050.0	fCangjie	SJGR
U+8050.0	fFourCorner	14161
U+8050.0	fCantonese	ngaat6; aat6 (also)
U+8050.0	fMandarin	ya4
U+8051.0	fUTF8	聑
U+8051.0	fCNS11643	2-3F2A
U+8051.0	fBig5	DB45
U+8051.0	fGB	3-604C
U+8051.0	fR/S	 128.6
U+8051.0	fTotalStrokes	 12
U+8051.0	fCangjie	SJSJ
U+8051.0	fFourCorner	11440
U+8051.0	fCantonese	dip3
U+8051.0	fMandarin	die2; zhe2; zhai1
U+8052.0	fUTF8	聒
U+8052.0	fCNS11643	1-6147
U+8052.0	fBig5	B5C1
U+8052.0	fGB	0-7178
U+8052.0	fR/S	 128.6
U+8052.0	fTotalStrokes	 12
U+8052.0	fCangjie	SJHJR
U+8052.0	fFourCorner	12164
U+8052.0	fHakka	 kwat7; gat7; gwat7
U+8052.0	fCantonese	kut3; kwut3 (also)
U+8052.0	fMandarin	gua1; guo1
U+8052.0	fEnglish	clamorous; uproarious
U+8053.0	fUTF8	聓
U+8053.0	fCNS11643	3-6729
U+8053.0	fGB	E-3942
U+8053.0	fR/S	 128.6
U+8053.0	fTotalStrokes	 12
U+8053.0	fCangjie	MNSJ
U+8053.0	fCantonese	sai3
U+8054.0	fUTF8	联
U+8054.0	fGB	0-412A
U+8054.0	fR/S	 128.6
U+8054.0	fTotalStrokes	 12
U+8054.0	fCangjie	SJTK
U+8054.0	fHakka	 lien2
U+8054.0	fCantonese	lyun4; lyun2 (also)
U+8054.0	fMandarin	lian2
U+8054.0	fEnglish	simplified form of &#x806F;
U+8055.0	fUTF8	聕
U+8055.0	fCNS11643	4-3F7C
U+8055.0	fGB	3-6051
U+8055.0	fR/S	  128.7
U+8055.0	fTotalStrokes	 13
U+8055.0	fCangjie	SJHGR
U+8055.0	fFourCorner	14160
U+8055.0	fCantonese	hou6
U+8055.0	fMandarin	hao4
U+8056.0	fUTF8	聖
U+8056.0	fCNS11643	1-657B
U+8056.0	fBig5	B874
U+8056.0	fGB	1-4A25
U+8056.0	fR/S	 128.7
U+8056.0	fTotalStrokes	 13
U+8056.0	fCangjie	SRHG
U+8056.0	fFourCorner	16104
U+8056.0	fHakka	  sin5; shin5
U+8056.0	fCantonese	sing3
U+8056.0	fMandarin	sheng4
U+8056.0	fEnglish	[1] holy; sacred [2] sage
U+8057.0	fUTF8	聗
U+8057.0	fCNS11643	3-4373
U+8057.0	fGB	3-604E
U+8057.0	fR/S	  128.7
U+8057.0	fTotalStrokes	 13
U+8057.0	fCangjie	SJKOO
U+8057.0	fFourCorner	14130
U+8057.0	fCantonese	lip6
U+8057.0	fMandarin	lie4
U+8058.0	fUTF8	聘
U+8058.0	fCNS11643	1-657C
U+8058.0	fBig5	B875
U+8058.0	fGB	0-4638
U+8058.0	fR/S	 128.7
U+8058.0	fTotalStrokes	 13
U+8058.0	fCangjie	SJLWS
U+8058.0	fFourCorner	15127
U+8058.0	fHakka	 pin5; pin3
U+8058.0	fCantonese	ping3
U+8058.0	fMandarin	pin4
U+8058.0	fEnglish	[1] invite for service; engage; employ [2] be betrothed; be engaged [3] pay respect by sending an envoy [4] ask; inquire
U+8058.1	fUTF8	聘
U+8058.1	fCNS11643	1-657C
U+8058.1	fBig5	B875
U+8058.1	fGB	0-4638
U+8058.1	fR/S	 128.7
U+8058.1	fTotalStrokes	 13
U+8058.1	fCangjie	SJLWS
U+8058.1	fFourCorner	15127
U+8058.1	fMandarin	ping4
U+8058.1	fEnglish	 give (a daughter) in marriage; marry
U+8059.0	fUTF8	聙
U+8059.0	fCNS11643	4-4633
U+8059.0	fGB	3-6052
U+8059.0	fR/S	  128.8
U+8059.0	fTotalStrokes	 14
U+8059.0	fCangjie	SJQMB
U+8059.0	fFourCorner	15120
U+8059.0	fCantonese	zing1
U+8059.0	fMandarin	jing1
U+805A.0	fUTF8	聚
U+805A.0	fCNS11643	1-6A4D
U+805A.0	fBig5	BB45
U+805A.0	fGB	0-3E5B
U+805A.0	fR/S	 128.8
U+805A.0	fTotalStrokes	 14
U+805A.0	fCangjie	SEOHO
U+805A.0	fFourCorner	17232
U+805A.0	fHakka	  ci5; ci6; ci1
U+805A.0	fCantonese	zeoi6
U+805A.0	fMandarin	ju4; zou1
U+805A.0	fEnglish	 come together; meet together; put together; gather; assemble; collect
U+805B.0	fUTF8	聛
U+805B.0	fCNS11643	3-4923
U+805B.0	fGB	5-5F53
U+805B.0	fR/S	 128.8
U+805B.0	fTotalStrokes	 14
U+805B.0	fCangjie	SJHWJ
U+805B.0	fFourCorner	16140
U+805B.0	fCantonese	bei2
U+805B.0	fMandarin	bi4
U+805C.0	fUTF8	聜
U+805C.0	fCNS11643	2-4D2A
U+805C.0	fBig5	E3A3
U+805C.0	fGB	E-3943
U+805C.0	fR/S	 128.8
U+805C.0	fTotalStrokes	 14
U+805C.0	fCangjie	SJJCM
U+805C.0	fFourCorner	13410
U+805C.0	fCantonese	dai2; zat1
U+805C.0	fMandarin	di3
U+805D.0	fUTF8	聝
U+805D.0	fCNS11643	2-4D29
U+805D.0	fBig5	E3A2
U+805D.0	fGB	5-5F51
U+805D.0	fR/S	 128.8
U+805D.0	fTotalStrokes	 14
U+805D.0	fCangjie	SJIRM
U+805D.0	fFourCorner	13450
U+805D.0	fHakka	 kwet7
U+805D.0	fCantonese	gwik1; gwok3; gok3 (also)
U+805D.0	fMandarin	guo2
U+805D.0	fEnglish	 cut the left ears of the slain
U+805E.0	fUTF8	聞
U+805E.0	fCNS11643	1-6A4C
U+805E.0	fBig5	BB44
U+805E.0	fGB	1-4E45
U+805E.0	fR/S	 128.8
U+805E.0	fTotalStrokes	 14
U+805E.0	fCangjie	ANSJ
U+805E.0	fFourCorner	77401
U+805E.0	fHakka	 wun2;  mun2; wun5; wun3; vun2
U+805E.0	fCantonese	man4; man2 (also); man6
U+805E.0	fMandarin	wen2
U+805E.0	fEnglish	[1] hear; have heard [2] smell [3] make known [4] learn; learning; understand [5] convey; forward; transmit (message, etc.) [6] a Chinese family name
U+805E.1	fUTF8	聞
U+805E.1	fCNS11643	1-6A4C
U+805E.1	fBig5	BB44
U+805E.1	fGB	1-4E45
U+805E.1	fR/S	 128.8
U+805E.1	fTotalStrokes	 14
U+805E.1	fCangjie	ANSJ
U+805E.1	fFourCorner	77401
U+805E.1	fMandarin	wen4
U+805E.1	fEnglish	reputation
U+805F.0	fUTF8	聟
U+805F.0	fCNS11643	3-4926
U+805F.0	fGB	E-3944
U+805F.0	fR/S	 128.8
U+805F.0	fTotalStrokes	 14
U+805F.0	fCangjie	ORSJ
U+805F.0	fFourCorner	86400
U+805F.0	fCantonese	sai3
U+805F.0	fMandarin	xu4
U+805F.0	fEnglish	 son-in-law husband
U+8060.0	fUTF8	聠
U+8060.0	fCNS11643	3-3E3B
U+8060.0	fGB	E-3945
U+8060.0	fR/S	  128.6
U+8060.0	fTotalStrokes	 12
U+8060.0	fCangjie	SJTT
U+8060.0	fFourCorner	18140
U+8060.0	fCantonese	ping1
U+8060.0	fMandarin	ping2; ping1
U+8061.0	fUTF8	聡
U+8061.0	fCNS11643	3-4925
U+8061.0	fGB	E-3946
U+8061.0	fR/S	 128.8
U+8061.0	fTotalStrokes	 14
U+8061.0	fCangjie	SJCIP
U+8061.0	fFourCorner	18430
U+8061.0	fCantonese	cung1
U+8061.0	fMandarin	cong1
U+8061.0	fEnglish	intelligent; clever; bright
U+8062.0	fUTF8	聢
U+8062.0	fGB	E-3947
U+8062.0	fR/S	 128.8
U+8062.0	fTotalStrokes	 14
U+8062.0	fCangjie	SJJMO
U+8062.0	fFourCorner	1318
U+8062.0	fCantonese	ding6
U+8062.0	fEnglish	certainly, definitely
U+8063.0	fUTF8	聣
U+8063.0	fCNS11643	3-6630
U+8063.0	fGB	E-3948
U+8063.0	fR/S	 128.8
U+8063.0	fTotalStrokes	 14
U+8063.0	fCangjie	SJHXU
U+8063.0	fFourCorner	1711
U+8063.0	fCantonese	ngai6; ai6 (also)
U+8064.0	fUTF8	聤
U+8064.0	fCNS11643	2-5325
U+8064.0	fBig5	E6FB
U+8064.0	fGB	3-6057
U+8064.0	fR/S	 128.9
U+8064.0	fTotalStrokes	 15
U+8064.0	fCangjie	SJYRN
U+8064.0	fFourCorner	10121
U+8064.0	fHakka	 tin2
U+8064.0	fCantonese	ting4
U+8064.0	fMandarin	ting2
U+8064.0	fEnglish	a discharge from the ear
U+8065.0	fUTF8	聥
U+8065.0	fCNS11643	3-4E34
U+8065.0	fGB	3-6056
U+8065.0	fR/S	  128.9
U+8065.0	fTotalStrokes	 15
U+8065.0	fCangjie	SJHLB
U+8065.0	fFourCorner	12120
U+8065.0	fCantonese	geoi2
U+8065.0	fMandarin	yu3; ju3; yu4; xu1
U+8066.0	fUTF8	聦
U+8066.0	fCNS11643	3-4E36
U+8066.0	fGB	E-3949
U+8066.0	fR/S	 128.9
U+8066.0	fTotalStrokes	 15
U+8066.0	fCangjie	SJPKP
U+8066.0	fFourCorner	17130
U+8066.0	fHakka	  cung1
U+8066.0	fCantonese	cung1
U+8066.0	fMandarin	cong1
U+8066.0	fEnglish	 the hearing; listen astute clever quick of apprehension
U+8067.0	fUTF8	聧
U+8067.0	fCNS11643	2-5326
U+8067.0	fBig5	E6FC
U+8067.0	fGB	5-5F59
U+8067.0	fR/S	 128.9
U+8067.0	fTotalStrokes	 15
U+8067.0	fCangjie	SJNOK
U+8067.0	fFourCorner	12134
U+8067.0	fHakka	 kui2; kwi2; k'wui2
U+8067.0	fCantonese	kwai1; kwai4
U+8067.0	fMandarin	kui2
U+8067.0	fEnglish	deaf
U+8068.0	fUTF8	聨
U+8068.0	fCNS11643	3-4E37
U+8068.0	fGB	E-394A
U+8068.0	fR/S	 128.9
U+8068.0	fTotalStrokes	 15
U+8068.0	fCangjie	SJVIT
U+8068.0	fFourCorner	12443
U+8068.0	fCantonese	lyun4
U+8068.0	fMandarin	lian2
U+8068.0	fEnglish	 connect; join associate; ally
U+8069.0	fUTF8	聩
U+8069.0	fGB	0-7179
U+8069.0	fR/S	 128.9
U+8069.0	fTotalStrokes	 15
U+8069.0	fCangjie	SJLMO
U+8069.0	fHakka	 k'wui5
U+8069.0	fCantonese	kui2; kwui2 (also); kui3 (also); kwui3 (also)
U+8069.0	fMandarin	kui4
U+8069.0	fEnglish	simplified form of &#x8075;
U+806A.0	fUTF8	聪
U+806A.0	fCNS11643	3-4E35
U+806A.0	fGB	0-344F
U+806A.0	fR/S	 128.9
U+806A.0	fTotalStrokes	 15
U+806A.0	fCangjie	SJCRP
U+806A.0	fFourCorner	18430
U+806A.0	fHakka	 cung1
U+806A.0	fCantonese	cung1
U+806A.0	fMandarin	cong1
U+806A.0	fEnglish	simplified form of &#x8070;
U+806B.0	fUTF8	聫
U+806B.0	fCNS11643	3-4E38
U+806B.0	fGB	E-394B
U+806B.0	fR/S	 128.9
U+806B.0	fTotalStrokes	 15
U+806B.0	fCangjie	SJVIK
U+806B.0	fFourCorner	12434
U+806B.0	fHakka	 lien1
U+806B.0	fCantonese	lyun4
U+806B.0	fMandarin	lian2
U+806B.0	fEnglish	 connect; join associate; ally
U+806C.0	fUTF8	聬
U+806C.0	fCNS11643	2-5953
U+806C.0	fBig5	EAE7
U+806C.0	fGB	5-5F5A
U+806C.0	fR/S	 128.10
U+806C.0	fTotalStrokes	 16
U+806C.0	fCangjie	SJCIM
U+806C.0	fFourCorner	18127
U+806C.0	fCantonese	jung2
U+806C.0	fMandarin	weng3
U+806D.0	fUTF8	聭
U+806D.0	fCNS11643	4-525C
U+806D.0	fGB	5-5F58
U+806D.0	fR/S	  128.10
U+806D.0	fTotalStrokes	 16
U+806D.0	fCangjie	SJHUI
U+806D.0	fFourCorner	16110
U+806D.0	fCantonese	gwai3
U+806D.0	fMandarin	kui4
U+806E.0	fUTF8	聮
U+806E.0	fCNS11643	3-5243
U+806E.0	fGB	E-394C
U+806E.0	fR/S	 128.12
U+806E.0	fTotalStrokes	 16; 18
U+806E.0	fCangjie	SJVIF
U+806E.0	fFourCorner	12433
U+806E.0	fHakka	 lien1
U+806E.0	fCantonese	lyun4
U+806E.0	fMandarin	lian2
U+806E.0	fEnglish	 connect; join associate; ally
U+806F.0	fUTF8	聯
U+806F.0	fCNS11643	1-747A
U+806F.0	fBig5	C170
U+806F.0	fGB	1-412A
U+806F.0	fR/S	 128.11
U+806F.0	fTotalStrokes	 17
U+806F.0	fCangjie	SJVIT
U+806F.0	fFourCorner	12172
U+806F.0	fHakka	 lien2; len2; lian3
U+806F.0	fCantonese	lyun4; lyun2 (also)
U+806F.0	fMandarin	lian2
U+806F.0	fEnglish	[1] unite; ally; connect; join; make an alliance with; associate [2] allied (forces, etc.); joint (effort, etc.); mutual (guaranties, etc.)
U+8070.0	fUTF8	聰
U+8070.0	fCNS11643	1-7479
U+8070.0	fBig5	C16F
U+8070.0	fGB	1-344F
U+8070.0	fR/S	 128.11
U+8070.0	fTotalStrokes	 17
U+8070.0	fCangjie	SJHWP
U+8070.0	fFourCorner	16130
U+8070.0	fHakka	  cung1
U+8070.0	fCantonese	cung1
U+8070.0	fMandarin	cong1
U+8070.0	fEnglish	[1] intelligent; clever; bright; astute; quick of apprehension [2] with a good faculty of hearing
U+8071.0	fUTF8	聱
U+8071.0	fCNS11643	1-7477
U+8071.0	fBig5	C16D
U+8071.0	fGB	0-717A
U+8071.0	fR/S	 128.11
U+8071.0	fTotalStrokes	 17
U+8071.0	fCangjie	GKSJ
U+8071.0	fFourCorner	48401
U+8071.0	fHakka	 ngau2
U+8071.0	fCantonese	ngou4; ou4 (also)
U+8071.0	fMandarin	ao2; you2; ao4
U+8071.0	fEnglish	[1] hard to read or understand; too many big words and a tortuous style [2] perverse; unruly [3] lying and winding (snakes, old trees, etc.) [4] refusing to hear
U+8072.0	fUTF8	聲
U+8072.0	fCNS11643	1-7478
U+8072.0	fBig5	C16E
U+8072.0	fGB	1-4979
U+8072.0	fR/S	 128.11
U+8072.0	fTotalStrokes	 17
U+8072.0	fCangjie	GESJ
U+8072.0	fFourCorner	47401
U+8072.0	fHakka	  sang1;  s'ang1; shang1
U+8072.0	fCantonese	seng1 (also); sing1
U+8072.0	fMandarin	sheng1
U+8072.0	fEnglish	[1] sound; voice; noise; tone [2] ; music [3] language; tongue [4] reputation; fame [5] make known
U+8072.1	fUTF8	聲
U+8072.1	fCNS11643	1-7478
U+8072.1	fBig5	C16E
U+8072.1	fGB	1-4979
U+8072.1	fR/S	 128.11
U+8072.1	fTotalStrokes	 17
U+8072.1	fCangjie	GESJ
U+8072.1	fFourCorner	47401
U+8072.1	fMandarin	sheng5
U+8073.0	fUTF8	聳
U+8073.0	fCNS11643	1-747B
U+8073.0	fBig5	C171
U+8073.0	fGB	1-4B4A
U+8073.0	fR/S	 128.11
U+8073.0	fTotalStrokes	 17
U+8073.0	fCangjie	HOSJ
U+8073.0	fFourCorner	28401
U+8073.0	fHakka	  sung3;  ziung3; chung5; cung5
U+8073.0	fCantonese	sung2
U+8073.0	fMandarin	song3; shuang3
U+8073.0	fEnglish	[1] urge on; egg on [2] rise up; stretch up erect or at full length [3] be born deaf [4] alarm; alert; warn; be sensational
U+8074.0	fUTF8	聴
U+8074.0	fCNS11643	3-5622
U+8074.0	fGB	E-394D
U+8074.0	fR/S	 128.12; 128.11
U+8074.0	fTotalStrokes	 17; 18
U+8074.0	fCangjie	SJJWP
U+8074.0	fFourCorner	14430
U+8074.0	fHakka	 tang5; tin5; tang1; tin1; ten1
U+8074.0	fCantonese	ting1; teng1; ting3
U+8074.0	fMandarin	ting1
U+8074.0	fEnglish	same as &#x807D;
U+8075.0	fUTF8	聵
U+8075.0	fCNS11643	2-6268
U+8075.0	fBig5	F07C
U+8075.0	fGB	1-7179
U+8075.0	fR/S	 128.12
U+8075.0	fTotalStrokes	 18
U+8075.0	fCangjie	SJLMC
U+8075.0	fFourCorner	65186
U+8075.0	fHakka	 kui3; kui6; kui5; k'wui5
U+8075.0	fCantonese	kui2; kui3 (also); kwui2 (also); kwui3 (also); ngoi6; wui6
U+8075.0	fMandarin	kui4
U+8075.0	fEnglish	[1] deaf; hard of hearing [2] stupid and unreasonable
U+8076.0	fUTF8	聶
U+8076.0	fCNS11643	1-772B
U+8076.0	fBig5	C2BF
U+8076.0	fGB	1-4474
U+8076.0	fR/S	 128.12
U+8076.0	fTotalStrokes	 18
U+8076.0	fCangjie	SJSJJ
U+8076.0	fFourCorner	10441
U+8076.0	fHakka	  ngiap7; gniap7
U+8076.0	fCantonese	nip6; lip6 (also); sip3
U+8076.0	fMandarin	nie4; ye4; ye1; zhe4; she4; zhe2
U+8076.0	fEnglish	[1] whisper into another's ear [2] a Chinese family name
U+8077.0	fUTF8	職
U+8077.0	fCNS11643	1-772A
U+8077.0	fBig5	C2BE
U+8077.0	fGB	1-5630
U+8077.0	fR/S	 128.12
U+8077.0	fTotalStrokes	 18
U+8077.0	fCangjie	SJYIA
U+8077.0	fFourCorner	13150
U+8077.0	fHakka	  zhit7;  zit7; tsiit7
U+8077.0	fCantonese	zik1
U+8077.0	fMandarin	zhi2; te4; zhi4; yi4
U+8077.0	fEnglish	[1] duty; profession; vocation; career [2] office; official duties [3] post; position [4] govern; direct; manage [5] used in place of "I" in documents to a superior [6] only; particularly
U+8078.0	fUTF8	聸
U+8078.0	fCNS11643	2-664A
U+8078.0	fBig5	F2C0
U+8078.0	fGB	5-5F60
U+8078.0	fR/S	 128.13
U+8078.0	fTotalStrokes	 19
U+8078.0	fCangjie	SJNCR
U+8078.0	fFourCorner	17461
U+8078.0	fCantonese	daam1
U+8078.0	fMandarin	dan1
U+8079.0	fUTF8	聹
U+8079.0	fCNS11643	2-695F
U+8079.0	fBig5	F4B2
U+8079.0	fGB	1-7177
U+8079.0	fR/S	 128.14
U+8079.0	fTotalStrokes	 20
U+8079.0	fCangjie	SJJPN
U+8079.0	fFourCorner	13121
U+8079.0	fHakka	 nen2;  nen3
U+8079.0	fCantonese	ning4; ling4 (also)
U+8079.0	fMandarin	ning2; ning3
U+8079.0	fEnglish	earwax
U+807A.0	fUTF8	聺
U+807A.0	fCNS11643	3-5C70
U+807A.0	fGB	E-394E
U+807A.0	fR/S	  128.14
U+807A.0	fTotalStrokes	 20
U+807A.0	fCangjie	SJJBF
U+807A.0	fFourCorner	13190
U+807A.0	fCantonese	cit3
U+807A.0	fMandarin	qie1
U+807B.0	fUTF8	聻
U+807B.0	fCNS11643	3-5C71
U+807B.0	fGB	3-605B
U+807B.0	fR/S	  128.14
U+807B.0	fTotalStrokes	 20
U+807B.0	fCangjie	ELSJ
U+807B.0	fFourCorner	32400
U+807B.0	fHakka	 ni3;  zit7; ciam5; ciam6
U+807B.0	fCantonese	zim6
U+807B.0	fMandarin	ji1; ni3; ji2; jian4
U+807B.0	fEnglish	death of a disembodied spirit; depraved
U+807C.0	fUTF8	聼
U+807C.0	fCNS11643	3-5C72
U+807C.0	fGB	E-394F
U+807C.0	fR/S	 128.14
U+807C.0	fTotalStrokes	 20
U+807C.0	fCangjie	MJJWP
U+807C.0	fFourCorner	14430
U+807C.0	fHakka	 tang5; tang1; tin5; tin1; ten1
U+807C.0	fCantonese	ting1; teng1 (also); ting3
U+807C.0	fMandarin	ting1; ting4
U+807C.0	fEnglish	same as &#x807D;
U+807D.0	fUTF8	聽
U+807D.0	fCNS11643	1-7B6F
U+807D.0	fBig5	C5A5
U+807D.0	fGB	1-4C7D
U+807D.0	fR/S	 128.16
U+807D.0	fTotalStrokes	 22
U+807D.0	fCangjie	SGJWP
U+807D.0	fFourCorner	14131
U+807D.0	fHakka	  tang1;  tang5; tin5; ten1; tin1
U+807D.0	fCantonese	ting1; teng1 (also); ting3
U+807D.0	fMandarin	ting1
U+807D.0	fEnglish	[1] hear; listen [2] follow; obey [3] wait for [4] hall
U+807D.1	fUTF8	聽
U+807D.1	fCNS11643	1-7B6F
U+807D.1	fBig5	C5A5
U+807D.1	fGB	1-4C7D
U+807D.1	fR/S	 128.16
U+807D.1	fTotalStrokes	 22
U+807D.1	fCangjie	SGJWP
U+807D.1	fFourCorner	14131
U+807D.1	fMandarin	ting4
U+807D.1	fEnglish	[1] let [2] comply with; submit to [3] manage; govern; rule [4] judge and decide
U+807D.2	fUTF8	聽
U+807D.2	fCNS11643	1-7B6F
U+807D.2	fBig5	C5A5
U+807D.2	fGB	1-4C7D
U+807D.2	fR/S	 128.16
U+807D.2	fTotalStrokes	 22
U+807D.2	fCangjie	SGJWP
U+807D.2	fFourCorner	14131
U+807D.2	fMandarin	ting5
U+807E.0	fUTF8	聾
U+807E.0	fCNS11643	1-7B6E
U+807E.0	fBig5	C5A4
U+807E.0	fGB	1-417B
U+807E.0	fR/S	 128.16
U+807E.0	fTotalStrokes	 22
U+807E.0	fCangjie	YPSJ
U+807E.0	fFourCorner	01401
U+807E.0	fHakka	  lung1;  lung2
U+807E.0	fCantonese	lung4; lung2 (also)
U+807E.0	fMandarin	long2
U+807E.0	fEnglish	[1] deaf; hard of hearing [2] deaf; stupid and ignorant
U+807F.0	fUTF8	聿
U+807F.0	fCNS11643	1-4857
U+807F.0	fBig5	A6D6
U+807F.0	fGB	0-6D32
U+807F.0	fR/S	 129.0
U+807F.0	fTotalStrokes	 6
U+807F.0	fCangjie	LQ
U+807F.0	fFourCorner	50007
U+807F.0	fHakka	  lut8
U+807F.0	fCantonese	wat6; leot6 (also); jyut6
U+807F.0	fMandarin	yu4
U+807F.0	fEnglish	[1] writing brush; pencil; pencil; writing instrument [2] a particle used in introducing a phrase or sentence [3] agile and quick; nimble
U+8080.0	fUTF8	肀
U+8080.0	fGB	0-6D31
U+8080.0	fR/S	 129.0
U+8080.0	fTotalStrokes	 4; 6
U+8080.0	fCangjie	XSJ
U+8080.0	fMandarin	yu4
U+8080.0	fEnglish	 pen Kangxi radical number 129
U+8081.0	fUTF8	肁
U+8081.0	fCNS11643	6-3B24
U+8081.0	fGB	5-5266
U+8081.0	fR/S	  129.4
U+8081.0	fTotalStrokes	 10; 9
U+8081.0	fCangjie	HSLQ
U+8081.0	fFourCorner	30250
U+8081.0	fHakka	 zau1; shau5
U+8081.0	fCantonese	ziu6
U+8081.0	fMandarin	zhao4; dao4
U+8082.0	fUTF8	肂
U+8082.0	fCNS11643	2-2F7D
U+8082.0	fBig5	D1FB
U+8082.0	fGB	3-4477
U+8082.0	fR/S	 129.4
U+8082.0	fTotalStrokes	 10
U+8082.0	fCangjie	MNLQ
U+8082.0	fFourCorner	15207
U+8082.0	fHakka	 si5; i5
U+8082.0	fCantonese	si3; sei3
U+8082.0	fMandarin	si4; yi4
U+8082.0	fEnglish	 dig a grave
U+8083.0	fUTF8	肃
U+8083.0	fGB	0-4B60
U+8083.0	fR/S	 129.4
U+8083.0	fTotalStrokes	 10
U+8083.0	fCangjie	XLX
U+8083.0	fHakka	 suk7; siuk7
U+8083.0	fCantonese	suk1
U+8083.0	fMandarin	su4; xiao4; suo1
U+8083.0	fEnglish	simplified form of &#x8085;
U+8084.0	fUTF8	肄
U+8084.0	fCNS11643	1-657E
U+8084.0	fBig5	B877
U+8084.0	fGB	0-525E
U+8084.0	fR/S	 129.7
U+8084.0	fTotalStrokes	 13
U+8084.0	fCangjie	PKLQ
U+8084.0	fFourCorner	25407
U+8084.0	fHakka	 ji5; ji3; ri5; i5
U+8084.0	fCantonese	ji6; si3 (also)
U+8084.0	fMandarin	yi4; si4
U+8084.0	fEnglish	[1] learn; practice; study [2] toil; work hard [3] remnants; leftovers [4] fresh twigs
U+8085.0	fUTF8	肅
U+8085.0	fCNS11643	1-6148
U+8085.0	fBig5	B5C2
U+8085.0	fGB	1-4B60
U+8085.0	fR/S	 129.7
U+8085.0	fTotalStrokes	 13
U+8085.0	fCangjie	LX
U+8085.0	fFourCorner	50227
U+8085.0	fHakka	  suk7;  siuk7; sok7
U+8085.0	fCantonese	suk1
U+8085.0	fMandarin	su4; xiao4; suo1
U+8085.0	fEnglish	[1] pay respects; reverently; respectful; reverential; salute [2] solemn; serious; majestic; awe-inspiring [3] usher in [4] neat and quiet [5] withdraw; shrink (as in cold weather) [6] a Chinese family name
U+8086.0	fUTF8	肆
U+8086.0	fCNS11643	1-657D
U+8086.0	fBig5	B876
U+8086.0	fGB	0-4B41
U+8086.0	fR/S	 129.7
U+8086.0	fTotalStrokes	 13
U+8086.0	fCangjie	SILQ
U+8086.0	fFourCorner	75707
U+8086.0	fHakka	  si5;  sii5; si3; ji5
U+8086.0	fCantonese	si3; sei3 (also)
U+8086.0	fMandarin	si4; ti4; xi4; yi4
U+8086.0	fEnglish	[1] indulge; let loose; behave without restraint [2] marketplace; shop; place to display goods [3] exhibit; display [4] execute a criminal and expose his corpse in the market [5] extend; expand [6] assault; attack suddenly [7] use to the utmost; exhaust [8] numeral four used in accountancy
U+8087.0	fUTF8	肇
U+8087.0	fCNS11643	1-6A4E
U+8087.0	fBig5	BB46
U+8087.0	fGB	0-5558
U+8087.0	fR/S	 129.8
U+8087.0	fTotalStrokes	 14
U+8087.0	fCangjie	HKLQ
U+8087.0	fFourCorner	38507
U+8087.0	fHakka	 shau5; sau5; seu5; shau6; cau5
U+8087.0	fCantonese	siu6
U+8087.0	fMandarin	zhao4; dao4
U+8087.0	fEnglish	[1] begin; start; commence [2] found; device [3] incur (misfortune, etc.) [4] adjust; make right
U+8088.0	fUTF8	肈
U+8088.0	fCNS11643	6-6060
U+8088.0	fGB	5-6736
U+8088.0	fR/S	 129.8
U+8088.0	fTotalStrokes	 14
U+8088.0	fCangjie	HILQ
U+8088.0	fFourCorner	33500
U+8088.0	fHakka	 shau5; seu5; sau5
U+8088.0	fCantonese	siu6
U+8088.0	fMandarin	zhao4; dao4
U+8088.0	fEnglish	 commence found devise at first; originate; strike
U+8089.0	fUTF8	肉
U+8089.0	fCNS11643	1-4858
U+8089.0	fBig5	A6D7
U+8089.0	fGB	0-4862
U+8089.0	fR/S	 130.0
U+8089.0	fTotalStrokes	 6
U+8089.0	fCangjie	OBO
U+8089.0	fFourCorner	40227
U+8089.0	fHakka	 ngiuk7; gniuk7; niuk7
U+8089.0	fCantonese	juk6; juk2 (also)
U+8089.0	fMandarin	rou4; ru4; rou2
U+8089.0	fEnglish	[1] flesh; meat [2] physical; carnal [3] meat of animals; meat or pulp of fruits [4] flesh and blood; dearest (as owns children) [5] slow-motion [6] Kangxi radical 130
U+808A.0	fUTF8	肊
U+808A.0	fCNS11643	2-2168
U+808A.0	fBig5	C9A9
U+808A.0	fGB	5-4E70
U+808A.0	fR/S	 130.1
U+808A.0	fTotalStrokes	 5
U+808A.0	fCangjie	BN
U+808A.0	fFourCorner	77217
U+808A.0	fCantonese	jik1; jik6
U+808A.0	fMandarin	yi4
U+808A.0	fEnglish	[1] breastbone [2] same as $
U+808B.0	fUTF8	肋
U+808B.0	fCNS11643	1-4859
U+808B.0	fBig5	A6D8
U+808B.0	fGB	0-405F
U+808B.0	fR/S	 130.2
U+808B.0	fTotalStrokes	 6
U+808B.0	fCangjie	BKS
U+808B.0	fFourCorner	74227
U+808B.0	fHakka	 let8; lak8; lit8
U+808B.0	fCantonese	lak6; laak6 (also); laak1
U+808B.0	fMandarin	lei4; jin1; le4; le1
U+808B.0	fEnglish	the ribs; the sides
U+808C.0	fUTF8	肌
U+808C.0	fCNS11643	1-485A
U+808C.0	fBig5	A6D9
U+808C.0	fGB	0-3C21
U+808C.0	fR/S	 130.2
U+808C.0	fTotalStrokes	 6
U+808C.0	fCangjie	BHN
U+808C.0	fFourCorner	77210
U+808C.0	fHakka	  gi1
U+808C.0	fCantonese	gei1
U+808C.0	fMandarin	ji1; ji4
U+808C.0	fEnglish	[1] muscle; tissue; meat on bones; flesh [2] the skin
U+808D.0	fUTF8	肍
U+808D.0	fCNS11643	3-2435
U+808D.0	fGB	3-4C3C
U+808D.0	fR/S	  130.2
U+808D.0	fTotalStrokes	 6
U+808D.0	fCangjie	XBKN
U+808D.0	fFourCorner	74210
U+808D.0	fCantonese	gau6
U+808D.0	fMandarin	qiu2
U+808E.0	fUTF8	肎
U+808E.0	fCNS11643	3-2436
U+808E.0	fGB	E-3950
U+808E.0	fR/S	  130.2
U+808E.0	fTotalStrokes	 6
U+808E.0	fCangjie	BB
U+808E.0	fFourCorner	37220
U+808E.0	fCantonese	hang2
U+808E.0	fMandarin	ken3
U+808F.0	fUTF8	肏
U+808F.0	fCNS11643	2-284B
U+808F.0	fBig5	CDAB
U+808F.0	fGB	3-305B
U+808F.0	fR/S	 130.2
U+808F.0	fTotalStrokes	 8
U+808F.0	fCangjie	OOBO
U+808F.0	fFourCorner	80227
U+808F.0	fCantonese	caau3; cou3
U+808F.0	fMandarin	cao4
U+808F.0	fEnglish	copulate
U+8090.0	fUTF8	肐
U+8090.0	fCNS11643	2-2476
U+8090.0	fBig5	CB76
U+8090.0	fGB	E-3951
U+8090.0	fR/S	 130.3
U+8090.0	fTotalStrokes	 7
U+8090.0	fCangjie	BON
U+8090.0	fFourCorner	78217
U+8090.0	fHakka	 jit7; rit7
U+8090.0	fCantonese	gaak3 (also); gat1 (also); gok3
U+8090.0	fMandarin	ge1; qi4; yi4
U+8090.0	fEnglish	the arm (from the armpit to the wrist)
U+8091.0	fUTF8	肑
U+8091.0	fCNS11643	3-2724
U+8091.0	fGB	3-4C40
U+8091.0	fR/S	  130.3
U+8091.0	fTotalStrokes	 7
U+8091.0	fCangjie	BPI
U+8091.0	fFourCorner	77220
U+8091.0	fCantonese	bok3
U+8091.0	fMandarin	di4; bo1; bi4
U+8092.0	fUTF8	肒
U+8092.0	fCNS11643	2-2477
U+8092.0	fBig5	CB77
U+8092.0	fGB	3-4C3F
U+8092.0	fR/S	 130.3
U+8092.0	fTotalStrokes	 7
U+8092.0	fCangjie	BKNI
U+8092.0	fFourCorner	75217
U+8092.0	fCantonese	wun6
U+8092.0	fMandarin	huan4; huan2
U+8093.0	fUTF8	肓
U+8093.0	fCNS11643	1-4B3A
U+8093.0	fBig5	A877
U+8093.0	fGB	0-6B41
U+8093.0	fR/S	 130.3
U+8093.0	fTotalStrokes	 7
U+8093.0	fCangjie	YVB
U+8093.0	fFourCorner	00227
U+8093.0	fHakka	 fong1
U+8093.0	fCantonese	fong1
U+8093.0	fMandarin	huang1
U+8093.0	fEnglish	the vitals; the region between the heart and the diaphragm
U+8094.0	fUTF8	肔
U+8094.0	fCNS11643	3-2726
U+8094.0	fGB	5-4E71
U+8094.0	fR/S	  130.3
U+8094.0	fTotalStrokes	 7
U+8094.0	fCangjie	BPD
U+8094.0	fFourCorner	74210
U+8094.0	fCantonese	ci2
U+8094.0	fMandarin	yi3; chi3; shi3
U+8095.0	fUTF8	肕
U+8095.0	fCNS11643	2-2474
U+8095.0	fBig5	CB74
U+8095.0	fGB	3-4C42
U+8095.0	fR/S	 130.3
U+8095.0	fTotalStrokes	 7
U+8095.0	fCangjie	BSHI
U+8095.0	fFourCorner	77220
U+8095.0	fHakka	 ngiun3; niun5; ngiun5; ngien3
U+8095.0	fCantonese	jan6; ngan6
U+8095.0	fMandarin	ren4
U+8095.0	fEnglish	tough; strong; hard
U+8096.0	fUTF8	肖
U+8096.0	fCNS11643	1-4B39
U+8096.0	fBig5	A876
U+8096.0	fGB	0-5024
U+8096.0	fR/S	 130.3
U+8096.0	fTotalStrokes	 7
U+8096.0	fCangjie	FB
U+8096.0	fFourCorner	90227
U+8096.0	fHakka	 siau5; ciau5; siau1; seu5
U+8096.0	fCantonese	ciu3; siu1
U+8096.0	fMandarin	xiao4; xiao1
U+8096.0	fEnglish	 look like; resemble; be like; alike; similar
U+8097.0	fUTF8	肗
U+8097.0	fCNS11643	3-2725
U+8097.0	fGB	3-4C41
U+8097.0	fR/S	  130.3
U+8097.0	fTotalStrokes	 7
U+8097.0	fCangjie	BV
U+8097.0	fFourCorner	74240
U+8097.0	fCantonese	jyu5
U+8097.0	fMandarin	ru3
U+8098.0	fUTF8	肘
U+8098.0	fCNS11643	1-4B3C
U+8098.0	fBig5	A879
U+8098.0	fGB	0-5662
U+8098.0	fR/S	 130.3
U+8098.0	fTotalStrokes	 7
U+8098.0	fCangjie	BDI
U+8098.0	fFourCorner	74200
U+8098.0	fHakka	  ziu3; cu5
U+8098.0	fCantonese	zaau2; zau2 (also)
U+8098.0	fMandarin	zhou3
U+8098.0	fEnglish	[1] the elbow [2] catch one by the elbow
U+8099.0	fUTF8	肙
U+8099.0	fCNS11643	2-2475
U+8099.0	fBig5	CB75
U+8099.0	fGB	3-3557
U+8099.0	fR/S	 130.3
U+8099.0	fTotalStrokes	 7
U+8099.0	fCangjie	RB
U+8099.0	fFourCorner	60227
U+8099.0	fHakka	 gien3; jen1; ren1
U+8099.0	fCantonese	jyun1
U+8099.0	fMandarin	yuan1; yuan4
U+8099.0	fEnglish	 a small worm twist surround empty
U+809A.0	fUTF8	肚
U+809A.0	fCNS11643	1-4B3E
U+809A.0	fBig5	A87B
U+809A.0	fGB	0-3647
U+809A.0	fR/S	 130.3
U+809A.0	fTotalStrokes	 7
U+809A.0	fCangjie	BG
U+809A.0	fFourCorner	74210
U+809A.0	fHakka	 du3
U+809A.0	fCantonese	tou5; tou2 (also)
U+809A.0	fMandarin	du4
U+809A.0	fEnglish	the belly; abdomen; bowels
U+809A.1	fUTF8	肚
U+809A.1	fCNS11643	1-4B3E
U+809A.1	fBig5	A87B
U+809A.1	fGB	0-3647
U+809A.1	fR/S	 130.3
U+809A.1	fTotalStrokes	 7
U+809A.1	fCangjie	BG
U+809A.1	fFourCorner	74210
U+809A.1	fHakka	 du3
U+809A.1	fMandarin	du3
U+809A.1	fEnglish	the stomach
U+809B.0	fUTF8	肛
U+809B.0	fCNS11643	1-4B3D
U+809B.0	fBig5	A87A
U+809B.0	fGB	0-3858
U+809B.0	fR/S	 130.3
U+809B.0	fTotalStrokes	 7
U+809B.0	fCangjie	BM
U+809B.0	fFourCorner	71210
U+809B.0	fHakka	  gong1
U+809B.0	fCantonese	gong1
U+809B.0	fMandarin	gang1
U+809B.0	fEnglish	the anus
U+809C.0	fUTF8	肜
U+809C.0	fCNS11643	2-2478
U+809C.0	fBig5	CB78
U+809C.0	fGB	0-6B40
U+809C.0	fR/S	 130.3
U+809C.0	fTotalStrokes	 7
U+809C.0	fCangjie	BHHH
U+809C.0	fFourCorner	72222
U+809C.0	fHakka	 jung2; rung2
U+809C.0	fCantonese	jung4
U+809C.0	fMandarin	rong2; chen1
U+809C.0	fEnglish	 sacrifice on two successive days; an ancient sacrificial ritual lasting two successive days
U+809D.0	fUTF8	肝
U+809D.0	fCNS11643	1-4B3B
U+809D.0	fBig5	A878
U+809D.0	fGB	0-384E
U+809D.0	fR/S	 130.3
U+809D.0	fTotalStrokes	 7
U+809D.0	fCangjie	BMJ
U+809D.0	fFourCorner	71240
U+809D.0	fHakka	  gon1
U+809D.0	fCantonese	gon1
U+809D.0	fMandarin	gan1
U+809D.0	fEnglish	the liver
U+809E.0	fUTF8	肞
U+809E.0	fCNS11643	4-2470
U+809E.0	fGB	E-3952
U+809E.0	fR/S	  130.3
U+809E.0	fTotalStrokes	 7
U+809E.0	fCangjie	BEI
U+809E.0	fFourCorner	74200
U+809E.0	fCantonese	caa1
U+809E.0	fMandarin	cha1
U+809F.0	fUTF8	肟
U+809F.0	fCNS11643	3-2723
U+809F.0	fGB	0-6B3F
U+809F.0	fR/S	 130.3
U+809F.0	fTotalStrokes	 7
U+809F.0	fCangjie	BMMS
U+809F.0	fFourCorner	71220
U+809F.0	fCantonese	ok3; ngok3 (also); wu1
U+809F.0	fMandarin	wo4
U+809F.0	fEnglish	an organic compound
U+80A0.0	fUTF8	肠
U+80A0.0	fGB	0-3326
U+80A0.0	fR/S	 130.3
U+80A0.0	fTotalStrokes	 7
U+80A0.0	fCangjie	BNSH
U+80A0.0	fHakka	 chong2
U+80A0.0	fCantonese	coeng4; coeng2 (also)
U+80A0.0	fMandarin	chang2
U+80A0.0	fEnglish	simplified form of &#x8178;
U+80A1.0	fUTF8	股
U+80A1.0	fCNS11643	1-4F34
U+80A1.0	fBig5	AAD1
U+80A1.0	fGB	0-3949
U+80A1.0	fR/S	 130.4
U+80A1.0	fTotalStrokes	 8
U+80A1.0	fCangjie	BHNE
U+80A1.0	fFourCorner	77247
U+80A1.0	fHakka	  gu3
U+80A1.0	fCantonese	gu2; gwu2 (also)
U+80A1.0	fMandarin	gu3
U+80A1.0	fEnglish	[1] the thigh; haunches; hips [2] department; section [3] shares; stock [4] puff; blast (of hot air) [5] a bunch or band (of bandits)
U+80A1.1	fUTF8	股
U+80A1.1	fCNS11643	1-4F34
U+80A1.1	fBig5	AAD1
U+80A1.1	fGB	0-3949
U+80A1.1	fR/S	 130.4
U+80A1.1	fTotalStrokes	 8
U+80A1.1	fCangjie	BHNE
U+80A1.1	fFourCorner	77247
U+80A1.1	fMandarin	gu5
U+80A2.0	fUTF8	肢
U+80A2.0	fCNS11643	1-4F32
U+80A2.0	fBig5	AACF
U+80A2.0	fGB	0-562B
U+80A2.0	fR/S	 130.4
U+80A2.0	fTotalStrokes	 8
U+80A2.0	fCangjie	BJE
U+80A2.0	fFourCorner	74247
U+80A2.0	fHakka	 gi1; zi1; zhi1; zih1; tje1; gi3
U+80A2.0	fCantonese	zi1
U+80A2.0	fMandarin	zhi1; shi4
U+80A2.0	fEnglish	[1] human limbs; the four limbs of a person [2] legs of an animal [3] the wings or feet of a bird
U+80A3.0	fUTF8	肣
U+80A3.0	fCNS11643	2-284D
U+80A3.0	fBig5	CDAD
U+80A3.0	fGB	3-4C4E
U+80A3.0	fR/S	 130.4
U+80A3.0	fTotalStrokes	 8
U+80A3.0	fCangjie	BOIN
U+80A3.0	fFourCorner	78227
U+80A3.0	fCantonese	ham4; haam4; ham5; kam4
U+80A3.0	fMandarin	han2; qin2; han4
U+80A4.0	fUTF8	肤
U+80A4.0	fCNS11643	3-2A56
U+80A4.0	fGB	0-3774
U+80A4.0	fR/S	 130.4
U+80A4.0	fTotalStrokes	 8
U+80A4.0	fCangjie	BQO
U+80A4.0	fFourCorner	75230
U+80A4.0	fHakka	 fu1
U+80A4.0	fCantonese	fu1
U+80A4.0	fMandarin	fu1; lu2
U+80A4.0	fEnglish	simplified form of &#x819A;
U+80A5.0	fUTF8	肥
U+80A5.0	fCNS11643	1-4F31
U+80A5.0	fBig5	AACE
U+80A5.0	fGB	0-374A
U+80A5.0	fR/S	 130.4
U+80A5.0	fTotalStrokes	 8
U+80A5.0	fCangjie	BAU
U+80A5.0	fFourCorner	77217
U+80A5.0	fHakka	 pui2;  fui2; pi2; pui3
U+80A5.0	fCantonese	fei4; fei2 (also); fai6
U+80A5.0	fMandarin	fei2; bi3
U+80A5.0	fEnglish	[1] fat; plump; obese; portly; corpulent [2] fat (of meat) [3] sufficiency; affluence; plenty [4] fertile [5] fertilise (land) [6] fertilisers [7] baggy
U+80A6.0	fUTF8	肦
U+80A6.0	fCNS11643	3-2A5B
U+80A6.0	fGB	E-3953
U+80A6.0	fR/S	  130.4
U+80A6.0	fTotalStrokes	 8
U+80A6.0	fCangjie	BCSH
U+80A6.0	fFourCorner	78220
U+80A6.0	fHakka	  ban1; fun1
U+80A6.0	fCantonese	baan1
U+80A6.0	fMandarin	ban1; fen2
U+80A6.0	fEnglish	 large head many high to divide; distribute
U+80A7.0	fUTF8	肧
U+80A7.0	fCNS11643	3-2A58
U+80A7.0	fGB	E-3954
U+80A7.0	fR/S	 130.4
U+80A7.0	fTotalStrokes	 8
U+80A7.0	fCangjie	BMF
U+80A7.0	fFourCorner	71290
U+80A7.0	fHakka	  pi1;  poi1
U+80A7.0	fCantonese	pui1; pei1 (also)
U+80A7.0	fMandarin	pei1
U+80A7.0	fEnglish	 embryo unfinished things
U+80A8.0	fUTF8	肨
U+80A8.0	fCNS11643	3-2A4F
U+80A8.0	fGB	3-4C44
U+80A8.0	fR/S	  130.4
U+80A8.0	fTotalStrokes	 8
U+80A8.0	fCangjie	BQJ
U+80A8.0	fFourCorner	75200
U+80A8.0	fHakka	 pong3;  pong1; pong5
U+80A8.0	fCantonese	bun6; pong3
U+80A8.0	fMandarin	pang4
U+80A8.0	fEnglish	 swell
U+80A9.0	fUTF8	肩
U+80A9.0	fCNS11643	1-4F36
U+80A9.0	fBig5	AAD3
U+80A9.0	fGB	0-3C67
U+80A9.0	fR/S	 130.4
U+80A9.0	fTotalStrokes	 8
U+80A9.0	fCangjie	HSB
U+80A9.0	fFourCorner	30227
U+80A9.0	fHakka	  gien1;  gen1; kian1; gian1; gin1
U+80A9.0	fCantonese	gin1
U+80A9.0	fMandarin	jian1; hen2; xian2; qian1
U+80A9.0	fEnglish	[1] shoulders [2] shoulder; bear (responsibility, etc.); sustain [3] employ; appoint
U+80AA.0	fUTF8	肪
U+80AA.0	fCNS11643	1-4F38
U+80AA.0	fBig5	AAD5
U+80AA.0	fGB	0-373E
U+80AA.0	fR/S	 130.4
U+80AA.0	fTotalStrokes	 8
U+80AA.0	fCangjie	BYHS
U+80AA.0	fFourCorner	70227
U+80AA.0	fHakka	  fong1
U+80AA.0	fCantonese	fong1
U+80AA.0	fMandarin	fang2
U+80AA.0	fEnglish	fat
U+80AB.0	fUTF8	肫
U+80AB.0	fCNS11643	1-4F35
U+80AB.0	fBig5	AAD2
U+80AB.0	fGB	0-6B46
U+80AB.0	fR/S	 130.4
U+80AB.0	fTotalStrokes	 8
U+80AB.0	fCangjie	BPU
U+80AB.0	fFourCorner	75217
U+80AB.0	fHakka	 kin1; dun1;  zhun1; zun1; tun2
U+80AB.0	fCantonese	zeon1; seon4
U+80AB.0	fMandarin	zhun1; tun2; chun2; zhun3; zhuo1; zhuo2; zui3
U+80AB.0	fEnglish	[1] gizzard of a fowl [2] honest; sincere
U+80AC.0	fUTF8	肬
U+80AC.0	fCNS11643	3-2A59
U+80AC.0	fGB	5-4E73
U+80AC.0	fR/S	  130.4
U+80AC.0	fTotalStrokes	 8
U+80AC.0	fCangjie	BIKU
U+80AC.0	fFourCorner	73210
U+80AC.0	fHakka	 ju2; riu2
U+80AC.0	fCantonese	jau4
U+80AC.0	fMandarin	you2
U+80AC.0	fEnglish	 wart tumor goiter papule
U+80AD.0	fUTF8	肭
U+80AD.0	fCNS11643	2-2850
U+80AD.0	fBig5	CDB0
U+80AD.0	fGB	0-6B47
U+80AD.0	fR/S	 130.4
U+80AD.0	fTotalStrokes	 8
U+80AD.0	fCangjie	BOB
U+80AD.0	fFourCorner	74227
U+80AD.0	fHakka	 lep7; lut7
U+80AD.0	fCantonese	neot6; nat6 (also); leot6 (also); lat6 (also); nuk6; luk6 (also)
U+80AD.0	fMandarin	na4; n4
U+80AD.0	fEnglish	 fat seal the character with the moon radical is different
U+80AE.0	fUTF8	肮
U+80AE.0	fCNS11643	2-284C
U+80AE.0	fBig5	CDAC
U+80AE.0	fGB	0-3039
U+80AE.0	fR/S	 130.4
U+80AE.0	fTotalStrokes	 8
U+80AE.0	fCangjie	BYHN
U+80AE.0	fFourCorner	70217
U+80AE.0	fHakka	 ong1; hong3; kong5; kong3
U+80AE.0	fCantonese	ong1; kon3; ngong1 (also); hong4; gong1; gong4; kong3
U+80AE.0	fMandarin	hang2; kang1; gang1; ang1; kang3
U+80AE.0	fEnglish	simplified form of &#x9AAF;
U+80AF.0	fUTF8	肯
U+80AF.0	fCNS11643	1-4F39
U+80AF.0	fBig5	AAD6
U+80AF.0	fGB	0-3F4F
U+80AF.0	fR/S	 130.4
U+80AF.0	fTotalStrokes	 8
U+80AF.0	fCangjie	YMB
U+80AF.0	fFourCorner	21227
U+80AF.0	fHakka	  ken3;  hen3; {hen3}
U+80AF.0	fCantonese	hang2; hoi2
U+80AF.0	fMandarin	ken3; kai3
U+80AF.0	fEnglish	[1] be willing; consent to; permit; agree; approve of [2] easy
U+80B0.0	fUTF8	肰
U+80B0.0	fCNS11643	3-2A52
U+80B0.0	fGB	5-4E72
U+80B0.0	fR/S	  130.4
U+80B0.0	fTotalStrokes	 8
U+80B0.0	fCangjie	BIK
U+80B0.0	fFourCorner	73230
U+80B0.0	fCantonese	jin4
U+80B0.0	fMandarin	ran2
U+80B1.0	fUTF8	肱
U+80B1.0	fCNS11643	1-4F33
U+80B1.0	fBig5	AAD0
U+80B1.0	fGB	0-6B45
U+80B1.0	fR/S	 130.4
U+80B1.0	fTotalStrokes	 8
U+80B1.0	fCangjie	BKI
U+80B1.0	fFourCorner	74232
U+80B1.0	fHakka	 kwen1; gang1; kwen3
U+80B1.0	fCantonese	gwang1
U+80B1.0	fMandarin	gong1
U+80B1.0	fEnglish	the forearm
U+80B2.0	fUTF8	育
U+80B2.0	fCNS11643	1-4B3F
U+80B2.0	fBig5	A87C
U+80B2.0	fGB	0-537D
U+80B2.0	fR/S	 130.4
U+80B2.0	fTotalStrokes	 8
U+80B2.0	fCangjie	YIB
U+80B2.0	fFourCorner	00227
U+80B2.0	fHakka	 juk7; juk8; jiuk7; ruk7; ruk8
U+80B2.0	fCantonese	juk6; jo1; juk1
U+80B2.0	fMandarin	yu4; yo1; zhou4
U+80B2.0	fEnglish	[1] produce; give birth to; breed [2] educate [3] raise; bring up; nourish; nurse
U+80B3.0	fUTF8	肳
U+80B3.0	fCNS11643	3-2A54
U+80B3.0	fGB	3-4C50
U+80B3.0	fR/S	  130.4
U+80B3.0	fTotalStrokes	 8
U+80B3.0	fCangjie	BPHH
U+80B3.0	fFourCorner	77220
U+80B3.0	fHakka	 wut7; vut7
U+80B3.0	fCantonese	man5
U+80B3.0	fMandarin	wen3
U+80B3.0	fEnglish	the corners of the mouth; the lips
U+80B4.0	fUTF8	肴
U+80B4.0	fCNS11643	1-4F37
U+80B4.0	fBig5	AAD4
U+80B4.0	fGB	0-6B48
U+80B4.0	fR/S	 130.4
U+80B4.0	fTotalStrokes	 8
U+80B4.0	fCangjie	KKB
U+80B4.0	fFourCorner	40227
U+80B4.0	fHakka	  ngau2
U+80B4.0	fCantonese	ngaau4; aau4 (also)
U+80B4.0	fMandarin	yao2; xiao2
U+80B4.0	fEnglish	cooked or prepared food (meat and fish); dishes
U+80B5.0	fUTF8	肵
U+80B5.0	fCNS11643	2-284F
U+80B5.0	fBig5	CDAF
U+80B5.0	fGB	3-4C4B
U+80B5.0	fR/S	 130.4
U+80B5.0	fTotalStrokes	 8
U+80B5.0	fCangjie	BHML
U+80B5.0	fFourCorner	72221
U+80B5.0	fHakka	 ki2; giun3
U+80B5.0	fCantonese	kei4
U+80B5.0	fMandarin	jin4; qi2
U+80B5.0	fEnglish	table
U+80B6.0	fUTF8	肶
U+80B6.0	fCNS11643	3-2A57
U+80B6.0	fGB	5-4E74
U+80B6.0	fR/S	 130.4
U+80B6.0	fTotalStrokes	 8
U+80B6.0	fCangjie	BPP
U+80B6.0	fFourCorner	71210
U+80B6.0	fHakka	 pi2
U+80B6.0	fCantonese	bei2
U+80B6.0	fMandarin	pi2
U+80B6.0	fEnglish	leg, thigh
U+80B7.0	fUTF8	肷
U+80B7.0	fCNS11643	4-2764
U+80B7.0	fGB	0-6B49
U+80B7.0	fR/S	  130.4
U+80B7.0	fTotalStrokes	 8
U+80B7.0	fCangjie	BNO
U+80B7.0	fFourCorner	77280
U+80B7.0	fHakka	 hit7;  fut7
U+80B7.0	fCantonese	him2; hip3 (also)
U+80B7.0	fMandarin	xu4; qian3; qian1
U+80B7.0	fEnglish	area between waist and hips
U+80B8.0	fUTF8	肸
U+80B8.0	fCNS11643	2-284E
U+80B8.0	fBig5	CDAE
U+80B8.0	fGB	3-4C4C
U+80B8.0	fR/S	 130.4
U+80B8.0	fTotalStrokes	 8
U+80B8.0	fCangjie	BCJ
U+80B8.0	fFourCorner	78240
U+80B8.0	fHakka	 hit7
U+80B8.0	fCantonese	jat6; bei3; hat1; hit3
U+80B8.0	fMandarin	xi4; bi4; xi1
U+80B8.0	fEnglish	[1] spread out; dispersed [2] diligent
U+80B9.0	fUTF8	肹
U+80B9.0	fCNS11643	4-2769
U+80B9.0	fGB	5-4E77
U+80B9.0	fR/S	 130.4
U+80B9.0	fTotalStrokes	 8
U+80B9.0	fCangjie	BCMS
U+80B9.0	fFourCorner	78220
U+80B9.0	fCantonese	jat6; bei3; hai1
U+80B9.0	fMandarin	xi4; xi1; bi4
U+80B9.0	fEnglish	vulgar, abusive term for female sex organ, cunt
U+80BA.0	fUTF8	肺
U+80BA.0	fCNS11643	1-4F30
U+80BA.0	fBig5	AACD
U+80BA.0	fGB	0-374E
U+80BA.0	fR/S	 130.4
U+80BA.0	fTotalStrokes	 8
U+80BA.0	fCangjie	BJB
U+80BA.0	fFourCorner	75227
U+80BA.0	fHakka	  pui5; fui5; hi5; pi5; fi5
U+80BA.0	fCantonese	fai3; pui3
U+80BA.0	fMandarin	fei4; pei4
U+80BA.0	fEnglish	the lungs
U+80BB.0	fUTF8	肻
U+80BB.0	fCNS11643	3-6631
U+80BB.0	fGB	E-3955
U+80BB.0	fR/S	 130.4
U+80BB.0	fTotalStrokes	 8
U+80BB.0	fCangjie	YBB
U+80BB.0	fCantonese	hang2
U+80BB.0	fMandarin	ken3
U+80BC.0	fUTF8	肼
U+80BC.0	fGB	0-6B42
U+80BC.0	fR/S	 130.4
U+80BC.0	fTotalStrokes	 8
U+80BC.0	fCangjie	BTT
U+80BC.0	fCantonese	zeng2
U+80BC.0	fMandarin	jing3
U+80BC.0	fEnglish	an organic compound
U+80BD.0	fUTF8	肽
U+80BD.0	fGB	0-6B44
U+80BD.0	fR/S	 130.4
U+80BD.0	fTotalStrokes	 8
U+80BD.0	fCangjie	BKI
U+80BD.0	fCantonese	taai3; tai3
U+80BD.0	fMandarin	tai4
U+80BD.0	fEnglish	a chemical compound
U+80BE.0	fUTF8	肾
U+80BE.0	fGB	0-4976
U+80BE.0	fR/S	 130.4
U+80BE.0	fTotalStrokes	 8
U+80BE.0	fCangjie	LEB
U+80BE.0	fHakka	 shin5
U+80BE.0	fCantonese	san5; san2 (also); san6 (also)
U+80BE.0	fMandarin	shen4
U+80BE.0	fEnglish	simplified form of &#x814E;
U+80BF.0	fUTF8	肿
U+80BF.0	fGB	0-5657
U+80BF.0	fR/S	 130.4
U+80BF.0	fTotalStrokes	 8
U+80BF.0	fCangjie	BL
U+80BF.0	fHakka	 zhung3
U+80BF.0	fCantonese	zung2
U+80BF.0	fMandarin	zhong3
U+80BF.0	fEnglish	simplified form of &#x816B;
U+80C0.0	fUTF8	胀
U+80C0.0	fGB	0-554D
U+80C0.0	fR/S	 130.4
U+80C0.0	fTotalStrokes	 8
U+80C0.0	fCangjie	BPO
U+80C0.0	fHakka	 zhong5
U+80C0.0	fCantonese	zoeng3
U+80C0.0	fMandarin	zhang4
U+80C0.0	fEnglish	simplified form of &#x8139;
U+80C1.0	fUTF8	胁
U+80C1.0	fGB	0-5032
U+80C1.0	fR/S	 130.4
U+80C1.0	fTotalStrokes	 8
U+80C1.0	fCangjie	BKSC
U+80C1.0	fHakka	 hiap8
U+80C1.0	fCantonese	hip3
U+80C1.0	fMandarin	xie2; xi1; xian4
U+80C1.0	fEnglish	simplified form of &#x8105;
U+80C2.0	fUTF8	胂
U+80C2.0	fCNS11643	2-2C7C
U+80C2.0	fBig5	D05B
U+80C2.0	fGB	0-6B4F
U+80C2.0	fR/S	 130.5
U+80C2.0	fTotalStrokes	 9
U+80C2.0	fCangjie	BLWL
U+80C2.0	fFourCorner	75200
U+80C2.0	fCantonese	san6; san1; seon3
U+80C2.0	fMandarin	shen1; chen1; shen4; yi2
U+80C3.0	fUTF8	胃
U+80C3.0	fCNS11643	1-532A
U+80C3.0	fBig5	AD47
U+80C3.0	fGB	0-4E38
U+80C3.0	fR/S	 130.5
U+80C3.0	fTotalStrokes	 9
U+80C3.0	fCangjie	WB
U+80C3.0	fFourCorner	60227
U+80C3.0	fHakka	 wui5; wui3; wi5; wui6; vui5
U+80C3.0	fCantonese	wai6
U+80C3.0	fMandarin	wei4
U+80C3.0	fEnglish	the stomach; gizzard of fowl
U+80C4.0	fUTF8	胄
U+80C4.0	fCNS11643	1-532B
U+80C4.0	fBig5	AD48
U+80C4.0	fGB	0-6B50
U+80C4.0	fR/S	 130.5
U+80C4.0	fTotalStrokes	 9
U+80C4.0	fCangjie	LWB
U+80C4.0	fFourCorner	50227
U+80C4.0	fHakka	 diu3;  ciu5; ciu3; ciu6
U+80C4.0	fCantonese	zau6
U+80C4.0	fMandarin	zhou4
U+80C4.0	fEnglish	[1] descendant; posterity; offspring [2] the eldest son
U+80C5.0	fUTF8	胅
U+80C5.0	fCNS11643	2-2C7E
U+80C5.0	fBig5	D05D
U+80C5.0	fGB	3-4C5D
U+80C5.0	fR/S	 130.5
U+80C5.0	fTotalStrokes	 9
U+80C5.0	fCangjie	BHQO
U+80C5.0	fFourCorner	75230
U+80C5.0	fCantonese	dit3; dit6
U+80C5.0	fMandarin	die2
U+80C6.0	fUTF8	胆
U+80C6.0	fCNS11643	3-2E66
U+80C6.0	fGB	0-3528
U+80C6.0	fR/S	 130.5
U+80C6.0	fTotalStrokes	 9
U+80C6.0	fCangjie	BAM
U+80C6.0	fFourCorner	76210
U+80C6.0	fHakka	  dam3
U+80C6.0	fCantonese	daam2
U+80C6.0	fMandarin	dan3; tan2; tan3; da2
U+80C6.0	fEnglish	simplified form of &#x81BD;
U+80C7.0	fUTF8	胇
U+80C7.0	fCNS11643	2-2C78
U+80C7.0	fBig5	D057
U+80C7.0	fGB	3-4C66
U+80C7.0	fR/S	 130.5
U+80C7.0	fTotalStrokes	 9
U+80C7.0	fCangjie	BLLN
U+80C7.0	fFourCorner	75227
U+80C7.0	fCantonese	fai3; fai6; bat6
U+80C7.0	fMandarin	fei4; bi4
U+80C8.0	fUTF8	胈
U+80C8.0	fCNS11643	2-2C7B
U+80C8.0	fBig5	D05A
U+80C8.0	fGB	3-4C57
U+80C8.0	fR/S	 130.5
U+80C8.0	fTotalStrokes	 9
U+80C8.0	fCangjie	BIKK
U+80C8.0	fFourCorner	73247
U+80C8.0	fHakka	  pat8
U+80C8.0	fCantonese	bat6; but6
U+80C8.0	fMandarin	ba2; pei4
U+80C8.0	fEnglish	hair on the legs
U+80C9.0	fUTF8	胉
U+80C9.0	fCNS11643	2-2D26
U+80C9.0	fBig5	D063
U+80C9.0	fGB	5-4F22
U+80C9.0	fR/S	 130.5
U+80C9.0	fTotalStrokes	 9
U+80C9.0	fCangjie	BHA
U+80C9.0	fFourCorner	76200
U+80C9.0	fCantonese	bok3; bok6
U+80C9.0	fMandarin	bo2; bai3; po4
U+80CA.0	fUTF8	胊
U+80CA.0	fCNS11643	2-2D24
U+80CA.0	fBig5	D061
U+80CA.0	fGB	E-3956
U+80CA.0	fR/S	 130.5
U+80CA.0	fTotalStrokes	 9
U+80CA.0	fCangjie	BPR
U+80CA.0	fFourCorner	77220
U+80CA.0	fHakka	 ki1;  gi1
U+80CA.0	fCantonese	keoi4
U+80CA.0	fMandarin	qu2; bo1; chun3; xu1; xu4
U+80CA.0	fEnglish	[1] dried meat strips [2] far; faraway
U+80CB.0	fUTF8	胋
U+80CB.0	fCNS11643	3-2E60
U+80CB.0	fGB	3-4C59
U+80CB.0	fR/S	  130.5
U+80CB.0	fTotalStrokes	 9
U+80CB.0	fCangjie	BYR
U+80CB.0	fFourCorner	71260
U+80CB.0	fCantonese	tim4
U+80CB.0	fMandarin	tian2
U+80CC.0	fUTF8	背
U+80CC.0	fCNS11643	1-532C
U+80CC.0	fBig5	AD49
U+80CC.0	fGB	0-3133
U+80CC.0	fR/S	 130.5
U+80CC.0	fTotalStrokes	 9
U+80CC.0	fCangjie	LPB
U+80CC.0	fFourCorner	11227
U+80CC.0	fHakka	  boi5;  poi5; poi3; ba2; poi6; poe3; poe5; bi5
U+80CC.0	fCantonese	bui3; bui6; bui2 (also)
U+80CC.0	fMandarin	bei4; bo1; pei4
U+80CC.0	fEnglish	[1] the back [2] the back side; the reverse side [3] cast away; turn one's back; give up [4] go against; rebel [5] remember by rote; commit to memory in detail [6] faint; lapse into a coma [7] bear or shoulder (load, burden, etc.); carry on the back [8] remember and cite by heart
U+80CC.1	fUTF8	背
U+80CC.1	fCNS11643	1-532C
U+80CC.1	fBig5	AD49
U+80CC.1	fGB	0-3133
U+80CC.1	fR/S	 130.5
U+80CC.1	fTotalStrokes	 9
U+80CC.1	fCangjie	LPB
U+80CC.1	fFourCorner	11227
U+80CC.1	fMandarin	bei1
U+80CD.0	fUTF8	胍
U+80CD.0	fCNS11643	2-2D2A
U+80CD.0	fBig5	D067
U+80CD.0	fGB	0-6B52
U+80CD.0	fR/S	 130.5; 126.6
U+80CD.0	fTotalStrokes	 9
U+80CD.0	fCangjie	BHVO
U+80CD.0	fFourCorner	72230
U+80CD.0	fHakka	  gu1; gu5
U+80CD.0	fCantonese	gwaa1; gu1
U+80CD.0	fMandarin	gua1; gu1; hu4
U+80CD.0	fEnglish	[1] corpulent [2] pot-bellied
U+80CE.0	fUTF8	胎
U+80CE.0	fCNS11643	1-532F
U+80CE.0	fBig5	AD4C
U+80CE.0	fGB	0-4C25
U+80CE.0	fR/S	 130.5
U+80CE.0	fTotalStrokes	 9
U+80CE.0	fCangjie	BIR
U+80CE.0	fFourCorner	73260
U+80CE.0	fHakka	  toi1
U+80CE.0	fCantonese	toi1
U+80CE.0	fMandarin	tai1
U+80CE.0	fEnglish	[1] unborn child; embryo; fetus [2] an unpolished, semi-processed molding of something
U+80CF.0	fUTF8	胏
U+80CF.0	fCNS11643	2-2D27
U+80CF.0	fBig5	D064
U+80CF.0	fGB	3-4C4F
U+80CF.0	fR/S	 130.5
U+80CF.0	fTotalStrokes	 9
U+80CF.0	fCangjie	BLXH
U+80CF.0	fFourCorner	75227
U+80CF.0	fHakka	  zi3
U+80CF.0	fCantonese	zi2
U+80CF.0	fMandarin	zi3; fei4
U+80CF.0	fEnglish	meat
U+80D0.0	fUTF8	胐
U+80D0.0	fCNS11643	2-2C7D
U+80D0.0	fBig5	D05C
U+80D0.0	fGB	E-3957
U+80D0.0	fR/S	 130.5
U+80D0.0	fTotalStrokes	 9
U+80D0.0	fCangjie	BUU
U+80D0.0	fFourCorner	72277
U+80D0.0	fHakka	 kut7; wut7; chot7; zot7; kwut7
U+80D0.0	fCantonese	pui3; fat1; deot1; fei2
U+80D0.0	fMandarin	ku1; pei2
U+80D0.0	fEnglish	buttocks
U+80D1.0	fUTF8	胑
U+80D1.0	fCNS11643	2-2C7A
U+80D1.0	fBig5	D059
U+80D1.0	fGB	3-4C5A
U+80D1.0	fR/S	 130.5
U+80D1.0	fTotalStrokes	 9
U+80D1.0	fCangjie	BRC
U+80D1.0	fFourCorner	76280
U+80D1.0	fHakka	 zhi1; zih1
U+80D1.0	fCantonese	zi1; zik1
U+80D1.0	fMandarin	zhi1; zhi2; zhu1
U+80D1.0	fEnglish	the limbs. wings of birds. legs of animals
U+80D2.0	fUTF8	胒
U+80D2.0	fCNS11643	3-2E61
U+80D2.0	fGB	3-4C65
U+80D2.0	fR/S	  130.5
U+80D2.0	fTotalStrokes	 9
U+80D2.0	fCangjie	BSP
U+80D2.0	fFourCorner	77210
U+80D2.0	fCantonese	nei6; lei6 (also)
U+80D2.0	fMandarin	ni4
U+80D3.0	fUTF8	胓
U+80D3.0	fCNS11643	3-2E5E
U+80D3.0	fGB	3-4C58
U+80D3.0	fR/S	  130.5
U+80D3.0	fTotalStrokes	 9
U+80D3.0	fCangjie	BMFJ
U+80D3.0	fFourCorner	71240
U+80D3.0	fCantonese	ping4; pun3; pun4
U+80D3.0	fMandarin	ping2
U+80D3.0	fEnglish	fat
U+80D4.0	fUTF8	胔
U+80D4.0	fCNS11643	2-3F2E
U+80D4.0	fBig5	DB49
U+80D4.0	fGB	3-6336
U+80D4.0	fR/S	 130.5
U+80D4.0	fTotalStrokes	 11
U+80D4.0	fCangjie	YPOBO
U+80D4.0	fFourCorner	21227
U+80D4.0	fHakka	 zih5; cih6; cih5; zih1
U+80D4.0	fCantonese	zi3; ci1; zi6
U+80D4.0	fMandarin	zi4; ci2; ji2
U+80D4.0	fEnglish	 rotten meat bones of dead anima
U+80D5.0	fUTF8	胕
U+80D5.0	fCNS11643	2-2D25
U+80D5.0	fBig5	D062
U+80D5.0	fGB	3-4C5F
U+80D5.0	fR/S	 130.5
U+80D5.0	fTotalStrokes	 9
U+80D5.0	fCangjie	BODI
U+80D5.0	fFourCorner	74200
U+80D5.0	fCantonese	fu1; fu4; fu2
U+80D5.0	fMandarin	fu4; fu1; fu2
U+80D6.0	fUTF8	胖
U+80D6.0	fCNS11643	1-5327
U+80D6.0	fBig5	AD44
U+80D6.0	fGB	0-4556
U+80D6.0	fR/S	 130.5
U+80D6.0	fTotalStrokes	 9
U+80D6.0	fCangjie	BFQ
U+80D6.0	fFourCorner	79250
U+80D6.0	fHakka	  pan5;  pang5; pan3; pan2; pan1
U+80D6.0	fCantonese	bun6; pun4; pun3
U+80D6.0	fMandarin	pang4; pan4
U+80D6.0	fEnglish	fat; obese; fatty; corpulent
U+80D6.1	fUTF8	胖
U+80D6.1	fCNS11643	1-5327
U+80D6.1	fBig5	AD44
U+80D6.1	fGB	0-4556
U+80D6.1	fR/S	 130.5
U+80D6.1	fTotalStrokes	 9
U+80D6.1	fCangjie	BFQ
U+80D6.1	fFourCorner	79250
U+80D6.1	fMandarin	pan2
U+80D6.1	fEnglish	comfortable
U+80D7.0	fUTF8	胗
U+80D7.0	fCNS11643	2-2D28
U+80D7.0	fBig5	D065
U+80D7.0	fGB	0-6B53
U+80D7.0	fR/S	 130.5
U+80D7.0	fTotalStrokes	 9
U+80D7.0	fCangjie	BOHH
U+80D7.0	fFourCorner	78222
U+80D7.0	fHakka	 kin3; zhin3; zin3
U+80D7.0	fCantonese	zan1; can2; zan2
U+80D7.0	fMandarin	zhen1; jin3; zhun1
U+80D7.0	fEnglish	the gizzard of a fowl
U+80D7.1	fUTF8	胗
U+80D7.1	fCNS11643	2-2D28
U+80D7.1	fBig5	D065
U+80D7.1	fGB	0-6B53
U+80D7.1	fR/S	 130.5
U+80D7.1	fTotalStrokes	 9
U+80D7.1	fCangjie	BOHH
U+80D7.1	fFourCorner	78222
U+80D7.1	fMandarin	zhen3
U+80D7.1	fEnglish	pustules; rash; measles
U+80D8.0	fUTF8	胘
U+80D8.0	fCNS11643	2-2C77
U+80D8.0	fBig5	D056
U+80D8.0	fGB	3-4C63
U+80D8.0	fR/S	 130.5
U+80D8.0	fTotalStrokes	 9
U+80D8.0	fCangjie	BYVI
U+80D8.0	fFourCorner	70232
U+80D8.0	fHakka	 hien2; hen2; hien1
U+80D8.0	fCantonese	jin4
U+80D8.0	fMandarin	xian2; xuan2
U+80D9.0	fUTF8	胙
U+80D9.0	fCNS11643	2-2D22
U+80D9.0	fBig5	D05F
U+80D9.0	fGB	0-6B51
U+80D9.0	fR/S	 130.5
U+80D9.0	fTotalStrokes	 9
U+80D9.0	fCangjie	BOS
U+80D9.0	fFourCorner	78211
U+80D9.0	fHakka	 cu6; cu5; za5; ze5; cih5
U+80D9.0	fCantonese	zou6
U+80D9.0	fMandarin	zuo4; zu4
U+80D9.0	fEnglish	[1] flesh, meat offered in sacrifices [2] blessings from Heaven [3] report
U+80DA.0	fUTF8	胚
U+80DA.0	fCNS11643	1-5329
U+80DA.0	fBig5	AD46
U+80DA.0	fGB	0-455F
U+80DA.0	fR/S	 130.5
U+80DA.0	fTotalStrokes	 9
U+80DA.0	fCangjie	BMFM
U+80DA.0	fFourCorner	71219
U+80DA.0	fHakka	  pi1;  poi1;  pui1
U+80DA.0	fCantonese	pui1; pei1 (also)
U+80DA.0	fMandarin	pei1
U+80DA.0	fEnglish	[1] a three-month-old fetus; three months of pregnancy [2] things in the embryonic stage; unfinished moldings; embryo; unfinished things [3] the tender sprouts of plants
U+80DB.0	fUTF8	胛
U+80DB.0	fCNS11643	1-532E
U+80DB.0	fBig5	AD4B
U+80DB.0	fGB	0-6B4E
U+80DB.0	fR/S	 130.5
U+80DB.0	fTotalStrokes	 9
U+80DB.0	fCangjie	BWL
U+80DB.0	fFourCorner	76250
U+80DB.0	fHakka	  gap7
U+80DB.0	fCantonese	gaap3
U+80DB.0	fMandarin	jia3
U+80DB.0	fEnglish	the shoulder; the shoulder blade
U+80DC.0	fUTF8	胜
U+80DC.0	fCNS11643	2-2D23
U+80DC.0	fBig5	D060
U+80DC.0	fGB	0-4A24
U+80DC.0	fR/S	 130.5
U+80DC.0	fTotalStrokes	 9
U+80DC.0	fCangjie	BHQM
U+80DC.0	fFourCorner	75210
U+80DC.0	fHakka	 siang1; shin5
U+80DC.0	fCantonese	saang1; sing3; sing1
U+80DC.0	fMandarin	sheng4; sheng1
U+80DC.0	fEnglish	simplified form of &#x52DD;
U+80DD.0	fUTF8	胝
U+80DD.0	fCNS11643	1-5332
U+80DD.0	fBig5	AD4F
U+80DD.0	fGB	0-6B55
U+80DD.0	fR/S	 130.5
U+80DD.0	fTotalStrokes	 9
U+80DD.0	fCangjie	BHPM
U+80DD.0	fFourCorner	72240
U+80DD.0	fHakka	 chi2; zhi1; cih2; zih1; chii2
U+80DD.0	fCantonese	zi1; dai1
U+80DD.0	fMandarin	zhi1; chi1; di4
U+80DD.0	fEnglish	calluses on hands or feet
U+80DE.0	fUTF8	胞
U+80DE.0	fCNS11643	1-5330
U+80DE.0	fBig5	AD4D
U+80DE.0	fGB	0-307B
U+80DE.0	fR/S	 130.5
U+80DE.0	fTotalStrokes	 9
U+80DE.0	fCangjie	BPRU
U+80DE.0	fFourCorner	77212
U+80DE.0	fHakka	  bau1
U+80DE.0	fCantonese	baau1; paau1
U+80DE.0	fMandarin	bao1; pao2; pao4; fou1; pao1
U+80DE.0	fEnglish	[1] the womb; placenta; fetal membrane [2] children of the same parents
U+80DF.0	fUTF8	胟
U+80DF.0	fCNS11643	3-2E65
U+80DF.0	fGB	5-4F26
U+80DF.0	fR/S	  130.5
U+80DF.0	fTotalStrokes	 9
U+80DF.0	fCangjie	BWYI
U+80DF.0	fFourCorner	77250
U+80DF.0	fHakka	 mu3; meu3
U+80DF.0	fCantonese	mou5
U+80DF.0	fMandarin	mu3
U+80E0.0	fUTF8	胠
U+80E0.0	fCNS11643	2-2C79
U+80E0.0	fBig5	D058
U+80E0.0	fGB	3-4C56
U+80E0.0	fR/S	 130.5
U+80E0.0	fTotalStrokes	 9
U+80E0.0	fCangjie	BGI
U+80E0.0	fFourCorner	74231
U+80E0.0	fHakka	 ki1
U+80E0.0	fCantonese	keoi1; koe1
U+80E0.0	fMandarin	qu1; qia4; qie4; qu4
U+80E0.0	fEnglish	[1] open [2] the armpit [3] the right flank of an army
U+80E1.0	fUTF8	胡
U+80E1.0	fCNS11643	1-532D
U+80E1.0	fBig5	AD4A
U+80E1.0	fGB	0-3A7A
U+80E1.0	fR/S	 130.5
U+80E1.0	fTotalStrokes	 9
U+80E1.0	fCangjie	JRB
U+80E1.0	fFourCorner	47620
U+80E1.0	fHakka	  fu2; fu3
U+80E1.0	fCantonese	wu4; wu2 (also)
U+80E1.0	fMandarin	hu2; hu4
U+80E1.0	fEnglish	[1] blunder; reckless; wildly; disorderly [2] stupidly; blindly; confusedly [3] a general name of the northern tribes [4] How?; Why?; When? [5] long and lasting [6] a Chinese family name
U+80E2.0	fUTF8	胢
U+80E2.0	fCNS11643	3-2E64
U+80E2.0	fGB	5-4E7E
U+80E2.0	fR/S	  130.5
U+80E2.0	fTotalStrokes	 9
U+80E2.0	fCangjie	BMNR
U+80E2.0	fFourCorner	71220
U+80E2.0	fCantonese	kaa3
U+80E2.0	fMandarin	ke1
U+80E3.0	fUTF8	胣
U+80E3.0	fCNS11643	2-2D21
U+80E3.0	fBig5	D05E
U+80E3.0	fGB	3-4C5E
U+80E3.0	fR/S	 130.5
U+80E3.0	fTotalStrokes	 9
U+80E3.0	fCangjie	BOPD
U+80E3.0	fFourCorner	78212
U+80E3.0	fCantonese	ci2; ji5
U+80E3.0	fMandarin	yi3; chi3; shi3
U+80E4.0	fUTF8	胤
U+80E4.0	fCNS11643	1-5331
U+80E4.0	fBig5	AD4E
U+80E4.0	fGB	0-5837
U+80E4.0	fR/S	 130.5
U+80E4.0	fTotalStrokes	 9
U+80E4.0	fCangjie	LVBU
U+80E4.0	fFourCorner	22010
U+80E4.0	fHakka	 jin3;  jin5; rin5; rin3
U+80E4.0	fCantonese	jan6
U+80E4.0	fMandarin	yin4
U+80E4.0	fEnglish	long successions of descendants; posterity
U+80E5.0	fUTF8	胥
U+80E5.0	fCNS11643	1-5328
U+80E5.0	fBig5	AD45
U+80E5.0	fGB	0-7163
U+80E5.0	fR/S	 130.5
U+80E5.0	fTotalStrokes	 9
U+80E5.0	fCangjie	NOB
U+80E5.0	fFourCorner	17227
U+80E5.0	fHakka	  si1
U+80E5.0	fCantonese	seoi1
U+80E5.0	fMandarin	xu1; xu3; shu1; su4
U+80E5.0	fEnglish	[1] all; together [2] wait for [3] assist; serve as an advisory role [4] survey; inspect [5] keep away from; separated [6] a final particle [7] mutual [8] a variety of official ranks through ancient dynasties [9] a Chinese family name
U+80E6.0	fUTF8	胦
U+80E6.0	fCNS11643	2-2D29
U+80E6.0	fBig5	D066
U+80E6.0	fGB	3-4C5B
U+80E6.0	fR/S	 130.5
U+80E6.0	fTotalStrokes	 9
U+80E6.0	fCangjie	BLBK
U+80E6.0	fFourCorner	75230
U+80E6.0	fHakka	 jong1; rong1
U+80E6.0	fCantonese	joeng1
U+80E6.0	fMandarin	yang1; yang3
U+80E6.0	fEnglish	the navel
U+80E7.0	fUTF8	胧
U+80E7.0	fGB	0-6B4A
U+80E7.0	fR/S	 130.5
U+80E7.0	fTotalStrokes	 9
U+80E7.0	fCangjie	BIKP
U+80E7.0	fHakka	 lung2
U+80E7.0	fCantonese	lung4
U+80E7.0	fMandarin	long2; long3
U+80E7.0	fEnglish	simplified form of &#x6727;
U+80E8.0	fUTF8	胨
U+80E8.0	fGB	0-6B4B
U+80E8.0	fR/S	 130.5
U+80E8.0	fTotalStrokes	 9
U+80E8.0	fCangjie	BKD
U+80E8.0	fCantonese	dung3
U+80E8.0	fMandarin	dong4
U+80E8.0	fEnglish	simplified form of &#x8156;
U+80E9.0	fUTF8	胩
U+80E9.0	fGB	0-6B4C
U+80E9.0	fR/S	 130.5
U+80E9.0	fTotalStrokes	 9
U+80E9.0	fCangjie	BYMY
U+80E9.0	fCantonese	kaa1
U+80E9.0	fMandarin	ka3
U+80E9.0	fEnglish	an organic compound
U+80EA.0	fUTF8	胪
U+80EA.0	fGB	0-6B4D
U+80EA.0	fR/S	 130.5
U+80EA.0	fTotalStrokes	 9
U+80EA.0	fCangjie	BYS
U+80EA.0	fHakka	 lu2
U+80EA.0	fCantonese	lou4
U+80EA.0	fMandarin	lu2; luu2; luu3
U+80EA.0	fEnglish	simplified form of &#x81DA;
U+80EB.0	fUTF8	胫
U+80EB.0	fGB	0-6B56
U+80EB.0	fR/S	 130.5
U+80EB.0	fTotalStrokes	 9
U+80EB.0	fCangjie	BNOM
U+80EB.0	fHakka	 gin5; gin1; kwang3
U+80EB.0	fCantonese	hing5; ging3 (also)
U+80EB.0	fMandarin	jing4; heng4; keng1; xing4
U+80EB.0	fEnglish	simplified form of &#x811B;
U+80EC.0	fUTF8	胬
U+80EC.0	fGB	0-6640
U+80EC.0	fR/S	 130.5
U+80EC.0	fTotalStrokes	 11
U+80EC.0	fCangjie	VEOBO
U+80EC.0	fHakka	 nu3
U+80EC.0	fCantonese	nou5; lou5 (also)
U+80EC.0	fMandarin	nu3; nu2
U+80EC.0	fEnglish	the name of a disease in Chinese medicine
U+80ED.0	fUTF8	胭
U+80ED.0	fCNS11643	1-575E
U+80ED.0	fBig5	AFDA
U+80ED.0	fGB	0-6B59
U+80ED.0	fR/S	 130.6
U+80ED.0	fTotalStrokes	 10
U+80ED.0	fCangjie	BWK
U+80ED.0	fFourCorner	76200
U+80ED.0	fHakka	 jen1; jan1; ren1
U+80ED.0	fCantonese	jin1
U+80ED.0	fMandarin	yan1
U+80ED.0	fEnglish	[1] rouge; face powder; cosmetics [2] the throat
U+80EE.0	fUTF8	胮
U+80EE.0	fCNS11643	3-334C
U+80EE.0	fGB	5-4F2D
U+80EE.0	fR/S	  130.6; 130.7
U+80EE.0	fTotalStrokes	 10
U+80EE.0	fCangjie	BHEQ
U+80EE.0	fFourCorner	77250
U+80EE.0	fHakka	 pong2;  pong5; pong1; pang1
U+80EE.0	fCantonese	pong1
U+80EE.0	fMandarin	pang2; pang1
U+80EE.0	fEnglish	to swell
U+80EF.0	fUTF8	胯
U+80EF.0	fCNS11643	1-5767
U+80EF.0	fBig5	AFE3
U+80EF.0	fGB	0-3F68
U+80EF.0	fR/S	 130.6
U+80EF.0	fTotalStrokes	 10
U+80EF.0	fCangjie	BKMS
U+80EF.0	fFourCorner	74227
U+80EF.0	fHakka	  kwa5;  ku5; ka1; kwa1
U+80EF.0	fCantonese	kwaa3; kwaa1; fu3
U+80EF.0	fMandarin	kua4; kua3; ku4; kua1
U+80EF.0	fEnglish	space between the legs; the groin
U+80F0.0	fUTF8	胰
U+80F0.0	fCNS11643	1-575C
U+80F0.0	fBig5	AFD8
U+80F0.0	fGB	0-5248
U+80F0.0	fR/S	 130.6
U+80F0.0	fTotalStrokes	 10
U+80F0.0	fCangjie	BKN
U+80F0.0	fFourCorner	75232
U+80F0.0	fHakka	 ji2; ri2; i2
U+80F0.0	fCantonese	ji4
U+80F0.0	fMandarin	yi2
U+80F0.0	fEnglish	the pancreas
U+80F1.0	fUTF8	胱
U+80F1.0	fCNS11643	1-575A
U+80F1.0	fBig5	AFD6
U+80F1.0	fGB	0-6B57
U+80F1.0	fR/S	 130.6
U+80F1.0	fTotalStrokes	 10
U+80F1.0	fCangjie	BFMU
U+80F1.0	fFourCorner	79211
U+80F1.0	fHakka	  gong1; gwong1; kwong1; gwang1; gwet7
U+80F1.0	fCantonese	gwong1; gong1 (also)
U+80F1.0	fMandarin	guang1
U+80F1.0	fEnglish	the bladder
U+80F2.0	fUTF8	胲
U+80F2.0	fCNS11643	2-322E
U+80F2.0	fBig5	D36A
U+80F2.0	fGB	0-6B5C
U+80F2.0	fR/S	 130.6
U+80F2.0	fTotalStrokes	 10
U+80F2.0	fCangjie	BYVO
U+80F2.0	fFourCorner	70282
U+80F2.0	fHakka	 goi1;  goi3
U+80F2.0	fCantonese	goi2; goi1; hoi2; haai5
U+80F2.0	fMandarin	hai3; gai1; gai3; gei3
U+80F3.0	fUTF8	胳
U+80F3.0	fCNS11643	1-5762
U+80F3.0	fBig5	AFDE
U+80F3.0	fGB	0-386C
U+80F3.0	fR/S	 130.6
U+80F3.0	fTotalStrokes	 10
U+80F3.0	fCangjie	BHER
U+80F3.0	fFourCorner	77264
U+80F3.0	fHakka	  gok7; gak7; get7
U+80F3.0	fCantonese	gaak3; gok3; lok3 (also); gat1 (also)
U+80F3.0	fMandarin	ge1; ga1; ge2
U+80F3.0	fEnglish	the armpit; arms
U+80F4.0	fUTF8	胴
U+80F4.0	fCNS11643	1-575F
U+80F4.0	fBig5	AFDB
U+80F4.0	fGB	0-6B58
U+80F4.0	fR/S	 130.6
U+80F4.0	fTotalStrokes	 10
U+80F4.0	fCangjie	BBMR
U+80F4.0	fFourCorner	77220
U+80F4.0	fHakka	 tung5; tung6; tung3
U+80F4.0	fCantonese	dung6
U+80F4.0	fMandarin	dong4
U+80F4.0	fEnglish	[1] the large intestine [2] the trunk; the body
U+80F5.0	fUTF8	胵
U+80F5.0	fCNS11643	2-3230
U+80F5.0	fBig5	D36C
U+80F5.0	fGB	3-4C6E
U+80F5.0	fR/S	 130.6
U+80F5.0	fTotalStrokes	 10
U+80F5.0	fCangjie	BMIG
U+80F5.0	fFourCorner	71214
U+80F5.0	fHakka	 chi1; cih1; chii1
U+80F5.0	fCantonese	ci1; ci3; ci4
U+80F5.0	fMandarin	zhi4; chi1
U+80F5.0	fEnglish	crop of a fowl
U+80F6.0	fUTF8	胶
U+80F6.0	fCNS11643	3-334A
U+80F6.0	fGB	0-3D3A
U+80F6.0	fR/S	 130.6
U+80F6.0	fTotalStrokes	 10
U+80F6.0	fCangjie	BYCK
U+80F6.0	fFourCorner	70240
U+80F6.0	fHakka	 ngau2; gau1
U+80F6.0	fCantonese	gaau1
U+80F6.0	fMandarin	xiao2; jiao1; jiao3; hao2; jiao4; nao3; qiao1
U+80F6.0	fEnglish	simplified form of &#x81A0;
U+80F7.0	fUTF8	胷
U+80F7.0	fCNS11643	3-3352
U+80F7.0	fGB	E-3958
U+80F7.0	fR/S	  130.6
U+80F7.0	fTotalStrokes	 10
U+80F7.0	fCangjie	PUB
U+80F7.0	fFourCorner	27220
U+80F7.0	fCantonese	hung1
U+80F7.0	fMandarin	xiong1
U+80F7.0	fEnglish	 breast; chest; thorax clamor
U+80F8.0	fUTF8	胸
U+80F8.0	fCNS11643	1-5761
U+80F8.0	fBig5	AFDD
U+80F8.0	fGB	0-5058
U+80F8.0	fR/S	 130.6
U+80F8.0	fTotalStrokes	 10
U+80F8.0	fCangjie	BPUK
U+80F8.0	fFourCorner	77220
U+80F8.0	fHakka	  hiung1
U+80F8.0	fCantonese	hung1
U+80F8.0	fMandarin	xiong1
U+80F8.0	fEnglish	[1] breast; bosom; chest; thorax; bust [2] one's ambition or aspiration [3] the mind (as narrow-minded, etc.); one's capacity
U+80F9.0	fUTF8	胹
U+80F9.0	fCNS11643	2-322F
U+80F9.0	fBig5	D36B
U+80F9.0	fGB	3-4C6B
U+80F9.0	fR/S	 130.6
U+80F9.0	fTotalStrokes	 10
U+80F9.0	fCangjie	BMBL
U+80F9.0	fFourCorner	71227
U+80F9.0	fHakka	 ji2; ri2; i2
U+80F9.0	fCantonese	ji4
U+80F9.0	fMandarin	er2
U+80F9.0	fEnglish	cooked; well-done
U+80FA.0	fUTF8	胺
U+80FA.0	fCNS11643	2-322D
U+80FA.0	fBig5	D369
U+80FA.0	fGB	0-3037
U+80FA.0	fR/S	 130.6
U+80FA.0	fTotalStrokes	 10
U+80FA.0	fCangjie	BJV
U+80FA.0	fFourCorner	73244
U+80FA.0	fHakka	 ot7
U+80FA.0	fCantonese	on1; ngon1 (also); on3 (also); aat3; ngaat3 (also); ngon3 (also)
U+80FA.0	fMandarin	e4; an4
U+80FA.0	fEnglish	amine
U+80FB.0	fUTF8	胻
U+80FB.0	fCNS11643	2-3232
U+80FB.0	fBig5	D36E
U+80FB.0	fGB	3-4C75
U+80FB.0	fR/S	 130.6
U+80FB.0	fTotalStrokes	 10
U+80FB.0	fCangjie	BHON
U+80FB.0	fFourCorner	71221
U+80FB.0	fCantonese	hang4
U+80FB.0	fMandarin	xing2; hang2; heng2; xing4
U+80FC.0	fUTF8	胼
U+80FC.0	fCNS11643	1-5766
U+80FC.0	fBig5	AFE2
U+80FC.0	fGB	0-6B5D
U+80FC.0	fR/S	 130.6
U+80FC.0	fTotalStrokes	 10
U+80FC.0	fCangjie	BTT
U+80FC.0	fFourCorner	78241
U+80FC.0	fHakka	  pin2; pien3; pen2
U+80FC.0	fCantonese	pin4; bin1
U+80FC.0	fMandarin	pian2
U+80FC.0	fEnglish	calluses
U+80FD.0	fUTF8	能
U+80FD.0	fCNS11643	1-5764
U+80FD.0	fBig5	AFE0
U+80FD.0	fGB	0-445C
U+80FD.0	fR/S	 130.6
U+80FD.0	fTotalStrokes	 10
U+80FD.0	fCangjie	IBPP
U+80FD.0	fFourCorner	21211
U+80FD.0	fHakka	  nen2; len2; lien2; nen3
U+80FD.0	fCantonese	nang4; lang4 (also); noi6; toi4
U+80FD.0	fMandarin	neng2; nai4; nai2; tai2; tai4; xiong2
U+80FD.0	fEnglish	[1] be able; can [2] ability; capability; talent; competence [3] energy (as atomic)
U+80FE.0	fUTF8	胾
U+80FE.0	fCNS11643	2-3F2D
U+80FE.0	fBig5	DB48
U+80FE.0	fGB	3-4633
U+80FE.0	fR/S	 130.6
U+80FE.0	fTotalStrokes	 12
U+80FE.0	fCangjie	JIOBO
U+80FE.0	fFourCorner	43250
U+80FE.0	fHakka	 ci5; chi6; zih5
U+80FE.0	fCantonese	zi3; zi1
U+80FE.0	fMandarin	zi4
U+80FE.0	fEnglish	 minced meat; meat cut into pieces; minced meat
U+80FF.0	fUTF8	胿
U+80FF.0	fCNS11643	3-334B
U+80FF.0	fGB	3-4C69
U+80FF.0	fR/S	  130.6
U+80FF.0	fTotalStrokes	 10
U+80FF.0	fCangjie	BGG
U+80FF.0	fFourCorner	74210
U+80FF.0	fCantonese	gwai1
U+80FF.0	fMandarin	gui1
U+8100.0	fUTF8	脀
U+8100.0	fCNS11643	2-3233
U+8100.0	fBig5	D36F
U+8100.0	fGB	3-4E2D
U+8100.0	fR/S	 130.6
U+8100.0	fTotalStrokes	 10
U+8100.0	fCangjie	NEMB
U+8100.0	fFourCorner	17227
U+8100.0	fHakka	 zhin1; zihn1; shin2; siin2
U+8100.0	fCantonese	zing1; cing1
U+8100.0	fMandarin	cheng2; zheng1; zheng4
U+8100.0	fEnglish	[1] cooked sacrificial meat [2] swollen [3] ascend
U+8101.0	fUTF8	脁
U+8101.0	fCNS11643	2-3231
U+8101.0	fBig5	D36D
U+8101.0	fGB	E-3959
U+8101.0	fR/S	 130.6
U+8101.0	fTotalStrokes	 10
U+8101.0	fCangjie	BLMO
U+8101.0	fFourCorner	72213
U+8101.0	fHakka	 tiau5; tiau1
U+8101.0	fCantonese	tiu2; tiu3; jiu4
U+8101.0	fMandarin	tiao4; chao4; tiao3; yao2
U+8101.0	fEnglish	sacrificial rites
U+8102.0	fUTF8	脂
U+8102.0	fCNS11643	1-575B
U+8102.0	fBig5	AFD7
U+8102.0	fGB	0-562C
U+8102.0	fR/S	 130.6
U+8102.0	fTotalStrokes	 10
U+8102.0	fCangjie	BPA
U+8102.0	fFourCorner	71261
U+8102.0	fHakka	 zhi1; zi1; zih1; tje1
U+8102.0	fCantonese	zi1
U+8102.0	fMandarin	zhi1; zhi3
U+8102.0	fEnglish	[1] fat of animals; grease; lard; tallow [2] the gum or sap of trees; resin [3] anoint; grease; lubricate [4] cosmetics [5] a Chinese family name
U+8103.0	fUTF8	脃
U+8103.0	fCNS11643	3-334E
U+8103.0	fGB	E-395A
U+8103.0	fR/S	 130.6
U+8103.0	fTotalStrokes	 10
U+8103.0	fCangjie	BNAU
U+8103.0	fFourCorner	77210
U+8103.0	fHakka	 cui5; coi5; ts'ioi5; cioi5; ce5
U+8103.0	fCantonese	ceoi3
U+8103.0	fMandarin	cui4
U+8103.0	fEnglish	gland
U+8104.0	fUTF8	脄
U+8104.0	fCNS11643	3-334D
U+8104.0	fGB	5-4F29
U+8104.0	fR/S	 130.6
U+8104.0	fTotalStrokes	 10
U+8104.0	fCangjie	BKF
U+8104.0	fFourCorner	71280
U+8104.0	fCantonese	mui4
U+8104.0	fMandarin	mei2
U+8105.0	fUTF8	脅
U+8105.0	fCNS11643	1-575D
U+8105.0	fBig5	AFD9
U+8105.0	fGB	1-5032
U+8105.0	fR/S	 130.6
U+8105.0	fTotalStrokes	 10
U+8105.0	fCangjie	KSKSB
U+8105.0	fFourCorner	40227
U+8105.0	fHakka	  hiap8; hiap7
U+8105.0	fCantonese	hip3; hip6
U+8105.0	fMandarin	xie2; xi1; xian4
U+8105.0	fEnglish	[1] the sides of the trunk from armpits to ribs; flank [2] threaten with force; force [3] shrug (shoulders); shrink
U+8106.0	fUTF8	脆
U+8106.0	fCNS11643	1-5760
U+8106.0	fBig5	AFDC
U+8106.0	fGB	0-3460
U+8106.0	fR/S	 130.6
U+8106.0	fTotalStrokes	 10
U+8106.0	fCangjie	BNMU
U+8106.0	fFourCorner	77212
U+8106.0	fHakka	 coi5;  cui5;  ce5; ch'oi5; ts'ioi5; cioi5
U+8106.0	fCantonese	ceoi3
U+8106.0	fMandarin	cui4
U+8106.0	fEnglish	[1] crisp [2] fragile; frail; brittle; hard but easily broken [3] light; shallow; thin [4] easy; quick and convenient (operation, etc.)
U+8107.0	fUTF8	脇
U+8107.0	fCNS11643	3-334F
U+8107.0	fGB	E-395B
U+8107.0	fR/S	 130.6
U+8107.0	fTotalStrokes	 10
U+8107.0	fCangjie	BKSS
U+8107.0	fFourCorner	74220
U+8107.0	fHakka	  hiap8;  hiap7
U+8107.0	fCantonese	hip3
U+8107.0	fMandarin	xie2
U+8107.0	fEnglish	 ribs; armpits flank threaten
U+8108.0	fUTF8	脈
U+8108.0	fCNS11643	1-5763
U+8108.0	fBig5	AFDF
U+8108.0	fGB	E-395C
U+8108.0	fR/S	 130.6
U+8108.0	fTotalStrokes	 10
U+8108.0	fCangjie	BHHV
U+8108.0	fFourCorner	72232
U+8108.0	fHakka	  mak7
U+8108.0	fCantonese	mak6; maak6
U+8108.0	fMandarin	mai4; mo4
U+8108.0	fEnglish	[1] the blood vessels; veins; arteries; the circulation system [2] the pulse [3] mountain range [4] things that are related and form a system of some kind [5] stipules or stems of a leaf
U+8109.0	fUTF8	脉
U+8109.0	fCNS11643	3-2E63
U+8109.0	fGB	0-4276
U+8109.0	fR/S	 130.5; 130.6
U+8109.0	fTotalStrokes	 9
U+8109.0	fCangjie	BINE
U+8109.0	fFourCorner	73230
U+8109.0	fHakka	  mak7
U+8109.0	fCantonese	mak6
U+8109.0	fMandarin	mai4; mo4
U+8109.0	fEnglish	blood vessels; veins; arteries
U+810A.0	fUTF8	脊
U+810A.0	fCNS11643	1-5765
U+810A.0	fBig5	AFE1
U+810A.0	fGB	0-3C39
U+810A.0	fR/S	 130.6
U+810A.0	fTotalStrokes	 10
U+810A.0	fCangjie	FCB
U+810A.0	fFourCorner	11227
U+810A.0	fHakka	 zit7; ziak7
U+810A.0	fCantonese	zik3; zek3 (also)
U+810A.0	fMandarin	ji3; ji2
U+810A.0	fEnglish	[1] the spine; backbone; the spinal column [2] the ridge
U+810B.0	fUTF8	脋
U+810B.0	fGB	E-395D
U+810B.0	fR/S	 130.6
U+810B.0	fTotalStrokes	 10
U+810B.0	fCangjie	SHSHB
U+810B.0	fCantonese	hip3
U+810B.0	fMandarin	xie2
U+810C.0	fUTF8	脌
U+810C.0	fGB	1-7C48
U+810C.0	fR/S	 130.6
U+810C.0	fTotalStrokes	 10
U+810C.0	fCangjie	BOQ
U+810C.0	fCantonese	nin1; lin1 (also)
U+810C.0	fEnglish	female breast; milk
U+810D.0	fUTF8	脍
U+810D.0	fGB	0-6B5A
U+810D.0	fR/S	 130.6
U+810D.0	fTotalStrokes	 10
U+810D.0	fCangjie	BOMI
U+810D.0	fHakka	 kwai5
U+810D.0	fCantonese	kui2; kwui2 (also)
U+810D.0	fMandarin	kuai4; guai4
U+810D.0	fEnglish	simplified form of &#x81BE;
U+810E.0	fUTF8	脎
U+810E.0	fGB	0-6B5B
U+810E.0	fR/S	 130.6
U+810E.0	fTotalStrokes	 10
U+810E.0	fCangjie	XBKD
U+810E.0	fCantonese	saat3
U+810E.0	fMandarin	sa4
U+810E.0	fEnglish	an organic compound
U+810F.0	fUTF8	脏
U+810F.0	fGB	0-5460
U+810F.0	fR/S	 130.6
U+810F.0	fTotalStrokes	 10
U+810F.0	fCangjie	BIG
U+810F.0	fHakka	 cong3; zong1; cong5
U+810F.0	fCantonese	zong1; zong6; zong3 (also)
U+810F.0	fMandarin	zang4; zang3
U+810F.0	fEnglish	simplified form of &#x81DF;
U+810F.1	fUTF8	脏
U+810F.1	fGB	0-5460
U+810F.1	fR/S	 130.6
U+810F.1	fTotalStrokes	 10
U+810F.1	fCangjie	BIG
U+810F.1	fMandarin	zang1
U+8110.0	fUTF8	脐
U+8110.0	fGB	0-466A
U+8110.0	fR/S	 130.6
U+8110.0	fTotalStrokes	 10
U+8110.0	fCangjie	BYKL
U+8110.0	fHakka	 ci2
U+8110.0	fCantonese	ci4
U+8110.0	fMandarin	qi2
U+8110.0	fEnglish	simplified form of &#x81CD;
U+8111.0	fUTF8	脑
U+8111.0	fGB	0-4454
U+8111.0	fR/S	 130.6
U+8111.0	fTotalStrokes	 10
U+8111.0	fCangjie	BYUK
U+8111.0	fHakka	 nau3; no3
U+8111.0	fCantonese	nou5; lou5 (also)
U+8111.0	fMandarin	nao3; nao4
U+8111.0	fEnglish	simplified form of &#x8166;
U+8112.0	fUTF8	脒
U+8112.0	fGB	0-6B5F
U+8112.0	fR/S	 130.6
U+8112.0	fTotalStrokes	 10
U+8112.0	fCangjie	BFD
U+8112.0	fCantonese	mai5
U+8112.0	fMandarin	mi3
U+8112.0	fEnglish	 open throw away
U+8113.0	fUTF8	脓
U+8113.0	fGB	0-4527
U+8113.0	fR/S	 130.6
U+8113.0	fTotalStrokes	 10
U+8113.0	fCangjie	BHBV
U+8113.0	fHakka	 nung2
U+8113.0	fCantonese	nung4; lung4 (also)
U+8113.0	fMandarin	nong2; neng2
U+8113.0	fEnglish	simplified form of &#x81BF;
U+8114.0	fUTF8	脔
U+8114.0	fCNS11643	3-3E44
U+8114.0	fGB	0-5975
U+8114.0	fR/S	  130.6
U+8114.0	fTotalStrokes	 12
U+8114.0	fCangjie	YCOBO
U+8114.0	fFourCorner	00220
U+8114.0	fHakka	 lan2
U+8114.0	fCantonese	lyun5; lyun2 (also); lyun4 (also)
U+8114.0	fMandarin	ji1; luan2; luan3
U+8114.0	fEnglish	simplified form of &#x81E0;
U+8115.0	fUTF8	脕
U+8115.0	fCNS11643	2-3852
U+8115.0	fBig5	D74E
U+8115.0	fGB	5-4F36
U+8115.0	fR/S	 130.7
U+8115.0	fTotalStrokes	 11
U+8115.0	fCangjie	BNAU
U+8115.0	fFourCorner	77217
U+8115.0	fCantonese	maan5; maan6 (also)
U+8115.0	fMandarin	wan3; wan4; wen4
U+8116.0	fUTF8	脖
U+8116.0	fCNS11643	1-5C69
U+8116.0	fBig5	B2E4
U+8116.0	fGB	0-3231
U+8116.0	fR/S	 130.7
U+8116.0	fTotalStrokes	 11
U+8116.0	fCangjie	BJBD
U+8116.0	fFourCorner	74247
U+8116.0	fHakka	 put8; {pat8}
U+8116.0	fCantonese	but6
U+8116.0	fMandarin	bo2
U+8116.0	fEnglish	the neck
U+8117.0	fUTF8	脗
U+8117.0	fCNS11643	3-3849
U+8117.0	fGB	E-395E
U+8117.0	fR/S	 130.7
U+8117.0	fTotalStrokes	 11
U+8117.0	fCangjie	BPHR
U+8117.0	fFourCorner	77260
U+8117.0	fHakka	  wut7; vut7
U+8117.0	fCantonese	man5
U+8117.0	fMandarin	wen3
U+8117.0	fEnglish	[1] kiss [2] join together; match; tally [3] the lips
U+8118.0	fUTF8	脘
U+8118.0	fCNS11643	2-3849
U+8118.0	fBig5	D745
U+8118.0	fGB	0-6B64
U+8118.0	fR/S	 130.7
U+8118.0	fTotalStrokes	 11
U+8118.0	fCangjie	BJMU
U+8118.0	fFourCorner	73211
U+8118.0	fHakka	 gon3; fan2; gwon3
U+8118.0	fCantonese	gun2; wun2 (also); gwun2 (also); wun5
U+8118.0	fMandarin	wan3; guan3; huan4; huan2; huan3
U+8118.0	fEnglish	the internal cavity of stomach; the inside of the stomach; gastric cavity; duct in the body
U+8119.0	fUTF8	脙
U+8119.0	fCNS11643	2-384B
U+8119.0	fBig5	D747
U+8119.0	fGB	5-4F31
U+8119.0	fR/S	 130.7
U+8119.0	fTotalStrokes	 11
U+8119.0	fCangjie	BIJE
U+8119.0	fFourCorner	73232
U+8119.0	fCantonese	jau1
U+8119.0	fMandarin	qiu2; xiu1
U+811A.0	fUTF8	脚
U+811A.0	fCNS11643	3-3847
U+811A.0	fGB	0-3D45
U+811A.0	fR/S	 130.7
U+811A.0	fTotalStrokes	 11
U+811A.0	fCangjie	BGIL
U+811A.0	fFourCorner	77220
U+811A.0	fHakka	  giok7; ka2
U+811A.0	fCantonese	goek3; gok3
U+811A.0	fMandarin	jiao3; jue2
U+811A.0	fEnglish	 leg; foot base; leg; foundation
U+811B.0	fUTF8	脛
U+811B.0	fCNS11643	2-384C
U+811B.0	fBig5	D748
U+811B.0	fGB	1-6B56
U+811B.0	fR/S	 130.7
U+811B.0	fTotalStrokes	 11
U+811B.0	fCangjie	BMVM
U+811B.0	fFourCorner	71211
U+811B.0	fHakka	 gin1;  gin5;  kwang3; gin6; gang3; kin1
U+811B.0	fCantonese	hing5; ging3 (also)
U+811B.0	fMandarin	jing4; heng4; keng1; xing4
U+811B.0	fEnglish	the calf; part of a of leg between the knee and ankle; the shank
U+811C.0	fUTF8	脜
U+811C.0	fCNS11643	4-344D
U+811C.0	fGB	E-395F
U+811C.0	fR/S	  130.7
U+811C.0	fTotalStrokes	 11
U+811C.0	fCangjie	BMBU
U+811C.0	fFourCorner	71260
U+811C.0	fCantonese	jau4
U+811C.0	fMandarin	rou2; you3
U+811D.0	fUTF8	脝
U+811D.0	fCNS11643	2-3854
U+811D.0	fBig5	D750
U+811D.0	fGB	5-4F37
U+811D.0	fR/S	 130.7
U+811D.0	fTotalStrokes	 11
U+811D.0	fCangjie	BYRN
U+811D.0	fFourCorner	70227
U+811D.0	fHakka	  hen1; kien1; ken1
U+811D.0	fCantonese	hang1
U+811D.0	fMandarin	heng1
U+811D.0	fEnglish	distend
U+811E.0	fUTF8	脞
U+811E.0	fCNS11643	2-3850
U+811E.0	fBig5	D74C
U+811E.0	fGB	0-6B62
U+811E.0	fR/S	 130.7
U+811E.0	fTotalStrokes	 11
U+811E.0	fCangjie	BOOG
U+811E.0	fFourCorner	78214
U+811E.0	fHakka	  co5
U+811E.0	fCantonese	co2
U+811E.0	fMandarin	cuo3; qie1; cuo2; qia1
U+811E.0	fEnglish	tiny; petty; little pieces
U+811F.0	fUTF8	脟
U+811F.0	fCNS11643	2-384E
U+811F.0	fBig5	D74A
U+811F.0	fGB	3-4D2A
U+811F.0	fR/S	 130.7
U+811F.0	fTotalStrokes	 11
U+811F.0	fCangjie	BBDI
U+811F.0	fFourCorner	72243
U+811F.0	fCantonese	lyut3; lyun2; lyun5; lyut6
U+811F.0	fMandarin	lie4; luan3; le4; pao1; luan2
U+8120.0	fUTF8	脠
U+8120.0	fCNS11643	4-3A3B
U+8120.0	fGB	3-4C73
U+8120.0	fR/S	 130.7; 130.8
U+8120.0	fTotalStrokes	 12; 11
U+8120.0	fCangjie	BNKM
U+8120.0	fFourCorner	72240
U+8120.0	fCantonese	zin1
U+8120.0	fMandarin	shan1; chan1
U+8121.0	fUTF8	脡
U+8121.0	fCNS11643	2-3851
U+8121.0	fBig5	D74D
U+8121.0	fGB	3-4C72
U+8121.0	fR/S	 130.7
U+8121.0	fTotalStrokes	 11
U+8121.0	fCangjie	BNKG
U+8121.0	fFourCorner	72241
U+8121.0	fHakka	  tin3
U+8121.0	fCantonese	ting5 (also); ting2
U+8121.0	fMandarin	ting3
U+8121.0	fEnglish	 strips of dried meat; jerky stiff and straight
U+8122.0	fUTF8	脢
U+8122.0	fCNS11643	2-3855
U+8122.0	fBig5	D751
U+8122.0	fGB	3-4D28
U+8122.0	fR/S	 130.7
U+8122.0	fTotalStrokes	 11
U+8122.0	fCangjie	BOWY
U+8122.0	fFourCorner	78257
U+8122.0	fHakka	  moi2; mi2; moi3; moe2
U+8122.0	fCantonese	mui4; mui6
U+8122.0	fMandarin	mei2; mei4
U+8122.0	fEnglish	flesh
U+8123.0	fUTF8	脣
U+8123.0	fCNS11643	1-5C6A
U+8123.0	fBig5	B2E5
U+8123.0	fGB	E-3960
U+8123.0	fR/S	 130.7
U+8123.0	fTotalStrokes	 11
U+8123.0	fCangjie	MVB
U+8123.0	fFourCorner	71227
U+8123.0	fHakka	 sun2;  shun2
U+8123.0	fCantonese	seon4
U+8123.0	fMandarin	chun2; min3
U+8123.0	fEnglish	the lips; the labia
U+8124.0	fUTF8	脤
U+8124.0	fCNS11643	1-5C6E
U+8124.0	fBig5	B2E9
U+8124.0	fGB	3-4C7D
U+8124.0	fR/S	 130.7
U+8124.0	fTotalStrokes	 11
U+8124.0	fCangjie	BMMV
U+8124.0	fFourCorner	71232
U+8124.0	fHakka	 shin6; sin5; shin5
U+8124.0	fCantonese	san5; san6; san4
U+8124.0	fMandarin	shen4
U+8124.0	fEnglish	[1] raw meat for a sacrifice [2] the buttocks; the bottom
U+8125.0	fUTF8	脥
U+8125.0	fCNS11643	2-384A
U+8125.0	fBig5	D746
U+8125.0	fGB	3-4C6C
U+8125.0	fR/S	 130.7
U+8125.0	fTotalStrokes	 11
U+8125.0	fCangjie	BKOO
U+8125.0	fFourCorner	74238
U+8125.0	fHakka	 kiap8; hiap8
U+8125.0	fCantonese	him2; gaap3; hip3
U+8125.0	fMandarin	qian3; xie2; qu1; qie4; jia2
U+8126.0	fUTF8	脦
U+8126.0	fCNS11643	4-3447
U+8126.0	fGB	3-4D21
U+8126.0	fR/S	  130.7
U+8126.0	fTotalStrokes	 11
U+8126.0	fCangjie	BIPP
U+8126.0	fFourCorner	73240
U+8126.0	fCantonese	tik1
U+8126.0	fMandarin	te4
U+8126.0	fEnglish	used in "lede"; slovenly
U+8127.0	fUTF8	脧
U+8127.0	fCNS11643	2-3853
U+8127.0	fBig5	D74F
U+8127.0	fGB	E-3961
U+8127.0	fR/S	 130.7
U+8127.0	fTotalStrokes	 11
U+8127.0	fCangjie	BICE
U+8127.0	fFourCorner	73247
U+8127.0	fHakka	 zoi1;  zui1;  zun5; zien1; zun1
U+8127.0	fCantonese	zeoi1; ceoi1; zyun1; zeon1; syun1
U+8127.0	fMandarin	juan1; zui1; xuan1; zui3; zuo1
U+8127.0	fEnglish	 reduce; contract; exploit reproductive organ of infant
U+8128.0	fUTF8	脨
U+8128.0	fCNS11643	4-3450
U+8128.0	fGB	3-4C7A
U+8128.0	fR/S	  130.7
U+8128.0	fTotalStrokes	 11
U+8128.0	fCangjie	BDL
U+8128.0	fFourCorner	75290
U+8128.0	fCantonese	cuk1
U+8128.0	fMandarin	cu4
U+8129.0	fUTF8	脩
U+8129.0	fCNS11643	1-5C6C
U+8129.0	fBig5	B2E7
U+8129.0	fGB	8-2C78
U+8129.0	fR/S	 130.7
U+8129.0	fTotalStrokes	 11
U+8129.0	fCangjie	OLOB
U+8129.0	fFourCorner	28227
U+8129.0	fHakka	  siu1
U+8129.0	fCantonese	sau1
U+8129.0	fMandarin	xiu1; you1; tiao2
U+8129.0	fEnglish	[1] do; act; restore; regulate; cultivate [2] dried meat [3] teachers payment in ancient times); salary for a teacher [4] a Chinese family name
U+812A.0	fUTF8	脪
U+812A.0	fCNS11643	3-3841
U+812A.0	fGB	3-4D29
U+812A.0	fR/S	 130.7
U+812A.0	fTotalStrokes	 11
U+812A.0	fCangjie	BKKB
U+812A.0	fFourCorner	74220
U+812A.0	fHakka	 hi1; hiun3; hiun5
U+812A.0	fCantonese	jan3
U+812A.0	fMandarin	xin4
U+812A.0	fEnglish	 Erysipelas sloughing of an ulcer
U+812B.0	fUTF8	脫
U+812B.0	fCNS11643	1-5C6B
U+812B.0	fBig5	B2E6
U+812B.0	fGB	E-3962
U+812B.0	fR/S	 130.7
U+812B.0	fTotalStrokes	 11
U+812B.0	fCangjie	BCRU
U+812B.0	fFourCorner	78216
U+812B.0	fHakka	 tot7
U+812B.0	fCantonese	tyut3; teoi3
U+812B.0	fMandarin	tuo1; tui4; duo2; tuo3
U+812B.0	fEnglish	[1] take off; peel off; strip; undress; take off [2] abandon; renounce; cast off [3] leave; escape from; get out of [4] omit; omission; miss out [5] slip off [6] if; in case; perhaps
U+812C.0	fUTF8	脬
U+812C.0	fCNS11643	2-384F
U+812C.0	fBig5	D74B
U+812C.0	fGB	0-6B63
U+812C.0	fR/S	 130.7
U+812C.0	fTotalStrokes	 11
U+812C.0	fCangjie	BBND
U+812C.0	fFourCorner	72147
U+812C.0	fHakka	  pau1
U+812C.0	fCantonese	paau1
U+812C.0	fMandarin	pao1; bao1
U+812C.0	fEnglish	the bladder
U+812D.0	fUTF8	脭
U+812D.0	fCNS11643	2-384D
U+812D.0	fBig5	D749
U+812D.0	fGB	3-4D26
U+812D.0	fR/S	 130.7
U+812D.0	fTotalStrokes	 11
U+812D.0	fCangjie	BRHG
U+812D.0	fFourCorner	76214
U+812D.0	fCantonese	cing4
U+812D.0	fMandarin	cheng2
U+812E.0	fUTF8	脮
U+812E.0	fCNS11643	3-3842
U+812E.0	fGB	3-4D2B
U+812E.0	fR/S	  130.7
U+812E.0	fTotalStrokes	 11
U+812E.0	fCangjie	BBV
U+812E.0	fFourCorner	72240
U+812E.0	fCantonese	neoi5; leoi5 (also)
U+812E.0	fMandarin	nei3
U+812F.0	fUTF8	脯
U+812F.0	fCNS11643	1-5C68
U+812F.0	fBig5	B2E3
U+812F.0	fGB	0-382C
U+812F.0	fR/S	 130.7
U+812F.0	fTotalStrokes	 11
U+812F.0	fCangjie	BIJB
U+812F.0	fFourCorner	73227
U+812F.0	fHakka	  pu3; bu3
U+812F.0	fCantonese	fu2; pou2 (also); pou4
U+812F.0	fMandarin	fu3; bu4
U+812F.0	fEnglish	[1] dried and seasoned meat [2] preserved fruits
U+812F.1	fUTF8	脯
U+812F.1	fCNS11643	1-5C68
U+812F.1	fBig5	B2E3
U+812F.1	fGB	0-382C
U+812F.1	fR/S	 130.7
U+812F.1	fTotalStrokes	 11
U+812F.1	fCangjie	BIJB
U+812F.1	fFourCorner	73227
U+812F.1	fHakka	 pu2
U+812F.1	fMandarin	pu2
U+812F.1	fEnglish	flesh or meat in the general area of the chest or breast
U+8130.0	fUTF8	脰
U+8130.0	fCNS11643	1-5C6D
U+8130.0	fBig5	B2E8
U+8130.0	fGB	3-4C7B
U+8130.0	fR/S	 130.7
U+8130.0	fTotalStrokes	 11
U+8130.0	fCangjie	BMRT
U+8130.0	fFourCorner	71218
U+8130.0	fHakka	 teu5; teu6
U+8130.0	fCantonese	dau6
U+8130.0	fMandarin	dou4
U+8130.0	fEnglish	the neck
U+8131.0	fUTF8	脱
U+8131.0	fCNS11643	3-3845
U+8131.0	fGB	0-4D51
U+8131.0	fR/S	 130.7
U+8131.0	fTotalStrokes	 11
U+8131.0	fCangjie	BCRU
U+8131.0	fFourCorner	78210
U+8131.0	fHakka	  tot7
U+8131.0	fCantonese	tyut3
U+8131.0	fMandarin	tuo1; dui4; yue4
U+8131.0	fEnglish	take off
U+8132.0	fUTF8	脲
U+8132.0	fGB	0-6B65
U+8132.0	fR/S	 130.7
U+8132.0	fTotalStrokes	 11
U+8132.0	fCangjie	BSE
U+8132.0	fCantonese	niu6; liu6 (also)
U+8132.0	fMandarin	niao4
U+8132.0	fEnglish	urea
U+8133.0	fUTF8	脳
U+8133.0	fGB	E-3963
U+8133.0	fR/S	 130.7
U+8133.0	fTotalStrokes	 11
U+8133.0	fCangjie	BFUK
U+8133.0	fCantonese	nou5; lou5 (also)
U+8133.0	fMandarin	nao3
U+8133.0	fEnglish	 brain
U+8134.0	fUTF8	脴
U+8134.0	fCNS11643	3-3848
U+8134.0	fGB	5-4F30
U+8134.0	fR/S	 130.7; 74.7
U+8134.0	fTotalStrokes	 11
U+8134.0	fCangjie	BMFR
U+8134.0	fFourCorner	71260
U+8134.0	fCantonese	pei2
U+8134.0	fMandarin	pi3
U+8135.0	fUTF8	脵
U+8135.0	fGB	E-3964
U+8135.0	fR/S	 130.7
U+8135.0	fTotalStrokes	 11
U+8135.0	fCangjie	BRMK
U+8135.0	fCantonese	ng4
U+8135.0	fMandarin	gu3
U+8135.0	fEnglish	 thighs; haunches; rump share
U+8136.0	fUTF8	脶
U+8136.0	fGB	0-6B61
U+8136.0	fR/S	 130.7
U+8136.0	fTotalStrokes	 11
U+8136.0	fCangjie	BROB
U+8136.0	fHakka	 lo2
U+8136.0	fCantonese	lo4
U+8136.0	fMandarin	gua1; luo2
U+8136.0	fEnglish	simplified form of &#x8161;
U+8137.0	fUTF8	脷
U+8137.0	fGB	1-7C49
U+8137.0	fR/S	 130.7
U+8137.0	fTotalStrokes	 11
U+8137.0	fCangjie	BHDN
U+8137.0	fCantonese	lei6
U+8137.0	fMandarin	li4
U+8137.0	fEnglish	 tongue
U+8138.0	fUTF8	脸
U+8138.0	fGB	0-4133
U+8138.0	fR/S	 130.7
U+8138.0	fTotalStrokes	 11
U+8138.0	fCangjie	BOMM
U+8138.0	fHakka	 giam3; liam3; lien1
U+8138.0	fCantonese	lim5
U+8138.0	fMandarin	lian3; jian3; qian1
U+8138.0	fEnglish	simplified form of &#x81C9;
U+8139.0	fUTF8	脹
U+8139.0	fCNS11643	1-614E
U+8139.0	fBig5	B5C8
U+8139.0	fGB	1-554D
U+8139.0	fR/S	 130.8
U+8139.0	fTotalStrokes	 12
U+8139.0	fCangjie	BSMV
U+8139.0	fFourCorner	71232
U+8139.0	fHakka	 zong5; zhong5
U+8139.0	fCantonese	zoeng3
U+8139.0	fMandarin	zhang4; chang2
U+8139.0	fEnglish	[1] full-stomached; glutted [2] swelling of the skin [3] inflate; expand; expansion
U+813A.0	fUTF8	脺
U+813A.0	fCNS11643	2-3F36
U+813A.0	fBig5	DB51
U+813A.0	fGB	3-4D3B
U+813A.0	fR/S	 130.8
U+813A.0	fTotalStrokes	 12
U+813A.0	fCangjie	BYOJ
U+813A.0	fFourCorner	70248
U+813A.0	fCantonese	seoi6; seoi5 (also); ceoi3
U+813A.0	fMandarin	cui4; cu4; sui4
U+813A.0	fEnglish	the pancreas
U+813B.0	fUTF8	脻
U+813B.0	fCNS11643	3-3E3E
U+813B.0	fGB	3-4D30
U+813B.0	fR/S	  130.8
U+813B.0	fTotalStrokes	 12
U+813B.0	fCangjie	BJLO
U+813B.0	fFourCorner	75280
U+813B.0	fCantonese	zip3
U+813B.0	fMandarin	jie1; jie2
U+813C.0	fUTF8	脼
U+813C.0	fCNS11643	4-3A3E
U+813C.0	fGB	3-4C7C
U+813C.0	fR/S	  130.8
U+813C.0	fTotalStrokes	 12
U+813C.0	fCangjie	BMLB
U+813C.0	fFourCorner	71220
U+813C.0	fCantonese	loeng5
U+813C.0	fMandarin	liang3
U+813D.0	fUTF8	脽
U+813D.0	fCNS11643	2-3F34
U+813D.0	fBig5	DB4F
U+813D.0	fGB	3-4D36
U+813D.0	fR/S	 130.8
U+813D.0	fTotalStrokes	 12
U+813D.0	fCangjie	BOG
U+813D.0	fFourCorner	70214
U+813D.0	fHakka	 shui2; zui1; sui2
U+813D.0	fCantonese	seoi4; zeoi1
U+813D.0	fMandarin	shui2; zhou1
U+813D.0	fEnglish	[1] the buttocks [2] a hill in Shansi with a temple to Earth
U+813E.0	fUTF8	脾
U+813E.0	fCNS11643	1-6150
U+813E.0	fBig5	B5CA
U+813E.0	fGB	0-4622
U+813E.0	fR/S	 130.8
U+813E.0	fTotalStrokes	 12
U+813E.0	fCangjie	BHWJ
U+813E.0	fFourCorner	76240
U+813E.0	fHakka	  pi2;  bi1
U+813E.0	fCantonese	pei4; pei2 (also)
U+813E.0	fMandarin	pi2; pai2; bi4; pi4
U+813E.0	fEnglish	[1] the spleen [2] disposition; temper
U+813F.0	fUTF8	脿
U+813F.0	fCNS11643	4-3A47
U+813F.0	fGB	5-4F3D
U+813F.0	fR/S	 130.8
U+813F.0	fTotalStrokes	 12
U+813F.0	fCangjie	BQMV
U+813F.0	fFourCorner	75230
U+813F.0	fCantonese	biu2
U+813F.0	fMandarin	biao1; biao4; biao3
U+8140.0	fUTF8	腀
U+8140.0	fCNS11643	3-3E43
U+8140.0	fGB	5-4E78
U+8140.0	fR/S	  130.8
U+8140.0	fTotalStrokes	 12
U+8140.0	fCangjie	BOMB
U+8140.0	fFourCorner	78220
U+8140.0	fCantonese	leon4
U+8140.0	fMandarin	lun2
U+8141.0	fUTF8	腁
U+8141.0	fCNS11643	3-3E45
U+8141.0	fGB	E-3965
U+8141.0	fR/S	 130.8
U+8141.0	fTotalStrokes	 12
U+8141.0	fCangjie	BYJJ
U+8141.0	fFourCorner	73240
U+8141.0	fCantonese	pin4
U+8141.0	fMandarin	pian2
U+8141.0	fEnglish	callus
U+8142.0	fUTF8	腂
U+8142.0	fCNS11643	3-3E41
U+8142.0	fGB	3-4D32
U+8142.0	fR/S	 130.8
U+8142.0	fTotalStrokes	 12
U+8142.0	fCangjie	BWD
U+8142.0	fFourCorner	76290
U+8142.0	fCantonese	leoi5
U+8142.0	fMandarin	guo4
U+8143.0	fUTF8	腃
U+8143.0	fCNS11643	2-3F2F
U+8143.0	fBig5	DB4A
U+8143.0	fGB	3-4D3D
U+8143.0	fR/S	 130.8
U+8143.0	fTotalStrokes	 12
U+8143.0	fCangjie	BFQU
U+8143.0	fFourCorner	69227
U+8143.0	fCantonese	wai3; kyun4
U+8143.0	fMandarin	kui4; quan1; juan4; quan2
U+8144.0	fUTF8	腄
U+8144.0	fCNS11643	2-4646
U+8144.0	fBig5	DFA1
U+8144.0	fGB	3-4D35
U+8144.0	fR/S	 130.8
U+8144.0	fTotalStrokes	 12
U+8144.0	fCangjie	BHJM
U+8144.0	fFourCorner	72214
U+8144.0	fHakka	 ch'ui2; shui2; cui2; sui2
U+8144.0	fCantonese	ceoi4
U+8144.0	fMandarin	chui2; chuai2; zhui1; hou2; shui2; you2; zhui4
U+8145.0	fUTF8	腅
U+8145.0	fCNS11643	3-3E3D
U+8145.0	fGB	3-4D3E
U+8145.0	fR/S	  130.8
U+8145.0	fTotalStrokes	 12
U+8145.0	fCangjie	BFF
U+8145.0	fFourCorner	79280
U+8145.0	fCantonese	daam6
U+8145.0	fMandarin	dan4
U+8146.0	fUTF8	腆
U+8146.0	fCNS11643	1-614F
U+8146.0	fBig5	B5C9
U+8146.0	fGB	0-4C73
U+8146.0	fR/S	 130.8
U+8146.0	fTotalStrokes	 12
U+8146.0	fCangjie	BTBC
U+8146.0	fFourCorner	75281
U+8146.0	fHakka	  dien3; tien1; tiam2; ten3; den3
U+8146.0	fCantonese	tin2
U+8146.0	fMandarin	tian3
U+8146.0	fEnglish	[1] prosperous; affluence [2] good; virtuous [3] protruding (as the belly, etc.) [4] bashful; blush
U+8147.0	fUTF8	腇
U+8147.0	fCNS11643	2-3F33
U+8147.0	fBig5	DB4E
U+8147.0	fGB	5-4F42
U+8147.0	fR/S	 130.8
U+8147.0	fTotalStrokes	 12
U+8147.0	fCangjie	BHDV
U+8147.0	fFourCorner	72244
U+8147.0	fHakka	 nui3
U+8147.0	fCantonese	neoi5; leoi5 (also)
U+8147.0	fMandarin	nei3; nei2
U+8148.0	fUTF8	腈
U+8148.0	fCNS11643	3-3E40
U+8148.0	fGB	0-6B66
U+8148.0	fR/S	 130.8
U+8148.0	fTotalStrokes	 12
U+8148.0	fCangjie	BQMB
U+8148.0	fFourCorner	75220
U+8148.0	fHakka	  ziang1;  zin1
U+8148.0	fCantonese	cing1
U+8148.0	fMandarin	jing1
U+8148.0	fEnglish	an organic compound
U+8149.0	fUTF8	腉
U+8149.0	fCNS11643	3-3E42
U+8149.0	fGB	5-4F43
U+8149.0	fR/S	  130.8
U+8149.0	fTotalStrokes	 12
U+8149.0	fCangjie	BHXU
U+8149.0	fFourCorner	77210
U+8149.0	fHakka	 ngi2
U+8149.0	fCantonese	naai4; laai4 (also)
U+8149.0	fMandarin	jie1
U+8149.0	fEnglish	[1] nipple [2] breast
U+814A.0	fUTF8	腊
U+814A.0	fCNS11643	2-3F30
U+814A.0	fBig5	DB4B
U+814A.0	fGB	0-4030
U+814A.0	fR/S	 130.8
U+814A.0	fTotalStrokes	 12
U+814A.0	fCangjie	BTA
U+814A.0	fFourCorner	74261
U+814A.0	fHakka	 sit7
U+814A.0	fCantonese	sik1; lip6 (also)
U+814A.0	fMandarin	xi1; xi2
U+814A.0	fEnglish	[1] dried meat [2] very; extremely
U+814A.1	fUTF8	腊
U+814A.1	fCNS11643	2-3F30
U+814A.1	fBig5	DB4B
U+814A.1	fGB	0-4030
U+814A.1	fR/S	 130.8
U+814A.1	fTotalStrokes	 12
U+814A.1	fCangjie	BTA
U+814A.1	fFourCorner	74261
U+814A.1	fHakka	 lap8; liap8
U+814A.1	fCantonese	laap6
U+814A.1	fMandarin	la4; lie4
U+814A.1	fEnglish	simplified form of &#x81D8;
U+814B.0	fUTF8	腋
U+814B.0	fCNS11643	1-614B
U+814B.0	fBig5	B5C5
U+814B.0	fGB	0-5238
U+814B.0	fR/S	 130.8
U+814B.0	fTotalStrokes	 12
U+814B.0	fCangjie	BYOK
U+814B.0	fFourCorner	70247
U+814B.0	fHakka	  jit8; rit8; kiap8
U+814B.0	fCantonese	jik6; jat6; jit6
U+814B.0	fMandarin	ye4; yi4
U+814B.0	fEnglish	the armpits; the part under the forelegs of animals
U+814C.0	fUTF8	腌
U+814C.0	fCNS11643	1-6151
U+814C.0	fBig5	B5CB
U+814C.0	fGB	0-6B67
U+814C.0	fR/S	 130.8
U+814C.0	fTotalStrokes	 12
U+814C.0	fCangjie	BKLU
U+814C.0	fFourCorner	74216
U+814C.0	fHakka	 jap7; jam1; rap7; ram1
U+814C.0	fCantonese	jim1; jip3 (also); ap1 (also); ngap1 (also)
U+814C.0	fMandarin	a1; ang1; yan1; an1
U+814C.0	fEnglish	unclean; dirty; filthy
U+814C.1	fUTF8	腌
U+814C.1	fCNS11643	1-6151
U+814C.1	fBig5	B5CB
U+814C.1	fGB	0-6B67
U+814C.1	fR/S	 130.8
U+814C.1	fTotalStrokes	 12
U+814C.1	fCangjie	BKLU
U+814C.1	fFourCorner	74216
U+814C.1	fMandarin	yan11}
U+814C.1	fEnglish	 salt; pickle
U+814D.0	fUTF8	腍
U+814D.0	fCNS11643	2-3F35
U+814D.0	fBig5	DB50
U+814D.0	fGB	3-4D39
U+814D.0	fR/S	 130.8
U+814D.0	fTotalStrokes	 12
U+814D.0	fCangjie	BOIP
U+814D.0	fFourCorner	78232
U+814D.0	fHakka	  ngim3; nem2; lem2; jim3
U+814D.0	fCantonese	nam4 (also); nam5; jam6 (also); lam5 (also); lam4 (also); nam6
U+814D.0	fMandarin	ren3; chen2; ren4; dian4
U+814D.0	fEnglish	soft; tender (food); easy-going (people)
U+814E.0	fUTF8	腎
U+814E.0	fCNS11643	1-614D
U+814E.0	fBig5	B5C7
U+814E.0	fGB	1-4976
U+814E.0	fR/S	 130.8
U+814E.0	fTotalStrokes	 12
U+814E.0	fCangjie	SEB
U+814E.0	fFourCorner	77227
U+814E.0	fHakka	 sin3;  shin5; shin6; siin5
U+814E.0	fCantonese	san5; san6 (also); san2 (also)
U+814E.0	fMandarin	shen4
U+814E.0	fEnglish	[1] the kidneys [2] the testicles
U+814F.0	fUTF8	腏
U+814F.0	fCNS11643	2-3F32
U+814F.0	fBig5	DB4D
U+814F.0	fGB	3-4D43
U+814F.0	fR/S	 130.8
U+814F.0	fTotalStrokes	 12
U+814F.0	fCangjie	BEEE
U+814F.0	fFourCorner	77247
U+814F.0	fCantonese	zyut3
U+814F.0	fMandarin	chuo4; zhui4; duo2; sui4
U+8150.0	fUTF8	腐
U+8150.0	fCNS11643	1-6A4F
U+8150.0	fBig5	BB47
U+8150.0	fGB	0-382F
U+8150.0	fR/S	 130.8
U+8150.0	fTotalStrokes	 14
U+8150.0	fCangjie	IOIB
U+8150.0	fFourCorner	00227
U+8150.0	fHakka	  fu3;  fu5; fu6
U+8150.0	fCantonese	fu6; fu2
U+8150.0	fMandarin	fu3
U+8150.0	fEnglish	[1] rot; decay; spoil; rotten; putrid; disintegrate; stale [2] corrupt; evil; worthless [3] old; worn-out; useless; worthless [4] castrate; castration (punishment) [5] short for bean curd
U+8150.1	fUTF8	腐
U+8150.1	fCNS11643	1-6A4F
U+8150.1	fBig5	BB47
U+8150.1	fGB	0-382F
U+8150.1	fR/S	 130.8
U+8150.1	fTotalStrokes	 14
U+8150.1	fCangjie	IOIB
U+8150.1	fFourCorner	00227
U+8150.1	fMandarin	fu5
U+8151.0	fUTF8	腑
U+8151.0	fCNS11643	1-614C
U+8151.0	fBig5	B5C6
U+8151.0	fGB	0-382D
U+8151.0	fR/S	 130.8
U+8151.0	fTotalStrokes	 12
U+8151.0	fCangjie	BIOI
U+8151.0	fFourCorner	70240
U+8151.0	fHakka	  fu3
U+8151.0	fCantonese	fu2
U+8151.0	fMandarin	fu3
U+8151.0	fEnglish	the bowels; entrails; internal organ; viscera
U+8152.0	fUTF8	腒
U+8152.0	fCNS11643	2-3F31
U+8152.0	fBig5	DB4C
U+8152.0	fGB	3-4D41
U+8152.0	fR/S	 130.8
U+8152.0	fTotalStrokes	 12
U+8152.0	fCangjie	BSJR
U+8152.0	fFourCorner	77264
U+8152.0	fHakka	 gi1
U+8152.0	fCantonese	geoi1; keoi4
U+8152.0	fMandarin	ju1; ju4; qu2
U+8152.0	fEnglish	game
U+8153.0	fUTF8	腓
U+8153.0	fCNS11643	1-6152
U+8153.0	fBig5	B5CC
U+8153.0	fGB	0-6B68
U+8153.0	fR/S	 130.8
U+8153.0	fTotalStrokes	 12
U+8153.0	fCangjie	BLMY
U+8153.0	fFourCorner	71211
U+8153.0	fHakka	  fui1;  pui2; fi1; pi2; fui2
U+8153.0	fCantonese	fei4
U+8153.0	fMandarin	fei2
U+8153.0	fEnglish	[1] the calf (of the legs) [2] sick; ill [3] avoid [4] decay
U+8154.0	fUTF8	腔
U+8154.0	fCNS11643	1-614A
U+8154.0	fBig5	B5C4
U+8154.0	fGB	0-473B
U+8154.0	fR/S	 130.8
U+8154.0	fTotalStrokes	 12
U+8154.0	fCangjie	BJCM
U+8154.0	fFourCorner	73211
U+8154.0	fHakka	  kiong1
U+8154.0	fCantonese	hong1
U+8154.0	fMandarin	qiang1; kong4
U+8154.0	fEnglish	[1] chest and belly cavity; [2] a cavity in any vessel [3] tune [4] an accent of one's pronunciation; a tone of one's voice [5] manner
U+8155.0	fUTF8	腕
U+8155.0	fCNS11643	1-6149
U+8155.0	fBig5	B5C3
U+8155.0	fGB	0-4D73
U+8155.0	fR/S	 130.8
U+8155.0	fTotalStrokes	 12
U+8155.0	fCangjie	BJNU
U+8155.0	fFourCorner	73212
U+8155.0	fHakka	 won3; wan3; von3; van3
U+8155.0	fCantonese	wun2
U+8155.0	fMandarin	wan4
U+8155.0	fEnglish	the wrist
U+8156.0	fUTF8	腖
U+8156.0	fGB	1-6B4B
U+8156.0	fR/S	 130.8
U+8156.0	fTotalStrokes	 12
U+8156.0	fCangjie	BDW
U+8156.0	fFourCorner	7523
U+8156.0	fCantonese	dung3
U+8156.0	fMandarin	dong4
U+8156.0	fEnglish	peptone
U+8157.0	fUTF8	腗
U+8157.0	fGB	E-3966
U+8157.0	fR/S	 130.8
U+8157.0	fTotalStrokes	 12
U+8157.0	fCangjie	XBLWL
U+8157.0	fCantonese	pei4
U+8157.0	fMandarin	pi2; pai2; pi4
U+8158.0	fUTF8	腘
U+8158.0	fCNS11643	6-435A
U+8158.0	fGB	8-7C55
U+8158.0	fR/S	 130.11; 130.8; 130.7
U+8158.0	fTotalStrokes	 12; 11
U+8158.0	fCangjie	BWMI
U+8158.0	fFourCorner	76200
U+8158.0	fCantonese	gwok3; gok3 (also)
U+8158.0	fMandarin	guo2
U+8159.0	fUTF8	腙
U+8159.0	fGB	0-6B6A
U+8159.0	fR/S	 130.8
U+8159.0	fTotalStrokes	 12
U+8159.0	fCangjie	BJMF
U+8159.0	fCantonese	zung1
U+8159.0	fMandarin	zong1
U+8159.0	fEnglish	an organic compound
U+815A.0	fUTF8	腚
U+815A.0	fGB	0-6B6B
U+815A.0	fR/S	 130.8
U+815A.0	fTotalStrokes	 12
U+815A.0	fCangjie	BJMO
U+815A.0	fFourCorner	7328
U+815A.0	fCantonese	ding6; ding3
U+815A.0	fMandarin	ding4
U+815A.0	fEnglish	buttock
U+815B.0	fUTF8	腛
U+815B.0	fCNS11643	2-463E
U+815B.0	fBig5	DF77
U+815B.0	fGB	3-4D5A
U+815B.0	fR/S	 130.9
U+815B.0	fTotalStrokes	 13
U+815B.0	fCangjie	BSMG
U+815B.0	fFourCorner	77214
U+815B.0	fHakka	 jo5
U+815B.0	fCantonese	ak1; ngak1 (also)
U+815B.0	fMandarin	wu1; wo4
U+815C.0	fUTF8	腜
U+815C.0	fCNS11643	2-463C
U+815C.0	fBig5	DF75
U+815C.0	fGB	3-4D46
U+815C.0	fR/S	 130.9
U+815C.0	fTotalStrokes	 13
U+815C.0	fCangjie	BTMD
U+815C.0	fFourCorner	74294
U+815C.0	fHakka	  moi2
U+815C.0	fCantonese	mui4
U+815C.0	fMandarin	mei2
U+815C.0	fEnglish	quickening of the foetus
U+815D.0	fUTF8	腝
U+815D.0	fCNS11643	3-437B
U+815D.0	fGB	3-4D49
U+815D.0	fR/S	  130.9
U+815D.0	fTotalStrokes	 13
U+815D.0	fCangjie	BMBK
U+815D.0	fFourCorner	71230
U+815D.0	fHakka	 ngion3
U+815D.0	fCantonese	jyun5; ji4
U+815D.0	fMandarin	ruan3; er2; nao4
U+815D.0	fEnglish	a disease of the feet
U+815E.0	fUTF8	腞
U+815E.0	fCNS11643	2-4642
U+815E.0	fBig5	DF7B
U+815E.0	fGB	5-4F56
U+815E.0	fR/S	 130.9
U+815E.0	fTotalStrokes	 13
U+815E.0	fCangjie	BVNO
U+815E.0	fFourCorner	77232
U+815E.0	fCantonese	syun6
U+815E.0	fMandarin	zhuan4; dun4; tu2; chuan3
U+815F.0	fUTF8	腟
U+815F.0	fCNS11643	3-437E
U+815F.0	fGB	5-4F54
U+815F.0	fR/S	  130.9
U+815F.0	fTotalStrokes	 13
U+815F.0	fCangjie	BJMG
U+815F.0	fFourCorner	73210
U+815F.0	fCantonese	cat1
U+815F.0	fMandarin	chi4; zhi4
U+815F.0	fEnglish	vagina
U+8160.0	fUTF8	腠
U+8160.0	fCNS11643	2-463A
U+8160.0	fBig5	DF73
U+8160.0	fGB	0-6B6D
U+8160.0	fR/S	 130.9
U+8160.0	fTotalStrokes	 13
U+8160.0	fCangjie	BQKK
U+8160.0	fFourCorner	75234
U+8160.0	fHakka	  ceu5
U+8160.0	fCantonese	cau3; zau3 (also)
U+8160.0	fMandarin	cou4
U+8160.0	fEnglish	the texture of muscle
U+8161.0	fUTF8	腡
U+8161.0	fCNS11643	2-4647
U+8161.0	fBig5	DFA2
U+8161.0	fGB	1-6B61
U+8161.0	fR/S	 130.9
U+8161.0	fTotalStrokes	 13
U+8161.0	fCangjie	BBBR
U+8161.0	fFourCorner	77227
U+8161.0	fHakka	 lo2; lo3
U+8161.0	fCantonese	lo4; gwaa1
U+8161.0	fMandarin	gua1; luo2
U+8161.0	fEnglish	lines
U+8162.0	fUTF8	腢
U+8162.0	fCNS11643	2-463F
U+8162.0	fBig5	DF78
U+8162.0	fGB	5-4F4F
U+8162.0	fR/S	 130.9
U+8162.0	fTotalStrokes	 13
U+8162.0	fCangjie	BWLB
U+8162.0	fFourCorner	76227
U+8162.0	fHakka	 ngi2; ngieu3
U+8162.0	fCantonese	ngau5; au5 (also); jyu4; ngau3
U+8162.0	fMandarin	ou3; ou2; wei2; weng2; yu2
U+8162.0	fEnglish	the collar-bone
U+8163.0	fUTF8	腣
U+8163.0	fCNS11643	3-4375
U+8163.0	fGB	3-4D54
U+8163.0	fR/S	  130.9
U+8163.0	fTotalStrokes	 13
U+8163.0	fCangjie	BYBB
U+8163.0	fFourCorner	70220
U+8163.0	fCantonese	dai3
U+8163.0	fMandarin	di4
U+8164.0	fUTF8	腤
U+8164.0	fCNS11643	2-4639
U+8164.0	fBig5	DF72
U+8164.0	fGB	3-4D53
U+8164.0	fR/S	 130.9
U+8164.0	fTotalStrokes	 13
U+8164.0	fCangjie	BYTA
U+8164.0	fFourCorner	70261
U+8164.0	fHakka	 am1
U+8164.0	fCantonese	am1; am3 (also); ngam1 (also); ngam3 (also)
U+8164.0	fMandarin	an1; an3
U+8164.0	fEnglish	[1] boil flesh [2] make soup
U+8165.0	fUTF8	腥
U+8165.0	fCNS11643	1-6624
U+8165.0	fBig5	B87B
U+8165.0	fGB	0-5048
U+8165.0	fR/S	 130.9
U+8165.0	fTotalStrokes	 13
U+8165.0	fCangjie	BAHM
U+8165.0	fFourCorner	76214
U+8165.0	fHakka	  siang1;  sen1
U+8165.0	fCantonese	sing1; seng1 (also); seng3; sing3
U+8165.0	fMandarin	xing1; xing4
U+8165.0	fEnglish	[1] raw, undress meat [2] offensive smell (especially fish or blood); strong-smelling
U+8166.0	fUTF8	腦
U+8166.0	fCNS11643	1-662A
U+8166.0	fBig5	B8A3
U+8166.0	fGB	1-4454
U+8166.0	fR/S	 130.9
U+8166.0	fTotalStrokes	 13
U+8166.0	fCangjie	BVVW
U+8166.0	fFourCorner	72263
U+8166.0	fHakka	  no3;  lau3; nau3
U+8166.0	fCantonese	nou5; lou5 (also)
U+8166.0	fMandarin	nao3; nao4
U+8166.0	fEnglish	the brain
U+8167.0	fUTF8	腧
U+8167.0	fCNS11643	2-4644
U+8167.0	fBig5	DF7D
U+8167.0	fGB	0-6B72
U+8167.0	fR/S	 130.9
U+8167.0	fTotalStrokes	 13
U+8167.0	fCangjie	BOMN
U+8167.0	fFourCorner	28221
U+8167.0	fCantonese	syu6 (also); syu3
U+8167.0	fMandarin	shu4; yu2
U+8168.0	fUTF8	腨
U+8168.0	fCNS11643	4-4029
U+8168.0	fGB	E-3967
U+8168.0	fR/S	  130.9
U+8168.0	fTotalStrokes	 13
U+8168.0	fCangjie	BUMB
U+8168.0	fFourCorner	72220
U+8168.0	fCantonese	syun6
U+8168.0	fMandarin	chuan3; shuan4
U+8169.0	fUTF8	腩
U+8169.0	fCNS11643	2-463D
U+8169.0	fBig5	DF76
U+8169.0	fGB	0-6B6E
U+8169.0	fR/S	 130.9
U+8169.0	fTotalStrokes	 13
U+8169.0	fCangjie	BJBJ
U+8169.0	fFourCorner	74227
U+8169.0	fHakka	 nam3; lam5; nam5; ngem5; nam1; nem2
U+8169.0	fCantonese	naam5; laam5 (also); naam4
U+8169.0	fMandarin	nan3; tan2
U+8169.0	fEnglish	[1] tender beef [2] boil meat [3] dried meat
U+816A.0	fUTF8	腪
U+816A.0	fCNS11643	3-4376
U+816A.0	fGB	3-4C79
U+816A.0	fR/S	  130.9
U+816A.0	fTotalStrokes	 13
U+816A.0	fCangjie	BBJJ
U+816A.0	fFourCorner	77250
U+816A.0	fCantonese	wan6
U+816A.0	fMandarin	yun4
U+816B.0	fUTF8	腫
U+816B.0	fCNS11643	1-6627
U+816B.0	fBig5	B87E
U+816B.0	fGB	1-5657
U+816B.0	fR/S	 130.9
U+816B.0	fTotalStrokes	 13
U+816B.0	fCangjie	BHJG
U+816B.0	fFourCorner	72214
U+816B.0	fHakka	  zung3; zhung3
U+816B.0	fCantonese	zung2
U+816B.0	fMandarin	zhong3
U+816B.0	fEnglish	 swell; swollen; swelling; boil
U+816C.0	fUTF8	腬
U+816C.0	fCNS11643	3-4377
U+816C.0	fGB	3-4D5B
U+816C.0	fR/S	  130.9
U+816C.0	fTotalStrokes	 13
U+816C.0	fCangjie	BNHD
U+816C.0	fFourCorner	77290
U+816C.0	fHakka	 ju2; riu2
U+816C.0	fCantonese	lau6 (also); nau6
U+816C.0	fMandarin	rou2
U+816C.0	fEnglish	[1] fat; juicy [2] abundant
U+816D.0	fUTF8	腭
U+816D.0	fCNS11643	3-4423
U+816D.0	fGB	0-6B71
U+816D.0	fR/S	 130.9; 130.12
U+816D.0	fTotalStrokes	 13; 16
U+816D.0	fCangjie	BRRS
U+816D.0	fFourCorner	76220
U+816D.0	fHakka	 ngok7; ngok8; ngong1; long1
U+816D.0	fCantonese	ngok6; ok6 (also)
U+816D.0	fMandarin	e4
U+816D.0	fEnglish	the roof of the mouth; the palate
U+816E.0	fUTF8	腮
U+816E.0	fCNS11643	1-6625
U+816E.0	fBig5	B87C
U+816E.0	fGB	0-4879
U+816E.0	fR/S	 130.9
U+816E.0	fTotalStrokes	 13
U+816E.0	fCangjie	BWP
U+816E.0	fFourCorner	76230
U+816E.0	fHakka	 soi1; sii1; sui1
U+816E.0	fCantonese	soi1
U+816E.0	fMandarin	sai1
U+816E.0	fEnglish	the cheeks
U+816F.0	fUTF8	腯
U+816F.0	fCNS11643	2-4645
U+816F.0	fBig5	DF7E
U+816F.0	fGB	3-4D51
U+816F.0	fR/S	 130.9
U+816F.0	fTotalStrokes	 13
U+816F.0	fCangjie	BHJU
U+816F.0	fFourCorner	72264
U+816F.0	fHakka	  tun3; tut7
U+816F.0	fCantonese	dat1; dat6; dyut1 (also); teon5
U+816F.0	fMandarin	tu2; dun4
U+816F.0	fEnglish	strong
U+8170.0	fUTF8	腰
U+8170.0	fCNS11643	1-6622
U+8170.0	fBig5	B879
U+8170.0	fGB	0-517C
U+8170.0	fR/S	 130.9
U+8170.0	fTotalStrokes	 13
U+8170.0	fCangjie	BMWV
U+8170.0	fFourCorner	71244
U+8170.0	fHakka	  jau1; rau1; jeu1
U+8170.0	fCantonese	jiu1
U+8170.0	fMandarin	yao1; yao4
U+8170.0	fEnglish	[1] the waist; midriff [2] the kidneys [3] the middle of something; the waist portion of a region
U+8171.0	fUTF8	腱
U+8171.0	fCNS11643	1-6621
U+8171.0	fBig5	B878
U+8171.0	fGB	0-6B6C
U+8171.0	fR/S	 130.9
U+8171.0	fTotalStrokes	 13
U+8171.0	fCangjie	BNKQ
U+8171.0	fFourCorner	75240
U+8171.0	fHakka	 ken5; kien6; kien5; gen5; kian5
U+8171.0	fCantonese	gin3; gin1; gin6
U+8171.0	fMandarin	jian4; qian2; jian1; jian3; jin1
U+8171.0	fEnglish	tendons
U+8172.0	fUTF8	腲
U+8172.0	fCNS11643	2-4640
U+8172.0	fBig5	DF79
U+8172.0	fGB	3-4D4E
U+8172.0	fR/S	 130.9
U+8172.0	fTotalStrokes	 13
U+8172.0	fCangjie	BWMV
U+8172.0	fFourCorner	76232
U+8172.0	fCantonese	wai2
U+8172.0	fMandarin	wei3
U+8173.0	fUTF8	腳
U+8173.0	fCNS11643	1-6626
U+8173.0	fBig5	B87D
U+8173.0	fGB	E-3968
U+8173.0	fR/S	 130.9
U+8173.0	fTotalStrokes	 13
U+8173.0	fCangjie	BCRL
U+8173.0	fFourCorner	77220
U+8173.0	fHakka	  giok7; ka2
U+8173.0	fCantonese	goek3; goek2 (also); gok3
U+8173.0	fMandarin	jiao3; jue2
U+8173.0	fEnglish	[1] foot; feet [2] base; foundation
U+8174.0	fUTF8	腴
U+8174.0	fCNS11643	1-6153
U+8174.0	fBig5	B5CD
U+8174.0	fGB	0-6B69
U+8174.0	fR/S	 130.9
U+8174.0	fTotalStrokes	 13
U+8174.0	fCangjie	BHXO
U+8174.0	fFourCorner	77237
U+8174.0	fHakka	 ji2; ri2; i2
U+8174.0	fCantonese	jyu4
U+8174.0	fMandarin	yu2
U+8174.0	fEnglish	[1] fat [2] fertile [3] plump; soft [4] rich [5] intestines of dogs and hogs
U+8175.0	fUTF8	腵
U+8175.0	fCNS11643	3-437C
U+8175.0	fGB	3-4D59
U+8175.0	fR/S	  130.9
U+8175.0	fTotalStrokes	 13
U+8175.0	fCangjie	BRSE
U+8175.0	fFourCorner	77240
U+8175.0	fCantonese	gaa1
U+8175.0	fMandarin	jia1
U+8176.0	fUTF8	腶
U+8176.0	fCNS11643	2-4643
U+8176.0	fBig5	DF7C
U+8176.0	fGB	5-4F51
U+8176.0	fR/S	 130.9
U+8176.0	fTotalStrokes	 13
U+8176.0	fCangjie	BHJE
U+8176.0	fFourCorner	77247
U+8176.0	fHakka	 ton5; ton6
U+8176.0	fCantonese	dyun3
U+8176.0	fMandarin	duan4
U+8176.0	fEnglish	meat
U+8177.0	fUTF8	腷
U+8177.0	fCNS11643	2-463B
U+8177.0	fBig5	DF74
U+8177.0	fGB	3-4D48
U+8177.0	fR/S	 130.9
U+8177.0	fTotalStrokes	 13
U+8177.0	fCangjie	BMRW
U+8177.0	fFourCorner	71266
U+8177.0	fCantonese	bik1
U+8177.0	fMandarin	bi4
U+8178.0	fUTF8	腸
U+8178.0	fCNS11643	1-6623
U+8178.0	fBig5	B87A
U+8178.0	fGB	1-3326
U+8178.0	fR/S	 130.9
U+8178.0	fTotalStrokes	 13
U+8178.0	fCangjie	BAMH
U+8178.0	fFourCorner	76227
U+8178.0	fHakka	  cong2;  chong2; chong3
U+8178.0	fCantonese	coeng4; coeng2 (also)
U+8178.0	fMandarin	chang2
U+8178.0	fEnglish	the intestines; the bowels
U+8179.0	fUTF8	腹
U+8179.0	fCNS11643	1-6628
U+8179.0	fBig5	B8A1
U+8179.0	fGB	0-3839
U+8179.0	fR/S	 130.9
U+8179.0	fTotalStrokes	 13
U+8179.0	fCangjie	BOAE
U+8179.0	fFourCorner	78247
U+8179.0	fHakka	  buk7; fuk8
U+8179.0	fCantonese	fuk1
U+8179.0	fMandarin	fu4
U+8179.0	fEnglish	[1] stomach; belly; abdomen; under the chest [2] inside; inner [3] the front part
U+817A.0	fUTF8	腺
U+817A.0	fCNS11643	1-6629
U+817A.0	fBig5	B8A2
U+817A.0	fGB	0-4F59
U+817A.0	fR/S	 130.9
U+817A.0	fTotalStrokes	 13
U+817A.0	fCangjie	BHAE
U+817A.0	fFourCorner	76232
U+817A.0	fHakka	 sen5
U+817A.0	fCantonese	sin3
U+817A.0	fMandarin	xian4
U+817A.0	fEnglish	 gland
U+817B.0	fUTF8	腻
U+817B.0	fGB	0-4465
U+817B.0	fR/S	 130.9
U+817B.0	fTotalStrokes	 13
U+817B.0	fCangjie	BIPO
U+817B.0	fHakka	 ngi5; ngie5
U+817B.0	fCantonese	nei6; lei6 (also)
U+817B.0	fMandarin	ni4
U+817B.0	fEnglish	simplified form of &#x81A9;
U+817C.0	fUTF8	腼
U+817C.0	fGB	0-6B6F
U+817C.0	fR/S	 130.9
U+817C.0	fTotalStrokes	 13
U+817C.0	fCangjie	BMWL
U+817C.0	fFourCorner	7126
U+817C.0	fCantonese	min5; tin2
U+817C.0	fMandarin	mian3
U+817C.0	fEnglish	[1] shy; bashful [2] quiet and graceful (girls)
U+817D.0	fUTF8	腽
U+817D.0	fCNS11643	3-4422
U+817D.0	fGB	0-6B70
U+817D.0	fR/S	 130.9
U+817D.0	fTotalStrokes	 13
U+817D.0	fCangjie	BABT
U+817D.0	fFourCorner	76210
U+817D.0	fCantonese	wat1
U+817D.0	fMandarin	wa4
U+817D.0	fEnglish	fat
U+817E.0	fUTF8	腾
U+817E.0	fGB	0-4C5A
U+817E.0	fR/S	 130.9
U+817E.0	fTotalStrokes	 13
U+817E.0	fCangjie	BFQM
U+817E.0	fHakka	 ten2; tin2
U+817E.0	fCantonese	tang4
U+817E.0	fMandarin	teng2
U+817E.0	fEnglish	simplified form of &#x9A30;
U+817E.1	fUTF8	腾
U+817E.1	fGB	0-4C5A
U+817E.1	fR/S	 130.9
U+817E.1	fTotalStrokes	 13
U+817E.1	fCangjie	BFQM
U+817E.1	fMandarin	teng5
U+817F.0	fUTF8	腿
U+817F.0	fCNS11643	1-6A54
U+817F.0	fBig5	BB4C
U+817F.0	fGB	0-4D48
U+817F.0	fR/S	 130.10
U+817F.0	fTotalStrokes	 14
U+817F.0	fCangjie	BYAV
U+817F.0	fFourCorner	77233
U+817F.0	fHakka	  tui3
U+817F.0	fCantonese	teoi2
U+817F.0	fMandarin	tui3
U+817F.0	fEnglish	the legs and the thighs
U+8180.0	fUTF8	膀
U+8180.0	fCNS11643	1-6A50
U+8180.0	fBig5	BB48
U+8180.0	fGB	0-3072
U+8180.0	fR/S	 130.10
U+8180.0	fTotalStrokes	 14
U+8180.0	fCangjie	BYBS
U+8180.0	fFourCorner	70227
U+8180.0	fHakka	  pong2; p'ong2pong3; pong3; bong3
U+8180.0	fCantonese	bong2; pong4; pong1; bong3
U+8180.0	fMandarin	bang3; pang1; bang4; pang3
U+8180.0	fEnglish	[1] the upper arms [2] [3] very full stomach [4] [5] legs
U+8180.1	fUTF8	膀
U+8180.1	fCNS11643	1-6A50
U+8180.1	fBig5	BB48
U+8180.1	fGB	0-3072
U+8180.1	fR/S	 130.10
U+8180.1	fTotalStrokes	 14
U+8180.1	fCangjie	BYBS
U+8180.1	fFourCorner	70227
U+8180.1	fMandarin	pang2
U+8180.1	fEnglish	the bladder
U+8181.0	fUTF8	膁
U+8181.0	fCNS11643	3-4927
U+8181.0	fGB	3-4D61
U+8181.0	fR/S	  130.10
U+8181.0	fTotalStrokes	 14
U+8181.0	fCangjie	BTXC
U+8181.0	fFourCorner	78230
U+8181.0	fHakka	 kiam3; jam3; ram3; hiam1
U+8181.0	fCantonese	him2; hip3 (also); jim2
U+8181.0	fMandarin	qian1; qian3; xian4; lian2
U+8181.0	fEnglish	小膁, loins, side of body
U+8182.0	fUTF8	膂
U+8182.0	fCNS11643	1-6A55
U+8182.0	fBig5	BB4D
U+8182.0	fGB	0-6B76
U+8182.0	fR/S	 130.10
U+8182.0	fTotalStrokes	 14
U+8182.0	fCangjie	YVB
U+8182.0	fFourCorner	08227
U+8182.0	fHakka	 li3
U+8182.0	fCantonese	leoi5; leoi3
U+8182.0	fMandarin	l3; luu3; lu3
U+8182.0	fEnglish	[1] the backbone; spinal column [2] one's physical strength
U+8183.0	fUTF8	膃
U+8183.0	fCNS11643	2-4D2D
U+8183.0	fBig5	E3A6
U+8183.0	fGB	E-3969
U+8183.0	fR/S	 130.10
U+8183.0	fTotalStrokes	 14
U+8183.0	fCangjie	BWOT
U+8183.0	fFourCorner	76217
U+8183.0	fHakka	 wut7; vut7
U+8183.0	fCantonese	wat1
U+8183.0	fMandarin	wa4
U+8183.0	fEnglish	fat
U+8184.0	fUTF8	膄
U+8184.0	fCNS11643	3-492A
U+8184.0	fGB	5-4F52
U+8184.0	fR/S	  130.10
U+8184.0	fTotalStrokes	 14
U+8184.0	fCangjie	BHXE
U+8184.0	fFourCorner	77240
U+8184.0	fCantonese	sau3
U+8184.0	fMandarin	sou4; shou4
U+8184.0	fEnglish	to become emaciated
U+8185.0	fUTF8	膅
U+8185.0	fCNS11643	4-4639
U+8185.0	fGB	E-396A
U+8185.0	fR/S	  130.10
U+8185.0	fTotalStrokes	 14
U+8185.0	fCangjie	BILR
U+8185.0	fFourCorner	70260
U+8185.0	fCantonese	tong4
U+8185.0	fMandarin	tang2
U+8186.0	fUTF8	膆
U+8186.0	fCNS11643	2-4D2C
U+8186.0	fBig5	E3A5
U+8186.0	fGB	5-4F57
U+8186.0	fR/S	 130.10
U+8186.0	fTotalStrokes	 14
U+8186.0	fCangjie	BQMF
U+8186.0	fFourCorner	75293
U+8186.0	fHakka	 su5; so5; sii5
U+8186.0	fCantonese	sou3; seoi3 (also)
U+8186.0	fMandarin	su4
U+8186.0	fEnglish	[1] the crop of a bird or fowl [2] fat
U+8187.0	fUTF8	膇
U+8187.0	fCNS11643	2-4D2E
U+8187.0	fBig5	E3A7
U+8187.0	fGB	3-4D50
U+8187.0	fR/S	 130.10
U+8187.0	fTotalStrokes	 14
U+8187.0	fCangjie	BYHR
U+8187.0	fFourCorner	77237
U+8187.0	fHakka	 chui5; zui5; zhui5
U+8187.0	fCantonese	zeoi6; deoi3
U+8187.0	fMandarin	zhui4
U+8187.0	fEnglish	swell
U+8188.0	fUTF8	膈
U+8188.0	fCNS11643	1-6A52
U+8188.0	fBig5	BB4A
U+8188.0	fGB	0-6B75
U+8188.0	fR/S	 130.10
U+8188.0	fTotalStrokes	 14
U+8188.0	fCangjie	BMRB
U+8188.0	fFourCorner	71227
U+8188.0	fHakka	  gak7
U+8188.0	fCantonese	gaak3
U+8188.0	fMandarin	ge2
U+8188.0	fEnglish	the diaphragm
U+8189.0	fUTF8	膉
U+8189.0	fCNS11643	2-4D2B
U+8189.0	fBig5	E3A4
U+8189.0	fGB	3-4D60
U+8189.0	fR/S	 130.10
U+8189.0	fTotalStrokes	 14
U+8189.0	fCangjie	BTCT
U+8189.0	fFourCorner	78217
U+8189.0	fCantonese	jik1
U+8189.0	fMandarin	yi4
U+8189.0	fEnglish	the smell of oily food that's gone bad
U+818A.0	fUTF8	膊
U+818A.0	fCNS11643	1-6A53
U+818A.0	fBig5	BB4B
U+818A.0	fGB	0-3232
U+818A.0	fR/S	 130.10
U+818A.0	fTotalStrokes	 14
U+818A.0	fCangjie	BIBI
U+818A.0	fFourCorner	73242
U+818A.0	fHakka	 bok7
U+818A.0	fCantonese	bok3
U+818A.0	fMandarin	bo2; lie4; po4; bo5; chun2; fu1; pei1
U+818A.0	fEnglish	the shoulders; upper arms
U+818B.0	fUTF8	膋
U+818B.0	fCNS11643	2-4D31
U+818B.0	fBig5	E3AA
U+818B.0	fGB	3-4E2B
U+818B.0	fR/S	 130.10
U+818B.0	fTotalStrokes	 14
U+818B.0	fCangjie	FFBB
U+818B.0	fFourCorner	99227
U+818B.0	fHakka	 liau2; lau2; liau5
U+818B.0	fCantonese	liu4
U+818B.0	fMandarin	liao2
U+818B.0	fEnglish	fat
U+818C.0	fUTF8	膌
U+818C.0	fCNS11643	2-4D30
U+818C.0	fBig5	E3A9
U+818C.0	fGB	5-4F5F
U+818C.0	fR/S	 130.10
U+818C.0	fTotalStrokes	 14
U+818C.0	fCangjie	BFCB
U+818C.0	fFourCorner	74227
U+818C.0	fCantonese	zek3; zik3 (also); zik6
U+818C.0	fMandarin	ji2; ji1
U+818C.0	fEnglish	become emaciated
U+818D.0	fUTF8	膍
U+818D.0	fCNS11643	2-4D2F
U+818D.0	fBig5	E3A8
U+818D.0	fGB	5-4F5C
U+818D.0	fR/S	 130.10
U+818D.0	fTotalStrokes	 14
U+818D.0	fCangjie	BHWP
U+818D.0	fFourCorner	76211
U+818D.0	fHakka	  pi2
U+818D.0	fCantonese	pei4; fu4
U+818D.0	fMandarin	pi2
U+818D.0	fEnglish	stomach of an animal
U+818E.0	fUTF8	膎
U+818E.0	fCNS11643	4-4646
U+818E.0	fGB	3-4D5F
U+818E.0	fR/S	  130.10
U+818E.0	fTotalStrokes	 14
U+818E.0	fCangjie	BBVK
U+818E.0	fFourCorner	72230
U+818E.0	fCantonese	haai4
U+818E.0	fMandarin	xie2
U+818F.0	fUTF8	膏
U+818F.0	fCNS11643	1-6A51
U+818F.0	fBig5	BB49
U+818F.0	fGB	0-3860
U+818F.0	fR/S	 130.10
U+818F.0	fTotalStrokes	 14
U+818F.0	fCangjie	YRBB
U+818F.0	fFourCorner	00227
U+818F.0	fHakka	  gau1; go1; go2
U+818F.0	fCantonese	gou1; gou3
U+818F.0	fMandarin	gao1
U+818F.0	fEnglish	[1] grease; fat [2] paste; ointment [3] fertile [4] the region just below the heart [5] grace; favours [6] sweet [7] a pest-like preparation for external use [8] plaster [9] cooked to a very thick or pastry form (food and fruit)
U+818F.1	fUTF8	膏
U+818F.1	fCNS11643	1-6A51
U+818F.1	fBig5	BB49
U+818F.1	fGB	0-3860
U+818F.1	fR/S	 130.10
U+818F.1	fTotalStrokes	 14
U+818F.1	fCangjie	YRBB
U+818F.1	fFourCorner	00227
U+818F.1	fMandarin	gao4
U+818F.1	fEnglish	[1] lubricate; grease; make smooth or glossy [2] enrich; freshen
U+8190.0	fUTF8	膐
U+8190.0	fCNS11643	4-5267
U+8190.0	fGB	E-396B
U+8190.0	fR/S	  130.10
U+8190.0	fTotalStrokes	 16
U+8190.0	fCangjie	YVOBO
U+8190.0	fFourCorner	08220
U+8190.0	fCantonese	leoi5
U+8190.0	fMandarin	luu3; l3
U+8191.0	fUTF8	膑
U+8191.0	fGB	0-6B77
U+8191.0	fR/S	 130.10
U+8191.0	fTotalStrokes	 14
U+8191.0	fCangjie	BJOC
U+8191.0	fHakka	 bin5
U+8191.0	fCantonese	ban3
U+8191.0	fMandarin	bin4; pin2; bin3
U+8191.0	fEnglish	simplified form of &#x81CF;
U+8192.0	fUTF8	膒
U+8192.0	fCNS11643	3-4E3B
U+8192.0	fGB	3-4C47
U+8192.0	fR/S	  130.11
U+8192.0	fTotalStrokes	 15
U+8192.0	fCangjie	BSRR
U+8192.0	fFourCorner	71210
U+8192.0	fCantonese	au1; ngau1 (also)
U+8192.0	fMandarin	ou1
U+8193.0	fUTF8	膓
U+8193.0	fCNS11643	3-4E3D
U+8193.0	fGB	E-396C
U+8193.0	fR/S	  130.11
U+8193.0	fTotalStrokes	 15
U+8193.0	fCangjie	BOAH
U+8193.0	fFourCorner	78220
U+8193.0	fCantonese	coeng4
U+8193.0	fMandarin	chang2
U+8193.0	fEnglish	 intestines emotions sausage
U+8194.0	fUTF8	膔
U+8194.0	fCNS11643	3-4E39
U+8194.0	fGB	5-4F65
U+8194.0	fR/S	  130.11
U+8194.0	fTotalStrokes	 15
U+8194.0	fCangjie	BIXP
U+8194.0	fFourCorner	70210
U+8194.0	fCantonese	luk6
U+8194.0	fMandarin	lu4
U+8195.0	fUTF8	膕
U+8195.0	fCNS11643	2-532A
U+8195.0	fBig5	E741
U+8195.0	fGB	3-4D33
U+8195.0	fR/S	 130.11
U+8195.0	fTotalStrokes	 15
U+8195.0	fCangjie	BWIM
U+8195.0	fFourCorner	76200
U+8195.0	fHakka	 kwet7; kwet8
U+8195.0	fCantonese	gwok3; gok3 (also); gwok6
U+8195.0	fMandarin	guo2
U+8195.0	fEnglish	hollow
U+8196.0	fUTF8	膖
U+8196.0	fCNS11643	3-4E3C
U+8196.0	fGB	E-396D
U+8196.0	fR/S	  130.11
U+8196.0	fTotalStrokes	 15
U+8196.0	fCangjie	BYHJ
U+8196.0	fFourCorner	77230
U+8196.0	fCantonese	pong1
U+8196.0	fMandarin	pang1
U+8197.0	fUTF8	膗
U+8197.0	fCNS11643	2-532D
U+8197.0	fBig5	E744
U+8197.0	fGB	3-4D67
U+8197.0	fR/S	 130.11
U+8197.0	fTotalStrokes	 15
U+8197.0	fCangjie	BUOG
U+8197.0	fFourCorner	72214
U+8197.0	fHakka	 shui2; sui2
U+8197.0	fCantonese	ceoi4 (also); saai4
U+8197.0	fMandarin	chuai2; chuai1
U+8197.0	fEnglish	 ugly and fat too fat to move
U+8198.0	fUTF8	膘
U+8198.0	fCNS11643	1-6E51
U+8198.0	fBig5	BDA8
U+8198.0	fGB	0-316C
U+8198.0	fR/S	 130.11
U+8198.0	fTotalStrokes	 15
U+8198.0	fCangjie	BMWF
U+8198.0	fFourCorner	71291
U+8198.0	fHakka	 piau5; biau1; piau3; peu5; piau1
U+8198.0	fCantonese	biu1
U+8198.0	fMandarin	biao1; piao3; biao4; jiao3; piao1
U+8198.0	fEnglish	 fat rump
U+8199.0	fUTF8	膙
U+8199.0	fCNS11643	2-532C
U+8199.0	fBig5	E743
U+8199.0	fGB	3-4D79
U+8199.0	fR/S	 130.11
U+8199.0	fTotalStrokes	 15
U+8199.0	fCangjie	BNII
U+8199.0	fFourCorner	73236
U+8199.0	fCantonese	koeng5; zoeng2 (also); koeng6
U+8199.0	fMandarin	jiang3
U+8199.0	fEnglish	callous skin on the feet
U+819A.0	fUTF8	膚
U+819A.0	fCNS11643	1-6E50
U+819A.0	fBig5	BDA7
U+819A.0	fGB	1-3774
U+819A.0	fR/S	 130.11
U+819A.0	fTotalStrokes	 15
U+819A.0	fCangjie	YPWB
U+819A.0	fFourCorner	21227
U+819A.0	fHakka	  fu1
U+819A.0	fCantonese	fu1
U+819A.0	fMandarin	fu1; lu2
U+819A.0	fEnglish	[1] the skin; surface [2] skin-deep; superficial; shallow [3] great (achievements; merit)
U+819B.0	fUTF8	膛
U+819B.0	fCNS11643	1-6E4C
U+819B.0	fBig5	BDA3
U+819B.0	fGB	0-4C45
U+819B.0	fR/S	 130.11
U+819B.0	fTotalStrokes	 15
U+819B.0	fCangjie	BFBG
U+819B.0	fFourCorner	79214
U+819B.0	fHakka	 tong2; tong3
U+819B.0	fCantonese	tong4
U+819B.0	fMandarin	tang2; tang1
U+819B.0	fEnglish	[1] the chest; breast [2] hollow space; cavity [3] the chamber of a firearm
U+819C.0	fUTF8	膜
U+819C.0	fCNS11643	1-6E4D
U+819C.0	fBig5	BDA4
U+819C.0	fGB	0-4424
U+819C.0	fR/S	 130.11
U+819C.0	fTotalStrokes	 15
U+819C.0	fCangjie	BTAK
U+819C.0	fFourCorner	74234
U+819C.0	fHakka	  mok8; mok7
U+819C.0	fCantonese	mok6; mou4; mok2 (also)
U+819C.0	fMandarin	mo4; mu2
U+819C.0	fEnglish	[1] membrane [2] film; thin coating
U+819C.1	fUTF8	膜
U+819C.1	fCNS11643	1-6E4D
U+819C.1	fBig5	BDA4
U+819C.1	fGB	0-4424
U+819C.1	fR/S	 130.11
U+819C.1	fTotalStrokes	 15
U+819C.1	fCangjie	BTAK
U+819C.1	fFourCorner	74234
U+819C.1	fMandarin	mo2
U+819C.1	fEnglish	 kneel and worship
U+819D.0	fUTF8	膝
U+819D.0	fCNS11643	1-6E4E
U+819D.0	fBig5	BDA5
U+819D.0	fGB	0-4F25
U+819D.0	fR/S	 130.11
U+819D.0	fTotalStrokes	 15
U+819D.0	fCangjie	BDOE
U+819D.0	fFourCorner	74232
U+819D.0	fHakka	  cit7
U+819D.0	fCantonese	sat1
U+819D.0	fMandarin	xi1
U+819D.0	fEnglish	the knee
U+819E.0	fUTF8	膞
U+819E.0	fCNS11643	2-5329
U+819E.0	fBig5	E740
U+819E.0	fGB	3-4C46
U+819E.0	fR/S	 130.11
U+819E.0	fTotalStrokes	 15
U+819E.0	fCangjie	BJII
U+819E.0	fFourCorner	75043
U+819E.0	fHakka	 bok7; pok7
U+819E.0	fCantonese	zyun1; seon4
U+819E.0	fMandarin	zhuan1; chun2; chuan2; chuan4; zhuan3
U+819E.0	fEnglish	[1] the upper arm [2] the shoulder [3] cut out; slice
U+819F.0	fUTF8	膟
U+819F.0	fCNS11643	2-5328
U+819F.0	fBig5	E6FE
U+819F.0	fGB	3-4D6A
U+819F.0	fR/S	 130.11
U+819F.0	fTotalStrokes	 15
U+819F.0	fCangjie	BYIJ
U+819F.0	fFourCorner	70243
U+819F.0	fHakka	  lut8
U+819F.0	fCantonese	leot6 (also); leot3
U+819F.0	fMandarin	l4; luu4; lu4; shuai4
U+819F.0	fEnglish	fat
U+81A0.0	fUTF8	膠
U+81A0.0	fCNS11643	1-6E4F
U+81A0.0	fBig5	BDA6
U+81A0.0	fGB	1-3D3A
U+81A0.0	fR/S	 130.11
U+81A0.0	fTotalStrokes	 15
U+81A0.0	fCangjie	BSMH
U+81A0.0	fFourCorner	77222
U+81A0.0	fHakka	  gau1
U+81A0.0	fCantonese	gaau1
U+81A0.0	fMandarin	jiao1; jiao3; hao2; jiao4; nao3; qiao1
U+81A0.0	fEnglish	[1] glue; gum [2] resin; sap [3] anything sticky [4] rubber; plastics [5] stick on; stick together; adhere [6] stubborn; obstinate [7] a Chinese family name
U+81A1.0	fUTF8	膡
U+81A1.0	fCNS11643	3-4E3A
U+81A1.0	fGB	3-4D6B
U+81A1.0	fR/S	  130.11
U+81A1.0	fTotalStrokes	 15
U+81A1.0	fCangjie	BFQU
U+81A1.0	fFourCorner	79260
U+81A1.0	fCantonese	jing6
U+81A1.0	fMandarin	ying4
U+81A2.0	fUTF8	膢
U+81A2.0	fCNS11643	2-532B
U+81A2.0	fBig5	E742
U+81A2.0	fGB	3-4D57
U+81A2.0	fR/S	 130.11
U+81A2.0	fTotalStrokes	 15
U+81A2.0	fCangjie	BLLV
U+81A2.0	fFourCorner	75246
U+81A2.0	fHakka	 leu2
U+81A2.0	fCantonese	leoi4; lau4 (also)
U+81A2.0	fMandarin	liu2; lou2; l2; luu2; lu2
U+81A3.0	fUTF8	膣
U+81A3.0	fCNS11643	2-5327
U+81A3.0	fBig5	E6FD
U+81A3.0	fGB	0-6B79
U+81A3.0	fR/S	 130.11
U+81A3.0	fTotalStrokes	 15
U+81A3.0	fCangjie	BJCG
U+81A3.0	fFourCorner	73214
U+81A3.0	fHakka	 zhi1; zih1; zi1
U+81A3.0	fCantonese	zat6
U+81A3.0	fMandarin	zhi4
U+81A3.0	fEnglish	the vagina
U+81A4.0	fUTF8	膤
U+81A4.0	fGB	E-396E
U+81A4.0	fR/S	 74.11; 130.11
U+81A4.0	fTotalStrokes	 15
U+81A4.0	fCangjie	BMBS
U+81A4.0	fCantonese	syut3
U+81A4.0	fMandarin	xue3
U+81A5.0	fUTF8	膥
U+81A5.0	fGB	1-7C5B
U+81A5.0	fR/S	 130.12
U+81A5.0	fTotalStrokes	 18
U+81A5.0	fCangjie	DSOBO
U+81A5.0	fHakka	 cun1; chun1; {lon3}; {chun1}
U+81A5.0	fCantonese	ceon1
U+81A5.0	fMandarin	chun1
U+81A5.0	fEnglish	 eggs of birds or reptiles testicles
U+81A6.0	fUTF8	膦
U+81A6.0	fCNS11643	2-5955
U+81A6.0	fBig5	EAE9
U+81A6.0	fGB	0-6C22
U+81A6.0	fR/S	 130.12
U+81A6.0	fTotalStrokes	 16
U+81A6.0	fCangjie	BFDQ
U+81A6.0	fFourCorner	79259
U+81A6.0	fCantonese	leon4 (also); leon6
U+81A6.0	fMandarin	lian3; lin4
U+81A7.0	fUTF8	膧
U+81A7.0	fCNS11643	2-595F
U+81A7.0	fBig5	EAF3
U+81A7.0	fGB	E-396F
U+81A7.0	fR/S	 130.12
U+81A7.0	fTotalStrokes	 16
U+81A7.0	fCangjie	BYTG
U+81A7.0	fFourCorner	70214
U+81A7.0	fCantonese	tung4; cong4
U+81A7.0	fMandarin	tong2; chuang2
U+81A8.0	fUTF8	膨
U+81A8.0	fCNS11643	1-7179
U+81A8.0	fBig5	BFB1
U+81A8.0	fGB	0-4572
U+81A8.0	fR/S	 130.12
U+81A8.0	fTotalStrokes	 16
U+81A8.0	fCangjie	BGTH
U+81A8.0	fFourCorner	72222
U+81A8.0	fHakka	  pang2
U+81A8.0	fCantonese	paang4; gu2
U+81A8.0	fMandarin	peng2; beng4; peng4
U+81A8.0	fEnglish	 swell; expand; inflate
U+81A9.0	fUTF8	膩
U+81A9.0	fCNS11643	1-7178
U+81A9.0	fBig5	BFB0
U+81A9.0	fGB	1-4465
U+81A9.0	fR/S	 130.12
U+81A9.0	fTotalStrokes	 16
U+81A9.0	fCangjie	BIPC
U+81A9.0	fFourCorner	73240
U+81A9.0	fHakka	 ngi5; li3; ni5; ne5; ngi6; ngie5
U+81A9.0	fCantonese	nau6 (also); lau6 (also); nei6; lei6 (also)
U+81A9.0	fMandarin	ni4
U+81A9.0	fEnglish	[1] fatty; greasy (food); oily [2] dirty [3] smooth [4] bored; tired; weary [5] intimate (friends)
U+81AA.0	fUTF8	膪
U+81AA.0	fCNS11643	3-5249
U+81AA.0	fGB	0-6B7A
U+81AA.0	fR/S	  130.12
U+81AA.0	fTotalStrokes	 16
U+81AA.0	fCangjie	BYBR
U+81AA.0	fFourCorner	70260
U+81AA.0	fHakka	 za5; mai5
U+81AA.0	fCantonese	zaai3; zaa6; ceoi2
U+81AA.0	fMandarin	zha4; zhai4; chuai4
U+81AA.0	fEnglish	pork
U+81AB.0	fUTF8	膫
U+81AB.0	fCNS11643	2-5959
U+81AB.0	fBig5	EAED
U+81AB.0	fGB	3-4D70
U+81AB.0	fR/S	 130.12
U+81AB.0	fTotalStrokes	 16
U+81AB.0	fCangjie	BKCF
U+81AB.0	fFourCorner	24296
U+81AB.0	fHakka	 liau2
U+81AB.0	fCantonese	liu4
U+81AB.0	fMandarin	liao2; lao2; liao4
U+81AB.0	fEnglish	 the fat covering the intestines the omentum
U+81AC.0	fUTF8	膬
U+81AC.0	fCNS11643	2-595B
U+81AC.0	fBig5	EAEF
U+81AC.0	fGB	5-4F6C
U+81AC.0	fR/S	 130.12
U+81AC.0	fTotalStrokes	 16
U+81AC.0	fCangjie	BHUU
U+81AC.0	fFourCorner	72215
U+81AC.0	fHakka	 cui5
U+81AC.0	fCantonese	ceoi3
U+81AC.0	fMandarin	cui4
U+81AC.0	fEnglish	crisp; brittle
U+81AD.0	fUTF8	膭
U+81AD.0	fCNS11643	3-5247
U+81AD.0	fGB	3-4D4F
U+81AD.0	fR/S	  130.12
U+81AD.0	fTotalStrokes	 16
U+81AD.0	fCangjie	BLMC
U+81AD.0	fFourCorner	75280
U+81AD.0	fCantonese	gwai1
U+81AD.0	fMandarin	gui1
U+81AE.0	fUTF8	膮
U+81AE.0	fCNS11643	2-5956
U+81AE.0	fBig5	EAEA
U+81AE.0	fGB	3-4C6D
U+81AE.0	fR/S	 130.12
U+81AE.0	fTotalStrokes	 16
U+81AE.0	fCangjie	BGGU
U+81AE.0	fFourCorner	74211
U+81AE.0	fHakka	 hiau1; hieu1
U+81AE.0	fCantonese	hiu1; hiu6
U+81AE.0	fMandarin	xiao1; xiao3; xiu1
U+81AE.0	fEnglish	[1] pork soup [2] savoury
U+81AF.0	fUTF8	膯
U+81AF.0	fCNS11643	3-5244
U+81AF.0	fGB	3-4D7A
U+81AF.0	fR/S	  130.12
U+81AF.0	fTotalStrokes	 16
U+81AF.0	fCangjie	BNOT
U+81AF.0	fFourCorner	72210
U+81AF.0	fCantonese	tang1
U+81AF.0	fMandarin	teng1
U+81B0.0	fUTF8	膰
U+81B0.0	fCNS11643	2-595A
U+81B0.0	fBig5	EAEE
U+81B0.0	fGB	3-4D75
U+81B0.0	fR/S	 130.12
U+81B0.0	fTotalStrokes	 16
U+81B0.0	fCangjie	BHDW
U+81B0.0	fFourCorner	72269
U+81B0.0	fHakka	 fan1
U+81B0.0	fCantonese	faan4
U+81B0.0	fMandarin	fan2; pan2; po2
U+81B0.0	fEnglish	cooked meat for sacrifice or offering
U+81B1.0	fUTF8	膱
U+81B1.0	fCNS11643	2-5954
U+81B1.0	fBig5	EAE8
U+81B1.0	fGB	E-3970
U+81B1.0	fR/S	 130.12
U+81B1.0	fTotalStrokes	 16
U+81B1.0	fCangjie	BYIA
U+81B1.0	fFourCorner	73250
U+81B1.0	fHakka	 zhit7; tsiit7
U+81B1.0	fCantonese	zik1; jik1
U+81B1.0	fMandarin	zhi2; zhu1
U+81B1.0	fEnglish	[1] dried or pickled meat [2] greasy
U+81B2.0	fUTF8	膲
U+81B2.0	fCNS11643	2-595D
U+81B2.0	fBig5	EAF1
U+81B2.0	fGB	3-4D73
U+81B2.0	fR/S	 130.12
U+81B2.0	fTotalStrokes	 16
U+81B2.0	fCangjie	BOGF
U+81B2.0	fFourCorner	70231
U+81B2.0	fHakka	 ziau1
U+81B2.0	fCantonese	ziu1
U+81B2.0	fMandarin	jiao1
U+81B2.0	fEnglish	the three divisions of the viscera (med.)
U+81B3.0	fUTF8	膳
U+81B3.0	fCNS11643	1-7177
U+81B3.0	fBig5	BFAF
U+81B3.0	fGB	0-4945
U+81B3.0	fR/S	 130.12
U+81B3.0	fTotalStrokes	 16
U+81B3.0	fCangjie	BTGR
U+81B3.0	fFourCorner	78265
U+81B3.0	fHakka	 shan5; sen3; san5; sen5; shen5
U+81B3.0	fCantonese	sin6
U+81B3.0	fMandarin	shan4
U+81B3.0	fEnglish	meals; provisions; food
U+81B4.0	fUTF8	膴
U+81B4.0	fCNS11643	2-595C
U+81B4.0	fBig5	EAF0
U+81B4.0	fGB	3-4C45
U+81B4.0	fR/S	 130.12
U+81B4.0	fTotalStrokes	 16
U+81B4.0	fCangjie	BOTF
U+81B4.0	fFourCorner	78231
U+81B4.0	fHakka	 fu2; wu2; vu3
U+81B4.0	fCantonese	fu1; mou5
U+81B4.0	fMandarin	hu1; wu2; mei2; wu3; mu2; xu3
U+81B4.0	fEnglish	fine
U+81B5.0	fUTF8	膵
U+81B5.0	fCNS11643	2-5958
U+81B5.0	fBig5	EAEC
U+81B5.0	fGB	3-4D64
U+81B5.0	fR/S	 130.12
U+81B5.0	fTotalStrokes	 16
U+81B5.0	fCangjie	BTYJ
U+81B5.0	fFourCorner	74248
U+81B5.0	fCantonese	seoi6; seoi5 (also); ceoi3
U+81B5.0	fMandarin	cui4
U+81B5.0	fEnglish	pancreas
U+81B6.0	fUTF8	膶
U+81B6.0	fGB	1-7C4A
U+81B6.0	fR/S	 74.12; 130.12
U+81B6.0	fTotalStrokes	 16
U+81B6.0	fCangjie	BANG
U+81B6.0	fCantonese	jeon2; jeon6 (also)
U+81B6.0	fMandarin	run4
U+81B6.0	fEnglish	 liver (except human liver)
U+81B7.0	fUTF8	膷
U+81B7.0	fCNS11643	2-595E
U+81B7.0	fBig5	EAF2
U+81B7.0	fGB	3-4C43
U+81B7.0	fR/S	 130.13
U+81B7.0	fTotalStrokes	 17
U+81B7.0	fCangjie	BVHL
U+81B7.0	fFourCorner	77227
U+81B7.0	fHakka	  hiong1
U+81B7.0	fCantonese	hoeng1
U+81B7.0	fMandarin	xiang1; xin4; xiang4
U+81B7.0	fEnglish	soup
U+81B8.0	fUTF8	膸
U+81B8.0	fCNS11643	3-5623
U+81B8.0	fGB	5-4F6A
U+81B8.0	fR/S	  130.13
U+81B8.0	fTotalStrokes	 17
U+81B8.0	fCangjie	BYKB
U+81B8.0	fFourCorner	74230
U+81B8.0	fCantonese	seoi5
U+81B8.0	fMandarin	sui3
U+81B9.0	fUTF8	膹
U+81B9.0	fCNS11643	2-5957
U+81B9.0	fBig5	EAEB
U+81B9.0	fGB	3-4D45
U+81B9.0	fR/S	 130.13
U+81B9.0	fTotalStrokes	 17
U+81B9.0	fCangjie	BJTC
U+81B9.0	fFourCorner	74286
U+81B9.0	fCantonese	fan5; fei2
U+81B9.0	fMandarin	fen4; fei2; fei4
U+81BA.0	fUTF8	膺
U+81BA.0	fCNS11643	1-747E
U+81BA.0	fBig5	C174
U+81BA.0	fGB	0-625F
U+81BA.0	fR/S	 130.13
U+81BA.0	fTotalStrokes	 17
U+81BA.0	fCangjie	IOGB
U+81BA.0	fFourCorner	00227
U+81BA.0	fHakka	 jin1; rin1
U+81BA.0	fCantonese	jing1
U+81BA.0	fMandarin	ying1; ying4
U+81BA.0	fEnglish	[1] the breast of a person [2] undertake; bear; receive; be given (responsibility, etc.); shoulder; sustain [3] a belt across the breast of a horse [4] punish (the enemy) by war
U+81BB.0	fUTF8	膻
U+81BB.0	fCNS11643	2-5E55
U+81BB.0	fBig5	EDE8
U+81BB.0	fGB	0-6B7E
U+81BB.0	fR/S	 130.13
U+81BB.0	fTotalStrokes	 17
U+81BB.0	fCangjie	BYWM
U+81BB.0	fFourCorner	70216
U+81BB.0	fHakka	 tan3; tan2
U+81BB.0	fCantonese	zin1; saan1 (also); daan6 (also); taan2; sin1
U+81BB.0	fMandarin	dan4; shan1; tan3
U+81BB.0	fEnglish	[1] the bad odor of sheep, deer, etc. [2] the part between the breasts
U+81BC.0	fUTF8	膼
U+81BC.0	fCNS11643	2-5E5B
U+81BC.0	fBig5	EDEE
U+81BC.0	fGB	3-4C6F
U+81BC.0	fR/S	 130.13
U+81BC.0	fTotalStrokes	 17
U+81BC.0	fCangjie	BYBB
U+81BC.0	fFourCorner	77232
U+81BC.0	fHakka	 ko1
U+81BC.0	fCantonese	zaa1; go1
U+81BC.0	fMandarin	zhua1; gua1
U+81BC.0	fEnglish	thigh
U+81BD.0	fUTF8	膽
U+81BD.0	fCNS11643	1-7524
U+81BD.0	fBig5	C178
U+81BD.0	fGB	1-3528
U+81BD.0	fR/S	 130.13
U+81BD.0	fTotalStrokes	 17
U+81BD.0	fCangjie	BNCR
U+81BD.0	fFourCorner	77261
U+81BD.0	fHakka	  dam3
U+81BD.0	fCantonese	daam2
U+81BD.0	fMandarin	dan3
U+81BD.0	fEnglish	[1] the gall [2] bravery; courage; audacity [3] the internal parts (vessel, etc.) [4] the tube of a tire; basketball, etc.
U+81BE.0	fUTF8	膾
U+81BE.0	fCNS11643	1-7526
U+81BE.0	fBig5	C17A
U+81BE.0	fGB	1-6B5A
U+81BE.0	fR/S	 130.13
U+81BE.0	fTotalStrokes	 17
U+81BE.0	fCangjie	BOMA
U+81BE.0	fFourCorner	78266
U+81BE.0	fHakka	  kwai5; kai5
U+81BE.0	fCantonese	kui2; kwui2 (also)
U+81BE.0	fMandarin	kuai4; guai4
U+81BE.0	fEnglish	minced meat
U+81BF.0	fUTF8	膿
U+81BF.0	fCNS11643	1-7523
U+81BF.0	fBig5	C177
U+81BF.0	fGB	1-4527
U+81BF.0	fR/S	 130.13
U+81BF.0	fTotalStrokes	 17
U+81BF.0	fCangjie	BTWV
U+81BF.0	fFourCorner	75232
U+81BF.0	fHakka	 lung2; nung2
U+81BF.0	fCantonese	nung4; lung4 (also)
U+81BF.0	fMandarin	nong2; neng2
U+81BF.0	fEnglish	 pus; purulent matter
U+81C0.0	fUTF8	臀
U+81C0.0	fCNS11643	1-7522
U+81C0.0	fBig5	C176
U+81C0.0	fGB	0-4D4E
U+81C0.0	fR/S	 130.13
U+81C0.0	fTotalStrokes	 17
U+81C0.0	fCangjie	SEB
U+81C0.0	fFourCorner	77227
U+81C0.0	fHakka	  tun2; tun3
U+81C0.0	fCantonese	tyun4
U+81C0.0	fMandarin	tun2
U+81C0.0	fEnglish	[1] the buttocks; the behind; the bottom; the rump [2] the bottom of a ware or vessel
U+81C1.0	fUTF8	臁
U+81C1.0	fCNS11643	4-5827
U+81C1.0	fGB	0-6C21
U+81C1.0	fR/S	  130.13
U+81C1.0	fTotalStrokes	 17; 16
U+81C1.0	fCangjie	BITC
U+81C1.0	fFourCorner	70230
U+81C1.0	fHakka	  liam2
U+81C1.0	fCantonese	lim4
U+81C1.0	fMandarin	lian2
U+81C1.0	fEnglish	calf
U+81C2.0	fUTF8	臂
U+81C2.0	fCNS11643	1-7521
U+81C2.0	fBig5	C175
U+81C2.0	fGB	0-315B
U+81C2.0	fR/S	 130.13
U+81C2.0	fTotalStrokes	 17
U+81C2.0	fCangjie	SJB
U+81C2.0	fFourCorner	70227
U+81C2.0	fHakka	  bi3; bi5; pi6
U+81C2.0	fCantonese	bei3
U+81C2.0	fMandarin	bi4; bei4
U+81C2.0	fEnglish	the arms (human being, tools, machine, etc.)
U+81C2.1	fUTF8	臂
U+81C2.1	fCNS11643	1-7521
U+81C2.1	fBig5	C175
U+81C2.1	fGB	0-315B
U+81C2.1	fR/S	 130.13
U+81C2.1	fTotalStrokes	 17
U+81C2.1	fCangjie	SJB
U+81C2.1	fFourCorner	70227
U+81C2.1	fMandarin	bei5
U+81C3.0	fUTF8	臃
U+81C3.0	fCNS11643	1-747D
U+81C3.0	fBig5	C173
U+81C3.0	fGB	0-5337
U+81C3.0	fR/S	 130.13
U+81C3.0	fTotalStrokes	 17
U+81C3.0	fCangjie	BYVG
U+81C3.0	fFourCorner	70214
U+81C3.0	fHakka	  jung1; rung1
U+81C3.0	fCantonese	jung2; jung1
U+81C3.0	fMandarin	yong1; yong3
U+81C3.0	fEnglish	[1] swell; swelling [2] fat and clumsy
U+81C4.0	fUTF8	臄
U+81C4.0	fCNS11643	2-5E56
U+81C4.0	fBig5	EDE9
U+81C4.0	fGB	3-4D7C
U+81C4.0	fR/S	 130.13
U+81C4.0	fTotalStrokes	 17
U+81C4.0	fCangjie	BYPO
U+81C4.0	fFourCorner	71232
U+81C4.0	fHakka	 kiok7; gi3
U+81C4.0	fCantonese	koek6; kek6; gek6
U+81C4.0	fMandarin	jue2; qu2
U+81C4.0	fEnglish	palate
U+81C5.0	fUTF8	臅
U+81C5.0	fCNS11643	2-5E59
U+81C5.0	fBig5	EDEC
U+81C5.0	fGB	3-4D7D
U+81C5.0	fR/S	 130.13
U+81C5.0	fTotalStrokes	 17
U+81C5.0	fCangjie	BWLI
U+81C5.0	fFourCorner	76227
U+81C5.0	fCantonese	zuk1; zuk6
U+81C5.0	fMandarin	chu4; su4; zhu2
U+81C6.0	fUTF8	臆
U+81C6.0	fCNS11643	1-747C
U+81C6.0	fBig5	C172
U+81C6.0	fGB	0-525C
U+81C6.0	fR/S	 130.13
U+81C6.0	fTotalStrokes	 17
U+81C6.0	fCangjie	BYTP
U+81C6.0	fFourCorner	70236
U+81C6.0	fHakka	 ji5; rit8; jit8; ri5
U+81C6.0	fCantonese	jik1
U+81C6.0	fMandarin	yi4; yi3
U+81C6.0	fEnglish	[1] one's breast, heart, thoughts, etc.; bosom; thought [2] one's personal view or feelings
U+81C7.0	fUTF8	臇
U+81C7.0	fCNS11643	2-5E5A
U+81C7.0	fBig5	EDED
U+81C7.0	fGB	E-3971
U+81C7.0	fR/S	 130.13
U+81C7.0	fTotalStrokes	 17
U+81C7.0	fCangjie	BOGS
U+81C7.0	fFourCorner	70227
U+81C7.0	fHakka	 zun5
U+81C7.0	fCantonese	zeon3; zyun2
U+81C7.0	fMandarin	juan3; zui1
U+81C7.0	fEnglish	 fat; rich a stew of fish
U+81C8.0	fUTF8	臈
U+81C8.0	fCNS11643	3-5624
U+81C8.0	fGB	E-3972
U+81C8.0	fR/S	 130.13
U+81C8.0	fTotalStrokes	 17
U+81C8.0	fCangjie	BTAV
U+81C8.0	fFourCorner	74220
U+81C8.0	fHakka	 lap8
U+81C8.0	fCantonese	laap6; lap6
U+81C8.0	fMandarin	la4; xi1
U+81C8.0	fEnglish	 year end sacrifice dried meat
U+81C9.0	fUTF8	臉
U+81C9.0	fCNS11643	1-7525
U+81C9.0	fBig5	C179
U+81C9.0	fGB	1-4133
U+81C9.0	fR/S	 130.13
U+81C9.0	fTotalStrokes	 17
U+81C9.0	fCangjie	BOMO
U+81C9.0	fFourCorner	78286
U+81C9.0	fHakka	 liam3; lien1;  giam3; lien3; liam5
U+81C9.0	fCantonese	lim5
U+81C9.0	fMandarin	lian3; jian3; qian1
U+81C9.0	fEnglish	the face; reputation
U+81CA.0	fUTF8	臊
U+81CA.0	fCNS11643	2-5E58
U+81CA.0	fBig5	EDEB
U+81CA.0	fGB	0-6B7D
U+81CA.0	fR/S	 130.13
U+81CA.0	fTotalStrokes	 17
U+81CA.0	fCangjie	BRRD
U+81CA.0	fFourCorner	76294
U+81CA.0	fHakka	  sau1; so1
U+81CA.0	fCantonese	sou1; sou3
U+81CA.0	fMandarin	sao1
U+81CA.0	fEnglish	a bad odor or small (decaying fish, meat, fox, sheep, etc)
U+81CA.1	fUTF8	臊
U+81CA.1	fCNS11643	2-5E58
U+81CA.1	fBig5	EDEB
U+81CA.1	fGB	0-6B7D
U+81CA.1	fR/S	 130.13
U+81CA.1	fTotalStrokes	 17
U+81CA.1	fCangjie	BRRD
U+81CA.1	fFourCorner	76294
U+81CA.1	fMandarin	sao4
U+81CA.1	fEnglish	[1] ashamed; bashful [2] minced meat
U+81CB.0	fUTF8	臋
U+81CB.0	fCNS11643	7-487C
U+81CB.0	fGB	E-3973
U+81CB.0	fR/S	 130.13
U+81CB.0	fTotalStrokes	 19
U+81CB.0	fCangjie	SEOBO
U+81CB.0	fFourCorner	77220
U+81CB.0	fCantonese	tyun4
U+81CB.0	fMandarin	tun2
U+81CC.0	fUTF8	臌
U+81CC.0	fCNS11643	2-5E57
U+81CC.0	fBig5	EDEA
U+81CC.0	fGB	0-6B7B
U+81CC.0	fR/S	 130.13
U+81CC.0	fTotalStrokes	 17
U+81CC.0	fCangjie	BGTE
U+81CC.0	fFourCorner	74247
U+81CC.0	fHakka	  gu3
U+81CC.0	fCantonese	gu2; gwu2 (also)
U+81CC.0	fMandarin	gu3
U+81CC.0	fEnglish	 expand; swell; swollen
U+81CD.0	fUTF8	臍
U+81CD.0	fCNS11643	1-772C
U+81CD.0	fBig5	C2C0
U+81CD.0	fGB	1-466A
U+81CD.0	fR/S	 130.14
U+81CD.0	fTotalStrokes	 18
U+81CD.0	fCangjie	BYX
U+81CD.0	fFourCorner	70223
U+81CD.0	fHakka	  ci2; ce2; ce5; ci3
U+81CD.0	fCantonese	ci4
U+81CD.0	fMandarin	qi2
U+81CD.0	fEnglish	[1] abdominal area of a crab [2] navel; umbilicus
U+81CE.0	fUTF8	臎
U+81CE.0	fCNS11643	3-5865
U+81CE.0	fGB	3-4E25
U+81CE.0	fR/S	  130.14
U+81CE.0	fTotalStrokes	 18
U+81CE.0	fCangjie	BSMJ
U+81CE.0	fFourCorner	77240
U+81CE.0	fHakka	 cui5
U+81CE.0	fCantonese	ceoi3
U+81CE.0	fMandarin	cui4; zui3
U+81CE.0	fEnglish	the tail of a bird
U+81CF.0	fUTF8	臏
U+81CF.0	fCNS11643	1-772D
U+81CF.0	fBig5	C2C1
U+81CF.0	fGB	1-6B77
U+81CF.0	fR/S	 130.14
U+81CF.0	fTotalStrokes	 18
U+81CF.0	fCangjie	BJMC
U+81CF.0	fFourCorner	73286
U+81CF.0	fHakka	 bin5
U+81CF.0	fCantonese	ban3; pan5
U+81CF.0	fMandarin	bin4; pin2; bin3
U+81CF.0	fEnglish	[1] the kneecap [2] punishment or removing kneecaps in ancient times
U+81D0.0	fUTF8	臐
U+81D0.0	fCNS11643	2-626B
U+81D0.0	fBig5	F0A1
U+81D0.0	fGB	3-4E22
U+81D0.0	fR/S	 130.14
U+81D0.0	fTotalStrokes	 18
U+81D0.0	fCangjie	BHGF
U+81D0.0	fFourCorner	72231
U+81D0.0	fHakka	 hiun1
U+81D0.0	fCantonese	fan1
U+81D0.0	fMandarin	xun1; xun4
U+81D0.0	fEnglish	broth
U+81D1.0	fUTF8	臑
U+81D1.0	fCNS11643	2-6269
U+81D1.0	fBig5	F07D
U+81D1.0	fGB	3-4E21
U+81D1.0	fR/S	 130.14
U+81D1.0	fTotalStrokes	 18
U+81D1.0	fCangjie	BMBB
U+81D1.0	fFourCorner	71227
U+81D1.0	fHakka	 nau2; ji2; ri2; nau5
U+81D1.0	fCantonese	naau6; jyu4; ji4; laau6 (also); nou6
U+81D1.0	fMandarin	ru2; nao4; er2; nao2; wan3; hun4
U+81D1.0	fEnglish	soft
U+81D2.0	fUTF8	臒
U+81D2.0	fCNS11643	2-626A
U+81D2.0	fBig5	F07E
U+81D2.0	fGB	3-4D7B
U+81D2.0	fR/S	 130.14
U+81D2.0	fTotalStrokes	 18
U+81D2.0	fCangjie	BTOE
U+81D2.0	fFourCorner	74247
U+81D2.0	fCantonese	wok3; wu6
U+81D2.0	fMandarin	huo4; hu4
U+81D3.0	fUTF8	臓
U+81D3.0	fGB	E-3974
U+81D3.0	fR/S	 130.14
U+81D3.0	fTotalStrokes	 18
U+81D3.0	fCangjie	BTIS
U+81D3.0	fCantonese	zong6
U+81D3.0	fMandarin	zang4
U+81D3.0	fEnglish	internal organs; viscera
U+81D4.0	fUTF8	臔
U+81D4.0	fCNS11643	3-5A77
U+81D4.0	fGB	3-4D31
U+81D4.0	fR/S	  130.15
U+81D4.0	fTotalStrokes	 19
U+81D4.0	fCangjie	BSEC
U+81D4.0	fFourCorner	77286
U+81D4.0	fCantonese	jin6
U+81D4.0	fMandarin	xian4
U+81D5.0	fUTF8	臕
U+81D5.0	fCNS11643	2-664C
U+81D5.0	fBig5	F2C2
U+81D5.0	fGB	E-3975
U+81D5.0	fR/S	 130.15
U+81D5.0	fTotalStrokes	 19
U+81D5.0	fCangjie	BIPF
U+81D5.0	fFourCorner	70231
U+81D5.0	fHakka	 biau1;  piau1; beu1
U+81D5.0	fCantonese	biu1
U+81D5.0	fMandarin	biao1; piao3
U+81D5.0	fEnglish	fat
U+81D6.0	fUTF8	臖
U+81D6.0	fCNS11643	3-5A78
U+81D6.0	fGB	3-4E2F
U+81D6.0	fR/S	  130.16
U+81D6.0	fTotalStrokes	 20; 19
U+81D6.0	fCangjie	HCB
U+81D6.0	fFourCorner	77220
U+81D6.0	fCantonese	hing3
U+81D6.0	fMandarin	xing4
U+81D7.0	fUTF8	臗
U+81D7.0	fCNS11643	2-664B
U+81D7.0	fBig5	F2C1
U+81D7.0	fGB	5-4F60
U+81D7.0	fR/S	 130.15; 130.14
U+81D7.0	fTotalStrokes	 19
U+81D7.0	fCangjie	BJTI
U+81D7.0	fFourCorner	73216
U+81D7.0	fCantonese	fun1
U+81D7.0	fMandarin	kuan1; kun1
U+81D8.0	fUTF8	臘
U+81D8.0	fCNS11643	1-7869
U+81D8.0	fBig5	C3BE
U+81D8.0	fGB	1-4030
U+81D8.0	fR/S	 130.15
U+81D8.0	fTotalStrokes	 19
U+81D8.0	fCangjie	BVVV
U+81D8.0	fFourCorner	72217
U+81D8.0	fHakka	  lap8; liap7; liap8
U+81D8.0	fCantonese	laap6; lip6 (also)
U+81D8.0	fMandarin	la4; lie4
U+81D8.0	fEnglish	[1] sacrifice at the end of the lunar year [2] the end of the lunar year [3] salted and smoked meat, fish, chicken, etc. [4] the age of a Buddhist monk
U+81D9.0	fUTF8	臙
U+81D9.0	fCNS11643	2-6961
U+81D9.0	fBig5	F4B4
U+81D9.0	fGB	E-3976
U+81D9.0	fR/S	 130.16
U+81D9.0	fTotalStrokes	 20
U+81D9.0	fCangjie	BTLF
U+81D9.0	fFourCorner	74231
U+81D9.0	fHakka	 jen1; ren1; jan1
U+81D9.0	fCantonese	jin1
U+81D9.0	fMandarin	yan1
U+81D9.0	fEnglish	 rouge; face powder; cosmetics
U+81DA.0	fUTF8	臚
U+81DA.0	fCNS11643	1-7A2F
U+81DA.0	fBig5	C4A4
U+81DA.0	fGB	1-6B4D
U+81DA.0	fR/S	 130.16
U+81DA.0	fTotalStrokes	 20
U+81DA.0	fCangjie	BYPT
U+81DA.0	fFourCorner	71217
U+81DA.0	fHakka	 lu2; lu3
U+81DA.0	fCantonese	lou4
U+81DA.0	fMandarin	lu2; luu2; l3; luu3
U+81DA.0	fEnglish	[1] arrange in order; display; exhibit [2] the belly [3] the skin [4] forward; convey; announce
U+81DB.0	fUTF8	臛
U+81DB.0	fCNS11643	2-6960
U+81DB.0	fBig5	F4B3
U+81DB.0	fGB	3-4E26
U+81DB.0	fR/S	 130.16
U+81DB.0	fTotalStrokes	 20
U+81DB.0	fCangjie	BMBG
U+81DB.0	fFourCorner	71214
U+81DB.0	fHakka	 kok7
U+81DB.0	fCantonese	fok3; kok3 (also); huk1
U+81DB.0	fMandarin	huo4; hu4
U+81DB.0	fEnglish	broth
U+81DC.0	fUTF8	臜
U+81DC.0	fGB	7-2127
U+81DC.0	fR/S	 130.16
U+81DC.0	fTotalStrokes	 20
U+81DC.0	fCangjie	BHUO
U+81DC.0	fCantonese	zim1; zap1 (also)
U+81DC.0	fMandarin	zang1; za1; zan1
U+81DC.0	fEnglish	dirty; filthy
U+81DD.0	fUTF8	臝
U+81DD.0	fCNS11643	2-6B7E
U+81DD.0	fBig5	F5F0
U+81DD.0	fGB	E-3977
U+81DD.0	fR/S	 130.17
U+81DD.0	fTotalStrokes	 21
U+81DD.0	fCangjie	YNWD
U+81DD.0	fFourCorner	00217
U+81DD.0	fHakka	 ko5;  lo3; ko3
U+81DD.0	fCantonese	lo2; lo5
U+81DD.0	fMandarin	luo3
U+81DD.0	fEnglish	 bare; nude
U+81DE.0	fUTF8	臞
U+81DE.0	fCNS11643	2-6D72
U+81DE.0	fBig5	F745
U+81DE.0	fGB	5-4F74
U+81DE.0	fR/S	 130.18
U+81DE.0	fTotalStrokes	 22
U+81DE.0	fCangjie	BBUG
U+81DE.0	fFourCorner	76214
U+81DE.0	fHakka	 ki1; ki3
U+81DE.0	fCantonese	keoi4
U+81DE.0	fMandarin	qu2; ju4
U+81DE.0	fEnglish	[1] thin [2] wasted
U+81DF.0	fUTF8	臟
U+81DF.0	fCNS11643	1-7B70
U+81DF.0	fBig5	C5A6
U+81DF.0	fGB	1-5460
U+81DF.0	fR/S	 130.18
U+81DF.0	fTotalStrokes	 22
U+81DF.0	fCangjie	BTIS
U+81DF.0	fFourCorner	74253
U+81DF.0	fHakka	  cong3;  cong5; cong6
U+81DF.0	fCantonese	zong6
U+81DF.0	fMandarin	zang4
U+81DF.0	fEnglish	the internal organs in the chest and abdomen; viscera
U+81DF.1	fUTF8	臟
U+81DF.1	fCNS11643	1-7B70
U+81DF.1	fBig5	C5A6
U+81DF.1	fGB	1-5460
U+81DF.1	fR/S	 130.18
U+81DF.1	fTotalStrokes	 22
U+81DF.1	fCangjie	BTIS
U+81DF.1	fFourCorner	74253
U+81DF.1	fMandarin	zang1
U+81E0.0	fUTF8	臠
U+81E0.0	fCNS11643	2-7132
U+81E0.0	fBig5	F943
U+81E0.0	fGB	1-5975
U+81E0.0	fR/S	 130.19
U+81E0.0	fTotalStrokes	 25
U+81E0.0	fCangjie	VFOBO
U+81E0.0	fFourCorner	22227
U+81E0.0	fHakka	 lan2
U+81E0.0	fCantonese	lyun5; lyun4 (also); lyun2 (also)
U+81E0.0	fMandarin	luan2; luan3
U+81E0.0	fEnglish	[1] meat chops or cuts; sliced meat [2] lean; thin
U+81E1.0	fUTF8	臡
U+81E1.0	fCNS11643	2-7133
U+81E1.0	fBig5	F944
U+81E1.0	fGB	5-6259
U+81E1.0	fR/S	 130.19
U+81E1.0	fTotalStrokes	 25
U+81E1.0	fCangjie	TGOBO
U+81E1.0	fFourCorner	40227
U+81E1.0	fHakka	 ni2
U+81E1.0	fCantonese	nai4; lai4 (also)
U+81E1.0	fMandarin	ni2; nuo2; ri2
U+81E1.0	fEnglish	pickled, minced meat
U+81E2.0	fUTF8	臢
U+81E2.0	fCNS11643	1-7C44
U+81E2.0	fBig5	C5D8
U+81E2.0	fGB	3-4E27
U+81E2.0	fR/S	 130.19
U+81E2.0	fTotalStrokes	 23
U+81E2.0	fCangjie	BHUC
U+81E2.0	fFourCorner	74286
U+81E2.0	fHakka	 zan1
U+81E2.0	fCantonese	zim1; zap1 (also); zaam1; zaan3; zim3
U+81E2.0	fMandarin	za1; zang1; zan1
U+81E2.0	fEnglish	dirty; filthy
U+81E3.0	fUTF8	臣
U+81E3.0	fCNS11643	1-485B
U+81E3.0	fBig5	A6DA
U+81E3.0	fGB	0-333C
U+81E3.0	fR/S	 131.0
U+81E3.0	fTotalStrokes	 6
U+81E3.0	fCangjie	SLSL
U+81E3.0	fFourCorner	71717
U+81E3.0	fHakka	 cin2;  shin2;  chin2; siin2; shin3; sin2; cihn2; chin3
U+81E3.0	fCantonese	san4; san2 (also)
U+81E3.0	fMandarin	chen2
U+81E3.0	fEnglish	[1] subject; vassal [2] subjugate; conquer [3] minister; statesman; official [4] a term for "I" used for official when addressing the king or emperor [5] a polite term for "I"
U+81E4.0	fUTF8	臤
U+81E4.0	fCNS11643	4-276C
U+81E4.0	fGB	5-5F62
U+81E4.0	fR/S	  131.2
U+81E4.0	fTotalStrokes	 8
U+81E4.0	fCangjie	SLE
U+81E4.0	fFourCorner	77740
U+81E4.0	fCantonese	haan1
U+81E4.0	fMandarin	qian1; keng1; kan1; qin4; xian2
U+81E4.0	fEnglish	hard; wise
U+81E5.0	fUTF8	臥
U+81E5.0	fCNS11643	1-4F3A
U+81E5.0	fBig5	AAD7
U+81E5.0	fGB	E-3978
U+81E5.0	fR/S	 131.2
U+81E5.0	fTotalStrokes	 8
U+81E5.0	fCangjie	SLO
U+81E5.0	fFourCorner	78700
U+81E5.0	fHakka	 ngo5; ngo3; ngo6; ngoi6
U+81E5.0	fCantonese	ngo6; o6
U+81E5.0	fMandarin	wo4
U+81E5.0	fEnglish	[1] lie down; rest; sleep [2] lay across; place across; lie across
U+81E6.0	fUTF8	臦
U+81E6.0	fCNS11643	2-3F37
U+81E6.0	fBig5	DB52
U+81E6.0	fGB	E-3979
U+81E6.0	fR/S	 131.6
U+81E6.0	fTotalStrokes	 12
U+81E6.0	fCangjie	SLSLL
U+81E6.0	fFourCorner	11117
U+81E6.0	fCantonese	gwong3; gong3 (also); gwing2
U+81E6.0	fMandarin	guang4; kuang2; wang3; jing3
U+81E7.0	fUTF8	臧
U+81E7.0	fCNS11643	1-6A56
U+81E7.0	fBig5	BB4E
U+81E7.0	fGB	0-6A30
U+81E7.0	fR/S	 131.8
U+81E7.0	fTotalStrokes	 14
U+81E7.0	fCangjie	IMSLL
U+81E7.0	fFourCorner	23250
U+81E7.0	fHakka	 zong1; cong2
U+81E7.0	fCantonese	zong1; cong4; zong6
U+81E7.0	fMandarin	zang4; zang2; zang1; cang2
U+81E7.0	fEnglish	[1] good; right; generous [2] slave; servant [3] stolen goods or loots [4] a Chinese family name
U+81E8.0	fUTF8	臨
U+81E8.0	fCNS11643	1-7527
U+81E8.0	fBig5	C17B
U+81E8.0	fGB	1-4159
U+81E8.0	fR/S	 131.11
U+81E8.0	fTotalStrokes	 17
U+81E8.0	fCangjie	SLORR
U+81E8.0	fFourCorner	78766
U+81E8.0	fHakka	  lim2; lim1; lim3
U+81E8.0	fCantonese	lam4; lam1 (also); lam6
U+81E8.0	fMandarin	lin2; lin4
U+81E8.0	fEnglish	[1] look down from above; preside over [2] draw near; approach; descend; come to; reach; visit [3] on the point of; near; during; at; whilst; while [4] copy; imitate [5] temporary; provisional [6] a Chinese family name
U+81E9.0	fUTF8	臩
U+81E9.0	fCNS11643	2-5E5C
U+81E9.0	fBig5	EDEF
U+81E9.0	fGB	E-397A
U+81E9.0	fR/S	 131.11
U+81E9.0	fTotalStrokes	 17
U+81E9.0	fCangjie	SSKLL
U+81E9.0	fFourCorner	11223
U+81E9.0	fCantonese	gwong3; gong3 (also); gwing2
U+81E9.0	fMandarin	guang4; jiong3
U+81EA.0	fUTF8	自
U+81EA.0	fCNS11643	1-485C
U+81EA.0	fBig5	A6DB
U+81EA.0	fGB	0-5754
U+81EA.0	fR/S	 132.0
U+81EA.0	fTotalStrokes	 6
U+81EA.0	fCangjie	HBU
U+81EA.0	fFourCorner	26000
U+81EA.0	fHakka	 cu3; cih6; cit7; cih5; cu5; ci2; ze5
U+81EA.0	fCantonese	zi6; zi2 (also)
U+81EA.0	fMandarin	zi4
U+81EA.0	fEnglish	[1] self; private; personal [2] from [3] natural; naturally; a Chinese family name
U+81EB.0	fUTF8	臫
U+81EB.0	fCNS11643	4-2473
U+81EB.0	fGB	E-397B
U+81EB.0	fR/S	  132.1
U+81EB.0	fTotalStrokes	 7
U+81EB.0	fCangjie	HUU
U+81EB.0	fFourCorner	22610
U+81EB.0	fCantonese	giu2
U+81EB.0	fMandarin	jiao3
U+81EC.0	fUTF8	臬
U+81EC.0	fCNS11643	1-5769
U+81EC.0	fBig5	AFE5
U+81EC.0	fGB	0-742B
U+81EC.0	fR/S	 132.4
U+81EC.0	fTotalStrokes	 10
U+81EC.0	fCangjie	HUD
U+81EC.0	fFourCorner	26904
U+81EC.0	fHakka	  ngiet7; nget7
U+81EC.0	fCantonese	jit6; nip6 (also); lip6 (also)
U+81EC.0	fMandarin	nie4; gui4
U+81EC.0	fEnglish	[1] law; rule; institution [2] another name for the provincial judge in China [3] door post
U+81ED.0	fUTF8	臭
U+81ED.0	fCNS11643	1-5768
U+81ED.0	fBig5	AFE4
U+81ED.0	fGB	0-3374
U+81ED.0	fR/S	 132.4
U+81ED.0	fTotalStrokes	 10
U+81ED.0	fCangjie	HUIK
U+81ED.0	fFourCorner	26430
U+81ED.0	fHakka	 cu5; chiu5; ciu5; chu5
U+81ED.0	fCantonese	cau3
U+81ED.0	fMandarin	chou4
U+81ED.0	fEnglish	[1] stinking; smelly [2] notorious; fragrant; disreputable [3] very; much; soundly; sternly [4] cool off (friendship, love, etc.)
U+81ED.1	fUTF8	臭
U+81ED.1	fCNS11643	1-5768
U+81ED.1	fBig5	AFE4
U+81ED.1	fGB	0-3374
U+81ED.1	fR/S	 132.4
U+81ED.1	fTotalStrokes	 10
U+81ED.1	fCangjie	HUIK
U+81ED.1	fFourCorner	26430
U+81ED.1	fMandarin	xiu4
U+81ED.1	fEnglish	[1] smell [2] scent; odors; smells
U+81EE.0	fUTF8	臮
U+81EE.0	fCNS11643	2-3F38
U+81EE.0	fBig5	DB53
U+81EE.0	fGB	E-397C
U+81EE.0	fR/S	 132.6
U+81EE.0	fTotalStrokes	 12
U+81EE.0	fCangjie	HUOOO
U+81EE.0	fFourCorner	26880
U+81EE.0	fCantonese	kei3; gei6
U+81EE.0	fMandarin	ji4
U+81EF.0	fUTF8	臯
U+81EF.0	fCNS11643	3-3E47
U+81EF.0	fGB	E-397D
U+81EF.0	fR/S	  132.6
U+81EF.0	fTotalStrokes	 12
U+81EF.0	fCangjie	HUEJ
U+81EF.0	fFourCorner	26400
U+81EF.0	fHakka	 gau1
U+81EF.0	fCantonese	gou1
U+81EF.0	fMandarin	gao1
U+81EF.0	fEnglish	[1] lake; pool [2] bank; shore [3] utter a long, wailing sound [4] a Chinese family name
U+81F0.0	fUTF8	臰
U+81F0.0	fCNS11643	4-3A4C
U+81F0.0	fGB	5-645F
U+81F0.0	fR/S	 132.6
U+81F0.0	fTotalStrokes	 12
U+81F0.0	fCangjie	HUMNP
U+81F0.0	fFourCorner	26210
U+81F0.0	fCantonese	cau3
U+81F0.0	fMandarin	chou4; xiu4
U+81F1.0	fUTF8	臱
U+81F1.0	fCNS11643	4-4C62
U+81F1.0	fGB	3-654A
U+81F1.0	fR/S	  132.9
U+81F1.0	fTotalStrokes	 15
U+81F1.0	fCangjie	HUJCS
U+81F1.0	fFourCorner	26220
U+81F1.0	fCantonese	min4
U+81F1.0	fMandarin	mian2
U+81F2.0	fUTF8	臲
U+81F2.0	fCNS11643	2-5960
U+81F2.0	fBig5	EAF4
U+81F2.0	fGB	5-6463
U+81F2.0	fR/S	 132.10
U+81F2.0	fTotalStrokes	 16
U+81F2.0	fCangjie	HDNMU
U+81F2.0	fFourCorner	27912
U+81F2.0	fHakka	 ngiet7; ngiap8; ngiap7; nget7
U+81F2.0	fCantonese	jit6; nip6 (also); lip6 (also); nit6
U+81F2.0	fMandarin	nie4
U+81F2.0	fEnglish	jumpy; jittery; worried
U+81F3.0	fUTF8	至
U+81F3.0	fCNS11643	1-485D
U+81F3.0	fBig5	A6DC
U+81F3.0	fGB	0-5641
U+81F3.0	fR/S	 133.0
U+81F3.0	fTotalStrokes	 6
U+81F3.0	fCangjie	MIG
U+81F3.0	fFourCorner	10104
U+81F3.0	fHakka	 zi5;  zhi5; zih5; tje5
U+81F3.0	fCantonese	zi3
U+81F3.0	fMandarin	zhi4; die2
U+81F3.0	fEnglish	[1] reach (destination); arrive at [2] extremely; very; to indicate the superlative degree; the most
U+81F4.0	fUTF8	致
U+81F4.0	fCNS11643	1-5333
U+81F4.0	fBig5	AD50
U+81F4.0	fGB	0-5642
U+81F4.0	fR/S	 133.3
U+81F4.0	fTotalStrokes	 9
U+81F4.0	fCangjie	MGOK
U+81F4.0	fFourCorner	18140
U+81F4.0	fHakka	  zi5; zhi5
U+81F4.0	fCantonese	zi3
U+81F4.0	fMandarin	zhi4; zhui4
U+81F4.0	fEnglish	[1] send; convey; present; transmit; extend (thanks, etc.) [2] cause to come; cause (injury, death, etc.) [3] achieve; attain; amass (fortune) [4] one's principle; interest; hobby, etc. [5] bring about; occasion; result in [6] retire; resign
U+81F5.0	fUTF8	臵
U+81F5.0	fCNS11643	4-3A4D
U+81F5.0	fGB	E-397E
U+81F5.0	fR/S	  133.6
U+81F5.0	fTotalStrokes	 12
U+81F5.0	fCangjie	MGHER
U+81F5.0	fFourCorner	17160
U+81F5.0	fCantonese	gaak3
U+81F5.0	fMandarin	ge2
U+81F6.0	fUTF8	臶
U+81F6.0	fCNS11643	3-3E48
U+81F6.0	fGB	5-604C
U+81F6.0	fR/S	 133.6
U+81F6.0	fTotalStrokes	 12
U+81F6.0	fCangjie	MGKLD
U+81F6.0	fFourCorner	14140
U+81F6.0	fHakka	  zien5
U+81F6.0	fCantonese	zin6
U+81F6.0	fMandarin	jian4; zun4
U+81F6.0	fEnglish	 repeat; come again
U+81F7.0	fUTF8	臷
U+81F7.0	fCNS11643	2-3F39
U+81F7.0	fBig5	DB54
U+81F7.0	fGB	5-482E
U+81F7.0	fR/S	 133.6
U+81F7.0	fTotalStrokes	 12
U+81F7.0	fCangjie	JIMIG
U+81F7.0	fFourCorner	43150
U+81F7.0	fCantonese	dit6
U+81F7.0	fMandarin	die2; zhi4; ti4
U+81F8.0	fUTF8	臸
U+81F8.0	fCNS11643	2-3F3A
U+81F8.0	fBig5	DB55
U+81F8.0	fGB	3-6141
U+81F8.0	fR/S	 133.6
U+81F8.0	fTotalStrokes	 12
U+81F8.0	fCangjie	MGMIG
U+81F8.0	fFourCorner	11114
U+81F8.0	fCantonese	zi1; zeon3
U+81F8.0	fMandarin	zhi4; ri4; zhi1; jin4
U+81F9.0	fUTF8	臹
U+81F9.0	fCNS11643	2-3F3B
U+81F9.0	fBig5	DB56
U+81F9.0	fGB	E-3A21
U+81F9.0	fR/S	 133.7
U+81F9.0	fTotalStrokes	 13
U+81F9.0	fCangjie	MGIHS
U+81F9.0	fFourCorner	13150
U+81F9.0	fCantonese	sau1
U+81F9.0	fMandarin	xiu1
U+81FA.0	fUTF8	臺
U+81FA.0	fCNS11643	1-6A57
U+81FA.0	fBig5	BB4F
U+81FA.0	fGB	1-4C28
U+81FA.0	fR/S	 133.8
U+81FA.0	fTotalStrokes	 14
U+81FA.0	fCangjie	GRBG
U+81FA.0	fFourCorner	40104
U+81FA.0	fHakka	 toi2; toi3
U+81FA.0	fCantonese	toi4; toi2 (also)
U+81FA.0	fMandarin	tai2; hu2; yi2; tai1
U+81FA.0	fEnglish	[1] tower; lookout; observatory [2] terrace; stage; platform; stand; elevated platform [3] a title of respect [4] short for Taiwan [5] a Chinese family name
U+81FB.0	fUTF8	臻
U+81FB.0	fCNS11643	1-717A
U+81FB.0	fBig5	BFB2
U+81FB.0	fGB	0-5569
U+81FB.0	fR/S	 133.10
U+81FB.0	fTotalStrokes	 16
U+81FB.0	fCangjie	MGQKD
U+81FB.0	fFourCorner	15194
U+81FB.0	fHakka	  zin1
U+81FB.0	fCantonese	zeon1
U+81FB.0	fMandarin	zhen1; jian1
U+81FB.0	fEnglish	[1] reach; arrive at [2] the utmost; superior; the best
U+81FC.0	fUTF8	臼
U+81FC.0	fCNS11643	1-485E
U+81FC.0	fBig5	A6DD
U+81FC.0	fGB	0-3E4A
U+81FC.0	fR/S	 134.0
U+81FC.0	fTotalStrokes	 6
U+81FC.0	fCangjie	HX
U+81FC.0	fFourCorner	77770
U+81FC.0	fHakka	  kiu1; kiu3
U+81FC.0	fCantonese	kau5; kau3 (also); guk1
U+81FC.0	fMandarin	jiu4; pou2
U+81FC.0	fEnglish	[1] mortar for unhusking rice [2] bone joint socket [3] a Chinese family name
U+81FD.0	fUTF8	臽
U+81FD.0	fCNS11643	4-276D
U+81FD.0	fGB	5-342A
U+81FD.0	fR/S	  134.2
U+81FD.0	fTotalStrokes	 8
U+81FD.0	fCangjie	NHX
U+81FD.0	fFourCorner	27770
U+81FD.0	fHakka	 ham5; ham3; kam3
U+81FD.0	fCantonese	haam6
U+81FD.0	fMandarin	xian4; kan3
U+81FD.0	fEnglish	a pit; a hole
U+81FE.0	fUTF8	臾
U+81FE.0	fCNS11643	1-4F3B
U+81FE.0	fBig5	AAD8
U+81FE.0	fGB	0-7427
U+81FE.0	fR/S	 134.2
U+81FE.0	fTotalStrokes	 8
U+81FE.0	fCangjie	HXO
U+81FE.0	fFourCorner	77237
U+81FE.0	fHakka	 ji2; kui3; ri2; i2
U+81FE.0	fCantonese	jyu4; jung5
U+81FE.0	fMandarin	yu2; yong3; kui4; yu3
U+81FE.0	fEnglish	[1] moment; instant; short while; little while; short time [2] a Chinese family name
U+81FF.0	fUTF8	臿
U+81FF.0	fCNS11643	2-2D2B
U+81FF.0	fBig5	D068
U+81FF.0	fGB	3-6542
U+81FF.0	fR/S	 134.3
U+81FF.0	fTotalStrokes	 9
U+81FF.0	fCangjie	HJHX
U+81FF.0	fFourCorner	10777
U+81FF.0	fHakka	 cap7
U+81FF.0	fCantonese	caap3
U+81FF.0	fMandarin	cha1
U+81FF.0	fEnglish	[1] a kind for farming instrument [2] same as $
U+8200.0	fUTF8	舀
U+8200.0	fCNS11643	1-576A
U+8200.0	fBig5	AFE6
U+8200.0	fGB	0-5228
U+8200.0	fR/S	 134.4
U+8200.0	fTotalStrokes	 10
U+8200.0	fCangjie	BHX
U+8200.0	fFourCorner	20777
U+8200.0	fHakka	 jau3; rau3; jeu3
U+8200.0	fCantonese	jiu5; jiu2 (also)
U+8200.0	fMandarin	yao3; wai3; kuai3; yu2; you2
U+8200.0	fEnglish	 ladle out (water)
U+8201.0	fUTF8	舁
U+8201.0	fCNS11643	2-3234
U+8201.0	fBig5	D370
U+8201.0	fGB	0-7428
U+8201.0	fR/S	 134.4
U+8201.0	fTotalStrokes	 10
U+8201.0	fCangjie	HXT
U+8201.0	fFourCorner	77447
U+8201.0	fHakka	 ji2; ri2; i2
U+8201.0	fCantonese	jyu4
U+8201.0	fMandarin	yu2; ju3; yu4
U+8201.0	fEnglish	 lift; raise; carry
U+8202.0	fUTF8	舂
U+8202.0	fCNS11643	1-5C6F
U+8202.0	fBig5	B2EA
U+8202.0	fGB	0-7429
U+8202.0	fR/S	 134.5
U+8202.0	fTotalStrokes	 11
U+8202.0	fCangjie	QKHX
U+8202.0	fFourCorner	50777
U+8202.0	fHakka	 zung1;  zhung1; cung1
U+8202.0	fCantonese	zung1
U+8202.0	fMandarin	chong1; chuang1; shong1; zhong1
U+8202.0	fEnglish	[1] thresh (grain in order to remove the husk) [2] pound; grind in mortar
U+8203.0	fUTF8	舃
U+8203.0	fCNS11643	3-3E49
U+8203.0	fGB	5-645C
U+8203.0	fR/S	  134.6
U+8203.0	fTotalStrokes	 12
U+8203.0	fCangjie	HXYF
U+8203.0	fFourCorner	77320
U+8203.0	fCantonese	sik1
U+8203.0	fMandarin	xi4; qiao3; que4; tuo1
U+8203.0	fEnglish	 shoe the sole of a shoe magpie
U+8204.0	fUTF8	舄
U+8204.0	fCNS11643	2-3F3C
U+8204.0	fBig5	DB57
U+8204.0	fGB	0-742A
U+8204.0	fR/S	 134.6
U+8204.0	fTotalStrokes	 12
U+8204.0	fCangjie	HXPF
U+8204.0	fFourCorner	77327
U+8204.0	fHakka	  sit7
U+8204.0	fCantonese	sik1
U+8204.0	fMandarin	xi4; qiao3; que4; tuo1
U+8204.0	fEnglish	 shoe; slipper
U+8205.0	fUTF8	舅
U+8205.0	fCNS11643	1-662B
U+8205.0	fBig5	B8A4
U+8205.0	fGB	0-3E4B
U+8205.0	fR/S	 134.7
U+8205.0	fTotalStrokes	 13
U+8205.0	fCangjie	HXWKS
U+8205.0	fFourCorner	77427
U+8205.0	fHakka	  kiu1;  kiu3; k'ieu1
U+8205.0	fCantonese	kau5; kau3 (also); kau2 (also)
U+8205.0	fMandarin	jiu4
U+8205.0	fEnglish	[1] a maternal uncle (one's mother's brother) [2] brother-in-law (one's wife's brother) [3] a woman's father-in-law [4] a man's father-in-law
U+8205.1	fUTF8	舅
U+8205.1	fCNS11643	1-662B
U+8205.1	fBig5	B8A4
U+8205.1	fGB	0-3E4B
U+8205.1	fR/S	 134.7
U+8205.1	fTotalStrokes	 13
U+8205.1	fCangjie	HXWKS
U+8205.1	fFourCorner	77427
U+8205.1	fMandarin	jiu5
U+8206.0	fUTF8	舆
U+8206.0	fGB	0-535F
U+8206.0	fR/S	 134.10; 12.12
U+8206.0	fTotalStrokes	 16
U+8206.0	fCangjie	HXKC
U+8206.0	fHakka	 ji2
U+8206.0	fCantonese	jyu4
U+8206.0	fMandarin	yu2
U+8206.0	fEnglish	simplified form of &#x8F3F;
U+8207.0	fUTF8	與
U+8207.0	fCNS11643	1-6A58
U+8207.0	fBig5	BB50
U+8207.0	fGB	1-536B
U+8207.0	fR/S	 134.8
U+8207.0	fTotalStrokes	 14
U+8207.0	fCangjie	HXYC
U+8207.0	fFourCorner	77801
U+8207.0	fHakka	 ji2; ri2; ji5; ji3; i1; i2; i5
U+8207.0	fCantonese	jyu5; jyu4; jyu6
U+8207.0	fMandarin	yu3; yu2; yu4; ai3
U+8207.0	fEnglish	[1] and; with; together with [2] give; grant; impart
U+8207.1	fUTF8	與
U+8207.1	fCNS11643	1-6A58
U+8207.1	fBig5	BB50
U+8207.1	fGB	1-536B
U+8207.1	fR/S	 134.8
U+8207.1	fTotalStrokes	 14
U+8207.1	fCangjie	HXYC
U+8207.1	fFourCorner	77801
U+8207.1	fHakka	 ji1
U+8207.1	fEnglish	 take part in; participate in
U+8208.0	fUTF8	興
U+8208.0	fCNS11643	1-717B
U+8208.0	fBig5	BFB3
U+8208.0	fGB	1-504B
U+8208.0	fR/S	 134.9
U+8208.0	fTotalStrokes	 15
U+8208.0	fCangjie	HXBC
U+8208.0	fFourCorner	77801
U+8208.0	fHakka	  hin1; him5; hin3
U+8208.0	fCantonese	hing1; hing3; jan6
U+8208.0	fMandarin	xing4; xin4
U+8208.0	fEnglish	[1] cheerful; happy; gay; merry [2] interest; enthusiasm; eagerness; willingness
U+8208.1	fUTF8	興
U+8208.1	fCNS11643	1-717B
U+8208.1	fBig5	BFB3
U+8208.1	fGB	1-504B
U+8208.1	fR/S	 134.9
U+8208.1	fTotalStrokes	 15
U+8208.1	fCangjie	HXBC
U+8208.1	fFourCorner	77801
U+8208.1	fHakka	 hin5
U+8208.1	fMandarin	xing1
U+8208.1	fEnglish	[1] rise; thrive; prosper; flourish [2] happen; take place; our [3] start; begin; launch; initiate; establish; found; open
U+8209.0	fUTF8	舉
U+8209.0	fCNS11643	1-7528
U+8209.0	fBig5	C17C
U+8209.0	fGB	1-3E59
U+8209.0	fR/S	 134.10
U+8209.0	fTotalStrokes	 16
U+8209.0	fCangjie	HCQ
U+8209.0	fFourCorner	77502
U+8209.0	fHakka	  gi3
U+8209.0	fCantonese	geoi2
U+8209.0	fMandarin	ju3
U+8209.0	fEnglish	[1] raise; lift [2] recommend; commend [3] praise [4] manner; deportment
U+8209.1	fUTF8	舉
U+8209.1	fCNS11643	1-7528
U+8209.1	fBig5	C17C
U+8209.1	fGB	1-3E59
U+8209.1	fR/S	 134.10
U+8209.1	fTotalStrokes	 16
U+8209.1	fCangjie	HCQ
U+8209.1	fFourCorner	77502
U+8209.1	fMandarin	ju5
U+820A.0	fUTF8	舊
U+820A.0	fCNS11643	1-772E
U+820A.0	fBig5	C2C2
U+820A.0	fGB	1-3E49
U+820A.0	fR/S	 134.12
U+820A.0	fTotalStrokes	 18
U+820A.0	fCangjie	TOGX
U+820A.0	fFourCorner	44777
U+820A.0	fHakka	 kiu6; kiu3; kiu5; k'ieu5
U+820A.0	fCantonese	gau6
U+820A.0	fMandarin	jiu4; xiu1
U+820A.0	fEnglish	[1] old; former; past [2] ancient; antique [3] longstanding
U+820B.0	fUTF8	舋
U+820B.0	fCNS11643	2-664D
U+820B.0	fBig5	F4B5
U+820B.0	fGB	3-302E
U+820B.0	fR/S	 134.13
U+820B.0	fTotalStrokes	 19
U+820B.0	fCangjie	HBBM
U+820B.0	fFourCorner	77107
U+820B.0	fHakka	 hin5; mun5
U+820B.0	fCantonese	jan6; man6; jan3; lang6
U+820B.0	fMandarin	xin4; wen4
U+820B.0	fEnglish	split
U+820C.0	fUTF8	舌
U+820C.0	fCNS11643	1-485F
U+820C.0	fBig5	A6DE
U+820C.0	fGB	0-4960
U+820C.0	fR/S	 135.0
U+820C.0	fTotalStrokes	 6
U+820C.0	fCangjie	HJR
U+820C.0	fFourCorner	20604
U+820C.0	fHakka	 set8; s'at7; siet8; sat7; s'at8; shat8; s'et8
U+820C.0	fCantonese	sit3; sit6 (also)
U+820C.0	fMandarin	she2; gua1
U+820C.0	fEnglish	[1] the tongue [2] Kangxi radical 135
U+820D.0	fUTF8	舍
U+820D.0	fCNS11643	1-4F3C
U+820D.0	fBig5	AAD9
U+820D.0	fGB	0-4961
U+820D.0	fR/S	 135.2
U+820D.0	fTotalStrokes	 8
U+820D.0	fCangjie	OMJR
U+820D.0	fFourCorner	80604
U+820D.0	fHakka	  sa5; sha5; sa3; sha3
U+820D.0	fCantonese	se3; se5 (also)
U+820D.0	fMandarin	shi4
U+820D.0	fEnglish	[1] house; dwelling; dwell; reside [2] inn [3] halt; stop; rest [4] a self-depreciatory possessive pronoun for the first person singular in formal speech
U+820D.1	fUTF8	舍
U+820D.1	fCNS11643	1-4F3C
U+820D.1	fBig5	AAD9
U+820D.1	fGB	0-4961
U+820D.1	fR/S	 135.2
U+820D.1	fTotalStrokes	 8
U+820D.1	fCangjie	OMJR
U+820D.1	fFourCorner	80604
U+820D.1	fHakka	 sha3
U+820D.1	fCantonese	se2
U+820D.1	fMandarin	she3; she4
U+820D.1	fEnglish	simplified form of &#x6368;
U+820E.0	fUTF8	舎
U+820E.0	fCNS11643	3-2A5D
U+820E.0	fGB	E-3A22
U+820E.0	fR/S	 135.2
U+820E.0	fTotalStrokes	 8
U+820E.0	fCangjie	OGR
U+820E.0	fFourCorner	80600
U+820E.0	fCantonese	se3
U+820E.0	fMandarin	she4; she3; shi4
U+820E.0	fEnglish	[1] house; dwelling dwell; reside [2] simplified form of &#x6368;
U+820F.0	fUTF8	舏
U+820F.0	fCNS11643	4-276E
U+820F.0	fGB	E-3A23
U+820F.0	fR/S	  135.2
U+820F.0	fTotalStrokes	 8
U+820F.0	fCangjie	HRVL
U+820F.0	fFourCorner	22600
U+820F.0	fCantonese	gau2
U+820F.0	fMandarin	jiu3
U+8210.0	fUTF8	舐
U+8210.0	fCNS11643	1-576B
U+8210.0	fBig5	AFE7
U+8210.0	fGB	0-7342
U+8210.0	fR/S	 135.4
U+8210.0	fTotalStrokes	 10
U+8210.0	fCangjie	HRHVP
U+8210.0	fFourCorner	22640
U+8210.0	fHakka	  se1; s'e1; tiam3; she1; shai1
U+8210.0	fCantonese	saai5; saai2 (also); laai2 (also); lem2 (also); lim2 (also); si6
U+8210.0	fMandarin	shi4
U+8210.0	fEnglish	 lick (with the tongue)
U+8211.0	fUTF8	舑
U+8211.0	fCNS11643	2-3856
U+8211.0	fBig5	D752
U+8211.0	fGB	3-6342
U+8211.0	fR/S	 135.5
U+8211.0	fTotalStrokes	 11
U+8211.0	fCangjie	HRGB
U+8211.0	fFourCorner	15657
U+8211.0	fHakka	 dam3; tam3
U+8211.0	fCantonese	daam1; taan1; tim3
U+8211.0	fMandarin	tan1
U+8212.0	fUTF8	舒
U+8212.0	fCNS11643	1-6154
U+8212.0	fBig5	B5CE
U+8212.0	fGB	0-4A66
U+8212.0	fR/S	 135.6
U+8212.0	fTotalStrokes	 12
U+8212.0	fCangjie	ORNIN
U+8212.0	fFourCorner	87622
U+8212.0	fHakka	  su1; shu1
U+8212.0	fCantonese	syu1
U+8212.0	fMandarin	shu1; yu4
U+8212.0	fEnglish	[1] open (up); unfold; stretch (out); relax [2] slow; unhurried; leisurely [3] a Chinese family name
U+8213.0	fUTF8	舓
U+8213.0	fCNS11643	3-492B
U+8213.0	fGB	5-626B
U+8213.0	fR/S	  135.8
U+8213.0	fTotalStrokes	 14
U+8213.0	fCangjie	HRAPH
U+8213.0	fFourCorner	26620
U+8213.0	fHakka	 sii5; s'e1; se1; sii6
U+8213.0	fCantonese	saai5
U+8213.0	fMandarin	shi4
U+8213.0	fEnglish	 lick
U+8214.0	fUTF8	舔
U+8214.0	fCNS11643	1-6A59
U+8214.0	fBig5	BB51
U+8214.0	fGB	0-4C72
U+8214.0	fR/S	 135.8
U+8214.0	fTotalStrokes	 14
U+8214.0	fCangjie	HRHKP
U+8214.0	fFourCorner	22633
U+8214.0	fHakka	  tiam3
U+8214.0	fCantonese	tim2
U+8214.0	fMandarin	tian3; tan1
U+8214.0	fEnglish	 lick (with the tongue); taste
U+8215.0	fUTF8	舕
U+8215.0	fCNS11643	2-4D32
U+8215.0	fBig5	E3AB
U+8215.0	fGB	3-6344
U+8215.0	fR/S	 135.8
U+8215.0	fTotalStrokes	 14
U+8215.0	fCangjie	HRFF
U+8215.0	fFourCorner	29689
U+8215.0	fCantonese	taam3; caam3
U+8215.0	fMandarin	tan4; dan4
U+8215.0	fEnglish	stick out the tongue
U+8216.0	fUTF8	舖
U+8216.0	fCNS11643	2-532E
U+8216.0	fBig5	E745
U+8216.0	fGB	8-2F76
U+8216.0	fR/S	 135.9
U+8216.0	fTotalStrokes	 15
U+8216.0	fCangjie	ORIJB
U+8216.0	fFourCorner	83627
U+8216.0	fHakka	  pu5;  pu1
U+8216.0	fCantonese	pou3; pou1; pou2 (also)
U+8216.0	fMandarin	pu4; pu1; bu1; fu1
U+8216.0	fEnglish	 store; shop
U+8217.0	fUTF8	舗
U+8217.0	fGB	E-3A24
U+8217.0	fR/S	 135.9
U+8217.0	fTotalStrokes	 15
U+8217.0	fCangjie	XORIJ
U+8217.0	fCantonese	pou3
U+8217.0	fMandarin	pu4; pu1
U+8217.0	fEnglish	store; shop
U+8218.0	fUTF8	舘
U+8218.0	fCNS11643	3-524A
U+8218.0	fGB	8-2F75
U+8218.0	fR/S	 135.10
U+8218.0	fTotalStrokes	 16
U+8218.0	fCangjie	ORJRR
U+8218.0	fFourCorner	83670
U+8218.0	fHakka	  gon3;  kwan3; gwon3
U+8218.0	fCantonese	gun2; gwun2 (also)
U+8218.0	fMandarin	guan3
U+8218.0	fEnglish	 mansion building
U+8219.0	fUTF8	舙
U+8219.0	fCNS11643	3-5868
U+8219.0	fGB	5-626E
U+8219.0	fR/S	  135.12
U+8219.0	fTotalStrokes	 18
U+8219.0	fCangjie	HRHRR
U+8219.0	fFourCorner	20660
U+8219.0	fCantonese	jyu5
U+8219.0	fMandarin	hua4
U+821A.0	fUTF8	舚
U+821A.0	fCNS11643	3-5A79
U+821A.0	fGB	5-626F
U+821A.0	fR/S	 135.13
U+821A.0	fTotalStrokes	 19
U+821A.0	fCangjie	HRNCR
U+821A.0	fFourCorner	27660
U+821A.0	fHakka	  tam1; dam1; tiam3
U+821A.0	fCantonese	daam1
U+821A.0	fMandarin	tan1; tian4
U+821A.0	fEnglish	 put out the tongue
U+821B.0	fUTF8	舛
U+821B.0	fCNS11643	1-4860
U+821B.0	fBig5	A6DF
U+821B.0	fGB	0-6236
U+821B.0	fR/S	 136.0
U+821B.0	fTotalStrokes	 6
U+821B.0	fCangjie	NIQ
U+821B.0	fFourCorner	25200
U+821B.0	fHakka	  con3;  chon3
U+821B.0	fCantonese	cyun2
U+821B.0	fMandarin	chuan3; chun3
U+821B.0	fEnglish	[1] chaotic; disorderly; messy; confused; mixed up [2] run counter to; disobey; oppose; deviate from; be contrary to [3] mishap sufferings; setbacks [4] Kangxi radical 136
U+821C.0	fUTF8	舜
U+821C.0	fCNS11643	1-6155
U+821C.0	fBig5	B5CF
U+821C.0	fGB	0-4B34
U+821C.0	fR/S	 136.6
U+821C.0	fTotalStrokes	 12
U+821C.0	fCangjie	BBNQ
U+821C.0	fFourCorner	20252
U+821C.0	fHakka	 shun6; sun5;  shun5; sun3
U+821C.0	fCantonese	seon3
U+821C.0	fMandarin	shun4
U+821C.0	fEnglish	Shun, a legendary ruler who ruled China around 2200 B.C.
U+821D.0	fUTF8	舝
U+821D.0	fCNS11643	2-4648
U+821D.0	fBig5	DFA3
U+821D.0	fGB	5-3039
U+821D.0	fR/S	 136.7
U+821D.0	fTotalStrokes	 13
U+821D.0	fCangjie	DMVVQ
U+821D.0	fFourCorner	40503
U+821D.0	fHakka	 hot7;  hat7
U+821D.0	fCantonese	hat6
U+821D.0	fMandarin	xia2
U+821D.0	fEnglish	 linch-pin
U+821E.0	fUTF8	舞
U+821E.0	fCNS11643	1-6A5A
U+821E.0	fBig5	BB52
U+821E.0	fGB	0-4E68
U+821E.0	fR/S	 136.8
U+821E.0	fTotalStrokes	 14
U+821E.0	fCangjie	OTNIQ
U+821E.0	fFourCorner	80251
U+821E.0	fHakka	  wu3;  mu3; vu3
U+821E.0	fCantonese	mou5
U+821E.0	fMandarin	wu3
U+821E.0	fEnglish	[1] dance; posture; prance [2] brandish; wave [3] stir up; agitate
U+821F.0	fUTF8	舟
U+821F.0	fCNS11643	1-4861
U+821F.0	fBig5	A6E0
U+821F.0	fGB	0-565B
U+821F.0	fR/S	 137.0
U+821F.0	fTotalStrokes	 6
U+821F.0	fCangjie	HBYI
U+821F.0	fFourCorner	27440
U+821F.0	fHakka	 zu1;  zhiu1; ziu1; zhu1
U+821F.0	fCantonese	zau1
U+821F.0	fMandarin	zhou1
U+821F.0	fEnglish	[1] boat; ship [2] Kangxi radical 137
U+8220.0	fUTF8	舠
U+8220.0	fCNS11643	2-2851
U+8220.0	fBig5	CDB1
U+8220.0	fGB	3-6550
U+8220.0	fR/S	 137.2
U+8220.0	fTotalStrokes	 8
U+8220.0	fCangjie	HYSH
U+8220.0	fFourCorner	27420
U+8220.0	fHakka	  dau1; do1
U+8220.0	fCantonese	dou1
U+8220.0	fMandarin	dao1
U+8220.0	fEnglish	 long narrow knife-shaped boat
U+8221.0	fUTF8	舡
U+8221.0	fCNS11643	2-2D2C
U+8221.0	fBig5	D069
U+8221.0	fGB	0-742D
U+8221.0	fR/S	 137.3
U+8221.0	fTotalStrokes	 9
U+8221.0	fCangjie	HYM
U+8221.0	fFourCorner	21410
U+8221.0	fHakka	 gong1; hong1; s'on2; son2; shon2; {s'on2}
U+8221.0	fCantonese	hong1; syun4; gong1; song1
U+8221.0	fMandarin	gang1; chuan2; qiang1; xiang1
U+8221.0	fEnglish	 boat; ship
U+8222.0	fUTF8	舢
U+8222.0	fCNS11643	1-5334
U+8222.0	fBig5	AD51
U+8222.0	fGB	0-742E
U+8222.0	fR/S	 137.3
U+8222.0	fTotalStrokes	 9
U+8222.0	fCangjie	HYU
U+8222.0	fFourCorner	22470
U+8222.0	fHakka	  san1
U+8222.0	fCantonese	saan1
U+8222.0	fMandarin	shan1
U+8222.0	fEnglish	 sampan
U+8223.0	fUTF8	舣
U+8223.0	fGB	0-742F
U+8223.0	fR/S	 137.3
U+8223.0	fTotalStrokes	 9
U+8223.0	fCangjie	HYIK
U+8223.0	fHakka	 ca1; ngi5
U+8223.0	fCantonese	ngai5; ai5 (also)
U+8223.0	fMandarin	yi3
U+8223.0	fEnglish	simplified form of &#x8264;
U+8224.0	fUTF8	舤
U+8224.0	fCNS11643	3-2E68
U+8224.0	fGB	E-3A25
U+8224.0	fR/S	 137.3
U+8224.0	fTotalStrokes	 9
U+8224.0	fCangjie	HYHNI
U+8224.0	fFourCorner	27410
U+8224.0	fCantonese	faan4
U+8224.0	fMandarin	fan2
U+8225.0	fUTF8	舥
U+8225.0	fCNS11643	2-3236
U+8225.0	fBig5	D372
U+8225.0	fGB	3-6557
U+8225.0	fR/S	 137.4
U+8225.0	fTotalStrokes	 10
U+8225.0	fCangjie	HYAU
U+8225.0	fFourCorner	27417
U+8225.0	fHakka	  ba1
U+8225.0	fCantonese	paa1; baa1
U+8225.0	fMandarin	pa1
U+8225.0	fEnglish	floating bridge
U+8226.0	fUTF8	舦
U+8226.0	fCNS11643	4-2F4F
U+8226.0	fGB	1-7C4B
U+8226.0	fR/S	  137.4
U+8226.0	fTotalStrokes	 10
U+8226.0	fCangjie	HYKI
U+8226.0	fFourCorner	24430
U+8226.0	fCantonese	taai5
U+8226.0	fMandarin	tai4
U+8226.0	fEnglish	rudder
U+8227.0	fUTF8	舧
U+8227.0	fCNS11643	3-3357
U+8227.0	fGB	5-6474
U+8227.0	fR/S	  137.4
U+8227.0	fTotalStrokes	 10
U+8227.0	fCangjie	XHYHN
U+8227.0	fFourCorner	27410
U+8227.0	fCantonese	faan4
U+8227.0	fMandarin	fan2
U+8228.0	fUTF8	舨
U+8228.0	fCNS11643	1-576E
U+8228.0	fBig5	AFEA
U+8228.0	fGB	0-7432
U+8228.0	fR/S	 137.4
U+8228.0	fTotalStrokes	 10
U+8228.0	fCangjie	HYHE
U+8228.0	fFourCorner	21447
U+8228.0	fHakka	 ban3
U+8228.0	fCantonese	baan2
U+8228.0	fMandarin	ban3
U+8228.0	fEnglish	 sampan
U+8229.0	fUTF8	舩
U+8229.0	fCNS11643	3-3358
U+8229.0	fGB	E-3A26
U+8229.0	fR/S	 137.4
U+8229.0	fTotalStrokes	 10
U+8229.0	fCangjie	HYCI
U+8229.0	fFourCorner	28430
U+8229.0	fCantonese	syun4
U+8229.0	fMandarin	chuan2
U+8229.0	fEnglish	boat; ship; vessel
U+822A.0	fUTF8	航
U+822A.0	fCNS11643	1-576C
U+822A.0	fBig5	AFE8
U+822A.0	fGB	0-3A3D
U+822A.0	fR/S	 137.4
U+822A.0	fTotalStrokes	 10
U+822A.0	fCangjie	HYYHN
U+822A.0	fFourCorner	20417
U+822A.0	fHakka	  hong2
U+822A.0	fCantonese	hong4
U+822A.0	fMandarin	hang2
U+822A.0	fEnglish	[1] sail; navigate [2] ship; boat; vessel
U+822B.0	fUTF8	舫
U+822B.0	fCNS11643	1-576D
U+822B.0	fBig5	AFE9
U+822B.0	fGB	0-7433
U+822B.0	fR/S	 137.4
U+822B.0	fTotalStrokes	 10
U+822B.0	fCangjie	HYYHS
U+822B.0	fFourCorner	20427
U+822B.0	fHakka	  fong3
U+822B.0	fCantonese	fong2
U+822B.0	fMandarin	fang3; fang4; fang1; bang4
U+822B.0	fEnglish	[1] two boats latched side by side [2] boat; yacht; ship
U+822C.0	fUTF8	般
U+822C.0	fCNS11643	1-576F
U+822C.0	fBig5	AFEB
U+822C.0	fGB	0-3063
U+822C.0	fR/S	 137.4
U+822C.0	fTotalStrokes	 10
U+822C.0	fCangjie	HYHNE
U+822C.0	fFourCorner	27447
U+822C.0	fHakka	  ban1
U+822C.0	fCantonese	bun1; bo1; pun4; baan1; but3
U+822C.0	fMandarin	ban1; ban3
U+822C.0	fEnglish	[1] sort; manner; kind; category [2] same as &#x642C; [3] same as &#x73ED;
U+822C.1	fUTF8	般
U+822C.1	fCNS11643	1-576F
U+822C.1	fBig5	AFEB
U+822C.1	fGB	0-3063
U+822C.1	fR/S	 137.4
U+822C.1	fTotalStrokes	 10
U+822C.1	fCangjie	HYHNE
U+822C.1	fFourCorner	27447
U+822C.1	fMandarin	pan2
U+822C.1	fEnglish	[1] linger [2] comfort [3] leather bag
U+822C.2	fUTF8	般
U+822C.2	fCNS11643	1-576F
U+822C.2	fBig5	AFEB
U+822C.2	fGB	0-3063
U+822C.2	fR/S	 137.4
U+822C.2	fTotalStrokes	 10
U+822C.2	fCangjie	HYHNE
U+822C.2	fFourCorner	27447
U+822C.2	fMandarin	bo1
U+822C.2	fEnglish	intelligence (Buddhism)
U+822D.0	fUTF8	舭
U+822D.0	fCNS11643	3-3355
U+822D.0	fGB	0-7430
U+822D.0	fR/S	 137.4
U+822D.0	fTotalStrokes	 10
U+822D.0	fCangjie	HYPP
U+822D.0	fFourCorner	21410
U+822D.0	fCantonese	bei2
U+822D.0	fMandarin	que4; bi3
U+822E.0	fUTF8	舮
U+822E.0	fGB	E-3A27
U+822E.0	fR/S	 137.4
U+822E.0	fTotalStrokes	 10
U+822E.0	fCangjie	XHYHS
U+822E.0	fCantonese	lou4
U+822E.0	fMandarin	lu2
U+822E.0	fEnglish	 bow or prow of boat
U+822F.0	fUTF8	舯
U+822F.0	fCNS11643	2-3235
U+822F.0	fBig5	D371
U+822F.0	fGB	0-7431
U+822F.0	fR/S	 137.4
U+822F.0	fTotalStrokes	 10
U+822F.0	fCangjie	HYL
U+822F.0	fFourCorner	25406
U+822F.0	fCantonese	zung1
U+822F.0	fMandarin	zhong1
U+8230.0	fUTF8	舰
U+8230.0	fGB	0-3D22
U+8230.0	fR/S	 137.4
U+8230.0	fTotalStrokes	 10
U+8230.0	fCangjie	HYBHU
U+8230.0	fHakka	 gam5
U+8230.0	fCantonese	laam6; cam1
U+8230.0	fMandarin	jian4
U+8230.0	fEnglish	simplified form of &#x8266;
U+8231.0	fUTF8	舱
U+8231.0	fGB	0-3255
U+8231.0	fR/S	 137.4
U+8231.0	fTotalStrokes	 10
U+8231.0	fCangjie	HYOSU
U+8231.0	fHakka	 cong1
U+8231.0	fCantonese	cong1
U+8231.0	fMandarin	cang1
U+8231.0	fEnglish	simplified form of &#x8259;
U+8232.0	fUTF8	舲
U+8232.0	fCNS11643	2-385B
U+8232.0	fBig5	D757
U+8232.0	fGB	3-655A
U+8232.0	fR/S	 137.5
U+8232.0	fTotalStrokes	 11
U+8232.0	fCangjie	HYOII
U+8232.0	fFourCorner	28437
U+8232.0	fHakka	 lin2
U+8232.0	fCantonese	ling4; lang1
U+8232.0	fMandarin	ling2
U+8232.0	fEnglish	a small boat with windows; small boat with portholes
U+8233.0	fUTF8	舳
U+8233.0	fCNS11643	2-3858
U+8233.0	fBig5	D754
U+8233.0	fGB	0-7436
U+8233.0	fR/S	 137.5
U+8233.0	fTotalStrokes	 11
U+8233.0	fCangjie	HYLW
U+8233.0	fFourCorner	25460
U+8233.0	fHakka	 ch'uk8; cuk8; chuk8
U+8233.0	fCantonese	zuk6
U+8233.0	fMandarin	zhu2; zhou4; zhou2; you4
U+8233.0	fEnglish	the stern of a ship
U+8234.0	fUTF8	舴
U+8234.0	fCNS11643	2-385A
U+8234.0	fBig5	D756
U+8234.0	fGB	0-7437
U+8234.0	fR/S	 137.5
U+8234.0	fTotalStrokes	 11
U+8234.0	fCangjie	HYOS
U+8234.0	fFourCorner	28411
U+8234.0	fHakka	 za5;  cok7; zak7
U+8234.0	fCantonese	zaak3
U+8234.0	fMandarin	ze2; zhe2; xi2
U+8234.0	fEnglish	 small boat
U+8235.0	fUTF8	舵
U+8235.0	fCNS11643	1-5C70
U+8235.0	fBig5	B2EB
U+8235.0	fGB	0-3666
U+8235.0	fR/S	 137.5
U+8235.0	fTotalStrokes	 11
U+8235.0	fCangjie	HYJP
U+8235.0	fFourCorner	23411
U+8235.0	fHakka	  to2; to6;  to5
U+8235.0	fCantonese	to4; to5; taai5
U+8235.0	fMandarin	duo4; tuo2
U+8235.0	fEnglish	 rudder; helm
U+8236.0	fUTF8	舶
U+8236.0	fCNS11643	1-5C72
U+8236.0	fBig5	B2ED
U+8236.0	fGB	0-3230
U+8236.0	fR/S	 137.5
U+8236.0	fTotalStrokes	 11
U+8236.0	fCangjie	HYHA
U+8236.0	fFourCorner	26400
U+8236.0	fHakka	 pok7; pak7
U+8236.0	fCantonese	bok6; paak3 (also); baak3 (also); paak6 (also); baak6
U+8236.0	fMandarin	bo2
U+8236.0	fEnglish	 large, ocean-going vessel; ocean-going ship
U+8237.0	fUTF8	舷
U+8237.0	fCNS11643	1-5C71
U+8237.0	fBig5	B2EC
U+8237.0	fGB	0-4F4F
U+8237.0	fR/S	 137.5
U+8237.0	fTotalStrokes	 11
U+8237.0	fCangjie	HYYVI
U+8237.0	fFourCorner	20432
U+8237.0	fHakka	  hien2; hen2
U+8237.0	fCantonese	jin4
U+8237.0	fMandarin	xian2
U+8237.0	fEnglish	 bulwarks (ship); gunwale; gunnel
U+8238.0	fUTF8	舸
U+8238.0	fCNS11643	2-3857
U+8238.0	fBig5	D753
U+8238.0	fGB	0-7434
U+8238.0	fR/S	 137.5
U+8238.0	fTotalStrokes	 11
U+8238.0	fCangjie	HYMNR
U+8238.0	fFourCorner	21420
U+8238.0	fHakka	 ko3
U+8238.0	fCantonese	go2; ho2 (also)
U+8238.0	fMandarin	ge3; ge1
U+8238.0	fEnglish	 large boat; barge
U+8239.0	fUTF8	船
U+8239.0	fCNS11643	1-5C73
U+8239.0	fBig5	B2EE
U+8239.0	fGB	0-342C
U+8239.0	fR/S	 137.5
U+8239.0	fTotalStrokes	 11
U+8239.0	fCangjie	HYCR
U+8239.0	fFourCorner	28460
U+8239.0	fHakka	  son2; s'on2; s'on3; shon2
U+8239.0	fCantonese	syun4; syun2 (also)
U+8239.0	fMandarin	chuan2; yan2
U+8239.0	fEnglish	 ship; boat; vessel; craft
U+823A.0	fUTF8	舺
U+823A.0	fCNS11643	2-3859
U+823A.0	fBig5	D755
U+823A.0	fGB	3-6559
U+823A.0	fR/S	 137.5
U+823A.0	fTotalStrokes	 11
U+823A.0	fCangjie	HYWL
U+823A.0	fFourCorner	26450
U+823A.0	fCantonese	gaap3
U+823A.0	fMandarin	xia2; jia2; jia3
U+823A.0	fEnglish	a place in Taiwan; Taipei
U+823B.0	fUTF8	舻
U+823B.0	fGB	0-7435
U+823B.0	fR/S	 137.5
U+823B.0	fTotalStrokes	 11
U+823B.0	fCangjie	HYYS
U+823B.0	fHakka	 lu2
U+823B.0	fCantonese	lou4
U+823B.0	fMandarin	lu2; lu3
U+823B.0	fEnglish	simplified form of &#x826B;
U+823C.0	fUTF8	舼
U+823C.0	fCNS11643	2-3F3D
U+823C.0	fBig5	DB58
U+823C.0	fGB	3-655D
U+823C.0	fR/S	 137.6
U+823C.0	fTotalStrokes	 12
U+823C.0	fCangjie	HYTC
U+823C.0	fFourCorner	24481
U+823C.0	fCantonese	kung4; hung4
U+823C.0	fMandarin	hong2; qiong2
U+823D.0	fUTF8	舽
U+823D.0	fCNS11643	2-3F3E
U+823D.0	fBig5	DB59
U+823D.0	fGB	3-655F
U+823D.0	fR/S	 137.6
U+823D.0	fTotalStrokes	 12
U+823D.0	fCangjie	HYHEQ
U+823D.0	fFourCorner	27454
U+823D.0	fHakka	 pong2
U+823D.0	fCantonese	pong4; fung4
U+823D.0	fMandarin	pang2
U+823D.0	fEnglish	 swell
U+823E.0	fUTF8	舾
U+823E.0	fGB	0-7438
U+823E.0	fR/S	 137.6
U+823E.0	fTotalStrokes	 12
U+823E.0	fCangjie	HYMCW
U+823E.0	fCantonese	sai1
U+823E.0	fMandarin	xi1
U+823E.0	fEnglish	equipment on a ship
U+823F.0	fUTF8	舿
U+823F.0	fCNS11643	2-3F3F
U+823F.0	fBig5	DB5A
U+823F.0	fGB	E-3A28
U+823F.0	fR/S	 137.6
U+823F.0	fTotalStrokes	 12
U+823F.0	fCangjie	HYKMS
U+823F.0	fFourCorner	24221
U+823F.0	fCantonese	kwaa1; kwaa3
U+8240.0	fUTF8	艀
U+8240.0	fCNS11643	2-464B
U+8240.0	fBig5	DFA6
U+8240.0	fGB	5-6522
U+8240.0	fR/S	 137.7
U+8240.0	fTotalStrokes	 13
U+8240.0	fCangjie	HYBND
U+8240.0	fFourCorner	22447
U+8240.0	fCantonese	fau4
U+8240.0	fMandarin	fu2; fou2
U+8240.0	fEnglish	a sampan, lighter, barge
U+8241.0	fUTF8	艁
U+8241.0	fCNS11643	3-4426
U+8241.0	fGB	5-647E
U+8241.0	fR/S	  137.7
U+8241.0	fTotalStrokes	 13
U+8241.0	fCangjie	HYHGR
U+8241.0	fFourCorner	25460
U+8241.0	fCantonese	zou6
U+8241.0	fMandarin	zao4
U+8242.0	fUTF8	艂
U+8242.0	fCNS11643	2-464C
U+8242.0	fBig5	DFA7
U+8242.0	fGB	E-3A29
U+8242.0	fR/S	 137.7
U+8242.0	fTotalStrokes	 13
U+8242.0	fCangjie	HYHEJ
U+8242.0	fFourCorner	27454
U+8242.0	fCantonese	fung4
U+8242.0	fMandarin	feng2
U+8243.0	fUTF8	艃
U+8243.0	fCNS11643	4-402E
U+8243.0	fGB	E-3A2A
U+8243.0	fR/S	  137.7
U+8243.0	fTotalStrokes	 13
U+8243.0	fCangjie	HYWG
U+8243.0	fFourCorner	26410
U+8243.0	fCantonese	lei4
U+8243.0	fMandarin	li2
U+8244.0	fUTF8	艄
U+8244.0	fCNS11643	2-464A
U+8244.0	fBig5	DFA5
U+8244.0	fGB	0-7439
U+8244.0	fR/S	 137.7
U+8244.0	fTotalStrokes	 13
U+8244.0	fCangjie	HYFB
U+8244.0	fFourCorner	29427
U+8244.0	fHakka	 sau1
U+8244.0	fCantonese	saau1
U+8244.0	fMandarin	shao1; shao4
U+8244.0	fEnglish	[1] the stern (boat) [2] rudder; helm
U+8245.0	fUTF8	艅
U+8245.0	fCNS11643	2-464D
U+8245.0	fBig5	DFA8
U+8245.0	fGB	5-6521
U+8245.0	fR/S	 137.7
U+8245.0	fTotalStrokes	 13
U+8245.0	fCangjie	HYOMD
U+8245.0	fFourCorner	28494
U+8245.0	fCantonese	jyu4
U+8245.0	fMandarin	yu2
U+8245.0	fEnglish	a despatch boat
U+8246.0	fUTF8	艆
U+8246.0	fCNS11643	4-402D
U+8246.0	fGB	3-6564
U+8246.0	fR/S	  137.7
U+8246.0	fTotalStrokes	 13
U+8246.0	fCangjie	HYIAV
U+8246.0	fFourCorner	23430
U+8246.0	fCantonese	long4
U+8246.0	fMandarin	lang2
U+8247.0	fUTF8	艇
U+8247.0	fCNS11643	1-662C
U+8247.0	fBig5	B8A5
U+8247.0	fGB	0-4D27
U+8247.0	fR/S	 137.7
U+8247.0	fTotalStrokes	 13
U+8247.0	fCangjie	HYNKG
U+8247.0	fFourCorner	22441
U+8247.0	fHakka	 tiang3;  tin3; tiang5
U+8247.0	fCantonese	ting5; teng5 (also); teng3
U+8247.0	fMandarin	ting3
U+8247.0	fEnglish	 small boat; long narrow boat; punt
U+8248.0	fUTF8	艈
U+8248.0	fCNS11643	3-6634
U+8248.0	fGB	E-3A2B
U+8248.0	fR/S	 137.7
U+8248.0	fTotalStrokes	 13
U+8248.0	fCangjie	HYYIU
U+8248.0	fFourCorner	2041
U+8248.0	fCantonese	juk1
U+8249.0	fUTF8	艉
U+8249.0	fCNS11643	2-4649
U+8249.0	fBig5	DFA4
U+8249.0	fGB	0-743A
U+8249.0	fR/S	 137.7
U+8249.0	fTotalStrokes	 13
U+8249.0	fCangjie	HYSHU
U+8249.0	fFourCorner	27414
U+8249.0	fCantonese	mei5
U+8249.0	fMandarin	wei3
U+824A.0	fUTF8	艊
U+824A.0	fCNS11643	3-492D
U+824A.0	fGB	E-3A2C
U+824A.0	fR/S	 137.8
U+824A.0	fTotalStrokes	 14
U+824A.0	fCangjie	HYHAB
U+824A.0	fFourCorner	26420
U+824A.0	fCantonese	bok6
U+824A.0	fMandarin	bo2
U+824B.0	fUTF8	艋
U+824B.0	fCNS11643	1-6A5B
U+824B.0	fBig5	BB53
U+824B.0	fGB	0-743B
U+824B.0	fR/S	 137.8
U+824B.0	fTotalStrokes	 14
U+824B.0	fCangjie	HYNDT
U+824B.0	fFourCorner	27417
U+824B.0	fHakka	  mang3
U+824B.0	fCantonese	maang5
U+824B.0	fMandarin	meng3
U+824B.0	fEnglish	 small boat
U+824C.0	fUTF8	艌
U+824C.0	fCNS11643	3-492E
U+824C.0	fGB	5-6525
U+824C.0	fR/S	 137.8
U+824C.0	fTotalStrokes	 14
U+824C.0	fCangjie	HYOIP
U+824C.0	fFourCorner	28430
U+824C.0	fHakka	 ngiam5; ngiam6; ngiam3
U+824C.0	fCantonese	nim6; lim6 (also)
U+824C.0	fMandarin	nian4; qian4
U+824C.0	fEnglish	 caulk
U+824D.0	fUTF8	艍
U+824D.0	fGB	1-7D22
U+824D.0	fR/S	 137.8
U+824D.0	fTotalStrokes	 14
U+824D.0	fCangjie	HYSJR
U+824D.0	fCantonese	geoi1
U+824D.0	fMandarin	ju1
U+824E.0	fUTF8	艎
U+824E.0	fCNS11643	2-5333
U+824E.0	fBig5	E74A
U+824E.0	fGB	5-6529
U+824E.0	fR/S	 137.9
U+824E.0	fTotalStrokes	 15
U+824E.0	fCangjie	HYHAG
U+824E.0	fFourCorner	26411
U+824E.0	fHakka	  fong2
U+824E.0	fCantonese	wong4
U+824E.0	fMandarin	huang2
U+824E.0	fEnglish	fast boat
U+824F.0	fUTF8	艏
U+824F.0	fCNS11643	2-532F
U+824F.0	fBig5	E746
U+824F.0	fGB	0-743C
U+824F.0	fR/S	 137.9
U+824F.0	fTotalStrokes	 15
U+824F.0	fCangjie	HYTHU
U+824F.0	fFourCorner	28461
U+824F.0	fHakka	 shiu3; su3; shu3
U+824F.0	fCantonese	sau2
U+824F.0	fMandarin	shou3
U+824F.0	fEnglish	the prow of a vessel
U+8250.0	fUTF8	艐
U+8250.0	fCNS11643	2-5332
U+8250.0	fBig5	E749
U+8250.0	fGB	3-6569
U+8250.0	fR/S	 137.9
U+8250.0	fTotalStrokes	 15
U+8250.0	fCangjie	HYUCE
U+8250.0	fFourCorner	22448
U+8250.0	fHakka	  zung1
U+8250.0	fCantonese	ho3; zung1; gaai3
U+8250.0	fMandarin	zong1; ke4; jie4
U+8250.0	fEnglish	[1] runaground [2] reach
U+8251.0	fUTF8	艑
U+8251.0	fCNS11643	2-5334
U+8251.0	fBig5	E74B
U+8251.0	fGB	3-656D
U+8251.0	fR/S	 137.9
U+8251.0	fTotalStrokes	 15
U+8251.0	fCangjie	HYHSB
U+8251.0	fFourCorner	23427
U+8251.0	fHakka	 pien1; bien3
U+8251.0	fCantonese	bin6; bin1; pin1
U+8251.0	fMandarin	bian4; pian1
U+8251.0	fEnglish	small boat
U+8252.0	fUTF8	艒
U+8252.0	fCNS11643	2-5331
U+8252.0	fBig5	E748
U+8252.0	fGB	3-6568
U+8252.0	fR/S	 137.9
U+8252.0	fTotalStrokes	 15
U+8252.0	fCangjie	HYABU
U+8252.0	fFourCorner	26460
U+8252.0	fCantonese	muk6
U+8252.0	fMandarin	mu4; mao4; mo4
U+8253.0	fUTF8	艓
U+8253.0	fCNS11643	2-5330
U+8253.0	fBig5	E747
U+8253.0	fGB	3-6567
U+8253.0	fR/S	 137.9
U+8253.0	fTotalStrokes	 15
U+8253.0	fCangjie	HYPTD
U+8253.0	fFourCorner	24494
U+8253.0	fHakka	  tiap8
U+8253.0	fCantonese	dip6
U+8253.0	fMandarin	die2
U+8253.0	fEnglish	 small boat; punt
U+8254.0	fUTF8	艔
U+8254.0	fGB	1-7C4C
U+8254.0	fR/S	 137.9
U+8254.0	fTotalStrokes	 15
U+8254.0	fCangjie	HYITE
U+8254.0	fCantonese	dou6; dou2 (also)
U+8254.0	fEnglish	boat
U+8255.0	fUTF8	艕
U+8255.0	fCNS11643	2-5961
U+8255.0	fBig5	EAF5
U+8255.0	fGB	3-656F
U+8255.0	fR/S	 137.10
U+8255.0	fTotalStrokes	 16
U+8255.0	fCangjie	HYYBS
U+8255.0	fFourCorner	20427
U+8255.0	fHakka	 bong3
U+8255.0	fCantonese	bong3; pong3 (also); bong6
U+8255.0	fMandarin	bang4; beng4
U+8255.0	fEnglish	two boats fastened side by side
U+8256.0	fUTF8	艖
U+8256.0	fCNS11643	2-5962
U+8256.0	fBig5	EAF6
U+8256.0	fGB	3-656B
U+8256.0	fR/S	 137.10
U+8256.0	fTotalStrokes	 16
U+8256.0	fCangjie	HYTQM
U+8256.0	fFourCorner	28412
U+8256.0	fHakka	  ca1
U+8256.0	fCantonese	caa1; co1; caa4
U+8256.0	fMandarin	cha1; chai1; cuo2; zha4
U+8256.0	fEnglish	 raft boat
U+8257.0	fUTF8	艗
U+8257.0	fCNS11643	2-5963
U+8257.0	fBig5	EAF7
U+8257.0	fGB	5-652B
U+8257.0	fR/S	 137.10
U+8257.0	fTotalStrokes	 16
U+8257.0	fCangjie	HYTCT
U+8257.0	fFourCorner	28417
U+8257.0	fHakka	 ngiet8; jit8; rit8; jit7
U+8257.0	fCantonese	jik6; jat6
U+8257.0	fMandarin	yi4
U+8257.0	fEnglish	bow of junk
U+8258.0	fUTF8	艘
U+8258.0	fCNS11643	1-717C
U+8258.0	fBig5	BFB4
U+8258.0	fGB	0-4B52
U+8258.0	fR/S	 137.10
U+8258.0	fTotalStrokes	 16
U+8258.0	fCangjie	HYHXE
U+8258.0	fFourCorner	27447
U+8258.0	fHakka	  seu1;  seu3;  siau1; siu1
U+8258.0	fCantonese	sau1; sau2 (also)
U+8258.0	fMandarin	sou1; xiao1; sao1
U+8258.0	fEnglish	a numerical classifier for ships
U+8259.0	fUTF8	艙
U+8259.0	fCNS11643	1-717D
U+8259.0	fBig5	BFB5
U+8259.0	fGB	1-3255
U+8259.0	fR/S	 137.10
U+8259.0	fTotalStrokes	 16
U+8259.0	fCangjie	HYOIR
U+8259.0	fFourCorner	28467
U+8259.0	fHakka	  cong1
U+8259.0	fCantonese	cong1
U+8259.0	fMandarin	cang1
U+8259.0	fEnglish	 hold of a ship; cabin of a ship
U+825A.0	fUTF8	艚
U+825A.0	fCNS11643	2-5E5E
U+825A.0	fBig5	EDF1
U+825A.0	fGB	0-743D
U+825A.0	fR/S	 137.11
U+825A.0	fTotalStrokes	 17
U+825A.0	fCangjie	HYTWA
U+825A.0	fFourCorner	25466
U+825A.0	fHakka	 cau2; co2
U+825A.0	fCantonese	cou4
U+825A.0	fMandarin	cao2
U+825A.0	fEnglish	ship
U+825B.0	fUTF8	艛
U+825B.0	fCNS11643	2-5E5D
U+825B.0	fBig5	EDF0
U+825B.0	fGB	3-656C
U+825B.0	fR/S	 137.11
U+825B.0	fTotalStrokes	 17
U+825B.0	fCangjie	HYLLV
U+825B.0	fFourCorner	25444
U+825B.0	fHakka	 leu2
U+825B.0	fCantonese	lau4
U+825B.0	fMandarin	lou2
U+825B.0	fEnglish	war-junk with a high poop from which archers shoot
U+825C.0	fUTF8	艜
U+825C.0	fCNS11643	2-5E5F
U+825C.0	fBig5	EDF2
U+825C.0	fGB	5-6527
U+825C.0	fR/S	 137.11
U+825C.0	fTotalStrokes	 17
U+825C.0	fCangjie	HYKPB
U+825C.0	fFourCorner	24427
U+825C.0	fHakka	 dai5; {tai5}
U+825C.0	fCantonese	daai3; taai5
U+825C.0	fMandarin	dai4
U+825C.0	fEnglish	a long narrow vessel with two masts
U+825D.0	fUTF8	艝
U+825D.0	fGB	E-3A2D
U+825D.0	fR/S	 137.11
U+825D.0	fTotalStrokes	 17
U+825D.0	fCangjie	HYMBS
U+825D.0	fCantonese	syut3
U+825E.0	fUTF8	艞
U+825E.0	fCNS11643	2-626D
U+825E.0	fBig5	F0A3
U+825E.0	fGB	5-652F
U+825E.0	fR/S	 137.12
U+825E.0	fTotalStrokes	 18
U+825E.0	fCangjie	HYHLO
U+825E.0	fFourCorner	28413
U+825E.0	fHakka	 tiau5; rau5; jeu5; jau6; jau5
U+825E.0	fCantonese	jiu6; tiu3; tiu1
U+825E.0	fMandarin	yao4; tiao4
U+825E.0	fEnglish	 large boat
U+825F.0	fUTF8	艟
U+825F.0	fCNS11643	2-626C
U+825F.0	fBig5	F0A2
U+825F.0	fGB	0-743E
U+825F.0	fR/S	 137.12
U+825F.0	fTotalStrokes	 18
U+825F.0	fCangjie	HYYTG
U+825F.0	fFourCorner	20414
U+825F.0	fHakka	 tung2
U+825F.0	fCantonese	tung4 (also); cung1
U+825F.0	fMandarin	tong2; chong1; zhuang4
U+825F.0	fEnglish	 ancient warship
U+8260.0	fUTF8	艠
U+8260.0	fGB	E-3A2E
U+8260.0	fR/S	 137.12
U+8260.0	fTotalStrokes	 18
U+8260.0	fCangjie	HYNOT
U+8260.0	fCantonese	dang1
U+8261.0	fUTF8	艡
U+8261.0	fCNS11643	2-664F
U+8261.0	fBig5	F2C4
U+8261.0	fGB	5-647A
U+8261.0	fR/S	 137.13
U+8261.0	fTotalStrokes	 19
U+8261.0	fCangjie	HYFBW
U+8261.0	fFourCorner	29466
U+8261.0	fCantonese	dong1
U+8261.0	fMandarin	dang1; tang4
U+8262.0	fUTF8	艢
U+8262.0	fCNS11643	3-5A7A
U+8262.0	fGB	E-3A2F
U+8262.0	fR/S	 137.13
U+8262.0	fTotalStrokes	 19
U+8262.0	fCangjie	HYGOW
U+8262.0	fFourCorner	24460
U+8262.0	fHakka	  siong2; ciong2
U+8262.0	fCantonese	coeng4
U+8262.0	fMandarin	tan2; qiang2
U+8262.0	fEnglish	 mast yard-arm; boom
U+8263.0	fUTF8	艣
U+8263.0	fCNS11643	2-6650
U+8263.0	fBig5	F2C5
U+8263.0	fGB	E-3A30
U+8263.0	fR/S	 137.13
U+8263.0	fTotalStrokes	 19
U+8263.0	fCangjie	HYYPS
U+8263.0	fFourCorner	27463
U+8263.0	fCantonese	lou5
U+8263.0	fMandarin	lu3
U+8263.0	fEnglish	stern oar; scull [?]
U+8264.0	fUTF8	艤
U+8264.0	fCNS11643	2-664E
U+8264.0	fBig5	F2C3
U+8264.0	fGB	1-742F
U+8264.0	fR/S	 137.13
U+8264.0	fTotalStrokes	 19
U+8264.0	fCangjie	HYTGI
U+8264.0	fFourCorner	28453
U+8264.0	fHakka	 ngi5; ngi6
U+8264.0	fCantonese	ngai5; ai5 (also)
U+8264.0	fMandarin	yi3
U+8264.0	fEnglish	 moor (a boat) to the bank
U+8265.0	fUTF8	艥
U+8265.0	fCNS11643	3-5A7B
U+8265.0	fGB	5-652E
U+8265.0	fR/S	 137.13
U+8265.0	fTotalStrokes	 19; 18
U+8265.0	fCangjie	HYRJI
U+8265.0	fFourCorner	23450
U+8265.0	fHakka	  ziap7
U+8265.0	fCantonese	zip3
U+8265.0	fMandarin	jie4
U+8265.0	fEnglish	an oar or paddle. to row
U+8266.0	fUTF8	艦
U+8266.0	fCNS11643	1-7A30
U+8266.0	fBig5	C4A5
U+8266.0	fGB	1-3D22
U+8266.0	fR/S	 137.14
U+8266.0	fTotalStrokes	 20
U+8266.0	fCangjie	HYSIT
U+8266.0	fFourCorner	28417
U+8266.0	fHakka	 gam5; lam5
U+8266.0	fCantonese	laam6
U+8266.0	fMandarin	jian4
U+8266.0	fEnglish	 warship; man-of-war; naval vessel
U+8267.0	fUTF8	艧
U+8267.0	fCNS11643	4-642C
U+8267.0	fGB	E-3A31
U+8267.0	fR/S	  137.14
U+8267.0	fTotalStrokes	 20
U+8267.0	fCangjie	HYTOE
U+8267.0	fFourCorner	24440
U+8267.0	fHakka	 wok7; vok7
U+8267.0	fCantonese	wok6
U+8267.0	fMandarin	huo4
U+8267.0	fEnglish	red paint
U+8268.0	fUTF8	艨
U+8268.0	fCNS11643	2-6962
U+8268.0	fBig5	F4B6
U+8268.0	fGB	0-743F
U+8268.0	fR/S	 137.14
U+8268.0	fTotalStrokes	 20
U+8268.0	fCangjie	HYTBO
U+8268.0	fFourCorner	24432
U+8268.0	fHakka	  mung2
U+8268.0	fCantonese	mung4
U+8268.0	fMandarin	meng2; meng4
U+8268.0	fEnglish	 ancient warship
U+8269.0	fUTF8	艩
U+8269.0	fCNS11643	2-6963
U+8269.0	fBig5	F4B7
U+8269.0	fGB	3-6560
U+8269.0	fR/S	 137.14
U+8269.0	fTotalStrokes	 20
U+8269.0	fCangjie	HYYX
U+8269.0	fFourCorner	20423
U+8269.0	fHakka	 ci2
U+8269.0	fCantonese	cai4
U+8269.0	fMandarin	qi2
U+8269.0	fEnglish	tholepin
U+826A.0	fUTF8	艪
U+826A.0	fCNS11643	3-5E55
U+826A.0	fGB	E-3A32
U+826A.0	fR/S	  137.15
U+826A.0	fTotalStrokes	 21
U+826A.0	fCangjie	HYNWA
U+826A.0	fFourCorner	27460
U+826A.0	fHakka	 lu3
U+826A.0	fCantonese	lou5
U+826A.0	fMandarin	lu3
U+826A.0	fEnglish	 oar; scull row
U+826B.0	fUTF8	艫
U+826B.0	fCNS11643	2-6D73
U+826B.0	fBig5	F746
U+826B.0	fGB	1-7435
U+826B.0	fR/S	 137.16
U+826B.0	fTotalStrokes	 22
U+826B.0	fCangjie	HYYPT
U+826B.0	fFourCorner	21417
U+826B.0	fHakka	 lu2
U+826B.0	fCantonese	lou4
U+826B.0	fMandarin	lu2; lu3
U+826B.0	fEnglish	the bow or prow of a boat
U+826C.0	fUTF8	艬
U+826C.0	fCNS11643	2-6F3E
U+826C.0	fBig5	F7EF
U+826C.0	fGB	3-6577
U+826C.0	fR/S	 137.17
U+826C.0	fTotalStrokes	 23
U+826C.0	fCangjie	HYNRI
U+826C.0	fFourCorner	27413
U+826C.0	fCantonese	caam4
U+826C.0	fMandarin	chan2; chan4
U+826D.0	fUTF8	艭
U+826D.0	fCNS11643	2-7049
U+826D.0	fBig5	F8BB
U+826D.0	fGB	3-6558
U+826D.0	fR/S	 137.18
U+826D.0	fTotalStrokes	 24
U+826D.0	fCangjie	HYOGE
U+826D.0	fFourCorner	20447
U+826D.0	fHakka	 sung1
U+826D.0	fCantonese	soeng1
U+826D.0	fMandarin	shuang1
U+826D.0	fEnglish	ship
U+826E.0	fUTF8	艮
U+826E.0	fCNS11643	1-4862
U+826E.0	fBig5	A6E1
U+826E.0	fGB	0-745E
U+826E.0	fR/S	 138.0
U+826E.0	fTotalStrokes	 6
U+826E.0	fCangjie	AV
U+826E.0	fFourCorner	77732
U+826E.0	fHakka	  gen5; gien5
U+826E.0	fCantonese	gan3; ngan4
U+826E.0	fMandarin	gen4; yin2; hen2
U+826E.0	fEnglish	the seventh of the Eight diagrams for divination
U+826E.1	fUTF8	艮
U+826E.1	fCNS11643	1-4862
U+826E.1	fBig5	A6E1
U+826E.1	fGB	0-745E
U+826E.1	fR/S	 138.0
U+826E.1	fTotalStrokes	 6
U+826E.1	fCangjie	AV
U+826E.1	fFourCorner	77732
U+826E.1	fMandarin	gen3
U+826E.1	fEnglish	[1] tough; leathery (food) [2] straightforward; outspoken; blunt [3] simple (clothing) [4] honest; upright (personality)
U+826F.0	fUTF8	良
U+826F.0	fCNS11643	1-4B40
U+826F.0	fBig5	A87D
U+826F.0	fGB	0-413C
U+826F.0	fR/S	 138.1
U+826F.0	fTotalStrokes	 7
U+826F.0	fCangjie	IAV
U+826F.0	fFourCorner	30732
U+826F.0	fHakka	  liong2; liong3
U+826F.0	fCantonese	loeng4
U+826F.0	fMandarin	liang2; liang3
U+826F.0	fEnglish	[1] good; fine; desirable; virtuous; respectable [2] very [3] instinctive; inborn; innate
U+8270.0	fUTF8	艰
U+8270.0	fGB	0-3C68
U+8270.0	fR/S	 138.2; 29.6
U+8270.0	fTotalStrokes	 8
U+8270.0	fCangjie	EAV
U+8270.0	fHakka	 gen1; gan1; gien1
U+8270.0	fCantonese	gaan1; gan1
U+8270.0	fMandarin	jian1
U+8270.0	fEnglish	simplified form of &#x8271;
U+8271.0	fUTF8	艱
U+8271.0	fCNS11643	1-7529
U+8271.0	fBig5	C17D
U+8271.0	fGB	1-3C68
U+8271.0	fR/S	 138.11
U+8271.0	fTotalStrokes	 17
U+8271.0	fCangjie	TOAV
U+8271.0	fFourCorner	47532
U+8271.0	fHakka	 gan1; gien1;  gen1; gian1
U+8271.0	fCantonese	gaan1; gan1
U+8271.0	fMandarin	jian1
U+8271.0	fEnglish	[1] difficult; hard; trying [2] mourning for one's parents
U+8272.0	fUTF8	色
U+8272.0	fCNS11643	1-4863
U+8272.0	fBig5	A6E2
U+8272.0	fGB	0-492B
U+8272.0	fR/S	 139.0
U+8272.0	fTotalStrokes	 6
U+8272.0	fCangjie	NAU
U+8272.0	fFourCorner	27717
U+8272.0	fHakka	  set7; siet7; sek7
U+8272.0	fCantonese	sik1
U+8272.0	fMandarin	se4; shai3; she4
U+8272.0	fEnglish	[1] colour; tint; tinge; hue; shade [2] facial expression; look; appearance [3] sensuality; desire for beauty; lust; lewdness; carnal pleasure [4] worldly things [5] a kind; sort [6] Kangxi radical 139
U+8273.0	fUTF8	艳
U+8273.0	fCNS11643	F-3232
U+8273.0	fGB	0-515E
U+8273.0	fR/S	 139.4
U+8273.0	fTotalStrokes	 10
U+8273.0	fCangjie	QJNAU
U+8273.0	fHakka	 jam5; jam3
U+8273.0	fCantonese	jim6
U+8273.0	fMandarin	yan4
U+8273.0	fEnglish	simplified form of &#x8277;
U+8274.0	fUTF8	艴
U+8274.0	fCNS11643	2-385C
U+8274.0	fBig5	D758
U+8274.0	fGB	0-6575
U+8274.0	fR/S	 139.5
U+8274.0	fTotalStrokes	 11
U+8274.0	fCangjie	LNNAU
U+8274.0	fFourCorner	57017
U+8274.0	fHakka	 fut7
U+8274.0	fCantonese	fat1; but6
U+8274.0	fMandarin	fu2; bo2; pei4
U+8274.0	fEnglish	 look angry
U+8275.0	fUTF8	艵
U+8275.0	fCNS11643	2-3F40
U+8275.0	fBig5	DB5B
U+8275.0	fGB	3-657A
U+8275.0	fR/S	 139.8
U+8275.0	fTotalStrokes	 14
U+8275.0	fCangjie	TTNAU
U+8275.0	fFourCorner	87417
U+8275.0	fCantonese	ping1
U+8275.0	fMandarin	ping2; ping1; ping3
U+8276.0	fUTF8	艶
U+8276.0	fCNS11643	3-5A7C
U+8276.0	fGB	E-3A33
U+8276.0	fR/S	  139.13
U+8276.0	fTotalStrokes	 19
U+8276.0	fCangjie	TTNAU
U+8276.0	fFourCorner	57117
U+8276.0	fHakka	  jam5; jam3; ram5; jam6
U+8276.0	fCantonese	jim6
U+8276.0	fMandarin	yan4
U+8276.0	fEnglish	beautiful; sexy; voluptuous
U+8277.0	fUTF8	艷
U+8277.0	fCNS11643	1-7C6C
U+8277.0	fBig5	C641
U+8277.0	fGB	1-515E
U+8277.0	fR/S	 139.18
U+8277.0	fTotalStrokes	 24
U+8277.0	fCangjie	UTNAU
U+8277.0	fFourCorner	57117
U+8277.0	fHakka	  jam5;  jam3; ram6; ram5
U+8277.0	fCantonese	jim6
U+8277.0	fMandarin	yan4
U+8277.0	fEnglish	beautiful; sexy; voluptuous
U+8278.0	fUTF8	艸
U+8278.0	fCNS11643	2-226A
U+8278.0	fBig5	CA4A
U+8278.0	fGB	5-6738
U+8278.0	fR/S	 140.0
U+8278.0	fTotalStrokes	 6
U+8278.0	fCangjie	UU
U+8278.0	fFourCorner	22447
U+8278.0	fHakka	 cau3; co3
U+8278.0	fCantonese	cou2; mong5
U+8278.0	fMandarin	cao3; zao4
U+8278.0	fEnglish	[1] grass; straw; weeds [2] Kangxi radical 140
U+8279.0	fUTF8	艹
U+8279.0	fCNS11643	4-2153
U+8279.0	fGB	0-5C33
U+8279.0	fR/S	 140.0
U+8279.0	fTotalStrokes	 4; 6
U+8279.0	fCangjie	XJJ
U+8279.0	fFourCorner	44000
U+8279.0	fCantonese	je6; jaa6
U+8279.0	fMandarin	cao3
U+8279.0	fEnglish	 grass Kangxi radical number 140
U+827A.0	fUTF8	艺
U+827A.0	fGB	0-5255
U+827A.0	fR/S	 140.1
U+827A.0	fTotalStrokes	 5; 7
U+827A.0	fCangjie	TN
U+827A.0	fHakka	 ngi3; ngi5
U+827A.0	fCantonese	ngai6; ai6 (also)
U+827A.0	fMandarin	yi4; nie4
U+827A.0	fEnglish	simplified form of &#x85DD;
U+827B.0	fUTF8	艻
U+827B.0	fCNS11643	3-2438
U+827B.0	fGB	E-3A34
U+827B.0	fR/S	 140.2
U+827B.0	fTotalStrokes	 6; 8
U+827B.0	fCangjie	TKS
U+827B.0	fFourCorner	44420
U+827B.0	fHakka	 net7;  let8; lit8
U+827B.0	fCantonese	lak6; gik1
U+827B.0	fMandarin	le4; ji2
U+827B.0	fEnglish	thorn
U+827C.0	fUTF8	艼
U+827C.0	fCNS11643	2-226B
U+827C.0	fBig5	CA4B
U+827C.0	fGB	5-6739
U+827C.0	fR/S	 140.2
U+827C.0	fTotalStrokes	 6; 8
U+827C.0	fCangjie	TMN
U+827C.0	fFourCorner	44201
U+827C.0	fCantonese	ding2
U+827C.0	fMandarin	ting1; ding3; ting3
U+827D.0	fUTF8	艽
U+827D.0	fCNS11643	2-226D
U+827D.0	fBig5	CA4D
U+827D.0	fGB	0-5C34
U+827D.0	fR/S	 140.2
U+827D.0	fTotalStrokes	 6; 8
U+827D.0	fCangjie	TKN
U+827D.0	fFourCorner	44417
U+827D.0	fHakka	 kiu2; kieu1; gau1; geu1; keu2
U+827D.0	fCantonese	kau4; gaau1; gau2; gau1
U+827D.0	fMandarin	qiu2; jiao1; jiu3; kui2
U+827D.0	fEnglish	a medicinal plant with leaves like lettuce
U+827E.0	fUTF8	艾
U+827E.0	fCNS11643	1-4864
U+827E.0	fBig5	A6E3
U+827E.0	fGB	0-302C
U+827E.0	fR/S	 140.2
U+827E.0	fTotalStrokes	 6; 8
U+827E.0	fCangjie	TK
U+827E.0	fFourCorner	44400
U+827E.0	fHakka	  ngioi5;  ngie5;  ngai5; gne5; ngioi1; ngiai5; nge5
U+827E.0	fCantonese	ngaai6; aai6 (also)
U+827E.0	fMandarin	ai4; yi4
U+827E.0	fEnglish	[1] Mugwort, artemisia, or any plant from which moxa is obtained [2] fine hair; beautiful; good [3] old; fifty years old [4] finish; end; cease; stop; discontinue [5] a general name for labiate plants, like mint [6] a Chinese family name
U+827F.0	fUTF8	艿
U+827F.0	fCNS11643	2-226E
U+827F.0	fBig5	CA4E
U+827F.0	fGB	0-5C35
U+827F.0	fR/S	 140.2
U+827F.0	fTotalStrokes	 6; 8
U+827F.0	fCangjie	TNHS
U+827F.0	fFourCorner	44227
U+827F.0	fHakka	 jin2; rin2
U+827F.0	fCantonese	naai5; jing4; laai5 (also); 2laai5
U+827F.0	fMandarin	reng2; nai3; reng4; er2
U+827F.0	fEnglish	[1] a kind of vegetable [2] old roots; stumps of plants
U+8280.0	fUTF8	芀
U+8280.0	fCNS11643	2-226C
U+8280.0	fBig5	CA4C
U+8280.0	fGB	3-682F
U+8280.0	fR/S	 140.2
U+8280.0	fTotalStrokes	 6; 8
U+8280.0	fCangjie	TSH
U+8280.0	fFourCorner	44227
U+8280.0	fHakka	 tiau2
U+8280.0	fCantonese	tiu4; siu4; lik6
U+8280.0	fMandarin	tiao2; shao2
U+8280.0	fEnglish	clover; pea
U+8281.0	fUTF8	芁
U+8281.0	fCNS11643	4-232F
U+8281.0	fGB	5-673C
U+8281.0	fR/S	 140.2
U+8281.0	fTotalStrokes	 6; 8
U+8281.0	fCangjie	THN
U+8281.0	fFourCorner	44210
U+8281.0	fCantonese	kau4
U+8281.0	fMandarin	jiao1; jiu1
U+8282.0	fUTF8	节
U+8282.0	fGB	0-3D5A
U+8282.0	fR/S	 140.2
U+8282.0	fTotalStrokes	 6; 8
U+8282.0	fCangjie	TSL
U+8282.0	fHakka	 ziet7
U+8282.0	fCantonese	zit3
U+8282.0	fMandarin	jie2; jie1
U+8282.0	fEnglish	simplified form of &#x7BC0;
U+8283.0	fUTF8	芃
U+8283.0	fCNS11643	2-2522
U+8283.0	fBig5	CBA2
U+8283.0	fGB	3-6835
U+8283.0	fR/S	 140.3
U+8283.0	fTotalStrokes	 7; 9
U+8283.0	fCangjie	THNI
U+8283.0	fFourCorner	44217
U+8283.0	fHakka	 pung2; pung3
U+8283.0	fCantonese	pung4
U+8283.0	fMandarin	peng2; feng2
U+8283.0	fEnglish	luxuriant growth; lush
U+8284.0	fUTF8	芄
U+8284.0	fCNS11643	2-2523
U+8284.0	fBig5	CBA3
U+8284.0	fGB	0-5C39
U+8284.0	fR/S	 140.3
U+8284.0	fTotalStrokes	 7; 9
U+8284.0	fCangjie	TKNI
U+8284.0	fFourCorner	44517
U+8284.0	fHakka	 jen2; ren2; jan2
U+8284.0	fCantonese	jyun4
U+8284.0	fMandarin	wan2
U+8284.0	fEnglish	Metaplexis japonica
U+8285.0	fUTF8	芅
U+8285.0	fCNS11643	2-247B
U+8285.0	fBig5	CB7B
U+8285.0	fGB	3-6833
U+8285.0	fR/S	 140.3
U+8285.0	fTotalStrokes	 7; 9
U+8285.0	fCangjie	TIP
U+8285.0	fFourCorner	44043
U+8285.0	fCantonese	jik6
U+8285.0	fMandarin	yi4
U+8286.0	fUTF8	芆
U+8286.0	fCNS11643	3-2728
U+8286.0	fGB	3-6837
U+8286.0	fR/S	  140.3
U+8286.0	fTotalStrokes	 7; 9
U+8286.0	fCangjie	TIK
U+8286.0	fFourCorner	44400
U+8286.0	fCantonese	ngaai6; aai6 (also)
U+8286.0	fMandarin	chai1; cha1
U+8287.0	fUTF8	芇
U+8287.0	fCNS11643	4-247B
U+8287.0	fGB	5-6742
U+8287.0	fR/S	  140.3
U+8287.0	fTotalStrokes	 7; 9
U+8287.0	fCangjie	TLB
U+8287.0	fFourCorner	44220
U+8287.0	fCantonese	min4
U+8287.0	fMandarin	mian2; mian3
U+8288.0	fUTF8	芈
U+8288.0	fGB	0-5842
U+8288.0	fR/S	 140.3; 123.1
U+8288.0	fTotalStrokes	 7; 9
U+8288.0	fCangjie	XTQ
U+8288.0	fCantonese	me1; me5 (also); mei5; me2
U+8288.0	fMandarin	mi3; mie1
U+8288.0	fEnglish	 bleat of sheep a Chinese family name
U+8289.0	fUTF8	芉
U+8289.0	fCNS11643	4-2475
U+8289.0	fGB	3-6830
U+8289.0	fR/S	  140.3
U+8289.0	fTotalStrokes	 7; 9
U+8289.0	fCangjie	TMJ
U+8289.0	fFourCorner	44400
U+8289.0	fCantonese	gon1
U+8289.0	fMandarin	gan1; gan3
U+828A.0	fUTF8	芊
U+828A.0	fCNS11643	2-2521
U+828A.0	fBig5	CBA1
U+828A.0	fGB	0-5C37
U+828A.0	fR/S	 140.3
U+828A.0	fTotalStrokes	 7; 9
U+828A.0	fCangjie	THJ
U+828A.0	fFourCorner	44402
U+828A.0	fHakka	  cien1; cen1
U+828A.0	fCantonese	cin1
U+828A.0	fMandarin	qian1; qian4
U+828A.0	fEnglish	exuberant and vigorous foliage; lush; green (grass)
U+828B.0	fUTF8	芋
U+828B.0	fCNS11643	1-4B42
U+828B.0	fBig5	A8A1
U+828B.0	fGB	0-5373
U+828B.0	fR/S	 140.3
U+828B.0	fTotalStrokes	 7; 9
U+828B.0	fCangjie	TMD
U+828B.0	fFourCorner	44401
U+828B.0	fHakka	 wu3; wu6; wu5; ri6; ji5; i5; vu3
U+828B.0	fCantonese	wu6; jyu1; wu2 (also)
U+828B.0	fMandarin	yu4; yu3; hu1; xu1; yu2
U+828B.0	fEnglish	 taro
U+828C.0	fUTF8	芌
U+828C.0	fCNS11643	3-2729
U+828C.0	fGB	E-3A35
U+828C.0	fR/S	 140.3
U+828C.0	fTotalStrokes	 7; 9
U+828C.0	fCangjie	TMMS
U+828C.0	fFourCorner	44720
U+828C.0	fCantonese	wu6
U+828C.0	fMandarin	yu4; yu2; xu1; yu3
U+828D.0	fUTF8	芍
U+828D.0	fCNS11643	1-4B43
U+828D.0	fBig5	A8A2
U+828D.0	fGB	0-4956
U+828D.0	fR/S	 140.3
U+828D.0	fTotalStrokes	 7; 9
U+828D.0	fCangjie	TPI
U+828D.0	fFourCorner	44127
U+828D.0	fHakka	 shok7; sok7; zok7; sok8; shok8; chok7
U+828D.0	fCantonese	coek3; zoek3
U+828D.0	fMandarin	xiao4; shao2; shuo4; que4; di4; zhuo2; ruo4
U+828D.0	fEnglish	peony (Paeonia)
U+828E.0	fUTF8	芎
U+828E.0	fCNS11643	2-247C
U+828E.0	fBig5	CB7C
U+828E.0	fGB	0-5C3A
U+828E.0	fR/S	 140.3
U+828E.0	fTotalStrokes	 7; 9
U+828E.0	fCangjie	TN
U+828E.0	fFourCorner	44727
U+828E.0	fHakka	  giung1
U+828E.0	fCantonese	gung1; hung1 (also); kung4 (also)
U+828E.0	fMandarin	qiong1; xiong1; qiong2
U+828E.0	fEnglish	a kind of herb (Cnidium officinale)
U+828F.0	fUTF8	芏
U+828F.0	fCNS11643	2-247A
U+828F.0	fBig5	CB7A
U+828F.0	fGB	0-5C36
U+828F.0	fR/S	 140.3
U+828F.0	fTotalStrokes	 7; 9
U+828F.0	fCangjie	TG
U+828F.0	fFourCorner	44104
U+828F.0	fHakka	 tu3; tu5
U+828F.0	fCantonese	dou6; tou2
U+828F.0	fMandarin	tu3; du4
U+828F.0	fEnglish	a plant (Cyperus tegetiformis) used for making mats
U+8290.0	fUTF8	芐
U+8290.0	fCNS11643	2-2479
U+8290.0	fBig5	CB79
U+8290.0	fGB	3-6831
U+8290.0	fR/S	 140.3
U+8290.0	fTotalStrokes	 7; 9
U+8290.0	fCangjie	TMY
U+8290.0	fFourCorner	44231
U+8290.0	fHakka	 fu5
U+8290.0	fCantonese	wu6; haa6 (also)
U+8290.0	fMandarin	hu4; xia4
U+8290.0	fEnglish	fox-glove; Rehmannia glutinosa
U+8291.0	fUTF8	芑
U+8291.0	fCNS11643	2-247D
U+8291.0	fBig5	CB7D
U+8291.0	fGB	0-5C3B
U+8291.0	fR/S	 140.3
U+8291.0	fTotalStrokes	 7; 9
U+8291.0	fCangjie	TSU
U+8291.0	fFourCorner	44717
U+8291.0	fHakka	 hi3; ki3
U+8291.0	fCantonese	hei2; gei2
U+8291.0	fMandarin	qi3; ji4
U+8291.0	fEnglish	white millet
U+8292.0	fUTF8	芒
U+8292.0	fCNS11643	1-4B41
U+8292.0	fBig5	A87E
U+8292.0	fGB	0-4322
U+8292.0	fR/S	 140.3
U+8292.0	fTotalStrokes	 7; 9
U+8292.0	fCangjie	TYV
U+8292.0	fFourCorner	44710
U+8292.0	fHakka	  mong2; miong2; ngiong2; mong1; miong3
U+8292.0	fCantonese	mong4; fong2; mong1
U+8292.0	fMandarin	mang2; wang2; huang1; huang3; mang3
U+8292.0	fEnglish	[1] Miscanthus sinensis, a kind of grass whose leaves can be used to make sandals [2] sharp point [3] awn; arista; beard of wheat (botany) [4] rays (stars) [5] a Chinese family name
U+8293.0	fUTF8	芓
U+8293.0	fCNS11643	2-247E
U+8293.0	fBig5	CB7E
U+8293.0	fGB	5-6747
U+8293.0	fR/S	 140.3
U+8293.0	fTotalStrokes	 7; 9
U+8293.0	fCangjie	TND
U+8293.0	fFourCorner	44407
U+8293.0	fCantonese	zi6; zi2
U+8293.0	fMandarin	zi3; zi4; zi1
U+8294.0	fUTF8	芔
U+8294.0	fCNS11643	2-2D2D
U+8294.0	fBig5	D06A
U+8294.0	fGB	5-3F3B
U+8294.0	fR/S	 140.3
U+8294.0	fTotalStrokes	 9
U+8294.0	fCangjie	UUU
U+8294.0	fFourCorner	22447
U+8294.0	fHakka	 fui3; fi3; wi3; wui3; vui3
U+8294.0	fCantonese	wai2; wai3
U+8294.0	fMandarin	hui3; hui4
U+8294.0	fEnglish	a general term for plants
U+8295.0	fUTF8	芕
U+8295.0	fCNS11643	3-272B
U+8295.0	fGB	E-3A36
U+8295.0	fR/S	 140.3
U+8295.0	fTotalStrokes	 7; 9
U+8295.0	fCangjie	TNI
U+8295.0	fFourCorner	44200
U+8295.0	fCantonese	zik6
U+8295.0	fMandarin	sui1; wei3
U+8296.0	fUTF8	芖
U+8296.0	fCNS11643	4-2479
U+8296.0	fGB	E-3A37
U+8296.0	fR/S	  140.3
U+8296.0	fTotalStrokes	 7; 9
U+8296.0	fCangjie	TK
U+8296.0	fFourCorner	44430
U+8296.0	fCantonese	zi6
U+8296.0	fMandarin	zhi4
U+8297.0	fUTF8	芗
U+8297.0	fGB	0-5C3C
U+8297.0	fR/S	 140.3
U+8297.0	fTotalStrokes	 7; 9
U+8297.0	fCangjie	TVVH
U+8297.0	fHakka	 hiong1
U+8297.0	fCantonese	hoeng1; hoeng2
U+8297.0	fMandarin	xiang1; xiang3
U+8297.0	fEnglish	simplified form of &#x858C;
U+8298.0	fUTF8	芘
U+8298.0	fCNS11643	2-2856
U+8298.0	fBig5	CDB6
U+8298.0	fGB	0-5C45
U+8298.0	fR/S	 140.4
U+8298.0	fTotalStrokes	 8; 10
U+8298.0	fCangjie	TPP
U+8298.0	fFourCorner	44711
U+8298.0	fHakka	 pi5; bi5; pi2
U+8298.0	fCantonese	bei3; pei4
U+8298.0	fMandarin	pi2; bi4; bi1
U+8298.0	fEnglish	a thorny plant
U+8299.0	fUTF8	芙
U+8299.0	fCNS11643	1-4F3F
U+8299.0	fBig5	AADC
U+8299.0	fGB	0-5C3D
U+8299.0	fR/S	 140.4
U+8299.0	fTotalStrokes	 8; 10
U+8299.0	fCangjie	TQO
U+8299.0	fFourCorner	44530
U+8299.0	fHakka	  fu2; pu2; fu3
U+8299.0	fCantonese	fu4
U+8299.0	fMandarin	fu2
U+8299.0	fEnglish	 hibiscus
U+829A.0	fUTF8	芚
U+829A.0	fCNS11643	2-2855
U+829A.0	fBig5	CDB5
U+829A.0	fGB	3-683C
U+829A.0	fR/S	 140.4
U+829A.0	fTotalStrokes	 8; 10
U+829A.0	fCangjie	TPU
U+829A.0	fFourCorner	44714
U+829A.0	fCantonese	tyun4; ceon1; teon1 (also)
U+829A.0	fMandarin	tun2; chun1
U+829A.0	fEnglish	a kind of vegetable
U+829B.0	fUTF8	芛
U+829B.0	fCNS11643	2-2857
U+829B.0	fBig5	CDB7
U+829B.0	fGB	3-6847
U+829B.0	fR/S	 140.4
U+829B.0	fTotalStrokes	 8; 10
U+829B.0	fCangjie	TSK
U+829B.0	fFourCorner	44507
U+829B.0	fCantonese	wai5; jyut6
U+829B.0	fMandarin	wei3; shu4; sun3; yu4
U+829C.0	fUTF8	芜
U+829C.0	fCNS11643	3-2A60
U+829C.0	fGB	0-4E5F
U+829C.0	fR/S	  140.4
U+829C.0	fTotalStrokes	 8; 10
U+829C.0	fCangjie	TMKU
U+829C.0	fFourCorner	44410
U+829C.0	fHakka	 vu2
U+829C.0	fCantonese	mou4
U+829C.0	fMandarin	wu2; wu3; wu4
U+829C.0	fEnglish	simplified form of &#x856A;
U+829D.0	fUTF8	芝
U+829D.0	fCNS11643	1-4F3E
U+829D.0	fBig5	AADB
U+829D.0	fGB	0-5625
U+829D.0	fR/S	 140.4
U+829D.0	fTotalStrokes	 8; 10
U+829D.0	fCangjie	TINO
U+829D.0	fFourCorner	44307
U+829D.0	fHakka	  zih1; zu1; zi1; tje1; zhi1
U+829D.0	fCantonese	zi1; zi4 (also)
U+829D.0	fMandarin	zhi1
U+829D.0	fEnglish	[1] purplish mushroom symbolising nobility [2] a kind of fragrant herb [3] a Chinese family name
U+829E.0	fUTF8	芞
U+829E.0	fCNS11643	2-285C
U+829E.0	fBig5	CDBC
U+829E.0	fGB	5-674D
U+829E.0	fR/S	 140.4
U+829E.0	fTotalStrokes	 8; 10
U+829E.0	fCangjie	TOMN
U+829E.0	fFourCorner	44017
U+829E.0	fHakka	 kiet7; ket7
U+829E.0	fCantonese	hat1; kit3
U+829E.0	fMandarin	qi3; xi4
U+829E.0	fEnglish	a fragrant plant or grass, offered at the autumn festival
U+829F.0	fUTF8	芟
U+829F.0	fCNS11643	1-4F42
U+829F.0	fBig5	AADF
U+829F.0	fGB	0-5C4F
U+829F.0	fR/S	 140.4
U+829F.0	fTotalStrokes	 8; 10
U+829F.0	fCangjie	THNE
U+829F.0	fFourCorner	44407
U+829F.0	fHakka	 san1;  sam1
U+829F.0	fCantonese	saam1; saan1 (also)
U+829F.0	fMandarin	shan1; wei3
U+829F.0	fEnglish	[1] mow; cut down; weed out [2] scythe [3] eliminate; exterminate
U+82A0.0	fUTF8	芠
U+82A0.0	fCNS11643	2-2852
U+82A0.0	fBig5	CDB2
U+82A0.0	fGB	3-6843
U+82A0.0	fR/S	 140.4
U+82A0.0	fTotalStrokes	 8; 10
U+82A0.0	fCangjie	TYK
U+82A0.0	fFourCorner	44401
U+82A0.0	fCantonese	man4
U+82A0.0	fMandarin	wen2
U+82A1.0	fUTF8	芡
U+82A1.0	fCNS11643	2-2860
U+82A1.0	fBig5	CDC0
U+82A1.0	fGB	0-5C4D
U+82A1.0	fR/S	 140.4
U+82A1.0	fTotalStrokes	 8; 10
U+82A1.0	fCangjie	TNO
U+82A1.0	fFourCorner	44802
U+82A1.0	fHakka	  kiam5
U+82A1.0	fCantonese	him3; hin3
U+82A1.0	fMandarin	qian4
U+82A1.0	fEnglish	Gorgon euryale; Euryale fox (botany)
U+82A2.0	fUTF8	芢
U+82A2.0	fCNS11643	2-2866
U+82A2.0	fBig5	CDC6
U+82A2.0	fGB	3-683F
U+82A2.0	fR/S	 140.4
U+82A2.0	fTotalStrokes	 8; 10
U+82A2.0	fCangjie	TOMM
U+82A2.0	fFourCorner	44211
U+82A2.0	fCantonese	jan4
U+82A2.0	fMandarin	ren2
U+82A3.0	fUTF8	芣
U+82A3.0	fCNS11643	1-4F49
U+82A3.0	fBig5	AAE6
U+82A3.0	fGB	3-6839
U+82A3.0	fR/S	 140.4
U+82A3.0	fTotalStrokes	 8; 10
U+82A3.0	fCangjie	TMF
U+82A3.0	fFourCorner	44901
U+82A3.0	fHakka	 feu2; fu2; pi1
U+82A3.0	fCantonese	fau4
U+82A3.0	fMandarin	fou2; fou3; fu2; fou1; fu1
U+82A3.0	fEnglish	 plantain
U+82A4.0	fUTF8	芤
U+82A4.0	fCNS11643	2-2863
U+82A4.0	fBig5	CDC3
U+82A4.0	fGB	0-5C52
U+82A4.0	fR/S	 140.4
U+82A4.0	fTotalStrokes	 8; 10
U+82A4.0	fCangjie	TNDU
U+82A4.0	fFourCorner	44410
U+82A4.0	fHakka	 k'ieu1; keu1
U+82A4.0	fCantonese	kau1; hau1 (also)
U+82A4.0	fMandarin	kou1
U+82A4.0	fEnglish	stalk of an onion, hollow
U+82A5.0	fUTF8	芥
U+82A5.0	fCNS11643	1-4F46
U+82A5.0	fBig5	AAE3
U+82A5.0	fGB	0-3D66
U+82A5.0	fR/S	 140.4
U+82A5.0	fTotalStrokes	 8; 10
U+82A5.0	fCangjie	TOLL
U+82A5.0	fFourCorner	44228
U+82A5.0	fHakka	  gai5; kie5; ke5; giai5
U+82A5.0	fCantonese	gaai3
U+82A5.0	fMandarin	jie4; gai4; jia2
U+82A5.0	fEnglish	[1] mustard plant [2] tiny
U+82A6.0	fUTF8	芦
U+82A6.0	fCNS11643	3-2A6B
U+82A6.0	fGB	0-422B
U+82A6.0	fR/S	  140.4
U+82A6.0	fTotalStrokes	 8; 10
U+82A6.0	fCangjie	TIS
U+82A6.0	fFourCorner	44270
U+82A6.0	fHakka	 lu2
U+82A6.0	fCantonese	lou4; lou2 (also)
U+82A6.0	fMandarin	xia4; lu2; lu3; luo2
U+82A6.0	fEnglish	simplified form of &#x8606;
U+82A6.1	fUTF8	芦
U+82A6.1	fCNS11643	3-2A6B
U+82A6.1	fGB	0-422B
U+82A6.1	fR/S	  140.4
U+82A6.1	fTotalStrokes	 8; 10
U+82A6.1	fCangjie	TIS
U+82A6.1	fFourCorner	44270
U+82A6.1	fMandarin	lu5
U+82A7.0	fUTF8	芧
U+82A7.0	fCNS11643	2-2859
U+82A7.0	fBig5	CDB9
U+82A7.0	fGB	3-684A
U+82A7.0	fR/S	 140.4
U+82A7.0	fTotalStrokes	 8; 10
U+82A7.0	fCangjie	TNIN
U+82A7.0	fFourCorner	44407
U+82A7.0	fHakka	 ji2; chu3; ri2; cu3; si5
U+82A7.0	fCantonese	zeoi6; cyu5; seoi6
U+82A7.0	fMandarin	xu4; zhu4; yu2
U+82A7.0	fEnglish	a kind of chestnut oak
U+82A8.0	fUTF8	芨
U+82A8.0	fCNS11643	2-285F
U+82A8.0	fBig5	CDBF
U+82A8.0	fGB	0-5C38
U+82A8.0	fR/S	 140.4
U+82A8.0	fTotalStrokes	 8; 10
U+82A8.0	fCangjie	TNHE
U+82A8.0	fFourCorner	44247
U+82A8.0	fHakka	  kip7; gip7
U+82A8.0	fCantonese	gap1 (also); kap6; kap1
U+82A8.0	fMandarin	ji1; ji2
U+82A8.0	fEnglish	name of a plant; the roots are used for various pur- poses
U+82A9.0	fUTF8	芩
U+82A9.0	fCNS11643	2-2861
U+82A9.0	fBig5	CDC1
U+82A9.0	fGB	0-5C4B
U+82A9.0	fR/S	 140.4
U+82A9.0	fTotalStrokes	 8; 10
U+82A9.0	fCangjie	TOIN
U+82A9.0	fFourCorner	44207
U+82A9.0	fHakka	  kim2; kim3
U+82A9.0	fCantonese	kam4; sam4 (also)
U+82A9.0	fMandarin	qin2; yin2
U+82A9.0	fEnglish	Phragmites japonica, a kind of herb whose root is medicinal
U+82AA.0	fUTF8	芪
U+82AA.0	fCNS11643	3-2A66
U+82AA.0	fGB	0-5C4E
U+82AA.0	fR/S	 140.4
U+82AA.0	fTotalStrokes	 8; 10
U+82AA.0	fCangjie	THVP
U+82AA.0	fFourCorner	44740
U+82AA.0	fHakka	 ki2
U+82AA.0	fCantonese	kei4
U+82AA.0	fMandarin	qi2; chi2
U+82AA.0	fEnglish	celery
U+82AB.0	fUTF8	芫
U+82AB.0	fCNS11643	2-2854
U+82AB.0	fBig5	CDB4
U+82AB.0	fGB	0-5C3E
U+82AB.0	fR/S	 140.4
U+82AB.0	fTotalStrokes	 8; 10
U+82AB.0	fCangjie	TMMU
U+82AB.0	fFourCorner	44211
U+82AB.0	fHakka	  ngien2; jen2; ren2; ngian2
U+82AB.0	fCantonese	jyun4; jim4 (also)
U+82AB.0	fMandarin	yan2; yuan2
U+82AB.0	fEnglish	Daphne genkwa, a kind of poisonous plant
U+82AC.0	fUTF8	芬
U+82AC.0	fCNS11643	1-4F45
U+82AC.0	fBig5	AAE2
U+82AC.0	fGB	0-3752
U+82AC.0	fR/S	 140.4
U+82AC.0	fTotalStrokes	 8; 10
U+82AC.0	fCangjie	TCSH
U+82AC.0	fFourCorner	44227
U+82AC.0	fHakka	  fun1
U+82AC.0	fCantonese	fan1
U+82AC.0	fMandarin	fen1; fen2
U+82AC.0	fEnglish	fragrance; aroma; perfume; sweet smell
U+82AD.0	fUTF8	芭
U+82AD.0	fCNS11643	1-4F40
U+82AD.0	fBig5	AADD
U+82AD.0	fGB	0-3045
U+82AD.0	fR/S	 140.4
U+82AD.0	fTotalStrokes	 8; 10
U+82AD.0	fCangjie	TAU
U+82AD.0	fFourCorner	44717
U+82AD.0	fHakka	  ba1
U+82AD.0	fCantonese	baa1
U+82AD.0	fMandarin	ba1; pa1
U+82AD.0	fEnglish	[1] plantain; palmetto [2] a fragrant plant
U+82AE.0	fUTF8	芮
U+82AE.0	fCNS11643	2-285A
U+82AE.0	fBig5	CDBA
U+82AE.0	fGB	0-5C47
U+82AE.0	fR/S	 140.4
U+82AE.0	fTotalStrokes	 8; 10
U+82AE.0	fCangjie	TOB
U+82AE.0	fFourCorner	44227
U+82AE.0	fHakka	 nui5; ni5; jui5; niu5; rui6; jui3; nui6
U+82AE.0	fCantonese	jeoi6
U+82AE.0	fMandarin	rui4; ruo4
U+82AE.0	fEnglish	[1] small; tiny [2] name of an ancient state in Shansi [3] a Chinese family name
U+82AF.0	fUTF8	芯
U+82AF.0	fCNS11643	1-4F47
U+82AF.0	fBig5	AAE4
U+82AF.0	fGB	0-503E
U+82AF.0	fR/S	 140.4
U+82AF.0	fTotalStrokes	 8; 10
U+82AF.0	fCangjie	TP
U+82AF.0	fFourCorner	44330
U+82AF.0	fHakka	 sim1
U+82AF.0	fCantonese	sam1; seon3
U+82AF.0	fMandarin	xin1
U+82AF.0	fEnglish	pith of rush (Juncus effusus)
U+82AF.1	fUTF8	芯
U+82AF.1	fCNS11643	1-4F47
U+82AF.1	fBig5	AAE4
U+82AF.1	fGB	0-503E
U+82AF.1	fR/S	 140.4
U+82AF.1	fTotalStrokes	 8; 10
U+82AF.1	fCangjie	TP
U+82AF.1	fFourCorner	44330
U+82AF.1	fMandarin	xin4
U+82AF.1	fEnglish	central part of an object
U+82B0.0	fUTF8	芰
U+82B0.0	fCNS11643	1-4F4A
U+82B0.0	fBig5	AAE7
U+82B0.0	fGB	0-5C41
U+82B0.0	fR/S	 140.4
U+82B0.0	fTotalStrokes	 8; 10
U+82B0.0	fCangjie	TJE
U+82B0.0	fFourCorner	44407
U+82B0.0	fHakka	  ki3; zhi3
U+82B0.0	fCantonese	gei6
U+82B0.0	fMandarin	ji4
U+82B0.0	fEnglish	 water caltrop
U+82B1.0	fUTF8	花
U+82B1.0	fCNS11643	1-4F44
U+82B1.0	fBig5	AAE1
U+82B1.0	fGB	0-3B28
U+82B1.0	fR/S	 140.4
U+82B1.0	fTotalStrokes	 8; 10
U+82B1.0	fCangjie	TOP
U+82B1.0	fFourCorner	44214
U+82B1.0	fHakka	  fa1
U+82B1.0	fCantonese	faa1; waa4; waa6
U+82B1.0	fMandarin	hua1
U+82B1.0	fEnglish	[1] flower; blossom [2] a flowering plant [3] prostitute; prostitution [4] used in smallpox [5] varicolored [6] fireworks [7] spend; expend
U+82B1.1	fUTF8	花
U+82B1.1	fCNS11643	1-4F44
U+82B1.1	fBig5	AAE1
U+82B1.1	fGB	0-3B28
U+82B1.1	fR/S	 140.4
U+82B1.1	fTotalStrokes	 8; 10
U+82B1.1	fCangjie	TOP
U+82B1.1	fFourCorner	44214
U+82B1.1	fMandarin	hua5
U+82B2.0	fUTF8	芲
U+82B2.0	fCNS11643	3-2A6C
U+82B2.0	fGB	E-3A38
U+82B2.0	fR/S	 140.4
U+82B2.0	fTotalStrokes	 8; 10
U+82B2.0	fCangjie	TOP
U+82B2.0	fFourCorner	44710
U+82B2.0	fCantonese	faa1
U+82B2.0	fMandarin	hua1; lun2
U+82B3.0	fUTF8	芳
U+82B3.0	fCNS11643	1-4F3D
U+82B3.0	fBig5	AADA
U+82B3.0	fGB	0-373C
U+82B3.0	fR/S	 140.4
U+82B3.0	fTotalStrokes	 8; 10
U+82B3.0	fCangjie	TYHS
U+82B3.0	fFourCorner	44227
U+82B3.0	fHakka	  fong1
U+82B3.0	fCantonese	fong1
U+82B3.0	fMandarin	fang1
U+82B3.0	fEnglish	[1] fragrant; sweet-smelling; aromatic [2] your (used commonly in addressing a young lady) [3] virtuous; honorable; good; respectable
U+82B4.0	fUTF8	芴
U+82B4.0	fCNS11643	2-285E
U+82B4.0	fBig5	CDBE
U+82B4.0	fGB	0-5C4C
U+82B4.0	fR/S	 140.4
U+82B4.0	fTotalStrokes	 8; 10
U+82B4.0	fCangjie	TPHH
U+82B4.0	fFourCorner	44227
U+82B4.0	fHakka	 wut8; vut8
U+82B4.0	fCantonese	fat1; mat6
U+82B4.0	fMandarin	wu4; hu1
U+82B4.0	fEnglish	a edible tuber with thick leaves and woody stem
U+82B5.0	fUTF8	芵
U+82B5.0	fCNS11643	2-2858
U+82B5.0	fBig5	CDB8
U+82B5.0	fGB	5-6757
U+82B5.0	fR/S	 140.4
U+82B5.0	fTotalStrokes	 8; 10
U+82B5.0	fCangjie	TDK
U+82B5.0	fFourCorner	44530
U+82B5.0	fHakka	 giet7; get7
U+82B5.0	fCantonese	kyut3
U+82B5.0	fMandarin	jue2
U+82B5.0	fEnglish	a plant (Cassia tora) used in eye diseases
U+82B6.0	fUTF8	芶
U+82B6.0	fCNS11643	2-2865
U+82B6.0	fBig5	CDC5
U+82B6.0	fGB	3-6842
U+82B6.0	fR/S	 140.4
U+82B6.0	fTotalStrokes	 8; 10
U+82B6.0	fCangjie	TPI
U+82B6.0	fFourCorner	44727
U+82B6.0	fHakka	 giu1
U+82B6.0	fCantonese	gau2; gau1
U+82B6.0	fMandarin	gou1
U+82B7.0	fUTF8	芷
U+82B7.0	fCNS11643	1-4F4C
U+82B7.0	fBig5	AAE9
U+82B7.0	fGB	0-5C46
U+82B7.0	fR/S	 140.4
U+82B7.0	fTotalStrokes	 8; 10
U+82B7.0	fCangjie	TYLM
U+82B7.0	fFourCorner	44101
U+82B7.0	fHakka	 zhi3; zih3; zi3; tje3; chii3; chi3
U+82B7.0	fCantonese	zi2
U+82B7.0	fMandarin	zhi3
U+82B7.0	fEnglish	angelica
U+82B8.0	fUTF8	芸
U+82B8.0	fCNS11643	1-4F48
U+82B8.0	fBig5	AAE5
U+82B8.0	fGB	0-5C3F
U+82B8.0	fR/S	 140.4
U+82B8.0	fTotalStrokes	 8; 10
U+82B8.0	fCangjie	TMMI
U+82B8.0	fFourCorner	44731
U+82B8.0	fHakka	  jun2; run2; jun3
U+82B8.0	fCantonese	wan4
U+82B8.0	fMandarin	yun2; yi4; yun4
U+82B8.0	fEnglish	[1] a strong-scented herb; rue, a herb used to keep insects away [2] same as $ [3] simplified form of &#x8553;
U+82B9.0	fUTF8	芹
U+82B9.0	fCNS11643	1-4F43
U+82B9.0	fBig5	AAE0
U+82B9.0	fGB	0-475B
U+82B9.0	fR/S	 140.4
U+82B9.0	fTotalStrokes	 8; 10
U+82B9.0	fCangjie	THML
U+82B9.0	fFourCorner	44221
U+82B9.0	fHakka	  kiun2; kiun3
U+82B9.0	fCantonese	kan4; kan2 (also)
U+82B9.0	fMandarin	qin2; jin3; qi2
U+82B9.0	fEnglish	celery
U+82BA.0	fUTF8	芺
U+82BA.0	fCNS11643	2-285D
U+82BA.0	fBig5	CDBD
U+82BA.0	fGB	3-683E
U+82BA.0	fR/S	 140.4
U+82BA.0	fTotalStrokes	 8; 10
U+82BA.0	fCangjie	THK
U+82BA.0	fFourCorner	44430
U+82BA.0	fHakka	 rau1; jeu1; jau1; jau3
U+82BA.0	fCantonese	ou3; ngou3 (also); jiu2
U+82BA.0	fMandarin	ao3; ao4; yao3
U+82BA.0	fEnglish	 thistle
U+82BB.0	fUTF8	芻
U+82BB.0	fCNS11643	1-5770
U+82BB.0	fBig5	AFEC
U+82BB.0	fGB	1-5B3B
U+82BB.0	fR/S	 140.4
U+82BB.0	fTotalStrokes	 10
U+82BB.0	fCangjie	PUPU
U+82BB.0	fFourCorner	27427
U+82BB.0	fHakka	 cu1; ci1; ze1; cih1
U+82BB.0	fCantonese	co1
U+82BB.0	fMandarin	chu2; zou1
U+82BB.0	fEnglish	[1] mow; cut grass [2] hay; fodder [3] feed (livestock) [4] animals that feed on grass
U+82BC.0	fUTF8	芼
U+82BC.0	fCNS11643	2-285B
U+82BC.0	fBig5	CDBB
U+82BC.0	fGB	3-683D
U+82BC.0	fR/S	 140.4
U+82BC.0	fTotalStrokes	 8; 10
U+82BC.0	fCangjie	THQU
U+82BC.0	fFourCorner	44714
U+82BC.0	fHakka	  mau3; mo3
U+82BC.0	fCantonese	mou6; mou4
U+82BC.0	fMandarin	mao4; mao2; mao1
U+82BC.0	fEnglish	[1] choose; select [2] green vegetables; greens
U+82BD.0	fUTF8	芽
U+82BD.0	fCNS11643	1-4F41
U+82BD.0	fBig5	AADE
U+82BD.0	fGB	0-513F
U+82BD.0	fR/S	 140.4
U+82BD.0	fTotalStrokes	 8; 10
U+82BD.0	fCangjie	TMVH
U+82BD.0	fFourCorner	44241
U+82BD.0	fHakka	  nga2; nga3
U+82BD.0	fCantonese	ngaa4; aa4 (also); ngaa2 (also); aa2 (also)
U+82BD.0	fMandarin	ya2; di2; e1; wu2
U+82BD.0	fEnglish	 bud; sprout; shoot
U+82BE.0	fUTF8	芾
U+82BE.0	fCNS11643	1-4F4B
U+82BE.0	fBig5	AAE8
U+82BE.0	fGB	0-5C40
U+82BE.0	fR/S	 140.4
U+82BE.0	fTotalStrokes	 8; 10
U+82BE.0	fCangjie	TJB
U+82BE.0	fFourCorner	44227
U+82BE.0	fHakka	 fui5; fi5; fut7
U+82BE.0	fCantonese	fai3; fat1
U+82BE.0	fMandarin	fei4; fu4
U+82BE.0	fEnglish	small; little; tiny
U+82BE.1	fUTF8	芾
U+82BE.1	fCNS11643	1-4F4B
U+82BE.1	fBig5	AAE8
U+82BE.1	fGB	0-5C40
U+82BE.1	fR/S	 140.4
U+82BE.1	fTotalStrokes	 8; 10
U+82BE.1	fCangjie	TJB
U+82BE.1	fFourCorner	44227
U+82BE.1	fMandarin	fu2
U+82BE.1	fEnglish	lush; luxuriant
U+82BF.0	fUTF8	芿
U+82BF.0	fCNS11643	4-277A
U+82BF.0	fGB	5-6750
U+82BF.0	fR/S	 140.5; 140.4
U+82BF.0	fTotalStrokes	 9; 8; 11
U+82BF.0	fCangjie	TONS
U+82BF.0	fFourCorner	44220
U+82BF.0	fCantonese	jing4
U+82BF.0	fMandarin	reng4; reng2
U+82BF.0	fEnglish	 grass which has been cut and regrown wild grass
U+82C0.0	fUTF8	苀
U+82C0.0	fCNS11643	2-2853
U+82C0.0	fBig5	CDB3
U+82C0.0	fGB	3-6844
U+82C0.0	fR/S	 140.4
U+82C0.0	fTotalStrokes	 8; 10
U+82C0.0	fCangjie	TYHN
U+82C0.0	fFourCorner	44217
U+82C0.0	fCantonese	gong1; hong4
U+82C0.0	fMandarin	hang2; gang1
U+82C1.0	fUTF8	苁
U+82C1.0	fCNS11643	4-277E
U+82C1.0	fGB	0-5C4A
U+82C1.0	fR/S	 140.4
U+82C1.0	fTotalStrokes	 8; 10
U+82C1.0	fCangjie	TOO
U+82C1.0	fFourCorner	44880
U+82C1.0	fHakka	 zung3; ciung2
U+82C1.0	fCantonese	cung1
U+82C1.0	fMandarin	cong1; song3; xuan4; zong3
U+82C1.0	fEnglish	simplified form of &#x84EF;
U+82C2.0	fUTF8	苂
U+82C2.0	fCNS11643	2-2862
U+82C2.0	fBig5	CDC2
U+82C2.0	fGB	E-3A39
U+82C2.0	fR/S	 140.4
U+82C2.0	fTotalStrokes	 8; 10
U+82C2.0	fCangjie	TF
U+82C2.0	fFourCorner	44809
U+82C2.0	fCantonese	cim4; jam4
U+82C2.0	fMandarin	yin2
U+82C3.0	fUTF8	苃
U+82C3.0	fCNS11643	2-2864
U+82C3.0	fBig5	CDC4
U+82C3.0	fGB	E-3A3A
U+82C3.0	fR/S	 140.4
U+82C3.0	fTotalStrokes	 8; 10
U+82C3.0	fCangjie	TKE
U+82C3.0	fFourCorner	44447
U+82C3.0	fCantonese	jau5
U+82C3.0	fMandarin	you3
U+82C4.0	fUTF8	苄
U+82C4.0	fCNS11643	3-2A5E
U+82C4.0	fGB	0-5C50
U+82C4.0	fR/S	  140.4
U+82C4.0	fTotalStrokes	 8; 10
U+82C4.0	fCangjie	TYY
U+82C4.0	fFourCorner	44230
U+82C4.0	fCantonese	bin6
U+82C4.0	fMandarin	bian4
U+82C4.0	fEnglish	benzyl
U+82C5.0	fUTF8	苅
U+82C5.0	fCNS11643	F-2863
U+82C5.0	fGB	5-6752
U+82C5.0	fR/S	 140.4
U+82C5.0	fTotalStrokes	 8; 10
U+82C5.0	fCangjie	TKLN
U+82C5.0	fFourCorner	4442
U+82C5.0	fCantonese	ngaai6; aai6 (also)
U+82C5.0	fMandarin	yi4
U+82C5.0	fEnglish	 cut off; reap; mow sickle
U+82C6.0	fUTF8	苆
U+82C6.0	fGB	E-3A3B
U+82C6.0	fR/S	 140.4
U+82C6.0	fTotalStrokes	 8; 10
U+82C6.0	fCangjie	TPSH
U+82C6.0	fCantonese	cit3
U+82C7.0	fUTF8	苇
U+82C7.0	fGB	0-4E2D
U+82C7.0	fR/S	 140.4
U+82C7.0	fTotalStrokes	 8; 10
U+82C7.0	fCangjie	TQS
U+82C7.0	fHakka	 vui3
U+82C7.0	fCantonese	wai5
U+82C7.0	fMandarin	wei3; wei2; wei4
U+82C7.0	fEnglish	simplified form of &#x8466;
U+82C8.0	fUTF8	苈
U+82C8.0	fGB	0-5C42
U+82C8.0	fR/S	 140.4
U+82C8.0	fTotalStrokes	 8; 10
U+82C8.0	fCangjie	XTMKS
U+82C8.0	fHakka	 let8
U+82C8.0	fCantonese	lik6
U+82C8.0	fMandarin	li4
U+82C8.0	fEnglish	simplified form of &#x85F6;
U+82C9.0	fUTF8	苉
U+82C9.0	fCNS11643	3-2A68
U+82C9.0	fGB	3-683B
U+82C9.0	fR/S	  140.4
U+82C9.0	fTotalStrokes	 8; 10
U+82C9.0	fCangjie	TSC
U+82C9.0	fFourCorner	44710
U+82C9.0	fCantonese	pat1; pot1
U+82C9.0	fMandarin	pi3
U+82CA.0	fUTF8	苊
U+82CA.0	fCNS11643	4-277B
U+82CA.0	fGB	0-5C43
U+82CA.0	fR/S	  140.4
U+82CA.0	fTotalStrokes	 8; 10
U+82CA.0	fCangjie	TMSU
U+82CA.0	fFourCorner	44210
U+82CA.0	fCantonese	ak1; ngak1 (also)
U+82CA.0	fMandarin	e4
U+82CA.0	fEnglish	an organic compound
U+82CB.0	fUTF8	苋
U+82CB.0	fGB	0-5C48
U+82CB.0	fR/S	 140.4
U+82CB.0	fTotalStrokes	 8; 10
U+82CB.0	fCangjie	TBHU
U+82CB.0	fHakka	 han5
U+82CB.0	fCantonese	jin6
U+82CB.0	fMandarin	xian4; huan2; huan3
U+82CB.0	fEnglish	simplified form of &#x83A7;
U+82CC.0	fUTF8	苌
U+82CC.0	fGB	0-5C49
U+82CC.0	fR/S	 140.4
U+82CC.0	fTotalStrokes	 8; 10
U+82CC.0	fCangjie	TPO
U+82CC.0	fHakka	 chong2
U+82CC.0	fCantonese	coeng4
U+82CC.0	fMandarin	chang2
U+82CC.0	fEnglish	simplified form of &#x8407;
U+82CD.0	fUTF8	苍
U+82CD.0	fGB	0-3254
U+82CD.0	fR/S	 140.4
U+82CD.0	fTotalStrokes	 8; 10
U+82CD.0	fCangjie	TOSU
U+82CD.0	fHakka	 cong1
U+82CD.0	fCantonese	cong1; cong2
U+82CD.0	fMandarin	cang1; cang3
U+82CD.0	fEnglish	simplified form of &#x84BC;
U+82CE.0	fUTF8	苎
U+82CE.0	fGB	0-5C51
U+82CE.0	fR/S	 140.4
U+82CE.0	fTotalStrokes	 8; 10
U+82CE.0	fCangjie	TJM
U+82CE.0	fHakka	 {chu1}; chu3
U+82CE.0	fCantonese	cyu5
U+82CE.0	fMandarin	zhu4; meng2; ning2; chu2
U+82CE.0	fEnglish	simplified form of &#x82E7;
U+82CF.0	fUTF8	苏
U+82CF.0	fGB	0-4B55
U+82CF.0	fR/S	 140.4
U+82CF.0	fTotalStrokes	 8; 10
U+82CF.0	fCangjie	TKSC
U+82CF.0	fHakka	 sii1
U+82CF.0	fCantonese	sou1
U+82CF.0	fMandarin	su1; shu1; su4
U+82CF.0	fEnglish	simplified form of &#x8607;
U+82D0.0	fUTF8	苐
U+82D0.0	fCNS11643	3-2E76
U+82D0.0	fGB	E-3A3C
U+82D0.0	fR/S	 140.5
U+82D0.0	fTotalStrokes	 9; 11
U+82D0.0	fCangjie	TNLH
U+82D0.0	fFourCorner	44220
U+82D0.0	fHakka	 tan3;  ti5; ti6
U+82D0.0	fCantonese	dai6; tai4; ji4
U+82D0.0	fMandarin	yi2; ti2; di4
U+82D0.0	fEnglish	 sequence; number grade; degree
U+82D1.0	fUTF8	苑
U+82D1.0	fCNS11643	1-5345
U+82D1.0	fBig5	AD62
U+82D1.0	fGB	0-5437
U+82D1.0	fR/S	 140.5
U+82D1.0	fTotalStrokes	 9; 11
U+82D1.0	fCangjie	TNIU
U+82D1.0	fFourCorner	44212
U+82D1.0	fHakka	 jen5; jen3; ren5; won3; ren3; jen2; jut7; jan3
U+82D1.0	fCantonese	jyun2; jyun4; wat1
U+82D1.0	fMandarin	yuan4; yu4; yuan3; wan3; yu2; yun3; yung4; yun1; yuan1
U+82D1.0	fEnglish	[1] park; garden [2] gathering place ???
U+82D1.1	fUTF8	苑
U+82D1.1	fCNS11643	1-5345
U+82D1.1	fBig5	AD62
U+82D1.1	fGB	0-5437
U+82D1.1	fR/S	 140.5
U+82D1.1	fTotalStrokes	 9; 11
U+82D1.1	fCangjie	TNIU
U+82D1.1	fFourCorner	44212
U+82D1.1	fMandarin	yuan2
U+82D1.1	fEnglish	a Chinese family name ???
U+82D2.0	fUTF8	苒
U+82D2.0	fCNS11643	1-533F
U+82D2.0	fBig5	AD5C
U+82D2.0	fGB	0-5C5B
U+82D2.0	fR/S	 140.5
U+82D2.0	fTotalStrokes	 9; 11
U+82D2.0	fCangjie	TGB
U+82D2.0	fFourCorner	44557
U+82D2.0	fHakka	 ngiam3; ngiam1; jam3; ram3; jam5
U+82D2.0	fCantonese	jim5
U+82D2.0	fMandarin	ran3
U+82D2.0	fEnglish	lush; delicate (flowers and grass)
U+82D3.0	fUTF8	苓
U+82D3.0	fCNS11643	1-5347
U+82D3.0	fBig5	AD64
U+82D3.0	fGB	0-5C5F
U+82D3.0	fR/S	 140.5
U+82D3.0	fTotalStrokes	 9; 11
U+82D3.0	fCangjie	TOII
U+82D3.0	fFourCorner	44307
U+82D3.0	fHakka	 lin2
U+82D3.0	fCantonese	ling4
U+82D3.0	fMandarin	ling2; lian2
U+82D3.0	fEnglish	[1] a variety of fungus [2] tuckahoe [3] licorice; liquorice
U+82D4.0	fUTF8	苔
U+82D4.0	fCNS11643	1-5344
U+82D4.0	fBig5	AD61
U+82D4.0	fGB	0-4C26
U+82D4.0	fR/S	 140.5
U+82D4.0	fTotalStrokes	 9; 11
U+82D4.0	fCangjie	TIR
U+82D4.0	fFourCorner	44603
U+82D4.0	fHakka	  toi2; toi3
U+82D4.0	fCantonese	toi4; toi1
U+82D4.0	fMandarin	tai2
U+82D4.0	fEnglish	 moss; lichen
U+82D4.1	fUTF8	苔
U+82D4.1	fCNS11643	1-5344
U+82D4.1	fBig5	AD61
U+82D4.1	fGB	0-4C26
U+82D4.1	fR/S	 140.5
U+82D4.1	fTotalStrokes	 9; 11
U+82D4.1	fCangjie	TIR
U+82D4.1	fFourCorner	44603
U+82D4.1	fMandarin	tai1
U+82D4.1	fEnglish	used in combination
U+82D5.0	fUTF8	苕
U+82D5.0	fCNS11643	2-2D34
U+82D5.0	fBig5	D071
U+82D5.0	fGB	0-5C66
U+82D5.0	fR/S	 140.5
U+82D5.0	fTotalStrokes	 9; 11
U+82D5.0	fCangjie	TSHR
U+82D5.0	fFourCorner	44602
U+82D5.0	fHakka	  tiau2
U+82D5.0	fCantonese	tiu4; siu4
U+82D5.0	fMandarin	tiao2; shao2
U+82D5.0	fEnglish	a plant much used for making brooms
U+82D6.0	fUTF8	苖
U+82D6.0	fCNS11643	2-2D37
U+82D6.0	fBig5	D074
U+82D6.0	fGB	3-684F
U+82D6.0	fR/S	 140.5
U+82D6.0	fTotalStrokes	 9; 11
U+82D6.0	fCangjie	TLW
U+82D6.0	fFourCorner	44600
U+82D6.0	fCantonese	miu4; dik6; cuk1
U+82D6.0	fMandarin	di3; di2; cun4; du2
U+82D7.0	fUTF8	苗
U+82D7.0	fCNS11643	1-5340
U+82D7.0	fBig5	AD5D
U+82D7.0	fGB	0-4367
U+82D7.0	fR/S	 140.5
U+82D7.0	fTotalStrokes	 9; 11
U+82D7.0	fCangjie	TW
U+82D7.0	fFourCorner	44600
U+82D7.0	fHakka	  miau2; meu2; miau3; meu5
U+82D7.0	fCantonese	miu4
U+82D7.0	fMandarin	miao2; qu3
U+82D7.0	fEnglish	[1] sprout [2] descendant; posterity [3] peevish; disobedient (children) [4] Miao nationality (Southwestern China) [5] summer hunting [6] beginning; omen [7] a Chinese family name
U+82D8.0	fUTF8	苘
U+82D8.0	fCNS11643	3-2E6A
U+82D8.0	fGB	0-5C5C
U+82D8.0	fR/S	  140.5
U+82D8.0	fTotalStrokes	 9; 11
U+82D8.0	fCangjie	TBR
U+82D8.0	fFourCorner	44220
U+82D8.0	fHakka	 kwen3
U+82D8.0	fCantonese	gwing2 (also); king2
U+82D8.0	fMandarin	qiong3; qing3
U+82D8.0	fEnglish	indian mallow
U+82D9.0	fUTF8	苙
U+82D9.0	fCNS11643	2-2D2E
U+82D9.0	fBig5	D06B
U+82D9.0	fGB	3-6857
U+82D9.0	fR/S	 140.5
U+82D9.0	fTotalStrokes	 9; 11
U+82D9.0	fCangjie	TYT
U+82D9.0	fFourCorner	44108
U+82D9.0	fHakka	  lip8;  lap7; lip7
U+82D9.0	fCantonese	lap1; lap6
U+82D9.0	fMandarin	li4; ji2
U+82D9.0	fEnglish	[1] pigsty; pigpen [2] a kind of medicinal herb
U+82DA.0	fUTF8	苚
U+82DA.0	fCNS11643	4-2B3A
U+82DA.0	fGB	E-3A3D
U+82DA.0	fR/S	  140.5
U+82DA.0	fTotalStrokes	 9; 11
U+82DA.0	fCangjie	TBQ
U+82DA.0	fFourCorner	44220
U+82DA.0	fCantonese	jung6
U+82DA.0	fMandarin	yong4
U+82DB.0	fUTF8	苛
U+82DB.0	fCNS11643	1-5339
U+82DB.0	fBig5	AD56
U+82DB.0	fGB	0-3F41
U+82DB.0	fR/S	 140.5
U+82DB.0	fTotalStrokes	 9; 11
U+82DB.0	fCangjie	TMNR
U+82DB.0	fFourCorner	44621
U+82DB.0	fHakka	  ko1; ho2
U+82DB.0	fCantonese	ho1; ho4
U+82DB.0	fMandarin	ke1; he2; he1; he4
U+82DB.0	fEnglish	harsh; rigorous; severe; caustic
U+82DC.0	fUTF8	苜
U+82DC.0	fCNS11643	1-5343
U+82DC.0	fBig5	AD60
U+82DC.0	fGB	0-5C59
U+82DC.0	fR/S	 140.5
U+82DC.0	fTotalStrokes	 9; 11
U+82DC.0	fCangjie	TBU
U+82DC.0	fFourCorner	44600
U+82DC.0	fHakka	 muk7
U+82DC.0	fCantonese	muk6
U+82DC.0	fMandarin	mu4
U+82DC.0	fEnglish	clover
U+82DD.0	fUTF8	苝
U+82DD.0	fCNS11643	3-2E6B
U+82DD.0	fGB	3-684D
U+82DD.0	fR/S	  140.5
U+82DD.0	fTotalStrokes	 9; 11
U+82DD.0	fCangjie	TLMP
U+82DD.0	fFourCorner	44110
U+82DD.0	fCantonese	bui6
U+82DD.0	fMandarin	pei4
U+82DE.0	fUTF8	苞
U+82DE.0	fCNS11643	1-5346
U+82DE.0	fBig5	AD63
U+82DE.0	fGB	0-307A
U+82DE.0	fR/S	 140.5
U+82DE.0	fTotalStrokes	 9; 11
U+82DE.0	fCangjie	TPRU
U+82DE.0	fFourCorner	44712
U+82DE.0	fHakka	  bau1; pau1
U+82DE.0	fCantonese	baau1
U+82DE.0	fMandarin	bao1; pao2; piao3; biao1
U+82DE.0	fEnglish	[1] a variety of rush [2] bract [3] wrap [4] profuse; thick [5] seeds with the germ already to burst; seeds bursting up
U+82DF.0	fUTF8	苟
U+82DF.0	fCNS11643	1-5348
U+82DF.0	fBig5	AD65
U+82DF.0	fGB	0-3936
U+82DF.0	fR/S	 140.5
U+82DF.0	fTotalStrokes	 9; 11
U+82DF.0	fCangjie	TPR
U+82DF.0	fFourCorner	44627
U+82DF.0	fHakka	  geu3; kieu3; giu3
U+82DF.0	fCantonese	gau2; gik1
U+82DF.0	fMandarin	gou3; ju3; gou1
U+82DF.0	fEnglish	[1] against principle; illicit; improper [2] careless; frivolous [3] [grammatical particle] if; but; if only [4] a Chinese family name [5] a grass name
U+82E0.0	fUTF8	苠
U+82E0.0	fCNS11643	2-2D43
U+82E0.0	fBig5	D0A2
U+82E0.0	fGB	0-5C65
U+82E0.0	fR/S	 140.5
U+82E0.0	fTotalStrokes	 9; 11
U+82E0.0	fCangjie	TRVP
U+82E0.0	fFourCorner	44747
U+82E0.0	fHakka	 min2
U+82E0.0	fCantonese	man4
U+82E0.0	fMandarin	min2
U+82E0.0	fEnglish	skin of the bamboo, multitude
U+82E1.0	fUTF8	苡
U+82E1.0	fCNS11643	2-2D3A
U+82E1.0	fBig5	D077
U+82E1.0	fGB	0-5C53
U+82E1.0	fR/S	 140.5
U+82E1.0	fTotalStrokes	 8; 11
U+82E1.0	fCangjie	TVIO
U+82E1.0	fFourCorner	44180
U+82E1.0	fHakka	 ji3; ji1; ri3; i3
U+82E1.0	fCantonese	ji5
U+82E1.0	fMandarin	yi3; si4; wo1
U+82E1.0	fEnglish	Coix lacryma-jobi, a kind of plant whose grains are used as food or medicine
U+82E2.0	fUTF8	苢
U+82E2.0	fCNS11643	3-2E78
U+82E2.0	fGB	5-675D
U+82E2.0	fR/S	 140.5
U+82E2.0	fTotalStrokes	 9; 11
U+82E2.0	fCangjie	TRLR
U+82E2.0	fFourCorner	44770
U+82E2.0	fHakka	 ji3; ri3
U+82E2.0	fCantonese	ji5
U+82E2.0	fMandarin	yi3
U+82E2.0	fEnglish	Coix agrestis
U+82E3.0	fUTF8	苣
U+82E3.0	fCNS11643	1-5338
U+82E3.0	fBig5	AD55
U+82E3.0	fGB	0-5C44
U+82E3.0	fR/S	 140.5
U+82E3.0	fTotalStrokes	 9; 11
U+82E3.0	fCangjie	TSS
U+82E3.0	fFourCorner	44717
U+82E3.0	fHakka	 ki3;  gi3
U+82E3.0	fCantonese	geoi6
U+82E3.0	fMandarin	ju4; qu3
U+82E3.0	fEnglish	kind of lettuce
U+82E4.0	fUTF8	苤
U+82E4.0	fCNS11643	2-2D42
U+82E4.0	fBig5	D0A1
U+82E4.0	fGB	0-5C56
U+82E4.0	fR/S	 140.5
U+82E4.0	fTotalStrokes	 9; 11
U+82E4.0	fCangjie	TMFM
U+82E4.0	fFourCorner	44109
U+82E4.0	fHakka	  pi1
U+82E4.0	fCantonese	pei2
U+82E4.0	fMandarin	pi1; pie3; pi3
U+82E4.0	fEnglish	[1] a kind of plant [2] luxuriant flora or vegetation
U+82E5.0	fUTF8	若
U+82E5.0	fCNS11643	1-533C
U+82E5.0	fBig5	AD59
U+82E5.0	fGB	0-4874
U+82E5.0	fR/S	 140.5
U+82E5.0	fTotalStrokes	 9; 11
U+82E5.0	fCangjie	TKR
U+82E5.0	fFourCorner	44604
U+82E5.0	fHakka	  jok8; ngia2; na1; rok8; ngiok8; na6; ngia1; na5; na2
U+82E5.0	fCantonese	joek6; je5
U+82E5.0	fMandarin	ruo4; re2; yao4; re4
U+82E5.0	fEnglish	[1] if; suppose; supposing; assuming; provided that [2] you [3] similar to; like
U+82E5.1	fUTF8	若
U+82E5.1	fCNS11643	1-533C
U+82E5.1	fBig5	AD59
U+82E5.1	fGB	0-4874
U+82E5.1	fR/S	 140.5
U+82E5.1	fTotalStrokes	 9; 11
U+82E5.1	fCangjie	TKR
U+82E5.1	fFourCorner	44604
U+82E5.1	fMandarin	re3
U+82E5.1	fEnglish	used in combination
U+82E6.0	fUTF8	苦
U+82E6.0	fCNS11643	1-533A
U+82E6.0	fBig5	AD57
U+82E6.0	fGB	0-3F60
U+82E6.0	fR/S	 140.5
U+82E6.0	fTotalStrokes	 9; 11
U+82E6.0	fCangjie	TJR
U+82E6.0	fFourCorner	44604
U+82E6.0	fHakka	  ku3;  fu3
U+82E6.0	fCantonese	fu2; gu2; gwu2 (also)
U+82E6.0	fMandarin	ku3; gu3; hu4; ku4
U+82E6.0	fEnglish	[1] bitter [2] painful; hardship; suffering; hard; difficult; laborious; miserable [3] strenuous; earnest; diligent [4] abhor [5] feel miserable about
U+82E7.0	fUTF8	苧
U+82E7.0	fCNS11643	1-5335
U+82E7.0	fBig5	AD52
U+82E7.0	fGB	1-5C51
U+82E7.0	fR/S	 140.5
U+82E7.0	fTotalStrokes	 9; 11
U+82E7.0	fCangjie	TJMN
U+82E7.0	fFourCorner	44201
U+82E7.0	fHakka	 chu3; cu1; chu1; cu3; {chu1}
U+82E7.0	fCantonese	cyu5; ning4; ling4 (also)
U+82E7.0	fMandarin	zhu4; ning2; chu2
U+82E7.0	fEnglish	China grass; ramie
U+82E8.0	fUTF8	苨
U+82E8.0	fCNS11643	2-2D32
U+82E8.0	fBig5	D06F
U+82E8.0	fGB	3-685B
U+82E8.0	fR/S	 140.5
U+82E8.0	fTotalStrokes	 9; 11
U+82E8.0	fCangjie	TSP
U+82E8.0	fFourCorner	44211
U+82E8.0	fHakka	 ni3
U+82E8.0	fCantonese	nei5; lei5 (also); nai5
U+82E8.0	fMandarin	ni3
U+82E8.0	fEnglish	 harebell
U+82E9.0	fUTF8	苩
U+82E9.0	fCNS11643	3-2E72
U+82E9.0	fGB	3-6854
U+82E9.0	fR/S	  140.5
U+82E9.0	fTotalStrokes	 9; 11
U+82E9.0	fCangjie	THA
U+82E9.0	fFourCorner	44600
U+82E9.0	fCantonese	baak6
U+82E9.0	fMandarin	bo2
U+82EA.0	fUTF8	苪
U+82EA.0	fCNS11643	2-2D41
U+82EA.0	fBig5	D07E
U+82EA.0	fGB	3-684C
U+82EA.0	fR/S	 140.5
U+82EA.0	fTotalStrokes	 9; 11
U+82EA.0	fCangjie	TMOB
U+82EA.0	fFourCorner	44227
U+82EA.0	fCantonese	bing2
U+82EA.0	fMandarin	bing3
U+82EA.0	fEnglish	 bright shining; splendid
U+82EB.0	fUTF8	苫
U+82EB.0	fCNS11643	2-2D36
U+82EB.0	fBig5	D073
U+82EB.0	fGB	0-493B
U+82EB.0	fR/S	 140.5
U+82EB.0	fTotalStrokes	 9; 11
U+82EB.0	fCangjie	TYR
U+82EB.0	fFourCorner	44601
U+82EB.0	fHakka	 diam1; zham1; zam1
U+82EB.0	fCantonese	sim1; sim3 (also)
U+82EB.0	fMandarin	shan1; shan4; tian1; chan1
U+82EB.0	fEnglish	 thatch
U+82EC.0	fUTF8	苬
U+82EC.0	fCNS11643	2-2D39
U+82EC.0	fBig5	D076
U+82EC.0	fGB	5-675E
U+82EC.0	fR/S	 140.5
U+82EC.0	fTotalStrokes	 9; 11
U+82EC.0	fCangjie	TWO
U+82EC.0	fFourCorner	44600
U+82EC.0	fHakka	 siu2
U+82EC.0	fCantonese	cau4
U+82EC.0	fMandarin	qiu2; you2
U+82ED.0	fUTF8	苭
U+82ED.0	fCNS11643	2-2D46
U+82ED.0	fBig5	D0A5
U+82ED.0	fGB	5-6768
U+82ED.0	fR/S	 140.5
U+82ED.0	fTotalStrokes	 9; 11
U+82ED.0	fCangjie	TVIS
U+82ED.0	fFourCorner	44727
U+82ED.0	fCantonese	jiu2
U+82ED.0	fMandarin	yao3
U+82EE.0	fUTF8	苮
U+82EE.0	fCNS11643	3-2E74
U+82EE.0	fGB	E-3A3E
U+82EE.0	fR/S	  140.5
U+82EE.0	fTotalStrokes	 9; 11
U+82EE.0	fCangjie	TOU
U+82EE.0	fFourCorner	44270
U+82EE.0	fCantonese	sin1
U+82EE.0	fMandarin	xian1
U+82EF.0	fUTF8	苯
U+82EF.0	fCNS11643	1-5349
U+82EF.0	fBig5	AD66
U+82EF.0	fGB	0-313D
U+82EF.0	fR/S	 140.5
U+82EF.0	fTotalStrokes	 9; 11
U+82EF.0	fCangjie	TDM
U+82EF.0	fFourCorner	44234
U+82EF.0	fHakka	 bun3
U+82EF.0	fCantonese	bun2
U+82EF.0	fMandarin	ben3
U+82EF.0	fEnglish	benzene
U+82F0.0	fUTF8	苰
U+82F0.0	fCNS11643	2-2D40
U+82F0.0	fBig5	D07D
U+82F0.0	fGB	3-685D
U+82F0.0	fR/S	 140.5
U+82F0.0	fTotalStrokes	 9; 11
U+82F0.0	fCangjie	TNI
U+82F0.0	fFourCorner	44232
U+82F0.0	fCantonese	wang4
U+82F0.0	fMandarin	hong2
U+82F1.0	fUTF8	英
U+82F1.0	fCNS11643	1-5341
U+82F1.0	fBig5	AD5E
U+82F1.0	fGB	0-5322
U+82F1.0	fR/S	 140.5
U+82F1.0	fTotalStrokes	 9; 11
U+82F1.0	fCangjie	TLBK
U+82F1.0	fFourCorner	44530
U+82F1.0	fHakka	  jin1; rin1
U+82F1.0	fCantonese	jing1
U+82F1.0	fMandarin	ying1; yang1; ying4
U+82F1.0	fEnglish	[1] petal; flower; leaf [2] surpassing; outstanding; prominent; distinguished [3] English; British [4] fine; handsome [5] hero; outstanding person [6] a Chinese family name
U+82F2.0	fUTF8	苲
U+82F2.0	fCNS11643	2-2D3B
U+82F2.0	fBig5	D078
U+82F2.0	fGB	3-6853
U+82F2.0	fR/S	 140.5
U+82F2.0	fTotalStrokes	 9; 11
U+82F2.0	fCangjie	TOS
U+82F2.0	fFourCorner	44811
U+82F2.0	fHakka	 za5; cok7
U+82F2.0	fCantonese	zaa3; zaa2
U+82F2.0	fMandarin	zha3; zha4; zuo2; ze2
U+82F2.0	fEnglish	[1] a water plant, called ground hemp [2] name of a district in Chehkiang
U+82F3.0	fUTF8	苳
U+82F3.0	fCNS11643	2-2D45
U+82F3.0	fBig5	D0A4
U+82F3.0	fGB	5-6763
U+82F3.0	fR/S	 140.5
U+82F3.0	fTotalStrokes	 9; 11
U+82F3.0	fCangjie	THEY
U+82F3.0	fFourCorner	44303
U+82F3.0	fHakka	 dung1
U+82F3.0	fCantonese	dung2; dung1
U+82F3.0	fMandarin	dong1
U+82F3.0	fEnglish	the name of a plant
U+82F4.0	fUTF8	苴
U+82F4.0	fCNS11643	2-2D38
U+82F4.0	fBig5	D075
U+82F4.0	fGB	0-5C5A
U+82F4.0	fR/S	 140.5
U+82F4.0	fTotalStrokes	 9; 11
U+82F4.0	fCangjie	TBM
U+82F4.0	fFourCorner	44107
U+82F4.0	fHakka	 zih1; zii1
U+82F4.0	fCantonese	zeoi1; caa4; zaa2; caa1; zaa3; zeoi2; ceoi1
U+82F4.0	fMandarin	ju1; bao1; zu1; qie3; zha3; qu1; zhu1; jie1; zu2; xie2
U+82F4.0	fEnglish	[1] pack [2] female hemp plant
U+82F4.1	fUTF8	苴
U+82F4.1	fCNS11643	2-2D38
U+82F4.1	fBig5	D075
U+82F4.1	fGB	0-5C5A
U+82F4.1	fR/S	 140.5
U+82F4.1	fTotalStrokes	 9; 11
U+82F4.1	fCangjie	TBM
U+82F4.1	fFourCorner	44107
U+82F4.1	fMandarin	cha2
U+82F4.1	fEnglish	[1] grass floating in the water [2] withered grass
U+82F5.0	fUTF8	苵
U+82F5.0	fCNS11643	2-2D3C
U+82F5.0	fBig5	D079
U+82F5.0	fGB	3-6852
U+82F5.0	fR/S	 140.5
U+82F5.0	fTotalStrokes	 9; 11
U+82F5.0	fCangjie	THQO
U+82F5.0	fFourCorner	44530
U+82F5.0	fCantonese	dit6
U+82F5.0	fMandarin	die2
U+82F6.0	fUTF8	苶
U+82F6.0	fCNS11643	2-2D3F
U+82F6.0	fBig5	D07C
U+82F6.0	fGB	3-6855
U+82F6.0	fR/S	 140.5
U+82F6.0	fTotalStrokes	 9; 11
U+82F6.0	fCangjie	TOF
U+82F6.0	fFourCorner	44908
U+82F6.0	fHakka	 ngiap7; let7
U+82F6.0	fCantonese	nip6; lip6 (also)
U+82F6.0	fMandarin	nie2
U+82F6.0	fEnglish	weary; tired; exhausting
U+82F7.0	fUTF8	苷
U+82F7.0	fCNS11643	3-2E69
U+82F7.0	fGB	0-5C55
U+82F7.0	fR/S	 140.5
U+82F7.0	fTotalStrokes	 9; 11
U+82F7.0	fCangjie	TTM
U+82F7.0	fFourCorner	44770
U+82F7.0	fHakka	 gam1
U+82F7.0	fCantonese	gam1
U+82F7.0	fMandarin	gan1
U+82F7.0	fEnglish	licorice
U+82F8.0	fUTF8	苸
U+82F8.0	fCNS11643	3-2E6E
U+82F8.0	fGB	5-6762
U+82F8.0	fR/S	  140.5
U+82F8.0	fTotalStrokes	 9; 11
U+82F8.0	fCangjie	THFD
U+82F8.0	fFourCorner	44400
U+82F8.0	fCantonese	fu1
U+82F8.0	fMandarin	hu1
U+82F9.0	fUTF8	苹
U+82F9.0	fCNS11643	2-2D30
U+82F9.0	fBig5	D06D
U+82F9.0	fGB	0-463B
U+82F9.0	fR/S	 140.5
U+82F9.0	fTotalStrokes	 9; 11
U+82F9.0	fCangjie	TMFJ
U+82F9.0	fFourCorner	44409
U+82F9.0	fHakka	  pin2
U+82F9.0	fCantonese	ping4
U+82F9.0	fMandarin	ping2; pin2; peng1
U+82F9.0	fEnglish	[1] a kind of herb [2] simplified form of &#x860B;
U+82FA.0	fUTF8	苺
U+82FA.0	fCNS11643	2-2D44
U+82FA.0	fBig5	D0A3
U+82FA.0	fGB	5-6767
U+82FA.0	fR/S	 140.5
U+82FA.0	fTotalStrokes	 9; 11
U+82FA.0	fCangjie	TWYI
U+82FA.0	fFourCorner	44507
U+82FA.0	fHakka	 moi2; mui3; mui2; mi2; moi1; mi3
U+82FA.0	fCantonese	mui4
U+82FA.0	fMandarin	mei2; mao3; mei4; mei3; mou4
U+82FA.0	fEnglish	 berries
U+82FB.0	fUTF8	苻
U+82FB.0	fCNS11643	2-2D3E
U+82FB.0	fBig5	D07B
U+82FB.0	fGB	0-5C5E
U+82FB.0	fR/S	 140.5
U+82FB.0	fTotalStrokes	 9; 11
U+82FB.0	fCangjie	TODI
U+82FB.0	fFourCorner	44240
U+82FB.0	fHakka	  pu2; fu2; fu1
U+82FB.0	fCantonese	fu4; pou4
U+82FB.0	fMandarin	fu2; pu2
U+82FB.0	fEnglish	a kind of herb
U+82FC.0	fUTF8	苼
U+82FC.0	fCNS11643	3-2E6F
U+82FC.0	fGB	3-6851
U+82FC.0	fR/S	  140.5
U+82FC.0	fTotalStrokes	 9; 11
U+82FC.0	fCangjie	THQM
U+82FC.0	fFourCorner	44100
U+82FC.0	fCantonese	sang1
U+82FC.0	fMandarin	sheng1
U+82FD.0	fUTF8	苽
U+82FD.0	fCNS11643	3-2E7A
U+82FD.0	fGB	5-6761
U+82FD.0	fR/S	  140.5; 140.6
U+82FD.0	fTotalStrokes	 9; 11
U+82FD.0	fCangjie	THVO
U+82FD.0	fFourCorner	44230
U+82FD.0	fHakka	  gu1
U+82FD.0	fCantonese	gu1; gwu1 (also)
U+82FD.0	fMandarin	gu1
U+82FD.0	fEnglish	Zizania latifolia
U+82FE.0	fUTF8	苾
U+82FE.0	fCNS11643	2-2D2F
U+82FE.0	fBig5	D06C
U+82FE.0	fGB	3-685A
U+82FE.0	fR/S	 140.5
U+82FE.0	fTotalStrokes	 9; 11
U+82FE.0	fCangjie	TPH
U+82FE.0	fFourCorner	44330
U+82FE.0	fHakka	 bit7; bi5
U+82FE.0	fCantonese	bat6; bit6 (also); bit1
U+82FE.0	fMandarin	bi4; bie2
U+82FE.0	fEnglish	fragrant; fragrance; aroma
U+82FF.0	fUTF8	苿
U+82FF.0	fCNS11643	6-343C
U+82FF.0	fGB	E-3A3F
U+82FF.0	fR/S	 140.5
U+82FF.0	fTotalStrokes	 9; 11
U+82FF.0	fCangjie	TJD
U+82FF.0	fFourCorner	44900
U+82FF.0	fCantonese	mei6
U+82FF.0	fMandarin	wei4
U+8300.0	fUTF8	茀
U+8300.0	fCNS11643	2-2D33
U+8300.0	fBig5	D070
U+8300.0	fGB	3-685C
U+8300.0	fR/S	 140.5
U+8300.0	fTotalStrokes	 9; 11
U+8300.0	fCangjie	TLLN
U+8300.0	fFourCorner	44527
U+8300.0	fHakka	  fut7
U+8300.0	fCantonese	fat1; but6; bat6
U+8300.0	fMandarin	fu2; bo2; bei4; bi4; fei4
U+8300.0	fEnglish	[1] overgrown with grass; a rank growth of weeds obstructing the way; obstacle; hindrance [2] weedy
U+8301.0	fUTF8	茁
U+8301.0	fCNS11643	1-5342
U+8301.0	fBig5	AD5F
U+8301.0	fGB	0-5742
U+8301.0	fR/S	 140.5
U+8301.0	fTotalStrokes	 9; 11
U+8301.0	fCangjie	TUU
U+8301.0	fFourCorner	44777
U+8301.0	fHakka	 zot7; chot7; zhot7
U+8301.0	fCantonese	zyut3; cyut3 (also)
U+8301.0	fMandarin	zhuo2; ju2; zhu2; zha2
U+8301.0	fEnglish	[1] sprouting; growing [2] strong; vigorous; sturdy
U+8302.0	fUTF8	茂
U+8302.0	fCNS11643	1-533D
U+8302.0	fBig5	AD5A
U+8302.0	fGB	0-432F
U+8302.0	fR/S	 140.5
U+8302.0	fTotalStrokes	 9; 11
U+8302.0	fCangjie	TIH
U+8302.0	fFourCorner	44253
U+8302.0	fHakka	 meu5; meu6; miau5; miu3; meu3; miau6; mung3
U+8302.0	fCantonese	mau6
U+8302.0	fMandarin	mao4; mou4
U+8302.0	fEnglish	[1] exuberant; thick; lush; dense; luxuriant; flourishing; healthy; vigorous; strong [2] fine; fair; excellent
U+8303.0	fUTF8	范
U+8303.0	fCNS11643	1-5336
U+8303.0	fBig5	AD53
U+8303.0	fGB	0-3736
U+8303.0	fR/S	 140.5
U+8303.0	fTotalStrokes	 9; 11
U+8303.0	fCangjie	TESU
U+8303.0	fFourCorner	44112
U+8303.0	fHakka	  fam5; fam6; fam3
U+8303.0	fCantonese	faan6
U+8303.0	fMandarin	fan4
U+8303.0	fEnglish	[1] a bee-like insect [2] a Chinese family name [3] simplified form of &#x7BC4;
U+8304.0	fUTF8	茄
U+8304.0	fCNS11643	1-533B
U+8304.0	fBig5	AD58
U+8304.0	fGB	0-4751
U+8304.0	fR/S	 140.5
U+8304.0	fTotalStrokes	 9; 11
U+8304.0	fCangjie	TKSR
U+8304.0	fFourCorner	44460
U+8304.0	fHakka	  kio2;  ga1; kia3
U+8304.0	fCantonese	ke4 (also); gaa1; ke1; ke2
U+8304.0	fMandarin	jia1; qie2
U+8304.0	fEnglish	 eggplant; aubergine
U+8305.0	fUTF8	茅
U+8305.0	fCNS11643	1-5337
U+8305.0	fBig5	AD54
U+8305.0	fGB	0-4329
U+8305.0	fR/S	 140.5
U+8305.0	fTotalStrokes	 9; 11
U+8305.0	fCangjie	TNIH
U+8305.0	fFourCorner	44222
U+8305.0	fHakka	  mau2; mau3; bau3
U+8305.0	fCantonese	maau4
U+8305.0	fMandarin	mao2; mao1; mei4
U+8305.0	fEnglish	[1] couch grass [2] a Chinese family name
U+8306.0	fUTF8	茆
U+8306.0	fCNS11643	1-534A
U+8306.0	fBig5	AD67
U+8306.0	fGB	0-5C62
U+8306.0	fR/S	 140.5
U+8306.0	fTotalStrokes	 9; 11
U+8306.0	fCangjie	THHL
U+8306.0	fFourCorner	44727
U+8306.0	fHakka	  mau3; mau2
U+8306.0	fCantonese	maau5; maau4; lau5
U+8306.0	fMandarin	mao3; mao2; liu3
U+8306.0	fEnglish	[1] a species of grass (Brasenia schreberi) [2] same as $ (couh grass) [3] a Chinese family name
U+8307.0	fUTF8	茇
U+8307.0	fCNS11643	2-2D31
U+8307.0	fBig5	D06E
U+8307.0	fGB	0-5C58
U+8307.0	fR/S	 140.5
U+8307.0	fTotalStrokes	 9; 11
U+8307.0	fCangjie	TIKK
U+8307.0	fFourCorner	44447
U+8307.0	fHakka	  pat8
U+8307.0	fCantonese	baat6 (also); bat6; pui3
U+8307.0	fMandarin	ba2; bei4; pei4; fei4; fu2
U+8307.0	fEnglish	[1] a grass root [2] halt in the open country, amid the grass
U+8308.0	fUTF8	茈
U+8308.0	fCNS11643	2-3247
U+8308.0	fBig5	D3A5
U+8308.0	fGB	0-5C6B
U+8308.0	fR/S	 140.5
U+8308.0	fTotalStrokes	 10; 11
U+8308.0	fCangjie	TYMP
U+8308.0	fFourCorner	44111
U+8308.0	fHakka	 zih3; zii3
U+8308.0	fCantonese	zi2; ci4; caai4; ci2
U+8308.0	fMandarin	zi3; ci2; ci3
U+8308.0	fEnglish	kind of herb
U+8308.1	fUTF8	茈
U+8308.1	fCNS11643	2-3247
U+8308.1	fBig5	D3A5
U+8308.1	fGB	0-5C6B
U+8308.1	fR/S	 140.5
U+8308.1	fTotalStrokes	 10; 11
U+8308.1	fCangjie	TYMP
U+8308.1	fFourCorner	44111
U+8308.1	fMandarin	chai2
U+8308.1	fEnglish	Bupleurum, a kind of herb medicine
U+8309.0	fUTF8	茉
U+8309.0	fCNS11643	1-533E
U+8309.0	fBig5	AD5B
U+8309.0	fGB	0-5C54
U+8309.0	fR/S	 140.5
U+8309.0	fTotalStrokes	 9; 11
U+8309.0	fCangjie	TDJ
U+8309.0	fFourCorner	44904
U+8309.0	fHakka	 mat8
U+8309.0	fCantonese	mut6
U+8309.0	fMandarin	mo4
U+8309.0	fEnglish	white jasmine
U+830A.0	fUTF8	茊
U+830A.0	fCNS11643	4-2B3D
U+830A.0	fGB	E-3A40
U+830A.0	fR/S	 140.5
U+830A.0	fTotalStrokes	 9; 11
U+830A.0	fCangjie	TOM
U+830A.0	fFourCorner	44100
U+830A.0	fCantonese	zi1
U+830A.0	fMandarin	zi1; ci2
U+830B.0	fUTF8	茋
U+830B.0	fCNS11643	3-2E79
U+830B.0	fGB	E-3A41
U+830B.0	fR/S	 140.5
U+830B.0	fTotalStrokes	 9; 11
U+830B.0	fCangjie	THPM
U+830B.0	fFourCorner	44740
U+830B.0	fCantonese	zi2
U+830B.0	fMandarin	di3
U+830C.0	fUTF8	茌
U+830C.0	fCNS11643	2-2D3D
U+830C.0	fBig5	D07A
U+830C.0	fGB	0-5C5D
U+830C.0	fR/S	 140.5
U+830C.0	fTotalStrokes	 9; 11
U+830C.0	fCangjie	TOG
U+830C.0	fFourCorner	44214
U+830C.0	fHakka	 zih1
U+830C.0	fCantonese	ci4
U+830C.0	fMandarin	chi2
U+830C.0	fEnglish	used in place name
U+830D.0	fUTF8	茍
U+830D.0	fCNS11643	2-2942
U+830D.0	fBig5	CE41
U+830D.0	fGB	E-3A42
U+830D.0	fR/S	 140.5
U+830D.0	fTotalStrokes	 9; 11
U+830D.0	fCangjie	TPR
U+830D.0	fFourCorner	44627
U+830D.0	fHakka	 giu3
U+830D.0	fCantonese	gik1; gau2
U+830D.0	fMandarin	gou3; ji4
U+830D.0	fEnglish	to be circumspect, cautious in ones behaviour 
U+830E.0	fUTF8	茎
U+830E.0	fCNS11643	3-2E77
U+830E.0	fGB	0-3E25
U+830E.0	fR/S	 140.5
U+830E.0	fTotalStrokes	 9; 11
U+830E.0	fCangjie	TNOM
U+830E.0	fFourCorner	44100
U+830E.0	fHakka	 gin1; kwang3
U+830E.0	fCantonese	hang1; ging3 (also); hang4
U+830E.0	fMandarin	jing1; xing2; ying1
U+830E.0	fEnglish	simplified form of &#x8396;
U+830F.0	fUTF8	茏
U+830F.0	fGB	0-5C57
U+830F.0	fR/S	 140.5
U+830F.0	fTotalStrokes	 9; 11
U+830F.0	fCangjie	TIKP
U+830F.0	fHakka	 lung2
U+830F.0	fCantonese	lung4
U+830F.0	fMandarin	long2; long3; long4
U+830F.0	fEnglish	simplified form of &#x8622;
U+8310.0	fUTF8	茐
U+8310.0	fCNS11643	3-6635
U+8310.0	fGB	E-3A43
U+8310.0	fR/S	 140.5
U+8310.0	fTotalStrokes	 9; 11
U+8310.0	fCangjie	TPKK
U+8310.0	fFourCorner	4442
U+8310.0	fCantonese	cung1
U+8311.0	fUTF8	茑
U+8311.0	fGB	0-5C60
U+8311.0	fR/S	 140.5; 140.6
U+8311.0	fTotalStrokes	 9; 11
U+8311.0	fCangjie	TPYM
U+8311.0	fHakka	 ngiau3
U+8311.0	fCantonese	niu5; liu5 (also)
U+8311.0	fMandarin	niao3; diao3; diao4
U+8311.0	fEnglish	simplified form of &#x8526;
U+8312.0	fUTF8	茒
U+8312.0	fCNS11643	3-6637
U+8312.0	fGB	E-3A44
U+8312.0	fR/S	 140.6
U+8312.0	fTotalStrokes	 10; 12
U+8312.0	fCangjie	TNIL
U+8312.0	fFourCorner	4422
U+8312.0	fCantonese	jyun2
U+8313.0	fUTF8	茓
U+8313.0	fGB	3-6859
U+8313.0	fR/S	 140.5
U+8313.0	fTotalStrokes	 9; 11
U+8313.0	fCangjie	TJC
U+8313.0	fCantonese	jyut6
U+8313.0	fMandarin	xue2
U+8314.0	fUTF8	茔
U+8314.0	fGB	0-5C63
U+8314.0	fR/S	 140.5
U+8314.0	fTotalStrokes	 9; 11
U+8314.0	fCangjie	XTBG
U+8314.0	fFourCorner	4410
U+8314.0	fHakka	 jin2
U+8314.0	fCantonese	jing4
U+8314.0	fMandarin	ying2
U+8314.0	fEnglish	simplified form of &#x584B;
U+8315.0	fUTF8	茕
U+8315.0	fGB	0-5C64
U+8315.0	fR/S	 140.5
U+8315.0	fTotalStrokes	 9; 11
U+8315.0	fCangjie	TBNJ
U+8315.0	fHakka	 kiun2
U+8315.0	fCantonese	king4
U+8315.0	fMandarin	qiong2
U+8315.0	fEnglish	simplified form of &#x7162;
U+8316.0	fUTF8	茖
U+8316.0	fCNS11643	2-324A
U+8316.0	fBig5	D3A8
U+8316.0	fGB	3-6873
U+8316.0	fR/S	 140.6
U+8316.0	fTotalStrokes	 10; 12
U+8316.0	fCangjie	THER
U+8316.0	fFourCorner	44604
U+8316.0	fHakka	 gak7; giet7; get7
U+8316.0	fCantonese	gaak3; gok3
U+8316.0	fMandarin	ge2; ge4
U+8316.0	fEnglish	Allium victorialis
U+8317.0	fUTF8	茗
U+8317.0	fCNS11643	1-577E
U+8317.0	fBig5	AFFA
U+8317.0	fGB	0-5C78
U+8317.0	fR/S	 140.6
U+8317.0	fTotalStrokes	 10; 12
U+8317.0	fCangjie	TNIR
U+8317.0	fFourCorner	44607
U+8317.0	fHakka	 miang2; men2; men3
U+8317.0	fCantonese	ming4; ming5 (also)
U+8317.0	fMandarin	ming2; ming3
U+8317.0	fEnglish	 tea; tea plant
U+8318.0	fUTF8	茘
U+8318.0	fCNS11643	3-3362
U+8318.0	fGB	E-3A45
U+8318.0	fR/S	 140.6
U+8318.0	fTotalStrokes	 10; 12
U+8318.0	fCangjie	TSHH
U+8318.0	fFourCorner	44220
U+8318.0	fHakka	 li6; li5
U+8318.0	fCantonese	lai6
U+8318.0	fMandarin	li4
U+8318.0	fEnglish	a scallion, small onion
U+8319.0	fUTF8	茙
U+8319.0	fCNS11643	2-323A
U+8319.0	fBig5	D376
U+8319.0	fGB	3-685F
U+8319.0	fR/S	 140.6
U+8319.0	fTotalStrokes	 10; 12
U+8319.0	fCangjie	TIJ
U+8319.0	fFourCorner	44453
U+8319.0	fHakka	 jung2; rung2
U+8319.0	fCantonese	jung4
U+8319.0	fMandarin	rong2
U+8319.0	fEnglish	a malvaceous plant resembling Hibiscus
U+831A.0	fUTF8	茚
U+831A.0	fCNS11643	4-2F5D
U+831A.0	fGB	0-5C61
U+831A.0	fR/S	 140.6; 140.5
U+831A.0	fTotalStrokes	 10; 12; 9
U+831A.0	fCangjie	THPL
U+831A.0	fFourCorner	44720
U+831A.0	fCantonese	jan3
U+831A.0	fMandarin	yin4
U+831A.0	fEnglish	an organic compound
U+831B.0	fUTF8	茛
U+831B.0	fCNS11643	2-3245
U+831B.0	fBig5	D3A3
U+831B.0	fGB	0-5D22
U+831B.0	fR/S	 140.6
U+831B.0	fTotalStrokes	 10; 12
U+831B.0	fCangjie	TAV
U+831B.0	fFourCorner	44732
U+831B.0	fHakka	 gien5; gen5
U+831B.0	fCantonese	gan3; loeng4; long4
U+831B.0	fMandarin	gen4; jian4
U+831B.0	fEnglish	Ranunculuc acris
U+831C.0	fUTF8	茜
U+831C.0	fCNS11643	2-3241
U+831C.0	fBig5	D37D
U+831C.0	fGB	0-5C67
U+831C.0	fR/S	 140.6
U+831C.0	fTotalStrokes	 10; 12
U+831C.0	fCangjie	TMCW
U+831C.0	fFourCorner	44601
U+831C.0	fHakka	  cien5;  cin5; cen3; cen5; cin1
U+831C.0	fCantonese	sin6; sai1; sin3
U+831C.0	fMandarin	qian4; xi1; qian1
U+831C.0	fEnglish	madder
U+831D.0	fUTF8	茝
U+831D.0	fCNS11643	3-384D
U+831D.0	fGB	3-687D
U+831D.0	fR/S	 140.7
U+831D.0	fTotalStrokes	 11; 13
U+831D.0	fCangjie	TSLL
U+831D.0	fFourCorner	44710
U+831D.0	fHakka	 ngi5; chi3
U+831D.0	fCantonese	coi2; zi2
U+831D.0	fMandarin	chai3; zhi3
U+831D.0	fEnglish	angelica
U+831E.0	fUTF8	茞
U+831E.0	fCNS11643	2-3254
U+831E.0	fBig5	D3B2
U+831E.0	fGB	5-676E
U+831E.0	fR/S	 140.6; 140.7
U+831E.0	fTotalStrokes	 10; 12
U+831E.0	fCangjie	TSLL
U+831E.0	fFourCorner	44712
U+831E.0	fCantonese	coi2; zi2 (also)
U+831E.0	fMandarin	chen2
U+831E.0	fEnglish	 fragrant plant Angelica
U+831F.0	fUTF8	茟
U+831F.0	fCNS11643	3-3359
U+831F.0	fGB	5-6823
U+831F.0	fR/S	  140.6
U+831F.0	fTotalStrokes	 10; 12
U+831F.0	fCangjie	TLQ
U+831F.0	fFourCorner	44500
U+831F.0	fCantonese	jyut6
U+831F.0	fMandarin	yu4
U+8320.0	fUTF8	茠
U+8320.0	fCNS11643	2-324C
U+8320.0	fBig5	D3AA
U+8320.0	fGB	3-686F
U+8320.0	fR/S	 140.6
U+8320.0	fTotalStrokes	 10; 12
U+8320.0	fCangjie	TOD
U+8320.0	fFourCorner	44294
U+8320.0	fHakka	 gau3;  hiu3
U+8320.0	fCantonese	hou1; jau1
U+8320.0	fMandarin	xiu1; hao1; hou4
U+8320.0	fEnglish	 weed eradicate
U+8321.0	fUTF8	茡
U+8321.0	fCNS11643	3-335F
U+8321.0	fGB	3-6877
U+8321.0	fR/S	  140.6
U+8321.0	fTotalStrokes	 10; 12
U+8321.0	fCangjie	TJND
U+8321.0	fFourCorner	44400
U+8321.0	fCantonese	zi6
U+8321.0	fMandarin	zi4
U+8322.0	fUTF8	茢
U+8322.0	fCNS11643	2-3242
U+8322.0	fBig5	D37E
U+8322.0	fGB	3-6865
U+8322.0	fR/S	 140.6
U+8322.0	fTotalStrokes	 10; 12
U+8322.0	fCangjie	TMNN
U+8322.0	fFourCorner	44220
U+8322.0	fHakka	  liet8; let8
U+8322.0	fCantonese	lit6
U+8322.0	fMandarin	lie4
U+8322.0	fEnglish	 sedges rushes
U+8323.0	fUTF8	茣
U+8323.0	fCNS11643	3-385B
U+8323.0	fGB	E-3A46
U+8323.0	fR/S	  140.7
U+8323.0	fTotalStrokes	 11; 13
U+8323.0	fCangjie	TRVK
U+8323.0	fFourCorner	44430
U+8323.0	fCantonese	ng4
U+8323.0	fMandarin	wu2
U+8323.0	fEnglish	mat, matting
U+8324.0	fUTF8	茤
U+8324.0	fCNS11643	2-324B
U+8324.0	fBig5	D3A9
U+8324.0	fGB	3-6874
U+8324.0	fR/S	 140.6
U+8324.0	fTotalStrokes	 10; 12
U+8324.0	fCangjie	TNIN
U+8324.0	fFourCorner	44207
U+8324.0	fCantonese	gei6; do1
U+8324.0	fMandarin	duo1; ji4
U+8325.0	fUTF8	茥
U+8325.0	fCNS11643	2-323C
U+8325.0	fBig5	D378
U+8325.0	fGB	5-676C
U+8325.0	fR/S	 140.6
U+8325.0	fTotalStrokes	 10; 12
U+8325.0	fCangjie	TGG
U+8325.0	fFourCorner	44104
U+8325.0	fCantonese	gwai1; kwai1
U+8325.0	fMandarin	gui1; kui1; gui4
U+8326.0	fUTF8	茦
U+8326.0	fCNS11643	2-3240
U+8326.0	fBig5	D37C
U+8326.0	fGB	3-6862
U+8326.0	fR/S	 140.6
U+8326.0	fTotalStrokes	 10; 12
U+8326.0	fCangjie	TDB
U+8326.0	fFourCorner	44902
U+8326.0	fHakka	  cet7;  cih5
U+8326.0	fCantonese	caak3; zaak3
U+8326.0	fMandarin	ce4; ci4
U+8326.0	fEnglish	thorn
U+8327.0	fUTF8	茧
U+8327.0	fCNS11643	2-3257
U+8327.0	fBig5	D3B5
U+8327.0	fGB	0-3C6B
U+8327.0	fR/S	 140.6
U+8327.0	fTotalStrokes	 10; 12
U+8327.0	fCangjie	TLMI
U+8327.0	fFourCorner	44136
U+8327.0	fHakka	 gan3
U+8327.0	fCantonese	gaan2; gin2
U+8327.0	fMandarin	jian3; chong2
U+8327.0	fEnglish	simplified form of &#x7E6D;
U+8328.0	fUTF8	茨
U+8328.0	fCNS11643	1-5823
U+8328.0	fBig5	AFFD
U+8328.0	fGB	0-3444
U+8328.0	fR/S	 140.6
U+8328.0	fTotalStrokes	 10; 12
U+8328.0	fCangjie	TIMO
U+8328.0	fFourCorner	44182
U+8328.0	fHakka	 cih1;  zih1; ze2
U+8328.0	fCantonese	ci4
U+8328.0	fMandarin	ci2
U+8328.0	fEnglish	[1] thatch; thatched house [2] Tribulus terrestris, a kind of thorny plant [3] fill with earth
U+8329.0	fUTF8	茩
U+8329.0	fCNS11643	2-324F
U+8329.0	fBig5	D3AD
U+8329.0	fGB	5-677C
U+8329.0	fR/S	 140.6
U+8329.0	fTotalStrokes	 10; 12
U+8329.0	fCangjie	THMR
U+8329.0	fFourCorner	44261
U+8329.0	fHakka	 giau5; geu3
U+8329.0	fCantonese	hau6; gau2
U+8329.0	fMandarin	gou3; hou4
U+8329.0	fEnglish	a medicinal plant the seeds of which are used for inflamed eyes
U+832A.0	fUTF8	茪
U+832A.0	fCNS11643	2-3246
U+832A.0	fBig5	D3A4
U+832A.0	fGB	5-6775
U+832A.0	fR/S	 140.6
U+832A.0	fTotalStrokes	 10; 12
U+832A.0	fCangjie	TFMU
U+832A.0	fFourCorner	44211
U+832A.0	fCantonese	gwong1; gong1 (also)
U+832A.0	fMandarin	guang1
U+832B.0	fUTF8	茫
U+832B.0	fCNS11643	1-5771
U+832B.0	fBig5	AFED
U+832B.0	fGB	0-4323
U+832B.0	fR/S	 140.6
U+832B.0	fTotalStrokes	 10; 12
U+832B.0	fCangjie	TEYV
U+832B.0	fFourCorner	44110
U+832B.0	fHakka	  mong2; mong3
U+832B.0	fCantonese	mong4
U+832B.0	fMandarin	mang2; mang3; huang3
U+832B.0	fEnglish	[1] vast; boundless; widespread [2] vague; uncertain
U+832B.1	fUTF8	茫
U+832B.1	fCNS11643	1-5771
U+832B.1	fBig5	AFED
U+832B.1	fGB	0-4323
U+832B.1	fR/S	 140.6
U+832B.1	fTotalStrokes	 10; 12
U+832B.1	fCangjie	TEYV
U+832B.1	fFourCorner	44110
U+832B.1	fMandarin	mang1
U+832C.0	fUTF8	茬
U+832C.0	fCNS11643	2-3255
U+832C.0	fBig5	D3B3
U+832C.0	fGB	0-3267
U+832C.0	fR/S	 140.6
U+832C.0	fTotalStrokes	 10; 12
U+832C.0	fCangjie	TKLG
U+832C.0	fFourCorner	44214
U+832C.0	fHakka	 zih1; chi1; ca2; cih1
U+832C.0	fCantonese	caa4; ci4
U+832C.0	fMandarin	chi2; cha2; zi1
U+832C.0	fEnglish	harvest
U+832D.0	fUTF8	茭
U+832D.0	fCNS11643	2-3238
U+832D.0	fBig5	D374
U+832D.0	fGB	0-5C7A
U+832D.0	fR/S	 140.6
U+832D.0	fTotalStrokes	 10; 12
U+832D.0	fCangjie	TYCK
U+832D.0	fFourCorner	44408
U+832D.0	fHakka	 gau3; gau1
U+832D.0	fCantonese	gaau1
U+832D.0	fMandarin	jiao1; qiao4; xiao4; ji1; xiao3
U+832D.0	fEnglish	Zizania latifolia, an aquatic grass, the stalks of which are eaten as a vegetable
U+832E.0	fUTF8	茮
U+832E.0	fCNS11643	4-2F5A
U+832E.0	fGB	3-686A
U+832E.0	fR/S	  140.6
U+832E.0	fTotalStrokes	 10; 12
U+832E.0	fCangjie	TYMF
U+832E.0	fFourCorner	44900
U+832E.0	fCantonese	ziu1
U+832E.0	fMandarin	jiao1
U+832F.0	fUTF8	茯
U+832F.0	fCNS11643	2-324E
U+832F.0	fBig5	D3AC
U+832F.0	fGB	0-5C72
U+832F.0	fR/S	 140.6
U+832F.0	fTotalStrokes	 10; 12
U+832F.0	fCangjie	TOIK
U+832F.0	fFourCorner	44234
U+832F.0	fHakka	 fuk8
U+832F.0	fCantonese	fuk6
U+832F.0	fMandarin	fu2; bei4; bo2
U+832F.0	fEnglish	tuckahoe
U+8330.0	fUTF8	茰
U+8330.0	fCNS11643	4-2F63
U+8330.0	fGB	E-3A47
U+8330.0	fR/S	  140.6
U+8330.0	fTotalStrokes	 10; 12
U+8330.0	fCangjie	TLWK
U+8330.0	fFourCorner	44400
U+8330.0	fHakka	 ji2; ri2
U+8330.0	fCantonese	jyu4
U+8330.0	fMandarin	yu2
U+8331.0	fUTF8	茱
U+8331.0	fCNS11643	1-5822
U+8331.0	fBig5	AFFC
U+8331.0	fGB	0-5C6F
U+8331.0	fR/S	 140.6
U+8331.0	fTotalStrokes	 10; 12
U+8331.0	fCangjie	THJD
U+8331.0	fFourCorner	44904
U+8331.0	fHakka	 zu1;  zhu1
U+8331.0	fCantonese	zyu1; syu4
U+8331.0	fMandarin	zhu1; shu1
U+8331.0	fEnglish	dogwood
U+8332.0	fUTF8	茲
U+8332.0	fCNS11643	1-577B
U+8332.0	fBig5	AFF7
U+8332.0	fGB	E-3A48
U+8332.0	fR/S	 140.6
U+8332.0	fTotalStrokes	 10; 12
U+8332.0	fCangjie	TVII
U+8332.0	fFourCorner	44732
U+8332.0	fHakka	 zih1; zu1; cih1; cu2; zii1
U+8332.0	fCantonese	zi1; ci4
U+8332.0	fMandarin	zi1; ci1
U+8332.0	fEnglish	[1] now; here; at present [2] this [3] year
U+8332.1	fUTF8	茲
U+8332.1	fCNS11643	1-577B
U+8332.1	fBig5	AFF7
U+8332.1	fGB	E-3A48
U+8332.1	fR/S	 140.6
U+8332.1	fTotalStrokes	 10; 12
U+8332.1	fCangjie	TVII
U+8332.1	fFourCorner	44732
U+8332.1	fMandarin	ci2
U+8332.1	fEnglish	used in combination
U+8333.0	fUTF8	茳
U+8333.0	fCNS11643	2-3237
U+8333.0	fBig5	D373
U+8333.0	fGB	0-5C7C
U+8333.0	fR/S	 140.6
U+8333.0	fTotalStrokes	 10; 12
U+8333.0	fCangjie	TEM
U+8333.0	fFourCorner	44111
U+8333.0	fHakka	 gong1
U+8333.0	fCantonese	gong1
U+8333.0	fMandarin	jiang1
U+8333.0	fEnglish	a leguminous plant (Cassia tora)
U+8334.0	fUTF8	茴
U+8334.0	fCNS11643	1-5779
U+8334.0	fBig5	AFF5
U+8334.0	fGB	0-5C6E
U+8334.0	fR/S	 140.6
U+8334.0	fTotalStrokes	 10; 12
U+8334.0	fCangjie	TWR
U+8334.0	fFourCorner	44600
U+8334.0	fHakka	 fui2; fi2; fui3
U+8334.0	fCantonese	wui4
U+8334.0	fMandarin	hui2
U+8334.0	fEnglish	fennel; aniseed
U+8335.0	fUTF8	茵
U+8335.0	fCNS11643	1-5778
U+8335.0	fBig5	AFF4
U+8335.0	fGB	0-5270
U+8335.0	fR/S	 140.6
U+8335.0	fTotalStrokes	 10; 12
U+8335.0	fCangjie	TWK
U+8335.0	fFourCorner	44600
U+8335.0	fHakka	 jin1; rin1
U+8335.0	fCantonese	jan1
U+8335.0	fMandarin	yin1
U+8335.0	fEnglish	 cushion; mattress; mat; carpet
U+8336.0	fUTF8	茶
U+8336.0	fCNS11643	1-577D
U+8336.0	fBig5	AFF9
U+8336.0	fGB	0-3268
U+8336.0	fR/S	 140.6
U+8336.0	fTotalStrokes	 10; 12
U+8336.0	fCangjie	TOD
U+8336.0	fFourCorner	44904
U+8336.0	fHakka	  ca2; ca3
U+8336.0	fCantonese	caa4; caa2 (also)
U+8336.0	fMandarin	cha2
U+8336.0	fEnglish	tea
U+8337.0	fUTF8	茷
U+8337.0	fCNS11643	2-324D
U+8337.0	fBig5	D3AB
U+8337.0	fGB	5-6779
U+8337.0	fR/S	 140.6
U+8337.0	fTotalStrokes	 10; 12
U+8337.0	fCangjie	TOI
U+8337.0	fFourCorner	44253
U+8337.0	fHakka	 fat8; pui5; pi5
U+8337.0	fCantonese	fat6; pui3; bui6
U+8337.0	fMandarin	fa2; fei4; pei4; ba2; bei4
U+8337.0	fEnglish	 flutter flower
U+8338.0	fUTF8	茸
U+8338.0	fCNS11643	1-5775
U+8338.0	fBig5	AFF1
U+8338.0	fGB	0-4857
U+8338.0	fR/S	 140.6
U+8338.0	fTotalStrokes	 10; 12
U+8338.0	fCangjie	TSJ
U+8338.0	fFourCorner	44401
U+8338.0	fHakka	 jung2; rung2; jung3
U+8338.0	fCantonese	jung4; jung5; jung2
U+8338.0	fMandarin	rong2; rong3
U+8338.0	fEnglish	[1] soft; down; fine hair [2] soft; downy; fluffy [3] untidy; messy; confused; disorderly [4] luxuriant; lush
U+8339.0	fUTF8	茹
U+8339.0	fCNS11643	1-577C
U+8339.0	fBig5	AFF8
U+8339.0	fGB	0-4863
U+8339.0	fR/S	 140.6
U+8339.0	fTotalStrokes	 10; 12
U+8339.0	fCangjie	TVR
U+8339.0	fFourCorner	44460
U+8339.0	fHakka	 ji2; ri2; ji3; i2; gu1
U+8339.0	fCantonese	jyu4; jyu6
U+8339.0	fMandarin	ru2; ru4; na2; ru3
U+8339.0	fEnglish	[1] entangled roots [2] eat; taste; mouth [3] experience [4] stinking
U+833A.0	fUTF8	茺
U+833A.0	fCNS11643	2-2D35
U+833A.0	fBig5	D072
U+833A.0	fGB	0-5C7B
U+833A.0	fR/S	 140.6
U+833A.0	fTotalStrokes	 10; 12
U+833A.0	fCangjie	TYIU
U+833A.0	fFourCorner	44213
U+833A.0	fHakka	 chung1; cung1
U+833A.0	fCantonese	cung1
U+833A.0	fMandarin	chong1
U+833A.0	fEnglish	a labiate plant used as medicine in female diseases
U+833B.0	fUTF8	茻
U+833B.0	fCNS11643	2-3F41
U+833B.0	fBig5	DB5C
U+833B.0	fGB	E-3A49
U+833B.0	fR/S	 140.6
U+833B.0	fTotalStrokes	 12
U+833B.0	fCangjie	UUUU
U+833B.0	fFourCorner	22447
U+833B.0	fCantonese	mong5; mou5
U+833B.0	fMandarin	mang3; mu3; wang3
U+833C.0	fUTF8	茼
U+833C.0	fCNS11643	2-3248
U+833C.0	fBig5	D3A6
U+833C.0	fGB	0-5C6D
U+833C.0	fR/S	 140.6
U+833C.0	fTotalStrokes	 10; 12
U+833C.0	fCangjie	TBMR
U+833C.0	fFourCorner	44227
U+833C.0	fHakka	 tung2
U+833C.0	fCantonese	tung4
U+833C.0	fMandarin	tong2
U+833C.0	fEnglish	a kind of green vegetable
U+833D.0	fUTF8	茽
U+833D.0	fCNS11643	4-2F5F
U+833D.0	fGB	E-3A4A
U+833D.0	fR/S	 140.6
U+833D.0	fTotalStrokes	 10; 12
U+833D.0	fCangjie	TOL
U+833D.0	fFourCorner	44250
U+833D.0	fCantonese	zung6
U+833D.0	fMandarin	zhong4
U+833E.0	fUTF8	茾
U+833E.0	fCNS11643	3-2A5F
U+833E.0	fGB	E-3A4B
U+833E.0	fR/S	 140.4
U+833E.0	fTotalStrokes	 8; 10
U+833E.0	fCangjie	TMT
U+833E.0	fFourCorner	44440
U+833E.0	fCantonese	hin1
U+833F.0	fUTF8	茿
U+833F.0	fCNS11643	2-323E
U+833F.0	fBig5	D37A
U+833F.0	fGB	5-676D
U+833F.0	fR/S	 140.6
U+833F.0	fTotalStrokes	 10; 12
U+833F.0	fCangjie	TMHN
U+833F.0	fFourCorner	44117
U+833F.0	fCantonese	zuk1
U+833F.0	fMandarin	zhu2
U+8340.0	fUTF8	荀
U+8340.0	fCNS11643	1-5821
U+8340.0	fBig5	AFFB
U+8340.0	fGB	0-5C77
U+8340.0	fR/S	 140.6
U+8340.0	fTotalStrokes	 10; 12
U+8340.0	fCangjie	TPA
U+8340.0	fFourCorner	44627
U+8340.0	fHakka	 sun2; sun3
U+8340.0	fCantonese	seon1
U+8340.0	fMandarin	xun2
U+8340.0	fEnglish	[1] name of an ancient state [2] a kind of herb [3] a Chinese family name
U+8341.0	fUTF8	荁
U+8341.0	fCNS11643	2-323F
U+8341.0	fBig5	D37B
U+8341.0	fGB	3-6861
U+8341.0	fR/S	 140.6
U+8341.0	fTotalStrokes	 10; 12
U+8341.0	fCangjie	TMAM
U+8341.0	fFourCorner	44106
U+8341.0	fHakka	  fan2
U+8341.0	fCantonese	wun4 (also); jyun4
U+8341.0	fMandarin	huan2
U+8341.0	fEnglish	celery
U+8342.0	fUTF8	荂
U+8342.0	fCNS11643	2-3243
U+8342.0	fBig5	D3A1
U+8342.0	fGB	5-6772
U+8342.0	fR/S	 140.6
U+8342.0	fTotalStrokes	 10; 12
U+8342.0	fCangjie	TKMS
U+8342.0	fFourCorner	44127
U+8342.0	fCantonese	fu1; heoi1; kwaa1
U+8342.0	fMandarin	kua1; fu1; hua1; xu1
U+8343.0	fUTF8	荃
U+8343.0	fCNS11643	1-5824
U+8343.0	fBig5	AFFE
U+8343.0	fGB	0-5C75
U+8343.0	fR/S	 140.6
U+8343.0	fTotalStrokes	 10; 12
U+8343.0	fCangjie	TOMG
U+8343.0	fFourCorner	44104
U+8343.0	fHakka	  cien2; cion2; cen2
U+8343.0	fCantonese	cyun4
U+8343.0	fMandarin	quan2; sun1; chuo4
U+8343.0	fEnglish	[1] a fragrant herb [2] fine cloth
U+8344.0	fUTF8	荄
U+8344.0	fCNS11643	2-3239
U+8344.0	fBig5	D375
U+8344.0	fGB	3-6875
U+8344.0	fR/S	 140.6
U+8344.0	fTotalStrokes	 10; 12
U+8344.0	fCangjie	TYVO
U+8344.0	fFourCorner	44802
U+8344.0	fHakka	  goi1
U+8344.0	fCantonese	goi1
U+8344.0	fMandarin	gai1; jie1
U+8344.0	fEnglish	roots of grass
U+8345.0	fUTF8	荅
U+8345.0	fCNS11643	2-3251
U+8345.0	fBig5	D3AF
U+8345.0	fGB	3-6870
U+8345.0	fR/S	 140.6
U+8345.0	fTotalStrokes	 10; 12
U+8345.0	fCangjie	TOMR
U+8345.0	fFourCorner	44601
U+8345.0	fHakka	 dap7
U+8345.0	fCantonese	taap3; daap3
U+8345.0	fMandarin	da1; da2; ta4
U+8345.0	fEnglish	 answer small bean thick
U+8346.0	fUTF8	荆
U+8346.0	fCNS11643	3-3E57
U+8346.0	fGB	0-3E23
U+8346.0	fR/S	 140.6; 18.8
U+8346.0	fTotalStrokes	 10; 12
U+8346.0	fCangjie	TJLN
U+8346.0	fFourCorner	42400
U+8346.0	fHakka	  gin1
U+8346.0	fCantonese	ging1
U+8346.0	fMandarin	jing1
U+8346.0	fEnglish	[1] a kind of bramble; thorn [2] a cane for punishment used in ancient China [3] my wife (a polite expression) [4] name of one of the nine political regions (ancient China) [5] a Chinese family name
U+8347.0	fUTF8	荇
U+8347.0	fCNS11643	2-3250
U+8347.0	fBig5	D3AE
U+8347.0	fGB	0-5C74
U+8347.0	fR/S	 140.6
U+8347.0	fTotalStrokes	 10; 12
U+8347.0	fCangjie	THON
U+8347.0	fFourCorner	44221
U+8347.0	fHakka	 hen5; hen6; hen3; hem6
U+8347.0	fCantonese	hang6
U+8347.0	fMandarin	xing4
U+8347.0	fEnglish	Nymphoides peltalum, a kind of vegetable; duckweed
U+8348.0	fUTF8	荈
U+8348.0	fCNS11643	2-3258
U+8348.0	fBig5	D3B6
U+8348.0	fGB	3-6872
U+8348.0	fR/S	 140.6
U+8348.0	fTotalStrokes	 10; 12
U+8348.0	fCangjie	TNIQ
U+8348.0	fFourCorner	44250
U+8348.0	fHakka	 con3;  chon3
U+8348.0	fCantonese	cyun2
U+8348.0	fMandarin	chuan3; chuan4; quan4
U+8348.0	fEnglish	old leaves on the tea-plant
U+8349.0	fUTF8	草
U+8349.0	fCNS11643	1-5777
U+8349.0	fBig5	AFF3
U+8349.0	fGB	0-325D
U+8349.0	fR/S	 140.6
U+8349.0	fTotalStrokes	 10; 12
U+8349.0	fCangjie	TAJ
U+8349.0	fFourCorner	44406
U+8349.0	fHakka	  cau3;  co3
U+8349.0	fCantonese	cou2
U+8349.0	fMandarin	cao3; zao4
U+8349.0	fEnglish	[1] grass; herbs; weed [2] coarse; crude [3] draft (writing); draft [4] the script type of Chinese calligraphy
U+834A.0	fUTF8	荊
U+834A.0	fCNS11643	1-5774
U+834A.0	fBig5	AFF0
U+834A.0	fGB	E-3A4C
U+834A.0	fR/S	 140.6
U+834A.0	fTotalStrokes	 10; 12
U+834A.0	fCangjie	TMTN
U+834A.0	fFourCorner	44400
U+834A.0	fHakka	 gin1
U+834A.0	fCantonese	ging1
U+834A.0	fMandarin	jing1
U+834A.0	fEnglish	 thorns brambles my wife cane
U+834B.0	fUTF8	荋
U+834B.0	fCNS11643	2-3256
U+834B.0	fBig5	D3B4
U+834B.0	fGB	3-6863
U+834B.0	fR/S	 140.6
U+834B.0	fTotalStrokes	 10; 12
U+834B.0	fCangjie	TMBL
U+834B.0	fFourCorner	44227
U+834B.0	fHakka	 ji2; ri2
U+834B.0	fCantonese	ji4
U+834B.0	fMandarin	er2
U+834B.0	fEnglish	a fungus
U+834C.0	fUTF8	荌
U+834C.0	fCNS11643	2-3252
U+834C.0	fBig5	D3B0
U+834C.0	fGB	5-6822
U+834C.0	fR/S	 140.6
U+834C.0	fTotalStrokes	 10; 12
U+834C.0	fCangjie	TJV
U+834C.0	fFourCorner	44404
U+834C.0	fCantonese	on3; ngon3 (also)
U+834C.0	fMandarin	an4
U+834D.0	fUTF8	荍
U+834D.0	fCNS11643	2-3249
U+834D.0	fBig5	D3A7
U+834D.0	fGB	E-3A4D
U+834D.0	fR/S	 140.6
U+834D.0	fTotalStrokes	 10; 12
U+834D.0	fCangjie	TVLK
U+834D.0	fFourCorner	44748
U+834D.0	fHakka	 kiau1; kiau2
U+834D.0	fCantonese	kiu4
U+834D.0	fMandarin	shou1; qiao2
U+834D.0	fEnglish	 buckwheat a herbal medicine
U+834E.0	fUTF8	荎
U+834E.0	fCNS11643	2-3244
U+834E.0	fBig5	D3A2
U+834E.0	fGB	3-6867
U+834E.0	fR/S	 140.6
U+834E.0	fTotalStrokes	 10; 12
U+834E.0	fCangjie	TMIG
U+834E.0	fFourCorner	44104
U+834E.0	fHakka	 diet7; zhi5; zih5
U+834E.0	fCantonese	ci4; zi3
U+834E.0	fMandarin	chi2; die2; zhi4
U+834E.0	fEnglish	seeds of a plant they resemble gall-nuts
U+834F.0	fUTF8	荏
U+834F.0	fCNS11643	1-577A
U+834F.0	fBig5	AFF6
U+834F.0	fGB	0-5C73
U+834F.0	fR/S	 140.6
U+834F.0	fTotalStrokes	 10; 12
U+834F.0	fCangjie	TOHG
U+834F.0	fFourCorner	44214
U+834F.0	fHakka	 ngim3; ngim5
U+834F.0	fCantonese	jam5; jam6 (also); jam3
U+834F.0	fMandarin	ren3
U+834F.0	fEnglish	[1] Perilla frutescens, whose seeds are bird feed [2] soft; weak; fragile
U+8350.0	fUTF8	荐
U+8350.0	fCNS11643	1-5776
U+8350.0	fBig5	AFF2
U+8350.0	fGB	0-3C76
U+8350.0	fR/S	 140.6
U+8350.0	fTotalStrokes	 10; 12
U+8350.0	fCangjie	TKLD
U+8350.0	fFourCorner	44247
U+8350.0	fHakka	  zien5; zen5
U+8350.0	fCantonese	zin3; zin6
U+8350.0	fMandarin	jian4; jin4
U+8350.0	fEnglish	[1] repeatedly; again and again; reoccur [2] recommend; offer [3] straw mattress [4] simplified form of &#x85A6;
U+8351.0	fUTF8	荑
U+8351.0	fCNS11643	2-323B
U+8351.0	fBig5	D377
U+8351.0	fGB	0-5C68
U+8351.0	fR/S	 140.6
U+8351.0	fTotalStrokes	 10; 12
U+8351.0	fCangjie	TKN
U+8351.0	fFourCorner	44532
U+8351.0	fHakka	 ji2; ti2; ri2
U+8351.0	fCantonese	tai4; ji4
U+8351.0	fMandarin	ti2
U+8351.0	fEnglish	sprouts
U+8351.1	fUTF8	荑
U+8351.1	fCNS11643	2-323B
U+8351.1	fBig5	D377
U+8351.1	fGB	0-5C68
U+8351.1	fR/S	 140.6
U+8351.1	fTotalStrokes	 10; 12
U+8351.1	fCangjie	TKN
U+8351.1	fFourCorner	44532
U+8351.1	fMandarin	yi2
U+8351.1	fEnglish	tares
U+8352.0	fUTF8	荒
U+8352.0	fCNS11643	1-5772
U+8352.0	fBig5	AFEE
U+8352.0	fGB	0-3B44
U+8352.0	fR/S	 140.6
U+8352.0	fTotalStrokes	 10; 12
U+8352.0	fCangjie	TYVU
U+8352.0	fFourCorner	44211
U+8352.0	fHakka	  fong1; fang5; fong3
U+8352.0	fCantonese	fong1
U+8352.0	fMandarin	huang1; huang5; huang3; kang1
U+8352.0	fEnglish	[1] wasteland; desolate; uncultivated; wild; deserted; barren; waste [2] absurd; ridiculous [3] famine; scarcity; deficiency [4] neglect [5] a Chinese family name
U+8353.0	fUTF8	荓
U+8353.0	fCNS11643	2-3253
U+8353.0	fBig5	D3B1
U+8353.0	fGB	3-6876
U+8353.0	fR/S	 140.8
U+8353.0	fTotalStrokes	 12; 14
U+8353.0	fCangjie	TTT
U+8353.0	fFourCorner	44441
U+8353.0	fHakka	 pen2; pin2
U+8353.0	fCantonese	ping4
U+8353.0	fMandarin	ping2; qian1; ping1
U+8353.0	fEnglish	a species of iris
U+8354.0	fUTF8	荔
U+8354.0	fCNS11643	1-5773
U+8354.0	fBig5	AFEF
U+8354.0	fGB	0-4073
U+8354.0	fR/S	 140.6
U+8354.0	fTotalStrokes	 10; 12
U+8354.0	fCangjie	TKSS
U+8354.0	fFourCorner	44227
U+8354.0	fHakka	 lai1;  li5; li6; lai3
U+8354.0	fCantonese	lai6
U+8354.0	fMandarin	li4; lian4
U+8354.0	fEnglish	the lichee
U+8355.0	fUTF8	荕
U+8355.0	fCNS11643	4-2F5C
U+8355.0	fGB	E-3A4E
U+8355.0	fR/S	  140.6
U+8355.0	fTotalStrokes	 10; 12
U+8355.0	fCangjie	TBKS
U+8355.0	fFourCorner	44220
U+8355.0	fCantonese	gan1
U+8355.0	fMandarin	jin1
U+8356.0	fUTF8	荖
U+8356.0	fCNS11643	2-323D
U+8356.0	fBig5	D379
U+8356.0	fGB	3-6860
U+8356.0	fR/S	 140.6
U+8356.0	fTotalStrokes	 10; 12
U+8356.0	fCangjie	TJKP
U+8356.0	fFourCorner	44411
U+8356.0	fHakka	 lau3; lo3; leu1
U+8356.0	fCantonese	lou5; lau1
U+8356.0	fMandarin	lao3; pei2; cha1
U+8356.0	fEnglish	sirih leaf eaten with the betel-nut
U+8357.0	fUTF8	荗
U+8357.0	fCNS11643	3-335A
U+8357.0	fGB	E-3A4F
U+8357.0	fR/S	  140.6
U+8357.0	fTotalStrokes	 10; 12
U+8357.0	fCangjie	TIHI
U+8357.0	fFourCorner	44250
U+8357.0	fCantonese	seot6
U+8357.0	fMandarin	shu4
U+8358.0	fUTF8	荘
U+8358.0	fCNS11643	3-3363
U+8358.0	fGB	E-3A50
U+8358.0	fR/S	 140.6
U+8358.0	fTotalStrokes	 10; 12
U+8358.0	fCangjie	TLMG
U+8358.0	fFourCorner	44110
U+8358.0	fCantonese	zong1
U+8358.0	fMandarin	zhuang1; zhuan1
U+8358.0	fEnglish	 village; hamlet villa a Chinese family name
U+8359.0	fUTF8	荙
U+8359.0	fGB	7-2122
U+8359.0	fR/S	 140.6
U+8359.0	fTotalStrokes	 10; 12
U+8359.0	fCangjie	XTYK
U+8359.0	fCantonese	daat6
U+8359.0	fMandarin	da2
U+8359.0	fEnglish	plantago major
U+835A.0	fUTF8	荚
U+835A.0	fGB	0-3C54
U+835A.0	fR/S	 140.6
U+835A.0	fTotalStrokes	 10; 12
U+835A.0	fCangjie	TKT
U+835A.0	fHakka	 giap7
U+835A.0	fCantonese	gaap3
U+835A.0	fMandarin	jia2
U+835A.0	fEnglish	simplified form of &#x83A2;
U+835B.0	fUTF8	荛
U+835B.0	fGB	0-5C69
U+835B.0	fR/S	 140.6
U+835B.0	fTotalStrokes	 10; 12
U+835B.0	fCangjie	XTJPU
U+835B.0	fHakka	 ngiau2
U+835B.0	fCantonese	jiu4
U+835B.0	fMandarin	rao2; yao2; nao2; rao4
U+835B.0	fEnglish	simplified form of &#x8558;
U+835C.0	fUTF8	荜
U+835C.0	fGB	0-5C6A
U+835C.0	fR/S	 140.6
U+835C.0	fTotalStrokes	 10; 12
U+835C.0	fCangjie	TPPJ
U+835C.0	fHakka	 bit7
U+835C.0	fCantonese	bat1
U+835C.0	fMandarin	bi4
U+835C.0	fEnglish	simplified form of &#x84FD;
U+835D.0	fUTF8	荝
U+835D.0	fGB	8-7E3D
U+835D.0	fR/S	 140.6; 140.9
U+835D.0	fTotalStrokes	 10; 12
U+835D.0	fCangjie	TBON
U+835D.0	fCantonese	zak1
U+835D.0	fMandarin	ze2; ce4
U+835E.0	fUTF8	荞
U+835E.0	fGB	0-5C71
U+835E.0	fR/S	 140.6
U+835E.0	fTotalStrokes	 10; 12
U+835E.0	fCangjie	THKL
U+835E.0	fHakka	 kiau2
U+835E.0	fCantonese	kiu4
U+835E.0	fMandarin	qiao2; jiao1
U+835E.0	fEnglish	simplified form of &#x854E;
U+835F.0	fUTF8	荟
U+835F.0	fGB	0-5C76
U+835F.0	fR/S	 140.6
U+835F.0	fTotalStrokes	 10; 12
U+835F.0	fCangjie	TOMI
U+835F.0	fHakka	 ve5; fui5
U+835F.0	fCantonese	wai3; kui2; wui6 (also); kwui2 (also); wui3
U+835F.0	fMandarin	hui4; wei4; yi4
U+835F.0	fEnglish	simplified form of &#x8588;
U+8360.0	fUTF8	荠
U+8360.0	fGB	0-5C79
U+8360.0	fR/S	 140.6
U+8360.0	fTotalStrokes	 10; 12
U+8360.0	fCangjie	TYKL
U+8360.0	fHakka	 ci2; zi3
U+8360.0	fCantonese	cai5; cai4; zai3 (also); ci4
U+8360.0	fMandarin	ji4; qi2; ci2; shi4; qi5
U+8360.0	fEnglish	simplified form of &#x85BA;
U+8361.0	fUTF8	荡
U+8361.0	fGB	0-3534
U+8361.0	fR/S	 140.6; 140.7
U+8361.0	fTotalStrokes	 10; 12
U+8361.0	fCangjie	TENH
U+8361.0	fHakka	 tong3
U+8361.0	fCantonese	dong6
U+8361.0	fMandarin	dang4; tang1; tang4
U+8361.0	fEnglish	simplified form of &#x8569;
U+8362.0	fUTF8	荢
U+8362.0	fCNS11643	3-6636
U+8362.0	fGB	E-3A51
U+8362.0	fR/S	 140.6
U+8362.0	fTotalStrokes	 10; 12
U+8362.0	fCangjie	TJMD
U+8362.0	fFourCorner	4440
U+8362.0	fCantonese	zi6
U+8363.0	fUTF8	荣
U+8363.0	fCNS11643	3-2D33
U+8363.0	fGB	0-4859
U+8363.0	fR/S	 140.6
U+8363.0	fTotalStrokes	 10; 9; 12
U+8363.0	fCangjie	TBD
U+8363.0	fFourCorner	44900
U+8363.0	fHakka	 jin2; jung2
U+8363.0	fCantonese	wing4
U+8363.0	fMandarin	rong2; ying2
U+8363.0	fEnglish	simplified form of &#x69AE;
U+8364.0	fUTF8	荤
U+8364.0	fGB	0-3B67
U+8364.0	fR/S	 140.6
U+8364.0	fTotalStrokes	 10; 12
U+8364.0	fCangjie	TBKQ
U+8364.0	fHakka	 fun1
U+8364.0	fCantonese	fan1
U+8364.0	fMandarin	hun1; xun1
U+8364.0	fEnglish	simplified form of &#x8477;
U+8365.0	fUTF8	荥
U+8365.0	fGB	0-5C7E
U+8365.0	fR/S	 140.6
U+8365.0	fTotalStrokes	 10; 12
U+8365.0	fCangjie	TBE
U+8365.0	fHakka	 jin2
U+8365.0	fCantonese	jing4
U+8365.0	fMandarin	ying2; xing2; ying3; ying4
U+8365.0	fEnglish	simplified form of &#x6ECE;
U+8366.0	fUTF8	荦
U+8366.0	fGB	0-5C7D
U+8366.0	fR/S	 140.6
U+8366.0	fTotalStrokes	 10; 12
U+8366.0	fCangjie	TBHQ
U+8366.0	fHakka	 lok7
U+8366.0	fCantonese	lok3; lok6 (also)
U+8366.0	fMandarin	luo4
U+8366.0	fEnglish	simplified form of &#x7296;
U+8367.0	fUTF8	荧
U+8367.0	fGB	0-532B
U+8367.0	fR/S	 140.6
U+8367.0	fTotalStrokes	 10; 12
U+8367.0	fCangjie	TBF
U+8367.0	fHakka	 jin2
U+8367.0	fCantonese	jing4
U+8367.0	fMandarin	ying2; hong2; jyong3; xing1; ying3; ying4
U+8367.0	fEnglish	simplified form of &#x7192;
U+8368.0	fUTF8	荨
U+8368.0	fGB	0-5D21
U+8368.0	fR/S	 140.6
U+8368.0	fTotalStrokes	 10; 12
U+8368.0	fCangjie	XTSMI
U+8368.0	fHakka	 tam2
U+8368.0	fCantonese	cam4; taam4; cim4 (also); tam4
U+8368.0	fMandarin	xun2; qian2; tan2
U+8368.0	fEnglish	simplified form of &#x8541;
U+8369.0	fUTF8	荩
U+8369.0	fGB	0-5D23
U+8369.0	fR/S	 140.6
U+8369.0	fTotalStrokes	 10; 12
U+8369.0	fCangjie	TSOY
U+8369.0	fHakka	 zin5
U+8369.0	fCantonese	zeon2
U+8369.0	fMandarin	jin4
U+8369.0	fEnglish	simplified form of &#x85CE;
U+836A.0	fUTF8	荪
U+836A.0	fGB	0-5D25
U+836A.0	fR/S	 140.6
U+836A.0	fTotalStrokes	 10; 12
U+836A.0	fCangjie	XTNDF
U+836A.0	fHakka	 sun1
U+836A.0	fCantonese	syun1
U+836A.0	fMandarin	sun1
U+836A.0	fEnglish	simplified form of &#x84C0;
U+836B.0	fUTF8	荫
U+836B.0	fGB	0-5271
U+836B.0	fR/S	 140.6; 140.7
U+836B.0	fTotalStrokes	 11; 12
U+836B.0	fCangjie	TNLB
U+836B.0	fHakka	 jim5
U+836B.0	fCantonese	jam3; jam1
U+836B.0	fMandarin	yin4; yin1
U+836B.0	fEnglish	simplified form of &#x852D;
U+836C.0	fUTF8	荬
U+836C.0	fGB	0-5D24
U+836C.0	fR/S	 140.6
U+836C.0	fTotalStrokes	 10; 12
U+836C.0	fCangjie	TNYK
U+836C.0	fHakka	 mai3; mak7
U+836C.0	fCantonese	maai5
U+836C.0	fMandarin	mai3
U+836C.0	fEnglish	simplified form of &#x8552;
U+836D.0	fUTF8	荭
U+836D.0	fGB	0-5D26
U+836D.0	fR/S	 140.6
U+836D.0	fTotalStrokes	 10; 12
U+836D.0	fCangjie	TVMM
U+836D.0	fHakka	 fung2
U+836D.0	fCantonese	hung4
U+836D.0	fMandarin	hong2
U+836D.0	fEnglish	simplified form of &#x8452;
U+836E.0	fUTF8	荮
U+836E.0	fGB	0-5D27
U+836E.0	fR/S	 140.6
U+836E.0	fTotalStrokes	 10; 12
U+836E.0	fCangjie	XXXTV
U+836E.0	fCantonese	zau6
U+836E.0	fMandarin	zhou4
U+836E.0	fEnglish	simplified form of &#x8464;
U+836F.0	fUTF8	药
U+836F.0	fGB	0-5229
U+836F.0	fR/S	 140.6
U+836F.0	fTotalStrokes	 10; 12
U+836F.0	fCangjie	TVMI
U+836F.0	fHakka	 jok8
U+836F.0	fCantonese	joek6; joek2 (also); joek3; joek1; ngok6; ok6
U+836F.0	fMandarin	yao4; yue4; lun4; shuo4
U+836F.0	fEnglish	simplified form of &#x85E5;
U+8370.0	fUTF8	荰
U+8370.0	fCNS11643	4-3467
U+8370.0	fGB	E-3A52
U+8370.0	fR/S	  140.7
U+8370.0	fTotalStrokes	 11; 13
U+8370.0	fCangjie	TDG
U+8370.0	fFourCorner	44910
U+8370.0	fCantonese	dou6
U+8370.0	fMandarin	du4
U+8371.0	fUTF8	荱
U+8371.0	fCNS11643	3-384E
U+8371.0	fGB	3-6939
U+8371.0	fR/S	  140.7
U+8371.0	fTotalStrokes	 11; 13
U+8371.0	fCangjie	TSHU
U+8371.0	fFourCorner	44210
U+8371.0	fCantonese	mei5
U+8371.0	fMandarin	wei3
U+8372.0	fUTF8	荲
U+8372.0	fCNS11643	3-3851
U+8372.0	fGB	3-6928
U+8372.0	fR/S	  140.7
U+8372.0	fTotalStrokes	 11; 13
U+8372.0	fCangjie	TWG
U+8372.0	fFourCorner	44100
U+8372.0	fCantonese	huk1
U+8372.0	fMandarin	chu4
U+8373.0	fUTF8	荳
U+8373.0	fCNS11643	2-3862
U+8373.0	fBig5	D75E
U+8373.0	fGB	E-3A53
U+8373.0	fR/S	 140.7
U+8373.0	fTotalStrokes	 11; 13
U+8373.0	fCangjie	TMRT
U+8373.0	fFourCorner	44108
U+8373.0	fHakka	  teu5; teu6; tiu3
U+8373.0	fCantonese	dau6; dau2 (also)
U+8373.0	fMandarin	dou4
U+8373.0	fEnglish	 beans; peas; bean-shaped; legumes
U+8374.0	fUTF8	荴
U+8374.0	fCNS11643	2-3864
U+8374.0	fBig5	D760
U+8374.0	fGB	3-6924
U+8374.0	fR/S	 140.7
U+8374.0	fTotalStrokes	 11; 13
U+8374.0	fCangjie	TQQO
U+8374.0	fFourCorner	44535
U+8374.0	fCantonese	fu1
U+8374.0	fMandarin	fu1
U+8375.0	fUTF8	荵
U+8375.0	fCNS11643	2-3869
U+8375.0	fBig5	D765
U+8375.0	fGB	3-693A
U+8375.0	fR/S	 140.7
U+8375.0	fTotalStrokes	 11; 13
U+8375.0	fCangjie	TSIP
U+8375.0	fFourCorner	44332
U+8375.0	fCantonese	jan2
U+8375.0	fMandarin	ren3
U+8376.0	fUTF8	荶
U+8376.0	fCNS11643	2-387D
U+8376.0	fBig5	D779
U+8376.0	fGB	E-3A54
U+8376.0	fR/S	 140.7
U+8376.0	fTotalStrokes	 11; 13
U+8376.0	fCangjie	TRON
U+8376.0	fFourCorner	44627
U+8376.0	fCantonese	jam4
U+8376.0	fMandarin	yin2
U+8377.0	fUTF8	荷
U+8377.0	fCNS11643	1-5D23
U+8377.0	fBig5	B2FC
U+8377.0	fGB	0-3A49
U+8377.0	fR/S	 140.7
U+8377.0	fTotalStrokes	 11; 13
U+8377.0	fCangjie	TOMR
U+8377.0	fFourCorner	44221
U+8377.0	fHakka	  ho2; ho3; o3; o1
U+8377.0	fCantonese	ho4; ho6; ho2 (also)
U+8377.0	fMandarin	he2; ge1; ke1; he1
U+8377.0	fEnglish	[1] the lotus; water lily [2] Holland; the Netherlands; Dutch
U+8377.1	fUTF8	荷
U+8377.1	fCNS11643	1-5D23
U+8377.1	fBig5	B2FC
U+8377.1	fGB	0-3A49
U+8377.1	fR/S	 140.7
U+8377.1	fTotalStrokes	 11; 13
U+8377.1	fCangjie	TOMR
U+8377.1	fFourCorner	44221
U+8377.1	fMandarin	he4
U+8377.1	fEnglish	[1] load; burden [2] bear; carry; shoulder [3] receive
U+8378.0	fUTF8	荸
U+8378.0	fCNS11643	1-5C77
U+8378.0	fBig5	B2F2
U+8378.0	fGB	0-5D29
U+8378.0	fR/S	 140.7
U+8378.0	fTotalStrokes	 11; 13
U+8378.0	fCangjie	TJBD
U+8378.0	fFourCorner	44407
U+8378.0	fHakka	 put8
U+8378.0	fCantonese	but6
U+8378.0	fMandarin	bi2; bo2
U+8378.0	fEnglish	 water chestnut
U+8379.0	fUTF8	荹
U+8379.0	fCNS11643	3-3854
U+8379.0	fGB	3-6925
U+8379.0	fR/S	  140.7
U+8379.0	fTotalStrokes	 11; 13
U+8379.0	fCangjie	TYLH
U+8379.0	fFourCorner	44200
U+8379.0	fHakka	 pu2; pu5; pu6
U+8379.0	fCantonese	bou6
U+8379.0	fMandarin	bu4
U+8379.0	fEnglish	[1] fodder for cattle [2] chopped straw
U+837A.0	fUTF8	荺
U+837A.0	fCNS11643	2-3861
U+837A.0	fBig5	D75D
U+837A.0	fGB	3-687B
U+837A.0	fR/S	 140.7
U+837A.0	fTotalStrokes	 11; 13
U+837A.0	fCangjie	TGPM
U+837A.0	fFourCorner	44127
U+837A.0	fCantonese	wan5
U+837A.0	fMandarin	yun2; yun3
U+837B.0	fUTF8	荻
U+837B.0	fCNS11643	1-5D24
U+837B.0	fBig5	B2FD
U+837B.0	fGB	0-5D36
U+837B.0	fR/S	 140.7
U+837B.0	fTotalStrokes	 11; 13
U+837B.0	fCangjie	TKHF
U+837B.0	fFourCorner	44289
U+837B.0	fHakka	  tit8
U+837B.0	fCantonese	dik6
U+837B.0	fMandarin	di2
U+837B.0	fEnglish	[1] a kind of reed (Miscanthus sacchariflorus) [2] Anaphalis yedoensis
U+837C.0	fUTF8	荼
U+837C.0	fCNS11643	1-5D25
U+837C.0	fBig5	B2FE
U+837C.0	fGB	0-5D31
U+837C.0	fR/S	 140.7
U+837C.0	fTotalStrokes	 11; 13
U+837C.0	fCangjie	TOMD
U+837C.0	fFourCorner	44904
U+837C.0	fHakka	 tu2; tu3
U+837C.0	fCantonese	tou4; caa4; syu1; ce4
U+837C.0	fMandarin	tu2; shu1; ye2; cai4; cha2; hu4; she2; yu2
U+837C.0	fEnglish	[1] Sonchus oleraceus, a bitter-tasting vegetable; sow thistle [2] harm; poison
U+837D.0	fUTF8	荽
U+837D.0	fCNS11643	2-386C
U+837D.0	fBig5	D768
U+837D.0	fGB	0-5D34
U+837D.0	fR/S	 140.7
U+837D.0	fTotalStrokes	 11; 13
U+837D.0	fCangjie	TBV
U+837D.0	fFourCorner	44404
U+837D.0	fHakka	  sui1
U+837D.0	fCantonese	seoi1
U+837D.0	fMandarin	sui1; wei1; nei2; sui2; wei3
U+837D.0	fEnglish	used in combination
U+837E.0	fUTF8	荾
U+837E.0	fCNS11643	2-3873
U+837E.0	fBig5	D76F
U+837E.0	fGB	3-693C
U+837E.0	fR/S	 140.7
U+837E.0	fTotalStrokes	 11; 13
U+837E.0	fCangjie	TICE
U+837E.0	fFourCorner	44407
U+837E.0	fCantonese	seoi1
U+837E.0	fMandarin	sui1
U+837F.0	fUTF8	荿
U+837F.0	fCNS11643	2-3879
U+837F.0	fBig5	D775
U+837F.0	fGB	5-6774
U+837F.0	fR/S	 140.7; 140.6
U+837F.0	fTotalStrokes	 10; 13
U+837F.0	fCangjie	TIHS
U+837F.0	fFourCorner	44253
U+837F.0	fCantonese	sing4
U+837F.0	fMandarin	cheng2
U+8380.0	fUTF8	莀
U+8380.0	fCNS11643	3-3850
U+8380.0	fGB	5-682C
U+8380.0	fR/S	  140.7
U+8380.0	fTotalStrokes	 11; 13
U+8380.0	fCangjie	TMMV
U+8380.0	fFourCorner	44230
U+8380.0	fHakka	  nung2; shin2; siin2
U+8380.0	fCantonese	san4
U+8380.0	fMandarin	chen2
U+8380.0	fEnglish	 farm farmer agriculture
U+8381.0	fUTF8	莁
U+8381.0	fCNS11643	2-3866
U+8381.0	fBig5	D762
U+8381.0	fGB	5-6829
U+8381.0	fR/S	 140.7
U+8381.0	fTotalStrokes	 11; 13
U+8381.0	fCangjie	TMOO
U+8381.0	fFourCorner	44108
U+8381.0	fHakka	 mu2
U+8381.0	fCantonese	mou4
U+8381.0	fMandarin	wu2
U+8382.0	fUTF8	莂
U+8382.0	fCNS11643	4-346D
U+8382.0	fGB	3-692B
U+8382.0	fR/S	  140.7
U+8382.0	fTotalStrokes	 11; 13
U+8382.0	fCangjie	TRSN
U+8382.0	fFourCorner	44220
U+8382.0	fCantonese	bit6
U+8382.0	fMandarin	bie2
U+8383.0	fUTF8	莃
U+8383.0	fCNS11643	2-386D
U+8383.0	fBig5	D769
U+8383.0	fGB	5-683A
U+8383.0	fR/S	 140.7
U+8383.0	fTotalStrokes	 11; 13
U+8383.0	fCangjie	TKKB
U+8383.0	fFourCorner	44227
U+8383.0	fCantonese	hei1
U+8383.0	fMandarin	xi1
U+8384.0	fUTF8	莄
U+8384.0	fCNS11643	3-384F
U+8384.0	fGB	5-682A
U+8384.0	fR/S	  140.7
U+8384.0	fTotalStrokes	 11; 13
U+8384.0	fCangjie	TMLK
U+8384.0	fFourCorner	44500
U+8384.0	fCantonese	gang2
U+8384.0	fMandarin	geng3
U+8385.0	fUTF8	莅
U+8385.0	fCNS11643	3-385D
U+8385.0	fGB	0-5D30
U+8385.0	fR/S	 140.7
U+8385.0	fTotalStrokes	 11; 13
U+8385.0	fCangjie	TOYT
U+8385.0	fFourCorner	44210
U+8385.0	fHakka	  li5; li6; li3
U+8385.0	fCantonese	lei6
U+8385.0	fMandarin	li4
U+8385.0	fEnglish	 arrive; be present
U+8386.0	fUTF8	莆
U+8386.0	fCNS11643	1-5D26
U+8386.0	fBig5	B340
U+8386.0	fGB	0-464E
U+8386.0	fR/S	 140.7
U+8386.0	fTotalStrokes	 11; 13
U+8386.0	fCangjie	TIJB
U+8386.0	fFourCorner	44227
U+8386.0	fHakka	  pu2; fu3
U+8386.0	fCantonese	pou4; fu2
U+8386.0	fMandarin	fu3
U+8386.0	fEnglish	a kind of legendary tree with small leaves
U+8386.1	fUTF8	莆
U+8386.1	fCNS11643	1-5D26
U+8386.1	fBig5	B340
U+8386.1	fGB	0-464E
U+8386.1	fR/S	 140.7
U+8386.1	fTotalStrokes	 11; 13
U+8386.1	fCangjie	TIJB
U+8386.1	fFourCorner	44227
U+8386.1	fMandarin	pu2
U+8386.1	fEnglish	used in place name like Putian, a county in Fukien province
U+8387.0	fUTF8	莇
U+8387.0	fCNS11643	2-387B
U+8387.0	fBig5	D777
U+8387.0	fGB	5-6834
U+8387.0	fR/S	 140.7
U+8387.0	fTotalStrokes	 11; 13
U+8387.0	fCangjie	TBMS
U+8387.0	fFourCorner	44127
U+8387.0	fCantonese	zo4; zo6
U+8387.0	fMandarin	zhu4
U+8388.0	fUTF8	莈
U+8388.0	fCNS11643	2-3876
U+8388.0	fBig5	D772
U+8388.0	fGB	E-3A55
U+8388.0	fR/S	 140.7
U+8388.0	fTotalStrokes	 11; 13
U+8388.0	fCangjie	TENE
U+8388.0	fFourCorner	44347
U+8388.0	fCantonese	mut6
U+8388.0	fMandarin	mo4
U+8389.0	fUTF8	莉
U+8389.0	fCNS11643	1-5D21
U+8389.0	fBig5	B2FA
U+8389.0	fGB	0-4072
U+8389.0	fR/S	 140.7
U+8389.0	fTotalStrokes	 11; 13
U+8389.0	fCangjie	THDN
U+8389.0	fFourCorner	44920
U+8389.0	fHakka	 li5; li6; li3
U+8389.0	fCantonese	lei6; lei2 (also)
U+8389.0	fMandarin	li4; chi2; li2
U+8389.0	fEnglish	white jasmine
U+838A.0	fUTF8	莊
U+838A.0	fCNS11643	1-5C7D
U+838A.0	fBig5	B2F8
U+838A.0	fGB	1-572F
U+838A.0	fR/S	 140.7
U+838A.0	fTotalStrokes	 11; 13
U+838A.0	fCangjie	TVMG
U+838A.0	fFourCorner	44214
U+838A.0	fHakka	  zong1
U+838A.0	fCantonese	zong1
U+838A.0	fMandarin	zhuang1; zhuan1; zhuang4
U+838A.0	fEnglish	[1] solemn; dignified; stately; august; sober; gravity [2] a large farmhouse; manor house [3] village; hamlet [4] market; shop; store; bank [5] a Chinese family name
U+838B.0	fUTF8	莋
U+838B.0	fCNS11643	2-3872
U+838B.0	fBig5	D76E
U+838B.0	fGB	3-692D
U+838B.0	fR/S	 140.7
U+838B.0	fTotalStrokes	 11; 13
U+838B.0	fCangjie	TOOS
U+838B.0	fFourCorner	44211
U+838B.0	fCantonese	zok6
U+838B.0	fMandarin	ji2; zuo2
U+838C.0	fUTF8	莌
U+838C.0	fCNS11643	2-386E
U+838C.0	fBig5	D76A
U+838C.0	fGB	3-6936
U+838C.0	fR/S	 140.7
U+838C.0	fTotalStrokes	 11; 13
U+838C.0	fCangjie	TCRU
U+838C.0	fFourCorner	44216
U+838C.0	fHakka	 tot8; tot7
U+838C.0	fCantonese	tyut3
U+838C.0	fMandarin	duo2; tuo4; tuo1
U+838C.0	fEnglish	the rice-paper plant
U+838D.0	fUTF8	莍
U+838D.0	fCNS11643	2-3860
U+838D.0	fBig5	D75C
U+838D.0	fGB	5-6831
U+838D.0	fR/S	 140.7
U+838D.0	fTotalStrokes	 11; 13
U+838D.0	fCangjie	TIJE
U+838D.0	fFourCorner	44232
U+838D.0	fHakka	  kiu2
U+838D.0	fCantonese	kau4
U+838D.0	fMandarin	qiu2
U+838D.0	fEnglish	seeds of the wild pepper tree (Boymia)
U+838E.0	fUTF8	莎
U+838E.0	fCNS11643	1-5C74
U+838E.0	fBig5	B2EF
U+838E.0	fGB	0-492F
U+838E.0	fR/S	 140.7
U+838E.0	fTotalStrokes	 11; 13
U+838E.0	fCangjie	TEFH
U+838E.0	fFourCorner	44129
U+838E.0	fHakka	  sa1;  so1
U+838E.0	fCantonese	so1; saa1
U+838E.0	fMandarin	suo1; sui1; cha2
U+838E.0	fEnglish	Cyperus rotundus, a kind of medicinal herb
U+838E.1	fUTF8	莎
U+838E.1	fCNS11643	1-5C74
U+838E.1	fBig5	B2EF
U+838E.1	fGB	0-492F
U+838E.1	fR/S	 140.7
U+838E.1	fTotalStrokes	 11; 13
U+838E.1	fCangjie	TEFH
U+838E.1	fFourCorner	44129
U+838E.1	fMandarin	sha1
U+838E.1	fEnglish	a kind of insect
U+838F.0	fUTF8	莏
U+838F.0	fCNS11643	2-3865
U+838F.0	fBig5	D761
U+838F.0	fGB	5-682D
U+838F.0	fR/S	 140.7
U+838F.0	fTotalStrokes	 11; 13
U+838F.0	fCangjie	TQFH
U+838F.0	fFourCorner	44529
U+838F.0	fHakka	 so1
U+838F.0	fCantonese	co1; so1
U+838F.0	fMandarin	suo1; cuo1; sui1
U+838F.0	fEnglish	 rub hands together
U+8390.0	fUTF8	莐
U+8390.0	fCNS11643	2-385D
U+8390.0	fBig5	D759
U+8390.0	fGB	5-683F
U+8390.0	fR/S	 140.7
U+8390.0	fTotalStrokes	 11; 13
U+8390.0	fCangjie	TELU
U+8390.0	fFourCorner	44112
U+8390.0	fCantonese	cam4
U+8390.0	fMandarin	chen2; dan3; zhen4
U+8391.0	fUTF8	莑
U+8391.0	fCNS11643	3-3855
U+8391.0	fGB	5-683C
U+8391.0	fR/S	 140.7
U+8391.0	fTotalStrokes	 11; 13
U+8391.0	fCangjie	THEJ
U+8391.0	fFourCorner	44500
U+8391.0	fCantonese	pung4
U+8391.0	fMandarin	feng1
U+8392.0	fUTF8	莒
U+8392.0	fCNS11643	1-5C7C
U+8392.0	fBig5	B2F7
U+8392.0	fGB	0-5C6C
U+8392.0	fR/S	 140.7
U+8392.0	fTotalStrokes	 11; 13
U+8392.0	fCangjie	TRHR
U+8392.0	fFourCorner	44606
U+8392.0	fHakka	  gi3; li1; li5
U+8392.0	fCantonese	geoi2
U+8392.0	fMandarin	ju3
U+8392.0	fEnglish	[1] taros [2] name of an ancient state [3] name of a county in Shantung
U+8393.0	fUTF8	莓
U+8393.0	fCNS11643	1-5C7E
U+8393.0	fBig5	B2F9
U+8393.0	fGB	0-5D2E
U+8393.0	fR/S	 140.7
U+8393.0	fTotalStrokes	 11; 13
U+8393.0	fCangjie	TOWY
U+8393.0	fFourCorner	44507
U+8393.0	fHakka	  moi2; moe2; mui2
U+8393.0	fCantonese	mui4
U+8393.0	fMandarin	mei2; mei4
U+8393.0	fEnglish	[1] moss; lichen [2] edible berries
U+8394.0	fUTF8	莔
U+8394.0	fCNS11643	2-386A
U+8394.0	fBig5	D766
U+8394.0	fGB	3-692C
U+8394.0	fR/S	 140.7
U+8394.0	fTotalStrokes	 11; 13
U+8394.0	fCangjie	TWCB
U+8394.0	fFourCorner	44600
U+8394.0	fHakka	 mang2
U+8394.0	fCantonese	maang4
U+8394.0	fMandarin	meng2; xi4
U+8394.0	fEnglish	the name of a plant whose berries are used in fevers
U+8395.0	fUTF8	莕
U+8395.0	fCNS11643	2-3867
U+8395.0	fBig5	D763
U+8395.0	fGB	3-687E
U+8395.0	fR/S	 140.7
U+8395.0	fTotalStrokes	 11; 13
U+8395.0	fCangjie	TDR
U+8395.0	fFourCorner	44609
U+8395.0	fHakka	 hen5; hen6
U+8395.0	fCantonese	hang6
U+8395.0	fMandarin	xing4
U+8395.0	fEnglish	Nymphoides pellatum, a kind of water plant
U+8396.0	fUTF8	莖
U+8396.0	fCNS11643	1-5C79
U+8396.0	fBig5	B2F4
U+8396.0	fGB	1-3E25
U+8396.0	fR/S	 140.7
U+8396.0	fTotalStrokes	 11; 13
U+8396.0	fCangjie	TMVM
U+8396.0	fFourCorner	44101
U+8396.0	fHakka	 gin1;  kwang3; gang3; gwang1; gwang3
U+8396.0	fCantonese	ging3 (also); hang1; hang4; ging1
U+8396.0	fMandarin	jing1; ying1; xing2
U+8396.0	fEnglish	 stem; stalk
U+8397.0	fUTF8	莗
U+8397.0	fCNS11643	2-3877
U+8397.0	fBig5	D773
U+8397.0	fGB	E-3A56
U+8397.0	fR/S	 140.7
U+8397.0	fTotalStrokes	 11; 13
U+8397.0	fCangjie	TJWJ
U+8397.0	fFourCorner	44500
U+8397.0	fCantonese	ce1
U+8397.0	fMandarin	che1
U+8398.0	fUTF8	莘
U+8398.0	fCNS11643	1-5C76
U+8398.0	fBig5	B2F1
U+8398.0	fGB	0-5D37
U+8398.0	fR/S	 140.7
U+8398.0	fTotalStrokes	 11; 13
U+8398.0	fCangjie	TYTJ
U+8398.0	fFourCorner	44401
U+8398.0	fHakka	 sin1
U+8398.0	fCantonese	san1
U+8398.0	fMandarin	xin1; shen1
U+8398.0	fEnglish	[1] long [2] numerous [3] Asarum sieboldi, a marsh plant whose root is used for medicine [4] a Chinese family name
U+8399.0	fUTF8	莙
U+8399.0	fCNS11643	2-3868
U+8399.0	fBig5	D764
U+8399.0	fGB	3-6938
U+8399.0	fR/S	 140.7
U+8399.0	fTotalStrokes	 11; 13
U+8399.0	fCangjie	TSKR
U+8399.0	fFourCorner	44607
U+8399.0	fHakka	  kiun2; kiun3
U+8399.0	fCantonese	gwan1; kwan2; kwan5
U+8399.0	fMandarin	jun1; jun3; jun4
U+8399.0	fEnglish	species of water plant
U+839A.0	fUTF8	莚
U+839A.0	fCNS11643	2-3F6A
U+839A.0	fBig5	D77A
U+839A.0	fGB	5-677A
U+839A.0	fR/S	 140.7
U+839A.0	fTotalStrokes	 12; 13
U+839A.0	fCangjie	TNKM
U+839A.0	fFourCorner	44401
U+839A.0	fHakka	 jen2; ren2; jan2
U+839A.0	fCantonese	jin4
U+839A.0	fMandarin	yan2; yan4; wei3
U+839A.0	fEnglish	 bamboo mat feast; banquet
U+839B.0	fUTF8	莛
U+839B.0	fCNS11643	2-3870
U+839B.0	fBig5	D76C
U+839B.0	fGB	0-5C70
U+839B.0	fR/S	 140.7
U+839B.0	fTotalStrokes	 11; 13
U+839B.0	fCangjie	TNKG
U+839B.0	fFourCorner	44401
U+839B.0	fHakka	  tin2; tin1; tin3
U+839B.0	fCantonese	ting4
U+839B.0	fMandarin	ting2; ting3
U+839B.0	fEnglish	stalks of grasses; etc.
U+839C.0	fUTF8	莜
U+839C.0	fCNS11643	3-385E
U+839C.0	fGB	0-5D2F
U+839C.0	fR/S	 140.7
U+839C.0	fTotalStrokes	 11; 13
U+839C.0	fCangjie	TOLK
U+839C.0	fFourCorner	44240
U+839C.0	fHakka	 tiau5; tiau1; tiau2
U+839C.0	fCantonese	diu6; jau4; tiu4
U+839C.0	fMandarin	diao4; di2; you2
U+839C.0	fEnglish	avena nuda
U+839D.0	fUTF8	莝
U+839D.0	fCNS11643	2-386F
U+839D.0	fBig5	D76B
U+839D.0	fGB	3-692E
U+839D.0	fR/S	 140.7
U+839D.0	fTotalStrokes	 11; 13
U+839D.0	fCangjie	TOOG
U+839D.0	fFourCorner	44104
U+839D.0	fHakka	 co5
U+839D.0	fCantonese	co3; zo6
U+839D.0	fMandarin	cuo4
U+839D.0	fEnglish	[1] chop straw fine for animals [2] chopped straw or hay
U+839E.0	fUTF8	莞
U+839E.0	fCNS11643	1-5C75
U+839E.0	fBig5	B2F0
U+839E.0	fGB	0-5D38
U+839E.0	fR/S	 140.7
U+839E.0	fTotalStrokes	 11; 13
U+839E.0	fCangjie	TJMU
U+839E.0	fFourCorner	44211
U+839E.0	fHakka	  gon3;  wan3; gwon3; fan3; van3
U+839E.0	fCantonese	gun1; gun2; gwaan1 (also); gwun1 (also); wun2; wun5 (also); gwun2 (also); wun1
U+839E.0	fMandarin	wan3; guan4; huan2; huan3; wan1
U+839E.0	fEnglish	smiling
U+839E.1	fUTF8	莞
U+839E.1	fCNS11643	1-5C75
U+839E.1	fBig5	B2F0
U+839E.1	fGB	0-5D38
U+839E.1	fR/S	 140.7
U+839E.1	fTotalStrokes	 11; 13
U+839E.1	fCangjie	TJMU
U+839E.1	fFourCorner	44211
U+839E.1	fMandarin	guan3
U+839E.1	fEnglish	used in Dongguan, a county in Kwangtung province
U+839E.2	fUTF8	莞
U+839E.2	fCNS11643	1-5C75
U+839E.2	fBig5	B2F0
U+839E.2	fGB	0-5D38
U+839E.2	fR/S	 140.7
U+839E.2	fTotalStrokes	 11; 13
U+839E.2	fCangjie	TJMU
U+839E.2	fFourCorner	44211
U+839E.2	fMandarin	guan1
U+839E.2	fEnglish	Scriptus lacstris, a kind of aquatic herb
U+839F.0	fUTF8	莟
U+839F.0	fCNS11643	3-3857
U+839F.0	fGB	3-692F
U+839F.0	fR/S	  140.7
U+839F.0	fTotalStrokes	 11; 13
U+839F.0	fCangjie	TOIR
U+839F.0	fFourCorner	44600
U+839F.0	fCantonese	ham6
U+839F.0	fMandarin	han4
U+839F.0	fEnglish	a bud
U+83A0.0	fUTF8	莠
U+83A0.0	fCNS11643	1-5D22
U+83A0.0	fBig5	B2FB
U+83A0.0	fGB	0-5D2C
U+83A0.0	fR/S	 140.7
U+83A0.0	fTotalStrokes	 11; 13
U+83A0.0	fCangjie	THDS
U+83A0.0	fFourCorner	44227
U+83A0.0	fHakka	  siu5;  ju3; riu3; jiu5; jiu3
U+83A0.0	fCantonese	jau5
U+83A0.0	fMandarin	you3; xiu4; you4
U+83A0.0	fEnglish	[1] foxtail (a kind of weed) [2] undesirable; evil; bad; ugly; detestable; vicious
U+83A1.0	fUTF8	莡
U+83A1.0	fCNS11643	3-3852
U+83A1.0	fGB	3-6929
U+83A1.0	fR/S	  140.7
U+83A1.0	fTotalStrokes	 11; 13
U+83A1.0	fCangjie	TRYO
U+83A1.0	fFourCorner	44800
U+83A1.0	fCantonese	cok3
U+83A1.0	fMandarin	cuo4
U+83A2.0	fUTF8	莢
U+83A2.0	fCNS11643	1-5C78
U+83A2.0	fBig5	B2F3
U+83A2.0	fGB	1-3C54
U+83A2.0	fR/S	 140.7
U+83A2.0	fTotalStrokes	 11; 13
U+83A2.0	fCangjie	TKOO
U+83A2.0	fFourCorner	44908
U+83A2.0	fHakka	  giap7; gap7; giap8
U+83A2.0	fCantonese	gaap3
U+83A2.0	fMandarin	jia2
U+83A2.0	fEnglish	 pod; shells of seeds of leguminous plants; pods
U+83A3.0	fUTF8	莣
U+83A3.0	fCNS11643	2-385E
U+83A3.0	fBig5	D75A
U+83A3.0	fGB	3-6934
U+83A3.0	fR/S	 140.7
U+83A3.0	fTotalStrokes	 11; 13
U+83A3.0	fCangjie	TYVP
U+83A3.0	fFourCorner	44331
U+83A3.0	fCantonese	mong4
U+83A3.0	fMandarin	wang2; wang4
U+83A4.0	fUTF8	莤
U+83A4.0	fCNS11643	2-3863
U+83A4.0	fBig5	D75F
U+83A4.0	fGB	3-6923
U+83A4.0	fR/S	 140.7
U+83A4.0	fTotalStrokes	 11; 13
U+83A4.0	fCangjie	TMCW
U+83A4.0	fFourCorner	44601
U+83A4.0	fHakka	 ju2; riu2
U+83A4.0	fCantonese	suk1; jau4
U+83A4.0	fMandarin	you2; suo1; you3
U+83A4.0	fEnglish	name of a water-plant
U+83A5.0	fUTF8	莥
U+83A5.0	fCNS11643	2-3874
U+83A5.0	fBig5	D770
U+83A5.0	fGB	3-6931
U+83A5.0	fR/S	 140.7
U+83A5.0	fTotalStrokes	 11; 13
U+83A5.0	fCangjie	TKHG
U+83A5.0	fFourCorner	44215
U+83A5.0	fCantonese	nau5; lau5 (also)
U+83A5.0	fMandarin	niu3; chou3; jiu4
U+83A6.0	fUTF8	莦
U+83A6.0	fCNS11643	2-387A
U+83A6.0	fBig5	D776
U+83A6.0	fGB	3-6927
U+83A6.0	fR/S	 140.7
U+83A6.0	fTotalStrokes	 11; 13
U+83A6.0	fCangjie	TFB
U+83A6.0	fFourCorner	44227
U+83A6.0	fHakka	  sau1
U+83A6.0	fCantonese	saau1
U+83A6.0	fMandarin	shao1; qiong3; xiao1
U+83A6.0	fEnglish	[1] jungle grass [2] lair [3] burrow
U+83A7.0	fUTF8	莧
U+83A7.0	fCNS11643	1-5D27
U+83A7.0	fBig5	B341
U+83A7.0	fGB	1-5C48
U+83A7.0	fR/S	 140.7
U+83A7.0	fTotalStrokes	 11; 13
U+83A7.0	fCangjie	TBUU
U+83A7.0	fFourCorner	44216
U+83A7.0	fHakka	 han3;  han5; han6; hen3
U+83A7.0	fCantonese	jin6
U+83A7.0	fMandarin	xian4; huan2; huan3; wan4
U+83A7.0	fEnglish	 amaranth
U+83A8.0	fUTF8	莨
U+83A8.0	fCNS11643	2-385F
U+83A8.0	fBig5	D75B
U+83A8.0	fGB	0-5D39
U+83A8.0	fR/S	 140.7
U+83A8.0	fTotalStrokes	 11; 13
U+83A8.0	fCangjie	TIAV
U+83A8.0	fFourCorner	44732
U+83A8.0	fHakka	  long2;  liong2; long5
U+83A8.0	fCantonese	loeng4; long4 (also); long6
U+83A8.0	fMandarin	lang2; lang4; liang2
U+83A8.0	fEnglish	Scopolia japonica, a kind of herb
U+83A9.0	fUTF8	莩
U+83A9.0	fCNS11643	2-386B
U+83A9.0	fBig5	D767
U+83A9.0	fGB	0-5D33
U+83A9.0	fR/S	 140.7
U+83A9.0	fTotalStrokes	 11; 13
U+83A9.0	fCangjie	TBND
U+83A9.0	fFourCorner	44407
U+83A9.0	fHakka	  fu1; piau3
U+83A9.0	fCantonese	fu1; piu5
U+83A9.0	fMandarin	fu2; piao3; fu1
U+83A9.0	fEnglish	a membrane in stems of rushes or reeds
U+83AA.0	fUTF8	莪
U+83AA.0	fCNS11643	2-3871
U+83AA.0	fBig5	D76D
U+83AA.0	fGB	0-5D2D
U+83AA.0	fR/S	 140.7
U+83AA.0	fTotalStrokes	 11; 13
U+83AA.0	fCangjie	THQI
U+83AA.0	fFourCorner	44553
U+83AA.0	fHakka	 ngo2
U+83AA.0	fCantonese	ngo4; o4
U+83AA.0	fMandarin	e2
U+83AA.0	fEnglish	artemisia, a kind of plant; the aster-southernwood
U+83AB.0	fUTF8	莫
U+83AB.0	fCNS11643	1-5C7B
U+83AB.0	fBig5	B2F6
U+83AB.0	fGB	0-442A
U+83AB.0	fR/S	 140.7
U+83AB.0	fTotalStrokes	 11; 13
U+83AB.0	fCangjie	TAK
U+83AB.0	fFourCorner	44430
U+83AB.0	fHakka	  mok8; mu5
U+83AB.0	fCantonese	mok6; mou6; mok2 (also)
U+83AB.0	fMandarin	mo4; mi4
U+83AB.0	fEnglish	[1] not; do not; is not; can not [2] a Chinese family name ???
U+83AB.1	fUTF8	莫
U+83AB.1	fCNS11643	1-5C7B
U+83AB.1	fBig5	B2F6
U+83AB.1	fGB	0-442A
U+83AB.1	fR/S	 140.7
U+83AB.1	fTotalStrokes	 11; 13
U+83AB.1	fCangjie	TAK
U+83AB.1	fFourCorner	44430
U+83AB.1	fMandarin	mu4
U+83AB.1	fEnglish	evening; dusk ???
U+83AB.2	fUTF8	莫
U+83AB.2	fCNS11643	1-5C7B
U+83AB.2	fBig5	B2F6
U+83AB.2	fGB	0-442A
U+83AB.2	fR/S	 140.7
U+83AB.2	fTotalStrokes	 11; 13
U+83AB.2	fCangjie	TAK
U+83AB.2	fFourCorner	44430
U+83AC.0	fUTF8	莬
U+83AC.0	fCNS11643	4-346F
U+83AC.0	fGB	5-683B
U+83AC.0	fR/S	 140.7; 140.8
U+83AC.0	fTotalStrokes	 11; 13; 12
U+83AC.0	fCangjie	TNAU
U+83AC.0	fFourCorner	44410
U+83AC.0	fCantonese	man6
U+83AC.0	fMandarin	wen4; wan3; mian3
U+83AD.0	fUTF8	莭
U+83AD.0	fCNS11643	3-3858
U+83AD.0	fGB	E-3A57
U+83AD.0	fR/S	 140.9; 140.7
U+83AD.0	fTotalStrokes	 13; 11; 15
U+83AD.0	fCangjie	TAIL
U+83AD.0	fFourCorner	44720
U+83AD.0	fCantonese	zit3
U+83AD.0	fMandarin	jie2
U+83AE.0	fUTF8	莮
U+83AE.0	fCNS11643	2-387C
U+83AE.0	fBig5	D778
U+83AE.0	fGB	E-3A58
U+83AE.0	fR/S	 140.7
U+83AE.0	fTotalStrokes	 11; 13
U+83AE.0	fCangjie	TWKS
U+83AE.0	fFourCorner	44427
U+83AE.0	fCantonese	naam4; laam4 (also)
U+83AE.0	fMandarin	nan2
U+83AF.0	fUTF8	莯
U+83AF.0	fCNS11643	2-3875
U+83AF.0	fBig5	D771
U+83AF.0	fGB	E-3A59
U+83AF.0	fR/S	 140.7
U+83AF.0	fTotalStrokes	 11; 13
U+83AF.0	fCangjie	TED
U+83AF.0	fFourCorner	44394
U+83AF.0	fCantonese	ping4; muk6
U+83AF.0	fMandarin	mu4
U+83B0.0	fUTF8	莰
U+83B0.0	fCNS11643	2-3878
U+83B0.0	fBig5	D774
U+83B0.0	fGB	0-5D28
U+83B0.0	fR/S	 140.7
U+83B0.0	fTotalStrokes	 11; 13
U+83B0.0	fCangjie	TGNO
U+83B0.0	fFourCorner	44182
U+83B0.0	fCantonese	ham2; haam2
U+83B0.0	fMandarin	kan3
U+83B1.0	fUTF8	莱
U+83B1.0	fGB	0-4033
U+83B1.0	fR/S	 140.7
U+83B1.0	fTotalStrokes	 11; 13
U+83B1.0	fCangjie	TDT
U+83B1.0	fHakka	 loi2
U+83B1.0	fCantonese	loi4
U+83B1.0	fMandarin	lai2; lai4
U+83B1.0	fEnglish	simplified form of &#x840A;
U+83B2.0	fUTF8	莲
U+83B2.0	fGB	0-412B
U+83B2.0	fR/S	 140.8; 140.7
U+83B2.0	fTotalStrokes	 12; 13
U+83B2.0	fCangjie	TYKQ
U+83B2.0	fHakka	 lien2
U+83B2.0	fCantonese	lin4; lin2 (also)
U+83B2.0	fMandarin	lian2; lian3; lian4
U+83B2.0	fEnglish	simplified form of &#x84EE;
U+83B3.0	fUTF8	莳
U+83B3.0	fGB	0-5D2A
U+83B3.0	fR/S	 140.7
U+83B3.0	fTotalStrokes	 11; 13
U+83B3.0	fCangjie	TADI
U+83B3.0	fHakka	 shi5
U+83B3.0	fCantonese	si4; si2 (also); si6
U+83B3.0	fMandarin	shi4; shi2
U+83B3.0	fEnglish	simplified form of &#x8494;
U+83B4.0	fUTF8	莴
U+83B4.0	fGB	0-5D2B
U+83B4.0	fR/S	 140.7
U+83B4.0	fTotalStrokes	 11; 13
U+83B4.0	fCangjie	TROB
U+83B4.0	fHakka	 vo1
U+83B4.0	fCantonese	wo1
U+83B4.0	fMandarin	wo1
U+83B4.0	fEnglish	simplified form of &#x8435;
U+83B5.0	fUTF8	莵
U+83B5.0	fGB	E-3A5A
U+83B5.0	fR/S	 140.7
U+83B5.0	fTotalStrokes	 11; 13
U+83B5.0	fCangjie	XTHAI
U+83B5.0	fCantonese	tou3
U+83B5.0	fMandarin	tu2; tu4
U+83B5.0	fEnglish	 dodder; creeper
U+83B6.0	fUTF8	莶
U+83B6.0	fGB	0-5D32
U+83B6.0	fR/S	 140.7
U+83B6.0	fTotalStrokes	 11; 13
U+83B6.0	fCangjie	XTOMM
U+83B6.0	fHakka	 liam2; liam3; liam5
U+83B6.0	fCantonese	cim1; lim5; lim6 (also)
U+83B6.0	fMandarin	xian1; lian3; lian2; lian4; nan4; yan2
U+83B6.0	fEnglish	simplified form of &#x859F;
U+83B7.0	fUTF8	获
U+83B7.0	fCNS11643	3-385F
U+83B7.0	fGB	0-3B71
U+83B7.0	fR/S	 140.7
U+83B7.0	fTotalStrokes	 11; 13
U+83B7.0	fCangjie	TKHK
U+83B7.0	fFourCorner	44234
U+83B7.0	fHakka	 wok7; fet8; vet7
U+83B7.0	fCantonese	wok6; wai4
U+83B7.0	fMandarin	huo4; hu4; hua4
U+83B7.0	fEnglish	simplified form of &#x7372;
U+83B8.0	fUTF8	莸
U+83B8.0	fGB	0-5D35
U+83B8.0	fR/S	 140.7
U+83B8.0	fTotalStrokes	 11; 13
U+83B8.0	fCangjie	TKHU
U+83B8.0	fHakka	 ju2
U+83B8.0	fCantonese	jau4
U+83B8.0	fMandarin	you2; you3
U+83B8.0	fEnglish	simplified form of &#x8555;
U+83B9.0	fUTF8	莹
U+83B9.0	fCNS11643	F-3778
U+83B9.0	fGB	0-5328
U+83B9.0	fR/S	 140.7
U+83B9.0	fTotalStrokes	 11; 13
U+83B9.0	fCangjie	TBMI
U+83B9.0	fFourCorner	4410
U+83B9.0	fHakka	 jin2
U+83B9.0	fCantonese	jing4; jing2 (also)
U+83B9.0	fMandarin	ying2; ying3; ying4
U+83B9.0	fEnglish	simplified form of &#x7469;
U+83BA.0	fUTF8	莺
U+83BA.0	fGB	0-5D3A
U+83BA.0	fR/S	 140.7
U+83BA.0	fTotalStrokes	 11; 13
U+83BA.0	fCangjie	TBPM
U+83BA.0	fHakka	 jin2
U+83BA.0	fCantonese	ang1; ngang1 (also)
U+83BA.0	fMandarin	ying1
U+83BA.0	fEnglish	simplified form of &#x9DAF;
U+83BB.0	fUTF8	莻
U+83BB.0	fGB	1-7D2D
U+83BB.0	fR/S	 140.7; 21.9
U+83BB.0	fTotalStrokes	 13
U+83BB.0	fCangjie	TNSP
U+83BB.0	fCantonese	jing4
U+83BB.0	fEnglish	 late
U+83BC.0	fUTF8	莼
U+83BC.0	fGB	0-5D3B
U+83BC.0	fR/S	 140.7
U+83BC.0	fTotalStrokes	 11; 13
U+83BC.0	fCangjie	TVMU
U+83BC.0	fHakka	 shun2
U+83BC.0	fCantonese	seon4
U+83BC.0	fMandarin	chun2
U+83BC.0	fEnglish	simplified form of &#x8493;
U+83BD.0	fUTF8	莽
U+83BD.0	fCNS11643	1-5C7A
U+83BD.0	fBig5	B2F5
U+83BD.0	fGB	0-4327
U+83BD.0	fR/S	 140.7; 140.8; 140.6
U+83BD.0	fTotalStrokes	 10; 12
U+83BD.0	fCangjie	TIKT
U+83BD.0	fFourCorner	44443
U+83BD.0	fHakka	 mong3;  mang3
U+83BD.0	fCantonese	mong5; mong6
U+83BD.0	fMandarin	mang3; mang2
U+83BD.0	fEnglish	[1] bushy; weedy [2] Illicim anisatum, a poisonous bushy plant [3] rude; uncultured; impolite; reckless
U+83BE.0	fUTF8	莾
U+83BE.0	fCNS11643	4-3A68
U+83BE.0	fGB	E-3A5B
U+83BE.0	fR/S	 140.8
U+83BE.0	fTotalStrokes	 12; 14
U+83BE.0	fCangjie	TKJT
U+83BE.0	fFourCorner	44440
U+83BE.0	fCantonese	mong5
U+83BE.0	fMandarin	mang3
U+83BF.0	fUTF8	莿
U+83BF.0	fCNS11643	2-3F51
U+83BF.0	fBig5	DB6C
U+83BF.0	fGB	5-6845
U+83BF.0	fR/S	 140.8
U+83BF.0	fTotalStrokes	 12; 14
U+83BF.0	fCangjie	TDBN
U+83BF.0	fFourCorner	44920
U+83BF.0	fHakka	 cih5; ze5
U+83BF.0	fCantonese	ci3; caak3
U+83BF.0	fMandarin	ci4
U+83BF.0	fEnglish	thorn
U+83C0.0	fUTF8	菀
U+83C0.0	fCNS11643	2-3F45
U+83C0.0	fBig5	DB60
U+83C0.0	fGB	0-5D52
U+83C0.0	fR/S	 140.8
U+83C0.0	fTotalStrokes	 12; 14
U+83C0.0	fCangjie	TJNU
U+83C0.0	fFourCorner	44212
U+83C0.0	fHakka	 won3;  ngut7; wan3; von3; van3
U+83C0.0	fCantonese	jyun2; wat1; wan5; wan3 (also); wan2
U+83C0.0	fMandarin	wan3; yu4; yuan4; yun3; yun4
U+83C0.0	fEnglish	[1] luxuriant; exuberant; lush [2] clogged; stagnant
U+83C1.0	fUTF8	菁
U+83C1.0	fCNS11643	1-615D
U+83C1.0	fBig5	B5D7
U+83C1.0	fGB	0-5D3C
U+83C1.0	fR/S	 140.8
U+83C1.0	fTotalStrokes	 12; 14
U+83C1.0	fCangjie	TQMB
U+83C1.0	fFourCorner	44227
U+83C1.0	fHakka	  zin1;  ziang1; cang1; ciang1
U+83C1.0	fCantonese	zing1; cing1 (also)
U+83C1.0	fMandarin	jing1; qing1
U+83C1.0	fEnglish	[1] the flower of the leek family [2] rape turnip [3] luxuriant; lush
U+83C2.0	fUTF8	菂
U+83C2.0	fCNS11643	2-3F62
U+83C2.0	fBig5	DB7D
U+83C2.0	fGB	3-6953
U+83C2.0	fR/S	 140.8
U+83C2.0	fTotalStrokes	 12; 14
U+83C2.0	fCangjie	THAI
U+83C2.0	fFourCorner	44627
U+83C2.0	fHakka	  dit7
U+83C2.0	fCantonese	dik1
U+83C2.0	fMandarin	di1; di4; hao4
U+83C2.0	fEnglish	the seeds of the lotus plant
U+83C3.0	fUTF8	菃
U+83C3.0	fCNS11643	2-3F6B
U+83C3.0	fBig5	DBA7
U+83C3.0	fGB	E-3A5C
U+83C3.0	fR/S	 140.8
U+83C3.0	fTotalStrokes	 12; 14
U+83C3.0	fCangjie	TESS
U+83C3.0	fFourCorner	44117
U+83C3.0	fCantonese	keoi4
U+83C3.0	fMandarin	qu2
U+83C4.0	fUTF8	菄
U+83C4.0	fCNS11643	2-3F6E
U+83C4.0	fBig5	DBAA
U+83C4.0	fGB	5-675A
U+83C4.0	fR/S	 140.8
U+83C4.0	fTotalStrokes	 12; 14
U+83C4.0	fCangjie	TDW
U+83C4.0	fFourCorner	44906
U+83C4.0	fHakka	 go3
U+83C4.0	fCantonese	dung1
U+83C4.0	fMandarin	dong1; dong4
U+83C5.0	fUTF8	菅
U+83C5.0	fCNS11643	1-615B
U+83C5.0	fBig5	B5D5
U+83C5.0	fGB	0-5D51
U+83C5.0	fR/S	 140.8
U+83C5.0	fTotalStrokes	 12; 14
U+83C5.0	fCangjie	TJRR
U+83C5.0	fFourCorner	44777
U+83C5.0	fHakka	 gan1; gon1; gien1; gen1; gian1; gwon1
U+83C5.0	fCantonese	gaan1
U+83C5.0	fMandarin	jian1; guan1
U+83C5.0	fEnglish	Themedia triandra, a coarse grass (used for making brushes, brooms, etc.)
U+83C6.0	fUTF8	菆
U+83C6.0	fCNS11643	2-3F4D
U+83C6.0	fBig5	DB68
U+83C6.0	fGB	3-6941
U+83C6.0	fR/S	 140.8
U+83C6.0	fTotalStrokes	 12; 14
U+83C6.0	fCangjie	TSJE
U+83C6.0	fFourCorner	44147
U+83C6.0	fHakka	  zeu1; ci1
U+83C6.0	fCantonese	zau1; cyun4
U+83C6.0	fMandarin	zou1; chu4; cong2; cuan2
U+83C6.0	fEnglish	jungle
U+83C7.0	fUTF8	菇
U+83C7.0	fCNS11643	2-3F66
U+83C7.0	fBig5	DBA3
U+83C7.0	fGB	0-393D
U+83C7.0	fR/S	 140.8
U+83C7.0	fTotalStrokes	 12; 14
U+83C7.0	fCangjie	TVJR
U+83C7.0	fFourCorner	44464
U+83C7.0	fHakka	 gu1
U+83C7.0	fCantonese	gu1; gwu1 (also)
U+83C7.0	fMandarin	gu1
U+83C7.0	fEnglish	 mushrooms; fungus
U+83C8.0	fUTF8	菈
U+83C8.0	fCNS11643	2-3F4E
U+83C8.0	fBig5	DB69
U+83C8.0	fGB	3-6947
U+83C8.0	fR/S	 140.8
U+83C8.0	fTotalStrokes	 12; 14
U+83C8.0	fCangjie	TQYT
U+83C8.0	fFourCorner	44518
U+83C8.0	fCantonese	laap6
U+83C8.0	fMandarin	la1
U+83C9.0	fUTF8	菉
U+83C9.0	fCNS11643	2-3F5C
U+83C9.0	fBig5	DB77
U+83C9.0	fGB	E-3A5D
U+83C9.0	fR/S	 140.8
U+83C9.0	fTotalStrokes	 12; 14
U+83C9.0	fCangjie	TVNE
U+83C9.0	fFourCorner	44232
U+83C9.0	fHakka	 liuk8; luk8
U+83C9.0	fCantonese	luk6
U+83C9.0	fMandarin	lu4; luu4
U+83C9.0	fEnglish	[1] a kind of grass [2] green
U+83CA.0	fUTF8	菊
U+83CA.0	fCNS11643	1-6168
U+83CA.0	fBig5	B5E2
U+83CA.0	fGB	0-3E55
U+83CA.0	fR/S	 140.8
U+83CA.0	fTotalStrokes	 12; 14
U+83CA.0	fCangjie	TPFD
U+83CA.0	fFourCorner	44927
U+83CA.0	fHakka	  kiuk7
U+83CA.0	fCantonese	guk1
U+83CA.0	fMandarin	ju2
U+83CA.0	fEnglish	the Chrysanthemum
U+83CB.0	fUTF8	菋
U+83CB.0	fCNS11643	2-3F58
U+83CB.0	fBig5	DB73
U+83CB.0	fGB	3-694E
U+83CB.0	fR/S	 140.8
U+83CB.0	fTotalStrokes	 12; 14
U+83CB.0	fCangjie	TRJD
U+83CB.0	fFourCorner	44695
U+83CB.0	fHakka	 mui6; mi5; mui5
U+83CB.0	fCantonese	mei6
U+83CB.0	fMandarin	wei4; mei4
U+83CB.0	fEnglish	a plant whose seeds are used as a tonic (Schizandra chinensis)
U+83CC.0	fUTF8	菌
U+83CC.0	fCNS11643	1-6165
U+83CC.0	fBig5	B5DF
U+83CC.0	fGB	0-3E7A
U+83CC.0	fR/S	 140.8
U+83CC.0	fTotalStrokes	 12; 14
U+83CC.0	fCangjie	TWHD
U+83CC.0	fFourCorner	44600
U+83CC.0	fHakka	  kiun2;  kiun3; kun2; kun5
U+83CC.0	fCantonese	kwan2; kwan5
U+83CC.0	fMandarin	jun1; juan4; jun4; qun1
U+83CC.0	fEnglish	[1] fungi; mushroom [2] bacteria; germ; microbe [3] bamboo shoot
U+83CD.0	fUTF8	菍
U+83CD.0	fCNS11643	3-3E5E
U+83CD.0	fGB	5-6851
U+83CD.0	fR/S	  140.8
U+83CD.0	fTotalStrokes	 12; 14
U+83CD.0	fCangjie	TOIP
U+83CD.0	fFourCorner	44330
U+83CD.0	fCantonese	nip6; lip6 (also)
U+83CD.0	fMandarin	nie4
U+83CE.0	fUTF8	菎
U+83CE.0	fCNS11643	2-3F59
U+83CE.0	fBig5	DB74
U+83CE.0	fGB	5-684B
U+83CE.0	fR/S	 140.8
U+83CE.0	fTotalStrokes	 12; 14
U+83CE.0	fCangjie	TAPP
U+83CE.0	fFourCorner	44711
U+83CE.0	fCantonese	kwan1
U+83CE.0	fMandarin	kun1; gun3; jun4
U+83CE.0	fEnglish	 beautiful jade bamboo
U+83CF.0	fUTF8	菏
U+83CF.0	fCNS11643	2-3F42
U+83CF.0	fBig5	DB5D
U+83CF.0	fGB	0-3A4A
U+83CF.0	fR/S	 140.8
U+83CF.0	fTotalStrokes	 12; 14
U+83CF.0	fCangjie	TEMR
U+83CF.0	fFourCorner	44121
U+83CF.0	fCantonese	go1; ho4 (also); ho1
U+83CF.0	fMandarin	he2; ge1
U+83CF.0	fEnglish	a tributary of the Chi river in ancient times
U+83D0.0	fUTF8	菐
U+83D0.0	fCNS11643	5-3B6F
U+83D0.0	fGB	E-3A5E
U+83D0.0	fR/S	 140.8
U+83D0.0	fTotalStrokes	 12; 14
U+83D0.0	fCangjie	TCTO
U+83D0.0	fFourCorner	23530
U+83D0.0	fCantonese	duk6
U+83D0.0	fMandarin	pu2
U+83D0.0	fEnglish	a thicket
U+83D1.0	fUTF8	菑
U+83D1.0	fCNS11643	2-3F67
U+83D1.0	fBig5	DBA4
U+83D1.0	fGB	8-2E74
U+83D1.0	fR/S	 140.8
U+83D1.0	fTotalStrokes	 12; 14
U+83D1.0	fCangjie	TVVW
U+83D1.0	fFourCorner	44602
U+83D1.0	fHakka	 zih1;  zai1; zu1; zii1
U+83D1.0	fCantonese	zi1; ji1 (also); zoi1
U+83D1.0	fMandarin	zi1; zi4
U+83D1.0	fEnglish	[1] weed grass [2] land under cultivation
U+83D1.1	fUTF8	菑
U+83D1.1	fCNS11643	2-3F67
U+83D1.1	fBig5	DBA4
U+83D1.1	fGB	8-2E74
U+83D1.1	fR/S	 140.8
U+83D1.1	fTotalStrokes	 12; 14
U+83D1.1	fCangjie	TVVW
U+83D1.1	fFourCorner	44602
U+83D1.1	fMandarin	zai1
U+83D1.1	fEnglish	misfortune; disaster; calamity
U+83D2.0	fUTF8	菒
U+83D2.0	fCNS11643	3-3E4F
U+83D2.0	fGB	3-694F
U+83D2.0	fR/S	  140.8
U+83D2.0	fTotalStrokes	 12; 14
U+83D2.0	fCangjie	TAD
U+83D2.0	fFourCorner	44900
U+83D2.0	fCantonese	gou2
U+83D2.0	fMandarin	gao3
U+83D3.0	fUTF8	菓
U+83D3.0	fCNS11643	3-3E59
U+83D3.0	fGB	E-3A5F
U+83D3.0	fR/S	 140.8
U+83D3.0	fTotalStrokes	 12; 14
U+83D3.0	fCangjie	TWD
U+83D3.0	fFourCorner	44900
U+83D3.0	fHakka	  go3; gwo3; kwo3
U+83D3.0	fCantonese	gwo2; go2 (also)
U+83D3.0	fMandarin	guo3
U+83D3.0	fEnglish	fruits and nuts
U+83D4.0	fUTF8	菔
U+83D4.0	fCNS11643	1-616E
U+83D4.0	fBig5	B5E8
U+83D4.0	fGB	0-5D4A
U+83D4.0	fR/S	 140.8
U+83D4.0	fTotalStrokes	 12; 14
U+83D4.0	fCangjie	TBSE
U+83D4.0	fFourCorner	44247
U+83D4.0	fHakka	  pet8; fuk8
U+83D4.0	fCantonese	fuk6; baak6 (also)
U+83D4.0	fMandarin	fu2; bo2
U+83D4.0	fEnglish	 turnip (botany)
U+83D5.0	fUTF8	菕
U+83D5.0	fCNS11643	2-3F64
U+83D5.0	fBig5	DBA1
U+83D5.0	fGB	3-6840
U+83D5.0	fR/S	 140.8
U+83D5.0	fTotalStrokes	 12; 14
U+83D5.0	fCangjie	TOMB
U+83D5.0	fFourCorner	44227
U+83D5.0	fHakka	 lun2
U+83D5.0	fCantonese	leon4
U+83D5.0	fMandarin	lun2; lun4
U+83D6.0	fUTF8	菖
U+83D6.0	fCNS11643	2-3F5A
U+83D6.0	fBig5	DB75
U+83D6.0	fGB	0-5D45
U+83D6.0	fR/S	 140.8
U+83D6.0	fTotalStrokes	 12; 14
U+83D6.0	fCangjie	TAA
U+83D6.0	fFourCorner	44606
U+83D6.0	fHakka	 cong1;  chong1
U+83D6.0	fCantonese	coeng1
U+83D6.0	fMandarin	chang1
U+83D6.0	fEnglish	 sweet flag; calamus
U+83D7.0	fUTF8	菗
U+83D7.0	fCNS11643	2-3F70
U+83D7.0	fBig5	DBAC
U+83D7.0	fGB	5-6846
U+83D7.0	fR/S	 140.8
U+83D7.0	fTotalStrokes	 12; 14
U+83D7.0	fCangjie	TQLW
U+83D7.0	fFourCorner	44560
U+83D7.0	fHakka	 chiu1; cu1; chiu2
U+83D7.0	fCantonese	cau4
U+83D7.0	fMandarin	chou2
U+83D7.0	fEnglish	a medicinal plant with a bitter root
U+83D8.0	fUTF8	菘
U+83D8.0	fCNS11643	2-3F55
U+83D8.0	fBig5	DB70
U+83D8.0	fGB	0-5D3F
U+83D8.0	fR/S	 140.8
U+83D8.0	fTotalStrokes	 12; 14
U+83D8.0	fCangjie	TDCI
U+83D8.0	fFourCorner	44932
U+83D8.0	fHakka	 sung1; siung1
U+83D8.0	fCantonese	sung1
U+83D8.0	fMandarin	song1; gong1
U+83D8.0	fEnglish	celery; cabbage
U+83D9.0	fUTF8	菙
U+83D9.0	fCNS11643	2-466D
U+83D9.0	fBig5	DFC8
U+83D9.0	fGB	5-684E
U+83D9.0	fR/S	 140.8
U+83D9.0	fTotalStrokes	 13; 14
U+83D9.0	fCangjie	THJM
U+83D9.0	fFourCorner	44104
U+83D9.0	fCantonese	ceoi4
U+83D9.0	fMandarin	chui2; zhui4
U+83D9.0	fEnglish	 bramble; thorn
U+83DA.0	fUTF8	菚
U+83DA.0	fCNS11643	3-3E4D
U+83DA.0	fGB	5-6758
U+83DA.0	fR/S	  140.8
U+83DA.0	fTotalStrokes	 12; 14
U+83DA.0	fCangjie	TII
U+83DA.0	fFourCorner	44500
U+83DA.0	fCantonese	zaan6
U+83DA.0	fMandarin	zhan4
U+83DB.0	fUTF8	菛
U+83DB.0	fCNS11643	2-3F73
U+83DB.0	fBig5	DBAF
U+83DB.0	fGB	5-6744
U+83DB.0	fR/S	 140.8
U+83DB.0	fTotalStrokes	 12; 14
U+83DB.0	fCangjie	TAN
U+83DB.0	fFourCorner	44227
U+83DB.0	fHakka	 mun2
U+83DB.0	fCantonese	mun4
U+83DB.0	fMandarin	men2
U+83DC.0	fUTF8	菜
U+83DC.0	fCNS11643	1-616C
U+83DC.0	fBig5	B5E6
U+83DC.0	fGB	0-324B
U+83DC.0	fR/S	 140.8
U+83DC.0	fTotalStrokes	 12; 14
U+83DC.0	fCangjie	TBD
U+83DC.0	fFourCorner	44904
U+83DC.0	fHakka	  coi5
U+83DC.0	fCantonese	coi3; coi2 (also)
U+83DC.0	fMandarin	cai4
U+83DC.0	fEnglish	[1] vegetables; greens [2] food eaten with rice or alcoholic drinks [3] dish; course
U+83DD.0	fUTF8	菝
U+83DD.0	fCNS11643	2-3F53
U+83DD.0	fBig5	DB6E
U+83DD.0	fGB	0-5D43
U+83DD.0	fR/S	 140.8
U+83DD.0	fTotalStrokes	 12; 14
U+83DD.0	fCangjie	TQIK
U+83DD.0	fFourCorner	44547
U+83DD.0	fHakka	  pat8
U+83DD.0	fCantonese	bat6; but6
U+83DD.0	fMandarin	ba2; bei4; bi4
U+83DD.0	fEnglish	Smilax China
U+83DE.0	fUTF8	菞
U+83DE.0	fCNS11643	2-3F5F
U+83DE.0	fBig5	DB7A
U+83DE.0	fGB	3-6952
U+83DE.0	fR/S	 140.8
U+83DE.0	fTotalStrokes	 12; 14
U+83DE.0	fCangjie	THDH
U+83DE.0	fFourCorner	44927
U+83DE.0	fCantonese	lai4
U+83DE.0	fMandarin	li2
U+83DF.0	fUTF8	菟
U+83DF.0	fCNS11643	1-616F
U+83DF.0	fBig5	B5E9
U+83DF.0	fGB	0-5D4B
U+83DF.0	fR/S	 140.8
U+83DF.0	fTotalStrokes	 12; 14
U+83DF.0	fCangjie	TNAI
U+83DF.0	fFourCorner	44413
U+83DF.0	fHakka	 tu5; tu2; tu3
U+83DF.0	fCantonese	tou3; tou4
U+83DF.0	fMandarin	tu4; tu2
U+83DF.0	fEnglish	 dodder (botany)
U+83E0.0	fUTF8	菠
U+83E0.0	fCNS11643	1-615A
U+83E0.0	fBig5	B5D4
U+83E0.0	fGB	0-3224
U+83E0.0	fR/S	 140.8
U+83E0.0	fTotalStrokes	 12; 14
U+83E0.0	fCangjie	TEDE
U+83E0.0	fFourCorner	44147
U+83E0.0	fHakka	  bo1
U+83E0.0	fCantonese	bo1
U+83E0.0	fMandarin	bo1; bo2
U+83E0.0	fEnglish	spinach
U+83E1.0	fUTF8	菡
U+83E1.0	fCNS11643	2-3F57
U+83E1.0	fBig5	DB72
U+83E1.0	fGB	0-5D55
U+83E1.0	fR/S	 140.8
U+83E1.0	fTotalStrokes	 12; 14
U+83E1.0	fCangjie	TUNE
U+83E1.0	fFourCorner	44772
U+83E1.0	fHakka	 ham3
U+83E1.0	fCantonese	haam5; aam2
U+83E1.0	fMandarin	han4
U+83E1.0	fEnglish	another name of water lily or lotus flower
U+83E2.0	fUTF8	菢
U+83E2.0	fCNS11643	2-3F71
U+83E2.0	fBig5	DBAD
U+83E2.0	fGB	3-6946
U+83E2.0	fR/S	 140.8
U+83E2.0	fTotalStrokes	 12; 14
U+83E2.0	fCangjie	TQPU
U+83E2.0	fFourCorner	44112
U+83E2.0	fHakka	  pau5; pu3; {pu5}
U+83E2.0	fCantonese	bou6
U+83E2.0	fMandarin	bao4
U+83E2.0	fEnglish	 incubate; brood; hatch (eggs)
U+83E3.0	fUTF8	菣
U+83E3.0	fCNS11643	2-3F50
U+83E3.0	fBig5	DB6B
U+83E3.0	fGB	E-3A60
U+83E3.0	fR/S	 140.8
U+83E3.0	fTotalStrokes	 12; 14
U+83E3.0	fCangjie	TSLE
U+83E3.0	fFourCorner	44747
U+83E3.0	fHakka	 kien5
U+83E3.0	fCantonese	jan3; kin3
U+83E3.0	fMandarin	qin4; qian4
U+83E4.0	fUTF8	菤
U+83E4.0	fCNS11643	2-3F49
U+83E4.0	fBig5	DB64
U+83E4.0	fGB	5-6856
U+83E4.0	fR/S	 140.8
U+83E4.0	fTotalStrokes	 12; 14
U+83E4.0	fCangjie	TFQU
U+83E4.0	fFourCorner	44712
U+83E4.0	fHakka	 kien1; gien3; gen3; gian3
U+83E4.0	fCantonese	gyun2; kyun2
U+83E4.0	fMandarin	juan3; juan4
U+83E4.0	fEnglish	mouse-ear
U+83E5.0	fUTF8	菥
U+83E5.0	fCNS11643	2-3F54
U+83E5.0	fBig5	DB6F
U+83E5.0	fGB	0-5D3E
U+83E5.0	fR/S	 140.8
U+83E5.0	fTotalStrokes	 12; 14
U+83E5.0	fCangjie	TDHL
U+83E5.0	fFourCorner	44921
U+83E5.0	fHakka	  sit7
U+83E5.0	fCantonese	sik1
U+83E5.0	fMandarin	xi1; si1
U+83E5.0	fEnglish	[1] a kind of vegetable, eaten as greens [2] Shepherd's purse
U+83E6.0	fUTF8	菦
U+83E6.0	fCNS11643	3-3E5F
U+83E6.0	fGB	E-3A61
U+83E6.0	fR/S	  140.8
U+83E6.0	fTotalStrokes	 12; 14
U+83E6.0	fCangjie	TYHL
U+83E6.0	fFourCorner	44300
U+83E6.0	fCantonese	kan4
U+83E6.0	fMandarin	qin2
U+83E7.0	fUTF8	菧
U+83E7.0	fCNS11643	2-3F48
U+83E7.0	fBig5	DB63
U+83E7.0	fGB	3-6958
U+83E7.0	fR/S	 140.8
U+83E7.0	fTotalStrokes	 12; 14
U+83E7.0	fCangjie	TIHM
U+83E7.0	fFourCorner	44240
U+83E7.0	fHakka	 di3
U+83E7.0	fCantonese	dai2
U+83E7.0	fMandarin	di3
U+83E8.0	fUTF8	菨
U+83E8.0	fCNS11643	2-3F46
U+83E8.0	fBig5	DB61
U+83E8.0	fGB	5-6855
U+83E8.0	fR/S	 140.8
U+83E8.0	fTotalStrokes	 12; 14
U+83E8.0	fCangjie	TYTV
U+83E8.0	fFourCorner	44404
U+83E8.0	fCantonese	zip3; cip3
U+83E8.0	fMandarin	jie1; sha4
U+83E9.0	fUTF8	菩
U+83E9.0	fCNS11643	1-6156
U+83E9.0	fBig5	B5D0
U+83E9.0	fGB	0-4650
U+83E9.0	fR/S	 140.8
U+83E9.0	fTotalStrokes	 12; 14
U+83E9.0	fCangjie	TYTR
U+83E9.0	fFourCorner	44601
U+83E9.0	fHakka	  pu2; pu3
U+83E9.0	fCantonese	pou4
U+83E9.0	fMandarin	pu2; bei4; bo2; bu4; fu4
U+83E9.0	fEnglish	a fragrant herb
U+83EA.0	fUTF8	菪
U+83EA.0	fCNS11643	2-3F68
U+83EA.0	fBig5	DBA5
U+83EA.0	fGB	0-5D50
U+83EA.0	fR/S	 140.8
U+83EA.0	fTotalStrokes	 12; 14
U+83EA.0	fCangjie	TJMR
U+83EA.0	fFourCorner	44601
U+83EA.0	fHakka	 tong3; tong5
U+83EA.0	fCantonese	dong6
U+83EA.0	fMandarin	dang4
U+83EA.0	fEnglish	name of a poisonous plant
U+83EB.0	fUTF8	菫
U+83EB.0	fCNS11643	2-3F4F
U+83EB.0	fBig5	DB6A
U+83EB.0	fGB	E-3A62
U+83EB.0	fR/S	 140.8
U+83EB.0	fTotalStrokes	 12; 14
U+83EB.0	fCangjie	TMLM
U+83EB.0	fFourCorner	44104
U+83EB.0	fHakka	  giun3
U+83EB.0	fCantonese	gan2; gan3
U+83EB.0	fMandarin	jin3; jin4; qin2
U+83EB.0	fEnglish	Viola verecunda, a violet
U+83EC.0	fUTF8	菬
U+83EC.0	fCNS11643	2-3F6C
U+83EC.0	fBig5	DBA8
U+83EC.0	fGB	5-685A
U+83EC.0	fR/S	 140.8
U+83EC.0	fTotalStrokes	 12; 14
U+83EC.0	fCangjie	TESR
U+83EC.0	fFourCorner	44162
U+83EC.0	fCantonese	ciu4; ziu2
U+83EC.0	fMandarin	zhao3; qiao2
U+83ED.0	fUTF8	菭
U+83ED.0	fCNS11643	3-3E4B
U+83ED.0	fGB	3-695B
U+83ED.0	fR/S	 140.8
U+83ED.0	fTotalStrokes	 12; 14
U+83ED.0	fCangjie	TEIR
U+83ED.0	fFourCorner	44160
U+83ED.0	fCantonese	toi4
U+83ED.0	fMandarin	tai2
U+83EE.0	fUTF8	菮
U+83EE.0	fCNS11643	2-3F6D
U+83EE.0	fBig5	DBA9
U+83EE.0	fGB	E-3A63
U+83EE.0	fR/S	 140.8
U+83EE.0	fTotalStrokes	 12; 14
U+83EE.0	fCangjie	TILO
U+83EE.0	fFourCorner	44234
U+83EE.0	fCantonese	gang1
U+83EE.0	fMandarin	geng1
U+83EF.0	fUTF8	華
U+83EF.0	fCNS11643	1-615E
U+83EF.0	fBig5	B5D8
U+83EF.0	fGB	1-3B2A
U+83EF.0	fR/S	 140.8
U+83EF.0	fTotalStrokes	 12; 14
U+83EF.0	fCangjie	TMTJ
U+83EF.0	fFourCorner	44504
U+83EF.0	fHakka	  fa2; fa1;  fa5; fa3
U+83EF.0	fCantonese	waa6; faa1; waa4; waa2 (also)
U+83EF.0	fMandarin	hua2; kua1
U+83EF.0	fEnglish	[1] Cathay; China [2] splendid; majestic; gorgeous; colorful; brilliant; bright; fine; beautiful; luxurious [3] prosperous; thriving
U+83EF.1	fUTF8	華
U+83EF.1	fCNS11643	1-615E
U+83EF.1	fBig5	B5D8
U+83EF.1	fGB	1-3B2A
U+83EF.1	fR/S	 140.8
U+83EF.1	fTotalStrokes	 12; 14
U+83EF.1	fCangjie	TMTJ
U+83EF.1	fFourCorner	44504
U+83EF.1	fMandarin	hua1
U+83EF.1	fEnglish	flowers
U+83EF.2	fUTF8	華
U+83EF.2	fCNS11643	1-615E
U+83EF.2	fBig5	B5D8
U+83EF.2	fGB	1-3B2A
U+83EF.2	fR/S	 140.8
U+83EF.2	fTotalStrokes	 12; 14
U+83EF.2	fCangjie	TMTJ
U+83EF.2	fFourCorner	44504
U+83EF.2	fMandarin	hua4
U+83EF.2	fEnglish	[1] luster; brilliancy; glory; splendor [2] a Chinese family name
U+83F0.0	fUTF8	菰
U+83F0.0	fCNS11643	1-6163
U+83F0.0	fBig5	B5DD
U+83F0.0	fGB	0-5D54
U+83F0.0	fR/S	 140.8
U+83F0.0	fTotalStrokes	 12; 14
U+83F0.0	fCangjie	TNDO
U+83F0.0	fFourCorner	44432
U+83F0.0	fHakka	  gu1
U+83F0.0	fCantonese	gu1; gwu1 (also)
U+83F0.0	fMandarin	gu1
U+83F0.0	fEnglish	Zizania latifolia, an aquatic grass, the stalk of which is eaten as vegetable
U+83F1.0	fUTF8	菱
U+83F1.0	fCNS11643	1-615F
U+83F1.0	fBig5	B5D9
U+83F1.0	fGB	0-4162
U+83F1.0	fR/S	 140.8
U+83F1.0	fTotalStrokes	 12; 14
U+83F1.0	fCangjie	TGCE
U+83F1.0	fFourCorner	44407
U+83F1.0	fHakka	  lin2; lin3
U+83F1.0	fCantonese	ling4
U+83F1.0	fMandarin	ling2
U+83F1.0	fEnglish	 water-chestnut
U+83F2.0	fUTF8	菲
U+83F2.0	fCNS11643	1-6167
U+83F2.0	fBig5	B5E1
U+83F2.0	fGB	0-3746
U+83F2.0	fR/S	 140.8
U+83F2.0	fTotalStrokes	 12; 14
U+83F2.0	fCangjie	TLMY
U+83F2.0	fFourCorner	44111
U+83F2.0	fHakka	  fui1; fi1; fui3; fi3
U+83F2.0	fCantonese	fei2; fei1
U+83F2.0	fMandarin	fei3; fei4
U+83F2.0	fEnglish	[1] a kind of radish, and edible vegetable [2] thin; trifling; meager [3] frugal; sparing
U+83F2.1	fUTF8	菲
U+83F2.1	fCNS11643	1-6167
U+83F2.1	fBig5	B5E1
U+83F2.1	fGB	0-3746
U+83F2.1	fR/S	 140.8
U+83F2.1	fTotalStrokes	 12; 14
U+83F2.1	fCangjie	TLMY
U+83F2.1	fFourCorner	44111
U+83F2.1	fMandarin	fei1
U+83F2.1	fEnglish	[1] fragrant [2] luxuriant [3] the Philippines
U+83F3.0	fUTF8	菳
U+83F3.0	fCNS11643	2-3F63
U+83F3.0	fBig5	DB7E
U+83F3.0	fGB	3-6955
U+83F3.0	fR/S	 140.8
U+83F3.0	fTotalStrokes	 12; 14
U+83F3.0	fCangjie	TC
U+83F3.0	fFourCorner	44109
U+83F3.0	fCantonese	kan4; gam1; kam1; kam4
U+83F3.0	fMandarin	jin1; bei1; qin1; qin2
U+83F4.0	fUTF8	菴
U+83F4.0	fCNS11643	1-6160
U+83F4.0	fBig5	B5DA
U+83F4.0	fGB	E-3A64
U+83F4.0	fR/S	 140.8
U+83F4.0	fTotalStrokes	 12; 14
U+83F4.0	fCangjie	TKLU
U+83F4.0	fFourCorner	44716
U+83F4.0	fHakka	  am1
U+83F4.0	fCantonese	am1; ngam1 (also)
U+83F4.0	fMandarin	an1; yan1; an3; yan3
U+83F4.0	fEnglish	[1] small round hut [2] small Buddhist monastery or nunnery; temple
U+83F4.1	fUTF8	菴
U+83F4.1	fCNS11643	1-6160
U+83F4.1	fBig5	B5DA
U+83F4.1	fGB	E-3A64
U+83F4.1	fR/S	 140.8
U+83F4.1	fTotalStrokes	 12; 14
U+83F4.1	fCangjie	TKLU
U+83F4.1	fFourCorner	44716
U+83F4.1	fMandarin	an4
U+83F4.1	fEnglish	luxuriant; prosperous
U+83F5.0	fUTF8	菵
U+83F5.0	fCNS11643	2-3F5B
U+83F5.0	fBig5	DB76
U+83F5.0	fGB	3-6950
U+83F5.0	fR/S	 140.8
U+83F5.0	fTotalStrokes	 12; 14
U+83F5.0	fCangjie	TBTV
U+83F5.0	fFourCorner	44227
U+83F5.0	fHakka	 miong3
U+83F5.0	fCantonese	mong5
U+83F5.0	fMandarin	wang3
U+83F6.0	fUTF8	菶
U+83F6.0	fCNS11643	2-3F4B
U+83F6.0	fBig5	DB66
U+83F6.0	fGB	3-693F
U+83F6.0	fR/S	 140.8
U+83F6.0	fTotalStrokes	 12; 14
U+83F6.0	fCangjie	TQKQ
U+83F6.0	fFourCorner	44503
U+83F6.0	fHakka	 fung3; bung3; bung1
U+83F6.0	fCantonese	bung2
U+83F6.0	fMandarin	beng3; peng2
U+83F6.0	fEnglish	 a species of water plant flower
U+83F7.0	fUTF8	菷
U+83F7.0	fCNS11643	F-3E66
U+83F7.0	fGB	5-685F
U+83F7.0	fR/S	 140.8
U+83F7.0	fTotalStrokes	 12; 14
U+83F7.0	fCangjie	TSMB
U+83F7.0	fCantonese	zau2; zaau2 (also)
U+83F7.0	fMandarin	zhou3
U+83F8.0	fUTF8	菸
U+83F8.0	fCNS11643	1-6158
U+83F8.0	fBig5	B5D2
U+83F8.0	fGB	0-5D4E
U+83F8.0	fR/S	 140.8
U+83F8.0	fTotalStrokes	 12; 14
U+83F8.0	fCangjie	TYSY
U+83F8.0	fFourCorner	44233
U+83F8.0	fHakka	 jen1; ji1; ren1; ri1; jan1; ji2
U+83F8.0	fCantonese	jin1
U+83F8.0	fMandarin	yan1; yu4; yu1; yu2
U+83F8.0	fEnglish	 tobacco leaf
U+83F9.0	fUTF8	菹
U+83F9.0	fCNS11643	2-3F43
U+83F9.0	fBig5	DB5E
U+83F9.0	fGB	0-5D4F
U+83F9.0	fR/S	 140.8
U+83F9.0	fTotalStrokes	 12; 14
U+83F9.0	fCangjie	TEBM
U+83F9.0	fFourCorner	44117
U+83F9.0	fHakka	 zu1; zii1; zih1
U+83F9.0	fCantonese	zeoi1
U+83F9.0	fMandarin	zu1; ju1; ju4; jie1
U+83F9.0	fEnglish	[1] salted or pickled vegetables [2] a pond or lake with a lot of weeds; grassy marshes [3] mince human flesh and bones (a form of capital punishment)
U+83FA.0	fUTF8	菺
U+83FA.0	fCNS11643	2-3F65
U+83FA.0	fBig5	DBA2
U+83FA.0	fGB	3-695D
U+83FA.0	fR/S	 140.8
U+83FA.0	fTotalStrokes	 12; 14
U+83FA.0	fCangjie	THSB
U+83FA.0	fFourCorner	44227
U+83FA.0	fCantonese	gin1
U+83FA.0	fMandarin	jian1
U+83FB.0	fUTF8	菻
U+83FB.0	fCNS11643	2-3F6F
U+83FB.0	fBig5	DBAB
U+83FB.0	fGB	3-6942
U+83FB.0	fR/S	 140.8
U+83FB.0	fTotalStrokes	 12; 14
U+83FB.0	fCangjie	TDD
U+83FB.0	fFourCorner	44994
U+83FB.0	fHakka	 lim3; ma2
U+83FB.0	fCantonese	lam4 (also); lam5; lam1
U+83FB.0	fMandarin	lin3; ma2
U+83FB.0	fEnglish	[1] artemisia [2] bud; flower bud
U+83FC.0	fUTF8	菼
U+83FC.0	fCNS11643	2-3F4A
U+83FC.0	fBig5	DB65
U+83FC.0	fGB	3-695A
U+83FC.0	fR/S	 140.8
U+83FC.0	fTotalStrokes	 12; 14
U+83FC.0	fCangjie	TFF
U+83FC.0	fFourCorner	44809
U+83FC.0	fHakka	 tam2; jam2; ram2
U+83FC.0	fCantonese	taam2; cim1
U+83FC.0	fMandarin	tan3
U+83FC.0	fEnglish	rush or sedge
U+83FD.0	fUTF8	菽
U+83FD.0	fCNS11643	1-6166
U+83FD.0	fBig5	B5E0
U+83FD.0	fGB	0-5D44
U+83FD.0	fR/S	 140.8
U+83FD.0	fTotalStrokes	 12; 14
U+83FD.0	fCangjie	TYFE
U+83FD.0	fFourCorner	44947
U+83FD.0	fHakka	 shuk7; suk7; siuk7
U+83FD.0	fCantonese	suk6; suk1
U+83FD.0	fMandarin	shu1; shu2; jiao1
U+83FD.0	fEnglish	beans and peas collectively; a general term for beans, peas, etc.
U+83FE.0	fUTF8	菾
U+83FE.0	fCNS11643	2-3F74
U+83FE.0	fBig5	DBB0
U+83FE.0	fGB	3-6940
U+83FE.0	fR/S	 140.8
U+83FE.0	fTotalStrokes	 12; 14
U+83FE.0	fCangjie	THKP
U+83FE.0	fFourCorner	44333
U+83FE.0	fHakka	  tiam2; tiam3
U+83FE.0	fCantonese	tim4
U+83FE.0	fMandarin	tian2; tian4
U+83FE.0	fEnglish	beet
U+83FF.0	fUTF8	菿
U+83FF.0	fCNS11643	2-3F56
U+83FF.0	fBig5	DB71
U+83FF.0	fGB	3-694A
U+83FF.0	fR/S	 140.8
U+83FF.0	fTotalStrokes	 12; 14
U+83FF.0	fCangjie	TMGN
U+83FF.0	fFourCorner	44120
U+83FF.0	fCantonese	dou3; doek3; dou2
U+83FF.0	fMandarin	dao4; dao3; zhao4; zhuo2
U+83FF.0	fEnglish	tall grass
U+8400.0	fUTF8	萀
U+8400.0	fCNS11643	3-3E52
U+8400.0	fGB	3-694C
U+8400.0	fR/S	  140.8
U+8400.0	fTotalStrokes	 12; 14
U+8400.0	fCangjie	TYPU
U+8400.0	fFourCorner	44210
U+8400.0	fCantonese	fu2
U+8400.0	fMandarin	hu3
U+8401.0	fUTF8	萁
U+8401.0	fCNS11643	2-3F52
U+8401.0	fBig5	DB6D
U+8401.0	fGB	0-5D3D
U+8401.0	fR/S	 140.8
U+8401.0	fTotalStrokes	 12; 14
U+8401.0	fCangjie	TTMC
U+8401.0	fFourCorner	44801
U+8401.0	fHakka	 gi1; ki2
U+8401.0	fCantonese	kei4; gei1
U+8401.0	fMandarin	qi2; ji1; gai1
U+8401.0	fEnglish	the stalks of beans
U+8402.0	fUTF8	萂
U+8402.0	fCNS11643	3-3E5C
U+8402.0	fGB	5-6850
U+8402.0	fR/S	  140.8
U+8402.0	fTotalStrokes	 12; 14
U+8402.0	fCangjie	THDR
U+8402.0	fFourCorner	44960
U+8402.0	fCantonese	wo4
U+8402.0	fMandarin	he2
U+8403.0	fUTF8	萃
U+8403.0	fCNS11643	1-6157
U+8403.0	fBig5	B5D1
U+8403.0	fGB	0-5D4D
U+8403.0	fR/S	 140.8
U+8403.0	fTotalStrokes	 12; 14
U+8403.0	fCangjie	TYOJ
U+8403.0	fFourCorner	44408
U+8403.0	fHakka	 cui5
U+8403.0	fCantonese	seoi6
U+8403.0	fMandarin	cui4; zu2; zui4
U+8403.0	fEnglish	[1] a dense, thick growth of grass [2] group; set [3] collect; gather; meet; congregate
U+8404.0	fUTF8	萄
U+8404.0	fCNS11643	1-616B
U+8404.0	fBig5	B5E5
U+8404.0	fGB	0-4C51
U+8404.0	fR/S	 140.8
U+8404.0	fTotalStrokes	 12; 14
U+8404.0	fCangjie	TPOU
U+8404.0	fFourCorner	44727
U+8404.0	fHakka	  tau2; to2; to3
U+8404.0	fCantonese	tou4; tou2 (also)
U+8404.0	fMandarin	tao2
U+8404.0	fEnglish	used in combination for grapes
U+8405.0	fUTF8	萅
U+8405.0	fCNS11643	3-3E5A
U+8405.0	fGB	5-6849
U+8405.0	fR/S	 140.8
U+8405.0	fTotalStrokes	 12; 14
U+8405.0	fCangjie	TPUA
U+8405.0	fFourCorner	44600
U+8405.0	fCantonese	ceon1
U+8405.0	fMandarin	chun1; chun3
U+8405.0	fEnglish	 spring wanton
U+8406.0	fUTF8	萆
U+8406.0	fCNS11643	2-3F61
U+8406.0	fBig5	DB7C
U+8406.0	fGB	0-5D49
U+8406.0	fR/S	 140.8
U+8406.0	fTotalStrokes	 12; 14
U+8406.0	fCangjie	THWJ
U+8406.0	fFourCorner	44406
U+8406.0	fHakka	  bi1; bi5
U+8406.0	fCantonese	bai3; bei1
U+8406.0	fMandarin	bei1; bi4; pi4; ba2; bi3
U+8406.0	fEnglish	a herb with long hard roots
U+8407.0	fUTF8	萇
U+8407.0	fCNS11643	1-616D
U+8407.0	fBig5	B5E7
U+8407.0	fGB	1-5C49
U+8407.0	fR/S	 140.8
U+8407.0	fTotalStrokes	 12; 14
U+8407.0	fCangjie	TSMV
U+8407.0	fFourCorner	44732
U+8407.0	fHakka	 chong2; cong2
U+8407.0	fCantonese	coeng4
U+8407.0	fMandarin	chang2
U+8407.0	fEnglish	the fruit of Averrhora carambola
U+8408.0	fUTF8	萈
U+8408.0	fCNS11643	3-3E50
U+8408.0	fGB	3-6850
U+8408.0	fR/S	  140.8
U+8408.0	fTotalStrokes	 12; 14
U+8408.0	fCangjie	TBUI
U+8408.0	fFourCorner	44210
U+8408.0	fCantonese	wun4
U+8408.0	fMandarin	huan2
U+8409.0	fUTF8	萉
U+8409.0	fCNS11643	2-3F5D
U+8409.0	fBig5	DB78
U+8409.0	fGB	5-6853
U+8409.0	fR/S	 140.8
U+8409.0	fTotalStrokes	 12; 14
U+8409.0	fCangjie	TBAU
U+8409.0	fFourCorner	44217
U+8409.0	fCantonese	fai6; fei4
U+8409.0	fMandarin	fei2; bo2; fei4; fen2; fu2
U+840A.0	fUTF8	萊
U+840A.0	fCNS11643	1-6162
U+840A.0	fBig5	B5DC
U+840A.0	fGB	1-4033
U+840A.0	fR/S	 140.8
U+840A.0	fTotalStrokes	 12; 14
U+840A.0	fCangjie	TDOO
U+840A.0	fFourCorner	44908
U+840A.0	fHakka	 loi2;  loi3
U+840A.0	fCantonese	loi4
U+840A.0	fMandarin	lai2; lai4
U+840A.0	fEnglish	[1] field lying fallow [2] wild weeds [3] a Chinese family name
U+840B.0	fUTF8	萋
U+840B.0	fCNS11643	1-615C
U+840B.0	fBig5	B5D6
U+840B.0	fGB	0-5D42
U+840B.0	fR/S	 140.8
U+840B.0	fTotalStrokes	 12; 14
U+840B.0	fCangjie	TJLV
U+840B.0	fFourCorner	44404
U+840B.0	fHakka	 ci1; zi1
U+840B.0	fCantonese	cai1
U+840B.0	fMandarin	qi1; ci3
U+840B.0	fEnglish	[1] luxuriant foliage; a dense growth of grass [2] many; crowded
U+840C.0	fUTF8	萌
U+840C.0	fCNS11643	1-6164
U+840C.0	fBig5	B5DE
U+840C.0	fGB	0-4348
U+840C.0	fR/S	 140.8
U+840C.0	fTotalStrokes	 12; 14
U+840C.0	fCangjie	TAB
U+840C.0	fFourCorner	44627
U+840C.0	fHakka	 men2; min2
U+840C.0	fCantonese	mang4; maang4
U+840C.0	fMandarin	meng2; ming2
U+840C.0	fEnglish	[1] bud; sprout; germinate; shoot forth [2] harbour (thought) [3] beginning; initiation; initial
U+840D.0	fUTF8	萍
U+840D.0	fCNS11643	1-6159
U+840D.0	fBig5	B5D3
U+840D.0	fGB	0-463C
U+840D.0	fR/S	 140.8
U+840D.0	fTotalStrokes	 12; 14
U+840D.0	fCangjie	TEMJ
U+840D.0	fFourCorner	44149
U+840D.0	fHakka	  pin2
U+840D.0	fCantonese	ping4
U+840D.0	fMandarin	ping2
U+840D.0	fEnglish	[1] duckweed [2] wandering; traveling; moving about rootlessly
U+840E.0	fUTF8	萎
U+840E.0	fCNS11643	1-616A
U+840E.0	fBig5	B5E4
U+840E.0	fGB	0-4E2E
U+840E.0	fR/S	 140.8
U+840E.0	fTotalStrokes	 12; 14
U+840E.0	fCangjie	THDV
U+840E.0	fFourCorner	44404
U+840E.0	fHakka	 wui1; wi1; vui3; wi3; wui3
U+840E.0	fCantonese	wai2; wai1; wai3
U+840E.0	fMandarin	wei1; wei3; wei4
U+840E.0	fEnglish	[1] wither; wilt [2] ill; sick [3] fall; decline; weaken
U+840F.0	fUTF8	萏
U+840F.0	fCNS11643	2-3F5E
U+840F.0	fBig5	DB79
U+840F.0	fGB	0-5D4C
U+840F.0	fR/S	 140.8
U+840F.0	fTotalStrokes	 12; 14
U+840F.0	fCangjie	TNHX
U+840F.0	fFourCorner	44777
U+840F.0	fHakka	  tam3
U+840F.0	fCantonese	daam6
U+840F.0	fMandarin	dan4
U+840F.0	fEnglish	used in @ (another name of water lily)
U+8410.0	fUTF8	萐
U+8410.0	fCNS11643	2-3F4C
U+8410.0	fBig5	DB67
U+8410.0	fGB	3-6945
U+8410.0	fR/S	 140.8
U+8410.0	fTotalStrokes	 12; 14
U+8410.0	fCangjie	TJLO
U+8410.0	fFourCorner	44801
U+8410.0	fCantonese	saap3; sip3; zit6
U+8410.0	fMandarin	sha4
U+8410.0	fEnglish	lucky plant
U+8411.0	fUTF8	萑
U+8411.0	fCNS11643	2-3F60
U+8411.0	fBig5	DB7B
U+8411.0	fGB	0-5D48
U+8411.0	fR/S	 140.8
U+8411.0	fTotalStrokes	 12; 14
U+8411.0	fCangjie	TOG
U+8411.0	fFourCorner	44214
U+8411.0	fHakka	 fan2; wan2; van2
U+8411.0	fCantonese	wun4; teoi1
U+8411.0	fMandarin	huan2; zhui1; tui1
U+8411.0	fEnglish	 grass used for making mats Huanpu; place famous for robbers
U+8412.0	fUTF8	萒
U+8412.0	fCNS11643	2-3F47
U+8412.0	fBig5	DB62
U+8412.0	fGB	3-6959
U+8412.0	fR/S	 140.8
U+8412.0	fTotalStrokes	 12; 14
U+8412.0	fCangjie	TYCU
U+8412.0	fFourCorner	44213
U+8412.0	fCantonese	jin5; gyun6
U+8412.0	fMandarin	yan3; juan4
U+8413.0	fUTF8	萓
U+8413.0	fCNS11643	2-3F69
U+8413.0	fBig5	DBA6
U+8413.0	fGB	5-685C
U+8413.0	fR/S	 140.8
U+8413.0	fTotalStrokes	 12; 14
U+8413.0	fCangjie	TJBM
U+8413.0	fFourCorner	44107
U+8413.0	fCantonese	ji4
U+8413.0	fMandarin	yi2
U+8414.0	fUTF8	萔
U+8414.0	fCNS11643	3-3E58
U+8414.0	fGB	5-6848
U+8414.0	fR/S	 140.8
U+8414.0	fTotalStrokes	 12; 14
U+8414.0	fCangjie	TQSR
U+8414.0	fFourCorner	44560
U+8414.0	fCantonese	tiu4
U+8414.0	fMandarin	tiao2
U+8415.0	fUTF8	萕
U+8415.0	fCNS11643	4-3A67
U+8415.0	fGB	E-3A65
U+8415.0	fR/S	 140.8
U+8415.0	fTotalStrokes	 12; 14
U+8415.0	fCangjie	TYKL
U+8415.0	fFourCorner	44220
U+8415.0	fCantonese	cai5
U+8415.0	fMandarin	qi2; ji4
U+8415.0	fEnglish	 water-chestnuts caltrop
U+8416.0	fUTF8	萖
U+8416.0	fCNS11643	3-3E5D
U+8416.0	fGB	E-3A66
U+8416.0	fR/S	  140.8
U+8416.0	fTotalStrokes	 12; 14
U+8416.0	fCangjie	THXU
U+8416.0	fFourCorner	44210
U+8416.0	fCantonese	gun1; gwun1 (also)
U+8416.0	fMandarin	wan3
U+8417.0	fUTF8	萗
U+8417.0	fCNS11643	3-3E56
U+8417.0	fGB	E-3A67
U+8417.0	fR/S	  140.8
U+8417.0	fTotalStrokes	 12; 14
U+8417.0	fCangjie	TJMF
U+8417.0	fFourCorner	44900
U+8417.0	fCantonese	caak3
U+8417.0	fMandarin	ce4
U+8418.0	fUTF8	萘
U+8418.0	fCNS11643	3-3E4E
U+8418.0	fGB	0-5D41
U+8418.0	fR/S	 140.8
U+8418.0	fTotalStrokes	 12; 14
U+8418.0	fCangjie	TKMF
U+8418.0	fFourCorner	44900
U+8418.0	fCantonese	noi6; loi6 (also)
U+8418.0	fMandarin	nai4
U+8418.0	fEnglish	naphthaline
U+8419.0	fUTF8	萙
U+8419.0	fGB	E-3A68
U+8419.0	fR/S	 140.8
U+8419.0	fTotalStrokes	 12; 14
U+8419.0	fCangjie	TDLU
U+8419.0	fCantonese	zam2
U+841A.0	fUTF8	萚
U+841A.0	fGB	8-7E44
U+841A.0	fR/S	 140.8; 140.16
U+841A.0	fTotalStrokes	 12; 14
U+841A.0	fCangjie	TQEQ
U+841A.0	fCantonese	tok3
U+841A.0	fMandarin	tuo4
U+841B.0	fUTF8	萛
U+841B.0	fCNS11643	2-3F72
U+841B.0	fBig5	DBAE
U+841B.0	fGB	E-3A69
U+841B.0	fR/S	 140.8
U+841B.0	fTotalStrokes	 12; 14
U+841B.0	fCangjie	TBUT
U+841B.0	fFourCorner	44446
U+841B.0	fCantonese	syun3; gau1
U+841B.0	fMandarin	jiu1
U+841C.0	fUTF8	萜
U+841C.0	fGB	0-5D46
U+841C.0	fR/S	 140.8
U+841C.0	fTotalStrokes	 12; 14
U+841C.0	fCangjie	TLBR
U+841C.0	fCantonese	tip3
U+841C.0	fMandarin	tie1
U+841C.0	fEnglish	C5H8
U+841D.0	fUTF8	萝
U+841D.0	fGB	0-425C
U+841D.0	fR/S	 140.8
U+841D.0	fTotalStrokes	 12; 14
U+841D.0	fCangjie	XTWLN
U+841D.0	fHakka	 lo2
U+841D.0	fCantonese	lo4
U+841D.0	fMandarin	luo2
U+841D.0	fEnglish	simplified form of &#x863F;
U+841E.0	fUTF8	萞
U+841E.0	fCNS11643	3-6639
U+841E.0	fGB	E-3A6A
U+841E.0	fR/S	 140.8
U+841E.0	fTotalStrokes	 12; 14
U+841E.0	fCangjie	TBOP
U+841E.0	fFourCorner	4471
U+841E.0	fCantonese	bei1
U+841F.0	fUTF8	萟
U+841F.0	fCNS11643	3-3E5B
U+841F.0	fGB	E-3A6B
U+841F.0	fR/S	  140.8
U+841F.0	fTotalStrokes	 12; 14
U+841F.0	fCangjie	THDI
U+841F.0	fFourCorner	44910
U+841F.0	fCantonese	ngai6; ai6 (also)
U+8420.0	fUTF8	萠
U+8420.0	fGB	E-3A6C
U+8420.0	fR/S	 140.8
U+8420.0	fTotalStrokes	 12; 14
U+8420.0	fCangjie	TBB
U+8420.0	fFourCorner	4422
U+8420.0	fCantonese	pang4
U+8420.0	fMandarin	meng2
U+8420.0	fEnglish	bud; sprout
U+8421.0	fUTF8	萡
U+8421.0	fCNS11643	3-6638
U+8421.0	fGB	E-3A6D
U+8421.0	fR/S	 140.8
U+8421.0	fTotalStrokes	 12; 14
U+8421.0	fCangjie	TEHA
U+8421.0	fFourCorner	4416
U+8421.0	fCantonese	bok6
U+8422.0	fUTF8	萢
U+8422.0	fGB	E-3A6E
U+8422.0	fR/S	 140.8
U+8422.0	fTotalStrokes	 12; 14
U+8422.0	fCangjie	TEPU
U+8422.0	fCantonese	paau1
U+8423.0	fUTF8	萣
U+8423.0	fCNS11643	2-3F44
U+8423.0	fBig5	DB5F
U+8423.0	fGB	E-3A6F
U+8423.0	fR/S	 140.8
U+8423.0	fTotalStrokes	 12; 14
U+8423.0	fCangjie	TJMO
U+8423.0	fFourCorner	44801
U+8423.0	fCantonese	ding6
U+8423.0	fMandarin	ding4
U+8424.0	fUTF8	萤
U+8424.0	fGB	0-5329
U+8424.0	fR/S	 140.8
U+8424.0	fTotalStrokes	 12; 14
U+8424.0	fCangjie	TBLI
U+8424.0	fHakka	 jin2
U+8424.0	fCantonese	jing4
U+8424.0	fMandarin	ying2; xing2
U+8424.0	fEnglish	simplified form of &#x87A2;
U+8425.0	fUTF8	营
U+8425.0	fGB	0-532A
U+8425.0	fR/S	 140.8
U+8425.0	fTotalStrokes	 12; 14
U+8425.0	fCangjie	TBRR
U+8425.0	fHakka	 jin2; jang2
U+8425.0	fCantonese	jing4
U+8425.0	fMandarin	ying2; xing2
U+8425.0	fEnglish	simplified form of &#x71DF;
U+8426.0	fUTF8	萦
U+8426.0	fGB	0-5D53
U+8426.0	fR/S	 140.8
U+8426.0	fTotalStrokes	 12; 14
U+8426.0	fCangjie	TBVF
U+8426.0	fHakka	 jin2
U+8426.0	fCantonese	jing4
U+8426.0	fMandarin	ying2
U+8426.0	fEnglish	simplified form of &#x7E08;
U+8427.0	fUTF8	萧
U+8427.0	fGB	0-4F74
U+8427.0	fR/S	 140.8
U+8427.0	fTotalStrokes	 12; 14
U+8427.0	fCangjie	XTLX
U+8427.0	fHakka	 siau1
U+8427.0	fCantonese	siu1
U+8427.0	fMandarin	xiao1
U+8427.0	fEnglish	simplified form of &#x856D;
U+8428.0	fUTF8	萨
U+8428.0	fGB	0-4878
U+8428.0	fR/S	 140.8; 140.9
U+8428.0	fTotalStrokes	 13; 14
U+8428.0	fCangjie	TNLH
U+8428.0	fHakka	 sat7
U+8428.0	fCantonese	saat3; saat2 (also)
U+8428.0	fMandarin	sa4
U+8428.0	fEnglish	simplified form of &#x85A9;
U+8429.0	fUTF8	萩
U+8429.0	fCNS11643	2-466C
U+8429.0	fBig5	DFC7
U+8429.0	fGB	3-6971
U+8429.0	fR/S	 140.9
U+8429.0	fTotalStrokes	 13; 15
U+8429.0	fCangjie	THDF
U+8429.0	fFourCorner	44989
U+8429.0	fHakka	  ciu1
U+8429.0	fCantonese	cau1
U+8429.0	fMandarin	qiu1; jiao1; jiao3
U+8429.0	fEnglish	 scandent hop tree
U+842A.0	fUTF8	萪
U+842A.0	fCNS11643	3-442D
U+842A.0	fGB	3-6972
U+842A.0	fR/S	  140.9
U+842A.0	fTotalStrokes	 13; 15
U+842A.0	fCangjie	THDJ
U+842A.0	fFourCorner	44940
U+842A.0	fCantonese	fo1
U+842A.0	fMandarin	ke1
U+842B.0	fUTF8	萫
U+842B.0	fCNS11643	2-4724
U+842B.0	fBig5	DFDD
U+842B.0	fGB	5-6879
U+842B.0	fR/S	 140.9
U+842B.0	fTotalStrokes	 13; 15
U+842B.0	fCangjie	THDA
U+842B.0	fFourCorner	44609
U+842B.0	fCantonese	hoeng3
U+842B.0	fMandarin	xiang4
U+842C.0	fUTF8	萬
U+842C.0	fCNS11643	1-655C
U+842C.0	fBig5	B855
U+842C.0	fGB	1-4D72
U+842C.0	fR/S	 140.9; 114.8
U+842C.0	fTotalStrokes	 13; 15
U+842C.0	fCangjie	TWLB
U+842C.0	fFourCorner	44427
U+842C.0	fHakka	 wan5; wan6; man3; man5; wan3; van5
U+842C.0	fCantonese	maan6; maak6
U+842C.0	fMandarin	wan4
U+842C.0	fEnglish	[1] ten thousand [2] all; omni- [3] a very great number; innumerable; myriad [4] very; extremely; absolutely [5] name of an ancient dane [6] a Chinese family name
U+842D.0	fUTF8	萭
U+842D.0	fCNS11643	2-4671
U+842D.0	fBig5	DFCC
U+842D.0	fGB	3-6975
U+842D.0	fR/S	 140.9
U+842D.0	fTotalStrokes	 13; 15
U+842D.0	fCangjie	THLB
U+842D.0	fFourCorner	44227
U+842D.0	fHakka	 ji3; ri3
U+842D.0	fCantonese	jyu5; geoi2
U+842D.0	fMandarin	yu3; ju1; ju3; ju4
U+842D.0	fEnglish	the name of a tree
U+842E.0	fUTF8	萮
U+842E.0	fCNS11643	3-4431
U+842E.0	fGB	3-6978
U+842E.0	fR/S	  140.9
U+842E.0	fTotalStrokes	 13; 15
U+842E.0	fCangjie	TOMN
U+842E.0	fFourCorner	44220
U+842E.0	fHakka	 ji2; ri2
U+842E.0	fCantonese	jyu4
U+842E.0	fMandarin	yu4
U+842E.0	fEnglish	a wild plant found in Honan
U+842F.0	fUTF8	萯
U+842F.0	fCNS11643	2-466F
U+842F.0	fBig5	DFCA
U+842F.0	fGB	3-6871
U+842F.0	fR/S	 140.9
U+842F.0	fTotalStrokes	 13; 15
U+842F.0	fCangjie	TNBC
U+842F.0	fFourCorner	44806
U+842F.0	fCantonese	fu6; fu5; pui5
U+842F.0	fMandarin	fu4; bei4; pei4; sai3
U+8430.0	fUTF8	萰
U+8430.0	fCNS11643	2-465A
U+8430.0	fBig5	DFB5
U+8430.0	fGB	3-6969
U+8430.0	fR/S	 140.9
U+8430.0	fTotalStrokes	 13; 15
U+8430.0	fCangjie	TDWF
U+8430.0	fFourCorner	44906
U+8430.0	fCantonese	lin6
U+8430.0	fMandarin	lian4
U+8431.0	fUTF8	萱
U+8431.0	fCNS11643	1-6630
U+8431.0	fBig5	B8A9
U+8431.0	fGB	0-5D66
U+8431.0	fR/S	 140.9
U+8431.0	fTotalStrokes	 13; 15
U+8431.0	fCangjie	TJMM
U+8431.0	fFourCorner	44106
U+8431.0	fHakka	  sien1; sian1; sen1; hien1
U+8431.0	fCantonese	hyun1
U+8431.0	fMandarin	xuan1
U+8431.0	fEnglish	Hemerocallis fulva, day-lily, whose dried flowers are edible
U+8432.0	fUTF8	萲
U+8432.0	fCNS11643	2-466A
U+8432.0	fBig5	DFC5
U+8432.0	fGB	E-3A70
U+8432.0	fR/S	 140.9
U+8432.0	fTotalStrokes	 13; 15
U+8432.0	fCangjie	TBME
U+8432.0	fFourCorner	44447
U+8432.0	fCantonese	hyun1; hyun3
U+8432.0	fMandarin	yuan2; xuan1; xuan3
U+8432.0	fEnglish	the day-lily, Hemerocallis flava
U+8433.0	fUTF8	萳
U+8433.0	fCNS11643	2-467E
U+8433.0	fBig5	DFD9
U+8433.0	fGB	5-6866
U+8433.0	fR/S	 140.9
U+8433.0	fTotalStrokes	 13; 15
U+8433.0	fCangjie	TJBJ
U+8433.0	fFourCorner	44227
U+8433.0	fCantonese	naam5; laam5 (also); naam4
U+8433.0	fMandarin	nan2; nan3
U+8434.0	fUTF8	萴
U+8434.0	fCNS11643	2-4666
U+8434.0	fBig5	DFC1
U+8434.0	fGB	3-686D
U+8434.0	fR/S	 140.9
U+8434.0	fTotalStrokes	 13; 15
U+8434.0	fCangjie	TBCN
U+8434.0	fFourCorner	44820
U+8434.0	fCantonese	zak1
U+8434.0	fMandarin	ze2; ce4
U+8434.0	fEnglish	medicinal poison plant
U+8435.0	fUTF8	萵
U+8435.0	fCNS11643	1-6638
U+8435.0	fBig5	B8B1
U+8435.0	fGB	1-5D2B
U+8435.0	fR/S	 140.9
U+8435.0	fTotalStrokes	 13; 15
U+8435.0	fCangjie	TBBR
U+8435.0	fFourCorner	44227
U+8435.0	fHakka	 wo1; vo1
U+8435.0	fCantonese	wo1
U+8435.0	fMandarin	wo1
U+8435.0	fEnglish	 lettuce
U+8436.0	fUTF8	萶
U+8436.0	fCNS11643	2-467D
U+8436.0	fBig5	DFD8
U+8436.0	fGB	5-6864
U+8436.0	fR/S	 140.9
U+8436.0	fTotalStrokes	 13; 15
U+8436.0	fCangjie	TQKA
U+8436.0	fFourCorner	44608
U+8436.0	fCantonese	ceon2
U+8436.0	fMandarin	chun3
U+8437.0	fUTF8	萷
U+8437.0	fCNS11643	2-4664
U+8437.0	fBig5	DFBF
U+8437.0	fGB	3-696D
U+8437.0	fR/S	 140.9
U+8437.0	fTotalStrokes	 13; 15
U+8437.0	fCangjie	TFBN
U+8437.0	fFourCorner	44220
U+8437.0	fCantonese	siu1; sok3; sok6
U+8437.0	fMandarin	xiao1; shao1; shuo4
U+8438.0	fUTF8	萸
U+8438.0	fCNS11643	1-6169
U+8438.0	fBig5	B5E3
U+8438.0	fGB	0-5D47
U+8438.0	fR/S	 140.9
U+8438.0	fTotalStrokes	 13; 15
U+8438.0	fCangjie	THXO
U+8438.0	fFourCorner	44237
U+8438.0	fHakka	 ji2; ri2
U+8438.0	fCantonese	jyu4
U+8438.0	fMandarin	yu2
U+8438.0	fEnglish	dogwood
U+8439.0	fUTF8	萹
U+8439.0	fCNS11643	2-4674
U+8439.0	fBig5	DFCF
U+8439.0	fGB	3-6A27
U+8439.0	fR/S	 140.9
U+8439.0	fTotalStrokes	 13; 15
U+8439.0	fCangjie	THSB
U+8439.0	fFourCorner	44227
U+8439.0	fHakka	  bien1; pien1
U+8439.0	fCantonese	pin1; bin1 (also); bin2
U+8439.0	fMandarin	pian1; bian1; pian2
U+8439.0	fEnglish	[1] in [2] Polygonum aviculare; Prostrate knotgrass
U+8439.1	fUTF8	萹
U+8439.1	fCNS11643	2-4674
U+8439.1	fBig5	DFCF
U+8439.1	fGB	3-6A27
U+8439.1	fR/S	 140.9
U+8439.1	fTotalStrokes	 13; 15
U+8439.1	fCangjie	THSB
U+8439.1	fFourCorner	44227
U+8439.1	fMandarin	bian3
U+8439.1	fEnglish	[1] in [2] Lablab purpureus, Hyacinth bean or "Xiaobiandou" Lensbrculinaris Lentil
U+843A.0	fUTF8	萺
U+843A.0	fCNS11643	2-4665
U+843A.0	fBig5	DFC0
U+843A.0	fGB	5-6871
U+843A.0	fR/S	 140.9
U+843A.0	fTotalStrokes	 13; 15
U+843A.0	fCangjie	TABU
U+843A.0	fFourCorner	44606
U+843A.0	fCantonese	mou6; muk6
U+843A.0	fMandarin	mao4; mu4
U+843B.0	fUTF8	萻
U+843B.0	fCNS11643	2-467B
U+843B.0	fBig5	DFD6
U+843B.0	fGB	5-6929
U+843B.0	fR/S	 140.9
U+843B.0	fTotalStrokes	 13; 15
U+843B.0	fCangjie	TYTA
U+843B.0	fFourCorner	44601
U+843B.0	fCantonese	jam1; am1
U+843B.0	fMandarin	an1
U+843C.0	fUTF8	萼
U+843C.0	fCNS11643	1-6637
U+843C.0	fBig5	B8B0
U+843C.0	fGB	0-5D60
U+843C.0	fR/S	 140.9
U+843C.0	fTotalStrokes	 13; 15
U+843C.0	fCangjie	TRRS
U+843C.0	fFourCorner	44207
U+843C.0	fHakka	 ngok8
U+843C.0	fCantonese	ngok6; ok6 (also)
U+843C.0	fMandarin	e4
U+843C.0	fEnglish	 calyx (flower)
U+843D.0	fUTF8	落
U+843D.0	fCNS11643	1-662F
U+843D.0	fBig5	B8A8
U+843D.0	fGB	0-4264
U+843D.0	fR/S	 140.9
U+843D.0	fTotalStrokes	 13; 15
U+843D.0	fCangjie	TEHR
U+843D.0	fFourCorner	44164
U+843D.0	fHakka	  lok8; lak8; lap7; lau5
U+843D.0	fCantonese	lok6; laai6; lok1 (also); lok2 (also); laai3
U+843D.0	fMandarin	luo4; luo1; le4
U+843D.0	fEnglish	[1] fall; decline; wither; weakened; fallen [2] lose [3] few and far-spaced; stand apart; loose and scattered [4] village; hamlet [5] put (pen to paper) [6] settle down [7] pile; heap
U+843D.1	fUTF8	落
U+843D.1	fCNS11643	1-662F
U+843D.1	fBig5	B8A8
U+843D.1	fGB	0-4264
U+843D.1	fR/S	 140.9
U+843D.1	fTotalStrokes	 13; 15
U+843D.1	fCangjie	TEHR
U+843D.1	fFourCorner	44164
U+843D.1	fMandarin	lao4
U+843D.1	fEnglish	[1] fall; drop (in prices, etc.) [2] land; perch (birds, etc.) [3] get [4] net income; surplus [5] place
U+843D.2	fUTF8	落
U+843D.2	fCNS11643	1-662F
U+843D.2	fBig5	B8A8
U+843D.2	fGB	0-4264
U+843D.2	fR/S	 140.9
U+843D.2	fTotalStrokes	 13; 15
U+843D.2	fCangjie	TEHR
U+843D.2	fFourCorner	44164
U+843D.2	fMandarin	la4
U+843D.2	fEnglish	[1] leave behind [2] fall behind [3] miss; omit
U+843E.0	fUTF8	萾
U+843E.0	fCNS11643	3-443A
U+843E.0	fGB	5-6931
U+843E.0	fR/S	 140.9
U+843E.0	fTotalStrokes	 13; 15
U+843E.0	fCangjie	TNST
U+843E.0	fFourCorner	44100
U+843E.0	fCantonese	jing4
U+843E.0	fMandarin	ying2
U+843F.0	fUTF8	萿
U+843F.0	fCNS11643	2-464F
U+843F.0	fBig5	DFAA
U+843F.0	fGB	3-6A25
U+843F.0	fR/S	 140.9
U+843F.0	fTotalStrokes	 13; 15
U+843F.0	fCangjie	TEHR
U+843F.0	fFourCorner	44164
U+843F.0	fCantonese	kut3; kwut3 (also); wut6
U+843F.0	fMandarin	huo2; gua1; kuo4
U+8440.0	fUTF8	葀
U+8440.0	fCNS11643	2-4657
U+8440.0	fBig5	DFB2
U+8440.0	fGB	3-696C
U+8440.0	fR/S	 140.9
U+8440.0	fTotalStrokes	 13; 15
U+8440.0	fCangjie	TQHR
U+8440.0	fFourCorner	44564
U+8440.0	fHakka	 gat7
U+8440.0	fCantonese	kut3; kwut3 (also)
U+8440.0	fMandarin	gua1
U+8441.0	fUTF8	葁
U+8441.0	fCNS11643	3-4435
U+8441.0	fGB	E-3A71
U+8441.0	fR/S	  140.9
U+8441.0	fTotalStrokes	 13; 15
U+8441.0	fCangjie	TTGV
U+8441.0	fFourCorner	44400
U+8441.0	fHakka	 giong1
U+8441.0	fCantonese	goeng1
U+8441.0	fMandarin	jiang1
U+8441.0	fEnglish	ginger
U+8442.0	fUTF8	葂
U+8442.0	fCNS11643	2-4670
U+8442.0	fBig5	DFCB
U+8442.0	fGB	3-697C
U+8442.0	fR/S	 140.9
U+8442.0	fTotalStrokes	 13; 15
U+8442.0	fCangjie	TNUS
U+8442.0	fFourCorner	44212
U+8442.0	fCantonese	min5; maan5
U+8442.0	fMandarin	wan3; mian3
U+8443.0	fUTF8	葃
U+8443.0	fCNS11643	2-4668
U+8443.0	fBig5	DFC3
U+8443.0	fGB	3-696F
U+8443.0	fR/S	 140.9
U+8443.0	fTotalStrokes	 13; 15
U+8443.0	fCangjie	TAOS
U+8443.0	fFourCorner	44611
U+8443.0	fCantonese	zok6; zaak6; zik6; zou6
U+8443.0	fMandarin	zuo2; ji2; ze2
U+8444.0	fUTF8	葄
U+8444.0	fCNS11643	2-4723
U+8444.0	fBig5	DFDC
U+8444.0	fGB	3-697B
U+8444.0	fR/S	 140.9
U+8444.0	fTotalStrokes	 13; 15
U+8444.0	fCangjie	TBOS
U+8444.0	fFourCorner	44211
U+8444.0	fCantonese	zou6; zou3
U+8444.0	fMandarin	zuo4; cu2; qu2
U+8444.0	fEnglish	mat
U+8445.0	fUTF8	葅
U+8445.0	fCNS11643	2-466B
U+8445.0	fBig5	DFC6
U+8445.0	fGB	5-6923
U+8445.0	fR/S	 140.9
U+8445.0	fTotalStrokes	 13; 15
U+8445.0	fCangjie	TOOM
U+8445.0	fFourCorner	44817
U+8445.0	fCantonese	zeoi1
U+8445.0	fMandarin	ju1; zu1
U+8445.0	fEnglish	[1] salted or pickled vegetable or meat [2] a marshland covered with reeds
U+8446.0	fUTF8	葆
U+8446.0	fCNS11643	1-663D
U+8446.0	fBig5	B8B6
U+8446.0	fGB	0-5D61
U+8446.0	fR/S	 140.9
U+8446.0	fTotalStrokes	 13; 15
U+8446.0	fCangjie	TORD
U+8446.0	fFourCorner	44294
U+8446.0	fHakka	 bau3;  bo3
U+8446.0	fCantonese	bou2
U+8446.0	fMandarin	bao3; bao1
U+8446.0	fEnglish	[1] reserved; hidden; not easily revealed [2] a dense or luxuriant growth of vegetation [3] protect; protection
U+8447.0	fUTF8	葇
U+8447.0	fCNS11643	2-467C
U+8447.0	fBig5	DFD7
U+8447.0	fGB	3-6A2E
U+8447.0	fR/S	 140.9
U+8447.0	fTotalStrokes	 13; 15
U+8447.0	fCangjie	TNHD
U+8447.0	fFourCorner	44904
U+8447.0	fCantonese	jau4; jau2
U+8447.0	fMandarin	rou2; rou3
U+8448.0	fUTF8	葈
U+8448.0	fCNS11643	3-442C
U+8448.0	fGB	3-6A2D
U+8448.0	fR/S	 140.9
U+8448.0	fTotalStrokes	 13; 15
U+8448.0	fCangjie	TIRD
U+8448.0	fFourCorner	44900
U+8448.0	fHakka	 sai3
U+8448.0	fCantonese	saai2
U+8448.0	fMandarin	xi3
U+8449.0	fUTF8	葉
U+8449.0	fCNS11643	1-6634
U+8449.0	fBig5	B8AD
U+8449.0	fGB	1-5236
U+8449.0	fR/S	 140.9
U+8449.0	fTotalStrokes	 13; 15
U+8449.0	fCangjie	TPTD
U+8449.0	fFourCorner	44904
U+8449.0	fHakka	  jap8; rap8
U+8449.0	fCantonese	jip6; sip3; jip2 (also)
U+8449.0	fMandarin	ye4; die2; xie2
U+8449.0	fEnglish	[1] leaf; petal (flower) [2] leaf; two pages (book) [3] period; era; epoch [4] something light and tiny (as a small boat in a lake) [5] a Chinese family name
U+8449.1	fUTF8	葉
U+8449.1	fCNS11643	1-6634
U+8449.1	fBig5	B8AD
U+8449.1	fGB	1-5236
U+8449.1	fR/S	 140.9
U+8449.1	fTotalStrokes	 13; 15
U+8449.1	fCangjie	TPTD
U+8449.1	fFourCorner	44904
U+8449.1	fMandarin	she4
U+8449.1	fEnglish	used in names of places
U+844A.0	fUTF8	葊
U+844A.0	fCNS11643	3-443E
U+844A.0	fGB	5-6922
U+844A.0	fR/S	 140.9
U+844A.0	fTotalStrokes	 13; 15
U+844A.0	fCangjie	TORT
U+844A.0	fFourCorner	44440
U+844A.0	fCantonese	am1; ngam1 (also)
U+844A.0	fMandarin	an1
U+844A.0	fEnglish	cottage
U+844B.0	fUTF8	葋
U+844B.0	fCNS11643	2-466E
U+844B.0	fBig5	DFC9
U+844B.0	fGB	5-6924
U+844B.0	fR/S	 140.9
U+844B.0	fTotalStrokes	 13; 15
U+844B.0	fCangjie	TBPR
U+844B.0	fFourCorner	44227
U+844B.0	fCantonese	keoi4
U+844B.0	fMandarin	qu2
U+844C.0	fUTF8	葌
U+844C.0	fCNS11643	2-4676
U+844C.0	fBig5	DFD1
U+844C.0	fGB	5-6930
U+844C.0	fR/S	 140.9
U+844C.0	fTotalStrokes	 13; 15
U+844C.0	fCangjie	TVVV
U+844C.0	fFourCorner	44444
U+844C.0	fCantonese	gaan1
U+844C.0	fMandarin	jian1
U+844D.0	fUTF8	葍
U+844D.0	fCNS11643	2-465B
U+844D.0	fBig5	DFB6
U+844D.0	fGB	3-6967
U+844D.0	fR/S	 140.9
U+844D.0	fTotalStrokes	 13; 15
U+844D.0	fCangjie	TMRW
U+844D.0	fFourCorner	44606
U+844D.0	fHakka	  fuk7; pet8
U+844D.0	fCantonese	fuk1; fuk6 (also)
U+844D.0	fMandarin	fu2
U+844D.0	fEnglish	plant
U+844E.0	fUTF8	葎
U+844E.0	fCNS11643	2-4675
U+844E.0	fBig5	DFD0
U+844E.0	fGB	3-6976
U+844E.0	fR/S	 140.9
U+844E.0	fTotalStrokes	 13; 15
U+844E.0	fCangjie	THOQ
U+844E.0	fFourCorner	44257
U+844E.0	fHakka	 lut8
U+844E.0	fCantonese	leot6
U+844E.0	fMandarin	l4; luu4; lu4
U+844E.0	fEnglish	the common wild hop of China and Japan
U+844F.0	fUTF8	葏
U+844F.0	fCNS11643	3-4427
U+844F.0	fGB	5-692E
U+844F.0	fR/S	  140.9
U+844F.0	fTotalStrokes	 13; 15
U+844F.0	fCangjie	TELQ
U+844F.0	fFourCorner	44150
U+844F.0	fCantonese	zin1
U+844F.0	fMandarin	jing1
U+8450.0	fUTF8	葐
U+8450.0	fCNS11643	2-4728
U+8450.0	fBig5	DFE1
U+8450.0	fGB	3-697A
U+8450.0	fR/S	 140.9
U+8450.0	fTotalStrokes	 13; 15
U+8450.0	fCangjie	TCST
U+8450.0	fFourCorner	44107
U+8450.0	fHakka	 pun2; fun2
U+8450.0	fCantonese	pun4; pun6
U+8450.0	fMandarin	pen2; fen2; pen3
U+8450.0	fEnglish	a kind of wild raspberry
U+8451.0	fUTF8	葑
U+8451.0	fCNS11643	2-4656
U+8451.0	fBig5	DFB1
U+8451.0	fGB	0-5D57
U+8451.0	fR/S	 140.9
U+8451.0	fTotalStrokes	 13; 15
U+8451.0	fCangjie	TGGI
U+8451.0	fFourCorner	44140
U+8451.0	fHakka	 fung1
U+8451.0	fCantonese	fung1; fung6; fung3
U+8451.0	fMandarin	feng1; feng4
U+8451.0	fEnglish	the rape-turnip
U+8452.0	fUTF8	葒
U+8452.0	fCNS11643	2-4677
U+8452.0	fBig5	DFD2
U+8452.0	fGB	1-5D26
U+8452.0	fR/S	 140.9
U+8452.0	fTotalStrokes	 13; 15
U+8452.0	fCangjie	TVFM
U+8452.0	fFourCorner	44911
U+8452.0	fHakka	 fung2
U+8452.0	fCantonese	hung4
U+8452.0	fMandarin	hong2
U+8452.0	fEnglish	Polygonum orientale
U+8453.0	fUTF8	葓
U+8453.0	fCNS11643	3-4433
U+8453.0	fGB	3-6A24
U+8453.0	fR/S	 140.9
U+8453.0	fTotalStrokes	 13; 15
U+8453.0	fCangjie	TETC
U+8453.0	fFourCorner	44180
U+8453.0	fCantonese	hung4
U+8453.0	fMandarin	hong2
U+8453.0	fEnglish	vegetable
U+8454.0	fUTF8	葔
U+8454.0	fCNS11643	2-4726
U+8454.0	fBig5	DFDF
U+8454.0	fGB	E-3A72
U+8454.0	fR/S	 140.9
U+8454.0	fTotalStrokes	 13; 15
U+8454.0	fCangjie	TONK
U+8454.0	fFourCorner	44234
U+8454.0	fCantonese	hau4
U+8454.0	fMandarin	hou2
U+8455.0	fUTF8	葕
U+8455.0	fCNS11643	3-443F
U+8455.0	fGB	5-687E
U+8455.0	fR/S	 140.9
U+8455.0	fTotalStrokes	 13; 15
U+8455.0	fCangjie	THON
U+8455.0	fFourCorner	44220
U+8455.0	fCantonese	jin4
U+8455.0	fMandarin	yan2
U+8456.0	fUTF8	葖
U+8456.0	fCNS11643	2-4650
U+8456.0	fBig5	DFAB
U+8456.0	fGB	3-6A26
U+8456.0	fR/S	 140.9
U+8456.0	fTotalStrokes	 13; 15
U+8456.0	fCangjie	TJCK
U+8456.0	fFourCorner	44433
U+8456.0	fHakka	 tut7
U+8456.0	fCantonese	dat6
U+8456.0	fMandarin	tu1; tu2
U+8456.0	fEnglish	a turnip (botany)
U+8457.0	fUTF8	著
U+8457.0	fCNS11643	1-6161
U+8457.0	fBig5	B5DB
U+8457.0	fGB	0-5678
U+8457.0	fR/S	 140.8; 140.9
U+8457.0	fTotalStrokes	 12; 14
U+8457.0	fCangjie	TJKA
U+8457.0	fFourCorner	44604
U+8457.0	fHakka	 tiok8; zok7;  zhu5; cok8; diau2; chok7; do3; ch'ok8; zhu6
U+8457.0	fCantonese	zyu3; zoek6; zoek3
U+8457.0	fMandarin	zhu4; zhu3; chu2
U+8457.0	fEnglish	[1] apparent; obvious; famous [2] write; author [3] writings; literary works; books [4] set forth; manifest; make known
U+8457.1	fUTF8	著
U+8457.1	fCNS11643	1-6161
U+8457.1	fBig5	B5DB
U+8457.1	fGB	0-5678
U+8457.1	fR/S	 140.8; 140.9
U+8457.1	fTotalStrokes	 12; 14
U+8457.1	fCangjie	TJKA
U+8457.1	fFourCorner	44604
U+8457.1	fHakka	 zu5
U+8457.1	fMandarin	zhuo2
U+8457.1	fEnglish	[1] wear (garments, etc.) [2] move (on the chessboard); in actions; plans [3] apply (color, etc.); start (an assignment, investigation, etc.)
U+8457.2	fUTF8	著
U+8457.2	fCNS11643	1-6161
U+8457.2	fBig5	B5DB
U+8457.2	fGB	0-5678
U+8457.2	fR/S	 140.8; 140.9
U+8457.2	fTotalStrokes	 12; 14
U+8457.2	fCangjie	TJKA
U+8457.2	fFourCorner	44604
U+8457.2	fMandarin	zhao2
U+8457.2	fEnglish	[1] hit the bull's eye; right to the point; very worthwhile [2] catch (fire, cold, etc.) [3] make a move; take action; use
U+8457.3	fUTF8	著
U+8457.3	fCNS11643	1-6161
U+8457.3	fBig5	B5DB
U+8457.3	fGB	0-5678
U+8457.3	fR/S	 140.8; 140.9
U+8457.3	fTotalStrokes	 12; 14
U+8457.3	fCangjie	TJKA
U+8457.3	fFourCorner	44604
U+8457.3	fMandarin	zhao1
U+8457.3	fEnglish	[1] bear; take [2] plan; method
U+8457.4	fUTF8	著
U+8457.4	fCNS11643	1-6161
U+8457.4	fBig5	B5DB
U+8457.4	fGB	0-5678
U+8457.4	fR/S	 140.8; 140.9
U+8457.4	fTotalStrokes	 12; 14
U+8457.4	fCangjie	TJKA
U+8457.4	fFourCorner	44604
U+8457.4	fMandarin	zhe5
U+8457.4	fEnglish	an adverbial particle
U+8458.0	fUTF8	葘
U+8458.0	fCNS11643	3-4440
U+8458.0	fGB	E-3A73
U+8458.0	fR/S	 140.9
U+8458.0	fTotalStrokes	 13; 15
U+8458.0	fCangjie	TVMW
U+8458.0	fFourCorner	44600
U+8458.0	fCantonese	zi1; zoi1
U+8458.0	fMandarin	zi1
U+8459.0	fUTF8	葙
U+8459.0	fCNS11643	2-465E
U+8459.0	fBig5	DFB9
U+8459.0	fGB	0-5D59
U+8459.0	fR/S	 140.9
U+8459.0	fTotalStrokes	 13; 15
U+8459.0	fCangjie	TDBU
U+8459.0	fFourCorner	44960
U+8459.0	fHakka	 siong1
U+8459.0	fCantonese	soeng1
U+8459.0	fMandarin	xiang1
U+8459.0	fEnglish	a medicinal plant (Celosia argentea)
U+845A.0	fUTF8	葚
U+845A.0	fCNS11643	2-465D
U+845A.0	fBig5	DFB8
U+845A.0	fGB	0-5D58
U+845A.0	fR/S	 140.9
U+845A.0	fTotalStrokes	 13; 15
U+845A.0	fCangjie	TTMV
U+845A.0	fFourCorner	44711
U+845A.0	fHakka	 shim5; sim3; siim5
U+845A.0	fCantonese	sam6; sam4 (also); zam1
U+845A.0	fMandarin	shen4; ren4
U+845A.0	fEnglish	the mulberry fruit
U+845B.0	fUTF8	葛
U+845B.0	fCNS11643	1-6636
U+845B.0	fBig5	B8AF
U+845B.0	fGB	0-3870
U+845B.0	fR/S	 140.9
U+845B.0	fTotalStrokes	 13; 15
U+845B.0	fCangjie	TAPV
U+845B.0	fFourCorner	44727
U+845B.0	fHakka	  got7; gok7
U+845B.0	fCantonese	got3; got2 (also)
U+845B.0	fMandarin	ge2; zang1
U+845B.0	fEnglish	dolichos (Pueraria thunbergiana), a creeping, edible bean whose fibers can be made into linen-like cloth and whose roots are used in herbal medicine
U+845B.1	fUTF8	葛
U+845B.1	fCNS11643	1-6636
U+845B.1	fBig5	B8AF
U+845B.1	fGB	0-3870
U+845B.1	fR/S	 140.9
U+845B.1	fTotalStrokes	 13; 15
U+845B.1	fCangjie	TAPV
U+845B.1	fFourCorner	44727
U+845B.1	fMandarin	ge3
U+845B.1	fEnglish	a Chinese family name
U+845C.0	fUTF8	葜
U+845C.0	fCNS11643	3-4432
U+845C.0	fGB	0-5D56
U+845C.0	fR/S	 140.9
U+845C.0	fTotalStrokes	 13; 15
U+845C.0	fCangjie	TQHK
U+845C.0	fFourCorner	44430
U+845C.0	fHakka	 giet7;  kat7
U+845C.0	fCantonese	kit3; hat6 (also)
U+845C.0	fMandarin	jie2; qia1
U+845C.0	fEnglish	Smilax china
U+845D.0	fUTF8	葝
U+845D.0	fCNS11643	2-4661
U+845D.0	fBig5	DFBC
U+845D.0	fGB	3-693B
U+845D.0	fR/S	 140.9
U+845D.0	fTotalStrokes	 13; 15
U+845D.0	fCangjie	TMMS
U+845D.0	fFourCorner	44127
U+845D.0	fCantonese	king4; ging3
U+845D.0	fMandarin	jing4; qing2
U+845E.0	fUTF8	葞
U+845E.0	fCNS11643	2-4663
U+845E.0	fBig5	DFBE
U+845E.0	fGB	3-6A29
U+845E.0	fR/S	 140.9
U+845E.0	fTotalStrokes	 13; 15
U+845E.0	fCangjie	TNSJ
U+845E.0	fFourCorner	44241
U+845E.0	fCantonese	mei5
U+845E.0	fMandarin	mi3; juan1
U+845F.0	fUTF8	葟
U+845F.0	fCNS11643	2-4672
U+845F.0	fBig5	DFCD
U+845F.0	fGB	3-6974
U+845F.0	fR/S	 140.9
U+845F.0	fTotalStrokes	 13; 15
U+845F.0	fCangjie	THAG
U+845F.0	fFourCorner	44104
U+845F.0	fCantonese	wong4
U+845F.0	fMandarin	huang2
U+8460.0	fUTF8	葠
U+8460.0	fCNS11643	2-4725
U+8460.0	fBig5	DFDE
U+8460.0	fGB	8-2D60
U+8460.0	fR/S	 140.9
U+8460.0	fTotalStrokes	 13; 15
U+8460.0	fCangjie	TOSE
U+8460.0	fFourCorner	44247
U+8460.0	fHakka	  sem1;  cam1; zim5
U+8460.0	fCantonese	sam1
U+8460.0	fMandarin	shen1; jin4
U+8460.0	fEnglish	[1] ginseng [2] one of the 28 constellations
U+8461.0	fUTF8	葡
U+8461.0	fCNS11643	1-6639
U+8461.0	fBig5	B8B2
U+8461.0	fGB	0-464F
U+8461.0	fR/S	 140.9
U+8461.0	fTotalStrokes	 13; 15
U+8461.0	fCangjie	TPIB
U+8461.0	fFourCorner	44227
U+8461.0	fHakka	 pu2; pu3
U+8461.0	fCantonese	pou4
U+8461.0	fMandarin	pu2; bei4
U+8461.0	fEnglish	[1] the grave; vine [2] short for Portugal
U+8462.0	fUTF8	葢
U+8462.0	fCNS11643	3-4438
U+8462.0	fGB	5-686C
U+8462.0	fR/S	  140.9
U+8462.0	fTotalStrokes	 13; 15
U+8462.0	fCangjie	TKIT
U+8462.0	fFourCorner	44100
U+8462.0	fHakka	  goi5
U+8462.0	fCantonese	goi3; koi3 (also); hap1; hap6 (also); gap3
U+8462.0	fMandarin	gai4; ge3; he2
U+8462.0	fEnglish	 cover; build, etc.
U+8463.0	fUTF8	董
U+8463.0	fCNS11643	1-663A
U+8463.0	fBig5	B8B3
U+8463.0	fGB	0-362D
U+8463.0	fR/S	 140.9
U+8463.0	fTotalStrokes	 13; 15
U+8463.0	fCangjie	THJG
U+8463.0	fFourCorner	44104
U+8463.0	fHakka	  dung3
U+8463.0	fCantonese	dung2
U+8463.0	fMandarin	dong3; zhong3
U+8463.0	fEnglish	[1] supervise; oversee; rectify; correct [2] short for directors (as a board of directors, etc.) [3] a Chinese family name
U+8464.0	fUTF8	葤
U+8464.0	fCNS11643	3-4430
U+8464.0	fGB	1-5D27
U+8464.0	fR/S	  140.9
U+8464.0	fTotalStrokes	 13; 15
U+8464.0	fCangjie	TVFI
U+8464.0	fFourCorner	44920
U+8464.0	fCantonese	zau6
U+8464.0	fMandarin	zhou4
U+8465.0	fUTF8	葥
U+8465.0	fCNS11643	2-4655
U+8465.0	fBig5	DFB0
U+8465.0	fGB	3-6A23
U+8465.0	fR/S	 140.9
U+8465.0	fTotalStrokes	 13; 15
U+8465.0	fCangjie	TTBN
U+8465.0	fFourCorner	44221
U+8465.0	fCantonese	zin3
U+8465.0	fMandarin	qian2; jian4
U+8466.0	fUTF8	葦
U+8466.0	fCNS11643	1-6632
U+8466.0	fBig5	B8AB
U+8466.0	fGB	1-4E2D
U+8466.0	fR/S	 140.9
U+8466.0	fTotalStrokes	 13; 15
U+8466.0	fCangjie	TDMQ
U+8466.0	fFourCorner	44506
U+8466.0	fHakka	  wui3; wi3; vui3
U+8466.0	fCantonese	wai5
U+8466.0	fMandarin	wei3; wei2; wei4
U+8466.0	fEnglish	 reed
U+8467.0	fUTF8	葧
U+8467.0	fCNS11643	2-4659
U+8467.0	fBig5	DFB4
U+8467.0	fGB	3-6966
U+8467.0	fR/S	 140.9
U+8467.0	fTotalStrokes	 13; 15
U+8467.0	fCangjie	TJDS
U+8467.0	fFourCorner	44427
U+8467.0	fCantonese	but6
U+8467.0	fMandarin	bo2; bi2
U+8468.0	fUTF8	葨
U+8468.0	fCNS11643	2-4721
U+8468.0	fBig5	DFDA
U+8468.0	fGB	5-6872
U+8468.0	fR/S	 140.9
U+8468.0	fTotalStrokes	 13; 15
U+8468.0	fCangjie	TWMV
U+8468.0	fFourCorner	44732
U+8468.0	fCantonese	wui1
U+8468.0	fMandarin	wei1
U+8469.0	fUTF8	葩
U+8469.0	fCNS11643	1-663B
U+8469.0	fBig5	B8B4
U+8469.0	fGB	0-5D62
U+8469.0	fR/S	 140.9
U+8469.0	fTotalStrokes	 13; 15
U+8469.0	fCangjie	THAU
U+8469.0	fFourCorner	44617
U+8469.0	fHakka	 ba1
U+8469.0	fCantonese	baa1; paa1 (also)
U+8469.0	fMandarin	pa1
U+8469.0	fEnglish	[1] flowers [2] used in combination
U+846A.0	fUTF8	葪
U+846A.0	fCNS11643	4-4044
U+846A.0	fGB	3-697D
U+846A.0	fR/S	  140.9
U+846A.0	fTotalStrokes	 13; 15
U+846A.0	fCangjie	TNBN
U+846A.0	fFourCorner	44220
U+846A.0	fCantonese	gai3; gaai3
U+846A.0	fMandarin	ji4; jie4
U+846B.0	fUTF8	葫
U+846B.0	fCNS11643	1-6633
U+846B.0	fBig5	B8AC
U+846B.0	fGB	0-3A79
U+846B.0	fR/S	 140.9
U+846B.0	fTotalStrokes	 13; 15
U+846B.0	fCangjie	TJRB
U+846B.0	fFourCorner	44627
U+846B.0	fHakka	  fu2; fu3
U+846B.0	fCantonese	wu4
U+846B.0	fMandarin	hu2; hu1
U+846B.0	fEnglish	the bottle-gourd; the calabash
U+846C.0	fUTF8	葬
U+846C.0	fCNS11643	1-6635
U+846C.0	fBig5	B8AE
U+846C.0	fGB	0-5461
U+846C.0	fR/S	 140.9
U+846C.0	fTotalStrokes	 13; 15
U+846C.0	fCangjie	TMPT
U+846C.0	fFourCorner	44441
U+846C.0	fHakka	  zong5
U+846C.0	fCantonese	zong3
U+846C.0	fMandarin	zang4
U+846C.0	fEnglish	 bury; inter or consign to a grave
U+846D.0	fUTF8	葭
U+846D.0	fCNS11643	1-663C
U+846D.0	fBig5	B8B5
U+846D.0	fGB	0-5D67
U+846D.0	fR/S	 140.9
U+846D.0	fTotalStrokes	 13; 15
U+846D.0	fCangjie	TRSE
U+846D.0	fFourCorner	44247
U+846D.0	fHakka	 ga1; ha3
U+846D.0	fCantonese	gaa1
U+846D.0	fMandarin	jia1; xia2
U+846D.0	fEnglish	[1] bulrush; reed [2] flute; whistle
U+846E.0	fUTF8	葮
U+846E.0	fCNS11643	2-4727
U+846E.0	fBig5	DFE0
U+846E.0	fGB	E-3A74
U+846E.0	fR/S	 140.9
U+846E.0	fTotalStrokes	 13; 15
U+846E.0	fCangjie	THJE
U+846E.0	fFourCorner	44447
U+846E.0	fHakka	 ton5; ton6
U+846E.0	fCantonese	dyun6
U+846E.0	fMandarin	duan4
U+846E.0	fEnglish	Hibiscus Syriacus
U+846F.0	fUTF8	葯
U+846F.0	fCNS11643	2-4678
U+846F.0	fBig5	DFD3
U+846F.0	fGB	1-7877
U+846F.0	fR/S	 140.9
U+846F.0	fTotalStrokes	 13; 15
U+846F.0	fCangjie	TVFI
U+846F.0	fFourCorner	44927
U+846F.0	fHakka	  jok8; rok8
U+846F.0	fCantonese	joek6; joek3; joek1; zau6
U+846F.0	fMandarin	yao4; yue4; di2
U+846F.0	fEnglish	[1] angelica [2] wrap up
U+8470.0	fUTF8	葰
U+8470.0	fCNS11643	2-4673
U+8470.0	fBig5	DFCE
U+8470.0	fGB	5-687D
U+8470.0	fR/S	 140.9
U+8470.0	fTotalStrokes	 13; 15
U+8470.0	fCangjie	TOIE
U+8470.0	fFourCorner	44243
U+8470.0	fCantonese	seoi1; zeon3
U+8470.0	fMandarin	jun4; sui1; suo3
U+8471.0	fUTF8	葱
U+8471.0	fCNS11643	3-443C
U+8471.0	fGB	0-3450
U+8471.0	fR/S	 140.9
U+8471.0	fTotalStrokes	 13; 15
U+8471.0	fCangjie	TPKP
U+8471.0	fFourCorner	44330
U+8471.0	fHakka	  cung1
U+8471.0	fCantonese	cung1
U+8471.0	fMandarin	cong1; chuang1
U+8471.0	fEnglish	[1] scallions; onions [2] bright green
U+8472.0	fUTF8	葲
U+8472.0	fCNS11643	3-442F
U+8472.0	fGB	E-3A75
U+8472.0	fR/S	 140.9
U+8472.0	fTotalStrokes	 13; 15
U+8472.0	fCangjie	THAE
U+8472.0	fFourCorner	44230
U+8472.0	fCantonese	cyun4
U+8472.0	fMandarin	quan2
U+8473.0	fUTF8	葳
U+8473.0	fCNS11643	2-4660
U+8473.0	fBig5	DFBB
U+8473.0	fGB	0-5D5A
U+8473.0	fR/S	 140.9
U+8473.0	fTotalStrokes	 13; 15
U+8473.0	fCangjie	TIHV
U+8473.0	fFourCorner	44253
U+8473.0	fHakka	 wi1; wui1; vui1
U+8473.0	fCantonese	wai1
U+8473.0	fMandarin	wei1
U+8473.0	fEnglish	[1] luxuriant; flourishing [2] used for various plants
U+8474.0	fUTF8	葴
U+8474.0	fCNS11643	2-465F
U+8474.0	fBig5	DFBA
U+8474.0	fGB	3-696A
U+8474.0	fR/S	 140.9
U+8474.0	fTotalStrokes	 13; 15
U+8474.0	fCangjie	TIHR
U+8474.0	fFourCorner	44253
U+8474.0	fHakka	 zhim1; zihm1; zim1; ham2
U+8474.0	fCantonese	zam1
U+8474.0	fMandarin	xian2; zhen1; qian2; jian1
U+8475.0	fUTF8	葵
U+8475.0	fCNS11643	1-6631
U+8475.0	fBig5	B8AA
U+8475.0	fGB	0-3F7B
U+8475.0	fR/S	 140.9
U+8475.0	fTotalStrokes	 13; 15
U+8475.0	fCangjie	TNOK
U+8475.0	fFourCorner	44430
U+8475.0	fHakka	 kui2; kui3; kwi2; k'wui2
U+8475.0	fCantonese	kwai4; kwai2 (also)
U+8475.0	fMandarin	kui2
U+8475.0	fEnglish	sunflower (Althaea rosea)
U+8476.0	fUTF8	葶
U+8476.0	fCNS11643	2-4651
U+8476.0	fBig5	DFAC
U+8476.0	fGB	0-5D63
U+8476.0	fR/S	 140.9
U+8476.0	fTotalStrokes	 13; 15
U+8476.0	fCangjie	TYRN
U+8476.0	fFourCorner	44201
U+8476.0	fHakka	  tin2
U+8476.0	fCantonese	ting4
U+8476.0	fMandarin	ting2; ding3
U+8476.0	fEnglish	name of a plant (said to poison fish)
U+8477.0	fUTF8	葷
U+8477.0	fCNS11643	1-662E
U+8477.0	fBig5	B8A7
U+8477.0	fGB	1-3B67
U+8477.0	fR/S	 140.9
U+8477.0	fTotalStrokes	 13; 15
U+8477.0	fCangjie	TBJJ
U+8477.0	fFourCorner	44506
U+8477.0	fHakka	  fun1; fun2
U+8477.0	fCantonese	fan1
U+8477.0	fMandarin	hun1
U+8477.0	fEnglish	[1] meat and fish diet; meat-eating (as opposed too vegetarian) [2] strong smelling foods or spices (such as onions, leeks, garlic, etc.) [3] obscene or dirty language, narration, films, etc.
U+8477.1	fUTF8	葷
U+8477.1	fCNS11643	1-662E
U+8477.1	fBig5	B8A7
U+8477.1	fGB	1-3B67
U+8477.1	fR/S	 140.9
U+8477.1	fTotalStrokes	 13; 15
U+8477.1	fCangjie	TBJJ
U+8477.1	fFourCorner	44506
U+8477.1	fMandarin	xun1
U+8477.1	fEnglish	name of a barbaric tribe in ancient China
U+8478.0	fUTF8	葸
U+8478.0	fCNS11643	2-4669
U+8478.0	fBig5	DFC4
U+8478.0	fGB	0-5D5F
U+8478.0	fR/S	 140.9
U+8478.0	fTotalStrokes	 13; 15
U+8478.0	fCangjie	TWP
U+8478.0	fFourCorner	44336
U+8478.0	fHakka	  sai3;  sii1; sau1; sii3
U+8478.0	fCantonese	saai2
U+8478.0	fMandarin	xi3
U+8478.0	fEnglish	[1] scared; timid; afraid [2] not pleasant; displease look
U+8479.0	fUTF8	葹
U+8479.0	fCNS11643	2-4652
U+8479.0	fBig5	DFAD
U+8479.0	fGB	3-697E
U+8479.0	fR/S	 140.9
U+8479.0	fTotalStrokes	 13; 15
U+8479.0	fCangjie	TYSD
U+8479.0	fFourCorner	44212
U+8479.0	fCantonese	si1
U+8479.0	fMandarin	shi1
U+8479.0	fEnglish	kind of chrysanthemum; xanthium
U+847A.0	fUTF8	葺
U+847A.0	fCNS11643	2-4667
U+847A.0	fBig5	DFC2
U+847A.0	fGB	0-5D5D
U+847A.0	fR/S	 140.9
U+847A.0	fTotalStrokes	 13; 15
U+847A.0	fCangjie	TRSJ
U+847A.0	fFourCorner	44401
U+847A.0	fHakka	 sip7;  cip8; sip8
U+847A.0	fCantonese	cap1
U+847A.0	fMandarin	qi4; ji2
U+847A.0	fEnglish	[1] thatched [2] fix; repair; have something put in repair [3] pile up; heap together
U+847B.0	fUTF8	葻
U+847B.0	fCNS11643	3-442E
U+847B.0	fGB	3-6841
U+847B.0	fR/S	  140.9
U+847B.0	fTotalStrokes	 13; 15
U+847B.0	fCangjie	XTHNI
U+847B.0	fFourCorner	44217
U+847B.0	fCantonese	laam4
U+847B.0	fMandarin	lan2
U+847C.0	fUTF8	葼
U+847C.0	fCNS11643	4-4041
U+847C.0	fGB	3-6979
U+847C.0	fR/S	  140.9
U+847C.0	fTotalStrokes	 13; 15
U+847C.0	fCangjie	TUCE
U+847C.0	fFourCorner	44400
U+847C.0	fHakka	  zung1
U+847C.0	fCantonese	zung1
U+847C.0	fMandarin	zong1
U+847C.0	fEnglish	[1] small twigs [2] a kind of grass
U+847D.0	fUTF8	葽
U+847D.0	fCNS11643	2-465C
U+847D.0	fBig5	DFB7
U+847D.0	fGB	3-6968
U+847D.0	fR/S	 140.9
U+847D.0	fTotalStrokes	 13; 15
U+847D.0	fCangjie	TMWV
U+847D.0	fFourCorner	44404
U+847D.0	fHakka	 rau1; jeu1; jau1
U+847D.0	fCantonese	jiu1
U+847D.0	fMandarin	yao1; yao3; yao4
U+847D.0	fEnglish	[1] a type of grass (Polygala japonica) [2] a dense or luxuriant growth of grass or weeds
U+847E.0	fUTF8	葾
U+847E.0	fCNS11643	2-4722
U+847E.0	fBig5	DFDB
U+847E.0	fGB	E-3A76
U+847E.0	fR/S	 140.9
U+847E.0	fTotalStrokes	 13; 15
U+847E.0	fCangjie	TNUP
U+847E.0	fFourCorner	44331
U+847E.0	fCantonese	jyun1
U+847E.0	fMandarin	yuan1
U+847E.0	fEnglish	a bad smell
U+847F.0	fUTF8	葿
U+847F.0	fCNS11643	3-442A
U+847F.0	fGB	3-6A2A
U+847F.0	fR/S	 140.9
U+847F.0	fTotalStrokes	 13; 15
U+847F.0	fCangjie	TAHU
U+847F.0	fFourCorner	44260
U+847F.0	fCantonese	mei4
U+847F.0	fMandarin	mei2
U+8480.0	fUTF8	蒀
U+8480.0	fCNS11643	3-4439
U+8480.0	fGB	3-696E
U+8480.0	fR/S	 140.9
U+8480.0	fTotalStrokes	 13; 15
U+8480.0	fCangjie	TABT
U+8480.0	fFourCorner	44100
U+8480.0	fCantonese	wan1
U+8480.0	fMandarin	yun1; wen1
U+8481.0	fUTF8	蒁
U+8481.0	fCNS11643	4-4036
U+8481.0	fGB	E-3A77
U+8481.0	fR/S	  140.9
U+8481.0	fTotalStrokes	 13; 15
U+8481.0	fCangjie	YTID
U+8481.0	fFourCorner	44300
U+8481.0	fCantonese	wat6
U+8481.0	fMandarin	shu4
U+8482.0	fUTF8	蒂
U+8482.0	fCNS11643	1-662D
U+8482.0	fBig5	B8A6
U+8482.0	fGB	0-3559
U+8482.0	fR/S	 140.9
U+8482.0	fTotalStrokes	 13; 15
U+8482.0	fCangjie	TYBB
U+8482.0	fFourCorner	44227
U+8482.0	fHakka	 di5;  ni5; li5; {ni5}
U+8482.0	fCantonese	dai3; ding3
U+8482.0	fMandarin	di4; chai4; dai4
U+8482.0	fEnglish	[1] peduncle; stem of plants; footstalk of a flower or fruit; base [2] (cigarette) butt
U+8483.0	fUTF8	蒃
U+8483.0	fCNS11643	3-443B
U+8483.0	fGB	E-3A78
U+8483.0	fR/S	 140.9
U+8483.0	fTotalStrokes	 13; 15
U+8483.0	fCangjie	TVNO
U+8483.0	fFourCorner	44230
U+8483.0	fCantonese	syun6
U+8483.0	fMandarin	zhuan4
U+8484.0	fUTF8	蒄
U+8484.0	fCNS11643	3-4428
U+8484.0	fGB	E-3A79
U+8484.0	fR/S	  140.9
U+8484.0	fTotalStrokes	 13; 15
U+8484.0	fCangjie	TBMI
U+8484.0	fFourCorner	44210
U+8484.0	fCantonese	gun1; gwun1 (also)
U+8484.0	fMandarin	guan1
U+8485.0	fUTF8	蒅
U+8485.0	fGB	E-3A7A
U+8485.0	fR/S	 140.9
U+8485.0	fTotalStrokes	 13; 15
U+8485.0	fCangjie	TEND
U+8485.0	fCantonese	jim5
U+8486.0	fUTF8	蒆
U+8486.0	fCNS11643	2-4658
U+8486.0	fBig5	DFB3
U+8486.0	fGB	E-3A7B
U+8486.0	fR/S	 140.9
U+8486.0	fTotalStrokes	 13; 15
U+8486.0	fCangjie	TMNV
U+8486.0	fFourCorner	44404
U+8486.0	fCantonese	zuk1; hoe1
U+8486.0	fMandarin	qiong2; xue1
U+8487.0	fUTF8	蒇
U+8487.0	fGB	0-5D5B
U+8487.0	fR/S	 140.9
U+8487.0	fTotalStrokes	 13; 15
U+8487.0	fCangjie	TIHO
U+8487.0	fHakka	 zhen3
U+8487.0	fCantonese	cin2; zin2 (also)
U+8487.0	fMandarin	chan3
U+8487.0	fEnglish	simplified form of &#x8546;
U+8488.0	fUTF8	蒈
U+8488.0	fCNS11643	4-4046
U+8488.0	fGB	0-5D5C
U+8488.0	fR/S	  140.9
U+8488.0	fTotalStrokes	 13; 15
U+8488.0	fCangjie	TPPA
U+8488.0	fFourCorner	44600
U+8488.0	fCantonese	kaai2
U+8488.0	fMandarin	kai3
U+8488.0	fEnglish	an organic compound
U+8489.0	fUTF8	蒉
U+8489.0	fGB	0-5D5E
U+8489.0	fR/S	 140.9
U+8489.0	fTotalStrokes	 13; 15
U+8489.0	fCangjie	TLMO
U+8489.0	fHakka	 k'wui5
U+8489.0	fCantonese	gwai6
U+8489.0	fMandarin	kui4; kuai4
U+8489.0	fEnglish	simplified form of &#x8562;
U+848A.0	fUTF8	蒊
U+848A.0	fGB	1-7D27
U+848A.0	fR/S	 140.9; 21.11
U+848A.0	fTotalStrokes	 13; 15
U+848A.0	fCangjie	TOPP
U+848A.0	fCantonese	faa1
U+848B.0	fUTF8	蒋
U+848B.0	fGB	0-3D2F
U+848B.0	fR/S	 140.9; 140.10
U+848B.0	fTotalStrokes	 13; 15
U+848B.0	fCangjie	TLMI
U+848B.0	fHakka	 ziong3
U+848B.0	fCantonese	zoeng2; zoeng1
U+848B.0	fMandarin	jiang3; jiang1
U+848B.0	fEnglish	simplified form of &#x8523;
U+848C.0	fUTF8	蒌
U+848C.0	fGB	0-5D64
U+848C.0	fR/S	 140.9
U+848C.0	fTotalStrokes	 13; 15
U+848C.0	fCangjie	TFDV
U+848C.0	fHakka	 leu2
U+848C.0	fCantonese	lau4; lau1 (also); lau5
U+848C.0	fMandarin	lou2; liu3; ju4; liu2; lou3
U+848C.0	fEnglish	simplified form of &#x851E;
U+848D.0	fUTF8	蒍
U+848D.0	fCNS11643	2-4654
U+848D.0	fBig5	DFAF
U+848D.0	fGB	E-3A7C
U+848D.0	fR/S	 140.9
U+848D.0	fTotalStrokes	 13; 15
U+848D.0	fCangjie	TIKF
U+848D.0	fFourCorner	44327
U+848D.0	fCantonese	wai2; wai5
U+848D.0	fMandarin	wei3; wei2; e2; hua1; kui1
U+848E.0	fUTF8	蒎
U+848E.0	fCNS11643	2-467A
U+848E.0	fBig5	DFD5
U+848E.0	fGB	0-5D65
U+848E.0	fR/S	 140.9
U+848E.0	fTotalStrokes	 13; 15
U+848E.0	fCangjie	TEHV
U+848E.0	fFourCorner	44132
U+848E.0	fCantonese	paai3
U+848E.0	fMandarin	pai4
U+848F.0	fUTF8	蒏
U+848F.0	fCNS11643	2-4653
U+848F.0	fBig5	DFAE
U+848F.0	fGB	E-3A7D
U+848F.0	fR/S	 140.9
U+848F.0	fTotalStrokes	 13; 15
U+848F.0	fCangjie	TBMW
U+848F.0	fFourCorner	44604
U+848F.0	fCantonese	wing6
U+8490.0	fUTF8	蒐
U+8490.0	fCNS11643	1-6A68
U+8490.0	fBig5	BB60
U+8490.0	fGB	E-3A7E
U+8490.0	fR/S	 140.10
U+8490.0	fTotalStrokes	 14; 16
U+8490.0	fCangjie	THUI
U+8490.0	fFourCorner	44213
U+8490.0	fHakka	  seu1; siu1
U+8490.0	fCantonese	sau1; sau2 (also)
U+8490.0	fMandarin	sou1; hui4; sou4
U+8490.0	fEnglish	[1] collect; gather; assemble [2] seek; hunt; hunting [3] Rudia cordifolia (botany); madder
U+8491.0	fUTF8	蒑
U+8491.0	fCNS11643	2-4D5A
U+8491.0	fBig5	E3D3
U+8491.0	fGB	5-6944
U+8491.0	fR/S	 140.10
U+8491.0	fTotalStrokes	 14; 16
U+8491.0	fCangjie	THSE
U+8491.0	fFourCorner	44247
U+8491.0	fCantonese	jan1
U+8491.0	fMandarin	yin1
U+8492.0	fUTF8	蒒
U+8492.0	fCNS11643	3-4935
U+8492.0	fGB	3-686B
U+8492.0	fR/S	 140.10
U+8492.0	fTotalStrokes	 14; 16
U+8492.0	fCangjie	THRB
U+8492.0	fFourCorner	44720
U+8492.0	fHakka	 sii1
U+8492.0	fCantonese	si1
U+8492.0	fMandarin	shi1
U+8492.0	fEnglish	a floating marine plant with edible seeds
U+8493.0	fUTF8	蒓
U+8493.0	fCNS11643	3-4944
U+8493.0	fGB	1-5D3B
U+8493.0	fR/S	 140.10
U+8493.0	fTotalStrokes	 14; 16
U+8493.0	fCangjie	TVFU
U+8493.0	fFourCorner	44910
U+8493.0	fHakka	 sun2; shun2
U+8493.0	fCantonese	seon4
U+8493.0	fMandarin	chun2
U+8493.0	fEnglish	Brasenia schreberi, water shield
U+8494.0	fUTF8	蒔
U+8494.0	fCNS11643	2-4D49
U+8494.0	fBig5	E3C2
U+8494.0	fGB	1-5D2A
U+8494.0	fR/S	 140.10
U+8494.0	fTotalStrokes	 14; 16
U+8494.0	fCangjie	TAGI
U+8494.0	fFourCorner	44641
U+8494.0	fHakka	 shi6; si5; sii5; shi3; si3; she5; shi5
U+8494.0	fCantonese	si4; si6; si2 (also)
U+8494.0	fMandarin	shi4; shi2
U+8494.0	fEnglish	[1] transplant; plant; cultivate [2] Anethum graveolen, a kind of spice
U+8495.0	fUTF8	蒕
U+8495.0	fCNS11643	3-4933
U+8495.0	fGB	E-3B21
U+8495.0	fR/S	  140.10
U+8495.0	fTotalStrokes	 14; 16
U+8495.0	fCangjie	TWOT
U+8495.0	fFourCorner	44100
U+8495.0	fCantonese	wan1
U+8495.0	fMandarin	yun1
U+8496.0	fUTF8	蒖
U+8496.0	fCNS11643	3-4938
U+8496.0	fGB	3-6A31
U+8496.0	fR/S	 140.10
U+8496.0	fTotalStrokes	 14; 16
U+8496.0	fCangjie	TJBC
U+8496.0	fFourCorner	44800
U+8496.0	fCantonese	zan1
U+8496.0	fMandarin	zhen1
U+8496.0	fEnglish	plant name
U+8497.0	fUTF8	蒗
U+8497.0	fCNS11643	2-4D33
U+8497.0	fBig5	E3AC
U+8497.0	fGB	0-5D75
U+8497.0	fR/S	 140.10
U+8497.0	fTotalStrokes	 14; 16
U+8497.0	fCangjie	TEIV
U+8497.0	fFourCorner	44132
U+8497.0	fHakka	 long3
U+8497.0	fCantonese	long6; long5
U+8497.0	fMandarin	lang4
U+8498.0	fUTF8	蒘
U+8498.0	fCNS11643	2-4D51
U+8498.0	fBig5	E3CA
U+8498.0	fGB	E-3B22
U+8498.0	fR/S	 140.10
U+8498.0	fTotalStrokes	 14; 16
U+8498.0	fCangjie	TVRQ
U+8498.0	fFourCorner	44502
U+8498.0	fCantonese	neoi4; leoi4 (also); jyu4; naa4
U+8498.0	fMandarin	nu2; na2
U+8499.0	fUTF8	蒙
U+8499.0	fCNS11643	1-6A60
U+8499.0	fBig5	BB58
U+8499.0	fGB	0-4349
U+8499.0	fR/S	 140.10
U+8499.0	fTotalStrokes	 14; 16
U+8499.0	fCangjie	TBMO
U+8499.0	fFourCorner	44232
U+8499.0	fHakka	  mung2; mung3
U+8499.0	fCantonese	mung4; mung2; mung1 (also)
U+8499.0	fMandarin	meng2; mang2; mou2
U+8499.0	fEnglish	[1] cover; cover up; wrap [2] naive; childish [3] ignorant; gullible; stupid [4] cheat; deceive; fool [5] suffer; bear; take [6] short for Mongolia [7] a Chinese family name
U+8499.1	fUTF8	蒙
U+8499.1	fCNS11643	1-6A60
U+8499.1	fBig5	BB58
U+8499.1	fGB	0-4349
U+8499.1	fR/S	 140.10
U+8499.1	fTotalStrokes	 14; 16
U+8499.1	fCangjie	TBMO
U+8499.1	fFourCorner	44232
U+8499.1	fMandarin	meng1
U+8499.2	fUTF8	蒙
U+8499.2	fCNS11643	1-6A60
U+8499.2	fBig5	BB58
U+8499.2	fGB	0-4349
U+8499.2	fR/S	 140.10
U+8499.2	fTotalStrokes	 14; 16
U+8499.2	fCangjie	TBMO
U+8499.2	fFourCorner	44232
U+8499.2	fMandarin	meng3
U+849A.0	fUTF8	蒚
U+849A.0	fCNS11643	2-4D42
U+849A.0	fBig5	E3BB
U+849A.0	fGB	5-693A
U+849A.0	fR/S	 140.10
U+849A.0	fTotalStrokes	 14; 16
U+849A.0	fCangjie	TMRB
U+849A.0	fFourCorner	44227
U+849A.0	fCantonese	lik6; hat6
U+849A.0	fMandarin	he2; li4
U+849B.0	fUTF8	蒛
U+849B.0	fCNS11643	2-4D4C
U+849B.0	fBig5	E3C5
U+849B.0	fGB	E-3B23
U+849B.0	fR/S	 140.10
U+849B.0	fTotalStrokes	 14; 16
U+849B.0	fCangjie	TOUK
U+849B.0	fFourCorner	44735
U+849B.0	fCantonese	kyut3
U+849B.0	fMandarin	que1
U+849C.0	fUTF8	蒜
U+849C.0	fCNS11643	1-6A63
U+849C.0	fBig5	BB5B
U+849C.0	fGB	0-4B62
U+849C.0	fR/S	 140.10
U+849C.0	fTotalStrokes	 14; 16
U+849C.0	fCangjie	TMFF
U+849C.0	fFourCorner	44991
U+849C.0	fHakka	  son5
U+849C.0	fCantonese	syun3
U+849C.0	fMandarin	suan4
U+849C.0	fEnglish	garlic
U+849D.0	fUTF8	蒝
U+849D.0	fCNS11643	2-4D45
U+849D.0	fBig5	E3BE
U+849D.0	fGB	3-6A33
U+849D.0	fR/S	 140.10
U+849D.0	fTotalStrokes	 14; 16
U+849D.0	fCangjie	TMHF
U+849D.0	fFourCorner	44296
U+849D.0	fCantonese	jyun4; wun4
U+849D.0	fMandarin	yuan2; huan2; quan4
U+849E.0	fUTF8	蒞
U+849E.0	fCNS11643	1-6A61
U+849E.0	fBig5	BB59
U+849E.0	fGB	8-2D5F
U+849E.0	fR/S	 140.10
U+849E.0	fTotalStrokes	 14; 16
U+849E.0	fCangjie	TEOT
U+849E.0	fFourCorner	44218
U+849E.0	fHakka	  li5; li3
U+849E.0	fCantonese	lei6
U+849E.0	fMandarin	li4
U+849E.0	fEnglish	 reach; arrive
U+849F.0	fUTF8	蒟
U+849F.0	fCNS11643	2-4D36
U+849F.0	fBig5	E3AF
U+849F.0	fGB	3-6A46
U+849F.0	fR/S	 140.10
U+849F.0	fTotalStrokes	 14; 16
U+849F.0	fCangjie	TYTR
U+849F.0	fFourCorner	44127
U+849F.0	fHakka	 gi3; neu1; {neu1}; {leu1}
U+849F.0	fCantonese	geoi2; lau1
U+849F.0	fMandarin	ju3; qu2
U+849F.0	fEnglish	 betel pepper Amorphaphalus konj
U+84A0.0	fUTF8	蒠
U+84A0.0	fCNS11643	2-4D54
U+84A0.0	fBig5	E3CD
U+84A0.0	fGB	3-6A3B
U+84A0.0	fR/S	 140.10
U+84A0.0	fTotalStrokes	 14; 16
U+84A0.0	fCangjie	THUP
U+84A0.0	fFourCorner	44336
U+84A0.0	fCantonese	sik1
U+84A0.0	fMandarin	xi2; xi1
U+84A1.0	fUTF8	蒡
U+84A1.0	fCNS11643	2-4D35
U+84A1.0	fBig5	E3AE
U+84A1.0	fGB	0-5D72
U+84A1.0	fR/S	 140.10
U+84A1.0	fTotalStrokes	 14; 16
U+84A1.0	fCangjie	TYBS
U+84A1.0	fFourCorner	44227
U+84A1.0	fHakka	 pong2
U+84A1.0	fCantonese	bong2; bong6 (also); pong4; pang4
U+84A1.0	fMandarin	bang4; pang2
U+84A1.0	fEnglish	burdock; herb
U+84A2.0	fUTF8	蒢
U+84A2.0	fCNS11643	2-4D48
U+84A2.0	fBig5	E3C1
U+84A2.0	fGB	3-6A2B
U+84A2.0	fR/S	 140.10
U+84A2.0	fTotalStrokes	 14; 16
U+84A2.0	fCangjie	TNLD
U+84A2.0	fFourCorner	44294
U+84A2.0	fHakka	 chu2; cu2
U+84A2.0	fCantonese	ceoi4
U+84A2.0	fMandarin	chu2
U+84A2.0	fEnglish	a small medicinal plant (Hedysarum brachyterum)
U+84A3.0	fUTF8	蒣
U+84A3.0	fCNS11643	3-493C
U+84A3.0	fGB	5-6943
U+84A3.0	fR/S	 140.10
U+84A3.0	fTotalStrokes	 14; 16
U+84A3.0	fCangjie	THOD
U+84A3.0	fFourCorner	44290
U+84A3.0	fCantonese	ceoi4
U+84A3.0	fMandarin	xu2
U+84A4.0	fUTF8	蒤
U+84A4.0	fCNS11643	2-4D34
U+84A4.0	fBig5	E3AD
U+84A4.0	fGB	5-694D
U+84A4.0	fR/S	 140.10
U+84A4.0	fTotalStrokes	 14; 16
U+84A4.0	fCangjie	TEOD
U+84A4.0	fFourCorner	44191
U+84A4.0	fCantonese	tou4
U+84A4.0	fMandarin	tu2
U+84A5.0	fUTF8	蒥
U+84A5.0	fCNS11643	3-4939
U+84A5.0	fGB	3-6A41
U+84A5.0	fR/S	  140.10
U+84A5.0	fTotalStrokes	 14; 16
U+84A5.0	fCangjie	THHW
U+84A5.0	fFourCorner	44600
U+84A5.0	fCantonese	lau4
U+84A5.0	fMandarin	liu2
U+84A6.0	fUTF8	蒦
U+84A6.0	fCNS11643	4-466C
U+84A6.0	fGB	3-6A3A
U+84A6.0	fR/S	  140.10
U+84A6.0	fTotalStrokes	 14; 16
U+84A6.0	fCangjie	TOGE
U+84A6.0	fFourCorner	44400
U+84A6.0	fHakka	 wok7; vok7
U+84A6.0	fCantonese	wok6
U+84A6.0	fMandarin	wo4; huo4; yue1
U+84A6.0	fEnglish	 measure calculate
U+84A7.0	fUTF8	蒧
U+84A7.0	fCNS11643	2-4D46
U+84A7.0	fBig5	E3BF
U+84A7.0	fGB	5-693C
U+84A7.0	fR/S	 140.10
U+84A7.0	fTotalStrokes	 14; 16
U+84A7.0	fCangjie	TIHR
U+84A7.0	fFourCorner	44253
U+84A7.0	fHakka	 diam3
U+84A7.0	fCantonese	dim2
U+84A7.0	fMandarin	zhen1; dian3
U+84A7.0	fEnglish	name of a herb
U+84A8.0	fUTF8	蒨
U+84A8.0	fCNS11643	2-4D4F
U+84A8.0	fBig5	E3C8
U+84A8.0	fGB	3-6A38
U+84A8.0	fR/S	 140.10
U+84A8.0	fTotalStrokes	 14; 16
U+84A8.0	fCangjie	TOQB
U+84A8.0	fFourCorner	44227
U+84A8.0	fHakka	  cien5; cin5; ziang1; cen3; cen5
U+84A8.0	fCantonese	sin6; sin3
U+84A8.0	fMandarin	qian4
U+84A8.0	fEnglish	[1] lush vegetation; luxuriant growth (grass, vegetation) [2] red; crimson
U+84A9.0	fUTF8	蒩
U+84A9.0	fCNS11643	2-4D4D
U+84A9.0	fBig5	E3C6
U+84A9.0	fGB	5-6940
U+84A9.0	fR/S	 140.10
U+84A9.0	fTotalStrokes	 14; 16
U+84A9.0	fCangjie	THDM
U+84A9.0	fFourCorner	44917
U+84A9.0	fCantonese	zou1; zou2
U+84A9.0	fMandarin	zu1; jie4; ju1; ju3; ju4; zha4; zu3
U+84AA.0	fUTF8	蒪
U+84AA.0	fCNS11643	2-4D41
U+84AA.0	fBig5	E3BA
U+84AA.0	fGB	3-6A32
U+84AA.0	fR/S	 140.10
U+84AA.0	fTotalStrokes	 14; 16
U+84AA.0	fCangjie	TIBI
U+84AA.0	fFourCorner	44346
U+84AA.0	fCantonese	pok3
U+84AA.0	fMandarin	po4
U+84AB.0	fUTF8	蒫
U+84AB.0	fCNS11643	2-4D3C
U+84AB.0	fBig5	E3B5
U+84AB.0	fGB	5-692B
U+84AB.0	fR/S	 140.10
U+84AB.0	fTotalStrokes	 14; 16
U+84AB.0	fCangjie	TTQM
U+84AB.0	fFourCorner	44510
U+84AB.0	fCantonese	co4; ze1; zi1
U+84AB.0	fMandarin	cuo1; jie1; zi1
U+84AC.0	fUTF8	蒬
U+84AC.0	fCNS11643	2-4D3A
U+84AC.0	fBig5	E3B3
U+84AC.0	fGB	5-6952
U+84AC.0	fR/S	 140.10
U+84AC.0	fTotalStrokes	 14; 16
U+84AC.0	fCangjie	TBNI
U+84AC.0	fFourCorner	44413
U+84AC.0	fCantonese	jyun1
U+84AC.0	fMandarin	yuan1; yuan3
U+84AD.0	fUTF8	蒭
U+84AD.0	fCNS11643	3-4942
U+84AD.0	fGB	5-6764
U+84AD.0	fR/S	  140.10
U+84AD.0	fTotalStrokes	 14; 16
U+84AD.0	fCangjie	TPUU
U+84AD.0	fFourCorner	44420
U+84AD.0	fHakka	 cu1; ze1; cih1; ci1
U+84AD.0	fCantonese	co1
U+84AD.0	fMandarin	chu2
U+84AD.0	fEnglish	[1] cut grass [2] hay; fodder
U+84AE.0	fUTF8	蒮
U+84AE.0	fCNS11643	2-4D3B
U+84AE.0	fBig5	E3B4
U+84AE.0	fGB	5-6951
U+84AE.0	fR/S	 140.10
U+84AE.0	fTotalStrokes	 14; 16
U+84AE.0	fCangjie	TOBG
U+84AE.0	fFourCorner	44214
U+84AE.0	fCantonese	juk6
U+84AE.0	fMandarin	yu4
U+84AF.0	fUTF8	蒯
U+84AF.0	fCNS11643	2-4D4E
U+84AF.0	fBig5	E3C7
U+84AF.0	fGB	0-5861
U+84AF.0	fR/S	 140.10
U+84AF.0	fTotalStrokes	 14; 16
U+84AF.0	fCangjie	TBLN
U+84AF.0	fFourCorner	42200
U+84AF.0	fHakka	  kwai5
U+84AF.0	fCantonese	gwaai2; faai3
U+84AF.0	fMandarin	kuai3; kuai4
U+84AF.0	fEnglish	[1] Scirpus cyperinus Kunth var cocolor Makino, a rush, of which various things are weaved [2] a Chinese family name
U+84B0.0	fUTF8	蒰
U+84B0.0	fCNS11643	2-4D59
U+84B0.0	fBig5	E3D2
U+84B0.0	fGB	5-6945
U+84B0.0	fR/S	 140.10
U+84B0.0	fTotalStrokes	 14; 16
U+84B0.0	fCangjie	THYE
U+84B0.0	fFourCorner	44447
U+84B0.0	fCantonese	pun4
U+84B0.0	fMandarin	pan2
U+84B1.0	fUTF8	蒱
U+84B1.0	fCNS11643	2-4D43
U+84B1.0	fBig5	E3BC
U+84B1.0	fGB	5-693D
U+84B1.0	fR/S	 140.10
U+84B1.0	fTotalStrokes	 14; 16
U+84B1.0	fCangjie	TQIB
U+84B1.0	fFourCorner	44527
U+84B1.0	fHakka	  pu2; pu3
U+84B1.0	fCantonese	pou4; po4
U+84B1.0	fMandarin	pu2
U+84B1.0	fEnglish	a gambling game with dice
U+84B2.0	fUTF8	蒲
U+84B2.0	fCNS11643	1-6A62
U+84B2.0	fBig5	BB5A
U+84B2.0	fGB	0-4651
U+84B2.0	fR/S	 140.10
U+84B2.0	fTotalStrokes	 14; 16
U+84B2.0	fCangjie	TEIB
U+84B2.0	fFourCorner	44127
U+84B2.0	fHakka	  pu2; pu3
U+84B2.0	fCantonese	pou4; bok6
U+84B2.0	fMandarin	pu2; bo2; bu4
U+84B2.0	fEnglish	[1] various kinds of rush from which mats, bags, etc. are made; vines of the rushes [2] a Chinese family name
U+84B3.0	fUTF8	蒳
U+84B3.0	fCNS11643	3-493B
U+84B3.0	fGB	3-693D
U+84B3.0	fR/S	  140.10
U+84B3.0	fTotalStrokes	 14; 16
U+84B3.0	fCangjie	TVFB
U+84B3.0	fFourCorner	44920
U+84B3.0	fHakka	 nap8
U+84B3.0	fCantonese	naap6; laap6 (also)
U+84B3.0	fMandarin	na4
U+84B3.0	fEnglish	name of a fragrant plant
U+84B4.0	fUTF8	蒴
U+84B4.0	fCNS11643	2-4D3E
U+84B4.0	fBig5	E3B7
U+84B4.0	fGB	0-5D74
U+84B4.0	fR/S	 140.10
U+84B4.0	fTotalStrokes	 14; 16
U+84B4.0	fCangjie	TTUB
U+84B4.0	fFourCorner	44427
U+84B4.0	fCantonese	sok3
U+84B4.0	fMandarin	shuo4
U+84B4.0	fEnglish	seed (capsule)
U+84B5.0	fUTF8	蒵
U+84B5.0	fCNS11643	3-493A
U+84B5.0	fGB	3-6A40
U+84B5.0	fR/S	  140.10
U+84B5.0	fTotalStrokes	 14; 16
U+84B5.0	fCangjie	TBVK
U+84B5.0	fFourCorner	44430
U+84B5.0	fCantonese	hai4
U+84B5.0	fMandarin	xi1; xi2; xi4
U+84B6.0	fUTF8	蒶
U+84B6.0	fCNS11643	2-4D52
U+84B6.0	fBig5	E3CB
U+84B6.0	fGB	3-693E
U+84B6.0	fR/S	 140.10
U+84B6.0	fTotalStrokes	 14; 16
U+84B6.0	fCangjie	TVFH
U+84B6.0	fFourCorner	44927
U+84B6.0	fCantonese	fan4
U+84B6.0	fMandarin	fen2
U+84B7.0	fUTF8	蒷
U+84B7.0	fCNS11643	3-4941
U+84B7.0	fGB	5-6835
U+84B7.0	fR/S	  140.10
U+84B7.0	fTotalStrokes	 14; 16
U+84B7.0	fCangjie	TRBC
U+84B7.0	fFourCorner	44800
U+84B7.0	fHakka	 jun2
U+84B7.0	fCantonese	wan4
U+84B7.0	fMandarin	yun2
U+84B8.0	fUTF8	蒸
U+84B8.0	fCNS11643	1-6A65
U+84B8.0	fBig5	BB5D
U+84B8.0	fGB	0-5574
U+84B8.0	fR/S	 140.10
U+84B8.0	fTotalStrokes	 14; 16
U+84B8.0	fCangjie	TNEF
U+84B8.0	fFourCorner	44331
U+84B8.0	fHakka	 zin1;  zhin1; zihn1
U+84B8.0	fCantonese	zing1
U+84B8.0	fMandarin	zheng1; zheng4
U+84B8.0	fEnglish	[1] steam; cook by steaming [2] evaporate [3] twigs or slender branches as fuel
U+84B9.0	fUTF8	蒹
U+84B9.0	fCNS11643	2-4D3D
U+84B9.0	fBig5	E3B6
U+84B9.0	fGB	0-5D73
U+84B9.0	fR/S	 140.10
U+84B9.0	fTotalStrokes	 14; 16
U+84B9.0	fCangjie	TTXC
U+84B9.0	fFourCorner	44237
U+84B9.0	fHakka	 giam1
U+84B9.0	fCantonese	gim1
U+84B9.0	fMandarin	jian1
U+84B9.0	fEnglish	a kind of reed with a pithy stem
U+84BA.0	fUTF8	蒺
U+84BA.0	fCNS11643	2-4D37
U+84BA.0	fBig5	E3B0
U+84BA.0	fGB	0-5D70
U+84BA.0	fR/S	 140.10
U+84BA.0	fTotalStrokes	 14; 16
U+84BA.0	fCangjie	TKOK
U+84BA.0	fFourCorner	44134
U+84BA.0	fHakka	  cit8
U+84BA.0	fCantonese	zat6; zaat6
U+84BA.0	fMandarin	ji2
U+84BA.0	fEnglish	 furze gorse
U+84BB.0	fUTF8	蒻
U+84BB.0	fCNS11643	2-4D47
U+84BB.0	fBig5	E3C0
U+84BB.0	fGB	3-6A4D
U+84BB.0	fR/S	 140.10
U+84BB.0	fTotalStrokes	 14; 16
U+84BB.0	fCangjie	TNMM
U+84BB.0	fFourCorner	44227
U+84BB.0	fHakka	 ngiok8; niok8
U+84BB.0	fCantonese	joek6
U+84BB.0	fMandarin	ruo4; nuo4
U+84BB.0	fEnglish	[1] water plant; wid arum [2] rush mat
U+84BC.0	fUTF8	蒼
U+84BC.0	fCNS11643	1-6A69
U+84BC.0	fBig5	BB61
U+84BC.0	fGB	1-3254
U+84BC.0	fR/S	 140.10
U+84BC.0	fTotalStrokes	 14; 16
U+84BC.0	fCangjie	TOIR
U+84BC.0	fFourCorner	44607
U+84BC.0	fHakka	  cong1; zong1
U+84BC.0	fCantonese	cong1; cong2
U+84BC.0	fMandarin	cang1; cang3
U+84BC.0	fEnglish	[1] green; deep blue; deep green [2] gray (hair); hoary [3] old [4] the masses [5] a Chinese family name
U+84BD.0	fUTF8	蒽
U+84BD.0	fCNS11643	3-4932
U+84BD.0	fGB	0-5D6C
U+84BD.0	fR/S	 140.10
U+84BD.0	fTotalStrokes	 14; 16
U+84BD.0	fCangjie	TWKP
U+84BD.0	fFourCorner	44330
U+84BD.0	fCantonese	jan1
U+84BD.0	fMandarin	en1
U+84BD.0	fEnglish	anthracene
U+84BE.0	fUTF8	蒾
U+84BE.0	fCNS11643	4-4658
U+84BE.0	fGB	3-6A22
U+84BE.0	fR/S	  140.10
U+84BE.0	fTotalStrokes	 14; 16
U+84BE.0	fCangjie	TYFD
U+84BE.0	fFourCorner	44300
U+84BE.0	fHakka	 mi2
U+84BE.0	fCantonese	mai4
U+84BE.0	fMandarin	mi2
U+84BE.0	fEnglish	a name of a plant (Viburnum dilatatum)
U+84BF.0	fUTF8	蒿
U+84BF.0	fCNS11643	1-6A5D
U+84BF.0	fBig5	BB55
U+84BF.0	fGB	0-5D6F
U+84BF.0	fR/S	 140.10
U+84BF.0	fTotalStrokes	 14; 16
U+84BF.0	fCangjie	TYRB
U+84BF.0	fFourCorner	44227
U+84BF.0	fHakka	 hau1;  gau1;  ko1
U+84BF.0	fCantonese	hou1
U+84BF.0	fMandarin	hao1; gao3
U+84BF.0	fEnglish	[1] plants of the mugwort or artemisia family; the southernwood [2] rising vapor
U+84C0.0	fUTF8	蓀
U+84C0.0	fCNS11643	1-6A66
U+84C0.0	fBig5	BB5E
U+84C0.0	fGB	1-5D25
U+84C0.0	fR/S	 140.10
U+84C0.0	fTotalStrokes	 14; 16
U+84C0.0	fCangjie	TNDF
U+84C0.0	fFourCorner	44493
U+84C0.0	fHakka	 sun1
U+84C0.0	fCantonese	syun1
U+84C0.0	fMandarin	sun1
U+84C0.0	fEnglish	a species of aromatic grass
U+84C1.0	fUTF8	蓁
U+84C1.0	fCNS11643	2-4D3F
U+84C1.0	fBig5	E3B8
U+84C1.0	fGB	0-5D68
U+84C1.0	fR/S	 140.10
U+84C1.0	fTotalStrokes	 14; 16
U+84C1.0	fCangjie	TQKD
U+84C1.0	fFourCorner	44904
U+84C1.0	fHakka	 zin1; cin2
U+84C1.0	fCantonese	zeon1; ceon4
U+84C1.0	fMandarin	zhen1; qin2
U+84C1.0	fEnglish	[1] abundant; luxuriant vegetation [2] a wild pepper
U+84C2.0	fUTF8	蓂
U+84C2.0	fCNS11643	2-4D39
U+84C2.0	fBig5	E3B2
U+84C2.0	fGB	3-6A4C
U+84C2.0	fR/S	 140.10
U+84C2.0	fTotalStrokes	 14; 16
U+84C2.0	fCangjie	TBAC
U+84C2.0	fFourCorner	44800
U+84C2.0	fHakka	  men2
U+84C2.0	fCantonese	ming4; mik6; ming5
U+84C2.0	fMandarin	ming2; mi4
U+84C2.0	fEnglish	lucky place
U+84C3.0	fUTF8	蓃
U+84C3.0	fCNS11643	3-4936
U+84C3.0	fGB	E-3B24
U+84C3.0	fR/S	  140.10
U+84C3.0	fTotalStrokes	 14; 16
U+84C3.0	fCangjie	THXE
U+84C3.0	fFourCorner	44400
U+84C3.0	fCantonese	sau1
U+84C3.0	fMandarin	sou1; huo4
U+84C4.0	fUTF8	蓄
U+84C4.0	fCNS11643	1-6A5F
U+84C4.0	fBig5	BB57
U+84C4.0	fGB	0-506E
U+84C4.0	fR/S	 140.10
U+84C4.0	fTotalStrokes	 14; 16
U+84C4.0	fCangjie	TYVW
U+84C4.0	fFourCorner	44603
U+84C4.0	fHakka	  hiuk7;  kiuk7
U+84C4.0	fCantonese	cuk1
U+84C4.0	fMandarin	xu4; chu4
U+84C4.0	fEnglish	[1] collect; store; save (up); gather; reserve [2] cultivate (long hair or a beard); grow [3] raise; rear; breed [4] wait; expect
U+84C5.0	fUTF8	蓅
U+84C5.0	fCNS11643	2-4679
U+84C5.0	fBig5	DFD4
U+84C5.0	fGB	5-694E
U+84C5.0	fR/S	 140.10
U+84C5.0	fTotalStrokes	 14; 16
U+84C5.0	fCangjie	TEYU
U+84C5.0	fFourCorner	44113
U+84C5.0	fCantonese	lau4
U+84C5.0	fMandarin	liu2
U+84C6.0	fUTF8	蓆
U+84C6.0	fCNS11643	1-6A5E
U+84C6.0	fBig5	BB56
U+84C6.0	fGB	8-2F6E
U+84C6.0	fR/S	 140.10
U+84C6.0	fTotalStrokes	 14; 16
U+84C6.0	fCangjie	TITB
U+84C6.0	fFourCorner	44227
U+84C6.0	fHakka	  ciak8;  cit8; tshiak8
U+84C6.0	fCantonese	zik6; zek6 (also)
U+84C6.0	fMandarin	xi2
U+84C6.0	fEnglish	 straw mat; mat
U+84C7.0	fUTF8	蓇
U+84C7.0	fCNS11643	2-4D4A
U+84C7.0	fBig5	E3C3
U+84C7.0	fGB	3-6970
U+84C7.0	fR/S	 140.10
U+84C7.0	fTotalStrokes	 14; 16
U+84C7.0	fCangjie	TBBB
U+84C7.0	fFourCorner	44227
U+84C7.0	fCantonese	gwat1
U+84C7.0	fMandarin	gu1; gu3
U+84C7.0	fEnglish	 follicles (plants)
U+84C8.0	fUTF8	蓈
U+84C8.0	fCNS11643	3-4436
U+84C8.0	fGB	5-685E
U+84C8.0	fR/S	 140.10; 140.9
U+84C8.0	fTotalStrokes	 14; 13; 16
U+84C8.0	fCangjie	TIIL
U+84C8.0	fFourCorner	44720
U+84C8.0	fCantonese	long4
U+84C8.0	fMandarin	lang2
U+84C9.0	fUTF8	蓉
U+84C9.0	fCNS11643	1-6A5C
U+84C9.0	fBig5	BB54
U+84C9.0	fGB	0-4858
U+84C9.0	fR/S	 140.10
U+84C9.0	fTotalStrokes	 14; 16
U+84C9.0	fCangjie	TJCR
U+84C9.0	fFourCorner	44608
U+84C9.0	fHakka	 jung2; rung2; jung3
U+84C9.0	fCantonese	jung4
U+84C9.0	fMandarin	rong2; wang2
U+84C9.0	fEnglish	used for hibiscus
U+84CA.0	fUTF8	蓊
U+84CA.0	fCNS11643	1-6A6B
U+84CA.0	fBig5	BB63
U+84CA.0	fGB	0-5D6E
U+84CA.0	fR/S	 140.10
U+84CA.0	fTotalStrokes	 14; 16
U+84CA.0	fCangjie	TCIM
U+84CA.0	fFourCorner	44227
U+84CA.0	fHakka	 wung3; wung5; vung3
U+84CA.0	fCantonese	jung2
U+84CA.0	fMandarin	weng3; weng1; weng4
U+84CA.0	fEnglish	luxuriant vegetation; lush; flourishing (vegetation)
U+84CB.0	fUTF8	蓋
U+84CB.0	fCNS11643	1-6A64
U+84CB.0	fBig5	BB5C
U+84CB.0	fGB	1-3847
U+84CB.0	fR/S	 140.10
U+84CB.0	fTotalStrokes	 14; 16
U+84CB.0	fCangjie	TGIT
U+84CB.0	fFourCorner	44107
U+84CB.0	fHakka	  goi5;  hap7; gai5; hap8
U+84CB.0	fCantonese	goi3; koi3 (also); hap6 (also); gap3; hap1; kap6
U+84CB.0	fMandarin	gai4
U+84CB.0	fEnglish	[1] cover; hide [2] lid; covering [3] build; construct; erect [4] affix (seal) [5] now; then; but; because (an initial particle) [6] for; perhaps; possibly; about (a particle indicating doubt) [7] surpass; excel [8] brag; boast
U+84CB.1	fUTF8	蓋
U+84CB.1	fCNS11643	1-6A64
U+84CB.1	fBig5	BB5C
U+84CB.1	fGB	1-3847
U+84CB.1	fR/S	 140.10
U+84CB.1	fTotalStrokes	 14; 16
U+84CB.1	fCangjie	TGIT
U+84CB.1	fFourCorner	44107
U+84CB.1	fMandarin	ge3
U+84CB.1	fEnglish	a Chinese family name
U+84CB.2	fUTF8	蓋
U+84CB.2	fCNS11643	1-6A64
U+84CB.2	fBig5	BB5C
U+84CB.2	fGB	1-3847
U+84CB.2	fR/S	 140.10
U+84CB.2	fTotalStrokes	 14; 16
U+84CB.2	fCangjie	TGIT
U+84CB.2	fFourCorner	44107
U+84CB.2	fMandarin	he2
U+84CB.2	fEnglish	why not?
U+84CB.3	fUTF8	蓋
U+84CB.3	fCNS11643	1-6A64
U+84CB.3	fBig5	BB5C
U+84CB.3	fGB	1-3847
U+84CB.3	fR/S	 140.10
U+84CB.3	fTotalStrokes	 14; 16
U+84CB.3	fCangjie	TGIT
U+84CB.3	fFourCorner	44107
U+84CC.0	fUTF8	蓌
U+84CC.0	fCNS11643	2-4D4B
U+84CC.0	fBig5	E3C4
U+84CC.0	fGB	3-6A3D
U+84CC.0	fR/S	 140.10
U+84CC.0	fTotalStrokes	 14; 16
U+84CC.0	fCangjie	TOOE
U+84CC.0	fFourCorner	44407
U+84CC.0	fHakka	 co5
U+84CC.0	fCantonese	co3; co5
U+84CC.0	fMandarin	cuo4
U+84CC.0	fEnglish	 pretend crouch
U+84CD.0	fUTF8	蓍
U+84CD.0	fCNS11643	2-4D40
U+84CD.0	fBig5	E3B9
U+84CD.0	fGB	0-5D69
U+84CD.0	fR/S	 140.10
U+84CD.0	fTotalStrokes	 14; 16
U+84CD.0	fCangjie	TJPA
U+84CD.0	fFourCorner	44601
U+84CD.0	fHakka	 shi1; sii1; ci1; cih1; she1
U+84CD.0	fCantonese	si1
U+84CD.0	fMandarin	shi1; zhi1
U+84CD.0	fEnglish	milfoil, the stalks of which were used in divination
U+84CE.0	fUTF8	蓎
U+84CE.0	fCNS11643	2-4D38
U+84CE.0	fBig5	E3B1
U+84CE.0	fGB	5-6948
U+84CE.0	fR/S	 140.10
U+84CE.0	fTotalStrokes	 14; 16
U+84CE.0	fCangjie	TILR
U+84CE.0	fFourCorner	44267
U+84CE.0	fCantonese	tong4
U+84CE.0	fMandarin	tang2
U+84CF.0	fUTF8	蓏
U+84CF.0	fCNS11643	2-4D53
U+84CF.0	fBig5	E3CC
U+84CF.0	fGB	3-6A3C
U+84CF.0	fR/S	 140.10; 140.12
U+84CF.0	fTotalStrokes	 14; 16
U+84CF.0	fCangjie	THOO
U+84CF.0	fFourCorner	44212
U+84CF.0	fHakka	 ko5;  lo3; ko3
U+84CF.0	fCantonese	lo2; lo5
U+84CF.0	fMandarin	luo3
U+84CF.0	fEnglish	 fruit of plant melon
U+84D0.0	fUTF8	蓐
U+84D0.0	fCNS11643	2-4D44
U+84D0.0	fBig5	E3BD
U+84D0.0	fGB	0-5D6A
U+84D0.0	fR/S	 140.10
U+84D0.0	fTotalStrokes	 14; 16
U+84D0.0	fCangjie	TMVI
U+84D0.0	fFourCorner	44243
U+84D0.0	fHakka	 juk8; ruk8
U+84D0.0	fCantonese	juk6
U+84D0.0	fMandarin	ru4
U+84D0.0	fEnglish	 straw bed mat; matting
U+84D1.0	fUTF8	蓑
U+84D1.0	fCNS11643	1-6A6A
U+84D1.0	fBig5	BB62
U+84D1.0	fGB	0-4B72
U+84D1.0	fR/S	 140.10
U+84D1.0	fTotalStrokes	 14; 16
U+84D1.0	fCangjie	TYWV
U+84D1.0	fFourCorner	44732
U+84D1.0	fHakka	  so1
U+84D1.0	fCantonese	so1
U+84D1.0	fMandarin	suo1; sui1; cui1; sai1; shuai1; sui2
U+84D1.0	fEnglish	[1] rain coat or cloak made of straw, coir, rushes, etc. [2] cover with grass
U+84D2.0	fUTF8	蓒
U+84D2.0	fCNS11643	2-4D57
U+84D2.0	fBig5	E3D0
U+84D2.0	fGB	5-6830
U+84D2.0	fR/S	 140.10
U+84D2.0	fTotalStrokes	 14; 16
U+84D2.0	fCangjie	TJJJ
U+84D2.0	fFourCorner	44541
U+84D2.0	fCantonese	hin1
U+84D2.0	fMandarin	xian1; xuan1
U+84D3.0	fUTF8	蓓
U+84D3.0	fCNS11643	1-6A67
U+84D3.0	fBig5	BB5F
U+84D3.0	fGB	0-5D6D
U+84D3.0	fR/S	 140.10
U+84D3.0	fTotalStrokes	 14; 16
U+84D3.0	fCangjie	TOYR
U+84D3.0	fFourCorner	44261
U+84D3.0	fHakka	 pui5; pi5; poi6; poi5; pi6
U+84D3.0	fCantonese	pui5; pui4 (also); bui3
U+84D3.0	fMandarin	bei4
U+84D3.0	fEnglish	 flower bud; bud
U+84D4.0	fUTF8	蓔
U+84D4.0	fCNS11643	2-4D56
U+84D4.0	fBig5	E3CF
U+84D4.0	fGB	5-694A
U+84D4.0	fR/S	 140.10
U+84D4.0	fTotalStrokes	 14; 16
U+84D4.0	fCangjie	TTGF
U+84D4.0	fFourCorner	44301
U+84D4.0	fCantonese	jiu5; jiu2; guk1
U+84D4.0	fMandarin	yao3; gu4
U+84D5.0	fUTF8	蓕
U+84D5.0	fCNS11643	3-493F
U+84D5.0	fGB	5-6937
U+84D5.0	fR/S	  140.10
U+84D5.0	fTotalStrokes	 14; 16
U+84D5.0	fCangjie	TDGG
U+84D5.0	fFourCorner	44910
U+84D5.0	fCantonese	gwai3
U+84D5.0	fMandarin	gui4
U+84D6.0	fUTF8	蓖
U+84D6.0	fCNS11643	2-4D50
U+84D6.0	fBig5	E3C9
U+84D6.0	fGB	0-314D
U+84D6.0	fR/S	 140.10
U+84D6.0	fTotalStrokes	 14; 16
U+84D6.0	fCangjie	THWP
U+84D6.0	fFourCorner	44711
U+84D6.0	fHakka	 bi1
U+84D6.0	fCantonese	bei1; bei6 (also); bai1; bei4
U+84D6.0	fMandarin	bi4; bi1
U+84D6.0	fEnglish	the castor-oil plant (botany)
U+84D7.0	fUTF8	蓗
U+84D7.0	fCNS11643	2-4D55
U+84D7.0	fBig5	E3CE
U+84D7.0	fGB	E-3B25
U+84D7.0	fR/S	 140.10
U+84D7.0	fTotalStrokes	 14; 16
U+84D7.0	fCangjie	THOO
U+84D7.0	fFourCorner	44281
U+84D7.0	fCantonese	zung2
U+84D7.0	fMandarin	zong3
U+84D7.0	fEnglish	a plant; five-fold
U+84D8.0	fUTF8	蓘
U+84D8.0	fCNS11643	3-493E
U+84D8.0	fGB	3-6A43
U+84D8.0	fR/S	  140.10
U+84D8.0	fTotalStrokes	 14; 16
U+84D8.0	fCangjie	TYCV
U+84D8.0	fFourCorner	44730
U+84D8.0	fCantonese	gwan2
U+84D8.0	fMandarin	gun3
U+84D9.0	fUTF8	蓙
U+84D9.0	fGB	E-3B26
U+84D9.0	fR/S	 140.10
U+84D9.0	fTotalStrokes	 14; 16
U+84D9.0	fCangjie	TIOG
U+84D9.0	fCantonese	zo6
U+84D9.0	fEnglish	a mat, matting
U+84DA.0	fUTF8	蓚
U+84DA.0	fCNS11643	3-4943
U+84DA.0	fGB	5-687B
U+84DA.0	fR/S	 140.10
U+84DA.0	fTotalStrokes	 14; 16
U+84DA.0	fCangjie	TOLH
U+84DA.0	fFourCorner	44220
U+84DA.0	fCantonese	tiu4
U+84DA.0	fMandarin	xiu1
U+84DA.0	fEnglish	oxalic (used in compounds)
U+84DB.0	fUTF8	蓛
U+84DB.0	fCNS11643	2-4D58
U+84DB.0	fBig5	E3D1
U+84DB.0	fGB	5-693B
U+84DB.0	fR/S	 140.10
U+84DB.0	fTotalStrokes	 14; 16
U+84DB.0	fCangjie	TDBK
U+84DB.0	fFourCorner	44948
U+84DB.0	fCantonese	cik1; caak3
U+84DB.0	fMandarin	ce4
U+84DC.0	fUTF8	蓜
U+84DC.0	fGB	E-3B27
U+84DC.0	fR/S	 140.10
U+84DC.0	fTotalStrokes	 14; 16
U+84DC.0	fCangjie	TMWU
U+84DC.0	fFourCorner	4461
U+84DC.0	fCantonese	pui3
U+84DD.0	fUTF8	蓝
U+84DD.0	fGB	0-4036
U+84DD.0	fR/S	 140.10
U+84DD.0	fTotalStrokes	 14; 16
U+84DD.0	fCangjie	TLIT
U+84DD.0	fHakka	 lam2
U+84DD.0	fCantonese	laam4; laam2 (also)
U+84DD.0	fMandarin	lan2; la5; lan4
U+84DD.0	fEnglish	simplified form of &#x85CD;
U+84DE.0	fUTF8	蓞
U+84DE.0	fCNS11643	3-663B
U+84DE.0	fGB	E-3B28
U+84DE.0	fR/S	 140.10
U+84DE.0	fTotalStrokes	 14; 16
U+84DE.0	fCangjie	TBHX
U+84DE.0	fFourCorner	4477
U+84DE.0	fCantonese	daam6
U+84DF.0	fUTF8	蓟
U+84DF.0	fGB	0-3C3B
U+84DF.0	fR/S	 140.10
U+84DF.0	fTotalStrokes	 14; 16
U+84DF.0	fCangjie	TNMN
U+84DF.0	fHakka	 gi5; ge5
U+84DF.0	fCantonese	gai3
U+84DF.0	fMandarin	ji4; jie1; jie4
U+84DF.0	fEnglish	simplified form of &#x858A;
U+84E0.0	fUTF8	蓠
U+84E0.0	fGB	0-5D71
U+84E0.0	fR/S	 140.10
U+84E0.0	fTotalStrokes	 14; 16
U+84E0.0	fCangjie	TYUB
U+84E0.0	fHakka	 li2
U+84E0.0	fCantonese	lei4
U+84E0.0	fMandarin	li2
U+84E0.0	fEnglish	simplified form of &#x863A;
U+84E1.0	fUTF8	蓡
U+84E1.0	fCNS11643	3-663C
U+84E1.0	fGB	E-3B29
U+84E1.0	fR/S	 140.10
U+84E1.0	fTotalStrokes	 14; 16
U+84E1.0	fCangjie	TESE
U+84E1.0	fCantonese	sam1
U+84E1.0	fMandarin	can1; shen1
U+84E2.0	fUTF8	蓢
U+84E2.0	fCNS11643	3-663A
U+84E2.0	fGB	E-3B2A
U+84E2.0	fR/S	 140.10
U+84E2.0	fTotalStrokes	 14; 16
U+84E2.0	fCangjie	TIIB
U+84E2.0	fFourCorner	4472
U+84E2.0	fCantonese	long5; long1
U+84E2.0	fMandarin	lang2
U+84E2.0	fEnglish	brake, fern
U+84E3.0	fUTF8	蓣
U+84E3.0	fGB	0-5D77
U+84E3.0	fR/S	 140.10
U+84E3.0	fTotalStrokes	 14; 16
U+84E3.0	fCangjie	TNNO
U+84E3.0	fHakka	 ji5
U+84E3.0	fCantonese	jyu6
U+84E3.0	fMandarin	yu4
U+84E3.0	fEnglish	simplified form of &#x8577;
U+84E4.0	fUTF8	蓤
U+84E4.0	fCNS11643	3-493D
U+84E4.0	fGB	E-3B2B
U+84E4.0	fR/S	 140.10
U+84E4.0	fTotalStrokes	 14; 16
U+84E4.0	fCangjie	TIME
U+84E4.0	fFourCorner	44140
U+84E4.0	fCantonese	ling4
U+84E5.0	fUTF8	蓥
U+84E5.0	fGB	0-5D76
U+84E5.0	fR/S	 140.10; 140.11
U+84E5.0	fTotalStrokes	 14; 17
U+84E5.0	fCangjie	TBC
U+84E5.0	fCantonese	jing4
U+84E5.0	fMandarin	ying4; ying2; jyong3; xing2; ying1
U+84E5.0	fEnglish	simplified form of &#x93A3;
U+84E6.0	fUTF8	蓦
U+84E6.0	fGB	0-5D6B
U+84E6.0	fR/S	 140.10; 187.10
U+84E6.0	fTotalStrokes	 14; 16
U+84E6.0	fCangjie	TAKM
U+84E6.0	fHakka	 met7; mok7
U+84E6.0	fCantonese	mak6
U+84E6.0	fMandarin	mo4; ma4
U+84E6.0	fEnglish	simplified form of &#x9A40;
U+84E7.0	fUTF8	蓧
U+84E7.0	fCNS11643	2-535C
U+84E7.0	fBig5	E773
U+84E7.0	fGB	3-6932
U+84E7.0	fR/S	 140.11
U+84E7.0	fTotalStrokes	 15; 17
U+84E7.0	fCangjie	TOLD
U+84E7.0	fFourCorner	44294
U+84E7.0	fHakka	 tiau2; tiau3; tiau5
U+84E7.0	fCantonese	diu6; tiu1; tiu4
U+84E7.0	fMandarin	diao4; tiao1; di2; tiao2
U+84E7.0	fEnglish	 bamboo basket weeder rake
U+84E8.0	fUTF8	蓨
U+84E8.0	fCNS11643	2-535D
U+84E8.0	fBig5	E774
U+84E8.0	fGB	3-6A39
U+84E8.0	fR/S	 140.11
U+84E8.0	fTotalStrokes	 15; 17
U+84E8.0	fCangjie	TOLB
U+84E8.0	fFourCorner	44227
U+84E8.0	fCantonese	tiu1; tiu4; sau1
U+84E8.0	fMandarin	tiao1; xiu1; ti1; tiao2
U+84E9.0	fUTF8	蓩
U+84E9.0	fCNS11643	2-5350
U+84E9.0	fBig5	E767
U+84E9.0	fGB	3-6A4F
U+84E9.0	fR/S	 140.11
U+84E9.0	fTotalStrokes	 15; 17
U+84E9.0	fCangjie	TNHS
U+84E9.0	fFourCorner	44224
U+84E9.0	fCantonese	mou6; muk6
U+84E9.0	fMandarin	mao4; wu4; mou4; mu4
U+84EA.0	fUTF8	蓪
U+84EA.0	fCNS11643	2-534F
U+84EA.0	fBig5	E766
U+84EA.0	fGB	5-6954
U+84EA.0	fR/S	 140.11
U+84EA.0	fTotalStrokes	 15; 17
U+84EA.0	fCangjie	TYNB
U+84EA.0	fFourCorner	44302
U+84EA.0	fHakka	 tung1
U+84EA.0	fCantonese	tung1
U+84EA.0	fMandarin	tong1
U+84EA.0	fEnglish	 paper-plant Aralia papyrifera
U+84EB.0	fUTF8	蓫
U+84EB.0	fCNS11643	2-534B
U+84EB.0	fBig5	E762
U+84EB.0	fGB	3-6A34
U+84EB.0	fR/S	 140.11
U+84EB.0	fTotalStrokes	 15; 17
U+84EB.0	fCangjie	TYMO
U+84EB.0	fFourCorner	44303
U+84EB.0	fHakka	 ch'uk8; cuk8
U+84EB.0	fCantonese	zuk6
U+84EB.0	fMandarin	zhu2; chu4; xu4
U+84EB.0	fEnglish	dock
U+84EC.0	fUTF8	蓬
U+84EC.0	fCNS11643	1-6E5D
U+84EC.0	fBig5	BDB4
U+84EC.0	fGB	0-456E
U+84EC.0	fR/S	 140.11
U+84EC.0	fTotalStrokes	 15; 17
U+84EC.0	fCangjie	TYHJ
U+84EC.0	fFourCorner	44304
U+84EC.0	fHakka	  pung2
U+84EC.0	fCantonese	pung4; fung4
U+84EC.0	fMandarin	peng2; beng4; peng4
U+84EC.0	fEnglish	[1] Erigeron acer, a species of raspberry [2] tangled [3] disheveled (hair) [4] flourishing; prospering [5] a Chinese family name
U+84ED.0	fUTF8	蓭
U+84ED.0	fCNS11643	3-4E4F
U+84ED.0	fGB	E-3B2C
U+84ED.0	fR/S	  140.11
U+84ED.0	fTotalStrokes	 15; 17
U+84ED.0	fCangjie	TIKU
U+84ED.0	fFourCorner	44210
U+84ED.0	fCantonese	am1; ngam1 (also)
U+84ED.0	fMandarin	an1
U+84EE.0	fUTF8	蓮
U+84EE.0	fCNS11643	1-6E55
U+84EE.0	fBig5	BDAC
U+84EE.0	fGB	1-412B
U+84EE.0	fR/S	 140.11
U+84EE.0	fTotalStrokes	 15; 17
U+84EE.0	fCangjie	TYJJ
U+84EE.0	fFourCorner	44304
U+84EE.0	fHakka	 lien2; len2; lian3
U+84EE.0	fCantonese	lin4; lin2 (also)
U+84EE.0	fMandarin	lian2; lian3; lian4
U+84EE.0	fEnglish	[1] the lotus; water lily [2] Buddhist paradise; the "clean" land (Buddhism)
U+84EF.0	fUTF8	蓯
U+84EF.0	fCNS11643	2-535F
U+84EF.0	fBig5	E776
U+84EF.0	fGB	1-5C4A
U+84EF.0	fR/S	 140.11
U+84EF.0	fTotalStrokes	 15; 17
U+84EF.0	fCangjie	THOO
U+84EF.0	fFourCorner	44281
U+84EF.0	fHakka	 zung3;  ciung2
U+84EF.0	fCantonese	cung1
U+84EF.0	fMandarin	cong1; song3; xuan4; zong3
U+84EF.0	fEnglish	medicinal herb
U+84F0.0	fUTF8	蓰
U+84F0.0	fCNS11643	2-535E
U+84F0.0	fBig5	E775
U+84F0.0	fGB	0-5D7B
U+84F0.0	fR/S	 140.11
U+84F0.0	fTotalStrokes	 15; 17
U+84F0.0	fCangjie	THOO
U+84F0.0	fFourCorner	44281
U+84F0.0	fHakka	  sai3; su3
U+84F0.0	fCantonese	saai2
U+84F0.0	fMandarin	xi3; si1
U+84F0.0	fEnglish	[1] a variety of grass [2] increase five-fold; five times of anything
U+84F1.0	fUTF8	蓱
U+84F1.0	fCNS11643	2-464E
U+84F1.0	fBig5	DFA9
U+84F1.0	fGB	5-692D
U+84F1.0	fR/S	 140.11
U+84F1.0	fTotalStrokes	 15; 17
U+84F1.0	fCangjie	TETT
U+84F1.0	fFourCorner	44341
U+84F1.0	fCantonese	ping4
U+84F1.0	fMandarin	ping2
U+84F2.0	fUTF8	蓲
U+84F2.0	fCNS11643	2-5348
U+84F2.0	fBig5	E75F
U+84F2.0	fGB	3-683A
U+84F2.0	fR/S	 140.11
U+84F2.0	fTotalStrokes	 15; 17
U+84F2.0	fCangjie	TSRR
U+84F2.0	fFourCorner	44716
U+84F2.0	fCantonese	jau1; hau1
U+84F2.0	fMandarin	qiu1; xu1; ou1; fu1; ou4; shu1; yu1; yu4
U+84F3.0	fUTF8	蓳
U+84F3.0	fCNS11643	2-534C
U+84F3.0	fBig5	E763
U+84F3.0	fGB	5-695A
U+84F3.0	fR/S	 140.11
U+84F3.0	fTotalStrokes	 15; 17
U+84F3.0	fCangjie	TTLM
U+84F3.0	fFourCorner	44104
U+84F3.0	fHakka	  giun3
U+84F3.0	fCantonese	kan4; gan2; gan3
U+84F3.0	fMandarin	jin4
U+84F3.0	fEnglish	[1] clay [2] yellow loam [3] plaster
U+84F4.0	fUTF8	蓴
U+84F4.0	fCNS11643	2-5346
U+84F4.0	fBig5	E75D
U+84F4.0	fGB	E-3B2D
U+84F4.0	fR/S	 140.11
U+84F4.0	fTotalStrokes	 15; 17
U+84F4.0	fCangjie	TJII
U+84F4.0	fFourCorner	44343
U+84F4.0	fHakka	 ton2; sun2; shun2
U+84F4.0	fCantonese	seon4
U+84F4.0	fMandarin	chun2; tuan2
U+84F4.0	fEnglish	Bracenia schreberi, water shield
U+84F5.0	fUTF8	蓵
U+84F5.0	fCNS11643	3-4E44
U+84F5.0	fGB	3-6A57
U+84F5.0	fR/S	  140.11
U+84F5.0	fTotalStrokes	 15; 17
U+84F5.0	fCangjie	TQJO
U+84F5.0	fFourCorner	44580
U+84F5.0	fCantonese	zit6
U+84F5.0	fMandarin	jie2
U+84F6.0	fUTF8	蓶
U+84F6.0	fCNS11643	2-5359
U+84F6.0	fBig5	E770
U+84F6.0	fGB	3-6A5B
U+84F6.0	fR/S	 140.11
U+84F6.0	fTotalStrokes	 15; 17
U+84F6.0	fCangjie	TROG
U+84F6.0	fFourCorner	44614
U+84F6.0	fCantonese	wai4; wai5
U+84F6.0	fMandarin	wei3; wei2
U+84F7.0	fUTF8	蓷
U+84F7.0	fCNS11643	2-534A
U+84F7.0	fBig5	E761
U+84F7.0	fGB	3-6A58
U+84F7.0	fR/S	 140.11
U+84F7.0	fTotalStrokes	 15; 17
U+84F7.0	fCangjie	TQOG
U+84F7.0	fFourCorner	44514
U+84F7.0	fHakka	 tui1; cui1
U+84F7.0	fCantonese	teoi1
U+84F7.0	fMandarin	tui1; chui1
U+84F7.0	fEnglish	leon urus sibiriacus; mother war
U+84F8.0	fUTF8	蓸
U+84F8.0	fCNS11643	3-4E42
U+84F8.0	fGB	3-6A54
U+84F8.0	fR/S	  140.11
U+84F8.0	fTotalStrokes	 15; 17
U+84F8.0	fCangjie	TTWA
U+84F8.0	fFourCorner	44600
U+84F8.0	fCantonese	cou4
U+84F8.0	fMandarin	cao2
U+84F9.0	fUTF8	蓹
U+84F9.0	fCNS11643	2-5360
U+84F9.0	fBig5	E777
U+84F9.0	fGB	5-697B
U+84F9.0	fR/S	 140.11
U+84F9.0	fTotalStrokes	 15; 17
U+84F9.0	fCangjie	THOL
U+84F9.0	fFourCorner	44227
U+84F9.0	fCantonese	jyu6
U+84F9.0	fMandarin	yu3; yu4
U+84FA.0	fUTF8	蓺
U+84FA.0	fCNS11643	2-5343
U+84FA.0	fBig5	E75A
U+84FA.0	fGB	5-6956
U+84FA.0	fR/S	 140.11
U+84FA.0	fTotalStrokes	 15; 17
U+84FA.0	fCangjie	TGGI
U+84FA.0	fFourCorner	44117
U+84FA.0	fHakka	 ngi5; ngi6
U+84FA.0	fCantonese	ngai6; ai6 (also)
U+84FA.0	fMandarin	yi4
U+84FA.0	fEnglish	 art talent; ability craft
U+84FB.0	fUTF8	蓻
U+84FB.0	fCNS11643	2-5341
U+84FB.0	fBig5	E758
U+84FB.0	fGB	3-6866
U+84FB.0	fR/S	 140.11
U+84FB.0	fTotalStrokes	 15; 17
U+84FB.0	fCangjie	TGJI
U+84FB.0	fFourCorner	44410
U+84FB.0	fCantonese	cap1; guk1; zap1
U+84FB.0	fMandarin	ji2; ju2; nie4
U+84FB.0	fEnglish	classifier for hair
U+84FC.0	fUTF8	蓼
U+84FC.0	fCNS11643	2-534D
U+84FC.0	fBig5	E764
U+84FC.0	fGB	0-5E24
U+84FC.0	fR/S	 140.11
U+84FC.0	fTotalStrokes	 15; 17
U+84FC.0	fCangjie	TSMH
U+84FC.0	fFourCorner	44202
U+84FC.0	fHakka	 liuk8;  liau3; liau6; liau5; luk7
U+84FC.0	fCantonese	liu5; luk6; luk5
U+84FC.0	fMandarin	liao3; lao3; liu3
U+84FC.0	fEnglish	[1] smartweed [2] name of a state in today's Honan province in the Spring and Autumn period
U+84FC.1	fUTF8	蓼
U+84FC.1	fCNS11643	2-534D
U+84FC.1	fBig5	E764
U+84FC.1	fGB	0-5E24
U+84FC.1	fR/S	 140.11
U+84FC.1	fTotalStrokes	 15; 17
U+84FC.1	fCangjie	TSMH
U+84FC.1	fFourCorner	44202
U+84FC.1	fMandarin	lu4
U+84FC.1	fEnglish	luxuriant; high (vegetation)
U+84FD.0	fUTF8	蓽
U+84FD.0	fCNS11643	2-5357
U+84FD.0	fBig5	E76E
U+84FD.0	fGB	1-5C6A
U+84FD.0	fR/S	 140.11
U+84FD.0	fTotalStrokes	 15; 17
U+84FD.0	fCangjie	TWTJ
U+84FD.0	fFourCorner	44504
U+84FD.0	fHakka	 bit7
U+84FD.0	fCantonese	bat1
U+84FD.0	fMandarin	bi4
U+84FD.0	fEnglish	[1] bamboo or wicker for making baskets, etc. [2] Piper longtum, a kind of herb growing among bamboos, used in Chinese medicine
U+84FE.0	fUTF8	蓾
U+84FE.0	fCNS11643	2-5352
U+84FE.0	fBig5	E769
U+84FE.0	fGB	3-6926
U+84FE.0	fR/S	 140.11
U+84FE.0	fTotalStrokes	 15; 17
U+84FE.0	fCangjie	TYWI
U+84FE.0	fFourCorner	44601
U+84FE.0	fHakka	 lu1
U+84FE.0	fCantonese	lou5
U+84FE.0	fMandarin	lu3
U+84FF.0	fUTF8	蓿
U+84FF.0	fCNS11643	1-6E5F
U+84FF.0	fBig5	BDB6
U+84FF.0	fGB	0-5E23
U+84FF.0	fR/S	 140.11
U+84FF.0	fTotalStrokes	 15; 17
U+84FF.0	fCangjie	TJOA
U+84FF.0	fFourCorner	44261
U+84FF.0	fHakka	  siuk7;  suk7
U+84FF.0	fCantonese	suk1
U+84FF.0	fMandarin	su4; xu5; obs; xu4
U+84FF.0	fEnglish	clover; lucerne
U+8500.0	fUTF8	蔀
U+8500.0	fCNS11643	2-5338
U+8500.0	fBig5	E74F
U+8500.0	fGB	3-6A45
U+8500.0	fR/S	 140.11
U+8500.0	fTotalStrokes	 15; 17
U+8500.0	fCangjie	TYRL
U+8500.0	fFourCorner	44627
U+8500.0	fHakka	 pu3; piau3
U+8500.0	fCantonese	bou6
U+8500.0	fMandarin	bu4; pou3
U+8500.0	fEnglish	screen
U+8501.0	fUTF8	蔁
U+8501.0	fCNS11643	3-4E40
U+8501.0	fGB	3-6A63
U+8501.0	fR/S	  140.11
U+8501.0	fTotalStrokes	 15; 17
U+8501.0	fCangjie	TYTJ
U+8501.0	fFourCorner	44400
U+8501.0	fCantonese	zoeng1
U+8501.0	fMandarin	zhang1
U+8502.0	fUTF8	蔂
U+8502.0	fCNS11643	2-5356
U+8502.0	fBig5	E76D
U+8502.0	fGB	3-6A5A
U+8502.0	fR/S	 140.11
U+8502.0	fTotalStrokes	 15; 17
U+8502.0	fCangjie	TWVF
U+8502.0	fFourCorner	44903
U+8502.0	fHakka	  lo2; lui2
U+8502.0	fCantonese	leoi4; lo4; lui4
U+8502.0	fMandarin	luo2; lei2
U+8502.0	fEnglish	basket
U+8503.0	fUTF8	蔃
U+8503.0	fCNS11643	3-4E4C
U+8503.0	fGB	3-6B22
U+8503.0	fR/S	 140.11
U+8503.0	fTotalStrokes	 15; 17
U+8503.0	fCangjie	TNII
U+8503.0	fFourCorner	44230
U+8503.0	fCantonese	koeng2; goeng2 (also); koeng5
U+8503.0	fMandarin	jiang4; qiang2
U+8503.0	fEnglish	roots of plants
U+8504.0	fUTF8	蔄
U+8504.0	fCNS11643	3-4E46
U+8504.0	fGB	E-3B2E
U+8504.0	fR/S	  140.11
U+8504.0	fTotalStrokes	 15; 17
U+8504.0	fCangjie	TANR
U+8504.0	fFourCorner	44220
U+8504.0	fCantonese	maan6
U+8504.0	fMandarin	man4
U+8505.0	fUTF8	蔅
U+8505.0	fCNS11643	3-4E54
U+8505.0	fGB	E-3B2F
U+8505.0	fR/S	 140.11
U+8505.0	fTotalStrokes	 15; 17
U+8505.0	fCangjie	TANV
U+8505.0	fFourCorner	44220
U+8505.0	fCantonese	jin4
U+8505.0	fMandarin	yan2
U+8506.0	fUTF8	蔆
U+8506.0	fCNS11643	1-6E60
U+8506.0	fBig5	BDB7
U+8506.0	fGB	E-3B30
U+8506.0	fR/S	 140.11
U+8506.0	fTotalStrokes	 15; 17
U+8506.0	fCangjie	TEGE
U+8506.0	fFourCorner	44147
U+8506.0	fHakka	 lin2
U+8506.0	fCantonese	ling4
U+8506.0	fMandarin	ling2; leng2
U+8506.0	fEnglish	water-chestnut; water caltrop
U+8507.0	fUTF8	蔇
U+8507.0	fCNS11643	2-4662
U+8507.0	fBig5	DFBD
U+8507.0	fGB	3-6A28
U+8507.0	fR/S	 140.11; 140.12
U+8507.0	fTotalStrokes	 15; 17
U+8507.0	fCangjie	TAIU
U+8507.0	fFourCorner	44714
U+8507.0	fHakka	  gi5
U+8507.0	fCantonese	gei3; gei6
U+8507.0	fMandarin	ji4; xi4
U+8507.0	fEnglish	 luxuriant growth extreme reach
U+8508.0	fUTF8	蔈
U+8508.0	fCNS11643	2-5344
U+8508.0	fBig5	E75B
U+8508.0	fGB	3-6A55
U+8508.0	fR/S	 140.11
U+8508.0	fTotalStrokes	 15; 17
U+8508.0	fCangjie	TMWF
U+8508.0	fFourCorner	44901
U+8508.0	fCantonese	biu1
U+8508.0	fMandarin	piao3; biao1; piao4
U+8509.0	fUTF8	蔉
U+8509.0	fCNS11643	2-533B
U+8509.0	fBig5	E752
U+8509.0	fGB	E-3B31
U+8509.0	fR/S	 140.11
U+8509.0	fTotalStrokes	 15; 17
U+8509.0	fCangjie	TYCV
U+8509.0	fFourCorner	44732
U+8509.0	fHakka	 gun3; kun3; gwun3
U+8509.0	fCantonese	gwan2
U+8509.0	fMandarin	gun3
U+8509.0	fEnglish	earth up
U+850A.0	fUTF8	蔊
U+850A.0	fCNS11643	2-533E
U+850A.0	fBig5	E755
U+850A.0	fGB	3-6A66
U+850A.0	fR/S	 140.11
U+850A.0	fTotalStrokes	 15; 17
U+850A.0	fCangjie	TFAJ
U+850A.0	fFourCorner	44941
U+850A.0	fHakka	 hon5
U+850A.0	fCantonese	hon2; hon6 (also); hon5
U+850A.0	fMandarin	han3; han4
U+850A.0	fEnglish	a pungent plant like cress
U+850B.0	fUTF8	蔋
U+850B.0	fCNS11643	2-5364
U+850B.0	fBig5	E77B
U+850B.0	fGB	5-6969
U+850B.0	fR/S	 140.11
U+850B.0	fTotalStrokes	 15; 17
U+850B.0	fCangjie	TEYE
U+850B.0	fFourCorner	44347
U+850B.0	fHakka	 tit8; ziau1; tit7
U+850B.0	fCantonese	dik6; tik1
U+850B.0	fMandarin	di2; du2; jiao1; shu2
U+850B.0	fEnglish	dry; scorched
U+850C.0	fUTF8	蔌
U+850C.0	fCNS11643	2-5345
U+850C.0	fBig5	E75C
U+850C.0	fGB	0-5D78
U+850C.0	fR/S	 140.11
U+850C.0	fTotalStrokes	 15; 17
U+850C.0	fCangjie	TDLO
U+850C.0	fFourCorner	44982
U+850C.0	fHakka	  suk7; siuk7
U+850C.0	fCantonese	cuk1; suk1
U+850C.0	fMandarin	su4; sou3
U+850C.0	fEnglish	vegetables
U+850D.0	fUTF8	蔍
U+850D.0	fCNS11643	2-533C
U+850D.0	fBig5	E753
U+850D.0	fGB	5-6965
U+850D.0	fR/S	 140.11
U+850D.0	fTotalStrokes	 15; 17
U+850D.0	fCangjie	TIXP
U+850D.0	fFourCorner	44211
U+850D.0	fCantonese	luk6; cou1
U+850D.0	fMandarin	lu4
U+850E.0	fUTF8	蔎
U+850E.0	fCNS11643	2-533A
U+850E.0	fBig5	E751
U+850E.0	fGB	3-6878
U+850E.0	fR/S	 140.11
U+850E.0	fTotalStrokes	 15; 17
U+850E.0	fCangjie	TYRE
U+850E.0	fFourCorner	44647
U+850E.0	fHakka	 s'at7; sat7; set7; s'et7
U+850E.0	fCantonese	cit3; cit6
U+850E.0	fMandarin	she4; sa1
U+850E.0	fEnglish	a fragrant plant from which an infusion is made
U+850F.0	fUTF8	蔏
U+850F.0	fCNS11643	2-5337
U+850F.0	fBig5	E74E
U+850F.0	fGB	3-6A64
U+850F.0	fR/S	 140.11
U+850F.0	fTotalStrokes	 15; 17
U+850F.0	fCangjie	TYCB
U+850F.0	fFourCorner	44227
U+850F.0	fHakka	 shong1; song1
U+850F.0	fCantonese	soeng1
U+850F.0	fMandarin	shang1; zhang1
U+850F.0	fEnglish	a plant, the root of which is used to cure ague
U+8510.0	fUTF8	蔐
U+8510.0	fCNS11643	4-4C7A
U+8510.0	fGB	5-6966
U+8510.0	fR/S	  140.11
U+8510.0	fTotalStrokes	 15; 17
U+8510.0	fCangjie	TYCB
U+8510.0	fFourCorner	44220
U+8510.0	fCantonese	dik6
U+8510.0	fMandarin	di2
U+8511.0	fUTF8	蔑
U+8511.0	fCNS11643	1-6E59
U+8511.0	fBig5	BDB0
U+8511.0	fGB	0-436F
U+8511.0	fR/S	 140.11
U+8511.0	fTotalStrokes	 15; 17
U+8511.0	fCangjie	TWLI
U+8511.0	fFourCorner	44253
U+8511.0	fHakka	 met7; mit7
U+8511.0	fCantonese	mit6
U+8511.0	fMandarin	mie4
U+8511.0	fEnglish	[1] disdain; slight; despise; neglect; disregard; feel contempt for [2] without; none; no [3] cast away [4] tiny; small
U+8512.0	fUTF8	蔒
U+8512.0	fCNS11643	2-534E
U+8512.0	fBig5	E765
U+8512.0	fGB	5-696D
U+8512.0	fR/S	 140.11
U+8512.0	fTotalStrokes	 15; 17
U+8512.0	fCangjie	TSRF
U+8512.0	fFourCorner	44336
U+8512.0	fCantonese	fan1
U+8512.0	fMandarin	xun1; hun1
U+8513.0	fUTF8	蔓
U+8513.0	fCNS11643	1-6E58
U+8513.0	fBig5	BDAF
U+8513.0	fGB	0-427B
U+8513.0	fR/S	 140.11
U+8513.0	fTotalStrokes	 15; 17
U+8513.0	fCangjie	TAWE
U+8513.0	fFourCorner	44407
U+8513.0	fHakka	 man3;  man5; man6; wan5; wan6; van5; man2
U+8513.0	fCantonese	maan6; maan4
U+8513.0	fMandarin	man4
U+8513.0	fEnglish	 creeping plants; plants with creeping tendrils
U+8513.1	fUTF8	蔓
U+8513.1	fCNS11643	1-6E58
U+8513.1	fBig5	BDAF
U+8513.1	fGB	0-427B
U+8513.1	fR/S	 140.11
U+8513.1	fTotalStrokes	 15; 17
U+8513.1	fCangjie	TAWE
U+8513.1	fFourCorner	44407
U+8513.1	fMandarin	wan4
U+8513.1	fEnglish	 tendril; creeper
U+8513.2	fUTF8	蔓
U+8513.2	fCNS11643	1-6E58
U+8513.2	fBig5	BDAF
U+8513.2	fGB	0-427B
U+8513.2	fR/S	 140.11
U+8513.2	fTotalStrokes	 15; 17
U+8513.2	fCangjie	TAWE
U+8513.2	fFourCorner	44407
U+8513.2	fMandarin	man2
U+8513.2	fEnglish	the rape turnip
U+8514.0	fUTF8	蔔
U+8514.0	fCNS11643	1-6E5C
U+8514.0	fBig5	BDB3
U+8514.0	fGB	1-7825
U+8514.0	fR/S	 140.11
U+8514.0	fTotalStrokes	 15; 17
U+8514.0	fCangjie	TPMW
U+8514.0	fFourCorner	44627
U+8514.0	fHakka	  pet8
U+8514.0	fCantonese	baak6; buk1
U+8514.0	fMandarin	bo5; bo2; bu3; fu2
U+8514.0	fEnglish	used in combination for a common name for edible roots like turnips, carrots, radishes
U+8515.0	fUTF8	蔕
U+8515.0	fCNS11643	2-5349
U+8515.0	fBig5	E760
U+8515.0	fGB	E-3B32
U+8515.0	fR/S	 140.11
U+8515.0	fTotalStrokes	 15; 17
U+8515.0	fCangjie	TKPB
U+8515.0	fFourCorner	44227
U+8515.0	fHakka	  di5; dai5; ni5
U+8515.0	fCantonese	dai3; daai3; ding3; dai6
U+8515.0	fMandarin	di4; chai4; dai4
U+8515.0	fEnglish	 peduncle, stem or footstalk of a flower or fruit
U+8516.0	fUTF8	蔖
U+8516.0	fCNS11643	2-5351
U+8516.0	fBig5	E768
U+8516.0	fGB	5-695D
U+8516.0	fR/S	 140.11
U+8516.0	fTotalStrokes	 15; 17
U+8516.0	fCangjie	TYPM
U+8516.0	fFourCorner	44217
U+8516.0	fCantonese	co4; zaa1
U+8516.0	fMandarin	cuo2; cu3; zha1; zhu3
U+8517.0	fUTF8	蔗
U+8517.0	fCNS11643	1-6E52
U+8517.0	fBig5	BDA9
U+8517.0	fGB	0-5561
U+8517.0	fR/S	 140.11
U+8517.0	fTotalStrokes	 15; 17
U+8517.0	fCangjie	TITF
U+8517.0	fFourCorner	44231
U+8517.0	fHakka	  za5; zha5
U+8517.0	fCantonese	ze3
U+8517.0	fMandarin	zhe4; zhu1
U+8517.0	fEnglish	sugar cane
U+8518.0	fUTF8	蔘
U+8518.0	fCNS11643	2-5361
U+8518.0	fBig5	E778
U+8518.0	fGB	3-6961
U+8518.0	fR/S	 140.11
U+8518.0	fTotalStrokes	 15; 17
U+8518.0	fCangjie	TIIH
U+8518.0	fFourCorner	44202
U+8518.0	fHakka	 sem1;  cam1; sam1
U+8518.0	fCantonese	sam1; saam1; saam2
U+8518.0	fMandarin	sen1; san1; shen1; shan3
U+8518.0	fEnglish	ginseng huge
U+8519.0	fUTF8	蔙
U+8519.0	fCNS11643	2-5365
U+8519.0	fBig5	E77C
U+8519.0	fGB	5-6967
U+8519.0	fR/S	 140.11
U+8519.0	fTotalStrokes	 15; 17
U+8519.0	fCangjie	TYSO
U+8519.0	fFourCorner	44281
U+8519.0	fCantonese	syun6; syun4
U+8519.0	fMandarin	xuan4; xuan2
U+851A.0	fUTF8	蔚
U+851A.0	fCNS11643	1-6E54
U+851A.0	fBig5	BDAB
U+851A.0	fGB	0-4E35
U+851A.0	fR/S	 140.11
U+851A.0	fTotalStrokes	 15; 17
U+851A.0	fCangjie	TSFI
U+851A.0	fFourCorner	44240
U+851A.0	fHakka	 wui3; wi3; vui3
U+851A.0	fCantonese	wai3; wat1; wat3
U+851A.0	fMandarin	wei4
U+851A.0	fEnglish	[1] luxuriant (vegetation); thick; ornamental and colorful [2] Artemisia japonica
U+851A.1	fUTF8	蔚
U+851A.1	fCNS11643	1-6E54
U+851A.1	fBig5	BDAB
U+851A.1	fGB	0-4E35
U+851A.1	fR/S	 140.11
U+851A.1	fTotalStrokes	 15; 17
U+851A.1	fCangjie	TSFI
U+851A.1	fFourCorner	44240
U+851A.1	fMandarin	yu4
U+851A.1	fEnglish	[1] a Chinese family name [2] name of a county in Hopeh province
U+851B.0	fUTF8	蔛
U+851B.0	fCNS11643	3-4E4B
U+851B.0	fGB	3-6A62
U+851B.0	fR/S	  140.11
U+851B.0	fTotalStrokes	 15; 17
U+851B.0	fCangjie	TNBJ
U+851B.0	fFourCorner	44240
U+851B.0	fCantonese	huk6
U+851B.0	fMandarin	hu2
U+851B.0	fEnglish	a small orchid-like plant it bears white flowers a dendrobium
U+851C.0	fUTF8	蔜
U+851C.0	fCNS11643	2-5340
U+851C.0	fBig5	E757
U+851C.0	fGB	5-6933
U+851C.0	fR/S	 140.11
U+851C.0	fTotalStrokes	 15; 17
U+851C.0	fCangjie	TGSK
U+851C.0	fFourCorner	44248
U+851C.0	fCantonese	ngou4; ou4 (also)
U+851C.0	fMandarin	ao2; ao4
U+851D.0	fUTF8	蔝
U+851D.0	fCNS11643	2-5354
U+851D.0	fBig5	E76B
U+851D.0	fGB	5-695E
U+851D.0	fR/S	 140.11
U+851D.0	fTotalStrokes	 15; 17
U+851D.0	fCangjie	TBUD
U+851D.0	fFourCorner	44694
U+851D.0	fCantonese	mai5
U+851D.0	fMandarin	mi3
U+851E.0	fUTF8	蔞
U+851E.0	fCNS11643	2-5358
U+851E.0	fBig5	E76F
U+851E.0	fGB	1-5D64
U+851E.0	fR/S	 140.11
U+851E.0	fTotalStrokes	 15; 17
U+851E.0	fCangjie	TLLV
U+851E.0	fFourCorner	44404
U+851E.0	fHakka	  leu2
U+851E.0	fCantonese	lau4; lau5; lau1 (also)
U+851E.0	fMandarin	lou2; ju4; liu3; liu2; l3; lou3
U+851E.0	fEnglish	a kind of artemisia whose tender leaves are edible
U+851F.0	fUTF8	蔟
U+851F.0	fCNS11643	2-533D
U+851F.0	fBig5	E754
U+851F.0	fGB	0-5D7D
U+851F.0	fR/S	 140.11
U+851F.0	fTotalStrokes	 15; 17
U+851F.0	fCangjie	TYSK
U+851F.0	fFourCorner	44234
U+851F.0	fHakka	  cuk8
U+851F.0	fCantonese	cuk1; cau3; coek3; zuk6; cuk6
U+851F.0	fMandarin	cu4; cou4; chuo4
U+851F.0	fEnglish	[1] cluster [2] a frame on which silkworms spin
U+8520.0	fUTF8	蔠
U+8520.0	fCNS11643	2-5362
U+8520.0	fBig5	E779
U+8520.0	fGB	5-6862
U+8520.0	fR/S	 140.11
U+8520.0	fTotalStrokes	 15; 17
U+8520.0	fCangjie	TVFY
U+8520.0	fFourCorner	44933
U+8520.0	fCantonese	zung1
U+8520.0	fMandarin	zhong1
U+8521.0	fUTF8	蔡
U+8521.0	fCNS11643	1-6E5B
U+8521.0	fBig5	BDB2
U+8521.0	fGB	0-324C
U+8521.0	fR/S	 140.11
U+8521.0	fTotalStrokes	 15; 17
U+8521.0	fCangjie	TBOF
U+8521.0	fFourCorner	44901
U+8521.0	fHakka	  cai5
U+8521.0	fCantonese	coi3; saat3
U+8521.0	fMandarin	cai4; sa4; ca3; ca1
U+8521.0	fEnglish	[1] a species of large tortoise or turtle, whose shell was used in divination [2] name of an ancient state in the Spring and Autumn period [3] a Chinese family name
U+8522.0	fUTF8	蔢
U+8522.0	fCNS11643	4-4C6A
U+8522.0	fGB	5-696B
U+8522.0	fR/S	  140.11
U+8522.0	fTotalStrokes	 15; 17
U+8522.0	fCangjie	TEEV
U+8522.0	fFourCorner	44400
U+8522.0	fHakka	  po2; bo5
U+8522.0	fCantonese	po4
U+8522.0	fMandarin	po2
U+8522.0	fEnglish	~ so1, said of a tree or bush, low and wide-spreading
U+8523.0	fUTF8	蔣
U+8523.0	fCNS11643	1-6E5A
U+8523.0	fBig5	BDB1
U+8523.0	fGB	1-3D2F
U+8523.0	fR/S	 140.11
U+8523.0	fTotalStrokes	 15; 17
U+8523.0	fCangjie	TVMI
U+8523.0	fFourCorner	44242
U+8523.0	fHakka	 ziong3; ziong1
U+8523.0	fCantonese	zoeng2; zoeng1
U+8523.0	fMandarin	jiang3; jiang1
U+8523.0	fEnglish	a Chinese family name
U+8524.0	fUTF8	蔤
U+8524.0	fCNS11643	2-5335
U+8524.0	fBig5	E74C
U+8524.0	fGB	3-6A67
U+8524.0	fR/S	 140.11
U+8524.0	fTotalStrokes	 15; 17
U+8524.0	fCangjie	TJPU
U+8524.0	fFourCorner	44773
U+8524.0	fHakka	 mit8; met8
U+8524.0	fCantonese	mat6
U+8524.0	fMandarin	mi4
U+8524.0	fEnglish	rootlets of the lotus
U+8525.0	fUTF8	蔥
U+8525.0	fCNS11643	1-6E5E
U+8525.0	fBig5	BDB5
U+8525.0	fGB	E-3B33
U+8525.0	fR/S	 140.11
U+8525.0	fTotalStrokes	 15; 17
U+8525.0	fCangjie	THWP
U+8525.0	fFourCorner	44332
U+8525.0	fHakka	  cung1
U+8525.0	fCantonese	cung1
U+8525.0	fMandarin	cong1; chuang1
U+8525.0	fEnglish	scallions; onions; leeks
U+8526.0	fUTF8	蔦
U+8526.0	fCNS11643	2-535B
U+8526.0	fBig5	E772
U+8526.0	fGB	1-5C60
U+8526.0	fR/S	 140.11
U+8526.0	fTotalStrokes	 15; 17
U+8526.0	fCangjie	THAF
U+8526.0	fFourCorner	44327
U+8526.0	fHakka	  ngiau3; ngiau1
U+8526.0	fCantonese	niu5; liu5 (also)
U+8526.0	fMandarin	niao3; diao3; diao4
U+8526.0	fEnglish	Ribes ambiguum, a kind of creeping plant; mistletoe
U+8527.0	fUTF8	蔧
U+8527.0	fCNS11643	2-533F
U+8527.0	fBig5	E756
U+8527.0	fGB	3-6A50
U+8527.0	fR/S	 140.11
U+8527.0	fTotalStrokes	 15; 17
U+8527.0	fCangjie	TQJM
U+8527.0	fFourCorner	44171
U+8527.0	fCantonese	seoi6; wai6 (also)
U+8527.0	fMandarin	hui4; sui4
U+8528.0	fUTF8	蔨
U+8528.0	fCNS11643	2-5353
U+8528.0	fBig5	E76A
U+8528.0	fGB	3-6A5D
U+8528.0	fR/S	 140.11
U+8528.0	fTotalStrokes	 15; 17
U+8528.0	fCangjie	TWFU
U+8528.0	fFourCorner	44600
U+8528.0	fCantonese	gyun6; kwan5
U+8528.0	fMandarin	jun4; juan4
U+8529.0	fUTF8	蔩
U+8529.0	fCNS11643	2-5339
U+8529.0	fBig5	E750
U+8529.0	fGB	5-696C
U+8529.0	fR/S	 140.11
U+8529.0	fTotalStrokes	 15; 17
U+8529.0	fCangjie	TJMC
U+8529.0	fFourCorner	44806
U+8529.0	fCantonese	jan4; ji4; jin5
U+8529.0	fMandarin	yin2; yan3; yi2
U+852A.0	fUTF8	蔪
U+852A.0	fCNS11643	2-5347
U+852A.0	fBig5	E75E
U+852A.0	fGB	3-6949
U+852A.0	fR/S	 140.11
U+852A.0	fTotalStrokes	 15; 17
U+852A.0	fCangjie	TJJL
U+852A.0	fFourCorner	44521
U+852A.0	fCantonese	zim6; saam1
U+852A.0	fMandarin	jian4; jian1; shan1; zhan3
U+852B.0	fUTF8	蔫
U+852B.0	fCNS11643	2-5342
U+852B.0	fBig5	E759
U+852B.0	fGB	0-4468
U+852B.0	fR/S	 140.11
U+852B.0	fTotalStrokes	 15; 17
U+852B.0	fCangjie	TMYF
U+852B.0	fFourCorner	44327
U+852B.0	fHakka	 jen1; jen2; ren1; jan1
U+852B.0	fCantonese	jin1
U+852B.0	fMandarin	nian1; yan4; nian2
U+852B.0	fEnglish	[1] withering; fading; decaying (plants) [2] spiritless; ennui; listless [3] calm and quiet; expressionless
U+852B.1	fUTF8	蔫
U+852B.1	fCNS11643	2-5342
U+852B.1	fBig5	E759
U+852B.1	fGB	0-4468
U+852B.1	fR/S	 140.11
U+852B.1	fTotalStrokes	 15; 17
U+852B.1	fCangjie	TMYF
U+852B.1	fFourCorner	44327
U+852B.1	fMandarin	yan1
U+852B.1	fEnglish	stale; rotting; decaying
U+852C.0	fUTF8	蔬
U+852C.0	fCNS11643	1-6E56
U+852C.0	fBig5	BDAD
U+852C.0	fGB	0-4A5F
U+852C.0	fR/S	 140.11
U+852C.0	fTotalStrokes	 15; 17
U+852C.0	fCangjie	TNMU
U+852C.0	fFourCorner	44113
U+852C.0	fHakka	  so1; su1; se1; sii1
U+852C.0	fCantonese	so1
U+852C.0	fMandarin	shu1; shu3; xu1; shu4; su1
U+852C.0	fEnglish	[1] vegetables; greens [2] vegetarian diet; vegetable food
U+852D.0	fUTF8	蔭
U+852D.0	fCNS11643	1-6E57
U+852D.0	fBig5	BDAE
U+852D.0	fGB	1-5271
U+852D.0	fR/S	 140.11
U+852D.0	fTotalStrokes	 15; 17
U+852D.0	fCangjie	TNLI
U+852D.0	fFourCorner	44231
U+852D.0	fHakka	  jim5; {lim5}; rim5; jim1; ngim5
U+852D.0	fCantonese	jam3; jam1
U+852D.0	fMandarin	yin4; yin1
U+852D.0	fEnglish	[1] shade; shade of trees [2] shelter; protect [3] (with) the support or blessing of
U+852E.0	fUTF8	蔮
U+852E.0	fCNS11643	2-5355
U+852E.0	fBig5	E76C
U+852E.0	fGB	E-3B34
U+852E.0	fR/S	 140.11
U+852E.0	fTotalStrokes	 15; 17
U+852E.0	fCangjie	TWIM
U+852E.0	fFourCorner	44600
U+852E.0	fCantonese	gwok3; gok3 (also); kui3
U+852E.0	fMandarin	kui4
U+852F.0	fUTF8	蔯
U+852F.0	fCNS11643	2-5366
U+852F.0	fBig5	E77D
U+852F.0	fGB	5-6842
U+852F.0	fR/S	 140.11
U+852F.0	fTotalStrokes	 15; 17
U+852F.0	fCangjie	TNLW
U+852F.0	fFourCorner	44296
U+852F.0	fHakka	 chin2; cihn2
U+852F.0	fCantonese	can4
U+852F.0	fMandarin	chen2
U+852F.0	fEnglish	a variety of artemisia
U+8530.0	fUTF8	蔰
U+8530.0	fCNS11643	2-5363
U+8530.0	fBig5	E77A
U+8530.0	fGB	E-3B35
U+8530.0	fR/S	 140.11
U+8530.0	fTotalStrokes	 15; 17
U+8530.0	fCangjie	THSU
U+8530.0	fFourCorner	44217
U+8530.0	fCantonese	wu6
U+8530.0	fMandarin	hu4
U+8531.0	fUTF8	蔱
U+8531.0	fCNS11643	2-535A
U+8531.0	fBig5	E771
U+8531.0	fGB	E-3B36
U+8531.0	fR/S	 140.11; 140.10
U+8531.0	fTotalStrokes	 15; 17
U+8531.0	fCangjie	TKCE
U+8531.0	fFourCorner	44947
U+8531.0	fCantonese	saat3
U+8531.0	fMandarin	sha1
U+8532.0	fUTF8	蔲
U+8532.0	fCNS11643	4-4C78
U+8532.0	fGB	E-3B37
U+8532.0	fR/S	 140.11; 140.10
U+8532.0	fTotalStrokes	 15; 17; 14
U+8532.0	fCangjie	TJMK
U+8532.0	fFourCorner	44210
U+8532.0	fCantonese	kau3
U+8532.0	fMandarin	kou4
U+8532.0	fEnglish	nutmeg
U+8533.0	fUTF8	蔳
U+8533.0	fCNS11643	3-4E4D
U+8533.0	fGB	5-6968
U+8533.0	fR/S	  140.11
U+8533.0	fTotalStrokes	 15; 17
U+8533.0	fCangjie	TEQB
U+8533.0	fFourCorner	44120
U+8533.0	fCantonese	sin6
U+8533.0	fMandarin	qian4
U+8534.0	fUTF8	蔴
U+8534.0	fCNS11643	3-4E4E
U+8534.0	fGB	8-2F6D
U+8534.0	fR/S	 140.11
U+8534.0	fTotalStrokes	 15; 17
U+8534.0	fCangjie	TIDD
U+8534.0	fFourCorner	44290
U+8534.0	fHakka	 ma2; ma3
U+8534.0	fCantonese	maa4
U+8534.0	fMandarin	ma2; ma1
U+8534.0	fEnglish	[1] hemp [2] sesame; sesamum
U+8535.0	fUTF8	蔵
U+8535.0	fCNS11643	3-4E52
U+8535.0	fGB	5-695C
U+8535.0	fR/S	 140.11
U+8535.0	fTotalStrokes	 15; 17
U+8535.0	fCangjie	TIHS
U+8535.0	fFourCorner	44250
U+8535.0	fCantonese	cong4
U+8535.0	fMandarin	cang2; zang1; zang4
U+8535.0	fEnglish	 hide; conceal hoard; store up
U+8536.0	fUTF8	蔶
U+8536.0	fGB	E-3B38
U+8536.0	fR/S	 140.11
U+8536.0	fTotalStrokes	 15; 17
U+8536.0	fCangjie	TQMC
U+8536.0	fCantonese	zaak3
U+8536.0	fMandarin	ze2
U+8537.0	fUTF8	蔷
U+8537.0	fGB	0-473E
U+8537.0	fR/S	 140.11
U+8537.0	fTotalStrokes	 15; 17
U+8537.0	fCangjie	TGCW
U+8537.0	fHakka	 siong2
U+8537.0	fCantonese	coeng4
U+8537.0	fMandarin	qiang2; se4
U+8537.0	fEnglish	simplified form of &#x8594;
U+8538.0	fUTF8	蔸
U+8538.0	fCNS11643	4-4C74
U+8538.0	fGB	0-5D7A
U+8538.0	fR/S	  140.11
U+8538.0	fTotalStrokes	 15; 17
U+8538.0	fCangjie	THVU
U+8538.0	fFourCorner	44210
U+8538.0	fCantonese	dau1
U+8538.0	fMandarin	dou1
U+8538.0	fEnglish	counter for plant
U+8539.0	fUTF8	蔹
U+8539.0	fGB	0-5D7C
U+8539.0	fR/S	 140.11
U+8539.0	fTotalStrokes	 15; 17
U+8539.0	fCangjie	TOMK
U+8539.0	fCantonese	lim5; lim6 (also); lim4
U+8539.0	fMandarin	lian4; lian2; lian3
U+8539.0	fEnglish	simplified form of &#x861E;
U+853A.0	fUTF8	蔺
U+853A.0	fGB	0-5D7E
U+853A.0	fR/S	 140.11
U+853A.0	fTotalStrokes	 15; 17
U+853A.0	fCangjie	TLSG
U+853A.0	fHakka	 nin3
U+853A.0	fCantonese	leon6
U+853A.0	fMandarin	lin4; lian4
U+853A.0	fEnglish	simplified form of &#x85FA;
U+853B.0	fUTF8	蔻
U+853B.0	fCNS11643	2-5336
U+853B.0	fBig5	E74D
U+853B.0	fGB	0-5E22
U+853B.0	fR/S	 140.11
U+853B.0	fTotalStrokes	 15; 17
U+853B.0	fCangjie	TJME
U+853B.0	fFourCorner	44214
U+853B.0	fHakka	 keu5; kiu5
U+853B.0	fCantonese	kau3
U+853B.0	fMandarin	kou4; hou4
U+853B.0	fEnglish	cardamom seeds
U+853C.0	fUTF8	蔼
U+853C.0	fGB	0-302A
U+853C.0	fR/S	 140.11
U+853C.0	fTotalStrokes	 15; 17
U+853C.0	fCangjie	TIVV
U+853C.0	fHakka	 ai5
U+853C.0	fCantonese	oi2; ngoi2 (also)
U+853C.0	fMandarin	ai3; ai4
U+853C.0	fEnglish	simplified form of &#x85F9;
U+853D.0	fUTF8	蔽
U+853D.0	fCNS11643	1-6E53
U+853D.0	fBig5	BDAA
U+853D.0	fGB	0-314E
U+853D.0	fR/S	 140.12
U+853D.0	fTotalStrokes	 16; 18
U+853D.0	fCangjie	TFBK
U+853D.0	fFourCorner	44248
U+853D.0	fHakka	  bi5
U+853D.0	fCantonese	bai3
U+853D.0	fMandarin	bi4; fu2; fu4; pie1
U+853D.0	fEnglish	[1] cover; cover up [2] hide; conceal; shelter [3] screen; separate
U+853E.0	fUTF8	蔾
U+853E.0	fCNS11643	2-5974
U+853E.0	fBig5	EB49
U+853E.0	fGB	E-3B39
U+853E.0	fR/S	 140.12
U+853E.0	fTotalStrokes	 16; 18
U+853E.0	fCangjie	THHD
U+853E.0	fFourCorner	44904
U+853E.0	fHakka	 lai2
U+853E.0	fCantonese	lai4; leoi5
U+853E.0	fMandarin	li2
U+853E.0	fEnglish	kind of bramble
U+853F.0	fUTF8	蔿
U+853F.0	fCNS11643	3-525A
U+853F.0	fGB	3-6845
U+853F.0	fR/S	 140.12
U+853F.0	fTotalStrokes	 16; 18
U+853F.0	fCangjie	TBHF
U+853F.0	fFourCorner	44220
U+853F.0	fHakka	 wi3; wui3; vui3
U+853F.0	fCantonese	wai2
U+853F.0	fMandarin	wei3; wei2
U+853F.0	fEnglish	 a Chinese family name a place name
U+8540.0	fUTF8	蕀
U+8540.0	fCNS11643	2-596B
U+8540.0	fBig5	EB40
U+8540.0	fGB	3-6A6C
U+8540.0	fR/S	 140.12
U+8540.0	fTotalStrokes	 16; 18
U+8540.0	fCangjie	TDBB
U+8540.0	fFourCorner	44992
U+8540.0	fCantonese	gik1
U+8540.0	fMandarin	ji2
U+8541.0	fUTF8	蕁
U+8541.0	fCNS11643	2-596E
U+8541.0	fBig5	EB43
U+8541.0	fGB	1-5D21
U+8541.0	fR/S	 140.12
U+8541.0	fTotalStrokes	 16; 18
U+8541.0	fCangjie	TSMI
U+8541.0	fFourCorner	44346
U+8541.0	fHakka	 tam2; tam3; cim2
U+8541.0	fCantonese	cam4; cim4 (also); taam4; tam4
U+8541.0	fMandarin	xun2; qian2; tan2
U+8541.0	fEnglish	used in urtica
U+8542.0	fUTF8	蕂
U+8542.0	fCNS11643	3-5252
U+8542.0	fGB	3-6A78
U+8542.0	fR/S	  140.12
U+8542.0	fTotalStrokes	 16; 18
U+8542.0	fCangjie	TBFS
U+8542.0	fFourCorner	44220
U+8542.0	fHakka	 shin5; siin5
U+8542.0	fCantonese	sing3
U+8542.0	fMandarin	sheng4
U+8542.0	fEnglish	sesame
U+8543.0	fUTF8	蕃
U+8543.0	fCNS11643	1-7225
U+8543.0	fBig5	BFBB
U+8543.0	fGB	0-5E2C
U+8543.0	fR/S	 140.12
U+8543.0	fTotalStrokes	 16; 18
U+8543.0	fCangjie	THDW
U+8543.0	fFourCorner	44609
U+8543.0	fHakka	  fan1; pan1; bi1; fan2
U+8543.0	fCantonese	faan4; faan1; pei4
U+8543.0	fMandarin	fan2; pi2; bo2; bo1
U+8543.0	fEnglish	[1] luxuriant; flourishing (vegetation) [2] increase; multiply; propagate [3] numerous; plentiful
U+8543.1	fUTF8	蕃
U+8543.1	fCNS11643	1-7225
U+8543.1	fBig5	BFBB
U+8543.1	fGB	0-5E2C
U+8543.1	fR/S	 140.12
U+8543.1	fTotalStrokes	 16; 18
U+8543.1	fCangjie	THDW
U+8543.1	fFourCorner	44609
U+8543.1	fMandarin	fan1
U+8543.1	fEnglish	barbarians (as opposed to native Chinese); foreign; uncivilized
U+8544.0	fUTF8	蕄
U+8544.0	fCNS11643	2-5970
U+8544.0	fBig5	EB45
U+8544.0	fGB	5-683D
U+8544.0	fR/S	 140.12
U+8544.0	fTotalStrokes	 16; 18
U+8544.0	fCangjie	TANP
U+8544.0	fFourCorner	44227
U+8544.0	fCantonese	mang4; man4
U+8544.0	fMandarin	meng2
U+8545.0	fUTF8	蕅
U+8545.0	fCNS11643	2-5965
U+8545.0	fBig5	EAF9
U+8545.0	fGB	5-6A25
U+8545.0	fR/S	 140.12
U+8545.0	fTotalStrokes	 16; 18
U+8545.0	fCangjie	TEWB
U+8545.0	fFourCorner	55327
U+8545.0	fCantonese	ngau5; au5 (also)
U+8545.0	fMandarin	ou3
U+8546.0	fUTF8	蕆
U+8546.0	fCNS11643	2-596C
U+8546.0	fBig5	EB41
U+8546.0	fGB	1-5D5B
U+8546.0	fR/S	 140.12
U+8546.0	fTotalStrokes	 16; 18
U+8546.0	fCangjie	TIHC
U+8546.0	fFourCorner	44250
U+8546.0	fHakka	 zhan3; zan3; chan3; zhen3
U+8546.0	fCantonese	cin2; zin2 (also)
U+8546.0	fMandarin	chan3
U+8546.0	fEnglish	 finish; complete
U+8547.0	fUTF8	蕇
U+8547.0	fCNS11643	2-5972
U+8547.0	fBig5	EB47
U+8547.0	fGB	5-6857
U+8547.0	fR/S	 140.12
U+8547.0	fTotalStrokes	 16; 18
U+8547.0	fCangjie	TRRJ
U+8547.0	fFourCorner	44406
U+8547.0	fCantonese	din2
U+8547.0	fMandarin	dian3
U+8548.0	fUTF8	蕈
U+8548.0	fCNS11643	1-7222
U+8548.0	fBig5	BFB8
U+8548.0	fGB	0-5E26
U+8548.0	fR/S	 140.12
U+8548.0	fTotalStrokes	 16; 18
U+8548.0	fCangjie	TMWJ
U+8548.0	fFourCorner	44406
U+8548.0	fHakka	 cim3; tam2
U+8548.0	fCantonese	cam5; seon3 (also); taam4
U+8548.0	fMandarin	xun4; tan2; jun4; dan4; dian4; hun1; xin2; xin4
U+8548.0	fEnglish	[1] mushrooms; fungus [2] mildew; mold
U+8549.0	fUTF8	蕉
U+8549.0	fCNS11643	1-7226
U+8549.0	fBig5	BFBC
U+8549.0	fGB	0-3D36
U+8549.0	fR/S	 140.12
U+8549.0	fTotalStrokes	 16; 18
U+8549.0	fCangjie	TOGF
U+8549.0	fFourCorner	44331
U+8549.0	fHakka	  ziau1; zeu1
U+8549.0	fCantonese	ziu1; ciu4
U+8549.0	fMandarin	jiao1; qiao2; qiao1
U+8549.0	fEnglish	[1] the banana [2] the plantain
U+854A.0	fUTF8	蕊
U+854A.0	fCNS11643	1-717E
U+854A.0	fBig5	BFB6
U+854A.0	fGB	0-486F
U+854A.0	fR/S	 140.12
U+854A.0	fTotalStrokes	 16; 18
U+854A.0	fCangjie	TPPP
U+854A.0	fFourCorner	44333
U+854A.0	fHakka	 lui3;  lui2; jui3
U+854A.0	fCantonese	jeoi5; jeoi6 (also)
U+854A.0	fMandarin	rui3; juan3; jue2
U+854A.0	fEnglish	[1] unopened flower; flower bud [2] the stamens or pistils of a flower
U+854B.0	fUTF8	蕋
U+854B.0	fCNS11643	3-5259
U+854B.0	fGB	E-3B3A
U+854B.0	fR/S	 140.12
U+854B.0	fTotalStrokes	 16; 18
U+854B.0	fCangjie	TYMM
U+854B.0	fFourCorner	44110
U+854B.0	fHakka	  lui3;  lui2
U+854B.0	fCantonese	jeoi5; jeoi6 (also); jeoi4
U+854B.0	fMandarin	rui3
U+854B.0	fEnglish	unopened flowers; flower buds
U+854C.0	fUTF8	蕌
U+854C.0	fCNS11643	3-5250
U+854C.0	fGB	E-3B3B
U+854C.0	fR/S	  140.12
U+854C.0	fTotalStrokes	 16; 18
U+854C.0	fCangjie	TAAA
U+854C.0	fFourCorner	44660
U+854C.0	fCantonese	leoi5
U+854C.0	fMandarin	lei3
U+854D.0	fUTF8	蕍
U+854D.0	fCNS11643	2-5967
U+854D.0	fBig5	EAFB
U+854D.0	fGB	3-6A7D
U+854D.0	fR/S	 140.12
U+854D.0	fTotalStrokes	 16; 18
U+854D.0	fCangjie	TEON
U+854D.0	fFourCorner	44121
U+854D.0	fCantonese	jyu4
U+854D.0	fMandarin	yu2
U+854E.0	fUTF8	蕎
U+854E.0	fCNS11643	2-5977
U+854E.0	fBig5	EB4C
U+854E.0	fGB	1-5C71
U+854E.0	fR/S	 140.12
U+854E.0	fTotalStrokes	 16; 18
U+854E.0	fCangjie	THKB
U+854E.0	fFourCorner	44227
U+854E.0	fHakka	 kiau2; kiau1
U+854E.0	fCantonese	kiu4
U+854E.0	fMandarin	qiao2; jiao1
U+854E.0	fEnglish	buckwheat
U+854F.0	fUTF8	蕏
U+854F.0	fCNS11643	4-4D25
U+854F.0	fGB	3-6A61
U+854F.0	fR/S	 140.12; 140.11
U+854F.0	fTotalStrokes	 16; 15; 18
U+854F.0	fCangjie	TKHA
U+854F.0	fFourCorner	44260
U+854F.0	fCantonese	cyu4; zyu1
U+854F.0	fMandarin	zhu1; chu2; zha1
U+8550.0	fUTF8	蕐
U+8550.0	fCNS11643	4-5333
U+8550.0	fGB	E-3B3C
U+8550.0	fR/S	 140.12
U+8550.0	fTotalStrokes	 16; 18
U+8550.0	fCangjie	TMOQ
U+8550.0	fFourCorner	44500
U+8550.0	fCantonese	waa4
U+8550.0	fMandarin	hua2; hua1
U+8551.0	fUTF8	蕑
U+8551.0	fCNS11643	2-5971
U+8551.0	fBig5	EB46
U+8551.0	fGB	3-6935
U+8551.0	fR/S	 140.12
U+8551.0	fTotalStrokes	 16; 18
U+8551.0	fCangjie	TANB
U+8551.0	fFourCorner	44227
U+8551.0	fHakka	 han2;  gan1; ham2
U+8551.0	fCantonese	gaan1
U+8551.0	fMandarin	jian1
U+8551.0	fEnglish	Valeriana villosa; climbing plant
U+8552.0	fUTF8	蕒
U+8552.0	fCNS11643	4-5324
U+8552.0	fGB	1-5D24
U+8552.0	fR/S	 140.12; 140.13
U+8552.0	fTotalStrokes	 16; 18; 17
U+8552.0	fCangjie	TWLC
U+8552.0	fFourCorner	44800
U+8552.0	fHakka	  mak7;  mai3
U+8552.0	fCantonese	maai5
U+8552.0	fMandarin	mai3
U+8552.0	fEnglish	an edible vegetable
U+8553.0	fUTF8	蕓
U+8553.0	fCNS11643	2-5968
U+8553.0	fBig5	EAFC
U+8553.0	fGB	1-5C3F
U+8553.0	fR/S	 140.12
U+8553.0	fTotalStrokes	 16; 18
U+8553.0	fCangjie	TMBI
U+8553.0	fFourCorner	44131
U+8553.0	fHakka	 jun2; run2; jun3
U+8553.0	fCantonese	wan4
U+8553.0	fMandarin	yun2
U+8553.0	fEnglish	used in combination
U+8554.0	fUTF8	蕔
U+8554.0	fCNS11643	2-5A22
U+8554.0	fBig5	EB55
U+8554.0	fGB	3-6A6A
U+8554.0	fR/S	 140.12
U+8554.0	fTotalStrokes	 16; 18
U+8554.0	fCangjie	TGJE
U+8554.0	fFourCorner	44447
U+8554.0	fCantonese	bou1
U+8554.0	fMandarin	bao1
U+8555.0	fUTF8	蕕
U+8555.0	fCNS11643	2-597A
U+8555.0	fBig5	EB4F
U+8555.0	fGB	1-5D35
U+8555.0	fR/S	 140.12
U+8555.0	fTotalStrokes	 16; 18
U+8555.0	fCangjie	TKHW
U+8555.0	fFourCorner	44261
U+8555.0	fHakka	 ju2; riu2
U+8555.0	fCantonese	jau4
U+8555.0	fMandarin	you2; you3
U+8555.0	fEnglish	Caryopteris divaricata, a water plant with a foul smell that lasts very long; evil person
U+8556.0	fUTF8	蕖
U+8556.0	fCNS11643	2-5964
U+8556.0	fBig5	EAF8
U+8556.0	fGB	0-5E21
U+8556.0	fR/S	 140.12
U+8556.0	fTotalStrokes	 16; 18
U+8556.0	fCangjie	TESD
U+8556.0	fFourCorner	44904
U+8556.0	fHakka	 ki2; gi1
U+8556.0	fCantonese	keoi4
U+8556.0	fMandarin	qu2
U+8556.0	fEnglish	[1] taro [2] used in combination
U+8557.0	fUTF8	蕗
U+8557.0	fCNS11643	2-5E72
U+8557.0	fBig5	EE46
U+8557.0	fGB	3-6B2D
U+8557.0	fR/S	 140.12
U+8557.0	fTotalStrokes	 16; 18
U+8557.0	fCangjie	TRMR
U+8557.0	fFourCorner	44164
U+8557.0	fCantonese	lou6
U+8557.0	fMandarin	lu4
U+8558.0	fUTF8	蕘
U+8558.0	fCNS11643	2-596A
U+8558.0	fBig5	EAFE
U+8558.0	fGB	1-5C69
U+8558.0	fR/S	 140.12
U+8558.0	fTotalStrokes	 16; 18
U+8558.0	fCangjie	TGGU
U+8558.0	fFourCorner	44211
U+8558.0	fHakka	  ngiau2; gniau3; nio1
U+8558.0	fCantonese	jiu4
U+8558.0	fMandarin	rao2; yao2; nao2; rao4
U+8558.0	fEnglish	[1] grass of rushes, used for fuel [2] firewood gatherers [3] Wikstroemia japonica, a kind of shrub about three feet tall with yellow flowers, whose bark is used for making paper [4] the turnip
U+8559.0	fUTF8	蕙
U+8559.0	fCNS11643	1-7221
U+8559.0	fBig5	BFB7
U+8559.0	fGB	0-5E25
U+8559.0	fR/S	 140.12
U+8559.0	fTotalStrokes	 16; 18
U+8559.0	fCangjie	TJIP
U+8559.0	fFourCorner	44333
U+8559.0	fHakka	 fui5; fi5
U+8559.0	fCantonese	wai6
U+8559.0	fMandarin	hui4
U+8559.0	fEnglish	[1] a species of fragrant orchid [2] a species of fragrant grass with red flowers and black seeds in early fall
U+855A.0	fUTF8	蕚
U+855A.0	fCNS11643	3-5258
U+855A.0	fGB	E-3B3D
U+855A.0	fR/S	 140.12
U+855A.0	fTotalStrokes	 16; 18
U+855A.0	fCangjie	TRRS
U+855A.0	fFourCorner	44200
U+855A.0	fCantonese	ngok6; ok6 (also)
U+855A.0	fMandarin	e4
U+855A.0	fEnglish	 calyx of flower younger brother
U+855B.0	fUTF8	蕛
U+855B.0	fCNS11643	2-5975
U+855B.0	fBig5	EB4A
U+855B.0	fGB	5-6977
U+855B.0	fR/S	 140.12
U+855B.0	fTotalStrokes	 16; 18
U+855B.0	fCangjie	THDH
U+855B.0	fFourCorner	44927
U+855B.0	fCantonese	tai4
U+855B.0	fMandarin	ti2; teng2
U+855C.0	fUTF8	蕜
U+855C.0	fCNS11643	3-5254
U+855C.0	fGB	5-6973
U+855C.0	fR/S	  140.12
U+855C.0	fTotalStrokes	 16; 18
U+855C.0	fCangjie	TLYP
U+855C.0	fFourCorner	44330
U+855C.0	fCantonese	fei2
U+855C.0	fMandarin	fei3
U+855D.0	fUTF8	蕝
U+855D.0	fCNS11643	2-5A21
U+855D.0	fBig5	EB54
U+855D.0	fGB	3-6A30
U+855D.0	fR/S	 140.12
U+855D.0	fTotalStrokes	 16; 18
U+855D.0	fCangjie	TVFU
U+855D.0	fFourCorner	44917
U+855D.0	fHakka	 ziet8; ciet8; cot7
U+855D.0	fCantonese	zyut3 (also); zyut6
U+855D.0	fMandarin	jue2; zui4
U+855D.0	fEnglish	used in combination
U+855E.0	fUTF8	蕞
U+855E.0	fCNS11643	1-7229
U+855E.0	fBig5	BFBF
U+855E.0	fGB	0-5E29
U+855E.0	fR/S	 140.12
U+855E.0	fTotalStrokes	 16; 18
U+855E.0	fCangjie	TASE
U+855E.0	fFourCorner	44147
U+855E.0	fHakka	 cot7; cui5; zui5
U+855E.0	fCantonese	zeoi3; zeoi6; zyut3
U+855E.0	fMandarin	zui4; jue2; zhuo2
U+855E.0	fEnglish	little; very small; tiny
U+855F.0	fUTF8	蕟
U+855F.0	fCNS11643	3-5256
U+855F.0	fGB	E-3B3E
U+855F.0	fR/S	 140.12
U+855F.0	fTotalStrokes	 16; 18
U+855F.0	fCangjie	TNOE
U+855F.0	fFourCorner	44240
U+855F.0	fCantonese	faat3
U+855F.0	fMandarin	fei4; fa4
U+8560.0	fUTF8	蕠
U+8560.0	fCNS11643	2-597C
U+8560.0	fBig5	EB51
U+8560.0	fGB	5-6A2B
U+8560.0	fR/S	 140.12
U+8560.0	fTotalStrokes	 16; 18
U+8560.0	fCangjie	TVRF
U+8560.0	fFourCorner	44903
U+8560.0	fCantonese	jyu4; jyu6
U+8560.0	fMandarin	ru2; nu2
U+8561.0	fUTF8	蕡
U+8561.0	fCNS11643	2-5969
U+8561.0	fBig5	EAFD
U+8561.0	fGB	3-6965
U+8561.0	fR/S	 140.12
U+8561.0	fTotalStrokes	 16; 18
U+8561.0	fCangjie	TJTC
U+8561.0	fFourCorner	44806
U+8561.0	fHakka	  fun2
U+8561.0	fCantonese	fan4; fan5
U+8561.0	fMandarin	fen2; fan2; fei4; fen4; pei4
U+8561.0	fEnglish	[1] the seeds of hemp [2] large and abundant (fruit) [3] a Chinese family name
U+8562.0	fUTF8	蕢
U+8562.0	fCNS11643	2-596F
U+8562.0	fBig5	EB44
U+8562.0	fGB	1-5D5E
U+8562.0	fR/S	 140.12
U+8562.0	fTotalStrokes	 16; 18
U+8562.0	fCangjie	TLMC
U+8562.0	fFourCorner	44806
U+8562.0	fHakka	 kui3; kui6; kui5; kwi5; k'wui5
U+8562.0	fCantonese	gwai6
U+8562.0	fMandarin	kui4; kuai4
U+8562.0	fEnglish	[1] the edible amaranth, a vegetable with a red stalk [2] straw basket [3] a Chinese family name
U+8563.0	fUTF8	蕣
U+8563.0	fCNS11643	2-5973
U+8563.0	fBig5	EB48
U+8563.0	fGB	3-6A77
U+8563.0	fR/S	 140.12
U+8563.0	fTotalStrokes	 16; 18
U+8563.0	fCangjie	TBBQ
U+8563.0	fFourCorner	44252
U+8563.0	fHakka	 shun5; sun5
U+8563.0	fCantonese	seon3
U+8563.0	fMandarin	shun4
U+8563.0	fEnglish	hibiscus
U+8564.0	fUTF8	蕤
U+8564.0	fCNS11643	2-596D
U+8564.0	fBig5	EB42
U+8564.0	fGB	0-5E28
U+8564.0	fR/S	 140.12
U+8564.0	fTotalStrokes	 16; 18
U+8564.0	fCangjie	TMOM
U+8564.0	fFourCorner	44231
U+8564.0	fHakka	 wui1; wi1; sui1; vui1
U+8564.0	fCantonese	jeoi4 (also); jeoi6
U+8564.0	fMandarin	rui2
U+8564.0	fEnglish	[1] drooping leaves; flowers or fruits [2] a shrub about three feet tall with narrow leaves and white flowers
U+8565.0	fUTF8	蕥
U+8565.0	fCNS11643	2-5A23
U+8565.0	fBig5	EB56
U+8565.0	fGB	5-6972
U+8565.0	fR/S	 140.12
U+8565.0	fTotalStrokes	 16; 18
U+8565.0	fCangjie	TMHG
U+8565.0	fFourCorner	44214
U+8565.0	fHakka	 nga3
U+8565.0	fCantonese	ngaa5; aa5 (also)
U+8565.0	fMandarin	ya3
U+8566.0	fUTF8	蕦
U+8566.0	fCNS11643	2-597E
U+8566.0	fBig5	EB53
U+8566.0	fGB	5-6921
U+8566.0	fR/S	 140.12
U+8566.0	fTotalStrokes	 16; 18
U+8566.0	fCangjie	THHC
U+8566.0	fFourCorner	44286
U+8566.0	fCantonese	seoi1
U+8566.0	fMandarin	xu1
U+8567.0	fUTF8	蕧
U+8567.0	fCNS11643	2-597B
U+8567.0	fBig5	EB50
U+8567.0	fGB	3-6A76
U+8567.0	fR/S	 140.12
U+8567.0	fTotalStrokes	 16; 18
U+8567.0	fCangjie	THOE
U+8567.0	fFourCorner	44247
U+8567.0	fCantonese	fuk6 (also); fuk1
U+8567.0	fMandarin	fu4
U+8568.0	fUTF8	蕨
U+8568.0	fCNS11643	1-7223
U+8568.0	fBig5	BFB9
U+8568.0	fGB	0-5E27
U+8568.0	fR/S	 140.12
U+8568.0	fTotalStrokes	 16; 18
U+8568.0	fCangjie	TMTO
U+8568.0	fFourCorner	44282
U+8568.0	fHakka	 giet7; kiet7; get7; ket7
U+8568.0	fCantonese	kyut3
U+8568.0	fMandarin	jue2
U+8568.0	fEnglish	Pteris aquilina, the common bracken
U+8569.0	fUTF8	蕩
U+8569.0	fCNS11643	1-7224
U+8569.0	fBig5	BFBA
U+8569.0	fGB	1-3534
U+8569.0	fR/S	 140.12
U+8569.0	fTotalStrokes	 16; 18
U+8569.0	fCangjie	TEAH
U+8569.0	fFourCorner	44127
U+8569.0	fHakka	  tong3; tong6; tong5
U+8569.0	fCantonese	dong6
U+8569.0	fMandarin	dang4; tang1; tang4
U+8569.0	fEnglish	[1] pond; pool [2] wash away; cleanse [3] shake; oscillate; move to and fro; unsettled; vagrant; loaf about [4] dissipated; wanton; debauched; licentious; of loose morals [5] agitated; disturbed [6] vast; large; magnificent
U+856A.0	fUTF8	蕪
U+856A.0	fCNS11643	1-7228
U+856A.0	fBig5	BFBE
U+856A.0	fGB	1-4E5F
U+856A.0	fR/S	 140.12
U+856A.0	fTotalStrokes	 16; 18
U+856A.0	fCangjie	TOTF
U+856A.0	fFourCorner	44331
U+856A.0	fHakka	  wu2; vu2
U+856A.0	fCantonese	mou4
U+856A.0	fMandarin	wu2; wu3; wu4
U+856A.0	fEnglish	[1] luxurious growth of weeds [2] decayed or rotten vegetation [3] confused; mixed up; in disorder [4] waste; neglected (land)
U+856B.0	fUTF8	蕫
U+856B.0	fCNS11643	2-5966
U+856B.0	fBig5	EAFA
U+856B.0	fGB	3-6A7A
U+856B.0	fR/S	 140.12
U+856B.0	fTotalStrokes	 16; 18
U+856B.0	fCangjie	TYTG
U+856B.0	fFourCorner	55104
U+856B.0	fHakka	 dung3;  tung2
U+856B.0	fCantonese	dung2
U+856B.0	fMandarin	tong2; dong3
U+856B.0	fEnglish	 correct
U+856C.0	fUTF8	蕬
U+856C.0	fCNS11643	2-5A24
U+856C.0	fBig5	EB57
U+856C.0	fGB	5-6769
U+856C.0	fR/S	 140.12
U+856C.0	fTotalStrokes	 16; 18
U+856C.0	fCangjie	TVFF
U+856C.0	fFourCorner	44993
U+856C.0	fHakka	 sii1; se1
U+856C.0	fCantonese	si1
U+856C.0	fMandarin	si1
U+856C.0	fEnglish	dodder
U+856D.0	fUTF8	蕭
U+856D.0	fCNS11643	1-7227
U+856D.0	fBig5	BFBD
U+856D.0	fGB	1-4F74
U+856D.0	fR/S	 140.12
U+856D.0	fTotalStrokes	 16; 18
U+856D.0	fCangjie	TLX
U+856D.0	fFourCorner	44227
U+856D.0	fHakka	  siau1; seu1
U+856D.0	fCantonese	siu1
U+856D.0	fMandarin	xiao1
U+856D.0	fEnglish	[1] a common variety of artemisia; oxtail-southernwood [2] reverent; respectful [3] quiet; lonely; desolate [4] a Chinese family name
U+856E.0	fUTF8	蕮
U+856E.0	fCNS11643	2-5978
U+856E.0	fBig5	EB4D
U+856E.0	fGB	3-6A74
U+856E.0	fR/S	 140.12
U+856E.0	fTotalStrokes	 16; 18
U+856E.0	fCangjie	THXF
U+856E.0	fFourCorner	44327
U+856E.0	fHakka	 sit7
U+856E.0	fCantonese	sik1
U+856E.0	fMandarin	xi4
U+856E.0	fEnglish	a kind of purslane
U+856F.0	fUTF8	蕯
U+856F.0	fCNS11643	3-524F
U+856F.0	fGB	3-6A69
U+856F.0	fR/S	  140.12
U+856F.0	fTotalStrokes	 16; 18
U+856F.0	fCangjie	TNLM
U+856F.0	fFourCorner	44210
U+856F.0	fCantonese	saat3
U+856F.0	fMandarin	long2; sa4
U+8570.0	fUTF8	蕰
U+8570.0	fCNS11643	3-5255
U+8570.0	fGB	3-6A7C
U+8570.0	fR/S	 140.12
U+8570.0	fTotalStrokes	 16; 18
U+8570.0	fCangjie	TEAT
U+8570.0	fFourCorner	44110
U+8570.0	fCantonese	wan1
U+8570.0	fMandarin	yun4; wen1
U+8570.0	fEnglish	the hippuris or mare's tail plant
U+8571.0	fUTF8	蕱
U+8571.0	fCNS11643	2-5976
U+8571.0	fBig5	EB4B
U+8571.0	fGB	E-3B3F
U+8571.0	fR/S	 140.12
U+8571.0	fTotalStrokes	 16; 18
U+8571.0	fCangjie	THDB
U+8571.0	fFourCorner	29920
U+8571.0	fCantonese	saau2
U+8571.0	fMandarin	shao1
U+8572.0	fUTF8	蕲
U+8572.0	fGB	0-5E2D
U+8572.0	fR/S	 140.12
U+8572.0	fTotalStrokes	 16; 18
U+8572.0	fCangjie	TCJL
U+8572.0	fHakka	 ki2
U+8572.0	fCantonese	kei4
U+8572.0	fMandarin	qi2; ji1; qin2
U+8572.0	fEnglish	simplified form of &#x8604;
U+8573.0	fUTF8	蕳
U+8573.0	fCNS11643	3-663D
U+8573.0	fGB	E-3B40
U+8573.0	fR/S	 140.12
U+8573.0	fTotalStrokes	 16; 18
U+8573.0	fCangjie	TANA
U+8573.0	fFourCorner	4422
U+8573.0	fHakka	 gian1
U+8573.0	fCantonese	gaan1
U+8573.0	fMandarin	jian1
U+8573.0	fEnglish	agueweed
U+8574.0	fUTF8	蕴
U+8574.0	fGB	0-544C
U+8574.0	fR/S	 140.12; 140.14
U+8574.0	fTotalStrokes	 16; 18
U+8574.0	fCangjie	TVMT
U+8574.0	fCantonese	wan3; wan5 (also); wan2 (also); wan1
U+8574.0	fMandarin	yun4
U+8574.0	fEnglish	simplified form of &#x85F4;
U+8575.0	fUTF8	蕵
U+8575.0	fCNS11643	2-5979
U+8575.0	fBig5	EB4E
U+8575.0	fGB	5-697D
U+8575.0	fR/S	 140.12
U+8575.0	fTotalStrokes	 16; 18
U+8575.0	fCangjie	TNIV
U+8575.0	fFourCorner	44232
U+8575.0	fCantonese	syun1
U+8575.0	fMandarin	sun1
U+8576.0	fUTF8	蕶
U+8576.0	fCNS11643	2-5F21
U+8576.0	fBig5	EE53
U+8576.0	fGB	E-3B41
U+8576.0	fR/S	 140.13
U+8576.0	fTotalStrokes	 17; 19
U+8576.0	fCangjie	TMBI
U+8576.0	fFourCorner	44307
U+8576.0	fCantonese	ling4
U+8576.0	fMandarin	ling2; ling3
U+8577.0	fUTF8	蕷
U+8577.0	fCNS11643	2-5E6C
U+8577.0	fBig5	EE40
U+8577.0	fGB	1-5D77
U+8577.0	fR/S	 140.13
U+8577.0	fTotalStrokes	 17; 19
U+8577.0	fCangjie	TNNC
U+8577.0	fFourCorner	44286
U+8577.0	fHakka	 ji3; ji5; ri5; ri3; i5
U+8577.0	fCantonese	jyu6
U+8577.0	fMandarin	yu4
U+8577.0	fEnglish	used in combination for yam
U+8578.0	fUTF8	蕸
U+8578.0	fCNS11643	2-5E71
U+8578.0	fBig5	EE45
U+8578.0	fGB	3-6B21
U+8578.0	fR/S	 140.13
U+8578.0	fTotalStrokes	 17; 19
U+8578.0	fCangjie	TYRE
U+8578.0	fFourCorner	44304
U+8578.0	fHakka	  ha2
U+8578.0	fCantonese	haa4
U+8578.0	fMandarin	xia2; jia1
U+8578.0	fEnglish	water lilly's leaf
U+8579.0	fUTF8	蕹
U+8579.0	fCNS11643	2-5E7E
U+8579.0	fBig5	EE52
U+8579.0	fGB	0-5E33
U+8579.0	fR/S	 140.13
U+8579.0	fTotalStrokes	 17; 19
U+8579.0	fCangjie	TYVG
U+8579.0	fFourCorner	44214
U+8579.0	fHakka	 wung5; jung1; rung1
U+8579.0	fCantonese	ung3; ngung3 (also); jung1
U+8579.0	fMandarin	weng4; yong1
U+8579.0	fEnglish	Ipomoea aquatica, used as a vegetable
U+857A.0	fUTF8	蕺
U+857A.0	fCNS11643	2-5E70
U+857A.0	fBig5	EE44
U+857A.0	fGB	0-5E2A
U+857A.0	fR/S	 140.13
U+857A.0	fTotalStrokes	 17; 19
U+857A.0	fCangjie	TRJI
U+857A.0	fFourCorner	44153
U+857A.0	fHakka	 sip8
U+857A.0	fCantonese	cap1; zap1
U+857A.0	fMandarin	ji2; qie4
U+857A.0	fEnglish	Houttuynia cordata, a smelly but edible vegetable with light yellow flowers in summer
U+857B.0	fUTF8	蕻
U+857B.0	fCNS11643	2-5E68
U+857B.0	fBig5	EDFB
U+857B.0	fGB	0-5E2E
U+857B.0	fR/S	 140.13
U+857B.0	fTotalStrokes	 17; 19
U+857B.0	fCangjie	TSIC
U+857B.0	fFourCorner	44781
U+857B.0	fHakka	 giung3; fung5; fung6
U+857B.0	fCantonese	hung4; hung6 (also)
U+857B.0	fMandarin	hong4; hong2
U+857B.0	fEnglish	flourishing-as a plant, budding; sprouting
U+857C.0	fUTF8	蕼
U+857C.0	fCNS11643	2-5E6D
U+857C.0	fBig5	EE41
U+857C.0	fGB	5-6A2F
U+857C.0	fR/S	 140.13
U+857C.0	fTotalStrokes	 17; 19
U+857C.0	fCangjie	TSIQ
U+857C.0	fFourCorner	44757
U+857C.0	fHakka	 si5
U+857C.0	fCantonese	sei3
U+857C.0	fMandarin	si4
U+857D.0	fUTF8	蕽
U+857D.0	fCNS11643	4-583E
U+857D.0	fGB	E-3B42
U+857D.0	fR/S	  140.13
U+857D.0	fTotalStrokes	 17; 16; 19
U+857D.0	fCangjie	TTWV
U+857D.0	fFourCorner	44230
U+857D.0	fCantonese	nung4; lung4 (also)
U+857D.0	fMandarin	nong2
U+857E.0	fUTF8	蕾
U+857E.0	fCNS11643	1-752C
U+857E.0	fBig5	C1A2
U+857E.0	fGB	0-4059
U+857E.0	fR/S	 140.13
U+857E.0	fTotalStrokes	 17; 19
U+857E.0	fCangjie	TMBW
U+857E.0	fFourCorner	44603
U+857E.0	fHakka	  lui3; lui2
U+857E.0	fCantonese	leoi5; leoi4
U+857E.0	fMandarin	lei3
U+857E.0	fEnglish	 buds; unopened flower; flower bud
U+857F.0	fUTF8	蕿
U+857F.0	fCNS11643	3-562B
U+857F.0	fGB	E-3B43
U+857F.0	fR/S	  140.13
U+857F.0	fTotalStrokes	 17; 19
U+857F.0	fCangjie	TFBE
U+857F.0	fFourCorner	44840
U+857F.0	fCantonese	hyun1
U+857F.0	fMandarin	xuan1
U+8580.0	fUTF8	薀
U+8580.0	fCNS11643	2-5E61
U+8580.0	fBig5	EDF4
U+8580.0	fGB	E-3B44
U+8580.0	fR/S	 140.13
U+8580.0	fTotalStrokes	 17; 19
U+8580.0	fCangjie	TEWT
U+8580.0	fFourCorner	44117
U+8580.0	fHakka	 wun3; vun3
U+8580.0	fCantonese	wan1; wan3; wan2
U+8580.0	fMandarin	yun4; wen1; yun3
U+8580.0	fEnglish	the hippuris or mare's tail plant
U+8581.0	fUTF8	薁
U+8581.0	fCNS11643	2-5E79
U+8581.0	fBig5	EE4D
U+8581.0	fGB	3-6A75
U+8581.0	fR/S	 140.13
U+8581.0	fTotalStrokes	 17; 19
U+8581.0	fCangjie	THBK
U+8581.0	fFourCorner	44430
U+8581.0	fHakka	 juk7; ruk7
U+8581.0	fCantonese	juk1
U+8581.0	fMandarin	yu4; ao4
U+8581.0	fEnglish	vine
U+8582.0	fUTF8	薂
U+8582.0	fCNS11643	2-5E7B
U+8582.0	fBig5	EE4F
U+8582.0	fGB	3-6B30
U+8582.0	fR/S	 140.13
U+8582.0	fTotalStrokes	 17; 19
U+8582.0	fCangjie	THSK
U+8582.0	fFourCorner	44248
U+8582.0	fCantonese	hat6
U+8582.0	fMandarin	xi2
U+8583.0	fUTF8	薃
U+8583.0	fCNS11643	2-5E60
U+8583.0	fBig5	EDF3
U+8583.0	fGB	3-6B36
U+8583.0	fR/S	 140.13
U+8583.0	fTotalStrokes	 17; 19
U+8583.0	fCangjie	TEYB
U+8583.0	fFourCorner	44127
U+8583.0	fCantonese	hou6
U+8583.0	fMandarin	hao4; hao3
U+8584.0	fUTF8	薄
U+8584.0	fCNS11643	1-752B
U+8584.0	fBig5	C1A1
U+8584.0	fGB	0-3121
U+8584.0	fR/S	 140.13
U+8584.0	fTotalStrokes	 17; 19
U+8584.0	fCangjie	TEII
U+8584.0	fFourCorner	44146
U+8584.0	fHakka	  pok8; bu1
U+8584.0	fCantonese	bok6; bok2 (also)
U+8584.0	fMandarin	bao2; bu4; bo2
U+8584.0	fEnglish	[1] thin; slight; light [2] despise; slight; disdain [3] barren; not fertile [4] cover; hide; conceal; shut [5] screen [6] path of grass [7] ah; so; now (initial particle) [8] close in; press near [9] frivolous [10] a Chinese family name
U+8584.1	fUTF8	薄
U+8584.1	fCNS11643	1-752B
U+8584.1	fBig5	C1A1
U+8584.1	fGB	0-3121
U+8584.1	fR/S	 140.13
U+8584.1	fTotalStrokes	 17; 19
U+8584.1	fCangjie	TEII
U+8584.1	fFourCorner	44146
U+8584.1	fEnglish	peppermint
U+8584.2	fUTF8	薄
U+8584.2	fCNS11643	1-752B
U+8584.2	fBig5	C1A1
U+8584.2	fGB	0-3121
U+8584.2	fR/S	 140.13
U+8584.2	fTotalStrokes	 17; 19
U+8584.2	fCangjie	TEII
U+8584.2	fFourCorner	44146
U+8584.2	fMandarin	bo4
U+8585.0	fUTF8	薅
U+8585.0	fCNS11643	2-5E7D
U+8585.0	fBig5	EE51
U+8585.0	fGB	0-5E36
U+8585.0	fR/S	 140.13
U+8585.0	fTotalStrokes	 17; 19
U+8585.0	fCangjie	TVMI
U+8585.0	fFourCorner	44443
U+8585.0	fHakka	  gau1; hau1
U+8585.0	fCantonese	hou1
U+8585.0	fMandarin	hao1
U+8585.0	fEnglish	 weed (rice fields, etc.); eradicate; root out; pick off; clear away
U+8586.0	fUTF8	薆
U+8586.0	fCNS11643	2-5E75
U+8586.0	fBig5	EE49
U+8586.0	fGB	3-6A3F
U+8586.0	fR/S	 140.13
U+8586.0	fTotalStrokes	 17; 19
U+8586.0	fCangjie	TBBE
U+8586.0	fFourCorner	44247
U+8586.0	fHakka	  oi5
U+8586.0	fCantonese	oi3; ngoi3 (also)
U+8586.0	fMandarin	ai4; yi4
U+8586.0	fEnglish	[1] cover; hide; conceal; be under cover; hidden by thick growth [2] luxuriant growth of vegetation
U+8587.0	fUTF8	薇
U+8587.0	fCNS11643	1-7532
U+8587.0	fBig5	C1A8
U+8587.0	fGB	0-5E31
U+8587.0	fR/S	 140.13
U+8587.0	fTotalStrokes	 17; 19
U+8587.0	fCangjie	THOK
U+8587.0	fFourCorner	44248
U+8587.0	fHakka	  mi2
U+8587.0	fCantonese	mei4; mei2 (also)
U+8587.0	fMandarin	wei2; wei1; mei2
U+8587.0	fEnglish	[1] Osmunda regalis, a species of fern; thorn ferns [2] used in combination for a variety of plants
U+8588.0	fUTF8	薈
U+8588.0	fCNS11643	2-5E7C
U+8588.0	fBig5	EE50
U+8588.0	fGB	1-5C76
U+8588.0	fR/S	 140.13
U+8588.0	fTotalStrokes	 17; 19
U+8588.0	fCangjie	TOMA
U+8588.0	fFourCorner	44606
U+8588.0	fHakka	 woi5; we5; ve5; woi3; fui5
U+8588.0	fCantonese	wai3; kui2; wui6 (also); kwui2 (also); wui3
U+8588.0	fMandarin	hui4; wei4; yi4
U+8588.0	fEnglish	[1] luxuriant growth of vegetation; flourishing; abundant [2] cover; conceal
U+8589.0	fUTF8	薉
U+8589.0	fCNS11643	2-5E6E
U+8589.0	fBig5	EE42
U+8589.0	fGB	5-6777
U+8589.0	fR/S	 140.13
U+8589.0	fTotalStrokes	 17; 19
U+8589.0	fCangjie	TYMH
U+8589.0	fFourCorner	44253
U+8589.0	fHakka	 woi5; we5; ve5
U+8589.0	fCantonese	wai3
U+8589.0	fMandarin	wei4; hui4
U+8589.0	fEnglish	 dirty; unclean immoral; obscene
U+858A.0	fUTF8	薊
U+858A.0	fCNS11643	1-7534
U+858A.0	fBig5	C1AA
U+858A.0	fGB	1-3C3B
U+858A.0	fR/S	 140.13
U+858A.0	fTotalStrokes	 17; 19
U+858A.0	fCangjie	TNFN
U+858A.0	fFourCorner	44320
U+858A.0	fHakka	  gi5; kie5; ge5
U+858A.0	fCantonese	gai3
U+858A.0	fMandarin	ji4; jie1; jie4
U+858A.0	fEnglish	[1] Circium, a family of thorny plants [2] a Chinese family name
U+858B.0	fUTF8	薋
U+858B.0	fCNS11643	2-5E66
U+858B.0	fBig5	EDF9
U+858B.0	fGB	3-6A44
U+858B.0	fR/S	 140.13
U+858B.0	fTotalStrokes	 17; 19
U+858B.0	fCangjie	TIOC
U+858B.0	fFourCorner	44806
U+858B.0	fCantonese	ci4
U+858B.0	fMandarin	ci1; ci2; zi1
U+858C.0	fUTF8	薌
U+858C.0	fCNS11643	2-597D
U+858C.0	fBig5	EB52
U+858C.0	fGB	1-5C3C
U+858C.0	fR/S	 140.13
U+858C.0	fTotalStrokes	 17; 19
U+858C.0	fCangjie	TVHL
U+858C.0	fFourCorner	44227
U+858C.0	fHakka	 hiong1
U+858C.0	fCantonese	hoeng1; hoeng2
U+858C.0	fMandarin	xiang1; xiang3
U+858C.0	fEnglish	[1] the smell of rice grains; fragrant smell of grain [2] aromatic; spicy; pleasant smell [3] incense used for fumigation
U+858D.0	fUTF8	薍
U+858D.0	fCNS11643	2-5E76
U+858D.0	fBig5	EE4A
U+858D.0	fGB	3-6B31
U+858D.0	fR/S	 140.13
U+858D.0	fTotalStrokes	 17; 19
U+858D.0	fCangjie	TBBU
U+858D.0	fFourCorner	44212
U+858D.0	fCantonese	waan6; lyun6
U+858D.0	fMandarin	luan4; wan4
U+858E.0	fUTF8	薎
U+858E.0	fCNS11643	2-5E73
U+858E.0	fBig5	EE47
U+858E.0	fGB	5-6A35
U+858E.0	fR/S	 140.13
U+858E.0	fTotalStrokes	 17; 19
U+858E.0	fCangjie	TWLI
U+858E.0	fFourCorner	44253
U+858E.0	fCantonese	mit6
U+858E.0	fMandarin	mie4
U+858F.0	fUTF8	薏
U+858F.0	fCNS11643	2-5E62
U+858F.0	fBig5	EDF5
U+858F.0	fGB	0-5E32
U+858F.0	fR/S	 140.13
U+858F.0	fTotalStrokes	 17; 19
U+858F.0	fCangjie	TYTP
U+858F.0	fFourCorner	44336
U+858F.0	fHakka	 ji5; ri5; i5
U+858F.0	fCantonese	ji3
U+858F.0	fMandarin	yi4
U+858F.0	fEnglish	[1] the heart of a lotus seed [2] the seeds of Job's tears; pearl barley, used for cooking congee with rice or in herbal medicine
U+8590.0	fUTF8	薐
U+8590.0	fCNS11643	2-5F23
U+8590.0	fBig5	EE55
U+8590.0	fGB	3-6B2E
U+8590.0	fR/S	 140.13
U+8590.0	fTotalStrokes	 17; 19
U+8590.0	fCangjie	THDE
U+8590.0	fFourCorner	44947
U+8590.0	fHakka	 lin2
U+8590.0	fCantonese	ling4
U+8590.0	fMandarin	leng2
U+8590.0	fEnglish	old name of spinach
U+8591.0	fUTF8	薑
U+8591.0	fCNS11643	1-752E
U+8591.0	fBig5	C1A4
U+8591.0	fGB	1-7844
U+8591.0	fR/S	 140.13
U+8591.0	fTotalStrokes	 17; 19
U+8591.0	fCangjie	TMWM
U+8591.0	fFourCorner	44106
U+8591.0	fHakka	  giong1
U+8591.0	fCantonese	goeng1
U+8591.0	fMandarin	jiang1
U+8591.0	fEnglish	ginger
U+8592.0	fUTF8	薒
U+8592.0	fCNS11643	3-5628
U+8592.0	fGB	E-3B45
U+8592.0	fR/S	  140.13
U+8592.0	fTotalStrokes	 17; 19
U+8592.0	fCangjie	TYED
U+8592.0	fFourCorner	44900
U+8592.0	fCantonese	caan3
U+8592.0	fMandarin	can4
U+8593.0	fUTF8	薓
U+8593.0	fCNS11643	3-5629
U+8593.0	fGB	5-6A3F
U+8593.0	fR/S	 140.13
U+8593.0	fTotalStrokes	 17; 19
U+8593.0	fCangjie	TESE
U+8593.0	fFourCorner	44140
U+8593.0	fCantonese	sam1
U+8593.0	fMandarin	shen1; jing1
U+8593.0	fEnglish	ginseng
U+8594.0	fUTF8	薔
U+8594.0	fCNS11643	1-752F
U+8594.0	fBig5	C1A5
U+8594.0	fGB	1-473E
U+8594.0	fR/S	 140.13
U+8594.0	fTotalStrokes	 17; 19
U+8594.0	fCangjie	TGOW
U+8594.0	fFourCorner	44601
U+8594.0	fHakka	 siong2;  set7;  ciong2; sep7
U+8594.0	fCantonese	coeng4; sik1
U+8594.0	fMandarin	qiang2; se4
U+8594.0	fEnglish	the roses
U+8595.0	fUTF8	薕
U+8595.0	fCNS11643	2-5E64
U+8595.0	fBig5	EDF7
U+8595.0	fGB	3-6B35
U+8595.0	fR/S	 140.13
U+8595.0	fTotalStrokes	 17; 19
U+8595.0	fCangjie	TITC
U+8595.0	fFourCorner	44237
U+8595.0	fCantonese	lim4
U+8595.0	fMandarin	lian2
U+8596.0	fUTF8	薖
U+8596.0	fCNS11643	2-5E74
U+8596.0	fBig5	EE48
U+8596.0	fGB	3-6868
U+8596.0	fR/S	 140.13
U+8596.0	fTotalStrokes	 17; 19
U+8596.0	fCangjie	TYBB
U+8596.0	fFourCorner	44302
U+8596.0	fHakka	  ko1
U+8596.0	fCantonese	gwo1; go1 (also); fo1
U+8596.0	fMandarin	ke1
U+8596.0	fEnglish	empty; hungry-looking
U+8597.0	fUTF8	薗
U+8597.0	fCNS11643	3-562D
U+8597.0	fGB	E-3B46
U+8597.0	fR/S	  140.13
U+8597.0	fTotalStrokes	 17; 19
U+8597.0	fCangjie	TWGV
U+8597.0	fFourCorner	44600
U+8597.0	fCantonese	jyun4
U+8597.0	fMandarin	yuan2
U+8597.0	fEnglish	 garden park; orchard
U+8598.0	fUTF8	薘
U+8598.0	fCNS11643	2-5F22
U+8598.0	fBig5	EE54
U+8598.0	fGB	3-6864
U+8598.0	fR/S	 140.13
U+8598.0	fTotalStrokes	 17; 19
U+8598.0	fCangjie	TYGQ
U+8598.0	fFourCorner	44304
U+8598.0	fHakka	 tat8
U+8598.0	fCantonese	daat6
U+8598.0	fMandarin	da2
U+8598.0	fEnglish	plantago major
U+8599.0	fUTF8	薙
U+8599.0	fCNS11643	2-5E77
U+8599.0	fBig5	EE4B
U+8599.0	fGB	E-3B47
U+8599.0	fR/S	 140.13
U+8599.0	fTotalStrokes	 17; 19
U+8599.0	fCangjie	TOKG
U+8599.0	fFourCorner	44414
U+8599.0	fHakka	 ti5; cih1; chi1; tai5; chii3; {chi1}; chi3
U+8599.0	fCantonese	tai3
U+8599.0	fMandarin	ti4; si4; zhi4
U+8599.0	fEnglish	[1] weed [2] cut (hair); shave (hair)
U+859A.0	fUTF8	薚
U+859A.0	fCNS11643	2-5E6A
U+859A.0	fBig5	EDFD
U+859A.0	fGB	E-3B48
U+859A.0	fR/S	 140.13
U+859A.0	fTotalStrokes	 17; 19
U+859A.0	fCangjie	TMNH
U+859A.0	fFourCorner	44227
U+859A.0	fCantonese	tong1
U+859A.0	fMandarin	tang2; chang2
U+859B.0	fUTF8	薛
U+859B.0	fCNS11643	1-7531
U+859B.0	fBig5	C1A7
U+859B.0	fGB	0-5126
U+859B.0	fR/S	 140.13
U+859B.0	fTotalStrokes	 17; 19
U+859B.0	fCangjie	THRJ
U+859B.0	fFourCorner	44741
U+859B.0	fHakka	  siet7; set7; siat7
U+859B.0	fCantonese	sit3
U+859B.0	fMandarin	xue1; xie1
U+859B.0	fEnglish	[1] a kind of marsh grass [2] the name of a feudal state in today's Shantung province [3] a Chinese family name
U+859C.0	fUTF8	薜
U+859C.0	fCNS11643	1-752D
U+859C.0	fBig5	C1A3
U+859C.0	fGB	0-5E35
U+859C.0	fR/S	 140.13
U+859C.0	fTotalStrokes	 17; 19
U+859C.0	fCangjie	TSRJ
U+859C.0	fFourCorner	44641
U+859C.0	fCantonese	bei6
U+859C.0	fMandarin	bo4; bai4; pi4
U+859C.0	fEnglish	Chinese angelica; Angelica radix
U+859C.1	fUTF8	薜
U+859C.1	fCNS11643	1-752D
U+859C.1	fBig5	C1A3
U+859C.1	fGB	0-5E35
U+859C.1	fR/S	 140.13
U+859C.1	fTotalStrokes	 17; 19
U+859C.1	fCangjie	TSRJ
U+859C.1	fFourCorner	44641
U+859C.1	fHakka	 pit7
U+859C.1	fCantonese	bai6; baak3
U+859C.1	fMandarin	bi4; bao2; bo2
U+859C.1	fEnglish	Ficus pumila, a climbing fig
U+859D.0	fUTF8	薝
U+859D.0	fCNS11643	2-5E78
U+859D.0	fBig5	EE4C
U+859D.0	fGB	3-6B32
U+859D.0	fR/S	 140.13
U+859D.0	fTotalStrokes	 17; 19
U+859D.0	fCangjie	TNCR
U+859D.0	fFourCorner	44261
U+859D.0	fHakka	 zham3; zam3; zham1
U+859D.0	fCantonese	zim1
U+859D.0	fMandarin	zhan2; zhan1; dan3
U+859D.0	fEnglish	a fragrant white flower from India
U+859E.0	fUTF8	薞
U+859E.0	fCNS11643	2-5E6B
U+859E.0	fBig5	EDFE
U+859E.0	fGB	E-3B49
U+859E.0	fR/S	 140.13
U+859E.0	fTotalStrokes	 17; 19
U+859E.0	fCangjie	TMNV
U+859E.0	fFourCorner	44232
U+859E.0	fCantonese	syun1
U+859E.0	fMandarin	sun1
U+859F.0	fUTF8	薟
U+859F.0	fCNS11643	2-5F24
U+859F.0	fBig5	EE56
U+859F.0	fGB	1-5D32
U+859F.0	fR/S	 140.13
U+859F.0	fTotalStrokes	 17; 19
U+859F.0	fCangjie	TOMO
U+859F.0	fFourCorner	44886
U+859F.0	fHakka	  liam5; liam2; liam3
U+859F.0	fCantonese	cim1; lim5; lim6 (also); him1; lim4
U+859F.0	fMandarin	lian3; lian2; xian1; yan2; lian4; nan4; kan4
U+859F.0	fEnglish	vine
U+85A0.0	fUTF8	薠
U+85A0.0	fCNS11643	2-5E65
U+85A0.0	fBig5	EDF8
U+85A0.0	fGB	3-6A47
U+85A0.0	fR/S	 140.13
U+85A0.0	fTotalStrokes	 17; 19
U+85A0.0	fCangjie	TFMC
U+85A0.0	fFourCorner	44886
U+85A0.0	fHakka	 fan2
U+85A0.0	fCantonese	faan4
U+85A0.0	fMandarin	fan2
U+85A0.0	fEnglish	an aquatic grass or sedge
U+85A1.0	fUTF8	薡
U+85A1.0	fCNS11643	2-5E6F
U+85A1.0	fBig5	EE43
U+85A1.0	fGB	3-6A73
U+85A1.0	fR/S	 140.13
U+85A1.0	fTotalStrokes	 17; 19
U+85A1.0	fCangjie	TBUL
U+85A1.0	fFourCorner	44227
U+85A1.0	fHakka	 din3
U+85A1.0	fCantonese	ding2
U+85A1.0	fMandarin	ding3
U+85A1.0	fEnglish	a kind of rush woven into sandals
U+85A2.0	fUTF8	薢
U+85A2.0	fCNS11643	2-5E7A
U+85A2.0	fBig5	EE4E
U+85A2.0	fGB	3-6B33
U+85A2.0	fR/S	 140.13
U+85A2.0	fTotalStrokes	 17; 19
U+85A2.0	fCangjie	TNBQ
U+85A2.0	fFourCorner	44252
U+85A2.0	fHakka	 hai5;  hai3;  gai5
U+85A2.0	fCantonese	gaai1 (also); haai6; gaai3 (also); haai5 (also); gaai2 (also)
U+85A2.0	fMandarin	jie1; xie4; jie3
U+85A2.0	fEnglish	woody climbing plant
U+85A3.0	fUTF8	薣
U+85A3.0	fCNS11643	2-5E67
U+85A3.0	fBig5	EDFA
U+85A3.0	fGB	E-3B4A
U+85A3.0	fR/S	 140.13
U+85A3.0	fTotalStrokes	 17; 19
U+85A3.0	fCangjie	TGTE
U+85A3.0	fFourCorner	44147
U+85A3.0	fCantonese	gu2; gwu2 (also)
U+85A3.0	fMandarin	gu3
U+85A4.0	fUTF8	薤
U+85A4.0	fCNS11643	2-5E69
U+85A4.0	fBig5	EDFC
U+85A4.0	fGB	0-5E2F
U+85A4.0	fR/S	 140.13
U+85A4.0	fTotalStrokes	 17; 19
U+85A4.0	fCangjie	TMNM
U+85A4.0	fFourCorner	44211
U+85A4.0	fHakka	 hai5; k'ieu1; k'ieu5; hai6
U+85A4.0	fCantonese	haai6
U+85A4.0	fMandarin	xie4; xi4
U+85A4.0	fEnglish	Allium bakeri, a vegetable roughly resembling a leek with tiny purple flowers in summer whose scaly stalks are edible when tender
U+85A5.0	fUTF8	薥
U+85A5.0	fCNS11643	4-5840
U+85A5.0	fGB	5-6A34
U+85A5.0	fR/S	  140.13
U+85A5.0	fTotalStrokes	 17; 16; 19
U+85A5.0	fCangjie	TWLI
U+85A5.0	fFourCorner	44120
U+85A5.0	fCantonese	suk6
U+85A5.0	fMandarin	shu3
U+85A6.0	fUTF8	薦
U+85A6.0	fCNS11643	1-7535
U+85A6.0	fBig5	C2CB
U+85A6.0	fGB	1-3C76
U+85A6.0	fR/S	 140.13
U+85A6.0	fTotalStrokes	 17; 19
U+85A6.0	fCangjie	TIXF
U+85A6.0	fFourCorner	44227
U+85A6.0	fHakka	 zien5; tsian5; zen5
U+85A6.0	fCantonese	zin3
U+85A6.0	fMandarin	jian4; jin4
U+85A6.0	fEnglish	[1] offer; present; recommend [2] fodder for animals; grass [3] straw mat; mat, especially for sleeping [4] food and dishes [5] repeat; repeatedly
U+85A7.0	fUTF8	薧
U+85A7.0	fCNS11643	2-5E63
U+85A7.0	fBig5	EDF6
U+85A7.0	fGB	5-6A3C
U+85A7.0	fR/S	 140.13
U+85A7.0	fTotalStrokes	 17; 19
U+85A7.0	fCangjie	TYRP
U+85A7.0	fFourCorner	44211
U+85A7.0	fCantonese	haau2; hou1
U+85A7.0	fMandarin	kao3; hao1; kao4
U+85A7.0	fEnglish	dried food
U+85A8.0	fUTF8	薨
U+85A8.0	fCNS11643	1-7533
U+85A8.0	fBig5	C1A9
U+85A8.0	fGB	0-5E30
U+85A8.0	fR/S	 140.13
U+85A8.0	fTotalStrokes	 17; 19
U+85A8.0	fCangjie	TWLP
U+85A8.0	fFourCorner	44211
U+85A8.0	fHakka	 kwen1; fun2; gang1; fen2; wen2; kwen2; fun1
U+85A8.0	fCantonese	gwang1
U+85A8.0	fMandarin	hong1
U+85A8.0	fEnglish	[1] the death of a feudal lord [2] the loud buzzing of insects in flight
U+85A9.0	fUTF8	薩
U+85A9.0	fCNS11643	1-7730
U+85A9.0	fBig5	C2C4
U+85A9.0	fGB	1-4878
U+85A9.0	fR/S	 140.14
U+85A9.0	fTotalStrokes	 18; 20
U+85A9.0	fCangjie	TNLM
U+85A9.0	fFourCorner	44214
U+85A9.0	fHakka	  sat7
U+85A9.0	fCantonese	saat3; saat2 (also)
U+85A9.0	fMandarin	sa4
U+85A9.0	fEnglish	[1] a general name of Buddhist gods or immortals, Buddha [2] transliteration of "sat" of boddhisattva etc. [3] a Chinese family name
U+85AA.0	fUTF8	薪
U+85AA.0	fCNS11643	1-752A
U+85AA.0	fBig5	C17E
U+85AA.0	fGB	0-503D
U+85AA.0	fR/S	 140.13
U+85AA.0	fTotalStrokes	 17; 19
U+85AA.0	fCangjie	TYDL
U+85AA.0	fFourCorner	44921
U+85AA.0	fHakka	  sin1; sim1
U+85AA.0	fCantonese	san1
U+85AA.0	fMandarin	xin1
U+85AA.0	fEnglish	[1] fuel; firewood; faggots [2] salary; pay
U+85AB.0	fUTF8	薫
U+85AB.0	fCNS11643	3-562C
U+85AB.0	fGB	E-3B4B
U+85AB.0	fR/S	 140.13
U+85AB.0	fTotalStrokes	 17; 19
U+85AB.0	fCangjie	THGF
U+85AB.0	fFourCorner	44330
U+85AB.0	fCantonese	fan1
U+85AB.0	fMandarin	xun1
U+85AB.0	fEnglish	 a medicinal herb cauterize
U+85AC.0	fUTF8	薬
U+85AC.0	fCNS11643	3-5630
U+85AC.0	fGB	E-3B4C
U+85AC.0	fR/S	 140.13
U+85AC.0	fTotalStrokes	 17; 19
U+85AC.0	fCangjie	TIOD
U+85AC.0	fFourCorner	44900
U+85AC.0	fCantonese	joek6; joek3
U+85AC.0	fMandarin	yao4
U+85AC.0	fEnglish	drugs; pharmaceuticals; medicine
U+85AD.0	fUTF8	薭
U+85AD.0	fGB	E-3B4D
U+85AD.0	fR/S	 140.14; 140.13
U+85AD.0	fTotalStrokes	 18; 20
U+85AD.0	fCangjie	XTHDJ
U+85AD.0	fCantonese	baai6
U+85AD.0	fMandarin	bai4
U+85AE.0	fUTF8	薮
U+85AE.0	fGB	0-5E34
U+85AE.0	fR/S	 140.13
U+85AE.0	fTotalStrokes	 17; 19
U+85AE.0	fCangjie	TFVK
U+85AE.0	fHakka	 seu5; deu5
U+85AE.0	fCantonese	sau2
U+85AE.0	fMandarin	sou3; cou4; shu3
U+85AE.0	fEnglish	simplified form of &#x85EA;
U+85AF.0	fUTF8	薯
U+85AF.0	fCNS11643	1-7530
U+85AF.0	fBig5	C1A6
U+85AF.0	fGB	0-4A6D
U+85AF.0	fR/S	 140.14
U+85AF.0	fTotalStrokes	 18; 20
U+85AF.0	fCangjie	TWLA
U+85AF.0	fFourCorner	44604
U+85AF.0	fHakka	 su2;  shu2
U+85AF.0	fCantonese	syu4; syu2 (also); syu5; syu6
U+85AF.0	fMandarin	shu3; shu4
U+85AF.0	fEnglish	yam; tuber; potato
U+85B0.0	fUTF8	薰
U+85B0.0	fCNS11643	1-7734
U+85B0.0	fBig5	C2C8
U+85B0.0	fGB	0-5E39
U+85B0.0	fR/S	 140.14
U+85B0.0	fTotalStrokes	 18; 20
U+85B0.0	fCangjie	THGF
U+85B0.0	fFourCorner	44331
U+85B0.0	fHakka	 hiun1; hun1
U+85B0.0	fCantonese	fan1
U+85B0.0	fMandarin	xun1; xun4
U+85B0.0	fEnglish	[1] Coumarouna odorata, a medicinal herb [2] cauterize [3] perfume; embalm [4] smoke; fumigate [5] warm
U+85B1.0	fUTF8	薱
U+85B1.0	fCNS11643	2-627D
U+85B1.0	fBig5	F0B3
U+85B1.0	fGB	3-685E
U+85B1.0	fR/S	 140.14
U+85B1.0	fTotalStrokes	 18; 20
U+85B1.0	fCangjie	TTGI
U+85B1.0	fFourCorner	44140
U+85B1.0	fHakka	  dui5
U+85B1.0	fCantonese	deoi6; deoi3
U+85B1.0	fMandarin	dui4
U+85B1.0	fEnglish	anundant vegetation
U+85B2.0	fUTF8	薲
U+85B2.0	fCNS11643	3-586C
U+85B2.0	fGB	3-6A49
U+85B2.0	fR/S	  140.14
U+85B2.0	fTotalStrokes	 18; 20
U+85B2.0	fCangjie	TJMC
U+85B2.0	fFourCorner	44800
U+85B2.0	fCantonese	pan4
U+85B2.0	fMandarin	pin2
U+85B3.0	fUTF8	薳
U+85B3.0	fCNS11643	2-6273
U+85B3.0	fBig5	F0A9
U+85B3.0	fGB	3-6B27
U+85B3.0	fR/S	 140.14
U+85B3.0	fTotalStrokes	 18; 20
U+85B3.0	fCangjie	TYGV
U+85B3.0	fFourCorner	44303
U+85B3.0	fHakka	 wui3; jen3; ren3; wi1; vui3; wui1
U+85B3.0	fCantonese	jyun5; wai2; wai5
U+85B3.0	fMandarin	wei3; yuan3
U+85B3.0	fEnglish	[1] polygata, name of a herb [2] a Chinese family name
U+85B4.0	fUTF8	薴
U+85B4.0	fCNS11643	2-626E
U+85B4.0	fBig5	F0A4
U+85B4.0	fGB	3-6858
U+85B4.0	fR/S	 140.14
U+85B4.0	fTotalStrokes	 18; 20
U+85B4.0	fCangjie	TJPN
U+85B4.0	fFourCorner	44201
U+85B4.0	fHakka	 nen2
U+85B4.0	fCantonese	ning4; ling4 (also)
U+85B4.0	fMandarin	neng2; ning4
U+85B4.0	fEnglish	dense vegetation
U+85B5.0	fUTF8	薵
U+85B5.0	fCNS11643	2-6274
U+85B5.0	fBig5	F0AA
U+85B5.0	fGB	3-687A
U+85B5.0	fR/S	 140.14
U+85B5.0	fTotalStrokes	 18; 20
U+85B5.0	fCangjie	TGNI
U+85B5.0	fFourCorner	44641
U+85B5.0	fCantonese	cau4
U+85B5.0	fMandarin	chou2; zhou4
U+85B6.0	fUTF8	薶
U+85B6.0	fCNS11643	2-627E
U+85B6.0	fBig5	F0B4
U+85B6.0	fGB	5-6A4E
U+85B6.0	fR/S	 140.14
U+85B6.0	fTotalStrokes	 18; 20
U+85B6.0	fCangjie	TBHG
U+85B6.0	fFourCorner	44214
U+85B6.0	fHakka	  mai2;  li2; mai3
U+85B6.0	fCantonese	maai4; wo1
U+85B6.0	fMandarin	mai2; li2; wo1; wei1; mai4
U+85B6.0	fEnglish	 bury to stop up dirty; filthy
U+85B7.0	fUTF8	薷
U+85B7.0	fCNS11643	2-6324
U+85B7.0	fBig5	F0B8
U+85B7.0	fGB	0-5E38
U+85B7.0	fR/S	 140.14
U+85B7.0	fTotalStrokes	 18; 20
U+85B7.0	fCangjie	TMBB
U+85B7.0	fFourCorner	44227
U+85B7.0	fHakka	 ji2; ri2
U+85B7.0	fCantonese	jyu4
U+85B7.0	fMandarin	ru2; piao2; rou2
U+85B7.0	fEnglish	a plant, from which a cooling drink is made
U+85B8.0	fUTF8	薸
U+85B8.0	fCNS11643	2-6323
U+85B8.0	fBig5	F0B7
U+85B8.0	fGB	3-6B3F
U+85B8.0	fR/S	 140.14
U+85B8.0	fTotalStrokes	 18; 20
U+85B8.0	fCangjie	TEMF
U+85B8.0	fFourCorner	44191
U+85B8.0	fHakka	 piau1; piau3; piau2
U+85B8.0	fCantonese	piu1; piu4 (also); piu2 (also)
U+85B8.0	fMandarin	piao1; miao2
U+85B8.0	fEnglish	duckweed
U+85B9.0	fUTF8	薹
U+85B9.0	fCNS11643	1-7736
U+85B9.0	fBig5	C2CA
U+85B9.0	fGB	0-5E37
U+85B9.0	fR/S	 140.14
U+85B9.0	fTotalStrokes	 18; 20
U+85B9.0	fCangjie	TGRG
U+85B9.0	fFourCorner	44104
U+85B9.0	fHakka	  toi2
U+85B9.0	fCantonese	toi4
U+85B9.0	fMandarin	tai2
U+85B9.0	fEnglish	Cyperus rotundus; type of sedge
U+85BA.0	fUTF8	薺
U+85BA.0	fCNS11643	1-7735
U+85BA.0	fBig5	C2C9
U+85BA.0	fGB	1-5C79
U+85BA.0	fR/S	 140.14
U+85BA.0	fTotalStrokes	 18; 20
U+85BA.0	fCangjie	TYX
U+85BA.0	fFourCorner	44223
U+85BA.0	fHakka	  ci2;  zi5; zi3
U+85BA.0	fCantonese	cai5; cai4; ci4; zai3 (also)
U+85BA.0	fMandarin	ci2; ji4; shi4; qi4; qi5
U+85BA.0	fEnglish	Capsella bursa-pastoris, a kind of vegetable, sheperd's purse
U+85BA.1	fUTF8	薺
U+85BA.1	fCNS11643	1-7735
U+85BA.1	fBig5	C2C9
U+85BA.1	fGB	1-5C79
U+85BA.1	fR/S	 140.14
U+85BA.1	fTotalStrokes	 18; 20
U+85BA.1	fCangjie	TYX
U+85BA.1	fFourCorner	44223
U+85BA.1	fMandarin	qi2
U+85BA.1	fEnglish	[1] water chestnut [2] the caltrop
U+85BB.0	fUTF8	薻
U+85BB.0	fCNS11643	3-586E
U+85BB.0	fGB	E-3B4E
U+85BB.0	fR/S	  140.14
U+85BB.0	fTotalStrokes	 18; 20
U+85BB.0	fCangjie	TEVD
U+85BB.0	fFourCorner	44190
U+85BB.0	fCantonese	zou2
U+85BB.0	fMandarin	zao3
U+85BC.0	fUTF8	薼
U+85BC.0	fCNS11643	4-5D2C
U+85BC.0	fGB	E-3B4F
U+85BC.0	fR/S	  140.14
U+85BC.0	fTotalStrokes	 18; 20
U+85BC.0	fCangjie	TIPG
U+85BC.0	fFourCorner	44210
U+85BC.0	fCantonese	can4
U+85BC.0	fMandarin	chen2
U+85BD.0	fUTF8	薽
U+85BD.0	fCNS11643	2-6275
U+85BD.0	fBig5	F0AB
U+85BD.0	fGB	3-6B2B
U+85BD.0	fR/S	 140.14
U+85BD.0	fTotalStrokes	 18; 20
U+85BD.0	fCangjie	TMGN
U+85BD.0	fFourCorner	44117
U+85BD.0	fHakka	 zhin1; zihn1
U+85BD.0	fCantonese	zan1
U+85BD.0	fMandarin	zhen1; zhen2
U+85BD.0	fEnglish	name of a bright blue orchid
U+85BE.0	fUTF8	薾
U+85BE.0	fCNS11643	2-6325
U+85BE.0	fBig5	F0B9
U+85BE.0	fGB	3-6856
U+85BE.0	fR/S	 140.14
U+85BE.0	fTotalStrokes	 18; 20
U+85BE.0	fCangjie	TMFB
U+85BE.0	fFourCorner	44227
U+85BE.0	fCantonese	ji5
U+85BE.0	fMandarin	er3; ni3
U+85BE.0	fEnglish	luxuriant growth of flower
U+85BF.0	fUTF8	薿
U+85BF.0	fCNS11643	2-6278
U+85BF.0	fBig5	F0AE
U+85BF.0	fGB	3-6B3C
U+85BF.0	fR/S	 140.14
U+85BF.0	fTotalStrokes	 18; 20
U+85BF.0	fCangjie	TPKO
U+85BF.0	fFourCorner	44481
U+85BF.0	fHakka	  ngi3
U+85BF.0	fCantonese	ji5; jik6
U+85BF.0	fMandarin	ni3; jian1; ni4
U+85BF.0	fEnglish	flower
U+85C0.0	fUTF8	藀
U+85C0.0	fCNS11643	2-6270
U+85C0.0	fBig5	F0A6
U+85C0.0	fGB	E-3B50
U+85C0.0	fR/S	 140.14
U+85C0.0	fTotalStrokes	 18; 20
U+85C0.0	fCangjie	TFFF
U+85C0.0	fFourCorner	44809
U+85C0.0	fCantonese	jing4
U+85C0.0	fMandarin	ying2
U+85C1.0	fUTF8	藁
U+85C1.0	fCNS11643	3-586F
U+85C1.0	fGB	0-5E3B
U+85C1.0	fR/S	 140.14
U+85C1.0	fTotalStrokes	 18; 20
U+85C1.0	fCangjie	TYRD
U+85C1.0	fFourCorner	44900
U+85C1.0	fHakka	 gau3; go3
U+85C1.0	fCantonese	gou2
U+85C1.0	fMandarin	gao3
U+85C1.0	fEnglish	 straw; hay dry; withered
U+85C2.0	fUTF8	藂
U+85C2.0	fCNS11643	2-6272
U+85C2.0	fBig5	F0A8
U+85C2.0	fGB	5-6A41
U+85C2.0	fR/S	 140.14
U+85C2.0	fTotalStrokes	 18; 20
U+85C2.0	fCangjie	TSEO
U+85C2.0	fFourCorner	44334
U+85C2.0	fHakka	 cung2
U+85C2.0	fCantonese	cung4
U+85C2.0	fMandarin	cong4; cong2; cong1
U+85C3.0	fUTF8	藃
U+85C3.0	fCNS11643	2-6271
U+85C3.0	fBig5	F0A7
U+85C3.0	fGB	3-6B3E
U+85C3.0	fR/S	 140.14
U+85C3.0	fTotalStrokes	 18; 20
U+85C3.0	fCangjie	TYBO
U+85C3.0	fFourCorner	44282
U+85C3.0	fCantonese	hiu1; hou3
U+85C3.0	fMandarin	xiao1; hao4; he4
U+85C4.0	fUTF8	藄
U+85C4.0	fCNS11643	2-6277
U+85C4.0	fBig5	F0AD
U+85C4.0	fGB	E-3B51
U+85C4.0	fR/S	 140.14
U+85C4.0	fTotalStrokes	 18; 20
U+85C4.0	fCangjie	TTCF
U+85C4.0	fFourCorner	44900
U+85C4.0	fHakka	 ki2
U+85C4.0	fCantonese	kei4
U+85C4.0	fMandarin	qi2; ji1
U+85C4.0	fEnglish	edible fern
U+85C5.0	fUTF8	藅
U+85C5.0	fCNS11643	2-627C
U+85C5.0	fBig5	F0B2
U+85C5.0	fGB	5-6876
U+85C5.0	fR/S	 140.14
U+85C5.0	fTotalStrokes	 18; 20
U+85C5.0	fCangjie	TWLN
U+85C5.0	fFourCorner	44626
U+85C5.0	fCantonese	fat6
U+85C5.0	fMandarin	fa2
U+85C6.0	fUTF8	藆
U+85C6.0	fCNS11643	2-626F
U+85C6.0	fBig5	F0A5
U+85C6.0	fGB	5-6A52
U+85C6.0	fR/S	 140.14
U+85C6.0	fTotalStrokes	 18; 20
U+85C6.0	fCangjie	TJTQ
U+85C6.0	fFourCorner	44502
U+85C6.0	fCantonese	gin2
U+85C6.0	fMandarin	jian3; yao4
U+85C7.0	fUTF8	藇
U+85C7.0	fCNS11643	2-6276
U+85C7.0	fBig5	F0AC
U+85C7.0	fGB	3-6832
U+85C7.0	fR/S	 140.14
U+85C7.0	fTotalStrokes	 18; 20
U+85C7.0	fCangjie	THXC
U+85C7.0	fFourCorner	44801
U+85C7.0	fHakka	  si5; ji2; ri2
U+85C7.0	fCantonese	zeoi6; jyu6
U+85C7.0	fMandarin	xu4; yu4; xu1; yu2; yu3
U+85C7.0	fEnglish	fine
U+85C8.0	fUTF8	藈
U+85C8.0	fCNS11643	2-627B
U+85C8.0	fBig5	F0B1
U+85C8.0	fGB	E-3B52
U+85C8.0	fR/S	 140.14
U+85C8.0	fTotalStrokes	 18; 20
U+85C8.0	fCangjie	TBUK
U+85C8.0	fFourCorner	44634
U+85C8.0	fCantonese	kwai1; gwai1
U+85C8.0	fMandarin	kui1; gui1
U+85C9.0	fUTF8	藉
U+85C9.0	fCNS11643	1-7733
U+85C9.0	fBig5	C2C7
U+85C9.0	fGB	0-3D65
U+85C9.0	fR/S	 140.14
U+85C9.0	fTotalStrokes	 18; 20
U+85C9.0	fCangjie	TQDA
U+85C9.0	fFourCorner	44961
U+85C9.0	fHakka	 zia5; cit8; sit7
U+85C9.0	fCantonese	zik6; ze6; ze3
U+85C9.0	fMandarin	jie4; ji2
U+85C9.0	fEnglish	[1] mat; pad; cushion of grass or straw [2] rely on; lean on; on the strength of; avail oneself of; by means of [3] on the excuse of; on pretext of
U+85C9.1	fUTF8	藉
U+85C9.1	fCNS11643	1-7733
U+85C9.1	fBig5	C2C7
U+85C9.1	fGB	0-3D65
U+85C9.1	fR/S	 140.14
U+85C9.1	fTotalStrokes	 18; 20
U+85C9.1	fCangjie	TQDA
U+85C9.1	fFourCorner	44961
U+85C9.1	fMandarin	jie2
U+85C9.1	fEnglish	disorder; confusion
U+85CA.0	fUTF8	藊
U+85CA.0	fCNS11643	4-5D28
U+85CA.0	fGB	5-6A4C
U+85CA.0	fR/S	  140.14
U+85CA.0	fTotalStrokes	 18; 20
U+85CA.0	fCangjie	THDB
U+85CA.0	fFourCorner	44920
U+85CA.0	fHakka	  bien3
U+85CA.0	fCantonese	bin2
U+85CA.0	fMandarin	bian3
U+85CA.0	fEnglish	a kind of bean with flat pods
U+85CB.0	fUTF8	藋
U+85CB.0	fCNS11643	2-6279
U+85CB.0	fBig5	F0AF
U+85CB.0	fGB	3-6B41
U+85CB.0	fR/S	 140.14
U+85CB.0	fTotalStrokes	 18; 20
U+85CB.0	fCangjie	TSMG
U+85CB.0	fFourCorner	44214
U+85CB.0	fHakka	 tit8; cok8; tiau5
U+85CB.0	fCantonese	diu3 (also); diu6; dik6; tik1
U+85CB.0	fMandarin	diao4; di2; zhuo2
U+85CB.0	fEnglish	a weed like hellebore
U+85CC.0	fUTF8	藌
U+85CC.0	fCNS11643	3-586B
U+85CC.0	fGB	5-6A53
U+85CC.0	fR/S	  140.14
U+85CC.0	fTotalStrokes	 18; 20
U+85CC.0	fCangjie	TJPI
U+85CC.0	fFourCorner	44130
U+85CC.0	fCantonese	mat6
U+85CC.0	fMandarin	mi4
U+85CD.0	fUTF8	藍
U+85CD.0	fCNS11643	1-7731
U+85CD.0	fBig5	C2C5
U+85CD.0	fGB	1-4036
U+85CD.0	fR/S	 140.14
U+85CD.0	fTotalStrokes	 18; 20
U+85CD.0	fCangjie	TSIT
U+85CD.0	fFourCorner	44107
U+85CD.0	fHakka	  lam2; lam3
U+85CD.0	fCantonese	laam4; laam2 (also)
U+85CD.0	fMandarin	lan2; la5; lan4
U+85CD.0	fEnglish	[1] blue; indigo [2] an indigo plant [3] a Chinese family name
U+85CE.0	fUTF8	藎
U+85CE.0	fCNS11643	2-627A
U+85CE.0	fBig5	F0B0
U+85CE.0	fGB	1-5D23
U+85CE.0	fR/S	 140.14
U+85CE.0	fTotalStrokes	 18; 20
U+85CE.0	fCangjie	TLMT
U+85CE.0	fFourCorner	44107
U+85CE.0	fHakka	 zin3; zin5
U+85CE.0	fCantonese	zeon2; zeon6
U+85CE.0	fMandarin	jin4
U+85CE.0	fEnglish	[1] Arthraxon hiipidus, a kind of weed whose stalk is used as a yellow dye [2] faithfulness; great loyalty and love
U+85CF.0	fUTF8	藏
U+85CF.0	fCNS11643	1-772F
U+85CF.0	fBig5	C2C3
U+85CF.0	fGB	0-3258
U+85CF.0	fR/S	 140.14
U+85CF.0	fTotalStrokes	 18; 20
U+85CF.0	fCangjie	TIMS
U+85CF.0	fFourCorner	44253
U+85CF.0	fHakka	 cong5; cong3; cong6
U+85CF.0	fCantonese	cong4; zong6; zong1
U+85CF.0	fMandarin	cang2; zang1
U+85CF.0	fEnglish	[1] hide; conceal [2] hoard; store up; save [3] a Chinese family name
U+85CF.1	fUTF8	藏
U+85CF.1	fCNS11643	1-772F
U+85CF.1	fBig5	C2C3
U+85CF.1	fGB	0-3258
U+85CF.1	fR/S	 140.14
U+85CF.1	fTotalStrokes	 18; 20
U+85CF.1	fCangjie	TIMS
U+85CF.1	fFourCorner	44253
U+85CF.1	fHakka	 cong2
U+85CF.1	fMandarin	zang4
U+85CF.1	fEnglish	[1] Tibet; Tibetans [2] storage; warehouse; depository [3] a collective name for the Buddhist and Taoist part of the Chinese race
U+85D0.0	fUTF8	藐
U+85D0.0	fCNS11643	1-7732
U+85D0.0	fBig5	C2C6
U+85D0.0	fGB	0-436A
U+85D0.0	fR/S	 140.14
U+85D0.0	fTotalStrokes	 18; 20
U+85D0.0	fCangjie	TBHU
U+85D0.0	fFourCorner	44216
U+85D0.0	fHakka	 miau3; meu3; miau1; miau6; meu5; mau6; miau5; mau3; mau5
U+85D0.0	fCantonese	miu5; miu2 (also); mok6
U+85D0.0	fMandarin	miao3; mao4; mo4
U+85D0.0	fEnglish	[1] disregard; slight; disdain; slight; treat with disdain [2] small; petite; petty; insignificant
U+85D1.0	fUTF8	藑
U+85D1.0	fCNS11643	2-6660
U+85D1.0	fBig5	F2D5
U+85D1.0	fGB	3-6B3B
U+85D1.0	fR/S	 140.14
U+85D1.0	fTotalStrokes	 18; 20
U+85D1.0	fCangjie	TNBE
U+85D1.0	fFourCorner	44407
U+85D1.0	fCantonese	king4
U+85D1.0	fMandarin	qiong2
U+85D2.0	fUTF8	藒
U+85D2.0	fCNS11643	2-6321
U+85D2.0	fBig5	F0B5
U+85D2.0	fGB	5-6A4B
U+85D2.0	fR/S	 140.14
U+85D2.0	fTotalStrokes	 18; 20
U+85D2.0	fCangjie	THDV
U+85D2.0	fFourCorner	44927
U+85D2.0	fHakka	 giet7; get7
U+85D2.0	fCantonese	kit3
U+85D2.0	fMandarin	qie4; qi4
U+85D2.0	fEnglish	a fragrant plant found in north-western Kiangsu
U+85D3.0	fUTF8	藓
U+85D3.0	fGB	0-5E3A
U+85D3.0	fR/S	 140.14
U+85D3.0	fTotalStrokes	 18; 20
U+85D3.0	fCangjie	TNMQ
U+85D3.0	fHakka	 sien3
U+85D3.0	fCantonese	sin2
U+85D3.0	fMandarin	xian3; li4
U+85D3.0	fEnglish	simplified form of &#x861A;
U+85D4.0	fUTF8	藔
U+85D4.0	fCNS11643	3-663E
U+85D4.0	fGB	E-3B53
U+85D4.0	fR/S	 140.15
U+85D4.0	fTotalStrokes	 19; 21
U+85D4.0	fCangjie	TJKF
U+85D4.0	fFourCorner	4490
U+85D4.0	fCantonese	liu4
U+85D5.0	fUTF8	藕
U+85D5.0	fCNS11643	1-786D
U+85D5.0	fBig5	C3C2
U+85D5.0	fGB	0-453A
U+85D5.0	fR/S	 140.15
U+85D5.0	fTotalStrokes	 19; 21
U+85D5.0	fCangjie	TQDB
U+85D5.0	fFourCorner	44927
U+85D5.0	fHakka	 ngieu3; ngiau1; niau1; ngeu3; liu1; ngiu1
U+85D5.0	fCantonese	ngau5; au5 (also); ngau2 (also); au2 (also); ngaau5; aau5
U+85D5.0	fMandarin	ou3
U+85D5.0	fEnglish	rhizomes, or rootstocks of the lotus
U+85D6.0	fUTF8	藖
U+85D6.0	fCNS11643	3-5A7E
U+85D6.0	fGB	3-694D
U+85D6.0	fR/S	  140.15
U+85D6.0	fTotalStrokes	 19; 21
U+85D6.0	fCangjie	TSEC
U+85D6.0	fFourCorner	44800
U+85D6.0	fCantonese	haan4
U+85D6.0	fMandarin	xian2
U+85D7.0	fUTF8	藗
U+85D7.0	fCNS11643	2-6658
U+85D7.0	fBig5	F2CD
U+85D7.0	fGB	5-6A43
U+85D7.0	fR/S	 140.15
U+85D7.0	fTotalStrokes	 19; 21
U+85D7.0	fCangjie	TYDO
U+85D7.0	fFourCorner	44308
U+85D7.0	fCantonese	suk1; cuk1
U+85D7.0	fMandarin	su4
U+85D8.0	fUTF8	藘
U+85D8.0	fCNS11643	2-665C
U+85D8.0	fBig5	F2D1
U+85D8.0	fGB	3-6A35
U+85D8.0	fR/S	 140.15
U+85D8.0	fTotalStrokes	 19; 21
U+85D8.0	fCangjie	TYPP
U+85D8.0	fFourCorner	44236
U+85D8.0	fHakka	 li5;  li1
U+85D8.0	fCantonese	leoi4
U+85D8.0	fMandarin	l2; luu2; lu2
U+85D8.0	fEnglish	madder
U+85D9.0	fUTF8	藙
U+85D9.0	fCNS11643	2-6654
U+85D9.0	fBig5	F2C9
U+85D9.0	fGB	3-6B4B
U+85D9.0	fR/S	 140.15
U+85D9.0	fTotalStrokes	 19; 21
U+85D9.0	fCangjie	TYOE
U+85D9.0	fFourCorner	44247
U+85D9.0	fHakka	 ngi5; ngi6
U+85D9.0	fCantonese	ngai6; ai6 (also)
U+85D9.0	fMandarin	yi4
U+85D9.0	fEnglish	dogwood
U+85DA.0	fUTF8	藚
U+85DA.0	fCNS11643	2-6657
U+85DA.0	fBig5	F2CC
U+85DA.0	fGB	3-6943
U+85DA.0	fR/S	 140.15
U+85DA.0	fTotalStrokes	 19; 21
U+85DA.0	fCangjie	TGWC
U+85DA.0	fFourCorner	44806
U+85DA.0	fHakka	 siuk8
U+85DA.0	fCantonese	zuk6
U+85DA.0	fMandarin	xu4
U+85DA.0	fEnglish	water plant
U+85DB.0	fUTF8	藛
U+85DB.0	fCNS11643	3-5A7D
U+85DB.0	fGB	5-6765
U+85DB.0	fR/S	  140.15
U+85DB.0	fTotalStrokes	 19; 21
U+85DB.0	fCangjie	TJHF
U+85DB.0	fFourCorner	44320
U+85DB.0	fHakka	 sia5; sia3
U+85DB.0	fCantonese	se2
U+85DB.0	fMandarin	xie3
U+85DB.0	fEnglish	a medicinal plant, from which diuretic medicines are obtained
U+85DC.0	fUTF8	藜
U+85DC.0	fCNS11643	2-665F
U+85DC.0	fBig5	F2D4
U+85DC.0	fGB	0-5E3C
U+85DC.0	fR/S	 140.15
U+85DC.0	fTotalStrokes	 19; 21
U+85DC.0	fCangjie	THHE
U+85DC.0	fFourCorner	44132
U+85DC.0	fHakka	  lai2;  li2
U+85DC.0	fCantonese	lai4; lei4
U+85DC.0	fMandarin	li2
U+85DC.0	fEnglish	Chenopodium album; pigweeds; lamb's quarters
U+85DD.0	fUTF8	藝
U+85DD.0	fCNS11643	1-786B
U+85DD.0	fBig5	C3C0
U+85DD.0	fGB	1-5255
U+85DD.0	fR/S	 140.15
U+85DD.0	fTotalStrokes	 19; 21
U+85DD.0	fCangjie	TGII
U+85DD.0	fFourCorner	44731
U+85DD.0	fHakka	 ngi5; gni5; ngi6; gni6; ngi3
U+85DD.0	fCantonese	ngai6; ai6 (also)
U+85DD.0	fMandarin	yi4; nie4
U+85DD.0	fEnglish	art; talent; ability; craft; dexterity
U+85DE.0	fUTF8	藞
U+85DE.0	fCNS11643	2-6664
U+85DE.0	fBig5	F2D9
U+85DE.0	fGB	3-6B44
U+85DE.0	fR/S	 140.15
U+85DE.0	fTotalStrokes	 19; 21
U+85DE.0	fCangjie	TMRR
U+85DE.0	fFourCorner	44661
U+85DE.0	fHakka	 la5; la3
U+85DE.0	fCantonese	laa5; lo5; laa3
U+85DE.0	fMandarin	la3
U+85DE.0	fEnglish	[1] uneven [2] rocky
U+85DF.0	fUTF8	藟
U+85DF.0	fCNS11643	2-665D
U+85DF.0	fBig5	F2D2
U+85DF.0	fGB	3-6B45
U+85DF.0	fR/S	 140.15
U+85DF.0	fTotalStrokes	 19; 21
U+85DF.0	fCangjie	TWWW
U+85DF.0	fFourCorner	44666
U+85DF.0	fHakka	  lui3
U+85DF.0	fCantonese	leoi5
U+85DF.0	fMandarin	lei3; lei2
U+85DF.0	fEnglish	[1] a variety of climbing plant [2] wind; entwine [3] same as $
U+85E0.0	fUTF8	藠
U+85E0.0	fCNS11643	4-607A
U+85E0.0	fGB	3-6B47
U+85E0.0	fR/S	  140.15
U+85E0.0	fTotalStrokes	 19; 21
U+85E0.0	fCangjie	THAA
U+85E0.0	fFourCorner	44660
U+85E0.0	fHakka	 kiau1; kiau5; k'ieu5; k'ieu1
U+85E0.0	fCantonese	kiu2; kiu5 (also); jiu2
U+85E0.0	fMandarin	xiao4; jiao4
U+85E0.0	fEnglish	a vegetable like the onion; chives
U+85E1.0	fUTF8	藡
U+85E1.0	fCNS11643	2-6655
U+85E1.0	fBig5	F2CA
U+85E1.0	fGB	E-3B54
U+85E1.0	fR/S	 140.15
U+85E1.0	fTotalStrokes	 19; 21
U+85E1.0	fCangjie	TYYB
U+85E1.0	fFourCorner	44302
U+85E1.0	fCantonese	dik6
U+85E1.0	fMandarin	di2
U+85E2.0	fUTF8	藢
U+85E2.0	fCNS11643	2-6665
U+85E2.0	fBig5	F2DA
U+85E2.0	fGB	5-6A59
U+85E2.0	fR/S	 140.15
U+85E2.0	fTotalStrokes	 19; 21
U+85E2.0	fCangjie	THOK
U+85E2.0	fFourCorner	44248
U+85E2.0	fCantonese	zi2
U+85E2.0	fMandarin	zhi3
U+85E3.0	fUTF8	藣
U+85E3.0	fCNS11643	2-665E
U+85E3.0	fBig5	F2D3
U+85E3.0	fGB	3-6A36
U+85E3.0	fR/S	 140.15
U+85E3.0	fTotalStrokes	 19; 21
U+85E3.0	fCangjie	TWLP
U+85E3.0	fFourCorner	44211
U+85E3.0	fCantonese	bei1
U+85E3.0	fMandarin	bei1; pi2; bi4; dei1
U+85E4.0	fUTF8	藤
U+85E4.0	fCNS11643	1-786E
U+85E4.0	fBig5	C3C3
U+85E4.0	fGB	0-4C59
U+85E4.0	fR/S	 140.15
U+85E4.0	fTotalStrokes	 19; 21
U+85E4.0	fCangjie	TBFE
U+85E4.0	fFourCorner	44232
U+85E4.0	fHakka	  ten2; cit7
U+85E4.0	fCantonese	tang4
U+85E4.0	fMandarin	teng2
U+85E4.0	fEnglish	 rattan; vine
U+85E5.0	fUTF8	藥
U+85E5.0	fCNS11643	1-786F
U+85E5.0	fBig5	C3C4
U+85E5.0	fGB	1-5229
U+85E5.0	fR/S	 140.15
U+85E5.0	fTotalStrokes	 19; 21
U+85E5.0	fCangjie	TVID
U+85E5.0	fFourCorner	44904
U+85E5.0	fHakka	  jok8; rok8
U+85E5.0	fCantonese	joek6; joek2 (also); ngok6; ok6
U+85E5.0	fMandarin	yao4; yue4; shuo4; lun4; le4
U+85E5.0	fEnglish	[1] drugs; pharmaceuticals; medicine; remedy [2] kill with poison; poison
U+85E6.0	fUTF8	藦
U+85E6.0	fCNS11643	2-6662
U+85E6.0	fBig5	F2D7
U+85E6.0	fGB	3-6B49
U+85E6.0	fR/S	 140.15
U+85E6.0	fTotalStrokes	 19; 21
U+85E6.0	fCangjie	TIDQ
U+85E6.0	fFourCorner	44252
U+85E6.0	fHakka	 mo5; mo6
U+85E6.0	fCantonese	mo4; mo6; mo5
U+85E6.0	fMandarin	mo4; mo2
U+85E6.0	fEnglish	edible mushrooms
U+85E7.0	fUTF8	藧
U+85E7.0	fCNS11643	3-5B22
U+85E7.0	fGB	E-3B55
U+85E7.0	fR/S	  140.15
U+85E7.0	fTotalStrokes	 19; 21
U+85E7.0	fCangjie	TVFE
U+85E7.0	fFourCorner	44940
U+85E7.0	fCantonese	wun6
U+85E7.0	fMandarin	huan3
U+85E8.0	fUTF8	藨
U+85E8.0	fCNS11643	2-6656
U+85E8.0	fBig5	F2CB
U+85E8.0	fGB	3-6B4A
U+85E8.0	fR/S	 140.15
U+85E8.0	fTotalStrokes	 19; 21
U+85E8.0	fCangjie	TIPF
U+85E8.0	fFourCorner	44231
U+85E8.0	fHakka	 piau1;  piau3; pau2
U+85E8.0	fCantonese	biu1
U+85E8.0	fMandarin	biao1; piao3; jiao3; pao1; pao2
U+85E8.0	fEnglish	as pecies of Scirpus
U+85E9.0	fUTF8	藩
U+85E9.0	fCNS11643	1-786A
U+85E9.0	fBig5	C3BF
U+85E9.0	fGB	0-372A
U+85E9.0	fR/S	 140.15
U+85E9.0	fTotalStrokes	 19; 21
U+85E9.0	fCangjie	TEHW
U+85E9.0	fFourCorner	44169
U+85E9.0	fHakka	 fan1;  pan1; fan2
U+85E9.0	fCantonese	faan4
U+85E9.0	fMandarin	fan1; fan2
U+85E9.0	fEnglish	 fence; boundary; outlying border; hedge; frontier; barrier
U+85EA.0	fUTF8	藪
U+85EA.0	fCNS11643	1-786C
U+85EA.0	fBig5	C3C1
U+85EA.0	fGB	1-5E34
U+85EA.0	fR/S	 140.15
U+85EA.0	fTotalStrokes	 19; 21
U+85EA.0	fCangjie	TLVK
U+85EA.0	fFourCorner	44448
U+85EA.0	fHakka	  deu5;  seu5; diu5
U+85EA.0	fCantonese	sau2
U+85EA.0	fMandarin	sou3; cou4; shu2; shu3
U+85EA.0	fEnglish	[1] marsh; swamp; shallow lake overgrown with wild plants [2] assembling place [3] nest [4] used in names of places
U+85EA.1	fUTF8	藪
U+85EA.1	fCNS11643	1-786C
U+85EA.1	fBig5	C3C1
U+85EA.1	fGB	1-5E34
U+85EA.1	fR/S	 140.15
U+85EA.1	fTotalStrokes	 19; 21
U+85EA.1	fCangjie	TLVK
U+85EA.1	fFourCorner	44448
U+85EB.0	fUTF8	藫
U+85EB.0	fCNS11643	2-6651
U+85EB.0	fBig5	F2C6
U+85EB.0	fGB	3-6B4C
U+85EB.0	fR/S	 140.15
U+85EB.0	fTotalStrokes	 19; 21
U+85EB.0	fCangjie	TEMJ
U+85EB.0	fFourCorner	44146
U+85EB.0	fHakka	  tam2; rim1; jim1
U+85EB.0	fCantonese	taam4
U+85EB.0	fMandarin	tan2
U+85EB.0	fEnglish	algae
U+85EC.0	fUTF8	藬
U+85EC.0	fCNS11643	2-6659
U+85EC.0	fBig5	F2CE
U+85EC.0	fGB	5-696E
U+85EC.0	fR/S	 140.15
U+85EC.0	fTotalStrokes	 19; 21
U+85EC.0	fCangjie	TNLC
U+85EC.0	fFourCorner	44286
U+85EC.0	fHakka	 tui1; kui1; tui2
U+85EC.0	fCantonese	teoi1; teoi4
U+85EC.0	fMandarin	tui1; tui2
U+85EC.0	fEnglish	a plant, identified with the docken
U+85ED.0	fUTF8	藭
U+85ED.0	fCNS11643	2-6653
U+85ED.0	fBig5	F2C8
U+85ED.0	fGB	3-6937
U+85ED.0	fR/S	 140.15
U+85ED.0	fTotalStrokes	 19; 21
U+85ED.0	fCangjie	TJCN
U+85ED.0	fFourCorner	44427
U+85ED.0	fHakka	  kiung2
U+85ED.0	fCantonese	kung4
U+85ED.0	fMandarin	qiong2
U+85ED.0	fEnglish	used in combination for Cnidium officinale, a kind of medicinal root of a reddish colour used for liver
U+85EE.0	fUTF8	藮
U+85EE.0	fCNS11643	3-5C7B
U+85EE.0	fGB	5-6A5F
U+85EE.0	fR/S	 140.16
U+85EE.0	fTotalStrokes	 20; 22
U+85EE.0	fCangjie	TDOF
U+85EE.0	fFourCorner	44931
U+85EE.0	fCantonese	ciu4
U+85EE.0	fMandarin	qiao2
U+85EF.0	fUTF8	藯
U+85EF.0	fCNS11643	2-6663
U+85EF.0	fBig5	F2D8
U+85EF.0	fGB	5-6A5A
U+85EF.0	fR/S	 140.15
U+85EF.0	fTotalStrokes	 19; 21
U+85EF.0	fCangjie	TSIP
U+85EF.0	fFourCorner	44334
U+85EF.0	fCantonese	wai3
U+85EF.0	fMandarin	wei4
U+85F0.0	fUTF8	藰
U+85F0.0	fCNS11643	2-6661
U+85F0.0	fBig5	F2D6
U+85F0.0	fGB	5-6821
U+85F0.0	fR/S	 140.15
U+85F0.0	fTotalStrokes	 19; 21
U+85F0.0	fCangjie	THCN
U+85F0.0	fFourCorner	44120
U+85F0.0	fCantonese	lau4; lau5
U+85F0.0	fMandarin	liu2; liu3
U+85F1.0	fUTF8	藱
U+85F1.0	fCNS11643	2-6652
U+85F1.0	fBig5	F2C7
U+85F1.0	fGB	5-6A4F
U+85F1.0	fR/S	 140.15
U+85F1.0	fTotalStrokes	 19; 21
U+85F1.0	fCangjie	TKHI
U+85F1.0	fFourCorner	44113
U+85F1.0	fCantonese	wui6; wui4
U+85F1.0	fMandarin	hui2; hui4
U+85F2.0	fUTF8	藲
U+85F2.0	fCNS11643	2-665A
U+85F2.0	fBig5	F2CF
U+85F2.0	fGB	E-3B56
U+85F2.0	fR/S	 140.15
U+85F2.0	fTotalStrokes	 19; 21
U+85F2.0	fCangjie	TDSR
U+85F2.0	fFourCorner	44916
U+85F2.0	fCantonese	au1; ngau1 (also)
U+85F2.0	fMandarin	shu1
U+85F3.0	fUTF8	藳
U+85F3.0	fCNS11643	4-607B
U+85F3.0	fGB	3-6B48
U+85F3.0	fR/S	 140.15
U+85F3.0	fTotalStrokes	 19; 21
U+85F3.0	fCangjie	TYRD
U+85F3.0	fFourCorner	44900
U+85F3.0	fCantonese	gou2
U+85F3.0	fMandarin	gao3; kao4; gao4
U+85F4.0	fUTF8	藴
U+85F4.0	fCNS11643	3-5B25
U+85F4.0	fGB	1-544C
U+85F4.0	fR/S	 140.15
U+85F4.0	fTotalStrokes	 19; 21
U+85F4.0	fCangjie	XTVFT
U+85F4.0	fFourCorner	44910
U+85F4.0	fCantonese	wan3; wan5 (also); wan2 (also); wan1
U+85F4.0	fMandarin	yun4
U+85F4.0	fEnglish	collect
U+85F5.0	fUTF8	藵
U+85F5.0	fCNS11643	3-663F
U+85F5.0	fGB	E-3B57
U+85F5.0	fR/S	 140.15
U+85F5.0	fTotalStrokes	 19; 21
U+85F5.0	fCangjie	TODV
U+85F5.0	fFourCorner	4473
U+85F5.0	fCantonese	bou2
U+85F6.0	fUTF8	藶
U+85F6.0	fCNS11643	2-696A
U+85F6.0	fBig5	F4BE
U+85F6.0	fGB	1-5C42
U+85F6.0	fR/S	 140.16
U+85F6.0	fTotalStrokes	 20; 22
U+85F6.0	fCangjie	TMHM
U+85F6.0	fFourCorner	44211
U+85F6.0	fHakka	 let8; lit8
U+85F6.0	fCantonese	lik6
U+85F6.0	fMandarin	li4
U+85F6.0	fEnglish	a kind of plant
U+85F7.0	fUTF8	藷
U+85F7.0	fCNS11643	1-7870
U+85F7.0	fBig5	C3C5
U+85F7.0	fGB	E-3B58
U+85F7.0	fR/S	 140.16
U+85F7.0	fTotalStrokes	 20; 22
U+85F7.0	fCangjie	TYRA
U+85F7.0	fFourCorner	44664
U+85F7.0	fHakka	 su2; zu1; shu2; zhu1; shu3
U+85F7.0	fCantonese	syu4; zyu1; syu5
U+85F7.0	fMandarin	shu3; zhu1; chu2; shu4
U+85F7.0	fEnglish	the sweet potato
U+85F8.0	fUTF8	藸
U+85F8.0	fCNS11643	2-665B
U+85F8.0	fBig5	F2D0
U+85F8.0	fGB	E-3B59
U+85F8.0	fR/S	 140.16
U+85F8.0	fTotalStrokes	 20; 22
U+85F8.0	fCangjie	TMOA
U+85F8.0	fFourCorner	44264
U+85F8.0	fCantonese	cyu4; zyu1
U+85F8.0	fMandarin	chu2; zhu1; zha1
U+85F9.0	fUTF8	藹
U+85F9.0	fCNS11643	1-7A32
U+85F9.0	fBig5	C4A7
U+85F9.0	fGB	1-302A
U+85F9.0	fR/S	 140.16
U+85F9.0	fTotalStrokes	 20; 22
U+85F9.0	fCangjie	TYRV
U+85F9.0	fFourCorner	44627
U+85F9.0	fHakka	  ai5; ai3; oi3
U+85F9.0	fCantonese	oi2; ngoi2 (also)
U+85F9.0	fMandarin	ai3; ai4
U+85F9.0	fEnglish	[1] exuberant; lush; luxuriant [2] gentle; affable; friendly; kind; amiable [3] shady; gloomy; dim [4] beautiful
U+85FA.0	fUTF8	藺
U+85FA.0	fCNS11643	1-7A34
U+85FA.0	fBig5	C4A9
U+85FA.0	fGB	1-5D7E
U+85FA.0	fR/S	 140.16
U+85FA.0	fTotalStrokes	 20; 22
U+85FA.0	fCangjie	TANG
U+85FA.0	fFourCorner	44227
U+85FA.0	fHakka	 nin3; nin5; nin6; lin5
U+85FA.0	fCantonese	leon6
U+85FA.0	fMandarin	lin4; lian4
U+85FA.0	fEnglish	[1] a variety of rush used in making mats [2] a Chinese family name
U+85FB.0	fUTF8	藻
U+85FB.0	fCNS11643	1-7A31
U+85FB.0	fBig5	C4A6
U+85FB.0	fGB	0-5465
U+85FB.0	fR/S	 140.16
U+85FB.0	fTotalStrokes	 20; 22
U+85FB.0	fCangjie	TERD
U+85FB.0	fFourCorner	44194
U+85FB.0	fHakka	  cau5; zau3
U+85FB.0	fCantonese	zou2
U+85FB.0	fMandarin	zao3; zhao3
U+85FB.0	fEnglish	[1] diction; wording; language [2] algae; pondweeds
U+85FC.0	fUTF8	藼
U+85FC.0	fCNS11643	3-5C76
U+85FC.0	fGB	E-3B5A
U+85FC.0	fR/S	 140.16
U+85FC.0	fTotalStrokes	 20; 22
U+85FC.0	fCangjie	TJQP
U+85FC.0	fFourCorner	44330
U+85FC.0	fHakka	  sien1; hien1
U+85FC.0	fCantonese	hyun1
U+85FC.0	fMandarin	xuan1
U+85FC.0	fEnglish	a kind of day-lily carried by women who wish to have sons
U+85FD.0	fUTF8	藽
U+85FD.0	fCNS11643	2-696F
U+85FD.0	fBig5	F4C3
U+85FD.0	fGB	5-6928
U+85FD.0	fR/S	 140.16
U+85FD.0	fTotalStrokes	 20; 22
U+85FD.0	fCangjie	TYDU
U+85FD.0	fFourCorner	44916
U+85FD.0	fCantonese	can3
U+85FD.0	fMandarin	chen4; guan4; qin1
U+85FE.0	fUTF8	藾
U+85FE.0	fCNS11643	2-6967
U+85FE.0	fBig5	F4BB
U+85FE.0	fGB	3-6B2A
U+85FE.0	fR/S	 140.16
U+85FE.0	fTotalStrokes	 20; 22
U+85FE.0	fCangjie	TDLC
U+85FE.0	fFourCorner	44986
U+85FE.0	fHakka	  lai5; lai6
U+85FE.0	fCantonese	laai6; laai3
U+85FE.0	fMandarin	lai4
U+85FE.0	fEnglish	shade
U+85FF.0	fUTF8	藿
U+85FF.0	fCNS11643	2-6965
U+85FF.0	fBig5	F4B9
U+85FF.0	fGB	0-5E3D
U+85FF.0	fR/S	 140.16
U+85FF.0	fTotalStrokes	 20; 22
U+85FF.0	fCangjie	TMBG
U+85FF.0	fFourCorner	44214
U+85FF.0	fHakka	  kok7
U+85FF.0	fCantonese	fok3
U+85FF.0	fMandarin	huo4; he4; he2; sui3
U+85FF.0	fEnglish	[1] leaves of a legume; bean sprouts [2] Agastache rugosa, a kind of medicinal herb
U+8600.0	fUTF8	蘀
U+8600.0	fCNS11643	2-6969
U+8600.0	fBig5	F4BD
U+8600.0	fGB	3-6948
U+8600.0	fR/S	 140.16
U+8600.0	fTotalStrokes	 20; 22
U+8600.0	fCangjie	TQWJ
U+8600.0	fFourCorner	44541
U+8600.0	fHakka	  tok8
U+8600.0	fCantonese	tok3
U+8600.0	fMandarin	tuo4; zhe2
U+8600.0	fEnglish	 falling leaves and barks; withered leaves
U+8601.0	fUTF8	蘁
U+8601.0	fCNS11643	2-6966
U+8601.0	fBig5	F4BA
U+8601.0	fGB	5-6A5E
U+8601.0	fR/S	 140.16
U+8601.0	fTotalStrokes	 20; 22
U+8601.0	fCangjie	TMGR
U+8601.0	fFourCorner	44106
U+8601.0	fCantonese	ng6; ngok6 (also); ok6 (also)
U+8601.0	fMandarin	wu4; e4
U+8602.0	fUTF8	蘂
U+8602.0	fCNS11643	3-5C78
U+8602.0	fGB	E-3B5B
U+8602.0	fR/S	 140.16
U+8602.0	fTotalStrokes	 20; 22
U+8602.0	fCangjie	TPPD
U+8602.0	fFourCorner	44900
U+8602.0	fHakka	  lui3
U+8602.0	fCantonese	jeoi5; jeoi6 (also)
U+8602.0	fMandarin	rui3
U+8602.0	fEnglish	stamen or pistil
U+8603.0	fUTF8	蘃
U+8603.0	fCNS11643	3-5C7C
U+8603.0	fGB	E-3B5C
U+8603.0	fR/S	  140.16
U+8603.0	fTotalStrokes	 20; 22
U+8603.0	fCangjie	TYMD
U+8603.0	fFourCorner	44900
U+8603.0	fCantonese	jeoi5
U+8603.0	fMandarin	rui3
U+8604.0	fUTF8	蘄
U+8604.0	fCNS11643	2-696B
U+8604.0	fBig5	F4BF
U+8604.0	fGB	1-5E2D
U+8604.0	fR/S	 140.16
U+8604.0	fTotalStrokes	 20; 22
U+8604.0	fCangjie	TRJL
U+8604.0	fFourCorner	44521
U+8604.0	fHakka	  ki2
U+8604.0	fCantonese	kei4
U+8604.0	fMandarin	qi2; ji1; qin2
U+8604.0	fEnglish	[1] ligusticum; levisticum [2] same as $
U+8605.0	fUTF8	蘅
U+8605.0	fCNS11643	2-696D
U+8605.0	fBig5	F4C1
U+8605.0	fGB	0-5E3F
U+8605.0	fR/S	 140.16
U+8605.0	fTotalStrokes	 20; 22
U+8605.0	fCangjie	THON
U+8605.0	fFourCorner	44221
U+8605.0	fHakka	 fen2
U+8605.0	fCantonese	hang4
U+8605.0	fMandarin	heng2
U+8605.0	fEnglish	Asarum blumei, a fragrant plant of which the root is medicinal
U+8606.0	fUTF8	蘆
U+8606.0	fCNS11643	1-7A35
U+8606.0	fBig5	C4AA
U+8606.0	fGB	1-422B
U+8606.0	fR/S	 140.16
U+8606.0	fTotalStrokes	 20; 22
U+8606.0	fCangjie	TYPT
U+8606.0	fFourCorner	44217
U+8606.0	fHakka	  lu2
U+8606.0	fCantonese	lou4; lou2 (also)
U+8606.0	fMandarin	lu2; lu3; luo2
U+8606.0	fEnglish	[1] rushes; reeds [2] gourds
U+8606.1	fUTF8	蘆
U+8606.1	fCNS11643	1-7A35
U+8606.1	fBig5	C4AA
U+8606.1	fGB	1-422B
U+8606.1	fR/S	 140.16
U+8606.1	fTotalStrokes	 20; 22
U+8606.1	fCangjie	TYPT
U+8606.1	fFourCorner	44217
U+8606.1	fMandarin	lu5
U+8607.0	fUTF8	蘇
U+8607.0	fCNS11643	1-7A37
U+8607.0	fBig5	C4AC
U+8607.0	fGB	1-4B55
U+8607.0	fR/S	 140.16
U+8607.0	fTotalStrokes	 20; 22
U+8607.0	fCangjie	TNFD
U+8607.0	fFourCorner	44394
U+8607.0	fHakka	  su1; sii1; se1
U+8607.0	fCantonese	sou1
U+8607.0	fMandarin	su1; su4; shu1
U+8607.0	fEnglish	[1] purpleperilla [2] revive; resurrect; come back to life [3] awake [4] rest [5] short for Soviet Russia [6] short for Kiangsu province or Soochow [7] a Chinese family name
U+8608.0	fUTF8	蘈
U+8608.0	fCNS11643	3-5D22
U+8608.0	fGB	5-6A37
U+8608.0	fR/S	 140.16
U+8608.0	fTotalStrokes	 20; 22
U+8608.0	fCangjie	THUC
U+8608.0	fFourCorner	44280
U+8608.0	fCantonese	teoi4
U+8608.0	fMandarin	tui2
U+8609.0	fUTF8	蘉
U+8609.0	fCNS11643	2-696C
U+8609.0	fBig5	F4C0
U+8609.0	fGB	3-6B51
U+8609.0	fR/S	 140.16
U+8609.0	fTotalStrokes	 20; 22
U+8609.0	fCangjie	TWLE
U+8609.0	fFourCorner	44242
U+8609.0	fHakka	  mong2
U+8609.0	fCantonese	mong4
U+8609.0	fMandarin	mang2; meng2
U+8609.0	fEnglish	 exert oneself
U+860A.0	fUTF8	蘊
U+860A.0	fCNS11643	1-7A38
U+860A.0	fBig5	C4AD
U+860A.0	fGB	E-3B5D
U+860A.0	fR/S	 140.16
U+860A.0	fTotalStrokes	 20; 22
U+860A.0	fCangjie	TVFT
U+860A.0	fFourCorner	44917
U+860A.0	fHakka	  wun3; wun5; wun6; vun3; vun5
U+860A.0	fCantonese	wan3; wan5 (also); wan2 (also); wan1
U+860A.0	fMandarin	yun4; wen1; yun1; yun3
U+860A.0	fEnglish	[1] collect; gather [2] store; have in store [3] deep; profound [4] sweltering; sultry
U+860B.0	fUTF8	蘋
U+860B.0	fCNS11643	1-7A36
U+860B.0	fBig5	C4AB
U+860B.0	fGB	1-463B
U+860B.0	fR/S	 140.16
U+860B.0	fTotalStrokes	 20; 22
U+860B.0	fCangjie	TYHC
U+860B.0	fFourCorner	44286
U+860B.0	fMandarin	ping2
U+860B.0	fEnglish	the apple
U+860B.1	fUTF8	蘋
U+860B.1	fCNS11643	1-7A36
U+860B.1	fBig5	C4AB
U+860B.1	fGB	1-463B
U+860B.1	fR/S	 140.16
U+860B.1	fTotalStrokes	 20; 22
U+860B.1	fCangjie	TYHC
U+860B.1	fFourCorner	44286
U+860B.1	fHakka	  pin2
U+860B.1	fCantonese	pan4; ping4
U+860B.1	fMandarin	pin2
U+860B.1	fEnglish	duckweeds
U+860C.0	fUTF8	蘌
U+860C.0	fCNS11643	2-696E
U+860C.0	fBig5	F4C2
U+860C.0	fGB	3-6B58
U+860C.0	fR/S	 140.16
U+860C.0	fTotalStrokes	 20; 22
U+860C.0	fCangjie	THLF
U+860C.0	fFourCorner	44901
U+860C.0	fCantonese	jyu5
U+860C.0	fMandarin	yu3
U+860D.0	fUTF8	蘍
U+860D.0	fCNS11643	3-5C7E
U+860D.0	fGB	E-3B5E
U+860D.0	fR/S	 140.16
U+860D.0	fTotalStrokes	 20; 22
U+860D.0	fCangjie	THFS
U+860D.0	fFourCorner	44320
U+860D.0	fCantonese	fan1
U+860D.0	fMandarin	xun1
U+860E.0	fUTF8	蘎
U+860E.0	fCNS11643	4-6437
U+860E.0	fGB	E-3B5F
U+860E.0	fR/S	 140.16
U+860E.0	fTotalStrokes	 20; 22
U+860E.0	fCangjie	TMBU
U+860E.0	fFourCorner	44210
U+860E.0	fCantonese	kei3
U+860E.0	fMandarin	ji4
U+860F.0	fUTF8	蘏
U+860F.0	fCNS11643	3-5D23
U+860F.0	fGB	E-3B60
U+860F.0	fR/S	 140.16
U+860F.0	fTotalStrokes	 20; 22
U+860F.0	fCangjie	TPFC
U+860F.0	fFourCorner	44980
U+860F.0	fCantonese	gwing2
U+860F.0	fMandarin	jiong3; jiong1
U+8610.0	fUTF8	蘐
U+8610.0	fCNS11643	3-5C77
U+8610.0	fGB	E-3B61
U+8610.0	fR/S	 140.16
U+8610.0	fTotalStrokes	 20; 22
U+8610.0	fCangjie	TYRE
U+8610.0	fFourCorner	44640
U+8610.0	fCantonese	hyun1
U+8610.0	fMandarin	xian1; xuan1
U+8611.0	fUTF8	蘑
U+8611.0	fCNS11643	1-7A33
U+8611.0	fBig5	C4A8
U+8611.0	fGB	0-4422
U+8611.0	fR/S	 140.16
U+8611.0	fTotalStrokes	 20; 22
U+8611.0	fCangjie	TIDR
U+8611.0	fFourCorner	44261
U+8611.0	fHakka	 mo2
U+8611.0	fCantonese	mo4
U+8611.0	fMandarin	mo2
U+8611.0	fEnglish	a species of edible mushroom
U+8612.0	fUTF8	蘒
U+8612.0	fGB	E-3B62
U+8612.0	fR/S	 140.17
U+8612.0	fTotalStrokes	 21; 23
U+8612.0	fCangjie	THDU
U+8612.0	fCantonese	cau1
U+8612.0	fEnglish	 used in "meiboku sendaihagi"
U+8613.0	fUTF8	蘓
U+8613.0	fCNS11643	3-5D21
U+8613.0	fGB	E-3B63
U+8613.0	fR/S	 140.16
U+8613.0	fTotalStrokes	 20; 22
U+8613.0	fCangjie	THDF
U+8613.0	fFourCorner	44930
U+8613.0	fCantonese	sou1
U+8613.0	fMandarin	su1
U+8613.0	fEnglish	 thyme revive; resurrect soviet
U+8614.0	fUTF8	蘔
U+8614.0	fGB	5-6A3B
U+8614.0	fR/S	 140.16
U+8614.0	fTotalStrokes	 20; 22
U+8614.0	fCangjie	TPDC
U+8614.0	fFourCorner	4498
U+8614.0	fCantonese	gwing2
U+8614.0	fMandarin	jiong1; jiong3
U+8615.0	fUTF8	蘕
U+8615.0	fCNS11643	3-5E5B
U+8615.0	fGB	E-3B64
U+8615.0	fR/S	  140.17
U+8615.0	fTotalStrokes	 21; 23
U+8615.0	fCangjie	TVFJ
U+8615.0	fFourCorner	44930
U+8615.0	fCantonese	pung4
U+8616.0	fUTF8	蘖
U+8616.0	fCNS11643	3-5E5A
U+8616.0	fGB	0-5E41
U+8616.0	fR/S	 140.17
U+8616.0	fTotalStrokes	 21; 23
U+8616.0	fCangjie	TJD
U+8616.0	fFourCorner	44900
U+8616.0	fHakka	 ngiet7; ngiap7; nget7
U+8616.0	fCantonese	jit6; jip6 (also); nip6 (also); lip6 (also)
U+8616.0	fMandarin	nie4; bi4; bo4
U+8616.0	fEnglish	sprouts and shoots from the stump of a felled tree
U+8617.0	fUTF8	蘗
U+8617.0	fCNS11643	1-7B21
U+8617.0	fBig5	C4F4
U+8617.0	fGB	5-6A6C
U+8617.0	fR/S	 140.17
U+8617.0	fTotalStrokes	 21; 23
U+8617.0	fCangjie	TSJD
U+8617.0	fFourCorner	44904
U+8617.0	fHakka	  bak7;  pit7
U+8617.0	fCantonese	baak3; paak3 (also); jit6
U+8617.0	fMandarin	bo4; nie4; bi4
U+8617.0	fEnglish	a cork tree (Phellodendron)
U+8618.0	fUTF8	蘘
U+8618.0	fCNS11643	2-6C21
U+8618.0	fBig5	F5F1
U+8618.0	fGB	3-6B5A
U+8618.0	fR/S	 140.17
U+8618.0	fTotalStrokes	 21; 23
U+8618.0	fCangjie	TYRV
U+8618.0	fFourCorner	44732
U+8618.0	fHakka	 siong1; jong2; rong2; mi3
U+8618.0	fCantonese	joeng4
U+8618.0	fMandarin	rang2; nang2; xiang1
U+8618.0	fEnglish	a kind of wild ginger
U+8619.0	fUTF8	蘙
U+8619.0	fCNS11643	2-6C27
U+8619.0	fBig5	F5F7
U+8619.0	fGB	3-6B55
U+8619.0	fR/S	 140.17
U+8619.0	fTotalStrokes	 21; 23
U+8619.0	fCangjie	TSEM
U+8619.0	fFourCorner	44127
U+8619.0	fHakka	 ji5; ri5; i5
U+8619.0	fCantonese	ai3; ngai3 (also); ji6
U+8619.0	fMandarin	yi4
U+8619.0	fEnglish	the name of a plant
U+861A.0	fUTF8	蘚
U+861A.0	fCNS11643	1-7B23
U+861A.0	fBig5	C4F6
U+861A.0	fGB	1-5E3A
U+861A.0	fR/S	 140.17
U+861A.0	fTotalStrokes	 21; 23
U+861A.0	fCangjie	TNFQ
U+861A.0	fFourCorner	44351
U+861A.0	fHakka	  sien3; sian3; sen3
U+861A.0	fCantonese	sin2
U+861A.0	fMandarin	xian3; li4
U+861A.0	fEnglish	 moss; lichen (botany)
U+861B.0	fUTF8	蘛
U+861B.0	fCNS11643	2-6968
U+861B.0	fBig5	F4BC
U+861B.0	fGB	5-6A63
U+861B.0	fR/S	 140.17
U+861B.0	fTotalStrokes	 21; 23
U+861B.0	fCangjie	TTVB
U+861B.0	fFourCorner	44727
U+861B.0	fCantonese	juk6
U+861B.0	fMandarin	yu2; yu4
U+861C.0	fUTF8	蘜
U+861C.0	fCNS11643	2-6C26
U+861C.0	fBig5	F5F6
U+861C.0	fGB	5-6A64
U+861C.0	fR/S	 140.17
U+861C.0	fTotalStrokes	 21; 23
U+861C.0	fCangjie	TTJD
U+861C.0	fFourCorner	44527
U+861C.0	fCantonese	guk1
U+861C.0	fMandarin	ju2
U+861D.0	fUTF8	蘝
U+861D.0	fCNS11643	3-5E5C
U+861D.0	fGB	E-3B65
U+861D.0	fR/S	  140.17
U+861D.0	fTotalStrokes	 21; 23
U+861D.0	fCangjie	TOOO
U+861D.0	fFourCorner	44880
U+861D.0	fHakka	  liam5;  liam3; liam2; lim1
U+861D.0	fCantonese	lim5
U+861D.0	fMandarin	lian4; lian3; lian2
U+861D.0	fEnglish	a wild vine
U+861E.0	fUTF8	蘞
U+861E.0	fCNS11643	2-6C2D
U+861E.0	fBig5	F5FD
U+861E.0	fGB	1-5D7C
U+861E.0	fR/S	 140.17
U+861E.0	fTotalStrokes	 21; 23
U+861E.0	fCangjie	TOOK
U+861E.0	fFourCorner	44848
U+861E.0	fCantonese	lim5; lim6 (also); lim4
U+861E.0	fMandarin	lian4; lian3; lian2; xian1
U+861E.0	fEnglish	a variety of vine; convulvus
U+861F.0	fUTF8	蘟
U+861F.0	fCNS11643	2-6C24
U+861F.0	fBig5	F5F4
U+861F.0	fGB	E-3B66
U+861F.0	fR/S	 140.17
U+861F.0	fTotalStrokes	 21; 23
U+861F.0	fCangjie	TNLP
U+861F.0	fFourCorner	44237
U+861F.0	fCantonese	jan2
U+861F.0	fMandarin	yin3
U+8620.0	fUTF8	蘠
U+8620.0	fCNS11643	2-6C2B
U+8620.0	fBig5	F5FB
U+8620.0	fGB	E-3B67
U+8620.0	fR/S	 140.17
U+8620.0	fTotalStrokes	 21; 23
U+8620.0	fCangjie	TVMW
U+8620.0	fFourCorner	44261
U+8620.0	fHakka	 siong2;  ciong2
U+8620.0	fCantonese	coeng4
U+8620.0	fMandarin	qiang2
U+8621.0	fUTF8	蘡
U+8621.0	fCNS11643	2-6C2A
U+8621.0	fBig5	F5FA
U+8621.0	fGB	3-6A5C
U+8621.0	fR/S	 140.17
U+8621.0	fTotalStrokes	 21; 23
U+8621.0	fCangjie	TBCV
U+8621.0	fFourCorner	44404
U+8621.0	fHakka	 jin1; rin1
U+8621.0	fCantonese	jing1
U+8621.0	fMandarin	ying1
U+8621.0	fEnglish	a kind of wild grape-vine
U+8622.0	fUTF8	蘢
U+8622.0	fCNS11643	2-6964
U+8622.0	fBig5	F4B8
U+8622.0	fGB	1-5C57
U+8622.0	fR/S	 140.16
U+8622.0	fTotalStrokes	 20; 22
U+8622.0	fCangjie	TYBP
U+8622.0	fFourCorner	44211
U+8622.0	fHakka	  lung2
U+8622.0	fCantonese	lung4
U+8622.0	fMandarin	long2; long3; long4
U+8622.0	fEnglish	a kind of tall grass
U+8623.0	fUTF8	蘣
U+8623.0	fCNS11643	2-6C25
U+8623.0	fBig5	F5F5
U+8623.0	fGB	3-6B4F
U+8623.0	fR/S	 140.17
U+8623.0	fTotalStrokes	 20; 23
U+8623.0	fCangjie	TTCG
U+8623.0	fFourCorner	44814
U+8623.0	fCantonese	tau2
U+8623.0	fMandarin	tong4; tou3
U+8624.0	fUTF8	蘤
U+8624.0	fCNS11643	2-6322
U+8624.0	fBig5	F0B6
U+8624.0	fGB	E-3B68
U+8624.0	fR/S	 140.14; 140.17
U+8624.0	fTotalStrokes	 21; 20
U+8624.0	fCangjie	THAF
U+8624.0	fFourCorner	44627
U+8624.0	fCantonese	faa1; wai5; wai2
U+8624.0	fMandarin	wei3; hua1; po2
U+8624.0	fEnglish	flower; blossoms
U+8625.0	fUTF8	蘥
U+8625.0	fCNS11643	2-6C2E
U+8625.0	fBig5	F5FE
U+8625.0	fGB	3-6B59
U+8625.0	fR/S	 140.17
U+8625.0	fTotalStrokes	 21; 23
U+8625.0	fCangjie	TOMB
U+8625.0	fFourCorner	44227
U+8625.0	fCantonese	joek6
U+8625.0	fMandarin	yue4
U+8626.0	fUTF8	蘦
U+8626.0	fCNS11643	2-6C23
U+8626.0	fBig5	F5F3
U+8626.0	fGB	3-6B56
U+8626.0	fR/S	 140.17
U+8626.0	fTotalStrokes	 21; 23
U+8626.0	fCangjie	TMBR
U+8626.0	fFourCorner	44662
U+8626.0	fCantonese	ling4
U+8626.0	fMandarin	ling2
U+8627.0	fUTF8	蘧
U+8627.0	fCNS11643	2-6C28
U+8627.0	fBig5	F5F8
U+8627.0	fGB	0-5E3E
U+8627.0	fR/S	 140.17
U+8627.0	fTotalStrokes	 21; 23
U+8627.0	fCangjie	TYYO
U+8627.0	fFourCorner	44303
U+8627.0	fHakka	 gi2;  ki2
U+8627.0	fCantonese	keoi4
U+8627.0	fMandarin	qu2; ju4; qu4
U+8627.0	fEnglish	[1] pleasantly surprised [2] a Chinese family name
U+8628.0	fUTF8	蘨
U+8628.0	fCNS11643	3-5E58
U+8628.0	fGB	5-6A69
U+8628.0	fR/S	 140.17
U+8628.0	fTotalStrokes	 21; 23
U+8628.0	fCangjie	TBUF
U+8628.0	fFourCorner	44790
U+8628.0	fHakka	 ju2; riu2; jau2; rau2; jeu2
U+8628.0	fCantonese	jiu4; jau4
U+8628.0	fMandarin	yao2
U+8628.0	fEnglish	luxuriant vegetation
U+8629.0	fUTF8	蘩
U+8629.0	fCNS11643	2-6C2C
U+8629.0	fBig5	F5FC
U+8629.0	fGB	0-5E40
U+8629.0	fR/S	 140.17
U+8629.0	fTotalStrokes	 21; 23
U+8629.0	fCangjie	TOKF
U+8629.0	fFourCorner	44903
U+8629.0	fHakka	 fan2
U+8629.0	fCantonese	faan4
U+8629.0	fMandarin	fan2
U+8629.0	fEnglish	Artemisia stelleriana, a kind of herb; white southernwood
U+862A.0	fUTF8	蘪
U+862A.0	fCNS11643	2-6C22
U+862A.0	fBig5	F5F2
U+862A.0	fGB	3-6B5C
U+862A.0	fR/S	 140.17
U+862A.0	fTotalStrokes	 21; 23
U+862A.0	fCangjie	TIPD
U+862A.0	fFourCorner	44904
U+862A.0	fCantonese	mei4
U+862A.0	fMandarin	mi2
U+862B.0	fUTF8	蘫
U+862B.0	fCNS11643	3-5E56
U+862B.0	fGB	5-6A3D
U+862B.0	fR/S	  140.17
U+862B.0	fTotalStrokes	 21; 23
U+862B.0	fCangjie	TEST
U+862B.0	fFourCorner	44110
U+862B.0	fCantonese	ham3
U+862B.0	fMandarin	lan2
U+862C.0	fUTF8	蘬
U+862C.0	fCNS11643	2-6D77
U+862C.0	fBig5	F74A
U+862C.0	fGB	3-684E
U+862C.0	fR/S	 140.17
U+862C.0	fTotalStrokes	 21; 23
U+862C.0	fCangjie	THMB
U+862C.0	fFourCorner	44127
U+862C.0	fCantonese	kwai1; kwai2; wai2; gwai1
U+862C.0	fMandarin	kui1; hui3; kui3
U+862D.0	fUTF8	蘭
U+862D.0	fCNS11643	1-7B22
U+862D.0	fBig5	C4F5
U+862D.0	fGB	1-403C
U+862D.0	fR/S	 140.17
U+862D.0	fTotalStrokes	 21; 23
U+862D.0	fCangjie	TANW
U+862D.0	fFourCorner	44227
U+862D.0	fHakka	 lan2; lan3; lan1
U+862D.0	fCantonese	laan4; laan1 (also); laan2 (also)
U+862D.0	fMandarin	lan2
U+862D.0	fEnglish	 orchid
U+862E.0	fUTF8	蘮
U+862E.0	fCNS11643	2-6C29
U+862E.0	fBig5	F5F9
U+862E.0	fGB	3-6B57
U+862E.0	fR/S	 140.17
U+862E.0	fTotalStrokes	 21; 23
U+862E.0	fCangjie	TWLN
U+862E.0	fFourCorner	44221
U+862E.0	fCantonese	gai3; gai1
U+862E.0	fMandarin	ji4
U+862F.0	fUTF8	蘯
U+862F.0	fCNS11643	3-5E57
U+862F.0	fGB	E-3B69
U+862F.0	fR/S	 108.16; 140.17
U+862F.0	fTotalStrokes	 21; 23
U+862F.0	fCangjie	TEHT
U+862F.0	fFourCorner	44100
U+862F.0	fCantonese	dong6
U+862F.0	fMandarin	dang4
U+862F.0	fEnglish	 toss about swing rock
U+8630.0	fUTF8	蘰
U+8630.0	fGB	E-3B6A
U+8630.0	fR/S	 140.17
U+8630.0	fTotalStrokes	 21; 23
U+8630.0	fCangjie	XTVFE
U+8630.0	fCantonese	maan6
U+8631.0	fUTF8	蘱
U+8631.0	fCNS11643	2-6F43
U+8631.0	fBig5	F7F4
U+8631.0	fGB	E-3B6B
U+8631.0	fR/S	 140.19
U+8631.0	fTotalStrokes	 23; 25
U+8631.0	fCangjie	TFKC
U+8631.0	fFourCorner	44486
U+8631.0	fCantonese	leoi6
U+8631.0	fMandarin	lei4
U+8632.0	fUTF8	蘲
U+8632.0	fCNS11643	2-6D78
U+8632.0	fBig5	F74B
U+8632.0	fGB	E-3B6C
U+8632.0	fR/S	 140.18
U+8632.0	fTotalStrokes	 22; 24
U+8632.0	fCangjie	TWWG
U+8632.0	fFourCorner	44104
U+8632.0	fCantonese	leoi4
U+8632.0	fMandarin	lei2; lei3; luo2
U+8633.0	fUTF8	蘳
U+8633.0	fCNS11643	2-6D76
U+8633.0	fBig5	F749
U+8633.0	fGB	3-6B54
U+8633.0	fR/S	 140.18
U+8633.0	fTotalStrokes	 21; 24
U+8633.0	fCangjie	TTCG
U+8633.0	fFourCorner	44814
U+8633.0	fCantonese	fai1
U+8633.0	fMandarin	hua3; tong4; hui1
U+8634.0	fUTF8	蘴
U+8634.0	fCNS11643	2-6D74
U+8634.0	fBig5	F747
U+8634.0	fGB	E-3B6D
U+8634.0	fR/S	 140.18
U+8634.0	fTotalStrokes	 22; 24
U+8634.0	fCangjie	TUJT
U+8634.0	fFourCorner	44108
U+8634.0	fCantonese	fung1
U+8634.0	fMandarin	feng1
U+8634.0	fEnglish	young shoots of the rape- turnip
U+8635.0	fUTF8	蘵
U+8635.0	fCNS11643	2-6D75
U+8635.0	fBig5	F748
U+8635.0	fGB	5-6957
U+8635.0	fR/S	 140.18
U+8635.0	fTotalStrokes	 22; 24
U+8635.0	fCangjie	TSJA
U+8635.0	fFourCorner	44153
U+8635.0	fCantonese	zik1
U+8635.0	fMandarin	zhi2
U+8636.0	fUTF8	蘶
U+8636.0	fCNS11643	2-6D79
U+8636.0	fBig5	F74C
U+8636.0	fGB	5-6A67
U+8636.0	fR/S	 140.18
U+8636.0	fTotalStrokes	 22; 24
U+8636.0	fCangjie	THVI
U+8636.0	fFourCorner	44413
U+8636.0	fHakka	 ngui5; ngui6
U+8636.0	fCantonese	ngai6; ai6 (also)
U+8636.0	fMandarin	wei4
U+8636.0	fEnglish	sprouts on old plants or trees
U+8637.0	fUTF8	蘷
U+8637.0	fCNS11643	3-6640
U+8637.0	fGB	E-3B6E
U+8637.0	fR/S	 140.18
U+8637.0	fTotalStrokes	 22; 24
U+8637.0	fCangjie	TMCE
U+8637.0	fFourCorner	4424
U+8637.0	fCantonese	kwai4
U+8637.0	fMandarin	kui2
U+8637.0	fEnglish	 one-legged monster walrus
U+8638.0	fUTF8	蘸
U+8638.0	fCNS11643	1-7C45
U+8638.0	fBig5	C5D9
U+8638.0	fGB	0-553A
U+8638.0	fR/S	 140.19
U+8638.0	fTotalStrokes	 23; 25
U+8638.0	fCangjie	TMGF
U+8638.0	fFourCorner	44631
U+8638.0	fHakka	 ziam3;  zam5; ziam5
U+8638.0	fCantonese	zaam3
U+8638.0	fMandarin	zhan4
U+8638.0	fEnglish	 dip
U+8639.0	fUTF8	蘹
U+8639.0	fCNS11643	2-6F41
U+8639.0	fBig5	F7F2
U+8639.0	fGB	E-3B6F
U+8639.0	fR/S	 140.19
U+8639.0	fTotalStrokes	 23; 25
U+8639.0	fCangjie	TPYV
U+8639.0	fFourCorner	44932
U+8639.0	fHakka	  fai2
U+8639.0	fCantonese	waai4
U+8639.0	fMandarin	huai4; huai2
U+8639.0	fEnglish	~ hiong1: a plant (dill or fennel)
U+863A.0	fUTF8	蘺
U+863A.0	fCNS11643	2-6F3F
U+863A.0	fBig5	F7F0
U+863A.0	fGB	1-5D71
U+863A.0	fR/S	 140.19
U+863A.0	fTotalStrokes	 23; 25
U+863A.0	fCangjie	TYBG
U+863A.0	fFourCorner	44410
U+863A.0	fHakka	  li2
U+863A.0	fCantonese	lei4
U+863A.0	fMandarin	li2
U+863A.0	fEnglish	Gracilaria verrucosa, a kind of fragrant herb
U+863B.0	fUTF8	蘻
U+863B.0	fCNS11643	2-6F44
U+863B.0	fBig5	F7F5
U+863B.0	fGB	E-3B70
U+863B.0	fR/S	 140.19
U+863B.0	fTotalStrokes	 23; 25
U+863B.0	fCangjie	TJEF
U+863B.0	fFourCorner	44900
U+863B.0	fCantonese	hai6; gai3
U+863B.0	fMandarin	ji4
U+863C.0	fUTF8	蘼
U+863C.0	fCNS11643	2-6F42
U+863C.0	fBig5	F7F3
U+863C.0	fGB	0-5E42
U+863C.0	fR/S	 140.19
U+863C.0	fTotalStrokes	 23; 25
U+863C.0	fCangjie	TIDY
U+863C.0	fFourCorner	44211
U+863C.0	fHakka	  mi2
U+863C.0	fCantonese	mei4
U+863C.0	fMandarin	mi2; mei3
U+863C.0	fEnglish	a kind of fragrant herb
U+863D.0	fUTF8	蘽
U+863D.0	fCNS11643	3-6043
U+863D.0	fGB	5-6A74
U+863D.0	fR/S	  140.19
U+863D.0	fTotalStrokes	 23; 25
U+863D.0	fCangjie	TWWD
U+863D.0	fFourCorner	44900
U+863D.0	fHakka	 lui3; lui5
U+863D.0	fCantonese	leoi5
U+863D.0	fMandarin	lei3
U+863D.0	fEnglish	the name of a creeper
U+863E.0	fUTF8	蘾
U+863E.0	fCNS11643	2-6F45
U+863E.0	fBig5	F7F6
U+863E.0	fGB	E-3B71
U+863E.0	fR/S	 140.19
U+863E.0	fTotalStrokes	 23; 25
U+863E.0	fCangjie	TGYV
U+863E.0	fFourCorner	44132
U+863E.0	fCantonese	waai6
U+863E.0	fMandarin	huai4
U+863F.0	fUTF8	蘿
U+863F.0	fCNS11643	1-7C46
U+863F.0	fBig5	C5DA
U+863F.0	fGB	1-425C
U+863F.0	fR/S	 140.19
U+863F.0	fTotalStrokes	 23; 25
U+863F.0	fCangjie	TWLG
U+863F.0	fFourCorner	44914
U+863F.0	fHakka	  lo2; lo3
U+863F.0	fCantonese	lo4
U+863F.0	fMandarin	luo2
U+863F.0	fEnglish	[1] a kind of creeping plant; a wistaria [2] radish; turnip
U+8640.0	fUTF8	虀
U+8640.0	fCNS11643	2-6F40
U+8640.0	fBig5	F7F1
U+8640.0	fGB	E-3B72
U+8640.0	fR/S	 140.19
U+8640.0	fTotalStrokes	 23; 25
U+8640.0	fCangjie	TYSM
U+8640.0	fFourCorner	44101
U+8640.0	fHakka	 zi1
U+8640.0	fCantonese	zai1
U+8640.0	fMandarin	ji1
U+8640.0	fEnglish	minced pickles
U+8641.0	fUTF8	虁
U+8641.0	fCNS11643	3-602F
U+8641.0	fGB	E-3B73
U+8641.0	fR/S	 140.19; 35.20
U+8641.0	fTotalStrokes	 23; 25
U+8641.0	fCangjie	TMCE
U+8641.0	fFourCorner	44007
U+8641.0	fCantonese	kwai4
U+8641.0	fMandarin	kui2
U+8642.0	fUTF8	虂
U+8642.0	fCNS11643	3-6146
U+8642.0	fGB	E-3B74
U+8642.0	fR/S	 140.20; 140.21
U+8642.0	fTotalStrokes	 25; 26
U+8642.0	fCangjie	TMBR
U+8642.0	fFourCorner	44160
U+8642.0	fCantonese	lou6
U+8642.0	fMandarin	lu4
U+8643.0	fUTF8	虃
U+8643.0	fCNS11643	2-704A
U+8643.0	fBig5	F8BC
U+8643.0	fGB	5-6A7A
U+8643.0	fR/S	 140.20
U+8643.0	fTotalStrokes	 24; 26
U+8643.0	fCangjie	TEOM
U+8643.0	fFourCorner	44153
U+8643.0	fCantonese	zim1; cim1
U+8643.0	fMandarin	jian1; xian1
U+8644.0	fUTF8	虄
U+8644.0	fGB	1-7D4A
U+8644.0	fR/S	 140.20
U+8644.0	fTotalStrokes	 24; 26
U+8644.0	fCangjie	TLIM
U+8644.0	fCantonese	saat3
U+8645.0	fUTF8	虅
U+8645.0	fCNS11643	3-6641
U+8645.0	fGB	E-3B75
U+8645.0	fR/S	 140.20
U+8645.0	fTotalStrokes	 24; 26
U+8645.0	fCangjie	TBFF
U+8645.0	fFourCorner	4422
U+8645.0	fCantonese	tang4
U+8646.0	fUTF8	虆
U+8646.0	fCNS11643	2-7134
U+8646.0	fBig5	F945
U+8646.0	fGB	E-3B76
U+8646.0	fR/S	 140.21
U+8646.0	fTotalStrokes	 25; 27
U+8646.0	fCangjie	TWWF
U+8646.0	fFourCorner	44903
U+8646.0	fHakka	 lui2;  lui3
U+8646.0	fCantonese	leoi4
U+8646.0	fMandarin	lei2; lei3; luo2
U+8646.0	fEnglish	[1] grass that creeps and spread fast and luxuriantly [2] basket for carrying earth
U+8647.0	fUTF8	虇
U+8647.0	fCNS11643	2-7135
U+8647.0	fBig5	F946
U+8647.0	fGB	5-6A7B
U+8647.0	fR/S	 140.21
U+8647.0	fTotalStrokes	 25; 27
U+8647.0	fCangjie	TNTG
U+8647.0	fFourCorner	44214
U+8647.0	fCantonese	hyun2
U+8647.0	fMandarin	quan3; quan4; quan2
U+8648.0	fUTF8	虈
U+8648.0	fCNS11643	2-7136
U+8648.0	fBig5	F947
U+8648.0	fGB	E-3B77
U+8648.0	fR/S	 140.21
U+8648.0	fTotalStrokes	 25; 27
U+8648.0	fCangjie	TRRR
U+8648.0	fFourCorner	44666
U+8648.0	fCantonese	hiu1
U+8648.0	fMandarin	xiao1
U+8649.0	fUTF8	虉
U+8649.0	fCNS11643	3-6147
U+8649.0	fGB	3-6B43
U+8649.0	fR/S	 140.21
U+8649.0	fTotalStrokes	 25; 27
U+8649.0	fCangjie	TMBF
U+8649.0	fFourCorner	44220
U+8649.0	fCantonese	jik6
U+8649.0	fMandarin	yi4
U+864A.0	fUTF8	虊
U+864A.0	fCNS11643	3-617C
U+864A.0	fGB	E-3B78
U+864A.0	fR/S	 140.23
U+864A.0	fTotalStrokes	 27; 29
U+864A.0	fCangjie	TVFD
U+864A.0	fFourCorner	44900
U+864A.0	fCantonese	lyun4
U+864A.0	fMandarin	luan2
U+864B.0	fUTF8	虋
U+864B.0	fCNS11643	2-7235
U+864B.0	fBig5	F9C7
U+864B.0	fGB	3-6B62
U+864B.0	fR/S	 140.25
U+864B.0	fTotalStrokes	 29; 31
U+864B.0	fCangjie	THBH
U+864B.0	fFourCorner	44227
U+864B.0	fCantonese	mun4
U+864B.0	fMandarin	men2; min2
U+864B.0	fEnglish	asparagus a variety of red-stalked millet
U+864C.0	fUTF8	虌
U+864C.0	fCNS11643	2-722D
U+864C.0	fBig5	F9BD
U+864C.0	fGB	E-3B79
U+864C.0	fR/S	 140.25
U+864C.0	fTotalStrokes	 29; 31
U+864C.0	fCangjie	TFKU
U+864C.0	fFourCorner	44711
U+864C.0	fHakka	 bet7; biet7
U+864C.0	fCantonese	bit3
U+864C.0	fMandarin	bie1
U+864D.0	fUTF8	虍
U+864D.0	fCNS11643	2-226F
U+864D.0	fBig5	CA4F
U+864D.0	fGB	0-722E
U+864D.0	fR/S	 141.0
U+864D.0	fTotalStrokes	 4
U+864D.0	fCangjie	YP
U+864D.0	fFourCorner	21214
U+864D.0	fHakka	  fu1
U+864D.0	fCantonese	fu1; fu2
U+864D.0	fMandarin	hu1
U+864D.0	fEnglish	Kangxi radical 141
U+864E.0	fUTF8	虎
U+864E.0	fCNS11643	1-4F4D
U+864E.0	fBig5	AAEA
U+864E.0	fGB	0-3B22
U+864E.0	fR/S	 141.2
U+864E.0	fTotalStrokes	 6
U+864E.0	fCangjie	YPHU
U+864E.0	fFourCorner	21217
U+864E.0	fHakka	  fu3
U+864E.0	fCantonese	fu2
U+864E.0	fMandarin	hu3; hu4
U+864E.0	fEnglish	[1] tiger [2] brave; fierce; savage; vigorous
U+864E.1	fUTF8	虎
U+864E.1	fCNS11643	1-4F4D
U+864E.1	fBig5	AAEA
U+864E.1	fGB	0-3B22
U+864E.1	fR/S	 141.2
U+864E.1	fTotalStrokes	 6
U+864E.1	fCangjie	YPHU
U+864E.1	fFourCorner	21217
U+864E.1	fMandarin	hu5
U+864F.0	fUTF8	虏
U+864F.0	fGB	0-4232
U+864F.0	fR/S	 141.2
U+864F.0	fTotalStrokes	 6
U+864F.0	fCangjie	YPKS
U+864F.0	fHakka	 lu1
U+864F.0	fCantonese	lou5
U+864F.0	fMandarin	lu3; luo3
U+864F.0	fEnglish	simplified form of &#x865C;
U+8650.0	fUTF8	虐
U+8650.0	fCNS11643	1-534B
U+8650.0	fBig5	AD68
U+8650.0	fGB	0-4530
U+8650.0	fR/S	 141.3
U+8650.0	fTotalStrokes	 7
U+8650.0	fCangjie	YPSM
U+8650.0	fFourCorner	21211
U+8650.0	fHakka	 ngiok7;  ngiok8; gniok7; niok7
U+8650.0	fCantonese	joek6
U+8650.0	fMandarin	nuue4; ne4
U+8650.0	fEnglish	[1] cruel; ferocious; atrocious [2] tyrannize over; oppress
U+8651.0	fUTF8	虑
U+8651.0	fCNS11643	4-2F68
U+8651.0	fGB	0-4247
U+8651.0	fR/S	 141.4; 61.6
U+8651.0	fTotalStrokes	 10; 8
U+8651.0	fCangjie	YPP
U+8651.0	fFourCorner	21230
U+8651.0	fHakka	 li5
U+8651.0	fCantonese	leoi6
U+8651.0	fMandarin	mi4; luu4; l4; lu4
U+8651.0	fEnglish	simplified form of &#x616E;
U+8652.0	fUTF8	虒
U+8652.0	fCNS11643	2-325A
U+8652.0	fBig5	D3B8
U+8652.0	fGB	3-3046
U+8652.0	fR/S	 141.4
U+8652.0	fTotalStrokes	 8
U+8652.0	fCangjie	HYPU
U+8652.0	fFourCorner	71217
U+8652.0	fHakka	 sii1; ti3
U+8652.0	fCantonese	si1
U+8652.0	fMandarin	si1; ti2; zhai4; zhi4; yi2
U+8652.0	fEnglish	 an amphibious beast resembling a tiger with one horn a place name
U+8653.0	fUTF8	虓
U+8653.0	fCNS11643	2-3259
U+8653.0	fBig5	D3B7
U+8653.0	fGB	3-6149
U+8653.0	fR/S	 141.4
U+8653.0	fTotalStrokes	 8
U+8653.0	fCangjie	KNYPU
U+8653.0	fFourCorner	41017
U+8653.0	fHakka	 hau1
U+8653.0	fCantonese	haau1
U+8653.0	fMandarin	xiao1
U+8653.0	fEnglish	roar
U+8654.0	fUTF8	虔
U+8654.0	fCNS11643	1-5825
U+8654.0	fBig5	B040
U+8654.0	fGB	0-722F
U+8654.0	fR/S	 141.4
U+8654.0	fTotalStrokes	 8
U+8654.0	fCangjie	YPYK
U+8654.0	fFourCorner	21240
U+8654.0	fHakka	 kien2; ken2; kian2
U+8654.0	fCantonese	kin4
U+8654.0	fMandarin	qian2
U+8654.0	fEnglish	reverence; reverent; respectful; pious
U+8655.0	fUTF8	處
U+8655.0	fCNS11643	1-5D28
U+8655.0	fBig5	B342
U+8655.0	fGB	1-3426
U+8655.0	fR/S	 141.5
U+8655.0	fTotalStrokes	 9
U+8655.0	fCangjie	YPHEN
U+8655.0	fFourCorner	21241
U+8655.0	fHakka	  chu3; chu5; cu3
U+8655.0	fCantonese	cyu2 (also); cyu3; cyu5; syu3; syu2 (also)
U+8655.0	fMandarin	chu4; ju4
U+8655.0	fEnglish	[1] place; location; spot; locality [2] department in a government agency [3] a special quality; distinguishing mark; point
U+8655.1	fUTF8	處
U+8655.1	fCNS11643	1-5D28
U+8655.1	fBig5	B342
U+8655.1	fGB	1-3426
U+8655.1	fR/S	 141.5
U+8655.1	fTotalStrokes	 9
U+8655.1	fCangjie	YPHEN
U+8655.1	fFourCorner	21241
U+8655.1	fHakka	 cu5
U+8655.1	fMandarin	chu3
U+8655.1	fEnglish	[1] place oneself in; be faced with; live in [2] get along [3] dispose of; deal with; manage; handle [4] sentence; punish [5] dwell; live
U+8655.2	fUTF8	處
U+8655.2	fCNS11643	1-5D28
U+8655.2	fBig5	B342
U+8655.2	fGB	1-3426
U+8655.2	fR/S	 141.5
U+8655.2	fTotalStrokes	 9
U+8655.2	fCangjie	YPHEN
U+8655.2	fFourCorner	21241
U+8656.0	fUTF8	虖
U+8656.0	fCNS11643	2-3921
U+8656.0	fBig5	D77C
U+8656.0	fGB	5-6054
U+8656.0	fR/S	 141.5
U+8656.0	fTotalStrokes	 9
U+8656.0	fCangjie	YPHFD
U+8656.0	fFourCorner	21249
U+8656.0	fHakka	  fu1; fu2
U+8656.0	fCantonese	fu4; fu1
U+8656.0	fMandarin	hu1; hu4; hu2; xu1; xing1
U+8656.0	fEnglish	 cry; shout; howl; roar
U+8657.0	fUTF8	虗
U+8657.0	fCNS11643	3-3863
U+8657.0	fGB	E-3B7A
U+8657.0	fR/S	 141.5
U+8657.0	fTotalStrokes	 11; 9
U+8657.0	fCangjie	YPOM
U+8657.0	fFourCorner	21210
U+8657.0	fCantonese	heoi1
U+8657.0	fMandarin	xu1
U+8658.0	fUTF8	虘
U+8658.0	fCNS11643	3-3860
U+8658.0	fGB	5-6051
U+8658.0	fR/S	  141.5
U+8658.0	fTotalStrokes	 11; 9
U+8658.0	fCangjie	YPBM
U+8658.0	fFourCorner	21210
U+8658.0	fCantonese	co4
U+8658.0	fMandarin	cuo2
U+8659.0	fUTF8	虙
U+8659.0	fCNS11643	2-387E
U+8659.0	fBig5	D77B
U+8659.0	fGB	5-6055
U+8659.0	fR/S	 141.5
U+8659.0	fTotalStrokes	 9
U+8659.0	fCangjie	YPPH
U+8659.0	fFourCorner	21231
U+8659.0	fHakka	 fuk8
U+8659.0	fCantonese	fuk6
U+8659.0	fMandarin	fu2
U+8659.0	fEnglish	 fall prostate, lie in ambush, a division of summer
U+865A.0	fUTF8	虚
U+865A.0	fCNS11643	3-3861
U+865A.0	fGB	0-5069
U+865A.0	fR/S	 141.5
U+865A.0	fTotalStrokes	 9; 11
U+865A.0	fCangjie	YPTC
U+865A.0	fFourCorner	21210
U+865A.0	fCantonese	heoi1
U+865A.0	fMandarin	xu1; qu1
U+865A.0	fEnglish	false
U+865B.0	fUTF8	虛
U+865B.0	fCNS11643	1-6170
U+865B.0	fBig5	B5EA
U+865B.0	fGB	E-3B7B
U+865B.0	fR/S	 141.6
U+865B.0	fTotalStrokes	 10
U+865B.0	fCangjie	YPTM
U+865B.0	fFourCorner	21212
U+865B.0	fHakka	  hi1
U+865B.0	fCantonese	heoi1
U+865B.0	fMandarin	xu1; qu1
U+865B.0	fEnglish	[1] false; worthless; unreal; deceptive; unfounded; groundless [2] empty; hollow; void; unoccupied [3] weak; feeble [4] abstract; shapeless
U+865C.0	fUTF8	虜
U+865C.0	fCNS11643	1-663F
U+865C.0	fBig5	B8B8
U+865C.0	fGB	1-4232
U+865C.0	fR/S	 141.6; 141.7
U+865C.0	fTotalStrokes	 10
U+865C.0	fCangjie	YPWKS
U+865C.0	fFourCorner	21227
U+865C.0	fHakka	 lu1;  lu3
U+865C.0	fCantonese	lou5
U+865C.0	fMandarin	lu3; luo3
U+865C.0	fEnglish	[1] captive; prisoner [2] imprison; seize; take prisoner; capture alive
U+865D.0	fUTF8	虝
U+865D.0	fCNS11643	3-3E61
U+865D.0	fGB	E-3B7C
U+865D.0	fR/S	  141.6
U+865D.0	fTotalStrokes	 12; 10
U+865D.0	fCangjie	YUPHH
U+865D.0	fFourCorner	27220
U+865D.0	fCantonese	fu2
U+865D.0	fMandarin	hu3; hu4
U+865E.0	fUTF8	虞
U+865E.0	fCNS11643	1-663E
U+865E.0	fBig5	B8B7
U+865E.0	fGB	0-535D
U+865E.0	fR/S	 141.7
U+865E.0	fTotalStrokes	 11
U+865E.0	fCangjie	YPRVK
U+865E.0	fFourCorner	21281
U+865E.0	fHakka	  ngi2;  ng2; ngiu2; gni3; ngu3
U+865E.0	fCantonese	jyu4
U+865E.0	fMandarin	yu2
U+865E.0	fEnglish	[1] concern; anxieties; worries [2] expect; anticipate [3] deceive; chat [4] name of a legendary dynasty [5] name of a state in the Spring and Autumn period
U+865F.0	fUTF8	號
U+865F.0	fCNS11643	1-6640
U+865F.0	fBig5	B8B9
U+865F.0	fGB	1-3A45
U+865F.0	fR/S	 141.7
U+865F.0	fTotalStrokes	 11
U+865F.0	fCangjie	RSYPU
U+865F.0	fFourCorner	61211
U+865F.0	fHakka	 hau5;  hau1; ho6; hau3; ho1; ho2;  hau2; ho5; hau6
U+865F.0	fCantonese	hou6; hou4; hou2 (also)
U+865F.0	fMandarin	hao4
U+865F.0	fEnglish	[1] designation; title [2] sizes [3] orders; command; call [4] mark; sign; symbol [5] number (for identification or classification) [6] store; shop [7] bugle [8] date
U+865F.1	fUTF8	號
U+865F.1	fCNS11643	1-6640
U+865F.1	fBig5	B8B9
U+865F.1	fGB	1-3A45
U+865F.1	fR/S	 141.7
U+865F.1	fTotalStrokes	 11
U+865F.1	fCangjie	RSYPU
U+865F.1	fFourCorner	61211
U+865F.1	fMandarin	hao2
U+865F.1	fEnglish	 cry; shout; howl; wail
U+865F.2	fUTF8	號
U+865F.2	fCNS11643	1-6640
U+865F.2	fBig5	B8B9
U+865F.2	fGB	1-3A45
U+865F.2	fR/S	 141.7
U+865F.2	fTotalStrokes	 11
U+865F.2	fCangjie	RSYPU
U+865F.2	fFourCorner	61211
U+8660.0	fUTF8	虠
U+8660.0	fCNS11643	5-494E
U+8660.0	fGB	E-3B7D
U+8660.0	fR/S	  141.8
U+8660.0	fTotalStrokes	 14; 12
U+8660.0	fCangjie	YKYPU
U+8660.0	fFourCorner	01410
U+8660.0	fCantonese	gaau2
U+8660.0	fMandarin	jiao3; jiao1; hao2
U+8661.0	fUTF8	虡
U+8661.0	fCNS11643	2-4D5B
U+8661.0	fBig5	E3D4
U+8661.0	fGB	3-6144
U+8661.0	fR/S	 141.8
U+8661.0	fTotalStrokes	 12
U+8661.0	fCangjie	YPTMC
U+8661.0	fFourCorner	21281
U+8661.0	fHakka	 ki3;  gi3
U+8661.0	fCantonese	geoi6
U+8661.0	fMandarin	ju4
U+8661.0	fEnglish	[1] a post in the framework for a bell [2] a small table with long legs placed beside a bed
U+8662.0	fUTF8	虢
U+8662.0	fCNS11643	2-5367
U+8662.0	fBig5	E77E
U+8662.0	fGB	0-6B3D
U+8662.0	fR/S	 141.9
U+8662.0	fTotalStrokes	 13
U+8662.0	fCangjie	BIYPU
U+8662.0	fFourCorner	21317
U+8662.0	fHakka	 ngiet7;  ngut7; get7; gok7; gwut7
U+8662.0	fCantonese	gwik1
U+8662.0	fMandarin	guo2
U+8662.0	fEnglish	an ancient feudal State in Shenxi and Hunan
U+8663.0	fUTF8	虣
U+8663.0	fCNS11643	2-5A25
U+8663.0	fBig5	EB58
U+8663.0	fGB	3-614E
U+8663.0	fR/S	 141.9
U+8663.0	fTotalStrokes	 13
U+8663.0	fCangjie	MMYPU
U+8663.0	fFourCorner	11117
U+8663.0	fHakka	  pau5
U+8663.0	fCantonese	bou6
U+8663.0	fMandarin	bao4
U+8663.0	fEnglish	cruel; violent; passionate
U+8664.0	fUTF8	虤
U+8664.0	fCNS11643	2-5A27
U+8664.0	fBig5	EB5A
U+8664.0	fGB	3-614F
U+8664.0	fR/S	 141.10
U+8664.0	fTotalStrokes	 14
U+8664.0	fCangjie	YUYPU
U+8664.0	fFourCorner	21217
U+8664.0	fCantonese	ngaan4; aan4 (also)
U+8664.0	fMandarin	yan2; xuan4
U+8665.0	fUTF8	虥
U+8665.0	fCNS11643	2-5A26
U+8665.0	fBig5	EB59
U+8665.0	fGB	5-605D
U+8665.0	fR/S	 141.10
U+8665.0	fTotalStrokes	 14
U+8665.0	fCangjie	YUII
U+8665.0	fFourCorner	23253
U+8665.0	fHakka	 zan1; zan5; can3
U+8665.0	fCantonese	zaan6; caan2
U+8665.0	fMandarin	zhan4; tan2; chan2
U+8665.0	fEnglish	a striped wild cat
U+8666.0	fUTF8	虦
U+8666.0	fCNS11643	7-3261
U+8666.0	fGB	3-614B
U+8666.0	fR/S	 141.10
U+8666.0	fTotalStrokes	 16; 14
U+8666.0	fCangjie	IIYPU
U+8666.0	fFourCorner	51510
U+8666.0	fCantonese	zaan6
U+8666.0	fMandarin	zhan4
U+8667.0	fUTF8	虧
U+8667.0	fCNS11643	1-7536
U+8667.0	fBig5	C1AB
U+8667.0	fGB	1-3F77
U+8667.0	fR/S	 141.11
U+8667.0	fTotalStrokes	 15
U+8667.0	fCangjie	YGMMS
U+8667.0	fFourCorner	21227
U+8667.0	fHakka	  kui1; kwi1; k'wui1
U+8667.0	fCantonese	kwai1; fai1
U+8667.0	fMandarin	kui1
U+8667.0	fEnglish	[1] lose; fail; damage; have deficit [2] lack; want; short; deficient [3] treat unfairly; be unfair to [4] fortunately; luckily; thanks to [5] used in a mocking sense [6] the waning of the moon; wane
U+8668.0	fUTF8	虨
U+8668.0	fCNS11643	2-5F25
U+8668.0	fBig5	EE57
U+8668.0	fGB	3-3A2F
U+8668.0	fR/S	 141.11
U+8668.0	fTotalStrokes	 15
U+8668.0	fCangjie	YDHHH
U+8668.0	fFourCorner	22222
U+8668.0	fCantonese	ban1
U+8668.0	fMandarin	ban1; pin1; bin1
U+8669.0	fUTF8	虩
U+8669.0	fCNS11643	2-6326
U+8669.0	fBig5	F0BA
U+8669.0	fGB	3-6151
U+8669.0	fR/S	 141.12
U+8669.0	fTotalStrokes	 16
U+8669.0	fCangjie	FFYPU
U+8669.0	fFourCorner	92917
U+8669.0	fHakka	  hit7
U+8669.0	fCantonese	gwik1; hik1
U+8669.0	fMandarin	xi4
U+8669.0	fEnglish	fright
U+866A.0	fUTF8	虪
U+866A.0	fCNS11643	2-7173
U+866A.0	fBig5	F9A5
U+866A.0	fGB	3-6152
U+866A.0	fR/S	 141.20
U+866A.0	fTotalStrokes	 24
U+866A.0	fCangjie	YUOLF
U+866A.0	fFourCorner	27231
U+866A.0	fCantonese	suk1
U+866A.0	fMandarin	shu2; shu4; yu4; shu1
U+866B.0	fUTF8	虫
U+866B.0	fCNS11643	1-4865
U+866B.0	fBig5	A6E4
U+866B.0	fGB	0-3366
U+866B.0	fR/S	 142.0
U+866B.0	fTotalStrokes	 6
U+866B.0	fCangjie	LMI
U+866B.0	fFourCorner	50136
U+866B.0	fHakka	 cung2;  chung2; fui3; chung3; fi3
U+866B.0	fCantonese	cung4; wai2; cung2 (also)
U+866B.0	fMandarin	chong2; tong2; hui3; zhong4
U+866B.0	fEnglish	[1] simplified form of &#x87F2; [2] Kangxi radical 142
U+866C.0	fUTF8	虬
U+866C.0	fCNS11643	3-272D
U+866C.0	fGB	0-7230
U+866C.0	fR/S	 142.1
U+866C.0	fTotalStrokes	 7
U+866C.0	fCangjie	LIU
U+866C.0	fFourCorner	52110
U+866C.0	fHakka	  kiu2
U+866C.0	fCantonese	kau4
U+866C.0	fMandarin	qiu2
U+866C.0	fEnglish	0
U+866D.0	fUTF8	虭
U+866D.0	fCNS11643	2-2869
U+866D.0	fBig5	CDC9
U+866D.0	fGB	5-6064
U+866D.0	fR/S	 142.2
U+866D.0	fTotalStrokes	 8
U+866D.0	fCangjie	LISH
U+866D.0	fFourCorner	57120
U+866D.0	fCantonese	diu1
U+866D.0	fMandarin	diao1
U+866E.0	fUTF8	虮
U+866E.0	fCNS11643	2-286A
U+866E.0	fBig5	CDCA
U+866E.0	fGB	0-7231
U+866E.0	fR/S	 142.2
U+866E.0	fTotalStrokes	 8
U+866E.0	fCangjie	LIHN
U+866E.0	fFourCorner	57110
U+866E.0	fHakka	 gi1
U+866E.0	fCantonese	gei1; gei2; kei4
U+866E.0	fMandarin	ji1; ji3; qi2
U+866E.0	fEnglish	simplified form of &#x87E3;
U+866F.0	fUTF8	虯
U+866F.0	fCNS11643	2-2868
U+866F.0	fBig5	CDC8
U+866F.0	fGB	E-3B7E
U+866F.0	fR/S	 142.2
U+866F.0	fTotalStrokes	 8
U+866F.0	fCangjie	LIVL
U+866F.0	fFourCorner	52100
U+866F.0	fHakka	  kiu2
U+866F.0	fCantonese	kau4
U+866F.0	fMandarin	qiu2; jiu1; jiao4
U+866F.0	fEnglish	 young dragon
U+8670.0	fUTF8	虰
U+8670.0	fCNS11643	2-2867
U+8670.0	fBig5	CDC7
U+8670.0	fGB	3-6153
U+8670.0	fR/S	 142.2
U+8670.0	fTotalStrokes	 8
U+8670.0	fCangjie	LIMN
U+8670.0	fFourCorner	51120
U+8670.0	fHakka	  den1; ten1
U+8670.0	fCantonese	ding1; cing1
U+8670.0	fMandarin	cheng2; cheng1; ding1
U+8670.0	fEnglish	 dragon-fly
U+8671.0	fUTF8	虱
U+8671.0	fCNS11643	1-4F4E
U+8671.0	fBig5	AAEB
U+8671.0	fGB	0-4A2D
U+8671.0	fR/S	 142.2
U+8671.0	fTotalStrokes	 8
U+8671.0	fCangjie	NHLI
U+8671.0	fFourCorner	17110
U+8671.0	fHakka	  sit7;  set7
U+8671.0	fCantonese	sat1
U+8671.0	fMandarin	shi1
U+8671.0	fEnglish	 louse
U+8672.0	fUTF8	虲
U+8672.0	fCNS11643	3-6642
U+8672.0	fGB	E-3C21
U+8672.0	fR/S	 142.2
U+8672.0	fTotalStrokes	 8
U+8672.0	fCangjie	LIY
U+8672.0	fFourCorner	5310
U+8672.0	fCantonese	buk1
U+8673.0	fUTF8	虳
U+8673.0	fCNS11643	2-2D4A
U+8673.0	fBig5	D0A9
U+8673.0	fGB	5-6066
U+8673.0	fR/S	 142.3
U+8673.0	fTotalStrokes	 9
U+8673.0	fCangjie	LIPI
U+8673.0	fFourCorner	57120
U+8673.0	fCantonese	gwat1; dik1
U+8673.0	fMandarin	di4; ju2
U+8674.0	fUTF8	虴
U+8674.0	fCNS11643	2-2D48
U+8674.0	fBig5	D0A7
U+8674.0	fGB	5-6065
U+8674.0	fR/S	 142.3
U+8674.0	fTotalStrokes	 9
U+8674.0	fCangjie	LIHP
U+8674.0	fFourCorner	52114
U+8674.0	fCantonese	zaak3
U+8674.0	fMandarin	zhe2; ze2; zha4
U+8675.0	fUTF8	虵
U+8675.0	fCNS11643	3-2F23
U+8675.0	fGB	E-3C22
U+8675.0	fR/S	  142.3
U+8675.0	fTotalStrokes	 9
U+8675.0	fCangjie	LIPD
U+8675.0	fFourCorner	54110
U+8675.0	fCantonese	se4; ji4
U+8675.0	fMandarin	she2; yi2
U+8676.0	fUTF8	虶
U+8676.0	fCNS11643	3-2E7C
U+8676.0	fGB	3-6155
U+8676.0	fR/S	  142.3
U+8676.0	fTotalStrokes	 9
U+8676.0	fCangjie	LIMD
U+8676.0	fFourCorner	51140
U+8676.0	fCantonese	jyu1
U+8676.0	fMandarin	yu1
U+8677.0	fUTF8	虷
U+8677.0	fCNS11643	2-2D47
U+8677.0	fBig5	D0A6
U+8677.0	fGB	3-6154
U+8677.0	fR/S	 142.3
U+8677.0	fTotalStrokes	 9
U+8677.0	fCangjie	LIMJ
U+8677.0	fFourCorner	51140
U+8677.0	fCantonese	hon4; gon1
U+8677.0	fMandarin	han2; gan1
U+8677.0	fEnglish	worm
U+8678.0	fUTF8	虸
U+8678.0	fCNS11643	3-2E7E
U+8678.0	fGB	3-6156
U+8678.0	fR/S	  142.3
U+8678.0	fTotalStrokes	 9
U+8678.0	fCangjie	LIND
U+8678.0	fFourCorner	57140
U+8678.0	fCantonese	zi2
U+8678.0	fMandarin	zi3
U+8679.0	fUTF8	虹
U+8679.0	fCNS11643	1-534C
U+8679.0	fBig5	AD69
U+8679.0	fGB	0-3A67
U+8679.0	fR/S	 142.3
U+8679.0	fTotalStrokes	 9
U+8679.0	fCangjie	LIM
U+8679.0	fFourCorner	51110
U+8679.0	fHakka	  fung2;  giung1; fung3; kiong3; kiung6; kiung5
U+8679.0	fCantonese	hung4
U+8679.0	fMandarin	hong2; hong4; jiang4; gong4; xiang2
U+8679.0	fEnglish	 rainbow
U+867A.0	fUTF8	虺
U+867A.0	fCNS11643	1-534E
U+867A.0	fBig5	AD6B
U+867A.0	fGB	0-7233
U+867A.0	fR/S	 142.3
U+867A.0	fTotalStrokes	 9
U+867A.0	fCangjie	MULMI
U+867A.0	fFourCorner	15213
U+867A.0	fHakka	  fui3; fi3; kui3; foi1; wui3; fi5; foi5; wi3; fui1; foi3; vui1
U+867A.0	fCantonese	wai2; fui1
U+867A.0	fMandarin	hui3
U+867A.0	fEnglish	a species of venomous snake
U+867A.1	fUTF8	虺
U+867A.1	fCNS11643	1-534E
U+867A.1	fBig5	AD6B
U+867A.1	fGB	0-7233
U+867A.1	fR/S	 142.3
U+867A.1	fTotalStrokes	 9
U+867A.1	fCangjie	MULMI
U+867A.1	fFourCorner	15213
U+867A.1	fMandarin	hui1
U+867A.1	fEnglish	[1] diseased; sick; ill [2] discouraged
U+867B.0	fUTF8	虻
U+867B.0	fCNS11643	1-534D
U+867B.0	fBig5	AD6A
U+867B.0	fGB	0-7235
U+867B.0	fR/S	 142.3
U+867B.0	fTotalStrokes	 9
U+867B.0	fCangjie	LIYV
U+867B.0	fFourCorner	50110
U+867B.0	fHakka	  mong2;  mang2; mong1
U+867B.0	fCantonese	mong4; mang4
U+867B.0	fMandarin	meng2
U+867B.0	fEnglish	 gadfly
U+867C.0	fUTF8	虼
U+867C.0	fCNS11643	2-2D49
U+867C.0	fBig5	D0A8
U+867C.0	fGB	0-7234
U+867C.0	fR/S	 142.3
U+867C.0	fTotalStrokes	 9
U+867C.0	fCangjie	LION
U+867C.0	fFourCorner	58117
U+867C.0	fHakka	 gak7; giet7
U+867C.0	fCantonese	gat1; gok3
U+867C.0	fMandarin	ge4
U+867C.0	fEnglish	 flea
U+867D.0	fUTF8	虽
U+867D.0	fCNS11643	3-2F24
U+867D.0	fGB	0-4B64
U+867D.0	fR/S	 142.3
U+867D.0	fTotalStrokes	 9
U+867D.0	fCangjie	RLMI
U+867D.0	fFourCorner	60130
U+867D.0	fHakka	 sui1
U+867D.0	fCantonese	seoi1
U+867D.0	fMandarin	sui1; sui2
U+867D.0	fEnglish	simplified form of &#x96D6;
U+867E.0	fUTF8	虾
U+867E.0	fCNS11643	F-2D3E
U+867E.0	fGB	0-4F3A
U+867E.0	fR/S	 142.3
U+867E.0	fTotalStrokes	 9
U+867E.0	fCangjie	LIMY
U+867E.0	fFourCorner	5113
U+867E.0	fHakka	 ha2
U+867E.0	fCantonese	haa1; haa4
U+867E.0	fMandarin	xia1; ha1; ha2
U+867E.0	fEnglish	simplified form of &#x8766;
U+867F.0	fUTF8	虿
U+867F.0	fGB	0-7232
U+867F.0	fR/S	 142.3
U+867F.0	fTotalStrokes	 9
U+867F.0	fCangjie	MSLMI
U+867F.0	fHakka	 mai5; cai5; dai5; cui5
U+867F.0	fCantonese	caai3
U+867F.0	fMandarin	chai4; ta4
U+867F.0	fEnglish	simplified form of &#x8806;
U+8680.0	fUTF8	蚀
U+8680.0	fGB	0-4A34
U+8680.0	fR/S	 184.6; 142.3
U+8680.0	fTotalStrokes	 9
U+8680.0	fCangjie	NVLMI
U+8680.0	fHakka	 shit8; s'et8
U+8680.0	fCantonese	sik6; sit6
U+8680.0	fMandarin	shi2; li4; long2
U+8680.0	fEnglish	simplified form of &#x8755;
U+8681.0	fUTF8	蚁
U+8681.0	fGB	0-524F
U+8681.0	fR/S	 142.3
U+8681.0	fTotalStrokes	 9
U+8681.0	fCangjie	LIIK
U+8681.0	fHakka	 ngi5; ngie5; ni1
U+8681.0	fCantonese	ngai5; ai5 (also)
U+8681.0	fMandarin	yi3
U+8681.0	fEnglish	simplified form of &#x87FB;
U+8682.0	fUTF8	蚂
U+8682.0	fGB	0-426C
U+8682.0	fR/S	 142.3
U+8682.0	fTotalStrokes	 9
U+8682.0	fCangjie	LINVM
U+8682.0	fHakka	 ma1; ma3
U+8682.0	fCantonese	maa5; maa6; maa1
U+8682.0	fMandarin	ma3; ma1; ma4
U+8682.0	fEnglish	simplified form of &#x879E;
U+8683.0	fUTF8	蚃
U+8683.0	fGB	8-7D7B
U+8683.0	fR/S	 142.3
U+8683.0	fTotalStrokes	 9
U+8683.0	fCangjie	VHLMI
U+8683.0	fCantonese	hoeng2
U+8683.0	fMandarin	xiang4; xiang3
U+8684.0	fUTF8	蚄
U+8684.0	fCNS11643	3-3369
U+8684.0	fGB	3-6165
U+8684.0	fR/S	  142.4
U+8684.0	fTotalStrokes	 10
U+8684.0	fCangjie	LIYHS
U+8684.0	fFourCorner	50120
U+8684.0	fCantonese	fong1
U+8684.0	fMandarin	fang1
U+8685.0	fUTF8	蚅
U+8685.0	fCNS11643	2-3266
U+8685.0	fBig5	D3C4
U+8685.0	fGB	5-606B
U+8685.0	fR/S	 142.4
U+8685.0	fTotalStrokes	 10
U+8685.0	fCangjie	LIHSU
U+8685.0	fFourCorner	51112
U+8685.0	fCantonese	ak1; ngak1 (also)
U+8685.0	fMandarin	e4
U+8686.0	fUTF8	蚆
U+8686.0	fCNS11643	2-3263
U+8686.0	fBig5	D3C1
U+8686.0	fGB	5-6079
U+8686.0	fR/S	 142.4
U+8686.0	fTotalStrokes	 10
U+8686.0	fCangjie	LIAU
U+8686.0	fFourCorner	57117
U+8686.0	fHakka	 ba1; pa5
U+8686.0	fCantonese	baa1; paa1
U+8686.0	fMandarin	pa1; ba1
U+8686.0	fEnglish	a kind of cowry
U+8687.0	fUTF8	蚇
U+8687.0	fCNS11643	2-3261
U+8687.0	fBig5	D3BF
U+8687.0	fGB	5-6077
U+8687.0	fR/S	 142.4
U+8687.0	fTotalStrokes	 10
U+8687.0	fCangjie	LISO
U+8687.0	fFourCorner	57137
U+8687.0	fHakka	 cak7; ch'ak7; chak7
U+8687.0	fCantonese	cik3; cek3
U+8687.0	fMandarin	chi3
U+8687.0	fEnglish	 inchworm; measuring worm; looper
U+8688.0	fUTF8	蚈
U+8688.0	fCNS11643	4-2F6D
U+8688.0	fGB	3-6159
U+8688.0	fR/S	 142.6; 142.4
U+8688.0	fTotalStrokes	 12; 10
U+8688.0	fCangjie	LIMT
U+8688.0	fFourCorner	51140
U+8688.0	fHakka	 kien1; ken1
U+8688.0	fCantonese	hin1
U+8688.0	fMandarin	qian1; qiu1
U+8688.0	fEnglish	 firefly
U+8689.0	fUTF8	蚉
U+8689.0	fCNS11643	3-336C
U+8689.0	fGB	5-5079
U+8689.0	fR/S	  142.4
U+8689.0	fTotalStrokes	 10
U+8689.0	fCangjie	YKLMI
U+8689.0	fFourCorner	00130
U+8689.0	fHakka	 mun1
U+8689.0	fCantonese	man1
U+8689.0	fMandarin	wen2
U+8689.0	fEnglish	variant of &#x868A;
U+868A.0	fUTF8	蚊
U+868A.0	fCNS11643	1-5826
U+868A.0	fBig5	B041
U+868A.0	fGB	0-4E43
U+868A.0	fR/S	 142.4
U+868A.0	fTotalStrokes	 10
U+868A.0	fCangjie	LIYK
U+868A.0	fFourCorner	50140
U+868A.0	fHakka	  mun1
U+868A.0	fCantonese	man4 (also); man1
U+868A.0	fMandarin	wen2
U+868A.0	fEnglish	 mosquito; gnat
U+868B.0	fUTF8	蚋
U+868B.0	fCNS11643	2-3264
U+868B.0	fBig5	D3C2
U+868B.0	fGB	0-7238
U+868B.0	fR/S	 142.4
U+868B.0	fTotalStrokes	 10
U+868B.0	fCangjie	LIOB
U+868B.0	fFourCorner	54127
U+868B.0	fHakka	 nui5; nut8; jui3; nui6
U+868B.0	fCantonese	jeoi6
U+868B.0	fMandarin	rui4
U+868B.0	fEnglish	 gnat
U+868C.0	fUTF8	蚌
U+868C.0	fCNS11643	1-582B
U+868C.0	fBig5	B046
U+868C.0	fGB	0-3076
U+868C.0	fR/S	 142.4
U+868C.0	fTotalStrokes	 10
U+868C.0	fCangjie	LIQJ
U+868C.0	fFourCorner	55100
U+868C.0	fHakka	  pong1; pong3; kong1; pang5
U+868C.0	fCantonese	pong5
U+868C.0	fMandarin	bang4; beng4; feng1; pi2
U+868C.0	fEnglish	 oyster; mussels; mother-of-pearl
U+868D.0	fUTF8	蚍
U+868D.0	fCNS11643	2-325E
U+868D.0	fBig5	D3BC
U+868D.0	fGB	0-7237
U+868D.0	fR/S	 142.4
U+868D.0	fTotalStrokes	 10
U+868D.0	fCangjie	LIPP
U+868D.0	fFourCorner	51110
U+868D.0	fHakka	 pi2; pi3
U+868D.0	fCantonese	pei4
U+868D.0	fMandarin	bi3; pi2; bi4; pi3
U+868D.0	fEnglish	a large ant; a species of big ant
U+868E.0	fUTF8	蚎
U+868E.0	fCNS11643	2-326D
U+868E.0	fBig5	D3CB
U+868E.0	fGB	5-606E
U+868E.0	fR/S	 142.4
U+868E.0	fTotalStrokes	 10
U+868E.0	fCangjie	LIA
U+868E.0	fFourCorner	56100
U+868E.0	fHakka	 zat7; jet7; jet8
U+868E.0	fCantonese	jyut6; jat6; jim4
U+868E.0	fMandarin	yue4
U+868E.0	fEnglish	the small land-crab
U+868F.0	fUTF8	蚏
U+868F.0	fCNS11643	3-336F
U+868F.0	fGB	3-6163
U+868F.0	fR/S	 142.4
U+868F.0	fTotalStrokes	 10
U+868F.0	fCangjie	LIB
U+868F.0	fFourCorner	57120
U+868F.0	fCantonese	jyut6
U+868F.0	fMandarin	yue4
U+8690.0	fUTF8	蚐
U+8690.0	fCNS11643	2-326F
U+8690.0	fBig5	D3CD
U+8690.0	fGB	E-3C23
U+8690.0	fR/S	 142.4
U+8690.0	fTotalStrokes	 10
U+8690.0	fCangjie	LIPMM
U+8690.0	fFourCorner	57120
U+8690.0	fCantonese	gwan1
U+8690.0	fMandarin	jun1
U+8691.0	fUTF8	蚑
U+8691.0	fCNS11643	2-325F
U+8691.0	fBig5	D3BD
U+8691.0	fGB	3-615B
U+8691.0	fR/S	 142.4
U+8691.0	fTotalStrokes	 10
U+8691.0	fCangjie	LIJE
U+8691.0	fFourCorner	54147
U+8691.0	fHakka	  ki2; ki3
U+8691.0	fCantonese	kei4
U+8691.0	fMandarin	qi2; qi4
U+8691.0	fEnglish	tip-toe
U+8692.0	fUTF8	蚒
U+8692.0	fCNS11643	3-336E
U+8692.0	fGB	5-6074
U+8692.0	fR/S	 142.4
U+8692.0	fTotalStrokes	 10
U+8692.0	fCangjie	LIBY
U+8692.0	fFourCorner	57140
U+8692.0	fHakka	 tung2
U+8692.0	fCantonese	jim4
U+8692.0	fMandarin	ran2; tong2
U+8692.0	fEnglish	[1] red [2] a small feudal state
U+8693.0	fUTF8	蚓
U+8693.0	fCNS11643	1-5828
U+8693.0	fBig5	B043
U+8693.0	fGB	0-723E
U+8693.0	fR/S	 142.4
U+8693.0	fTotalStrokes	 10
U+8693.0	fCangjie	LINL
U+8693.0	fFourCorner	52100
U+8693.0	fHakka	  jin3; rin3; rin1; jin1
U+8693.0	fCantonese	jan5
U+8693.0	fMandarin	yin3
U+8693.0	fEnglish	used in combination for earthworm
U+8694.0	fUTF8	蚔
U+8694.0	fCNS11643	2-3270
U+8694.0	fBig5	D3CE
U+8694.0	fGB	5-6073
U+8694.0	fR/S	 142.4
U+8694.0	fTotalStrokes	 10
U+8694.0	fCangjie	LIHVP
U+8694.0	fFourCorner	52140
U+8694.0	fCantonese	kei4; zi6
U+8694.0	fMandarin	qi2; chi2
U+8694.0	fEnglish	ant eggs
U+8695.0	fUTF8	蚕
U+8695.0	fCNS11643	2-326B
U+8695.0	fBig5	D3C9
U+8695.0	fGB	0-324F
U+8695.0	fR/S	 142.4
U+8695.0	fTotalStrokes	 10
U+8695.0	fCangjie	HKLMI
U+8695.0	fFourCorner	20136
U+8695.0	fHakka	  cam2; tien3
U+8695.0	fCantonese	caam4; caam2 (also); tim5
U+8695.0	fMandarin	can2; tian3
U+8695.0	fEnglish	simplified form of &#x8836;
U+8696.0	fUTF8	蚖
U+8696.0	fCNS11643	2-325D
U+8696.0	fBig5	D3BB
U+8696.0	fGB	3-615A
U+8696.0	fR/S	 142.4
U+8696.0	fTotalStrokes	 10
U+8696.0	fCangjie	LIMMU
U+8696.0	fFourCorner	51111
U+8696.0	fHakka	 ngien2; ngian2
U+8696.0	fCantonese	jyun4
U+8696.0	fMandarin	yuan2; wan2
U+8696.0	fEnglish	a small venomous snake
U+8697.0	fUTF8	蚗
U+8697.0	fCNS11643	2-3262
U+8697.0	fBig5	D3C0
U+8697.0	fGB	3-6166
U+8697.0	fR/S	 142.4
U+8697.0	fTotalStrokes	 10
U+8697.0	fCangjie	LIDK
U+8697.0	fFourCorner	55130
U+8697.0	fCantonese	kyut3
U+8697.0	fMandarin	jue2
U+8698.0	fUTF8	蚘
U+8698.0	fCNS11643	2-326C
U+8698.0	fBig5	D3CA
U+8698.0	fGB	E-3C24
U+8698.0	fR/S	 142.4
U+8698.0	fTotalStrokes	 10
U+8698.0	fCangjie	LIIKU
U+8698.0	fFourCorner	53114
U+8698.0	fHakka	 ju2; fui2; riu2; fi2
U+8698.0	fCantonese	wui4; jau4
U+8698.0	fMandarin	hui2; you2
U+8698.0	fEnglish	tapeworm
U+8699.0	fUTF8	蚙
U+8699.0	fCNS11643	2-3268
U+8699.0	fBig5	D3C6
U+8699.0	fGB	3-6162
U+8699.0	fR/S	 142.4
U+8699.0	fTotalStrokes	 10
U+8699.0	fCangjie	LIOIN
U+8699.0	fFourCorner	58127
U+8699.0	fCantonese	kam4; kim4
U+8699.0	fMandarin	qin2; qian2
U+869A.0	fUTF8	蚚
U+869A.0	fCNS11643	2-3265
U+869A.0	fBig5	D3C3
U+869A.0	fGB	3-615F
U+869A.0	fR/S	 142.4
U+869A.0	fTotalStrokes	 10
U+869A.0	fCangjie	LIHML
U+869A.0	fFourCorner	52121
U+869A.0	fHakka	 ki2
U+869A.0	fCantonese	kei4; kei2
U+869A.0	fMandarin	qi2; hui2; hui4
U+869A.0	fEnglish	a fierce fly, which is constantly rubbing its head
U+869B.0	fUTF8	蚛
U+869B.0	fCNS11643	3-336B
U+869B.0	fGB	3-615E
U+869B.0	fR/S	  142.4
U+869B.0	fTotalStrokes	 10
U+869B.0	fCangjie	LIL
U+869B.0	fFourCorner	55106
U+869B.0	fCantonese	zung6
U+869B.0	fMandarin	zhong4
U+869C.0	fUTF8	蚜
U+869C.0	fCNS11643	1-582D
U+869C.0	fBig5	B048
U+869C.0	fGB	0-5141
U+869C.0	fR/S	 142.4; 142.5
U+869C.0	fTotalStrokes	 10
U+869C.0	fCangjie	LIMVH
U+869C.0	fFourCorner	51140
U+869C.0	fHakka	 nga2
U+869C.0	fCantonese	ngaa4; aa4 (also)
U+869C.0	fMandarin	ya2
U+869C.0	fEnglish	 plant louse; aphid
U+869D.0	fUTF8	蚝
U+869D.0	fCNS11643	2-326E
U+869D.0	fBig5	D3CC
U+869D.0	fGB	0-723A
U+869D.0	fR/S	 142.4
U+869D.0	fTotalStrokes	 10
U+869D.0	fCangjie	LIHQU
U+869D.0	fFourCorner	52115
U+869D.0	fHakka	 cih5; ci5; ze5
U+869D.0	fCantonese	ci3; hou4
U+869D.0	fMandarin	ci4; hao2
U+869D.0	fEnglish	hairy and poisonous caterpillars
U+869E.0	fUTF8	蚞
U+869E.0	fCNS11643	2-3260
U+869E.0	fBig5	D3BE
U+869E.0	fGB	E-3C25
U+869E.0	fR/S	 142.4
U+869E.0	fTotalStrokes	 10
U+869E.0	fCangjie	LID
U+869E.0	fFourCorner	54190
U+869E.0	fCantonese	muk6
U+869E.0	fMandarin	mu4
U+869F.0	fUTF8	蚟
U+869F.0	fCNS11643	3-336A
U+869F.0	fGB	3-6158
U+869F.0	fR/S	  142.4
U+869F.0	fTotalStrokes	 10
U+869F.0	fCangjie	LIMG
U+869F.0	fFourCorner	51110
U+869F.0	fCantonese	wong4
U+869F.0	fMandarin	wang2
U+86A0.0	fUTF8	蚠
U+86A0.0	fCNS11643	3-3370
U+86A0.0	fGB	E-3C26
U+86A0.0	fR/S	 142.4
U+86A0.0	fTotalStrokes	 10
U+86A0.0	fCangjie	CSHI
U+86A0.0	fFourCorner	80130
U+86A0.0	fCantonese	fan4
U+86A0.0	fMandarin	fen2
U+86A0.0	fEnglish	person's name
U+86A1.0	fUTF8	蚡
U+86A1.0	fCNS11643	2-3269
U+86A1.0	fBig5	D3C7
U+86A1.0	fGB	5-6072
U+86A1.0	fR/S	 142.4
U+86A1.0	fTotalStrokes	 10
U+86A1.0	fCangjie	LICSH
U+86A1.0	fFourCorner	58127
U+86A1.0	fHakka	 fun3
U+86A1.0	fCantonese	fan4
U+86A1.0	fMandarin	fen2; fen4
U+86A1.0	fEnglish	mole
U+86A2.0	fUTF8	蚢
U+86A2.0	fCNS11643	2-325B
U+86A2.0	fBig5	D3B9
U+86A2.0	fGB	3-6164
U+86A2.0	fR/S	 142.4
U+86A2.0	fTotalStrokes	 10
U+86A2.0	fCangjie	LIYHN
U+86A2.0	fFourCorner	50117
U+86A2.0	fHakka	 hong2; kong1
U+86A2.0	fCantonese	hong4
U+86A2.0	fMandarin	hang2; kang4
U+86A3.0	fUTF8	蚣
U+86A3.0	fCNS11643	1-582C
U+86A3.0	fBig5	B047
U+86A3.0	fGB	0-723C
U+86A3.0	fR/S	 142.4
U+86A3.0	fTotalStrokes	 10
U+86A3.0	fCangjie	LICI
U+86A3.0	fFourCorner	58132
U+86A3.0	fHakka	  gung1
U+86A3.0	fCantonese	gung1
U+86A3.0	fMandarin	gong1; zhong1; song1
U+86A3.0	fEnglish	used in combination for centipede
U+86A4.0	fUTF8	蚤
U+86A4.0	fCNS11643	1-5829
U+86A4.0	fBig5	B044
U+86A4.0	fGB	0-5469
U+86A4.0	fR/S	 142.4
U+86A4.0	fTotalStrokes	 10
U+86A4.0	fCangjie	EILMI
U+86A4.0	fFourCorner	17136
U+86A4.0	fHakka	  zau3; zo3
U+86A4.0	fCantonese	zou2
U+86A4.0	fMandarin	zao3; zhao3
U+86A4.0	fEnglish	 flea
U+86A5.0	fUTF8	蚥
U+86A5.0	fCNS11643	2-3267
U+86A5.0	fBig5	D3C5
U+86A5.0	fGB	5-6071
U+86A5.0	fR/S	 142.4
U+86A5.0	fTotalStrokes	 10
U+86A5.0	fCangjie	LICK
U+86A5.0	fFourCorner	58140
U+86A5.0	fCantonese	fu6; fu2
U+86A5.0	fMandarin	fu3; fu4
U+86A6.0	fUTF8	蚦
U+86A6.0	fCNS11643	3-336D
U+86A6.0	fGB	E-3C27
U+86A6.0	fR/S	  142.4
U+86A6.0	fTotalStrokes	 10
U+86A6.0	fCangjie	LIBMM
U+86A6.0	fFourCorner	57150
U+86A6.0	fCantonese	jim4; naam4 (also); laam4 (also); tim3
U+86A6.0	fMandarin	ran2; tian4
U+86A6.0	fEnglish	a boa constrictor
U+86A7.0	fUTF8	蚧
U+86A7.0	fCNS11643	2-326A
U+86A7.0	fBig5	D3C8
U+86A7.0	fGB	0-723B
U+86A7.0	fR/S	 142.4
U+86A7.0	fTotalStrokes	 10
U+86A7.0	fCangjie	LIOLL
U+86A7.0	fFourCorner	58120
U+86A7.0	fHakka	  gai5; kie5; giai5; ke3
U+86A7.0	fCantonese	gaai3; gaai2
U+86A7.0	fMandarin	jie4
U+86A7.0	fEnglish	a red spotted lizard; used as medicine
U+86A8.0	fUTF8	蚨
U+86A8.0	fCNS11643	2-325C
U+86A8.0	fBig5	D3BA
U+86A8.0	fGB	0-7236
U+86A8.0	fR/S	 142.4
U+86A8.0	fTotalStrokes	 10
U+86A8.0	fCangjie	LIQO
U+86A8.0	fFourCorner	55130
U+86A8.0	fHakka	  fu2; fu3
U+86A8.0	fCantonese	fu4; fu3
U+86A8.0	fMandarin	fu2
U+86A8.0	fEnglish	[1] a kind of water-beetle cash [2] used in combination for money or cash
U+86A9.0	fUTF8	蚩
U+86A9.0	fCNS11643	1-582A
U+86A9.0	fBig5	B045
U+86A9.0	fGB	0-723F
U+86A9.0	fR/S	 142.4
U+86A9.0	fTotalStrokes	 10
U+86A9.0	fCangjie	UMLI
U+86A9.0	fFourCorner	22136
U+86A9.0	fHakka	 chi1; cih1; ci1; che1; ce1
U+86A9.0	fCantonese	ci1
U+86A9.0	fMandarin	chi1
U+86A9.0	fEnglish	[1] a kind of worm [2] ignorant; stupid [3] laugh [4] ugly
U+86AA.0	fUTF8	蚪
U+86AA.0	fCNS11643	1-5827
U+86AA.0	fBig5	B042
U+86AA.0	fGB	0-723D
U+86AA.0	fR/S	 142.4
U+86AA.0	fTotalStrokes	 10
U+86AA.0	fCangjie	LIYJ
U+86AA.0	fFourCorner	54100
U+86AA.0	fHakka	  deu3; diu3
U+86AA.0	fCantonese	dau2
U+86AA.0	fMandarin	dou3
U+86AA.0	fEnglish	tadpole
U+86AB.0	fUTF8	蚫
U+86AB.0	fGB	E-3C28
U+86AB.0	fR/S	 142.5
U+86AB.0	fTotalStrokes	 11
U+86AB.0	fCangjie	LIPRU
U+86AB.0	fFourCorner	5711
U+86AB.0	fCantonese	baau1
U+86AB.0	fMandarin	piao2; bao4; pao2
U+86AB.0	fEnglish	 abalone dried fish a Chinese family name
U+86AC.0	fUTF8	蚬
U+86AC.0	fGB	0-7239
U+86AC.0	fR/S	 142.4
U+86AC.0	fTotalStrokes	 10
U+86AC.0	fCangjie	XLIBH
U+86AC.0	fHakka	 hien3; han3; gan5
U+86AC.0	fCantonese	hin2
U+86AC.0	fMandarin	xian4; xian3; xian2
U+86AC.0	fEnglish	simplified form of &#x8706;
U+86AD.0	fUTF8	蚭
U+86AD.0	fCNS11643	4-347A
U+86AD.0	fGB	E-3C29
U+86AD.0	fR/S	 142.5
U+86AD.0	fTotalStrokes	 11
U+86AD.0	fCangjie	LISP
U+86AD.0	fFourCorner	57110
U+86AD.0	fCantonese	nei4; lei4 (also)
U+86AD.0	fMandarin	ni2
U+86AE.0	fUTF8	蚮
U+86AE.0	fCNS11643	3-3866
U+86AE.0	fGB	3-6172
U+86AE.0	fR/S	  142.5
U+86AE.0	fTotalStrokes	 11
U+86AE.0	fCangjie	LIOIP
U+86AE.0	fFourCorner	53140
U+86AE.0	fCantonese	taai3
U+86AE.0	fMandarin	te4
U+86AF.0	fUTF8	蚯
U+86AF.0	fCNS11643	1-5D32
U+86AF.0	fBig5	B34C
U+86AF.0	fGB	0-7247
U+86AF.0	fR/S	 142.5
U+86AF.0	fTotalStrokes	 11
U+86AF.0	fCangjie	LIOM
U+86AF.0	fFourCorner	52111
U+86AF.0	fHakka	  hiu1; kiu1
U+86AF.0	fCantonese	jau1
U+86AF.0	fMandarin	qiu1
U+86AF.0	fEnglish	 earthworm
U+86B0.0	fUTF8	蚰
U+86B0.0	fCNS11643	2-3928
U+86B0.0	fBig5	D7A5
U+86B0.0	fGB	0-7244
U+86B0.0	fR/S	 142.5
U+86B0.0	fTotalStrokes	 11
U+86B0.0	fCangjie	LILW
U+86B0.0	fFourCorner	55160
U+86B0.0	fHakka	  ju2; ru2; jiu2; cuk8
U+86B0.0	fCantonese	jau4
U+86B0.0	fMandarin	you2; zhu2
U+86B0.0	fEnglish	 millipede; millipede
U+86B1.0	fUTF8	蚱
U+86B1.0	fCNS11643	1-5D31
U+86B1.0	fBig5	B34B
U+86B1.0	fGB	0-7246
U+86B1.0	fR/S	 142.5
U+86B1.0	fTotalStrokes	 11
U+86B1.0	fCangjie	LIOS
U+86B1.0	fFourCorner	58111
U+86B1.0	fHakka	 za5; zak7; zok7
U+86B1.0	fCantonese	zaak3; zaa3 (also)
U+86B1.0	fMandarin	zha4; ze2
U+86B1.0	fEnglish	 grasshopper; locust
U+86B2.0	fUTF8	蚲
U+86B2.0	fCNS11643	3-3864
U+86B2.0	fGB	3-616B
U+86B2.0	fR/S	  142.5
U+86B2.0	fTotalStrokes	 11
U+86B2.0	fCangjie	LIMFJ
U+86B2.0	fFourCorner	51140
U+86B2.0	fCantonese	ping4
U+86B2.0	fMandarin	ping2
U+86B3.0	fUTF8	蚳
U+86B3.0	fCNS11643	2-392B
U+86B3.0	fBig5	D7A8
U+86B3.0	fGB	3-6174
U+86B3.0	fR/S	 142.5
U+86B3.0	fTotalStrokes	 11
U+86B3.0	fCangjie	LIHPM
U+86B3.0	fFourCorner	52140
U+86B3.0	fHakka	 chi2; cih2; chii2
U+86B3.0	fCantonese	ci4
U+86B3.0	fMandarin	chi2; chi1
U+86B3.0	fEnglish	ant eggs
U+86B4.0	fUTF8	蚴
U+86B4.0	fCNS11643	2-392E
U+86B4.0	fBig5	D7AB
U+86B4.0	fGB	0-724A
U+86B4.0	fR/S	 142.5
U+86B4.0	fTotalStrokes	 11
U+86B4.0	fCangjie	LIVIS
U+86B4.0	fFourCorner	54127
U+86B4.0	fHakka	 jiu5
U+86B4.0	fCantonese	jau2; jau3; jau1 (also)
U+86B4.0	fMandarin	you3; you4; you1; niu4
U+86B5.0	fUTF8	蚵
U+86B5.0	fCNS11643	1-5D2E
U+86B5.0	fBig5	B348
U+86B5.0	fGB	0-7242
U+86B5.0	fR/S	 142.5
U+86B5.0	fTotalStrokes	 11
U+86B5.0	fCangjie	LIMNR
U+86B5.0	fFourCorner	51120
U+86B5.0	fHakka	  ho2; ko1; o2; ko3
U+86B5.0	fCantonese	ho4; ho2; hou4
U+86B5.0	fMandarin	ke4; he2; hao2; ke1; ke3
U+86B5.0	fEnglish	oyster
U+86B6.0	fUTF8	蚶
U+86B6.0	fCNS11643	1-5D2C
U+86B6.0	fBig5	B346
U+86B6.0	fGB	0-7240
U+86B6.0	fR/S	 142.5
U+86B6.0	fTotalStrokes	 11
U+86B6.0	fCangjie	LITM
U+86B6.0	fFourCorner	54170
U+86B6.0	fHakka	  ham1; gam1
U+86B6.0	fCantonese	ham1
U+86B6.0	fMandarin	han1
U+86B6.0	fEnglish	Arca inflata, a kind of clam
U+86B7.0	fUTF8	蚷
U+86B7.0	fCNS11643	2-3923
U+86B7.0	fBig5	D77E
U+86B7.0	fGB	3-615C
U+86B7.0	fR/S	 142.5
U+86B7.0	fTotalStrokes	 11
U+86B7.0	fCangjie	LISS
U+86B7.0	fFourCorner	57117
U+86B7.0	fCantonese	geoi6
U+86B7.0	fMandarin	ju4; qu2
U+86B8.0	fUTF8	蚸
U+86B8.0	fCNS11643	2-392C
U+86B8.0	fBig5	D7A9
U+86B8.0	fGB	3-6173
U+86B8.0	fR/S	 142.5
U+86B8.0	fTotalStrokes	 11
U+86B8.0	fCangjie	LIHMY
U+86B8.0	fFourCorner	52131
U+86B8.0	fCantonese	cik1
U+86B8.0	fMandarin	li4; chi4
U+86B9.0	fUTF8	蚹
U+86B9.0	fCNS11643	2-392A
U+86B9.0	fBig5	D7A7
U+86B9.0	fGB	3-6171
U+86B9.0	fR/S	 142.5
U+86B9.0	fTotalStrokes	 11
U+86B9.0	fCangjie	LIODI
U+86B9.0	fFourCorner	54100
U+86B9.0	fHakka	  fu5
U+86B9.0	fCantonese	fu6
U+86B9.0	fMandarin	fu4; pu2
U+86BA.0	fUTF8	蚺
U+86BA.0	fCNS11643	2-3927
U+86BA.0	fBig5	D7A4
U+86BA.0	fGB	0-7245
U+86BA.0	fR/S	 142.5
U+86BA.0	fTotalStrokes	 11
U+86BA.0	fCangjie	LIGB
U+86BA.0	fFourCorner	55157
U+86BA.0	fHakka	 ngiam1; ngiam2; ram1; lam2; jam1
U+86BA.0	fCantonese	jim4; naam4 (also); laam4 (also); cim3
U+86BA.0	fMandarin	ran2; tian4
U+86BA.0	fEnglish	a boa constrictor
U+86BB.0	fUTF8	蚻
U+86BB.0	fCNS11643	2-392F
U+86BB.0	fBig5	D7AC
U+86BB.0	fGB	3-6169
U+86BB.0	fR/S	 142.5
U+86BB.0	fTotalStrokes	 11
U+86BB.0	fCangjie	DULMI
U+86BB.0	fFourCorner	42136
U+86BB.0	fHakka	  zap7; cat8
U+86BB.0	fCantonese	zaat3; zaap3
U+86BB.0	fMandarin	zha2
U+86BB.0	fEnglish	small species of cicada
U+86BC.0	fUTF8	蚼
U+86BC.0	fCNS11643	2-3930
U+86BC.0	fBig5	D7AD
U+86BC.0	fGB	3-6175
U+86BC.0	fR/S	 142.5
U+86BC.0	fTotalStrokes	 11
U+86BC.0	fCangjie	LIPR
U+86BC.0	fFourCorner	57120
U+86BC.0	fCantonese	ngau1; au1 (also); gau2; keoi4
U+86BC.0	fMandarin	gou3; hou3; ju1; qu2
U+86BD.0	fUTF8	蚽
U+86BD.0	fCNS11643	2-3932
U+86BD.0	fBig5	D7AF
U+86BD.0	fGB	5-607D
U+86BD.0	fR/S	 142.5
U+86BD.0	fTotalStrokes	 11
U+86BD.0	fCangjie	LIMFM
U+86BD.0	fFourCorner	51119
U+86BD.0	fCantonese	pei4
U+86BD.0	fMandarin	pi2
U+86BE.0	fUTF8	蚾
U+86BE.0	fCNS11643	2-3933
U+86BE.0	fBig5	D7B0
U+86BE.0	fGB	3-617B
U+86BE.0	fR/S	 142.5
U+86BE.0	fTotalStrokes	 11
U+86BE.0	fCangjie	LIDHE
U+86BE.0	fFourCorner	54147
U+86BE.0	fHakka	 bo5;  pi2
U+86BE.0	fCantonese	pei4; bo3
U+86BE.0	fMandarin	bo3; pi2
U+86BE.0	fEnglish	 toad
U+86BF.0	fUTF8	蚿
U+86BF.0	fCNS11643	2-3922
U+86BF.0	fBig5	D77D
U+86BF.0	fGB	3-6176
U+86BF.0	fR/S	 142.5
U+86BF.0	fTotalStrokes	 11
U+86BF.0	fCangjie	LIYVI
U+86BF.0	fFourCorner	50132
U+86BF.0	fHakka	  hien2; hen2
U+86BF.0	fCantonese	jin4
U+86BF.0	fMandarin	xian2
U+86BF.0	fEnglish	millipede
U+86C0.0	fUTF8	蛀
U+86C0.0	fCNS11643	1-5D2B
U+86C0.0	fBig5	B345
U+86C0.0	fGB	0-567B
U+86C0.0	fR/S	 142.5
U+86C0.0	fTotalStrokes	 11
U+86C0.0	fCangjie	LIYG
U+86C0.0	fFourCorner	50114
U+86C0.0	fHakka	 zu5; zhu5
U+86C0.0	fCantonese	zyu3
U+86C0.0	fMandarin	zhu4
U+86C0.0	fEnglish	[1] an insects that eats into wood or books [2] eat into; bore (insects, worms)
U+86C1.0	fUTF8	蛁
U+86C1.0	fCNS11643	2-3925
U+86C1.0	fBig5	D7A2
U+86C1.0	fGB	3-617A
U+86C1.0	fR/S	 142.5
U+86C1.0	fTotalStrokes	 11
U+86C1.0	fCangjie	LISHR
U+86C1.0	fFourCorner	57162
U+86C1.0	fHakka	 diau1
U+86C1.0	fCantonese	diu1
U+86C1.0	fMandarin	diao1
U+86C2.0	fUTF8	蛂
U+86C2.0	fCNS11643	2-3924
U+86C2.0	fBig5	D7A1
U+86C2.0	fGB	5-607E
U+86C2.0	fR/S	 142.5
U+86C2.0	fTotalStrokes	 11
U+86C2.0	fCangjie	LIIKK
U+86C2.0	fFourCorner	53144
U+86C2.0	fCantonese	bit6
U+86C2.0	fMandarin	bie3; bie2; fei4; fu2; pi2
U+86C3.0	fUTF8	蛃
U+86C3.0	fCNS11643	2-3931
U+86C3.0	fBig5	D7AE
U+86C3.0	fGB	5-607C
U+86C3.0	fR/S	 142.5
U+86C3.0	fTotalStrokes	 11
U+86C3.0	fCangjie	LIMOB
U+86C3.0	fFourCorner	51127
U+86C3.0	fHakka	  biang3
U+86C3.0	fCantonese	bing2
U+86C3.0	fMandarin	bing3; beng4
U+86C3.0	fEnglish	a moth which attacks books and clothes
U+86C4.0	fUTF8	蛄
U+86C4.0	fCNS11643	1-5D2D
U+86C4.0	fBig5	B347
U+86C4.0	fGB	0-7241
U+86C4.0	fR/S	 142.5
U+86C4.0	fTotalStrokes	 11
U+86C4.0	fCangjie	LIJR
U+86C4.0	fFourCorner	54160
U+86C4.0	fHakka	 gu1
U+86C4.0	fCantonese	gu1; gwu1 (also)
U+86C4.0	fMandarin	gu1; gu3
U+86C4.0	fEnglish	 mole cricket
U+86C5.0	fUTF8	蛅
U+86C5.0	fCNS11643	2-3926
U+86C5.0	fBig5	D7A3
U+86C5.0	fGB	3-616D
U+86C5.0	fR/S	 142.5
U+86C5.0	fTotalStrokes	 11
U+86C5.0	fCangjie	LIYR
U+86C5.0	fFourCorner	51160
U+86C5.0	fHakka	 ngiam1; zham1; zam1
U+86C5.0	fCantonese	jim4; zim1 (also)
U+86C5.0	fMandarin	ran2; zhan1
U+86C5.0	fEnglish	a caterpillar
U+86C6.0	fUTF8	蛆
U+86C6.0	fCNS11643	1-5D2F
U+86C6.0	fBig5	B349
U+86C6.0	fGB	0-4779
U+86C6.0	fR/S	 142.5
U+86C6.0	fTotalStrokes	 11
U+86C6.0	fCangjie	LIBM
U+86C6.0	fFourCorner	57110
U+86C6.0	fHakka	 zu3; zu5; zii1; zih1
U+86C6.0	fCantonese	zeoi1; ceoi1 (also)
U+86C6.0	fMandarin	ju1; qu1
U+86C6.0	fEnglish	 maggot
U+86C7.0	fUTF8	蛇
U+86C7.0	fCNS11643	1-5D2A
U+86C7.0	fBig5	B344
U+86C7.0	fGB	0-495F
U+86C7.0	fR/S	 142.5
U+86C7.0	fTotalStrokes	 11
U+86C7.0	fCangjie	LIJP
U+86C7.0	fFourCorner	53111
U+86C7.0	fHakka	  sa2;  sha2; sha3; ri2; ji2; ji5
U+86C7.0	fCantonese	se4; ji4; se2 (also)
U+86C7.0	fMandarin	she2; chi2; tuo1; tuo2; ye2
U+86C7.0	fEnglish	 snake; serpent
U+86C7.1	fUTF8	蛇
U+86C7.1	fCNS11643	1-5D2A
U+86C7.1	fBig5	B344
U+86C7.1	fGB	0-495F
U+86C7.1	fR/S	 142.5
U+86C7.1	fTotalStrokes	 11
U+86C7.1	fCangjie	LIJP
U+86C7.1	fFourCorner	53111
U+86C7.1	fMandarin	yi2
U+86C7.1	fEnglish	[1] complacent [2] pretend cordiality
U+86C8.0	fUTF8	蛈
U+86C8.0	fCNS11643	2-3929
U+86C8.0	fBig5	D7A6
U+86C8.0	fGB	5-6123
U+86C8.0	fR/S	 142.5
U+86C8.0	fTotalStrokes	 11
U+86C8.0	fCangjie	LIHQO
U+86C8.0	fFourCorner	55130
U+86C8.0	fHakka	 tiet7; diet7
U+86C8.0	fCantonese	tit3
U+86C8.0	fMandarin	tie4; ti4
U+86C8.0	fEnglish	a kind of spider
U+86C9.0	fUTF8	蛉
U+86C9.0	fCNS11643	1-5D33
U+86C9.0	fBig5	B34D
U+86C9.0	fGB	0-7248
U+86C9.0	fR/S	 142.5
U+86C9.0	fTotalStrokes	 11
U+86C9.0	fCangjie	LIOII
U+86C9.0	fFourCorner	58137
U+86C9.0	fHakka	 lin2
U+86C9.0	fCantonese	ling4
U+86C9.0	fMandarin	ling2
U+86C9.0	fEnglish	[1] used in combination for a variety of dragonfly [2] used in combination for the larva of Chilo simplex, a kind of harmful insect; the mulberry insect
U+86CA.0	fUTF8	蛊
U+86CA.0	fCNS11643	4-3526
U+86CA.0	fGB	0-3946
U+86CA.0	fR/S	 142.5
U+86CA.0	fTotalStrokes	 11
U+86CA.0	fCangjie	LIBT
U+86CA.0	fFourCorner	50100
U+86CA.0	fHakka	 gu3
U+86CA.0	fCantonese	gu2; gwu2 (also)
U+86CA.0	fMandarin	gu3; gu4; ye3
U+86CA.0	fEnglish	simplified form of &#x8831;
U+86CB.0	fUTF8	蛋
U+86CB.0	fCNS11643	1-5D30
U+86CB.0	fBig5	B34A
U+86CB.0	fGB	0-3530
U+86CB.0	fR/S	 142.5
U+86CB.0	fTotalStrokes	 11
U+86CB.0	fCangjie	NOLMI
U+86CB.0	fFourCorner	17136
U+86CB.0	fHakka	  tan5; tan3; tan6; dan5
U+86CB.0	fCantonese	daan2 (also); daan6
U+86CB.0	fMandarin	dan4
U+86CB.0	fEnglish	[1] egg [2] fellow
U+86CC.0	fUTF8	蛌
U+86CC.0	fCNS11643	2-392D
U+86CC.0	fBig5	D7AA
U+86CC.0	fGB	5-6125
U+86CC.0	fR/S	 142.5
U+86CC.0	fTotalStrokes	 11
U+86CC.0	fCangjie	LIHVO
U+86CC.0	fFourCorner	52130
U+86CC.0	fHakka	 gu1
U+86CC.0	fCantonese	gu2; gwu2 (also)
U+86CC.0	fMandarin	gu3
U+86CC.0	fEnglish	cricke
U+86CD.0	fUTF8	蛍
U+86CD.0	fGB	E-3C2A
U+86CD.0	fR/S	 42.8; 142.5
U+86CD.0	fTotalStrokes	 11
U+86CD.0	fCangjie	FBLMI
U+86CD.0	fCantonese	jing4
U+86CD.0	fMandarin	ying2; ying1
U+86CD.0	fEnglish	glow-worm; luminous insect
U+86CE.0	fUTF8	蛎
U+86CE.0	fGB	0-7243
U+86CE.0	fR/S	 142.5
U+86CE.0	fTotalStrokes	 11
U+86CE.0	fCangjie	LIMMS
U+86CE.0	fHakka	 li5
U+86CE.0	fCantonese	lai6
U+86CE.0	fMandarin	li4
U+86CE.0	fEnglish	simplified form of &#x8823;
U+86CF.0	fUTF8	蛏
U+86CF.0	fCNS11643	F-383A
U+86CF.0	fGB	0-7249
U+86CF.0	fR/S	 142.5
U+86CF.0	fTotalStrokes	 11
U+86CF.0	fCangjie	LIEG
U+86CF.0	fFourCorner	5711
U+86CF.0	fHakka	 gwai3; shin5
U+86CF.0	fCantonese	cing1
U+86CF.0	fMandarin	cheng1
U+86CF.0	fEnglish	simplified form of &#x87F6;
U+86D0.0	fUTF8	蛐
U+86D0.0	fCNS11643	1-6177
U+86D0.0	fBig5	B5F1
U+86D0.0	fGB	0-7250
U+86D0.0	fR/S	 142.6
U+86D0.0	fTotalStrokes	 12
U+86D0.0	fCangjie	LITW
U+86D0.0	fFourCorner	55160
U+86D0.0	fHakka	  kiuk7
U+86D0.0	fCantonese	kuk1
U+86D0.0	fMandarin	qu1; qu2
U+86D0.0	fEnglish	 cricket
U+86D1.0	fUTF8	蛑
U+86D1.0	fCNS11643	2-4025
U+86D1.0	fBig5	DBBF
U+86D1.0	fGB	0-7256
U+86D1.0	fR/S	 142.6
U+86D1.0	fTotalStrokes	 12
U+86D1.0	fCangjie	LIIHQ
U+86D1.0	fFourCorner	53150
U+86D1.0	fHakka	  meu2
U+86D1.0	fCantonese	mau4; mai4
U+86D1.0	fMandarin	mou2
U+86D1.0	fEnglish	a marine crab
U+86D2.0	fUTF8	蛒
U+86D2.0	fCNS11643	4-3A71
U+86D2.0	fGB	3-622A
U+86D2.0	fR/S	  142.6
U+86D2.0	fTotalStrokes	 12
U+86D2.0	fCangjie	LIHER
U+86D2.0	fFourCorner	57160
U+86D2.0	fHakka	 gak7; giet7; get7
U+86D2.0	fCantonese	gaak3
U+86D2.0	fMandarin	ge2
U+86D2.0	fEnglish	a species of beetle
U+86D3.0	fUTF8	蛓
U+86D3.0	fCNS11643	2-3F78
U+86D3.0	fBig5	DBB4
U+86D3.0	fGB	5-482F
U+86D3.0	fR/S	 142.6
U+86D3.0	fTotalStrokes	 12
U+86D3.0	fCangjie	JILMI
U+86D3.0	fFourCorner	43150
U+86D3.0	fHakka	 cih5; ci5; ze5
U+86D3.0	fCantonese	ci3; zi1
U+86D3.0	fMandarin	ci4
U+86D3.0	fEnglish	caterpillar
U+86D4.0	fUTF8	蛔
U+86D4.0	fCNS11643	1-6174
U+86D4.0	fBig5	B5EE
U+86D4.0	fGB	0-3B57
U+86D4.0	fR/S	 142.6
U+86D4.0	fTotalStrokes	 12
U+86D4.0	fCangjie	LIWR
U+86D4.0	fFourCorner	56100
U+86D4.0	fHakka	  fui2; fi2
U+86D4.0	fCantonese	wui4
U+86D4.0	fMandarin	hui2
U+86D4.0	fEnglish	 ascarid; roundworm
U+86D5.0	fUTF8	蛕
U+86D5.0	fCNS11643	3-3E62
U+86D5.0	fGB	E-3C2B
U+86D5.0	fR/S	  142.6
U+86D5.0	fTotalStrokes	 12
U+86D5.0	fCangjie	LIKB
U+86D5.0	fFourCorner	54120
U+86D5.0	fHakka	 fui2; fi2
U+86D5.0	fCantonese	wui4; wui5
U+86D5.0	fMandarin	hui2
U+86D5.0	fEnglish	the common intestinal worms; the tape-worm
U+86D6.0	fUTF8	蛖
U+86D6.0	fCNS11643	2-472E
U+86D6.0	fBig5	DFE7
U+86D6.0	fGB	3-6237
U+86D6.0	fR/S	 142.7; 142.6
U+86D6.0	fTotalStrokes	 13
U+86D6.0	fCangjie	LIIUH
U+86D6.0	fFourCorner	53112
U+86D6.0	fCantonese	mong4; pong5
U+86D6.0	fMandarin	mang2; bang4; meng2
U+86D7.0	fUTF8	蛗
U+86D7.0	fCNS11643	2-4023
U+86D7.0	fBig5	DBBD
U+86D7.0	fGB	E-3C2C
U+86D7.0	fR/S	 142.6
U+86D7.0	fTotalStrokes	 12
U+86D7.0	fCangjie	HRLMI
U+86D7.0	fFourCorner	27136
U+86D7.0	fCantonese	fau6
U+86D7.0	fMandarin	fu4
U+86D7.0	fEnglish	a grasshopper, locust
U+86D8.0	fUTF8	蛘
U+86D8.0	fCNS11643	2-3F75
U+86D8.0	fBig5	DBB1
U+86D8.0	fGB	0-7255
U+86D8.0	fR/S	 142.6
U+86D8.0	fTotalStrokes	 12
U+86D8.0	fCangjie	LITQ
U+86D8.0	fFourCorner	58151
U+86D8.0	fHakka	 jong2; rong2
U+86D8.0	fCantonese	joeng5; joeng4; mai5 (also)
U+86D8.0	fMandarin	yang2; yang3
U+86D8.0	fEnglish	a weevil found in rice; etc.
U+86D9.0	fUTF8	蛙
U+86D9.0	fCNS11643	1-6172
U+86D9.0	fBig5	B5EC
U+86D9.0	fGB	0-4D5C
U+86D9.0	fR/S	 142.6
U+86D9.0	fTotalStrokes	 12
U+86D9.0	fCangjie	LIGG
U+86D9.0	fFourCorner	54114
U+86D9.0	fHakka	  wa1;  gwai3; goi2
U+86D9.0	fCantonese	waa1
U+86D9.0	fMandarin	wa1; wa2; jue2
U+86D9.0	fEnglish	 frog
U+86DA.0	fUTF8	蛚
U+86DA.0	fCNS11643	2-3F7A
U+86DA.0	fBig5	DBB6
U+86DA.0	fGB	3-6224
U+86DA.0	fR/S	 142.6
U+86DA.0	fTotalStrokes	 12
U+86DA.0	fCangjie	LIMNN
U+86DA.0	fFourCorner	52100
U+86DA.0	fHakka	 liet8
U+86DA.0	fCantonese	lit6
U+86DA.0	fMandarin	lie4
U+86DB.0	fUTF8	蛛
U+86DB.0	fCNS11643	1-6175
U+86DB.0	fBig5	B5EF
U+86DB.0	fGB	0-566B
U+86DB.0	fR/S	 142.6
U+86DB.0	fTotalStrokes	 12
U+86DB.0	fCangjie	LIHJD
U+86DB.0	fFourCorner	55190
U+86DB.0	fHakka	  zhu1; zu1; du1
U+86DB.0	fCantonese	zyu1
U+86DB.0	fMandarin	zhu1
U+86DB.0	fEnglish	 spider
U+86DC.0	fUTF8	蛜
U+86DC.0	fCNS11643	2-3F7E
U+86DC.0	fBig5	DBBA
U+86DC.0	fGB	5-6135
U+86DC.0	fR/S	 142.6
U+86DC.0	fTotalStrokes	 12
U+86DC.0	fCangjie	LIOSK
U+86DC.0	fFourCorner	57157
U+86DC.0	fHakka	 ri2; ji2; i1; ji1
U+86DC.0	fCantonese	ji1
U+86DC.0	fMandarin	yi1
U+86DC.0	fEnglish	the sow-bug (Oniscus)
U+86DD.0	fUTF8	蛝
U+86DD.0	fCNS11643	2-3F7C
U+86DD.0	fBig5	DBB8
U+86DD.0	fGB	3-6230
U+86DD.0	fR/S	 142.6
U+86DD.0	fTotalStrokes	 12
U+86DD.0	fCangjie	LIAV
U+86DD.0	fFourCorner	57132
U+86DD.0	fCantonese	haan4
U+86DD.0	fMandarin	xian2; yin2
U+86DE.0	fUTF8	蛞
U+86DE.0	fCNS11643	1-6178
U+86DE.0	fBig5	B5F2
U+86DE.0	fGB	0-7252
U+86DE.0	fR/S	 142.6
U+86DE.0	fTotalStrokes	 12
U+86DE.0	fCangjie	LIHJR
U+86DE.0	fFourCorner	52164
U+86DE.0	fHakka	 kwat7; k'wat7; {se2}; gat7
U+86DE.0	fCantonese	kut3; fut3 (also); kwut3 (also)
U+86DE.0	fMandarin	kuo4; she2
U+86DE.0	fEnglish	 snail slug; mole cricket
U+86DF.0	fUTF8	蛟
U+86DF.0	fCNS11643	1-6171
U+86DF.0	fBig5	B5EB
U+86DF.0	fGB	0-7254
U+86DF.0	fR/S	 142.6
U+86DF.0	fTotalStrokes	 12
U+86DF.0	fCangjie	LIYCK
U+86DF.0	fFourCorner	50148
U+86DF.0	fHakka	  gau1
U+86DF.0	fCantonese	gaau1
U+86DF.0	fMandarin	jiao1
U+86DF.0	fEnglish	flood dragon, a mythical creature capable of invoking storms and floods
U+86E0.0	fUTF8	蛠
U+86E0.0	fCNS11643	4-3A76
U+86E0.0	fGB	E-3C2D
U+86E0.0	fR/S	  142.6
U+86E0.0	fTotalStrokes	 12
U+86E0.0	fCangjie	LIKSS
U+86E0.0	fFourCorner	54120
U+86E0.0	fCantonese	lai6
U+86E0.0	fMandarin	li4
U+86E1.0	fUTF8	蛡
U+86E1.0	fCNS11643	3-3E63
U+86E1.0	fGB	3-6231
U+86E1.0	fR/S	  142.6
U+86E1.0	fTotalStrokes	 12
U+86E1.0	fCangjie	LISMM
U+86E1.0	fFourCorner	57120
U+86E1.0	fHakka	 hi3
U+86E1.0	fCantonese	jik6
U+86E1.0	fMandarin	yi4
U+86E2.0	fUTF8	蛢
U+86E2.0	fCNS11643	2-3F76
U+86E2.0	fBig5	DBB2
U+86E2.0	fGB	3-622C
U+86E2.0	fR/S	 142.6
U+86E2.0	fTotalStrokes	 12
U+86E2.0	fCangjie	LITT
U+86E2.0	fFourCorner	58141
U+86E2.0	fCantonese	ping4
U+86E2.0	fMandarin	ping2
U+86E3.0	fUTF8	蛣
U+86E3.0	fCNS11643	2-3F79
U+86E3.0	fBig5	DBB5
U+86E3.0	fGB	3-6222
U+86E3.0	fR/S	 142.6
U+86E3.0	fTotalStrokes	 12
U+86E3.0	fCangjie	LIGR
U+86E3.0	fFourCorner	54161
U+86E3.0	fHakka	 giet7; git7; get7
U+86E3.0	fCantonese	kit3; gat1
U+86E3.0	fMandarin	jie2; ji1; qie4; ji4
U+86E3.0	fEnglish	(dung) beetle
U+86E4.0	fUTF8	蛤
U+86E4.0	fCNS11643	1-6176
U+86E4.0	fBig5	B5F0
U+86E4.0	fGB	0-3872
U+86E4.0	fR/S	 142.6
U+86E4.0	fTotalStrokes	 12
U+86E4.0	fCangjie	LIOMR
U+86E4.0	fFourCorner	58161
U+86E4.0	fHakka	 gop7
U+86E4.0	fCantonese	gaap3 (also); haa1 (also)
U+86E4.0	fMandarin	ha2; ha1; e4
U+86E4.0	fEnglish	 toad
U+86E4.1	fUTF8	蛤
U+86E4.1	fCNS11643	1-6176
U+86E4.1	fBig5	B5F0
U+86E4.1	fGB	0-3872
U+86E4.1	fR/S	 142.6
U+86E4.1	fTotalStrokes	 12
U+86E4.1	fCangjie	LIOMR
U+86E4.1	fFourCorner	58161
U+86E4.1	fHakka	 gap7; kap7;  ha2; ham1
U+86E4.1	fCantonese	gap3; haa4
U+86E4.1	fMandarin	ge2
U+86E4.1	fEnglish	 clam
U+86E5.0	fUTF8	蛥
U+86E5.0	fCNS11643	4-3A6F
U+86E5.0	fGB	3-622B
U+86E5.0	fR/S	  142.6
U+86E5.0	fTotalStrokes	 12
U+86E5.0	fCangjie	LININ
U+86E5.0	fFourCorner	57120
U+86E5.0	fCantonese	sit6
U+86E5.0	fMandarin	she2
U+86E6.0	fUTF8	蛦
U+86E6.0	fCNS11643	2-3F77
U+86E6.0	fBig5	DBB3
U+86E6.0	fGB	3-6225
U+86E6.0	fR/S	 142.6
U+86E6.0	fTotalStrokes	 12
U+86E6.0	fCangjie	LIKN
U+86E6.0	fFourCorner	55132
U+86E6.0	fHakka	 ji2
U+86E6.0	fCantonese	ji4
U+86E6.0	fMandarin	yi2; ti2
U+86E7.0	fUTF8	蛧
U+86E7.0	fCNS11643	5-3B7B
U+86E7.0	fGB	5-6132
U+86E7.0	fR/S	  142.6
U+86E7.0	fTotalStrokes	 12
U+86E7.0	fCangjie	LIBKK
U+86E7.0	fFourCorner	57120
U+86E7.0	fCantonese	mong5
U+86E7.0	fMandarin	wang3
U+86E8.0	fUTF8	蛨
U+86E8.0	fCNS11643	2-4024
U+86E8.0	fBig5	DBBE
U+86E8.0	fGB	3-6223
U+86E8.0	fR/S	 142.6
U+86E8.0	fTotalStrokes	 12
U+86E8.0	fCangjie	LIMA
U+86E8.0	fFourCorner	51160
U+86E8.0	fCantonese	mak6
U+86E8.0	fMandarin	mo4
U+86E9.0	fUTF8	蛩
U+86E9.0	fCNS11643	2-4022
U+86E9.0	fBig5	DBBC
U+86E9.0	fGB	0-724B
U+86E9.0	fR/S	 142.6
U+86E9.0	fTotalStrokes	 12
U+86E9.0	fCangjie	MNLMI
U+86E9.0	fFourCorner	17136
U+86E9.0	fHakka	  kiung2
U+86E9.0	fCantonese	kung4
U+86E9.0	fMandarin	qiong2; gong3
U+86E9.0	fEnglish	[1] cricket [2] locust
U+86EA.0	fUTF8	蛪
U+86EA.0	fCNS11643	2-3F7B
U+86EA.0	fBig5	DBB7
U+86EA.0	fGB	3-617E
U+86EA.0	fR/S	 142.6
U+86EA.0	fTotalStrokes	 12
U+86EA.0	fCangjie	QHLMI
U+86EA.0	fFourCorner	57136
U+86EA.0	fCantonese	kit3; jit6
U+86EA.0	fMandarin	qie4; nie4; ye1
U+86EB.0	fUTF8	蛫
U+86EB.0	fCNS11643	2-3F7D
U+86EB.0	fBig5	DBB9
U+86EB.0	fGB	3-6228
U+86EB.0	fR/S	 142.6
U+86EB.0	fTotalStrokes	 12
U+86EB.0	fCangjie	LINMU
U+86EB.0	fFourCorner	57112
U+86EB.0	fCantonese	gwai2
U+86EB.0	fMandarin	gui3
U+86EC.0	fUTF8	蛬
U+86EC.0	fCNS11643	2-4021
U+86EC.0	fBig5	DBBB
U+86EC.0	fGB	5-612B
U+86EC.0	fR/S	 142.6
U+86EC.0	fTotalStrokes	 12
U+86EC.0	fCangjie	TCLMI
U+86EC.0	fFourCorner	44136
U+86EC.0	fHakka	  kiung2;  giung3
U+86EC.0	fCantonese	gung2 (also); kung4
U+86EC.0	fMandarin	gong3; qiong2; gong4
U+86ED.0	fUTF8	蛭
U+86ED.0	fCNS11643	1-6173
U+86ED.0	fBig5	B5ED
U+86ED.0	fGB	0-724E
U+86ED.0	fR/S	 142.6
U+86ED.0	fTotalStrokes	 12
U+86ED.0	fCangjie	LIMIG
U+86ED.0	fFourCorner	51114
U+86ED.0	fHakka	 zhit7; diet7; tsiit7; zit7; det7
U+86ED.0	fCantonese	zat6
U+86ED.0	fMandarin	zhi4; die2
U+86ED.0	fEnglish	 leech
U+86EE.0	fUTF8	蛮
U+86EE.0	fCNS11643	3-3E68
U+86EE.0	fGB	0-4279
U+86EE.0	fR/S	 142.6
U+86EE.0	fTotalStrokes	 12
U+86EE.0	fCangjie	YCLMI
U+86EE.0	fFourCorner	00130
U+86EE.0	fHakka	 man2
U+86EE.0	fCantonese	maan4
U+86EE.0	fMandarin	man2
U+86EE.0	fEnglish	simplified form of &#x883B;
U+86EF.0	fUTF8	蛯
U+86EF.0	fGB	E-3C2E
U+86EF.0	fR/S	 142.6
U+86EF.0	fTotalStrokes	 12
U+86EF.0	fCangjie	LIJKP
U+86EF.0	fCantonese	lou5
U+86EF.0	fEnglish	 shrimp
U+86F0.0	fUTF8	蛰
U+86F0.0	fGB	0-555D
U+86F0.0	fR/S	 142.6
U+86F0.0	fTotalStrokes	 12
U+86F0.0	fCangjie	QILMI
U+86F0.0	fHakka	 zhi5; chit8
U+86F0.0	fCantonese	zat6; zik6 (also); zit3 (also)
U+86F0.0	fMandarin	zhi2; zhe2; chi4
U+86F0.0	fEnglish	simplified form of &#x87C4;
U+86F1.0	fUTF8	蛱
U+86F1.0	fGB	0-724C
U+86F1.0	fR/S	 142.6
U+86F1.0	fTotalStrokes	 12
U+86F1.0	fCangjie	LIKT
U+86F1.0	fHakka	 giap7
U+86F1.0	fCantonese	gaap3
U+86F1.0	fMandarin	jia2; xia2
U+86F1.0	fEnglish	simplified form of &#x86FA;
U+86F2.0	fUTF8	蛲
U+86F2.0	fGB	0-724D
U+86F2.0	fR/S	 142.6
U+86F2.0	fTotalStrokes	 12
U+86F2.0	fCangjie	LIJPU
U+86F2.0	fHakka	 ngiau2
U+86F2.0	fCantonese	jiu4; naau4 (also); laau4 (also)
U+86F2.0	fMandarin	rao2; nao2; rao4; yao1; yao2
U+86F2.0	fEnglish	simplified form of &#x87EF;
U+86F3.0	fUTF8	蛳
U+86F3.0	fGB	0-724F
U+86F3.0	fR/S	 142.6
U+86F3.0	fTotalStrokes	 12
U+86F3.0	fCangjie	LILLB
U+86F3.0	fHakka	 sii1
U+86F3.0	fCantonese	si1
U+86F3.0	fMandarin	si1; shi1
U+86F3.0	fEnglish	simplified form of &#x8784;
U+86F4.0	fUTF8	蛴
U+86F4.0	fGB	0-7253
U+86F4.0	fR/S	 142.6
U+86F4.0	fTotalStrokes	 12
U+86F4.0	fCangjie	LIYKL
U+86F4.0	fHakka	 ci2
U+86F4.0	fCantonese	cai4
U+86F4.0	fMandarin	qi2
U+86F4.0	fEnglish	simplified form of &#x8810;
U+86F5.0	fUTF8	蛵
U+86F5.0	fCNS11643	2-472F
U+86F5.0	fBig5	DFE8
U+86F5.0	fGB	3-617D
U+86F5.0	fR/S	 142.7
U+86F5.0	fTotalStrokes	 13
U+86F5.0	fCangjie	LIMVM
U+86F5.0	fFourCorner	51111
U+86F5.0	fHakka	 hin1; kin1
U+86F5.0	fCantonese	hing1
U+86F5.0	fMandarin	xing1
U+86F5.0	fEnglish	dragonfly
U+86F6.0	fUTF8	蛶
U+86F6.0	fCNS11643	2-4735
U+86F6.0	fBig5	DFEE
U+86F6.0	fGB	E-3C2F
U+86F6.0	fR/S	 142.7
U+86F6.0	fTotalStrokes	 13
U+86F6.0	fCangjie	LIBDI
U+86F6.0	fFourCorner	52143
U+86F6.0	fCantonese	lyut6
U+86F6.0	fMandarin	lie4
U+86F7.0	fUTF8	蛷
U+86F7.0	fCNS11643	2-472B
U+86F7.0	fBig5	DFE4
U+86F7.0	fGB	5-6144
U+86F7.0	fR/S	 142.7
U+86F7.0	fTotalStrokes	 13
U+86F7.0	fCangjie	LIIJE
U+86F7.0	fFourCorner	53132
U+86F7.0	fHakka	 kiu2
U+86F7.0	fCantonese	kau4
U+86F7.0	fMandarin	qiu2; ju1
U+86F7.0	fEnglish	[1] spider-millipede [2] sore
U+86F8.0	fUTF8	蛸
U+86F8.0	fCNS11643	2-4731
U+86F8.0	fBig5	DFEA
U+86F8.0	fGB	0-7259
U+86F8.0	fR/S	 142.7
U+86F8.0	fTotalStrokes	 13
U+86F8.0	fCangjie	LIFB
U+86F8.0	fFourCorner	59127
U+86F8.0	fHakka	 siau1; sau1; seu1
U+86F8.0	fCantonese	saau1; siu1
U+86F8.0	fMandarin	xiao1
U+86F8.0	fEnglish	used in combination for the mantis's egg capsule
U+86F8.1	fUTF8	蛸
U+86F8.1	fCNS11643	2-4731
U+86F8.1	fBig5	DFEA
U+86F8.1	fGB	0-7259
U+86F8.1	fR/S	 142.7
U+86F8.1	fTotalStrokes	 13
U+86F8.1	fCangjie	LIFB
U+86F8.1	fFourCorner	59127
U+86F8.1	fMandarin	shao1
U+86F8.1	fEnglish	a variety of spider
U+86F9.0	fUTF8	蛹
U+86F9.0	fCNS11643	1-6641
U+86F9.0	fBig5	B8BA
U+86F9.0	fGB	0-533C
U+86F9.0	fR/S	 142.7
U+86F9.0	fTotalStrokes	 13
U+86F9.0	fCangjie	LINIB
U+86F9.0	fFourCorner	57127
U+86F9.0	fHakka	  jung3; rung3
U+86F9.0	fCantonese	jung2; jung5
U+86F9.0	fMandarin	yong3
U+86F9.0	fEnglish	 chrysalis; pupa
U+86FA.0	fUTF8	蛺
U+86FA.0	fCNS11643	2-472D
U+86FA.0	fBig5	DFE6
U+86FA.0	fGB	1-724C
U+86FA.0	fR/S	 142.7
U+86FA.0	fTotalStrokes	 13
U+86FA.0	fCangjie	LIKOO
U+86FA.0	fFourCorner	54138
U+86FA.0	fHakka	 giap7; hiap7; gap7
U+86FA.0	fCantonese	gaap3
U+86FA.0	fMandarin	jia2; xia2
U+86FA.0	fEnglish	butterflies
U+86FB.0	fUTF8	蛻
U+86FB.0	fCNS11643	1-6647
U+86FB.0	fBig5	B8C0
U+86FB.0	fGB	E-3C30
U+86FB.0	fR/S	 142.7
U+86FB.0	fTotalStrokes	 13
U+86FB.0	fCangjie	LICRU
U+86FB.0	fFourCorner	58116
U+86FB.0	fHakka	 tui5;  toi5; tui3
U+86FB.0	fCantonese	teoi3; seoi3 (also)
U+86FB.0	fMandarin	tui4; shui4; yue4; shuo1; tuo4
U+86FB.0	fEnglish	[1] slough; exuviate [2] exuviate; slough
U+86FC.0	fUTF8	蛼
U+86FC.0	fCNS11643	3-4444
U+86FC.0	fGB	5-606C
U+86FC.0	fR/S	  142.7
U+86FC.0	fTotalStrokes	 13
U+86FC.0	fCangjie	LIJWJ
U+86FC.0	fFourCorner	55100
U+86FC.0	fCantonese	ce1
U+86FC.0	fMandarin	che1
U+86FD.0	fUTF8	蛽
U+86FD.0	fCNS11643	3-4445
U+86FD.0	fGB	E-3C31
U+86FD.0	fR/S	  142.7
U+86FD.0	fTotalStrokes	 13
U+86FD.0	fCangjie	LIBUC
U+86FD.0	fFourCorner	56180
U+86FD.0	fCantonese	bui3
U+86FD.0	fMandarin	bai4
U+86FE.0	fUTF8	蛾
U+86FE.0	fCNS11643	1-6646
U+86FE.0	fBig5	B8BF
U+86FE.0	fGB	0-366A
U+86FE.0	fR/S	 142.7
U+86FE.0	fTotalStrokes	 13
U+86FE.0	fCangjie	LIHQI
U+86FE.0	fFourCorner	53150
U+86FE.0	fHakka	  ngo2
U+86FE.0	fCantonese	ngo4; ngai5 (also); ai5 (also); o4
U+86FE.0	fMandarin	e2; yi3
U+86FE.0	fEnglish	 moth
U+86FF.0	fUTF8	蛿
U+86FF.0	fCNS11643	3-4449
U+86FF.0	fGB	5-6148
U+86FF.0	fR/S	  142.7
U+86FF.0	fTotalStrokes	 13
U+86FF.0	fCangjie	LIOIR
U+86FF.0	fFourCorner	28160
U+86FF.0	fCantonese	ham5
U+86FF.0	fMandarin	han4
U+8700.0	fUTF8	蜀
U+8700.0	fCNS11643	1-6645
U+8700.0	fBig5	B8BE
U+8700.0	fGB	0-4A71
U+8700.0	fR/S	 142.7
U+8700.0	fTotalStrokes	 13
U+8700.0	fCangjie	WLPLI
U+8700.0	fFourCorner	60127
U+8700.0	fHakka	 chuk7; zuk7; suk7; zhuk7
U+8700.0	fCantonese	suk6
U+8700.0	fMandarin	shu3
U+8700.0	fEnglish	Shu, name of an ancient state in what is Szechwan today
U+8701.0	fUTF8	蜁
U+8701.0	fCNS11643	2-4734
U+8701.0	fBig5	DFED
U+8701.0	fGB	E-3C32
U+8701.0	fR/S	 142.7
U+8701.0	fTotalStrokes	 13
U+8701.0	fCangjie	LIONO
U+8701.0	fFourCorner	58181
U+8701.0	fCantonese	syun4
U+8701.0	fMandarin	xuan2
U+8702.0	fUTF8	蜂
U+8702.0	fCNS11643	1-6648
U+8702.0	fBig5	B8C1
U+8702.0	fGB	0-3764
U+8702.0	fR/S	 142.7
U+8702.0	fTotalStrokes	 13
U+8702.0	fCangjie	LIHEJ
U+8702.0	fFourCorner	57154
U+8702.0	fHakka	  fung1;  pung1; bung1
U+8702.0	fCantonese	fung1
U+8702.0	fMandarin	feng1; peng2
U+8702.0	fEnglish	 bee; wasp; hornet
U+8703.0	fUTF8	蜃
U+8703.0	fCNS11643	1-6649
U+8703.0	fBig5	B8C2
U+8703.0	fGB	0-7257
U+8703.0	fR/S	 142.7
U+8703.0	fTotalStrokes	 13
U+8703.0	fCangjie	MVLMI
U+8703.0	fFourCorner	71136
U+8703.0	fHakka	 shin3; siin3; sin3; zhin3
U+8703.0	fCantonese	san6; san5 (also); san4 (also)
U+8703.0	fMandarin	shen4
U+8703.0	fEnglish	clams
U+8704.0	fUTF8	蜄
U+8704.0	fCNS11643	2-472A
U+8704.0	fBig5	DFE3
U+8704.0	fGB	5-6140
U+8704.0	fR/S	 142.7
U+8704.0	fTotalStrokes	 13
U+8704.0	fCangjie	LIMMV
U+8704.0	fFourCorner	51132
U+8704.0	fHakka	 zhin3
U+8704.0	fCantonese	san6; zan3
U+8704.0	fMandarin	zhen4; shen4; zhen3
U+8705.0	fUTF8	蜅
U+8705.0	fCNS11643	2-4737
U+8705.0	fBig5	DFF0
U+8705.0	fGB	3-6236
U+8705.0	fR/S	 142.7
U+8705.0	fTotalStrokes	 13
U+8705.0	fCangjie	LIIJB
U+8705.0	fFourCorner	53127
U+8705.0	fCantonese	fu2
U+8705.0	fMandarin	fu3; pu2
U+8706.0	fUTF8	蜆
U+8706.0	fCNS11643	1-664A
U+8706.0	fBig5	B8C3
U+8706.0	fGB	1-7239
U+8706.0	fR/S	 142.7
U+8706.0	fTotalStrokes	 13
U+8706.0	fCangjie	LIBUU
U+8706.0	fFourCorner	56110
U+8706.0	fHakka	 han3;  hien3;  gan5; hen3; han5
U+8706.0	fCantonese	hin2
U+8706.0	fMandarin	xian4; xian3; xian2
U+8706.0	fEnglish	Corbicula leana, a variety of bivalve
U+8707.0	fUTF8	蜇
U+8707.0	fCNS11643	1-6644
U+8707.0	fBig5	B8BD
U+8707.0	fGB	0-7258
U+8707.0	fR/S	 142.7
U+8707.0	fTotalStrokes	 13
U+8707.0	fCangjie	QLLMI
U+8707.0	fFourCorner	52136
U+8707.0	fHakka	 zet7; zhat7; zhet7; zat7
U+8707.0	fCantonese	zit3
U+8707.0	fMandarin	zhe2; zhe1
U+8707.0	fEnglish	[1] used in combination for jellyfish [2] sting
U+8708.0	fUTF8	蜈
U+8708.0	fCNS11643	1-6643
U+8708.0	fBig5	B8BC
U+8708.0	fGB	0-725A
U+8708.0	fR/S	 142.7
U+8708.0	fTotalStrokes	 13
U+8708.0	fCangjie	LIRVK
U+8708.0	fFourCorner	56134
U+8708.0	fHakka	  ng2; ngu2
U+8708.0	fCantonese	ng4
U+8708.0	fMandarin	wu2
U+8708.0	fEnglish	 centipede
U+8709.0	fUTF8	蜉
U+8709.0	fCNS11643	2-4733
U+8709.0	fBig5	DFEC
U+8709.0	fGB	0-725D
U+8709.0	fR/S	 142.7
U+8709.0	fTotalStrokes	 13
U+8709.0	fCangjie	LIBND
U+8709.0	fFourCorner	52147
U+8709.0	fHakka	  feu2; fiu2
U+8709.0	fCantonese	fau4
U+8709.0	fMandarin	fu2
U+8709.0	fEnglish	 mayfly; ephemera; ephemerid
U+870A.0	fUTF8	蜊
U+870A.0	fCNS11643	1-664B
U+870A.0	fBig5	B8C4
U+870A.0	fGB	0-725B
U+870A.0	fR/S	 142.7
U+870A.0	fTotalStrokes	 13
U+870A.0	fCangjie	LIHDN
U+870A.0	fFourCorner	52100
U+870A.0	fHakka	 li2; li1; li5
U+870A.0	fCantonese	lei4; lei6
U+870A.0	fMandarin	li2
U+870A.0	fEnglish	[1] used in combination for a bivalve [2] used in combination for a dragon-fly
U+870B.0	fUTF8	蜋
U+870B.0	fCNS11643	2-4729
U+870B.0	fBig5	DFE2
U+870B.0	fGB	E-3C33
U+870B.0	fR/S	 142.7
U+870B.0	fTotalStrokes	 13
U+870B.0	fCangjie	LIIAV
U+870B.0	fFourCorner	53132
U+870B.0	fHakka	 long2
U+870B.0	fCantonese	long4; loeng4
U+870B.0	fMandarin	lang2
U+870B.0	fEnglish	[1] used in combination for mantis [2] cicada
U+870B.1	fUTF8	蜋
U+870B.1	fCNS11643	2-4729
U+870B.1	fBig5	DFE2
U+870B.1	fGB	E-3C33
U+870B.1	fR/S	 142.7
U+870B.1	fTotalStrokes	 13
U+870B.1	fCangjie	LIIAV
U+870B.1	fFourCorner	53132
U+870B.1	fMandarin	liang2
U+870B.1	fEnglish	used in combination for the dung beetle (Geotropes laevistriatus)
U+870C.0	fUTF8	蜌
U+870C.0	fCNS11643	2-472C
U+870C.0	fBig5	DFE5
U+870C.0	fGB	3-6238
U+870C.0	fR/S	 142.7
U+870C.0	fTotalStrokes	 13
U+870C.0	fCangjie	LIPPG
U+870C.0	fFourCorner	51114
U+870C.0	fHakka	 pi6; pi5; pi3
U+870C.0	fCantonese	bai6
U+870C.0	fMandarin	bi4
U+870C.0	fEnglish	[1] various kinds of shell-fish [2] mussel
U+870D.0	fUTF8	蜍
U+870D.0	fCNS11643	2-4736
U+870D.0	fBig5	DFEF
U+870D.0	fGB	0-725C
U+870D.0	fR/S	 142.7
U+870D.0	fTotalStrokes	 13
U+870D.0	fCangjie	LIOMD
U+870D.0	fFourCorner	58194
U+870D.0	fHakka	 su2;  chu2; cu2; shu2; shu3
U+870D.0	fCantonese	ceoi4; cyu4 (also); syu4 (also); cyu1
U+870D.0	fMandarin	chu2; yu2; shu2; le4
U+870D.0	fEnglish	used in combination for toad
U+870E.0	fUTF8	蜎
U+870E.0	fCNS11643	2-4732
U+870E.0	fBig5	DFEB
U+870E.0	fGB	3-6239
U+870E.0	fR/S	 142.7
U+870E.0	fTotalStrokes	 13
U+870E.0	fCangjie	LIRB
U+870E.0	fFourCorner	56127
U+870E.0	fHakka	 gien1; gian1; gen1
U+870E.0	fCantonese	jyun1
U+870E.0	fMandarin	yuan1; juan4; xuan1; xuan3; xuan4; yuan3
U+870E.0	fEnglish	mosquito larva
U+870F.0	fUTF8	蜏
U+870F.0	fCNS11643	4-4059
U+870F.0	fGB	3-623A
U+870F.0	fR/S	  142.7
U+870F.0	fTotalStrokes	 13
U+870F.0	fCangjie	LIHDS
U+870F.0	fFourCorner	52120
U+870F.0	fHakka	 ju3; ju5; ru3; riu6
U+870F.0	fCantonese	jau5
U+870F.0	fMandarin	you3
U+870F.0	fEnglish	a short-lived fly, like the silk worms
U+8710.0	fUTF8	蜐
U+8710.0	fCNS11643	4-4052
U+8710.0	fGB	5-613B
U+8710.0	fR/S	  142.7
U+8710.0	fTotalStrokes	 13
U+8710.0	fCangjie	LIGIS
U+8710.0	fFourCorner	54120
U+8710.0	fHakka	 giap7
U+8710.0	fCantonese	gip3
U+8710.0	fMandarin	jie2
U+8710.0	fEnglish	the sea-anemone
U+8711.0	fUTF8	蜑
U+8711.0	fCNS11643	2-4D7B
U+8711.0	fBig5	E3F4
U+8711.0	fGB	5-6133
U+8711.0	fR/S	 142.7
U+8711.0	fTotalStrokes	 13
U+8711.0	fCangjie	NMLMI
U+8711.0	fFourCorner	11136
U+8711.0	fHakka	 tan5; tan6
U+8711.0	fCantonese	daan6
U+8711.0	fMandarin	dan4
U+8711.0	fEnglish	the boat people in the coastal areas of Fukien and Kwangtung
U+8712.0	fUTF8	蜒
U+8712.0	fCNS11643	2-4D70
U+8712.0	fBig5	E3E9
U+8712.0	fGB	0-5151
U+8712.0	fR/S	 142.7
U+8712.0	fTotalStrokes	 13
U+8712.0	fCangjie	LINKM
U+8712.0	fFourCorner	52141
U+8712.0	fHakka	 jen2; tan6; tan5; ren2
U+8712.0	fCantonese	jin4
U+8712.0	fMandarin	yan2; dan4; yan4; yan3
U+8712.0	fEnglish	[1] used in combination for millipede [2] used in combination for stretch for miles
U+8713.0	fUTF8	蜓
U+8713.0	fCNS11643	1-6642
U+8713.0	fBig5	B8BB
U+8713.0	fGB	0-7251
U+8713.0	fR/S	 142.7
U+8713.0	fTotalStrokes	 13
U+8713.0	fCangjie	LINKG
U+8713.0	fFourCorner	52131
U+8713.0	fHakka	  tin2; tien5; tin3
U+8713.0	fCantonese	ting4
U+8713.0	fMandarin	ting2; tian3; dian4; ting3
U+8713.0	fEnglish	used in combination for dragonfly
U+8714.0	fUTF8	蜔
U+8714.0	fCNS11643	5-425D
U+8714.0	fGB	E-3C34
U+8714.0	fR/S	  142.7
U+8714.0	fTotalStrokes	 13
U+8714.0	fCangjie	LIPW
U+8714.0	fFourCorner	57120
U+8714.0	fCantonese	tin4; din6
U+8714.0	fMandarin	dian4
U+8714.0	fEnglish	inlaid shell-work
U+8715.0	fUTF8	蜕
U+8715.0	fCNS11643	3-4446
U+8715.0	fGB	0-4D49
U+8715.0	fR/S	 142.7
U+8715.0	fTotalStrokes	 13
U+8715.0	fCangjie	LICRU
U+8715.0	fFourCorner	58110
U+8715.0	fCantonese	teoi3; seoi3 (also)
U+8715.0	fMandarin	tui4; shui4
U+8715.0	fEnglish	molt
U+8716.0	fUTF8	蜖
U+8716.0	fCNS11643	3-4448
U+8716.0	fGB	E-3C35
U+8716.0	fR/S	 142.7
U+8716.0	fTotalStrokes	 13
U+8716.0	fCangjie	LIWSL
U+8716.0	fFourCorner	56100
U+8716.0	fCantonese	wui4
U+8716.0	fMandarin	hui2
U+8716.0	fEnglish	tapeworm
U+8717.0	fUTF8	蜗
U+8717.0	fGB	0-4E4F
U+8717.0	fR/S	 142.7
U+8717.0	fTotalStrokes	 13
U+8717.0	fCangjie	LIROB
U+8717.0	fHakka	 vo1
U+8717.0	fCantonese	wo1; waa1
U+8717.0	fMandarin	gua1; wo1; ge1; luo2
U+8717.0	fEnglish	simplified form of &#x8778;
U+8718.0	fUTF8	蜘
U+8718.0	fCNS11643	1-6A72
U+8718.0	fBig5	BB6A
U+8718.0	fGB	0-5629
U+8718.0	fR/S	 142.8
U+8718.0	fTotalStrokes	 14
U+8718.0	fCangjie	LIOKR
U+8718.0	fFourCorner	56100
U+8718.0	fHakka	 di1; zhi1; zih1; ci1; tje1
U+8718.0	fCantonese	zi1
U+8718.0	fMandarin	zhi1
U+8718.0	fEnglish	 spider
U+8719.0	fUTF8	蜙
U+8719.0	fCNS11643	2-4D64
U+8719.0	fBig5	E3DD
U+8719.0	fGB	5-6154
U+8719.0	fR/S	 142.8
U+8719.0	fTotalStrokes	 14
U+8719.0	fCangjie	LIDCI
U+8719.0	fFourCorner	58132
U+8719.0	fCantonese	sung1
U+8719.0	fMandarin	song1
U+871A.0	fUTF8	蜚
U+871A.0	fCNS11643	2-4D79
U+871A.0	fBig5	E3F2
U+871A.0	fGB	0-7263
U+871A.0	fR/S	 142.8
U+871A.0	fTotalStrokes	 14
U+871A.0	fCangjie	LYLMI
U+871A.0	fFourCorner	11136
U+871A.0	fHakka	  fui1; fi1; fui3; fi3
U+871A.0	fCantonese	fei2; fei1; fai6
U+871A.0	fMandarin	fei3; fei4; pei4; bei4
U+871A.0	fEnglish	 cockroach
U+871A.1	fUTF8	蜚
U+871A.1	fCNS11643	2-4D79
U+871A.1	fBig5	E3F2
U+871A.1	fGB	0-7263
U+871A.1	fR/S	 142.8
U+871A.1	fTotalStrokes	 14
U+871A.1	fCangjie	LYLMI
U+871A.1	fFourCorner	11136
U+871A.1	fMandarin	fei1
U+871A.1	fEnglish	 fly
U+871B.0	fUTF8	蜛
U+871B.0	fCNS11643	2-4D65
U+871B.0	fBig5	E3DE
U+871B.0	fGB	E-3C36
U+871B.0	fR/S	 142.8
U+871B.0	fTotalStrokes	 14
U+871B.0	fCangjie	LISJR
U+871B.0	fFourCorner	57164
U+871B.0	fCantonese	geoi1
U+871B.0	fMandarin	ju1; chuan3
U+871C.0	fUTF8	蜜
U+871C.0	fCNS11643	1-6A6D
U+871C.0	fBig5	BB65
U+871C.0	fGB	0-435B
U+871C.0	fR/S	 142.8
U+871C.0	fTotalStrokes	 14
U+871C.0	fCangjie	JPHI
U+871C.0	fFourCorner	30136
U+871C.0	fHakka	  mit8;  met8
U+871C.0	fCantonese	mat6
U+871C.0	fMandarin	mi4
U+871C.0	fEnglish	[1] honey; nectar (flower) [2] sweet; syrupy; honeyed
U+871D.0	fUTF8	蜝
U+871D.0	fCNS11643	3-494F
U+871D.0	fGB	3-623D
U+871D.0	fR/S	  142.8
U+871D.0	fTotalStrokes	 14
U+871D.0	fCangjie	TCLMI
U+871D.0	fFourCorner	44130
U+871D.0	fHakka	 ki2
U+871D.0	fCantonese	kei4
U+871D.0	fMandarin	qi2
U+871E.0	fUTF8	蜞
U+871E.0	fCNS11643	2-4D62
U+871E.0	fBig5	E3DB
U+871E.0	fGB	0-7260
U+871E.0	fR/S	 142.8
U+871E.0	fTotalStrokes	 14
U+871E.0	fCangjie	LITMC
U+871E.0	fFourCorner	54181
U+871E.0	fHakka	 ki2; ki3
U+871E.0	fCantonese	kei4; kei2 (also)
U+871E.0	fMandarin	qi2
U+871E.0	fEnglish	a small land crab found in rice fields
U+871F.0	fUTF8	蜟
U+871F.0	fCNS11643	4-4055
U+871F.0	fGB	3-6247
U+871F.0	fR/S	 142.8; 142.7
U+871F.0	fTotalStrokes	 14; 13
U+871F.0	fCangjie	LIYIB
U+871F.0	fFourCorner	50120
U+871F.0	fHakka	 juk7; ruk7
U+871F.0	fCantonese	juk6
U+871F.0	fMandarin	yu4
U+871F.0	fEnglish	the larvae of cicada
U+8720.0	fUTF8	蜠
U+8720.0	fCNS11643	2-4D6B
U+8720.0	fBig5	E3E4
U+8720.0	fGB	3-623F
U+8720.0	fR/S	 142.8
U+8720.0	fTotalStrokes	 14
U+8720.0	fCangjie	LIWHD
U+8720.0	fFourCorner	56100
U+8720.0	fCantonese	kwan5
U+8720.0	fMandarin	jun3; qun1
U+8721.0	fUTF8	蜡
U+8721.0	fCNS11643	2-4D63
U+8721.0	fBig5	E3DC
U+8721.0	fGB	0-402F
U+8721.0	fR/S	 142.8
U+8721.0	fTotalStrokes	 14
U+8721.0	fCangjie	LITA
U+8721.0	fFourCorner	54161
U+8721.0	fHakka	  za5; lap8
U+8721.0	fCantonese	zaa3; laap6; caa3; zaa6; cai3; caai3
U+8721.0	fMandarin	la4; zha4; cha4; qu4; ji2
U+8721.0	fEnglish	[1] year-end sacrifice of the Chou dynasty [2] simplified form of &#x881F;
U+8722.0	fUTF8	蜢
U+8722.0	fCNS11643	1-6A6F
U+8722.0	fBig5	BB67
U+8722.0	fGB	0-726C
U+8722.0	fR/S	 142.8
U+8722.0	fTotalStrokes	 14
U+8722.0	fCangjie	LINDT
U+8722.0	fFourCorner	57117
U+8722.0	fHakka	  mang3; mang1
U+8722.0	fCantonese	maang5; maang2 (also)
U+8722.0	fMandarin	meng3; meng4
U+8722.0	fEnglish	used in combination for grasshopper
U+8723.0	fUTF8	蜣
U+8723.0	fCNS11643	2-4D5D
U+8723.0	fBig5	E3D6
U+8723.0	fGB	0-725E
U+8723.0	fR/S	 142.8
U+8723.0	fTotalStrokes	 14
U+8723.0	fCangjie	LITGU
U+8723.0	fFourCorner	58111
U+8723.0	fHakka	  giong1
U+8723.0	fCantonese	goeng1
U+8723.0	fMandarin	qiang1
U+8723.0	fEnglish	 dung beetle
U+8724.0	fUTF8	蜤
U+8724.0	fCNS11643	2-4D78
U+8724.0	fBig5	E3F1
U+8724.0	fGB	5-6153
U+8724.0	fR/S	 142.8
U+8724.0	fTotalStrokes	 14
U+8724.0	fCangjie	DLLMI
U+8724.0	fFourCorner	42136
U+8724.0	fCantonese	si1; sik1
U+8724.0	fMandarin	si1
U+8725.0	fUTF8	蜥
U+8725.0	fCNS11643	1-6A70
U+8725.0	fBig5	BB68
U+8725.0	fGB	0-7261
U+8725.0	fR/S	 142.8
U+8725.0	fTotalStrokes	 14
U+8725.0	fCangjie	LIDHL
U+8725.0	fFourCorner	52121
U+8725.0	fHakka	 sit7
U+8725.0	fCantonese	sik1
U+8725.0	fMandarin	xi1
U+8725.0	fEnglish	 lizard
U+8726.0	fUTF8	蜦
U+8726.0	fCNS11643	2-4D75
U+8726.0	fBig5	E3EE
U+8726.0	fGB	3-6161
U+8726.0	fR/S	 142.8
U+8726.0	fTotalStrokes	 14
U+8726.0	fCangjie	LIOMB
U+8726.0	fFourCorner	58127
U+8726.0	fHakka	  lun2; lun3
U+8726.0	fCantonese	leon4
U+8726.0	fMandarin	lun2; li4; lun3; lun4
U+8726.0	fEnglish	[1] crawl as a snake [2] wriggle [3] a snake-eating frog
U+8727.0	fUTF8	蜧
U+8727.0	fCNS11643	2-4D76
U+8727.0	fBig5	E3EF
U+8727.0	fGB	5-6160
U+8727.0	fR/S	 142.8
U+8727.0	fTotalStrokes	 14
U+8727.0	fCangjie	LIHSK
U+8727.0	fFourCorner	53143
U+8727.0	fHakka	 lui5; lui6
U+8727.0	fCantonese	lai6
U+8727.0	fMandarin	li4
U+8727.0	fEnglish	water sprout, believed to be a black dragon-snake
U+8728.0	fUTF8	蜨
U+8728.0	fCNS11643	2-4D5E
U+8728.0	fBig5	E3D7
U+8728.0	fGB	E-3C37
U+8728.0	fR/S	 142.8
U+8728.0	fTotalStrokes	 14
U+8728.0	fCangjie	LIJLO
U+8728.0	fFourCorner	55181
U+8728.0	fHakka	 tiap8; ciap8; tiap7
U+8728.0	fCantonese	dip6
U+8728.0	fMandarin	die2; xie4
U+8728.0	fEnglish	 butterfly
U+8729.0	fUTF8	蜩
U+8729.0	fCNS11643	1-6A75
U+8729.0	fBig5	BB6D
U+8729.0	fGB	0-7268
U+8729.0	fR/S	 142.8
U+8729.0	fTotalStrokes	 14
U+8729.0	fCangjie	LIBGR
U+8729.0	fFourCorner	57120
U+8729.0	fHakka	 tiau2
U+8729.0	fCantonese	tiu4
U+8729.0	fMandarin	tiao2; diao4; chou2
U+8729.0	fEnglish	cicada
U+872A.0	fUTF8	蜪
U+872A.0	fCNS11643	2-4D6D
U+872A.0	fBig5	E3E6
U+872A.0	fGB	3-6244
U+872A.0	fR/S	 142.8
U+872A.0	fTotalStrokes	 14
U+872A.0	fCangjie	LIPOU
U+872A.0	fFourCorner	57120
U+872A.0	fCantonese	tou1
U+872A.0	fMandarin	tao1; tao2
U+872B.0	fUTF8	蜫
U+872B.0	fCNS11643	3-494A
U+872B.0	fGB	5-6158
U+872B.0	fR/S	 142.7; 142.8
U+872B.0	fTotalStrokes	 14; 13
U+872B.0	fCangjie	LIAPP
U+872B.0	fFourCorner	56110
U+872B.0	fCantonese	gwan1
U+872B.0	fMandarin	kun1
U+872B.0	fEnglish	insects
U+872C.0	fUTF8	蜬
U+872C.0	fCNS11643	2-4D67
U+872C.0	fBig5	E3E0
U+872C.0	fGB	5-6166
U+872C.0	fR/S	 142.8
U+872C.0	fTotalStrokes	 14
U+872C.0	fCangjie	LIUNE
U+872C.0	fFourCorner	57172
U+872C.0	fCantonese	ham4
U+872C.0	fMandarin	gan1; han2
U+872D.0	fUTF8	蜭
U+872D.0	fCNS11643	2-4D6E
U+872D.0	fBig5	E3E7
U+872D.0	fGB	E-3C38
U+872D.0	fR/S	 142.8
U+872D.0	fTotalStrokes	 14
U+872D.0	fCangjie	LINHX
U+872D.0	fFourCorner	57177
U+872D.0	fCantonese	ham5
U+872D.0	fMandarin	han4; biao3
U+872E.0	fUTF8	蜮
U+872E.0	fCNS11643	2-4D61
U+872E.0	fBig5	E3DA
U+872E.0	fGB	0-7262
U+872E.0	fR/S	 142.8
U+872E.0	fTotalStrokes	 14
U+872E.0	fCangjie	LIIRM
U+872E.0	fFourCorner	53150
U+872E.0	fHakka	 wet7; vet7
U+872E.0	fCantonese	wik6; waak6
U+872E.0	fMandarin	yu4; guo1
U+872E.0	fEnglish	a fabulous tortoise-like creature, the sand cast out of whose mouth is believed deadly to humans
U+872F.0	fUTF8	蜯
U+872F.0	fCNS11643	4-467E
U+872F.0	fGB	5-614F
U+872F.0	fR/S	  142.8
U+872F.0	fTotalStrokes	 14
U+872F.0	fCangjie	LIQKQ
U+872F.0	fFourCorner	55150
U+872F.0	fCantonese	pong5
U+872F.0	fMandarin	beng4; bang4
U+8730.0	fUTF8	蜰
U+8730.0	fCNS11643	2-4D7A
U+8730.0	fBig5	E3F3
U+8730.0	fGB	3-6243
U+8730.0	fR/S	 142.8
U+8730.0	fTotalStrokes	 14
U+8730.0	fCangjie	BULMI
U+8730.0	fFourCorner	77136
U+8730.0	fHakka	 pui2; pi2; fi2; fui2; pi3; pui3
U+8730.0	fCantonese	fei4; fai6; fei2
U+8730.0	fMandarin	fei2; chan1; fei3; fei4
U+8730.0	fEnglish	[1] grub [2] a kind of snake
U+8731.0	fUTF8	蜱
U+8731.0	fCNS11643	2-4D72
U+8731.0	fBig5	E3EB
U+8731.0	fGB	0-7267
U+8731.0	fR/S	 142.8
U+8731.0	fTotalStrokes	 14
U+8731.0	fCangjie	LIHWJ
U+8731.0	fFourCorner	56140
U+8731.0	fHakka	  bi1
U+8731.0	fCantonese	pei4
U+8731.0	fMandarin	pi2; miao2; bei1; piao2
U+8731.0	fEnglish	a tick, mite
U+8732.0	fUTF8	蜲
U+8732.0	fCNS11643	2-4D6C
U+8732.0	fBig5	E3E5
U+8732.0	fGB	5-615A
U+8732.0	fR/S	 142.8
U+8732.0	fTotalStrokes	 14
U+8732.0	fCangjie	LIHDV
U+8732.0	fFourCorner	52144
U+8732.0	fHakka	 wui3; wi3; wui1; vui3
U+8732.0	fCantonese	wai1; wai2
U+8732.0	fMandarin	wei3; wei1
U+8732.0	fEnglish	the wood-louse
U+8733.0	fUTF8	蜳
U+8733.0	fCNS11643	2-4D5C
U+8733.0	fBig5	E3D5
U+8733.0	fGB	5-615E
U+8733.0	fR/S	 142.8
U+8733.0	fTotalStrokes	 14
U+8733.0	fCangjie	LIYRD
U+8733.0	fFourCorner	50147
U+8733.0	fHakka	 dun1
U+8733.0	fCantonese	deon1; tan1
U+8733.0	fMandarin	dun1; yun3
U+8734.0	fUTF8	蜴
U+8734.0	fCNS11643	1-6A71
U+8734.0	fBig5	BB69
U+8734.0	fGB	0-7266
U+8734.0	fR/S	 142.8
U+8734.0	fTotalStrokes	 14
U+8734.0	fCangjie	LIAPH
U+8734.0	fFourCorner	56127
U+8734.0	fHakka	 jit8; rit8
U+8734.0	fCantonese	jik6
U+8734.0	fMandarin	yi4; xi2
U+8734.0	fEnglish	lizard
U+8735.0	fUTF8	蜵
U+8735.0	fCNS11643	2-4D73
U+8735.0	fBig5	E3EC
U+8735.0	fGB	3-6253
U+8735.0	fR/S	 142.8
U+8735.0	fTotalStrokes	 14
U+8735.0	fCangjie	LILXL
U+8735.0	fFourCorner	52100
U+8735.0	fCantonese	jyun1
U+8735.0	fMandarin	yuan1; yun1
U+8736.0	fUTF8	蜶
U+8736.0	fCNS11643	3-4945
U+8736.0	fGB	E-3C39
U+8736.0	fR/S	  142.8
U+8736.0	fTotalStrokes	 14
U+8736.0	fCangjie	LIYOJ
U+8736.0	fFourCorner	50140
U+8736.0	fCantonese	seot1; zeot1
U+8736.0	fMandarin	su4
U+8736.0	fEnglish	a house cricket
U+8737.0	fUTF8	蜷
U+8737.0	fCNS11643	1-6A74
U+8737.0	fBig5	BB6C
U+8737.0	fGB	0-7269
U+8737.0	fR/S	 142.8
U+8737.0	fTotalStrokes	 14
U+8737.0	fCangjie	LIFQU
U+8737.0	fFourCorner	59112
U+8737.0	fHakka	 kien2; ken2
U+8737.0	fCantonese	kyun4
U+8737.0	fMandarin	quan2; juan3
U+8737.0	fEnglish	 wriggle; be coiled; be curled up
U+8738.0	fUTF8	蜸
U+8738.0	fCNS11643	2-4D77
U+8738.0	fBig5	E3F0
U+8738.0	fGB	5-606D
U+8738.0	fR/S	 142.8
U+8738.0	fTotalStrokes	 14
U+8738.0	fCangjie	SELMI
U+8738.0	fFourCorner	77136
U+8738.0	fCantonese	hin2
U+8738.0	fMandarin	qian3
U+8739.0	fUTF8	蜹
U+8739.0	fCNS11643	3-494C
U+8739.0	fGB	3-6234
U+8739.0	fR/S	  142.8
U+8739.0	fTotalStrokes	 14
U+8739.0	fCangjie	LITOB
U+8739.0	fFourCorner	54120
U+8739.0	fHakka	 nui5; rui3; jui3; nui6
U+8739.0	fCantonese	jeoi6; jeoi3
U+8739.0	fMandarin	rui4
U+8739.0	fEnglish	gnat; mosquito
U+873A.0	fUTF8	蜺
U+873A.0	fCNS11643	2-4D71
U+873A.0	fBig5	E3EA
U+873A.0	fGB	E-3C3A
U+873A.0	fR/S	 142.8
U+873A.0	fTotalStrokes	 14
U+873A.0	fCangjie	LIHXU
U+873A.0	fFourCorner	57117
U+873A.0	fHakka	  ngi2
U+873A.0	fCantonese	ngai4; ai4 (also)
U+873A.0	fMandarin	ni2; nie4
U+873A.0	fEnglish	 reflection of a rainbow
U+873B.0	fUTF8	蜻
U+873B.0	fCNS11643	1-6A6E
U+873B.0	fBig5	BB66
U+873B.0	fGB	0-725F
U+873B.0	fR/S	 142.8
U+873B.0	fTotalStrokes	 14
U+873B.0	fCangjie	LIQMB
U+873B.0	fFourCorner	55127
U+873B.0	fHakka	 cin1; zin1; ciang1; ziang1
U+873B.0	fCantonese	cing1; zing1
U+873B.0	fMandarin	qing1; jing1; jing4; qing3
U+873B.0	fEnglish	 dragonfly
U+873C.0	fUTF8	蜼
U+873C.0	fCNS11643	2-4D6F
U+873C.0	fBig5	E3E8
U+873C.0	fGB	3-6241
U+873C.0	fR/S	 142.8
U+873C.0	fTotalStrokes	 14
U+873C.0	fCangjie	LIOG
U+873C.0	fFourCorner	50114
U+873C.0	fHakka	 wi3; wui3; wui5; wi5; vui3; vui5
U+873C.0	fCantonese	wai2; wai6; leoi5
U+873C.0	fMandarin	wei4; wei3; hui4; lei3; you4
U+873C.0	fEnglish	 long-tailed monkey
U+873D.0	fUTF8	蜽
U+873D.0	fCNS11643	3-4949
U+873D.0	fGB	5-613E
U+873D.0	fR/S	  142.8
U+873D.0	fTotalStrokes	 14
U+873D.0	fCangjie	LIMLB
U+873D.0	fFourCorner	51120
U+873D.0	fCantonese	loeng5
U+873D.0	fMandarin	liang3
U+873E.0	fUTF8	蜾
U+873E.0	fCNS11643	2-4D69
U+873E.0	fBig5	E3E2
U+873E.0	fGB	0-7264
U+873E.0	fR/S	 142.8
U+873E.0	fTotalStrokes	 14
U+873E.0	fCangjie	LIWD
U+873E.0	fFourCorner	56194
U+873E.0	fHakka	 go3; kwo3
U+873E.0	fCantonese	gwo2; go2 (also); gwok2
U+873E.0	fMandarin	guo3; luo3; guo1
U+873E.0	fEnglish	a kind of solitary wasp; the sphex
U+873F.0	fUTF8	蜿
U+873F.0	fCNS11643	1-6A6C
U+873F.0	fBig5	BB64
U+873F.0	fGB	0-726A
U+873F.0	fR/S	 142.8
U+873F.0	fTotalStrokes	 14
U+873F.0	fCangjie	LIJNU
U+873F.0	fFourCorner	53112
U+873F.0	fHakka	 ren1; jen1; jen2; van1
U+873F.0	fCantonese	jyun1; jyun2
U+873F.0	fMandarin	wan1; wan3; yuan1; yuan3
U+873F.0	fEnglish	 creep; crawl; wind up; wriggle
U+8740.0	fUTF8	蝀
U+8740.0	fCNS11643	2-4D60
U+8740.0	fBig5	E3D9
U+8740.0	fGB	3-616C
U+8740.0	fR/S	 142.8
U+8740.0	fTotalStrokes	 14
U+8740.0	fCangjie	LIDW
U+8740.0	fFourCorner	55196
U+8740.0	fHakka	  dung5;  dung1
U+8740.0	fCantonese	dung1; dung3 (also); dung2
U+8740.0	fMandarin	dong1; dong4; dong3
U+8740.0	fEnglish	used in combination for rainbow
U+8741.0	fUTF8	蝁
U+8741.0	fCNS11643	2-4D68
U+8741.0	fBig5	E3E1
U+8741.0	fGB	E-3C3B
U+8741.0	fR/S	 142.8
U+8741.0	fTotalStrokes	 14
U+8741.0	fCangjie	MMLMI
U+8741.0	fFourCorner	10136
U+8741.0	fCantonese	ok3; ngok3 (also)
U+8741.0	fMandarin	e4; wu4
U+8742.0	fUTF8	蝂
U+8742.0	fCNS11643	2-4D74
U+8742.0	fBig5	E3ED
U+8742.0	fGB	5-615C
U+8742.0	fR/S	 142.8
U+8742.0	fTotalStrokes	 14
U+8742.0	fCangjie	LILLE
U+8742.0	fFourCorner	52147
U+8742.0	fCantonese	baan2
U+8742.0	fMandarin	ban3; ban1
U+8742.0	fEnglish	used in combination for a kind of small insect which can move things many times its weight
U+8743.0	fUTF8	蝃
U+8743.0	fCNS11643	2-4D66
U+8743.0	fBig5	E3DF
U+8743.0	fGB	5-6164
U+8743.0	fR/S	 142.8
U+8743.0	fTotalStrokes	 14
U+8743.0	fCangjie	LIEEE
U+8743.0	fFourCorner	57147
U+8743.0	fHakka	 dot7; di5
U+8743.0	fCantonese	dai3; zyut3
U+8743.0	fMandarin	di4; zhuo2; duo2
U+8743.0	fEnglish	rainbow
U+8744.0	fUTF8	蝄
U+8744.0	fCNS11643	F-4C33
U+8744.0	fGB	5-6159
U+8744.0	fR/S	 142.8
U+8744.0	fTotalStrokes	 14
U+8744.0	fCangjie	LIBTV
U+8744.0	fFourCorner	5712
U+8744.0	fCantonese	mong5
U+8744.0	fMandarin	wang3
U+8744.0	fEnglish	a kind of monster
U+8745.0	fUTF8	蝅
U+8745.0	fCNS11643	3-494E
U+8745.0	fGB	E-3C3C
U+8745.0	fR/S	 142.8
U+8745.0	fTotalStrokes	 14
U+8745.0	fCangjie	MKLMI
U+8745.0	fFourCorner	11130
U+8745.0	fCantonese	caam4
U+8745.0	fMandarin	can2
U+8746.0	fUTF8	蝆
U+8746.0	fCNS11643	2-4D6A
U+8746.0	fBig5	E3E3
U+8746.0	fGB	E-3C3D
U+8746.0	fR/S	 142.7
U+8746.0	fTotalStrokes	 13
U+8746.0	fCangjie	XLITQ
U+8746.0	fFourCorner	51150
U+8746.0	fCantonese	me1; mai5; joeng4
U+8746.0	fMandarin	yang3; mi3; yang2
U+8747.0	fUTF8	蝇
U+8747.0	fCNS11643	3-494B
U+8747.0	fGB	0-532C
U+8747.0	fR/S	 142.8
U+8747.0	fTotalStrokes	 14
U+8747.0	fCangjie	LIRLU
U+8747.0	fFourCorner	56110
U+8747.0	fHakka	 shin2; {jin2}
U+8747.0	fCantonese	jing4; jing1 (also)
U+8747.0	fMandarin	ying2
U+8747.0	fEnglish	simplified form of &#x8805;
U+8748.0	fUTF8	蝈
U+8748.0	fGB	0-7265
U+8748.0	fR/S	 142.8
U+8748.0	fTotalStrokes	 14
U+8748.0	fCangjie	LIWMI
U+8748.0	fHakka	 kwet7
U+8748.0	fCantonese	gwok3; gok3 (also)
U+8748.0	fMandarin	guo1
U+8748.0	fEnglish	simplified form of &#x87C8;
U+8749.0	fUTF8	蝉
U+8749.0	fGB	0-3275
U+8749.0	fR/S	 142.8; 142.9
U+8749.0	fTotalStrokes	 14
U+8749.0	fCangjie	LICWJ
U+8749.0	fHakka	 sham2
U+8749.0	fCantonese	sim4; sin4 (also); sim2 (also)
U+8749.0	fMandarin	chan2; qian2; shan4; ti2
U+8749.0	fEnglish	simplified form of &#x87EC;
U+874A.0	fUTF8	蝊
U+874A.0	fCNS11643	3-6643
U+874A.0	fGB	E-3C3E
U+874A.0	fR/S	 142.8
U+874A.0	fTotalStrokes	 14
U+874A.0	fCangjie	LIJMO
U+874A.0	fFourCorner	5318
U+874A.0	fCantonese	ding6
U+874B.0	fUTF8	蝋
U+874B.0	fGB	E-3C3F
U+874B.0	fR/S	 142.8
U+874B.0	fTotalStrokes	 14
U+874B.0	fCangjie	LIFHN
U+874B.0	fCantonese	laap6
U+874B.0	fMandarin	la4; zha4
U+874B.0	fEnglish	 wax candle waxy; glazed
U+874C.0	fUTF8	蝌
U+874C.0	fCNS11643	1-6E6A
U+874C.0	fBig5	BDC1
U+874C.0	fGB	0-7272
U+874C.0	fR/S	 142.9
U+874C.0	fTotalStrokes	 15
U+874C.0	fCangjie	LIHDJ
U+874C.0	fFourCorner	54100
U+874C.0	fHakka	 ko1
U+874C.0	fCantonese	fo1
U+874C.0	fMandarin	ke1
U+874C.0	fEnglish	 tadpole
U+874D.0	fUTF8	蝍
U+874D.0	fCNS11643	2-4730
U+874D.0	fBig5	DFE9
U+874D.0	fGB	3-623C
U+874D.0	fR/S	 142.9
U+874D.0	fTotalStrokes	 15
U+874D.0	fCangjie	LIAIL
U+874D.0	fFourCorner	57120
U+874D.0	fHakka	  zit7
U+874D.0	fCantonese	zik1
U+874D.0	fMandarin	ji2
U+874D.0	fEnglish	[1] cricket [2] centipede [3] dragonfly
U+874E.0	fUTF8	蝎
U+874E.0	fCNS11643	2-5379
U+874E.0	fBig5	E7B2
U+874E.0	fGB	0-502B
U+874E.0	fR/S	 142.9
U+874E.0	fTotalStrokes	 15
U+874E.0	fCangjie	LIAPV
U+874E.0	fFourCorner	56127
U+874E.0	fHakka	 hot7;  giet7; ket7; het7; hiet7; get7
U+874E.0	fCantonese	hot3; hit3; kit3 (also); hot6
U+874E.0	fMandarin	he2; xie1; hong2; xie5
U+874E.0	fEnglish	 scorpion
U+874F.0	fUTF8	蝏
U+874F.0	fCNS11643	2-5424
U+874F.0	fBig5	E7BB
U+874F.0	fGB	3-6259
U+874F.0	fR/S	 142.9
U+874F.0	fTotalStrokes	 15
U+874F.0	fCangjie	LIYRN
U+874F.0	fFourCorner	50121
U+874F.0	fCantonese	ting4
U+874F.0	fMandarin	ting2
U+8750.0	fUTF8	蝐
U+8750.0	fCNS11643	2-5378
U+8750.0	fBig5	E7B1
U+8750.0	fGB	5-616A
U+8750.0	fR/S	 142.9
U+8750.0	fTotalStrokes	 15
U+8750.0	fCangjie	LIABU
U+8750.0	fFourCorner	56160
U+8750.0	fCantonese	mou6; mui6
U+8750.0	fMandarin	mai4; mei4; mao4
U+8751.0	fUTF8	蝑
U+8751.0	fCNS11643	2-5374
U+8751.0	fBig5	E7AD
U+8751.0	fGB	3-625C
U+8751.0	fR/S	 142.9
U+8751.0	fTotalStrokes	 15
U+8751.0	fCangjie	LINOB
U+8751.0	fFourCorner	57122
U+8751.0	fCantonese	seoi1
U+8751.0	fMandarin	xu1; xie4; xu3
U+8752.0	fUTF8	蝒
U+8752.0	fCNS11643	2-5371
U+8752.0	fBig5	E7AA
U+8752.0	fGB	3-624F
U+8752.0	fR/S	 142.9
U+8752.0	fTotalStrokes	 15
U+8752.0	fCangjie	LIMWL
U+8752.0	fFourCorner	51160
U+8752.0	fCantonese	min4
U+8752.0	fMandarin	mian2
U+8753.0	fUTF8	蝓
U+8753.0	fCNS11643	1-6E6B
U+8753.0	fBig5	BDC2
U+8753.0	fGB	0-7275
U+8753.0	fR/S	 142.9
U+8753.0	fTotalStrokes	 15
U+8753.0	fCangjie	LIOMN
U+8753.0	fFourCorner	58121
U+8753.0	fHakka	 ji2; ri2; ze2; je2
U+8753.0	fCantonese	jyu4
U+8753.0	fMandarin	yu2; you2
U+8753.0	fEnglish	snail
U+8754.0	fUTF8	蝔
U+8754.0	fCNS11643	2-536F
U+8754.0	fBig5	E7A8
U+8754.0	fGB	E-3C40
U+8754.0	fR/S	 142.9
U+8754.0	fTotalStrokes	 15
U+8754.0	fCangjie	LIPPA
U+8754.0	fFourCorner	51161
U+8754.0	fCantonese	gaai1
U+8754.0	fMandarin	jie1; xie2
U+8755.0	fUTF8	蝕
U+8755.0	fCNS11643	1-6A73
U+8755.0	fBig5	BB6B
U+8755.0	fGB	1-4A34
U+8755.0	fR/S	 142.9
U+8755.0	fTotalStrokes	 15
U+8755.0	fCangjie	OILMI
U+8755.0	fFourCorner	85736
U+8755.0	fHakka	 sit8; s'at8; set8; shit8; siet8; siit8; sat8; shat8; s'et8
U+8755.0	fCantonese	sik6; sit6
U+8755.0	fMandarin	shi2; li4; long2
U+8755.0	fEnglish	[1] nibble away; erode; eat slowly [2] eclipse
U+8756.0	fUTF8	蝖
U+8756.0	fCNS11643	2-5368
U+8756.0	fBig5	E7A1
U+8756.0	fGB	3-625B
U+8756.0	fR/S	 142.9
U+8756.0	fTotalStrokes	 15
U+8756.0	fCangjie	LIJMM
U+8756.0	fFourCorner	53116
U+8756.0	fCantonese	hyun1
U+8756.0	fMandarin	xuan1
U+8757.0	fUTF8	蝗
U+8757.0	fCNS11643	1-6E69
U+8757.0	fBig5	BDC0
U+8757.0	fGB	0-3B48
U+8757.0	fR/S	 142.9
U+8757.0	fTotalStrokes	 15
U+8757.0	fCangjie	LIHAG
U+8757.0	fFourCorner	56114
U+8757.0	fHakka	 fong2; wong2
U+8757.0	fCantonese	wong4
U+8757.0	fMandarin	huang2
U+8757.0	fEnglish	 locust
U+8758.0	fUTF8	蝘
U+8758.0	fCNS11643	2-536E
U+8758.0	fBig5	E7A7
U+8758.0	fGB	3-624D
U+8758.0	fR/S	 142.9
U+8758.0	fTotalStrokes	 15
U+8758.0	fCangjie	LISAV
U+8758.0	fFourCorner	51114
U+8758.0	fHakka	 jen3; ren3; jan3
U+8758.0	fCantonese	jin2
U+8758.0	fMandarin	yan3; yan4
U+8758.0	fEnglish	[1] gecko [2] a species of cicada
U+8759.0	fUTF8	蝙
U+8759.0	fCNS11643	1-6E68
U+8759.0	fBig5	BDBF
U+8759.0	fGB	0-7279
U+8759.0	fR/S	 142.9
U+8759.0	fTotalStrokes	 15
U+8759.0	fCangjie	LIHSB
U+8759.0	fFourCorner	53127
U+8759.0	fHakka	  bien1; pen1; ben2; pau2
U+8759.0	fCantonese	bin1; pin1 (also); bin2
U+8759.0	fMandarin	bian1; pian2
U+8759.0	fEnglish	 bat
U+875A.0	fUTF8	蝚
U+875A.0	fCNS11643	2-5373
U+875A.0	fBig5	E7AC
U+875A.0	fGB	3-625D
U+875A.0	fR/S	 142.9
U+875A.0	fTotalStrokes	 15
U+875A.0	fCangjie	LINHD
U+875A.0	fFourCorner	57194
U+875A.0	fCantonese	jau4; nou4
U+875A.0	fMandarin	rou2; nao2
U+875B.0	fUTF8	蝛
U+875B.0	fCNS11643	2-5370
U+875B.0	fBig5	E7A9
U+875B.0	fGB	5-6168
U+875B.0	fR/S	 142.9
U+875B.0	fTotalStrokes	 15
U+875B.0	fCangjie	LIIHV
U+875B.0	fFourCorner	53150
U+875B.0	fHakka	 wi1; wui1; vui1
U+875B.0	fCantonese	wai1
U+875B.0	fMandarin	wei1
U+875B.0	fEnglish	the sow-bug; wood-louse
U+875C.0	fUTF8	蝜
U+875C.0	fCNS11643	2-5422
U+875C.0	fBig5	E7B9
U+875C.0	fGB	5-6138
U+875C.0	fR/S	 142.9
U+875C.0	fTotalStrokes	 15
U+875C.0	fCangjie	LINBC
U+875C.0	fFourCorner	57186
U+875C.0	fCantonese	fu6
U+875C.0	fMandarin	fu4
U+875C.0	fEnglish	a kind of small insect which can move things many times its weight
U+875D.0	fUTF8	蝝
U+875D.0	fCNS11643	2-537B
U+875D.0	fBig5	E7B4
U+875D.0	fGB	3-625E
U+875D.0	fR/S	 142.9
U+875D.0	fTotalStrokes	 15
U+875D.0	fCangjie	LIVNO
U+875D.0	fFourCorner	57132
U+875D.0	fHakka	 jen2; ren2
U+875D.0	fCantonese	jyun4
U+875D.0	fMandarin	yuan2; ju4; xuan3; yung4
U+875D.0	fEnglish	the larva of locust
U+875E.0	fUTF8	蝞
U+875E.0	fCNS11643	2-5375
U+875E.0	fBig5	E7AE
U+875E.0	fGB	E-3C41
U+875E.0	fR/S	 142.9
U+875E.0	fTotalStrokes	 15
U+875E.0	fCangjie	LIAHU
U+875E.0	fFourCorner	57167
U+875E.0	fCantonese	mei6; mei1
U+875E.0	fMandarin	mei4
U+875E.0	fEnglish	塘蝞, dragonfly
U+875F.0	fUTF8	蝟
U+875F.0	fCNS11643	2-537A
U+875F.0	fBig5	E7B3
U+875F.0	fGB	E-3C42
U+875F.0	fR/S	 142.9
U+875F.0	fTotalStrokes	 15
U+875F.0	fCangjie	LIWB
U+875F.0	fFourCorner	56127
U+875F.0	fHakka	 wui3; wi5; wui6; vui5; wui5
U+875F.0	fCantonese	wai6
U+875F.0	fMandarin	wei4
U+875F.0	fEnglish	 hedgehog
U+8760.0	fUTF8	蝠
U+8760.0	fCNS11643	1-6E64
U+8760.0	fBig5	BDBB
U+8760.0	fGB	0-7270
U+8760.0	fR/S	 142.9
U+8760.0	fTotalStrokes	 15
U+8760.0	fCangjie	LIMRW
U+8760.0	fFourCorner	51166
U+8760.0	fHakka	  fuk7; pit8;  pet8; buk7
U+8760.0	fCantonese	fuk1
U+8760.0	fMandarin	fu2
U+8760.0	fEnglish	[1] used in combination for bat [2] used in combination for bat (p'it8 p'o2)
U+8761.0	fUTF8	蝡
U+8761.0	fCNS11643	2-5372
U+8761.0	fBig5	E7AB
U+8761.0	fGB	E-3C43
U+8761.0	fR/S	 142.9
U+8761.0	fTotalStrokes	 15
U+8761.0	fCangjie	LIMBK
U+8761.0	fFourCorner	51134
U+8761.0	fHakka	 ngion3; nion1; ji2
U+8761.0	fCantonese	jyu4; jyun5
U+8761.0	fMandarin	ruan3; run3; ru2
U+8761.0	fEnglish	 wriggle
U+8762.0	fUTF8	蝢
U+8762.0	fCNS11643	2-5427
U+8762.0	fBig5	E7BE
U+8762.0	fGB	5-612C
U+8762.0	fR/S	 142.9
U+8762.0	fTotalStrokes	 15
U+8762.0	fCangjie	LIMBC
U+8762.0	fFourCorner	51186
U+8762.0	fCantonese	kit3; fu6
U+8762.0	fMandarin	xie2
U+8763.0	fUTF8	蝣
U+8763.0	fCNS11643	2-5369
U+8763.0	fBig5	E7A2
U+8763.0	fGB	0-7276
U+8763.0	fR/S	 142.9
U+8763.0	fTotalStrokes	 15
U+8763.0	fCangjie	LIYSD
U+8763.0	fFourCorner	58147
U+8763.0	fHakka	  ju2; riu2; jiu3; jiu2
U+8763.0	fCantonese	jau4; jau6
U+8763.0	fMandarin	you2; liu2
U+8763.0	fEnglish	used in combination for mayfly, ephemera
U+8764.0	fUTF8	蝤
U+8764.0	fCNS11643	2-536A
U+8764.0	fBig5	E7A3
U+8764.0	fGB	0-7278
U+8764.0	fR/S	 142.9
U+8764.0	fTotalStrokes	 15
U+8764.0	fCangjie	LITCW
U+8764.0	fFourCorner	58161
U+8764.0	fHakka	  ciu2;  siu2; ju2; riu2
U+8764.0	fCantonese	jau4; cau4; zau1
U+8764.0	fMandarin	qiu2; you2; jiu1
U+8764.0	fEnglish	[1] the chrysalis of a kind of beetle [2] ephemera; mayfly
U+8765.0	fUTF8	蝥
U+8765.0	fCNS11643	2-5423
U+8765.0	fBig5	E7BA
U+8765.0	fGB	0-727A
U+8765.0	fR/S	 142.9
U+8765.0	fTotalStrokes	 15
U+8765.0	fCangjie	NKLMI
U+8765.0	fFourCorner	18136
U+8765.0	fHakka	 mau2; meu2; meu3
U+8765.0	fCantonese	maau4; mau4
U+8765.0	fMandarin	mou2; mao2; wu2; wu4; mao1
U+8765.0	fEnglish	a kind of noxious insect that feeds on the roots of rice plants
U+8766.0	fUTF8	蝦
U+8766.0	fCNS11643	1-6E65
U+8766.0	fBig5	BDBC
U+8766.0	fGB	1-4F3A
U+8766.0	fR/S	 142.9
U+8766.0	fTotalStrokes	 15
U+8766.0	fCangjie	LIRSE
U+8766.0	fFourCorner	57147
U+8766.0	fHakka	  ha2; ha3; ha1
U+8766.0	fCantonese	haa1; haa4
U+8766.0	fMandarin	xia1; ha1; jia3
U+8766.0	fEnglish	 shrimp; prawn
U+8766.1	fUTF8	蝦
U+8766.1	fCNS11643	1-6E65
U+8766.1	fBig5	BDBC
U+8766.1	fGB	1-4F3A
U+8766.1	fR/S	 142.9
U+8766.1	fTotalStrokes	 15
U+8766.1	fCangjie	LIRSE
U+8766.1	fFourCorner	57147
U+8766.1	fMandarin	ha2
U+8766.1	fEnglish	used in combination for toad
U+8767.0	fUTF8	蝧
U+8767.0	fCNS11643	2-5428
U+8767.0	fBig5	E7BF
U+8767.0	fGB	E-3C44
U+8767.0	fR/S	 142.9
U+8767.0	fTotalStrokes	 15
U+8767.0	fCangjie	LITLK
U+8767.0	fFourCorner	54135
U+8767.0	fCantonese	jing1
U+8767.0	fMandarin	ying1
U+8768.0	fUTF8	蝨
U+8768.0	fCNS11643	1-6E67
U+8768.0	fBig5	BDBE
U+8768.0	fGB	E-3C45
U+8768.0	fR/S	 142.9
U+8768.0	fTotalStrokes	 15
U+8768.0	fCangjie	NJLII
U+8768.0	fFourCorner	17136
U+8768.0	fHakka	  sit7; set7
U+8768.0	fCantonese	sat1
U+8768.0	fMandarin	shi1
U+8768.0	fEnglish	 louse
U+8769.0	fUTF8	蝩
U+8769.0	fCNS11643	2-5429
U+8769.0	fBig5	E7C0
U+8769.0	fGB	3-6252
U+8769.0	fR/S	 142.9
U+8769.0	fTotalStrokes	 15
U+8769.0	fCangjie	LIHJG
U+8769.0	fFourCorner	52114
U+8769.0	fCantonese	cung4; zung1
U+8769.0	fMandarin	chong2; zhong1
U+876A.0	fUTF8	蝪
U+876A.0	fCNS11643	2-5377
U+876A.0	fBig5	E7B0
U+876A.0	fGB	5-6068
U+876A.0	fR/S	 142.9
U+876A.0	fTotalStrokes	 15
U+876A.0	fCangjie	LIAMH
U+876A.0	fFourCorner	56127
U+876A.0	fHakka	 tong1
U+876A.0	fCantonese	tong1; jik6
U+876A.0	fMandarin	tang1; tang2
U+876A.0	fEnglish	a kind of spider
U+876B.0	fUTF8	蝫
U+876B.0	fCNS11643	2-4D5F
U+876B.0	fBig5	E3D8
U+876B.0	fGB	E-3C46
U+876B.0	fR/S	 142.9
U+876B.0	fTotalStrokes	 15
U+876B.0	fCangjie	LIJKA
U+876B.0	fFourCorner	54160
U+876B.0	fHakka	 zhu1; zu1
U+876B.0	fCantonese	zyu1
U+876B.0	fMandarin	zhu1
U+876B.0	fEnglish	an edible worm
U+876C.0	fUTF8	蝬
U+876C.0	fCNS11643	2-537D
U+876C.0	fBig5	E7B6
U+876C.0	fGB	E-3C47
U+876C.0	fR/S	 142.9
U+876C.0	fTotalStrokes	 15
U+876C.0	fCangjie	LIUCE
U+876C.0	fFourCorner	52147
U+876C.0	fCantonese	zung1
U+876C.0	fMandarin	zong1
U+876D.0	fUTF8	蝭
U+876D.0	fCNS11643	2-5376
U+876D.0	fBig5	E7AF
U+876D.0	fGB	3-6250
U+876D.0	fR/S	 142.9
U+876D.0	fTotalStrokes	 15
U+876D.0	fCangjie	LIAMO
U+876D.0	fFourCorner	56181
U+876D.0	fHakka	 ti2; ti3
U+876D.0	fCantonese	tai4
U+876D.0	fMandarin	ti2; chi2; di2; di4
U+876D.0	fEnglish	a small kind of cicada
U+876E.0	fUTF8	蝮
U+876E.0	fCNS11643	2-5421
U+876E.0	fBig5	E7B8
U+876E.0	fGB	0-7273
U+876E.0	fR/S	 142.9
U+876E.0	fTotalStrokes	 15
U+876E.0	fCangjie	LIOAE
U+876E.0	fFourCorner	58147
U+876E.0	fHakka	  fuk7; buk7; pit8; fuk8
U+876E.0	fCantonese	fuk1
U+876E.0	fMandarin	fu4; bo2; fu2
U+876E.0	fEnglish	 viper
U+876F.0	fUTF8	蝯
U+876F.0	fCNS11643	2-537C
U+876F.0	fBig5	E7B5
U+876F.0	fGB	E-3C48
U+876F.0	fR/S	 142.9
U+876F.0	fTotalStrokes	 15
U+876F.0	fCangjie	LIBME
U+876F.0	fFourCorner	52147
U+876F.0	fHakka	 jen1; ren1
U+876F.0	fCantonese	jyun4
U+876F.0	fMandarin	yuan2
U+876F.0	fEnglish	 monkey; ape
U+8770.0	fUTF8	蝰
U+8770.0	fCNS11643	5-5055
U+8770.0	fGB	0-7271
U+8770.0	fR/S	 142.9
U+8770.0	fTotalStrokes	 15
U+8770.0	fCangjie	LIKGG
U+8770.0	fFourCorner	54110
U+8770.0	fCantonese	kwai1; fui1 (also)
U+8770.0	fMandarin	kui1; kui2; hui3
U+8770.0	fEnglish	Vipera russelii siamensis
U+8771.0	fUTF8	蝱
U+8771.0	fCNS11643	3-4E5D
U+8771.0	fGB	E-3C49
U+8771.0	fR/S	 142.9
U+8771.0	fTotalStrokes	 15
U+8771.0	fCangjie	YVLII
U+8771.0	fFourCorner	00130
U+8771.0	fHakka	 mang3;  mang2; mong2; mang1
U+8771.0	fCantonese	mang4; mong4
U+8771.0	fMandarin	meng2
U+8771.0	fEnglish	 gadfly
U+8772.0	fUTF8	蝲
U+8772.0	fCNS11643	4-4D28
U+8772.0	fGB	3-624E
U+8772.0	fR/S	  142.9
U+8772.0	fTotalStrokes	 15
U+8772.0	fCangjie	LIDLN
U+8772.0	fFourCorner	52100
U+8772.0	fHakka	 la2
U+8772.0	fCantonese	laat6; lat6
U+8772.0	fMandarin	la4
U+8773.0	fUTF8	蝳
U+8773.0	fCNS11643	2-536D
U+8773.0	fBig5	E7A6
U+8773.0	fGB	5-6167
U+8773.0	fR/S	 142.9; 142.8
U+8773.0	fTotalStrokes	 15
U+8773.0	fCangjie	LIQMY
U+8773.0	fFourCorner	55157
U+8773.0	fCantonese	doi6
U+8773.0	fMandarin	du2; dai4
U+8774.0	fUTF8	蝴
U+8774.0	fCNS11643	1-6E62
U+8774.0	fBig5	BDB9
U+8774.0	fGB	0-3A7B
U+8774.0	fR/S	 142.9
U+8774.0	fTotalStrokes	 15
U+8774.0	fCangjie	LIJRB
U+8774.0	fFourCorner	57120
U+8774.0	fHakka	 fu2; fu3
U+8774.0	fCantonese	wu4
U+8774.0	fMandarin	hu2
U+8774.0	fEnglish	 butterfly
U+8775.0	fUTF8	蝵
U+8775.0	fCNS11643	2-5426
U+8775.0	fBig5	E7BD
U+8775.0	fGB	5-6170
U+8775.0	fR/S	 142.9
U+8775.0	fTotalStrokes	 15
U+8775.0	fCangjie	HFLMI
U+8775.0	fFourCorner	29136
U+8775.0	fCantonese	cau1
U+8775.0	fMandarin	qiu1
U+8776.0	fUTF8	蝶
U+8776.0	fCNS11643	1-6E63
U+8776.0	fBig5	BDBA
U+8776.0	fGB	0-357B
U+8776.0	fR/S	 142.9
U+8776.0	fTotalStrokes	 15
U+8776.0	fCangjie	LIPTD
U+8776.0	fFourCorner	54194
U+8776.0	fHakka	  tiap8;  jap8; jak8; rap8; zak8
U+8776.0	fCantonese	dip6; dip2 (also)
U+8776.0	fMandarin	die2; tie3; tie1
U+8776.0	fEnglish	 butterfly
U+8777.0	fUTF8	蝷
U+8777.0	fCNS11643	2-536B
U+8777.0	fBig5	E7A4
U+8777.0	fGB	E-3C4A
U+8777.0	fR/S	 142.9
U+8777.0	fTotalStrokes	 15
U+8777.0	fCangjie	LIITU
U+8777.0	fFourCorner	50147
U+8777.0	fCantonese	lik6; sik1
U+8777.0	fMandarin	li4; xi2
U+8778.0	fUTF8	蝸
U+8778.0	fCNS11643	1-6E66
U+8778.0	fBig5	BDBD
U+8778.0	fGB	1-4E4F
U+8778.0	fR/S	 142.9
U+8778.0	fTotalStrokes	 15
U+8778.0	fCangjie	LIBBR
U+8778.0	fFourCorner	57127
U+8778.0	fHakka	  wo1; vo1
U+8778.0	fCantonese	wo1; waa1; gwaa1
U+8778.0	fMandarin	gua1; wo1; luo2; ge1; guo3
U+8778.0	fEnglish	 snail
U+8779.0	fUTF8	蝹
U+8779.0	fCNS11643	2-5A32
U+8779.0	fBig5	EB64
U+8779.0	fGB	5-616C
U+8779.0	fR/S	 142.10
U+8779.0	fTotalStrokes	 16
U+8779.0	fCangjie	LIWOT
U+8779.0	fFourCorner	56117
U+8779.0	fHakka	 wun1; vun1
U+8779.0	fCantonese	wan1; ou2
U+8779.0	fMandarin	yun1; ao3; xian2; yuan1
U+8779.0	fEnglish	 wringle (snake)
U+877A.0	fUTF8	蝺
U+877A.0	fCNS11643	2-537E
U+877A.0	fBig5	E7B7
U+877A.0	fGB	5-6172
U+877A.0	fR/S	 142.9
U+877A.0	fTotalStrokes	 15
U+877A.0	fCangjie	LIHLB
U+877A.0	fFourCorner	52127
U+877A.0	fCantonese	geoi2; jyu5
U+877A.0	fMandarin	ju3; yu3
U+877B.0	fUTF8	蝻
U+877B.0	fCNS11643	2-5425
U+877B.0	fBig5	E7BC
U+877B.0	fGB	0-726F
U+877B.0	fR/S	 142.9
U+877B.0	fTotalStrokes	 15
U+877B.0	fCangjie	LIJBJ
U+877B.0	fFourCorner	54127
U+877B.0	fHakka	  nam2; lam2
U+877B.0	fCantonese	naam4; laam4 (also); naam5
U+877B.0	fMandarin	nan3; nan2; an2
U+877B.0	fEnglish	immature locusts, whose wings have not fully grown
U+877C.0	fUTF8	蝼
U+877C.0	fCNS11643	4-4D31
U+877C.0	fGB	0-7277
U+877C.0	fR/S	 142.9
U+877C.0	fTotalStrokes	 15
U+877C.0	fCangjie	LIFDV
U+877C.0	fFourCorner	59140
U+877C.0	fHakka	 leu2
U+877C.0	fCantonese	lau4
U+877C.0	fMandarin	lou2; lou4
U+877C.0	fEnglish	simplified form of &#x87BB;
U+877D.0	fUTF8	蝽
U+877D.0	fCNS11643	3-4E59
U+877D.0	fGB	0-726D
U+877D.0	fR/S	  142.9
U+877D.0	fTotalStrokes	 15
U+877D.0	fCangjie	LIQKA
U+877D.0	fFourCorner	55160
U+877D.0	fCantonese	ceon1
U+877D.0	fMandarin	chun3; chuan3; qun3; chun1
U+877E.0	fUTF8	蝾
U+877E.0	fGB	0-726E
U+877E.0	fR/S	 142.9
U+877E.0	fTotalStrokes	 15
U+877E.0	fCangjie	LITBD
U+877E.0	fHakka	 jin2
U+877E.0	fCantonese	wing4
U+877E.0	fMandarin	rong2
U+877E.0	fEnglish	simplified form of &#x8811;
U+877F.0	fUTF8	蝿
U+877F.0	fGB	E-3C4B
U+877F.0	fR/S	 142.9
U+877F.0	fTotalStrokes	 15
U+877F.0	fCangjie	LIWLU
U+877F.0	fCantonese	jing4
U+877F.0	fMandarin	ying2
U+877F.0	fEnglish	flies
U+8780.0	fUTF8	螀
U+8780.0	fGB	8-2D78
U+8780.0	fR/S	 142.9
U+8780.0	fTotalStrokes	 15
U+8780.0	fCangjie	LILMI
U+8780.0	fCantonese	zoeng1
U+8780.0	fMandarin	jiang1
U+8780.0	fEnglish	kind of cicada
U+8781.0	fUTF8	螁
U+8781.0	fCNS11643	2-5A2F
U+8781.0	fBig5	EB61
U+8781.0	fGB	E-3C4C
U+8781.0	fR/S	 142.10
U+8781.0	fTotalStrokes	 16
U+8781.0	fCangjie	LIYAV
U+8781.0	fFourCorner	57133
U+8781.0	fCantonese	teoi3
U+8781.0	fMandarin	tui4; tun4
U+8782.0	fUTF8	螂
U+8782.0	fCNS11643	1-6E61
U+8782.0	fBig5	BDB8
U+8782.0	fGB	0-726B
U+8782.0	fR/S	 142.10
U+8782.0	fTotalStrokes	 16
U+8782.0	fCangjie	LIIIL
U+8782.0	fFourCorner	57127
U+8782.0	fHakka	  long2; long3
U+8782.0	fCantonese	long4; long2 (also)
U+8782.0	fMandarin	lang2; liang2
U+8782.0	fEnglish	same as $
U+8783.0	fUTF8	螃
U+8783.0	fCNS11643	1-722A
U+8783.0	fBig5	BFC0
U+8783.0	fGB	0-7326
U+8783.0	fR/S	 142.10
U+8783.0	fTotalStrokes	 16
U+8783.0	fCangjie	LIYBS
U+8783.0	fFourCorner	50127
U+8783.0	fHakka	  pong2; pong3
U+8783.0	fCantonese	pong4; bong2
U+8783.0	fMandarin	pang2; bang3; bang4; beng1
U+8783.0	fEnglish	 crab
U+8784.0	fUTF8	螄
U+8784.0	fCNS11643	2-5A39
U+8784.0	fBig5	EB6B
U+8784.0	fGB	1-724F
U+8784.0	fR/S	 142.10
U+8784.0	fTotalStrokes	 16
U+8784.0	fCangjie	LIHRB
U+8784.0	fFourCorner	51127
U+8784.0	fHakka	 sii1; se1
U+8784.0	fCantonese	si1
U+8784.0	fMandarin	si1; shi1
U+8784.0	fEnglish	a species of mollusk with a spiral shell
U+8785.0	fUTF8	螅
U+8785.0	fCNS11643	2-5A35
U+8785.0	fBig5	EB67
U+8785.0	fGB	0-7323
U+8785.0	fR/S	 142.10
U+8785.0	fTotalStrokes	 16
U+8785.0	fCangjie	LIHUP
U+8785.0	fFourCorner	56130
U+8785.0	fCantonese	sik1
U+8785.0	fMandarin	xi1; ci4; xi2
U+8786.0	fUTF8	螆
U+8786.0	fCNS11643	4-533C
U+8786.0	fGB	5-6175
U+8786.0	fR/S	 142.10; 142.9
U+8786.0	fTotalStrokes	 16; 15
U+8786.0	fCangjie	LITVI
U+8786.0	fFourCorner	54130
U+8786.0	fHakka	 cih5; ci5; zi2
U+8786.0	fCantonese	zi1
U+8786.0	fMandarin	ci4
U+8786.0	fEnglish	hairy and poisonous caterpillars
U+8787.0	fUTF8	螇
U+8787.0	fCNS11643	2-5A33
U+8787.0	fBig5	EB65
U+8787.0	fGB	3-6265
U+8787.0	fR/S	 142.10
U+8787.0	fTotalStrokes	 16
U+8787.0	fCangjie	LIBVK
U+8787.0	fFourCorner	52135
U+8787.0	fHakka	 hi1
U+8787.0	fCantonese	hai4; hei1
U+8787.0	fMandarin	xi1; qi1
U+8787.0	fEnglish	a kind of grasshopper
U+8788.0	fUTF8	螈
U+8788.0	fCNS11643	2-5A2E
U+8788.0	fBig5	EB60
U+8788.0	fGB	0-7322
U+8788.0	fR/S	 142.10
U+8788.0	fTotalStrokes	 16
U+8788.0	fCangjie	LIMHF
U+8788.0	fFourCorner	51196
U+8788.0	fHakka	 ngien2; ngian2
U+8788.0	fCantonese	jyun4
U+8788.0	fMandarin	yuan2
U+8788.0	fEnglish	[1] Diemycttylus pyrrhogaster, a kind of amphibious reptile; salamander [2] a species of silkworm which produces silk only once a season
U+8789.0	fUTF8	螉
U+8789.0	fCNS11643	2-5A3D
U+8789.0	fBig5	EB6F
U+8789.0	fGB	3-6266
U+8789.0	fR/S	 142.10
U+8789.0	fTotalStrokes	 16
U+8789.0	fCangjie	LICIM
U+8789.0	fFourCorner	58127
U+8789.0	fHakka	 wung1; vung1
U+8789.0	fCantonese	jung1
U+8789.0	fMandarin	weng1; weng3
U+8789.0	fEnglish	wasp
U+878A.0	fUTF8	螊
U+878A.0	fCNS11643	3-525B
U+878A.0	fGB	3-6269
U+878A.0	fR/S	  142.10
U+878A.0	fTotalStrokes	 16
U+878A.0	fCangjie	LITXC
U+878A.0	fFourCorner	58130
U+878A.0	fHakka	 liam2
U+878A.0	fCantonese	lim4; ham1
U+878A.0	fMandarin	lian2; xian2
U+878A.0	fEnglish	an insect which smells like ginger
U+878B.0	fUTF8	螋
U+878B.0	fCNS11643	3-525D
U+878B.0	fGB	0-7274
U+878B.0	fR/S	 142.10; 142.9
U+878B.0	fTotalStrokes	 16; 15
U+878B.0	fCangjie	LIHXE
U+878B.0	fFourCorner	67140
U+878B.0	fCantonese	sau1; sau3 (also); sau2 (also)
U+878B.0	fMandarin	sou1
U+878B.0	fEnglish	spider millipede
U+878C.0	fUTF8	螌
U+878C.0	fCNS11643	4-5344
U+878C.0	fGB	3-6264
U+878C.0	fR/S	 142.10
U+878C.0	fTotalStrokes	 16
U+878C.0	fCangjie	HELMI
U+878C.0	fFourCorner	27130
U+878C.0	fHakka	  ban1
U+878C.0	fCantonese	baan1; pun4
U+878C.0	fMandarin	ban1; pan2
U+878C.0	fEnglish	a striped; poisonous fly
U+878D.0	fUTF8	融
U+878D.0	fCNS11643	1-722E
U+878D.0	fBig5	BFC4
U+878D.0	fGB	0-485A
U+878D.0	fR/S	 142.10
U+878D.0	fTotalStrokes	 16
U+878D.0	fCangjie	MBLMI
U+878D.0	fFourCorner	15236
U+878D.0	fHakka	 jung2; rung2; jung3
U+878D.0	fCantonese	jung4
U+878D.0	fMandarin	rong2
U+878D.0	fEnglish	[1] very bright; glowing; burning [2] melt [3] melt into; fuse; blend; harmonize [4] cheerful; happy; joyful [5] a Chinese family name
U+878E.0	fUTF8	螎
U+878E.0	fCNS11643	3-525F
U+878E.0	fGB	E-3C4D
U+878E.0	fR/S	 142.10
U+878E.0	fTotalStrokes	 16
U+878E.0	fCangjie	LIMRB
U+878E.0	fFourCorner	51120
U+878E.0	fHakka	 jung2
U+878E.0	fCantonese	jung4
U+878E.0	fMandarin	rong2
U+878F.0	fUTF8	螏
U+878F.0	fCNS11643	2-5A2A
U+878F.0	fBig5	EB5C
U+878F.0	fGB	5-6225
U+878F.0	fR/S	 142.10
U+878F.0	fTotalStrokes	 16
U+878F.0	fCangjie	LIKOK
U+878F.0	fFourCorner	50134
U+878F.0	fCantonese	zat6
U+878F.0	fMandarin	ji2
U+8790.0	fUTF8	螐
U+8790.0	fCNS11643	2-5A36
U+8790.0	fBig5	EB68
U+8790.0	fGB	E-3C4E
U+8790.0	fR/S	 142.10
U+8790.0	fTotalStrokes	 16
U+8790.0	fCangjie	LIHRF
U+8790.0	fFourCorner	57127
U+8790.0	fCantonese	wu1
U+8790.0	fMandarin	wu1
U+8791.0	fUTF8	螑
U+8791.0	fCNS11643	2-5A37
U+8791.0	fBig5	EB69
U+8791.0	fGB	E-3C4F
U+8791.0	fR/S	 142.10
U+8791.0	fTotalStrokes	 16
U+8791.0	fCangjie	LIHUK
U+8791.0	fFourCorner	56134
U+8791.0	fCantonese	jau3; cau3
U+8791.0	fMandarin	qiu4; xiu4
U+8792.0	fUTF8	螒
U+8792.0	fCNS11643	2-5A2D
U+8792.0	fBig5	EB5F
U+8792.0	fGB	3-743B
U+8792.0	fR/S	 142.10
U+8792.0	fTotalStrokes	 16
U+8792.0	fCangjie	JJOLI
U+8792.0	fFourCorner	58036
U+8792.0	fCantonese	hon6
U+8792.0	fMandarin	han4
U+8793.0	fUTF8	螓
U+8793.0	fCNS11643	2-5A2C
U+8793.0	fBig5	EB5E
U+8793.0	fGB	0-727B
U+8793.0	fR/S	 142.10
U+8793.0	fTotalStrokes	 16
U+8793.0	fCangjie	LIQKD
U+8793.0	fFourCorner	55191
U+8793.0	fHakka	  cin2; zin1; cin3
U+8793.0	fCantonese	ceon4
U+8793.0	fMandarin	qin2
U+8793.0	fEnglish	a small cicada with a square head
U+8794.0	fUTF8	螔
U+8794.0	fCNS11643	2-5A3A
U+8794.0	fBig5	EB6C
U+8794.0	fGB	3-6263
U+8794.0	fR/S	 142.10
U+8794.0	fTotalStrokes	 16
U+8794.0	fCangjie	LIHYU
U+8794.0	fFourCorner	52117
U+8794.0	fHakka	 sii1; ji2; ri2
U+8794.0	fCantonese	ji4
U+8794.0	fMandarin	yi2; si1
U+8794.0	fEnglish	(ji2) an insect (sii1) a reptile
U+8795.0	fUTF8	螕
U+8795.0	fCNS11643	4-5341
U+8795.0	fGB	5-6223
U+8795.0	fR/S	  142.10
U+8795.0	fTotalStrokes	 16
U+8795.0	fCangjie	LIHWP
U+8795.0	fFourCorner	56110
U+8795.0	fHakka	 pi2
U+8795.0	fCantonese	bai1
U+8795.0	fMandarin	bi1
U+8795.0	fEnglish	cattle-tick
U+8796.0	fUTF8	螖
U+8796.0	fCNS11643	2-5A30
U+8796.0	fBig5	EB62
U+8796.0	fGB	3-6251
U+8796.0	fR/S	 142.10
U+8796.0	fTotalStrokes	 16
U+8796.0	fCangjie	LIBBB
U+8796.0	fFourCorner	57127
U+8796.0	fCantonese	waat6
U+8796.0	fMandarin	hua2; gu3
U+8797.0	fUTF8	螗
U+8797.0	fCNS11643	2-5A2B
U+8797.0	fBig5	EB5D
U+8797.0	fGB	0-7325
U+8797.0	fR/S	 142.10
U+8797.0	fTotalStrokes	 16
U+8797.0	fCangjie	LIILR
U+8797.0	fFourCorner	50167
U+8797.0	fHakka	  tong2; tong3
U+8797.0	fCantonese	tong4
U+8797.0	fMandarin	tang2
U+8797.0	fEnglish	a species of cicada
U+8798.0	fUTF8	螘
U+8798.0	fCNS11643	2-5A31
U+8798.0	fBig5	EB63
U+8798.0	fGB	5-6131
U+8798.0	fR/S	 142.10
U+8798.0	fTotalStrokes	 16
U+8798.0	fCangjie	LIUMT
U+8798.0	fFourCorner	52118
U+8798.0	fHakka	 ngie5
U+8798.0	fCantonese	ngai5; ai5 (also)
U+8798.0	fMandarin	yi3
U+8798.0	fEnglish	ant
U+8799.0	fUTF8	螙
U+8799.0	fCNS11643	3-5262
U+8799.0	fGB	5-455E
U+8799.0	fR/S	  142.10
U+8799.0	fTotalStrokes	 16
U+8799.0	fCangjie	DLII
U+8799.0	fFourCorner	40130
U+8799.0	fHakka	  du5; zhu5; zu5
U+8799.0	fCantonese	dou3; tou3
U+8799.0	fMandarin	du4
U+8799.0	fEnglish	grubs in wood; worms in books; clothes; etc. used figuratively of corrupt officials
U+879A.0	fUTF8	螚
U+879A.0	fCNS11643	2-5A3C
U+879A.0	fBig5	EB6E
U+879A.0	fGB	3-626C
U+879A.0	fR/S	 142.10
U+879A.0	fTotalStrokes	 16
U+879A.0	fCangjie	IPLMI
U+879A.0	fFourCorner	21136
U+879A.0	fCantonese	noi6; loi6 (also)
U+879A.0	fMandarin	nai4
U+879B.0	fUTF8	螛
U+879B.0	fCNS11643	2-5A29
U+879B.0	fBig5	EB5B
U+879B.0	fGB	3-626A
U+879B.0	fR/S	 142.10
U+879B.0	fTotalStrokes	 16
U+879B.0	fCangjie	LIJQR
U+879B.0	fFourCorner	53161
U+879B.0	fCantonese	hot6; hat6
U+879B.0	fMandarin	he2; xia2
U+879C.0	fUTF8	螜
U+879C.0	fCNS11643	2-5A3B
U+879C.0	fBig5	EB6D
U+879C.0	fGB	3-625F
U+879C.0	fR/S	 142.10
U+879C.0	fTotalStrokes	 16
U+879C.0	fCangjie	GELMI
U+879C.0	fFourCorner	47136
U+879C.0	fCantonese	huk6
U+879C.0	fMandarin	hu2
U+879D.0	fUTF8	螝
U+879D.0	fCNS11643	2-5A38
U+879D.0	fBig5	EB6A
U+879D.0	fGB	3-6254
U+879D.0	fR/S	 142.10
U+879D.0	fTotalStrokes	 16
U+879D.0	fCangjie	LIHUI
U+879D.0	fFourCorner	56113
U+879D.0	fCantonese	gwai1; kui2
U+879D.0	fMandarin	hui4; xi1; gui1; gui4
U+879E.0	fUTF8	螞
U+879E.0	fCNS11643	1-722C
U+879E.0	fBig5	BFC2
U+879E.0	fGB	1-426C
U+879E.0	fR/S	 142.10
U+879E.0	fTotalStrokes	 16
U+879E.0	fCangjie	LISQF
U+879E.0	fFourCorner	51127
U+879E.0	fHakka	 ma1;  ma3
U+879E.0	fCantonese	maa5; maa6; maa1
U+879E.0	fMandarin	ma3; ma1; ma4
U+879E.0	fEnglish	[1] ant [2] a kind of leech
U+879F.0	fUTF8	螟
U+879F.0	fCNS11643	1-722B
U+879F.0	fBig5	BFC1
U+879F.0	fGB	0-4378
U+879F.0	fR/S	 142.10
U+879F.0	fTotalStrokes	 16
U+879F.0	fCangjie	LIBAC
U+879F.0	fFourCorner	57180
U+879F.0	fHakka	 men2
U+879F.0	fCantonese	ming4
U+879F.0	fMandarin	ming2
U+879F.0	fEnglish	a kind of caterpillar; the larva of Chilo simplex
U+87A0.0	fUTF8	螠
U+87A0.0	fCNS11643	4-5339
U+87A0.0	fGB	3-6268
U+87A0.0	fR/S	  142.10
U+87A0.0	fTotalStrokes	 16
U+87A0.0	fCangjie	LITCT
U+87A0.0	fFourCorner	58110
U+87A0.0	fCantonese	jik1; si3 (also)
U+87A0.0	fMandarin	yi4
U+87A1.0	fUTF8	螡
U+87A1.0	fCNS11643	3-525E
U+87A1.0	fGB	E-3C50
U+87A1.0	fR/S	 142.10
U+87A1.0	fTotalStrokes	 16
U+87A1.0	fCangjie	YKLII
U+87A1.0	fFourCorner	00130
U+87A1.0	fCantonese	man1
U+87A1.0	fMandarin	wen2
U+87A1.0	fEnglish	 mosquito
U+87A2.0	fUTF8	螢
U+87A2.0	fCNS11643	1-722D
U+87A2.0	fBig5	BFC3
U+87A2.0	fGB	1-5329
U+87A2.0	fR/S	 142.10
U+87A2.0	fTotalStrokes	 16
U+87A2.0	fCangjie	FFBLI
U+87A2.0	fFourCorner	99136
U+87A2.0	fHakka	 jin2; rin2; jin3; jam3
U+87A2.0	fCantonese	jing4
U+87A2.0	fMandarin	ying2; xing2
U+87A2.0	fEnglish	 glow-worm; luminous insect; firefly
U+87A3.0	fUTF8	螣
U+87A3.0	fCNS11643	2-5A34
U+87A3.0	fBig5	EB66
U+87A3.0	fGB	3-4D77
U+87A3.0	fR/S	 142.10
U+87A3.0	fTotalStrokes	 16
U+87A3.0	fCangjie	BFQI
U+87A3.0	fFourCorner	79236
U+87A3.0	fHakka	 ten2;  tot7; tit8
U+87A3.0	fCantonese	dak6; tang4
U+87A3.0	fMandarin	teng2; te4; dai4; zhen4
U+87A3.0	fEnglish	mythological wingless dragon of
U+87A4.0	fUTF8	螤
U+87A4.0	fCNS11643	2-5A28
U+87A4.0	fBig5	F0CB
U+87A4.0	fGB	E-3C51
U+87A4.0	fR/S	 142.10
U+87A4.0	fTotalStrokes	 16
U+87A4.0	fCangjie	YILWO
U+87A4.0	fFourCorner	07137
U+87A4.0	fCantonese	zung1
U+87A4.0	fMandarin	yu3
U+87A5.0	fUTF8	螥
U+87A5.0	fCNS11643	3-6645
U+87A5.0	fGB	E-3C52
U+87A5.0	fR/S	 142.10
U+87A5.0	fTotalStrokes	 16
U+87A5.0	fCangjie	LIOIR
U+87A5.0	fFourCorner	5816
U+87A5.0	fHakka	 cong1
U+87A5.0	fCantonese	cong1
U+87A5.0	fMandarin	cang1
U+87A5.0	fEnglish	the house-fly
U+87A6.0	fUTF8	螦
U+87A6.0	fGB	1-7D4E
U+87A6.0	fR/S	 142.10
U+87A6.0	fTotalStrokes	 16
U+87A6.0	fCangjie	LIQMF
U+87A6.0	fCantonese	sou3
U+87A7.0	fUTF8	螧
U+87A7.0	fGB	E-3C53
U+87A7.0	fR/S	 142.10
U+87A7.0	fTotalStrokes	 16
U+87A7.0	fCangjie	LIJPA
U+87A7.0	fCantonese	kei4
U+87A8.0	fUTF8	螨
U+87A8.0	fGB	0-727D
U+87A8.0	fR/S	 142.10
U+87A8.0	fTotalStrokes	 16
U+87A8.0	fCangjie	LITMB
U+87A8.0	fCantonese	mun5
U+87A8.0	fMandarin	man3
U+87A8.0	fEnglish	simplified form of &#x87CE;
U+87A9.0	fUTF8	螩
U+87A9.0	fCNS11643	3-6644
U+87A9.0	fGB	E-3C54
U+87A9.0	fR/S	 142.10
U+87A9.0	fTotalStrokes	 16
U+87A9.0	fCangjie	LIOLD
U+87A9.0	fFourCorner	5719
U+87A9.0	fCantonese	tiu4
U+87AA.0	fUTF8	螪
U+87AA.0	fCNS11643	2-5F27
U+87AA.0	fBig5	EE59
U+87AA.0	fGB	5-6226
U+87AA.0	fR/S	 142.11
U+87AA.0	fTotalStrokes	 17
U+87AA.0	fCangjie	LIYCB
U+87AA.0	fFourCorner	50127
U+87AA.0	fCantonese	soeng1; zek3
U+87AA.0	fMandarin	shang1
U+87AB.0	fUTF8	螫
U+87AB.0	fCNS11643	1-753C
U+87AB.0	fBig5	C1B1
U+87AB.0	fGB	0-7327
U+87AB.0	fR/S	 142.11
U+87AB.0	fTotalStrokes	 17
U+87AB.0	fCangjie	GKLMI
U+87AB.0	fFourCorner	48136
U+87AB.0	fHakka	  sit7; diau1
U+87AB.0	fCantonese	sik1; cik1 (also)
U+87AB.0	fMandarin	shi4; zhe1; he4; she4
U+87AB.0	fEnglish	[1] sting [2] a poisonous insect; scorpion
U+87AC.0	fUTF8	螬
U+87AC.0	fCNS11643	2-5F2B
U+87AC.0	fBig5	EE5D
U+87AC.0	fGB	0-7329
U+87AC.0	fR/S	 142.11
U+87AC.0	fTotalStrokes	 17
U+87AC.0	fCangjie	LITWA
U+87AC.0	fFourCorner	54267
U+87AC.0	fHakka	  cau2; co2
U+87AC.0	fCantonese	cou4
U+87AC.0	fMandarin	cao2
U+87AC.0	fEnglish	 grubs (larvae of a beetle)
U+87AD.0	fUTF8	螭
U+87AD.0	fCNS11643	2-5F28
U+87AD.0	fBig5	EE5A
U+87AD.0	fGB	0-7324
U+87AD.0	fR/S	 142.11
U+87AD.0	fTotalStrokes	 17
U+87AD.0	fCangjie	LIYUB
U+87AD.0	fFourCorner	50127
U+87AD.0	fHakka	 li2; chi1
U+87AD.0	fCantonese	ci1
U+87AD.0	fMandarin	chi1
U+87AD.0	fEnglish	a dragon whose horns have not grown; a hornless dragon (usually represented in engravings adorning palaces, etc.)
U+87AE.0	fUTF8	螮
U+87AE.0	fCNS11643	2-5F2F
U+87AE.0	fBig5	EE61
U+87AE.0	fGB	3-624C
U+87AE.0	fR/S	 142.11
U+87AE.0	fTotalStrokes	 17
U+87AE.0	fCangjie	LIKPB
U+87AE.0	fFourCorner	54127
U+87AE.0	fHakka	 di5;  dai5; tai5
U+87AE.0	fCantonese	dai3
U+87AE.0	fMandarin	di4; dai4
U+87AE.0	fEnglish	 rainbow
U+87AF.0	fUTF8	螯
U+87AF.0	fCNS11643	2-5F35
U+87AF.0	fBig5	EE67
U+87AF.0	fGB	0-727C
U+87AF.0	fR/S	 142.11
U+87AF.0	fTotalStrokes	 17
U+87AF.0	fCangjie	GKLMI
U+87AF.0	fFourCorner	48136
U+87AF.0	fHakka	 ngau2
U+87AF.0	fCantonese	ngou4; ou4 (also); gong6
U+87AF.0	fMandarin	ao2
U+87AF.0	fEnglish	nippers or large claws of a crab
U+87B0.0	fUTF8	螰
U+87B0.0	fCNS11643	2-5F2A
U+87B0.0	fBig5	EE5C
U+87B0.0	fGB	3-6274
U+87B0.0	fR/S	 142.11
U+87B0.0	fTotalStrokes	 17
U+87B0.0	fCangjie	LIIXP
U+87B0.0	fFourCorner	50111
U+87B0.0	fHakka	  luk8
U+87B0.0	fCantonese	luk6
U+87B0.0	fMandarin	lu4
U+87B0.0	fEnglish	a species of cicada
U+87B1.0	fUTF8	螱
U+87B1.0	fCNS11643	3-563A
U+87B1.0	fGB	3-6278
U+87B1.0	fR/S	 142.11
U+87B1.0	fTotalStrokes	 17
U+87B1.0	fCangjie	SILMI
U+87B1.0	fFourCorner	74130
U+87B1.0	fCantonese	wai3
U+87B1.0	fMandarin	wei4
U+87B2.0	fUTF8	螲
U+87B2.0	fCNS11643	2-5F3E
U+87B2.0	fBig5	EE70
U+87B2.0	fGB	3-6277
U+87B2.0	fR/S	 142.11
U+87B2.0	fTotalStrokes	 17
U+87B2.0	fCangjie	LIJCG
U+87B2.0	fFourCorner	53114
U+87B2.0	fHakka	 zhit7; tsiit7
U+87B2.0	fCantonese	dit6; zat6; zat1; dit3
U+87B2.0	fMandarin	zhi4; die2
U+87B2.0	fEnglish	a kind of spider
U+87B3.0	fUTF8	螳
U+87B3.0	fCNS11643	1-7539
U+87B3.0	fBig5	C1AE
U+87B3.0	fGB	0-732B
U+87B3.0	fR/S	 142.11
U+87B3.0	fTotalStrokes	 17
U+87B3.0	fCangjie	LIFBG
U+87B3.0	fFourCorner	59114
U+87B3.0	fHakka	  tong2; tong3
U+87B3.0	fCantonese	tong4
U+87B3.0	fMandarin	tang2
U+87B3.0	fEnglish	 mantis
U+87B4.0	fUTF8	螴
U+87B4.0	fCNS11643	2-5F38
U+87B4.0	fBig5	EE6A
U+87B4.0	fGB	5-614E
U+87B4.0	fR/S	 142.11
U+87B4.0	fTotalStrokes	 17
U+87B4.0	fCangjie	NWLMI
U+87B4.0	fFourCorner	75296
U+87B4.0	fCantonese	can4
U+87B4.0	fMandarin	chen2; chen1; chen4
U+87B5.0	fUTF8	螵
U+87B5.0	fCNS11643	2-5F2D
U+87B5.0	fBig5	EE5F
U+87B5.0	fGB	0-732A
U+87B5.0	fR/S	 142.11
U+87B5.0	fTotalStrokes	 17
U+87B5.0	fCangjie	LIMWF
U+87B5.0	fFourCorner	51191
U+87B5.0	fHakka	  piau1
U+87B5.0	fCantonese	piu1; piu4
U+87B5.0	fMandarin	piao1; piao2; pi2
U+87B5.0	fEnglish	a chrysalis
U+87B6.0	fUTF8	螶
U+87B6.0	fCNS11643	2-5F39
U+87B6.0	fBig5	EE6B
U+87B6.0	fGB	5-617C
U+87B6.0	fR/S	 142.11
U+87B6.0	fTotalStrokes	 17
U+87B6.0	fCangjie	SSLII
U+87B6.0	fFourCorner	71136
U+87B6.0	fCantonese	keoi4; geoi2
U+87B6.0	fMandarin	qu2; ju4
U+87B7.0	fUTF8	螷
U+87B7.0	fCNS11643	2-5F34
U+87B7.0	fBig5	EE66
U+87B7.0	fGB	E-3C55
U+87B7.0	fR/S	 142.11
U+87B7.0	fTotalStrokes	 17
U+87B7.0	fCangjie	IHJI
U+87B7.0	fFourCorner	00236
U+87B7.0	fHakka	 pe1
U+87B7.0	fCantonese	pei4
U+87B7.0	fMandarin	pi2; beng4; bi4
U+87B8.0	fUTF8	螸
U+87B8.0	fCNS11643	2-5F3B
U+87B8.0	fBig5	EE6D
U+87B8.0	fGB	E-3C56
U+87B8.0	fR/S	 142.11
U+87B8.0	fTotalStrokes	 17
U+87B8.0	fCangjie	COLMI
U+87B8.0	fFourCorner	87136
U+87B8.0	fCantonese	jyu4
U+87B8.0	fMandarin	yu2; yu4
U+87B9.0	fUTF8	螹
U+87B9.0	fCNS11643	2-5F2C
U+87B9.0	fBig5	EE5E
U+87B9.0	fGB	5-6155
U+87B9.0	fR/S	 142.11
U+87B9.0	fTotalStrokes	 17
U+87B9.0	fCangjie	LIJJL
U+87B9.0	fFourCorner	52121
U+87B9.0	fCantonese	zim6; caam4
U+87B9.0	fMandarin	jian4; qian2; chan2
U+87BA.0	fUTF8	螺
U+87BA.0	fCNS11643	1-753E
U+87BA.0	fBig5	C1B3
U+87BA.0	fGB	0-425D
U+87BA.0	fR/S	 142.11
U+87BA.0	fTotalStrokes	 17
U+87BA.0	fCangjie	LIWVF
U+87BA.0	fFourCorner	56193
U+87BA.0	fHakka	  lo2; lo3
U+87BA.0	fCantonese	lo4; lo2 (also)
U+87BA.0	fMandarin	luo2
U+87BA.0	fEnglish	[1] spiral shell; conch [2] spiral [3] an alias for wine cups
U+87BB.0	fUTF8	螻
U+87BB.0	fCNS11643	1-753D
U+87BB.0	fBig5	C1B2
U+87BB.0	fGB	1-7277
U+87BB.0	fR/S	 142.11
U+87BB.0	fTotalStrokes	 17
U+87BB.0	fCangjie	LILLV
U+87BB.0	fFourCorner	55144
U+87BB.0	fHakka	 leu2; liu2; leu1
U+87BB.0	fCantonese	lau4
U+87BB.0	fMandarin	lou2; lou4
U+87BB.0	fEnglish	Gryllotalpa africana, the mole cricket
U+87BC.0	fUTF8	螼
U+87BC.0	fCNS11643	2-5F2E
U+87BC.0	fBig5	EE60
U+87BC.0	fGB	5-622A
U+87BC.0	fR/S	 142.11
U+87BC.0	fTotalStrokes	 17
U+87BC.0	fCangjie	LITLM
U+87BC.0	fFourCorner	54114
U+87BC.0	fCantonese	jan2; gan3
U+87BC.0	fMandarin	qin3; qiu3
U+87BD.0	fUTF8	螽
U+87BD.0	fCNS11643	2-5F3C
U+87BD.0	fBig5	EE6E
U+87BD.0	fGB	0-732E
U+87BD.0	fR/S	 142.11
U+87BD.0	fTotalStrokes	 17
U+87BD.0	fCangjie	HEYLI
U+87BD.0	fFourCorner	27136
U+87BD.0	fHakka	 zung1;  zhung1; chung3
U+87BD.0	fCantonese	zung1
U+87BD.0	fMandarin	zhong1
U+87BD.0	fEnglish	 katydid, a long-horned grasshopper; locust
U+87BE.0	fUTF8	螾
U+87BE.0	fCNS11643	2-5F26
U+87BE.0	fBig5	EE58
U+87BE.0	fGB	3-6276
U+87BE.0	fR/S	 142.11
U+87BE.0	fTotalStrokes	 17
U+87BE.0	fCangjie	LIJMC
U+87BE.0	fFourCorner	53186
U+87BE.0	fHakka	 jin3; ri2; ji2; rin3
U+87BE.0	fCantonese	jan5
U+87BE.0	fMandarin	yin3; yin2; yin4
U+87BE.0	fEnglish	the earthworm
U+87BF.0	fUTF8	螿
U+87BF.0	fCNS11643	2-5F3A
U+87BF.0	fBig5	EE6C
U+87BF.0	fGB	3-6258
U+87BF.0	fR/S	 142.11
U+87BF.0	fTotalStrokes	 17
U+87BF.0	fCangjie	VILMI
U+87BF.0	fFourCorner	27136
U+87BF.0	fHakka	  ziong1
U+87BF.0	fCantonese	zoeng1
U+87BF.0	fMandarin	jiang1
U+87BF.0	fEnglish	a kind of cicada
U+87C0.0	fUTF8	蟀
U+87C0.0	fCNS11643	1-7537
U+87C0.0	fBig5	C1AC
U+87C0.0	fGB	0-7330
U+87C0.0	fR/S	 142.11
U+87C0.0	fTotalStrokes	 17
U+87C0.0	fCangjie	LIYIJ
U+87C0.0	fFourCorner	50143
U+87C0.0	fHakka	  sut7
U+87C0.0	fCantonese	seot1
U+87C0.0	fMandarin	shuai4; shuo4
U+87C0.0	fEnglish	used in combination for cricket
U+87C1.0	fUTF8	蟁
U+87C1.0	fCNS11643	3-5639
U+87C1.0	fGB	E-3C57
U+87C1.0	fR/S	 142.11
U+87C1.0	fTotalStrokes	 17
U+87C1.0	fCangjie	RPLII
U+87C1.0	fFourCorner	77130
U+87C1.0	fHakka	  mun1
U+87C1.0	fCantonese	man1; man4 (also)
U+87C1.0	fMandarin	wen2
U+87C1.0	fEnglish	a mosquite; a gnat
U+87C2.0	fUTF8	蟂
U+87C2.0	fCNS11643	2-5F32
U+87C2.0	fBig5	EE64
U+87C2.0	fGB	3-6246
U+87C2.0	fR/S	 142.11
U+87C2.0	fTotalStrokes	 17
U+87C2.0	fCangjie	LIHAD
U+87C2.0	fFourCorner	57127
U+87C2.0	fCantonese	giu1
U+87C2.0	fMandarin	jiao1
U+87C3.0	fUTF8	蟃
U+87C3.0	fCNS11643	2-5F31
U+87C3.0	fBig5	EE63
U+87C3.0	fGB	3-6271
U+87C3.0	fR/S	 142.11
U+87C3.0	fTotalStrokes	 17
U+87C3.0	fCangjie	LIAWE
U+87C3.0	fFourCorner	57147
U+87C3.0	fCantonese	maan6
U+87C3.0	fMandarin	wan4
U+87C4.0	fUTF8	蟄
U+87C4.0	fCNS11643	2-5F36
U+87C4.0	fBig5	EE68
U+87C4.0	fGB	1-555D
U+87C4.0	fR/S	 142.11
U+87C4.0	fTotalStrokes	 17
U+87C4.0	fCangjie	GILMI
U+87C4.0	fFourCorner	45136
U+87C4.0	fHakka	 cit8; chit8; zhi5; ts'iit8; zih5
U+87C4.0	fCantonese	zat6; zik6 (also); zit3 (also); zaap6
U+87C4.0	fMandarin	zhe2; zhi2; chi4
U+87C4.0	fEnglish	 hibernate; hibernation
U+87C5.0	fUTF8	蟅
U+87C5.0	fCNS11643	2-5F29
U+87C5.0	fBig5	EE5B
U+87C5.0	fGB	3-6272
U+87C5.0	fR/S	 142.11
U+87C5.0	fTotalStrokes	 17
U+87C5.0	fCangjie	LIITF
U+87C5.0	fFourCorner	50137
U+87C5.0	fHakka	 zha5
U+87C5.0	fCantonese	ze3
U+87C5.0	fMandarin	zhe4; zhi2
U+87C6.0	fUTF8	蟆
U+87C6.0	fCNS11643	1-753B
U+87C6.0	fBig5	C1B0
U+87C6.0	fGB	0-7321
U+87C6.0	fR/S	 142.11
U+87C6.0	fTotalStrokes	 17
U+87C6.0	fCangjie	LITAK
U+87C6.0	fFourCorner	44136
U+87C6.0	fHakka	  ma2; mok8; ma3
U+87C6.0	fCantonese	maa4; mou1 (also)
U+87C6.0	fMandarin	ma2; mo4; ma5
U+87C6.0	fEnglish	used in combination for toad
U+87C7.0	fUTF8	蟇
U+87C7.0	fCNS11643	3-563B
U+87C7.0	fGB	E-3C58
U+87C7.0	fR/S	 142.11; 142.13; 140.13
U+87C7.0	fTotalStrokes	 17
U+87C7.0	fCangjie	TAKI
U+87C7.0	fFourCorner	44130
U+87C7.0	fHakka	 ma2
U+87C7.0	fCantonese	maa4; mou1 (also)
U+87C7.0	fMandarin	ma2
U+87C7.0	fEnglish	frog; toad
U+87C8.0	fUTF8	蟈
U+87C8.0	fCNS11643	1-753F
U+87C8.0	fBig5	C1B4
U+87C8.0	fGB	1-7265
U+87C8.0	fR/S	 142.11
U+87C8.0	fTotalStrokes	 17
U+87C8.0	fCangjie	LIWIM
U+87C8.0	fFourCorner	56100
U+87C8.0	fHakka	 kwet7; kwet8; get7
U+87C8.0	fCantonese	gwok3; gok3 (also)
U+87C8.0	fMandarin	guo1; yu4
U+87C8.0	fEnglish	[1] a small green frog [2] mole cricket
U+87C9.0	fUTF8	蟉
U+87C9.0	fCNS11643	2-5F30
U+87C9.0	fBig5	EE62
U+87C9.0	fGB	3-6279
U+87C9.0	fR/S	 142.11
U+87C9.0	fTotalStrokes	 17
U+87C9.0	fCangjie	LISMH
U+87C9.0	fFourCorner	57122
U+87C9.0	fHakka	 liu2; liau6; liau5
U+87C9.0	fCantonese	lau4
U+87C9.0	fMandarin	liao4; liu2; jiu4; liu3; qiu2
U+87C9.0	fEnglish	wriggling movement of a snake
U+87CA.0	fUTF8	蟊
U+87CA.0	fCNS11643	2-5F37
U+87CA.0	fBig5	EE69
U+87CA.0	fGB	0-7331
U+87CA.0	fR/S	 142.11
U+87CA.0	fTotalStrokes	 17
U+87CA.0	fCangjie	NHLII
U+87CA.0	fFourCorner	17136
U+87CA.0	fHakka	  mau2
U+87CA.0	fCantonese	maau4; mau4
U+87CA.0	fMandarin	mao2; meng2; mao1; mou2
U+87CA.0	fEnglish	insects that are injurious to crops; devourers of the grain
U+87CB.0	fUTF8	蟋
U+87CB.0	fCNS11643	1-7540
U+87CB.0	fBig5	C1B5
U+87CB.0	fGB	0-732C
U+87CB.0	fR/S	 142.11
U+87CB.0	fTotalStrokes	 17
U+87CB.0	fCangjie	LIHDP
U+87CB.0	fFourCorner	52139
U+87CB.0	fHakka	  sit7; zit7
U+87CB.0	fCantonese	sik1
U+87CB.0	fMandarin	xi1; se4
U+87CB.0	fEnglish	 cricket
U+87CC.0	fUTF8	蟌
U+87CC.0	fCNS11643	2-5F33
U+87CC.0	fBig5	EE65
U+87CC.0	fGB	5-622D
U+87CC.0	fR/S	 142.11
U+87CC.0	fTotalStrokes	 17
U+87CC.0	fCangjie	LIHWP
U+87CC.0	fFourCorner	56130
U+87CC.0	fCantonese	cung1
U+87CC.0	fMandarin	cong1
U+87CD.0	fUTF8	蟍
U+87CD.0	fCNS11643	3-5636
U+87CD.0	fGB	E-3C59
U+87CD.0	fR/S	  142.11
U+87CD.0	fTotalStrokes	 17
U+87CD.0	fCangjie	LIHND
U+87CD.0	fFourCorner	52190
U+87CD.0	fCantonese	lei4
U+87CD.0	fMandarin	li2
U+87CE.0	fUTF8	蟎
U+87CE.0	fCNS11643	3-5634
U+87CE.0	fGB	1-727D
U+87CE.0	fR/S	 142.11
U+87CE.0	fTotalStrokes	 17
U+87CE.0	fCangjie	LITLB
U+87CE.0	fFourCorner	54120
U+87CE.0	fCantonese	mun5
U+87CE.0	fMandarin	man3
U+87CF.0	fUTF8	蟏
U+87CF.0	fGB	8-7D7E
U+87CF.0	fR/S	 142.11
U+87CF.0	fTotalStrokes	 17
U+87CF.0	fCangjie	XLITL
U+87CF.0	fCantonese	siu1
U+87CF.0	fMandarin	xiao1
U+87CF.0	fEnglish	a kind of spider with long legs
U+87D0.0	fUTF8	蟐
U+87D0.0	fGB	E-3C5A
U+87D0.0	fR/S	 142.11
U+87D0.0	fTotalStrokes	 17
U+87D0.0	fCangjie	LIFBB
U+87D0.0	fCantonese	kei4
U+87D1.0	fUTF8	蟑
U+87D1.0	fCNS11643	1-7538
U+87D1.0	fBig5	C1AD
U+87D1.0	fGB	0-732F
U+87D1.0	fR/S	 142.11
U+87D1.0	fTotalStrokes	 17
U+87D1.0	fCangjie	LIYTJ
U+87D1.0	fFourCorner	50146
U+87D1.0	fHakka	 zong1
U+87D1.0	fCantonese	zoeng1
U+87D1.0	fMandarin	zhang1
U+87D1.0	fEnglish	 cockroach; roach
U+87D2.0	fUTF8	蟒
U+87D2.0	fCNS11643	1-753A
U+87D2.0	fBig5	C1AF
U+87D2.0	fGB	0-727E
U+87D2.0	fR/S	 142.13
U+87D2.0	fTotalStrokes	 19
U+87D2.0	fCangjie	LITIT
U+87D2.0	fFourCorner	54143
U+87D2.0	fHakka	  mang3; mang1; mong3
U+87D2.0	fCantonese	mong5
U+87D2.0	fMandarin	mang3; meng3
U+87D2.0	fEnglish	[1] python; Boa constrictor [2] ceremonial robes worn by mandarins or the embroidered patterns on such robes
U+87D3.0	fUTF8	蟓
U+87D3.0	fCNS11643	2-6333
U+87D3.0	fBig5	F0C7
U+87D3.0	fGB	0-732D
U+87D3.0	fR/S	 142.12
U+87D3.0	fTotalStrokes	 18
U+87D3.0	fCangjie	LINAO
U+87D3.0	fFourCorner	57132
U+87D3.0	fCantonese	zoeng6
U+87D3.0	fMandarin	xiang4; xiang3
U+87D4.0	fUTF8	蟔
U+87D4.0	fCNS11643	2-6331
U+87D4.0	fBig5	F0C5
U+87D4.0	fGB	3-6323
U+87D4.0	fR/S	 142.12
U+87D4.0	fTotalStrokes	 18
U+87D4.0	fCangjie	LIWGF
U+87D4.0	fFourCorner	56131
U+87D4.0	fCantonese	mat6
U+87D4.0	fMandarin	mo4
U+87D5.0	fUTF8	蟕
U+87D5.0	fCNS11643	3-5B28
U+87D5.0	fGB	5-6240
U+87D5.0	fR/S	 142.12; 142.13
U+87D5.0	fTotalStrokes	 19; 18
U+87D5.0	fCangjie	LIYPB
U+87D5.0	fFourCorner	51120
U+87D5.0	fCantonese	zai1
U+87D5.0	fMandarin	zui1
U+87D5.0	fEnglish	a species of tortoise found near the mouth of the Yellow River
U+87D6.0	fUTF8	蟖
U+87D6.0	fCNS11643	3-5876
U+87D6.0	fGB	5-6236
U+87D6.0	fR/S	 142.12
U+87D6.0	fTotalStrokes	 18
U+87D6.0	fCangjie	LITCL
U+87D6.0	fFourCorner	52120
U+87D6.0	fCantonese	si1
U+87D6.0	fMandarin	si1
U+87D6.0	fEnglish	the larva of a month; a grasshopper
U+87D7.0	fUTF8	蟗
U+87D7.0	fCNS11643	2-6337
U+87D7.0	fBig5	F0CC
U+87D7.0	fGB	5-6238
U+87D7.0	fR/S	 142.10; 142.12
U+87D7.0	fTotalStrokes	 16
U+87D7.0	fCangjie	JBUUI
U+87D7.0	fFourCorner	40136
U+87D7.0	fCantonese	cau1
U+87D7.0	fMandarin	qiu1
U+87D8.0	fUTF8	蟘
U+87D8.0	fCNS11643	2-6335
U+87D8.0	fBig5	F0C9
U+87D8.0	fGB	5-6171
U+87D8.0	fR/S	 142.12
U+87D8.0	fTotalStrokes	 18
U+87D8.0	fCangjie	LIOPC
U+87D8.0	fFourCorner	53186
U+87D8.0	fHakka	 tit8
U+87D8.0	fCantonese	dak6; dat6 (also); dik6
U+87D8.0	fMandarin	te4
U+87D8.0	fEnglish	grubs which eat leaves
U+87D9.0	fUTF8	蟙
U+87D9.0	fCNS11643	2-6338
U+87D9.0	fBig5	F0CD
U+87D9.0	fGB	3-6170
U+87D9.0	fR/S	 142.12
U+87D9.0	fTotalStrokes	 18
U+87D9.0	fCangjie	LIYIA
U+87D9.0	fFourCorner	53150
U+87D9.0	fCantonese	zik1
U+87D9.0	fMandarin	zhi2
U+87DA.0	fUTF8	蟚
U+87DA.0	fCNS11643	3-5878
U+87DA.0	fGB	5-6235
U+87DA.0	fR/S	 142.12
U+87DA.0	fTotalStrokes	 18
U+87DA.0	fCangjie	GHLMI
U+87DA.0	fFourCorner	42130
U+87DA.0	fHakka	  pang2
U+87DA.0	fCantonese	paang4; pang4
U+87DA.0	fMandarin	peng2
U+87DA.0	fEnglish	a land-crab
U+87DB.0	fUTF8	蟛
U+87DB.0	fCNS11643	2-632A
U+87DB.0	fBig5	F0BE
U+87DB.0	fGB	0-7332
U+87DB.0	fR/S	 142.12
U+87DB.0	fTotalStrokes	 18
U+87DB.0	fCangjie	LIGTH
U+87DB.0	fFourCorner	52122
U+87DB.0	fHakka	 pang2
U+87DB.0	fCantonese	paang4
U+87DB.0	fMandarin	peng2
U+87DB.0	fEnglish	a land-crab
U+87DC.0	fUTF8	蟜
U+87DC.0	fCNS11643	2-6332
U+87DC.0	fBig5	F0C6
U+87DC.0	fGB	3-6227
U+87DC.0	fR/S	 142.12
U+87DC.0	fTotalStrokes	 18
U+87DC.0	fCangjie	LIHKB
U+87DC.0	fFourCorner	52127
U+87DC.0	fHakka	  giau3; geu3
U+87DC.0	fCantonese	giu2
U+87DC.0	fMandarin	jiao3
U+87DC.0	fEnglish	insect
U+87DD.0	fUTF8	蟝
U+87DD.0	fCNS11643	2-633C
U+87DD.0	fBig5	F0D1
U+87DD.0	fGB	5-6230
U+87DD.0	fR/S	 142.12
U+87DD.0	fTotalStrokes	 18
U+87DD.0	fCangjie	LIESD
U+87DD.0	fFourCorner	51194
U+87DD.0	fCantonese	keoi4; keoi2 (also)
U+87DD.0	fMandarin	qu2
U+87DD.0	fEnglish	蠄蟝, a dragonfly
U+87DE.0	fUTF8	蟞
U+87DE.0	fCNS11643	2-5F3D
U+87DE.0	fBig5	EE6F
U+87DE.0	fGB	3-6270
U+87DE.0	fR/S	 142.12
U+87DE.0	fTotalStrokes	 18
U+87DE.0	fCangjie	FKLMI
U+87DE.0	fFourCorner	97136
U+87DE.0	fHakka	 biet7; piet7
U+87DE.0	fCantonese	bit3; bit6
U+87DE.0	fMandarin	bie2; bie4; pie1
U+87DE.0	fEnglish	a large reddish black ant
U+87DF.0	fUTF8	蟟
U+87DF.0	fCNS11643	2-632E
U+87DF.0	fBig5	F0C2
U+87DF.0	fGB	3-6321
U+87DF.0	fR/S	 142.12
U+87DF.0	fTotalStrokes	 18
U+87DF.0	fCangjie	LIKCF
U+87DF.0	fFourCorner	54196
U+87DF.0	fHakka	 liau2
U+87DF.0	fCantonese	liu4
U+87DF.0	fMandarin	liao2; liao3
U+87DF.0	fEnglish	used in combination for Pomponia muculaticollis, a kind of cicada
U+87E0.0	fUTF8	蟠
U+87E0.0	fCNS11643	1-773A
U+87E0.0	fBig5	C2CF
U+87E0.0	fGB	0-7334
U+87E0.0	fR/S	 142.12
U+87E0.0	fTotalStrokes	 18
U+87E0.0	fCangjie	LIHDW
U+87E0.0	fFourCorner	52169
U+87E0.0	fHakka	 pan2; fan1; pan1; fan2
U+87E0.0	fCantonese	pun4; faan4
U+87E0.0	fMandarin	pan2; fan2
U+87E0.0	fEnglish	[1] coil; coiling; curlup [2] occupy
U+87E1.0	fUTF8	蟡
U+87E1.0	fCNS11643	2-536C
U+87E1.0	fBig5	E7A5
U+87E1.0	fGB	5-6075
U+87E1.0	fR/S	 142.9
U+87E1.0	fTotalStrokes	 15
U+87E1.0	fCangjie	LIIKF
U+87E1.0	fFourCorner	52127
U+87E1.0	fCantonese	wai1
U+87E1.0	fMandarin	gui3; wei1
U+87E2.0	fUTF8	蟢
U+87E2.0	fCNS11643	2-6329
U+87E2.0	fBig5	F0BD
U+87E2.0	fGB	3-627B
U+87E2.0	fR/S	 142.12
U+87E2.0	fTotalStrokes	 18
U+87E2.0	fCangjie	LIGRR
U+87E2.0	fFourCorner	54161
U+87E2.0	fHakka	 hi3; shan5
U+87E2.0	fCantonese	hei2
U+87E2.0	fMandarin	xi3
U+87E2.0	fEnglish	a kind of long-bodied and long-legged spider
U+87E3.0	fUTF8	蟣
U+87E3.0	fCNS11643	2-6336
U+87E3.0	fBig5	F0CA
U+87E3.0	fGB	1-7231
U+87E3.0	fR/S	 142.12
U+87E3.0	fTotalStrokes	 18
U+87E3.0	fCangjie	LIVII
U+87E3.0	fFourCorner	52153
U+87E3.0	fHakka	 gi1
U+87E3.0	fCantonese	gei2; kei4
U+87E3.0	fMandarin	ji3; qi2; ji1
U+87E3.0	fEnglish	[1] larvae of lice [2] leeches
U+87E4.0	fUTF8	蟤
U+87E4.0	fCNS11643	2-6330
U+87E4.0	fBig5	F0C4
U+87E4.0	fGB	3-6326
U+87E4.0	fR/S	 142.12
U+87E4.0	fTotalStrokes	 18
U+87E4.0	fCangjie	LIRUC
U+87E4.0	fFourCorner	57181
U+87E4.0	fCantonese	zyun1; cyun4
U+87E4.0	fMandarin	zhuan1; quan2
U+87E5.0	fUTF8	蟥
U+87E5.0	fCNS11643	2-632D
U+87E5.0	fBig5	F0C1
U+87E5.0	fGB	0-7328
U+87E5.0	fR/S	 142.12
U+87E5.0	fTotalStrokes	 18
U+87E5.0	fCangjie	LITLC
U+87E5.0	fFourCorner	54186
U+87E5.0	fHakka	 wong2; wong3; vong2
U+87E5.0	fCantonese	wong4
U+87E5.0	fMandarin	huang2
U+87E5.0	fEnglish	used in combination for horseleech
U+87E6.0	fUTF8	蟦
U+87E6.0	fCNS11643	2-6328
U+87E6.0	fBig5	F0BC
U+87E6.0	fGB	3-624B
U+87E6.0	fR/S	 142.12
U+87E6.0	fTotalStrokes	 18
U+87E6.0	fCangjie	LIJTC
U+87E6.0	fFourCorner	54186
U+87E6.0	fHakka	 pun1; pui2; pi2
U+87E6.0	fCantonese	fei4; ban1; pun3
U+87E6.0	fMandarin	fei4; ben1; fei2
U+87E6.0	fEnglish	the dung-beetle
U+87E7.0	fUTF8	蟧
U+87E7.0	fCNS11643	2-6327
U+87E7.0	fBig5	F0BB
U+87E7.0	fGB	3-6235
U+87E7.0	fR/S	 142.12
U+87E7.0	fTotalStrokes	 18
U+87E7.0	fCangjie	LIFFS
U+87E7.0	fFourCorner	59127
U+87E7.0	fHakka	 lau2; lo2; lo3; liau2
U+87E7.0	fCantonese	lou4; lou2 (also)
U+87E7.0	fMandarin	lao2; liao2
U+87E7.0	fEnglish	a univalve shell-fish
U+87E8.0	fUTF8	蟨
U+87E8.0	fCNS11643	2-633B
U+87E8.0	fBig5	F0D0
U+87E8.0	fGB	5-3071
U+87E8.0	fR/S	 142.12
U+87E8.0	fTotalStrokes	 18
U+87E8.0	fCangjie	MOLMI
U+87E8.0	fFourCorner	71136
U+87E8.0	fHakka	 kiet7; ket7
U+87E8.0	fCantonese	kyut3; kyut6
U+87E8.0	fMandarin	jue2
U+87E8.0	fEnglish	the Siberian jerboa
U+87E9.0	fUTF8	蟩
U+87E9.0	fCNS11643	3-5873
U+87E9.0	fGB	3-627E
U+87E9.0	fR/S	  142.12
U+87E9.0	fTotalStrokes	 18
U+87E9.0	fCangjie	LIMTO
U+87E9.0	fFourCorner	51180
U+87E9.0	fCantonese	kyut3
U+87E9.0	fMandarin	jue2
U+87EA.0	fUTF8	蟪
U+87EA.0	fCNS11643	2-632C
U+87EA.0	fBig5	F0C0
U+87EA.0	fGB	0-7333
U+87EA.0	fR/S	 142.12
U+87EA.0	fTotalStrokes	 18
U+87EA.0	fCangjie	LIJIP
U+87EA.0	fFourCorner	55133
U+87EA.0	fHakka	 fui5; fi5
U+87EA.0	fCantonese	wai6
U+87EA.0	fMandarin	hui4
U+87EA.0	fEnglish	Platypleura haempferi, a kind of bright-colored cicada
U+87EB.0	fUTF8	蟫
U+87EB.0	fCNS11643	2-632B
U+87EB.0	fBig5	F0BF
U+87EB.0	fGB	3-627D
U+87EB.0	fR/S	 142.12
U+87EB.0	fTotalStrokes	 18
U+87EB.0	fCangjie	LIMWJ
U+87EB.0	fFourCorner	51146
U+87EB.0	fHakka	 jim1; rim1; jim2; tam2
U+87EB.0	fCantonese	taam4 (also); jam4; cam4
U+87EB.0	fMandarin	yin2; tan2; tan3; xun2; xin2
U+87EB.0	fEnglish	the insect; commonly known as the silver-fish; which destroys books and clothing; etc.
U+87EC.0	fUTF8	蟬
U+87EC.0	fCNS11643	1-7738
U+87EC.0	fBig5	C2CD
U+87EC.0	fGB	1-3275
U+87EC.0	fR/S	 142.12
U+87EC.0	fTotalStrokes	 18
U+87EC.0	fCangjie	LIRRJ
U+87EC.0	fFourCorner	56156
U+87EC.0	fHakka	  sam2;  sham2; siam2; sham3; san1; san2; sen2
U+87EC.0	fCantonese	sim4; sin4 (also); sim2 (also)
U+87EC.0	fMandarin	chan2; shan4; qian2; ti2
U+87EC.0	fEnglish	[1] cicada [2] continuous; uninterrupted
U+87ED.0	fUTF8	蟭
U+87ED.0	fCNS11643	2-6334
U+87ED.0	fBig5	F0C8
U+87ED.0	fGB	5-623A
U+87ED.0	fR/S	 142.12
U+87ED.0	fTotalStrokes	 18
U+87ED.0	fCangjie	LIOGF
U+87ED.0	fFourCorner	50131
U+87ED.0	fCantonese	ziu1
U+87ED.0	fMandarin	jiao1; jiu1
U+87ED.0	fEnglish	very small bug
U+87EE.0	fUTF8	蟮
U+87EE.0	fCNS11643	3-5874
U+87EE.0	fGB	0-7335
U+87EE.0	fR/S	 142.12
U+87EE.0	fTotalStrokes	 18
U+87EE.0	fCangjie	LITTR
U+87EE.0	fFourCorner	58160
U+87EE.0	fHakka	 san3; sen3; shen3; shen5
U+87EE.0	fCantonese	sin6
U+87EE.0	fMandarin	shan4
U+87EE.0	fEnglish	 earthworm
U+87EF.0	fUTF8	蟯
U+87EF.0	fCNS11643	1-7737
U+87EF.0	fBig5	C2CC
U+87EF.0	fGB	1-724D
U+87EF.0	fR/S	 142.12
U+87EF.0	fTotalStrokes	 18
U+87EF.0	fCangjie	LIGGU
U+87EF.0	fFourCorner	54111
U+87EF.0	fHakka	 ngiau2; ngieu2
U+87EF.0	fCantonese	jiu4; naau4 (also); laau4 (also)
U+87EF.0	fMandarin	nao2; rao2; rao4; yao1; yao2
U+87EF.0	fEnglish	worms
U+87F0.0	fUTF8	蟰
U+87F0.0	fCNS11643	3-5875
U+87F0.0	fGB	5-6161
U+87F0.0	fR/S	  142.12
U+87F0.0	fTotalStrokes	 18
U+87F0.0	fCangjie	LILX
U+87F0.0	fFourCorner	55120
U+87F0.0	fHakka	 siau1
U+87F0.0	fCantonese	siu1
U+87F0.0	fMandarin	xiao1
U+87F0.0	fEnglish	small spider with long legs
U+87F1.0	fUTF8	蟱
U+87F1.0	fCNS11643	5-6448
U+87F1.0	fGB	E-3C5B
U+87F1.0	fR/S	  142.12
U+87F1.0	fTotalStrokes	 18
U+87F1.0	fCangjie	LIOTF
U+87F1.0	fFourCorner	58130
U+87F1.0	fCantonese	mou4
U+87F1.0	fMandarin	mou2; wu2
U+87F2.0	fUTF8	蟲
U+87F2.0	fCNS11643	1-7739
U+87F2.0	fBig5	C2CE
U+87F2.0	fGB	1-3366
U+87F2.0	fR/S	 142.12
U+87F2.0	fTotalStrokes	 18
U+87F2.0	fCangjie	LILII
U+87F2.0	fFourCorner	50136
U+87F2.0	fHakka	 cung2;  chung2
U+87F2.0	fCantonese	cung4; cung2 (also)
U+87F2.0	fMandarin	chong2; tong2; zhong4
U+87F2.0	fEnglish	 worms; insects
U+87F3.0	fUTF8	蟳
U+87F3.0	fCNS11643	2-632F
U+87F3.0	fBig5	F0C3
U+87F3.0	fGB	3-622F
U+87F3.0	fR/S	 142.12
U+87F3.0	fTotalStrokes	 18
U+87F3.0	fCangjie	LISMI
U+87F3.0	fFourCorner	57146
U+87F3.0	fHakka	 cim2; zim2
U+87F3.0	fCantonese	cing4; cam4
U+87F3.0	fMandarin	xun2
U+87F3.0	fEnglish	a kind of crab
U+87F4.0	fUTF8	蟴
U+87F4.0	fCNS11643	2-633A
U+87F4.0	fBig5	F0CF
U+87F4.0	fGB	3-627C
U+87F4.0	fR/S	 142.12
U+87F4.0	fTotalStrokes	 18
U+87F4.0	fCangjie	TLLMI
U+87F4.0	fFourCorner	42136
U+87F4.0	fCantonese	si1
U+87F4.0	fMandarin	si1
U+87F5.0	fUTF8	蟵
U+87F5.0	fCNS11643	3-6646
U+87F5.0	fGB	E-3C5C
U+87F5.0	fR/S	 142.12
U+87F5.0	fTotalStrokes	 18
U+87F5.0	fCangjie	LIMMI
U+87F5.0	fFourCorner	5114
U+87F5.0	fCantonese	cyu4
U+87F6.0	fUTF8	蟶
U+87F6.0	fCNS11643	2-6669
U+87F6.0	fBig5	F2DE
U+87F6.0	fGB	1-7249
U+87F6.0	fR/S	 142.13
U+87F6.0	fTotalStrokes	 19
U+87F6.0	fCangjie	LISRG
U+87F6.0	fFourCorner	56114
U+87F6.0	fHakka	 shin5; siin5; sin5
U+87F6.0	fCantonese	cing1
U+87F6.0	fMandarin	cheng1
U+87F6.0	fEnglish	 razor clam; razor shell
U+87F7.0	fUTF8	蟷
U+87F7.0	fCNS11643	2-666A
U+87F7.0	fBig5	F2DF
U+87F7.0	fGB	3-6226
U+87F7.0	fR/S	 142.13
U+87F7.0	fTotalStrokes	 19
U+87F7.0	fCangjie	LIFBW
U+87F7.0	fFourCorner	59166
U+87F7.0	fCantonese	dong1
U+87F7.0	fMandarin	dang1; tang2
U+87F7.0	fEnglish	mantis
U+87F8.0	fUTF8	蟸
U+87F8.0	fCNS11643	3-5B2C
U+87F8.0	fGB	E-3C5D
U+87F8.0	fR/S	 142.13
U+87F8.0	fTotalStrokes	 19
U+87F8.0	fCangjie	MOLII
U+87F8.0	fFourCorner	10130
U+87F8.0	fCantonese	lai5
U+87F8.0	fMandarin	li3; li2; luo3; luo2; li4
U+87F9.0	fUTF8	蟹
U+87F9.0	fCNS11643	1-7874
U+87F9.0	fBig5	C3C9
U+87F9.0	fGB	0-5037
U+87F9.0	fR/S	 142.13
U+87F9.0	fTotalStrokes	 19
U+87F9.0	fCangjie	NQLMI
U+87F9.0	fFourCorner	27136
U+87F9.0	fHakka	  hai3
U+87F9.0	fCantonese	haai5
U+87F9.0	fMandarin	xie4; xie3
U+87F9.0	fEnglish	 crab; brachyura
U+87FA.0	fUTF8	蟺
U+87FA.0	fCNS11643	2-6667
U+87FA.0	fBig5	F2DC
U+87FA.0	fGB	3-6329
U+87FA.0	fR/S	 142.13
U+87FA.0	fTotalStrokes	 19
U+87FA.0	fCangjie	LIYWM
U+87FA.0	fFourCorner	50116
U+87FA.0	fHakka	 shan5; sen5; shen3
U+87FA.0	fCantonese	sin6; sin5
U+87FA.0	fMandarin	shan4; chan2; dan4; tuo2
U+87FA.0	fEnglish	 earthworm
U+87FB.0	fUTF8	蟻
U+87FB.0	fCNS11643	1-7871
U+87FB.0	fBig5	C3C6
U+87FB.0	fGB	1-524F
U+87FB.0	fR/S	 142.13
U+87FB.0	fTotalStrokes	 19
U+87FB.0	fCangjie	LITGI
U+87FB.0	fFourCorner	58153
U+87FB.0	fHakka	 ngie5;  ngi5; ngi3;  ni1; gni1; gne5; li1
U+87FB.0	fCantonese	ngai5; ai5 (also)
U+87FB.0	fMandarin	yi3; ji3
U+87FB.0	fEnglish	 ant
U+87FC.0	fUTF8	蟼
U+87FC.0	fCNS11643	2-666F
U+87FC.0	fBig5	F2E4
U+87FC.0	fGB	5-6237
U+87FC.0	fR/S	 142.13
U+87FC.0	fTotalStrokes	 19
U+87FC.0	fCangjie	TKLMI
U+87FC.0	fFourCorner	48136
U+87FC.0	fCantonese	ging2
U+87FC.0	fMandarin	jing3; jing1
U+87FD.0	fUTF8	蟽
U+87FD.0	fCNS11643	3-5B27
U+87FD.0	fGB	E-3C5E
U+87FD.0	fR/S	  142.13
U+87FD.0	fTotalStrokes	 19
U+87FD.0	fCangjie	LIYGQ
U+87FD.0	fFourCorner	54130
U+87FD.0	fCantonese	daat6
U+87FD.0	fMandarin	da2
U+87FE.0	fUTF8	蟾
U+87FE.0	fCNS11643	1-7875
U+87FE.0	fBig5	C3CA
U+87FE.0	fGB	0-7338
U+87FE.0	fR/S	 142.13
U+87FE.0	fTotalStrokes	 19
U+87FE.0	fCangjie	LINCR
U+87FE.0	fFourCorner	57161
U+87FE.0	fHakka	 sham2; sam2; cam2; sham3; shan5; san5
U+87FE.0	fCantonese	sim4
U+87FE.0	fMandarin	chan2; zhan1
U+87FE.0	fEnglish	 toad (especially the one supposed to live on the moon)
U+87FF.0	fUTF8	蟿
U+87FF.0	fCNS11643	2-6671
U+87FF.0	fBig5	F2E6
U+87FF.0	fGB	E-3C5F
U+87FF.0	fR/S	 142.13
U+87FF.0	fTotalStrokes	 19
U+87FF.0	fCangjie	JELMI
U+87FF.0	fFourCorner	47136
U+87FF.0	fCantonese	kai3; gai3
U+87FF.0	fMandarin	qi4; ji4
U+87FF.0	fEnglish	Acrida chinensis
U+8800.0	fUTF8	蠀
U+8800.0	fCNS11643	2-6666
U+8800.0	fBig5	F2DB
U+8800.0	fGB	3-6267
U+8800.0	fR/S	 142.13
U+8800.0	fTotalStrokes	 19
U+8800.0	fCangjie	LIIOC
U+8800.0	fFourCorner	57186
U+8800.0	fCantonese	ci1; zik1
U+8800.0	fMandarin	ci1; zi1; ci2; ji2
U+8801.0	fUTF8	蠁
U+8801.0	fCNS11643	2-6339
U+8801.0	fBig5	F0CE
U+8801.0	fGB	3-6157
U+8801.0	fR/S	 142.13
U+8801.0	fTotalStrokes	 19
U+8801.0	fCangjie	VLLMI
U+8801.0	fFourCorner	27136
U+8801.0	fHakka	  hiong3
U+8801.0	fCantonese	hoeng2; hoeng3
U+8801.0	fMandarin	xiang4; xiang3
U+8801.0	fEnglish	larvae; grubs
U+8802.0	fUTF8	蠂
U+8802.0	fCNS11643	2-6673
U+8802.0	fBig5	F2E8
U+8802.0	fGB	E-3C60
U+8802.0	fR/S	 142.13
U+8802.0	fTotalStrokes	 19
U+8802.0	fCangjie	LITPD
U+8802.0	fFourCorner	54194
U+8802.0	fCantonese	sip3
U+8802.0	fMandarin	she4
U+8803.0	fUTF8	蠃
U+8803.0	fCNS11643	2-6668
U+8803.0	fBig5	F2DD
U+8803.0	fGB	0-5979
U+8803.0	fR/S	 142.13
U+8803.0	fTotalStrokes	 19
U+8803.0	fCangjie	YNLMI
U+8803.0	fFourCorner	00217
U+8803.0	fHakka	  lo3; lo2
U+8803.0	fCantonese	lo2; lo4; lo5
U+8803.0	fMandarin	luo3; guo3; luo2
U+8803.0	fEnglish	the solitary wasp
U+8804.0	fUTF8	蠄
U+8804.0	fCNS11643	5-6943
U+8804.0	fGB	1-7C4E
U+8804.0	fR/S	 142.13
U+8804.0	fTotalStrokes	 19
U+8804.0	fCangjie	LIOYB
U+8804.0	fFourCorner	58120
U+8804.0	fCantonese	kam4
U+8804.0	fMandarin	qin2
U+8804.0	fEnglish	蠄蟧, a spider; 蠄蟝, a toad
U+8805.0	fUTF8	蠅
U+8805.0	fCNS11643	1-7872
U+8805.0	fBig5	C3C7
U+8805.0	fGB	1-532C
U+8805.0	fR/S	 142.13
U+8805.0	fTotalStrokes	 19
U+8805.0	fCangjie	LIRXU
U+8805.0	fFourCorner	56116
U+8805.0	fHakka	 jin2; shin2; jin3; rin2; sin2; siin2; {jin2}
U+8805.0	fCantonese	jing4; jing1 (also)
U+8805.0	fMandarin	ying2
U+8805.0	fEnglish	 fly
U+8806.0	fUTF8	蠆
U+8806.0	fCNS11643	2-666E
U+8806.0	fBig5	F2E3
U+8806.0	fGB	1-7232
U+8806.0	fR/S	 142.13
U+8806.0	fTotalStrokes	 19
U+8806.0	fCangjie	TWBI
U+8806.0	fFourCorner	44136
U+8806.0	fHakka	 cai5;  dai5; mai5; mai6; cui5
U+8806.0	fCantonese	caai3
U+8806.0	fMandarin	chai4; ta4
U+8806.0	fEnglish	a kind of scorpion
U+8807.0	fUTF8	蠇
U+8807.0	fCNS11643	3-5B29
U+8807.0	fGB	E-3C61
U+8807.0	fR/S	 142.13
U+8807.0	fTotalStrokes	 19
U+8807.0	fCangjie	LITWB
U+8807.0	fFourCorner	54120
U+8807.0	fHakka	 cai5
U+8807.0	fCantonese	lai6
U+8807.0	fMandarin	li4
U+8808.0	fUTF8	蠈
U+8808.0	fCNS11643	2-6670
U+8808.0	fBig5	F2E5
U+8808.0	fGB	E-3C62
U+8808.0	fR/S	 142.13
U+8808.0	fTotalStrokes	 19
U+8808.0	fCangjie	BJLMI
U+8808.0	fFourCorner	63136
U+8808.0	fHakka	 cet8
U+8808.0	fCantonese	caak3; caak6
U+8808.0	fMandarin	ze2
U+8808.0	fEnglish	the larvae of an insect that eats growing rice
U+8809.0	fUTF8	蠉
U+8809.0	fCNS11643	2-666B
U+8809.0	fBig5	F2E0
U+8809.0	fGB	3-6327
U+8809.0	fR/S	 142.13
U+8809.0	fTotalStrokes	 19
U+8809.0	fCangjie	LIWLV
U+8809.0	fFourCorner	56132
U+8809.0	fHakka	 hien2; fan2
U+8809.0	fCantonese	hyun1
U+8809.0	fMandarin	xuan1; xuan3; xuan4
U+8809.0	fEnglish	~ sung1 t'et7 li1, lost its solidity (like sweets exposed to weather)
U+880A.0	fUTF8	蠊
U+880A.0	fCNS11643	2-6672
U+880A.0	fBig5	F2E7
U+880A.0	fGB	0-7339
U+880A.0	fR/S	 142.13
U+880A.0	fTotalStrokes	 19
U+880A.0	fCangjie	LIITC
U+880A.0	fFourCorner	50137
U+880A.0	fCantonese	lim4
U+880A.0	fMandarin	lian2
U+880B.0	fUTF8	蠋
U+880B.0	fCNS11643	2-666D
U+880B.0	fBig5	F2E2
U+880B.0	fGB	3-6328
U+880B.0	fR/S	 142.13
U+880B.0	fTotalStrokes	 19
U+880B.0	fCangjie	LIWLI
U+880B.0	fFourCorner	56127
U+880B.0	fHakka	 zuk7; chuk7; ch'uk7; cuk7; zhuk7
U+880B.0	fCantonese	zuk1; suk6
U+880B.0	fMandarin	zhu2; shu3
U+880B.0	fEnglish	caterpillar
U+880C.0	fUTF8	蠌
U+880C.0	fCNS11643	2-666C
U+880C.0	fBig5	F2E1
U+880C.0	fGB	3-617C
U+880C.0	fR/S	 142.13
U+880C.0	fTotalStrokes	 19
U+880C.0	fCangjie	LIWLJ
U+880C.0	fFourCorner	56141
U+880C.0	fCantonese	zaak6
U+880C.0	fMandarin	ze2; duo2
U+880D.0	fUTF8	蠍
U+880D.0	fCNS11643	1-7873
U+880D.0	fBig5	C3C8
U+880D.0	fGB	E-3C63
U+880D.0	fR/S	 142.13
U+880D.0	fTotalStrokes	 19
U+880D.0	fCangjie	LIAVO
U+880D.0	fFourCorner	57182
U+880D.0	fHakka	  hiet7; het7; ket7; get7; giet7
U+880D.0	fCantonese	hit3; kit3 (also)
U+880D.0	fMandarin	xie1; xie5
U+880D.0	fEnglish	 scorpion
U+880E.0	fUTF8	蠎
U+880E.0	fCNS11643	3-5877
U+880E.0	fGB	E-3C64
U+880E.0	fR/S	  142.12
U+880E.0	fTotalStrokes	 18
U+880E.0	fCangjie	LITKT
U+880E.0	fFourCorner	54140
U+880E.0	fCantonese	mong5
U+880E.0	fMandarin	mang3; meng3
U+880E.0	fEnglish	python; boa constrictor
U+880F.0	fUTF8	蠏
U+880F.0	fCNS11643	3-5B2B
U+880F.0	fGB	E-3C65
U+880F.0	fR/S	 142.13
U+880F.0	fTotalStrokes	 19
U+880F.0	fCangjie	LINBQ
U+880F.0	fFourCorner	57150
U+880F.0	fHakka	 hai3
U+880F.0	fCantonese	haai5; hai5
U+880F.0	fMandarin	xie4; xie3
U+880F.0	fEnglish	crab; brachyura
U+8810.0	fUTF8	蠐
U+8810.0	fCNS11643	2-6971
U+8810.0	fBig5	F4C5
U+8810.0	fGB	1-7253
U+8810.0	fR/S	 142.14
U+8810.0	fTotalStrokes	 20
U+8810.0	fCangjie	LIYX
U+8810.0	fFourCorner	50123
U+8810.0	fHakka	  ci2; zi1
U+8810.0	fCantonese	cai4
U+8810.0	fMandarin	qi2
U+8810.0	fEnglish	 grub (a kind of larva)
U+8811.0	fUTF8	蠑
U+8811.0	fCNS11643	2-6972
U+8811.0	fBig5	F4C6
U+8811.0	fGB	1-726E
U+8811.0	fR/S	 142.14
U+8811.0	fTotalStrokes	 20
U+8811.0	fCangjie	LIFFD
U+8811.0	fFourCorner	59194
U+8811.0	fHakka	 jin2; rin2; rung2; jung2
U+8811.0	fCantonese	wing4
U+8811.0	fMandarin	rong2
U+8811.0	fEnglish	[1] a kind of mollusk [2] a kind of reptile; lizard
U+8812.0	fUTF8	蠒
U+8812.0	fCNS11643	3-5D25
U+8812.0	fGB	E-3C66
U+8812.0	fR/S	  142.14
U+8812.0	fTotalStrokes	 20
U+8812.0	fCangjie	MBLMI
U+8812.0	fFourCorner	10130
U+8812.0	fHakka	 gian3
U+8812.0	fCantonese	gaan2
U+8812.0	fMandarin	jian3
U+8813.0	fUTF8	蠓
U+8813.0	fCNS11643	2-6974
U+8813.0	fBig5	F4C8
U+8813.0	fGB	0-7337
U+8813.0	fR/S	 142.14
U+8813.0	fTotalStrokes	 20
U+8813.0	fCangjie	LITBO
U+8813.0	fFourCorner	54132
U+8813.0	fHakka	  mung3; mung2
U+8813.0	fCantonese	mung5; mung2
U+8813.0	fMandarin	meng3; meng2
U+8813.0	fEnglish	a kind of gnat; midge; biting midge
U+8814.0	fUTF8	蠔
U+8814.0	fCNS11643	1-7A39
U+8814.0	fBig5	C4AE
U+8814.0	fGB	E-3C67
U+8814.0	fR/S	 142.14
U+8814.0	fTotalStrokes	 20
U+8814.0	fCangjie	LIYRO
U+8814.0	fFourCorner	50132
U+8814.0	fHakka	 hau2;  ho2; o2
U+8814.0	fCantonese	hou4
U+8814.0	fMandarin	hao2
U+8814.0	fEnglish	 oyster
U+8815.0	fUTF8	蠕
U+8815.0	fCNS11643	1-7A3A
U+8815.0	fBig5	C4AF
U+8815.0	fGB	0-4864
U+8815.0	fR/S	 142.14
U+8815.0	fTotalStrokes	 20
U+8815.0	fCangjie	LIMBB
U+8815.0	fFourCorner	51127
U+8815.0	fHakka	 ngion3;  ji2; nuk7; ngion1; ri2; luk8; nion1
U+8815.0	fCantonese	jyu4; jyun5
U+8815.0	fMandarin	ru2; ruan3; ruan2
U+8815.0	fEnglish	 wriggle; squirm
U+8816.0	fUTF8	蠖
U+8816.0	fCNS11643	2-6975
U+8816.0	fBig5	F4C9
U+8816.0	fGB	0-7336
U+8816.0	fR/S	 142.14
U+8816.0	fTotalStrokes	 20
U+8816.0	fCangjie	LITOE
U+8816.0	fFourCorner	54147
U+8816.0	fHakka	  wok7; vok7
U+8816.0	fCantonese	wok6; wok3
U+8816.0	fMandarin	huo4; wo4; yue4
U+8816.0	fEnglish	 measuring worm; looper caterpillar; looper
U+8817.0	fUTF8	蠗
U+8817.0	fCNS11643	2-6973
U+8817.0	fBig5	F4C7
U+8817.0	fGB	3-632F
U+8817.0	fR/S	 142.14
U+8817.0	fTotalStrokes	 20
U+8817.0	fCangjie	LISMG
U+8817.0	fFourCorner	57114
U+8817.0	fCantonese	zok6
U+8817.0	fMandarin	zhuo2
U+8818.0	fUTF8	蠘
U+8818.0	fCNS11643	3-5D24
U+8818.0	fGB	3-632C
U+8818.0	fR/S	 142.14
U+8818.0	fTotalStrokes	 20
U+8818.0	fCangjie	LIJIG
U+8818.0	fFourCorner	53150
U+8818.0	fCantonese	zit6
U+8818.0	fMandarin	jie2
U+8819.0	fUTF8	蠙
U+8819.0	fCNS11643	2-6970
U+8819.0	fBig5	F4C4
U+8819.0	fGB	3-626B
U+8819.0	fR/S	 142.14
U+8819.0	fTotalStrokes	 20
U+8819.0	fCangjie	LIJMC
U+8819.0	fFourCorner	53186
U+8819.0	fHakka	  bin1
U+8819.0	fCantonese	ban1 (also); pan4
U+8819.0	fMandarin	bin1; pin2; pian2; pin4
U+8819.0	fEnglish	oyster
U+881A.0	fUTF8	蠚
U+881A.0	fCNS11643	3-5E5D
U+881A.0	fGB	3-632D
U+881A.0	fR/S	  142.15
U+881A.0	fTotalStrokes	 21
U+881A.0	fCangjie	TKRI
U+881A.0	fFourCorner	44130
U+881A.0	fHakka	  kok7; siok7
U+881A.0	fCantonese	kok3; hok3 (also)
U+881A.0	fMandarin	he4; he1; ruo4
U+881A.0	fEnglish	poisonous sting of an insect
U+881B.0	fUTF8	蠛
U+881B.0	fCNS11643	2-6C31
U+881B.0	fBig5	F642
U+881B.0	fGB	0-733A
U+881B.0	fR/S	 142.15
U+881B.0	fTotalStrokes	 21
U+881B.0	fCangjie	LITWI
U+881B.0	fFourCorner	54153
U+881B.0	fHakka	  met7
U+881B.0	fCantonese	mit6
U+881B.0	fMandarin	mie4
U+881B.0	fEnglish	flies; small insects pro- duced in damp places
U+881C.0	fUTF8	蠜
U+881C.0	fCNS11643	2-6C34
U+881C.0	fBig5	F645
U+881C.0	fGB	5-6249
U+881C.0	fR/S	 142.15
U+881C.0	fTotalStrokes	 21
U+881C.0	fCangjie	DDKLI
U+881C.0	fFourCorner	22136
U+881C.0	fHakka	  fan2
U+881C.0	fCantonese	faan4; faan6
U+881C.0	fMandarin	fan2
U+881C.0	fEnglish	grasshopper
U+881D.0	fUTF8	蠝
U+881D.0	fCNS11643	2-6C30
U+881D.0	fBig5	F641
U+881D.0	fGB	5-624B
U+881D.0	fR/S	 142.15
U+881D.0	fTotalStrokes	 21
U+881D.0	fCangjie	LIWWW
U+881D.0	fFourCorner	56160
U+881D.0	fCantonese	leoi5
U+881D.0	fMandarin	lei2; lei3
U+881E.0	fUTF8	蠞
U+881E.0	fCNS11643	4-6126
U+881E.0	fGB	E-3C68
U+881E.0	fR/S	 142.15; 142.13
U+881E.0	fTotalStrokes	 19; 21
U+881E.0	fCangjie	HALI
U+881E.0	fFourCorner	88130
U+881E.0	fCantonese	zit6
U+881E.0	fMandarin	jie2
U+881F.0	fUTF8	蠟
U+881F.0	fCNS11643	1-7B27
U+881F.0	fBig5	C4FA
U+881F.0	fGB	1-402F
U+881F.0	fR/S	 142.15
U+881F.0	fTotalStrokes	 21
U+881F.0	fCangjie	LIVVV
U+881F.0	fFourCorner	52116
U+881F.0	fHakka	  lap8
U+881F.0	fCantonese	laap6
U+881F.0	fMandarin	la4; zha4
U+881F.0	fEnglish	[1] wax [2] candle
U+8820.0	fUTF8	蠠
U+8820.0	fCNS11643	2-6C32
U+8820.0	fBig5	F643
U+8820.0	fGB	E-3C69
U+8820.0	fR/S	 142.15
U+8820.0	fTotalStrokes	 21
U+8820.0	fCangjie	MWLII
U+8820.0	fFourCorner	10136
U+8820.0	fCantonese	man5
U+8820.0	fMandarin	mi4; min3
U+8821.0	fUTF8	蠡
U+8821.0	fCNS11643	1-7B26
U+8821.0	fBig5	C4F9
U+8821.0	fGB	0-733B
U+8821.0	fR/S	 142.15
U+8821.0	fTotalStrokes	 21
U+8821.0	fCangjie	VOLII
U+8821.0	fFourCorner	27136
U+8821.0	fHakka	 li3; bok7; li5; lo2
U+8821.0	fCantonese	lai5; lai4; lo4
U+8821.0	fMandarin	li3; luo2; li4; luo3
U+8821.0	fEnglish	[1] a wood-boring insect [2] bore into wood; eat wood (insects) [3] worm-eaten
U+8821.1	fUTF8	蠡
U+8821.1	fCNS11643	1-7B26
U+8821.1	fBig5	C4F9
U+8821.1	fGB	0-733B
U+8821.1	fR/S	 142.15
U+8821.1	fTotalStrokes	 21
U+8821.1	fCangjie	VOLII
U+8821.1	fFourCorner	27136
U+8821.1	fMandarin	li2
U+8821.1	fEnglish	[1] calabash [2] calabash shell serving as a dipper; dipper
U+8822.0	fUTF8	蠢
U+8822.0	fCNS11643	1-7B25
U+8822.0	fBig5	C4F8
U+8822.0	fGB	0-3440
U+8822.0	fR/S	 142.15
U+8822.0	fTotalStrokes	 21
U+8822.0	fCangjie	QKALI
U+8822.0	fFourCorner	50136
U+8822.0	fHakka	  cun3; chun3; cung1
U+8822.0	fCantonese	ceon2
U+8822.0	fMandarin	chun3
U+8822.0	fEnglish	[1] wriggle; squirm [2] stupid; silly; dull; foolish
U+8823.0	fUTF8	蠣
U+8823.0	fCNS11643	1-7B24
U+8823.0	fBig5	C4F7
U+8823.0	fGB	1-7243
U+8823.0	fR/S	 142.15
U+8823.0	fTotalStrokes	 21
U+8823.0	fCangjie	LIMTB
U+8823.0	fFourCorner	51127
U+8823.0	fHakka	 li5; li6; li3
U+8823.0	fCantonese	lai6
U+8823.0	fMandarin	li4
U+8823.0	fEnglish	 oyster
U+8824.0	fUTF8	蠤
U+8824.0	fCNS11643	2-6C33
U+8824.0	fBig5	F644
U+8824.0	fGB	E-3C6A
U+8824.0	fR/S	 142.15
U+8824.0	fTotalStrokes	 21
U+8824.0	fCangjie	TWLII
U+8824.0	fFourCorner	80136
U+8824.0	fCantonese	cau1
U+8824.0	fMandarin	qiu1
U+8825.0	fUTF8	蠥
U+8825.0	fCNS11643	2-6D7E
U+8825.0	fBig5	F751
U+8825.0	fGB	E-3C6B
U+8825.0	fR/S	 142.16
U+8825.0	fTotalStrokes	 22
U+8825.0	fCangjie	UJLMI
U+8825.0	fFourCorner	20136
U+8825.0	fCantonese	jit6; jip6 (also)
U+8825.0	fMandarin	nie4
U+8826.0	fUTF8	蠦
U+8826.0	fCNS11643	2-6D7C
U+8826.0	fBig5	F74F
U+8826.0	fGB	3-616E
U+8826.0	fR/S	 142.16
U+8826.0	fTotalStrokes	 22
U+8826.0	fCangjie	LIYPT
U+8826.0	fFourCorner	51117
U+8826.0	fCantonese	lou4
U+8826.0	fMandarin	lu2
U+8827.0	fUTF8	蠧
U+8827.0	fCNS11643	3-5F5F
U+8827.0	fGB	5-3726
U+8827.0	fR/S	 142.16
U+8827.0	fTotalStrokes	 22
U+8827.0	fCangjie	GBMI
U+8827.0	fFourCorner	40130
U+8827.0	fHakka	 du5; zhu5; zu5
U+8827.0	fCantonese	dou3
U+8827.0	fMandarin	du4
U+8827.0	fEnglish	 moth insects which eat into cloth
U+8828.0	fUTF8	蠨
U+8828.0	fCNS11643	2-6D7B
U+8828.0	fBig5	F74E
U+8828.0	fGB	3-626E
U+8828.0	fR/S	 142.16
U+8828.0	fTotalStrokes	 22
U+8828.0	fCangjie	LITLX
U+8828.0	fFourCorner	54127
U+8828.0	fHakka	 siau1
U+8828.0	fCantonese	siu1
U+8828.0	fMandarin	xiao1
U+8828.0	fEnglish	Teraguathat, a kind of spider with long legs
U+8829.0	fUTF8	蠩
U+8829.0	fCNS11643	2-6C2F
U+8829.0	fBig5	F640
U+8829.0	fGB	5-6227
U+8829.0	fR/S	 142.16
U+8829.0	fTotalStrokes	 22
U+8829.0	fCangjie	LIYRA
U+8829.0	fFourCorner	54160
U+8829.0	fHakka	 zhu1; zu1; chu1; cu1
U+8829.0	fCantonese	zyu1; syu4
U+8829.0	fMandarin	zhu1; chu2
U+8829.0	fEnglish	a toad
U+882A.0	fUTF8	蠪
U+882A.0	fCNS11643	2-6D7D
U+882A.0	fBig5	F750
U+882A.0	fGB	3-587D
U+882A.0	fR/S	 142.16
U+882A.0	fTotalStrokes	 22
U+882A.0	fCangjie	YPLMI
U+882A.0	fFourCorner	01136
U+882A.0	fHakka	 lung2
U+882A.0	fCantonese	lung4
U+882A.0	fMandarin	long2
U+882A.0	fEnglish	an insect found on the olive
U+882B.0	fUTF8	蠫
U+882B.0	fCNS11643	2-6C35
U+882B.0	fBig5	F646
U+882B.0	fGB	E-3C6C
U+882B.0	fR/S	 142.16
U+882B.0	fTotalStrokes	 22
U+882B.0	fCangjie	BNLII
U+882B.0	fFourCorner	22136
U+882B.0	fCantonese	lai5; lai6; lei4
U+882B.0	fMandarin	li4; li2
U+882C.0	fUTF8	蠬
U+882C.0	fCNS11643	2-6D7A
U+882C.0	fBig5	F74D
U+882C.0	fGB	E-3C6D
U+882C.0	fR/S	 142.16
U+882C.0	fTotalStrokes	 22
U+882C.0	fCangjie	LIYBP
U+882C.0	fFourCorner	51111
U+882C.0	fCantonese	lung4
U+882C.0	fMandarin	long2
U+882D.0	fUTF8	蠭
U+882D.0	fCNS11643	3-6045
U+882D.0	fGB	5-624D
U+882D.0	fR/S	 142.17
U+882D.0	fTotalStrokes	 23
U+882D.0	fCangjie	YJLII
U+882D.0	fFourCorner	37130
U+882D.0	fHakka	  fung1;  pung1
U+882D.0	fCantonese	fung1
U+882D.0	fMandarin	feng1
U+882D.0	fEnglish	 bee; wasp
U+882E.0	fUTF8	蠮
U+882E.0	fCNS11643	2-6F48
U+882E.0	fBig5	F7F9
U+882E.0	fGB	3-6332
U+882E.0	fR/S	 142.17
U+882E.0	fTotalStrokes	 23
U+882E.0	fCangjie	LISEM
U+882E.0	fFourCorner	57127
U+882E.0	fHakka	 ji5; rit7; jit7; ri5
U+882E.0	fCantonese	jit6 (also); jit3
U+882E.0	fMandarin	ye1
U+882E.0	fEnglish	bee
U+882F.0	fUTF8	蠯
U+882F.0	fCNS11643	2-6F26
U+882F.0	fBig5	F7D7
U+882F.0	fGB	E-3C6E
U+882F.0	fR/S	 142.17
U+882F.0	fTotalStrokes	 23
U+882F.0	fCangjie	XIHJI
U+882F.0	fFourCorner	00236
U+882F.0	fHakka	 pin3; bi1
U+882F.0	fCantonese	pei4; ban2
U+882F.0	fMandarin	beng4; pi2
U+882F.0	fEnglish	the shell of a long narrow bivalve, used as a scoop
U+8830.0	fUTF8	蠰
U+8830.0	fCNS11643	2-6F46
U+8830.0	fBig5	F7F7
U+8830.0	fGB	3-6333
U+8830.0	fR/S	 142.17
U+8830.0	fTotalStrokes	 23
U+8830.0	fCangjie	LIYRV
U+8830.0	fFourCorner	50132
U+8830.0	fCantonese	joeng6; joeng4
U+8830.0	fMandarin	shang4; rang2; nang2; shang1; rang3; shuang1
U+8831.0	fUTF8	蠱
U+8831.0	fCNS11643	1-7C47
U+8831.0	fBig5	C5DB
U+8831.0	fGB	1-3946
U+8831.0	fR/S	 142.17
U+8831.0	fTotalStrokes	 23
U+8831.0	fCangjie	LILIT
U+8831.0	fFourCorner	50107
U+8831.0	fHakka	  gu3
U+8831.0	fCantonese	gu2; gwu2 (also)
U+8831.0	fMandarin	gu3; gu4; ye3
U+8831.0	fEnglish	[1] poison; venom; harm [2] bewitch; enchant
U+8832.0	fUTF8	蠲
U+8832.0	fCNS11643	2-6F47
U+8832.0	fBig5	F7F8
U+8832.0	fGB	0-6E43
U+8832.0	fR/S	 142.17
U+8832.0	fTotalStrokes	 23
U+8832.0	fCangjie	TTWLI
U+8832.0	fFourCorner	86127
U+8832.0	fHakka	 gien1; gian1; gen1
U+8832.0	fCantonese	gyun1
U+8832.0	fMandarin	juan1; gui1
U+8832.0	fEnglish	[1] millipede [2] clean; pure; bright [3] clean; wash; cleanse [4] remit; remove (taxes, sentences, etc.)
U+8833.0	fUTF8	蠳
U+8833.0	fCNS11643	2-6F49
U+8833.0	fBig5	F7FA
U+8833.0	fGB	5-622C
U+8833.0	fR/S	 142.17
U+8833.0	fTotalStrokes	 23
U+8833.0	fCangjie	LIBCV
U+8833.0	fFourCorner	56144
U+8833.0	fCantonese	ang1; ngang1 (also); jing1
U+8833.0	fMandarin	ying1
U+8834.0	fUTF8	蠴
U+8834.0	fCNS11643	3-6647
U+8834.0	fGB	E-3C6F
U+8834.0	fR/S	 142.15
U+8834.0	fTotalStrokes	 21
U+8834.0	fCangjie	XLITW
U+8834.0	fFourCorner	5416
U+8834.0	fCantonese	syu4
U+8835.0	fUTF8	蠵
U+8835.0	fCNS11643	2-704D
U+8835.0	fBig5	F8BF
U+8835.0	fGB	3-6335
U+8835.0	fR/S	 142.18
U+8835.0	fTotalStrokes	 24
U+8835.0	fCangjie	LIUOB
U+8835.0	fFourCorner	52127
U+8835.0	fHakka	  hi1
U+8835.0	fCantonese	kwai4
U+8835.0	fMandarin	xi1; wei2; xi2
U+8835.0	fEnglish	a kind of large turtle
U+8836.0	fUTF8	蠶
U+8836.0	fCNS11643	1-7C66
U+8836.0	fBig5	C5FA
U+8836.0	fGB	1-324F
U+8836.0	fR/S	 142.18
U+8836.0	fTotalStrokes	 24
U+8836.0	fCangjie	MUALI
U+8836.0	fFourCorner	11136
U+8836.0	fHakka	  cam2; cam3
U+8836.0	fCantonese	caam4; caam2 (also)
U+8836.0	fMandarin	can2
U+8836.0	fEnglish	 silkworm
U+8837.0	fUTF8	蠷
U+8837.0	fCNS11643	2-704C
U+8837.0	fBig5	F8BE
U+8837.0	fGB	3-6334
U+8837.0	fR/S	 142.18
U+8837.0	fTotalStrokes	 24
U+8837.0	fCangjie	LIBUG
U+8837.0	fFourCorner	56114
U+8837.0	fCantonese	keoi4; fok3
U+8837.0	fMandarin	qu2
U+8838.0	fUTF8	蠸
U+8838.0	fCNS11643	2-704B
U+8838.0	fBig5	F8BD
U+8838.0	fGB	3-6331
U+8838.0	fR/S	 142.18
U+8838.0	fTotalStrokes	 24
U+8838.0	fCangjie	LITRG
U+8838.0	fFourCorner	54114
U+8838.0	fCantonese	kyun4
U+8838.0	fMandarin	quan2; huan1; huan4
U+8839.0	fUTF8	蠹
U+8839.0	fCNS11643	1-7C67
U+8839.0	fBig5	C5FB
U+8839.0	fGB	0-733C
U+8839.0	fR/S	 142.18
U+8839.0	fTotalStrokes	 24
U+8839.0	fCangjie	JBMRI
U+8839.0	fFourCorner	40136
U+8839.0	fHakka	  du5; zhu5; zu5
U+8839.0	fCantonese	dou3
U+8839.0	fMandarin	du4
U+8839.0	fEnglish	[1] moth; a kind of insects which eats into books and clothing [2] moth-eaten; worm-eaten [3] an insect that eats up resources; embezzler
U+883A.0	fUTF8	蠺
U+883A.0	fCNS11643	4-6C23
U+883A.0	fGB	E-3C70
U+883A.0	fR/S	 142.18
U+883A.0	fTotalStrokes	 24
U+883A.0	fCangjie	MKALI
U+883A.0	fFourCorner	11130
U+883A.0	fCantonese	caam4
U+883A.0	fMandarin	can2
U+883A.0	fEnglish	same as &#x8695;
U+883B.0	fUTF8	蠻
U+883B.0	fCNS11643	1-7D27
U+883B.0	fBig5	C65A
U+883B.0	fGB	1-4279
U+883B.0	fR/S	 142.19
U+883B.0	fTotalStrokes	 25
U+883B.0	fCangjie	VFLMI
U+883B.0	fFourCorner	22136
U+883B.0	fHakka	  man2; man3
U+883B.0	fCantonese	maan4
U+883B.0	fMandarin	man2
U+883B.0	fEnglish	[1] barbarians in the south; barbarous; savage [2] quite; pretty; fairly; very [3] very
U+883C.0	fUTF8	蠼
U+883C.0	fCNS11643	2-715D
U+883C.0	fBig5	F96E
U+883C.0	fGB	0-733D
U+883C.0	fR/S	 142.20
U+883C.0	fTotalStrokes	 26
U+883C.0	fCangjie	LIBUE
U+883C.0	fFourCorner	56147
U+883C.0	fCantonese	keoi4; fok3
U+883C.0	fMandarin	jue2; qu2
U+883C.0	fEnglish	[1] female monkey [2] a black insect with six legs, capable of ejecting poison for self-defense
U+883D.0	fUTF8	蠽
U+883D.0	fCNS11643	2-7175
U+883D.0	fBig5	F9A7
U+883D.0	fGB	E-3C71
U+883D.0	fR/S	 142.21
U+883D.0	fTotalStrokes	 27
U+883D.0	fCangjie	FILII
U+883D.0	fFourCorner	93136
U+883D.0	fHakka	 cat8
U+883D.0	fCantonese	zit3; zit6
U+883D.0	fMandarin	jie2
U+883E.0	fUTF8	蠾
U+883E.0	fCNS11643	2-7174
U+883E.0	fBig5	F9A6
U+883E.0	fGB	3-6325
U+883E.0	fR/S	 142.21
U+883E.0	fTotalStrokes	 27
U+883E.0	fCangjie	LISEI
U+883E.0	fFourCorner	57127
U+883E.0	fHakka	 chuk7; zuk7; zhuk7
U+883E.0	fCantonese	zuk1; suk6
U+883E.0	fMandarin	zhu2; shu3
U+883F.0	fUTF8	蠿
U+883F.0	fCNS11643	2-7176
U+883F.0	fBig5	F9A8
U+883F.0	fGB	E-3C72
U+883F.0	fR/S	 142.22
U+883F.0	fTotalStrokes	 28
U+883F.0	fCangjie	VMLII
U+883F.0	fFourCorner	22136
U+883F.0	fCantonese	zyut3
U+883F.0	fMandarin	zha2; zhuo2
U+8840.0	fUTF8	血
U+8840.0	fCNS11643	1-4866
U+8840.0	fBig5	A6E5
U+8840.0	fGB	0-512A
U+8840.0	fR/S	 143.0
U+8840.0	fTotalStrokes	 6
U+8840.0	fCangjie	HBT
U+8840.0	fFourCorner	27100
U+8840.0	fHakka	  hiet7;  het7; hiat7
U+8840.0	fCantonese	hyut3
U+8840.0	fMandarin	xue4
U+8840.0	fEnglish	[1] blood; blood relationship [2] the menses [3] Kangxi radical 143
U+8840.1	fUTF8	血
U+8840.1	fCNS11643	1-4866
U+8840.1	fBig5	A6E5
U+8840.1	fGB	0-512A
U+8840.1	fR/S	 143.0
U+8840.1	fTotalStrokes	 6
U+8840.1	fCangjie	HBT
U+8840.1	fFourCorner	27100
U+8840.1	fMandarin	xie3
U+8841.0	fUTF8	衁
U+8841.0	fCNS11643	2-2D4B
U+8841.0	fBig5	D0AA
U+8841.0	fGB	3-654B
U+8841.0	fR/S	 143.3
U+8841.0	fTotalStrokes	 9
U+8841.0	fCangjie	YVHBT
U+8841.0	fFourCorner	00107
U+8841.0	fHakka	  fong1; wong6; wong5
U+8841.0	fCantonese	fong1
U+8841.0	fMandarin	huang1
U+8841.0	fEnglish	 blood
U+8842.0	fUTF8	衂
U+8842.0	fCNS11643	3-2F26
U+8842.0	fGB	E-3C73
U+8842.0	fR/S	 143.3
U+8842.0	fTotalStrokes	 9
U+8842.0	fCangjie	HTSHI
U+8842.0	fFourCorner	27120
U+8842.0	fHakka	 nuk8
U+8842.0	fCantonese	nuk6; luk6 (also)
U+8842.0	fMandarin	niu4; nuu4; n4
U+8842.0	fEnglish	 be defeated
U+8843.0	fUTF8	衃
U+8843.0	fCNS11643	2-3271
U+8843.0	fBig5	D3CF
U+8843.0	fGB	5-6465
U+8843.0	fR/S	 143.4
U+8843.0	fTotalStrokes	 10
U+8843.0	fCangjie	HTMF
U+8843.0	fFourCorner	21190
U+8843.0	fHakka	  pi1;  feu2
U+8843.0	fCantonese	pui1; fau4; bei1
U+8843.0	fMandarin	pei1; fou1; fou2; fou3; pou1
U+8843.0	fEnglish	coagulated blood
U+8844.0	fUTF8	衄
U+8844.0	fCNS11643	2-3272
U+8844.0	fBig5	D3D0
U+8844.0	fGB	0-742C
U+8844.0	fR/S	 143.4
U+8844.0	fTotalStrokes	 10
U+8844.0	fCangjie	HTNG
U+8844.0	fFourCorner	27115
U+8844.0	fHakka	  nuk8;  nut8
U+8844.0	fCantonese	nuk6; luk6 (also)
U+8844.0	fMandarin	n4; nuu4; rou4; niu4
U+8844.0	fEnglish	[1] nose bleeding; bleeding nose [2] be defeated; given a bleeding nose
U+8845.0	fUTF8	衅
U+8845.0	fCNS11643	3-386A
U+8845.0	fGB	0-5046
U+8845.0	fR/S	 143.5
U+8845.0	fTotalStrokes	 11
U+8845.0	fCangjie	HTFQ
U+8845.0	fFourCorner	29150
U+8845.0	fHakka	  hin5; hiun5
U+8845.0	fCantonese	jan6; jan3
U+8845.0	fMandarin	xin4
U+8845.0	fEnglish	simplified form of &#x91C1;
U+8846.0	fUTF8	衆
U+8846.0	fCNS11643	3-3E6A
U+8846.0	fGB	1-565A
U+8846.0	fR/S	 143.6
U+8846.0	fTotalStrokes	 12
U+8846.0	fCangjie	HTOHO
U+8846.0	fFourCorner	27230
U+8846.0	fHakka	  zung5; zhung5
U+8846.0	fCantonese	zung3
U+8846.0	fMandarin	zhong4
U+8846.0	fEnglish	[1] multitude; crowd; the people [2] many; numerous [3] all
U+8847.0	fUTF8	衇
U+8847.0	fCNS11643	3-3E69
U+8847.0	fGB	E-3C74
U+8847.0	fR/S	  143.6
U+8847.0	fTotalStrokes	 12
U+8847.0	fCangjie	HTHHV
U+8847.0	fFourCorner	22130
U+8847.0	fHakka	 mak7
U+8847.0	fCantonese	mak6
U+8847.0	fMandarin	mo4; mai4
U+8847.0	fEnglish	blood vessels; veins; arteries
U+8848.0	fUTF8	衈
U+8848.0	fCNS11643	2-4026
U+8848.0	fBig5	DBC0
U+8848.0	fGB	3-654C
U+8848.0	fR/S	 143.6
U+8848.0	fTotalStrokes	 12
U+8848.0	fCangjie	HTSJ
U+8848.0	fFourCorner	21140
U+8848.0	fHakka	 ngi3; ngi5
U+8848.0	fCantonese	nei6; lei6 (also); ji6
U+8848.0	fMandarin	er4; er3
U+8848.0	fEnglish	the blood of a sacrificial fowl whichwas sprinkled on the doors and vessels
U+8849.0	fUTF8	衉
U+8849.0	fCNS11643	6-4E4F
U+8849.0	fGB	3-654D
U+8849.0	fR/S	  143.6
U+8849.0	fTotalStrokes	 12
U+8849.0	fCangjie	HTHER
U+8849.0	fFourCorner	27160
U+8849.0	fHakka	  kak7
U+8849.0	fCantonese	haak3
U+8849.0	fMandarin	ke4
U+8849.0	fEnglish	[1] be smeared with blood [2] vomit blood
U+884A.0	fUTF8	衊
U+884A.0	fCNS11643	2-6C36
U+884A.0	fBig5	F647
U+884A.0	fGB	1-785A
U+884A.0	fR/S	 143.15
U+884A.0	fTotalStrokes	 21
U+884A.0	fCangjie	HTTWI
U+884A.0	fFourCorner	24153
U+884A.0	fHakka	 met7
U+884A.0	fCantonese	mit6
U+884A.0	fMandarin	mie4; mo4; mian4; man2
U+884A.0	fEnglish	[1] stain with blood [2] slander; trump up a charge; smear
U+884B.0	fUTF8	衋
U+884B.0	fCNS11643	2-704E
U+884B.0	fBig5	F8C0
U+884B.0	fGB	3-682D
U+884B.0	fR/S	 143.18
U+884B.0	fTotalStrokes	 24
U+884B.0	fCangjie	LQMAT
U+884B.0	fFourCorner	50107
U+884B.0	fHakka	 hit7
U+884B.0	fCantonese	sik1; hik1
U+884B.0	fMandarin	xi4; xu4
U+884B.0	fEnglish	grief of heart at tyranny
U+884C.0	fUTF8	行
U+884C.0	fCNS11643	1-4867
U+884C.0	fBig5	A6E6
U+884C.0	fGB	0-5050
U+884C.0	fR/S	 144.0
U+884C.0	fTotalStrokes	 6
U+884C.0	fCangjie	HOMMN
U+884C.0	fFourCorner	21221
U+884C.0	fHakka	  hong2;  hang3; hen6; hen5; hang5; hong3; {hen5}
U+884C.0	fCantonese	hang4; haang4 (also); hong4; hang2 (also); hang6; haang2 (also); hong2 (also); hong6
U+884C.0	fMandarin	xing2; hang4; heng2
U+884C.0	fEnglish	[1] go; walk; go on foot [2] move; travel; circulate; go [3] act; do; work [4] publish [5] be current; prevail [6] able; capable [7] all right; O.K.; enough [8] baggage for travel [9] road; path [10] ready to ... [11] Kangxi radical 144
U+884C.1	fUTF8	行
U+884C.1	fCNS11643	1-4867
U+884C.1	fBig5	A6E6
U+884C.1	fGB	0-5050
U+884C.1	fR/S	 144.0
U+884C.1	fTotalStrokes	 6
U+884C.1	fCangjie	HOMMN
U+884C.1	fFourCorner	21221
U+884C.1	fMandarin	hang2
U+884C.1	fEnglish	[1] row; line [2] business firm; company [3] trade; line; profession [4] order of brothers (and sisters) according to seniority [5] generation
U+884C.2	fUTF8	行
U+884C.2	fCNS11643	1-4867
U+884C.2	fBig5	A6E6
U+884C.2	fGB	0-5050
U+884C.2	fR/S	 144.0
U+884C.2	fTotalStrokes	 6
U+884C.2	fCangjie	HOMMN
U+884C.2	fFourCorner	21221
U+884C.2	fHakka	 hang2
U+884C.2	fMandarin	xing4
U+884C.2	fEnglish	one's conduct; one's behavior
U+884D.0	fUTF8	衍
U+884D.0	fCNS11643	1-534F
U+884D.0	fBig5	AD6C
U+884D.0	fGB	0-515C
U+884D.0	fR/S	 144.3
U+884D.0	fTotalStrokes	 9
U+884D.0	fCangjie	HOEMN
U+884D.0	fFourCorner	21103
U+884D.0	fHakka	 jen5; jen3; ren3; jan1; jen1; ren5; jan3
U+884D.0	fCantonese	jin2; hin2 (also); jin5 (also)
U+884D.0	fMandarin	yan3; yan2; yan4
U+884D.0	fEnglish	[1] overflow; spill over; spread out [2] plenty and abundant; ample [3] level; plain and even (fields, plains, etc.) [4] lake; marsh [5] slope [6] superfluous [7] bamboo chest or box [8] a Chinese family name
U+884E.0	fUTF8	衎
U+884E.0	fCNS11643	2-2D4C
U+884E.0	fBig5	D0AB
U+884E.0	fGB	3-397A
U+884E.0	fR/S	 144.3
U+884E.0	fTotalStrokes	 9
U+884E.0	fCangjie	HOMJN
U+884E.0	fFourCorner	21221
U+884E.0	fHakka	 kan1;  kon3; kan3
U+884E.0	fCantonese	hon3; hon2
U+884E.0	fMandarin	kan4; kan3
U+884E.0	fEnglish	 give pleasure be pleased happy
U+884F.0	fUTF8	衏
U+884F.0	fCNS11643	3-3371
U+884F.0	fGB	3-397C
U+884F.0	fR/S	 144.4
U+884F.0	fTotalStrokes	 10
U+884F.0	fCangjie	HOMUN
U+884F.0	fFourCorner	21211
U+884F.0	fCantonese	jyun2; jyun6 (also)
U+884F.0	fMandarin	yuan4
U+8850.0	fUTF8	衐
U+8850.0	fCNS11643	3-6648
U+8850.0	fGB	E-3C75
U+8850.0	fR/S	 144.5
U+8850.0	fTotalStrokes	 11
U+8850.0	fCangjie	HOSSN
U+8850.0	fFourCorner	2171
U+8850.0	fCantonese	geoi6
U+8851.0	fUTF8	衑
U+8851.0	fCNS11643	4-3527
U+8851.0	fGB	E-3C76
U+8851.0	fR/S	  144.5
U+8851.0	fTotalStrokes	 11
U+8851.0	fCangjie	HOOIN
U+8851.0	fFourCorner	21307
U+8851.0	fCantonese	ling4
U+8851.0	fMandarin	ling2
U+8852.0	fUTF8	衒
U+8852.0	fCNS11643	2-3934
U+8852.0	fBig5	D7B1
U+8852.0	fGB	3-3A23
U+8852.0	fR/S	 144.5
U+8852.0	fTotalStrokes	 11
U+8852.0	fCangjie	HOYIN
U+8852.0	fFourCorner	21732
U+8852.0	fHakka	 hien2;  hien5
U+8852.0	fCantonese	jyun6; jyun4 (also)
U+8852.0	fMandarin	xuan4; juan4
U+8852.0	fEnglish	[1] brag; boast; show off [2] recommend oneself; promote oneself
U+8853.0	fUTF8	術
U+8853.0	fCNS11643	1-5D34
U+8853.0	fBig5	B34E
U+8853.0	fGB	1-4A75
U+8853.0	fR/S	 144.5
U+8853.0	fTotalStrokes	 11
U+8853.0	fCangjie	HOICN
U+8853.0	fFourCorner	21904
U+8853.0	fHakka	  sut8;  shut8
U+8853.0	fCantonese	seot6; seot2 (also); seoi6
U+8853.0	fMandarin	shu4; zhu2
U+8853.0	fEnglish	[1] art; skill; special feat [2] method to do something; way to do something
U+8853.1	fUTF8	術
U+8853.1	fCNS11643	1-5D34
U+8853.1	fBig5	B34E
U+8853.1	fGB	1-4A75
U+8853.1	fR/S	 144.5
U+8853.1	fTotalStrokes	 11
U+8853.1	fCangjie	HOICN
U+8853.1	fFourCorner	21904
U+8853.1	fMandarin	sui4
U+8853.1	fEnglish	an administrative district in ancient China in the suburbs of a city
U+8854.0	fUTF8	衔
U+8854.0	fGB	0-4F4E
U+8854.0	fR/S	 60.8; 144.5
U+8854.0	fTotalStrokes	 11
U+8854.0	fCangjie	XHOCM
U+8854.0	fHakka	 ham2
U+8854.0	fCantonese	haam4; ham4 (also)
U+8854.0	fMandarin	xian2
U+8854.0	fEnglish	simplified form of &#x929C;
U+8855.0	fUTF8	衕
U+8855.0	fCNS11643	2-4028
U+8855.0	fBig5	DBC2
U+8855.0	fGB	E-3C77
U+8855.0	fR/S	 144.6
U+8855.0	fTotalStrokes	 12
U+8855.0	fCangjie	HOBRN
U+8855.0	fFourCorner	21221
U+8855.0	fHakka	 tung2; tung5
U+8855.0	fCantonese	dung6; tung4
U+8855.0	fMandarin	tong4; tong2; dong4
U+8855.0	fEnglish	 alley; lane
U+8856.0	fUTF8	衖
U+8856.0	fCNS11643	2-4027
U+8856.0	fBig5	DBC1
U+8856.0	fGB	E-3C78
U+8856.0	fR/S	 144.6
U+8856.0	fTotalStrokes	 12
U+8856.0	fCangjie	HOTCN
U+8856.0	fFourCorner	21221
U+8856.0	fHakka	 hong5; hong6; hong3
U+8856.0	fCantonese	hong6; hong2 (also); lung6; nung6 (also)
U+8856.0	fMandarin	long4; xiang4
U+8856.0	fEnglish	 lane; alley
U+8857.0	fUTF8	街
U+8857.0	fCNS11643	1-6179
U+8857.0	fBig5	B5F3
U+8857.0	fGB	0-3D56
U+8857.0	fR/S	 144.6
U+8857.0	fTotalStrokes	 12
U+8857.0	fCangjie	HOGGN
U+8857.0	fFourCorner	21104
U+8857.0	fHakka	  gai1; kie1; ke1; kiai1; giai1; ge1
U+8857.0	fCantonese	gaai1
U+8857.0	fMandarin	jie1
U+8857.0	fEnglish	 street; road; thoroughfare
U+8858.0	fUTF8	衘
U+8858.0	fCNS11643	3-444D
U+8858.0	fGB	E-3C79
U+8858.0	fR/S	 144.6
U+8858.0	fTotalStrokes	 13; 12
U+8858.0	fCangjie	HOOMN
U+8858.0	fFourCorner	21101
U+8858.0	fHakka	 ham2
U+8858.0	fCantonese	haam4; ham4 (also)
U+8858.0	fMandarin	yu4; xian2
U+8859.0	fUTF8	衙
U+8859.0	fCNS11643	1-664C
U+8859.0	fBig5	B8C5
U+8859.0	fGB	0-5143
U+8859.0	fR/S	 144.7
U+8859.0	fTotalStrokes	 13
U+8859.0	fCangjie	HOMRN
U+8859.0	fFourCorner	21601
U+8859.0	fHakka	  nga2; nga3
U+8859.0	fCantonese	ngaa4; aa4 (also)
U+8859.0	fMandarin	yu2; ya2; yu3; yu4
U+8859.0	fEnglish	[1] public office; official residence; government office [2] gather; meet; congregate [3] front hall of the palace [4] a Chinese family name
U+885A.0	fUTF8	衚
U+885A.0	fCNS11643	2-542A
U+885A.0	fBig5	E7C1
U+885A.0	fGB	E-3C7A
U+885A.0	fR/S	 144.9
U+885A.0	fTotalStrokes	 15
U+885A.0	fCangjie	HOJRN
U+885A.0	fFourCorner	21221
U+885A.0	fHakka	  fu2
U+885A.0	fCantonese	wu4
U+885A.0	fMandarin	hu2
U+885A.0	fEnglish	 lane; alley; side street; sublane
U+885B.0	fUTF8	衛
U+885B.0	fCNS11643	1-6E6C
U+885B.0	fBig5	BDC3
U+885B.0	fGB	1-4E40
U+885B.0	fR/S	 144.9
U+885B.0	fTotalStrokes	 15
U+885B.0	fCangjie	HODQN
U+885B.0	fFourCorner	21227
U+885B.0	fHakka	 wui6; wui3; wi5; wi3; vui5; wui5
U+885B.0	fCantonese	wai6
U+885B.0	fMandarin	wei4
U+885B.0	fEnglish	[1] guard; protect; defend; stand guard over [2] a Chinese familly name
U+885C.0	fUTF8	衜
U+885C.0	fCNS11643	3-4E5F
U+885C.0	fGB	5-3C4E
U+885C.0	fR/S	  144.9
U+885C.0	fTotalStrokes	 15
U+885C.0	fCangjie	HOTUN
U+885C.0	fFourCorner	21600
U+885C.0	fHakka	 tau5; tau6; to6; to5
U+885C.0	fCantonese	dou6; dou3 (also)
U+885C.0	fMandarin	dao4
U+885C.0	fEnglish	way; path
U+885D.0	fUTF8	衝
U+885D.0	fCNS11643	1-6E6D
U+885D.0	fBig5	BDC4
U+885D.0	fGB	1-3365
U+885D.0	fR/S	 144.9
U+885D.0	fTotalStrokes	 15
U+885D.0	fCangjie	HOHGN
U+885D.0	fFourCorner	21104
U+885D.0	fHakka	  chung1;  cung1; zhung1; zung1
U+885D.0	fCantonese	cung1; cung3
U+885D.0	fMandarin	chong1; chong4; chong3
U+885D.0	fEnglish	[1] rush (against); thrust; forge ahead; dash [2] charge forward; charge ahead; hit with force [3] thoroughfare; hub; strategic place [4] offend
U+885E.0	fUTF8	衞
U+885E.0	fCNS11643	3-5263
U+885E.0	fGB	E-3C7B
U+885E.0	fR/S	 144.10
U+885E.0	fTotalStrokes	 16
U+885E.0	fCangjie	HODBN
U+885E.0	fFourCorner	21227
U+885E.0	fHakka	  wui3; wi5; wui6; wi3; vui5; wui5
U+885E.0	fCantonese	wai6
U+885E.0	fMandarin	wei4
U+885E.0	fEnglish	guard; protect; defend
U+885F.0	fUTF8	衟
U+885F.0	fCNS11643	3-5264
U+885F.0	fGB	5-3C50
U+885F.0	fR/S	 144.10
U+885F.0	fTotalStrokes	 16
U+885F.0	fCangjie	HOVUN
U+885F.0	fFourCorner	21601
U+885F.0	fCantonese	dou6
U+885F.0	fMandarin	dao4
U+8860.0	fUTF8	衠
U+8860.0	fCNS11643	4-534B
U+8860.0	fGB	3-3A2A
U+8860.0	fR/S	 144.10
U+8860.0	fTotalStrokes	 16
U+8860.0	fCangjie	HOJCN
U+8860.0	fFourCorner	21800
U+8860.0	fHakka	 zhun1; zun1
U+8860.0	fCantonese	zeon1
U+8860.0	fMandarin	zhun1
U+8860.0	fEnglish	[1] genuine [2] pure
U+8861.0	fUTF8	衡
U+8861.0	fCNS11643	1-722F
U+8861.0	fBig5	BFC5
U+8861.0	fGB	0-3A62
U+8861.0	fR/S	 144.10
U+8861.0	fTotalStrokes	 16
U+8861.0	fCangjie	HONKN
U+8861.0	fFourCorner	21221
U+8861.0	fHakka	 fen2; wang2; hen2; vang2
U+8861.0	fCantonese	hang4; waang4; wang4
U+8861.0	fMandarin	heng2
U+8861.0	fEnglish	[1] measure; weigh; assess; consider; judge [2] horizontal [3] railings [4] beam [5] beam of a steelyard [6] a Chinese family name
U+8862.0	fUTF8	衢
U+8862.0	fCNS11643	1-7C68
U+8862.0	fBig5	C5FC
U+8862.0	fGB	0-6169
U+8862.0	fR/S	 144.18
U+8862.0	fTotalStrokes	 24
U+8862.0	fCangjie	HOBGN
U+8862.0	fFourCorner	21221
U+8862.0	fHakka	 ki1; hi1
U+8862.0	fCantonese	keoi4
U+8862.0	fMandarin	qu2; ju4
U+8862.0	fEnglish	 highway junction; thoroughfare; intersect
U+8863.0	fUTF8	衣
U+8863.0	fCNS11643	1-4868
U+8863.0	fBig5	A6E7
U+8863.0	fGB	0-5242
U+8863.0	fR/S	 145.0
U+8863.0	fTotalStrokes	 6
U+8863.0	fCangjie	YHV
U+8863.0	fFourCorner	00732
U+8863.0	fHakka	  ji1; ri1; i1; i5
U+8863.0	fCantonese	ji1; ji3
U+8863.0	fMandarin	yi1
U+8863.0	fEnglish	[1] clothing; clothes; dress; apparel; garments; attire [2] coating; covering [3] skin or peel of fruits [4] a Chinese family name [5] Kangxi radical 145
U+8863.1	fUTF8	衣
U+8863.1	fCNS11643	1-4868
U+8863.1	fBig5	A6E7
U+8863.1	fGB	0-5242
U+8863.1	fR/S	 145.0
U+8863.1	fTotalStrokes	 6
U+8863.1	fCangjie	YHV
U+8863.1	fFourCorner	00732
U+8863.1	fMandarin	yi4
U+8863.1	fEnglish	 clothe; wear; dress
U+8864.0	fUTF8	衤
U+8864.0	fGB	0-7142
U+8864.0	fR/S	 145.0
U+8864.0	fTotalStrokes	 5; 6
U+8864.0	fCangjie	INLO
U+8864.0	fHakka	 ji1; ri1
U+8864.0	fMandarin	yi1zi4pang2
U+8864.0	fEnglish	 clothes Kangxi radical number 145
U+8865.0	fUTF8	补
U+8865.0	fGB	0-3239
U+8865.0	fR/S	 145.2
U+8865.0	fTotalStrokes	 7; 8
U+8865.0	fCangjie	LY
U+8865.0	fFourCorner	3320
U+8865.0	fHakka	 bu3
U+8865.0	fCantonese	bou2
U+8865.0	fMandarin	bu3
U+8865.0	fEnglish	simplified form of &#x88DC;
U+8866.0	fUTF8	衦
U+8866.0	fCNS11643	3-2F27
U+8866.0	fGB	5-653E
U+8866.0	fR/S	 145.3
U+8866.0	fTotalStrokes	 8; 9
U+8866.0	fCangjie	LMJ
U+8866.0	fFourCorner	31240
U+8866.0	fCantonese	gon2
U+8866.0	fMandarin	gan3; gan4
U+8866.0	fEnglish	smooth out the clothes
U+8867.0	fUTF8	衧
U+8867.0	fCNS11643	2-2D4D
U+8867.0	fBig5	D0AC
U+8867.0	fGB	5-653F
U+8867.0	fR/S	 145.3
U+8867.0	fTotalStrokes	 8; 9
U+8867.0	fCangjie	LMD
U+8867.0	fFourCorner	31240
U+8867.0	fCantonese	jyu1
U+8867.0	fMandarin	yu2
U+8868.0	fUTF8	表
U+8868.0	fCNS11643	1-4F50
U+8868.0	fBig5	AAED
U+8868.0	fGB	0-316D
U+8868.0	fR/S	 145.3
U+8868.0	fTotalStrokes	 9
U+8868.0	fCangjie	QMV
U+8868.0	fFourCorner	50732
U+8868.0	fHakka	  biau3; biau1; beu3
U+8868.0	fCantonese	biu2; biu1
U+8868.0	fMandarin	biao3; biao1
U+8868.0	fEnglish	[1] outside; external; apparent; appearance; exteriors; superficial [2] announce; manifest; show [3] relatives on the side of one's mother's sisters or brothers; relatives on the side of one's father's sisters [4] report to the emperor [5] table; schedule; chart; form
U+8869.0	fUTF8	衩
U+8869.0	fCNS11643	2-2D4F
U+8869.0	fBig5	D0AE
U+8869.0	fGB	0-7143
U+8869.0	fR/S	 145.3
U+8869.0	fTotalStrokes	 8; 9
U+8869.0	fCangjie	LEI
U+8869.0	fFourCorner	37240
U+8869.0	fHakka	  cai5;  ca5
U+8869.0	fCantonese	caa3; caa2 (also)
U+8869.0	fMandarin	cha4; chai4; cha3
U+8869.0	fEnglish	the open seam of a garment which allows freedom of movement
U+886A.0	fUTF8	衪
U+886A.0	fCNS11643	2-2D4E
U+886A.0	fBig5	D0AD
U+886A.0	fGB	5-6544
U+886A.0	fR/S	 145.3
U+886A.0	fTotalStrokes	 8; 9
U+886A.0	fCangjie	LPD
U+886A.0	fFourCorner	34212
U+886A.0	fCantonese	ji4 (also); ji5
U+886A.0	fMandarin	yi3; yi2; yi4
U+886B.0	fUTF8	衫
U+886B.0	fCNS11643	1-5350
U+886B.0	fBig5	AD6D
U+886B.0	fGB	0-4940
U+886B.0	fR/S	 145.3
U+886B.0	fTotalStrokes	 8; 9
U+886B.0	fCangjie	LHHH
U+886B.0	fFourCorner	32222
U+886B.0	fHakka	  sam1
U+886B.0	fCantonese	saam1
U+886B.0	fMandarin	shan1
U+886B.0	fEnglish	 shirt; gown; jacket; garment
U+886C.0	fUTF8	衬
U+886C.0	fGB	0-3344
U+886C.0	fR/S	 145.3
U+886C.0	fTotalStrokes	 8; 9
U+886C.0	fCangjie	LDI
U+886C.0	fHakka	 cin5; cang5
U+886C.0	fCantonese	can3
U+886C.0	fMandarin	chen4
U+886C.0	fEnglish	simplified form of &#x896F;
U+886D.0	fUTF8	衭
U+886D.0	fCNS11643	2-3273
U+886D.0	fBig5	D3D1
U+886D.0	fGB	3-662A
U+886D.0	fR/S	 145.4
U+886D.0	fTotalStrokes	 9; 10
U+886D.0	fCangjie	LQO
U+886D.0	fFourCorner	35230
U+886D.0	fHakka	  fu1; fu5
U+886D.0	fCantonese	fu1; fu3
U+886D.0	fMandarin	fu1
U+886D.0	fEnglish	the lapel of a garment
U+886E.0	fUTF8	衮
U+886E.0	fCNS11643	3-3374
U+886E.0	fGB	0-5972
U+886E.0	fR/S	 145.4
U+886E.0	fTotalStrokes	 9; 10
U+886E.0	fCangjie	YCIHV
U+886E.0	fFourCorner	00730
U+886E.0	fHakka	 gun3; gwun3
U+886E.0	fCantonese	gwan2
U+886E.0	fMandarin	gun3
U+886E.0	fEnglish	[1] the imperial robe with embroidered dragons [2] robes of very high officials
U+886F.0	fUTF8	衯
U+886F.0	fCNS11643	2-327A
U+886F.0	fBig5	D3D8
U+886F.0	fGB	3-6633
U+886F.0	fR/S	 145.4
U+886F.0	fTotalStrokes	 9; 10
U+886F.0	fCangjie	LCSH
U+886F.0	fFourCorner	38227
U+886F.0	fHakka	  fun1
U+886F.0	fCantonese	fan1; pun4
U+886F.0	fMandarin	fen1; pen2
U+886F.0	fEnglish	long flowing robes
U+8870.0	fUTF8	衰
U+8870.0	fCNS11643	1-582E
U+8870.0	fBig5	B049
U+8870.0	fGB	0-4B25
U+8870.0	fR/S	 145.4
U+8870.0	fTotalStrokes	 10
U+8870.0	fCangjie	YWMV
U+8870.0	fFourCorner	00732
U+8870.0	fHakka	  soi1; sui1; so1
U+8870.0	fCantonese	seoi1; ceoi1
U+8870.0	fMandarin	shuai1; suo1; chuai1
U+8870.0	fEnglish	[1] decline; failing (health, etc.); weakening; debility [2] declining or falling (nations, etc.)
U+8870.1	fUTF8	衰
U+8870.1	fCNS11643	1-582E
U+8870.1	fBig5	B049
U+8870.1	fGB	0-4B25
U+8870.1	fR/S	 145.4
U+8870.1	fTotalStrokes	 10
U+8870.1	fCangjie	YWMV
U+8870.1	fFourCorner	00732
U+8870.1	fMandarin	cui1
U+8870.1	fEnglish	[1] order; series; from the top downward [2] mourning garments (of hemp, etc.)
U+8871.0	fUTF8	衱
U+8871.0	fCNS11643	2-3278
U+8871.0	fBig5	D3D6
U+8871.0	fGB	3-6629
U+8871.0	fR/S	 145.4
U+8871.0	fTotalStrokes	 9; 10
U+8871.0	fCangjie	LNHE
U+8871.0	fFourCorner	37247
U+8871.0	fHakka	 giap7; kip7
U+8871.0	fCantonese	gip3; git3; gik3
U+8871.0	fMandarin	jie2
U+8871.0	fEnglish	the hinder skirt of a robe
U+8872.0	fUTF8	衲
U+8872.0	fCNS11643	2-3276
U+8872.0	fBig5	D3D4
U+8872.0	fGB	0-7144
U+8872.0	fR/S	 145.4
U+8872.0	fTotalStrokes	 9; 10
U+8872.0	fCangjie	LOB
U+8872.0	fFourCorner	34227
U+8872.0	fHakka	  nap8; lap8
U+8872.0	fCantonese	naap6; laap6 (also); naap2 (also); laap2 (also)
U+8872.0	fMandarin	na4; rui4
U+8872.0	fEnglish	[1] mend; sew; patch [2] robe of a monk [3] monk
U+8873.0	fUTF8	衳
U+8873.0	fCNS11643	3-3377
U+8873.0	fGB	3-6634
U+8873.0	fR/S	  145.4
U+8873.0	fTotalStrokes	 9; 10
U+8873.0	fCangjie	LCI
U+8873.0	fFourCorner	38230
U+8873.0	fCantonese	zung1
U+8873.0	fMandarin	zhong1; cong3
U+8874.0	fUTF8	衴
U+8874.0	fCNS11643	2-327D
U+8874.0	fBig5	D3DB
U+8874.0	fGB	3-6637
U+8874.0	fR/S	 145.4
U+8874.0	fTotalStrokes	 9; 10
U+8874.0	fCangjie	LLBU
U+8874.0	fFourCorner	34212
U+8874.0	fCantonese	dam2
U+8874.0	fMandarin	dan3
U+8875.0	fUTF8	衵
U+8875.0	fCNS11643	2-3274
U+8875.0	fBig5	D3D2
U+8875.0	fGB	3-662F
U+8875.0	fR/S	 145.4
U+8875.0	fTotalStrokes	 9; 10
U+8875.0	fCangjie	LA
U+8875.0	fFourCorner	36200
U+8875.0	fHakka	 ngit7; ngit8
U+8875.0	fCantonese	nik1; jat6; lik1 (also); nik6
U+8875.0	fMandarin	ri4; ni4
U+8875.0	fEnglish	 women's underwear
U+8876.0	fUTF8	衶
U+8876.0	fCNS11643	2-3275
U+8876.0	fBig5	D3D3
U+8876.0	fGB	3-6630
U+8876.0	fR/S	 145.4
U+8876.0	fTotalStrokes	 9; 10
U+8876.0	fCangjie	LL
U+8876.0	fFourCorner	35206
U+8876.0	fCantonese	zung6
U+8876.0	fMandarin	zhong4; zhong3; chong1
U+8877.0	fUTF8	衷
U+8877.0	fCNS11643	1-582F
U+8877.0	fBig5	B04A
U+8877.0	fGB	0-5654
U+8877.0	fR/S	 145.4
U+8877.0	fTotalStrokes	 10
U+8877.0	fCangjie	YLHV
U+8877.0	fFourCorner	50732
U+8877.0	fHakka	 chung1; cung1; zhung1; zung1
U+8877.0	fCantonese	zung1; cung1 (also)
U+8877.0	fMandarin	zhong1; zhong4
U+8877.0	fEnglish	[1] bottom of one's heart; honest; sincere [2] good and virtuous; goodness [3] proper; appropriate; befitting; propriety [4] undergarments [5] a Chinese family name
U+8878.0	fUTF8	衸
U+8878.0	fCNS11643	3-3372
U+8878.0	fGB	3-6631
U+8878.0	fR/S	  145.4
U+8878.0	fTotalStrokes	 9; 10
U+8878.0	fCangjie	LOLL
U+8878.0	fFourCorner	38220
U+8878.0	fHakka	 gai5; kie5; hai5
U+8878.0	fCantonese	gaai3
U+8878.0	fMandarin	xie4; jie4
U+8878.0	fEnglish	facings on uniforms; long robes
U+8879.0	fUTF8	衹
U+8879.0	fCNS11643	1-5833
U+8879.0	fBig5	B04E
U+8879.0	fGB	1-563B
U+8879.0	fR/S	 145.4
U+8879.0	fTotalStrokes	 9; 10
U+8879.0	fCangjie	LHVP
U+8879.0	fFourCorner	32240
U+8879.0	fHakka	 ki2; zi3
U+8879.0	fCantonese	zi2; kei4
U+8879.0	fMandarin	qi2; zhi1; zhi3; ti3
U+8879.0	fEnglish	only; merely; but
U+887A.0	fUTF8	衺
U+887A.0	fCNS11643	3-3373
U+887A.0	fGB	E-3C7C
U+887A.0	fR/S	  145.4
U+887A.0	fTotalStrokes	 10
U+887A.0	fCangjie	YMHV
U+887A.0	fFourCorner	00730
U+887A.0	fHakka	  sia2
U+887A.0	fCantonese	ce4
U+887A.0	fMandarin	xie2
U+887A.0	fEnglish	evil
U+887B.0	fUTF8	衻
U+887B.0	fCNS11643	3-3375
U+887B.0	fGB	E-3C7D
U+887B.0	fR/S	  145.4
U+887B.0	fTotalStrokes	 9; 10
U+887B.0	fCangjie	LBMM
U+887B.0	fFourCorner	37250
U+887B.0	fCantonese	jim4
U+887B.0	fMandarin	ran2; nian2; chan1
U+887C.0	fUTF8	衼
U+887C.0	fCNS11643	2-327E
U+887C.0	fBig5	D3DC
U+887C.0	fGB	3-662D
U+887C.0	fR/S	 145.4
U+887C.0	fTotalStrokes	 9; 10
U+887C.0	fCangjie	LJE
U+887C.0	fFourCorner	34247
U+887C.0	fCantonese	zi1
U+887C.0	fMandarin	zhi1
U+887D.0	fUTF8	衽
U+887D.0	fCNS11643	1-5832
U+887D.0	fBig5	B04D
U+887D.0	fGB	0-7145
U+887D.0	fR/S	 145.4
U+887D.0	fTotalStrokes	 9; 10
U+887D.0	fCangjie	LHG
U+887D.0	fFourCorner	32214
U+887D.0	fHakka	 ngim5; ngim6; jim5
U+887D.0	fCantonese	jam6; jam5 (also)
U+887D.0	fMandarin	ren4; ren3
U+887D.0	fEnglish	[1] lapel; collar (garment) [2] sleeves [3] arrange (one's lapel, etc.) [4] sleeping mat
U+887E.0	fUTF8	衾
U+887E.0	fCNS11643	2-327C
U+887E.0	fBig5	D3DA
U+887E.0	fGB	0-7440
U+887E.0	fR/S	 145.4
U+887E.0	fTotalStrokes	 10
U+887E.0	fCangjie	OINV
U+887E.0	fFourCorner	80732
U+887E.0	fHakka	 kim1
U+887E.0	fCantonese	kam1
U+887E.0	fMandarin	qin1; qin2
U+887E.0	fEnglish	[1] large coverlet; quilt [2] garments or ress for the deceased
U+887F.0	fUTF8	衿
U+887F.0	fCNS11643	2-3279
U+887F.0	fBig5	D3D7
U+887F.0	fGB	0-7146
U+887F.0	fR/S	 145.4
U+887F.0	fTotalStrokes	 9; 10
U+887F.0	fCangjie	LOIN
U+887F.0	fFourCorner	38227
U+887F.0	fHakka	 kim1; gim; 1; gim1
U+887F.0	fCantonese	kam1; gam1 (also)
U+887F.0	fMandarin	jin1; jin4; qin4; qian2
U+887F.0	fEnglish	[1] the front of a Chinese gown [2] the collar or lapel of a Chinese dress (worn by the literati in former days)
U+8880.0	fUTF8	袀
U+8880.0	fCNS11643	2-3277
U+8880.0	fBig5	D3D5
U+8880.0	fGB	3-6635
U+8880.0	fR/S	 145.4
U+8880.0	fTotalStrokes	 9; 10
U+8880.0	fCangjie	LPMM
U+8880.0	fFourCorner	37220
U+8880.0	fHakka	  giun1
U+8880.0	fCantonese	gwan1
U+8880.0	fMandarin	jun1
U+8880.0	fEnglish	[1] soldier's uniform [2] soldier
U+8881.0	fUTF8	袁
U+8881.0	fCNS11643	1-5830
U+8881.0	fBig5	B04B
U+8881.0	fGB	0-542C
U+8881.0	fR/S	 145.4
U+8881.0	fTotalStrokes	 10
U+8881.0	fCangjie	GRHV
U+8881.0	fFourCorner	40732
U+8881.0	fHakka	 jen2; jan3; ren2; jan2
U+8881.0	fCantonese	jyun4
U+8881.0	fMandarin	yuan2
U+8881.0	fEnglish	[1] the graceful look of a flowing robe [2] a Chinese family name
U+8882.0	fUTF8	袂
U+8882.0	fCNS11643	1-5831
U+8882.0	fBig5	B04C
U+8882.0	fGB	0-7147
U+8882.0	fR/S	 145.4
U+8882.0	fTotalStrokes	 9; 10
U+8882.0	fCangjie	LDK
U+8882.0	fFourCorner	35237
U+8882.0	fHakka	 mi5; mi6; mi3
U+8882.0	fCantonese	mai6
U+8882.0	fMandarin	mei4; yi4; rui4; jue2
U+8882.0	fEnglish	sleeves
U+8883.0	fUTF8	袃
U+8883.0	fCNS11643	2-327B
U+8883.0	fBig5	D3D9
U+8883.0	fGB	E-3C7E
U+8883.0	fR/S	 145.4
U+8883.0	fTotalStrokes	 10
U+8883.0	fCangjie	PHYHV
U+8883.0	fFourCorner	47732
U+8883.0	fCantonese	caai3
U+8883.0	fMandarin	chai4; dai4; di4
U+8884.0	fUTF8	袄
U+8884.0	fCNS11643	3-3376
U+8884.0	fGB	0-3040
U+8884.0	fR/S	  145.4
U+8884.0	fTotalStrokes	 9; 10
U+8884.0	fCangjie	LHK
U+8884.0	fFourCorner	32230
U+8884.0	fHakka	 au3
U+8884.0	fCantonese	ou2; ngou2 (also)
U+8884.0	fMandarin	ao3
U+8884.0	fEnglish	simplified form of &#x8956;
U+8885.0	fUTF8	袅
U+8885.0	fGB	0-7441
U+8885.0	fR/S	 145.4
U+8885.0	fTotalStrokes	 10
U+8885.0	fCangjie	PYSV
U+8885.0	fHakka	 ngiau3
U+8885.0	fCantonese	niu5; liu5 (also)
U+8885.0	fMandarin	niao3
U+8885.0	fEnglish	simplified form of &#x88CA;
U+8886.0	fUTF8	袆
U+8886.0	fGB	8-7E30
U+8886.0	fR/S	 145.4
U+8886.0	fTotalStrokes	 9; 10
U+8886.0	fCangjie	LQS
U+8886.0	fCantonese	fai1
U+8886.0	fMandarin	hui1
U+8886.0	fEnglish	ceremonial gowns of a queen
U+8887.0	fUTF8	袇
U+8887.0	fCNS11643	3-6649
U+8887.0	fGB	E-3D21
U+8887.0	fR/S	 145.4
U+8887.0	fTotalStrokes	 9; 10
U+8887.0	fCangjie	LBY
U+8887.0	fFourCorner	3724
U+8887.0	fCantonese	jim4
U+8887.0	fMandarin	ran2
U+8888.0	fUTF8	袈
U+8888.0	fCNS11643	1-5D36
U+8888.0	fBig5	B350
U+8888.0	fGB	0-7442
U+8888.0	fR/S	 145.5
U+8888.0	fTotalStrokes	 11
U+8888.0	fCangjie	KRYHV
U+8888.0	fFourCorner	46732
U+8888.0	fHakka	 ga1
U+8888.0	fCantonese	gaa1
U+8888.0	fMandarin	jia1
U+8888.0	fEnglish	the cassock or robe of a Buddhist monk; Buddhist cassock
U+8889.0	fUTF8	袉
U+8889.0	fCNS11643	2-3935
U+8889.0	fBig5	D7B2
U+8889.0	fGB	3-664C
U+8889.0	fR/S	 145.5
U+8889.0	fTotalStrokes	 10; 11
U+8889.0	fCangjie	LJP
U+8889.0	fFourCorner	33211
U+8889.0	fCantonese	to4
U+8889.0	fMandarin	tuo2; duo4; tuo3
U+888A.0	fUTF8	袊
U+888A.0	fCNS11643	3-3872
U+888A.0	fGB	3-6647
U+888A.0	fR/S	  145.5
U+888A.0	fTotalStrokes	 10; 11
U+888A.0	fCangjie	LOII
U+888A.0	fFourCorner	38230
U+888A.0	fHakka	 liang3; liang1; liang5
U+888A.0	fCantonese	ling5
U+888A.0	fMandarin	ling3
U+888A.0	fEnglish	underclothing
U+888B.0	fUTF8	袋
U+888B.0	fCNS11643	1-5D3B
U+888B.0	fBig5	B355
U+888B.0	fGB	0-347C
U+888B.0	fR/S	 145.5
U+888B.0	fTotalStrokes	 11
U+888B.0	fCangjie	OPYHV
U+888B.0	fFourCorner	23732
U+888B.0	fHakka	  toi5; toi3; toi6
U+888B.0	fCantonese	doi6; doi2 (also)
U+888B.0	fMandarin	dai4
U+888B.0	fEnglish	 pocket; bag; sack; pouch
U+888B.1	fUTF8	袋
U+888B.1	fCNS11643	1-5D3B
U+888B.1	fBig5	B355
U+888B.1	fGB	0-347C
U+888B.1	fR/S	 145.5
U+888B.1	fTotalStrokes	 11
U+888B.1	fCangjie	OPYHV
U+888B.1	fFourCorner	23732
U+888B.1	fMandarin	dai5
U+888C.0	fUTF8	袌
U+888C.0	fCNS11643	2-3945
U+888C.0	fBig5	D7C2
U+888C.0	fGB	5-3351
U+888C.0	fR/S	 145.5
U+888C.0	fTotalStrokes	 11
U+888C.0	fCangjie	YPUV
U+888C.0	fFourCorner	00732
U+888C.0	fHakka	 pau3; pau2
U+888C.0	fCantonese	pou5; pou4; paau3; po5
U+888C.0	fMandarin	bao4; pao2; pao4
U+888D.0	fUTF8	袍
U+888D.0	fCNS11643	1-5D3A
U+888D.0	fBig5	B354
U+888D.0	fGB	0-455B
U+888D.0	fR/S	 145.5
U+888D.0	fTotalStrokes	 10; 11
U+888D.0	fCangjie	LPRU
U+888D.0	fFourCorner	37212
U+888D.0	fHakka	  pau2; po2; pau3
U+888D.0	fCantonese	pou4; pou2 (also)
U+888D.0	fMandarin	pao2; bao4
U+888D.0	fEnglish	 long gown; robe; cloak
U+888E.0	fUTF8	袎
U+888E.0	fCNS11643	2-3947
U+888E.0	fBig5	D7C4
U+888E.0	fGB	5-654F
U+888E.0	fR/S	 145.5
U+888E.0	fTotalStrokes	 10; 11
U+888E.0	fCangjie	LVIS
U+888E.0	fFourCorner	34227
U+888E.0	fHakka	 ju5; rau1; jeu1; riu5; jau5
U+888E.0	fCantonese	jau3; aau3
U+888E.0	fMandarin	yao4
U+888E.0	fEnglish	the leg of a boot
U+888F.0	fUTF8	袏
U+888F.0	fCNS11643	3-386E
U+888F.0	fGB	3-663D
U+888F.0	fR/S	  145.5
U+888F.0	fTotalStrokes	 10; 11
U+888F.0	fCangjie	LKM
U+888F.0	fFourCorner	34210
U+888F.0	fCantonese	zo3
U+888F.0	fMandarin	zuo4
U+8890.0	fUTF8	袐
U+8890.0	fCNS11643	3-3874
U+8890.0	fGB	5-654E
U+8890.0	fR/S	  145.5
U+8890.0	fTotalStrokes	 10; 11
U+8890.0	fCangjie	LPH
U+8890.0	fFourCorner	33200
U+8890.0	fCantonese	bat1
U+8890.0	fMandarin	bi4
U+8891.0	fUTF8	袑
U+8891.0	fCNS11643	2-393B
U+8891.0	fBig5	D7B8
U+8891.0	fGB	3-664F
U+8891.0	fR/S	 145.5
U+8891.0	fTotalStrokes	 10; 11
U+8891.0	fCangjie	LSHR
U+8891.0	fFourCorner	37262
U+8891.0	fHakka	 shau5; sau5
U+8891.0	fCantonese	siu6
U+8891.0	fMandarin	shao4
U+8891.0	fEnglish	the seat, or fork, of a pair of trousers
U+8892.0	fUTF8	袒
U+8892.0	fCNS11643	1-5D38
U+8892.0	fBig5	B352
U+8892.0	fGB	0-4C3B
U+8892.0	fR/S	 145.5
U+8892.0	fTotalStrokes	 10; 11
U+8892.0	fCangjie	LAM
U+8892.0	fFourCorner	36210
U+8892.0	fHakka	 tan3
U+8892.0	fCantonese	taan2
U+8892.0	fMandarin	tan3; zhan4
U+8892.0	fEnglish	[1] strip; bare; bared; naked [2] protect; screen (with an implication of prejudice)
U+8893.0	fUTF8	袓
U+8893.0	fCNS11643	2-3946
U+8893.0	fBig5	D7C3
U+8893.0	fGB	3-6641
U+8893.0	fR/S	 145.5
U+8893.0	fTotalStrokes	 10; 11
U+8893.0	fCangjie	LBM
U+8893.0	fFourCorner	37210
U+8893.0	fCantonese	zeoi6; ze1
U+8893.0	fMandarin	ju3; jie1
U+8894.0	fUTF8	袔
U+8894.0	fCNS11643	3-386C
U+8894.0	fGB	3-663C
U+8894.0	fR/S	  145.5
U+8894.0	fTotalStrokes	 10; 11
U+8894.0	fCangjie	LMNR
U+8894.0	fFourCorner	31220
U+8894.0	fCantonese	ho6
U+8894.0	fMandarin	he4
U+8895.0	fUTF8	袕
U+8895.0	fCNS11643	2-3936
U+8895.0	fBig5	D7B3
U+8895.0	fGB	3-664B
U+8895.0	fR/S	 145.5
U+8895.0	fTotalStrokes	 10; 11
U+8895.0	fCangjie	LJC
U+8895.0	fFourCorner	33282
U+8895.0	fCantonese	jyut6; seot6
U+8895.0	fMandarin	shu4; xue4
U+8896.0	fUTF8	袖
U+8896.0	fCNS11643	1-5D39
U+8896.0	fBig5	B353
U+8896.0	fGB	0-5064
U+8896.0	fR/S	 145.5
U+8896.0	fTotalStrokes	 10; 11
U+8896.0	fCangjie	LLW
U+8896.0	fFourCorner	35260
U+8896.0	fHakka	 ciu5; ciu6; ciu3; {siu5}; ts'ieu5
U+8896.0	fCantonese	zau6
U+8896.0	fMandarin	xiu4
U+8896.0	fEnglish	[1] sleeve [2] hide or put something in the sleeves
U+8897.0	fUTF8	袗
U+8897.0	fCNS11643	2-3942
U+8897.0	fBig5	D7BF
U+8897.0	fGB	3-6646
U+8897.0	fR/S	 145.5
U+8897.0	fTotalStrokes	 10; 11
U+8897.0	fCangjie	LOHH
U+8897.0	fFourCorner	38222
U+8897.0	fHakka	 zhin3; zin3
U+8897.0	fCantonese	can2; zan2 (also); zan1
U+8897.0	fMandarin	zhen3; zhen1; zhen4
U+8897.0	fEnglish	 unlined garments hem of garment
U+8898.0	fUTF8	袘
U+8898.0	fCNS11643	2-393E
U+8898.0	fBig5	D7BB
U+8898.0	fGB	3-6644
U+8898.0	fR/S	 145.5
U+8898.0	fTotalStrokes	 10; 11
U+8898.0	fCangjie	LOPD
U+8898.0	fFourCorner	38212
U+8898.0	fCantonese	ji4 (also); ji5
U+8898.0	fMandarin	yi2; yi4; yi3
U+8899.0	fUTF8	袙
U+8899.0	fCNS11643	2-3940
U+8899.0	fBig5	D7BD
U+8899.0	fGB	5-654B
U+8899.0	fR/S	 145.5
U+8899.0	fTotalStrokes	 10; 11
U+8899.0	fCangjie	LHA
U+8899.0	fFourCorner	36202
U+8899.0	fHakka	 pa5
U+8899.0	fCantonese	paa3; paak3; mak6; nik1
U+8899.0	fMandarin	pa4; ma4
U+8899.0	fEnglish	 turban
U+889A.0	fUTF8	袚
U+889A.0	fCNS11643	2-393A
U+889A.0	fBig5	D7B7
U+889A.0	fGB	5-654A
U+889A.0	fR/S	 145.5
U+889A.0	fTotalStrokes	 10; 11
U+889A.0	fCangjie	LIKK
U+889A.0	fFourCorner	33244
U+889A.0	fHakka	  pat8;  fut7
U+889A.0	fCantonese	fat1; fai3
U+889A.0	fMandarin	bo1; fu2; fei4
U+889A.0	fEnglish	 greave knee-pads buskins
U+889B.0	fUTF8	袛
U+889B.0	fCNS11643	2-3941
U+889B.0	fBig5	D7BE
U+889B.0	fGB	3-6648
U+889B.0	fR/S	 145.5
U+889B.0	fTotalStrokes	 10; 11
U+889B.0	fCangjie	LHPM
U+889B.0	fFourCorner	32240
U+889B.0	fHakka	 di1; di3
U+889B.0	fCantonese	dai1; zi1
U+889B.0	fMandarin	di1
U+889C.0	fUTF8	袜
U+889C.0	fCNS11643	3-386B
U+889C.0	fGB	0-4D60
U+889C.0	fR/S	 145.5
U+889C.0	fTotalStrokes	 10; 11
U+889C.0	fCangjie	LDJ
U+889C.0	fFourCorner	35290
U+889C.0	fCantonese	mut6; mat2 (also); maat6 (also)
U+889C.0	fMandarin	mo4
U+889C.0	fEnglish	 stomacher
U+889C.1	fUTF8	袜
U+889C.1	fCNS11643	3-386B
U+889C.1	fGB	0-4D60
U+889C.1	fR/S	 145.5
U+889C.1	fTotalStrokes	 10; 11
U+889C.1	fCangjie	LDJ
U+889C.1	fFourCorner	35290
U+889C.1	fHakka	 mat7
U+889C.1	fCantonese	mat6
U+889C.1	fMandarin	wa4
U+889C.1	fEnglish	simplified form of &#x896A;
U+889D.0	fUTF8	袝
U+889D.0	fCNS11643	4-352F
U+889D.0	fGB	3-6645
U+889D.0	fR/S	  145.5
U+889D.0	fTotalStrokes	 10; 11
U+889D.0	fCangjie	LODI
U+889D.0	fFourCorner	34200
U+889D.0	fHakka	 fu5
U+889D.0	fCantonese	fu6
U+889D.0	fMandarin	fu4
U+889E.0	fUTF8	袞
U+889E.0	fCNS11643	1-5D35
U+889E.0	fBig5	B34F
U+889E.0	fGB	E-3D22
U+889E.0	fR/S	 145.5
U+889E.0	fTotalStrokes	 11
U+889E.0	fCangjie	YCRHV
U+889E.0	fFourCorner	00732
U+889E.0	fHakka	 gun3; kun3; gun2; gwun3
U+889E.0	fCantonese	gwan2; kwan2
U+889E.0	fMandarin	gun3
U+889E.0	fEnglish	the ceremonial dress worn by the emperor or very high officials
U+889F.0	fUTF8	袟
U+889F.0	fCNS11643	2-393D
U+889F.0	fBig5	D7BA
U+889F.0	fGB	E-3D23
U+889F.0	fR/S	 145.5
U+889F.0	fTotalStrokes	 10; 11
U+889F.0	fCangjie	LHQO
U+889F.0	fFourCorner	35230
U+889F.0	fHakka	 zhit7; tsiit7
U+889F.0	fCantonese	dit6
U+889F.0	fMandarin	zhi4
U+889F.0	fEnglish	 book cover satchel or bag
U+88A0.0	fUTF8	袠
U+88A0.0	fCNS11643	3-3873
U+88A0.0	fGB	E-3D24
U+88A0.0	fR/S	 145.5
U+88A0.0	fTotalStrokes	 11
U+88A0.0	fCangjie	YHOV
U+88A0.0	fFourCorner	00730
U+88A0.0	fHakka	 zhit7; tsiit7
U+88A0.0	fCantonese	dit6
U+88A0.0	fMandarin	zhi4
U+88A0.0	fEnglish	 book cover satchel or bag
U+88A1.0	fUTF8	袡
U+88A1.0	fCNS11643	2-393C
U+88A1.0	fBig5	D7B9
U+88A1.0	fGB	3-6643
U+88A1.0	fR/S	 145.5
U+88A1.0	fTotalStrokes	 10; 11
U+88A1.0	fCangjie	LGB
U+88A1.0	fFourCorner	35257
U+88A1.0	fHakka	 ngiam1; ngiam2
U+88A1.0	fCantonese	jim4; naam4
U+88A1.0	fMandarin	ran2
U+88A1.0	fEnglish	[1] a woman's dress [2] broad hem [3] knee-pad
U+88A2.0	fUTF8	袢
U+88A2.0	fCNS11643	2-3938
U+88A2.0	fBig5	D7B5
U+88A2.0	fGB	0-7148
U+88A2.0	fR/S	 145.5
U+88A2.0	fTotalStrokes	 10; 11
U+88A2.0	fCangjie	LFQ
U+88A2.0	fFourCorner	39250
U+88A2.0	fHakka	 pan5; fan2
U+88A2.0	fCantonese	paan3; faan4
U+88A2.0	fMandarin	pan4; fan2
U+88A2.0	fEnglish	robe
U+88A3.0	fUTF8	袣
U+88A3.0	fCNS11643	3-386D
U+88A3.0	fGB	3-663A
U+88A3.0	fR/S	  145.5
U+88A3.0	fTotalStrokes	 10; 11
U+88A3.0	fCangjie	LPT
U+88A3.0	fFourCorner	34210
U+88A3.0	fCantonese	ji3
U+88A3.0	fMandarin	yi4
U+88A4.0	fUTF8	袤
U+88A4.0	fCNS11643	2-3943
U+88A4.0	fBig5	D7C0
U+88A4.0	fGB	0-5973
U+88A4.0	fR/S	 145.5
U+88A4.0	fTotalStrokes	 11
U+88A4.0	fCangjie	YNHV
U+88A4.0	fFourCorner	00732
U+88A4.0	fHakka	 meu2;  meu5; miu3; meu6
U+88A4.0	fCantonese	mau6
U+88A4.0	fMandarin	mao4; mou2; mou4
U+88A4.0	fEnglish	longitude; lengthwise; length; from north to south
U+88A5.0	fUTF8	袥
U+88A5.0	fCNS11643	3-386F
U+88A5.0	fGB	3-663E
U+88A5.0	fR/S	  145.5
U+88A5.0	fTotalStrokes	 10; 11
U+88A5.0	fCangjie	LMR
U+88A5.0	fFourCorner	31260
U+88A5.0	fHakka	  tok7
U+88A5.0	fCantonese	tok3
U+88A5.0	fMandarin	tui0; tuo1
U+88A5.0	fEnglish	 loosen the dress at the collar
U+88A6.0	fUTF8	袦
U+88A6.0	fCNS11643	3-3871
U+88A6.0	fGB	3-664E
U+88A6.0	fR/S	  145.5
U+88A6.0	fTotalStrokes	 10; 11
U+88A6.0	fCangjie	LUU
U+88A6.0	fFourCorner	32270
U+88A6.0	fCantonese	neot6; leot6 (also)
U+88A6.0	fMandarin	na4
U+88A7.0	fUTF8	袧
U+88A7.0	fCNS11643	2-393F
U+88A7.0	fBig5	D7BC
U+88A7.0	fGB	3-6649
U+88A7.0	fR/S	 145.5
U+88A7.0	fTotalStrokes	 10; 11
U+88A7.0	fCangjie	LPR
U+88A7.0	fFourCorner	37220
U+88A7.0	fCantonese	gau1 (also); kau1; gau6
U+88A7.0	fMandarin	kou1; gou1; gou4; kou4
U+88A8.0	fUTF8	袨
U+88A8.0	fCNS11643	2-3937
U+88A8.0	fBig5	D7B4
U+88A8.0	fGB	3-664A
U+88A8.0	fR/S	 145.5
U+88A8.0	fTotalStrokes	 10; 11
U+88A8.0	fCangjie	LYVI
U+88A8.0	fFourCorner	30232
U+88A8.0	fHakka	 hien2; hien3; hien5
U+88A8.0	fCantonese	jyun6; jin4
U+88A8.0	fMandarin	xuan4; xian4
U+88A8.0	fEnglish	fine clothes
U+88A9.0	fUTF8	袩
U+88A9.0	fCNS11643	3-3870
U+88A9.0	fGB	3-6640
U+88A9.0	fR/S	  145.5
U+88A9.0	fTotalStrokes	 10; 11
U+88A9.0	fCangjie	LYR
U+88A9.0	fFourCorner	31260
U+88A9.0	fCantonese	dim1
U+88A9.0	fMandarin	chan1
U+88AA.0	fUTF8	袪
U+88AA.0	fCNS11643	2-3939
U+88AA.0	fBig5	D7B6
U+88AA.0	fGB	3-6639
U+88AA.0	fR/S	 145.5
U+88AA.0	fTotalStrokes	 10; 11
U+88AA.0	fCangjie	LGI
U+88AA.0	fFourCorner	34231
U+88AA.0	fHakka	 ki1; kia5; ki2
U+88AA.0	fCantonese	keoi1
U+88AA.0	fMandarin	qu1; qu4
U+88AA.0	fEnglish	[1] sleeves; cuff [2] raise the sleeves; lift up one's arm [3] relieve
U+88AB.0	fUTF8	被
U+88AB.0	fCNS11643	1-5D37
U+88AB.0	fBig5	B351
U+88AB.0	fGB	0-313B
U+88AB.0	fR/S	 145.5
U+88AB.0	fTotalStrokes	 10; 11
U+88AB.0	fCangjie	LDHE
U+88AB.0	fFourCorner	34247
U+88AB.0	fHakka	  pi1; pi3; bi1; bi5; pi5
U+88AB.0	fCantonese	bei6; pei5; pei1
U+88AB.0	fMandarin	bei4; bi4; pi2; pi4
U+88AB.0	fEnglish	[1] bedding; coverlet; quilt [2] cover; shroud [3] spread; reach [4] placed before verbs to show a passive voice [5] because of; due to [6] a Chinese family name
U+88AB.1	fUTF8	被
U+88AB.1	fCNS11643	1-5D37
U+88AB.1	fBig5	B351
U+88AB.1	fGB	0-313B
U+88AB.1	fR/S	 145.5
U+88AB.1	fTotalStrokes	 10; 11
U+88AB.1	fCangjie	LDHE
U+88AB.1	fFourCorner	34247
U+88AB.1	fMandarin	pi1
U+88AB.1	fEnglish	[1] put on; throw on (garment's etc.) [2] open [3] disperse; spread out [4] read desultorily; thumb through
U+88AC.0	fUTF8	袬
U+88AC.0	fCNS11643	2-3944
U+88AC.0	fBig5	D7C1
U+88AC.0	fGB	E-3D25
U+88AC.0	fR/S	 145.5
U+88AC.0	fTotalStrokes	 11
U+88AC.0	fCangjie	YIRV
U+88AC.0	fFourCorner	00732
U+88AC.0	fCantonese	juk6; gwan2
U+88AC.0	fMandarin	yu4; gun3
U+88AD.0	fUTF8	袭
U+88AD.0	fGB	0-4F2E
U+88AD.0	fR/S	 145.5; 212.6
U+88AD.0	fTotalStrokes	 10; 11
U+88AD.0	fCangjie	IPYHV
U+88AD.0	fHakka	 sip8
U+88AD.0	fCantonese	zaap6
U+88AD.0	fMandarin	xi2
U+88AD.0	fEnglish	simplified form of &#x8972;
U+88AE.0	fUTF8	袮
U+88AE.0	fGB	E-3D26
U+88AE.0	fR/S	 145.5
U+88AE.0	fTotalStrokes	 10; 11
U+88AE.0	fCangjie	LOF
U+88AE.0	fCantonese	ji5
U+88AF.0	fUTF8	袯
U+88AF.0	fGB	8-2C5D
U+88AF.0	fR/S	 145.6
U+88AF.0	fTotalStrokes	 11; 12
U+88AF.0	fCangjie	LIVE
U+88AF.0	fCantonese	but6; but3 (also)
U+88AF.0	fMandarin	bo2
U+88AF.0	fEnglish	raincoat
U+88B0.0	fUTF8	袰
U+88B0.0	fGB	E-3D27
U+88B0.0	fR/S	 145.5
U+88B0.0	fTotalStrokes	 11
U+88B0.0	fCangjie	WYYHV
U+88B0.0	fCantonese	mou5
U+88B1.0	fUTF8	袱
U+88B1.0	fCNS11643	1-617C
U+88B1.0	fBig5	B5F6
U+88B1.0	fGB	0-3824
U+88B1.0	fR/S	 145.6
U+88B1.0	fTotalStrokes	 11; 12
U+88B1.0	fCangjie	LOIK
U+88B1.0	fFourCorner	33234
U+88B1.0	fHakka	  fuk8
U+88B1.0	fCantonese	fuk6; fuk2 (also)
U+88B1.0	fMandarin	fu2
U+88B1.0	fEnglish	used in combination for piece of cloth used to wrap bundles
U+88B2.0	fUTF8	袲
U+88B2.0	fCNS11643	2-4033
U+88B2.0	fBig5	DBCD
U+88B2.0	fGB	3-3255
U+88B2.0	fR/S	 145.6
U+88B2.0	fTotalStrokes	 12
U+88B2.0	fCangjie	YNIV
U+88B2.0	fFourCorner	00732
U+88B2.0	fCantonese	ji4
U+88B2.0	fMandarin	yi2; chi3; nuo3
U+88B3.0	fUTF8	袳
U+88B3.0	fCNS11643	4-3A7B
U+88B3.0	fGB	3-665D
U+88B3.0	fR/S	 145.6
U+88B3.0	fTotalStrokes	 11; 12
U+88B3.0	fCangjie	LNIN
U+88B3.0	fFourCorner	37220
U+88B3.0	fCantonese	ci2
U+88B3.0	fMandarin	chi3; qi3; duo3; nuo3
U+88B3.0	fEnglish	spread one's clothes
U+88B4.0	fUTF8	袴
U+88B4.0	fCNS11643	3-3E70
U+88B4.0	fGB	E-3D28
U+88B4.0	fR/S	 145.6
U+88B4.0	fTotalStrokes	 11; 12
U+88B4.0	fCangjie	LKMS
U+88B4.0	fFourCorner	34220
U+88B4.0	fHakka	  fu5;  ku5; ka1; fut5
U+88B4.0	fCantonese	fu3
U+88B4.0	fMandarin	ku4
U+88B4.0	fEnglish	trousers; drawers; breeches; pants; panties
U+88B5.0	fUTF8	袵
U+88B5.0	fCNS11643	3-3E72
U+88B5.0	fGB	E-3D29
U+88B5.0	fR/S	 145.6
U+88B5.0	fTotalStrokes	 11; 12
U+88B5.0	fCangjie	LOHG
U+88B5.0	fFourCorner	32210
U+88B5.0	fHakka	 ngim5; ngim6; jim5
U+88B5.0	fCantonese	jam6; jam5 (also); jam4
U+88B5.0	fMandarin	ren4
U+88B5.0	fEnglish	[1] lapels; collars [2] sleeve [3] bedding
U+88B6.0	fUTF8	袶
U+88B6.0	fCNS11643	2-402F
U+88B6.0	fBig5	DBC9
U+88B6.0	fGB	E-3D2A
U+88B6.0	fR/S	 145.6
U+88B6.0	fTotalStrokes	 11; 12
U+88B6.0	fCangjie	LHEQ
U+88B6.0	fFourCorner	37254
U+88B6.0	fCantonese	gong3
U+88B6.0	fMandarin	jiang4
U+88B7.0	fUTF8	袷
U+88B7.0	fCNS11643	2-4031
U+88B7.0	fBig5	DBCB
U+88B7.0	fGB	0-714A
U+88B7.0	fR/S	 145.6
U+88B7.0	fTotalStrokes	 11; 12
U+88B7.0	fCangjie	LOMR
U+88B7.0	fFourCorner	38261
U+88B7.0	fHakka	  giap7; gap7; dap7
U+88B7.0	fCantonese	gip3; hap1; gaap3
U+88B7.0	fMandarin	jia2; qia1; xia2; yi2
U+88B7.0	fEnglish	lined garment; garment with linings
U+88B7.1	fUTF8	袷
U+88B7.1	fCNS11643	2-4031
U+88B7.1	fBig5	DBCB
U+88B7.1	fGB	0-714A
U+88B7.1	fR/S	 145.6
U+88B7.1	fTotalStrokes	 11; 12
U+88B7.1	fCangjie	LOMR
U+88B7.1	fFourCorner	38261
U+88B7.1	fMandarin	jie2
U+88B7.1	fEnglish	the lapel or collar of robes, or, in former times, of ceremonial dress in imperial audience
U+88B8.0	fUTF8	袸
U+88B8.0	fCNS11643	2-402C
U+88B8.0	fBig5	DBC6
U+88B8.0	fGB	3-6656
U+88B8.0	fR/S	 145.6
U+88B8.0	fTotalStrokes	 11; 12
U+88B8.0	fCangjie	LKLD
U+88B8.0	fFourCorner	34247
U+88B8.0	fCantonese	zin6; cyun4; zyun6
U+88B8.0	fMandarin	cun2; jian4; zun4
U+88B9.0	fUTF8	袹
U+88B9.0	fCNS11643	2-402B
U+88B9.0	fBig5	DBC5
U+88B9.0	fGB	3-6654
U+88B9.0	fR/S	 145.6
U+88B9.0	fTotalStrokes	 11; 12
U+88B9.0	fCangjie	LMA
U+88B9.0	fFourCorner	31260
U+88B9.0	fCantonese	baak3; mat6; mak6
U+88B9.0	fMandarin	mo4
U+88BA.0	fUTF8	袺
U+88BA.0	fCNS11643	2-4029
U+88BA.0	fBig5	DBC3
U+88BA.0	fGB	3-6653
U+88BA.0	fR/S	 145.6
U+88BA.0	fTotalStrokes	 11; 12
U+88BA.0	fCangjie	LGR
U+88BA.0	fFourCorner	34261
U+88BA.0	fHakka	  giet7; get7
U+88BA.0	fCantonese	git3
U+88BA.0	fMandarin	jie2; jie1; jia2
U+88BA.0	fEnglish	hold up
U+88BB.0	fUTF8	袻
U+88BB.0	fCNS11643	3-3E6D
U+88BB.0	fGB	3-6655
U+88BB.0	fR/S	  145.6
U+88BB.0	fTotalStrokes	 11; 12
U+88BB.0	fCangjie	LMBL
U+88BB.0	fFourCorner	31220
U+88BB.0	fCantonese	ji4
U+88BB.0	fMandarin	er2
U+88BC.0	fUTF8	袼
U+88BC.0	fCNS11643	2-4030
U+88BC.0	fBig5	DBCA
U+88BC.0	fGB	0-714B
U+88BC.0	fR/S	 145.6
U+88BC.0	fTotalStrokes	 11; 12
U+88BC.0	fCangjie	LHER
U+88BC.0	fFourCorner	37264
U+88BC.0	fHakka	 lok7; gok7; get7; lok8
U+88BC.0	fCantonese	gok3; gaak3
U+88BC.0	fMandarin	luo4; ge1; ge4
U+88BC.0	fEnglish	[1] child's bib [2] short-sleeved jacket
U+88BD.0	fUTF8	袽
U+88BD.0	fCNS11643	2-4032
U+88BD.0	fBig5	DBCC
U+88BD.0	fGB	3-6662
U+88BD.0	fR/S	 145.6
U+88BD.0	fTotalStrokes	 11; 12
U+88BD.0	fCangjie	LVR
U+88BD.0	fFourCorner	36200
U+88BD.0	fHakka	 ji2; ri2
U+88BD.0	fCantonese	jyu4
U+88BD.0	fMandarin	ru2; nu2
U+88BD.0	fEnglish	clothes
U+88BE.0	fUTF8	袾
U+88BE.0	fCNS11643	2-402E
U+88BE.0	fBig5	DBC8
U+88BE.0	fGB	3-665A
U+88BE.0	fR/S	 145.6
U+88BE.0	fTotalStrokes	 11; 12
U+88BE.0	fCangjie	LHJD
U+88BE.0	fFourCorner	35290
U+88BE.0	fHakka	  zhu1; zu1
U+88BE.0	fCantonese	zyu1
U+88BE.0	fMandarin	zhu1; shu1
U+88BE.0	fEnglish	red garments, elegant
U+88BF.0	fUTF8	袿
U+88BF.0	fCNS11643	3-3E6F
U+88BF.0	fGB	3-6652
U+88BF.0	fR/S	 145.6
U+88BF.0	fTotalStrokes	 11; 12
U+88BF.0	fCangjie	LGG
U+88BF.0	fFourCorner	34210
U+88BF.0	fHakka	 gwa1; gui1; gwi1; kwui1
U+88BF.0	fCantonese	gwai1; gwaa3
U+88BF.0	fMandarin	gui1
U+88BF.0	fEnglish	gown
U+88C0.0	fUTF8	裀
U+88C0.0	fCNS11643	2-402D
U+88C0.0	fBig5	DBC7
U+88C0.0	fGB	3-6659
U+88C0.0	fR/S	 145.6
U+88C0.0	fTotalStrokes	 11; 12
U+88C0.0	fCangjie	LWK
U+88C0.0	fFourCorner	36200
U+88C0.0	fHakka	 jin1; rin1
U+88C0.0	fCantonese	jan1
U+88C0.0	fMandarin	yin1
U+88C0.0	fEnglish	 mat; coverlet underwear
U+88C1.0	fUTF8	裁
U+88C1.0	fCNS11643	1-617A
U+88C1.0	fBig5	B5F4
U+88C1.0	fGB	0-3243
U+88C1.0	fR/S	 145.6
U+88C1.0	fTotalStrokes	 12
U+88C1.0	fCangjie	JIYHV
U+88C1.0	fFourCorner	43750
U+88C1.0	fHakka	  cai2; cai3; coi2
U+88C1.0	fCantonese	coi4
U+88C1.0	fMandarin	cai2; zai4
U+88C1.0	fEnglish	[1] cut paper, cloth, etc. with a knife or scissors; cut out; cut off [2] diminish; decrease; reduce [3] delete [4] consider; discern; decide; judge [5] form; style [6] sanctions [7] weight; measure [8] kill
U+88C2.0	fUTF8	裂
U+88C2.0	fCNS11643	1-617B
U+88C2.0	fBig5	B5F5
U+88C2.0	fGB	0-4151
U+88C2.0	fR/S	 145.6
U+88C2.0	fTotalStrokes	 12
U+88C2.0	fCangjie	MNYHV
U+88C2.0	fFourCorner	12732
U+88C2.0	fHakka	  liet8; let8; liat8
U+88C2.0	fCantonese	lit6; lit3
U+88C2.0	fMandarin	lie4; lie3; li4
U+88C2.0	fEnglish	[1] split; crack; break [2] split; divide up (profits, etc.); rend; sever; rip open; break open
U+88C3.0	fUTF8	裃
U+88C3.0	fGB	E-3D2B
U+88C3.0	fR/S	 145.6
U+88C3.0	fTotalStrokes	 11; 12
U+88C3.0	fCangjie	LYMY
U+88C3.0	fCantonese	lung6
U+88C3.0	fEnglish	old ceremonial garb; samurai garb
U+88C4.0	fUTF8	裄
U+88C4.0	fGB	E-3D2C
U+88C4.0	fR/S	 145.6
U+88C4.0	fTotalStrokes	 11; 12
U+88C4.0	fCangjie	LHON
U+88C4.0	fFourCorner	3122
U+88C4.0	fCantonese	hang4
U+88C4.0	fEnglish	sleeve length
U+88C5.0	fUTF8	装
U+88C5.0	fGB	0-5730
U+88C5.0	fR/S	 145.6
U+88C5.0	fTotalStrokes	 12
U+88C5.0	fCangjie	LGYHV
U+88C5.0	fHakka	 zong1
U+88C5.0	fCantonese	zong1
U+88C5.0	fMandarin	zhuang1; zhuan1; zhuang4
U+88C5.0	fEnglish	simplified form of &#x88DD;
U+88C6.0	fUTF8	裆
U+88C6.0	fGB	0-7149
U+88C6.0	fR/S	 145.6
U+88C6.0	fTotalStrokes	 11; 12
U+88C6.0	fCangjie	LFSM
U+88C6.0	fHakka	 dong1; long5; nong5
U+88C6.0	fCantonese	dong1
U+88C6.0	fMandarin	dang1
U+88C6.0	fEnglish	simplified form of &#x8960;
U+88C7.0	fUTF8	裇
U+88C7.0	fGB	1-7C4F
U+88C7.0	fR/S	 145.6
U+88C7.0	fTotalStrokes	 11; 12
U+88C7.0	fCangjie	LHBT
U+88C7.0	fCantonese	seot1; seot7
U+88C7.0	fEnglish	shirt
U+88C8.0	fUTF8	裈
U+88C8.0	fGB	8-7E33
U+88C8.0	fR/S	 145.6
U+88C8.0	fTotalStrokes	 11; 12
U+88C8.0	fCangjie	LBKQ
U+88C8.0	fCantonese	gwan1
U+88C8.0	fMandarin	kun1
U+88C8.0	fEnglish	trousers, pants, shorts, drawers
U+88C9.0	fUTF8	裉
U+88C9.0	fCNS11643	2-4035
U+88C9.0	fBig5	DBCF
U+88C9.0	fGB	0-714C
U+88C9.0	fR/S	 145.6
U+88C9.0	fTotalStrokes	 11; 12
U+88C9.0	fCangjie	LAV
U+88C9.0	fFourCorner	37232
U+88C9.0	fHakka	 ken5; gen5
U+88C9.0	fCantonese	kang3
U+88C9.0	fMandarin	ken4
U+88C9.0	fEnglish	the seams below the sleeves in a garment
U+88CA.0	fUTF8	裊
U+88CA.0	fCNS11643	1-6654
U+88CA.0	fBig5	B8CD
U+88CA.0	fGB	1-7441
U+88CA.0	fR/S	 145.7
U+88CA.0	fTotalStrokes	 13
U+88CA.0	fCangjie	HAYV
U+88CA.0	fFourCorner	27732
U+88CA.0	fHakka	 ngiau3; ngiau1; ngieu3
U+88CA.0	fCantonese	niu5; liu5 (also); niu1
U+88CA.0	fMandarin	niao3
U+88CA.0	fEnglish	[1] curling up(wards) (smoke, etc.); wavering gently [2] around; all around, as sound of music or voices of spring
U+88CB.0	fUTF8	裋
U+88CB.0	fCNS11643	2-4739
U+88CB.0	fBig5	DFF2
U+88CB.0	fGB	3-6664
U+88CB.0	fR/S	 145.7
U+88CB.0	fTotalStrokes	 12; 13
U+88CB.0	fCangjie	LMRT
U+88CB.0	fFourCorner	31218
U+88CB.0	fHakka	 shu2; su2; shu5
U+88CB.0	fCantonese	syu6
U+88CB.0	fMandarin	shu4; duan3
U+88CB.0	fEnglish	cotton clothes of a boy servant
U+88CC.0	fUTF8	裌
U+88CC.0	fCNS11643	2-473F
U+88CC.0	fBig5	DFF8
U+88CC.0	fGB	E-3D2D
U+88CC.0	fR/S	 145.7
U+88CC.0	fTotalStrokes	 12; 13
U+88CC.0	fCangjie	LKOO
U+88CC.0	fFourCorner	34238
U+88CC.0	fHakka	 gap7;  giap7
U+88CC.0	fCantonese	gaap3; kwan2; cing4; leoi5; lei5; gyun1; san1; pau4; fau4 (also)
U+88CC.0	fMandarin	jia2; xie4
U+88CC.0	fEnglish	a lined garment or dress
U+88CD.0	fUTF8	裍
U+88CD.0	fCNS11643	2-473A
U+88CD.0	fBig5	DFF3
U+88CD.0	fGB	5-6555
U+88CD.0	fR/S	 145.7
U+88CD.0	fTotalStrokes	 12; 13
U+88CD.0	fCangjie	LWD
U+88CD.0	fFourCorner	36200
U+88CD.0	fHakka	 kun3; kwun3
U+88CD.0	fCantonese	kwan2
U+88CD.0	fMandarin	kun3
U+88CD.0	fEnglish	a border or band on the edge of a dress
U+88CE.0	fUTF8	裎
U+88CE.0	fCNS11643	2-473B
U+88CE.0	fBig5	DFF4
U+88CE.0	fGB	0-714E
U+88CE.0	fR/S	 145.7
U+88CE.0	fTotalStrokes	 12; 13
U+88CE.0	fCangjie	LRHG
U+88CE.0	fFourCorner	36214
U+88CE.0	fHakka	 chin2; cihn2
U+88CE.0	fCantonese	cing4; cing2
U+88CE.0	fMandarin	cheng2; cheng3; zheng4
U+88CE.0	fEnglish	bare; nude; naked; bare
U+88CF.0	fUTF8	裏
U+88CF.0	fCNS11643	3-444E
U+88CF.0	fGB	1-406F
U+88CF.0	fR/S	  145.7
U+88CF.0	fTotalStrokes	 13
U+88CF.0	fCangjie	YWGV
U+88CF.0	fFourCorner	00732
U+88CF.0	fHakka	  li3; li1; di1; {li1}; li2
U+88CF.0	fCantonese	lei5; lei2 (also); leoi5
U+88CF.0	fMandarin	li3
U+88CF.0	fEnglish	[1] inside; within [2] used to indicate time of day, night, a season, etc. [3] the lining of a dress or clothes
U+88CF.1	fUTF8	裏
U+88CF.1	fCNS11643	3-444E
U+88CF.1	fGB	1-406F
U+88CF.1	fR/S	  145.7
U+88CF.1	fTotalStrokes	 13
U+88CF.1	fCangjie	YWGV
U+88CF.1	fFourCorner	00732
U+88CF.1	fMandarin	li5
U+88CF.1	fEnglish	used in combination for here, there
U+88D0.0	fUTF8	裐
U+88D0.0	fCNS11643	2-4740
U+88D0.0	fBig5	DFF9
U+88D0.0	fGB	E-3D2E
U+88D0.0	fR/S	 145.7
U+88D0.0	fTotalStrokes	 12; 13
U+88D0.0	fCangjie	LRB
U+88D0.0	fFourCorner	36227
U+88D0.0	fCantonese	gyun1
U+88D0.0	fMandarin	juan1
U+88D1.0	fUTF8	裑
U+88D1.0	fCNS11643	5-426C
U+88D1.0	fGB	5-6556
U+88D1.0	fR/S	  145.7
U+88D1.0	fTotalStrokes	 12; 13
U+88D1.0	fCangjie	LHXH
U+88D1.0	fFourCorner	37240
U+88D1.0	fMandarin	shen1
U+88D2.0	fUTF8	裒
U+88D2.0	fCNS11643	1-6656
U+88D2.0	fBig5	B8CF
U+88D2.0	fGB	0-5976
U+88D2.0	fR/S	 145.7
U+88D2.0	fTotalStrokes	 13
U+88D2.0	fCangjie	YHXV
U+88D2.0	fFourCorner	00732
U+88D2.0	fHakka	  piau1;  feu2; peu1
U+88D2.0	fCantonese	fau4; pau4
U+88D2.0	fMandarin	pou2; bao1; fou2
U+88D2.0	fEnglish	[1] collect; gather; assemble; scrape together [2] reduce; deduct
U+88D3.0	fUTF8	裓
U+88D3.0	fCNS11643	4-4061
U+88D3.0	fGB	3-6663
U+88D3.0	fR/S	  145.7
U+88D3.0	fTotalStrokes	 12; 13
U+88D3.0	fCangjie	LIT
U+88D3.0	fFourCorner	33250
U+88D3.0	fCantonese	gat1; gaai1
U+88D3.0	fMandarin	ge2; jie1
U+88D4.0	fUTF8	裔
U+88D4.0	fCNS11643	1-664E
U+88D4.0	fBig5	B8C7
U+88D4.0	fGB	0-5261
U+88D4.0	fR/S	 145.7
U+88D4.0	fTotalStrokes	 13
U+88D4.0	fCangjie	YVBCR
U+88D4.0	fFourCorner	00227
U+88D4.0	fHakka	 ji3; ri3; i5
U+88D4.0	fCantonese	jeoi6
U+88D4.0	fMandarin	yi4
U+88D4.0	fEnglish	[1] progeny; descendants; posterity [2] the hem of a garment, robe, etc. [3] remote or border regions [4] a general name of northern barbarians [5] a Chinese family name
U+88D5.0	fUTF8	裕
U+88D5.0	fCNS11643	1-6655
U+88D5.0	fBig5	B8CE
U+88D5.0	fGB	0-5423
U+88D5.0	fR/S	 145.7
U+88D5.0	fTotalStrokes	 12; 13
U+88D5.0	fCangjie	LCOR
U+88D5.0	fFourCorner	38268
U+88D5.0	fHakka	 ji5; ji3; ri5; i5
U+88D5.0	fCantonese	jyu6
U+88D5.0	fMandarin	yu4
U+88D5.0	fEnglish	[1] abundant; affluent; rich; plenty; plentiful; be abundant [2] tolerant (administration, etc.); lenient (punishment, etc.) [3] generous; magnanimous [4] slowly; take time
U+88D6.0	fUTF8	裖
U+88D6.0	fCNS11643	2-4738
U+88D6.0	fBig5	DFF1
U+88D6.0	fGB	3-6667
U+88D6.0	fR/S	 145.7
U+88D6.0	fTotalStrokes	 12; 13
U+88D6.0	fCangjie	LMMV
U+88D6.0	fFourCorner	31232
U+88D6.0	fCantonese	zan3
U+88D6.0	fMandarin	zhen3; chen3; zhen4
U+88D7.0	fUTF8	裗
U+88D7.0	fCNS11643	2-402A
U+88D7.0	fBig5	DBC4
U+88D7.0	fGB	5-655A
U+88D7.0	fR/S	 145.7
U+88D7.0	fTotalStrokes	 12; 13
U+88D7.0	fCangjie	LYIU
U+88D7.0	fFourCorner	30213
U+88D7.0	fHakka	 liu2
U+88D7.0	fCantonese	lau4
U+88D7.0	fMandarin	liu2
U+88D8.0	fUTF8	裘
U+88D8.0	fCNS11643	1-6651
U+88D8.0	fBig5	B8CA
U+88D8.0	fGB	0-7443
U+88D8.0	fR/S	 145.7
U+88D8.0	fTotalStrokes	 13
U+88D8.0	fCangjie	IEYHV
U+88D8.0	fFourCorner	43732
U+88D8.0	fHakka	 kiu2; k'ieu2; kiu3
U+88D8.0	fCantonese	kau4
U+88D8.0	fMandarin	qiu2; ju2
U+88D8.0	fEnglish	[1] fur garments [2] a Chinese family name
U+88D9.0	fUTF8	裙
U+88D9.0	fCNS11643	1-664F
U+88D9.0	fBig5	B8C8
U+88D9.0	fGB	0-4839
U+88D9.0	fR/S	 145.7
U+88D9.0	fTotalStrokes	 12; 13
U+88D9.0	fCangjie	LSKR
U+88D9.0	fFourCorner	37267
U+88D9.0	fHakka	  kiun2; kiun3
U+88D9.0	fCantonese	kwan4; kwan2 (also)
U+88D9.0	fMandarin	qun2
U+88D9.0	fEnglish	 skirt; apron; petticoat
U+88DA.0	fUTF8	裚
U+88DA.0	fCNS11643	2-473E
U+88DA.0	fBig5	DFF7
U+88DA.0	fGB	5-653C
U+88DA.0	fR/S	 145.7
U+88DA.0	fTotalStrokes	 13
U+88DA.0	fCangjie	QLYHV
U+88DA.0	fFourCorner	52732
U+88DA.0	fCantonese	zai3
U+88DA.0	fMandarin	ji4
U+88DB.0	fUTF8	裛
U+88DB.0	fCNS11643	2-473D
U+88DB.0	fBig5	DFF6
U+88DB.0	fGB	3-3256
U+88DB.0	fR/S	 145.7
U+88DB.0	fTotalStrokes	 13
U+88DB.0	fCangjie	YRAV
U+88DB.0	fFourCorner	00732
U+88DB.0	fHakka	 jip7; rip7
U+88DB.0	fCantonese	jap1
U+88DB.0	fMandarin	yi4; ye4
U+88DB.0	fEnglish	[1] wrap and bind [2] damp; dripping; wet [3] book bag
U+88DC.0	fUTF8	補
U+88DC.0	fCNS11643	1-6650
U+88DC.0	fBig5	B8C9
U+88DC.0	fGB	1-3239
U+88DC.0	fR/S	 145.7
U+88DC.0	fTotalStrokes	 12; 13
U+88DC.0	fCangjie	LIJB
U+88DC.0	fFourCorner	33227
U+88DC.0	fHakka	  bu3; bu1
U+88DC.0	fCantonese	bou2
U+88DC.0	fMandarin	bu3
U+88DC.0	fEnglish	[1] mend; patch; fix; repair; restore; fill [2] add up; supplement; supply [3] addenda; supplements; replenishments; complements [4] nutritious; nutrient [5] rich foods; tonics [6] mourish [7] make up; help (finance, etc.); subsidise [8] appoint to; fill a post [9] be of help; benefit; use
U+88DD.0	fUTF8	裝
U+88DD.0	fCNS11643	1-6652
U+88DD.0	fBig5	B8CB
U+88DD.0	fGB	1-5730
U+88DD.0	fR/S	 145.7
U+88DD.0	fTotalStrokes	 13
U+88DD.0	fCangjie	VGYHV
U+88DD.0	fFourCorner	24732
U+88DD.0	fHakka	  zong1; zhong1
U+88DD.0	fCantonese	zong1
U+88DD.0	fMandarin	zhuang1; zhuan1; zhuang4
U+88DD.0	fEnglish	[1] fill in; fill up; pack; load [2] pretend; feign [3] adorn; dress; make up; ornamental dressing; decorate (room, etc.) [4] disguise [5] store up; keep [6] install (machines; equipment, etc.) [7] clothes and personal effects; outfit
U+88DE.0	fUTF8	裞
U+88DE.0	fCNS11643	2-473C
U+88DE.0	fBig5	DFF5
U+88DE.0	fGB	3-666B
U+88DE.0	fR/S	 145.7
U+88DE.0	fTotalStrokes	 12; 13
U+88DE.0	fCangjie	LCRU
U+88DE.0	fFourCorner	38216
U+88DE.0	fCantonese	seoi3
U+88DE.0	fMandarin	shui4; tui4; zhui4
U+88DF.0	fUTF8	裟
U+88DF.0	fCNS11643	1-664D
U+88DF.0	fBig5	B8C6
U+88DF.0	fGB	0-7444
U+88DF.0	fR/S	 145.7
U+88DF.0	fTotalStrokes	 13
U+88DF.0	fCangjie	EHYHV
U+88DF.0	fFourCorner	39732
U+88DF.0	fHakka	  sa1
U+88DF.0	fCantonese	saa1
U+88DF.0	fMandarin	sha1
U+88DF.0	fEnglish	 cassock or robe of a Buddhist monk
U+88E0.0	fUTF8	裠
U+88E0.0	fCNS11643	3-444F
U+88E0.0	fGB	E-3D2F
U+88E0.0	fR/S	  145.7
U+88E0.0	fTotalStrokes	 13
U+88E0.0	fCangjie	SRYHV
U+88E0.0	fFourCorner	17730
U+88E0.0	fHakka	 kiun2
U+88E0.0	fCantonese	kwan4
U+88E0.0	fMandarin	qun2
U+88E0.0	fEnglish	a short skirt
U+88E1.0	fUTF8	裡
U+88E1.0	fCNS11643	1-6653
U+88E1.0	fBig5	B8CC
U+88E1.0	fGB	E-3D30
U+88E1.0	fR/S	 145.7
U+88E1.0	fTotalStrokes	 12; 13
U+88E1.0	fCangjie	LWG
U+88E1.0	fFourCorner	36214
U+88E1.0	fHakka	 li3;  li1; di1; ngia3; {li1}; ja3; ye3; le2; li2; i3
U+88E1.0	fCantonese	leoi5; lei5
U+88E1.0	fMandarin	li3; li4
U+88E1.0	fEnglish	[1] inside; within [2] used to indicate time of day, night, a season, etc. [3] the lining of a dress or clothes
U+88E1.1	fUTF8	裡
U+88E1.1	fCNS11643	1-6653
U+88E1.1	fBig5	B8CC
U+88E1.1	fGB	E-3D30
U+88E1.1	fR/S	 145.7
U+88E1.1	fTotalStrokes	 12; 13
U+88E1.1	fCangjie	LWG
U+88E1.1	fFourCorner	36214
U+88E2.0	fUTF8	裢
U+88E2.0	fGB	0-714D
U+88E2.0	fR/S	 145.7
U+88E2.0	fTotalStrokes	 12; 13
U+88E2.0	fCangjie	XLYKQ
U+88E2.0	fHakka	 lien2
U+88E2.0	fCantonese	lin4
U+88E2.0	fMandarin	lian2
U+88E2.0	fEnglish	simplified form of &#x8933;
U+88E3.0	fUTF8	裣
U+88E3.0	fGB	0-714F
U+88E3.0	fR/S	 145.7
U+88E3.0	fTotalStrokes	 12; 13
U+88E3.0	fCangjie	LOMM
U+88E3.0	fHakka	 liam2
U+88E3.0	fCantonese	lim6 (also); lim5
U+88E3.0	fMandarin	lian4; lian3; chan1; lian2
U+88E3.0	fEnglish	simplified form of &#x895D;
U+88E4.0	fUTF8	裤
U+88E4.0	fGB	0-3F63
U+88E4.0	fR/S	 145.7
U+88E4.0	fTotalStrokes	 12; 13
U+88E4.0	fCangjie	LIKQ
U+88E4.0	fHakka	 fu5; ku5
U+88E4.0	fCantonese	fu3
U+88E4.0	fMandarin	ku4
U+88E4.0	fEnglish	simplified form of &#x8932;
U+88E5.0	fUTF8	裥
U+88E5.0	fGB	0-7150
U+88E5.0	fR/S	 145.7
U+88E5.0	fTotalStrokes	 12; 13
U+88E5.0	fCangjie	LLSA
U+88E5.0	fHakka	 gan3; gan5
U+88E5.0	fCantonese	gaan2; gan2
U+88E5.0	fMandarin	jian3
U+88E5.0	fEnglish	simplified form of &#x8947;
U+88E6.0	fUTF8	裦
U+88E6.0	fCNS11643	3-4E60
U+88E6.0	fGB	E-3D31
U+88E6.0	fR/S	 145.9
U+88E6.0	fTotalStrokes	 15
U+88E6.0	fCangjie	YHHV
U+88E6.0	fFourCorner	00730
U+88E6.0	fCantonese	bou1
U+88E6.0	fMandarin	fou2
U+88E7.0	fUTF8	裧
U+88E7.0	fCNS11643	2-4D7D
U+88E7.0	fBig5	E3F6
U+88E7.0	fGB	5-656B
U+88E7.0	fR/S	 145.8
U+88E7.0	fTotalStrokes	 13; 14
U+88E7.0	fCangjie	LFF
U+88E7.0	fFourCorner	39289
U+88E7.0	fHakka	 zham1; zam1
U+88E7.0	fCantonese	sim4 (also); cim1; zim1
U+88E7.0	fMandarin	chan1; tan3; chan4
U+88E7.0	fEnglish	curtain; screen; fringe
U+88E8.0	fUTF8	裨
U+88E8.0	fCNS11643	1-6A7C
U+88E8.0	fBig5	BB74
U+88E8.0	fGB	0-7154
U+88E8.0	fR/S	 145.8
U+88E8.0	fTotalStrokes	 13; 14
U+88E8.0	fCangjie	LHWJ
U+88E8.0	fFourCorner	36240
U+88E8.0	fHakka	 bi1; pi1
U+88E8.0	fCantonese	pei4; bei1
U+88E8.0	fMandarin	bi4; bei1
U+88E8.0	fEnglish	[1] aid; supplement [2] benefit; help
U+88E8.1	fUTF8	裨
U+88E8.1	fCNS11643	1-6A7C
U+88E8.1	fBig5	BB74
U+88E8.1	fGB	0-7154
U+88E8.1	fR/S	 145.8
U+88E8.1	fTotalStrokes	 13; 14
U+88E8.1	fCangjie	LHWJ
U+88E8.1	fFourCorner	36240
U+88E8.1	fMandarin	pi2
U+88E8.1	fEnglish	[1] small; petty [2] subordinate [3] a Chinese family name
U+88E9.0	fUTF8	裩
U+88E9.0	fCNS11643	3-4959
U+88E9.0	fGB	5-6564
U+88E9.0	fR/S	  145.8
U+88E9.0	fTotalStrokes	 13; 14
U+88E9.0	fCangjie	LAPP
U+88E9.0	fFourCorner	36210
U+88E9.0	fHakka	 gun3; kun3
U+88E9.0	fCantonese	gwan1
U+88E9.0	fMandarin	gun1
U+88EA.0	fUTF8	裪
U+88EA.0	fCNS11643	3-4952
U+88EA.0	fGB	5-656A
U+88EA.0	fR/S	 145.8
U+88EA.0	fTotalStrokes	 13; 14
U+88EA.0	fCangjie	LPOU
U+88EA.0	fFourCorner	37220
U+88EA.0	fHakka	  tau2; to2
U+88EA.0	fCantonese	tou4
U+88EA.0	fMandarin	tao2
U+88EA.0	fEnglish	the sleeve
U+88EB.0	fUTF8	裫
U+88EB.0	fCNS11643	2-4E2A
U+88EB.0	fBig5	E442
U+88EB.0	fGB	3-6727
U+88EB.0	fR/S	 145.8
U+88EB.0	fTotalStrokes	 13; 14
U+88EB.0	fCangjie	LLXL
U+88EB.0	fFourCorner	32200
U+88EB.0	fCantonese	jyun3; jyun1
U+88EB.0	fMandarin	yuan4; yuan1; yung4
U+88EC.0	fUTF8	裬
U+88EC.0	fCNS11643	2-4E29
U+88EC.0	fBig5	E441
U+88EC.0	fGB	5-655C
U+88EC.0	fR/S	 145.8
U+88EC.0	fTotalStrokes	 13; 14
U+88EC.0	fCangjie	LGCE
U+88EC.0	fFourCorner	34247
U+88EC.0	fCantonese	ling4
U+88EC.0	fMandarin	ling2
U+88ED.0	fUTF8	裭
U+88ED.0	fCNS11643	3-4957
U+88ED.0	fGB	E-3D32
U+88ED.0	fR/S	 145.8
U+88ED.0	fTotalStrokes	 13; 14
U+88ED.0	fCangjie	LYPU
U+88ED.0	fFourCorner	31210
U+88ED.0	fCantonese	ci2
U+88ED.0	fMandarin	chi3
U+88EE.0	fUTF8	裮
U+88EE.0	fCNS11643	2-4E24
U+88EE.0	fBig5	E3FB
U+88EE.0	fGB	5-6565
U+88EE.0	fR/S	 145.8
U+88EE.0	fTotalStrokes	 13; 14
U+88EE.0	fCangjie	LAA
U+88EE.0	fFourCorner	36260
U+88EE.0	fHakka	 chong1; cong1
U+88EE.0	fCantonese	coeng1
U+88EE.0	fMandarin	chang1
U+88EF.0	fUTF8	裯
U+88EF.0	fCNS11643	1-6A7E
U+88EF.0	fBig5	BB76
U+88EF.0	fGB	3-6670
U+88EF.0	fR/S	 145.8
U+88EF.0	fTotalStrokes	 13; 14
U+88EF.0	fCangjie	LBGR
U+88EF.0	fFourCorner	37220
U+88EF.0	fHakka	 chiu2; ciu2; chu2
U+88EF.0	fCantonese	cau4; dou1
U+88EF.0	fMandarin	chou2; dao1; chu2; diao1
U+88EF.0	fEnglish	 coverlet bedspread
U+88F0.0	fUTF8	裰
U+88F0.0	fCNS11643	2-4E28
U+88F0.0	fBig5	E440
U+88F0.0	fGB	0-7156
U+88F0.0	fR/S	 145.8
U+88F0.0	fTotalStrokes	 13; 14
U+88F0.0	fCangjie	LEEE
U+88F0.0	fFourCorner	37247
U+88F0.0	fHakka	  dot7
U+88F0.0	fCantonese	zyut3
U+88F0.0	fMandarin	duo2; duo1; xi1
U+88F0.0	fEnglish	 darn; mend (clothes); patch (clothes)
U+88F1.0	fUTF8	裱
U+88F1.0	fCNS11643	2-4D7E
U+88F1.0	fBig5	E3F7
U+88F1.0	fGB	0-7151
U+88F1.0	fR/S	 145.8
U+88F1.0	fTotalStrokes	 13; 14
U+88F1.0	fCangjie	LQMV
U+88F1.0	fFourCorner	35332
U+88F1.0	fHakka	 biau3; biau5; beu3
U+88F1.0	fCantonese	biu2
U+88F1.0	fMandarin	biao3; biao4
U+88F1.0	fEnglish	[1] mount (maps, paintings, calligraphy, etc.) [2] scarf
U+88F2.0	fUTF8	裲
U+88F2.0	fCNS11643	2-4E21
U+88F2.0	fBig5	E3F8
U+88F2.0	fGB	3-6665
U+88F2.0	fR/S	 145.8
U+88F2.0	fTotalStrokes	 13; 14
U+88F2.0	fCangjie	LMLB
U+88F2.0	fFourCorner	31227
U+88F2.0	fCantonese	loeng5
U+88F2.0	fMandarin	liang3
U+88F2.0	fEnglish	waistcoat
U+88F3.0	fUTF8	裳
U+88F3.0	fCNS11643	1-6A76
U+88F3.0	fBig5	BB6E
U+88F3.0	fGB	0-4951
U+88F3.0	fR/S	 145.8
U+88F3.0	fTotalStrokes	 14
U+88F3.0	fCangjie	FBRYV
U+88F3.0	fFourCorner	90732
U+88F3.0	fHakka	  song2;  shong2; song1; shong3; song3
U+88F3.0	fCantonese	soeng4
U+88F3.0	fMandarin	chang2; shang5; shang2; shang1
U+88F3.0	fEnglish	[1] clothes; dress; clothing; garment [2] skirt (worn in ancient China); the lower garments
U+88F4.0	fUTF8	裴
U+88F4.0	fCNS11643	1-6A78
U+88F4.0	fBig5	BB70
U+88F4.0	fGB	0-4561
U+88F4.0	fR/S	 145.8
U+88F4.0	fTotalStrokes	 14
U+88F4.0	fCangjie	LYYHV
U+88F4.0	fFourCorner	11732
U+88F4.0	fHakka	 pui2;  fui1; pui5; pi2; fi1
U+88F4.0	fCantonese	pui4
U+88F4.0	fMandarin	pei2; fei2
U+88F4.0	fEnglish	[1] the look of a flowing gown [2] a Chinese family name
U+88F5.0	fUTF8	裵
U+88F5.0	fCNS11643	3-4954
U+88F5.0	fGB	E-3D33
U+88F5.0	fR/S	 145.8
U+88F5.0	fTotalStrokes	 14
U+88F5.0	fCangjie	YLYV
U+88F5.0	fFourCorner	00730
U+88F5.0	fCantonese	pui4
U+88F5.0	fMandarin	pei2
U+88F5.0	fEnglish	 a Chinese family name look of a flowing gown
U+88F6.0	fUTF8	裶
U+88F6.0	fCNS11643	2-4E26
U+88F6.0	fBig5	E3FD
U+88F6.0	fGB	5-6562
U+88F6.0	fR/S	 145.8
U+88F6.0	fTotalStrokes	 13; 14
U+88F6.0	fCangjie	LLMY
U+88F6.0	fFourCorner	31211
U+88F6.0	fHakka	 fui1; fi1
U+88F6.0	fCantonese	fei1
U+88F6.0	fMandarin	fei1
U+88F6.0	fEnglish	 train of a dress
U+88F7.0	fUTF8	裷
U+88F7.0	fCNS11643	2-4D7C
U+88F7.0	fBig5	E3F5
U+88F7.0	fGB	3-6674
U+88F7.0	fR/S	 145.8
U+88F7.0	fTotalStrokes	 13; 14
U+88F7.0	fCangjie	LFQU
U+88F7.0	fFourCorner	39212
U+88F7.0	fCantonese	gwan2 (also); jyun1
U+88F7.0	fMandarin	yuan1; gun3; juan3; quan3
U+88F8.0	fUTF8	裸
U+88F8.0	fCNS11643	1-6A7A
U+88F8.0	fBig5	BB72
U+88F8.0	fGB	0-4263
U+88F8.0	fR/S	 145.8
U+88F8.0	fTotalStrokes	 13; 14
U+88F8.0	fCangjie	LWD
U+88F8.0	fFourCorner	36294
U+88F8.0	fHakka	 lo3; ko5; la3; ko6; ko3
U+88F8.0	fCantonese	lo2; lo5
U+88F8.0	fMandarin	luo3
U+88F8.0	fEnglish	bare; nude; undressed; naked
U+88F9.0	fUTF8	裹
U+88F9.0	fCNS11643	1-6A79
U+88F9.0	fBig5	BB71
U+88F9.0	fGB	0-397C
U+88F9.0	fR/S	 145.8
U+88F9.0	fTotalStrokes	 14
U+88F9.0	fCangjie	YWDV
U+88F9.0	fFourCorner	00732
U+88F9.0	fHakka	 go3; gwo3; kwo3
U+88F9.0	fCantonese	gwo2; go2 (also)
U+88F9.0	fMandarin	guo3; guo4
U+88F9.0	fEnglish	[1] wrap; bind; tie up [2] encircle; confine; encompass; surround [3] things wrapped, as a parcel [4] close in and force obedience
U+88FA.0	fUTF8	裺
U+88FA.0	fCNS11643	2-4E22
U+88FA.0	fBig5	E3F9
U+88FA.0	fGB	3-666E
U+88FA.0	fR/S	 145.8
U+88FA.0	fTotalStrokes	 13; 14
U+88FA.0	fCangjie	LKLU
U+88FA.0	fFourCorner	34216
U+88FA.0	fHakka	 jam3; ram3
U+88FA.0	fCantonese	jim2
U+88FA.0	fMandarin	yan3; an1; yan4
U+88FA.0	fEnglish	the selvedge of cloth
U+88FB.0	fUTF8	裻
U+88FB.0	fCNS11643	2-4E27
U+88FB.0	fBig5	E3FE
U+88FB.0	fGB	3-6625
U+88FB.0	fR/S	 145.8
U+88FB.0	fTotalStrokes	 14
U+88FB.0	fCangjie	YEYHV
U+88FB.0	fFourCorner	27732
U+88FB.0	fHakka	  duk7
U+88FB.0	fCantonese	duk1
U+88FB.0	fMandarin	du3; du1
U+88FB.0	fEnglish	the seam at the back of a garment
U+88FC.0	fUTF8	裼
U+88FC.0	fCNS11643	2-4E25
U+88FC.0	fBig5	E3FC
U+88FC.0	fGB	0-7153
U+88FC.0	fR/S	 145.8
U+88FC.0	fTotalStrokes	 13; 14
U+88FC.0	fCangjie	LAPH
U+88FC.0	fFourCorner	36227
U+88FC.0	fHakka	  sit7;  tit7
U+88FC.0	fCantonese	sik3 (also); tik1; sik1; tai3 (also)
U+88FC.0	fMandarin	ti4; xi1; xi2
U+88FC.0	fEnglish	[1] take off one's top garment [2] a wrapper or outer garments worn over a fur (in ancient China)
U+88FD.0	fUTF8	製
U+88FD.0	fCNS11643	1-6A7B
U+88FD.0	fBig5	BB73
U+88FD.0	fGB	1-7925
U+88FD.0	fR/S	 145.8
U+88FD.0	fTotalStrokes	 14
U+88FD.0	fCangjie	HNYHV
U+88FD.0	fFourCorner	22732
U+88FD.0	fHakka	 zhi5; zih5; ze5; zi5; tje5
U+88FD.0	fCantonese	zai3
U+88FD.0	fMandarin	zhi4
U+88FD.0	fEnglish	[1] make; manufacture; produce; create [2] compose (writings, literature, etc.); literary works [3] cut out garments and make them [4] form; model; fashion; pattern
U+88FE.0	fUTF8	裾
U+88FE.0	fCNS11643	2-4E23
U+88FE.0	fBig5	E3FA
U+88FE.0	fGB	0-7155
U+88FE.0	fR/S	 145.8
U+88FE.0	fTotalStrokes	 13; 14
U+88FE.0	fCangjie	LSJR
U+88FE.0	fFourCorner	37264
U+88FE.0	fHakka	  gi1
U+88FE.0	fCantonese	geoi1
U+88FE.0	fMandarin	ju1; ju4; qu2
U+88FE.0	fEnglish	overlap of a robe; hinder part of a garment; lapel of a coat
U+88FF.0	fUTF8	裿
U+88FF.0	fCNS11643	3-4951
U+88FF.0	fGB	3-666D
U+88FF.0	fR/S	  145.8
U+88FF.0	fTotalStrokes	 13; 14
U+88FF.0	fCangjie	LKMR
U+88FF.0	fFourCorner	34220
U+88FF.0	fCantonese	ji2
U+88FF.0	fMandarin	qi3
U+8900.0	fUTF8	褀
U+8900.0	fCNS11643	3-4956
U+8900.0	fGB	5-655D
U+8900.0	fR/S	  145.8
U+8900.0	fTotalStrokes	 13; 14
U+8900.0	fCangjie	LTMC
U+8900.0	fFourCorner	34280
U+8900.0	fCantonese	gei6
U+8900.0	fMandarin	ji4
U+8901.0	fUTF8	褁
U+8901.0	fCNS11643	2-4034
U+8901.0	fBig5	DBCE
U+8901.0	fGB	E-3D34
U+8901.0	fR/S	 145.8
U+8901.0	fTotalStrokes	 14
U+8901.0	fCangjie	WDHV
U+8901.0	fFourCorner	60732
U+8901.0	fCantonese	gwo2; go2 (also)
U+8901.0	fMandarin	zhi2
U+8902.0	fUTF8	褂
U+8902.0	fCNS11643	1-6A77
U+8902.0	fBig5	BB6F
U+8902.0	fGB	0-3953
U+8902.0	fR/S	 145.8
U+8902.0	fTotalStrokes	 13; 14
U+8902.0	fCangjie	LGGY
U+8902.0	fFourCorner	33200
U+8902.0	fHakka	  gwa5; kua5; ga5
U+8902.0	fCantonese	gwaa3; kwaa2 (also); gwaa2 (also)
U+8902.0	fMandarin	gua4
U+8902.0	fEnglish	 jacket; overcoat; coat; gown; robe
U+8903.0	fUTF8	褃
U+8903.0	fCNS11643	3-4958
U+8903.0	fGB	5-6563
U+8903.0	fR/S	  145.8
U+8903.0	fTotalStrokes	 13; 14
U+8903.0	fCangjie	LYMB
U+8903.0	fFourCorner	31220
U+8903.0	fCantonese	kang3
U+8903.0	fMandarin	ken4
U+8903.0	fEnglish	a seam in a garment
U+8904.0	fUTF8	褄
U+8904.0	fGB	E-3D35
U+8904.0	fR/S	 145.8
U+8904.0	fTotalStrokes	 13; 14
U+8904.0	fCangjie	LJLV
U+8904.0	fCantonese	cai1
U+8904.0	fEnglish	a skirt
U+8905.0	fUTF8	褅
U+8905.0	fCNS11643	2-542B
U+8905.0	fBig5	E7C2
U+8905.0	fGB	5-6578
U+8905.0	fR/S	 145.9
U+8905.0	fTotalStrokes	 14; 15
U+8905.0	fCangjie	LYBB
U+8905.0	fFourCorner	30117
U+8905.0	fHakka	 di5
U+8905.0	fCantonese	tai3
U+8905.0	fMandarin	ti4
U+8906.0	fUTF8	褆
U+8906.0	fCNS11643	2-5432
U+8906.0	fBig5	E7C9
U+8906.0	fGB	3-6722
U+8906.0	fR/S	 145.9
U+8906.0	fTotalStrokes	 14; 15
U+8906.0	fCangjie	LAMO
U+8906.0	fFourCorner	36281
U+8906.0	fHakka	 ti2; shi6; sii5; shi5
U+8906.0	fCantonese	tai4; si6
U+8906.0	fMandarin	ti2; shi4; zhi4
U+8907.0	fUTF8	複
U+8907.0	fCNS11643	1-6E6F
U+8907.0	fBig5	BDC6
U+8907.0	fGB	1-7834
U+8907.0	fR/S	 145.9
U+8907.0	fTotalStrokes	 14; 15
U+8907.0	fCangjie	LOAE
U+8907.0	fFourCorner	38247
U+8907.0	fHakka	  fuk8; fuk7; buk7
U+8907.0	fCantonese	fuk1
U+8907.0	fMandarin	fu4
U+8907.0	fEnglish	[1] repeat; double; overlapping [2] complex (concepts, etc.); compound (interest, etc.) [3] repeat; reiterate [4] a lined garment, dress, etc.
U+8908.0	fUTF8	褈
U+8908.0	fCNS11643	4-4D46
U+8908.0	fGB	3-6726
U+8908.0	fR/S	 145.9
U+8908.0	fTotalStrokes	 14; 15
U+8908.0	fCangjie	LHJG
U+8908.0	fFourCorner	32210
U+8908.0	fCantonese	cung4
U+8908.0	fMandarin	chong2; chong1; zhong4
U+8909.0	fUTF8	褉
U+8909.0	fCNS11643	2-5436
U+8909.0	fBig5	E7CD
U+8909.0	fGB	5-656D
U+8909.0	fR/S	 145.9
U+8909.0	fTotalStrokes	 14; 15
U+8909.0	fCangjie	LQHK
U+8909.0	fFourCorner	37234
U+8909.0	fHakka	 giet7; get7; ke5
U+8909.0	fCantonese	kit3; hit3
U+8909.0	fMandarin	xie1; xi4; xie5
U+8909.0	fEnglish	short garments
U+890A.0	fUTF8	褊
U+890A.0	fCNS11643	1-6E73
U+890A.0	fBig5	BDCA
U+890A.0	fGB	0-715B
U+890A.0	fR/S	 145.9
U+890A.0	fTotalStrokes	 14; 15
U+890A.0	fCangjie	LHSB
U+890A.0	fFourCorner	32227
U+890A.0	fHakka	  bien3; pien1; ben3
U+890A.0	fCantonese	bin2; pin5; pin4
U+890A.0	fMandarin	bian3; pian2; pian1
U+890A.0	fEnglish	[1] cramped; narrow; small; petty [2] small size clothes [3] narrow minded
U+890B.0	fUTF8	褋
U+890B.0	fCNS11643	2-542E
U+890B.0	fBig5	E7C5
U+890B.0	fGB	3-667B
U+890B.0	fR/S	 145.9
U+890B.0	fTotalStrokes	 14; 15
U+890B.0	fCangjie	LPTD
U+890B.0	fFourCorner	34294
U+890B.0	fCantonese	dip6
U+890B.0	fMandarin	die2
U+890C.0	fUTF8	褌
U+890C.0	fCNS11643	2-542C
U+890C.0	fBig5	E7C3
U+890C.0	fGB	3-6660
U+890C.0	fR/S	 145.9
U+890C.0	fTotalStrokes	 14; 15
U+890C.0	fCangjie	LBJJ
U+890C.0	fFourCorner	37256
U+890C.0	fHakka	 kun2; kwun1
U+890C.0	fCantonese	gwan1
U+890C.0	fMandarin	kun1; hui1
U+890C.0	fEnglish	 trousers; pants; shorts; drawers; short pants
U+890D.0	fUTF8	褍
U+890D.0	fCNS11643	4-4D3E
U+890D.0	fGB	3-6724
U+890D.0	fR/S	  145.9
U+890D.0	fTotalStrokes	 14; 15
U+890D.0	fCangjie	LUMB
U+890D.0	fFourCorner	32220
U+890D.0	fCantonese	dyun1
U+890D.0	fMandarin	duan1
U+890E.0	fUTF8	褎
U+890E.0	fCNS11643	2-5435
U+890E.0	fBig5	E7CC
U+890E.0	fGB	3-3258
U+890E.0	fR/S	 145.9
U+890E.0	fTotalStrokes	 15
U+890E.0	fCangjie	YHDV
U+890E.0	fFourCorner	00732
U+890E.0	fHakka	 ciu5; ju5; ciu6; riu6
U+890E.0	fCantonese	zau6; jau6
U+890E.0	fMandarin	xiu4; you4
U+890E.0	fEnglish	 sleeve ample flowing robes
U+890F.0	fUTF8	褏
U+890F.0	fCNS11643	3-4E61
U+890F.0	fGB	5-3359
U+890F.0	fR/S	  145.9
U+890F.0	fTotalStrokes	 15
U+890F.0	fCangjie	YHWV
U+890F.0	fFourCorner	00730
U+890F.0	fCantonese	zau6; jau6
U+890F.0	fMandarin	xiu4; you4
U+890F.0	fEnglish	 sleeve ample flowing robes
U+8910.0	fUTF8	褐
U+8910.0	fCNS11643	1-6E6E
U+8910.0	fBig5	BDC5
U+8910.0	fGB	0-3A56
U+8910.0	fR/S	 145.9; 145.8
U+8910.0	fTotalStrokes	 14; 15
U+8910.0	fCangjie	LAPV
U+8910.0	fFourCorner	36227
U+8910.0	fHakka	  hot7; hot8
U+8910.0	fCantonese	hot3; hot6
U+8910.0	fMandarin	he4; he2; ge2; xie2
U+8910.0	fEnglish	[1] coarse woolen cloth; coarse cloth; haircloth; rough cloth [2] the poor or destitute [3] brown
U+8911.0	fUTF8	褑
U+8911.0	fCNS11643	2-5434
U+8911.0	fBig5	E7CB
U+8911.0	fGB	3-672A
U+8911.0	fR/S	 145.9
U+8911.0	fTotalStrokes	 14; 15
U+8911.0	fCangjie	LBME
U+8911.0	fFourCorner	32247
U+8911.0	fHakka	 ren1; jen1; ren3; jen3; jan5; jen5
U+8911.0	fCantonese	jyun6; jyun4
U+8911.0	fMandarin	yuan4; yung4
U+8911.0	fEnglish	a girdle to which ornaments are attached
U+8912.0	fUTF8	褒
U+8912.0	fCNS11643	1-6E70
U+8912.0	fBig5	BDC7
U+8912.0	fGB	0-307D
U+8912.0	fR/S	 145.9
U+8912.0	fTotalStrokes	 15
U+8912.0	fCangjie	YODV
U+8912.0	fFourCorner	00732
U+8912.0	fHakka	  bau1;  bo1
U+8912.0	fCantonese	bou1
U+8912.0	fMandarin	bao1; pou2
U+8912.0	fEnglish	[1] praise; cite [2] big; great
U+8913.0	fUTF8	褓
U+8913.0	fCNS11643	1-6E71
U+8913.0	fBig5	BDC8
U+8913.0	fGB	0-7159
U+8913.0	fR/S	 145.9
U+8913.0	fTotalStrokes	 14; 15
U+8913.0	fCangjie	LORD
U+8913.0	fFourCorner	36294
U+8913.0	fHakka	 bau3; bo3
U+8913.0	fCantonese	bou2
U+8913.0	fMandarin	bao3
U+8913.0	fEnglish	[1] swaddling cloth; swaddling clothes [2] used in combination for infancy
U+8914.0	fUTF8	褔
U+8914.0	fCNS11643	2-542D
U+8914.0	fBig5	E7C4
U+8914.0	fGB	3-667C
U+8914.0	fR/S	 145.9
U+8914.0	fTotalStrokes	 14; 15
U+8914.0	fCangjie	LMRW
U+8914.0	fFourCorner	31266
U+8914.0	fCantonese	fu3
U+8914.0	fMandarin	fu4
U+8915.0	fUTF8	褕
U+8915.0	fCNS11643	1-6E72
U+8915.0	fBig5	BDC9
U+8915.0	fGB	3-6729
U+8915.0	fR/S	 145.9
U+8915.0	fTotalStrokes	 14; 15
U+8915.0	fCangjie	LOMN
U+8915.0	fFourCorner	38221
U+8915.0	fHakka	 ji2; ri2
U+8915.0	fCantonese	jyu4
U+8915.0	fMandarin	yu2; tou2; yao2
U+8915.0	fEnglish	[1] beautiful; pretty (dresses, etc,) [2] loose garment or cloak
U+8916.0	fUTF8	褖
U+8916.0	fCNS11643	2-5433
U+8916.0	fBig5	E7CA
U+8916.0	fGB	3-672D
U+8916.0	fR/S	 145.9
U+8916.0	fTotalStrokes	 14; 15
U+8916.0	fCangjie	LVNO
U+8916.0	fFourCorner	37232
U+8916.0	fHakka	 chon5; con5
U+8916.0	fCantonese	teon3; teon5
U+8916.0	fMandarin	tuan4
U+8916.0	fEnglish	black robes formerly worn by the Empress
U+8917.0	fUTF8	褗
U+8917.0	fCNS11643	2-542F
U+8917.0	fBig5	E7C6
U+8917.0	fGB	5-6570
U+8917.0	fR/S	 145.9
U+8917.0	fTotalStrokes	 14; 15
U+8917.0	fCangjie	LSAV
U+8917.0	fFourCorner	31214
U+8917.0	fHakka	 jen3; ren3; rau1; jeu1
U+8917.0	fCantonese	jin2; jin6
U+8917.0	fMandarin	yan3; yan4
U+8917.0	fEnglish	a collar or band on the neck of a robe
U+8918.0	fUTF8	褘
U+8918.0	fCNS11643	2-5430
U+8918.0	fBig5	E7C7
U+8918.0	fGB	3-662B
U+8918.0	fR/S	 145.9; 145.10
U+8918.0	fTotalStrokes	 14; 15
U+8918.0	fCangjie	LDMQ
U+8918.0	fFourCorner	35256
U+8918.0	fHakka	 fi1; fui1; wui2; wi2; vui3
U+8918.0	fCantonese	fai1; wai4; ji1
U+8918.0	fMandarin	hui1; wei2; yi1
U+8918.0	fEnglish	ceremonial gowns of a queen
U+8919.0	fUTF8	褙
U+8919.0	fCNS11643	2-5431
U+8919.0	fBig5	E7C8
U+8919.0	fGB	0-7158
U+8919.0	fR/S	 145.9
U+8919.0	fTotalStrokes	 14; 15
U+8919.0	fCangjie	LLPB
U+8919.0	fFourCorner	31227
U+8919.0	fHakka	 boi5; poe5
U+8919.0	fCantonese	bui3
U+8919.0	fMandarin	bei4
U+8919.0	fEnglish	 mount (paintings, calligraphic works)
U+891A.0	fUTF8	褚
U+891A.0	fCNS11643	1-6A7D
U+891A.0	fBig5	BB75
U+891A.0	fGB	0-7152
U+891A.0	fR/S	 145.9
U+891A.0	fTotalStrokes	 14; 15
U+891A.0	fCangjie	LJKA
U+891A.0	fFourCorner	34264
U+891A.0	fHakka	 du5; za3; du3; zha3; tu3; chu3
U+891A.0	fCantonese	cyu2 (also); cyu5; ze2; zyu2
U+891A.0	fMandarin	chu3; zhu3; zhe3
U+891A.0	fEnglish	[1] bag; valise [2] stuff a lined garment with cotton [3] reserve; save
U+891B.0	fUTF8	褛
U+891B.0	fCNS11643	3-4E62
U+891B.0	fGB	0-715A
U+891B.0	fR/S	 145.9
U+891B.0	fTotalStrokes	 14; 15
U+891B.0	fCangjie	LFDV
U+891B.0	fFourCorner	39240
U+891B.0	fHakka	 leu3; li3
U+891B.0	fCantonese	leoi5; lau5 (also); lau1
U+891B.0	fMandarin	luu3; l3; lou2; lu3
U+891B.0	fEnglish	simplified form of &#x8938;
U+891C.0	fUTF8	褜
U+891C.0	fGB	E-3D36
U+891C.0	fR/S	 145.9
U+891C.0	fTotalStrokes	 15
U+891C.0	fCangjie	BUYHV
U+891C.0	fCantonese	baau1
U+891D.0	fUTF8	褝
U+891D.0	fGB	E-3D37
U+891D.0	fR/S	 145.9
U+891D.0	fTotalStrokes	 14; 15
U+891D.0	fCangjie	LFWJ
U+891D.0	fCantonese	daan1
U+891D.0	fEnglish	 unlined garment
U+891E.0	fUTF8	褞
U+891E.0	fCNS11643	2-5A3E
U+891E.0	fBig5	EB70
U+891E.0	fGB	5-6573
U+891E.0	fR/S	 145.10
U+891E.0	fTotalStrokes	 15; 16
U+891E.0	fCangjie	LWOT
U+891E.0	fFourCorner	36217
U+891E.0	fCantonese	wan2; wan1
U+891E.0	fMandarin	yun3; wen1
U+891E.0	fEnglish	a robe
U+891F.0	fUTF8	褟
U+891F.0	fCNS11643	2-5A4A
U+891F.0	fBig5	EB7C
U+891F.0	fGB	3-6731
U+891F.0	fR/S	 145.10
U+891F.0	fTotalStrokes	 15; 16
U+891F.0	fCangjie	LASM
U+891F.0	fFourCorner	36227
U+891F.0	fCantonese	taap3; daap3; zip3; tap3
U+891F.0	fMandarin	da2; ta1
U+891F.0	fEnglish	[1] the lace-trimmed hem of a dress; lace of a dress [2] singlet; a thin T-shirt as underwear
U+8920.0	fUTF8	褠
U+8920.0	fCNS11643	4-5353
U+8920.0	fGB	3-672E
U+8920.0	fR/S	  145.10
U+8920.0	fTotalStrokes	 15; 16
U+8920.0	fCangjie	LTTB
U+8920.0	fFourCorner	35240
U+8920.0	fHakka	 kieu1; geu1
U+8920.0	fCantonese	kau1
U+8920.0	fMandarin	gou1
U+8921.0	fUTF8	褡
U+8921.0	fCNS11643	1-7234
U+8921.0	fBig5	BFCA
U+8921.0	fGB	0-7157
U+8921.0	fR/S	 145.10
U+8921.0	fTotalStrokes	 15; 16
U+8921.0	fCangjie	LTOR
U+8921.0	fFourCorner	34261
U+8921.0	fHakka	 dap7
U+8921.0	fCantonese	daap3
U+8921.0	fMandarin	da1
U+8921.0	fEnglish	[1] a kerchief hung at the waist [2] purse; tiny sack [3] worn clothes
U+8922.0	fUTF8	褢
U+8922.0	fCNS11643	2-5A45
U+8922.0	fBig5	EB77
U+8922.0	fGB	5-335A
U+8922.0	fR/S	 145.10
U+8922.0	fTotalStrokes	 16
U+8922.0	fCangjie	YHIV
U+8922.0	fFourCorner	00732
U+8922.0	fHakka	 fai2
U+8922.0	fCantonese	waai4
U+8922.0	fMandarin	huai2
U+8922.0	fEnglish	 carry in the bosom or the sleeve wrap; conceal
U+8923.0	fUTF8	褣
U+8923.0	fCNS11643	2-5A47
U+8923.0	fBig5	EB79
U+8923.0	fGB	5-6625
U+8923.0	fR/S	 145.10
U+8923.0	fTotalStrokes	 15; 16
U+8923.0	fCangjie	LJCR
U+8923.0	fFourCorner	33268
U+8923.0	fCantonese	jung4
U+8923.0	fMandarin	rong2
U+8924.0	fUTF8	褤
U+8924.0	fCNS11643	3-5265
U+8924.0	fGB	5-657A
U+8924.0	fR/S	  145.10
U+8924.0	fTotalStrokes	 15; 16
U+8924.0	fCangjie	LGRV
U+8924.0	fFourCorner	34230
U+8924.0	fCantonese	jyun6
U+8924.0	fMandarin	yuan4
U+8925.0	fUTF8	褥
U+8925.0	fCNS11643	1-7232
U+8925.0	fBig5	BFC8
U+8925.0	fGB	0-486C
U+8925.0	fR/S	 145.10
U+8925.0	fTotalStrokes	 15; 16
U+8925.0	fCangjie	LMVI
U+8925.0	fFourCorner	31243
U+8925.0	fHakka	  juk8; ruk8; niuk8
U+8925.0	fCantonese	juk6; juk2 (also)
U+8925.0	fMandarin	ru4; nu4
U+8925.0	fEnglish	 mattress; cushion; bedding; quilts or coverlets; bedclothes
U+8926.0	fUTF8	褦
U+8926.0	fCNS11643	2-5A3F
U+8926.0	fBig5	EB71
U+8926.0	fGB	3-6734
U+8926.0	fR/S	 145.10
U+8926.0	fTotalStrokes	 15; 16
U+8926.0	fCangjie	LIBP
U+8926.0	fFourCorner	31211
U+8926.0	fHakka	 nai5; nai6
U+8926.0	fCantonese	naai6; le5 (also); laai6 (also); nang3; lang3 (also); noi6; nang2
U+8926.0	fMandarin	nai4
U+8926.0	fEnglish	[1] stupid; ignorant; naive [2] a palm-leaf had worn in summer for shading the sun or rain
U+8926.1	fUTF8	褦
U+8926.1	fCNS11643	2-5A3F
U+8926.1	fBig5	EB71
U+8926.1	fGB	3-6734
U+8926.1	fR/S	 145.10
U+8926.1	fTotalStrokes	 15; 16
U+8926.1	fCangjie	LIBP
U+8926.1	fFourCorner	31211
U+8926.1	fMandarin	le1
U+8926.1	fEnglish	untidily dressed
U+8927.0	fUTF8	褧
U+8927.0	fCNS11643	2-5A43
U+8927.0	fBig5	EB75
U+8927.0	fGB	3-6626
U+8927.0	fR/S	 145.10
U+8927.0	fTotalStrokes	 16
U+8927.0	fCangjie	SFYHV
U+8927.0	fFourCorner	19732
U+8927.0	fHakka	 kwen3
U+8927.0	fCantonese	gwing2
U+8927.0	fMandarin	jiong3
U+8927.0	fEnglish	 unlined clothes light overcoat
U+8928.0	fUTF8	褨
U+8928.0	fCNS11643	4-5350
U+8928.0	fGB	3-672B
U+8928.0	fR/S	  145.10
U+8928.0	fTotalStrokes	 15; 16
U+8928.0	fCangjie	LTQM
U+8928.0	fFourCorner	38210
U+8928.0	fCantonese	so2
U+8928.0	fMandarin	suo3
U+8929.0	fUTF8	褩
U+8929.0	fCNS11643	2-5A46
U+8929.0	fBig5	EB78
U+8929.0	fGB	E-3D38
U+8929.0	fR/S	 145.10
U+8929.0	fTotalStrokes	 16
U+8929.0	fCangjie	HEYHV
U+8929.0	fFourCorner	27732
U+8929.0	fCantonese	bun1
U+8929.0	fMandarin	ban1
U+892A.0	fUTF8	褪
U+892A.0	fCNS11643	1-7230
U+892A.0	fBig5	BFC6
U+892A.0	fGB	0-4D4A
U+892A.0	fR/S	 145.10
U+892A.0	fTotalStrokes	 15; 16
U+892A.0	fCangjie	LYAV
U+892A.0	fFourCorner	37233
U+892A.0	fHakka	  tui5;  tun5; tun2; dun5
U+892A.0	fCantonese	tan3; teoi3
U+892A.0	fMandarin	tui4; tun4
U+892A.0	fEnglish	[1] strip; undress; take off one's clothing; slip out of something [2] fall off; fade (color) [3] retreat; move backwards
U+892B.0	fUTF8	褫
U+892B.0	fCNS11643	1-7233
U+892B.0	fBig5	BFC9
U+892B.0	fGB	0-715D
U+892B.0	fR/S	 145.10
U+892B.0	fTotalStrokes	 15; 16
U+892B.0	fCangjie	LHYU
U+892B.0	fFourCorner	32217
U+892B.0	fHakka	 ti5;  chi3; cih3; ci3; chii3
U+892B.0	fCantonese	ci2
U+892B.0	fMandarin	chi3; chi2; yi2; yi3; zhi4
U+892B.0	fEnglish	[1] strip off; tear off; deprive of [2] undress forcibly
U+892C.0	fUTF8	褬
U+892C.0	fCNS11643	2-5A49
U+892C.0	fBig5	EB7B
U+892C.0	fGB	E-3D39
U+892C.0	fR/S	 145.10
U+892C.0	fTotalStrokes	 15; 16
U+892C.0	fCangjie	LEED
U+892C.0	fFourCorner	37294
U+892C.0	fCantonese	song2
U+892C.0	fMandarin	sang3
U+892D.0	fUTF8	褭
U+892D.0	fCNS11643	2-5A41
U+892D.0	fBig5	EB73
U+892D.0	fGB	E-3D3A
U+892D.0	fR/S	 145.10
U+892D.0	fTotalStrokes	 16
U+892D.0	fCangjie	YSFV
U+892D.0	fFourCorner	00732
U+892D.0	fHakka	 ngiau3; ngieu3
U+892D.0	fCantonese	niu5; liu5 (also)
U+892D.0	fMandarin	niao3; rao3
U+892D.0	fEnglish	 tie up a horse with a halter
U+892E.0	fUTF8	褮
U+892E.0	fCNS11643	2-5A42
U+892E.0	fBig5	EB74
U+892E.0	fGB	3-6624
U+892E.0	fR/S	 145.10
U+892E.0	fTotalStrokes	 16
U+892E.0	fCangjie	FFBYV
U+892E.0	fFourCorner	99732
U+892E.0	fCantonese	ang1; ngang1 (also); jing3
U+892E.0	fMandarin	ying1; ying2; ying4
U+892F.0	fUTF8	褯
U+892F.0	fCNS11643	2-5A48
U+892F.0	fBig5	EB7A
U+892F.0	fGB	3-6733
U+892F.0	fR/S	 145.10
U+892F.0	fTotalStrokes	 15; 16
U+892F.0	fCangjie	LITB
U+892F.0	fFourCorner	30227
U+892F.0	fHakka	 sit7;  zia5; ciak8
U+892F.0	fCantonese	zik6
U+892F.0	fMandarin	jie4; xi2
U+892F.0	fEnglish	children's garments. a mat for wrapping garments
U+8930.0	fUTF8	褰
U+8930.0	fCNS11643	2-5A40
U+8930.0	fBig5	EB72
U+8930.0	fGB	0-653D
U+8930.0	fR/S	 145.10
U+8930.0	fTotalStrokes	 16
U+8930.0	fCangjie	JTCV
U+8930.0	fFourCorner	30732
U+8930.0	fHakka	 kien1; ken1; kian1
U+8930.0	fCantonese	hin1
U+8930.0	fMandarin	qian1; jian3
U+8930.0	fEnglish	[1] lift; raise (one's dress, skirt, etc.) [2] trousers; paints; drawers [3] worn clothes
U+8931.0	fUTF8	褱
U+8931.0	fCNS11643	2-5A44
U+8931.0	fBig5	EB76
U+8931.0	fGB	5-335D
U+8931.0	fR/S	 145.10
U+8931.0	fTotalStrokes	 16
U+8931.0	fCangjie	YWLV
U+8931.0	fFourCorner	00732
U+8931.0	fHakka	  fai2; fai3
U+8931.0	fCantonese	waai4
U+8931.0	fMandarin	huai2
U+8931.0	fEnglish	 carry in the bosom or the sleeve wrap; conceal
U+8932.0	fUTF8	褲
U+8932.0	fCNS11643	1-7231
U+8932.0	fBig5	BFC7
U+8932.0	fGB	1-3F63
U+8932.0	fR/S	 145.10
U+8932.0	fTotalStrokes	 15; 16
U+8932.0	fCangjie	LIJJ
U+8932.0	fFourCorner	34227
U+8932.0	fHakka	  fu5;  ku5
U+8932.0	fCantonese	fu3
U+8932.0	fMandarin	ku4
U+8932.0	fEnglish	trousers; pants; drawers
U+8933.0	fUTF8	褳
U+8933.0	fCNS11643	2-5F40
U+8933.0	fBig5	EE72
U+8933.0	fGB	1-714D
U+8933.0	fR/S	 145.11
U+8933.0	fTotalStrokes	 16; 17
U+8933.0	fCangjie	LYJJ
U+8933.0	fFourCorner	35230
U+8933.0	fHakka	  lien2
U+8933.0	fCantonese	lin4
U+8933.0	fMandarin	lian2
U+8933.0	fEnglish	used in combination for folding purse inserted in belt
U+8934.0	fUTF8	褴
U+8934.0	fGB	0-715C
U+8934.0	fR/S	 145.10
U+8934.0	fTotalStrokes	 15; 16
U+8934.0	fCangjie	LLIT
U+8934.0	fHakka	 lam2
U+8934.0	fCantonese	laam4
U+8934.0	fMandarin	lan2; bao3
U+8934.0	fEnglish	simplified form of &#x8964;
U+8935.0	fUTF8	褵
U+8935.0	fCNS11643	2-5F3F
U+8935.0	fBig5	EE71
U+8935.0	fGB	5-6622
U+8935.0	fR/S	 145.11
U+8935.0	fTotalStrokes	 16; 17
U+8935.0	fCangjie	LYUB
U+8935.0	fFourCorner	30227
U+8935.0	fHakka	  li2
U+8935.0	fCantonese	lei4
U+8935.0	fMandarin	li2
U+8935.0	fEnglish	 bridal veil
U+8936.0	fUTF8	褶
U+8936.0	fCNS11643	1-7542
U+8936.0	fBig5	C1B7
U+8936.0	fGB	0-715E
U+8936.0	fR/S	 145.11
U+8936.0	fTotalStrokes	 16; 17
U+8936.0	fCangjie	LSMA
U+8936.0	fFourCorner	37262
U+8936.0	fHakka	 tiap8; sip8; zap7
U+8936.0	fCantonese	zip3; zaap3 (also); zaap6; dip6
U+8936.0	fMandarin	die2; xi2; zhe3; shi2
U+8936.0	fEnglish	a lined garment or dress
U+8936.1	fUTF8	褶
U+8936.1	fCNS11643	1-7542
U+8936.1	fBig5	C1B7
U+8936.1	fGB	0-715E
U+8936.1	fR/S	 145.11
U+8936.1	fTotalStrokes	 16; 17
U+8936.1	fCangjie	LSMA
U+8936.1	fFourCorner	37262
U+8936.1	fMandarin	zhe2
U+8936.1	fEnglish	pleated; fold
U+8937.0	fUTF8	褷
U+8937.0	fCNS11643	2-5F45
U+8937.0	fBig5	EE77
U+8937.0	fGB	3-673A
U+8937.0	fR/S	 145.11
U+8937.0	fTotalStrokes	 16; 17
U+8937.0	fCangjie	LHOO
U+8937.0	fFourCorner	31281
U+8937.0	fHakka	 se1; si1
U+8937.0	fCantonese	si1
U+8937.0	fMandarin	shi1; xi3
U+8938.0	fUTF8	褸
U+8938.0	fCNS11643	1-7544
U+8938.0	fBig5	C1B9
U+8938.0	fGB	1-715A
U+8938.0	fR/S	 145.11
U+8938.0	fTotalStrokes	 16; 17
U+8938.0	fCangjie	LLLV
U+8938.0	fFourCorner	35244
U+8938.0	fHakka	  leu3; leu2; lui3; lui5; liu1; liu3; li3
U+8938.0	fCantonese	leoi5; lau5 (also); lau1
U+8938.0	fMandarin	l3; luu3; lou2; lu3
U+8938.0	fEnglish	[1] lapel; collar (garment) [2] tattered; in rags (clothes)
U+8939.0	fUTF8	褹
U+8939.0	fCNS11643	3-5640
U+8939.0	fGB	5-6627
U+8939.0	fR/S	  145.11
U+8939.0	fTotalStrokes	 16; 17
U+8939.0	fCangjie	LGGI
U+8939.0	fFourCorner	35217
U+8939.0	fHakka	 ngi5; ngi6
U+8939.0	fCantonese	ngai6; ai6 (also)
U+8939.0	fMandarin	yi4
U+8939.0	fEnglish	the sleeves of a robe
U+893A.0	fUTF8	褺
U+893A.0	fCNS11643	4-5862
U+893A.0	fGB	5-653A
U+893A.0	fR/S	  145.11
U+893A.0	fTotalStrokes	 17; 16
U+893A.0	fCangjie	GIYHV
U+893A.0	fFourCorner	45732
U+893A.0	fCantonese	dip6
U+893A.0	fMandarin	die2; die1
U+893A.0	fEnglish	 dirty; ragged undress; treat with irreverence
U+893B.0	fUTF8	褻
U+893B.0	fCNS11643	1-7541
U+893B.0	fBig5	C1B6
U+893B.0	fGB	1-5974
U+893B.0	fR/S	 145.11
U+893B.0	fTotalStrokes	 17
U+893B.0	fCangjie	YGIV
U+893B.0	fFourCorner	00732
U+893B.0	fHakka	  siet7; set7; siet8
U+893B.0	fCantonese	sit3
U+893B.0	fMandarin	xie4; die2
U+893B.0	fEnglish	[1] underwear; clothes worn in one's bedroom or house [2] dirty; filthy [3] intimate (sometimes denoting a degree of indecency) [4] slight; look down upon [5] be familiar; acquainted with
U+893C.0	fUTF8	褼
U+893C.0	fCNS11643	2-5F41
U+893C.0	fBig5	EE73
U+893C.0	fGB	3-673E
U+893C.0	fR/S	 145.11
U+893C.0	fTotalStrokes	 16; 17
U+893C.0	fCangjie	LMWU
U+893C.0	fFourCorner	31212
U+893C.0	fHakka	  sien1;  cien1
U+893C.0	fCantonese	sin1
U+893C.0	fMandarin	xian1
U+893C.0	fEnglish	the appearance of clothes
U+893D.0	fUTF8	褽
U+893D.0	fCNS11643	1-7545
U+893D.0	fBig5	C1BA
U+893D.0	fGB	3-6627
U+893D.0	fR/S	 145.11
U+893D.0	fTotalStrokes	 17
U+893D.0	fCangjie	SIYHV
U+893D.0	fFourCorner	74732
U+893D.0	fCantonese	wai3
U+893D.0	fMandarin	wei4
U+893D.0	fEnglish	cushion
U+893E.0	fUTF8	褾
U+893E.0	fCNS11643	2-5F42
U+893E.0	fBig5	EE74
U+893E.0	fGB	3-6736
U+893E.0	fR/S	 145.11
U+893E.0	fTotalStrokes	 16; 17
U+893E.0	fCangjie	LMWF
U+893E.0	fFourCorner	31391
U+893E.0	fCantonese	biu2
U+893E.0	fMandarin	biao3; biao4
U+893F.0	fUTF8	褿
U+893F.0	fCNS11643	3-563F
U+893F.0	fGB	3-6735
U+893F.0	fR/S	  145.11
U+893F.0	fTotalStrokes	 16; 17
U+893F.0	fCangjie	LTWA
U+893F.0	fFourCorner	35260
U+893F.0	fHakka	 cau2; co1
U+893F.0	fCantonese	cou4
U+893F.0	fMandarin	cao2
U+893F.0	fEnglish	 short skirt; tunic
U+8940.0	fUTF8	襀
U+8940.0	fCNS11643	3-563D
U+8940.0	fGB	3-666C
U+8940.0	fR/S	  145.11
U+8940.0	fTotalStrokes	 16; 17
U+8940.0	fCangjie	LQMC
U+8940.0	fFourCorner	35280
U+8940.0	fCantonese	zik1
U+8940.0	fMandarin	ji1
U+8940.0	fEnglish	a pleat, fold, tuck, crease
U+8941.0	fUTF8	襁
U+8941.0	fCNS11643	2-5F43
U+8941.0	fBig5	EE75
U+8941.0	fGB	0-715F
U+8941.0	fR/S	 145.11
U+8941.0	fTotalStrokes	 16; 17
U+8941.0	fCangjie	LNII
U+8941.0	fFourCorner	36236
U+8941.0	fHakka	  giong3
U+8941.0	fCantonese	koeng5
U+8941.0	fMandarin	qiang3; jiang3
U+8941.0	fEnglish	swaddling clothes for an infant; broad bandage for carrying an infant on the back
U+8942.0	fUTF8	襂
U+8942.0	fCNS11643	2-5F46
U+8942.0	fBig5	EE78
U+8942.0	fGB	3-6679
U+8942.0	fR/S	 145.11
U+8942.0	fTotalStrokes	 16; 17
U+8942.0	fCangjie	LIIH
U+8942.0	fFourCorner	33222
U+8942.0	fCantonese	sam1; saam1
U+8942.0	fMandarin	sen1; shan1; shen1
U+8943.0	fUTF8	襃
U+8943.0	fCNS11643	3-5642
U+8943.0	fGB	E-3D3B
U+8943.0	fR/S	 145.11
U+8943.0	fTotalStrokes	 17
U+8943.0	fCangjie	YHDV
U+8943.0	fFourCorner	00730
U+8943.0	fHakka	  bau1; bo1; ju5; ciu5
U+8943.0	fCantonese	bou1
U+8943.0	fMandarin	bao1; pou2
U+8943.0	fEnglish	[1] praise; cite; commend; citation [2] big; great [3] name of an ancient state in today's Shensi province [4] a Chinese family name
U+8944.0	fUTF8	襄
U+8944.0	fCNS11643	1-7543
U+8944.0	fBig5	C1B8
U+8944.0	fGB	0-4F65
U+8944.0	fR/S	 145.11
U+8944.0	fTotalStrokes	 17
U+8944.0	fCangjie	YRRV
U+8944.0	fFourCorner	00732
U+8944.0	fHakka	  siong1
U+8944.0	fCantonese	soeng1
U+8944.0	fMandarin	xiang1
U+8944.0	fEnglish	[1] aid; help; assist [2] achieve; accomplish; complete [3] rise; raise [4] remove [5] high [6] a Chinese family name
U+8945.0	fUTF8	襅
U+8945.0	fGB	E-3D3C
U+8945.0	fR/S	 145.11
U+8945.0	fTotalStrokes	 16; 17
U+8945.0	fCangjie	LWTJ
U+8945.0	fCantonese	bat1
U+8946.0	fUTF8	襆
U+8946.0	fCNS11643	2-6341
U+8946.0	fBig5	F0D6
U+8946.0	fGB	3-6741
U+8946.0	fR/S	 145.12
U+8946.0	fTotalStrokes	 17; 18
U+8946.0	fCangjie	LTCO
U+8946.0	fFourCorner	32234
U+8946.0	fHakka	  puk7
U+8946.0	fCantonese	fuk6; buk6 (also)
U+8946.0	fMandarin	pu2; bu3; fu2
U+8946.0	fEnglish	hood or cowl
U+8947.0	fUTF8	襇
U+8947.0	fGB	1-7150
U+8947.0	fR/S	 145.12
U+8947.0	fTotalStrokes	 17; 18
U+8947.0	fCangjie	LANA
U+8947.0	fFourCorner	3722
U+8947.0	fHakka	 gan3; gien1; gan5
U+8947.0	fCantonese	gaan3 (also); gaan2; gan2
U+8947.0	fMandarin	jian3
U+8947.0	fEnglish	the pleat of a skirt
U+8948.0	fUTF8	襈
U+8948.0	fCNS11643	3-5879
U+8948.0	fGB	3-6747
U+8948.0	fR/S	  145.12
U+8948.0	fTotalStrokes	 17; 18
U+8948.0	fCangjie	LRUC
U+8948.0	fFourCorner	37280
U+8948.0	fCantonese	zyun6
U+8948.0	fMandarin	zhuan4
U+8949.0	fUTF8	襉
U+8949.0	fCNS11643	2-6344
U+8949.0	fBig5	F0D9
U+8949.0	fGB	E-3D3D
U+8949.0	fR/S	 145.12
U+8949.0	fTotalStrokes	 17; 18
U+8949.0	fCangjie	LANB
U+8949.0	fFourCorner	37220
U+8949.0	fCantonese	gaan3; gaan2
U+8949.0	fMandarin	jian4; jian3
U+894A.0	fUTF8	襊
U+894A.0	fCNS11643	3-587A
U+894A.0	fGB	3-6742
U+894A.0	fR/S	  145.12
U+894A.0	fTotalStrokes	 17; 18
U+894A.0	fCangjie	LASE
U+894A.0	fFourCorner	36240
U+894A.0	fCantonese	ceoi3
U+894A.0	fMandarin	cui4; zui4; cuo1
U+894B.0	fUTF8	襋
U+894B.0	fCNS11643	2-633E
U+894B.0	fBig5	F0D3
U+894B.0	fGB	3-673F
U+894B.0	fR/S	 145.12
U+894B.0	fTotalStrokes	 17; 18
U+894B.0	fCangjie	LDBB
U+894B.0	fFourCorner	35292
U+894B.0	fHakka	  git7
U+894B.0	fCantonese	gik1
U+894B.0	fMandarin	ji2
U+894B.0	fEnglish	collar
U+894C.0	fUTF8	襌
U+894C.0	fCNS11643	2-6340
U+894C.0	fBig5	F0D5
U+894C.0	fGB	3-6675
U+894C.0	fR/S	 145.12
U+894C.0	fTotalStrokes	 17; 18
U+894C.0	fCangjie	LRRJ
U+894C.0	fFourCorner	36256
U+894C.0	fHakka	  dan1
U+894C.0	fCantonese	daan1
U+894C.0	fMandarin	dan1
U+894C.0	fEnglish	unlined garment
U+894D.0	fUTF8	襍
U+894D.0	fCNS11643	3-587C
U+894D.0	fGB	E-3D3E
U+894D.0	fR/S	 145.12
U+894D.0	fTotalStrokes	 17; 18
U+894D.0	fCangjie	LOGD
U+894D.0	fFourCorner	30290
U+894D.0	fHakka	 cap8
U+894D.0	fCantonese	zaap6
U+894D.0	fMandarin	za2; yong1
U+894D.0	fEnglish	 mixed; blended mix; mingle
U+894E.0	fUTF8	襎
U+894E.0	fCNS11643	3-587B
U+894E.0	fGB	3-6743
U+894E.0	fR/S	  145.12
U+894E.0	fTotalStrokes	 17; 18
U+894E.0	fCangjie	LHDW
U+894E.0	fFourCorner	32260
U+894E.0	fCantonese	faan4
U+894E.0	fMandarin	fan2
U+894F.0	fUTF8	襏
U+894F.0	fCNS11643	2-633F
U+894F.0	fBig5	F0D4
U+894F.0	fGB	3-6650
U+894F.0	fR/S	 145.12
U+894F.0	fTotalStrokes	 17; 18
U+894F.0	fCangjie	LNOE
U+894F.0	fFourCorner	32247
U+894F.0	fHakka	 pat7
U+894F.0	fCantonese	but6; but3 (also)
U+894F.0	fMandarin	bo2; bo1
U+894F.0	fEnglish	raincoat
U+8950.0	fUTF8	襐
U+8950.0	fCNS11643	2-6342
U+8950.0	fBig5	F0D7
U+8950.0	fGB	3-673B
U+8950.0	fR/S	 145.12
U+8950.0	fTotalStrokes	 17; 18
U+8950.0	fCangjie	LNAO
U+8950.0	fFourCorner	37232
U+8950.0	fCantonese	zoeng6
U+8950.0	fMandarin	xiang4; dang4
U+8951.0	fUTF8	襑
U+8951.0	fCNS11643	2-6343
U+8951.0	fBig5	F0D8
U+8951.0	fGB	5-6554
U+8951.0	fR/S	 145.12
U+8951.0	fTotalStrokes	 17; 18
U+8951.0	fCangjie	LSMI
U+8951.0	fFourCorner	37246
U+8951.0	fCantonese	cam4; taam4
U+8951.0	fMandarin	xin2; tan3; xun2
U+8952.0	fUTF8	襒
U+8952.0	fCNS11643	2-5F44
U+8952.0	fBig5	EE76
U+8952.0	fGB	3-6738
U+8952.0	fR/S	 145.12
U+8952.0	fTotalStrokes	 17; 18
U+8952.0	fCangjie	LFBK
U+8952.0	fFourCorner	38340
U+8952.0	fCantonese	bit6; bit3
U+8952.0	fMandarin	bie2; bi4
U+8953.0	fUTF8	襓
U+8953.0	fCNS11643	2-633D
U+8953.0	fBig5	F0D2
U+8953.0	fGB	5-6550
U+8953.0	fR/S	 145.12
U+8953.0	fTotalStrokes	 17; 18
U+8953.0	fCangjie	LGGU
U+8953.0	fFourCorner	34211
U+8953.0	fHakka	 ngiau2; ngieu2
U+8953.0	fCantonese	jiu4
U+8953.0	fMandarin	rao2; rao4
U+8953.0	fEnglish	cloth cover of a scabbard
U+8954.0	fUTF8	襔
U+8954.0	fCNS11643	3-664A
U+8954.0	fGB	E-3D3F
U+8954.0	fR/S	 145.11
U+8954.0	fTotalStrokes	 16; 17
U+8954.0	fCangjie	LTLB
U+8954.0	fFourCorner	3422
U+8954.0	fCantonese	loeng5
U+8954.0	fMandarin	man3
U+8955.0	fUTF8	襕
U+8955.0	fGB	8-2C5E
U+8955.0	fR/S	 145.12
U+8955.0	fTotalStrokes	 17; 18
U+8955.0	fCangjie	LLSW
U+8955.0	fCantonese	laan4
U+8955.0	fMandarin	lan2
U+8956.0	fUTF8	襖
U+8956.0	fCNS11643	1-7878
U+8956.0	fBig5	C3CD
U+8956.0	fGB	1-3040
U+8956.0	fR/S	 145.13; 145.12
U+8956.0	fTotalStrokes	 18; 19
U+8956.0	fCangjie	LHBK
U+8956.0	fFourCorner	37234
U+8956.0	fHakka	  au3;  o3
U+8956.0	fCantonese	ou3 (also); ngou3 (also); ou2; ngou2 (also)
U+8956.0	fMandarin	ao3
U+8956.0	fEnglish	 outer garments padded with cotton or lined with fur; coat; jacket
U+8957.0	fUTF8	襗
U+8957.0	fCNS11643	2-6677
U+8957.0	fBig5	F2EC
U+8957.0	fGB	3-6651
U+8957.0	fR/S	 145.13
U+8957.0	fTotalStrokes	 18; 19
U+8957.0	fCangjie	LWLJ
U+8957.0	fFourCorner	36241
U+8957.0	fHakka	  tok8; jit8; rit8
U+8957.0	fCantonese	zaak6; dok6 (also); jik6
U+8957.0	fMandarin	duo2; yi4; ze4; ze2
U+8957.0	fEnglish	underclothes
U+8958.0	fUTF8	襘
U+8958.0	fCNS11643	2-667A
U+8958.0	fBig5	F2EF
U+8958.0	fGB	3-665C
U+8958.0	fR/S	 145.13
U+8958.0	fTotalStrokes	 18; 19
U+8958.0	fCangjie	LOMA
U+8958.0	fFourCorner	38266
U+8958.0	fHakka	  kwai5; k'uai5
U+8958.0	fCantonese	kui2; kwui2 (also); kui3; fui2
U+8958.0	fMandarin	gui4; hui4
U+8958.0	fEnglish	[1] fastening of a girdle or collar [2] girdle [3] collar
U+8959.0	fUTF8	襙
U+8959.0	fCNS11643	2-667C
U+8959.0	fBig5	F2F1
U+8959.0	fGB	E-3D40
U+8959.0	fR/S	 145.13
U+8959.0	fTotalStrokes	 18; 19
U+8959.0	fCangjie	LRRD
U+8959.0	fFourCorner	36294
U+8959.0	fCantonese	cou3; cou4
U+8959.0	fMandarin	cao4
U+895A.0	fUTF8	襚
U+895A.0	fCNS11643	2-6675
U+895A.0	fBig5	F2EA
U+895A.0	fGB	3-6745
U+895A.0	fR/S	 145.13
U+895A.0	fTotalStrokes	 18; 19
U+895A.0	fCangjie	LYTO
U+895A.0	fFourCorner	38133
U+895A.0	fHakka	 sui5; sui6
U+895A.0	fCantonese	seoi6
U+895A.0	fMandarin	sui4
U+895A.0	fEnglish	grave clothes
U+895B.0	fUTF8	襛
U+895B.0	fCNS11643	2-6676
U+895B.0	fBig5	F2EB
U+895B.0	fGB	3-6661
U+895B.0	fR/S	 145.13
U+895B.0	fTotalStrokes	 18; 19
U+895B.0	fCangjie	LTWV
U+895B.0	fFourCorner	35232
U+895B.0	fHakka	 nung2;  ngiung2; jung3; niung2
U+895B.0	fCantonese	nung4; lung4 (also)
U+895B.0	fMandarin	nong2; rong2
U+895B.0	fEnglish	fine clothes
U+895C.0	fUTF8	襜
U+895C.0	fCNS11643	2-6679
U+895C.0	fBig5	F2EE
U+895C.0	fGB	3-674A
U+895C.0	fR/S	 145.13
U+895C.0	fTotalStrokes	 18; 19
U+895C.0	fCangjie	LNCR
U+895C.0	fFourCorner	37261
U+895C.0	fHakka	 zham1; zam1
U+895C.0	fCantonese	cim1; sim4 (also); zim1; lim5
U+895C.0	fMandarin	chan1; chan4; dan1
U+895C.0	fEnglish	[1] lower front of a robe [2] clean and neat (appearances) [3] shaking; vibrating; flapping (curtains)
U+895D.0	fUTF8	襝
U+895D.0	fCNS11643	2-667B
U+895D.0	fBig5	F2F0
U+895D.0	fGB	1-714F
U+895D.0	fR/S	 145.13
U+895D.0	fTotalStrokes	 18; 19
U+895D.0	fCangjie	LOMO
U+895D.0	fFourCorner	38286
U+895D.0	fHakka	 liam2
U+895D.0	fCantonese	lim5; lim6 (also)
U+895D.0	fMandarin	lian4; lian3; chan1; lian2
U+895D.0	fEnglish	draw one's hands into sleeve
U+895E.0	fUTF8	襞
U+895E.0	fCNS11643	1-7879
U+895E.0	fBig5	C3CE
U+895E.0	fGB	0-7445
U+895E.0	fR/S	 145.13
U+895E.0	fTotalStrokes	 19
U+895E.0	fCangjie	SJYHV
U+895E.0	fFourCorner	70732
U+895E.0	fHakka	 piak7; pit7
U+895E.0	fCantonese	bik1; pik1
U+895E.0	fMandarin	bi4
U+895E.0	fEnglish	 fold clothes
U+895F.0	fUTF8	襟
U+895F.0	fCNS11643	1-7877
U+895F.0	fBig5	C3CC
U+895F.0	fGB	0-3D73
U+895F.0	fR/S	 145.13
U+895F.0	fTotalStrokes	 18; 19
U+895F.0	fCangjie	LDDF
U+895F.0	fFourCorner	34291
U+895F.0	fHakka	  kim1; pan3
U+895F.0	fCantonese	kam1; gam1
U+895F.0	fMandarin	jin1
U+895F.0	fEnglish	[1] lapel; collar of a garment or robe [2] aspiration; ambition; the mental outlook [3] converge (waters) [4] the husbands of one's sisters
U+8960.0	fUTF8	襠
U+8960.0	fCNS11643	1-7876
U+8960.0	fBig5	C3CB
U+8960.0	fGB	1-7149
U+8960.0	fR/S	 145.13
U+8960.0	fTotalStrokes	 18; 19
U+8960.0	fCangjie	LFBW
U+8960.0	fFourCorner	39266
U+8960.0	fHakka	 dong1;  long5; long6; nong5
U+8960.0	fCantonese	dong1
U+8960.0	fMandarin	dang1
U+8960.0	fEnglish	the crotch or seat of pants; pants; panties
U+8961.0	fUTF8	襡
U+8961.0	fCNS11643	2-6678
U+8961.0	fBig5	F2ED
U+8961.0	fGB	3-6749
U+8961.0	fR/S	 145.13
U+8961.0	fTotalStrokes	 18; 19
U+8961.0	fCangjie	LWLI
U+8961.0	fFourCorner	36227
U+8961.0	fHakka	 chuk7; zuk7; zhuk7; shuk8
U+8961.0	fCantonese	suk6; duk6
U+8961.0	fMandarin	shu2; shu3; dou3; dou4; du2; lou4; shu4; zhuo2
U+8961.0	fEnglish	short coat
U+8962.0	fUTF8	襢
U+8962.0	fCNS11643	2-6674
U+8962.0	fBig5	F2E9
U+8962.0	fGB	E-3D41
U+8962.0	fR/S	 145.13
U+8962.0	fTotalStrokes	 18; 19
U+8962.0	fCangjie	LYWM
U+8962.0	fFourCorner	30216
U+8962.0	fHakka	  tan3
U+8962.0	fCantonese	zin3; taan2; daan6
U+8962.0	fMandarin	tan3; tan2; zhan4; zhan3
U+8962.0	fEnglish	 strip lay bare bared naked
U+8963.0	fUTF8	襣
U+8963.0	fCNS11643	2-6976
U+8963.0	fBig5	F4CA
U+8963.0	fGB	3-674B
U+8963.0	fR/S	 145.14
U+8963.0	fTotalStrokes	 19; 20
U+8963.0	fCangjie	LHUL
U+8963.0	fFourCorner	36246
U+8963.0	fCantonese	bei6; jyu4
U+8963.0	fMandarin	bi4; bi2
U+8964.0	fUTF8	襤
U+8964.0	fCNS11643	1-7A3B
U+8964.0	fBig5	C4B0
U+8964.0	fGB	1-715C
U+8964.0	fR/S	 145.14
U+8964.0	fTotalStrokes	 19; 20
U+8964.0	fCangjie	LSIT
U+8964.0	fFourCorner	38217
U+8964.0	fHakka	  lam2; nam2; lam3
U+8964.0	fCantonese	laam4
U+8964.0	fMandarin	lan2
U+8964.0	fEnglish	[1] ragged garments; clothes without a hem [2] shabbily dressed
U+8965.0	fUTF8	襥
U+8965.0	fCNS11643	3-5D29
U+8965.0	fGB	E-3D42
U+8965.0	fR/S	 145.14
U+8965.0	fTotalStrokes	 19; 20
U+8965.0	fCangjie	LOTO
U+8965.0	fFourCorner	32230
U+8965.0	fCantonese	fuk6; buk6 (also)
U+8965.0	fMandarin	pu2
U+8966.0	fUTF8	襦
U+8966.0	fCNS11643	2-6977
U+8966.0	fBig5	F4CB
U+8966.0	fGB	0-7160
U+8966.0	fR/S	 145.14
U+8966.0	fTotalStrokes	 19; 20
U+8966.0	fCangjie	LMBB
U+8966.0	fFourCorner	31227
U+8966.0	fHakka	 si1; ji2; ri2; i2
U+8966.0	fCantonese	jyu4
U+8966.0	fMandarin	ru2
U+8966.0	fEnglish	[1] short coat; jacket; short top garment [2] the neckwear for babies [3] very fine silk fabric
U+8967.0	fUTF8	襧
U+8967.0	fCNS11643	3-5D28
U+8967.0	fGB	E-3D43
U+8967.0	fR/S	  145.14
U+8967.0	fTotalStrokes	 19; 20
U+8967.0	fCangjie	LMFB
U+8967.0	fFourCorner	31220
U+8967.0	fHakka	 ni3
U+8967.0	fCantonese	zi2
U+8967.0	fMandarin	zhi3
U+8968.0	fUTF8	襨
U+8968.0	fGB	1-7D36
U+8968.0	fR/S	 145.14
U+8968.0	fTotalStrokes	 19; 20
U+8968.0	fCangjie	XLTGI
U+8968.0	fCantonese	deoi3
U+8969.0	fUTF8	襩
U+8969.0	fCNS11643	2-6C38
U+8969.0	fBig5	F649
U+8969.0	fGB	5-6560
U+8969.0	fR/S	 145.15
U+8969.0	fTotalStrokes	 20; 21
U+8969.0	fCangjie	LGWC
U+8969.0	fFourCorner	34286
U+8969.0	fHakka	 tuk8; shuk8; suk8; zhuk7
U+8969.0	fCantonese	suk6; suk1
U+8969.0	fMandarin	shu3; du2; long3
U+8969.0	fEnglish	a short skirt or tunic
U+896A.0	fUTF8	襪
U+896A.0	fCNS11643	1-7B28
U+896A.0	fBig5	C4FB
U+896A.0	fGB	1-4D60
U+896A.0	fR/S	 145.15
U+896A.0	fTotalStrokes	 20; 21
U+896A.0	fCangjie	LTWI
U+896A.0	fFourCorner	34253
U+896A.0	fHakka	  mat7
U+896A.0	fCantonese	mat6; maat6 (also); mat2 (also)
U+896A.0	fMandarin	wa4
U+896A.0	fEnglish	socks; stockings
U+896B.0	fUTF8	襫
U+896B.0	fCNS11643	2-6C3A
U+896B.0	fBig5	F64B
U+896B.0	fGB	3-674C
U+896B.0	fR/S	 145.15
U+896B.0	fTotalStrokes	 20; 21
U+896B.0	fCangjie	LKMA
U+896B.0	fFourCorner	34236
U+896B.0	fHakka	 sit7; shit7; siit7
U+896B.0	fCantonese	sik1
U+896B.0	fMandarin	shi4
U+896B.0	fEnglish	raincoat
U+896C.0	fUTF8	襬
U+896C.0	fCNS11643	1-7B29
U+896C.0	fBig5	C4FC
U+896C.0	fGB	1-7821
U+896C.0	fR/S	 145.15
U+896C.0	fTotalStrokes	 20; 21
U+896C.0	fCangjie	LWLP
U+896C.0	fFourCorner	36211
U+896C.0	fCantonese	baai2; bei1
U+896C.0	fMandarin	bai3; bei1; pei4
U+896C.0	fEnglish	lower part of a robe or Chinese gown
U+896D.0	fUTF8	襭
U+896D.0	fCNS11643	2-6C37
U+896D.0	fBig5	F648
U+896D.0	fGB	3-673D
U+896D.0	fR/S	 145.15
U+896D.0	fTotalStrokes	 20; 21
U+896D.0	fCangjie	LGRC
U+896D.0	fFourCorner	31286
U+896D.0	fHakka	  giet7; get7
U+896D.0	fCantonese	kit3; git3
U+896D.0	fMandarin	xie2
U+896D.0	fEnglish	tuck up hem of garment and wrap
U+896E.0	fUTF8	襮
U+896E.0	fCNS11643	2-6C39
U+896E.0	fBig5	F64A
U+896E.0	fGB	3-674D
U+896E.0	fR/S	 145.15
U+896E.0	fTotalStrokes	 20; 21
U+896E.0	fCangjie	LATE
U+896E.0	fFourCorner	36232
U+896E.0	fHakka	  pok7; bok7
U+896E.0	fCantonese	bok3; buk6 (also); buk1
U+896E.0	fMandarin	bo2
U+896E.0	fEnglish	 collar expose
U+896F.0	fUTF8	襯
U+896F.0	fCNS11643	1-7B72
U+896F.0	fBig5	C5A8
U+896F.0	fGB	1-3344
U+896F.0	fR/S	 145.16
U+896F.0	fTotalStrokes	 21; 22
U+896F.0	fCangjie	LYDU
U+896F.0	fFourCorner	36210
U+896F.0	fHakka	 cang5;  cen5; cin5; chin5; cihn5
U+896F.0	fCantonese	can3
U+896F.0	fMandarin	chen4
U+896F.0	fEnglish	[1] underwear; inner garments [2] provide a background; background; bring to the fore; supplement [3] lining; liner [4] line; place something underneath [5] give alms (as to Buddhist monks, etc.)
U+8970.0	fUTF8	襰
U+8970.0	fCNS11643	5-7438
U+8970.0	fGB	5-6633
U+8970.0	fR/S	 145.12; 145.16
U+8970.0	fTotalStrokes	 21; 22; 18
U+8970.0	fCangjie	LDLC
U+8970.0	fFourCorner	37280
U+8970.0	fCantonese	laai6
U+8970.0	fMandarin	lai4
U+8971.0	fUTF8	襱
U+8971.0	fCNS11643	2-6E21
U+8971.0	fBig5	F752
U+8971.0	fGB	3-663F
U+8971.0	fR/S	 145.16
U+8971.0	fTotalStrokes	 21; 22
U+8971.0	fCangjie	LYBP
U+8971.0	fFourCorner	31211
U+8971.0	fHakka	 lung2
U+8971.0	fCantonese	lung4; lung5 (also)
U+8971.0	fMandarin	long2; long3; long4; zhong4
U+8971.0	fEnglish	 legs of trousers over-alls worn by workmen
U+8972.0	fUTF8	襲
U+8972.0	fCNS11643	1-7B71
U+8972.0	fBig5	C5A7
U+8972.0	fGB	1-4F2E
U+8972.0	fR/S	 145.16
U+8972.0	fTotalStrokes	 22
U+8972.0	fCangjie	YPYHV
U+8972.0	fFourCorner	01732
U+8972.0	fHakka	  sip8;  cip8; sip7
U+8972.0	fCantonese	zaap6; zap6
U+8972.0	fMandarin	xi2
U+8972.0	fEnglish	[1] put on; clothe in; wear [2] suit (clothes); set of (dress) [3] repeated; double [4] hereditary; inherit [5] raid; attack; take by surprise; assail [6] plagiarize; appropriate [7] enter (town, etc.) [8] unite; accord with [9] confirm to [10] a Chinese family name
U+8973.0	fUTF8	襳
U+8973.0	fCNS11643	2-6F4C
U+8973.0	fBig5	F7FD
U+8973.0	fGB	3-674F
U+8973.0	fR/S	 145.17
U+8973.0	fTotalStrokes	 22; 23
U+8973.0	fCangjie	LOIM
U+8973.0	fFourCorner	33250
U+8973.0	fHakka	  siam2
U+8973.0	fCantonese	sim1 (also); cim1; sam1
U+8973.0	fMandarin	xian1; shen1
U+8973.0	fEnglish	[1] underclothes [2] girdle [3] clothes of fur [4] waving
U+8974.0	fUTF8	襴
U+8974.0	fCNS11643	2-6F4B
U+8974.0	fBig5	F7FC
U+8974.0	fGB	3-6744
U+8974.0	fR/S	 145.17
U+8974.0	fTotalStrokes	 22; 23
U+8974.0	fCangjie	LANW
U+8974.0	fFourCorner	37220
U+8974.0	fHakka	 lan2
U+8974.0	fCantonese	laan4
U+8974.0	fMandarin	lan2
U+8974.0	fEnglish	a one piece garment
U+8975.0	fUTF8	襵
U+8975.0	fCNS11643	4-6C25
U+8975.0	fGB	5-657C
U+8975.0	fR/S	  145.18
U+8975.0	fTotalStrokes	 23; 24
U+8975.0	fCangjie	LSJJ
U+8975.0	fFourCorner	31240
U+8975.0	fHakka	 ngiap7
U+8975.0	fCantonese	zip3; zaap3 (also)
U+8975.0	fMandarin	zhe2; zhe3
U+8975.0	fEnglish	a pleat in a skirt
U+8976.0	fUTF8	襶
U+8976.0	fCNS11643	2-6F4A
U+8976.0	fBig5	F7FB
U+8976.0	fGB	3-674E
U+8976.0	fR/S	 145.18
U+8976.0	fTotalStrokes	 23; 24
U+8976.0	fCangjie	LJIC
U+8976.0	fFourCorner	33250
U+8976.0	fHakka	  dai5
U+8976.0	fCantonese	daai3; de2 (also)
U+8976.0	fMandarin	dai4; de1
U+8976.0	fEnglish	used in combination
U+8976.1	fUTF8	襶
U+8976.1	fCNS11643	2-6F4A
U+8976.1	fBig5	F7FB
U+8976.1	fGB	3-674E
U+8976.1	fR/S	 145.18
U+8976.1	fTotalStrokes	 23; 24
U+8976.1	fCangjie	LJIC
U+8976.1	fFourCorner	33250
U+8976.1	fMandarin	de5
U+8976.1	fEnglish	used in combination
U+8977.0	fUTF8	襷
U+8977.0	fCNS11643	F-684A
U+8977.0	fGB	E-3D44
U+8977.0	fR/S	 145.18
U+8977.0	fTotalStrokes	 23; 24
U+8977.0	fCangjie	LHCQ
U+8977.0	fCantonese	geoi2
U+8977.0	fEnglish	a cord for holding up sleeves
U+8978.0	fUTF8	襸
U+8978.0	fCNS11643	3-6149
U+8978.0	fGB	5-6637
U+8978.0	fR/S	  145.19
U+8978.0	fTotalStrokes	 24; 25
U+8978.0	fCangjie	LHUC
U+8978.0	fFourCorner	34280
U+8978.0	fCantonese	zaan3
U+8978.0	fMandarin	zan4
U+8979.0	fUTF8	襹
U+8979.0	fCNS11643	2-7137
U+8979.0	fBig5	F948
U+8979.0	fGB	3-6666
U+8979.0	fR/S	 145.19
U+8979.0	fTotalStrokes	 24; 25
U+8979.0	fCangjie	LMMP
U+8979.0	fFourCorner	31211
U+8979.0	fCantonese	si1
U+8979.0	fMandarin	shi1; shi4; xi3
U+897A.0	fUTF8	襺
U+897A.0	fCNS11643	2-7138
U+897A.0	fBig5	F949
U+897A.0	fGB	E-3D45
U+897A.0	fR/S	 145.19
U+897A.0	fTotalStrokes	 24; 25
U+897A.0	fCangjie	LTBI
U+897A.0	fFourCorner	34227
U+897A.0	fHakka	 gan3; kian3; gian3
U+897A.0	fCantonese	gaan2; gin2
U+897A.0	fMandarin	jian3
U+897A.0	fEnglish	silk clothed wadded with cotton
U+897B.0	fUTF8	襻
U+897B.0	fCNS11643	2-713A
U+897B.0	fBig5	F94B
U+897B.0	fGB	0-7161
U+897B.0	fR/S	 145.19
U+897B.0	fTotalStrokes	 24; 25
U+897B.0	fCangjie	LDDQ
U+897B.0	fFourCorner	34252
U+897B.0	fHakka	 pan5
U+897B.0	fCantonese	paan3
U+897B.0	fMandarin	pan4
U+897B.0	fEnglish	[1] loop for a button [2] tie or fasten with a rope, string, etc.
U+897C.0	fUTF8	襼
U+897C.0	fCNS11643	2-7139
U+897C.0	fBig5	F94A
U+897C.0	fGB	3-662C
U+897C.0	fR/S	 145.19
U+897C.0	fTotalStrokes	 24; 25
U+897C.0	fCangjie	LTGI
U+897C.0	fFourCorner	34231
U+897C.0	fHakka	 ngi6; ngi5; mi5; mui5
U+897C.0	fCantonese	ngai6; ai6 (also); mai3
U+897C.0	fMandarin	yi4
U+897C.0	fEnglish	'kimono' sleeve
U+897D.0	fUTF8	襽
U+897D.0	fCNS11643	3-617D
U+897D.0	fGB	E-3D46
U+897D.0	fR/S	 145.17
U+897D.0	fTotalStrokes	 26; 27; 23
U+897D.0	fCangjie	LTAW
U+897D.0	fFourCorner	34220
U+897D.0	fCantonese	laan4
U+897D.0	fMandarin	lan2
U+897E.0	fUTF8	襾
U+897E.0	fCNS11643	2-2270
U+897E.0	fBig5	CA50
U+897E.0	fGB	8-287C
U+897E.0	fR/S	 146.0
U+897E.0	fTotalStrokes	 6
U+897E.0	fCangjie	MUB
U+897E.0	fFourCorner	10227
U+897E.0	fHakka	 a3; a5
U+897E.0	fCantonese	aa3; ngaa3 (also); haa2; kaa3; kaa1; aa1
U+897E.0	fMandarin	ya4; xi1
U+897E.0	fEnglish	 cover Kangxi radical number 146
U+897F.0	fUTF8	西
U+897F.0	fCNS11643	1-4869
U+897F.0	fBig5	A6E8
U+897F.0	fGB	0-4E77
U+897F.0	fR/S	 146.0
U+897F.0	fTotalStrokes	 6
U+897F.0	fCangjie	MCW
U+897F.0	fFourCorner	10600
U+897F.0	fHakka	  si1
U+897F.0	fCantonese	sai1
U+897F.0	fMandarin	xi1
U+897F.0	fEnglish	[1] west(ern); the west [2] Western; the West; European; American; Occidental; foreign [3] a Chinese family name
U+897F.1	fUTF8	西
U+897F.1	fCNS11643	1-4869
U+897F.1	fBig5	A6E8
U+897F.1	fGB	0-4E77
U+897F.1	fR/S	 146.0
U+897F.1	fTotalStrokes	 6
U+897F.1	fCangjie	MCW
U+897F.1	fFourCorner	10600
U+897F.1	fMandarin	xi5
U+8980.0	fUTF8	覀
U+8980.0	fGB	8-2F79
U+8980.0	fR/S	 146.0
U+8980.0	fTotalStrokes	 6
U+8980.0	fCangjie	MLLW
U+8980.0	fMandarin	xi1
U+8980.0	fEnglish	variant of radical 146
U+8981.0	fUTF8	要
U+8981.0	fCNS11643	1-5351
U+8981.0	fBig5	AD6E
U+8981.0	fGB	0-522A
U+8981.0	fR/S	 146.3
U+8981.0	fTotalStrokes	 9
U+8981.0	fCangjie	MWV
U+8981.0	fFourCorner	10404
U+8981.0	fHakka	  jau1; jeu1; rau1; rau5; jeu5
U+8981.0	fCantonese	jiu3; jiu1; jiu2
U+8981.0	fMandarin	yao4; yao3
U+8981.0	fEnglish	[1] necessary; essential; necessity; important [2] must; should; ought to [3] want; demand; need; require; desire; take [4] summarise; summary; generalisation; synopsis [5] will; shall (future tense) [6] brief [7] if; in case
U+8981.1	fUTF8	要
U+8981.1	fCNS11643	1-5351
U+8981.1	fBig5	AD6E
U+8981.1	fGB	0-522A
U+8981.1	fR/S	 146.3
U+8981.1	fTotalStrokes	 9
U+8981.1	fCangjie	MWV
U+8981.1	fFourCorner	10404
U+8981.1	fHakka	 jau5
U+8981.1	fMandarin	yao1
U+8981.1	fEnglish	[1] invite; request the presence of [2] engage; date; make an agreement [3] ask for; demand; claim; make a claim; request; requests [4] coerce; force; blackmail; threaten [5] stop [6] waist; midriff [7] a Chinese family name
U+8982.0	fUTF8	覂
U+8982.0	fCNS11643	2-3948
U+8982.0	fBig5	D7C5
U+8982.0	fGB	3-605F
U+8982.0	fR/S	 146.5
U+8982.0	fTotalStrokes	 11
U+8982.0	fCangjie	MWHIO
U+8982.0	fFourCorner	10307
U+8982.0	fHakka	  fung3
U+8982.0	fCantonese	fung2; fung3; faan3
U+8982.0	fMandarin	feng3; ban3; fan4; feng4
U+8982.0	fEnglish	to throw a rider
U+8983.0	fUTF8	覃
U+8983.0	fCNS11643	1-617D
U+8983.0	fBig5	B5F7
U+8983.0	fGB	0-717B
U+8983.0	fR/S	 146.6
U+8983.0	fTotalStrokes	 12
U+8983.0	fCangjie	MWAJ
U+8983.0	fFourCorner	10406
U+8983.0	fHakka	 tam2
U+8983.0	fCantonese	taam4; cam4
U+8983.0	fMandarin	tan2; yan3; shen3; yan2
U+8983.0	fEnglish	[1] spread to; involve [2] deep and vast; profound; extensive
U+8983.1	fUTF8	覃
U+8983.1	fCNS11643	1-617D
U+8983.1	fBig5	B5F7
U+8983.1	fGB	0-717B
U+8983.1	fR/S	 146.6
U+8983.1	fTotalStrokes	 12
U+8983.1	fCangjie	MWAJ
U+8983.1	fFourCorner	10406
U+8983.1	fMandarin	qin2
U+8983.1	fEnglish	a Chinese family name
U+8984.0	fUTF8	覄
U+8984.0	fCNS11643	3-664B
U+8984.0	fGB	E-3D47
U+8984.0	fR/S	 146.6
U+8984.0	fTotalStrokes	 12
U+8984.0	fCangjie	MWOIK
U+8984.0	fFourCorner	1023
U+8984.0	fCantonese	fuk1
U+8985.0	fUTF8	覅
U+8985.0	fCNS11643	2-4741
U+8985.0	fBig5	DFFA
U+8985.0	fGB	3-6060
U+8985.0	fR/S	 146.7
U+8985.0	fTotalStrokes	 13
U+8985.0	fCangjie	MVPHH
U+8985.0	fFourCorner	17420
U+8985.0	fCantonese	fiu3
U+8985.0	fMandarin	fiao4; biao4
U+8985.0	fEnglish	 contraction of 'do not'
U+8986.0	fUTF8	覆
U+8986.0	fCNS11643	1-773B
U+8986.0	fBig5	C2D0
U+8986.0	fGB	0-3832
U+8986.0	fR/S	 146.12
U+8986.0	fTotalStrokes	 18
U+8986.0	fCangjie	MWHOE
U+8986.0	fFourCorner	10247
U+8986.0	fHakka	  fuk8; fuk7; puk7; fu5; puk8
U+8986.0	fCantonese	fuk1; fau6; fu3
U+8986.0	fMandarin	fu4
U+8986.0	fEnglish	[1] pour out [2] overturn; topple; upset; capsize; put something upside down [3] reply; respond [4] defeat; destroy; conquer [5] investigate; examine carefully [6] cover; screen [7] repeat; a second time; again [8] hide; conceal
U+8987.0	fUTF8	覇
U+8987.0	fCNS11643	3-5B2F
U+8987.0	fGB	E-3D48
U+8987.0	fR/S	 146.13
U+8987.0	fTotalStrokes	 19
U+8987.0	fCangjie	MWTJB
U+8987.0	fFourCorner	10520
U+8987.0	fHakka	 ba5
U+8987.0	fCantonese	baa3
U+8987.0	fMandarin	ba4; po4
U+8987.0	fEnglish	rule by might rather than right
U+8988.0	fUTF8	覈
U+8988.0	fCNS11643	2-667D
U+8988.0	fBig5	F2F2
U+8988.0	fGB	E-3D49
U+8988.0	fR/S	 146.13
U+8988.0	fTotalStrokes	 19
U+8988.0	fCangjie	MWHSK
U+8988.0	fFourCorner	10248
U+8988.0	fHakka	 het7; het8
U+8988.0	fCantonese	hat6
U+8988.0	fMandarin	he2; hu2; qiao4; xie2
U+8988.0	fEnglish	[1] investigate; test; examine [2] deep; deeply [3] the stone (fruit)
U+8989.0	fUTF8	覉
U+8989.0	fCNS11643	3-6046
U+8989.0	fGB	5-5F68
U+8989.0	fR/S	 146.17
U+8989.0	fTotalStrokes	 23
U+8989.0	fCangjie	MWTJR
U+8989.0	fFourCorner	10520
U+8989.0	fCantonese	gei1
U+8989.0	fMandarin	ji1
U+8989.0	fEnglish	variant of &#x7F87;
U+898A.0	fUTF8	覊
U+898A.0	fCNS11643	3-614A
U+898A.0	fGB	E-3D4A
U+898A.0	fR/S	 146.19
U+898A.0	fTotalStrokes	 25
U+898A.0	fCangjie	MWTJF
U+898A.0	fFourCorner	10520
U+898A.0	fHakka	 gi1
U+898A.0	fCantonese	gei1
U+898A.0	fMandarin	ji1
U+898A.0	fEnglish	 halter restrain; hold; control
U+898B.0	fUTF8	見
U+898B.0	fCNS11643	1-4B44
U+898B.0	fBig5	A8A3
U+898B.0	fGB	1-3C7B
U+898B.0	fR/S	 147.0
U+898B.0	fTotalStrokes	 7
U+898B.0	fCangjie	BUHU
U+898B.0	fFourCorner	60210
U+898B.0	fHakka	  gien5;  gen5;  hien5; kian5; hien6; hen3; gian5
U+898B.0	fCantonese	gin3; jin6
U+898B.0	fMandarin	jian4
U+898B.0	fEnglish	[1] see; observe; perceive; catch a sight of; understand; examine [2] visit; see; call on; call at; meet [3] receive (visitors, etc.); come face to face with [4] vide; see [5] to be used roughly as a verb to be in the passive voice [6] move toward a certain direction (said of a situation, condition, etc., as getting worse, better, slimmer, etc.) [7] Kangxi radical
U+898B.1	fUTF8	見
U+898B.1	fCNS11643	1-4B44
U+898B.1	fBig5	A8A3
U+898B.1	fGB	1-3C7B
U+898B.1	fR/S	 147.0
U+898B.1	fTotalStrokes	 7
U+898B.1	fCangjie	BUHU
U+898B.1	fFourCorner	60210
U+898B.1	fMandarin	xian4
U+898B.1	fEnglish	[1] appear; be visible [2] introduce
U+898B.2	fUTF8	見
U+898B.2	fCNS11643	1-4B44
U+898B.2	fBig5	A8A3
U+898B.2	fGB	1-3C7B
U+898B.2	fR/S	 147.0
U+898B.2	fTotalStrokes	 7
U+898B.2	fCangjie	BUHU
U+898B.2	fFourCorner	60210
U+898B.2	fMandarin	jian5
U+898C.0	fUTF8	覌
U+898C.0	fCNS11643	3-2F29
U+898C.0	fGB	E-3D4B
U+898C.0	fR/S	 147.2
U+898C.0	fTotalStrokes	 9
U+898C.0	fCangjie	EBUU
U+898C.0	fFourCorner	16410
U+898C.0	fHakka	 gon1
U+898C.0	fCantonese	gun1; gwun1 (also)
U+898C.0	fMandarin	huan1; guan1; guan4
U+898C.0	fEnglish	 see; observe; view appearance
U+898D.0	fUTF8	覍
U+898D.0	fCNS11643	4-2F76
U+898D.0	fGB	E-3D4C
U+898D.0	fR/S	 147.3
U+898D.0	fTotalStrokes	 10
U+898D.0	fCangjie	FBUU
U+898D.0	fFourCorner	90210
U+898D.0	fCantonese	bin6
U+898D.0	fMandarin	bian4; pan2
U+898E.0	fUTF8	覎
U+898E.0	fCNS11643	4-2F75
U+898E.0	fGB	5-4A5A
U+898E.0	fR/S	 147.3
U+898E.0	fTotalStrokes	 10
U+898E.0	fCangjie	BUND
U+898E.0	fFourCorner	67210
U+898E.0	fCantonese	jin6 (also); jim3
U+898E.0	fMandarin	yan4
U+898F.0	fUTF8	規
U+898F.0	fCNS11643	1-5D3D
U+898F.0	fBig5	B357
U+898F.0	fGB	1-3966
U+898F.0	fR/S	 147.4
U+898F.0	fTotalStrokes	 11
U+898F.0	fCangjie	QOBUU
U+898F.0	fFourCorner	56010
U+898F.0	fHakka	 gui1; kui1; gwi1; kwui1
U+898F.0	fCantonese	kwai1
U+898F.0	fMandarin	gui1; gui4; kui1; xi1; xu4
U+898F.0	fEnglish	[1] rules; regulations; customs; law; usages [2] a pair of compasses [3] plan; scheme [4] advisee so as to correct; admonish [5] copy; imitate
U+8990.0	fUTF8	覐
U+8990.0	fCNS11643	4-3534
U+8990.0	fGB	E-3D4D
U+8990.0	fR/S	 147.4
U+8990.0	fTotalStrokes	 11
U+8990.0	fCangjie	BUKK
U+8990.0	fFourCorner	64210
U+8990.0	fCantonese	gok3; gaau3
U+8990.0	fMandarin	jue2; jiao4
U+8990.0	fEnglish	a sleep to realise to feel
U+8991.0	fUTF8	覑
U+8991.0	fCNS11643	3-3876
U+8991.0	fGB	3-4C2A
U+8991.0	fR/S	  147.4
U+8991.0	fTotalStrokes	 11
U+8991.0	fCangjie	LLBUU
U+8991.0	fFourCorner	26010
U+8991.0	fCantonese	pin2
U+8991.0	fMandarin	pian3
U+8992.0	fUTF8	覒
U+8992.0	fCNS11643	3-3875
U+8992.0	fGB	3-4B5E
U+8992.0	fR/S	  147.4
U+8992.0	fTotalStrokes	 11
U+8992.0	fCangjie	HUBUU
U+8992.0	fFourCorner	26710
U+8992.0	fCantonese	mou6
U+8992.0	fMandarin	mao2
U+8993.0	fUTF8	覓
U+8993.0	fCNS11643	1-5D3C
U+8993.0	fBig5	B356
U+8993.0	fGB	1-4359
U+8993.0	fR/S	 147.4
U+8993.0	fTotalStrokes	 11
U+8993.0	fCangjie	BBUU
U+8993.0	fFourCorner	20216
U+8993.0	fHakka	 met7; mit7; mit8; mi6; mi5
U+8993.0	fCantonese	mik6
U+8993.0	fMandarin	mi4
U+8993.0	fEnglish	 seek; search; look for
U+8994.0	fUTF8	覔
U+8994.0	fCNS11643	3-3877
U+8994.0	fGB	E-3D4E
U+8994.0	fR/S	 147.4
U+8994.0	fTotalStrokes	 11
U+8994.0	fCangjie	MFBUU
U+8994.0	fFourCorner	10210
U+8994.0	fHakka	 met7; mit7; mi6; mi5
U+8994.0	fCantonese	mik6
U+8994.0	fMandarin	mi4
U+8994.0	fEnglish	 search for look after
U+8995.0	fUTF8	覕
U+8995.0	fCNS11643	2-4036
U+8995.0	fBig5	DBD0
U+8995.0	fGB	5-4A60
U+8995.0	fR/S	 147.5
U+8995.0	fTotalStrokes	 12
U+8995.0	fCangjie	PHBUU
U+8995.0	fFourCorner	36010
U+8995.0	fCantonese	pit3; bit1
U+8995.0	fMandarin	mie4; bi4; pie1; bie2; bin4; pie3
U+8996.0	fUTF8	視
U+8996.0	fCNS11643	1-617E
U+8996.0	fBig5	B5F8
U+8996.0	fGB	1-4A53
U+8996.0	fR/S	 147.4; 113.7; 147.5
U+8996.0	fTotalStrokes	 11
U+8996.0	fCangjie	IFBUU
U+8996.0	fFourCorner	36210
U+8996.0	fHakka	 si5; shi5; shi6; sii5; she5; shi3
U+8996.0	fCantonese	si6; si2 (also)
U+8996.0	fMandarin	shi4
U+8996.0	fEnglish	[1] look at; inspect; observe; see; watch [2] consider; regard as; take it for; look upon as [3] show [4] receive (present, etc.) [5] take as a model; imitate
U+8996.1	fUTF8	視
U+8996.1	fCNS11643	1-617E
U+8996.1	fBig5	B5F8
U+8996.1	fGB	1-4A53
U+8996.1	fR/S	 147.4; 113.7; 147.5
U+8996.1	fTotalStrokes	 11
U+8996.1	fCangjie	IFBUU
U+8996.1	fFourCorner	36210
U+8996.1	fMandarin	shi5
U+8997.0	fUTF8	覗
U+8997.0	fCNS11643	2-4038
U+8997.0	fBig5	DBD2
U+8997.0	fGB	5-4A61
U+8997.0	fR/S	 147.5
U+8997.0	fTotalStrokes	 12
U+8997.0	fCangjie	SRBUU
U+8997.0	fFourCorner	16610
U+8997.0	fHakka	 sii1; s'e1; se1; s'e3; se3
U+8997.0	fCantonese	zi6
U+8997.0	fMandarin	si1; si4
U+8997.0	fEnglish	peek
U+8998.0	fUTF8	覘
U+8998.0	fCNS11643	2-4037
U+8998.0	fBig5	DBD1
U+8998.0	fGB	1-6A68
U+8998.0	fR/S	 147.5
U+8998.0	fTotalStrokes	 12
U+8998.0	fCangjie	YRBUU
U+8998.0	fFourCorner	26610
U+8998.0	fHakka	 zham1; zam1; tiam1
U+8998.0	fCantonese	zim1; cim1
U+8998.0	fMandarin	zhan1; chan1; dan1; chan3; chan4; ji1
U+8998.0	fEnglish	 peek; spy on; watch; investigate secretly; inspect; see
U+8999.0	fUTF8	覙
U+8999.0	fCNS11643	6-4E60
U+8999.0	fGB	3-485F
U+8999.0	fR/S	 147.2; 147.5
U+8999.0	fTotalStrokes	 12; 9
U+8999.0	fCangjie	NFBUU
U+8999.0	fFourCorner	86910
U+8999.0	fCantonese	can2
U+8999.0	fMandarin	zhen3; luo2
U+8999.0	fEnglish	explain in detail
U+899A.0	fUTF8	覚
U+899A.0	fCNS11643	3-3E74
U+899A.0	fGB	E-3D4F
U+899A.0	fR/S	 147.5
U+899A.0	fTotalStrokes	 12
U+899A.0	fCangjie	FBBUU
U+899A.0	fFourCorner	23216
U+899A.0	fCantonese	gok3; gaau3
U+899A.0	fMandarin	jue2; jiao4
U+899A.0	fEnglish	 wake up from sleep be conscious
U+899B.0	fUTF8	覛
U+899B.0	fCNS11643	2-4742
U+899B.0	fBig5	DFFB
U+899B.0	fGB	5-4A62
U+899B.0	fR/S	 147.6
U+899B.0	fTotalStrokes	 13
U+899B.0	fCangjie	HVBUU
U+899B.0	fFourCorner	76210
U+899B.0	fCantonese	mak6 (also); mik6
U+899B.0	fMandarin	mo4; mi4
U+899C.0	fUTF8	覜
U+899C.0	fCNS11643	1-6657
U+899C.0	fBig5	B8D0
U+899C.0	fGB	E-3D50
U+899C.0	fR/S	 147.6
U+899C.0	fTotalStrokes	 13
U+899C.0	fCangjie	LMUOU
U+899C.0	fFourCorner	36116
U+899C.0	fHakka	  tiau5
U+899C.0	fCantonese	tiu3; tiu6
U+899C.0	fMandarin	tiao4; tiao1
U+899C.0	fEnglish	mission
U+899D.0	fUTF8	覝
U+899D.0	fCNS11643	2-4E2B
U+899D.0	fBig5	E443
U+899D.0	fGB	5-4A64
U+899D.0	fR/S	 147.7
U+899D.0	fTotalStrokes	 14
U+899D.0	fCangjie	MFBUU
U+899D.0	fFourCorner	16810
U+899D.0	fCantonese	lim4
U+899D.0	fMandarin	lian2
U+899E.0	fUTF8	覞
U+899E.0	fCNS11643	2-4E2E
U+899E.0	fBig5	E446
U+899E.0	fGB	3-485D
U+899E.0	fR/S	 147.7
U+899E.0	fTotalStrokes	 14
U+899E.0	fCangjie	BUBUU
U+899E.0	fFourCorner	66210
U+899E.0	fCantonese	jiu6
U+899E.0	fMandarin	yao4; chao4; shi4
U+899F.0	fUTF8	覟
U+899F.0	fCNS11643	2-4E2D
U+899F.0	fBig5	E445
U+899F.0	fGB	5-4A65
U+899F.0	fR/S	 147.7
U+899F.0	fTotalStrokes	 14
U+899F.0	fCangjie	GPBUU
U+899F.0	fFourCorner	46310
U+899F.0	fHakka	 zhi5; zih5
U+899F.0	fCantonese	zi3
U+899F.0	fMandarin	zhi4
U+899F.0	fEnglish	 examine; inspect
U+89A0.0	fUTF8	覠
U+89A0.0	fCNS11643	4-4735
U+89A0.0	fGB	3-4867
U+89A0.0	fR/S	  147.7
U+89A0.0	fTotalStrokes	 14
U+89A0.0	fCangjie	SRBUU
U+89A0.0	fFourCorner	16610
U+89A0.0	fCantonese	gwan1
U+89A0.0	fMandarin	jun1
U+89A1.0	fUTF8	覡
U+89A1.0	fCNS11643	2-4E2C
U+89A1.0	fBig5	E444
U+89A1.0	fGB	1-6A6A
U+89A1.0	fR/S	 147.7
U+89A1.0	fTotalStrokes	 14
U+89A1.0	fCangjie	MOBUU
U+89A1.0	fFourCorner	16110
U+89A1.0	fHakka	 hit7; sang5; s'ang5; s'ang6; shang5
U+89A1.0	fCantonese	hat6
U+89A1.0	fMandarin	xi2; he2
U+89A1.0	fEnglish	 wizard
U+89A2.0	fUTF8	覢
U+89A2.0	fCNS11643	2-5437
U+89A2.0	fBig5	E7CE
U+89A2.0	fGB	5-4A6A
U+89A2.0	fR/S	 147.8
U+89A2.0	fTotalStrokes	 15
U+89A2.0	fCangjie	FFBUU
U+89A2.0	fFourCorner	96810
U+89A2.0	fCantonese	sim2
U+89A2.0	fMandarin	shan3
U+89A3.0	fUTF8	覣
U+89A3.0	fCNS11643	2-5439
U+89A3.0	fBig5	E7D0
U+89A3.0	fGB	3-486A
U+89A3.0	fR/S	 147.8
U+89A3.0	fTotalStrokes	 15
U+89A3.0	fCangjie	XHVBU
U+89A3.0	fFourCorner	26410
U+89A3.0	fCantonese	wai1
U+89A3.0	fMandarin	wei1; wo1
U+89A4.0	fUTF8	覤
U+89A4.0	fCNS11643	2-5438
U+89A4.0	fBig5	E7CF
U+89A4.0	fGB	5-605C
U+89A4.0	fR/S	 147.8
U+89A4.0	fTotalStrokes	 15
U+89A4.0	fCangjie	YUBUU
U+89A4.0	fFourCorner	26210
U+89A4.0	fCantonese	gwik1
U+89A4.0	fMandarin	xi4
U+89A5.0	fUTF8	覥
U+89A5.0	fCNS11643	3-4E65
U+89A5.0	fGB	5-4A68
U+89A5.0	fR/S	  147.8
U+89A5.0	fTotalStrokes	 15
U+89A5.0	fCangjie	TCBUU
U+89A5.0	fFourCorner	56810
U+89A5.0	fHakka	 tien3
U+89A5.0	fCantonese	tin2
U+89A5.0	fMandarin	tian3
U+89A5.0	fEnglish	ashamed
U+89A6.0	fUTF8	覦
U+89A6.0	fCNS11643	1-7236
U+89A6.0	fBig5	BFCC
U+89A6.0	fGB	1-6A6C
U+89A6.0	fR/S	 147.9
U+89A6.0	fTotalStrokes	 16
U+89A6.0	fCangjie	ONBUU
U+89A6.0	fFourCorner	86210
U+89A6.0	fHakka	 ji2;  ji3; ri2; ri3; i2
U+89A6.0	fCantonese	jyu4; jyu6
U+89A6.0	fMandarin	yu2; ju4; yu4
U+89A6.0	fEnglish	a strong desire for possession; covet
U+89A7.0	fUTF8	覧
U+89A7.0	fCNS11643	3-5266
U+89A7.0	fGB	E-3D51
U+89A7.0	fR/S	 147.9
U+89A7.0	fTotalStrokes	 16
U+89A7.0	fCangjie	SIBUU
U+89A7.0	fFourCorner	78210
U+89A7.0	fCantonese	laam5
U+89A7.0	fMandarin	lan3
U+89A7.0	fEnglish	 look at; inspect perceive
U+89A8.0	fUTF8	覨
U+89A8.0	fCNS11643	4-5359
U+89A8.0	fGB	E-3D52
U+89A8.0	fR/S	  147.9
U+89A8.0	fTotalStrokes	 16
U+89A8.0	fCangjie	RSBUU
U+89A8.0	fFourCorner	66210
U+89A8.0	fCantonese	ngok6; ok6 (also)
U+89A8.0	fMandarin	e4
U+89A9.0	fUTF8	覩
U+89A9.0	fCNS11643	3-4E64
U+89A9.0	fGB	E-3D53
U+89A9.0	fR/S	 147.9; 147.8
U+89A9.0	fTotalStrokes	 16; 15
U+89A9.0	fCangjie	JABUU
U+89A9.0	fFourCorner	46610
U+89A9.0	fHakka	 du5;  du3
U+89A9.0	fCantonese	dou2
U+89A9.0	fMandarin	du3
U+89A9.0	fEnglish	 see; gaze at; witness; observe
U+89AA.0	fUTF8	親
U+89AA.0	fCNS11643	1-7235
U+89AA.0	fBig5	BFCB
U+89AA.0	fGB	1-4757
U+89AA.0	fR/S	 147.9
U+89AA.0	fTotalStrokes	 16
U+89AA.0	fCangjie	YDBUU
U+89AA.0	fFourCorner	06910
U+89AA.0	fHakka	  cin1; zim1
U+89AA.0	fCantonese	can1; can3 (also)
U+89AA.0	fMandarin	qin1; qin4; xin1
U+89AA.0	fEnglish	[1] relatives [2] parents [3] love; intimate; near to; dear [4] personally; personal; person; self [5] kiss
U+89AA.1	fUTF8	親
U+89AA.1	fCNS11643	1-7235
U+89AA.1	fBig5	BFCB
U+89AA.1	fGB	1-4757
U+89AA.1	fR/S	 147.9
U+89AA.1	fTotalStrokes	 16
U+89AA.1	fCangjie	YDBUU
U+89AA.1	fFourCorner	06910
U+89AA.1	fMandarin	qing4
U+89AA.1	fEnglish	relatives by marriage
U+89AA.2	fUTF8	親
U+89AA.2	fCNS11643	1-7235
U+89AA.2	fBig5	BFCB
U+89AA.2	fGB	1-4757
U+89AA.2	fR/S	 147.9
U+89AA.2	fTotalStrokes	 16
U+89AA.2	fCangjie	YDBUU
U+89AA.2	fFourCorner	06910
U+89AA.2	fMandarin	qin5
U+89AB.0	fUTF8	覫
U+89AB.0	fCNS11643	3-5646
U+89AB.0	fGB	3-486F
U+89AB.0	fR/S	  147.10
U+89AB.0	fTotalStrokes	 17
U+89AB.0	fCangjie	YSBUU
U+89AB.0	fFourCorner	06210
U+89AB.0	fCantonese	pong2
U+89AB.0	fMandarin	pang3
U+89AC.0	fUTF8	覬
U+89AC.0	fCNS11643	1-7546
U+89AC.0	fBig5	C1BB
U+89AC.0	fGB	1-6A69
U+89AC.0	fR/S	 147.10
U+89AC.0	fTotalStrokes	 17
U+89AC.0	fCangjie	UTBUU
U+89AC.0	fFourCorner	26110
U+89AC.0	fHakka	  kai3; koi3; ki3; gi5
U+89AC.0	fCantonese	gei3; kei3
U+89AC.0	fMandarin	ji4; xi4
U+89AC.0	fEnglish	 covet; long for; desire for something belonging to others
U+89AD.0	fUTF8	覭
U+89AD.0	fCNS11643	2-5F47
U+89AD.0	fBig5	EE79
U+89AD.0	fGB	3-4870
U+89AD.0	fR/S	 147.10
U+89AD.0	fTotalStrokes	 17
U+89AD.0	fCangjie	BCBUU
U+89AD.0	fFourCorner	76810
U+89AD.0	fHakka	 men2
U+89AD.0	fCantonese	ming4; mik6
U+89AD.0	fMandarin	ming2; mi4; mo4
U+89AD.0	fEnglish	[1] scrutinize [2] examine closely
U+89AE.0	fUTF8	覮
U+89AE.0	fCNS11643	2-5F49
U+89AE.0	fBig5	EE7B
U+89AE.0	fGB	E-3D54
U+89AE.0	fR/S	 147.10
U+89AE.0	fTotalStrokes	 17
U+89AE.0	fCangjie	FFBBU
U+89AE.0	fFourCorner	99216
U+89AE.0	fCantonese	jing4; ping1
U+89AE.0	fMandarin	ping1; ying2
U+89AF.0	fUTF8	覯
U+89AF.0	fCNS11643	2-5F48
U+89AF.0	fBig5	EE7A
U+89AF.0	fGB	1-6A6D
U+89AF.0	fR/S	 147.10
U+89AF.0	fTotalStrokes	 17
U+89AF.0	fCangjie	TBBUU
U+89AF.0	fFourCorner	56510
U+89AF.0	fHakka	 kieu5; geu5
U+89AF.0	fCantonese	gau3; kau3
U+89AF.0	fMandarin	gou4; hou2; hou4; jiang3; jue2
U+89AF.0	fEnglish	 meet; see unexpectedly
U+89B0.0	fUTF8	覰
U+89B0.0	fCNS11643	3-587E
U+89B0.0	fGB	5-4A72
U+89B0.0	fR/S	  147.11
U+89B0.0	fTotalStrokes	 18
U+89B0.0	fCangjie	YMBUU
U+89B0.0	fFourCorner	26210
U+89B0.0	fHakka	  ci5
U+89B0.0	fCantonese	ceoi3
U+89B0.0	fMandarin	qu4; qu1
U+89B0.0	fEnglish	[1] spy watch for [2] watch with almost closed eyes
U+89B1.0	fUTF8	覱
U+89B1.0	fCNS11643	3-587D
U+89B1.0	fGB	3-4869
U+89B1.0	fR/S	  147.11
U+89B1.0	fTotalStrokes	 18
U+89B1.0	fCangjie	JLBUU
U+89B1.0	fFourCorner	52210
U+89B1.0	fCantonese	zaam3
U+89B1.0	fMandarin	zhan4
U+89B2.0	fUTF8	覲
U+89B2.0	fCNS11643	1-773C
U+89B2.0	fBig5	C2D1
U+89B2.0	fGB	1-6A6E
U+89B2.0	fR/S	 147.11
U+89B2.0	fTotalStrokes	 18
U+89B2.0	fCangjie	TMBUU
U+89B2.0	fFourCorner	46110
U+89B2.0	fHakka	 giun3
U+89B2.0	fCantonese	gan2 (also); gan3; gan6 (also)
U+89B2.0	fMandarin	jin4; jin3
U+89B2.0	fEnglish	 have imperial audience; have an audience with a chief of state
U+89B3.0	fUTF8	観
U+89B3.0	fCNS11643	3-5921
U+89B3.0	fGB	E-3D55
U+89B3.0	fR/S	 147.11
U+89B3.0	fTotalStrokes	 18
U+89B3.0	fCangjie	OGBUU
U+89B3.0	fFourCorner	86210
U+89B3.0	fCantonese	gun1; gwun1 (also)
U+89B3.0	fMandarin	guan1; guan4
U+89B3.0	fEnglish	 see; observe; view appearance
U+89B4.0	fUTF8	覴
U+89B4.0	fCNS11643	4-612F
U+89B4.0	fGB	5-4A77
U+89B4.0	fR/S	  147.12
U+89B4.0	fTotalStrokes	 19
U+89B4.0	fCangjie	NTBUU
U+89B4.0	fFourCorner	16110
U+89B4.0	fCantonese	dang1
U+89B4.0	fMandarin	deng1
U+89B5.0	fUTF8	覵
U+89B5.0	fCNS11643	3-5B31
U+89B5.0	fGB	E-3D56
U+89B5.0	fR/S	  147.12
U+89B5.0	fTotalStrokes	 19
U+89B5.0	fCangjie	ABBUU
U+89B5.0	fFourCorner	76210
U+89B5.0	fHakka	 han2
U+89B5.0	fCantonese	gaan3; haan2 (also)
U+89B5.0	fMandarin	jian4
U+89B5.0	fEnglish	 steal look peep at spy on
U+89B6.0	fUTF8	覶
U+89B6.0	fCNS11643	2-6721
U+89B6.0	fBig5	F2F4
U+89B6.0	fGB	5-4A75
U+89B6.0	fR/S	 147.12
U+89B6.0	fTotalStrokes	 19
U+89B6.0	fCangjie	BBBUU
U+89B6.0	fFourCorner	26210
U+89B6.0	fHakka	 lo2
U+89B6.0	fCantonese	lo4
U+89B6.0	fMandarin	luo2; luan3; luan4
U+89B6.0	fEnglish	 see carefully
U+89B7.0	fUTF8	覷
U+89B7.0	fCNS11643	2-667E
U+89B7.0	fBig5	F2F3
U+89B7.0	fGB	1-6A6F
U+89B7.0	fR/S	 147.12
U+89B7.0	fTotalStrokes	 19
U+89B7.0	fCangjie	YMBUU
U+89B7.0	fFourCorner	26210
U+89B7.0	fHakka	 ci5; zii1
U+89B7.0	fCantonese	ceoi3
U+89B7.0	fMandarin	qu4; qu1
U+89B7.0	fEnglish	 peep at; watch; spy on; look; gaze
U+89B8.0	fUTF8	覸
U+89B8.0	fCNS11643	F-6377
U+89B8.0	fGB	3-4866
U+89B8.0	fR/S	 147.12
U+89B8.0	fTotalStrokes	 19
U+89B8.0	fCangjie	AABUU
U+89B8.0	fFourCorner	7621
U+89B8.0	fHakka	 gan5
U+89B8.0	fCantonese	gaan3
U+89B8.0	fMandarin	jian4; xian2
U+89B8.0	fEnglish	 steal look peep at spy on
U+89B9.0	fUTF8	覹
U+89B9.0	fCNS11643	2-6978
U+89B9.0	fBig5	F4CC
U+89B9.0	fGB	5-4A79
U+89B9.0	fR/S	 147.13
U+89B9.0	fTotalStrokes	 20
U+89B9.0	fCangjie	BUHOK
U+89B9.0	fFourCorner	78214
U+89B9.0	fCantonese	mei4
U+89B9.0	fMandarin	wei2
U+89BA.0	fUTF8	覺
U+89BA.0	fCNS11643	1-7A3C
U+89BA.0	fBig5	C4B1
U+89BA.0	fGB	1-3E75
U+89BA.0	fR/S	 147.13
U+89BA.0	fTotalStrokes	 20
U+89BA.0	fCangjie	HBBUU
U+89BA.0	fFourCorner	77216
U+89BA.0	fHakka	  gok7; gau5
U+89BA.0	fCantonese	gok3; gaau3; gaau1 (also); gaau4 (also)
U+89BA.0	fMandarin	jue2
U+89BA.0	fEnglish	[1] wake up from sleep; [2] be conscious of; sense [3] senses [4] awaken; realize; discover [5] tell; feel [6] high; large, and straight
U+89BA.1	fUTF8	覺
U+89BA.1	fCNS11643	1-7A3C
U+89BA.1	fBig5	C4B1
U+89BA.1	fGB	1-3E75
U+89BA.1	fR/S	 147.13
U+89BA.1	fTotalStrokes	 20
U+89BA.1	fCangjie	HBBUU
U+89BA.1	fFourCorner	77216
U+89BA.1	fHakka	 gau5
U+89BA.1	fMandarin	jiao4; jiao3
U+89BA.1	fEnglish	 sleep; nap
U+89BB.0	fUTF8	覻
U+89BB.0	fCNS11643	3-5D2A
U+89BB.0	fGB	5-4A78
U+89BB.0	fR/S	 147.13
U+89BB.0	fTotalStrokes	 20
U+89BB.0	fCangjie	YTBUU
U+89BB.0	fFourCorner	26210
U+89BB.0	fCantonese	ceoi3
U+89BB.0	fMandarin	qu4
U+89BC.0	fUTF8	覼
U+89BC.0	fCNS11643	3-5E61
U+89BC.0	fGB	E-3D57
U+89BC.0	fR/S	 147.14
U+89BC.0	fTotalStrokes	 21
U+89BC.0	fCangjie	MBBUU
U+89BC.0	fFourCorner	16210
U+89BC.0	fHakka	 lo2
U+89BC.0	fCantonese	lo4
U+89BC.0	fMandarin	luo2
U+89BC.0	fEnglish	explain in detail
U+89BD.0	fUTF8	覽
U+89BD.0	fCNS11643	1-7B2A
U+89BD.0	fBig5	C4FD
U+89BD.0	fGB	1-4040
U+89BD.0	fR/S	 147.14
U+89BD.0	fTotalStrokes	 21
U+89BD.0	fCangjie	SWBUU
U+89BD.0	fFourCorner	78216
U+89BD.0	fHakka	 lam3; lam5; gam5
U+89BD.0	fCantonese	laam5
U+89BD.0	fMandarin	lan3; lan4
U+89BD.0	fEnglish	[1] look at; inspect; perceive; sightsee [2] read [3] listen to (what others say) [4] a Chinese family name
U+89BE.0	fUTF8	覾
U+89BE.0	fCNS11643	2-6E23
U+89BE.0	fBig5	F754
U+89BE.0	fGB	5-4A6B
U+89BE.0	fR/S	 147.15
U+89BE.0	fTotalStrokes	 22
U+89BE.0	fCangjie	JWBUU
U+89BE.0	fFourCorner	06610
U+89BE.0	fCantonese	sam2
U+89BE.0	fMandarin	shen3
U+89BF.0	fUTF8	覿
U+89BF.0	fCNS11643	2-6E22
U+89BF.0	fBig5	F753
U+89BF.0	fGB	1-6A6B
U+89BF.0	fR/S	 147.15
U+89BF.0	fTotalStrokes	 22
U+89BF.0	fCangjie	GCBUU
U+89BF.0	fFourCorner	46810
U+89BF.0	fHakka	  tuk8;  tit8
U+89BF.0	fCantonese	dik6
U+89BF.0	fMandarin	di2; ji2; du2; ti4
U+89BF.0	fEnglish	 see (each other); interview; meet
U+89C0.0	fUTF8	觀
U+89C0.0	fCNS11643	1-7D28
U+89C0.0	fBig5	C65B
U+89C0.0	fGB	1-395B
U+89C0.0	fR/S	 147.18
U+89C0.0	fTotalStrokes	 25
U+89C0.0	fCangjie	TGBUU
U+89C0.0	fFourCorner	46210
U+89C0.0	fHakka	  gon1;  kwan1; gwon1; gon5; gwon5
U+89C0.0	fCantonese	gun1; gun3; gwun1 (also); gwun3 (also)
U+89C0.0	fMandarin	guan1; quan4
U+89C0.0	fEnglish	[1] see; observe; view; appearance; behold; take a view of; look; inspect [2] sights; views [3] display [4] a point of view; conception
U+89C0.1	fUTF8	觀
U+89C0.1	fCNS11643	1-7D28
U+89C0.1	fBig5	C65B
U+89C0.1	fGB	1-395B
U+89C0.1	fR/S	 147.18
U+89C0.1	fTotalStrokes	 25
U+89C0.1	fCangjie	TGBUU
U+89C0.1	fFourCorner	46210
U+89C0.1	fHakka	 gwon5
U+89C0.1	fMandarin	guan4
U+89C0.1	fEnglish	a Taoist temple or shrine
U+89C1.0	fUTF8	见
U+89C1.0	fGB	0-3C7B
U+89C1.0	fR/S	 147.0
U+89C1.0	fTotalStrokes	 4
U+89C1.0	fCangjie	BHU
U+89C1.0	fHakka	 gen5; hien5; gien5
U+89C1.0	fCantonese	gin3; jin6
U+89C1.0	fMandarin	jian4
U+89C1.0	fEnglish	simplified form of &#x898B;
U+89C1.1	fUTF8	见
U+89C1.1	fGB	0-3C7B
U+89C1.1	fR/S	 147.0
U+89C1.1	fTotalStrokes	 4
U+89C1.1	fCangjie	BHU
U+89C1.1	fMandarin	xian4
U+89C1.2	fUTF8	见
U+89C1.2	fGB	0-3C7B
U+89C1.2	fR/S	 147.0
U+89C1.2	fTotalStrokes	 4
U+89C1.2	fCangjie	BHU
U+89C1.2	fMandarin	jian5
U+89C2.0	fUTF8	观
U+89C2.0	fGB	0-395B
U+89C2.0	fR/S	 147.2
U+89C2.0	fTotalStrokes	 6
U+89C2.0	fCangjie	EBHU
U+89C2.0	fHakka	 kwan1; gwon1; gon1
U+89C2.0	fCantonese	gun1; gwun1 (also); gun3; gwun3 (also)
U+89C2.0	fMandarin	guan1; quan4; guan4
U+89C2.0	fEnglish	simplified form of &#x89C0;
U+89C3.0	fUTF8	觃
U+89C3.0	fGB	8-2E4F
U+89C3.0	fR/S	 147.3
U+89C3.0	fTotalStrokes	 7
U+89C3.0	fCangjie	XBUND
U+89C3.0	fCantonese	jim3
U+89C3.0	fMandarin	yan4
U+89C3.0	fEnglish	a place name
U+89C4.0	fUTF8	规
U+89C4.0	fGB	0-3966
U+89C4.0	fR/S	 147.4
U+89C4.0	fTotalStrokes	 8
U+89C4.0	fCangjie	QOBHU
U+89C4.0	fHakka	 kwui1
U+89C4.0	fCantonese	kwai1
U+89C4.0	fMandarin	gui1; gui4; xi1
U+89C4.0	fEnglish	simplified form of &#x898F;
U+89C5.0	fUTF8	觅
U+89C5.0	fGB	0-4359
U+89C5.0	fR/S	 87.4; 147.4
U+89C5.0	fTotalStrokes	 8
U+89C5.0	fCangjie	BBHU
U+89C5.0	fHakka	 met7
U+89C5.0	fCantonese	mik6
U+89C5.0	fMandarin	mi4
U+89C5.0	fEnglish	simplified form of &#x8993;
U+89C6.0	fUTF8	视
U+89C6.0	fGB	0-4A53
U+89C6.0	fR/S	 113.4; 147.4
U+89C6.0	fTotalStrokes	 9
U+89C6.0	fCangjie	IFBHU
U+89C6.0	fHakka	 shi5
U+89C6.0	fCantonese	si6; si2 (also)
U+89C6.0	fMandarin	shi4
U+89C6.0	fEnglish	simplified form of &#x8996;
U+89C6.1	fUTF8	视
U+89C6.1	fGB	0-4A53
U+89C6.1	fR/S	 113.4; 147.4
U+89C6.1	fTotalStrokes	 9
U+89C6.1	fCangjie	IFBHU
U+89C6.1	fMandarin	shi5
U+89C7.0	fUTF8	觇
U+89C7.0	fGB	0-6A68
U+89C7.0	fR/S	 147.5
U+89C7.0	fTotalStrokes	 9
U+89C7.0	fCangjie	YRBHU
U+89C7.0	fHakka	 zham1
U+89C7.0	fCantonese	zim1
U+89C7.0	fMandarin	zhan1; chan1; chan3; chan4; dan1
U+89C7.0	fEnglish	simplified form of &#x8998;
U+89C8.0	fUTF8	览
U+89C8.0	fGB	0-4040
U+89C8.0	fR/S	 147.5
U+89C8.0	fTotalStrokes	 9
U+89C8.0	fCangjie	LIBHU
U+89C8.0	fHakka	 lam3; lam5
U+89C8.0	fCantonese	laam5
U+89C8.0	fMandarin	lan3; lan4
U+89C8.0	fEnglish	simplified form of &#x89BD;
U+89C9.0	fUTF8	觉
U+89C9.0	fGB	0-3E75
U+89C9.0	fR/S	 147.5
U+89C9.0	fTotalStrokes	 9
U+89C9.0	fCangjie	FBBHU
U+89C9.0	fHakka	 gok7; gau5
U+89C9.0	fCantonese	gok3; gaau3; gaau1 (also); gaau4 (also)
U+89C9.0	fMandarin	jue2
U+89C9.0	fEnglish	simplified form of &#x89BA;
U+89C9.1	fUTF8	觉
U+89C9.1	fGB	0-3E75
U+89C9.1	fR/S	 147.5
U+89C9.1	fTotalStrokes	 9
U+89C9.1	fCangjie	FBBHU
U+89C9.1	fMandarin	jiao4
U+89CA.0	fUTF8	觊
U+89CA.0	fGB	0-6A69
U+89CA.0	fR/S	 147.6
U+89CA.0	fTotalStrokes	 10
U+89CA.0	fCangjie	UUBHU
U+89CA.0	fHakka	 gi5; ki3; koi3; kai3
U+89CA.0	fCantonese	gei3
U+89CA.0	fMandarin	ji4; xi4
U+89CA.0	fEnglish	simplified form of &#x89AC;
U+89CB.0	fUTF8	觋
U+89CB.0	fGB	0-6A6A
U+89CB.0	fR/S	 147.7
U+89CB.0	fTotalStrokes	 11
U+89CB.0	fCangjie	MOBHU
U+89CB.0	fHakka	 s'ang5; hit7
U+89CB.0	fCantonese	hat6
U+89CB.0	fMandarin	xi2; he2
U+89CB.0	fEnglish	simplified form of &#x89A1;
U+89CC.0	fUTF8	觌
U+89CC.0	fGB	0-6A6B
U+89CC.0	fR/S	 147.8
U+89CC.0	fTotalStrokes	 12
U+89CC.0	fCangjie	JKBHU
U+89CC.0	fHakka	 tuk8; tit8
U+89CC.0	fCantonese	dik6
U+89CC.0	fMandarin	di2; du2; ji2; ti4
U+89CC.0	fEnglish	simplified form of &#x89BF;
U+89CD.0	fUTF8	觍
U+89CD.0	fGB	8-2E50
U+89CD.0	fR/S	 147.8
U+89CD.0	fTotalStrokes	 12
U+89CD.0	fCangjie	TCBHU
U+89CD.0	fCantonese	tin2
U+89CD.0	fMandarin	tian3
U+89CD.0	fEnglish	ashamed
U+89CE.0	fUTF8	觎
U+89CE.0	fGB	0-6A6C
U+89CE.0	fR/S	 147.9
U+89CE.0	fTotalStrokes	 13
U+89CE.0	fCangjie	ONBHU
U+89CE.0	fHakka	 ji2; ji3
U+89CE.0	fCantonese	jyu4
U+89CE.0	fMandarin	yu2; ju4; yu4
U+89CE.0	fEnglish	simplified form of &#x89A6;
U+89CF.0	fUTF8	觏
U+89CF.0	fGB	0-6A6D
U+89CF.0	fR/S	 147.10
U+89CF.0	fTotalStrokes	 14
U+89CF.0	fCangjie	TBBHU
U+89CF.0	fHakka	 geu5
U+89CF.0	fCantonese	gau3
U+89CF.0	fMandarin	gou4; hou2; hou4; jiang3; jue2
U+89CF.0	fEnglish	simplified form of &#x89AF;
U+89D0.0	fUTF8	觐
U+89D0.0	fGB	0-6A6E
U+89D0.0	fR/S	 147.11
U+89D0.0	fTotalStrokes	 15
U+89D0.0	fCangjie	TMBHU
U+89D0.0	fHakka	 giun3
U+89D0.0	fCantonese	gan2 (also); gan3; gan6 (also)
U+89D0.0	fMandarin	jin3; jin4
U+89D0.0	fEnglish	simplified form of &#x89B2;
U+89D1.0	fUTF8	觑
U+89D1.0	fGB	0-6A6F
U+89D1.0	fR/S	 147.11
U+89D1.0	fTotalStrokes	 15
U+89D1.0	fCangjie	YCBHU
U+89D1.0	fCantonese	ceoi3
U+89D1.0	fMandarin	qu4; qu1
U+89D1.0	fEnglish	simplified form of &#x89B7;
U+89D2.0	fUTF8	角
U+89D2.0	fCNS11643	1-4B45
U+89D2.0	fBig5	A8A4
U+89D2.0	fGB	0-3D47
U+89D2.0	fR/S	 148.0
U+89D2.0	fTotalStrokes	 7
U+89D2.0	fCangjie	NBG
U+89D2.0	fFourCorner	27227
U+89D2.0	fHakka	  gok7; luk8
U+89D2.0	fCantonese	gok3; luk6; gok2 (also)
U+89D2.0	fMandarin	jiao3; gu3
U+89D2.0	fEnglish	[1] horn of an animal [2] direction; corner [3] angle [4] dime; a tenth of a dollar; 10-cent piece [5] something in the shape of a horn [6] cape; promontory
U+89D2.1	fUTF8	角
U+89D2.1	fCNS11643	1-4B45
U+89D2.1	fBig5	A8A4
U+89D2.1	fGB	0-3D47
U+89D2.1	fR/S	 148.0
U+89D2.1	fTotalStrokes	 7
U+89D2.1	fCangjie	NBG
U+89D2.1	fFourCorner	27227
U+89D2.1	fMandarin	jue2
U+89D2.1	fEnglish	[1] compete; contest; wrestle [2] corner [3] ancient, three-legged wine-cup [4] dramatic role; character [5] one of the five musical notes in ancient Chinese music
U+89D2.2	fUTF8	角
U+89D2.2	fCNS11643	1-4B45
U+89D2.2	fBig5	A8A4
U+89D2.2	fGB	0-3D47
U+89D2.2	fR/S	 148.0
U+89D2.2	fTotalStrokes	 7
U+89D2.2	fCangjie	NBG
U+89D2.2	fFourCorner	27227
U+89D2.2	fMandarin	lu4
U+89D2.2	fEnglish	used in combination for a place name
U+89D3.0	fUTF8	觓
U+89D3.0	fCNS11643	2-2D50
U+89D3.0	fBig5	D0AF
U+89D3.0	fGB	5-707C
U+89D3.0	fR/S	 148.2
U+89D3.0	fTotalStrokes	 9
U+89D3.0	fCangjie	NBVL
U+89D3.0	fFourCorner	22200
U+89D3.0	fCantonese	kau4
U+89D3.0	fMandarin	jiu1; qiu2; jiu3
U+89D4.0	fUTF8	觔
U+89D4.0	fCNS11643	1-5352
U+89D4.0	fBig5	AD6F
U+89D4.0	fGB	8-2C7C
U+89D4.0	fR/S	 148.2
U+89D4.0	fTotalStrokes	 9
U+89D4.0	fCangjie	NBKS
U+89D4.0	fFourCorner	24227
U+89D4.0	fHakka	  gin1; kin5
U+89D4.0	fCantonese	gan1
U+89D4.0	fMandarin	jin1
U+89D4.0	fEnglish	[1] sinews; muscular strength [2] a catty (1.10231 pounds)
U+89D5.0	fUTF8	觕
U+89D5.0	fCNS11643	2-394B
U+89D5.0	fBig5	D7C8
U+89D5.0	fGB	E-3D58
U+89D5.0	fR/S	 148.4
U+89D5.0	fTotalStrokes	 11
U+89D5.0	fCangjie	HQNBG
U+89D5.0	fFourCorner	27527
U+89D5.0	fHakka	 cu1; ch'uk7; cuk7; cih1; ch'uk8
U+89D5.0	fCantonese	cou1
U+89D5.0	fMandarin	cu1; ceng4; cheng2; chu4; zu4
U+89D5.0	fEnglish	coarse; rough; rude
U+89D6.0	fUTF8	觖
U+89D6.0	fCNS11643	2-3949
U+89D6.0	fBig5	D7C6
U+89D6.0	fGB	0-757B
U+89D6.0	fR/S	 148.4
U+89D6.0	fTotalStrokes	 11
U+89D6.0	fCangjie	NBDK
U+89D6.0	fFourCorner	25230
U+89D6.0	fHakka	  giet7; get7
U+89D6.0	fCantonese	kyut3; kwai3
U+89D6.0	fMandarin	jue2; gui4; kui4; qi3
U+89D6.0	fEnglish	dissatisfied; discontented; not satisfactory
U+89D7.0	fUTF8	觗
U+89D7.0	fCNS11643	4-3539
U+89D7.0	fGB	E-3D59
U+89D7.0	fR/S	  148.4
U+89D7.0	fTotalStrokes	 11
U+89D7.0	fCangjie	NBHVP
U+89D7.0	fFourCorner	22240
U+89D7.0	fCantonese	zi3
U+89D7.0	fMandarin	zhi4
U+89D8.0	fUTF8	觘
U+89D8.0	fCNS11643	4-3538
U+89D8.0	fGB	3-7236
U+89D8.0	fR/S	  148.4
U+89D8.0	fTotalStrokes	 11
U+89D8.0	fCangjie	NBFH
U+89D8.0	fFourCorner	29220
U+89D8.0	fCantonese	caau3
U+89D8.0	fMandarin	chao4
U+89D9.0	fUTF8	觙
U+89D9.0	fCNS11643	2-394A
U+89D9.0	fBig5	D7C7
U+89D9.0	fGB	5-707E
U+89D9.0	fR/S	 148.4
U+89D9.0	fTotalStrokes	 11
U+89D9.0	fCangjie	NBNHE
U+89D9.0	fFourCorner	27247
U+89D9.0	fCantonese	kap1; gap1
U+89D9.0	fMandarin	ji2; fan2
U+89DA.0	fUTF8	觚
U+89DA.0	fCNS11643	2-403A
U+89DA.0	fBig5	DBD4
U+89DA.0	fGB	0-757D
U+89DA.0	fR/S	 148.5
U+89DA.0	fTotalStrokes	 12
U+89DA.0	fCangjie	NBHVO
U+89DA.0	fFourCorner	22230
U+89DA.0	fHakka	 gu1
U+89DA.0	fCantonese	gu1; gwu1 (also)
U+89DA.0	fMandarin	gu1
U+89DA.0	fEnglish	[1] an ancient wine vessel; winecup; beaker; goblet [2] corner; angle; angular [3] rule [4] square [5] correspondence; file
U+89DB.0	fUTF8	觛
U+89DB.0	fCNS11643	2-403B
U+89DB.0	fBig5	DBD5
U+89DB.0	fGB	5-7125
U+89DB.0	fR/S	 148.5
U+89DB.0	fTotalStrokes	 12
U+89DB.0	fCangjie	NBAM
U+89DB.0	fFourCorner	26210
U+89DB.0	fCantonese	daan6; daan3
U+89DB.0	fMandarin	dan4; dan3
U+89DC.0	fUTF8	觜
U+89DC.0	fCNS11643	2-4749
U+89DC.0	fBig5	E043
U+89DC.0	fGB	0-757E
U+89DC.0	fR/S	 148.6
U+89DC.0	fTotalStrokes	 13
U+89DC.0	fCangjie	YPNBG
U+89DC.0	fFourCorner	21227
U+89DC.0	fHakka	 zui1; zui3
U+89DC.0	fCantonese	zi1; zeoi2
U+89DC.0	fMandarin	zi1; zui3; zui1
U+89DC.0	fEnglish	beak
U+89DD.0	fUTF8	觝
U+89DD.0	fCNS11643	2-4039
U+89DD.0	fBig5	DBD3
U+89DD.0	fGB	3-7238
U+89DD.0	fR/S	 148.5
U+89DD.0	fTotalStrokes	 12
U+89DD.0	fCangjie	NBHPM
U+89DD.0	fFourCorner	22240
U+89DD.0	fHakka	  di3
U+89DD.0	fCantonese	dai2
U+89DD.0	fMandarin	di3; zhi3
U+89DD.0	fEnglish	 butt; resist; push
U+89DE.0	fUTF8	觞
U+89DE.0	fGB	0-757C
U+89DE.0	fR/S	 148.5
U+89DE.0	fTotalStrokes	 12
U+89DE.0	fCangjie	NBONH
U+89DE.0	fHakka	 chong3; shong1
U+89DE.0	fCantonese	soeng1
U+89DE.0	fMandarin	shang1
U+89DE.0	fEnglish	simplified form of &#x89F4;
U+89DF.0	fUTF8	觟
U+89DF.0	fCNS11643	2-4743
U+89DF.0	fBig5	DFFC
U+89DF.0	fGB	3-723D
U+89DF.0	fR/S	 148.6
U+89DF.0	fTotalStrokes	 13
U+89DF.0	fCangjie	NBGG
U+89DF.0	fFourCorner	24214
U+89DF.0	fHakka	 gwa5; hai5; hai6
U+89DF.0	fCantonese	waa6; haai6; haai5
U+89DF.0	fMandarin	hua4; xie4
U+89E0.0	fUTF8	觠
U+89E0.0	fCNS11643	2-4747
U+89E0.0	fBig5	E041
U+89E0.0	fGB	5-712A
U+89E0.0	fR/S	 148.6
U+89E0.0	fTotalStrokes	 13
U+89E0.0	fCangjie	FQNBG
U+89E0.0	fFourCorner	90227
U+89E0.0	fCantonese	kyun4; gyun3
U+89E0.0	fMandarin	quan2; juan3; juan4
U+89E1.0	fUTF8	觡
U+89E1.0	fCNS11643	2-4746
U+89E1.0	fBig5	E040
U+89E1.0	fGB	3-7240
U+89E1.0	fR/S	 148.6
U+89E1.0	fTotalStrokes	 13
U+89E1.0	fCangjie	NBHER
U+89E1.0	fFourCorner	27264
U+89E1.0	fHakka	  gok7; giet7; get7
U+89E1.0	fCantonese	gaak3
U+89E1.0	fMandarin	ge2
U+89E1.0	fEnglish	antlers
U+89E2.0	fUTF8	觢
U+89E2.0	fCNS11643	2-4748
U+89E2.0	fBig5	E042
U+89E2.0	fGB	3-723C
U+89E2.0	fR/S	 148.6
U+89E2.0	fTotalStrokes	 13
U+89E2.0	fCangjie	QHNBG
U+89E2.0	fFourCorner	57227
U+89E2.0	fCantonese	sai6
U+89E2.0	fMandarin	chi4; zhi4; shi4
U+89E3.0	fUTF8	解
U+89E3.0	fCNS11643	1-6658
U+89E3.0	fBig5	B8D1
U+89E3.0	fGB	0-3D62
U+89E3.0	fR/S	 148.6
U+89E3.0	fTotalStrokes	 13
U+89E3.0	fCangjie	NBSHQ
U+89E3.0	fFourCorner	27252
U+89E3.0	fHakka	  gai3; gai5; kie3; kiai3; gai6; kie6; kiai5; ke3; kie5; giai3; ge3; giai5
U+89E3.0	fCantonese	gaai2; gaai3; haai6; haai5
U+89E3.0	fMandarin	jie3
U+89E3.0	fEnglish	[1] loosen; unfasten; untie; undo [2] solve (difficult problems, etc.) [3] explain; clarify; interpret; explanation; interpretation [4] understand [5] ideas; views [6] break up; separate; disperse
U+89E3.1	fUTF8	解
U+89E3.1	fCNS11643	1-6658
U+89E3.1	fBig5	B8D1
U+89E3.1	fGB	0-3D62
U+89E3.1	fR/S	 148.6
U+89E3.1	fTotalStrokes	 13
U+89E3.1	fCangjie	NBSHQ
U+89E3.1	fFourCorner	27252
U+89E3.1	fMandarin	jie4
U+89E3.1	fEnglish	 escort (prisoners, goods, etc.) from one place to another; forward; send someone in custody to dispatch
U+89E3.2	fUTF8	解
U+89E3.2	fCNS11643	1-6658
U+89E3.2	fBig5	B8D1
U+89E3.2	fGB	0-3D62
U+89E3.2	fR/S	 148.6
U+89E3.2	fTotalStrokes	 13
U+89E3.2	fCangjie	NBSHQ
U+89E3.2	fFourCorner	27252
U+89E3.2	fMandarin	xie4
U+89E3.2	fEnglish	a Chinese family name
U+89E4.0	fUTF8	觤
U+89E4.0	fCNS11643	2-4745
U+89E4.0	fBig5	DFFE
U+89E4.0	fGB	5-7129
U+89E4.0	fR/S	 148.6
U+89E4.0	fTotalStrokes	 13
U+89E4.0	fCangjie	NBNMU
U+89E4.0	fFourCorner	27212
U+89E4.0	fCantonese	gwai2
U+89E4.0	fMandarin	gui3
U+89E5.0	fUTF8	觥
U+89E5.0	fCNS11643	2-4744
U+89E5.0	fBig5	DFFD
U+89E5.0	fGB	0-7621
U+89E5.0	fR/S	 148.6
U+89E5.0	fTotalStrokes	 13
U+89E5.0	fCangjie	NBFMU
U+89E5.0	fFourCorner	29211
U+89E5.0	fHakka	 kwen1; gong1; gang1; gwen2; gwen1
U+89E5.0	fCantonese	gwang1
U+89E5.0	fMandarin	gong1
U+89E5.0	fEnglish	[1] a wine vessel made of horn [2] big; great
U+89E6.0	fUTF8	触
U+89E6.0	fCNS11643	2-474A
U+89E6.0	fBig5	E044
U+89E6.0	fGB	0-3425
U+89E6.0	fR/S	 148.6
U+89E6.0	fTotalStrokes	 13
U+89E6.0	fCangjie	NBLMI
U+89E6.0	fFourCorner	25236
U+89E6.0	fHakka	 ch'uk7
U+89E6.0	fCantonese	zuk1; cuk1 (also)
U+89E6.0	fMandarin	chu4; hong2
U+89E6.0	fEnglish	simplified form of &#x89F8;
U+89E7.0	fUTF8	觧
U+89E7.0	fCNS11643	3-4454
U+89E7.0	fGB	E-3D5A
U+89E7.0	fR/S	 148.6
U+89E7.0	fTotalStrokes	 13
U+89E7.0	fCangjie	NBTQ
U+89E7.0	fFourCorner	28250
U+89E7.0	fHakka	 gai3;  kiai3; kie3; gai5; gai6; kie6; kie5
U+89E7.0	fCantonese	gaai3; gaai2; haai6
U+89E7.0	fMandarin	jie3; jie4; xie4
U+89E7.0	fEnglish	 loosen; unfasten; untie explain
U+89E8.0	fUTF8	觨
U+89E8.0	fCNS11643	2-4E31
U+89E8.0	fBig5	E449
U+89E8.0	fGB	E-3D5B
U+89E8.0	fR/S	 148.7
U+89E8.0	fTotalStrokes	 14
U+89E8.0	fCangjie	NBME
U+89E8.0	fFourCorner	21232
U+89E8.0	fCantonese	wan6
U+89E8.0	fMandarin	hun4; huan4
U+89E9.0	fUTF8	觩
U+89E9.0	fCNS11643	2-4E2F
U+89E9.0	fBig5	E447
U+89E9.0	fGB	3-7242
U+89E9.0	fR/S	 148.7
U+89E9.0	fTotalStrokes	 14
U+89E9.0	fCangjie	NBIJE
U+89E9.0	fFourCorner	23232
U+89E9.0	fHakka	  kiu2
U+89E9.0	fCantonese	kau4
U+89E9.0	fMandarin	qiu2
U+89E9.0	fEnglish	strong
U+89EA.0	fUTF8	觪
U+89EA.0	fCNS11643	3-495C
U+89EA.0	fGB	3-7243
U+89EA.0	fR/S	  148.7
U+89EA.0	fTotalStrokes	 14
U+89EA.0	fCangjie	NBYTJ
U+89EA.0	fFourCorner	20240
U+89EA.0	fHakka	 sin1
U+89EA.0	fCantonese	sing1
U+89EA.0	fMandarin	xing1
U+89EA.0	fEnglish	lithe and strong, like a horn bow
U+89EB.0	fUTF8	觫
U+89EB.0	fCNS11643	2-4E30
U+89EB.0	fBig5	E448
U+89EB.0	fGB	0-7622
U+89EB.0	fR/S	 148.7
U+89EB.0	fTotalStrokes	 14
U+89EB.0	fCangjie	NBDL
U+89EB.0	fFourCorner	25296
U+89EB.0	fHakka	  suk7
U+89EB.0	fCantonese	cuk1; suk1
U+89EB.0	fMandarin	su4
U+89EB.0	fEnglish	 shrink and tremble in fear
U+89EC.0	fUTF8	觬
U+89EC.0	fCNS11643	2-543C
U+89EC.0	fBig5	E7D3
U+89EC.0	fGB	3-7246
U+89EC.0	fR/S	 148.8
U+89EC.0	fTotalStrokes	 15
U+89EC.0	fCangjie	NBHXU
U+89EC.0	fFourCorner	27217
U+89EC.0	fCantonese	ngai4; ai4 (also)
U+89EC.0	fMandarin	ni2; ni3
U+89ED.0	fUTF8	觭
U+89ED.0	fCNS11643	2-543A
U+89ED.0	fBig5	E7D1
U+89ED.0	fGB	3-7244
U+89ED.0	fR/S	 148.8
U+89ED.0	fTotalStrokes	 15
U+89ED.0	fCangjie	NBKMR
U+89ED.0	fFourCorner	24221
U+89ED.0	fHakka	 ki2
U+89ED.0	fCantonese	gei1; kei1 (also)
U+89ED.0	fMandarin	ji1; qi3; qi1; qi2
U+89ED.0	fEnglish	[1] one turning up and one turning down (horns of an animal) [2] obtain; get [3] odd (as contrasted to even)
U+89EE.0	fUTF8	觮
U+89EE.0	fCNS11643	3-4E66
U+89EE.0	fGB	5-7134
U+89EE.0	fR/S	  148.8
U+89EE.0	fTotalStrokes	 15
U+89EE.0	fCangjie	NBNME
U+89EE.0	fFourCorner	27230
U+89EE.0	fCantonese	luk6
U+89EE.0	fMandarin	lu4
U+89EF.0	fUTF8	觯
U+89EF.0	fGB	0-7623
U+89EF.0	fR/S	 148.8
U+89EF.0	fTotalStrokes	 15
U+89EF.0	fCangjie	NBCWJ
U+89EF.0	fHakka	 zhi5
U+89EF.0	fCantonese	zi3
U+89EF.0	fMandarin	zhi4; zhi1
U+89EF.0	fEnglish	simplified form of &#x89F6;
U+89F0.0	fUTF8	觰
U+89F0.0	fCNS11643	2-543B
U+89F0.0	fBig5	E7D2
U+89F0.0	fGB	5-712F
U+89F0.0	fR/S	 148.9
U+89F0.0	fTotalStrokes	 16
U+89F0.0	fCangjie	NBJKA
U+89F0.0	fFourCorner	24260
U+89F0.0	fHakka	 cha1; ca1; zha1; za1
U+89F0.0	fCantonese	zaa1; ce1
U+89F0.0	fMandarin	zha1; zha3; zhu1; da3; zha4
U+89F0.0	fEnglish	[1] wide, spreading horns [2] strike an ox across the horns
U+89F1.0	fUTF8	觱
U+89F1.0	fCNS11643	2-5A4B
U+89F1.0	fBig5	EB7D
U+89F1.0	fGB	3-724A
U+89F1.0	fR/S	 148.9
U+89F1.0	fTotalStrokes	 16
U+89F1.0	fCangjie	IRNBG
U+89F1.0	fFourCorner	73227
U+89F1.0	fHakka	  bit7;  bit8; bet7; {bu2}
U+89F1.0	fCantonese	bit1; bat1
U+89F1.0	fMandarin	bi4; yu4
U+89F1.0	fEnglish	[1] water emerging from ground [2] chilly wind
U+89F2.0	fUTF8	觲
U+89F2.0	fCNS11643	2-5F4A
U+89F2.0	fBig5	EE7C
U+89F2.0	fGB	5-713B
U+89F2.0	fR/S	 148.10
U+89F2.0	fTotalStrokes	 17
U+89F2.0	fCangjie	NBTGQ
U+89F2.0	fFourCorner	28255
U+89F2.0	fCantonese	sing1
U+89F2.0	fMandarin	xing1
U+89F3.0	fUTF8	觳
U+89F3.0	fCNS11643	2-5F4B
U+89F3.0	fBig5	EE7D
U+89F3.0	fGB	0-6C32
U+89F3.0	fR/S	 148.10
U+89F3.0	fTotalStrokes	 17
U+89F3.0	fCangjie	GBHNE
U+89F3.0	fFourCorner	47247
U+89F3.0	fHakka	 fuk7; guk7; kwuk7
U+89F3.0	fCantonese	huk6; kok3; gok3
U+89F3.0	fMandarin	hu2; jue2; que4
U+89F3.0	fEnglish	[1] an ancient measure for volume [2] shrink and tremble in fear
U+89F3.1	fUTF8	觳
U+89F3.1	fCNS11643	2-5F4B
U+89F3.1	fBig5	EE7D
U+89F3.1	fGB	0-6C32
U+89F3.1	fR/S	 148.10
U+89F3.1	fTotalStrokes	 17
U+89F3.1	fCangjie	GBHNE
U+89F3.1	fFourCorner	47247
U+89F3.1	fMandarin	gue4
U+89F3.1	fEnglish	[1] barren; unproductive [2] simple and crude
U+89F4.0	fUTF8	觴
U+89F4.0	fCNS11643	1-773D
U+89F4.0	fBig5	C2D2
U+89F4.0	fGB	1-757C
U+89F4.0	fR/S	 148.11
U+89F4.0	fTotalStrokes	 18
U+89F4.0	fCangjie	NBOAH
U+89F4.0	fFourCorner	28227
U+89F4.0	fHakka	 song1;  shong1; cong2; chong2; chong3
U+89F4.0	fCantonese	soeng1
U+89F4.0	fMandarin	shang1
U+89F4.0	fEnglish	[1] a general name for all sorts of wine vessels [2] offer drinks to others
U+89F5.0	fUTF8	觵
U+89F5.0	fCNS11643	3-5B32
U+89F5.0	fGB	5-713C
U+89F5.0	fR/S	  148.12
U+89F5.0	fTotalStrokes	 19
U+89F5.0	fCangjie	NBTLC
U+89F5.0	fFourCorner	24280
U+89F5.0	fHakka	 gong1; kwong1
U+89F5.0	fCantonese	gwang1
U+89F5.0	fMandarin	gong1
U+89F5.0	fEnglish	a cup made of corn obstinate
U+89F6.0	fUTF8	觶
U+89F6.0	fCNS11643	2-6722
U+89F6.0	fBig5	F2F5
U+89F6.0	fGB	1-7623
U+89F6.0	fR/S	 148.12
U+89F6.0	fTotalStrokes	 19
U+89F6.0	fCangjie	NBRRJ
U+89F6.0	fFourCorner	26256
U+89F6.0	fHakka	 zhi5; zih5; tje5
U+89F6.0	fCantonese	zi3; zi1
U+89F6.0	fMandarin	zhi4; zhi1
U+89F6.0	fEnglish	wine goblet made from horn
U+89F7.0	fUTF8	觷
U+89F7.0	fCNS11643	2-6979
U+89F7.0	fBig5	F4CD
U+89F7.0	fGB	3-723A
U+89F7.0	fR/S	 148.13
U+89F7.0	fTotalStrokes	 20
U+89F7.0	fCangjie	HBNBG
U+89F7.0	fFourCorner	77227
U+89F7.0	fCantonese	hok6
U+89F7.0	fMandarin	xue2; wu4
U+89F7.0	fEnglish	 process or carve horns
U+89F8.0	fUTF8	觸
U+89F8.0	fCNS11643	1-7A3D
U+89F8.0	fBig5	C4B2
U+89F8.0	fGB	1-3425
U+89F8.0	fR/S	 148.13
U+89F8.0	fTotalStrokes	 20
U+89F8.0	fCangjie	NBWLI
U+89F8.0	fFourCorner	26227
U+89F8.0	fHakka	 cuk7; ch'uk7; chuk7
U+89F8.0	fCantonese	zuk1; cuk1 (also)
U+89F8.0	fMandarin	chu4
U+89F8.0	fEnglish	[1] touch; contact [2] ram with the horn (animal); fram; butt [3] move or touch emotionally [4] offend; infuriate
U+89F9.0	fUTF8	觹
U+89F9.0	fCNS11643	4-6137
U+89F9.0	fGB	E-3D5C
U+89F9.0	fR/S	 148.13
U+89F9.0	fTotalStrokes	 20; 19
U+89F9.0	fCangjie	NBOGS
U+89F9.0	fFourCorner	20220
U+89F9.0	fCantonese	kwai4
U+89F9.0	fMandarin	xi1; wei2
U+89FA.0	fUTF8	觺
U+89FA.0	fCNS11643	2-6C3B
U+89FA.0	fBig5	F64C
U+89FA.0	fGB	3-7250
U+89FA.0	fR/S	 148.14
U+89FA.0	fTotalStrokes	 21
U+89FA.0	fCangjie	PONBG
U+89FA.0	fFourCorner	27227
U+89FA.0	fCantonese	ji4
U+89FA.0	fMandarin	yi2; ni4
U+89FB.0	fUTF8	觻
U+89FB.0	fCNS11643	2-6E24
U+89FB.0	fBig5	F755
U+89FB.0	fGB	3-7239
U+89FB.0	fR/S	 148.15
U+89FB.0	fTotalStrokes	 22
U+89FB.0	fCangjie	NBVID
U+89FB.0	fFourCorner	22294
U+89FB.0	fCantonese	luk6
U+89FB.0	fMandarin	lu4; li4
U+89FC.0	fUTF8	觼
U+89FC.0	fCNS11643	1-7B73
U+89FC.0	fBig5	C5A9
U+89FC.0	fGB	5-7141
U+89FC.0	fR/S	 148.15
U+89FC.0	fTotalStrokes	 22
U+89FC.0	fCangjie	NBNBE
U+89FC.0	fFourCorner	27247
U+89FC.0	fHakka	 giet7
U+89FC.0	fCantonese	kyut3
U+89FC.0	fMandarin	jue2
U+89FC.0	fEnglish	buckle
U+89FD.0	fUTF8	觽
U+89FD.0	fCNS11643	4-6945
U+89FD.0	fGB	E-3D5D
U+89FD.0	fR/S	  148.16
U+89FD.0	fTotalStrokes	 23; 21
U+89FD.0	fCangjie	NBUOS
U+89FD.0	fFourCorner	22227
U+89FD.0	fHakka	 hi1; kui2; kwui1; k'wui2
U+89FD.0	fCantonese	kwai4
U+89FD.0	fMandarin	xi1
U+89FE.0	fUTF8	觾
U+89FE.0	fCNS11643	2-6F4D
U+89FE.0	fBig5	F7FE
U+89FE.0	fGB	5-7143
U+89FE.0	fR/S	 148.16
U+89FE.0	fTotalStrokes	 23
U+89FE.0	fCangjie	NBTLF
U+89FE.0	fFourCorner	24231
U+89FE.0	fCantonese	jin3
U+89FE.0	fMandarin	yan4
U+89FF.0	fUTF8	觿
U+89FF.0	fCNS11643	2-713B
U+89FF.0	fBig5	F94C
U+89FF.0	fGB	3-7252
U+89FF.0	fR/S	 148.18
U+89FF.0	fTotalStrokes	 25
U+89FF.0	fCangjie	NBUOB
U+89FF.0	fFourCorner	22227
U+89FF.0	fCantonese	kwai4; fai1; kwai5kwai4; kwai5
U+89FF.0	fMandarin	xi1; hui1; wei2; zhui1
U+89FF.0	fEnglish	a bodkin made of ivory, horn, etc. used for undoing knots
U+8A00.0	fUTF8	言
U+8A00.0	fCNS11643	1-4B46
U+8A00.0	fBig5	A8A5
U+8A00.0	fGB	0-5154
U+8A00.0	fR/S	 149.0
U+8A00.0	fTotalStrokes	 7
U+8A00.0	fCangjie	YMMR
U+8A00.0	fFourCorner	00601
U+8A00.0	fHakka	  ngien2; nien2; gnian3; ngen2; ngian2
U+8A00.0	fCantonese	jin4
U+8A00.0	fMandarin	yan2; yan4; yin2
U+8A00.0	fEnglish	[1] words; speech [2] speak; say; talk; mean; express [3] language; dialect; tongue [4] used as a particle, adverb, etc. in various ancient expressions without meaning [5] Kangxi radical 149
U+8A01.0	fUTF8	訁
U+8A01.0	fGB	1-5A25
U+8A01.0	fR/S	 149.0
U+8A01.0	fTotalStrokes	 7
U+8A01.0	fCangjie	YMMR
U+8A01.0	fMandarin	yan2zi4pang2
U+8A01.0	fEnglish	Kangxi radical 149 (used on the left side of a character)
U+8A02.0	fUTF8	訂
U+8A02.0	fCNS11643	1-5354
U+8A02.0	fBig5	AD71
U+8A02.0	fGB	1-3629
U+8A02.0	fR/S	 149.2
U+8A02.0	fTotalStrokes	 9
U+8A02.0	fCangjie	YRMN
U+8A02.0	fFourCorner	01620
U+8A02.0	fHakka	  dang5;  den5; din5; dang1
U+8A02.0	fCantonese	ding3; deng6; ding6 (also)
U+8A02.0	fMandarin	ding4; ting1; ting2; ting3
U+8A02.0	fEnglish	[1] draw up agreement, contract; treaty, etc. [2] subscribe to (magazine, newspaper, etc.) [3] edit; collate; revise; make corrections [4] arrange; settle; fix [5] make reservations (in a hotel, restaurant, etc.)
U+8A03.0	fUTF8	訃
U+8A03.0	fCNS11643	1-5355
U+8A03.0	fBig5	AD72
U+8A03.0	fGB	1-383C
U+8A03.0	fR/S	 149.2
U+8A03.0	fTotalStrokes	 9
U+8A03.0	fCangjie	YRY
U+8A03.0	fFourCorner	03600
U+8A03.0	fHakka	  fu5; {fu1}; fu6
U+8A03.0	fCantonese	fu6
U+8A03.0	fMandarin	fu4
U+8A03.0	fEnglish	 obituary; a notice announcing the death of a person
U+8A04.0	fUTF8	訄
U+8A04.0	fCNS11643	2-2D51
U+8A04.0	fBig5	D0B0
U+8A04.0	fGB	3-7253
U+8A04.0	fR/S	 149.2
U+8A04.0	fTotalStrokes	 9
U+8A04.0	fCangjie	KNYMR
U+8A04.0	fFourCorner	40016
U+8A04.0	fHakka	  kiu2;  kau1; kiu1
U+8A04.0	fCantonese	kau4
U+8A04.0	fMandarin	qiu2; kao1; nao2; qiu1
U+8A04.0	fEnglish	 harass
U+8A05.0	fUTF8	訅
U+8A05.0	fCNS11643	3-2F2A
U+8A05.0	fGB	5-7151
U+8A05.0	fR/S	  149.2
U+8A05.0	fTotalStrokes	 9
U+8A05.0	fCangjie	YRKN
U+8A05.0	fFourCorner	04610
U+8A05.0	fCantonese	kau4
U+8A05.0	fMandarin	qiu2
U+8A06.0	fUTF8	訆
U+8A06.0	fCNS11643	3-2F2C
U+8A06.0	fGB	5-7153
U+8A06.0	fR/S	  149.2
U+8A06.0	fTotalStrokes	 9
U+8A06.0	fCangjie	YRVL
U+8A06.0	fFourCorner	02600
U+8A06.0	fHakka	  giau5; kieu5
U+8A06.0	fCantonese	giu3
U+8A06.0	fMandarin	jiao4
U+8A06.0	fEnglish	[1] call in a loud voice; yell; scream [2] tell a falsehood; talk wildly
U+8A07.0	fUTF8	訇
U+8A07.0	fCNS11643	2-2D52
U+8A07.0	fBig5	D0B1
U+8A07.0	fGB	0-596A
U+8A07.0	fR/S	 149.2
U+8A07.0	fTotalStrokes	 9
U+8A07.0	fCangjie	PYMR
U+8A07.0	fFourCorner	27620
U+8A07.0	fHakka	 wang2; vang1
U+8A07.0	fCantonese	gwang1
U+8A07.0	fMandarin	hong1; jun4; xuan2; heng1
U+8A07.0	fEnglish	[1] loudly; stentorian [2] a Chinese family name
U+8A08.0	fUTF8	計
U+8A08.0	fCNS11643	1-5353
U+8A08.0	fBig5	AD70
U+8A08.0	fGB	1-3C46
U+8A08.0	fR/S	 149.2
U+8A08.0	fTotalStrokes	 9
U+8A08.0	fCangjie	YRJ
U+8A08.0	fFourCorner	04600
U+8A08.0	fHakka	  ge5;  kie5; gi5; ke2; ke5
U+8A08.0	fCantonese	gai3; gei3; gai2 (also)
U+8A08.0	fMandarin	ji4; jie2
U+8A08.0	fEnglish	[1] plan; plot; scheme; stratagem; trick; coup; trap; ruse [2] plan; program; discuss; plan [3] calculate; count [4] a mechanical measuring device [5] a Chinese family name
U+8A08.1	fUTF8	計
U+8A08.1	fCNS11643	1-5353
U+8A08.1	fBig5	AD70
U+8A08.1	fGB	1-3C46
U+8A08.1	fR/S	 149.2
U+8A08.1	fTotalStrokes	 9
U+8A08.1	fCangjie	YRJ
U+8A08.1	fFourCorner	04600
U+8A08.1	fMandarin	ji5
U+8A09.0	fUTF8	訉
U+8A09.0	fCNS11643	4-2F7C
U+8A09.0	fGB	5-7156
U+8A09.0	fR/S	  149.3
U+8A09.0	fTotalStrokes	 10
U+8A09.0	fCangjie	YRHNI
U+8A09.0	fFourCorner	07610
U+8A09.0	fCantonese	faan6
U+8A09.0	fMandarin	fan4
U+8A0A.0	fUTF8	訊
U+8A0A.0	fCNS11643	1-5839
U+8A0A.0	fBig5	B054
U+8A0A.0	fGB	1-5136
U+8A0A.0	fR/S	 149.3
U+8A0A.0	fTotalStrokes	 10
U+8A0A.0	fCangjie	YRNJ
U+8A0A.0	fFourCorner	07610
U+8A0A.0	fHakka	 sin3
U+8A0A.0	fCantonese	seon3
U+8A0A.0	fMandarin	xun4; sui4; xin4
U+8A0A.0	fEnglish	[1] inquire; ask; question [2] information; news [3] put on trial; question in court; interrogate; interrogation
U+8A0B.0	fUTF8	訋
U+8A0B.0	fCNS11643	3-337B
U+8A0B.0	fGB	3-725B
U+8A0B.0	fR/S	  149.3
U+8A0B.0	fTotalStrokes	 10
U+8A0B.0	fCangjie	YRPI
U+8A0B.0	fFourCorner	07620
U+8A0B.0	fHakka	 diau5
U+8A0B.0	fCantonese	diu3
U+8A0B.0	fMandarin	diao4
U+8A0B.0	fEnglish	a Chinese family name
U+8A0C.0	fUTF8	訌
U+8A0C.0	fCNS11643	1-5837
U+8A0C.0	fBig5	B052
U+8A0C.0	fGB	1-5A27
U+8A0C.0	fR/S	 149.3
U+8A0C.0	fTotalStrokes	 10
U+8A0C.0	fCangjie	YRM
U+8A0C.0	fFourCorner	01610
U+8A0C.0	fHakka	 fung2;  gung5; fung1
U+8A0C.0	fCantonese	hung3 (also); hung6; hung4
U+8A0C.0	fMandarin	hong2; hong4; gong1
U+8A0C.0	fEnglish	confusion; strife; quarrel; discord
U+8A0D.0	fUTF8	訍
U+8A0D.0	fCNS11643	3-337A
U+8A0D.0	fGB	3-725F
U+8A0D.0	fR/S	  149.3
U+8A0D.0	fTotalStrokes	 10
U+8A0D.0	fCangjie	YREI
U+8A0D.0	fFourCorner	07640
U+8A0D.0	fCantonese	caai3
U+8A0D.0	fMandarin	cha4
U+8A0E.0	fUTF8	討
U+8A0E.0	fCNS11643	1-5836
U+8A0E.0	fBig5	B051
U+8A0E.0	fGB	1-4C56
U+8A0E.0	fR/S	 149.3
U+8A0E.0	fTotalStrokes	 10
U+8A0E.0	fCangjie	YRDI
U+8A0E.0	fFourCorner	04600
U+8A0E.0	fHakka	  tau3;  to3; tau5
U+8A0E.0	fCantonese	tou2
U+8A0E.0	fMandarin	tao3
U+8A0E.0	fEnglish	[1] quell (an uprising, rebellion); punish; suppress; put down; put down [2] denounce; condemn [3] demand; ask for; beg; get [4] marry (a wife or concubine) [5] study; examine into; research [6] govern; administer
U+8A0F.0	fUTF8	訏
U+8A0F.0	fCNS11643	1-583D
U+8A0F.0	fBig5	B058
U+8A0F.0	fGB	3-7259
U+8A0F.0	fR/S	 149.3
U+8A0F.0	fTotalStrokes	 10
U+8A0F.0	fCangjie	YRMD
U+8A0F.0	fFourCorner	01640
U+8A0F.0	fHakka	  hi1; fu1
U+8A0F.0	fCantonese	heoi1; heoi2
U+8A0F.0	fMandarin	xu1; xu3; hu1
U+8A0F.0	fEnglish	[1] boasts; braggings [2] big; great [3] sigh
U+8A10.0	fUTF8	訐
U+8A10.0	fCNS11643	1-5835
U+8A10.0	fBig5	B050
U+8A10.0	fGB	1-5A26
U+8A10.0	fR/S	 149.3
U+8A10.0	fTotalStrokes	 10
U+8A10.0	fCangjie	YRMJ
U+8A10.0	fFourCorner	01640
U+8A10.0	fHakka	  giet7; get7
U+8A10.0	fCantonese	kit3; gai3; git3
U+8A10.0	fMandarin	jie2; ji4
U+8A10.0	fEnglish	[1] expose other's secrets; pry into another's secret [2] accuse; charge; defame; malign (a person)
U+8A11.0	fUTF8	訑
U+8A11.0	fCNS11643	1-583E
U+8A11.0	fBig5	B059
U+8A11.0	fGB	3-725C
U+8A11.0	fR/S	 149.3
U+8A11.0	fTotalStrokes	 10
U+8A11.0	fCangjie	YRPD
U+8A11.0	fFourCorner	04612
U+8A11.0	fHakka	 ji2; ri2; i2
U+8A11.0	fCantonese	ji4; daan3; to4
U+8A11.0	fMandarin	yi2; dan4; dian4; shi1; tuo1; tuo3
U+8A11.0	fEnglish	overbearing; assuming; self-satisfaction
U+8A11.1	fUTF8	訑
U+8A11.1	fCNS11643	1-583E
U+8A11.1	fBig5	B059
U+8A11.1	fGB	3-725C
U+8A11.1	fR/S	 149.3
U+8A11.1	fTotalStrokes	 10
U+8A11.1	fCangjie	YRPD
U+8A11.1	fFourCorner	04612
U+8A11.1	fMandarin	tuo2
U+8A11.1	fEnglish	 deceive; cheat
U+8A12.0	fUTF8	訒
U+8A12.0	fCNS11643	2-3321
U+8A12.0	fBig5	D3DD
U+8A12.0	fGB	3-725E
U+8A12.0	fR/S	 149.3
U+8A12.0	fTotalStrokes	 10
U+8A12.0	fCangjie	YRSHI
U+8A12.0	fFourCorner	07620
U+8A12.0	fHakka	 ngiun3; ngin5; ngin6; niun5
U+8A12.0	fCantonese	jan6
U+8A12.0	fMandarin	ren4; ren3
U+8A12.0	fEnglish	difficult to speak out; hesitate say; be reluctant; cautious in speak
U+8A13.0	fUTF8	訓
U+8A13.0	fCNS11643	1-583B
U+8A13.0	fBig5	B056
U+8A13.0	fGB	1-5135
U+8A13.0	fR/S	 149.3
U+8A13.0	fTotalStrokes	 10
U+8A13.0	fCangjie	YRLLL
U+8A13.0	fFourCorner	02600
U+8A13.0	fHakka	  hiun5
U+8A13.0	fCantonese	fan3
U+8A13.0	fMandarin	xun4; xun2
U+8A13.0	fEnglish	[1] lecture; teach; instruct; exhort; admonish; lecture; scolding [2] lesson (to serve as); model [3] an old proverb, motto, etc.
U+8A13.1	fUTF8	訓
U+8A13.1	fCNS11643	1-583B
U+8A13.1	fBig5	B056
U+8A13.1	fGB	1-5135
U+8A13.1	fR/S	 149.3
U+8A13.1	fTotalStrokes	 10
U+8A13.1	fCangjie	YRLLL
U+8A13.1	fFourCorner	02600
U+8A13.1	fMandarin	xun5
U+8A14.0	fUTF8	訔
U+8A14.0	fCNS11643	4-2F7D
U+8A14.0	fGB	3-387B
U+8A14.0	fR/S	  149.3
U+8A14.0	fTotalStrokes	 10
U+8A14.0	fCangjie	UYMR
U+8A14.0	fFourCorner	22600
U+8A14.0	fHakka	 niun3
U+8A14.0	fCantonese	ngan4; an4 (also)
U+8A14.0	fMandarin	yin2
U+8A15.0	fUTF8	訕
U+8A15.0	fCNS11643	1-5838
U+8A15.0	fBig5	B053
U+8A15.0	fGB	1-5A28
U+8A15.0	fR/S	 149.3
U+8A15.0	fTotalStrokes	 10
U+8A15.0	fCangjie	YRU
U+8A15.0	fFourCorner	02670
U+8A15.0	fHakka	  san1;  san5; san3; sien3; sien5
U+8A15.0	fCantonese	saan3
U+8A15.0	fMandarin	shan4; shan1
U+8A15.0	fEnglish	[1] laugh at; sneer; ridicule [2] slander; abuse [3] embarrassed; awkward shame-faced
U+8A16.0	fUTF8	訖
U+8A16.0	fCNS11643	1-583C
U+8A16.0	fBig5	B057
U+8A16.0	fGB	1-467D
U+8A16.0	fR/S	 149.3
U+8A16.0	fTotalStrokes	 10
U+8A16.0	fCangjie	YRON
U+8A16.0	fFourCorner	08617
U+8A16.0	fHakka	  ngiet7; tet7; nget7; hit7
U+8A16.0	fCantonese	ngat6; gat1 (also); at6 (also)
U+8A16.0	fMandarin	qi4; ji4; xi4
U+8A16.0	fEnglish	[1] finish; conclude; stop; come to and end; cleared; settled [2] until; up to
U+8A17.0	fUTF8	託
U+8A17.0	fCNS11643	1-583A
U+8A17.0	fBig5	B055
U+8A17.0	fGB	E-3D5E
U+8A17.0	fR/S	 149.3
U+8A17.0	fTotalStrokes	 10
U+8A17.0	fCangjie	YRHP
U+8A17.0	fFourCorner	02614
U+8A17.0	fHakka	  tok7
U+8A17.0	fCantonese	tok3
U+8A17.0	fMandarin	tuo1; zha4
U+8A17.0	fEnglish	[1] entrust to; rely on; commission; charge with [2] ask; request [3] consign; consignment [4] use as an excuse or pretext [5] appear (in a dream, said of a dead person, etc.) [6] send (messages, etc.) indirectly
U+8A18.0	fUTF8	記
U+8A18.0	fCNS11643	1-5834
U+8A18.0	fBig5	B04F
U+8A18.0	fGB	1-3C47
U+8A18.0	fR/S	 149.3
U+8A18.0	fTotalStrokes	 10
U+8A18.0	fCangjie	YRSU
U+8A18.0	fFourCorner	07617
U+8A18.0	fHakka	  gi5
U+8A18.0	fCantonese	gei3
U+8A18.0	fMandarin	ji4
U+8A18.0	fEnglish	[1] keep in mind; remember; call to mind; bear in mind [2] record; register [3] a book recording persons, anecdotes, etc. [4] seals; chops; sign; mark
U+8A18.1	fUTF8	記
U+8A18.1	fCNS11643	1-5834
U+8A18.1	fBig5	B04F
U+8A18.1	fGB	1-3C47
U+8A18.1	fR/S	 149.3
U+8A18.1	fTotalStrokes	 10
U+8A18.1	fCangjie	YRSU
U+8A18.1	fFourCorner	07617
U+8A18.1	fMandarin	ji5
U+8A19.0	fUTF8	訙
U+8A19.0	fCNS11643	3-337C
U+8A19.0	fGB	E-3D5F
U+8A19.0	fR/S	 149.3
U+8A19.0	fTotalStrokes	 10
U+8A19.0	fCangjie	YRKNI
U+8A19.0	fFourCorner	05610
U+8A19.0	fCantonese	seon3
U+8A19.0	fMandarin	xun4
U+8A1A.0	fUTF8	訚
U+8A1A.0	fGB	8-7B2E
U+8A1A.0	fR/S	 149.3
U+8A1A.0	fTotalStrokes	 10
U+8A1A.0	fCangjie	LSYMR
U+8A1A.0	fCantonese	ngan4; an4 (also)
U+8A1A.0	fMandarin	yin2
U+8A1B.0	fUTF8	訛
U+8A1B.0	fCNS11643	1-5D45
U+8A1B.0	fBig5	B35F
U+8A1B.0	fGB	1-366F
U+8A1B.0	fR/S	 149.4
U+8A1B.0	fTotalStrokes	 11
U+8A1B.0	fCangjie	YROP
U+8A1B.0	fFourCorner	04610
U+8A1B.0	fHakka	  ngo2; ngo3
U+8A1B.0	fCantonese	ngo4; o4
U+8A1B.0	fMandarin	e2
U+8A1B.0	fEnglish	[1] rumors [2] errors; erroneous; wrong [3] extort; swindle; deceive; bluff [4] move about [5] change; transform [6] a Chinese family name
U+8A1C.0	fUTF8	訜
U+8A1C.0	fCNS11643	3-387E
U+8A1C.0	fGB	5-7164
U+8A1C.0	fR/S	 149.4
U+8A1C.0	fTotalStrokes	 11
U+8A1C.0	fCangjie	YRCSH
U+8A1C.0	fFourCorner	08620
U+8A1C.0	fCantonese	fan1
U+8A1C.0	fMandarin	fen1
U+8A1D.0	fUTF8	訝
U+8A1D.0	fCNS11643	1-5D3F
U+8A1D.0	fBig5	B359
U+8A1D.0	fGB	1-5148
U+8A1D.0	fR/S	 149.4
U+8A1D.0	fTotalStrokes	 11
U+8A1D.0	fCangjie	YRMVH
U+8A1D.0	fFourCorner	01640
U+8A1D.0	fHakka	  nga3; nga1
U+8A1D.0	fCantonese	ngaa6; aa6 (also)
U+8A1D.0	fMandarin	ya4
U+8A1D.0	fEnglish	[1] express surprise; be surprised; wonder [2] welcome; receive
U+8A1E.0	fUTF8	訞
U+8A1E.0	fCNS11643	2-394F
U+8A1E.0	fBig5	D7CC
U+8A1E.0	fGB	3-7266
U+8A1E.0	fR/S	 149.4
U+8A1E.0	fTotalStrokes	 11
U+8A1E.0	fCangjie	YRHK
U+8A1E.0	fFourCorner	02634
U+8A1E.0	fCantonese	jiu1
U+8A1E.0	fMandarin	yao1
U+8A1F.0	fUTF8	訟
U+8A1F.0	fCNS11643	1-5D44
U+8A1F.0	fBig5	B35E
U+8A1F.0	fGB	1-4B4F
U+8A1F.0	fR/S	 149.4
U+8A1F.0	fTotalStrokes	 11
U+8A1F.0	fCangjie	YRCI
U+8A1F.0	fFourCorner	08632
U+8A1F.0	fHakka	 siung5; ciung2; siung3
U+8A1F.0	fCantonese	zung6
U+8A1F.0	fMandarin	song4; song1; rong2
U+8A1F.0	fEnglish	[1] lawsuit; litigation; bring a dispute to court [2] argue over the right and wrong of something; dispute; demand justice [3] publicly; in public [4] accuse; regret; repent; blame
U+8A20.0	fUTF8	訠
U+8A20.0	fCNS11643	3-3923
U+8A20.0	fGB	5-7169
U+8A20.0	fR/S	  149.4
U+8A20.0	fTotalStrokes	 11
U+8A20.0	fCangjie	YRNL
U+8A20.0	fFourCorner	02600
U+8A20.0	fHakka	 jin3; rin3
U+8A20.0	fCantonese	can2
U+8A20.0	fMandarin	shen3
U+8A20.0	fEnglish	 much more; still more the gums
U+8A21.0	fUTF8	訡
U+8A21.0	fCNS11643	3-3929
U+8A21.0	fGB	E-3D60
U+8A21.0	fR/S	 149.4
U+8A21.0	fTotalStrokes	 11
U+8A21.0	fCangjie	YROIN
U+8A21.0	fFourCorner	08620
U+8A21.0	fHakka	 jim2; ngim2
U+8A21.0	fCantonese	jam4
U+8A21.0	fMandarin	yin2; yin3; jin4
U+8A21.0	fEnglish	 chant; moan; sigh
U+8A22.0	fUTF8	訢
U+8A22.0	fCNS11643	1-5D46
U+8A22.0	fBig5	B360
U+8A22.0	fGB	3-7267
U+8A22.0	fR/S	 149.4
U+8A22.0	fTotalStrokes	 11
U+8A22.0	fCangjie	YRHML
U+8A22.0	fFourCorner	02621
U+8A22.0	fHakka	  hiun1
U+8A22.0	fCantonese	jan1; hei1; ngan4; an4 (also)
U+8A22.0	fMandarin	xin1; xi1; yin2
U+8A22.0	fEnglish	pleased; delighted; happy; joy
U+8A23.0	fUTF8	訣
U+8A23.0	fCNS11643	1-5D40
U+8A23.0	fBig5	B35A
U+8A23.0	fGB	1-3E77
U+8A23.0	fR/S	 149.4
U+8A23.0	fTotalStrokes	 11
U+8A23.0	fCangjie	YRDK
U+8A23.0	fFourCorner	05630
U+8A23.0	fHakka	 giet7; get7; kiat7; ket7
U+8A23.0	fCantonese	kyut3
U+8A23.0	fMandarin	jue2; gui4
U+8A23.0	fEnglish	[1] part; separate [2] sorcery; an esoteric occult art [3] a way of formula of doing something; the secret or knack of doing things
U+8A24.0	fUTF8	訤
U+8A24.0	fCNS11643	6-4470
U+8A24.0	fGB	E-3D61
U+8A24.0	fR/S	 149.4
U+8A24.0	fTotalStrokes	 11
U+8A24.0	fCangjie	YRKK
U+8A24.0	fFourCorner	04640
U+8A24.0	fCantonese	haau4
U+8A24.0	fMandarin	nao2; xiao2; na2
U+8A25.0	fUTF8	訥
U+8A25.0	fCNS11643	1-5D41
U+8A25.0	fBig5	B35B
U+8A25.0	fGB	1-5A2B
U+8A25.0	fR/S	 149.4
U+8A25.0	fTotalStrokes	 11
U+8A25.0	fCangjie	YROB
U+8A25.0	fFourCorner	04627
U+8A25.0	fHakka	  nut8; lut8; lap8
U+8A25.0	fCantonese	neot6; naap6 (also); nat6 (also); leot6 (also); laap6 (also); lat6 (also); leo6
U+8A25.0	fMandarin	ne4; na4
U+8A25.0	fEnglish	slow of speech; stammer
U+8A26.0	fUTF8	訦
U+8A26.0	fCNS11643	3-3879
U+8A26.0	fGB	5-7167
U+8A26.0	fR/S	  149.4
U+8A26.0	fTotalStrokes	 11
U+8A26.0	fCangjie	YRLBU
U+8A26.0	fFourCorner	04610
U+8A26.0	fHakka	 shim1
U+8A26.0	fCantonese	sam4
U+8A26.0	fMandarin	chen2; shen2
U+8A26.0	fEnglish	 sincere faithful
U+8A27.0	fUTF8	訧
U+8A27.0	fCNS11643	2-394D
U+8A27.0	fBig5	D7CA
U+8A27.0	fGB	5-715C
U+8A27.0	fR/S	 149.4
U+8A27.0	fTotalStrokes	 11
U+8A27.0	fCangjie	YRIKU
U+8A27.0	fFourCorner	03614
U+8A27.0	fHakka	 ju2; riu2
U+8A27.0	fCantonese	jau4
U+8A27.0	fMandarin	you2
U+8A27.0	fEnglish	fault
U+8A28.0	fUTF8	訨
U+8A28.0	fCNS11643	3-387B
U+8A28.0	fGB	3-7263
U+8A28.0	fR/S	  149.4
U+8A28.0	fTotalStrokes	 11
U+8A28.0	fCangjie	YRYLM
U+8A28.0	fFourCorner	01610
U+8A28.0	fHakka	 zhi3
U+8A28.0	fCantonese	zi2
U+8A28.0	fMandarin	zhi3
U+8A29.0	fUTF8	訩
U+8A29.0	fCNS11643	3-3925
U+8A29.0	fGB	3-726A
U+8A29.0	fR/S	  149.4
U+8A29.0	fTotalStrokes	 11
U+8A29.0	fCangjie	YRUK
U+8A29.0	fFourCorner	02670
U+8A29.0	fHakka	  hiung1
U+8A29.0	fCantonese	hung1
U+8A29.0	fMandarin	xiong1
U+8A29.0	fEnglish	[1] noisy; loudly arguing (said of a crowd) [2] litigation; be engaged in a lawsuit [3] disorders; miseries
U+8A2A.0	fUTF8	訪
U+8A2A.0	fCNS11643	1-5D3E
U+8A2A.0	fBig5	B358
U+8A2A.0	fGB	1-3743
U+8A2A.0	fR/S	 149.4
U+8A2A.0	fTotalStrokes	 11
U+8A2A.0	fCangjie	YRYHS
U+8A2A.0	fFourCorner	00627
U+8A2A.0	fHakka	  fong3; fong5
U+8A2A.0	fCantonese	fong2
U+8A2A.0	fMandarin	fang3; fang4
U+8A2A.0	fEnglish	[1] visit; call on [2] ask; inquire; look for; find out [3] a Chinese family name
U+8A2B.0	fUTF8	訫
U+8A2B.0	fCNS11643	3-3922
U+8A2B.0	fGB	5-7168
U+8A2B.0	fR/S	 149.4
U+8A2B.0	fTotalStrokes	 11
U+8A2B.0	fCangjie	YRP
U+8A2B.0	fFourCorner	03600
U+8A2B.0	fCantonese	seon3
U+8A2B.0	fMandarin	xin4
U+8A2C.0	fUTF8	訬
U+8A2C.0	fCNS11643	2-394E
U+8A2C.0	fBig5	D7CB
U+8A2C.0	fGB	3-7264
U+8A2C.0	fR/S	 149.4
U+8A2C.0	fTotalStrokes	 11
U+8A2C.0	fCangjie	YRFH
U+8A2C.0	fFourCorner	09620
U+8A2C.0	fHakka	 cau2;  cau1; cau5
U+8A2C.0	fCantonese	caau2; miu5; caau1
U+8A2C.0	fMandarin	chao1; miao3; chao3; chao4; qiao4
U+8A2C.0	fEnglish	clamour;uproar; annoyance; etc.
U+8A2D.0	fUTF8	設
U+8A2D.0	fCNS11643	1-5D43
U+8A2D.0	fBig5	B35D
U+8A2D.0	fGB	1-4968
U+8A2D.0	fR/S	 149.4
U+8A2D.0	fTotalStrokes	 11
U+8A2D.0	fCangjie	YRHNE
U+8A2D.0	fFourCorner	07647
U+8A2D.0	fHakka	 set7; sat7; siet7; s'at7; shat7; s'et7
U+8A2D.0	fCantonese	cit3
U+8A2D.0	fMandarin	she4
U+8A2D.0	fEnglish	[1] lay out; display [2] establish; set up; found [3] furnish; provide [4] arrange; plan; devise [5] particle of hypothesis; supposing that; what if; in case of
U+8A2E.0	fUTF8	訮
U+8A2E.0	fCNS11643	4-353D
U+8A2E.0	fGB	3-7260
U+8A2E.0	fR/S	  149.4
U+8A2E.0	fTotalStrokes	 11
U+8A2E.0	fCangjie	YRMT
U+8A2E.0	fFourCorner	01640
U+8A2E.0	fCantonese	ngaan4; aan4 (also)
U+8A2E.0	fMandarin	xian1
U+8A2F.0	fUTF8	訯
U+8A2F.0	fCNS11643	5-355B
U+8A2F.0	fGB	E-3D62
U+8A2F.0	fR/S	  149.4
U+8A2F.0	fTotalStrokes	 11
U+8A2F.0	fCangjie	YRNHE
U+8A2F.0	fFourCorner	07640
U+8A2F.0	fCantonese	saa2
U+8A2F.0	fMandarin	sha3; sa3; sa4
U+8A30.0	fUTF8	訰
U+8A30.0	fCNS11643	2-394C
U+8A30.0	fBig5	D7C9
U+8A30.0	fGB	5-715D
U+8A30.0	fR/S	 149.4
U+8A30.0	fTotalStrokes	 11
U+8A30.0	fCangjie	YRPU
U+8A30.0	fFourCorner	05617
U+8A30.0	fHakka	 zhun1
U+8A30.0	fCantonese	zeon3; zeon1
U+8A30.0	fMandarin	tun2; zhun1; zhun4
U+8A31.0	fUTF8	許
U+8A31.0	fCNS11643	1-5D42
U+8A31.0	fBig5	B35C
U+8A31.0	fGB	1-506D
U+8A31.0	fR/S	 149.4
U+8A31.0	fTotalStrokes	 11
U+8A31.0	fCangjie	YROJ
U+8A31.0	fFourCorner	08640
U+8A31.0	fHakka	  hi3; e5; i5
U+8A31.0	fCantonese	heoi2; fu2
U+8A31.0	fMandarin	xu3; hu3
U+8A31.0	fEnglish	[1] allow; permit; promise; approve [2] praise; commend [3] be betrothed (young girl); promise to marry [4] expect [5] perhaps; maybe [6] about; a little more than (said of a persons age) [7] a place [8] a final particle [9] so; this [10] a Chinese family name
U+8A32.0	fUTF8	訲
U+8A32.0	fCNS11643	3-387A
U+8A32.0	fGB	3-7265
U+8A32.0	fR/S	  149.4
U+8A32.0	fTotalStrokes	 11
U+8A32.0	fCangjie	YRL
U+8A32.0	fFourCorner	05600
U+8A32.0	fCantonese	jik1
U+8A32.0	fMandarin	yi4
U+8A33.0	fUTF8	訳
U+8A33.0	fGB	E-3D63
U+8A33.0	fR/S	 149.4
U+8A33.0	fTotalStrokes	 11
U+8A33.0	fCangjie	YRSO
U+8A33.0	fCantonese	jik6
U+8A33.0	fMandarin	yi4
U+8A33.0	fEnglish	 translate decode encode
U+8A34.0	fUTF8	訴
U+8A34.0	fCNS11643	1-622B
U+8A34.0	fBig5	B644
U+8A34.0	fGB	1-4B5F
U+8A34.0	fR/S	 149.5
U+8A34.0	fTotalStrokes	 12
U+8A34.0	fCangjie	YRHMY
U+8A34.0	fFourCorner	02631
U+8A34.0	fHakka	 su5; sii5; se5; shu5
U+8A34.0	fCantonese	sou3
U+8A34.0	fMandarin	su4; chi3
U+8A34.0	fEnglish	[1] accuse; sue; file a complaint; charge [2] inform; narrate; tell [3] appeal; petition [4] resort to
U+8A34.1	fUTF8	訴
U+8A34.1	fCNS11643	1-622B
U+8A34.1	fBig5	B644
U+8A34.1	fGB	1-4B5F
U+8A34.1	fR/S	 149.5
U+8A34.1	fTotalStrokes	 12
U+8A34.1	fCangjie	YRHMY
U+8A34.1	fFourCorner	02631
U+8A35.0	fUTF8	訵
U+8A35.0	fCNS11643	3-3E7C
U+8A35.0	fGB	5-7172
U+8A35.0	fR/S	  149.5
U+8A35.0	fTotalStrokes	 12
U+8A35.0	fCangjie	YRWC
U+8A35.0	fFourCorner	06600
U+8A35.0	fCantonese	ci1
U+8A35.0	fMandarin	chi1
U+8A36.0	fUTF8	訶
U+8A36.0	fCNS11643	1-622D
U+8A36.0	fBig5	B646
U+8A36.0	fGB	1-5A2D
U+8A36.0	fR/S	 149.5
U+8A36.0	fTotalStrokes	 12
U+8A36.0	fCangjie	YRMNR
U+8A36.0	fFourCorner	01620
U+8A36.0	fHakka	  ko1; o1
U+8A36.0	fCantonese	ho1
U+8A36.0	fMandarin	he1
U+8A36.0	fEnglish	scold loudly; curse; blame in a loud voice
U+8A37.0	fUTF8	訷
U+8A37.0	fCNS11643	4-3B2F
U+8A37.0	fGB	5-716E
U+8A37.0	fR/S	  149.5
U+8A37.0	fTotalStrokes	 12
U+8A37.0	fCangjie	YRLWL
U+8A37.0	fFourCorner	05600
U+8A37.0	fCantonese	san1
U+8A37.0	fMandarin	shen1
U+8A38.0	fUTF8	訸
U+8A38.0	fCNS11643	3-3E7D
U+8A38.0	fGB	3-7275
U+8A38.0	fR/S	  149.5
U+8A38.0	fTotalStrokes	 12
U+8A38.0	fCangjie	YRHD
U+8A38.0	fFourCorner	02690
U+8A38.0	fCantonese	wo4; wo6
U+8A38.0	fMandarin	he2
U+8A39.0	fUTF8	訹
U+8A39.0	fCNS11643	2-403E
U+8A39.0	fBig5	DBD8
U+8A39.0	fGB	3-7270
U+8A39.0	fR/S	 149.5
U+8A39.0	fTotalStrokes	 12
U+8A39.0	fCangjie	YRIJC
U+8A39.0	fFourCorner	03694
U+8A39.0	fHakka	 tut7; shut7; sut7
U+8A39.0	fCantonese	seot1; zeot1; seot6
U+8A39.0	fMandarin	xu4; xiu3
U+8A39.0	fEnglish	 beguile with false stories
U+8A3A.0	fUTF8	診
U+8A3A.0	fCNS11643	1-622C
U+8A3A.0	fBig5	B645
U+8A3A.0	fGB	1-556F
U+8A3A.0	fR/S	 149.5
U+8A3A.0	fTotalStrokes	 12
U+8A3A.0	fCangjie	YROHH
U+8A3A.0	fFourCorner	08622
U+8A3A.0	fHakka	 zhin3; zihn3; zin3; zin1
U+8A3A.0	fCantonese	can2; zan2 (also)
U+8A3A.0	fMandarin	zhen3; zhen1; zhen4
U+8A3A.0	fEnglish	[1] examine (diseases, ailments, etc.); diagnose [2] tell; report
U+8A3B.0	fUTF8	註
U+8A3B.0	fCNS11643	1-6221
U+8A3B.0	fBig5	B5F9
U+8A3B.0	fGB	E-3D64
U+8A3B.0	fR/S	 149.5
U+8A3B.0	fTotalStrokes	 12
U+8A3B.0	fCangjie	YRYG
U+8A3B.0	fFourCorner	00614
U+8A3B.0	fHakka	 zu5; zhu5
U+8A3B.0	fCantonese	zyu3
U+8A3B.0	fMandarin	zhu4
U+8A3B.0	fEnglish	[1] explanatory note; footnote; commentary; remark; annotations [2] register; record; list
U+8A3C.0	fUTF8	証
U+8A3C.0	fCNS11643	1-6225
U+8A3C.0	fBig5	B5FD
U+8A3C.0	fGB	E-3D65
U+8A3C.0	fR/S	 149.5
U+8A3C.0	fTotalStrokes	 12
U+8A3C.0	fCangjie	YRMYM
U+8A3C.0	fFourCorner	01611
U+8A3C.0	fHakka	  zhin5;  zin5; zihn5
U+8A3C.0	fCantonese	zing3
U+8A3C.0	fMandarin	zheng4; zheng1
U+8A3C.0	fEnglish	 certificate; evidence; proof; certify
U+8A3D.0	fUTF8	訽
U+8A3D.0	fCNS11643	3-3F24
U+8A3D.0	fGB	5-7178
U+8A3D.0	fR/S	 149.5
U+8A3D.0	fTotalStrokes	 12
U+8A3D.0	fCangjie	YRPR
U+8A3D.0	fFourCorner	07620
U+8A3D.0	fHakka	 kieu5; geu5
U+8A3D.0	fCantonese	gau3
U+8A3D.0	fMandarin	gou4
U+8A3D.0	fEnglish	blame
U+8A3E.0	fUTF8	訾
U+8A3E.0	fCNS11643	1-666B
U+8A3E.0	fBig5	B8E4
U+8A3E.0	fGB	0-7624
U+8A3E.0	fR/S	 149.5
U+8A3E.0	fTotalStrokes	 12
U+8A3E.0	fCangjie	YPYMR
U+8A3E.0	fFourCorner	21601
U+8A3E.0	fHakka	 zih3; zu3; zii3; zih1
U+8A3E.0	fCantonese	zi2; zi1; ci1; ci4 (also)
U+8A3E.0	fMandarin	zi3; zi1; ci1
U+8A3E.0	fEnglish	[1] slander; traduce; attack [2] faults; blemishes [3] measure [4] consider [5] estimate [6] poor (food); meager
U+8A3F.0	fUTF8	訿
U+8A3F.0	fCNS11643	2-474F
U+8A3F.0	fBig5	E049
U+8A3F.0	fGB	3-7326
U+8A3F.0	fR/S	 149.5
U+8A3F.0	fTotalStrokes	 12
U+8A3F.0	fCangjie	YRYMP
U+8A3F.0	fFourCorner	01610
U+8A3F.0	fHakka	 zii3
U+8A3F.0	fCantonese	zi2
U+8A3F.0	fMandarin	zi3; zi1
U+8A3F.0	fEnglish	 defame; slander
U+8A40.0	fUTF8	詀
U+8A40.0	fCNS11643	2-4040
U+8A40.0	fBig5	DBDA
U+8A40.0	fGB	3-7271
U+8A40.0	fR/S	 149.5
U+8A40.0	fTotalStrokes	 12
U+8A40.0	fCangjie	YRYR
U+8A40.0	fFourCorner	01660
U+8A40.0	fHakka	 diam1;  zham1; zam1
U+8A40.0	fCantonese	zaam1; cip3; zim1; zip3
U+8A40.0	fMandarin	zhan1; chan1; che4; dian1; tian1; tie1; zhan4
U+8A40.0	fEnglish	garrulous to whisper. to joke
U+8A41.0	fUTF8	詁
U+8A41.0	fCNS11643	1-6226
U+8A41.0	fBig5	B5FE
U+8A41.0	fGB	1-5A2C
U+8A41.0	fR/S	 149.5
U+8A41.0	fTotalStrokes	 12
U+8A41.0	fCangjie	YRJR
U+8A41.0	fFourCorner	04660
U+8A41.0	fHakka	 gu3
U+8A41.0	fCantonese	gu2; gu3 (also); gwu2 (also); gwu3 (also)
U+8A41.0	fMandarin	gu3; gu4
U+8A41.0	fEnglish	[1] explanatory notes or commentaries [2] transcribe the classics in everyday language
U+8A42.0	fUTF8	詂
U+8A42.0	fCNS11643	3-3E7E
U+8A42.0	fGB	5-7175
U+8A42.0	fR/S	  149.5
U+8A42.0	fTotalStrokes	 12
U+8A42.0	fCangjie	YRODI
U+8A42.0	fFourCorner	04600
U+8A42.0	fCantonese	fu6
U+8A42.0	fMandarin	fu4
U+8A43.0	fUTF8	詃
U+8A43.0	fCNS11643	3-3E75
U+8A43.0	fGB	5-717A
U+8A43.0	fR/S	  149.5
U+8A43.0	fTotalStrokes	 12
U+8A43.0	fCangjie	YRYVI
U+8A43.0	fFourCorner	00630
U+8A43.0	fCantonese	gyun2
U+8A43.0	fMandarin	quan3; jian3
U+8A44.0	fUTF8	詄
U+8A44.0	fCNS11643	2-4043
U+8A44.0	fBig5	DBDD
U+8A44.0	fGB	3-7274
U+8A44.0	fR/S	 149.5
U+8A44.0	fTotalStrokes	 12
U+8A44.0	fCangjie	YRHQO
U+8A44.0	fFourCorner	05630
U+8A44.0	fHakka	  diet7
U+8A44.0	fCantonese	dit6
U+8A44.0	fMandarin	die2; shi4; yi4
U+8A44.0	fEnglish	[1] forget [2] make a mistake
U+8A45.0	fUTF8	詅
U+8A45.0	fCNS11643	2-4044
U+8A45.0	fBig5	DBDE
U+8A45.0	fGB	3-7276
U+8A45.0	fR/S	 149.5
U+8A45.0	fTotalStrokes	 12
U+8A45.0	fCangjie	YROII
U+8A45.0	fFourCorner	08637
U+8A45.0	fCantonese	ling4
U+8A45.0	fMandarin	ling2; ling4
U+8A45.0	fEnglish	 sell
U+8A46.0	fUTF8	詆
U+8A46.0	fCNS11643	1-622A
U+8A46.0	fBig5	B643
U+8A46.0	fGB	1-5A2E
U+8A46.0	fR/S	 149.5
U+8A46.0	fTotalStrokes	 12
U+8A46.0	fCangjie	YRHPM
U+8A46.0	fFourCorner	02640
U+8A46.0	fHakka	 di3
U+8A46.0	fCantonese	dai2
U+8A46.0	fMandarin	di3; ti4; di1; di4; ti2
U+8A46.0	fEnglish	 censure; slander; defame
U+8A47.0	fUTF8	詇
U+8A47.0	fCNS11643	4-3B30
U+8A47.0	fGB	3-7272
U+8A47.0	fR/S	  149.5
U+8A47.0	fTotalStrokes	 12
U+8A47.0	fCangjie	YRLBK
U+8A47.0	fFourCorner	05630
U+8A47.0	fCantonese	joeng3
U+8A47.0	fMandarin	yang4
U+8A48.0	fUTF8	詈
U+8A48.0	fCNS11643	2-4046
U+8A48.0	fBig5	DBE0
U+8A48.0	fGB	0-6E3A
U+8A48.0	fR/S	 149.5
U+8A48.0	fTotalStrokes	 12
U+8A48.0	fCangjie	WLYMR
U+8A48.0	fFourCorner	60601
U+8A48.0	fHakka	  li5; li6; li3
U+8A48.0	fCantonese	lei6
U+8A48.0	fMandarin	li4
U+8A48.0	fEnglish	 scold; berate; upbraid; revile; vituperate; curse
U+8A49.0	fUTF8	詉
U+8A49.0	fCNS11643	3-3F21
U+8A49.0	fGB	5-717D
U+8A49.0	fR/S	 149.5
U+8A49.0	fTotalStrokes	 12
U+8A49.0	fCangjie	YRVE
U+8A49.0	fFourCorner	07640
U+8A49.0	fHakka	  nau2; nu2
U+8A49.0	fCantonese	naau4; laau4 (also); naa4
U+8A49.0	fMandarin	nao2
U+8A49.0	fEnglish	[1] gibberish [2] wrangle
U+8A4A.0	fUTF8	詊
U+8A4A.0	fCNS11643	2-4048
U+8A4A.0	fBig5	DBE2
U+8A4A.0	fGB	3-7279
U+8A4A.0	fR/S	 149.5
U+8A4A.0	fTotalStrokes	 12
U+8A4A.0	fCangjie	YRFQ
U+8A4A.0	fFourCorner	09650
U+8A4A.0	fCantonese	pun3
U+8A4A.0	fMandarin	pan4
U+8A4B.0	fUTF8	詋
U+8A4B.0	fCNS11643	3-3F23
U+8A4B.0	fGB	5-7170
U+8A4B.0	fR/S	 149.5
U+8A4B.0	fTotalStrokes	 12
U+8A4B.0	fCangjie	YRRHU
U+8A4B.0	fFourCorner	06610
U+8A4B.0	fHakka	 zhiu5; zu5; zhu5
U+8A4B.0	fCantonese	zau3
U+8A4B.0	fMandarin	zhou4
U+8A4B.0	fEnglish	 curse swear an oath incantations
U+8A4C.0	fUTF8	詌
U+8A4C.0	fCNS11643	2-4049
U+8A4C.0	fBig5	DBE3
U+8A4C.0	fGB	5-716A
U+8A4C.0	fR/S	 149.5
U+8A4C.0	fTotalStrokes	 12
U+8A4C.0	fCangjie	YRTM
U+8A4C.0	fFourCorner	04670
U+8A4C.0	fHakka	 gam5; kiam2
U+8A4C.0	fCantonese	gam3
U+8A4C.0	fMandarin	gan4
U+8A4C.0	fEnglish	 keep one's mouth shut; restrain oneself
U+8A4D.0	fUTF8	詍
U+8A4D.0	fCNS11643	2-403D
U+8A4D.0	fBig5	DBD7
U+8A4D.0	fGB	3-726F
U+8A4D.0	fR/S	 149.5
U+8A4D.0	fTotalStrokes	 12
U+8A4D.0	fCangjie	YRPT
U+8A4D.0	fFourCorner	04617
U+8A4D.0	fHakka	 ri3; ji3; i5; ji5
U+8A4D.0	fCantonese	jai6; ji6
U+8A4D.0	fMandarin	yi4; shi4; xie4
U+8A4D.0	fEnglish	talkative; loquacious
U+8A4E.0	fUTF8	詎
U+8A4E.0	fCNS11643	2-403C
U+8A4E.0	fBig5	DBD6
U+8A4E.0	fGB	1-5A2A
U+8A4E.0	fR/S	 149.5
U+8A4E.0	fTotalStrokes	 12
U+8A4E.0	fCangjie	YRSS
U+8A4E.0	fFourCorner	01617
U+8A4E.0	fHakka	  ki3; gi5
U+8A4E.0	fCantonese	geoi6
U+8A4E.0	fMandarin	ju4
U+8A4E.0	fEnglish	interjection used to express surprise
U+8A4F.0	fUTF8	詏
U+8A4F.0	fCNS11643	2-404A
U+8A4F.0	fBig5	DBE4
U+8A4F.0	fGB	5-7222
U+8A4F.0	fR/S	 149.5
U+8A4F.0	fTotalStrokes	 12
U+8A4F.0	fCangjie	YRVIS
U+8A4F.0	fFourCorner	04627
U+8A4F.0	fHakka	 jau5; rau5; jeu5
U+8A4F.0	fCantonese	aau2 (also); aau3; ngaau2 (also); ngaau3 (also); jiu6
U+8A4F.0	fMandarin	ao4
U+8A4F.0	fEnglish	 contradict
U+8A50.0	fUTF8	詐
U+8A50.0	fCNS11643	1-6229
U+8A50.0	fBig5	B642
U+8A50.0	fGB	1-5529
U+8A50.0	fR/S	 149.5
U+8A50.0	fTotalStrokes	 12
U+8A50.0	fCangjie	YROS
U+8A50.0	fFourCorner	08611
U+8A50.0	fHakka	  za5
U+8A50.0	fCantonese	zaa3
U+8A50.0	fMandarin	zha4; zha3
U+8A50.0	fEnglish	[1] cheat; deceive; swindle; trick; lie; pretend; feign [2] deceitful; false; fake; cunning; crafty; artful; fraudulent [3] trick into; bluff somebody into giving information [4] sudden(ly); unexpected(ly)
U+8A51.0	fUTF8	詑
U+8A51.0	fCNS11643	2-4047
U+8A51.0	fBig5	DBE1
U+8A51.0	fGB	3-727B
U+8A51.0	fR/S	 149.5
U+8A51.0	fTotalStrokes	 12
U+8A51.0	fCangjie	YRJP
U+8A51.0	fFourCorner	03611
U+8A51.0	fHakka	 to2; i2
U+8A51.0	fCantonese	to4; ji4; to5
U+8A51.0	fMandarin	tuo2; tuo1; duo4; yi2
U+8A51.0	fEnglish	cheat
U+8A52.0	fUTF8	詒
U+8A52.0	fCNS11643	2-4045
U+8A52.0	fBig5	DBDF
U+8A52.0	fGB	1-5A31
U+8A52.0	fR/S	 149.5
U+8A52.0	fTotalStrokes	 12
U+8A52.0	fCangjie	YRIR
U+8A52.0	fFourCorner	03660
U+8A52.0	fHakka	 toi2; ji2; ri2; i2
U+8A52.0	fCantonese	ji4; doi6; toi5
U+8A52.0	fMandarin	yi2; dai4; tai2; tai3; yi4
U+8A52.0	fEnglish	[1] words as a gift usually given on parting [2] pass on to future generation; hand down to posterity
U+8A53.0	fUTF8	詓
U+8A53.0	fCNS11643	4-3B2C
U+8A53.0	fGB	3-726E
U+8A53.0	fR/S	  149.5
U+8A53.0	fTotalStrokes	 12
U+8A53.0	fCangjie	YRGI
U+8A53.0	fFourCorner	04630
U+8A53.0	fCantonese	heoi2
U+8A53.0	fMandarin	qu3
U+8A54.0	fUTF8	詔
U+8A54.0	fCNS11643	1-6227
U+8A54.0	fBig5	B640
U+8A54.0	fGB	1-5A2F
U+8A54.0	fR/S	 149.5
U+8A54.0	fTotalStrokes	 12
U+8A54.0	fCangjie	YRSHR
U+8A54.0	fFourCorner	07662
U+8A54.0	fHakka	  zhau5;  zau5; sau5; zeu5; shau5; seu5; sau2; cheu5
U+8A54.0	fCantonese	ziu3; ziu6 (also)
U+8A54.0	fMandarin	zhao4; zhao1
U+8A54.0	fEnglish	[1] decree; proclaim; announce [2] instruct; teach and direct; coach [3] imperial decree or mandate; edict
U+8A55.0	fUTF8	評
U+8A55.0	fCNS11643	1-6223
U+8A55.0	fBig5	B5FB
U+8A55.0	fGB	1-4640
U+8A55.0	fR/S	 149.5
U+8A55.0	fTotalStrokes	 12
U+8A55.0	fCangjie	YRMFJ
U+8A55.0	fFourCorner	01649
U+8A55.0	fHakka	  pin2; pin3
U+8A55.0	fCantonese	ping4
U+8A55.0	fMandarin	ping2; bing4
U+8A55.0	fEnglish	[1] comment; review; criticise; evaluate [2] comments; reviews [3] judge; decision after comparison, etc. by a group, as judges in a beauty contest [4] a Chinese family name
U+8A56.0	fUTF8	詖
U+8A56.0	fCNS11643	1-622E
U+8A56.0	fBig5	B647
U+8A56.0	fGB	3-7321
U+8A56.0	fR/S	 149.5
U+8A56.0	fTotalStrokes	 12
U+8A56.0	fCangjie	YRDHE
U+8A56.0	fFourCorner	04647
U+8A56.0	fHakka	  bi5; pi3
U+8A56.0	fCantonese	bei3; bei1; pei4
U+8A56.0	fMandarin	bi4; bei1; po1
U+8A56.0	fEnglish	[1] unfair; erroneous; wrong [2] argue; debate
U+8A57.0	fUTF8	詗
U+8A57.0	fCNS11643	2-4041
U+8A57.0	fBig5	DBDB
U+8A57.0	fGB	3-7273
U+8A57.0	fR/S	 149.5
U+8A57.0	fTotalStrokes	 12
U+8A57.0	fCangjie	YRBR
U+8A57.0	fFourCorner	07620
U+8A57.0	fHakka	 kwen3; hin5
U+8A57.0	fCantonese	gwing2; hing3 (also)
U+8A57.0	fMandarin	xiong4; cheng4; xiong3
U+8A57.0	fEnglish	 spy give information shrewd
U+8A58.0	fUTF8	詘
U+8A58.0	fCNS11643	2-4042
U+8A58.0	fBig5	DBDC
U+8A58.0	fGB	1-5A30
U+8A58.0	fR/S	 149.5
U+8A58.0	fTotalStrokes	 12
U+8A58.0	fCangjie	YRUU
U+8A58.0	fFourCorner	02677
U+8A58.0	fHakka	  kiut7; tut7; kut7
U+8A58.0	fCantonese	wat1; zeot1
U+8A58.0	fMandarin	qu1; chu4; qu4
U+8A58.0	fEnglish	[1] bend; stoop; crouch [2] yield (to another's views, etc.); submit [3] short [4] a Chinese family name
U+8A59.0	fUTF8	詙
U+8A59.0	fCNS11643	2-403F
U+8A59.0	fBig5	DBD9
U+8A59.0	fGB	E-3D66
U+8A59.0	fR/S	 149.5
U+8A59.0	fTotalStrokes	 12
U+8A59.0	fCangjie	YRIKK
U+8A59.0	fFourCorner	03647
U+8A59.0	fCantonese	bat6
U+8A59.0	fMandarin	ba2
U+8A5A.0	fUTF8	詚
U+8A5A.0	fCNS11643	3-3E78
U+8A5A.0	fGB	5-716C
U+8A5A.0	fR/S	  149.5
U+8A5A.0	fTotalStrokes	 12
U+8A5A.0	fCangjie	YRAM
U+8A5A.0	fFourCorner	06610
U+8A5A.0	fCantonese	daat3
U+8A5A.0	fMandarin	da2
U+8A5B.0	fUTF8	詛
U+8A5B.0	fCNS11643	1-6228
U+8A5B.0	fBig5	B641
U+8A5B.0	fGB	1-5767
U+8A5B.0	fR/S	 149.5
U+8A5B.0	fTotalStrokes	 12
U+8A5B.0	fCangjie	YRBM
U+8A5B.0	fFourCorner	07610
U+8A5B.0	fHakka	 zu3; zhu5; zu5; zih3; zii3; zih1
U+8A5B.0	fCantonese	zo3; zo2 (also)
U+8A5B.0	fMandarin	zu3; zu4
U+8A5B.0	fEnglish	[1] curse; swear; imprecate [2] vow; pledge; take an oath
U+8A5C.0	fUTF8	詜
U+8A5C.0	fCNS11643	4-3B31
U+8A5C.0	fGB	3-727E
U+8A5C.0	fR/S	  149.5
U+8A5C.0	fTotalStrokes	 12
U+8A5C.0	fCangjie	YRUE
U+8A5C.0	fFourCorner	02640
U+8A5C.0	fCantonese	tou1
U+8A5C.0	fMandarin	tao1
U+8A5D.0	fUTF8	詝
U+8A5D.0	fCNS11643	3-3F22
U+8A5D.0	fGB	3-726C
U+8A5D.0	fR/S	  149.5
U+8A5D.0	fTotalStrokes	 12
U+8A5D.0	fCangjie	YRJMN
U+8A5D.0	fFourCorner	03620
U+8A5D.0	fCantonese	cyu5
U+8A5D.0	fMandarin	zhu3
U+8A5E.0	fUTF8	詞
U+8A5E.0	fCNS11643	1-6224
U+8A5E.0	fBig5	B5FC
U+8A5E.0	fGB	1-344A
U+8A5E.0	fR/S	 149.5
U+8A5E.0	fTotalStrokes	 12
U+8A5E.0	fCangjie	YRSMR
U+8A5E.0	fFourCorner	07620
U+8A5E.0	fHakka	 cih2; cu2; cih3; ze2
U+8A5E.0	fCantonese	ci4
U+8A5E.0	fMandarin	ci2
U+8A5E.0	fEnglish	[1] word; phrase; expression [2] a part of speech in grammar [3] talk; speak; tell [4] a form of poetry characterized by lines of irregular length
U+8A5F.0	fUTF8	詟
U+8A5F.0	fGB	8-7D49
U+8A5F.0	fR/S	 149.5
U+8A5F.0	fTotalStrokes	 12
U+8A5F.0	fCangjie	IPYMR
U+8A5F.0	fCantonese	sip3; zip3 (also)
U+8A5F.0	fMandarin	zhe2
U+8A5F.0	fEnglish	 fear envy loquacious
U+8A60.0	fUTF8	詠
U+8A60.0	fCNS11643	1-6222
U+8A60.0	fBig5	B5FA
U+8A60.0	fGB	E-3D67
U+8A60.0	fR/S	 149.5
U+8A60.0	fTotalStrokes	 12
U+8A60.0	fCangjie	YRINE
U+8A60.0	fFourCorner	03632
U+8A60.0	fHakka	  jun3; run3; jun1
U+8A60.0	fCantonese	wing6
U+8A60.0	fMandarin	yong3; yong4
U+8A60.0	fEnglish	[1] sing; hum; chant [2] the chirping of birds
U+8A61.0	fUTF8	詡
U+8A61.0	fCNS11643	2-474E
U+8A61.0	fBig5	E048
U+8A61.0	fGB	1-5A3C
U+8A61.0	fR/S	 149.6
U+8A61.0	fTotalStrokes	 13
U+8A61.0	fCangjie	YRSMM
U+8A61.0	fFourCorner	07620
U+8A61.0	fHakka	 hi3
U+8A61.0	fCantonese	heoi2
U+8A61.0	fMandarin	xu3
U+8A61.0	fEnglish	[1] boast; brag; exaggerate [2] popular; make widely known; common [3] clever and brave [4] harmonious
U+8A62.0	fUTF8	詢
U+8A62.0	fCNS11643	1-6666
U+8A62.0	fBig5	B8DF
U+8A62.0	fGB	1-512F
U+8A62.0	fR/S	 149.6
U+8A62.0	fTotalStrokes	 13
U+8A62.0	fCangjie	YRPA
U+8A62.0	fFourCorner	07620
U+8A62.0	fHakka	 sun2; sun3
U+8A62.0	fCantonese	seon1
U+8A62.0	fMandarin	xun2
U+8A62.0	fEnglish	[1] inquire; ask [2] deliberate and plan [3] truly; honesty
U+8A63.0	fUTF8	詣
U+8A63.0	fCNS11643	1-6661
U+8A63.0	fBig5	B8DA
U+8A63.0	fGB	1-5268
U+8A63.0	fR/S	 149.6
U+8A63.0	fTotalStrokes	 13
U+8A63.0	fCangjie	YRPA
U+8A63.0	fFourCorner	01661
U+8A63.0	fHakka	 ngi5; ngi6; ngi3; ji5
U+8A63.0	fCantonese	ngai6; ai6 (also)
U+8A63.0	fMandarin	yi4; ye4
U+8A63.0	fEnglish	[1] go (to a place); arrive; reach [2] visit; call on [3] achievement; accomplishment; attainments
U+8A64.0	fUTF8	詤
U+8A64.0	fCNS11643	3-4460
U+8A64.0	fGB	E-3D68
U+8A64.0	fR/S	  149.6
U+8A64.0	fTotalStrokes	 13
U+8A64.0	fCangjie	YROVU
U+8A64.0	fFourCorner	08610
U+8A64.0	fHakka	 fong1; fong3
U+8A64.0	fCantonese	fong2 (also); fong1
U+8A64.0	fMandarin	huang3
U+8A64.0	fEnglish	 make wild statements to lie to misstate lies falsehood
U+8A65.0	fUTF8	詥
U+8A65.0	fCNS11643	3-4459
U+8A65.0	fGB	3-732E
U+8A65.0	fR/S	  149.6
U+8A65.0	fTotalStrokes	 13
U+8A65.0	fCangjie	YROMR
U+8A65.0	fFourCorner	08660
U+8A65.0	fCantonese	hap6
U+8A65.0	fMandarin	he2
U+8A66.0	fUTF8	試
U+8A66.0	fCNS11643	1-665C
U+8A66.0	fBig5	B8D5
U+8A66.0	fGB	1-4A54
U+8A66.0	fR/S	 149.6
U+8A66.0	fTotalStrokes	 13
U+8A66.0	fCangjie	YRIPM
U+8A66.0	fFourCorner	03640
U+8A66.0	fHakka	 si5;  shi5;  chi5; sii5; cih5; ci5; chii5
U+8A66.0	fCantonese	si5 (also); si3
U+8A66.0	fMandarin	shi4
U+8A66.0	fEnglish	[1] test; try; experiment; trial; test [2] use [3] examine; examination [4] sound out; put up a trial balloon [5] compare [6] a Chinese family name
U+8A67.0	fUTF8	詧
U+8A67.0	fCNS11643	3-4463
U+8A67.0	fGB	E-3D69
U+8A67.0	fR/S	 149.6
U+8A67.0	fTotalStrokes	 13
U+8A67.0	fCangjie	BOYMR
U+8A67.0	fFourCorner	27600
U+8A67.0	fHakka	 cat7
U+8A67.0	fCantonese	caat3
U+8A67.0	fMandarin	cha2
U+8A67.0	fEnglish	 examine into judicially to find out
U+8A68.0	fUTF8	詨
U+8A68.0	fCNS11643	1-666C
U+8A68.0	fBig5	B8E5
U+8A68.0	fGB	3-7333
U+8A68.0	fR/S	 149.6
U+8A68.0	fTotalStrokes	 13
U+8A68.0	fCangjie	YRYCK
U+8A68.0	fFourCorner	00648
U+8A68.0	fHakka	 hau5; gau1; hau1; hau2
U+8A68.0	fCantonese	haau6; gaau1; haau3
U+8A68.0	fMandarin	jiao1; xiao4; jiao4; xiao1
U+8A68.0	fEnglish	kind of cicada; cosmopsaltria
U+8A69.0	fUTF8	詩
U+8A69.0	fCNS11643	1-665D
U+8A69.0	fBig5	B8D6
U+8A69.0	fGB	1-4A2B
U+8A69.0	fR/S	 149.6
U+8A69.0	fTotalStrokes	 13
U+8A69.0	fCangjie	YRGDI
U+8A69.0	fFourCorner	04641
U+8A69.0	fHakka	 si1;  shi1; sii1; she1
U+8A69.0	fCantonese	si1
U+8A69.0	fMandarin	shi1
U+8A69.0	fEnglish	[1] poetry; poem; verse; ode; poetic [2] anything or quality as an offspring of pure imagination [3] short for the Book of Odes edited by Confucius
U+8A6A.0	fUTF8	詪
U+8A6A.0	fCNS11643	3-4455
U+8A6A.0	fGB	3-7335
U+8A6A.0	fR/S	  149.6
U+8A6A.0	fTotalStrokes	 13
U+8A6A.0	fCangjie	YRAV
U+8A6A.0	fFourCorner	07630
U+8A6A.0	fHakka	 gien1; gien5; gen5
U+8A6A.0	fCantonese	han2
U+8A6A.0	fMandarin	hen3
U+8A6A.0	fEnglish	difficulty in speaking wrangling
U+8A6B.0	fUTF8	詫
U+8A6B.0	fCNS11643	1-6659
U+8A6B.0	fBig5	B8D2
U+8A6B.0	fGB	1-326F
U+8A6B.0	fR/S	 149.6
U+8A6B.0	fTotalStrokes	 13
U+8A6B.0	fCangjie	YRJHP
U+8A6B.0	fFourCorner	03614
U+8A6B.0	fHakka	  ca5; za5
U+8A6B.0	fCantonese	caa3
U+8A6B.0	fMandarin	cha4; du4; xia4
U+8A6B.0	fEnglish	[1] surprised; wonder [2] brag; boast [3] cheat; deceive [4] inform
U+8A6C.0	fUTF8	詬
U+8A6C.0	fCNS11643	1-6668
U+8A6C.0	fBig5	B8E1
U+8A6C.0	fGB	1-5A38
U+8A6C.0	fR/S	 149.6
U+8A6C.0	fTotalStrokes	 13
U+8A6C.0	fCangjie	YRHMR
U+8A6C.0	fFourCorner	02661
U+8A6C.0	fHakka	 kieu5; giu5; geu5; heu6; heu5
U+8A6C.0	fCantonese	gau3; kau3; gau2; hau2
U+8A6C.0	fMandarin	gou4; hou4; gou3; kou4
U+8A6C.0	fEnglish	[1] abuse; vituperate; berate [2] insult; shame
U+8A6D.0	fUTF8	詭
U+8A6D.0	fCNS11643	1-6665
U+8A6D.0	fBig5	B8DE
U+8A6D.0	fGB	1-396E
U+8A6D.0	fR/S	 149.6
U+8A6D.0	fTotalStrokes	 13
U+8A6D.0	fCangjie	YRNMU
U+8A6D.0	fFourCorner	07612
U+8A6D.0	fHakka	  gui3; gwi3; kwui3
U+8A6D.0	fCantonese	gwai2
U+8A6D.0	fMandarin	gui3
U+8A6D.0	fEnglish	[1] deceive; cheat; defraud; feign [2] strange; rare; odd; peculiar; uncanny; weird [3] cunning; shrewd; stealthy [4] urge oneself; consider as one's responsibility [5] go against; defy; contradict
U+8A6E.0	fUTF8	詮
U+8A6E.0	fCNS11643	1-6667
U+8A6E.0	fBig5	B8E0
U+8A6E.0	fGB	1-5A39
U+8A6E.0	fR/S	 149.6
U+8A6E.0	fTotalStrokes	 13
U+8A6E.0	fCangjie	YROMG
U+8A6E.0	fFourCorner	08614
U+8A6E.0	fHakka	  cien2; cion2; cen2; zien1
U+8A6E.0	fCantonese	cyun4
U+8A6E.0	fMandarin	quan2
U+8A6E.0	fEnglish	[1] explain; expound; illustrate [2] the truth or core of something [3] weigh; assess; rate; appraise
U+8A6F.0	fUTF8	詯
U+8A6F.0	fCNS11643	3-4458
U+8A6F.0	fGB	3-732C
U+8A6F.0	fR/S	  149.6
U+8A6F.0	fTotalStrokes	 13
U+8A6F.0	fCangjie	YRHBU
U+8A6F.0	fFourCorner	06600
U+8A6F.0	fCantonese	fui3
U+8A6F.0	fMandarin	hui4
U+8A70.0	fUTF8	詰
U+8A70.0	fCNS11643	1-665E
U+8A70.0	fBig5	B8D7
U+8A70.0	fGB	1-5A35
U+8A70.0	fR/S	 149.6
U+8A70.0	fTotalStrokes	 13
U+8A70.0	fCangjie	YRGR
U+8A70.0	fFourCorner	04661
U+8A70.0	fHakka	  giet7; get7
U+8A70.0	fCantonese	kit3; gat1
U+8A70.0	fMandarin	jie2; qie4
U+8A70.0	fEnglish	[1] question; interrogate; ask [2] punish [3] prohibit; restrain
U+8A71.0	fUTF8	話
U+8A71.0	fCNS11643	1-6663
U+8A71.0	fBig5	B8DC
U+8A71.0	fGB	1-3B30
U+8A71.0	fR/S	 149.6
U+8A71.0	fTotalStrokes	 13
U+8A71.0	fCangjie	YRHJR
U+8A71.0	fFourCorner	02664
U+8A71.0	fHakka	  wa5; fa5; wa3; fa6; wa6; va5; wo3; voe1
U+8A71.0	fCantonese	waa6; waa2 (also)
U+8A71.0	fMandarin	hua4
U+8A71.0	fEnglish	[1] speech; talk; conversation; words; sayings [2] speak; talk; converse; say [3] language; dialect [4] advise; counsel [5] say; tell
U+8A71.1	fUTF8	話
U+8A71.1	fCNS11643	1-6663
U+8A71.1	fBig5	B8DC
U+8A71.1	fGB	1-3B30
U+8A71.1	fR/S	 149.6
U+8A71.1	fTotalStrokes	 13
U+8A71.1	fCangjie	YRHJR
U+8A71.1	fFourCorner	02664
U+8A71.1	fMandarin	hua5
U+8A72.0	fUTF8	該
U+8A72.0	fCNS11643	1-665A
U+8A72.0	fBig5	B8D3
U+8A72.0	fGB	1-3843
U+8A72.0	fR/S	 149.6
U+8A72.0	fTotalStrokes	 13
U+8A72.0	fCangjie	YRYVO
U+8A72.0	fFourCorner	00682
U+8A72.0	fHakka	  goi1; gai1; gai2; ge5; gai5
U+8A72.0	fCantonese	goi1
U+8A72.0	fMandarin	gai1
U+8A72.0	fEnglish	[1] should; ought to; need to; obliged to [2] fated to; preordained [3] owe [4] that; the said (person, matter, place, etc.) [5] be somebody's turn to do something [6] deserve [7] inclusive; complete [8] facing
U+8A73.0	fUTF8	詳
U+8A73.0	fCNS11643	1-665B
U+8A73.0	fBig5	B8D4
U+8A73.0	fGB	1-4F6A
U+8A73.0	fR/S	 149.6
U+8A73.0	fTotalStrokes	 13
U+8A73.0	fCangjie	YRTQ
U+8A73.0	fFourCorner	08651
U+8A73.0	fHakka	  siong2; siong3
U+8A73.0	fCantonese	coeng4; joeng4
U+8A73.0	fMandarin	xiang2; yang2
U+8A73.0	fEnglish	[1] detailed; complete; thorough; details; particulars; minute [2] know; know the details [3] please see ... for details [4] explain; interpret
U+8A74.0	fUTF8	詴
U+8A74.0	fCNS11643	2-4756
U+8A74.0	fBig5	E050
U+8A74.0	fGB	5-7227
U+8A74.0	fR/S	 149.6
U+8A74.0	fTotalStrokes	 13
U+8A74.0	fCangjie	YRKB
U+8A74.0	fFourCorner	04627
U+8A74.0	fCantonese	wui1; wui4
U+8A74.0	fMandarin	wei1; hui4
U+8A75.0	fUTF8	詵
U+8A75.0	fCNS11643	2-4753
U+8A75.0	fBig5	E04D
U+8A75.0	fGB	1-5A37
U+8A75.0	fR/S	 149.6
U+8A75.0	fTotalStrokes	 13
U+8A75.0	fCangjie	YRHGU
U+8A75.0	fFourCorner	04611
U+8A75.0	fHakka	  sin1; siin1; shin1; sien3
U+8A75.0	fCantonese	san1; saang2
U+8A75.0	fMandarin	shen1; xian4
U+8A75.0	fEnglish	 ask; inquire; question
U+8A76.0	fUTF8	詶
U+8A76.0	fCNS11643	2-474B
U+8A76.0	fBig5	E045
U+8A76.0	fGB	E-3D6A
U+8A76.0	fR/S	 149.6
U+8A76.0	fTotalStrokes	 13
U+8A76.0	fCangjie	YRILL
U+8A76.0	fFourCorner	02600
U+8A76.0	fHakka	 chiu2; zhiu5; zu5; cu2; chu2
U+8A76.0	fCantonese	zau3; cau4
U+8A76.0	fMandarin	chou2; zhou4; chou4
U+8A76.0	fEnglish	 answer
U+8A77.0	fUTF8	詷
U+8A77.0	fCNS11643	2-4750
U+8A77.0	fBig5	E04A
U+8A77.0	fGB	3-7327
U+8A77.0	fR/S	 149.6
U+8A77.0	fTotalStrokes	 13
U+8A77.0	fCangjie	YRBMR
U+8A77.0	fFourCorner	07620
U+8A77.0	fHakka	  tung2
U+8A77.0	fCantonese	tung4; dung6
U+8A77.0	fMandarin	tong2; dong4
U+8A77.0	fEnglish	[1] all together [2] hasty, as in speech
U+8A78.0	fUTF8	詸
U+8A78.0	fCNS11643	3-445B
U+8A78.0	fGB	5-7235
U+8A78.0	fR/S	  149.6
U+8A78.0	fTotalStrokes	 13
U+8A78.0	fCangjie	YRFD
U+8A78.0	fFourCorner	09690
U+8A78.0	fHakka	 mi5
U+8A78.0	fCantonese	mai4
U+8A78.0	fMandarin	mi2
U+8A79.0	fUTF8	詹
U+8A79.0	fCNS11643	1-6669
U+8A79.0	fBig5	B8E2
U+8A79.0	fGB	0-5532
U+8A79.0	fR/S	 149.6
U+8A79.0	fTotalStrokes	 13
U+8A79.0	fCangjie	NCYMR
U+8A79.0	fFourCorner	27261
U+8A79.0	fHakka	 zham1; zam1
U+8A79.0	fCantonese	zim1
U+8A79.0	fMandarin	zhan1; dan4; shan4
U+8A79.0	fEnglish	[1] talk too much; verbosity [2] reach [3] divine [4] look up [5] a Chinese family name
U+8A7A.0	fUTF8	詺
U+8A7A.0	fCNS11643	2-4757
U+8A7A.0	fBig5	E051
U+8A7A.0	fGB	5-7232
U+8A7A.0	fR/S	 149.6
U+8A7A.0	fTotalStrokes	 13
U+8A7A.0	fCangjie	YRNIR
U+8A7A.0	fFourCorner	07662
U+8A7A.0	fHakka	  miang5; miang2
U+8A7A.0	fCantonese	ming6; ming5
U+8A7A.0	fMandarin	ming4; ming2
U+8A7A.0	fEnglish	[1] distinguish thing by their names [2] discuss names
U+8A7B.0	fUTF8	詻
U+8A7B.0	fCNS11643	1-666A
U+8A7B.0	fBig5	B8E3
U+8A7B.0	fGB	3-7330
U+8A7B.0	fR/S	 149.6
U+8A7B.0	fTotalStrokes	 13
U+8A7B.0	fCangjie	YRHER
U+8A7B.0	fFourCorner	07664
U+8A7B.0	fHakka	  ngiak7;  liok8; gak7
U+8A7B.0	fCantonese	ngaak6; aak6 (also); ngaak3
U+8A7B.0	fMandarin	e4; luo4; lun4
U+8A7B.0	fEnglish	orders
U+8A7C.0	fUTF8	詼
U+8A7C.0	fCNS11643	1-6660
U+8A7C.0	fBig5	B8D9
U+8A7C.0	fGB	1-5A36
U+8A7C.0	fR/S	 149.6
U+8A7C.0	fTotalStrokes	 13
U+8A7C.0	fCangjie	YRKF
U+8A7C.0	fFourCorner	04689
U+8A7C.0	fHakka	  foi1; kui1; foe1; k'wui1
U+8A7C.0	fCantonese	fui1
U+8A7C.0	fMandarin	hui1; kui1
U+8A7C.0	fEnglish	[1] funny; ridicule; mock; humorous [2] ridicule; joke
U+8A7D.0	fUTF8	詽
U+8A7D.0	fCNS11643	3-445C
U+8A7D.0	fGB	E-3D6B
U+8A7D.0	fR/S	  149.6
U+8A7D.0	fTotalStrokes	 13
U+8A7D.0	fCangjie	YRMJJ
U+8A7D.0	fFourCorner	01640
U+8A7D.0	fCantonese	ngaan4; aan4 (also)
U+8A7D.0	fMandarin	yan2
U+8A7E.0	fUTF8	詾
U+8A7E.0	fCNS11643	3-4461
U+8A7E.0	fGB	5-7231
U+8A7E.0	fR/S	 149.6
U+8A7E.0	fTotalStrokes	 13
U+8A7E.0	fCangjie	YRPUK
U+8A7E.0	fFourCorner	07620
U+8A7E.0	fHakka	  hiung1
U+8A7E.0	fCantonese	hung1
U+8A7E.0	fMandarin	xiong1
U+8A7E.0	fEnglish	 noisily uproarious troublesome
U+8A7F.0	fUTF8	詿
U+8A7F.0	fCNS11643	2-474D
U+8A7F.0	fBig5	E047
U+8A7F.0	fGB	1-5A34
U+8A7F.0	fR/S	 149.6
U+8A7F.0	fTotalStrokes	 13
U+8A7F.0	fCangjie	YRGG
U+8A7F.0	fFourCorner	04641
U+8A7F.0	fHakka	 gwa5; gwa1
U+8A7F.0	fCantonese	gwaa3; waa6
U+8A7F.0	fMandarin	gua4; hua4
U+8A7F.0	fEnglish	[1] error; mistake [2] deceive; mislead; deceive
U+8A80.0	fUTF8	誀
U+8A80.0	fCNS11643	3-4456
U+8A80.0	fGB	5-7223
U+8A80.0	fR/S	  149.6
U+8A80.0	fTotalStrokes	 13
U+8A80.0	fCangjie	YRSJ
U+8A80.0	fFourCorner	01640
U+8A80.0	fCantonese	nei6; lei6 (also)
U+8A80.0	fMandarin	er4
U+8A81.0	fUTF8	誁
U+8A81.0	fCNS11643	2-4755
U+8A81.0	fBig5	E04F
U+8A81.0	fGB	3-7334
U+8A81.0	fR/S	 149.6
U+8A81.0	fTotalStrokes	 13
U+8A81.0	fCangjie	YRTT
U+8A81.0	fFourCorner	04640
U+8A81.0	fCantonese	bang2; pin5
U+8A81.0	fMandarin	beng3
U+8A82.0	fUTF8	誂
U+8A82.0	fCNS11643	2-4751
U+8A82.0	fBig5	E04B
U+8A82.0	fGB	3-732F
U+8A82.0	fR/S	 149.6
U+8A82.0	fTotalStrokes	 13
U+8A82.0	fCangjie	YRLMO
U+8A82.0	fFourCorner	02613
U+8A82.0	fHakka	  tiau5; tiau3; tiau2
U+8A82.0	fCantonese	tiu1 (also); tiu5; diu3; diu6
U+8A82.0	fMandarin	tiao3; diao4
U+8A82.0	fEnglish	tempt
U+8A83.0	fUTF8	誃
U+8A83.0	fCNS11643	2-4754
U+8A83.0	fBig5	E04E
U+8A83.0	fGB	3-7331
U+8A83.0	fR/S	 149.6
U+8A83.0	fTotalStrokes	 13
U+8A83.0	fCangjie	YRNIN
U+8A83.0	fFourCorner	07627
U+8A83.0	fHakka	 chi3; cih3; chii3
U+8A83.0	fCantonese	ci2 (also); ci4; ji4; zi2
U+8A83.0	fMandarin	chi3; chi2; yi2; chi4; duo4
U+8A83.0	fEnglish	part
U+8A84.0	fUTF8	誄
U+8A84.0	fCNS11643	2-4752
U+8A84.0	fBig5	E04C
U+8A84.0	fGB	1-5A33
U+8A84.0	fR/S	 149.6
U+8A84.0	fTotalStrokes	 13
U+8A84.0	fCangjie	YRQD
U+8A84.0	fFourCorner	05690
U+8A84.0	fHakka	 loi3
U+8A84.0	fCantonese	loi6; leoi6 (also); leoi5
U+8A84.0	fMandarin	lei3
U+8A84.0	fEnglish	[1] writings eulogizing a dead person; speech; ode; in praise of the dead [2] eulogize in prayer [3] confer a posthumous title [4] pray for the death
U+8A85.0	fUTF8	誅
U+8A85.0	fCNS11643	1-6664
U+8A85.0	fBig5	B8DD
U+8A85.0	fGB	1-566F
U+8A85.0	fR/S	 149.6
U+8A85.0	fTotalStrokes	 13
U+8A85.0	fCangjie	YRHJD
U+8A85.0	fFourCorner	05690
U+8A85.0	fHakka	 zu1; zhu1
U+8A85.0	fCantonese	zyu1
U+8A85.0	fMandarin	zhu1
U+8A85.0	fEnglish	[1] execute; kill; put to death; execution [2] punish [3] weed out; exterminate
U+8A86.0	fUTF8	誆
U+8A86.0	fCNS11643	2-474C
U+8A86.0	fBig5	E046
U+8A86.0	fGB	1-5A32
U+8A86.0	fR/S	 149.6
U+8A86.0	fTotalStrokes	 13
U+8A86.0	fCangjie	YRSMG
U+8A86.0	fFourCorner	01611
U+8A86.0	fHakka	 kiong1; kong5; k'wong5; {kiong1}
U+8A86.0	fCantonese	hong1; kwong4; kong4 (also); gwong2 (also); gong2 (also); gwaang6; gwong6
U+8A86.0	fMandarin	kuang1; guang4; kuang2; kuang4
U+8A86.0	fEnglish	 cheat; swindle; lie; hoax
U+8A87.0	fUTF8	誇
U+8A87.0	fCNS11643	1-665F
U+8A87.0	fBig5	B8D8
U+8A87.0	fGB	1-3F64
U+8A87.0	fR/S	 149.6
U+8A87.0	fTotalStrokes	 13
U+8A87.0	fCangjie	YRKMS
U+8A87.0	fFourCorner	04627
U+8A87.0	fHakka	  kwa1; ka1; k'ua1
U+8A87.0	fCantonese	kwaa1
U+8A87.0	fMandarin	kua1; qu4
U+8A87.0	fEnglish	[1] exaggerate; brag; boast [2] big; great [3] show off; flaunt [4] praise
U+8A88.0	fUTF8	誈
U+8A88.0	fCNS11643	3-445D
U+8A88.0	fGB	E-3D6C
U+8A88.0	fR/S	  149.6
U+8A88.0	fTotalStrokes	 13
U+8A88.0	fCangjie	YRMIG
U+8A88.0	fFourCorner	01610
U+8A88.0	fHakka	 mu2
U+8A88.0	fCantonese	mou4
U+8A88.0	fMandarin	wu2
U+8A89.0	fUTF8	誉
U+8A89.0	fGB	0-537E
U+8A89.0	fR/S	 149.6
U+8A89.0	fTotalStrokes	 13
U+8A89.0	fCangjie	FCYMR
U+8A89.0	fHakka	 ji2; ji5
U+8A89.0	fCantonese	jyu6
U+8A89.0	fMandarin	yu4; yu2
U+8A89.0	fEnglish	simplified form of &#x8B7D;
U+8A8A.0	fUTF8	誊
U+8A8A.0	fGB	0-4C5C
U+8A8A.0	fR/S	 149.6
U+8A8A.0	fTotalStrokes	 13
U+8A8A.0	fCangjie	FQYMR
U+8A8A.0	fHakka	 tien2; ten2
U+8A8A.0	fCantonese	tang4
U+8A8A.0	fMandarin	teng2
U+8A8A.0	fEnglish	simplified form of &#x8B04;
U+8A8B.0	fUTF8	誋
U+8A8B.0	fCNS11643	2-4E34
U+8A8B.0	fBig5	E44C
U+8A8B.0	fGB	3-7347
U+8A8B.0	fR/S	 149.7
U+8A8B.0	fTotalStrokes	 14
U+8A8B.0	fCangjie	YRSUP
U+8A8B.0	fFourCorner	07631
U+8A8B.0	fHakka	 ki3
U+8A8B.0	fCantonese	gei6
U+8A8B.0	fMandarin	ji4; gao4
U+8A8C.0	fUTF8	誌
U+8A8C.0	fCNS11643	1-6B22
U+8A8C.0	fBig5	BB78
U+8A8C.0	fGB	E-3D6D
U+8A8C.0	fR/S	 149.7
U+8A8C.0	fTotalStrokes	 14
U+8A8C.0	fCangjie	YRGP
U+8A8C.0	fFourCorner	04631
U+8A8C.0	fHakka	 zhi5; zi5; zih5; tje5
U+8A8C.0	fCantonese	zi3
U+8A8C.0	fMandarin	zhi4
U+8A8C.0	fEnglish	[1] write down; record; put down [2] record
U+8A8D.0	fUTF8	認
U+8A8D.0	fCNS11643	1-6B25
U+8A8D.0	fBig5	BB7B
U+8A8D.0	fGB	1-484F
U+8A8D.0	fR/S	 149.7
U+8A8D.0	fTotalStrokes	 14
U+8A8D.0	fCangjie	YRSIP
U+8A8D.0	fFourCorner	07632
U+8A8D.0	fHakka	 ngin3; ngin5; gnin5; gnin6; ngin6
U+8A8D.0	fCantonese	jing6; jing2 (also); jan6
U+8A8D.0	fMandarin	ren4; reng4
U+8A8D.0	fEnglish	[1] recognise; know; understand; make out; identify [2] admit; acknowledge; own [3] accept; resign oneself to [4] enter into a certain relationship with; adopt [5] promise to do something; subscribe
U+8A8E.0	fUTF8	誎
U+8A8E.0	fCNS11643	3-495F
U+8A8E.0	fGB	3-733A
U+8A8E.0	fR/S	  149.7
U+8A8E.0	fTotalStrokes	 14
U+8A8E.0	fCangjie	YRDL
U+8A8E.0	fFourCorner	05690
U+8A8E.0	fCantonese	cuk1
U+8A8E.0	fMandarin	su4
U+8A8F.0	fUTF8	誏
U+8A8F.0	fCNS11643	2-4E36
U+8A8F.0	fBig5	E44E
U+8A8F.0	fGB	5-723F
U+8A8F.0	fR/S	 149.7
U+8A8F.0	fTotalStrokes	 14
U+8A8F.0	fCangjie	YRIAV
U+8A8F.0	fFourCorner	03632
U+8A8F.0	fHakka	 long5; long6
U+8A8F.0	fCantonese	long5; long6
U+8A8F.0	fMandarin	lang3; lang4
U+8A8F.0	fEnglish	[1] quibble [2] play upon words
U+8A90.0	fUTF8	誐
U+8A90.0	fCNS11643	3-4964
U+8A90.0	fGB	3-7343
U+8A90.0	fR/S	 149.7
U+8A90.0	fTotalStrokes	 14
U+8A90.0	fCangjie	YRHQI
U+8A90.0	fFourCorner	03650
U+8A90.0	fHakka	  ngo2; o1
U+8A90.0	fCantonese	ngo4
U+8A90.0	fMandarin	e2
U+8A90.0	fEnglish	 chant; sing
U+8A91.0	fUTF8	誑
U+8A91.0	fCNS11643	1-6B2D
U+8A91.0	fBig5	BBA5
U+8A91.0	fGB	1-5A3F
U+8A91.0	fR/S	 149.7
U+8A91.0	fTotalStrokes	 14
U+8A91.0	fCangjie	YRKHG
U+8A91.0	fFourCorner	01614
U+8A91.0	fHakka	 kong2; kong3; k'wong2; k'oue2; k'wong5; kwen2
U+8A91.0	fCantonese	gwong2; kong4 (also); gong2 (also); kwong4 (also); gwong3; wong2
U+8A91.0	fMandarin	kuang2; guang4; kuang4
U+8A91.0	fEnglish	 deceive; lie; delude; cheat
U+8A92.0	fUTF8	誒
U+8A92.0	fCNS11643	2-4E35
U+8A92.0	fBig5	E44D
U+8A92.0	fGB	1-5A40
U+8A92.0	fR/S	 149.7
U+8A92.0	fTotalStrokes	 14
U+8A92.0	fCangjie	YRIOK
U+8A92.0	fFourCorner	03634
U+8A92.0	fHakka	  hi1
U+8A92.0	fCantonese	ei6; ngei6 (also); ei1 (also); ngei1 (also); ei3 (also); oi1; ngei3 (also); e6; aai2; ngoi1 (also); ngaai2 (also); hei1; nge6 (also)
U+8A92.0	fMandarin	ei2; ai1; e4; xi1; yi4; ei1; ai4; hai1; e3
U+8A92.0	fEnglish	an exclamation of confirmation; an exclamation of affirmation
U+8A92.1	fUTF8	誒
U+8A92.1	fCNS11643	2-4E35
U+8A92.1	fBig5	E44D
U+8A92.1	fGB	1-5A40
U+8A92.1	fR/S	 149.7
U+8A92.1	fTotalStrokes	 14
U+8A92.1	fCangjie	YRIOK
U+8A92.1	fFourCorner	03634
U+8A92.1	fMandarin	ei3
U+8A92.2	fUTF8	誒
U+8A92.2	fCNS11643	2-4E35
U+8A92.2	fBig5	E44D
U+8A92.2	fGB	1-5A40
U+8A92.2	fR/S	 149.7
U+8A92.2	fTotalStrokes	 14
U+8A92.2	fCangjie	YRIOK
U+8A92.2	fFourCorner	03634
U+8A92.2	fMandarin	ei4
U+8A93.0	fUTF8	誓
U+8A93.0	fCNS11643	1-6B27
U+8A93.0	fBig5	BB7D
U+8A93.0	fGB	0-4A44
U+8A93.0	fR/S	 149.7
U+8A93.0	fTotalStrokes	 14
U+8A93.0	fCangjie	QLYMR
U+8A93.0	fFourCorner	52601
U+8A93.0	fHakka	 si5; shi6; sii5; s'e5; se5; she5; shi5
U+8A93.0	fCantonese	sai6
U+8A93.0	fMandarin	shi4; she2; yi4
U+8A93.0	fEnglish	[1] swear; pledge; oath; vow [2] take an oath (of allegiance, office, etc.)
U+8A94.0	fUTF8	誔
U+8A94.0	fCNS11643	3-4963
U+8A94.0	fGB	3-7328
U+8A94.0	fR/S	  149.7
U+8A94.0	fTotalStrokes	 14
U+8A94.0	fCangjie	YRNKG
U+8A94.0	fFourCorner	02640
U+8A94.0	fCantonese	ting5
U+8A94.0	fMandarin	ting3
U+8A95.0	fUTF8	誕
U+8A95.0	fCNS11643	1-6E78
U+8A95.0	fBig5	BDCF
U+8A95.0	fGB	1-352E
U+8A95.0	fR/S	 149.7
U+8A95.0	fTotalStrokes	 14
U+8A95.0	fCangjie	YRNKM
U+8A95.0	fFourCorner	02641
U+8A95.0	fHakka	  dan5
U+8A95.0	fCantonese	daan3; daan6
U+8A95.0	fMandarin	dan4
U+8A95.0	fEnglish	[1] birth [2] preposterous; absurd [3] an initial particle
U+8A96.0	fUTF8	誖
U+8A96.0	fCNS11643	2-4E37
U+8A96.0	fBig5	E44F
U+8A96.0	fGB	E-3D6E
U+8A96.0	fR/S	 149.7
U+8A96.0	fTotalStrokes	 14
U+8A96.0	fCangjie	YRJBD
U+8A96.0	fFourCorner	04647
U+8A96.0	fHakka	  put8
U+8A96.0	fCantonese	bui6; bui3 (also); but6
U+8A96.0	fMandarin	bei4; bo2; fei4; fu2; nei4; pei4
U+8A96.0	fEnglish	 obstinate disorderly perverse
U+8A97.0	fUTF8	誗
U+8A97.0	fCNS11643	4-4745
U+8A97.0	fGB	3-7341
U+8A97.0	fR/S	  149.7
U+8A97.0	fTotalStrokes	 14
U+8A97.0	fCangjie	YRHDN
U+8A97.0	fFourCorner	02600
U+8A97.0	fCantonese	cim4
U+8A97.0	fMandarin	chan2
U+8A98.0	fUTF8	誘
U+8A98.0	fCNS11643	1-6B2C
U+8A98.0	fBig5	BBA4
U+8A98.0	fGB	1-5355
U+8A98.0	fR/S	 149.7
U+8A98.0	fTotalStrokes	 14
U+8A98.0	fCangjie	YRHDS
U+8A98.0	fFourCorner	02627
U+8A98.0	fHakka	 jiu5;  ju3; ju5; riu6; riu3
U+8A98.0	fCantonese	jau5
U+8A98.0	fMandarin	you4; you3
U+8A98.0	fEnglish	[1] guide; lead [2] decoy; tempt; allure; lure; captivate; entice
U+8A99.0	fUTF8	誙
U+8A99.0	fCNS11643	2-4E33
U+8A99.0	fBig5	E44B
U+8A99.0	fGB	3-7322
U+8A99.0	fR/S	 149.7
U+8A99.0	fTotalStrokes	 14
U+8A99.0	fCangjie	YRMVM
U+8A99.0	fFourCorner	01611
U+8A99.0	fCantonese	hang1; haang1
U+8A99.0	fMandarin	heng2; keng1; xing4
U+8A9A.0	fUTF8	誚
U+8A9A.0	fCNS11643	1-6B2E
U+8A9A.0	fBig5	BBA6
U+8A9A.0	fGB	1-5A3D
U+8A9A.0	fR/S	 149.7
U+8A9A.0	fTotalStrokes	 14
U+8A9A.0	fCangjie	YRFB
U+8A9A.0	fFourCorner	09627
U+8A9A.0	fHakka	 ciau3;  ciau5; {se3}; sau1; seu1; siau5; ch'iau3; ziau1
U+8A9A.0	fCantonese	ciu3; ziu6
U+8A9A.0	fMandarin	qiao4
U+8A9A.0	fEnglish	 blame; reproach; censure
U+8A9B.0	fUTF8	誛
U+8A9B.0	fCNS11643	3-495E
U+8A9B.0	fGB	3-7346
U+8A9B.0	fR/S	  149.7
U+8A9B.0	fTotalStrokes	 14
U+8A9B.0	fCangjie	YRSME
U+8A9B.0	fFourCorner	07640
U+8A9B.0	fHakka	 cim3
U+8A9B.0	fCantonese	cam1
U+8A9B.0	fMandarin	qin1
U+8A9C.0	fUTF8	誜
U+8A9C.0	fCNS11643	3-4962
U+8A9C.0	fGB	3-7349
U+8A9C.0	fR/S	 149.7
U+8A9C.0	fTotalStrokes	 14
U+8A9C.0	fCangjie	YRICE
U+8A9C.0	fFourCorner	03640
U+8A9C.0	fCantonese	saa3
U+8A9C.0	fMandarin	shua4
U+8A9D.0	fUTF8	誝
U+8A9D.0	fCNS11643	3-496B
U+8A9D.0	fGB	E-3D6F
U+8A9D.0	fR/S	 149.7
U+8A9D.0	fTotalStrokes	 14
U+8A9D.0	fCangjie	YROIR
U+8A9D.0	fFourCorner	08660
U+8A9D.0	fCantonese	am1; ngam1 (also)
U+8A9D.0	fMandarin	an1
U+8A9E.0	fUTF8	語
U+8A9E.0	fCNS11643	1-6B23
U+8A9E.0	fBig5	BB79
U+8A9E.0	fGB	1-536F
U+8A9E.0	fR/S	 149.7
U+8A9E.0	fTotalStrokes	 14
U+8A9E.0	fCangjie	YRMMR
U+8A9E.0	fFourCorner	01661
U+8A9E.0	fHakka	  ngi1;  ngi3; gni1; ni1
U+8A9E.0	fCantonese	jyu5; jyu6
U+8A9E.0	fMandarin	yu3
U+8A9E.0	fEnglish	[1] language; speech [2] word; sentence; expression [3] saying; proverb [4] sign; signal [5] speak; say; talk
U+8A9E.1	fUTF8	語
U+8A9E.1	fCNS11643	1-6B23
U+8A9E.1	fBig5	BB79
U+8A9E.1	fGB	1-536F
U+8A9E.1	fR/S	 149.7
U+8A9E.1	fTotalStrokes	 14
U+8A9E.1	fCangjie	YRMMR
U+8A9E.1	fFourCorner	01661
U+8A9E.1	fMandarin	yu4
U+8A9E.1	fEnglish	 tell; inform; admonish
U+8A9F.0	fUTF8	誟
U+8A9F.0	fCNS11643	3-4968
U+8A9F.0	fGB	E-3D70
U+8A9F.0	fR/S	 149.7
U+8A9F.0	fTotalStrokes	 14
U+8A9F.0	fCangjie	YRJKD
U+8A9F.0	fFourCorner	04640
U+8A9F.0	fHakka	 hau5
U+8A9F.0	fCantonese	haau6
U+8A9F.0	fMandarin	xiao4; jiao1
U+8AA0.0	fUTF8	誠
U+8AA0.0	fCNS11643	1-6662
U+8AA0.0	fBig5	B8DB
U+8AA0.0	fGB	1-334F
U+8AA0.0	fR/S	 149.7; 149.6
U+8AA0.0	fTotalStrokes	 14
U+8AA0.0	fCangjie	YRIHS
U+8AA0.0	fFourCorner	03650
U+8AA0.0	fHakka	 sin2;  shin2; siin2; shin3
U+8AA0.0	fCantonese	sing4; sing2 (also)
U+8AA0.0	fMandarin	cheng2
U+8AA0.0	fEnglish	[1] sincere; honest; cordial; sincerity [2] true; real; truly; indeed; actually
U+8AA1.0	fUTF8	誡
U+8AA1.0	fCNS11643	1-6B26
U+8AA1.0	fBig5	BB7C
U+8AA1.0	fGB	1-3D6B
U+8AA1.0	fR/S	 149.7
U+8AA1.0	fTotalStrokes	 14
U+8AA1.0	fCangjie	YRIT
U+8AA1.0	fFourCorner	03650
U+8AA1.0	fHakka	  gai5; kie5; kiai5; giai5
U+8AA1.0	fCantonese	gaai3
U+8AA1.0	fMandarin	jie4
U+8AA1.0	fEnglish	[1] warn; admonish [2] commandment
U+8AA2.0	fUTF8	誢
U+8AA2.0	fCNS11643	3-4960
U+8AA2.0	fGB	E-3D71
U+8AA2.0	fR/S	  149.7
U+8AA2.0	fTotalStrokes	 14
U+8AA2.0	fCangjie	YRBUU
U+8AA2.0	fFourCorner	06610
U+8AA2.0	fCantonese	jin6
U+8AA2.0	fMandarin	xian4
U+8AA3.0	fUTF8	誣
U+8AA3.0	fCNS11643	1-6B24
U+8AA3.0	fBig5	BB7A
U+8AA3.0	fGB	1-4E5C
U+8AA3.0	fR/S	 149.7
U+8AA3.0	fTotalStrokes	 14
U+8AA3.0	fCangjie	YRMOO
U+8AA3.0	fFourCorner	01618
U+8AA3.0	fHakka	  mu2
U+8AA3.0	fCantonese	mou4
U+8AA3.0	fMandarin	wu1; wu2
U+8AA3.0	fEnglish	 make false accusation; accuse falsely; bring a false charge against
U+8AA4.0	fUTF8	誤
U+8AA4.0	fCNS11643	1-6B28
U+8AA4.0	fBig5	BB7E
U+8AA4.0	fGB	1-4E73
U+8AA4.0	fR/S	 149.7
U+8AA4.0	fTotalStrokes	 14
U+8AA4.0	fCangjie	YRRVK
U+8AA4.0	fFourCorner	06681
U+8AA4.0	fHakka	  ngu5; ngu3
U+8AA4.0	fCantonese	ng6
U+8AA4.0	fMandarin	wu4
U+8AA4.0	fEnglish	[1] err; make mistake; error [2] harm; suffer; injure [3] delay [4] miss [5] by mistake; by accident
U+8AA4.1	fUTF8	誤
U+8AA4.1	fCNS11643	1-6B28
U+8AA4.1	fBig5	BB7E
U+8AA4.1	fGB	1-4E73
U+8AA4.1	fR/S	 149.7
U+8AA4.1	fTotalStrokes	 14
U+8AA4.1	fCangjie	YRRVK
U+8AA4.1	fFourCorner	06681
U+8AA4.1	fMandarin	wu5
U+8AA5.0	fUTF8	誥
U+8AA5.0	fCNS11643	1-6B2A
U+8AA5.0	fBig5	BBA2
U+8AA5.0	fGB	1-5A3E
U+8AA5.0	fR/S	 149.7
U+8AA5.0	fTotalStrokes	 14
U+8AA5.0	fCangjie	YRHGR
U+8AA5.0	fFourCorner	04661
U+8AA5.0	fHakka	  kau5; gau5; go5; gau1
U+8AA5.0	fCantonese	gou3
U+8AA5.0	fMandarin	gao4; gu4
U+8AA5.0	fEnglish	[1] grant; confer; bestow [2] admonish [3] a written admonition; imperial mandate
U+8AA6.0	fUTF8	誦
U+8AA6.0	fCNS11643	1-6B21
U+8AA6.0	fBig5	BB77
U+8AA6.0	fGB	1-4B50
U+8AA6.0	fR/S	 149.7
U+8AA6.0	fTotalStrokes	 14
U+8AA6.0	fCangjie	YRNIB
U+8AA6.0	fFourCorner	07627
U+8AA6.0	fHakka	  siung5; siung6; sung3; siung3
U+8AA6.0	fCantonese	zung6
U+8AA6.0	fMandarin	song4
U+8AA6.0	fEnglish	[1] recite; chant; intone [2] poetry; poem; songs
U+8AA7.0	fUTF8	誧
U+8AA7.0	fCNS11643	1-6B2F
U+8AA7.0	fBig5	BBA7
U+8AA7.0	fGB	3-7339
U+8AA7.0	fR/S	 149.7
U+8AA7.0	fTotalStrokes	 14
U+8AA7.0	fCangjie	YRIJB
U+8AA7.0	fFourCorner	03627
U+8AA7.0	fHakka	  pu3;  pu2; pu1
U+8AA7.0	fCantonese	bou1; bou3; pou1; pou2; pou4
U+8AA7.0	fMandarin	pu3; bu1; pu1; pu4
U+8AA7.0	fEnglish	 huge admonish
U+8AA8.0	fUTF8	誨
U+8AA8.0	fCNS11643	1-6B2B
U+8AA8.0	fBig5	BBA3
U+8AA8.0	fGB	1-3B65
U+8AA8.0	fR/S	 149.7
U+8AA8.0	fTotalStrokes	 14
U+8AA8.0	fCangjie	YROWY
U+8AA8.0	fFourCorner	08657
U+8AA8.0	fHakka	  fui5; fi5
U+8AA8.0	fCantonese	fui3
U+8AA8.0	fMandarin	hui4
U+8AA8.0	fEnglish	[1] teach; instruct [2] instructions [3] admonish [4] induce
U+8AA9.0	fUTF8	誩
U+8AA9.0	fCNS11643	3-4965
U+8AA9.0	fGB	E-3D72
U+8AA9.0	fR/S	 149.7
U+8AA9.0	fTotalStrokes	 14
U+8AA9.0	fCangjie	YRYMR
U+8AA9.0	fFourCorner	00660
U+8AA9.0	fHakka	 kin5; kin6
U+8AA9.0	fCantonese	ging6
U+8AA9.0	fMandarin	jing4
U+8AA9.0	fEnglish	 quarrel wrangle
U+8AAA.0	fUTF8	說
U+8AAA.0	fCNS11643	1-6B29
U+8AAA.0	fBig5	BBA1
U+8AAA.0	fGB	E-3D73
U+8AAA.0	fR/S	 149.7
U+8AAA.0	fTotalStrokes	 14
U+8AAA.0	fCangjie	YRCRU
U+8AAA.0	fFourCorner	08616
U+8AAA.0	fHakka	  sot7; shot7;  jet8; s'oi5; s'oi6; soi5; ret8
U+8AAA.0	fCantonese	syut3; seoi3; jyut6
U+8AAA.0	fMandarin	shuo1; tuo1; rui4
U+8AAA.0	fEnglish	[1] speak; say; talk; utter [2] explain; clarify [3] description; narration; statement [4] theory [5] scold; talk to; talking-to; scolding
U+8AAA.1	fUTF8	說
U+8AAA.1	fCNS11643	1-6B29
U+8AAA.1	fBig5	BBA1
U+8AAA.1	fGB	E-3D73
U+8AAA.1	fR/S	 149.7
U+8AAA.1	fTotalStrokes	 14
U+8AAA.1	fCangjie	YRCRU
U+8AAA.1	fFourCorner	08616
U+8AAA.1	fMandarin	shui4
U+8AAA.1	fEnglish	 persuade; influence
U+8AAA.2	fUTF8	說
U+8AAA.2	fCNS11643	1-6B29
U+8AAA.2	fBig5	BBA1
U+8AAA.2	fGB	E-3D73
U+8AAA.2	fR/S	 149.7
U+8AAA.2	fTotalStrokes	 14
U+8AAA.2	fCangjie	YRCRU
U+8AAA.2	fFourCorner	08616
U+8AAA.2	fMandarin	yue4
U+8AAA.2	fEnglish	 delight; please
U+8AAB.0	fUTF8	誫
U+8AAB.0	fCNS11643	2-4E32
U+8AAB.0	fBig5	E44A
U+8AAB.0	fGB	E-3D74
U+8AAB.0	fR/S	 149.7
U+8AAB.0	fTotalStrokes	 14
U+8AAB.0	fCangjie	YRMMV
U+8AAB.0	fFourCorner	01632
U+8AAB.0	fCantonese	zan3
U+8AAB.0	fMandarin	zhen4
U+8AAC.0	fUTF8	説
U+8AAC.0	fCNS11643	3-4966
U+8AAC.0	fGB	1-4B35
U+8AAC.0	fR/S	 149.7
U+8AAC.0	fTotalStrokes	 14
U+8AAC.0	fCangjie	YRCRU
U+8AAC.0	fFourCorner	08610
U+8AAC.0	fHakka	 sot7; jet8
U+8AAC.0	fCantonese	syut3; seoi3; jyut6
U+8AAC.0	fMandarin	shuo1; shui4; yue4; tuo1
U+8AAC.0	fEnglish	speak
U+8AAD.0	fUTF8	読
U+8AAD.0	fCNS11643	F-4C41
U+8AAD.0	fGB	E-3D75
U+8AAD.0	fR/S	 149.7
U+8AAD.0	fTotalStrokes	 14
U+8AAD.0	fCangjie	YRGBU
U+8AAD.0	fCantonese	duk6
U+8AAD.0	fMandarin	du2; dou4
U+8AAD.0	fEnglish	 read; study pronounce
U+8AAE.0	fUTF8	誮
U+8AAE.0	fGB	E-3D76
U+8AAE.0	fR/S	 149.7
U+8AAE.0	fTotalStrokes	 14
U+8AAE.0	fCangjie	YRTOP
U+8AAE.0	fCantonese	waa1
U+8AAF.0	fUTF8	誯
U+8AAF.0	fCNS11643	3-4E74
U+8AAF.0	fGB	5-7247
U+8AAF.0	fR/S	 149.8
U+8AAF.0	fTotalStrokes	 15
U+8AAF.0	fCangjie	YRAA
U+8AAF.0	fFourCorner	06660
U+8AAF.0	fHakka	 chong5
U+8AAF.0	fCantonese	coeng3
U+8AAF.0	fMandarin	chang4
U+8AB0.0	fUTF8	誰
U+8AB0.0	fCNS11643	1-6F21
U+8AB0.0	fBig5	BDD6
U+8AB0.0	fGB	1-4B2D
U+8AB0.0	fR/S	 149.8
U+8AB0.0	fTotalStrokes	 15
U+8AB0.0	fCangjie	YROG
U+8AB0.0	fFourCorner	00614
U+8AB0.0	fHakka	  sui2;  shui2; sui3
U+8AB0.0	fCantonese	seoi4; seoi2 (also)
U+8AB0.0	fMandarin	shei2; shui2
U+8AB0.0	fEnglish	[1] who?; whom?; whose? [2] anyone?; someone?
U+8AB1.0	fUTF8	誱
U+8AB1.0	fCNS11643	3-4E67
U+8AB1.0	fGB	5-7245
U+8AB1.0	fR/S	  149.8
U+8AB1.0	fTotalStrokes	 15
U+8AB1.0	fCangjie	YRJLO
U+8AB1.0	fFourCorner	05680
U+8AB1.0	fCantonese	zit6
U+8AB1.0	fMandarin	jie2
U+8AB2.0	fUTF8	課
U+8AB2.0	fCNS11643	1-6E7B
U+8AB2.0	fBig5	BDD2
U+8AB2.0	fGB	1-3F4E
U+8AB2.0	fR/S	 149.8
U+8AB2.0	fTotalStrokes	 15
U+8AB2.0	fCangjie	YRWD
U+8AB2.0	fFourCorner	06694
U+8AB2.0	fHakka	  ko5
U+8AB2.0	fCantonese	fo3
U+8AB2.0	fMandarin	ke4; ke1
U+8AB2.0	fEnglish	[1] class meeting [2] lesson [3] course (of study) [4] impose; levy; tax [5] a session at divination [6] suboffice; bureau [7] supervise
U+8AB3.0	fUTF8	誳
U+8AB3.0	fCNS11643	3-4E72
U+8AB3.0	fGB	3-7357
U+8AB3.0	fR/S	  149.8
U+8AB3.0	fTotalStrokes	 15
U+8AB3.0	fCangjie	YRSUU
U+8AB3.0	fFourCorner	07670
U+8AB3.0	fHakka	 kiut7
U+8AB3.0	fCantonese	wat1
U+8AB3.0	fMandarin	qu1
U+8AB4.0	fUTF8	誴
U+8AB4.0	fCNS11643	3-4E70
U+8AB4.0	fGB	5-724D
U+8AB4.0	fR/S	  149.8
U+8AB4.0	fTotalStrokes	 15
U+8AB4.0	fCangjie	YRJMF
U+8AB4.0	fFourCorner	03690
U+8AB4.0	fCantonese	cung4
U+8AB4.0	fMandarin	cong2
U+8AB5.0	fUTF8	誵
U+8AB5.0	fCNS11643	3-4E6E
U+8AB5.0	fGB	E-3D77
U+8AB5.0	fR/S	  149.8
U+8AB5.0	fTotalStrokes	 15
U+8AB5.0	fCangjie	YRKKB
U+8AB5.0	fFourCorner	04620
U+8AB5.0	fCantonese	haau4
U+8AB5.0	fMandarin	xiao2
U+8AB6.0	fUTF8	誶
U+8AB6.0	fCNS11643	1-6F24
U+8AB6.0	fBig5	BDD9
U+8AB6.0	fGB	1-5A47
U+8AB6.0	fR/S	 149.8
U+8AB6.0	fTotalStrokes	 15
U+8AB6.0	fCangjie	YRYOJ
U+8AB6.0	fFourCorner	00648
U+8AB6.0	fHakka	  cui5; sui5
U+8AB6.0	fCantonese	seoi6; seoi3
U+8AB6.0	fMandarin	sui4; cui4
U+8AB6.0	fEnglish	 reproach; scold; upbraid
U+8AB7.0	fUTF8	誷
U+8AB7.0	fCNS11643	3-4E6C
U+8AB7.0	fGB	3-7350
U+8AB7.0	fR/S	  149.8
U+8AB7.0	fTotalStrokes	 15
U+8AB7.0	fCangjie	YRBTV
U+8AB7.0	fFourCorner	07620
U+8AB7.0	fHakka	 wong3; vong3
U+8AB7.0	fCantonese	mong5
U+8AB7.0	fMandarin	wang3
U+8AB8.0	fUTF8	誸
U+8AB8.0	fCNS11643	2-543F
U+8AB8.0	fBig5	E7D6
U+8AB8.0	fGB	3-7358
U+8AB8.0	fR/S	 149.8
U+8AB8.0	fTotalStrokes	 15
U+8AB8.0	fCangjie	YRNYI
U+8AB8.0	fFourCorner	00632
U+8AB8.0	fCantonese	jin4
U+8AB8.0	fMandarin	xuan2; xian2
U+8AB9.0	fUTF8	誹
U+8AB9.0	fCNS11643	1-6F25
U+8AB9.0	fBig5	BDDA
U+8AB9.0	fGB	1-374C
U+8AB9.0	fR/S	 149.8
U+8AB9.0	fTotalStrokes	 15
U+8AB9.0	fCangjie	YRLMY
U+8AB9.0	fFourCorner	01611
U+8AB9.0	fHakka	 fui3; fui1; fi3; fi1
U+8AB9.0	fCantonese	fei2
U+8AB9.0	fMandarin	fei3; fei1; fei4
U+8AB9.0	fEnglish	 slander; attack; condemn
U+8ABA.0	fUTF8	誺
U+8ABA.0	fCNS11643	2-544B
U+8ABA.0	fBig5	E7E2
U+8ABA.0	fGB	3-733C
U+8ABA.0	fR/S	 149.8
U+8ABA.0	fTotalStrokes	 15
U+8ABA.0	fCangjie	YRDOO
U+8ABA.0	fFourCorner	04698
U+8ABA.0	fCantonese	ci1; loi6
U+8ABA.0	fMandarin	chi1; chi4; lai4
U+8ABB.0	fUTF8	誻
U+8ABB.0	fCNS11643	2-5444
U+8ABB.0	fBig5	E7DB
U+8ABB.0	fGB	5-7250
U+8ABB.0	fR/S	 149.8
U+8ABB.0	fTotalStrokes	 15
U+8ABB.0	fCangjie	YREA
U+8ABB.0	fFourCorner	02663
U+8ABB.0	fCantonese	daap6; taap3 (also); taap6
U+8ABB.0	fMandarin	ta4; ta3
U+8ABC.0	fUTF8	誼
U+8ABC.0	fCNS11643	1-6E74
U+8ABC.0	fBig5	BDCB
U+8ABC.0	fGB	1-526A
U+8ABC.0	fR/S	 149.8
U+8ABC.0	fTotalStrokes	 15
U+8ABC.0	fCangjie	YRJBM
U+8ABC.0	fFourCorner	03617
U+8ABC.0	fHakka	 ngi2;  ni3;  ngi5; ji2; ngi6
U+8ABC.0	fCantonese	ji4; ji6
U+8ABC.0	fMandarin	yi4; yi2
U+8ABC.0	fEnglish	[1] friendship [2] same as /@/
U+8ABD.0	fUTF8	誽
U+8ABD.0	fCNS11643	2-544C
U+8ABD.0	fBig5	E7E3
U+8ABD.0	fGB	3-7352
U+8ABD.0	fR/S	 149.8
U+8ABD.0	fTotalStrokes	 15
U+8ABD.0	fCangjie	YRHXU
U+8ABD.0	fFourCorner	07627
U+8ABD.0	fCantonese	ngai6; ai6 (also)
U+8ABD.0	fMandarin	na2; nai2; yi4
U+8ABE.0	fUTF8	誾
U+8ABE.0	fCNS11643	2-5446
U+8ABE.0	fBig5	E7DD
U+8ABE.0	fGB	3-3B23
U+8ABE.0	fR/S	 149.8
U+8ABE.0	fTotalStrokes	 15
U+8ABE.0	fCangjie	ANYMR
U+8ABE.0	fFourCorner	77601
U+8ABE.0	fHakka	 ngiun2; niun3
U+8ABE.0	fCantonese	ngan4; an4 (also)
U+8ABE.0	fMandarin	yin2
U+8ABE.0	fEnglish	 speak gently respectful
U+8ABF.0	fUTF8	調
U+8ABF.0	fCNS11643	1-6E7E
U+8ABF.0	fBig5	BDD5
U+8ABF.0	fGB	1-3577
U+8ABF.0	fR/S	 149.8
U+8ABF.0	fTotalStrokes	 15
U+8ABF.0	fCangjie	YRBGR
U+8ABF.0	fFourCorner	07620
U+8ABF.0	fHakka	  tiau2;  tiau5; tiau3; diau5; tiau6; diau6; tiu5
U+8ABF.0	fCantonese	diu6; deu6 (also); tiu4; diu2 (also)
U+8ABF.0	fMandarin	diao4; zhou1; tiao4
U+8ABF.0	fEnglish	[1] mix; blend [2] regulate; adjust [3] balance; regular [4] make fun of; tease [5] mediate
U+8ABF.1	fUTF8	調
U+8ABF.1	fCNS11643	1-6E7E
U+8ABF.1	fBig5	BDD5
U+8ABF.1	fGB	1-3577
U+8ABF.1	fR/S	 149.8
U+8ABF.1	fTotalStrokes	 15
U+8ABF.1	fCangjie	YRBGR
U+8ABF.1	fFourCorner	07620
U+8ABF.1	fMandarin	tiao2
U+8AC0.0	fUTF8	諀
U+8AC0.0	fCNS11643	2-5447
U+8AC0.0	fBig5	E7DE
U+8AC0.0	fGB	3-7354
U+8AC0.0	fR/S	 149.8
U+8AC0.0	fTotalStrokes	 15
U+8AC0.0	fCangjie	YRHWJ
U+8AC0.0	fFourCorner	06640
U+8AC0.0	fCantonese	pei2
U+8AC0.0	fMandarin	pi3; bei1
U+8AC1.0	fUTF8	諁
U+8AC1.0	fCNS11643	3-4E73
U+8AC1.0	fGB	3-735A
U+8AC1.0	fR/S	  149.8
U+8AC1.0	fTotalStrokes	 15
U+8AC1.0	fCangjie	YREEE
U+8AC1.0	fFourCorner	07640
U+8AC1.0	fCantonese	zyut3
U+8AC1.0	fMandarin	chuo4
U+8AC1.0	fEnglish	to suck, sip, sob, kiss; to chatter
U+8AC2.0	fUTF8	諂
U+8AC2.0	fCNS11643	1-6E7D
U+8AC2.0	fBig5	BDD4
U+8AC2.0	fGB	1-5A46
U+8AC2.0	fR/S	 149.8
U+8AC2.0	fTotalStrokes	 15
U+8AC2.0	fCangjie	YRNHX
U+8AC2.0	fFourCorner	07677
U+8AC2.0	fHakka	  cam3;  tiam5; cam5; cham3; cham5
U+8AC2.0	fCantonese	cim2
U+8AC2.0	fMandarin	chan3
U+8AC2.0	fEnglish	 flatter; cringe; fawn
U+8AC3.0	fUTF8	諃
U+8AC3.0	fCNS11643	2-544A
U+8AC3.0	fBig5	E7E1
U+8AC3.0	fGB	3-734C
U+8AC3.0	fR/S	 149.8
U+8AC3.0	fTotalStrokes	 15
U+8AC3.0	fCangjie	YRDD
U+8AC3.0	fFourCorner	04690
U+8AC3.0	fCantonese	cam1
U+8AC3.0	fMandarin	chen1
U+8AC4.0	fUTF8	諄
U+8AC4.0	fCNS11643	1-6E77
U+8AC4.0	fBig5	BDCE
U+8AC4.0	fGB	1-573B
U+8AC4.0	fR/S	 149.8
U+8AC4.0	fTotalStrokes	 15
U+8AC4.0	fCangjie	YRYRD
U+8AC4.0	fFourCorner	00647
U+8AC4.0	fHakka	 zun1; dun1; zhun1; shun2
U+8AC4.0	fCantonese	zeon1; deon1
U+8AC4.0	fMandarin	zhun1; zhun4; ma4; yi2; zhun3
U+8AC4.0	fEnglish	patient; earnest (in explaining, teaching, etc.); earnestly
U+8AC5.0	fUTF8	諅
U+8AC5.0	fCNS11643	2-5448
U+8AC5.0	fBig5	E7DF
U+8AC5.0	fGB	5-7146
U+8AC5.0	fR/S	 149.8
U+8AC5.0	fTotalStrokes	 15
U+8AC5.0	fCangjie	TCYMR
U+8AC5.0	fFourCorner	44601
U+8AC5.0	fCantonese	gei6; gei1
U+8AC5.0	fMandarin	ji1; ji4
U+8AC6.0	fUTF8	諆
U+8AC6.0	fCNS11643	2-543E
U+8AC6.0	fBig5	E7D5
U+8AC6.0	fGB	3-734A
U+8AC6.0	fR/S	 149.8
U+8AC6.0	fTotalStrokes	 15
U+8AC6.0	fCangjie	YRTMC
U+8AC6.0	fFourCorner	04681
U+8AC6.0	fCantonese	hei1
U+8AC6.0	fMandarin	qi1; ji1; qi2
U+8AC6.0	fEnglish	 cheat
U+8AC7.0	fUTF8	談
U+8AC7.0	fCNS11643	1-6E76
U+8AC7.0	fBig5	BDCD
U+8AC7.0	fGB	1-4C38
U+8AC7.0	fR/S	 149.8
U+8AC7.0	fTotalStrokes	 15
U+8AC7.0	fCangjie	YRFF
U+8AC7.0	fFourCorner	09689
U+8AC7.0	fHakka	  tam2; dam2; tam3
U+8AC7.0	fCantonese	taam4
U+8AC7.0	fMandarin	tan2
U+8AC7.0	fEnglish	[1] talk; converse; chat [2] what is said or talked about; talk; gossip [3] a Chinese family name
U+8AC8.0	fUTF8	諈
U+8AC8.0	fCNS11643	2-5A56
U+8AC8.0	fBig5	EBAA
U+8AC8.0	fGB	3-7351
U+8AC8.0	fR/S	 149.8
U+8AC8.0	fTotalStrokes	 15
U+8AC8.0	fCangjie	YRHJM
U+8AC8.0	fFourCorner	02614
U+8AC8.0	fCantonese	zeoi3; zeoi6; ceoi4
U+8AC8.0	fMandarin	zhui4; chui4; chui2
U+8AC9.0	fUTF8	諉
U+8AC9.0	fCNS11643	1-6E7C
U+8AC9.0	fBig5	BDD3
U+8AC9.0	fGB	1-5A43
U+8AC9.0	fR/S	 149.8
U+8AC9.0	fTotalStrokes	 15
U+8AC9.0	fCangjie	YRHDV
U+8AC9.0	fFourCorner	02644
U+8AC9.0	fHakka	 wui3; wui1; wi3; wi1; vui3
U+8AC9.0	fCantonese	wai2
U+8AC9.0	fMandarin	wei3; rui4; wei1; wei4
U+8AC9.0	fEnglish	 shirk; evade; blame upon others
U+8ACA.0	fUTF8	諊
U+8ACA.0	fCNS11643	3-4E75
U+8ACA.0	fGB	5-724B
U+8ACA.0	fR/S	  149.8
U+8ACA.0	fTotalStrokes	 15
U+8ACA.0	fCangjie	YRPFD
U+8ACA.0	fFourCorner	07620
U+8ACA.0	fHakka	  kiuk7
U+8ACA.0	fCantonese	guk1
U+8ACA.0	fMandarin	ju2
U+8ACA.0	fEnglish	 make a judicial investigation
U+8ACB.0	fUTF8	請
U+8ACB.0	fCNS11643	1-6E79
U+8ACB.0	fBig5	BDD0
U+8ACB.0	fGB	1-476B
U+8ACB.0	fR/S	 149.8
U+8ACB.0	fTotalStrokes	 15
U+8ACB.0	fCangjie	YRQMB
U+8ACB.0	fFourCorner	05627
U+8ACB.0	fHakka	  ciang3; ciang5
U+8ACB.0	fCantonese	cing2; ceng2 (also); cing1; king4; zing6
U+8ACB.0	fMandarin	qing3; qing1; qing2; jing4; qing4
U+8ACB.0	fEnglish	[1] ask; request; beg for; beseech; entreat [2] please; [3] hire; seek the service of; engage
U+8ACC.0	fUTF8	諌
U+8ACC.0	fCNS11643	3-4E69
U+8ACC.0	fGB	E-3D78
U+8ACC.0	fR/S	  149.8
U+8ACC.0	fTotalStrokes	 15
U+8ACC.0	fCangjie	YRDW
U+8ACC.0	fFourCorner	05690
U+8ACC.0	fCantonese	dung2
U+8ACC.0	fMandarin	dong3; jian4
U+8ACC.0	fEnglish	remonstrate; admonish
U+8ACD.0	fUTF8	諍
U+8ACD.0	fCNS11643	1-6F23
U+8ACD.0	fBig5	BDD8
U+8ACD.0	fGB	1-5A3A
U+8ACD.0	fR/S	 149.8
U+8ACD.0	fTotalStrokes	 15
U+8ACD.0	fCangjie	YRBSD
U+8ACD.0	fFourCorner	02657
U+8ACD.0	fHakka	 zen5; zen1; zan5; zang1; cang5
U+8ACD.0	fCantonese	zaang3; zang3 (also); zaang1; zang1 (also)
U+8ACD.0	fMandarin	zheng1; zheng4
U+8ACD.0	fEnglish	[1] expostulate; remonstrate; criticize somebody's faults frankly; admonish [2] dispute; compete; vie
U+8ACE.0	fUTF8	諎
U+8ACE.0	fCNS11643	3-4E6A
U+8ACE.0	fGB	3-734B
U+8ACE.0	fR/S	  149.8
U+8ACE.0	fTotalStrokes	 15
U+8ACE.0	fCangjie	YRTA
U+8ACE.0	fFourCorner	04660
U+8ACE.0	fHakka	  zit7;  zia5; co5
U+8ACE.0	fCantonese	zaak3
U+8ACE.0	fMandarin	ze2
U+8ACE.0	fEnglish	 reply; cry out
U+8ACF.0	fUTF8	諏
U+8ACF.0	fCNS11643	2-543D
U+8ACF.0	fBig5	E7D4
U+8ACF.0	fGB	1-5A41
U+8ACF.0	fR/S	 149.8
U+8ACF.0	fTotalStrokes	 15
U+8ACF.0	fCangjie	YRSJE
U+8ACF.0	fFourCorner	01647
U+8ACF.0	fHakka	  zeu1;  ci1
U+8ACF.0	fCantonese	zau1
U+8ACF.0	fMandarin	zou1; fu2; zhou1; ju1
U+8ACF.0	fEnglish	[1] consult; seek the advice of [2] confer
U+8AD0.0	fUTF8	諐
U+8AD0.0	fCNS11643	3-4E76
U+8AD0.0	fGB	E-3D79
U+8AD0.0	fR/S	  149.8
U+8AD0.0	fTotalStrokes	 15
U+8AD0.0	fCangjie	OUYMR
U+8AD0.0	fFourCorner	26600
U+8AD0.0	fCantonese	hin1
U+8AD0.0	fMandarin	qian1
U+8AD0.0	fEnglish	fault
U+8AD1.0	fUTF8	諑
U+8AD1.0	fCNS11643	2-5441
U+8AD1.0	fBig5	E7D8
U+8AD1.0	fGB	1-5A42
U+8AD1.0	fR/S	 149.8
U+8AD1.0	fTotalStrokes	 15
U+8AD1.0	fCangjie	YRMSO
U+8AD1.0	fFourCorner	01632
U+8AD1.0	fHakka	 dok7; duk7
U+8AD1.0	fCantonese	doek3; doek6
U+8AD1.0	fMandarin	zhuo2
U+8AD1.0	fEnglish	gossip; rumors
U+8AD2.0	fUTF8	諒
U+8AD2.0	fCNS11643	1-6E75
U+8AD2.0	fBig5	BDCC
U+8AD2.0	fGB	1-4142
U+8AD2.0	fR/S	 149.8
U+8AD2.0	fTotalStrokes	 15
U+8AD2.0	fCangjie	YRYRF
U+8AD2.0	fFourCorner	00696
U+8AD2.0	fHakka	  liong5; liong6
U+8AD2.0	fCantonese	loeng6; loeng4
U+8AD2.0	fMandarin	liang4
U+8AD2.0	fEnglish	[1] honest; sincere [2] forgive; excuse; understand [3] conjecture; guess; presume; think; assume; presume; expect; suppose [4] stubborn
U+8AD2.1	fUTF8	諒
U+8AD2.1	fCNS11643	1-6E75
U+8AD2.1	fBig5	BDCC
U+8AD2.1	fGB	1-4142
U+8AD2.1	fR/S	 149.8
U+8AD2.1	fTotalStrokes	 15
U+8AD2.1	fCangjie	YRYRF
U+8AD2.1	fFourCorner	00696
U+8AD2.1	fMandarin	liang2
U+8AD2.1	fEnglish	used in combination
U+8AD3.0	fUTF8	諓
U+8AD3.0	fCNS11643	2-5440
U+8AD3.0	fBig5	E7D7
U+8AD3.0	fGB	3-726D
U+8AD3.0	fR/S	 149.8
U+8AD3.0	fTotalStrokes	 15
U+8AD3.0	fCangjie	YRII
U+8AD3.0	fFourCorner	03653
U+8AD3.0	fHakka	  cien3;  zien3
U+8AD3.0	fCantonese	zin3; zin2 (also); zin6
U+8AD3.0	fMandarin	jian4; jian1; yan3
U+8AD3.0	fEnglish	artful; cunning
U+8AD4.0	fUTF8	諔
U+8AD4.0	fCNS11643	2-5442
U+8AD4.0	fBig5	E7D9
U+8AD4.0	fGB	3-734E
U+8AD4.0	fR/S	 149.8
U+8AD4.0	fTotalStrokes	 15
U+8AD4.0	fCangjie	YRYFE
U+8AD4.0	fFourCorner	07640
U+8AD4.0	fHakka	 sip7
U+8AD4.0	fCantonese	cuk1; suk1
U+8AD4.0	fMandarin	zhu4; chu4; ji2
U+8AD5.0	fUTF8	諕
U+8AD5.0	fCNS11643	2-5443
U+8AD5.0	fBig5	E7DA
U+8AD5.0	fGB	3-734F
U+8AD5.0	fR/S	 149.8
U+8AD5.0	fTotalStrokes	 15
U+8AD5.0	fCangjie	YRYPU
U+8AD5.0	fFourCorner	01617
U+8AD5.0	fHakka	  hau2; fu3
U+8AD5.0	fCantonese	haak3; hou4
U+8AD5.0	fMandarin	hao2; xia4; huo4
U+8AD5.0	fEnglish	 intimidate
U+8AD6.0	fUTF8	論
U+8AD6.0	fCNS11643	1-6F22
U+8AD6.0	fBig5	BDD7
U+8AD6.0	fGB	1-425B
U+8AD6.0	fR/S	 149.8
U+8AD6.0	fTotalStrokes	 15
U+8AD6.0	fCangjie	YROMB
U+8AD6.0	fFourCorner	08627
U+8AD6.0	fHakka	  lun5;  lun2; lun3; lun6
U+8AD6.0	fCantonese	leon6; leon4 (also); leon2 (also)
U+8AD6.0	fMandarin	lun4
U+8AD6.0	fEnglish	[1] debate; dispute; argue [2] discuss; discourse; comment on; appraise; evaluate; talk about [3] theory; system of thoughts; dissertation; essay [4] mention; regard; consider [5] by; in terms of
U+8AD6.1	fUTF8	論
U+8AD6.1	fCNS11643	1-6F22
U+8AD6.1	fBig5	BDD7
U+8AD6.1	fGB	1-425B
U+8AD6.1	fR/S	 149.8
U+8AD6.1	fTotalStrokes	 15
U+8AD6.1	fCangjie	YROMB
U+8AD6.1	fFourCorner	08627
U+8AD6.1	fMandarin	lun2
U+8AD6.1	fEnglish	[1] alternative of @ for some phrases [2] a Chinese family name
U+8AD7.0	fUTF8	諗
U+8AD7.0	fCNS11643	2-5445
U+8AD7.0	fBig5	E7DC
U+8AD7.0	fGB	1-5A45
U+8AD7.0	fR/S	 149.8
U+8AD7.0	fTotalStrokes	 15
U+8AD7.0	fCangjie	YROIP
U+8AD7.0	fFourCorner	08632
U+8AD7.0	fHakka	 shim3; siim3; sim3
U+8AD7.0	fCantonese	sam2; nam2; lam2 (also)
U+8AD7.0	fMandarin	shen3; shen4; nie4
U+8AD7.0	fEnglish	[1] think of; consider; ponder over; think over [2] let know; tell; announce [3] remonstrate [4] hide; conceal
U+8AD8.0	fUTF8	諘
U+8AD8.0	fCNS11643	2-5449
U+8AD8.0	fBig5	E7E0
U+8AD8.0	fGB	5-7242
U+8AD8.0	fR/S	 149.8
U+8AD8.0	fTotalStrokes	 15
U+8AD8.0	fCangjie	YRQMV
U+8AD8.0	fFourCorner	05632
U+8AD8.0	fCantonese	biu2
U+8AD8.0	fMandarin	biao3
U+8AD9.0	fUTF8	諙
U+8AD9.0	fCNS11643	2-544D
U+8AD9.0	fBig5	E7E4
U+8AD9.0	fGB	E-3D7A
U+8AD9.0	fR/S	 149.8
U+8AD9.0	fTotalStrokes	 15
U+8AD9.0	fCangjie	YRHPA
U+8AD9.0	fFourCorner	02664
U+8AD9.0	fCantonese	waa6
U+8AD9.0	fMandarin	huai4
U+8ADA.0	fUTF8	諚
U+8ADA.0	fGB	E-3D7B
U+8ADA.0	fR/S	 149.8
U+8ADA.0	fTotalStrokes	 15
U+8ADA.0	fCangjie	YRJMO
U+8ADA.0	fFourCorner	0368
U+8ADA.0	fCantonese	pin4
U+8ADA.0	fMandarin	pian2
U+8ADA.0	fEnglish	(kokuji) command
U+8ADB.0	fUTF8	諛
U+8ADB.0	fCNS11643	1-6F26
U+8ADB.0	fBig5	BDDB
U+8ADB.0	fGB	1-5A44
U+8ADB.0	fR/S	 149.9
U+8ADB.0	fTotalStrokes	 16
U+8ADB.0	fCangjie	YRHXO
U+8ADB.0	fFourCorner	07637
U+8ADB.0	fHakka	 ji2; ji3; ri2; ri5; ji5; i2
U+8ADB.0	fCantonese	jyu4
U+8ADB.0	fMandarin	yu2; yu4
U+8ADB.0	fEnglish	 flatter; toady
U+8ADC.0	fUTF8	諜
U+8ADC.0	fCNS11643	1-723C
U+8ADC.0	fBig5	BFD2
U+8ADC.0	fGB	1-357D
U+8ADC.0	fR/S	 149.9
U+8ADC.0	fTotalStrokes	 16
U+8ADC.0	fCangjie	YRPTD
U+8ADC.0	fFourCorner	04694
U+8ADC.0	fHakka	  tiap8
U+8ADC.0	fCantonese	dip6
U+8ADC.0	fMandarin	die2; xie4; tie1
U+8ADC.0	fEnglish	[1] glib; garrulous; voluble [2] spying; espionage
U+8ADD.0	fUTF8	諝
U+8ADD.0	fCNS11643	2-5A51
U+8ADD.0	fBig5	EBA5
U+8ADD.0	fGB	3-7367
U+8ADD.0	fR/S	 149.9
U+8ADD.0	fTotalStrokes	 16
U+8ADD.0	fCangjie	YRNOB
U+8ADD.0	fFourCorner	07627
U+8ADD.0	fHakka	  si1
U+8ADD.0	fCantonese	seoi1; seoi2 (also)
U+8ADD.0	fMandarin	xu3; xu1
U+8ADD.0	fEnglish	[1] wisdom; sagacity [2] clever idea; stratagem
U+8ADE.0	fUTF8	諞
U+8ADE.0	fCNS11643	2-5A57
U+8ADE.0	fBig5	EBAB
U+8ADE.0	fGB	1-5A52
U+8ADE.0	fR/S	 149.9
U+8ADE.0	fTotalStrokes	 16
U+8ADE.0	fCangjie	YRHSB
U+8ADE.0	fFourCorner	03627
U+8ADE.0	fHakka	  pien1; pien3; pien5
U+8ADE.0	fCantonese	pin4; pin5
U+8ADE.0	fMandarin	pian2; bian3; bian4
U+8ADE.0	fEnglish	 quibble
U+8ADE.1	fUTF8	諞
U+8ADE.1	fCNS11643	2-5A57
U+8ADE.1	fBig5	EBAB
U+8ADE.1	fGB	1-5A52
U+8ADE.1	fR/S	 149.9
U+8ADE.1	fTotalStrokes	 16
U+8ADE.1	fCangjie	YRHSB
U+8ADE.1	fFourCorner	03627
U+8ADE.1	fMandarin	pian3
U+8ADE.1	fEnglish	 boast; show off
U+8ADF.0	fUTF8	諟
U+8ADF.0	fCNS11643	2-5A54
U+8ADF.0	fBig5	EBA8
U+8ADF.0	fGB	5-7256
U+8ADF.0	fR/S	 149.9
U+8ADF.0	fTotalStrokes	 16
U+8ADF.0	fCangjie	YRAMO
U+8ADF.0	fFourCorner	06681
U+8ADF.0	fHakka	 shi6; sii5; shi3; she5; shi5
U+8ADF.0	fCantonese	si6
U+8ADF.0	fMandarin	shi4; di4
U+8ADF.0	fEnglish	[1] yes; be [2] examine and correct
U+8AE0.0	fUTF8	諠
U+8AE0.0	fCNS11643	2-5A4C
U+8AE0.0	fBig5	EB7E
U+8AE0.0	fGB	E-3D7C
U+8AE0.0	fR/S	 149.9
U+8AE0.0	fTotalStrokes	 16
U+8AE0.0	fCangjie	YRJMM
U+8AE0.0	fFourCorner	03616
U+8AE0.0	fHakka	  sien1; sen1
U+8AE0.0	fCantonese	hyun1
U+8AE0.0	fMandarin	xuan1; xuan3
U+8AE0.0	fEnglish	 bawl; shout
U+8AE1.0	fUTF8	諡
U+8AE1.0	fCNS11643	2-5A58
U+8AE1.0	fBig5	EBAC
U+8AE1.0	fGB	E-3D7D
U+8AE1.0	fR/S	 149.9
U+8AE1.0	fTotalStrokes	 16
U+8AE1.0	fCangjie	YRCMT
U+8AE1.0	fFourCorner	08617
U+8AE1.0	fHakka	 shi6; sii5; si5; she5; shi5
U+8AE1.0	fCantonese	si3; si6
U+8AE1.0	fMandarin	shi4; yi4
U+8AE1.0	fEnglish	a posthumous title
U+8AE2.0	fUTF8	諢
U+8AE2.0	fCNS11643	2-5A4D
U+8AE2.0	fBig5	EBA1
U+8AE2.0	fGB	1-5A3B
U+8AE2.0	fR/S	 149.9
U+8AE2.0	fTotalStrokes	 16
U+8AE2.0	fCangjie	YRBJJ
U+8AE2.0	fFourCorner	07656
U+8AE2.0	fHakka	 fun5; fun6
U+8AE2.0	fCantonese	wan6
U+8AE2.0	fMandarin	hun4; wen4
U+8AE2.0	fEnglish	ridicule; derision; joke; jest
U+8AE3.0	fUTF8	諣
U+8AE3.0	fCNS11643	3-526A
U+8AE3.0	fGB	3-7340
U+8AE3.0	fR/S	  149.9
U+8AE3.0	fTotalStrokes	 16
U+8AE3.0	fCangjie	YRBBR
U+8AE3.0	fFourCorner	07620
U+8AE3.0	fCantonese	faa3
U+8AE3.0	fMandarin	hua4
U+8AE4.0	fUTF8	諤
U+8AE4.0	fCNS11643	2-5A53
U+8AE4.0	fBig5	EBA7
U+8AE4.0	fGB	1-5A4C
U+8AE4.0	fR/S	 149.9
U+8AE4.0	fTotalStrokes	 16
U+8AE4.0	fCangjie	YRRRS
U+8AE4.0	fFourCorner	06627
U+8AE4.0	fHakka	 ngok8
U+8AE4.0	fCantonese	ngok6; ok6 (also)
U+8AE4.0	fMandarin	e4
U+8AE4.0	fEnglish	honest speech; straightforward; frank comments
U+8AE5.0	fUTF8	諥
U+8AE5.0	fCNS11643	3-526B
U+8AE5.0	fGB	5-7257
U+8AE5.0	fR/S	  149.9
U+8AE5.0	fTotalStrokes	 16
U+8AE5.0	fCangjie	YRHJG
U+8AE5.0	fFourCorner	02610
U+8AE5.0	fHakka	 zhung5; zung5
U+8AE5.0	fCantonese	zung3
U+8AE5.0	fMandarin	zhong4
U+8AE6.0	fUTF8	諦
U+8AE6.0	fCNS11643	1-7237
U+8AE6.0	fBig5	BFCD
U+8AE6.0	fGB	1-5A50
U+8AE6.0	fR/S	 149.9
U+8AE6.0	fTotalStrokes	 16
U+8AE6.0	fCangjie	YRYBB
U+8AE6.0	fFourCorner	00627
U+8AE6.0	fHakka	  di5; dai2
U+8AE6.0	fCantonese	dai3
U+8AE6.0	fMandarin	di4; ti2
U+8AE6.0	fEnglish	[1] careful; attentive [2] truth; true (Buddhism) [3] tease; laugh at
U+8AE7.0	fUTF8	諧
U+8AE7.0	fCNS11643	1-723D
U+8AE7.0	fBig5	BFD3
U+8AE7.0	fGB	1-5033
U+8AE7.0	fR/S	 149.9
U+8AE7.0	fTotalStrokes	 16
U+8AE7.0	fCangjie	YRPPA
U+8AE7.0	fFourCorner	01661
U+8AE7.0	fHakka	  hai2; hai3
U+8AE7.0	fCantonese	haai4
U+8AE7.0	fMandarin	xie2
U+8AE7.0	fEnglish	[1] harmonious; congruous; in harmony; in accord [2] come to an agreement; settle [3] joke; jest; funny; humorous
U+8AE8.0	fUTF8	諨
U+8AE8.0	fCNS11643	2-5A59
U+8AE8.0	fBig5	EBAD
U+8AE8.0	fGB	E-3D7E
U+8AE8.0	fR/S	 149.9
U+8AE8.0	fTotalStrokes	 16
U+8AE8.0	fCangjie	YRMRW
U+8AE8.0	fFourCorner	01666
U+8AE8.0	fCantonese	fuk1
U+8AE8.0	fMandarin	fu2
U+8AE9.0	fUTF8	諩
U+8AE9.0	fCNS11643	3-4E6F
U+8AE9.0	fGB	E-3E21
U+8AE9.0	fR/S	 149.8
U+8AE9.0	fTotalStrokes	 15
U+8AE9.0	fCangjie	YRTTC
U+8AE9.0	fFourCorner	08610
U+8AE9.0	fHakka	 pu3
U+8AE9.0	fCantonese	pou2
U+8AE9.0	fMandarin	pu3
U+8AEA.0	fUTF8	諪
U+8AEA.0	fCNS11643	3-5267
U+8AEA.0	fGB	E-3E22
U+8AEA.0	fR/S	  149.9
U+8AEA.0	fTotalStrokes	 16
U+8AEA.0	fCangjie	YRYRN
U+8AEA.0	fFourCorner	00620
U+8AEA.0	fCantonese	ting4
U+8AEA.0	fMandarin	ting2
U+8AEB.0	fUTF8	諫
U+8AEB.0	fCNS11643	1-7239
U+8AEB.0	fBig5	BFCF
U+8AEB.0	fGB	1-5A49
U+8AEB.0	fR/S	 149.9
U+8AEB.0	fTotalStrokes	 16
U+8AEB.0	fCangjie	YRDWF
U+8AEB.0	fFourCorner	05696
U+8AEB.0	fHakka	  gan3;  kian5;  gan5; kian3; gian3
U+8AEB.0	fCantonese	gaan3
U+8AEB.0	fMandarin	jian4; jian1; lan4
U+8AEB.0	fEnglish	 remonstrate; admonish
U+8AEC.0	fUTF8	諬
U+8AEC.0	fCNS11643	3-526E
U+8AEC.0	fGB	E-3E23
U+8AEC.0	fR/S	 149.9
U+8AEC.0	fTotalStrokes	 16
U+8AEC.0	fCangjie	HUYMR
U+8AEC.0	fFourCorner	23600
U+8AEC.0	fCantonese	kai2
U+8AEC.0	fMandarin	qi3
U+8AEC.0	fEnglish	 open to begin to explain to inform a letter
U+8AED.0	fUTF8	諭
U+8AED.0	fCNS11643	1-7243
U+8AED.0	fBig5	BFD9
U+8AED.0	fGB	1-5A4D
U+8AED.0	fR/S	 149.9
U+8AED.0	fTotalStrokes	 16
U+8AED.0	fCangjie	YROMN
U+8AED.0	fFourCorner	08621
U+8AED.0	fHakka	 ji5; ji3; ri5; i5
U+8AED.0	fCantonese	jyu6
U+8AED.0	fMandarin	yu4; tou3
U+8AED.0	fEnglish	[1] notify or inform by a directive, edict etc.; instruct; tell [2] decree; edict; instruction; command
U+8AEE.0	fUTF8	諮
U+8AEE.0	fCNS11643	1-723E
U+8AEE.0	fBig5	BFD4
U+8AEE.0	fGB	1-5A51
U+8AEE.0	fR/S	 149.9
U+8AEE.0	fTotalStrokes	 16
U+8AEE.0	fCangjie	YRIOR
U+8AEE.0	fFourCorner	07668
U+8AEE.0	fHakka	 zih1; zu1; zii1
U+8AEE.0	fCantonese	zi1
U+8AEE.0	fMandarin	zi1
U+8AEE.0	fEnglish	[1] inquire; consult; confer [2] an official communication between offices of the same level
U+8AEF.0	fUTF8	諯
U+8AEF.0	fCNS11643	2-5A5B
U+8AEF.0	fBig5	EBAF
U+8AEF.0	fGB	3-7361
U+8AEF.0	fR/S	 149.9
U+8AEF.0	fTotalStrokes	 16
U+8AEF.0	fCangjie	YRUMB
U+8AEF.0	fFourCorner	02627
U+8AEF.0	fHakka	 zhon1; chon5; zon1; con5
U+8AEF.0	fCantonese	zyun1; cyun3; syun4
U+8AEF.0	fMandarin	chuan2; chuan4; xuan4; zhan4
U+8AEF.0	fEnglish	number yield
U+8AF0.0	fUTF8	諰
U+8AF0.0	fCNS11643	2-5A55
U+8AF0.0	fBig5	EBA9
U+8AF0.0	fGB	3-7360
U+8AF0.0	fR/S	 149.9
U+8AF0.0	fTotalStrokes	 16
U+8AF0.0	fCangjie	YRWP
U+8AF0.0	fFourCorner	06630
U+8AF0.0	fHakka	 sai3; se1
U+8AF0.0	fCantonese	saai2; si2 (also)
U+8AF0.0	fMandarin	xi3; ai1; sai3; shai1
U+8AF0.0	fEnglish	[1] speak frankly [2] apprehensive
U+8AF1.0	fUTF8	諱
U+8AF1.0	fCNS11643	1-723A
U+8AF1.0	fBig5	BFD0
U+8AF1.0	fGB	1-3B64
U+8AF1.0	fR/S	 149.9
U+8AF1.0	fTotalStrokes	 16
U+8AF1.0	fCangjie	YRDMQ
U+8AF1.0	fFourCorner	04656
U+8AF1.0	fHakka	 wui3; fi5; fui5; wui2; wi3; vui3; wi5; wui5
U+8AF1.0	fCantonese	wai5; wai3
U+8AF1.0	fMandarin	hui4
U+8AF1.0	fEnglish	[1] conceal; hide [2] shun; regard as taboo; taboo; avoid [3] name of a deceased elder member of the family
U+8AF2.0	fUTF8	諲
U+8AF2.0	fCNS11643	2-5A4E
U+8AF2.0	fBig5	EBA2
U+8AF2.0	fGB	3-735C
U+8AF2.0	fR/S	 149.9
U+8AF2.0	fTotalStrokes	 16
U+8AF2.0	fCangjie	YRMWG
U+8AF2.0	fFourCorner	01614
U+8AF2.0	fHakka	 jin1; jen1; rin1; ren1
U+8AF2.0	fCantonese	jan1
U+8AF2.0	fMandarin	yin1; zhen1
U+8AF2.0	fEnglish	 respect; venerate
U+8AF3.0	fUTF8	諳
U+8AF3.0	fCNS11643	1-7244
U+8AF3.0	fBig5	BFDA
U+8AF3.0	fGB	1-5A4F
U+8AF3.0	fR/S	 149.9
U+8AF3.0	fTotalStrokes	 16
U+8AF3.0	fCangjie	YRYTA
U+8AF3.0	fFourCorner	00661
U+8AF3.0	fHakka	 am1; am5
U+8AF3.0	fCantonese	am1; am2 (also); ngam1; ngam2 (also)
U+8AF3.0	fMandarin	an1; tou3; an4
U+8AF3.0	fEnglish	[1] versed in; familiar with; fully acquainted with; skilled in [2] recite
U+8AF4.0	fUTF8	諴
U+8AF4.0	fCNS11643	2-5A4F
U+8AF4.0	fBig5	EBA3
U+8AF4.0	fGB	3-735D
U+8AF4.0	fR/S	 149.9
U+8AF4.0	fTotalStrokes	 16
U+8AF4.0	fCangjie	YRIHR
U+8AF4.0	fFourCorner	03650
U+8AF4.0	fHakka	 ham2
U+8AF4.0	fCantonese	haam4
U+8AF4.0	fMandarin	xian2; gan1; yan2
U+8AF4.0	fEnglish	[1] in harmony; in agreement [2] sincerity; honestly
U+8AF5.0	fUTF8	諵
U+8AF5.0	fCNS11643	2-5A50
U+8AF5.0	fBig5	EBA4
U+8AF5.0	fGB	5-7254
U+8AF5.0	fR/S	 149.9
U+8AF5.0	fTotalStrokes	 16
U+8AF5.0	fCangjie	YRJBJ
U+8AF5.0	fFourCorner	04627
U+8AF5.0	fHakka	  nam2
U+8AF5.0	fCantonese	naam4; laam4 (also)
U+8AF5.0	fMandarin	nan2; nan4
U+8AF5.0	fEnglish	 chatter; gabble
U+8AF6.0	fUTF8	諶
U+8AF6.0	fCNS11643	1-7245
U+8AF6.0	fBig5	BFDB
U+8AF6.0	fGB	1-5A48
U+8AF6.0	fR/S	 149.9
U+8AF6.0	fTotalStrokes	 16
U+8AF6.0	fCangjie	YRTMV
U+8AF6.0	fFourCorner	04611
U+8AF6.0	fHakka	 shim1; zhim1; siim1; zihm1; shim3
U+8AF6.0	fCantonese	sam4
U+8AF6.0	fMandarin	chen2; shen2
U+8AF6.0	fEnglish	[1] sincere; honest; candid [2] a Chinese family name
U+8AF7.0	fUTF8	諷
U+8AF7.0	fCNS11643	1-7242
U+8AF7.0	fBig5	BFD8
U+8AF7.0	fGB	1-376D
U+8AF7.0	fR/S	 149.9
U+8AF7.0	fTotalStrokes	 16
U+8AF7.0	fCangjie	YRHNI
U+8AF7.0	fFourCorner	07610
U+8AF7.0	fHakka	  fung3;  fung1; fung5
U+8AF7.0	fCantonese	fung3
U+8AF7.0	fMandarin	feng3; feng1; feng4
U+8AF7.0	fEnglish	[1] recite; chant [2] satirize; mock; ridicule [3] admonish in a roundabout way
U+8AF8.0	fUTF8	諸
U+8AF8.0	fCNS11643	1-6E7A
U+8AF8.0	fBig5	BDD1
U+8AF8.0	fGB	1-566E
U+8AF8.0	fR/S	 149.9; 149.8
U+8AF8.0	fTotalStrokes	 16
U+8AF8.0	fCangjie	YRJKA
U+8AF8.0	fFourCorner	04660
U+8AF8.0	fHakka	  zhu1; zu1
U+8AF8.0	fCantonese	zyu1
U+8AF8.0	fMandarin	zhu1; zhe1; chu2
U+8AF8.0	fEnglish	[1] several; various; all [2] a contracted of @ [3] a Chinese family name
U+8AF9.0	fUTF8	諹
U+8AF9.0	fCNS11643	3-5269
U+8AF9.0	fGB	3-725D
U+8AF9.0	fR/S	  149.9
U+8AF9.0	fTotalStrokes	 16
U+8AF9.0	fCangjie	YRAMH
U+8AF9.0	fFourCorner	06620
U+8AF9.0	fCantonese	joeng4
U+8AF9.0	fMandarin	yang2
U+8AFA.0	fUTF8	諺
U+8AFA.0	fCNS11643	1-7238
U+8AFA.0	fBig5	BFCE
U+8AFA.0	fGB	1-5168
U+8AFA.0	fR/S	 149.9
U+8AFA.0	fTotalStrokes	 16
U+8AFA.0	fCangjie	YRYHH
U+8AFA.0	fFourCorner	00622
U+8AFA.0	fHakka	 ngien5; ngan3;  ngan5; ngien6; ngan6; ngian5
U+8AFA.0	fCantonese	jin6
U+8AFA.0	fMandarin	yan4
U+8AFA.0	fEnglish	 proverb; aphorism; saying; saw; adage
U+8AFB.0	fUTF8	諻
U+8AFB.0	fCNS11643	2-5A5C
U+8AFB.0	fBig5	EBB0
U+8AFB.0	fGB	3-7364
U+8AFB.0	fR/S	 149.9
U+8AFB.0	fTotalStrokes	 16
U+8AFB.0	fCangjie	YRHAG
U+8AFB.0	fFourCorner	06614
U+8AFB.0	fCantonese	wong4; waang4
U+8AFB.0	fMandarin	heng1; heng2
U+8AFC.0	fUTF8	諼
U+8AFC.0	fCNS11643	1-7246
U+8AFC.0	fBig5	BFDC
U+8AFC.0	fGB	1-5A4E
U+8AFC.0	fR/S	 149.9
U+8AFC.0	fTotalStrokes	 16
U+8AFC.0	fCangjie	YRBME
U+8AFC.0	fFourCorner	02647
U+8AFC.0	fHakka	 sien1; fon5
U+8AFC.0	fCantonese	hyun1; hyun2
U+8AFC.0	fMandarin	xuan1; shuan4; xuan3
U+8AFC.0	fEnglish	[1] forget [2] lie; cheat; deceive; fail in promise or appointment
U+8AFD.0	fUTF8	諽
U+8AFD.0	fCNS11643	3-5268
U+8AFD.0	fGB	3-735B
U+8AFD.0	fR/S	  149.9
U+8AFD.0	fTotalStrokes	 16
U+8AFD.0	fCangjie	YRTLJ
U+8AFD.0	fFourCorner	04650
U+8AFD.0	fCantonese	gaak3
U+8AFD.0	fMandarin	ge2
U+8AFE.0	fUTF8	諾
U+8AFE.0	fCNS11643	1-723F
U+8AFE.0	fBig5	BFD5
U+8AFE.0	fGB	1-4535
U+8AFE.0	fR/S	 149.9
U+8AFE.0	fTotalStrokes	 16
U+8AFE.0	fCangjie	YRTKR
U+8AFE.0	fFourCorner	04664
U+8AFE.0	fHakka	  nok7; lok7; nok8
U+8AFE.0	fCantonese	nok6; lok6 (also)
U+8AFE.0	fMandarin	nuo4
U+8AFE.0	fEnglish	[1] promise; pledge [2] assent
U+8AFF.0	fUTF8	諿
U+8AFF.0	fCNS11643	2-5A5A
U+8AFF.0	fBig5	EBAE
U+8AFF.0	fGB	3-735E
U+8AFF.0	fR/S	 149.9
U+8AFF.0	fTotalStrokes	 16
U+8AFF.0	fCangjie	YRRSJ
U+8AFF.0	fFourCorner	06641
U+8AFF.0	fCantonese	cap1; seoi2
U+8AFF.0	fMandarin	qi4; xu3
U+8B00.0	fUTF8	謀
U+8B00.0	fCNS11643	1-723B
U+8B00.0	fBig5	BFD1
U+8B00.0	fGB	1-4431
U+8B00.0	fR/S	 149.9
U+8B00.0	fTotalStrokes	 16
U+8B00.0	fCangjie	YRTMD
U+8B00.0	fFourCorner	04694
U+8B00.0	fHakka	  meu2; meu3; miu2; miau3
U+8B00.0	fCantonese	mau4
U+8B00.0	fMandarin	mou2
U+8B00.0	fEnglish	[1] plan; scheme; plot; design; devise [2] scheme; stratagem; plan; conspiracy [3] resourceful; astute [4] consult [5] seek; try to get
U+8B00.1	fUTF8	謀
U+8B00.1	fCNS11643	1-723B
U+8B00.1	fBig5	BFD1
U+8B00.1	fGB	1-4431
U+8B00.1	fR/S	 149.9
U+8B00.1	fTotalStrokes	 16
U+8B00.1	fCangjie	YRTMD
U+8B00.1	fFourCorner	04694
U+8B00.1	fMandarin	mou5
U+8B01.0	fUTF8	謁
U+8B01.0	fCNS11643	1-7240
U+8B01.0	fBig5	BFD6
U+8B01.0	fGB	1-5A4B
U+8B01.0	fR/S	 149.9; 149.8
U+8B01.0	fTotalStrokes	 16
U+8B01.0	fCangjie	YRAPV
U+8B01.0	fFourCorner	06627
U+8B01.0	fHakka	 jet7;  giet7; jat7; zat7; get7
U+8B01.0	fCantonese	jit3; kit3 (also); zit3
U+8B01.0	fMandarin	ye4; ai4
U+8B01.0	fEnglish	 have and audience with; see a superior
U+8B02.0	fUTF8	謂
U+8B02.0	fCNS11643	1-7241
U+8B02.0	fBig5	BFD7
U+8B02.0	fGB	1-4E3D
U+8B02.0	fR/S	 149.9
U+8B02.0	fTotalStrokes	 16
U+8B02.0	fCangjie	YRWB
U+8B02.0	fFourCorner	06627
U+8B02.0	fHakka	 wui3; wi5; wui5; wui6; vui5
U+8B02.0	fCantonese	wai6
U+8B02.0	fMandarin	wei4
U+8B02.0	fEnglish	[1] say; tell [2] call; name; be called; designate [3] think; be of opinion; assume [4] meaning; sense
U+8B03.0	fUTF8	謃
U+8B03.0	fCNS11643	3-664C
U+8B03.0	fGB	E-3E24
U+8B03.0	fR/S	 149.9
U+8B03.0	fTotalStrokes	 16
U+8B03.0	fCangjie	YRAHM
U+8B03.0	fFourCorner	0661
U+8B03.0	fCantonese	sing1
U+8B04.0	fUTF8	謄
U+8B04.0	fCNS11643	1-754E
U+8B04.0	fBig5	C1C3
U+8B04.0	fGB	1-4C5C
U+8B04.0	fR/S	 149.10
U+8B04.0	fTotalStrokes	 17
U+8B04.0	fCangjie	BFQR
U+8B04.0	fFourCorner	79261
U+8B04.0	fHakka	  ten2;  tien2; ten5
U+8B04.0	fCantonese	tang4
U+8B04.0	fMandarin	teng2
U+8B04.0	fEnglish	 copy; transcribe
U+8B05.0	fUTF8	謅
U+8B05.0	fCNS11643	2-5F50
U+8B05.0	fBig5	EEA4
U+8B05.0	fGB	1-565F
U+8B05.0	fR/S	 149.10
U+8B05.0	fTotalStrokes	 17
U+8B05.0	fCangjie	YRPUU
U+8B05.0	fFourCorner	07627
U+8B05.0	fHakka	  ceu1;  cau3; ziu1
U+8B05.0	fCantonese	zau1; zau3 (also)
U+8B05.0	fMandarin	zou1; zhou1; jie4; chao3; chou1
U+8B05.0	fEnglish	 talk nonsense
U+8B06.0	fUTF8	謆
U+8B06.0	fCNS11643	2-5F59
U+8B06.0	fBig5	EEAD
U+8B06.0	fGB	3-7373
U+8B06.0	fR/S	 149.10
U+8B06.0	fTotalStrokes	 17
U+8B06.0	fCangjie	YRHSM
U+8B06.0	fFourCorner	03627
U+8B06.0	fHakka	 sen5; shan5; san5; shen5
U+8B06.0	fCantonese	sin3
U+8B06.0	fMandarin	shan4
U+8B06.0	fEnglish	beguile; cajole
U+8B07.0	fUTF8	謇
U+8B07.0	fCNS11643	2-5F56
U+8B07.0	fBig5	EEAA
U+8B07.0	fGB	0-6540
U+8B07.0	fR/S	 149.10
U+8B07.0	fTotalStrokes	 17
U+8B07.0	fCangjie	JTCR
U+8B07.0	fFourCorner	30601
U+8B07.0	fHakka	 kien3; ken3
U+8B07.0	fCantonese	gin2; hin1 (also)
U+8B07.0	fMandarin	jian3
U+8B07.0	fEnglish	[1] stutter; stammer [2] speak out boldly
U+8B08.0	fUTF8	謈
U+8B08.0	fCNS11643	2-5F58
U+8B08.0	fBig5	EEAC
U+8B08.0	fGB	3-4833
U+8B08.0	fR/S	 149.10
U+8B08.0	fTotalStrokes	 17
U+8B08.0	fCangjie	ATCR
U+8B08.0	fFourCorner	60601
U+8B08.0	fHakka	 pok7; pau5; pau2
U+8B08.0	fCantonese	bok6
U+8B08.0	fMandarin	bo2; bao4; bo1; pao2; po4
U+8B08.0	fEnglish	 cry out from pain
U+8B09.0	fUTF8	謉
U+8B09.0	fCNS11643	3-5649
U+8B09.0	fGB	3-7365
U+8B09.0	fR/S	  149.10
U+8B09.0	fTotalStrokes	 17
U+8B09.0	fCangjie	YRHUI
U+8B09.0	fFourCorner	06610
U+8B09.0	fCantonese	kwai5
U+8B09.0	fMandarin	kui4
U+8B0A.0	fUTF8	謊
U+8B0A.0	fCNS11643	1-754B
U+8B0A.0	fBig5	C1C0
U+8B0A.0	fGB	1-3B51
U+8B0A.0	fR/S	 149.10; 149.9
U+8B0A.0	fTotalStrokes	 17
U+8B0A.0	fCangjie	YRTYU
U+8B0A.0	fFourCorner	04611
U+8B0A.0	fHakka	  fong1; fong3
U+8B0A.0	fCantonese	fong1; fong2 (also)
U+8B0A.0	fMandarin	huang3; huang1
U+8B0A.0	fEnglish	 lie
U+8B0B.0	fUTF8	謋
U+8B0B.0	fCNS11643	2-5F51
U+8B0B.0	fBig5	EEA5
U+8B0B.0	fGB	3-7370
U+8B0B.0	fR/S	 149.10; 149.11
U+8B0B.0	fTotalStrokes	 17
U+8B0B.0	fCangjie	YRNQD
U+8B0B.0	fFourCorner	05694
U+8B0B.0	fCantonese	waak6; faak3
U+8B0B.0	fMandarin	huo4; te4; xi4
U+8B0C.0	fUTF8	謌
U+8B0C.0	fCNS11643	3-564A
U+8B0C.0	fGB	E-3E25
U+8B0C.0	fR/S	 149.10
U+8B0C.0	fTotalStrokes	 17
U+8B0C.0	fCangjie	YRMRR
U+8B0C.0	fFourCorner	01620
U+8B0C.0	fHakka	  go1
U+8B0C.0	fCantonese	go1
U+8B0C.0	fMandarin	ge1
U+8B0C.0	fEnglish	slander; defame
U+8B0D.0	fUTF8	謍
U+8B0D.0	fCNS11643	2-5F57
U+8B0D.0	fBig5	EEAB
U+8B0D.0	fGB	3-7254
U+8B0D.0	fR/S	 149.10
U+8B0D.0	fTotalStrokes	 17
U+8B0D.0	fCangjie	FFBYR
U+8B0D.0	fFourCorner	99601
U+8B0D.0	fCantonese	jing4; gwang1
U+8B0D.0	fMandarin	ying2; hong1
U+8B0E.0	fUTF8	謎
U+8B0E.0	fCNS11643	1-7547
U+8B0E.0	fBig5	C1BC
U+8B0E.0	fGB	1-4355
U+8B0E.0	fR/S	 149.10
U+8B0E.0	fTotalStrokes	 17
U+8B0E.0	fCangjie	YRYFD
U+8B0E.0	fFourCorner	09639
U+8B0E.0	fHakka	 mi3; mi2;  mi5; mi6
U+8B0E.0	fCantonese	mai4; mai2 (also); mai6
U+8B0E.0	fMandarin	mi2; mei4; mi4
U+8B0E.0	fEnglish	 riddle; conundrum; puzzle; enigma; mystery
U+8B0F.0	fUTF8	謏
U+8B0F.0	fCNS11643	2-5F53
U+8B0F.0	fBig5	EEA7
U+8B0F.0	fGB	3-7363
U+8B0F.0	fR/S	 149.10
U+8B0F.0	fTotalStrokes	 17
U+8B0F.0	fCangjie	YRHXE
U+8B0F.0	fFourCorner	07647
U+8B0F.0	fHakka	 seu5; siau3
U+8B0F.0	fCantonese	sau2; siu2
U+8B0F.0	fMandarin	xiao3; sou3; sou4; suo1
U+8B0F.0	fEnglish	[1] small; little [2] induce
U+8B10.0	fUTF8	謐
U+8B10.0	fCNS11643	1-754F
U+8B10.0	fBig5	C1C4
U+8B10.0	fGB	1-5A57
U+8B10.0	fR/S	 149.10
U+8B10.0	fTotalStrokes	 17
U+8B10.0	fCangjie	YRPHT
U+8B10.0	fFourCorner	03617
U+8B10.0	fHakka	 met8; mit8
U+8B10.0	fCantonese	mat6
U+8B10.0	fMandarin	mi4
U+8B10.0	fEnglish	silent; calm; quiet; still; serene
U+8B11.0	fUTF8	謑
U+8B11.0	fCNS11643	2-5F4F
U+8B11.0	fBig5	EEA3
U+8B11.0	fGB	3-736F
U+8B11.0	fR/S	 149.10
U+8B11.0	fTotalStrokes	 17
U+8B11.0	fCangjie	YRBVK
U+8B11.0	fFourCorner	02634
U+8B11.0	fCantonese	hai5
U+8B11.0	fMandarin	xi4; xi1; xi3; xia4; xie4
U+8B11.0	fEnglish	shame; disgrace
U+8B12.0	fUTF8	謒
U+8B12.0	fCNS11643	2-5F54
U+8B12.0	fBig5	EEA8
U+8B12.0	fGB	E-3E26
U+8B12.0	fR/S	 149.10
U+8B12.0	fTotalStrokes	 17
U+8B12.0	fCangjie	YROIR
U+8B12.0	fFourCorner	08667
U+8B12.0	fHakka	  ciong1; cong1
U+8B12.0	fCantonese	coeng1
U+8B12.0	fMandarin	qiang1
U+8B12.0	fEnglish	[1] contradict [2] scold
U+8B13.0	fUTF8	謓
U+8B13.0	fCNS11643	2-5F5B
U+8B13.0	fBig5	EEAF
U+8B13.0	fGB	5-7260
U+8B13.0	fR/S	 149.10
U+8B13.0	fTotalStrokes	 17
U+8B13.0	fCangjie	YRJBC
U+8B13.0	fFourCorner	04681
U+8B13.0	fCantonese	can1; zan3
U+8B13.0	fMandarin	chen1; zhen4
U+8B14.0	fUTF8	謔
U+8B14.0	fCNS11643	2-5A52
U+8B14.0	fBig5	EBA6
U+8B14.0	fGB	1-5A4A
U+8B14.0	fR/S	 149.10
U+8B14.0	fTotalStrokes	 17
U+8B14.0	fCangjie	YRYPM
U+8B14.0	fFourCorner	01614
U+8B14.0	fHakka	  ngiok7;  hiok7; nak7
U+8B14.0	fCantonese	joek6
U+8B14.0	fMandarin	xue4; nuue4; ne4
U+8B14.0	fEnglish	[1] jest; joke; banter; tease [2] ridicule; satirise
U+8B15.0	fUTF8	謕
U+8B15.0	fCNS11643	2-5F55
U+8B15.0	fBig5	EEA9
U+8B15.0	fGB	5-7263
U+8B15.0	fR/S	 149.10
U+8B15.0	fTotalStrokes	 17
U+8B15.0	fCangjie	YRHYU
U+8B15.0	fFourCorner	02617
U+8B15.0	fCantonese	tai4
U+8B15.0	fMandarin	ti2; si1; ti1
U+8B16.0	fUTF8	謖
U+8B16.0	fCNS11643	2-5F4E
U+8B16.0	fBig5	EEA2
U+8B16.0	fGB	1-5A55
U+8B16.0	fR/S	 149.10
U+8B16.0	fTotalStrokes	 17
U+8B16.0	fCangjie	YRWCE
U+8B16.0	fFourCorner	06647
U+8B16.0	fHakka	  suk7
U+8B16.0	fCantonese	suk1
U+8B16.0	fMandarin	su4
U+8B16.0	fEnglish	[1] rise; raise [2] folded; closed
U+8B17.0	fUTF8	謗
U+8B17.0	fCNS11643	1-7548
U+8B17.0	fBig5	C1BD
U+8B17.0	fGB	1-3079
U+8B17.0	fR/S	 149.10
U+8B17.0	fTotalStrokes	 17
U+8B17.0	fCangjie	YRYBS
U+8B17.0	fFourCorner	00627
U+8B17.0	fHakka	  bong3;  bong5
U+8B17.0	fCantonese	pong3; bong3
U+8B17.0	fMandarin	bang4; bang1
U+8B17.0	fEnglish	 slander; libel; condemn
U+8B18.0	fUTF8	謘
U+8B18.0	fCNS11643	2-5F4D
U+8B18.0	fBig5	EEA1
U+8B18.0	fGB	5-7267
U+8B18.0	fR/S	 149.10
U+8B18.0	fTotalStrokes	 17
U+8B18.0	fCangjie	YRSYJ
U+8B18.0	fFourCorner	07641
U+8B18.0	fCantonese	ci4; zi6
U+8B18.0	fMandarin	chi2; zhi4
U+8B19.0	fUTF8	謙
U+8B19.0	fCNS11643	1-7549
U+8B19.0	fBig5	C1BE
U+8B19.0	fGB	1-472B
U+8B19.0	fR/S	 149.10
U+8B19.0	fTotalStrokes	 17
U+8B19.0	fCangjie	YRTXC
U+8B19.0	fFourCorner	08637
U+8B19.0	fHakka	  kiam1; hiam1
U+8B19.0	fCantonese	him1; him3
U+8B19.0	fMandarin	qian1; qian4; zhan4; qian3; qie4
U+8B19.0	fEnglish	humble; modest; retiring; self-effacing
U+8B1A.0	fUTF8	謚
U+8B1A.0	fCNS11643	2-5F5C
U+8B1A.0	fBig5	EEB0
U+8B1A.0	fGB	1-5A56
U+8B1A.0	fR/S	 149.10
U+8B1A.0	fTotalStrokes	 17
U+8B1A.0	fCangjie	YRTCT
U+8B1A.0	fFourCorner	08617
U+8B1A.0	fHakka	 jit7; rit7; shi6; shi5
U+8B1A.0	fCantonese	si3; si6
U+8B1A.0	fMandarin	shi4; yi4; xi4
U+8B1A.0	fEnglish	 confer posthumous titles
U+8B1B.0	fUTF8	講
U+8B1B.0	fCNS11643	1-754A
U+8B1B.0	fBig5	C1BF
U+8B1B.0	fGB	1-3D32
U+8B1B.0	fR/S	 149.10
U+8B1B.0	fTotalStrokes	 17
U+8B1B.0	fCangjie	YRTTB
U+8B1B.0	fFourCorner	05657
U+8B1B.0	fHakka	  gong3
U+8B1B.0	fCantonese	gong2
U+8B1B.0	fMandarin	jiang3; gou4
U+8B1B.0	fEnglish	[1] speak; talk; say [2] pay particular attention to; be particular about; elaborate about [3] explain; explicate [4] as far as something is concerned; when it comes to; as to; as regards [5] have recourse to
U+8B1C.0	fUTF8	謜
U+8B1C.0	fCNS11643	2-5F5A
U+8B1C.0	fBig5	EEAE
U+8B1C.0	fGB	3-736C
U+8B1C.0	fR/S	 149.10
U+8B1C.0	fTotalStrokes	 17
U+8B1C.0	fCangjie	YRMHF
U+8B1C.0	fFourCorner	01696
U+8B1C.0	fHakka	 chin1; shim3
U+8B1C.0	fCantonese	jyun4; cyun4
U+8B1C.0	fMandarin	yuan4; jie1; quan2; quan1; yuan2; yung4
U+8B1D.0	fUTF8	謝
U+8B1D.0	fCNS11643	1-754D
U+8B1D.0	fBig5	C1C2
U+8B1D.0	fGB	1-503B
U+8B1D.0	fR/S	 149.10
U+8B1D.0	fTotalStrokes	 17
U+8B1D.0	fCangjie	YRHHI
U+8B1D.0	fFourCorner	04600
U+8B1D.0	fHakka	 cia5; cia6; cia3
U+8B1D.0	fCantonese	ze6
U+8B1D.0	fMandarin	xie4
U+8B1D.0	fEnglish	[1] thank [2] decline [3] fade; wither [4] a Chinese family name
U+8B1D.1	fUTF8	謝
U+8B1D.1	fCNS11643	1-754D
U+8B1D.1	fBig5	C1C2
U+8B1D.1	fGB	1-503B
U+8B1D.1	fR/S	 149.10
U+8B1D.1	fTotalStrokes	 17
U+8B1D.1	fCangjie	YRHHI
U+8B1D.1	fFourCorner	04600
U+8B1D.1	fMandarin	xie5
U+8B1E.0	fUTF8	謞
U+8B1E.0	fCNS11643	2-5F4C
U+8B1E.0	fBig5	EE7E
U+8B1E.0	fGB	3-7371
U+8B1E.0	fR/S	 149.10
U+8B1E.0	fTotalStrokes	 17
U+8B1E.0	fCangjie	YRYRB
U+8B1E.0	fFourCorner	00627
U+8B1E.0	fHakka	 kok7;  hau5
U+8B1E.0	fCantonese	hok6; haau1
U+8B1E.0	fMandarin	xue4; xiao4; he4
U+8B1E.0	fEnglish	 slander
U+8B1F.0	fUTF8	謟
U+8B1F.0	fCNS11643	3-5648
U+8B1F.0	fGB	3-736E
U+8B1F.0	fR/S	 149.10
U+8B1F.0	fTotalStrokes	 17
U+8B1F.0	fCangjie	YRBHX
U+8B1F.0	fFourCorner	02677
U+8B1F.0	fHakka	  tau1; to1
U+8B1F.0	fCantonese	tou1
U+8B1F.0	fMandarin	tao1
U+8B1F.0	fEnglish	confusing; uncertain
U+8B20.0	fUTF8	謠
U+8B20.0	fCNS11643	1-754C
U+8B20.0	fBig5	C1C1
U+8B20.0	fGB	E-3E27
U+8B20.0	fR/S	 149.10
U+8B20.0	fTotalStrokes	 17
U+8B20.0	fCangjie	YRBOU
U+8B20.0	fFourCorner	07672
U+8B20.0	fHakka	 jau2; rau2; jau3; jeu2
U+8B20.0	fCantonese	jiu4
U+8B20.0	fMandarin	yao2
U+8B20.0	fEnglish	[1] folksong; ballad; song [2] rumor
U+8B21.0	fUTF8	謡
U+8B21.0	fGB	1-5225
U+8B21.0	fR/S	 149.10; 149.9
U+8B21.0	fTotalStrokes	 17
U+8B21.0	fCangjie	YRBOU
U+8B21.0	fCantonese	jiu4
U+8B21.0	fMandarin	yao2
U+8B21.0	fEnglish	 sing folksong; ballad rumor
U+8B22.0	fUTF8	謢
U+8B22.0	fCNS11643	2-5F52
U+8B22.0	fBig5	EEA6
U+8B22.0	fGB	E-3E28
U+8B22.0	fR/S	 149.10
U+8B22.0	fTotalStrokes	 17
U+8B22.0	fCangjie	YROGE
U+8B22.0	fFourCorner	00647
U+8B22.0	fCantonese	wu6; lou1
U+8B22.0	fMandarin	hu4
U+8B23.0	fUTF8	謣
U+8B23.0	fCNS11643	2-6347
U+8B23.0	fBig5	F0DC
U+8B23.0	fGB	3-7377
U+8B23.0	fR/S	 149.11
U+8B23.0	fTotalStrokes	 18
U+8B23.0	fCangjie	YRMBS
U+8B23.0	fFourCorner	07627
U+8B23.0	fCantonese	jyu4
U+8B23.0	fMandarin	yu2; xu1; yu1
U+8B24.0	fUTF8	謤
U+8B24.0	fCNS11643	2-6355
U+8B24.0	fBig5	F0EA
U+8B24.0	fGB	3-7375
U+8B24.0	fR/S	 149.11
U+8B24.0	fTotalStrokes	 18
U+8B24.0	fCangjie	YRMWF
U+8B24.0	fFourCorner	01691
U+8B24.0	fCantonese	biu1; piu1
U+8B24.0	fMandarin	biao1; piao1
U+8B25.0	fUTF8	謥
U+8B25.0	fCNS11643	2-6350
U+8B25.0	fBig5	F0E5
U+8B25.0	fGB	5-726C
U+8B25.0	fR/S	 149.11
U+8B25.0	fTotalStrokes	 18
U+8B25.0	fCangjie	YRHWP
U+8B25.0	fFourCorner	06630
U+8B25.0	fCantonese	cung3
U+8B25.0	fMandarin	cong4; song4
U+8B26.0	fUTF8	謦
U+8B26.0	fCNS11643	2-6352
U+8B26.0	fBig5	F0E7
U+8B26.0	fGB	0-7625
U+8B26.0	fR/S	 149.11
U+8B26.0	fTotalStrokes	 18
U+8B26.0	fCangjie	GEYMR
U+8B26.0	fFourCorner	47601
U+8B26.0	fHakka	 kin5;  kin3
U+8B26.0	fCantonese	hing3; king2
U+8B26.0	fMandarin	qing4; qing3; qiug3
U+8B26.0	fEnglish	 cough slightly
U+8B27.0	fUTF8	謧
U+8B27.0	fCNS11643	2-6346
U+8B27.0	fBig5	F0DB
U+8B27.0	fGB	3-7372
U+8B27.0	fR/S	 149.11
U+8B27.0	fTotalStrokes	 18
U+8B27.0	fCangjie	YRYUB
U+8B27.0	fFourCorner	00627
U+8B27.0	fHakka	  li2
U+8B27.0	fCantonese	lei4; lai4
U+8B27.0	fMandarin	li2
U+8B27.0	fEnglish	[1] jest [2] chaff
U+8B28.0	fUTF8	謨
U+8B28.0	fCNS11643	1-773E
U+8B28.0	fBig5	C2D3
U+8B28.0	fGB	1-5A53
U+8B28.0	fR/S	 149.11
U+8B28.0	fTotalStrokes	 18
U+8B28.0	fCangjie	YRTAK
U+8B28.0	fFourCorner	04634
U+8B28.0	fHakka	 mu2; mo3
U+8B28.0	fCantonese	mou4
U+8B28.0	fMandarin	mo2; mo4; mu2; mu4
U+8B28.0	fEnglish	[1] scheme; plan; course of action [2] be without; have not; without; lack
U+8B29.0	fUTF8	謩
U+8B29.0	fCNS11643	3-592A
U+8B29.0	fGB	E-3E29
U+8B29.0	fR/S	  149.11
U+8B29.0	fTotalStrokes	 18
U+8B29.0	fCangjie	TAKR
U+8B29.0	fFourCorner	44600
U+8B29.0	fHakka	 mu2
U+8B29.0	fCantonese	mou4
U+8B29.0	fMandarin	mo4; mo2
U+8B2A.0	fUTF8	謪
U+8B2A.0	fCNS11643	2-6345
U+8B2A.0	fBig5	F0DA
U+8B2A.0	fGB	5-726F
U+8B2A.0	fR/S	 149.11
U+8B2A.0	fTotalStrokes	 18
U+8B2A.0	fCangjie	YRYCB
U+8B2A.0	fFourCorner	00627
U+8B2A.0	fHakka	 song1; shong1; zak7; sak7
U+8B2A.0	fCantonese	soeng1
U+8B2A.0	fMandarin	shang1
U+8B2A.0	fEnglish	[1] censure; reproach [2] punish; penalize [3] one's fault [4] exile; banish (an official) to a distant place
U+8B2B.0	fUTF8	謫
U+8B2B.0	fCNS11643	1-7741
U+8B2B.0	fBig5	C2D6
U+8B2B.0	fGB	1-5A58
U+8B2B.0	fR/S	 149.11
U+8B2B.0	fTotalStrokes	 18
U+8B2B.0	fCangjie	YRYCB
U+8B2B.0	fFourCorner	00627
U+8B2B.0	fHakka	 zak7; shong1
U+8B2B.0	fCantonese	zaak6
U+8B2B.0	fMandarin	zhe2; ze2; di4
U+8B2B.0	fEnglish	 charge; blame disgrace demote
U+8B2C.0	fUTF8	謬
U+8B2C.0	fCNS11643	1-7740
U+8B2C.0	fBig5	C2D5
U+8B2C.0	fGB	1-437D
U+8B2C.0	fR/S	 149.11
U+8B2C.0	fTotalStrokes	 18
U+8B2C.0	fCangjie	YRSMH
U+8B2C.0	fFourCorner	07622
U+8B2C.0	fHakka	  meu3; miu3; miau3
U+8B2C.0	fCantonese	mau6
U+8B2C.0	fMandarin	miu4; niu4
U+8B2C.0	fEnglish	[1] error; erroneous; incorrect; wrong; mistaken [2] absurd; unreasonable
U+8B2D.0	fUTF8	謭
U+8B2D.0	fCNS11643	3-5926
U+8B2D.0	fGB	1-5A59
U+8B2D.0	fR/S	 149.11
U+8B2D.0	fTotalStrokes	 18
U+8B2D.0	fCangjie	YRTBH
U+8B2D.0	fFourCorner	08620
U+8B2D.0	fHakka	 zien3
U+8B2D.0	fCantonese	zin2
U+8B2D.0	fMandarin	jian3
U+8B2D.0	fEnglish	 shallow stupid
U+8B2E.0	fUTF8	謮
U+8B2E.0	fCNS11643	2-6354
U+8B2E.0	fBig5	F0E9
U+8B2E.0	fGB	5-7241
U+8B2E.0	fR/S	 149.11
U+8B2E.0	fTotalStrokes	 18
U+8B2E.0	fCangjie	YRQMC
U+8B2E.0	fFourCorner	05686
U+8B2E.0	fCantonese	zaak3
U+8B2E.0	fMandarin	ze2
U+8B2F.0	fUTF8	謯
U+8B2F.0	fCNS11643	2-634C
U+8B2F.0	fBig5	F0E1
U+8B2F.0	fGB	3-7378
U+8B2F.0	fR/S	 149.11
U+8B2F.0	fTotalStrokes	 18
U+8B2F.0	fCangjie	YRYPM
U+8B2F.0	fFourCorner	01617
U+8B2F.0	fHakka	  za1
U+8B2F.0	fCantonese	ze1; zo3; zaa2
U+8B2F.0	fMandarin	jie1; zha1; ju1; zha3; zu3
U+8B2F.0	fEnglish	[1] afraid to speak out [2] angry [3] an exclamation of regret and surprise [4] chant or sing
U+8B30.0	fUTF8	謰
U+8B30.0	fCNS11643	2-6349
U+8B30.0	fBig5	F0DE
U+8B30.0	fGB	3-733D
U+8B30.0	fR/S	 149.11
U+8B30.0	fTotalStrokes	 18
U+8B30.0	fCangjie	YRYJJ
U+8B30.0	fFourCorner	05630
U+8B30.0	fCantonese	lin4
U+8B30.0	fMandarin	lian2; lian3
U+8B30.0	fEnglish	[1] talk vaguely [2] the combination of two closely related characters which forms an inseparable thought unit
U+8B31.0	fUTF8	謱
U+8B31.0	fCNS11643	2-634F
U+8B31.0	fBig5	F0E4
U+8B31.0	fGB	3-7366
U+8B31.0	fR/S	 149.11
U+8B31.0	fTotalStrokes	 18
U+8B31.0	fCangjie	YRLLV
U+8B31.0	fFourCorner	05644
U+8B31.0	fHakka	  leu2
U+8B31.0	fCantonese	lou4 (also); leoi5; lau4; lau1
U+8B31.0	fMandarin	lou2; lou3; l3
U+8B31.0	fEnglish	[1] talkative [2] chatter
U+8B32.0	fUTF8	謲
U+8B32.0	fCNS11643	3-5924
U+8B32.0	fGB	3-7359
U+8B32.0	fR/S	  149.11
U+8B32.0	fTotalStrokes	 18
U+8B32.0	fCangjie	YRIIH
U+8B32.0	fFourCorner	03620
U+8B32.0	fHakka	 cam1
U+8B32.0	fCantonese	cam3
U+8B32.0	fMandarin	can1
U+8B33.0	fUTF8	謳
U+8B33.0	fCNS11643	2-6348
U+8B33.0	fBig5	F0DD
U+8B33.0	fGB	1-5A29
U+8B33.0	fR/S	 149.11
U+8B33.0	fTotalStrokes	 18
U+8B33.0	fCangjie	YRSRR
U+8B33.0	fFourCorner	01616
U+8B33.0	fHakka	  eu1; iu1
U+8B33.0	fCantonese	au1; ngau1 (also)
U+8B33.0	fMandarin	ou1; xu1; xu2; yu1
U+8B33.0	fEnglish	 sing; chant
U+8B34.0	fUTF8	謴
U+8B34.0	fCNS11643	3-5928
U+8B34.0	fGB	E-3E2A
U+8B34.0	fR/S	  149.11
U+8B34.0	fTotalStrokes	 18
U+8B34.0	fCangjie	YRWJC
U+8B34.0	fFourCorner	07686
U+8B34.0	fCantonese	gwan3
U+8B34.0	fMandarin	gun4; guan4
U+8B35.0	fUTF8	謵
U+8B35.0	fCNS11643	2-634A
U+8B35.0	fBig5	F0DF
U+8B35.0	fGB	3-737C
U+8B35.0	fR/S	 149.11
U+8B35.0	fTotalStrokes	 18
U+8B35.0	fCangjie	YRSMA
U+8B35.0	fFourCorner	07662
U+8B35.0	fCantonese	zaap6; cip3
U+8B35.0	fMandarin	xi2
U+8B36.0	fUTF8	謶
U+8B36.0	fCNS11643	2-6353
U+8B36.0	fBig5	F0E8
U+8B36.0	fGB	5-726E
U+8B36.0	fR/S	 149.11
U+8B36.0	fTotalStrokes	 18
U+8B36.0	fCangjie	YRITF
U+8B36.0	fFourCorner	00637
U+8B36.0	fHakka	 za1; shok7; sok7; zha1
U+8B36.0	fCantonese	zoek3; ze1; syu3; sok3
U+8B36.0	fMandarin	zhuo2; shu4; zhe1
U+8B36.0	fEnglish	[1] reprimand [2] hope for [3] deceive
U+8B37.0	fUTF8	謷
U+8B37.0	fCNS11643	2-6351
U+8B37.0	fBig5	F0E6
U+8B37.0	fGB	3-7256
U+8B37.0	fR/S	 149.11
U+8B37.0	fTotalStrokes	 18
U+8B37.0	fCangjie	GKYMR
U+8B37.0	fFourCorner	58601
U+8B37.0	fHakka	  ngau2; ngau3
U+8B37.0	fCantonese	ngou4; ngou6; ou4 (also); ou6 (also)
U+8B37.0	fMandarin	ao2; ao4
U+8B37.0	fEnglish	[1] slander; abuse; revile [2] huge; big; massive; great [3] worthless fellow [4] sounds of weeping
U+8B38.0	fUTF8	謸
U+8B38.0	fCNS11643	3-5923
U+8B38.0	fGB	3-7369
U+8B38.0	fR/S	  149.11
U+8B38.0	fTotalStrokes	 18; 17
U+8B38.0	fCangjie	YRGSK
U+8B38.0	fFourCorner	08640
U+8B38.0	fCantonese	ngou6; ou6 (also)
U+8B38.0	fMandarin	ao2; ao4
U+8B39.0	fUTF8	謹
U+8B39.0	fCNS11643	1-773F
U+8B39.0	fBig5	C2D4
U+8B39.0	fGB	1-3D77
U+8B39.0	fR/S	 149.11; 149.10
U+8B39.0	fTotalStrokes	 18
U+8B39.0	fCangjie	YRTLM
U+8B39.0	fFourCorner	04614
U+8B39.0	fHakka	  giun3
U+8B39.0	fCantonese	gan2
U+8B39.0	fMandarin	jin3; qin2
U+8B39.0	fEnglish	[1] prudent; cautious; attentive; careful [2] respectful; reverent; deferential; sincerely
U+8B3A.0	fUTF8	謺
U+8B3A.0	fCNS11643	2-6358
U+8B3A.0	fBig5	F0ED
U+8B3A.0	fGB	3-7255
U+8B3A.0	fR/S	 149.11
U+8B3A.0	fTotalStrokes	 18
U+8B3A.0	fCangjie	GIYMR
U+8B3A.0	fFourCorner	45601
U+8B3A.0	fHakka	 chip7; tsiip7
U+8B3A.0	fCantonese	zip3; zap1
U+8B3A.0	fMandarin	zhe2; die2; zhi2
U+8B3A.0	fEnglish	[1] mimic [2] talkative
U+8B3B.0	fUTF8	謻
U+8B3B.0	fCNS11643	2-6356
U+8B3B.0	fBig5	F0EB
U+8B3B.0	fGB	3-7379
U+8B3B.0	fR/S	 149.11
U+8B3B.0	fTotalStrokes	 18
U+8B3B.0	fCangjie	YRHDN
U+8B3B.0	fFourCorner	07627
U+8B3B.0	fHakka	 ji2; ri2
U+8B3B.0	fCantonese	ci2 (also); ji4
U+8B3B.0	fMandarin	yi2; chi2; chi3
U+8B3B.0	fEnglish	 side door
U+8B3C.0	fUTF8	謼
U+8B3C.0	fCNS11643	2-634D
U+8B3C.0	fBig5	F0E2
U+8B3C.0	fGB	E-3E2B
U+8B3C.0	fR/S	 149.11
U+8B3C.0	fTotalStrokes	 18
U+8B3C.0	fCangjie	YRYPD
U+8B3C.0	fFourCorner	01649
U+8B3C.0	fHakka	 fu5; fu1
U+8B3C.0	fCantonese	fu1
U+8B3C.0	fMandarin	hu4; hu1; xiao1
U+8B3C.0	fEnglish	 shout mourn to invoke
U+8B3D.0	fUTF8	謽
U+8B3D.0	fCNS11643	2-6357
U+8B3D.0	fBig5	F0EC
U+8B3D.0	fGB	5-714C
U+8B3D.0	fR/S	 149.11
U+8B3D.0	fTotalStrokes	 18
U+8B3D.0	fCangjie	NIYMR
U+8B3D.0	fFourCorner	13601
U+8B3D.0	fCantonese	koeng5
U+8B3D.0	fMandarin	jiang4
U+8B3E.0	fUTF8	謾
U+8B3E.0	fCNS11643	2-634E
U+8B3E.0	fBig5	F0E3
U+8B3E.0	fGB	1-4321
U+8B3E.0	fR/S	 149.11
U+8B3E.0	fTotalStrokes	 18
U+8B3E.0	fCangjie	YRAWE
U+8B3E.0	fFourCorner	06647
U+8B3E.0	fHakka	  man2;  man3; man5; man6
U+8B3E.0	fCantonese	maan4; maan6
U+8B3E.0	fMandarin	man4; mian2
U+8B3E.0	fEnglish	[1] scorn; insult [2] disrespectful; rude
U+8B3E.1	fUTF8	謾
U+8B3E.1	fCNS11643	2-634E
U+8B3E.1	fBig5	F0E3
U+8B3E.1	fGB	1-4321
U+8B3E.1	fR/S	 149.11
U+8B3E.1	fTotalStrokes	 18
U+8B3E.1	fCangjie	YRAWE
U+8B3E.1	fFourCorner	06647
U+8B3E.1	fMandarin	man2
U+8B3E.1	fEnglish	 deceive
U+8B3F.0	fUTF8	謿
U+8B3F.0	fCNS11643	3-5B34
U+8B3F.0	fGB	5-7278
U+8B3F.0	fR/S	 149.12
U+8B3F.0	fTotalStrokes	 19
U+8B3F.0	fCangjie	YRJJB
U+8B3F.0	fFourCorner	07620
U+8B3F.0	fHakka	 sau3; zau1
U+8B3F.0	fCantonese	zaau1
U+8B3F.0	fMandarin	chao2; zhao1
U+8B3F.0	fEnglish	ridicule; deride; scorn; jeer at
U+8B40.0	fUTF8	譀
U+8B40.0	fCNS11643	2-6726
U+8B40.0	fBig5	F2F9
U+8B40.0	fGB	3-737A
U+8B40.0	fR/S	 149.12
U+8B40.0	fTotalStrokes	 19
U+8B40.0	fCangjie	YRMJK
U+8B40.0	fFourCorner	08640
U+8B40.0	fCantonese	ham6; ham3
U+8B40.0	fMandarin	han4; xia2; xian2; xian4
U+8B41.0	fUTF8	譁
U+8B41.0	fCNS11643	1-787A
U+8B41.0	fBig5	C3CF
U+8B41.0	fGB	E-3E2C
U+8B41.0	fR/S	 149.12
U+8B41.0	fTotalStrokes	 19
U+8B41.0	fCangjie	YRTMJ
U+8B41.0	fFourCorner	04654
U+8B41.0	fHakka	 fa2; wa2
U+8B41.0	fCantonese	waa1
U+8B41.0	fMandarin	hua2; hua1; wa2
U+8B41.0	fEnglish	noise; tumult; uproar; clamor; hubbub
U+8B42.0	fUTF8	譂
U+8B42.0	fCNS11643	2-672D
U+8B42.0	fBig5	F341
U+8B42.0	fGB	E-3E2D
U+8B42.0	fR/S	 149.12
U+8B42.0	fTotalStrokes	 19
U+8B42.0	fCangjie	YRRRJ
U+8B42.0	fFourCorner	06656
U+8B42.0	fHakka	 shan3; san3; sen3; shan5; shen5; shen3
U+8B42.0	fCantonese	cin2; sin6
U+8B42.0	fMandarin	chan3
U+8B42.0	fEnglish	[1] incoherent talk [2] incorrect expressions
U+8B43.0	fUTF8	譃
U+8B43.0	fCNS11643	3-5B35
U+8B43.0	fGB	5-7269
U+8B43.0	fR/S	 149.12
U+8B43.0	fTotalStrokes	 19
U+8B43.0	fCangjie	YRYPM
U+8B43.0	fFourCorner	01610
U+8B43.0	fCantonese	jyu4
U+8B43.0	fMandarin	xu1
U+8B44.0	fUTF8	譄
U+8B44.0	fCNS11643	4-613A
U+8B44.0	fGB	5-7324
U+8B44.0	fR/S	  149.12
U+8B44.0	fTotalStrokes	 19
U+8B44.0	fCangjie	YRCWA
U+8B44.0	fFourCorner	08660
U+8B44.0	fHakka	  zen1
U+8B44.0	fCantonese	zang1
U+8B44.0	fMandarin	zeng1
U+8B44.0	fEnglish	 add; state further
U+8B45.0	fUTF8	譅
U+8B45.0	fCNS11643	2-6C3E
U+8B45.0	fBig5	F64F
U+8B45.0	fGB	3-7430
U+8B45.0	fR/S	 149.12; 149.14
U+8B45.0	fTotalStrokes	 19
U+8B45.0	fCangjie	YRSIM
U+8B45.0	fFourCorner	07611
U+8B45.0	fCantonese	sap1
U+8B45.0	fMandarin	se4; nao3
U+8B45.0	fEnglish	wriggling
U+8B46.0	fUTF8	譆
U+8B46.0	fCNS11643	1-7923
U+8B46.0	fBig5	C3D6
U+8B46.0	fGB	E-3E2E
U+8B46.0	fR/S	 149.12
U+8B46.0	fTotalStrokes	 19
U+8B46.0	fCangjie	YRGRR
U+8B46.0	fFourCorner	04661
U+8B46.0	fHakka	 hi1
U+8B46.0	fCantonese	hei1
U+8B46.0	fMandarin	xi1
U+8B46.0	fEnglish	scream
U+8B47.0	fUTF8	譇
U+8B47.0	fCNS11643	2-634B
U+8B47.0	fBig5	F0E0
U+8B47.0	fGB	3-7376
U+8B47.0	fR/S	 149.12
U+8B47.0	fTotalStrokes	 19
U+8B47.0	fCangjie	YRKJA
U+8B47.0	fFourCorner	04664
U+8B47.0	fHakka	 cha1; ca1
U+8B47.0	fCantonese	zaa1; ce1
U+8B47.0	fMandarin	she1; cha1; zha1
U+8B47.0	fEnglish	[1] speak hesitatingly; afraid to speak out [2] angry; disturbed in mind
U+8B48.0	fUTF8	譈
U+8B48.0	fCNS11643	2-6724
U+8B48.0	fBig5	F2F7
U+8B48.0	fGB	5-7321
U+8B48.0	fR/S	 149.12
U+8B48.0	fTotalStrokes	 19
U+8B48.0	fCangjie	YRYDK
U+8B48.0	fFourCorner	08640
U+8B48.0	fHakka	 tui5; dui5
U+8B48.0	fCantonese	deoi6; teoi6
U+8B48.0	fMandarin	dui4
U+8B48.0	fEnglish	dislike
U+8B49.0	fUTF8	證
U+8B49.0	fCNS11643	1-787D
U+8B49.0	fBig5	C3D2
U+8B49.0	fGB	1-5624
U+8B49.0	fR/S	 149.12
U+8B49.0	fTotalStrokes	 19
U+8B49.0	fCangjie	YRNOT
U+8B49.0	fFourCorner	02618
U+8B49.0	fHakka	 zhin5; zin5; zihn5
U+8B49.0	fCantonese	zing3
U+8B49.0	fMandarin	zheng4
U+8B49.0	fEnglish	[1] give evidence; prove; testify; verify; confirm; bear testimony; bear witness; corroborate; attest; certify [2] evidence; proof; testimony [3] certificate; proof; testimony [4] symptom; indication
U+8B4A.0	fUTF8	譊
U+8B4A.0	fCNS11643	2-6725
U+8B4A.0	fBig5	F2F8
U+8B4A.0	fGB	3-7325
U+8B4A.0	fR/S	 149.12
U+8B4A.0	fTotalStrokes	 19
U+8B4A.0	fCangjie	YRGGU
U+8B4A.0	fFourCorner	04611
U+8B4A.0	fHakka	  nau2; ngiau2
U+8B4A.0	fCantonese	naau4; laau4 (also); hiu1
U+8B4A.0	fMandarin	nao2; xiao1
U+8B4A.0	fEnglish	 argue; dispute; wrangle; clamor
U+8B4B.0	fUTF8	譋
U+8B4B.0	fCNS11643	2-672A
U+8B4B.0	fBig5	F2FD
U+8B4B.0	fGB	E-3E2F
U+8B4B.0	fR/S	 149.12
U+8B4B.0	fTotalStrokes	 19
U+8B4B.0	fCangjie	YRANB
U+8B4B.0	fFourCorner	07620
U+8B4B.0	fCantonese	laan4; laan5
U+8B4B.0	fMandarin	lan2; lan3; lan4
U+8B4C.0	fUTF8	譌
U+8B4C.0	fCNS11643	3-5B36
U+8B4C.0	fGB	E-3E30
U+8B4C.0	fR/S	 149.12
U+8B4C.0	fTotalStrokes	 19
U+8B4C.0	fCangjie	YRBHF
U+8B4C.0	fFourCorner	02620
U+8B4C.0	fHakka	  ngo2
U+8B4C.0	fCantonese	ngo4; o4
U+8B4C.0	fMandarin	e2
U+8B4C.0	fEnglish	false; erroneous
U+8B4D.0	fUTF8	譍
U+8B4D.0	fCNS11643	3-5D33
U+8B4D.0	fGB	5-3D49
U+8B4D.0	fR/S	 149.13
U+8B4D.0	fTotalStrokes	 20
U+8B4D.0	fCangjie	IOGR
U+8B4D.0	fFourCorner	00260
U+8B4D.0	fHakka	 jin1; rin1
U+8B4D.0	fCantonese	jing1; jing3
U+8B4D.0	fMandarin	ying4; ying2
U+8B4D.0	fEnglish	answer
U+8B4E.0	fUTF8	譎
U+8B4E.0	fCNS11643	1-7921
U+8B4E.0	fBig5	C3D4
U+8B4E.0	fGB	1-5A5C
U+8B4E.0	fR/S	 149.12
U+8B4E.0	fTotalStrokes	 19
U+8B4E.0	fCangjie	YRNHB
U+8B4E.0	fFourCorner	07627
U+8B4E.0	fHakka	 git7; giet7; get7
U+8B4E.0	fCantonese	kyut3; gwat1 (also)
U+8B4E.0	fMandarin	jue2
U+8B4E.0	fEnglish	[1] cheat; deceive; swindle [2] cunning; crafty; sly; wily; artful; tricky
U+8B4F.0	fUTF8	譏
U+8B4F.0	fCNS11643	1-7922
U+8B4F.0	fBig5	C3D5
U+8B4F.0	fGB	1-3C25
U+8B4F.0	fR/S	 149.12
U+8B4F.0	fTotalStrokes	 19
U+8B4F.0	fCangjie	YRVII
U+8B4F.0	fFourCorner	02653
U+8B4F.0	fHakka	  gi1
U+8B4F.0	fCantonese	gei1
U+8B4F.0	fMandarin	ji1
U+8B4F.0	fEnglish	[1] ridicule; jeer; sneer [2] inspect
U+8B50.0	fUTF8	譐
U+8B50.0	fCNS11643	2-6723
U+8B50.0	fBig5	F2F6
U+8B50.0	fGB	E-3E31
U+8B50.0	fR/S	 149.12
U+8B50.0	fTotalStrokes	 19
U+8B50.0	fCangjie	YRTWI
U+8B50.0	fFourCorner	08646
U+8B50.0	fHakka	  zun3
U+8B50.0	fCantonese	zyun2; cyun5
U+8B50.0	fMandarin	zun3
U+8B50.0	fEnglish	 talk together; conversation
U+8B51.0	fUTF8	譑
U+8B51.0	fCNS11643	2-672C
U+8B51.0	fBig5	F340
U+8B51.0	fGB	3-7329
U+8B51.0	fR/S	 149.12
U+8B51.0	fTotalStrokes	 19
U+8B51.0	fCangjie	YRHKB
U+8B51.0	fFourCorner	02627
U+8B51.0	fHakka	 giau3
U+8B51.0	fCantonese	giu2
U+8B51.0	fMandarin	jiao3; qiao4
U+8B51.0	fEnglish	 tattle
U+8B52.0	fUTF8	譒
U+8B52.0	fCNS11643	2-672E
U+8B52.0	fBig5	F342
U+8B52.0	fGB	5-727E
U+8B52.0	fR/S	 149.12
U+8B52.0	fTotalStrokes	 19
U+8B52.0	fCangjie	YRHDW
U+8B52.0	fFourCorner	02669
U+8B52.0	fHakka	 bo5
U+8B52.0	fCantonese	bo3
U+8B52.0	fMandarin	bo4; bo1
U+8B52.0	fEnglish	[1] spread abroad [2] promulgate
U+8B53.0	fUTF8	譓
U+8B53.0	fCNS11643	2-6727
U+8B53.0	fBig5	F2FA
U+8B53.0	fGB	5-727B
U+8B53.0	fR/S	 149.12
U+8B53.0	fTotalStrokes	 19
U+8B53.0	fCangjie	YRJIP
U+8B53.0	fFourCorner	05633
U+8B53.0	fHakka	 fi5; fui5
U+8B53.0	fCantonese	wai6
U+8B53.0	fMandarin	hui4
U+8B53.0	fEnglish	quick; sagacious
U+8B54.0	fUTF8	譔
U+8B54.0	fCNS11643	2-6729
U+8B54.0	fBig5	F2FC
U+8B54.0	fGB	E-3E32
U+8B54.0	fR/S	 149.12
U+8B54.0	fTotalStrokes	 19
U+8B54.0	fCangjie	YRRUC
U+8B54.0	fFourCorner	07681
U+8B54.0	fHakka	 con5;  cien2; con6; sien3; con3
U+8B54.0	fCantonese	zaan6; zaan3 (also)
U+8B54.0	fMandarin	zhuan4; quan1; xuan4
U+8B54.0	fEnglish	 eulogize compose; write
U+8B55.0	fUTF8	譕
U+8B55.0	fCNS11643	2-672B
U+8B55.0	fBig5	F2FE
U+8B55.0	fGB	3-7262
U+8B55.0	fR/S	 149.12
U+8B55.0	fTotalStrokes	 19
U+8B55.0	fCangjie	YROTF
U+8B55.0	fFourCorner	08631
U+8B55.0	fCantonese	mou4
U+8B55.0	fMandarin	mu2; wu2; wu3
U+8B56.0	fUTF8	譖
U+8B56.0	fCNS11643	2-6728
U+8B56.0	fBig5	F2FB
U+8B56.0	fGB	1-5A5A
U+8B56.0	fR/S	 149.12
U+8B56.0	fTotalStrokes	 19
U+8B56.0	fCangjie	YRMUA
U+8B56.0	fFourCorner	05661
U+8B56.0	fHakka	 ziam5
U+8B56.0	fCantonese	zam3; cam3 (also); cam6
U+8B56.0	fMandarin	zen4; jian4; zen3
U+8B56.0	fEnglish	 slander; charge falsely
U+8B57.0	fUTF8	譗
U+8B57.0	fCNS11643	2-672F
U+8B57.0	fBig5	F343
U+8B57.0	fGB	3-737E
U+8B57.0	fR/S	 149.12
U+8B57.0	fTotalStrokes	 19
U+8B57.0	fCangjie	YRHOR
U+8B57.0	fFourCorner	08661
U+8B57.0	fCantonese	zaap3
U+8B57.0	fMandarin	zha2
U+8B58.0	fUTF8	識
U+8B58.0	fCNS11643	1-787C
U+8B58.0	fBig5	C3D1
U+8B58.0	fGB	1-4A36
U+8B58.0	fR/S	 149.12
U+8B58.0	fTotalStrokes	 19
U+8B58.0	fCangjie	YRYIA
U+8B58.0	fFourCorner	03650
U+8B58.0	fHakka	  shit7;  sit7; zhi5; siit7; zih5
U+8B58.0	fCantonese	sik1; zi3
U+8B58.0	fMandarin	shi2; chi4; shi4
U+8B58.0	fEnglish	[1] recognise; know; discern [2] opinion; view [3] knowledge
U+8B58.1	fUTF8	識
U+8B58.1	fCNS11643	1-787C
U+8B58.1	fBig5	C3D1
U+8B58.1	fGB	1-4A36
U+8B58.1	fR/S	 149.12
U+8B58.1	fTotalStrokes	 19
U+8B58.1	fCangjie	YRYIA
U+8B58.1	fFourCorner	03650
U+8B58.1	fMandarin	zhi4
U+8B58.1	fEnglish	[1] record; remember [2] mark; sign
U+8B58.2	fUTF8	識
U+8B58.2	fCNS11643	1-787C
U+8B58.2	fBig5	C3D1
U+8B58.2	fGB	1-4A36
U+8B58.2	fR/S	 149.12
U+8B58.2	fTotalStrokes	 19
U+8B58.2	fCangjie	YRYIA
U+8B58.2	fFourCorner	03650
U+8B58.2	fMandarin	shi5
U+8B59.0	fUTF8	譙
U+8B59.0	fCNS11643	1-7924
U+8B59.0	fBig5	C3D7
U+8B59.0	fGB	1-5A5B
U+8B59.0	fR/S	 149.12
U+8B59.0	fTotalStrokes	 19
U+8B59.0	fCangjie	YROGF
U+8B59.0	fFourCorner	00631
U+8B59.0	fHakka	  ziau1; zeu1; ciau5
U+8B59.0	fCantonese	ciu4; ciu3; ziu6
U+8B59.0	fMandarin	qiao2; shui2
U+8B59.0	fEnglish	[1] tower [2] a Chinese family name
U+8B59.1	fUTF8	譙
U+8B59.1	fCNS11643	1-7924
U+8B59.1	fBig5	C3D7
U+8B59.1	fGB	1-5A5B
U+8B59.1	fR/S	 149.12
U+8B59.1	fTotalStrokes	 19
U+8B59.1	fCangjie	YROGF
U+8B59.1	fFourCorner	00631
U+8B59.1	fMandarin	qiao4
U+8B59.1	fEnglish	 scold; blame
U+8B5A.0	fUTF8	譚
U+8B5A.0	fCNS11643	1-787E
U+8B5A.0	fBig5	C3D3
U+8B5A.0	fGB	1-4C37
U+8B5A.0	fR/S	 149.12
U+8B5A.0	fTotalStrokes	 19
U+8B5A.0	fCangjie	YRMWJ
U+8B5A.0	fFourCorner	01646
U+8B5A.0	fHakka	  tam2; tam3
U+8B5A.0	fCantonese	taam4
U+8B5A.0	fMandarin	tan2; dan4
U+8B5A.0	fEnglish	[1] a Chinese family name [2] same as $ (talk)
U+8B5B.0	fUTF8	譛
U+8B5B.0	fGB	E-3E33
U+8B5B.0	fR/S	 149.12
U+8B5B.0	fTotalStrokes	 19
U+8B5B.0	fCangjie	YRQOA
U+8B5B.0	fFourCorner	0566
U+8B5B.0	fCantonese	zam3
U+8B5B.0	fMandarin	zen4
U+8B5B.0	fEnglish	 slander
U+8B5C.0	fUTF8	譜
U+8B5C.0	fCNS11643	1-787B
U+8B5C.0	fBig5	C3D0
U+8B5C.0	fGB	1-4657
U+8B5C.0	fR/S	 149.12
U+8B5C.0	fTotalStrokes	 19
U+8B5C.0	fCangjie	YRTCA
U+8B5C.0	fFourCorner	08661
U+8B5C.0	fHakka	  pu3
U+8B5C.0	fCantonese	pou2
U+8B5C.0	fMandarin	pu3; bu3
U+8B5C.0	fEnglish	[1] register; list; table; record [2] musical score [3] compose (song) [4] a general idea; rough picture; something to count on [5] a collection of examples for reference purpose; manual; guide
U+8B5D.0	fUTF8	譝
U+8B5D.0	fCNS11643	2-697C
U+8B5D.0	fBig5	F4D0
U+8B5D.0	fGB	5-7248
U+8B5D.0	fR/S	 149.13
U+8B5D.0	fTotalStrokes	 20
U+8B5D.0	fCangjie	YRRXU
U+8B5D.0	fFourCorner	07617
U+8B5D.0	fCantonese	sing4
U+8B5D.0	fMandarin	sheng2
U+8B5E.0	fUTF8	譞
U+8B5E.0	fCNS11643	4-644D
U+8B5E.0	fGB	3-7425
U+8B5E.0	fR/S	  149.13
U+8B5E.0	fTotalStrokes	 20
U+8B5E.0	fCangjie	YRWLV
U+8B5E.0	fFourCorner	06630
U+8B5E.0	fCantonese	hyun1
U+8B5E.0	fMandarin	xuan1
U+8B5F.0	fUTF8	譟
U+8B5F.0	fCNS11643	1-7A42
U+8B5F.0	fBig5	C4B7
U+8B5F.0	fGB	E-3E34
U+8B5F.0	fR/S	 149.13
U+8B5F.0	fTotalStrokes	 20
U+8B5F.0	fCangjie	YRRRD
U+8B5F.0	fFourCorner	06694
U+8B5F.0	fHakka	 cau5
U+8B5F.0	fCantonese	cou3
U+8B5F.0	fMandarin	zao4; cao1; sao4
U+8B5F.0	fEnglish	[1] noise of a crowd; shout noisily [2] slander; abuse
U+8B60.0	fUTF8	譠
U+8B60.0	fCNS11643	2-697A
U+8B60.0	fBig5	F4CE
U+8B60.0	fGB	3-7428
U+8B60.0	fR/S	 149.13
U+8B60.0	fTotalStrokes	 20
U+8B60.0	fCangjie	YRYWM
U+8B60.0	fFourCorner	00617
U+8B60.0	fCantonese	taan1
U+8B60.0	fMandarin	tan1; zhan1; chan1; shan4
U+8B61.0	fUTF8	譡
U+8B61.0	fCNS11643	3-5D30
U+8B61.0	fGB	5-722A
U+8B61.0	fR/S	  149.13
U+8B61.0	fTotalStrokes	 20
U+8B61.0	fCangjie	YRFBW
U+8B61.0	fFourCorner	09660
U+8B61.0	fHakka	 dong3
U+8B61.0	fCantonese	dong2
U+8B61.0	fMandarin	dang3
U+8B62.0	fUTF8	譢
U+8B62.0	fCNS11643	3-5D2C
U+8B62.0	fGB	3-7422
U+8B62.0	fR/S	  149.13
U+8B62.0	fTotalStrokes	 20
U+8B62.0	fCangjie	YRYTO
U+8B62.0	fFourCorner	08630
U+8B62.0	fHakka	 sui5
U+8B62.0	fCantonese	seoi6
U+8B62.0	fMandarin	sui4
U+8B63.0	fUTF8	譣
U+8B63.0	fCNS11643	2-697E
U+8B63.0	fBig5	F4D2
U+8B63.0	fGB	3-7345
U+8B63.0	fR/S	 149.13
U+8B63.0	fTotalStrokes	 20
U+8B63.0	fCangjie	YROMO
U+8B63.0	fFourCorner	08686
U+8B63.0	fCantonese	him2; cim1
U+8B63.0	fMandarin	qian1; xian1; xian3; yan4
U+8B64.0	fUTF8	譤
U+8B64.0	fCNS11643	3-5D32
U+8B64.0	fGB	3-7427
U+8B64.0	fR/S	  149.13
U+8B64.0	fTotalStrokes	 20
U+8B64.0	fCangjie	YRHSK
U+8B64.0	fFourCorner	08640
U+8B64.0	fCantonese	gik1
U+8B64.0	fMandarin	ji1
U+8B65.0	fUTF8	譥
U+8B65.0	fCNS11643	2-6A21
U+8B65.0	fBig5	F4D3
U+8B65.0	fGB	3-7258
U+8B65.0	fR/S	 149.13
U+8B65.0	fTotalStrokes	 20
U+8B65.0	fCangjie	HKYMR
U+8B65.0	fFourCorner	28601
U+8B65.0	fCantonese	giu3; giu2
U+8B65.0	fMandarin	jiao4
U+8B66.0	fUTF8	警
U+8B66.0	fCNS11643	1-7A40
U+8B66.0	fBig5	C4B5
U+8B66.0	fGB	0-3E2F
U+8B66.0	fR/S	 149.13
U+8B66.0	fTotalStrokes	 20
U+8B66.0	fCangjie	TKYMR
U+8B66.0	fFourCorner	48601
U+8B66.0	fHakka	  gin3; gin5
U+8B66.0	fCantonese	ging2
U+8B66.0	fMandarin	jing3
U+8B66.0	fEnglish	[1] guard; watch; keep watch [2] alert; warn [3] alarm [4] quick; alert; agile [5] the police
U+8B67.0	fUTF8	譧
U+8B67.0	fCNS11643	2-6A22
U+8B67.0	fBig5	F4D4
U+8B67.0	fGB	3-7429
U+8B67.0	fR/S	 149.13
U+8B67.0	fTotalStrokes	 20
U+8B67.0	fCangjie	YRITC
U+8B67.0	fFourCorner	00637
U+8B67.0	fHakka	 liam2
U+8B67.0	fCantonese	zaam1; lim4
U+8B67.0	fMandarin	lian2; zhan4
U+8B67.0	fEnglish	[1] deceive [2] humbug
U+8B68.0	fUTF8	譨
U+8B68.0	fCNS11643	2-697D
U+8B68.0	fBig5	F4D1
U+8B68.0	fGB	5-7236
U+8B68.0	fR/S	 149.13
U+8B68.0	fTotalStrokes	 20
U+8B68.0	fCangjie	YRTWV
U+8B68.0	fFourCorner	05632
U+8B68.0	fCantonese	nung4; lung4 (also)
U+8B68.0	fMandarin	nou2; nou4; nong2
U+8B69.0	fUTF8	譩
U+8B69.0	fCNS11643	3-5D2B
U+8B69.0	fGB	3-742A
U+8B69.0	fR/S	 149.13
U+8B69.0	fTotalStrokes	 20
U+8B69.0	fCangjie	YRYTP
U+8B69.0	fFourCorner	00630
U+8B69.0	fHakka	 ji1; ri1; ri5; ji5; i1
U+8B69.0	fCantonese	ji1
U+8B69.0	fMandarin	yi1
U+8B69.0	fEnglish	[1] an interjection [2] dislike
U+8B6A.0	fUTF8	譪
U+8B6A.0	fCNS11643	2-697B
U+8B6A.0	fBig5	F4CF
U+8B6A.0	fGB	5-7277
U+8B6A.0	fR/S	 149.13
U+8B6A.0	fTotalStrokes	 20
U+8B6A.0	fCangjie	YRTAV
U+8B6A.0	fFourCorner	04627
U+8B6A.0	fCantonese	oi2; ngoi2 (also)
U+8B6A.0	fMandarin	ai4
U+8B6B.0	fUTF8	譫
U+8B6B.0	fCNS11643	1-7A43
U+8B6B.0	fBig5	C4B8
U+8B6B.0	fGB	1-5A5E
U+8B6B.0	fR/S	 149.13
U+8B6B.0	fTotalStrokes	 20
U+8B6B.0	fCangjie	YRNCR
U+8B6B.0	fFourCorner	07661
U+8B6B.0	fHakka	 zam1;  zham1
U+8B6B.0	fCantonese	zim1
U+8B6B.0	fMandarin	zhan1; ta4; zhe2
U+8B6B.0	fEnglish	 talkative incoherent talk
U+8B6C.0	fUTF8	譬
U+8B6C.0	fCNS11643	1-7A3F
U+8B6C.0	fBig5	C4B4
U+8B6C.0	fGB	0-4629
U+8B6C.0	fR/S	 149.13
U+8B6C.0	fTotalStrokes	 20
U+8B6C.0	fCangjie	SJYMR
U+8B6C.0	fFourCorner	70601
U+8B6C.0	fHakka	 pit7; bi3
U+8B6C.0	fCantonese	pei3
U+8B6C.0	fMandarin	pi4
U+8B6C.0	fEnglish	[1] liken; compare [2] simile; example [3] tell [4] understand
U+8B6D.0	fUTF8	譭
U+8B6D.0	fCNS11643	2-6A23
U+8B6D.0	fBig5	F4D5
U+8B6D.0	fGB	E-3E35
U+8B6D.0	fR/S	 149.13
U+8B6D.0	fTotalStrokes	 20
U+8B6D.0	fCangjie	YRHGE
U+8B6D.0	fFourCorner	07647
U+8B6D.0	fHakka	 kui3; fui3; fi3; fui6; fi5; kwi3; k'wui3
U+8B6D.0	fCantonese	wai2
U+8B6D.0	fMandarin	hui3; hui4
U+8B6D.0	fEnglish	 slander defame
U+8B6E.0	fUTF8	譮
U+8B6E.0	fCNS11643	3-5D2E
U+8B6E.0	fGB	3-732D
U+8B6E.0	fR/S	  149.13
U+8B6E.0	fTotalStrokes	 20
U+8B6E.0	fCangjie	YROMA
U+8B6E.0	fFourCorner	08660
U+8B6E.0	fHakka	 wa5; woi1; fui6; fi5; va5
U+8B6E.0	fCantonese	waa6
U+8B6E.0	fMandarin	hua4
U+8B6E.0	fEnglish	[1] words [2] talk; speak
U+8B6F.0	fUTF8	譯
U+8B6F.0	fCNS11643	1-7A41
U+8B6F.0	fBig5	C4B6
U+8B6F.0	fGB	1-526B
U+8B6F.0	fR/S	 149.13
U+8B6F.0	fTotalStrokes	 20
U+8B6F.0	fCangjie	YRWLJ
U+8B6F.0	fFourCorner	06641
U+8B6F.0	fHakka	  jit8; rit8; cak8
U+8B6F.0	fCantonese	jik6; jik2 (also)
U+8B6F.0	fMandarin	yi4
U+8B6F.0	fEnglish	 translate; decode; encode; translation
U+8B70.0	fUTF8	議
U+8B70.0	fCNS11643	1-7A3E
U+8B70.0	fBig5	C4B3
U+8B70.0	fGB	1-5269
U+8B70.0	fR/S	 149.13
U+8B70.0	fTotalStrokes	 20
U+8B70.0	fCangjie	YRTGI
U+8B70.0	fFourCorner	08653
U+8B70.0	fHakka	 ngi5; ngi1; gni1; ngi6; gni6; ji3; li3; {ngi1}; ni1; ni3; ngi35; gnie5; ngiai5
U+8B70.0	fCantonese	ji5; ji3 (also); ji6
U+8B70.0	fMandarin	yi4; yi2
U+8B70.0	fEnglish	[1] discuss; argue; debate; negotiate; talk over [2] opinion; view [3] criticise; comment; criticism; comment [4] argumentative writing; argumentation; essay; treatise
U+8B71.0	fUTF8	譱
U+8B71.0	fCNS11643	3-5C6D
U+8B71.0	fGB	E-3E36
U+8B71.0	fR/S	 149.13; 123.13
U+8B71.0	fTotalStrokes	 20
U+8B71.0	fCangjie	TQYRR
U+8B71.0	fFourCorner	80660
U+8B71.0	fCantonese	sin6
U+8B71.0	fMandarin	shan4
U+8B72.0	fUTF8	譲
U+8B72.0	fGB	E-3E37
U+8B72.0	fR/S	 149.13
U+8B72.0	fTotalStrokes	 20
U+8B72.0	fCangjie	YRYCV
U+8B72.0	fCantonese	joeng6
U+8B72.0	fMandarin	rang4
U+8B72.0	fEnglish	allow; permit; yield; concede
U+8B73.0	fUTF8	譳
U+8B73.0	fCNS11643	4-673B
U+8B73.0	fGB	3-742B
U+8B73.0	fR/S	  149.14
U+8B73.0	fTotalStrokes	 21
U+8B73.0	fCangjie	YRMBB
U+8B73.0	fFourCorner	01620
U+8B73.0	fCantonese	nau6; lau6 (also)
U+8B73.0	fMandarin	nou4
U+8B74.0	fUTF8	譴
U+8B74.0	fCNS11643	1-7B2B
U+8B74.0	fBig5	C4FE
U+8B74.0	fGB	1-4734
U+8B74.0	fR/S	 149.14
U+8B74.0	fTotalStrokes	 21
U+8B74.0	fCangjie	YRYLR
U+8B74.0	fFourCorner	05637
U+8B74.0	fHakka	 kien3; ken3; kian3; kien5
U+8B74.0	fCantonese	hin2
U+8B74.0	fMandarin	qian3; qian4
U+8B74.0	fEnglish	[1] reprimand; reproach; upbraid [2] punishment
U+8B75.0	fUTF8	譵
U+8B75.0	fCNS11643	3-5E62
U+8B75.0	fGB	5-7221
U+8B75.0	fR/S	 149.14
U+8B75.0	fTotalStrokes	 21
U+8B75.0	fCangjie	YRTGI
U+8B75.0	fFourCorner	04600
U+8B75.0	fCantonese	deoi6
U+8B75.0	fMandarin	zhui4
U+8B76.0	fUTF8	譶
U+8B76.0	fCNS11643	4-6741
U+8B76.0	fGB	5-7150
U+8B76.0	fR/S	  149.14
U+8B76.0	fTotalStrokes	 21
U+8B76.0	fCangjie	YRYRR
U+8B76.0	fFourCorner	00660
U+8B76.0	fCantonese	daap6
U+8B76.0	fMandarin	ta4
U+8B77.0	fUTF8	護
U+8B77.0	fCNS11643	1-7B2C
U+8B77.0	fBig5	C540
U+8B77.0	fGB	1-3B24
U+8B77.0	fR/S	 149.14
U+8B77.0	fTotalStrokes	 21
U+8B77.0	fCangjie	YRTOE
U+8B77.0	fFourCorner	04647
U+8B77.0	fHakka	  fu5; fo5; fu6
U+8B77.0	fCantonese	wu6
U+8B77.0	fMandarin	hu4
U+8B77.0	fEnglish	[1] protect; guard; defend; shelter; shield [2] take sides; be partial to
U+8B78.0	fUTF8	譸
U+8B78.0	fCNS11643	2-6C3D
U+8B78.0	fBig5	F64E
U+8B78.0	fGB	3-7336
U+8B78.0	fR/S	 149.14
U+8B78.0	fTotalStrokes	 21
U+8B78.0	fCangjie	YRGNI
U+8B78.0	fFourCorner	04641
U+8B78.0	fHakka	 zhiu1; chiu2; zu1; cu2
U+8B78.0	fCantonese	zau1
U+8B78.0	fMandarin	zhou1; chou2
U+8B78.0	fEnglish	 deceive; cheat hurried; bustlin
U+8B79.0	fUTF8	譹
U+8B79.0	fCNS11643	2-6C3C
U+8B79.0	fBig5	F64D
U+8B79.0	fGB	5-7328
U+8B79.0	fR/S	 149.14
U+8B79.0	fTotalStrokes	 21
U+8B79.0	fCangjie	YRYRO
U+8B79.0	fFourCorner	00632
U+8B79.0	fHakka	  hau2; ho2; hau3
U+8B79.0	fCantonese	haak3; hou4
U+8B79.0	fMandarin	hao2; xia4; huo4
U+8B79.0	fEnglish	 shout; roar; terrify swiftly
U+8B7A.0	fUTF8	譺
U+8B7A.0	fCNS11643	2-6C3F
U+8B7A.0	fBig5	F650
U+8B7A.0	fGB	3-742F
U+8B7A.0	fR/S	 149.14
U+8B7A.0	fTotalStrokes	 21
U+8B7A.0	fCangjie	YRPKO
U+8B7A.0	fFourCorner	07681
U+8B7A.0	fHakka	  ngi3
U+8B7A.0	fCantonese	ngaai6; aai6 (also); jik6; ji5
U+8B7A.0	fMandarin	ye4; ni3; ni4; yi4
U+8B7A.0	fEnglish	[1] consult; deliberate [2] delude
U+8B7B.0	fUTF8	譻
U+8B7B.0	fCNS11643	2-6C40
U+8B7B.0	fBig5	F651
U+8B7B.0	fGB	5-7147
U+8B7B.0	fR/S	 149.14
U+8B7B.0	fTotalStrokes	 21
U+8B7B.0	fCangjie	BCYMR
U+8B7B.0	fFourCorner	66601
U+8B7B.0	fCantonese	jing1
U+8B7B.0	fMandarin	ying1
U+8B7C.0	fUTF8	譼
U+8B7C.0	fCNS11643	3-5E63
U+8B7C.0	fGB	E-3E38
U+8B7C.0	fR/S	 149.14; 149.15
U+8B7C.0	fTotalStrokes	 21
U+8B7C.0	fCangjie	SWYMR
U+8B7C.0	fFourCorner	78600
U+8B7C.0	fCantonese	gaam1
U+8B7C.0	fMandarin	jian1; jian4; kan4
U+8B7C.0	fEnglish	 supervise to confine a government establishment
U+8B7D.0	fUTF8	譽
U+8B7D.0	fCNS11643	1-7B2D
U+8B7D.0	fBig5	C541
U+8B7D.0	fGB	1-537E
U+8B7D.0	fR/S	 149.14
U+8B7D.0	fTotalStrokes	 21
U+8B7D.0	fCangjie	HCYMR
U+8B7D.0	fFourCorner	77601
U+8B7D.0	fHakka	 ji5; ji3; ji2; ri5; ri2; i5; i2
U+8B7D.0	fCantonese	jyu6; jyu4
U+8B7D.0	fMandarin	yu4; yu2
U+8B7D.0	fEnglish	[1] fame; honor; reputation [2] eulogize; praise
U+8B7E.0	fUTF8	譾
U+8B7E.0	fCNS11643	2-6E25
U+8B7E.0	fBig5	F756
U+8B7E.0	fGB	E-3E39
U+8B7E.0	fR/S	 149.15
U+8B7E.0	fTotalStrokes	 22
U+8B7E.0	fCangjie	YRTBM
U+8B7E.0	fFourCorner	08627
U+8B7E.0	fHakka	  zien3
U+8B7E.0	fCantonese	zin2
U+8B7E.0	fMandarin	jian3
U+8B7E.0	fEnglish	shallow (mentally); stupid
U+8B7F.0	fUTF8	譿
U+8B7F.0	fCNS11643	2-6E2A
U+8B7F.0	fBig5	F75B
U+8B7F.0	fGB	5-7329
U+8B7F.0	fR/S	 149.15
U+8B7F.0	fTotalStrokes	 22
U+8B7F.0	fCangjie	YRQJP
U+8B7F.0	fFourCorner	04637
U+8B7F.0	fCantonese	wai6
U+8B7F.0	fMandarin	hui4
U+8B80.0	fUTF8	讀
U+8B80.0	fCNS11643	1-7B74
U+8B80.0	fBig5	C5AA
U+8B80.0	fGB	1-3641
U+8B80.0	fR/S	 149.15
U+8B80.0	fTotalStrokes	 22
U+8B80.0	fCangjie	YRGWC
U+8B80.0	fFourCorner	04686
U+8B80.0	fHakka	  tuk8;  teu5; teu6; tiu3
U+8B80.0	fCantonese	duk6; dau6
U+8B80.0	fMandarin	du2
U+8B80.0	fEnglish	[1] read; pronounce [2] attend school; go to school; college, etc. [3] study
U+8B80.1	fUTF8	讀
U+8B80.1	fCNS11643	1-7B74
U+8B80.1	fBig5	C5AA
U+8B80.1	fGB	1-3641
U+8B80.1	fR/S	 149.15
U+8B80.1	fTotalStrokes	 22
U+8B80.1	fCangjie	YRGWC
U+8B80.1	fFourCorner	04686
U+8B80.1	fMandarin	dou4
U+8B80.1	fEnglish	pauses in a sentence
U+8B81.0	fUTF8	讁
U+8B81.0	fCNS11643	3-5F61
U+8B81.0	fGB	E-3E3A
U+8B81.0	fR/S	  149.15
U+8B81.0	fTotalStrokes	 22
U+8B81.0	fCangjie	YRYYB
U+8B81.0	fFourCorner	00630
U+8B81.0	fHakka	  zak7; dit7
U+8B81.0	fCantonese	zaak6
U+8B81.0	fMandarin	zhe2
U+8B81.0	fEnglish	 punish; blame
U+8B82.0	fUTF8	讂
U+8B82.0	fCNS11643	2-6E27
U+8B82.0	fBig5	F758
U+8B82.0	fGB	3-742E
U+8B82.0	fR/S	 149.15
U+8B82.0	fTotalStrokes	 22
U+8B82.0	fCangjie	YRNBE
U+8B82.0	fFourCorner	07647
U+8B82.0	fCantonese	gyun3; hyun3
U+8B82.0	fMandarin	xuan4; juan4; xuan1
U+8B83.0	fUTF8	讃
U+8B83.0	fCNS11643	3-5F62
U+8B83.0	fGB	E-3E3B
U+8B83.0	fR/S	  149.15
U+8B83.0	fTotalStrokes	 22
U+8B83.0	fCangjie	YRQOC
U+8B83.0	fFourCorner	05680
U+8B83.0	fHakka	 zan5
U+8B83.0	fCantonese	zaan3
U+8B83.0	fMandarin	zan4
U+8B83.0	fEnglish	praise; commend; eulogize
U+8B84.0	fUTF8	讄
U+8B84.0	fCNS11643	2-6E26
U+8B84.0	fBig5	F757
U+8B84.0	fGB	5-732B
U+8B84.0	fR/S	 149.15
U+8B84.0	fTotalStrokes	 22
U+8B84.0	fCangjie	YRWWW
U+8B84.0	fFourCorner	06660
U+8B84.0	fCantonese	leoi5
U+8B84.0	fMandarin	lei3
U+8B85.0	fUTF8	讅
U+8B85.0	fCNS11643	2-6E29
U+8B85.0	fBig5	F75A
U+8B85.0	fGB	3-7356
U+8B85.0	fR/S	 149.15
U+8B85.0	fTotalStrokes	 22
U+8B85.0	fCangjie	YRJHW
U+8B85.0	fFourCorner	03669
U+8B85.0	fHakka	 sim3; shim3
U+8B85.0	fCantonese	sam2
U+8B85.0	fMandarin	shen3
U+8B85.0	fEnglish	 know; be aware
U+8B86.0	fUTF8	讆
U+8B86.0	fCNS11643	2-6E28
U+8B86.0	fBig5	F759
U+8B86.0	fGB	E-3E3C
U+8B86.0	fR/S	 149.16
U+8B86.0	fTotalStrokes	 23
U+8B86.0	fCangjie	HNYMR
U+8B86.0	fFourCorner	21601
U+8B86.0	fCantonese	wai6; ngai6; ai6
U+8B86.0	fMandarin	wei4
U+8B86.0	fEnglish	 exaggerate incredible
U+8B87.0	fUTF8	讇
U+8B87.0	fCNS11643	3-6048
U+8B87.0	fGB	5-7271
U+8B87.0	fR/S	  149.16
U+8B87.0	fTotalStrokes	 23
U+8B87.0	fCangjie	YRANX
U+8B87.0	fFourCorner	07620
U+8B87.0	fHakka	 tiam3; ram2; jam2; cam3; cham5
U+8B87.0	fCantonese	cim2
U+8B87.0	fMandarin	chan3
U+8B87.0	fEnglish	 flatter suspect; be uncertain
U+8B88.0	fUTF8	讈
U+8B88.0	fCNS11643	2-6F51
U+8B88.0	fBig5	F843
U+8B88.0	fGB	E-3E3D
U+8B88.0	fR/S	 149.16
U+8B88.0	fTotalStrokes	 23
U+8B88.0	fCangjie	YRMHM
U+8B88.0	fFourCorner	01611
U+8B88.0	fCantonese	lik6
U+8B88.0	fMandarin	li4
U+8B89.0	fUTF8	讉
U+8B89.0	fCNS11643	4-6A7C
U+8B89.0	fGB	3-737D
U+8B89.0	fR/S	  149.16
U+8B89.0	fTotalStrokes	 23
U+8B89.0	fCangjie	YRYLC
U+8B89.0	fFourCorner	05630
U+8B89.0	fCantonese	wai4
U+8B89.0	fMandarin	yi2
U+8B8A.0	fUTF8	變
U+8B8A.0	fCNS11643	1-7C48
U+8B8A.0	fBig5	C5DC
U+8B8A.0	fGB	1-3164
U+8B8A.0	fR/S	 149.16
U+8B8A.0	fTotalStrokes	 23
U+8B8A.0	fCangjie	VFOK
U+8B8A.0	fFourCorner	22408
U+8B8A.0	fHakka	 bien5; ben5; pian5
U+8B8A.0	fCantonese	bin3
U+8B8A.0	fMandarin	bian4
U+8B8A.0	fEnglish	[1] change; transform; alter [2] change; become different [3] turn into; become [4] extraordinarily; uncommon [5] accident; misfortune; tragedy; upheaval; disturbance; unexpected turn of events [6] changeable
U+8B8B.0	fUTF8	讋
U+8B8B.0	fCNS11643	2-6F50
U+8B8B.0	fBig5	F842
U+8B8B.0	fGB	3-587E
U+8B8B.0	fR/S	 149.16
U+8B8B.0	fTotalStrokes	 23
U+8B8B.0	fCangjie	YPYMR
U+8B8B.0	fFourCorner	01601
U+8B8B.0	fCantonese	zip3 (also); sip3
U+8B8B.0	fMandarin	zhe2; ta4; she4
U+8B8B.0	fEnglish	 fear envy loquacious
U+8B8C.0	fUTF8	讌
U+8B8C.0	fCNS11643	2-6F4E
U+8B8C.0	fBig5	F840
U+8B8C.0	fGB	3-7433
U+8B8C.0	fR/S	 149.16
U+8B8C.0	fTotalStrokes	 23
U+8B8C.0	fCangjie	YRTLF
U+8B8C.0	fFourCorner	04631
U+8B8C.0	fHakka	 jen5; ren5; jan5
U+8B8C.0	fCantonese	jin3
U+8B8C.0	fMandarin	yan4
U+8B8C.0	fEnglish	 feast; banquet; feast
U+8B8D.0	fUTF8	讍
U+8B8D.0	fCNS11643	3-6047
U+8B8D.0	fGB	E-3E3E
U+8B8D.0	fR/S	 149.16
U+8B8D.0	fTotalStrokes	 23
U+8B8D.0	fCangjie	YRMGR
U+8B8D.0	fFourCorner	01610
U+8B8D.0	fHakka	 ngok8
U+8B8D.0	fCantonese	ngok6; ok6 (also)
U+8B8D.0	fMandarin	e4
U+8B8D.0	fEnglish	honest speech
U+8B8E.0	fUTF8	讎
U+8B8E.0	fCNS11643	2-6F4F
U+8B8E.0	fBig5	F841
U+8B8E.0	fGB	1-7645
U+8B8E.0	fR/S	 149.16
U+8B8E.0	fTotalStrokes	 23
U+8B8E.0	fCangjie	OGYRG
U+8B8E.0	fFourCorner	20214
U+8B8E.0	fHakka	 cu2;  chiu2; su2; shiu2; ciu2; siu2; shu2
U+8B8E.0	fCantonese	cau4; sau4
U+8B8E.0	fMandarin	chou2; shou4
U+8B8E.0	fEnglish	[1] enemy; rival; opponent; foe [2] collate; compare [3] toast [4] answer
U+8B8F.0	fUTF8	讏
U+8B8F.0	fGB	E-3E3F
U+8B8F.0	fR/S	 149.16
U+8B8F.0	fTotalStrokes	 23
U+8B8F.0	fCangjie	XHNYM
U+8B8F.0	fCantonese	wai6
U+8B8F.0	fMandarin	wei4
U+8B90.0	fUTF8	讐
U+8B90.0	fCNS11643	3-6049
U+8B90.0	fGB	E-3E40
U+8B90.0	fR/S	 149.16
U+8B90.0	fTotalStrokes	 23
U+8B90.0	fCangjie	OGYMR
U+8B90.0	fFourCorner	20600
U+8B90.0	fHakka	 su2;  shiu2; cu2;  chiu2; shu3; ciu2; siu2; shu2
U+8B90.0	fCantonese	cau4; sau4
U+8B90.0	fMandarin	chou2
U+8B90.0	fEnglish	enemy; rival; opponent
U+8B91.0	fUTF8	讑
U+8B91.0	fCNS11643	3-607B
U+8B91.0	fGB	3-7434
U+8B91.0	fR/S	 149.17
U+8B91.0	fTotalStrokes	 24
U+8B91.0	fCangjie	YROMB
U+8B91.0	fFourCorner	08620
U+8B91.0	fCantonese	jiu6
U+8B91.0	fMandarin	yao4
U+8B92.0	fUTF8	讒
U+8B92.0	fCNS11643	1-7C6A
U+8B92.0	fBig5	C5FE
U+8B92.0	fGB	1-3277
U+8B92.0	fR/S	 149.17
U+8B92.0	fTotalStrokes	 24
U+8B92.0	fCangjie	YRNRI
U+8B92.0	fFourCorner	07616
U+8B92.0	fHakka	  cam2
U+8B92.0	fCantonese	caam4
U+8B92.0	fMandarin	chan2; zhan4
U+8B92.0	fEnglish	 slander; defame; misrepresent; calumniate
U+8B93.0	fUTF8	讓
U+8B93.0	fCNS11643	1-7C69
U+8B93.0	fBig5	C5FD
U+8B93.0	fGB	1-4843
U+8B93.0	fR/S	 149.17
U+8B93.0	fTotalStrokes	 24
U+8B93.0	fCangjie	YRYRV
U+8B93.0	fFourCorner	00632
U+8B93.0	fHakka	 ngiong5;  jong5; jong6; rong5
U+8B93.0	fCantonese	joeng6
U+8B93.0	fMandarin	rang4
U+8B93.0	fEnglish	[1] give way; make a concession; back down; yield; allow; give ground [2] allow; let; permit [3] turn over; transfer; surrender; cede [4] by [5] step aside; make way; let by
U+8B94.0	fUTF8	讔
U+8B94.0	fCNS11643	2-704F
U+8B94.0	fBig5	F8C1
U+8B94.0	fGB	3-737B
U+8B94.0	fR/S	 149.17
U+8B94.0	fTotalStrokes	 24
U+8B94.0	fCangjie	YRNLP
U+8B94.0	fFourCorner	02637
U+8B94.0	fCantonese	jan2
U+8B94.0	fMandarin	yin3
U+8B95.0	fUTF8	讕
U+8B95.0	fCNS11643	2-7050
U+8B95.0	fBig5	F8C2
U+8B95.0	fGB	1-403E
U+8B95.0	fR/S	 149.17
U+8B95.0	fTotalStrokes	 24
U+8B95.0	fCangjie	YRANW
U+8B95.0	fFourCorner	07620
U+8B95.0	fHakka	 lan2
U+8B95.0	fCantonese	laan4; laan5
U+8B95.0	fMandarin	lan2; lan3; lan4
U+8B95.0	fEnglish	 abuse; revile; slander; libel
U+8B96.0	fUTF8	讖
U+8B96.0	fCNS11643	1-7C6B
U+8B96.0	fBig5	C640
U+8B96.0	fGB	1-5A5F
U+8B96.0	fR/S	 149.17
U+8B96.0	fTotalStrokes	 24
U+8B96.0	fCangjie	YROIM
U+8B96.0	fFourCorner	03650
U+8B96.0	fHakka	 cam5; siam2; ciam5
U+8B96.0	fCantonese	caam3 (also); cam3
U+8B96.0	fMandarin	chen4; chan4
U+8B96.0	fEnglish	[1] prophecy; hint; omen [2] books about omens
U+8B97.0	fUTF8	讗
U+8B97.0	fCNS11643	3-614B
U+8B97.0	fGB	3-7435
U+8B97.0	fR/S	  149.18
U+8B97.0	fTotalStrokes	 25
U+8B97.0	fCangjie	YRUOB
U+8B97.0	fFourCorner	02620
U+8B97.0	fCantonese	kwai4
U+8B97.0	fMandarin	huo4
U+8B98.0	fUTF8	讘
U+8B98.0	fCNS11643	2-713C
U+8B98.0	fBig5	F94D
U+8B98.0	fGB	3-736A
U+8B98.0	fR/S	 149.18
U+8B98.0	fTotalStrokes	 25
U+8B98.0	fCangjie	YRSJJ
U+8B98.0	fFourCorner	01641
U+8B98.0	fHakka	  ngiap7; zhap7; zap7
U+8B98.0	fCantonese	zip3; jip6
U+8B98.0	fMandarin	zhe2; re4
U+8B98.0	fEnglish	[1] chatter [2] abuse
U+8B99.0	fUTF8	讙
U+8B99.0	fCNS11643	2-713D
U+8B99.0	fBig5	F94E
U+8B99.0	fGB	E-3E41
U+8B99.0	fR/S	 149.18
U+8B99.0	fTotalStrokes	 25
U+8B99.0	fCangjie	YRTRG
U+8B99.0	fFourCorner	04614
U+8B99.0	fHakka	 fon1; fan1
U+8B99.0	fCantonese	fun1; hyun1
U+8B99.0	fMandarin	huan1; xuan1; guan4
U+8B99.0	fEnglish	[1] cheer; be noisy; clamor [2] same as $
U+8B9A.0	fUTF8	讚
U+8B9A.0	fCNS11643	1-7D34
U+8B9A.0	fBig5	C667
U+8B9A.0	fGB	E-3E42
U+8B9A.0	fR/S	 149.19
U+8B9A.0	fTotalStrokes	 26
U+8B9A.0	fCangjie	YRHUC
U+8B9A.0	fFourCorner	04686
U+8B9A.0	fHakka	  zan5
U+8B9A.0	fCantonese	zaan3; zaan2 (also)
U+8B9A.0	fMandarin	zan4
U+8B9A.0	fEnglish	[1] praise; commend; eulogize; applaud; laud [2] same as $
U+8B9B.0	fUTF8	讛
U+8B9B.0	fCNS11643	3-616B
U+8B9B.0	fGB	5-715A
U+8B9B.0	fR/S	 149.19
U+8B9B.0	fTotalStrokes	 26
U+8B9B.0	fCangjie	YRTGI
U+8B9B.0	fFourCorner	04630
U+8B9B.0	fHakka	 ngi5; ngi6
U+8B9B.0	fCantonese	ngai6; ai6 (also)
U+8B9B.0	fMandarin	yi4
U+8B9B.0	fEnglish	[1] talk in one's sleep [2] whisper
U+8B9C.0	fUTF8	讜
U+8B9C.0	fCNS11643	1-7D3A
U+8B9C.0	fBig5	C66D
U+8B9C.0	fGB	1-5A54
U+8B9C.0	fR/S	 149.20
U+8B9C.0	fTotalStrokes	 27
U+8B9C.0	fCangjie	YRFBF
U+8B9C.0	fFourCorner	09631
U+8B9C.0	fHakka	 dong3
U+8B9C.0	fCantonese	dong2
U+8B9C.0	fMandarin	dang3; dang4; tang4
U+8B9C.0	fEnglish	 speak out boldly (advice, comment); honest; unbiased; straightforward
U+8B9D.0	fUTF8	讝
U+8B9D.0	fCNS11643	3-617E
U+8B9D.0	fGB	3-7337
U+8B9D.0	fR/S	 149.20
U+8B9D.0	fTotalStrokes	 27
U+8B9D.0	fCangjie	YRRRK
U+8B9D.0	fFourCorner	06640
U+8B9D.0	fCantonese	zim1
U+8B9D.0	fMandarin	zhan1
U+8B9E.0	fUTF8	讞
U+8B9E.0	fCNS11643	2-7177
U+8B9E.0	fBig5	F9A9
U+8B9E.0	fGB	1-5A5D
U+8B9E.0	fR/S	 149.20
U+8B9E.0	fTotalStrokes	 27
U+8B9E.0	fCangjie	YRYBK
U+8B9E.0	fFourCorner	03534
U+8B9E.0	fHakka	 hien5; ngien5; hen5
U+8B9E.0	fCantonese	jin6; jin5; jit6
U+8B9E.0	fMandarin	yan4; nie4; yan3
U+8B9E.0	fEnglish	 judge at a court of law; verdict
U+8B9F.0	fUTF8	讟
U+8B9F.0	fCNS11643	2-7236
U+8B9F.0	fBig5	F9C8
U+8B9F.0	fGB	E-3E43
U+8B9F.0	fR/S	 149.22
U+8B9F.0	fTotalStrokes	 29
U+8B9F.0	fCangjie	YRGCR
U+8B9F.0	fFourCorner	00661
U+8B9F.0	fHakka	  tuk8
U+8B9F.0	fCantonese	duk6
U+8B9F.0	fMandarin	du2; ta4
U+8B9F.0	fEnglish	slander; utter evil words
U+8BA0.0	fUTF8	讠
U+8BA0.0	fGB	0-5A25
U+8BA0.0	fR/S	 149.0
U+8BA0.0	fTotalStrokes	 2
U+8BA0.0	fCangjie	INV
U+8BA0.0	fMandarin	yan2zi4pang2; yan2
U+8BA0.0	fEnglish	simplified form of &#x8A01;
U+8BA1.0	fUTF8	计
U+8BA1.0	fGB	0-3C46
U+8BA1.0	fR/S	 149.2
U+8BA1.0	fTotalStrokes	 4
U+8BA1.0	fCangjie	IVJ
U+8BA1.0	fHakka	 ge5; gi5
U+8BA1.0	fCantonese	gai3; gai2 (also); gei3
U+8BA1.0	fMandarin	ji4; jie2
U+8BA1.0	fEnglish	simplified form of &#x8A08;
U+8BA1.1	fUTF8	计
U+8BA1.1	fGB	0-3C46
U+8BA1.1	fR/S	 149.2
U+8BA1.1	fTotalStrokes	 4
U+8BA1.1	fCangjie	IVJ
U+8BA1.1	fMandarin	ji5
U+8BA2.0	fUTF8	订
U+8BA2.0	fGB	0-3629
U+8BA2.0	fR/S	 149.2
U+8BA2.0	fTotalStrokes	 4
U+8BA2.0	fCangjie	IVMN
U+8BA2.0	fHakka	 dang5; den5
U+8BA2.0	fCantonese	ding3; ding6 (also); deng6
U+8BA2.0	fMandarin	ding4; ting1; ting2; ting3
U+8BA2.0	fEnglish	simplified form of &#x8A02;
U+8BA3.0	fUTF8	讣
U+8BA3.0	fGB	0-383C
U+8BA3.0	fR/S	 149.2
U+8BA3.0	fTotalStrokes	 4
U+8BA3.0	fCangjie	IVY
U+8BA3.0	fHakka	 fu5
U+8BA3.0	fCantonese	fu6
U+8BA3.0	fMandarin	fu4
U+8BA3.0	fEnglish	simplified form of &#x8A03;
U+8BA4.0	fUTF8	认
U+8BA4.0	fGB	0-484F
U+8BA4.0	fR/S	 149.2
U+8BA4.0	fTotalStrokes	 4
U+8BA4.0	fCangjie	IVO
U+8BA4.0	fHakka	 ngin5
U+8BA4.0	fCantonese	jing6; jing2 (also)
U+8BA4.0	fMandarin	ren4; reng4
U+8BA4.0	fEnglish	simplified form of &#x8A8D;
U+8BA5.0	fUTF8	讥
U+8BA5.0	fGB	0-3C25
U+8BA5.0	fR/S	 149.2
U+8BA5.0	fTotalStrokes	 4
U+8BA5.0	fCangjie	IVHN
U+8BA5.0	fHakka	 gi1
U+8BA5.0	fCantonese	gei1
U+8BA5.0	fMandarin	ji1
U+8BA5.0	fEnglish	simplified form of &#x8B4F;
U+8BA6.0	fUTF8	讦
U+8BA6.0	fGB	0-5A26
U+8BA6.0	fR/S	 149.3
U+8BA6.0	fTotalStrokes	 5
U+8BA6.0	fCangjie	IVMJ
U+8BA6.0	fHakka	 get7; giet7
U+8BA6.0	fCantonese	kit3; gai3
U+8BA6.0	fMandarin	jie2; ji4
U+8BA6.0	fEnglish	simplified form of &#x8A10;
U+8BA7.0	fUTF8	讧
U+8BA7.0	fGB	0-5A27
U+8BA7.0	fR/S	 149.3
U+8BA7.0	fTotalStrokes	 5
U+8BA7.0	fCangjie	IVM
U+8BA7.0	fHakka	 fung2; fung1
U+8BA7.0	fCantonese	hung6; hung3 (also); hung4
U+8BA7.0	fMandarin	hong2; hong4; gong1
U+8BA7.0	fEnglish	simplified form of &#x8A0C;
U+8BA8.0	fUTF8	讨
U+8BA8.0	fGB	0-4C56
U+8BA8.0	fR/S	 149.3
U+8BA8.0	fTotalStrokes	 5
U+8BA8.0	fCangjie	IVDI
U+8BA8.0	fHakka	 tau3; to3
U+8BA8.0	fCantonese	tou2
U+8BA8.0	fMandarin	tao3
U+8BA8.0	fEnglish	simplified form of &#x8A0E;
U+8BA9.0	fUTF8	让
U+8BA9.0	fGB	0-4843
U+8BA9.0	fR/S	 149.3
U+8BA9.0	fTotalStrokes	 5
U+8BA9.0	fCangjie	IVYM
U+8BA9.0	fHakka	 jong5; ngiong5
U+8BA9.0	fCantonese	joeng6
U+8BA9.0	fMandarin	rang4
U+8BA9.0	fEnglish	simplified form of &#x8B93;
U+8BAA.0	fUTF8	讪
U+8BAA.0	fGB	0-5A28
U+8BAA.0	fR/S	 149.3
U+8BAA.0	fTotalStrokes	 5
U+8BAA.0	fCangjie	IVU
U+8BAA.0	fHakka	 san1; san5
U+8BAA.0	fCantonese	saan3
U+8BAA.0	fMandarin	shan4; shan1
U+8BAA.0	fEnglish	simplified form of &#x8A15;
U+8BAB.0	fUTF8	讫
U+8BAB.0	fGB	0-467D
U+8BAB.0	fR/S	 149.3
U+8BAB.0	fTotalStrokes	 5
U+8BAB.0	fCangjie	IVON
U+8BAB.0	fHakka	 ngiet7
U+8BAB.0	fCantonese	ngat6; gat1 (also); at6 (also)
U+8BAB.0	fMandarin	qi4; ji4; xi4
U+8BAB.0	fEnglish	simplified form of &#x8A16;
U+8BAC.0	fUTF8	讬
U+8BAC.0	fGB	8-2C77
U+8BAC.0	fR/S	 149.3
U+8BAC.0	fTotalStrokes	 5
U+8BAC.0	fCangjie	IVHP
U+8BAC.0	fCantonese	tok3
U+8BAC.0	fMandarin	tuo1
U+8BAC.0	fEnglish	entrust, rely on; commission
U+8BAD.0	fUTF8	训
U+8BAD.0	fGB	0-5135
U+8BAD.0	fR/S	 149.3
U+8BAD.0	fTotalStrokes	 5
U+8BAD.0	fCangjie	IVLLL
U+8BAD.0	fHakka	 hiun5
U+8BAD.0	fCantonese	fan3
U+8BAD.0	fMandarin	xun4; xun2
U+8BAD.0	fEnglish	simplified form of &#x8A13;
U+8BAD.1	fUTF8	训
U+8BAD.1	fGB	0-5135
U+8BAD.1	fR/S	 149.3
U+8BAD.1	fTotalStrokes	 5
U+8BAD.1	fCangjie	IVLLL
U+8BAD.1	fMandarin	xun5
U+8BAE.0	fUTF8	议
U+8BAE.0	fGB	0-5269
U+8BAE.0	fR/S	 149.3
U+8BAE.0	fTotalStrokes	 5
U+8BAE.0	fCangjie	IVIK
U+8BAE.0	fHakka	 ngi5; {ngi1}; ni3
U+8BAE.0	fCantonese	ji5; ji3 (also); ji6
U+8BAE.0	fMandarin	yi4; yi2
U+8BAE.0	fEnglish	simplified form of &#x8B70;
U+8BAF.0	fUTF8	讯
U+8BAF.0	fGB	0-5136
U+8BAF.0	fR/S	 149.3
U+8BAF.0	fTotalStrokes	 5
U+8BAF.0	fCangjie	IVNJ
U+8BAF.0	fCantonese	seon3
U+8BAF.0	fMandarin	xun4; sui4; xin4
U+8BAF.0	fEnglish	simplified form of &#x8A0A;
U+8BB0.0	fUTF8	记
U+8BB0.0	fGB	0-3C47
U+8BB0.0	fR/S	 149.3
U+8BB0.0	fTotalStrokes	 5
U+8BB0.0	fCangjie	IVSU
U+8BB0.0	fHakka	 gi5
U+8BB0.0	fCantonese	gei3
U+8BB0.0	fMandarin	ji4
U+8BB0.0	fEnglish	simplified form of &#x8A18;
U+8BB0.1	fUTF8	记
U+8BB0.1	fGB	0-3C47
U+8BB0.1	fR/S	 149.3
U+8BB0.1	fTotalStrokes	 5
U+8BB0.1	fCangjie	IVSU
U+8BB0.1	fMandarin	ji5
U+8BB1.0	fUTF8	讱
U+8BB1.0	fGB	8-7E72
U+8BB1.0	fR/S	 149.3
U+8BB1.0	fTotalStrokes	 5
U+8BB1.0	fCangjie	IVSHI
U+8BB1.0	fCantonese	jan6
U+8BB1.0	fMandarin	ren4
U+8BB1.0	fEnglish	hesitate to say, reluctant to speak
U+8BB2.0	fUTF8	讲
U+8BB2.0	fGB	0-3D32
U+8BB2.0	fR/S	 149.4
U+8BB2.0	fTotalStrokes	 6
U+8BB2.0	fCangjie	IVTT
U+8BB2.0	fHakka	 gong3
U+8BB2.0	fCantonese	gong2
U+8BB2.0	fMandarin	jiang3; gou4
U+8BB2.0	fEnglish	simplified form of &#x8B1B;
U+8BB3.0	fUTF8	讳
U+8BB3.0	fGB	0-3B64
U+8BB3.0	fR/S	 149.4
U+8BB3.0	fTotalStrokes	 6
U+8BB3.0	fCangjie	IVQS
U+8BB3.0	fHakka	 vui3; fui5
U+8BB3.0	fCantonese	wai5
U+8BB3.0	fMandarin	hui4
U+8BB3.0	fEnglish	simplified form of &#x8AF1;
U+8BB4.0	fUTF8	讴
U+8BB4.0	fGB	0-5A29
U+8BB4.0	fR/S	 149.4
U+8BB4.0	fTotalStrokes	 6
U+8BB4.0	fCangjie	IVSK
U+8BB4.0	fHakka	 eu1
U+8BB4.0	fCantonese	au1; ngau1 (also)
U+8BB4.0	fMandarin	ou1; xu1; yu1
U+8BB4.0	fEnglish	simplified form of &#x8B33;
U+8BB5.0	fUTF8	讵
U+8BB5.0	fGB	0-5A2A
U+8BB5.0	fR/S	 149.5; 149.4
U+8BB5.0	fTotalStrokes	 6
U+8BB5.0	fCangjie	IVSS
U+8BB5.0	fHakka	 ki3
U+8BB5.0	fCantonese	geoi6
U+8BB5.0	fMandarin	ju4
U+8BB5.0	fEnglish	simplified form of &#x8A4E;
U+8BB6.0	fUTF8	讶
U+8BB6.0	fGB	0-5148
U+8BB6.0	fR/S	 149.4
U+8BB6.0	fTotalStrokes	 6
U+8BB6.0	fCangjie	IVMVH
U+8BB6.0	fHakka	 nga3; nga1
U+8BB6.0	fCantonese	ngaa6; aa6 (also)
U+8BB6.0	fMandarin	ya4
U+8BB6.0	fEnglish	simplified form of &#x8A1D;
U+8BB7.0	fUTF8	讷
U+8BB7.0	fGB	0-5A2B
U+8BB7.0	fR/S	 149.4
U+8BB7.0	fTotalStrokes	 6
U+8BB7.0	fCangjie	IVOB
U+8BB7.0	fHakka	 nut8
U+8BB7.0	fCantonese	neot6; leot6 (also); naap6 (also); nat6 (also); laap6 (also); lat6 (also); leo6
U+8BB7.0	fMandarin	ne4; na4
U+8BB7.0	fEnglish	simplified form of &#x8A25;
U+8BB8.0	fUTF8	许
U+8BB8.0	fGB	0-506D
U+8BB8.0	fR/S	 149.4
U+8BB8.0	fTotalStrokes	 6
U+8BB8.0	fCangjie	IVOJ
U+8BB8.0	fHakka	 hi3
U+8BB8.0	fCantonese	heoi2; fu2
U+8BB8.0	fMandarin	xu3; hu3
U+8BB8.0	fEnglish	simplified form of &#x8A31;
U+8BB9.0	fUTF8	讹
U+8BB9.0	fGB	0-366F
U+8BB9.0	fR/S	 149.4
U+8BB9.0	fTotalStrokes	 6
U+8BB9.0	fCangjie	XIVOP
U+8BB9.0	fHakka	 ngo2
U+8BB9.0	fCantonese	ngo4; o4
U+8BB9.0	fMandarin	e2
U+8BB9.0	fEnglish	simplified form of &#x8A1B;
U+8BBA.0	fUTF8	论
U+8BBA.0	fGB	0-425B
U+8BBA.0	fR/S	 149.4
U+8BBA.0	fTotalStrokes	 6
U+8BBA.0	fCangjie	IVOP
U+8BBA.0	fHakka	 lun5
U+8BBA.0	fCantonese	leon6; leon2 (also); leon4 (also)
U+8BBA.0	fMandarin	lun4; lun2
U+8BBA.0	fEnglish	simplified form of &#x8AD6;
U+8BBB.0	fUTF8	讻
U+8BBB.0	fGB	8-7E73
U+8BBB.0	fR/S	 149.4
U+8BBB.0	fTotalStrokes	 6
U+8BBB.0	fCangjie	IVUK
U+8BBB.0	fCantonese	hung1
U+8BBB.0	fMandarin	xiong1
U+8BBB.0	fEnglish	noisily; uproarious; troublesome
U+8BBC.0	fUTF8	讼
U+8BBC.0	fGB	0-4B4F
U+8BBC.0	fR/S	 149.4
U+8BBC.0	fTotalStrokes	 6
U+8BBC.0	fCangjie	IVCI
U+8BBC.0	fHakka	 siung5
U+8BBC.0	fCantonese	zung6
U+8BBC.0	fMandarin	song4; song1; rong2
U+8BBC.0	fEnglish	simplified form of &#x8A1F;
U+8BBD.0	fUTF8	讽
U+8BBD.0	fGB	0-376D
U+8BBD.0	fR/S	 149.4
U+8BBD.0	fTotalStrokes	 6
U+8BBD.0	fCangjie	IVHNK
U+8BBD.0	fHakka	 fung3; fung5
U+8BBD.0	fCantonese	fung3
U+8BBD.0	fMandarin	feng1; feng3; feng4
U+8BBD.0	fEnglish	simplified form of &#x8AF7;
U+8BBE.0	fUTF8	设
U+8BBE.0	fGB	0-4968
U+8BBE.0	fR/S	 149.4
U+8BBE.0	fTotalStrokes	 6
U+8BBE.0	fCangjie	IVHNE
U+8BBE.0	fHakka	 s'et7
U+8BBE.0	fCantonese	cit3
U+8BBE.0	fMandarin	she4
U+8BBE.0	fEnglish	simplified form of &#x8A2D;
U+8BBF.0	fUTF8	访
U+8BBF.0	fGB	0-3743
U+8BBF.0	fR/S	 149.4
U+8BBF.0	fTotalStrokes	 6
U+8BBF.0	fCangjie	IVYHS
U+8BBF.0	fHakka	 fong3; fong5
U+8BBF.0	fCantonese	fong2
U+8BBF.0	fMandarin	fang3; fang4
U+8BBF.0	fEnglish	simplified form of &#x8A2A;
U+8BC0.0	fUTF8	诀
U+8BC0.0	fGB	0-3E77
U+8BC0.0	fR/S	 149.4
U+8BC0.0	fTotalStrokes	 6
U+8BC0.0	fCangjie	IVDK
U+8BC0.0	fHakka	 get7; giet7
U+8BC0.0	fCantonese	kyut3
U+8BC0.0	fMandarin	jue2; gui4
U+8BC0.0	fEnglish	simplified form of &#x8A23;
U+8BC1.0	fUTF8	证
U+8BC1.0	fGB	0-5624
U+8BC1.0	fR/S	 149.5
U+8BC1.0	fTotalStrokes	 7
U+8BC1.0	fCangjie	IVMYM
U+8BC1.0	fHakka	 zhin5
U+8BC1.0	fCantonese	zing3
U+8BC1.0	fMandarin	zheng4
U+8BC1.0	fEnglish	simplified form of &#x8B49;
U+8BC2.0	fUTF8	诂
U+8BC2.0	fGB	0-5A2C
U+8BC2.0	fR/S	 149.5
U+8BC2.0	fTotalStrokes	 7
U+8BC2.0	fCangjie	IVJR
U+8BC2.0	fHakka	 gu3
U+8BC2.0	fCantonese	gu2; gwu2 (also); gu3 (also); gwu3 (also)
U+8BC2.0	fMandarin	gu3; gu4
U+8BC2.0	fEnglish	simplified form of &#x8A41;
U+8BC3.0	fUTF8	诃
U+8BC3.0	fGB	0-5A2D
U+8BC3.0	fR/S	 149.5
U+8BC3.0	fTotalStrokes	 7
U+8BC3.0	fCangjie	IVMNR
U+8BC3.0	fHakka	 ko1; o1
U+8BC3.0	fCantonese	ho1
U+8BC3.0	fMandarin	he1
U+8BC3.0	fEnglish	simplified form of &#x8A36;
U+8BC4.0	fUTF8	评
U+8BC4.0	fGB	0-4640
U+8BC4.0	fR/S	 149.5
U+8BC4.0	fTotalStrokes	 7
U+8BC4.0	fCangjie	IVMFJ
U+8BC4.0	fHakka	 pin2
U+8BC4.0	fCantonese	ping4
U+8BC4.0	fMandarin	ping2; bing4
U+8BC4.0	fEnglish	simplified form of &#x8A55;
U+8BC5.0	fUTF8	诅
U+8BC5.0	fGB	0-5767
U+8BC5.0	fR/S	 149.5
U+8BC5.0	fTotalStrokes	 7
U+8BC5.0	fCangjie	IVBM
U+8BC5.0	fHakka	 zih3; zih1
U+8BC5.0	fCantonese	zo3; zo2 (also)
U+8BC5.0	fMandarin	zu3; zu4
U+8BC5.0	fEnglish	simplified form of &#x8A5B;
U+8BC6.0	fUTF8	识
U+8BC6.0	fGB	0-4A36
U+8BC6.0	fR/S	 149.5
U+8BC6.0	fTotalStrokes	 7
U+8BC6.0	fCangjie	IVRC
U+8BC6.0	fHakka	 shit7; zhi5
U+8BC6.0	fCantonese	sik1; zi3; hei1
U+8BC6.0	fMandarin	shi4; shi2; chi4; zhi4
U+8BC6.0	fEnglish	simplified form of &#x8B58;
U+8BC6.1	fUTF8	识
U+8BC6.1	fGB	0-4A36
U+8BC6.1	fR/S	 149.5
U+8BC6.1	fTotalStrokes	 7
U+8BC6.1	fCangjie	IVRC
U+8BC6.1	fMandarin	shi5
U+8BC7.0	fUTF8	诇
U+8BC7.0	fGB	8-7E74
U+8BC7.0	fR/S	 149.5
U+8BC7.0	fTotalStrokes	 7
U+8BC7.0	fCangjie	IVBR
U+8BC7.0	fCantonese	gwing2; hing3 (also)
U+8BC7.0	fMandarin	xiong4
U+8BC7.0	fEnglish	to spy; to give information; shrewd
U+8BC8.0	fUTF8	诈
U+8BC8.0	fGB	0-5529
U+8BC8.0	fR/S	 149.5
U+8BC8.0	fTotalStrokes	 7
U+8BC8.0	fCangjie	IVOS
U+8BC8.0	fHakka	 za5
U+8BC8.0	fCantonese	zaa3
U+8BC8.0	fMandarin	zha4; zha3
U+8BC8.0	fEnglish	simplified form of &#x8A50;
U+8BC9.0	fUTF8	诉
U+8BC9.0	fGB	0-4B5F
U+8BC9.0	fR/S	 149.5
U+8BC9.0	fTotalStrokes	 7
U+8BC9.0	fCangjie	IVHMY
U+8BC9.0	fHakka	 sii5
U+8BC9.0	fCantonese	sou3
U+8BC9.0	fMandarin	su4; chi3
U+8BC9.0	fEnglish	simplified form of &#x8A34;
U+8BC9.1	fUTF8	诉
U+8BC9.1	fGB	0-4B5F
U+8BC9.1	fR/S	 149.5
U+8BC9.1	fTotalStrokes	 7
U+8BC9.1	fCangjie	IVHMY
U+8BC9.1	fMandarin	su5
U+8BCA.0	fUTF8	诊
U+8BCA.0	fGB	0-556F
U+8BCA.0	fR/S	 149.5
U+8BCA.0	fTotalStrokes	 7
U+8BCA.0	fCangjie	IVOHH
U+8BCA.0	fHakka	 zhin3
U+8BCA.0	fCantonese	can2; zan2 (also)
U+8BCA.0	fMandarin	zhen3; zhen1; zhen4
U+8BCA.0	fEnglish	simplified form of &#x8A3A;
U+8BCB.0	fUTF8	诋
U+8BCB.0	fGB	0-5A2E
U+8BCB.0	fR/S	 149.5
U+8BCB.0	fTotalStrokes	 7
U+8BCB.0	fCangjie	IVHVI
U+8BCB.0	fHakka	 di3
U+8BCB.0	fCantonese	dai2
U+8BCB.0	fMandarin	di3; di1; di4; ti2; ti4
U+8BCB.0	fEnglish	simplified form of &#x8A46;
U+8BCC.0	fUTF8	诌
U+8BCC.0	fGB	0-565F
U+8BCC.0	fR/S	 149.5
U+8BCC.0	fTotalStrokes	 7
U+8BCC.0	fCangjie	IVNSM
U+8BCC.0	fHakka	 ceu1; cau3
U+8BCC.0	fCantonese	zau1; zau3 (also)
U+8BCC.0	fMandarin	zou1; zhou1; jie4
U+8BCC.0	fEnglish	simplified form of &#x8B05;
U+8BCD.0	fUTF8	词
U+8BCD.0	fGB	0-344A
U+8BCD.0	fR/S	 149.5
U+8BCD.0	fTotalStrokes	 7
U+8BCD.0	fCangjie	IVSMR
U+8BCD.0	fHakka	 cih2
U+8BCD.0	fCantonese	ci4
U+8BCD.0	fMandarin	ci2
U+8BCD.0	fEnglish	simplified form of &#x8A5E;
U+8BCE.0	fUTF8	诎
U+8BCE.0	fGB	0-5A30
U+8BCE.0	fR/S	 149.5
U+8BCE.0	fTotalStrokes	 7
U+8BCE.0	fCangjie	IVUU
U+8BCE.0	fHakka	 kiut7
U+8BCE.0	fCantonese	wat1; zeot1
U+8BCE.0	fMandarin	qu1; chu4; qu4
U+8BCE.0	fEnglish	simplified form of &#x8A58;
U+8BCF.0	fUTF8	诏
U+8BCF.0	fGB	0-5A2F
U+8BCF.0	fR/S	 149.5
U+8BCF.0	fTotalStrokes	 7
U+8BCF.0	fCangjie	IVSHR
U+8BCF.0	fHakka	 zhau5; cheu5
U+8BCF.0	fCantonese	ziu3; ziu6 (also)
U+8BCF.0	fMandarin	zhao4; zhao1
U+8BCF.0	fEnglish	simplified form of &#x8A54;
U+8BD0.0	fUTF8	诐
U+8BD0.0	fGB	7-2133
U+8BD0.0	fR/S	 149.5
U+8BD0.0	fTotalStrokes	 7
U+8BD0.0	fCangjie	IVDHE
U+8BD0.0	fCantonese	bei3; bei1
U+8BD0.0	fMandarin	bi4
U+8BD0.0	fEnglish	argue; biased; one-sided
U+8BD1.0	fUTF8	译
U+8BD1.0	fGB	0-526B
U+8BD1.0	fR/S	 149.5
U+8BD1.0	fTotalStrokes	 7
U+8BD1.0	fCangjie	IVEQ
U+8BD1.0	fHakka	 jit8
U+8BD1.0	fCantonese	jik6; jik2 (also)
U+8BD1.0	fMandarin	yi4
U+8BD1.0	fEnglish	simplified form of &#x8B6F;
U+8BD2.0	fUTF8	诒
U+8BD2.0	fGB	0-5A31
U+8BD2.0	fR/S	 149.5
U+8BD2.0	fTotalStrokes	 7
U+8BD2.0	fCangjie	IVIR
U+8BD2.0	fHakka	 ji2; toi2
U+8BD2.0	fCantonese	ji4; doi6
U+8BD2.0	fMandarin	yi2; dai4; tai2; tai3; yi4
U+8BD2.0	fEnglish	simplified form of &#x8A52;
U+8BD3.0	fUTF8	诓
U+8BD3.0	fGB	0-5A32
U+8BD3.0	fR/S	 149.6
U+8BD3.0	fTotalStrokes	 8
U+8BD3.0	fCangjie	IVSMG
U+8BD3.0	fHakka	 k'wong5; {kiong1}
U+8BD3.0	fCantonese	hong1; kwong4; kong4 (also); gwong2 (also); gong2 (also)
U+8BD3.0	fMandarin	kuang1; guang4; kuang2; kuang4
U+8BD3.0	fEnglish	simplified form of &#x8A86;
U+8BD4.0	fUTF8	诔
U+8BD4.0	fGB	0-5A33
U+8BD4.0	fR/S	 149.6
U+8BD4.0	fTotalStrokes	 8
U+8BD4.0	fCangjie	IVQD
U+8BD4.0	fHakka	 loi3
U+8BD4.0	fCantonese	loi6; leoi6 (also)
U+8BD4.0	fMandarin	lei3
U+8BD4.0	fEnglish	simplified form of &#x8A84;
U+8BD5.0	fUTF8	试
U+8BD5.0	fGB	0-4A54
U+8BD5.0	fR/S	 149.6
U+8BD5.0	fTotalStrokes	 8
U+8BD5.0	fCangjie	IVIPM
U+8BD5.0	fHakka	 chi5; shi5
U+8BD5.0	fCantonese	si3
U+8BD5.0	fMandarin	shi4
U+8BD5.0	fEnglish	simplified form of &#x8A66;
U+8BD6.0	fUTF8	诖
U+8BD6.0	fGB	0-5A34
U+8BD6.0	fR/S	 149.6
U+8BD6.0	fTotalStrokes	 8
U+8BD6.0	fCangjie	IVGG
U+8BD6.0	fHakka	 gwa5
U+8BD6.0	fCantonese	gwaa3
U+8BD6.0	fMandarin	gua4; hua4
U+8BD6.0	fEnglish	simplified form of &#x8A7F;
U+8BD7.0	fUTF8	诗
U+8BD7.0	fGB	0-4A2B
U+8BD7.0	fR/S	 149.6
U+8BD7.0	fTotalStrokes	 8
U+8BD7.0	fCangjie	IVGDI
U+8BD7.0	fHakka	 shi1
U+8BD7.0	fCantonese	si1
U+8BD7.0	fMandarin	shi1
U+8BD7.0	fEnglish	simplified form of &#x8A69;
U+8BD8.0	fUTF8	诘
U+8BD8.0	fGB	0-5A35
U+8BD8.0	fR/S	 149.6
U+8BD8.0	fTotalStrokes	 8
U+8BD8.0	fCangjie	IVGR
U+8BD8.0	fHakka	 get7; giet7
U+8BD8.0	fCantonese	kit3; gat1
U+8BD8.0	fMandarin	jie2; qie4
U+8BD8.0	fEnglish	simplified form of &#x8A70;
U+8BD9.0	fUTF8	诙
U+8BD9.0	fGB	0-5A36
U+8BD9.0	fR/S	 149.6
U+8BD9.0	fTotalStrokes	 8
U+8BD9.0	fCangjie	IVKF
U+8BD9.0	fHakka	 foi1; k'wui1
U+8BD9.0	fCantonese	fui1
U+8BD9.0	fMandarin	hui1; kui1
U+8BD9.0	fEnglish	simplified form of &#x8A7C;
U+8BDA.0	fUTF8	诚
U+8BDA.0	fGB	0-334F
U+8BDA.0	fR/S	 149.6
U+8BDA.0	fTotalStrokes	 8
U+8BDA.0	fCangjie	IVIHS
U+8BDA.0	fHakka	 shin2
U+8BDA.0	fCantonese	sing4; sing2 (also)
U+8BDA.0	fMandarin	cheng2
U+8BDA.0	fEnglish	simplified form of &#x8AA0;
U+8BDB.0	fUTF8	诛
U+8BDB.0	fGB	0-566F
U+8BDB.0	fR/S	 149.6
U+8BDB.0	fTotalStrokes	 8
U+8BDB.0	fCangjie	IVHJD
U+8BDB.0	fHakka	 zhu1
U+8BDB.0	fCantonese	zyu1
U+8BDB.0	fMandarin	zhu1
U+8BDB.0	fEnglish	simplified form of &#x8A85;
U+8BDC.0	fUTF8	诜
U+8BDC.0	fGB	0-5A37
U+8BDC.0	fR/S	 149.6
U+8BDC.0	fTotalStrokes	 8
U+8BDC.0	fCangjie	IVHGU
U+8BDC.0	fHakka	 sin1
U+8BDC.0	fCantonese	san1
U+8BDC.0	fMandarin	shen1; xian4
U+8BDC.0	fEnglish	simplified form of &#x8A75;
U+8BDD.0	fUTF8	话
U+8BDD.0	fGB	0-3B30
U+8BDD.0	fR/S	 149.6
U+8BDD.0	fTotalStrokes	 8
U+8BDD.0	fCangjie	IVHJR
U+8BDD.0	fHakka	 fa5; va5
U+8BDD.0	fCantonese	waa6; waa2 (also)
U+8BDD.0	fMandarin	hua4
U+8BDD.0	fEnglish	simplified form of &#x8A71;
U+8BDD.1	fUTF8	话
U+8BDD.1	fGB	0-3B30
U+8BDD.1	fR/S	 149.6
U+8BDD.1	fTotalStrokes	 8
U+8BDD.1	fCangjie	IVHJR
U+8BDD.1	fMandarin	hua5
U+8BDE.0	fUTF8	诞
U+8BDE.0	fGB	0-352E
U+8BDE.0	fR/S	 149.8; 149.6
U+8BDE.0	fTotalStrokes	 8
U+8BDE.0	fCangjie	IVNKV
U+8BDE.0	fHakka	 dan5
U+8BDE.0	fCantonese	daan3
U+8BDE.0	fMandarin	dan4
U+8BDE.0	fEnglish	simplified form of &#x8A95;
U+8BDF.0	fUTF8	诟
U+8BDF.0	fGB	0-5A38
U+8BDF.0	fR/S	 149.6
U+8BDF.0	fTotalStrokes	 8
U+8BDF.0	fCangjie	IVHMR
U+8BDF.0	fHakka	 geu5
U+8BDF.0	fCantonese	gau3; gau2; hau2; kau3
U+8BDF.0	fMandarin	gou4; gou3; hou4; kou4
U+8BDF.0	fEnglish	simplified form of &#x8A6C;
U+8BE0.0	fUTF8	诠
U+8BE0.0	fGB	0-5A39
U+8BE0.0	fR/S	 149.6
U+8BE0.0	fTotalStrokes	 8
U+8BE0.0	fCangjie	IVOMG
U+8BE0.0	fHakka	 cien2; zien1
U+8BE0.0	fCantonese	cyun4
U+8BE0.0	fMandarin	quan2
U+8BE0.0	fEnglish	simplified form of &#x8A6E;
U+8BE1.0	fUTF8	诡
U+8BE1.0	fGB	0-396E
U+8BE1.0	fR/S	 149.6
U+8BE1.0	fTotalStrokes	 8
U+8BE1.0	fCangjie	IVNMU
U+8BE1.0	fHakka	 kwui3
U+8BE1.0	fCantonese	gwai2
U+8BE1.0	fMandarin	gui3
U+8BE1.0	fEnglish	simplified form of &#x8A6D;
U+8BE2.0	fUTF8	询
U+8BE2.0	fGB	0-512F
U+8BE2.0	fR/S	 149.6
U+8BE2.0	fTotalStrokes	 8
U+8BE2.0	fCangjie	IVPA
U+8BE2.0	fHakka	 sun2
U+8BE2.0	fCantonese	seon1
U+8BE2.0	fMandarin	xun2
U+8BE2.0	fEnglish	simplified form of &#x8A62;
U+8BE3.0	fUTF8	诣
U+8BE3.0	fGB	0-5268
U+8BE3.0	fR/S	 149.6
U+8BE3.0	fTotalStrokes	 8
U+8BE3.0	fCangjie	XIVPA
U+8BE3.0	fHakka	 ji5; ngi5
U+8BE3.0	fCantonese	ngai6; ai6 (also)
U+8BE3.0	fMandarin	yi4; ye4
U+8BE3.0	fEnglish	simplified form of &#x8A63;
U+8BE4.0	fUTF8	诤
U+8BE4.0	fGB	0-5A3A
U+8BE4.0	fR/S	 149.6
U+8BE4.0	fTotalStrokes	 8
U+8BE4.0	fCangjie	IVNSD
U+8BE4.0	fHakka	 zen5
U+8BE4.0	fCantonese	zaang3; zang3 (also); zaang1; zang1 (also)
U+8BE4.0	fMandarin	zheng1; zheng4
U+8BE4.0	fEnglish	simplified form of &#x8ACD;
U+8BE5.0	fUTF8	该
U+8BE5.0	fGB	0-3843
U+8BE5.0	fR/S	 149.6
U+8BE5.0	fTotalStrokes	 8
U+8BE5.0	fCangjie	IVYVO
U+8BE5.0	fHakka	 goi1; gai1
U+8BE5.0	fCantonese	goi1
U+8BE5.0	fMandarin	gai1
U+8BE5.0	fEnglish	simplified form of &#x8A72;
U+8BE6.0	fUTF8	详
U+8BE6.0	fGB	0-4F6A
U+8BE6.0	fR/S	 149.6
U+8BE6.0	fTotalStrokes	 8
U+8BE6.0	fCangjie	IVTQ
U+8BE6.0	fHakka	 siong2
U+8BE6.0	fCantonese	coeng4
U+8BE6.0	fMandarin	xiang2; yang2
U+8BE6.0	fEnglish	simplified form of &#x8A73;
U+8BE7.0	fUTF8	诧
U+8BE7.0	fGB	0-326F
U+8BE7.0	fR/S	 149.6
U+8BE7.0	fTotalStrokes	 8
U+8BE7.0	fCangjie	IVJHP
U+8BE7.0	fHakka	 za5; ca5
U+8BE7.0	fCantonese	caa3
U+8BE7.0	fMandarin	cha4; du4; xia4
U+8BE7.0	fEnglish	simplified form of &#x8A6B;
U+8BE8.0	fUTF8	诨
U+8BE8.0	fGB	0-5A3B
U+8BE8.0	fR/S	 149.6
U+8BE8.0	fTotalStrokes	 8
U+8BE8.0	fCangjie	IVBKQ
U+8BE8.0	fHakka	 fun5
U+8BE8.0	fCantonese	wan6
U+8BE8.0	fMandarin	hun4; wen4
U+8BE8.0	fEnglish	simplified form of &#x8AE2;
U+8BE9.0	fUTF8	诩
U+8BE9.0	fGB	0-5A3C
U+8BE9.0	fR/S	 149.6
U+8BE9.0	fTotalStrokes	 8
U+8BE9.0	fCangjie	IVSMM
U+8BE9.0	fHakka	 hi3
U+8BE9.0	fCantonese	heoi2
U+8BE9.0	fMandarin	xu3
U+8BE9.0	fEnglish	simplified form of &#x8A61;
U+8BEA.0	fUTF8	诪
U+8BEA.0	fGB	8-7E75
U+8BEA.0	fR/S	 149.14; 149.7
U+8BEA.0	fTotalStrokes	 9
U+8BEA.0	fCangjie	IVQKI
U+8BEA.0	fCantonese	zau1
U+8BEA.0	fMandarin	zhou1
U+8BEA.0	fEnglish	deceive, cheat; hurried, bustling
U+8BEB.0	fUTF8	诫
U+8BEB.0	fGB	0-3D6B
U+8BEB.0	fR/S	 149.7
U+8BEB.0	fTotalStrokes	 9
U+8BEB.0	fCangjie	IVIT
U+8BEB.0	fHakka	 gai5
U+8BEB.0	fCantonese	gaai3
U+8BEB.0	fMandarin	jie4
U+8BEB.0	fEnglish	simplified form of &#x8AA1;
U+8BEC.0	fUTF8	诬
U+8BEC.0	fGB	0-4E5C
U+8BEC.0	fR/S	 149.7
U+8BEC.0	fTotalStrokes	 9
U+8BEC.0	fCangjie	IVMOO
U+8BEC.0	fHakka	 mu2
U+8BEC.0	fCantonese	mou4
U+8BEC.0	fMandarin	wu1; wu2
U+8BEC.0	fEnglish	simplified form of &#x8AA3;
U+8BED.0	fUTF8	语
U+8BED.0	fGB	0-536F
U+8BED.0	fR/S	 149.7
U+8BED.0	fTotalStrokes	 9
U+8BED.0	fCangjie	IVMMR
U+8BED.0	fHakka	 ngi1; ngi3
U+8BED.0	fCantonese	jyu5; jyu6
U+8BED.0	fMandarin	yu3; yu4
U+8BED.0	fEnglish	simplified form of &#x8A9E;
U+8BEE.0	fUTF8	诮
U+8BEE.0	fGB	0-5A3D
U+8BEE.0	fR/S	 149.7
U+8BEE.0	fTotalStrokes	 9
U+8BEE.0	fCangjie	IVFB
U+8BEE.0	fHakka	 ciau3; ciau5
U+8BEE.0	fCantonese	ciu3
U+8BEE.0	fMandarin	qiao4
U+8BEE.0	fEnglish	simplified form of &#x8A9A;
U+8BEF.0	fUTF8	误
U+8BEF.0	fGB	0-4E73
U+8BEF.0	fR/S	 149.7
U+8BEF.0	fTotalStrokes	 9
U+8BEF.0	fCangjie	IVRMK
U+8BEF.0	fHakka	 ngu5
U+8BEF.0	fCantonese	ng6
U+8BEF.0	fMandarin	wu4
U+8BEF.0	fEnglish	simplified form of &#x8AA4;
U+8BEF.1	fUTF8	误
U+8BEF.1	fGB	0-4E73
U+8BEF.1	fR/S	 149.7
U+8BEF.1	fTotalStrokes	 9
U+8BEF.1	fCangjie	IVRMK
U+8BEF.1	fMandarin	wu5
U+8BF0.0	fUTF8	诰
U+8BF0.0	fGB	0-5A3E
U+8BF0.0	fR/S	 149.7
U+8BF0.0	fTotalStrokes	 9
U+8BF0.0	fCangjie	IVHGR
U+8BF0.0	fHakka	 kau5; gau1
U+8BF0.0	fCantonese	gou3
U+8BF0.0	fMandarin	gao4; gu4
U+8BF0.0	fEnglish	simplified form of &#x8AA5;
U+8BF1.0	fUTF8	诱
U+8BF1.0	fGB	0-5355
U+8BF1.0	fR/S	 149.7
U+8BF1.0	fTotalStrokes	 9
U+8BF1.0	fCangjie	IVHDS
U+8BF1.0	fHakka	 ju3
U+8BF1.0	fCantonese	jau5
U+8BF1.0	fMandarin	you4; you3
U+8BF1.0	fEnglish	simplified form of &#x8A98;
U+8BF2.0	fUTF8	诲
U+8BF2.0	fGB	0-3B65
U+8BF2.0	fR/S	 149.7
U+8BF2.0	fTotalStrokes	 9
U+8BF2.0	fCangjie	IVOWY
U+8BF2.0	fHakka	 fui5
U+8BF2.0	fCantonese	fui3
U+8BF2.0	fMandarin	hui4; hui3
U+8BF2.0	fEnglish	simplified form of &#x8AA8;
U+8BF3.0	fUTF8	诳
U+8BF3.0	fGB	0-5A3F
U+8BF3.0	fR/S	 149.7
U+8BF3.0	fTotalStrokes	 9
U+8BF3.0	fCangjie	IVKHG
U+8BF3.0	fHakka	 k'wong2; k'wong5
U+8BF3.0	fCantonese	gwong2; gong2 (also); kong4 (also); kwong4 (also)
U+8BF3.0	fMandarin	kuang2; guang4; kuang4
U+8BF3.0	fEnglish	simplified form of &#x8A91;
U+8BF4.0	fUTF8	说
U+8BF4.0	fGB	0-4B35
U+8BF4.0	fR/S	 149.7
U+8BF4.0	fTotalStrokes	 9
U+8BF4.0	fCangjie	IVCRU
U+8BF4.0	fCantonese	syut3; seoi3; jyut6
U+8BF4.0	fMandarin	shuo1; shui4; yue4; tuo1
U+8BF4.0	fEnglish	simplified form of &#x8AAC;
U+8BF5.0	fUTF8	诵
U+8BF5.0	fGB	0-4B50
U+8BF5.0	fR/S	 149.7
U+8BF5.0	fTotalStrokes	 9
U+8BF5.0	fCangjie	IVNIB
U+8BF5.0	fHakka	 siung5
U+8BF5.0	fCantonese	zung6
U+8BF5.0	fMandarin	song4
U+8BF5.0	fEnglish	simplified form of &#x8AA6;
U+8BF6.0	fUTF8	诶
U+8BF6.0	fGB	0-5A40
U+8BF6.0	fR/S	 149.7
U+8BF6.0	fTotalStrokes	 9
U+8BF6.0	fCangjie	IVIOK
U+8BF6.0	fHakka	 hi1
U+8BF6.0	fCantonese	ei6; ngei6 (also); ei1 (also); ngei1 (also); ei3 (also); oi1; ngei3 (also); aai2; ngoi1 (also); ngaai2 (also); hei1; e6; nge6 (also)
U+8BF6.0	fMandarin	ei2; ai1; xi1; ei1; yi4; ai4; e4; hai1; e3
U+8BF6.0	fEnglish	simplified form of &#x8A92;
U+8BF6.1	fUTF8	诶
U+8BF6.1	fGB	0-5A40
U+8BF6.1	fR/S	 149.7
U+8BF6.1	fTotalStrokes	 9
U+8BF6.1	fCangjie	IVIOK
U+8BF6.1	fMandarin	ei3
U+8BF6.2	fUTF8	诶
U+8BF6.2	fGB	0-5A40
U+8BF6.2	fR/S	 149.7
U+8BF6.2	fTotalStrokes	 9
U+8BF6.2	fCangjie	IVIOK
U+8BF6.2	fMandarin	ei4
U+8BF7.0	fUTF8	请
U+8BF7.0	fGB	0-476B
U+8BF7.0	fR/S	 149.8
U+8BF7.0	fTotalStrokes	 10
U+8BF7.0	fCangjie	IVQMB
U+8BF7.0	fHakka	 ciang3
U+8BF7.0	fCantonese	cing2; ceng2 (also)
U+8BF7.0	fMandarin	qing3; qing1; qing2; jing4; qing4
U+8BF7.0	fEnglish	simplified form of &#x8ACB;
U+8BF8.0	fUTF8	诸
U+8BF8.0	fGB	0-566E
U+8BF8.0	fR/S	 149.8
U+8BF8.0	fTotalStrokes	 10
U+8BF8.0	fCangjie	IVJKA
U+8BF8.0	fHakka	 zhu1
U+8BF8.0	fCantonese	zyu1
U+8BF8.0	fMandarin	zhu1; zhe1
U+8BF8.0	fEnglish	simplified form of &#x8AF8;
U+8BF9.0	fUTF8	诹
U+8BF9.0	fGB	0-5A41
U+8BF9.0	fR/S	 149.8
U+8BF9.0	fTotalStrokes	 10
U+8BF9.0	fCangjie	IVSJE
U+8BF9.0	fHakka	 zeu1; ci1
U+8BF9.0	fCantonese	zau1
U+8BF9.0	fMandarin	zou1; fu2; ju1
U+8BF9.0	fEnglish	simplified form of &#x8ACF;
U+8BFA.0	fUTF8	诺
U+8BFA.0	fGB	0-4535
U+8BFA.0	fR/S	 149.8
U+8BFA.0	fTotalStrokes	 10
U+8BFA.0	fCangjie	IVTKR
U+8BFA.0	fHakka	 nok7
U+8BFA.0	fCantonese	nok6; lok6 (also)
U+8BFA.0	fMandarin	nuo4
U+8BFA.0	fEnglish	simplified form of &#x8AFE;
U+8BFB.0	fUTF8	读
U+8BFB.0	fGB	0-3641
U+8BFB.0	fR/S	 149.8
U+8BFB.0	fTotalStrokes	 10
U+8BFB.0	fCangjie	IVJNK
U+8BFB.0	fHakka	 tuk8; teu5
U+8BFB.0	fCantonese	duk6; dau6
U+8BFB.0	fMandarin	du2; dou4
U+8BFB.0	fEnglish	simplified form of &#x8B80;
U+8BFC.0	fUTF8	诼
U+8BFC.0	fGB	0-5A42
U+8BFC.0	fR/S	 149.8
U+8BFC.0	fTotalStrokes	 10
U+8BFC.0	fCangjie	IVMSO
U+8BFC.0	fHakka	 dok7
U+8BFC.0	fCantonese	doek3; doek6
U+8BFC.0	fMandarin	zhuo2
U+8BFC.0	fEnglish	simplified form of &#x8AD1;
U+8BFD.0	fUTF8	诽
U+8BFD.0	fGB	0-374C
U+8BFD.0	fR/S	 149.8
U+8BFD.0	fTotalStrokes	 10
U+8BFD.0	fCangjie	IVLMY
U+8BFD.0	fHakka	 fui3
U+8BFD.0	fCantonese	fei2
U+8BFD.0	fMandarin	fei3; fei1; fei4
U+8BFD.0	fEnglish	simplified form of &#x8AB9;
U+8BFE.0	fUTF8	课
U+8BFE.0	fGB	0-3F4E
U+8BFE.0	fR/S	 149.8
U+8BFE.0	fTotalStrokes	 10
U+8BFE.0	fCangjie	IVWD
U+8BFE.0	fHakka	 ko5
U+8BFE.0	fCantonese	fo3
U+8BFE.0	fMandarin	ke4; ke1
U+8BFE.0	fEnglish	simplified form of &#x8AB2;
U+8BFF.0	fUTF8	诿
U+8BFF.0	fGB	0-5A43
U+8BFF.0	fR/S	 149.8
U+8BFF.0	fTotalStrokes	 10
U+8BFF.0	fCangjie	IVHDV
U+8BFF.0	fHakka	 vui3
U+8BFF.0	fCantonese	wai2
U+8BFF.0	fMandarin	wei3; rui4; wei1; wei4
U+8BFF.0	fEnglish	simplified form of &#x8AC9;
U+8C00.0	fUTF8	谀
U+8C00.0	fGB	0-5A44
U+8C00.0	fR/S	 149.8
U+8C00.0	fTotalStrokes	 10
U+8C00.0	fCangjie	IVHXO
U+8C00.0	fHakka	 ji2; ji3
U+8C00.0	fCantonese	jyu4
U+8C00.0	fMandarin	yu2; yu4
U+8C00.0	fEnglish	simplified form of &#x8ADB;
U+8C01.0	fUTF8	谁
U+8C01.0	fGB	0-4B2D
U+8C01.0	fR/S	 149.8
U+8C01.0	fTotalStrokes	 10
U+8C01.0	fCangjie	IVOG
U+8C01.0	fHakka	 shui2
U+8C01.0	fCantonese	seoi4; seoi2 (also)
U+8C01.0	fMandarin	shei2; shui2
U+8C01.0	fEnglish	simplified form of &#x8AB0;
U+8C02.0	fUTF8	谂
U+8C02.0	fGB	0-5A45
U+8C02.0	fR/S	 149.8
U+8C02.0	fTotalStrokes	 10
U+8C02.0	fCangjie	IVOIP
U+8C02.0	fHakka	 shim3
U+8C02.0	fCantonese	sam2; nam2; lam2 (also)
U+8C02.0	fMandarin	shen3; shen4
U+8C02.0	fEnglish	simplified form of &#x8AD7;
U+8C03.0	fUTF8	调
U+8C03.0	fGB	0-3577
U+8C03.0	fR/S	 149.8
U+8C03.0	fTotalStrokes	 10
U+8C03.0	fCangjie	IVBGR
U+8C03.0	fHakka	 tiau5; tiau2
U+8C03.0	fCantonese	diu6; diu2 (also); tiu4; deu6 (also)
U+8C03.0	fMandarin	diao4; zhou1; tiao4
U+8C03.0	fEnglish	simplified form of &#x8ABF;
U+8C03.1	fUTF8	调
U+8C03.1	fGB	0-3577
U+8C03.1	fR/S	 149.8
U+8C03.1	fTotalStrokes	 10
U+8C03.1	fCangjie	IVBGR
U+8C03.1	fMandarin	tiao2
U+8C04.0	fUTF8	谄
U+8C04.0	fGB	0-5A46
U+8C04.0	fR/S	 149.8
U+8C04.0	fTotalStrokes	 10
U+8C04.0	fCangjie	IVNHX
U+8C04.0	fHakka	 cham5; cam3; tiam5
U+8C04.0	fCantonese	cim2
U+8C04.0	fMandarin	chan3
U+8C04.0	fEnglish	simplified form of &#x8AC2;
U+8C05.0	fUTF8	谅
U+8C05.0	fGB	0-4142
U+8C05.0	fR/S	 149.8
U+8C05.0	fTotalStrokes	 10
U+8C05.0	fCangjie	IVYRF
U+8C05.0	fHakka	 liong5
U+8C05.0	fCantonese	loeng6; loeng4
U+8C05.0	fMandarin	liang4; liang2
U+8C05.0	fEnglish	simplified form of &#x8AD2;
U+8C06.0	fUTF8	谆
U+8C06.0	fGB	0-573B
U+8C06.0	fR/S	 149.8
U+8C06.0	fTotalStrokes	 10
U+8C06.0	fCangjie	IVYRD
U+8C06.0	fHakka	 zhun1
U+8C06.0	fCantonese	zeon1; deon1
U+8C06.0	fMandarin	zhun1; zhun4; ma4; yi2; zhun3
U+8C06.0	fEnglish	simplified form of &#x8AC4;
U+8C07.0	fUTF8	谇
U+8C07.0	fGB	0-5A47
U+8C07.0	fR/S	 149.8
U+8C07.0	fTotalStrokes	 10
U+8C07.0	fCangjie	IVYOJ
U+8C07.0	fHakka	 cui5; sui5
U+8C07.0	fCantonese	seoi6
U+8C07.0	fMandarin	sui4; cui4
U+8C07.0	fEnglish	simplified form of &#x8AB6;
U+8C08.0	fUTF8	谈
U+8C08.0	fGB	0-4C38
U+8C08.0	fR/S	 149.8
U+8C08.0	fTotalStrokes	 10
U+8C08.0	fCangjie	IVFF
U+8C08.0	fHakka	 tam2
U+8C08.0	fCantonese	taam4
U+8C08.0	fMandarin	tan2
U+8C08.0	fEnglish	simplified form of &#x8AC7;
U+8C09.0	fUTF8	谉
U+8C09.0	fGB	8-7E76
U+8C09.0	fR/S	 149.8
U+8C09.0	fTotalStrokes	 10
U+8C09.0	fCangjie	IVJLL
U+8C09.0	fCantonese	sam2
U+8C09.0	fMandarin	shen3
U+8C0A.0	fUTF8	谊
U+8C0A.0	fGB	0-526A
U+8C0A.0	fR/S	 149.8
U+8C0A.0	fTotalStrokes	 10
U+8C0A.0	fCangjie	IVJBM
U+8C0A.0	fHakka	 ngi5; ni3
U+8C0A.0	fCantonese	ji4; ji6
U+8C0A.0	fMandarin	yi2; yi4
U+8C0A.0	fEnglish	simplified form of &#x8ABC;
U+8C0B.0	fUTF8	谋
U+8C0B.0	fGB	0-4431
U+8C0B.0	fR/S	 149.9
U+8C0B.0	fTotalStrokes	 11
U+8C0B.0	fCangjie	IVTMD
U+8C0B.0	fHakka	 meu2; miau3
U+8C0B.0	fCantonese	mau4
U+8C0B.0	fMandarin	mou2
U+8C0B.0	fEnglish	simplified form of &#x8B00;
U+8C0B.1	fUTF8	谋
U+8C0B.1	fGB	0-4431
U+8C0B.1	fR/S	 149.9
U+8C0B.1	fTotalStrokes	 11
U+8C0B.1	fCangjie	IVTMD
U+8C0B.1	fMandarin	mou5
U+8C0C.0	fUTF8	谌
U+8C0C.0	fGB	0-5A48
U+8C0C.0	fR/S	 149.9
U+8C0C.0	fTotalStrokes	 11
U+8C0C.0	fCangjie	IVTMV
U+8C0C.0	fHakka	 shim1; zhim1; shim3
U+8C0C.0	fCantonese	sam4
U+8C0C.0	fMandarin	chen2
U+8C0C.0	fEnglish	simplified form of &#x8AF6;
U+8C0D.0	fUTF8	谍
U+8C0D.0	fGB	0-357D
U+8C0D.0	fR/S	 149.9
U+8C0D.0	fTotalStrokes	 11
U+8C0D.0	fCangjie	IVPTD
U+8C0D.0	fHakka	 tiap8
U+8C0D.0	fCantonese	dip6
U+8C0D.0	fMandarin	die2; tie1; xie4
U+8C0D.0	fEnglish	simplified form of &#x8ADC;
U+8C0E.0	fUTF8	谎
U+8C0E.0	fGB	0-3B51
U+8C0E.0	fR/S	 149.9
U+8C0E.0	fTotalStrokes	 11
U+8C0E.0	fCangjie	IVTYU
U+8C0E.0	fHakka	 fong1; fong3
U+8C0E.0	fCantonese	fong1; fong2 (also)
U+8C0E.0	fMandarin	huang3; huang1
U+8C0E.0	fEnglish	simplified form of &#x8B0A;
U+8C0F.0	fUTF8	谏
U+8C0F.0	fGB	0-5A49
U+8C0F.0	fR/S	 149.9
U+8C0F.0	fTotalStrokes	 11
U+8C0F.0	fCangjie	IVDWF
U+8C0F.0	fHakka	 gan5
U+8C0F.0	fCantonese	gaan3
U+8C0F.0	fMandarin	jian4; jian1; lan4
U+8C0F.0	fEnglish	simplified form of &#x8AEB;
U+8C10.0	fUTF8	谐
U+8C10.0	fGB	0-5033
U+8C10.0	fR/S	 149.9
U+8C10.0	fTotalStrokes	 11
U+8C10.0	fCangjie	IVPPA
U+8C10.0	fHakka	 hai2
U+8C10.0	fCantonese	haai4
U+8C10.0	fMandarin	xie2
U+8C10.0	fEnglish	simplified form of &#x8AE7;
U+8C11.0	fUTF8	谑
U+8C11.0	fGB	0-5A4A
U+8C11.0	fR/S	 149.9
U+8C11.0	fTotalStrokes	 11
U+8C11.0	fCangjie	IVYPM
U+8C11.0	fHakka	 ngiok7; hiok7
U+8C11.0	fCantonese	joek6
U+8C11.0	fMandarin	xue4; nuue4; ne4
U+8C11.0	fEnglish	simplified form of &#x8B14;
U+8C12.0	fUTF8	谒
U+8C12.0	fGB	0-5A4B
U+8C12.0	fR/S	 149.9
U+8C12.0	fTotalStrokes	 11
U+8C12.0	fCangjie	IVAPV
U+8C12.0	fHakka	 get7; giet7; jet7
U+8C12.0	fCantonese	jit3; kit3 (also); zit3
U+8C12.0	fMandarin	ye4; ai4
U+8C12.0	fEnglish	simplified form of &#x8B01;
U+8C13.0	fUTF8	谓
U+8C13.0	fGB	0-4E3D
U+8C13.0	fR/S	 149.9
U+8C13.0	fTotalStrokes	 11
U+8C13.0	fCangjie	IVWB
U+8C13.0	fHakka	 vui5
U+8C13.0	fCantonese	wai6
U+8C13.0	fMandarin	wei4
U+8C13.0	fEnglish	simplified form of &#x8B02;
U+8C14.0	fUTF8	谔
U+8C14.0	fGB	0-5A4C
U+8C14.0	fR/S	 149.9
U+8C14.0	fTotalStrokes	 11
U+8C14.0	fCangjie	IVRRS
U+8C14.0	fHakka	 ngok8
U+8C14.0	fCantonese	ngok6; ok6 (also)
U+8C14.0	fMandarin	e4
U+8C14.0	fEnglish	simplified form of &#x8AE4;
U+8C15.0	fUTF8	谕
U+8C15.0	fGB	0-5A4D
U+8C15.0	fR/S	 149.9
U+8C15.0	fTotalStrokes	 11
U+8C15.0	fCangjie	IVOMN
U+8C15.0	fHakka	 ji5
U+8C15.0	fCantonese	jyu6
U+8C15.0	fMandarin	yu4; tou3
U+8C15.0	fEnglish	simplified form of &#x8AED;
U+8C16.0	fUTF8	谖
U+8C16.0	fGB	0-5A4E
U+8C16.0	fR/S	 149.9
U+8C16.0	fTotalStrokes	 11
U+8C16.0	fCangjie	IVBME
U+8C16.0	fHakka	 sien1
U+8C16.0	fCantonese	hyun1
U+8C16.0	fMandarin	xuan1; shuan4; xuan3
U+8C16.0	fEnglish	simplified form of &#x8AFC;
U+8C17.0	fUTF8	谗
U+8C17.0	fGB	0-3277
U+8C17.0	fR/S	 149.9
U+8C17.0	fTotalStrokes	 11
U+8C17.0	fCangjie	IVNUY
U+8C17.0	fHakka	 cam2
U+8C17.0	fCantonese	caam4
U+8C17.0	fMandarin	chan2; zhan4
U+8C17.0	fEnglish	simplified form of &#x8B92;
U+8C18.0	fUTF8	谘
U+8C18.0	fGB	0-5A51
U+8C18.0	fR/S	 149.9
U+8C18.0	fTotalStrokes	 11
U+8C18.0	fCangjie	IVIOR
U+8C18.0	fHakka	 zih1
U+8C18.0	fCantonese	zi1
U+8C18.0	fMandarin	zi1
U+8C18.0	fEnglish	simplified form of &#x8AEE;
U+8C19.0	fUTF8	谙
U+8C19.0	fGB	0-5A4F
U+8C19.0	fR/S	 149.9
U+8C19.0	fTotalStrokes	 11
U+8C19.0	fCangjie	IVYTA
U+8C19.0	fHakka	 am1
U+8C19.0	fCantonese	am1; ngam1; am2 (also); ngam2 (also)
U+8C19.0	fMandarin	an1; an4; tou3
U+8C19.0	fEnglish	simplified form of &#x8AF3;
U+8C1A.0	fUTF8	谚
U+8C1A.0	fGB	0-5168
U+8C1A.0	fR/S	 149.9
U+8C1A.0	fTotalStrokes	 11
U+8C1A.0	fCangjie	IVYHH
U+8C1A.0	fHakka	 ngien5; ngan5
U+8C1A.0	fCantonese	jin6
U+8C1A.0	fMandarin	yan4
U+8C1A.0	fEnglish	simplified form of &#x8AFA;
U+8C1B.0	fUTF8	谛
U+8C1B.0	fGB	0-5A50
U+8C1B.0	fR/S	 149.9
U+8C1B.0	fTotalStrokes	 11
U+8C1B.0	fCangjie	IVYBB
U+8C1B.0	fHakka	 di5
U+8C1B.0	fCantonese	dai3
U+8C1B.0	fMandarin	di4; ti2
U+8C1B.0	fEnglish	simplified form of &#x8AE6;
U+8C1C.0	fUTF8	谜
U+8C1C.0	fGB	0-4355
U+8C1C.0	fR/S	 149.9
U+8C1C.0	fTotalStrokes	 11
U+8C1C.0	fCangjie	IVYFD
U+8C1C.0	fHakka	 mi5; mi3
U+8C1C.0	fCantonese	mai4; mai2 (also); mai6
U+8C1C.0	fMandarin	mi2; mei4; mi4
U+8C1C.0	fEnglish	simplified form of &#x8B0E;
U+8C1D.0	fUTF8	谝
U+8C1D.0	fGB	0-5A52
U+8C1D.0	fR/S	 149.9
U+8C1D.0	fTotalStrokes	 11
U+8C1D.0	fCangjie	IVISB
U+8C1D.0	fHakka	 pien1
U+8C1D.0	fCantonese	pin4; pin5
U+8C1D.0	fMandarin	pian2; bian3; pian3; bian4
U+8C1D.0	fEnglish	simplified form of &#x8ADE;
U+8C1E.0	fUTF8	谞
U+8C1E.0	fGB	8-7E77
U+8C1E.0	fR/S	 149.9
U+8C1E.0	fTotalStrokes	 11
U+8C1E.0	fCangjie	IVNOB
U+8C1E.0	fCantonese	seoi1; seoi2 (also)
U+8C1E.0	fMandarin	xu3
U+8C1E.0	fEnglish	knowledge; discrimination; treachery
U+8C1F.0	fUTF8	谟
U+8C1F.0	fGB	0-5A53
U+8C1F.0	fR/S	 149.10
U+8C1F.0	fTotalStrokes	 12
U+8C1F.0	fCangjie	IVTAK
U+8C1F.0	fHakka	 mu2
U+8C1F.0	fCantonese	mou4
U+8C1F.0	fMandarin	mo2; mo4; mu2; mu4
U+8C1F.0	fEnglish	simplified form of &#x8B28;
U+8C20.0	fUTF8	谠
U+8C20.0	fGB	0-5A54
U+8C20.0	fR/S	 149.10
U+8C20.0	fTotalStrokes	 12
U+8C20.0	fCangjie	IVFBU
U+8C20.0	fHakka	 dong3
U+8C20.0	fCantonese	dong2
U+8C20.0	fMandarin	dang3; dang4; tang4
U+8C20.0	fEnglish	simplified form of &#x8B9C;
U+8C21.0	fUTF8	谡
U+8C21.0	fGB	0-5A55
U+8C21.0	fR/S	 149.10
U+8C21.0	fTotalStrokes	 12
U+8C21.0	fCangjie	IVWCE
U+8C21.0	fHakka	 suk7
U+8C21.0	fCantonese	suk1
U+8C21.0	fMandarin	su4
U+8C21.0	fEnglish	simplified form of &#x8B16;
U+8C22.0	fUTF8	谢
U+8C22.0	fGB	0-503B
U+8C22.0	fR/S	 149.10
U+8C22.0	fTotalStrokes	 12
U+8C22.0	fCangjie	IVHHI
U+8C22.0	fHakka	 cia5
U+8C22.0	fCantonese	ze6
U+8C22.0	fMandarin	xie4
U+8C22.0	fEnglish	simplified form of &#x8B1D;
U+8C22.1	fUTF8	谢
U+8C22.1	fGB	0-503B
U+8C22.1	fR/S	 149.10
U+8C22.1	fTotalStrokes	 12
U+8C22.1	fCangjie	IVHHI
U+8C22.1	fMandarin	xie5
U+8C23.0	fUTF8	谣
U+8C23.0	fGB	0-5225
U+8C23.0	fR/S	 149.10
U+8C23.0	fTotalStrokes	 12
U+8C23.0	fCangjie	IVBOU
U+8C23.0	fCantonese	jiu4
U+8C23.0	fMandarin	yao2
U+8C23.0	fEnglish	simplified form of &#x8B21;
U+8C24.0	fUTF8	谤
U+8C24.0	fGB	0-3079
U+8C24.0	fR/S	 149.10
U+8C24.0	fTotalStrokes	 12
U+8C24.0	fCangjie	IVYBS
U+8C24.0	fHakka	 bong3
U+8C24.0	fCantonese	pong3
U+8C24.0	fMandarin	bang4; bang1
U+8C24.0	fEnglish	simplified form of &#x8B17;
U+8C25.0	fUTF8	谥
U+8C25.0	fGB	0-5A56
U+8C25.0	fR/S	 149.10
U+8C25.0	fTotalStrokes	 12
U+8C25.0	fCangjie	IVTCT
U+8C25.0	fHakka	 shi5; jit7
U+8C25.0	fCantonese	si3
U+8C25.0	fMandarin	shi4; yi4; xi4
U+8C25.0	fEnglish	simplified form of &#x8B1A;
U+8C26.0	fUTF8	谦
U+8C26.0	fGB	0-472B
U+8C26.0	fR/S	 149.10
U+8C26.0	fTotalStrokes	 12
U+8C26.0	fCangjie	IVTXC
U+8C26.0	fHakka	 kiam1
U+8C26.0	fCantonese	him1; him3
U+8C26.0	fMandarin	qian1; qian4; qian3; qie4; zhan4
U+8C26.0	fEnglish	simplified form of &#x8B19;
U+8C27.0	fUTF8	谧
U+8C27.0	fGB	0-5A57
U+8C27.0	fR/S	 149.10
U+8C27.0	fTotalStrokes	 12
U+8C27.0	fCangjie	IVPHT
U+8C27.0	fHakka	 met8
U+8C27.0	fCantonese	mat6
U+8C27.0	fMandarin	mi4
U+8C27.0	fEnglish	simplified form of &#x8B10;
U+8C28.0	fUTF8	谨
U+8C28.0	fGB	0-3D77
U+8C28.0	fR/S	 149.11
U+8C28.0	fTotalStrokes	 13
U+8C28.0	fCangjie	IVTLM
U+8C28.0	fHakka	 giun3
U+8C28.0	fCantonese	gan2
U+8C28.0	fMandarin	jin3; qin2
U+8C28.0	fEnglish	simplified form of &#x8B39;
U+8C29.0	fUTF8	谩
U+8C29.0	fGB	0-4321
U+8C29.0	fR/S	 149.11
U+8C29.0	fTotalStrokes	 13
U+8C29.0	fCangjie	IVAWE
U+8C29.0	fHakka	 man5
U+8C29.0	fCantonese	maan4; maan6
U+8C29.0	fMandarin	man4; man2; mian2
U+8C29.0	fEnglish	simplified form of &#x8B3E;
U+8C2A.0	fUTF8	谪
U+8C2A.0	fGB	0-5A58
U+8C2A.0	fR/S	 149.11
U+8C2A.0	fTotalStrokes	 13
U+8C2A.0	fCangjie	IVYCB
U+8C2A.0	fHakka	 zak7
U+8C2A.0	fCantonese	zaak6
U+8C2A.0	fMandarin	zhe2; di4; ze2
U+8C2A.0	fEnglish	simplified form of &#x8B2B;
U+8C2B.0	fUTF8	谫
U+8C2B.0	fGB	0-5A59
U+8C2B.0	fR/S	 149.11
U+8C2B.0	fTotalStrokes	 13
U+8C2B.0	fCangjie	IVTBH
U+8C2B.0	fCantonese	zin2
U+8C2B.0	fMandarin	jian3
U+8C2B.0	fEnglish	simplified form of &#x8B2D;
U+8C2C.0	fUTF8	谬
U+8C2C.0	fGB	0-437D
U+8C2C.0	fR/S	 149.11
U+8C2C.0	fTotalStrokes	 13
U+8C2C.0	fCangjie	IVSMH
U+8C2C.0	fHakka	 meu3
U+8C2C.0	fCantonese	mau6
U+8C2C.0	fMandarin	miu4; niu4
U+8C2C.0	fEnglish	simplified form of &#x8B2C;
U+8C2D.0	fUTF8	谭
U+8C2D.0	fGB	0-4C37
U+8C2D.0	fR/S	 149.12
U+8C2D.0	fTotalStrokes	 14
U+8C2D.0	fCangjie	IVMWJ
U+8C2D.0	fHakka	 tam2
U+8C2D.0	fCantonese	taam4
U+8C2D.0	fMandarin	tan2; dan4
U+8C2D.0	fEnglish	simplified form of &#x8B5A;
U+8C2E.0	fUTF8	谮
U+8C2E.0	fGB	0-5A5A
U+8C2E.0	fR/S	 149.12
U+8C2E.0	fTotalStrokes	 14
U+8C2E.0	fCangjie	IVMUA
U+8C2E.0	fHakka	 ziam5
U+8C2E.0	fCantonese	zam3; cam3 (also)
U+8C2E.0	fMandarin	zen4; jian4; zen3
U+8C2E.0	fEnglish	simplified form of &#x8B56;
U+8C2F.0	fUTF8	谯
U+8C2F.0	fGB	0-5A5B
U+8C2F.0	fR/S	 149.12
U+8C2F.0	fTotalStrokes	 14
U+8C2F.0	fCangjie	IVOGF
U+8C2F.0	fHakka	 ziau1
U+8C2F.0	fCantonese	ciu4; ciu3
U+8C2F.0	fMandarin	qiao2; shui2; qiao4
U+8C2F.0	fEnglish	simplified form of &#x8B59;
U+8C30.0	fUTF8	谰
U+8C30.0	fGB	0-403E
U+8C30.0	fR/S	 149.12
U+8C30.0	fTotalStrokes	 14
U+8C30.0	fCangjie	IVLSW
U+8C30.0	fHakka	 lan2
U+8C30.0	fCantonese	laan4
U+8C30.0	fMandarin	lan2; lan3; lan4
U+8C30.0	fEnglish	simplified form of &#x8B95;
U+8C31.0	fUTF8	谱
U+8C31.0	fGB	0-4657
U+8C31.0	fR/S	 149.12
U+8C31.0	fTotalStrokes	 14
U+8C31.0	fCangjie	IVTCA
U+8C31.0	fHakka	 pu3
U+8C31.0	fCantonese	pou2
U+8C31.0	fMandarin	pu3; bu3
U+8C31.0	fEnglish	simplified form of &#x8B5C;
U+8C32.0	fUTF8	谲
U+8C32.0	fGB	0-5A5C
U+8C32.0	fR/S	 149.12
U+8C32.0	fTotalStrokes	 14
U+8C32.0	fCangjie	IVNHB
U+8C32.0	fHakka	 git7
U+8C32.0	fCantonese	kyut3; gwat1 (also)
U+8C32.0	fMandarin	jue2
U+8C32.0	fEnglish	simplified form of &#x8B4E;
U+8C33.0	fUTF8	谳
U+8C33.0	fGB	0-5A5D
U+8C33.0	fR/S	 149.13
U+8C33.0	fTotalStrokes	 15
U+8C33.0	fCangjie	IVJBK
U+8C33.0	fHakka	 hien5
U+8C33.0	fCantonese	jin6
U+8C33.0	fMandarin	yan4; nie4; yan3
U+8C33.0	fEnglish	simplified form of &#x8B9E;
U+8C34.0	fUTF8	谴
U+8C34.0	fGB	0-4734
U+8C34.0	fR/S	 149.13
U+8C34.0	fTotalStrokes	 15
U+8C34.0	fCangjie	IVYLR
U+8C34.0	fHakka	 ken3; kien5
U+8C34.0	fCantonese	hin2
U+8C34.0	fMandarin	qian3; qian4
U+8C34.0	fEnglish	simplified form of &#x8B74;
U+8C35.0	fUTF8	谵
U+8C35.0	fGB	0-5A5E
U+8C35.0	fR/S	 149.13
U+8C35.0	fTotalStrokes	 15
U+8C35.0	fCangjie	IVNCR
U+8C35.0	fHakka	 zham1
U+8C35.0	fCantonese	zim1
U+8C35.0	fMandarin	zhan1; ta4; zhe2
U+8C35.0	fEnglish	simplified form of &#x8B6B;
U+8C36.0	fUTF8	谶
U+8C36.0	fGB	0-5A5F
U+8C36.0	fR/S	 149.17
U+8C36.0	fTotalStrokes	 19
U+8C36.0	fCangjie	IVOIM
U+8C36.0	fHakka	 ciam5; siam2
U+8C36.0	fCantonese	cam3
U+8C36.0	fMandarin	chen4; chan4
U+8C36.0	fEnglish	simplified form of &#x8B96;
U+8C37.0	fUTF8	谷
U+8C37.0	fCNS11643	1-4B47
U+8C37.0	fBig5	A8A6
U+8C37.0	fGB	0-3948
U+8C37.0	fR/S	 150.0
U+8C37.0	fTotalStrokes	 7
U+8C37.0	fCangjie	COR
U+8C37.0	fFourCorner	80608
U+8C37.0	fHakka	  guk7; guk8; kwuk7
U+8C37.0	fCantonese	guk1; juk6; luk6
U+8C37.0	fMandarin	gu3; lu4
U+8C37.0	fEnglish	[1] valley; ravine; waterway between two mountains [2] hollow; pit [3] dilemma; difficult situation; predicament [4] a Chinese family name
U+8C37.1	fUTF8	谷
U+8C37.1	fCNS11643	1-4B47
U+8C37.1	fBig5	A8A6
U+8C37.1	fGB	0-3948
U+8C37.1	fR/S	 150.0
U+8C37.1	fTotalStrokes	 7
U+8C37.1	fCangjie	COR
U+8C37.1	fFourCorner	80608
U+8C37.1	fMandarin	yu4
U+8C37.1	fEnglish	a Chinese family name
U+8C38.0	fUTF8	谸
U+8C38.0	fCNS11643	5-3046
U+8C38.0	fGB	5-7051
U+8C38.0	fR/S	  150.3
U+8C38.0	fTotalStrokes	 10
U+8C38.0	fCangjie	HJCOR
U+8C38.0	fFourCorner	28460
U+8C38.0	fHakka	  cien1
U+8C38.0	fCantonese	cin1
U+8C38.0	fMandarin	qian1
U+8C38.0	fEnglish	[1] road [2] green; verdent
U+8C39.0	fUTF8	谹
U+8C39.0	fCNS11643	2-3950
U+8C39.0	fBig5	D7CD
U+8C39.0	fGB	3-717D
U+8C39.0	fR/S	 150.4
U+8C39.0	fTotalStrokes	 11
U+8C39.0	fCangjie	CRKI
U+8C39.0	fFourCorner	84632
U+8C39.0	fCantonese	wang4
U+8C39.0	fMandarin	hong2
U+8C3A.0	fUTF8	谺
U+8C3A.0	fCNS11643	3-392A
U+8C3A.0	fGB	5-7054
U+8C3A.0	fR/S	  150.4
U+8C3A.0	fTotalStrokes	 11
U+8C3A.0	fCangjie	CRMVH
U+8C3A.0	fFourCorner	81640
U+8C3A.0	fHakka	  ha2
U+8C3A.0	fCantonese	haa1
U+8C3A.0	fMandarin	xia1
U+8C3A.0	fEnglish	the mouth of a valley
U+8C3B.0	fUTF8	谻
U+8C3B.0	fCNS11643	2-3951
U+8C3B.0	fBig5	D7CE
U+8C3B.0	fGB	5-7059
U+8C3B.0	fR/S	 150.4
U+8C3B.0	fTotalStrokes	 11
U+8C3B.0	fCangjie	CRNQ
U+8C3B.0	fFourCorner	87610
U+8C3B.0	fCantonese	gik1; goek3
U+8C3B.0	fMandarin	jue2; ji3; ju4
U+8C3C.0	fUTF8	谼
U+8C3C.0	fCNS11643	2-4758
U+8C3C.0	fBig5	E052
U+8C3C.0	fGB	3-7221
U+8C3C.0	fR/S	 150.6
U+8C3C.0	fTotalStrokes	 13
U+8C3C.0	fCangjie	CRTC
U+8C3C.0	fFourCorner	84681
U+8C3C.0	fCantonese	hung4
U+8C3C.0	fMandarin	hong2
U+8C3D.0	fUTF8	谽
U+8C3D.0	fCNS11643	2-4E38
U+8C3D.0	fBig5	E450
U+8C3D.0	fGB	5-705C
U+8C3D.0	fR/S	 150.7
U+8C3D.0	fTotalStrokes	 14
U+8C3D.0	fCangjie	CROIR
U+8C3D.0	fFourCorner	88662
U+8C3D.0	fHakka	 ham2
U+8C3D.0	fCantonese	ham4; ham1; haam4
U+8C3D.0	fMandarin	han1; xian1
U+8C3D.0	fEnglish	a mouth or opening
U+8C3E.0	fUTF8	谾
U+8C3E.0	fCNS11643	2-544E
U+8C3E.0	fBig5	E7E5
U+8C3E.0	fGB	5-705F
U+8C3E.0	fR/S	 150.8
U+8C3E.0	fTotalStrokes	 15
U+8C3E.0	fCangjie	CRJCM
U+8C3E.0	fFourCorner	83611
U+8C3E.0	fCantonese	hung1; lung4
U+8C3E.0	fMandarin	hong1; long2; qiang1; xiang1
U+8C3F.0	fUTF8	谿
U+8C3F.0	fCNS11643	1-7551
U+8C3F.0	fBig5	C1C6
U+8C3F.0	fGB	E-3E44
U+8C3F.0	fR/S	 150.10
U+8C3F.0	fTotalStrokes	 17
U+8C3F.0	fCangjie	BKCOR
U+8C3F.0	fFourCorner	28468
U+8C3F.0	fHakka	 hai1;  hi2; k'ie1; kai1; k'e1; ke1
U+8C3F.0	fCantonese	kai1; kai4
U+8C3F.0	fMandarin	xi1; qi1; ji1
U+8C3F.0	fEnglish	[1] valley; gorge [2] mountain stream; stream; creek; brook
U+8C40.0	fUTF8	豀
U+8C40.0	fCNS11643	3-564D
U+8C40.0	fGB	5-7061
U+8C40.0	fR/S	 150.10
U+8C40.0	fTotalStrokes	 17
U+8C40.0	fCangjie	CRBVK
U+8C40.0	fFourCorner	82630
U+8C40.0	fCantonese	hai4; kai1
U+8C40.0	fMandarin	xi1; xi2
U+8C40.0	fEnglish	 squabble; quarrel; brawl
U+8C41.0	fUTF8	豁
U+8C41.0	fCNS11643	1-7550
U+8C41.0	fBig5	C1C5
U+8C41.0	fGB	0-3B6D
U+8C41.0	fR/S	 150.10
U+8C41.0	fTotalStrokes	 17
U+8C41.0	fCangjie	JRCOR
U+8C41.0	fFourCorner	38668
U+8C41.0	fHakka	  hot7;  wok7;  wat7; gut7; hat7; vat7
U+8C41.0	fCantonese	kut3; waa1; kwut3 (also); waak6 (also)
U+8C41.0	fMandarin	huo1; huo4
U+8C41.0	fEnglish	[1] crack; break; breach [2] crack; open up; break open; split [3] give up; sacrifice; risk one's life for
U+8C41.1	fUTF8	豁
U+8C41.1	fCNS11643	1-7550
U+8C41.1	fBig5	C1C5
U+8C41.1	fGB	0-3B6D
U+8C41.1	fR/S	 150.10
U+8C41.1	fTotalStrokes	 17
U+8C41.1	fCangjie	JRCOR
U+8C41.1	fFourCorner	38668
U+8C41.1	fMandarin	huo4huo1}
U+8C41.1	fEnglish	[1] open up; clear [2] exempt from (duties, etc.)
U+8C41.2	fUTF8	豁
U+8C41.2	fCNS11643	1-7550
U+8C41.2	fBig5	C1C5
U+8C41.2	fGB	0-3B6D
U+8C41.2	fR/S	 150.10
U+8C41.2	fTotalStrokes	 17
U+8C41.2	fCangjie	JRCOR
U+8C41.2	fFourCorner	38668
U+8C41.2	fMandarin	he4
U+8C41.2	fEnglish	bright and spacious
U+8C41.3	fUTF8	豁
U+8C41.3	fCNS11643	1-7550
U+8C41.3	fBig5	C1C5
U+8C41.3	fGB	0-3B6D
U+8C41.3	fR/S	 150.10
U+8C41.3	fTotalStrokes	 17
U+8C41.3	fCangjie	JRCOR
U+8C41.3	fFourCorner	38668
U+8C41.3	fMandarin	hua2
U+8C41.3	fEnglish	 play a finger-guessing game
U+8C42.0	fUTF8	豂
U+8C42.0	fCNS11643	2-6359
U+8C42.0	fBig5	F0EE
U+8C42.0	fGB	5-7062
U+8C42.0	fR/S	 150.11
U+8C42.0	fTotalStrokes	 18
U+8C42.0	fCangjie	CRSMH
U+8C42.0	fFourCorner	87622
U+8C42.0	fCantonese	liu4
U+8C42.0	fMandarin	liao2
U+8C43.0	fUTF8	豃
U+8C43.0	fCNS11643	2-6730
U+8C43.0	fBig5	F344
U+8C43.0	fGB	3-7225
U+8C43.0	fR/S	 150.12
U+8C43.0	fTotalStrokes	 19
U+8C43.0	fCangjie	CRMJK
U+8C43.0	fFourCorner	88640
U+8C43.0	fCantonese	haam2; haam3
U+8C43.0	fMandarin	han3
U+8C44.0	fUTF8	豄
U+8C44.0	fCNS11643	3-5F64
U+8C44.0	fGB	5-705D
U+8C44.0	fR/S	 150.15
U+8C44.0	fTotalStrokes	 22
U+8C44.0	fCangjie	CRGWC
U+8C44.0	fFourCorner	84680
U+8C44.0	fCantonese	duk6
U+8C44.0	fMandarin	du2
U+8C45.0	fUTF8	豅
U+8C45.0	fCNS11643	2-6F52
U+8C45.0	fBig5	F844
U+8C45.0	fGB	3-717E
U+8C45.0	fR/S	 150.16
U+8C45.0	fTotalStrokes	 23
U+8C45.0	fCangjie	CRYBP
U+8C45.0	fFourCorner	81611
U+8C45.0	fCantonese	lung4; lung6
U+8C45.0	fMandarin	long2; long3; long4
U+8C46.0	fUTF8	豆
U+8C46.0	fCNS11643	1-4B48
U+8C46.0	fBig5	A8A7
U+8C46.0	fGB	0-3639
U+8C46.0	fR/S	 151.0
U+8C46.0	fTotalStrokes	 7
U+8C46.0	fCangjie	MRT
U+8C46.0	fFourCorner	10108
U+8C46.0	fHakka	  teu5; teu6; teu3; tiu3
U+8C46.0	fCantonese	dau6; dau2 (also); dau4 (also)
U+8C46.0	fMandarin	dou4; dou3; xiu1
U+8C46.0	fEnglish	[1] beans; peas; bean-shaped; beans and peas collectively [2] a vessel of wood for containing flesh, sauces, etc. at sacrifices and feasts [3] a Chinese family name
U+8C47.0	fUTF8	豇
U+8C47.0	fCNS11643	2-3322
U+8C47.0	fBig5	D3DE
U+8C47.0	fGB	0-7478
U+8C47.0	fR/S	 151.3
U+8C47.0	fTotalStrokes	 10
U+8C47.0	fCangjie	MTM
U+8C47.0	fFourCorner	11110
U+8C47.0	fHakka	 gong1
U+8C47.0	fCantonese	gong1
U+8C47.0	fMandarin	jiang1
U+8C47.0	fEnglish	a cowpea
U+8C48.0	fUTF8	豈
U+8C48.0	fCNS11643	1-583F
U+8C48.0	fBig5	B05A
U+8C48.0	fGB	1-4671
U+8C48.0	fR/S	 151.3
U+8C48.0	fTotalStrokes	 10
U+8C48.0	fCangjie	UMRT
U+8C48.0	fFourCorner	22108
U+8C48.0	fHakka	  ki3
U+8C48.0	fCantonese	hei2; hoi2
U+8C48.0	fMandarin	qi3
U+8C48.0	fEnglish	how?; what?; an interrogative particle implying a conflicting or dissenting view or answer
U+8C48.1	fUTF8	豈
U+8C48.1	fCNS11643	1-583F
U+8C48.1	fBig5	B05A
U+8C48.1	fGB	1-4671
U+8C48.1	fR/S	 151.3
U+8C48.1	fTotalStrokes	 10
U+8C48.1	fCangjie	UMRT
U+8C48.1	fFourCorner	22108
U+8C48.1	fMandarin	kai3
U+8C48.1	fEnglish	happy; delighted; jubilant
U+8C49.0	fUTF8	豉
U+8C49.0	fCNS11643	1-5D47
U+8C49.0	fBig5	B361
U+8C49.0	fGB	0-7479
U+8C49.0	fR/S	 151.4
U+8C49.0	fTotalStrokes	 11
U+8C49.0	fCangjie	MTJE
U+8C49.0	fFourCorner	14147
U+8C49.0	fHakka	 si3; shi6; sii5; si5; shi5; she5
U+8C49.0	fCantonese	si6; si2 (also)
U+8C49.0	fMandarin	chi3; shi4
U+8C49.0	fEnglish	fermented beans; pickled black beans
U+8C4A.0	fUTF8	豊
U+8C4A.0	fCNS11643	2-475A
U+8C4A.0	fBig5	E054
U+8C4A.0	fGB	3-6E46
U+8C4A.0	fR/S	 151.6
U+8C4A.0	fTotalStrokes	 13
U+8C4A.0	fCangjie	TWMRT
U+8C4A.0	fFourCorner	55108
U+8C4A.0	fHakka	 li3; li1
U+8C4A.0	fCantonese	lai5
U+8C4A.0	fMandarin	li3; feng1
U+8C4A.0	fEnglish	a kind of ritual vessel in ancient times
U+8C4B.0	fUTF8	豋
U+8C4B.0	fCNS11643	2-4759
U+8C4B.0	fBig5	E053
U+8C4B.0	fGB	5-6D76
U+8C4B.0	fR/S	 151.6
U+8C4B.0	fTotalStrokes	 13
U+8C4B.0	fCangjie	BOMRT
U+8C4B.0	fFourCorner	27108
U+8C4B.0	fHakka	 den1
U+8C4B.0	fCantonese	dang1
U+8C4B.0	fMandarin	deng1
U+8C4B.0	fEnglish	vessels for food used in ancient times
U+8C4C.0	fUTF8	豌
U+8C4C.0	fCNS11643	1-6F27
U+8C4C.0	fBig5	BDDC
U+8C4C.0	fGB	0-4D63
U+8C4C.0	fR/S	 151.8
U+8C4C.0	fTotalStrokes	 15
U+8C4C.0	fCangjie	MTJNU
U+8C4C.0	fFourCorner	13112
U+8C4C.0	fHakka	 wan1; won3; wan3; van1
U+8C4C.0	fCantonese	wun1; wun2 (also)
U+8C4C.0	fMandarin	wan1
U+8C4C.0	fEnglish	peas; garden peas
U+8C4D.0	fUTF8	豍
U+8C4D.0	fCNS11643	2-544F
U+8C4D.0	fBig5	E7E6
U+8C4D.0	fGB	5-6D78
U+8C4D.0	fR/S	 151.8
U+8C4D.0	fTotalStrokes	 15
U+8C4D.0	fCangjie	MTHWJ
U+8C4D.0	fFourCorner	16140
U+8C4D.0	fCantonese	bai1; bin2
U+8C4D.0	fMandarin	bi1; bian3
U+8C4E.0	fUTF8	豎
U+8C4E.0	fCNS11643	1-6F28
U+8C4E.0	fBig5	BDDD
U+8C4E.0	fGB	E-3E45
U+8C4E.0	fR/S	 151.8
U+8C4E.0	fTotalStrokes	 15
U+8C4E.0	fCangjie	SEMRT
U+8C4E.0	fFourCorner	77108
U+8C4E.0	fHakka	 su5; shu6; shu5
U+8C4E.0	fCantonese	syu6
U+8C4E.0	fMandarin	shu4
U+8C4E.0	fEnglish	[1] erect; set up; stand [2] upright; vertical; perpendicular [3] boy who have not come of age; young servant [4] petty officers in the palace [5] a downward, perpendicular stroke in writing
U+8C4F.0	fUTF8	豏
U+8C4F.0	fCNS11643	2-5F5D
U+8C4F.0	fBig5	EEB1
U+8C4F.0	fGB	3-6E4E
U+8C4F.0	fR/S	 151.10
U+8C4F.0	fTotalStrokes	 17
U+8C4F.0	fCangjie	MTTXC
U+8C4F.0	fFourCorner	18137
U+8C4F.0	fCantonese	haam5; hin3
U+8C4F.0	fMandarin	xian4
U+8C4F.0	fEnglish	half-grown beans
U+8C50.0	fUTF8	豐
U+8C50.0	fCNS11643	1-7742
U+8C50.0	fBig5	C2D7
U+8C50.0	fGB	1-3761
U+8C50.0	fR/S	 151.11
U+8C50.0	fTotalStrokes	 18
U+8C50.0	fCangjie	UJMRT
U+8C50.0	fFourCorner	55108
U+8C50.0	fHakka	  fung1
U+8C50.0	fCantonese	fung1
U+8C50.0	fMandarin	feng1
U+8C50.0	fEnglish	[1] abundant; lush; bountiful; plenty; luxuriant; copious; fruitful; plentiful; thick; big [2] crop; harvest [3] a Chinese family name
U+8C51.0	fUTF8	豑
U+8C51.0	fCNS11643	3-5D34
U+8C51.0	fGB	5-6D79
U+8C51.0	fR/S	  151.13
U+8C51.0	fTotalStrokes	 20
U+8C51.0	fCangjie	TTCNH
U+8C51.0	fFourCorner	58120
U+8C51.0	fCantonese	dit6
U+8C51.0	fMandarin	zhi4
U+8C52.0	fUTF8	豒
U+8C52.0	fCNS11643	3-664D
U+8C52.0	fGB	E-3E46
U+8C52.0	fR/S	 151.18
U+8C52.0	fTotalStrokes	 25
U+8C52.0	fCangjie	UTCNH
U+8C52.0	fMandarin	zhi4
U+8C53.0	fUTF8	豓
U+8C53.0	fCNS11643	3-6221
U+8C53.0	fGB	E-3E47
U+8C53.0	fR/S	 151.20
U+8C53.0	fTotalStrokes	 27
U+8C53.0	fCangjie	UTKIT
U+8C53.0	fFourCorner	24117
U+8C53.0	fHakka	 jam5; ram6
U+8C53.0	fCantonese	jim6
U+8C53.0	fMandarin	yan4
U+8C53.0	fEnglish	beautiful; captivating; plump; voluptuous
U+8C54.0	fUTF8	豔
U+8C54.0	fCNS11643	1-7D43
U+8C54.0	fBig5	C676
U+8C54.0	fGB	E-3E48
U+8C54.0	fR/S	 151.21
U+8C54.0	fTotalStrokes	 28
U+8C54.0	fCangjie	UTGIT
U+8C54.0	fFourCorner	54117
U+8C54.0	fHakka	  jam5; jam3; ram6
U+8C54.0	fCantonese	jim6
U+8C54.0	fMandarin	yan4
U+8C54.0	fEnglish	[1] plump; voluptuous [2] beautiful and captivating (literary writings, etc.) [3] gorgeous; colorful; gaudy [4] anything pertaining to love (love story, love song); amorous [5] beauty [6] radiant [7] admire; envy
U+8C55.0	fUTF8	豕
U+8C55.0	fCNS11643	1-4B49
U+8C55.0	fBig5	A8A8
U+8C55.0	fGB	0-7539
U+8C55.0	fR/S	 152.0
U+8C55.0	fTotalStrokes	 7
U+8C55.0	fCangjie	MSHO
U+8C55.0	fFourCorner	10232
U+8C55.0	fHakka	 chi3; chi1; ci3; cih3; cih1; ci1; chii3
U+8C55.0	fCantonese	ci2
U+8C55.0	fMandarin	shi3
U+8C55.0	fEnglish	[1] pigs; big; hog [2] Kangxi radical 152
U+8C56.0	fUTF8	豖
U+8C56.0	fCNS11643	2-286B
U+8C56.0	fBig5	CDCB
U+8C56.0	fGB	3-6F2B
U+8C56.0	fR/S	 152.1
U+8C56.0	fTotalStrokes	 8
U+8C56.0	fCangjie	MSKO
U+8C56.0	fFourCorner	10232
U+8C56.0	fHakka	 hiuk7
U+8C56.0	fCantonese	cuk1; cuk3; doek3
U+8C56.0	fMandarin	chu4
U+8C56.0	fEnglish	a shackled pig
U+8C57.0	fUTF8	豗
U+8C57.0	fCNS11643	2-3323
U+8C57.0	fBig5	D3DF
U+8C57.0	fGB	3-3543
U+8C57.0	fR/S	 152.3
U+8C57.0	fTotalStrokes	 10
U+8C57.0	fCangjie	MUMSO
U+8C57.0	fFourCorner	11213
U+8C57.0	fHakka	 foi1; foe1
U+8C57.0	fCantonese	fui1
U+8C57.0	fMandarin	hui1
U+8C57.0	fEnglish	clamour
U+8C58.0	fUTF8	豘
U+8C58.0	fCNS11643	3-392D
U+8C58.0	fGB	5-6E3D
U+8C58.0	fR/S	 152.4
U+8C58.0	fTotalStrokes	 11
U+8C58.0	fCangjie	MOPU
U+8C58.0	fFourCorner	15210
U+8C58.0	fHakka	 tun3
U+8C58.0	fCantonese	tyun4
U+8C58.0	fMandarin	tun2
U+8C58.0	fEnglish	a small pig
U+8C59.0	fUTF8	豙
U+8C59.0	fCNS11643	4-354D
U+8C59.0	fGB	5-6E3E
U+8C59.0	fR/S	 152.4
U+8C59.0	fTotalStrokes	 11
U+8C59.0	fCangjie	YTPO
U+8C59.0	fFourCorner	00230
U+8C59.0	fCantonese	ngai6; ai6 (also)
U+8C59.0	fMandarin	yi4
U+8C5A.0	fUTF8	豚
U+8C5A.0	fCNS11643	1-5D48
U+8C5A.0	fBig5	B362
U+8C5A.0	fGB	0-6B60
U+8C5A.0	fR/S	 152.4
U+8C5A.0	fTotalStrokes	 11
U+8C5A.0	fCangjie	BMSO
U+8C5A.0	fFourCorner	71232
U+8C5A.0	fHakka	 tun2; dun3
U+8C5A.0	fCantonese	tyun4
U+8C5A.0	fMandarin	tun2; dun4; dun1
U+8C5A.0	fEnglish	[1] small pig; suckling pig [2] a Chinese family name
U+8C5B.0	fUTF8	豛
U+8C5B.0	fCNS11643	3-392C
U+8C5B.0	fGB	3-6F30
U+8C5B.0	fR/S	 152.4
U+8C5B.0	fTotalStrokes	 11
U+8C5B.0	fCangjie	MOHNE
U+8C5B.0	fFourCorner	17240
U+8C5B.0	fCantonese	jik6
U+8C5B.0	fMandarin	yi4
U+8C5C.0	fUTF8	豜
U+8C5C.0	fCNS11643	2-3952
U+8C5C.0	fBig5	D7CF
U+8C5C.0	fGB	3-6F2D
U+8C5C.0	fR/S	 152.4
U+8C5C.0	fTotalStrokes	 11
U+8C5C.0	fCangjie	MOMT
U+8C5C.0	fFourCorner	11240
U+8C5C.0	fCantonese	gin1
U+8C5C.0	fMandarin	jian1
U+8C5C.0	fEnglish	pig of 3 years
U+8C5D.0	fUTF8	豝
U+8C5D.0	fCNS11643	2-3953
U+8C5D.0	fBig5	D7D0
U+8C5D.0	fGB	3-6F31
U+8C5D.0	fR/S	 152.4
U+8C5D.0	fTotalStrokes	 11
U+8C5D.0	fCangjie	MOAU
U+8C5D.0	fFourCorner	17217
U+8C5D.0	fHakka	 ba1
U+8C5D.0	fCantonese	baa1
U+8C5D.0	fMandarin	ba1
U+8C5D.0	fEnglish	 sow; female pig
U+8C5E.0	fUTF8	豞
U+8C5E.0	fCNS11643	3-3F2A
U+8C5E.0	fGB	3-6F34
U+8C5E.0	fR/S	 152.5
U+8C5E.0	fTotalStrokes	 12
U+8C5E.0	fCangjie	MOPR
U+8C5E.0	fFourCorner	17220
U+8C5E.0	fCantonese	kau3
U+8C5E.0	fMandarin	hou4
U+8C5F.0	fUTF8	豟
U+8C5F.0	fCNS11643	2-404B
U+8C5F.0	fBig5	DBE5
U+8C5F.0	fGB	3-6F35
U+8C5F.0	fR/S	 152.5
U+8C5F.0	fTotalStrokes	 12
U+8C5F.0	fCangjie	MOHSN
U+8C5F.0	fFourCorner	11217
U+8C5F.0	fCantonese	ak1; ngak1 (also)
U+8C5F.0	fMandarin	e4
U+8C60.0	fUTF8	豠
U+8C60.0	fCNS11643	3-3F29
U+8C60.0	fGB	3-6F32
U+8C60.0	fR/S	 152.5
U+8C60.0	fTotalStrokes	 12
U+8C60.0	fCangjie	MOBM
U+8C60.0	fFourCorner	17210
U+8C60.0	fCantonese	co4
U+8C60.0	fMandarin	cu2
U+8C61.0	fUTF8	象
U+8C61.0	fCNS11643	1-622F
U+8C61.0	fBig5	B648
U+8C61.0	fGB	0-4F73
U+8C61.0	fR/S	 152.5
U+8C61.0	fTotalStrokes	 12
U+8C61.0	fCangjie	NAPO
U+8C61.0	fFourCorner	27232
U+8C61.0	fHakka	 siong5; siong3; siong6
U+8C61.0	fCantonese	zoeng6
U+8C61.0	fMandarin	xiang4
U+8C61.0	fEnglish	[1] the elephant [2] ivory [3] portrait; figure; image; snapshot [4] phenomenon; the outward appearance or expression of anything (especially weather, heavenly bodies, etc.); shape; image
U+8C62.0	fUTF8	豢
U+8C62.0	fCNS11643	1-666D
U+8C62.0	fBig5	B8E6
U+8C62.0	fGB	0-3B3F
U+8C62.0	fR/S	 152.6
U+8C62.0	fTotalStrokes	 13
U+8C62.0	fCangjie	FQMSO
U+8C62.0	fFourCorner	90232
U+8C62.0	fHakka	  fam5; fam6; fon6; fan6; fon3; fan5
U+8C62.0	fCantonese	waan6
U+8C62.0	fMandarin	huan4; huan3
U+8C62.0	fEnglish	[1] domestic animals [2] feed animals with grains [3] tempt or entice people with profit or gains
U+8C63.0	fUTF8	豣
U+8C63.0	fCNS11643	3-4464
U+8C63.0	fGB	E-3E49
U+8C63.0	fR/S	 152.6
U+8C63.0	fTotalStrokes	 13
U+8C63.0	fCangjie	MOMJJ
U+8C63.0	fFourCorner	11240
U+8C63.0	fHakka	 gien1; gian1; gen1
U+8C63.0	fCantonese	gin1
U+8C63.0	fMandarin	jian1; yan4
U+8C63.0	fEnglish	pig of 3 years
U+8C64.0	fUTF8	豤
U+8C64.0	fCNS11643	2-475C
U+8C64.0	fBig5	E056
U+8C64.0	fGB	5-6E45
U+8C64.0	fR/S	 152.6
U+8C64.0	fTotalStrokes	 13
U+8C64.0	fCangjie	MOAV
U+8C64.0	fFourCorner	17232
U+8C64.0	fCantonese	han2
U+8C64.0	fMandarin	ken3; kun1; kun3; qian3
U+8C65.0	fUTF8	豥
U+8C65.0	fCNS11643	2-475B
U+8C65.0	fBig5	E055
U+8C65.0	fGB	5-6E44
U+8C65.0	fR/S	 152.6
U+8C65.0	fTotalStrokes	 13
U+8C65.0	fCangjie	MOYVO
U+8C65.0	fFourCorner	10282
U+8C65.0	fCantonese	goi1; haai4
U+8C65.0	fMandarin	gai1; ai2; hai2; hai4
U+8C66.0	fUTF8	豦
U+8C66.0	fCNS11643	2-475D
U+8C66.0	fBig5	E057
U+8C66.0	fGB	3-6143
U+8C66.0	fR/S	 152.6
U+8C66.0	fTotalStrokes	 13
U+8C66.0	fCangjie	YPMSO
U+8C66.0	fFourCorner	21232
U+8C66.0	fHakka	 gi2;  gi3
U+8C66.0	fCantonese	geoi3; keoi4; geoi6
U+8C66.0	fMandarin	qu2; ju4
U+8C66.0	fEnglish	 a wild boar fight
U+8C67.0	fUTF8	豧
U+8C67.0	fCNS11643	3-496D
U+8C67.0	fGB	3-6F37
U+8C67.0	fR/S	 152.7
U+8C67.0	fTotalStrokes	 14
U+8C67.0	fCangjie	MOIJB
U+8C67.0	fFourCorner	13220
U+8C67.0	fCantonese	fu1
U+8C67.0	fMandarin	fu1
U+8C68.0	fUTF8	豨
U+8C68.0	fCNS11643	2-4E39
U+8C68.0	fBig5	E451
U+8C68.0	fGB	3-6F38
U+8C68.0	fR/S	 152.7
U+8C68.0	fTotalStrokes	 14
U+8C68.0	fCangjie	MOKKB
U+8C68.0	fFourCorner	14227
U+8C68.0	fHakka	 hi1; hi3
U+8C68.0	fCantonese	hei1; hei2
U+8C68.0	fMandarin	xi1; xi3
U+8C68.0	fEnglish	 pig; hog
U+8C69.0	fUTF8	豩
U+8C69.0	fCNS11643	2-4E3A
U+8C69.0	fBig5	E452
U+8C69.0	fGB	5-6E47
U+8C69.0	fR/S	 152.7
U+8C69.0	fTotalStrokes	 14
U+8C69.0	fCangjie	MOMSO
U+8C69.0	fFourCorner	11232
U+8C69.0	fCantonese	ban1; waan1
U+8C69.0	fMandarin	bin1; huan1
U+8C6A.0	fUTF8	豪
U+8C6A.0	fCNS11643	1-6B30
U+8C6A.0	fBig5	BBA8
U+8C6A.0	fGB	0-3A40
U+8C6A.0	fR/S	 152.7
U+8C6A.0	fTotalStrokes	 14
U+8C6A.0	fCangjie	YRBO
U+8C6A.0	fFourCorner	00232
U+8C6A.0	fHakka	  hau2;  ho2; gau1; hau3
U+8C6A.0	fCantonese	hou4
U+8C6A.0	fMandarin	hao2
U+8C6A.0	fEnglish	[1] heroic person; person outstanding in intelligence or talent [2] leader; ringleader [3] a proclivity to the use of force; bullying ways, etc. [4] tiny; fine hair
U+8C6B.0	fUTF8	豫
U+8C6B.0	fCNS11643	1-7247
U+8C6B.0	fBig5	BFDD
U+8C6B.0	fGB	0-5425
U+8C6B.0	fR/S	 152.9
U+8C6B.0	fTotalStrokes	 16
U+8C6B.0	fCangjie	NNNAO
U+8C6B.0	fFourCorner	17232
U+8C6B.0	fHakka	 ji5; ji3; ri6; i5
U+8C6B.0	fCantonese	jyu6
U+8C6B.0	fMandarin	yu4; shu1; xu4; xie4
U+8C6B.0	fEnglish	[1] relaxed; comfortable; comfort; be at ease [2] same as $ [3] travel; make an excursion or trip [4] cheat; lie [5] one of the nine political divisions in ancient China [6] hesitate [7] short for Honan [8] happy; delighted; pleased [9] a Chinese family name
U+8C6C.0	fUTF8	豬
U+8C6C.0	fCNS11643	1-6F29
U+8C6C.0	fBig5	BDDE
U+8C6C.0	fGB	E-3E4A
U+8C6C.0	fR/S	 152.9
U+8C6C.0	fTotalStrokes	 16
U+8C6C.0	fCangjie	MOJKA
U+8C6C.0	fFourCorner	14260
U+8C6C.0	fHakka	 zu1;  zhu1
U+8C6C.0	fCantonese	zyu1
U+8C6C.0	fMandarin	zhu1; zhe1
U+8C6C.0	fEnglish	[1] pig; hog; swine [2] pigheaded person
U+8C6D.0	fUTF8	豭
U+8C6D.0	fCNS11643	1-7248
U+8C6D.0	fBig5	BFDE
U+8C6D.0	fGB	3-6F3E
U+8C6D.0	fR/S	 152.9
U+8C6D.0	fTotalStrokes	 16
U+8C6D.0	fCangjie	MORSE
U+8C6D.0	fFourCorner	17247
U+8C6D.0	fHakka	 ga1
U+8C6D.0	fCantonese	gaa1
U+8C6D.0	fMandarin	jia1
U+8C6D.0	fEnglish	boar; male pig
U+8C6E.0	fUTF8	豮
U+8C6E.0	fGB	8-7E5D
U+8C6E.0	fR/S	 152.9
U+8C6E.0	fTotalStrokes	 16
U+8C6E.0	fCangjie	MOJTO
U+8C6E.0	fCantonese	fan4
U+8C6E.0	fMandarin	fen2
U+8C6E.0	fEnglish	castrate pig
U+8C6F.0	fUTF8	豯
U+8C6F.0	fCNS11643	2-5F61
U+8C6F.0	fBig5	EEB5
U+8C6F.0	fGB	3-6F41
U+8C6F.0	fR/S	 152.10
U+8C6F.0	fTotalStrokes	 17
U+8C6F.0	fCangjie	MOBVK
U+8C6F.0	fFourCorner	12234
U+8C6F.0	fCantonese	hai4
U+8C6F.0	fMandarin	xi1
U+8C70.0	fUTF8	豰
U+8C70.0	fCNS11643	2-5F5E
U+8C70.0	fBig5	EEB2
U+8C70.0	fGB	3-4F3C
U+8C70.0	fR/S	 152.10
U+8C70.0	fTotalStrokes	 17
U+8C70.0	fCangjie	GOHNE
U+8C70.0	fFourCorner	47247
U+8C70.0	fCantonese	bok6; huk1
U+8C70.0	fMandarin	hu4; bo2; huo4; bao2; hu2; gou4; xiao1
U+8C71.0	fUTF8	豱
U+8C71.0	fCNS11643	2-5F60
U+8C71.0	fBig5	EEB4
U+8C71.0	fGB	5-6E4A
U+8C71.0	fR/S	 152.10
U+8C71.0	fTotalStrokes	 17
U+8C71.0	fCangjie	MOWOT
U+8C71.0	fFourCorner	16217
U+8C71.0	fCantonese	wan1
U+8C71.0	fMandarin	wen1
U+8C72.0	fUTF8	豲
U+8C72.0	fCNS11643	2-5F5F
U+8C72.0	fBig5	EEB3
U+8C72.0	fGB	3-6F40
U+8C72.0	fR/S	 152.10
U+8C72.0	fTotalStrokes	 17
U+8C72.0	fCangjie	MOMHF
U+8C72.0	fFourCorner	11296
U+8C72.0	fHakka	 ngien2;  fan2
U+8C72.0	fCantonese	jyun4; wun4 (also)
U+8C72.0	fMandarin	huan2; yuan2
U+8C73.0	fUTF8	豳
U+8C73.0	fCNS11643	1-7552
U+8C73.0	fBig5	C1C7
U+8C73.0	fGB	0-6159
U+8C73.0	fR/S	 152.10
U+8C73.0	fTotalStrokes	 17
U+8C73.0	fCangjie	UMOO
U+8C73.0	fFourCorner	22770
U+8C73.0	fHakka	  bin1
U+8C73.0	fCantonese	ban1
U+8C73.0	fMandarin	bin1; ban1
U+8C73.0	fEnglish	[1] Pin,a state during the Chou dynasty [2] a mountain in Shensi
U+8C74.0	fUTF8	豴
U+8C74.0	fCNS11643	3-592B
U+8C74.0	fGB	5-6E4F
U+8C74.0	fR/S	 152.11
U+8C74.0	fTotalStrokes	 18
U+8C74.0	fCangjie	MOYCB
U+8C74.0	fFourCorner	10220
U+8C74.0	fHakka	  dit7
U+8C74.0	fCantonese	dik1
U+8C74.0	fMandarin	di2
U+8C74.0	fEnglish	the hoof of a pig
U+8C75.0	fUTF8	豵
U+8C75.0	fCNS11643	2-635A
U+8C75.0	fBig5	F0EF
U+8C75.0	fGB	3-6F2E
U+8C75.0	fR/S	 152.11
U+8C75.0	fTotalStrokes	 18
U+8C75.0	fCangjie	MOHOO
U+8C75.0	fFourCorner	18281
U+8C75.0	fHakka	  zung1
U+8C75.0	fCantonese	zung1
U+8C75.0	fMandarin	zong1; cong3
U+8C75.0	fEnglish	young pig
U+8C76.0	fUTF8	豶
U+8C76.0	fCNS11643	2-6732
U+8C76.0	fBig5	F346
U+8C76.0	fGB	3-6F3C
U+8C76.0	fR/S	 152.13
U+8C76.0	fTotalStrokes	 20
U+8C76.0	fCangjie	MOJTC
U+8C76.0	fFourCorner	14286
U+8C76.0	fHakka	  fun2
U+8C76.0	fCantonese	fan4
U+8C76.0	fMandarin	fen2
U+8C76.0	fEnglish	castrate pig
U+8C77.0	fUTF8	豷
U+8C77.0	fCNS11643	2-6731
U+8C77.0	fBig5	F345
U+8C77.0	fGB	3-6F42
U+8C77.0	fR/S	 152.12
U+8C77.0	fTotalStrokes	 19
U+8C77.0	fCangjie	MOGBT
U+8C77.0	fFourCorner	14218
U+8C77.0	fCantonese	ai3; ngai3 (also); ji3
U+8C77.0	fMandarin	yi4; xi4
U+8C77.0	fEnglish	man
U+8C78.0	fUTF8	豸
U+8C78.0	fCNS11643	2-2524
U+8C78.0	fBig5	CBA4
U+8C78.0	fGB	0-7574
U+8C78.0	fR/S	 153.0
U+8C78.0	fTotalStrokes	 7
U+8C78.0	fCangjie	BSHH
U+8C78.0	fFourCorner	20222
U+8C78.0	fHakka	 sai3; chi5; cih3; chi3; ci1; cih5; zi1; cih2; chi2; chi1; chii5
U+8C78.0	fCantonese	zi6; zaai6 (also)
U+8C78.0	fMandarin	zhi4; zhai4
U+8C78.0	fEnglish	[1] legless insects [2] Kangxi radical 153
U+8C79.0	fUTF8	豹
U+8C79.0	fCNS11643	1-5841
U+8C79.0	fBig5	B05C
U+8C79.0	fGB	0-312A
U+8C79.0	fR/S	 153.3
U+8C79.0	fTotalStrokes	 10
U+8C79.0	fCangjie	BHPI
U+8C79.0	fFourCorner	27220
U+8C79.0	fHakka	  bau5; ba5
U+8C79.0	fCantonese	paau3; baau3
U+8C79.0	fMandarin	bao4
U+8C79.0	fEnglish	[1] leopard; panther [2] a Chinese family name
U+8C7A.0	fUTF8	豺
U+8C7A.0	fCNS11643	1-5840
U+8C7A.0	fBig5	B05B
U+8C7A.0	fGB	0-3272
U+8C7A.0	fR/S	 153.3
U+8C7A.0	fTotalStrokes	 10
U+8C7A.0	fCangjie	BHDH
U+8C7A.0	fFourCorner	24200
U+8C7A.0	fHakka	  sai2; sai1; cai2
U+8C7A.0	fCantonese	caai4
U+8C7A.0	fMandarin	chai2
U+8C7A.0	fEnglish	[1] Canis hodphilax, a ravenous beast, akin to the wolf [2] cruel; wickedly; mean; cunning
U+8C7B.0	fUTF8	豻
U+8C7B.0	fCNS11643	2-3324
U+8C7B.0	fBig5	D3E0
U+8C7B.0	fGB	3-7227
U+8C7B.0	fR/S	 153.3
U+8C7B.0	fTotalStrokes	 10
U+8C7B.0	fCangjie	BHMJ
U+8C7B.0	fFourCorner	11240
U+8C7B.0	fHakka	  gon1; ngan5
U+8C7B.0	fCantonese	ngon6; hon6; on6 (also); hon4
U+8C7B.0	fMandarin	an4; han4
U+8C7B.0	fEnglish	[1] prison; lockup [2] a kind of wild dog
U+8C7C.0	fUTF8	豼
U+8C7C.0	fCNS11643	3-392E
U+8C7C.0	fGB	E-3E4B
U+8C7C.0	fR/S	 153.4
U+8C7C.0	fTotalStrokes	 11
U+8C7C.0	fCangjie	BHPP
U+8C7C.0	fFourCorner	21210
U+8C7C.0	fHakka	 pi1
U+8C7C.0	fCantonese	pei4
U+8C7C.0	fMandarin	pi2
U+8C7C.0	fEnglish	fox; leopard; panther
U+8C7D.0	fUTF8	豽
U+8C7D.0	fCNS11643	2-3954
U+8C7D.0	fBig5	D7D1
U+8C7D.0	fGB	3-7229
U+8C7D.0	fR/S	 153.4
U+8C7D.0	fTotalStrokes	 11
U+8C7D.0	fCangjie	BHOB
U+8C7D.0	fFourCorner	14227
U+8C7D.0	fHakka	 nut8; nui6; nui5
U+8C7D.0	fCantonese	neot6; leot6 (also); naap6
U+8C7D.0	fMandarin	na4
U+8C7D.0	fEnglish	a kind of striped seal
U+8C7E.0	fUTF8	豾
U+8C7E.0	fCNS11643	3-3F2C
U+8C7E.0	fGB	5-7065
U+8C7E.0	fR/S	 153.5
U+8C7E.0	fTotalStrokes	 12
U+8C7E.0	fCangjie	BHMFM
U+8C7E.0	fFourCorner	21210
U+8C7E.0	fHakka	  pi1
U+8C7E.0	fCantonese	pei1
U+8C7E.0	fMandarin	pi1
U+8C7E.0	fEnglish	cub of a fox
U+8C7F.0	fUTF8	豿
U+8C7F.0	fCNS11643	3-3F2B
U+8C7F.0	fGB	3-722B
U+8C7F.0	fR/S	 153.5
U+8C7F.0	fTotalStrokes	 12
U+8C7F.0	fCangjie	BHPR
U+8C7F.0	fFourCorner	27220
U+8C7F.0	fCantonese	gau2
U+8C7F.0	fMandarin	gou3
U+8C80.0	fUTF8	貀
U+8C80.0	fCNS11643	2-404D
U+8C80.0	fBig5	DBE7
U+8C80.0	fGB	5-7069
U+8C80.0	fR/S	 153.5
U+8C80.0	fTotalStrokes	 12
U+8C80.0	fCangjie	BHUU
U+8C80.0	fFourCorner	12277
U+8C80.0	fHakka	 nut8
U+8C80.0	fCantonese	neot6; leot6 (also); zyut3
U+8C80.0	fMandarin	na4; duo4
U+8C80.0	fEnglish	a kind of striped seal
U+8C81.0	fUTF8	貁
U+8C81.0	fCNS11643	2-404C
U+8C81.0	fBig5	DBE6
U+8C81.0	fGB	5-7068
U+8C81.0	fR/S	 153.5
U+8C81.0	fTotalStrokes	 12
U+8C81.0	fCangjie	BHJC
U+8C81.0	fFourCorner	13282
U+8C81.0	fHakka	 ju5; riu5
U+8C81.0	fCantonese	jau6; jau5
U+8C81.0	fMandarin	you4
U+8C81.0	fEnglish	the name of an animal
U+8C82.0	fUTF8	貂
U+8C82.0	fCNS11643	1-6230
U+8C82.0	fBig5	B649
U+8C82.0	fGB	0-7575
U+8C82.0	fR/S	 153.5
U+8C82.0	fTotalStrokes	 12
U+8C82.0	fCangjie	BHSHR
U+8C82.0	fFourCorner	27262
U+8C82.0	fHakka	  diau1
U+8C82.0	fCantonese	diu1
U+8C82.0	fMandarin	diao1
U+8C82.0	fEnglish	the marten; sable; mink
U+8C83.0	fUTF8	貃
U+8C83.0	fCNS11643	3-3F2D
U+8C83.0	fGB	5-7066
U+8C83.0	fR/S	 153.5
U+8C83.0	fTotalStrokes	 12
U+8C83.0	fCangjie	BHHA
U+8C83.0	fFourCorner	26200
U+8C83.0	fCantonese	mak6
U+8C83.0	fMandarin	mo4
U+8C84.0	fUTF8	貄
U+8C84.0	fCNS11643	2-475F
U+8C84.0	fBig5	E059
U+8C84.0	fGB	E-3E4C
U+8C84.0	fR/S	 153.6
U+8C84.0	fTotalStrokes	 13
U+8C84.0	fCangjie	BHLQ
U+8C84.0	fFourCorner	15207
U+8C84.0	fCantonese	si3; gei6; sei3
U+8C84.0	fMandarin	si4
U+8C85.0	fUTF8	貅
U+8C85.0	fCNS11643	2-4760
U+8C85.0	fBig5	E05A
U+8C85.0	fGB	0-7577
U+8C85.0	fR/S	 153.6
U+8C85.0	fTotalStrokes	 13
U+8C85.0	fCangjie	BHOD
U+8C85.0	fFourCorner	24290
U+8C85.0	fHakka	 hiu1
U+8C85.0	fCantonese	jau1
U+8C85.0	fMandarin	xiu1
U+8C85.0	fEnglish	[1] a kind of animal like a tiger [2] a fierce and courageous soldier
U+8C86.0	fUTF8	貆
U+8C86.0	fCNS11643	2-475E
U+8C86.0	fBig5	E058
U+8C86.0	fGB	3-722D
U+8C86.0	fR/S	 153.6
U+8C86.0	fTotalStrokes	 13
U+8C86.0	fCangjie	BHMAM
U+8C86.0	fFourCorner	11216
U+8C86.0	fHakka	 fan2; sien2
U+8C86.0	fCantonese	wun4; hyun1 (also); fun1
U+8C86.0	fMandarin	huan2; yuan2; huan1
U+8C86.0	fEnglish	 small badger porcupine
U+8C87.0	fUTF8	貇
U+8C87.0	fCNS11643	4-4078
U+8C87.0	fGB	5-706F
U+8C87.0	fR/S	 153.6
U+8C87.0	fTotalStrokes	 13
U+8C87.0	fCangjie	BHAV
U+8C87.0	fFourCorner	27230
U+8C87.0	fHakka	  ken3; kun1; kwun1; kwun3
U+8C87.0	fCantonese	han2
U+8C87.0	fMandarin	kun1
U+8C87.0	fEnglish	 dig; gnaw
U+8C88.0	fUTF8	貈
U+8C88.0	fCNS11643	3-4465
U+8C88.0	fGB	5-706E
U+8C88.0	fR/S	 153.6
U+8C88.0	fTotalStrokes	 13
U+8C88.0	fCangjie	BHHBY
U+8C88.0	fFourCorner	27240
U+8C88.0	fHakka	 hok8; hok7
U+8C88.0	fCantonese	hok6; mak6
U+8C88.0	fMandarin	he2; hao2; ma4; mo4
U+8C88.0	fEnglish	an animal like a fox
U+8C89.0	fUTF8	貉
U+8C89.0	fCNS11643	1-666F
U+8C89.0	fBig5	B8E8
U+8C89.0	fGB	0-3A51
U+8C89.0	fR/S	 153.6
U+8C89.0	fTotalStrokes	 13
U+8C89.0	fCangjie	BHHER
U+8C89.0	fFourCorner	27264
U+8C89.0	fHakka	  hok8;  met8; met7; hok7
U+8C89.0	fCantonese	hok6; mak6; maak6; lok3
U+8C89.0	fMandarin	he2; hao2; ma4
U+8C89.0	fEnglish	 badger; raccoon dog; foxlike animal nocturnal in habit
U+8C89.1	fUTF8	貉
U+8C89.1	fCNS11643	1-666F
U+8C89.1	fBig5	B8E8
U+8C89.1	fGB	0-3A51
U+8C89.1	fR/S	 153.6
U+8C89.1	fTotalStrokes	 13
U+8C89.1	fCangjie	BHHER
U+8C89.1	fFourCorner	27264
U+8C89.1	fMandarin	mo4
U+8C89.1	fEnglish	[1] the name of a northern barbarian tribe [2] quiet
U+8C8A.0	fUTF8	貊
U+8C8A.0	fCNS11643	1-666E
U+8C8A.0	fBig5	B8E7
U+8C8A.0	fGB	0-7576
U+8C8A.0	fR/S	 153.6
U+8C8A.0	fTotalStrokes	 13
U+8C8A.0	fCangjie	BHMA
U+8C8A.0	fFourCorner	21260
U+8C8A.0	fHakka	 met8; met7
U+8C8A.0	fCantonese	mak6; maak6
U+8C8A.0	fMandarin	mo4
U+8C8A.0	fEnglish	 leopard ancient tribe in north
U+8C8B.0	fUTF8	貋
U+8C8B.0	fCNS11643	4-4754
U+8C8B.0	fGB	E-3E4D
U+8C8B.0	fR/S	 153.7
U+8C8B.0	fTotalStrokes	 14
U+8C8B.0	fCangjie	BHAMJ
U+8C8B.0	fFourCorner	26240
U+8C8B.0	fCantonese	ngon6; on6 (also)
U+8C8B.0	fMandarin	han4; an4
U+8C8C.0	fUTF8	貌
U+8C8C.0	fCNS11643	1-6B32
U+8C8C.0	fBig5	BBAA
U+8C8C.0	fGB	0-4332
U+8C8C.0	fR/S	 153.7
U+8C8C.0	fTotalStrokes	 14
U+8C8C.0	fCangjie	BHHAU
U+8C8C.0	fFourCorner	26210
U+8C8C.0	fHakka	  mau5; mau6; mau3
U+8C8C.0	fCantonese	maau6
U+8C8C.0	fMandarin	mao4
U+8C8C.0	fEnglish	[1] countenance; facial appearance; features [2] general appearance; manner; form; bearing [3] appear; pretend to be like [4] ceremonious manner
U+8C8D.0	fUTF8	貍
U+8C8D.0	fCNS11643	1-6B31
U+8C8D.0	fBig5	BBA9
U+8C8D.0	fGB	E-3E4E
U+8C8D.0	fR/S	 153.7
U+8C8D.0	fTotalStrokes	 14
U+8C8D.0	fCangjie	BHWG
U+8C8D.0	fFourCorner	26214
U+8C8D.0	fHakka	 li2
U+8C8D.0	fCantonese	lei4; maai4
U+8C8D.0	fMandarin	li2; mai2; mei4; yu4
U+8C8D.0	fEnglish	Nyctereutus procynoides, a fox-like animal
U+8C8E.0	fUTF8	貎
U+8C8E.0	fCNS11643	3-4E7A
U+8C8E.0	fGB	E-3E4F
U+8C8E.0	fR/S	 153.8
U+8C8E.0	fTotalStrokes	 15
U+8C8E.0	fCangjie	BHHXU
U+8C8E.0	fFourCorner	27210
U+8C8E.0	fCantonese	maau6
U+8C8E.0	fMandarin	ni2
U+8C8E.0	fEnglish	 lion wild beast wild horse
U+8C8F.0	fUTF8	貏
U+8C8F.0	fCNS11643	2-5450
U+8C8F.0	fBig5	E7E7
U+8C8F.0	fGB	3-722E
U+8C8F.0	fR/S	 153.8
U+8C8F.0	fTotalStrokes	 15
U+8C8F.0	fCangjie	BHHWJ
U+8C8F.0	fFourCorner	26240
U+8C8F.0	fHakka	 bi3
U+8C8F.0	fCantonese	bei2; bei3
U+8C8F.0	fMandarin	bi3; bei4
U+8C8F.0	fEnglish	 slope; bank
U+8C90.0	fUTF8	貐
U+8C90.0	fCNS11643	2-5A5F
U+8C90.0	fBig5	EBB3
U+8C90.0	fGB	5-7074
U+8C90.0	fR/S	 153.9
U+8C90.0	fTotalStrokes	 16
U+8C90.0	fCangjie	BHOMN
U+8C90.0	fFourCorner	28221
U+8C90.0	fCantonese	jyu5
U+8C90.0	fMandarin	yu3; wei3
U+8C91.0	fUTF8	貑
U+8C91.0	fCNS11643	2-5A5D
U+8C91.0	fBig5	EBB1
U+8C91.0	fGB	3-7230
U+8C91.0	fR/S	 153.9
U+8C91.0	fTotalStrokes	 16
U+8C91.0	fCangjie	BHRSE
U+8C91.0	fFourCorner	27247
U+8C91.0	fCantonese	gaa1
U+8C91.0	fMandarin	jia1
U+8C92.0	fUTF8	貒
U+8C92.0	fCNS11643	2-5A5E
U+8C92.0	fBig5	EBB2
U+8C92.0	fGB	3-722F
U+8C92.0	fR/S	 153.9
U+8C92.0	fTotalStrokes	 16
U+8C92.0	fCangjie	BHUMB
U+8C92.0	fFourCorner	22227
U+8C92.0	fHakka	 ton2
U+8C92.0	fCantonese	teon1; tyun1
U+8C92.0	fMandarin	tuan1; huan1; tuan4
U+8C92.0	fEnglish	a species of small badger
U+8C93.0	fUTF8	貓
U+8C93.0	fCNS11643	1-7249
U+8C93.0	fBig5	BFDF
U+8C93.0	fGB	E-3E50
U+8C93.0	fR/S	 153.9; 153.8
U+8C93.0	fTotalStrokes	 16
U+8C93.0	fCangjie	BHTW
U+8C93.0	fFourCorner	44260
U+8C93.0	fHakka	  miau5;  miau2; ngiau5; ngeu5
U+8C93.0	fCantonese	maau1; miu4; maau4
U+8C93.0	fMandarin	mao1; mao2
U+8C93.0	fEnglish	the cat
U+8C94.0	fUTF8	貔
U+8C94.0	fCNS11643	2-5F63
U+8C94.0	fBig5	EEB7
U+8C94.0	fGB	0-7579
U+8C94.0	fR/S	 153.10
U+8C94.0	fTotalStrokes	 17
U+8C94.0	fCangjie	BHHWP
U+8C94.0	fFourCorner	26211
U+8C94.0	fHakka	  pi1; pi2
U+8C94.0	fCantonese	pei4
U+8C94.0	fMandarin	pi2
U+8C94.0	fEnglish	a fierce animal of the panther family
U+8C95.0	fUTF8	貕
U+8C95.0	fCNS11643	2-5F62
U+8C95.0	fBig5	EEB6
U+8C95.0	fGB	3-7231
U+8C95.0	fR/S	 153.10
U+8C95.0	fTotalStrokes	 17
U+8C95.0	fCangjie	BHBVK
U+8C95.0	fFourCorner	12234
U+8C95.0	fCantonese	hai4
U+8C95.0	fMandarin	xi1
U+8C96.0	fUTF8	貖
U+8C96.0	fCNS11643	3-564F
U+8C96.0	fGB	E-3E51
U+8C96.0	fR/S	 153.10
U+8C96.0	fTotalStrokes	 17
U+8C96.0	fCangjie	BHTCT
U+8C96.0	fFourCorner	28210
U+8C96.0	fCantonese	jik1
U+8C96.0	fMandarin	e4
U+8C97.0	fUTF8	貗
U+8C97.0	fCNS11643	2-635D
U+8C97.0	fBig5	F0F2
U+8C97.0	fGB	E-3E52
U+8C97.0	fR/S	 153.11
U+8C97.0	fTotalStrokes	 18
U+8C97.0	fCangjie	BHLLV
U+8C97.0	fFourCorner	25244
U+8C97.0	fCantonese	geoi6
U+8C97.0	fMandarin	ju4
U+8C98.0	fUTF8	貘
U+8C98.0	fCNS11643	2-635C
U+8C98.0	fBig5	F0F1
U+8C98.0	fGB	0-7578
U+8C98.0	fR/S	 153.11
U+8C98.0	fTotalStrokes	 18
U+8C98.0	fCangjie	BHTAK
U+8C98.0	fFourCorner	24234
U+8C98.0	fHakka	 met8
U+8C98.0	fCantonese	mak6; mok6 (also)
U+8C98.0	fMandarin	mo4
U+8C98.0	fEnglish	[1] the panther; lepoard [2] the tapir
U+8C99.0	fUTF8	貙
U+8C99.0	fCNS11643	2-635B
U+8C99.0	fBig5	F0F0
U+8C99.0	fGB	3-7228
U+8C99.0	fR/S	 153.11
U+8C99.0	fTotalStrokes	 18
U+8C99.0	fCangjie	BHSRR
U+8C99.0	fFourCorner	21216
U+8C99.0	fHakka	 ki1; ji1; ri1
U+8C99.0	fCantonese	ceoi1; keoi1; syu1; jyu6
U+8C99.0	fMandarin	chu1; chu2
U+8C9A.0	fUTF8	貚
U+8C9A.0	fCNS11643	2-6733
U+8C9A.0	fBig5	F347
U+8C9A.0	fGB	E-3E53
U+8C9A.0	fR/S	 153.12
U+8C9A.0	fTotalStrokes	 19
U+8C9A.0	fCangjie	BHRRJ
U+8C9A.0	fFourCorner	26246
U+8C9A.0	fCantonese	taan4; tin4
U+8C9A.0	fMandarin	tan2; dan4; tan1; tian2
U+8C9B.0	fUTF8	貛
U+8C9B.0	fCNS11643	3-614D
U+8C9B.0	fGB	E-3E54
U+8C9B.0	fR/S	 153.18
U+8C9B.0	fTotalStrokes	 25
U+8C9B.0	fCangjie	BHTRG
U+8C9B.0	fFourCorner	24210
U+8C9B.0	fHakka	 fon1; fan1
U+8C9B.0	fCantonese	fun1
U+8C9B.0	fMandarin	huan1
U+8C9B.0	fEnglish	badger
U+8C9C.0	fUTF8	貜
U+8C9C.0	fCNS11643	2-7178
U+8C9C.0	fBig5	F9AA
U+8C9C.0	fGB	5-7078
U+8C9C.0	fR/S	 153.20
U+8C9C.0	fTotalStrokes	 27
U+8C9C.0	fCangjie	BHBUE
U+8C9C.0	fFourCorner	16247
U+8C9C.0	fCantonese	fok3; gwok3
U+8C9C.0	fMandarin	jue2; yue4
U+8C9C.0	fEnglish	ape
U+8C9D.0	fUTF8	貝
U+8C9D.0	fCNS11643	1-4B4A
U+8C9D.0	fBig5	A8A9
U+8C9D.0	fGB	1-3134
U+8C9D.0	fR/S	 154.0
U+8C9D.0	fTotalStrokes	 7
U+8C9D.0	fCangjie	BUC
U+8C9D.0	fFourCorner	60800
U+8C9D.0	fHakka	  bui5;  bi5
U+8C9D.0	fCantonese	bui3
U+8C9D.0	fMandarin	bei4
U+8C9D.0	fEnglish	[1] shells; cowries used as money [2] precious; treasure; valuable [3] bell (physics) [4] a Chinese family name [5] Kangxi radical 154
U+8C9E.0	fUTF8	貞
U+8C9E.0	fCNS11643	1-5356
U+8C9E.0	fBig5	AD73
U+8C9E.0	fGB	1-556A
U+8C9E.0	fR/S	 154.2
U+8C9E.0	fTotalStrokes	 9
U+8C9E.0	fCangjie	YBUC
U+8C9E.0	fFourCorner	21806
U+8C9E.0	fHakka	  zin1;  zhin1; zihn1
U+8C9E.0	fCantonese	zing1
U+8C9E.0	fMandarin	zhen1; zheng1
U+8C9E.0	fEnglish	[1] chastity of a woman [2] virtuous; chaste; pure [3] be incorruptable; be loyal; be correctly firm [4] devotion; dedication [5] divine; inquire by divination
U+8C9F.0	fUTF8	貟
U+8C9F.0	fCNS11643	3-2F2E
U+8C9F.0	fGB	5-352A
U+8C9F.0	fR/S	 154.2
U+8C9F.0	fTotalStrokes	 9
U+8C9F.0	fCangjie	IBUC
U+8C9F.0	fFourCorner	23800
U+8C9F.0	fHakka	 jan2
U+8C9F.0	fCantonese	jyun4; wan6
U+8C9F.0	fMandarin	yuan2; yun2
U+8C9F.0	fEnglish	 member personnel; staff member
U+8CA0.0	fUTF8	負
U+8CA0.0	fCNS11643	1-5357
U+8CA0.0	fBig5	AD74
U+8CA0.0	fGB	1-383A
U+8CA0.0	fR/S	 154.2
U+8CA0.0	fTotalStrokes	 9
U+8CA0.0	fCangjie	NBUC
U+8CA0.0	fFourCorner	27806
U+8CA0.0	fHakka	  fu5
U+8CA0.0	fCantonese	fu6
U+8CA0.0	fMandarin	fu4
U+8CA0.0	fEnglish	[1] defeated; beaten; lose; fail; defeat [2] bear; sustain; carry on the back; shoulder [3] take refuge in [4] be proud and complacent [5] owe [6] negative; minus [7] turn one's back on; ungrateful [8] let someone down; disappoint [9] a Chinese family name
U+8CA0.1	fUTF8	負
U+8CA0.1	fCNS11643	1-5357
U+8CA0.1	fBig5	AD74
U+8CA0.1	fGB	1-383A
U+8CA0.1	fR/S	 154.2
U+8CA0.1	fTotalStrokes	 9
U+8CA0.1	fCangjie	NBUC
U+8CA0.1	fFourCorner	27806
U+8CA0.1	fMandarin	fu5
U+8CA1.0	fUTF8	財
U+8CA1.0	fCNS11643	1-5842
U+8CA1.0	fBig5	B05D
U+8CA1.0	fGB	1-3246
U+8CA1.0	fR/S	 154.3
U+8CA1.0	fTotalStrokes	 10
U+8CA1.0	fCangjie	BCDH
U+8CA1.0	fFourCorner	64800
U+8CA1.0	fHakka	  coi2; coi3
U+8CA1.0	fCantonese	coi4; coi2 (also)
U+8CA1.0	fMandarin	cai2; zai4
U+8CA1.0	fEnglish	wealth; valuables; riches; money
U+8CA2.0	fUTF8	貢
U+8CA2.0	fCNS11643	1-5843
U+8CA2.0	fBig5	B05E
U+8CA2.0	fGB	1-3931
U+8CA2.0	fR/S	 154.3
U+8CA2.0	fTotalStrokes	 10
U+8CA2.0	fCangjie	MBUC
U+8CA2.0	fFourCorner	10806
U+8CA2.0	fHakka	  gung5
U+8CA2.0	fCantonese	gung3
U+8CA2.0	fMandarin	gong4
U+8CA2.0	fEnglish	[1] offer tribute; the tribute from a vasal state [2] recommend (person, office, etc.); submit [3] land tax (in ancient China) [4] contribute; offer [5] a Chinese family name
U+8CA3.0	fUTF8	貣
U+8CA3.0	fCNS11643	2-3326
U+8CA3.0	fBig5	D3E2
U+8CA3.0	fGB	3-354C
U+8CA3.0	fR/S	 154.3
U+8CA3.0	fTotalStrokes	 10
U+8CA3.0	fCangjie	IPBUC
U+8CA3.0	fFourCorner	43806
U+8CA3.0	fHakka	 tit7; tit8
U+8CA3.0	fCantonese	tik1
U+8CA3.0	fMandarin	te4
U+8CA3.0	fEnglish	 borrow
U+8CA4.0	fUTF8	貤
U+8CA4.0	fCNS11643	2-3325
U+8CA4.0	fBig5	D3E1
U+8CA4.0	fGB	3-4840
U+8CA4.0	fR/S	 154.3
U+8CA4.0	fTotalStrokes	 10
U+8CA4.0	fCangjie	BCPD
U+8CA4.0	fFourCorner	64812
U+8CA4.0	fHakka	 ri3; ji3; ji5; ji2
U+8CA4.0	fCantonese	ji4; ji6; ji2
U+8CA4.0	fMandarin	yi4; shi4
U+8CA4.0	fEnglish	[1] pile one upon another; superimpose [2] extend; prolong
U+8CA4.1	fUTF8	貤
U+8CA4.1	fCNS11643	2-3325
U+8CA4.1	fBig5	D3E1
U+8CA4.1	fGB	3-4840
U+8CA4.1	fR/S	 154.3
U+8CA4.1	fTotalStrokes	 10
U+8CA4.1	fCangjie	BCPD
U+8CA4.1	fFourCorner	64812
U+8CA4.1	fMandarin	yi2
U+8CA4.1	fEnglish	 move; remove; turn
U+8CA5.0	fUTF8	貥
U+8CA5.0	fCNS11643	2-3955
U+8CA5.0	fBig5	D7D2
U+8CA5.0	fGB	E-3E55
U+8CA5.0	fR/S	 154.4
U+8CA5.0	fTotalStrokes	 11
U+8CA5.0	fCangjie	BCYHN
U+8CA5.0	fFourCorner	60817
U+8CA5.0	fCantonese	hong4
U+8CA5.0	fMandarin	hang2
U+8CA6.0	fUTF8	貦
U+8CA6.0	fCNS11643	3-3930
U+8CA6.0	fGB	5-4A41
U+8CA6.0	fR/S	 154.4
U+8CA6.0	fTotalStrokes	 11
U+8CA6.0	fCangjie	BCMMU
U+8CA6.0	fFourCorner	61810
U+8CA6.0	fCantonese	wun6
U+8CA6.0	fMandarin	wan4
U+8CA7.0	fUTF8	貧
U+8CA7.0	fCNS11643	1-5D4E
U+8CA7.0	fBig5	B368
U+8CA7.0	fGB	1-4636
U+8CA7.0	fR/S	 154.4
U+8CA7.0	fTotalStrokes	 11
U+8CA7.0	fCangjie	CSHC
U+8CA7.0	fFourCorner	80806
U+8CA7.0	fHakka	  pin2; pin3
U+8CA7.0	fCantonese	pan4
U+8CA7.0	fMandarin	pin2
U+8CA7.0	fEnglish	[1] poverty; poor; impoverished; needy; destitute [2] deficiency; deficient; lack [3] stingy; tight-fisted [4] long, repetitious or superfluous [5] garulous
U+8CA8.0	fUTF8	貨
U+8CA8.0	fCNS11643	1-5D4C
U+8CA8.0	fBig5	B366
U+8CA8.0	fGB	1-3B75
U+8CA8.0	fR/S	 154.4
U+8CA8.0	fTotalStrokes	 11
U+8CA8.0	fCangjie	OPBUC
U+8CA8.0	fFourCorner	24806
U+8CA8.0	fHakka	  fo5
U+8CA8.0	fCantonese	fo3; fo6
U+8CA8.0	fMandarin	huo4
U+8CA8.0	fEnglish	[1] goods; commodities; products; freight; cargo [2] money; currency; property [3] bribe; bribery [4] sell [5] used as a term of reviling with an abusive suffix
U+8CA9.0	fUTF8	販
U+8CA9.0	fCNS11643	1-5D49
U+8CA9.0	fBig5	B363
U+8CA9.0	fGB	1-3737
U+8CA9.0	fR/S	 154.4
U+8CA9.0	fTotalStrokes	 11
U+8CA9.0	fCangjie	BCHE
U+8CA9.0	fFourCorner	61847
U+8CA9.0	fHakka	  fan5; pan3; pan5
U+8CA9.0	fCantonese	faan3; faan5 (also); faan2 (also)
U+8CA9.0	fMandarin	fan4
U+8CA9.0	fEnglish	[1] buy and sell; deal in; trade in [2] carry about for sale; peddle [3] peddler; seller of goods; monger; street merchant
U+8CAA.0	fUTF8	貪
U+8CAA.0	fCNS11643	1-5D4D
U+8CAA.0	fBig5	B367
U+8CAA.0	fGB	1-4C30
U+8CAA.0	fR/S	 154.4
U+8CAA.0	fTotalStrokes	 11
U+8CAA.0	fCangjie	OINC
U+8CAA.0	fFourCorner	80806
U+8CAA.0	fHakka	  tam1
U+8CAA.0	fCantonese	taam1
U+8CAA.0	fMandarin	tan1; tan4
U+8CAA.0	fEnglish	[1] desire for more than one's rightful share; greedy; covet; covetous [2] hope for; wish for; probe for; search for
U+8CAB.0	fUTF8	貫
U+8CAB.0	fCNS11643	1-5D4B
U+8CAB.0	fBig5	B365
U+8CAB.0	fGB	1-3961
U+8CAB.0	fR/S	 154.4
U+8CAB.0	fTotalStrokes	 11
U+8CAB.0	fCangjie	WJBUC
U+8CAB.0	fFourCorner	77806
U+8CAB.0	fHakka	 gon5;  kwan5; gan5; gwon5; gwan5
U+8CAB.0	fCantonese	gun3; gwaan3; gwun3 (also)
U+8CAB.0	fMandarin	guan4; wan1; guan1
U+8CAB.0	fEnglish	[1] a thread for stringing holed copper coins; string on a thread [2] a string of 1,000 holed copper coins [3] pierce through; go through; pass through; see through; throughout; thorough [4] be linked together; follow in a continuous line [5] hit the target [6] one's native place [7] a weight measurement equivalent to about 7 pounds [8] a Chinese family name
U+8CAC.0	fUTF8	責
U+8CAC.0	fCNS11643	1-5D4A
U+8CAC.0	fBig5	B364
U+8CAC.0	fGB	1-5470
U+8CAC.0	fR/S	 154.4
U+8CAC.0	fTotalStrokes	 11
U+8CAC.0	fCangjie	QMBUC
U+8CAC.0	fFourCorner	50806
U+8CAC.0	fHakka	  zit7; zak7; zit8
U+8CAC.0	fCantonese	zaak3; zaai3
U+8CAC.0	fMandarin	ze2; zhai4
U+8CAC.0	fEnglish	[1] one's responsibility; one's duty; responsibility [2] demand; be strict with [3] punish; punishment [4] upbraid; censure; reprimand; blame
U+8CAD.0	fUTF8	貭
U+8CAD.0	fCNS11643	3-3932
U+8CAD.0	fGB	E-3E56
U+8CAD.0	fR/S	 154.4
U+8CAD.0	fTotalStrokes	 11
U+8CAD.0	fCangjie	HJBC
U+8CAD.0	fFourCorner	72280
U+8CAD.0	fHakka	 zhit7; zhi5; tsiit7; zih5
U+8CAD.0	fCantonese	zat1
U+8CAD.0	fMandarin	zhi2; zhi4
U+8CAD.0	fEnglish	matter; material; substance
U+8CAE.0	fUTF8	貮
U+8CAE.0	fCNS11643	3-3931
U+8CAE.0	fGB	E-3E57
U+8CAE.0	fR/S	 154.4
U+8CAE.0	fTotalStrokes	 11
U+8CAE.0	fCangjie	MPBUC
U+8CAE.0	fFourCorner	13840
U+8CAE.0	fCantonese	ji6
U+8CAE.0	fMandarin	er4
U+8CAE.0	fEnglish	number two
U+8CAF.0	fUTF8	貯
U+8CAF.0	fCNS11643	1-6231
U+8CAF.0	fBig5	B64A
U+8CAF.0	fGB	1-567C
U+8CAF.0	fR/S	 154.5
U+8CAF.0	fTotalStrokes	 12
U+8CAF.0	fCangjie	BCJMN
U+8CAF.0	fFourCorner	63821
U+8CAF.0	fHakka	  du3; du5; zhu5; zu5
U+8CAF.0	fCantonese	cyu5; cyu2; zyu2
U+8CAF.0	fMandarin	zhu3; zhu4
U+8CAF.0	fEnglish	 storeup; stockpile; hoard; save up; deposit
U+8CB0.0	fUTF8	貰
U+8CB0.0	fCNS11643	2-4050
U+8CB0.0	fBig5	DBEA
U+8CB0.0	fGB	1-6A5B
U+8CB0.0	fR/S	 154.5
U+8CB0.0	fTotalStrokes	 12
U+8CB0.0	fCangjie	PTBUC
U+8CB0.0	fFourCorner	44806
U+8CB0.0	fHakka	 sa5; shi5; sii5; si5; sha5; she5; ji5
U+8CB0.0	fCantonese	sai3
U+8CB0.0	fMandarin	shi4; she4
U+8CB0.0	fEnglish	[1] lend; loan; lease [2] pardon; pardon; forgive [3] sell on credit; buy on credit
U+8CB1.0	fUTF8	貱
U+8CB1.0	fCNS11643	3-3F2F
U+8CB1.0	fGB	3-4848
U+8CB1.0	fR/S	 154.5
U+8CB1.0	fTotalStrokes	 12
U+8CB1.0	fCangjie	BCDHE
U+8CB1.0	fFourCorner	67840
U+8CB1.0	fHakka	  bi5
U+8CB1.0	fCantonese	bei3
U+8CB1.0	fMandarin	bi4
U+8CB1.0	fEnglish	 transfer
U+8CB2.0	fUTF8	貲
U+8CB2.0	fCNS11643	1-6674
U+8CB2.0	fBig5	B8ED
U+8CB2.0	fGB	1-6A5F
U+8CB2.0	fR/S	 154.5
U+8CB2.0	fTotalStrokes	 12
U+8CB2.0	fCangjie	YPBUC
U+8CB2.0	fFourCorner	21806
U+8CB2.0	fHakka	 zih1; zu1; zii1
U+8CB2.0	fCantonese	zi1
U+8CB2.0	fMandarin	zi1
U+8CB2.0	fEnglish	[1] property; wealth; riches; money [2] count; measure; estimate [3] fine
U+8CB3.0	fUTF8	貳
U+8CB3.0	fCNS11643	1-6233
U+8CB3.0	fBig5	B64C
U+8CB3.0	fGB	1-3721
U+8CB3.0	fR/S	 154.5
U+8CB3.0	fTotalStrokes	 12
U+8CB3.0	fCangjie	IPMMC
U+8CB3.0	fFourCorner	43800
U+8CB3.0	fHakka	 ngi5; ngi6; gni6
U+8CB3.0	fCantonese	ji6
U+8CB3.0	fMandarin	er4
U+8CB3.0	fEnglish	[1] deputy; serve as a deputy (in ancient China) [2] suspect; doubt; distrust [3] changeable [4] two used in accountancy [5] repeat [6] doubleness [7] a Chinese family name
U+8CB4.0	fUTF8	貴
U+8CB4.0	fCNS11643	1-6238
U+8CB4.0	fBig5	B651
U+8CB4.0	fGB	1-3973
U+8CB4.0	fR/S	 154.5
U+8CB4.0	fTotalStrokes	 12
U+8CB4.0	fCangjie	LMBUC
U+8CB4.0	fFourCorner	50806
U+8CB4.0	fHakka	  gui5; gwi5; kwui5
U+8CB4.0	fCantonese	gwai3
U+8CB4.0	fMandarin	gui4
U+8CB4.0	fEnglish	[1] high-placed; high-ranking; honorable; distinguished [2] expensive; costly; valuable; high-priced [3] esteem; treat with respect [4] treasure; value highly; prize [5] valuable; precious [6] a polite expression referring to another person; you; your [7] a Chinese family name [8] short for Kweichow province
U+8CB5.0	fUTF8	貵
U+8CB5.0	fCNS11643	2-4052
U+8CB5.0	fBig5	DBEC
U+8CB5.0	fGB	E-3E58
U+8CB5.0	fR/S	 154.5
U+8CB5.0	fTotalStrokes	 12
U+8CB5.0	fCangjie	IKBUC
U+8CB5.0	fFourCorner	23806
U+8CB5.0	fCantonese	pin2
U+8CB5.0	fMandarin	pian3; pian1
U+8CB6.0	fUTF8	貶
U+8CB6.0	fCNS11643	1-623A
U+8CB6.0	fBig5	B653
U+8CB6.0	fGB	1-3161
U+8CB6.0	fR/S	 154.5
U+8CB6.0	fTotalStrokes	 12
U+8CB6.0	fCangjie	BCHIO
U+8CB6.0	fFourCorner	62837
U+8CB6.0	fHakka	  bien3; ben3; pian3
U+8CB6.0	fCantonese	bin2
U+8CB6.0	fMandarin	bian3; ban3; fa2
U+8CB6.0	fEnglish	[1] decrease; devalue; lower (prices, etc.) [2] degrade; reduce; demote [3] disparage; condemn; censure [4] dismiss; send away
U+8CB7.0	fUTF8	買
U+8CB7.0	fCNS11643	1-6239
U+8CB7.0	fBig5	B652
U+8CB7.0	fGB	1-4272
U+8CB7.0	fR/S	 154.5
U+8CB7.0	fTotalStrokes	 12
U+8CB7.0	fCangjie	WLBUC
U+8CB7.0	fFourCorner	60806
U+8CB7.0	fHakka	  mai1;  mai3
U+8CB7.0	fCantonese	maai5
U+8CB7.0	fMandarin	mai3
U+8CB7.0	fEnglish	[1] buy; purchase [2] win over (usually with a promise or favors in return); bribe
U+8CB8.0	fUTF8	貸
U+8CB8.0	fCNS11643	1-623C
U+8CB8.0	fBig5	B655
U+8CB8.0	fGB	1-347B
U+8CB8.0	fR/S	 154.5
U+8CB8.0	fTotalStrokes	 12
U+8CB8.0	fCangjie	OPBUC
U+8CB8.0	fFourCorner	23806
U+8CB8.0	fHakka	 toi5; toi6; tai5; toi3
U+8CB8.0	fCantonese	taai3; tik1
U+8CB8.0	fMandarin	dai4; te4; tai3
U+8CB8.0	fEnglish	[1] loan; lend; borrow [2] pardon; be lenient [3] the credit side in bookkeeping [4] shift (responsibility); shirk
U+8CB9.0	fUTF8	貹
U+8CB9.0	fCNS11643	2-4051
U+8CB9.0	fBig5	DBEB
U+8CB9.0	fGB	3-4844
U+8CB9.0	fR/S	 154.5
U+8CB9.0	fTotalStrokes	 12
U+8CB9.0	fCangjie	BCHQM
U+8CB9.0	fFourCorner	65810
U+8CB9.0	fHakka	 sen5; sen3
U+8CB9.0	fCantonese	sang3; sing3
U+8CB9.0	fMandarin	sheng4
U+8CB9.0	fEnglish	rich; wealthy
U+8CBA.0	fUTF8	貺
U+8CBA.0	fCNS11643	2-404E
U+8CBA.0	fBig5	DBE8
U+8CBA.0	fGB	1-6A5C
U+8CBA.0	fR/S	 154.5
U+8CBA.0	fTotalStrokes	 12
U+8CBA.0	fCangjie	BCRHU
U+8CBA.0	fFourCorner	66810
U+8CBA.0	fHakka	 kong3; kwong3; k'wong3
U+8CBA.0	fCantonese	fong3
U+8CBA.0	fMandarin	kuang4; huang4
U+8CBA.0	fEnglish	[1] give; grant; bestow; confer [2] be favored wiht (a polite expression) [3] a Chinese family name
U+8CBB.0	fUTF8	費
U+8CBB.0	fCNS11643	1-6236
U+8CBB.0	fBig5	B64F
U+8CBB.0	fGB	1-3751
U+8CBB.0	fR/S	 154.5
U+8CBB.0	fTotalStrokes	 12
U+8CBB.0	fCangjie	LNBUC
U+8CBB.0	fFourCorner	55806
U+8CBB.0	fHakka	  fui5; fi5; bui5; bi5; fui6
U+8CBB.0	fCantonese	fai3; bei3
U+8CBB.0	fMandarin	fei4
U+8CBB.0	fEnglish	[1] expenses; expenditure; fees; dues; charges [2] waste; use more than needed; wastful; consuming too much [3] consume; use; spend; cost; expend [4] a Chinese family name
U+8CBB.1	fUTF8	費
U+8CBB.1	fCNS11643	1-6236
U+8CBB.1	fBig5	B64F
U+8CBB.1	fGB	1-3751
U+8CBB.1	fR/S	 154.5
U+8CBB.1	fTotalStrokes	 12
U+8CBB.1	fCangjie	LNBUC
U+8CBB.1	fFourCorner	55806
U+8CBB.1	fMandarin	bi4
U+8CBB.1	fEnglish	a town in the Spring and Autumn period in today's Shantung province
U+8CBC.0	fUTF8	貼
U+8CBC.0	fCNS11643	1-6232
U+8CBC.0	fBig5	B64B
U+8CBC.0	fGB	1-4C79
U+8CBC.0	fR/S	 154.5
U+8CBC.0	fTotalStrokes	 12
U+8CBC.0	fCangjie	BCYR
U+8CBC.0	fFourCorner	61860
U+8CBC.0	fHakka	  tiap7;  diap7; dap7; dap8
U+8CBC.0	fCantonese	tip3; tip1 (also)
U+8CBC.0	fMandarin	tie1
U+8CBC.0	fEnglish	[1] paste; stick; attach; glue [2] keep close; nestle closely to [3] make up the deficiency; subsidise; subsidies; allowance [4] proper; appropiate; comfortable [5] attached to
U+8CBD.0	fUTF8	貽
U+8CBD.0	fCNS11643	1-6234
U+8CBD.0	fBig5	B64D
U+8CBD.0	fGB	1-6A5D
U+8CBD.0	fR/S	 154.5
U+8CBD.0	fTotalStrokes	 12
U+8CBD.0	fCangjie	BCIR
U+8CBD.0	fFourCorner	63860
U+8CBD.0	fHakka	 ji2; ri2; i2
U+8CBD.0	fCantonese	ji4
U+8CBD.0	fMandarin	yi2; yi4
U+8CBD.0	fEnglish	[1] give to; present to [2] hand down; bequeath; transmit; pass on to; leave behind
U+8CBE.0	fUTF8	貾
U+8CBE.0	fCNS11643	2-404F
U+8CBE.0	fBig5	DBE9
U+8CBE.0	fGB	5-4A45
U+8CBE.0	fR/S	 154.5
U+8CBE.0	fTotalStrokes	 12
U+8CBE.0	fCangjie	BCHPM
U+8CBE.0	fFourCorner	62840
U+8CBE.0	fCantonese	ci4
U+8CBE.0	fMandarin	chi2; chi1
U+8CBF.0	fUTF8	貿
U+8CBF.0	fCNS11643	1-623B
U+8CBF.0	fBig5	B654
U+8CBF.0	fGB	1-4333
U+8CBF.0	fR/S	 154.5
U+8CBF.0	fTotalStrokes	 12
U+8CBF.0	fCangjie	HHBUC
U+8CBF.0	fFourCorner	77806
U+8CBF.0	fHakka	 meu3; meu6; miu3; meu5; miau3
U+8CBF.0	fCantonese	mau6
U+8CBF.0	fMandarin	mao4; mou4
U+8CBF.0	fEnglish	[1] trade; barter; exchange [2] mixed [3] rashly
U+8CC0.0	fUTF8	賀
U+8CC0.0	fCNS11643	1-6237
U+8CC0.0	fBig5	B650
U+8CC0.0	fGB	1-3A58
U+8CC0.0	fR/S	 154.5
U+8CC0.0	fTotalStrokes	 12
U+8CC0.0	fCangjie	KRBUC
U+8CC0.0	fFourCorner	46806
U+8CC0.0	fHakka	  fo5; ho5; fo3
U+8CC0.0	fCantonese	ho6; ho3
U+8CC0.0	fMandarin	he4
U+8CC0.0	fEnglish	[1] congratulate; send a present in congratulation [2] a Chinese family name
U+8CC1.0	fUTF8	賁
U+8CC1.0	fCNS11643	1-6235
U+8CC1.0	fBig5	B64E
U+8CC1.0	fGB	1-6A5A
U+8CC1.0	fR/S	 154.5
U+8CC1.0	fTotalStrokes	 12
U+8CC1.0	fCangjie	JTBC
U+8CC1.0	fFourCorner	40806
U+8CC1.0	fHakka	 bun1; bi5; pun1; fun3; fun2; fun1
U+8CC1.0	fCantonese	bei3; fan4; ban1; fan5; ban3
U+8CC1.0	fMandarin	bi4; fei2; fen4
U+8CC1.0	fEnglish	[1] adorn; ornamental [2] bright; luminous
U+8CC1.1	fUTF8	賁
U+8CC1.1	fCNS11643	1-6235
U+8CC1.1	fBig5	B64E
U+8CC1.1	fGB	1-6A5A
U+8CC1.1	fR/S	 154.5
U+8CC1.1	fTotalStrokes	 12
U+8CC1.1	fCangjie	JTBC
U+8CC1.1	fFourCorner	40806
U+8CC1.1	fMandarin	ben1
U+8CC1.1	fEnglish	[1] forge ahead [2] energetic; strenuous [3] a Chinese family name
U+8CC1.2	fUTF8	賁
U+8CC1.2	fCNS11643	1-6235
U+8CC1.2	fBig5	B64E
U+8CC1.2	fGB	1-6A5A
U+8CC1.2	fR/S	 154.5
U+8CC1.2	fTotalStrokes	 12
U+8CC1.2	fCangjie	JTBC
U+8CC1.2	fFourCorner	40806
U+8CC1.2	fMandarin	fen2
U+8CC1.2	fEnglish	large great
U+8CC2.0	fUTF8	賂
U+8CC2.0	fCNS11643	1-6676
U+8CC2.0	fBig5	B8EF
U+8CC2.0	fGB	1-4238
U+8CC2.0	fR/S	 154.6
U+8CC2.0	fTotalStrokes	 13
U+8CC2.0	fCangjie	BCHER
U+8CC2.0	fFourCorner	67864
U+8CC2.0	fHakka	 lu5; lu3; lu6
U+8CC2.0	fCantonese	lou6
U+8CC2.0	fMandarin	lu4
U+8CC2.0	fEnglish	[1] bribe [2] give a present; send a present
U+8CC3.0	fUTF8	賃
U+8CC3.0	fCNS11643	1-6675
U+8CC3.0	fBig5	B8EE
U+8CC3.0	fGB	1-415E
U+8CC3.0	fR/S	 154.6
U+8CC3.0	fTotalStrokes	 13
U+8CC3.0	fCangjie	OGBUC
U+8CC3.0	fFourCorner	22806
U+8CC3.0	fHakka	 jim5; gnim5; rim6; ngim3; ngim5
U+8CC3.0	fCantonese	jam6
U+8CC3.0	fMandarin	ren4; lin4
U+8CC3.0	fEnglish	[1] rent; hire [2] hired person; hireling
U+8CC4.0	fUTF8	賄
U+8CC4.0	fCNS11643	1-6673
U+8CC4.0	fBig5	B8EC
U+8CC4.0	fGB	1-3B5F
U+8CC4.0	fR/S	 154.6
U+8CC4.0	fTotalStrokes	 13
U+8CC4.0	fCangjie	BCKB
U+8CC4.0	fFourCorner	64827
U+8CC4.0	fHakka	 fui3;  ju3; ju5; fui5; riu6; riu3; fui6; fi5; fi3
U+8CC4.0	fCantonese	kui2; fui2 (also); kwui2 (also)
U+8CC4.0	fMandarin	hui4; hui3
U+8CC4.0	fEnglish	[1] bribe; bribery [2] meney; riches; wealth
U+8CC5.0	fUTF8	賅
U+8CC5.0	fCNS11643	1-6677
U+8CC5.0	fBig5	B8F0
U+8CC5.0	fGB	1-6A60
U+8CC5.0	fR/S	 154.6
U+8CC5.0	fTotalStrokes	 13
U+8CC5.0	fCangjie	BCYVO
U+8CC5.0	fFourCorner	60882
U+8CC5.0	fHakka	 goi1
U+8CC5.0	fCantonese	goi1
U+8CC5.0	fMandarin	gai1; hai4
U+8CC5.0	fEnglish	[1] provided for; inclusive; included in; all-inclusive; nothing left out [2] have; possess
U+8CC6.0	fUTF8	賆
U+8CC6.0	fCNS11643	4-407E
U+8CC6.0	fGB	E-3E59
U+8CC6.0	fR/S	 154.6
U+8CC6.0	fTotalStrokes	 13
U+8CC6.0	fCangjie	BCTT
U+8CC6.0	fFourCorner	68840
U+8CC6.0	fCantonese	pin4
U+8CC6.0	fMandarin	pian2
U+8CC7.0	fUTF8	資
U+8CC7.0	fCNS11643	1-6671
U+8CC7.0	fBig5	B8EA
U+8CC7.0	fGB	1-574A
U+8CC7.0	fR/S	 154.6
U+8CC7.0	fTotalStrokes	 13
U+8CC7.0	fCangjie	IOBUC
U+8CC7.0	fFourCorner	37806
U+8CC7.0	fHakka	  zih1;  zu1; zii1
U+8CC7.0	fCantonese	zi1
U+8CC7.0	fMandarin	zi1; zi4
U+8CC7.0	fEnglish	[1] money; property; wealth; capital [2] expenses; fees; charges [3] natural endowments; natural gifts; one's disposition [4] avail of [5] aid; help; assist; subsidise; support [6] supply; provide [7] one's qualifications, position, or record of service [8] trust to
U+8CC8.0	fUTF8	賈
U+8CC8.0	fCNS11643	1-6672
U+8CC8.0	fBig5	B8EB
U+8CC8.0	fGB	1-3C56
U+8CC8.0	fR/S	 154.6
U+8CC8.0	fTotalStrokes	 13
U+8CC8.0	fCangjie	MWBUC
U+8CC8.0	fFourCorner	10806
U+8CC8.0	fHakka	  gu3;  ga3; ga5
U+8CC8.0	fCantonese	gaa2; gu2; gaa3; gwu2 (also)
U+8CC8.0	fMandarin	jia3; jia4
U+8CC8.0	fEnglish	a Chinses family name
U+8CC8.1	fUTF8	賈
U+8CC8.1	fCNS11643	1-6672
U+8CC8.1	fBig5	B8EB
U+8CC8.1	fGB	1-3C56
U+8CC8.1	fR/S	 154.6
U+8CC8.1	fTotalStrokes	 13
U+8CC8.1	fCangjie	MWBUC
U+8CC8.1	fFourCorner	10806
U+8CC8.1	fMandarin	gu3
U+8CC8.1	fEnglish	[1] merchant; businessman [2] buy; trade
U+8CC9.0	fUTF8	賉
U+8CC9.0	fCNS11643	3-4469
U+8CC9.0	fGB	E-3E5A
U+8CC9.0	fR/S	 154.6
U+8CC9.0	fTotalStrokes	 13
U+8CC9.0	fCangjie	BCHBT
U+8CC9.0	fFourCorner	67810
U+8CC9.0	fHakka	  sit7
U+8CC9.0	fCantonese	seot1
U+8CC9.0	fMandarin	xu4
U+8CC9.0	fEnglish	[1] sympathy; pity [2] give alms; give compensation
U+8CCA.0	fUTF8	賊
U+8CCA.0	fCNS11643	1-6670
U+8CCA.0	fBig5	B8E9
U+8CCA.0	fGB	1-5474
U+8CCA.0	fR/S	 154.6
U+8CCA.0	fTotalStrokes	 13
U+8CCA.0	fCangjie	BCIJ
U+8CCA.0	fFourCorner	63850
U+8CCA.0	fHakka	  cet8
U+8CCA.0	fCantonese	caak6 (also); caak3; caak2 (also)
U+8CCA.0	fMandarin	zei2; ze2
U+8CCA.0	fEnglish	[1] thief; burglar; robber; bandit [2] rebel; traitor [3] harm [4] kil [5] pests on the farm [6] a term of reviling [7] clever; cunning; crafty
U+8CCB.0	fUTF8	賋
U+8CCB.0	fCNS11643	5-432D
U+8CCB.0	fGB	E-3E5B
U+8CCB.0	fR/S	 154.6
U+8CCB.0	fTotalStrokes	 13
U+8CCB.0	fCangjie	BCYCK
U+8CCB.0	fFourCorner	60840
U+8CCB.0	fCantonese	gaau2
U+8CCB.0	fMandarin	jiao3
U+8CCC.0	fUTF8	賌
U+8CCC.0	fCNS11643	2-4761
U+8CCC.0	fBig5	E05B
U+8CCC.0	fGB	5-4A49
U+8CCC.0	fR/S	 154.6
U+8CCC.0	fTotalStrokes	 13
U+8CCC.0	fCangjie	YVOC
U+8CCC.0	fFourCorner	00806
U+8CCC.0	fCantonese	goi1
U+8CCC.0	fMandarin	gai1; gai4
U+8CCD.0	fUTF8	賍
U+8CCD.0	fCNS11643	3-4467
U+8CCD.0	fGB	E-3E5C
U+8CCD.0	fR/S	 154.6
U+8CCD.0	fTotalStrokes	 13
U+8CCD.0	fCangjie	BCIG
U+8CCD.0	fFourCorner	60810
U+8CCD.0	fHakka	 zong1; cong2
U+8CCD.0	fCantonese	zong6; zong1
U+8CCD.0	fMandarin	zang1
U+8CCD.0	fEnglish	stolen goods or property; loot
U+8CCE.0	fUTF8	賎
U+8CCE.0	fGB	E-3E5D
U+8CCE.0	fR/S	 154.6
U+8CCE.0	fTotalStrokes	 13
U+8CCE.0	fCangjie	BCIQ
U+8CCE.0	fCantonese	zin6
U+8CCE.0	fMandarin	jian4
U+8CCE.0	fEnglish	 mean; low cheap; worthless
U+8CCF.0	fUTF8	賏
U+8CCF.0	fCNS11643	2-4E3C
U+8CCF.0	fBig5	E454
U+8CCF.0	fGB	E-3E5E
U+8CCF.0	fR/S	 154.7
U+8CCF.0	fTotalStrokes	 14
U+8CCF.0	fCangjie	BCBUC
U+8CCF.0	fFourCorner	66880
U+8CCF.0	fHakka	 jin1; jin3; rin1; rin3
U+8CCF.0	fCantonese	jing1
U+8CCF.0	fMandarin	ying4; ying1
U+8CCF.0	fEnglish	pearls or shells strung together
U+8CD0.0	fUTF8	賐
U+8CD0.0	fCNS11643	3-4970
U+8CD0.0	fGB	E-3E5F
U+8CD0.0	fR/S	 154.7
U+8CD0.0	fTotalStrokes	 14
U+8CD0.0	fCangjie	BCICE
U+8CD0.0	fFourCorner	63840
U+8CD0.0	fCantonese	seon3
U+8CD0.0	fMandarin	xun4
U+8CD1.0	fUTF8	賑
U+8CD1.0	fCNS11643	1-6B34
U+8CD1.0	fBig5	BBAC
U+8CD1.0	fGB	1-6A62
U+8CD1.0	fR/S	 154.7
U+8CD1.0	fTotalStrokes	 14
U+8CD1.0	fCangjie	BCMMV
U+8CD1.0	fFourCorner	61832
U+8CD1.0	fHakka	 zhin3; zihn3; zen3; zin3
U+8CD1.0	fCantonese	zan3
U+8CD1.0	fMandarin	zhen4; zhen3
U+8CD1.0	fEnglish	[1] relieve; give aid to distressed; support [2] rich; wealthy; prosperous
U+8CD2.0	fUTF8	賒
U+8CD2.0	fCNS11643	1-6B35
U+8CD2.0	fBig5	BBAD
U+8CD2.0	fGB	1-495E
U+8CD2.0	fR/S	 154.7
U+8CD2.0	fTotalStrokes	 14
U+8CD2.0	fCangjie	BCOMF
U+8CD2.0	fFourCorner	68891
U+8CD2.0	fHakka	 cha1; ca1; se1
U+8CD2.0	fCantonese	se1
U+8CD2.0	fMandarin	she1; sha1
U+8CD2.0	fEnglish	[1] buy or sell on credit [2] distant; far away [3] slow; slowly [4] put of; postpone [5] luxurious; extravagant
U+8CD3.0	fUTF8	賓
U+8CD3.0	fCNS11643	1-6B33
U+8CD3.0	fBig5	BBAB
U+8CD3.0	fGB	1-3176
U+8CD3.0	fR/S	 154.7; 154.8
U+8CD3.0	fTotalStrokes	 14
U+8CD3.0	fCangjie	JMHC
U+8CD3.0	fFourCorner	30806
U+8CD3.0	fHakka	  bin1
U+8CD3.0	fCantonese	ban1; ban3
U+8CD3.0	fMandarin	bin1; bin4
U+8CD3.0	fEnglish	[1] guest; visitor [2] treat as a guest [3] obey; follow instructions; submit [4] a Chinese family name
U+8CD4.0	fUTF8	賔
U+8CD4.0	fCNS11643	3-4972
U+8CD4.0	fGB	E-3E60
U+8CD4.0	fR/S	 154.7; 154.11; 40.11
U+8CD4.0	fTotalStrokes	 14
U+8CD4.0	fCangjie	JAHC
U+8CD4.0	fFourCorner	30286
U+8CD4.0	fHakka	 bin1
U+8CD4.0	fCantonese	ban1
U+8CD4.0	fMandarin	bin1; bin4
U+8CD4.0	fEnglish	 guest; visitor a Chinese family name submit
U+8CD5.0	fUTF8	賕
U+8CD5.0	fCNS11643	2-4E3B
U+8CD5.0	fBig5	E453
U+8CD5.0	fGB	1-6A64
U+8CD5.0	fR/S	 154.7
U+8CD5.0	fTotalStrokes	 14
U+8CD5.0	fCangjie	BCIJE
U+8CD5.0	fFourCorner	63832
U+8CD5.0	fHakka	  kiu2
U+8CD5.0	fCantonese	kau4
U+8CD5.0	fMandarin	qiu2
U+8CD5.0	fEnglish	 bribe
U+8CD6.0	fUTF8	賖
U+8CD6.0	fCNS11643	3-4971
U+8CD6.0	fGB	E-3E61
U+8CD6.0	fR/S	 154.7
U+8CD6.0	fTotalStrokes	 14
U+8CD6.0	fCangjie	BCOMD
U+8CD6.0	fFourCorner	68890
U+8CD6.0	fCantonese	se1
U+8CD6.0	fMandarin	she1
U+8CD7.0	fUTF8	賗
U+8CD7.0	fCNS11643	2-4E3D
U+8CD7.0	fBig5	E455
U+8CD7.0	fGB	E-3E62
U+8CD7.0	fR/S	 154.7
U+8CD7.0	fTotalStrokes	 14
U+8CD7.0	fCangjie	BCLL
U+8CD7.0	fFourCorner	65806
U+8CD7.0	fCantonese	cyun3
U+8CD7.0	fMandarin	chuan4
U+8CD8.0	fUTF8	賘
U+8CD8.0	fCNS11643	3-664E
U+8CD8.0	fGB	E-3E63
U+8CD8.0	fR/S	 154.7
U+8CD8.0	fTotalStrokes	 14
U+8CD8.0	fCangjie	BCIGI
U+8CD8.0	fFourCorner	6081
U+8CD8.0	fCantonese	zong1
U+8CD8.0	fMandarin	zang1
U+8CD9.0	fUTF8	賙
U+8CD9.0	fCNS11643	2-5453
U+8CD9.0	fBig5	E7EA
U+8CD9.0	fGB	5-4A4F
U+8CD9.0	fR/S	 154.8
U+8CD9.0	fTotalStrokes	 15
U+8CD9.0	fCangjie	BCBGR
U+8CD9.0	fFourCorner	67820
U+8CD9.0	fHakka	 zhiu1; zu1; zhu1
U+8CD9.0	fCantonese	zau1
U+8CD9.0	fMandarin	zhou1
U+8CD9.0	fEnglish	 give for charity; give; aid; relieve
U+8CDA.0	fUTF8	賚
U+8CDA.0	fCNS11643	2-5455
U+8CDA.0	fBig5	E7EC
U+8CDA.0	fGB	1-6A63
U+8CDA.0	fR/S	 154.8
U+8CDA.0	fTotalStrokes	 15
U+8CDA.0	fCangjie	DOBUC
U+8CDA.0	fFourCorner	40806
U+8CDA.0	fHakka	 loi3; loi5
U+8CDA.0	fCantonese	loi6
U+8CDA.0	fMandarin	lai4; lai2
U+8CDA.0	fEnglish	 bestow; confer
U+8CDB.0	fUTF8	賛
U+8CDB.0	fCNS11643	3-4E7E
U+8CDB.0	fGB	E-3E64
U+8CDB.0	fR/S	 154.8
U+8CDB.0	fTotalStrokes	 15
U+8CDB.0	fCangjie	QOBUC
U+8CDB.0	fFourCorner	55800
U+8CDB.0	fHakka	  zan5
U+8CDB.0	fCantonese	zaan3
U+8CDB.0	fMandarin	zan4
U+8CDB.0	fEnglish	help; support; assist; aid
U+8CDC.0	fUTF8	賜
U+8CDC.0	fCNS11643	1-6F32
U+8CDC.0	fBig5	BDE7
U+8CDC.0	fGB	1-344D
U+8CDC.0	fR/S	 154.8
U+8CDC.0	fTotalStrokes	 15
U+8CDC.0	fCangjie	BCAPH
U+8CDC.0	fFourCorner	66827
U+8CDC.0	fHakka	  su5; sii5; se5; ze5; chu5
U+8CDC.0	fCantonese	ci3
U+8CDC.0	fMandarin	si4; ci4
U+8CDC.0	fEnglish	[1] confer on an inferior; bestow on an inferior; grant [2] favors; benefits [3] order; appoint
U+8CDD.0	fUTF8	賝
U+8CDD.0	fCNS11643	2-5456
U+8CDD.0	fBig5	E7ED
U+8CDD.0	fGB	5-4A50
U+8CDD.0	fR/S	 154.8
U+8CDD.0	fTotalStrokes	 15
U+8CDD.0	fCangjie	BCBCD
U+8CDD.0	fFourCorner	67894
U+8CDD.0	fHakka	 chim1; cihm1
U+8CDD.0	fCantonese	cam1; sam1
U+8CDD.0	fMandarin	chen1
U+8CDD.0	fEnglish	preious
U+8CDE.0	fUTF8	賞
U+8CDE.0	fCNS11643	1-6F2B
U+8CDE.0	fBig5	BDE0
U+8CDE.0	fGB	1-494D
U+8CDE.0	fR/S	 154.8
U+8CDE.0	fTotalStrokes	 15
U+8CDE.0	fCangjie	FBRBC
U+8CDE.0	fFourCorner	90806
U+8CDE.0	fHakka	  song3; shong3
U+8CDE.0	fCantonese	soeng2
U+8CDE.0	fMandarin	shang3
U+8CDE.0	fEnglish	[1] reward; grant; bestow; award; give to an inferior [2] reward; award [3] appreciate; enjoy; admire [4] respect [5] be given the honor of ... (a polite expression)
U+8CDF.0	fUTF8	賟
U+8CDF.0	fCNS11643	2-5452
U+8CDF.0	fBig5	E7E9
U+8CDF.0	fGB	E-3E65
U+8CDF.0	fR/S	 154.8
U+8CDF.0	fTotalStrokes	 15
U+8CDF.0	fCangjie	BCTBC
U+8CDF.0	fFourCorner	65881
U+8CDF.0	fCantonese	tin2
U+8CDF.0	fMandarin	tian3
U+8CE0.0	fUTF8	賠
U+8CE0.0	fCNS11643	1-6F2A
U+8CE0.0	fBig5	BDDF
U+8CE0.0	fGB	1-4562
U+8CE0.0	fR/S	 154.8
U+8CE0.0	fTotalStrokes	 15
U+8CE0.0	fCangjie	BCYTR
U+8CE0.0	fFourCorner	60861
U+8CE0.0	fHakka	  poi2; poi5; pui5
U+8CE0.0	fCantonese	pui4
U+8CE0.0	fMandarin	pei2
U+8CE0.0	fEnglish	[1] compensate; indemnify; make up for a loss due to one's fault; pay for [2] offer (an apology) [3] lose money
U+8CE1.0	fUTF8	賡
U+8CE1.0	fCNS11643	1-6F34
U+8CE1.0	fBig5	BDE9
U+8CE1.0	fGB	1-6259
U+8CE1.0	fR/S	 154.8
U+8CE1.0	fTotalStrokes	 15
U+8CE1.0	fCangjie	ILOC
U+8CE1.0	fFourCorner	00286
U+8CE1.0	fHakka	  gang1
U+8CE1.0	fCantonese	gang1; zuk6
U+8CE1.0	fMandarin	geng1; geng4
U+8CE1.0	fEnglish	 continue; carry on
U+8CE2.0	fUTF8	賢
U+8CE2.0	fCNS11643	1-6F30
U+8CE2.0	fBig5	BDE5
U+8CE2.0	fGB	1-4F4D
U+8CE2.0	fR/S	 154.8
U+8CE2.0	fTotalStrokes	 15
U+8CE2.0	fCangjie	SEBUC
U+8CE2.0	fFourCorner	77806
U+8CE2.0	fHakka	  hien2;  hen2; hian3
U+8CE2.0	fCantonese	jin4
U+8CE2.0	fMandarin	xian2; xian4
U+8CE2.0	fEnglish	[1] capable; able; versatile; talented [2] virtuous; worthy; good [3] admire; praise; esteem [4] a term of respectful address to another
U+8CE3.0	fUTF8	賣
U+8CE3.0	fCNS11643	1-6F31
U+8CE3.0	fBig5	BDE6
U+8CE3.0	fGB	1-4274
U+8CE3.0	fR/S	 154.8
U+8CE3.0	fTotalStrokes	 15
U+8CE3.0	fCangjie	GWLC
U+8CE3.0	fFourCorner	40806
U+8CE3.0	fHakka	  mai3;  mai5; mai6
U+8CE3.0	fCantonese	maai6; maai2 (also); juk6
U+8CE3.0	fMandarin	mai4; yu4
U+8CE3.0	fEnglish	[1] sell [2] betray; harm another in order to benefit oneself [3] show off; flaunt
U+8CE3.1	fUTF8	賣
U+8CE3.1	fCNS11643	1-6F31
U+8CE3.1	fBig5	BDE6
U+8CE3.1	fGB	1-4274
U+8CE3.1	fR/S	 154.8
U+8CE3.1	fTotalStrokes	 15
U+8CE3.1	fCangjie	GWLC
U+8CE3.1	fFourCorner	40806
U+8CE3.1	fMandarin	mai5
U+8CE4.0	fUTF8	賤
U+8CE4.0	fCNS11643	1-6F2D
U+8CE4.0	fBig5	BDE2
U+8CE4.0	fGB	1-3C7A
U+8CE4.0	fR/S	 154.8
U+8CE4.0	fTotalStrokes	 15
U+8CE4.0	fCangjie	BCII
U+8CE4.0	fFourCorner	63853
U+8CE4.0	fHakka	 cien5; cen3; cien3; ts'ian6; cien6
U+8CE4.0	fCantonese	zin6; daam5
U+8CE4.0	fMandarin	jian4
U+8CE4.0	fEnglish	[1] cheap; inexpensive; low cost [2] lowly; humble; inferior in position [3] low-down; base; ignoble; despicable [4] slight; look down to [5] my; a self-derogatory expression [6] a Chinese family name
U+8CE5.0	fUTF8	賥
U+8CE5.0	fCNS11643	2-5451
U+8CE5.0	fBig5	E7E8
U+8CE5.0	fGB	3-484D
U+8CE5.0	fR/S	 154.8
U+8CE5.0	fTotalStrokes	 15
U+8CE5.0	fCangjie	BCYOJ
U+8CE5.0	fFourCorner	60848
U+8CE5.0	fCantonese	seoi6
U+8CE5.0	fMandarin	sui4
U+8CE6.0	fUTF8	賦
U+8CE6.0	fCNS11643	1-6F2C
U+8CE6.0	fBig5	BDE1
U+8CE6.0	fGB	1-3833
U+8CE6.0	fR/S	 154.8; 154.7
U+8CE6.0	fTotalStrokes	 15
U+8CE6.0	fCangjie	BCMPM
U+8CE6.0	fFourCorner	63840
U+8CE6.0	fHakka	  fu5; fu3
U+8CE6.0	fCantonese	fu3
U+8CE6.0	fMandarin	fu4
U+8CE6.0	fEnglish	[1] tax; revenue [2] give; bestow [3] troops; army; military levies [4] spread; diffuse [5] natural endowments or gifts [6] compose; sing (especially poems) [7] one of the Chinese literary form akin to poetry
U+8CE7.0	fUTF8	賧
U+8CE7.0	fCNS11643	2-5457
U+8CE7.0	fBig5	E7EE
U+8CE7.0	fGB	1-6A66
U+8CE7.0	fR/S	 154.8
U+8CE7.0	fTotalStrokes	 15
U+8CE7.0	fCangjie	BCFF
U+8CE7.0	fFourCorner	69889
U+8CE7.0	fHakka	 tam5; tam6
U+8CE7.0	fCantonese	daam6; zin6
U+8CE7.0	fMandarin	tan4; dan4; tan3
U+8CE7.0	fEnglish	fine
U+8CE8.0	fUTF8	賨
U+8CE8.0	fCNS11643	2-5454
U+8CE8.0	fBig5	E7EB
U+8CE8.0	fGB	3-484E
U+8CE8.0	fR/S	 154.8
U+8CE8.0	fTotalStrokes	 15
U+8CE8.0	fCangjie	JMMC
U+8CE8.0	fFourCorner	30806
U+8CE8.0	fHakka	  zung1
U+8CE8.0	fCantonese	cung4
U+8CE8.0	fMandarin	cong2
U+8CE8.0	fEnglish	tribute paid in cloth by the tribes in Szechwan and Yunnan under the Han dynasty
U+8CE9.0	fUTF8	賩
U+8CE9.0	fCNS11643	3-4E7D
U+8CE9.0	fGB	E-3E66
U+8CE9.0	fR/S	 154.8
U+8CE9.0	fTotalStrokes	 15
U+8CE9.0	fCangjie	BCJMF
U+8CE9.0	fFourCorner	63890
U+8CE9.0	fHakka	  zung1
U+8CE9.0	fCantonese	cung4
U+8CE9.0	fMandarin	cong2
U+8CE9.0	fEnglish	presents of cloth given as tribute under the Han dynasty
U+8CEA.0	fUTF8	質
U+8CEA.0	fCNS11643	1-6F33
U+8CEA.0	fBig5	BDE8
U+8CEA.0	fGB	1-564A
U+8CEA.0	fR/S	 154.8
U+8CEA.0	fTotalStrokes	 15
U+8CEA.0	fCangjie	HLBUC
U+8CEA.0	fFourCorner	72806
U+8CEA.0	fHakka	 zit7;  zhit7; zhi5; tsiit7; chip7; zi5; zih5; tje5
U+8CEA.0	fCantonese	zat1; zi3
U+8CEA.0	fMandarin	zhi2
U+8CEA.0	fEnglish	[1] matter; material; substance; elements [2] one's disposition or temperament; qualities [3] simple; plain [4] question [5] confront
U+8CEA.1	fUTF8	質
U+8CEA.1	fCNS11643	1-6F33
U+8CEA.1	fBig5	BDE8
U+8CEA.1	fGB	1-564A
U+8CEA.1	fR/S	 154.8
U+8CEA.1	fTotalStrokes	 15
U+8CEA.1	fCangjie	HLBUC
U+8CEA.1	fFourCorner	72806
U+8CEA.1	fMandarin	zhi4
U+8CEA.1	fEnglish	[1] pawn [2] pledge; hostage
U+8CEB.0	fUTF8	賫
U+8CEB.0	fCNS11643	3-4F22
U+8CEB.0	fGB	1-6A65
U+8CEB.0	fR/S	 154.8
U+8CEB.0	fTotalStrokes	 15
U+8CEB.0	fCangjie	JBBUC
U+8CEB.0	fFourCorner	40800
U+8CEB.0	fHakka	 zi1; zih1
U+8CEB.0	fCantonese	zai1; cai4
U+8CEB.0	fMandarin	ji1; qi2
U+8CEB.0	fEnglish	present
U+8CEC.0	fUTF8	賬
U+8CEC.0	fCNS11643	1-6F2E
U+8CEC.0	fBig5	BDE3
U+8CEC.0	fGB	1-554B
U+8CEC.0	fR/S	 154.8
U+8CEC.0	fTotalStrokes	 15
U+8CEC.0	fCangjie	BCSMV
U+8CEC.0	fFourCorner	61832
U+8CEC.0	fHakka	 zong5; zhong5
U+8CEC.0	fCantonese	zoeng3; zan3
U+8CEC.0	fMandarin	zhang4
U+8CEC.0	fEnglish	[1] accounts [2] debts [3] credits; loans; bills
U+8CED.0	fUTF8	賭
U+8CED.0	fCNS11643	1-6F2F
U+8CED.0	fBig5	BDE4
U+8CED.0	fGB	1-3644
U+8CED.0	fR/S	 154.9
U+8CED.0	fTotalStrokes	 16
U+8CED.0	fCangjie	BCJKA
U+8CED.0	fFourCorner	64863
U+8CED.0	fHakka	  du3
U+8CED.0	fCantonese	dou2
U+8CED.0	fMandarin	du3
U+8CED.0	fEnglish	[1] bet; gamble; wager [2] compete [3] swear
U+8CEE.0	fUTF8	賮
U+8CEE.0	fCNS11643	2-5A61
U+8CEE.0	fBig5	EBB5
U+8CEE.0	fGB	5-4A53
U+8CEE.0	fR/S	 154.9
U+8CEE.0	fTotalStrokes	 16
U+8CEE.0	fCangjie	LMFBC
U+8CEE.0	fFourCorner	50806
U+8CEE.0	fHakka	 zin3; zin5
U+8CEE.0	fCantonese	zeon2; zeon6
U+8CEE.0	fMandarin	jin4; xin4
U+8CEE.0	fEnglish	farewell gift
U+8CEF.0	fUTF8	賯
U+8CEF.0	fCNS11643	3-5275
U+8CEF.0	fGB	E-3E67
U+8CEF.0	fR/S	 154.9
U+8CEF.0	fTotalStrokes	 16
U+8CEF.0	fCangjie	BCPAD
U+8CEF.0	fFourCorner	67840
U+8CEF.0	fCantonese	hing1
U+8CEF.0	fMandarin	min2; xiong1
U+8CF0.0	fUTF8	賰
U+8CF0.0	fCNS11643	2-5A63
U+8CF0.0	fBig5	EBB7
U+8CF0.0	fGB	E-3E68
U+8CF0.0	fR/S	 154.9
U+8CF0.0	fTotalStrokes	 16
U+8CF0.0	fCangjie	BCQKA
U+8CF0.0	fFourCorner	65863
U+8CF0.0	fCantonese	seon2; ceon2
U+8CF0.0	fMandarin	shun3
U+8CF1.0	fUTF8	賱
U+8CF1.0	fCNS11643	2-5A62
U+8CF1.0	fBig5	EBB6
U+8CF1.0	fGB	E-3E69
U+8CF1.0	fR/S	 154.9
U+8CF1.0	fTotalStrokes	 16
U+8CF1.0	fCangjie	BCBJJ
U+8CF1.0	fFourCorner	67852
U+8CF1.0	fCantonese	wan2; wan3
U+8CF1.0	fMandarin	yun3; yun4
U+8CF2.0	fUTF8	賲
U+8CF2.0	fCNS11643	3-5273
U+8CF2.0	fGB	3-4854
U+8CF2.0	fR/S	  154.9
U+8CF2.0	fTotalStrokes	 16
U+8CF2.0	fCangjie	ODBUC
U+8CF2.0	fFourCorner	26800
U+8CF2.0	fCantonese	bou2
U+8CF2.0	fMandarin	bao3
U+8CF3.0	fUTF8	賳
U+8CF3.0	fCNS11643	2-5A64
U+8CF3.0	fBig5	EBB8
U+8CF3.0	fGB	E-3E6A
U+8CF3.0	fR/S	 154.9
U+8CF3.0	fTotalStrokes	 16
U+8CF3.0	fCangjie	BCJIR
U+8CF3.0	fFourCorner	63850
U+8CF3.0	fCantonese	zoi1
U+8CF3.0	fMandarin	zai1
U+8CF4.0	fUTF8	賴
U+8CF4.0	fCNS11643	1-724A
U+8CF4.0	fBig5	BFE0
U+8CF4.0	fGB	1-4035
U+8CF4.0	fR/S	 154.9
U+8CF4.0	fTotalStrokes	 16
U+8CF4.0	fCangjie	DLSHC
U+8CF4.0	fFourCorner	51986
U+8CF4.0	fHakka	 lai5;  lai3; lai2; lai8; lai6
U+8CF4.0	fCantonese	laai6; laai2 (also); laai3 (also)
U+8CF4.0	fMandarin	lai4
U+8CF4.0	fEnglish	[1] rely on; depend on [2] accuse falsely; accuse without grounds or evidence; put the blame on somebody else [3] repudiate (a debt); disavow; deny something which one has said or done [4] postpone or procratinate intentionally [5] no good; poor; bad [6] lazy [7] a Chinese family name
U+8CF5.0	fUTF8	賵
U+8CF5.0	fCNS11643	2-5A60
U+8CF5.0	fBig5	EBB4
U+8CF5.0	fGB	3-4853
U+8CF5.0	fR/S	 154.9
U+8CF5.0	fTotalStrokes	 16
U+8CF5.0	fCangjie	BCABU
U+8CF5.0	fFourCorner	66860
U+8CF5.0	fHakka	 fung3; fung5
U+8CF5.0	fCantonese	fung3; fung6 (also)
U+8CF5.0	fMandarin	feng4
U+8CF5.0	fEnglish	gift
U+8CF6.0	fUTF8	賶
U+8CF6.0	fCNS11643	3-5651
U+8CF6.0	fGB	E-3E6B
U+8CF6.0	fR/S	 154.10
U+8CF6.0	fTotalStrokes	 17
U+8CF6.0	fCangjie	BCOIR
U+8CF6.0	fFourCorner	68860
U+8CF6.0	fCantonese	cong3
U+8CF6.0	fMandarin	cang4
U+8CF7.0	fUTF8	賷
U+8CF7.0	fCNS11643	3-5652
U+8CF7.0	fGB	E-3E6C
U+8CF7.0	fR/S	 154.10
U+8CF7.0	fTotalStrokes	 17
U+8CF7.0	fCangjie	JBBUC
U+8CF7.0	fFourCorner	40800
U+8CF7.0	fHakka	 zi1;  zih1; zii1
U+8CF7.0	fCantonese	zai1; cai4; zaan3
U+8CF7.0	fMandarin	ji1; qi2
U+8CF7.0	fEnglish	[1] take in both hands, and offer [2] send, as apresent or a letter
U+8CF8.0	fUTF8	賸
U+8CF8.0	fCNS11643	1-7556
U+8CF8.0	fBig5	C1CB
U+8CF8.0	fGB	E-3E6D
U+8CF8.0	fR/S	 154.10
U+8CF8.0	fTotalStrokes	 17
U+8CF8.0	fCangjie	BFQC
U+8CF8.0	fFourCorner	79286
U+8CF8.0	fHakka	 jin5; shin5; siin5; sin3; rin5; jin3; chun1; cun1; chin5
U+8CF8.0	fCantonese	sing6; zing6 (also)
U+8CF8.0	fMandarin	sheng4; ying4
U+8CF8.0	fEnglish	[1] leftovers; residue; remains; remnants [2] surplus; overplus [3] superfluous
U+8CF9.0	fUTF8	賹
U+8CF9.0	fCNS11643	2-5F64
U+8CF9.0	fBig5	EEB8
U+8CF9.0	fGB	3-4857
U+8CF9.0	fR/S	 154.10
U+8CF9.0	fTotalStrokes	 17
U+8CF9.0	fCangjie	BCTCT
U+8CF9.0	fFourCorner	68817
U+8CF9.0	fCantonese	jat6; aai3
U+8CF9.0	fMandarin	ai4
U+8CFA.0	fUTF8	賺
U+8CFA.0	fCNS11643	1-7553
U+8CFA.0	fBig5	C1C8
U+8CFA.0	fGB	1-572C
U+8CFA.0	fR/S	 154.10
U+8CFA.0	fTotalStrokes	 17
U+8CFA.0	fCangjie	BCTXC
U+8CFA.0	fFourCorner	68837
U+8CFA.0	fHakka	 con6; can3; ciam3; liam2; con5; can5
U+8CFA.0	fCantonese	zaan6; zaan2 (also); zaam6
U+8CFA.0	fMandarin	zhuan4; zuan4; lian2; zhan4
U+8CFA.0	fEnglish	[1] make money; earn; gain; profit [2] cheat; deceive
U+8CFB.0	fUTF8	賻
U+8CFB.0	fCNS11643	1-7557
U+8CFB.0	fBig5	C1CC
U+8CFB.0	fGB	1-6A67
U+8CFB.0	fR/S	 154.10
U+8CFB.0	fTotalStrokes	 17
U+8CFB.0	fCangjie	BCIBI
U+8CFB.0	fFourCorner	63842
U+8CFB.0	fHakka	  fu5; fu6
U+8CFB.0	fCantonese	fu6
U+8CFB.0	fMandarin	fu4
U+8CFB.0	fEnglish	 help another with money in financing a funeral
U+8CFC.0	fUTF8	購
U+8CFC.0	fCNS11643	1-7555
U+8CFC.0	fBig5	C1CA
U+8CFC.0	fGB	1-393A
U+8CFC.0	fR/S	 154.10
U+8CFC.0	fTotalStrokes	 17
U+8CFC.0	fCangjie	BCTTB
U+8CFC.0	fFourCorner	65857
U+8CFC.0	fHakka	 kieu5; giu5; keu5; geu5
U+8CFC.0	fCantonese	gau3; kau3 (also)
U+8CFC.0	fMandarin	gou4; gou1
U+8CFC.0	fEnglish	 buy; purchase
U+8CFD.0	fUTF8	賽
U+8CFD.0	fCNS11643	1-7554
U+8CFD.0	fBig5	C1C9
U+8CFD.0	fGB	1-487C
U+8CFD.0	fR/S	 154.10
U+8CFD.0	fTotalStrokes	 17
U+8CFD.0	fCangjie	JTCC
U+8CFD.0	fFourCorner	30806
U+8CFD.0	fHakka	  soi5; sai1
U+8CFD.0	fCantonese	coi3
U+8CFD.0	fMandarin	sai4
U+8CFD.0	fEnglish	[1] compete; contend for superiority; contest [2] race; tournament; match; game [3] surpass [4] an ancient sacrifice [5] a Chinese family name
U+8CFE.0	fUTF8	賾
U+8CFE.0	fCNS11643	2-635E
U+8CFE.0	fBig5	F0F3
U+8CFE.0	fGB	1-5853
U+8CFE.0	fR/S	 154.11
U+8CFE.0	fTotalStrokes	 18
U+8CFE.0	fCangjie	SLQMC
U+8CFE.0	fFourCorner	75786
U+8CFE.0	fHakka	  zit7
U+8CFE.0	fCantonese	zaak3; zaak6; zak3
U+8CFE.0	fMandarin	ze2; ze4
U+8CFE.0	fEnglish	abstruse; deep; profound
U+8CFF.0	fUTF8	賿
U+8CFF.0	fCNS11643	4-5D69
U+8CFF.0	fGB	E-3E6E
U+8CFF.0	fR/S	 154.11
U+8CFF.0	fTotalStrokes	 18
U+8CFF.0	fCangjie	BCSMH
U+8CFF.0	fFourCorner	67820
U+8CFF.0	fCantonese	laau4
U+8CFF.0	fMandarin	liao2
U+8D00.0	fUTF8	贀
U+8D00.0	fCNS11643	2-6361
U+8D00.0	fBig5	F0F6
U+8D00.0	fGB	3-4858
U+8D00.0	fR/S	 154.11
U+8D00.0	fTotalStrokes	 18
U+8D00.0	fCangjie	SEBUC
U+8D00.0	fFourCorner	77806
U+8D00.0	fCantonese	ji3; ai3
U+8D00.0	fMandarin	wei4; yi4
U+8D01.0	fUTF8	贁
U+8D01.0	fCNS11643	7-4352
U+8D01.0	fGB	E-3E6F
U+8D01.0	fR/S	 154.11; 66.14
U+8D01.0	fTotalStrokes	 18
U+8D01.0	fCangjie	BCOK
U+8D01.0	fFourCorner	68840
U+8D01.0	fCantonese	baai6
U+8D01.0	fMandarin	bai4
U+8D01.0	fEnglish	 defeat to spoil or be spoiled corrupt or be corrupted
U+8D02.0	fUTF8	贂
U+8D02.0	fCNS11643	2-6360
U+8D02.0	fBig5	F0F5
U+8D02.0	fGB	E-3E70
U+8D02.0	fR/S	 154.11
U+8D02.0	fTotalStrokes	 18
U+8D02.0	fCangjie	BCIIH
U+8D02.0	fFourCorner	63822
U+8D02.0	fHakka	 cam3
U+8D02.0	fCantonese	cam2
U+8D02.0	fMandarin	chen3
U+8D03.0	fUTF8	贃
U+8D03.0	fCNS11643	3-592E
U+8D03.0	fGB	E-3E71
U+8D03.0	fR/S	 154.11
U+8D03.0	fTotalStrokes	 18
U+8D03.0	fCangjie	BCLLP
U+8D03.0	fFourCorner	65830
U+8D03.0	fHakka	 con5; can5; con6; can6
U+8D03.0	fCantonese	taan3; zaam6; zaan6
U+8D03.0	fMandarin	wan4; zhuan4; zuan4
U+8D03.0	fEnglish	 earn; make a profit
U+8D04.0	fUTF8	贄
U+8D04.0	fCNS11643	2-635F
U+8D04.0	fBig5	F0F4
U+8D04.0	fGB	1-6A5E
U+8D04.0	fR/S	 154.11
U+8D04.0	fTotalStrokes	 18
U+8D04.0	fCangjie	GIBUC
U+8D04.0	fFourCorner	45806
U+8D04.0	fHakka	 zhi5; zi5; zih5; tje5
U+8D04.0	fCantonese	zi3
U+8D04.0	fMandarin	zhi4; zhi2; nie4
U+8D04.0	fEnglish	[1] presents given at the first meeting [2] gifts to a superior
U+8D05.0	fUTF8	贅
U+8D05.0	fCNS11643	1-7743
U+8D05.0	fBig5	C2D8
U+8D05.0	fGB	1-5738
U+8D05.0	fR/S	 154.11
U+8D05.0	fTotalStrokes	 18
U+8D05.0	fCangjie	GKBUC
U+8D05.0	fFourCorner	48806
U+8D05.0	fHakka	 zui5; chui5; cui5; zhui5
U+8D05.0	fCantonese	zeoi6; zeoi3
U+8D05.0	fMandarin	zhui4; ao2
U+8D05.0	fEnglish	[1] useless; unnecessary; superfluous; redundant [2] repetition; repeat; repetitious [3] follow around (children) [4] be burdensome [5] pawn things for money [6] meet; congregate [7] a son-in-law who takes the place of a son inn his wife's parental family which is lacking for an heir
U+8D06.0	fUTF8	贆
U+8D06.0	fCNS11643	2-6734
U+8D06.0	fBig5	F348
U+8D06.0	fGB	E-3E72
U+8D06.0	fR/S	 154.12
U+8D06.0	fTotalStrokes	 19
U+8D06.0	fCangjie	BCIKK
U+8D06.0	fFourCorner	63889
U+8D06.0	fCantonese	biu1
U+8D06.0	fMandarin	biao1
U+8D07.0	fUTF8	贇
U+8D07.0	fCNS11643	2-6735
U+8D07.0	fBig5	F349
U+8D07.0	fGB	3-485A
U+8D07.0	fR/S	 154.12; 154.11
U+8D07.0	fTotalStrokes	 19
U+8D07.0	fCangjie	YMBUC
U+8D07.0	fFourCorner	03806
U+8D07.0	fCantonese	wan1
U+8D07.0	fMandarin	yun1
U+8D07.0	fEnglish	affable; agreeable; pleasant; fine
U+8D08.0	fUTF8	贈
U+8D08.0	fCNS11643	1-7925
U+8D08.0	fBig5	C3D8
U+8D08.0	fGB	1-5479
U+8D08.0	fR/S	 154.12; 154.11
U+8D08.0	fTotalStrokes	 19
U+8D08.0	fCangjie	BCCWA
U+8D08.0	fFourCorner	68866
U+8D08.0	fHakka	  zen5; zin5
U+8D08.0	fCantonese	zang6
U+8D08.0	fMandarin	zeng4
U+8D08.0	fEnglish	 send (gifts); give; bestow; confer
U+8D09.0	fUTF8	贉
U+8D09.0	fCNS11643	2-6736
U+8D09.0	fBig5	F34A
U+8D09.0	fGB	3-4859
U+8D09.0	fR/S	 154.12
U+8D09.0	fTotalStrokes	 19
U+8D09.0	fCangjie	BCMWJ
U+8D09.0	fFourCorner	61846
U+8D09.0	fCantonese	taam5; daam6
U+8D09.0	fMandarin	tan3; tan4; dan3; dan4
U+8D09.0	fEnglish	 earnest money silk label on boo
U+8D0A.0	fUTF8	贊
U+8D0A.0	fCNS11643	1-7926
U+8D0A.0	fBig5	C3D9
U+8D0A.0	fGB	1-545E
U+8D0A.0	fR/S	 154.12
U+8D0A.0	fTotalStrokes	 19
U+8D0A.0	fCangjie	HUBUC
U+8D0A.0	fFourCorner	24806
U+8D0A.0	fHakka	  zan5
U+8D0A.0	fCantonese	zaan3
U+8D0A.0	fMandarin	zan4
U+8D0A.0	fEnglish	[1] help; support; assist; aid; back [2] praise; commend; exalt; glorify; extol; eulogize
U+8D0B.0	fUTF8	贋
U+8D0B.0	fCNS11643	3-5B38
U+8D0B.0	fGB	1-584D
U+8D0B.0	fR/S	 154.12
U+8D0B.0	fTotalStrokes	 19
U+8D0B.0	fCangjie	MOGC
U+8D0B.0	fFourCorner	71280
U+8D0B.0	fHakka	 ngien5;  ngan5; ngan3; ngan6
U+8D0B.0	fCantonese	ngaan6; aan6 (also); ngan3 (also); an3 (also); ngan6
U+8D0B.0	fMandarin	yan4
U+8D0B.0	fEnglish	 counterfeit; sham; fake; bogus; spurious; forged
U+8D0C.0	fUTF8	贌
U+8D0C.0	fCNS11643	3-664F
U+8D0C.0	fGB	E-3E73
U+8D0C.0	fR/S	 154.12
U+8D0C.0	fTotalStrokes	 19
U+8D0C.0	fCangjie	BCTCO
U+8D0C.0	fFourCorner	6283
U+8D0C.0	fHakka	 p'iok8
U+8D0C.0	fCantonese	duk6
U+8D0D.0	fUTF8	贍
U+8D0D.0	fCNS11643	1-7A45
U+8D0D.0	fBig5	C4BA
U+8D0D.0	fGB	1-4944
U+8D0D.0	fR/S	 154.13
U+8D0D.0	fTotalStrokes	 20
U+8D0D.0	fCangjie	BCNCR
U+8D0D.0	fFourCorner	67861
U+8D0D.0	fHakka	 sam3;  sham3
U+8D0D.0	fCantonese	sim6 (also); sin6
U+8D0D.0	fMandarin	shan4; dan4
U+8D0D.0	fEnglish	[1] provide; supply [2] sufficient; abundance; adequate; plenty
U+8D0E.0	fUTF8	贎
U+8D0E.0	fCNS11643	4-644F
U+8D0E.0	fGB	3-483F
U+8D0E.0	fR/S	 154.13
U+8D0E.0	fTotalStrokes	 20
U+8D0E.0	fCangjie	BCTWB
U+8D0E.0	fFourCorner	64820
U+8D0E.0	fCantonese	maan6
U+8D0E.0	fMandarin	wan4
U+8D0F.0	fUTF8	贏
U+8D0F.0	fCNS11643	1-7A44
U+8D0F.0	fBig5	C4B9
U+8D0F.0	fGB	1-532E
U+8D0F.0	fR/S	 154.13
U+8D0F.0	fTotalStrokes	 20
U+8D0F.0	fCangjie	YNBUC
U+8D0F.0	fFourCorner	00217
U+8D0F.0	fHakka	  jang2; jin2; rang2; jang3; rin2; jin3
U+8D0F.0	fCantonese	jing4; jeng4 (also)
U+8D0F.0	fMandarin	ying2
U+8D0F.0	fEnglish	[1] win; beat [2] gain; profit
U+8D10.0	fUTF8	贐
U+8D10.0	fCNS11643	2-6C41
U+8D10.0	fBig5	F652
U+8D10.0	fGB	1-6A61
U+8D10.0	fR/S	 154.14
U+8D10.0	fTotalStrokes	 21
U+8D10.0	fCangjie	BCLMT
U+8D10.0	fFourCorner	65817
U+8D10.0	fHakka	 zin3; zin5
U+8D10.0	fCantonese	zeon2; zeon6
U+8D10.0	fMandarin	jin4; xin4
U+8D10.0	fEnglish	 farewell present
U+8D11.0	fUTF8	贑
U+8D11.0	fCNS11643	3-5E65
U+8D11.0	fGB	E-3E74
U+8D11.0	fR/S	 154.14
U+8D11.0	fTotalStrokes	 21
U+8D11.0	fCangjie	YJMBC
U+8D11.0	fFourCorner	01480
U+8D11.0	fCantonese	gam3
U+8D11.0	fMandarin	gan3; gan4; gong4
U+8D11.0	fEnglish	 Jiangxi province
U+8D12.0	fUTF8	贒
U+8D12.0	fCNS11643	3-5E66
U+8D12.0	fGB	E-3E75
U+8D12.0	fR/S	 154.14
U+8D12.0	fTotalStrokes	 21
U+8D12.0	fCangjie	SPBUC
U+8D12.0	fFourCorner	75800
U+8D12.0	fHakka	 hien2
U+8D12.0	fCantonese	jin4
U+8D12.0	fMandarin	xian2
U+8D12.0	fEnglish	wise; sagacious
U+8D13.0	fUTF8	贓
U+8D13.0	fCNS11643	1-7B2E
U+8D13.0	fBig5	C542
U+8D13.0	fGB	E-3E76
U+8D13.0	fR/S	 154.14
U+8D13.0	fTotalStrokes	 21
U+8D13.0	fCangjie	BCIMS
U+8D13.0	fFourCorner	63850
U+8D13.0	fHakka	  zong1; cong2
U+8D13.0	fCantonese	zong1
U+8D13.0	fMandarin	zang1
U+8D13.0	fEnglish	[1] booty; loot; stolen goods; plunder; spoils [2] bribe
U+8D14.0	fUTF8	贔
U+8D14.0	fCNS11643	2-6C42
U+8D14.0	fBig5	F653
U+8D14.0	fGB	3-484C
U+8D14.0	fR/S	 154.14
U+8D14.0	fTotalStrokes	 21
U+8D14.0	fCangjie	BCBCC
U+8D14.0	fFourCorner	60886
U+8D14.0	fHakka	  pi5; bi5; bi1
U+8D14.0	fCantonese	bei6 (also); bai3; pei6 (also)
U+8D14.0	fMandarin	bi4
U+8D14.0	fEnglish	strong
U+8D15.0	fUTF8	贕
U+8D15.0	fCNS11643	2-6E2B
U+8D15.0	fBig5	F75C
U+8D15.0	fGB	3-3033
U+8D15.0	fR/S	 154.15
U+8D15.0	fTotalStrokes	 22
U+8D15.0	fCangjie	HHSLC
U+8D15.0	fFourCorner	74286
U+8D15.0	fCantonese	duk6
U+8D15.0	fMandarin	du2
U+8D16.0	fUTF8	贖
U+8D16.0	fCNS11643	1-7B75
U+8D16.0	fBig5	C5AB
U+8D16.0	fGB	1-4A6A
U+8D16.0	fR/S	 154.15
U+8D16.0	fTotalStrokes	 22
U+8D16.0	fCangjie	BCGWC
U+8D16.0	fFourCorner	64886
U+8D16.0	fHakka	  shuk8;  suk8; siuk8
U+8D16.0	fCantonese	suk6; zuk6
U+8D16.0	fMandarin	shu2; shu4
U+8D16.0	fEnglish	[1] buy [2] redeem; ransome [3] atone for; expiate
U+8D17.0	fUTF8	贗
U+8D17.0	fCNS11643	1-7B76
U+8D17.0	fBig5	C5AC
U+8D17.0	fGB	E-3E77
U+8D17.0	fR/S	 154.15
U+8D17.0	fTotalStrokes	 22
U+8D17.0	fCangjie	MOFC
U+8D17.0	fFourCorner	71286
U+8D17.0	fHakka	 ngan5; ngien5; ngien6; ngan3; ngan6
U+8D17.0	fCantonese	ngaan6; ngan3 (also); aan6 (also); an3 (also)
U+8D17.0	fMandarin	yan4
U+8D17.0	fEnglish	counterfeit; bogus; false
U+8D18.0	fUTF8	贘
U+8D18.0	fCNS11643	3-6650
U+8D18.0	fGB	E-3E78
U+8D18.0	fR/S	 154.15
U+8D18.0	fTotalStrokes	 22
U+8D18.0	fCangjie	BCFBC
U+8D18.0	fFourCorner	6988
U+8D18.0	fCantonese	soeng2
U+8D19.0	fUTF8	贙
U+8D19.0	fCNS11643	2-6F53
U+8D19.0	fBig5	F845
U+8D19.0	fGB	3-485C
U+8D19.0	fR/S	 154.16
U+8D19.0	fTotalStrokes	 23
U+8D19.0	fCangjie	YUBUC
U+8D19.0	fFourCorner	21806
U+8D19.0	fCantonese	jyun5; jyun6
U+8D19.0	fMandarin	xuan4
U+8D1A.0	fUTF8	贚
U+8D1A.0	fCNS11643	3-604A
U+8D1A.0	fGB	3-4843
U+8D1A.0	fR/S	 154.16
U+8D1A.0	fTotalStrokes	 23
U+8D1A.0	fCangjie	BCYBP
U+8D1A.0	fFourCorner	61810
U+8D1A.0	fCantonese	lung6
U+8D1A.0	fMandarin	long4
U+8D1B.0	fUTF8	贛
U+8D1B.0	fCNS11643	1-7C6D
U+8D1B.0	fBig5	C642
U+8D1B.0	fGB	1-3853
U+8D1B.0	fR/S	 154.17
U+8D1B.0	fTotalStrokes	 24
U+8D1B.0	fCangjie	YJHEC
U+8D1B.0	fFourCorner	07486
U+8D1B.0	fHakka	 gung5;  gam5
U+8D1B.0	fCantonese	gam3; gung3
U+8D1B.0	fMandarin	gan4; gong4; zhuang4; gan3; hong4; xiang4
U+8D1B.0	fEnglish	[1] another name for Kiangsi province [2] a river in Kiangsi province [3] a county in Kiangsi province
U+8D1C.0	fUTF8	贜
U+8D1C.0	fCNS11643	3-614E
U+8D1C.0	fGB	1-545F
U+8D1C.0	fR/S	 154.18
U+8D1C.0	fTotalStrokes	 25
U+8D1C.0	fCangjie	BCTIS
U+8D1C.0	fFourCorner	64850
U+8D1C.0	fCantonese	zong1
U+8D1C.0	fMandarin	zang1
U+8D1C.0	fEnglish	 hide accept bribes booty
U+8D1D.0	fUTF8	贝
U+8D1D.0	fGB	0-3134
U+8D1D.0	fR/S	 154.0
U+8D1D.0	fTotalStrokes	 4
U+8D1D.0	fCangjie	BO
U+8D1D.0	fHakka	 bui5; bi5
U+8D1D.0	fCantonese	bui3
U+8D1D.0	fMandarin	bei4
U+8D1D.0	fEnglish	simplified form of &#x8C9D;
U+8D1E.0	fUTF8	贞
U+8D1E.0	fGB	0-556A
U+8D1E.0	fR/S	 25.4; 154.2
U+8D1E.0	fTotalStrokes	 6
U+8D1E.0	fCangjie	YBO
U+8D1E.0	fFourCorner	2180
U+8D1E.0	fHakka	 zhin1
U+8D1E.0	fCantonese	zing1
U+8D1E.0	fMandarin	zhen1; zheng1
U+8D1E.0	fEnglish	simplified form of &#x8C9E;
U+8D1F.0	fUTF8	负
U+8D1F.0	fGB	0-383A
U+8D1F.0	fR/S	 154.2
U+8D1F.0	fTotalStrokes	 6
U+8D1F.0	fCangjie	NBO
U+8D1F.0	fHakka	 fu5
U+8D1F.0	fCantonese	fu6
U+8D1F.0	fMandarin	fu4
U+8D1F.0	fEnglish	simplified form of &#x8CA0;
U+8D1F.1	fUTF8	负
U+8D1F.1	fGB	0-383A
U+8D1F.1	fR/S	 154.2
U+8D1F.1	fTotalStrokes	 6
U+8D1F.1	fCangjie	NBO
U+8D1F.1	fMandarin	fu5
U+8D20.0	fUTF8	贠
U+8D20.0	fGB	8-2E2C
U+8D20.0	fR/S	 154.2
U+8D20.0	fTotalStrokes	 6
U+8D20.0	fCangjie	IBO
U+8D20.0	fCantonese	jyun4; wan6
U+8D20.0	fMandarin	yuan2; yun2
U+8D20.0	fEnglish	member; personnel, staff member
U+8D21.0	fUTF8	贡
U+8D21.0	fGB	0-3931
U+8D21.0	fR/S	 154.3
U+8D21.0	fTotalStrokes	 7
U+8D21.0	fCangjie	XMBO
U+8D21.0	fHakka	 gung5
U+8D21.0	fCantonese	gung3
U+8D21.0	fMandarin	gong4
U+8D21.0	fEnglish	simplified form of &#x8CA2;
U+8D22.0	fUTF8	财
U+8D22.0	fGB	0-3246
U+8D22.0	fR/S	 154.3
U+8D22.0	fTotalStrokes	 7
U+8D22.0	fCangjie	BODH
U+8D22.0	fHakka	 coi2
U+8D22.0	fCantonese	coi4; coi2 (also)
U+8D22.0	fMandarin	cai2; zai4
U+8D22.0	fEnglish	simplified form of &#x8CA1;
U+8D23.0	fUTF8	责
U+8D23.0	fGB	0-5470
U+8D23.0	fR/S	 154.4
U+8D23.0	fTotalStrokes	 8
U+8D23.0	fCangjie	QMBO
U+8D23.0	fHakka	 zit7
U+8D23.0	fCantonese	zaak3; zaai3
U+8D23.0	fMandarin	ze2; zhai4
U+8D23.0	fEnglish	simplified form of &#x8CAC;
U+8D24.0	fUTF8	贤
U+8D24.0	fGB	0-4F4D
U+8D24.0	fR/S	 154.4
U+8D24.0	fTotalStrokes	 8
U+8D24.0	fCangjie	LEBO
U+8D24.0	fHakka	 hien2
U+8D24.0	fCantonese	jin4
U+8D24.0	fMandarin	xian2; xian4
U+8D24.0	fEnglish	simplified form of &#x8CE2;
U+8D25.0	fUTF8	败
U+8D25.0	fGB	0-305C
U+8D25.0	fR/S	 154.4
U+8D25.0	fTotalStrokes	 8
U+8D25.0	fCangjie	BOOK
U+8D25.0	fHakka	 pai5
U+8D25.0	fCantonese	baai6
U+8D25.0	fMandarin	bai4
U+8D25.0	fEnglish	simplified form of &#x6557;
U+8D26.0	fUTF8	账
U+8D26.0	fGB	0-554B
U+8D26.0	fR/S	 154.4
U+8D26.0	fTotalStrokes	 8
U+8D26.0	fCangjie	BOPO
U+8D26.0	fHakka	 zhong5
U+8D26.0	fCantonese	zoeng3; zan3
U+8D26.0	fMandarin	zhang4
U+8D26.0	fEnglish	simplified form of &#x8CEC;
U+8D27.0	fUTF8	货
U+8D27.0	fGB	0-3B75
U+8D27.0	fR/S	 154.4
U+8D27.0	fTotalStrokes	 8
U+8D27.0	fCangjie	OPBO
U+8D27.0	fHakka	 fo5
U+8D27.0	fCantonese	fo3
U+8D27.0	fMandarin	huo4
U+8D27.0	fEnglish	simplified form of &#x8CA8;
U+8D28.0	fUTF8	质
U+8D28.0	fGB	0-564A
U+8D28.0	fR/S	 27.4; 154.4
U+8D28.0	fTotalStrokes	 8
U+8D28.0	fCangjie	HJBO
U+8D28.0	fHakka	 zhit7; zhi5
U+8D28.0	fCantonese	zat1; zi3
U+8D28.0	fMandarin	zhi2; zhi4
U+8D28.0	fEnglish	simplified form of &#x8CEA;
U+8D29.0	fUTF8	贩
U+8D29.0	fGB	0-3737
U+8D29.0	fR/S	 154.4
U+8D29.0	fTotalStrokes	 8
U+8D29.0	fCangjie	BOHE
U+8D29.0	fHakka	 fan5; pan5
U+8D29.0	fCantonese	faan3; faan2 (also); faan5 (also)
U+8D29.0	fMandarin	fan4
U+8D29.0	fEnglish	simplified form of &#x8CA9;
U+8D2A.0	fUTF8	贪
U+8D2A.0	fGB	0-4C30
U+8D2A.0	fR/S	 154.4
U+8D2A.0	fTotalStrokes	 8
U+8D2A.0	fCangjie	XOINO
U+8D2A.0	fHakka	 tam1
U+8D2A.0	fCantonese	taam1
U+8D2A.0	fMandarin	tan1; tan4
U+8D2A.0	fEnglish	simplified form of &#x8CAA;
U+8D2B.0	fUTF8	贫
U+8D2B.0	fGB	0-4636
U+8D2B.0	fR/S	 154.4
U+8D2B.0	fTotalStrokes	 8
U+8D2B.0	fCangjie	CSHO
U+8D2B.0	fHakka	 pin2
U+8D2B.0	fCantonese	pan4
U+8D2B.0	fMandarin	pin2
U+8D2B.0	fEnglish	simplified form of &#x8CA7;
U+8D2C.0	fUTF8	贬
U+8D2C.0	fGB	0-3161
U+8D2C.0	fR/S	 154.5; 154.4
U+8D2C.0	fTotalStrokes	 8
U+8D2C.0	fCangjie	BOHIO
U+8D2C.0	fHakka	 bien3
U+8D2C.0	fCantonese	bin2
U+8D2C.0	fMandarin	bian3; ban3; fa2
U+8D2C.0	fEnglish	simplified form of &#x8CB6;
U+8D2D.0	fUTF8	购
U+8D2D.0	fGB	0-393A
U+8D2D.0	fR/S	 154.4
U+8D2D.0	fTotalStrokes	 8
U+8D2D.0	fCangjie	BOPI
U+8D2D.0	fHakka	 geu5
U+8D2D.0	fCantonese	gau3; kau3 (also)
U+8D2D.0	fMandarin	gou4; gou1
U+8D2D.0	fEnglish	simplified form of &#x8CFC;
U+8D2E.0	fUTF8	贮
U+8D2E.0	fGB	0-567C
U+8D2E.0	fR/S	 154.4
U+8D2E.0	fTotalStrokes	 8
U+8D2E.0	fCangjie	BOJM
U+8D2E.0	fHakka	 du3
U+8D2E.0	fCantonese	cyu5
U+8D2E.0	fMandarin	zhu3; zhu4
U+8D2E.0	fEnglish	simplified form of &#x8CAF;
U+8D2F.0	fUTF8	贯
U+8D2F.0	fGB	0-3961
U+8D2F.0	fR/S	 154.4
U+8D2F.0	fTotalStrokes	 8
U+8D2F.0	fCangjie	WJBO
U+8D2F.0	fHakka	 gwon5; kwan5
U+8D2F.0	fCantonese	gun3; gwun3 (also); gwaan3
U+8D2F.0	fMandarin	guan4; wan1; guan1
U+8D2F.0	fEnglish	simplified form of &#x8CAB;
U+8D30.0	fUTF8	贰
U+8D30.0	fGB	0-3721
U+8D30.0	fR/S	 56.6; 154.5
U+8D30.0	fTotalStrokes	 9
U+8D30.0	fCangjie	IPMMO
U+8D30.0	fHakka	 ngi5
U+8D30.0	fCantonese	ji6
U+8D30.0	fMandarin	er4
U+8D30.0	fEnglish	simplified form of &#x8CB3;
U+8D31.0	fUTF8	贱
U+8D31.0	fGB	0-3C7A
U+8D31.0	fR/S	 154.5
U+8D31.0	fTotalStrokes	 9
U+8D31.0	fCangjie	XBOIJ
U+8D31.0	fHakka	 cien5
U+8D31.0	fCantonese	zin6; daam5
U+8D31.0	fMandarin	jian4
U+8D31.0	fEnglish	simplified form of &#x8CE4;
U+8D32.0	fUTF8	贲
U+8D32.0	fGB	0-6A5A
U+8D32.0	fR/S	 154.5
U+8D32.0	fTotalStrokes	 9
U+8D32.0	fCangjie	JTBO
U+8D32.0	fHakka	 bun1; fun1; bi5
U+8D32.0	fCantonese	bei3; ban1; fan4; fan5
U+8D32.0	fMandarin	bi4; fei2; fen4; ben1; fen2
U+8D32.0	fEnglish	simplified form of &#x8CC1;
U+8D33.0	fUTF8	贳
U+8D33.0	fGB	0-6A5B
U+8D33.0	fR/S	 154.5
U+8D33.0	fTotalStrokes	 9
U+8D33.0	fCangjie	XPTBO
U+8D33.0	fHakka	 shi5; ji5
U+8D33.0	fCantonese	sai3
U+8D33.0	fMandarin	shi4; she4
U+8D33.0	fEnglish	simplified form of &#x8CB0;
U+8D34.0	fUTF8	贴
U+8D34.0	fGB	0-4C79
U+8D34.0	fR/S	 154.5
U+8D34.0	fTotalStrokes	 9
U+8D34.0	fCangjie	BOYR
U+8D34.0	fHakka	 dap7; tiap7; diap7
U+8D34.0	fCantonese	tip3; tip1 (also)
U+8D34.0	fMandarin	tie1
U+8D34.0	fEnglish	simplified form of &#x8CBC;
U+8D35.0	fUTF8	贵
U+8D35.0	fGB	0-3973
U+8D35.0	fR/S	 154.5
U+8D35.0	fTotalStrokes	 9
U+8D35.0	fCangjie	LMBO
U+8D35.0	fHakka	 kwui5
U+8D35.0	fCantonese	gwai3
U+8D35.0	fMandarin	gui4
U+8D35.0	fEnglish	simplified form of &#x8CB4;
U+8D36.0	fUTF8	贶
U+8D36.0	fGB	0-6A5C
U+8D36.0	fR/S	 154.5
U+8D36.0	fTotalStrokes	 9
U+8D36.0	fCangjie	BORHU
U+8D36.0	fHakka	 k'wong3
U+8D36.0	fCantonese	fong3
U+8D36.0	fMandarin	kuang4; huang4
U+8D36.0	fEnglish	simplified form of &#x8CBA;
U+8D37.0	fUTF8	贷
U+8D37.0	fGB	0-347B
U+8D37.0	fR/S	 154.5
U+8D37.0	fTotalStrokes	 9
U+8D37.0	fCangjie	XOPBO
U+8D37.0	fHakka	 toi5
U+8D37.0	fCantonese	taai3; tik1
U+8D37.0	fMandarin	dai4; te4; tai3
U+8D37.0	fEnglish	simplified form of &#x8CB8;
U+8D38.0	fUTF8	贸
U+8D38.0	fGB	0-4333
U+8D38.0	fR/S	 154.5
U+8D38.0	fTotalStrokes	 9
U+8D38.0	fCangjie	HHBO
U+8D38.0	fHakka	 meu3
U+8D38.0	fCantonese	mau6
U+8D38.0	fMandarin	mao4; mou4
U+8D38.0	fEnglish	simplified form of &#x8CBF;
U+8D39.0	fUTF8	费
U+8D39.0	fGB	0-3751
U+8D39.0	fR/S	 154.5
U+8D39.0	fTotalStrokes	 9
U+8D39.0	fCangjie	LNBO
U+8D39.0	fHakka	 fui5; bui5; bi5
U+8D39.0	fCantonese	fai3; bei3
U+8D39.0	fMandarin	fei4; bi4
U+8D39.0	fEnglish	simplified form of &#x8CBB;
U+8D3A.0	fUTF8	贺
U+8D3A.0	fGB	0-3A58
U+8D3A.0	fR/S	 154.5
U+8D3A.0	fTotalStrokes	 9
U+8D3A.0	fCangjie	KRBO
U+8D3A.0	fHakka	 fo5; ho5
U+8D3A.0	fCantonese	ho6
U+8D3A.0	fMandarin	he4
U+8D3A.0	fEnglish	simplified form of &#x8CC0;
U+8D3B.0	fUTF8	贻
U+8D3B.0	fGB	0-6A5D
U+8D3B.0	fR/S	 154.5
U+8D3B.0	fTotalStrokes	 9
U+8D3B.0	fCangjie	BOIR
U+8D3B.0	fHakka	 ji2
U+8D3B.0	fCantonese	ji4
U+8D3B.0	fMandarin	yi2; yi4
U+8D3B.0	fEnglish	simplified form of &#x8CBD;
U+8D3C.0	fUTF8	贼
U+8D3C.0	fGB	0-5474
U+8D3C.0	fR/S	 154.6
U+8D3C.0	fTotalStrokes	 10
U+8D3C.0	fCangjie	BOIJ
U+8D3C.0	fHakka	 cet8
U+8D3C.0	fCantonese	caak6 (also); caak3; caak2 (also)
U+8D3C.0	fMandarin	zei2; ze2
U+8D3C.0	fEnglish	simplified form of &#x8CCA;
U+8D3D.0	fUTF8	贽
U+8D3D.0	fGB	0-6A5E
U+8D3D.0	fR/S	 154.6
U+8D3D.0	fTotalStrokes	 10
U+8D3D.0	fCangjie	QIBO
U+8D3D.0	fHakka	 zhi5
U+8D3D.0	fCantonese	zi3
U+8D3D.0	fMandarin	zhi4; nie4; zhi2
U+8D3D.0	fEnglish	simplified form of &#x8D04;
U+8D3E.0	fUTF8	贾
U+8D3E.0	fGB	0-3C56
U+8D3E.0	fR/S	 154.6
U+8D3E.0	fTotalStrokes	 10
U+8D3E.0	fCangjie	MWBO
U+8D3E.0	fHakka	 gu3; ga5; ga3
U+8D3E.0	fCantonese	gaa2; gu2; gaa3; gwu2 (also)
U+8D3E.0	fMandarin	jia3; jia4; gu3
U+8D3E.0	fEnglish	simplified form of &#x8CC8;
U+8D3F.0	fUTF8	贿
U+8D3F.0	fGB	0-3B5F
U+8D3F.0	fR/S	 154.6
U+8D3F.0	fTotalStrokes	 10
U+8D3F.0	fCangjie	BOKB
U+8D3F.0	fHakka	 fui5; ju3
U+8D3F.0	fCantonese	kui2; fui2 (also); kwui2 (also)
U+8D3F.0	fMandarin	hui4; hui3
U+8D3F.0	fEnglish	simplified form of &#x8CC4;
U+8D40.0	fUTF8	赀
U+8D40.0	fGB	0-6A5F
U+8D40.0	fR/S	 154.6
U+8D40.0	fTotalStrokes	 10
U+8D40.0	fCangjie	YPBO
U+8D40.0	fHakka	 zih1
U+8D40.0	fCantonese	zi1
U+8D40.0	fMandarin	zi1
U+8D40.0	fEnglish	simplified form of &#x8CB2;
U+8D41.0	fUTF8	赁
U+8D41.0	fGB	0-415E
U+8D41.0	fR/S	 154.6
U+8D41.0	fTotalStrokes	 10
U+8D41.0	fCangjie	OGBO
U+8D41.0	fHakka	 ngim5; jim5
U+8D41.0	fCantonese	jam6
U+8D41.0	fMandarin	ren4; lin4
U+8D41.0	fEnglish	simplified form of &#x8CC3;
U+8D42.0	fUTF8	赂
U+8D42.0	fGB	0-4238
U+8D42.0	fR/S	 154.6
U+8D42.0	fTotalStrokes	 10
U+8D42.0	fCangjie	BOHER
U+8D42.0	fHakka	 lu5
U+8D42.0	fCantonese	lou6
U+8D42.0	fMandarin	lu4
U+8D42.0	fEnglish	simplified form of &#x8CC2;
U+8D43.0	fUTF8	赃
U+8D43.0	fGB	0-545F
U+8D43.0	fR/S	 154.6
U+8D43.0	fTotalStrokes	 10
U+8D43.0	fCangjie	BOIG
U+8D43.0	fCantonese	zong1; zong6
U+8D43.0	fMandarin	zang1
U+8D43.0	fEnglish	simplified form of &#x8D1C;
U+8D44.0	fUTF8	资
U+8D44.0	fGB	0-574A
U+8D44.0	fR/S	 154.6
U+8D44.0	fTotalStrokes	 10
U+8D44.0	fCangjie	IOBO
U+8D44.0	fHakka	 zih1
U+8D44.0	fCantonese	zi1
U+8D44.0	fMandarin	zi1; zi4
U+8D44.0	fEnglish	simplified form of &#x8CC7;
U+8D45.0	fUTF8	赅
U+8D45.0	fGB	0-6A60
U+8D45.0	fR/S	 154.6
U+8D45.0	fTotalStrokes	 10
U+8D45.0	fCangjie	BOYVO
U+8D45.0	fHakka	 goi1
U+8D45.0	fCantonese	goi1
U+8D45.0	fMandarin	gai1; hai4
U+8D45.0	fEnglish	simplified form of &#x8CC5;
U+8D46.0	fUTF8	赆
U+8D46.0	fGB	0-6A61
U+8D46.0	fR/S	 154.6
U+8D46.0	fTotalStrokes	 10
U+8D46.0	fCangjie	BOSOY
U+8D46.0	fHakka	 zin5
U+8D46.0	fCantonese	zeon2
U+8D46.0	fMandarin	jin4; xin4
U+8D46.0	fEnglish	simplified form of &#x8D10;
U+8D47.0	fUTF8	赇
U+8D47.0	fGB	0-6A64
U+8D47.0	fR/S	 154.7
U+8D47.0	fTotalStrokes	 11
U+8D47.0	fCangjie	BOIJE
U+8D47.0	fHakka	 kiu2
U+8D47.0	fCantonese	kau4
U+8D47.0	fMandarin	qiu2
U+8D47.0	fEnglish	simplified form of &#x8CD5;
U+8D48.0	fUTF8	赈
U+8D48.0	fGB	0-6A62
U+8D48.0	fR/S	 154.7
U+8D48.0	fTotalStrokes	 11
U+8D48.0	fCangjie	BOMMV
U+8D48.0	fHakka	 zhin3
U+8D48.0	fCantonese	zan3
U+8D48.0	fMandarin	zhen4; zhen3
U+8D48.0	fEnglish	simplified form of &#x8CD1;
U+8D49.0	fUTF8	赉
U+8D49.0	fGB	0-6A63
U+8D49.0	fR/S	 154.7
U+8D49.0	fTotalStrokes	 11
U+8D49.0	fCangjie	XDTBO
U+8D49.0	fHakka	 loi3
U+8D49.0	fCantonese	loi6
U+8D49.0	fMandarin	lai4; lai2
U+8D49.0	fEnglish	simplified form of &#x8CDA;
U+8D4A.0	fUTF8	赊
U+8D4A.0	fGB	0-495E
U+8D4A.0	fR/S	 154.7
U+8D4A.0	fTotalStrokes	 11
U+8D4A.0	fCangjie	BOOMF
U+8D4A.0	fHakka	 cha1
U+8D4A.0	fCantonese	se1
U+8D4A.0	fMandarin	she1; sha1
U+8D4A.0	fEnglish	simplified form of &#x8CD2;
U+8D4B.0	fUTF8	赋
U+8D4B.0	fGB	0-3833
U+8D4B.0	fR/S	 154.8
U+8D4B.0	fTotalStrokes	 12
U+8D4B.0	fCangjie	BOMPM
U+8D4B.0	fHakka	 fu5
U+8D4B.0	fCantonese	fu3
U+8D4B.0	fMandarin	fu4
U+8D4B.0	fEnglish	simplified form of &#x8CE6;
U+8D4C.0	fUTF8	赌
U+8D4C.0	fGB	0-3644
U+8D4C.0	fR/S	 154.8
U+8D4C.0	fTotalStrokes	 12
U+8D4C.0	fCangjie	BOJKA
U+8D4C.0	fHakka	 du3
U+8D4C.0	fCantonese	dou2
U+8D4C.0	fMandarin	du3
U+8D4C.0	fEnglish	simplified form of &#x8CED;
U+8D4D.0	fUTF8	赍
U+8D4D.0	fGB	0-6A65
U+8D4D.0	fR/S	 154.8
U+8D4D.0	fTotalStrokes	 12
U+8D4D.0	fCangjie	JBBO
U+8D4D.0	fHakka	 zi1; zih1
U+8D4D.0	fCantonese	zai1; cai4
U+8D4D.0	fMandarin	ji1; qi2
U+8D4D.0	fEnglish	simplified form of &#x8CEB;
U+8D4E.0	fUTF8	赎
U+8D4E.0	fGB	0-4A6A
U+8D4E.0	fR/S	 154.8
U+8D4E.0	fTotalStrokes	 12
U+8D4E.0	fCangjie	BOJNK
U+8D4E.0	fHakka	 shuk8
U+8D4E.0	fCantonese	suk6; zuk6
U+8D4E.0	fMandarin	shu2; shu4
U+8D4E.0	fEnglish	simplified form of &#x8D16;
U+8D4F.0	fUTF8	赏
U+8D4F.0	fGB	0-494D
U+8D4F.0	fR/S	 154.8
U+8D4F.0	fTotalStrokes	 12
U+8D4F.0	fCangjie	FBRBO
U+8D4F.0	fHakka	 shong3
U+8D4F.0	fCantonese	soeng2
U+8D4F.0	fMandarin	shang3
U+8D4F.0	fEnglish	simplified form of &#x8CDE;
U+8D50.0	fUTF8	赐
U+8D50.0	fGB	0-344D
U+8D50.0	fR/S	 154.8
U+8D50.0	fTotalStrokes	 12
U+8D50.0	fCangjie	BOAPH
U+8D50.0	fHakka	 sii5
U+8D50.0	fCantonese	ci3
U+8D50.0	fMandarin	si4; ci4
U+8D50.0	fEnglish	simplified form of &#x8CDC;
U+8D51.0	fUTF8	赑
U+8D51.0	fGB	8-7C42
U+8D51.0	fR/S	 154.14; 154.8
U+8D51.0	fTotalStrokes	 12
U+8D51.0	fCangjie	BOBOO
U+8D51.0	fCantonese	bai3; pei6 (also); bei6 (also)
U+8D51.0	fMandarin	bi4
U+8D51.0	fEnglish	strong
U+8D52.0	fUTF8	赒
U+8D52.0	fGB	8-2E4E
U+8D52.0	fR/S	 154.8
U+8D52.0	fTotalStrokes	 12
U+8D52.0	fCangjie	BOBGR
U+8D52.0	fCantonese	zau1
U+8D52.0	fMandarin	zhou1
U+8D52.0	fEnglish	give for charity
U+8D53.0	fUTF8	赓
U+8D53.0	fGB	0-6259
U+8D53.0	fR/S	 53.9; 154.8
U+8D53.0	fTotalStrokes	 12
U+8D53.0	fCangjie	ILOO
U+8D53.0	fHakka	 gang1
U+8D53.0	fCantonese	gang1
U+8D53.0	fMandarin	geng1; geng4
U+8D53.0	fEnglish	simplified form of &#x8CE1;
U+8D54.0	fUTF8	赔
U+8D54.0	fGB	0-4562
U+8D54.0	fR/S	 154.8
U+8D54.0	fTotalStrokes	 12
U+8D54.0	fCangjie	BOYTR
U+8D54.0	fHakka	 poi2
U+8D54.0	fCantonese	pui4
U+8D54.0	fMandarin	pei2
U+8D54.0	fEnglish	simplified form of &#x8CE0;
U+8D55.0	fUTF8	赕
U+8D55.0	fGB	0-6A66
U+8D55.0	fR/S	 154.8
U+8D55.0	fTotalStrokes	 12
U+8D55.0	fCangjie	BOFF
U+8D55.0	fHakka	 tam5
U+8D55.0	fCantonese	daam6; zin6
U+8D55.0	fMandarin	tan4; dan4; tan3
U+8D55.0	fEnglish	simplified form of &#x8CE7;
U+8D56.0	fUTF8	赖
U+8D56.0	fGB	0-4035
U+8D56.0	fR/S	 154.9
U+8D56.0	fTotalStrokes	 13
U+8D56.0	fCangjie	DLNBO
U+8D56.0	fHakka	 lai5
U+8D56.0	fCantonese	laai6; laai2 (also); laai3 (also)
U+8D56.0	fMandarin	lai4
U+8D56.0	fEnglish	simplified form of &#x8CF4;
U+8D57.0	fUTF8	赗
U+8D57.0	fGB	8-7C43
U+8D57.0	fR/S	 154.9
U+8D57.0	fTotalStrokes	 13
U+8D57.0	fCangjie	BOABU
U+8D57.0	fCantonese	fung3; fung6 (also)
U+8D57.0	fMandarin	feng4
U+8D57.0	fEnglish	gift
U+8D58.0	fUTF8	赘
U+8D58.0	fGB	0-5738
U+8D58.0	fR/S	 154.10
U+8D58.0	fTotalStrokes	 14
U+8D58.0	fCangjie	QKBO
U+8D58.0	fHakka	 chui5
U+8D58.0	fCantonese	zeoi6; zeoi3
U+8D58.0	fMandarin	zhui4; ao2
U+8D58.0	fEnglish	simplified form of &#x8D05;
U+8D59.0	fUTF8	赙
U+8D59.0	fGB	0-6A67
U+8D59.0	fR/S	 154.10
U+8D59.0	fTotalStrokes	 14
U+8D59.0	fCangjie	BOIBI
U+8D59.0	fHakka	 fu5
U+8D59.0	fCantonese	fu6
U+8D59.0	fMandarin	fu4
U+8D59.0	fEnglish	simplified form of &#x8CFB;
U+8D5A.0	fUTF8	赚
U+8D5A.0	fGB	0-572C
U+8D5A.0	fR/S	 154.10
U+8D5A.0	fTotalStrokes	 14
U+8D5A.0	fCangjie	BOTXC
U+8D5A.0	fHakka	 can5
U+8D5A.0	fCantonese	zaan6; zaan2 (also); zaam6
U+8D5A.0	fMandarin	zhuan4; zuan4; lian2; zhan4
U+8D5A.0	fEnglish	simplified form of &#x8CFA;
U+8D5B.0	fUTF8	赛
U+8D5B.0	fGB	0-487C
U+8D5B.0	fR/S	 40.11; 154.10
U+8D5B.0	fTotalStrokes	 14
U+8D5B.0	fCangjie	JTCO
U+8D5B.0	fHakka	 soi5
U+8D5B.0	fCantonese	coi3
U+8D5B.0	fMandarin	sai4
U+8D5B.0	fEnglish	simplified form of &#x8CFD;
U+8D5C.0	fUTF8	赜
U+8D5C.0	fGB	0-5853
U+8D5C.0	fR/S	 154.11; 22.13
U+8D5C.0	fTotalStrokes	 18
U+8D5C.0	fCangjie	SLQMO
U+8D5C.0	fHakka	 zit7
U+8D5C.0	fCantonese	zaak3; zak3
U+8D5C.0	fMandarin	ze2; ze4
U+8D5C.0	fEnglish	simplified form of &#x8CFE;
U+8D5D.0	fUTF8	赝
U+8D5D.0	fGB	0-584D
U+8D5D.0	fR/S	 27.14; 154.12
U+8D5D.0	fTotalStrokes	 19
U+8D5D.0	fCangjie	MOGO
U+8D5D.0	fHakka	 ngien5; ngan5
U+8D5D.0	fCantonese	ngaan6; aan6 (also); ngan3 (also); an3 (also); ngan6
U+8D5D.0	fMandarin	yan4
U+8D5D.0	fEnglish	simplified form of &#x8D0B;
U+8D5E.0	fUTF8	赞
U+8D5E.0	fGB	0-545E
U+8D5E.0	fR/S	 154.12
U+8D5E.0	fTotalStrokes	 16
U+8D5E.0	fCangjie	HUBO
U+8D5E.0	fHakka	 zan5
U+8D5E.0	fCantonese	zaan3; zaan2 (also)
U+8D5E.0	fMandarin	zan4
U+8D5E.0	fEnglish	simplified form of &#x8D0A;
U+8D5F.0	fUTF8	赟
U+8D5F.0	fGB	8-2C55
U+8D5F.0	fR/S	 154.12
U+8D5F.0	fTotalStrokes	 16
U+8D5F.0	fCangjie	YMBO
U+8D5F.0	fCantonese	wan1
U+8D5F.0	fMandarin	yun1
U+8D5F.0	fEnglish	affable, agreeable, pleasant
U+8D60.0	fUTF8	赠
U+8D60.0	fGB	0-5479
U+8D60.0	fR/S	 154.12
U+8D60.0	fTotalStrokes	 16
U+8D60.0	fCangjie	BOCWA
U+8D60.0	fHakka	 zen5
U+8D60.0	fCantonese	zang6
U+8D60.0	fMandarin	zeng4
U+8D60.0	fEnglish	simplified form of &#x8D08;
U+8D61.0	fUTF8	赡
U+8D61.0	fGB	0-4944
U+8D61.0	fR/S	 154.13
U+8D61.0	fTotalStrokes	 17
U+8D61.0	fCangjie	BONCR
U+8D61.0	fHakka	 sham3
U+8D61.0	fCantonese	sim6 (also); sin6
U+8D61.0	fMandarin	shan4; dan4
U+8D61.0	fEnglish	simplified form of &#x8D0D;
U+8D62.0	fUTF8	赢
U+8D62.0	fGB	0-532E
U+8D62.0	fR/S	 8.15; 154.13
U+8D62.0	fTotalStrokes	 17
U+8D62.0	fCangjie	YNBO
U+8D62.0	fHakka	 jang2; jin2
U+8D62.0	fCantonese	jing4; jeng4 (also)
U+8D62.0	fMandarin	ying2
U+8D62.0	fEnglish	simplified form of &#x8D0F;
U+8D63.0	fUTF8	赣
U+8D63.0	fGB	0-3853
U+8D63.0	fR/S	 154.17
U+8D63.0	fTotalStrokes	 21
U+8D63.0	fCangjie	YJHEO
U+8D63.0	fHakka	 gam5; gung5
U+8D63.0	fCantonese	gam3; gung3
U+8D63.0	fMandarin	gan4; gong4; gan3; hong4; xiang4; zhuang4
U+8D63.0	fEnglish	simplified form of &#x8D1B;
U+8D64.0	fUTF8	赤
U+8D64.0	fCNS11643	1-4B4B
U+8D64.0	fBig5	A8AA
U+8D64.0	fGB	0-3360
U+8D64.0	fR/S	 155.0
U+8D64.0	fTotalStrokes	 7
U+8D64.0	fCangjie	GLNC
U+8D64.0	fFourCorner	40331
U+8D64.0	fHakka	  cak7;  ch'ak7; chak7
U+8D64.0	fCantonese	cek3; cik3 (also)
U+8D64.0	fMandarin	chi4
U+8D64.0	fEnglish	[1] red [2] bare; naked [3] sincere; loyal; single-hearted
U+8D65.0	fUTF8	赥
U+8D65.0	fCNS11643	3-3933
U+8D65.0	fGB	5-6D6D
U+8D65.0	fR/S	 155.4
U+8D65.0	fTotalStrokes	 11
U+8D65.0	fCangjie	GCNO
U+8D65.0	fFourCorner	47380
U+8D65.0	fCantonese	hik1
U+8D65.0	fMandarin	xi4; xi1
U+8D66.0	fUTF8	赦
U+8D66.0	fCNS11643	1-5D50
U+8D66.0	fBig5	B36A
U+8D66.0	fGB	0-4962
U+8D66.0	fR/S	 155.4
U+8D66.0	fTotalStrokes	 11
U+8D66.0	fCangjie	GCOK
U+8D66.0	fFourCorner	48340
U+8D66.0	fHakka	  sa5; sha5
U+8D66.0	fCantonese	se3
U+8D66.0	fMandarin	she4; ce4
U+8D66.0	fEnglish	 forgive; remit; pardon; excuse; amnesty
U+8D67.0	fUTF8	赧
U+8D67.0	fCNS11643	1-5D4F
U+8D67.0	fBig5	B369
U+8D67.0	fGB	0-7476
U+8D67.0	fR/S	 155.5
U+8D67.0	fTotalStrokes	 12
U+8D67.0	fCangjie	GCSLE
U+8D67.0	fFourCorner	47347
U+8D67.0	fHakka	  nan3; lan3
U+8D67.0	fCantonese	naan5; laan5 (also); naan2
U+8D67.0	fMandarin	nan3
U+8D67.0	fEnglish	 blush; turn red from shame or embarrassment
U+8D68.0	fUTF8	赨
U+8D68.0	fCNS11643	2-4762
U+8D68.0	fBig5	E05C
U+8D68.0	fGB	3-6E3B
U+8D68.0	fR/S	 155.6
U+8D68.0	fTotalStrokes	 13
U+8D68.0	fCangjie	GCLMI
U+8D68.0	fFourCorner	45236
U+8D68.0	fCantonese	tung4
U+8D68.0	fMandarin	xiong2; tong2; rong2
U+8D69.0	fUTF8	赩
U+8D69.0	fCNS11643	2-4763
U+8D69.0	fBig5	E05D
U+8D69.0	fGB	3-6E3C
U+8D69.0	fR/S	 155.6
U+8D69.0	fTotalStrokes	 13
U+8D69.0	fCangjie	GCNAU
U+8D69.0	fFourCorner	47317
U+8D69.0	fCantonese	sik1; hik1; haak3
U+8D69.0	fMandarin	xi4; he4
U+8D6A.0	fUTF8	赪
U+8D6A.0	fGB	8-7E5A
U+8D6A.0	fR/S	 155.6
U+8D6A.0	fTotalStrokes	 13
U+8D6A.0	fCangjie	GCYBO
U+8D6A.0	fCantonese	cing1
U+8D6A.0	fMandarin	cheng1
U+8D6A.0	fEnglish	deep red
U+8D6B.0	fUTF8	赫
U+8D6B.0	fCNS11643	1-6B36
U+8D6B.0	fBig5	BBAE
U+8D6B.0	fGB	0-3A55
U+8D6B.0	fR/S	 155.7
U+8D6B.0	fTotalStrokes	 14
U+8D6B.0	fCangjie	GCGLC
U+8D6B.0	fFourCorner	44331
U+8D6B.0	fHakka	 het7; hak7; het8
U+8D6B.0	fCantonese	haak1; hak1 (also)
U+8D6B.0	fMandarin	he4; shi4; xi4; xia4
U+8D6B.0	fEnglish	[1] bright; radiant; glowing [2] brilliant; glorious [3] angry; indignant [4] a Chinese family name
U+8D6C.0	fUTF8	赬
U+8D6C.0	fCNS11643	2-5A65
U+8D6C.0	fBig5	EBB9
U+8D6C.0	fGB	3-6E3A
U+8D6C.0	fR/S	 155.9
U+8D6C.0	fTotalStrokes	 16
U+8D6C.0	fCangjie	GCYBC
U+8D6C.0	fFourCorner	41286
U+8D6C.0	fHakka	 shin5; shin1; siin1; siin5
U+8D6C.0	fCantonese	cing1; zing1
U+8D6C.0	fMandarin	cheng1
U+8D6C.0	fEnglish	red
U+8D6D.0	fUTF8	赭
U+8D6D.0	fCNS11643	1-6F35
U+8D6D.0	fBig5	BDEA
U+8D6D.0	fGB	0-7477
U+8D6D.0	fR/S	 155.9
U+8D6D.0	fTotalStrokes	 16
U+8D6D.0	fCangjie	GCJKA
U+8D6D.0	fFourCorner	44360
U+8D6D.0	fHakka	 zha3; za3; chii3
U+8D6D.0	fCantonese	ze2
U+8D6D.0	fMandarin	zhe3
U+8D6D.0	fEnglish	[1] red [2] ochre
U+8D6E.0	fUTF8	赮
U+8D6E.0	fCNS11643	2-5A66
U+8D6E.0	fBig5	EBBA
U+8D6E.0	fGB	5-6D71
U+8D6E.0	fR/S	 155.9
U+8D6E.0	fTotalStrokes	 16
U+8D6E.0	fCangjie	GCRSE
U+8D6E.0	fFourCorner	47347
U+8D6E.0	fCantonese	haa4
U+8D6E.0	fMandarin	xia2
U+8D6F.0	fUTF8	赯
U+8D6F.0	fCNS11643	2-5F65
U+8D6F.0	fBig5	EEB9
U+8D6F.0	fGB	3-6E3E
U+8D6F.0	fR/S	 155.10
U+8D6F.0	fTotalStrokes	 17
U+8D6F.0	fCangjie	GCILR
U+8D6F.0	fFourCorner	40267
U+8D6F.0	fCantonese	tong4
U+8D6F.0	fMandarin	tang2
U+8D70.0	fUTF8	走
U+8D70.0	fCNS11643	1-4B4C
U+8D70.0	fBig5	A8AB
U+8D70.0	fGB	0-575F
U+8D70.0	fR/S	 156.0
U+8D70.0	fTotalStrokes	 7
U+8D70.0	fCangjie	GYO
U+8D70.0	fFourCorner	40801
U+8D70.0	fHakka	  zeu3; tsieu3; ziu3
U+8D70.0	fCantonese	zau2
U+8D70.0	fMandarin	zou3; zou4
U+8D70.0	fEnglish	[1] walk; go on foot [2] run; go swiftly [3] go; travel [4] leave; go away; depart [5] let out; let lose (unintentionally); leak out [6] visit
U+8D71.0	fUTF8	赱
U+8D71.0	fCNS11643	3-243A
U+8D71.0	fGB	E-3E79
U+8D71.0	fR/S	 156.0
U+8D71.0	fTotalStrokes	 6; 7
U+8D71.0	fCangjie	GCO
U+8D71.0	fFourCorner	40300
U+8D71.0	fCantonese	zau2
U+8D71.0	fMandarin	zou3
U+8D71.0	fEnglish	 walk; go on foot run leave
U+8D72.0	fUTF8	赲
U+8D72.0	fCNS11643	2-2D53
U+8D72.0	fBig5	D0B2
U+8D72.0	fGB	3-6D6E
U+8D72.0	fR/S	 156.2
U+8D72.0	fTotalStrokes	 9
U+8D72.0	fCangjie	GOKS
U+8D72.0	fFourCorner	44802
U+8D72.0	fCantonese	lik6
U+8D72.0	fMandarin	li4
U+8D73.0	fUTF8	赳
U+8D73.0	fCNS11643	1-5359
U+8D73.0	fBig5	AD76
U+8D73.0	fGB	0-7471
U+8D73.0	fR/S	 156.2
U+8D73.0	fTotalStrokes	 9
U+8D73.0	fCangjie	GOVL
U+8D73.0	fFourCorner	42800
U+8D73.0	fHakka	 giu5; giu1; giu3; giu2
U+8D73.0	fCantonese	gau2; dau2 (also)
U+8D73.0	fMandarin	jiu1; jiu3; jiu4
U+8D73.0	fEnglish	gallant; valiant
U+8D74.0	fUTF8	赴
U+8D74.0	fCNS11643	1-5358
U+8D74.0	fBig5	AD75
U+8D74.0	fGB	0-3830
U+8D74.0	fR/S	 156.2
U+8D74.0	fTotalStrokes	 9
U+8D74.0	fCangjie	GOY
U+8D74.0	fFourCorner	43800
U+8D74.0	fHakka	  fu5; fu6
U+8D74.0	fCantonese	fu6
U+8D74.0	fMandarin	fu4
U+8D74.0	fEnglish	 go to; proceed to
U+8D75.0	fUTF8	赵
U+8D75.0	fGB	0-5554
U+8D75.0	fR/S	 156.2
U+8D75.0	fTotalStrokes	 9
U+8D75.0	fCangjie	GOK
U+8D75.0	fHakka	 chau5
U+8D75.0	fCantonese	ziu6
U+8D75.0	fMandarin	zhao4; tiao3; qiao3
U+8D75.0	fEnglish	simplified form of &#x8D99;
U+8D76.0	fUTF8	赶
U+8D76.0	fCNS11643	2-3327
U+8D76.0	fBig5	D3E3
U+8D76.0	fGB	0-384F
U+8D76.0	fR/S	 156.3
U+8D76.0	fTotalStrokes	 10
U+8D76.0	fCangjie	GOMJ
U+8D76.0	fFourCorner	42804
U+8D76.0	fHakka	 gon3
U+8D76.0	fCantonese	gon2
U+8D76.0	fMandarin	gan3; qian2
U+8D76.0	fEnglish	simplified form of &#x8D95;
U+8D77.0	fUTF8	起
U+8D77.0	fCNS11643	1-5844
U+8D77.0	fBig5	B05F
U+8D77.0	fGB	0-4670
U+8D77.0	fR/S	 156.3
U+8D77.0	fTotalStrokes	 10
U+8D77.0	fCangjie	GORU
U+8D77.0	fFourCorner	47801
U+8D77.0	fHakka	  hi3; ki3
U+8D77.0	fCantonese	hei2
U+8D77.0	fMandarin	qi3
U+8D77.0	fEnglish	[1] begin; start [2] rise; stand up; go up; get up [3] happen; take place [4] unfold; uncover [5] build; establish [6] a numerical classifier for incidents
U+8D78.0	fUTF8	赸
U+8D78.0	fCNS11643	2-3328
U+8D78.0	fBig5	D3E4
U+8D78.0	fGB	3-6D70
U+8D78.0	fR/S	 156.3
U+8D78.0	fTotalStrokes	 10
U+8D78.0	fCangjie	GOU
U+8D78.0	fFourCorner	42807
U+8D78.0	fCantonese	saan3
U+8D78.0	fMandarin	shan4
U+8D78.0	fEnglish	 try to conceal embarrassment
U+8D79.0	fUTF8	赹
U+8D79.0	fCNS11643	2-3958
U+8D79.0	fBig5	D7D5
U+8D79.0	fGB	5-6D37
U+8D79.0	fR/S	 156.4
U+8D79.0	fTotalStrokes	 11
U+8D79.0	fCangjie	GOPMM
U+8D79.0	fFourCorner	47802
U+8D79.0	fCantonese	king4
U+8D79.0	fMandarin	qiong2; qi2
U+8D7A.0	fUTF8	赺
U+8D7A.0	fCNS11643	3-3939
U+8D7A.0	fGB	3-6D76
U+8D7A.0	fR/S	 156.4
U+8D7A.0	fTotalStrokes	 11
U+8D7A.0	fCangjie	GOOIN
U+8D7A.0	fFourCorner	48800
U+8D7A.0	fCantonese	jam5
U+8D7A.0	fMandarin	qin2
U+8D7B.0	fUTF8	赻
U+8D7B.0	fCNS11643	2-3957
U+8D7B.0	fBig5	D7D4
U+8D7B.0	fGB	E-3E7A
U+8D7B.0	fR/S	 156.4
U+8D7B.0	fTotalStrokes	 11
U+8D7B.0	fCangjie	GOFH
U+8D7B.0	fFourCorner	49802
U+8D7B.0	fCantonese	sin2
U+8D7B.0	fMandarin	xian3; xian1
U+8D7C.0	fUTF8	赼
U+8D7C.0	fCNS11643	3-3937
U+8D7C.0	fGB	E-3E7B
U+8D7C.0	fR/S	 156.4
U+8D7C.0	fTotalStrokes	 11
U+8D7C.0	fCangjie	GONO
U+8D7C.0	fFourCorner	47800
U+8D7C.0	fCantonese	zi1
U+8D7C.0	fMandarin	ci1
U+8D7D.0	fUTF8	赽
U+8D7D.0	fCNS11643	2-3956
U+8D7D.0	fBig5	D7D3
U+8D7D.0	fGB	5-6D39
U+8D7D.0	fR/S	 156.4
U+8D7D.0	fTotalStrokes	 11
U+8D7D.0	fCangjie	GODK
U+8D7D.0	fFourCorner	45803
U+8D7D.0	fCantonese	kyut3; gwai3
U+8D7D.0	fMandarin	jue2; gui4
U+8D7E.0	fUTF8	赾
U+8D7E.0	fCNS11643	3-3935
U+8D7E.0	fGB	3-6D75
U+8D7E.0	fR/S	 156.4
U+8D7E.0	fTotalStrokes	 11
U+8D7E.0	fCangjie	GOHML
U+8D7E.0	fFourCorner	42800
U+8D7E.0	fCantonese	jan2
U+8D7E.0	fMandarin	qin3
U+8D7F.0	fUTF8	赿
U+8D7F.0	fCNS11643	3-3938
U+8D7F.0	fGB	E-3E7C
U+8D7F.0	fR/S	 156.4
U+8D7F.0	fTotalStrokes	 11
U+8D7F.0	fCangjie	GOHVP
U+8D7F.0	fFourCorner	42800
U+8D7F.0	fCantonese	ci4
U+8D7F.0	fMandarin	chi2; di4
U+8D7F.0	fEnglish	 late slow dilatory tardy to delay dull stupid
U+8D80.0	fUTF8	趀
U+8D80.0	fCNS11643	2-4054
U+8D80.0	fBig5	DBEE
U+8D80.0	fGB	5-6D35
U+8D80.0	fR/S	 156.5
U+8D80.0	fTotalStrokes	 12
U+8D80.0	fCangjie	GOLXH
U+8D80.0	fFourCorner	45802
U+8D80.0	fCantonese	zi1; ci1
U+8D80.0	fMandarin	ci1
U+8D81.0	fUTF8	趁
U+8D81.0	fCNS11643	1-623F
U+8D81.0	fBig5	B658
U+8D81.0	fGB	0-3343
U+8D81.0	fR/S	 156.5
U+8D81.0	fTotalStrokes	 12
U+8D81.0	fCangjie	GOOHH
U+8D81.0	fFourCorner	48802
U+8D81.0	fHakka	 chin5; zhin3; cin3; zihn3; cin5; chin3; cihn3; cihn5; cen5
U+8D81.0	fCantonese	can3
U+8D81.0	fMandarin	chen4; chen2; zhen1; nian3; zhen3
U+8D81.0	fEnglish	[1] take advantage of; avail oneself to [2] while
U+8D82.0	fUTF8	趂
U+8D82.0	fCNS11643	3-3F39
U+8D82.0	fGB	E-3E7D
U+8D82.0	fR/S	 156.5
U+8D82.0	fTotalStrokes	 12
U+8D82.0	fCangjie	GONF
U+8D82.0	fFourCorner	47800
U+8D82.0	fHakka	 chin5; zhin3; zihn3; chin3; cihn3; cihn5
U+8D82.0	fCantonese	can3; ji5
U+8D82.0	fMandarin	chen4
U+8D82.0	fEnglish	 take advantage of avail oneself of; follow go
U+8D83.0	fUTF8	趃
U+8D83.0	fCNS11643	3-3F34
U+8D83.0	fGB	5-6D3C
U+8D83.0	fR/S	  156.5
U+8D83.0	fTotalStrokes	 12
U+8D83.0	fCangjie	GOHQO
U+8D83.0	fFourCorner	45800
U+8D83.0	fCantonese	dit6
U+8D83.0	fMandarin	die2
U+8D84.0	fUTF8	趄
U+8D84.0	fCNS11643	2-4053
U+8D84.0	fBig5	DBED
U+8D84.0	fGB	0-7472
U+8D84.0	fR/S	 156.5
U+8D84.0	fTotalStrokes	 12
U+8D84.0	fCangjie	GOBM
U+8D84.0	fFourCorner	47801
U+8D84.0	fHakka	 zu1; zii1; zih1
U+8D84.0	fCantonese	zeoi1; ce3; cit3 (also); ceoi1
U+8D84.0	fMandarin	ju1; qie4; qu1
U+8D84.0	fEnglish	used in combination for falter
U+8D85.0	fUTF8	超
U+8D85.0	fCNS11643	1-623E
U+8D85.0	fBig5	B657
U+8D85.0	fGB	0-332C
U+8D85.0	fR/S	 156.5
U+8D85.0	fTotalStrokes	 12
U+8D85.0	fCangjie	GOSHR
U+8D85.0	fFourCorner	47806
U+8D85.0	fHakka	  chau1;  cau1; ceu1; ch'eu1
U+8D85.0	fCantonese	ciu1
U+8D85.0	fMandarin	chao1; chao3; tiao4; chao4
U+8D85.0	fEnglish	[1] jump over; leap over; fly across [2] be more than; exceed [3] be better than; excel; surpass [4] rise above; transcend [5] overtake
U+8D86.0	fUTF8	趆
U+8D86.0	fCNS11643	3-3F37
U+8D86.0	fGB	3-6D78
U+8D86.0	fR/S	  156.5
U+8D86.0	fTotalStrokes	 12
U+8D86.0	fCangjie	GOHPM
U+8D86.0	fFourCorner	42800
U+8D86.0	fCantonese	dai1
U+8D86.0	fMandarin	di1
U+8D87.0	fUTF8	趇
U+8D87.0	fCNS11643	3-3F30
U+8D87.0	fGB	E-3E7E
U+8D87.0	fR/S	  156.5
U+8D87.0	fTotalStrokes	 12
U+8D87.0	fCangjie	GOYT
U+8D87.0	fFourCorner	40800
U+8D87.0	fCantonese	sap1
U+8D87.0	fMandarin	se4
U+8D88.0	fUTF8	趈
U+8D88.0	fCNS11643	3-3F33
U+8D88.0	fGB	5-6D3B
U+8D88.0	fR/S	 156.5
U+8D88.0	fTotalStrokes	 12
U+8D88.0	fCangjie	GOYR
U+8D88.0	fFourCorner	41800
U+8D88.0	fCantonese	zaam6
U+8D88.0	fMandarin	zhan1
U+8D89.0	fUTF8	趉
U+8D89.0	fCNS11643	2-4055
U+8D89.0	fBig5	DBEF
U+8D89.0	fGB	3-6D7A
U+8D89.0	fR/S	 156.5
U+8D89.0	fTotalStrokes	 12
U+8D89.0	fCangjie	GOUU
U+8D89.0	fFourCorner	42807
U+8D89.0	fCantonese	gwat6; gyut6; kyut3
U+8D89.0	fMandarin	zhu2; jue2
U+8D8A.0	fUTF8	越
U+8D8A.0	fCNS11643	1-623D
U+8D8A.0	fBig5	B656
U+8D8A.0	fGB	0-543D
U+8D8A.0	fR/S	 156.5
U+8D8A.0	fTotalStrokes	 12
U+8D8A.0	fCangjie	GOIV
U+8D8A.0	fFourCorner	43805
U+8D8A.0	fHakka	 jet8; jat8; zat8; ze1; je1
U+8D8A.0	fCantonese	jyut6
U+8D8A.0	fMandarin	yue4; huo2
U+8D8A.0	fEnglish	[1] exceed; go beyond; transgress [2] skip; climb over; cross over; go across [3] even more; the more [4] the name of an ancient state [5] a Chinese family name
U+8D8B.0	fUTF8	趋
U+8D8B.0	fCNS11643	3-3F38
U+8D8B.0	fGB	0-4777
U+8D8B.0	fR/S	 156.5
U+8D8B.0	fTotalStrokes	 12
U+8D8B.0	fCangjie	GONSM
U+8D8B.0	fFourCorner	47800
U+8D8B.0	fHakka	  ci1
U+8D8B.0	fCantonese	ceoi1; cuk1
U+8D8B.0	fMandarin	qu1; cu4; cou3; qu4
U+8D8B.0	fEnglish	simplified form of &#x8DA8;
U+8D8C.0	fUTF8	趌
U+8D8C.0	fCNS11643	2-4765
U+8D8C.0	fBig5	E05F
U+8D8C.0	fGB	E-3F21
U+8D8C.0	fR/S	 156.6
U+8D8C.0	fTotalStrokes	 13
U+8D8C.0	fCangjie	GOGR
U+8D8C.0	fFourCorner	44806
U+8D8C.0	fCantonese	gat1; gat6
U+8D8C.0	fMandarin	jie2; ji2
U+8D8C.0	fEnglish	order someone to leave
U+8D8D.0	fUTF8	趍
U+8D8D.0	fCNS11643	2-4768
U+8D8D.0	fBig5	E062
U+8D8D.0	fGB	5-6D46
U+8D8D.0	fR/S	 156.6
U+8D8D.0	fTotalStrokes	 13
U+8D8D.0	fCangjie	GONIN
U+8D8D.0	fFourCorner	47802
U+8D8D.0	fHakka	 chi1; cih1; chi2; chii2
U+8D8D.0	fCantonese	ceoi1; cuk1
U+8D8D.0	fMandarin	chi2; qu1
U+8D8D.0	fEnglish	 move rapidly
U+8D8E.0	fUTF8	趎
U+8D8E.0	fCNS11643	2-4766
U+8D8E.0	fBig5	E060
U+8D8E.0	fGB	3-6D7D
U+8D8E.0	fR/S	 156.6
U+8D8E.0	fTotalStrokes	 13
U+8D8E.0	fCangjie	GOHJD
U+8D8E.0	fFourCorner	45809
U+8D8E.0	fCantonese	zyu1; syu4
U+8D8E.0	fMandarin	chu2; chou1; chou2; chu1; shu1
U+8D8F.0	fUTF8	趏
U+8D8F.0	fCNS11643	2-4767
U+8D8F.0	fBig5	E061
U+8D8F.0	fGB	5-6D44
U+8D8F.0	fR/S	 156.6
U+8D8F.0	fTotalStrokes	 13
U+8D8F.0	fCangjie	GOHJR
U+8D8F.0	fFourCorner	42806
U+8D8F.0	fCantonese	gwaat3; wut6
U+8D8F.0	fMandarin	gua1; huo2
U+8D90.0	fUTF8	趐
U+8D90.0	fCNS11643	2-476B
U+8D90.0	fBig5	E065
U+8D90.0	fGB	5-6D49
U+8D90.0	fR/S	 156.6
U+8D90.0	fTotalStrokes	 13
U+8D90.0	fCangjie	GOSMM
U+8D90.0	fFourCorner	47802
U+8D90.0	fCantonese	hyut3; ci3
U+8D90.0	fMandarin	xue4
U+8D91.0	fUTF8	趑
U+8D91.0	fCNS11643	2-4764
U+8D91.0	fBig5	E05E
U+8D91.0	fGB	0-7474
U+8D91.0	fR/S	 156.6
U+8D91.0	fTotalStrokes	 13
U+8D91.0	fCangjie	GOIMO
U+8D91.0	fFourCorner	47808
U+8D91.0	fHakka	 zih1; cih1; zii1
U+8D91.0	fCantonese	zi1; ci1
U+8D91.0	fMandarin	ci1; zi1; ci4
U+8D91.0	fEnglish	 falter
U+8D92.0	fUTF8	趒
U+8D92.0	fCNS11643	2-476C
U+8D92.0	fBig5	E066
U+8D92.0	fGB	E-3F22
U+8D92.0	fR/S	 156.6
U+8D92.0	fTotalStrokes	 13
U+8D92.0	fCangjie	GOLMO
U+8D92.0	fFourCorner	42801
U+8D92.0	fHakka	 tiau5; tiau2
U+8D92.0	fCantonese	tiu4; tiu3; tiu1; tiu2
U+8D92.0	fMandarin	tiao2; tiao1; tiao3; tiao4
U+8D92.0	fEnglish	 jump; climb over; leap; posture gangway
U+8D93.0	fUTF8	趓
U+8D93.0	fCNS11643	2-4769
U+8D93.0	fBig5	E063
U+8D93.0	fGB	3-6E21
U+8D93.0	fR/S	 156.6
U+8D93.0	fTotalStrokes	 13
U+8D93.0	fCangjie	GOHND
U+8D93.0	fFourCorner	47809
U+8D93.0	fCantonese	do2
U+8D93.0	fMandarin	duo3
U+8D94.0	fUTF8	趔
U+8D94.0	fCNS11643	2-476A
U+8D94.0	fBig5	E064
U+8D94.0	fGB	0-7473
U+8D94.0	fR/S	 156.6
U+8D94.0	fTotalStrokes	 13
U+8D94.0	fCangjie	GOMNN
U+8D94.0	fFourCorner	42800
U+8D94.0	fHakka	 liet8
U+8D94.0	fCantonese	lit6
U+8D94.0	fMandarin	lie4
U+8D94.0	fEnglish	[1] falling behind; not progressing [2] unskillful; awkward
U+8D95.0	fUTF8	趕
U+8D95.0	fCNS11643	1-6B38
U+8D95.0	fBig5	BBB0
U+8D95.0	fGB	1-384F
U+8D95.0	fR/S	 156.7
U+8D95.0	fTotalStrokes	 14
U+8D95.0	fCangjie	GOAMJ
U+8D95.0	fFourCorner	46804
U+8D95.0	fHakka	  gon3
U+8D95.0	fCantonese	gon2
U+8D95.0	fMandarin	gan3
U+8D95.0	fEnglish	[1] pursue; catch up with; overtake; keep up with [2] expel; drive away [3] hurry; rush; hasten [4] try to catch; make a dash for; rush for
U+8D96.0	fUTF8	趖
U+8D96.0	fCNS11643	2-4E3E
U+8D96.0	fBig5	E456
U+8D96.0	fGB	3-6E24
U+8D96.0	fR/S	 156.7
U+8D96.0	fTotalStrokes	 14
U+8D96.0	fCangjie	GOOOG
U+8D96.0	fFourCorner	48801
U+8D96.0	fHakka	 so5
U+8D96.0	fCantonese	so1
U+8D96.0	fMandarin	suo1
U+8D97.0	fUTF8	趗
U+8D97.0	fCNS11643	3-4975
U+8D97.0	fGB	3-6E23
U+8D97.0	fR/S	  156.7
U+8D97.0	fTotalStrokes	 14
U+8D97.0	fCangjie	GORYO
U+8D97.0	fFourCorner	46800
U+8D97.0	fHakka	 ch'uk8; cuk8
U+8D97.0	fCantonese	cuk1
U+8D97.0	fMandarin	cu4
U+8D97.0	fEnglish	walk briskly; ch'uk8 s'ong1, ch'uk8 ha1, to walk hurriedly to and fro
U+8D98.0	fUTF8	趘
U+8D98.0	fCNS11643	3-4973
U+8D98.0	fGB	E-3F23
U+8D98.0	fR/S	 156.7
U+8D98.0	fTotalStrokes	 14
U+8D98.0	fCangjie	GOSHU
U+8D98.0	fFourCorner	47800
U+8D98.0	fCantonese	hat6
U+8D98.0	fMandarin	xi2
U+8D99.0	fUTF8	趙
U+8D99.0	fCNS11643	1-6B37
U+8D99.0	fBig5	BBAF
U+8D99.0	fGB	1-5554
U+8D99.0	fR/S	 156.7
U+8D99.0	fTotalStrokes	 14
U+8D99.0	fCangjie	GOFB
U+8D99.0	fFourCorner	49802
U+8D99.0	fHakka	 cau3; chau6; cau5; ceu5; chau5
U+8D99.0	fCantonese	ziu6; tiu5
U+8D99.0	fMandarin	zhao4; tiao3; qiao3; diao4
U+8D99.0	fEnglish	[1] a Chinese family name [2] the name of an ancient feudal state
U+8D9A.0	fUTF8	趚
U+8D9A.0	fCNS11643	3-4974
U+8D9A.0	fGB	E-3F24
U+8D9A.0	fR/S	  156.7
U+8D9A.0	fTotalStrokes	 14
U+8D9A.0	fCangjie	GODL
U+8D9A.0	fFourCorner	44800
U+8D9A.0	fCantonese	suk1
U+8D9A.0	fMandarin	su4
U+8D9B.0	fUTF8	趛
U+8D9B.0	fCNS11643	2-545B
U+8D9B.0	fBig5	E7F2
U+8D9B.0	fGB	E-3F25
U+8D9B.0	fR/S	 156.8
U+8D9B.0	fTotalStrokes	 15
U+8D9B.0	fCangjie	GOC
U+8D9B.0	fFourCorner	48801
U+8D9B.0	fCantonese	jam5; gam3
U+8D9B.0	fMandarin	yin3
U+8D9C.0	fUTF8	趜
U+8D9C.0	fCNS11643	2-5459
U+8D9C.0	fBig5	E7F0
U+8D9C.0	fGB	3-6E29
U+8D9C.0	fR/S	 156.8
U+8D9C.0	fTotalStrokes	 15
U+8D9C.0	fCangjie	GOPFD
U+8D9C.0	fFourCorner	47802
U+8D9C.0	fHakka	 giuk7; kiu2
U+8D9C.0	fCantonese	guk1; kau4; kuk1
U+8D9C.0	fMandarin	ju2; qiu2; qu1; qu2
U+8D9C.0	fEnglish	suffocate
U+8D9D.0	fUTF8	趝
U+8D9D.0	fCNS11643	3-4F26
U+8D9D.0	fGB	3-6E27
U+8D9D.0	fR/S	  156.8
U+8D9D.0	fTotalStrokes	 15
U+8D9D.0	fCangjie	GOOIP
U+8D9D.0	fFourCorner	48800
U+8D9D.0	fHakka	 ngiam5; giam5
U+8D9D.0	fCantonese	gim3
U+8D9D.0	fMandarin	jian4
U+8D9D.0	fEnglish	 pursue follow up
U+8D9E.0	fUTF8	趞
U+8D9E.0	fCNS11643	4-4D75
U+8D9E.0	fGB	5-6D4F
U+8D9E.0	fR/S	 156.8
U+8D9E.0	fTotalStrokes	 15
U+8D9E.0	fCangjie	GOTA
U+8D9E.0	fFourCorner	44800
U+8D9E.0	fCantonese	coek3
U+8D9E.0	fMandarin	que4
U+8D9F.0	fUTF8	趟
U+8D9F.0	fCNS11643	1-6F36
U+8D9F.0	fBig5	BDEB
U+8D9F.0	fGB	0-4C4B
U+8D9F.0	fR/S	 156.8
U+8D9F.0	fTotalStrokes	 15
U+8D9F.0	fCangjie	GOFBR
U+8D9F.0	fFourCorner	49802
U+8D9F.0	fHakka	 tong5; tong6; tong3
U+8D9F.0	fCantonese	tong3; tong2 (also); tong1 (also); caang1
U+8D9F.0	fMandarin	tang4; tang1; zheng1; zheng4; cheng1; cheng2
U+8D9F.0	fEnglish	[1] an auxilliary noun for verbs meaning [2] time
U+8DA0.0	fUTF8	趠
U+8DA0.0	fCNS11643	2-5458
U+8DA0.0	fBig5	E7EF
U+8DA0.0	fGB	5-6D51
U+8DA0.0	fR/S	 156.8
U+8DA0.0	fTotalStrokes	 15
U+8DA0.0	fCangjie	GOYAJ
U+8DA0.0	fFourCorner	41804
U+8DA0.0	fHakka	  zok7; tiau5; chok7
U+8DA0.0	fCantonese	coek3
U+8DA0.0	fMandarin	chuo4; chao4; chao1; tiao4; zhuo2
U+8DA0.0	fEnglish	 hasten; travel fast
U+8DA1.0	fUTF8	趡
U+8DA1.0	fCNS11643	2-545A
U+8DA1.0	fBig5	E7F1
U+8DA1.0	fGB	3-6E26
U+8DA1.0	fR/S	 156.8
U+8DA1.0	fTotalStrokes	 15
U+8DA1.0	fCangjie	GOOG
U+8DA1.0	fFourCorner	40801
U+8DA1.0	fCantonese	ceoi2
U+8DA1.0	fMandarin	cui3; jiao4; ju3; wei3
U+8DA1.0	fEnglish	place
U+8DA2.0	fUTF8	趢
U+8DA2.0	fCNS11643	3-4F25
U+8DA2.0	fGB	3-6E2A
U+8DA2.0	fR/S	  156.8
U+8DA2.0	fTotalStrokes	 15
U+8DA2.0	fCangjie	GOVNE
U+8DA2.0	fFourCorner	47800
U+8DA2.0	fCantonese	luk6
U+8DA2.0	fMandarin	lu4
U+8DA3.0	fUTF8	趣
U+8DA3.0	fCNS11643	1-6F37
U+8DA3.0	fBig5	BDEC
U+8DA3.0	fGB	0-4824
U+8DA3.0	fR/S	 156.8
U+8DA3.0	fTotalStrokes	 15
U+8DA3.0	fCangjie	GOSJE
U+8DA3.0	fFourCorner	41804
U+8DA3.0	fHakka	  ci5; ci6; ce5
U+8DA3.0	fCantonese	ceoi3; cuk1; ceoi1; ceoi2 (also); cau2
U+8DA3.0	fMandarin	qu4; qu1; cou3; zou1; zhou3
U+8DA3.0	fEnglish	interest; fun; interesting; funny
U+8DA3.1	fUTF8	趣
U+8DA3.1	fCNS11643	1-6F37
U+8DA3.1	fBig5	BDEC
U+8DA3.1	fGB	0-4824
U+8DA3.1	fR/S	 156.8
U+8DA3.1	fTotalStrokes	 15
U+8DA3.1	fCangjie	GOSJE
U+8DA3.1	fFourCorner	41804
U+8DA3.1	fMandarin	cu4
U+8DA3.1	fEnglish	 hasten
U+8DA4.0	fUTF8	趤
U+8DA4.0	fCNS11643	3-4F23
U+8DA4.0	fGB	E-3F26
U+8DA4.0	fR/S	  156.8
U+8DA4.0	fTotalStrokes	 15
U+8DA4.0	fCangjie	GOJMR
U+8DA4.0	fFourCorner	43800
U+8DA4.0	fCantonese	dong6
U+8DA4.0	fMandarin	dang4
U+8DA5.0	fUTF8	趥
U+8DA5.0	fCNS11643	2-5A67
U+8DA5.0	fBig5	EBBB
U+8DA5.0	fGB	5-6D5B
U+8DA5.0	fR/S	 156.9
U+8DA5.0	fTotalStrokes	 16
U+8DA5.0	fCangjie	GOTCW
U+8DA5.0	fFourCorner	48806
U+8DA5.0	fCantonese	cau1; cuk1
U+8DA5.0	fMandarin	qiu1; cong4; jiu1; zong4
U+8DA6.0	fUTF8	趦
U+8DA6.0	fCNS11643	3-5277
U+8DA6.0	fGB	5-6D5A
U+8DA6.0	fR/S	 156.9
U+8DA6.0	fTotalStrokes	 16
U+8DA6.0	fCangjie	GOIOR
U+8DA6.0	fFourCorner	47800
U+8DA6.0	fHakka	 zii1
U+8DA6.0	fCantonese	zi1; ci1
U+8DA6.0	fMandarin	zi1
U+8DA6.0	fEnglish	 can't move falter
U+8DA7.0	fUTF8	趧
U+8DA7.0	fCNS11643	2-5A68
U+8DA7.0	fBig5	EBBC
U+8DA7.0	fGB	5-6D56
U+8DA7.0	fR/S	 156.9
U+8DA7.0	fTotalStrokes	 16
U+8DA7.0	fCangjie	GOAMO
U+8DA7.0	fFourCorner	46801
U+8DA7.0	fCantonese	tai4
U+8DA7.0	fMandarin	ti2; di1
U+8DA8.0	fUTF8	趨
U+8DA8.0	fCNS11643	1-7558
U+8DA8.0	fBig5	C1CD
U+8DA8.0	fGB	1-4777
U+8DA8.0	fR/S	 156.10
U+8DA8.0	fTotalStrokes	 17
U+8DA8.0	fCangjie	GOPUU
U+8DA8.0	fFourCorner	47802
U+8DA8.0	fHakka	 ci1; chi1; cui1
U+8DA8.0	fCantonese	ceoi1; cuk1
U+8DA8.0	fMandarin	qu1; cu4; cou3; qu4
U+8DA8.0	fEnglish	[1] go quickly; hasten; hurry [2] be inclined; tend; follow
U+8DA9.0	fUTF8	趩
U+8DA9.0	fCNS11643	3-5931
U+8DA9.0	fGB	3-6E30
U+8DA9.0	fR/S	 156.12
U+8DA9.0	fTotalStrokes	 18; 19
U+8DA9.0	fCangjie	GOWTC
U+8DA9.0	fFourCorner	46800
U+8DA9.0	fCantonese	cik1
U+8DA9.0	fMandarin	chi4
U+8DAA.0	fUTF8	趪
U+8DAA.0	fCNS11643	2-6738
U+8DAA.0	fBig5	F34C
U+8DAA.0	fGB	3-6E2F
U+8DAA.0	fR/S	 156.12
U+8DAA.0	fTotalStrokes	 19
U+8DAA.0	fCangjie	GOTLC
U+8DAA.0	fFourCorner	44808
U+8DAA.0	fHakka	 wong2; vong2
U+8DAA.0	fCantonese	wong4; wong2
U+8DAA.0	fMandarin	huang2; guang1
U+8DAA.0	fEnglish	 exert oneself; bustle-about
U+8DAB.0	fUTF8	趫
U+8DAB.0	fCNS11643	2-673A
U+8DAB.0	fBig5	F34E
U+8DAB.0	fGB	3-6D7E
U+8DAB.0	fR/S	 156.12
U+8DAB.0	fTotalStrokes	 19
U+8DAB.0	fCangjie	GOHKB
U+8DAB.0	fFourCorner	42802
U+8DAB.0	fHakka	  kiau2
U+8DAB.0	fCantonese	kiu4
U+8DAB.0	fMandarin	qiao2; chao3; jiao3; qiao1
U+8DAB.0	fEnglish	nimble
U+8DAC.0	fUTF8	趬
U+8DAC.0	fCNS11643	2-6737
U+8DAC.0	fBig5	F34B
U+8DAC.0	fGB	3-6D7C
U+8DAC.0	fR/S	 156.12
U+8DAC.0	fTotalStrokes	 19
U+8DAC.0	fCangjie	GOGGU
U+8DAC.0	fFourCorner	44801
U+8DAC.0	fCantonese	hiu1
U+8DAC.0	fMandarin	qiao2; qiao1; qiao4
U+8DAD.0	fUTF8	趭
U+8DAD.0	fCNS11643	2-6739
U+8DAD.0	fBig5	F34D
U+8DAD.0	fGB	3-6E32
U+8DAD.0	fR/S	 156.12
U+8DAD.0	fTotalStrokes	 19
U+8DAD.0	fCangjie	GOOGF
U+8DAD.0	fFourCorner	40803
U+8DAD.0	fHakka	 ziau5; rau6; jeu5; jau5
U+8DAD.0	fCantonese	ziu3; ziu1
U+8DAD.0	fMandarin	yao4; jiao4; qiao4
U+8DAD.0	fEnglish	 run away
U+8DAE.0	fUTF8	趮
U+8DAE.0	fCNS11643	2-6A24
U+8DAE.0	fBig5	F4D6
U+8DAE.0	fGB	5-6D65
U+8DAE.0	fR/S	 156.13
U+8DAE.0	fTotalStrokes	 20
U+8DAE.0	fCangjie	GORRD
U+8DAE.0	fFourCorner	46809
U+8DAE.0	fHakka	 cau5
U+8DAE.0	fCantonese	cou3
U+8DAE.0	fMandarin	zao4
U+8DAE.0	fEnglish	 easily provoked; hasty fierce; cruel
U+8DAF.0	fUTF8	趯
U+8DAF.0	fCNS11643	2-6C43
U+8DAF.0	fBig5	F654
U+8DAF.0	fGB	3-6E37
U+8DAF.0	fR/S	 156.14
U+8DAF.0	fTotalStrokes	 21
U+8DAF.0	fCangjie	GOSMG
U+8DAF.0	fFourCorner	47801
U+8DAF.0	fHakka	  jok7; tit8; tit7; rok7
U+8DAF.0	fCantonese	tik1; joek3; dek3; joek1
U+8DAF.0	fMandarin	yue4; ti4
U+8DAF.0	fEnglish	jump
U+8DB0.0	fUTF8	趰
U+8DB0.0	fCNS11643	3-6651
U+8DB0.0	fGB	E-3F27
U+8DB0.0	fR/S	 156.14
U+8DB0.0	fTotalStrokes	 21
U+8DB0.0	fCangjie	GOMFB
U+8DB0.0	fFourCorner	4180
U+8DB0.0	fCantonese	ji5
U+8DB1.0	fUTF8	趱
U+8DB1.0	fGB	0-7475
U+8DB1.0	fR/S	 156.16
U+8DB1.0	fTotalStrokes	 23
U+8DB1.0	fCangjie	GOHUO
U+8DB1.0	fHakka	 zan3
U+8DB1.0	fCantonese	zaan2; zaan3
U+8DB1.0	fMandarin	zan3; zan4; zu1; zuan3
U+8DB1.0	fEnglish	simplified form of &#x8DB2;
U+8DB2.0	fUTF8	趲
U+8DB2.0	fCNS11643	2-715E
U+8DB2.0	fBig5	F96F
U+8DB2.0	fGB	1-7475
U+8DB2.0	fR/S	 156.19
U+8DB2.0	fTotalStrokes	 26
U+8DB2.0	fCangjie	GOHUC
U+8DB2.0	fFourCorner	44808
U+8DB2.0	fHakka	  zan3
U+8DB2.0	fCantonese	zaan2; zaan3; zyun2
U+8DB2.0	fMandarin	zan3; zan4; zu1; zuan3
U+8DB2.0	fEnglish	[1] hurry; hasten; rash through [2] urge [3] save (money)
U+8DB3.0	fUTF8	足
U+8DB3.0	fCNS11643	1-4B4D
U+8DB3.0	fBig5	A8AC
U+8DB3.0	fGB	0-5763
U+8DB3.0	fR/S	 157.0
U+8DB3.0	fTotalStrokes	 7
U+8DB3.0	fCangjie	RYO
U+8DB3.0	fFourCorner	60801
U+8DB3.0	fHakka	  ziuk7;  zuk7
U+8DB3.0	fCantonese	zuk1; zeoi3; zyu3
U+8DB3.0	fMandarin	zu2
U+8DB3.0	fEnglish	[1] the foot; leg [2] base (of an object) [3] sufficient; satisfy; enough; adequate; full [4] Kangxi radical 157
U+8DB3.1	fUTF8	足
U+8DB3.1	fCNS11643	1-4B4D
U+8DB3.1	fBig5	A8AC
U+8DB3.1	fGB	0-5763
U+8DB3.1	fR/S	 157.0
U+8DB3.1	fTotalStrokes	 7
U+8DB3.1	fCangjie	RYO
U+8DB3.1	fFourCorner	60801
U+8DB3.1	fMandarin	ju4
U+8DB3.1	fEnglish	overly modest or humble
U+8DB4.0	fUTF8	趴
U+8DB4.0	fCNS11643	1-535A
U+8DB4.0	fBig5	AD77
U+8DB4.0	fGB	0-453F
U+8DB4.0	fR/S	 157.2
U+8DB4.0	fTotalStrokes	 9
U+8DB4.0	fCangjie	RMC
U+8DB4.0	fFourCorner	68100
U+8DB4.0	fHakka	  pa2; ba2; bak7; pa3
U+8DB4.0	fCantonese	paa1
U+8DB4.0	fMandarin	pa1
U+8DB4.0	fEnglish	[1] prostrate oneself; lie face downwards [2] bend over
U+8DB5.0	fUTF8	趵
U+8DB5.0	fCNS11643	2-3329
U+8DB5.0	fBig5	D3E5
U+8DB5.0	fGB	0-7540
U+8DB5.0	fR/S	 157.3
U+8DB5.0	fTotalStrokes	 10
U+8DB5.0	fCangjie	RMPI
U+8DB5.0	fFourCorner	67120
U+8DB5.0	fHakka	 bau5; bok7; bo5
U+8DB5.0	fCantonese	paau3; bok3; baau3
U+8DB5.0	fMandarin	bao4; bo1; zhuo2; zhi1; chuo4; pao2
U+8DB5.0	fEnglish	 jump; leap
U+8DB6.0	fUTF8	趶
U+8DB6.0	fCNS11643	2-332B
U+8DB6.0	fBig5	D3E7
U+8DB6.0	fGB	E-3F28
U+8DB6.0	fR/S	 157.3
U+8DB6.0	fTotalStrokes	 10
U+8DB6.0	fCangjie	RMMD
U+8DB6.0	fFourCorner	61140
U+8DB6.0	fCantonese	fu3
U+8DB6.0	fMandarin	ku4; wu4
U+8DB7.0	fUTF8	趷
U+8DB7.0	fCNS11643	2-332A
U+8DB7.0	fBig5	D3E6
U+8DB7.0	fGB	E-3F29
U+8DB7.0	fR/S	 157.3
U+8DB7.0	fTotalStrokes	 10
U+8DB7.0	fCangjie	RMON
U+8DB7.0	fFourCorner	68117
U+8DB7.0	fHakka	 ngiet7
U+8DB7.0	fCantonese	gat6; kaak1
U+8DB7.0	fMandarin	he2; ke1
U+8DB7.0	fEnglish	 jolt
U+8DB8.0	fUTF8	趸
U+8DB8.0	fGB	0-753B
U+8DB8.0	fR/S	 157.3
U+8DB8.0	fTotalStrokes	 10
U+8DB8.0	fCangjie	MSRYO
U+8DB8.0	fHakka	 dun3
U+8DB8.0	fCantonese	dan2
U+8DB8.0	fMandarin	dun3
U+8DB8.0	fEnglish	simplified form of &#x8E89;
U+8DB9.0	fUTF8	趹
U+8DB9.0	fCNS11643	2-395B
U+8DB9.0	fBig5	D7D8
U+8DB9.0	fGB	3-6F5C
U+8DB9.0	fR/S	 157.4
U+8DB9.0	fTotalStrokes	 11
U+8DB9.0	fCangjie	RMDK
U+8DB9.0	fFourCorner	65130
U+8DB9.0	fHakka	 giet7; get7
U+8DB9.0	fCantonese	kyut3
U+8DB9.0	fMandarin	jue2; gui4
U+8DB9.0	fEnglish	 kick with hoof horse's gallop
U+8DBA.0	fUTF8	趺
U+8DBA.0	fCNS11643	1-5D52
U+8DBA.0	fBig5	B36C
U+8DBA.0	fGB	0-7543
U+8DBA.0	fR/S	 157.4
U+8DBA.0	fTotalStrokes	 11
U+8DBA.0	fCangjie	RMQO
U+8DBA.0	fFourCorner	65130
U+8DBA.0	fHakka	  fu1
U+8DBA.0	fCantonese	fu1
U+8DBA.0	fMandarin	fu1
U+8DBA.0	fEnglish	[1] back of the foot [2] sit cross-legged (as a Buddhist monk does)
U+8DBB.0	fUTF8	趻
U+8DBB.0	fCNS11643	3-393E
U+8DBB.0	fGB	3-6F55
U+8DBB.0	fR/S	  157.4
U+8DBB.0	fTotalStrokes	 11
U+8DBB.0	fCangjie	RMOIN
U+8DBB.0	fFourCorner	68120
U+8DBB.0	fCantonese	cam2
U+8DBB.0	fMandarin	chen3
U+8DBC.0	fUTF8	趼
U+8DBC.0	fCNS11643	2-3959
U+8DBC.0	fBig5	D7D6
U+8DBC.0	fGB	0-7542
U+8DBC.0	fR/S	 157.6
U+8DBC.0	fTotalStrokes	 13
U+8DBC.0	fCangjie	RMMT
U+8DBC.0	fFourCorner	61140
U+8DBC.0	fCantonese	gin2 (also); gaan2; jin4
U+8DBC.0	fMandarin	jian3; yan4
U+8DBC.0	fEnglish	the callous skin on hands or feet; blisters on hands or feet
U+8DBD.0	fUTF8	趽
U+8DBD.0	fCNS11643	3-393A
U+8DBD.0	fGB	3-6F5B
U+8DBD.0	fR/S	  157.4
U+8DBD.0	fTotalStrokes	 11
U+8DBD.0	fCangjie	RMYHS
U+8DBD.0	fFourCorner	60120
U+8DBD.0	fCantonese	fong3
U+8DBD.0	fMandarin	fang4
U+8DBE.0	fUTF8	趾
U+8DBE.0	fCNS11643	1-5D51
U+8DBE.0	fBig5	B36B
U+8DBE.0	fGB	0-563A
U+8DBE.0	fR/S	 157.4
U+8DBE.0	fTotalStrokes	 11
U+8DBE.0	fCangjie	RMYLM
U+8DBE.0	fFourCorner	61110
U+8DBE.0	fHakka	 zhi3; zi3; zih3; tje3
U+8DBE.0	fCantonese	zi2
U+8DBE.0	fMandarin	zhi3
U+8DBE.0	fEnglish	[1] toe [2] foot [3] tracks; footprints
U+8DBF.0	fUTF8	趿
U+8DBF.0	fCNS11643	2-395C
U+8DBF.0	fBig5	D7D9
U+8DBF.0	fGB	0-7541
U+8DBF.0	fR/S	 157.4
U+8DBF.0	fTotalStrokes	 11
U+8DBF.0	fCangjie	RMNHE
U+8DBF.0	fFourCorner	67147
U+8DBF.0	fHakka	 sap7; cap8
U+8DBF.0	fCantonese	taat3; saap3 (also); cap1
U+8DBF.0	fMandarin	sa4; qi4
U+8DBF.0	fEnglish	 pick up with the foot
U+8DBF.1	fUTF8	趿
U+8DBF.1	fCNS11643	2-395C
U+8DBF.1	fBig5	D7D9
U+8DBF.1	fGB	0-7541
U+8DBF.1	fR/S	 157.4
U+8DBF.1	fTotalStrokes	 11
U+8DBF.1	fCangjie	RMNHE
U+8DBF.1	fFourCorner	67147
U+8DBF.1	fMandarin	ta1
U+8DBF.1	fEnglish	 wear (shoes) in a casual way
U+8DC0.0	fUTF8	跀
U+8DC0.0	fCNS11643	3-393D
U+8DC0.0	fGB	3-6F57
U+8DC0.0	fR/S	  157.4
U+8DC0.0	fTotalStrokes	 11
U+8DC0.0	fCangjie	RMB
U+8DC0.0	fFourCorner	67120
U+8DC0.0	fHakka	 nget8
U+8DC0.0	fCantonese	gat6; jyut6
U+8DC0.0	fMandarin	yue4
U+8DC0.0	fEnglish	to cut off the feet, to stretch up
U+8DC1.0	fUTF8	跁
U+8DC1.0	fCNS11643	2-395D
U+8DC1.0	fBig5	D7DA
U+8DC1.0	fGB	3-6F5D
U+8DC1.0	fR/S	 157.4
U+8DC1.0	fTotalStrokes	 11
U+8DC1.0	fCangjie	RMAU
U+8DC1.0	fFourCorner	67117
U+8DC1.0	fHakka	 pa2; pa5; pa3
U+8DC1.0	fCantonese	baa6; paa1
U+8DC1.0	fMandarin	pa2; ba4
U+8DC1.0	fEnglish	[1] trample on, tread on; search for [2] squat down; lie down
U+8DC2.0	fUTF8	跂
U+8DC2.0	fCNS11643	2-395A
U+8DC2.0	fBig5	D7D7
U+8DC2.0	fGB	3-6F51
U+8DC2.0	fR/S	 157.4
U+8DC2.0	fTotalStrokes	 11
U+8DC2.0	fCangjie	RMJE
U+8DC2.0	fFourCorner	64147
U+8DC2.0	fHakka	 ki1;  ki3
U+8DC2.0	fCantonese	kei4; kei5
U+8DC2.0	fMandarin	qi2; qi3; ju4; li3; zhi1
U+8DC2.0	fEnglish	[1] extra toe [2] creeping; crawling (insects)
U+8DC2.1	fUTF8	跂
U+8DC2.1	fCNS11643	2-395A
U+8DC2.1	fBig5	D7D7
U+8DC2.1	fGB	3-6F51
U+8DC2.1	fR/S	 157.4
U+8DC2.1	fTotalStrokes	 11
U+8DC2.1	fCangjie	RMJE
U+8DC2.1	fFourCorner	64147
U+8DC2.1	fMandarin	qi4
U+8DC2.1	fEnglish	 stand on tiptoe
U+8DC3.0	fUTF8	跃
U+8DC3.0	fCNS11643	6-4538
U+8DC3.0	fGB	0-543E
U+8DC3.0	fR/S	  157.4
U+8DC3.0	fTotalStrokes	 11
U+8DC3.0	fCangjie	RMHK
U+8DC3.0	fFourCorner	62130
U+8DC3.0	fHakka	 jok7; tit8
U+8DC3.0	fCantonese	joek6; joek3 (also); tik1
U+8DC3.0	fMandarin	yue4; ti1; ti4; yao4
U+8DC3.0	fEnglish	simplified form of &#x8E8D;
U+8DC4.0	fUTF8	跄
U+8DC4.0	fGB	0-7544
U+8DC4.0	fR/S	 157.4
U+8DC4.0	fTotalStrokes	 11
U+8DC4.0	fCangjie	RMOSU
U+8DC4.0	fHakka	 cong1
U+8DC4.0	fCantonese	coeng1; coeng3
U+8DC4.0	fMandarin	qiang1; qiang2; qiang4
U+8DC4.0	fEnglish	simplified form of &#x8E4C;
U+8DC5.0	fUTF8	跅
U+8DC5.0	fCNS11643	2-4061
U+8DC5.0	fBig5	DBFB
U+8DC5.0	fGB	3-6F64
U+8DC5.0	fR/S	 157.5
U+8DC5.0	fTotalStrokes	 12
U+8DC5.0	fCangjie	RMHMY
U+8DC5.0	fFourCorner	62131
U+8DC5.0	fHakka	 tok8; ch'ak7; cak7
U+8DC5.0	fCantonese	tok3
U+8DC5.0	fMandarin	tuo4; chi4
U+8DC5.0	fEnglish	 careless; lax dissolute
U+8DC6.0	fUTF8	跆
U+8DC6.0	fCNS11643	1-6247
U+8DC6.0	fBig5	B660
U+8DC6.0	fGB	0-754C
U+8DC6.0	fR/S	 157.5
U+8DC6.0	fTotalStrokes	 12
U+8DC6.0	fCangjie	RMIR
U+8DC6.0	fFourCorner	63160
U+8DC6.0	fHakka	 toi2
U+8DC6.0	fCantonese	toi4
U+8DC6.0	fMandarin	tai2
U+8DC6.0	fEnglish	trample
U+8DC7.0	fUTF8	跇
U+8DC7.0	fCNS11643	2-4059
U+8DC7.0	fBig5	DBF3
U+8DC7.0	fGB	3-6F5F
U+8DC7.0	fR/S	 157.5
U+8DC7.0	fTotalStrokes	 12
U+8DC7.0	fCangjie	RMPT
U+8DC7.0	fFourCorner	64117
U+8DC7.0	fCantonese	jai6
U+8DC7.0	fMandarin	yi4; chi4; shi4
U+8DC8.0	fUTF8	跈
U+8DC8.0	fCNS11643	2-405F
U+8DC8.0	fBig5	DBF9
U+8DC8.0	fGB	3-6F65
U+8DC8.0	fR/S	 157.5
U+8DC8.0	fTotalStrokes	 12
U+8DC8.0	fCangjie	RMOHH
U+8DC8.0	fFourCorner	68122
U+8DC8.0	fHakka	 ngien3; ten2; chen5; cen5
U+8DC8.0	fCantonese	nin2; lin2 (also); nin5
U+8DC8.0	fMandarin	nian3; jian4; chen2; tian3
U+8DC8.0	fEnglish	 stamp on, tread to powder; tread in another's stpes
U+8DC9.0	fUTF8	跉
U+8DC9.0	fCNS11643	3-3F3F
U+8DC9.0	fGB	3-6F66
U+8DC9.0	fR/S	  157.5
U+8DC9.0	fTotalStrokes	 12
U+8DC9.0	fCangjie	RMOII
U+8DC9.0	fFourCorner	68130
U+8DC9.0	fCantonese	ling4
U+8DC9.0	fMandarin	ling2
U+8DCA.0	fUTF8	跊
U+8DCA.0	fCNS11643	3-3F3A
U+8DCA.0	fGB	E-3F2A
U+8DCA.0	fR/S	  157.5
U+8DCA.0	fTotalStrokes	 12
U+8DCA.0	fCangjie	RMJD
U+8DCA.0	fFourCorner	65190
U+8DCA.0	fCantonese	mui6
U+8DCA.0	fMandarin	mei4
U+8DCB.0	fUTF8	跋
U+8DCB.0	fCNS11643	1-6242
U+8DCB.0	fBig5	B65B
U+8DCB.0	fGB	0-304F
U+8DCB.0	fR/S	 157.5
U+8DCB.0	fTotalStrokes	 12
U+8DCB.0	fCangjie	RMIKK
U+8DCB.0	fFourCorner	63144
U+8DCB.0	fHakka	  pat8; bat7
U+8DCB.0	fCantonese	bat6; but6
U+8DCB.0	fMandarin	ba2; bei4; bo1
U+8DCB.0	fEnglish	[1] travel [2] postscript; epilogue [3] trample
U+8DCC.0	fUTF8	跌
U+8DCC.0	fCNS11643	1-6245
U+8DCC.0	fBig5	B65E
U+8DCC.0	fGB	0-3578
U+8DCC.0	fR/S	 157.5
U+8DCC.0	fTotalStrokes	 12
U+8DCC.0	fCangjie	RMHQO
U+8DCC.0	fFourCorner	65130
U+8DCC.0	fHakka	 diet7; det7; tiat7
U+8DCC.0	fCantonese	dit3; daat3
U+8DCC.0	fMandarin	die1; tu2; die2
U+8DCC.0	fEnglish	[1] stumble; slip; fall down; fall; drop [2] stamp [3] fall
U+8DCD.0	fUTF8	跍
U+8DCD.0	fCNS11643	2-4058
U+8DCD.0	fBig5	DBF2
U+8DCD.0	fGB	E-3F2B
U+8DCD.0	fR/S	 157.5
U+8DCD.0	fTotalStrokes	 12
U+8DCD.0	fCangjie	RMJR
U+8DCD.0	fFourCorner	64160
U+8DCD.0	fHakka	  ku1;  ku2; gu1; bu1
U+8DCD.0	fCantonese	fu1
U+8DCD.0	fMandarin	ku1
U+8DCD.0	fEnglish	[1] crouch down [2] squat
U+8DCE.0	fUTF8	跎
U+8DCE.0	fCNS11643	1-6240
U+8DCE.0	fBig5	B659
U+8DCE.0	fGB	0-7549
U+8DCE.0	fR/S	 157.5
U+8DCE.0	fTotalStrokes	 12
U+8DCE.0	fCangjie	RMJP
U+8DCE.0	fFourCorner	63111
U+8DCE.0	fHakka	 to2
U+8DCE.0	fCantonese	to4
U+8DCE.0	fMandarin	tuo2
U+8DCE.0	fEnglish	[1] miss one's footing; stumble [2] miss the opporunity; waste time
U+8DCF.0	fUTF8	跏
U+8DCF.0	fCNS11643	2-405C
U+8DCF.0	fBig5	DBF6
U+8DCF.0	fGB	0-754A
U+8DCF.0	fR/S	 157.5
U+8DCF.0	fTotalStrokes	 12
U+8DCF.0	fCangjie	RMKSR
U+8DCF.0	fFourCorner	66100
U+8DCF.0	fHakka	  ga1
U+8DCF.0	fCantonese	gaa1
U+8DCF.0	fMandarin	jia1
U+8DCF.0	fEnglish	 sit cross-legged squat
U+8DD0.0	fUTF8	跐
U+8DD0.0	fCNS11643	2-4772
U+8DD0.0	fBig5	E06C
U+8DD0.0	fGB	3-6F6E
U+8DD0.0	fR/S	 157.5
U+8DD0.0	fTotalStrokes	 12
U+8DD0.0	fCangjie	RMYMP
U+8DD0.0	fFourCorner	61110
U+8DD0.0	fHakka	 cih3
U+8DD0.0	fCantonese	ci2; caai2; ci1; coi2
U+8DD0.0	fMandarin	ci3; cai3; ci1; zhi3; zi3
U+8DD0.0	fEnglish	 trample; step; walk on the ball of the foot
U+8DD1.0	fUTF8	跑
U+8DD1.0	fCNS11643	1-6244
U+8DD1.0	fBig5	B65D
U+8DD1.0	fGB	0-455C
U+8DD1.0	fR/S	 157.5
U+8DD1.0	fTotalStrokes	 12
U+8DD1.0	fCangjie	RMPRU
U+8DD1.0	fFourCorner	67112
U+8DD1.0	fHakka	 pau1;  pau3
U+8DD1.0	fCantonese	paau2; paau4
U+8DD1.0	fMandarin	pao3; pao2; bo2; bao2; pu2
U+8DD1.0	fEnglish	[1] run [2] run away; flee; leave in hurry
U+8DD2.0	fUTF8	跒
U+8DD2.0	fCNS11643	3-3F3B
U+8DD2.0	fGB	3-6F61
U+8DD2.0	fR/S	 157.5
U+8DD2.0	fTotalStrokes	 12
U+8DD2.0	fCangjie	RMMNR
U+8DD2.0	fFourCorner	61120
U+8DD2.0	fHakka	  ka5; ka1; ka3
U+8DD2.0	fCantonese	kaa2
U+8DD2.0	fMandarin	qia3
U+8DD2.0	fEnglish	 walk; step
U+8DD3.0	fUTF8	跓
U+8DD3.0	fCNS11643	2-4057
U+8DD3.0	fBig5	DBF1
U+8DD3.0	fGB	5-6E6D
U+8DD3.0	fR/S	 157.5
U+8DD3.0	fTotalStrokes	 12
U+8DD3.0	fCangjie	RMYG
U+8DD3.0	fFourCorner	60114
U+8DD3.0	fCantonese	cyu5
U+8DD3.0	fMandarin	zhu4
U+8DD4.0	fUTF8	跔
U+8DD4.0	fCNS11643	3-3F3E
U+8DD4.0	fGB	3-6F68
U+8DD4.0	fR/S	 157.5
U+8DD4.0	fTotalStrokes	 12
U+8DD4.0	fCangjie	RMPR
U+8DD4.0	fFourCorner	67120
U+8DD4.0	fHakka	  gi1; giu2; giu1
U+8DD4.0	fCantonese	keoi1; koe4
U+8DD4.0	fMandarin	ju1
U+8DD4.0	fEnglish	feet benumbed, stiff with cold
U+8DD5.0	fUTF8	跕
U+8DD5.0	fCNS11643	2-405D
U+8DD5.0	fBig5	DBF7
U+8DD5.0	fGB	3-6F62
U+8DD5.0	fR/S	 157.5
U+8DD5.0	fTotalStrokes	 12
U+8DD5.0	fCangjie	RMYR
U+8DD5.0	fFourCorner	61160
U+8DD5.0	fHakka	  tiap7; dem5
U+8DD5.0	fCantonese	tip3; dip6 (also); dip3; zaam6; dim2; dit3; dim3; cip3
U+8DD5.0	fMandarin	die2; tie1
U+8DD5.0	fEnglish	shuffle
U+8DD6.0	fUTF8	跖
U+8DD6.0	fCNS11643	2-405A
U+8DD6.0	fBig5	DBF4
U+8DD6.0	fGB	0-7545
U+8DD6.0	fR/S	 157.5
U+8DD6.0	fTotalStrokes	 12
U+8DD6.0	fCangjie	RMMR
U+8DD6.0	fFourCorner	61160
U+8DD6.0	fHakka	 tak8; chak7; zak7; zhak7
U+8DD6.0	fCantonese	zek3 (also); zik3
U+8DD6.0	fMandarin	zhi2; zhi1; zhe4
U+8DD6.0	fEnglish	sole (of the foot)
U+8DD7.0	fUTF8	跗
U+8DD7.0	fCNS11643	2-4060
U+8DD7.0	fBig5	DBFA
U+8DD7.0	fGB	0-7546
U+8DD7.0	fR/S	 157.5
U+8DD7.0	fTotalStrokes	 12
U+8DD7.0	fCangjie	RMODI
U+8DD7.0	fFourCorner	64100
U+8DD7.0	fHakka	 fu1;  fu5; fu2
U+8DD7.0	fCantonese	fu1
U+8DD7.0	fMandarin	fu1; fu4
U+8DD7.0	fEnglish	the instep
U+8DD8.0	fUTF8	跘
U+8DD8.0	fCNS11643	2-4056
U+8DD8.0	fBig5	DBF0
U+8DD8.0	fGB	5-6E6E
U+8DD8.0	fR/S	 157.5
U+8DD8.0	fTotalStrokes	 12
U+8DD8.0	fCangjie	RMFQ
U+8DD8.0	fFourCorner	69150
U+8DD8.0	fCantonese	pun4
U+8DD8.0	fMandarin	pan2; ban3
U+8DD9.0	fUTF8	跙
U+8DD9.0	fCNS11643	2-405E
U+8DD9.0	fBig5	DBF8
U+8DD9.0	fGB	3-6F63
U+8DD9.0	fR/S	 157.5
U+8DD9.0	fTotalStrokes	 12
U+8DD9.0	fCangjie	RMBM
U+8DD9.0	fFourCorner	67110
U+8DD9.0	fHakka	 zu1; zii1; zih1
U+8DD9.0	fCantonese	zeoi1 (also); zeoi2
U+8DD9.0	fMandarin	ju3; qie3; qie4; qu1; zhu4
U+8DD9.0	fEnglish	weak; lame
U+8DDA.0	fUTF8	跚
U+8DDA.0	fCNS11643	1-6243
U+8DDA.0	fBig5	B65C
U+8DDA.0	fGB	0-7547
U+8DDA.0	fR/S	 157.5
U+8DDA.0	fTotalStrokes	 12
U+8DDA.0	fCangjie	RMBT
U+8DDA.0	fFourCorner	67140
U+8DDA.0	fHakka	 san1
U+8DDA.0	fCantonese	saan1
U+8DDA.0	fMandarin	shan1; yuan1
U+8DDA.0	fEnglish	 walk unsteady; stagger
U+8DDB.0	fUTF8	跛
U+8DDB.0	fCNS11643	1-6246
U+8DDB.0	fBig5	B65F
U+8DDB.0	fGB	0-754B
U+8DDB.0	fR/S	 157.5
U+8DDB.0	fTotalStrokes	 12
U+8DDB.0	fCangjie	RMDHE
U+8DDB.0	fFourCorner	64147
U+8DDB.0	fHakka	  bai1;  bai2
U+8DDB.0	fCantonese	bai1; bo2 (also); bei3
U+8DDB.0	fMandarin	bo3; po1
U+8DDB.0	fEnglish	lame; crippled
U+8DDB.1	fUTF8	跛
U+8DDB.1	fCNS11643	1-6246
U+8DDB.1	fBig5	B65F
U+8DDB.1	fGB	0-754B
U+8DDB.1	fR/S	 157.5
U+8DDB.1	fTotalStrokes	 12
U+8DDB.1	fCangjie	RMDHE
U+8DDB.1	fFourCorner	64147
U+8DDB.1	fMandarin	bi4
U+8DDB.1	fEnglish	 lean; be partial
U+8DDC.0	fUTF8	跜
U+8DDC.0	fCNS11643	2-405B
U+8DDC.0	fBig5	DBF5
U+8DDC.0	fGB	E-3F2C
U+8DDC.0	fR/S	 157.5
U+8DDC.0	fTotalStrokes	 12
U+8DDC.0	fCangjie	RMSP
U+8DDC.0	fFourCorner	67111
U+8DDC.0	fCantonese	nei4; lei4 (also)
U+8DDC.0	fMandarin	ni2
U+8DDD.0	fUTF8	距
U+8DDD.0	fCNS11643	1-6241
U+8DDD.0	fBig5	B65A
U+8DDD.0	fGB	0-3E60
U+8DDD.0	fR/S	 157.5
U+8DDD.0	fTotalStrokes	 12
U+8DDD.0	fCangjie	RMSS
U+8DDD.0	fFourCorner	61117
U+8DDD.0	fHakka	  ki3; ki1; gi3
U+8DDD.0	fCantonese	keoi5; geoi6
U+8DDD.0	fMandarin	ju4
U+8DDD.0	fEnglish	[1] distance [2] bird's spur
U+8DDE.0	fUTF8	跞
U+8DDE.0	fGB	0-7548
U+8DDE.0	fR/S	 157.5
U+8DDE.0	fTotalStrokes	 12
U+8DDE.0	fCangjie	RMHVD
U+8DDE.0	fHakka	 lit8
U+8DDE.0	fCantonese	lik1; lok3 (also); lok6
U+8DDE.0	fMandarin	li4; luo4; yao4; yue4
U+8DDE.0	fEnglish	simplified form of &#x8E92;
U+8DDF.0	fUTF8	跟
U+8DDF.0	fCNS11643	1-6679
U+8DDF.0	fBig5	B8F2
U+8DDF.0	fGB	0-387A
U+8DDF.0	fR/S	 157.6
U+8DDF.0	fTotalStrokes	 13
U+8DDF.0	fCangjie	RMAV
U+8DDF.0	fFourCorner	67132
U+8DDF.0	fHakka	 gen1;  gien1; gin1
U+8DDF.0	fCantonese	gan1
U+8DDF.0	fMandarin	gen1
U+8DDF.0	fEnglish	[1] heel [2] follow [3] attend upon [4] and; with
U+8DE0.0	fUTF8	跠
U+8DE0.0	fCNS11643	2-476E
U+8DE0.0	fBig5	E068
U+8DE0.0	fGB	5-6E73
U+8DE0.0	fR/S	 157.6
U+8DE0.0	fTotalStrokes	 13
U+8DE0.0	fCangjie	RMKN
U+8DE0.0	fFourCorner	66132
U+8DE0.0	fHakka	 ji2; ri2; i2
U+8DE0.0	fCantonese	ji4
U+8DE0.0	fMandarin	yi2; chi2
U+8DE0.0	fEnglish	 squat on the heels
U+8DE1.0	fUTF8	跡
U+8DE1.0	fCNS11643	1-6678
U+8DE1.0	fBig5	B8F1
U+8DE1.0	fGB	E-3F2D
U+8DE1.0	fR/S	 157.6
U+8DE1.0	fTotalStrokes	 13
U+8DE1.0	fCangjie	RMYLC
U+8DE1.0	fFourCorner	60130
U+8DE1.0	fHakka	  ziak7; zit7
U+8DE1.0	fCantonese	zik1; zek3
U+8DE1.0	fMandarin	ji1
U+8DE1.0	fEnglish	 tracks; traces; relics; print
U+8DE2.0	fUTF8	跢
U+8DE2.0	fCNS11643	2-4775
U+8DE2.0	fBig5	E06F
U+8DE2.0	fGB	3-6F76
U+8DE2.0	fR/S	 157.6
U+8DE2.0	fTotalStrokes	 13
U+8DE2.0	fCangjie	RMNIN
U+8DE2.0	fFourCorner	67127
U+8DE2.0	fHakka	  doi1;  dai5;  do3; {doi1}
U+8DE2.0	fCantonese	do2; do3; daai3; ci4; do1
U+8DE2.0	fMandarin	dai4; chi2; duo1; duo4; zhi4
U+8DE2.0	fEnglish	the unsteady walk of a child
U+8DE3.0	fUTF8	跣
U+8DE3.0	fCNS11643	2-4774
U+8DE3.0	fBig5	E06E
U+8DE3.0	fGB	0-7550
U+8DE3.0	fR/S	 157.6
U+8DE3.0	fTotalStrokes	 13
U+8DE3.0	fCangjie	RMHGU
U+8DE3.0	fFourCorner	64111
U+8DE3.0	fHakka	  sien3; sien5; sen3; sen1
U+8DE3.0	fCantonese	sin2
U+8DE3.0	fMandarin	xian3; sun3; xian1; qian3
U+8DE3.0	fEnglish	barefooted
U+8DE4.0	fUTF8	跤
U+8DE4.0	fCNS11643	1-6721
U+8DE4.0	fBig5	B8F8
U+8DE4.0	fGB	0-7553
U+8DE4.0	fR/S	 157.6
U+8DE4.0	fTotalStrokes	 13
U+8DE4.0	fCangjie	RMYCK
U+8DE4.0	fFourCorner	60148
U+8DE4.0	fHakka	  gau1
U+8DE4.0	fCantonese	gaau1; haau1
U+8DE4.0	fMandarin	jiao1; qiao1
U+8DE4.0	fEnglish	 stumble; fall
U+8DE5.0	fUTF8	跥
U+8DE5.0	fGB	E-3F2E
U+8DE5.0	fR/S	 157.6
U+8DE5.0	fTotalStrokes	 13
U+8DE5.0	fCangjie	RMNSD
U+8DE5.0	fCantonese	do2; to2
U+8DE5.0	fMandarin	duo4
U+8DE5.0	fEnglish	stamp feet
U+8DE6.0	fUTF8	跦
U+8DE6.0	fCNS11643	1-6722
U+8DE6.0	fBig5	B8F9
U+8DE6.0	fGB	3-6F71
U+8DE6.0	fR/S	 157.6
U+8DE6.0	fTotalStrokes	 13
U+8DE6.0	fCangjie	RMHJD
U+8DE6.0	fFourCorner	65194
U+8DE6.0	fHakka	 zhu1; zu1
U+8DE6.0	fCantonese	zyu1
U+8DE6.0	fMandarin	zhu1; chu2; zhou1
U+8DE6.0	fEnglish	hop
U+8DE7.0	fUTF8	跧
U+8DE7.0	fCNS11643	2-4776
U+8DE7.0	fBig5	E070
U+8DE7.0	fGB	3-6F73
U+8DE7.0	fR/S	 157.6
U+8DE7.0	fTotalStrokes	 13
U+8DE7.0	fCangjie	RMOMG
U+8DE7.0	fFourCorner	68114
U+8DE7.0	fHakka	  cien2;  zien1; cion2
U+8DE7.0	fCantonese	cyun4
U+8DE7.0	fMandarin	quan2; zhuan1; zun1; quan1
U+8DE7.0	fEnglish	crouch
U+8DE8.0	fUTF8	跨
U+8DE8.0	fCNS11643	1-667A
U+8DE8.0	fBig5	B8F3
U+8DE8.0	fGB	0-3F67
U+8DE8.0	fR/S	 157.6
U+8DE8.0	fTotalStrokes	 13
U+8DE8.0	fCangjie	RMKMS
U+8DE8.0	fFourCorner	64127
U+8DE8.0	fHakka	  kwa1;  kwa5; ka1; kiam5; k'ua5; kiam6; hia1; kia2; kia5; kia3; pia1; sia1
U+8DE8.0	fCantonese	kwaa3; kwaa1 (also)
U+8DE8.0	fMandarin	kua4; ku4; kua1; kua3
U+8DE8.0	fEnglish	[1] straddle; ride; sit astride on [2] stride; take a stride [3] cut across; go beyond; extend across
U+8DE9.0	fUTF8	跩
U+8DE9.0	fCNS11643	2-4773
U+8DE9.0	fBig5	E06D
U+8DE9.0	fGB	3-6F6F
U+8DE9.0	fR/S	 157.6
U+8DE9.0	fTotalStrokes	 13
U+8DE9.0	fCangjie	RMLWP
U+8DE9.0	fFourCorner	65106
U+8DE9.0	fCantonese	jai6; zai6
U+8DE9.0	fMandarin	zhuai3; shi4
U+8DE9.0	fEnglish	 waddle; waddling
U+8DEA.0	fUTF8	跪
U+8DEA.0	fCNS11643	1-667E
U+8DEA.0	fBig5	B8F7
U+8DEA.0	fGB	0-3972
U+8DEA.0	fR/S	 157.6
U+8DEA.0	fTotalStrokes	 13
U+8DEA.0	fCangjie	RMNMU
U+8DEA.0	fFourCorner	67112
U+8DEA.0	fHakka	 kui3; kui6; kwi3; k'wui3
U+8DEA.0	fCantonese	gwai6
U+8DEA.0	fMandarin	gui4; gui3; kui3; wei2
U+8DEA.0	fEnglish	 kneel
U+8DEB.0	fUTF8	跫
U+8DEB.0	fCNS11643	2-4778
U+8DEB.0	fBig5	E072
U+8DEB.0	fGB	0-753C
U+8DEB.0	fR/S	 157.6
U+8DEB.0	fTotalStrokes	 13
U+8DEB.0	fCangjie	MNRYO
U+8DEB.0	fFourCorner	17801
U+8DEB.0	fHakka	  kiung2
U+8DEB.0	fCantonese	kung4
U+8DEB.0	fMandarin	qiong2; qiong1; qiang1; kong3; xiong1
U+8DEB.0	fEnglish	the sound of steps; footsteps
U+8DEC.0	fUTF8	跬
U+8DEC.0	fCNS11643	2-476F
U+8DEC.0	fBig5	E069
U+8DEC.0	fGB	0-754D
U+8DEC.0	fR/S	 157.6
U+8DEC.0	fTotalStrokes	 13
U+8DEC.0	fCangjie	RMGG
U+8DEC.0	fFourCorner	64114
U+8DEC.0	fHakka	 gui1; gwi3; kwui3; va1
U+8DEC.0	fCantonese	kwai2
U+8DEC.0	fMandarin	kui3; xie4; kuai1
U+8DEC.0	fEnglish	half a pace
U+8DED.0	fUTF8	跭
U+8DED.0	fCNS11643	3-446B
U+8DED.0	fGB	3-6F75
U+8DED.0	fR/S	 157.6
U+8DED.0	fTotalStrokes	 13
U+8DED.0	fCangjie	RMHEQ
U+8DED.0	fFourCorner	67150
U+8DED.0	fCantonese	hong4
U+8DED.0	fMandarin	xiang2
U+8DEE.0	fUTF8	跮
U+8DEE.0	fCNS11643	2-4771
U+8DEE.0	fBig5	E06B
U+8DEE.0	fGB	3-6F6D
U+8DEE.0	fR/S	 157.6
U+8DEE.0	fTotalStrokes	 13
U+8DEE.0	fCangjie	RMMIG
U+8DEE.0	fFourCorner	61114
U+8DEE.0	fHakka	 zhi5; zih5; zhit7; tsiit7; tje5
U+8DEE.0	fCantonese	ci3 (also); dit6; zi3
U+8DEE.0	fMandarin	chi4; die2
U+8DEE.0	fEnglish	 walk back and forth
U+8DEF.0	fUTF8	路
U+8DEF.0	fCNS11643	1-667B
U+8DEF.0	fBig5	B8F4
U+8DEF.0	fGB	0-4237
U+8DEF.0	fR/S	 157.6
U+8DEF.0	fTotalStrokes	 13
U+8DEF.0	fCangjie	RMHER
U+8DEF.0	fFourCorner	67164
U+8DEF.0	fHakka	  lu5;  lu3; lu6
U+8DEF.0	fCantonese	lou6; lou2 (also)
U+8DEF.0	fMandarin	lu4; luo4
U+8DEF.0	fEnglish	[1] way; road; path; street [2] sort; kind; gang [3] way; means [4] directions; courses [5] province (an administrative division during the Sung dynasty) [6] a Chinese family name
U+8DF0.0	fUTF8	跰
U+8DF0.0	fCNS11643	2-476D
U+8DF0.0	fBig5	E067
U+8DF0.0	fGB	5-6E74
U+8DF0.0	fR/S	 157.6
U+8DF0.0	fTotalStrokes	 13
U+8DF0.0	fCangjie	RMTT
U+8DF0.0	fFourCorner	68141
U+8DF0.0	fCantonese	pin4
U+8DF0.0	fMandarin	beng4; pian2; bing3
U+8DF1.0	fUTF8	跱
U+8DF1.0	fCNS11643	2-4770
U+8DF1.0	fBig5	E06A
U+8DF1.0	fGB	5-6E70
U+8DF1.0	fR/S	 157.6
U+8DF1.0	fTotalStrokes	 13
U+8DF1.0	fCangjie	RMGDI
U+8DF1.0	fFourCorner	64141
U+8DF1.0	fCantonese	zi6; ci5 (also); si6
U+8DF1.0	fMandarin	zhi4; shi4
U+8DF1.0	fEnglish	stop
U+8DF2.0	fUTF8	跲
U+8DF2.0	fCNS11643	2-4777
U+8DF2.0	fBig5	E071
U+8DF2.0	fGB	3-6F74
U+8DF2.0	fR/S	 157.6
U+8DF2.0	fTotalStrokes	 13
U+8DF2.0	fCangjie	RMOMR
U+8DF2.0	fFourCorner	68161
U+8DF2.0	fHakka	  kap7;  giap7
U+8DF2.0	fCantonese	gaap3
U+8DF2.0	fMandarin	jia2; ji2; jie2
U+8DF2.0	fEnglish	 stumble
U+8DF3.0	fUTF8	跳
U+8DF3.0	fCNS11643	1-667C
U+8DF3.0	fBig5	B8F5
U+8DF3.0	fGB	0-4C78
U+8DF3.0	fR/S	 157.6
U+8DF3.0	fTotalStrokes	 13
U+8DF3.0	fCangjie	RMLMO
U+8DF3.0	fFourCorner	62113
U+8DF3.0	fHakka	  tiau5;  tiau2; tau2; tiau3
U+8DF3.0	fCantonese	tiu3; tiu4; tiu2 (also); tou4
U+8DF3.0	fMandarin	tiao4; tao2; tiao2; tiao3; diao4
U+8DF3.0	fEnglish	[1] jump; leap; spring; bounce [2] throb; pulsate; beat [3] skip over (over); jump; make omissions
U+8DF4.0	fUTF8	跴
U+8DF4.0	fCNS11643	2-4779
U+8DF4.0	fBig5	E073
U+8DF4.0	fGB	E-3F2F
U+8DF4.0	fR/S	 157.6
U+8DF4.0	fTotalStrokes	 13
U+8DF4.0	fCangjie	RMMCW
U+8DF4.0	fFourCorner	61160
U+8DF4.0	fHakka	  cai3
U+8DF4.0	fCantonese	caai2; coi2; cai2
U+8DF4.0	fMandarin	cai3
U+8DF4.0	fEnglish	[1] step on; tread on; trample [2] search for; chase; pursue
U+8DF5.0	fUTF8	践
U+8DF5.0	fGB	0-3C79
U+8DF5.0	fR/S	 157.5; 157.6
U+8DF5.0	fTotalStrokes	 12
U+8DF5.0	fCangjie	RMIJ
U+8DF5.0	fHakka	 cien5
U+8DF5.0	fCantonese	cin5; zin6 (also)
U+8DF5.0	fMandarin	jian4
U+8DF5.0	fEnglish	simplified form of &#x8E10;
U+8DF6.0	fUTF8	跶
U+8DF6.0	fGB	7-2132
U+8DF6.0	fR/S	 157.6
U+8DF6.0	fTotalStrokes	 13
U+8DF6.0	fCangjie	RMYK
U+8DF6.0	fCantonese	taat3
U+8DF6.0	fMandarin	ta4; da5
U+8DF6.0	fEnglish	stumble, slip
U+8DF7.0	fUTF8	跷
U+8DF7.0	fGB	0-754E
U+8DF7.0	fR/S	 157.6
U+8DF7.0	fTotalStrokes	 13
U+8DF7.0	fCangjie	RMJPU
U+8DF7.0	fHakka	 kiau1
U+8DF7.0	fCantonese	kiu5; hiu1
U+8DF7.0	fMandarin	qiao1
U+8DF7.0	fEnglish	simplified form of &#x8E7A;
U+8DF8.0	fUTF8	跸
U+8DF8.0	fGB	0-754F
U+8DF8.0	fR/S	 157.6
U+8DF8.0	fTotalStrokes	 13
U+8DF8.0	fCangjie	RMPPJ
U+8DF8.0	fHakka	 bit7
U+8DF8.0	fCantonese	bat1
U+8DF8.0	fMandarin	bi4
U+8DF8.0	fEnglish	simplified form of &#x8E55;
U+8DF9.0	fUTF8	跹
U+8DF9.0	fGB	0-7551
U+8DF9.0	fR/S	 157.6
U+8DF9.0	fTotalStrokes	 13
U+8DF9.0	fCangjie	RMYHJ
U+8DF9.0	fHakka	 cien1
U+8DF9.0	fCantonese	sin1
U+8DF9.0	fMandarin	xian1
U+8DF9.0	fEnglish	simplified form of &#x8E9A;
U+8DFA.0	fUTF8	跺
U+8DFA.0	fCNS11643	1-667D
U+8DFA.0	fBig5	B8F6
U+8DFA.0	fGB	0-3665
U+8DFA.0	fR/S	 157.6
U+8DFA.0	fTotalStrokes	 13
U+8DFA.0	fCangjie	RMHND
U+8DFA.0	fFourCorner	67194
U+8DFA.0	fHakka	 do3
U+8DFA.0	fCantonese	do2
U+8DFA.0	fMandarin	duo4; duo3
U+8DFA.0	fEnglish	 stamp feet
U+8DFB.0	fUTF8	跻
U+8DFB.0	fGB	0-7552
U+8DFB.0	fR/S	 157.6
U+8DFB.0	fTotalStrokes	 13
U+8DFB.0	fCangjie	RMYKL
U+8DFB.0	fHakka	 zi1; zi5
U+8DFB.0	fCantonese	zai1
U+8DFB.0	fMandarin	ji1; ji4
U+8DFB.0	fEnglish	simplified form of &#x8E8B;
U+8DFC.0	fUTF8	跼
U+8DFC.0	fCNS11643	1-6B39
U+8DFC.0	fBig5	BBB1
U+8DFC.0	fGB	E-3F30
U+8DFC.0	fR/S	 157.7
U+8DFC.0	fTotalStrokes	 14
U+8DFC.0	fCangjie	RMSSR
U+8DFC.0	fFourCorner	67127
U+8DFC.0	fHakka	 kiuk8
U+8DFC.0	fCantonese	guk6
U+8DFC.0	fMandarin	ju2; qu2
U+8DFC.0	fEnglish	[1] bent [2] contracted; cramped; confined
U+8DFD.0	fUTF8	跽
U+8DFD.0	fCNS11643	2-4E43
U+8DFD.0	fBig5	E45B
U+8DFD.0	fGB	0-7555
U+8DFD.0	fR/S	 157.7
U+8DFD.0	fTotalStrokes	 14
U+8DFD.0	fCangjie	RMSUP
U+8DFD.0	fFourCorner	67131
U+8DFD.0	fHakka	  ki3
U+8DFD.0	fCantonese	gei6
U+8DFD.0	fMandarin	ji4
U+8DFD.0	fEnglish	 kneel for a long time
U+8DFE.0	fUTF8	跾
U+8DFE.0	fCNS11643	2-4E49
U+8DFE.0	fBig5	E461
U+8DFE.0	fGB	E-3F31
U+8DFE.0	fR/S	 157.7
U+8DFE.0	fTotalStrokes	 14
U+8DFE.0	fCangjie	OKRYO
U+8DFE.0	fFourCorner	28801
U+8DFE.0	fCantonese	suk1; cau1
U+8DFE.0	fMandarin	shu2; chou1
U+8DFF.0	fUTF8	跿
U+8DFF.0	fCNS11643	2-4E41
U+8DFF.0	fBig5	E459
U+8DFF.0	fGB	3-6F77
U+8DFF.0	fR/S	 157.7
U+8DFF.0	fTotalStrokes	 14
U+8DFF.0	fCangjie	RMGYO
U+8DFF.0	fFourCorner	64181
U+8DFF.0	fHakka	 tu2
U+8DFF.0	fCantonese	tou4; cou4
U+8DFF.0	fMandarin	tu2
U+8DFF.0	fEnglish	 jump about
U+8E00.0	fUTF8	踀
U+8E00.0	fCNS11643	2-4E4A
U+8E00.0	fBig5	E462
U+8E00.0	fGB	E-3F32
U+8E00.0	fR/S	 157.7
U+8E00.0	fTotalStrokes	 14
U+8E00.0	fCangjie	RMRYO
U+8E00.0	fFourCorner	66181
U+8E00.0	fCantonese	cuk1
U+8E00.0	fMandarin	chu4
U+8E01.0	fUTF8	踁
U+8E01.0	fCNS11643	3-497B
U+8E01.0	fGB	E-3F33
U+8E01.0	fR/S	 157.7
U+8E01.0	fTotalStrokes	 14
U+8E01.0	fCangjie	RMMVM
U+8E01.0	fFourCorner	61110
U+8E01.0	fCantonese	hing5; ging3 (also); hang1
U+8E01.0	fMandarin	jing4; keng1
U+8E01.0	fEnglish	tinkling, obstinate
U+8E02.0	fUTF8	踂
U+8E02.0	fCNS11643	2-4E40
U+8E02.0	fBig5	E458
U+8E02.0	fGB	3-6F78
U+8E02.0	fR/S	 157.7
U+8E02.0	fTotalStrokes	 14
U+8E02.0	fCangjie	RMSJU
U+8E02.0	fFourCorner	62110
U+8E02.0	fCantonese	nip6; lip6 (also)
U+8E02.0	fMandarin	nie4
U+8E03.0	fUTF8	踃
U+8E03.0	fCNS11643	2-4E45
U+8E03.0	fBig5	E45D
U+8E03.0	fGB	3-6F7C
U+8E03.0	fR/S	 157.7
U+8E03.0	fTotalStrokes	 14
U+8E03.0	fCangjie	RMFB
U+8E03.0	fFourCorner	69127
U+8E03.0	fCantonese	siu1; ciu3
U+8E03.0	fMandarin	xiao1; qiao4
U+8E04.0	fUTF8	踄
U+8E04.0	fCNS11643	2-4E4B
U+8E04.0	fBig5	E463
U+8E04.0	fGB	3-6F7B
U+8E04.0	fR/S	 157.7
U+8E04.0	fTotalStrokes	 14
U+8E04.0	fCangjie	RMYLH
U+8E04.0	fFourCorner	61121
U+8E04.0	fCantonese	bok6; bou6
U+8E04.0	fMandarin	bo2; bu4
U+8E05.0	fUTF8	踅
U+8E05.0	fCNS11643	2-4E48
U+8E05.0	fBig5	E460
U+8E05.0	fGB	0-753D
U+8E05.0	fR/S	 157.7
U+8E05.0	fTotalStrokes	 14
U+8E05.0	fCangjie	QLRYO
U+8E05.0	fFourCorner	42801
U+8E05.0	fCantonese	cyut3; zai6; ci3 (also); zit3; cit3; jyut6; zyut6; zi3
U+8E05.0	fMandarin	chi4
U+8E05.0	fEnglish	 go on one leg
U+8E05.1	fUTF8	踅
U+8E05.1	fCNS11643	2-4E48
U+8E05.1	fBig5	E460
U+8E05.1	fGB	0-753D
U+8E05.1	fR/S	 157.7
U+8E05.1	fTotalStrokes	 14
U+8E05.1	fCangjie	QLRYO
U+8E05.1	fFourCorner	42801
U+8E05.1	fMandarin	xue2
U+8E05.1	fEnglish	 loiter around; hang about
U+8E06.0	fUTF8	踆
U+8E06.0	fCNS11643	2-4E47
U+8E06.0	fBig5	E45F
U+8E06.0	fGB	3-7024
U+8E06.0	fR/S	 157.7
U+8E06.0	fTotalStrokes	 14
U+8E06.0	fCangjie	RMICE
U+8E06.0	fFourCorner	63147
U+8E06.0	fHakka	 zun1; zun3
U+8E06.0	fCantonese	cyun1 (also); seon1; cyun4; ceon1; zeon3
U+8E06.0	fMandarin	qun1; cun1; cun2; zhun1; zun1
U+8E06.0	fEnglish	squat
U+8E07.0	fUTF8	踇
U+8E07.0	fCNS11643	2-4E46
U+8E07.0	fBig5	E45E
U+8E07.0	fGB	E-3F34
U+8E07.0	fR/S	 157.7
U+8E07.0	fTotalStrokes	 14
U+8E07.0	fCangjie	RMOWY
U+8E07.0	fFourCorner	68157
U+8E07.0	fCantonese	mou5; mau5
U+8E07.0	fMandarin	mou3; mu3
U+8E08.0	fUTF8	踈
U+8E08.0	fCNS11643	3-497A
U+8E08.0	fGB	E-3F35
U+8E08.0	fR/S	  157.7
U+8E08.0	fTotalStrokes	 14
U+8E08.0	fCangjie	RMDL
U+8E08.0	fFourCorner	65190
U+8E08.0	fCantonese	so1
U+8E08.0	fMandarin	shu1
U+8E08.0	fEnglish	 neglect careless; lax
U+8E09.0	fUTF8	踉
U+8E09.0	fCNS11643	2-4E3F
U+8E09.0	fBig5	E457
U+8E09.0	fGB	0-7554
U+8E09.0	fR/S	 157.7
U+8E09.0	fTotalStrokes	 14
U+8E09.0	fCangjie	RMIAV
U+8E09.0	fFourCorner	63132
U+8E09.0	fHakka	 liong2; long2; liong3; liong5
U+8E09.0	fCantonese	loeng6; loeng4; long4 (also)
U+8E09.0	fMandarin	lang2; liang2; lang4
U+8E09.0	fEnglish	 hop about; jump about
U+8E09.1	fUTF8	踉
U+8E09.1	fCNS11643	2-4E3F
U+8E09.1	fBig5	E457
U+8E09.1	fGB	0-7554
U+8E09.1	fR/S	 157.7
U+8E09.1	fTotalStrokes	 14
U+8E09.1	fCangjie	RMIAV
U+8E09.1	fFourCorner	63132
U+8E09.1	fMandarin	liang4
U+8E09.1	fEnglish	 walk unsteadily; limp
U+8E0A.0	fUTF8	踊
U+8E0A.0	fCNS11643	2-4E44
U+8E0A.0	fBig5	E45C
U+8E0A.0	fGB	0-533B
U+8E0A.0	fR/S	 157.7
U+8E0A.0	fTotalStrokes	 14
U+8E0A.0	fCangjie	RMNIB
U+8E0A.0	fFourCorner	67127
U+8E0A.0	fHakka	 jung3; rung3
U+8E0A.0	fCantonese	jung2; jung5
U+8E0A.0	fMandarin	yong3
U+8E0A.0	fEnglish	[1] leap; jump [2] rise [3] the shoes worn by the person whose feet were cut off as a form of punishment [4] simplified form of &#x8E34;
U+8E0B.0	fUTF8	踋
U+8E0B.0	fCNS11643	3-4979
U+8E0B.0	fGB	E-3F36
U+8E0B.0	fR/S	 157.7
U+8E0B.0	fTotalStrokes	 14
U+8E0B.0	fCangjie	RMGIL
U+8E0B.0	fFourCorner	67120
U+8E0B.0	fCantonese	goek3
U+8E0B.0	fMandarin	jiao3; jia3; jue2
U+8E0B.0	fEnglish	the foot or feet cast of a play
U+8E0C.0	fUTF8	踌
U+8E0C.0	fGB	0-336C
U+8E0C.0	fR/S	 157.7
U+8E0C.0	fTotalStrokes	 14
U+8E0C.0	fCangjie	RMQKI
U+8E0C.0	fHakka	 chiu2
U+8E0C.0	fCantonese	cau4
U+8E0C.0	fMandarin	chou2
U+8E0C.0	fEnglish	simplified form of &#x8E8A;
U+8E0D.0	fUTF8	踍
U+8E0D.0	fCNS11643	2-4E42
U+8E0D.0	fBig5	E45A
U+8E0D.0	fGB	E-3F37
U+8E0D.0	fR/S	 157.7
U+8E0D.0	fTotalStrokes	 14
U+8E0D.0	fCangjie	RMJKD
U+8E0D.0	fFourCorner	64147
U+8E0D.0	fCantonese	haau1
U+8E0D.0	fMandarin	xiao4; qiao1
U+8E0E.0	fUTF8	踎
U+8E0E.0	fGB	1-7C50
U+8E0E.0	fR/S	 157.7
U+8E0E.0	fTotalStrokes	 14
U+8E0E.0	fCangjie	RMMFR
U+8E0E.0	fCantonese	mau1
U+8E0E.0	fEnglish	 squat down
U+8E0F.0	fUTF8	踏
U+8E0F.0	fCNS11643	1-6F3C
U+8E0F.0	fBig5	BDF1
U+8E0F.0	fGB	0-4C24
U+8E0F.0	fR/S	 157.8
U+8E0F.0	fTotalStrokes	 15
U+8E0F.0	fCangjie	RMEA
U+8E0F.0	fFourCorner	62163
U+8E0F.0	fHakka	  tap8
U+8E0F.0	fCantonese	daap6
U+8E0F.0	fMandarin	ta4
U+8E0F.0	fEnglish	[1] step on; trample; tread on [2] go to the spot (make an investigation or survey)
U+8E0F.1	fUTF8	踏
U+8E0F.1	fCNS11643	1-6F3C
U+8E0F.1	fBig5	BDF1
U+8E0F.1	fGB	0-4C24
U+8E0F.1	fR/S	 157.8
U+8E0F.1	fTotalStrokes	 15
U+8E0F.1	fCangjie	RMEA
U+8E0F.1	fFourCorner	62163
U+8E0F.1	fMandarin	ta1
U+8E10.0	fUTF8	踐
U+8E10.0	fCNS11643	1-6F39
U+8E10.0	fBig5	BDEE
U+8E10.0	fGB	1-3C79
U+8E10.0	fR/S	 157.8
U+8E10.0	fTotalStrokes	 15
U+8E10.0	fCangjie	RMII
U+8E10.0	fFourCorner	63153
U+8E10.0	fHakka	 cien5; cen3; zien5; zien3; ts'ian6; cien6
U+8E10.0	fCantonese	cin5; zin6 (also)
U+8E10.0	fMandarin	jian4
U+8E10.0	fEnglish	[1] trample; tread upon; walk on [2] fulfill; carry out; perform [3] asscend; occupy
U+8E11.0	fUTF8	踑
U+8E11.0	fCNS11643	2-5464
U+8E11.0	fBig5	E7FB
U+8E11.0	fGB	5-6E79
U+8E11.0	fR/S	 157.8
U+8E11.0	fTotalStrokes	 15
U+8E11.0	fCangjie	RMTMC
U+8E11.0	fFourCorner	64181
U+8E11.0	fHakka	  ki2
U+8E11.0	fCantonese	gei1; kei1 (also); kei2
U+8E11.0	fMandarin	qi2; ji1; ji4
U+8E11.0	fEnglish	[1] tracks of a horse [2] cross the legs
U+8E12.0	fUTF8	踒
U+8E12.0	fCNS11643	2-5469
U+8E12.0	fBig5	E841
U+8E12.0	fGB	3-702D
U+8E12.0	fR/S	 157.8
U+8E12.0	fTotalStrokes	 15
U+8E12.0	fCangjie	RMHDV
U+8E12.0	fFourCorner	62144
U+8E12.0	fHakka	 wo1; wi1; wui1; vui1
U+8E12.0	fCantonese	wo1
U+8E12.0	fMandarin	wo1; rui2; wei1; wo4
U+8E12.0	fEnglish	 slip and sprain a blimb
U+8E13.0	fUTF8	踓
U+8E13.0	fCNS11643	2-546B
U+8E13.0	fBig5	E843
U+8E13.0	fGB	5-6F21
U+8E13.0	fR/S	 157.8
U+8E13.0	fTotalStrokes	 15
U+8E13.0	fCangjie	RMOG
U+8E13.0	fFourCorner	60114
U+8E13.0	fCantonese	wai5; cuk1
U+8E13.0	fMandarin	wei3; cu4; cui3
U+8E14.0	fUTF8	踔
U+8E14.0	fCNS11643	2-5468
U+8E14.0	fBig5	E840
U+8E14.0	fGB	0-7556
U+8E14.0	fR/S	 157.8
U+8E14.0	fTotalStrokes	 15
U+8E14.0	fCangjie	RMYAJ
U+8E14.0	fFourCorner	61146
U+8E14.0	fHakka	  zok7; zau5; tiau5; ch'iok7; ts'iok7
U+8E14.0	fCantonese	coek3; diu6
U+8E14.0	fMandarin	chuo1; zhuo2; diao4; tiao4; chao4; zhuo1; chuo4; tiao3; zhao4
U+8E14.0	fEnglish	[1] go across; go beyond [2] very high; very far
U+8E15.0	fUTF8	踕
U+8E15.0	fCNS11643	2-5461
U+8E15.0	fBig5	E7F8
U+8E15.0	fGB	3-7029
U+8E15.0	fR/S	 157.8
U+8E15.0	fTotalStrokes	 15
U+8E15.0	fCangjie	RMJLO
U+8E15.0	fFourCorner	65181
U+8E15.0	fHakka	 diak7; ciak7
U+8E15.0	fCantonese	zit6
U+8E15.0	fMandarin	jie2; qi2
U+8E16.0	fUTF8	踖
U+8E16.0	fCNS11643	2-5463
U+8E16.0	fBig5	E7FA
U+8E16.0	fGB	3-7026
U+8E16.0	fR/S	 157.8
U+8E16.0	fTotalStrokes	 15
U+8E16.0	fCangjie	RMTA
U+8E16.0	fFourCorner	64161
U+8E16.0	fHakka	  sit7;  zit7
U+8E16.0	fCantonese	zik1; zik6; cik1
U+8E16.0	fMandarin	ji2; qi4; que4; xi2
U+8E16.0	fEnglish	 trample; tread upon
U+8E17.0	fUTF8	踗
U+8E17.0	fCNS11643	2-546D
U+8E17.0	fBig5	E845
U+8E17.0	fGB	E-3F38
U+8E17.0	fR/S	 157.8
U+8E17.0	fTotalStrokes	 15
U+8E17.0	fCangjie	RMOIP
U+8E17.0	fFourCorner	68132
U+8E17.0	fHakka	 ngiap7
U+8E17.0	fCantonese	nip6; lip6 (also)
U+8E17.0	fMandarin	nie1
U+8E18.0	fUTF8	踘
U+8E18.0	fCNS11643	2-546A
U+8E18.0	fBig5	E842
U+8E18.0	fGB	5-6F23
U+8E18.0	fR/S	 157.8
U+8E18.0	fTotalStrokes	 15
U+8E18.0	fCangjie	RMPFD
U+8E18.0	fFourCorner	67120
U+8E18.0	fHakka	  kiuk7
U+8E18.0	fCantonese	guk1; guk6
U+8E18.0	fMandarin	ju2; ju1
U+8E18.0	fEnglish	ball
U+8E19.0	fUTF8	踙
U+8E19.0	fCNS11643	2-5465
U+8E19.0	fBig5	E7FC
U+8E19.0	fGB	E-3F39
U+8E19.0	fR/S	 157.8
U+8E19.0	fTotalStrokes	 15
U+8E19.0	fCangjie	RMSJE
U+8E19.0	fFourCorner	67140
U+8E19.0	fCantonese	nip6; lip6 (also)
U+8E19.0	fMandarin	ju1; nie4
U+8E1A.0	fUTF8	踚
U+8E1A.0	fCNS11643	2-546E
U+8E1A.0	fBig5	E846
U+8E1A.0	fGB	3-6F54
U+8E1A.0	fR/S	 157.8
U+8E1A.0	fTotalStrokes	 15
U+8E1A.0	fCangjie	RMOMB
U+8E1A.0	fFourCorner	68127
U+8E1A.0	fCantonese	leon4
U+8E1A.0	fMandarin	lun2
U+8E1B.0	fUTF8	踛
U+8E1B.0	fCNS11643	2-5462
U+8E1B.0	fBig5	E7F9
U+8E1B.0	fGB	5-6E78
U+8E1B.0	fR/S	 157.8
U+8E1B.0	fTotalStrokes	 15
U+8E1B.0	fCangjie	RMGCG
U+8E1B.0	fFourCorner	64114
U+8E1B.0	fCantonese	luk6
U+8E1B.0	fMandarin	lu4
U+8E1C.0	fUTF8	踜
U+8E1C.0	fCNS11643	2-546C
U+8E1C.0	fBig5	E844
U+8E1C.0	fGB	3-7025
U+8E1C.0	fR/S	 157.8
U+8E1C.0	fTotalStrokes	 15
U+8E1C.0	fCangjie	RMGCE
U+8E1C.0	fFourCorner	64147
U+8E1C.0	fCantonese	ling6; ling4
U+8E1C.0	fMandarin	leng4; cheng3; leng2
U+8E1C.0	fEnglish	 stumble slip fall
U+8E1D.0	fUTF8	踝
U+8E1D.0	fCNS11643	1-6F3A
U+8E1D.0	fBig5	BDEF
U+8E1D.0	fGB	0-7557
U+8E1D.0	fR/S	 157.8
U+8E1D.0	fTotalStrokes	 15
U+8E1D.0	fCangjie	RMWD
U+8E1D.0	fFourCorner	66194
U+8E1D.0	fHakka	 ko5; kwa3; fa2; kwa5; ko3
U+8E1D.0	fCantonese	waa5
U+8E1D.0	fMandarin	huai2; hua4
U+8E1D.0	fEnglish	the ankle
U+8E1E.0	fUTF8	踞
U+8E1E.0	fCNS11643	1-6F40
U+8E1E.0	fBig5	BDF5
U+8E1E.0	fGB	0-3E61
U+8E1E.0	fR/S	 157.8
U+8E1E.0	fTotalStrokes	 15
U+8E1E.0	fCangjie	RMSJR
U+8E1E.0	fFourCorner	67164
U+8E1E.0	fHakka	  gi3; gi1; gi5
U+8E1E.0	fCantonese	geoi3
U+8E1E.0	fMandarin	ju4; ju1
U+8E1E.0	fEnglish	[1] crouch; squat [2] occupy
U+8E1F.0	fUTF8	踟
U+8E1F.0	fCNS11643	1-6F3E
U+8E1F.0	fBig5	BDF3
U+8E1F.0	fGB	0-7558
U+8E1F.0	fR/S	 157.8
U+8E1F.0	fTotalStrokes	 15
U+8E1F.0	fCangjie	RMOKR
U+8E1F.0	fFourCorner	66100
U+8E1F.0	fHakka	 chi1; cih1; ci1; chii2; chi2
U+8E1F.0	fCantonese	ci4
U+8E1F.0	fMandarin	chi2
U+8E1F.0	fEnglish	 hesitate; undecided
U+8E20.0	fUTF8	踠
U+8E20.0	fCNS11643	2-545C
U+8E20.0	fBig5	E7F3
U+8E20.0	fGB	3-7031
U+8E20.0	fR/S	 157.8
U+8E20.0	fTotalStrokes	 15
U+8E20.0	fCangjie	RMJNU
U+8E20.0	fFourCorner	63112
U+8E20.0	fHakka	 wan3
U+8E20.0	fCantonese	jyun2
U+8E20.0	fMandarin	wan3; wan4; wo4
U+8E20.0	fEnglish	 crooked leg
U+8E21.0	fUTF8	踡
U+8E21.0	fCNS11643	1-6F3F
U+8E21.0	fBig5	BDF4
U+8E21.0	fGB	5-6F25
U+8E21.0	fR/S	 157.8
U+8E21.0	fTotalStrokes	 15
U+8E21.0	fCangjie	RMFQU
U+8E21.0	fFourCorner	69112
U+8E21.0	fHakka	 kien2; ken2; kian2
U+8E21.0	fCantonese	kyun4
U+8E21.0	fMandarin	quan2
U+8E21.0	fEnglish	curled; coiled; drawn together; not stretched
U+8E22.0	fUTF8	踢
U+8E22.0	fCNS11643	1-6F3B
U+8E22.0	fBig5	BDF0
U+8E22.0	fGB	0-4C5F
U+8E22.0	fR/S	 157.8
U+8E22.0	fTotalStrokes	 15
U+8E22.0	fCangjie	RMAPH
U+8E22.0	fFourCorner	66127
U+8E22.0	fHakka	 tet7; tiak7; tit7; dem3
U+8E22.0	fCantonese	tek3
U+8E22.0	fMandarin	ti1; die2; chuo4; shuo4
U+8E22.0	fEnglish	[1] kick [2] play (football)
U+8E23.0	fUTF8	踣
U+8E23.0	fCNS11643	2-545D
U+8E23.0	fBig5	E7F4
U+8E23.0	fGB	0-755B
U+8E23.0	fR/S	 157.8
U+8E23.0	fTotalStrokes	 15
U+8E23.0	fCangjie	RMYTR
U+8E23.0	fFourCorner	60161
U+8E23.0	fCantonese	baak6; fau6
U+8E23.0	fMandarin	bo2; pou4; fu4; pei2; pou2
U+8E23.0	fEnglish	[1] stumble and fall [2] stiff; dead
U+8E24.0	fUTF8	踤
U+8E24.0	fCNS11643	2-545F
U+8E24.0	fBig5	E7F6
U+8E24.0	fGB	3-702E
U+8E24.0	fR/S	 157.8
U+8E24.0	fTotalStrokes	 15
U+8E24.0	fCangjie	RMYOJ
U+8E24.0	fFourCorner	60148
U+8E24.0	fHakka	  cut7;  cui5; zut8
U+8E24.0	fCantonese	zeot1; seoi6; cyut3
U+8E24.0	fMandarin	zu2; cui4; cu4
U+8E24.0	fEnglish	[1] (ts'ut7) butt against; kick (ts'ui5) [2] crowd together
U+8E25.0	fUTF8	踥
U+8E25.0	fCNS11643	2-545E
U+8E25.0	fBig5	E7F5
U+8E25.0	fGB	3-702F
U+8E25.0	fR/S	 157.8
U+8E25.0	fTotalStrokes	 15
U+8E25.0	fCangjie	RMYTV
U+8E25.0	fFourCorner	60144
U+8E25.0	fCantonese	cip3; sip3
U+8E25.0	fMandarin	qie4
U+8E25.0	fEnglish	 walk; be in motion
U+8E26.0	fUTF8	踦
U+8E26.0	fCNS11643	2-5466
U+8E26.0	fBig5	E7FD
U+8E26.0	fGB	3-7028
U+8E26.0	fR/S	 157.8
U+8E26.0	fTotalStrokes	 15
U+8E26.0	fCangjie	RMKMR
U+8E26.0	fFourCorner	64121
U+8E26.0	fHakka	  ki2
U+8E26.0	fCantonese	kei1; gei2; gei1; ji2; hei1 (also); hei2
U+8E26.0	fMandarin	ji3; qi1; ji1; ji4; qi2; yi4
U+8E26.0	fEnglish	[1] the shin [2] lean against
U+8E26.1	fUTF8	踦
U+8E26.1	fCNS11643	2-5466
U+8E26.1	fBig5	E7FD
U+8E26.1	fGB	3-7028
U+8E26.1	fR/S	 157.8
U+8E26.1	fTotalStrokes	 15
U+8E26.1	fCangjie	RMKMR
U+8E26.1	fFourCorner	64121
U+8E26.1	fMandarin	yi3
U+8E26.1	fEnglish	 pierce; touch
U+8E27.0	fUTF8	踧
U+8E27.0	fCNS11643	2-5467
U+8E27.0	fBig5	E7FE
U+8E27.0	fGB	3-702A
U+8E27.0	fR/S	 157.8
U+8E27.0	fTotalStrokes	 15
U+8E27.0	fCangjie	RMYFE
U+8E27.0	fFourCorner	67140
U+8E27.0	fHakka	 tit8; shuk7; tit7; zuk7; suk7
U+8E27.0	fCantonese	cuk1; dik6; zuk1
U+8E27.0	fMandarin	cu4
U+8E27.0	fEnglish	reverent and nervous
U+8E27.1	fUTF8	踧
U+8E27.1	fCNS11643	2-5467
U+8E27.1	fBig5	E7FE
U+8E27.1	fGB	3-702A
U+8E27.1	fR/S	 157.8
U+8E27.1	fTotalStrokes	 15
U+8E27.1	fCangjie	RMYFE
U+8E27.1	fFourCorner	67140
U+8E27.1	fMandarin	di2
U+8E27.1	fEnglish	level and easy
U+8E28.0	fUTF8	踨
U+8E28.0	fCNS11643	3-4F2C
U+8E28.0	fGB	E-3F3A
U+8E28.0	fR/S	 157.8
U+8E28.0	fTotalStrokes	 15
U+8E28.0	fCangjie	RMOOO
U+8E28.0	fFourCorner	68180
U+8E28.0	fHakka	 ciok7
U+8E28.0	fCantonese	zung1
U+8E28.0	fMandarin	zong1
U+8E29.0	fUTF8	踩
U+8E29.0	fCNS11643	1-6F3D
U+8E29.0	fBig5	BDF2
U+8E29.0	fGB	0-3248
U+8E29.0	fR/S	 157.8
U+8E29.0	fTotalStrokes	 15
U+8E29.0	fCangjie	RMBD
U+8E29.0	fFourCorner	62194
U+8E29.0	fHakka	  cai3; cai2
U+8E29.0	fCantonese	caai2; coi2; ai2
U+8E29.0	fMandarin	cai3; kui2; lie4
U+8E29.0	fEnglish	 thread upon; step on; trample
U+8E2A.0	fUTF8	踪
U+8E2A.0	fCNS11643	3-4F2B
U+8E2A.0	fGB	0-5759
U+8E2A.0	fR/S	 157.8
U+8E2A.0	fTotalStrokes	 15
U+8E2A.0	fCangjie	RMJMF
U+8E2A.0	fFourCorner	63190
U+8E2A.0	fHakka	  zung1; ziung1
U+8E2A.0	fCantonese	zung1
U+8E2A.0	fMandarin	zong1
U+8E2A.0	fEnglish	 footprint; traces; tracks
U+8E2B.0	fUTF8	踫
U+8E2B.0	fCNS11643	1-6F38
U+8E2B.0	fBig5	BDED
U+8E2B.0	fGB	E-3F3B
U+8E2B.0	fR/S	 157.9
U+8E2B.0	fTotalStrokes	 16
U+8E2B.0	fCangjie	RMTTC
U+8E2B.0	fFourCorner	68117
U+8E2B.0	fCantonese	pung3
U+8E2B.0	fMandarin	peng4; pan2
U+8E2B.0	fEnglish	collide; bump into
U+8E2C.0	fUTF8	踬
U+8E2C.0	fGB	0-7559
U+8E2C.0	fR/S	 157.8
U+8E2C.0	fTotalStrokes	 15
U+8E2C.0	fCangjie	RMHJO
U+8E2C.0	fHakka	 zhit7; zhi5
U+8E2C.0	fCantonese	zi3
U+8E2C.0	fMandarin	zhi4; zhi1; zhi2
U+8E2C.0	fEnglish	simplified form of &#x8E93;
U+8E2D.0	fUTF8	踭
U+8E2D.0	fGB	1-7C51
U+8E2D.0	fR/S	 157.8
U+8E2D.0	fTotalStrokes	 15
U+8E2D.0	fCangjie	XRMNS
U+8E2D.0	fHakka	 zang1
U+8E2D.0	fCantonese	zaang1; zang1
U+8E2D.0	fMandarin	zheng1
U+8E2D.0	fEnglish	 the heel
U+8E2E.0	fUTF8	踮
U+8E2E.0	fCNS11643	2-5460
U+8E2E.0	fBig5	E7F7
U+8E2E.0	fGB	0-755A
U+8E2E.0	fR/S	 157.8
U+8E2E.0	fTotalStrokes	 15
U+8E2E.0	fCangjie	RMIYR
U+8E2E.0	fFourCorner	60161
U+8E2E.0	fHakka	 diam5
U+8E2E.0	fCantonese	dim2; dim3
U+8E2E.0	fMandarin	dian3; dian4
U+8E2E.0	fEnglish	tip toe
U+8E2F.0	fUTF8	踯
U+8E2F.0	fGB	0-755C
U+8E2F.0	fR/S	 157.8
U+8E2F.0	fTotalStrokes	 15
U+8E2F.0	fCangjie	XRMTK
U+8E2F.0	fHakka	 chit7
U+8E2F.0	fCantonese	zaak6
U+8E2F.0	fMandarin	zhi2
U+8E2F.0	fEnglish	simplified form of &#x8E91;
U+8E30.0	fUTF8	踰
U+8E30.0	fCNS11643	2-5A72
U+8E30.0	fBig5	EBC6
U+8E30.0	fGB	E-3F3C
U+8E30.0	fR/S	 157.9
U+8E30.0	fTotalStrokes	 16
U+8E30.0	fCangjie	RMOMN
U+8E30.0	fFourCorner	68121
U+8E30.0	fHakka	 ji2; ri2; ri5; ji5; i2
U+8E30.0	fCantonese	jyu4; jyu6
U+8E30.0	fMandarin	yu2; yao2
U+8E30.0	fEnglish	[1] same as $ [2] excessive; overly
U+8E31.0	fUTF8	踱
U+8E31.0	fCNS11643	1-724C
U+8E31.0	fBig5	BFE2
U+8E31.0	fGB	0-7562
U+8E31.0	fR/S	 157.9
U+8E31.0	fTotalStrokes	 16
U+8E31.0	fCangjie	RMITE
U+8E31.0	fFourCorner	60147
U+8E31.0	fHakka	  tok8; dok7; dok8
U+8E31.0	fCantonese	dok6
U+8E31.0	fMandarin	duo2; chuo4; duo4
U+8E31.0	fEnglish	 stroll; walk slowly
U+8E32.0	fUTF8	踲
U+8E32.0	fCNS11643	3-527E
U+8E32.0	fGB	5-6F2C
U+8E32.0	fR/S	  157.9
U+8E32.0	fTotalStrokes	 16
U+8E32.0	fCangjie	RMHJU
U+8E32.0	fFourCorner	62160
U+8E32.0	fCantonese	dan6
U+8E32.0	fMandarin	dun4
U+8E33.0	fUTF8	踳
U+8E33.0	fCNS11643	2-5A69
U+8E33.0	fBig5	EBBD
U+8E33.0	fGB	3-7036
U+8E33.0	fR/S	 157.9
U+8E33.0	fTotalStrokes	 16
U+8E33.0	fCangjie	RMQKA
U+8E33.0	fFourCorner	65162
U+8E33.0	fHakka	 chun3; cun3
U+8E33.0	fCantonese	cyun2; ceon2; ceon1
U+8E33.0	fMandarin	chun3; chuan3
U+8E33.0	fEnglish	incongruous; disorderly
U+8E34.0	fUTF8	踴
U+8E34.0	fCNS11643	1-724D
U+8E34.0	fBig5	BFE3
U+8E34.0	fGB	1-533B
U+8E34.0	fR/S	 157.9
U+8E34.0	fTotalStrokes	 16
U+8E34.0	fCangjie	RMNBS
U+8E34.0	fFourCorner	67127
U+8E34.0	fHakka	  jung3; rung3
U+8E34.0	fCantonese	jung2; jung5
U+8E34.0	fMandarin	yong3
U+8E34.0	fEnglish	leap; jump
U+8E35.0	fUTF8	踵
U+8E35.0	fCNS11643	1-7250
U+8E35.0	fBig5	BFE6
U+8E35.0	fGB	0-7560
U+8E35.0	fR/S	 157.9
U+8E35.0	fTotalStrokes	 16
U+8E35.0	fCangjie	RMHJG
U+8E35.0	fFourCorner	62114
U+8E35.0	fHakka	 zung3;  zhung3
U+8E35.0	fCantonese	zung2; dung2 (also)
U+8E35.0	fMandarin	zhong3; zhong4
U+8E35.0	fEnglish	[1] the heel [2] follow [3] call personally at; go personally; call in person
U+8E36.0	fUTF8	踶
U+8E36.0	fCNS11643	2-5A6E
U+8E36.0	fBig5	EBC2
U+8E36.0	fGB	3-703E
U+8E36.0	fR/S	 157.9
U+8E36.0	fTotalStrokes	 16
U+8E36.0	fCangjie	RMAMO
U+8E36.0	fFourCorner	66181
U+8E36.0	fHakka	  ti2;  ti5
U+8E36.0	fCantonese	dai6; tai4; ci2; ci4; zi6
U+8E36.0	fMandarin	di4; chi2; ti2; zhi4; shi4
U+8E36.0	fEnglish	paw
U+8E37.0	fUTF8	踷
U+8E37.0	fCNS11643	5-5148
U+8E37.0	fGB	E-3F3D
U+8E37.0	fR/S	 157.9; 157.8
U+8E37.0	fTotalStrokes	 16; 15
U+8E37.0	fCangjie	RMJKA
U+8E37.0	fFourCorner	64160
U+8E37.0	fHakka	 zha5
U+8E37.0	fCantonese	zaa2
U+8E37.0	fMandarin	zhe3; zha3
U+8E38.0	fUTF8	踸
U+8E38.0	fCNS11643	2-5A6B
U+8E38.0	fBig5	EBBF
U+8E38.0	fGB	3-7037
U+8E38.0	fR/S	 157.9
U+8E38.0	fTotalStrokes	 16
U+8E38.0	fCangjie	RMTMV
U+8E38.0	fFourCorner	64111
U+8E38.0	fCantonese	cam2
U+8E38.0	fMandarin	chen3; chan3; chen1
U+8E39.0	fUTF8	踹
U+8E39.0	fCNS11643	1-724F
U+8E39.0	fBig5	BFE5
U+8E39.0	fGB	0-755F
U+8E39.0	fR/S	 157.9
U+8E39.0	fTotalStrokes	 16
U+8E39.0	fCangjie	RMUMB
U+8E39.0	fFourCorner	62127
U+8E39.0	fHakka	  cai3;  ton5; don1; cai2; don3
U+8E39.0	fCantonese	cyun2; caai2; jaai2 (also); dyun3
U+8E39.0	fMandarin	chuai4; shuan4; duan4
U+8E39.0	fEnglish	[1] trample; tread; stamp [2] kick
U+8E3A.0	fUTF8	踺
U+8E3A.0	fCNS11643	4-5429
U+8E3A.0	fGB	0-755D
U+8E3A.0	fR/S	 157.8; 157.9
U+8E3A.0	fTotalStrokes	 16; 15
U+8E3A.0	fCangjie	RMNKQ
U+8E3A.0	fFourCorner	65140
U+8E3A.0	fHakka	 kien6; kien5; ken5
U+8E3A.0	fCantonese	gin6
U+8E3A.0	fMandarin	jian4
U+8E3A.0	fEnglish	 walk
U+8E3B.0	fUTF8	踻
U+8E3B.0	fCNS11643	3-527B
U+8E3B.0	fGB	3-6F7D
U+8E3B.0	fR/S	  157.9
U+8E3B.0	fTotalStrokes	 16
U+8E3B.0	fCangjie	RMBBR
U+8E3B.0	fFourCorner	67120
U+8E3B.0	fCantonese	gwaa1
U+8E3B.0	fMandarin	gua1
U+8E3C.0	fUTF8	踼
U+8E3C.0	fCNS11643	2-5A6F
U+8E3C.0	fBig5	EBC3
U+8E3C.0	fGB	3-6F4E
U+8E3C.0	fR/S	 157.9
U+8E3C.0	fTotalStrokes	 16
U+8E3C.0	fCangjie	RMAMH
U+8E3C.0	fFourCorner	66127
U+8E3C.0	fHakka	 tong5; tong3; tet7; tiak7
U+8E3C.0	fCantonese	dong6 (also); tong4
U+8E3C.0	fMandarin	tang2; dang4; tang1; tang3
U+8E3C.0	fEnglish	 fall flat fall on the face
U+8E3D.0	fUTF8	踽
U+8E3D.0	fCNS11643	2-5A70
U+8E3D.0	fBig5	EBC4
U+8E3D.0	fGB	0-7561
U+8E3D.0	fR/S	 157.9
U+8E3D.0	fTotalStrokes	 16
U+8E3D.0	fCangjie	RMHLB
U+8E3D.0	fFourCorner	62127
U+8E3D.0	fHakka	  gi3;  ki3
U+8E3D.0	fCantonese	geoi2
U+8E3D.0	fMandarin	ju3; qu3
U+8E3D.0	fEnglish	 walk alone; self-reliant
U+8E3E.0	fUTF8	踾
U+8E3E.0	fCNS11643	2-5A6A
U+8E3E.0	fBig5	EBBE
U+8E3E.0	fGB	3-703B
U+8E3E.0	fR/S	 157.9
U+8E3E.0	fTotalStrokes	 16
U+8E3E.0	fCangjie	RMMRW
U+8E3E.0	fFourCorner	61166
U+8E3E.0	fCantonese	fuk1; bik1; pik1
U+8E3E.0	fMandarin	fu2; bi1; bi4
U+8E3F.0	fUTF8	踿
U+8E3F.0	fCNS11643	2-5A73
U+8E3F.0	fBig5	EBC7
U+8E3F.0	fGB	5-6F2B
U+8E3F.0	fR/S	 157.9
U+8E3F.0	fTotalStrokes	 16
U+8E3F.0	fCangjie	RMHDF
U+8E3F.0	fFourCorner	69180
U+8E3F.0	fCantonese	zuk1
U+8E3F.0	fMandarin	zu2
U+8E40.0	fUTF8	蹀
U+8E40.0	fCNS11643	2-5A6C
U+8E40.0	fBig5	EBC0
U+8E40.0	fGB	0-755E
U+8E40.0	fR/S	 157.9
U+8E40.0	fTotalStrokes	 16
U+8E40.0	fCangjie	RMPTD
U+8E40.0	fFourCorner	64194
U+8E40.0	fHakka	 tiap8; dia1; dia2
U+8E40.0	fCantonese	dip6
U+8E40.0	fMandarin	die2; tie1
U+8E40.0	fEnglish	 stamp one's foot
U+8E41.0	fUTF8	蹁
U+8E41.0	fCNS11643	2-5A71
U+8E41.0	fBig5	EBC5
U+8E41.0	fGB	0-7564
U+8E41.0	fR/S	 157.9
U+8E41.0	fTotalStrokes	 16
U+8E41.0	fCangjie	RMHSB
U+8E41.0	fFourCorner	67127
U+8E41.0	fHakka	  pien1; pen1
U+8E41.0	fCantonese	pin4; pin1 (also)
U+8E41.0	fMandarin	pian2; bian1
U+8E41.0	fEnglish	 walk with a limp; walking unsteadily; limping
U+8E42.0	fUTF8	蹂
U+8E42.0	fCNS11643	1-724E
U+8E42.0	fBig5	BFE4
U+8E42.0	fGB	0-7565
U+8E42.0	fR/S	 157.9
U+8E42.0	fTotalStrokes	 16
U+8E42.0	fCangjie	RMNHD
U+8E42.0	fFourCorner	67194
U+8E42.0	fHakka	 ju2; ngiu3; riu2; jiu2
U+8E42.0	fCantonese	jau4; jau5
U+8E42.0	fMandarin	rou2; rou3; rou4
U+8E42.0	fEnglish	[1] tread upon; trample [2] tread out grain
U+8E43.0	fUTF8	蹃
U+8E43.0	fCNS11643	3-527C
U+8E43.0	fGB	3-7027
U+8E43.0	fR/S	  157.9
U+8E43.0	fTotalStrokes	 16
U+8E43.0	fCangjie	RMTKR
U+8E43.0	fFourCorner	64160
U+8E43.0	fHakka	 nok7
U+8E43.0	fCantonese	noek6; loek6 (also)
U+8E43.0	fMandarin	nuo4
U+8E43.0	fEnglish	[1] step firmly [2] thread down
U+8E44.0	fUTF8	蹄
U+8E44.0	fCNS11643	1-724B
U+8E44.0	fBig5	BFE1
U+8E44.0	fGB	0-4C63
U+8E44.0	fR/S	 157.9
U+8E44.0	fTotalStrokes	 16
U+8E44.0	fCangjie	RMYBB
U+8E44.0	fFourCorner	60127
U+8E44.0	fHakka	  tai2;  ti2; tai3
U+8E44.0	fCantonese	tai4; tai2 (also); dai6
U+8E44.0	fMandarin	ti2; di4
U+8E44.0	fEnglish	[1] hoof (zoology) [2] leg of beasts
U+8E45.0	fUTF8	蹅
U+8E45.0	fCNS11643	2-5A6D
U+8E45.0	fBig5	EBC1
U+8E45.0	fGB	3-703A
U+8E45.0	fR/S	 157.9
U+8E45.0	fTotalStrokes	 16
U+8E45.0	fCangjie	RMDAM
U+8E45.0	fFourCorner	64116
U+8E45.0	fHakka	 za1
U+8E45.0	fCantonese	caa1 (also); caa5; zaa1
U+8E45.0	fMandarin	cha3
U+8E45.0	fEnglish	 tread on; walk through
U+8E46.0	fUTF8	蹆
U+8E46.0	fCNS11643	3-5654
U+8E46.0	fGB	5-6F2E
U+8E46.0	fR/S	  157.10
U+8E46.0	fTotalStrokes	 17
U+8E46.0	fCangjie	RMYAV
U+8E46.0	fFourCorner	67133
U+8E46.0	fCantonese	tan3
U+8E46.0	fMandarin	tui3
U+8E46.0	fEnglish	move in a straight line
U+8E47.0	fUTF8	蹇
U+8E47.0	fCNS11643	2-5F6B
U+8E47.0	fBig5	EEBF
U+8E47.0	fGB	0-653F
U+8E47.0	fR/S	 157.10
U+8E47.0	fTotalStrokes	 17
U+8E47.0	fCangjie	JTCO
U+8E47.0	fFourCorner	30801
U+8E47.0	fHakka	 kien3; ken3; kian3; ken5
U+8E47.0	fCantonese	gin2; zin2 (also); hin1
U+8E47.0	fMandarin	jian3; qian1
U+8E47.0	fEnglish	[1] lame; crippled [2] slow [3] difficult; hard [4] haughty
U+8E48.0	fUTF8	蹈
U+8E48.0	fCNS11643	1-755B
U+8E48.0	fBig5	C1D0
U+8E48.0	fGB	0-3538
U+8E48.0	fR/S	 157.10
U+8E48.0	fTotalStrokes	 17
U+8E48.0	fCangjie	RMBHX
U+8E48.0	fFourCorner	62177
U+8E48.0	fHakka	 tau1;  tau5; tap8; tau6; to6; to5
U+8E48.0	fCantonese	dou6; tou1 (also)
U+8E48.0	fMandarin	dao3; dao4
U+8E48.0	fEnglish	[1] stamp one's feet; tread; step [2] follow; pursue
U+8E49.0	fUTF8	蹉
U+8E49.0	fCNS11643	1-7559
U+8E49.0	fBig5	C1CE
U+8E49.0	fGB	0-7563
U+8E49.0	fR/S	 157.10
U+8E49.0	fTotalStrokes	 17
U+8E49.0	fCangjie	RMTQM
U+8E49.0	fFourCorner	68111
U+8E49.0	fHakka	 ca1;  co2; co1; co5
U+8E49.0	fCantonese	co1
U+8E49.0	fMandarin	cuo1; ci1; chuai4
U+8E49.0	fEnglish	 error; mistake; slip; failure
U+8E4A.0	fUTF8	蹊
U+8E4A.0	fCNS11643	1-755C
U+8E4A.0	fBig5	C1D1
U+8E4A.0	fGB	0-7568
U+8E4A.0	fR/S	 157.10
U+8E4A.0	fTotalStrokes	 17
U+8E4A.0	fCangjie	RMBVK
U+8E4A.0	fFourCorner	62134
U+8E4A.0	fHakka	  hi1; hi2; kai1
U+8E4A.0	fCantonese	hai4; kai1 (also); kei1; hei1
U+8E4A.0	fMandarin	xi1; qi1; xi4; xi2
U+8E4A.0	fEnglish	[1] path; footpath [2] trample; tread
U+8E4B.0	fUTF8	蹋
U+8E4B.0	fCNS11643	1-755A
U+8E4B.0	fBig5	C1CF
U+8E4B.0	fGB	0-4C23
U+8E4B.0	fR/S	 157.10
U+8E4B.0	fTotalStrokes	 17
U+8E4B.0	fCangjie	RMASM
U+8E4B.0	fFourCorner	66127
U+8E4B.0	fHakka	 tap8;  tap7; {lap7}; tat7
U+8E4B.0	fCantonese	daap6; taap3 (also)
U+8E4B.0	fMandarin	ta4; da1
U+8E4B.0	fEnglish	 step on; tread on
U+8E4C.0	fUTF8	蹌
U+8E4C.0	fCNS11643	2-5F6A
U+8E4C.0	fBig5	EEBE
U+8E4C.0	fGB	1-7544
U+8E4C.0	fR/S	 157.10
U+8E4C.0	fTotalStrokes	 17
U+8E4C.0	fCangjie	RMOIR
U+8E4C.0	fFourCorner	68167
U+8E4C.0	fHakka	 cong1; cong5; cong6; ciong1
U+8E4C.0	fCantonese	coeng1; coeng3
U+8E4C.0	fMandarin	qiang1; qiang2
U+8E4C.0	fEnglish	walk rapidly
U+8E4C.1	fUTF8	蹌
U+8E4C.1	fCNS11643	2-5F6A
U+8E4C.1	fBig5	EEBE
U+8E4C.1	fGB	1-7544
U+8E4C.1	fR/S	 157.10
U+8E4C.1	fTotalStrokes	 17
U+8E4C.1	fCangjie	RMOIR
U+8E4C.1	fFourCorner	68167
U+8E4C.1	fMandarin	qiang4
U+8E4C.1	fEnglish	walking unsteadily; limping
U+8E4D.0	fUTF8	蹍
U+8E4D.0	fCNS11643	2-5F67
U+8E4D.0	fBig5	EEBB
U+8E4D.0	fGB	3-7048
U+8E4D.0	fR/S	 157.10
U+8E4D.0	fTotalStrokes	 17
U+8E4D.0	fCangjie	RMSTV
U+8E4D.0	fFourCorner	67132
U+8E4D.0	fHakka	 zhan3; zan3; zen3
U+8E4D.0	fCantonese	nin5; zin2 (also); lin5 (also)
U+8E4D.0	fMandarin	zhan3; nian3
U+8E4E.0	fUTF8	蹎
U+8E4E.0	fCNS11643	2-5F66
U+8E4E.0	fBig5	EEBA
U+8E4E.0	fGB	5-6F30
U+8E4E.0	fR/S	 157.10
U+8E4E.0	fTotalStrokes	 17
U+8E4E.0	fCangjie	RMJBC
U+8E4E.0	fFourCorner	64181
U+8E4E.0	fHakka	  dien1
U+8E4E.0	fCantonese	din1
U+8E4E.0	fMandarin	dian1
U+8E4E.0	fEnglish	 stumble; trip
U+8E4F.0	fUTF8	蹏
U+8E4F.0	fCNS11643	3-5655
U+8E4F.0	fGB	E-3F3E
U+8E4F.0	fR/S	 157.10
U+8E4F.0	fTotalStrokes	 17
U+8E4F.0	fCangjie	RMHYU
U+8E4F.0	fFourCorner	62110
U+8E4F.0	fHakka	  tai2;  ti2
U+8E4F.0	fCantonese	tai4; dai6
U+8E4F.0	fMandarin	ti2; di4
U+8E4F.0	fEnglish	 hoof leg of pork little witch a horse
U+8E50.0	fUTF8	蹐
U+8E50.0	fCNS11643	2-5F69
U+8E50.0	fBig5	EEBD
U+8E50.0	fGB	3-7046
U+8E50.0	fR/S	 157.10
U+8E50.0	fTotalStrokes	 17
U+8E50.0	fCangjie	RMFCB
U+8E50.0	fFourCorner	61127
U+8E50.0	fHakka	  zit7
U+8E50.0	fCantonese	zik3; zek3 (also)
U+8E50.0	fMandarin	ji2
U+8E50.0	fEnglish	 take short steps; walk with small steps; walk daintily
U+8E51.0	fUTF8	蹑
U+8E51.0	fCNS11643	4-592D
U+8E51.0	fGB	0-7566
U+8E51.0	fR/S	 157.10
U+8E51.0	fTotalStrokes	 17; 16
U+8E51.0	fCangjie	RMSJE
U+8E51.0	fFourCorner	61140
U+8E51.0	fHakka	 ngiap7
U+8E51.0	fCantonese	nip6; lip6 (also)
U+8E51.0	fMandarin	nie4
U+8E51.0	fEnglish	simplified form of &#x8EA1;
U+8E52.0	fUTF8	蹒
U+8E52.0	fGB	0-7567
U+8E52.0	fR/S	 157.10
U+8E52.0	fTotalStrokes	 17
U+8E52.0	fCangjie	RMTMB
U+8E52.0	fHakka	 pan2; man2
U+8E52.0	fCantonese	pun4; mun4
U+8E52.0	fMandarin	man2; pan2
U+8E52.0	fEnglish	simplified form of &#x8E63;
U+8E53.0	fUTF8	蹓
U+8E53.0	fCNS11643	2-5F68
U+8E53.0	fBig5	EEBC
U+8E53.0	fGB	3-7045
U+8E53.0	fR/S	 157.10
U+8E53.0	fTotalStrokes	 17
U+8E53.0	fCangjie	RMHHW
U+8E53.0	fFourCorner	67160
U+8E53.0	fCantonese	lau6; lau4
U+8E53.0	fMandarin	liu4; liu1
U+8E53.0	fEnglish	stroll; take walk
U+8E54.0	fUTF8	蹔
U+8E54.0	fCNS11643	2-636F
U+8E54.0	fBig5	F145
U+8E54.0	fGB	E-3F3F
U+8E54.0	fR/S	 157.11
U+8E54.0	fTotalStrokes	 18
U+8E54.0	fCangjie	JLRYO
U+8E54.0	fFourCorner	52801
U+8E54.0	fHakka	 ciam5; ciam6
U+8E54.0	fCantonese	zaam6
U+8E54.0	fMandarin	zhan4; zan4; can2
U+8E54.0	fEnglish	same as $ (temporary; fleeting; ephemeral)
U+8E55.0	fUTF8	蹕
U+8E55.0	fCNS11643	1-7749
U+8E55.0	fBig5	C2DE
U+8E55.0	fGB	1-754F
U+8E55.0	fR/S	 157.11
U+8E55.0	fTotalStrokes	 18
U+8E55.0	fCangjie	RMWTJ
U+8E55.0	fFourCorner	66150
U+8E55.0	fHakka	  bit7
U+8E55.0	fCantonese	bat1
U+8E55.0	fMandarin	bi4
U+8E55.0	fEnglish	[1] clear way; make room for; clear (the emperor's route) of traffic [2] imperial carriage
U+8E56.0	fUTF8	蹖
U+8E56.0	fCNS11643	2-6366
U+8E56.0	fBig5	F0FB
U+8E56.0	fGB	E-3F40
U+8E56.0	fR/S	 157.11
U+8E56.0	fTotalStrokes	 18
U+8E56.0	fCangjie	RMQKX
U+8E56.0	fFourCorner	65177
U+8E56.0	fHakka	 zhung1; zung1
U+8E56.0	fCantonese	zung1
U+8E56.0	fMandarin	chong1
U+8E56.0	fEnglish	 tread on
U+8E57.0	fUTF8	蹗
U+8E57.0	fCNS11643	2-6365
U+8E57.0	fBig5	F0FA
U+8E57.0	fGB	5-6F37
U+8E57.0	fR/S	 157.11
U+8E57.0	fTotalStrokes	 18
U+8E57.0	fCangjie	RMIXP
U+8E57.0	fFourCorner	60111
U+8E57.0	fCantonese	luk6
U+8E57.0	fMandarin	lu4
U+8E58.0	fUTF8	蹘
U+8E58.0	fCNS11643	3-5932
U+8E58.0	fGB	3-704D
U+8E58.0	fR/S	  157.11
U+8E58.0	fTotalStrokes	 18
U+8E58.0	fCangjie	RMSMH
U+8E58.0	fFourCorner	67120
U+8E58.0	fHakka	 liau6
U+8E58.0	fCantonese	laau4; liu1; mau1
U+8E58.0	fMandarin	liao2
U+8E58.0	fEnglish	roost
U+8E59.0	fUTF8	蹙
U+8E59.0	fCNS11643	1-7744
U+8E59.0	fBig5	C2D9
U+8E59.0	fGB	0-753E
U+8E59.0	fR/S	 157.11
U+8E59.0	fTotalStrokes	 18
U+8E59.0	fCangjie	IFRYO
U+8E59.0	fFourCorner	53801
U+8E59.0	fHakka	  cuk7
U+8E59.0	fCantonese	cuk1; zuk1
U+8E59.0	fMandarin	cu4; qi1; zu2
U+8E59.0	fEnglish	[1] urgent; imminent [2] contract; draw together [3] sad; sorrowful; discomposed
U+8E5A.0	fUTF8	蹚
U+8E5A.0	fCNS11643	2-636B
U+8E5A.0	fBig5	F141
U+8E5A.0	fGB	E-3F41
U+8E5A.0	fR/S	 157.11
U+8E5A.0	fTotalStrokes	 18
U+8E5A.0	fCangjie	RMFBG
U+8E5A.0	fFourCorner	69114
U+8E5A.0	fCantonese	tong2 (also); tong1 (also); tong3; tong4
U+8E5A.0	fMandarin	tang1; cheng1; cheng2
U+8E5A.0	fEnglish	[1] tread on; tread [2] walk through mud or water; wade; ford
U+8E5B.0	fUTF8	蹛
U+8E5B.0	fCNS11643	2-636A
U+8E5B.0	fBig5	F140
U+8E5B.0	fGB	3-7039
U+8E5B.0	fR/S	 157.11
U+8E5B.0	fTotalStrokes	 18
U+8E5B.0	fCangjie	RMKPB
U+8E5B.0	fFourCorner	64127
U+8E5B.0	fHakka	 dai5
U+8E5B.0	fCantonese	daai3; zai6
U+8E5B.0	fMandarin	dai4; duo1; zhi4; zhi2
U+8E5C.0	fUTF8	蹜
U+8E5C.0	fCNS11643	2-6362
U+8E5C.0	fBig5	F0F7
U+8E5C.0	fGB	3-704C
U+8E5C.0	fR/S	 157.11
U+8E5C.0	fTotalStrokes	 18
U+8E5C.0	fCangjie	RMJOA
U+8E5C.0	fFourCorner	63161
U+8E5C.0	fHakka	  suk7;  siuk7
U+8E5C.0	fCantonese	suk1
U+8E5C.0	fMandarin	su4; suo1
U+8E5C.0	fEnglish	walk carefully
U+8E5D.0	fUTF8	蹝
U+8E5D.0	fCNS11643	2-636D
U+8E5D.0	fBig5	F143
U+8E5D.0	fGB	3-704A
U+8E5D.0	fR/S	 157.11
U+8E5D.0	fTotalStrokes	 18
U+8E5D.0	fCangjie	RMHOO
U+8E5D.0	fFourCorner	61181
U+8E5D.0	fHakka	  sai3; si3
U+8E5D.0	fCantonese	saai2
U+8E5D.0	fMandarin	xi3
U+8E5D.0	fEnglish	sandal
U+8E5E.0	fUTF8	蹞
U+8E5E.0	fCNS11643	2-6367
U+8E5E.0	fBig5	F0FC
U+8E5E.0	fGB	5-6E7B
U+8E5E.0	fR/S	 157.11
U+8E5E.0	fTotalStrokes	 18
U+8E5E.0	fCangjie	RMPMC
U+8E5E.0	fFourCorner	61186
U+8E5E.0	fCantonese	kwai2
U+8E5E.0	fMandarin	kui3
U+8E5F.0	fUTF8	蹟
U+8E5F.0	fCNS11643	1-7748
U+8E5F.0	fBig5	C2DD
U+8E5F.0	fGB	E-3F42
U+8E5F.0	fR/S	 157.11
U+8E5F.0	fTotalStrokes	 18
U+8E5F.0	fCangjie	RMQMC
U+8E5F.0	fFourCorner	65186
U+8E5F.0	fHakka	 ziak7; zit7
U+8E5F.0	fCantonese	zik1
U+8E5F.0	fMandarin	ji1
U+8E5F.0	fEnglish	 traces; footprints
U+8E60.0	fUTF8	蹠
U+8E60.0	fCNS11643	2-6364
U+8E60.0	fBig5	F0F9
U+8E60.0	fGB	E-3F43
U+8E60.0	fR/S	 157.11
U+8E60.0	fTotalStrokes	 18
U+8E60.0	fCangjie	RMITF
U+8E60.0	fFourCorner	60137
U+8E60.0	fHakka	 chak7; zak7; zhak7
U+8E60.0	fCantonese	zik3; zek3 (also)
U+8E60.0	fMandarin	zhi2; zhu4; zhuo2
U+8E60.0	fEnglish	[1] step on; tread on [2] the sole (of the foot)
U+8E61.0	fUTF8	蹡
U+8E61.0	fCNS11643	2-636C
U+8E61.0	fBig5	F142
U+8E61.0	fGB	5-6F2D
U+8E61.0	fR/S	 157.11
U+8E61.0	fTotalStrokes	 18
U+8E61.0	fCangjie	RMVMI
U+8E61.0	fFourCorner	62143
U+8E61.0	fHakka	 ziong1;  cong1
U+8E61.0	fCantonese	coeng1; coeng3
U+8E61.0	fMandarin	qiang1; qiang4
U+8E61.0	fEnglish	walking; in motion
U+8E61.1	fUTF8	蹡
U+8E61.1	fCNS11643	2-636C
U+8E61.1	fBig5	F142
U+8E61.1	fGB	5-6F2D
U+8E61.1	fR/S	 157.11
U+8E61.1	fTotalStrokes	 18
U+8E61.1	fCangjie	RMVMI
U+8E61.1	fFourCorner	62143
U+8E61.1	fMandarin	qiang4qiang1}
U+8E61.1	fEnglish	 limp
U+8E62.0	fUTF8	蹢
U+8E62.0	fCNS11643	2-6363
U+8E62.0	fBig5	F0F8
U+8E62.0	fGB	3-704B
U+8E62.0	fR/S	 157.11
U+8E62.0	fTotalStrokes	 18
U+8E62.0	fCangjie	RMYCB
U+8E62.0	fFourCorner	60127
U+8E62.0	fHakka	  dit7;  zak7
U+8E62.0	fCantonese	zaak6; dik1; zik6
U+8E62.0	fMandarin	zhi2; zhe2
U+8E62.0	fEnglish	 falter; hesitate
U+8E62.1	fUTF8	蹢
U+8E62.1	fCNS11643	2-6363
U+8E62.1	fBig5	F0F8
U+8E62.1	fGB	3-704B
U+8E62.1	fR/S	 157.11
U+8E62.1	fTotalStrokes	 18
U+8E62.1	fCangjie	RMYCB
U+8E62.1	fFourCorner	60127
U+8E62.1	fMandarin	di2
U+8E62.1	fEnglish	 hoof
U+8E63.0	fUTF8	蹣
U+8E63.0	fCNS11643	1-7745
U+8E63.0	fBig5	C2DA
U+8E63.0	fGB	1-7567
U+8E63.0	fR/S	 157.11
U+8E63.0	fTotalStrokes	 18
U+8E63.0	fCangjie	RMTLB
U+8E63.0	fFourCorner	64127
U+8E63.0	fHakka	 pan2; man2
U+8E63.0	fCantonese	mun4 (also); pun4
U+8E63.0	fMandarin	pan2; man2; liang3
U+8E63.0	fEnglish	[1] jump over [2] limp
U+8E64.0	fUTF8	蹤
U+8E64.0	fCNS11643	1-7747
U+8E64.0	fBig5	C2DC
U+8E64.0	fGB	E-3F44
U+8E64.0	fR/S	 157.11
U+8E64.0	fTotalStrokes	 18
U+8E64.0	fCangjie	RMHOO
U+8E64.0	fFourCorner	68181
U+8E64.0	fHakka	 zung1; ziung1
U+8E64.0	fCantonese	zung1
U+8E64.0	fMandarin	zong1
U+8E64.0	fEnglish	[1] footprint; traces; track; vestige [2] follow tracks of; keep track of; trail
U+8E65.0	fUTF8	蹥
U+8E65.0	fCNS11643	2-6368
U+8E65.0	fBig5	F0FD
U+8E65.0	fGB	5-6E76
U+8E65.0	fR/S	 157.11
U+8E65.0	fTotalStrokes	 18
U+8E65.0	fCangjie	RMYJJ
U+8E65.0	fFourCorner	65130
U+8E65.0	fCantonese	lin4
U+8E65.0	fMandarin	lian2
U+8E66.0	fUTF8	蹦
U+8E66.0	fCNS11643	1-7746
U+8E66.0	fBig5	C2DB
U+8E66.0	fGB	0-3144
U+8E66.0	fR/S	 157.11
U+8E66.0	fTotalStrokes	 18
U+8E66.0	fCangjie	RMUBB
U+8E66.0	fFourCorner	62127
U+8E66.0	fHakka	 bang1
U+8E66.0	fCantonese	bang1; baang6
U+8E66.0	fMandarin	beng4
U+8E66.0	fEnglish	 hop; leap; jump; skip; caper; trip
U+8E67.0	fUTF8	蹧
U+8E67.0	fCNS11643	2-6369
U+8E67.0	fBig5	F0FE
U+8E67.0	fGB	E-3F45
U+8E67.0	fR/S	 157.11
U+8E67.0	fTotalStrokes	 18
U+8E67.0	fCangjie	RMTWA
U+8E67.0	fFourCorner	64166
U+8E67.0	fHakka	 zau1
U+8E67.0	fCantonese	zou1
U+8E67.0	fMandarin	zao1
U+8E67.0	fEnglish	 spoil; ruin
U+8E68.0	fUTF8	蹨
U+8E68.0	fCNS11643	3-5B3A
U+8E68.0	fGB	3-704F
U+8E68.0	fR/S	  157.12
U+8E68.0	fTotalStrokes	 19
U+8E68.0	fCangjie	RMBKF
U+8E68.0	fFourCorner	63130
U+8E68.0	fHakka	 ngian3
U+8E68.0	fCantonese	nim3; lim3 (also)
U+8E68.0	fMandarin	nian3
U+8E68.0	fEnglish	stand on tiptoe
U+8E69.0	fUTF8	蹩
U+8E69.0	fCNS11643	2-636E
U+8E69.0	fBig5	F144
U+8E69.0	fGB	0-753F
U+8E69.0	fR/S	 157.12
U+8E69.0	fTotalStrokes	 19
U+8E69.0	fCangjie	FKRYO
U+8E69.0	fFourCorner	97301
U+8E69.0	fHakka	 pet7; piet8
U+8E69.0	fCantonese	bit6; pit3
U+8E69.0	fMandarin	bie2; pie1
U+8E69.0	fEnglish	 limp
U+8E6A.0	fUTF8	蹪
U+8E6A.0	fCNS11643	2-673E
U+8E6A.0	fBig5	F352
U+8E6A.0	fGB	3-703F
U+8E6A.0	fR/S	 157.12
U+8E6A.0	fTotalStrokes	 19
U+8E6A.0	fCangjie	RMLMC
U+8E6A.0	fFourCorner	65186
U+8E6A.0	fCantonese	teoi4
U+8E6A.0	fMandarin	tui2
U+8E6B.0	fUTF8	蹫
U+8E6B.0	fCNS11643	3-5B39
U+8E6B.0	fGB	5-6F43
U+8E6B.0	fR/S	 157.12
U+8E6B.0	fTotalStrokes	 19
U+8E6B.0	fCangjie	RMNHB
U+8E6B.0	fFourCorner	67120
U+8E6B.0	fHakka	 git7
U+8E6B.0	fCantonese	gwat1
U+8E6B.0	fMandarin	ju2
U+8E6B.0	fEnglish	[1] lame [2] run about wildly
U+8E6C.0	fUTF8	蹬
U+8E6C.0	fCNS11643	1-792B
U+8E6C.0	fBig5	C3DE
U+8E6C.0	fGB	0-3545
U+8E6C.0	fR/S	 157.12
U+8E6C.0	fTotalStrokes	 19
U+8E6C.0	fCangjie	RMNOT
U+8E6C.0	fFourCorner	62118
U+8E6C.0	fHakka	 den5;  dem3;  ten5; ten3
U+8E6C.0	fCantonese	dang1; dang6
U+8E6C.0	fMandarin	deng4; deng1
U+8E6C.0	fEnglish	[1] step on; tread on [2] lose energy; deprived of power or influence
U+8E6D.0	fUTF8	蹭
U+8E6D.0	fCNS11643	2-673B
U+8E6D.0	fBig5	F34F
U+8E6D.0	fGB	0-3264
U+8E6D.0	fR/S	 157.12
U+8E6D.0	fTotalStrokes	 19
U+8E6D.0	fCangjie	RMCWA
U+8E6D.0	fFourCorner	68166
U+8E6D.0	fHakka	  cen5; can5
U+8E6D.0	fCantonese	sang3; cang3
U+8E6D.0	fMandarin	ceng4; ceng2
U+8E6D.0	fEnglish	[1] deprived of power [2] stroll [3] protract
U+8E6E.0	fUTF8	蹮
U+8E6E.0	fCNS11643	4-6152
U+8E6E.0	fGB	5-6F3D
U+8E6E.0	fR/S	 157.11; 157.12
U+8E6E.0	fTotalStrokes	 18; 19
U+8E6E.0	fCangjie	RMMWU
U+8E6E.0	fFourCorner	61110
U+8E6E.0	fHakka	  cien1;  cin1; sien1
U+8E6E.0	fCantonese	sin1
U+8E6E.0	fMandarin	xian1
U+8E6E.0	fEnglish	 walk round and round
U+8E6F.0	fUTF8	蹯
U+8E6F.0	fCNS11643	2-673F
U+8E6F.0	fBig5	F353
U+8E6F.0	fGB	0-756C
U+8E6F.0	fR/S	 157.12
U+8E6F.0	fTotalStrokes	 19
U+8E6F.0	fCangjie	RMHDW
U+8E6F.0	fFourCorner	62169
U+8E6F.0	fHakka	 fan1; fan2; pan1; pan2
U+8E6F.0	fCantonese	faan4
U+8E6F.0	fMandarin	fan2; fen2; fan1
U+8E6F.0	fEnglish	paws
U+8E70.0	fUTF8	蹰
U+8E70.0	fCNS11643	3-5B3B
U+8E70.0	fGB	0-7569
U+8E70.0	fR/S	 157.12
U+8E70.0	fTotalStrokes	 19
U+8E70.0	fCangjie	RMMMI
U+8E70.0	fFourCorner	61140
U+8E70.0	fCantonese	cyu4
U+8E70.0	fMandarin	chu2
U+8E70.0	fEnglish	same as $
U+8E71.0	fUTF8	蹱
U+8E71.0	fCNS11643	4-614F
U+8E71.0	fGB	5-6F40
U+8E71.0	fR/S	  157.12
U+8E71.0	fTotalStrokes	 19
U+8E71.0	fCangjie	RMYTG
U+8E71.0	fFourCorner	60114
U+8E71.0	fHakka	 zhung1; zung1
U+8E71.0	fCantonese	zung1
U+8E71.0	fMandarin	zhong1
U+8E71.0	fEnglish	 stagger along fall
U+8E72.0	fUTF8	蹲
U+8E72.0	fCNS11643	1-7928
U+8E72.0	fBig5	C3DB
U+8E72.0	fGB	0-3657
U+8E72.0	fR/S	 157.12
U+8E72.0	fTotalStrokes	 19
U+8E72.0	fCangjie	RMTWI
U+8E72.0	fFourCorner	68146
U+8E72.0	fHakka	 cun2; zun1; cun3
U+8E72.0	fCantonese	cyun4; deon1; zeon1 (also)
U+8E72.0	fMandarin	dun1; cun2; cuan2; dun2; cun1; zun1; qun1; cun3; zun3
U+8E72.0	fEnglish	[1] squat; crouch [2] stay
U+8E73.0	fUTF8	蹳
U+8E73.0	fCNS11643	2-673D
U+8E73.0	fBig5	F351
U+8E73.0	fGB	3-6F6B
U+8E73.0	fR/S	 157.12
U+8E73.0	fTotalStrokes	 19
U+8E73.0	fCangjie	RMNOE
U+8E73.0	fFourCorner	62147
U+8E73.0	fHakka	 boi3; {boi1}
U+8E73.0	fCantonese	but6; but3
U+8E73.0	fMandarin	bo1; ba2
U+8E74.0	fUTF8	蹴
U+8E74.0	fCNS11643	1-792D
U+8E74.0	fBig5	C3E0
U+8E74.0	fGB	0-756D
U+8E74.0	fR/S	 157.12
U+8E74.0	fTotalStrokes	 19
U+8E74.0	fCangjie	RMYFU
U+8E74.0	fFourCorner	63114
U+8E74.0	fHakka	 cuk7; cok7; chok7
U+8E74.0	fCantonese	cuk1; zuk1
U+8E74.0	fMandarin	cu4; zu2
U+8E74.0	fEnglish	[1] kick [2] tread on [3] respectful
U+8E75.0	fUTF8	蹵
U+8E75.0	fCNS11643	3-5B3C
U+8E75.0	fGB	E-3F46
U+8E75.0	fR/S	 157.12
U+8E75.0	fTotalStrokes	 19
U+8E75.0	fCangjie	YURYO
U+8E75.0	fFourCorner	03800
U+8E75.0	fHakka	 cuk7
U+8E75.0	fCantonese	cuk1
U+8E75.0	fMandarin	zu2; cu4
U+8E75.0	fEnglish	[1] tread [2] kick
U+8E76.0	fUTF8	蹶
U+8E76.0	fCNS11643	1-792A
U+8E76.0	fBig5	C3DD
U+8E76.0	fGB	0-756A
U+8E76.0	fR/S	 157.12
U+8E76.0	fTotalStrokes	 19
U+8E76.0	fCangjie	RMMTO
U+8E76.0	fFourCorner	61182
U+8E76.0	fHakka	 kiet7;  kui3; kiet8; ket7; ket8; k'wui5
U+8E76.0	fCantonese	kyut3; gwai3
U+8E76.0	fMandarin	jue2; gui4; jue1
U+8E76.0	fEnglish	[1] stumble and fall [2] tread; trample [3] overturn; overthrow [4] daunt; frustrate; suffer a setback
U+8E76.1	fUTF8	蹶
U+8E76.1	fCNS11643	1-792A
U+8E76.1	fBig5	C3DD
U+8E76.1	fGB	0-756A
U+8E76.1	fR/S	 157.12
U+8E76.1	fTotalStrokes	 19
U+8E76.1	fCangjie	RMMTO
U+8E76.1	fFourCorner	61182
U+8E76.1	fMandarin	jue3
U+8E76.1	fEnglish	 kick backward
U+8E77.0	fUTF8	蹷
U+8E77.0	fCNS11643	3-5B3D
U+8E77.0	fGB	5-3072
U+8E77.0	fR/S	 157.12
U+8E77.0	fTotalStrokes	 19
U+8E77.0	fCangjie	MORYO
U+8E77.0	fFourCorner	71800
U+8E77.0	fCantonese	kyut3
U+8E77.0	fMandarin	jue2
U+8E78.0	fUTF8	蹸
U+8E78.0	fCNS11643	2-673C
U+8E78.0	fBig5	F350
U+8E78.0	fGB	5-6F41
U+8E78.0	fR/S	 157.12
U+8E78.0	fTotalStrokes	 19
U+8E78.0	fCangjie	RMFDQ
U+8E78.0	fFourCorner	69159
U+8E78.0	fHakka	 nin5; lin5
U+8E78.0	fCantonese	leon6
U+8E78.0	fMandarin	lin4; lin2
U+8E79.0	fUTF8	蹹
U+8E79.0	fCNS11643	4-6154
U+8E79.0	fGB	5-6F3F
U+8E79.0	fR/S	  157.12
U+8E79.0	fTotalStrokes	 19
U+8E79.0	fCangjie	RMORM
U+8E79.0	fFourCorner	68120
U+8E79.0	fHakka	 tap7; tap8; {lap7}
U+8E79.0	fCantonese	daap6
U+8E79.0	fMandarin	ta4
U+8E79.0	fEnglish	 tread on stamp walk
U+8E7A.0	fUTF8	蹺
U+8E7A.0	fCNS11643	1-792C
U+8E7A.0	fBig5	C3DF
U+8E7A.0	fGB	1-754E
U+8E7A.0	fR/S	 157.12
U+8E7A.0	fTotalStrokes	 19
U+8E7A.0	fCangjie	RMGGU
U+8E7A.0	fFourCorner	64111
U+8E7A.0	fHakka	 kiau1; ngiau3
U+8E7A.0	fCantonese	hiu1; kiu1 (also); kiu2; kiu5; kiu3
U+8E7A.0	fMandarin	qiao1; qiao4
U+8E7A.0	fEnglish	[1] on tiptoe [2] stilts
U+8E7B.0	fUTF8	蹻
U+8E7B.0	fCNS11643	2-6740
U+8E7B.0	fBig5	F354
U+8E7B.0	fGB	E-3F47
U+8E7B.0	fR/S	 157.12
U+8E7B.0	fTotalStrokes	 19
U+8E7B.0	fCangjie	RMHKB
U+8E7B.0	fFourCorner	62127
U+8E7B.0	fHakka	 kiau1;  kiau2; kau1
U+8E7B.0	fCantonese	giu2; hiu1; kiu5; goek3; kiu1 (also); kiu2; kiu3; koek6
U+8E7B.0	fMandarin	qiao1; jue2; jie3
U+8E7B.0	fEnglish	[1] same as $ [2] stilts
U+8E7B.1	fUTF8	蹻
U+8E7B.1	fCNS11643	2-6740
U+8E7B.1	fBig5	F354
U+8E7B.1	fGB	E-3F47
U+8E7B.1	fR/S	 157.12
U+8E7B.1	fTotalStrokes	 19
U+8E7B.1	fCangjie	RMHKB
U+8E7B.1	fFourCorner	62127
U+8E7B.1	fMandarin	jiao3
U+8E7B.1	fEnglish	[1] strong; brave [2] haughty; overbearing
U+8E7C.0	fUTF8	蹼
U+8E7C.0	fCNS11643	1-7927
U+8E7C.0	fBig5	C3DA
U+8E7C.0	fGB	0-756B
U+8E7C.0	fR/S	 157.12
U+8E7C.0	fTotalStrokes	 19
U+8E7C.0	fCangjie	RMTCO
U+8E7C.0	fFourCorner	62134
U+8E7C.0	fHakka	 buk7; puk7
U+8E7C.0	fCantonese	buk6; puk3 (also); pok3; buk1
U+8E7C.0	fMandarin	pu2; pu3; bu3
U+8E7C.0	fEnglish	 webbed feet of waterfowl; webs on the feet of water fowls
U+8E7D.0	fUTF8	蹽
U+8E7D.0	fGB	3-704E
U+8E7D.0	fR/S	 157.12
U+8E7D.0	fTotalStrokes	 19
U+8E7D.0	fCangjie	RMKCF
U+8E7D.0	fCantonese	liu1; liu4 (also)
U+8E7D.0	fMandarin	liao1
U+8E7E.0	fUTF8	蹾
U+8E7E.0	fGB	3-7050
U+8E7E.0	fR/S	 157.12
U+8E7E.0	fTotalStrokes	 19
U+8E7E.0	fCangjie	RMYDK
U+8E7E.0	fCantonese	dan1; dan3
U+8E7E.0	fMandarin	dun1
U+8E7E.0	fEnglish	 squat crouch
U+8E7F.0	fUTF8	蹿
U+8E7F.0	fGB	0-345A
U+8E7F.0	fR/S	 157.12
U+8E7F.0	fTotalStrokes	 19
U+8E7F.0	fCangjie	RMJCL
U+8E7F.0	fCantonese	cyun1
U+8E7F.0	fMandarin	cuan1
U+8E7F.0	fEnglish	simplified form of &#x8EA5;
U+8E80.0	fUTF8	躀
U+8E80.0	fGB	1-7C52
U+8E80.0	fR/S	 157.11
U+8E80.0	fTotalStrokes	 18
U+8E80.0	fCangjie	RMWJC
U+8E80.0	fCantonese	gwaan3; kwaang3; kwong3
U+8E80.0	fMandarin	kuang4
U+8E80.0	fEnglish	 stumble over; fall (physically)
U+8E81.0	fUTF8	躁
U+8E81.0	fCNS11643	1-7A47
U+8E81.0	fBig5	C4BC
U+8E81.0	fGB	0-546A
U+8E81.0	fR/S	 157.13
U+8E81.0	fTotalStrokes	 20
U+8E81.0	fCangjie	RMRRD
U+8E81.0	fFourCorner	66194
U+8E81.0	fHakka	  cau5
U+8E81.0	fCantonese	cou3
U+8E81.0	fMandarin	zao4
U+8E81.0	fEnglish	[1] irritable; hot-tempered [2] restless; uneasy [3] rashness
U+8E82.0	fUTF8	躂
U+8E82.0	fCNS11643	1-7A49
U+8E82.0	fBig5	C4BE
U+8E82.0	fGB	3-6F6C
U+8E82.0	fR/S	 157.13
U+8E82.0	fTotalStrokes	 20
U+8E82.0	fCangjie	RMYGQ
U+8E82.0	fFourCorner	64135
U+8E82.0	fHakka	 tat7; dat7; tiat7; tat8
U+8E82.0	fCantonese	taat3; daat3
U+8E82.0	fMandarin	ta4; da5
U+8E82.0	fEnglish	 stumble; slip
U+8E83.0	fUTF8	躃
U+8E83.0	fCNS11643	3-5D37
U+8E83.0	fGB	3-7055
U+8E83.0	fR/S	  157.13
U+8E83.0	fTotalStrokes	 20
U+8E83.0	fCangjie	RMSRJ
U+8E83.0	fFourCorner	60140
U+8E83.0	fHakka	  pit7
U+8E83.0	fCantonese	bik1
U+8E83.0	fMandarin	bi4
U+8E83.0	fEnglish	same as@
U+8E84.0	fUTF8	躄
U+8E84.0	fCNS11643	2-6A27
U+8E84.0	fBig5	F4D9
U+8E84.0	fGB	3-6F4B
U+8E84.0	fR/S	 157.13
U+8E84.0	fTotalStrokes	 20
U+8E84.0	fCangjie	SJRYO
U+8E84.0	fFourCorner	70801
U+8E84.0	fHakka	  pit7
U+8E84.0	fCantonese	bik1; pik1
U+8E84.0	fMandarin	bi4; pi4
U+8E84.0	fEnglish	[1] crippled in both legs; having both legs disabled [2] read bie2
U+8E85.0	fUTF8	躅
U+8E85.0	fCNS11643	1-7A48
U+8E85.0	fBig5	C4BD
U+8E85.0	fGB	0-756E
U+8E85.0	fR/S	 157.13
U+8E85.0	fTotalStrokes	 20
U+8E85.0	fCangjie	RMWLI
U+8E85.0	fFourCorner	66127
U+8E85.0	fHakka	 cuk7; ch'uk7; cok7; chuk7
U+8E85.0	fCantonese	zuk6; zok6
U+8E85.0	fMandarin	zhu2; zhuo2
U+8E85.0	fEnglish	 hesitate; falter
U+8E86.0	fUTF8	躆
U+8E86.0	fCNS11643	2-6A25
U+8E86.0	fBig5	F4D7
U+8E86.0	fGB	3-7052
U+8E86.0	fR/S	 157.13
U+8E86.0	fTotalStrokes	 20
U+8E86.0	fCangjie	RMYPO
U+8E86.0	fFourCorner	61132
U+8E86.0	fHakka	 gi3
U+8E86.0	fCantonese	geoi3
U+8E86.0	fMandarin	ju4; ji3
U+8E87.0	fUTF8	躇
U+8E87.0	fCNS11643	1-7929
U+8E87.0	fBig5	C3DC
U+8E87.0	fGB	0-3379
U+8E87.0	fR/S	 157.13
U+8E87.0	fTotalStrokes	 20
U+8E87.0	fCangjie	RMTJA
U+8E87.0	fFourCorner	64164
U+8E87.0	fHakka	 chu2; cu2; chiu2
U+8E87.0	fCantonese	cyu4; coek3; cyu1
U+8E87.0	fMandarin	chu2; chuo4; zhu4
U+8E87.0	fEnglish	 hesitate; falter; be undecided
U+8E88.0	fUTF8	躈
U+8E88.0	fCNS11643	2-6A26
U+8E88.0	fBig5	F4D8
U+8E88.0	fGB	3-7053
U+8E88.0	fR/S	 157.13
U+8E88.0	fTotalStrokes	 20
U+8E88.0	fCangjie	RMHSK
U+8E88.0	fFourCorner	68140
U+8E88.0	fCantonese	kiu3
U+8E88.0	fMandarin	qiao4
U+8E89.0	fUTF8	躉
U+8E89.0	fCNS11643	1-7A46
U+8E89.0	fBig5	C4BB
U+8E89.0	fGB	1-753B
U+8E89.0	fR/S	 157.13; 157.12
U+8E89.0	fTotalStrokes	 20
U+8E89.0	fCangjie	TWBO
U+8E89.0	fFourCorner	44801
U+8E89.0	fHakka	  dun3
U+8E89.0	fCantonese	dan2
U+8E89.0	fMandarin	dun3
U+8E89.0	fEnglish	[1] a whole batch or amount [2] sell or buy wholesale
U+8E8A.0	fUTF8	躊
U+8E8A.0	fCNS11643	1-7B2F
U+8E8A.0	fBig5	C543
U+8E8A.0	fGB	1-336C
U+8E8A.0	fR/S	 157.14
U+8E8A.0	fTotalStrokes	 21
U+8E8A.0	fCangjie	RMGNI
U+8E8A.0	fFourCorner	64141
U+8E8A.0	fHakka	 cu2;  chiu2; ciu2; chu2
U+8E8A.0	fCantonese	cau4
U+8E8A.0	fMandarin	chou2
U+8E8A.0	fEnglish	[1] hesitant [2] complacent; confident
U+8E8B.0	fUTF8	躋
U+8E8B.0	fCNS11643	1-7B31
U+8E8B.0	fBig5	C545
U+8E8B.0	fGB	1-7552
U+8E8B.0	fR/S	 157.14
U+8E8B.0	fTotalStrokes	 21
U+8E8B.0	fCangjie	RMYX
U+8E8B.0	fFourCorner	60123
U+8E8B.0	fHakka	  zi5; zi1
U+8E8B.0	fCantonese	zai1
U+8E8B.0	fMandarin	ji1; ji4
U+8E8B.0	fEnglish	 ascend; go up; rise
U+8E8C.0	fUTF8	躌
U+8E8C.0	fCNS11643	2-6C45
U+8E8C.0	fBig5	F656
U+8E8C.0	fGB	E-3F48
U+8E8C.0	fR/S	 157.14
U+8E8C.0	fTotalStrokes	 21
U+8E8C.0	fCangjie	RMOTQ
U+8E8C.0	fFourCorner	68151
U+8E8C.0	fCantonese	mou5
U+8E8C.0	fMandarin	wu3
U+8E8D.0	fUTF8	躍
U+8E8D.0	fCNS11643	1-7B30
U+8E8D.0	fBig5	C544
U+8E8D.0	fGB	1-543E
U+8E8D.0	fR/S	 157.14
U+8E8D.0	fTotalStrokes	 21
U+8E8D.0	fCangjie	RMSMG
U+8E8D.0	fFourCorner	67114
U+8E8D.0	fHakka	  jok7; ts'iok7; rok7; tit8
U+8E8D.0	fCantonese	joek6; joek3 (also); tik1
U+8E8D.0	fMandarin	yue4; ti4; ti1; yao4
U+8E8D.0	fEnglish	 skip; jump; frolic; leap; bound; spring
U+8E8E.0	fUTF8	躎
U+8E8E.0	fCNS11643	2-6C44
U+8E8E.0	fBig5	F655
U+8E8E.0	fGB	E-3F49
U+8E8E.0	fR/S	 157.14
U+8E8E.0	fTotalStrokes	 21
U+8E8E.0	fCangjie	RMMFB
U+8E8E.0	fFourCorner	61127
U+8E8E.0	fCantonese	nin2; lin2 (also); nin5
U+8E8E.0	fMandarin	nian3
U+8E8F.0	fUTF8	躏
U+8E8F.0	fGB	0-756F
U+8E8F.0	fR/S	 157.14
U+8E8F.0	fTotalStrokes	 21
U+8E8F.0	fCangjie	RMTLG
U+8E8F.0	fHakka	 nin5
U+8E8F.0	fCantonese	leon6
U+8E8F.0	fMandarin	lin4
U+8E8F.0	fEnglish	simplified form of &#x8EAA;
U+8E90.0	fUTF8	躐
U+8E90.0	fCNS11643	2-6E30
U+8E90.0	fBig5	F761
U+8E90.0	fGB	0-7571
U+8E90.0	fR/S	 157.15
U+8E90.0	fTotalStrokes	 22
U+8E90.0	fCangjie	RMVVV
U+8E90.0	fFourCorner	62116
U+8E90.0	fHakka	  liap8; liap7
U+8E90.0	fCantonese	lip6
U+8E90.0	fMandarin	lie4
U+8E90.0	fEnglish	 step across; overstep; transgress
U+8E91.0	fUTF8	躑
U+8E91.0	fCNS11643	1-7B77
U+8E91.0	fBig5	C5AD
U+8E91.0	fGB	1-755C
U+8E91.0	fR/S	 157.15
U+8E91.0	fTotalStrokes	 22
U+8E91.0	fCangjie	RMTKL
U+8E91.0	fFourCorner	67127
U+8E91.0	fHakka	 chit7; ts'iit7; cit7
U+8E91.0	fCantonese	zaak6; zik6
U+8E91.0	fMandarin	zhi2
U+8E91.0	fEnglish	 waver; hesitate; be irresolute; falter
U+8E92.0	fUTF8	躒
U+8E92.0	fCNS11643	2-6E2F
U+8E92.0	fBig5	F760
U+8E92.0	fGB	1-7548
U+8E92.0	fR/S	 157.15
U+8E92.0	fTotalStrokes	 22
U+8E92.0	fCangjie	RMVID
U+8E92.0	fFourCorner	62194
U+8E92.0	fHakka	 lit8; lok8; lit7
U+8E92.0	fCantonese	lik1; lok3 (also); lok6; lik6
U+8E92.0	fMandarin	li4; luo4; yue4; yao4
U+8E92.0	fEnglish	walk; move
U+8E93.0	fUTF8	躓
U+8E93.0	fCNS11643	1-7B78
U+8E93.0	fBig5	C5AE
U+8E93.0	fGB	1-7559
U+8E93.0	fR/S	 157.15
U+8E93.0	fTotalStrokes	 22
U+8E93.0	fCangjie	RMHLC
U+8E93.0	fFourCorner	62186
U+8E93.0	fHakka	 zhit7; zhi5; zih5; tsiit7
U+8E93.0	fCantonese	zi3
U+8E93.0	fMandarin	zhi4; zhi1; zhi2
U+8E93.0	fEnglish	[1] stumble; trip [2] be frustrated; suffer a setback
U+8E94.0	fUTF8	躔
U+8E94.0	fCNS11643	2-6E2D
U+8E94.0	fBig5	F75E
U+8E94.0	fGB	0-7570
U+8E94.0	fR/S	 157.15
U+8E94.0	fTotalStrokes	 22
U+8E94.0	fCangjie	RMIWG
U+8E94.0	fFourCorner	60114
U+8E94.0	fHakka	 chan2; can2; cen2; chen2
U+8E94.0	fCantonese	cin4
U+8E94.0	fMandarin	chan2; zhan4
U+8E94.0	fEnglish	 follow in; imitate rut; path
U+8E95.0	fUTF8	躕
U+8E95.0	fCNS11643	2-6E2C
U+8E95.0	fBig5	F75D
U+8E95.0	fGB	E-3F4A
U+8E95.0	fR/S	 157.15
U+8E95.0	fTotalStrokes	 22
U+8E95.0	fCangjie	RMIGI
U+8E95.0	fFourCorner	61140
U+8E95.0	fHakka	 cu2; chu2
U+8E95.0	fCantonese	cyu4; cyu1
U+8E95.0	fMandarin	chu2
U+8E95.0	fEnglish	 hesitate; waver; falter
U+8E96.0	fUTF8	躖
U+8E96.0	fCNS11643	2-6E31
U+8E96.0	fBig5	F762
U+8E96.0	fGB	E-3F4B
U+8E96.0	fR/S	 157.15
U+8E96.0	fTotalStrokes	 22
U+8E96.0	fCangjie	RMVIM
U+8E96.0	fFourCorner	62170
U+8E96.0	fCantonese	teon2; dyun6
U+8E96.0	fMandarin	duan4; tuan3
U+8E97.0	fUTF8	躗
U+8E97.0	fCNS11643	2-6E32
U+8E97.0	fBig5	F763
U+8E97.0	fGB	E-3F4C
U+8E97.0	fR/S	 157.16
U+8E97.0	fTotalStrokes	 23
U+8E97.0	fCangjie	HNRYO
U+8E97.0	fFourCorner	21801
U+8E97.0	fCantonese	wai6
U+8E97.0	fMandarin	wei4; hua4; huai4
U+8E97.0	fEnglish	exaggerate
U+8E98.0	fUTF8	躘
U+8E98.0	fCNS11643	2-6F54
U+8E98.0	fBig5	F846
U+8E98.0	fGB	5-6E6B
U+8E98.0	fR/S	 157.16
U+8E98.0	fTotalStrokes	 23
U+8E98.0	fCangjie	RMYBP
U+8E98.0	fFourCorner	61111
U+8E98.0	fHakka	 lung2;  liung1; liung2; nung5; lung5
U+8E98.0	fCantonese	lung4
U+8E98.0	fMandarin	long2
U+8E98.0	fEnglish	 walk
U+8E99.0	fUTF8	躙
U+8E99.0	fCNS11643	3-604B
U+8E99.0	fGB	5-6F39
U+8E99.0	fR/S	  157.16
U+8E99.0	fTotalStrokes	 23
U+8E99.0	fCangjie	RMANG
U+8E99.0	fFourCorner	67120
U+8E99.0	fCantonese	leon6
U+8E99.0	fMandarin	lin4
U+8E99.0	fEnglish	trample down; oppress; overrun
U+8E9A.0	fUTF8	躚
U+8E9A.0	fCNS11643	2-6E2E
U+8E9A.0	fBig5	F75F
U+8E9A.0	fGB	1-7551
U+8E9A.0	fR/S	 157.16
U+8E9A.0	fTotalStrokes	 23
U+8E9A.0	fCangjie	RMYMU
U+8E9A.0	fFourCorner	61131
U+8E9A.0	fHakka	  cien1;  cin1; sien1; zen1; sen1
U+8E9A.0	fCantonese	sin1
U+8E9A.0	fMandarin	xian1
U+8E9A.0	fEnglish	 wander about; walk around; revolve; turn round and round
U+8E9B.0	fUTF8	躛
U+8E9B.0	fGB	E-3F4D
U+8E9B.0	fR/S	 157.16
U+8E9B.0	fTotalStrokes	 23
U+8E9B.0	fCangjie	XHNRY
U+8E9B.0	fHakka	 wui6; wi5; vui5
U+8E9B.0	fCantonese	wai6
U+8E9B.0	fMandarin	wei4
U+8E9B.0	fEnglish	exaggerate
U+8E9C.0	fUTF8	躜
U+8E9C.0	fGB	0-7572
U+8E9C.0	fR/S	 157.16
U+8E9C.0	fTotalStrokes	 23
U+8E9C.0	fCangjie	RMHUO
U+8E9C.0	fHakka	 zan1
U+8E9C.0	fCantonese	zyun1; zyun2
U+8E9C.0	fMandarin	zuan1; cuo2
U+8E9C.0	fEnglish	simplified form of &#x8EA6;
U+8E9D.0	fUTF8	躝
U+8E9D.0	fCNS11643	2-7054
U+8E9D.0	fBig5	F8C6
U+8E9D.0	fGB	E-3F4E
U+8E9D.0	fR/S	 157.17
U+8E9D.0	fTotalStrokes	 24
U+8E9D.0	fCangjie	RMANW
U+8E9D.0	fFourCorner	67120
U+8E9D.0	fHakka	 lan2
U+8E9D.0	fCantonese	laan1; laan4
U+8E9D.0	fMandarin	lan2; tan2
U+8E9D.0	fEnglish	 crawl; creep; slither
U+8E9E.0	fUTF8	躞
U+8E9E.0	fCNS11643	2-7051
U+8E9E.0	fBig5	F8C3
U+8E9E.0	fGB	0-7573
U+8E9E.0	fR/S	 157.17
U+8E9E.0	fTotalStrokes	 24
U+8E9E.0	fCangjie	RMFFE
U+8E9E.0	fFourCorner	60147
U+8E9E.0	fHakka	  siet7; siap8
U+8E9E.0	fCantonese	sip3; sit3 (also)
U+8E9E.0	fMandarin	xie4
U+8E9E.0	fEnglish	[1] walking; proceeding [2] a pin or rod for rolling ascroll; axis
U+8E9F.0	fUTF8	躟
U+8E9F.0	fCNS11643	2-7052
U+8E9F.0	fBig5	F8C4
U+8E9F.0	fGB	5-6F4B
U+8E9F.0	fR/S	 157.17
U+8E9F.0	fTotalStrokes	 24
U+8E9F.0	fCangjie	RMYRV
U+8E9F.0	fFourCorner	60132
U+8E9F.0	fHakka	 nang5
U+8E9F.0	fCantonese	joeng4; nong5
U+8E9F.0	fMandarin	rang2
U+8EA0.0	fUTF8	躠
U+8EA0.0	fCNS11643	2-7053
U+8EA0.0	fBig5	F8C5
U+8EA0.0	fGB	3-6F4C
U+8EA0.0	fR/S	 157.17
U+8EA0.0	fTotalStrokes	 24
U+8EA0.0	fCangjie	TJRYO
U+8EA0.0	fFourCorner	44801
U+8EA0.0	fCantonese	sit3
U+8EA0.0	fMandarin	sa3; xie4; xie3
U+8EA0.0	fEnglish	 limp
U+8EA1.0	fUTF8	躡
U+8EA1.0	fCNS11643	1-7D29
U+8EA1.0	fBig5	C65C
U+8EA1.0	fGB	1-7566
U+8EA1.0	fR/S	 157.18
U+8EA1.0	fTotalStrokes	 25
U+8EA1.0	fCangjie	RMSJJ
U+8EA1.0	fFourCorner	61141
U+8EA1.0	fHakka	  ngiap7
U+8EA1.0	fCantonese	nip6; lip6 (also)
U+8EA1.0	fMandarin	nie4
U+8EA1.0	fEnglish	[1] tread; step over [2] follow; pursue [3] walk lightly; tiptoe
U+8EA2.0	fUTF8	躢
U+8EA2.0	fCNS11643	3-6150
U+8EA2.0	fGB	5-6F45
U+8EA2.0	fR/S	  157.18
U+8EA2.0	fTotalStrokes	 25
U+8EA2.0	fCangjie	RMANM
U+8EA2.0	fFourCorner	67120
U+8EA2.0	fCantonese	daap6; taap3 (also)
U+8EA2.0	fMandarin	ta4
U+8EA3.0	fUTF8	躣
U+8EA3.0	fCNS11643	2-7140
U+8EA3.0	fBig5	F951
U+8EA3.0	fGB	3-7059
U+8EA3.0	fR/S	 157.18
U+8EA3.0	fTotalStrokes	 25
U+8EA3.0	fCangjie	RMBUG
U+8EA3.0	fFourCorner	66114
U+8EA3.0	fCantonese	keoi4
U+8EA3.0	fMandarin	qu2
U+8EA4.0	fUTF8	躤
U+8EA4.0	fCNS11643	2-713F
U+8EA4.0	fBig5	F950
U+8EA4.0	fGB	5-6F4C
U+8EA4.0	fR/S	 157.18
U+8EA4.0	fTotalStrokes	 25
U+8EA4.0	fCangjie	RMTQA
U+8EA4.0	fFourCorner	64161
U+8EA4.0	fCantonese	zik6
U+8EA4.0	fMandarin	jie4; ji2
U+8EA5.0	fUTF8	躥
U+8EA5.0	fCNS11643	2-713E
U+8EA5.0	fBig5	F94F
U+8EA5.0	fGB	1-345A
U+8EA5.0	fR/S	 157.18
U+8EA5.0	fTotalStrokes	 25
U+8EA5.0	fCangjie	RMJCV
U+8EA5.0	fFourCorner	63117
U+8EA5.0	fHakka	 con1
U+8EA5.0	fCantonese	cyun1
U+8EA5.0	fMandarin	cuan1
U+8EA5.0	fEnglish	[1] leap; jump [2] get promoted (someone) [3] spurt out
U+8EA6.0	fUTF8	躦
U+8EA6.0	fCNS11643	2-715F
U+8EA6.0	fBig5	F970
U+8EA6.0	fGB	1-7572
U+8EA6.0	fR/S	 157.19
U+8EA6.0	fTotalStrokes	 26
U+8EA6.0	fCangjie	RMHUC
U+8EA6.0	fFourCorner	64186
U+8EA6.0	fHakka	  zan1
U+8EA6.0	fCantonese	zyun1; zyun2
U+8EA6.0	fMandarin	zuan1; cuo2
U+8EA6.0	fEnglish	 jump
U+8EA7.0	fUTF8	躧
U+8EA7.0	fCNS11643	3-616C
U+8EA7.0	fGB	5-6E75
U+8EA7.0	fR/S	 157.19
U+8EA7.0	fTotalStrokes	 26
U+8EA7.0	fCangjie	RMMMP
U+8EA7.0	fFourCorner	61110
U+8EA7.0	fHakka	  sai3; si3
U+8EA7.0	fCantonese	saai2; sai2
U+8EA7.0	fMandarin	xi3
U+8EA7.0	fEnglish	sandal
U+8EA8.0	fUTF8	躨
U+8EA8.0	fCNS11643	2-722E
U+8EA8.0	fBig5	F9BE
U+8EA8.0	fGB	E-3F4F
U+8EA8.0	fR/S	 157.22; 157.21
U+8EA8.0	fTotalStrokes	 28
U+8EA8.0	fCangjie	RMTCE
U+8EA8.0	fFourCorner	68147
U+8EA8.0	fCantonese	kwai4
U+8EA8.0	fMandarin	kui2
U+8EA9.0	fUTF8	躩
U+8EA9.0	fCNS11643	2-7179
U+8EA9.0	fBig5	F9AB
U+8EA9.0	fGB	3-705A
U+8EA9.0	fR/S	 157.20
U+8EA9.0	fTotalStrokes	 27
U+8EA9.0	fCangjie	RMBUE
U+8EA9.0	fFourCorner	66147
U+8EA9.0	fHakka	  kiok7; giok7
U+8EA9.0	fCantonese	fok3; gwok3; kwok3
U+8EA9.0	fMandarin	jue2
U+8EA9.0	fEnglish	take quick; short steps
U+8EAA.0	fUTF8	躪
U+8EAA.0	fCNS11643	1-7D3B
U+8EAA.0	fBig5	C66E
U+8EAA.0	fGB	1-756F
U+8EAA.0	fR/S	 157.20
U+8EAA.0	fTotalStrokes	 27
U+8EAA.0	fCangjie	RMTAG
U+8EAA.0	fFourCorner	67120
U+8EAA.0	fHakka	 nin5; nang5; lin5; nin6
U+8EAA.0	fCantonese	leon6
U+8EAA.0	fMandarin	lin4
U+8EAA.0	fEnglish	 trample; devastate; overrun; lay a place waste
U+8EAB.0	fUTF8	身
U+8EAB.0	fCNS11643	1-4B4E
U+8EAB.0	fBig5	A8AD
U+8EAB.0	fGB	0-496D
U+8EAB.0	fR/S	 158.0
U+8EAB.0	fTotalStrokes	 7
U+8EAB.0	fCangjie	HXH
U+8EAB.0	fFourCorner	27400
U+8EAB.0	fHakka	  shin1;  sin1; siin1
U+8EAB.0	fCantonese	san1; gyun1; kin4; jyun4; kyun1
U+8EAB.0	fMandarin	shen1; gui4; yuan2
U+8EAB.0	fEnglish	[1] body; trunk [2] one's own person; oneself [3] a child in the womb [4] in person; personally [5] Kangxi radical 158
U+8EAB.1	fUTF8	身
U+8EAB.1	fCNS11643	1-4B4E
U+8EAB.1	fBig5	A8AD
U+8EAB.1	fGB	0-496D
U+8EAB.1	fR/S	 158.0
U+8EAB.1	fTotalStrokes	 7
U+8EAB.1	fCangjie	HXH
U+8EAB.1	fFourCorner	27400
U+8EAB.1	fMandarin	juan1
U+8EAB.1	fEnglish	used in combination for an ancient name of India
U+8EAC.0	fUTF8	躬
U+8EAC.0	fCNS11643	1-5845
U+8EAC.0	fBig5	B060
U+8EAC.0	fGB	0-392A
U+8EAC.0	fR/S	 158.3
U+8EAC.0	fTotalStrokes	 10
U+8EAC.0	fCangjie	HHN
U+8EAC.0	fFourCorner	27227
U+8EAC.0	fHakka	  giung1
U+8EAC.0	fCantonese	gung1
U+8EAC.0	fMandarin	gong1
U+8EAC.0	fEnglish	[1] the body; the person [2] in person; personally [3] bend (the body)
U+8EAD.0	fUTF8	躭
U+8EAD.0	fCNS11643	3-393F
U+8EAD.0	fGB	8-2D7C
U+8EAD.0	fR/S	 158.4
U+8EAD.0	fTotalStrokes	 11
U+8EAD.0	fCangjie	HHLBU
U+8EAD.0	fFourCorner	24210
U+8EAD.0	fHakka	 dam1;  tam1; {dam2}
U+8EAD.0	fCantonese	daam1
U+8EAD.0	fMandarin	dan1
U+8EAD.0	fEnglish	 delay (unintentionally)
U+8EAE.0	fUTF8	躮
U+8EAE.0	fCNS11643	F-386C
U+8EAE.0	fGB	E-3F50
U+8EAE.0	fR/S	 158.4
U+8EAE.0	fTotalStrokes	 11
U+8EAE.0	fCangjie	HHCSH
U+8EAE.0	fCantonese	fan1
U+8EAF.0	fUTF8	躯
U+8EAF.0	fCNS11643	3-3940
U+8EAF.0	fGB	0-477B
U+8EAF.0	fR/S	 158.4
U+8EAF.0	fTotalStrokes	 11
U+8EAF.0	fCangjie	HHSK
U+8EAF.0	fFourCorner	21210
U+8EAF.0	fHakka	 ki1
U+8EAF.0	fCantonese	keoi1
U+8EAF.0	fMandarin	qu1
U+8EAF.0	fEnglish	simplified form of &#x8EC0;
U+8EB0.0	fUTF8	躰
U+8EB0.0	fCNS11643	3-3F40
U+8EB0.0	fGB	5-6F75
U+8EB0.0	fR/S	 158.5
U+8EB0.0	fTotalStrokes	 12
U+8EB0.0	fCangjie	HHDM
U+8EB0.0	fFourCorner	25430
U+8EB0.0	fCantonese	tai2
U+8EB0.0	fMandarin	ti3
U+8EB0.0	fEnglish	 body group; class; body; unit
U+8EB1.0	fUTF8	躱
U+8EB1.0	fCNS11643	3-4473
U+8EB1.0	fGB	E-3F51
U+8EB1.0	fR/S	 158.6
U+8EB1.0	fTotalStrokes	 13
U+8EB1.0	fCangjie	HHNSD
U+8EB1.0	fFourCorner	27490
U+8EB1.0	fCantonese	do2
U+8EB1.0	fMandarin	duo3
U+8EB1.0	fEnglish	hide
U+8EB2.0	fUTF8	躲
U+8EB2.0	fCNS11643	1-6723
U+8EB2.0	fBig5	B8FA
U+8EB2.0	fGB	0-3663
U+8EB2.0	fR/S	 158.6
U+8EB2.0	fTotalStrokes	 13
U+8EB2.0	fCangjie	HHHND
U+8EB2.0	fFourCorner	27294
U+8EB2.0	fHakka	  do3
U+8EB2.0	fCantonese	do2
U+8EB2.0	fMandarin	duo3
U+8EB2.0	fEnglish	[1] dodge; shun; avoid; escape [2] hide
U+8EB3.0	fUTF8	躳
U+8EB3.0	fCNS11643	3-497D
U+8EB3.0	fGB	E-3F52
U+8EB3.0	fR/S	  158.7
U+8EB3.0	fTotalStrokes	 14
U+8EB3.0	fCangjie	HHRHR
U+8EB3.0	fFourCorner	26260
U+8EB3.0	fHakka	  giung1
U+8EB3.0	fCantonese	gung1
U+8EB3.0	fMandarin	gong1
U+8EB3.0	fEnglish	 body self
U+8EB4.0	fUTF8	躴
U+8EB4.0	fCNS11643	3-497C
U+8EB4.0	fGB	3-714C
U+8EB4.0	fR/S	  158.7
U+8EB4.0	fTotalStrokes	 14
U+8EB4.0	fCangjie	HHIAV
U+8EB4.0	fFourCorner	23430
U+8EB4.0	fCantonese	long4
U+8EB4.0	fMandarin	lang2
U+8EB4.0	fEnglish	tall
U+8EB5.0	fUTF8	躵
U+8EB5.0	fGB	E-3F53
U+8EB5.0	fR/S	 158.7
U+8EB5.0	fTotalStrokes	 14
U+8EB5.0	fCangjie	HHSIP
U+8EB5.0	fCantonese	jan2
U+8EB6.0	fUTF8	躶
U+8EB6.0	fCNS11643	3-4F30
U+8EB6.0	fGB	E-3F54
U+8EB6.0	fR/S	 158.8
U+8EB6.0	fTotalStrokes	 15
U+8EB6.0	fCangjie	HHWD
U+8EB6.0	fFourCorner	26290
U+8EB6.0	fHakka	 ko5; ko3
U+8EB6.0	fCantonese	lo2
U+8EB6.0	fMandarin	luo3
U+8EB6.0	fEnglish	 bare; nude undress; strip
U+8EB7.0	fUTF8	躷
U+8EB7.0	fCNS11643	3-4F32
U+8EB7.0	fGB	5-6F77
U+8EB7.0	fR/S	 158.8
U+8EB7.0	fTotalStrokes	 15
U+8EB7.0	fCangjie	HHHDV
U+8EB7.0	fFourCorner	22440
U+8EB7.0	fHakka	 ai3
U+8EB7.0	fCantonese	ai2; ngai2 (also)
U+8EB7.0	fMandarin	ai3
U+8EB7.0	fEnglish	of short stature; low in height
U+8EB8.0	fUTF8	躸
U+8EB8.0	fCNS11643	3-4F2F
U+8EB8.0	fGB	3-714D
U+8EB8.0	fR/S	  158.8
U+8EB8.0	fTotalStrokes	 15
U+8EB8.0	fCangjie	HHKMR
U+8EB8.0	fFourCorner	24220
U+8EB8.0	fCantonese	gei1
U+8EB8.0	fMandarin	ji1
U+8EB9.0	fUTF8	躹
U+8EB9.0	fCNS11643	3-4F31
U+8EB9.0	fGB	E-3F55
U+8EB9.0	fR/S	  158.8
U+8EB9.0	fTotalStrokes	 15
U+8EB9.0	fCangjie	HHPFD
U+8EB9.0	fFourCorner	27220
U+8EB9.0	fHakka	 kiuk7
U+8EB9.0	fCantonese	guk1
U+8EB9.0	fMandarin	ju2
U+8EBA.0	fUTF8	躺
U+8EBA.0	fCNS11643	1-6F41
U+8EBA.0	fBig5	BDF6
U+8EBA.0	fGB	0-4C49
U+8EBA.0	fR/S	 158.8
U+8EBA.0	fTotalStrokes	 15
U+8EBA.0	fCangjie	HHFBR
U+8EBA.0	fFourCorner	29427
U+8EBA.0	fHakka	  tong3
U+8EBA.0	fCantonese	tong2
U+8EBA.0	fMandarin	tang3
U+8EBA.0	fEnglish	 be in a lying position
U+8EBB.0	fUTF8	躻
U+8EBB.0	fGB	E-3F56
U+8EBB.0	fR/S	 158.8
U+8EBB.0	fTotalStrokes	 15
U+8EBB.0	fCangjie	HHJCM
U+8EBB.0	fCantonese	hung1
U+8EBC.0	fUTF8	躼
U+8EBC.0	fCNS11643	3-6652
U+8EBC.0	fGB	E-3F57
U+8EBC.0	fR/S	 158.8
U+8EBC.0	fTotalStrokes	 15
U+8EBC.0	fCangjie	HHSMV
U+8EBC.0	fFourCorner	2123
U+8EBC.0	fCantonese	lou6
U+8EBD.0	fUTF8	躽
U+8EBD.0	fCNS11643	2-5A74
U+8EBD.0	fBig5	EBC8
U+8EBD.0	fGB	3-714F
U+8EBD.0	fR/S	 158.9
U+8EBD.0	fTotalStrokes	 16
U+8EBD.0	fCangjie	HHSAV
U+8EBD.0	fFourCorner	21214
U+8EBD.0	fCantonese	jin2; zin2
U+8EBD.0	fMandarin	yan3; yan4
U+8EBD.0	fEnglish	躽胸凸肚, to push out one's chest and belly
U+8EBE.0	fUTF8	躾
U+8EBE.0	fCNS11643	F-5833
U+8EBE.0	fGB	E-3F58
U+8EBE.0	fR/S	 158.9
U+8EBE.0	fTotalStrokes	 16
U+8EBE.0	fCangjie	HHTGK
U+8EBE.0	fFourCorner	2843
U+8EBE.0	fCantonese	mei5
U+8EBE.0	fEnglish	to discipline, train, bring up; discipline, training
U+8EBF.0	fUTF8	躿
U+8EBF.0	fCNS11643	3-5935
U+8EBF.0	fGB	3-7152
U+8EBF.0	fR/S	 158.10; 158.11
U+8EBF.0	fTotalStrokes	 18; 17
U+8EBF.0	fCangjie	HHILE
U+8EBF.0	fFourCorner	20430
U+8EBF.0	fCantonese	hong1
U+8EBF.0	fMandarin	kang1
U+8EBF.0	fEnglish	tall
U+8EC0.0	fUTF8	軀
U+8EC0.0	fCNS11643	1-774A
U+8EC0.0	fBig5	C2DF
U+8EC0.0	fGB	1-477B
U+8EC0.0	fR/S	 158.11
U+8EC0.0	fTotalStrokes	 18
U+8EC0.0	fCangjie	HHSRR
U+8EC0.0	fFourCorner	21216
U+8EC0.0	fHakka	 ki1
U+8EC0.0	fCantonese	keoi1
U+8EC0.0	fMandarin	qu1
U+8EC0.0	fEnglish	[1] the body; trunk [2] a child in the womb
U+8EC1.0	fUTF8	軁
U+8EC1.0	fCNS11643	3-5936
U+8EC1.0	fGB	3-7151
U+8EC1.0	fR/S	  158.11
U+8EC1.0	fTotalStrokes	 18
U+8EC1.0	fCangjie	HHLLV
U+8EC1.0	fFourCorner	25440
U+8EC1.0	fCantonese	lau4
U+8EC1.0	fMandarin	lou2
U+8EC2.0	fUTF8	軂
U+8EC2.0	fCNS11643	2-6741
U+8EC2.0	fBig5	F355
U+8EC2.0	fGB	E-3F59
U+8EC2.0	fR/S	 158.12
U+8EC2.0	fTotalStrokes	 19
U+8EC2.0	fCangjie	HHFFS
U+8EC2.0	fFourCorner	29227
U+8EC2.0	fCantonese	lou6
U+8EC2.0	fMandarin	lao4
U+8EC3.0	fUTF8	軃
U+8EC3.0	fCNS11643	3-5B3F
U+8EC3.0	fGB	3-714E
U+8EC3.0	fR/S	 158.12
U+8EC3.0	fTotalStrokes	 19
U+8EC3.0	fCangjie	HHRRJ
U+8EC3.0	fFourCorner	26250
U+8EC3.0	fHakka	 to3; do3
U+8EC3.0	fCantonese	do2
U+8EC3.0	fMandarin	tuo3; duo3
U+8EC3.0	fEnglish	[1] wide [2] full [3] hanging down
U+8EC4.0	fUTF8	軄
U+8EC4.0	fCNS11643	4-6156
U+8EC4.0	fGB	E-3F5A
U+8EC4.0	fR/S	 158.12
U+8EC4.0	fTotalStrokes	 19
U+8EC4.0	fCangjie	HHYIA
U+8EC4.0	fFourCorner	23450
U+8EC4.0	fHakka	 zhit7; tsiit7
U+8EC4.0	fCantonese	zik1
U+8EC4.0	fMandarin	zhi2
U+8EC4.0	fEnglish	 govern; oversee; manage; direct official duty; office
U+8EC5.0	fUTF8	軅
U+8EC5.0	fGB	E-3F5B
U+8EC5.0	fR/S	 158.12
U+8EC5.0	fTotalStrokes	 19
U+8EC5.0	fCangjie	HHMOG
U+8EC5.0	fCantonese	ngaan6; aan6 (also)
U+8EC6.0	fUTF8	軆
U+8EC6.0	fCNS11643	3-5D38
U+8EC6.0	fGB	E-3F5C
U+8EC6.0	fR/S	  158.13
U+8EC6.0	fTotalStrokes	 20
U+8EC6.0	fCangjie	HHTWT
U+8EC6.0	fFourCorner	26210
U+8EC6.0	fCantonese	tai2
U+8EC6.0	fMandarin	ti3
U+8EC6.0	fEnglish	 body group; class; body; unit
U+8EC7.0	fUTF8	軇
U+8EC7.0	fCNS11643	4-6751
U+8EC7.0	fGB	E-3F5D
U+8EC7.0	fR/S	  158.14
U+8EC7.0	fTotalStrokes	 21
U+8EC7.0	fCangjie	HHGNI
U+8EC7.0	fFourCorner	24440
U+8EC7.0	fCantonese	dou6
U+8EC7.0	fMandarin	dao4
U+8EC8.0	fUTF8	軈
U+8EC8.0	fGB	E-3F5E
U+8EC8.0	fR/S	 158.17
U+8EC8.0	fTotalStrokes	 24
U+8EC8.0	fCangjie	HHIOP
U+8EC8.0	fCantonese	jing3
U+8EC8.0	fEnglish	soon after, soon; almost; no more than; after all
U+8EC9.0	fUTF8	軉
U+8EC9.0	fCNS11643	2-717A
U+8EC9.0	fBig5	F9AC
U+8EC9.0	fGB	E-3F5F
U+8EC9.0	fR/S	 158.20
U+8EC9.0	fTotalStrokes	 27
U+8EC9.0	fCangjie	HHJMC
U+8EC9.0	fFourCorner	23486
U+8EC9.0	fHakka	 ngiuk8
U+8EC9.0	fCantonese	juk6; juk1
U+8EC9.0	fMandarin	yu4
U+8EC9.0	fEnglish	vitality; virility
U+8ECA.0	fUTF8	車
U+8ECA.0	fCNS11643	1-4B4F
U+8ECA.0	fBig5	A8AE
U+8ECA.0	fGB	1-3335
U+8ECA.0	fR/S	 159.0
U+8ECA.0	fTotalStrokes	 7
U+8ECA.0	fCangjie	JWJ
U+8ECA.0	fFourCorner	50006
U+8ECA.0	fMandarin	che1
U+8ECA.0	fEnglish	[1] cart; vehicle [2] wheeled machine [3] carry in a cart [4] shape (things) on a lathe; lathe; turn [5] lift water by a water wheel [6] Kangxi radical 159
U+8ECA.1	fUTF8	車
U+8ECA.1	fCNS11643	1-4B4F
U+8ECA.1	fBig5	A8AE
U+8ECA.1	fGB	1-3335
U+8ECA.1	fR/S	 159.0
U+8ECA.1	fTotalStrokes	 7
U+8ECA.1	fCangjie	JWJ
U+8ECA.1	fFourCorner	50006
U+8ECA.1	fHakka	 gi1; ca1;  cha1; ca
U+8ECA.1	fCantonese	ce1; geoi1
U+8ECA.1	fMandarin	ju1
U+8ECA.1	fEnglish	the name of a chessman in a Chinese chess
U+8ECB.0	fUTF8	軋
U+8ECB.0	fCNS11643	1-4F51
U+8ECB.0	fBig5	AAEE
U+8ECB.0	fGB	1-547E
U+8ECB.0	fR/S	 159.1
U+8ECB.0	fTotalStrokes	 8
U+8ECB.0	fCangjie	JJU
U+8ECB.0	fFourCorner	52010
U+8ECB.0	fHakka	 zat7;  jet7; zap7; zhap7; ap7
U+8ECB.0	fCantonese	aat3; zaat3; gaat3; ngaat3 (also); aaat3
U+8ECB.0	fMandarin	ya4; zha2
U+8ECB.0	fEnglish	 crush; grind
U+8ECB.1	fUTF8	軋
U+8ECB.1	fCNS11643	1-4F51
U+8ECB.1	fBig5	AAEE
U+8ECB.1	fGB	1-547E
U+8ECB.1	fR/S	 159.1
U+8ECB.1	fTotalStrokes	 8
U+8ECB.1	fCangjie	JJU
U+8ECB.1	fFourCorner	52010
U+8ECB.1	fMandarin	ga2
U+8ECB.1	fEnglish	[1] crowd [2] make friends [3] check
U+8ECC.0	fUTF8	軌
U+8ECC.0	fCNS11643	1-535C
U+8ECC.0	fBig5	AD79
U+8ECC.0	fGB	1-396C
U+8ECC.0	fR/S	 159.2
U+8ECC.0	fTotalStrokes	 9
U+8ECC.0	fCangjie	JJKN
U+8ECC.0	fFourCorner	54017
U+8ECC.0	fHakka	 gui3; kui3; gwi3; kwui3
U+8ECC.0	fCantonese	gwai2
U+8ECC.0	fMandarin	gui3
U+8ECC.0	fEnglish	[1] the space between the right and left wheels of a vehicle [2] track; rut; path [3] orbit [4] rule; regulation [5] follow; comply with; obey
U+8ECD.0	fUTF8	軍
U+8ECD.0	fCNS11643	1-535B
U+8ECD.0	fBig5	AD78
U+8ECD.0	fGB	1-3E7C
U+8ECD.0	fR/S	 159.2
U+8ECD.0	fTotalStrokes	 9
U+8ECD.0	fCangjie	BJWJ
U+8ECD.0	fFourCorner	37506
U+8ECD.0	fHakka	  giun1
U+8ECD.0	fCantonese	gwan1
U+8ECD.0	fMandarin	jun1
U+8ECD.0	fEnglish	[1] army; military; soldiers; troops; forces; of national defence [2] corps (a military unit) [3] armed service
U+8ECE.0	fUTF8	軎
U+8ECE.0	fCNS11643	5-3055
U+8ECE.0	fGB	0-6A26
U+8ECE.0	fR/S	  159.3
U+8ECE.0	fTotalStrokes	 10
U+8ECE.0	fCangjie	JJR
U+8ECE.0	fFourCorner	50600
U+8ECE.0	fCantonese	wai6
U+8ECE.0	fMandarin	wei4
U+8ECE.0	fEnglish	a brass parts for car in ancient
U+8ECF.0	fUTF8	軏
U+8ECF.0	fCNS11643	1-5848
U+8ECF.0	fBig5	B063
U+8ECF.0	fGB	3-4538
U+8ECF.0	fR/S	 159.3
U+8ECF.0	fTotalStrokes	 10
U+8ECF.0	fCangjie	JJMU
U+8ECF.0	fFourCorner	51010
U+8ECF.0	fHakka	 ngiet7; ngiet8; ngut7
U+8ECF.0	fCantonese	jyut6
U+8ECF.0	fMandarin	yue4; wu4
U+8ECF.0	fEnglish	a cross-bar at end of the poles of a cart, etc.; an arragnement for yoking the horses in a light carriage
U+8ED0.0	fUTF8	軐
U+8ED0.0	fCNS11643	3-3421
U+8ED0.0	fGB	3-453A
U+8ED0.0	fR/S	 159.3
U+8ED0.0	fTotalStrokes	 10
U+8ED0.0	fCangjie	JJNJ
U+8ED0.0	fFourCorner	57010
U+8ED0.0	fCantonese	seon3
U+8ED0.0	fMandarin	xian4; xin4
U+8ED1.0	fUTF8	軑
U+8ED1.0	fCNS11643	2-332C
U+8ED1.0	fBig5	D3E8
U+8ED1.0	fGB	3-4537
U+8ED1.0	fR/S	 159.3
U+8ED1.0	fTotalStrokes	 10
U+8ED1.0	fCangjie	JJK
U+8ED1.0	fFourCorner	54030
U+8ED1.0	fHakka	 ti5; ti6; tai5; di3
U+8ED1.0	fCantonese	daai6 (also); dai6
U+8ED1.0	fMandarin	dai4; di4
U+8ED1.0	fEnglish	the linch-pin of a wheel and axle
U+8ED2.0	fUTF8	軒
U+8ED2.0	fCNS11643	1-5846
U+8ED2.0	fBig5	B061
U+8ED2.0	fGB	1-5079
U+8ED2.0	fR/S	 159.3
U+8ED2.0	fTotalStrokes	 10
U+8ED2.0	fCangjie	JJMJ
U+8ED2.0	fFourCorner	51040
U+8ED2.0	fHakka	  hien1; hen1; hian1
U+8ED2.0	fCantonese	hin1
U+8ED2.0	fMandarin	xuan1; xian3; xian4; jian1; han3
U+8ED2.0	fEnglish	[1] carriage formerly used by high officials [2] the high front of a chariot or carriage [3] porch; balcony [4] windows [5] open; wide [6] high; lofty [7] smiling; laughing; delighted [8] studio; room
U+8ED3.0	fUTF8	軓
U+8ED3.0	fCNS11643	2-332D
U+8ED3.0	fBig5	D3E9
U+8ED3.0	fGB	E-3F60
U+8ED3.0	fR/S	 159.3
U+8ED3.0	fTotalStrokes	 10
U+8ED3.0	fCangjie	JJHNI
U+8ED3.0	fFourCorner	57010
U+8ED3.0	fCantonese	faan6; faan4
U+8ED3.0	fMandarin	fan4
U+8ED4.0	fUTF8	軔
U+8ED4.0	fCNS11643	1-5847
U+8ED4.0	fBig5	B062
U+8ED4.0	fGB	1-696D
U+8ED4.0	fR/S	 159.3
U+8ED4.0	fTotalStrokes	 10
U+8ED4.0	fCangjie	JJSHI
U+8ED4.0	fFourCorner	57020
U+8ED4.0	fHakka	  ngiun3; niun5
U+8ED4.0	fCantonese	jan6
U+8ED4.0	fMandarin	ren4
U+8ED4.0	fEnglish	[1] block; obstruct [2] skid (for checking the motion of a vehicle) [3] soft [4] idle [5] a measure of length equalling to 8 cubits
U+8ED5.0	fUTF8	軕
U+8ED5.0	fCNS11643	3-6653
U+8ED5.0	fGB	E-3F61
U+8ED5.0	fR/S	 159.3
U+8ED5.0	fTotalStrokes	 10
U+8ED5.0	fCangjie	XXJJU
U+8ED5.0	fFourCorner	5207
U+8ED5.0	fCantonese	sim1
U+8ED5.0	fMandarin	shan1
U+8ED6.0	fUTF8	軖
U+8ED6.0	fCNS11643	3-3941
U+8ED6.0	fGB	3-453C
U+8ED6.0	fR/S	  159.4
U+8ED6.0	fTotalStrokes	 11
U+8ED6.0	fCangjie	JJMG
U+8ED6.0	fFourCorner	51014
U+8ED6.0	fCantonese	kwong4; kong4 (also)
U+8ED6.0	fMandarin	ren4; kuang2; qiang2
U+8ED7.0	fUTF8	軗
U+8ED7.0	fCNS11643	2-3962
U+8ED7.0	fBig5	D7DF
U+8ED7.0	fGB	E-3F62
U+8ED7.0	fR/S	 159.4
U+8ED7.0	fTotalStrokes	 11
U+8ED7.0	fCangjie	JJHNE
U+8ED7.0	fFourCorner	57047
U+8ED7.0	fCantonese	syu4; guk1
U+8ED7.0	fMandarin	shu1
U+8ED8.0	fUTF8	軘
U+8ED8.0	fCNS11643	2-395E
U+8ED8.0	fBig5	D7DB
U+8ED8.0	fGB	3-453E
U+8ED8.0	fR/S	 159.4
U+8ED8.0	fTotalStrokes	 11
U+8ED8.0	fCangjie	JJPU
U+8ED8.0	fFourCorner	55017
U+8ED8.0	fHakka	  tun2
U+8ED8.0	fCantonese	tyun4
U+8ED8.0	fMandarin	tun2; tun1
U+8ED8.0	fEnglish	war car
U+8ED9.0	fUTF8	軙
U+8ED9.0	fCNS11643	3-3947
U+8ED9.0	fGB	E-3F63
U+8ED9.0	fR/S	 159.4
U+8ED9.0	fTotalStrokes	 11
U+8ED9.0	fCangjie	JJYE
U+8ED9.0	fFourCorner	51047
U+8ED9.0	fCantonese	can4
U+8ED9.0	fMandarin	chen2; qi2
U+8ED9.0	fEnglish	 a Chinese family name arrange display wheel
U+8EDA.0	fUTF8	軚
U+8EDA.0	fCNS11643	4-3563
U+8EDA.0	fGB	1-7C53
U+8EDA.0	fR/S	 159.4
U+8EDA.0	fTotalStrokes	 11
U+8EDA.0	fCangjie	JJKI
U+8EDA.0	fFourCorner	54030
U+8EDA.0	fHakka	 ti5; ti6; tai5; di3
U+8EDA.0	fCantonese	taai5; taai3
U+8EDA.0	fMandarin	di4; dai4
U+8EDA.0	fEnglish	[1] the linch-pin of a wheel and axle [2] tyre
U+8EDB.0	fUTF8	軛
U+8EDB.0	fCNS11643	1-5D53
U+8EDB.0	fBig5	B36D
U+8EDB.0	fGB	1-696E
U+8EDB.0	fR/S	 159.4
U+8EDB.0	fTotalStrokes	 11
U+8EDB.0	fCangjie	JJMSU
U+8EDB.0	fFourCorner	51012
U+8EDB.0	fHakka	  ak7; ngak7
U+8EDB.0	fCantonese	aak1; ngaak1 (also)
U+8EDB.0	fMandarin	e4
U+8EDB.0	fEnglish	 yoke
U+8EDC.0	fUTF8	軜
U+8EDC.0	fCNS11643	2-3961
U+8EDC.0	fBig5	D7DE
U+8EDC.0	fGB	3-453F
U+8EDC.0	fR/S	 159.4
U+8EDC.0	fTotalStrokes	 11
U+8EDC.0	fCangjie	JJOB
U+8EDC.0	fFourCorner	54027
U+8EDC.0	fHakka	 nap8
U+8EDC.0	fCantonese	naap6; laap6 (also)
U+8EDC.0	fMandarin	na4
U+8EDC.0	fEnglish	reins
U+8EDD.0	fUTF8	軝
U+8EDD.0	fCNS11643	2-3960
U+8EDD.0	fBig5	D7DD
U+8EDD.0	fGB	3-4541
U+8EDD.0	fR/S	 159.4
U+8EDD.0	fTotalStrokes	 11
U+8EDD.0	fCangjie	JJHVP
U+8EDD.0	fFourCorner	52040
U+8EDD.0	fHakka	 ki2
U+8EDD.0	fCantonese	kei4
U+8EDD.0	fMandarin	qi2
U+8EDD.0	fEnglish	axle-tip
U+8EDE.0	fUTF8	軞
U+8EDE.0	fCNS11643	2-395F
U+8EDE.0	fBig5	D7DC
U+8EDE.0	fGB	5-475F
U+8EDE.0	fR/S	 159.4
U+8EDE.0	fTotalStrokes	 11
U+8EDE.0	fCangjie	JJHQU
U+8EDE.0	fFourCorner	52014
U+8EDE.0	fHakka	 mau3
U+8EDE.0	fCantonese	mou4
U+8EDE.0	fMandarin	mao2; mao1
U+8EDF.0	fUTF8	軟
U+8EDF.0	fCNS11643	1-5D54
U+8EDF.0	fBig5	B36E
U+8EDF.0	fGB	1-486D
U+8EDF.0	fR/S	 159.4
U+8EDF.0	fTotalStrokes	 11
U+8EDF.0	fCangjie	JJNO
U+8EDF.0	fFourCorner	57082
U+8EDF.0	fHakka	 ngion1;  ngion3; gnion1; nion1
U+8EDF.0	fCantonese	jyun5
U+8EDF.0	fMandarin	ruan3
U+8EDF.0	fEnglish	[1] soft; flexible; pliable; tender; plastic; supple [2] weak; feeble; effeminate; cowardly [3] soft; gentle; mild [4] poor in quality; ability, etc. [5] easily moved or influenced
U+8EE0.0	fUTF8	軠
U+8EE0.0	fCNS11643	2-3963
U+8EE0.0	fBig5	D7E0
U+8EE0.0	fGB	E-3F64
U+8EE0.0	fR/S	 159.4
U+8EE0.0	fTotalStrokes	 11
U+8EE0.0	fCangjie	JJHG
U+8EE0.0	fFourCorner	52014
U+8EE0.0	fCantonese	kwong4; kong4 (also)
U+8EE0.0	fMandarin	ren4
U+8EE1.0	fUTF8	軡
U+8EE1.0	fCNS11643	2-3964
U+8EE1.0	fBig5	D7E1
U+8EE1.0	fGB	E-3F65
U+8EE1.0	fR/S	 159.4
U+8EE1.0	fTotalStrokes	 11
U+8EE1.0	fCangjie	JJOIN
U+8EE1.0	fFourCorner	58027
U+8EE1.0	fCantonese	kim4; kam4; ling4
U+8EE1.0	fMandarin	fan3; qin2; qian2
U+8EE2.0	fUTF8	転
U+8EE2.0	fGB	E-3F66
U+8EE2.0	fR/S	 159.4
U+8EE2.0	fTotalStrokes	 11
U+8EE2.0	fCangjie	JJMMI
U+8EE2.0	fCantonese	zyun2
U+8EE2.0	fMandarin	zhuan3; zhuai3; zhuan4
U+8EE2.0	fEnglish	shift; move; turn
U+8EE3.0	fUTF8	軣
U+8EE3.0	fGB	E-3F67
U+8EE3.0	fR/S	 159.4
U+8EE3.0	fTotalStrokes	 11
U+8EE3.0	fCangjie	JJIMO
U+8EE3.0	fCantonese	gwang1
U+8EE3.0	fMandarin	hong1
U+8EE3.0	fEnglish	rumble; explosion; blast
U+8EE4.0	fUTF8	軤
U+8EE4.0	fCNS11643	3-3F45
U+8EE4.0	fGB	1-6975
U+8EE4.0	fR/S	  159.5
U+8EE4.0	fTotalStrokes	 12
U+8EE4.0	fCangjie	JJHFD
U+8EE4.0	fFourCorner	52040
U+8EE4.0	fCantonese	fu1
U+8EE4.0	fMandarin	hu1
U+8EE5.0	fUTF8	軥
U+8EE5.0	fCNS11643	2-4068
U+8EE5.0	fBig5	DC43
U+8EE5.0	fGB	3-454B
U+8EE5.0	fR/S	 159.5
U+8EE5.0	fTotalStrokes	 12
U+8EE5.0	fCangjie	JJPR
U+8EE5.0	fFourCorner	57020
U+8EE5.0	fHakka	 ki1; kieu1; geu1
U+8EE5.0	fCantonese	keoi4; gau1 (also)
U+8EE5.0	fMandarin	qu2; gou1; gou4; ju1
U+8EE5.0	fEnglish	yoke
U+8EE6.0	fUTF8	軦
U+8EE6.0	fCNS11643	2-4066
U+8EE6.0	fBig5	DC41
U+8EE6.0	fGB	E-3F68
U+8EE6.0	fR/S	 159.5
U+8EE6.0	fTotalStrokes	 12
U+8EE6.0	fCangjie	JJRHU
U+8EE6.0	fFourCorner	56010
U+8EE6.0	fCantonese	fong3
U+8EE6.0	fMandarin	huang4
U+8EE7.0	fUTF8	軧
U+8EE7.0	fCNS11643	2-406A
U+8EE7.0	fBig5	DC45
U+8EE7.0	fGB	3-454A
U+8EE7.0	fR/S	 159.5
U+8EE7.0	fTotalStrokes	 12
U+8EE7.0	fCangjie	JJHPM
U+8EE7.0	fFourCorner	52040
U+8EE7.0	fCantonese	dai2
U+8EE7.0	fMandarin	di3; chi2; zhi3
U+8EE8.0	fUTF8	軨
U+8EE8.0	fCNS11643	2-406B
U+8EE8.0	fBig5	DC46
U+8EE8.0	fGB	3-4549
U+8EE8.0	fR/S	 159.5
U+8EE8.0	fTotalStrokes	 12
U+8EE8.0	fCangjie	JJOII
U+8EE8.0	fFourCorner	58037
U+8EE8.0	fCantonese	ling4
U+8EE8.0	fMandarin	ling2
U+8EE8.0	fEnglish	[1] the framework on a carraige [2] the wheels of a carriage
U+8EE9.0	fUTF8	軩
U+8EE9.0	fCNS11643	2-4071
U+8EE9.0	fBig5	DC4C
U+8EE9.0	fGB	E-3F69
U+8EE9.0	fR/S	 159.5
U+8EE9.0	fTotalStrokes	 12
U+8EE9.0	fCangjie	JJIR
U+8EE9.0	fFourCorner	53060
U+8EE9.0	fCantonese	toi5
U+8EE9.0	fMandarin	dai4
U+8EEA.0	fUTF8	軪
U+8EEA.0	fCNS11643	3-3F47
U+8EEA.0	fGB	3-454F
U+8EEA.0	fR/S	  159.5
U+8EEA.0	fTotalStrokes	 12
U+8EEA.0	fCangjie	JJVIS
U+8EEA.0	fFourCorner	54020
U+8EEA.0	fCantonese	au1; ngau1 (also)
U+8EEA.0	fMandarin	ao1
U+8EEB.0	fUTF8	軫
U+8EEB.0	fCNS11643	2-406D
U+8EEB.0	fBig5	DC48
U+8EEB.0	fGB	1-6974
U+8EEB.0	fR/S	 159.5
U+8EEB.0	fTotalStrokes	 12
U+8EEB.0	fCangjie	JJOHH
U+8EEB.0	fFourCorner	58022
U+8EEB.0	fHakka	 zhin3; zihn3; zin3
U+8EEB.0	fCantonese	zan2; can2 (also)
U+8EEB.0	fMandarin	zhen3
U+8EEB.0	fEnglish	[1] the wooden bumper at the rear of a cart or carriage [2] a Chinese family name
U+8EEC.0	fUTF8	軬
U+8EEC.0	fCNS11643	2-406F
U+8EEC.0	fBig5	DC4A
U+8EEC.0	fGB	3-3352
U+8EEC.0	fR/S	 159.5
U+8EEC.0	fTotalStrokes	 12
U+8EEC.0	fCangjie	IKJWJ
U+8EEC.0	fFourCorner	23506
U+8EEC.0	fCantonese	faan6; faan5; faan4
U+8EEC.0	fMandarin	fan4; ben4
U+8EED.0	fUTF8	軭
U+8EED.0	fCNS11643	5-4350
U+8EED.0	fGB	5-4766
U+8EED.0	fR/S	  159.6
U+8EED.0	fTotalStrokes	 13
U+8EED.0	fCangjie	JJSMG
U+8EED.0	fFourCorner	51010
U+8EED.0	fCantonese	waang1
U+8EED.0	fMandarin	kuang1
U+8EED.0	fEnglish	spoil; ruin
U+8EEE.0	fUTF8	軮
U+8EEE.0	fCNS11643	2-4067
U+8EEE.0	fBig5	DC42
U+8EEE.0	fGB	5-4764
U+8EEE.0	fR/S	 159.5
U+8EEE.0	fTotalStrokes	 12
U+8EEE.0	fCangjie	JJLBK
U+8EEE.0	fFourCorner	55030
U+8EEE.0	fCantonese	ong3 (also); ong2; ngong2 (also); ngong3 (also)
U+8EEE.0	fMandarin	ang3; fu2; yang3
U+8EEF.0	fUTF8	軯
U+8EEF.0	fCNS11643	2-4062
U+8EEF.0	fBig5	DBFC
U+8EEF.0	fGB	3-4546
U+8EEF.0	fR/S	 159.5
U+8EEF.0	fTotalStrokes	 12
U+8EEF.0	fCangjie	JJMFJ
U+8EEF.0	fFourCorner	51049
U+8EEF.0	fCantonese	paang1
U+8EEF.0	fMandarin	peng1
U+8EF0.0	fUTF8	軰
U+8EF0.0	fCNS11643	4-3B55
U+8EF0.0	fGB	E-3F6A
U+8EF0.0	fR/S	 159.5
U+8EF0.0	fTotalStrokes	 12
U+8EF0.0	fCangjie	LPJWJ
U+8EF0.0	fFourCorner	11500
U+8EF0.0	fHakka	 bui5; bi5
U+8EF0.0	fCantonese	bui3
U+8EF0.0	fMandarin	bei4
U+8EF0.0	fEnglish	 a row of carriages ge- neration class; a series a kind denotes the plural
U+8EF1.0	fUTF8	軱
U+8EF1.0	fCNS11643	2-406E
U+8EF1.0	fBig5	DC49
U+8EF1.0	fGB	3-4548
U+8EF1.0	fR/S	 159.5; 159.6
U+8EF1.0	fTotalStrokes	 12
U+8EF1.0	fCangjie	JJHVO
U+8EF1.0	fFourCorner	52030
U+8EF1.0	fCantonese	gu1; gwu1 (also)
U+8EF1.0	fMandarin	gu1
U+8EF2.0	fUTF8	軲
U+8EF2.0	fCNS11643	3-3F44
U+8EF2.0	fGB	1-696F
U+8EF2.0	fR/S	  159.5
U+8EF2.0	fTotalStrokes	 12
U+8EF2.0	fCangjie	JJJR
U+8EF2.0	fFourCorner	54060
U+8EF2.0	fHakka	 ku1
U+8EF2.0	fCantonese	gu1; gwu1 (also); ku1
U+8EF2.0	fMandarin	ku1; gu1
U+8EF2.0	fEnglish	[1] wheel [2] turn; revolve; rotate
U+8EF3.0	fUTF8	軳
U+8EF3.0	fCNS11643	3-3F46
U+8EF3.0	fGB	3-454C
U+8EF3.0	fR/S	  159.5
U+8EF3.0	fTotalStrokes	 12
U+8EF3.0	fCangjie	JJPRU
U+8EF3.0	fFourCorner	57010
U+8EF3.0	fCantonese	pou4
U+8EF3.0	fMandarin	pao2
U+8EF4.0	fUTF8	軴
U+8EF4.0	fCNS11643	2-4070
U+8EF4.0	fBig5	DC4B
U+8EF4.0	fGB	E-3F6B
U+8EF4.0	fR/S	 159.5
U+8EF4.0	fTotalStrokes	 12
U+8EF4.0	fCangjie	JJYG
U+8EF4.0	fFourCorner	50014
U+8EF4.0	fCantonese	zyu3
U+8EF4.0	fMandarin	zhu4
U+8EF5.0	fUTF8	軵
U+8EF5.0	fCNS11643	2-4069
U+8EF5.0	fBig5	DC44
U+8EF5.0	fGB	3-4547
U+8EF5.0	fR/S	 159.5
U+8EF5.0	fTotalStrokes	 12
U+8EF5.0	fCangjie	JJODI
U+8EF5.0	fFourCorner	54000
U+8EF5.0	fHakka	 fu5; jung3; rung3
U+8EF5.0	fCantonese	fu2; jung5 (also); fu6
U+8EF5.0	fMandarin	rong3; fu3; rong2
U+8EF5.0	fEnglish	 tilt up a cart
U+8EF6.0	fUTF8	軶
U+8EF6.0	fCNS11643	2-406C
U+8EF6.0	fBig5	DC47
U+8EF6.0	fGB	5-4765
U+8EF6.0	fR/S	 159.5
U+8EF6.0	fTotalStrokes	 12
U+8EF6.0	fCangjie	JJHSN
U+8EF6.0	fFourCorner	51017
U+8EF6.0	fHakka	 ak7
U+8EF6.0	fCantonese	aak1; ngaak1 (also); ak1
U+8EF6.0	fMandarin	e4
U+8EF7.0	fUTF8	軷
U+8EF7.0	fCNS11643	2-4063
U+8EF7.0	fBig5	DBFD
U+8EF7.0	fGB	3-4545
U+8EF7.0	fR/S	 159.5
U+8EF7.0	fTotalStrokes	 12
U+8EF7.0	fCangjie	JJIKK
U+8EF7.0	fFourCorner	53047
U+8EF7.0	fHakka	  pat8
U+8EF7.0	fCantonese	bat6; but6
U+8EF7.0	fMandarin	ba2; pei4
U+8EF7.0	fEnglish	sacrifice
U+8EF8.0	fUTF8	軸
U+8EF8.0	fCNS11643	1-6249
U+8EF8.0	fBig5	B662
U+8EF8.0	fGB	1-5661
U+8EF8.0	fR/S	 159.5
U+8EF8.0	fTotalStrokes	 12
U+8EF8.0	fCangjie	JJLW
U+8EF8.0	fFourCorner	55060
U+8EF8.0	fHakka	 cuk8; ch'uk8; chuk8
U+8EF8.0	fCantonese	zuk6; zau6
U+8EF8.0	fMandarin	zhou2; zhou4; zhu2
U+8EF8.0	fEnglish	[1] axle; pivot; axis [2] scroll (mounted paintings or calligraphic works
U+8EF9.0	fUTF8	軹
U+8EF9.0	fCNS11643	2-4065
U+8EF9.0	fBig5	DC40
U+8EF9.0	fGB	1-6972
U+8EF9.0	fR/S	 159.5
U+8EF9.0	fTotalStrokes	 12
U+8EF9.0	fCangjie	JJRC
U+8EF9.0	fFourCorner	56080
U+8EF9.0	fHakka	 zhi3; zih3; tje3
U+8EF9.0	fCantonese	zi2
U+8EF9.0	fMandarin	zhi3
U+8EF9.0	fEnglish	the ends of an axle
U+8EFA.0	fUTF8	軺
U+8EFA.0	fCNS11643	2-4064
U+8EFA.0	fBig5	DBFE
U+8EFA.0	fGB	1-6977
U+8EFA.0	fR/S	 159.5
U+8EFA.0	fTotalStrokes	 12
U+8EFA.0	fCangjie	JJSHR
U+8EFA.0	fFourCorner	57062
U+8EFA.0	fHakka	 shau2; rau2; jeu2; sau2; jau2
U+8EFA.0	fCantonese	jiu4
U+8EFA.0	fMandarin	yao2; diao1; shao2
U+8EFA.0	fEnglish	a small light carriage; a kind of light carriage
U+8EFB.0	fUTF8	軻
U+8EFB.0	fCNS11643	1-6248
U+8EFB.0	fBig5	B661
U+8EFB.0	fGB	1-6970
U+8EFB.0	fR/S	 159.5
U+8EFB.0	fTotalStrokes	 12
U+8EFB.0	fCangjie	JJMNR
U+8EFB.0	fFourCorner	51020
U+8EFB.0	fHakka	  ko1; ko2
U+8EFB.0	fCantonese	ngo1 (also); ho2; o1
U+8EFB.0	fMandarin	ke1; ke3; ke4
U+8EFB.0	fEnglish	[1] the name of Mencius [2] a kind of ancient cart
U+8EFC.0	fUTF8	軼
U+8EFC.0	fCNS11643	1-624A
U+8EFC.0	fBig5	B663
U+8EFC.0	fGB	1-6973
U+8EFC.0	fR/S	 159.5
U+8EFC.0	fTotalStrokes	 12
U+8EFC.0	fCangjie	JJHQO
U+8EFC.0	fFourCorner	55030
U+8EFC.0	fHakka	 jit8; zhit7; rit7; jit7; tsiit7
U+8EFC.0	fCantonese	jat6; dit6
U+8EFC.0	fMandarin	yi4; die2; zhe2; die1
U+8EFC.0	fEnglish	[1] excel; surpass [2] be scattered; go loose
U+8EFD.0	fUTF8	軽
U+8EFD.0	fCNS11643	3-3F49
U+8EFD.0	fGB	E-3F6C
U+8EFD.0	fR/S	 159.5
U+8EFD.0	fTotalStrokes	 12
U+8EFD.0	fCangjie	JJEG
U+8EFD.0	fFourCorner	57010
U+8EFD.0	fCantonese	zi3
U+8EFD.0	fMandarin	qing1; qing4
U+8EFD.0	fEnglish	 light easy; simple gentle
U+8EFE.0	fUTF8	軾
U+8EFE.0	fCNS11643	1-6726
U+8EFE.0	fBig5	B8FD
U+8EFE.0	fGB	1-6978
U+8EFE.0	fR/S	 159.6
U+8EFE.0	fTotalStrokes	 13
U+8EFE.0	fCangjie	JJIPM
U+8EFE.0	fFourCorner	53040
U+8EFE.0	fHakka	 shit7; siit7; sit7
U+8EFE.0	fCantonese	sik1
U+8EFE.0	fMandarin	shi4
U+8EFE.0	fEnglish	the horizontal wooden bar in front of a sedan chair, cart, or carriage
U+8EFF.0	fUTF8	軿
U+8EFF.0	fCNS11643	2-477B
U+8EFF.0	fBig5	E075
U+8EFF.0	fGB	3-4557
U+8EFF.0	fR/S	 159.6
U+8EFF.0	fTotalStrokes	 13
U+8EFF.0	fCangjie	JJTT
U+8EFF.0	fFourCorner	58041
U+8EFF.0	fHakka	 pen1
U+8EFF.0	fCantonese	ping4
U+8EFF.0	fMandarin	ping2; peng1; pian2
U+8EFF.0	fEnglish	a curtained carriage used by women
U+8F00.0	fUTF8	輀
U+8F00.0	fCNS11643	2-477D
U+8F00.0	fBig5	E077
U+8F00.0	fGB	3-4553
U+8F00.0	fR/S	 159.6
U+8F00.0	fTotalStrokes	 13
U+8F00.0	fCangjie	JJMBL
U+8F00.0	fFourCorner	51027
U+8F00.0	fHakka	 ji2; ri2
U+8F00.0	fCantonese	ji4
U+8F00.0	fMandarin	er2
U+8F00.0	fEnglish	hearse
U+8F01.0	fUTF8	輁
U+8F01.0	fCNS11643	2-477C
U+8F01.0	fBig5	E076
U+8F01.0	fGB	3-4551
U+8F01.0	fR/S	 159.6
U+8F01.0	fTotalStrokes	 13
U+8F01.0	fCangjie	JJTC
U+8F01.0	fFourCorner	54081
U+8F01.0	fCantonese	gung2
U+8F01.0	fMandarin	qiong2; gong3
U+8F02.0	fUTF8	輂
U+8F02.0	fCNS11643	2-4823
U+8F02.0	fBig5	E07B
U+8F02.0	fGB	3-4552
U+8F02.0	fR/S	 159.6
U+8F02.0	fTotalStrokes	 13
U+8F02.0	fCangjie	TCJWJ
U+8F02.0	fFourCorner	44503
U+8F02.0	fHakka	 kiuk8
U+8F02.0	fCantonese	guk1; guk6 (also)
U+8F02.0	fMandarin	ju2
U+8F02.0	fEnglish	 dung cart
U+8F03.0	fUTF8	較
U+8F03.0	fCNS11643	1-6724
U+8F03.0	fBig5	B8FB
U+8F03.0	fGB	1-3D4F
U+8F03.0	fR/S	 159.6
U+8F03.0	fTotalStrokes	 13
U+8F03.0	fCangjie	JJYCK
U+8F03.0	fFourCorner	50048
U+8F03.0	fHakka	  gau3; gok7;  gau5; ha5
U+8F03.0	fCantonese	gaau3; gok3
U+8F03.0	fMandarin	jiao4; jue2; jiao3; xiao4
U+8F03.0	fEnglish	[1] compare [2] in a greater or lesser degree; more or less; earlier or later [3] clear; conspicuous; obvious; marked [4] compete; dispute [5] fairly; quite; relatively
U+8F04.0	fUTF8	輄
U+8F04.0	fCNS11643	3-4476
U+8F04.0	fGB	E-3F6D
U+8F04.0	fR/S	  159.6
U+8F04.0	fTotalStrokes	 13
U+8F04.0	fCangjie	JJFMU
U+8F04.0	fFourCorner	59010
U+8F04.0	fCantonese	gwong1; gong1 (also)
U+8F04.0	fMandarin	guang1
U+8F05.0	fUTF8	輅
U+8F05.0	fCNS11643	2-477E
U+8F05.0	fBig5	E078
U+8F05.0	fGB	1-697B
U+8F05.0	fR/S	 159.6
U+8F05.0	fTotalStrokes	 13
U+8F05.0	fCangjie	JJHER
U+8F05.0	fFourCorner	57064
U+8F05.0	fHakka	 lu5; lu6
U+8F05.0	fCantonese	lou6
U+8F05.0	fMandarin	lu4; he2; ya4; luo4
U+8F05.0	fEnglish	[1] heavy carriage [2] the horizontal front bar on a cart or carriage [3] state carriage
U+8F06.0	fUTF8	輆
U+8F06.0	fCNS11643	2-477A
U+8F06.0	fBig5	E074
U+8F06.0	fGB	3-4556
U+8F06.0	fR/S	 159.6
U+8F06.0	fTotalStrokes	 13
U+8F06.0	fCangjie	JJYVO
U+8F06.0	fFourCorner	50082
U+8F06.0	fCantonese	hoi2; koi3
U+8F06.0	fMandarin	kai3; kai4
U+8F07.0	fUTF8	輇
U+8F07.0	fCNS11643	2-4821
U+8F07.0	fBig5	E079
U+8F07.0	fGB	1-697A
U+8F07.0	fR/S	 159.6
U+8F07.0	fTotalStrokes	 13
U+8F07.0	fCangjie	JJOMG
U+8F07.0	fFourCorner	58014
U+8F07.0	fHakka	  cien2; cion2; zien1
U+8F07.0	fCantonese	cyun4; syun4
U+8F07.0	fMandarin	quan2; chun1; chuan2
U+8F07.0	fEnglish	cart wheel with no spokes
U+8F08.0	fUTF8	輈
U+8F08.0	fCNS11643	2-4822
U+8F08.0	fBig5	E07A
U+8F08.0	fGB	3-4554
U+8F08.0	fR/S	 159.6
U+8F08.0	fTotalStrokes	 13
U+8F08.0	fCangjie	JJHBY
U+8F08.0	fFourCorner	57040
U+8F08.0	fHakka	 zhiu1; zu1
U+8F08.0	fCantonese	zau1
U+8F08.0	fMandarin	zhou1
U+8F08.0	fEnglish	the shaft or pole (of a cart or carriage)
U+8F09.0	fUTF8	載
U+8F09.0	fCNS11643	1-6725
U+8F09.0	fBig5	B8FC
U+8F09.0	fGB	1-5458
U+8F09.0	fR/S	 159.6
U+8F09.0	fTotalStrokes	 13
U+8F09.0	fCangjie	JIJWJ
U+8F09.0	fFourCorner	43550
U+8F09.0	fHakka	  zoi5; zhoi5; zai3
U+8F09.0	fCantonese	zoi3; zoi2
U+8F09.0	fMandarin	zai4; dai4; zai1; zi1
U+8F09.0	fEnglish	[1] load; carry; transport; convey (vehicles, vessels, etc.) [2] record; publish [3] fill
U+8F09.1	fUTF8	載
U+8F09.1	fCNS11643	1-6725
U+8F09.1	fBig5	B8FC
U+8F09.1	fGB	1-5458
U+8F09.1	fR/S	 159.6
U+8F09.1	fTotalStrokes	 13
U+8F09.1	fCangjie	JIJWJ
U+8F09.1	fFourCorner	43550
U+8F09.1	fHakka	 zai5
U+8F09.1	fMandarin	zai3
U+8F09.1	fEnglish	 year
U+8F0A.0	fUTF8	輊
U+8F0A.0	fCNS11643	1-6727
U+8F0A.0	fBig5	B8FE
U+8F0A.0	fGB	1-6979
U+8F0A.0	fR/S	 159.6
U+8F0A.0	fTotalStrokes	 13
U+8F0A.0	fCangjie	JJMIG
U+8F0A.0	fFourCorner	51014
U+8F0A.0	fHakka	 zhi5; zu5; zih5; tje5
U+8F0A.0	fCantonese	zi3
U+8F0A.0	fMandarin	zhi4; shi4
U+8F0A.0	fEnglish	low rear portion of cart
U+8F0B.0	fUTF8	輋
U+8F0B.0	fCNS11643	2-4824
U+8F0B.0	fBig5	E07C
U+8F0B.0	fGB	5-3B2A
U+8F0B.0	fR/S	 159.6
U+8F0B.0	fTotalStrokes	 13
U+8F0B.0	fCangjie	UKJJ
U+8F0B.0	fFourCorner	22502
U+8F0B.0	fHakka	 can3; san1; cia2
U+8F0B.0	fCantonese	ce4; se4
U+8F0B.0	fMandarin	ju1; she1
U+8F0C.0	fUTF8	輌
U+8F0C.0	fGB	E-3F6E
U+8F0C.0	fR/S	 159.6
U+8F0C.0	fTotalStrokes	 13
U+8F0C.0	fCangjie	JJMUB
U+8F0C.0	fCantonese	loeng6
U+8F0C.0	fMandarin	liang4
U+8F0C.0	fEnglish	numerary adjunct for vehicles
U+8F0D.0	fUTF8	輍
U+8F0D.0	fCNS11643	2-4E4F
U+8F0D.0	fBig5	E467
U+8F0D.0	fGB	E-3F6F
U+8F0D.0	fR/S	 159.7
U+8F0D.0	fTotalStrokes	 14
U+8F0D.0	fCangjie	JJCOR
U+8F0D.0	fFourCorner	58068
U+8F0D.0	fCantonese	juk6
U+8F0D.0	fMandarin	yu4
U+8F0E.0	fUTF8	輎
U+8F0E.0	fCNS11643	2-4E4E
U+8F0E.0	fBig5	E466
U+8F0E.0	fGB	E-3F70
U+8F0E.0	fR/S	 159.7
U+8F0E.0	fTotalStrokes	 14
U+8F0E.0	fCangjie	JJFB
U+8F0E.0	fFourCorner	59027
U+8F0E.0	fCantonese	saau1
U+8F0E.0	fMandarin	shao1
U+8F0F.0	fUTF8	輏
U+8F0F.0	fCNS11643	4-476A
U+8F0F.0	fGB	E-3F71
U+8F0F.0	fR/S	 159.7
U+8F0F.0	fTotalStrokes	 14
U+8F0F.0	fCangjie	JJMCW
U+8F0F.0	fFourCorner	51060
U+8F0F.0	fCantonese	jau4
U+8F0F.0	fMandarin	you2
U+8F0F.0	fEnglish	 light light carriage
U+8F10.0	fUTF8	輐
U+8F10.0	fCNS11643	2-4E4C
U+8F10.0	fBig5	E464
U+8F10.0	fGB	5-476B
U+8F10.0	fR/S	 159.7
U+8F10.0	fTotalStrokes	 14
U+8F10.0	fCangjie	JJJMU
U+8F10.0	fFourCorner	53011
U+8F10.0	fCantonese	wun6
U+8F10.0	fMandarin	huan3; wan4
U+8F11.0	fUTF8	輑
U+8F11.0	fCNS11643	2-4E4D
U+8F11.0	fBig5	E465
U+8F11.0	fGB	3-455C
U+8F11.0	fR/S	 159.7
U+8F11.0	fTotalStrokes	 14
U+8F11.0	fCangjie	JJSKR
U+8F11.0	fFourCorner	57067
U+8F11.0	fCantonese	wan5; kwan1
U+8F11.0	fMandarin	yun3; qun1
U+8F12.0	fUTF8	輒
U+8F12.0	fCNS11643	1-6B3B
U+8F12.0	fBig5	BBB3
U+8F12.0	fGB	1-697C
U+8F12.0	fR/S	 159.7
U+8F12.0	fTotalStrokes	 14
U+8F12.0	fCangjie	JJSJU
U+8F12.0	fFourCorner	51010
U+8F12.0	fHakka	 zhap7; zap7; ziap8
U+8F12.0	fCantonese	zip3; zit3 (also)
U+8F12.0	fMandarin	zhe2
U+8F12.0	fEnglish	[1] sides of a chariot, etc. that point downward at the front [2] arbitrary; dictatorial [3] then; in that case [4] every time; always
U+8F13.0	fUTF8	輓
U+8F13.0	fCNS11643	1-6B3D
U+8F13.0	fBig5	BBB5
U+8F13.0	fGB	E-3F72
U+8F13.0	fR/S	 159.7
U+8F13.0	fTotalStrokes	 14
U+8F13.0	fCangjie	JJNAU
U+8F13.0	fFourCorner	57016
U+8F13.0	fHakka	  wan3; man1; van3
U+8F13.0	fCantonese	waan5; maan5
U+8F13.0	fMandarin	wan3; wan4
U+8F13.0	fEnglish	[1] mourn [2] pull; draw (cart) [3] late
U+8F14.0	fUTF8	輔
U+8F14.0	fCNS11643	1-6B3A
U+8F14.0	fBig5	BBB2
U+8F14.0	fGB	1-3828
U+8F14.0	fR/S	 159.7
U+8F14.0	fTotalStrokes	 14
U+8F14.0	fCangjie	JJIJB
U+8F14.0	fFourCorner	53027
U+8F14.0	fHakka	 pu3; fu5; fu3
U+8F14.0	fCantonese	fu6
U+8F14.0	fMandarin	fu3
U+8F14.0	fEnglish	[1] the human cheeks [2] protective bars on both sides of a cart or carriage [3] help; assist; complement
U+8F15.0	fUTF8	輕
U+8F15.0	fCNS11643	1-6B3C
U+8F15.0	fBig5	BBB4
U+8F15.0	fGB	1-4761
U+8F15.0	fR/S	 159.7
U+8F15.0	fTotalStrokes	 14
U+8F15.0	fCangjie	JJMVM
U+8F15.0	fFourCorner	51011
U+8F15.0	fHakka	  kiang1;  kin1
U+8F15.0	fCantonese	hing1; heng1 (also); heng6 (also)
U+8F15.0	fMandarin	qing1; qing4
U+8F15.0	fEnglish	[1] light [2] easy; simple; facile [3] gentle; mild; soft; tender; lightly; gently; softly [4] mean; base; lowly; unimportant [5] frivolous; flippant; fickle; rash; reckless [6] slight; neglect; ignore; underestimate; disparage
U+8F16.0	fUTF8	輖
U+8F16.0	fCNS11643	2-5475
U+8F16.0	fBig5	E84D
U+8F16.0	fGB	3-4568
U+8F16.0	fR/S	 159.8
U+8F16.0	fTotalStrokes	 15
U+8F16.0	fCangjie	JJBGR
U+8F16.0	fFourCorner	57020
U+8F16.0	fHakka	 zhiu1; zu1; zhu1
U+8F16.0	fCantonese	zau1
U+8F16.0	fMandarin	zhou1
U+8F16.0	fEnglish	heavily loaded cart
U+8F17.0	fUTF8	輗
U+8F17.0	fCNS11643	2-5476
U+8F17.0	fBig5	E84E
U+8F17.0	fGB	3-4565
U+8F17.0	fR/S	 159.8
U+8F17.0	fTotalStrokes	 15
U+8F17.0	fCangjie	JJHXU
U+8F17.0	fFourCorner	57017
U+8F17.0	fHakka	  ngi2
U+8F17.0	fCantonese	ngai4; ai4 (also)
U+8F17.0	fMandarin	ni2; yi4
U+8F17.0	fEnglish	the crossbar at end of carriage poles
U+8F18.0	fUTF8	輘
U+8F18.0	fCNS11643	2-5471
U+8F18.0	fBig5	E849
U+8F18.0	fGB	3-4560
U+8F18.0	fR/S	 159.8
U+8F18.0	fTotalStrokes	 15
U+8F18.0	fCangjie	JJGCE
U+8F18.0	fFourCorner	54047
U+8F18.0	fHakka	 lin2
U+8F18.0	fCantonese	ling4
U+8F18.0	fMandarin	ling2; leng2; leng4
U+8F18.0	fEnglish	run over by a vehicle
U+8F19.0	fUTF8	輙
U+8F19.0	fCNS11643	3-4F36
U+8F19.0	fGB	E-3F73
U+8F19.0	fR/S	 159.8
U+8F19.0	fTotalStrokes	 15
U+8F19.0	fCangjie	JJSJE
U+8F19.0	fFourCorner	57040
U+8F19.0	fHakka	 zhap7; ziap8; zap7
U+8F19.0	fCantonese	zip3; zit3 (also)
U+8F19.0	fMandarin	zhe2
U+8F19.0	fEnglish	same as $
U+8F1A.0	fUTF8	輚
U+8F1A.0	fCNS11643	2-5472
U+8F1A.0	fBig5	E84A
U+8F1A.0	fGB	3-4544
U+8F1A.0	fR/S	 159.8
U+8F1A.0	fTotalStrokes	 15
U+8F1A.0	fCangjie	JJII
U+8F1A.0	fFourCorner	53053
U+8F1A.0	fHakka	 zan5; can5
U+8F1A.0	fCantonese	zaan6
U+8F1A.0	fMandarin	zhan4
U+8F1A.0	fEnglish	a carriage used by military officials
U+8F1B.0	fUTF8	輛
U+8F1B.0	fCNS11643	1-6F43
U+8F1B.0	fBig5	BDF8
U+8F1B.0	fGB	1-413E
U+8F1B.0	fR/S	 159.8
U+8F1B.0	fTotalStrokes	 15
U+8F1B.0	fCangjie	JJMLB
U+8F1B.0	fFourCorner	51027
U+8F1B.0	fHakka	  liong2;  liong3; liong5
U+8F1B.0	fCantonese	loeng6; loeng2 (also)
U+8F1B.0	fMandarin	liang4; liang3
U+8F1B.0	fEnglish	numerical classifier for vehicles
U+8F1C.0	fUTF8	輜
U+8F1C.0	fCNS11643	1-6F48
U+8F1C.0	fBig5	BDFD
U+8F1C.0	fGB	1-6A22
U+8F1C.0	fR/S	 159.8
U+8F1C.0	fTotalStrokes	 15
U+8F1C.0	fCangjie	JJVVW
U+8F1C.0	fFourCorner	52063
U+8F1C.0	fHakka	 zih1; zu1; zii1
U+8F1C.0	fCantonese	zi1
U+8F1C.0	fMandarin	zi1; zi4
U+8F1C.0	fEnglish	supply cart; covered wagon; dray
U+8F1D.0	fUTF8	輝
U+8F1D.0	fCNS11643	1-6F42
U+8F1D.0	fBig5	BDF7
U+8F1D.0	fGB	1-3B54
U+8F1D.0	fR/S	 159.8
U+8F1D.0	fTotalStrokes	 15
U+8F1D.0	fCangjie	FUBJJ
U+8F1D.0	fFourCorner	97256
U+8F1D.0	fHakka	  fui1; fi1
U+8F1D.0	fCantonese	fai1
U+8F1D.0	fMandarin	hui1
U+8F1D.0	fEnglish	brightness; luster; brilliance; splendor; light; luminosity
U+8F1E.0	fUTF8	輞
U+8F1E.0	fCNS11643	1-6F49
U+8F1E.0	fBig5	BDFE
U+8F1E.0	fGB	1-697E
U+8F1E.0	fR/S	 159.8
U+8F1E.0	fTotalStrokes	 15
U+8F1E.0	fCangjie	JJBTV
U+8F1E.0	fFourCorner	57020
U+8F1E.0	fHakka	 wong3; miong3; mong3; vong3
U+8F1E.0	fCantonese	mong5
U+8F1E.0	fMandarin	wang3
U+8F1E.0	fEnglish	 rim of wheel; felly; felloe
U+8F1F.0	fUTF8	輟
U+8F1F.0	fCNS11643	1-6F44
U+8F1F.0	fBig5	BDF9
U+8F1F.0	fGB	1-6A21
U+8F1F.0	fR/S	 159.8
U+8F1F.0	fTotalStrokes	 15
U+8F1F.0	fCangjie	JJEEE
U+8F1F.0	fFourCorner	57047
U+8F1F.0	fHakka	  dot7
U+8F1F.0	fCantonese	zyut3; zyut6 (also)
U+8F1F.0	fMandarin	chuo4; zhui4; zhuo2
U+8F1F.0	fEnglish	 stop; suspend; halt; cease
U+8F20.0	fUTF8	輠
U+8F20.0	fCNS11643	2-5473
U+8F20.0	fBig5	E84B
U+8F20.0	fGB	3-4564
U+8F20.0	fR/S	 159.8
U+8F20.0	fTotalStrokes	 15
U+8F20.0	fCangjie	JJWD
U+8F20.0	fFourCorner	56094
U+8F20.0	fHakka	 ko5; go3; gwo3; kwo3
U+8F20.0	fCantonese	gwo2; go2 (also); waa5; wui6
U+8F20.0	fMandarin	guo3; hua4; hui4; guo1; huo3
U+8F20.0	fEnglish	grease pot hung under axle of a car
U+8F21.0	fUTF8	輡
U+8F21.0	fCNS11643	4-4E2D
U+8F21.0	fGB	5-476C
U+8F21.0	fR/S	  159.8
U+8F21.0	fTotalStrokes	 15
U+8F21.0	fCangjie	JJNHX
U+8F21.0	fFourCorner	57070
U+8F21.0	fCantonese	ham2
U+8F21.0	fMandarin	kan3
U+8F21.0	fEnglish	same as &#x8F57;
U+8F22.0	fUTF8	輢
U+8F22.0	fCNS11643	3-4F33
U+8F22.0	fGB	3-4561
U+8F22.0	fR/S	  159.8
U+8F22.0	fTotalStrokes	 15
U+8F22.0	fCangjie	JJKMR
U+8F22.0	fFourCorner	54020
U+8F22.0	fCantonese	ji2
U+8F22.0	fMandarin	yi3
U+8F23.0	fUTF8	輣
U+8F23.0	fCNS11643	2-5474
U+8F23.0	fBig5	E84C
U+8F23.0	fGB	3-4567
U+8F23.0	fR/S	 159.8
U+8F23.0	fTotalStrokes	 15
U+8F23.0	fCangjie	JJBB
U+8F23.0	fFourCorner	57020
U+8F23.0	fHakka	 pen2
U+8F23.0	fCantonese	paang4; pang4
U+8F23.0	fMandarin	peng2
U+8F24.0	fUTF8	輤
U+8F24.0	fCNS11643	2-5470
U+8F24.0	fBig5	E848
U+8F24.0	fGB	3-455D
U+8F24.0	fR/S	 159.8
U+8F24.0	fTotalStrokes	 15
U+8F24.0	fCangjie	JJQMB
U+8F24.0	fFourCorner	55027
U+8F24.0	fHakka	  cien5
U+8F24.0	fCantonese	sin3
U+8F24.0	fMandarin	qian4
U+8F24.0	fEnglish	a pall to cover the hearse
U+8F25.0	fUTF8	輥
U+8F25.0	fCNS11643	1-6F4A
U+8F25.0	fBig5	BE40
U+8F25.0	fGB	1-3975
U+8F25.0	fR/S	 159.8
U+8F25.0	fTotalStrokes	 15
U+8F25.0	fCangjie	JJAPP
U+8F25.0	fFourCorner	56011
U+8F25.0	fHakka	  gun3; gwun3
U+8F25.0	fCantonese	gwan2; kwan2
U+8F25.0	fMandarin	gun3; hun4
U+8F25.0	fEnglish	 turn round; revolve roller
U+8F26.0	fUTF8	輦
U+8F26.0	fCNS11643	1-6F46
U+8F26.0	fBig5	BDFB
U+8F26.0	fGB	1-697D
U+8F26.0	fR/S	 159.8
U+8F26.0	fTotalStrokes	 15
U+8F26.0	fCangjie	QOJWJ
U+8F26.0	fFourCorner	55506
U+8F26.0	fHakka	  lien3; len3; lien6; lien5
U+8F26.0	fCantonese	lin5
U+8F26.0	fMandarin	nian3; lian3
U+8F26.0	fEnglish	[1] hand-cart [2] king's carriage; imperial carriage
U+8F27.0	fUTF8	輧
U+8F27.0	fCNS11643	3-4F35
U+8F27.0	fGB	E-3F74
U+8F27.0	fR/S	 159.8
U+8F27.0	fTotalStrokes	 15
U+8F27.0	fCangjie	XJJYJ
U+8F27.0	fFourCorner	58040
U+8F27.0	fHakka	  pen1;  piang2; bin3
U+8F27.0	fCantonese	ping4
U+8F27.0	fMandarin	ping2; pian2
U+8F27.0	fEnglish	curtained carriage used by women
U+8F28.0	fUTF8	輨
U+8F28.0	fCNS11643	4-4E2B
U+8F28.0	fGB	3-456B
U+8F28.0	fR/S	  159.8
U+8F28.0	fTotalStrokes	 15
U+8F28.0	fCangjie	JJJRR
U+8F28.0	fFourCorner	53070
U+8F28.0	fHakka	 kwan3; gon3; gwon5; gwon3
U+8F28.0	fCantonese	gun2; gwun2 (also)
U+8F28.0	fMandarin	guan3
U+8F28.0	fEnglish	iron band on the hub of a wheel to prevent it from splitting
U+8F29.0	fUTF8	輩
U+8F29.0	fCNS11643	1-6F45
U+8F29.0	fBig5	BDFA
U+8F29.0	fGB	1-3132
U+8F29.0	fR/S	 159.8
U+8F29.0	fTotalStrokes	 15
U+8F29.0	fCangjie	LYJWJ
U+8F29.0	fFourCorner	11506
U+8F29.0	fHakka	 bui5; bi5; bui1; boi5
U+8F29.0	fCantonese	bui3; bei3
U+8F29.0	fMandarin	bei4
U+8F29.0	fEnglish	[1] generation; lifetime [2] rank; grade; class
U+8F2A.0	fUTF8	輪
U+8F2A.0	fCNS11643	1-6F47
U+8F2A.0	fBig5	BDFC
U+8F2A.0	fGB	1-4256
U+8F2A.0	fR/S	 159.8
U+8F2A.0	fTotalStrokes	 15
U+8F2A.0	fCangjie	JJOMB
U+8F2A.0	fFourCorner	58027
U+8F2A.0	fHakka	  lun2;  lin1;  lin2; lin5; lun3
U+8F2A.0	fCantonese	leon4; leon2 (also)
U+8F2A.0	fMandarin	lun2; lun1
U+8F2A.0	fEnglish	[1] wheel [2] alternate; recur [3] majestic; stately [4] take turns; by turns; in relays
U+8F2B.0	fUTF8	輫
U+8F2B.0	fCNS11643	3-4F34
U+8F2B.0	fGB	3-4562
U+8F2B.0	fR/S	  159.8
U+8F2B.0	fTotalStrokes	 15
U+8F2B.0	fCangjie	JJLMY
U+8F2B.0	fFourCorner	51010
U+8F2B.0	fCantonese	paai4; pui4 (also)
U+8F2B.0	fMandarin	pai2; pei2
U+8F2C.0	fUTF8	輬
U+8F2C.0	fCNS11643	2-546F
U+8F2C.0	fBig5	E847
U+8F2C.0	fGB	3-4569
U+8F2C.0	fR/S	 159.8
U+8F2C.0	fTotalStrokes	 15
U+8F2C.0	fCangjie	JJYRF
U+8F2C.0	fFourCorner	50096
U+8F2C.0	fHakka	  liong2
U+8F2C.0	fCantonese	loeng4
U+8F2C.0	fMandarin	liang2
U+8F2C.0	fEnglish	[1] hearse [2] sleeping carriage
U+8F2D.0	fUTF8	輭
U+8F2D.0	fCNS11643	3-5324
U+8F2D.0	fGB	E-3F75
U+8F2D.0	fR/S	 159.9
U+8F2D.0	fTotalStrokes	 16
U+8F2D.0	fCangjie	JJMBK
U+8F2D.0	fFourCorner	51030
U+8F2D.0	fHakka	 ngion1;  ngion3; nion1
U+8F2D.0	fCantonese	jyun5
U+8F2D.0	fMandarin	ruan3
U+8F2D.0	fEnglish	same as $
U+8F2E.0	fUTF8	輮
U+8F2E.0	fCNS11643	2-5A76
U+8F2E.0	fBig5	EBCA
U+8F2E.0	fGB	3-4575
U+8F2E.0	fR/S	 159.9
U+8F2E.0	fTotalStrokes	 16
U+8F2E.0	fCangjie	JJNHD
U+8F2E.0	fFourCorner	57094
U+8F2E.0	fHakka	 ngu3; ngiu3; riu2; jiu2; ju3
U+8F2E.0	fCantonese	jau4; jau5
U+8F2E.0	fMandarin	rou2; rou3; rou4
U+8F2E.0	fEnglish	exterior rim of wheel; felly
U+8F2F.0	fUTF8	輯
U+8F2F.0	fCNS11643	1-7252
U+8F2F.0	fBig5	BFE8
U+8F2F.0	fGB	1-3C2D
U+8F2F.0	fR/S	 159.9
U+8F2F.0	fTotalStrokes	 16
U+8F2F.0	fCangjie	JJRSJ
U+8F2F.0	fFourCorner	56041
U+8F2F.0	fHakka	  cip8; sip8; cip7
U+8F2F.0	fCantonese	cap1
U+8F2F.0	fMandarin	ji2
U+8F2F.0	fEnglish	[1] friendly [2] collect; gather; compile
U+8F2F.1	fUTF8	輯
U+8F2F.1	fCNS11643	1-7252
U+8F2F.1	fBig5	BFE8
U+8F2F.1	fGB	1-3C2D
U+8F2F.1	fR/S	 159.9
U+8F2F.1	fTotalStrokes	 16
U+8F2F.1	fCangjie	JJRSJ
U+8F2F.1	fFourCorner	56041
U+8F2F.1	fMandarin	ji5
U+8F30.0	fUTF8	輰
U+8F30.0	fCNS11643	3-5322
U+8F30.0	fGB	3-453B
U+8F30.0	fR/S	 159.9
U+8F30.0	fTotalStrokes	 16
U+8F30.0	fCangjie	JJAMH
U+8F30.0	fFourCorner	56020
U+8F30.0	fCantonese	joeng4
U+8F30.0	fMandarin	yang2
U+8F31.0	fUTF8	輱
U+8F31.0	fCNS11643	3-5321
U+8F31.0	fGB	E-3F76
U+8F31.0	fR/S	 159.9
U+8F31.0	fTotalStrokes	 16
U+8F31.0	fCangjie	JJIHR
U+8F31.0	fFourCorner	53050
U+8F31.0	fCantonese	haam4
U+8F31.0	fMandarin	xian2
U+8F32.0	fUTF8	輲
U+8F32.0	fCNS11643	2-5A78
U+8F32.0	fBig5	EBCC
U+8F32.0	fGB	3-456F
U+8F32.0	fR/S	 159.9
U+8F32.0	fTotalStrokes	 16
U+8F32.0	fCangjie	JJUMB
U+8F32.0	fFourCorner	52027
U+8F32.0	fCantonese	syun4 (also); cyun4; jyun5
U+8F32.0	fMandarin	chuan2; shuan4
U+8F33.0	fUTF8	輳
U+8F33.0	fCNS11643	1-7254
U+8F33.0	fBig5	BFEA
U+8F33.0	fGB	1-6A23
U+8F33.0	fR/S	 159.9
U+8F33.0	fTotalStrokes	 16
U+8F33.0	fCangjie	JJQKK
U+8F33.0	fFourCorner	55034
U+8F33.0	fHakka	  ceu5; ciu5
U+8F33.0	fCantonese	cau3
U+8F33.0	fMandarin	cou4
U+8F33.0	fEnglish	 converge (as spokes of a wheel do at the hub)
U+8F34.0	fUTF8	輴
U+8F34.0	fCNS11643	2-5A7B
U+8F34.0	fBig5	EBCF
U+8F34.0	fGB	3-4571
U+8F34.0	fR/S	 159.9
U+8F34.0	fTotalStrokes	 16
U+8F34.0	fCangjie	JJHJU
U+8F34.0	fFourCorner	52064
U+8F34.0	fHakka	 chun1; dun3; cun1
U+8F34.0	fCantonese	ceon1
U+8F34.0	fMandarin	chun1; shun1; qun1
U+8F34.0	fEnglish	sled
U+8F35.0	fUTF8	輵
U+8F35.0	fCNS11643	2-5A77
U+8F35.0	fBig5	EBCB
U+8F35.0	fGB	5-476F
U+8F35.0	fR/S	 159.9
U+8F35.0	fTotalStrokes	 16
U+8F35.0	fCangjie	JJAPV
U+8F35.0	fFourCorner	56027
U+8F35.0	fHakka	 got7; hot7
U+8F35.0	fCantonese	got3
U+8F35.0	fMandarin	ge2; e4; ke3; ya4
U+8F35.0	fEnglish	great array of spears and chariots
U+8F36.0	fUTF8	輶
U+8F36.0	fCNS11643	2-5A75
U+8F36.0	fBig5	EBC9
U+8F36.0	fGB	3-4573
U+8F36.0	fR/S	 159.9
U+8F36.0	fTotalStrokes	 16
U+8F36.0	fCangjie	JJTCW
U+8F36.0	fFourCorner	58061
U+8F36.0	fHakka	 ju2; riu2; ju3
U+8F36.0	fCantonese	jau4; jau5
U+8F36.0	fMandarin	you2
U+8F36.0	fEnglish	[1] light [2] light carriage
U+8F37.0	fUTF8	輷
U+8F37.0	fCNS11643	2-5A7A
U+8F37.0	fBig5	EBCE
U+8F37.0	fGB	5-4770
U+8F37.0	fR/S	 159.9
U+8F37.0	fTotalStrokes	 16
U+8F37.0	fCangjie	JJPYR
U+8F37.0	fFourCorner	57020
U+8F37.0	fCantonese	gwang1
U+8F37.0	fMandarin	hong1; hong4
U+8F38.0	fUTF8	輸
U+8F38.0	fCNS11643	1-7253
U+8F38.0	fBig5	BFE9
U+8F38.0	fGB	1-4A64
U+8F38.0	fR/S	 159.9
U+8F38.0	fTotalStrokes	 16
U+8F38.0	fCangjie	JJOMN
U+8F38.0	fFourCorner	58021
U+8F38.0	fHakka	 su1; shu1
U+8F38.0	fCantonese	syu1
U+8F38.0	fMandarin	shu1; shu4
U+8F38.0	fEnglish	[1] transport; convey; carry; haul [2] hand in; contribute; donate; submit [3] be beaten; lose
U+8F39.0	fUTF8	輹
U+8F39.0	fCNS11643	2-5A79
U+8F39.0	fBig5	EBCD
U+8F39.0	fGB	3-4570
U+8F39.0	fR/S	 159.9
U+8F39.0	fTotalStrokes	 16
U+8F39.0	fCangjie	JJOAE
U+8F39.0	fFourCorner	58047
U+8F39.0	fHakka	 fuk7; buk7
U+8F39.0	fCantonese	fuk1; fuk6
U+8F39.0	fMandarin	fu4
U+8F39.0	fEnglish	pieces of wood holding the axle underneath a cart
U+8F3A.0	fUTF8	輺
U+8F3A.0	fCNS11643	7-3432
U+8F3A.0	fGB	E-3F77
U+8F3A.0	fR/S	 159.9
U+8F3A.0	fTotalStrokes	 16
U+8F3A.0	fCangjie	JJVMW
U+8F3A.0	fFourCorner	52060
U+8F3A.0	fCantonese	zi1
U+8F3A.0	fMandarin	zi1; zi4
U+8F3B.0	fUTF8	輻
U+8F3B.0	fCNS11643	1-7251
U+8F3B.0	fBig5	BFE7
U+8F3B.0	fGB	1-3778
U+8F3B.0	fR/S	 159.9
U+8F3B.0	fTotalStrokes	 16
U+8F3B.0	fCangjie	JJMRW
U+8F3B.0	fFourCorner	51066
U+8F3B.0	fHakka	 fuk7; buk7
U+8F3B.0	fCantonese	fuk1
U+8F3B.0	fMandarin	fu2
U+8F3B.0	fEnglish	 spokes of a wheel
U+8F3C.0	fUTF8	輼
U+8F3C.0	fCNS11643	3-5325
U+8F3C.0	fGB	3-456E
U+8F3C.0	fR/S	 159.9
U+8F3C.0	fTotalStrokes	 16
U+8F3C.0	fCangjie	JJABT
U+8F3C.0	fFourCorner	56010
U+8F3C.0	fCantonese	wan1
U+8F3C.0	fMandarin	wen1
U+8F3D.0	fUTF8	輽
U+8F3D.0	fCNS11643	3-5659
U+8F3D.0	fGB	E-3F78
U+8F3D.0	fR/S	 159.10
U+8F3D.0	fTotalStrokes	 17
U+8F3D.0	fCangjie	JJIKW
U+8F3D.0	fFourCorner	53060
U+8F3D.0	fCantonese	ban6
U+8F3D.0	fMandarin	ben4; fan4
U+8F3D.0	fEnglish	vehicle
U+8F3E.0	fUTF8	輾
U+8F3E.0	fCNS11643	1-755E
U+8F3E.0	fBig5	C1D3
U+8F3E.0	fGB	1-5537
U+8F3E.0	fR/S	 159.10
U+8F3E.0	fTotalStrokes	 17
U+8F3E.0	fCangjie	JJSTV
U+8F3E.0	fFourCorner	57032
U+8F3E.0	fHakka	 zen3; zhan3; zan5; zan3; zhan5; chan5; can5; zhen3
U+8F3E.0	fCantonese	zin2; nin5; lin5 (also); nin6
U+8F3E.0	fMandarin	zhan3; nian4; nian3
U+8F3E.0	fEnglish	 turn over; roll over
U+8F3E.1	fUTF8	輾
U+8F3E.1	fCNS11643	1-755E
U+8F3E.1	fBig5	C1D3
U+8F3E.1	fGB	1-5537
U+8F3E.1	fR/S	 159.10
U+8F3E.1	fTotalStrokes	 17
U+8F3E.1	fCangjie	JJSTV
U+8F3E.1	fFourCorner	57032
U+8F3E.1	fMandarin	nain3
U+8F3E.1	fEnglish	same as $
U+8F3F.0	fUTF8	輿
U+8F3F.0	fCNS11643	1-7561
U+8F3F.0	fBig5	C1D6
U+8F3F.0	fGB	1-535F
U+8F3F.0	fR/S	 159.10
U+8F3F.0	fTotalStrokes	 17
U+8F3F.0	fCangjie	HXJC
U+8F3F.0	fFourCorner	77801
U+8F3F.0	fHakka	 gi1; ji2; ri2; ji3; i2
U+8F3F.0	fCantonese	jyu4
U+8F3F.0	fMandarin	yu2; yu4
U+8F3F.0	fEnglish	[1] carriage; vehicle; cart [2] sedan chair [3] the land; the earth [4] all; general [5] carry; transport
U+8F40.0	fUTF8	轀
U+8F40.0	fCNS11643	2-5F6D
U+8F40.0	fBig5	EEC1
U+8F40.0	fGB	E-3F79
U+8F40.0	fR/S	 159.10
U+8F40.0	fTotalStrokes	 17
U+8F40.0	fCangjie	JJWOT
U+8F40.0	fFourCorner	56017
U+8F40.0	fCantonese	wan1
U+8F40.0	fMandarin	wen1; yun1
U+8F40.0	fEnglish	[1] hearse [2] sleeping carriage
U+8F41.0	fUTF8	轁
U+8F41.0	fCNS11643	3-565A
U+8F41.0	fGB	E-3F7A
U+8F41.0	fR/S	 159.10
U+8F41.0	fTotalStrokes	 17
U+8F41.0	fCangjie	JJBHX
U+8F41.0	fFourCorner	52070
U+8F41.0	fCantonese	tou1
U+8F41.0	fMandarin	kan3; tao1
U+8F42.0	fUTF8	轂
U+8F42.0	fCNS11643	1-755F
U+8F42.0	fBig5	C1D4
U+8F42.0	fGB	1-6C31
U+8F42.0	fR/S	 159.10
U+8F42.0	fTotalStrokes	 17
U+8F42.0	fCangjie	GJHNE
U+8F42.0	fFourCorner	47547
U+8F42.0	fHakka	 guk7; kwuk7
U+8F42.0	fCantonese	guk1; gu1; gwu1 (also)
U+8F42.0	fMandarin	gu3
U+8F42.0	fEnglish	the hub of a wheel
U+8F42.1	fUTF8	轂
U+8F42.1	fCNS11643	1-755F
U+8F42.1	fBig5	C1D4
U+8F42.1	fGB	1-6C31
U+8F42.1	fR/S	 159.10
U+8F42.1	fTotalStrokes	 17
U+8F42.1	fCangjie	GJHNE
U+8F42.1	fFourCorner	47547
U+8F42.1	fMandarin	gu1
U+8F42.1	fEnglish	 wheel
U+8F43.0	fUTF8	轃
U+8F43.0	fCNS11643	2-5F6C
U+8F43.0	fBig5	EEC0
U+8F43.0	fGB	3-4576
U+8F43.0	fR/S	 159.10
U+8F43.0	fTotalStrokes	 17
U+8F43.0	fCangjie	JJQKD
U+8F43.0	fFourCorner	55094
U+8F43.0	fCantonese	zeon1
U+8F43.0	fMandarin	zhen1; jian1
U+8F44.0	fUTF8	轄
U+8F44.0	fCNS11643	1-755D
U+8F44.0	fBig5	C1D2
U+8F44.0	fGB	1-4F3D
U+8F44.0	fR/S	 159.10
U+8F44.0	fTotalStrokes	 17
U+8F44.0	fCangjie	JJJQR
U+8F44.0	fFourCorner	53061
U+8F44.0	fHakka	  hat7;  hot7; hat8
U+8F44.0	fCantonese	hat6
U+8F44.0	fMandarin	xia2; he2; kuai4; yu4
U+8F44.0	fEnglish	[1] linchpin of a wheel [2] govern; control; administer; manage; administration
U+8F45.0	fUTF8	轅
U+8F45.0	fCNS11643	1-7560
U+8F45.0	fBig5	C1D5
U+8F45.0	fGB	1-542F
U+8F45.0	fR/S	 159.10
U+8F45.0	fTotalStrokes	 17
U+8F45.0	fCangjie	JJGRV
U+8F45.0	fFourCorner	54032
U+8F45.0	fHakka	  jen2; jan3; ren2; jan2
U+8F45.0	fCantonese	jyun4
U+8F45.0	fMandarin	yuan2; yung4
U+8F45.0	fEnglish	[1] shafts (cart, carriage) [2] the magistrate's office; yamen [3] a Chinese family name
U+8F46.0	fUTF8	轆
U+8F46.0	fCNS11643	2-6370
U+8F46.0	fBig5	F146
U+8F46.0	fGB	1-6A24
U+8F46.0	fR/S	 159.11
U+8F46.0	fTotalStrokes	 18
U+8F46.0	fCangjie	JJIXP
U+8F46.0	fFourCorner	50011
U+8F46.0	fHakka	  luk8; luk7
U+8F46.0	fCantonese	luk1; luk6
U+8F46.0	fMandarin	lu4
U+8F46.0	fEnglish	[1] wheel [2] capstan
U+8F47.0	fUTF8	轇
U+8F47.0	fCNS11643	2-6371
U+8F47.0	fBig5	F147
U+8F47.0	fGB	3-4621
U+8F47.0	fR/S	 159.11
U+8F47.0	fTotalStrokes	 18
U+8F47.0	fCangjie	JJSMH
U+8F47.0	fFourCorner	57022
U+8F47.0	fHakka	 gau1
U+8F47.0	fCantonese	gaau1
U+8F47.0	fMandarin	jiu1; jiao1
U+8F47.0	fEnglish	[1] deep and vast [2] dispute [3] disorder
U+8F48.0	fUTF8	轈
U+8F48.0	fCNS11643	2-6372
U+8F48.0	fBig5	F148
U+8F48.0	fGB	5-4776
U+8F48.0	fR/S	 159.11
U+8F48.0	fTotalStrokes	 18
U+8F48.0	fCangjie	JJVVD
U+8F48.0	fFourCorner	52094
U+8F48.0	fHakka	  cau2;  sau1
U+8F48.0	fCantonese	caau4
U+8F48.0	fMandarin	chao2
U+8F48.0	fEnglish	turret on a chariot
U+8F49.0	fUTF8	轉
U+8F49.0	fCNS11643	1-774B
U+8F49.0	fBig5	C2E0
U+8F49.0	fGB	1-572A
U+8F49.0	fR/S	 159.11
U+8F49.0	fTotalStrokes	 18
U+8F49.0	fCangjie	JJJII
U+8F49.0	fFourCorner	55043
U+8F49.0	fHakka	  zhon3; zhon5; zon5; zhong3
U+8F49.0	fCantonese	zyun2; zyun3
U+8F49.0	fMandarin	zhuan3; zhuai3
U+8F49.0	fEnglish	[1] turn [2] take a turn [3] transport; convey; transfer; pass on [4] indirect; roundabout [5] roll [6] shift; move; migrate
U+8F49.1	fUTF8	轉
U+8F49.1	fCNS11643	1-774B
U+8F49.1	fBig5	C2E0
U+8F49.1	fGB	1-572A
U+8F49.1	fR/S	 159.11
U+8F49.1	fTotalStrokes	 18
U+8F49.1	fCangjie	JJJII
U+8F49.1	fFourCorner	55043
U+8F49.1	fHakka	 zon3
U+8F49.1	fMandarin	zhuan4
U+8F49.1	fEnglish	 turn round and round; rotate; revolve; gyrate
U+8F4A.0	fUTF8	轊
U+8F4A.0	fCNS11643	3-5937
U+8F4A.0	fGB	5-4775
U+8F4A.0	fR/S	 159.11
U+8F4A.0	fTotalStrokes	 18
U+8F4A.0	fCangjie	JJQJM
U+8F4A.0	fFourCorner	55070
U+8F4A.0	fMandarin	wei4
U+8F4A.0	fEnglish	axle-tip
U+8F4B.0	fUTF8	轋
U+8F4B.0	fCNS11643	2-6373
U+8F4B.0	fBig5	F149
U+8F4B.0	fGB	5-4769
U+8F4B.0	fR/S	 159.11
U+8F4B.0	fTotalStrokes	 18
U+8F4B.0	fCangjie	JJYJJ
U+8F4B.0	fFourCorner	55030
U+8F4B.0	fCantonese	wan4; hin1
U+8F4B.0	fMandarin	hun2; han3; hun1; xuan1
U+8F4C.0	fUTF8	轌
U+8F4C.0	fGB	E-3F7B
U+8F4C.0	fR/S	 159.11
U+8F4C.0	fTotalStrokes	 18
U+8F4C.0	fCangjie	JJMBS
U+8F4C.0	fCantonese	syut3
U+8F4C.0	fEnglish	 sled
U+8F4D.0	fUTF8	轍
U+8F4D.0	fCNS11643	1-774C
U+8F4D.0	fBig5	C2E1
U+8F4D.0	fGB	1-555E
U+8F4D.0	fR/S	 159.12
U+8F4D.0	fTotalStrokes	 19
U+8F4D.0	fCangjie	JJYBK
U+8F4D.0	fFourCorner	58040
U+8F4D.0	fHakka	 cet8; chat8; chat7; cat8; chet8
U+8F4D.0	fCantonese	cit3
U+8F4D.0	fMandarin	zhe2; che4
U+8F4D.0	fEnglish	 ruts; wheel tracks
U+8F4E.0	fUTF8	轎
U+8F4E.0	fCNS11643	1-792F
U+8F4E.0	fBig5	C3E2
U+8F4E.0	fGB	1-3D4E
U+8F4E.0	fR/S	 159.12
U+8F4E.0	fTotalStrokes	 19
U+8F4E.0	fCangjie	JJHKB
U+8F4E.0	fFourCorner	52027
U+8F4E.0	fHakka	  kiau5; kiau6; kiau3; k'ieu5
U+8F4E.0	fCantonese	giu2; giu6 (also); kiu2 (also); kiu4
U+8F4E.0	fMandarin	jiao4; chan1
U+8F4E.0	fEnglish	 chair; sedan-chair; palanquin; palankeen
U+8F4F.0	fUTF8	轏
U+8F4F.0	fCNS11643	2-6744
U+8F4F.0	fBig5	F358
U+8F4F.0	fGB	3-4628
U+8F4F.0	fR/S	 159.12
U+8F4F.0	fTotalStrokes	 19
U+8F4F.0	fCangjie	JJSND
U+8F4F.0	fFourCorner	57047
U+8F4F.0	fHakka	  can5;  zan3
U+8F4F.0	fCantonese	zaan6
U+8F4F.0	fMandarin	zhan4
U+8F4F.0	fEnglish	sleeping cart
U+8F50.0	fUTF8	轐
U+8F50.0	fCNS11643	2-6745
U+8F50.0	fBig5	F359
U+8F50.0	fGB	5-4778
U+8F50.0	fR/S	 159.12
U+8F50.0	fTotalStrokes	 19
U+8F50.0	fCangjie	JJTCO
U+8F50.0	fFourCorner	42434
U+8F50.0	fHakka	 puk7; puk8
U+8F50.0	fCantonese	buk6; pok3
U+8F50.0	fMandarin	pu2; bu2
U+8F50.0	fEnglish	the pieces of a cart which hold an axle firm
U+8F51.0	fUTF8	轑
U+8F51.0	fCNS11643	2-6743
U+8F51.0	fBig5	F357
U+8F51.0	fGB	3-4624
U+8F51.0	fR/S	 159.12
U+8F51.0	fTotalStrokes	 19
U+8F51.0	fCangjie	JJKCF
U+8F51.0	fFourCorner	54096
U+8F51.0	fCantonese	lou5; lou4; liu4
U+8F51.0	fMandarin	lao3; lao2; liao3; liao2
U+8F51.0	fEnglish	rut
U+8F52.0	fUTF8	轒
U+8F52.0	fCNS11643	2-6742
U+8F52.0	fBig5	F356
U+8F52.0	fGB	3-456D
U+8F52.0	fR/S	 159.12
U+8F52.0	fTotalStrokes	 19
U+8F52.0	fCangjie	JJJTC
U+8F52.0	fFourCorner	54086
U+8F52.0	fCantonese	fan4
U+8F52.0	fMandarin	fen2; fen4
U+8F52.0	fEnglish	chariot
U+8F53.0	fUTF8	轓
U+8F53.0	fCNS11643	2-6746
U+8F53.0	fBig5	F35A
U+8F53.0	fGB	3-4625
U+8F53.0	fR/S	 159.12
U+8F53.0	fTotalStrokes	 19
U+8F53.0	fCangjie	JJHDW
U+8F53.0	fFourCorner	52069
U+8F53.0	fHakka	 fan1
U+8F53.0	fCantonese	faan1
U+8F53.0	fMandarin	fan1; fan3
U+8F53.0	fEnglish	hood for a cart
U+8F54.0	fUTF8	轔
U+8F54.0	fCNS11643	1-792E
U+8F54.0	fBig5	C3E1
U+8F54.0	fGB	1-6A25
U+8F54.0	fR/S	 159.12; 159.13
U+8F54.0	fTotalStrokes	 19
U+8F54.0	fCangjie	JJFDQ
U+8F54.0	fFourCorner	59059
U+8F54.0	fHakka	 lin2;  din1
U+8F54.0	fCantonese	leon4; leon6
U+8F54.0	fMandarin	lin2; lin4; lin3
U+8F54.0	fEnglish	[1] rumbling of vehicles (n); noise of wheels (n) [2] threshold (n) [3] wheels (n) [4] wheels (n)
U+8F55.0	fUTF8	轕
U+8F55.0	fCNS11643	2-6A2B
U+8F55.0	fBig5	F4DD
U+8F55.0	fGB	3-4623
U+8F55.0	fR/S	 159.13; 159.12
U+8F55.0	fTotalStrokes	 20
U+8F55.0	fCangjie	JJTAV
U+8F55.0	fFourCorner	44427
U+8F55.0	fHakka	  got7
U+8F55.0	fCantonese	got3
U+8F55.0	fMandarin	ge2
U+8F55.0	fEnglish	[1] dispute; [2] deep and vast [3] disorder
U+8F56.0	fUTF8	轖
U+8F56.0	fCNS11643	2-6A29
U+8F56.0	fBig5	F4DB
U+8F56.0	fGB	3-457C
U+8F56.0	fR/S	 159.13
U+8F56.0	fTotalStrokes	 20
U+8F56.0	fCangjie	JJGOW
U+8F56.0	fFourCorner	54061
U+8F56.0	fCantonese	sik1
U+8F56.0	fMandarin	se4
U+8F56.0	fEnglish	framework of lattice on cart
U+8F57.0	fUTF8	轗
U+8F57.0	fCNS11643	2-6A2A
U+8F57.0	fBig5	F4DC
U+8F57.0	fGB	5-477A
U+8F57.0	fR/S	 159.13
U+8F57.0	fTotalStrokes	 20
U+8F57.0	fCangjie	JJIRP
U+8F57.0	fFourCorner	53035
U+8F57.0	fHakka	  kam3
U+8F57.0	fCantonese	ham2
U+8F57.0	fMandarin	kan3; kan4
U+8F57.0	fEnglish	 proceed with difficulty (vehicles)
U+8F58.0	fUTF8	轘
U+8F58.0	fCNS11643	2-6A2C
U+8F58.0	fBig5	F4DE
U+8F58.0	fGB	3-462B
U+8F58.0	fR/S	 159.13
U+8F58.0	fTotalStrokes	 20
U+8F58.0	fCangjie	JJWLV
U+8F58.0	fFourCorner	56032
U+8F58.0	fHakka	 fan2; fan5; fam5; fam2; van5
U+8F58.0	fCantonese	waan6 (also); waan4
U+8F58.0	fMandarin	huan4; huan2; se4
U+8F58.0	fEnglish	 tear (an offender) sunder between chariots (punishment)
U+8F59.0	fUTF8	轙
U+8F59.0	fCNS11643	2-6A28
U+8F59.0	fBig5	F4DA
U+8F59.0	fGB	3-4539
U+8F59.0	fR/S	 159.13
U+8F59.0	fTotalStrokes	 20
U+8F59.0	fCangjie	JJTGI
U+8F59.0	fFourCorner	58053
U+8F59.0	fCantonese	ngai5; ai5 (also); ji5
U+8F59.0	fMandarin	yi3; yi2
U+8F59.0	fEnglish	[1] rings on the yokes [2] wait
U+8F5A.0	fUTF8	轚
U+8F5A.0	fCNS11643	2-6A2D
U+8F5A.0	fBig5	F4DF
U+8F5A.0	fGB	E-3F7C
U+8F5A.0	fR/S	 159.13
U+8F5A.0	fTotalStrokes	 20
U+8F5A.0	fCangjie	JEJWJ
U+8F5A.0	fFourCorner	57504
U+8F5A.0	fCantonese	gik1
U+8F5A.0	fMandarin	ji2; ji4
U+8F5B.0	fUTF8	轛
U+8F5B.0	fCNS11643	2-6C47
U+8F5B.0	fBig5	F658
U+8F5B.0	fGB	3-454E
U+8F5B.0	fR/S	 159.14
U+8F5B.0	fTotalStrokes	 21
U+8F5B.0	fCangjie	JJTGI
U+8F5B.0	fFourCorner	54000
U+8F5B.0	fCantonese	deoi3
U+8F5B.0	fMandarin	dui4; zhui4
U+8F5C.0	fUTF8	轜
U+8F5C.0	fCNS11643	3-5E67
U+8F5C.0	fGB	5-477D
U+8F5C.0	fR/S	 159.14
U+8F5C.0	fTotalStrokes	 21
U+8F5C.0	fCangjie	JJMBB
U+8F5C.0	fFourCorner	51020
U+8F5C.0	fCantonese	ji4
U+8F5C.0	fMandarin	er2
U+8F5C.0	fEnglish	hearse
U+8F5D.0	fUTF8	轝
U+8F5D.0	fCNS11643	2-6C48
U+8F5D.0	fBig5	F659
U+8F5D.0	fGB	E-3F7D
U+8F5D.0	fR/S	 159.14
U+8F5D.0	fTotalStrokes	 21
U+8F5D.0	fCangjie	HCJWJ
U+8F5D.0	fFourCorner	77506
U+8F5D.0	fHakka	 i2
U+8F5D.0	fCantonese	jyu4
U+8F5D.0	fMandarin	yu2; yu4
U+8F5D.0	fEnglish	same as $
U+8F5E.0	fUTF8	轞
U+8F5E.0	fCNS11643	2-6C46
U+8F5E.0	fBig5	F657
U+8F5E.0	fGB	5-4772
U+8F5E.0	fR/S	 159.14
U+8F5E.0	fTotalStrokes	 21
U+8F5E.0	fCangjie	JJSIT
U+8F5E.0	fFourCorner	58017
U+8F5E.0	fHakka	 gam5
U+8F5E.0	fCantonese	laam6; haam5
U+8F5E.0	fMandarin	xian4
U+8F5E.0	fEnglish	[1] noise of wheels [2] vehicle for transporting prisoners; sealed carts for transporting criminals
U+8F5F.0	fUTF8	轟
U+8F5F.0	fCNS11643	1-7B32
U+8F5F.0	fBig5	C546
U+8F5F.0	fGB	1-3A64
U+8F5F.0	fR/S	 159.14
U+8F5F.0	fTotalStrokes	 21
U+8F5F.0	fCangjie	JJJJJ
U+8F5F.0	fFourCorner	50556
U+8F5F.0	fHakka	  wang2;  wang1;  kwen1; {fung1}; gang2; gang1; vang2; gwang2; vang1; kwang2
U+8F5F.0	fCantonese	gwang1
U+8F5F.0	fMandarin	hong1; hong4
U+8F5F.0	fEnglish	[1] noise of a number of vehicles [2] noise; uproar [3] grand; magnificent [4] bombard; blast
U+8F60.0	fUTF8	轠
U+8F60.0	fCNS11643	2-6E33
U+8F60.0	fBig5	F764
U+8F60.0	fGB	3-462D
U+8F60.0	fR/S	 159.15
U+8F60.0	fTotalStrokes	 22
U+8F60.0	fCangjie	JJWWW
U+8F60.0	fFourCorner	56060
U+8F60.0	fCantonese	leoi4
U+8F60.0	fMandarin	lei2; lei3
U+8F61.0	fUTF8	轡
U+8F61.0	fCNS11643	1-7B79
U+8F61.0	fBig5	C5AF
U+8F61.0	fGB	1-604E
U+8F61.0	fR/S	 159.15
U+8F61.0	fTotalStrokes	 22
U+8F61.0	fCangjie	VFR
U+8F61.0	fFourCorner	22609
U+8F61.0	fHakka	 bi5; pi5; bi3
U+8F61.0	fCantonese	bei3
U+8F61.0	fMandarin	pei4; bi4; jian1
U+8F61.0	fEnglish	 bridle of horse; reins
U+8F62.0	fUTF8	轢
U+8F62.0	fCNS11643	2-6E34
U+8F62.0	fBig5	F765
U+8F62.0	fGB	1-6976
U+8F62.0	fR/S	 159.15
U+8F62.0	fTotalStrokes	 22
U+8F62.0	fCangjie	JJVID
U+8F62.0	fFourCorner	52094
U+8F62.0	fHakka	  lit8
U+8F62.0	fCantonese	lik1; lik6
U+8F62.0	fMandarin	li4; la4; luo4
U+8F62.0	fEnglish	[1] run over (wheel) [2] oppress; bully
U+8F63.0	fUTF8	轣
U+8F63.0	fCNS11643	2-6F56
U+8F63.0	fBig5	F848
U+8F63.0	fGB	3-453D
U+8F63.0	fR/S	 159.16
U+8F63.0	fTotalStrokes	 23
U+8F63.0	fCangjie	JJMHM
U+8F63.0	fFourCorner	51011
U+8F63.0	fHakka	  lit8
U+8F63.0	fCantonese	lik6 (also); lit6; lik1
U+8F63.0	fMandarin	li4
U+8F63.0	fEnglish	[1] the rut of a wheel [2] crush under a wheel
U+8F64.0	fUTF8	轤
U+8F64.0	fCNS11643	2-6F55
U+8F64.0	fBig5	F847
U+8F64.0	fGB	1-6971
U+8F64.0	fR/S	 159.16
U+8F64.0	fTotalStrokes	 23
U+8F64.0	fCangjie	JJYPT
U+8F64.0	fFourCorner	51017
U+8F64.0	fHakka	  lu2
U+8F64.0	fCantonese	lou4
U+8F64.0	fMandarin	lu2
U+8F64.0	fEnglish	[1] pulley for drawing water [2] windlass; capstan [3] rails for vehicles
U+8F65.0	fUTF8	轥
U+8F65.0	fCNS11643	3-6222
U+8F65.0	fGB	3-462C
U+8F65.0	fR/S	  159.20
U+8F65.0	fTotalStrokes	 27
U+8F65.0	fCangjie	JJTAG
U+8F65.0	fFourCorner	54020
U+8F65.0	fCantonese	leon6
U+8F65.0	fMandarin	lin4
U+8F66.0	fUTF8	车
U+8F66.0	fGB	0-3335
U+8F66.0	fR/S	 159.0
U+8F66.0	fTotalStrokes	 4
U+8F66.0	fCangjie	KQ
U+8F66.0	fCantonese	ce1; geoi1
U+8F66.0	fMandarin	ju1; che1
U+8F66.0	fEnglish	simplified form of &#x8ECA;
U+8F67.0	fUTF8	轧
U+8F67.0	fGB	0-547E
U+8F67.0	fR/S	 159.1
U+8F67.0	fTotalStrokes	 5
U+8F67.0	fCangjie	KQU
U+8F67.0	fHakka	 jet7; zhap7
U+8F67.0	fCantonese	aat3; zaat3; ngaat3 (also); gaat3; aaat3
U+8F67.0	fMandarin	ya4; ga2; zha2
U+8F67.0	fEnglish	simplified form of &#x8ECB;
U+8F68.0	fUTF8	轨
U+8F68.0	fGB	0-396C
U+8F68.0	fR/S	 159.2
U+8F68.0	fTotalStrokes	 6
U+8F68.0	fCangjie	KQKN
U+8F68.0	fHakka	 kwui3
U+8F68.0	fCantonese	gwai2
U+8F68.0	fMandarin	gui3
U+8F68.0	fEnglish	simplified form of &#x8ECC;
U+8F69.0	fUTF8	轩
U+8F69.0	fGB	0-5079
U+8F69.0	fR/S	 159.3
U+8F69.0	fTotalStrokes	 7
U+8F69.0	fCangjie	KQMJ
U+8F69.0	fHakka	 hien1
U+8F69.0	fCantonese	hin1
U+8F69.0	fMandarin	xuan1; jian1; xian3; xian4
U+8F69.0	fEnglish	simplified form of &#x8ED2;
U+8F6A.0	fUTF8	轪
U+8F6A.0	fGB	7-2125
U+8F6A.0	fR/S	 159.3
U+8F6A.0	fTotalStrokes	 7
U+8F6A.0	fCangjie	KQK
U+8F6A.0	fCantonese	daai6 (also); dai6
U+8F6A.0	fMandarin	di4; dai4
U+8F6B.0	fUTF8	轫
U+8F6B.0	fGB	0-696D
U+8F6B.0	fR/S	 159.3
U+8F6B.0	fTotalStrokes	 7
U+8F6B.0	fCangjie	KQSHI
U+8F6B.0	fHakka	 ngiun3
U+8F6B.0	fCantonese	jan6
U+8F6B.0	fMandarin	ren4
U+8F6B.0	fEnglish	simplified form of &#x8ED4;
U+8F6C.0	fUTF8	转
U+8F6C.0	fGB	0-572A
U+8F6C.0	fR/S	 159.4
U+8F6C.0	fTotalStrokes	 8
U+8F6C.0	fCangjie	KQQNI
U+8F6C.0	fHakka	 zhon3
U+8F6C.0	fCantonese	zyun2; zyun3
U+8F6C.0	fMandarin	zhuan3; zhuai3
U+8F6C.0	fEnglish	simplified form of &#x8F49;
U+8F6C.1	fUTF8	转
U+8F6C.1	fGB	0-572A
U+8F6C.1	fR/S	 159.4
U+8F6C.1	fTotalStrokes	 8
U+8F6C.1	fCangjie	KQQNI
U+8F6C.1	fMandarin	zhuan4
U+8F6D.0	fUTF8	轭
U+8F6D.0	fGB	0-696E
U+8F6D.0	fR/S	 159.4
U+8F6D.0	fTotalStrokes	 8
U+8F6D.0	fCangjie	KQMSU
U+8F6D.0	fHakka	 ak7
U+8F6D.0	fCantonese	aak1; ngaak1 (also)
U+8F6D.0	fMandarin	e4
U+8F6D.0	fEnglish	simplified form of &#x8EDB;
U+8F6E.0	fUTF8	轮
U+8F6E.0	fGB	0-4256
U+8F6E.0	fR/S	 159.4
U+8F6E.0	fTotalStrokes	 8
U+8F6E.0	fCangjie	KQOP
U+8F6E.0	fHakka	 lun2; lin2
U+8F6E.0	fCantonese	leon4; leon2 (also)
U+8F6E.0	fMandarin	lun2; lun1
U+8F6E.0	fEnglish	simplified form of &#x8F2A;
U+8F6F.0	fUTF8	软
U+8F6F.0	fGB	0-486D
U+8F6F.0	fR/S	 159.4
U+8F6F.0	fTotalStrokes	 8
U+8F6F.0	fCangjie	KQNO
U+8F6F.0	fHakka	 ngion1; ngion3
U+8F6F.0	fCantonese	jyun5
U+8F6F.0	fMandarin	ruan3
U+8F6F.0	fEnglish	simplified form of &#x8EDF;
U+8F70.0	fUTF8	轰
U+8F70.0	fGB	0-3A64
U+8F70.0	fR/S	 159.4
U+8F70.0	fTotalStrokes	 8
U+8F70.0	fCangjie	KQEE
U+8F70.0	fHakka	 vang2; vang1; kwen1
U+8F70.0	fCantonese	gwang1
U+8F70.0	fMandarin	hong1; hong4
U+8F70.0	fEnglish	simplified form of &#x8F5F;
U+8F71.0	fUTF8	轱
U+8F71.0	fGB	0-696F
U+8F71.0	fR/S	 159.5
U+8F71.0	fTotalStrokes	 9
U+8F71.0	fCangjie	KQJR
U+8F71.0	fHakka	 ku1
U+8F71.0	fCantonese	gu1; gwu1 (also)
U+8F71.0	fMandarin	gu1; ku1
U+8F71.0	fEnglish	simplified form of &#x8EF2;
U+8F72.0	fUTF8	轲
U+8F72.0	fGB	0-6970
U+8F72.0	fR/S	 159.5
U+8F72.0	fTotalStrokes	 9
U+8F72.0	fCangjie	KQMNR
U+8F72.0	fHakka	 ko1
U+8F72.0	fCantonese	ngo1 (also); ho2; o1
U+8F72.0	fMandarin	ke1; ke3; ke4
U+8F72.0	fEnglish	simplified form of &#x8EFB;
U+8F73.0	fUTF8	轳
U+8F73.0	fGB	0-6971
U+8F73.0	fR/S	 159.5
U+8F73.0	fTotalStrokes	 9
U+8F73.0	fCangjie	KQYS
U+8F73.0	fHakka	 lu2
U+8F73.0	fCantonese	lou4
U+8F73.0	fMandarin	lu2
U+8F73.0	fEnglish	simplified form of &#x8F64;
U+8F74.0	fUTF8	轴
U+8F74.0	fGB	0-5661
U+8F74.0	fR/S	 159.5
U+8F74.0	fTotalStrokes	 9
U+8F74.0	fCangjie	KQLW
U+8F74.0	fHakka	 ch'uk8
U+8F74.0	fCantonese	zuk6
U+8F74.0	fMandarin	zhou2; zhu2; zhou4
U+8F74.0	fEnglish	simplified form of &#x8EF8;
U+8F75.0	fUTF8	轵
U+8F75.0	fGB	0-6972
U+8F75.0	fR/S	 159.5
U+8F75.0	fTotalStrokes	 9
U+8F75.0	fCangjie	KQRC
U+8F75.0	fHakka	 zhi3
U+8F75.0	fCantonese	zi2
U+8F75.0	fMandarin	zhi3
U+8F75.0	fEnglish	simplified form of &#x8EF9;
U+8F76.0	fUTF8	轶
U+8F76.0	fGB	0-6973
U+8F76.0	fR/S	 159.5
U+8F76.0	fTotalStrokes	 9
U+8F76.0	fCangjie	KQHQO
U+8F76.0	fHakka	 jit8; zhit7
U+8F76.0	fCantonese	jat6; dit6
U+8F76.0	fMandarin	yi4; die2; die1; zhe2
U+8F76.0	fEnglish	simplified form of &#x8EFC;
U+8F77.0	fUTF8	轷
U+8F77.0	fGB	0-6975
U+8F77.0	fR/S	 159.5
U+8F77.0	fTotalStrokes	 9
U+8F77.0	fCangjie	KQHFD
U+8F77.0	fCantonese	fu1
U+8F77.0	fMandarin	hu1
U+8F77.0	fEnglish	simplified form of &#x8EE4;
U+8F78.0	fUTF8	轸
U+8F78.0	fGB	0-6974
U+8F78.0	fR/S	 159.5
U+8F78.0	fTotalStrokes	 9
U+8F78.0	fCangjie	KQOHH
U+8F78.0	fHakka	 zhin3
U+8F78.0	fCantonese	zan2; can2 (also)
U+8F78.0	fMandarin	zhen3
U+8F78.0	fEnglish	simplified form of &#x8EEB;
U+8F79.0	fUTF8	轹
U+8F79.0	fGB	0-6976
U+8F79.0	fR/S	 159.5
U+8F79.0	fTotalStrokes	 9
U+8F79.0	fCangjie	KQHVD
U+8F79.0	fHakka	 lit8
U+8F79.0	fCantonese	lik1
U+8F79.0	fMandarin	li4; la4; luo4
U+8F79.0	fEnglish	simplified form of &#x8F62;
U+8F7A.0	fUTF8	轺
U+8F7A.0	fGB	0-6977
U+8F7A.0	fR/S	 159.5
U+8F7A.0	fTotalStrokes	 9
U+8F7A.0	fCangjie	KQSHR
U+8F7A.0	fHakka	 jau2; shau2
U+8F7A.0	fCantonese	jiu4
U+8F7A.0	fMandarin	yao2; diao1; shao2
U+8F7A.0	fEnglish	simplified form of &#x8EFA;
U+8F7B.0	fUTF8	轻
U+8F7B.0	fGB	0-4761
U+8F7B.0	fR/S	 159.5
U+8F7B.0	fTotalStrokes	 9
U+8F7B.0	fCangjie	KQNOM
U+8F7B.0	fHakka	 kiang1
U+8F7B.0	fCantonese	hing1; heng1 (also); heng6 (also)
U+8F7B.0	fMandarin	qing1; qing4
U+8F7B.0	fEnglish	simplified form of &#x8F15;
U+8F7C.0	fUTF8	轼
U+8F7C.0	fGB	0-6978
U+8F7C.0	fR/S	 159.6
U+8F7C.0	fTotalStrokes	 10
U+8F7C.0	fCangjie	KQIPM
U+8F7C.0	fHakka	 shit7
U+8F7C.0	fCantonese	sik1
U+8F7C.0	fMandarin	shi4
U+8F7C.0	fEnglish	simplified form of &#x8EFE;
U+8F7D.0	fUTF8	载
U+8F7D.0	fGB	0-5458
U+8F7D.0	fR/S	 62.6; 159.6
U+8F7D.0	fTotalStrokes	 10
U+8F7D.0	fCangjie	JIKQ
U+8F7D.0	fHakka	 zai5; zoi5
U+8F7D.0	fCantonese	zoi3; zoi2
U+8F7D.0	fMandarin	zai4; dai4
U+8F7D.0	fEnglish	simplified form of &#x8F09;
U+8F7D.1	fUTF8	载
U+8F7D.1	fGB	0-5458
U+8F7D.1	fR/S	 62.6; 159.6
U+8F7D.1	fTotalStrokes	 10
U+8F7D.1	fCangjie	JIKQ
U+8F7D.1	fMandarin	zai3
U+8F7E.0	fUTF8	轾
U+8F7E.0	fGB	0-6979
U+8F7E.0	fR/S	 159.6
U+8F7E.0	fTotalStrokes	 10
U+8F7E.0	fCangjie	KQMIG
U+8F7E.0	fHakka	 zhi5
U+8F7E.0	fCantonese	zi3
U+8F7E.0	fMandarin	zhi4; shi4
U+8F7E.0	fEnglish	simplified form of &#x8F0A;
U+8F7F.0	fUTF8	轿
U+8F7F.0	fGB	0-3D4E
U+8F7F.0	fR/S	 159.6
U+8F7F.0	fTotalStrokes	 10
U+8F7F.0	fCangjie	KQHKL
U+8F7F.0	fHakka	 kiau5
U+8F7F.0	fCantonese	giu6 (also); giu2; kiu2 (also)
U+8F7F.0	fMandarin	jiao4; chan1
U+8F7F.0	fEnglish	simplified form of &#x8F4E;
U+8F80.0	fUTF8	辀
U+8F80.0	fGB	8-2C53
U+8F80.0	fR/S	 159.6
U+8F80.0	fTotalStrokes	 10
U+8F80.0	fCangjie	KQHBY
U+8F80.0	fCantonese	zau1
U+8F80.0	fMandarin	zhou1
U+8F80.0	fEnglish	shaft
U+8F81.0	fUTF8	辁
U+8F81.0	fGB	0-697A
U+8F81.0	fR/S	 159.6
U+8F81.0	fTotalStrokes	 10
U+8F81.0	fCangjie	KQOMG
U+8F81.0	fHakka	 cien2; zien1
U+8F81.0	fCantonese	cyun4
U+8F81.0	fMandarin	quan2; chuan2; chun1
U+8F81.0	fEnglish	simplified form of &#x8F07;
U+8F82.0	fUTF8	辂
U+8F82.0	fGB	0-697B
U+8F82.0	fR/S	 159.6
U+8F82.0	fTotalStrokes	 10
U+8F82.0	fCangjie	KQHER
U+8F82.0	fHakka	 lu5
U+8F82.0	fCantonese	lou6
U+8F82.0	fMandarin	lu4; ya4; he2; luo4
U+8F82.0	fEnglish	simplified form of &#x8F05;
U+8F83.0	fUTF8	较
U+8F83.0	fGB	0-3D4F
U+8F83.0	fR/S	 159.6
U+8F83.0	fTotalStrokes	 10
U+8F83.0	fCangjie	KQYCK
U+8F83.0	fHakka	 gau3; gau5; gok7
U+8F83.0	fCantonese	gaau3; gok3
U+8F83.0	fMandarin	jiao4; jue2; jiao3
U+8F83.0	fEnglish	simplified form of &#x8F03;
U+8F84.0	fUTF8	辄
U+8F84.0	fGB	0-697C
U+8F84.0	fR/S	 159.7
U+8F84.0	fTotalStrokes	 11
U+8F84.0	fCangjie	KQSJU
U+8F84.0	fHakka	 zhap7
U+8F84.0	fCantonese	zip3; zit3 (also)
U+8F84.0	fMandarin	zhe2
U+8F84.0	fEnglish	simplified form of &#x8F12;
U+8F85.0	fUTF8	辅
U+8F85.0	fGB	0-3828
U+8F85.0	fR/S	 159.7
U+8F85.0	fTotalStrokes	 11
U+8F85.0	fCangjie	KQIJB
U+8F85.0	fHakka	 pu3
U+8F85.0	fCantonese	fu6
U+8F85.0	fMandarin	fu3
U+8F85.0	fEnglish	simplified form of &#x8F14;
U+8F86.0	fUTF8	辆
U+8F86.0	fGB	0-413E
U+8F86.0	fR/S	 159.7
U+8F86.0	fTotalStrokes	 11
U+8F86.0	fCangjie	KQMOB
U+8F86.0	fHakka	 liong3; liong5
U+8F86.0	fCantonese	loeng2 (also); loeng6
U+8F86.0	fMandarin	liang4; liang3
U+8F86.0	fEnglish	simplified form of &#x8F1B;
U+8F87.0	fUTF8	辇
U+8F87.0	fGB	0-697D
U+8F87.0	fR/S	 159.8
U+8F87.0	fTotalStrokes	 12
U+8F87.0	fCangjie	QOKQ
U+8F87.0	fHakka	 lien3
U+8F87.0	fCantonese	lin5
U+8F87.0	fMandarin	nian3; lian3
U+8F87.0	fEnglish	simplified form of &#x8F26;
U+8F88.0	fUTF8	辈
U+8F88.0	fGB	0-3132
U+8F88.0	fR/S	 175.4; 159.8
U+8F88.0	fTotalStrokes	 12
U+8F88.0	fCangjie	LYKQ
U+8F88.0	fHakka	 bui5
U+8F88.0	fCantonese	bui3
U+8F88.0	fMandarin	bei4
U+8F88.0	fEnglish	simplified form of &#x8F29;
U+8F89.0	fUTF8	辉
U+8F89.0	fGB	0-3B54
U+8F89.0	fR/S	 42.9; 159.8
U+8F89.0	fTotalStrokes	 15
U+8F89.0	fCangjie	FUBKQ
U+8F89.0	fHakka	 fui1
U+8F89.0	fCantonese	fai1
U+8F89.0	fMandarin	hui1
U+8F89.0	fEnglish	simplified form of &#x8F1D;
U+8F8A.0	fUTF8	辊
U+8F8A.0	fGB	0-3975
U+8F8A.0	fR/S	 159.8
U+8F8A.0	fTotalStrokes	 12
U+8F8A.0	fCangjie	KQAPP
U+8F8A.0	fHakka	 gwun3
U+8F8A.0	fCantonese	gwan2
U+8F8A.0	fMandarin	gun3; hun4
U+8F8A.0	fEnglish	simplified form of &#x8F25;
U+8F8B.0	fUTF8	辋
U+8F8B.0	fGB	0-697E
U+8F8B.0	fR/S	 159.8
U+8F8B.0	fTotalStrokes	 12
U+8F8B.0	fCangjie	KQBTV
U+8F8B.0	fHakka	 vong3
U+8F8B.0	fCantonese	mong5
U+8F8B.0	fMandarin	wang3
U+8F8B.0	fEnglish	simplified form of &#x8F1E;
U+8F8C.0	fUTF8	辌
U+8F8C.0	fGB	7-2126
U+8F8C.0	fR/S	 159.8
U+8F8C.0	fTotalStrokes	 12
U+8F8C.0	fCangjie	KQYRF
U+8F8C.0	fCantonese	loeng4
U+8F8C.0	fMandarin	liang2
U+8F8C.0	fEnglish	a hearse; a carriage
U+8F8D.0	fUTF8	辍
U+8F8D.0	fGB	0-6A21
U+8F8D.0	fR/S	 159.8
U+8F8D.0	fTotalStrokes	 12
U+8F8D.0	fCangjie	KQEEE
U+8F8D.0	fHakka	 dot7
U+8F8D.0	fCantonese	zyut3; zyut6 (also)
U+8F8D.0	fMandarin	chuo4; zhui4; zhuo2
U+8F8D.0	fEnglish	simplified form of &#x8F1F;
U+8F8E.0	fUTF8	辎
U+8F8E.0	fGB	0-6A22
U+8F8E.0	fR/S	 159.8
U+8F8E.0	fTotalStrokes	 12
U+8F8E.0	fCangjie	KQVVW
U+8F8E.0	fHakka	 zih1
U+8F8E.0	fCantonese	zi1
U+8F8E.0	fMandarin	zi1; zi4
U+8F8E.0	fEnglish	simplified form of &#x8F1C;
U+8F8F.0	fUTF8	辏
U+8F8F.0	fGB	0-6A23
U+8F8F.0	fR/S	 159.9
U+8F8F.0	fTotalStrokes	 13
U+8F8F.0	fCangjie	KQQKK
U+8F8F.0	fHakka	 ceu5
U+8F8F.0	fCantonese	cau3
U+8F8F.0	fMandarin	cou4
U+8F8F.0	fEnglish	simplified form of &#x8F33;
U+8F90.0	fUTF8	辐
U+8F90.0	fGB	0-3778
U+8F90.0	fR/S	 159.9
U+8F90.0	fTotalStrokes	 13
U+8F90.0	fCangjie	KQMRW
U+8F90.0	fHakka	 fuk7
U+8F90.0	fCantonese	fuk1
U+8F90.0	fMandarin	fu2
U+8F90.0	fEnglish	simplified form of &#x8F3B;
U+8F91.0	fUTF8	辑
U+8F91.0	fGB	0-3C2D
U+8F91.0	fR/S	 159.9
U+8F91.0	fTotalStrokes	 13
U+8F91.0	fCangjie	KQRSJ
U+8F91.0	fHakka	 sip8; cip7
U+8F91.0	fCantonese	cap1
U+8F91.0	fMandarin	ji2
U+8F91.0	fEnglish	simplified form of &#x8F2F;
U+8F91.1	fUTF8	辑
U+8F91.1	fGB	0-3C2D
U+8F91.1	fR/S	 159.9
U+8F91.1	fTotalStrokes	 13
U+8F91.1	fCangjie	KQRSJ
U+8F91.1	fMandarin	ji5
U+8F92.0	fUTF8	辒
U+8F92.0	fGB	8-2C54
U+8F92.0	fR/S	 159.9
U+8F92.0	fTotalStrokes	 13
U+8F92.0	fCangjie	KQABT
U+8F92.0	fCantonese	wan1
U+8F92.0	fMandarin	wen1
U+8F92.0	fEnglish	hearse
U+8F93.0	fUTF8	输
U+8F93.0	fGB	0-4A64
U+8F93.0	fR/S	 159.9
U+8F93.0	fTotalStrokes	 13
U+8F93.0	fCangjie	KQOMN
U+8F93.0	fHakka	 shu1
U+8F93.0	fCantonese	syu1
U+8F93.0	fMandarin	shu1; shu4
U+8F93.0	fEnglish	simplified form of &#x8F38;
U+8F94.0	fUTF8	辔
U+8F94.0	fGB	0-604E
U+8F94.0	fR/S	 30.10; 159.9
U+8F94.0	fTotalStrokes	 13
U+8F94.0	fCangjie	XVMR
U+8F94.0	fHakka	 bi5; bi3
U+8F94.0	fCantonese	bei3
U+8F94.0	fMandarin	pei4; bi4; jian1
U+8F94.0	fEnglish	simplified form of &#x8F61;
U+8F95.0	fUTF8	辕
U+8F95.0	fGB	0-542F
U+8F95.0	fR/S	 159.10
U+8F95.0	fTotalStrokes	 14
U+8F95.0	fCangjie	KQGRV
U+8F95.0	fHakka	 jen2
U+8F95.0	fCantonese	jyun4
U+8F95.0	fMandarin	yuan2; yung4
U+8F95.0	fEnglish	simplified form of &#x8F45;
U+8F96.0	fUTF8	辖
U+8F96.0	fGB	0-4F3D
U+8F96.0	fR/S	 159.10
U+8F96.0	fTotalStrokes	 14
U+8F96.0	fCangjie	KQJQR
U+8F96.0	fHakka	 hat7; hot7
U+8F96.0	fCantonese	hat6
U+8F96.0	fMandarin	xia2; he2; kuai4; yu4
U+8F96.0	fEnglish	simplified form of &#x8F44;
U+8F97.0	fUTF8	辗
U+8F97.0	fGB	0-5537
U+8F97.0	fR/S	 159.10
U+8F97.0	fTotalStrokes	 14
U+8F97.0	fCangjie	KQSTV
U+8F97.0	fHakka	 zhen3
U+8F97.0	fCantonese	zin2; nin5; lin5 (also)
U+8F97.0	fMandarin	zhan3; nian4; nian3
U+8F97.0	fEnglish	simplified form of &#x8F3E;
U+8F98.0	fUTF8	辘
U+8F98.0	fGB	0-6A24
U+8F98.0	fR/S	 159.11
U+8F98.0	fTotalStrokes	 15
U+8F98.0	fCangjie	KQIXP
U+8F98.0	fHakka	 luk8
U+8F98.0	fCantonese	luk1
U+8F98.0	fMandarin	lu4
U+8F98.0	fEnglish	simplified form of &#x8F46;
U+8F99.0	fUTF8	辙
U+8F99.0	fGB	0-555E
U+8F99.0	fR/S	 159.12
U+8F99.0	fTotalStrokes	 16
U+8F99.0	fCangjie	KQYBK
U+8F99.0	fHakka	 chet8
U+8F99.0	fCantonese	cit3
U+8F99.0	fMandarin	zhe2; che4
U+8F99.0	fEnglish	simplified form of &#x8F4D;
U+8F9A.0	fUTF8	辚
U+8F9A.0	fGB	0-6A25
U+8F9A.0	fR/S	 159.12
U+8F9A.0	fTotalStrokes	 16
U+8F9A.0	fCangjie	KQFDQ
U+8F9A.0	fHakka	 lin2
U+8F9A.0	fCantonese	leon4; leon6
U+8F9A.0	fMandarin	lin2; lin4; lin3
U+8F9A.0	fEnglish	simplified form of &#x8F54;
U+8F9B.0	fUTF8	辛
U+8F9B.0	fCNS11643	1-4B50
U+8F9B.0	fBig5	A8AF
U+8F9B.0	fGB	0-5041
U+8F9B.0	fR/S	 160.0
U+8F9B.0	fTotalStrokes	 7
U+8F9B.0	fCangjie	YTJ
U+8F9B.0	fFourCorner	00401
U+8F9B.0	fHakka	  sin1
U+8F9B.0	fCantonese	san1
U+8F9B.0	fMandarin	xin1
U+8F9B.0	fEnglish	[1] the eight of the ten Celestial stems [2] bitter; acrid [3] bitter; toilsome; laborious; hard [4] a Chinese family name [5] Kangxi radical 160
U+8F9C.0	fUTF8	辜
U+8F9C.0	fCNS11643	1-624B
U+8F9C.0	fBig5	B664
U+8F9C.0	fGB	0-393C
U+8F9C.0	fR/S	 160.5
U+8F9C.0	fTotalStrokes	 12
U+8F9C.0	fCangjie	JRYTJ
U+8F9C.0	fFourCorner	40401
U+8F9C.0	fHakka	  ku1; gu1
U+8F9C.0	fCantonese	gu1; gwu1 (also)
U+8F9C.0	fMandarin	gu1
U+8F9C.0	fEnglish	[1] sin; crime; criminal offense; guilt [2] be negligent in an obligation or expectation; fail [3] a Chinese familly name
U+8F9D.0	fUTF8	辝
U+8F9D.0	fCNS11643	3-3F4B
U+8F9D.0	fGB	E-3F7E
U+8F9D.0	fR/S	 160.5
U+8F9D.0	fTotalStrokes	 12
U+8F9D.0	fCangjie	IRYTJ
U+8F9D.0	fFourCorner	20640
U+8F9D.0	fCantonese	ci4
U+8F9D.0	fMandarin	ci2
U+8F9E.0	fUTF8	辞
U+8F9E.0	fCNS11643	3-4477
U+8F9E.0	fGB	0-3447
U+8F9E.0	fR/S	 135.7; 160.6
U+8F9E.0	fTotalStrokes	 13
U+8F9E.0	fCangjie	HRYTJ
U+8F9E.0	fFourCorner	20640
U+8F9E.0	fHakka	 cih2; ze2
U+8F9E.0	fCantonese	ci4
U+8F9E.0	fMandarin	ci2
U+8F9E.0	fEnglish	simplified form of &#x8FAD;
U+8F9F.0	fUTF8	辟
U+8F9F.0	fCNS11643	1-6728
U+8F9F.0	fBig5	B940
U+8F9F.0	fGB	0-3159
U+8F9F.0	fR/S	 160.6
U+8F9F.0	fTotalStrokes	 13
U+8F9F.0	fCangjie	SRYTJ
U+8F9F.0	fFourCorner	70641
U+8F9F.0	fHakka	  pit7
U+8F9F.0	fCantonese	pik1; bik1 (also); bei6; mai5
U+8F9F.0	fMandarin	bi4; pi1; bai3; bei1; mi3; bo2; pi2
U+8F9F.0	fEnglish	[1] monarch [2] summon; rule [3] govern; take the law (to people); rule [4] avoid; escape
U+8F9F.1	fUTF8	辟
U+8F9F.1	fCNS11643	1-6728
U+8F9F.1	fBig5	B940
U+8F9F.1	fGB	0-3159
U+8F9F.1	fR/S	 160.6
U+8F9F.1	fTotalStrokes	 13
U+8F9F.1	fCangjie	SRYTJ
U+8F9F.1	fFourCorner	70641
U+8F9F.1	fMandarin	pi4
U+8F9F.1	fEnglish	same as $
U+8FA0.0	fUTF8	辠
U+8FA0.0	fCNS11643	3-4478
U+8FA0.0	fGB	E-4021
U+8FA0.0	fR/S	  160.6
U+8FA0.0	fTotalStrokes	 13
U+8FA0.0	fCangjie	HUYTJ
U+8FA0.0	fFourCorner	26400
U+8FA0.0	fHakka	 cui5; cui6
U+8FA0.0	fCantonese	zeoi6
U+8FA0.0	fMandarin	zui4
U+8FA0.0	fEnglish	same as $
U+8FA1.0	fUTF8	辡
U+8FA1.0	fCNS11643	3-4A21
U+8FA1.0	fGB	E-4022
U+8FA1.0	fR/S	  160.7
U+8FA1.0	fTotalStrokes	 14
U+8FA1.0	fCangjie	YJYTJ
U+8FA1.0	fFourCorner	00440
U+8FA1.0	fHakka	 pien3; pien5
U+8FA1.0	fCantonese	bin6
U+8FA1.0	fMandarin	bian4
U+8FA1.0	fEnglish	 recrimination wrangling
U+8FA2.0	fUTF8	辢
U+8FA2.0	fCNS11643	3-4A22
U+8FA2.0	fGB	E-4023
U+8FA2.0	fR/S	  160.7
U+8FA2.0	fTotalStrokes	 14
U+8FA2.0	fCangjie	DLYTJ
U+8FA2.0	fFourCorner	50940
U+8FA2.0	fHakka	 lat8
U+8FA2.0	fCantonese	laat6; lat6
U+8FA2.0	fMandarin	la4
U+8FA2.0	fEnglish	bitter; pungent;; acrid
U+8FA3.0	fUTF8	辣
U+8FA3.0	fCNS11643	1-6B3E
U+8FA3.0	fBig5	BBB6
U+8FA3.0	fGB	0-4031
U+8FA3.0	fR/S	 160.7
U+8FA3.0	fTotalStrokes	 14
U+8FA3.0	fCangjie	YJDL
U+8FA3.0	fFourCorner	05496
U+8FA3.0	fHakka	  lat8; lat7
U+8FA3.0	fCantonese	laat6; laat2 (also); lat6
U+8FA3.0	fMandarin	la4
U+8FA3.0	fEnglish	[1] peppery; pungent; hot; piquant [2] burn; bite (smell, taste) [3] be
U+8FA4.0	fUTF8	辤
U+8FA4.0	fCNS11643	3-4F37
U+8FA4.0	fGB	E-4024
U+8FA4.0	fR/S	  160.8
U+8FA4.0	fTotalStrokes	 15
U+8FA4.0	fCangjie	BEYTJ
U+8FA4.0	fFourCorner	20440
U+8FA4.0	fHakka	 cih2; ze2
U+8FA4.0	fCantonese	ci4
U+8FA4.0	fMandarin	ci2
U+8FA4.0	fEnglish	 words; speech sentence; expression; phrase message
U+8FA5.0	fUTF8	辥
U+8FA5.0	fCNS11643	3-5327
U+8FA5.0	fGB	5-7331
U+8FA5.0	fR/S	 160.9
U+8FA5.0	fTotalStrokes	 16
U+8FA5.0	fCangjie	URYTJ
U+8FA5.0	fFourCorner	20740
U+8FA5.0	fHakka	 siet7
U+8FA5.0	fCantonese	sit3
U+8FA5.0	fMandarin	xue1
U+8FA5.0	fEnglish	variety of marsh grass
U+8FA6.0	fUTF8	辦
U+8FA6.0	fCNS11643	1-7256
U+8FA6.0	fBig5	BFEC
U+8FA6.0	fGB	1-306C
U+8FA6.0	fR/S	 160.9
U+8FA6.0	fTotalStrokes	 16
U+8FA6.0	fCangjie	YJKSJ
U+8FA6.0	fFourCorner	00441
U+8FA6.0	fHakka	 pan5; pan3; pan6; pen3
U+8FA6.0	fCantonese	baan6; baan2 (also)
U+8FA6.0	fMandarin	ban4
U+8FA6.0	fEnglish	[1] manage; handle; deal with; attend to; transact [2] try and punish [3] purchase
U+8FA7.0	fUTF8	辧
U+8FA7.0	fCNS11643	3-5326
U+8FA7.0	fGB	E-4025
U+8FA7.0	fR/S	 160.9; 18.14
U+8FA7.0	fTotalStrokes	 16
U+8FA7.0	fCangjie	YJSHJ
U+8FA7.0	fFourCorner	00440
U+8FA7.0	fHakka	 pien5
U+8FA7.0	fCantonese	bin6
U+8FA7.0	fMandarin	bian4; pan2
U+8FA7.0	fEnglish	 manage; do; handle deal with
U+8FA8.0	fUTF8	辨
U+8FA8.0	fCNS11643	1-7255
U+8FA8.0	fBig5	BFEB
U+8FA8.0	fGB	0-3166
U+8FA8.0	fR/S	 160.9
U+8FA8.0	fTotalStrokes	 16
U+8FA8.0	fCangjie	YJILJ
U+8FA8.0	fFourCorner	00441
U+8FA8.0	fHakka	 pien5; pan5; pien6; pan3; pen3; p'ian6
U+8FA8.0	fCantonese	bin6; bin3
U+8FA8.0	fMandarin	bian4; ban4; bian3; pian4; ban1
U+8FA8.0	fEnglish	[1] distinguish; discriminate; discern [2] identify; recognize
U+8FA9.0	fUTF8	辩
U+8FA9.0	fGB	0-3167
U+8FA9.0	fR/S	 160.9
U+8FA9.0	fTotalStrokes	 16
U+8FA9.0	fCangjie	YJIVJ
U+8FA9.0	fHakka	 pien5
U+8FA9.0	fCantonese	bin6
U+8FA9.0	fMandarin	bian4; pian2; bie2; pian4
U+8FA9.0	fEnglish	simplified form of &#x8FAF;
U+8FAA.0	fUTF8	辪
U+8FAA.0	fCNS11643	3-6654
U+8FAA.0	fGB	E-4026
U+8FAA.0	fR/S	 160.9
U+8FAA.0	fTotalStrokes	 16
U+8FAA.0	fCangjie	YRYTJ
U+8FAA.0	fFourCorner	2074
U+8FAA.0	fCantonese	sit3
U+8FAB.0	fUTF8	辫
U+8FAB.0	fGB	0-3168
U+8FAB.0	fR/S	 160.10
U+8FAB.0	fTotalStrokes	 17
U+8FAB.0	fCangjie	YJVMJ
U+8FAB.0	fHakka	 pien1; bin1; bien1
U+8FAB.0	fCantonese	bin1
U+8FAB.0	fMandarin	bian4
U+8FAB.0	fEnglish	simplified form of &#x8FAE;
U+8FAC.0	fUTF8	辬
U+8FAC.0	fCNS11643	3-5938
U+8FAC.0	fGB	5-7334
U+8FAC.0	fR/S	 160.11
U+8FAC.0	fTotalStrokes	 18
U+8FAC.0	fCangjie	YJYKJ
U+8FAC.0	fFourCorner	00440
U+8FAC.0	fCantonese	baan1
U+8FAC.0	fMandarin	ban1
U+8FAD.0	fUTF8	辭
U+8FAD.0	fCNS11643	1-7930
U+8FAD.0	fBig5	C3E3
U+8FAD.0	fGB	1-3447
U+8FAD.0	fR/S	 160.12
U+8FAD.0	fTotalStrokes	 19
U+8FAD.0	fCangjie	BBYTJ
U+8FAD.0	fFourCorner	20241
U+8FAD.0	fHakka	 cu2;  cih2; cih3; ze2
U+8FAD.0	fCantonese	ci4
U+8FAD.0	fMandarin	ci2
U+8FAD.0	fEnglish	[1] language; words; speech; expression; phrase [2] decline; refuse [3] leave; part from; depart; bid adieu [4] resign; dismiss
U+8FAE.0	fUTF8	辮
U+8FAE.0	fCNS11643	1-7A29
U+8FAE.0	fBig5	C47C
U+8FAE.0	fGB	1-3168
U+8FAE.0	fR/S	 120.14; 160.13
U+8FAE.0	fTotalStrokes	 20
U+8FAE.0	fCangjie	YJVFJ
U+8FAE.0	fFourCorner	00441
U+8FAE.0	fHakka	  bien1;  bin1;  pien1; ben1; pan5
U+8FAE.0	fCantonese	bin1
U+8FAE.0	fMandarin	bian4
U+8FAE.0	fEnglish	[1] plait; braid [2] pigtail; brait; queue
U+8FAF.0	fUTF8	辯
U+8FAF.0	fCNS11643	1-7B33
U+8FAF.0	fBig5	C547
U+8FAF.0	fGB	1-3167
U+8FAF.0	fR/S	 160.14
U+8FAF.0	fTotalStrokes	 21
U+8FAF.0	fCangjie	YJYRJ
U+8FAF.0	fFourCorner	00441
U+8FAF.0	fHakka	 pien5; pen3; pan5; pan3; pien6
U+8FAF.0	fCantonese	bin6
U+8FAF.0	fMandarin	bian4; pian2; bie2; bian3; pian4; ban4
U+8FAF.0	fEnglish	[1] dispute; argue; debate; discuss [2] use specious arguments
U+8FB0.0	fUTF8	辰
U+8FB0.0	fCNS11643	1-4B51
U+8FB0.0	fBig5	A8B0
U+8FB0.0	fGB	0-333D
U+8FB0.0	fR/S	 161.0
U+8FB0.0	fTotalStrokes	 7
U+8FB0.0	fCangjie	MMMV
U+8FB0.0	fFourCorner	71232
U+8FB0.0	fHakka	 sin2;  shin2; siin2; shin3
U+8FB0.0	fCantonese	san4
U+8FB0.0	fMandarin	chen2
U+8FB0.0	fEnglish	[1] the fifth of the twelve Terrestrial Branches [2] early morning; 7:00-9:00 in the morning [3] a time [4] fortune; luck [5] a heavenly body; sun, moon, and stars [6] Kangxi radical 161
U+8FB1.0	fUTF8	辱
U+8FB1.0	fCNS11643	1-5849
U+8FB1.0	fBig5	B064
U+8FB1.0	fGB	0-4868
U+8FB1.0	fR/S	 161.3
U+8FB1.0	fTotalStrokes	 10
U+8FB1.0	fCangjie	MVDI
U+8FB1.0	fFourCorner	71343
U+8FB1.0	fHakka	  juk8; ruk8
U+8FB1.0	fCantonese	juk6
U+8FB1.0	fMandarin	ru3; ru4
U+8FB1.0	fEnglish	[1] disgrace [2] disgrace; humiliate; insult; abuse; dishonour [3] condescend; deign [4] undeservingly
U+8FB2.0	fUTF8	農
U+8FB2.0	fCNS11643	1-6729
U+8FB2.0	fBig5	B941
U+8FB2.0	fGB	1-4529
U+8FB2.0	fR/S	 161.6
U+8FB2.0	fTotalStrokes	 13
U+8FB2.0	fCangjie	TWMMV
U+8FB2.0	fFourCorner	55232
U+8FB2.0	fHakka	  nung2;  lung2; nung3
U+8FB2.0	fCantonese	nung4; lung4 (also)
U+8FB2.0	fMandarin	nong2
U+8FB2.0	fEnglish	[1] agriculture; farming [2] farm [3] farmer; peasant; husbandman
U+8FB3.0	fUTF8	辳
U+8FB3.0	fCNS11643	3-4F39
U+8FB3.0	fGB	E-4027
U+8FB3.0	fR/S	 161.8; 161.11; 75.11
U+8FB3.0	fTotalStrokes	 15
U+8FB3.0	fCangjie	DDMMV
U+8FB3.0	fFourCorner	44230
U+8FB3.0	fCantonese	nung4; lung4 (also)
U+8FB3.0	fMandarin	nong2
U+8FB4.0	fUTF8	辴
U+8FB4.0	fCNS11643	2-6747
U+8FB4.0	fBig5	F35B
U+8FB4.0	fGB	5-6E3C
U+8FB4.0	fR/S	 161.12
U+8FB4.0	fTotalStrokes	 19
U+8FB4.0	fCangjie	RJMMV
U+8FB4.0	fFourCorner	61532
U+8FB4.0	fHakka	 zhan3
U+8FB4.0	fCantonese	can2; zin2 (also)
U+8FB4.0	fMandarin	zhen3; chen3; chi1
U+8FB5.0	fUTF8	辵
U+8FB5.0	fCNS11643	3-272F
U+8FB5.0	fGB	3-7154
U+8FB5.0	fR/S	 162.0
U+8FB5.0	fTotalStrokes	 7
U+8FB5.0	fCangjie	HHHO
U+8FB5.0	fFourCorner	20800
U+8FB5.0	fHakka	 cok7; ch'ok7; chok7
U+8FB5.0	fCantonese	coek3
U+8FB5.0	fMandarin	chuo4
U+8FB5.0	fEnglish	 walk walking Kangxi radical number 162
U+8FB6.0	fUTF8	辶
U+8FB6.0	fCNS11643	4-2154
U+8FB6.0	fGB	0-6541
U+8FB6.0	fR/S	 162.0
U+8FB6.0	fTotalStrokes	 4
U+8FB6.0	fCangjie	INNO
U+8FB6.0	fFourCorner	30300
U+8FB6.0	fHakka	 cok7; ch'ok7; chok7
U+8FB6.0	fMandarin	chuo4
U+8FB6.0	fEnglish	 walk walking Kangxi radical number 162
U+8FB7.0	fUTF8	辷
U+8FB7.0	fCNS11643	F-216A
U+8FB7.0	fGB	E-4028
U+8FB7.0	fR/S	 162.1
U+8FB7.0	fTotalStrokes	 5
U+8FB7.0	fCangjie	YM
U+8FB7.0	fFourCorner	3130
U+8FB7.0	fCantonese	jat1
U+8FB7.0	fEnglish	smooth; slippery
U+8FB8.0	fUTF8	辸
U+8FB8.0	fCNS11643	4-2330
U+8FB8.0	fGB	5-6F7B
U+8FB8.0	fR/S	  162.2
U+8FB8.0	fTotalStrokes	 6
U+8FB8.0	fCangjie	YNHS
U+8FB8.0	fFourCorner	37300
U+8FB8.0	fCantonese	jing4
U+8FB8.0	fMandarin	reng2
U+8FB9.0	fUTF8	边
U+8FB9.0	fCNS11643	3-243B
U+8FB9.0	fGB	0-315F
U+8FB9.0	fR/S	 162.2
U+8FB9.0	fTotalStrokes	 6
U+8FB9.0	fCangjie	YKS
U+8FB9.0	fFourCorner	34300
U+8FB9.0	fHakka	 bien1
U+8FB9.0	fCantonese	bin1; bin6 (also)
U+8FB9.0	fMandarin	bian1; niao3
U+8FB9.0	fEnglish	simplified form of &#x908A;
U+8FB9.1	fUTF8	边
U+8FB9.1	fCNS11643	3-243B
U+8FB9.1	fGB	0-315F
U+8FB9.1	fR/S	 162.2
U+8FB9.1	fTotalStrokes	 6
U+8FB9.1	fCangjie	YKS
U+8FB9.1	fFourCorner	34300
U+8FB9.1	fMandarin	bian5
U+8FBA.0	fUTF8	辺
U+8FBA.0	fGB	E-4029
U+8FBA.0	fR/S	 162.2
U+8FBA.0	fTotalStrokes	 6
U+8FBA.0	fCangjie	YSH
U+8FBA.0	fCantonese	dou6; bin1
U+8FBA.0	fMandarin	bian1
U+8FBA.0	fEnglish	simplified form of &#x908A;
U+8FBB.0	fUTF8	辻
U+8FBB.0	fCNS11643	F-2271
U+8FBB.0	fGB	E-402A
U+8FBB.0	fR/S	 162.2
U+8FBB.0	fTotalStrokes	 6
U+8FBB.0	fCangjie	YJ
U+8FBB.0	fFourCorner	3430
U+8FBB.0	fCantonese	sap6
U+8FBB.0	fEnglish	 crossroads street
U+8FBC.0	fUTF8	込
U+8FBC.0	fCNS11643	F-2272
U+8FBC.0	fGB	E-402B
U+8FBC.0	fR/S	 162.2
U+8FBC.0	fTotalStrokes	 6
U+8FBC.0	fCangjie	YO
U+8FBC.0	fFourCorner	3830
U+8FBC.0	fCantonese	jyu1
U+8FBC.0	fEnglish	crowd into; go into
U+8FBD.0	fUTF8	辽
U+8FBD.0	fGB	0-4149
U+8FBD.0	fR/S	 162.2
U+8FBD.0	fTotalStrokes	 6
U+8FBD.0	fCangjie	YNN
U+8FBD.0	fHakka	 liau2
U+8FBD.0	fCantonese	liu4
U+8FBD.0	fMandarin	liao2
U+8FBD.0	fEnglish	simplified form of &#x907C;
U+8FBE.0	fUTF8	达
U+8FBE.0	fCNS11643	3-2730
U+8FBE.0	fGB	0-346F
U+8FBE.0	fR/S	 162.3
U+8FBE.0	fTotalStrokes	 7
U+8FBE.0	fCangjie	YK
U+8FBE.0	fFourCorner	34300
U+8FBE.0	fHakka	 tat8
U+8FBE.0	fCantonese	dai3; daat6; taat3
U+8FBE.0	fMandarin	ti4; da2; ta4
U+8FBE.0	fEnglish	simplified form of &#x9054;
U+8FBF.0	fUTF8	辿
U+8FBF.0	fCNS11643	2-2526
U+8FBF.0	fBig5	CBA6
U+8FBF.0	fGB	3-7155
U+8FBF.0	fR/S	 162.3
U+8FBF.0	fTotalStrokes	 7
U+8FBF.0	fCangjie	YU
U+8FBF.0	fFourCorner	32307
U+8FBF.0	fCantonese	cin4; caam1
U+8FBF.0	fMandarin	chan1; chan2
U+8FBF.0	fEnglish	follow; pursue
U+8FC0.0	fUTF8	迀
U+8FC0.0	fCNS11643	4-2521
U+8FC0.0	fGB	5-6F7C
U+8FC0.0	fR/S	 162.3
U+8FC0.0	fTotalStrokes	 7
U+8FC0.0	fCangjie	YMJ
U+8FC0.0	fFourCorner	31300
U+8FC0.0	fHakka	 gon1
U+8FC0.0	fCantonese	gon1
U+8FC0.0	fMandarin	gan1
U+8FC1.0	fUTF8	迁
U+8FC1.0	fCNS11643	3-2733
U+8FC1.0	fGB	0-4728
U+8FC1.0	fR/S	 162.3
U+8FC1.0	fTotalStrokes	 7
U+8FC1.0	fCangjie	YHJ
U+8FC1.0	fFourCorner	32300
U+8FC1.0	fHakka	  cien1; ts'ian1
U+8FC1.0	fCantonese	cin1
U+8FC1.0	fMandarin	qian1; gan1
U+8FC1.0	fEnglish	simplified form of &#x9077;
U+8FC2.0	fUTF8	迂
U+8FC2.0	fCNS11643	1-4B52
U+8FC2.0	fBig5	A8B1
U+8FC2.0	fGB	0-5358
U+8FC2.0	fR/S	 162.3
U+8FC2.0	fTotalStrokes	 7
U+8FC2.0	fCangjie	YMD
U+8FC2.0	fFourCorner	31304
U+8FC2.0	fHakka	  ji1; hi1; ri1; i1
U+8FC2.0	fCantonese	jyu1; jyu4; heoi1
U+8FC2.0	fMandarin	yu1; yu2; yu4; yu3
U+8FC2.0	fEnglish	[1] impractical; unrealistic; old-fashioned; trite; hackneyed; stale; pedantic [2] roundabout; indirect; circuitous; winding [3] make a detour [4] absurd; preposterous
U+8FC3.0	fUTF8	迃
U+8FC3.0	fCNS11643	4-2523
U+8FC3.0	fGB	E-402C
U+8FC3.0	fR/S	 162.3
U+8FC3.0	fTotalStrokes	 7
U+8FC3.0	fCangjie	YMMS
U+8FC3.0	fFourCorner	31300
U+8FC3.0	fHakka	 i1
U+8FC3.0	fCantonese	jyu1
U+8FC3.0	fMandarin	yu1
U+8FC4.0	fUTF8	迄
U+8FC4.0	fCNS11643	1-4B55
U+8FC4.0	fBig5	A8B4
U+8FC4.0	fGB	0-4679
U+8FC4.0	fR/S	 162.3
U+8FC4.0	fTotalStrokes	 7
U+8FC4.0	fCangjie	YON
U+8FC4.0	fFourCorner	38301
U+8FC4.0	fHakka	 hit7; ngiet7; gniat7; nget7
U+8FC4.0	fCantonese	ngat6; hat1 (also); at6 (also)
U+8FC4.0	fMandarin	qi4; xi4
U+8FC4.0	fEnglish	until; till; up to; down to; so far
U+8FC5.0	fUTF8	迅
U+8FC5.0	fCNS11643	1-4B54
U+8FC5.0	fBig5	A8B3
U+8FC5.0	fGB	0-5138
U+8FC5.0	fR/S	 162.3
U+8FC5.0	fTotalStrokes	 7
U+8FC5.0	fCangjie	YNJ
U+8FC5.0	fFourCorner	37301
U+8FC5.0	fHakka	 sin3; shin5; sin5; sun5
U+8FC5.0	fCantonese	seon3
U+8FC5.0	fMandarin	xun4; xin4
U+8FC5.0	fEnglish	swift; quick; rapid; sudden
U+8FC6.0	fUTF8	迆
U+8FC6.0	fCNS11643	1-4B53
U+8FC6.0	fBig5	A8B2
U+8FC6.0	fGB	5-7022
U+8FC6.0	fR/S	 162.3
U+8FC6.0	fTotalStrokes	 7
U+8FC6.0	fCangjie	YPD
U+8FC6.0	fFourCorner	34301
U+8FC6.0	fHakka	 ji3; ri3; i2
U+8FC6.0	fCantonese	ji5; ji4; to1; j
U+8FC6.0	fMandarin	yi3
U+8FC6.0	fEnglish	[1] extend obliquely or along a zigzag course; proceed in a winding way [2] connected; joined
U+8FC6.1	fUTF8	迆
U+8FC6.1	fCNS11643	1-4B53
U+8FC6.1	fBig5	A8B2
U+8FC6.1	fGB	5-7022
U+8FC6.1	fR/S	 162.3
U+8FC6.1	fTotalStrokes	 7
U+8FC6.1	fCangjie	YPD
U+8FC6.1	fFourCorner	34301
U+8FC6.1	fMandarin	yi2
U+8FC6.1	fEnglish	meandering; winding
U+8FC7.0	fUTF8	过
U+8FC7.0	fCNS11643	3-2731
U+8FC7.0	fGB	0-397D
U+8FC7.0	fR/S	 162.3
U+8FC7.0	fTotalStrokes	 7
U+8FC7.0	fCangjie	YDI
U+8FC7.0	fFourCorner	34300
U+8FC7.0	fHakka	 gwo5; kwo5
U+8FC7.0	fCantonese	gwo3; go3 (also); gwo2 (also); go2 (also); gwo1; go1 (also)
U+8FC7.0	fMandarin	guo4; guo1; huo4
U+8FC7.0	fEnglish	simplified form of &#x904E;
U+8FC7.1	fUTF8	过
U+8FC7.1	fCNS11643	3-2731
U+8FC7.1	fGB	0-397D
U+8FC7.1	fR/S	 162.3
U+8FC7.1	fTotalStrokes	 7
U+8FC7.1	fCangjie	YDI
U+8FC7.1	fFourCorner	34300
U+8FC7.1	fMandarin	guo5
U+8FC8.0	fUTF8	迈
U+8FC8.0	fCNS11643	F-2532
U+8FC8.0	fGB	0-4275
U+8FC8.0	fR/S	 162.3
U+8FC8.0	fTotalStrokes	 7
U+8FC8.0	fCangjie	YMS
U+8FC8.0	fHakka	 mai3; mai5
U+8FC8.0	fCantonese	maai6
U+8FC8.0	fMandarin	mai4
U+8FC8.0	fEnglish	simplified form of &#x9081;
U+8FC9.0	fUTF8	迉
U+8FC9.0	fCNS11643	2-2525
U+8FC9.0	fBig5	CBA5
U+8FC9.0	fGB	E-402D
U+8FC9.0	fR/S	 162.3
U+8FC9.0	fTotalStrokes	 7
U+8FC9.0	fCangjie	YS
U+8FC9.0	fFourCorner	37300
U+8FC9.0	fCantonese	cai1; ci4
U+8FC9.0	fMandarin	qi2
U+8FCA.0	fUTF8	迊
U+8FCA.0	fCNS11643	3-2A72
U+8FCA.0	fGB	E-402E
U+8FCA.0	fR/S	  162.4
U+8FCA.0	fTotalStrokes	 8
U+8FCA.0	fCangjie	YMLB
U+8FCA.0	fFourCorner	31300
U+8FCA.0	fHakka	 zap7; zhap7
U+8FCA.0	fCantonese	jing4; jing6
U+8FCA.0	fMandarin	za1
U+8FCA.0	fEnglish	 go round; make a circuit; make a revolution; turn round
U+8FCB.0	fUTF8	迋
U+8FCB.0	fCNS11643	2-286D
U+8FCB.0	fBig5	CDCD
U+8FCB.0	fGB	3-7157
U+8FCB.0	fR/S	 162.4
U+8FCB.0	fTotalStrokes	 8
U+8FCB.0	fCangjie	YMG
U+8FCB.0	fFourCorner	31301
U+8FCB.0	fHakka	 wong6; wong5; vong5
U+8FCB.0	fCantonese	wong6; kwaang3 (also); gwong2; gong2 (also); gwong3
U+8FCB.0	fMandarin	wang4; guang3; guang4; kuang2; kuang1
U+8FCB.0	fEnglish	 travel; go to deceive scare
U+8FCC.0	fUTF8	迌
U+8FCC.0	fCNS11643	3-2A70
U+8FCC.0	fGB	E-402F
U+8FCC.0	fR/S	  162.4
U+8FCC.0	fTotalStrokes	 8
U+8FCC.0	fCangjie	YB
U+8FCC.0	fFourCorner	37300
U+8FCC.0	fCantonese	teot1
U+8FCC.0	fMandarin	jia1
U+8FCC.0	fEnglish	 used in compound for colloquial 'play'
U+8FCD.0	fUTF8	迍
U+8FCD.0	fCNS11643	2-286F
U+8FCD.0	fBig5	CDCF
U+8FCD.0	fGB	3-7158
U+8FCD.0	fR/S	 162.4
U+8FCD.0	fTotalStrokes	 8
U+8FCD.0	fCangjie	YPU
U+8FCD.0	fFourCorner	35301
U+8FCD.0	fHakka	 zhun2; zun2; tun2; zhun1
U+8FCD.0	fCantonese	zeon1; tyun4
U+8FCD.0	fMandarin	zhun1; tun2
U+8FCD.0	fEnglish	a difficult position
U+8FCE.0	fUTF8	迎
U+8FCE.0	fCNS11643	1-4F52
U+8FCE.0	fBig5	AAEF
U+8FCE.0	fGB	0-532D
U+8FCE.0	fR/S	 162.4
U+8FCE.0	fTotalStrokes	 8
U+8FCE.0	fCangjie	YHVL
U+8FCE.0	fFourCorner	37302
U+8FCE.0	fHakka	  ngiang2; ngiang5; niang2; gniang3; ngiang6
U+8FCE.0	fCantonese	jing4; jing6
U+8FCE.0	fMandarin	ying2
U+8FCE.0	fEnglish	 receive; welcome; greet; meet
U+8FCE.1	fUTF8	迎
U+8FCE.1	fCNS11643	1-4F52
U+8FCE.1	fBig5	AAEF
U+8FCE.1	fGB	0-532D
U+8FCE.1	fR/S	 162.4
U+8FCE.1	fTotalStrokes	 8
U+8FCE.1	fCangjie	YHVL
U+8FCE.1	fFourCorner	37302
U+8FCE.1	fMandarin	ying4
U+8FCE.1	fEnglish	 go out in person to meet (a visitor)
U+8FCF.0	fUTF8	迏
U+8FCF.0	fCNS11643	3-2A74
U+8FCF.0	fGB	E-4030
U+8FCF.0	fR/S	  162.4
U+8FCF.0	fTotalStrokes	 8
U+8FCF.0	fCangjie	YKI
U+8FCF.0	fFourCorner	34300
U+8FCF.0	fCantonese	dai3
U+8FCF.0	fMandarin	ti4
U+8FD0.0	fUTF8	运
U+8FD0.0	fCNS11643	4-2826
U+8FD0.0	fGB	0-544B
U+8FD0.0	fR/S	 162.4
U+8FD0.0	fTotalStrokes	 8
U+8FD0.0	fCangjie	YMMI
U+8FD0.0	fFourCorner	31300
U+8FD0.0	fHakka	 jun5
U+8FD0.0	fCantonese	wan6
U+8FD0.0	fMandarin	yun4; yun3
U+8FD0.0	fEnglish	simplified form of &#x904B;
U+8FD1.0	fUTF8	近
U+8FD1.0	fCNS11643	1-4F54
U+8FD1.0	fBig5	AAF1
U+8FD1.0	fGB	0-3D7C
U+8FD1.0	fR/S	 162.4
U+8FD1.0	fTotalStrokes	 8
U+8FD1.0	fCangjie	YHML
U+8FD1.0	fFourCorner	32302
U+8FD1.0	fHakka	  kiun1;  kiun3;  kiun5; kiun6
U+8FD1.0	fCantonese	gan6; kan5 (also); gan2 (also)
U+8FD1.0	fMandarin	jin4; ji4; ji2
U+8FD1.0	fEnglish	[1] near; close (in space) [2] near; close (in time); immediate; recent [3] near; close (in abstract relation); intimate [4] approach; approximate
U+8FD2.0	fUTF8	迒
U+8FD2.0	fCNS11643	2-286C
U+8FD2.0	fBig5	CDCC
U+8FD2.0	fGB	3-7159
U+8FD2.0	fR/S	 162.4
U+8FD2.0	fTotalStrokes	 8
U+8FD2.0	fCangjie	YYHN
U+8FD2.0	fFourCorner	30301
U+8FD2.0	fHakka	 hong2
U+8FD2.0	fCantonese	hong4; gong1
U+8FD2.0	fMandarin	hang2; gang1; heng4; xiang2
U+8FD2.0	fEnglish	[1] animals tracks [2] path; way; road
U+8FD3.0	fUTF8	迓
U+8FD3.0	fCNS11643	2-286E
U+8FD3.0	fBig5	CDCE
U+8FD3.0	fGB	0-6542
U+8FD3.0	fR/S	 162.4; 162.5
U+8FD3.0	fTotalStrokes	 8
U+8FD3.0	fCangjie	YMVH
U+8FD3.0	fFourCorner	31304
U+8FD3.0	fHakka	  nga3; la2; nga2; nga5
U+8FD3.0	fCantonese	ngaa6; aa6 (also)
U+8FD3.0	fMandarin	ya4; yu4
U+8FD3.0	fEnglish	 go out to meet; receive; go out to receive
U+8FD4.0	fUTF8	返
U+8FD4.0	fCNS11643	1-4F53
U+8FD4.0	fBig5	AAF0
U+8FD4.0	fGB	0-3735
U+8FD4.0	fR/S	 162.4
U+8FD4.0	fTotalStrokes	 8
U+8FD4.0	fCangjie	YHE
U+8FD4.0	fFourCorner	31304
U+8FD4.0	fHakka	  fan3; fan1
U+8FD4.0	fCantonese	faan2; faan1 (also)
U+8FD4.0	fMandarin	fan3
U+8FD4.0	fEnglish	[1] go back; return; revert to; restore [2] send back; give back; return
U+8FD5.0	fUTF8	迕
U+8FD5.0	fCNS11643	2-2871
U+8FD5.0	fBig5	CDD1
U+8FD5.0	fGB	0-6543
U+8FD5.0	fR/S	 162.4
U+8FD5.0	fTotalStrokes	 8
U+8FD5.0	fCangjie	YOJ
U+8FD5.0	fFourCorner	38304
U+8FD5.0	fHakka	 ng3;  ngu3; ngu6
U+8FD5.0	fCantonese	ng6 (also); ng5
U+8FD5.0	fMandarin	wu3; wu4
U+8FD5.0	fEnglish	[1] meet [2] oppose; disobey
U+8FD6.0	fUTF8	迖
U+8FD6.0	fCNS11643	2-2870
U+8FD6.0	fBig5	CDD0
U+8FD6.0	fGB	E-4031
U+8FD6.0	fR/S	 162.4
U+8FD6.0	fTotalStrokes	 8
U+8FD6.0	fCangjie	YIK
U+8FD6.0	fFourCorner	33303
U+8FD6.0	fCantonese	dai3; daat6; taat3; tai3
U+8FD6.0	fMandarin	da2; ta4; ti4; wu4
U+8FD7.0	fUTF8	迗
U+8FD7.0	fCNS11643	2-2872
U+8FD7.0	fBig5	CDD2
U+8FD7.0	fGB	E-4032
U+8FD7.0	fR/S	 162.4
U+8FD7.0	fTotalStrokes	 8
U+8FD7.0	fCangjie	YMK
U+8FD7.0	fFourCorner	32303
U+8FD7.0	fCantonese	ngo4
U+8FD7.0	fMandarin	e2
U+8FD8.0	fUTF8	还
U+8FD8.0	fCNS11643	3-2A73
U+8FD8.0	fGB	0-3B39
U+8FD8.0	fR/S	 162.4
U+8FD8.0	fTotalStrokes	 8
U+8FD8.0	fCangjie	YMF
U+8FD8.0	fFourCorner	31300
U+8FD8.0	fHakka	 fan2;  wan2; han2; van2; {han2}
U+8FD8.0	fCantonese	waan4; syun4
U+8FD8.0	fMandarin	hai2; huan4; ying2; xuan2
U+8FD8.0	fEnglish	simplified form of &#x9084;
U+8FD8.1	fUTF8	还
U+8FD8.1	fCNS11643	3-2A73
U+8FD8.1	fGB	0-3B39
U+8FD8.1	fR/S	 162.4
U+8FD8.1	fTotalStrokes	 8
U+8FD8.1	fCangjie	YMF
U+8FD8.1	fFourCorner	31300
U+8FD8.1	fMandarin	huan2
U+8FD9.0	fUTF8	这
U+8FD9.0	fCNS11643	3-2A71
U+8FD9.0	fGB	0-5562
U+8FD9.0	fR/S	 162.4
U+8FD9.0	fTotalStrokes	 8
U+8FD9.0	fCangjie	YYK
U+8FD9.0	fFourCorner	30300
U+8FD9.0	fHakka	 zhe3
U+8FD9.0	fCantonese	ze5; ze3 (also); ze2 (also)
U+8FD9.0	fMandarin	zhe4; yan4; zhei4
U+8FD9.0	fEnglish	simplified form of &#x9019;
U+8FDA.0	fUTF8	迚
U+8FDA.0	fGB	E-4033
U+8FDA.0	fR/S	 162.4
U+8FDA.0	fTotalStrokes	 8
U+8FDA.0	fCangjie	YL
U+8FDA.0	fFourCorner	3530
U+8FDA.0	fCantonese	zung1
U+8FDA.0	fEnglish	 very somehow
U+8FDB.0	fUTF8	进
U+8FDB.0	fGB	0-3D78
U+8FDB.0	fR/S	 162.4
U+8FDB.0	fTotalStrokes	 8
U+8FDB.0	fCangjie	YTT
U+8FDB.0	fHakka	 zin5
U+8FDB.0	fCantonese	zeon3
U+8FDB.0	fMandarin	jin4
U+8FDB.0	fEnglish	simplified form of &#x9032;
U+8FDC.0	fUTF8	远
U+8FDC.0	fCNS11643	F-2871
U+8FDC.0	fGB	0-5436
U+8FDC.0	fR/S	 162.4
U+8FDC.0	fTotalStrokes	 8
U+8FDC.0	fCangjie	YMMU
U+8FDC.0	fFourCorner	3130
U+8FDC.0	fHakka	 jen3
U+8FDC.0	fCantonese	jyun5; jyun2 (also); jyun6
U+8FDC.0	fMandarin	yuan3; yung4; yuan4
U+8FDC.0	fEnglish	simplified form of &#x9060;
U+8FDD.0	fUTF8	违
U+8FDD.0	fGB	0-4E25
U+8FDD.0	fR/S	 162.4
U+8FDD.0	fTotalStrokes	 8
U+8FDD.0	fCangjie	YQS
U+8FDD.0	fHakka	 vui2
U+8FDD.0	fCantonese	wai4
U+8FDD.0	fMandarin	wei2; hui2
U+8FDD.0	fEnglish	simplified form of &#x9055;
U+8FDE.0	fUTF8	连
U+8FDE.0	fGB	0-412C
U+8FDE.0	fR/S	 162.4
U+8FDE.0	fTotalStrokes	 8
U+8FDE.0	fCangjie	YKQ
U+8FDE.0	fHakka	 lien2
U+8FDE.0	fCantonese	lin4
U+8FDE.0	fMandarin	lian2; lan4; lian3; lian4
U+8FDE.0	fEnglish	simplified form of &#x9023;
U+8FDF.0	fUTF8	迟
U+8FDF.0	fCNS11643	F-286E
U+8FDF.0	fGB	0-3359
U+8FDF.0	fR/S	 162.4
U+8FDF.0	fTotalStrokes	 8
U+8FDF.0	fCangjie	YSO
U+8FDF.0	fHakka	 chi2
U+8FDF.0	fCantonese	ci4; zi3
U+8FDF.0	fMandarin	chi2; zhi2
U+8FDF.0	fEnglish	simplified form of &#x9072;
U+8FE0.0	fUTF8	迠
U+8FE0.0	fCNS11643	2-2D57
U+8FE0.0	fBig5	D0B6
U+8FE0.0	fGB	E-4034
U+8FE0.0	fR/S	 162.5
U+8FE0.0	fTotalStrokes	 9
U+8FE0.0	fCangjie	YYR
U+8FE0.0	fFourCorner	31306
U+8FE0.0	fCantonese	gin3; cip3
U+8FE0.0	fMandarin	che4; xi2
U+8FE0.0	fEnglish	made
U+8FE1.0	fUTF8	迡
U+8FE1.0	fCNS11643	2-2D55
U+8FE1.0	fBig5	D0B4
U+8FE1.0	fGB	5-702C
U+8FE1.0	fR/S	 162.5
U+8FE1.0	fTotalStrokes	 9
U+8FE1.0	fCangjie	YSP
U+8FE1.0	fFourCorner	37301
U+8FE1.0	fCantonese	nai6; lai6 (also); ci4
U+8FE1.0	fMandarin	ni4
U+8FE2.0	fUTF8	迢
U+8FE2.0	fCNS11643	1-535F
U+8FE2.0	fBig5	AD7C
U+8FE2.0	fGB	0-4C76
U+8FE2.0	fR/S	 162.5
U+8FE2.0	fTotalStrokes	 9
U+8FE2.0	fCangjie	YSHR
U+8FE2.0	fFourCorner	37306
U+8FE2.0	fHakka	  tiau2; tiau1; siau2; tiau3
U+8FE2.0	fCantonese	tiu4; siu4 (also)
U+8FE2.0	fMandarin	tiao2
U+8FE2.0	fEnglish	far; distant; remote
U+8FE3.0	fUTF8	迣
U+8FE3.0	fCNS11643	2-2D54
U+8FE3.0	fBig5	D0B3
U+8FE3.0	fGB	3-715A
U+8FE3.0	fR/S	 162.5
U+8FE3.0	fTotalStrokes	 9
U+8FE3.0	fCangjie	YPT
U+8FE3.0	fFourCorner	34307
U+8FE3.0	fCantonese	lit6; lai6; sai3
U+8FE3.0	fMandarin	zhi4; chi4; li4
U+8FE3.0	fEnglish	 leap over
U+8FE4.0	fUTF8	迤
U+8FE4.0	fCNS11643	1-5364
U+8FE4.0	fBig5	ADA3
U+8FE4.0	fGB	0-6546
U+8FE4.0	fR/S	 162.5
U+8FE4.0	fTotalStrokes	 9
U+8FE4.0	fCangjie	YOPD
U+8FE4.0	fFourCorner	38301
U+8FE4.0	fHakka	 ji3; ri3; i2; ji2
U+8FE4.0	fCantonese	ji5; ji4; to1
U+8FE4.0	fMandarin	yi3; yi2; tuo2; tuo1
U+8FE4.0	fEnglish	same as $
U+8FE5.0	fUTF8	迥
U+8FE5.0	fCNS11643	1-5361
U+8FE5.0	fBig5	AD7E
U+8FE5.0	fGB	0-6544
U+8FE5.0	fR/S	 162.5
U+8FE5.0	fTotalStrokes	 9
U+8FE5.0	fCangjie	YBR
U+8FE5.0	fFourCorner	37302
U+8FE5.0	fHakka	 kwen3; gen3; gwen3
U+8FE5.0	fCantonese	gwing2
U+8FE5.0	fMandarin	jiong3; xiong4
U+8FE5.0	fEnglish	[1] faraway; distant; far [2] widely different
U+8FE6.0	fUTF8	迦
U+8FE6.0	fCNS11643	1-535E
U+8FE6.0	fBig5	AD7B
U+8FE6.0	fGB	0-6548
U+8FE6.0	fR/S	 162.5
U+8FE6.0	fTotalStrokes	 9
U+8FE6.0	fCangjie	YKSR
U+8FE6.0	fFourCorner	36300
U+8FE6.0	fHakka	 ga1
U+8FE6.0	fCantonese	gaa1; haai6
U+8FE6.0	fMandarin	jia1; xie4
U+8FE6.0	fEnglish	a character used for transliteration
U+8FE7.0	fUTF8	迧
U+8FE7.0	fCNS11643	3-2F31
U+8FE7.0	fGB	E-4035
U+8FE7.0	fR/S	 162.5
U+8FE7.0	fTotalStrokes	 9
U+8FE7.0	fCangjie	YLWL
U+8FE7.0	fFourCorner	35300
U+8FE7.0	fCantonese	can4
U+8FE7.0	fMandarin	chen2; zhen4
U+8FE7.0	fEnglish	 a Chinese family name arrange display a column of troops
U+8FE8.0	fUTF8	迨
U+8FE8.0	fCNS11643	1-5365
U+8FE8.0	fBig5	ADA4
U+8FE8.0	fGB	0-654A
U+8FE8.0	fR/S	 162.5
U+8FE8.0	fTotalStrokes	 9
U+8FE8.0	fCangjie	YIR
U+8FE8.0	fFourCorner	33306
U+8FE8.0	fHakka	  tai5; tai6
U+8FE8.0	fCantonese	doi6; toi5 (also)
U+8FE8.0	fMandarin	dai4
U+8FE8.0	fEnglish	until; by the time when; wait till
U+8FE9.0	fUTF8	迩
U+8FE9.0	fCNS11643	3-2F32
U+8FE9.0	fGB	0-6547
U+8FE9.0	fR/S	  162.5
U+8FE9.0	fTotalStrokes	 9
U+8FE9.0	fCangjie	YNF
U+8FE9.0	fFourCorner	37300
U+8FE9.0	fHakka	 ni3
U+8FE9.0	fCantonese	ji5
U+8FE9.0	fMandarin	er3
U+8FE9.0	fEnglish	simplified form of &#x9087;
U+8FEA.0	fUTF8	迪
U+8FEA.0	fCNS11643	1-5360
U+8FEA.0	fBig5	AD7D
U+8FEA.0	fGB	0-354F
U+8FEA.0	fR/S	 162.5
U+8FEA.0	fTotalStrokes	 9
U+8FEA.0	fCangjie	YLW
U+8FEA.0	fFourCorner	35306
U+8FEA.0	fHakka	 tit8; tit7
U+8FEA.0	fCantonese	dik6
U+8FEA.0	fMandarin	di2
U+8FEA.0	fEnglish	[1] advance; progress [2] enlighten; teach
U+8FEB.0	fUTF8	迫
U+8FEB.0	fCNS11643	1-5363
U+8FEB.0	fBig5	ADA2
U+8FEB.0	fGB	0-4648
U+8FEB.0	fR/S	 162.5
U+8FEB.0	fTotalStrokes	 9
U+8FEB.0	fCangjie	YHA
U+8FEB.0	fFourCorner	36300
U+8FEB.0	fHakka	  bet7; pek7; bit7
U+8FEB.0	fCantonese	baak1; bik1; baak3
U+8FEB.0	fMandarin	po4; pai3; bo2
U+8FEB.0	fEnglish	[1] coerce; force; compel; stress; press [2] pressing; urgent; immiment [3] pressed; distressed
U+8FEC.0	fUTF8	迬
U+8FEC.0	fCNS11643	6-3460
U+8FEC.0	fGB	E-4036
U+8FEC.0	fR/S	 162.4
U+8FEC.0	fTotalStrokes	 9; 8
U+8FEC.0	fCangjie	YYG
U+8FEC.0	fFourCorner	30300
U+8FEC.0	fCantonese	zyu3
U+8FEC.0	fMandarin	wang3
U+8FEC.0	fEnglish	go
U+8FED.0	fUTF8	迭
U+8FED.0	fCNS11643	1-5362
U+8FED.0	fBig5	ADA1
U+8FED.0	fGB	0-357C
U+8FED.0	fR/S	 162.5
U+8FED.0	fTotalStrokes	 9
U+8FED.0	fCangjie	YHQO
U+8FED.0	fFourCorner	35303
U+8FED.0	fHakka	  diet7; det7
U+8FED.0	fCantonese	dit6; jat6
U+8FED.0	fMandarin	die2
U+8FED.0	fEnglish	[1] alternate [2] repeatedly; frequently [3] used in combination for to stop [4] here
U+8FEE.0	fUTF8	迮
U+8FEE.0	fCNS11643	2-2D56
U+8FEE.0	fBig5	D0B5
U+8FEE.0	fGB	0-6545
U+8FEE.0	fR/S	 162.5
U+8FEE.0	fTotalStrokes	 9
U+8FEE.0	fCangjie	YOS
U+8FEE.0	fFourCorner	38301
U+8FEE.0	fCantonese	zaak3; zok3
U+8FEE.0	fMandarin	ze2; zuo4
U+8FEE.0	fEnglish	[1] pressing; urgent [2] hurried; helter-skelter
U+8FEF.0	fUTF8	迯
U+8FEF.0	fCNS11643	3-2F33
U+8FEF.0	fGB	5-702A
U+8FEF.0	fR/S	 162.5
U+8FEF.0	fTotalStrokes	 9
U+8FEF.0	fCangjie	YNIY
U+8FEF.0	fFourCorner	33300
U+8FEF.0	fHakka	  tau2; to2
U+8FEF.0	fCantonese	tou4
U+8FEF.0	fMandarin	tao2
U+8FEF.0	fEnglish	 escape; flee abscond; dodge
U+8FF0.0	fUTF8	述
U+8FF0.0	fCNS11643	1-535D
U+8FF0.0	fBig5	AD7A
U+8FF0.0	fGB	0-4A76
U+8FF0.0	fR/S	 162.5
U+8FF0.0	fTotalStrokes	 9
U+8FF0.0	fCangjie	YIJC
U+8FF0.0	fFourCorner	33309
U+8FF0.0	fHakka	  sut8;  shut8; sut7
U+8FF0.0	fCantonese	seot6
U+8FF0.0	fMandarin	shu4
U+8FF0.0	fEnglish	[1] give an account of; explain; expound [2] follow (precedents); carry forward; continue (another's work)
U+8FF1.0	fUTF8	迱
U+8FF1.0	fCNS11643	3-2F30
U+8FF1.0	fGB	5-702B
U+8FF1.0	fR/S	 162.5
U+8FF1.0	fTotalStrokes	 9
U+8FF1.0	fCangjie	YJP
U+8FF1.0	fFourCorner	33300
U+8FF1.0	fCantonese	to4
U+8FF1.0	fMandarin	tuo2
U+8FF2.0	fUTF8	迲
U+8FF2.0	fGB	1-7D25
U+8FF2.0	fR/S	 162.5
U+8FF2.0	fTotalStrokes	 9
U+8FF2.0	fCangjie	YGI
U+8FF2.0	fFourCorner	3430
U+8FF2.0	fCantonese	heoi3
U+8FF3.0	fUTF8	迳
U+8FF3.0	fGB	0-6549
U+8FF3.0	fR/S	 162.5
U+8FF3.0	fTotalStrokes	 9
U+8FF3.0	fCangjie	YNOM
U+8FF3.0	fHakka	 gang5
U+8FF3.0	fCantonese	ging3; gaang3 (also)
U+8FF3.0	fMandarin	jing4
U+8FF3.0	fEnglish	simplified form of &#x9015;
U+8FF4.0	fUTF8	迴
U+8FF4.0	fCNS11643	1-584F
U+8FF4.0	fBig5	B06A
U+8FF4.0	fGB	1-783C
U+8FF4.0	fR/S	 162.6
U+8FF4.0	fTotalStrokes	 10
U+8FF4.0	fCangjie	YWR
U+8FF4.0	fFourCorner	36300
U+8FF4.0	fHakka	  fui2; fi2
U+8FF4.0	fCantonese	wui4
U+8FF4.0	fMandarin	hui2
U+8FF4.0	fEnglish	[1] turn; revolve; rotate [2] wind; zigzag
U+8FF5.0	fUTF8	迵
U+8FF5.0	fCNS11643	2-332F
U+8FF5.0	fBig5	D3EB
U+8FF5.0	fGB	3-715E
U+8FF5.0	fR/S	 162.6
U+8FF5.0	fTotalStrokes	 10
U+8FF5.0	fCangjie	YBMR
U+8FF5.0	fFourCorner	37302
U+8FF5.0	fCantonese	dung6
U+8FF5.0	fMandarin	tong2; dong4
U+8FF6.0	fUTF8	迶
U+8FF6.0	fCNS11643	2-3335
U+8FF6.0	fBig5	D3F1
U+8FF6.0	fGB	E-4037
U+8FF6.0	fR/S	 162.6
U+8FF6.0	fTotalStrokes	 10
U+8FF6.0	fCangjie	YKB
U+8FF6.0	fFourCorner	34302
U+8FF6.0	fCantonese	jau6
U+8FF6.0	fMandarin	you4
U+8FF7.0	fUTF8	迷
U+8FF7.0	fCNS11643	1-584C
U+8FF7.0	fBig5	B067
U+8FF7.0	fGB	0-4354
U+8FF7.0	fR/S	 162.6
U+8FF7.0	fTotalStrokes	 10
U+8FF7.0	fCangjie	YFD
U+8FF7.0	fFourCorner	39309
U+8FF7.0	fHakka	  mi2; mi3
U+8FF7.0	fCantonese	mai4
U+8FF7.0	fMandarin	mi2; mei4; mi4
U+8FF7.0	fEnglish	[1] indistinct; vague; dim [2] bewitch; charm; infatuate; be crazy about; enchant; fascinate [3] fan; fiend
U+8FF8.0	fUTF8	迸
U+8FF8.0	fCNS11643	1-5853
U+8FF8.0	fBig5	B06E
U+8FF8.0	fGB	0-3145
U+8FF8.0	fR/S	 162.8
U+8FF8.0	fTotalStrokes	 12
U+8FF8.0	fCangjie	YTT
U+8FF8.0	fFourCorner	38305
U+8FF8.0	fHakka	  bin5
U+8FF8.0	fCantonese	bing3; bing2
U+8FF8.0	fMandarin	beng4; bing3; peng1
U+8FF8.0	fEnglish	[1] scatter; explode [2] crack; burst forth; spit out
U+8FF9.0	fUTF8	迹
U+8FF9.0	fCNS11643	3-3425
U+8FF9.0	fGB	0-3C23
U+8FF9.0	fR/S	 162.6
U+8FF9.0	fTotalStrokes	 10
U+8FF9.0	fCangjie	YYLC
U+8FF9.0	fFourCorner	30300
U+8FF9.0	fHakka	  ziak7; zit7
U+8FF9.0	fCantonese	zik1
U+8FF9.0	fMandarin	ji1
U+8FF9.0	fEnglish	footprints; traces; tracks
U+8FFA.0	fUTF8	迺
U+8FFA.0	fCNS11643	1-584E
U+8FFA.0	fBig5	B069
U+8FFA.0	fGB	8-2D5C
U+8FFA.0	fR/S	 162.6
U+8FFA.0	fTotalStrokes	 10
U+8FFA.0	fCangjie	YMCW
U+8FFA.0	fFourCorner	31306
U+8FFA.0	fHakka	 nai3; lai1; nai1; ne3; {nai1}
U+8FFA.0	fCantonese	naai5; laai5 (also); oi2
U+8FFA.0	fMandarin	nai3
U+8FFA.0	fEnglish	a conjunctive and disjunctive particle with a variety of meanings; then; thereupon; only then
U+8FFB.0	fUTF8	迻
U+8FFB.0	fCNS11643	2-3332
U+8FFB.0	fBig5	D3EE
U+8FFB.0	fGB	E-4038
U+8FFB.0	fR/S	 162.6
U+8FFB.0	fTotalStrokes	 10
U+8FFB.0	fCangjie	YNIN
U+8FFB.0	fFourCorner	37302
U+8FFB.0	fHakka	 ji2; ri2; i2
U+8FFB.0	fCantonese	ji4
U+8FFB.0	fMandarin	yi2
U+8FFB.0	fEnglish	 transfer; translate
U+8FFC.0	fUTF8	迼
U+8FFC.0	fCNS11643	2-3334
U+8FFC.0	fBig5	D3F0
U+8FFC.0	fGB	3-715C
U+8FFC.0	fR/S	 162.6
U+8FFC.0	fTotalStrokes	 10
U+8FFC.0	fCangjie	YGR
U+8FFC.0	fFourCorner	34306
U+8FFC.0	fCantonese	kit3; git3
U+8FFC.0	fMandarin	jie2
U+8FFD.0	fUTF8	追
U+8FFD.0	fCNS11643	1-5851
U+8FFD.0	fBig5	B06C
U+8FFD.0	fGB	0-5737
U+8FFD.0	fR/S	 162.6
U+8FFD.0	fTotalStrokes	 10
U+8FFD.0	fCangjie	YHRR
U+8FFD.0	fFourCorner	37307
U+8FFD.0	fHakka	  zui1;  chui1;  dui1; zhui1
U+8FFD.0	fCantonese	zeoi1; deoi1
U+8FFD.0	fMandarin	zhui1; dui1; sui2; tui1; tui4; zhui4
U+8FFD.0	fEnglish	[1] pursue; chase; follow; trace [2] expel; drive [3] demand insistently; dun for [4] try to recover (stolen goods, etc.); retrieve
U+8FFE.0	fUTF8	迾
U+8FFE.0	fCNS11643	2-332E
U+8FFE.0	fBig5	D3EA
U+8FFE.0	fGB	3-715D
U+8FFE.0	fR/S	 162.6
U+8FFE.0	fTotalStrokes	 10
U+8FFE.0	fCangjie	YMNN
U+8FFE.0	fFourCorner	32300
U+8FFE.0	fHakka	 liet8
U+8FFE.0	fCantonese	lit6; laat6 (also)
U+8FFE.0	fMandarin	lie4; li4
U+8FFE.0	fEnglish	row
U+8FFF.0	fUTF8	迿
U+8FFF.0	fCNS11643	2-3331
U+8FFF.0	fBig5	D3ED
U+8FFF.0	fGB	3-7160
U+8FFF.0	fR/S	 162.6
U+8FFF.0	fTotalStrokes	 10
U+8FFF.0	fCangjie	YPA
U+8FFF.0	fFourCorner	37302
U+8FFF.0	fHakka	 sun5;  zun5
U+8FFF.0	fCantonese	seon6; seon3
U+8FFF.0	fMandarin	xun4; xuan4; xun2
U+8FFF.0	fEnglish	 be the first to begin a quarrel
U+9000.0	fUTF8	退
U+9000.0	fCNS11643	1-584D
U+9000.0	fBig5	B068
U+9000.0	fGB	0-4D4B
U+9000.0	fR/S	 162.6
U+9000.0	fTotalStrokes	 10
U+9000.0	fCangjie	YAV
U+9000.0	fFourCorner	37303
U+9000.0	fHakka	  tui5; toi5
U+9000.0	fCantonese	teoi3
U+9000.0	fMandarin	tui4; tun4
U+9000.0	fEnglish	[1] step back; retreat; withdraw; recede; regress; retrogress [2] recoil; shrink [3] bow out; retire [4] send back; give back; return
U+9001.0	fUTF8	送
U+9001.0	fCNS11643	1-584A
U+9001.0	fBig5	B065
U+9001.0	fGB	0-4B4D
U+9001.0	fR/S	 162.6
U+9001.0	fTotalStrokes	 10
U+9001.0	fCangjie	YTK
U+9001.0	fFourCorner	38303
U+9001.0	fHakka	  sung5
U+9001.0	fCantonese	sung3
U+9001.0	fMandarin	song4
U+9001.0	fEnglish	[1] send; dispatch; deliver; convey [2] present; give [3] see someone off; send off; wish Godspeed to; send away in a complimetary manner
U+9002.0	fUTF8	适
U+9002.0	fCNS11643	2-3330
U+9002.0	fBig5	D3EC
U+9002.0	fGB	0-4A4A
U+9002.0	fR/S	 162.6
U+9002.0	fTotalStrokes	 10
U+9002.0	fCangjie	YHJR
U+9002.0	fFourCorner	32306
U+9002.0	fHakka	 kwat7; gat7; gwat7; shit7
U+9002.0	fCantonese	sik1; kut3; dik1
U+9002.0	fMandarin	shi4; kuo4; gua1; di4; ti4; zhe2; zhi1
U+9002.0	fEnglish	[1] quick; swift; fast [2] simplified form of &#x9069;
U+9003.0	fUTF8	逃
U+9003.0	fCNS11643	1-5850
U+9003.0	fBig5	B06B
U+9003.0	fGB	0-4C53
U+9003.0	fR/S	 162.6
U+9003.0	fTotalStrokes	 10
U+9003.0	fCangjie	YLMO
U+9003.0	fFourCorner	33300
U+9003.0	fHakka	  tau2; to2; to3
U+9003.0	fCantonese	tou4
U+9003.0	fMandarin	tao2
U+9003.0	fEnglish	[1] run away; escape; flee; abscond; fly [2] dodge; evade; avoid; shirk
U+9004.0	fUTF8	逄
U+9004.0	fCNS11643	2-3333
U+9004.0	fBig5	D3EF
U+9004.0	fGB	0-654C
U+9004.0	fR/S	 162.6; 162.7
U+9004.0	fTotalStrokes	 10
U+9004.0	fCangjie	YHEQ
U+9004.0	fFourCorner	37305
U+9004.0	fHakka	 pong2
U+9004.0	fCantonese	pong4
U+9004.0	fMandarin	pang2; feng2
U+9004.0	fEnglish	a Chinese family name
U+9005.0	fUTF8	逅
U+9005.0	fCNS11643	1-5852
U+9005.0	fBig5	B06D
U+9005.0	fGB	0-654B
U+9005.0	fR/S	 162.6
U+9005.0	fTotalStrokes	 10
U+9005.0	fCangjie	YHMR
U+9005.0	fFourCorner	32306
U+9005.0	fHakka	 heu5; kieu5; heu6; hiu3; geu5
U+9005.0	fCantonese	hau6
U+9005.0	fMandarin	hou4; gou4; hou2
U+9005.0	fEnglish	 meet unexpectedly; come across; run into
U+9006.0	fUTF8	逆
U+9006.0	fCNS11643	1-584B
U+9006.0	fBig5	B066
U+9006.0	fGB	0-4466
U+9006.0	fR/S	 162.6
U+9006.0	fTotalStrokes	 10
U+9006.0	fCangjie	YTU
U+9006.0	fFourCorner	38304
U+9006.0	fHakka	  ngiak8; gniak8
U+9006.0	fCantonese	jik6; ngaak6 (also); aak6 (also)
U+9006.0	fMandarin	ni4
U+9006.0	fEnglish	[1] meet; welcome [2] oppose; go against; disobey; rebel; rebel [3] beforehand; in advance [4] inverse; converse; adverse; contrary
U+9007.0	fUTF8	逇
U+9007.0	fCNS11643	4-302F
U+9007.0	fGB	1-7D5C
U+9007.0	fR/S	 162.6
U+9007.0	fTotalStrokes	 10
U+9007.0	fCangjie	YGPD
U+9007.0	fFourCorner	34300
U+9007.0	fCantonese	deon6
U+9007.0	fMandarin	dun4
U+9008.0	fUTF8	逈
U+9008.0	fCNS11643	3-3427
U+9008.0	fGB	E-4039
U+9008.0	fR/S	 162.6
U+9008.0	fTotalStrokes	 10
U+9008.0	fCangjie	YHBR
U+9008.0	fFourCorner	37300
U+9008.0	fHakka	 gwen3
U+9008.0	fCantonese	gwing2
U+9008.0	fMandarin	jiong3
U+9008.0	fEnglish	far
U+9009.0	fUTF8	选
U+9009.0	fCNS11643	3-3426
U+9009.0	fGB	0-5121
U+9009.0	fR/S	 162.6
U+9009.0	fTotalStrokes	 10
U+9009.0	fCangjie	YHGU
U+9009.0	fFourCorner	34300
U+9009.0	fHakka	 sien3; sen3
U+9009.0	fCantonese	syun2; syun3
U+9009.0	fMandarin	xuan3; suan4; shua1; xuan4; zhuan4
U+9009.0	fEnglish	simplified form of &#x9078;
U+900A.0	fUTF8	逊
U+900A.0	fGB	0-5137
U+900A.0	fR/S	 162.6
U+900A.0	fTotalStrokes	 10
U+900A.0	fCangjie	YNDF
U+900A.0	fHakka	 sun5
U+900A.0	fCantonese	seon3
U+900A.0	fMandarin	xun4; sun4
U+900A.0	fEnglish	simplified form of &#x905C;
U+900B.0	fUTF8	逋
U+900B.0	fCNS11643	2-3966
U+900B.0	fBig5	D7E3
U+900B.0	fGB	0-654D
U+900B.0	fR/S	 162.7
U+900B.0	fTotalStrokes	 11
U+900B.0	fCangjie	YIJB
U+900B.0	fFourCorner	33302
U+900B.0	fHakka	  pu2
U+900B.0	fCantonese	bou1
U+900B.0	fMandarin	bu1
U+900B.0	fEnglish	[1] flee; run away; abscond [2] neglect
U+900C.0	fUTF8	逌
U+900C.0	fCNS11643	2-3969
U+900C.0	fBig5	D7E6
U+900C.0	fGB	3-7165
U+900C.0	fR/S	 162.7
U+900C.0	fTotalStrokes	 11
U+900C.0	fCangjie	YYWS
U+900C.0	fFourCorner	31306
U+900C.0	fHakka	 ju2; riu2
U+900C.0	fCantonese	jau4
U+900C.0	fMandarin	you2
U+900C.0	fEnglish	pleased; smiling; conplacent
U+900D.0	fUTF8	逍
U+900D.0	fCNS11643	1-5D56
U+900D.0	fBig5	B370
U+900D.0	fGB	0-6550
U+900D.0	fR/S	 162.7
U+900D.0	fTotalStrokes	 11
U+900D.0	fCangjie	YFB
U+900D.0	fFourCorner	39302
U+900D.0	fHakka	  siau1
U+900D.0	fCantonese	siu1
U+900D.0	fMandarin	xiao1
U+900D.0	fEnglish	[1] wander in a leisurely manner; saunter; stroll; loiter [2] free and unfettered; be leisurely and carefree; take one's ease; at ease
U+900E.0	fUTF8	逎
U+900E.0	fCNS11643	3-394B
U+900E.0	fGB	3-7164
U+900E.0	fR/S	 162.7
U+900E.0	fTotalStrokes	 11
U+900E.0	fCangjie	YMCW
U+900E.0	fFourCorner	31300
U+900E.0	fCantonese	cau4
U+900E.0	fMandarin	qiu2
U+900E.0	fEnglish	strong; unyielding; forceful
U+900F.0	fUTF8	透
U+900F.0	fCNS11643	1-5D60
U+900F.0	fBig5	B37A
U+900F.0	fGB	0-4D38
U+900F.0	fR/S	 162.7
U+900F.0	fTotalStrokes	 11
U+900F.0	fCangjie	YHDS
U+900F.0	fFourCorner	32302
U+900F.0	fHakka	  teu5; tiu5; teu6; teu3
U+900F.0	fCantonese	tau3
U+900F.0	fMandarin	tou4; shu2; shu1
U+900F.0	fEnglish	[1] penetrate; pass through [2] let out; let through [3] thorough; quite; complete [4] appear; show
U+9010.0	fUTF8	逐
U+9010.0	fCNS11643	1-5D5C
U+9010.0	fBig5	B376
U+9010.0	fGB	0-5670
U+9010.0	fR/S	 162.7
U+9010.0	fTotalStrokes	 11
U+9010.0	fCangjie	YMSO
U+9010.0	fFourCorner	31303
U+9010.0	fHakka	 cuk8;  ch'uk8; giuk7; lak7; chuk8
U+9010.0	fCantonese	zuk6; dik6
U+9010.0	fMandarin	zhu2; di2; tun2; zhou4
U+9010.0	fEnglish	[1] chase; pursue; follow [2] drive off; expel; banish; exile [3] little by little; one by one; gradually
U+9011.0	fUTF8	逑
U+9011.0	fCNS11643	2-3967
U+9011.0	fBig5	D7E4
U+9011.0	fGB	0-654F
U+9011.0	fR/S	 162.7
U+9011.0	fTotalStrokes	 11
U+9011.0	fCangjie	YIJE
U+9011.0	fFourCorner	33303
U+9011.0	fHakka	 kiu2; kiu3
U+9011.0	fCantonese	kau4
U+9011.0	fMandarin	qiu2
U+9011.0	fEnglish	[1] collect; draw together; gather together [2] match; pair; marry [3] mate
U+9012.0	fUTF8	递
U+9012.0	fCNS11643	3-394A
U+9012.0	fGB	0-355D
U+9012.0	fR/S	 162.7
U+9012.0	fTotalStrokes	 11
U+9012.0	fCangjie	YCNH
U+9012.0	fFourCorner	38300
U+9012.0	fHakka	 ti5; tai1; ti3
U+9012.0	fCantonese	dai6
U+9012.0	fMandarin	di4; shi4; dai4
U+9012.0	fEnglish	simplified form of &#x905E;
U+9013.0	fUTF8	逓
U+9013.0	fCNS11643	3-394E
U+9013.0	fGB	E-403A
U+9013.0	fR/S	 162.7
U+9013.0	fTotalStrokes	 11
U+9013.0	fCangjie	YHQB
U+9013.0	fFourCorner	32300
U+9013.0	fHakka	 ti5; tai1
U+9013.0	fCantonese	dai6
U+9013.0	fMandarin	di4
U+9013.0	fEnglish	 hand over; deliver substitute
U+9014.0	fUTF8	途
U+9014.0	fCNS11643	1-5D64
U+9014.0	fBig5	B37E
U+9014.0	fGB	0-4D3E
U+9014.0	fR/S	 162.7
U+9014.0	fTotalStrokes	 11
U+9014.0	fCangjie	YOMD
U+9014.0	fFourCorner	38309
U+9014.0	fHakka	  tu2; tu3; tu5
U+9014.0	fCantonese	tou4
U+9014.0	fMandarin	tu2
U+9014.0	fEnglish	 way; road; path
U+9015.0	fUTF8	逕
U+9015.0	fCNS11643	1-5D5D
U+9015.0	fBig5	B377
U+9015.0	fGB	1-6549
U+9015.0	fR/S	 162.7
U+9015.0	fTotalStrokes	 11
U+9015.0	fCangjie	YMVM
U+9015.0	fFourCorner	31301
U+9015.0	fHakka	 gang5;  gin5; gin1; gang3; kang5
U+9015.0	fCantonese	ging3; gaang3 (also)
U+9015.0	fMandarin	jing4
U+9015.0	fEnglish	 pass by; approach direct
U+9016.0	fUTF8	逖
U+9016.0	fCNS11643	1-5D62
U+9016.0	fBig5	B37C
U+9016.0	fGB	0-6551
U+9016.0	fR/S	 162.7
U+9016.0	fTotalStrokes	 11
U+9016.0	fCangjie	YKHF
U+9016.0	fFourCorner	39308
U+9016.0	fHakka	 tit8
U+9016.0	fCantonese	tik1; dik6
U+9016.0	fMandarin	ti4
U+9016.0	fEnglish	far; distant
U+9017.0	fUTF8	逗
U+9017.0	fCNS11643	1-5D58
U+9017.0	fBig5	B372
U+9017.0	fGB	0-363A
U+9017.0	fR/S	 162.7
U+9017.0	fTotalStrokes	 11
U+9017.0	fCangjie	YMRT
U+9017.0	fFourCorner	31301
U+9017.0	fHakka	  teu5; tiu5; teu3; teu6; tiu3
U+9017.0	fCantonese	dau6
U+9017.0	fMandarin	dou4; tou2; zhu4; qi4; tou4; qi2
U+9017.0	fEnglish	[1] stay; linger; remain; pause [2] arouse; stir; tickle [3] funny [4] a slight pause in reading
U+9018.0	fUTF8	逘
U+9018.0	fCNS11643	4-3568
U+9018.0	fGB	E-403B
U+9018.0	fR/S	 162.7
U+9018.0	fTotalStrokes	 11
U+9018.0	fCangjie	YIOK
U+9018.0	fFourCorner	33300
U+9018.0	fCantonese	ji4
U+9018.0	fMandarin	yi3
U+9019.0	fUTF8	這
U+9019.0	fCNS11643	1-5D55
U+9019.0	fBig5	B36F
U+9019.0	fGB	1-5562
U+9019.0	fR/S	 162.7
U+9019.0	fTotalStrokes	 11
U+9019.0	fCangjie	YYMR
U+9019.0	fFourCorner	30306
U+9019.0	fHakka	 zhe3; ze3; lia3; li3; ja3; ra3; chii3; chii5
U+9019.0	fCantonese	ze5; ze3 (also); ze2 (also)
U+9019.0	fMandarin	zhe4; yan4
U+9019.0	fEnglish	[1] this (pronoun) [2] this (demonstrative adjective); such
U+9019.1	fUTF8	這
U+9019.1	fCNS11643	1-5D55
U+9019.1	fBig5	B36F
U+9019.1	fGB	1-5562
U+9019.1	fR/S	 162.7
U+9019.1	fTotalStrokes	 11
U+9019.1	fCangjie	YYMR
U+9019.1	fFourCorner	30306
U+9019.1	fMandarin	zhei4
U+9019.1	fEnglish	this; that (emphatice demonstrative adjective)
U+901A.0	fUTF8	通
U+901A.0	fCNS11643	1-5D57
U+901A.0	fBig5	B371
U+901A.0	fGB	0-4D28
U+901A.0	fR/S	 162.7
U+901A.0	fTotalStrokes	 11
U+901A.0	fCangjie	YNIB
U+901A.0	fFourCorner	37302
U+901A.0	fHakka	  tung1; tung2
U+901A.0	fCantonese	tung1
U+901A.0	fMandarin	tong1
U+901A.0	fEnglish	[1] go; move; flow unobstructed [2] interchange; communicate [3] lead to; reach [4] understand thoroughly; be versed in [5] let through; through [6] smoot; fluent [7] open; passable [8] all; general; overal; throughout [9] thorough [10] common; popular [11] well-constructed; containing no fallacy (sentence)
U+901A.1	fUTF8	通
U+901A.1	fCNS11643	1-5D57
U+901A.1	fBig5	B371
U+901A.1	fGB	0-4D28
U+901A.1	fR/S	 162.7
U+901A.1	fTotalStrokes	 11
U+901A.1	fCangjie	YNIB
U+901A.1	fFourCorner	37302
U+901A.1	fMandarin	tong4
U+901B.0	fUTF8	逛
U+901B.0	fCNS11643	1-5D63
U+901B.0	fBig5	B37D
U+901B.0	fGB	0-3964
U+901B.0	fR/S	 162.7
U+901B.0	fTotalStrokes	 11
U+901B.0	fCangjie	YKHG
U+901B.0	fFourCorner	32301
U+901B.0	fHakka	 kong3; kang5; ong2; k'wong3
U+901B.0	fCantonese	kwaang3; gwaang6 (also)
U+901B.0	fMandarin	guang4; guang3; kuang2
U+901B.0	fEnglish	 ramble; stroll; roam; wander about
U+901C.0	fUTF8	逜
U+901C.0	fCNS11643	2-3968
U+901C.0	fBig5	D7E5
U+901C.0	fGB	5-7031
U+901C.0	fR/S	 162.7
U+901C.0	fTotalStrokes	 11
U+901C.0	fCangjie	YMMR
U+901C.0	fFourCorner	31306
U+901C.0	fCantonese	ng6
U+901C.0	fMandarin	wu4; wu3
U+901D.0	fUTF8	逝
U+901D.0	fCNS11643	1-5D5B
U+901D.0	fBig5	B375
U+901D.0	fGB	0-4A45
U+901D.0	fR/S	 162.7
U+901D.0	fTotalStrokes	 11
U+901D.0	fCangjie	YQHL
U+901D.0	fFourCorner	32302
U+901D.0	fHakka	 si5; shi5; sii5; she5
U+901D.0	fCantonese	sai6
U+901D.0	fMandarin	shi4; she2; zhi4
U+901D.0	fEnglish	[1] pass; be gone; depart [2] pass away; die
U+901E.0	fUTF8	逞
U+901E.0	fCNS11643	1-5D5E
U+901E.0	fBig5	B378
U+901E.0	fGB	0-3351
U+901E.0	fR/S	 162.7
U+901E.0	fTotalStrokes	 11
U+901E.0	fCangjie	YRHG
U+901E.0	fFourCorner	36301
U+901E.0	fHakka	  chin3; cihn3; cin2; cin3
U+901E.0	fCantonese	cing2
U+901E.0	fMandarin	cheng3; ying2; cheng1
U+901E.0	fEnglish	 indulge oneself brag; show off
U+901F.0	fUTF8	速
U+901F.0	fCNS11643	1-5D5A
U+901F.0	fBig5	B374
U+901F.0	fGB	0-4B59
U+901F.0	fR/S	 162.7
U+901F.0	fTotalStrokes	 11
U+901F.0	fCangjie	YDL
U+901F.0	fFourCorner	35309
U+901F.0	fHakka	 suk7; cuk7; ciuk7
U+901F.0	fCantonese	cuk1
U+901F.0	fMandarin	su4
U+901F.0	fEnglish	[1] quick; prompt; speedy; speed [2] invite
U+9020.0	fUTF8	造
U+9020.0	fCNS11643	1-5D5F
U+9020.0	fBig5	B379
U+9020.0	fGB	0-546C
U+9020.0	fR/S	 162.7
U+9020.0	fTotalStrokes	 11
U+9020.0	fCangjie	YHGR
U+9020.0	fFourCorner	34306
U+9020.0	fHakka	 cau3; co6; ciau5; co5; cau5; co3
U+9020.0	fCantonese	zou6; cou3; cou5; zou2 (also)
U+9020.0	fMandarin	zao4; cao1; cao4
U+9020.0	fEnglish	[1] create; make [2] manufacture; make; produce [3] make up; invent; manufacture [4] construct; build [5] arrive at; reach [6] a party concerned in the suit (law) [7] era; period [8] institute
U+9021.0	fUTF8	逡
U+9021.0	fCNS11643	2-396A
U+9021.0	fBig5	D7E7
U+9021.0	fGB	0-6552
U+9021.0	fR/S	 162.7
U+9021.0	fTotalStrokes	 11
U+9021.0	fCangjie	YICE
U+9021.0	fFourCorner	33304
U+9021.0	fHakka	  zun1
U+9021.0	fCantonese	seon1; ceon1 (also)
U+9021.0	fMandarin	qun1; jun4; xun4; suo1
U+9021.0	fEnglish	 retreat; withdraw; fall back; move backward
U+9022.0	fUTF8	逢
U+9022.0	fCNS11643	1-5D61
U+9022.0	fBig5	B37B
U+9022.0	fGB	0-376A
U+9022.0	fR/S	 162.7
U+9022.0	fTotalStrokes	 11
U+9022.0	fCangjie	YHEJ
U+9022.0	fFourCorner	37304
U+9022.0	fHakka	  fung2; fung3
U+9022.0	fCantonese	fung4; pung4
U+9022.0	fMandarin	feng2; peng2; pang2
U+9022.0	fEnglish	[1] meet; come across [2] happen; fall in with [3] talk or act in order to please (a superior); flatter [4] a Chinese family name
U+9023.0	fUTF8	連
U+9023.0	fCNS11643	1-5D59
U+9023.0	fBig5	B373
U+9023.0	fGB	1-412C
U+9023.0	fR/S	 162.7
U+9023.0	fTotalStrokes	 11
U+9023.0	fCangjie	YJWJ
U+9023.0	fFourCorner	35300
U+9023.0	fHakka	 lien2; len2; lian3
U+9023.0	fCantonese	lin4
U+9023.0	fMandarin	lian2; lian3; lan4; lian4
U+9023.0	fEnglish	[1] join; connect [2] in succession; continuous [3] company (soldiers) [4] together with [5] even; and; including [6] a Chinese family name
U+9024.0	fUTF8	逤
U+9024.0	fCNS11643	2-3965
U+9024.0	fBig5	D7E2
U+9024.0	fGB	E-403C
U+9024.0	fR/S	 162.7
U+9024.0	fTotalStrokes	 11
U+9024.0	fCangjie	YEFH
U+9024.0	fFourCorner	39302
U+9024.0	fCantonese	so3; saa1
U+9024.0	fMandarin	suo4
U+9025.0	fUTF8	逥
U+9025.0	fCNS11643	3-394C
U+9025.0	fGB	E-403D
U+9025.0	fR/S	 162.7
U+9025.0	fTotalStrokes	 11
U+9025.0	fCangjie	YWSL
U+9025.0	fFourCorner	36300
U+9025.0	fCantonese	wui4
U+9025.0	fMandarin	hui2
U+9025.0	fEnglish	 return; turn around a time
U+9026.0	fUTF8	逦
U+9026.0	fGB	0-654E
U+9026.0	fR/S	 162.7
U+9026.0	fTotalStrokes	 11
U+9026.0	fCangjie	YMBB
U+9026.0	fHakka	 li5
U+9026.0	fCantonese	lei5
U+9026.0	fMandarin	li3
U+9026.0	fEnglish	simplified form of &#x9090;
U+9027.0	fUTF8	逧
U+9027.0	fGB	E-403E
U+9027.0	fR/S	 162.7
U+9027.0	fTotalStrokes	 11
U+9027.0	fCangjie	YCOR
U+9027.0	fCantonese	guk1
U+9028.0	fUTF8	逨
U+9028.0	fCNS11643	3-3F4F
U+9028.0	fGB	5-7032
U+9028.0	fR/S	 162.8
U+9028.0	fTotalStrokes	 12
U+9028.0	fCangjie	YDOO
U+9028.0	fFourCorner	34300
U+9028.0	fCantonese	loi4
U+9028.0	fMandarin	lai2; lai4
U+9029.0	fUTF8	逩
U+9029.0	fCNS11643	3-3F50
U+9029.0	fGB	E-403F
U+9029.0	fR/S	 162.8
U+9029.0	fTotalStrokes	 12
U+9029.0	fCangjie	YKJT
U+9029.0	fFourCorner	34300
U+9029.0	fHakka	  bun5; bun1
U+9029.0	fCantonese	ban3; ban1 (also)
U+9029.0	fMandarin	ben4; ben1
U+9029.0	fEnglish	 run quickly
U+902A.0	fUTF8	逪
U+902A.0	fCNS11643	3-3F4D
U+902A.0	fGB	5-7035
U+902A.0	fR/S	 162.8
U+902A.0	fTotalStrokes	 12
U+902A.0	fCangjie	YTA
U+902A.0	fFourCorner	34300
U+902A.0	fCantonese	cok3
U+902A.0	fMandarin	cuo4
U+902B.0	fUTF8	逫
U+902B.0	fCNS11643	3-3F4C
U+902B.0	fGB	3-716A
U+902B.0	fR/S	 162.8
U+902B.0	fTotalStrokes	 12
U+902B.0	fCangjie	YEEE
U+902B.0	fFourCorner	37300
U+902B.0	fCantonese	zyut3
U+902B.0	fMandarin	jue2
U+902C.0	fUTF8	逬
U+902C.0	fCNS11643	3-3F51
U+902C.0	fGB	E-4040
U+902C.0	fR/S	 162.8
U+902C.0	fTotalStrokes	 12
U+902C.0	fCangjie	YYJJ
U+902C.0	fFourCorner	33300
U+902C.0	fCantonese	bing3; bing2
U+902C.0	fMandarin	beng4
U+902C.0	fEnglish	 scatter expel crack; split jump
U+902D.0	fUTF8	逭
U+902D.0	fCNS11643	2-4072
U+902D.0	fBig5	DC4D
U+902D.0	fGB	0-6555
U+902D.0	fR/S	 162.8
U+902D.0	fTotalStrokes	 12
U+902D.0	fCangjie	YJRR
U+902D.0	fFourCorner	33307
U+902D.0	fHakka	 fon3; gon3; gwon3
U+902D.0	fCantonese	wun6
U+902D.0	fMandarin	huan4
U+902D.0	fEnglish	 escape from; flee to avoid; avoid; evade; run away
U+902E.0	fUTF8	逮
U+902E.0	fCNS11643	1-624C
U+902E.0	fBig5	B665
U+902E.0	fGB	0-347E
U+902E.0	fR/S	 162.8
U+902E.0	fTotalStrokes	 12
U+902E.0	fCangjie	YLE
U+902E.0	fFourCorner	35303
U+902E.0	fHakka	 ti5;  tai1;  tai5; tai6
U+902E.0	fCantonese	dai6; doi6
U+902E.0	fMandarin	dai4; di4
U+902E.0	fEnglish	[1] come up to; reach; arrive [2] be after; hunt; chase and make arrest
U+902E.1	fUTF8	逮
U+902E.1	fCNS11643	1-624C
U+902E.1	fBig5	B665
U+902E.1	fGB	0-347E
U+902E.1	fR/S	 162.8
U+902E.1	fTotalStrokes	 12
U+902E.1	fCangjie	YLE
U+902E.1	fFourCorner	35303
U+902E.1	fMandarin	dai3
U+902E.1	fEnglish	 capture; catch
U+902F.0	fUTF8	逯
U+902F.0	fCNS11643	2-4074
U+902F.0	fBig5	DC4F
U+902F.0	fGB	0-6556
U+902F.0	fR/S	 162.8
U+902F.0	fTotalStrokes	 12
U+902F.0	fCangjie	YVNE
U+902F.0	fFourCorner	37303
U+902F.0	fHakka	  liuk8
U+902F.0	fCantonese	luk6
U+902F.0	fMandarin	lu4; dai4; di4
U+902F.0	fEnglish	[1] leave without reason; go away suddenly for no particular reason [2] a Chinese family name
U+9030.0	fUTF8	逰
U+9030.0	fCNS11643	3-3F55
U+9030.0	fGB	E-4041
U+9030.0	fR/S	 162.8
U+9030.0	fTotalStrokes	 12
U+9030.0	fCangjie	YQOD
U+9030.0	fFourCorner	38300
U+9030.0	fCantonese	jau4
U+9030.0	fMandarin	you2
U+9030.0	fEnglish	wander; roam; travel
U+9031.0	fUTF8	週
U+9031.0	fCNS11643	1-624E
U+9031.0	fBig5	B667
U+9031.0	fGB	E-4042
U+9031.0	fR/S	 162.8
U+9031.0	fTotalStrokes	 12
U+9031.0	fCangjie	YBGR
U+9031.0	fFourCorner	37302
U+9031.0	fHakka	  zhiu1; zu1; ziu1; zhu1
U+9031.0	fCantonese	zau1
U+9031.0	fMandarin	zhou1
U+9031.0	fEnglish	[1] week; period [2] turn; cycle; revolution; revolve
U+9032.0	fUTF8	進
U+9032.0	fCNS11643	1-6250
U+9032.0	fBig5	B669
U+9032.0	fGB	1-3D78
U+9032.0	fR/S	 162.8
U+9032.0	fTotalStrokes	 12
U+9032.0	fCangjie	YOG
U+9032.0	fFourCorner	30301
U+9032.0	fHakka	  zin5
U+9032.0	fCantonese	zeon3
U+9032.0	fMandarin	jin4
U+9032.0	fEnglish	[1] advance; make progress; go ahead; move forward; proceed [2] improve; improvement; progress [3] recommend; introduce [4] offer (advice, presents, etc.) [5] generation [6] income [7] rooms in a house divided by a courtyard; courtyard [8] eat; take; have [9] enter [10] a Chinese family name
U+9033.0	fUTF8	逳
U+9033.0	fCNS11643	4-3566
U+9033.0	fGB	3-7168
U+9033.0	fR/S	  162.8
U+9033.0	fTotalStrokes	 12; 11
U+9033.0	fCangjie	YYIB
U+9033.0	fFourCorner	30300
U+9033.0	fCantonese	juk1
U+9033.0	fMandarin	yu4
U+9033.0	fEnglish	to move, touch, hit
U+9034.0	fUTF8	逴
U+9034.0	fCNS11643	2-4073
U+9034.0	fBig5	DC4E
U+9034.0	fGB	3-7167
U+9034.0	fR/S	 162.8
U+9034.0	fTotalStrokes	 12
U+9034.0	fCangjie	YYAJ
U+9034.0	fFourCorner	31304
U+9034.0	fHakka	 zok7
U+9034.0	fCantonese	coek3
U+9034.0	fMandarin	chuo4; zhuo2
U+9034.0	fEnglish	 argue; quarrel; squabble far
U+9035.0	fUTF8	逵
U+9035.0	fCNS11643	1-624D
U+9035.0	fBig5	B666
U+9035.0	fGB	0-6553
U+9035.0	fR/S	 162.8
U+9035.0	fTotalStrokes	 12
U+9035.0	fCangjie	YGCG
U+9035.0	fFourCorner	34301
U+9035.0	fHakka	  kui2; kwi2; k'wui2
U+9035.0	fCantonese	kwai4
U+9035.0	fMandarin	kui2
U+9035.0	fEnglish	 thoroughfare; crossroads; place where many ways meet
U+9036.0	fUTF8	逶
U+9036.0	fCNS11643	1-6251
U+9036.0	fBig5	B66A
U+9036.0	fGB	0-6554
U+9036.0	fR/S	 162.8
U+9036.0	fTotalStrokes	 12
U+9036.0	fCangjie	YHDV
U+9036.0	fFourCorner	32304
U+9036.0	fHakka	 wui1; wi1; vui1
U+9036.0	fCantonese	wai1; wai2 (also)
U+9036.0	fMandarin	wei1
U+9036.0	fEnglish	winding; curving; tortuous (way, road)
U+9037.0	fUTF8	逷
U+9037.0	fCNS11643	3-3F56
U+9037.0	fGB	5-7039
U+9037.0	fR/S	 162.8
U+9037.0	fTotalStrokes	 12
U+9037.0	fCangjie	YAPH
U+9037.0	fFourCorner	36300
U+9037.0	fHakka	  tit8
U+9037.0	fCantonese	tik1
U+9037.0	fMandarin	ti4
U+9037.0	fEnglish	distant; far away
U+9038.0	fUTF8	逸
U+9038.0	fCNS11643	1-624F
U+9038.0	fBig5	B668
U+9038.0	fGB	0-525D
U+9038.0	fR/S	 162.8
U+9038.0	fTotalStrokes	 12
U+9038.0	fCangjie	YNAI
U+9038.0	fFourCorner	37301
U+9038.0	fHakka	  jit8; rit8
U+9038.0	fCantonese	jat6; jat2 (also)
U+9038.0	fMandarin	yi4
U+9038.0	fEnglish	[1] flee; escape; break loose; run away [2] go beyond; exceed; go to excess [3] rusticate; live in retirement [4] ease; idleness; leisure [5] let loose; let go [6] fault; error; mistake [7] quick; rapid [8] lost [9] superior; outstanding
U+9039.0	fUTF8	逹
U+9039.0	fGB	E-4043
U+9039.0	fR/S	 162.8
U+9039.0	fTotalStrokes	 12
U+9039.0	fCangjie	YGTJ
U+9039.0	fHakka	 tat8
U+9039.0	fCantonese	daat6
U+9039.0	fMandarin	da2; da4
U+9039.0	fEnglish	 arrive at; reach intelligent
U+903A.0	fUTF8	逺
U+903A.0	fCNS11643	3-3F54
U+903A.0	fGB	E-4044
U+903A.0	fR/S	 162.8
U+903A.0	fTotalStrokes	 12
U+903A.0	fCangjie	YGVV
U+903A.0	fFourCorner	34300
U+903A.0	fCantonese	jyun5; jyun6
U+903A.0	fMandarin	yuan3; yuan4
U+903A.0	fEnglish	 distant; remote; far profound
U+903B.0	fUTF8	逻
U+903B.0	fCNS11643	3-3F57
U+903B.0	fGB	0-425F
U+903B.0	fR/S	 162.8
U+903B.0	fTotalStrokes	 12
U+903B.0	fCangjie	YWLN
U+903B.0	fFourCorner	36300
U+903B.0	fHakka	 lo3; lau3
U+903B.0	fCantonese	lo4; lo6
U+903B.0	fMandarin	luo2; luo4; luo3
U+903B.0	fEnglish	simplified form of &#x908F;
U+903C.0	fUTF8	逼
U+903C.0	fCNS11643	1-672F
U+903C.0	fBig5	B947
U+903C.0	fGB	0-3146
U+903C.0	fR/S	 162.9
U+903C.0	fTotalStrokes	 13
U+903C.0	fCangjie	YMRW
U+903C.0	fFourCorner	31306
U+903C.0	fHakka	  bet7; bit7; biet7
U+903C.0	fCantonese	bik1
U+903C.0	fMandarin	bi1; bi4
U+903C.0	fEnglish	[1] press; compel; pressure; force; coerce [2] close in; draw near; press up to [3] importune; harass; annoy [4] narrow; straight
U+903D.0	fUTF8	逽
U+903D.0	fCNS11643	2-4829
U+903D.0	fBig5	E0A3
U+903D.0	fGB	3-7166
U+903D.0	fR/S	 162.9
U+903D.0	fTotalStrokes	 13
U+903D.0	fCangjie	YTKR
U+903D.0	fFourCorner	34306
U+903D.0	fCantonese	noek6; loek6 (also); nok6
U+903D.0	fMandarin	nuo4
U+903E.0	fUTF8	逾
U+903E.0	fCNS11643	1-6737
U+903E.0	fBig5	B94F
U+903E.0	fGB	0-5362
U+903E.0	fR/S	 162.9
U+903E.0	fTotalStrokes	 13
U+903E.0	fCangjie	YOMN
U+903E.0	fFourCorner	38302
U+903E.0	fHakka	  ji2; ji3; ri2; ri5; ji5; i2
U+903E.0	fCantonese	jyu4; jyu6
U+903E.0	fMandarin	yu2; dou4
U+903E.0	fEnglish	[1] go over; pass over; exceed; over; more than [2] transgress [3] added; even more
U+903F.0	fUTF8	逿
U+903F.0	fCNS11643	2-4826
U+903F.0	fBig5	E07E
U+903F.0	fGB	3-7156
U+903F.0	fR/S	 162.9
U+903F.0	fTotalStrokes	 13
U+903F.0	fCangjie	YAMH
U+903F.0	fFourCorner	36302
U+903F.0	fHakka	  tong5
U+903F.0	fCantonese	dong6; tong4; tik1
U+903F.0	fMandarin	dang4; tang2
U+903F.0	fEnglish	[1] pass by [2] miss
U+9040.0	fUTF8	遀
U+9040.0	fCNS11643	3-447D
U+9040.0	fGB	E-4045
U+9040.0	fR/S	  162.9
U+9040.0	fTotalStrokes	 13
U+9040.0	fCangjie	YKMB
U+9040.0	fFourCorner	34300
U+9040.0	fHakka	 sui2
U+9040.0	fCantonese	ceoi4
U+9040.0	fMandarin	sui2
U+9041.0	fUTF8	遁
U+9041.0	fCNS11643	1-6738
U+9041.0	fBig5	B950
U+9041.0	fGB	0-365D
U+9041.0	fR/S	 162.9
U+9041.0	fTotalStrokes	 13
U+9041.0	fCangjie	YHJU
U+9041.0	fFourCorner	32306
U+9041.0	fHakka	  tun3; dun3; tun6; tun5
U+9041.0	fCantonese	deon6
U+9041.0	fMandarin	dun4; qun1; xun2
U+9041.0	fEnglish	[1] hide; conceal oneself; retire [2] escape; run away
U+9042.0	fUTF8	遂
U+9042.0	fCNS11643	1-672D
U+9042.0	fBig5	B945
U+9042.0	fGB	0-4B6C
U+9042.0	fR/S	 162.9
U+9042.0	fTotalStrokes	 13
U+9042.0	fCangjie	YTPO
U+9042.0	fFourCorner	38303
U+9042.0	fHakka	  sui5; sui3; sui6
U+9042.0	fCantonese	seoi6
U+9042.0	fMandarin	sui4; sui2
U+9042.0	fEnglish	[1] have one's will; satisfy; fulfull [2] successful; succeed [3] proceed; reach [4] then; consequently; thereupon [5] flee; escape
U+9043.0	fUTF8	遃
U+9043.0	fCNS11643	3-4479
U+9043.0	fGB	3-716E
U+9043.0	fR/S	 162.9
U+9043.0	fTotalStrokes	 13
U+9043.0	fCangjie	YYHH
U+9043.0	fFourCorner	30302
U+9043.0	fCantonese	jin5
U+9043.0	fMandarin	yan3
U+9044.0	fUTF8	遄
U+9044.0	fCNS11643	2-4827
U+9044.0	fBig5	E0A1
U+9044.0	fGB	0-6557
U+9044.0	fR/S	 162.9
U+9044.0	fTotalStrokes	 13
U+9044.0	fCangjie	YUMB
U+9044.0	fFourCorner	32302
U+9044.0	fHakka	 zhon1; zon1
U+9044.0	fCantonese	cyun4; cyun2; syun4
U+9044.0	fMandarin	chuan2
U+9044.0	fEnglish	[1] swiftly; quickly; hurry [2] go to and fro; coming and going frequently and swiftly
U+9045.0	fUTF8	遅
U+9045.0	fCNS11643	3-447C
U+9045.0	fGB	E-4046
U+9045.0	fR/S	 162.9
U+9045.0	fTotalStrokes	 13
U+9045.0	fCangjie	YSTQ
U+9045.0	fFourCorner	37300
U+9045.0	fCantonese	ci4
U+9045.0	fMandarin	chi2; zhi4
U+9045.0	fEnglish	 late; tardy slow delay
U+9046.0	fUTF8	遆
U+9046.0	fCNS11643	3-447A
U+9046.0	fGB	3-716F
U+9046.0	fR/S	 162.9
U+9046.0	fTotalStrokes	 13
U+9046.0	fCangjie	YYBB
U+9046.0	fFourCorner	30300
U+9046.0	fCantonese	tai4
U+9046.0	fMandarin	ti2
U+9047.0	fUTF8	遇
U+9047.0	fCNS11643	1-6732
U+9047.0	fBig5	B94A
U+9047.0	fGB	0-5376
U+9047.0	fR/S	 162.9
U+9047.0	fTotalStrokes	 13
U+9047.0	fCangjie	YWLB
U+9047.0	fFourCorner	36302
U+9047.0	fHakka	  ngi5; gni5; ngi6; gni6; ngi3; gni3
U+9047.0	fCantonese	jyu6
U+9047.0	fMandarin	yu4; ou3
U+9047.0	fEnglish	[1] meet; come across; encounter; run into [2] treat; treatment [3] opportunity; luck [4] meeting of minds; win confidence (of a superior, king, etc.) [5] rival; match with [6] a Chinese family name
U+9048.0	fUTF8	遈
U+9048.0	fCNS11643	3-447B
U+9048.0	fGB	E-4047
U+9048.0	fR/S	  162.9
U+9048.0	fTotalStrokes	 13
U+9048.0	fCangjie	YAMO
U+9048.0	fFourCorner	36300
U+9048.0	fCantonese	zik6
U+9048.0	fMandarin	shi2
U+9049.0	fUTF8	遉
U+9049.0	fCNS11643	2-4828
U+9049.0	fBig5	E0A2
U+9049.0	fGB	E-4048
U+9049.0	fR/S	 162.9
U+9049.0	fTotalStrokes	 13
U+9049.0	fCangjie	YYBC
U+9049.0	fFourCorner	31308
U+9049.0	fCantonese	zing1
U+9049.0	fMandarin	zhen1; zheng1; cheng4
U+9049.0	fEnglish	 spy; reconnoiter detective
U+904A.0	fUTF8	遊
U+904A.0	fCNS11643	1-672B
U+904A.0	fBig5	B943
U+904A.0	fGB	E-4049
U+904A.0	fR/S	 162.9
U+904A.0	fTotalStrokes	 13
U+904A.0	fCangjie	YYSD
U+904A.0	fFourCorner	38304
U+904A.0	fHakka	  ju2; jeu2; riu2; jiu3; jiu2
U+904A.0	fCantonese	jau4; jau6
U+904A.0	fMandarin	you2
U+904A.0	fEnglish	[1] wander; roam; saunter [2] travel; go to a distance [3] befriend; make friends [4] freely wield (a sword), move (one;s eyes), stretch (one's sight), etc.
U+904B.0	fUTF8	運
U+904B.0	fCNS11643	1-672A
U+904B.0	fBig5	B942
U+904B.0	fGB	1-544B
U+904B.0	fR/S	 162.9
U+904B.0	fTotalStrokes	 13
U+904B.0	fCangjie	YBJJ
U+904B.0	fFourCorner	37304
U+904B.0	fHakka	  jun5; jun3; run6; jun6
U+904B.0	fCantonese	wan6
U+904B.0	fMandarin	yun4
U+904B.0	fEnglish	[1] move; revolve [2] transport; ship [3] utilize; make use of [4] one's luck; fortune
U+904C.0	fUTF8	遌
U+904C.0	fCNS11643	3-447E
U+904C.0	fGB	3-716D
U+904C.0	fR/S	 162.9
U+904C.0	fTotalStrokes	 13
U+904C.0	fCangjie	YRRS
U+904C.0	fFourCorner	36300
U+904C.0	fCantonese	ngok6; ok6 (also)
U+904C.0	fMandarin	e4
U+904D.0	fUTF8	遍
U+904D.0	fCNS11643	1-6735
U+904D.0	fBig5	B94D
U+904D.0	fGB	0-3169
U+904D.0	fR/S	 162.9
U+904D.0	fTotalStrokes	 13
U+904D.0	fCangjie	YHSB
U+904D.0	fFourCorner	33302
U+904D.0	fHakka	  bien5; pen1; pien1; p'ian5
U+904D.0	fCantonese	pin3; bin3
U+904D.0	fMandarin	bian4; pian4
U+904D.0	fEnglish	[1] time [2] everywhere; all over; throughout
U+904E.0	fUTF8	過
U+904E.0	fCNS11643	1-6734
U+904E.0	fBig5	B94C
U+904E.0	fGB	1-397D
U+904E.0	fR/S	 162.9
U+904E.0	fTotalStrokes	 13
U+904E.0	fCangjie	YBBR
U+904E.0	fFourCorner	37302
U+904E.0	fHakka	  go5; go3; ko1; gau5; kwo5; gwo1; {gwo2}; {gwo3}; gwo5
U+904E.0	fCantonese	gwo3; gwo1; go3 (also); gwo2 (also); go2 (also); go1 (also)
U+904E.0	fMandarin	guo4; huo4; guo5
U+904E.0	fEnglish	[1] pass; pass through; pass by; ford [2] across; past; through; over [3] spend; pass (time) [4] after; past [5] go beyong thee ordinary or proper limits; surpass [6] too much; excessive [7] mistake; demerit; fault; sin [8] (wine) [9] contagious
U+904E.1	fUTF8	過
U+904E.1	fCNS11643	1-6734
U+904E.1	fBig5	B94C
U+904E.1	fGB	1-397D
U+904E.1	fR/S	 162.9
U+904E.1	fTotalStrokes	 13
U+904E.1	fCangjie	YBBR
U+904E.1	fFourCorner	37302
U+904E.1	fMandarin	guo1
U+904E.1	fEnglish	a Chinese family name
U+904E.2	fUTF8	過
U+904E.2	fCNS11643	1-6734
U+904E.2	fBig5	B94C
U+904E.2	fGB	1-397D
U+904E.2	fR/S	 162.9
U+904E.2	fTotalStrokes	 13
U+904E.2	fCangjie	YBBR
U+904E.2	fFourCorner	37302
U+904F.0	fUTF8	遏
U+904F.0	fCNS11643	1-6733
U+904F.0	fBig5	B94B
U+904F.0	fGB	0-3674
U+904F.0	fR/S	 162.9
U+904F.0	fTotalStrokes	 13
U+904F.0	fCangjie	YAPV
U+904F.0	fFourCorner	36302
U+904F.0	fHakka	 ot7
U+904F.0	fCantonese	aat3; ngaat3 (also); kit3 (also)
U+904F.0	fMandarin	e4
U+904F.0	fEnglish	[1] stop; suppress; curb; restrain; prevent [2] cause one's own extinction; extinguish; ruin
U+9050.0	fUTF8	遐
U+9050.0	fCNS11643	1-6731
U+9050.0	fBig5	B949
U+9050.0	fGB	0-655A
U+9050.0	fR/S	 162.9
U+9050.0	fTotalStrokes	 13
U+9050.0	fCangjie	YRSE
U+9050.0	fFourCorner	37304
U+9050.0	fHakka	  ha2; ha3
U+9050.0	fCantonese	haa4
U+9050.0	fMandarin	xia2
U+9050.0	fEnglish	[1] far; distant [2] a long time [3] advanced in years [4] die down; vanish [5] abandon; cast off [6] Why not?; How?; What?
U+9051.0	fUTF8	遑
U+9051.0	fCNS11643	1-6736
U+9051.0	fBig5	B94E
U+9051.0	fGB	0-6558
U+9051.0	fR/S	 162.9
U+9051.0	fTotalStrokes	 13
U+9051.0	fCangjie	YHAG
U+9051.0	fFourCorner	36301
U+9051.0	fHakka	 fong2
U+9051.0	fCantonese	wong4
U+9051.0	fMandarin	huang2
U+9051.0	fEnglish	[1] hurry; hurry [2] anxious; disturbed [3] leisurely [4] not to
U+9052.0	fUTF8	遒
U+9052.0	fCNS11643	2-4825
U+9052.0	fBig5	E07D
U+9052.0	fGB	0-6559
U+9052.0	fR/S	 162.9
U+9052.0	fTotalStrokes	 13
U+9052.0	fCangjie	YTCW
U+9052.0	fFourCorner	38306
U+9052.0	fHakka	  ciu2;  siu2; ciu1
U+9052.0	fCantonese	cau4
U+9052.0	fMandarin	qiu2; qiu1; jiu1
U+9052.0	fEnglish	[1] strong; forceful; powerful; vigorous; sturdy [2] come to an end; close [3] gather; concentrate
U+9053.0	fUTF8	道
U+9053.0	fCNS11643	1-672C
U+9053.0	fBig5	B944
U+9053.0	fGB	0-3540
U+9053.0	fR/S	 162.9
U+9053.0	fTotalStrokes	 13
U+9053.0	fCangjie	YTHU
U+9053.0	fFourCorner	38306
U+9053.0	fHakka	  tau5; to6; do5; to5; tau3
U+9053.0	fCantonese	dou6; dou3 (also); dou2 (also)
U+9053.0	fMandarin	dao4; dao3
U+9053.0	fEnglish	[1] path; road; street [2] method; way [3] the way (in the metaphysical sense) [4] Taoism; Taoist [5] say; speak [6] an administrative district in old China [7] theory; doctrine [8] govern; lead [9] think; suppose [10] skill; art; craft [11] a Chinese family name
U+9053.1	fUTF8	道
U+9053.1	fCNS11643	1-672C
U+9053.1	fBig5	B944
U+9053.1	fGB	0-3540
U+9053.1	fR/S	 162.9
U+9053.1	fTotalStrokes	 13
U+9053.1	fCangjie	YTHU
U+9053.1	fFourCorner	38306
U+9053.1	fMandarin	dao5
U+9054.0	fUTF8	達
U+9054.0	fCNS11643	1-672E
U+9054.0	fBig5	B946
U+9054.0	fGB	1-346F
U+9054.0	fR/S	 162.9
U+9054.0	fTotalStrokes	 13
U+9054.0	fCangjie	YGTQ
U+9054.0	fFourCorner	34304
U+9054.0	fHakka	  tat8
U+9054.0	fCantonese	daat6; taat3
U+9054.0	fMandarin	da2; ti4
U+9054.0	fEnglish	[1] arrive at; reach [2] intelligent; smart; understanding; reasonable [3] prominent; successful [4] inform; tell [5] open-minded [6] a Chinese familly name
U+9054.1	fUTF8	達
U+9054.1	fCNS11643	1-672E
U+9054.1	fBig5	B946
U+9054.1	fGB	1-346F
U+9054.1	fR/S	 162.9
U+9054.1	fTotalStrokes	 13
U+9054.1	fCangjie	YGTQ
U+9054.1	fFourCorner	34304
U+9054.1	fMandarin	ta4
U+9054.1	fEnglish	loose and impudent
U+9055.0	fUTF8	違
U+9055.0	fCNS11643	1-6730
U+9055.0	fBig5	B948
U+9055.0	fGB	1-4E25
U+9055.0	fR/S	 162.9
U+9055.0	fTotalStrokes	 13
U+9055.0	fCangjie	YDMQ
U+9055.0	fFourCorner	34304
U+9055.0	fHakka	 wui2; wi2; wui3; vui2
U+9055.0	fCantonese	wai4
U+9055.0	fMandarin	wei2; hui2
U+9055.0	fEnglish	[1] go against; disobey; violate; defy; disregard [2] be apart; be separated [3] avoid [4] evil; fault
U+9056.0	fUTF8	遖
U+9056.0	fGB	E-404A
U+9056.0	fR/S	 162.9
U+9056.0	fTotalStrokes	 13
U+9056.0	fCangjie	YJBJ
U+9056.0	fCantonese	naam4; laam4 (also)
U+9057.0	fUTF8	遗
U+9057.0	fGB	0-5245
U+9057.0	fR/S	 162.9
U+9057.0	fTotalStrokes	 13
U+9057.0	fCangjie	YLMO
U+9057.0	fHakka	 vui2
U+9057.0	fCantonese	wai4; wai6
U+9057.0	fMandarin	yi2; sui2; wei4; sui4
U+9057.0	fEnglish	simplified form of &#x907A;
U+9058.0	fUTF8	遘
U+9058.0	fCNS11643	1-6B40
U+9058.0	fBig5	BBB8
U+9058.0	fGB	0-655C
U+9058.0	fR/S	 162.10
U+9058.0	fTotalStrokes	 14
U+9058.0	fCangjie	YTTB
U+9058.0	fFourCorner	35305
U+9058.0	fHakka	 kieu5; giu5; geu5
U+9058.0	fCantonese	gau3; kau3
U+9058.0	fMandarin	gou4
U+9058.0	fEnglish	 to meet; come across; encounter
U+9059.0	fUTF8	遙
U+9059.0	fCNS11643	1-6B43
U+9059.0	fBig5	BBBB
U+9059.0	fGB	E-404B
U+9059.0	fR/S	 162.10
U+9059.0	fTotalStrokes	 14
U+9059.0	fCangjie	YBOU
U+9059.0	fFourCorner	37307
U+9059.0	fHakka	  jau2; rau2; jau3; jeu2
U+9059.0	fCantonese	jiu4
U+9059.0	fMandarin	yao2
U+9059.0	fEnglish	far; distant; remote
U+905A.0	fUTF8	遚
U+905A.0	fCNS11643	3-4A24
U+905A.0	fGB	E-404C
U+905A.0	fR/S	 162.10
U+905A.0	fTotalStrokes	 14
U+905A.0	fCangjie	YHXE
U+905A.0	fFourCorner	37300
U+905A.0	fCantonese	cau3
U+905A.0	fMandarin	chu4
U+905B.0	fUTF8	遛
U+905B.0	fCNS11643	1-6B47
U+905B.0	fBig5	BBBF
U+905B.0	fGB	0-655E
U+905B.0	fR/S	 162.10
U+905B.0	fTotalStrokes	 14
U+905B.0	fCangjie	YHHW
U+905B.0	fFourCorner	37306
U+905B.0	fHakka	 liu2; liu5
U+905B.0	fCantonese	lau6; lau4; liu4 (also)
U+905B.0	fMandarin	liu2
U+905B.0	fEnglish	 hang around; linger at place
U+905B.1	fUTF8	遛
U+905B.1	fCNS11643	1-6B47
U+905B.1	fBig5	BBBF
U+905B.1	fGB	0-655E
U+905B.1	fR/S	 162.10
U+905B.1	fTotalStrokes	 14
U+905B.1	fCangjie	YHHW
U+905B.1	fFourCorner	37306
U+905B.1	fMandarin	liu4
U+905B.1	fEnglish	 take a walk; stroll; walk slowly; roam
U+905C.0	fUTF8	遜
U+905C.0	fCNS11643	1-6B41
U+905C.0	fBig5	BBB9
U+905C.0	fGB	1-5137
U+905C.0	fR/S	 162.10
U+905C.0	fTotalStrokes	 14
U+905C.0	fCangjie	YNDF
U+905C.0	fFourCorner	32309
U+905C.0	fHakka	  sun5; sun3
U+905C.0	fCantonese	seon3
U+905C.0	fMandarin	xun4; sun4
U+905C.0	fEnglish	[1] respectful and compliant; obedient [2] resign; resigning; surrender; abdicate; yielding [3] humble; modest [4] not as good as; inferior to [5] a Chinese family name
U+905D.0	fUTF8	遝
U+905D.0	fCNS11643	1-6B46
U+905D.0	fBig5	BBBE
U+905D.0	fGB	3-7172
U+905D.0	fR/S	 162.10
U+905D.0	fTotalStrokes	 14
U+905D.0	fCangjie	YWLE
U+905D.0	fFourCorner	37303
U+905D.0	fHakka	  tap7; tap8
U+905D.0	fCantonese	daap6
U+905D.0	fMandarin	ta4; dai4
U+905D.0	fEnglish	numerous; many; a large number of
U+905E.0	fUTF8	遞
U+905E.0	fCNS11643	1-6B44
U+905E.0	fBig5	BBBC
U+905E.0	fGB	1-355D
U+905E.0	fR/S	 162.10
U+905E.0	fTotalStrokes	 14
U+905E.0	fCangjie	YHYU
U+905E.0	fFourCorner	32301
U+905E.0	fHakka	 ti5; tai1
U+905E.0	fCantonese	dai6; daai3; sai6
U+905E.0	fMandarin	di4; shi4; dai4
U+905E.0	fEnglish	[1] forward; transmit; hand over; deliver [2] substitute; alternate
U+905F.0	fUTF8	遟
U+905F.0	fCNS11643	3-4A27
U+905F.0	fGB	5-703F
U+905F.0	fR/S	 162.10
U+905F.0	fTotalStrokes	 14
U+905F.0	fCangjie	YSYJ
U+905F.0	fFourCorner	37300
U+905F.0	fCantonese	ci4; zi3
U+905F.0	fMandarin	chi2; zhi4; xi1
U+905F.0	fEnglish	 late; tardy slow delay
U+9060.0	fUTF8	遠
U+9060.0	fCNS11643	1-6B3F
U+9060.0	fBig5	BBB7
U+9060.0	fGB	1-5436
U+9060.0	fR/S	 162.10
U+9060.0	fTotalStrokes	 14
U+9060.0	fCangjie	YGRV
U+9060.0	fFourCorner	34303
U+9060.0	fHakka	 jen3;  jan3; ren3; jan5
U+9060.0	fCantonese	jyun5; jyun6; jyun2 (also)
U+9060.0	fMandarin	yuan3; yung4
U+9060.0	fEnglish	[1] distant; remote; far [2] deep; profound [3] keep at a distance; keep away
U+9060.1	fUTF8	遠
U+9060.1	fCNS11643	1-6B3F
U+9060.1	fBig5	BBB7
U+9060.1	fGB	1-5436
U+9060.1	fR/S	 162.10
U+9060.1	fTotalStrokes	 14
U+9060.1	fCangjie	YGRV
U+9060.1	fFourCorner	34303
U+9060.1	fMandarin	yuan4
U+9060.1	fEnglish	 keep at a distance; keep away; shun; avoid
U+9061.0	fUTF8	遡
U+9061.0	fCNS11643	3-4A26
U+9061.0	fGB	E-404D
U+9061.0	fR/S	 162.10
U+9061.0	fTotalStrokes	 14
U+9061.0	fCangjie	YTUB
U+9061.0	fFourCorner	37300
U+9061.0	fHakka	 sok7; su5; sii5; shok7
U+9061.0	fCantonese	sou3
U+9061.0	fMandarin	su4
U+9061.0	fEnglish	[1] trace back to a source [2] go against a stream
U+9062.0	fUTF8	遢
U+9062.0	fCNS11643	1-6B45
U+9062.0	fBig5	BBBD
U+9062.0	fGB	0-655D
U+9062.0	fR/S	 162.10
U+9062.0	fTotalStrokes	 14
U+9062.0	fCangjie	YASM
U+9062.0	fFourCorner	36302
U+9062.0	fHakka	 tap7
U+9062.0	fCantonese	taap3; taat3 (also); tat3
U+9062.0	fMandarin	ta4; ta1
U+9062.0	fEnglish	untidy; careless; negligent; slipshod
U+9063.0	fUTF8	遣
U+9063.0	fCNS11643	1-6B42
U+9063.0	fBig5	BBBA
U+9063.0	fGB	0-4732
U+9063.0	fR/S	 162.10
U+9063.0	fTotalStrokes	 14
U+9063.0	fCangjie	YLMR
U+9063.0	fFourCorner	35307
U+9063.0	fHakka	  kien3;  ken3; kian3
U+9063.0	fCantonese	hin2
U+9063.0	fMandarin	qian3; qian4
U+9063.0	fEnglish	[1] send; dispatch; send off [2] exile; banish; drive away [3] kill (time); forget (one's sorrow); divert [4] release
U+9064.0	fUTF8	遤
U+9064.0	fCNS11643	F-4C6B
U+9064.0	fGB	1-7D6F
U+9064.0	fR/S	 162.10
U+9064.0	fTotalStrokes	 14
U+9064.0	fCangjie	YSQF
U+9064.0	fFourCorner	3130
U+9064.0	fCantonese	maa5
U+9065.0	fUTF8	遥
U+9065.0	fGB	0-5223
U+9065.0	fR/S	 162.9; 162.10
U+9065.0	fTotalStrokes	 14
U+9065.0	fCangjie	YBOU
U+9065.0	fCantonese	jiu4
U+9065.0	fMandarin	yao2
U+9065.0	fEnglish	far away; distant; remote
U+9066.0	fUTF8	遦
U+9066.0	fCNS11643	4-4E32
U+9066.0	fGB	5-703C
U+9066.0	fR/S	 162.11
U+9066.0	fTotalStrokes	 15
U+9066.0	fCangjie	YWJC
U+9066.0	fFourCorner	37300
U+9066.0	fCantonese	gwaan3
U+9066.0	fMandarin	guan4
U+9067.0	fUTF8	遧
U+9067.0	fCNS11643	2-547A
U+9067.0	fBig5	E852
U+9067.0	fGB	5-7044
U+9067.0	fR/S	 162.11
U+9067.0	fTotalStrokes	 15
U+9067.0	fCangjie	YYTJ
U+9067.0	fFourCorner	30304
U+9067.0	fCantonese	zoeng1
U+9067.0	fMandarin	zhang1
U+9068.0	fUTF8	遨
U+9068.0	fCNS11643	1-6F4D
U+9068.0	fBig5	BE43
U+9068.0	fGB	0-655B
U+9068.0	fR/S	 162.11
U+9068.0	fTotalStrokes	 15
U+9068.0	fCangjie	YGSK
U+9068.0	fFourCorner	38304
U+9068.0	fHakka	 ngau2; ngau3; lau1
U+9068.0	fCantonese	ngou4; ngou6 (also); ou4 (also); ou6 (also)
U+9068.0	fMandarin	ao2
U+9068.0	fEnglish	[1] ramble; travel for pleasure [2] divert oneself
U+9069.0	fUTF8	適
U+9069.0	fCNS11643	1-6F4B
U+9069.0	fBig5	BE41
U+9069.0	fGB	1-4A4A
U+9069.0	fR/S	 162.11
U+9069.0	fTotalStrokes	 15
U+9069.0	fCangjie	YYCB
U+9069.0	fFourCorner	30302
U+9069.0	fHakka	 dit7; sit7;  shit7; dui5; siit7; zak7; tit8
U+9069.0	fCantonese	dik1; zaak6 (also); tik1; zaak3; kut3; sik1; kwut3 (also)
U+9069.0	fMandarin	shi4; ti4; zhe2; kuo4; di4; zhi1
U+9069.0	fEnglish	[1] go; arrive at; reach [2] just right; exactly; appropiate; fit; just [3] comfortable; at ease with oneself [4] marry (said of a girl) [5] follow; be faithful to [6] only [7] by chance; accidentilly [8] just now
U+9069.1	fUTF8	適
U+9069.1	fCNS11643	1-6F4B
U+9069.1	fBig5	BE41
U+9069.1	fGB	1-4A4A
U+9069.1	fR/S	 162.11
U+9069.1	fTotalStrokes	 15
U+9069.1	fCangjie	YYCB
U+9069.1	fFourCorner	30302
U+9069.1	fMandarin	di2
U+9069.1	fEnglish	legal (wife, as opposed to concubines, etc.); sons born of one's legal wife
U+906A.0	fUTF8	遪
U+906A.0	fCNS11643	3-4F3C
U+906A.0	fGB	3-7169
U+906A.0	fR/S	 162.11
U+906A.0	fTotalStrokes	 15
U+906A.0	fCangjie	YIIH
U+906A.0	fFourCorner	33300
U+906A.0	fCantonese	caap3
U+906A.0	fMandarin	ce4
U+906B.0	fUTF8	遫
U+906B.0	fCNS11643	2-547B
U+906B.0	fBig5	E853
U+906B.0	fGB	3-7173
U+906B.0	fR/S	 162.11
U+906B.0	fTotalStrokes	 15
U+906B.0	fCangjie	YDLK
U+906B.0	fFourCorner	38304
U+906B.0	fCantonese	cik1; cuk1
U+906B.0	fMandarin	chi4; su4
U+906B.0	fEnglish	speed
U+906C.0	fUTF8	遬
U+906C.0	fCNS11643	4-4E33
U+906C.0	fGB	3-7174
U+906C.0	fR/S	 162.11
U+906C.0	fTotalStrokes	 15
U+906C.0	fCangjie	YDLO
U+906C.0	fFourCorner	37300
U+906C.0	fHakka	 suk7
U+906C.0	fCantonese	cuk1
U+906C.0	fMandarin	su4
U+906C.0	fEnglish	active; nimble
U+906D.0	fUTF8	遭
U+906D.0	fCNS11643	1-6F4E
U+906D.0	fBig5	BE44
U+906D.0	fGB	0-5462
U+906D.0	fR/S	 162.11
U+906D.0	fTotalStrokes	 15
U+906D.0	fCangjie	YTWA
U+906D.0	fFourCorner	35306
U+906D.0	fHakka	  zau1; zo1
U+906D.0	fCantonese	zou1
U+906D.0	fMandarin	zao1
U+906D.0	fEnglish	[1] come across; meet with; encounter; icur; be victimized; suffer [2] times of binding or turning around, as with a rope [3] time; turn
U+906E.0	fUTF8	遮
U+906E.0	fCNS11643	1-6F4C
U+906E.0	fBig5	BE42
U+906E.0	fGB	0-555A
U+906E.0	fR/S	 162.11
U+906E.0	fTotalStrokes	 15
U+906E.0	fCangjie	YITF
U+906E.0	fFourCorner	30303
U+906E.0	fHakka	  za1; zha1
U+906E.0	fCantonese	ze1
U+906E.0	fMandarin	zhe1; zhe5; zhe3; zhe4
U+906E.0	fEnglish	[1] cover; shield; protect; shade; screen; shut out; conceal [2] intercept; block
U+906F.0	fUTF8	遯
U+906F.0	fCNS11643	2-5479
U+906F.0	fBig5	E851
U+906F.0	fGB	E-404E
U+906F.0	fR/S	 162.11
U+906F.0	fTotalStrokes	 15
U+906F.0	fCangjie	YBMO
U+906F.0	fFourCorner	31303
U+906F.0	fHakka	 tun3
U+906F.0	fCantonese	deon6
U+906F.0	fMandarin	dun4; tun2
U+906F.0	fEnglish	[1] deceive [2] escape; flee; run off
U+9070.0	fUTF8	遰
U+9070.0	fCNS11643	2-5478
U+9070.0	fBig5	E850
U+9070.0	fGB	3-716C
U+9070.0	fR/S	 162.11
U+9070.0	fTotalStrokes	 15
U+9070.0	fCangjie	YKPB
U+9070.0	fFourCorner	34302
U+9070.0	fHakka	  ti5; {che5}; {ce5}
U+9070.0	fCantonese	dai6; sai6; dai3
U+9070.0	fMandarin	di4; zhi4; dai4; shi4
U+9070.0	fEnglish	[1] grunting sound of pigs; clash [2] towards; to; in the direction [3] infect
U+9071.0	fUTF8	遱
U+9071.0	fCNS11643	3-4F3B
U+9071.0	fGB	3-7170
U+9071.0	fR/S	 162.11
U+9071.0	fTotalStrokes	 15
U+9071.0	fCangjie	YLLV
U+9071.0	fFourCorner	35304
U+9071.0	fHakka	 leu2
U+9071.0	fCantonese	lau4
U+9071.0	fMandarin	lou2
U+9071.0	fEnglish	continuous
U+9072.0	fUTF8	遲
U+9072.0	fCNS11643	1-725A
U+9072.0	fBig5	BFF0
U+9072.0	fGB	1-3359
U+9072.0	fR/S	 162.12
U+9072.0	fTotalStrokes	 16
U+9072.0	fCangjie	YSEQ
U+9072.0	fFourCorner	37304
U+9072.0	fHakka	 ci2;  chi2; cih2; chi3; chii2
U+9072.0	fCantonese	ci4; zi3; zi6
U+9072.0	fMandarin	chi2; xi1; zhi2
U+9072.0	fEnglish	[1] late [2] tardy; slow; dilatory [3] delay [4] dull; stupid (person) [5] a Chinese family name
U+9072.1	fUTF8	遲
U+9072.1	fCNS11643	1-725A
U+9072.1	fBig5	BFF0
U+9072.1	fGB	1-3359
U+9072.1	fR/S	 162.12
U+9072.1	fTotalStrokes	 16
U+9072.1	fCangjie	YSEQ
U+9072.1	fFourCorner	37304
U+9072.1	fMandarin	zhi4
U+9072.1	fEnglish	 wait
U+9073.0	fUTF8	遳
U+9073.0	fCNS11643	2-5477
U+9073.0	fBig5	E84F
U+9073.0	fGB	3-7171
U+9073.0	fR/S	 162.11
U+9073.0	fTotalStrokes	 15
U+9073.0	fCangjie	YTOG
U+9073.0	fFourCorner	34301
U+9073.0	fCantonese	co1
U+9073.0	fMandarin	cuo1
U+9074.0	fUTF8	遴
U+9074.0	fCNS11643	1-7258
U+9074.0	fBig5	BFEE
U+9074.0	fGB	0-6560
U+9074.0	fR/S	 162.12; 162.13
U+9074.0	fTotalStrokes	 16
U+9074.0	fCangjie	YFDQ
U+9074.0	fFourCorner	39305
U+9074.0	fHakka	 lin2; lin6; lin5
U+9074.0	fCantonese	leon4; leon6
U+9074.0	fMandarin	lin2; lin4
U+9074.0	fEnglish	 select carefully; choose carefully
U+9075.0	fUTF8	遵
U+9075.0	fCNS11643	1-7257
U+9075.0	fBig5	BFED
U+9075.0	fGB	0-5771
U+9075.0	fR/S	 162.12
U+9075.0	fTotalStrokes	 16
U+9075.0	fCangjie	YTWI
U+9075.0	fFourCorner	38304
U+9075.0	fHakka	  zun1
U+9075.0	fCantonese	zeon1; zyun1 (also)
U+9075.0	fMandarin	zun1
U+9075.0	fEnglish	[1] follow [2] observe (rules, regulations, etc.); abide by (laws, etc.)
U+9076.0	fUTF8	遶
U+9076.0	fCNS11643	2-5A7C
U+9076.0	fBig5	EBD0
U+9076.0	fGB	E-404F
U+9076.0	fR/S	 162.12
U+9076.0	fTotalStrokes	 16
U+9076.0	fCangjie	YGGU
U+9076.0	fFourCorner	34301
U+9076.0	fHakka	 ngiau3; ngiau2; ngieu3
U+9076.0	fCantonese	jiu5 (also); jiu2
U+9076.0	fMandarin	rao4; rao3; shao4
U+9076.0	fEnglish	 entwine; wind around; surround
U+9077.0	fUTF8	遷
U+9077.0	fCNS11643	1-6F4F
U+9077.0	fBig5	BE45
U+9077.0	fGB	1-4728
U+9077.0	fR/S	 162.12
U+9077.0	fTotalStrokes	 16
U+9077.0	fCangjie	YMWU
U+9077.0	fFourCorner	31301
U+9077.0	fHakka	  cien1; cen1
U+9077.0	fCantonese	cin1
U+9077.0	fMandarin	qian1
U+9077.0	fEnglish	[1] move; remove [2] change [3] get transfered (officials, etc.)
U+9078.0	fUTF8	選
U+9078.0	fCNS11643	1-7259
U+9078.0	fBig5	BFEF
U+9078.0	fGB	1-5121
U+9078.0	fR/S	 162.12
U+9078.0	fTotalStrokes	 16
U+9078.0	fCangjie	YRUC
U+9078.0	fFourCorner	37308
U+9078.0	fHakka	 sien3;  sen3; sian3
U+9078.0	fCantonese	syun2; syun3
U+9078.0	fMandarin	xuan3; suan4; xuan4; shua1; zhuan4
U+9078.0	fEnglish	[1] choose; select; choice [2] elect; election
U+9079.0	fUTF8	遹
U+9079.0	fCNS11643	2-5A7D
U+9079.0	fBig5	EBD1
U+9079.0	fGB	3-7178
U+9079.0	fR/S	 162.12
U+9079.0	fTotalStrokes	 16
U+9079.0	fCangjie	YNHB
U+9079.0	fFourCorner	37302
U+9079.0	fHakka	 jut8; lut8; rut8
U+9079.0	fCantonese	wat6; leot6; jyut6
U+9079.0	fMandarin	yu4; shu4
U+9079.0	fEnglish	[1] follow; comply with; obey [2] shun; avoid [3] be perverse; be bad [4] a Chinese family name
U+907A.0	fUTF8	遺
U+907A.0	fCNS11643	1-725C
U+907A.0	fBig5	BFF2
U+907A.0	fGB	1-5245
U+907A.0	fR/S	 162.12
U+907A.0	fTotalStrokes	 16
U+907A.0	fCangjie	YLMC
U+907A.0	fFourCorner	35308
U+907A.0	fHakka	 wui2; wi2; wui3; ri3; ji3; vui2
U+907A.0	fCantonese	wai4; wai6
U+907A.0	fMandarin	yi2; sui2; sui4
U+907A.0	fEnglish	[1] lose; lost [2] things lost; articles lost [3] miss; an omission due to negligence [4] forget [5] leave over [6] remnants; leftovers [7] abandon; desert [8] anything left behind by the deceased; bequeath; hand down; legacy [9] urinate [10] short for nocturnal emission
U+907A.1	fUTF8	遺
U+907A.1	fCNS11643	1-725C
U+907A.1	fBig5	BFF2
U+907A.1	fGB	1-5245
U+907A.1	fR/S	 162.12
U+907A.1	fTotalStrokes	 16
U+907A.1	fCangjie	YLMC
U+907A.1	fFourCorner	35308
U+907A.1	fMandarin	wei4
U+907A.1	fEnglish	[1] send a gift; present a gift [2] be left to; be laid on
U+907B.0	fUTF8	遻
U+907B.0	fCNS11643	2-5A7E
U+907B.0	fBig5	EBD2
U+907B.0	fGB	5-7047
U+907B.0	fR/S	 162.12
U+907B.0	fTotalStrokes	 16
U+907B.0	fCangjie	YRRU
U+907B.0	fFourCorner	36304
U+907B.0	fHakka	  ngok8;  ngu5; ngu6
U+907B.0	fCantonese	ng6 (also); ngok6; ok6 (also)
U+907B.0	fMandarin	wu4; e4
U+907B.0	fEnglish	meet
U+907C.0	fUTF8	遼
U+907C.0	fCNS11643	1-725B
U+907C.0	fBig5	BFF1
U+907C.0	fGB	1-4149
U+907C.0	fR/S	 162.12
U+907C.0	fTotalStrokes	 16
U+907C.0	fCangjie	YKCF
U+907C.0	fFourCorner	34309
U+907C.0	fHakka	  liau2; liau3
U+907C.0	fCantonese	liu4
U+907C.0	fMandarin	liao2
U+907C.0	fEnglish	[1] distant; far [2] the Liao river in Manchuria [3] the Liao dynasty (916-1125) founded by the Kitan Tartars in the greater part of northern China
U+907D.0	fUTF8	遽
U+907D.0	fCNS11643	1-7563
U+907D.0	fBig5	C1D8
U+907D.0	fGB	0-6561
U+907D.0	fR/S	 162.13
U+907D.0	fTotalStrokes	 17
U+907D.0	fCangjie	YYPO
U+907D.0	fFourCorner	31303
U+907D.0	fHakka	  gi3;  giak7; ki3
U+907D.0	fCantonese	geoi6; geoi3
U+907D.0	fMandarin	ju4; qu2
U+907D.0	fEnglish	[1] suddenly; unexpectedly; abruptly; hastily; hurriedly [2] scared; frightened; agitated; agitation [3] stagecoach
U+907E.0	fUTF8	遾
U+907E.0	fCNS11643	2-5F6F
U+907E.0	fBig5	EEC3
U+907E.0	fGB	3-7179
U+907E.0	fR/S	 162.13
U+907E.0	fTotalStrokes	 17
U+907E.0	fCangjie	YHMO
U+907E.0	fFourCorner	38301
U+907E.0	fCantonese	sai6
U+907E.0	fMandarin	shi4
U+907F.0	fUTF8	避
U+907F.0	fCNS11643	1-7562
U+907F.0	fBig5	C1D7
U+907F.0	fGB	0-315C
U+907F.0	fR/S	 162.13
U+907F.0	fTotalStrokes	 17
U+907F.0	fCangjie	YSRJ
U+907F.0	fFourCorner	30304
U+907F.0	fHakka	  pit7; pi5
U+907F.0	fCantonese	bei6
U+907F.0	fMandarin	bi4; bei4
U+907F.0	fEnglish	[1] avoid; shun; evade [2] prevent; keep away; repel
U+9080.0	fUTF8	邀
U+9080.0	fCNS11643	1-7567
U+9080.0	fBig5	C1DC
U+9080.0	fGB	0-517B
U+9080.0	fR/S	 162.13
U+9080.0	fTotalStrokes	 17
U+9080.0	fCangjie	YHSK
U+9080.0	fFourCorner	38304
U+9080.0	fHakka	  jau1; jeu1; rau1
U+9080.0	fCantonese	jiu1
U+9080.0	fMandarin	yao1; jiao1
U+9080.0	fEnglish	[1] invite; ask; request [2] intercept [3] weigh; measure
U+9081.0	fUTF8	邁
U+9081.0	fCNS11643	1-7565
U+9081.0	fBig5	C1DA
U+9081.0	fGB	1-4275
U+9081.0	fR/S	 162.13
U+9081.0	fTotalStrokes	 17
U+9081.0	fCangjie	YTWB
U+9081.0	fFourCorner	34302
U+9081.0	fHakka	  mai3; mai6; mai5
U+9081.0	fCantonese	maai6
U+9081.0	fMandarin	mai4
U+9081.0	fEnglish	[1] surpass; exceed [2] stride; step [3] go on a long jourey [4] old (age) [5] pass
U+9082.0	fUTF8	邂
U+9082.0	fCNS11643	1-7566
U+9082.0	fBig5	C1DB
U+9082.0	fGB	0-6562
U+9082.0	fR/S	 162.13
U+9082.0	fTotalStrokes	 17
U+9082.0	fCangjie	YNBQ
U+9082.0	fFourCorner	37304
U+9082.0	fHakka	 hai5; hai6; hai3
U+9082.0	fCantonese	haai5; haai6 (also)
U+9082.0	fMandarin	xie4; xie2
U+9082.0	fEnglish	 meet without a prior engagement; unexpected meeting; encounter by chance; meet by chance
U+9083.0	fUTF8	邃
U+9083.0	fCNS11643	1-774E
U+9083.0	fBig5	C2E3
U+9083.0	fGB	0-6564
U+9083.0	fR/S	 162.13; 162.14
U+9083.0	fTotalStrokes	 17
U+9083.0	fCangjie	YJCO
U+9083.0	fFourCorner	33303
U+9083.0	fHakka	 sui5
U+9083.0	fCantonese	seoi6; seoi3
U+9083.0	fMandarin	sui4
U+9083.0	fEnglish	[1] deep; far [2] profound; depth (of learning); abyss (of one's mind)
U+9084.0	fUTF8	還
U+9084.0	fCNS11643	1-7564
U+9084.0	fBig5	C1D9
U+9084.0	fGB	1-3B39
U+9084.0	fR/S	 162.13
U+9084.0	fTotalStrokes	 17
U+9084.0	fCangjie	YWLV
U+9084.0	fFourCorner	36303
U+9084.0	fHakka	  wan2;  han2; fan2; sien2; fan3; sion2; van2; {han2}; an5
U+9084.0	fCantonese	waan4; syun4
U+9084.0	fMandarin	huan2; huan4; ying2
U+9084.0	fEnglish	[1] return; come back [2] repay; pay back; resotre; give back [3] a Chinese family name
U+9084.1	fUTF8	還
U+9084.1	fCNS11643	1-7564
U+9084.1	fBig5	C1D9
U+9084.1	fGB	1-3B39
U+9084.1	fR/S	 162.13
U+9084.1	fTotalStrokes	 17
U+9084.1	fCangjie	YWLV
U+9084.1	fFourCorner	36303
U+9084.1	fMandarin	xuan2
U+9084.1	fEnglish	 revolve; turn back
U+9084.2	fUTF8	還
U+9084.2	fCNS11643	1-7564
U+9084.2	fBig5	C1D9
U+9084.2	fGB	1-3B39
U+9084.2	fR/S	 162.13
U+9084.2	fTotalStrokes	 17
U+9084.2	fCangjie	YWLV
U+9084.2	fFourCorner	36303
U+9084.2	fMandarin	hai2
U+9084.2	fEnglish	[1] yet; still [2] passably; fairly; quite [3] also [4] even [5] at the same time [6] or [7] had better
U+9085.0	fUTF8	邅
U+9085.0	fCNS11643	2-5F6E
U+9085.0	fBig5	EEC2
U+9085.0	fGB	3-717A
U+9085.0	fR/S	 162.13
U+9085.0	fTotalStrokes	 17
U+9085.0	fCangjie	YYWM
U+9085.0	fFourCorner	30181
U+9085.0	fHakka	 zhan1; zan1; zen1; zhen1
U+9085.0	fCantonese	zin1
U+9085.0	fMandarin	zhan1; zhan4
U+9085.0	fEnglish	very difficult to proceed
U+9086.0	fUTF8	邆
U+9086.0	fCNS11643	2-5B21
U+9086.0	fBig5	EBD3
U+9086.0	fGB	3-7177
U+9086.0	fR/S	 162.12
U+9086.0	fTotalStrokes	 16
U+9086.0	fCangjie	YNOT
U+9086.0	fFourCorner	37301
U+9086.0	fCantonese	tang4
U+9086.0	fMandarin	teng2; deng4
U+9087.0	fUTF8	邇
U+9087.0	fCNS11643	1-774D
U+9087.0	fBig5	C2E2
U+9087.0	fGB	1-6547
U+9087.0	fR/S	 162.14
U+9087.0	fTotalStrokes	 18
U+9087.0	fCangjie	YMFB
U+9087.0	fFourCorner	31302
U+9087.0	fHakka	  ni3; li1; li3
U+9087.0	fCantonese	ji5
U+9087.0	fMandarin	er3
U+9087.0	fEnglish	[1] near; close to [2] lately; recently
U+9088.0	fUTF8	邈
U+9088.0	fCNS11643	1-774F
U+9088.0	fBig5	C2E4
U+9088.0	fGB	0-6563
U+9088.0	fR/S	 162.14
U+9088.0	fTotalStrokes	 18
U+9088.0	fCangjie	YBHU
U+9088.0	fFourCorner	37301
U+9088.0	fHakka	  miau3;  mau3;  mok7; meu3; mau5
U+9088.0	fCantonese	miu5; mok6 (also)
U+9088.0	fMandarin	miao3; mao4; miao2; mo4
U+9088.0	fEnglish	[1] far; distant; remote [2] same ase @ slight
U+9089.0	fUTF8	邉
U+9089.0	fCNS11643	3-593A
U+9089.0	fGB	E-4050
U+9089.0	fR/S	 162.13; 162.14
U+9089.0	fTotalStrokes	 18; 17
U+9089.0	fCangjie	YHUR
U+9089.0	fFourCorner	36300
U+9089.0	fCantonese	bin1
U+9089.0	fMandarin	bian1
U+9089.0	fEnglish	edge; margin; side; border
U+908A.0	fUTF8	邊
U+908A.0	fCNS11643	1-7931
U+908A.0	fBig5	C3E4
U+908A.0	fGB	1-315F
U+908A.0	fR/S	 162.15
U+908A.0	fTotalStrokes	 19
U+908A.0	fCangjie	YHUS
U+908A.0	fFourCorner	36302
U+908A.0	fHakka	 bien1; ben1; pian1
U+908A.0	fCantonese	bin1; bin6 (also)
U+908A.0	fMandarin	bian1; niao3
U+908A.0	fEnglish	[1] edge; end of something; margin; verge [2] side [3] hem; decorative border [4] border of a nation's territory; boundary [5] limits; bounds [6] nearby; near to [7] a Chinese family name
U+908A.1	fUTF8	邊
U+908A.1	fCNS11643	1-7931
U+908A.1	fBig5	C3E4
U+908A.1	fGB	1-315F
U+908A.1	fR/S	 162.15
U+908A.1	fTotalStrokes	 19
U+908A.1	fCangjie	YHUS
U+908A.1	fFourCorner	36302
U+908A.1	fMandarin	bian5
U+908B.0	fUTF8	邋
U+908B.0	fCNS11643	1-7932
U+908B.0	fBig5	C3E5
U+908B.0	fGB	0-6565
U+908B.0	fR/S	 162.15
U+908B.0	fTotalStrokes	 19
U+908B.0	fCangjie	YVVV
U+908B.0	fFourCorner	32301
U+908B.0	fHakka	 liap8; lap8; lap7
U+908B.0	fCantonese	laap6; laat6 (also); lip6; laat3 (also); lat6
U+908B.0	fMandarin	la1; lie4; la2
U+908B.0	fEnglish	rags
U+908C.0	fUTF8	邌
U+908C.0	fCNS11643	4-615F
U+908C.0	fGB	5-704A
U+908C.0	fR/S	 162.15
U+908C.0	fTotalStrokes	 19
U+908C.0	fCangjie	YHHE
U+908C.0	fFourCorner	37300
U+908C.0	fCantonese	lai4
U+908C.0	fMandarin	li2
U+908C.0	fEnglish	to walk slowly; to parade
U+908D.0	fUTF8	邍
U+908D.0	fCNS11643	2-6A2E
U+908D.0	fBig5	F4E0
U+908D.0	fGB	E-4051
U+908D.0	fR/S	 162.16
U+908D.0	fTotalStrokes	 20
U+908D.0	fCangjie	YHWE
U+908D.0	fFourCorner	31309
U+908D.0	fCantonese	jyun4
U+908D.0	fMandarin	yuan2
U+908E.0	fUTF8	邎
U+908E.0	fCNS11643	4-6952
U+908E.0	fGB	E-4052
U+908E.0	fR/S	 162.17
U+908E.0	fTotalStrokes	 21
U+908E.0	fCangjie	YBRF
U+908E.0	fFourCorner	32300
U+908E.0	fCantonese	jau4
U+908E.0	fMandarin	yao2; you2
U+908F.0	fUTF8	邏
U+908F.0	fCNS11643	1-7C4A
U+908F.0	fBig5	C5DE
U+908F.0	fGB	1-425F
U+908F.0	fR/S	 162.19
U+908F.0	fTotalStrokes	 23
U+908F.0	fCangjie	YWLG
U+908F.0	fFourCorner	36301
U+908F.0	fHakka	 lo2;  lau3;  lo3; lau1
U+908F.0	fCantonese	lo4; lo6
U+908F.0	fMandarin	luo2; luo4; luo3
U+908F.0	fEnglish	 patrol; inspect; watch
U+9090.0	fUTF8	邐
U+9090.0	fCNS11643	1-7C49
U+9090.0	fBig5	C5DD
U+9090.0	fGB	1-654E
U+9090.0	fR/S	 162.19
U+9090.0	fTotalStrokes	 23
U+9090.0	fCangjie	YMMP
U+9090.0	fFourCorner	31301
U+9090.0	fHakka	 li5; li3
U+9090.0	fCantonese	lei5
U+9090.0	fMandarin	li3
U+9090.0	fEnglish	continuous and meandering; winding (mountains, roads, etc.)
U+9091.0	fUTF8	邑
U+9091.0	fCNS11643	1-4B57
U+9091.0	fBig5	A8B6
U+9091.0	fGB	0-5258
U+9091.0	fR/S	 163.0
U+9091.0	fTotalStrokes	 7
U+9091.0	fCangjie	RAU
U+9091.0	fFourCorner	60717
U+9091.0	fHakka	  jip7; rip7
U+9091.0	fCantonese	jap1; ap1; ngap1 (also)
U+9091.0	fMandarin	yi4; e4
U+9091.0	fEnglish	[1] town [2] a political district in ancient China [3] county [4] state [5] capital city [6] same as $ [7] Kangxi radical 163
U+9092.0	fUTF8	邒
U+9092.0	fCNS11643	3-225E
U+9092.0	fGB	3-705C
U+9092.0	fR/S	 163.2
U+9092.0	fTotalStrokes	 5
U+9092.0	fCangjie	MNNL
U+9092.0	fFourCorner	17220
U+9092.0	fCantonese	ting4
U+9092.0	fMandarin	ting2
U+9093.0	fUTF8	邓
U+9093.0	fCNS11643	3-225D
U+9093.0	fGB	0-354B
U+9093.0	fR/S	  163.2
U+9093.0	fTotalStrokes	 5
U+9093.0	fCangjie	ENL
U+9093.0	fFourCorner	77420
U+9093.0	fHakka	 ten5
U+9093.0	fCantonese	dang6
U+9093.0	fMandarin	deng4; shan1
U+9093.0	fEnglish	simplified form of &#x9127;
U+9094.0	fUTF8	邔
U+9094.0	fCNS11643	2-2275
U+9094.0	fBig5	CA55
U+9094.0	fGB	E-4053
U+9094.0	fR/S	 163.3
U+9094.0	fTotalStrokes	 6
U+9094.0	fCangjie	SUNL
U+9094.0	fFourCorner	17727
U+9094.0	fHakka	  ki3; gi3
U+9094.0	fCantonese	hei2; gei2; gei6
U+9094.0	fMandarin	qi3; ji4
U+9094.0	fEnglish	an old name for a district in the north of Hupeh
U+9095.0	fUTF8	邕
U+9095.0	fCNS11643	1-5854
U+9095.0	fBig5	B06F
U+9095.0	fGB	0-675F
U+9095.0	fR/S	 163.3
U+9095.0	fTotalStrokes	 10
U+9095.0	fCangjie	VVRAU
U+9095.0	fFourCorner	22717
U+9095.0	fHakka	 jung1; rung1
U+9095.0	fCantonese	jung1
U+9095.0	fMandarin	yong1; yong3; yong4
U+9095.0	fEnglish	[1] harmony; peaceful; harmonious [2] cultivate (plants) [3] short for a county in Kwangsi province
U+9096.0	fUTF8	邖
U+9096.0	fCNS11643	3-243C
U+9096.0	fGB	3-383E
U+9096.0	fR/S	 163.3; 46.3
U+9096.0	fTotalStrokes	 6
U+9096.0	fCangjie	UNL
U+9096.0	fFourCorner	27720
U+9096.0	fCantonese	saan1
U+9096.0	fMandarin	shan1
U+9097.0	fUTF8	邗
U+9097.0	fCNS11643	2-2272
U+9097.0	fBig5	CA52
U+9097.0	fGB	0-5A75
U+9097.0	fR/S	 163.3
U+9097.0	fTotalStrokes	 6
U+9097.0	fCangjie	MJNL
U+9097.0	fFourCorner	17427
U+9097.0	fHakka	 hon2
U+9097.0	fCantonese	hon4
U+9097.0	fMandarin	han2
U+9097.0	fEnglish	an ancient place in the state of Wu
U+9098.0	fUTF8	邘
U+9098.0	fCNS11643	2-2273
U+9098.0	fBig5	CA53
U+9098.0	fGB	3-705D
U+9098.0	fR/S	 163.3
U+9098.0	fTotalStrokes	 6
U+9098.0	fCangjie	MDNL
U+9098.0	fFourCorner	17427
U+9098.0	fHakka	 ri1; ji1; ji2
U+9098.0	fCantonese	jyu1; jyu4
U+9098.0	fMandarin	yu2
U+9098.0	fEnglish	a state in Henan province
U+9099.0	fUTF8	邙
U+9099.0	fCNS11643	2-2271
U+9099.0	fBig5	CA51
U+9099.0	fGB	0-5A78
U+9099.0	fR/S	 163.3
U+9099.0	fTotalStrokes	 6
U+9099.0	fCangjie	YVNL
U+9099.0	fFourCorner	07727
U+9099.0	fHakka	  mong2
U+9099.0	fCantonese	mong4
U+9099.0	fMandarin	mang2
U+9099.0	fEnglish	a hill near Loyang in Honan province
U+909A.0	fUTF8	邚
U+909A.0	fCNS11643	3-243D
U+909A.0	fGB	E-4054
U+909A.0	fR/S	 163.3
U+909A.0	fTotalStrokes	 6
U+909A.0	fCangjie	VNL
U+909A.0	fFourCorner	47420
U+909A.0	fCantonese	jyu4
U+909A.0	fMandarin	ru2
U+909B.0	fUTF8	邛
U+909B.0	fCNS11643	2-2274
U+909B.0	fBig5	CA54
U+909B.0	fGB	0-5A76
U+909B.0	fR/S	 163.3
U+909B.0	fTotalStrokes	 6
U+909B.0	fCangjie	MNL
U+909B.0	fFourCorner	17127
U+909B.0	fHakka	  kiung2
U+909B.0	fCantonese	kung4
U+909B.0	fMandarin	qiong2
U+909B.0	fEnglish	[1] the name of a county in the Han dynasty in today's Shantung province [2] hill; the heights [3] illness; ailment [4] a Chinese family name
U+909C.0	fUTF8	邜
U+909C.0	fCNS11643	3-6655
U+909C.0	fGB	E-4055
U+909C.0	fR/S	 163.3
U+909C.0	fTotalStrokes	 6
U+909C.0	fCangjie	NINL
U+909C.0	fFourCorner	1722
U+909C.0	fCantonese	zik6
U+909D.0	fUTF8	邝
U+909D.0	fGB	0-5A77
U+909D.0	fR/S	 163.3
U+909D.0	fTotalStrokes	 6
U+909D.0	fCangjie	INL
U+909D.0	fHakka	 kwong3
U+909D.0	fCantonese	kwong3; kong3 (also); kwong2 (also); kong2 (also)
U+909D.0	fMandarin	kuang4; guang3; huang1
U+909D.0	fEnglish	simplified form of &#x913A;
U+909E.0	fUTF8	邞
U+909E.0	fCNS11643	2-252A
U+909E.0	fBig5	CBAA
U+909E.0	fGB	E-4056
U+909E.0	fR/S	 163.4
U+909E.0	fTotalStrokes	 7
U+909E.0	fCangjie	QONL
U+909E.0	fFourCorner	57027
U+909E.0	fCantonese	fu1
U+909E.0	fMandarin	fu1; fu2
U+909F.0	fUTF8	邟
U+909F.0	fCNS11643	2-2527
U+909F.0	fBig5	CBA7
U+909F.0	fGB	3-7062
U+909F.0	fR/S	 163.4
U+909F.0	fTotalStrokes	 7
U+909F.0	fCangjie	YNNL
U+909F.0	fFourCorner	07227
U+909F.0	fHakka	 kong5
U+909F.0	fCantonese	kong3
U+909F.0	fMandarin	kang4; geng1; kang2; hang2; kang1
U+90A0.0	fUTF8	邠
U+90A0.0	fCNS11643	2-252C
U+90A0.0	fBig5	CBAC
U+90A0.0	fGB	3-7061
U+90A0.0	fR/S	 163.4
U+90A0.0	fTotalStrokes	 7
U+90A0.0	fCangjie	CHNL
U+90A0.0	fFourCorner	87227
U+90A0.0	fHakka	 bin1
U+90A0.0	fCantonese	ban1
U+90A0.0	fMandarin	bin1
U+90A0.0	fEnglish	an ancient state in Shensi province
U+90A1.0	fUTF8	邡
U+90A1.0	fCNS11643	2-2528
U+90A1.0	fBig5	CBA8
U+90A1.0	fGB	0-5A7A
U+90A1.0	fR/S	 163.4
U+90A1.0	fTotalStrokes	 7
U+90A1.0	fCangjie	YSNL
U+90A1.0	fFourCorner	07227
U+90A1.0	fHakka	  fong1; fong3
U+90A1.0	fCantonese	fong1
U+90A1.0	fMandarin	fang1; fang4; fang3
U+90A1.0	fEnglish	an old name for a district in Chehkiang
U+90A2.0	fUTF8	邢
U+90A2.0	fCNS11643	1-4B58
U+90A2.0	fBig5	A8B7
U+90A2.0	fGB	0-504F
U+90A2.0	fR/S	 163.4
U+90A2.0	fTotalStrokes	 7
U+90A2.0	fCangjie	MTNL
U+90A2.0	fFourCorner	17427
U+90A2.0	fHakka	  hin2
U+90A2.0	fCantonese	jing4
U+90A2.0	fMandarin	xing2; geng3
U+90A2.0	fEnglish	an ancient state in today's Hopeh province during the Spring and Autumn period
U+90A3.0	fUTF8	那
U+90A3.0	fCNS11643	1-4B5B
U+90A3.0	fBig5	A8BA
U+90A3.0	fGB	0-4447
U+90A3.0	fR/S	 163.4
U+90A3.0	fTotalStrokes	 7
U+90A3.0	fCangjie	SQNL
U+90A3.0	fFourCorner	17527
U+90A3.0	fHakka	 na3; la3; no2; lo2; nai5; ne5; mak7; man3; ngai5; lai5
U+90A3.0	fCantonese	naa5; no4; laa5 (also); lo4 (also); no1; naa4; laa4 (also); naa1; naa6; laa1 (also); no5; no6; lo1 (also); aa6
U+90A3.0	fMandarin	na4; nuo2; ne4; nei2; ne2; nuo4
U+90A3.0	fEnglish	[1] that; that one; those [2] then; in that case
U+90A3.1	fUTF8	那
U+90A3.1	fCNS11643	1-4B5B
U+90A3.1	fBig5	A8BA
U+90A3.1	fGB	0-4447
U+90A3.1	fR/S	 163.4
U+90A3.1	fTotalStrokes	 7
U+90A3.1	fCangjie	SQNL
U+90A3.1	fFourCorner	17527
U+90A3.1	fMandarin	nei4
U+90A3.1	fEnglish	used in combination to indicate emphasis or contempt
U+90A3.2	fUTF8	那
U+90A3.2	fCNS11643	1-4B5B
U+90A3.2	fBig5	A8BA
U+90A3.2	fGB	0-4447
U+90A3.2	fR/S	 163.4
U+90A3.2	fTotalStrokes	 7
U+90A3.2	fCangjie	SQNL
U+90A3.2	fFourCorner	17527
U+90A3.2	fMandarin	na3
U+90A3.2	fEnglish	an interrogative particle; Who?; Which?; What?; Where?
U+90A3.3	fUTF8	那
U+90A3.3	fCNS11643	1-4B5B
U+90A3.3	fBig5	A8BA
U+90A3.3	fGB	0-4447
U+90A3.3	fR/S	 163.4
U+90A3.3	fTotalStrokes	 7
U+90A3.3	fCangjie	SQNL
U+90A3.3	fFourCorner	17527
U+90A3.3	fMandarin	nei3
U+90A3.3	fEnglish	an interrogative particle; which
U+90A3.4	fUTF8	那
U+90A3.4	fCNS11643	1-4B5B
U+90A3.4	fBig5	A8BA
U+90A3.4	fGB	0-4447
U+90A3.4	fR/S	 163.4
U+90A3.4	fTotalStrokes	 7
U+90A3.4	fCangjie	SQNL
U+90A3.4	fFourCorner	17527
U+90A3.4	fMandarin	na1
U+90A3.4	fEnglish	a Chinese family name
U+90A4.0	fUTF8	邤
U+90A4.0	fCNS11643	3-2735
U+90A4.0	fGB	3-4C32
U+90A4.0	fR/S	 163.4
U+90A4.0	fTotalStrokes	 7
U+90A4.0	fCangjie	HLNL
U+90A4.0	fFourCorner	77220
U+90A4.0	fCantonese	jan1
U+90A4.0	fMandarin	xin1
U+90A5.0	fUTF8	邥
U+90A5.0	fCNS11643	2-2529
U+90A5.0	fBig5	CBA9
U+90A5.0	fGB	3-7064
U+90A5.0	fR/S	 163.4
U+90A5.0	fTotalStrokes	 7
U+90A5.0	fCangjie	LUNL
U+90A5.0	fFourCorner	47027
U+90A5.0	fHakka	 shim3; siim3
U+90A5.0	fCantonese	sam2
U+90A5.0	fMandarin	shen3
U+90A6.0	fUTF8	邦
U+90A6.0	fCNS11643	1-4B5A
U+90A6.0	fBig5	A8B9
U+90A6.0	fGB	0-306E
U+90A6.0	fR/S	 163.4
U+90A6.0	fTotalStrokes	 7
U+90A6.0	fCangjie	QJNL
U+90A6.0	fFourCorner	57027
U+90A6.0	fHakka	 bang1; bong1
U+90A6.0	fCantonese	bong1
U+90A6.0	fMandarin	bang1
U+90A6.0	fEnglish	[1] nation; country; state [2] a manor given to a nobleman by the emperor in feudal China
U+90A7.0	fUTF8	邧
U+90A7.0	fCNS11643	2-252B
U+90A7.0	fBig5	CBAB
U+90A7.0	fGB	3-705E
U+90A7.0	fR/S	 163.4
U+90A7.0	fTotalStrokes	 7
U+90A7.0	fCangjie	MUNL
U+90A7.0	fFourCorner	17227
U+90A7.0	fCantonese	jyun4
U+90A7.0	fMandarin	yuan2; yuan3
U+90A7.0	fEnglish	a town in Shanxi province
U+90A8.0	fUTF8	邨
U+90A8.0	fCNS11643	3-2739
U+90A8.0	fGB	8-2F66
U+90A8.0	fR/S	 163.4
U+90A8.0	fTotalStrokes	 7
U+90A8.0	fCangjie	PUNL
U+90A8.0	fFourCorner	57720
U+90A8.0	fHakka	 cun1
U+90A8.0	fCantonese	cyun1
U+90A8.0	fMandarin	cun1
U+90A8.0	fEnglish	[1] same as $ [2] coarse; vulgar [3] name-calling; abusive language [4] scold [5] embarass
U+90A9.0	fUTF8	邩
U+90A9.0	fCNS11643	3-2734
U+90A9.0	fGB	3-4F4B
U+90A9.0	fR/S	 163.4
U+90A9.0	fTotalStrokes	 7
U+90A9.0	fCangjie	FNL
U+90A9.0	fFourCorner	97820
U+90A9.0	fCantonese	fo2
U+90A9.0	fMandarin	huo3
U+90AA.0	fUTF8	邪
U+90AA.0	fCNS11643	1-4B59
U+90AA.0	fBig5	A8B8
U+90AA.0	fGB	0-5030
U+90AA.0	fR/S	 163.4
U+90AA.0	fTotalStrokes	 7
U+90AA.0	fCangjie	MHNL
U+90AA.0	fFourCorner	17227
U+90AA.0	fHakka	  sia2;  ja2; sia3; ra2; ra1; ja1
U+90AA.0	fCantonese	ce4; je4
U+90AA.0	fMandarin	xie2; xu2; she2; yu2; ya2
U+90AA.0	fEnglish	[1] wrong; evil; wicked; mean; depraved; vicious [2] pertaining to sorcery or demonism; abnormal
U+90AA.1	fUTF8	邪
U+90AA.1	fCNS11643	1-4B59
U+90AA.1	fBig5	A8B8
U+90AA.1	fGB	0-5030
U+90AA.1	fR/S	 163.4
U+90AA.1	fTotalStrokes	 7
U+90AA.1	fCangjie	MHNL
U+90AA.1	fFourCorner	17227
U+90AA.1	fMandarin	ye2
U+90AA.1	fEnglish	[1] answer in unison [2] same as $
U+90AB.0	fUTF8	邫
U+90AB.0	fCNS11643	3-3951
U+90AB.0	fGB	E-4057
U+90AB.0	fR/S	 163.4
U+90AB.0	fTotalStrokes	 11; 8
U+90AB.0	fCangjie	QJRAU
U+90AB.0	fFourCorner	56010
U+90AB.0	fCantonese	bong1
U+90AB.0	fMandarin	bang1
U+90AC.0	fUTF8	邬
U+90AC.0	fGB	0-5A79
U+90AC.0	fR/S	 163.4
U+90AC.0	fTotalStrokes	 7
U+90AC.0	fCangjie	PMNL
U+90AC.0	fHakka	 vu1; vu3
U+90AC.0	fCantonese	wu1
U+90AC.0	fMandarin	wu1; wu3; yu4
U+90AC.0	fEnglish	simplified form of &#x9114;
U+90AD.0	fUTF8	邭
U+90AD.0	fCNS11643	3-2A77
U+90AD.0	fGB	E-4058
U+90AD.0	fR/S	  163.5
U+90AD.0	fTotalStrokes	 8
U+90AD.0	fCangjie	PRNL
U+90AD.0	fFourCorner	27620
U+90AD.0	fCantonese	geoi3
U+90AD.0	fMandarin	ju4
U+90AE.0	fUTF8	邮
U+90AE.0	fCNS11643	4-282B
U+90AE.0	fGB	0-534A
U+90AE.0	fR/S	 163.5
U+90AE.0	fTotalStrokes	 8
U+90AE.0	fCangjie	LWNL
U+90AE.0	fFourCorner	57620
U+90AE.0	fHakka	 ju2
U+90AE.0	fCantonese	jau4
U+90AE.0	fMandarin	you2; chui2
U+90AE.0	fEnglish	simplified form of &#x90F5;
U+90AF.0	fUTF8	邯
U+90AF.0	fCNS11643	2-2875
U+90AF.0	fBig5	CDD5
U+90AF.0	fGB	0-3A2A
U+90AF.0	fR/S	 163.5
U+90AF.0	fTotalStrokes	 8
U+90AF.0	fCangjie	TMNL
U+90AF.0	fFourCorner	47727
U+90AF.0	fHakka	  hon2; ham2
U+90AF.0	fCantonese	hon4; ham4
U+90AF.0	fMandarin	han2; han4; han1
U+90AF.0	fEnglish	[1] a county in Hopeh province [2] a river in Chinghai [3] a hill in Hopeh
U+90B0.0	fUTF8	邰
U+90B0.0	fCNS11643	2-2877
U+90B0.0	fBig5	CDD7
U+90B0.0	fGB	0-5B22
U+90B0.0	fR/S	 163.5
U+90B0.0	fTotalStrokes	 8
U+90B0.0	fCangjie	IRNL
U+90B0.0	fFourCorner	27627
U+90B0.0	fHakka	  toi2
U+90B0.0	fCantonese	toi4; toi1
U+90B0.0	fMandarin	tai2; tai1
U+90B0.0	fEnglish	[1] an ancient state in today's Shensi province [2] a Chinese family name
U+90B1.0	fUTF8	邱
U+90B1.0	fCNS11643	1-4F57
U+90B1.0	fBig5	AAF4
U+90B1.0	fGB	0-4771
U+90B1.0	fR/S	 163.5
U+90B1.0	fTotalStrokes	 8
U+90B1.0	fCangjie	OMNL
U+90B1.0	fFourCorner	77127
U+90B1.0	fHakka	  hiu1;  kiu1
U+90B1.0	fCantonese	jau1
U+90B1.0	fMandarin	qiu1
U+90B1.0	fEnglish	[1] same as $ [2] the name of a county in Shantung province [3] a Chinese family name
U+90B2.0	fUTF8	邲
U+90B2.0	fCNS11643	2-2873
U+90B2.0	fBig5	CDD3
U+90B2.0	fGB	3-7065
U+90B2.0	fR/S	 163.5
U+90B2.0	fTotalStrokes	 8
U+90B2.0	fCangjie	PHNL
U+90B2.0	fFourCorner	37027
U+90B2.0	fHakka	 bit7
U+90B2.0	fCantonese	bei3; bat6 (also); bin6; bit1
U+90B2.0	fMandarin	bi4
U+90B2.0	fEnglish	[1] good-looking [2] the name of an ancient place in today's Honan province
U+90B3.0	fUTF8	邳
U+90B3.0	fCNS11643	2-2876
U+90B3.0	fBig5	CDD6
U+90B3.0	fGB	0-5A7C
U+90B3.0	fR/S	 163.5
U+90B3.0	fTotalStrokes	 8
U+90B3.0	fCangjie	MMNL
U+90B3.0	fFourCorner	17127
U+90B3.0	fHakka	 pi1
U+90B3.0	fCantonese	pei4
U+90B3.0	fMandarin	pi1; pei2
U+90B3.0	fEnglish	a department in the State of Lu in what is now Shantung or Northern China
U+90B4.0	fUTF8	邴
U+90B4.0	fCNS11643	2-2874
U+90B4.0	fBig5	CDD4
U+90B4.0	fGB	0-5A7B
U+90B4.0	fR/S	 163.5
U+90B4.0	fTotalStrokes	 8
U+90B4.0	fCangjie	MBNL
U+90B4.0	fFourCorner	17227
U+90B4.0	fHakka	  biang3
U+90B4.0	fCantonese	bing2
U+90B4.0	fMandarin	bing3
U+90B4.0	fEnglish	[1] joyful; jubilant [2] the name of a city in the ancient state of Song in today's Shantung province
U+90B5.0	fUTF8	邵
U+90B5.0	fCNS11643	1-4F55
U+90B5.0	fBig5	AAF2
U+90B5.0	fGB	0-495B
U+90B5.0	fR/S	 163.5
U+90B5.0	fTotalStrokes	 8
U+90B5.0	fCangjie	SRNL
U+90B5.0	fFourCorner	17627
U+90B5.0	fHakka	 sau5; shau5; seu5; chau5; sau2; cau5; zhau5; cheu5
U+90B5.0	fCantonese	siu6
U+90B5.0	fMandarin	shao4
U+90B5.0	fEnglish	[1] advanced (age) [2] a Chinese family name
U+90B6.0	fUTF8	邶
U+90B6.0	fCNS11643	1-4F58
U+90B6.0	fBig5	AAF5
U+90B6.0	fGB	0-5A7D
U+90B6.0	fR/S	 163.5
U+90B6.0	fTotalStrokes	 8
U+90B6.0	fCangjie	LMPNL
U+90B6.0	fFourCorner	17127
U+90B6.0	fHakka	 pui5; pi3
U+90B6.0	fCantonese	bui3; bui6
U+90B6.0	fMandarin	bei4
U+90B6.0	fEnglish	an ancient state in today's Honan province
U+90B7.0	fUTF8	邷
U+90B7.0	fCNS11643	3-2A75
U+90B7.0	fGB	3-463D
U+90B7.0	fR/S	 163.5
U+90B7.0	fTotalStrokes	 8
U+90B7.0	fCangjie	XMNNL
U+90B7.0	fFourCorner	17720
U+90B7.0	fCantonese	ngaa5; aa5 (also)
U+90B7.0	fMandarin	wa3
U+90B8.0	fUTF8	邸
U+90B8.0	fCNS11643	1-4F56
U+90B8.0	fBig5	AAF3
U+90B8.0	fGB	0-5B21
U+90B8.0	fR/S	 163.5
U+90B8.0	fTotalStrokes	 8
U+90B8.0	fCangjie	HMNL
U+90B8.0	fFourCorner	77727
U+90B8.0	fHakka	 di3; dai5; dai3
U+90B8.0	fCantonese	dai2
U+90B8.0	fMandarin	di3; chi2; di4
U+90B8.0	fEnglish	[1] the residence of a prince or the nobility; official residence; residence of a high official [2] princes and noblemen [3] screen [4] the bottom of something [5] a Chinese family name
U+90B9.0	fUTF8	邹
U+90B9.0	fCNS11643	3-2A78
U+90B9.0	fGB	0-575E
U+90B9.0	fR/S	 163.5
U+90B9.0	fTotalStrokes	 8
U+90B9.0	fCangjie	NSNL
U+90B9.0	fFourCorner	27120
U+90B9.0	fHakka	 zeu1
U+90B9.0	fCantonese	zau1
U+90B9.0	fMandarin	zou1; ju4
U+90B9.0	fEnglish	simplified form of &#x9112;
U+90BA.0	fUTF8	邺
U+90BA.0	fGB	0-5A7E
U+90BA.0	fR/S	 163.5
U+90BA.0	fTotalStrokes	 8
U+90BA.0	fCangjie	TCNL
U+90BA.0	fHakka	 ngiap8
U+90BA.0	fCantonese	jip6
U+90BA.0	fMandarin	ye4
U+90BA.0	fEnglish	simplified form of &#x9134;
U+90BB.0	fUTF8	邻
U+90BB.0	fGB	0-415A
U+90BB.0	fR/S	 163.5
U+90BB.0	fTotalStrokes	 8
U+90BB.0	fCangjie	OINL
U+90BB.0	fHakka	 lin2
U+90BB.0	fCantonese	leon4
U+90BB.0	fMandarin	lin2; lin4
U+90BB.0	fEnglish	simplified form of &#x9130;
U+90BC.0	fUTF8	邼
U+90BC.0	fCNS11643	3-2F35
U+90BC.0	fGB	E-4059
U+90BC.0	fR/S	 163.6
U+90BC.0	fTotalStrokes	 9
U+90BC.0	fCangjie	SGNL
U+90BC.0	fFourCorner	17720
U+90BC.0	fCantonese	hong1
U+90BC.0	fMandarin	kuang1
U+90BD.0	fUTF8	邽
U+90BD.0	fCNS11643	2-2D59
U+90BD.0	fBig5	D0B8
U+90BD.0	fGB	3-7067
U+90BD.0	fR/S	 163.6
U+90BD.0	fTotalStrokes	 9
U+90BD.0	fCangjie	GGNL
U+90BD.0	fFourCorner	47127
U+90BD.0	fHakka	 gui1; gwi1; kwui1
U+90BD.0	fCantonese	gwai1
U+90BD.0	fMandarin	gui1
U+90BD.0	fEnglish	a county in the Han dynasty in today's Kansu province
U+90BE.0	fUTF8	邾
U+90BE.0	fCNS11643	2-2D5D
U+90BE.0	fBig5	D0BC
U+90BE.0	fGB	0-5B25
U+90BE.0	fR/S	 163.6
U+90BE.0	fTotalStrokes	 9
U+90BE.0	fCangjie	HDNL
U+90BE.0	fFourCorner	27927
U+90BE.0	fHakka	 zhu1; zu1
U+90BE.0	fCantonese	zyu1
U+90BE.0	fMandarin	zhu1
U+90BE.0	fEnglish	a feudal state which existed B.C.700-469 in today's Shantung province
U+90BF.0	fUTF8	邿
U+90BF.0	fCNS11643	2-2D5A
U+90BF.0	fBig5	D0B9
U+90BF.0	fGB	3-7068
U+90BF.0	fR/S	 163.6
U+90BF.0	fTotalStrokes	 9
U+90BF.0	fCangjie	GINL
U+90BF.0	fFourCorner	47327
U+90BF.0	fHakka	 shi1; sii1
U+90BF.0	fCantonese	si1
U+90BF.0	fMandarin	shi1
U+90BF.0	fEnglish	a state in Shandong province
U+90C0.0	fUTF8	郀
U+90C0.0	fCNS11643	3-2F38
U+90C0.0	fGB	3-706B
U+90C0.0	fR/S	 163.6
U+90C0.0	fTotalStrokes	 9
U+90C0.0	fCangjie	KSNL
U+90C0.0	fFourCorner	47220
U+90C0.0	fCantonese	fu1
U+90C0.0	fMandarin	ku1
U+90C1.0	fUTF8	郁
U+90C1.0	fCNS11643	1-5368
U+90C1.0	fBig5	ADA7
U+90C1.0	fGB	0-5374
U+90C1.0	fR/S	 163.6
U+90C1.0	fTotalStrokes	 9
U+90C1.0	fCangjie	KBNL
U+90C1.0	fFourCorner	47227
U+90C1.0	fHakka	  juk7; hiuk7; ruk7; juk8; jut7
U+90C1.0	fCantonese	juk1; wat1; jyu4
U+90C1.0	fMandarin	yu4
U+90C1.0	fEnglish	[1] adorned; colorfully ornamented; beautiful; refined [2] a Chinese family name [3] simplified form of &#x9B31;
U+90C2.0	fUTF8	郂
U+90C2.0	fCNS11643	3-2F34
U+90C2.0	fGB	3-7072
U+90C2.0	fR/S	 163.6
U+90C2.0	fTotalStrokes	 9
U+90C2.0	fCangjie	YONL
U+90C2.0	fFourCorner	07820
U+90C2.0	fCantonese	goi1
U+90C2.0	fMandarin	gai1
U+90C3.0	fUTF8	郃
U+90C3.0	fCNS11643	1-5369
U+90C3.0	fBig5	ADA8
U+90C3.0	fGB	3-7070
U+90C3.0	fR/S	 163.6
U+90C3.0	fTotalStrokes	 9
U+90C3.0	fCangjie	ORNL
U+90C3.0	fFourCorner	87627
U+90C3.0	fHakka	 hap8
U+90C3.0	fCantonese	hap6; gap3 (also)
U+90C3.0	fMandarin	he2; ge2; xia2
U+90C3.0	fEnglish	[1] a county in Shensi province [2] a Chinese family name
U+90C4.0	fUTF8	郄
U+90C4.0	fCNS11643	3-2F3E
U+90C4.0	fGB	0-5B27
U+90C4.0	fR/S	 163.6
U+90C4.0	fTotalStrokes	 9
U+90C4.0	fCangjie	KINL
U+90C4.0	fFourCorner	47420
U+90C4.0	fHakka	  hit7; kiok7
U+90C4.0	fCantonese	gwik1
U+90C4.0	fMandarin	xi4; qie4; que4
U+90C4.0	fEnglish	a Chinese family name
U+90C5.0	fUTF8	郅
U+90C5.0	fCNS11643	2-2D5C
U+90C5.0	fBig5	D0BB
U+90C5.0	fGB	0-5B24
U+90C5.0	fR/S	 163.6
U+90C5.0	fTotalStrokes	 9
U+90C5.0	fCangjie	MGNL
U+90C5.0	fFourCorner	17127
U+90C5.0	fHakka	 zhit7; zhi5; tsiit7; zih5
U+90C5.0	fCantonese	zat6; gat1; zat1
U+90C5.0	fMandarin	zhi4; ji2; zhi1
U+90C5.0	fEnglish	[1] very; extremely [2] a Chinese family name
U+90C6.0	fUTF8	郆
U+90C6.0	fCNS11643	3-2F37
U+90C6.0	fGB	3-7069
U+90C6.0	fR/S	 163.6
U+90C6.0	fTotalStrokes	 9
U+90C6.0	fCangjie	GRNL
U+90C6.0	fFourCorner	47620
U+90C6.0	fCantonese	gat1
U+90C6.0	fMandarin	ji2
U+90C7.0	fUTF8	郇
U+90C7.0	fCNS11643	2-2D5E
U+90C7.0	fBig5	D0BD
U+90C7.0	fGB	0-5B28
U+90C7.0	fR/S	 163.6
U+90C7.0	fTotalStrokes	 9
U+90C7.0	fCangjie	PANL
U+90C7.0	fFourCorner	27627
U+90C7.0	fHakka	 sun2; sun3
U+90C7.0	fCantonese	seon1; wan4; waan4
U+90C7.0	fMandarin	xun2; huan2
U+90C7.0	fEnglish	[1] an ancient feudal State in today's Shensi province [2] a Chinese family name
U+90C8.0	fUTF8	郈
U+90C8.0	fCNS11643	2-2D60
U+90C8.0	fBig5	D0BF
U+90C8.0	fGB	3-706F
U+90C8.0	fR/S	 163.6
U+90C8.0	fTotalStrokes	 9
U+90C8.0	fCangjie	HRNL
U+90C8.0	fFourCorner	77227
U+90C8.0	fHakka	 heu5; heu6; hiu5
U+90C8.0	fCantonese	hau6; hau5
U+90C8.0	fMandarin	hou4; hou2
U+90C8.0	fEnglish	a place in Shandong province
U+90C9.0	fUTF8	郉
U+90C9.0	fCNS11643	3-2F3C
U+90C9.0	fGB	E-405A
U+90C9.0	fR/S	  163.6
U+90C9.0	fTotalStrokes	 9
U+90C9.0	fCangjie	MJMJL
U+90C9.0	fFourCorner	17420
U+90C9.0	fCantonese	jing4
U+90C9.0	fMandarin	xing2
U+90C9.0	fEnglish	[1] an ancient state in China [2] a Chinese family name
U+90CA.0	fUTF8	郊
U+90CA.0	fCNS11643	1-5366
U+90CA.0	fBig5	ADA5
U+90CA.0	fGB	0-3D3C
U+90CA.0	fR/S	 163.6
U+90CA.0	fTotalStrokes	 9
U+90CA.0	fCangjie	YKNL
U+90CA.0	fFourCorner	07427
U+90CA.0	fHakka	  gau1;  kau1
U+90CA.0	fCantonese	gaau1
U+90CA.0	fMandarin	jiao1
U+90CA.0	fEnglish	[1] suburbs of a city [2] a ceremony for offering sacrifices to Heaven and Earth
U+90CB.0	fUTF8	郋
U+90CB.0	fCNS11643	2-2D5F
U+90CB.0	fBig5	D0BE
U+90CB.0	fGB	3-6548
U+90CB.0	fR/S	 163.6
U+90CB.0	fTotalStrokes	 9
U+90CB.0	fCangjie	HUNL
U+90CB.0	fFourCorner	27627
U+90CB.0	fCantonese	hai4
U+90CB.0	fMandarin	xi1; xi2
U+90CC.0	fUTF8	郌
U+90CC.0	fCNS11643	4-4139
U+90CC.0	fGB	E-405B
U+90CC.0	fR/S	 163.6
U+90CC.0	fTotalStrokes	 13; 9
U+90CC.0	fCangjie	GGRAU
U+90CC.0	fFourCorner	46110
U+90CC.0	fCantonese	gwai1
U+90CC.0	fMandarin	gui1
U+90CC.0	fEnglish	an ancient county in modern Gansu/Shaanxi area.
U+90CD.0	fUTF8	郍
U+90CD.0	fCNS11643	3-2F3B
U+90CD.0	fGB	3-654F
U+90CD.0	fR/S	 163.6
U+90CD.0	fTotalStrokes	 9
U+90CD.0	fCangjie	HYNL
U+90CD.0	fFourCorner	27427
U+90CD.0	fCantonese	naa5; laa5 (also)
U+90CD.0	fMandarin	nuo2
U+90CE.0	fUTF8	郎
U+90CE.0	fCNS11643	1-5367
U+90CE.0	fBig5	ADA6
U+90CE.0	fGB	0-4049
U+90CE.0	fR/S	 163.6
U+90CE.0	fTotalStrokes	 9
U+90CE.0	fCangjie	IINL
U+90CE.0	fFourCorner	37727
U+90CE.0	fHakka	 long3; long2
U+90CE.0	fCantonese	long4; long2 (also)
U+90CE.0	fMandarin	lang2; lang4
U+90CE.0	fEnglish	[1] man; gentleman; young man [2] husband; beau [3] an official rank in ancient times [4] the master (as opposite to servants) [5] a Chinese family name
U+90CF.0	fUTF8	郏
U+90CF.0	fGB	0-5B23
U+90CF.0	fR/S	 163.6
U+90CF.0	fTotalStrokes	 9
U+90CF.0	fCangjie	KTNL
U+90CF.0	fHakka	 giap7
U+90CF.0	fCantonese	gaap3
U+90CF.0	fMandarin	jia2
U+90CF.0	fEnglish	simplified form of &#x90DF;
U+90D0.0	fUTF8	郐
U+90D0.0	fGB	0-5B26
U+90D0.0	fR/S	 163.6
U+90D0.0	fTotalStrokes	 9
U+90D0.0	fCangjie	XOINL
U+90D0.0	fHakka	 kwai5
U+90D0.0	fCantonese	kui2; kwui2 (also)
U+90D0.0	fMandarin	kuai4; gui4
U+90D0.0	fEnglish	simplified form of &#x9136;
U+90D1.0	fUTF8	郑
U+90D1.0	fGB	0-5623
U+90D1.0	fR/S	 163.6
U+90D1.0	fTotalStrokes	 9
U+90D1.0	fCangjie	XTKNL
U+90D1.0	fHakka	 ch'ang5
U+90D1.0	fCantonese	zeng6; zing6 (also); ze6
U+90D1.0	fMandarin	zheng4
U+90D1.0	fEnglish	simplified form of &#x912D;
U+90D2.0	fUTF8	郒
U+90D2.0	fCNS11643	F-456D
U+90D2.0	fGB	E-405C
U+90D2.0	fR/S	 163.7
U+90D2.0	fTotalStrokes	 14
U+90D2.0	fCangjie	IIRAU
U+90D2.0	fFourCorner	3671
U+90D2.0	fCantonese	long4
U+90D2.0	fMandarin	lang2; lang4
U+90D3.0	fUTF8	郓
U+90D3.0	fGB	0-5B29
U+90D3.0	fR/S	 163.6
U+90D3.0	fTotalStrokes	 9
U+90D3.0	fCangjie	BQNL
U+90D3.0	fHakka	 jun5
U+90D3.0	fCantonese	wan6
U+90D3.0	fMandarin	yun4; yun2
U+90D3.0	fEnglish	simplified form of &#x9106;
U+90D4.0	fUTF8	郔
U+90D4.0	fCNS11643	2-3971
U+90D4.0	fBig5	D7EE
U+90D4.0	fGB	3-706E
U+90D4.0	fR/S	 163.7
U+90D4.0	fTotalStrokes	 10
U+90D4.0	fCangjie	NMNL
U+90D4.0	fFourCorner	17427
U+90D4.0	fHakka	 jen2; zan2
U+90D4.0	fCantonese	jin4
U+90D4.0	fMandarin	yan2
U+90D4.0	fEnglish	a place in Shandong province
U+90D5.0	fUTF8	郕
U+90D5.0	fCNS11643	2-2D5B
U+90D5.0	fBig5	D0BA
U+90D5.0	fGB	3-706C
U+90D5.0	fR/S	 163.7; 163.6
U+90D5.0	fTotalStrokes	 10
U+90D5.0	fCangjie	ISNL
U+90D5.0	fFourCorner	77227
U+90D5.0	fHakka	 shin2; siin2
U+90D5.0	fCantonese	sing4
U+90D5.0	fMandarin	cheng2
U+90D5.0	fEnglish	[1] a place in Honan province [2] an ancient state in today's Shantung province [3] a Chinese family name
U+90D6.0	fUTF8	郖
U+90D6.0	fCNS11643	2-3336
U+90D6.0	fBig5	D3F2
U+90D6.0	fGB	3-6E41
U+90D6.0	fR/S	 163.7
U+90D6.0	fTotalStrokes	 10
U+90D6.0	fCangjie	MTNL
U+90D6.0	fFourCorner	17127
U+90D6.0	fCantonese	dau1; dau6 (also)
U+90D6.0	fMandarin	dou1; dou4
U+90D7.0	fUTF8	郗
U+90D7.0	fCNS11643	2-333F
U+90D7.0	fBig5	D3FB
U+90D7.0	fGB	0-5B2D
U+90D7.0	fR/S	 163.7
U+90D7.0	fTotalStrokes	 10
U+90D7.0	fCangjie	KBNL
U+90D7.0	fFourCorner	47227
U+90D7.0	fHakka	 hi1
U+90D7.0	fCantonese	ci1; hei1 (also)
U+90D7.0	fMandarin	xi1; chi1; obs
U+90D7.0	fEnglish	[1] a town in the Chou dynasty [2] a Chinese family name
U+90D8.0	fUTF8	郘
U+90D8.0	fCNS11643	2-333D
U+90D8.0	fBig5	D3F9
U+90D8.0	fGB	3-706D
U+90D8.0	fR/S	 163.7
U+90D8.0	fTotalStrokes	 10
U+90D8.0	fCangjie	RRNL
U+90D8.0	fFourCorner	67627
U+90D8.0	fCantonese	leoi5
U+90D8.0	fMandarin	l3; luu3; lu3
U+90D9.0	fUTF8	郙
U+90D9.0	fCNS11643	2-3338
U+90D9.0	fBig5	D3F4
U+90D9.0	fGB	3-7077
U+90D9.0	fR/S	 163.7
U+90D9.0	fTotalStrokes	 10
U+90D9.0	fCangjie	IBNL
U+90D9.0	fFourCorner	57227
U+90D9.0	fCantonese	fu2
U+90D9.0	fMandarin	fu3; dou1; fu1
U+90D9.0	fEnglish	 be exiled delay payment for
U+90DA.0	fUTF8	郚
U+90DA.0	fCNS11643	2-3339
U+90DA.0	fBig5	D3F5
U+90DA.0	fGB	3-7079
U+90DA.0	fR/S	 163.7
U+90DA.0	fTotalStrokes	 10
U+90DA.0	fCangjie	MRNL
U+90DA.0	fFourCorner	17627
U+90DA.0	fHakka	 ng2
U+90DA.0	fCantonese	ng4
U+90DA.0	fMandarin	wu2; yu2
U+90DA.0	fEnglish	towns in Shandong province
U+90DB.0	fUTF8	郛
U+90DB.0	fCNS11643	2-333E
U+90DB.0	fBig5	D3FA
U+90DB.0	fGB	0-5B2E
U+90DB.0	fR/S	 163.7
U+90DB.0	fTotalStrokes	 10
U+90DB.0	fCangjie	BDNL
U+90DB.0	fFourCorner	27427
U+90DB.0	fHakka	  fu1
U+90DB.0	fCantonese	fu1
U+90DB.0	fMandarin	fu2; fu1
U+90DB.0	fEnglish	 suburbs; outer city
U+90DC.0	fUTF8	郜
U+90DC.0	fCNS11643	2-3340
U+90DC.0	fBig5	D3FC
U+90DC.0	fGB	0-5B2C
U+90DC.0	fR/S	 163.7
U+90DC.0	fTotalStrokes	 10
U+90DC.0	fCangjie	HRNL
U+90DC.0	fFourCorner	27627
U+90DC.0	fHakka	 gau5; kau5
U+90DC.0	fCantonese	gou3
U+90DC.0	fMandarin	gao4; gu4
U+90DC.0	fEnglish	[1] an ancient state in today's Shantung province [2] a place in the state of Tsin in the Spring and Autumn period in today's Shansi province [3] a Chinese family name
U+90DD.0	fUTF8	郝
U+90DD.0	fCNS11643	1-5856
U+90DD.0	fBig5	B071
U+90DD.0	fGB	0-3A42
U+90DD.0	fR/S	 163.7
U+90DD.0	fTotalStrokes	 10
U+90DD.0	fCangjie	GCNL
U+90DD.0	fFourCorner	47327
U+90DD.0	fHakka	 kok7; hau5
U+90DD.0	fCantonese	kok3; gok3 (also)
U+90DD.0	fMandarin	hao3; shi4; he4; chi3
U+90DD.0	fEnglish	[1] an ancient place in today's Shensi province [2] a Chinese family name
U+90DE.0	fUTF8	郞
U+90DE.0	fCNS11643	3-3429
U+90DE.0	fGB	E-405D
U+90DE.0	fR/S	 163.7
U+90DE.0	fTotalStrokes	 10
U+90DE.0	fCangjie	IVNL
U+90DE.0	fFourCorner	37720
U+90DE.0	fHakka	  long2
U+90DE.0	fCantonese	long4
U+90DE.0	fMandarin	lang2; lang4
U+90DE.0	fEnglish	gentleman
U+90DF.0	fUTF8	郟
U+90DF.0	fCNS11643	2-333B
U+90DF.0	fBig5	D3F7
U+90DF.0	fGB	1-5B23
U+90DF.0	fR/S	 163.7
U+90DF.0	fTotalStrokes	 10
U+90DF.0	fCangjie	KONL
U+90DF.0	fFourCorner	47027
U+90DF.0	fHakka	 giap7; hiap8
U+90DF.0	fCantonese	gaap3
U+90DF.0	fMandarin	jia2
U+90DF.0	fEnglish	[1] name of various places in China [2] a Chinese family name
U+90E0.0	fUTF8	郠
U+90E0.0	fCNS11643	2-3337
U+90E0.0	fBig5	D3F3
U+90E0.0	fGB	3-7078
U+90E0.0	fR/S	 163.7
U+90E0.0	fTotalStrokes	 10
U+90E0.0	fCangjie	MKNL
U+90E0.0	fFourCorner	17427
U+90E0.0	fCantonese	gang2
U+90E0.0	fMandarin	geng3
U+90E0.0	fEnglish	a place in Shandong province
U+90E1.0	fUTF8	郡
U+90E1.0	fCNS11643	1-5855
U+90E1.0	fBig5	B070
U+90E1.0	fGB	0-3F24
U+90E1.0	fR/S	 163.7
U+90E1.0	fTotalStrokes	 10
U+90E1.0	fCangjie	SRNL
U+90E1.0	fFourCorner	17627
U+90E1.0	fHakka	 kiun3; kiun5; kiun6; gun5; kiun2
U+90E1.0	fCantonese	gwan6
U+90E1.0	fMandarin	jun4
U+90E1.0	fEnglish	an administrative division in ancient China; prefecture; county
U+90E2.0	fUTF8	郢
U+90E2.0	fCNS11643	1-5857
U+90E2.0	fBig5	B072
U+90E2.0	fGB	0-5B2B
U+90E2.0	fR/S	 163.7
U+90E2.0	fTotalStrokes	 10
U+90E2.0	fCangjie	RGNL
U+90E2.0	fFourCorner	67127
U+90E2.0	fHakka	 jin3; rin3; chin3
U+90E2.0	fCantonese	jing5; cing2 (also)
U+90E2.0	fMandarin	ying3; ying4; cheng2
U+90E2.0	fEnglish	the capital of the state of Chu during the Spring and Autumn period in today's Hupeh province
U+90E3.0	fUTF8	郣
U+90E3.0	fCNS11643	2-333A
U+90E3.0	fBig5	D3F6
U+90E3.0	fGB	3-7076
U+90E3.0	fR/S	 163.7
U+90E3.0	fTotalStrokes	 10
U+90E3.0	fCangjie	JDNL
U+90E3.0	fFourCorner	47427
U+90E3.0	fCantonese	but6
U+90E3.0	fMandarin	bo2
U+90E4.0	fUTF8	郤
U+90E4.0	fCNS11643	2-3341
U+90E4.0	fBig5	D3FD
U+90E4.0	fGB	3-717C
U+90E4.0	fR/S	 163.7
U+90E4.0	fTotalStrokes	 10
U+90E4.0	fCangjie	CRNL
U+90E4.0	fFourCorner	87627
U+90E4.0	fHakka	 hit7; kio1
U+90E4.0	fCantonese	gwik1; kwik1
U+90E4.0	fMandarin	xi4
U+90E4.0	fEnglish	[1] an area in the ancient state of Tsin [2] crack; opening [3] a Chinese family name
U+90E5.0	fUTF8	郥
U+90E5.0	fCNS11643	2-333C
U+90E5.0	fBig5	D3F8
U+90E5.0	fGB	E-405E
U+90E5.0	fR/S	 163.7
U+90E5.0	fTotalStrokes	 10
U+90E5.0	fCangjie	BCNL
U+90E5.0	fFourCorner	67827
U+90E5.0	fCantonese	kwik1
U+90E5.0	fMandarin	bei4
U+90E6.0	fUTF8	郦
U+90E6.0	fGB	0-5B2A
U+90E6.0	fR/S	 163.7
U+90E6.0	fTotalStrokes	 10
U+90E6.0	fCangjie	XMBNL
U+90E6.0	fHakka	 li5
U+90E6.0	fCantonese	lik6
U+90E6.0	fMandarin	li4; zhi2; li2
U+90E6.0	fEnglish	simplified form of &#x9148;
U+90E7.0	fUTF8	郧
U+90E7.0	fGB	0-5447
U+90E7.0	fR/S	 163.7
U+90E7.0	fTotalStrokes	 10
U+90E7.0	fCangjie	RONL
U+90E7.0	fHakka	 jun2
U+90E7.0	fCantonese	wan4
U+90E7.0	fMandarin	yun2
U+90E7.0	fEnglish	simplified form of &#x9116;
U+90E8.0	fUTF8	部
U+90E8.0	fCNS11643	1-5D65
U+90E8.0	fBig5	B3A1
U+90E8.0	fGB	0-323F
U+90E8.0	fR/S	 163.8
U+90E8.0	fTotalStrokes	 11
U+90E8.0	fCangjie	YRNL
U+90E8.0	fFourCorner	07627
U+90E8.0	fHakka	  pu5; pu6; pu1
U+90E8.0	fCantonese	bou6
U+90E8.0	fMandarin	bu4; pou3
U+90E8.0	fEnglish	[1] genus; division; section; department; class; sort [2] cabinet ministry [3] volume; complete work; novel; writing [4] lead; head
U+90E9.0	fUTF8	郩
U+90E9.0	fCNS11643	2-3974
U+90E9.0	fBig5	D7F1
U+90E9.0	fGB	5-6F60
U+90E9.0	fR/S	 163.8
U+90E9.0	fTotalStrokes	 11
U+90E9.0	fCangjie	KBNL
U+90E9.0	fFourCorner	47427
U+90E9.0	fCantonese	haau4; ou2
U+90E9.0	fMandarin	xiao2; ao3
U+90EA.0	fUTF8	郪
U+90EA.0	fCNS11643	2-396C
U+90EA.0	fBig5	D7E9
U+90EA.0	fGB	3-7123
U+90EA.0	fR/S	 163.8
U+90EA.0	fTotalStrokes	 11; 12
U+90EA.0	fCangjie	JVNL
U+90EA.0	fFourCorner	57427
U+90EA.0	fHakka	 ci1
U+90EA.0	fCantonese	cai1
U+90EA.0	fMandarin	qi1; ci1; xi1
U+90EA.0	fEnglish	a stream in Szechwan province
U+90EB.0	fUTF8	郫
U+90EB.0	fCNS11643	2-3972
U+90EB.0	fBig5	D7EF
U+90EB.0	fGB	0-5B2F
U+90EB.0	fR/S	 163.8
U+90EB.0	fTotalStrokes	 11
U+90EB.0	fCangjie	HJNL
U+90EB.0	fFourCorner	27427
U+90EB.0	fHakka	 pi2; bi1
U+90EB.0	fCantonese	pei4
U+90EB.0	fMandarin	pi2; pai2
U+90EB.0	fEnglish	[1] a town in the Spring and Autumn period in today's Honan province [2] a county in Szechwan
U+90EC.0	fUTF8	郬
U+90EC.0	fCNS11643	2-3973
U+90EC.0	fBig5	D7F0
U+90EC.0	fGB	E-405F
U+90EC.0	fR/S	 163.8
U+90EC.0	fTotalStrokes	 11
U+90EC.0	fCangjie	QBNL
U+90EC.0	fFourCorner	57227
U+90EC.0	fCantonese	cing1
U+90EC.0	fMandarin	qing1
U+90ED.0	fUTF8	郭
U+90ED.0	fCNS11643	1-5D66
U+90ED.0	fBig5	B3A2
U+90ED.0	fGB	0-3979
U+90ED.0	fR/S	 163.8
U+90ED.0	fTotalStrokes	 11
U+90ED.0	fCangjie	YDNL
U+90ED.0	fFourCorner	07427
U+90ED.0	fHakka	 gok7; kok7; gwok7
U+90ED.0	fCantonese	gwok3; gok3 (also); gwik1
U+90ED.0	fMandarin	guo1; kuo4; guo2
U+90ED.0	fEnglish	[1] used in combination for the name of a town [2] outer wall; outer part (of a city) [3] the outer part of anything [4] a Chinese family name
U+90EE.0	fUTF8	郮
U+90EE.0	fCNS11643	3-394F
U+90EE.0	fGB	E-4060
U+90EE.0	fR/S	  163.8
U+90EE.0	fTotalStrokes	 11
U+90EE.0	fCangjie	BRNL
U+90EE.0	fFourCorner	77220
U+90EE.0	fCantonese	zau1
U+90EE.0	fMandarin	zhou1
U+90EF.0	fUTF8	郯
U+90EF.0	fCNS11643	2-396B
U+90EF.0	fBig5	D7E8
U+90EF.0	fGB	0-5B30
U+90EF.0	fR/S	 163.8
U+90EF.0	fTotalStrokes	 11
U+90EF.0	fCangjie	FFNL
U+90EF.0	fFourCorner	97827
U+90EF.0	fHakka	  tam2; ram2; jam2
U+90EF.0	fCantonese	taam4
U+90EF.0	fMandarin	tan2
U+90EF.0	fEnglish	[1] a small ancient state in today's Shantung province [2] a Chinese family name
U+90F0.0	fUTF8	郰
U+90F0.0	fCNS11643	2-396D
U+90F0.0	fBig5	D7EA
U+90F0.0	fGB	3-707E
U+90F0.0	fR/S	 163.8
U+90F0.0	fTotalStrokes	 11
U+90F0.0	fCangjie	SJENL
U+90F0.0	fFourCorner	17127
U+90F0.0	fHakka	  zeu1
U+90F0.0	fCantonese	zau1
U+90F0.0	fMandarin	zou1; suan4
U+90F0.0	fEnglish	a place in Shantong; birthplace of Confucius
U+90F1.0	fUTF8	郱
U+90F1.0	fCNS11643	2-2D58
U+90F1.0	fBig5	D0B7
U+90F1.0	fGB	3-7073
U+90F1.0	fR/S	 163.8
U+90F1.0	fTotalStrokes	 11
U+90F1.0	fCangjie	TTNL
U+90F1.0	fFourCorner	87427
U+90F1.0	fHakka	 pin2;  piang2
U+90F1.0	fCantonese	ping4
U+90F1.0	fMandarin	ping2
U+90F1.0	fEnglish	an ancient place in Shantung
U+90F2.0	fUTF8	郲
U+90F2.0	fCNS11643	2-396F
U+90F2.0	fBig5	D7EC
U+90F2.0	fGB	3-707A
U+90F2.0	fR/S	 163.8
U+90F2.0	fTotalStrokes	 11
U+90F2.0	fCangjie	DONL
U+90F2.0	fFourCorner	47927
U+90F2.0	fHakka	  loi2
U+90F2.0	fCantonese	loi4
U+90F2.0	fMandarin	lai2; lei3
U+90F2.0	fEnglish	an ancient city in the state of Ch'ang
U+90F3.0	fUTF8	郳
U+90F3.0	fCNS11643	2-3970
U+90F3.0	fBig5	D7ED
U+90F3.0	fGB	3-7125
U+90F3.0	fR/S	 163.8
U+90F3.0	fTotalStrokes	 11
U+90F3.0	fCangjie	HUNL
U+90F3.0	fFourCorner	77227
U+90F3.0	fHakka	  ngi2; ri2; ji2
U+90F3.0	fCantonese	ngai4; ai4 (also)
U+90F3.0	fMandarin	ni2
U+90F3.0	fEnglish	a state in Shandong province
U+90F4.0	fUTF8	郴
U+90F4.0	fCNS11643	2-396E
U+90F4.0	fBig5	D7EB
U+90F4.0	fGB	0-333B
U+90F4.0	fR/S	 163.8
U+90F4.0	fTotalStrokes	 11
U+90F4.0	fCangjie	DDNL
U+90F4.0	fFourCorner	47927
U+90F4.0	fHakka	 chim1; cihm1
U+90F4.0	fCantonese	sam1; cam1
U+90F4.0	fMandarin	chen1; lan2
U+90F4.0	fEnglish	[1] a county in Hunan province [2] a Chinese family name
U+90F5.0	fUTF8	郵
U+90F5.0	fCNS11643	1-6253
U+90F5.0	fBig5	B66C
U+90F5.0	fGB	1-534A
U+90F5.0	fR/S	 163.8
U+90F5.0	fTotalStrokes	 11
U+90F5.0	fCangjie	HMNL
U+90F5.0	fFourCorner	27127
U+90F5.0	fHakka	  ju2; jiu2; jeu2; jiu3; riu2
U+90F5.0	fCantonese	jau4
U+90F5.0	fMandarin	you2; chui2
U+90F5.0	fEnglish	[1] postal; mail [2] post office [3] deliver mails, letters, etc. [4] a wayside station where couriers on government service change horses [5] hut; lodge in the field
U+90F6.0	fUTF8	郶
U+90F6.0	fCNS11643	4-4E3B
U+90F6.0	fGB	E-4061
U+90F6.0	fR/S	 163.8
U+90F6.0	fTotalStrokes	 15
U+90F6.0	fCangjie	YRRAU
U+90F6.0	fFourCorner	06610
U+90F6.0	fCantonese	bou6
U+90F6.0	fMandarin	bu4
U+90F7.0	fUTF8	郷
U+90F7.0	fCNS11643	3-3952
U+90F7.0	fGB	E-4062
U+90F7.0	fR/S	 163.8
U+90F7.0	fTotalStrokes	 11
U+90F7.0	fCangjie	VHAIL
U+90F7.0	fFourCorner	27220
U+90F7.0	fCantonese	hoeng1
U+90F7.0	fMandarin	xiang1; xiang3; xiang4
U+90F7.0	fEnglish	 country rural village
U+90F8.0	fUTF8	郸
U+90F8.0	fGB	0-3526
U+90F8.0	fR/S	 163.8
U+90F8.0	fTotalStrokes	 11
U+90F8.0	fCangjie	CJNL
U+90F8.0	fHakka	 dan1
U+90F8.0	fCantonese	daan1
U+90F8.0	fMandarin	dan1; duo1
U+90F8.0	fEnglish	simplified form of &#x9132;
U+90F9.0	fUTF8	郹
U+90F9.0	fCNS11643	2-407B
U+90F9.0	fBig5	DC56
U+90F9.0	fGB	3-7128
U+90F9.0	fR/S	 163.9
U+90F9.0	fTotalStrokes	 12
U+90F9.0	fCangjie	BKNL
U+90F9.0	fFourCorner	67427
U+90F9.0	fCantonese	gwik1
U+90F9.0	fMandarin	ju2
U+90FA.0	fUTF8	郺
U+90FA.0	fCNS11643	2-5B22
U+90FA.0	fBig5	EBD4
U+90FA.0	fGB	E-4063
U+90FA.0	fR/S	 163.9
U+90FA.0	fTotalStrokes	 16; 12
U+90FA.0	fCangjie	NNVVU
U+90FA.0	fFourCorner	22217
U+90FA.0	fCantonese	jung1; jung2
U+90FA.0	fMandarin	yong1; weng3
U+90FB.0	fUTF8	郻
U+90FB.0	fCNS11643	2-407C
U+90FB.0	fBig5	DC57
U+90FB.0	fGB	E-4064
U+90FB.0	fR/S	 163.9
U+90FB.0	fTotalStrokes	 12
U+90FB.0	fCangjie	BVNL
U+90FB.0	fFourCorner	67227
U+90FB.0	fCantonese	hiu1
U+90FB.0	fMandarin	qiao1
U+90FC.0	fUTF8	郼
U+90FC.0	fCNS11643	2-4079
U+90FC.0	fBig5	DC54
U+90FC.0	fGB	E-4065
U+90FC.0	fR/S	 163.9
U+90FC.0	fTotalStrokes	 12
U+90FC.0	fCangjie	DQNL
U+90FC.0	fFourCorner	47527
U+90FC.0	fCantonese	ji1
U+90FC.0	fMandarin	yi1
U+90FD.0	fUTF8	都
U+90FD.0	fCNS11643	1-5D67
U+90FD.0	fBig5	B3A3
U+90FD.0	fGB	0-363C
U+90FD.0	fR/S	 163.9; 163.8
U+90FD.0	fTotalStrokes	 12
U+90FD.0	fCangjie	JANL
U+90FD.0	fFourCorner	47627
U+90FD.0	fMandarin	dou1
U+90FD.0	fEnglish	[1] all; together [2] even [3] already
U+90FD.1	fUTF8	都
U+90FD.1	fCNS11643	1-5D67
U+90FD.1	fBig5	B3A3
U+90FD.1	fGB	0-363C
U+90FD.1	fR/S	 163.9; 163.8
U+90FD.1	fTotalStrokes	 12
U+90FD.1	fCangjie	JANL
U+90FD.1	fFourCorner	47627
U+90FD.1	fHakka	  du1; do1
U+90FD.1	fCantonese	dou1
U+90FD.1	fMandarin	du1; zhu1
U+90FD.1	fEnglish	[1] large town; city; metropolis [2] capital of a nation; make a city the national capital [3] beautiful; elegant; fine [4] live; stay in [5] a Chinese family name
U+90FE.0	fUTF8	郾
U+90FE.0	fCNS11643	1-6255
U+90FE.0	fBig5	B66E
U+90FE.0	fGB	0-5B31
U+90FE.0	fR/S	 163.9
U+90FE.0	fTotalStrokes	 12
U+90FE.0	fCangjie	SVNL
U+90FE.0	fFourCorner	77727
U+90FE.0	fHakka	 ren5; jen5; jen3
U+90FE.0	fCantonese	jin2
U+90FE.0	fMandarin	yan3; yan1
U+90FE.0	fEnglish	a county in Henan province
U+90FF.0	fUTF8	郿
U+90FF.0	fCNS11643	2-4078
U+90FF.0	fBig5	DC53
U+90FF.0	fGB	3-712E
U+90FF.0	fR/S	 163.9
U+90FF.0	fTotalStrokes	 12
U+90FF.0	fCangjie	AUNL
U+90FF.0	fFourCorner	77227
U+90FF.0	fHakka	 mi2
U+90FF.0	fCantonese	mei4
U+90FF.0	fMandarin	mei2; fei2; mei4
U+90FF.0	fEnglish	[1] Mei, a county in Shensi province [2] a town of Lu in the Spring and Autumn period
U+9100.0	fUTF8	鄀
U+9100.0	fCNS11643	2-407E
U+9100.0	fBig5	DC59
U+9100.0	fGB	3-7121
U+9100.0	fR/S	 163.9; 163.8
U+9100.0	fTotalStrokes	 12
U+9100.0	fCangjie	TRNL
U+9100.0	fFourCorner	47627
U+9100.0	fHakka	 jok8; rok8
U+9100.0	fCantonese	joek6
U+9100.0	fMandarin	ruo4; chuo4
U+9100.0	fEnglish	a state in Henan province
U+9101.0	fUTF8	鄁
U+9101.0	fCNS11643	2-407D
U+9101.0	fBig5	DC58
U+9101.0	fGB	E-4066
U+9101.0	fR/S	 163.9
U+9101.0	fTotalStrokes	 12
U+9101.0	fCangjie	LBNL
U+9101.0	fFourCorner	17227
U+9101.0	fCantonese	bui3; bui6
U+9101.0	fMandarin	bei4
U+9102.0	fUTF8	鄂
U+9102.0	fCNS11643	1-6252
U+9102.0	fBig5	B66B
U+9102.0	fGB	0-3675
U+9102.0	fR/S	 163.9
U+9102.0	fTotalStrokes	 12
U+9102.0	fCangjie	RSNL
U+9102.0	fFourCorner	67227
U+9102.0	fHakka	  ngok8
U+9102.0	fCantonese	ngok6; ok6 (also)
U+9102.0	fMandarin	e4
U+9102.0	fEnglish	[1] edge; verge; brink; brim [2] short for Hupeh province [3] startled; surprised [4] blunt; honest [5] a Chinese family name
U+9103.0	fUTF8	鄃
U+9103.0	fCNS11643	2-4123
U+9103.0	fBig5	DC5C
U+9103.0	fGB	3-712C
U+9103.0	fR/S	 163.9
U+9103.0	fTotalStrokes	 12
U+9103.0	fCangjie	ONNL
U+9103.0	fFourCorner	87227
U+9103.0	fHakka	 ji2; ri2; shu1; su1; ji1
U+9103.0	fCantonese	syu1
U+9103.0	fMandarin	yu2; shu1
U+9103.0	fEnglish	a old name for a district in Shantung
U+9104.0	fUTF8	鄄
U+9104.0	fCNS11643	2-4077
U+9104.0	fBig5	DC52
U+9104.0	fGB	0-5B32
U+9104.0	fR/S	 163.9
U+9104.0	fTotalStrokes	 12
U+9104.0	fCangjie	MGNL
U+9104.0	fFourCorner	17127
U+9104.0	fHakka	 gien5; rin1; jin1; gen5
U+9104.0	fCantonese	gyun3
U+9104.0	fMandarin	juan4; jian1; yin1; zhan1; zhen1
U+9104.0	fEnglish	an ancient place in the kingdom of Wei
U+9105.0	fUTF8	鄅
U+9105.0	fCNS11643	2-4122
U+9105.0	fBig5	DC5B
U+9105.0	fGB	3-712A
U+9105.0	fR/S	 163.9
U+9105.0	fTotalStrokes	 12
U+9105.0	fCangjie	HBNL
U+9105.0	fFourCorner	27227
U+9105.0	fHakka	 ji3; ri3
U+9105.0	fCantonese	jyu5; geoi2
U+9105.0	fMandarin	yu3; ju3
U+9105.0	fEnglish	a small feudal state
U+9106.0	fUTF8	鄆
U+9106.0	fCNS11643	2-4075
U+9106.0	fBig5	DC50
U+9106.0	fGB	1-5B29
U+9106.0	fR/S	 163.9
U+9106.0	fTotalStrokes	 12
U+9106.0	fCangjie	BJNL
U+9106.0	fFourCorner	77527
U+9106.0	fHakka	 jun5; run6
U+9106.0	fCantonese	wan6
U+9106.0	fMandarin	yun4; yun2
U+9106.0	fEnglish	[1] the name of an ancient town in Shantung province [2] a Chinese family name
U+9107.0	fUTF8	鄇
U+9107.0	fCNS11643	2-4121
U+9107.0	fBig5	DC5A
U+9107.0	fGB	3-712B
U+9107.0	fR/S	 163.9
U+9107.0	fTotalStrokes	 12
U+9107.0	fCangjie	ONKL
U+9107.0	fFourCorner	27227
U+9107.0	fHakka	 heu2; heu3; heu6; heu5
U+9107.0	fCantonese	hau6; hau4
U+9107.0	fMandarin	hou4; hou2
U+9107.0	fEnglish	a town in Henan province
U+9108.0	fUTF8	鄈
U+9108.0	fCNS11643	2-407A
U+9108.0	fBig5	DC55
U+9108.0	fGB	3-712F
U+9108.0	fR/S	 163.9
U+9108.0	fTotalStrokes	 12
U+9108.0	fCangjie	NKNL
U+9108.0	fFourCorner	17427
U+9108.0	fHakka	 kui2
U+9108.0	fCantonese	kwai4
U+9108.0	fMandarin	kui2
U+9108.0	fEnglish	 slippers; sandals dance shoes
U+9109.0	fUTF8	鄉
U+9109.0	fCNS11643	1-6254
U+9109.0	fBig5	B66D
U+9109.0	fGB	1-4F67
U+9109.0	fR/S	 163.9
U+9109.0	fTotalStrokes	 12
U+9109.0	fCangjie	VHIIL
U+9109.0	fFourCorner	27727
U+9109.0	fHakka	 hiong1; hiong3; hiong5
U+9109.0	fCantonese	hoeng1; hoeng3
U+9109.0	fMandarin	xiang1; xiang3
U+9109.0	fEnglish	[1] village; country [2] rural [3] a small administrative unit comprising of several villages [4] one's native place; birth place; one's village; one's country
U+9109.1	fUTF8	鄉
U+9109.1	fCNS11643	1-6254
U+9109.1	fBig5	B66D
U+9109.1	fGB	1-4F67
U+9109.1	fR/S	 163.9
U+9109.1	fTotalStrokes	 12
U+9109.1	fCangjie	VHIIL
U+9109.1	fFourCorner	27727
U+9109.1	fMandarin	xiang4
U+9109.1	fEnglish	[1] incline toward; direct to [2] guide; approach [3] advise
U+910A.0	fUTF8	鄊
U+910A.0	fCNS11643	3-3F59
U+910A.0	fGB	E-4067
U+910A.0	fR/S	 163.9
U+910A.0	fTotalStrokes	 12
U+910A.0	fCangjie	VHSJL
U+910A.0	fFourCorner	27220
U+910A.0	fCantonese	hoeng1
U+910A.0	fMandarin	xiang1
U+910B.0	fUTF8	鄋
U+910B.0	fCNS11643	2-4830
U+910B.0	fBig5	E0AA
U+910B.0	fGB	3-7129
U+910B.0	fR/S	 163.10
U+910B.0	fTotalStrokes	 13
U+910B.0	fCangjie	HENL
U+910B.0	fFourCorner	77427
U+910B.0	fHakka	  seu1
U+910B.0	fCantonese	sau1; sau2
U+910B.0	fMandarin	sou1; sao1
U+910B.0	fEnglish	county
U+910C.0	fUTF8	鄌
U+910C.0	fCNS11643	3-4523
U+910C.0	fGB	3-7134
U+910C.0	fR/S	 163.10
U+910C.0	fTotalStrokes	 13
U+910C.0	fCangjie	IRNL
U+910C.0	fFourCorner	07220
U+910C.0	fCantonese	tong4
U+910C.0	fMandarin	tang2
U+910D.0	fUTF8	鄍
U+910D.0	fCNS11643	2-482B
U+910D.0	fBig5	E0A5
U+910D.0	fGB	3-7136
U+910D.0	fR/S	 163.10
U+910D.0	fTotalStrokes	 13
U+910D.0	fCangjie	BCNL
U+910D.0	fFourCorner	77827
U+910D.0	fCantonese	ming4
U+910D.0	fMandarin	ming2
U+910D.0	fEnglish	a town in Shandong province
U+910E.0	fUTF8	鄎
U+910E.0	fCNS11643	2-4831
U+910E.0	fBig5	E0AB
U+910E.0	fGB	5-6F65
U+910E.0	fR/S	 163.10
U+910E.0	fTotalStrokes	 13
U+910E.0	fCangjie	HPNL
U+910E.0	fFourCorner	27327
U+910E.0	fHakka	 sit7
U+910E.0	fCantonese	sik1
U+910E.0	fMandarin	xi4; xi2
U+910E.0	fEnglish	a place in Henan province
U+910F.0	fUTF8	鄏
U+910F.0	fCNS11643	2-482C
U+910F.0	fBig5	E0A6
U+910F.0	fGB	3-7132
U+910F.0	fR/S	 163.10
U+910F.0	fTotalStrokes	 13
U+910F.0	fCangjie	MINL
U+910F.0	fFourCorner	77327
U+910F.0	fCantonese	juk6
U+910F.0	fMandarin	ru4; ru3
U+910F.0	fEnglish	a place in Henan province
U+9110.0	fUTF8	鄐
U+9110.0	fCNS11643	2-482A
U+9110.0	fBig5	E0A4
U+9110.0	fGB	3-7135
U+9110.0	fR/S	 163.10
U+9110.0	fTotalStrokes	 13
U+9110.0	fCangjie	YWNL
U+9110.0	fFourCorner	07627
U+9110.0	fHakka	 hiuk7
U+9110.0	fCantonese	cuk1
U+9110.0	fMandarin	chu4; xu4
U+9110.0	fEnglish	a town in Henan province
U+9111.0	fUTF8	鄑
U+9111.0	fCNS11643	2-482D
U+9111.0	fBig5	E0A7
U+9111.0	fGB	3-7131
U+9111.0	fR/S	 163.10
U+9111.0	fTotalStrokes	 13
U+9111.0	fCangjie	MANL
U+9111.0	fFourCorner	17627
U+9111.0	fHakka	 zih1; zin3; zin5
U+9111.0	fCantonese	zi1
U+9111.0	fMandarin	zi1; jin4
U+9111.0	fEnglish	 kick place in Shantung province
U+9112.0	fUTF8	鄒
U+9112.0	fCNS11643	1-6739
U+9112.0	fBig5	B951
U+9112.0	fGB	1-575E
U+9112.0	fR/S	 163.10
U+9112.0	fTotalStrokes	 13
U+9112.0	fCangjie	PUNL
U+9112.0	fFourCorner	27427
U+9112.0	fHakka	 zeu1; ziu1
U+9112.0	fCantonese	zau1
U+9112.0	fMandarin	zou1; ju4
U+9112.0	fEnglish	[1] an ancient state in the Warring states period [2] a Chinese family name
U+9113.0	fUTF8	鄓
U+9113.0	fCNS11643	3-4524
U+9113.0	fGB	E-4068
U+9113.0	fR/S	 163.10
U+9113.0	fTotalStrokes	 13
U+9113.0	fCangjie	HKNL
U+9113.0	fFourCorner	27420
U+9113.0	fCantonese	jip3
U+9113.0	fMandarin	ju2
U+9114.0	fUTF8	鄔
U+9114.0	fCNS11643	2-482F
U+9114.0	fBig5	E0A9
U+9114.0	fGB	1-5A79
U+9114.0	fR/S	 163.10
U+9114.0	fTotalStrokes	 13
U+9114.0	fCangjie	HFNL
U+9114.0	fFourCorner	27327
U+9114.0	fHakka	 wu1;  wu3; vu1; vu3
U+9114.0	fCantonese	wu1
U+9114.0	fMandarin	wu1; wu3; yu4
U+9114.0	fEnglish	[1] the name of various places in ancient times [2] a Chinese family name
U+9115.0	fUTF8	鄕
U+9115.0	fCNS11643	3-4525
U+9115.0	fGB	E-4069
U+9115.0	fR/S	 163.10
U+9115.0	fTotalStrokes	 13
U+9115.0	fCangjie	VHHPL
U+9115.0	fFourCorner	27727
U+9115.0	fHakka	 hiong1; hiong5
U+9115.0	fCantonese	hoeng1; hoeng3
U+9115.0	fMandarin	xiang1; xiang3; xiang4
U+9115.0	fEnglish	 country rural village
U+9116.0	fUTF8	鄖
U+9116.0	fCNS11643	2-482E
U+9116.0	fBig5	E0A8
U+9116.0	fGB	1-5447
U+9116.0	fR/S	 163.10
U+9116.0	fTotalStrokes	 13
U+9116.0	fCangjie	RCNL
U+9116.0	fFourCorner	67827
U+9116.0	fHakka	 jun2; run2
U+9116.0	fCantonese	wan4
U+9116.0	fMandarin	yun2
U+9116.0	fEnglish	a county in Hubei province
U+9117.0	fUTF8	鄗
U+9117.0	fCNS11643	1-673A
U+9117.0	fBig5	B952
U+9117.0	fGB	3-7B6E
U+9117.0	fR/S	 163.10
U+9117.0	fTotalStrokes	 13
U+9117.0	fCangjie	YBNL
U+9117.0	fFourCorner	07227
U+9117.0	fHakka	  hau3; gau1; hau5
U+9117.0	fCantonese	hou6; haau1; hok3
U+9117.0	fMandarin	hao4; huo4; jiao1; qiao1
U+9117.0	fEnglish	an ancient town in Hupeh province
U+9118.0	fUTF8	鄘
U+9118.0	fCNS11643	1-6B49
U+9118.0	fBig5	BBC1
U+9118.0	fGB	3-713A
U+9118.0	fR/S	 163.11
U+9118.0	fTotalStrokes	 14
U+9118.0	fCangjie	IBNL
U+9118.0	fFourCorner	07227
U+9118.0	fHakka	 jung2; rung2
U+9118.0	fCantonese	jung4; tong4
U+9118.0	fMandarin	yong1; yong2
U+9118.0	fEnglish	an ancient state in today's Honan
U+9119.0	fUTF8	鄙
U+9119.0	fCNS11643	1-6B48
U+9119.0	fBig5	BBC0
U+9119.0	fGB	0-3149
U+9119.0	fR/S	 163.11
U+9119.0	fTotalStrokes	 14
U+9119.0	fCangjie	RWNL
U+9119.0	fFourCorner	67627
U+9119.0	fHakka	  pi3; pe3
U+9119.0	fCantonese	pei2
U+9119.0	fMandarin	bi3; bi4
U+9119.0	fEnglish	[1] mean; lowly; base; despicable [2] superficial; shallow [3] remote; out-of-the-way [4] despise; scorn
U+911A.0	fUTF8	鄚
U+911A.0	fCNS11643	2-4E56
U+911A.0	fBig5	E46E
U+911A.0	fGB	3-7130
U+911A.0	fR/S	 163.11; 163.10
U+911A.0	fTotalStrokes	 14
U+911A.0	fCangjie	TKNL
U+911A.0	fFourCorner	47427
U+911A.0	fHakka	 mok7
U+911A.0	fCantonese	mok6
U+911A.0	fMandarin	mo4; mao4; obs
U+911A.0	fEnglish	a place in today's Hebei province
U+911B.0	fUTF8	鄛
U+911B.0	fCNS11643	2-4E59
U+911B.0	fBig5	E471
U+911B.0	fGB	3-713D
U+911B.0	fR/S	 163.11
U+911B.0	fTotalStrokes	 14
U+911B.0	fCangjie	VDNL
U+911B.0	fFourCorner	27927
U+911B.0	fCantonese	caau4
U+911B.0	fMandarin	chao2; zhao1
U+911C.0	fUTF8	鄜
U+911C.0	fCNS11643	2-4E51
U+911C.0	fBig5	E469
U+911C.0	fGB	3-7C2B
U+911C.0	fR/S	 163.11
U+911C.0	fTotalStrokes	 14
U+911C.0	fCangjie	IPNL
U+911C.0	fFourCorner	02227
U+911C.0	fHakka	 fu1
U+911C.0	fCantonese	fu1
U+911C.0	fMandarin	fu1; lu4
U+911C.0	fEnglish	a county in Shensi province
U+911D.0	fUTF8	鄝
U+911D.0	fCNS11643	2-4E55
U+911D.0	fBig5	E46D
U+911D.0	fGB	3-713C
U+911D.0	fR/S	 163.11
U+911D.0	fTotalStrokes	 14
U+911D.0	fCangjie	SHNL
U+911D.0	fFourCorner	17227
U+911D.0	fHakka	 liau3; liau5
U+911D.0	fCantonese	liu5
U+911D.0	fMandarin	liao3; liao4
U+911D.0	fEnglish	a small feudal state in Honan
U+911E.0	fUTF8	鄞
U+911E.0	fCNS11643	1-6B4A
U+911E.0	fBig5	BBC2
U+911E.0	fGB	0-5B34
U+911E.0	fR/S	 163.11
U+911E.0	fTotalStrokes	 14
U+911E.0	fCangjie	TMNL
U+911E.0	fFourCorner	47127
U+911E.0	fHakka	  ngiun2; giun3
U+911E.0	fCantonese	ngan4; an4 (also)
U+911E.0	fMandarin	yin2
U+911E.0	fEnglish	a county in Chekiang province
U+911F.0	fUTF8	鄟
U+911F.0	fCNS11643	2-4E54
U+911F.0	fBig5	E46C
U+911F.0	fGB	3-705F
U+911F.0	fR/S	 163.11
U+911F.0	fTotalStrokes	 14
U+911F.0	fCangjie	JINL
U+911F.0	fFourCorner	57327
U+911F.0	fHakka	 zhon1; zon1
U+911F.0	fCantonese	zyun1
U+911F.0	fMandarin	zhuan1; shuan4; tuan2
U+911F.0	fEnglish	a name of a place
U+9120.0	fUTF8	鄠
U+9120.0	fCNS11643	2-4E52
U+9120.0	fBig5	E46A
U+9120.0	fGB	3-7137
U+9120.0	fR/S	 163.11
U+9120.0	fTotalStrokes	 14
U+9120.0	fCangjie	MSNL
U+9120.0	fFourCorner	17227
U+9120.0	fHakka	 fu1; fu5; fu6
U+9120.0	fCantonese	wu6
U+9120.0	fMandarin	hu4
U+9120.0	fEnglish	a county in Shaanxi province
U+9121.0	fUTF8	鄡
U+9121.0	fCNS11643	2-4E58
U+9121.0	fBig5	E470
U+9121.0	fGB	3-7127
U+9121.0	fR/S	 163.11
U+9121.0	fTotalStrokes	 14
U+9121.0	fCangjie	HDNL
U+9121.0	fFourCorner	77927
U+9121.0	fHakka	 kiau1
U+9121.0	fCantonese	hiu1
U+9121.0	fMandarin	qiao1
U+9122.0	fUTF8	鄢
U+9122.0	fCNS11643	2-4E53
U+9122.0	fBig5	E46B
U+9122.0	fGB	0-5B33
U+9122.0	fR/S	 163.11
U+9122.0	fTotalStrokes	 14
U+9122.0	fCangjie	MFNL
U+9122.0	fFourCorner	17327
U+9122.0	fHakka	 jen2; ren2; jen3; ren3; jen1
U+9122.0	fCantonese	jin1
U+9122.0	fMandarin	yan1; yan3; yan4
U+9122.0	fEnglish	[1] an ancient state in today's Honan [2] a Chinese family name
U+9123.0	fUTF8	鄣
U+9123.0	fCNS11643	2-4E50
U+9123.0	fBig5	E468
U+9123.0	fGB	0-5B35
U+9123.0	fR/S	 163.11
U+9123.0	fTotalStrokes	 14
U+9123.0	fCangjie	YJNL
U+9123.0	fFourCorner	07427
U+9123.0	fHakka	 zhong1; zong1
U+9123.0	fCantonese	zoeng1; zoeng3
U+9123.0	fMandarin	zhang1; zhang4
U+9123.0	fEnglish	an ancient city in northern Jiangsu near Shandong
U+9124.0	fUTF8	鄤
U+9124.0	fCNS11643	2-4E57
U+9124.0	fBig5	E46F
U+9124.0	fGB	3-7138
U+9124.0	fR/S	 163.11
U+9124.0	fTotalStrokes	 14
U+9124.0	fCangjie	AENL
U+9124.0	fFourCorner	67427
U+9124.0	fHakka	 man5; man6
U+9124.0	fCantonese	maan6; maa6
U+9124.0	fMandarin	man4; fan4; wan4; man2
U+9124.0	fEnglish	a place in Henan province
U+9125.0	fUTF8	鄥
U+9125.0	fCNS11643	3-4A2A
U+9125.0	fGB	E-406A
U+9125.0	fR/S	  163.11
U+9125.0	fTotalStrokes	 14
U+9125.0	fCangjie	HFNL
U+9125.0	fFourCorner	27320
U+9125.0	fMandarin	qiao1
U+9126.0	fUTF8	鄦
U+9126.0	fCNS11643	2-5523
U+9126.0	fBig5	E859
U+9126.0	fGB	3-402D
U+9126.0	fR/S	 163.12
U+9126.0	fTotalStrokes	 15
U+9126.0	fCangjie	OFNL
U+9126.0	fFourCorner	87327
U+9126.0	fCantonese	heoi2
U+9126.0	fMandarin	xu3
U+9126.0	fEnglish	an ancient state in today's Honan
U+9127.0	fUTF8	鄧
U+9127.0	fCNS11643	1-6F52
U+9127.0	fBig5	BE48
U+9127.0	fGB	1-354B
U+9127.0	fR/S	 163.12
U+9127.0	fTotalStrokes	 15
U+9127.0	fCangjie	NTNL
U+9127.0	fFourCorner	17127
U+9127.0	fHakka	 ten6; ten3;  ten5
U+9127.0	fCantonese	dang6
U+9127.0	fMandarin	deng4
U+9127.0	fEnglish	[1] a Chinese family name [2] an ancient state in today's Honan
U+9128.0	fUTF8	鄨
U+9128.0	fCNS11643	2-6374
U+9128.0	fBig5	F14A
U+9128.0	fGB	3-705B
U+9128.0	fR/S	 163.12
U+9128.0	fTotalStrokes	 19
U+9128.0	fCangjie	FKRAU
U+9128.0	fFourCorner	97717
U+9128.0	fCantonese	bai3; bit3; bai6
U+9128.0	fMandarin	bi4; bie1
U+9129.0	fUTF8	鄩
U+9129.0	fCNS11643	2-547E
U+9129.0	fBig5	E856
U+9129.0	fGB	3-7075
U+9129.0	fR/S	 163.12
U+9129.0	fTotalStrokes	 15
U+9129.0	fCangjie	SINL
U+9129.0	fFourCorner	27327
U+9129.0	fHakka	 cim2
U+9129.0	fCantonese	cam4
U+9129.0	fMandarin	xin2; xun2
U+9129.0	fEnglish	a county in Shandong province
U+912A.0	fUTF8	鄪
U+912A.0	fCNS11643	2-5521
U+912A.0	fBig5	E857
U+912A.0	fGB	5-6F63
U+912A.0	fR/S	 163.12
U+912A.0	fTotalStrokes	 15
U+912A.0	fCangjie	LCNL
U+912A.0	fFourCorner	57227
U+912A.0	fCantonese	bei3
U+912A.0	fMandarin	bi4
U+912B.0	fUTF8	鄫
U+912B.0	fCNS11643	2-547D
U+912B.0	fBig5	E855
U+912B.0	fGB	3-7141
U+912B.0	fR/S	 163.12
U+912B.0	fTotalStrokes	 15
U+912B.0	fCangjie	CANL
U+912B.0	fFourCorner	87627
U+912B.0	fHakka	 zen1;  cin2
U+912B.0	fCantonese	cang4; kui2
U+912B.0	fMandarin	ceng2; zeng1
U+912B.0	fEnglish	a state in today's Shandong province
U+912C.0	fUTF8	鄬
U+912C.0	fCNS11643	2-4076
U+912C.0	fBig5	DC51
U+912C.0	fGB	3-7063
U+912C.0	fR/S	 163.12
U+912C.0	fTotalStrokes	 15
U+912C.0	fCangjie	IFNL
U+912C.0	fFourCorner	27327
U+912C.0	fCantonese	wai4
U+912C.0	fMandarin	wei2; hui1; wei3
U+912C.0	fEnglish	a place in Henan province
U+912D.0	fUTF8	鄭
U+912D.0	fCNS11643	1-6F51
U+912D.0	fBig5	BE47
U+912D.0	fGB	1-5623
U+912D.0	fR/S	 163.12
U+912D.0	fTotalStrokes	 15
U+912D.0	fCangjie	TKNL
U+912D.0	fFourCorner	87427
U+912D.0	fHakka	 ch'ang6; cang3; cang5; chang5; ch'ang5
U+912D.0	fCantonese	zeng6; zing6 (also); ze6
U+912D.0	fMandarin	zheng4
U+912D.0	fEnglish	[1] solemn; formal; serious [2] a Chinese family name [3] the name of an ancient state in today's Honan
U+912E.0	fUTF8	鄮
U+912E.0	fCNS11643	2-5524
U+912E.0	fBig5	E85A
U+912E.0	fGB	3-712D
U+912E.0	fR/S	 163.12
U+912E.0	fTotalStrokes	 15
U+912E.0	fCangjie	HCNL
U+912E.0	fFourCorner	77827
U+912E.0	fHakka	 meu6; meu5; meu3
U+912E.0	fCantonese	mau6
U+912E.0	fMandarin	mao4; mou4
U+912E.0	fEnglish	an old name for region about Ningpo
U+912F.0	fUTF8	鄯
U+912F.0	fCNS11643	2-547C
U+912F.0	fBig5	E854
U+912F.0	fGB	0-5B37
U+912F.0	fR/S	 163.12
U+912F.0	fTotalStrokes	 15
U+912F.0	fCangjie	TRNL
U+912F.0	fFourCorner	87627
U+912F.0	fCantonese	sin6
U+912F.0	fMandarin	shan4
U+912F.0	fEnglish	a region in Chinese Turkestan
U+9130.0	fUTF8	鄰
U+9130.0	fCNS11643	1-6F50
U+9130.0	fBig5	BE46
U+9130.0	fGB	1-415A
U+9130.0	fR/S	 163.12
U+9130.0	fTotalStrokes	 15
U+9130.0	fCangjie	FQNL
U+9130.0	fFourCorner	97227
U+9130.0	fHakka	  lin2
U+9130.0	fCantonese	leon4
U+9130.0	fMandarin	lin2; lin4
U+9130.0	fEnglish	[1] neighboring; adjacent; adjoining; contigious [2] neighborhood; community [3] neighbor [4] a basic community which consists of a number of families in the same neighborhood
U+9131.0	fUTF8	鄱
U+9131.0	fCNS11643	1-6F53
U+9131.0	fBig5	BE49
U+9131.0	fGB	0-5B36
U+9131.0	fR/S	 163.12
U+9131.0	fTotalStrokes	 15
U+9131.0	fCangjie	HWNL
U+9131.0	fFourCorner	27627
U+9131.0	fHakka	 po2
U+9131.0	fCantonese	po4; bo3 (also)
U+9131.0	fMandarin	po2; pan2; pi2
U+9131.0	fEnglish	a county in Kiangsi
U+9132.0	fUTF8	鄲
U+9132.0	fCNS11643	2-5522
U+9132.0	fBig5	E858
U+9132.0	fGB	1-3526
U+9132.0	fR/S	 163.12
U+9132.0	fTotalStrokes	 15
U+9132.0	fCangjie	RJNL
U+9132.0	fFourCorner	67527
U+9132.0	fHakka	 dan1
U+9132.0	fCantonese	daan1
U+9132.0	fMandarin	dan1; duo1
U+9132.0	fEnglish	used in combination for Hantan
U+9133.0	fUTF8	鄳
U+9133.0	fCNS11643	2-5B23
U+9133.0	fBig5	EBD5
U+9133.0	fGB	3-7522
U+9133.0	fR/S	 163.13
U+9133.0	fTotalStrokes	 16
U+9133.0	fCangjie	RUNL
U+9133.0	fFourCorner	77727
U+9133.0	fHakka	 mang2; men2
U+9133.0	fCantonese	maang4; maang5; mong4
U+9133.0	fMandarin	meng2; meng3; ming3
U+9133.0	fEnglish	an old name for part of Honam
U+9134.0	fUTF8	鄴
U+9134.0	fCNS11643	1-725D
U+9134.0	fBig5	BFF3
U+9134.0	fGB	1-5A7E
U+9134.0	fR/S	 163.13
U+9134.0	fTotalStrokes	 16
U+9134.0	fCangjie	TDNL
U+9134.0	fFourCorner	37927
U+9134.0	fHakka	 ngiap8; gniap8
U+9134.0	fCantonese	jip6
U+9134.0	fMandarin	ye4
U+9134.0	fEnglish	the ancient name of a part of what is today's Honan province
U+9135.0	fUTF8	鄵
U+9135.0	fCNS11643	2-5B24
U+9135.0	fBig5	EBD6
U+9135.0	fGB	3-7143
U+9135.0	fR/S	 163.13
U+9135.0	fTotalStrokes	 16
U+9135.0	fCangjie	RDNL
U+9135.0	fFourCorner	67927
U+9135.0	fHakka	 cau5
U+9135.0	fCantonese	cou3
U+9135.0	fMandarin	cao1; cao4; can1; qiao1; sao1
U+9135.0	fEnglish	an ancient city in the state of Ch'ang
U+9136.0	fUTF8	鄶
U+9136.0	fCNS11643	2-5B25
U+9136.0	fBig5	EBD7
U+9136.0	fGB	1-5B26
U+9136.0	fR/S	 163.13
U+9136.0	fTotalStrokes	 16
U+9136.0	fCangjie	OANL
U+9136.0	fFourCorner	87627
U+9136.0	fHakka	 kwai5
U+9136.0	fCantonese	kui3 (also); kwui3 (also); kui2; kwui2 (also)
U+9136.0	fMandarin	kuai4; gui4
U+9136.0	fEnglish	an ancient state in today's Honan province
U+9137.0	fUTF8	鄷
U+9137.0	fCNS11643	3-532B
U+9137.0	fGB	E-406B
U+9137.0	fR/S	 163.13
U+9137.0	fTotalStrokes	 16
U+9137.0	fCangjie	TTNL
U+9137.0	fFourCorner	57127
U+9137.0	fCantonese	fung1
U+9137.0	fMandarin	feng1
U+9138.0	fUTF8	鄸
U+9138.0	fCNS11643	2-5F70
U+9138.0	fBig5	EEC4
U+9138.0	fGB	3-7142
U+9138.0	fR/S	 163.14
U+9138.0	fTotalStrokes	 17
U+9138.0	fCangjie	TNNL
U+9138.0	fFourCorner	47227
U+9138.0	fCantonese	mung4; mung6
U+9138.0	fMandarin	meng2; meng4
U+9139.0	fUTF8	鄹
U+9139.0	fCNS11643	1-7568
U+9139.0	fBig5	C1DD
U+9139.0	fGB	0-5B38
U+9139.0	fR/S	 163.14
U+9139.0	fTotalStrokes	 17
U+9139.0	fCangjie	SONL
U+9139.0	fFourCorner	17227
U+9139.0	fHakka	  zeu1
U+9139.0	fCantonese	zau1
U+9139.0	fMandarin	zou1; ju4; suan4
U+9139.0	fEnglish	the birthplace of Confucius in today's Chuufu, Shantung
U+913A.0	fUTF8	鄺
U+913A.0	fCNS11643	2-6375
U+913A.0	fBig5	F14B
U+913A.0	fGB	1-5A77
U+913A.0	fR/S	 163.15
U+913A.0	fTotalStrokes	 18
U+913A.0	fCangjie	ICNL
U+913A.0	fFourCorner	07227
U+913A.0	fHakka	 gong3; kong5; kwong3
U+913A.0	fCantonese	kwong3; kong3 (also); kwong2 (also); kong2 (also)
U+913A.0	fMandarin	kuang4; guang3; kuo4; huang1
U+913A.0	fEnglish	a Chinese family name
U+913B.0	fUTF8	鄻
U+913B.0	fCNS11643	2-6376
U+913B.0	fBig5	F14C
U+913B.0	fGB	3-713E
U+913B.0	fR/S	 163.15
U+913B.0	fTotalStrokes	 18
U+913B.0	fCangjie	QJNL
U+913B.0	fFourCorner	57527
U+913B.0	fCantonese	lin5
U+913B.0	fMandarin	lian3; lian2
U+913B.0	fEnglish	spoil; ruin
U+913C.0	fUTF8	鄼
U+913C.0	fCNS11643	3-593C
U+913C.0	fGB	E-406C
U+913C.0	fR/S	 163.15
U+913C.0	fTotalStrokes	 18
U+913C.0	fCangjie	QCNL
U+913C.0	fFourCorner	57820
U+913C.0	fCantonese	zaan3; co4
U+913C.0	fMandarin	zan4; cuo2
U+913D.0	fUTF8	鄽
U+913D.0	fCNS11643	3-593B
U+913D.0	fGB	5-6F6F
U+913D.0	fR/S	 163.15
U+913D.0	fTotalStrokes	 18
U+913D.0	fCangjie	IGNL
U+913D.0	fFourCorner	07220
U+913D.0	fCantonese	cin4
U+913D.0	fMandarin	chan2
U+913E.0	fUTF8	鄾
U+913E.0	fCNS11643	2-6377
U+913E.0	fBig5	F14D
U+913E.0	fGB	3-7145
U+913E.0	fR/S	 163.15
U+913E.0	fTotalStrokes	 18
U+913E.0	fCangjie	MENL
U+913E.0	fFourCorner	12427
U+913E.0	fHakka	 ju2; riu2
U+913E.0	fCantonese	jau1
U+913E.0	fMandarin	you1
U+913E.0	fEnglish	a place in Hubei province
U+913F.0	fUTF8	鄿
U+913F.0	fCNS11643	2-6749
U+913F.0	fBig5	F35D
U+913F.0	fGB	5-6F67
U+913F.0	fR/S	 163.16
U+913F.0	fTotalStrokes	 19
U+913F.0	fCangjie	TJNL
U+913F.0	fFourCorner	47527
U+913F.0	fCantonese	gei1; kei4
U+913F.0	fMandarin	qi2
U+913F.0	fEnglish	a county in Xinjiang province
U+9140.0	fUTF8	酀
U+9140.0	fCNS11643	2-6748
U+9140.0	fBig5	F35C
U+9140.0	fGB	5-6F70
U+9140.0	fR/S	 163.16
U+9140.0	fTotalStrokes	 19
U+9140.0	fCangjie	TFNL
U+9140.0	fFourCorner	47327
U+9140.0	fCantonese	jin3
U+9140.0	fMandarin	yan1; yan3; yan4
U+9141.0	fUTF8	酁
U+9141.0	fCNS11643	2-6A30
U+9141.0	fBig5	F4E2
U+9141.0	fGB	E-406D
U+9141.0	fR/S	 163.17
U+9141.0	fTotalStrokes	 20
U+9141.0	fCangjie	NINL
U+9141.0	fFourCorner	27227
U+9141.0	fCantonese	caam4
U+9141.0	fMandarin	chan2
U+9142.0	fUTF8	酂
U+9142.0	fGB	8-7E6D
U+9142.0	fR/S	 163.16
U+9142.0	fTotalStrokes	 19
U+9142.0	fCangjie	HONL
U+9142.0	fCantonese	zaan3; co4; zyun2
U+9142.0	fMandarin	zan4; cuo2; da2
U+9142.0	fEnglish	collect; small administration
U+9143.0	fUTF8	酃
U+9143.0	fCNS11643	2-6A2F
U+9143.0	fBig5	F4E1
U+9143.0	fGB	0-5B39
U+9143.0	fR/S	 163.17
U+9143.0	fTotalStrokes	 20
U+9143.0	fCangjie	MRNL
U+9143.0	fFourCorner	17627
U+9143.0	fHakka	 lin2
U+9143.0	fCantonese	ling4
U+9143.0	fMandarin	ling2
U+9143.0	fEnglish	[1] a lake in Hunan whose water is used to bre wine [2] a county in Hunan
U+9144.0	fUTF8	酄
U+9144.0	fCNS11643	2-6C4A
U+9144.0	fBig5	F65B
U+9144.0	fGB	5-6F72
U+9144.0	fR/S	 163.18
U+9144.0	fTotalStrokes	 21
U+9144.0	fCangjie	TGNL
U+9144.0	fFourCorner	47227
U+9144.0	fCantonese	fun1; hyun1
U+9144.0	fMandarin	huan1; quan1
U+9145.0	fUTF8	酅
U+9145.0	fCNS11643	2-6C4B
U+9145.0	fBig5	F65C
U+9145.0	fGB	3-7147
U+9145.0	fR/S	 163.18
U+9145.0	fTotalStrokes	 21
U+9145.0	fCangjie	UBNL
U+9145.0	fFourCorner	27227
U+9145.0	fHakka	 hi1
U+9145.0	fCantonese	kwai4
U+9145.0	fMandarin	xi1; xi4
U+9145.0	fEnglish	 raise feet town in Shantung province
U+9146.0	fUTF8	酆
U+9146.0	fCNS11643	2-6C49
U+9146.0	fBig5	F65A
U+9146.0	fGB	0-5B3A
U+9146.0	fR/S	 163.18
U+9146.0	fTotalStrokes	 21
U+9146.0	fCangjie	UTNL
U+9146.0	fFourCorner	57127
U+9146.0	fHakka	 fung1
U+9146.0	fCantonese	fung1
U+9146.0	fMandarin	feng1
U+9146.0	fEnglish	[1] the capital of the Chou dynasty under King Wen [2] a Chinese family name
U+9147.0	fUTF8	酇
U+9147.0	fCNS11643	2-6E35
U+9147.0	fBig5	F766
U+9147.0	fGB	3-7146
U+9147.0	fR/S	 163.19
U+9147.0	fTotalStrokes	 22
U+9147.0	fCangjie	HCNL
U+9147.0	fFourCorner	27827
U+9147.0	fHakka	 zan3; zon5; zan5; zon1
U+9147.0	fCantonese	zaan3; co4; zyun2
U+9147.0	fMandarin	zan4; cuo2; da2; cuan2; zuan3
U+9147.0	fEnglish	[1] a community of a hundred families during the Chou dynasty [2] the name of a feudal state in the Han dynasty
U+9148.0	fUTF8	酈
U+9148.0	fCNS11643	1-7B7A
U+9148.0	fBig5	C5B0
U+9148.0	fGB	1-5B2A
U+9148.0	fR/S	 163.19
U+9148.0	fTotalStrokes	 22
U+9148.0	fCangjie	MPNL
U+9148.0	fFourCorner	17227
U+9148.0	fHakka	 li2;  li5; li6
U+9148.0	fCantonese	lik6
U+9148.0	fMandarin	li4; li2; zhi2
U+9148.0	fEnglish	[1] the ancient name for a part what is today's Honan province [2] a Chinese family name
U+9149.0	fUTF8	酉
U+9149.0	fCNS11643	1-4B5C
U+9149.0	fBig5	A8BB
U+9149.0	fGB	0-534F
U+9149.0	fR/S	 164.0
U+9149.0	fTotalStrokes	 7
U+9149.0	fCangjie	MCWM
U+9149.0	fFourCorner	10600
U+9149.0	fHakka	 jiu1;  ju1;  ju3; riu1; riu3; jiu3; ju2
U+9149.0	fCantonese	jau5
U+9149.0	fMandarin	you3
U+9149.0	fEnglish	[1] the tenth of the twelve Terrestrial Branches [2] 5:00-7:00 p.m. [3] Kangxi radical 164
U+914A.0	fUTF8	酊
U+914A.0	fCNS11643	1-536B
U+914A.0	fBig5	ADAA
U+914A.0	fGB	0-747A
U+914A.0	fR/S	 164.2
U+914A.0	fTotalStrokes	 9
U+914A.0	fCangjie	MWMN
U+914A.0	fFourCorner	11620
U+914A.0	fHakka	 din3; den1; dang1
U+914A.0	fCantonese	ding2; ding1
U+914A.0	fMandarin	ding1; ding3
U+914A.0	fEnglish	drunk; intoxicated
U+914B.0	fUTF8	酋
U+914B.0	fCNS11643	1-536A
U+914B.0	fBig5	ADA9
U+914B.0	fGB	0-4775
U+914B.0	fR/S	 164.2
U+914B.0	fTotalStrokes	 9
U+914B.0	fCangjie	TCWM
U+914B.0	fFourCorner	80601
U+914B.0	fHakka	 ju2;  siu2; {ciu2}; riu2; jiu2; ciu2
U+914B.0	fCantonese	jau4; cau4 (also)
U+914B.0	fMandarin	qiu2
U+914B.0	fEnglish	[1] chief of a clan, tribe; chieftain [2] end
U+914C.0	fUTF8	酌
U+914C.0	fCNS11643	1-585A
U+914C.0	fBig5	B075
U+914C.0	fGB	0-5743
U+914C.0	fR/S	 164.3
U+914C.0	fTotalStrokes	 10
U+914C.0	fCangjie	MWPI
U+914C.0	fFourCorner	17620
U+914C.0	fHakka	 zok7;  chok7; tsek7; zhok7
U+914C.0	fCantonese	zoek3
U+914C.0	fMandarin	zhuo2; shuo4
U+914C.0	fEnglish	[1] drink [2] pour wine; serve wine [3] weigh and consider
U+914D.0	fUTF8	配
U+914D.0	fCNS11643	1-5859
U+914D.0	fBig5	B074
U+914D.0	fGB	0-4564
U+914D.0	fR/S	 164.3
U+914D.0	fTotalStrokes	 10
U+914D.0	fCangjie	MWSU
U+914D.0	fFourCorner	17617
U+914D.0	fHakka	  pui5; poi5; pi5
U+914D.0	fCantonese	pui3
U+914D.0	fMandarin	pei4
U+914D.0	fEnglish	[1] join in marriage [2] mate [3] match; pair [4] fit; suit; be a match for; match; equal [5] dispense (medicines); prepare (according to a demand); distibute (according to a plan) [6] exile [7] spouse; partner [8] subordinate; supplementary; supporting; attached [9] deserve; be worthy; of; be qualified
U+914E.0	fUTF8	酎
U+914E.0	fCNS11643	2-3343
U+914E.0	fBig5	D440
U+914E.0	fGB	0-747C
U+914E.0	fR/S	 164.3
U+914E.0	fTotalStrokes	 10
U+914E.0	fCangjie	MWDI
U+914E.0	fFourCorner	14600
U+914E.0	fHakka	 ziu3; ciu3; ciu5
U+914E.0	fCantonese	zau6
U+914E.0	fMandarin	zhou4
U+914E.0	fEnglish	double-fermented wine; vintage w
U+914F.0	fUTF8	酏
U+914F.0	fCNS11643	2-3344
U+914F.0	fBig5	D441
U+914F.0	fGB	0-747D
U+914F.0	fR/S	 164.3
U+914F.0	fTotalStrokes	 10
U+914F.0	fCangjie	MWPD
U+914F.0	fFourCorner	14612
U+914F.0	fHakka	 ji2; ri2
U+914F.0	fCantonese	ji4; ji5
U+914F.0	fMandarin	yi2; yi3; hu4; tuo2
U+914F.0	fEnglish	millet wine
U+9150.0	fUTF8	酐
U+9150.0	fCNS11643	2-3342
U+9150.0	fBig5	D3FE
U+9150.0	fGB	0-747B
U+9150.0	fR/S	 164.3
U+9150.0	fTotalStrokes	 10
U+9150.0	fCangjie	MWMJ
U+9150.0	fFourCorner	11640
U+9150.0	fHakka	 hang3; gon1
U+9150.0	fCantonese	gon1; hong5
U+9150.0	fMandarin	hang4; gan1; hang3; han2
U+9151.0	fUTF8	酑
U+9151.0	fCNS11643	3-342A
U+9151.0	fGB	E-406E
U+9151.0	fR/S	 164.3
U+9151.0	fTotalStrokes	 10
U+9151.0	fCangjie	MWMD
U+9151.0	fFourCorner	11640
U+9151.0	fCantonese	jyu1
U+9151.0	fMandarin	yu3
U+9152.0	fUTF8	酒
U+9152.0	fCNS11643	1-5858
U+9152.0	fBig5	B073
U+9152.0	fGB	0-3E46
U+9152.0	fR/S	 164.3
U+9152.0	fTotalStrokes	 10
U+9152.0	fCangjie	EMCW
U+9152.0	fFourCorner	31160
U+9152.0	fHakka	  ziu3; tsieu3
U+9152.0	fCantonese	zau2
U+9152.0	fMandarin	jiu3
U+9152.0	fEnglish	 wine; spirits; liquor; alcoholic drinks (brewed or distilled)
U+9153.0	fUTF8	酓
U+9153.0	fCNS11643	2-3978
U+9153.0	fBig5	D7F5
U+9153.0	fGB	3-6E53
U+9153.0	fR/S	 164.4
U+9153.0	fTotalStrokes	 11
U+9153.0	fCangjie	OINW
U+9153.0	fFourCorner	80601
U+9153.0	fCantonese	jim2; jam2; jim3
U+9153.0	fMandarin	yan3; han1; tan2; yan4
U+9154.0	fUTF8	酔
U+9154.0	fCNS11643	3-3955
U+9154.0	fGB	E-406F
U+9154.0	fR/S	 164.4
U+9154.0	fTotalStrokes	 11
U+9154.0	fCangjie	MWKNJ
U+9154.0	fFourCorner	14640
U+9154.0	fCantonese	zeoi3
U+9154.0	fMandarin	zui4
U+9154.0	fEnglish	 intoxicated; drunk addicted to
U+9155.0	fUTF8	酕
U+9155.0	fCNS11643	2-3979
U+9155.0	fBig5	D7F6
U+9155.0	fGB	3-6E52
U+9155.0	fR/S	 164.4
U+9155.0	fTotalStrokes	 11
U+9155.0	fCangjie	MWHQU
U+9155.0	fFourCorner	12614
U+9155.0	fHakka	 mau2; mau3
U+9155.0	fCantonese	mou4
U+9155.0	fMandarin	mao2; mao1
U+9155.0	fEnglish	drunk
U+9156.0	fUTF8	酖
U+9156.0	fCNS11643	2-3975
U+9156.0	fBig5	D7F2
U+9156.0	fGB	E-4070
U+9156.0	fR/S	 164.4
U+9156.0	fTotalStrokes	 11
U+9156.0	fCangjie	MWLBU
U+9156.0	fFourCorner	14612
U+9156.0	fHakka	  dam1; chim2; cim3; cihm2; cim2; tam1
U+9156.0	fCantonese	daam1; zam6
U+9156.0	fMandarin	dan1
U+9156.0	fEnglish	addicted to alcoholic drinks
U+9156.1	fUTF8	酖
U+9156.1	fCNS11643	2-3975
U+9156.1	fBig5	D7F2
U+9156.1	fGB	E-4070
U+9156.1	fR/S	 164.4
U+9156.1	fTotalStrokes	 11
U+9156.1	fCangjie	MWLBU
U+9156.1	fFourCorner	14612
U+9156.1	fMandarin	zhen4
U+9156.1	fEnglish	 wine with poison added; poisonous wine
U+9157.0	fUTF8	酗
U+9157.0	fCNS11643	1-5D68
U+9157.0	fBig5	B3A4
U+9157.0	fGB	0-506F
U+9157.0	fR/S	 164.4
U+9157.0	fTotalStrokes	 11
U+9157.0	fCangjie	MWUK
U+9157.0	fFourCorner	12670
U+9157.0	fHakka	  hi5;  hiung1
U+9157.0	fCantonese	jyu3; heoi3 (also); heoi2 (also)
U+9157.0	fMandarin	xu4
U+9157.0	fEnglish	 lose one's temper when drunk; become violent under the influence of alcohol
U+9158.0	fUTF8	酘
U+9158.0	fCNS11643	2-3976
U+9158.0	fBig5	D7F3
U+9158.0	fGB	3-6E55
U+9158.0	fR/S	 164.4
U+9158.0	fTotalStrokes	 11
U+9158.0	fCangjie	MWHNE
U+9158.0	fFourCorner	17647
U+9158.0	fHakka	  teu2; teu5
U+9158.0	fCantonese	tau4; cau4
U+9158.0	fMandarin	tou2; dou4
U+9158.0	fEnglish	[1] cover [2] above
U+9159.0	fUTF8	酙
U+9159.0	fCNS11643	3-3954
U+9159.0	fGB	E-4071
U+9159.0	fR/S	 164.4
U+9159.0	fTotalStrokes	 11
U+9159.0	fCangjie	MWYJ
U+9159.0	fFourCorner	14600
U+9159.0	fCantonese	zam1
U+9159.0	fMandarin	zhen1
U+915A.0	fUTF8	酚
U+915A.0	fCNS11643	2-3977
U+915A.0	fBig5	D7F4
U+915A.0	fGB	0-3753
U+915A.0	fR/S	 164.4
U+915A.0	fTotalStrokes	 11
U+915A.0	fCangjie	MWCSH
U+915A.0	fFourCorner	18627
U+915A.0	fHakka	 fun1
U+915A.0	fCantonese	fan1
U+915A.0	fMandarin	fen1
U+915A.0	fEnglish	 carbolic acid phenol
U+915B.0	fUTF8	酛
U+915B.0	fGB	E-4072
U+915B.0	fR/S	 164.4
U+915B.0	fTotalStrokes	 11
U+915B.0	fCangjie	MWMMU
U+915B.0	fCantonese	jyun4
U+915C.0	fUTF8	酜
U+915C.0	fCNS11643	3-6657
U+915C.0	fGB	E-4073
U+915C.0	fR/S	 164.4
U+915C.0	fTotalStrokes	 11
U+915C.0	fCangjie	MWQO
U+915C.0	fFourCorner	1563
U+915C.0	fCantonese	fu1
U+915D.0	fUTF8	酝
U+915D.0	fGB	0-544D
U+915D.0	fR/S	 164.4
U+915D.0	fTotalStrokes	 11
U+915D.0	fCangjie	MWMMI
U+915D.0	fCantonese	wan3 (also); wan5
U+915D.0	fMandarin	yun4
U+915D.0	fEnglish	simplified form of &#x9196;
U+915E.0	fUTF8	酞
U+915E.0	fGB	0-4C2A
U+915E.0	fR/S	 164.4
U+915E.0	fTotalStrokes	 11
U+915E.0	fCangjie	MWKI
U+915E.0	fCantonese	taai3; tai3
U+915E.0	fMandarin	tai4
U+915E.0	fEnglish	phthalein
U+915F.0	fUTF8	酟
U+915F.0	fCNS11643	2-4126
U+915F.0	fBig5	DC5F
U+915F.0	fGB	3-6E58
U+915F.0	fR/S	 164.5
U+915F.0	fTotalStrokes	 12
U+915F.0	fCangjie	MWYR
U+915F.0	fFourCorner	11660
U+915F.0	fCantonese	tim1; cim1
U+915F.0	fMandarin	tian1
U+9160.0	fUTF8	酠
U+9160.0	fCNS11643	2-4128
U+9160.0	fBig5	DC61
U+9160.0	fGB	E-4074
U+9160.0	fR/S	 164.5
U+9160.0	fTotalStrokes	 12
U+9160.0	fCangjie	MWMNR
U+9160.0	fFourCorner	17620
U+9160.0	fCantonese	kaa2; gaa3
U+9160.0	fMandarin	qia3
U+9161.0	fUTF8	酡
U+9161.0	fCNS11643	2-4124
U+9161.0	fBig5	DC5D
U+9161.0	fGB	0-7522
U+9161.0	fR/S	 164.5
U+9161.0	fTotalStrokes	 12
U+9161.0	fCangjie	MWJP
U+9161.0	fFourCorner	13611
U+9161.0	fHakka	 to2
U+9161.0	fCantonese	to4
U+9161.0	fMandarin	tuo2; duo4
U+9161.0	fEnglish	 flushed rubicund
U+9162.0	fUTF8	酢
U+9162.0	fCNS11643	2-4127
U+9162.0	fBig5	DC60
U+9162.0	fGB	0-7521
U+9162.0	fR/S	 164.5
U+9162.0	fTotalStrokes	 12
U+9162.0	fCangjie	MWOS
U+9162.0	fFourCorner	18611
U+9162.0	fHakka	 cok7;  cu5; cih5
U+9162.0	fCantonese	zok6; cou3; zaa3
U+9162.0	fMandarin	zuo4; cu4
U+9162.0	fEnglish	 toast one's host with (said of a guest)
U+9163.0	fUTF8	酣
U+9163.0	fCNS11643	1-6256
U+9163.0	fBig5	B66F
U+9163.0	fGB	0-3A28
U+9163.0	fR/S	 164.5
U+9163.0	fTotalStrokes	 12
U+9163.0	fCangjie	MWTM
U+9163.0	fFourCorner	14670
U+9163.0	fHakka	 tiam2; ham2; gam1
U+9163.0	fCantonese	ham4
U+9163.0	fMandarin	han1; han2; han4
U+9163.0	fEnglish	[1] enjoy intoxicants; drink wine [2] to one's hear's content; as much as one wants; without inhibition
U+9164.0	fUTF8	酤
U+9164.0	fCNS11643	2-4125
U+9164.0	fBig5	DC5E
U+9164.0	fGB	0-747E
U+9164.0	fR/S	 164.5
U+9164.0	fTotalStrokes	 12
U+9164.0	fCangjie	MWJR
U+9164.0	fFourCorner	14660
U+9164.0	fHakka	  gu1
U+9164.0	fCantonese	gu1; gwu1 (also); gu3; wu6
U+9164.0	fMandarin	gu1; gu4; hu4
U+9164.0	fEnglish	 deal in spirits; sell or buy alcoholic drinks; sell or buy spirits
U+9165.0	fUTF8	酥
U+9165.0	fCNS11643	1-6257
U+9165.0	fBig5	B670
U+9165.0	fGB	0-4B56
U+9165.0	fR/S	 164.5
U+9165.0	fTotalStrokes	 12
U+9165.0	fCangjie	MWHD
U+9165.0	fFourCorner	12694
U+9165.0	fHakka	  su1; se1; sii1
U+9165.0	fCantonese	sou1
U+9165.0	fMandarin	su1
U+9165.0	fEnglish	[1] brittle; frail; fragile [2] crunchy; crispy [3] lustrous
U+9166.0	fUTF8	酦
U+9166.0	fGB	7-2131
U+9166.0	fR/S	 164.6
U+9166.0	fTotalStrokes	 13
U+9166.0	fCangjie	MWIVE
U+9166.0	fCantonese	put3
U+9166.0	fMandarin	po1; po4; fa1
U+9166.0	fEnglish	fermentation, brewing
U+9167.0	fUTF8	酧
U+9167.0	fCNS11643	3-4528
U+9167.0	fGB	E-4075
U+9167.0	fR/S	 164.6; 146.6
U+9167.0	fTotalStrokes	 13; 12
U+9167.0	fCangjie	MWJDI
U+9167.0	fFourCorner	13642
U+9167.0	fHakka	 chiu2; cu2; chu2
U+9167.0	fCantonese	cau4
U+9167.0	fMandarin	chou2
U+9167.0	fEnglish	 pledge with wine
U+9168.0	fUTF8	酨
U+9168.0	fCNS11643	2-4279
U+9168.0	fBig5	DD73
U+9168.0	fGB	3-4634
U+9168.0	fR/S	 164.6
U+9168.0	fTotalStrokes	 13
U+9168.0	fCangjie	JIMCW
U+9168.0	fFourCorner	43650
U+9168.0	fCantonese	doi6; zoi6 (also)
U+9168.0	fMandarin	zai4; dai4; zui4
U+9169.0	fUTF8	酩
U+9169.0	fCNS11643	1-673D
U+9169.0	fBig5	B955
U+9169.0	fGB	0-7524
U+9169.0	fR/S	 164.6
U+9169.0	fTotalStrokes	 13
U+9169.0	fCangjie	MWNIR
U+9169.0	fFourCorner	17660
U+9169.0	fHakka	  men3; miang2; min2
U+9169.0	fCantonese	ming5; ming4 (also)
U+9169.0	fMandarin	ming3; ming2
U+9169.0	fEnglish	drunk; intoxicated; tipsy; inebriate; enebriety
U+916A.0	fUTF8	酪
U+916A.0	fCNS11643	1-673C
U+916A.0	fBig5	B954
U+916A.0	fGB	0-4052
U+916A.0	fR/S	 164.6
U+916A.0	fTotalStrokes	 13
U+916A.0	fCangjie	MWHER
U+916A.0	fFourCorner	17664
U+916A.0	fHakka	  lok8; lok7
U+916A.0	fCantonese	lok3; lou6 (also); lok6
U+916A.0	fMandarin	lao4; luo4; lu4
U+916A.0	fEnglish	[1] alcoholic drinks [2] animal milk [3] cheese [4] fruit jam [5] junket [6] thick fruit juice; fruit jelly
U+916B.0	fUTF8	酫
U+916B.0	fCNS11643	3-4527
U+916B.0	fGB	E-4076
U+916B.0	fR/S	 164.6
U+916B.0	fTotalStrokes	 13
U+916B.0	fCangjie	MWOMG
U+916B.0	fFourCorner	18610
U+916B.0	fCantonese	cyut3
U+916B.0	fMandarin	chuo4
U+916C.0	fUTF8	酬
U+916C.0	fCNS11643	1-673B
U+916C.0	fBig5	B953
U+916C.0	fGB	0-336A
U+916C.0	fR/S	 164.6
U+916C.0	fTotalStrokes	 13
U+916C.0	fCangjie	MWILL
U+916C.0	fFourCorner	12600
U+916C.0	fHakka	 cu2;  chiu2; chiu3; siu2; ciu2; chu2
U+916C.0	fCantonese	cau4
U+916C.0	fMandarin	chou2
U+916C.0	fEnglish	[1] toast; offer or present a cup of spirits [2] reward; requite; reciprocate; reward [3] fulfill; realise
U+916D.0	fUTF8	酭
U+916D.0	fCNS11643	4-413A
U+916D.0	fGB	5-6E24
U+916D.0	fR/S	 164.6
U+916D.0	fTotalStrokes	 13
U+916D.0	fCangjie	MWKB
U+916D.0	fFourCorner	14620
U+916D.0	fHakka	 ju5
U+916D.0	fCantonese	jau6
U+916D.0	fMandarin	you4
U+916E.0	fUTF8	酮
U+916E.0	fCNS11643	2-4832
U+916E.0	fBig5	E0AC
U+916E.0	fGB	0-4D2A
U+916E.0	fR/S	 164.6
U+916E.0	fTotalStrokes	 13
U+916E.0	fCangjie	MWBMR
U+916E.0	fFourCorner	17620
U+916E.0	fHakka	 tung2
U+916E.0	fCantonese	tung4
U+916E.0	fMandarin	tong2; chong2; dong4
U+916E.0	fEnglish	ketones
U+916F.0	fUTF8	酯
U+916F.0	fCNS11643	2-4833
U+916F.0	fBig5	E0AD
U+916F.0	fGB	0-7525
U+916F.0	fR/S	 164.6
U+916F.0	fTotalStrokes	 13
U+916F.0	fCangjie	MWPA
U+916F.0	fFourCorner	11681
U+916F.0	fHakka	 zi3
U+916F.0	fCantonese	zi2
U+916F.0	fMandarin	zhi3
U+916F.0	fEnglish	esters
U+9170.0	fUTF8	酰
U+9170.0	fCNS11643	4-413B
U+9170.0	fGB	0-7523
U+9170.0	fR/S	 164.6
U+9170.0	fTotalStrokes	 13
U+9170.0	fCangjie	MWHGU
U+9170.0	fFourCorner	14610
U+9170.0	fCantonese	sin1
U+9170.0	fMandarin	xian1
U+9170.0	fEnglish	acyl
U+9171.0	fUTF8	酱
U+9171.0	fGB	0-3D34
U+9171.0	fR/S	 164.6
U+9171.0	fTotalStrokes	 13
U+9171.0	fCangjie	LNMCW
U+9171.0	fHakka	 ziong5
U+9171.0	fCantonese	zoeng3
U+9171.0	fMandarin	jiang4
U+9171.0	fEnglish	simplified form of &#x91AC;
U+9172.0	fUTF8	酲
U+9172.0	fCNS11643	2-4E5B
U+9172.0	fBig5	E473
U+9172.0	fGB	0-7528
U+9172.0	fR/S	 164.7
U+9172.0	fTotalStrokes	 14
U+9172.0	fCangjie	MWRHG
U+9172.0	fFourCorner	16614
U+9172.0	fHakka	 chin2; cihn2; cin2
U+9172.0	fCantonese	cing4
U+9172.0	fMandarin	cheng2; cheng1
U+9172.0	fEnglish	 hangover after heavy drinking; be intoxicated; stupefied by drinking
U+9173.0	fUTF8	酳
U+9173.0	fCNS11643	2-4E5D
U+9173.0	fBig5	E475
U+9173.0	fGB	3-6E64
U+9173.0	fR/S	 164.7
U+9173.0	fTotalStrokes	 14
U+9173.0	fCangjie	MWVIB
U+9173.0	fFourCorner	12627
U+9173.0	fHakka	 jin3; rin3
U+9173.0	fCantonese	jan6; jan2; seon3
U+9173.0	fMandarin	yin4; zhen4
U+9173.0	fEnglish	 rinse out the mouth with wine
U+9174.0	fUTF8	酴
U+9174.0	fCNS11643	1-6B4E
U+9174.0	fBig5	BBC6
U+9174.0	fGB	0-7529
U+9174.0	fR/S	 164.7
U+9174.0	fTotalStrokes	 14
U+9174.0	fCangjie	MWOMD
U+9174.0	fFourCorner	18694
U+9174.0	fHakka	 tu2
U+9174.0	fCantonese	tou4
U+9174.0	fMandarin	tu2
U+9174.0	fEnglish	[1] distiller's leaven or yeast [2] wine brewed for the second time
U+9175.0	fUTF8	酵
U+9175.0	fCNS11643	1-6B4B
U+9175.0	fBig5	BBC3
U+9175.0	fGB	0-3D4D
U+9175.0	fR/S	 164.7
U+9175.0	fTotalStrokes	 14
U+9175.0	fCangjie	MWJKD
U+9175.0	fFourCorner	14647
U+9175.0	fHakka	 gau5; hau5
U+9175.0	fCantonese	haau1; gaau3 (also); haau3
U+9175.0	fMandarin	jiao4; xiao4
U+9175.0	fEnglish	yeast; leaven
U+9176.0	fUTF8	酶
U+9176.0	fCNS11643	3-4A2C
U+9176.0	fGB	0-4338
U+9176.0	fR/S	 164.7
U+9176.0	fTotalStrokes	 14
U+9176.0	fCangjie	MWOWY
U+9176.0	fFourCorner	18650
U+9176.0	fHakka	  moi2; mi2; mui2; moe2
U+9176.0	fCantonese	mui4
U+9176.0	fMandarin	mei2
U+9176.0	fEnglish	enzymes
U+9177.0	fUTF8	酷
U+9177.0	fCNS11643	1-6B4D
U+9177.0	fBig5	BBC5
U+9177.0	fGB	0-3F61
U+9177.0	fR/S	 164.7
U+9177.0	fTotalStrokes	 14
U+9177.0	fCangjie	MWHGR
U+9177.0	fFourCorner	14661
U+9177.0	fHakka	 kuk7; kiok7; kok7; k'wuk7; guk7
U+9177.0	fCantonese	huk6
U+9177.0	fMandarin	ku4; he4
U+9177.0	fEnglish	[1] strong (intoxicants) [2] very stimulating (fragrance) [3] cruel; relentless; ruthless; brutal; severe; harsh [4] exceedingly
U+9178.0	fUTF8	酸
U+9178.0	fCNS11643	1-6B4C
U+9178.0	fBig5	BBC4
U+9178.0	fGB	0-4B61
U+9178.0	fR/S	 164.7
U+9178.0	fTotalStrokes	 14
U+9178.0	fCangjie	MWICE
U+9178.0	fFourCorner	13647
U+9178.0	fHakka	  son1
U+9178.0	fCantonese	syun1
U+9178.0	fMandarin	suan1
U+9178.0	fEnglish	[1] sour; acid [2] stale; spoiled [3] sad; grieved; sorrowful [4] aching; tingle; ache [5] jealous; envious [6] stingy [7] acid (chemistry)
U+9179.0	fUTF8	酹
U+9179.0	fCNS11643	2-4E5C
U+9179.0	fBig5	E474
U+9179.0	fGB	0-752A
U+9179.0	fR/S	 164.7
U+9179.0	fTotalStrokes	 14
U+9179.0	fCangjie	MWBDI
U+9179.0	fFourCorner	12643
U+9179.0	fHakka	  lot7; lai5; lui1
U+9179.0	fCantonese	lyut3; laai6 (also); lyut6 (also); leoi6; loi6
U+9179.0	fMandarin	lei4; le4
U+9179.0	fEnglish	 make a libation
U+917A.0	fUTF8	酺
U+917A.0	fCNS11643	2-4E5A
U+917A.0	fBig5	E472
U+917A.0	fGB	3-6E61
U+917A.0	fR/S	 164.7
U+917A.0	fTotalStrokes	 14
U+917A.0	fCangjie	MWIJB
U+917A.0	fFourCorner	13627
U+917A.0	fHakka	 pu2
U+917A.0	fCantonese	pou4
U+917A.0	fMandarin	pu2; bu4
U+917A.0	fEnglish	 drink with others; drink in company
U+917B.0	fUTF8	酻
U+917B.0	fCNS11643	3-4A2B
U+917B.0	fGB	E-4077
U+917B.0	fR/S	 164.7
U+917B.0	fTotalStrokes	 14
U+917B.0	fCangjie	MWBND
U+917B.0	fFourCorner	12640
U+917B.0	fCantonese	zeoi1
U+917B.0	fMandarin	zui4
U+917C.0	fUTF8	酼
U+917C.0	fCNS11643	3-4A2D
U+917C.0	fGB	E-4078
U+917C.0	fR/S	 164.7; 164.6; 146.6
U+917C.0	fTotalStrokes	 14; 12
U+917C.0	fCangjie	MWYIU
U+917C.0	fFourCorner	10613
U+917C.0	fCantonese	hoi2
U+917C.0	fMandarin	hai3
U+917D.0	fUTF8	酽
U+917D.0	fGB	0-7526
U+917D.0	fR/S	 164.7
U+917D.0	fTotalStrokes	 14
U+917D.0	fCangjie	MWMTH
U+917D.0	fHakka	 ngiam2
U+917D.0	fCantonese	jim6
U+917D.0	fMandarin	yan4
U+917D.0	fEnglish	simplified form of &#x91C5;
U+917E.0	fUTF8	酾
U+917E.0	fGB	0-7527
U+917E.0	fR/S	 164.7
U+917E.0	fTotalStrokes	 14
U+917E.0	fCangjie	XMWMB
U+917E.0	fHakka	 sii1
U+917E.0	fCantonese	si1
U+917E.0	fMandarin	xi3; li2; shi1; shuai1; shai1
U+917E.0	fEnglish	simplified form of &#x91C3;
U+917F.0	fUTF8	酿
U+917F.0	fGB	0-4470
U+917F.0	fR/S	 164.7
U+917F.0	fTotalStrokes	 14
U+917F.0	fCangjie	MWIAV
U+917F.0	fHakka	 jong5
U+917F.0	fCantonese	joeng6; joeng2 (also)
U+917F.0	fMandarin	niang4; niang2
U+917F.0	fEnglish	simplified form of &#x91C0;
U+9180.0	fUTF8	醀
U+9180.0	fCNS11643	2-552B
U+9180.0	fBig5	E861
U+9180.0	fGB	3-6E69
U+9180.0	fR/S	 164.8
U+9180.0	fTotalStrokes	 15
U+9180.0	fCangjie	MWOG
U+9180.0	fFourCorner	10614
U+9180.0	fCantonese	wai4; zeoi3
U+9180.0	fMandarin	wei2; zhui4
U+9181.0	fUTF8	醁
U+9181.0	fCNS11643	2-5528
U+9181.0	fBig5	E85E
U+9181.0	fGB	3-6E6C
U+9181.0	fR/S	 164.8
U+9181.0	fTotalStrokes	 15
U+9181.0	fCangjie	MWVNE
U+9181.0	fFourCorner	17632
U+9181.0	fHakka	 liuk8
U+9181.0	fCantonese	luk6
U+9181.0	fMandarin	lu4
U+9181.0	fEnglish	a kind of green-colored wine
U+9182.0	fUTF8	醂
U+9182.0	fCNS11643	2-5529
U+9182.0	fBig5	E85F
U+9182.0	fGB	3-6E67
U+9182.0	fR/S	 164.8
U+9182.0	fTotalStrokes	 15
U+9182.0	fCangjie	MWDD
U+9182.0	fFourCorner	14690
U+9182.0	fCantonese	lam5
U+9182.0	fMandarin	lan3
U+9182.0	fEnglish	to remove astringency; to bleach in water
U+9183.0	fUTF8	醃
U+9183.0	fCNS11643	1-6F57
U+9183.0	fBig5	BE4D
U+9183.0	fGB	E-4079
U+9183.0	fR/S	 164.8
U+9183.0	fTotalStrokes	 15
U+9183.0	fCangjie	MWKLU
U+9183.0	fFourCorner	14617
U+9183.0	fHakka	  jam1;  jam3; ram1; am1; jan1
U+9183.0	fCantonese	jim1; jip3 (also)
U+9183.0	fMandarin	yan1
U+9183.0	fEnglish	 marinate; pickle; salt
U+9184.0	fUTF8	醄
U+9184.0	fCNS11643	2-552A
U+9184.0	fBig5	E860
U+9184.0	fGB	3-6E6A
U+9184.0	fR/S	 164.8
U+9184.0	fTotalStrokes	 15
U+9184.0	fCangjie	MWPOU
U+9184.0	fFourCorner	17620
U+9184.0	fHakka	  tau2; to2
U+9184.0	fCantonese	tou4
U+9184.0	fMandarin	tao2
U+9184.0	fEnglish	drunk
U+9185.0	fUTF8	醅
U+9185.0	fCNS11643	2-5525
U+9185.0	fBig5	E85B
U+9185.0	fGB	0-752C
U+9185.0	fR/S	 164.8
U+9185.0	fTotalStrokes	 15
U+9185.0	fCangjie	MWYTR
U+9185.0	fFourCorner	10661
U+9185.0	fHakka	 pi2; poi2; pui2
U+9185.0	fCantonese	pui1
U+9185.0	fMandarin	pei1; pou1; pou3
U+9185.0	fEnglish	unstrained spirits; unstrained wine
U+9186.0	fUTF8	醆
U+9186.0	fCNS11643	2-5526
U+9186.0	fBig5	E85C
U+9186.0	fGB	E-407A
U+9186.0	fR/S	 164.8
U+9186.0	fTotalStrokes	 15
U+9186.0	fCangjie	MWII
U+9186.0	fFourCorner	13653
U+9186.0	fHakka	 zan3
U+9186.0	fCantonese	zaan2; zaan1
U+9186.0	fMandarin	zhan3
U+9186.0	fEnglish	wine cup
U+9187.0	fUTF8	醇
U+9187.0	fCNS11643	1-6F54
U+9187.0	fBig5	BE4A
U+9187.0	fGB	0-343C
U+9187.0	fR/S	 164.8
U+9187.0	fTotalStrokes	 15
U+9187.0	fCangjie	MWYRD
U+9187.0	fFourCorner	10647
U+9187.0	fHakka	 sun2;  shun2
U+9187.0	fCantonese	seon4
U+9187.0	fMandarin	chun2
U+9187.0	fEnglish	[1] rich wine; strong wine [2] pure; unadulterated [3] gentle; gracious; judicious [4] ethyl alcohol
U+9188.0	fUTF8	醈
U+9188.0	fCNS11643	3-4F3F
U+9188.0	fGB	5-6E2A
U+9188.0	fR/S	 164.8
U+9188.0	fTotalStrokes	 15
U+9188.0	fCangjie	MWFF
U+9188.0	fFourCorner	19680
U+9188.0	fCantonese	taam4
U+9188.0	fMandarin	tan2
U+9189.0	fUTF8	醉
U+9189.0	fCNS11643	1-6F55
U+9189.0	fBig5	BE4B
U+9189.0	fGB	0-576D
U+9189.0	fR/S	 164.8
U+9189.0	fTotalStrokes	 15
U+9189.0	fCangjie	MWYOJ
U+9189.0	fFourCorner	10648
U+9189.0	fHakka	  zui5
U+9189.0	fCantonese	zeoi3
U+9189.0	fMandarin	zui4
U+9189.0	fEnglish	[1] intoxicated; drunk; tipsy [2] infatuated; charmed
U+918A.0	fUTF8	醊
U+918A.0	fCNS11643	2-5527
U+918A.0	fBig5	E85D
U+918A.0	fGB	3-6E6E
U+918A.0	fR/S	 164.8
U+918A.0	fTotalStrokes	 15
U+918A.0	fCangjie	MWEEE
U+918A.0	fFourCorner	17647
U+918A.0	fHakka	  dot7; ch'ot7; cot7; chot7
U+918A.0	fCantonese	zyut3; zeoi3 (also)
U+918A.0	fMandarin	chuo4; zhui4; zhui1; zhuo2
U+918A.0	fEnglish	 pour wine in a libation
U+918B.0	fUTF8	醋
U+918B.0	fCNS11643	1-6F56
U+918B.0	fBig5	BE4C
U+918B.0	fGB	0-3457
U+918B.0	fR/S	 164.8
U+918B.0	fTotalStrokes	 15
U+918B.0	fCangjie	MWTA
U+918B.0	fFourCorner	14661
U+918B.0	fHakka	  cu5; sii5; cih5; chih5; ze5
U+918B.0	fCantonese	cou3; zok6
U+918B.0	fMandarin	cu4; zuo4
U+918B.0	fEnglish	vinegar
U+918C.0	fUTF8	醌
U+918C.0	fCNS11643	3-4F40
U+918C.0	fGB	0-752B
U+918C.0	fR/S	 164.8
U+918C.0	fTotalStrokes	 15
U+918C.0	fCangjie	MWAPP
U+918C.0	fFourCorner	16610
U+918C.0	fCantonese	kwan1
U+918C.0	fMandarin	kun1
U+918D.0	fUTF8	醍
U+918D.0	fCNS11643	2-5B29
U+918D.0	fBig5	EBDB
U+918D.0	fGB	0-752E
U+918D.0	fR/S	 164.9
U+918D.0	fTotalStrokes	 16
U+918D.0	fCangjie	MWAMO
U+918D.0	fFourCorner	16681
U+918D.0	fHakka	 ti2;  ti3
U+918D.0	fCantonese	tai4; tai2
U+918D.0	fMandarin	ti2; ti3
U+918D.0	fEnglish	[1] cream (of milk) [2] a kind of reddish wine
U+918E.0	fUTF8	醎
U+918E.0	fCNS11643	3-532D
U+918E.0	fGB	5-6E2D
U+918E.0	fR/S	 164.9
U+918E.0	fTotalStrokes	 16
U+918E.0	fCangjie	MWIHR
U+918E.0	fFourCorner	13650
U+918E.0	fCantonese	haam4
U+918E.0	fMandarin	mian2; xian2; jian3
U+918F.0	fUTF8	醏
U+918F.0	fCNS11643	2-5B2A
U+918F.0	fBig5	EBDC
U+918F.0	fGB	E-407B
U+918F.0	fR/S	 164.9
U+918F.0	fTotalStrokes	 16
U+918F.0	fCangjie	MWJKA
U+918F.0	fFourCorner	14660
U+918F.0	fCantonese	dou1
U+918F.0	fMandarin	du1
U+9190.0	fUTF8	醐
U+9190.0	fCNS11643	2-5B27
U+9190.0	fBig5	EBD9
U+9190.0	fGB	0-752D
U+9190.0	fR/S	 164.9
U+9190.0	fTotalStrokes	 16
U+9190.0	fCangjie	MWJRB
U+9190.0	fFourCorner	17620
U+9190.0	fHakka	 fu2
U+9190.0	fCantonese	wu4
U+9190.0	fMandarin	hu2
U+9190.0	fEnglish	used in combination for clarified butter
U+9191.0	fUTF8	醑
U+9191.0	fCNS11643	2-5B28
U+9191.0	fBig5	EBDA
U+9191.0	fGB	0-752F
U+9191.0	fR/S	 164.9
U+9191.0	fTotalStrokes	 16
U+9191.0	fCangjie	MWNOB
U+9191.0	fFourCorner	17627
U+9191.0	fHakka	 si1
U+9191.0	fCantonese	seoi2
U+9191.0	fMandarin	xu3
U+9191.0	fEnglish	 strain spirits
U+9192.0	fUTF8	醒
U+9192.0	fCNS11643	1-725E
U+9192.0	fBig5	BFF4
U+9192.0	fGB	0-5051
U+9192.0	fR/S	 164.9
U+9192.0	fTotalStrokes	 16
U+9192.0	fCangjie	MWAHM
U+9192.0	fFourCorner	16614
U+9192.0	fHakka	  siang3; siang1; ciang3
U+9192.0	fCantonese	sing2; seng2 (also); sing1
U+9192.0	fMandarin	xing3; xing1; jing1; cheng2; xing4
U+9192.0	fEnglish	[1] recover from (drunkenness, stupor, etc.); sober up; come to [2] awake; wake up; sober up; be roused [3] be clear; be cool in mind
U+9193.0	fUTF8	醓
U+9193.0	fCNS11643	2-5B26
U+9193.0	fBig5	EBD8
U+9193.0	fGB	3-6E75
U+9193.0	fR/S	 164.9
U+9193.0	fTotalStrokes	 16
U+9193.0	fCangjie	MWLUT
U+9193.0	fFourCorner	14617
U+9193.0	fHakka	  tam3
U+9193.0	fCantonese	taam2; taam5; taam1
U+9193.0	fMandarin	tan3
U+9193.0	fEnglish	salted meat; brine from pickled
U+9194.0	fUTF8	醔
U+9194.0	fCNS11643	4-5442
U+9194.0	fGB	E-407C
U+9194.0	fR/S	  164.9
U+9194.0	fTotalStrokes	 16
U+9194.0	fCangjie	HFMCW
U+9194.0	fFourCorner	29600
U+9194.0	fCantonese	jau4
U+9194.0	fMandarin	jiu1; qiu2; chou1
U+9195.0	fUTF8	醕
U+9195.0	fCNS11643	3-532C
U+9195.0	fGB	E-407D
U+9195.0	fR/S	 164.9
U+9195.0	fTotalStrokes	 16
U+9195.0	fCangjie	MWYRA
U+9195.0	fFourCorner	10660
U+9195.0	fCantonese	seon4
U+9195.0	fMandarin	chun2
U+9196.0	fUTF8	醖
U+9196.0	fCNS11643	3-532F
U+9196.0	fGB	1-544D
U+9196.0	fR/S	 164.9
U+9196.0	fTotalStrokes	 16
U+9196.0	fCangjie	MWABT
U+9196.0	fFourCorner	16610
U+9196.0	fCantonese	wan3 (also); wan5
U+9196.0	fMandarin	yun4
U+9196.0	fEnglish	liquor
U+9197.0	fUTF8	醗
U+9197.0	fGB	E-407E
U+9197.0	fR/S	 164.9
U+9197.0	fTotalStrokes	 16
U+9197.0	fCangjie	MWNOP
U+9197.0	fCantonese	put3
U+9197.0	fMandarin	po4; po1; fa1
U+9197.0	fEnglish	 brew for the second time
U+9198.0	fUTF8	醘
U+9198.0	fCNS11643	4-593F
U+9198.0	fGB	E-4121
U+9198.0	fR/S	 164.10
U+9198.0	fTotalStrokes	 17; 16
U+9198.0	fCangjie	MWGIT
U+9198.0	fFourCorner	14610
U+9198.0	fCantonese	hap1
U+9198.0	fMandarin	ke4; ke1
U+9199.0	fUTF8	醙
U+9199.0	fCNS11643	2-5F74
U+9199.0	fBig5	EEC8
U+9199.0	fGB	E-4122
U+9199.0	fR/S	 164.10
U+9199.0	fTotalStrokes	 17
U+9199.0	fCangjie	MWHXE
U+9199.0	fFourCorner	17647
U+9199.0	fCantonese	sau1; sau2
U+9199.0	fMandarin	sou1
U+919A.0	fUTF8	醚
U+919A.0	fCNS11643	2-5F71
U+919A.0	fBig5	EEC5
U+919A.0	fGB	0-4351
U+919A.0	fR/S	 164.10
U+919A.0	fTotalStrokes	 17
U+919A.0	fCangjie	MWYFD
U+919A.0	fFourCorner	19639
U+919A.0	fHakka	 mi2
U+919A.0	fCantonese	mai4
U+919A.0	fMandarin	mi2
U+919A.0	fEnglish	ether
U+919B.0	fUTF8	醛
U+919B.0	fCNS11643	2-5F73
U+919B.0	fBig5	EEC7
U+919B.0	fGB	0-4829
U+919B.0	fR/S	 164.10
U+919B.0	fTotalStrokes	 17
U+919B.0	fCangjie	MWTOG
U+919B.0	fFourCorner	14614
U+919B.0	fHakka	 cen2
U+919B.0	fCantonese	cyun4
U+919B.0	fMandarin	quan2; chuo4
U+919B.0	fEnglish	aldehyde
U+919C.0	fUTF8	醜
U+919C.0	fCNS11643	1-756B
U+919C.0	fBig5	C1E0
U+919C.0	fGB	1-3373
U+919C.0	fR/S	 164.10
U+919C.0	fTotalStrokes	 17
U+919C.0	fCangjie	MWHUI
U+919C.0	fFourCorner	16613
U+919C.0	fHakka	 cu3; chiu3; ciu3; chu3
U+919C.0	fCantonese	cau2
U+919C.0	fMandarin	chou3
U+919C.0	fEnglish	[1] ugly; homely; bad-looking [2] abomiable; vile; bad [3] shameful; disgraceful; famous [4] compare
U+919D.0	fUTF8	醝
U+919D.0	fCNS11643	2-5F77
U+919D.0	fBig5	EECB
U+919D.0	fGB	3-6E74
U+919D.0	fR/S	 164.10
U+919D.0	fTotalStrokes	 17
U+919D.0	fCangjie	MWTQM
U+919D.0	fFourCorner	18610
U+919D.0	fCantonese	co4; co1
U+919D.0	fMandarin	cuo2; cuo3; cuo1
U+919D.0	fEnglish	white wine
U+919E.0	fUTF8	醞
U+919E.0	fCNS11643	1-756A
U+919E.0	fBig5	C1DF
U+919E.0	fGB	E-4123
U+919E.0	fR/S	 164.10
U+919E.0	fTotalStrokes	 17
U+919E.0	fCangjie	MWWOT
U+919E.0	fFourCorner	16617
U+919E.0	fHakka	 wun3; wun5; vun5
U+919E.0	fCantonese	wan3 (also); wan5; wan2
U+919E.0	fMandarin	yun4; yun3
U+919E.0	fEnglish	[1] brew; ferment [2] preliminary informal discussion; deliberate; on; deliberation [3] liquor; spirits; wine
U+919F.0	fUTF8	醟
U+919F.0	fCNS11643	2-5F75
U+919F.0	fBig5	EEC9
U+919F.0	fGB	3-6E56
U+919F.0	fR/S	 164.10
U+919F.0	fTotalStrokes	 17
U+919F.0	fCangjie	FFBMW
U+919F.0	fFourCorner	99601
U+919F.0	fHakka	 jun3; jin2; run3; rin2
U+919F.0	fCantonese	wing6; wing4
U+919F.0	fMandarin	yong4; xiong4; yong3; ying2
U+919F.0	fEnglish	 get tipsy
U+91A0.0	fUTF8	醠
U+91A0.0	fCNS11643	2-5F78
U+91A0.0	fBig5	EECC
U+91A0.0	fGB	5-6E34
U+91A0.0	fR/S	 164.10
U+91A0.0	fTotalStrokes	 17
U+91A0.0	fCangjie	MWLKT
U+91A0.0	fFourCorner	15617
U+91A0.0	fHakka	 ong1
U+91A0.0	fCantonese	ong3; ngong3 (also)
U+91A0.0	fMandarin	ang4; ang3
U+91A1.0	fUTF8	醡
U+91A1.0	fCNS11643	2-5F76
U+91A1.0	fBig5	EECA
U+91A1.0	fGB	5-6E35
U+91A1.0	fR/S	 164.10
U+91A1.0	fTotalStrokes	 17
U+91A1.0	fCangjie	MWJCS
U+91A1.0	fFourCorner	13611
U+91A1.0	fHakka	 za5
U+91A1.0	fCantonese	zaa3
U+91A1.0	fMandarin	zha4; zhan4
U+91A1.0	fEnglish	wine press
U+91A2.0	fUTF8	醢
U+91A2.0	fCNS11643	2-5F72
U+91A2.0	fBig5	EEC6
U+91A2.0	fGB	0-7530
U+91A2.0	fR/S	 164.10
U+91A2.0	fTotalStrokes	 17
U+91A2.0	fCangjie	MWKRT
U+91A2.0	fFourCorner	14617
U+91A2.0	fHakka	  hoi3
U+91A2.0	fCantonese	hoi2
U+91A2.0	fMandarin	hai3
U+91A2.0	fEnglish	[1] minced and hashed meat [2] mince a criminal as a punishment in ancient China
U+91A3.0	fUTF8	醣
U+91A3.0	fCNS11643	1-7569
U+91A3.0	fBig5	C1DE
U+91A3.0	fGB	0-7531
U+91A3.0	fR/S	 164.10
U+91A3.0	fTotalStrokes	 17
U+91A3.0	fCangjie	MWILR
U+91A3.0	fFourCorner	10667
U+91A3.0	fHakka	 tong2
U+91A3.0	fCantonese	tong4
U+91A3.0	fMandarin	tang2
U+91A3.0	fEnglish	carbohydrate
U+91A4.0	fUTF8	醤
U+91A4.0	fGB	E-4124
U+91A4.0	fR/S	 164.10
U+91A4.0	fTotalStrokes	 17
U+91A4.0	fCangjie	LIMCW
U+91A4.0	fCantonese	zoeng3
U+91A4.0	fMandarin	jiang4
U+91A4.0	fEnglish	any jam-like or paste-like food
U+91A5.0	fUTF8	醥
U+91A5.0	fCNS11643	2-6379
U+91A5.0	fBig5	F14F
U+91A5.0	fGB	E-4125
U+91A5.0	fR/S	 164.11
U+91A5.0	fTotalStrokes	 18
U+91A5.0	fCangjie	MWMWF
U+91A5.0	fFourCorner	11691
U+91A5.0	fHakka	  piau5; peu5
U+91A5.0	fCantonese	piu2; piu5
U+91A5.0	fMandarin	piao3
U+91A5.0	fEnglish	clear spirits
U+91A6.0	fUTF8	醦
U+91A6.0	fCNS11643	4-5E24
U+91A6.0	fGB	3-6E6D
U+91A6.0	fR/S	 164.11
U+91A6.0	fTotalStrokes	 18
U+91A6.0	fCangjie	MWIIH
U+91A6.0	fFourCorner	13620
U+91A6.0	fHakka	  cam3
U+91A6.0	fCantonese	cam2
U+91A6.0	fMandarin	shan3; chen3; chan3
U+91A6.0	fEnglish	vinegar-like; sour
U+91A7.0	fUTF8	醧
U+91A7.0	fCNS11643	2-637A
U+91A7.0	fBig5	F150
U+91A7.0	fGB	5-6D7B
U+91A7.0	fR/S	 164.11
U+91A7.0	fTotalStrokes	 18
U+91A7.0	fCangjie	MWSRR
U+91A7.0	fFourCorner	11616
U+91A7.0	fCantonese	jyu3
U+91A7.0	fMandarin	yu4; ou1; ou4
U+91A8.0	fUTF8	醨
U+91A8.0	fCNS11643	2-6378
U+91A8.0	fBig5	F14E
U+91A8.0	fGB	3-6E76
U+91A8.0	fR/S	 164.11
U+91A8.0	fTotalStrokes	 18
U+91A8.0	fCangjie	MWYUB
U+91A8.0	fFourCorner	10627
U+91A8.0	fHakka	  li2
U+91A8.0	fCantonese	lei4
U+91A8.0	fMandarin	li2
U+91A8.0	fEnglish	[1] weak wine or liquor [2] light; weak (wie, tea, etc.)
U+91A9.0	fUTF8	醩
U+91A9.0	fCNS11643	3-593D
U+91A9.0	fGB	5-6E36
U+91A9.0	fR/S	 164.11
U+91A9.0	fTotalStrokes	 18
U+91A9.0	fCangjie	MWTWA
U+91A9.0	fFourCorner	15660
U+91A9.0	fHakka	  zau1; zo1
U+91A9.0	fCantonese	zou1
U+91A9.0	fMandarin	zao2; zao1
U+91A9.0	fEnglish	 sediment; dregs grains from a distillery disordered; spoilt
U+91AA.0	fUTF8	醪
U+91AA.0	fCNS11643	2-637C
U+91AA.0	fBig5	F152
U+91AA.0	fGB	0-7532
U+91AA.0	fR/S	 164.11
U+91AA.0	fTotalStrokes	 18
U+91AA.0	fCangjie	MWSMH
U+91AA.0	fFourCorner	17622
U+91AA.0	fHakka	  lau2; lo3; gau3
U+91AA.0	fCantonese	lou4
U+91AA.0	fMandarin	lao2
U+91AA.0	fEnglish	[1] unstrained wine or liquor [2] mellow wine
U+91AB.0	fUTF8	醫
U+91AB.0	fCNS11643	1-7750
U+91AB.0	fBig5	C2E5
U+91AB.0	fGB	1-523D
U+91AB.0	fR/S	 164.11
U+91AB.0	fTotalStrokes	 18
U+91AB.0	fCangjie	SEMCW
U+91AB.0	fFourCorner	77601
U+91AB.0	fHakka	 ji1; ri1; i1
U+91AB.0	fCantonese	ji1
U+91AB.0	fMandarin	yi1; yi3; yi4
U+91AB.0	fEnglish	[1] cure; heal; treat (disease) [2] doctor; surgeon; physician [3] medical science; medical service; medicine
U+91AC.0	fUTF8	醬
U+91AC.0	fCNS11643	1-7751
U+91AC.0	fBig5	C2E6
U+91AC.0	fGB	1-3D34
U+91AC.0	fR/S	 164.11
U+91AC.0	fTotalStrokes	 18
U+91AC.0	fCangjie	VIMCW
U+91AC.0	fFourCorner	27601
U+91AC.0	fHakka	  ziong5; ziong3
U+91AC.0	fCantonese	zoeng3
U+91AC.0	fMandarin	jiang4
U+91AC.0	fEnglish	[1] soybean sause; soy [2] food in the form of a paste; jam; any jam-like or paste-like food
U+91AD.0	fUTF8	醭
U+91AD.0	fCNS11643	2-674B
U+91AD.0	fBig5	F35F
U+91AD.0	fGB	0-7533
U+91AD.0	fR/S	 164.12
U+91AD.0	fTotalStrokes	 19
U+91AD.0	fCangjie	MWTCO
U+91AD.0	fFourCorner	13634
U+91AD.0	fHakka	 puk8; pu3; puk7
U+91AD.0	fCantonese	buk6; pok3 (also)
U+91AD.0	fMandarin	bu2; pu1; pu2
U+91AD.0	fEnglish	white specks of mildew
U+91AE.0	fUTF8	醮
U+91AE.0	fCNS11643	1-7934
U+91AE.0	fBig5	C3E7
U+91AE.0	fGB	0-7534
U+91AE.0	fR/S	 164.12
U+91AE.0	fTotalStrokes	 19
U+91AE.0	fCangjie	MWOGF
U+91AE.0	fFourCorner	10631
U+91AE.0	fHakka	  ziau5; zeu5
U+91AE.0	fCantonese	ziu3
U+91AE.0	fMandarin	jiao4; qiao2; zhao4; zhan4
U+91AE.0	fEnglish	[1] religious service [2] wedding; marriage
U+91AF.0	fUTF8	醯
U+91AF.0	fCNS11643	2-637B
U+91AF.0	fBig5	F151
U+91AF.0	fGB	0-7535
U+91AF.0	fR/S	 164.12
U+91AF.0	fTotalStrokes	 19
U+91AF.0	fCangjie	MWYIT
U+91AF.0	fFourCorner	10617
U+91AF.0	fHakka	 hi1
U+91AF.0	fCantonese	hei1
U+91AF.0	fMandarin	xi1; xi4
U+91AF.0	fEnglish	vinegar; pickle; acide
U+91B0.0	fUTF8	醰
U+91B0.0	fCNS11643	2-674A
U+91B0.0	fBig5	F35E
U+91B0.0	fGB	3-6E7A
U+91B0.0	fR/S	 164.12
U+91B0.0	fTotalStrokes	 19
U+91B0.0	fCangjie	MWMWJ
U+91B0.0	fFourCorner	11646
U+91B0.0	fHakka	 tam1; tam3; tam2
U+91B0.0	fCantonese	taam4
U+91B0.0	fMandarin	tan2; dan4; tan3
U+91B0.0	fEnglish	 bitter taste in wine rich full flavoured
U+91B1.0	fUTF8	醱
U+91B1.0	fCNS11643	1-7933
U+91B1.0	fBig5	C3E6
U+91B1.0	fGB	3-6E5A
U+91B1.0	fR/S	 164.12
U+91B1.0	fTotalStrokes	 19
U+91B1.0	fCangjie	MWNOE
U+91B1.0	fFourCorner	12647
U+91B1.0	fHakka	  pat7
U+91B1.0	fCantonese	faat3; but3; put3
U+91B1.0	fMandarin	po1; po4; fa1; bo1
U+91B1.0	fEnglish	 brew for the second time
U+91B2.0	fUTF8	醲
U+91B2.0	fCNS11643	2-6A33
U+91B2.0	fBig5	F4E5
U+91B2.0	fGB	3-6E60
U+91B2.0	fR/S	 164.13
U+91B2.0	fTotalStrokes	 20
U+91B2.0	fCangjie	MWTWV
U+91B2.0	fFourCorner	15632
U+91B2.0	fHakka	  ngiung2; neu2; nung1; niung2
U+91B2.0	fCantonese	nung4; lung4 (also)
U+91B2.0	fMandarin	nong2
U+91B2.0	fEnglish	[1] strong wine; rich wine [2] dense; thick; saturated
U+91B3.0	fUTF8	醳
U+91B3.0	fCNS11643	2-6A34
U+91B3.0	fBig5	F4E6
U+91B3.0	fGB	3-6E5B
U+91B3.0	fR/S	 164.13
U+91B3.0	fTotalStrokes	 20
U+91B3.0	fCangjie	MWWLJ
U+91B3.0	fFourCorner	16641
U+91B3.0	fCantonese	jik6; sik1
U+91B3.0	fMandarin	yi4; shi4; xi2
U+91B4.0	fUTF8	醴
U+91B4.0	fCNS11643	1-7A4A
U+91B4.0	fBig5	C4BF
U+91B4.0	fGB	0-7537
U+91B4.0	fR/S	 164.13
U+91B4.0	fTotalStrokes	 20
U+91B4.0	fCangjie	MWTWT
U+91B4.0	fFourCorner	15618
U+91B4.0	fHakka	 li3;  ti3; li1
U+91B4.0	fCantonese	lai5
U+91B4.0	fMandarin	li3
U+91B4.0	fEnglish	[1] sweet wine [2] sweet water from a spring
U+91B5.0	fUTF8	醵
U+91B5.0	fCNS11643	2-6A32
U+91B5.0	fBig5	F4E4
U+91B5.0	fGB	0-7536
U+91B5.0	fR/S	 164.13
U+91B5.0	fTotalStrokes	 20
U+91B5.0	fCangjie	MWYPO
U+91B5.0	fFourCorner	11632
U+91B5.0	fHakka	 kiok7; gi3; gi5
U+91B5.0	fCantonese	geoi6; koek6 (also); kek6
U+91B5.0	fMandarin	ju4; jue2; qu2
U+91B5.0	fEnglish	 pool money (for a feast, etc.); contribute for drinks
U+91B6.0	fUTF8	醶
U+91B6.0	fCNS11643	4-6465
U+91B6.0	fGB	5-6E28
U+91B6.0	fR/S	  164.13
U+91B6.0	fTotalStrokes	 20
U+91B6.0	fCangjie	MWOMO
U+91B6.0	fFourCorner	18680
U+91B6.0	fCantonese	jim6
U+91B6.0	fMandarin	yan4; jiao4
U+91B7.0	fUTF8	醷
U+91B7.0	fCNS11643	2-6A31
U+91B7.0	fBig5	F4E3
U+91B7.0	fGB	3-6F21
U+91B7.0	fR/S	 164.13
U+91B7.0	fTotalStrokes	 20
U+91B7.0	fCangjie	MWYTP
U+91B7.0	fFourCorner	10636
U+91B7.0	fCantonese	jik1; ji3
U+91B7.0	fMandarin	yi4; tan3; yi3
U+91B8.0	fUTF8	醸
U+91B8.0	fGB	E-4126
U+91B8.0	fR/S	 164.13
U+91B8.0	fTotalStrokes	 20
U+91B8.0	fCangjie	MWYCV
U+91B8.0	fCantonese	joeng6
U+91B8.0	fMandarin	niang4
U+91B8.0	fEnglish	brew; ferment
U+91B9.0	fUTF8	醹
U+91B9.0	fCNS11643	2-6C4C
U+91B9.0	fBig5	F65D
U+91B9.0	fGB	3-6F22
U+91B9.0	fR/S	 164.14
U+91B9.0	fTotalStrokes	 21
U+91B9.0	fCangjie	MWMBB
U+91B9.0	fFourCorner	11627
U+91B9.0	fHakka	 neu2; ji2; ri2; ri5; ji5; si1
U+91B9.0	fCantonese	jyu4; jyu5
U+91B9.0	fMandarin	ru2; ru3
U+91B9.0	fEnglish	strong wine
U+91BA.0	fUTF8	醺
U+91BA.0	fCNS11643	1-7B34
U+91BA.0	fBig5	C548
U+91BA.0	fGB	0-7538
U+91BA.0	fR/S	 164.14
U+91BA.0	fTotalStrokes	 21
U+91BA.0	fCangjie	MWHGF
U+91BA.0	fFourCorner	12631
U+91BA.0	fHakka	 hiun1
U+91BA.0	fCantonese	fan1
U+91BA.0	fMandarin	xun1
U+91BA.0	fEnglish	drunk; intoxicated; tipsy
U+91BB.0	fUTF8	醻
U+91BB.0	fCNS11643	3-5E68
U+91BB.0	fGB	E-4127
U+91BB.0	fR/S	 164.14
U+91BB.0	fTotalStrokes	 21
U+91BB.0	fCangjie	MWGNI
U+91BB.0	fFourCorner	14640
U+91BB.0	fHakka	 chiu2; cu2
U+91BB.0	fCantonese	cau4
U+91BB.0	fMandarin	chou2
U+91BC.0	fUTF8	醼
U+91BC.0	fCNS11643	2-6F57
U+91BC.0	fBig5	F849
U+91BC.0	fGB	E-4128
U+91BC.0	fR/S	 164.16
U+91BC.0	fTotalStrokes	 23
U+91BC.0	fCangjie	MWTLF
U+91BC.0	fFourCorner	14631
U+91BC.0	fCantonese	jin3
U+91BC.0	fMandarin	yan4
U+91BC.0	fEnglish	offer food and wine one's guests
U+91BD.0	fUTF8	醽
U+91BD.0	fCNS11643	2-7056
U+91BD.0	fBig5	F8C8
U+91BD.0	fGB	3-6F24
U+91BD.0	fR/S	 164.17
U+91BD.0	fTotalStrokes	 24
U+91BD.0	fCangjie	MWMBR
U+91BD.0	fFourCorner	11662
U+91BD.0	fHakka	 lin2
U+91BD.0	fCantonese	ling4
U+91BD.0	fMandarin	ling2
U+91BD.0	fEnglish	a kind of wine
U+91BE.0	fUTF8	醾
U+91BE.0	fCNS11643	2-7055
U+91BE.0	fBig5	F8C7
U+91BE.0	fGB	3-6F26
U+91BE.0	fR/S	 164.17
U+91BE.0	fTotalStrokes	 24
U+91BE.0	fCangjie	MWIDD
U+91BE.0	fFourCorner	10694
U+91BE.0	fHakka	 mi2
U+91BE.0	fCantonese	mei4
U+91BE.0	fMandarin	mi2
U+91BE.0	fEnglish	used in combination for wine brewed for the second time
U+91BF.0	fUTF8	醿
U+91BF.0	fCNS11643	3-607E
U+91BF.0	fGB	E-4129
U+91BF.0	fR/S	 164.17
U+91BF.0	fTotalStrokes	 24
U+91BF.0	fCangjie	MWIDF
U+91BF.0	fFourCorner	10690
U+91BF.0	fHakka	  mi2
U+91BF.0	fCantonese	mei4
U+91BF.0	fMandarin	mi2
U+91BF.0	fEnglish	a kind of liquor distilled from grain
U+91C0.0	fUTF8	釀
U+91C0.0	fCNS11643	1-7C6E
U+91C0.0	fBig5	C643
U+91C0.0	fGB	1-4470
U+91C0.0	fR/S	 164.17
U+91C0.0	fTotalStrokes	 24
U+91C0.0	fCangjie	MWYRV
U+91C0.0	fFourCorner	10632
U+91C0.0	fHakka	  jong5; ngiong2; rong5; jong6; ngiong5
U+91C0.0	fCantonese	joeng6; joeng2 (also)
U+91C0.0	fMandarin	niang4; niang2
U+91C0.0	fEnglish	[1] brew; ferment [2] take shape; form slowly [3] wine
U+91C1.0	fUTF8	釁
U+91C1.0	fCNS11643	1-7D2A
U+91C1.0	fBig5	C65D
U+91C1.0	fGB	1-5046
U+91C1.0	fR/S	 164.18
U+91C1.0	fTotalStrokes	 25
U+91C1.0	fCangjie	HBMCH
U+91C1.0	fFourCorner	77227
U+91C1.0	fHakka	 hin5
U+91C1.0	fCantonese	jan6; jan3
U+91C1.0	fMandarin	xin4
U+91C1.0	fEnglish	[1] anoint (drums, bells, etc.) with blood in sacrifice [2] anoint (the boody) [3] rift (between people)
U+91C2.0	fUTF8	釂
U+91C2.0	fCNS11643	2-7057
U+91C2.0	fBig5	F8C9
U+91C2.0	fGB	3-6F25
U+91C2.0	fR/S	 164.18
U+91C2.0	fTotalStrokes	 25
U+91C2.0	fCangjie	MWBWI
U+91C2.0	fFourCorner	12646
U+91C2.0	fHakka	  ziau5; ziok7
U+91C2.0	fCantonese	ziu3
U+91C2.0	fMandarin	jiao4
U+91C2.0	fEnglish	drain
U+91C3.0	fUTF8	釃
U+91C3.0	fCNS11643	2-7160
U+91C3.0	fBig5	F971
U+91C3.0	fGB	1-7527
U+91C3.0	fR/S	 164.19
U+91C3.0	fTotalStrokes	 26
U+91C3.0	fCangjie	MWMMP
U+91C3.0	fFourCorner	11611
U+91C3.0	fHakka	 sii1; sai3; li2; se1; si1
U+91C3.0	fCantonese	si1
U+91C3.0	fMandarin	xi3; li2; shi1; shuai1; shai1
U+91C3.0	fEnglish	strain
U+91C4.0	fUTF8	釄
U+91C4.0	fCNS11643	3-616D
U+91C4.0	fGB	E-412A
U+91C4.0	fR/S	 164.19
U+91C4.0	fTotalStrokes	 26
U+91C4.0	fCangjie	MWIDY
U+91C4.0	fFourCorner	10610
U+91C4.0	fCantonese	mei4
U+91C4.0	fMandarin	mi2
U+91C4.0	fEnglish	wine brew for the second time
U+91C5.0	fUTF8	釅
U+91C5.0	fCNS11643	1-7D3C
U+91C5.0	fBig5	C66F
U+91C5.0	fGB	1-7526
U+91C5.0	fR/S	 164.20
U+91C5.0	fTotalStrokes	 27
U+91C5.0	fCangjie	MWRRK
U+91C5.0	fFourCorner	16648
U+91C5.0	fHakka	 ngiam2; gniam6; ngiong2
U+91C5.0	fCantonese	jim6
U+91C5.0	fMandarin	yan4
U+91C5.0	fEnglish	thick; strong (beverage)
U+91C6.0	fUTF8	釆
U+91C6.0	fCNS11643	1-4B5D
U+91C6.0	fBig5	A8BC
U+91C6.0	fGB	E-412B
U+91C6.0	fR/S	 165.0
U+91C6.0	fTotalStrokes	 7
U+91C6.0	fCangjie	HFD
U+91C6.0	fFourCorner	20909
U+91C6.0	fHakka	 pien5; pien6
U+91C6.0	fCantonese	bin6
U+91C6.0	fMandarin	bian4; cai3
U+91C6.0	fEnglish	Kangxi radical 165
U+91C7.0	fUTF8	采
U+91C7.0	fCNS11643	1-4F59
U+91C7.0	fBig5	AAF6
U+91C7.0	fGB	0-3249
U+91C7.0	fR/S	 165.1
U+91C7.0	fTotalStrokes	 8
U+91C7.0	fCangjie	BD
U+91C7.0	fFourCorner	20904
U+91C7.0	fHakka	  cai3; coi3; coi2
U+91C7.0	fCantonese	coi2; coi3
U+91C7.0	fMandarin	cai3; zou3
U+91C7.0	fEnglish	[1] collect; gather [2] pick; select; pluck [3] bright colors
U+91C7.1	fUTF8	采
U+91C7.1	fCNS11643	1-4F59
U+91C7.1	fBig5	AAF6
U+91C7.1	fGB	0-3249
U+91C7.1	fR/S	 165.1
U+91C7.1	fTotalStrokes	 8
U+91C7.1	fCangjie	BD
U+91C7.1	fFourCorner	20904
U+91C7.1	fMandarin	cai4
U+91C7.1	fEnglish	 fief; vassalage
U+91C8.0	fUTF8	釈
U+91C8.0	fGB	E-412C
U+91C8.0	fR/S	 165.4
U+91C8.0	fTotalStrokes	 11
U+91C8.0	fCangjie	HDSO
U+91C8.0	fCantonese	sik1
U+91C8.0	fMandarin	shi4; yi4
U+91C8.0	fEnglish	 interprete; elucidate release
U+91C9.0	fUTF8	釉
U+91C9.0	fCNS11643	1-673E
U+91C9.0	fBig5	B956
U+91C9.0	fGB	0-5354
U+91C9.0	fR/S	 165.5
U+91C9.0	fTotalStrokes	 12
U+91C9.0	fCangjie	HDLW
U+91C9.0	fFourCorner	25960
U+91C9.0	fHakka	  ju5; riu5; jiu3; jui5
U+91C9.0	fCantonese	jau6; jau2 (also)
U+91C9.0	fMandarin	you4
U+91C9.0	fEnglish	glaze (pottery, etc.)
U+91CA.0	fUTF8	释
U+91CA.0	fGB	0-4A4D
U+91CA.0	fR/S	 165.5
U+91CA.0	fTotalStrokes	 12
U+91CA.0	fCangjie	HDEQ
U+91CA.0	fHakka	 shit7; sit7
U+91CA.0	fCantonese	sik1; jik6
U+91CA.0	fMandarin	shi4; yi4
U+91CA.0	fEnglish	simplified form of &#x91CB;
U+91CB.0	fUTF8	釋
U+91CB.0	fCNS11643	1-7A4B
U+91CB.0	fBig5	C4C0
U+91CB.0	fGB	1-4A4D
U+91CB.0	fR/S	 165.13
U+91CB.0	fTotalStrokes	 20
U+91CB.0	fCangjie	HDWLJ
U+91CB.0	fFourCorner	26941
U+91CB.0	fHakka	  sit7;  shit7; shit8; sit752
U+91CB.0	fCantonese	sik1; jik6
U+91CB.0	fMandarin	shi4; yi4
U+91CB.0	fEnglish	[1] explain; interprete [2] set free; release [3] relieve [4] disperse; dispel [5] of Buddha; of Buddhism
U+91CC.0	fUTF8	里
U+91CC.0	fCNS11643	1-4B5E
U+91CC.0	fBig5	A8BD
U+91CC.0	fGB	0-406F
U+91CC.0	fR/S	 166.0
U+91CC.0	fTotalStrokes	 7
U+91CC.0	fCangjie	WG
U+91CC.0	fFourCorner	60104
U+91CC.0	fHakka	  li1; li3; {li1}
U+91CC.0	fCantonese	lei5; leoi5
U+91CC.0	fMandarin	li3
U+91CC.0	fEnglish	[1] neighborhood or community of 25 families (in ancient times); neighborhood; village [2] li, unit of length [3] simplified form of &#x88CF;
U+91CC.1	fUTF8	里
U+91CC.1	fCNS11643	1-4B5E
U+91CC.1	fBig5	A8BD
U+91CC.1	fGB	0-406F
U+91CC.1	fR/S	 166.0
U+91CC.1	fTotalStrokes	 7
U+91CC.1	fCangjie	WG
U+91CC.1	fFourCorner	60104
U+91CC.1	fMandarin	li5
U+91CD.0	fUTF8	重
U+91CD.0	fCNS11643	1-536C
U+91CD.0	fBig5	ADAB
U+91CD.0	fGB	0-5658
U+91CD.0	fR/S	 166.2
U+91CD.0	fTotalStrokes	 9
U+91CD.0	fCangjie	HJWG
U+91CD.0	fFourCorner	20104
U+91CD.0	fHakka	  cung1;  chung1; cung5; chung3;  chung5;  chung2; cung3; {chung1}
U+91CD.0	fCantonese	cung5; zung6; cung4
U+91CD.0	fMandarin	zhong4; tong2
U+91CD.0	fEnglish	[1] heavy; weighty; much [2] weight; weight [3] difficult [4] serious; grave [5] severe [6] important; significant [7] value; emphasise
U+91CD.1	fUTF8	重
U+91CD.1	fCNS11643	1-536C
U+91CD.1	fBig5	ADAB
U+91CD.1	fGB	0-5658
U+91CD.1	fR/S	 166.2
U+91CD.1	fTotalStrokes	 9
U+91CD.1	fCangjie	HJWG
U+91CD.1	fFourCorner	20104
U+91CD.1	fHakka	 cung2
U+91CD.1	fMandarin	chong2
U+91CD.1	fEnglish	[1] pile one upon another [2] repeat; duplicate [3] layers [4] doubl; manifold [5] numerous; countless
U+91CE.0	fUTF8	野
U+91CE.0	fCNS11643	1-5D69
U+91CE.0	fBig5	B3A5
U+91CE.0	fGB	0-5230
U+91CE.0	fR/S	 166.4
U+91CE.0	fTotalStrokes	 11
U+91CE.0	fCangjie	WGNIN
U+91CE.0	fFourCorner	67122
U+91CE.0	fHakka	 ja3;  ja1; ra1; ra3; ja5
U+91CE.0	fCantonese	je5; je2 (also)
U+91CE.0	fMandarin	ye3; shu4; yu3
U+91CE.0	fEnglish	[1] countryside; open country; fields; wilderness [2] the people (as opposed to the government) [3] wild; uncultured; undomesticated; coarse; barbarous; rude
U+91CF.0	fUTF8	量
U+91CF.0	fCNS11643	1-6258
U+91CF.0	fBig5	B671
U+91CF.0	fGB	0-413F
U+91CF.0	fR/S	 166.5
U+91CF.0	fTotalStrokes	 12
U+91CF.0	fCangjie	AMWG
U+91CF.0	fFourCorner	60104
U+91CF.0	fHakka	  liong2;  liong5; liong3; liong6
U+91CF.0	fCantonese	loeng4; loeng6
U+91CF.0	fMandarin	liang4
U+91CF.0	fEnglish	 measure
U+91CF.1	fUTF8	量
U+91CF.1	fCNS11643	1-6258
U+91CF.1	fBig5	B671
U+91CF.1	fGB	0-413F
U+91CF.1	fR/S	 166.5
U+91CF.1	fTotalStrokes	 12
U+91CF.1	fCangjie	AMWG
U+91CF.1	fFourCorner	60104
U+91CF.1	fMandarin	liang2
U+91CF.1	fEnglish	[1] quantity [2] capacity [3] estimate
U+91CF.2	fUTF8	量
U+91CF.2	fCNS11643	1-6258
U+91CF.2	fBig5	B671
U+91CF.2	fGB	0-413F
U+91CF.2	fR/S	 166.5
U+91CF.2	fTotalStrokes	 12
U+91CF.2	fCangjie	AMWG
U+91CF.2	fFourCorner	60104
U+91CF.2	fMandarin	liang5
U+91D0.0	fUTF8	釐
U+91D0.0	fCNS11643	1-7752
U+91D0.0	fBig5	C2E7
U+91D0.0	fGB	E-412D
U+91D0.0	fR/S	 166.11
U+91D0.0	fTotalStrokes	 18
U+91D0.0	fCangjie	JKMWG
U+91D0.0	fFourCorner	58214
U+91D0.0	fHakka	 li2; li3; hi1
U+91D0.0	fCantonese	hei1; hei2 (also); lei4
U+91D0.0	fMandarin	li2; lai2; lai4; tai1
U+91D0.0	fEnglish	[1] manage; control; administer; arrange [2] a unit of length equal to 1/1000 of a Chinese foot [3] a unit of weight equal to 1/1000 of the tael [4] revise; reform; correct [5] same as $
U+91D0.1	fUTF8	釐
U+91D0.1	fCNS11643	1-7752
U+91D0.1	fBig5	C2E7
U+91D0.1	fGB	E-412D
U+91D0.1	fR/S	 166.11
U+91D0.1	fTotalStrokes	 18
U+91D0.1	fCangjie	JKMWG
U+91D0.1	fFourCorner	58214
U+91D0.1	fMandarin	xi1
U+91D0.1	fEnglish	 blessing; happiness; bliss
U+91D1.0	fUTF8	金
U+91D1.0	fCNS11643	1-4F5A
U+91D1.0	fBig5	AAF7
U+91D1.0	fGB	0-3D70
U+91D1.0	fR/S	 167.0
U+91D1.0	fTotalStrokes	 8
U+91D1.0	fCangjie	C
U+91D1.0	fFourCorner	80109
U+91D1.0	fHakka	  gim1
U+91D1.0	fCantonese	gam1
U+91D1.0	fMandarin	jin1; jin4
U+91D1.0	fEnglish	[1] gold [2] metal; metals in general [3] money; wealth [4] weapons; arms [5] precious; excelltent; fine [6] golden [7] durable [8] name of the Tungusic dynasty (1115-1234), which dominated Manchuria and North China [9] a Chinese family name [10] Kangxi radical 167
U+91D2.0	fUTF8	釒
U+91D2.0	fGB	1-6E44
U+91D2.0	fR/S	 167.0
U+91D2.0	fTotalStrokes	 8
U+91D2.0	fCangjie	OMGC
U+91D2.0	fMandarin	jin1zi4pang2
U+91D2.0	fEnglish	variant of Kangxi radical 167, used in the left part of a character
U+91D3.0	fUTF8	釓
U+91D3.0	fCNS11643	2-2D62
U+91D3.0	fBig5	D0C1
U+91D3.0	fGB	1-6E45
U+91D3.0	fR/S	 167.1
U+91D3.0	fTotalStrokes	 9
U+91D3.0	fCangjie	CU
U+91D3.0	fFourCorner	82110
U+91D3.0	fCantonese	gaat3 (also); gaa1; kau4
U+91D3.0	fMandarin	ga2; qiu2
U+91D3.0	fEnglish	gadolinium
U+91D4.0	fUTF8	釔
U+91D4.0	fCNS11643	2-2D61
U+91D4.0	fBig5	D0C0
U+91D4.0	fGB	1-6E46
U+91D4.0	fR/S	 167.1
U+91D4.0	fTotalStrokes	 9
U+91D4.0	fCangjie	CN
U+91D4.0	fFourCorner	87110
U+91D4.0	fCantonese	jyut6; jyut3 (also)
U+91D4.0	fMandarin	yi3
U+91D4.0	fEnglish	ytrium
U+91D5.0	fUTF8	釕
U+91D5.0	fCNS11643	2-3345
U+91D5.0	fBig5	D442
U+91D5.0	fGB	1-6E49
U+91D5.0	fR/S	 167.2
U+91D5.0	fTotalStrokes	 10
U+91D5.0	fCangjie	CNN
U+91D5.0	fFourCorner	87127
U+91D5.0	fCantonese	liu5; liu6; liu4
U+91D5.0	fMandarin	liao3; liao4; diao3
U+91D5.0	fEnglish	ruthenium
U+91D6.0	fUTF8	釖
U+91D6.0	fCNS11643	3-3430
U+91D6.0	fGB	5-7537
U+91D6.0	fR/S	 167.2
U+91D6.0	fTotalStrokes	 10
U+91D6.0	fCangjie	CSH
U+91D6.0	fFourCorner	87120
U+91D6.0	fCantonese	dou1
U+91D6.0	fMandarin	dao1
U+91D6.0	fEnglish	knife; sword
U+91D7.0	fUTF8	釗
U+91D7.0	fCNS11643	1-585D
U+91D7.0	fBig5	B078
U+91D7.0	fGB	1-6E48
U+91D7.0	fR/S	 167.2
U+91D7.0	fTotalStrokes	 10
U+91D7.0	fCangjie	CLN
U+91D7.0	fFourCorner	87120
U+91D7.0	fHakka	 zau1;  zhau1
U+91D7.0	fCantonese	ciu1; ziu1
U+91D7.0	fMandarin	zhao1; jiao1
U+91D7.0	fEnglish	 encourage
U+91D8.0	fUTF8	釘
U+91D8.0	fCNS11643	1-585B
U+91D8.0	fBig5	B076
U+91D8.0	fGB	1-3624
U+91D8.0	fR/S	 167.2
U+91D8.0	fTotalStrokes	 10
U+91D8.0	fCangjie	CMN
U+91D8.0	fFourCorner	81120
U+91D8.0	fHakka	 dang5;  den1; den5
U+91D8.0	fCantonese	ding1; deng1 (also)
U+91D8.0	fMandarin	ding1; ling2; ding3
U+91D8.0	fEnglish	[1] nail; spike (for fastening) [2] look steadily
U+91D8.1	fUTF8	釘
U+91D8.1	fCNS11643	1-585B
U+91D8.1	fBig5	B076
U+91D8.1	fGB	1-3624
U+91D8.1	fR/S	 167.2
U+91D8.1	fTotalStrokes	 10
U+91D8.1	fCangjie	CMN
U+91D8.1	fFourCorner	81120
U+91D8.1	fHakka	 dang1
U+91D8.1	fMandarin	ding4
U+91D8.1	fEnglish	 fasten (with nails, etc.)
U+91D9.0	fUTF8	釙
U+91D9.0	fCNS11643	1-585F
U+91D9.0	fBig5	B07A
U+91D9.0	fGB	1-6E47
U+91D9.0	fR/S	 167.2
U+91D9.0	fTotalStrokes	 10
U+91D9.0	fCangjie	CY
U+91D9.0	fFourCorner	83100
U+91D9.0	fHakka	 puk7
U+91D9.0	fCantonese	pok3; puk3 (also)
U+91D9.0	fMandarin	po4; pu2; li4; po1
U+91D9.0	fEnglish	polonium
U+91DA.0	fUTF8	釚
U+91DA.0	fCNS11643	2-3347
U+91DA.0	fBig5	D444
U+91DA.0	fGB	5-7536
U+91DA.0	fR/S	 167.2
U+91DA.0	fTotalStrokes	 10
U+91DA.0	fCangjie	CKN
U+91DA.0	fFourCorner	84117
U+91DA.0	fCantonese	kau4
U+91DA.0	fMandarin	qiu2
U+91DA.0	fEnglish	chemical element 109 (chemistry)
U+91DB.0	fUTF8	釛
U+91DB.0	fCNS11643	3-342D
U+91DB.0	fGB	E-412E
U+91DB.0	fR/S	 167.2
U+91DB.0	fTotalStrokes	 10
U+91DB.0	fCangjie	CKS
U+91DB.0	fFourCorner	84120
U+91DB.0	fCantonese	baat3
U+91DB.0	fMandarin	he2
U+91DC.0	fUTF8	釜
U+91DC.0	fCNS11643	1-585E
U+91DC.0	fBig5	B079
U+91DC.0	fGB	0-382A
U+91DC.0	fR/S	 167.2
U+91DC.0	fTotalStrokes	 10
U+91DC.0	fCangjie	CKMGC
U+91DC.0	fFourCorner	80109
U+91DC.0	fHakka	  fu5; fu3; fu1; bu3; pu3
U+91DC.0	fCantonese	fu2
U+91DC.0	fMandarin	fu3
U+91DC.0	fEnglish	[1] cauldron; pot; kettle [2] an ancient unit of capacity
U+91DD.0	fUTF8	針
U+91DD.0	fCNS11643	1-585C
U+91DD.0	fBig5	B077
U+91DD.0	fGB	1-556B
U+91DD.0	fR/S	 167.2
U+91DD.0	fTotalStrokes	 10
U+91DD.0	fCangjie	CJ
U+91DD.0	fFourCorner	84100
U+91DD.0	fHakka	  zhim1; zim1; zihm1; zhim5
U+91DD.0	fCantonese	zam1; kim4
U+91DD.0	fMandarin	zhen1; zhen4
U+91DD.0	fEnglish	[1] needle; pin; probe [2] stitch
U+91DE.0	fUTF8	釞
U+91DE.0	fCNS11643	3-342F
U+91DE.0	fGB	5-7535
U+91DE.0	fR/S	  167.2
U+91DE.0	fTotalStrokes	 10
U+91DE.0	fCangjie	CO
U+91DE.0	fFourCorner	88100
U+91DE.0	fCantonese	zap1
U+91DE.0	fMandarin	zhi2
U+91DF.0	fUTF8	釟
U+91DF.0	fCNS11643	3-342E
U+91DF.0	fGB	E-412F
U+91DF.0	fR/S	 167.2
U+91DF.0	fTotalStrokes	 10
U+91DF.0	fCangjie	CC
U+91DF.0	fFourCorner	88100
U+91DF.0	fCantonese	baat3
U+91DF.0	fMandarin	ba1
U+91DF.0	fEnglish	chemical element 108 (chemistry)
U+91E0.0	fUTF8	釠
U+91E0.0	fCNS11643	3-3431
U+91E0.0	fGB	E-4130
U+91E0.0	fR/S	 167.2
U+91E0.0	fTotalStrokes	 10
U+91E0.0	fCangjie	CHN
U+91E0.0	fFourCorner	87110
U+91E0.0	fCantonese	lyun6
U+91E0.0	fMandarin	luan4
U+91E1.0	fUTF8	釡
U+91E1.0	fGB	E-4131
U+91E1.0	fR/S	 167.2
U+91E1.0	fTotalStrokes	 10
U+91E1.0	fCangjie	XCC
U+91E1.0	fCantonese	fu2
U+91E1.0	fMandarin	fu3
U+91E1.0	fEnglish	cauldron; pot; kettle
U+91E2.0	fUTF8	釢
U+91E2.0	fCNS11643	2-3346
U+91E2.0	fBig5	D443
U+91E2.0	fGB	E-4132
U+91E2.0	fR/S	 167.2
U+91E2.0	fTotalStrokes	 10
U+91E2.0	fCangjie	CNHS
U+91E2.0	fFourCorner	87127
U+91E2.0	fHakka	 lai1
U+91E2.0	fCantonese	naai5; laai5 (also)
U+91E2.0	fMandarin	nai2; nai3
U+91E3.0	fUTF8	釣
U+91E3.0	fCNS11643	1-5D6C
U+91E3.0	fBig5	B3A8
U+91E3.0	fGB	1-3576
U+91E3.0	fR/S	 167.3
U+91E3.0	fTotalStrokes	 11
U+91E3.0	fCangjie	CPI
U+91E3.0	fFourCorner	87120
U+91E3.0	fHakka	  diau5
U+91E3.0	fCantonese	diu3
U+91E3.0	fMandarin	diao4
U+91E3.0	fEnglish	[1] fish (with a hook and a line); angle [2] lure; tempt [3] fishhook
U+91E4.0	fUTF8	釤
U+91E4.0	fCNS11643	2-3A21
U+91E4.0	fBig5	D7FC
U+91E4.0	fGB	1-6E4C
U+91E4.0	fR/S	 167.3
U+91E4.0	fTotalStrokes	 11
U+91E4.0	fCangjie	CHHH
U+91E4.0	fFourCorner	82122
U+91E4.0	fHakka	  cam5;  sam3; sam1; sam5
U+91E4.0	fCantonese	saam1; saam3; sin3 (also)
U+91E4.0	fMandarin	shan4; shan1; xian1
U+91E4.0	fEnglish	samarium
U+91E5.0	fUTF8	釥
U+91E5.0	fCNS11643	3-3958
U+91E5.0	fGB	3-755E
U+91E5.0	fR/S	 167.3
U+91E5.0	fTotalStrokes	 11
U+91E5.0	fCangjie	CF
U+91E5.0	fFourCorner	89100
U+91E5.0	fCantonese	ciu2
U+91E5.0	fMandarin	qiao3
U+91E6.0	fUTF8	釦
U+91E6.0	fCNS11643	1-5D6B
U+91E6.0	fBig5	B3A7
U+91E6.0	fGB	E-4133
U+91E6.0	fR/S	 167.3
U+91E6.0	fTotalStrokes	 11
U+91E6.0	fCangjie	CR
U+91E6.0	fFourCorner	86100
U+91E6.0	fHakka	 keu5; k'ieu5; kiu5
U+91E6.0	fCantonese	kau3
U+91E6.0	fMandarin	kou4; kou3
U+91E6.0	fEnglish	 button (on garments)
U+91E7.0	fUTF8	釧
U+91E7.0	fCNS11643	1-5D6D
U+91E7.0	fBig5	B3A9
U+91E7.0	fGB	1-6E4B
U+91E7.0	fR/S	 167.3
U+91E7.0	fTotalStrokes	 11
U+91E7.0	fCangjie	CLLL
U+91E7.0	fFourCorner	82100
U+91E7.0	fHakka	 con5;  chon5
U+91E7.0	fCantonese	cyun3; cyun1 (also)
U+91E7.0	fMandarin	chuan4; chuan1
U+91E7.0	fEnglish	 bracelet; armlet
U+91E8.0	fUTF8	釨
U+91E8.0	fCNS11643	2-3A26
U+91E8.0	fBig5	D842
U+91E8.0	fGB	E-4134
U+91E8.0	fR/S	 167.3
U+91E8.0	fTotalStrokes	 11
U+91E8.0	fCangjie	CND
U+91E8.0	fFourCorner	87147
U+91E8.0	fCantonese	zi2
U+91E8.0	fMandarin	zi3
U+91E9.0	fUTF8	釩
U+91E9.0	fCNS11643	1-5D6F
U+91E9.0	fBig5	B3AB
U+91E9.0	fGB	1-3730
U+91E9.0	fR/S	 167.3
U+91E9.0	fTotalStrokes	 11
U+91E9.0	fCangjie	CHNI
U+91E9.0	fFourCorner	87110
U+91E9.0	fHakka	 fam2
U+91E9.0	fCantonese	faan4
U+91E9.0	fMandarin	fan2; fan3; fan4
U+91E9.0	fEnglish	vanadium
U+91EA.0	fUTF8	釪
U+91EA.0	fCNS11643	2-3A23
U+91EA.0	fBig5	D7FE
U+91EA.0	fGB	3-7556
U+91EA.0	fR/S	 167.3
U+91EA.0	fTotalStrokes	 11
U+91EA.0	fCangjie	CMD
U+91EA.0	fFourCorner	81140
U+91EA.0	fHakka	 fa2
U+91EA.0	fCantonese	waa4; jyu4
U+91EA.0	fMandarin	yu2
U+91EA.0	fEnglish	 alms bowl small bell
U+91EB.0	fUTF8	釫
U+91EB.0	fCNS11643	2-3A24
U+91EB.0	fBig5	D840
U+91EB.0	fGB	E-4135
U+91EB.0	fR/S	 167.3
U+91EB.0	fTotalStrokes	 11
U+91EB.0	fCangjie	CMMS
U+91EB.0	fFourCorner	81127
U+91EB.0	fHakka	 fa2; fu2
U+91EB.0	fCantonese	waa4; wu1
U+91EB.0	fMandarin	hua2; hu2; she2; wu1
U+91EB.0	fEnglish	[1] trowel [2] plaster
U+91EC.0	fUTF8	釬
U+91EC.0	fCNS11643	2-397A
U+91EC.0	fBig5	D7F7
U+91EC.0	fGB	E-4136
U+91EC.0	fR/S	 167.3
U+91EC.0	fTotalStrokes	 11
U+91EC.0	fCangjie	CMJ
U+91EC.0	fFourCorner	81140
U+91EC.0	fHakka	 hon5; hon6; hon3
U+91EC.0	fCantonese	hon6
U+91EC.0	fMandarin	han4; gan1
U+91EC.0	fEnglish	solder
U+91ED.0	fUTF8	釭
U+91ED.0	fCNS11643	1-5D6E
U+91ED.0	fBig5	B3AA
U+91ED.0	fGB	3-7557
U+91ED.0	fR/S	 167.3
U+91ED.0	fTotalStrokes	 11
U+91ED.0	fCangjie	CM
U+91ED.0	fFourCorner	81110
U+91ED.0	fHakka	 gong1
U+91ED.0	fCantonese	gong1; gung1
U+91ED.0	fMandarin	gong1; gang1
U+91ED.0	fEnglish	 tire of wheel band
U+91EE.0	fUTF8	釮
U+91EE.0	fCNS11643	2-3A27
U+91EE.0	fBig5	D843
U+91EE.0	fGB	E-4137
U+91EE.0	fR/S	 167.3
U+91EE.0	fTotalStrokes	 11
U+91EE.0	fCangjie	CDH
U+91EE.0	fFourCorner	84100
U+91EE.0	fCantonese	cai4
U+91EE.0	fMandarin	qi2
U+91EF.0	fUTF8	釯
U+91EF.0	fCNS11643	4-357B
U+91EF.0	fGB	E-4138
U+91EF.0	fR/S	 167.3
U+91EF.0	fTotalStrokes	 11
U+91EF.0	fCangjie	CYV
U+91EF.0	fFourCorner	80110
U+91EF.0	fHakka	 mong2
U+91EF.0	fCantonese	mong4
U+91EF.0	fMandarin	mang2
U+91EF.0	fEnglish	 point of a sword sharp point
U+91F0.0	fUTF8	釰
U+91F0.0	fCNS11643	3-3957
U+91F0.0	fGB	3-755D
U+91F0.0	fR/S	  167.3
U+91F0.0	fTotalStrokes	 11
U+91F0.0	fCangjie	CSHI
U+91F0.0	fFourCorner	87120
U+91F0.0	fCantonese	jat6
U+91F0.0	fMandarin	ri4; jian4
U+91F1.0	fUTF8	釱
U+91F1.0	fCNS11643	2-397C
U+91F1.0	fBig5	D7F9
U+91F1.0	fGB	3-7558
U+91F1.0	fR/S	 167.3
U+91F1.0	fTotalStrokes	 11
U+91F1.0	fCangjie	CK
U+91F1.0	fFourCorner	84130
U+91F1.0	fHakka	 ti5; ti6
U+91F1.0	fCantonese	daai6 (also); dai6
U+91F1.0	fMandarin	di4; dai4; tai3
U+91F1.0	fEnglish	 fetters fetter
U+91F2.0	fUTF8	釲
U+91F2.0	fCNS11643	3-3956
U+91F2.0	fGB	5-7538
U+91F2.0	fR/S	  167.3
U+91F2.0	fTotalStrokes	 11
U+91F2.0	fCangjie	CRU
U+91F2.0	fFourCorner	87110
U+91F2.0	fCantonese	zi6
U+91F2.0	fMandarin	si4
U+91F3.0	fUTF8	釳
U+91F3.0	fCNS11643	2-397D
U+91F3.0	fBig5	D7FA
U+91F3.0	fGB	3-755A
U+91F3.0	fR/S	 167.3
U+91F3.0	fTotalStrokes	 11
U+91F3.0	fCangjie	CON
U+91F3.0	fFourCorner	88117
U+91F3.0	fCantonese	hat1
U+91F3.0	fMandarin	xi4; yi4
U+91F4.0	fUTF8	釴
U+91F4.0	fCNS11643	2-397B
U+91F4.0	fBig5	D7F8
U+91F4.0	fGB	3-7559
U+91F4.0	fR/S	 167.3
U+91F4.0	fTotalStrokes	 11
U+91F4.0	fCangjie	CIP
U+91F4.0	fFourCorner	83140
U+91F4.0	fHakka	 jit8; rit8
U+91F4.0	fCantonese	jik6
U+91F4.0	fMandarin	yi4
U+91F4.0	fEnglish	the ears or side-ornaments of a tripod
U+91F5.0	fUTF8	釵
U+91F5.0	fCNS11643	1-5D6A
U+91F5.0	fBig5	B3A6
U+91F5.0	fGB	1-6E4E
U+91F5.0	fR/S	 167.3
U+91F5.0	fTotalStrokes	 11
U+91F5.0	fCangjie	CEI
U+91F5.0	fFourCorner	87140
U+91F5.0	fHakka	  cai1
U+91F5.0	fCantonese	caai1; caa1
U+91F5.0	fMandarin	chai1; cha1
U+91F5.0	fEnglish	 ornamental hairpin; a kind of hair pin
U+91F6.0	fUTF8	釶
U+91F6.0	fCNS11643	3-3959
U+91F6.0	fGB	5-7539
U+91F6.0	fR/S	  167.3
U+91F6.0	fTotalStrokes	 11
U+91F6.0	fCangjie	CPD
U+91F6.0	fFourCorner	84110
U+91F6.0	fCantonese	si1; se4 (also); jaa5
U+91F6.0	fMandarin	shi1; yi2; she2
U+91F7.0	fUTF8	釷
U+91F7.0	fCNS11643	2-3A25
U+91F7.0	fBig5	D841
U+91F7.0	fGB	1-6E4A
U+91F7.0	fR/S	 167.3
U+91F7.0	fTotalStrokes	 11
U+91F7.0	fCangjie	CG
U+91F7.0	fFourCorner	84110
U+91F7.0	fHakka	 tu3
U+91F7.0	fCantonese	tou2
U+91F7.0	fMandarin	tu3
U+91F7.0	fEnglish	thorium
U+91F8.0	fUTF8	釸
U+91F8.0	fCNS11643	2-397E
U+91F8.0	fBig5	D7FB
U+91F8.0	fGB	E-4139
U+91F8.0	fR/S	 167.3
U+91F8.0	fTotalStrokes	 11
U+91F8.0	fCangjie	CNI
U+91F8.0	fFourCorner	87120
U+91F8.0	fCantonese	zik6
U+91F8.0	fMandarin	xi4
U+91F9.0	fUTF8	釹
U+91F9.0	fCNS11643	2-3A22
U+91F9.0	fBig5	D7FD
U+91F9.0	fGB	1-6E4F
U+91F9.0	fR/S	 167.3
U+91F9.0	fTotalStrokes	 11
U+91F9.0	fCangjie	CV
U+91F9.0	fFourCorner	84144
U+91F9.0	fCantonese	neoi5; leoi5 (also); neoi4 (also); leoi4 (also)
U+91F9.0	fMandarin	nuu3; n3
U+91F9.0	fEnglish	neodymium
U+91FA.0	fUTF8	釺
U+91FA.0	fGB	1-4725
U+91FA.0	fR/S	 167.3
U+91FA.0	fTotalStrokes	 11
U+91FA.0	fCangjie	CHJ
U+91FA.0	fCantonese	cin1
U+91FA.0	fMandarin	qian1
U+91FA.0	fEnglish	tool for boring holes
U+91FB.0	fUTF8	釻
U+91FB.0	fGB	E-413A
U+91FB.0	fR/S	 167.3
U+91FB.0	fTotalStrokes	 11
U+91FB.0	fCangjie	CKNI
U+91FB.0	fCantonese	jyun4
U+91FC.0	fUTF8	釼
U+91FC.0	fGB	E-413B
U+91FC.0	fR/S	 167.3
U+91FC.0	fTotalStrokes	 11
U+91FC.0	fCangjie	CSK
U+91FC.0	fCantonese	jat6
U+91FC.0	fMandarin	jian4
U+91FC.0	fEnglish	sword; dagger; saber
U+91FD.0	fUTF8	釽
U+91FD.0	fCNS11643	2-4134
U+91FD.0	fBig5	DC6D
U+91FD.0	fGB	5-7540
U+91FD.0	fR/S	 167.4
U+91FD.0	fTotalStrokes	 12
U+91FD.0	fCangjie	CHLO
U+91FD.0	fFourCorner	82110
U+91FD.0	fCantonese	pik1; zaau2
U+91FD.0	fMandarin	pi1; pai1; pi4; po4
U+91FD.0	fEnglish	a farmer's hoeing fork
U+91FE.0	fUTF8	釾
U+91FE.0	fCNS11643	3-3F5D
U+91FE.0	fGB	5-753D
U+91FE.0	fR/S	 167.4
U+91FE.0	fTotalStrokes	 12
U+91FE.0	fCangjie	CMVH
U+91FE.0	fFourCorner	81120
U+91FE.0	fHakka	 ja2; ra2
U+91FE.0	fCantonese	je4; ngaa4; aa4 (also)
U+91FE.0	fMandarin	ye2
U+91FE.0	fEnglish	the name of a famous sword
U+91FF.0	fUTF8	釿
U+91FF.0	fCNS11643	2-4133
U+91FF.0	fBig5	DC6C
U+91FF.0	fGB	5-753F
U+91FF.0	fR/S	 167.4
U+91FF.0	fTotalStrokes	 12
U+91FF.0	fCangjie	CHML
U+91FF.0	fFourCorner	82121
U+91FF.0	fHakka	  gin1; ngiun3
U+91FF.0	fCantonese	gan1; ngan4; an4
U+91FF.0	fMandarin	yin2; jin1
U+91FF.0	fEnglish	 chop; chip with an adze; smooth
U+9200.0	fUTF8	鈀
U+9200.0	fCNS11643	2-4131
U+9200.0	fBig5	DC6A
U+9200.0	fGB	1-6E59
U+9200.0	fR/S	 167.4
U+9200.0	fTotalStrokes	 12
U+9200.0	fCangjie	CAU
U+9200.0	fFourCorner	87117
U+9200.0	fHakka	 pa2; pa3
U+9200.0	fCantonese	baa2; paa4; baa1
U+9200.0	fMandarin	ba3; pa2; ba1; pa1
U+9200.0	fEnglish	palladium
U+9201.0	fUTF8	鈁
U+9201.0	fCNS11643	2-4129
U+9201.0	fBig5	DC62
U+9201.0	fGB	1-6E55
U+9201.0	fR/S	 167.4
U+9201.0	fTotalStrokes	 12
U+9201.0	fCangjie	CYHS
U+9201.0	fFourCorner	80127
U+9201.0	fHakka	 fong1
U+9201.0	fCantonese	fong1
U+9201.0	fMandarin	fang1
U+9201.0	fEnglish	francium
U+9202.0	fUTF8	鈂
U+9202.0	fCNS11643	2-4138
U+9202.0	fBig5	DC71
U+9202.0	fGB	3-7568
U+9202.0	fR/S	 167.4
U+9202.0	fTotalStrokes	 12
U+9202.0	fCangjie	CLBU
U+9202.0	fFourCorner	84112
U+9202.0	fCantonese	cam4; zam3
U+9202.0	fMandarin	chen2; qin2; zhen4
U+9203.0	fUTF8	鈃
U+9203.0	fCNS11643	2-412C
U+9203.0	fBig5	DC65
U+9203.0	fGB	3-755F
U+9203.0	fR/S	 167.4
U+9203.0	fTotalStrokes	 12
U+9203.0	fCangjie	CMT
U+9203.0	fFourCorner	81140
U+9203.0	fCantonese	gin1; jing4
U+9203.0	fMandarin	jian1; xing2
U+9203.0	fEnglish	ancient wine vessel
U+9204.0	fUTF8	鈄
U+9204.0	fCNS11643	2-4136
U+9204.0	fBig5	DC6F
U+9204.0	fGB	1-6E57
U+9204.0	fR/S	 167.4
U+9204.0	fTotalStrokes	 12
U+9204.0	fCangjie	CYJ
U+9204.0	fFourCorner	84100
U+9204.0	fHakka	 deu3
U+9204.0	fCantonese	dau2
U+9204.0	fMandarin	tou3; dou4
U+9204.0	fEnglish	a wine flagon
U+9205.0	fUTF8	鈅
U+9205.0	fCNS11643	2-413D
U+9205.0	fBig5	DC76
U+9205.0	fGB	E-413C
U+9205.0	fR/S	 167.4
U+9205.0	fTotalStrokes	 12
U+9205.0	fCangjie	CB
U+9205.0	fFourCorner	87120
U+9205.0	fCantonese	jyut6; joek6
U+9205.0	fMandarin	yue4; yao4
U+9206.0	fUTF8	鈆
U+9206.0	fCNS11643	2-4135
U+9206.0	fBig5	DC6E
U+9206.0	fGB	E-413D
U+9206.0	fR/S	 167.4
U+9206.0	fTotalStrokes	 12
U+9206.0	fCangjie	CCI
U+9206.0	fFourCorner	88132
U+9206.0	fHakka	 jen2; jan3; ren2; jan2
U+9206.0	fCantonese	jyun4
U+9206.0	fMandarin	yan2; qian1; zhong1
U+9206.0	fEnglish	lead
U+9207.0	fUTF8	鈇
U+9207.0	fCNS11643	1-6260
U+9207.0	fBig5	B679
U+9207.0	fGB	3-7560
U+9207.0	fR/S	 167.4
U+9207.0	fTotalStrokes	 12
U+9207.0	fCangjie	CQO
U+9207.0	fFourCorner	85130
U+9207.0	fHakka	 fu1
U+9207.0	fCantonese	fu1; fu2
U+9207.0	fMandarin	fu1; fu3
U+9207.0	fEnglish	 ax; hatchet
U+9208.0	fUTF8	鈈
U+9208.0	fCNS11643	3-3F66
U+9208.0	fGB	1-6E50
U+9208.0	fR/S	 167.4
U+9208.0	fTotalStrokes	 12
U+9208.0	fCangjie	CMF
U+9208.0	fFourCorner	81190
U+9208.0	fCantonese	bat1
U+9208.0	fMandarin	pi1; bu4
U+9208.0	fEnglish	plutonium
U+9209.0	fUTF8	鈉
U+9209.0	fCNS11643	1-625C
U+9209.0	fBig5	B675
U+9209.0	fGB	1-4446
U+9209.0	fR/S	 167.4
U+9209.0	fTotalStrokes	 12
U+9209.0	fCangjie	COB
U+9209.0	fFourCorner	84127
U+9209.0	fHakka	 nap8; nut8; lap8; nat7
U+9209.0	fCantonese	naap6; laap6 (also); naat3
U+9209.0	fMandarin	na4; rui4
U+9209.0	fEnglish	sodium; natrium
U+920A.0	fUTF8	鈊
U+920A.0	fCNS11643	2-412A
U+920A.0	fBig5	DC63
U+920A.0	fGB	3-7569
U+920A.0	fR/S	 167.4
U+920A.0	fTotalStrokes	 12
U+920A.0	fCangjie	CP
U+920A.0	fFourCorner	83100
U+920A.0	fCantonese	sam1; sam3
U+920A.0	fMandarin	xin1; qin4
U+920B.0	fUTF8	鈋
U+920B.0	fCNS11643	3-3F60
U+920B.0	fGB	3-7564
U+920B.0	fR/S	 167.4
U+920B.0	fTotalStrokes	 12
U+920B.0	fCangjie	COP
U+920B.0	fFourCorner	81110
U+920B.0	fHakka	  ngo2
U+920B.0	fCantonese	ngo4
U+920B.0	fMandarin	e2
U+920B.0	fEnglish	[1] pare [2] scrape
U+920C.0	fUTF8	鈌
U+920C.0	fCNS11643	2-4130
U+920C.0	fBig5	DC69
U+920C.0	fGB	3-756A
U+920C.0	fR/S	 167.4
U+920C.0	fTotalStrokes	 12
U+920C.0	fCangjie	CDK
U+920C.0	fFourCorner	85130
U+920C.0	fCantonese	kyut3
U+920C.0	fMandarin	jue2; gui4
U+920C.0	fEnglish	 pierce; stab take
U+920D.0	fUTF8	鈍
U+920D.0	fCNS11643	1-625E
U+920D.0	fBig5	B677
U+920D.0	fGB	1-365B
U+920D.0	fR/S	 167.4
U+920D.0	fTotalStrokes	 12
U+920D.0	fCangjie	CPU
U+920D.0	fFourCorner	85117
U+920D.0	fHakka	 dun3;  tun5; tun6; tun3; tun2
U+920D.0	fCantonese	deon6
U+920D.0	fMandarin	dun4
U+920D.0	fEnglish	[1] blunt; obtuse; dull [2] dull; obtuse
U+920E.0	fUTF8	鈎
U+920E.0	fCNS11643	3-3F68
U+920E.0	fGB	1-3933
U+920E.0	fR/S	 167.4
U+920E.0	fTotalStrokes	 12
U+920E.0	fCangjie	CPI
U+920E.0	fFourCorner	87120
U+920E.0	fHakka	  geu1;  kieu1; giu1
U+920E.0	fCantonese	ngau1; au1 (also); gau1
U+920E.0	fMandarin	gou1
U+920E.0	fEnglish	 hook
U+920F.0	fUTF8	鈏
U+920F.0	fCNS11643	2-412F
U+920F.0	fBig5	DC68
U+920F.0	fGB	3-756B
U+920F.0	fR/S	 167.4
U+920F.0	fTotalStrokes	 12
U+920F.0	fCangjie	CNL
U+920F.0	fFourCorner	82100
U+920F.0	fCantonese	jan6; jan5
U+920F.0	fMandarin	yin3; shen4; yin4
U+9210.0	fUTF8	鈐
U+9210.0	fCNS11643	1-625F
U+9210.0	fBig5	B678
U+9210.0	fGB	1-6E54
U+9210.0	fR/S	 167.4
U+9210.0	fTotalStrokes	 12
U+9210.0	fCangjie	COIN
U+9210.0	fFourCorner	88127
U+9210.0	fHakka	 kiam2
U+9210.0	fCantonese	kim4
U+9210.0	fMandarin	qian2; han2; jin1
U+9210.0	fEnglish	[1] lock; latch [2] stamp; seal; chop
U+9211.0	fUTF8	鈑
U+9211.0	fCNS11643	1-6261
U+9211.0	fBig5	B67A
U+9211.0	fGB	1-6E53
U+9211.0	fR/S	 167.4
U+9211.0	fTotalStrokes	 12
U+9211.0	fCangjie	CHE
U+9211.0	fFourCorner	81147
U+9211.0	fHakka	 ban3
U+9211.0	fCantonese	baan2
U+9211.0	fMandarin	ban3
U+9211.0	fEnglish	plate
U+9212.0	fUTF8	鈒
U+9212.0	fCNS11643	2-4132
U+9212.0	fBig5	DC6B
U+9212.0	fGB	3-755B
U+9212.0	fR/S	 167.4
U+9212.0	fTotalStrokes	 12
U+9212.0	fCangjie	CNHE
U+9212.0	fFourCorner	87147
U+9212.0	fHakka	  sap7; ep7
U+9212.0	fCantonese	zaap6; kap1 (also); saap3
U+9212.0	fMandarin	ji2; sa4; se4; xi1
U+9212.0	fEnglish	spear
U+9213.0	fUTF8	鈓
U+9213.0	fCNS11643	3-3F69
U+9213.0	fGB	E-413E
U+9213.0	fR/S	  167.4
U+9213.0	fTotalStrokes	 12
U+9213.0	fCangjie	CHG
U+9213.0	fFourCorner	81114
U+9213.0	fCantonese	jam5
U+9213.0	fMandarin	ren2
U+9214.0	fUTF8	鈔
U+9214.0	fCNS11643	1-6259
U+9214.0	fBig5	B672
U+9214.0	fGB	1-332E
U+9214.0	fR/S	 167.4
U+9214.0	fTotalStrokes	 12
U+9214.0	fCangjie	CFH
U+9214.0	fFourCorner	89120
U+9214.0	fHakka	  cau1; cau5
U+9214.0	fCantonese	caau1; caau3
U+9214.0	fMandarin	chao1; chao3; miao3; chao4
U+9214.0	fEnglish	[1] paper money; bank notes [2] copy; transcribe
U+9215.0	fUTF8	鈕
U+9215.0	fCNS11643	1-625A
U+9215.0	fBig5	B673
U+9215.0	fGB	1-4525
U+9215.0	fR/S	 167.4
U+9215.0	fTotalStrokes	 12
U+9215.0	fCangjie	CNG
U+9215.0	fFourCorner	87115
U+9215.0	fHakka	 neu3; lieu3; liu3
U+9215.0	fCantonese	nau2; lau2 (also); nau5
U+9215.0	fMandarin	niu3; chou3
U+9215.0	fEnglish	[1] button; knob [2] a Chinese family name
U+9216.0	fUTF8	鈖
U+9216.0	fCNS11643	2-413E
U+9216.0	fBig5	DC77
U+9216.0	fGB	E-413F
U+9216.0	fR/S	 167.4
U+9216.0	fTotalStrokes	 12
U+9216.0	fCangjie	CCSH
U+9216.0	fFourCorner	88107
U+9216.0	fCantonese	fan1
U+9216.0	fMandarin	fen1
U+9217.0	fUTF8	鈗
U+9217.0	fCNS11643	2-413C
U+9217.0	fBig5	DC75
U+9217.0	fGB	E-4140
U+9217.0	fR/S	 167.4
U+9217.0	fTotalStrokes	 12
U+9217.0	fCangjie	CIHU
U+9217.0	fFourCorner	83110
U+9217.0	fCantonese	wan5; deoi6; jeoi6
U+9217.0	fMandarin	yun3; dui4; rui4
U+9218.0	fUTF8	鈘
U+9218.0	fCNS11643	3-3F62
U+9218.0	fGB	E-4141
U+9218.0	fR/S	 167.4
U+9218.0	fTotalStrokes	 12
U+9218.0	fCangjie	CJE
U+9218.0	fFourCorner	84140
U+9218.0	fCantonese	ngai5; ai5 (also)
U+9218.0	fMandarin	ji3; yi3
U+9219.0	fUTF8	鈙
U+9219.0	fCNS11643	2-413B
U+9219.0	fBig5	DC74
U+9219.0	fGB	5-753E
U+9219.0	fR/S	 167.4
U+9219.0	fTotalStrokes	 12
U+9219.0	fCangjie	CYE
U+9219.0	fFourCorner	81147
U+9219.0	fCantonese	kam4; gam6
U+9219.0	fMandarin	qin2
U+921A.0	fUTF8	鈚
U+921A.0	fCNS11643	2-412D
U+921A.0	fBig5	DC66
U+921A.0	fGB	5-753C
U+921A.0	fR/S	 167.4
U+921A.0	fTotalStrokes	 12
U+921A.0	fCangjie	CPP
U+921A.0	fFourCorner	81111
U+921A.0	fHakka	 pi2; bi5
U+921A.0	fCantonese	pai1; pei1 (also); pei4
U+921A.0	fMandarin	pi2; pi1
U+921A.0	fEnglish	[1] ploughshare [2] barb of an arrow
U+921B.0	fUTF8	鈛
U+921B.0	fCNS11643	3-3F65
U+921B.0	fGB	E-4142
U+921B.0	fR/S	 167.4
U+921B.0	fTotalStrokes	 12
U+921B.0	fCangjie	CI
U+921B.0	fFourCorner	83150
U+921B.0	fCantonese	gwo1; go1 (also)
U+921B.0	fMandarin	guo1
U+921C.0	fUTF8	鈜
U+921C.0	fCNS11643	2-4139
U+921C.0	fBig5	DC72
U+921C.0	fGB	E-4143
U+921C.0	fR/S	 167.4
U+921C.0	fTotalStrokes	 12
U+921C.0	fCangjie	CKI
U+921C.0	fFourCorner	84132
U+921C.0	fCantonese	wang4
U+921C.0	fMandarin	hong2
U+921D.0	fUTF8	鈝
U+921D.0	fCNS11643	4-3B66
U+921D.0	fGB	E-4144
U+921D.0	fR/S	 167.4
U+921D.0	fTotalStrokes	 12
U+921D.0	fCangjie	CHQ
U+921D.0	fFourCorner	85100
U+921D.0	fCantonese	jam4
U+921D.0	fMandarin	yin2
U+921E.0	fUTF8	鈞
U+921E.0	fCNS11643	1-625D
U+921E.0	fBig5	B676
U+921E.0	fGB	1-3E7B
U+921E.0	fR/S	 167.4
U+921E.0	fTotalStrokes	 12
U+921E.0	fCangjie	CPIM
U+921E.0	fFourCorner	87120
U+921E.0	fHakka	  giun1
U+921E.0	fCantonese	gwan1; kwan1 (also)
U+921E.0	fMandarin	jun1
U+921E.0	fEnglish	[1] a unit of weight (equal to 30 catties) in former times [2] you; your (in addressing a superior in a letter)
U+921F.0	fUTF8	鈟
U+921F.0	fCNS11643	6-4F6F
U+921F.0	fGB	E-4145
U+921F.0	fR/S	 167.4
U+921F.0	fTotalStrokes	 12
U+921F.0	fCangjie	CNL
U+921F.0	fFourCorner	87120
U+921F.0	fCantonese	diu3
U+921F.0	fMandarin	diao4; shi1
U+9220.0	fUTF8	鈠
U+9220.0	fCNS11643	3-3F5E
U+9220.0	fGB	3-7567
U+9220.0	fR/S	 167.4
U+9220.0	fTotalStrokes	 12
U+9220.0	fCangjie	CHNE
U+9220.0	fFourCorner	87140
U+9220.0	fCantonese	jik6
U+9220.0	fMandarin	yi4
U+9221.0	fUTF8	鈡
U+9221.0	fCNS11643	3-6659
U+9221.0	fGB	E-4146
U+9221.0	fR/S	 167.4
U+9221.0	fTotalStrokes	 12
U+9221.0	fCangjie	CL
U+9221.0	fFourCorner	8510
U+9221.0	fCantonese	zung1
U+9221.0	fMandarin	zhong1
U+9222.0	fUTF8	鈢
U+9222.0	fCNS11643	3-3F63
U+9222.0	fGB	E-4147
U+9222.0	fR/S	 167.4
U+9222.0	fTotalStrokes	 12
U+9222.0	fCangjie	CD
U+9222.0	fFourCorner	84190
U+9222.0	fCantonese	saai2
U+9222.0	fMandarin	nie1
U+9223.0	fUTF8	鈣
U+9223.0	fCNS11643	1-625B
U+9223.0	fBig5	B674
U+9223.0	fGB	1-3846
U+9223.0	fR/S	 167.4
U+9223.0	fTotalStrokes	 12
U+9223.0	fCangjie	CMYS
U+9223.0	fFourCorner	81127
U+9223.0	fHakka	 goi5
U+9223.0	fCantonese	koi3
U+9223.0	fMandarin	gai4
U+9223.0	fEnglish	calcium
U+9224.0	fUTF8	鈤
U+9224.0	fCNS11643	2-413A
U+9224.0	fBig5	DC73
U+9224.0	fGB	E-4148
U+9224.0	fR/S	 167.4
U+9224.0	fTotalStrokes	 12
U+9224.0	fCangjie	CA
U+9224.0	fFourCorner	86100
U+9224.0	fCantonese	jat6
U+9224.0	fMandarin	ri4
U+9225.0	fUTF8	鈥
U+9225.0	fCNS11643	2-412B
U+9225.0	fBig5	DC64
U+9225.0	fGB	1-6E58
U+9225.0	fR/S	 167.4
U+9225.0	fTotalStrokes	 12
U+9225.0	fCangjie	CF
U+9225.0	fFourCorner	89180
U+9225.0	fCantonese	fo2
U+9225.0	fMandarin	huo3; huo2
U+9225.0	fEnglish	holmium
U+9226.0	fUTF8	鈦
U+9226.0	fCNS11643	2-412E
U+9226.0	fBig5	DC67
U+9226.0	fGB	1-6E51
U+9226.0	fR/S	 167.4
U+9226.0	fTotalStrokes	 12
U+9226.0	fCangjie	CKI
U+9226.0	fFourCorner	84130
U+9226.0	fHakka	 tai5
U+9226.0	fCantonese	taai3
U+9226.0	fMandarin	tai4; tai3
U+9226.0	fEnglish	titanium
U+9227.0	fUTF8	鈧
U+9227.0	fCNS11643	2-4137
U+9227.0	fBig5	DC70
U+9227.0	fGB	1-6E56
U+9227.0	fR/S	 167.4
U+9227.0	fTotalStrokes	 12
U+9227.0	fCangjie	CYHN
U+9227.0	fFourCorner	80117
U+9227.0	fCantonese	kong3
U+9227.0	fMandarin	kang4
U+9227.0	fEnglish	scandium
U+9228.0	fUTF8	鈨
U+9228.0	fGB	E-4149
U+9228.0	fR/S	 167.4
U+9228.0	fTotalStrokes	 12
U+9228.0	fCangjie	CMMU
U+9228.0	fFourCorner	8111
U+9228.0	fCantonese	jyun4
U+9229.0	fUTF8	鈩
U+9229.0	fGB	E-414A
U+9229.0	fR/S	 167.4
U+9229.0	fTotalStrokes	 12
U+9229.0	fCangjie	CIS
U+9229.0	fCantonese	lou4
U+9229.0	fMandarin	lu2
U+9229.0	fEnglish	 fireplace; stove; oven; furnace
U+922A.0	fUTF8	鈪
U+922A.0	fGB	1-7C54
U+922A.0	fR/S	 167.4
U+922A.0	fTotalStrokes	 12
U+922A.0	fCangjie	CMSU
U+922A.0	fFourCorner	8111
U+922A.0	fHakka	  ak7
U+922A.0	fCantonese	aak3; aak2 (also); ngaak3 (also); ngaak2 (also)
U+922A.0	fEnglish	[1] bracelet [2] bangle
U+922B.0	fUTF8	鈫
U+922B.0	fCNS11643	3-6658
U+922B.0	fGB	E-414B
U+922B.0	fR/S	 167.4
U+922B.0	fTotalStrokes	 12
U+922B.0	fCangjie	CYK
U+922B.0	fFourCorner	8014
U+922B.0	fCantonese	man4
U+922C.0	fUTF8	鈬
U+922C.0	fGB	E-414C
U+922C.0	fR/S	 167.4
U+922C.0	fTotalStrokes	 12
U+922C.0	fCangjie	CSO
U+922C.0	fCantonese	dok6
U+922C.0	fMandarin	duo2
U+922C.0	fEnglish	bell; a Chinese family name
U+922D.0	fUTF8	鈭
U+922D.0	fCNS11643	2-4F22
U+922D.0	fBig5	E4BA
U+922D.0	fGB	E-414D
U+922D.0	fR/S	 167.5
U+922D.0	fTotalStrokes	 13
U+922D.0	fCangjie	YPC
U+922D.0	fFourCorner	21109
U+922D.0	fCantonese	zi1; ci1
U+922D.0	fMandarin	zi1; ci1
U+922E.0	fUTF8	鈮
U+922E.0	fCNS11643	2-483D
U+922E.0	fBig5	E0B7
U+922E.0	fGB	1-6E6A
U+922E.0	fR/S	 167.5
U+922E.0	fTotalStrokes	 13
U+922E.0	fCangjie	CSP
U+922E.0	fFourCorner	87111
U+922E.0	fCantonese	nei4; nei6 (also); lei4 (also); lei6 (also)
U+922E.0	fMandarin	ni2; ni3; ni4
U+922E.0	fEnglish	niobium
U+922F.0	fUTF8	鈯
U+922F.0	fCNS11643	3-452E
U+922F.0	fGB	3-7577
U+922F.0	fR/S	 167.5
U+922F.0	fTotalStrokes	 13
U+922F.0	fCangjie	CUU
U+922F.0	fFourCorner	82172
U+922F.0	fCantonese	dat6
U+922F.0	fMandarin	tu2
U+9230.0	fUTF8	鈰
U+9230.0	fCNS11643	2-4836
U+9230.0	fBig5	E0B0
U+9230.0	fGB	1-6E66
U+9230.0	fR/S	 167.5
U+9230.0	fTotalStrokes	 13
U+9230.0	fCangjie	CYLB
U+9230.0	fFourCorner	80127
U+9230.0	fCantonese	si5
U+9230.0	fMandarin	shi4
U+9230.0	fEnglish	cerium
U+9231.0	fUTF8	鈱
U+9231.0	fCNS11643	2-4849
U+9231.0	fBig5	E0C3
U+9231.0	fGB	5-7547
U+9231.0	fR/S	 167.5
U+9231.0	fTotalStrokes	 13
U+9231.0	fCangjie	CRVP
U+9231.0	fFourCorner	87147
U+9231.0	fCantonese	man4
U+9231.0	fMandarin	min2
U+9232.0	fUTF8	鈲
U+9232.0	fCNS11643	2-4852
U+9232.0	fBig5	E0CC
U+9232.0	fGB	E-414E
U+9232.0	fR/S	 167.5
U+9232.0	fTotalStrokes	 13
U+9232.0	fCangjie	CHVO
U+9232.0	fFourCorner	82130
U+9232.0	fCantonese	gu1; gwu1 (also)
U+9232.0	fMandarin	gu1
U+9233.0	fUTF8	鈳
U+9233.0	fCNS11643	2-4839
U+9233.0	fBig5	E0B3
U+9233.0	fGB	1-6E5D
U+9233.0	fR/S	 167.5
U+9233.0	fTotalStrokes	 13
U+9233.0	fCangjie	CMNR
U+9233.0	fFourCorner	81120
U+9233.0	fCantonese	ko1; aa3; ho1; o1
U+9233.0	fMandarin	ke1; e3; e1
U+9233.0	fEnglish	columbium
U+9234.0	fUTF8	鈴
U+9234.0	fCNS11643	1-6749
U+9234.0	fBig5	B961
U+9234.0	fGB	1-4165
U+9234.0	fR/S	 167.5
U+9234.0	fTotalStrokes	 13
U+9234.0	fCangjie	COII
U+9234.0	fFourCorner	88137
U+9234.0	fHakka	  lang2; liang2; long3; lang3; liang5; liang6
U+9234.0	fCantonese	ling4; ling1 (also); ling2 (also)
U+9234.0	fMandarin	ling2
U+9234.0	fEnglish	jingling bells
U+9235.0	fUTF8	鈵
U+9235.0	fCNS11643	3-452B
U+9235.0	fGB	E-414F
U+9235.0	fR/S	 167.5
U+9235.0	fTotalStrokes	 13
U+9235.0	fCangjie	CMOB
U+9235.0	fFourCorner	81120
U+9235.0	fCantonese	bing3
U+9235.0	fMandarin	bing4; bing3
U+9236.0	fUTF8	鈶
U+9236.0	fCNS11643	2-4846
U+9236.0	fBig5	E0C0
U+9236.0	fGB	3-757A
U+9236.0	fR/S	 167.5
U+9236.0	fTotalStrokes	 13
U+9236.0	fCangjie	CIR
U+9236.0	fFourCorner	83160
U+9236.0	fHakka	 sii5; ji2; sii6; ri2
U+9236.0	fCantonese	zi6; ci4; ci5; toi4
U+9236.0	fMandarin	yi2; ci2; si4; tai2
U+9237.0	fUTF8	鈷
U+9237.0	fCNS11643	1-673F
U+9237.0	fBig5	B957
U+9237.0	fGB	1-6E5C
U+9237.0	fR/S	 167.5
U+9237.0	fTotalStrokes	 13
U+9237.0	fCangjie	CJR
U+9237.0	fFourCorner	84160
U+9237.0	fHakka	 gu3; gu1
U+9237.0	fCantonese	gu2; gu1 (also); gwu2 (also); gwu1 (also)
U+9237.0	fMandarin	gu1; gu4; hu2; gu2
U+9237.0	fEnglish	cobalt
U+9237.1	fUTF8	鈷
U+9237.1	fCNS11643	1-673F
U+9237.1	fBig5	B957
U+9237.1	fGB	1-6E5C
U+9237.1	fR/S	 167.5
U+9237.1	fTotalStrokes	 13
U+9237.1	fCangjie	CJR
U+9237.1	fFourCorner	84160
U+9237.1	fMandarin	gu3
U+9237.1	fEnglish	 iron
U+9238.0	fUTF8	鈸
U+9238.0	fCNS11643	1-6741
U+9238.0	fBig5	B959
U+9238.0	fGB	1-6E60
U+9238.0	fR/S	 167.5
U+9238.0	fTotalStrokes	 13
U+9238.0	fCangjie	CIKK
U+9238.0	fFourCorner	84140
U+9238.0	fHakka	 pat8; pat7
U+9238.0	fCantonese	bat6; but6 (also)
U+9238.0	fMandarin	ba2; bo2
U+9238.0	fEnglish	cymbals
U+9239.0	fUTF8	鈹
U+9239.0	fCNS11643	1-674D
U+9239.0	fBig5	B965
U+9239.0	fGB	1-6E6B
U+9239.0	fR/S	 167.5
U+9239.0	fTotalStrokes	 13
U+9239.0	fCangjie	CDHE
U+9239.0	fFourCorner	84147
U+9239.0	fHakka	 pi1
U+9239.0	fCantonese	pei4; pei1
U+9239.0	fMandarin	pi2; pi1
U+9239.0	fEnglish	beryllium
U+923A.0	fUTF8	鈺
U+923A.0	fCNS11643	2-4837
U+923A.0	fBig5	E0B1
U+923A.0	fGB	1-6E5A
U+923A.0	fR/S	 167.5
U+923A.0	fTotalStrokes	 13
U+923A.0	fCangjie	CMGI
U+923A.0	fFourCorner	81113
U+923A.0	fHakka	 ngiuk7; ngiuk8; niuk8
U+923A.0	fCantonese	juk6; juk1
U+923A.0	fMandarin	yu4
U+923A.0	fEnglish	a hard variety of gold
U+923B.0	fUTF8	鈻
U+923B.0	fCNS11643	3-4537
U+923B.0	fGB	3-7570
U+923B.0	fR/S	  167.5
U+923B.0	fTotalStrokes	 13
U+923B.0	fCangjie	CRLR
U+923B.0	fFourCorner	87170
U+923B.0	fCantonese	zi6
U+923B.0	fMandarin	si4
U+923C.0	fUTF8	鈼
U+923C.0	fCNS11643	3-452F
U+923C.0	fGB	3-7572
U+923C.0	fR/S	  167.5
U+923C.0	fTotalStrokes	 13
U+923C.0	fCangjie	COS
U+923C.0	fFourCorner	88110
U+923C.0	fCantonese	zok6
U+923C.0	fMandarin	zuo2
U+923D.0	fUTF8	鈽
U+923D.0	fCNS11643	1-6742
U+923D.0	fBig5	B95A
U+923D.0	fGB	1-6E5F
U+923D.0	fR/S	 167.5
U+923D.0	fTotalStrokes	 13
U+923D.0	fCangjie	CKLB
U+923D.0	fFourCorner	84127
U+923D.0	fCantonese	bou1; bou3
U+923D.0	fMandarin	bu1; bu4
U+923D.0	fEnglish	plutonium
U+923E.0	fUTF8	鈾
U+923E.0	fCNS11643	1-6744
U+923E.0	fBig5	B95C
U+923E.0	fGB	1-534B
U+923E.0	fR/S	 167.5
U+923E.0	fTotalStrokes	 13
U+923E.0	fCangjie	CLW
U+923E.0	fFourCorner	85160
U+923E.0	fHakka	 ju2; jiu2
U+923E.0	fCantonese	jau4; jau2 (also)
U+923E.0	fMandarin	you2; zhou4
U+923E.0	fEnglish	uranium
U+923F.0	fUTF8	鈿
U+923F.0	fCNS11643	1-674E
U+923F.0	fBig5	B966
U+923F.0	fGB	1-6E64
U+923F.0	fR/S	 167.5
U+923F.0	fTotalStrokes	 13
U+923F.0	fCangjie	CW
U+923F.0	fFourCorner	86100
U+923F.0	fHakka	  tien2; ten2
U+923F.0	fCantonese	tin4; din6 (also)
U+923F.0	fMandarin	dian4; tian2
U+923F.0	fEnglish	filigree
U+9240.0	fUTF8	鉀
U+9240.0	fCNS11643	1-6743
U+9240.0	fBig5	B95B
U+9240.0	fGB	1-3C58
U+9240.0	fR/S	 167.5
U+9240.0	fTotalStrokes	 13
U+9240.0	fCangjie	CWL
U+9240.0	fFourCorner	86150
U+9240.0	fHakka	 gap7
U+9240.0	fCantonese	gaap3
U+9240.0	fMandarin	jia3; ge2; he2
U+9240.0	fEnglish	potassium
U+9241.0	fUTF8	鉁
U+9241.0	fCNS11643	3-4539
U+9241.0	fGB	E-4150
U+9241.0	fR/S	 167.5
U+9241.0	fTotalStrokes	 13
U+9241.0	fCangjie	COHH
U+9241.0	fFourCorner	88120
U+9241.0	fCantonese	zan1
U+9241.0	fMandarin	zhen1
U+9242.0	fUTF8	鉂
U+9242.0	fCNS11643	4-4141
U+9242.0	fGB	3-756F
U+9242.0	fR/S	  167.5
U+9242.0	fTotalStrokes	 13
U+9242.0	fCangjie	CLK
U+9242.0	fFourCorner	85100
U+9242.0	fCantonese	si2
U+9242.0	fMandarin	shi3
U+9243.0	fUTF8	鉃
U+9243.0	fCNS11643	4-4146
U+9243.0	fGB	E-4151
U+9243.0	fR/S	  167.5
U+9243.0	fTotalStrokes	 13
U+9243.0	fCangjie	COK
U+9243.0	fFourCorner	88130
U+9243.0	fCantonese	si6
U+9243.0	fMandarin	shi4
U+9244.0	fUTF8	鉄
U+9244.0	fCNS11643	3-4538
U+9244.0	fGB	E-4152
U+9244.0	fR/S	 167.5
U+9244.0	fTotalStrokes	 13
U+9244.0	fCangjie	CHQO
U+9244.0	fFourCorner	85130
U+9244.0	fHakka	 tiet7; zhit7; zit7
U+9244.0	fCantonese	tit3
U+9244.0	fMandarin	tie3
U+9244.0	fEnglish	 iron strong; solid; firm
U+9245.0	fUTF8	鉅
U+9245.0	fCNS11643	1-674C
U+9245.0	fBig5	B964
U+9245.0	fGB	1-6E52
U+9245.0	fR/S	 167.5
U+9245.0	fTotalStrokes	 13
U+9245.0	fCangjie	CSS
U+9245.0	fFourCorner	81117
U+9245.0	fHakka	  ki3; ki1
U+9245.0	fCantonese	geoi6
U+9245.0	fMandarin	ju4
U+9245.0	fEnglish	[1] steel; iron [2] great
U+9246.0	fUTF8	鉆
U+9246.0	fCNS11643	2-483F
U+9246.0	fBig5	E0B9
U+9246.0	fGB	E-4153
U+9246.0	fR/S	 167.5
U+9246.0	fTotalStrokes	 13
U+9246.0	fCangjie	CYR
U+9246.0	fFourCorner	81160
U+9246.0	fHakka	 kiam2
U+9246.0	fCantonese	cim1; zyun3; zyun1; kim4; cyun3
U+9246.0	fMandarin	chan1; qian2; zhan1; qin2; tie1; zhen1
U+9246.0	fEnglish	tweezers
U+9247.0	fUTF8	鉇
U+9247.0	fCNS11643	4-4148
U+9247.0	fGB	E-4154
U+9247.0	fR/S	 167.5
U+9247.0	fTotalStrokes	 13
U+9247.0	fCangjie	COPD
U+9247.0	fFourCorner	88110
U+9247.0	fCantonese	si1; se4 (also)
U+9247.0	fMandarin	she2; shi1; yi2
U+9248.0	fUTF8	鉈
U+9248.0	fCNS11643	2-4834
U+9248.0	fBig5	E0AE
U+9248.0	fGB	1-6E68
U+9248.0	fR/S	 167.5
U+9248.0	fTotalStrokes	 13
U+9248.0	fCangjie	CJP
U+9248.0	fFourCorner	83111
U+9248.0	fHakka	 to2; tau2; sa2; ta1
U+9248.0	fCantonese	taa1; to4; si1; to2 (also); se4 (also); ta1
U+9248.0	fMandarin	shi1; she2; tuo2; ta1; yi2; shi4
U+9248.0	fEnglish	thallium
U+9249.0	fUTF8	鉉
U+9249.0	fCNS11643	1-674A
U+9249.0	fBig5	B962
U+9249.0	fGB	1-6E67
U+9249.0	fR/S	 167.5
U+9249.0	fTotalStrokes	 13
U+9249.0	fCangjie	CYVI
U+9249.0	fFourCorner	80132
U+9249.0	fHakka	  hien3; hien2
U+9249.0	fCantonese	jyun5
U+9249.0	fMandarin	xuan4; jian1; juan1; jyong1; xian2
U+9249.0	fEnglish	a device for carrying a tripod
U+924A.0	fUTF8	鉊
U+924A.0	fCNS11643	2-483E
U+924A.0	fBig5	E0B8
U+924A.0	fGB	E-4155
U+924A.0	fR/S	 167.5
U+924A.0	fTotalStrokes	 13
U+924A.0	fCangjie	CSHR
U+924A.0	fFourCorner	87162
U+924A.0	fHakka	 zhau1; zau1
U+924A.0	fCantonese	ziu1; ciu1
U+924A.0	fMandarin	zhao1
U+924A.0	fEnglish	sickle
U+924B.0	fUTF8	鉋
U+924B.0	fCNS11643	1-6746
U+924B.0	fBig5	B95E
U+924B.0	fGB	E-4156
U+924B.0	fR/S	 167.5
U+924B.0	fTotalStrokes	 13
U+924B.0	fCangjie	CPRU
U+924B.0	fFourCorner	87112
U+924B.0	fHakka	 pau2; pau3
U+924B.0	fCantonese	paau4
U+924B.0	fMandarin	bao4; pao2; bao2; bo2
U+924B.0	fEnglish	[1] plane (wood) [2] carpenter's plane
U+924C.0	fUTF8	鉌
U+924C.0	fCNS11643	2-4850
U+924C.0	fBig5	E0CA
U+924C.0	fGB	5-7543
U+924C.0	fR/S	 167.5
U+924C.0	fTotalStrokes	 13
U+924C.0	fCangjie	CHD
U+924C.0	fFourCorner	82194
U+924C.0	fCantonese	wo4
U+924C.0	fMandarin	he2
U+924C.0	fEnglish	the bells on the cart in ancient
U+924D.0	fUTF8	鉍
U+924D.0	fCNS11643	1-674B
U+924D.0	fBig5	B963
U+924D.0	fGB	1-6E69
U+924D.0	fR/S	 167.5
U+924D.0	fTotalStrokes	 13
U+924D.0	fCangjie	CPH
U+924D.0	fFourCorner	83100
U+924D.0	fHakka	 bi5
U+924D.0	fCantonese	bei3; bit1 (also)
U+924D.0	fMandarin	bi4; se4
U+924D.0	fEnglish	bismuth
U+924E.0	fUTF8	鉎
U+924E.0	fCNS11643	2-484E
U+924E.0	fBig5	E0C8
U+924E.0	fGB	3-7571
U+924E.0	fR/S	 167.5
U+924E.0	fTotalStrokes	 13
U+924E.0	fCangjie	CHQM
U+924E.0	fFourCorner	85110
U+924E.0	fHakka	  sang1; siang5; ciang5
U+924E.0	fCantonese	sang1; saang1; sau3
U+924E.0	fMandarin	sheng1; xing1
U+924E.0	fEnglish	iron ore
U+924F.0	fUTF8	鉏
U+924F.0	fCNS11643	2-4842
U+924F.0	fBig5	E0BC
U+924F.0	fGB	E-4157
U+924F.0	fR/S	 167.5
U+924F.0	fTotalStrokes	 13
U+924F.0	fCangjie	CBM
U+924F.0	fFourCorner	87112
U+924F.0	fHakka	 cu2; co2; cho2; ze2; cih2; cio2
U+924F.0	fCantonese	co4; ceoi4 (also); zeoi2; zeoi6; ci4
U+924F.0	fMandarin	chu2; xu2; zhu4; zu1; zu3
U+924F.0	fEnglish	[1] hoe [2] hoe [3] eradicate; eliminate; uproot [4] a Chinese family name
U+924F.1	fUTF8	鉏
U+924F.1	fCNS11643	2-4842
U+924F.1	fBig5	E0BC
U+924F.1	fGB	E-4157
U+924F.1	fR/S	 167.5
U+924F.1	fTotalStrokes	 13
U+924F.1	fCangjie	CBM
U+924F.1	fFourCorner	87112
U+924F.1	fMandarin	ju3
U+924F.1	fEnglish	discordant; disharmonious
U+9250.0	fUTF8	鉐
U+9250.0	fCNS11643	2-484C
U+9250.0	fBig5	E0C6
U+9250.0	fGB	E-4158
U+9250.0	fR/S	 167.5
U+9250.0	fTotalStrokes	 13
U+9250.0	fCangjie	CMR
U+9250.0	fFourCorner	81160
U+9250.0	fHakka	 shak8; sak8; cuk8
U+9250.0	fCantonese	sek6; zuk1
U+9250.0	fMandarin	shi2
U+9250.0	fEnglish	a Chinese family name
U+9251.0	fUTF8	鉑
U+9251.0	fCNS11643	1-6748
U+9251.0	fBig5	B960
U+9251.0	fGB	1-322C
U+9251.0	fR/S	 167.5
U+9251.0	fTotalStrokes	 13
U+9251.0	fCangjie	CHA
U+9251.0	fFourCorner	86100
U+9251.0	fHakka	 pok8; pak8
U+9251.0	fCantonese	bok6
U+9251.0	fMandarin	bo2
U+9251.0	fEnglish	[1] platinum [2] thin sheet of metal; foil
U+9252.0	fUTF8	鉒
U+9252.0	fCNS11643	2-4835
U+9252.0	fBig5	E0AF
U+9252.0	fGB	3-7573
U+9252.0	fR/S	 167.5
U+9252.0	fTotalStrokes	 13
U+9252.0	fCangjie	CYG
U+9252.0	fFourCorner	80114
U+9252.0	fHakka	 zhu5; zu5; zhu6
U+9252.0	fCantonese	zyu3
U+9252.0	fMandarin	zhu4
U+9252.0	fEnglish	[1] inter valuables with the dead [2] indicate the presence of minerals in the soil
U+9253.0	fUTF8	鉓
U+9253.0	fCNS11643	2-484F
U+9253.0	fBig5	E0C9
U+9253.0	fGB	E-4159
U+9253.0	fR/S	 167.5
U+9253.0	fTotalStrokes	 13
U+9253.0	fCangjie	COLB
U+9253.0	fFourCorner	88127
U+9253.0	fCantonese	cik1
U+9253.0	fMandarin	chi4
U+9254.0	fUTF8	鉔
U+9254.0	fCNS11643	2-484A
U+9254.0	fBig5	E0C4
U+9254.0	fGB	E-415A
U+9254.0	fR/S	 167.5
U+9254.0	fTotalStrokes	 13
U+9254.0	fCangjie	CSLB
U+9254.0	fFourCorner	81112
U+9254.0	fHakka	 zap7; zhap7
U+9254.0	fCantonese	zaap3; saap3
U+9254.0	fMandarin	za1
U+9254.0	fEnglish	a heavy iron rake with 3 or 4 long teeth
U+9255.0	fUTF8	鉕
U+9255.0	fCNS11643	3-452A
U+9255.0	fGB	1-6E5E
U+9255.0	fR/S	 167.5
U+9255.0	fTotalStrokes	 13
U+9255.0	fCangjie	CSR
U+9255.0	fFourCorner	81110
U+9255.0	fCantonese	po2
U+9255.0	fMandarin	po1; po3
U+9255.0	fEnglish	promethium bronze
U+9256.0	fUTF8	鉖
U+9256.0	fCNS11643	2-4851
U+9256.0	fBig5	E0CB
U+9256.0	fGB	E-415B
U+9256.0	fR/S	 167.5
U+9256.0	fTotalStrokes	 13
U+9256.0	fCangjie	CHEY
U+9256.0	fFourCorner	87133
U+9256.0	fCantonese	tung4
U+9256.0	fMandarin	tong2
U+9257.0	fUTF8	鉗
U+9257.0	fCNS11643	1-6740
U+9257.0	fBig5	B958
U+9257.0	fGB	1-472F
U+9257.0	fR/S	 167.5
U+9257.0	fTotalStrokes	 13
U+9257.0	fCangjie	CTM
U+9257.0	fFourCorner	84170
U+9257.0	fHakka	  kiam2; kiam3
U+9257.0	fCantonese	kim4; kim2 (also)
U+9257.0	fMandarin	qian2; an1
U+9257.0	fEnglish	[1] pincers; forceps; pliers; tongs; tweezers; clamps [2] chains put around a prisoner's neck [3] hold with tongs, etc.
U+9258.0	fUTF8	鉘
U+9258.0	fCNS11643	4-4140
U+9258.0	fGB	E-415C
U+9258.0	fR/S	  167.5
U+9258.0	fTotalStrokes	 13
U+9258.0	fCangjie	CLLN
U+9258.0	fFourCorner	85120
U+9258.0	fCantonese	fat6
U+9258.0	fMandarin	fu2
U+9259.0	fUTF8	鉙
U+9259.0	fCNS11643	3-452D
U+9259.0	fGB	E-415D
U+9259.0	fR/S	  167.5
U+9259.0	fTotalStrokes	 13
U+9259.0	fCangjie	CRC
U+9259.0	fFourCorner	86180
U+9259.0	fCantonese	zaai2
U+9259.0	fMandarin	zhai3
U+925A.0	fUTF8	鉚
U+925A.0	fCNS11643	1-674F
U+925A.0	fBig5	B967
U+925A.0	fGB	1-432D
U+925A.0	fR/S	 167.5
U+925A.0	fTotalStrokes	 13
U+925A.0	fCangjie	CHHL
U+925A.0	fFourCorner	87120
U+925A.0	fHakka	 liu1
U+925A.0	fCantonese	maau5
U+925A.0	fMandarin	liu3; mao3
U+925A.0	fEnglish	rivet
U+925B.0	fUTF8	鉛
U+925B.0	fCNS11643	1-6745
U+925B.0	fBig5	B95D
U+925B.0	fGB	1-4726
U+925B.0	fR/S	 167.5
U+925B.0	fTotalStrokes	 13
U+925B.0	fCangjie	CCR
U+925B.0	fFourCorner	88160
U+925B.0	fHakka	 jen2; ren2; jan2
U+925B.0	fCantonese	jyun4
U+925B.0	fMandarin	qian1; yan2
U+925B.0	fEnglish	 lead (metal); plumbum
U+925C.0	fUTF8	鉜
U+925C.0	fCNS11643	3-4531
U+925C.0	fGB	5-7544
U+925C.0	fR/S	 167.5
U+925C.0	fTotalStrokes	 13
U+925C.0	fCangjie	CODI
U+925C.0	fFourCorner	84100
U+925C.0	fCantonese	fau4
U+925C.0	fMandarin	fu2
U+925D.0	fUTF8	鉝
U+925D.0	fCNS11643	3-4529
U+925D.0	fGB	E-415E
U+925D.0	fR/S	 167.5
U+925D.0	fTotalStrokes	 13
U+925D.0	fCangjie	CYT
U+925D.0	fFourCorner	80110
U+925D.0	fCantonese	laap6
U+925D.0	fMandarin	li4
U+925E.0	fUTF8	鉞
U+925E.0	fCNS11643	2-483B
U+925E.0	fBig5	E0B5
U+925E.0	fGB	1-6E61
U+925E.0	fR/S	 167.5
U+925E.0	fTotalStrokes	 13
U+925E.0	fCangjie	CIV
U+925E.0	fFourCorner	83150
U+925E.0	fHakka	 jet8; ret8; jat8
U+925E.0	fCantonese	jyut6
U+925E.0	fMandarin	yue4; hui4
U+925E.0	fEnglish	 broad-axe; battle axe; large ax
U+925F.0	fUTF8	鉟
U+925F.0	fCNS11643	3-4534
U+925F.0	fGB	E-415F
U+925F.0	fR/S	 167.5
U+925F.0	fTotalStrokes	 13
U+925F.0	fCangjie	CMFM
U+925F.0	fFourCorner	81110
U+925F.0	fHakka	 pi1
U+925F.0	fCantonese	pei1
U+925F.0	fMandarin	pi1
U+9260.0	fUTF8	鉠
U+9260.0	fCNS11643	2-4843
U+9260.0	fBig5	E0BD
U+9260.0	fGB	5-7542
U+9260.0	fR/S	 167.5
U+9260.0	fTotalStrokes	 13
U+9260.0	fCangjie	CLBK
U+9260.0	fFourCorner	85130
U+9260.0	fCantonese	joeng1
U+9260.0	fMandarin	yang1; ying1
U+9261.0	fUTF8	鉡
U+9261.0	fCNS11643	2-4847
U+9261.0	fBig5	E0C1
U+9261.0	fGB	3-7574
U+9261.0	fR/S	 167.5
U+9261.0	fTotalStrokes	 13
U+9261.0	fCangjie	CFQ
U+9261.0	fFourCorner	89150
U+9261.0	fCantonese	bun6
U+9261.0	fMandarin	ban4
U+9262.0	fUTF8	鉢
U+9262.0	fCNS11643	3-4533
U+9262.0	fGB	1-3227
U+9262.0	fR/S	 167.5
U+9262.0	fTotalStrokes	 13
U+9262.0	fCangjie	CDM
U+9262.0	fFourCorner	85130
U+9262.0	fHakka	  bat7
U+9262.0	fCantonese	but3
U+9262.0	fMandarin	bo1
U+9262.0	fEnglish	[1] earthenware basin or bowl [2] a Buddhist's priest's rice bowl
U+9263.0	fUTF8	鉣
U+9263.0	fCNS11643	2-484B
U+9263.0	fBig5	E0C5
U+9263.0	fGB	E-4160
U+9263.0	fR/S	 167.5
U+9263.0	fTotalStrokes	 13
U+9263.0	fCangjie	CGI
U+9263.0	fFourCorner	84131
U+9263.0	fHakka	 giap7
U+9263.0	fCantonese	gip3; hip3
U+9263.0	fMandarin	jie2
U+9263.0	fEnglish	an iron hook fastened to the girdle
U+9264.0	fUTF8	鉤
U+9264.0	fCNS11643	1-6747
U+9264.0	fBig5	B95F
U+9264.0	fGB	E-4161
U+9264.0	fR/S	 167.5
U+9264.0	fTotalStrokes	 13
U+9264.0	fCangjie	CPR
U+9264.0	fFourCorner	87120
U+9264.0	fHakka	 kieu1; giu1; geu1
U+9264.0	fCantonese	ngau1; au1 (also); gau1
U+9264.0	fMandarin	gou1; gou4; qu2
U+9264.0	fEnglish	[1] hook [2] probe; investigate [3] entice; lure
U+9265.0	fUTF8	鉥
U+9265.0	fCNS11643	2-483A
U+9265.0	fBig5	E0B4
U+9265.0	fGB	3-756C
U+9265.0	fR/S	 167.5
U+9265.0	fTotalStrokes	 13
U+9265.0	fCangjie	CIJC
U+9265.0	fFourCorner	83134
U+9265.0	fCantonese	seot6
U+9265.0	fMandarin	shu4; xu4
U+9265.0	fEnglish	needle
U+9266.0	fUTF8	鉦
U+9266.0	fCNS11643	2-4838
U+9266.0	fBig5	E0B2
U+9266.0	fGB	1-6E5B
U+9266.0	fR/S	 167.5
U+9266.0	fTotalStrokes	 13
U+9266.0	fCangjie	CMYM
U+9266.0	fFourCorner	81111
U+9266.0	fHakka	 zhin1; zihn1
U+9266.0	fCantonese	zing1; zing3
U+9266.0	fMandarin	zheng1
U+9266.0	fEnglish	a kind of gong used in ancient times (used by marching troops)
U+9267.0	fUTF8	鉧
U+9267.0	fCNS11643	2-4844
U+9267.0	fBig5	E0BE
U+9267.0	fGB	3-757B
U+9267.0	fR/S	 167.5
U+9267.0	fTotalStrokes	 13
U+9267.0	fCangjie	CWYI
U+9267.0	fFourCorner	87150
U+9267.0	fCantonese	mou5
U+9267.0	fMandarin	mu3
U+9267.0	fEnglish	 iron
U+9268.0	fUTF8	鉨
U+9268.0	fCNS11643	4-4142
U+9268.0	fGB	E-4162
U+9268.0	fR/S	 167.5
U+9268.0	fTotalStrokes	 13
U+9268.0	fCangjie	CNF
U+9268.0	fFourCorner	87190
U+9268.0	fCantonese	nai5; lai5 (also)
U+9268.0	fMandarin	ni3; nie3; xi3
U+9269.0	fUTF8	鉩
U+9269.0	fCNS11643	4-4143
U+9269.0	fGB	5-7545
U+9269.0	fR/S	  167.5
U+9269.0	fTotalStrokes	 13
U+9269.0	fCangjie	COF
U+9269.0	fFourCorner	88190
U+9269.0	fCantonese	saai2
U+9269.0	fMandarin	nie1; xi3; nie4
U+926A.0	fUTF8	鉪
U+926A.0	fCNS11643	3-4532
U+926A.0	fGB	E-4163
U+926A.0	fR/S	 167.5
U+926A.0	fTotalStrokes	 13
U+926A.0	fCangjie	CPT
U+926A.0	fFourCorner	84110
U+926A.0	fCantonese	dai6
U+926A.0	fMandarin	di4
U+926B.0	fUTF8	鉫
U+926B.0	fCNS11643	3-4535
U+926B.0	fGB	E-4164
U+926B.0	fR/S	  167.5
U+926B.0	fTotalStrokes	 13
U+926B.0	fCangjie	CKSR
U+926B.0	fFourCorner	86100
U+926B.0	fCantonese	gaa1
U+926B.0	fMandarin	jia1
U+926C.0	fUTF8	鉬
U+926C.0	fCNS11643	2-4841
U+926C.0	fBig5	E0BB
U+926C.0	fGB	1-6E62
U+926C.0	fR/S	 167.5
U+926C.0	fTotalStrokes	 13
U+926C.0	fCangjie	CBU
U+926C.0	fFourCorner	87110
U+926C.0	fHakka	 muk7
U+926C.0	fCantonese	muk6
U+926C.0	fMandarin	mu4
U+926C.0	fEnglish	molybdenum
U+926D.0	fUTF8	鉭
U+926D.0	fCNS11643	2-4840
U+926D.0	fBig5	E0BA
U+926D.0	fGB	1-6E63
U+926D.0	fR/S	 167.5
U+926D.0	fTotalStrokes	 13
U+926D.0	fCangjie	CAM
U+926D.0	fFourCorner	86110
U+926D.0	fCantonese	daan3 (also); taan2
U+926D.0	fMandarin	tan3; dan4
U+926D.0	fEnglish	tantalum
U+926E.0	fUTF8	鉮
U+926E.0	fCNS11643	3-4536
U+926E.0	fGB	3-756E
U+926E.0	fR/S	 167.5
U+926E.0	fTotalStrokes	 13
U+926E.0	fCangjie	CLWL
U+926E.0	fFourCorner	85100
U+926E.0	fCantonese	san4
U+926E.0	fMandarin	shen1; shen2
U+926E.0	fEnglish	arsonium
U+926F.0	fUTF8	鉯
U+926F.0	fCNS11643	2-4845
U+926F.0	fBig5	E0BF
U+926F.0	fGB	E-4165
U+926F.0	fR/S	 167.5
U+926F.0	fTotalStrokes	 13
U+926F.0	fCangjie	CVIO
U+926F.0	fFourCorner	88100
U+926F.0	fCantonese	ji5
U+926F.0	fMandarin	yi3
U+926F.0	fEnglish	illinium
U+9270.0	fUTF8	鉰
U+9270.0	fCNS11643	2-4848
U+9270.0	fBig5	E0C2
U+9270.0	fGB	E-4166
U+9270.0	fR/S	 167.5
U+9270.0	fTotalStrokes	 13
U+9270.0	fCangjie	CSMR
U+9270.0	fFourCorner	87120
U+9270.0	fCantonese	si1
U+9270.0	fMandarin	si1
U+9271.0	fUTF8	鉱
U+9271.0	fGB	E-4167
U+9271.0	fR/S	 167.5
U+9271.0	fTotalStrokes	 13
U+9271.0	fCangjie	XCII
U+9271.0	fCantonese	kong3
U+9271.0	fMandarin	kuang4; gong3
U+9271.0	fEnglish	 mine mineral; ore
U+9272.0	fUTF8	鉲
U+9272.0	fCNS11643	2-484D
U+9272.0	fBig5	E0C7
U+9272.0	fGB	E-4168
U+9272.0	fR/S	 167.5
U+9272.0	fTotalStrokes	 13
U+9272.0	fCangjie	CYMY
U+9272.0	fFourCorner	81131
U+9272.0	fCantonese	kaa1; kaa2
U+9272.0	fMandarin	ka3; ka1; ge2; li4
U+9272.0	fEnglish	cadmium
U+9273.0	fUTF8	鉳
U+9273.0	fCNS11643	3-665A
U+9273.0	fGB	E-4169
U+9273.0	fR/S	 167.5
U+9273.0	fTotalStrokes	 13
U+9273.0	fCangjie	CLMP
U+9273.0	fFourCorner	8111
U+9273.0	fCantonese	bak1
U+9273.0	fMandarin	bei3
U+9274.0	fUTF8	鉴
U+9274.0	fCNS11643	F-4572
U+9274.0	fGB	0-3C78
U+9274.0	fR/S	 167.5
U+9274.0	fTotalStrokes	 13
U+9274.0	fCangjie	LIC
U+9274.0	fHakka	 gam5
U+9274.0	fCantonese	gaam3
U+9274.0	fMandarin	jian4; han4; jian1; xian4
U+9274.0	fEnglish	simplified form of &#x9452;
U+9275.0	fUTF8	鉵
U+9275.0	fCNS11643	4-4775
U+9275.0	fGB	3-7627
U+9275.0	fR/S	 167.6
U+9275.0	fTotalStrokes	 14
U+9275.0	fCangjie	CLMI
U+9275.0	fFourCorner	85130
U+9275.0	fCantonese	tung4
U+9275.0	fMandarin	tong2
U+9276.0	fUTF8	鉶
U+9276.0	fCNS11643	2-4E60
U+9276.0	fBig5	E478
U+9276.0	fGB	3-757C
U+9276.0	fR/S	 167.6
U+9276.0	fTotalStrokes	 14
U+9276.0	fCangjie	CMTN
U+9276.0	fFourCorner	82100
U+9276.0	fHakka	  hin2
U+9276.0	fCantonese	jing4
U+9276.0	fMandarin	xing2
U+9276.0	fEnglish	a kind of receptacle for food in ancient times
U+9277.0	fUTF8	鉷
U+9277.0	fCNS11643	F-4D22
U+9277.0	fGB	3-757E
U+9277.0	fR/S	 167.6
U+9277.0	fTotalStrokes	 14
U+9277.0	fCangjie	CTC
U+9277.0	fFourCorner	8418
U+9277.0	fCantonese	hung4
U+9277.0	fMandarin	hong2
U+9277.0	fEnglish	trigger
U+9278.0	fUTF8	鉸
U+9278.0	fCNS11643	1-6B4F
U+9278.0	fBig5	BBC7
U+9278.0	fGB	1-3D42
U+9278.0	fR/S	 167.6
U+9278.0	fTotalStrokes	 14
U+9278.0	fCangjie	CYCK
U+9278.0	fFourCorner	80148
U+9278.0	fHakka	  gau5; gau3
U+9278.0	fCantonese	gaau2; gaau3 (also)
U+9278.0	fMandarin	jiao3; jia3; jiao1; jiao4
U+9278.0	fEnglish	[1] hinge [2] shears; scissors [3] shear
U+9279.0	fUTF8	鉹
U+9279.0	fCNS11643	2-4E6A
U+9279.0	fBig5	E4A4
U+9279.0	fGB	3-7631
U+9279.0	fR/S	 167.6
U+9279.0	fTotalStrokes	 14
U+9279.0	fCangjie	CNIN
U+9279.0	fFourCorner	87127
U+9279.0	fCantonese	ci2; ji4
U+9279.0	fMandarin	chi3; shi3; yi2
U+927A.0	fUTF8	鉺
U+927A.0	fCNS11643	2-4E62
U+927A.0	fBig5	E47A
U+927A.0	fGB	1-6E6F
U+927A.0	fR/S	 167.6
U+927A.0	fTotalStrokes	 14
U+927A.0	fCangjie	CSJ
U+927A.0	fFourCorner	81140
U+927A.0	fHakka	 ngi5; ngi3
U+927A.0	fCantonese	ji5; ji6
U+927A.0	fMandarin	er3; er4; keng1
U+927A.0	fEnglish	erbium
U+927B.0	fUTF8	鉻
U+927B.0	fCNS11643	1-6B55
U+927B.0	fBig5	BBCC
U+927B.0	fGB	1-3875
U+927B.0	fR/S	 167.6
U+927B.0	fTotalStrokes	 14
U+927B.0	fCangjie	CHER
U+927B.0	fFourCorner	87164
U+927B.0	fHakka	 gok7
U+927B.0	fCantonese	gok3 (also); lok6; lok3
U+927B.0	fMandarin	ge4; luo4; ge2; ge1
U+927B.0	fEnglish	chromium
U+927C.0	fUTF8	鉼
U+927C.0	fCNS11643	1-6B59
U+927C.0	fBig5	BBD0
U+927C.0	fGB	3-7632
U+927C.0	fR/S	 167.8
U+927C.0	fTotalStrokes	 16
U+927C.0	fCangjie	CTT
U+927C.0	fFourCorner	88141
U+927C.0	fCantonese	beng2; ping4; bing2; bang1; bing4
U+927C.0	fMandarin	bing3; bing4; ping2
U+927C.0	fEnglish	plate
U+927D.0	fUTF8	鉽
U+927D.0	fCNS11643	2-4E73
U+927D.0	fBig5	E4AD
U+927D.0	fGB	E-416A
U+927D.0	fR/S	 167.6
U+927D.0	fTotalStrokes	 14
U+927D.0	fCangjie	CIPM
U+927D.0	fFourCorner	83140
U+927D.0	fCantonese	sik1; sing4
U+927D.0	fMandarin	shi4
U+927E.0	fUTF8	鉾
U+927E.0	fCNS11643	2-4E7B
U+927E.0	fBig5	E4B5
U+927E.0	fGB	3-7633
U+927E.0	fR/S	 167.6
U+927E.0	fTotalStrokes	 14
U+927E.0	fCangjie	CIHQ
U+927E.0	fFourCorner	83150
U+927E.0	fCantonese	mau4; maau4
U+927E.0	fMandarin	mou2
U+927E.0	fEnglish	spear
U+927F.0	fUTF8	鉿
U+927F.0	fCNS11643	2-4E6C
U+927F.0	fBig5	E4A6
U+927F.0	fGB	1-6E7E
U+927F.0	fR/S	 167.6
U+927F.0	fTotalStrokes	 14
U+927F.0	fCangjie	COMR
U+927F.0	fFourCorner	88161
U+927F.0	fCantonese	hap6; hap1 (also); gaap3; haa1
U+927F.0	fMandarin	ha1; jia2; jia1; ge2; ge1; he2; ke1
U+927F.0	fEnglish	hafnium
U+9280.0	fUTF8	銀
U+9280.0	fCNS11643	1-6B51
U+9280.0	fBig5	BBC8
U+9280.0	fGB	1-5278
U+9280.0	fR/S	 167.6
U+9280.0	fTotalStrokes	 14
U+9280.0	fCangjie	CAV
U+9280.0	fFourCorner	87132
U+9280.0	fHakka	 ngiun2; niun2; gniun3
U+9280.0	fCantonese	ngan4; an4 (also); ngan2 (also); an2 (also)
U+9280.0	fMandarin	yin2
U+9280.0	fEnglish	[1] silver [2] cash; money; wealth [3] silvery
U+9281.0	fUTF8	銁
U+9281.0	fCNS11643	3-4A34
U+9281.0	fGB	E-416B
U+9281.0	fR/S	  167.6
U+9281.0	fTotalStrokes	 14
U+9281.0	fCangjie	CPA
U+9281.0	fFourCorner	87120
U+9281.0	fHakka	  giun1
U+9281.0	fCantonese	gwan1
U+9281.0	fMandarin	jun1
U+9281.0	fEnglish	a weight of thirty cattics
U+9282.0	fUTF8	銂
U+9282.0	fCNS11643	2-4E70
U+9282.0	fBig5	E4AA
U+9282.0	fGB	E-416C
U+9282.0	fR/S	 167.6
U+9282.0	fTotalStrokes	 14
U+9282.0	fCangjie	CILL
U+9282.0	fFourCorner	82100
U+9282.0	fCantonese	zau1
U+9282.0	fMandarin	zhou1
U+9283.0	fUTF8	銃
U+9283.0	fCNS11643	2-483C
U+9283.0	fBig5	E0B6
U+9283.0	fGB	1-6F25
U+9283.0	fR/S	 167.6
U+9283.0	fTotalStrokes	 14
U+9283.0	fCangjie	CYIU
U+9283.0	fFourCorner	80113
U+9283.0	fHakka	 chung5; cung5; chung3; cung3; cung1
U+9283.0	fCantonese	cung3
U+9283.0	fMandarin	chong4; chu4
U+9283.0	fEnglish	[1] firearms [2] a joint of the metal head to the handle of an ax
U+9284.0	fUTF8	銄
U+9284.0	fCNS11643	3-4A35
U+9284.0	fGB	E-416D
U+9284.0	fR/S	 167.6
U+9284.0	fTotalStrokes	 14
U+9284.0	fCangjie	CHBR
U+9284.0	fFourCorner	87120
U+9284.0	fCantonese	hoeng2
U+9284.0	fMandarin	shang4
U+9285.0	fUTF8	銅
U+9285.0	fCNS11643	1-6B52
U+9285.0	fBig5	BBC9
U+9285.0	fGB	1-4D2D
U+9285.0	fR/S	 167.6
U+9285.0	fTotalStrokes	 14
U+9285.0	fCangjie	CBMR
U+9285.0	fFourCorner	87120
U+9285.0	fHakka	  tung2; tung3
U+9285.0	fCantonese	tung4
U+9285.0	fMandarin	tong2
U+9285.0	fEnglish	copper; brass; bronze
U+9286.0	fUTF8	銆
U+9286.0	fCNS11643	2-4E77
U+9286.0	fBig5	E4B1
U+9286.0	fGB	E-416E
U+9286.0	fR/S	 167.6
U+9286.0	fTotalStrokes	 14
U+9286.0	fCangjie	CMA
U+9286.0	fFourCorner	81160
U+9286.0	fCantonese	mak6
U+9286.0	fMandarin	mo4
U+9287.0	fUTF8	銇
U+9287.0	fCNS11643	2-4E7C
U+9287.0	fBig5	E4B6
U+9287.0	fGB	5-7549
U+9287.0	fR/S	 167.6
U+9287.0	fTotalStrokes	 14
U+9287.0	fCangjie	CQD
U+9287.0	fFourCorner	85190
U+9287.0	fHakka	 loi6; loi5; lui3
U+9287.0	fCantonese	loi6; leoi6
U+9287.0	fMandarin	lei4
U+9288.0	fUTF8	銈
U+9288.0	fCNS11643	2-4E74
U+9288.0	fBig5	E4AE
U+9288.0	fGB	5-754A
U+9288.0	fR/S	 167.6
U+9288.0	fTotalStrokes	 14
U+9288.0	fCangjie	CGG
U+9288.0	fFourCorner	84114
U+9288.0	fCantonese	gwai1; gai1
U+9288.0	fMandarin	ji1
U+9289.0	fUTF8	銉
U+9289.0	fCNS11643	3-4A2F
U+9289.0	fGB	E-416F
U+9289.0	fR/S	 167.6
U+9289.0	fTotalStrokes	 14
U+9289.0	fCangjie	CLQ
U+9289.0	fFourCorner	85100
U+9289.0	fCantonese	jyut6
U+9289.0	fMandarin	yu4
U+928A.0	fUTF8	銊
U+928A.0	fCNS11643	2-4E76
U+928A.0	fBig5	E4B0
U+928A.0	fGB	3-7623
U+928A.0	fR/S	 167.6
U+928A.0	fTotalStrokes	 14
U+928A.0	fCangjie	CIHM
U+928A.0	fFourCorner	83150
U+928A.0	fCantonese	seot1
U+928A.0	fMandarin	xu4
U+928B.0	fUTF8	銋
U+928B.0	fCNS11643	2-4F21
U+928B.0	fBig5	E4B9
U+928B.0	fGB	3-762C
U+928B.0	fR/S	 167.6
U+928B.0	fTotalStrokes	 14
U+928B.0	fCangjie	COHG
U+928B.0	fFourCorner	82114
U+928B.0	fCantonese	jam4 (also); jam5
U+928B.0	fMandarin	ren2; nin3; ren3
U+928C.0	fUTF8	銌
U+928C.0	fCNS11643	2-4E78
U+928C.0	fBig5	E4B2
U+928C.0	fGB	E-4170
U+928C.0	fR/S	 167.6
U+928C.0	fTotalStrokes	 14
U+928C.0	fCangjie	CKLD
U+928C.0	fFourCorner	84147
U+928C.0	fCantonese	zyun6
U+928C.0	fMandarin	zun4
U+928D.0	fUTF8	銍
U+928D.0	fCNS11643	2-4E66
U+928D.0	fBig5	E47E
U+928D.0	fGB	3-7626
U+928D.0	fR/S	 167.6
U+928D.0	fTotalStrokes	 14
U+928D.0	fCangjie	CMIG
U+928D.0	fFourCorner	81114
U+928D.0	fHakka	 zhit7; tsiit7
U+928D.0	fCantonese	zat6; zat1
U+928D.0	fMandarin	zhi4; zhi2
U+928D.0	fEnglish	 sickle
U+928E.0	fUTF8	銎
U+928E.0	fCNS11643	2-4E6F
U+928E.0	fBig5	E4A9
U+928E.0	fGB	0-7646
U+928E.0	fR/S	 167.6
U+928E.0	fTotalStrokes	 14
U+928E.0	fCangjie	MNC
U+928E.0	fFourCorner	17109
U+928E.0	fHakka	  kiung2
U+928E.0	fCantonese	kung4
U+928E.0	fMandarin	qiong1; qiong2; mei4; xiong1
U+928E.0	fEnglish	eye-hole of an axe
U+928F.0	fUTF8	銏
U+928F.0	fCNS11643	3-4530
U+928F.0	fGB	E-4171
U+928F.0	fR/S	 167.5
U+928F.0	fTotalStrokes	 13
U+928F.0	fCangjie	CBBM
U+928F.0	fFourCorner	87140
U+928F.0	fCantonese	saan3
U+928F.0	fMandarin	shan4
U+9290.0	fUTF8	銐
U+9290.0	fCNS11643	3-4A38
U+9290.0	fGB	3-754E
U+9290.0	fR/S	  167.6
U+9290.0	fTotalStrokes	 14
U+9290.0	fCangjie	MNC
U+9290.0	fFourCorner	12100
U+9290.0	fCantonese	cai3
U+9290.0	fMandarin	chi4
U+9291.0	fUTF8	銑
U+9291.0	fCNS11643	1-6B5A
U+9291.0	fBig5	BBD1
U+9291.0	fGB	1-4F33
U+9291.0	fR/S	 167.6
U+9291.0	fTotalStrokes	 14
U+9291.0	fCangjie	CHGU
U+9291.0	fFourCorner	84111
U+9291.0	fHakka	  sien3; se3; sen3
U+9291.0	fCantonese	sin2
U+9291.0	fMandarin	xian3; xi3; xian1
U+9291.0	fEnglish	[1] shiny metal [2] pig iron
U+9292.0	fUTF8	銒
U+9292.0	fCNS11643	3-4A33
U+9292.0	fGB	E-4172
U+9292.0	fR/S	 167.6
U+9292.0	fTotalStrokes	 14
U+9292.0	fCangjie	CMJJ
U+9292.0	fFourCorner	81140
U+9292.0	fHakka	 biang3
U+9292.0	fCantonese	jing4; gin1
U+9292.0	fMandarin	jian1; xing2
U+9292.0	fEnglish	a plate of gold or silver, formerly used as money
U+9293.0	fUTF8	銓
U+9293.0	fCNS11643	1-6B56
U+9293.0	fBig5	BBCD
U+9293.0	fGB	1-6E7D
U+9293.0	fR/S	 167.6
U+9293.0	fTotalStrokes	 14
U+9293.0	fCangjie	COMG
U+9293.0	fFourCorner	88114
U+9293.0	fHakka	  cien2; cion2; zien1; cen2
U+9293.0	fCantonese	cyun4
U+9293.0	fMandarin	quan2
U+9293.0	fEnglish	[1] weigh; measure [2] evaluate qualificaation in selecting officials [3] a Chinese family name
U+9294.0	fUTF8	銔
U+9294.0	fCNS11643	2-4E64
U+9294.0	fBig5	E47C
U+9294.0	fGB	3-7624
U+9294.0	fR/S	 167.6
U+9294.0	fTotalStrokes	 14
U+9294.0	fCangjie	CMFJ
U+9294.0	fFourCorner	81149
U+9294.0	fCantonese	pei1
U+9294.0	fMandarin	pi1; pei2
U+9295.0	fUTF8	銕
U+9295.0	fCNS11643	2-4E71
U+9295.0	fBig5	E4AB
U+9295.0	fGB	5-754D
U+9295.0	fR/S	 167.6
U+9295.0	fTotalStrokes	 14
U+9295.0	fCangjie	CKN
U+9295.0	fFourCorner	85132
U+9295.0	fHakka	 tiet7
U+9295.0	fCantonese	tit3
U+9295.0	fMandarin	yi2; tie3; ti2
U+9295.0	fEnglish	 iron strong; solid; firm
U+9296.0	fUTF8	銖
U+9296.0	fCNS11643	1-6B54
U+9296.0	fBig5	BBCB
U+9296.0	fGB	1-6E79
U+9296.0	fR/S	 167.6
U+9296.0	fTotalStrokes	 14
U+9296.0	fCangjie	CHJD
U+9296.0	fFourCorner	85190
U+9296.0	fHakka	 su2; zu1; shu2; zhu1
U+9296.0	fCantonese	zyu1; syu4
U+9296.0	fMandarin	zhu1; shu1
U+9296.0	fEnglish	[1] an ancient unit of weight (1/48 or 1/24 of a tael) [2] the ancient coinage of Han [3] blunt; dull; ontuse [4] a Chinese family name
U+9297.0	fUTF8	銗
U+9297.0	fCNS11643	2-4E6B
U+9297.0	fBig5	E4A5
U+9297.0	fGB	3-762D
U+9297.0	fR/S	 167.6
U+9297.0	fTotalStrokes	 14
U+9297.0	fCangjie	CHMR
U+9297.0	fFourCorner	82161
U+9297.0	fCantonese	hong6; hau4 (also); hau6 (also); hong2
U+9297.0	fMandarin	hou2; xiang4; hou4
U+9298.0	fUTF8	銘
U+9298.0	fCNS11643	1-6B53
U+9298.0	fBig5	BBCA
U+9298.0	fGB	1-437A
U+9298.0	fR/S	 167.6
U+9298.0	fTotalStrokes	 14
U+9298.0	fCangjie	CNIR
U+9298.0	fFourCorner	87160
U+9298.0	fHakka	 men2; min2;  miang2
U+9298.0	fCantonese	ming4; ming5 (also); ming2 (also)
U+9298.0	fMandarin	ming2
U+9298.0	fEnglish	[1] imprint; inscribe; engrave [2] inscriptions
U+9299.0	fUTF8	銙
U+9299.0	fCNS11643	2-4E79
U+9299.0	fBig5	E4B3
U+9299.0	fGB	3-7625
U+9299.0	fR/S	 167.6
U+9299.0	fTotalStrokes	 14
U+9299.0	fCangjie	CKMS
U+9299.0	fFourCorner	84127
U+9299.0	fCantonese	kwaa1; kwaa3
U+9299.0	fMandarin	kua3
U+929A.0	fUTF8	銚
U+929A.0	fCNS11643	2-4E68
U+929A.0	fBig5	E4A2
U+929A.0	fGB	1-6F22
U+929A.0	fR/S	 167.6
U+929A.0	fTotalStrokes	 14
U+929A.0	fCangjie	CLMO
U+929A.0	fFourCorner	82113
U+929A.0	fHakka	 tiau1; tiau2; rau2; jeu2; jau2; tiau5
U+929A.0	fCantonese	diu6; jiu4; tiu4; siu6
U+929A.0	fMandarin	yao2; qiao1; tiao2; yao4; tiao1; tiao4; zhao4
U+929A.0	fEnglish	[1] a kind of farm tool [2] a Chinese family name
U+929A.1	fUTF8	銚
U+929A.1	fCNS11643	2-4E68
U+929A.1	fBig5	E4A2
U+929A.1	fGB	1-6F22
U+929A.1	fR/S	 167.6
U+929A.1	fTotalStrokes	 14
U+929A.1	fCangjie	CLMO
U+929A.1	fFourCorner	82113
U+929A.1	fMandarin	diao4
U+929A.1	fEnglish	[1] small pot with a handle [2] brass cymbal
U+929B.0	fUTF8	銛
U+929B.0	fCNS11643	2-4E61
U+929B.0	fBig5	E479
U+929B.0	fGB	3-7629
U+929B.0	fR/S	 167.6
U+929B.0	fTotalStrokes	 14
U+929B.0	fCangjie	CHJR
U+929B.0	fFourCorner	82164
U+929B.0	fHakka	  siam2; tiam3; ciam3
U+929B.0	fCantonese	cim1
U+929B.0	fMandarin	xian1; gua1; tian3; xian4
U+929B.0	fEnglish	sharp; keen-edged
U+929C.0	fUTF8	銜
U+929C.0	fCNS11643	1-6B57
U+929C.0	fBig5	BBCE
U+929C.0	fGB	1-4F4E
U+929C.0	fR/S	 167.6
U+929C.0	fTotalStrokes	 14
U+929C.0	fCangjie	HOCMN
U+929C.0	fFourCorner	21109
U+929C.0	fHakka	  ham2; ham3
U+929C.0	fCantonese	haam4; ham4 (also)
U+929C.0	fMandarin	xian2
U+929C.0	fEnglish	[1] bit (in a horse's mouth) [2] the title (of an official) [3] hold in mouth [4] harbour; cherish [5] follow (orders)
U+929D.0	fUTF8	銝
U+929D.0	fCNS11643	2-4E7E
U+929D.0	fBig5	E4B8
U+929D.0	fGB	E-4173
U+929D.0	fR/S	 167.6
U+929D.0	fTotalStrokes	 14
U+929D.0	fCangjie	COD
U+929D.0	fFourCorner	84190
U+929D.0	fHakka	 hiu1
U+929D.0	fCantonese	jau1
U+929D.0	fMandarin	xiu1
U+929E.0	fUTF8	銞
U+929E.0	fCNS11643	3-4A39
U+929E.0	fGB	E-4174
U+929E.0	fR/S	 167.6
U+929E.0	fTotalStrokes	 14
U+929E.0	fCangjie	PAC
U+929E.0	fFourCorner	27100
U+929E.0	fCantonese	gwan1
U+929E.0	fMandarin	jun1
U+929F.0	fUTF8	銟
U+929F.0	fCNS11643	4-4777
U+929F.0	fGB	E-4175
U+929F.0	fR/S	 167.6
U+929F.0	fTotalStrokes	 14
U+929F.0	fCangjie	CULB
U+929F.0	fFourCorner	82120
U+929F.0	fCantonese	caa1
U+929F.0	fMandarin	cha1
U+92A0.0	fUTF8	銠
U+92A0.0	fCNS11643	2-4E63
U+92A0.0	fBig5	E47B
U+92A0.0	fGB	1-6E6E
U+92A0.0	fR/S	 167.6
U+92A0.0	fTotalStrokes	 14
U+92A0.0	fCangjie	CJKP
U+92A0.0	fFourCorner	84111
U+92A0.0	fHakka	 lau3
U+92A0.0	fCantonese	lou5
U+92A0.0	fMandarin	lao3
U+92A0.0	fEnglish	rhodium
U+92A1.0	fUTF8	銡
U+92A1.0	fCNS11643	2-4E75
U+92A1.0	fBig5	E4AF
U+92A1.0	fGB	3-757D
U+92A1.0	fR/S	 167.6
U+92A1.0	fTotalStrokes	 14
U+92A1.0	fCangjie	CGR
U+92A1.0	fFourCorner	84161
U+92A1.0	fCantonese	gat1
U+92A1.0	fMandarin	ji2
U+92A2.0	fUTF8	銢
U+92A2.0	fCNS11643	2-4E72
U+92A2.0	fBig5	E4AC
U+92A2.0	fGB	E-4176
U+92A2.0	fR/S	 167.6
U+92A2.0	fTotalStrokes	 14
U+92A2.0	fCangjie	CINE
U+92A2.0	fFourCorner	83199
U+92A2.0	fCantonese	pik1
U+92A2.0	fMandarin	yong3; pi3
U+92A3.0	fUTF8	銣
U+92A3.0	fCNS11643	2-4E6D
U+92A3.0	fBig5	E4A7
U+92A3.0	fGB	1-6F28
U+92A3.0	fR/S	 167.6
U+92A3.0	fTotalStrokes	 14
U+92A3.0	fCangjie	CVR
U+92A3.0	fFourCorner	86100
U+92A3.0	fCantonese	jyu4; jyu6 (also)
U+92A3.0	fMandarin	ru2; ru3; ru4
U+92A3.0	fEnglish	rubidium
U+92A4.0	fUTF8	銤
U+92A4.0	fCNS11643	2-4E5F
U+92A4.0	fBig5	E477
U+92A4.0	fGB	5-7550
U+92A4.0	fR/S	 167.6
U+92A4.0	fTotalStrokes	 14
U+92A4.0	fCangjie	CFD
U+92A4.0	fFourCorner	89194
U+92A4.0	fCantonese	mai5
U+92A4.0	fMandarin	mi3
U+92A5.0	fUTF8	銥
U+92A5.0	fCNS11643	2-4E5E
U+92A5.0	fBig5	E476
U+92A5.0	fGB	1-523F
U+92A5.0	fR/S	 167.6
U+92A5.0	fTotalStrokes	 14
U+92A5.0	fCangjie	CYHV
U+92A5.0	fFourCorner	80132
U+92A5.0	fHakka	 ji1
U+92A5.0	fCantonese	ji1; ji2
U+92A5.0	fMandarin	yi1; yi3
U+92A5.0	fEnglish	iridium
U+92A6.0	fUTF8	銦
U+92A6.0	fCNS11643	2-4E67
U+92A6.0	fBig5	E4A1
U+92A6.0	fGB	1-6E77
U+92A6.0	fR/S	 167.6
U+92A6.0	fTotalStrokes	 14
U+92A6.0	fCangjie	CWK
U+92A6.0	fFourCorner	86100
U+92A6.0	fCantonese	jan1
U+92A6.0	fMandarin	yin1
U+92A6.0	fEnglish	indium
U+92A7.0	fUTF8	銧
U+92A7.0	fCNS11643	2-4E7A
U+92A7.0	fBig5	E4B4
U+92A7.0	fGB	E-4177
U+92A7.0	fR/S	 167.6
U+92A7.0	fTotalStrokes	 14
U+92A7.0	fCangjie	CFMU
U+92A7.0	fFourCorner	89111
U+92A7.0	fHakka	 gong1
U+92A7.0	fCantonese	gwong1; gong1 (also)
U+92A7.0	fMandarin	guang1
U+92A7.0	fEnglish	point of a sword
U+92A8.0	fUTF8	銨
U+92A8.0	fCNS11643	1-6B58
U+92A8.0	fBig5	BBCF
U+92A8.0	fGB	1-6F27
U+92A8.0	fR/S	 167.6
U+92A8.0	fTotalStrokes	 14
U+92A8.0	fCangjie	CJV
U+92A8.0	fFourCorner	83144
U+92A8.0	fHakka	 on1
U+92A8.0	fCantonese	on1; ngon1 (also); on2 (also); ngon2 (also)
U+92A8.0	fMandarin	an1; an3
U+92A8.0	fEnglish	ammonium
U+92A9.0	fUTF8	銩
U+92A9.0	fCNS11643	2-4E7D
U+92A9.0	fBig5	E4B7
U+92A9.0	fGB	1-6E7B
U+92A9.0	fR/S	 167.6
U+92A9.0	fTotalStrokes	 14
U+92A9.0	fCangjie	CHGI
U+92A9.0	fFourCorner	81131
U+92A9.0	fCantonese	diu1
U+92A9.0	fMandarin	diu1
U+92A9.0	fEnglish	thulium
U+92AA.0	fUTF8	銪
U+92AA.0	fCNS11643	2-4E65
U+92AA.0	fBig5	E47D
U+92AA.0	fGB	1-6E70
U+92AA.0	fR/S	 167.6
U+92AA.0	fTotalStrokes	 14
U+92AA.0	fCangjie	CKB
U+92AA.0	fFourCorner	84127
U+92AA.0	fCantonese	jau5
U+92AA.0	fMandarin	you3
U+92AA.0	fEnglish	europium
U+92AB.0	fUTF8	銫
U+92AB.0	fCNS11643	2-4E69
U+92AB.0	fBig5	E4A3
U+92AB.0	fGB	1-6F24
U+92AB.0	fR/S	 167.6
U+92AB.0	fTotalStrokes	 14
U+92AB.0	fCangjie	CNAU
U+92AB.0	fFourCorner	87117
U+92AB.0	fHakka	 set7
U+92AB.0	fCantonese	sik1
U+92AB.0	fMandarin	se4
U+92AB.0	fEnglish	cesium; caesium
U+92AC.0	fUTF8	銬
U+92AC.0	fCNS11643	1-6B50
U+92AC.0	fBig5	BE52
U+92AC.0	fGB	1-6E6D
U+92AC.0	fR/S	 167.6
U+92AC.0	fTotalStrokes	 14
U+92AC.0	fCangjie	CJKS
U+92AC.0	fFourCorner	84127
U+92AC.0	fHakka	 kau5; kau3; kwa3
U+92AC.0	fCantonese	kaau3; kau3 (also)
U+92AC.0	fMandarin	kao4; kao3
U+92AC.0	fEnglish	manacles; handcuffs
U+92AD.0	fUTF8	銭
U+92AD.0	fCNS11643	3-4A32
U+92AD.0	fGB	E-4178
U+92AD.0	fR/S	 167.6
U+92AD.0	fTotalStrokes	 14
U+92AD.0	fCangjie	CIQ
U+92AD.0	fFourCorner	83150
U+92AD.0	fCantonese	cin4
U+92AD.0	fMandarin	qian2; jian3
U+92AD.0	fEnglish	money; currency; coins
U+92AE.0	fUTF8	銮
U+92AE.0	fCNS11643	3-4A37
U+92AE.0	fGB	0-7647
U+92AE.0	fR/S	 167.6
U+92AE.0	fTotalStrokes	 14
U+92AE.0	fCangjie	YCC
U+92AE.0	fFourCorner	00100
U+92AE.0	fHakka	 lan2
U+92AE.0	fCantonese	lyun4
U+92AE.0	fMandarin	luan2; luo2
U+92AE.0	fEnglish	simplified form of &#x947E;
U+92AF.0	fUTF8	銯
U+92AF.0	fCNS11643	F-4D27
U+92AF.0	fGB	E-4179
U+92AF.0	fR/S	 167.6
U+92AF.0	fTotalStrokes	 14
U+92AF.0	fCangjie	CVIF
U+92AF.0	fFourCorner	8219
U+92AF.0	fCantonese	mik6
U+92B0.0	fUTF8	銰
U+92B0.0	fCNS11643	3-672A
U+92B0.0	fGB	E-417A
U+92B0.0	fR/S	 167.6
U+92B0.0	fTotalStrokes	 14
U+92B0.0	fCangjie	CTK
U+92B0.0	fCantonese	ngaai6; aai6 (also)
U+92B0.0	fMandarin	ai1
U+92B1.0	fUTF8	銱
U+92B1.0	fCNS11643	3-4A31
U+92B1.0	fGB	1-6E76
U+92B1.0	fR/S	  167.6
U+92B1.0	fTotalStrokes	 14
U+92B1.0	fCangjie	CRLB
U+92B1.0	fFourCorner	86120
U+92B1.0	fCantonese	diu3
U+92B1.0	fMandarin	diao4
U+92B2.0	fUTF8	銲
U+92B2.0	fCNS11643	1-6F63
U+92B2.0	fBig5	BE5A
U+92B2.0	fGB	E-417B
U+92B2.0	fR/S	 167.7
U+92B2.0	fTotalStrokes	 15
U+92B2.0	fCangjie	CAMJ
U+92B2.0	fFourCorner	86141
U+92B2.0	fHakka	 hon5; hon6; hon3
U+92B2.0	fCantonese	hon6
U+92B2.0	fMandarin	han4
U+92B2.0	fEnglish	 solder; weld
U+92B3.0	fUTF8	銳
U+92B3.0	fCNS11643	1-6F5E
U+92B3.0	fBig5	BE55
U+92B3.0	fGB	E-417C
U+92B3.0	fR/S	 167.7
U+92B3.0	fTotalStrokes	 15
U+92B3.0	fCangjie	CCRU
U+92B3.0	fFourCorner	88116
U+92B3.0	fHakka	 jui3;  jui5; rui5
U+92B3.0	fCantonese	jeoi6
U+92B3.0	fMandarin	rui4; dui4; yue4
U+92B3.0	fEnglish	[1] sharp; keen; acute; pointed [2] quick-witted; intelligent; clever [3] energetic; vigorous
U+92B4.0	fUTF8	銴
U+92B4.0	fCNS11643	2-554D
U+92B4.0	fBig5	E8A4
U+92B4.0	fGB	3-7551
U+92B4.0	fR/S	 167.7
U+92B4.0	fTotalStrokes	 15
U+92B4.0	fCangjie	QLC
U+92B4.0	fFourCorner	52109
U+92B4.0	fCantonese	sai6; zai6; jeoi6
U+92B4.0	fMandarin	shi4; bi4; rui4; zhi4
U+92B5.0	fUTF8	銵
U+92B5.0	fCNS11643	2-5549
U+92B5.0	fBig5	E8A1
U+92B5.0	fGB	5-7559
U+92B5.0	fR/S	 167.7
U+92B5.0	fTotalStrokes	 15
U+92B5.0	fCangjie	CHXH
U+92B5.0	fFourCorner	87140
U+92B5.0	fCantonese	hang1
U+92B5.0	fMandarin	keng1
U+92B6.0	fUTF8	銶
U+92B6.0	fCNS11643	2-5531
U+92B6.0	fBig5	E867
U+92B6.0	fGB	3-7637
U+92B6.0	fR/S	 167.7
U+92B6.0	fTotalStrokes	 15
U+92B6.0	fCangjie	CIJE
U+92B6.0	fFourCorner	83132
U+92B6.0	fHakka	  kiu2
U+92B6.0	fCantonese	kau4
U+92B6.0	fMandarin	qiu2
U+92B6.0	fEnglish	chisel
U+92B7.0	fUTF8	銷
U+92B7.0	fCNS11643	1-6F5A
U+92B7.0	fBig5	BE50
U+92B7.0	fGB	1-4F7A
U+92B7.0	fR/S	 167.7
U+92B7.0	fTotalStrokes	 15
U+92B7.0	fCangjie	CFB
U+92B7.0	fFourCorner	89127
U+92B7.0	fHakka	  siau1; seu1
U+92B7.0	fCantonese	siu1
U+92B7.0	fMandarin	xiao1
U+92B7.0	fEnglish	[1] melt; fuse [2] be marketed; sell; be circulated [3] vanish; cancel; dispel [4] pig iron; crude iron
U+92B8.0	fUTF8	銸
U+92B8.0	fCNS11643	3-4F42
U+92B8.0	fGB	5-7554
U+92B8.0	fR/S	  167.7
U+92B8.0	fTotalStrokes	 15
U+92B8.0	fCangjie	CSJU
U+92B8.0	fFourCorner	82110
U+92B8.0	fCantonese	zip3
U+92B8.0	fMandarin	zhe2
U+92B9.0	fUTF8	銹
U+92B9.0	fCNS11643	3-4F50
U+92B9.0	fGB	1-5062
U+92B9.0	fR/S	  167.7
U+92B9.0	fTotalStrokes	 15
U+92B9.0	fCangjie	CHDS
U+92B9.0	fFourCorner	82127
U+92B9.0	fHakka	  siu5; sieu5
U+92B9.0	fCantonese	sau3
U+92B9.0	fMandarin	xiu4
U+92B9.0	fEnglish	rust
U+92BA.0	fUTF8	銺
U+92BA.0	fCNS11643	4-4E45
U+92BA.0	fGB	E-417D
U+92BA.0	fR/S	 167.7
U+92BA.0	fTotalStrokes	 15
U+92BA.0	fCangjie	VGC
U+92BA.0	fFourCorner	24100
U+92BA.0	fCantonese	zong6
U+92BA.0	fMandarin	zang4
U+92BB.0	fUTF8	銻
U+92BB.0	fCNS11643	1-6F59
U+92BB.0	fBig5	BE4F
U+92BB.0	fGB	1-4C60
U+92BB.0	fR/S	 167.7
U+92BB.0	fTotalStrokes	 15
U+92BB.0	fCangjie	CCNH
U+92BB.0	fFourCorner	88127
U+92BB.0	fHakka	 tai1
U+92BB.0	fCantonese	tai1
U+92BB.0	fMandarin	ti1; ti4; ti2
U+92BB.0	fEnglish	antimony
U+92BC.0	fUTF8	銼
U+92BC.0	fCNS11643	1-6F5F
U+92BC.0	fBig5	BE56
U+92BC.0	fGB	1-6F31
U+92BC.0	fR/S	 167.7
U+92BC.0	fTotalStrokes	 15
U+92BC.0	fCangjie	COOG
U+92BC.0	fFourCorner	88114
U+92BC.0	fHakka	  co5
U+92BC.0	fCantonese	co3; co2 (also)
U+92BC.0	fMandarin	cuo4; cuo2; zu2
U+92BC.0	fEnglish	[1] a kind of widemouthed cauldron used in ancient China [2] carpenter's file (steel tool) [3] file smooth; make smooth with a file; file
U+92BD.0	fUTF8	銽
U+92BD.0	fCNS11643	4-4E47
U+92BD.0	fGB	E-417E
U+92BD.0	fR/S	 167.7
U+92BD.0	fTotalStrokes	 15
U+92BD.0	fCangjie	CHPR
U+92BD.0	fFourCorner	82160
U+92BD.0	fCantonese	cim1
U+92BD.0	fMandarin	gua1
U+92BE.0	fUTF8	銾
U+92BE.0	fCNS11643	3-4F43
U+92BE.0	fGB	5-7553
U+92BE.0	fR/S	 167.7
U+92BE.0	fTotalStrokes	 15
U+92BE.0	fCangjie	CME
U+92BE.0	fFourCorner	81130
U+92BE.0	fCantonese	hung3
U+92BE.0	fMandarin	gong3
U+92BE.0	fEnglish	element mercury
U+92BF.0	fUTF8	銿
U+92BF.0	fCNS11643	3-4F41
U+92BF.0	fGB	E-4221
U+92BF.0	fR/S	  167.7
U+92BF.0	fTotalStrokes	 15
U+92BF.0	fCangjie	CNIB
U+92BF.0	fFourCorner	87120
U+92BF.0	fCantonese	zung1
U+92BF.0	fMandarin	zhong1; yong1
U+92C0.0	fUTF8	鋀
U+92C0.0	fCNS11643	2-552F
U+92C0.0	fBig5	E865
U+92C0.0	fGB	3-7635
U+92C0.0	fR/S	 167.7
U+92C0.0	fTotalStrokes	 15
U+92C0.0	fCangjie	CMRT
U+92C0.0	fFourCorner	81118
U+92C0.0	fCantonese	dau6; tau1
U+92C0.0	fMandarin	dou4; tou1
U+92C1.0	fUTF8	鋁
U+92C1.0	fCNS11643	1-6F5D
U+92C1.0	fBig5	BE54
U+92C1.0	fGB	1-4241
U+92C1.0	fR/S	 167.7
U+92C1.0	fTotalStrokes	 15
U+92C1.0	fCangjie	CRHR
U+92C1.0	fFourCorner	86160
U+92C1.0	fHakka	 li1
U+92C1.0	fCantonese	leoi5
U+92C1.0	fMandarin	l3; luu3; lu3; l4
U+92C1.0	fEnglish	aluminum; aluminium
U+92C2.0	fUTF8	鋂
U+92C2.0	fCNS11643	2-553B
U+92C2.0	fBig5	E871
U+92C2.0	fGB	3-763A
U+92C2.0	fR/S	 167.7
U+92C2.0	fTotalStrokes	 15
U+92C2.0	fCangjie	COWY
U+92C2.0	fFourCorner	88157
U+92C2.0	fHakka	 moi2; mi2; mui2; moe2
U+92C2.0	fCantonese	mui4
U+92C2.0	fMandarin	mei2
U+92C2.0	fEnglish	bit cup
U+92C3.0	fUTF8	鋃
U+92C3.0	fCNS11643	2-552D
U+92C3.0	fBig5	E863
U+92C3.0	fGB	1-6F36
U+92C3.0	fR/S	 167.7
U+92C3.0	fTotalStrokes	 15
U+92C3.0	fCangjie	CIAV
U+92C3.0	fFourCorner	83132
U+92C3.0	fHakka	 long2
U+92C3.0	fCantonese	long4
U+92C3.0	fMandarin	lang2
U+92C3.0	fEnglish	[1] chains for prisoners [2] the tolling of a bell
U+92C4.0	fUTF8	鋄
U+92C4.0	fCNS11643	2-552E
U+92C4.0	fBig5	E864
U+92C4.0	fGB	E-4222
U+92C4.0	fR/S	 167.7
U+92C4.0	fTotalStrokes	 15
U+92C4.0	fCangjie	CMFE
U+92C4.0	fFourCorner	81147
U+92C4.0	fHakka	 man5; man6
U+92C4.0	fCantonese	faan6; maan3
U+92C4.0	fMandarin	wan4; wan3; jian3; liao3
U+92C4.0	fEnglish	 cover with silver or gold
U+92C5.0	fUTF8	鋅
U+92C5.0	fCNS11643	1-6F58
U+92C5.0	fBig5	BE4E
U+92C5.0	fGB	1-503F
U+92C5.0	fR/S	 167.7
U+92C5.0	fTotalStrokes	 15
U+92C5.0	fCangjie	CYTJ
U+92C5.0	fFourCorner	80141
U+92C5.0	fHakka	 sin1; hin1
U+92C5.0	fCantonese	san1
U+92C5.0	fMandarin	xin1; zi3
U+92C5.0	fEnglish	zincum; zinc
U+92C6.0	fUTF8	鋆
U+92C6.0	fCNS11643	2-554C
U+92C6.0	fBig5	E8A3
U+92C6.0	fGB	3-7550
U+92C6.0	fR/S	 167.7
U+92C6.0	fTotalStrokes	 15
U+92C6.0	fCangjie	GMC
U+92C6.0	fFourCorner	47109
U+92C6.0	fHakka	 giun1;  jun2; run2
U+92C6.0	fCantonese	wan4; gwan1
U+92C6.0	fMandarin	yun2
U+92C6.0	fEnglish	 gold character used in personal name
U+92C7.0	fUTF8	鋇
U+92C7.0	fCNS11643	1-6F61
U+92C7.0	fBig5	BE58
U+92C7.0	fGB	1-3135
U+92C7.0	fR/S	 167.7
U+92C7.0	fTotalStrokes	 15
U+92C7.0	fCangjie	CBUC
U+92C7.0	fFourCorner	86180
U+92C7.0	fHakka	 bui5
U+92C7.0	fCantonese	bui3
U+92C7.0	fMandarin	bei4
U+92C7.0	fEnglish	barium
U+92C8.0	fUTF8	鋈
U+92C8.0	fCNS11643	2-553E
U+92C8.0	fBig5	E874
U+92C8.0	fGB	0-7648
U+92C8.0	fR/S	 167.7
U+92C8.0	fTotalStrokes	 15
U+92C8.0	fCangjie	EKC
U+92C8.0	fFourCorner	32109
U+92C8.0	fHakka	 wok7; vok7
U+92C8.0	fCantonese	juk1
U+92C8.0	fMandarin	wu4; wo4
U+92C8.0	fEnglish	silver plating
U+92C9.0	fUTF8	鋉
U+92C9.0	fCNS11643	2-5543
U+92C9.0	fBig5	E879
U+92C9.0	fGB	E-4223
U+92C9.0	fR/S	 167.7
U+92C9.0	fTotalStrokes	 15
U+92C9.0	fCangjie	CDL
U+92C9.0	fFourCorner	85196
U+92C9.0	fCantonese	suk1; cuk1
U+92C9.0	fMandarin	su4
U+92CA.0	fUTF8	鋊
U+92CA.0	fCNS11643	2-553D
U+92CA.0	fBig5	E873
U+92CA.0	fGB	3-763C
U+92CA.0	fR/S	 167.7
U+92CA.0	fTotalStrokes	 15
U+92CA.0	fCangjie	CCOR
U+92CA.0	fFourCorner	88168
U+92CA.0	fHakka	 juk8; ruk8
U+92CA.0	fCantonese	juk6; guk1
U+92CA.0	fMandarin	yu4; gu3; rong2
U+92CA.0	fEnglish	 a poker brass filings file
U+92CB.0	fUTF8	鋋
U+92CB.0	fCNS11643	2-5B3C
U+92CB.0	fBig5	EBEE
U+92CB.0	fGB	3-762B
U+92CB.0	fR/S	 167.7
U+92CB.0	fTotalStrokes	 15
U+92CB.0	fCangjie	CNKM
U+92CB.0	fFourCorner	82141
U+92CB.0	fHakka	 tin3
U+92CB.0	fCantonese	jin4 (also); sin4; ding3; ting5
U+92CB.0	fMandarin	chan2; yan2
U+92CB.0	fEnglish	[1] ingots [2] barb of an arrow [3] hollow
U+92CC.0	fUTF8	鋌
U+92CC.0	fCNS11643	2-5539
U+92CC.0	fBig5	E86F
U+92CC.0	fGB	1-6E7A
U+92CC.0	fR/S	 167.7
U+92CC.0	fTotalStrokes	 15
U+92CC.0	fCangjie	CNKG
U+92CC.0	fFourCorner	82141
U+92CC.0	fHakka	 tin3
U+92CC.0	fCantonese	ting5; ding3; ding6 (also); ting2; jin4
U+92CC.0	fMandarin	ting3; ding4
U+92CC.0	fEnglish	 rush; hurry
U+92CD.0	fUTF8	鋍
U+92CD.0	fCNS11643	2-5541
U+92CD.0	fBig5	E877
U+92CD.0	fGB	3-7634
U+92CD.0	fR/S	 167.7
U+92CD.0	fTotalStrokes	 15
U+92CD.0	fCangjie	CJBD
U+92CD.0	fFourCorner	84147
U+92CD.0	fCantonese	but6
U+92CD.0	fMandarin	bo2
U+92CE.0	fUTF8	鋎
U+92CE.0	fCNS11643	2-553F
U+92CE.0	fBig5	E875
U+92CE.0	fGB	E-4224
U+92CE.0	fR/S	 167.7
U+92CE.0	fTotalStrokes	 15
U+92CE.0	fCangjie	CJMU
U+92CE.0	fFourCorner	83111
U+92CE.0	fCantonese	waan5; wun5
U+92CE.0	fMandarin	han4
U+92CF.0	fUTF8	鋏
U+92CF.0	fCNS11643	2-5532
U+92CF.0	fBig5	E868
U+92CF.0	fGB	1-6E72
U+92CF.0	fR/S	 167.7
U+92CF.0	fTotalStrokes	 15
U+92CF.0	fCangjie	CKOO
U+92CF.0	fFourCorner	84138
U+92CF.0	fHakka	  giap7; gap7
U+92CF.0	fCantonese	gaap3
U+92CF.0	fMandarin	jia2
U+92CF.0	fEnglish	[1] tongs; pincers [2] sword [3] hilt
U+92D0.0	fUTF8	鋐
U+92D0.0	fCNS11643	2-552C
U+92D0.0	fBig5	E862
U+92D0.0	fGB	3-763E
U+92D0.0	fR/S	 167.7
U+92D0.0	fTotalStrokes	 15
U+92D0.0	fCangjie	CJKI
U+92D0.0	fFourCorner	83132
U+92D0.0	fCantonese	wang4
U+92D0.0	fMandarin	hong2
U+92D0.0	fEnglish	a state in today's Hubei province
U+92D1.0	fUTF8	鋑
U+92D1.0	fCNS11643	2-5547
U+92D1.0	fBig5	E87D
U+92D1.0	fGB	E-4225
U+92D1.0	fR/S	 167.7
U+92D1.0	fTotalStrokes	 15
U+92D1.0	fCangjie	CICE
U+92D1.0	fFourCorner	83147
U+92D1.0	fHakka	 son1; zun1
U+92D1.0	fCantonese	cyun1; zeon3
U+92D1.0	fMandarin	cuan1; jian1; juan1
U+92D1.0	fEnglish	 engrave or carve (as a block for printing)
U+92D2.0	fUTF8	鋒
U+92D2.0	fCNS11643	1-6F60
U+92D2.0	fBig5	BE57
U+92D2.0	fGB	1-3766
U+92D2.0	fR/S	 167.7
U+92D2.0	fTotalStrokes	 15
U+92D2.0	fCangjie	CHEJ
U+92D2.0	fFourCorner	87154
U+92D2.0	fHakka	  fung1
U+92D2.0	fCantonese	fung1
U+92D2.0	fMandarin	feng1
U+92D2.0	fEnglish	[1] sharp point (as the tip of a spear, lance, pencil, etc.) [2] vanguard; van
U+92D3.0	fUTF8	鋓
U+92D3.0	fCNS11643	2-5548
U+92D3.0	fBig5	E87E
U+92D3.0	fGB	E-4226
U+92D3.0	fR/S	 167.7
U+92D3.0	fTotalStrokes	 15
U+92D3.0	fCangjie	CHDN
U+92D3.0	fFourCorner	82100
U+92D3.0	fCantonese	cim1
U+92D3.0	fMandarin	chan1
U+92D4.0	fUTF8	鋔
U+92D4.0	fCNS11643	3-4F48
U+92D4.0	fGB	E-4227
U+92D4.0	fR/S	 167.7
U+92D4.0	fTotalStrokes	 15
U+92D4.0	fCangjie	CNAU
U+92D4.0	fFourCorner	87110
U+92D4.0	fCantonese	waan5
U+92D4.0	fMandarin	wan3
U+92D5.0	fUTF8	鋕
U+92D5.0	fCNS11643	2-5542
U+92D5.0	fBig5	E878
U+92D5.0	fGB	5-7552
U+92D5.0	fR/S	 167.7
U+92D5.0	fTotalStrokes	 15
U+92D5.0	fCangjie	CGP
U+92D5.0	fFourCorner	84131
U+92D5.0	fHakka	 zhi5; zih5
U+92D5.0	fCantonese	zi3
U+92D5.0	fMandarin	zhi4
U+92D5.0	fEnglish	engrave
U+92D6.0	fUTF8	鋖
U+92D6.0	fCNS11643	3-4F49
U+92D6.0	fGB	5-755A
U+92D6.0	fR/S	  167.7
U+92D6.0	fTotalStrokes	 15
U+92D6.0	fCangjie	CBV
U+92D6.0	fFourCorner	82140
U+92D6.0	fCantonese	si1
U+92D6.0	fMandarin	si1
U+92D7.0	fUTF8	鋗
U+92D7.0	fCNS11643	2-5537
U+92D7.0	fBig5	E86D
U+92D7.0	fGB	3-7639
U+92D7.0	fR/S	 167.7
U+92D7.0	fTotalStrokes	 15
U+92D7.0	fCangjie	CRB
U+92D7.0	fFourCorner	86127
U+92D7.0	fCantonese	hyun1; kyun1
U+92D7.0	fMandarin	xuan1; xuan4; juan1
U+92D7.0	fEnglish	a small basin; rings on a cart of carriage
U+92D8.0	fUTF8	鋘
U+92D8.0	fCNS11643	2-5535
U+92D8.0	fBig5	E86B
U+92D8.0	fGB	5-7558
U+92D8.0	fR/S	 167.7
U+92D8.0	fTotalStrokes	 15
U+92D8.0	fCangjie	CRVK
U+92D8.0	fFourCorner	86134
U+92D8.0	fHakka	  fa2;  fu2
U+92D8.0	fCantonese	waa4; ng4
U+92D8.0	fMandarin	wu2; hua2; hu2
U+92D8.0	fEnglish	sword
U+92D9.0	fUTF8	鋙
U+92D9.0	fCNS11643	2-5530
U+92D9.0	fBig5	E866
U+92D9.0	fGB	5-7556
U+92D9.0	fR/S	 167.7
U+92D9.0	fTotalStrokes	 15
U+92D9.0	fCangjie	CMMR
U+92D9.0	fFourCorner	81161
U+92D9.0	fHakka	 ngi3;  ngu2; ngi2
U+92D9.0	fCantonese	jyu5; ng4
U+92D9.0	fMandarin	wu2; yu3; yu2
U+92D9.0	fEnglish	[1] disharmonious; disordant [2] a kid of music instrument
U+92DA.0	fUTF8	鋚
U+92DA.0	fCNS11643	3-4F4B
U+92DA.0	fGB	5-7531
U+92DA.0	fR/S	 167.7; 167.12; 9.12
U+92DA.0	fTotalStrokes	 15; 14
U+92DA.0	fCangjie	OKC
U+92DA.0	fFourCorner	28100
U+92DA.0	fCantonese	tiu4
U+92DA.0	fMandarin	tiao2
U+92DA.0	fEnglish	 iron bronze decoration on bridle
U+92DB.0	fUTF8	鋛
U+92DB.0	fCNS11643	3-4F4E
U+92DB.0	fGB	E-4228
U+92DB.0	fR/S	  167.7
U+92DB.0	fTotalStrokes	 15
U+92DB.0	fCangjie	CLL
U+92DB.0	fFourCorner	85100
U+92DB.0	fCantonese	kong3
U+92DB.0	fMandarin	gong3
U+92DC.0	fUTF8	鋜
U+92DC.0	fCNS11643	3-4F44
U+92DC.0	fGB	3-7638
U+92DC.0	fR/S	  167.7
U+92DC.0	fTotalStrokes	 15
U+92DC.0	fCangjie	CRYO
U+92DC.0	fFourCorner	86180
U+92DC.0	fHakka	 cuk7; cok7
U+92DC.0	fCantonese	zok6; zuk1; zuk6
U+92DC.0	fMandarin	zhuo2
U+92DC.0	fEnglish	[1] put fetters on the feet [2] hoe
U+92DD.0	fUTF8	鋝
U+92DD.0	fCNS11643	2-5538
U+92DD.0	fBig5	E86E
U+92DD.0	fGB	1-6F32
U+92DD.0	fR/S	 167.7
U+92DD.0	fTotalStrokes	 15
U+92DD.0	fCangjie	CBDI
U+92DD.0	fFourCorner	82143
U+92DD.0	fHakka	  lot7
U+92DD.0	fCantonese	lyut3; lyut6
U+92DD.0	fMandarin	le4; luue4; lun4; shua1
U+92DD.0	fEnglish	 6 oz ancient measurement
U+92DE.0	fUTF8	鋞
U+92DE.0	fCNS11643	2-5545
U+92DE.0	fBig5	E87B
U+92DE.0	fGB	3-7579
U+92DE.0	fR/S	 167.7
U+92DE.0	fTotalStrokes	 15
U+92DE.0	fCangjie	CMVM
U+92DE.0	fFourCorner	81111
U+92DE.0	fCantonese	jing4; hang6
U+92DE.0	fMandarin	xing2; xing4
U+92DF.0	fUTF8	鋟
U+92DF.0	fCNS11643	2-5534
U+92DF.0	fBig5	E86A
U+92DF.0	fGB	1-6F37
U+92DF.0	fR/S	 167.7
U+92DF.0	fTotalStrokes	 15
U+92DF.0	fCangjie	CSME
U+92DF.0	fFourCorner	87147
U+92DF.0	fHakka	  cim1;  ciam1; zim5
U+92DF.0	fCantonese	cim1; cam2; zim1
U+92DF.0	fMandarin	qian1; qin3; qin1; jin4; jian1; qiu3
U+92DF.0	fEnglish	carve
U+92E0.0	fUTF8	鋠
U+92E0.0	fCNS11643	2-5544
U+92E0.0	fBig5	E87A
U+92E0.0	fGB	3-7636
U+92E0.0	fR/S	 167.7
U+92E0.0	fTotalStrokes	 15
U+92E0.0	fCangjie	CMMV
U+92E0.0	fFourCorner	81132
U+92E0.0	fCantonese	san6
U+92E0.0	fMandarin	shen4
U+92E1.0	fUTF8	鋡
U+92E1.0	fCNS11643	2-554A
U+92E1.0	fBig5	E8A2
U+92E1.0	fGB	3-763D
U+92E1.0	fR/S	 167.7
U+92E1.0	fTotalStrokes	 15
U+92E1.0	fCangjie	COIR
U+92E1.0	fFourCorner	88162
U+92E1.0	fCantonese	ham4
U+92E1.0	fMandarin	han2
U+92E2.0	fUTF8	鋢
U+92E2.0	fCNS11643	3-4F51
U+92E2.0	fGB	E-4229
U+92E2.0	fR/S	 167.7
U+92E2.0	fTotalStrokes	 15
U+92E2.0	fCangjie	CBND
U+92E2.0	fFourCorner	82140
U+92E2.0	fMandarin	luue4; le4
U+92E3.0	fUTF8	鋣
U+92E3.0	fCNS11643	4-4E46
U+92E3.0	fGB	1-6E74
U+92E3.0	fR/S	  167.7; 167.8
U+92E3.0	fTotalStrokes	 15
U+92E3.0	fCangjie	CMHL
U+92E3.0	fFourCorner	87120
U+92E3.0	fHakka	 ja2; ra2
U+92E3.0	fCantonese	je4
U+92E3.0	fMandarin	ye2; xie2
U+92E3.0	fEnglish	the name of a famous sword
U+92E4.0	fUTF8	鋤
U+92E4.0	fCNS11643	1-6F5C
U+92E4.0	fBig5	BE53
U+92E4.0	fGB	1-337A
U+92E4.0	fR/S	 167.7
U+92E4.0	fTotalStrokes	 15
U+92E4.0	fCangjie	CBMS
U+92E4.0	fFourCorner	84127
U+92E4.0	fHakka	 co2;  ts'io2; cu2; co3; cih2; ze2; cio2
U+92E4.0	fCantonese	co4
U+92E4.0	fMandarin	chu2; ju3; zhu4
U+92E4.0	fEnglish	 hoe
U+92E5.0	fUTF8	鋥
U+92E5.0	fCNS11643	3-4F45
U+92E5.0	fGB	1-6F2D
U+92E5.0	fR/S	 167.7
U+92E5.0	fTotalStrokes	 15
U+92E5.0	fCangjie	CRMG
U+92E5.0	fFourCorner	86110
U+92E5.0	fCantonese	caang3; cang3 (also); cing4
U+92E5.0	fMandarin	zheng4; zeng4
U+92E6.0	fUTF8	鋦
U+92E6.0	fCNS11643	2-5540
U+92E6.0	fBig5	E876
U+92E6.0	fGB	1-6F38
U+92E6.0	fR/S	 167.7
U+92E6.0	fTotalStrokes	 15
U+92E6.0	fCangjie	CSSR
U+92E6.0	fFourCorner	87127
U+92E6.0	fHakka	  kiuk8; kuk8
U+92E6.0	fCantonese	guk6; guk1; geoi3; geoi1
U+92E6.0	fMandarin	ju2; ju1
U+92E6.0	fEnglish	[1] a kind of nail for mending crockery [2] curium
U+92E7.0	fUTF8	鋧
U+92E7.0	fCNS11643	2-5546
U+92E7.0	fBig5	E87C
U+92E7.0	fGB	3-7562
U+92E7.0	fR/S	 167.7
U+92E7.0	fTotalStrokes	 15
U+92E7.0	fCangjie	CBUU
U+92E7.0	fFourCorner	86110
U+92E7.0	fHakka	 hien5; hien6
U+92E7.0	fCantonese	jin6; hin2
U+92E7.0	fMandarin	xian4; xian3
U+92E7.0	fEnglish	[1] small chisel [2] javelin
U+92E8.0	fUTF8	鋨
U+92E8.0	fCNS11643	2-553C
U+92E8.0	fBig5	E872
U+92E8.0	fGB	1-6F30
U+92E8.0	fR/S	 167.7
U+92E8.0	fTotalStrokes	 15
U+92E8.0	fCangjie	CHQI
U+92E8.0	fFourCorner	83150
U+92E8.0	fCantonese	ngo4; o4
U+92E8.0	fMandarin	e2; tie3
U+92E8.0	fEnglish	osmium
U+92E9.0	fUTF8	鋩
U+92E9.0	fCNS11643	2-5536
U+92E9.0	fBig5	E86C
U+92E9.0	fGB	3-7621
U+92E9.0	fR/S	 167.7
U+92E9.0	fTotalStrokes	 15
U+92E9.0	fCangjie	CTYV
U+92E9.0	fFourCorner	84110
U+92E9.0	fHakka	  mong2
U+92E9.0	fCantonese	mong4
U+92E9.0	fMandarin	mang2; wang2
U+92E9.0	fEnglish	the point of sword or knif; sharp point
U+92EA.0	fUTF8	鋪
U+92EA.0	fCNS11643	1-6F5B
U+92EA.0	fBig5	BE51
U+92EA.0	fGB	1-464C
U+92EA.0	fR/S	 167.7
U+92EA.0	fTotalStrokes	 15
U+92EA.0	fCangjie	CIJB
U+92EA.0	fFourCorner	83127
U+92EA.0	fHakka	  pu5; pu1
U+92EA.0	fCantonese	pou1; pou3
U+92EA.0	fMandarin	pu1; bu1; fu1
U+92EA.0	fEnglish	 spread out; arrange; lay in order; pave
U+92EA.1	fUTF8	鋪
U+92EA.1	fCNS11643	1-6F5B
U+92EA.1	fBig5	BE51
U+92EA.1	fGB	1-464C
U+92EA.1	fR/S	 167.7
U+92EA.1	fTotalStrokes	 15
U+92EA.1	fCangjie	CIJB
U+92EA.1	fFourCorner	83127
U+92EA.1	fHakka	 pu1
U+92EA.1	fMandarin	pu4
U+92EA.1	fEnglish	 shop; store; grocery
U+92EA.2	fUTF8	鋪
U+92EA.2	fCNS11643	1-6F5B
U+92EA.2	fBig5	BE51
U+92EA.2	fGB	1-464C
U+92EA.2	fR/S	 167.7
U+92EA.2	fTotalStrokes	 15
U+92EA.2	fCangjie	CIJB
U+92EA.2	fFourCorner	83127
U+92EB.0	fUTF8	鋫
U+92EB.0	fCNS11643	3-4F52
U+92EB.0	fGB	E-422A
U+92EB.0	fR/S	  167.7
U+92EB.0	fTotalStrokes	 15
U+92EB.0	fCangjie	HNC
U+92EB.0	fFourCorner	22100
U+92EB.0	fCantonese	lei4
U+92EB.0	fMandarin	li2
U+92EC.0	fUTF8	鋬
U+92EC.0	fCNS11643	4-4E44
U+92EC.0	fGB	3-7552
U+92EC.0	fR/S	 167.7
U+92EC.0	fTotalStrokes	 15
U+92EC.0	fCangjie	QEC
U+92EC.0	fFourCorner	52100
U+92EC.0	fCantonese	paan3
U+92EC.0	fMandarin	pan4
U+92ED.0	fUTF8	鋭
U+92ED.0	fCNS11643	3-4F4C
U+92ED.0	fGB	1-4871
U+92ED.0	fR/S	 167.7
U+92ED.0	fTotalStrokes	 15
U+92ED.0	fCangjie	CCRU
U+92ED.0	fFourCorner	88110
U+92ED.0	fCantonese	jeoi6
U+92ED.0	fMandarin	rui4; dui4
U+92ED.0	fEnglish	sharp
U+92EE.0	fUTF8	鋮
U+92EE.0	fCNS11643	2-4E6E
U+92EE.0	fBig5	E4A8
U+92EE.0	fGB	1-6E71
U+92EE.0	fR/S	 167.7; 167.6
U+92EE.0	fTotalStrokes	 15
U+92EE.0	fCangjie	CIHS
U+92EE.0	fFourCorner	83150
U+92EE.0	fCantonese	sing4
U+92EE.0	fMandarin	cheng2
U+92EE.0	fEnglish	person's name
U+92EF.0	fUTF8	鋯
U+92EF.0	fCNS11643	2-553A
U+92EF.0	fBig5	E870
U+92EF.0	fGB	1-6F2F
U+92EF.0	fR/S	 167.7
U+92EF.0	fTotalStrokes	 15
U+92EF.0	fCangjie	CHGR
U+92EF.0	fFourCorner	84161
U+92EF.0	fCantonese	gou3
U+92EF.0	fMandarin	gao4
U+92EF.0	fEnglish	zirconium
U+92F0.0	fUTF8	鋰
U+92F0.0	fCNS11643	1-6F62
U+92F0.0	fBig5	BE59
U+92F0.0	fGB	1-6F2E
U+92F0.0	fR/S	 167.7
U+92F0.0	fTotalStrokes	 15
U+92F0.0	fCangjie	CWG
U+92F0.0	fFourCorner	86114
U+92F0.0	fHakka	 li1
U+92F0.0	fCantonese	lei5
U+92F0.0	fMandarin	li3
U+92F0.0	fEnglish	lithium
U+92F1.0	fUTF8	鋱
U+92F1.0	fCNS11643	2-5533
U+92F1.0	fBig5	E869
U+92F1.0	fGB	1-6F2B
U+92F1.0	fR/S	 167.7
U+92F1.0	fTotalStrokes	 15
U+92F1.0	fCangjie	CIPP
U+92F1.0	fFourCorner	83140
U+92F1.0	fHakka	 tit7
U+92F1.0	fCantonese	tik1
U+92F1.0	fMandarin	te4
U+92F1.0	fEnglish	terbium
U+92F2.0	fUTF8	鋲
U+92F2.0	fCNS11643	F-526B
U+92F2.0	fGB	E-422B
U+92F2.0	fR/S	 167.7
U+92F2.0	fTotalStrokes	 15
U+92F2.0	fCangjie	COMC
U+92F2.0	fFourCorner	8218
U+92F2.0	fCantonese	bing1
U+92F2.0	fEnglish	rivet
U+92F3.0	fUTF8	鋳
U+92F3.0	fCNS11643	3-4F4D
U+92F3.0	fGB	E-422C
U+92F3.0	fR/S	 167.7
U+92F3.0	fTotalStrokes	 15
U+92F3.0	fCangjie	CQKI
U+92F3.0	fFourCorner	85140
U+92F3.0	fCantonese	zyu3
U+92F3.0	fMandarin	zhu4
U+92F3.0	fEnglish	 melt; cast coin; mint
U+92F4.0	fUTF8	鋴
U+92F4.0	fCNS11643	3-665B
U+92F4.0	fGB	E-422D
U+92F4.0	fR/S	 167.7
U+92F4.0	fTotalStrokes	 15
U+92F4.0	fCangjie	CJRU
U+92F4.0	fFourCorner	8411
U+92F4.0	fCantonese	hak1
U+92F5.0	fUTF8	鋵
U+92F5.0	fCNS11643	3-665C
U+92F5.0	fGB	E-422E
U+92F5.0	fR/S	 167.7
U+92F5.0	fTotalStrokes	 15
U+92F5.0	fCangjie	CHDU
U+92F5.0	fFourCorner	8211
U+92F5.0	fCantonese	tuk1
U+92F5.0	fMandarin	tu1
U+92F6.0	fUTF8	鋶
U+92F6.0	fCNS11643	3-4A30
U+92F6.0	fGB	1-6F33
U+92F6.0	fR/S	 167.7; 167.6
U+92F6.0	fTotalStrokes	 15; 14
U+92F6.0	fCangjie	CYIU
U+92F6.0	fFourCorner	80110
U+92F6.0	fCantonese	lau5
U+92F6.0	fMandarin	liu3
U+92F7.0	fUTF8	鋷
U+92F7.0	fCNS11643	2-5B41
U+92F7.0	fBig5	EBF4
U+92F7.0	fGB	E-422F
U+92F7.0	fR/S	 167.8
U+92F7.0	fTotalStrokes	 16
U+92F7.0	fCangjie	CSJE
U+92F7.0	fFourCorner	87140
U+92F7.0	fCantonese	je4; zeoi3
U+92F7.0	fMandarin	zui4
U+92F8.0	fUTF8	鋸
U+92F8.0	fCNS11643	1-7261
U+92F8.0	fBig5	BFF7
U+92F8.0	fGB	1-3E62
U+92F8.0	fR/S	 167.8
U+92F8.0	fTotalStrokes	 16
U+92F8.0	fCangjie	CSJR
U+92F8.0	fFourCorner	87164
U+92F8.0	fHakka	  gi5
U+92F8.0	fCantonese	geoi3; goe3 (also); guk1; geoi1
U+92F8.0	fMandarin	ju4; ju4ju1}; ju1
U+92F8.0	fEnglish	[1] saw [2] saw; cut with a saw [3] amputate
U+92F8.1	fUTF8	鋸
U+92F8.1	fCNS11643	1-7261
U+92F8.1	fBig5	BFF7
U+92F8.1	fGB	1-3E62
U+92F8.1	fR/S	 167.8
U+92F8.1	fTotalStrokes	 16
U+92F8.1	fCangjie	CSJR
U+92F8.1	fFourCorner	87164
U+92F8.1	fMandarin	ju1ju4}
U+92F8.1	fEnglish	[1] a kind of nail for mending crockery [2] mend (crockery) [3] curium
U+92F9.0	fUTF8	鋹
U+92F9.0	fCNS11643	2-5B40
U+92F9.0	fBig5	EBF3
U+92F9.0	fGB	3-7563
U+92F9.0	fR/S	 167.8
U+92F9.0	fTotalStrokes	 16
U+92F9.0	fCangjie	CSMV
U+92F9.0	fFourCorner	81132
U+92F9.0	fCantonese	cong2; coeng2 (also)
U+92F9.0	fMandarin	chang3
U+92F9.0	fEnglish	 sharp a keen edge sharp point
U+92FA.0	fUTF8	鋺
U+92FA.0	fCNS11643	2-5B3E
U+92FA.0	fBig5	EBF0
U+92FA.0	fGB	3-7653
U+92FA.0	fR/S	 167.8
U+92FA.0	fTotalStrokes	 16
U+92FA.0	fCangjie	CJNU
U+92FA.0	fFourCorner	83112
U+92FA.0	fCantonese	jyun2; jyun1
U+92FA.0	fMandarin	yuan1; wan3; yuan3
U+92FA.0	fEnglish	a metal bowl
U+92FB.0	fUTF8	鋻
U+92FB.0	fCNS11643	2-5B50
U+92FB.0	fBig5	EC44
U+92FB.0	fGB	3-754D
U+92FB.0	fR/S	 167.8; 167.9
U+92FB.0	fTotalStrokes	 16
U+92FB.0	fCangjie	SEC
U+92FB.0	fFourCorner	77109
U+92FB.0	fCantonese	gin3; gin1
U+92FB.0	fMandarin	jian4; jian1
U+92FC.0	fUTF8	鋼
U+92FC.0	fCNS11643	1-7265
U+92FC.0	fBig5	BFFB
U+92FC.0	fGB	1-3856
U+92FC.0	fR/S	 167.8
U+92FC.0	fTotalStrokes	 16
U+92FC.0	fCangjie	CBTU
U+92FC.0	fFourCorner	87120
U+92FC.0	fHakka	  gong5;  gong1
U+92FC.0	fCantonese	gong3; gong1 (also)
U+92FC.0	fMandarin	gang1; gang4
U+92FC.0	fEnglish	steel
U+92FD.0	fUTF8	鋽
U+92FD.0	fCNS11643	3-5335
U+92FD.0	fGB	3-7646
U+92FD.0	fR/S	  167.8
U+92FD.0	fTotalStrokes	 16
U+92FD.0	fCangjie	CYAJ
U+92FD.0	fFourCorner	81140
U+92FD.0	fCantonese	tiu3
U+92FD.0	fMandarin	diao4
U+92FE.0	fUTF8	鋾
U+92FE.0	fCNS11643	2-5B4D
U+92FE.0	fBig5	EC41
U+92FE.0	fGB	5-7563
U+92FE.0	fR/S	 167.8
U+92FE.0	fTotalStrokes	 16
U+92FE.0	fCangjie	CPOU
U+92FE.0	fFourCorner	87120
U+92FE.0	fHakka	 tau2; to3; rau2; jeu2
U+92FE.0	fCantonese	tou4
U+92FE.0	fMandarin	tao2
U+92FE.0	fEnglish	blunt (knife)
U+92FF.0	fUTF8	鋿
U+92FF.0	fCNS11643	2-5B45
U+92FF.0	fBig5	EBF8
U+92FF.0	fGB	5-7560
U+92FF.0	fR/S	 167.8
U+92FF.0	fTotalStrokes	 16
U+92FF.0	fCangjie	CFBR
U+92FF.0	fFourCorner	89127
U+92FF.0	fCantonese	soeng4
U+92FF.0	fMandarin	chang2
U+9300.0	fUTF8	錀
U+9300.0	fCNS11643	2-5B4F
U+9300.0	fBig5	EC43
U+9300.0	fGB	3-7566
U+9300.0	fR/S	 167.8
U+9300.0	fTotalStrokes	 16
U+9300.0	fCangjie	COMB
U+9300.0	fFourCorner	88127
U+9300.0	fCantonese	leon4; fan1
U+9300.0	fMandarin	lun2; fen1; lun3
U+9301.0	fUTF8	錁
U+9301.0	fCNS11643	2-5B37
U+9301.0	fBig5	EBE9
U+9301.0	fGB	1-6F3E
U+9301.0	fR/S	 167.8
U+9301.0	fTotalStrokes	 16
U+9301.0	fCangjie	CWD
U+9301.0	fFourCorner	86194
U+9301.0	fHakka	  ko5; go3
U+9301.0	fCantonese	gwo2; fo3; go2 (also)
U+9301.0	fMandarin	guo3; kua3; ke4
U+9301.0	fEnglish	 ingot acrobatic move
U+9302.0	fUTF8	錂
U+9302.0	fCNS11643	2-5B43
U+9302.0	fBig5	EBF6
U+9302.0	fGB	5-755D
U+9302.0	fR/S	 167.8
U+9302.0	fTotalStrokes	 16
U+9302.0	fCangjie	CGCE
U+9302.0	fFourCorner	84147
U+9302.0	fCantonese	ling4
U+9302.0	fMandarin	ling2
U+9303.0	fUTF8	錃
U+9303.0	fCNS11643	3-533D
U+9303.0	fGB	E-4230
U+9303.0	fR/S	 167.8
U+9303.0	fTotalStrokes	 16
U+9303.0	fCangjie	EEC
U+9303.0	fFourCorner	34100
U+9303.0	fCantonese	bei1
U+9303.0	fMandarin	bei1
U+9304.0	fUTF8	錄
U+9304.0	fCNS11643	1-7267
U+9304.0	fBig5	BFFD
U+9304.0	fGB	E-4231
U+9304.0	fR/S	 167.8
U+9304.0	fTotalStrokes	 16
U+9304.0	fCangjie	CVNE
U+9304.0	fFourCorner	87132
U+9304.0	fHakka	  liuk8; luk8
U+9304.0	fCantonese	luk6; luk2 (also)
U+9304.0	fMandarin	lu4
U+9304.0	fEnglish	copy; write down; record
U+9305.0	fUTF8	錅
U+9305.0	fCNS11643	3-533E
U+9305.0	fGB	E-4232
U+9305.0	fR/S	 167.8
U+9305.0	fTotalStrokes	 16
U+9305.0	fCangjie	HHC
U+9305.0	fFourCorner	27100
U+9305.0	fCantonese	lei4
U+9305.0	fMandarin	li2
U+9306.0	fUTF8	錆
U+9306.0	fCNS11643	2-5B2F
U+9306.0	fBig5	EBE1
U+9306.0	fGB	1-6F3A
U+9306.0	fR/S	 167.8
U+9306.0	fTotalStrokes	 16
U+9306.0	fCangjie	CQMB
U+9306.0	fFourCorner	85127
U+9306.0	fCantonese	coeng1
U+9306.0	fMandarin	qiang1
U+9306.0	fEnglish	the color of a mineral
U+9307.0	fUTF8	錇
U+9307.0	fCNS11643	3-5338
U+9307.0	fGB	1-6F42
U+9307.0	fR/S	 167.8
U+9307.0	fTotalStrokes	 16
U+9307.0	fCangjie	CYTR
U+9307.0	fFourCorner	80160
U+9307.0	fCantonese	pui4
U+9307.0	fMandarin	pou2; fu2; pei2
U+9307.0	fEnglish	berkelium
U+9308.0	fUTF8	錈
U+9308.0	fCNS11643	2-5B2D
U+9308.0	fBig5	EBDF
U+9308.0	fGB	1-6F43
U+9308.0	fR/S	 167.8
U+9308.0	fTotalStrokes	 16
U+9308.0	fCangjie	CFQU
U+9308.0	fFourCorner	89112
U+9308.0	fHakka	 gien3; kien3; gen3; ken3; gen1
U+9308.0	fCantonese	gyun2; gyun6
U+9308.0	fMandarin	juan4; juan3; quan1
U+9308.0	fEnglish	 bend iron
U+9309.0	fUTF8	錉
U+9309.0	fCNS11643	2-5B4E
U+9309.0	fBig5	EC42
U+9309.0	fGB	5-7562
U+9309.0	fR/S	 167.8
U+9309.0	fTotalStrokes	 16
U+9309.0	fCangjie	CHPA
U+9309.0	fFourCorner	82164
U+9309.0	fCantonese	man4
U+9309.0	fMandarin	min2
U+930A.0	fUTF8	錊
U+930A.0	fCNS11643	3-5332
U+930A.0	fGB	3-7650
U+930A.0	fR/S	  167.8
U+930A.0	fTotalStrokes	 16
U+930A.0	fCangjie	CYOJ
U+930A.0	fFourCorner	80140
U+930A.0	fCantonese	zeoi3
U+930A.0	fMandarin	zui4
U+930B.0	fUTF8	錋
U+930B.0	fCNS11643	2-5B4C
U+930B.0	fBig5	EC40
U+930B.0	fGB	E-4233
U+930B.0	fR/S	 167.8
U+930B.0	fTotalStrokes	 16
U+930B.0	fCangjie	CBB
U+930B.0	fFourCorner	87120
U+930B.0	fCantonese	paang4
U+930B.0	fMandarin	peng2; beng4
U+930C.0	fUTF8	錌
U+930C.0	fCNS11643	2-5B4B
U+930C.0	fBig5	EBFE
U+930C.0	fGB	E-4234
U+930C.0	fR/S	 167.8
U+930C.0	fTotalStrokes	 16
U+930C.0	fCangjie	CUMJ
U+930C.0	fFourCorner	82141
U+930C.0	fCantonese	ngon6; on6 (also)
U+930C.0	fMandarin	an4
U+930D.0	fUTF8	錍
U+930D.0	fCNS11643	2-5B3B
U+930D.0	fBig5	EBED
U+930D.0	fGB	3-764B
U+930D.0	fR/S	 167.8
U+930D.0	fTotalStrokes	 16
U+930D.0	fCangjie	CHWJ
U+930D.0	fFourCorner	86140
U+930D.0	fCantonese	pai1
U+930D.0	fMandarin	pi2; pi1; bei1; bi1
U+930E.0	fUTF8	錎
U+930E.0	fCNS11643	2-5B3A
U+930E.0	fBig5	EBEC
U+930E.0	fGB	3-764E
U+930E.0	fR/S	 167.8
U+930E.0	fTotalStrokes	 16
U+930E.0	fCangjie	CNHX
U+930E.0	fFourCorner	87177
U+930E.0	fCantonese	haam6
U+930E.0	fMandarin	xian4; fen1; gan4; kan3
U+930F.0	fUTF8	錏
U+930F.0	fCNS11643	2-5B30
U+930F.0	fBig5	EBE2
U+930F.0	fGB	3-7622
U+930F.0	fR/S	 167.8
U+930F.0	fTotalStrokes	 16
U+930F.0	fCangjie	CMLM
U+930F.0	fFourCorner	81117
U+930F.0	fHakka	 a3
U+930F.0	fCantonese	aa1; ngaa1 (also); aa3
U+930F.0	fMandarin	ya4; ya1
U+930F.0	fEnglish	 soft steel ammonium
U+9310.0	fUTF8	錐
U+9310.0	fCNS11643	1-7269
U+9310.0	fBig5	C040
U+9310.0	fGB	1-5736
U+9310.0	fR/S	 167.8
U+9310.0	fTotalStrokes	 16
U+9310.0	fCangjie	COG
U+9310.0	fFourCorner	80114
U+9310.0	fHakka	 zui1;  chui1; zui2; jui1; zhui1
U+9310.0	fCantonese	zeoi1; jeoi1
U+9310.0	fMandarin	zhui1
U+9310.0	fEnglish	[1] awl [2] pierce; drill; make a hole; bore [3] conical
U+9311.0	fUTF8	錑
U+9311.0	fCNS11643	4-5443
U+9311.0	fGB	3-7654
U+9311.0	fR/S	 167.8
U+9311.0	fTotalStrokes	 16
U+9311.0	fCangjie	CHSK
U+9311.0	fFourCorner	83134
U+9311.0	fCantonese	leoi6
U+9311.0	fMandarin	lei4
U+9312.0	fUTF8	錒
U+9312.0	fCNS11643	2-5B36
U+9312.0	fBig5	EBE8
U+9312.0	fGB	1-6F39
U+9312.0	fR/S	 167.8
U+9312.0	fTotalStrokes	 16
U+9312.0	fCangjie	CNLR
U+9312.0	fFourCorner	81120
U+9312.0	fHakka	 a1
U+9312.0	fCantonese	aa3; ngaa3 (also); aa1 (also); ngaa1 (also); ngo1 (also); o1
U+9312.0	fMandarin	a1; e1; ke1
U+9312.0	fEnglish	actinium
U+9313.0	fUTF8	錓
U+9313.0	fCNS11643	2-5B3F
U+9313.0	fBig5	EBF2
U+9313.0	fGB	E-4235
U+9313.0	fR/S	 167.8
U+9313.0	fTotalStrokes	 16
U+9313.0	fCangjie	CTGU
U+9313.0	fFourCorner	88111
U+9313.0	fCantonese	kwang1; haang1
U+9313.0	fMandarin	kong1
U+9314.0	fUTF8	錔
U+9314.0	fCNS11643	2-5B4A
U+9314.0	fBig5	EBFD
U+9314.0	fGB	3-7658
U+9314.0	fR/S	 167.8
U+9314.0	fTotalStrokes	 16
U+9314.0	fCangjie	CEA
U+9314.0	fFourCorner	82163
U+9314.0	fHakka	 tap7; {lap7}
U+9314.0	fCantonese	daap6 (also); taap3
U+9314.0	fMandarin	ta4
U+9314.0	fEnglish	cap
U+9315.0	fUTF8	錕
U+9315.0	fCNS11643	1-726C
U+9315.0	fBig5	C043
U+9315.0	fGB	1-6F3F
U+9315.0	fR/S	 167.8
U+9315.0	fTotalStrokes	 16
U+9315.0	fCangjie	CAPP
U+9315.0	fFourCorner	86111
U+9315.0	fHakka	 kun1; kwun1; gwun3
U+9315.0	fCantonese	kwan1; gwan1
U+9315.0	fMandarin	kun1; gun3; hun4; ran2
U+9315.0	fEnglish	[CC@] the name of an ancient treasured sword
U+9316.0	fUTF8	錖
U+9316.0	fCNS11643	2-5B51
U+9316.0	fBig5	EC45
U+9316.0	fGB	5-7532
U+9316.0	fR/S	 167.8
U+9316.0	fTotalStrokes	 16
U+9316.0	fCangjie	YEC
U+9316.0	fFourCorner	27109
U+9316.0	fCantonese	duk1
U+9316.0	fMandarin	du3
U+9317.0	fUTF8	錗
U+9317.0	fCNS11643	3-5336
U+9317.0	fGB	3-764A
U+9317.0	fR/S	 167.8
U+9317.0	fTotalStrokes	 16
U+9317.0	fCangjie	CHDV
U+9317.0	fFourCorner	82140
U+9317.0	fCantonese	neoi6; leoi6 (also)
U+9317.0	fMandarin	wei4
U+9318.0	fUTF8	錘
U+9318.0	fCNS11643	1-7573
U+9318.0	fBig5	C1E8
U+9318.0	fGB	1-3438
U+9318.0	fR/S	 167.8
U+9318.0	fTotalStrokes	 16
U+9318.0	fCangjie	CHJM
U+9318.0	fFourCorner	82114
U+9318.0	fHakka	  cui2; ch'ui2; ch'ui6; cui5; chui2; ch'ui5
U+9318.0	fCantonese	ceoi4; ceoi2 (also); seoi4
U+9318.0	fMandarin	chui2; zhui3; zhui4
U+9318.0	fEnglish	[1] balance weight on scale; a weight on a steelyard [2] hammer [3] hammer; pound [4] a kind of ancient weapon [5] an ancient unit of weight
U+9319.0	fUTF8	錙
U+9319.0	fCNS11643	1-726E
U+9319.0	fBig5	C045
U+9319.0	fGB	1-6F45
U+9319.0	fR/S	 167.8
U+9319.0	fTotalStrokes	 16
U+9319.0	fCangjie	CVVW
U+9319.0	fFourCorner	82160
U+9319.0	fHakka	 zih1; zu1; zii1
U+9319.0	fCantonese	zi1
U+9319.0	fMandarin	zi1; wa4
U+9319.0	fEnglish	an ancient unit of weight (8 tael)
U+931A.0	fUTF8	錚
U+931A.0	fCNS11643	1-7268
U+931A.0	fBig5	BFFE
U+931A.0	fGB	1-6F23
U+931A.0	fR/S	 167.8
U+931A.0	fTotalStrokes	 16
U+931A.0	fCangjie	CBSD
U+931A.0	fFourCorner	82153
U+931A.0	fHakka	  zen1; zang1
U+931A.0	fCantonese	zang1; zaang1 (also); caang1
U+931A.0	fMandarin	zheng1; zheng4; cheng1; jian3; qian2
U+931A.0	fEnglish	[1] clang of metal [2] gongs
U+931B.0	fUTF8	錛
U+931B.0	fCNS11643	2-5B34
U+931B.0	fBig5	EBE6
U+931B.0	fGB	1-6F3C
U+931B.0	fR/S	 167.8
U+931B.0	fTotalStrokes	 16
U+931B.0	fCangjie	CKJT
U+931B.0	fFourCorner	84144
U+931B.0	fHakka	 bun1
U+931B.0	fCantonese	ban1
U+931B.0	fMandarin	ben1
U+931B.0	fEnglish	 adz; adze
U+931C.0	fUTF8	錜
U+931C.0	fCNS11643	3-5337
U+931C.0	fGB	3-764D
U+931C.0	fR/S	 167.8
U+931C.0	fTotalStrokes	 16
U+931C.0	fCangjie	COIP
U+931C.0	fFourCorner	88130
U+931C.0	fHakka	 ngiap7; ngiam5; ngiam6
U+931C.0	fCantonese	nip6; lip6 (also)
U+931C.0	fMandarin	nie1; nie4
U+931C.0	fEnglish	[1] small hairpin [2] nail with a small head
U+931D.0	fUTF8	錝
U+931D.0	fCNS11643	2-5B3D
U+931D.0	fBig5	EBEF
U+931D.0	fGB	E-4236
U+931D.0	fR/S	 167.8
U+931D.0	fTotalStrokes	 16
U+931D.0	fCangjie	CJMF
U+931D.0	fFourCorner	83191
U+931D.0	fCantonese	zung3; cung4
U+931D.0	fMandarin	cong2; zong4
U+931E.0	fUTF8	錞
U+931E.0	fCNS11643	2-5B2C
U+931E.0	fBig5	EBDE
U+931E.0	fGB	3-764F
U+931E.0	fR/S	 167.8
U+931E.0	fTotalStrokes	 16
U+931E.0	fCangjie	CYRD
U+931E.0	fFourCorner	80147
U+931E.0	fHakka	  dun1; shun2; sun2; dui5
U+931E.0	fCantonese	deoi6; seon4
U+931E.0	fMandarin	chun2; dui4; duo4; dun1; qun2
U+931E.0	fEnglish	instrument
U+931F.0	fUTF8	錟
U+931F.0	fCNS11643	2-5B2E
U+931F.0	fBig5	EBE0
U+931F.0	fGB	1-6F44
U+931F.0	fR/S	 167.8
U+931F.0	fTotalStrokes	 16
U+931F.0	fCangjie	CFF
U+931F.0	fFourCorner	89189
U+931F.0	fHakka	  tam2; jam3; ram3
U+931F.0	fCantonese	taam4; cim1; taan2
U+931F.0	fMandarin	tan2; xian1; yan3; jian4; tan1
U+931F.0	fEnglish	long spear
U+9320.0	fUTF8	錠
U+9320.0	fCNS11643	1-725F
U+9320.0	fBig5	BFF5
U+9320.0	fGB	1-3627
U+9320.0	fR/S	 167.8
U+9320.0	fTotalStrokes	 16
U+9320.0	fCangjie	CJMO
U+9320.0	fFourCorner	83181
U+9320.0	fHakka	 tin6; tin3;  tin5; dun1
U+9320.0	fCantonese	ding3; ding6 (also); ding2
U+9320.0	fMandarin	ding4; dian4
U+9320.0	fEnglish	[1] a kind of ancient utensil [2] ingots of gold or silver [3] spindle [4] medical tablet
U+9321.0	fUTF8	錡
U+9321.0	fCNS11643	1-726B
U+9321.0	fBig5	C042
U+9321.0	fGB	3-7640
U+9321.0	fR/S	 167.8
U+9321.0	fTotalStrokes	 16
U+9321.0	fCangjie	CKMR
U+9321.0	fFourCorner	84121
U+9321.0	fHakka	 ki3; ki5; ji3; ri3
U+9321.0	fCantonese	kei4; ji2
U+9321.0	fMandarin	qi2; yi3; ji4
U+9321.0	fEnglish	[1] a kind of chisel [2] tripod
U+9322.0	fUTF8	錢
U+9322.0	fCNS11643	1-7264
U+9322.0	fBig5	BFFA
U+9322.0	fGB	1-472E
U+9322.0	fR/S	 167.8
U+9322.0	fTotalStrokes	 16
U+9322.0	fCangjie	CII
U+9322.0	fFourCorner	83153
U+9322.0	fHakka	  cien2;  cen2; ts'ian3; cen3
U+9322.0	fCantonese	cin4; zin2; cin2 (also)
U+9322.0	fMandarin	qian2; jian3; zi1
U+9322.0	fEnglish	[1] cash; money; currency; coins [2] a unit of weight (1/10 of a tael) [3] a Chinese family name
U+9322.1	fUTF8	錢
U+9322.1	fCNS11643	1-7264
U+9322.1	fBig5	BFFA
U+9322.1	fGB	1-472E
U+9322.1	fR/S	 167.8
U+9322.1	fTotalStrokes	 16
U+9322.1	fCangjie	CII
U+9322.1	fFourCorner	83153
U+9322.1	fMandarin	qian5
U+9323.0	fUTF8	錣
U+9323.0	fCNS11643	2-5B35
U+9323.0	fBig5	EBE7
U+9323.0	fGB	3-7656
U+9323.0	fR/S	 167.8
U+9323.0	fTotalStrokes	 16
U+9323.0	fCangjie	CEEE
U+9323.0	fFourCorner	87147
U+9323.0	fHakka	  dot7; duk7
U+9323.0	fCantonese	zyut3; zeoi3
U+9323.0	fMandarin	zhuo2; zhui4; zheng1
U+9323.0	fEnglish	[1] needle; awl [2] sharp [3] present
U+9324.0	fUTF8	錤
U+9324.0	fCNS11643	2-5B44
U+9324.0	fBig5	EBF7
U+9324.0	fGB	5-755E
U+9324.0	fR/S	 167.8
U+9324.0	fTotalStrokes	 16
U+9324.0	fCangjie	CTMC
U+9324.0	fFourCorner	84181
U+9324.0	fHakka	 ki2; gi1
U+9324.0	fCantonese	gei1
U+9324.0	fMandarin	qi2; ji1
U+9324.0	fEnglish	hoe
U+9325.0	fUTF8	錥
U+9325.0	fCNS11643	2-554B
U+9325.0	fBig5	EBF1
U+9325.0	fGB	3-7651
U+9325.0	fR/S	 167.8
U+9325.0	fTotalStrokes	 16
U+9325.0	fCangjie	CYIB
U+9325.0	fFourCorner	80127
U+9325.0	fCantonese	juk6
U+9325.0	fMandarin	yu4
U+9326.0	fUTF8	錦
U+9326.0	fCNS11643	1-726A
U+9326.0	fBig5	C041
U+9326.0	fGB	1-3D75
U+9326.0	fR/S	 167.8
U+9326.0	fTotalStrokes	 16
U+9326.0	fCangjie	CHAB
U+9326.0	fFourCorner	86127
U+9326.0	fHakka	  gim3
U+9326.0	fCantonese	gam2
U+9326.0	fMandarin	jin3
U+9326.0	fEnglish	[1] brocade; tapestry [2] brilliant and beautiful [3] glorious
U+9327.0	fUTF8	錧
U+9327.0	fCNS11643	2-5B2B
U+9327.0	fBig5	EBDD
U+9327.0	fGB	3-7652
U+9327.0	fR/S	 167.8
U+9327.0	fTotalStrokes	 16
U+9327.0	fCangjie	CJRR
U+9327.0	fFourCorner	83177
U+9327.0	fHakka	 gon3; gwon5
U+9327.0	fCantonese	gun2; gwun2 (also); gun3
U+9327.0	fMandarin	guan3; guan4
U+9328.0	fUTF8	錨
U+9328.0	fCNS11643	1-756E
U+9328.0	fBig5	C1E3
U+9328.0	fGB	1-432A
U+9328.0	fR/S	 167.9
U+9328.0	fTotalStrokes	 17
U+9328.0	fCangjie	CTW
U+9328.0	fFourCorner	84160
U+9328.0	fHakka	  mau2; nau2;  miau2; nau1; lau2
U+9328.0	fCantonese	naau4; maau4 (also); laau4 (also)
U+9328.0	fMandarin	mao2; mao1
U+9328.0	fEnglish	 anchor
U+9329.0	fUTF8	錩
U+9329.0	fCNS11643	2-5B46
U+9329.0	fBig5	EBF9
U+9329.0	fGB	3-7648
U+9329.0	fR/S	 167.8
U+9329.0	fTotalStrokes	 16
U+9329.0	fCangjie	CAA
U+9329.0	fFourCorner	86160
U+9329.0	fCantonese	coeng1
U+9329.0	fMandarin	chang1
U+9329.0	fEnglish	vessel
U+932A.0	fUTF8	錪
U+932A.0	fCNS11643	2-5B49
U+932A.0	fBig5	EBFC
U+932A.0	fGB	3-7649
U+932A.0	fR/S	 167.8
U+932A.0	fTotalStrokes	 16
U+932A.0	fCangjie	CTBC
U+932A.0	fFourCorner	85181
U+932A.0	fCantonese	tin2; din2
U+932A.0	fMandarin	tian3; dian3; tun2
U+932B.0	fUTF8	錫
U+932B.0	fCNS11643	1-7266
U+932B.0	fBig5	BFFC
U+932B.0	fGB	1-4E7D
U+932B.0	fR/S	 167.8
U+932B.0	fTotalStrokes	 16
U+932B.0	fCangjie	CAPH
U+932B.0	fFourCorner	86127
U+932B.0	fHakka	  siak7; sit7
U+932B.0	fCantonese	sek3; sik3 (also); sek1; sik1
U+932B.0	fMandarin	xi1; xi2; ti4; di4; si4
U+932B.0	fEnglish	[1] tin; stannum [2] bestow ... on; confer ... on
U+932C.0	fUTF8	錬
U+932C.0	fCNS11643	3-533B
U+932C.0	fGB	E-4237
U+932C.0	fR/S	 167.8
U+932C.0	fTotalStrokes	 16
U+932C.0	fCangjie	CDW
U+932C.0	fFourCorner	85190
U+932C.0	fMandarin	lian4
U+932C.0	fEnglish	smelt metals
U+932D.0	fUTF8	錭
U+932D.0	fCNS11643	2-5B39
U+932D.0	fBig5	EBEB
U+932D.0	fGB	5-7561
U+932D.0	fR/S	 167.8
U+932D.0	fTotalStrokes	 16
U+932D.0	fCangjie	CBGR
U+932D.0	fFourCorner	87120
U+932D.0	fHakka	 tiau1
U+932D.0	fCantonese	diu1
U+932D.0	fMandarin	tao2; diao1; dao1
U+932E.0	fUTF8	錮
U+932E.0	fCNS11643	1-726D
U+932E.0	fBig5	C044
U+932E.0	fGB	1-6F40
U+932E.0	fR/S	 167.8
U+932E.0	fTotalStrokes	 16
U+932E.0	fCangjie	CWJR
U+932E.0	fFourCorner	86100
U+932E.0	fHakka	  gu5
U+932E.0	fCantonese	gu3; gwu3 (also)
U+932E.0	fMandarin	gu4
U+932E.0	fEnglish	[1] run metal into cracks [2] confine; keep in custody; imprison [3] sturdy; secure
U+932F.0	fUTF8	錯
U+932F.0	fCNS11643	1-7263
U+932F.0	fBig5	BFF9
U+932F.0	fGB	1-346D
U+932F.0	fR/S	 167.8
U+932F.0	fTotalStrokes	 16
U+932F.0	fCangjie	CTA
U+932F.0	fFourCorner	84161
U+932F.0	fHakka	  co5;  cok7; dok7; cu5; cih5
U+932F.0	fCantonese	co3; cok3; cou3; laap6
U+932F.0	fMandarin	cuo4; cu4; que4; xi1; xi2
U+932F.0	fEnglish	[1] erroroneous; mistaken; wrong [2] error; mistake; blunder; fault [3] disorderly; untidy; irregular; uneven; complicated; intricate [4] grindstone [5] prudent; cautious
U+9330.0	fUTF8	錰
U+9330.0	fCNS11643	3-533C
U+9330.0	fGB	E-4238
U+9330.0	fR/S	  167.8
U+9330.0	fTotalStrokes	 16
U+9330.0	fCangjie	CDF
U+9330.0	fFourCorner	84130
U+9330.0	fCantonese	seot6
U+9330.0	fMandarin	shu4
U+9331.0	fUTF8	錱
U+9331.0	fCNS11643	3-5339
U+9331.0	fGB	E-4239
U+9331.0	fR/S	 167.8
U+9331.0	fTotalStrokes	 16
U+9331.0	fCangjie	CMGG
U+9331.0	fFourCorner	80110
U+9331.0	fCantonese	zan1
U+9331.0	fMandarin	zhen1
U+9332.0	fUTF8	録
U+9332.0	fCNS11643	3-533A
U+9332.0	fGB	1-423C
U+9332.0	fR/S	 167.8
U+9332.0	fTotalStrokes	 16
U+9332.0	fCangjie	CNME
U+9332.0	fFourCorner	87130
U+9332.0	fCantonese	luk6
U+9332.0	fMandarin	lu4; l4
U+9332.0	fEnglish	copy
U+9333.0	fUTF8	錳
U+9333.0	fCNS11643	1-7262
U+9333.0	fBig5	BFF8
U+9333.0	fGB	1-434C
U+9333.0	fR/S	 167.8
U+9333.0	fTotalStrokes	 16
U+9333.0	fCangjie	CNDT
U+9333.0	fFourCorner	87117
U+9333.0	fHakka	 mang1
U+9333.0	fCantonese	maang5
U+9333.0	fMandarin	meng3
U+9333.0	fEnglish	manganese
U+9334.0	fUTF8	錴
U+9334.0	fCNS11643	2-5B42
U+9334.0	fBig5	EBF5
U+9334.0	fGB	E-423A
U+9334.0	fR/S	 167.8
U+9334.0	fTotalStrokes	 16
U+9334.0	fCangjie	CGCG
U+9334.0	fFourCorner	84114
U+9334.0	fCantonese	luk6
U+9334.0	fMandarin	lu4
U+9335.0	fUTF8	錵
U+9335.0	fCNS11643	2-5B48
U+9335.0	fBig5	EBFB
U+9335.0	fGB	E-423B
U+9335.0	fR/S	 167.8
U+9335.0	fTotalStrokes	 16
U+9335.0	fCangjie	CTOP
U+9335.0	fFourCorner	84114
U+9335.0	fCantonese	waa4; faa1
U+9335.0	fMandarin	hua1
U+9336.0	fUTF8	錶
U+9336.0	fCNS11643	1-7260
U+9336.0	fBig5	BFF6
U+9336.0	fGB	1-7823
U+9336.0	fR/S	 167.8
U+9336.0	fTotalStrokes	 16
U+9336.0	fCangjie	CQMV
U+9336.0	fFourCorner	85132
U+9336.0	fHakka	 biau1
U+9336.0	fCantonese	biu1; biu2
U+9336.0	fMandarin	biao3
U+9336.0	fEnglish	 watch (timepiece); clock
U+9337.0	fUTF8	錷
U+9337.0	fCNS11643	4-5445
U+9337.0	fGB	E-423C
U+9337.0	fR/S	 167.8
U+9337.0	fTotalStrokes	 16
U+9337.0	fCangjie	CJJU
U+9337.0	fFourCorner	82110
U+9337.0	fCantonese	aat3; ngaat3 (also); gaa1
U+9337.0	fMandarin	ga2
U+9337.0	fEnglish	gadolinium
U+9338.0	fUTF8	錸
U+9338.0	fCNS11643	2-5B32
U+9338.0	fBig5	EBE4
U+9338.0	fGB	1-6F2A
U+9338.0	fR/S	 167.8
U+9338.0	fTotalStrokes	 16
U+9338.0	fCangjie	CDOO
U+9338.0	fFourCorner	84198
U+9338.0	fHakka	 loi2
U+9338.0	fCantonese	loi4
U+9338.0	fMandarin	lai2
U+9338.0	fEnglish	rhenium
U+9339.0	fUTF8	錹
U+9339.0	fCNS11643	2-5B47
U+9339.0	fBig5	EBFA
U+9339.0	fGB	E-423D
U+9339.0	fR/S	 167.8
U+9339.0	fTotalStrokes	 16
U+9339.0	fCangjie	CYMB
U+9339.0	fFourCorner	81127
U+9339.0	fCantonese	hang2
U+9339.0	fMandarin	ken3
U+933A.0	fUTF8	錺
U+933A.0	fCNS11643	F-5859
U+933A.0	fGB	E-423E
U+933A.0	fR/S	 167.8
U+933A.0	fTotalStrokes	 16
U+933A.0	fCangjie	CTYS
U+933A.0	fFourCorner	8412
U+933A.0	fCantonese	fong1
U+933A.0	fMandarin	zhui4
U+933A.0	fEnglish	metal jewelry
U+933B.0	fUTF8	錻
U+933B.0	fCNS11643	F-5848
U+933B.0	fGB	E-423F
U+933B.0	fR/S	 167.8
U+933B.0	fTotalStrokes	 16
U+933B.0	fCangjie	CMPM
U+933B.0	fFourCorner	8314
U+933B.0	fCantonese	mou5
U+933B.0	fEnglish	a tin plate
U+933C.0	fUTF8	錼
U+933C.0	fCNS11643	2-5B33
U+933C.0	fBig5	EBE5
U+933C.0	fGB	E-4240
U+933C.0	fR/S	 167.8
U+933C.0	fTotalStrokes	 16
U+933C.0	fCangjie	CKMF
U+933C.0	fFourCorner	80191
U+933C.0	fCantonese	noi6; loi6 (also)
U+933C.0	fMandarin	nai4
U+933C.0	fEnglish	np
U+933D.0	fUTF8	錽
U+933D.0	fCNS11643	7-3463
U+933D.0	fGB	3-7642
U+933D.0	fR/S	 167.8
U+933D.0	fTotalStrokes	 16
U+933D.0	fCangjie	CMFE
U+933D.0	fFourCorner	81140
U+933D.0	fCantonese	faan6
U+933D.0	fMandarin	wan3; wan4; jian3
U+933E.0	fUTF8	錾
U+933E.0	fGB	0-7649
U+933E.0	fR/S	 167.8
U+933E.0	fTotalStrokes	 16
U+933E.0	fCangjie	KLC
U+933E.0	fHakka	 cam5
U+933E.0	fCantonese	zaam6
U+933E.0	fMandarin	zan4; can2; chan2; jian4; keng1
U+933E.0	fEnglish	simplified form of &#x93E8;
U+933F.0	fUTF8	錿
U+933F.0	fCNS11643	3-665D
U+933F.0	fGB	E-4241
U+933F.0	fR/S	 167.8
U+933F.0	fTotalStrokes	 16
U+933F.0	fCangjie	CYPU
U+933F.0	fFourCorner	8111
U+933F.0	fCantonese	fu2
U+9340.0	fUTF8	鍀
U+9340.0	fCNS11643	F-5855
U+9340.0	fGB	1-6F3D
U+9340.0	fR/S	 167.8
U+9340.0	fTotalStrokes	 16
U+9340.0	fCangjie	CAMI
U+9340.0	fFourCorner	8614
U+9340.0	fCantonese	dak1
U+9340.0	fMandarin	de2
U+9340.0	fEnglish	technetium
U+9341.0	fUTF8	鍁
U+9341.0	fGB	1-4F47
U+9341.0	fR/S	 167.8
U+9341.0	fTotalStrokes	 16
U+9341.0	fCangjie	CHLO
U+9341.0	fFourCorner	8718
U+9341.0	fCantonese	him1; hin1 (also); jan1
U+9341.0	fMandarin	xian1
U+9342.0	fUTF8	鍂
U+9342.0	fCNS11643	3-665E
U+9342.0	fGB	E-4242
U+9342.0	fR/S	 167.8
U+9342.0	fTotalStrokes	 16
U+9342.0	fCangjie	CC
U+9342.0	fFourCorner	8811
U+9342.0	fCantonese	gam1
U+9343.0	fUTF8	鍃
U+9343.0	fCNS11643	4-5449
U+9343.0	fGB	1-6F41
U+9343.0	fR/S	 167.8
U+9343.0	fTotalStrokes	 16
U+9343.0	fCangjie	CPHP
U+9343.0	fFourCorner	87130
U+9343.0	fCantonese	fat1; kut3 (also); kwut3 (also)
U+9343.0	fMandarin	cong1; huo1; huo4
U+9344.0	fUTF8	鍄
U+9344.0	fGB	E-4243
U+9344.0	fR/S	 167.8
U+9344.0	fTotalStrokes	 16
U+9344.0	fCangjie	CYRF
U+9344.0	fFourCorner	8019
U+9344.0	fCantonese	loeng6
U+9344.0	fMandarin	liang4
U+9345.0	fUTF8	鍅
U+9345.0	fCNS11643	3-5331
U+9345.0	fGB	E-4244
U+9345.0	fR/S	 167.8
U+9345.0	fTotalStrokes	 16
U+9345.0	fCangjie	CEGI
U+9345.0	fFourCorner	84130
U+9345.0	fCantonese	faat3
U+9346.0	fUTF8	鍆
U+9346.0	fCNS11643	2-5B38
U+9346.0	fBig5	EBEA
U+9346.0	fGB	1-6E4D
U+9346.0	fR/S	 167.8
U+9346.0	fTotalStrokes	 16
U+9346.0	fCangjie	CAN
U+9346.0	fFourCorner	87120
U+9346.0	fCantonese	mun4
U+9346.0	fMandarin	men2
U+9346.0	fEnglish	Md mendelerium
U+9347.0	fUTF8	鍇
U+9347.0	fCNS11643	2-5F7E
U+9347.0	fBig5	EED2
U+9347.0	fGB	1-6F47
U+9347.0	fR/S	 167.9
U+9347.0	fTotalStrokes	 17
U+9347.0	fCangjie	CPPA
U+9347.0	fFourCorner	81161
U+9347.0	fHakka	  gai1; koi3
U+9347.0	fCantonese	kaai2; gaai1 (also)
U+9347.0	fMandarin	kai3; jie1; jie3; xie2
U+9347.0	fEnglish	 refined iron; high quality iron
U+9348.0	fUTF8	鍈
U+9348.0	fCNS11643	3-5665
U+9348.0	fGB	3-763F
U+9348.0	fR/S	 167.8; 167.9
U+9348.0	fTotalStrokes	 17; 16
U+9348.0	fCangjie	CTLK
U+9348.0	fFourCorner	84130
U+9348.0	fHakka	 jin1
U+9348.0	fCantonese	joeng1
U+9348.0	fMandarin	ying1
U+9349.0	fUTF8	鍉
U+9349.0	fCNS11643	2-6025
U+9349.0	fBig5	EED7
U+9349.0	fGB	5-7569
U+9349.0	fR/S	 167.9
U+9349.0	fTotalStrokes	 17
U+9349.0	fCangjie	CAMO
U+9349.0	fFourCorner	86181
U+9349.0	fHakka	 ti2; shi2; shi3; sii2
U+9349.0	fCantonese	dik6; si4; dik1; tai4
U+9349.0	fMandarin	di1; chi2; di4; di2; ti2; shi2
U+9349.0	fEnglish	spoon; key
U+934A.0	fUTF8	鍊
U+934A.0	fCNS11643	1-7570
U+934A.0	fBig5	C1E5
U+934A.0	fGB	E-4245
U+934A.0	fR/S	 167.9
U+934A.0	fTotalStrokes	 17
U+934A.0	fCangjie	CDWF
U+934A.0	fFourCorner	85196
U+934A.0	fHakka	  lien5; lien6; len3; lian6
U+934A.0	fCantonese	lin6; lin2 (also)
U+934A.0	fMandarin	lian4; jian4
U+934A.0	fEnglish	[1] smelt; forge; refine; temper [2] polish [3] chain
U+934B.0	fUTF8	鍋
U+934B.0	fCNS11643	1-7572
U+934B.0	fBig5	C1E7
U+934B.0	fGB	1-3978
U+934B.0	fR/S	 167.9
U+934B.0	fTotalStrokes	 17
U+934B.0	fCangjie	CBBR
U+934B.0	fFourCorner	87127
U+934B.0	fHakka	  wok8; ko5; ko1; go1; vok8
U+934B.0	fCantonese	wo1
U+934B.0	fMandarin	guo1; guo3
U+934B.0	fEnglish	[1] pan; cooking-pot; boiler; caldron [2] bowl (said of a pipe, etc.)
U+934C.0	fUTF8	鍌
U+934C.0	fCNS11643	2-602B
U+934C.0	fBig5	EEDD
U+934C.0	fGB	3-7554
U+934C.0	fR/S	 167.9
U+934C.0	fTotalStrokes	 17
U+934C.0	fCangjie	EUC
U+934C.0	fFourCorner	34109
U+934C.0	fCantonese	sin2
U+934C.0	fMandarin	xian3
U+934D.0	fUTF8	鍍
U+934D.0	fCNS11643	1-756C
U+934D.0	fBig5	C1E1
U+934D.0	fGB	1-3646
U+934D.0	fR/S	 167.9
U+934D.0	fTotalStrokes	 17
U+934D.0	fCangjie	CITE
U+934D.0	fFourCorner	80147
U+934D.0	fHakka	 tu5; du5
U+934D.0	fCantonese	dou6
U+934D.0	fMandarin	du4; tu2
U+934D.0	fEnglish	 plate; gild
U+934E.0	fUTF8	鍎
U+934E.0	fCNS11643	2-6039
U+934E.0	fBig5	EEEC
U+934E.0	fGB	E-4246
U+934E.0	fR/S	 167.9
U+934E.0	fTotalStrokes	 17
U+934E.0	fCangjie	CHJU
U+934E.0	fFourCorner	82164
U+934E.0	fCantonese	dat6
U+934E.0	fMandarin	tu2
U+934F.0	fUTF8	鍏
U+934F.0	fCNS11643	2-6031
U+934F.0	fBig5	EEE3
U+934F.0	fGB	5-753A
U+934F.0	fR/S	 167.9
U+934F.0	fTotalStrokes	 17
U+934F.0	fCangjie	CDMQ
U+934F.0	fFourCorner	84156
U+934F.0	fCantonese	wai4; wai5
U+934F.0	fMandarin	wei2; wei3
U+9350.0	fUTF8	鍐
U+9350.0	fCNS11643	2-6026
U+9350.0	fBig5	EED8
U+9350.0	fGB	E-4247
U+9350.0	fR/S	 167.9
U+9350.0	fTotalStrokes	 17
U+9350.0	fCangjie	CUCE
U+9350.0	fFourCorner	82147
U+9350.0	fHakka	 zung1
U+9350.0	fCantonese	zung1
U+9350.0	fMandarin	cong1; wan3
U+9350.0	fEnglish	a headstall; ornament on a bride
U+9351.0	fUTF8	鍑
U+9351.0	fCNS11643	2-6027
U+9351.0	fBig5	EED9
U+9351.0	fGB	3-7660
U+9351.0	fR/S	 167.9
U+9351.0	fTotalStrokes	 17
U+9351.0	fCangjie	COAE
U+9351.0	fFourCorner	88147
U+9351.0	fCantonese	fuk1; fu3 (also)
U+9351.0	fMandarin	fu4; fu2
U+9352.0	fUTF8	鍒
U+9352.0	fCNS11643	2-6030
U+9352.0	fBig5	EEE2
U+9352.0	fGB	E-4248
U+9352.0	fR/S	 167.9
U+9352.0	fTotalStrokes	 17
U+9352.0	fCangjie	CNHD
U+9352.0	fFourCorner	87194
U+9352.0	fCantonese	jau4
U+9352.0	fMandarin	rou2
U+9353.0	fUTF8	鍓
U+9353.0	fCNS11643	3-5666
U+9353.0	fGB	E-4249
U+9353.0	fR/S	 167.9
U+9353.0	fTotalStrokes	 17
U+9353.0	fCangjie	CRSJ
U+9353.0	fFourCorner	86140
U+9353.0	fCantonese	cip3
U+9353.0	fMandarin	ji2
U+9354.0	fUTF8	鍔
U+9354.0	fCNS11643	1-7579
U+9354.0	fBig5	C1EE
U+9354.0	fGB	1-6F49
U+9354.0	fR/S	 167.9
U+9354.0	fTotalStrokes	 17
U+9354.0	fCangjie	CRRS
U+9354.0	fFourCorner	86127
U+9354.0	fHakka	  ngok8
U+9354.0	fCantonese	ngok6; ok6 (also)
U+9354.0	fMandarin	e4
U+9354.0	fEnglish	[1] towering; high; lofty [2] edege of knife
U+9355.0	fUTF8	鍕
U+9355.0	fCNS11643	2-602F
U+9355.0	fBig5	EEE1
U+9355.0	fGB	E-424A
U+9355.0	fR/S	 167.9
U+9355.0	fTotalStrokes	 17
U+9355.0	fCangjie	CBJJ
U+9355.0	fFourCorner	87152
U+9355.0	fCantonese	gwan1
U+9355.0	fMandarin	rou2; rou3; jun1
U+9356.0	fUTF8	鍖
U+9356.0	fCNS11643	2-5F7D
U+9356.0	fBig5	EED1
U+9356.0	fGB	3-765A
U+9356.0	fR/S	 167.9
U+9356.0	fTotalStrokes	 17
U+9356.0	fCangjie	CTMV
U+9356.0	fFourCorner	84111
U+9356.0	fHakka	 zem1
U+9356.0	fCantonese	zam1; cam2
U+9356.0	fMandarin	chen3; zhen1; tou1
U+9357.0	fUTF8	鍗
U+9357.0	fCNS11643	2-602E
U+9357.0	fBig5	EEE0
U+9357.0	fGB	3-7666
U+9357.0	fR/S	 167.9
U+9357.0	fTotalStrokes	 17
U+9357.0	fCangjie	CYBB
U+9357.0	fFourCorner	80127
U+9357.0	fCantonese	tai4; dai3
U+9357.0	fMandarin	ti2
U+9358.0	fUTF8	鍘
U+9358.0	fCNS11643	2-6022
U+9358.0	fBig5	EED4
U+9358.0	fGB	1-5521
U+9358.0	fR/S	 167.9
U+9358.0	fTotalStrokes	 17
U+9358.0	fCangjie	CBCN
U+9358.0	fFourCorner	82100
U+9358.0	fHakka	 zap8; cet7
U+9358.0	fCantonese	zaat3 (also); zaap6
U+9358.0	fMandarin	zha2
U+9358.0	fEnglish	[1] a long knife hinged at one end for cutting grass or hay [2] cut up with a or hay cutter
U+9359.0	fUTF8	鍙
U+9359.0	fCNS11643	2-603A
U+9359.0	fBig5	EEED
U+9359.0	fGB	E-424B
U+9359.0	fR/S	 167.9
U+9359.0	fTotalStrokes	 17
U+9359.0	fCangjie	ECC
U+9359.0	fFourCorner	34109
U+9359.0	fCantonese	huk1
U+9359.0	fMandarin	hong4
U+935A.0	fUTF8	鍚
U+935A.0	fCNS11643	1-7578
U+935A.0	fBig5	C1ED
U+935A.0	fGB	3-755C
U+935A.0	fR/S	 167.9
U+935A.0	fTotalStrokes	 17
U+935A.0	fCangjie	CAMH
U+935A.0	fFourCorner	86127
U+935A.0	fHakka	 jong2; rong2; tong5
U+935A.0	fCantonese	joeng4
U+935A.0	fMandarin	yang2
U+935A.0	fEnglish	frontlet
U+935B.0	fUTF8	鍛
U+935B.0	fCNS11643	1-7576
U+935B.0	fBig5	C1EB
U+935B.0	fGB	1-364D
U+935B.0	fR/S	 167.9
U+935B.0	fTotalStrokes	 17
U+935B.0	fCangjie	CHJE
U+935B.0	fFourCorner	87147
U+935B.0	fHakka	 ton3;  ton5; ton6; don5
U+935B.0	fCantonese	dyun3
U+935B.0	fMandarin	duan4
U+935B.0	fEnglish	[1] forge (iron, metal, etc.) [2] smelt; temper; refine
U+935C.0	fUTF8	鍜
U+935C.0	fCNS11643	2-6023
U+935C.0	fBig5	EED5
U+935C.0	fGB	E-424C
U+935C.0	fR/S	 167.9
U+935C.0	fTotalStrokes	 17
U+935C.0	fCangjie	CRSE
U+935C.0	fFourCorner	87147
U+935C.0	fHakka	 ha2
U+935C.0	fCantonese	haa4; dyun3
U+935C.0	fMandarin	xia1; xia2
U+935D.0	fUTF8	鍝
U+935D.0	fCNS11643	3-565D
U+935D.0	fGB	3-765E
U+935D.0	fR/S	 167.9
U+935D.0	fTotalStrokes	 17
U+935D.0	fCangjie	CWLB
U+935D.0	fFourCorner	86120
U+935D.0	fHakka	 ngi2
U+935D.0	fCantonese	jyu4
U+935D.0	fMandarin	yu2
U+935D.0	fEnglish	awl
U+935E.0	fUTF8	鍞
U+935E.0	fCNS11643	2-6036
U+935E.0	fBig5	EEE8
U+935E.0	fGB	E-424D
U+935E.0	fR/S	 167.9
U+935E.0	fTotalStrokes	 17
U+935E.0	fCangjie	CYBC
U+935E.0	fFourCorner	81186
U+935E.0	fCantonese	hang1
U+935E.0	fMandarin	keng1
U+935F.0	fUTF8	鍟
U+935F.0	fCNS11643	3-5667
U+935F.0	fGB	E-424E
U+935F.0	fR/S	 167.9
U+935F.0	fTotalStrokes	 17
U+935F.0	fCangjie	CAHM
U+935F.0	fFourCorner	86110
U+935F.0	fHakka	 sen1
U+935F.0	fCantonese	seng3
U+935F.0	fMandarin	xing1
U+935F.0	fEnglish	rust
U+9360.0	fUTF8	鍠
U+9360.0	fCNS11643	2-6028
U+9360.0	fBig5	EEDA
U+9360.0	fGB	3-7661
U+9360.0	fR/S	 167.9
U+9360.0	fTotalStrokes	 17
U+9360.0	fCangjie	CHAG
U+9360.0	fFourCorner	86114
U+9360.0	fHakka	  fong2
U+9360.0	fCantonese	wong4; caang4
U+9360.0	fMandarin	huang2; heng1; heng2
U+9360.0	fEnglish	weapon
U+9361.0	fUTF8	鍡
U+9361.0	fCNS11643	2-6035
U+9361.0	fBig5	EEE7
U+9361.0	fGB	E-424F
U+9361.0	fR/S	 167.9
U+9361.0	fTotalStrokes	 17
U+9361.0	fCangjie	CWMV
U+9361.0	fFourCorner	86132
U+9361.0	fCantonese	wui2
U+9361.0	fMandarin	wei3
U+9362.0	fUTF8	鍢
U+9362.0	fCNS11643	3-5664
U+9362.0	fGB	E-4250
U+9362.0	fR/S	 167.9
U+9362.0	fTotalStrokes	 17
U+9362.0	fCangjie	CMRW
U+9362.0	fFourCorner	81160
U+9362.0	fCantonese	fuk1
U+9362.0	fMandarin	fu4
U+9363.0	fUTF8	鍣
U+9363.0	fCNS11643	2-6037
U+9363.0	fBig5	EEE9
U+9363.0	fGB	E-4251
U+9363.0	fR/S	 167.9
U+9363.0	fTotalStrokes	 17
U+9363.0	fCangjie	CTSR
U+9363.0	fFourCorner	84162
U+9363.0	fCantonese	ziu1; ciu1
U+9363.0	fMandarin	zhao1; diao1; tiao2
U+9364.0	fUTF8	鍤
U+9364.0	fCNS11643	2-5F7C
U+9364.0	fBig5	EED0
U+9364.0	fGB	1-6F4A
U+9364.0	fR/S	 167.9
U+9364.0	fTotalStrokes	 17
U+9364.0	fCangjie	CHJX
U+9364.0	fFourCorner	81177
U+9364.0	fHakka	  cap7
U+9364.0	fCantonese	caap3
U+9364.0	fMandarin	cha2; cha1
U+9364.0	fEnglish	 spade; shovel marking pin; hemm
U+9365.0	fUTF8	鍥
U+9365.0	fCNS11643	1-7571
U+9365.0	fBig5	C1E6
U+9365.0	fGB	1-6F46
U+9365.0	fR/S	 167.9
U+9365.0	fTotalStrokes	 17
U+9365.0	fCangjie	CQHK
U+9365.0	fFourCorner	87134
U+9365.0	fHakka	 kiet7; ket7
U+9365.0	fCantonese	kit3
U+9365.0	fMandarin	qie4; jie2; qi4
U+9365.0	fEnglish	 carve; engrave
U+9366.0	fUTF8	鍦
U+9366.0	fCNS11643	3-565B
U+9366.0	fGB	3-7667
U+9366.0	fR/S	 167.9
U+9366.0	fTotalStrokes	 17
U+9366.0	fCangjie	CYSD
U+9366.0	fFourCorner	88110
U+9366.0	fCantonese	se4; si1 (also)
U+9366.0	fMandarin	she2; shi1
U+9367.0	fUTF8	鍧
U+9367.0	fCNS11643	2-6038
U+9367.0	fBig5	EEEA
U+9367.0	fGB	3-7665
U+9367.0	fR/S	 167.9
U+9367.0	fTotalStrokes	 17
U+9367.0	fCangjie	CPYR
U+9367.0	fFourCorner	87120
U+9367.0	fHakka	 wang2; vang2; wang1
U+9367.0	fCantonese	gwang1; wang4
U+9367.0	fMandarin	hong1
U+9367.0	fEnglish	sound; noise
U+9368.0	fUTF8	鍨
U+9368.0	fCNS11643	3-5668
U+9368.0	fGB	E-4252
U+9368.0	fR/S	 167.9
U+9368.0	fTotalStrokes	 17
U+9368.0	fCangjie	CNOK
U+9368.0	fFourCorner	82130
U+9368.0	fCantonese	kwai4
U+9368.0	fMandarin	kui2
U+9369.0	fUTF8	鍩
U+9369.0	fCNS11643	3-565E
U+9369.0	fGB	1-6F3B
U+9369.0	fR/S	 167.9
U+9369.0	fTotalStrokes	 17
U+9369.0	fCangjie	CTKR
U+9369.0	fFourCorner	84160
U+9369.0	fCantonese	nok6; lok6 (also); zok6
U+9369.0	fMandarin	tian3; nuo4
U+9369.0	fEnglish	nobelium
U+936A.0	fUTF8	鍪
U+936A.0	fCNS11643	2-602C
U+936A.0	fBig5	EEDE
U+936A.0	fGB	0-764A
U+936A.0	fR/S	 167.9
U+936A.0	fTotalStrokes	 17
U+936A.0	fCangjie	NKC
U+936A.0	fFourCorner	18109
U+936A.0	fHakka	 meu2; meu3
U+936A.0	fCantonese	mau4
U+936A.0	fMandarin	mou2; mou4
U+936A.0	fEnglish	[1] a kind of cooking pot; iron pan [2] a kind of etal helmet; a metal cap
U+936B.0	fUTF8	鍫
U+936B.0	fCNS11643	3-566B
U+936B.0	fGB	E-4253
U+936B.0	fR/S	 167.9
U+936B.0	fTotalStrokes	 17
U+936B.0	fCangjie	HFC
U+936B.0	fFourCorner	29100
U+936B.0	fHakka	 ciau1; ts'io1; ch'io1; cio1
U+936B.0	fCantonese	ciu1; ciu4
U+936B.0	fMandarin	qiao1
U+936B.0	fEnglish	 shovel; spade
U+936C.0	fUTF8	鍬
U+936C.0	fCNS11643	1-7575
U+936C.0	fBig5	C1EA
U+936C.0	fGB	1-4742
U+936C.0	fR/S	 167.9
U+936C.0	fTotalStrokes	 17
U+936C.0	fCangjie	CHDF
U+936C.0	fFourCorner	89180
U+936C.0	fHakka	 ciau1; ts'io1; cio1
U+936C.0	fCantonese	ciu1
U+936C.0	fMandarin	qiao1
U+936C.0	fEnglish	 shovel; spade
U+936D.0	fUTF8	鍭
U+936D.0	fCNS11643	2-6029
U+936D.0	fBig5	EEDB
U+936D.0	fGB	3-7663
U+936D.0	fR/S	 167.9
U+936D.0	fTotalStrokes	 17
U+936D.0	fCangjie	CONK
U+936D.0	fFourCorner	87134
U+936D.0	fHakka	  heu2; hiu2; hiu3
U+936D.0	fCantonese	hau4
U+936D.0	fMandarin	hou2; hou4
U+936D.0	fEnglish	arrow
U+936E.0	fUTF8	鍮
U+936E.0	fCNS11643	3-5661
U+936E.0	fGB	3-7664
U+936E.0	fR/S	 167.9
U+936E.0	fTotalStrokes	 17
U+936E.0	fCangjie	COMN
U+936E.0	fFourCorner	88120
U+936E.0	fHakka	  teu1
U+936E.0	fCantonese	tau1
U+936E.0	fMandarin	tou1; zhen1
U+936E.0	fEnglish	brass
U+936F.0	fUTF8	鍯
U+936F.0	fCNS11643	4-5949
U+936F.0	fGB	E-4254
U+936F.0	fR/S	 167.9
U+936F.0	fTotalStrokes	 17; 16
U+936F.0	fCangjie	CPKP
U+936F.0	fFourCorner	87130
U+936F.0	fCantonese	cung1
U+936F.0	fMandarin	cong1; zong3; obs
U+9370.0	fUTF8	鍰
U+9370.0	fCNS11643	1-7577
U+9370.0	fBig5	C1EC
U+9370.0	fGB	1-6F4C
U+9370.0	fR/S	 167.9
U+9370.0	fTotalStrokes	 17
U+9370.0	fCangjie	CBME
U+9370.0	fFourCorner	82147
U+9370.0	fHakka	 fan2
U+9370.0	fCantonese	waan4; wun4 (also)
U+9370.0	fMandarin	huan2; huan4; yung4
U+9370.0	fEnglish	[1] an ancient unit of wieght [2] money; coins; cash
U+9371.0	fUTF8	鍱
U+9371.0	fCNS11643	2-6032
U+9371.0	fBig5	EEE4
U+9371.0	fGB	3-765B
U+9371.0	fR/S	 167.9
U+9371.0	fTotalStrokes	 17
U+9371.0	fCangjie	CPTD
U+9371.0	fFourCorner	84194
U+9371.0	fHakka	 jap8; rap8
U+9371.0	fCantonese	jip6; dip6
U+9371.0	fMandarin	ye4; die2; she4; zui4
U+9371.0	fEnglish	thin plates of metal
U+9372.0	fUTF8	鍲
U+9372.0	fCNS11643	3-5662
U+9372.0	fGB	5-756D
U+9372.0	fR/S	 167.9
U+9372.0	fTotalStrokes	 17
U+9372.0	fCangjie	CRPA
U+9372.0	fFourCorner	87160
U+9372.0	fCantonese	man4
U+9372.0	fMandarin	min2
U+9373.0	fUTF8	鍳
U+9373.0	fCNS11643	3-5663
U+9373.0	fGB	E-4255
U+9373.0	fR/S	 167.9; 167.10
U+9373.0	fTotalStrokes	 17
U+9373.0	fCangjie	SIC
U+9373.0	fFourCorner	78100
U+9373.0	fCantonese	gaam3
U+9373.0	fMandarin	jian4
U+9374.0	fUTF8	鍴
U+9374.0	fCNS11643	3-565F
U+9374.0	fGB	3-765F
U+9374.0	fR/S	 167.9
U+9374.0	fTotalStrokes	 17
U+9374.0	fCangjie	CUMB
U+9374.0	fFourCorner	82120
U+9374.0	fCantonese	dyun1
U+9374.0	fMandarin	duan1
U+9375.0	fUTF8	鍵
U+9375.0	fCNS11643	1-756F
U+9375.0	fBig5	C1E4
U+9375.0	fGB	1-3C7C
U+9375.0	fR/S	 167.9
U+9375.0	fTotalStrokes	 17
U+9375.0	fCangjie	CNKQ
U+9375.0	fFourCorner	85140
U+9375.0	fHakka	 ken5; kien6; kien5; ken3; kian3
U+9375.0	fCantonese	gin6
U+9375.0	fMandarin	jian4; jian3; qian2
U+9375.0	fEnglish	 key (to a door or an musical instrument); key to a situation
U+9376.0	fUTF8	鍶
U+9376.0	fCNS11643	2-6024
U+9376.0	fBig5	EED6
U+9376.0	fGB	1-6F48
U+9376.0	fR/S	 167.9
U+9376.0	fTotalStrokes	 17
U+9376.0	fCangjie	CWP
U+9376.0	fFourCorner	86130
U+9376.0	fCantonese	si1
U+9376.0	fMandarin	si1; song1
U+9376.0	fEnglish	strontium
U+9377.0	fUTF8	鍷
U+9377.0	fCNS11643	2-6033
U+9377.0	fBig5	EEE5
U+9377.0	fGB	E-4256
U+9377.0	fR/S	 167.9
U+9377.0	fTotalStrokes	 17
U+9377.0	fCangjie	CKGG
U+9377.0	fFourCorner	84114
U+9377.0	fCantonese	fui1; kwai1
U+9377.0	fMandarin	kui1
U+9378.0	fUTF8	鍸
U+9378.0	fCNS11643	3-565C
U+9378.0	fGB	E-4257
U+9378.0	fR/S	 167.9
U+9378.0	fTotalStrokes	 17
U+9378.0	fCangjie	CJRB
U+9378.0	fFourCorner	87120
U+9378.0	fCantonese	wu4
U+9378.0	fMandarin	hu2
U+9379.0	fUTF8	鍹
U+9379.0	fCNS11643	2-602D
U+9379.0	fBig5	EEDF
U+9379.0	fGB	5-756B
U+9379.0	fR/S	 167.9
U+9379.0	fTotalStrokes	 17
U+9379.0	fCangjie	CJMM
U+9379.0	fFourCorner	83116
U+9379.0	fCantonese	syun1
U+9379.0	fMandarin	xuan1
U+937A.0	fUTF8	鍺
U+937A.0	fCNS11643	2-5B31
U+937A.0	fBig5	EBE3
U+937A.0	fGB	1-5560
U+937A.0	fR/S	 167.9
U+937A.0	fTotalStrokes	 17
U+937A.0	fCangjie	CJKA
U+937A.0	fFourCorner	84160
U+937A.0	fHakka	 za3
U+937A.0	fCantonese	ze2
U+937A.0	fMandarin	zhe3; duo3; zang1; du3
U+937A.0	fEnglish	germanium
U+937B.0	fUTF8	鍻
U+937B.0	fCNS11643	2-6034
U+937B.0	fBig5	EEE6
U+937B.0	fGB	E-4258
U+937B.0	fR/S	 167.9
U+937B.0	fTotalStrokes	 17
U+937B.0	fCangjie	CAPV
U+937B.0	fFourCorner	86127
U+937B.0	fCantonese	git3
U+937B.0	fMandarin	jie2
U+937C.0	fUTF8	鍼
U+937C.0	fCNS11643	2-6021
U+937C.0	fBig5	EED3
U+937C.0	fGB	E-4259
U+937C.0	fR/S	 167.9
U+937C.0	fTotalStrokes	 17
U+937C.0	fCangjie	CIHR
U+937C.0	fFourCorner	83150
U+937C.0	fHakka	  zhim1; zim1; kiam2; zihm1
U+937C.0	fCantonese	kim4; zam1
U+937C.0	fMandarin	zhen1; qian2; huang2; jian1
U+937C.0	fEnglish	 needle; pin; probe
U+937D.0	fUTF8	鍽
U+937D.0	fCNS11643	3-5660
U+937D.0	fGB	E-425A
U+937D.0	fR/S	 167.9
U+937D.0	fTotalStrokes	 17
U+937D.0	fCangjie	CHSB
U+937D.0	fFourCorner	83127
U+937D.0	fCantonese	bin1
U+937D.0	fMandarin	bian1
U+937E.0	fUTF8	鍾
U+937E.0	fCNS11643	1-7574
U+937E.0	fBig5	C1E9
U+937E.0	fGB	1-6F71
U+937E.0	fR/S	 167.9
U+937E.0	fTotalStrokes	 17
U+937E.0	fCangjie	CHJG
U+937E.0	fFourCorner	82114
U+937E.0	fHakka	  zhung1; zung1
U+937E.0	fCantonese	zung1
U+937E.0	fMandarin	zhong1; zhong4
U+937E.0	fEnglish	[1] a kind of wine container [2] concentrate; accumulate [3] a Chinese family name
U+937F.0	fUTF8	鍿
U+937F.0	fCNS11643	3-566A
U+937F.0	fGB	E-425B
U+937F.0	fR/S	 167.9
U+937F.0	fTotalStrokes	 17
U+937F.0	fCangjie	CVMW
U+937F.0	fFourCorner	82160
U+937F.0	fCantonese	zi1
U+937F.0	fMandarin	zi1
U+9380.0	fUTF8	鎀
U+9380.0	fCNS11643	2-642F
U+9380.0	fBig5	EEEB
U+9380.0	fGB	E-425C
U+9380.0	fR/S	 167.9
U+9380.0	fTotalStrokes	 17
U+9380.0	fCangjie	COLH
U+9380.0	fFourCorner	87122
U+9380.0	fCantonese	sau1
U+9380.0	fMandarin	xiu1
U+9381.0	fUTF8	鎁
U+9381.0	fCNS11643	4-5947
U+9381.0	fGB	5-755F
U+9381.0	fR/S	 167.9
U+9381.0	fTotalStrokes	 17; 16
U+9381.0	fCangjie	CSJL
U+9381.0	fFourCorner	87120
U+9381.0	fCantonese	je4
U+9381.0	fMandarin	ye2
U+9382.0	fUTF8	鎂
U+9382.0	fCNS11643	1-756D
U+9382.0	fBig5	C1E2
U+9382.0	fGB	1-433E
U+9382.0	fR/S	 167.9
U+9382.0	fTotalStrokes	 17
U+9382.0	fCangjie	CTGK
U+9382.0	fFourCorner	88135
U+9382.0	fHakka	 mui1
U+9382.0	fCantonese	mei5
U+9382.0	fMandarin	mei3
U+9382.0	fEnglish	magensium
U+9383.0	fUTF8	鎃
U+9383.0	fCNS11643	2-5F7A
U+9383.0	fBig5	EECE
U+9383.0	fGB	E-425D
U+9383.0	fR/S	 167.9
U+9383.0	fTotalStrokes	 17
U+9383.0	fCangjie	CEHV
U+9383.0	fFourCorner	82132
U+9383.0	fCantonese	paai3
U+9383.0	fMandarin	pai4
U+9383.0	fEnglish	protoactinium
U+9384.0	fUTF8	鎄
U+9384.0	fCNS11643	4-5946
U+9384.0	fGB	1-6F4D
U+9384.0	fR/S	 167.9
U+9384.0	fTotalStrokes	 17; 16
U+9384.0	fCangjie	CYRV
U+9384.0	fFourCorner	80130
U+9384.0	fCantonese	oi1; ngoi1 (also)
U+9384.0	fMandarin	ai1; yi3
U+9384.0	fEnglish	iridium einsteinium
U+9385.0	fUTF8	鎅
U+9385.0	fCNS11643	F-5D41
U+9385.0	fGB	1-7C55
U+9385.0	fR/S	 167.9
U+9385.0	fTotalStrokes	 17
U+9385.0	fCangjie	CWOL
U+9385.0	fCantonese	gaai3; gaai
U+9385.0	fMandarin	jie4
U+9385.0	fEnglish	 saw; cut
U+9386.0	fUTF8	鎆
U+9386.0	fCNS11643	3-665F
U+9386.0	fGB	E-425E
U+9386.0	fR/S	 167.9
U+9386.0	fTotalStrokes	 17
U+9386.0	fCangjie	CTBN
U+9386.0	fFourCorner	8812
U+9386.0	fCantonese	zin2
U+9387.0	fUTF8	鎇
U+9387.0	fCNS11643	F-5D39
U+9387.0	fGB	1-6F51
U+9387.0	fR/S	 167.9
U+9387.0	fTotalStrokes	 17
U+9387.0	fCangjie	CAHU
U+9387.0	fFourCorner	8716
U+9387.0	fCantonese	mei4
U+9387.0	fMandarin	mei2
U+9387.0	fEnglish	americium
U+9388.0	fUTF8	鎈
U+9388.0	fCNS11643	2-642C
U+9388.0	fBig5	F160
U+9388.0	fGB	3-7668
U+9388.0	fR/S	 167.10
U+9388.0	fTotalStrokes	 18
U+9388.0	fCangjie	CTQM
U+9388.0	fFourCorner	88112
U+9388.0	fCantonese	so2; caa1
U+9388.0	fMandarin	chuo1; cha1
U+9389.0	fUTF8	鎉
U+9389.0	fCNS11643	2-6425
U+9389.0	fBig5	F159
U+9389.0	fGB	5-7576
U+9389.0	fR/S	 167.10
U+9389.0	fTotalStrokes	 18
U+9389.0	fCangjie	CASM
U+9389.0	fFourCorner	86127
U+9389.0	fCantonese	taap3
U+9389.0	fMandarin	ta4; da2
U+9389.0	fEnglish	thallium
U+938A.0	fUTF8	鎊
U+938A.0	fCNS11643	1-7754
U+938A.0	fBig5	C2E9
U+938A.0	fGB	1-3077
U+938A.0	fR/S	 167.10
U+938A.0	fTotalStrokes	 18
U+938A.0	fCangjie	CYBS
U+938A.0	fFourCorner	80127
U+938A.0	fHakka	 pong2; bong5; pong5
U+938A.0	fCantonese	bong6; bong2 (also)
U+938A.0	fMandarin	bang4; pang1
U+938A.0	fEnglish	pound sterling
U+938B.0	fUTF8	鎋
U+938B.0	fCNS11643	3-5943
U+938B.0	fGB	5-757C
U+938B.0	fR/S	  167.10
U+938B.0	fTotalStrokes	 18
U+938B.0	fCangjie	CJQR
U+938B.0	fFourCorner	83160
U+938B.0	fHakka	 hat7; hot7
U+938B.0	fCantonese	hat6
U+938B.0	fMandarin	xia2
U+938B.0	fEnglish	 linchpin
U+938C.0	fUTF8	鎌
U+938C.0	fCNS11643	2-637E
U+938C.0	fBig5	F154
U+938C.0	fGB	E-425F
U+938C.0	fR/S	 167.10
U+938C.0	fTotalStrokes	 18
U+938C.0	fCangjie	CTXC
U+938C.0	fFourCorner	88137
U+938C.0	fHakka	  liam2; liam3
U+938C.0	fCantonese	lim4
U+938C.0	fMandarin	lian2
U+938C.0	fEnglish	 sickle
U+938D.0	fUTF8	鎍
U+938D.0	fCNS11643	2-6430
U+938D.0	fBig5	F163
U+938D.0	fGB	5-7573
U+938D.0	fR/S	 167.10
U+938D.0	fTotalStrokes	 18
U+938D.0	fCangjie	CJBF
U+938D.0	fFourCorner	84193
U+938D.0	fHakka	  sok7
U+938D.0	fCantonese	sok3
U+938D.0	fMandarin	suo3
U+938D.0	fEnglish	 chain wire
U+938E.0	fUTF8	鎎
U+938E.0	fCNS11643	2-6427
U+938E.0	fBig5	F15B
U+938E.0	fGB	E-4260
U+938E.0	fR/S	 167.10
U+938E.0	fTotalStrokes	 18
U+938E.0	fCangjie	COND
U+938E.0	fFourCorner	88117
U+938E.0	fCantonese	hei3
U+938E.0	fMandarin	xi4; kai4
U+938F.0	fUTF8	鎏
U+938F.0	fCNS11643	2-602A
U+938F.0	fBig5	EEDC
U+938F.0	fGB	0-764C
U+938F.0	fR/S	 167.10
U+938F.0	fTotalStrokes	 18
U+938F.0	fCangjie	EUC
U+938F.0	fFourCorner	30109
U+938F.0	fHakka	 liu2
U+938F.0	fCantonese	lau4
U+938F.0	fMandarin	liu2
U+938F.0	fEnglish	pure gold
U+9390.0	fUTF8	鎐
U+9390.0	fCNS11643	3-5940
U+9390.0	fGB	3-7671
U+9390.0	fR/S	 167.10
U+9390.0	fTotalStrokes	 18
U+9390.0	fCangjie	CBOU
U+9390.0	fFourCorner	87170
U+9390.0	fCantonese	jiu4
U+9390.0	fMandarin	zu2; yao2
U+9390.0	fEnglish	 family name wine cup
U+9391.0	fUTF8	鎑
U+9391.0	fCNS11643	2-6432
U+9391.0	fBig5	F165
U+9391.0	fGB	3-766A
U+9391.0	fR/S	 167.10
U+9391.0	fTotalStrokes	 18
U+9391.0	fCangjie	CGIT
U+9391.0	fFourCorner	84117
U+9391.0	fCantonese	jip6; taap3
U+9391.0	fMandarin	ye4
U+9392.0	fUTF8	鎒
U+9392.0	fCNS11643	2-6421
U+9392.0	fBig5	F155
U+9392.0	fGB	5-7574
U+9392.0	fR/S	 167.10
U+9392.0	fTotalStrokes	 18
U+9392.0	fCangjie	CMVI
U+9392.0	fFourCorner	81143
U+9392.0	fHakka	 ngieu3
U+9392.0	fCantonese	nau6; lau6 (also)
U+9392.0	fMandarin	nou4; hao1
U+9392.0	fEnglish	 hoe weed
U+9393.0	fUTF8	鎓
U+9393.0	fCNS11643	3-5942
U+9393.0	fGB	3-7672
U+9393.0	fR/S	  167.10
U+9393.0	fTotalStrokes	 18
U+9393.0	fCangjie	CCIM
U+9393.0	fFourCorner	88120
U+9393.0	fCantonese	jung1
U+9393.0	fMandarin	weng1
U+9394.0	fUTF8	鎔
U+9394.0	fCNS11643	1-7753
U+9394.0	fBig5	C2E8
U+9394.0	fGB	E-4261
U+9394.0	fR/S	 167.10
U+9394.0	fTotalStrokes	 18
U+9394.0	fCangjie	CJCR
U+9394.0	fFourCorner	83168
U+9394.0	fHakka	  jung2; rung2; jung3
U+9394.0	fCantonese	jung4
U+9394.0	fMandarin	rong2; tian2; zhen4
U+9394.0	fEnglish	[1] fuse; melt; smelt [2] mold [3] a kind of spear
U+9395.0	fUTF8	鎕
U+9395.0	fCNS11643	2-642B
U+9395.0	fBig5	F15F
U+9395.0	fGB	5-757B
U+9395.0	fR/S	 167.10
U+9395.0	fTotalStrokes	 18
U+9395.0	fCangjie	CILR
U+9395.0	fFourCorner	80167
U+9395.0	fCantonese	tong4
U+9395.0	fMandarin	tang2
U+9396.0	fUTF8	鎖
U+9396.0	fCNS11643	1-7755
U+9396.0	fBig5	C2EA
U+9396.0	fGB	1-4B78
U+9396.0	fR/S	 167.10
U+9396.0	fTotalStrokes	 18
U+9396.0	fCangjie	CFBC
U+9396.0	fFourCorner	89186
U+9396.0	fHakka	  so3
U+9396.0	fCantonese	so2
U+9396.0	fMandarin	suo3
U+9396.0	fEnglish	[1] lock; padlock [2] fetters; shackles; chains [3] lock [4] confine [5] lockstitch
U+9397.0	fUTF8	鎗
U+9397.0	fCNS11643	1-775D
U+9397.0	fBig5	C2F2
U+9397.0	fGB	E-4262
U+9397.0	fR/S	 167.10
U+9397.0	fTotalStrokes	 18
U+9397.0	fCangjie	COIR
U+9397.0	fFourCorner	88167
U+9397.0	fHakka	  ciong1; ziong1
U+9397.0	fCantonese	coeng1; caang1
U+9397.0	fMandarin	qiang1; cheng1; qiang4
U+9397.0	fEnglish	 fire arms; small arms; hand gun; rifle; pistol
U+9398.0	fUTF8	鎘
U+9398.0	fCNS11643	1-775B
U+9398.0	fBig5	C2F0
U+9398.0	fGB	1-6F53
U+9398.0	fR/S	 167.10
U+9398.0	fTotalStrokes	 18
U+9398.0	fCangjie	CMRB
U+9398.0	fFourCorner	81127
U+9398.0	fHakka	 gak7
U+9398.0	fCantonese	gaak3; lik6
U+9398.0	fMandarin	ge2; li4
U+9398.0	fEnglish	cadmium
U+9399.0	fUTF8	鎙
U+9399.0	fCNS11643	2-642D
U+9399.0	fBig5	F161
U+9399.0	fGB	E-4263
U+9399.0	fR/S	 167.10
U+9399.0	fTotalStrokes	 18
U+9399.0	fCangjie	CTUB
U+9399.0	fFourCorner	87120
U+9399.0	fHakka	  sok7
U+9399.0	fCantonese	sok3
U+9399.0	fMandarin	shuo4
U+939A.0	fUTF8	鎚
U+939A.0	fCNS11643	1-775C
U+939A.0	fBig5	C2F1
U+939A.0	fGB	E-4264
U+939A.0	fR/S	 167.10
U+939A.0	fTotalStrokes	 18
U+939A.0	fCangjie	CYHR
U+939A.0	fFourCorner	87137
U+939A.0	fHakka	 cui2; ch'ui2; ch'ui3; chui2
U+939A.0	fCantonese	ceoi4; ceoi2 (also)
U+939A.0	fMandarin	chui2; dui1; zhui4
U+939A.0	fEnglish	[1] hammer [2] hammer; mallet [3] an ancient weapon
U+939B.0	fUTF8	鎛
U+939B.0	fCNS11643	2-6423
U+939B.0	fBig5	F157
U+939B.0	fGB	3-766B
U+939B.0	fR/S	 167.10
U+939B.0	fTotalStrokes	 18
U+939B.0	fCangjie	CIBI
U+939B.0	fFourCorner	83142
U+939B.0	fHakka	  bok7
U+939B.0	fCantonese	bok3
U+939B.0	fMandarin	bo2; yong1
U+939B.0	fEnglish	[1] a kind of ancient bell [2] a variety of hoe
U+939C.0	fUTF8	鎜
U+939C.0	fCNS11643	3-5949
U+939C.0	fGB	E-4265
U+939C.0	fR/S	 167.10
U+939C.0	fTotalStrokes	 18; 17
U+939C.0	fCangjie	HEC
U+939C.0	fFourCorner	27100
U+939C.0	fCantonese	pun4
U+939C.0	fMandarin	pan2
U+939D.0	fUTF8	鎝
U+939D.0	fCNS11643	2-6424
U+939D.0	fBig5	F158
U+939D.0	fGB	3-765C
U+939D.0	fR/S	 167.10
U+939D.0	fTotalStrokes	 18
U+939D.0	fCangjie	CTOR
U+939D.0	fFourCorner	84161
U+939D.0	fHakka	 dap7; sap7
U+939D.0	fCantonese	saap3; daap3 (also)
U+939D.0	fMandarin	sa4; ta3; da2; da1
U+939D.0	fEnglish	tc
U+939E.0	fUTF8	鎞
U+939E.0	fCNS11643	2-6429
U+939E.0	fBig5	F15D
U+939E.0	fGB	3-7670
U+939E.0	fR/S	 167.10
U+939E.0	fTotalStrokes	 18
U+939E.0	fCangjie	CHWP
U+939E.0	fFourCorner	86111
U+939E.0	fHakka	 bi1; pi2; bi5
U+939E.0	fCantonese	bai1; pei1; pai1; bei6
U+939E.0	fMandarin	bi4; bi1; pi1; pi2
U+939E.0	fEnglish	 plowshare barb; lancet
U+939F.0	fUTF8	鎟
U+939F.0	fCNS11643	2-642E
U+939F.0	fBig5	F162
U+939F.0	fGB	E-4266
U+939F.0	fR/S	 167.10
U+939F.0	fTotalStrokes	 18
U+939F.0	fCangjie	CEED
U+939F.0	fFourCorner	87194
U+939F.0	fCantonese	song2
U+939F.0	fMandarin	sang3
U+93A0.0	fUTF8	鎠
U+93A0.0	fCNS11643	3-593E
U+93A0.0	fGB	E-4267
U+93A0.0	fR/S	 167.10
U+93A0.0	fTotalStrokes	 18
U+93A0.0	fCangjie	CWLM
U+93A0.0	fFourCorner	86110
U+93A0.0	fCantonese	gong3
U+93A0.0	fMandarin	gang1
U+93A1.0	fUTF8	鎡
U+93A1.0	fCNS11643	2-5F79
U+93A1.0	fBig5	EECD
U+93A1.0	fGB	3-7669
U+93A1.0	fR/S	 167.10; 167.9
U+93A1.0	fTotalStrokes	 18
U+93A1.0	fCangjie	CTVI
U+93A1.0	fFourCorner	88132
U+93A1.0	fHakka	 zih1; zii1
U+93A1.0	fCantonese	zi1
U+93A1.0	fMandarin	zi1
U+93A1.0	fEnglish	 hoe; mattock
U+93A2.0	fUTF8	鎢
U+93A2.0	fCNS11643	1-7756
U+93A2.0	fBig5	C2EB
U+93A2.0	fGB	1-4E59
U+93A2.0	fR/S	 167.10
U+93A2.0	fTotalStrokes	 18
U+93A2.0	fCangjie	CHRF
U+93A2.0	fFourCorner	87127
U+93A2.0	fHakka	 wu1
U+93A2.0	fCantonese	wu1
U+93A2.0	fMandarin	wu1; wu4
U+93A2.0	fEnglish	tungsten; wolfram
U+93A3.0	fUTF8	鎣
U+93A3.0	fCNS11643	2-6437
U+93A3.0	fBig5	F16A
U+93A3.0	fGB	1-5D76
U+93A3.0	fR/S	 167.10
U+93A3.0	fTotalStrokes	 18
U+93A3.0	fCangjie	FFBC
U+93A3.0	fFourCorner	99109
U+93A3.0	fCantonese	jing4
U+93A3.0	fMandarin	ying4; ying2; jyong3; jiong3; xing2; ying1
U+93A3.0	fEnglish	polish
U+93A4.0	fUTF8	鎤
U+93A4.0	fCNS11643	2-6434
U+93A4.0	fBig5	F167
U+93A4.0	fGB	5-7575
U+93A4.0	fR/S	 167.10
U+93A4.0	fTotalStrokes	 18
U+93A4.0	fCangjie	CAFU
U+93A4.0	fFourCorner	86111
U+93A4.0	fHakka	  fong3
U+93A4.0	fCantonese	fong2
U+93A4.0	fMandarin	huang3
U+93A4.0	fEnglish	 sound of a bell small bell
U+93A5.0	fUTF8	鎥
U+93A5.0	fCNS11643	2-6761
U+93A5.0	fBig5	F16B
U+93A5.0	fGB	E-4268
U+93A5.0	fR/S	 167.10
U+93A5.0	fTotalStrokes	 18
U+93A5.0	fCangjie	ODC
U+93A5.0	fFourCorner	27109
U+93A5.0	fCantonese	tiu4
U+93A5.0	fMandarin	tiao2
U+93A6.0	fUTF8	鎦
U+93A6.0	fCNS11643	2-642A
U+93A6.0	fBig5	F15E
U+93A6.0	fGB	1-6F56
U+93A6.0	fR/S	 167.10
U+93A6.0	fTotalStrokes	 18
U+93A6.0	fCangjie	CHHW
U+93A6.0	fFourCorner	82163
U+93A6.0	fHakka	 liu2
U+93A6.0	fCantonese	lau4; lau6; lau5
U+93A6.0	fMandarin	liu2; liu4
U+93A6.0	fEnglish	 distil lutecium a Chinese family name
U+93A7.0	fUTF8	鎧
U+93A7.0	fCNS11643	2-6426
U+93A7.0	fBig5	F15A
U+93A7.0	fGB	1-6E78
U+93A7.0	fR/S	 167.10
U+93A7.0	fTotalStrokes	 18
U+93A7.0	fCangjie	CUMT
U+93A7.0	fFourCorner	82118
U+93A7.0	fHakka	  koi3; kai3
U+93A7.0	fCantonese	hoi2; koi3
U+93A7.0	fMandarin	kai3; kai4
U+93A7.0	fEnglish	armor (worn by a warrior)
U+93A8.0	fUTF8	鎨
U+93A8.0	fCNS11643	2-6435
U+93A8.0	fBig5	F168
U+93A8.0	fGB	5-7578
U+93A8.0	fR/S	 167.10
U+93A8.0	fTotalStrokes	 18
U+93A8.0	fCangjie	COGJ
U+93A8.0	fFourCorner	80141
U+93A8.0	fCantonese	seon2
U+93A8.0	fMandarin	sun3
U+93A9.0	fUTF8	鎩
U+93A9.0	fCNS11643	2-6756
U+93A9.0	fBig5	F36A
U+93A9.0	fGB	1-6F21
U+93A9.0	fR/S	 167.11; 167.10
U+93A9.0	fTotalStrokes	 19
U+93A9.0	fCangjie	CKCE
U+93A9.0	fFourCorner	87147
U+93A9.0	fHakka	  sai5; sat7
U+93A9.0	fCantonese	saat3
U+93A9.0	fMandarin	sha1; se4; shi4; shai4; she4
U+93A9.0	fEnglish	[1] lance [2] shed (feathers)
U+93AA.0	fUTF8	鎪
U+93AA.0	fCNS11643	2-6428
U+93AA.0	fBig5	F15C
U+93AA.0	fGB	1-6F4B
U+93AA.0	fR/S	 167.10
U+93AA.0	fTotalStrokes	 18
U+93AA.0	fCangjie	CHXE
U+93AA.0	fFourCorner	87147
U+93AA.0	fHakka	  seu1
U+93AA.0	fCantonese	sau1; sau2 (also); suk1
U+93AA.0	fMandarin	sou1
U+93AA.0	fEnglish	 engrave (metal or wood)
U+93AB.0	fUTF8	鎫
U+93AB.0	fCNS11643	4-5E31
U+93AB.0	fGB	E-4269
U+93AB.0	fR/S	 167.10
U+93AB.0	fTotalStrokes	 18
U+93AB.0	fCangjie	CHBE
U+93AB.0	fFourCorner	87140
U+93AB.0	fCantonese	faan6
U+93AB.0	fMandarin	wan3; wan4; jian3
U+93AB.0	fEnglish	ornment over the horse's head
U+93AC.0	fUTF8	鎬
U+93AC.0	fCNS11643	1-7759
U+93AC.0	fBig5	C2EE
U+93AC.0	fGB	1-3864
U+93AC.0	fR/S	 167.10
U+93AC.0	fTotalStrokes	 18
U+93AC.0	fCangjie	CYRB
U+93AC.0	fFourCorner	80127
U+93AC.0	fHakka	 hau3; hau1; hau5
U+93AC.0	fCantonese	gou2; hou6
U+93AC.0	fMandarin	hao4; yi4
U+93AC.0	fEnglish	[1] bright [2] an ancient place name (the first capital of the Chou dynasty)
U+93AC.1	fUTF8	鎬
U+93AC.1	fCNS11643	1-7759
U+93AC.1	fBig5	C2EE
U+93AC.1	fGB	1-3864
U+93AC.1	fR/S	 167.10
U+93AC.1	fTotalStrokes	 18
U+93AC.1	fCangjie	CYRB
U+93AC.1	fFourCorner	80127
U+93AC.1	fMandarin	gao3
U+93AC.1	fEnglish	a kind of hoe
U+93AD.0	fUTF8	鎭
U+93AD.0	fCNS11643	3-5944
U+93AD.0	fGB	E-426A
U+93AD.0	fR/S	 167.10
U+93AD.0	fTotalStrokes	 18
U+93AD.0	fCangjie	CPBC
U+93AD.0	fFourCorner	81180
U+93AD.0	fHakka	  din3;  zhin3; zihn3; zin3
U+93AD.0	fCantonese	zan3
U+93AD.0	fMandarin	zhen4; hao4; zhen1; tian2
U+93AD.0	fEnglish	 town; market place suppress
U+93AE.0	fUTF8	鎮
U+93AE.0	fCNS11643	1-7758
U+93AE.0	fBig5	C2ED
U+93AE.0	fGB	1-5572
U+93AE.0	fR/S	 167.10
U+93AE.0	fTotalStrokes	 18
U+93AE.0	fCangjie	CJBC
U+93AE.0	fFourCorner	84181
U+93AE.0	fHakka	 zhin6; zin3; cin3; zen5
U+93AE.0	fCantonese	zan3
U+93AE.0	fMandarin	zhen4; tian2; zhen1
U+93AE.0	fEnglish	[1] subdue; suppress; quel; put down [2] cool with water or ice [3] weight (for pressing down) [4] whole [5] town; township
U+93AF.0	fUTF8	鎯
U+93AF.0	fCNS11643	2-5F7B
U+93AF.0	fBig5	EECF
U+93AF.0	fGB	5-7564
U+93AF.0	fR/S	 167.10
U+93AF.0	fTotalStrokes	 18
U+93AF.0	fCangjie	CIIL
U+93AF.0	fFourCorner	87127
U+93AF.0	fCantonese	long4
U+93AF.0	fMandarin	luo3; lang2
U+93B0.0	fUTF8	鎰
U+93B0.0	fCNS11643	1-775A
U+93B0.0	fBig5	C2EF
U+93B0.0	fGB	1-6F57
U+93B0.0	fR/S	 167.10
U+93B0.0	fTotalStrokes	 18
U+93B0.0	fCangjie	CTCT
U+93B0.0	fFourCorner	88117
U+93B0.0	fHakka	 jit7; rit7
U+93B0.0	fCantonese	jat6
U+93B0.0	fMandarin	yi4
U+93B0.0	fEnglish	an ancient unit of weight
U+93B1.0	fUTF8	鎱
U+93B1.0	fCNS11643	2-6431
U+93B1.0	fBig5	F164
U+93B1.0	fGB	E-426B
U+93B1.0	fR/S	 167.10
U+93B1.0	fTotalStrokes	 18
U+93B1.0	fCangjie	CGRV
U+93B1.0	fFourCorner	84132
U+93B1.0	fCantonese	jyun4
U+93B1.0	fMandarin	yuan2
U+93B2.0	fUTF8	鎲
U+93B2.0	fCNS11643	2-6433
U+93B2.0	fBig5	F166
U+93B2.0	fGB	E-426C
U+93B2.0	fR/S	 167.10
U+93B2.0	fTotalStrokes	 18
U+93B2.0	fCangjie	CFBU
U+93B2.0	fFourCorner	89116
U+93B2.0	fCantonese	tong2
U+93B2.0	fMandarin	tang3
U+93B3.0	fUTF8	鎳
U+93B3.0	fCNS11643	1-7757
U+93B3.0	fBig5	C2EC
U+93B3.0	fGB	1-4478
U+93B3.0	fR/S	 167.10
U+93B3.0	fTotalStrokes	 18
U+93B3.0	fCangjie	CHUD
U+93B3.0	fFourCorner	87194
U+93B3.0	fHakka	 ngiet7
U+93B3.0	fCantonese	nip1; nip6 (also); lip1 (also); lip6 (also)
U+93B3.0	fMandarin	nie4
U+93B3.0	fEnglish	nickel
U+93B4.0	fUTF8	鎴
U+93B4.0	fCNS11643	2-6436
U+93B4.0	fBig5	F169
U+93B4.0	fGB	E-426D
U+93B4.0	fR/S	 167.10
U+93B4.0	fTotalStrokes	 18
U+93B4.0	fCangjie	CHUP
U+93B4.0	fFourCorner	86130
U+93B4.0	fCantonese	sik1
U+93B4.0	fMandarin	xi2
U+93B5.0	fUTF8	鎵
U+93B5.0	fCNS11643	2-637D
U+93B5.0	fBig5	F153
U+93B5.0	fGB	1-6F58
U+93B5.0	fR/S	 167.10
U+93B5.0	fTotalStrokes	 18
U+93B5.0	fCangjie	CJMO
U+93B5.0	fFourCorner	83132
U+93B5.0	fCantonese	gaa1
U+93B5.0	fMandarin	jia1
U+93B5.0	fEnglish	gallium
U+93B6.0	fUTF8	鎶
U+93B6.0	fCNS11643	4-5E29
U+93B6.0	fGB	E-426E
U+93B6.0	fR/S	 167.10
U+93B6.0	fTotalStrokes	 18
U+93B6.0	fCangjie	CMRR
U+93B6.0	fFourCorner	81120
U+93B6.0	fCantonese	go1
U+93B6.0	fMandarin	ge1
U+93B7.0	fUTF8	鎷
U+93B7.0	fCNS11643	2-6422
U+93B7.0	fBig5	F156
U+93B7.0	fGB	E-426F
U+93B7.0	fR/S	 167.10
U+93B7.0	fTotalStrokes	 18
U+93B7.0	fCangjie	CSQF
U+93B7.0	fFourCorner	81127
U+93B7.0	fHakka	 ma1
U+93B7.0	fCantonese	maa5
U+93B7.0	fMandarin	ma3
U+93B7.0	fEnglish	masarium
U+93B8.0	fUTF8	鎸
U+93B8.0	fCNS11643	3-5946
U+93B8.0	fGB	1-6F54
U+93B8.0	fR/S	 167.10
U+93B8.0	fTotalStrokes	 18
U+93B8.0	fCangjie	COGS
U+93B8.0	fFourCorner	80120
U+93B8.0	fCantonese	zyun1; zeon3 (also)
U+93B8.0	fMandarin	juan1
U+93B9.0	fUTF8	鎹
U+93B9.0	fCNS11643	F-6133
U+93B9.0	fGB	E-4270
U+93B9.0	fR/S	 167.10
U+93B9.0	fTotalStrokes	 18
U+93B9.0	fCangjie	CYTK
U+93B9.0	fFourCorner	8813
U+93B9.0	fCantonese	sung3
U+93B9.0	fMandarin	rong2
U+93B9.0	fEnglish	a clamp
U+93BA.0	fUTF8	鎺
U+93BA.0	fGB	E-4271
U+93BA.0	fR/S	 167.10
U+93BA.0	fTotalStrokes	 18
U+93BA.0	fCangjie	CIFM
U+93BA.0	fCantonese	zou2
U+93BB.0	fUTF8	鎻
U+93BB.0	fCNS11643	3-5945
U+93BB.0	fGB	E-4272
U+93BB.0	fR/S	 167.10
U+93BB.0	fTotalStrokes	 18
U+93BB.0	fCangjie	CVVC
U+93BB.0	fFourCorner	82180
U+93BB.0	fCantonese	so2
U+93BB.0	fMandarin	suo3
U+93BB.0	fEnglish	 lock; padlock shackles; chains
U+93BC.0	fUTF8	鎼
U+93BC.0	fCNS11643	3-6661
U+93BC.0	fGB	E-4273
U+93BC.0	fR/S	 167.10
U+93BC.0	fTotalStrokes	 18
U+93BC.0	fCangjie	CYPQ
U+93BC.0	fFourCorner	8115
U+93BC.0	fCantonese	laa3
U+93BD.0	fUTF8	鎽
U+93BD.0	fCNS11643	3-6662
U+93BD.0	fGB	E-4274
U+93BD.0	fR/S	 167.10
U+93BD.0	fTotalStrokes	 18
U+93BD.0	fCangjie	CUHJ
U+93BD.0	fFourCorner	8215
U+93BD.0	fCantonese	fung1
U+93BE.0	fUTF8	鎾
U+93BE.0	fCNS11643	3-6660
U+93BE.0	fGB	E-4275
U+93BE.0	fR/S	 167.10
U+93BE.0	fTotalStrokes	 18
U+93BE.0	fCangjie	CABT
U+93BE.0	fFourCorner	8611
U+93BE.0	fCantonese	wan1
U+93BF.0	fUTF8	鎿
U+93BF.0	fCNS11643	F-6143
U+93BF.0	fGB	1-6F55
U+93BF.0	fR/S	 167.10
U+93BF.0	fTotalStrokes	 18
U+93BF.0	fCangjie	CORQ
U+93BF.0	fFourCorner	8815
U+93BF.0	fCantonese	naa4; laa4 (also)
U+93BF.0	fMandarin	na2
U+93BF.0	fEnglish	neptunium
U+93C0.0	fUTF8	鏀
U+93C0.0	fCNS11643	2-675F
U+93C0.0	fBig5	F373
U+93C0.0	fGB	5-7557
U+93C0.0	fR/S	 167.11
U+93C0.0	fTotalStrokes	 19
U+93C0.0	fCangjie	CYWI
U+93C0.0	fFourCorner	81160
U+93C0.0	fCantonese	lou5
U+93C0.0	fMandarin	lu3
U+93C1.0	fUTF8	鏁
U+93C1.0	fCNS11643	3-5B50
U+93C1.0	fGB	5-7626
U+93C1.0	fR/S	 167.11
U+93C1.0	fTotalStrokes	 19
U+93C1.0	fCangjie	CVVD
U+93C1.0	fFourCorner	82190
U+93C1.0	fHakka	 so3
U+93C1.0	fCantonese	so2
U+93C1.0	fMandarin	suo3
U+93C1.0	fEnglish	 lock
U+93C2.0	fUTF8	鏂
U+93C2.0	fCNS11643	2-674F
U+93C2.0	fBig5	F363
U+93C2.0	fGB	3-7561
U+93C2.0	fR/S	 167.11
U+93C2.0	fTotalStrokes	 19
U+93C2.0	fCangjie	CSRR
U+93C2.0	fFourCorner	81116
U+93C2.0	fCantonese	au1; ngau1 (also)
U+93C2.0	fMandarin	kou1; ou1; qu1
U+93C3.0	fUTF8	鏃
U+93C3.0	fCNS11643	1-7938
U+93C3.0	fBig5	C3EB
U+93C3.0	fGB	1-6F5F
U+93C3.0	fR/S	 167.11
U+93C3.0	fTotalStrokes	 19
U+93C3.0	fCangjie	CYSK
U+93C3.0	fFourCorner	88134
U+93C3.0	fHakka	  cuk8
U+93C3.0	fCantonese	zuk6; zuk1
U+93C3.0	fMandarin	zu2; chuo4; cu4; cou4; qiang1; zhuo1
U+93C3.0	fEnglish	correct form of &#x arrowhead;
U+93C4.0	fUTF8	鏄
U+93C4.0	fCNS11643	2-675D
U+93C4.0	fBig5	F371
U+93C4.0	fGB	5-753B
U+93C4.0	fR/S	 167.11
U+93C4.0	fTotalStrokes	 19
U+93C4.0	fCangjie	CJII
U+93C4.0	fFourCorner	85143
U+93C4.0	fCantonese	tyun4
U+93C4.0	fMandarin	tuan2
U+93C5.0	fUTF8	鏅
U+93C5.0	fCNS11643	3-5B44
U+93C5.0	fGB	3-766F
U+93C5.0	fR/S	  167.11
U+93C5.0	fTotalStrokes	 19
U+93C5.0	fCangjie	COLB
U+93C5.0	fFourCorner	87120
U+93C5.0	fCantonese	sau1
U+93C5.0	fMandarin	xiu1
U+93C6.0	fUTF8	鏆
U+93C6.0	fCNS11643	3-5B45
U+93C6.0	fGB	3-7657
U+93C6.0	fR/S	 167.11
U+93C6.0	fTotalStrokes	 19
U+93C6.0	fCangjie	CWJC
U+93C6.0	fFourCorner	87180
U+93C6.0	fHakka	 gwon5;  kwan5
U+93C6.0	fCantonese	gun3; gwun3 (also)
U+93C6.0	fMandarin	guan4
U+93C6.0	fEnglish	 iron bar
U+93C7.0	fUTF8	鏇
U+93C7.0	fCNS11643	2-674D
U+93C7.0	fBig5	F361
U+93C7.0	fGB	1-6F60
U+93C7.0	fR/S	 167.11
U+93C7.0	fTotalStrokes	 19
U+93C7.0	fCangjie	CYSO
U+93C7.0	fFourCorner	88181
U+93C7.0	fHakka	 sien5; sien6
U+93C7.0	fCantonese	syun4; syun6 (also)
U+93C7.0	fMandarin	xuan4; xuan2
U+93C7.0	fEnglish	[1] a kind of wine heater [2] a kind of metal tray [3] pare with a knife
U+93C8.0	fUTF8	鏈
U+93C8.0	fCNS11643	1-7939
U+93C8.0	fBig5	C3EC
U+93C8.0	fGB	1-4134
U+93C8.0	fR/S	 167.11
U+93C8.0	fTotalStrokes	 19
U+93C8.0	fCangjie	CYJJ
U+93C8.0	fFourCorner	85130
U+93C8.0	fHakka	  lien2;  lien5; lien6; len3; lian3
U+93C8.0	fCantonese	lin6; lin2 (also); lin4
U+93C8.0	fMandarin	lian4; lian2; liu2
U+93C8.0	fEnglish	 chain
U+93C9.0	fUTF8	鏉
U+93C9.0	fCNS11643	4-6167
U+93C9.0	fGB	5-757E
U+93C9.0	fR/S	 167.11
U+93C9.0	fTotalStrokes	 19
U+93C9.0	fCangjie	CDLO
U+93C9.0	fFourCorner	87180
U+93C9.0	fCantonese	sau3
U+93C9.0	fMandarin	shou4
U+93CA.0	fUTF8	鏊
U+93CA.0	fCNS11643	2-6758
U+93CA.0	fBig5	F36C
U+93CA.0	fGB	0-764B
U+93CA.0	fR/S	 167.11
U+93CA.0	fTotalStrokes	 19
U+93CA.0	fCangjie	GKC
U+93CA.0	fFourCorner	58109
U+93CA.0	fHakka	 ngau2; ngau5; ngau6
U+93CA.0	fCantonese	ngou6; ngou4 (also); ou6 (also); ou4 (also)
U+93CA.0	fMandarin	ao2; ao4; yao4
U+93CA.0	fEnglish	a round iron cooking utensil, flat and shallow
U+93CB.0	fUTF8	鏋
U+93CB.0	fCNS11643	3-5B43
U+93CB.0	fGB	E-4276
U+93CB.0	fR/S	 167.11
U+93CB.0	fTotalStrokes	 19
U+93CB.0	fCangjie	CTLB
U+93CB.0	fFourCorner	84120
U+93CB.0	fCantonese	mun5
U+93CB.0	fMandarin	man3
U+93CC.0	fUTF8	鏌
U+93CC.0	fCNS11643	2-6754
U+93CC.0	fBig5	F368
U+93CC.0	fGB	1-6F52
U+93CC.0	fR/S	 167.11; 167.10
U+93CC.0	fTotalStrokes	 19
U+93CC.0	fCangjie	CTAK
U+93CC.0	fFourCorner	84134
U+93CC.0	fHakka	  mok8
U+93CC.0	fCantonese	mok6
U+93CC.0	fMandarin	mo4
U+93CC.0	fEnglish	sword
U+93CD.0	fUTF8	鏍
U+93CD.0	fCNS11643	1-793E
U+93CD.0	fBig5	C3F1
U+93CD.0	fGB	1-6F5D
U+93CD.0	fR/S	 167.11
U+93CD.0	fTotalStrokes	 19
U+93CD.0	fCangjie	CWVF
U+93CD.0	fFourCorner	86193
U+93CD.0	fCantonese	lo4
U+93CD.0	fMandarin	luo2
U+93CE.0	fUTF8	鏎
U+93CE.0	fCNS11643	2-675E
U+93CE.0	fBig5	F372
U+93CE.0	fGB	E-4277
U+93CE.0	fR/S	 167.11
U+93CE.0	fTotalStrokes	 19
U+93CE.0	fCangjie	CWTJ
U+93CE.0	fFourCorner	86144
U+93CE.0	fCantonese	bat1
U+93CE.0	fMandarin	bi4
U+93CF.0	fUTF8	鏏
U+93CF.0	fCNS11643	2-674E
U+93CF.0	fBig5	F362
U+93CF.0	fGB	3-7675
U+93CF.0	fR/S	 167.11
U+93CF.0	fTotalStrokes	 19
U+93CF.0	fCangjie	CQJM
U+93CF.0	fFourCorner	85174
U+93CF.0	fCantonese	wai6; seoi6
U+93CF.0	fMandarin	wei4; sui4
U+93D0.0	fUTF8	鏐
U+93D0.0	fCNS11643	2-6751
U+93D0.0	fBig5	F365
U+93D0.0	fGB	3-767D
U+93D0.0	fR/S	 167.11
U+93D0.0	fTotalStrokes	 19
U+93D0.0	fCangjie	CSMH
U+93D0.0	fFourCorner	87122
U+93D0.0	fHakka	  liu2; kiu2
U+93D0.0	fCantonese	lau4
U+93D0.0	fMandarin	liu2; liao2; liu4; man4; qiu2
U+93D0.0	fEnglish	pure gold
U+93D1.0	fUTF8	鏑
U+93D1.0	fCNS11643	1-7936
U+93D1.0	fBig5	C3E9
U+93D1.0	fGB	1-6F61
U+93D1.0	fR/S	 167.11
U+93D1.0	fTotalStrokes	 19
U+93D1.0	fCangjie	CYCB
U+93D1.0	fFourCorner	80127
U+93D1.0	fHakka	  dit7
U+93D1.0	fCantonese	dik1; dik6
U+93D1.0	fMandarin	di2; di1; li2
U+93D1.0	fEnglish	[1] dysprosium [2] arrowhead
U+93D2.0	fUTF8	鏒
U+93D2.0	fCNS11643	2-6760
U+93D2.0	fBig5	F374
U+93D2.0	fGB	3-7655
U+93D2.0	fR/S	 167.11
U+93D2.0	fTotalStrokes	 19
U+93D2.0	fCangjie	CIIH
U+93D2.0	fFourCorner	83122
U+93D2.0	fCantonese	saam2; sam3; saam1
U+93D2.0	fMandarin	qiao1; can4
U+93D3.0	fUTF8	鏓
U+93D3.0	fCNS11643	3-5B48
U+93D3.0	fGB	3-767B
U+93D3.0	fR/S	  167.11
U+93D3.0	fTotalStrokes	 19
U+93D3.0	fCangjie	CHWP
U+93D3.0	fFourCorner	86130
U+93D3.0	fCantonese	cung1
U+93D3.0	fMandarin	cong1; zong3; obs; song3
U+93D4.0	fUTF8	鏔
U+93D4.0	fCNS11643	2-6759
U+93D4.0	fBig5	F36D
U+93D4.0	fGB	E-4278
U+93D4.0	fR/S	 167.11
U+93D4.0	fTotalStrokes	 19
U+93D4.0	fCangjie	CJMC
U+93D4.0	fFourCorner	83186
U+93D4.0	fCantonese	ji4
U+93D4.0	fMandarin	yi2; yin2
U+93D5.0	fUTF8	鏕
U+93D5.0	fCNS11643	2-675C
U+93D5.0	fBig5	F370
U+93D5.0	fGB	5-7623
U+93D5.0	fR/S	 167.11
U+93D5.0	fTotalStrokes	 19
U+93D5.0	fCangjie	CIXP
U+93D5.0	fFourCorner	80111
U+93D5.0	fCantonese	luk6; ou1; ou4
U+93D5.0	fMandarin	lu4; ao2
U+93D6.0	fUTF8	鏖
U+93D6.0	fCNS11643	1-793C
U+93D6.0	fBig5	C3EF
U+93D6.0	fGB	0-7769
U+93D6.0	fR/S	 167.11
U+93D6.0	fTotalStrokes	 19
U+93D6.0	fCangjie	IPC
U+93D6.0	fFourCorner	00109
U+93D6.0	fHakka	  au1; piau1
U+93D6.0	fCantonese	ngou4; ou1 (also); ngou1 (also); ou4 (also)
U+93D6.0	fMandarin	ao2; biao1
U+93D6.0	fEnglish	[1] fight; slaughter; give no quarter [2] deceive
U+93D7.0	fUTF8	鏗
U+93D7.0	fCNS11643	1-7941
U+93D7.0	fBig5	C3F4
U+93D7.0	fGB	1-6F2C
U+93D7.0	fR/S	 167.11
U+93D7.0	fTotalStrokes	 19
U+93D7.0	fCangjie	CSEG
U+93D7.0	fFourCorner	87114
U+93D7.0	fHakka	 gien1; kien1; ken1; gen1; kian1
U+93D7.0	fCantonese	hang1
U+93D7.0	fMandarin	keng1; ao2
U+93D7.0	fEnglish	[1] the clang of metal; clatter [2] the twang of a string [3] strike; beat; smash
U+93D8.0	fUTF8	鏘
U+93D8.0	fCNS11643	1-793F
U+93D8.0	fBig5	C3F2
U+93D8.0	fGB	1-6F4F
U+93D8.0	fR/S	 167.11
U+93D8.0	fTotalStrokes	 19
U+93D8.0	fCangjie	CVMI
U+93D8.0	fFourCorner	87142
U+93D8.0	fHakka	 ciong1; cong1; ziong1
U+93D8.0	fCantonese	coeng1
U+93D8.0	fMandarin	qiang1; cheng1
U+93D8.0	fEnglish	 tinkle; clang; jingle
U+93D9.0	fUTF8	鏙
U+93D9.0	fCNS11643	2-6755
U+93D9.0	fBig5	F369
U+93D9.0	fGB	3-7679
U+93D9.0	fR/S	 167.11
U+93D9.0	fTotalStrokes	 19
U+93D9.0	fCangjie	CUOG
U+93D9.0	fFourCorner	82114
U+93D9.0	fCantonese	ceoi1
U+93D9.0	fMandarin	cui1; cui3
U+93DA.0	fUTF8	鏚
U+93DA.0	fCNS11643	2-6750
U+93DA.0	fBig5	F364
U+93DA.0	fGB	5-7621
U+93DA.0	fR/S	 167.11
U+93DA.0	fTotalStrokes	 19
U+93DA.0	fCangjie	CIHF
U+93DA.0	fFourCorner	83150
U+93DA.0	fHakka	  cit7
U+93DA.0	fCantonese	cik1
U+93DA.0	fMandarin	qi4; qi1
U+93DA.0	fEnglish	axe
U+93DB.0	fUTF8	鏛
U+93DB.0	fCNS11643	3-5B4D
U+93DB.0	fGB	5-7622
U+93DB.0	fR/S	 167.11
U+93DB.0	fTotalStrokes	 19
U+93DB.0	fCangjie	CFBB
U+93DB.0	fFourCorner	89120
U+93DB.0	fCantonese	soeng4
U+93DB.0	fMandarin	chang2
U+93DB.0	fEnglish	 polish iron rim on a wagon wheel
U+93DC.0	fUTF8	鏜
U+93DC.0	fCNS11643	1-793A
U+93DC.0	fBig5	C3ED
U+93DC.0	fGB	1-6F5B
U+93DC.0	fR/S	 167.11
U+93DC.0	fTotalStrokes	 19
U+93DC.0	fCangjie	CFBG
U+93DC.0	fFourCorner	89114
U+93DC.0	fHakka	  tong2; dong1
U+93DC.0	fCantonese	tong1; tong4
U+93DC.0	fMandarin	tang1
U+93DC.0	fEnglish	[1] the noise of drums or gongs [2] a small gong
U+93DC.1	fUTF8	鏜
U+93DC.1	fCNS11643	1-793A
U+93DC.1	fBig5	C3ED
U+93DC.1	fGB	1-6F5B
U+93DC.1	fR/S	 167.11
U+93DC.1	fTotalStrokes	 19
U+93DC.1	fCangjie	CFBG
U+93DC.1	fFourCorner	89114
U+93DC.1	fMandarin	tang2
U+93DC.1	fEnglish	boring (machinery)
U+93DD.0	fUTF8	鏝
U+93DD.0	fCNS11643	1-793B
U+93DD.0	fBig5	C3EE
U+93DD.0	fGB	1-6F5C
U+93DD.0	fR/S	 167.11
U+93DD.0	fTotalStrokes	 19
U+93DD.0	fCangjie	CAWE
U+93DD.0	fFourCorner	86147
U+93DD.0	fHakka	  man5;  man2; man3
U+93DD.0	fCantonese	maan6; maan4
U+93DD.0	fMandarin	man4; man2; zu2
U+93DD.0	fEnglish	 trowel
U+93DE.0	fUTF8	鏞
U+93DE.0	fCNS11643	2-674C
U+93DE.0	fBig5	F360
U+93DE.0	fGB	1-6F5E
U+93DE.0	fR/S	 167.11
U+93DE.0	fTotalStrokes	 19
U+93DE.0	fCangjie	CILB
U+93DE.0	fFourCorner	80127
U+93DE.0	fHakka	  jung2; rung2
U+93DE.0	fCantonese	jung4
U+93DE.0	fMandarin	yong1; rong2
U+93DE.0	fEnglish	large bell used as musical instr
U+93DF.0	fUTF8	鏟
U+93DF.0	fCNS11643	1-7937
U+93DF.0	fBig5	C3EA
U+93DF.0	fGB	1-3279
U+93DF.0	fR/S	 167.11
U+93DF.0	fTotalStrokes	 19
U+93DF.0	fCangjie	CYHM
U+93DF.0	fFourCorner	80115
U+93DF.0	fHakka	  can3
U+93DF.0	fCantonese	caan2
U+93DF.0	fMandarin	chan3; chan4
U+93DF.0	fEnglish	[1] spade; shovel; trowel; scoop [2] shovel; scoop
U+93E0.0	fUTF8	鏠
U+93E0.0	fCNS11643	3-5B4F
U+93E0.0	fGB	5-757A
U+93E0.0	fR/S	 167.11
U+93E0.0	fTotalStrokes	 19
U+93E0.0	fCangjie	CYHJ
U+93E0.0	fFourCorner	87130
U+93E0.0	fCantonese	fung1
U+93E0.0	fMandarin	feng1
U+93E1.0	fUTF8	鏡
U+93E1.0	fCNS11643	1-7935
U+93E1.0	fBig5	C3E8
U+93E1.0	fGB	1-3E35
U+93E1.0	fR/S	 167.11
U+93E1.0	fTotalStrokes	 19
U+93E1.0	fCangjie	CYTU
U+93E1.0	fFourCorner	80116
U+93E1.0	fHakka	  giang5
U+93E1.0	fCantonese	geng3; geng2 (also)
U+93E1.0	fMandarin	jing4
U+93E1.0	fEnglish	[1] mirror [2] lens; glass; glasses; spectacles [3] mirror [4] take warning (from a past failur)
U+93E2.0	fUTF8	鏢
U+93E2.0	fCNS11643	1-793D
U+93E2.0	fBig5	C3F0
U+93E2.0	fGB	1-6F5A
U+93E2.0	fR/S	 167.11
U+93E2.0	fTotalStrokes	 19
U+93E2.0	fCangjie	CMWF
U+93E2.0	fFourCorner	81191
U+93E2.0	fHakka	  biau1; piau1; beu1; peu1
U+93E2.0	fCantonese	biu1
U+93E2.0	fMandarin	biao1; piao1
U+93E2.0	fEnglish	[1] javelin; dart; spear; harpoon; dartlike weapon [2] escort; guard; body-guard
U+93E3.0	fUTF8	鏣
U+93E3.0	fCNS11643	2-675B
U+93E3.0	fBig5	F36F
U+93E3.0	fGB	E-4279
U+93E3.0	fR/S	 167.11
U+93E3.0	fTotalStrokes	 19
U+93E3.0	fCangjie	CITF
U+93E3.0	fFourCorner	80137
U+93E3.0	fCantonese	syu3
U+93E3.0	fMandarin	shu4
U+93E4.0	fUTF8	鏤
U+93E4.0	fCNS11643	1-7940
U+93E4.0	fBig5	C3F3
U+93E4.0	fGB	1-6F4E
U+93E4.0	fR/S	 167.11
U+93E4.0	fTotalStrokes	 19
U+93E4.0	fCangjie	CLLV
U+93E4.0	fFourCorner	85144
U+93E4.0	fHakka	  leu3;  leu2; liu3; liu2; leu5
U+93E4.0	fCantonese	lau6
U+93E4.0	fMandarin	lou4; lian2; l2
U+93E4.0	fEnglish	 carve; engrave
U+93E5.0	fUTF8	鏥
U+93E5.0	fCNS11643	3-5B4A
U+93E5.0	fGB	E-427A
U+93E5.0	fR/S	  167.11
U+93E5.0	fTotalStrokes	 19
U+93E5.0	fCangjie	CJOA
U+93E5.0	fFourCorner	83160
U+93E5.0	fHakka	 siu5
U+93E5.0	fCantonese	sau3
U+93E5.0	fMandarin	xiu4; zan4
U+93E5.0	fEnglish	rust; corrosion
U+93E6.0	fUTF8	鏦
U+93E6.0	fCNS11643	2-6757
U+93E6.0	fBig5	F36B
U+93E6.0	fGB	3-7565
U+93E6.0	fR/S	 167.11
U+93E6.0	fTotalStrokes	 19
U+93E6.0	fCangjie	CHOO
U+93E6.0	fFourCorner	88181
U+93E6.0	fHakka	  ciung2; cung1
U+93E6.0	fCantonese	cung1
U+93E6.0	fMandarin	cong1; chuang1
U+93E6.0	fEnglish	[1] spear [2] pierce with a spear [3] the clang of metal
U+93E7.0	fUTF8	鏧
U+93E7.0	fCNS11643	2-6762
U+93E7.0	fBig5	F375
U+93E7.0	fGB	E-427B
U+93E7.0	fR/S	 167.11
U+93E7.0	fTotalStrokes	 19
U+93E7.0	fCangjie	GEC
U+93E7.0	fFourCorner	47109
U+93E7.0	fCantonese	lung4
U+93E7.0	fMandarin	long2; long1; pang2; tong2
U+93E8.0	fUTF8	鏨
U+93E8.0	fCNS11643	1-7942
U+93E8.0	fBig5	C3F5
U+93E8.0	fGB	1-7649
U+93E8.0	fR/S	 167.11
U+93E8.0	fTotalStrokes	 19
U+93E8.0	fCangjie	JLC
U+93E8.0	fFourCorner	52109
U+93E8.0	fHakka	 cam6;  cam5; {cam3}; ciam5
U+93E8.0	fCantonese	zaam6
U+93E8.0	fMandarin	zan4; jian4; can2; chan2; keng1
U+93E8.0	fEnglish	[1] chisel; engraving tool [2] chisel; engrave; carve
U+93E9.0	fUTF8	鏩
U+93E9.0	fCNS11643	3-5B41
U+93E9.0	fGB	3-7643
U+93E9.0	fR/S	 167.11
U+93E9.0	fTotalStrokes	 19
U+93E9.0	fCangjie	CJJL
U+93E9.0	fFourCorner	82120
U+93E9.0	fCantonese	zim6
U+93E9.0	fMandarin	jian4
U+93EA.0	fUTF8	鏪
U+93EA.0	fCNS11643	3-5B42
U+93EA.0	fGB	E-427C
U+93EA.0	fR/S	 167.11
U+93EA.0	fTotalStrokes	 19
U+93EA.0	fCangjie	CTWA
U+93EA.0	fFourCorner	85160
U+93EA.0	fCantonese	cou4
U+93EA.0	fMandarin	cao2
U+93EB.0	fUTF8	鏫
U+93EB.0	fCNS11643	3-5B4E
U+93EB.0	fGB	E-427D
U+93EB.0	fR/S	 167.11
U+93EB.0	fTotalStrokes	 19
U+93EB.0	fCangjie	CHND
U+93EB.0	fFourCorner	82190
U+93EB.0	fCantonese	lei4
U+93EB.0	fMandarin	li2
U+93EC.0	fUTF8	鏬
U+93EC.0	fCNS11643	2-6753
U+93EC.0	fBig5	F367
U+93EC.0	fGB	E-427E
U+93EC.0	fR/S	 167.11
U+93EC.0	fTotalStrokes	 19
U+93EC.0	fCangjie	CYPD
U+93EC.0	fFourCorner	81149
U+93EC.0	fCantonese	laa3
U+93EC.0	fMandarin	xia4
U+93ED.0	fUTF8	鏭
U+93ED.0	fCNS11643	3-5B47
U+93ED.0	fGB	E-4321
U+93ED.0	fR/S	  167.11
U+93ED.0	fTotalStrokes	 19
U+93ED.0	fCangjie	CHDP
U+93ED.0	fFourCorner	82130
U+93ED.0	fCantonese	sik1
U+93ED.0	fMandarin	xi1
U+93EE.0	fUTF8	鏮
U+93EE.0	fCNS11643	2-675A
U+93EE.0	fBig5	F36E
U+93EE.0	fGB	E-4322
U+93EE.0	fR/S	 167.11
U+93EE.0	fTotalStrokes	 19
U+93EE.0	fCangjie	CILE
U+93EE.0	fFourCorner	80132
U+93EE.0	fHakka	 kong1
U+93EE.0	fCantonese	hong1; kong3
U+93EE.0	fMandarin	kang1
U+93EF.0	fUTF8	鏯
U+93EF.0	fCNS11643	3-6665
U+93EF.0	fGB	E-4323
U+93EF.0	fR/S	 167.11
U+93EF.0	fTotalStrokes	 19
U+93EF.0	fCangjie	CKKK
U+93EF.0	fFourCorner	8413
U+93EF.0	fCantonese	song2
U+93F0.0	fUTF8	鏰
U+93F0.0	fGB	3-767A
U+93F0.0	fR/S	 167.11
U+93F0.0	fTotalStrokes	 19
U+93F0.0	fCangjie	CUBB
U+93F0.0	fCantonese	bang1
U+93F0.0	fMandarin	beng4
U+93F0.0	fEnglish	coin; money
U+93F1.0	fUTF8	鏱
U+93F1.0	fCNS11643	3-6663
U+93F1.0	fGB	E-4324
U+93F1.0	fR/S	 167.11
U+93F1.0	fTotalStrokes	 19
U+93F1.0	fCangjie	CYTJ
U+93F1.0	fFourCorner	8014
U+93F1.0	fCantonese	gam2
U+93F2.0	fUTF8	鏲
U+93F2.0	fCNS11643	3-6664
U+93F2.0	fGB	E-4325
U+93F2.0	fR/S	 167.11
U+93F2.0	fTotalStrokes	 19
U+93F2.0	fCangjie	CYVQ
U+93F2.0	fFourCorner	8015
U+93F2.0	fCantonese	cim1
U+93F3.0	fUTF8	鏳
U+93F3.0	fCNS11643	3-5D41
U+93F3.0	fGB	E-4326
U+93F3.0	fR/S	 167.12
U+93F3.0	fTotalStrokes	 20
U+93F3.0	fCangjie	CCWA
U+93F3.0	fFourCorner	88160
U+93F3.0	fHakka	  zen1
U+93F3.0	fCantonese	caang4
U+93F3.0	fMandarin	zheng1
U+93F3.0	fEnglish	the ringing sound given by metal when struck
U+93F4.0	fUTF8	鏴
U+93F4.0	fCNS11643	3-5E6A
U+93F4.0	fGB	E-4327
U+93F4.0	fR/S	  167.13; 167.12
U+93F4.0	fTotalStrokes	 21
U+93F4.0	fCangjie	CRMR
U+93F4.0	fFourCorner	87160
U+93F4.0	fHakka	 lu5; lu6
U+93F4.0	fCantonese	lou6
U+93F4.0	fMandarin	lu4
U+93F4.0	fEnglish	 plug a hole
U+93F5.0	fUTF8	鏵
U+93F5.0	fCNS11643	2-6A41
U+93F5.0	fBig5	F4F3
U+93F5.0	fGB	1-6E7C
U+93F5.0	fR/S	 167.12
U+93F5.0	fTotalStrokes	 20
U+93F5.0	fCangjie	CTMJ
U+93F5.0	fFourCorner	84154
U+93F5.0	fHakka	  fa2; fa3
U+93F5.0	fCantonese	waa4; waa1
U+93F5.0	fMandarin	hua2
U+93F5.0	fEnglish	spade; shovel; plowshare
U+93F6.0	fUTF8	鏶
U+93F6.0	fCNS11643	2-6A50
U+93F6.0	fBig5	F542
U+93F6.0	fGB	E-4328
U+93F6.0	fR/S	 167.12
U+93F6.0	fTotalStrokes	 20
U+93F6.0	fCangjie	COGD
U+93F6.0	fFourCorner	80194
U+93F6.0	fCantonese	cip3; zaap6
U+93F6.0	fMandarin	ji2; qie4
U+93F7.0	fUTF8	鏷
U+93F7.0	fCNS11643	2-6A43
U+93F7.0	fBig5	F4F5
U+93F7.0	fGB	1-6F64
U+93F7.0	fR/S	 167.12
U+93F7.0	fTotalStrokes	 20
U+93F7.0	fCangjie	CTCO
U+93F7.0	fFourCorner	82135
U+93F7.0	fHakka	 puk8; pok7
U+93F7.0	fCantonese	pok3; buk6
U+93F7.0	fMandarin	pu2
U+93F7.0	fEnglish	protactinium
U+93F8.0	fUTF8	鏸
U+93F8.0	fCNS11643	2-6A4A
U+93F8.0	fBig5	F4FC
U+93F8.0	fGB	3-7723
U+93F8.0	fR/S	 167.12
U+93F8.0	fTotalStrokes	 20
U+93F8.0	fCangjie	CJIP
U+93F8.0	fFourCorner	85133
U+93F8.0	fCantonese	wai6; jeoi6; seoi6
U+93F8.0	fMandarin	hui4; rui4; sui4
U+93F9.0	fUTF8	鏹
U+93F9.0	fCNS11643	2-6752
U+93F9.0	fBig5	F366
U+93F9.0	fGB	1-6F6A
U+93F9.0	fR/S	 167.11
U+93F9.0	fTotalStrokes	 19
U+93F9.0	fCangjie	CNII
U+93F9.0	fFourCorner	83136
U+93F9.0	fHakka	 giong3
U+93F9.0	fCantonese	koeng5
U+93F9.0	fMandarin	qiang3; jiang3
U+93F9.0	fEnglish	 money; wealth
U+93F9.1	fUTF8	鏹
U+93F9.1	fCNS11643	2-6752
U+93F9.1	fBig5	F366
U+93F9.1	fGB	1-6F6A
U+93F9.1	fR/S	 167.11
U+93F9.1	fTotalStrokes	 19
U+93F9.1	fCangjie	CNII
U+93F9.1	fFourCorner	83136
U+93F9.1	fMandarin	qiang1
U+93F9.1	fEnglish	[1] corrosive [2] sulphuric acid; strong acid
U+93FA.0	fUTF8	鏺
U+93FA.0	fCNS11643	2-6A48
U+93FA.0	fBig5	F4FA
U+93FA.0	fGB	3-7578
U+93FA.0	fR/S	 167.12
U+93FA.0	fTotalStrokes	 20
U+93FA.0	fCangjie	CNOE
U+93FA.0	fFourCorner	82147
U+93FA.0	fHakka	  pat7
U+93FA.0	fCantonese	put3
U+93FA.0	fMandarin	po1; bo1
U+93FA.0	fEnglish	knife to cut grass
U+93FB.0	fUTF8	鏻
U+93FB.0	fCNS11643	2-6A37
U+93FB.0	fBig5	F4E9
U+93FB.0	fGB	3-7728
U+93FB.0	fR/S	 167.12; 167.13
U+93FB.0	fTotalStrokes	 20
U+93FB.0	fCangjie	CFDQ
U+93FB.0	fFourCorner	89159
U+93FB.0	fHakka	 lin2
U+93FB.0	fCantonese	leon4
U+93FB.0	fMandarin	lin2; lin4; ling2
U+93FB.0	fEnglish	phosphonium
U+93FC.0	fUTF8	鏼
U+93FC.0	fCNS11643	2-6A4E
U+93FC.0	fBig5	F540
U+93FC.0	fGB	E-4329
U+93FC.0	fR/S	 167.12
U+93FC.0	fTotalStrokes	 20
U+93FC.0	fCangjie	CHDB
U+93FC.0	fFourCorner	88192
U+93FC.0	fCantonese	saak3; suk1
U+93FC.0	fMandarin	suo3
U+93FD.0	fUTF8	鏽
U+93FD.0	fCNS11643	1-7A4E
U+93FD.0	fBig5	C4C3
U+93FD.0	fGB	E-432A
U+93FD.0	fR/S	 167.12; 167.13
U+93FD.0	fTotalStrokes	 20
U+93FD.0	fCangjie	CLX
U+93FD.0	fFourCorner	82127
U+93FD.0	fHakka	 siu5; lu1
U+93FD.0	fCantonese	sau3
U+93FD.0	fMandarin	xiu4; su4
U+93FD.0	fEnglish	 rust; corrode
U+93FE.0	fUTF8	鏾
U+93FE.0	fCNS11643	2-6A3B
U+93FE.0	fBig5	F4ED
U+93FE.0	fGB	3-7722
U+93FE.0	fR/S	 167.12
U+93FE.0	fTotalStrokes	 20
U+93FE.0	fCangjie	CTBK
U+93FE.0	fFourCorner	88140
U+93FE.0	fHakka	  san5; sien5; san3
U+93FE.0	fCantonese	sin3
U+93FE.0	fMandarin	san3; xian4; sa4; san4
U+93FE.0	fEnglish	 the trigger of a crossbow crossbow
U+93FF.0	fUTF8	鏿
U+93FF.0	fCNS11643	2-6A4D
U+93FF.0	fBig5	F4FE
U+93FF.0	fGB	E-432B
U+93FF.0	fR/S	 167.12
U+93FF.0	fTotalStrokes	 20
U+93FF.0	fCangjie	CFBD
U+93FF.0	fFourCorner	89194
U+93FF.0	fCantonese	coeng1; caang1
U+93FF.0	fMandarin	cheng1; qiang1
U+9400.0	fUTF8	鐀
U+9400.0	fCNS11643	2-6A42
U+9400.0	fBig5	F4F4
U+9400.0	fGB	5-756A
U+9400.0	fR/S	 167.12
U+9400.0	fTotalStrokes	 20
U+9400.0	fCangjie	CLMC
U+9400.0	fFourCorner	85186
U+9400.0	fHakka	 kui3; kui6; kui5; kwi5; k'wui5
U+9400.0	fCantonese	gwai6; gwai3
U+9400.0	fMandarin	kui4; gui4
U+9400.0	fEnglish	 cupboard press wardrobe shop-counter
U+9401.0	fUTF8	鐁
U+9401.0	fCNS11643	4-6467
U+9401.0	fGB	E-432C
U+9401.0	fR/S	 167.12
U+9401.0	fTotalStrokes	 20
U+9401.0	fCangjie	CTCL
U+9401.0	fFourCorner	82120
U+9401.0	fCantonese	si1
U+9401.0	fMandarin	si1
U+9402.0	fUTF8	鐂
U+9402.0	fCNS11643	3-5D43
U+9402.0	fGB	E-432D
U+9402.0	fR/S	 167.12
U+9402.0	fTotalStrokes	 20
U+9402.0	fCangjie	CMLW
U+9402.0	fFourCorner	81160
U+9402.0	fCantonese	lau4; lau6 (also)
U+9402.0	fMandarin	liu2; liu4
U+9402.0	fEnglish	lutecium
U+9403.0	fUTF8	鐃
U+9403.0	fCNS11643	1-7A4D
U+9403.0	fBig5	C4C2
U+9403.0	fGB	1-6E73
U+9403.0	fR/S	 167.12
U+9403.0	fTotalStrokes	 20
U+9403.0	fCangjie	CGGU
U+9403.0	fFourCorner	84111
U+9403.0	fHakka	  ngiau2;  nau2; gniau3; ngieu2; lau2
U+9403.0	fCantonese	naau4; laau4 (also); naau6
U+9403.0	fMandarin	nao2; nao4; dui1; dui4
U+9403.0	fEnglish	[1] a kind of bell used in the army in ancient times [2] cymbals
U+9404.0	fUTF8	鐄
U+9404.0	fCNS11643	3-5D3C
U+9404.0	fGB	3-7677
U+9404.0	fR/S	 167.12
U+9404.0	fTotalStrokes	 20
U+9404.0	fCangjie	CTLC
U+9404.0	fFourCorner	84180
U+9404.0	fCantonese	waang4; wong4
U+9404.0	fMandarin	heng2; huang2
U+9404.0	fEnglish	bell
U+9405.0	fUTF8	鐅
U+9405.0	fCNS11643	3-5D3F
U+9405.0	fGB	5-7533
U+9405.0	fR/S	 167.12
U+9405.0	fTotalStrokes	 20; 19
U+9405.0	fCangjie	FKC
U+9405.0	fFourCorner	98100
U+9405.0	fCantonese	pit3
U+9405.0	fMandarin	pie3
U+9406.0	fUTF8	鐆
U+9406.0	fCNS11643	2-6A52
U+9406.0	fBig5	F544
U+9406.0	fGB	5-752F
U+9406.0	fR/S	 167.12
U+9406.0	fTotalStrokes	 20
U+9406.0	fCangjie	NOC
U+9406.0	fFourCorner	74109
U+9406.0	fHakka	 sui5; sui6
U+9406.0	fCantonese	seoi6
U+9406.0	fMandarin	sui4
U+9406.0	fEnglish	 lens; burning glass
U+9407.0	fUTF8	鐇
U+9407.0	fCNS11643	2-6A44
U+9407.0	fBig5	F4F6
U+9407.0	fGB	3-7727
U+9407.0	fR/S	 167.12
U+9407.0	fTotalStrokes	 20
U+9407.0	fCangjie	CHDW
U+9407.0	fFourCorner	82169
U+9407.0	fCantonese	faan4
U+9407.0	fMandarin	fan2; fan1; nao2
U+9407.0	fEnglish	vanadium
U+9408.0	fUTF8	鐈
U+9408.0	fCNS11643	3-5D3E
U+9408.0	fGB	3-762A
U+9408.0	fR/S	 167.12
U+9408.0	fTotalStrokes	 20
U+9408.0	fCangjie	CHKB
U+9408.0	fFourCorner	82120
U+9408.0	fCantonese	kiu4
U+9408.0	fMandarin	qiao2
U+9409.0	fUTF8	鐉
U+9409.0	fCNS11643	2-6A49
U+9409.0	fBig5	F4FB
U+9409.0	fGB	E-432E
U+9409.0	fR/S	 167.12
U+9409.0	fTotalStrokes	 20
U+9409.0	fCangjie	CRUC
U+9409.0	fFourCorner	87181
U+9409.0	fCantonese	cyun1; cyun4
U+9409.0	fMandarin	quan1; chuan1
U+940A.0	fUTF8	鐊
U+940A.0	fCNS11643	2-6A4C
U+940A.0	fBig5	F4FD
U+940A.0	fGB	E-432F
U+940A.0	fR/S	 167.12
U+940A.0	fTotalStrokes	 20
U+940A.0	fCangjie	CNLH
U+940A.0	fFourCorner	86127
U+940A.0	fCantonese	joeng4
U+940A.0	fMandarin	yang2
U+940B.0	fUTF8	鐋
U+940B.0	fCNS11643	2-6A35
U+940B.0	fBig5	F4E7
U+940B.0	fGB	1-6F26
U+940B.0	fR/S	 167.12
U+940B.0	fTotalStrokes	 20
U+940B.0	fCangjie	CEAH
U+940B.0	fFourCorner	86127
U+940B.0	fHakka	 tong5; tong3; tong1
U+940B.0	fCantonese	tong1; tong2; tong3
U+940B.0	fMandarin	tang4; tang1
U+940B.0	fEnglish	gong
U+940C.0	fUTF8	鐌
U+940C.0	fCNS11643	2-6A4F
U+940C.0	fBig5	F541
U+940C.0	fGB	3-767C
U+940C.0	fR/S	 167.12
U+940C.0	fTotalStrokes	 20
U+940C.0	fCangjie	CNAO
U+940C.0	fFourCorner	87132
U+940C.0	fCantonese	zoeng6
U+940C.0	fMandarin	xiang4
U+940D.0	fUTF8	鐍
U+940D.0	fCNS11643	2-6A40
U+940D.0	fBig5	F4F2
U+940D.0	fGB	3-772B
U+940D.0	fR/S	 167.12
U+940D.0	fTotalStrokes	 20
U+940D.0	fCangjie	CNHB
U+940D.0	fFourCorner	87127
U+940D.0	fHakka	 git7;  giet7
U+940D.0	fCantonese	kyut3
U+940D.0	fMandarin	jue2; yu4
U+940E.0	fUTF8	鐎
U+940E.0	fCNS11643	2-6A45
U+940E.0	fBig5	F4F7
U+940E.0	fGB	3-7725
U+940E.0	fR/S	 167.12
U+940E.0	fTotalStrokes	 20
U+940E.0	fCangjie	COGF
U+940E.0	fFourCorner	80131
U+940E.0	fHakka	  ziau1; tsieu1
U+940E.0	fCantonese	ziu1; ciu4
U+940E.0	fMandarin	jiao1; jiu1; qiao2
U+940E.0	fEnglish	a soldier's brass kettle
U+940F.0	fUTF8	鐏
U+940F.0	fCNS11643	2-6A39
U+940F.0	fBig5	F4EB
U+940F.0	fGB	3-7729
U+940F.0	fR/S	 167.12
U+940F.0	fTotalStrokes	 20
U+940F.0	fCangjie	CTWI
U+940F.0	fFourCorner	88146
U+940F.0	fHakka	  zun1; zun5
U+940F.0	fCantonese	zeon1; zyun3 (also)
U+940F.0	fMandarin	zun1; zuan3; zuan4
U+940F.0	fEnglish	cap
U+9410.0	fUTF8	鐐
U+9410.0	fCNS11643	2-6A3D
U+9410.0	fBig5	F4EF
U+9410.0	fGB	1-414D
U+9410.0	fR/S	 167.12
U+9410.0	fTotalStrokes	 20
U+9410.0	fCangjie	CKCF
U+9410.0	fFourCorner	84196
U+9410.0	fHakka	 liau2; liau1
U+9410.0	fCantonese	liu4
U+9410.0	fMandarin	liao2; liao4; lao4
U+9410.0	fEnglish	fetters; shackles
U+9411.0	fUTF8	鐑
U+9411.0	fCNS11643	2-6A51
U+9411.0	fBig5	F543
U+9411.0	fGB	E-4330
U+9411.0	fR/S	 167.12
U+9411.0	fTotalStrokes	 20
U+9411.0	fCangjie	CVFR
U+9411.0	fFourCorner	84161
U+9411.0	fCantonese	kit3
U+9411.0	fMandarin	jie2
U+9412.0	fUTF8	鐒
U+9412.0	fCNS11643	2-6A47
U+9412.0	fBig5	F4F9
U+9412.0	fGB	1-6F29
U+9412.0	fR/S	 167.12
U+9412.0	fTotalStrokes	 20
U+9412.0	fCangjie	CFFS
U+9412.0	fFourCorner	89127
U+9412.0	fHakka	 lau2
U+9412.0	fCantonese	lou4
U+9412.0	fMandarin	lao2
U+9412.0	fEnglish	lawrencium
U+9413.0	fUTF8	鐓
U+9413.0	fCNS11643	2-6A36
U+9413.0	fBig5	F4E8
U+9413.0	fGB	1-6F66
U+9413.0	fR/S	 167.12
U+9413.0	fTotalStrokes	 20
U+9413.0	fCangjie	CYDK
U+9413.0	fFourCorner	88140
U+9413.0	fHakka	  dun1;  tui5
U+9413.0	fCantonese	deoi6; deon1; deoi1
U+9413.0	fMandarin	dui4; dun1; dui1; duan4; dun4; tan2; xun2
U+9413.0	fEnglish	 ferrule castrate
U+9414.0	fUTF8	鐔
U+9414.0	fCNS11643	2-6A3A
U+9414.0	fBig5	F4EC
U+9414.0	fGB	1-6F62
U+9414.0	fR/S	 167.12
U+9414.0	fTotalStrokes	 20
U+9414.0	fCangjie	CMWJ
U+9414.0	fFourCorner	81146
U+9414.0	fHakka	  tam2;  cim2; jam2
U+9414.0	fCantonese	taam4; cam4 (also); jam4
U+9414.0	fMandarin	tan2; xin2; xin1; chan2; dan4; duo2; xin4; ya1; yin2
U+9414.0	fEnglish	 dagger small sword
U+9415.0	fUTF8	鐕
U+9415.0	fCNS11643	2-6A3C
U+9415.0	fBig5	F4EE
U+9415.0	fGB	3-7724
U+9415.0	fR/S	 167.12; 167.14
U+9415.0	fTotalStrokes	 20
U+9415.0	fCangjie	CMUA
U+9415.0	fFourCorner	81163
U+9415.0	fCantonese	zaam1
U+9415.0	fMandarin	zan1; zan4; zen1
U+9416.0	fUTF8	鐖
U+9416.0	fCNS11643	2-6A46
U+9416.0	fBig5	F4F8
U+9416.0	fGB	3-7555
U+9416.0	fR/S	 167.12
U+9416.0	fTotalStrokes	 20
U+9416.0	fCangjie	CVII
U+9416.0	fFourCorner	82153
U+9416.0	fHakka	 gi3; gi1
U+9416.0	fCantonese	gei1
U+9416.0	fMandarin	ji1; ai2; qi2
U+9416.0	fEnglish	[1] barb on a hook [2] fluke
U+9417.0	fUTF8	鐗
U+9417.0	fCNS11643	3-5D3D
U+9417.0	fGB	E-4331
U+9417.0	fR/S	  167.12
U+9417.0	fTotalStrokes	 20
U+9417.0	fCangjie	CANB
U+9417.0	fFourCorner	87120
U+9417.0	fCantonese	gaan2
U+9417.0	fMandarin	jian3; jian4
U+9418.0	fUTF8	鐘
U+9418.0	fCNS11643	1-7A4C
U+9418.0	fBig5	C4C1
U+9418.0	fGB	1-5653
U+9418.0	fR/S	 167.12
U+9418.0	fTotalStrokes	 20
U+9418.0	fCangjie	CYTG
U+9418.0	fFourCorner	80114
U+9418.0	fHakka	  zung1; zhung1
U+9418.0	fCantonese	zung1
U+9418.0	fMandarin	zhong1
U+9418.0	fEnglish	[1] clock [2] bell (which tolls as distinct from that which jingles [3] time as measured in hours and minutes [4] a Chinese family name
U+9419.0	fUTF8	鐙
U+9419.0	fCNS11643	2-6A3F
U+9419.0	fBig5	F4F1
U+9419.0	fGB	1-6F6B
U+9419.0	fR/S	 167.12
U+9419.0	fTotalStrokes	 20
U+9419.0	fCangjie	CNOT
U+9419.0	fFourCorner	82118
U+9419.0	fHakka	 den5
U+9419.0	fCantonese	dang3
U+9419.0	fMandarin	deng4
U+9419.0	fEnglish	 stirrup
U+9419.1	fUTF8	鐙
U+9419.1	fCNS11643	2-6A3F
U+9419.1	fBig5	F4F1
U+9419.1	fGB	1-6F6B
U+9419.1	fR/S	 167.12
U+9419.1	fTotalStrokes	 20
U+9419.1	fCangjie	CNOT
U+9419.1	fFourCorner	82118
U+9419.1	fHakka	 den1
U+9419.1	fCantonese	dang1
U+9419.1	fMandarin	deng1
U+9419.1	fEnglish	[1] same as &#x71C8; [2] a kind of cooking vessel
U+941A.0	fUTF8	鐚
U+941A.0	fCNS11643	3-5D44
U+941A.0	fGB	E-4332
U+941A.0	fR/S	 167.12
U+941A.0	fTotalStrokes	 20
U+941A.0	fCangjie	CMMP
U+941A.0	fFourCorner	81130
U+941A.0	fCantonese	aa1; ngaa1 (also)
U+941A.0	fMandarin	ya4; ya1; lou4
U+941A.0	fEnglish	 soft steel ammonium
U+941B.0	fUTF8	鐛
U+941B.0	fCNS11643	3-5D45
U+941B.0	fGB	E-4333
U+941B.0	fR/S	  167.12
U+941B.0	fTotalStrokes	 20
U+941B.0	fCangjie	CAYF
U+941B.0	fFourCorner	86190
U+941B.0	fMandarin	ying4
U+941C.0	fUTF8	鐜
U+941C.0	fCNS11643	3-5D47
U+941C.0	fGB	5-7534
U+941C.0	fR/S	  167.12
U+941C.0	fTotalStrokes	 20
U+941C.0	fCangjie	YKC
U+941C.0	fFourCorner	08100
U+941C.0	fCantonese	deoi6
U+941C.0	fMandarin	dui4
U+941D.0	fUTF8	鐝
U+941D.0	fCNS11643	4-646B
U+941D.0	fGB	1-6F63
U+941D.0	fR/S	  167.12
U+941D.0	fTotalStrokes	 20
U+941D.0	fCangjie	CMTO
U+941D.0	fFourCorner	87180
U+941D.0	fHakka	 giet7; ket7
U+941D.0	fCantonese	kyut3
U+941D.0	fMandarin	jue2
U+941D.0	fEnglish	a pick; a hoe
U+941E.0	fUTF8	鐞
U+941E.0	fCNS11643	3-5D42
U+941E.0	fGB	E-4334
U+941E.0	fR/S	 167.12
U+941E.0	fTotalStrokes	 20
U+941E.0	fCangjie	CSMI
U+941E.0	fFourCorner	87140
U+941E.0	fCantonese	nau6; lau6 (also)
U+941E.0	fMandarin	nou4
U+941F.0	fUTF8	鐟
U+941F.0	fCNS11643	3-5D3A
U+941F.0	fGB	E-4335
U+941F.0	fR/S	 167.12
U+941F.0	fTotalStrokes	 20
U+941F.0	fCangjie	CQOA
U+941F.0	fFourCorner	85160
U+941F.0	fCantonese	zaam1
U+941F.0	fMandarin	ti4
U+9420.0	fUTF8	鐠
U+9420.0	fCNS11643	2-6A38
U+9420.0	fBig5	F4EA
U+9420.0	fGB	1-6F68
U+9420.0	fR/S	 167.12
U+9420.0	fTotalStrokes	 20
U+9420.0	fCangjie	CTCA
U+9420.0	fFourCorner	82161
U+9420.0	fHakka	 pu1
U+9420.0	fCantonese	pou2
U+9420.0	fMandarin	pu3
U+9420.0	fEnglish	praseodymium
U+9421.0	fUTF8	鐡
U+9421.0	fGB	E-4336
U+9421.0	fR/S	 167.12
U+9421.0	fTotalStrokes	 20
U+9421.0	fCangjie	CJIT
U+9421.0	fCantonese	tit3
U+9421.0	fMandarin	tie3
U+9421.0	fEnglish	 iron strong; solid; firm
U+9422.0	fUTF8	鐢
U+9422.0	fCNS11643	3-6666
U+9422.0	fGB	E-4337
U+9422.0	fR/S	 167.12
U+9422.0	fTotalStrokes	 20
U+9422.0	fCangjie	DDC
U+9422.0	fFourCorner	4410
U+9422.0	fCantonese	faan4
U+9423.0	fUTF8	鐣
U+9423.0	fCNS11643	3-6667
U+9423.0	fGB	E-4338
U+9423.0	fR/S	 167.12
U+9423.0	fTotalStrokes	 20
U+9423.0	fCangjie	CFBQ
U+9423.0	fFourCorner	8915
U+9423.0	fCantonese	zoeng2
U+9424.0	fUTF8	鐤
U+9424.0	fCNS11643	3-6668
U+9424.0	fGB	E-4339
U+9424.0	fR/S	 167.13; 167.12
U+9424.0	fTotalStrokes	 20
U+9424.0	fCangjie	CBUL
U+9424.0	fCantonese	ding2
U+9424.0	fMandarin	ding3; zhen1
U+9424.0	fEnglish	 huge tripod of bronze with two ears sacrificial vessel
U+9425.0	fUTF8	鐥
U+9425.0	fCNS11643	F-6657
U+9425.0	fGB	5-762B
U+9425.0	fR/S	 167.12
U+9425.0	fTotalStrokes	 20
U+9425.0	fCangjie	CTTR
U+9425.0	fFourCorner	8816
U+9425.0	fCantonese	sin3; sin6 (also)
U+9425.0	fMandarin	shan4
U+9426.0	fUTF8	鐦
U+9426.0	fGB	1-6F34
U+9426.0	fR/S	 167.12
U+9426.0	fTotalStrokes	 20
U+9426.0	fCangjie	CANT
U+9426.0	fFourCorner	8712
U+9426.0	fCantonese	hoi1
U+9426.0	fMandarin	kai1
U+9426.0	fEnglish	californium
U+9427.0	fUTF8	鐧
U+9427.0	fCNS11643	3-5D46
U+9427.0	fGB	1-6F35
U+9427.0	fR/S	 167.12
U+9427.0	fTotalStrokes	 20
U+9427.0	fCangjie	CANA
U+9427.0	fFourCorner	87120
U+9427.0	fHakka	 gan3; gan5; gan1
U+9427.0	fCantonese	gaan2; gaan3
U+9427.0	fMandarin	jian3; jian1
U+9427.0	fEnglish	a kind of ancient weapon; mace
U+9427.1	fUTF8	鐧
U+9427.1	fCNS11643	3-5D46
U+9427.1	fGB	1-6F35
U+9427.1	fR/S	 167.12
U+9427.1	fTotalStrokes	 20
U+9427.1	fCangjie	CANA
U+9427.1	fFourCorner	87120
U+9427.1	fMandarin	jian4
U+9427.1	fEnglish	protective metal on the axis of a wheel
U+9428.0	fUTF8	鐨
U+9428.0	fCNS11643	2-6A3E
U+9428.0	fBig5	F4F0
U+9428.0	fGB	1-6F50
U+9428.0	fR/S	 167.12
U+9428.0	fTotalStrokes	 20
U+9428.0	fCangjie	CLNC
U+9428.0	fFourCorner	85186
U+9428.0	fCantonese	fai3
U+9428.0	fMandarin	fei4
U+9428.0	fEnglish	fermium
U+9429.0	fUTF8	鐩
U+9429.0	fCNS11643	2-6C50
U+9429.0	fBig5	F661
U+9429.0	fGB	5-762C
U+9429.0	fR/S	 167.13
U+9429.0	fTotalStrokes	 21
U+9429.0	fCangjie	CYTO
U+9429.0	fFourCorner	88133
U+9429.0	fCantonese	seoi6
U+9429.0	fMandarin	sui4
U+9429.0	fEnglish	lens
U+942A.0	fUTF8	鐪
U+942A.0	fCNS11643	2-6C54
U+942A.0	fBig5	F666
U+942A.0	fGB	3-7647
U+942A.0	fR/S	 167.13
U+942A.0	fTotalStrokes	 21
U+942A.0	fCangjie	CYPS
U+942A.0	fFourCorner	81127
U+942A.0	fCantonese	lou5
U+942A.0	fMandarin	lu3
U+942B.0	fUTF8	鐫
U+942B.0	fCNS11643	1-7B3B
U+942B.0	fBig5	C54F
U+942B.0	fGB	E-433A
U+942B.0	fR/S	 167.13
U+942B.0	fTotalStrokes	 21
U+942B.0	fCangjie	COGS
U+942B.0	fFourCorner	80127
U+942B.0	fHakka	 son1; cion1; zun5; cien2; zun1
U+942B.0	fCantonese	zyun1; zeon3 (also); zeon1; syun6
U+942B.0	fMandarin	juan1; huan2; juan3
U+942B.0	fEnglish	[1] carve; engrave [2] be deboted (said of an official)
U+942C.0	fUTF8	鐬
U+942C.0	fCNS11643	2-6C56
U+942C.0	fBig5	F668
U+942C.0	fGB	3-7628
U+942C.0	fR/S	 167.13
U+942C.0	fTotalStrokes	 21
U+942C.0	fCangjie	CYMH
U+942C.0	fFourCorner	81153
U+942C.0	fCantonese	wai3
U+942C.0	fMandarin	hui4
U+942D.0	fUTF8	鐭
U+942D.0	fCNS11643	3-5E6D
U+942D.0	fGB	5-762A
U+942D.0	fR/S	  167.13
U+942D.0	fTotalStrokes	 21
U+942D.0	fCangjie	CHBK
U+942D.0	fFourCorner	87130
U+942D.0	fCantonese	ou3; ngou3 (also)
U+942D.0	fMandarin	yu4
U+942E.0	fUTF8	鐮
U+942E.0	fCNS11643	1-7B35
U+942E.0	fBig5	C549
U+942E.0	fGB	1-412D
U+942E.0	fR/S	 167.13
U+942E.0	fTotalStrokes	 21
U+942E.0	fCangjie	CITC
U+942E.0	fFourCorner	80137
U+942E.0	fHakka	 liam2
U+942E.0	fCantonese	lim4
U+942E.0	fMandarin	lian2
U+942E.0	fEnglish	 sickle
U+942F.0	fUTF8	鐯
U+942F.0	fCNS11643	4-646F
U+942F.0	fGB	3-7676
U+942F.0	fR/S	 167.13; 167.12
U+942F.0	fTotalStrokes	 21; 20
U+942F.0	fCangjie	CTJA
U+942F.0	fFourCorner	84160
U+942F.0	fCantonese	zoek3; zoek6 (also)
U+942F.0	fMandarin	zhuo2; zhuo1
U+9430.0	fUTF8	鐰
U+9430.0	fCNS11643	2-6C52
U+9430.0	fBig5	F664
U+9430.0	fGB	3-772E
U+9430.0	fR/S	 167.13
U+9430.0	fTotalStrokes	 21
U+9430.0	fCangjie	CRRD
U+9430.0	fFourCorner	86194
U+9430.0	fHakka	 ciau1;  cau5;  sau5
U+9430.0	fCantonese	ciu1; cou3; cou4
U+9430.0	fMandarin	qiao1; cao2; sao4
U+9430.0	fEnglish	fine steel
U+9431.0	fUTF8	鐱
U+9431.0	fCNS11643	2-6C58
U+9431.0	fBig5	F66A
U+9431.0	fGB	3-763B
U+9431.0	fR/S	 167.13
U+9431.0	fTotalStrokes	 21
U+9431.0	fCangjie	COMO
U+9431.0	fFourCorner	88186
U+9431.0	fCantonese	gim3; cim1; gim6
U+9431.0	fMandarin	qian1; jian4
U+9432.0	fUTF8	鐲
U+9432.0	fCNS11643	1-7B3A
U+9432.0	fBig5	C54E
U+9432.0	fGB	1-6F6D
U+9432.0	fR/S	 167.13
U+9432.0	fTotalStrokes	 21
U+9432.0	fCangjie	CWLI
U+9432.0	fFourCorner	86127
U+9432.0	fHakka	  cuk8; zuk7; cuk7
U+9432.0	fCantonese	zuk6
U+9432.0	fMandarin	zhuo2; shu3
U+9432.0	fEnglish	[1] bracelet; armlet [2] a kind of bell used in the army in ancient times
U+9433.0	fUTF8	鐳
U+9433.0	fCNS11643	1-7B36
U+9433.0	fBig5	C54A
U+9433.0	fGB	1-4058
U+9433.0	fR/S	 167.13
U+9433.0	fTotalStrokes	 21
U+9433.0	fCangjie	CMBW
U+9433.0	fFourCorner	81162
U+9433.0	fHakka	 lui2
U+9433.0	fCantonese	leoi4
U+9433.0	fMandarin	lei2
U+9433.0	fEnglish	[1] pot; jar [2] radium
U+9434.0	fUTF8	鐴
U+9434.0	fCNS11643	4-675C
U+9434.0	fGB	3-7731
U+9434.0	fR/S	 167.13
U+9434.0	fTotalStrokes	 21
U+9434.0	fCangjie	CSRJ
U+9434.0	fFourCorner	80140
U+9434.0	fCantonese	bik1
U+9434.0	fMandarin	bi4; bei4; obs
U+9435.0	fUTF8	鐵
U+9435.0	fCNS11643	1-7B37
U+9435.0	fBig5	C54B
U+9435.0	fGB	1-4C7A
U+9435.0	fR/S	 167.13
U+9435.0	fTotalStrokes	 21
U+9435.0	fCangjie	CJIG
U+9435.0	fFourCorner	83150
U+9435.0	fHakka	  tiet7; tet7; t'iat7
U+9435.0	fCantonese	tit3
U+9435.0	fMandarin	tie3; die2; juan1
U+9435.0	fEnglish	[1] iron [2] strong; solid; firm; indisputable; unyielding (like iron) [3] cruel; mercilss; unfeeling [4] arms; weapons
U+9436.0	fUTF8	鐶
U+9436.0	fCNS11643	2-6C4F
U+9436.0	fBig5	F660
U+9436.0	fGB	3-772F
U+9436.0	fR/S	 167.13
U+9436.0	fTotalStrokes	 21
U+9436.0	fCangjie	CWLV
U+9436.0	fFourCorner	86132
U+9436.0	fHakka	  fan2; fan3; van2
U+9436.0	fCantonese	waan4
U+9436.0	fMandarin	huan2; liao2; liao4; xuan4
U+9436.0	fEnglish	 ring; measure
U+9437.0	fUTF8	鐷
U+9437.0	fCNS11643	2-6C55
U+9437.0	fBig5	F667
U+9437.0	fGB	E-433B
U+9437.0	fR/S	 167.13
U+9437.0	fTotalStrokes	 21
U+9437.0	fCangjie	CTPD
U+9437.0	fFourCorner	84194
U+9437.0	fCantonese	jip6
U+9437.0	fMandarin	ye4
U+9438.0	fUTF8	鐸
U+9438.0	fCNS11643	1-7B39
U+9438.0	fBig5	C54D
U+9438.0	fGB	1-6E6C
U+9438.0	fR/S	 167.13
U+9438.0	fTotalStrokes	 21
U+9438.0	fCangjie	CWLJ
U+9438.0	fFourCorner	86141
U+9438.0	fHakka	 tok8; dok7
U+9438.0	fCantonese	dok6; nok6
U+9438.0	fMandarin	duo2
U+9438.0	fEnglish	[1] large bell [2] a Chinese family name
U+9439.0	fUTF8	鐹
U+9439.0	fCNS11643	2-6C53
U+9439.0	fBig5	F665
U+9439.0	fGB	5-754E
U+9439.0	fR/S	 167.13
U+9439.0	fTotalStrokes	 21
U+9439.0	fCangjie	CYBB
U+9439.0	fFourCorner	87132
U+9439.0	fCantonese	gwo2; go2 (also); gwo1; gwo3
U+9439.0	fMandarin	guo3; ge1; guo4
U+943A.0	fUTF8	鐺
U+943A.0	fCNS11643	1-7B38
U+943A.0	fBig5	C54C
U+943A.0	fGB	1-6E75
U+943A.0	fR/S	 167.13
U+943A.0	fTotalStrokes	 21
U+943A.0	fCangjie	CFBW
U+943A.0	fFourCorner	89166
U+943A.0	fHakka	 dong1
U+943A.0	fCantonese	dong1; caang1; cing1; tong1; cang1
U+943A.0	fMandarin	dang1; tang1
U+943A.0	fEnglish	[1] the sound of striking a gong [2] shackles
U+943A.1	fUTF8	鐺
U+943A.1	fCNS11643	1-7B38
U+943A.1	fBig5	C54C
U+943A.1	fGB	1-6E75
U+943A.1	fR/S	 167.13
U+943A.1	fTotalStrokes	 21
U+943A.1	fCangjie	CFBW
U+943A.1	fFourCorner	89166
U+943A.1	fMandarin	cheng1
U+943A.1	fEnglish	[1] a kind of wine heater [2] a cauldron-like vessel with legs [3] a pan for frying; a shallow pot
U+943B.0	fUTF8	鐻
U+943B.0	fCNS11643	2-6C4E
U+943B.0	fBig5	F65F
U+943B.0	fGB	3-772D
U+943B.0	fR/S	 167.13
U+943B.0	fTotalStrokes	 21
U+943B.0	fCangjie	CYPO
U+943B.0	fFourCorner	81132
U+943B.0	fHakka	 gi3; ki3; gi2
U+943B.0	fCantonese	geoi6; keoi4; geoi3
U+943B.0	fMandarin	ju4; qu2; ju1
U+943B.0	fEnglish	drumstick
U+943C.0	fUTF8	鐼
U+943C.0	fCNS11643	2-6A4B
U+943C.0	fBig5	F663
U+943C.0	fGB	3-7659
U+943C.0	fR/S	 167.13
U+943C.0	fTotalStrokes	 21
U+943C.0	fCangjie	CJTC
U+943C.0	fFourCorner	84186
U+943C.0	fCantonese	fan4; ban1; fan3
U+943C.0	fMandarin	fen2; ben1; xuan1; xun4
U+943D.0	fUTF8	鐽
U+943D.0	fCNS11643	2-6C51
U+943D.0	fBig5	F662
U+943D.0	fGB	E-433C
U+943D.0	fR/S	 167.13
U+943D.0	fTotalStrokes	 21
U+943D.0	fCangjie	CYGQ
U+943D.0	fFourCorner	84134
U+943D.0	fCantonese	daat6
U+943D.0	fMandarin	da2
U+943E.0	fUTF8	鐾
U+943E.0	fCNS11643	4-6762
U+943E.0	fGB	0-764D
U+943E.0	fR/S	 167.13
U+943E.0	fTotalStrokes	 21
U+943E.0	fCangjie	SJC
U+943E.0	fFourCorner	70100
U+943E.0	fCantonese	bai3
U+943E.0	fMandarin	bi4; bei4; obs
U+943F.0	fUTF8	鐿
U+943F.0	fCNS11643	2-6C4D
U+943F.0	fBig5	F65E
U+943F.0	fGB	1-6F6E
U+943F.0	fR/S	 167.13
U+943F.0	fTotalStrokes	 21
U+943F.0	fCangjie	CYTP
U+943F.0	fFourCorner	80136
U+943F.0	fCantonese	ji3
U+943F.0	fMandarin	yi4
U+943F.0	fEnglish	ytterbium
U+9440.0	fUTF8	鑀
U+9440.0	fCNS11643	2-6C57
U+9440.0	fBig5	F669
U+9440.0	fGB	E-433D
U+9440.0	fR/S	 167.13
U+9440.0	fTotalStrokes	 21
U+9440.0	fCangjie	CBBE
U+9440.0	fFourCorner	82147
U+9440.0	fCantonese	oi3; ngoi3 (also)
U+9440.0	fMandarin	ai4
U+9440.0	fEnglish	ionium
U+9441.0	fUTF8	鑁
U+9441.0	fGB	E-433E
U+9441.0	fR/S	 167.13
U+9441.0	fTotalStrokes	 21
U+9441.0	fCangjie	XCUCE
U+9441.0	fFourCorner	8214
U+9441.0	fCantonese	zung1
U+9441.0	fMandarin	zong1; wan3; dang1; zheng1
U+9442.0	fUTF8	鑂
U+9442.0	fCNS11643	3-5F69
U+9442.0	fGB	E-433F
U+9442.0	fR/S	 167.14; 167.13
U+9442.0	fTotalStrokes	 22; 21
U+9442.0	fCangjie	CHGF
U+9442.0	fFourCorner	82130
U+9442.0	fCantonese	fan3
U+9442.0	fMandarin	xun4
U+9443.0	fUTF8	鑃
U+9443.0	fCNS11643	3-5F6E
U+9443.0	fGB	E-4340
U+9443.0	fR/S	  167.14
U+9443.0	fTotalStrokes	 22
U+9443.0	fCangjie	CSMG
U+9443.0	fFourCorner	87110
U+9443.0	fCantonese	diu6
U+9443.0	fMandarin	diao4; yao2
U+9444.0	fUTF8	鑄
U+9444.0	fCNS11643	1-7B7B
U+9444.0	fBig5	C5B1
U+9444.0	fGB	1-567D
U+9444.0	fR/S	 167.14
U+9444.0	fTotalStrokes	 22
U+9444.0	fCangjie	CGNI
U+9444.0	fFourCorner	84141
U+9444.0	fHakka	 zu5; zhu5
U+9444.0	fCantonese	zyu3
U+9444.0	fMandarin	zhu4; jian4; zhou4
U+9444.0	fEnglish	[1] melt; cast; coin; mint; cast metal [2] make or commit (blunders, etc.) [3] educate and influence (a person) [4] a Chinese family name
U+9445.0	fUTF8	鑅
U+9445.0	fCNS11643	2-6E3C
U+9445.0	fBig5	F76D
U+9445.0	fGB	5-7567
U+9445.0	fR/S	 167.14
U+9445.0	fTotalStrokes	 22
U+9445.0	fCangjie	CFFD
U+9445.0	fFourCorner	89194
U+9445.0	fCantonese	waang4
U+9445.0	fMandarin	heng2
U+9446.0	fUTF8	鑆
U+9446.0	fCNS11643	2-6E3F
U+9446.0	fBig5	F770
U+9446.0	fGB	E-4341
U+9446.0	fR/S	 167.14
U+9446.0	fTotalStrokes	 22
U+9446.0	fCangjie	CTGI
U+9446.0	fFourCorner	84100
U+9446.0	fCantonese	zeoi6
U+9446.0	fMandarin	zhui4
U+9447.0	fUTF8	鑇
U+9447.0	fCNS11643	2-6E3B
U+9447.0	fBig5	F76C
U+9447.0	fGB	5-754F
U+9447.0	fR/S	 167.14
U+9447.0	fTotalStrokes	 22
U+9447.0	fCangjie	CYX
U+9447.0	fFourCorner	80123
U+9447.0	fCantonese	zai1
U+9447.0	fMandarin	ji1
U+9448.0	fUTF8	鑈
U+9448.0	fCNS11643	2-6E3D
U+9448.0	fBig5	F76E
U+9448.0	fGB	5-7546
U+9448.0	fR/S	 167.14
U+9448.0	fTotalStrokes	 22
U+9448.0	fCangjie	CMFB
U+9448.0	fFourCorner	81127
U+9448.0	fCantonese	nip6; lip6 (also); nai5
U+9448.0	fMandarin	nie1; ni3; nie4
U+9449.0	fUTF8	鑉
U+9449.0	fCNS11643	2-6E3E
U+9449.0	fBig5	F76F
U+9449.0	fGB	E-4342
U+9449.0	fR/S	 167.14
U+9449.0	fTotalStrokes	 22
U+9449.0	fCangjie	CTGT
U+9449.0	fFourCorner	84117
U+9449.0	fCantonese	hap6; taap3; koi3
U+9449.0	fMandarin	ta4; he2
U+944A.0	fUTF8	鑊
U+944A.0	fCNS11643	2-6E38
U+944A.0	fBig5	F769
U+944A.0	fGB	1-6F6C
U+944A.0	fR/S	 167.14; 167.13
U+944A.0	fTotalStrokes	 22
U+944A.0	fCangjie	CTOE
U+944A.0	fFourCorner	84147
U+944A.0	fHakka	  wok8; wok7; vok8
U+944A.0	fCantonese	wok6; wok2 (also)
U+944A.0	fMandarin	huo4
U+944A.0	fEnglish	 cauldron for cooking; large iron pot; wok
U+944B.0	fUTF8	鑋
U+944B.0	fCNS11643	2-6E39
U+944B.0	fBig5	F76A
U+944B.0	fGB	3-7553
U+944B.0	fR/S	 167.14
U+944B.0	fTotalStrokes	 22
U+944B.0	fCangjie	JMC
U+944B.0	fFourCorner	51109
U+944B.0	fCantonese	hing1
U+944B.0	fMandarin	qing4; qing1
U+944C.0	fUTF8	鑌
U+944C.0	fCNS11643	2-6E36
U+944C.0	fBig5	F767
U+944C.0	fGB	1-6F59
U+944C.0	fR/S	 167.14
U+944C.0	fTotalStrokes	 22
U+944C.0	fCangjie	CJMC
U+944C.0	fFourCorner	83186
U+944C.0	fHakka	 bin1
U+944C.0	fCantonese	ban1
U+944C.0	fMandarin	bin1
U+944C.0	fEnglish	high quality iron
U+944D.0	fUTF8	鑍
U+944D.0	fCNS11643	3-5F6A
U+944D.0	fGB	E-4343
U+944D.0	fR/S	 167.14
U+944D.0	fTotalStrokes	 22
U+944D.0	fCangjie	BCC
U+944D.0	fFourCorner	66100
U+944D.0	fCantonese	jing1
U+944D.0	fMandarin	ying1
U+944E.0	fUTF8	鑎
U+944E.0	fCNS11643	3-5F6C
U+944E.0	fGB	E-4344
U+944E.0	fR/S	  167.14
U+944E.0	fTotalStrokes	 22
U+944E.0	fCangjie	CSLC
U+944E.0	fFourCorner	81110
U+944E.0	fCantonese	gwai6
U+944E.0	fMandarin	kui4
U+944F.0	fUTF8	鑏
U+944F.0	fCNS11643	2-6E3A
U+944F.0	fBig5	F76B
U+944F.0	fGB	3-7575
U+944F.0	fR/S	 167.14
U+944F.0	fTotalStrokes	 22
U+944F.0	fCangjie	CJPN
U+944F.0	fFourCorner	83112
U+944F.0	fCantonese	ning4; ling4 (also)
U+944F.0	fMandarin	ning2; ning3
U+9450.0	fUTF8	鑐
U+9450.0	fCNS11643	2-6E37
U+9450.0	fBig5	F768
U+9450.0	fGB	3-7733
U+9450.0	fR/S	 167.14
U+9450.0	fTotalStrokes	 22
U+9450.0	fCangjie	CMBB
U+9450.0	fFourCorner	81127
U+9450.0	fHakka	 si1
U+9450.0	fCantonese	seoi1; jau4; jyu4
U+9450.0	fMandarin	xu1; ru2; rou2
U+9450.0	fEnglish	bolt of a Chinese lock
U+9451.0	fUTF8	鑑
U+9451.0	fCNS11643	1-7B7C
U+9451.0	fBig5	C5B2
U+9451.0	fGB	E-4345
U+9451.0	fR/S	 167.14
U+9451.0	fTotalStrokes	 22
U+9451.0	fCangjie	CSIT
U+9451.0	fFourCorner	88117
U+9451.0	fHakka	  gam5
U+9451.0	fCantonese	gaam3
U+9451.0	fMandarin	jian4; han4; jian1; xian4
U+9451.0	fEnglish	[1] mirror; looking glass [2] reflect; mirror; shine [3] study; examine; scrutinize [4] an example serving as a rule or warning
U+9452.0	fUTF8	鑒
U+9452.0	fCNS11643	1-7B7D
U+9452.0	fBig5	C5B3
U+9452.0	fGB	1-3C78
U+9452.0	fR/S	 167.14
U+9452.0	fTotalStrokes	 22
U+9452.0	fCangjie	SWC
U+9452.0	fFourCorner	78109
U+9452.0	fHakka	  gam5
U+9452.0	fCantonese	gaam3
U+9452.0	fMandarin	jian4; han4; jian1; xian4
U+9452.0	fEnglish	 study; examine; scrutinize
U+9453.0	fUTF8	鑓
U+9453.0	fGB	E-4346
U+9453.0	fR/S	 167.14
U+9453.0	fTotalStrokes	 22
U+9453.0	fCangjie	CYLR
U+9453.0	fCantonese	hin2
U+9453.0	fMandarin	qiang1
U+9453.0	fEnglish	 spear, lance, javelin
U+9454.0	fUTF8	鑔
U+9454.0	fCNS11643	3-5F6B
U+9454.0	fGB	1-6F6F
U+9454.0	fR/S	  167.14
U+9454.0	fTotalStrokes	 22
U+9454.0	fCangjie	CJBF
U+9454.0	fFourCorner	83190
U+9454.0	fCantonese	caa2
U+9454.0	fMandarin	cha3
U+9455.0	fUTF8	鑕
U+9455.0	fCNS11643	2-6F59
U+9455.0	fBig5	F84B
U+9455.0	fGB	3-764C
U+9455.0	fR/S	 167.15
U+9455.0	fTotalStrokes	 23
U+9455.0	fCangjie	CHLC
U+9455.0	fFourCorner	82186
U+9455.0	fHakka	 zhit7; tsiit7
U+9455.0	fCantonese	zat1
U+9455.0	fMandarin	zhi4
U+9455.0	fEnglish	an ancient version of a guillotine
U+9456.0	fUTF8	鑖
U+9456.0	fCNS11643	3-6052
U+9456.0	fGB	3-7732
U+9456.0	fR/S	 167.15
U+9456.0	fTotalStrokes	 23
U+9456.0	fCangjie	CTWI
U+9456.0	fFourCorner	84150
U+9456.0	fCantonese	mit6
U+9456.0	fMandarin	mie4; mi4
U+9457.0	fUTF8	鑗
U+9457.0	fCNS11643	2-6F5B
U+9457.0	fBig5	F84D
U+9457.0	fGB	E-4347
U+9457.0	fR/S	 167.15
U+9457.0	fTotalStrokes	 23
U+9457.0	fCangjie	CHHE
U+9457.0	fFourCorner	87132
U+9457.0	fCantonese	lei4
U+9457.0	fMandarin	li2
U+9458.0	fUTF8	鑘
U+9458.0	fCNS11643	4-6B29
U+9458.0	fGB	3-7734
U+9458.0	fR/S	 167.15
U+9458.0	fTotalStrokes	 23
U+9458.0	fCangjie	CWWW
U+9458.0	fFourCorner	86160
U+9458.0	fCantonese	leoi4
U+9458.0	fMandarin	lei2
U+9459.0	fUTF8	鑙
U+9459.0	fCNS11643	5-765D
U+9459.0	fGB	E-4348
U+9459.0	fR/S	 167.15
U+9459.0	fTotalStrokes	 23
U+9459.0	fCangjie	CHUA
U+9459.0	fFourCorner	83160
U+9459.0	fCantonese	kai1
U+9459.0	fMandarin	ji1
U+945A.0	fUTF8	鑚
U+945A.0	fCNS11643	3-6055
U+945A.0	fGB	E-4349
U+945A.0	fR/S	 167.15
U+945A.0	fTotalStrokes	 23
U+945A.0	fCangjie	CQOC
U+945A.0	fFourCorner	85180
U+945A.0	fHakka	  zon1;  zon5
U+945A.0	fCantonese	zyun3; zyun1 (also)
U+945A.0	fMandarin	zuan4; zuan1; zuan3
U+945A.0	fEnglish	 drill; bore pierce diamond
U+945B.0	fUTF8	鑛
U+945B.0	fCNS11643	3-6054
U+945B.0	fGB	E-434A
U+945B.0	fR/S	 167.15
U+945B.0	fTotalStrokes	 23
U+945B.0	fCangjie	CITC
U+945B.0	fFourCorner	80180
U+945B.0	fHakka	 kwang3; kong5; gwang3
U+945B.0	fCantonese	kong3; kwong3 (also); gwong3
U+945B.0	fMandarin	kuang4; zhu4
U+945B.0	fEnglish	 mine; mineral
U+945C.0	fUTF8	鑜
U+945C.0	fCNS11643	3-6056
U+945C.0	fGB	E-434B
U+945C.0	fR/S	 167.15
U+945C.0	fTotalStrokes	 23
U+945C.0	fCangjie	CFBC
U+945C.0	fFourCorner	89180
U+945C.0	fCantonese	soeng2
U+945C.0	fMandarin	shang4; shang3
U+945D.0	fUTF8	鑝
U+945D.0	fCNS11643	2-6F5A
U+945D.0	fBig5	F84C
U+945D.0	fGB	5-762D
U+945D.0	fR/S	 167.15
U+945D.0	fTotalStrokes	 23
U+945D.0	fCangjie	CTYJ
U+945D.0	fFourCorner	84114
U+945D.0	fCantonese	pung4
U+945D.0	fMandarin	peng2
U+945E.0	fUTF8	鑞
U+945E.0	fCNS11643	2-6F5C
U+945E.0	fBig5	F84E
U+945E.0	fGB	3-7736
U+945E.0	fR/S	 167.15
U+945E.0	fTotalStrokes	 23
U+945E.0	fCangjie	CVVV
U+945E.0	fFourCorner	82116
U+945E.0	fHakka	 lap8
U+945E.0	fCantonese	laap6; lap6
U+945E.0	fMandarin	la4
U+945E.0	fEnglish	 solder; solder tin; an alloy of tin and lead for welding
U+945F.0	fUTF8	鑟
U+945F.0	fCNS11643	3-604F
U+945F.0	fGB	E-434C
U+945F.0	fR/S	 167.15
U+945F.0	fTotalStrokes	 23
U+945F.0	fCangjie	CGWC
U+945F.0	fFourCorner	84180
U+945F.0	fCantonese	duk6
U+945F.0	fMandarin	du2
U+9460.0	fUTF8	鑠
U+9460.0	fCNS11643	1-7C4C
U+9460.0	fBig5	C5E0
U+9460.0	fGB	1-6E65
U+9460.0	fR/S	 167.15
U+9460.0	fTotalStrokes	 23
U+9460.0	fCangjie	CVID
U+9460.0	fFourCorner	82194
U+9460.0	fHakka	 sok7; shok7; shok8
U+9460.0	fCantonese	soek3
U+9460.0	fMandarin	shuo4; li4; yue4; gong3; kuang4; yao4
U+9460.0	fEnglish	[1] melt metals with fire or heat; smelt [2] wear off [3] shining; lustrous [4] powerful
U+9461.0	fUTF8	鑡
U+9461.0	fCNS11643	3-6053
U+9461.0	fGB	3-7645
U+9461.0	fR/S	 167.15
U+9461.0	fTotalStrokes	 23
U+9461.0	fCangjie	CYMU
U+9461.0	fFourCorner	81170
U+9461.0	fCantonese	cuk1
U+9461.0	fMandarin	chuo4
U+9462.0	fUTF8	鑢
U+9462.0	fCNS11643	2-6F58
U+9462.0	fBig5	F84A
U+9462.0	fGB	3-766D
U+9462.0	fR/S	 167.15
U+9462.0	fTotalStrokes	 23
U+9462.0	fCangjie	CYPP
U+9462.0	fFourCorner	81136
U+9462.0	fHakka	  li5; li6
U+9462.0	fCantonese	leoi6; leoi3
U+9462.0	fMandarin	l4; luu4; lu4; shuo4
U+9462.0	fEnglish	 file; rasp file polish
U+9463.0	fUTF8	鑣
U+9463.0	fCNS11643	1-7C4B
U+9463.0	fBig5	C5DF
U+9463.0	fGB	1-6F70
U+9463.0	fR/S	 167.15
U+9463.0	fTotalStrokes	 23
U+9463.0	fCangjie	CIPF
U+9463.0	fFourCorner	80131
U+9463.0	fHakka	  biau1; piau1; beu1
U+9463.0	fCantonese	biu1
U+9463.0	fMandarin	biao1
U+9463.0	fEnglish	[1] bit for a horse; bridle [2] said of a horse [3] a dart-like projectile thrown by hand as a weapon
U+9464.0	fUTF8	鑤
U+9464.0	fCNS11643	1-7C4D
U+9464.0	fBig5	C5E1
U+9464.0	fGB	E-434D
U+9464.0	fR/S	 167.15
U+9464.0	fTotalStrokes	 23
U+9464.0	fCangjie	CATE
U+9464.0	fFourCorner	8613
U+9464.0	fCantonese	paau4
U+9464.0	fMandarin	bao4; pao2
U+9464.0	fEnglish	carpenter's plane
U+9465.0	fUTF8	鑥
U+9465.0	fCNS11643	3-6057
U+9465.0	fGB	1-6F65
U+9465.0	fR/S	  167.15
U+9465.0	fTotalStrokes	 23
U+9465.0	fCangjie	CNWA
U+9465.0	fFourCorner	87160
U+9465.0	fCantonese	lou5
U+9465.0	fMandarin	lu3
U+9465.0	fEnglish	lutetium
U+9466.0	fUTF8	鑦
U+9466.0	fCNS11643	3-666A
U+9466.0	fGB	E-434E
U+9466.0	fR/S	 167.15
U+9466.0	fTotalStrokes	 23
U+9466.0	fCangjie	CSEC
U+9466.0	fFourCorner	8718
U+9466.0	fCantonese	jin4
U+9467.0	fUTF8	鑧
U+9467.0	fCNS11643	3-6669
U+9467.0	fGB	E-434F
U+9467.0	fR/S	 167.14
U+9467.0	fTotalStrokes	 22
U+9467.0	fCangjie	CJTI
U+9467.0	fFourCorner	8311
U+9467.0	fCantonese	jin6
U+9468.0	fUTF8	鑨
U+9468.0	fCNS11643	2-7059
U+9468.0	fBig5	F8CB
U+9468.0	fGB	3-756D
U+9468.0	fR/S	 167.16
U+9468.0	fTotalStrokes	 24
U+9468.0	fCangjie	CYBP
U+9468.0	fFourCorner	81111
U+9468.0	fCantonese	lung4
U+9468.0	fMandarin	long2
U+9469.0	fUTF8	鑩
U+9469.0	fCNS11643	2-705A
U+9469.0	fBig5	F8CC
U+9469.0	fGB	E-4350
U+9469.0	fR/S	 167.16
U+9469.0	fTotalStrokes	 24
U+9469.0	fCangjie	CMGR
U+9469.0	fFourCorner	81116
U+9469.0	fCantonese	ngok6; ok6 (also)
U+9469.0	fMandarin	e4
U+946A.0	fUTF8	鑪
U+946A.0	fCNS11643	1-7C6F
U+946A.0	fBig5	C644
U+946A.0	fGB	E-4351
U+946A.0	fR/S	 167.16
U+946A.0	fTotalStrokes	 24
U+946A.0	fCangjie	CYPT
U+946A.0	fFourCorner	81117
U+946A.0	fHakka	 lu2
U+946A.0	fCantonese	lou4; lou5
U+946A.0	fMandarin	lu2; la4
U+946A.0	fEnglish	[1] fireplace; stove; oven; furnace [2] rutherfordium
U+946B.0	fUTF8	鑫
U+946B.0	fCNS11643	2-7058
U+946B.0	fBig5	F8CA
U+946B.0	fGB	0-764E
U+946B.0	fR/S	 167.16
U+946B.0	fTotalStrokes	 24
U+946B.0	fCangjie	CCC
U+946B.0	fFourCorner	80119
U+946B.0	fHakka	 him1
U+946B.0	fCantonese	jam1
U+946B.0	fMandarin	xin1; xun4; xiong1
U+946B.0	fEnglish	used in names with a connotation of prosperity and good profit
U+946C.0	fUTF8	鑬
U+946C.0	fCNS11643	3-6122
U+946C.0	fGB	E-4352
U+946C.0	fR/S	 167.16
U+946C.0	fTotalStrokes	 24
U+946C.0	fCangjie	CSIU
U+946C.0	fFourCorner	88110
U+946C.0	fCantonese	gaam3
U+946C.0	fMandarin	jian4
U+946D.0	fUTF8	鑭
U+946D.0	fCNS11643	2-7142
U+946D.0	fBig5	F953
U+946D.0	fGB	1-6F67
U+946D.0	fR/S	 167.17
U+946D.0	fTotalStrokes	 25
U+946D.0	fCangjie	CANW
U+946D.0	fFourCorner	87120
U+946D.0	fHakka	 lan5; lan6; lan2
U+946D.0	fCantonese	laan4
U+946D.0	fMandarin	lan4; lan2
U+946D.0	fEnglish	lanthanum
U+946E.0	fUTF8	鑮
U+946E.0	fCNS11643	2-7141
U+946E.0	fBig5	F952
U+946E.0	fGB	5-7632
U+946E.0	fR/S	 167.17
U+946E.0	fTotalStrokes	 25
U+946E.0	fCangjie	CTEI
U+946E.0	fFourCorner	84142
U+946E.0	fHakka	 pok8
U+946E.0	fCantonese	bok3; bok6
U+946E.0	fMandarin	bo2; po4
U+946E.0	fEnglish	 large bell
U+946F.0	fUTF8	鑯
U+946F.0	fCNS11643	2-7143
U+946F.0	fBig5	F954
U+946F.0	fGB	5-7633
U+946F.0	fR/S	 167.17
U+946F.0	fTotalStrokes	 25
U+946F.0	fCangjie	COIM
U+946F.0	fFourCorner	83150
U+946F.0	fHakka	  ciam1; ziam1; ciam3
U+946F.0	fCantonese	zim1; cim1
U+946F.0	fMandarin	jian1; qian1
U+946F.0	fEnglish	 awl; sharp iron point
U+9470.0	fUTF8	鑰
U+9470.0	fCNS11643	1-7D2C
U+9470.0	fBig5	C65F
U+9470.0	fGB	1-543F
U+9470.0	fR/S	 167.17
U+9470.0	fTotalStrokes	 25
U+9470.0	fCangjie	COMB
U+9470.0	fFourCorner	88127
U+9470.0	fHakka	  jok7; rok7
U+9470.0	fCantonese	joek6
U+9470.0	fMandarin	yao4; yue4
U+9470.0	fEnglish	[1] key [2] lock
U+9471.0	fUTF8	鑱
U+9471.0	fCNS11643	2-7144
U+9471.0	fBig5	F955
U+9471.0	fGB	3-7737
U+9471.0	fR/S	 167.17
U+9471.0	fTotalStrokes	 25
U+9471.0	fCangjie	CNRI
U+9471.0	fFourCorner	87113
U+9471.0	fHakka	 cam2; cam1
U+9471.0	fCantonese	caam4
U+9471.0	fMandarin	chan2; chan4
U+9471.0	fEnglish	spade; trowel
U+9472.0	fUTF8	鑲
U+9472.0	fCNS11643	1-7D2B
U+9472.0	fBig5	C65E
U+9472.0	fGB	1-4F62
U+9472.0	fR/S	 167.17
U+9472.0	fTotalStrokes	 25
U+9472.0	fCangjie	CYRV
U+9472.0	fFourCorner	80132
U+9472.0	fHakka	  siong1
U+9472.0	fCantonese	soeng1
U+9472.0	fMandarin	xiang1; rang2; niang2
U+9472.0	fEnglish	[1] fill in (tooth, etc.); mount; inlay; set (jewels, etc.) [2] edge; border; hem; bordered [3] name of an ancient weapon
U+9473.0	fUTF8	鑳
U+9473.0	fCNS11643	2-7145
U+9473.0	fBig5	F956
U+9473.0	fGB	E-4353
U+9473.0	fR/S	 167.17
U+9473.0	fTotalStrokes	 25
U+9473.0	fCangjie	CJTO
U+9473.0	fFourCorner	83181
U+9473.0	fCantonese	gin6
U+9473.0	fMandarin	jian4; jian3; qian2
U+9474.0	fUTF8	鑴
U+9474.0	fCNS11643	2-7161
U+9474.0	fBig5	F972
U+9474.0	fGB	3-7738
U+9474.0	fR/S	 167.18
U+9474.0	fTotalStrokes	 26
U+9474.0	fCangjie	CUOB
U+9474.0	fFourCorner	82127
U+9474.0	fHakka	  hi1; gui3; kwui1
U+9474.0	fCantonese	kwai4
U+9474.0	fMandarin	xi1; hui1; xi4; gui1; jiu1; sui1
U+9474.0	fEnglish	 engrave or carve (as a block for printing)
U+9475.0	fUTF8	鑵
U+9475.0	fCNS11643	2-7164
U+9475.0	fBig5	F975
U+9475.0	fGB	E-4354
U+9475.0	fR/S	 167.18
U+9475.0	fTotalStrokes	 26
U+9475.0	fCangjie	CTRG
U+9475.0	fFourCorner	84114
U+9475.0	fHakka	 gon5; gwon5
U+9475.0	fCantonese	gun3; gwun3 (also)
U+9475.0	fMandarin	guan4; yao4; yue4
U+9475.0	fEnglish	 jar can
U+9476.0	fUTF8	鑶
U+9476.0	fCNS11643	2-7163
U+9476.0	fBig5	F974
U+9476.0	fGB	E-4355
U+9476.0	fR/S	 167.18
U+9476.0	fTotalStrokes	 26
U+9476.0	fCangjie	CTIS
U+9476.0	fFourCorner	84153
U+9476.0	fCantonese	cong4
U+9476.0	fMandarin	cang2
U+9477.0	fUTF8	鑷
U+9477.0	fCNS11643	1-7D35
U+9477.0	fBig5	C668
U+9477.0	fGB	1-4477
U+9477.0	fR/S	 167.18
U+9477.0	fTotalStrokes	 26
U+9477.0	fCangjie	CSJJ
U+9477.0	fFourCorner	81141
U+9477.0	fHakka	  ngiap7
U+9477.0	fCantonese	nip6; lip6 (also)
U+9477.0	fMandarin	nie4; guan4
U+9477.0	fEnglish	[1] tweezers; forceps; pincers [2] pull out; nip
U+9478.0	fUTF8	鑸
U+9478.0	fCNS11643	2-7162
U+9478.0	fBig5	F973
U+9478.0	fGB	E-4356
U+9478.0	fR/S	 167.18
U+9478.0	fTotalStrokes	 26
U+9478.0	fCangjie	CWWG
U+9478.0	fFourCorner	86114
U+9478.0	fCantonese	leoi5
U+9478.0	fMandarin	lei3; xi1
U+9479.0	fUTF8	鑹
U+9479.0	fCNS11643	4-6D62
U+9479.0	fGB	1-6F69
U+9479.0	fR/S	  167.18
U+9479.0	fTotalStrokes	 26
U+9479.0	fCangjie	CJCV
U+9479.0	fFourCorner	83110
U+9479.0	fCantonese	cyun1; cyun3
U+9479.0	fMandarin	cuan4; cuan1
U+947A.0	fUTF8	鑺
U+947A.0	fCNS11643	3-616F
U+947A.0	fGB	5-7634
U+947A.0	fR/S	  167.18
U+947A.0	fTotalStrokes	 26
U+947A.0	fCangjie	CBUG
U+947A.0	fFourCorner	86110
U+947A.0	fCantonese	keoi4
U+947A.0	fMandarin	qu2
U+947B.0	fUTF8	鑻
U+947B.0	fCNS11643	3-6223
U+947B.0	fGB	E-4357
U+947B.0	fR/S	  167.19
U+947B.0	fTotalStrokes	 27
U+947B.0	fCangjie	CDDQ
U+947B.0	fFourCorner	84150
U+947B.0	fCantonese	paan3
U+947B.0	fMandarin	pan4
U+947C.0	fUTF8	鑼
U+947C.0	fCNS11643	1-7D3F
U+947C.0	fBig5	C672
U+947C.0	fGB	1-4260
U+947C.0	fR/S	 167.19
U+947C.0	fTotalStrokes	 27
U+947C.0	fCangjie	CWLG
U+947C.0	fFourCorner	86114
U+947C.0	fHakka	  lo2; lo3
U+947C.0	fCantonese	lo4; lo2 (also)
U+947C.0	fMandarin	luo2
U+947C.0	fEnglish	 gong
U+947D.0	fUTF8	鑽
U+947D.0	fCNS11643	1-7D3D
U+947D.0	fBig5	C670
U+947D.0	fGB	1-576A
U+947D.0	fR/S	 167.19
U+947D.0	fTotalStrokes	 27
U+947D.0	fCangjie	CHUC
U+947D.0	fFourCorner	84186
U+947D.0	fHakka	 zon1; son5; nung5
U+947D.0	fCantonese	zyun3; zyun1 (also); zyun2 (also)
U+947D.0	fMandarin	nie4
U+947D.0	fEnglish	[1] drill; bore; pierce; dig through; penetrate [2] go through; make one's way into [3] gain (profit, positio, etc.) through special favor, contact, relations, etc. [4] study intensively; dig into; bury onself in
U+947D.1	fUTF8	鑽
U+947D.1	fCNS11643	1-7D3D
U+947D.1	fBig5	C670
U+947D.1	fGB	1-576A
U+947D.1	fR/S	 167.19
U+947D.1	fTotalStrokes	 27
U+947D.1	fCangjie	CHUC
U+947D.1	fFourCorner	84186
U+947D.1	fHakka	 zon5
U+947D.1	fMandarin	zuan3; zuan1
U+947D.1	fEnglish	 bore; pierce a hole
U+947D.2	fUTF8	鑽
U+947D.2	fCNS11643	1-7D3D
U+947D.2	fBig5	C670
U+947D.2	fGB	1-576A
U+947D.2	fR/S	 167.19
U+947D.2	fTotalStrokes	 27
U+947D.2	fCangjie	CHUC
U+947D.2	fFourCorner	84186
U+947D.2	fMandarin	zuan4
U+947D.2	fEnglish	[1] diamond; jewel [2] gimlet; awl; auger; borer; drill
U+947E.0	fUTF8	鑾
U+947E.0	fCNS11643	1-7D3E
U+947E.0	fBig5	C671
U+947E.0	fGB	1-7647
U+947E.0	fR/S	 167.19
U+947E.0	fTotalStrokes	 27
U+947E.0	fCangjie	VFC
U+947E.0	fFourCorner	22109
U+947E.0	fHakka	 lan2; lan3
U+947E.0	fCantonese	lyun4
U+947E.0	fMandarin	luan2; luo2
U+947E.0	fEnglish	[1] bells around the neck of a horse; bells hung on the imperial chariot [2] imperial carriage
U+947F.0	fUTF8	鑿
U+947F.0	fCNS11643	1-7D44
U+947F.0	fBig5	C677
U+947F.0	fGB	1-5464
U+947F.0	fR/S	 167.19; 167.20
U+947F.0	fTotalStrokes	 27
U+947F.0	fCangjie	TEC
U+947F.0	fFourCorner	37109
U+947F.0	fHakka	  cok8
U+947F.0	fCantonese	zok6; zok2 (also)
U+947F.0	fMandarin	zuo4; zao4; zu2; cao4; jue2
U+947F.0	fEnglish	[1] an instrument for boring wood [2] bore; pierce through; chisel [3] real; true; acutal; indisputable; authentic; conclusive [4] polish rice [5] make a forced interpretation of text
U+947F.1	fUTF8	鑿
U+947F.1	fCNS11643	1-7D44
U+947F.1	fBig5	C677
U+947F.1	fGB	1-5464
U+947F.1	fR/S	 167.19; 167.20
U+947F.1	fTotalStrokes	 27
U+947F.1	fCangjie	TEC
U+947F.1	fFourCorner	37109
U+947F.1	fMandarin	zao2
U+947F.1	fEnglish	[1] chisel; dig; bore; pierce through [2] scuttle [3] chisel
U+9480.0	fUTF8	钀
U+9480.0	fCNS11643	2-7230
U+9480.0	fBig5	F9C0
U+9480.0	fGB	3-772C
U+9480.0	fR/S	 167.20
U+9480.0	fTotalStrokes	 28
U+9480.0	fCangjie	CYBK
U+9480.0	fFourCorner	83134
U+9480.0	fCantonese	jit6; ngai5
U+9480.0	fMandarin	nie4; yi2; yi3
U+9481.0	fUTF8	钁
U+9481.0	fCNS11643	2-7231
U+9481.0	fBig5	F9C1
U+9481.0	fGB	5-7635
U+9481.0	fR/S	 167.20
U+9481.0	fTotalStrokes	 28
U+9481.0	fCangjie	CBUE
U+9481.0	fFourCorner	86147
U+9481.0	fHakka	 giok7; kiok7
U+9481.0	fCantonese	kyut3 (also); fok3; gwok3
U+9481.0	fMandarin	jue2; luan2
U+9481.0	fEnglish	a mattock; a billhook
U+9482.0	fUTF8	钂
U+9482.0	fCNS11643	2-722F
U+9482.0	fBig5	F9BF
U+9482.0	fGB	3-766E
U+9482.0	fR/S	 167.20
U+9482.0	fTotalStrokes	 28
U+9482.0	fCangjie	CFBF
U+9482.0	fFourCorner	89131
U+9482.0	fCantonese	tong2; dong2
U+9482.0	fMandarin	tang3
U+9483.0	fUTF8	钃
U+9483.0	fCNS11643	2-7237
U+9483.0	fBig5	F9C9
U+9483.0	fGB	3-772A
U+9483.0	fR/S	 167.21
U+9483.0	fTotalStrokes	 29
U+9483.0	fCangjie	CSEI
U+9483.0	fFourCorner	87127
U+9483.0	fCantonese	suk6; zuk1 (also)
U+9483.0	fMandarin	shu3; zhu2
U+9484.0	fUTF8	钄
U+9484.0	fCNS11643	3-6236
U+9484.0	fGB	E-4358
U+9484.0	fR/S	 167.17; 167.21
U+9484.0	fTotalStrokes	 29; 25
U+9484.0	fCangjie	CTAW
U+9484.0	fFourCorner	84120
U+9484.0	fCantonese	laan4
U+9484.0	fMandarin	lan2
U+9485.0	fUTF8	钅
U+9485.0	fGB	0-6E44
U+9485.0	fR/S	 167.0
U+9485.0	fTotalStrokes	 5
U+9485.0	fCangjie	OMP
U+9485.0	fMandarin	jin1zi4pang2; jin1
U+9485.0	fEnglish	simplified form of &#x91D2;
U+9486.0	fUTF8	钆
U+9486.0	fGB	0-6E45
U+9486.0	fR/S	 167.1
U+9486.0	fTotalStrokes	 6
U+9486.0	fCangjie	XCU
U+9486.0	fCantonese	gaa1; gaat3
U+9486.0	fMandarin	ga2; qiu2
U+9486.0	fEnglish	simplified form of &#x91D3;
U+9487.0	fUTF8	钇
U+9487.0	fGB	0-6E46
U+9487.0	fR/S	 167.1
U+9487.0	fTotalStrokes	 6
U+9487.0	fCangjie	XCN
U+9487.0	fCantonese	jyut6; jyut3 (also)
U+9487.0	fMandarin	yi3
U+9487.0	fEnglish	simplified form of &#x91D4;
U+9488.0	fUTF8	针
U+9488.0	fGB	0-556B
U+9488.0	fR/S	 167.2
U+9488.0	fTotalStrokes	 7
U+9488.0	fCangjie	XCJ
U+9488.0	fHakka	 zhim1
U+9488.0	fCantonese	zam1
U+9488.0	fMandarin	zhen1; zhen4
U+9488.0	fEnglish	simplified form of &#x91DD;
U+9489.0	fUTF8	钉
U+9489.0	fGB	0-3624
U+9489.0	fR/S	 167.2
U+9489.0	fTotalStrokes	 7
U+9489.0	fCangjie	XCMN
U+9489.0	fHakka	 dang1; dang5; den1; den5
U+9489.0	fCantonese	ding1; deng1 (also)
U+9489.0	fMandarin	ding1; ding3; ding4; ling2
U+9489.0	fEnglish	simplified form of &#x91D8;
U+948A.0	fUTF8	钊
U+948A.0	fGB	0-6E48
U+948A.0	fR/S	 167.2
U+948A.0	fTotalStrokes	 7
U+948A.0	fCangjie	XCLN
U+948A.0	fHakka	 zhau1
U+948A.0	fCantonese	ciu1
U+948A.0	fMandarin	zhao1; jiao1
U+948A.0	fEnglish	simplified form of &#x91D7;
U+948B.0	fUTF8	钋
U+948B.0	fGB	0-6E47
U+948B.0	fR/S	 167.2
U+948B.0	fTotalStrokes	 7
U+948B.0	fCangjie	XCY
U+948B.0	fCantonese	pok3; puk3 (also)
U+948B.0	fMandarin	po4; pu2; li4
U+948B.0	fEnglish	simplified form of &#x91D9;
U+948C.0	fUTF8	钌
U+948C.0	fGB	0-6E49
U+948C.0	fR/S	 167.2
U+948C.0	fTotalStrokes	 7
U+948C.0	fCangjie	XCNN
U+948C.0	fCantonese	liu5; liu6
U+948C.0	fMandarin	liao3; diao3; liao4
U+948C.0	fEnglish	simplified form of &#x91D5;
U+948D.0	fUTF8	钍
U+948D.0	fGB	0-6E4A
U+948D.0	fR/S	 167.3
U+948D.0	fTotalStrokes	 8
U+948D.0	fCangjie	XCG
U+948D.0	fCantonese	tou2
U+948D.0	fMandarin	tu3
U+948D.0	fEnglish	simplified form of &#x91F7;
U+948E.0	fUTF8	钎
U+948E.0	fGB	0-4725
U+948E.0	fR/S	 167.3
U+948E.0	fTotalStrokes	 8
U+948E.0	fCangjie	XCHJ
U+948E.0	fCantonese	cin1
U+948E.0	fMandarin	qian1
U+948E.0	fEnglish	simplified form of &#x91FA;
U+948F.0	fUTF8	钏
U+948F.0	fGB	0-6E4B
U+948F.0	fR/S	 167.3
U+948F.0	fTotalStrokes	 8
U+948F.0	fCangjie	XCLLL
U+948F.0	fHakka	 chon5
U+948F.0	fCantonese	cyun3; cyun1 (also)
U+948F.0	fMandarin	chuan4; chuan1
U+948F.0	fEnglish	simplified form of &#x91E7;
U+9490.0	fUTF8	钐
U+9490.0	fGB	0-6E4C
U+9490.0	fR/S	 167.3
U+9490.0	fTotalStrokes	 8
U+9490.0	fCangjie	XCHHH
U+9490.0	fHakka	 sam3; cam5
U+9490.0	fCantonese	saam1; saam3; sin3 (also)
U+9490.0	fMandarin	shan4; shan1; xian1
U+9490.0	fEnglish	simplified form of &#x91E4;
U+9491.0	fUTF8	钑
U+9491.0	fGB	8-7E7E
U+9491.0	fR/S	 167.3
U+9491.0	fTotalStrokes	 8
U+9491.0	fCangjie	XCNHE
U+9491.0	fCantonese	zaap6; saap3; kap1 (also)
U+9491.0	fMandarin	ji2; sa4
U+9491.0	fEnglish	spear
U+9492.0	fUTF8	钒
U+9492.0	fGB	0-3730
U+9492.0	fR/S	 167.3
U+9492.0	fTotalStrokes	 8
U+9492.0	fCangjie	XCHNI
U+9492.0	fCantonese	faan4
U+9492.0	fMandarin	fan2; fan3; fan4
U+9492.0	fEnglish	simplified form of &#x91E9;
U+9493.0	fUTF8	钓
U+9493.0	fGB	0-3576
U+9493.0	fR/S	 167.3
U+9493.0	fTotalStrokes	 8
U+9493.0	fCangjie	XCPI
U+9493.0	fHakka	 diau5
U+9493.0	fCantonese	diu3
U+9493.0	fMandarin	diao4
U+9493.0	fEnglish	simplified form of &#x91E3;
U+9494.0	fUTF8	钔
U+9494.0	fGB	0-6E4D
U+9494.0	fR/S	 167.3
U+9494.0	fTotalStrokes	 8
U+9494.0	fCangjie	CLS
U+9494.0	fCantonese	mun4
U+9494.0	fMandarin	men2
U+9494.0	fEnglish	simplified form of &#x9346;
U+9495.0	fUTF8	钕
U+9495.0	fGB	0-6E4F
U+9495.0	fR/S	 167.3
U+9495.0	fTotalStrokes	 8
U+9495.0	fCangjie	XCV
U+9495.0	fCantonese	neoi5; leoi5 (also); neoi4 (also); leoi4 (also)
U+9495.0	fMandarin	nuu3; n3
U+9495.0	fEnglish	simplified form of &#x91F9;
U+9496.0	fUTF8	钖
U+9496.0	fGB	8-2D21
U+9496.0	fR/S	 167.3
U+9496.0	fTotalStrokes	 8
U+9496.0	fCangjie	CNSH
U+9496.0	fCantonese	joeng4
U+9496.0	fMandarin	xi1; xi2; ti4
U+9496.0	fEnglish	frontlet
U+9497.0	fUTF8	钗
U+9497.0	fGB	0-6E4E
U+9497.0	fR/S	 167.3
U+9497.0	fTotalStrokes	 8
U+9497.0	fCangjie	XCEI
U+9497.0	fHakka	 cai1
U+9497.0	fCantonese	caai1; caa1
U+9497.0	fMandarin	chai1
U+9497.0	fEnglish	simplified form of &#x91F5;
U+9498.0	fUTF8	钘
U+9498.0	fGB	8-2D22
U+9498.0	fR/S	 167.4
U+9498.0	fTotalStrokes	 9
U+9498.0	fCangjie	XCMT
U+9498.0	fCantonese	jing4; gin1
U+9498.0	fMandarin	xing2; jian1
U+9498.0	fEnglish	ancient wine vessel
U+9499.0	fUTF8	钙
U+9499.0	fGB	0-3846
U+9499.0	fR/S	 167.4
U+9499.0	fTotalStrokes	 9
U+9499.0	fCangjie	XCMYS
U+9499.0	fCantonese	koi3
U+9499.0	fMandarin	gai4
U+9499.0	fEnglish	simplified form of &#x9223;
U+949A.0	fUTF8	钚
U+949A.0	fGB	0-6E50
U+949A.0	fR/S	 167.4
U+949A.0	fTotalStrokes	 9
U+949A.0	fCangjie	XCMF
U+949A.0	fCantonese	bat1
U+949A.0	fMandarin	bu4; pi1
U+949A.0	fEnglish	simplified form of &#x9208;
U+949B.0	fUTF8	钛
U+949B.0	fGB	0-6E51
U+949B.0	fR/S	 167.4
U+949B.0	fTotalStrokes	 9
U+949B.0	fCangjie	XXCKI
U+949B.0	fCantonese	taai3
U+949B.0	fMandarin	tai4; tai3
U+949B.0	fEnglish	simplified form of &#x9226;
U+949C.0	fUTF8	钜
U+949C.0	fGB	0-6E52
U+949C.0	fR/S	 167.4
U+949C.0	fTotalStrokes	 9
U+949C.0	fCangjie	XCSS
U+949C.0	fHakka	 ki3
U+949C.0	fCantonese	geoi6
U+949C.0	fMandarin	ju4
U+949C.0	fEnglish	simplified form of &#x9245;
U+949D.0	fUTF8	钝
U+949D.0	fGB	0-365B
U+949D.0	fR/S	 167.4
U+949D.0	fTotalStrokes	 9
U+949D.0	fCangjie	XCPU
U+949D.0	fHakka	 tun5; tun2
U+949D.0	fCantonese	deon6
U+949D.0	fMandarin	dun4
U+949D.0	fEnglish	simplified form of &#x920D;
U+949E.0	fUTF8	钞
U+949E.0	fGB	0-332E
U+949E.0	fR/S	 167.4
U+949E.0	fTotalStrokes	 9
U+949E.0	fCangjie	XCFH
U+949E.0	fHakka	 cau1; cau5
U+949E.0	fCantonese	caau1
U+949E.0	fMandarin	chao1; chao3; miao3; chao4
U+949E.0	fEnglish	simplified form of &#x9214;
U+949F.0	fUTF8	钟
U+949F.0	fGB	0-5653
U+949F.0	fR/S	 167.4
U+949F.0	fTotalStrokes	 9
U+949F.0	fCangjie	XCL
U+949F.0	fHakka	 zhung1
U+949F.0	fCantonese	zung1
U+949F.0	fMandarin	zhong1
U+949F.0	fEnglish	simplified form of &#x9418;
U+94A0.0	fUTF8	钠
U+94A0.0	fGB	0-4446
U+94A0.0	fR/S	 167.4
U+94A0.0	fTotalStrokes	 9
U+94A0.0	fCangjie	XCOB
U+94A0.0	fHakka	 nap8
U+94A0.0	fCantonese	naap6; laap6 (also)
U+94A0.0	fMandarin	na4; rui4
U+94A0.0	fEnglish	simplified form of &#x9209;
U+94A1.0	fUTF8	钡
U+94A1.0	fGB	0-3135
U+94A1.0	fR/S	 167.4
U+94A1.0	fTotalStrokes	 9
U+94A1.0	fCangjie	CBO
U+94A1.0	fCantonese	bui3
U+94A1.0	fMandarin	bei4
U+94A1.0	fEnglish	simplified form of &#x92C7;
U+94A2.0	fUTF8	钢
U+94A2.0	fGB	0-3856
U+94A2.0	fR/S	 167.4
U+94A2.0	fTotalStrokes	 9
U+94A2.0	fCangjie	CBK
U+94A2.0	fHakka	 gong5; gong1
U+94A2.0	fCantonese	gong3; gong1 (also)
U+94A2.0	fMandarin	gang1; gang4
U+94A2.0	fEnglish	simplified form of &#x92FC;
U+94A3.0	fUTF8	钣
U+94A3.0	fGB	0-6E53
U+94A3.0	fR/S	 167.4
U+94A3.0	fTotalStrokes	 9
U+94A3.0	fCangjie	XCHE
U+94A3.0	fCantonese	baan2
U+94A3.0	fMandarin	ban3
U+94A3.0	fEnglish	simplified form of &#x9211;
U+94A4.0	fUTF8	钤
U+94A4.0	fGB	0-6E54
U+94A4.0	fR/S	 167.4
U+94A4.0	fTotalStrokes	 9
U+94A4.0	fCangjie	XCOIN
U+94A4.0	fHakka	 kiam2
U+94A4.0	fCantonese	kim4
U+94A4.0	fMandarin	qian2; han2; jin1
U+94A4.0	fEnglish	simplified form of &#x9210;
U+94A5.0	fUTF8	钥
U+94A5.0	fGB	0-543F
U+94A5.0	fR/S	 167.4
U+94A5.0	fTotalStrokes	 9
U+94A5.0	fCangjie	XCB
U+94A5.0	fHakka	 jok7
U+94A5.0	fCantonese	joek6; jyut6
U+94A5.0	fMandarin	yao4; yue4
U+94A5.0	fEnglish	simplified form of &#x9470;
U+94A6.0	fUTF8	钦
U+94A6.0	fGB	0-4755
U+94A6.0	fR/S	 167.4
U+94A6.0	fTotalStrokes	 9
U+94A6.0	fCangjie	XCNO
U+94A6.0	fHakka	 kim1
U+94A6.0	fCantonese	jam1
U+94A6.0	fMandarin	qin1
U+94A6.0	fEnglish	simplified form of &#x6B3D;
U+94A7.0	fUTF8	钧
U+94A7.0	fGB	0-3E7B
U+94A7.0	fR/S	 167.4
U+94A7.0	fTotalStrokes	 9
U+94A7.0	fCangjie	XCPIM
U+94A7.0	fHakka	 giun1
U+94A7.0	fCantonese	gwan1; kwan1 (also)
U+94A7.0	fMandarin	jun1
U+94A7.0	fEnglish	simplified form of &#x921E;
U+94A8.0	fUTF8	钨
U+94A8.0	fGB	0-4E59
U+94A8.0	fR/S	 167.5; 167.4
U+94A8.0	fTotalStrokes	 9
U+94A8.0	fCangjie	CPVM
U+94A8.0	fCantonese	wu1
U+94A8.0	fMandarin	wu1; wu4
U+94A8.0	fEnglish	simplified form of &#x93A2;
U+94A9.0	fUTF8	钩
U+94A9.0	fGB	0-3933
U+94A9.0	fR/S	 167.5; 167.4
U+94A9.0	fTotalStrokes	 9
U+94A9.0	fCangjie	XXXCP
U+94A9.0	fHakka	 geu1
U+94A9.0	fCantonese	ngau1; au1 (also); gau1
U+94A9.0	fMandarin	gou1
U+94A9.0	fEnglish	simplified form of &#x920E;
U+94AA.0	fUTF8	钪
U+94AA.0	fGB	0-6E56
U+94AA.0	fR/S	 167.4
U+94AA.0	fTotalStrokes	 9
U+94AA.0	fCangjie	XCYHN
U+94AA.0	fCantonese	kong3
U+94AA.0	fMandarin	kang4
U+94AA.0	fEnglish	simplified form of &#x9227;
U+94AB.0	fUTF8	钫
U+94AB.0	fGB	0-6E55
U+94AB.0	fR/S	 167.4
U+94AB.0	fTotalStrokes	 9
U+94AB.0	fCangjie	XCYHS
U+94AB.0	fHakka	 fong1
U+94AB.0	fCantonese	fong1
U+94AB.0	fMandarin	fang1
U+94AB.0	fEnglish	simplified form of &#x9201;
U+94AC.0	fUTF8	钬
U+94AC.0	fGB	0-6E58
U+94AC.0	fR/S	 167.4
U+94AC.0	fTotalStrokes	 9
U+94AC.0	fCangjie	XCF
U+94AC.0	fCantonese	fo2
U+94AC.0	fMandarin	huo3; huo2
U+94AC.0	fEnglish	simplified form of &#x9225;
U+94AD.0	fUTF8	钭
U+94AD.0	fGB	0-6E57
U+94AD.0	fR/S	 167.4
U+94AD.0	fTotalStrokes	 9
U+94AD.0	fCangjie	XCYJ
U+94AD.0	fHakka	 deu3
U+94AD.0	fCantonese	dau2
U+94AD.0	fMandarin	tou3; dou4
U+94AD.0	fEnglish	simplified form of &#x9204;
U+94AE.0	fUTF8	钮
U+94AE.0	fGB	0-4525
U+94AE.0	fR/S	 167.4
U+94AE.0	fTotalStrokes	 9
U+94AE.0	fCangjie	XCNG
U+94AE.0	fHakka	 neu3
U+94AE.0	fCantonese	nau2; lau2 (also)
U+94AE.0	fMandarin	niu3; chou3
U+94AE.0	fEnglish	simplified form of &#x9215;
U+94AF.0	fUTF8	钯
U+94AF.0	fGB	0-6E59
U+94AF.0	fR/S	 167.4
U+94AF.0	fTotalStrokes	 9
U+94AF.0	fCangjie	XCAU
U+94AF.0	fHakka	 pa2
U+94AF.0	fCantonese	baa2; paa4; baa1
U+94AF.0	fMandarin	ba3; pa2; ba1; pa1
U+94AF.0	fEnglish	simplified form of &#x9200;
U+94B0.0	fUTF8	钰
U+94B0.0	fGB	0-6E5A
U+94B0.0	fR/S	 167.5
U+94B0.0	fTotalStrokes	 10
U+94B0.0	fCangjie	XCMGI
U+94B0.0	fHakka	 ngiuk8
U+94B0.0	fCantonese	juk6; juk1
U+94B0.0	fMandarin	yu4
U+94B0.0	fEnglish	simplified form of &#x923A;
U+94B1.0	fUTF8	钱
U+94B1.0	fGB	0-472E
U+94B1.0	fR/S	 167.5
U+94B1.0	fTotalStrokes	 10
U+94B1.0	fCangjie	CIJ
U+94B1.0	fHakka	 cien2
U+94B1.0	fCantonese	cin4; cin2 (also); zin2
U+94B1.0	fMandarin	qian2; jian3; zi1
U+94B1.0	fEnglish	simplified form of &#x9322;
U+94B1.1	fUTF8	钱
U+94B1.1	fGB	0-472E
U+94B1.1	fR/S	 167.5
U+94B1.1	fTotalStrokes	 10
U+94B1.1	fCangjie	CIJ
U+94B1.1	fMandarin	qian5
U+94B2.0	fUTF8	钲
U+94B2.0	fGB	0-6E5B
U+94B2.0	fR/S	 167.5
U+94B2.0	fTotalStrokes	 10
U+94B2.0	fCangjie	XCMYM
U+94B2.0	fHakka	 zhin1
U+94B2.0	fCantonese	zing1; zing3
U+94B2.0	fMandarin	zheng1
U+94B2.0	fEnglish	simplified form of &#x9266;
U+94B3.0	fUTF8	钳
U+94B3.0	fGB	0-472F
U+94B3.0	fR/S	 167.5
U+94B3.0	fTotalStrokes	 10
U+94B3.0	fCangjie	XCTM
U+94B3.0	fHakka	 kiam2
U+94B3.0	fCantonese	kim4; kim2 (also)
U+94B3.0	fMandarin	qian2
U+94B3.0	fEnglish	simplified form of &#x9257;
U+94B4.0	fUTF8	钴
U+94B4.0	fGB	0-6E5C
U+94B4.0	fR/S	 167.5
U+94B4.0	fTotalStrokes	 10
U+94B4.0	fCangjie	XCJR
U+94B4.0	fHakka	 gu3
U+94B4.0	fCantonese	gu2; gwu2 (also); gu1 (also); gwu1 (also)
U+94B4.0	fMandarin	gu1; gu4; gu2; hu2; gu3
U+94B4.0	fEnglish	simplified form of &#x9237;
U+94B5.0	fUTF8	钵
U+94B5.0	fGB	0-3227
U+94B5.0	fR/S	 167.5
U+94B5.0	fTotalStrokes	 10
U+94B5.0	fCangjie	XCDM
U+94B5.0	fHakka	 bat7
U+94B5.0	fCantonese	but3
U+94B5.0	fMandarin	bo1
U+94B5.0	fEnglish	simplified form of &#x9262;
U+94B6.0	fUTF8	钶
U+94B6.0	fGB	0-6E5D
U+94B6.0	fR/S	 167.5
U+94B6.0	fTotalStrokes	 10
U+94B6.0	fCangjie	XCMNR
U+94B6.0	fCantonese	ko1; o1
U+94B6.0	fMandarin	ke1; e1; e3
U+94B6.0	fEnglish	simplified form of &#x9233;
U+94B7.0	fUTF8	钷
U+94B7.0	fGB	0-6E5E
U+94B7.0	fR/S	 167.5
U+94B7.0	fTotalStrokes	 10
U+94B7.0	fCangjie	XCSR
U+94B7.0	fCantonese	po2
U+94B7.0	fMandarin	po3; po1
U+94B7.0	fEnglish	simplified form of &#x9255;
U+94B8.0	fUTF8	钸
U+94B8.0	fGB	0-6E5F
U+94B8.0	fR/S	 167.5
U+94B8.0	fTotalStrokes	 10
U+94B8.0	fCangjie	XCKLB
U+94B8.0	fCantonese	bou1; bou3
U+94B8.0	fMandarin	bu4; bu1
U+94B8.0	fEnglish	simplified form of &#x923D;
U+94B9.0	fUTF8	钹
U+94B9.0	fGB	0-6E60
U+94B9.0	fR/S	 167.5
U+94B9.0	fTotalStrokes	 10
U+94B9.0	fCangjie	XCIKE
U+94B9.0	fHakka	 pat8
U+94B9.0	fCantonese	bat6; but6 (also)
U+94B9.0	fMandarin	ba2; bo2
U+94B9.0	fEnglish	simplified form of &#x9238;
U+94BA.0	fUTF8	钺
U+94BA.0	fGB	0-6E61
U+94BA.0	fR/S	 167.5
U+94BA.0	fTotalStrokes	 10
U+94BA.0	fCangjie	XCIV
U+94BA.0	fHakka	 jet8
U+94BA.0	fCantonese	jyut6
U+94BA.0	fMandarin	yue4; hui4
U+94BA.0	fEnglish	simplified form of &#x925E;
U+94BB.0	fUTF8	钻
U+94BB.0	fGB	0-576A
U+94BB.0	fR/S	 167.5
U+94BB.0	fTotalStrokes	 10
U+94BB.0	fCangjie	XCYR
U+94BB.0	fHakka	 zon5; zon1; nung5
U+94BB.0	fCantonese	zyun3; zyun1 (also); zyun2 (also); cim1
U+94BB.0	fMandarin	zuan1; zuan4; nie4; zuan3
U+94BB.0	fEnglish	simplified form of &#x947D;
U+94BC.0	fUTF8	钼
U+94BC.0	fGB	0-6E62
U+94BC.0	fR/S	 167.5
U+94BC.0	fTotalStrokes	 10
U+94BC.0	fCangjie	XCBU
U+94BC.0	fCantonese	muk6
U+94BC.0	fMandarin	mu4
U+94BC.0	fEnglish	simplified form of &#x926C;
U+94BD.0	fUTF8	钽
U+94BD.0	fGB	0-6E63
U+94BD.0	fR/S	 167.5
U+94BD.0	fTotalStrokes	 10
U+94BD.0	fCangjie	XCAM
U+94BD.0	fCantonese	daan3 (also); taan2
U+94BD.0	fMandarin	tan3; dan4
U+94BD.0	fEnglish	simplified form of &#x926D;
U+94BE.0	fUTF8	钾
U+94BE.0	fGB	0-3C58
U+94BE.0	fR/S	 167.5
U+94BE.0	fTotalStrokes	 10
U+94BE.0	fCangjie	XCWL
U+94BE.0	fCantonese	gaap3
U+94BE.0	fMandarin	jia3; he2; ge2
U+94BE.0	fEnglish	simplified form of &#x9240;
U+94BF.0	fUTF8	钿
U+94BF.0	fGB	0-6E64
U+94BF.0	fR/S	 167.5
U+94BF.0	fTotalStrokes	 10
U+94BF.0	fCangjie	XCW
U+94BF.0	fHakka	 tien2
U+94BF.0	fCantonese	tin4; din6 (also)
U+94BF.0	fMandarin	dian4; tian2
U+94BF.0	fEnglish	simplified form of &#x923F;
U+94C0.0	fUTF8	铀
U+94C0.0	fGB	0-534B
U+94C0.0	fR/S	 167.5
U+94C0.0	fTotalStrokes	 10
U+94C0.0	fCangjie	XCLW
U+94C0.0	fCantonese	jau4; jau2 (also)
U+94C0.0	fMandarin	you2; zhou4
U+94C0.0	fEnglish	simplified form of &#x923E;
U+94C1.0	fUTF8	铁
U+94C1.0	fGB	0-4C7A
U+94C1.0	fR/S	 167.5
U+94C1.0	fTotalStrokes	 10
U+94C1.0	fCangjie	XCHQO
U+94C1.0	fHakka	 tiet7
U+94C1.0	fCantonese	tit3
U+94C1.0	fMandarin	tie3; die2; juan1
U+94C1.0	fEnglish	simplified form of &#x9435;
U+94C2.0	fUTF8	铂
U+94C2.0	fGB	0-322C
U+94C2.0	fR/S	 167.5
U+94C2.0	fTotalStrokes	 10
U+94C2.0	fCangjie	XCHA
U+94C2.0	fHakka	 pak8; pok8
U+94C2.0	fCantonese	bok6
U+94C2.0	fMandarin	bo2
U+94C2.0	fEnglish	simplified form of &#x9251;
U+94C3.0	fUTF8	铃
U+94C3.0	fGB	0-4165
U+94C3.0	fR/S	 167.5
U+94C3.0	fTotalStrokes	 10
U+94C3.0	fCangjie	XCOII
U+94C3.0	fHakka	 lang2
U+94C3.0	fCantonese	ling4; ling1 (also); ling2 (also)
U+94C3.0	fMandarin	ling2
U+94C3.0	fEnglish	simplified form of &#x9234;
U+94C4.0	fUTF8	铄
U+94C4.0	fGB	0-6E65
U+94C4.0	fR/S	 167.5
U+94C4.0	fTotalStrokes	 10
U+94C4.0	fCangjie	CHVD
U+94C4.0	fHakka	 shok7
U+94C4.0	fCantonese	soek3
U+94C4.0	fMandarin	shuo4; gong3; kuang4; li4; yao4; yue4
U+94C4.0	fEnglish	simplified form of &#x9460;
U+94C5.0	fUTF8	铅
U+94C5.0	fGB	0-4726
U+94C5.0	fR/S	 167.5
U+94C5.0	fTotalStrokes	 10
U+94C5.0	fCangjie	XCHNR
U+94C5.0	fHakka	 jen2
U+94C5.0	fCantonese	jyun4
U+94C5.0	fMandarin	qian1; yan2
U+94C5.0	fEnglish	simplified form of &#x925B;
U+94C6.0	fUTF8	铆
U+94C6.0	fGB	0-432D
U+94C6.0	fR/S	 167.5
U+94C6.0	fTotalStrokes	 10
U+94C6.0	fCangjie	XCHHL
U+94C6.0	fCantonese	maau5
U+94C6.0	fMandarin	liu3; mao3
U+94C6.0	fEnglish	simplified form of &#x925A;
U+94C7.0	fUTF8	铇
U+94C7.0	fGB	8-2C7E
U+94C7.0	fR/S	 167.5
U+94C7.0	fTotalStrokes	 10
U+94C7.0	fCangjie	XCPRU
U+94C7.0	fCantonese	paau4
U+94C7.0	fMandarin	bao4
U+94C7.0	fEnglish	carpenter's plane
U+94C8.0	fUTF8	铈
U+94C8.0	fGB	0-6E66
U+94C8.0	fR/S	 167.5
U+94C8.0	fTotalStrokes	 10
U+94C8.0	fCangjie	XCYLB
U+94C8.0	fCantonese	si5
U+94C8.0	fMandarin	shi4
U+94C8.0	fEnglish	simplified form of &#x9230;
U+94C9.0	fUTF8	铉
U+94C9.0	fGB	0-6E67
U+94C9.0	fR/S	 167.5
U+94C9.0	fTotalStrokes	 10
U+94C9.0	fCangjie	XCYVI
U+94C9.0	fHakka	 hien3
U+94C9.0	fCantonese	jyun5
U+94C9.0	fMandarin	xuan4; jian1; juan1; jyong1; xian2
U+94C9.0	fEnglish	simplified form of &#x9249;
U+94CA.0	fUTF8	铊
U+94CA.0	fGB	0-6E68
U+94CA.0	fR/S	 167.5
U+94CA.0	fTotalStrokes	 10
U+94CA.0	fCangjie	XCJP
U+94CA.0	fHakka	 to2
U+94CA.0	fCantonese	taa1; to4; to2 (also); si1; se4 (also)
U+94CA.0	fMandarin	she2; shi1; tuo2; ta1; shi4
U+94CA.0	fEnglish	simplified form of &#x9248;
U+94CB.0	fUTF8	铋
U+94CB.0	fGB	0-6E69
U+94CB.0	fR/S	 167.5
U+94CB.0	fTotalStrokes	 10
U+94CB.0	fCangjie	XCPH
U+94CB.0	fCantonese	bit1 (also); bei3
U+94CB.0	fMandarin	bi4
U+94CB.0	fEnglish	simplified form of &#x924D;
U+94CC.0	fUTF8	铌
U+94CC.0	fGB	0-6E6A
U+94CC.0	fR/S	 167.5
U+94CC.0	fTotalStrokes	 10
U+94CC.0	fCangjie	XCSP
U+94CC.0	fCantonese	nei4; lei4 (also); nei6 (also); lei6 (also)
U+94CC.0	fMandarin	ni3; ni2; ni4
U+94CC.0	fEnglish	simplified form of &#x922E;
U+94CD.0	fUTF8	铍
U+94CD.0	fGB	0-6E6B
U+94CD.0	fR/S	 167.5
U+94CD.0	fTotalStrokes	 10
U+94CD.0	fCangjie	XCDHE
U+94CD.0	fHakka	 pi1
U+94CD.0	fCantonese	pei1; pei4
U+94CD.0	fMandarin	pi2; pi1
U+94CD.0	fEnglish	simplified form of &#x9239;
U+94CE.0	fUTF8	铎
U+94CE.0	fGB	0-6E6C
U+94CE.0	fR/S	 167.5
U+94CE.0	fTotalStrokes	 10
U+94CE.0	fCangjie	CEQ
U+94CE.0	fHakka	 tok8
U+94CE.0	fCantonese	dok6
U+94CE.0	fMandarin	duo2
U+94CE.0	fEnglish	simplified form of &#x9438;
U+94CF.0	fUTF8	铏
U+94CF.0	fGB	8-2D23
U+94CF.0	fR/S	 167.6
U+94CF.0	fTotalStrokes	 11
U+94CF.0	fCangjie	XCMTN
U+94CF.0	fCantonese	jing4
U+94CF.0	fMandarin	xing2
U+94CF.0	fEnglish	sacrificial cauldron
U+94D0.0	fUTF8	铐
U+94D0.0	fGB	0-6E6D
U+94D0.0	fR/S	 167.6
U+94D0.0	fTotalStrokes	 11
U+94D0.0	fCangjie	XCJKS
U+94D0.0	fCantonese	kaau3; kau3 (also)
U+94D0.0	fMandarin	kao4; kao3
U+94D0.0	fEnglish	simplified form of &#x92AC;
U+94D1.0	fUTF8	铑
U+94D1.0	fGB	0-6E6E
U+94D1.0	fR/S	 167.6
U+94D1.0	fTotalStrokes	 11
U+94D1.0	fCangjie	XCJKP
U+94D1.0	fCantonese	lou5
U+94D1.0	fMandarin	lao3
U+94D1.0	fEnglish	simplified form of &#x92A0;
U+94D2.0	fUTF8	铒
U+94D2.0	fGB	0-6E6F
U+94D2.0	fR/S	 167.6
U+94D2.0	fTotalStrokes	 11
U+94D2.0	fCangjie	XCSJ
U+94D2.0	fCantonese	ji5; ji6
U+94D2.0	fMandarin	er4; er3
U+94D2.0	fEnglish	simplified form of &#x927A;
U+94D3.0	fUTF8	铓
U+94D3.0	fGB	8-2D24
U+94D3.0	fR/S	 167.7; 167.6
U+94D3.0	fTotalStrokes	 11
U+94D3.0	fCangjie	XCTYV
U+94D3.0	fCantonese	mong4
U+94D3.0	fMandarin	mang2
U+94D3.0	fEnglish	point of sword; sharp point
U+94D4.0	fUTF8	铔
U+94D4.0	fGB	8-2D25
U+94D4.0	fR/S	 167.8; 167.6
U+94D4.0	fTotalStrokes	 11
U+94D4.0	fCangjie	CMTC
U+94D4.0	fCantonese	aa1; ngaa1 (also); aa3
U+94D4.0	fMandarin	ya4
U+94D4.0	fEnglish	soft steel; ammonium
U+94D5.0	fUTF8	铕
U+94D5.0	fGB	0-6E70
U+94D5.0	fR/S	 167.6
U+94D5.0	fTotalStrokes	 11
U+94D5.0	fCangjie	XCKB
U+94D5.0	fCantonese	jau5
U+94D5.0	fMandarin	you3
U+94D5.0	fEnglish	simplified form of &#x92AA;
U+94D6.0	fUTF8	铖
U+94D6.0	fGB	0-6E71
U+94D6.0	fR/S	 167.6
U+94D6.0	fTotalStrokes	 11
U+94D6.0	fCangjie	XCIHS
U+94D6.0	fCantonese	sing4
U+94D6.0	fMandarin	cheng2
U+94D6.0	fEnglish	simplified form of &#x92EE;
U+94D7.0	fUTF8	铗
U+94D7.0	fGB	0-6E72
U+94D7.0	fR/S	 167.6
U+94D7.0	fTotalStrokes	 11
U+94D7.0	fCangjie	CKT
U+94D7.0	fHakka	 giap7
U+94D7.0	fCantonese	gaap3
U+94D7.0	fMandarin	jia2
U+94D7.0	fEnglish	simplified form of &#x92CF;
U+94D8.0	fUTF8	铘
U+94D8.0	fGB	0-6E74
U+94D8.0	fR/S	 167.7; 167.6
U+94D8.0	fTotalStrokes	 11
U+94D8.0	fCangjie	XCMHL
U+94D8.0	fHakka	 ja2
U+94D8.0	fCantonese	je4
U+94D8.0	fMandarin	ye2
U+94D8.0	fEnglish	simplified form of &#x92E3;
U+94D9.0	fUTF8	铙
U+94D9.0	fGB	0-6E73
U+94D9.0	fR/S	 167.6
U+94D9.0	fTotalStrokes	 11
U+94D9.0	fCangjie	CJPU
U+94D9.0	fHakka	 ngiau2; nau2
U+94D9.0	fCantonese	naau4; laau4 (also)
U+94D9.0	fMandarin	nao2; dui1; dui4; nao4
U+94D9.0	fEnglish	simplified form of &#x9403;
U+94DA.0	fUTF8	铚
U+94DA.0	fGB	8-2C60
U+94DA.0	fR/S	 167.6
U+94DA.0	fTotalStrokes	 11
U+94DA.0	fCangjie	XCMIG
U+94DA.0	fCantonese	zat6
U+94DA.0	fMandarin	zhi4
U+94DA.0	fEnglish	a sickle
U+94DB.0	fUTF8	铛
U+94DB.0	fGB	0-6E75
U+94DB.0	fR/S	 167.6
U+94DB.0	fTotalStrokes	 11
U+94DB.0	fCangjie	CFSM
U+94DB.0	fHakka	 dong1
U+94DB.0	fCantonese	dong1; caang1; cang1
U+94DB.0	fMandarin	dang1; tang1; cheng1
U+94DB.0	fEnglish	simplified form of &#x943A;
U+94DC.0	fUTF8	铜
U+94DC.0	fGB	0-4D2D
U+94DC.0	fR/S	 167.6
U+94DC.0	fTotalStrokes	 11
U+94DC.0	fCangjie	XCBMR
U+94DC.0	fHakka	 tung2
U+94DC.0	fCantonese	tung4
U+94DC.0	fMandarin	tong2
U+94DC.0	fEnglish	simplified form of &#x9285;
U+94DD.0	fUTF8	铝
U+94DD.0	fGB	0-4241
U+94DD.0	fR/S	 167.6
U+94DD.0	fTotalStrokes	 11
U+94DD.0	fCangjie	XCRR
U+94DD.0	fCantonese	leoi5
U+94DD.0	fMandarin	l3; luu3; lu3
U+94DD.0	fEnglish	simplified form of &#x92C1;
U+94DE.0	fUTF8	铞
U+94DE.0	fGB	0-6E76
U+94DE.0	fR/S	 167.6
U+94DE.0	fTotalStrokes	 11
U+94DE.0	fCangjie	XCRLB
U+94DE.0	fCantonese	diu3
U+94DE.0	fMandarin	diao4
U+94DE.0	fEnglish	simplified form of &#x92B1;
U+94DF.0	fUTF8	铟
U+94DF.0	fGB	0-6E77
U+94DF.0	fR/S	 167.6
U+94DF.0	fTotalStrokes	 11
U+94DF.0	fCangjie	XCWK
U+94DF.0	fCantonese	jan1
U+94DF.0	fMandarin	yin1
U+94DF.0	fEnglish	simplified form of &#x92A6;
U+94E0.0	fUTF8	铠
U+94E0.0	fGB	0-6E78
U+94E0.0	fR/S	 167.6
U+94E0.0	fTotalStrokes	 11
U+94E0.0	fCangjie	CUSU
U+94E0.0	fHakka	 koi3
U+94E0.0	fCantonese	hoi2
U+94E0.0	fMandarin	kai3; kai4
U+94E0.0	fEnglish	simplified form of &#x93A7;
U+94E1.0	fUTF8	铡
U+94E1.0	fGB	0-5521
U+94E1.0	fR/S	 167.6
U+94E1.0	fTotalStrokes	 11
U+94E1.0	fCangjie	CBON
U+94E1.0	fCantonese	zaap6; zaat3
U+94E1.0	fMandarin	zha2
U+94E1.0	fEnglish	simplified form of &#x9358;
U+94E2.0	fUTF8	铢
U+94E2.0	fGB	0-6E79
U+94E2.0	fR/S	 167.6
U+94E2.0	fTotalStrokes	 11
U+94E2.0	fCangjie	XCHJD
U+94E2.0	fHakka	 shu2
U+94E2.0	fCantonese	zyu1
U+94E2.0	fMandarin	zhu1; shu1
U+94E2.0	fEnglish	simplified form of &#x9296;
U+94E3.0	fUTF8	铣
U+94E3.0	fGB	0-4F33
U+94E3.0	fR/S	 167.6
U+94E3.0	fTotalStrokes	 11
U+94E3.0	fCangjie	XCHGU
U+94E3.0	fHakka	 sien3
U+94E3.0	fCantonese	sin2
U+94E3.0	fMandarin	xian3; xi3
U+94E3.0	fEnglish	simplified form of &#x9291;
U+94E4.0	fUTF8	铤
U+94E4.0	fGB	0-6E7A
U+94E4.0	fR/S	 167.7; 167.6
U+94E4.0	fTotalStrokes	 11
U+94E4.0	fCangjie	XCNKG
U+94E4.0	fCantonese	ding3; ting5; ding6 (also); jin4
U+94E4.0	fMandarin	ting3; ding4
U+94E4.0	fEnglish	simplified form of &#x92CC;
U+94E5.0	fUTF8	铥
U+94E5.0	fGB	0-6E7B
U+94E5.0	fR/S	 167.6
U+94E5.0	fTotalStrokes	 11
U+94E5.0	fCangjie	XCHGI
U+94E5.0	fCantonese	diu1
U+94E5.0	fMandarin	diu1
U+94E5.0	fEnglish	simplified form of &#x92A9;
U+94E6.0	fUTF8	铦
U+94E6.0	fGB	8-2D26
U+94E6.0	fR/S	 167.6
U+94E6.0	fTotalStrokes	 11
U+94E6.0	fCangjie	XCHJR
U+94E6.0	fCantonese	cim1
U+94E6.0	fMandarin	xian1
U+94E6.0	fEnglish	sharp, keen-edged; a kind of hoe
U+94E7.0	fUTF8	铧
U+94E7.0	fGB	0-6E7C
U+94E7.0	fR/S	 167.6
U+94E7.0	fTotalStrokes	 11
U+94E7.0	fCangjie	COPJ
U+94E7.0	fHakka	 fa2
U+94E7.0	fCantonese	waa4
U+94E7.0	fMandarin	hua2
U+94E7.0	fEnglish	simplified form of &#x93F5;
U+94E8.0	fUTF8	铨
U+94E8.0	fGB	0-6E7D
U+94E8.0	fR/S	 167.6
U+94E8.0	fTotalStrokes	 11
U+94E8.0	fCangjie	XCOMG
U+94E8.0	fHakka	 cien2
U+94E8.0	fCantonese	cyun4
U+94E8.0	fMandarin	quan2
U+94E8.0	fEnglish	simplified form of &#x9293;
U+94E9.0	fUTF8	铩
U+94E9.0	fGB	0-6F21
U+94E9.0	fR/S	 167.6
U+94E9.0	fTotalStrokes	 11
U+94E9.0	fCangjie	CKD
U+94E9.0	fHakka	 sai5
U+94E9.0	fCantonese	saat3
U+94E9.0	fMandarin	sha1; se4; shai4; she4
U+94E9.0	fEnglish	simplified form of &#x93A9;
U+94EA.0	fUTF8	铪
U+94EA.0	fGB	0-6E7E
U+94EA.0	fR/S	 167.6
U+94EA.0	fTotalStrokes	 11
U+94EA.0	fCangjie	XCOMR
U+94EA.0	fCantonese	haa1; hap6
U+94EA.0	fMandarin	jia2; ha1; ge2; jia1; he2; ke1
U+94EA.0	fEnglish	simplified form of &#x927F;
U+94EB.0	fUTF8	铫
U+94EB.0	fGB	0-6F22
U+94EB.0	fR/S	 167.6
U+94EB.0	fTotalStrokes	 11
U+94EB.0	fCangjie	XCLMO
U+94EB.0	fHakka	 tiau2; jau2; tiau5
U+94EB.0	fCantonese	diu6; jiu4; tiu4
U+94EB.0	fMandarin	yao2; tiao2; qiao1; tiao1; tiao4; yao4; zhao4
U+94EB.0	fEnglish	simplified form of &#x929A;
U+94EC.0	fUTF8	铬
U+94EC.0	fGB	0-3875
U+94EC.0	fR/S	 167.6
U+94EC.0	fTotalStrokes	 11
U+94EC.0	fCangjie	XCHER
U+94EC.0	fCantonese	gok3 (also); lok3; lok6
U+94EC.0	fMandarin	ge4; luo4; ge2
U+94EC.0	fEnglish	simplified form of &#x927B;
U+94ED.0	fUTF8	铭
U+94ED.0	fGB	0-437A
U+94ED.0	fR/S	 167.6
U+94ED.0	fTotalStrokes	 11
U+94ED.0	fCangjie	XCNIR
U+94ED.0	fHakka	 men2; miang2
U+94ED.0	fCantonese	ming4; ming2 (also); ming5 (also)
U+94ED.0	fMandarin	ming2
U+94ED.0	fEnglish	simplified form of &#x9298;
U+94EE.0	fUTF8	铮
U+94EE.0	fGB	0-6F23
U+94EE.0	fR/S	 167.8; 167.6
U+94EE.0	fTotalStrokes	 11
U+94EE.0	fCangjie	XCNSD
U+94EE.0	fHakka	 zen1
U+94EE.0	fCantonese	zang1; zaang1 (also)
U+94EE.0	fMandarin	zheng1; zheng4; cheng1; jian3; qian2
U+94EE.0	fEnglish	simplified form of &#x931A;
U+94EF.0	fUTF8	铯
U+94EF.0	fGB	0-6F24
U+94EF.0	fR/S	 167.6
U+94EF.0	fTotalStrokes	 11
U+94EF.0	fCangjie	XCNAU
U+94EF.0	fCantonese	sik1
U+94EF.0	fMandarin	se4
U+94EF.0	fEnglish	simplified form of &#x92AB;
U+94F0.0	fUTF8	铰
U+94F0.0	fGB	0-3D42
U+94F0.0	fR/S	 167.6
U+94F0.0	fTotalStrokes	 11
U+94F0.0	fCangjie	XCYCK
U+94F0.0	fHakka	 gau5; gau3
U+94F0.0	fCantonese	gaau2; gaau3 (also)
U+94F0.0	fMandarin	jiao3; jia3; jiao1; jiao4
U+94F0.0	fEnglish	simplified form of &#x9278;
U+94F1.0	fUTF8	铱
U+94F1.0	fGB	0-523F
U+94F1.0	fR/S	 167.6
U+94F1.0	fTotalStrokes	 11
U+94F1.0	fCangjie	XCYHV
U+94F1.0	fCantonese	ji1
U+94F1.0	fMandarin	yi1; yi3
U+94F1.0	fEnglish	simplified form of &#x92A5;
U+94F2.0	fUTF8	铲
U+94F2.0	fGB	0-3279
U+94F2.0	fR/S	 167.6
U+94F2.0	fTotalStrokes	 11
U+94F2.0	fCangjie	CYTH
U+94F2.0	fHakka	 can3
U+94F2.0	fCantonese	caan2
U+94F2.0	fMandarin	chan3; chan4
U+94F2.0	fEnglish	simplified form of &#x93DF;
U+94F3.0	fUTF8	铳
U+94F3.0	fGB	0-6F25
U+94F3.0	fR/S	 167.5; 167.6
U+94F3.0	fTotalStrokes	 11
U+94F3.0	fCangjie	XXXCY
U+94F3.0	fHakka	 chung5
U+94F3.0	fCantonese	cung3
U+94F3.0	fMandarin	chong4; chu4
U+94F3.0	fEnglish	simplified form of &#x9283;
U+94F4.0	fUTF8	铴
U+94F4.0	fGB	0-6F26
U+94F4.0	fR/S	 167.7; 167.6
U+94F4.0	fTotalStrokes	 11
U+94F4.0	fCangjie	CENH
U+94F4.0	fCantonese	tong1; tong2; tong3
U+94F4.0	fMandarin	tang4; tang1
U+94F4.0	fEnglish	simplified form of &#x940B;
U+94F5.0	fUTF8	铵
U+94F5.0	fGB	0-6F27
U+94F5.0	fR/S	 167.6
U+94F5.0	fTotalStrokes	 11
U+94F5.0	fCangjie	XCJV
U+94F5.0	fCantonese	on1; ngon1 (also); on2 (also); ngon2 (also)
U+94F5.0	fMandarin	an1; an3
U+94F5.0	fEnglish	simplified form of &#x92A8;
U+94F6.0	fUTF8	银
U+94F6.0	fGB	0-5278
U+94F6.0	fR/S	 167.6
U+94F6.0	fTotalStrokes	 11
U+94F6.0	fCangjie	XCAV
U+94F6.0	fHakka	 ngiun2
U+94F6.0	fCantonese	ngan4; an4 (also); ngan2 (also); an2 (also)
U+94F6.0	fMandarin	yin2
U+94F6.0	fEnglish	simplified form of &#x9280;
U+94F7.0	fUTF8	铷
U+94F7.0	fGB	0-6F28
U+94F7.0	fR/S	 167.6
U+94F7.0	fTotalStrokes	 11
U+94F7.0	fCangjie	XCVR
U+94F7.0	fCantonese	jyu4; jyu6 (also)
U+94F7.0	fMandarin	ru3; ru2; ru4
U+94F7.0	fEnglish	simplified form of &#x92A3;
U+94F8.0	fUTF8	铸
U+94F8.0	fGB	0-567D
U+94F8.0	fR/S	 167.7
U+94F8.0	fTotalStrokes	 12
U+94F8.0	fCangjie	XCQKI
U+94F8.0	fHakka	 zhu5
U+94F8.0	fCantonese	zyu3
U+94F8.0	fMandarin	zhu4; jian4; zhou4
U+94F8.0	fEnglish	simplified form of &#x9444;
U+94F9.0	fUTF8	铹
U+94F9.0	fGB	0-6F29
U+94F9.0	fR/S	 167.7
U+94F9.0	fTotalStrokes	 12
U+94F9.0	fCangjie	CTBS
U+94F9.0	fCantonese	lou4
U+94F9.0	fMandarin	lao2
U+94F9.0	fEnglish	simplified form of &#x9412;
U+94FA.0	fUTF8	铺
U+94FA.0	fGB	0-464C
U+94FA.0	fR/S	 167.7
U+94FA.0	fTotalStrokes	 12
U+94FA.0	fCangjie	XCIJB
U+94FA.0	fHakka	 pu5; pu1
U+94FA.0	fCantonese	pou1; pou3; pou2 (also)
U+94FA.0	fMandarin	pu1; bu1; fu1
U+94FA.0	fEnglish	simplified form of &#x92EA;
U+94FA.1	fUTF8	铺
U+94FA.1	fGB	0-464C
U+94FA.1	fR/S	 167.7
U+94FA.1	fTotalStrokes	 12
U+94FA.1	fCangjie	XCIJB
U+94FA.1	fMandarin	pu4
U+94FA.2	fUTF8	铺
U+94FA.2	fGB	0-464C
U+94FA.2	fR/S	 167.7
U+94FA.2	fTotalStrokes	 12
U+94FA.2	fCangjie	XCIJB
U+94FA.2	fMandarin	pu5
U+94FB.0	fUTF8	铻
U+94FB.0	fGB	8-2E7C
U+94FB.0	fR/S	 167.7
U+94FB.0	fTotalStrokes	 12
U+94FB.0	fCangjie	XCMMR
U+94FB.0	fCantonese	jyu5; ng4
U+94FB.0	fMandarin	wu2; yu3
U+94FB.0	fEnglish	misfit; disordant; music instrument; hoe
U+94FC.0	fUTF8	铼
U+94FC.0	fGB	0-6F2A
U+94FC.0	fR/S	 167.7
U+94FC.0	fTotalStrokes	 12
U+94FC.0	fCangjie	CDT
U+94FC.0	fCantonese	loi4
U+94FC.0	fMandarin	lai2
U+94FC.0	fEnglish	simplified form of &#x9338;
U+94FD.0	fUTF8	铽
U+94FD.0	fGB	0-6F2B
U+94FD.0	fR/S	 167.7
U+94FD.0	fTotalStrokes	 12
U+94FD.0	fCangjie	XCIPP
U+94FD.0	fCantonese	tik1
U+94FD.0	fMandarin	te4
U+94FD.0	fEnglish	simplified form of &#x92F1;
U+94FE.0	fUTF8	链
U+94FE.0	fGB	0-4134
U+94FE.0	fR/S	 167.7
U+94FE.0	fTotalStrokes	 12
U+94FE.0	fCangjie	CYKQ
U+94FE.0	fHakka	 lien2; lien5
U+94FE.0	fCantonese	lin6; lin2 (also)
U+94FE.0	fMandarin	lian4; lian2; liu2
U+94FE.0	fEnglish	simplified form of &#x93C8;
U+94FF.0	fUTF8	铿
U+94FF.0	fGB	0-6F2C
U+94FF.0	fR/S	 167.7
U+94FF.0	fTotalStrokes	 12
U+94FF.0	fCangjie	CLEG
U+94FF.0	fHakka	 ken1; gen1; gien1
U+94FF.0	fCantonese	hang1
U+94FF.0	fMandarin	keng1; ao2
U+94FF.0	fEnglish	simplified form of &#x93D7;
U+9500.0	fUTF8	销
U+9500.0	fGB	0-4F7A
U+9500.0	fR/S	 167.7
U+9500.0	fTotalStrokes	 12
U+9500.0	fCangjie	XCFB
U+9500.0	fHakka	 siau1
U+9500.0	fCantonese	siu1
U+9500.0	fMandarin	xiao1
U+9500.0	fEnglish	simplified form of &#x92B7;
U+9501.0	fUTF8	锁
U+9501.0	fGB	0-4B78
U+9501.0	fR/S	 167.7
U+9501.0	fTotalStrokes	 12
U+9501.0	fCangjie	CFBO
U+9501.0	fHakka	 so3
U+9501.0	fCantonese	so2
U+9501.0	fMandarin	suo3
U+9501.0	fEnglish	simplified form of &#x9396;
U+9502.0	fUTF8	锂
U+9502.0	fGB	0-6F2E
U+9502.0	fR/S	 167.7
U+9502.0	fTotalStrokes	 12
U+9502.0	fCangjie	XCWG
U+9502.0	fCantonese	lei5
U+9502.0	fMandarin	li3
U+9502.0	fEnglish	simplified form of &#x92F0;
U+9503.0	fUTF8	锃
U+9503.0	fGB	0-6F2D
U+9503.0	fR/S	 167.7
U+9503.0	fTotalStrokes	 12
U+9503.0	fCangjie	XCRMG
U+9503.0	fCantonese	caang3; cang3 (also); cing4
U+9503.0	fMandarin	zeng4; zheng4
U+9503.0	fEnglish	simplified form of &#x92E5;
U+9504.0	fUTF8	锄
U+9504.0	fGB	0-337A
U+9504.0	fR/S	 167.7
U+9504.0	fTotalStrokes	 12
U+9504.0	fCangjie	XCBMS
U+9504.0	fHakka	 cih2; co2; ts'io2
U+9504.0	fCantonese	co4
U+9504.0	fMandarin	chu2; ju3; zhu4
U+9504.0	fEnglish	simplified form of &#x92E4;
U+9505.0	fUTF8	锅
U+9505.0	fGB	0-3978
U+9505.0	fR/S	 167.7
U+9505.0	fTotalStrokes	 12
U+9505.0	fCangjie	CROB
U+9505.0	fHakka	 ko1; vok8
U+9505.0	fCantonese	wo1
U+9505.0	fMandarin	guo1; guo3
U+9505.0	fEnglish	simplified form of &#x934B;
U+9506.0	fUTF8	锆
U+9506.0	fGB	0-6F2F
U+9506.0	fR/S	 167.7
U+9506.0	fTotalStrokes	 12
U+9506.0	fCangjie	XCHGR
U+9506.0	fCantonese	gou3
U+9506.0	fMandarin	gao4
U+9506.0	fEnglish	simplified form of &#x92EF;
U+9507.0	fUTF8	锇
U+9507.0	fGB	0-6F30
U+9507.0	fR/S	 167.7
U+9507.0	fTotalStrokes	 12
U+9507.0	fCangjie	XCHQI
U+9507.0	fCantonese	ngo4; o4
U+9507.0	fMandarin	e2; tie3
U+9507.0	fEnglish	simplified form of &#x92E8;
U+9508.0	fUTF8	锈
U+9508.0	fGB	0-5062
U+9508.0	fR/S	 167.7
U+9508.0	fTotalStrokes	 12
U+9508.0	fCangjie	XCHDS
U+9508.0	fHakka	 siu5
U+9508.0	fCantonese	sau3
U+9508.0	fMandarin	xiu4
U+9508.0	fEnglish	simplified form of &#x92B9;
U+9509.0	fUTF8	锉
U+9509.0	fGB	0-6F31
U+9509.0	fR/S	 167.7
U+9509.0	fTotalStrokes	 12
U+9509.0	fCangjie	XCOOG
U+9509.0	fHakka	 co5
U+9509.0	fCantonese	co3; co2 (also)
U+9509.0	fMandarin	cuo4; cuo2; zu2
U+9509.0	fEnglish	simplified form of &#x92BC;
U+950A.0	fUTF8	锊
U+950A.0	fGB	0-6F32
U+950A.0	fR/S	 167.7
U+950A.0	fTotalStrokes	 12
U+950A.0	fCangjie	XCBDI
U+950A.0	fHakka	 lot7
U+950A.0	fCantonese	lyut3
U+950A.0	fMandarin	le4; luue4; lun4; shua1
U+950A.0	fEnglish	simplified form of &#x92DD;
U+950B.0	fUTF8	锋
U+950B.0	fGB	0-3766
U+950B.0	fR/S	 167.7
U+950B.0	fTotalStrokes	 12
U+950B.0	fCangjie	XCHEJ
U+950B.0	fHakka	 fung1
U+950B.0	fCantonese	fung1
U+950B.0	fMandarin	feng1
U+950B.0	fEnglish	simplified form of &#x92D2;
U+950C.0	fUTF8	锌
U+950C.0	fGB	0-503F
U+950C.0	fR/S	 167.7
U+950C.0	fTotalStrokes	 12
U+950C.0	fCangjie	XCYTJ
U+950C.0	fCantonese	san1
U+950C.0	fMandarin	xin1; zi3
U+950C.0	fEnglish	simplified form of &#x92C5;
U+950D.0	fUTF8	锍
U+950D.0	fGB	0-6F33
U+950D.0	fR/S	 167.7
U+950D.0	fTotalStrokes	 12
U+950D.0	fCangjie	XCYIU
U+950D.0	fCantonese	lau5
U+950D.0	fMandarin	liu3
U+950D.0	fEnglish	simplified form of &#x92F6;
U+950E.0	fUTF8	锎
U+950E.0	fGB	0-6F34
U+950E.0	fR/S	 167.7
U+950E.0	fTotalStrokes	 12
U+950E.0	fCangjie	CLST
U+950E.0	fCantonese	hoi1
U+950E.0	fMandarin	kai1
U+950E.0	fEnglish	simplified form of &#x9426;
U+950F.0	fUTF8	锏
U+950F.0	fGB	0-6F35
U+950F.0	fR/S	 167.7
U+950F.0	fTotalStrokes	 12
U+950F.0	fCangjie	CLSA
U+950F.0	fHakka	 gan5; gan3
U+950F.0	fCantonese	gaan2; gaan3
U+950F.0	fMandarin	jian3; jian4
U+950F.0	fEnglish	simplified form of &#x9427;
U+9510.0	fUTF8	锐
U+9510.0	fGB	0-4871
U+9510.0	fR/S	 167.7
U+9510.0	fTotalStrokes	 12
U+9510.0	fCangjie	XCCRU
U+9510.0	fCantonese	jeoi6
U+9510.0	fMandarin	rui4; dui4; yue4
U+9510.0	fEnglish	simplified form of &#x92ED;
U+9511.0	fUTF8	锑
U+9511.0	fGB	0-4C60
U+9511.0	fR/S	 167.7
U+9511.0	fTotalStrokes	 12
U+9511.0	fCangjie	XCCNH
U+9511.0	fCantonese	tai1
U+9511.0	fMandarin	ti4; ti1
U+9511.0	fEnglish	simplified form of &#x92BB;
U+9512.0	fUTF8	锒
U+9512.0	fGB	0-6F36
U+9512.0	fR/S	 167.7
U+9512.0	fTotalStrokes	 12
U+9512.0	fCangjie	XCIAV
U+9512.0	fHakka	 long2
U+9512.0	fCantonese	long4
U+9512.0	fMandarin	lang2
U+9512.0	fEnglish	simplified form of &#x92C3;
U+9513.0	fUTF8	锓
U+9513.0	fGB	0-6F37
U+9513.0	fR/S	 167.7
U+9513.0	fTotalStrokes	 12
U+9513.0	fCangjie	XCSME
U+9513.0	fHakka	 cim1; ciam1
U+9513.0	fCantonese	cim1
U+9513.0	fMandarin	qian1; qin3; qin1; jian1; jin4; qiu3
U+9513.0	fEnglish	simplified form of &#x92DF;
U+9514.0	fUTF8	锔
U+9514.0	fGB	0-6F38
U+9514.0	fR/S	 167.7
U+9514.0	fTotalStrokes	 12
U+9514.0	fCangjie	XCSSR
U+9514.0	fHakka	 kiuk8
U+9514.0	fCantonese	guk1; guk6; geoi1
U+9514.0	fMandarin	ju2; ju1
U+9514.0	fEnglish	simplified form of &#x92E6;
U+9515.0	fUTF8	锕
U+9515.0	fGB	0-6F39
U+9515.0	fR/S	 167.8; 167.7
U+9515.0	fTotalStrokes	 12
U+9515.0	fCangjie	XCNLR
U+9515.0	fCantonese	aa1 (also); aa3; ngaa3 (also); ngaa1 (also); ngo1 (also)
U+9515.0	fMandarin	a1; e1
U+9515.0	fEnglish	simplified form of &#x9312;
U+9516.0	fUTF8	锖
U+9516.0	fGB	0-6F3A
U+9516.0	fR/S	 167.8
U+9516.0	fTotalStrokes	 13
U+9516.0	fCangjie	XCQMB
U+9516.0	fCantonese	coeng1
U+9516.0	fMandarin	qiang1
U+9516.0	fEnglish	simplified form of &#x9306;
U+9517.0	fUTF8	锗
U+9517.0	fGB	0-5560
U+9517.0	fR/S	 167.8
U+9517.0	fTotalStrokes	 13
U+9517.0	fCangjie	XCJKA
U+9517.0	fCantonese	ze2
U+9517.0	fMandarin	duo3; zhe3; zang1
U+9517.0	fEnglish	simplified form of &#x937A;
U+9518.0	fUTF8	锘
U+9518.0	fGB	0-6F3B
U+9518.0	fR/S	 167.8
U+9518.0	fTotalStrokes	 13
U+9518.0	fCangjie	XCTKR
U+9518.0	fCantonese	nok6; lok6 (also)
U+9518.0	fMandarin	tian3; nuo4
U+9518.0	fEnglish	simplified form of &#x9369;
U+9519.0	fUTF8	错
U+9519.0	fGB	0-346D
U+9519.0	fR/S	 167.8
U+9519.0	fTotalStrokes	 13
U+9519.0	fCangjie	XCTA
U+9519.0	fHakka	 cok7; cih5; co5
U+9519.0	fCantonese	co3; cok3; cou3
U+9519.0	fMandarin	cuo4; cu4; que4; xi2
U+9519.0	fEnglish	simplified form of &#x932F;
U+951A.0	fUTF8	锚
U+951A.0	fGB	0-432A
U+951A.0	fR/S	 167.8
U+951A.0	fTotalStrokes	 13
U+951A.0	fCangjie	XCTW
U+951A.0	fHakka	 nau2; mau2; miau2
U+951A.0	fCantonese	naau4; laau4 (also); maau4 (also)
U+951A.0	fMandarin	mao2; mao1
U+951A.0	fEnglish	simplified form of &#x9328;
U+951B.0	fUTF8	锛
U+951B.0	fGB	0-6F3C
U+951B.0	fR/S	 167.8
U+951B.0	fTotalStrokes	 13
U+951B.0	fCangjie	XCKJT
U+951B.0	fHakka	 bun1
U+951B.0	fCantonese	ban1
U+951B.0	fMandarin	ben1
U+951B.0	fEnglish	simplified form of &#x931B;
U+951C.0	fUTF8	锜
U+951C.0	fGB	8-2D28
U+951C.0	fR/S	 167.8
U+951C.0	fTotalStrokes	 13
U+951C.0	fCangjie	XCKMR
U+951C.0	fCantonese	kei4; ji2
U+951C.0	fMandarin	qi2; yi3
U+951C.0	fEnglish	a kind of pen; a kind of chisel
U+951D.0	fUTF8	锝
U+951D.0	fGB	0-6F3D
U+951D.0	fR/S	 167.8
U+951D.0	fTotalStrokes	 13
U+951D.0	fCangjie	XCAMI
U+951D.0	fCantonese	dak1
U+951D.0	fMandarin	de2
U+951D.0	fEnglish	simplified form of &#x9340;
U+951E.0	fUTF8	锞
U+951E.0	fGB	0-6F3E
U+951E.0	fR/S	 167.8
U+951E.0	fTotalStrokes	 13
U+951E.0	fCangjie	XCWD
U+951E.0	fHakka	 ko5
U+951E.0	fCantonese	gwo2; go2 (also); fo3
U+951E.0	fMandarin	kua3; guo3; ke4
U+951E.0	fEnglish	simplified form of &#x9301;
U+951F.0	fUTF8	锟
U+951F.0	fGB	0-6F3F
U+951F.0	fR/S	 167.8
U+951F.0	fTotalStrokes	 13
U+951F.0	fCangjie	XCAPP
U+951F.0	fHakka	 kwun1; gwun3
U+951F.0	fCantonese	kwan1; gwan1
U+951F.0	fMandarin	kun1; gun3; hun4; ran2
U+951F.0	fEnglish	simplified form of &#x9315;
U+9520.0	fUTF8	锠
U+9520.0	fGB	8-2D29
U+9520.0	fR/S	 167.8
U+9520.0	fTotalStrokes	 13
U+9520.0	fCangjie	XCAA
U+9520.0	fCantonese	coeng1
U+9520.0	fMandarin	chang1
U+9520.0	fEnglish	vessel
U+9521.0	fUTF8	锡
U+9521.0	fGB	0-4E7D
U+9521.0	fR/S	 167.8
U+9521.0	fTotalStrokes	 13
U+9521.0	fCangjie	XCAPH
U+9521.0	fHakka	 siak7; sit7
U+9521.0	fCantonese	sek3; sik3 (also); sek1
U+9521.0	fMandarin	xi2; ti4; xi1; di4; si4
U+9521.0	fEnglish	simplified form of &#x932B;
U+9522.0	fUTF8	锢
U+9522.0	fGB	0-6F40
U+9522.0	fR/S	 167.8
U+9522.0	fTotalStrokes	 13
U+9522.0	fCangjie	XCWJR
U+9522.0	fHakka	 gu5
U+9522.0	fCantonese	gu3; gwu3 (also)
U+9522.0	fMandarin	gu4
U+9522.0	fEnglish	simplified form of &#x932E;
U+9523.0	fUTF8	锣
U+9523.0	fGB	0-4260
U+9523.0	fR/S	 167.8
U+9523.0	fTotalStrokes	 13
U+9523.0	fCangjie	CWLN
U+9523.0	fHakka	 lo2
U+9523.0	fCantonese	lo4; lo2 (also)
U+9523.0	fMandarin	luo2
U+9523.0	fEnglish	simplified form of &#x947C;
U+9524.0	fUTF8	锤
U+9524.0	fGB	0-3438
U+9524.0	fR/S	 167.8
U+9524.0	fTotalStrokes	 13
U+9524.0	fCangjie	XCHJM
U+9524.0	fHakka	 ch'ui2; ch'ui5
U+9524.0	fCantonese	ceoi4; ceoi2 (also)
U+9524.0	fMandarin	chui2; zhui3; zhui4
U+9524.0	fEnglish	simplified form of &#x9318;
U+9525.0	fUTF8	锥
U+9525.0	fGB	0-5736
U+9525.0	fR/S	 167.8
U+9525.0	fTotalStrokes	 13
U+9525.0	fCangjie	XCOG
U+9525.0	fHakka	 chui1
U+9525.0	fCantonese	zeoi1
U+9525.0	fMandarin	zhui1
U+9525.0	fEnglish	simplified form of &#x9310;
U+9526.0	fUTF8	锦
U+9526.0	fGB	0-3D75
U+9526.0	fR/S	 167.8
U+9526.0	fTotalStrokes	 13
U+9526.0	fCangjie	XCHAB
U+9526.0	fHakka	 gim3
U+9526.0	fCantonese	gam2
U+9526.0	fMandarin	jin3
U+9526.0	fEnglish	simplified form of &#x9326;
U+9527.0	fUTF8	锧
U+9527.0	fGB	8-2D2A
U+9527.0	fR/S	 167.15; 167.8
U+9527.0	fTotalStrokes	 13
U+9527.0	fCangjie	CHJO
U+9527.0	fCantonese	zat1
U+9527.0	fMandarin	zhi4
U+9527.0	fEnglish	tungsten, wolfram
U+9528.0	fUTF8	锨
U+9528.0	fGB	0-4F47
U+9528.0	fR/S	 167.8
U+9528.0	fTotalStrokes	 13
U+9528.0	fCangjie	XXCHL
U+9528.0	fCantonese	him1; hin1 (also); jan1
U+9528.0	fMandarin	xian1
U+9528.0	fEnglish	simplified form of &#x9341;
U+9529.0	fUTF8	锩
U+9529.0	fGB	0-6F43
U+9529.0	fR/S	 167.8
U+9529.0	fTotalStrokes	 13
U+9529.0	fCangjie	XCFQU
U+9529.0	fHakka	 ken3; gen1
U+9529.0	fCantonese	gyun2
U+9529.0	fMandarin	juan4; juan3; quan1
U+9529.0	fEnglish	simplified form of &#x9308;
U+952A.0	fUTF8	锪
U+952A.0	fGB	0-6F41
U+952A.0	fR/S	 167.8
U+952A.0	fTotalStrokes	 13
U+952A.0	fCangjie	XCPHP
U+952A.0	fCantonese	fat1; kut3 (also); kwut3 (also)
U+952A.0	fMandarin	huo1; cong1; huo4
U+952A.0	fEnglish	simplified form of &#x9343;
U+952B.0	fUTF8	锫
U+952B.0	fGB	0-6F42
U+952B.0	fR/S	 167.8
U+952B.0	fTotalStrokes	 13
U+952B.0	fCangjie	XCYTR
U+952B.0	fCantonese	pui4
U+952B.0	fMandarin	pou2; fu2; pei2
U+952B.0	fEnglish	simplified form of &#x9307;
U+952C.0	fUTF8	锬
U+952C.0	fGB	0-6F44
U+952C.0	fR/S	 167.8
U+952C.0	fTotalStrokes	 13
U+952C.0	fCangjie	XCFF
U+952C.0	fHakka	 jam3; tam2
U+952C.0	fCantonese	taam4; cim1
U+952C.0	fMandarin	tan2; xian1; jian4; tan1; yan3
U+952C.0	fEnglish	simplified form of &#x931F;
U+952D.0	fUTF8	锭
U+952D.0	fGB	0-3627
U+952D.0	fR/S	 167.8
U+952D.0	fTotalStrokes	 13
U+952D.0	fCangjie	XCJMO
U+952D.0	fHakka	 tin5
U+952D.0	fCantonese	ding3; ding6 (also)
U+952D.0	fMandarin	ding4; dian4
U+952D.0	fEnglish	simplified form of &#x9320;
U+952E.0	fUTF8	键
U+952E.0	fGB	0-3C7C
U+952E.0	fR/S	 167.8
U+952E.0	fTotalStrokes	 13
U+952E.0	fCangjie	XCNKQ
U+952E.0	fHakka	 ken5
U+952E.0	fCantonese	gin6
U+952E.0	fMandarin	jian4; jian3; qian2
U+952E.0	fEnglish	simplified form of &#x9375;
U+952F.0	fUTF8	锯
U+952F.0	fGB	0-3E62
U+952F.0	fR/S	 167.8
U+952F.0	fTotalStrokes	 13
U+952F.0	fCangjie	XCSJR
U+952F.0	fHakka	 gi5
U+952F.0	fCantonese	geoi3; goe3 (also); guk1; geoi1
U+952F.0	fMandarin	ju4; ju1
U+952F.0	fEnglish	simplified form of &#x92F8;
U+9530.0	fUTF8	锰
U+9530.0	fGB	0-434C
U+9530.0	fR/S	 167.8
U+9530.0	fTotalStrokes	 13
U+9530.0	fCangjie	XCNDT
U+9530.0	fCantonese	maang5
U+9530.0	fMandarin	meng3
U+9530.0	fEnglish	simplified form of &#x9333;
U+9531.0	fUTF8	锱
U+9531.0	fGB	0-6F45
U+9531.0	fR/S	 167.8
U+9531.0	fTotalStrokes	 13
U+9531.0	fCangjie	XCVVW
U+9531.0	fHakka	 zih1
U+9531.0	fCantonese	zi1
U+9531.0	fMandarin	zi1; wa4
U+9531.0	fEnglish	simplified form of &#x9319;
U+9532.0	fUTF8	锲
U+9532.0	fGB	0-6F46
U+9532.0	fR/S	 167.9
U+9532.0	fTotalStrokes	 14
U+9532.0	fCangjie	XCQHK
U+9532.0	fHakka	 ket7; kiet7
U+9532.0	fCantonese	kit3
U+9532.0	fMandarin	qie4; jie2; qi4
U+9532.0	fEnglish	simplified form of &#x9365;
U+9533.0	fUTF8	锳
U+9533.0	fGB	8-2D27
U+9533.0	fR/S	 167.8; 167.9
U+9533.0	fTotalStrokes	 14
U+9533.0	fCangjie	XCTLK
U+9533.0	fCantonese	joeng1
U+9533.0	fMandarin	ying1
U+9534.0	fUTF8	锴
U+9534.0	fGB	0-6F47
U+9534.0	fR/S	 167.9
U+9534.0	fTotalStrokes	 14
U+9534.0	fCangjie	XCPPA
U+9534.0	fHakka	 gai1
U+9534.0	fCantonese	kaai2; gaai1 (also)
U+9534.0	fMandarin	kai3; jie1; jie3; xie2
U+9534.0	fEnglish	simplified form of &#x9347;
U+9535.0	fUTF8	锵
U+9535.0	fGB	0-6F4F
U+9535.0	fR/S	 167.9
U+9535.0	fTotalStrokes	 14
U+9535.0	fCangjie	XCLMI
U+9535.0	fHakka	 ciong1
U+9535.0	fCantonese	coeng1
U+9535.0	fMandarin	qiang1; cheng1
U+9535.0	fEnglish	simplified form of &#x93D8;
U+9536.0	fUTF8	锶
U+9536.0	fGB	0-6F48
U+9536.0	fR/S	 167.9
U+9536.0	fTotalStrokes	 14
U+9536.0	fCangjie	XCWP
U+9536.0	fCantonese	si1
U+9536.0	fMandarin	si1; song1
U+9536.0	fEnglish	simplified form of &#x9376;
U+9537.0	fUTF8	锷
U+9537.0	fGB	0-6F49
U+9537.0	fR/S	 167.9
U+9537.0	fTotalStrokes	 14
U+9537.0	fCangjie	XCRRS
U+9537.0	fHakka	 ngok8
U+9537.0	fCantonese	ngok6; ok6 (also)
U+9537.0	fMandarin	e4
U+9537.0	fEnglish	simplified form of &#x9354;
U+9538.0	fUTF8	锸
U+9538.0	fGB	0-6F4A
U+9538.0	fR/S	 167.9
U+9538.0	fTotalStrokes	 14
U+9538.0	fCangjie	XCHJX
U+9538.0	fHakka	 cap7
U+9538.0	fCantonese	caap3
U+9538.0	fMandarin	cha2; cha1
U+9538.0	fEnglish	simplified form of &#x9364;
U+9539.0	fUTF8	锹
U+9539.0	fGB	0-4742
U+9539.0	fR/S	 167.9
U+9539.0	fTotalStrokes	 14
U+9539.0	fCangjie	XCHDF
U+9539.0	fCantonese	ciu1
U+9539.0	fMandarin	qiao1
U+9539.0	fEnglish	simplified form of &#x936C;
U+953A.0	fUTF8	锺
U+953A.0	fGB	0-6F71
U+953A.0	fR/S	 167.9
U+953A.0	fTotalStrokes	 14
U+953A.0	fCangjie	XCHJG
U+953A.0	fHakka	 zhung1
U+953A.0	fCantonese	zung1
U+953A.0	fMandarin	zhong1; zhong4
U+953A.0	fEnglish	simplified form of &#x937E;
U+953B.0	fUTF8	锻
U+953B.0	fGB	0-364D
U+953B.0	fR/S	 167.9
U+953B.0	fTotalStrokes	 14
U+953B.0	fCangjie	XCHJE
U+953B.0	fHakka	 ton5
U+953B.0	fCantonese	dyun3
U+953B.0	fMandarin	duan4
U+953B.0	fEnglish	simplified form of &#x935B;
U+953C.0	fUTF8	锼
U+953C.0	fGB	0-6F4B
U+953C.0	fR/S	 167.9
U+953C.0	fTotalStrokes	 14
U+953C.0	fCangjie	XCHXE
U+953C.0	fHakka	 seu1
U+953C.0	fCantonese	sau1; sau2 (also)
U+953C.0	fMandarin	sou1
U+953C.0	fEnglish	simplified form of &#x93AA;
U+953D.0	fUTF8	锽
U+953D.0	fGB	8-2D2B
U+953D.0	fR/S	 167.9
U+953D.0	fTotalStrokes	 14
U+953D.0	fCangjie	XCHAG
U+953D.0	fCantonese	wong4
U+953D.0	fMandarin	huang2
U+953D.0	fEnglish	weapon
U+953E.0	fUTF8	锾
U+953E.0	fGB	0-6F4C
U+953E.0	fR/S	 167.9
U+953E.0	fTotalStrokes	 14
U+953E.0	fCangjie	XCBME
U+953E.0	fHakka	 fan2
U+953E.0	fCantonese	waan4; wun4 (also)
U+953E.0	fMandarin	huan2; huan4; yung4
U+953E.0	fEnglish	simplified form of &#x9370;
U+953F.0	fUTF8	锿
U+953F.0	fGB	0-6F4D
U+953F.0	fR/S	 167.9
U+953F.0	fTotalStrokes	 14
U+953F.0	fCangjie	XXXCY
U+953F.0	fCantonese	oi1; ngoi1 (also)
U+953F.0	fMandarin	ai1
U+953F.0	fEnglish	simplified form of &#x9384;
U+9540.0	fUTF8	镀
U+9540.0	fGB	0-3646
U+9540.0	fR/S	 167.9
U+9540.0	fTotalStrokes	 14
U+9540.0	fCangjie	XCITE
U+9540.0	fHakka	 tu5
U+9540.0	fCantonese	dou6
U+9540.0	fMandarin	du4; tu2
U+9540.0	fEnglish	simplified form of &#x934D;
U+9541.0	fUTF8	镁
U+9541.0	fGB	0-433E
U+9541.0	fR/S	 167.9
U+9541.0	fTotalStrokes	 14
U+9541.0	fCangjie	XCTGK
U+9541.0	fCantonese	mei5
U+9541.0	fMandarin	mei3
U+9541.0	fEnglish	simplified form of &#x9382;
U+9542.0	fUTF8	镂
U+9542.0	fGB	0-6F4E
U+9542.0	fR/S	 167.9
U+9542.0	fTotalStrokes	 14
U+9542.0	fCangjie	CFDV
U+9542.0	fHakka	 leu3; leu2; leu5
U+9542.0	fCantonese	lau6
U+9542.0	fMandarin	lou4; lian2
U+9542.0	fEnglish	simplified form of &#x93E4;
U+9543.0	fUTF8	镃
U+9543.0	fGB	8-2D2C
U+9543.0	fR/S	 167.9
U+9543.0	fTotalStrokes	 14
U+9543.0	fCangjie	XCTVI
U+9543.0	fCantonese	zi1
U+9543.0	fMandarin	zi1
U+9543.0	fEnglish	hoe; mattock
U+9544.0	fUTF8	镄
U+9544.0	fGB	0-6F50
U+9544.0	fR/S	 167.9
U+9544.0	fTotalStrokes	 14
U+9544.0	fCangjie	CLNO
U+9544.0	fCantonese	fai3
U+9544.0	fMandarin	fei4
U+9544.0	fEnglish	simplified form of &#x9428;
U+9545.0	fUTF8	镅
U+9545.0	fGB	0-6F51
U+9545.0	fR/S	 167.9
U+9545.0	fTotalStrokes	 14
U+9545.0	fCangjie	XCAHU
U+9545.0	fCantonese	mei4
U+9545.0	fMandarin	mei2
U+9545.0	fEnglish	simplified form of &#x9387;
U+9546.0	fUTF8	镆
U+9546.0	fGB	0-6F52
U+9546.0	fR/S	 167.10
U+9546.0	fTotalStrokes	 15
U+9546.0	fCangjie	XCTAK
U+9546.0	fHakka	 mok8
U+9546.0	fCantonese	mok6
U+9546.0	fMandarin	mo4
U+9546.0	fEnglish	simplified form of &#x93CC;
U+9547.0	fUTF8	镇
U+9547.0	fGB	0-5572
U+9547.0	fR/S	 167.10
U+9547.0	fTotalStrokes	 15
U+9547.0	fCangjie	XCJBC
U+9547.0	fCantonese	zan3
U+9547.0	fMandarin	zhen4; tian2; zhen1
U+9547.0	fEnglish	simplified form of &#x93AE;
U+9548.0	fUTF8	镈
U+9548.0	fGB	7-2134
U+9548.0	fR/S	 167.10
U+9548.0	fTotalStrokes	 15
U+9548.0	fCangjie	XCIBI
U+9548.0	fCantonese	bok3
U+9548.0	fMandarin	bo2
U+9548.0	fEnglish	large bell; hoe, spade
U+9549.0	fUTF8	镉
U+9549.0	fGB	0-6F53
U+9549.0	fR/S	 167.10
U+9549.0	fTotalStrokes	 15
U+9549.0	fCangjie	XCMRB
U+9549.0	fCantonese	gaak3; lik6
U+9549.0	fMandarin	ge2; li4
U+9549.0	fEnglish	simplified form of &#x9398;
U+954A.0	fUTF8	镊
U+954A.0	fGB	0-4477
U+954A.0	fR/S	 167.10
U+954A.0	fTotalStrokes	 15
U+954A.0	fCangjie	XCSJE
U+954A.0	fHakka	 ngiap7
U+954A.0	fCantonese	nip6; lip6 (also)
U+954A.0	fMandarin	nie4; guan4
U+954A.0	fEnglish	simplified form of &#x9477;
U+954B.0	fUTF8	镋
U+954B.0	fGB	7-2135
U+954B.0	fR/S	 167.20; 167.10
U+954B.0	fTotalStrokes	 15
U+954B.0	fCangjie	XCFBU
U+954B.0	fCantonese	tong2
U+954B.0	fMandarin	tang3
U+954C.0	fUTF8	镌
U+954C.0	fGB	0-6F54
U+954C.0	fR/S	 167.10
U+954C.0	fTotalStrokes	 15
U+954C.0	fCangjie	XXCOG
U+954C.0	fCantonese	zyun1; zeon3 (also)
U+954C.0	fMandarin	juan1
U+954C.0	fEnglish	simplified form of &#x93B8;
U+954D.0	fUTF8	镍
U+954D.0	fGB	0-4478
U+954D.0	fR/S	 167.10
U+954D.0	fTotalStrokes	 15
U+954D.0	fCangjie	XCHUD
U+954D.0	fCantonese	nip1; lip1 (also); nip6 (also); lip6 (also)
U+954D.0	fMandarin	nie4
U+954D.0	fEnglish	simplified form of &#x93B3;
U+954E.0	fUTF8	镎
U+954E.0	fGB	0-6F55
U+954E.0	fR/S	 167.10
U+954E.0	fTotalStrokes	 15
U+954E.0	fCangjie	XCORQ
U+954E.0	fCantonese	naa4; laa4 (also)
U+954E.0	fMandarin	na2
U+954E.0	fEnglish	simplified form of &#x93BF;
U+954F.0	fUTF8	镏
U+954F.0	fGB	0-6F56
U+954F.0	fR/S	 167.10
U+954F.0	fTotalStrokes	 15
U+954F.0	fCangjie	XCHHW
U+954F.0	fCantonese	lau4; lau6; lau5
U+954F.0	fMandarin	liu2; liu4
U+954F.0	fEnglish	simplified form of &#x93A6;
U+9550.0	fUTF8	镐
U+9550.0	fGB	0-3864
U+9550.0	fR/S	 167.10
U+9550.0	fTotalStrokes	 15
U+9550.0	fCangjie	XCYRB
U+9550.0	fHakka	 hau3
U+9550.0	fCantonese	hou6; gou2
U+9550.0	fMandarin	hao4; yi4; gao3
U+9550.0	fEnglish	simplified form of &#x93AC;
U+9551.0	fUTF8	镑
U+9551.0	fGB	0-3077
U+9551.0	fR/S	 167.10
U+9551.0	fTotalStrokes	 15
U+9551.0	fCangjie	XCYBS
U+9551.0	fHakka	 pong2
U+9551.0	fCantonese	bong6; bong2 (also)
U+9551.0	fMandarin	bang4; pang1
U+9551.0	fEnglish	simplified form of &#x938A;
U+9552.0	fUTF8	镒
U+9552.0	fGB	0-6F57
U+9552.0	fR/S	 167.10
U+9552.0	fTotalStrokes	 15
U+9552.0	fCangjie	XCTCT
U+9552.0	fHakka	 jit7
U+9552.0	fCantonese	jat6
U+9552.0	fMandarin	yi4
U+9552.0	fEnglish	simplified form of &#x93B0;
U+9553.0	fUTF8	镓
U+9553.0	fGB	0-6F58
U+9553.0	fR/S	 167.10
U+9553.0	fTotalStrokes	 15
U+9553.0	fCangjie	XXXCJ
U+9553.0	fCantonese	gaa1
U+9553.0	fMandarin	jia1
U+9553.0	fEnglish	simplified form of &#x93B5;
U+9554.0	fUTF8	镔
U+9554.0	fGB	0-6F59
U+9554.0	fR/S	 167.10
U+9554.0	fTotalStrokes	 15
U+9554.0	fCangjie	CJOC
U+9554.0	fHakka	 bin1
U+9554.0	fCantonese	ban1
U+9554.0	fMandarin	bin1
U+9554.0	fEnglish	simplified form of &#x944C;
U+9555.0	fUTF8	镕
U+9555.0	fGB	7-214B
U+9555.0	fR/S	 167.10
U+9555.0	fTotalStrokes	 15
U+9555.0	fCangjie	XCJCR
U+9555.0	fCantonese	jung4
U+9555.0	fMandarin	rong2
U+9555.0	fEnglish	 fuse; melt; smelt mold
U+9556.0	fUTF8	镖
U+9556.0	fGB	0-6F5A
U+9556.0	fR/S	 167.11
U+9556.0	fTotalStrokes	 16
U+9556.0	fCangjie	XCMWF
U+9556.0	fHakka	 piau1; biau1
U+9556.0	fCantonese	biu1
U+9556.0	fMandarin	biao1; piao1
U+9556.0	fEnglish	simplified form of &#x93E2;
U+9557.0	fUTF8	镗
U+9557.0	fGB	0-6F5B
U+9557.0	fR/S	 167.11
U+9557.0	fTotalStrokes	 16
U+9557.0	fCangjie	XCFBG
U+9557.0	fHakka	 tong2
U+9557.0	fCantonese	tong1; tong4
U+9557.0	fMandarin	tang1
U+9557.0	fEnglish	simplified form of &#x93DC;
U+9558.0	fUTF8	镘
U+9558.0	fGB	0-6F5C
U+9558.0	fR/S	 167.11
U+9558.0	fTotalStrokes	 16
U+9558.0	fCangjie	XCAWE
U+9558.0	fHakka	 man5
U+9558.0	fCantonese	maan6
U+9558.0	fMandarin	man4; man2; zu2
U+9558.0	fEnglish	simplified form of &#x93DD;
U+9559.0	fUTF8	镙
U+9559.0	fGB	0-6F5D
U+9559.0	fR/S	 167.11
U+9559.0	fTotalStrokes	 16
U+9559.0	fCangjie	XCWVF
U+9559.0	fCantonese	lo4
U+9559.0	fMandarin	luo2
U+9559.0	fEnglish	simplified form of &#x93CD;
U+955A.0	fUTF8	镚
U+955A.0	fGB	8-2C61
U+955A.0	fR/S	 167.11
U+955A.0	fTotalStrokes	 16
U+955A.0	fCangjie	XCUBB
U+955A.0	fCantonese	bang1
U+955A.0	fMandarin	beng4
U+955A.0	fEnglish	small coin
U+955B.0	fUTF8	镛
U+955B.0	fGB	0-6F5E
U+955B.0	fR/S	 167.11
U+955B.0	fTotalStrokes	 16
U+955B.0	fCangjie	XCILB
U+955B.0	fHakka	 jung2
U+955B.0	fCantonese	jung4
U+955B.0	fMandarin	yong1; rong2
U+955B.0	fEnglish	simplified form of &#x93DE;
U+955C.0	fUTF8	镜
U+955C.0	fGB	0-3E35
U+955C.0	fR/S	 167.11
U+955C.0	fTotalStrokes	 16
U+955C.0	fCangjie	XCYTU
U+955C.0	fHakka	 giang5
U+955C.0	fCantonese	geng3; geng2 (also)
U+955C.0	fMandarin	jing4
U+955C.0	fEnglish	simplified form of &#x93E1;
U+955D.0	fUTF8	镝
U+955D.0	fGB	0-6F61
U+955D.0	fR/S	 167.11
U+955D.0	fTotalStrokes	 16
U+955D.0	fCangjie	XCYCB
U+955D.0	fHakka	 dit7
U+955D.0	fCantonese	dik1; dik6
U+955D.0	fMandarin	di2; di1; li2
U+955D.0	fEnglish	simplified form of &#x93D1;
U+955E.0	fUTF8	镞
U+955E.0	fGB	0-6F5F
U+955E.0	fR/S	 167.11
U+955E.0	fTotalStrokes	 16
U+955E.0	fCangjie	XCYSK
U+955E.0	fHakka	 cuk8
U+955E.0	fCantonese	zuk6
U+955E.0	fMandarin	zu2; cu4; chuo4; cou4; qiang1; zhuo1
U+955E.0	fEnglish	simplified form of &#x93C3;
U+955F.0	fUTF8	镟
U+955F.0	fGB	0-6F60
U+955F.0	fR/S	 167.11
U+955F.0	fTotalStrokes	 16
U+955F.0	fCangjie	XCYSO
U+955F.0	fHakka	 sien5
U+955F.0	fCantonese	syun4; syun6 (also)
U+955F.0	fMandarin	xuan4; xuan2
U+955F.0	fEnglish	simplified form of &#x93C7;
U+9560.0	fUTF8	镠
U+9560.0	fGB	8-2C62
U+9560.0	fR/S	 167.11
U+9560.0	fTotalStrokes	 16
U+9560.0	fCangjie	XCSMH
U+9560.0	fCantonese	lau4
U+9560.0	fMandarin	liu2
U+9560.0	fEnglish	pure gold
U+9561.0	fUTF8	镡
U+9561.0	fGB	0-6F62
U+9561.0	fR/S	 167.12
U+9561.0	fTotalStrokes	 17
U+9561.0	fCangjie	XCMWJ
U+9561.0	fHakka	 tam2; jam2
U+9561.0	fCantonese	taam4; cam4 (also)
U+9561.0	fMandarin	tan2; xin2; chan2; dan4; duo2; xin4; ya1; yin2
U+9561.0	fEnglish	simplified form of &#x9414;
U+9562.0	fUTF8	镢
U+9562.0	fGB	0-6F63
U+9562.0	fR/S	 167.12
U+9562.0	fTotalStrokes	 17
U+9562.0	fCangjie	XCMTO
U+9562.0	fHakka	 ket7
U+9562.0	fCantonese	kyut3; fok3
U+9562.0	fMandarin	jue2
U+9562.0	fEnglish	simplified form of &#x941D;
U+9563.0	fUTF8	镣
U+9563.0	fGB	0-414D
U+9563.0	fR/S	 167.12
U+9563.0	fTotalStrokes	 17
U+9563.0	fCangjie	XCKCF
U+9563.0	fHakka	 liau2; liau1
U+9563.0	fCantonese	liu4
U+9563.0	fMandarin	liao2; liao4; lao4
U+9563.0	fEnglish	simplified form of &#x9410;
U+9564.0	fUTF8	镤
U+9564.0	fGB	0-6F64
U+9564.0	fR/S	 167.12
U+9564.0	fTotalStrokes	 17
U+9564.0	fCangjie	XCTCO
U+9564.0	fHakka	 puk8
U+9564.0	fCantonese	buk6; pok3
U+9564.0	fMandarin	pu2
U+9564.0	fEnglish	simplified form of &#x93F7;
U+9565.0	fUTF8	镥
U+9565.0	fGB	0-6F65
U+9565.0	fR/S	 167.12
U+9565.0	fTotalStrokes	 17
U+9565.0	fCangjie	XCNWA
U+9565.0	fCantonese	lou5
U+9565.0	fMandarin	lu3
U+9565.0	fEnglish	simplified form of &#x9465;
U+9566.0	fUTF8	镦
U+9566.0	fGB	0-6F66
U+9566.0	fR/S	 167.12
U+9566.0	fTotalStrokes	 17
U+9566.0	fCangjie	XCYDK
U+9566.0	fHakka	 tui5; dun1
U+9566.0	fCantonese	deoi6; deon1; deoi1
U+9566.0	fMandarin	dui4; dun1; duan4; dui1; dun4; tan2; xun2
U+9566.0	fEnglish	simplified form of &#x9413;
U+9567.0	fUTF8	镧
U+9567.0	fGB	0-6F67
U+9567.0	fR/S	 167.12
U+9567.0	fTotalStrokes	 17
U+9567.0	fCangjie	CLSW
U+9567.0	fHakka	 lan5
U+9567.0	fCantonese	laan4
U+9567.0	fMandarin	lan4; lan2
U+9567.0	fEnglish	simplified form of &#x946D;
U+9568.0	fUTF8	镨
U+9568.0	fGB	0-6F68
U+9568.0	fR/S	 167.12
U+9568.0	fTotalStrokes	 17
U+9568.0	fCangjie	XCTCA
U+9568.0	fCantonese	pou2
U+9568.0	fMandarin	pu3
U+9568.0	fEnglish	simplified form of &#x9420;
U+9569.0	fUTF8	镩
U+9569.0	fGB	0-6F69
U+9569.0	fR/S	 167.12
U+9569.0	fTotalStrokes	 17
U+9569.0	fCangjie	CJCL
U+9569.0	fCantonese	cyun1; cyun3
U+9569.0	fMandarin	cuan4; cuan1
U+9569.0	fEnglish	simplified form of &#x9479;
U+956A.0	fUTF8	镪
U+956A.0	fGB	0-6F6A
U+956A.0	fR/S	 167.12
U+956A.0	fTotalStrokes	 17
U+956A.0	fCangjie	XXXCN
U+956A.0	fHakka	 giong3
U+956A.0	fCantonese	koeng5
U+956A.0	fMandarin	qiang3; qiang1; jiang3
U+956A.0	fEnglish	simplified form of &#x93F9;
U+956B.0	fUTF8	镫
U+956B.0	fGB	0-6F6B
U+956B.0	fR/S	 167.12
U+956B.0	fTotalStrokes	 17
U+956B.0	fCangjie	XCNOT
U+956B.0	fHakka	 den1
U+956B.0	fCantonese	dang3; dang1
U+956B.0	fMandarin	deng4; deng1
U+956B.0	fEnglish	simplified form of &#x9419;
U+956C.0	fUTF8	镬
U+956C.0	fGB	0-6F6C
U+956C.0	fR/S	 167.13
U+956C.0	fTotalStrokes	 18
U+956C.0	fCangjie	XCTOE
U+956C.0	fHakka	 vok8
U+956C.0	fCantonese	wok6; wok2 (also)
U+956C.0	fMandarin	huo4
U+956C.0	fEnglish	simplified form of &#x944A;
U+956D.0	fUTF8	镭
U+956D.0	fGB	0-4058
U+956D.0	fR/S	 167.13
U+956D.0	fTotalStrokes	 18
U+956D.0	fCangjie	XCMBW
U+956D.0	fHakka	 lui2
U+956D.0	fCantonese	leoi4
U+956D.0	fMandarin	lei2
U+956D.0	fEnglish	simplified form of &#x9433;
U+956E.0	fUTF8	镮
U+956E.0	fGB	8-2D2D
U+956E.0	fR/S	 167.13
U+956E.0	fTotalStrokes	 18
U+956E.0	fCangjie	XCWLV
U+956E.0	fCantonese	waan4
U+956E.0	fMandarin	huan2
U+956E.0	fEnglish	metal ring; measure of currency
U+956F.0	fUTF8	镯
U+956F.0	fGB	0-6F6D
U+956F.0	fR/S	 167.13
U+956F.0	fTotalStrokes	 18
U+956F.0	fCangjie	XCWLI
U+956F.0	fHakka	 cuk8
U+956F.0	fCantonese	zuk6
U+956F.0	fMandarin	zhuo2; shu3
U+956F.0	fEnglish	simplified form of &#x9432;
U+9570.0	fUTF8	镰
U+9570.0	fGB	0-412D
U+9570.0	fR/S	 167.13
U+9570.0	fTotalStrokes	 18
U+9570.0	fCangjie	XCITC
U+9570.0	fHakka	 liam2
U+9570.0	fCantonese	lim4
U+9570.0	fMandarin	lian2
U+9570.0	fEnglish	simplified form of &#x942E;
U+9571.0	fUTF8	镱
U+9571.0	fGB	0-6F6E
U+9571.0	fR/S	 167.13
U+9571.0	fTotalStrokes	 18
U+9571.0	fCangjie	XCYTP
U+9571.0	fCantonese	ji3
U+9571.0	fMandarin	yi4
U+9571.0	fEnglish	simplified form of &#x943F;
U+9572.0	fUTF8	镲
U+9572.0	fGB	0-6F6F
U+9572.0	fR/S	 167.13; 167.14
U+9572.0	fTotalStrokes	 19
U+9572.0	fCangjie	XXCJB
U+9572.0	fCantonese	caa2
U+9572.0	fMandarin	cha3
U+9572.0	fEnglish	simplified form of &#x9454;
U+9573.0	fUTF8	镳
U+9573.0	fGB	0-6F70
U+9573.0	fR/S	 167.15
U+9573.0	fTotalStrokes	 20
U+9573.0	fCangjie	XCIPF
U+9573.0	fHakka	 piau1
U+9573.0	fCantonese	biu1
U+9573.0	fMandarin	biao1
U+9573.0	fEnglish	simplified form of &#x9463;
U+9574.0	fUTF8	镴
U+9574.0	fGB	8-2D2E
U+9574.0	fR/S	 167.15
U+9574.0	fTotalStrokes	 20
U+9574.0	fCangjie	XCVVV
U+9574.0	fCantonese	laap6; lap6
U+9574.0	fMandarin	la4
U+9574.0	fEnglish	solder; tin
U+9575.0	fUTF8	镵
U+9575.0	fGB	8-2D7D
U+9575.0	fR/S	 167.17
U+9575.0	fTotalStrokes	 22
U+9575.0	fCangjie	XCNRI
U+9575.0	fCantonese	caam4
U+9575.0	fMandarin	chan2
U+9575.0	fEnglish	spade, trowel
U+9576.0	fUTF8	镶
U+9576.0	fGB	0-4F62
U+9576.0	fR/S	 167.17
U+9576.0	fTotalStrokes	 22
U+9576.0	fCangjie	XCYRV
U+9576.0	fHakka	 siong1
U+9576.0	fCantonese	soeng1
U+9576.0	fMandarin	xiang1; rang2; niang2
U+9576.0	fEnglish	simplified form of &#x9472;
U+9577.0	fUTF8	長
U+9577.0	fCNS11643	1-4F5B
U+9577.0	fBig5	AAF8
U+9577.0	fGB	1-3324
U+9577.0	fR/S	 168.0
U+9577.0	fTotalStrokes	 8
U+9577.0	fCangjie	SMV
U+9577.0	fFourCorner	71732
U+9577.0	fHakka	  zong3; zhong3;  chong2; chong3; zhong5; zong5
U+9577.0	fCantonese	coeng4; zoeng2; zoeng6 (also); coeng1 (also); coeng2 (also)
U+9577.0	fMandarin	chang2
U+9577.0	fEnglish	[1] long; length [2] forte; strong point [3] excel in; be good at [4] a Chinese family name [5] Kangxi radical 168
U+9577.1	fUTF8	長
U+9577.1	fCNS11643	1-4F5B
U+9577.1	fBig5	AAF8
U+9577.1	fGB	1-3324
U+9577.1	fR/S	 168.0
U+9577.1	fTotalStrokes	 8
U+9577.1	fCangjie	SMV
U+9577.1	fFourCorner	71732
U+9577.1	fHakka	 cong2
U+9577.1	fMandarin	zhang3
U+9577.1	fEnglish	[1] senior; old [2] the eldest [3] leader; head; chief; commander; chairman [4] grow [5] increase; advance [6] look appear; become
U+9578.0	fUTF8	镸
U+9578.0	fCNS11643	4-252B
U+9578.0	fGB	5-4E4B
U+9578.0	fR/S	 168.0
U+9578.0	fTotalStrokes	 7; 8
U+9578.0	fCangjie	SMI
U+9578.0	fFourCorner	71730
U+9578.0	fCantonese	coeng4
U+9578.0	fMandarin	chang2; zhang3
U+9578.0	fEnglish	long
U+9579.0	fUTF8	镹
U+9579.0	fCNS11643	4-303A
U+9579.0	fGB	E-4359
U+9579.0	fR/S	 168.3
U+9579.0	fTotalStrokes	 10; 11
U+9579.0	fCangjie	SINO
U+9579.0	fFourCorner	77780
U+9579.0	fCantonese	gau2
U+9579.0	fMandarin	jiu3
U+957A.0	fUTF8	镺
U+957A.0	fCNS11643	2-3A28
U+957A.0	fBig5	D844
U+957A.0	fGB	3-4C25
U+957A.0	fR/S	 168.4
U+957A.0	fTotalStrokes	 12
U+957A.0	fCangjie	SIHK
U+957A.0	fFourCorner	72734
U+957A.0	fCantonese	ou2; ngou2 (also)
U+957A.0	fMandarin	ao3; ao4
U+957B.0	fUTF8	镻
U+957B.0	fCNS11643	2-413F
U+957B.0	fBig5	DC78
U+957B.0	fGB	E-435A
U+957B.0	fR/S	 168.5
U+957B.0	fTotalStrokes	 13
U+957B.0	fCangjie	SIHQO
U+957B.0	fFourCorner	75730
U+957B.0	fCantonese	dit6
U+957B.0	fMandarin	die2
U+957C.0	fUTF8	镼
U+957C.0	fCNS11643	2-554E
U+957C.0	fBig5	E8A5
U+957C.0	fGB	5-4E51
U+957C.0	fR/S	 168.8
U+957C.0	fTotalStrokes	 16
U+957C.0	fCangjie	SISUU
U+957C.0	fFourCorner	77777
U+957C.0	fCantonese	gwat6; wat1
U+957C.0	fMandarin	qu1; jue2
U+957D.0	fUTF8	镽
U+957D.0	fCNS11643	2-6763
U+957D.0	fBig5	F376
U+957D.0	fGB	3-4C29
U+957D.0	fR/S	 168.12
U+957D.0	fTotalStrokes	 20
U+957D.0	fCangjie	SIKCF
U+957D.0	fFourCorner	74796
U+957D.0	fHakka	 liau2; liau5
U+957D.0	fCantonese	liu5; liu4; liu6
U+957D.0	fMandarin	liao3; liao2
U+957D.0	fEnglish	tall
U+957E.0	fUTF8	镾
U+957E.0	fCNS11643	5-7156
U+957E.0	fGB	5-4E4C
U+957E.0	fR/S	  168.14
U+957E.0	fTotalStrokes	 21; 22
U+957E.0	fCangjie	SIMFB
U+957E.0	fFourCorner	71720
U+957E.0	fCantonese	nei4; lei4 (also)
U+957E.0	fMandarin	mi2
U+957F.0	fUTF8	长
U+957F.0	fGB	0-3324
U+957F.0	fR/S	 168.0
U+957F.0	fTotalStrokes	 5
U+957F.0	fCangjie	PO
U+957F.0	fHakka	 zhong3; chong2
U+957F.0	fCantonese	coeng4; coeng1 (also); coeng2 (also); zoeng6 (also); zoeng2
U+957F.0	fMandarin	chang2; zhang4
U+957F.0	fEnglish	simplified form of &#x9577;
U+957F.1	fUTF8	长
U+957F.1	fGB	0-3324
U+957F.1	fR/S	 168.0
U+957F.1	fTotalStrokes	 5
U+957F.1	fCangjie	PO
U+957F.1	fMandarin	zhang3
U+9580.0	fUTF8	門
U+9580.0	fCNS11643	1-4F5C
U+9580.0	fBig5	AAF9
U+9580.0	fGB	1-4345
U+9580.0	fR/S	 169.0
U+9580.0	fTotalStrokes	 8
U+9580.0	fCangjie	AN
U+9580.0	fFourCorner	77777
U+9580.0	fHakka	  mun2; mun3
U+9580.0	fCantonese	mun4; mun2 (also)
U+9580.0	fMandarin	men2
U+9580.0	fEnglish	[1] gate; door; entrance; opening [2] family; clan [3] sect; school; gang [4] class; category [5] the key; turning point [6] a piece of (artillery); cannon [7] gate-keeping [8] a Chinese familly name [9] Kangxi radical 169
U+9580.1	fUTF8	門
U+9580.1	fCNS11643	1-4F5C
U+9580.1	fBig5	AAF9
U+9580.1	fGB	1-4345
U+9580.1	fR/S	 169.0
U+9580.1	fTotalStrokes	 8
U+9580.1	fCangjie	AN
U+9580.1	fFourCorner	77777
U+9580.1	fMandarin	men5
U+9581.0	fUTF8	閁
U+9581.0	fCNS11643	3-2F40
U+9581.0	fGB	E-435B
U+9581.0	fR/S	  169.1
U+9581.0	fTotalStrokes	 9
U+9581.0	fCangjie	ANH
U+9581.0	fFourCorner	77200
U+9581.0	fCantonese	maat3
U+9581.0	fMandarin	ma4
U+9582.0	fUTF8	閂
U+9582.0	fCNS11643	1-536D
U+9582.0	fBig5	ADAC
U+9582.0	fGB	1-6345
U+9582.0	fR/S	 169.1
U+9582.0	fTotalStrokes	 9
U+9582.0	fCangjie	ANM
U+9582.0	fFourCorner	77107
U+9582.0	fHakka	  con1; san1
U+9582.0	fCantonese	saan1
U+9582.0	fMandarin	shuan1; zao2; zao4; zuo4
U+9582.0	fEnglish	[1] fasten with a bolt; latch [2] latch of a door; crossbar
U+9583.0	fUTF8	閃
U+9583.0	fCNS11643	1-5860
U+9583.0	fBig5	B07B
U+9583.0	fGB	1-4941
U+9583.0	fR/S	 169.2
U+9583.0	fTotalStrokes	 10
U+9583.0	fCangjie	ANO
U+9583.0	fFourCorner	77807
U+9583.0	fHakka	  sham3;  sam3; sap7; sham5
U+9583.0	fCantonese	sim2
U+9583.0	fMandarin	shan3; shan4
U+9583.0	fEnglish	[1] flash (light); very brief glimpse [2] avoid; dodge; evade [3] twist; strain; sprain (one's back, etc.) [4] cast away; leave behind
U+9584.0	fUTF8	閄
U+9584.0	fCNS11643	4-303B
U+9584.0	fGB	E-435C
U+9584.0	fR/S	  169.2
U+9584.0	fTotalStrokes	 10
U+9584.0	fCangjie	ANO
U+9584.0	fFourCorner	78200
U+9584.0	fCantonese	wik6
U+9584.0	fMandarin	huo4; shan3
U+9585.0	fUTF8	閅
U+9585.0	fCNS11643	3-3432
U+9585.0	fGB	E-435D
U+9585.0	fR/S	 169.2
U+9585.0	fTotalStrokes	 10
U+9585.0	fCangjie	ANMN
U+9585.0	fFourCorner	77200
U+9585.0	fCantonese	mun4
U+9585.0	fMandarin	men2
U+9586.0	fUTF8	閆
U+9586.0	fCNS11643	2-3A29
U+9586.0	fBig5	D845
U+9586.0	fGB	1-6346
U+9586.0	fR/S	 169.3
U+9586.0	fTotalStrokes	 11
U+9586.0	fCangjie	ANMMM
U+9586.0	fFourCorner	77101
U+9586.0	fHakka	 jam2; ram2
U+9586.0	fCantonese	jim4
U+9586.0	fMandarin	yan2; yan3; yan4
U+9586.0	fEnglish	village gate
U+9587.0	fUTF8	閇
U+9587.0	fCNS11643	3-395C
U+9587.0	fGB	E-435E
U+9587.0	fR/S	 169.3
U+9587.0	fTotalStrokes	 11
U+9587.0	fCangjie	ANMY
U+9587.0	fFourCorner	77230
U+9587.0	fHakka	 bi5
U+9587.0	fCantonese	bai3
U+9587.0	fMandarin	bi4; shuan1
U+9587.0	fEnglish	 shut; close obstruct; block up
U+9588.0	fUTF8	閈
U+9588.0	fCNS11643	2-3A2A
U+9588.0	fBig5	D846
U+9588.0	fGB	3-3A73
U+9588.0	fR/S	 169.3
U+9588.0	fTotalStrokes	 11
U+9588.0	fCangjie	ANMJ
U+9588.0	fFourCorner	77401
U+9588.0	fHakka	 hon5; hon6
U+9588.0	fCantonese	hon6
U+9588.0	fMandarin	han4
U+9588.0	fEnglish	village
U+9589.0	fUTF8	閉
U+9589.0	fCNS11643	1-5D70
U+9589.0	fBig5	B3AC
U+9589.0	fGB	1-3155
U+9589.0	fR/S	 169.3
U+9589.0	fTotalStrokes	 11
U+9589.0	fCangjie	ANDH
U+9589.0	fFourCorner	77247
U+9589.0	fHakka	  bi5
U+9589.0	fCantonese	bai3
U+9589.0	fMandarin	bi4; bie1
U+9589.0	fEnglish	[1] shut; close [2] obstruct; block up; stop [3] conclude; end (conference) [4] restrain
U+958A.0	fUTF8	閊
U+958A.0	fGB	E-435F
U+958A.0	fR/S	 169.3
U+958A.0	fTotalStrokes	 11
U+958A.0	fCangjie	ANU
U+958A.0	fCantonese	saan1
U+958A.0	fEnglish	to be obstructed, blocked; to break down
U+958B.0	fUTF8	開
U+958B.0	fCNS11643	1-6264
U+958B.0	fBig5	B67D
U+958B.0	fGB	1-3F2A
U+958B.0	fR/S	 169.4
U+958B.0	fTotalStrokes	 12
U+958B.0	fCangjie	ANMT
U+958B.0	fFourCorner	77441
U+958B.0	fHakka	  koi1; hoi1
U+958B.0	fCantonese	hoi1
U+958B.0	fMandarin	kai1; qian1
U+958B.0	fEnglish	[1] open [2] drive [3] initiate; begin; start [4] reveal; disclose [5] state; explain [6] found; expand
U+958C.0	fUTF8	閌
U+958C.0	fCNS11643	2-4141
U+958C.0	fBig5	DC7A
U+958C.0	fGB	1-634A
U+958C.0	fR/S	 169.4
U+958C.0	fTotalStrokes	 12
U+958C.0	fCangjie	ANYHN
U+958C.0	fFourCorner	77211
U+958C.0	fHakka	  kong5; kong1
U+958C.0	fCantonese	kong3
U+958C.0	fMandarin	kang4; kang1
U+958C.0	fEnglish	door
U+958D.0	fUTF8	閍
U+958D.0	fCNS11643	2-4140
U+958D.0	fBig5	DC79
U+958D.0	fGB	5-3D4F
U+958D.0	fR/S	 169.4
U+958D.0	fTotalStrokes	 12
U+958D.0	fCangjie	ANYHS
U+958D.0	fFourCorner	77227
U+958D.0	fCantonese	paang1
U+958D.0	fMandarin	beng1; fang1; mian2
U+958E.0	fUTF8	閎
U+958E.0	fCNS11643	1-6268
U+958E.0	fBig5	B6A3
U+958E.0	fGB	1-6348
U+958E.0	fR/S	 169.4
U+958E.0	fTotalStrokes	 12
U+958E.0	fCangjie	ANKI
U+958E.0	fFourCorner	77430
U+958E.0	fHakka	 fen2
U+958E.0	fCantonese	wang4
U+958E.0	fMandarin	hong2
U+958E.0	fEnglish	[1] gate; barrier across a lane [2] big; large [3] spacious inside [4] a Chinese family name
U+958F.0	fUTF8	閏
U+958F.0	fCNS11643	1-6263
U+958F.0	fBig5	B67C
U+958F.0	fGB	1-4872
U+958F.0	fR/S	 169.4
U+958F.0	fTotalStrokes	 12
U+958F.0	fCangjie	ANMG
U+958F.0	fFourCorner	77104
U+958F.0	fHakka	  jun5; run6; jun3; jun6
U+958F.0	fCantonese	jeon6
U+958F.0	fMandarin	run4
U+958F.0	fEnglish	[1] lef over; extra; with surplus [2] extra; inserted between others (day, month); intercalate
U+9590.0	fUTF8	閐
U+9590.0	fCNS11643	2-4142
U+9590.0	fBig5	DC7B
U+9590.0	fGB	E-4360
U+9590.0	fR/S	 169.4
U+9590.0	fTotalStrokes	 12
U+9590.0	fCangjie	ANHQU
U+9590.0	fFourCorner	77714
U+9590.0	fCantonese	saam3; sam3
U+9590.0	fMandarin	san4
U+9591.0	fUTF8	閑
U+9591.0	fCNS11643	1-6265
U+9591.0	fBig5	B67E
U+9591.0	fGB	1-4F50
U+9591.0	fR/S	 169.4
U+9591.0	fTotalStrokes	 12
U+9591.0	fCangjie	AND
U+9591.0	fFourCorner	77904
U+9591.0	fHakka	  han2; han3
U+9591.0	fCantonese	haan4; haan2 (also)
U+9591.0	fMandarin	xian2
U+9591.0	fEnglish	[1] fence; barrier; bar [2] defend [3] big [4] familiar with; accustomed to; well-versed in [5] same as $ [6] laws; regulations [7] stable
U+9592.0	fUTF8	閒
U+9592.0	fCNS11643	1-6267
U+9592.0	fBig5	B6A2
U+9592.0	fGB	E-4361
U+9592.0	fR/S	 169.4
U+9592.0	fTotalStrokes	 12
U+9592.0	fCangjie	ANB
U+9592.0	fFourCorner	77227
U+9592.0	fHakka	  han2;  gan1; han3; gan5
U+9592.0	fCantonese	haan4; gaan1; gaan3
U+9592.0	fMandarin	xian2; jian3; xia2
U+9592.0	fEnglish	[1] peaceful; tranquil; calm; quiet [2] leisure; spare time
U+9592.1	fUTF8	閒
U+9592.1	fCNS11643	1-6267
U+9592.1	fBig5	B6A2
U+9592.1	fGB	E-4361
U+9592.1	fR/S	 169.4
U+9592.1	fTotalStrokes	 12
U+9592.1	fCangjie	ANB
U+9592.1	fFourCorner	77227
U+9592.1	fMandarin	jian1
U+9592.1	fEnglish	same as $
U+9592.2	fUTF8	閒
U+9592.2	fCNS11643	1-6267
U+9592.2	fBig5	B6A2
U+9592.2	fGB	E-4361
U+9592.2	fR/S	 169.4
U+9592.2	fTotalStrokes	 12
U+9592.2	fCangjie	ANB
U+9592.2	fFourCorner	77227
U+9592.2	fMandarin	jian4
U+9592.2	fEnglish	same as $
U+9593.0	fUTF8	間
U+9593.0	fCNS11643	1-6266
U+9593.0	fBig5	B6A1
U+9593.0	fGB	1-3C64
U+9593.0	fR/S	 169.4
U+9593.0	fTotalStrokes	 12
U+9593.0	fCangjie	ANA
U+9593.0	fFourCorner	77607
U+9593.0	fHakka	  gan1;  gan3; kian1; gien3; gan5; gian1; gian5; gen1; gien1
U+9593.0	fCantonese	gaan1; gaan3; haan4
U+9593.0	fMandarin	jian1; jian3
U+9593.0	fEnglish	[1] interval; between to things; the space between; among [2] a numerical classifier for rooms [3] within a definite time or space
U+9593.1	fUTF8	間
U+9593.1	fCNS11643	1-6266
U+9593.1	fBig5	B6A1
U+9593.1	fGB	1-3C64
U+9593.1	fR/S	 169.4
U+9593.1	fTotalStrokes	 12
U+9593.1	fCangjie	ANA
U+9593.1	fFourCorner	77607
U+9593.1	fMandarin	jian4
U+9593.2	fUTF8	間
U+9593.2	fCNS11643	1-6266
U+9593.2	fBig5	B6A1
U+9593.2	fGB	1-3C64
U+9593.2	fR/S	 169.4
U+9593.2	fTotalStrokes	 12
U+9593.2	fCangjie	ANA
U+9593.2	fFourCorner	77607
U+9593.2	fMandarin	jian5
U+9593.2	fEnglish	[1] crevice; leak; space in between [2] divide; a division of a house; separate [3] put space between; drive a wedge between; part friends [4] change; substitute [5] block up [6] vet a little better (illness) [7] occasionally
U+9594.0	fUTF8	閔
U+9594.0	fCNS11643	1-6262
U+9594.0	fBig5	B67B
U+9594.0	fGB	1-6349
U+9594.0	fR/S	 169.4
U+9594.0	fTotalStrokes	 12
U+9594.0	fCangjie	ANYK
U+9594.0	fFourCorner	77400
U+9594.0	fHakka	  men3; mun3; men1
U+9594.0	fCantonese	man5
U+9594.0	fMandarin	min3; min2
U+9594.0	fEnglish	[1] mourn; grieve; pity; commiserat; be symphathetic; condole [2] urge; incite; encourage [3] trouble; distress [4] be grieved [5] a Chinese family name
U+9595.0	fUTF8	閕
U+9595.0	fCNS11643	3-3F6C
U+9595.0	fGB	3-3A76
U+9595.0	fR/S	  169.4
U+9595.0	fTotalStrokes	 12
U+9595.0	fCangjie	ANMVH
U+9595.0	fFourCorner	77400
U+9595.0	fHakka	  ha2; hia1; hia5; nga1
U+9595.0	fCantonese	ngaa6; aa6 (also); haa1
U+9595.0	fMandarin	xia1; xia4
U+9595.0	fEnglish	 set a door ajar
U+9596.0	fUTF8	閖
U+9596.0	fCNS11643	F-3F7E
U+9596.0	fGB	E-4362
U+9596.0	fR/S	 169.4
U+9596.0	fTotalStrokes	 12
U+9596.0	fCangjie	ANE
U+9596.0	fCantonese	seoi2
U+9596.0	fMandarin	min3
U+9596.0	fEnglish	flower name; a water-gate, sluice; toponym
U+9597.0	fUTF8	閗
U+9597.0	fCNS11643	3-666B
U+9597.0	fGB	E-4363
U+9597.0	fR/S	 169.4
U+9597.0	fTotalStrokes	 12
U+9597.0	fCangjie	ANYJ
U+9597.0	fFourCorner	7740
U+9597.0	fHakka	 deu5
U+9597.0	fCantonese	dau3
U+9597.0	fMandarin	dou4
U+9598.0	fUTF8	閘
U+9598.0	fCNS11643	1-6750
U+9598.0	fBig5	B968
U+9598.0	fGB	1-5522
U+9598.0	fR/S	 169.5
U+9598.0	fTotalStrokes	 13
U+9598.0	fCangjie	ANWL
U+9598.0	fFourCorner	77506
U+9598.0	fHakka	  ap7; cak8; {gap7}; shan5
U+9598.0	fCantonese	zaap6
U+9598.0	fMandarin	zha2; ya1; ge2
U+9598.0	fEnglish	[1] sluice; flood gate; canal lock [2] brake; device on a vehicle for halting its motion [3] a switch or similar devices
U+9599.0	fUTF8	閙
U+9599.0	fCNS11643	3-453C
U+9599.0	fGB	E-4364
U+9599.0	fR/S	 169.5
U+9599.0	fTotalStrokes	 13
U+9599.0	fCangjie	ANYLB
U+9599.0	fFourCorner	77220
U+9599.0	fHakka	 nau5; nau6
U+9599.0	fCantonese	naau6; laau6 (also)
U+9599.0	fMandarin	nao4; zha2
U+9599.0	fEnglish	 quarrel dispute hotly
U+959A.0	fUTF8	閚
U+959A.0	fCNS11643	3-453A
U+959A.0	fGB	E-4365
U+959A.0	fR/S	  169.5
U+959A.0	fTotalStrokes	 13
U+959A.0	fCangjie	ANYT
U+959A.0	fFourCorner	77100
U+959A.0	fCantonese	zaam1
U+959A.0	fMandarin	jian1
U+959B.0	fUTF8	閛
U+959B.0	fCNS11643	2-4856
U+959B.0	fBig5	E0D0
U+959B.0	fGB	3-3A7A
U+959B.0	fR/S	 169.5
U+959B.0	fTotalStrokes	 13
U+959B.0	fCangjie	ANMFJ
U+959B.0	fFourCorner	77409
U+959B.0	fHakka	 pin2
U+959B.0	fCantonese	ping1; ping6
U+959B.0	fMandarin	peng1; peng4
U+959B.0	fEnglish	the cracking of a door
U+959C.0	fUTF8	閜
U+959C.0	fCNS11643	2-4854
U+959C.0	fBig5	E0CE
U+959C.0	fGB	3-3A79
U+959C.0	fR/S	 169.5
U+959C.0	fTotalStrokes	 13
U+959C.0	fCangjie	ANMNR
U+959C.0	fFourCorner	77621
U+959C.0	fHakka	 ha5;  ko3
U+959C.0	fCantonese	haa1; ho2; haa3; ho3
U+959C.0	fMandarin	xia3; ke3; xia4
U+959C.0	fEnglish	 throw wide open as a door
U+959D.0	fUTF8	閝
U+959D.0	fCNS11643	4-4153
U+959D.0	fGB	E-4366
U+959D.0	fR/S	  169.5
U+959D.0	fTotalStrokes	 13
U+959D.0	fCangjie	ANOII
U+959D.0	fFourCorner	77300
U+959D.0	fCantonese	ling4
U+959D.0	fMandarin	ling2
U+959E.0	fUTF8	閞
U+959E.0	fCNS11643	2-4855
U+959E.0	fBig5	E0CF
U+959E.0	fGB	E-4367
U+959E.0	fR/S	 169.5
U+959E.0	fTotalStrokes	 13
U+959E.0	fCangjie	ANIT
U+959E.0	fFourCorner	77442
U+959E.0	fCantonese	bin6; faan6
U+959E.0	fMandarin	bian4; fan4
U+959F.0	fUTF8	閟
U+959F.0	fCNS11643	2-4853
U+959F.0	fBig5	E0CD
U+959F.0	fGB	3-3A7D
U+959F.0	fR/S	 169.5
U+959F.0	fTotalStrokes	 13
U+959F.0	fCangjie	ANPH
U+959F.0	fFourCorner	77330
U+959F.0	fHakka	  bi5
U+959F.0	fCantonese	bei3
U+959F.0	fMandarin	bi4
U+959F.0	fEnglish	[1] close the door [2] shut; repress [3] deep; obscure; solemn
U+95A0.0	fUTF8	閠
U+95A0.0	fCNS11643	F-457B
U+95A0.0	fGB	E-4368
U+95A0.0	fR/S	 169.5
U+95A0.0	fTotalStrokes	 13
U+95A0.0	fCangjie	ANMGI
U+95A0.0	fFourCorner	7710
U+95A0.0	fCantonese	jeon6
U+95A0.0	fMandarin	run4; nao4
U+95A0.0	fEnglish	 intercalary extra; surplus
U+95A1.0	fUTF8	閡
U+95A1.0	fCNS11643	1-6B5B
U+95A1.0	fBig5	BBD2
U+95A1.0	fGB	1-3A52
U+95A1.0	fR/S	 169.6
U+95A1.0	fTotalStrokes	 14
U+95A1.0	fCangjie	ANYVO
U+95A1.0	fFourCorner	77280
U+95A1.0	fHakka	 ngoi5; ngoi3; ngoi6; ngoe3
U+95A1.0	fCantonese	hat6; goi1; ngoi6; oi6 (also)
U+95A1.0	fMandarin	he2; ai4; hai4; kai3; gai1
U+95A1.0	fEnglish	[1] block; seperate; obstruct; hinder [2] prevent; shut out; shut a door to keep others out
U+95A2.0	fUTF8	関
U+95A2.0	fCNS11643	3-4A3D
U+95A2.0	fGB	E-4369
U+95A2.0	fR/S	 169.6
U+95A2.0	fTotalStrokes	 14
U+95A2.0	fCangjie	ANTK
U+95A2.0	fFourCorner	77430
U+95A2.0	fHakka	 gwan1
U+95A2.0	fCantonese	gwaan1; gwan1
U+95A2.0	fMandarin	guan1; wan1; wan3
U+95A2.0	fEnglish	 frontier pass close relation
U+95A3.0	fUTF8	閣
U+95A3.0	fCNS11643	1-6B5E
U+95A3.0	fBig5	BBD5
U+95A3.0	fGB	1-3873
U+95A3.0	fR/S	 169.6
U+95A3.0	fTotalStrokes	 14
U+95A3.0	fCangjie	ANHER
U+95A3.0	fFourCorner	77604
U+95A3.0	fHakka	  gok7
U+95A3.0	fCantonese	gok3; gok2 (also)
U+95A3.0	fMandarin	ge2; gao3
U+95A3.0	fEnglish	 chamber; pavilion cabinet
U+95A4.0	fUTF8	閤
U+95A4.0	fCNS11643	1-6B60
U+95A4.0	fBig5	BBD7
U+95A4.0	fGB	1-7839
U+95A4.0	fR/S	 169.6
U+95A4.0	fTotalStrokes	 14
U+95A4.0	fCangjie	ANOMR
U+95A4.0	fFourCorner	77601
U+95A4.0	fHakka	 hap8
U+95A4.0	fCantonese	gap3; hap6; gok3
U+95A4.0	fMandarin	he2; ge2
U+95A4.0	fEnglish	[1] small side door [2] same as $
U+95A5.0	fUTF8	閥
U+95A5.0	fCNS11643	1-6B5F
U+95A5.0	fBig5	BBD6
U+95A5.0	fGB	1-3727
U+95A5.0	fR/S	 169.6
U+95A5.0	fTotalStrokes	 14
U+95A5.0	fCangjie	ANOI
U+95A5.0	fFourCorner	77253
U+95A5.0	fHakka	 fat8
U+95A5.0	fCantonese	fat6
U+95A5.0	fMandarin	fa2
U+95A5.0	fEnglish	[1] threshold; doorsill [2] powerful and influential group; an influential person, family, or clique; bloc [3] valve
U+95A6.0	fUTF8	閦
U+95A6.0	fCNS11643	4-4822
U+95A6.0	fGB	3-3B22
U+95A6.0	fR/S	 169.6
U+95A6.0	fTotalStrokes	 14
U+95A6.0	fCangjie	ANOOO
U+95A6.0	fFourCorner	77880
U+95A6.0	fHakka	 cuk7
U+95A6.0	fCantonese	cuk1
U+95A6.0	fMandarin	chu4
U+95A6.0	fEnglish	 crowd transliteration of Sanskrit "kSo"; e.g. akSobhya
U+95A7.0	fUTF8	閧
U+95A7.0	fCNS11643	3-4A3E
U+95A7.0	fGB	E-436A
U+95A7.0	fR/S	 169.6
U+95A7.0	fTotalStrokes	 14
U+95A7.0	fCangjie	ANTC
U+95A7.0	fFourCorner	77800
U+95A7.0	fHakka	  fung5; kiung6; gung3; pung5
U+95A7.0	fCantonese	hung3; hung6 (also)
U+95A7.0	fMandarin	hong4; hong1; hong3; gui1
U+95A7.0	fEnglish	 boisterous clamor; noise
U+95A8.0	fUTF8	閨
U+95A8.0	fCNS11643	1-6B5C
U+95A8.0	fBig5	BBD3
U+95A8.0	fGB	1-396B
U+95A8.0	fR/S	 169.6
U+95A8.0	fTotalStrokes	 14
U+95A8.0	fCangjie	ANGG
U+95A8.0	fFourCorner	77104
U+95A8.0	fHakka	 gui1; kui1; gwi1; kwui1
U+95A8.0	fCantonese	gwai1
U+95A8.0	fMandarin	gui1
U+95A8.0	fEnglish	[1] small entrance; small door [2] women's quarters; womens apartment [3] feminine
U+95A9.0	fUTF8	閩
U+95A9.0	fCNS11643	1-6B5D
U+95A9.0	fBig5	BBD4
U+95A9.0	fGB	1-4376
U+95A9.0	fR/S	 169.6
U+95A9.0	fTotalStrokes	 14
U+95A9.0	fCangjie	ANLMI
U+95A9.0	fFourCorner	77136
U+95A9.0	fHakka	 men2;  man2; men1; men3; min2
U+95A9.0	fCantonese	man5; man4
U+95A9.0	fMandarin	min3; man2; min2; wen2
U+95A9.0	fEnglish	[1] another name for Fukien province [2] a river in Fukien province [3] a tribe in Fukien province [4] a Chinese family name
U+95AA.0	fUTF8	閪
U+95AA.0	fGB	1-7D4B
U+95AA.0	fR/S	 169.6
U+95AA.0	fTotalStrokes	 14
U+95AA.0	fCangjie	ANMCW
U+95AA.0	fCantonese	hai1
U+95AA.0	fEnglish	 lose lost item
U+95AB.0	fUTF8	閫
U+95AB.0	fCNS11643	2-5550
U+95AB.0	fBig5	E8A7
U+95AB.0	fGB	1-634D
U+95AB.0	fR/S	 169.7
U+95AB.0	fTotalStrokes	 15
U+95AB.0	fCangjie	ANWD
U+95AB.0	fFourCorner	77600
U+95AB.0	fHakka	  kun5; kun3; kwun5
U+95AB.0	fCantonese	kwan2
U+95AB.0	fMandarin	kun3; kun4
U+95AB.0	fEnglish	[1] same as $ threshold [2] women's quarters; apartments or quarter for ladies [3] feminine
U+95AC.0	fUTF8	閬
U+95AC.0	fCNS11643	2-554F
U+95AC.0	fBig5	E8A6
U+95AC.0	fGB	1-634F
U+95AC.0	fR/S	 169.7
U+95AC.0	fTotalStrokes	 15
U+95AC.0	fCangjie	ANIAV
U+95AC.0	fFourCorner	77732
U+95AC.0	fHakka	 long5; long6
U+95AC.0	fCantonese	long5; long6; long4
U+95AC.0	fMandarin	lang3; lang2; du1; lang4; liang3; tang2
U+95AC.0	fEnglish	[1] high door; high gate [2] high; loft; tall; big [3] clear; bright; lucid [4] open and spacious
U+95AD.0	fUTF8	閭
U+95AD.0	fCNS11643	1-6F64
U+95AD.0	fBig5	BE5B
U+95AD.0	fGB	1-634C
U+95AD.0	fR/S	 169.7
U+95AD.0	fTotalStrokes	 15
U+95AD.0	fCangjie	ANRHR
U+95AD.0	fFourCorner	77606
U+95AD.0	fHakka	 li1;  lu2
U+95AD.0	fCantonese	leoi4; leoi5 (also)
U+95AD.0	fMandarin	l2; luu2; hong4; lu2
U+95AD.0	fEnglish	[1] a community or village of twenty-five families in ancient China; community; neighborhood [2] gate of a village [3] meet; gather together [4] a Chinese family name
U+95AE.0	fUTF8	閮
U+95AE.0	fCNS11643	2-5551
U+95AE.0	fBig5	E8A8
U+95AE.0	fGB	5-3D5A
U+95AE.0	fR/S	 169.7
U+95AE.0	fTotalStrokes	 15
U+95AE.0	fCangjie	ANNKG
U+95AE.0	fFourCorner	77401
U+95AE.0	fCantonese	ting4; ting5
U+95AE.0	fMandarin	ting2; ting3
U+95AF.0	fUTF8	閯
U+95AF.0	fCNS11643	3-4F53
U+95AF.0	fGB	E-436B
U+95AF.0	fR/S	  169.7
U+95AF.0	fTotalStrokes	 15
U+95AF.0	fCangjie	ANEFH
U+95AF.0	fFourCorner	77120
U+95AF.0	fHakka	 sa5
U+95AF.0	fCantonese	saa3
U+95AF.0	fMandarin	sha4
U+95B0.0	fUTF8	閰
U+95B0.0	fCNS11643	2-5552
U+95B0.0	fBig5	E8A9
U+95B0.0	fGB	E-436C
U+95B0.0	fR/S	 169.7
U+95B0.0	fTotalStrokes	 15
U+95B0.0	fCangjie	ANHX
U+95B0.0	fFourCorner	77777
U+95B0.0	fCantonese	guk1
U+95B0.0	fMandarin	ju2; yan2
U+95B1.0	fUTF8	閱
U+95B1.0	fCNS11643	1-6F65
U+95B1.0	fBig5	BE5C
U+95B1.0	fGB	E-436D
U+95B1.0	fR/S	 169.7
U+95B1.0	fTotalStrokes	 15
U+95B1.0	fCangjie	ANCRU
U+95B1.0	fFourCorner	77216
U+95B1.0	fHakka	  jet8; ret8; jat8
U+95B1.0	fCantonese	jyut6
U+95B1.0	fMandarin	yue4
U+95B1.0	fEnglish	[1] examine; inspect; review; observe [2] read; go over (examination papers) [3] experience [4] pass [5] gather; collect [6] appearance; looks
U+95B2.0	fUTF8	閲
U+95B2.0	fCNS11643	3-4F55
U+95B2.0	fGB	1-5444
U+95B2.0	fR/S	 169.7
U+95B2.0	fTotalStrokes	 15
U+95B2.0	fCangjie	XANCR
U+95B2.0	fFourCorner	77210
U+95B2.0	fCantonese	jyut6
U+95B2.0	fMandarin	yue4
U+95B2.0	fEnglish	examine
U+95B3.0	fUTF8	閳
U+95B3.0	fCNS11643	3-4F56
U+95B3.0	fGB	E-436E
U+95B3.0	fR/S	 169.7
U+95B3.0	fTotalStrokes	 15
U+95B3.0	fCangjie	ANJWJ
U+95B3.0	fFourCorner	77500
U+95B3.0	fCantonese	zin2
U+95B3.0	fMandarin	chan3
U+95B4.0	fUTF8	閴
U+95B4.0	fCNS11643	4-4E4A
U+95B4.0	fGB	E-436F
U+95B4.0	fR/S	 169.7
U+95B4.0	fTotalStrokes	 15
U+95B4.0	fCangjie	ANBUC
U+95B4.0	fFourCorner	77800
U+95B4.0	fCantonese	gwik1
U+95B4.0	fMandarin	qu4
U+95B4.0	fEnglish	quiet
U+95B5.0	fUTF8	閵
U+95B5.0	fCNS11643	2-5B59
U+95B5.0	fBig5	EC4D
U+95B5.0	fGB	5-3D65
U+95B5.0	fR/S	 169.8
U+95B5.0	fTotalStrokes	 16
U+95B5.0	fCangjie	ANOG
U+95B5.0	fFourCorner	77214
U+95B5.0	fHakka	 nin5
U+95B5.0	fCantonese	leon6
U+95B5.0	fMandarin	lin4; lian4
U+95B6.0	fUTF8	閶
U+95B6.0	fCNS11643	2-5B57
U+95B6.0	fBig5	EC4B
U+95B6.0	fGB	1-6351
U+95B6.0	fR/S	 169.8
U+95B6.0	fTotalStrokes	 16
U+95B6.0	fCangjie	ANAA
U+95B6.0	fFourCorner	77606
U+95B6.0	fHakka	 cong1;  chong1
U+95B6.0	fCantonese	coeng1
U+95B6.0	fMandarin	chang1; tang1
U+95B6.0	fEnglish	[1] gate of heaven [2] main gate of palace; front gate of a palace
U+95B7.0	fUTF8	閷
U+95B7.0	fCNS11643	2-6040
U+95B7.0	fBig5	EEF3
U+95B7.0	fGB	3-415F
U+95B7.0	fR/S	 169.9
U+95B7.0	fTotalStrokes	 17
U+95B7.0	fCangjie	KCANO
U+95B7.0	fFourCorner	47920
U+95B7.0	fCantonese	saat3; saai3
U+95B7.0	fMandarin	shai4; sha1
U+95B8.0	fUTF8	閸
U+95B8.0	fCNS11643	3-5342
U+95B8.0	fGB	E-4370
U+95B8.0	fR/S	  169.8
U+95B8.0	fTotalStrokes	 16
U+95B8.0	fCangjie	ANMLM
U+95B8.0	fFourCorner	77100
U+95B8.0	fCantonese	kwan2
U+95B8.0	fMandarin	kun3
U+95B9.0	fUTF8	閹
U+95B9.0	fCNS11643	2-5B55
U+95B9.0	fBig5	EC49
U+95B9.0	fGB	1-514B
U+95B9.0	fR/S	 169.8
U+95B9.0	fTotalStrokes	 16
U+95B9.0	fCangjie	ANKLU
U+95B9.0	fFourCorner	77716
U+95B9.0	fHakka	  jam1; ram1
U+95B9.0	fCantonese	jim1
U+95B9.0	fMandarin	yan1; yan2; yan3
U+95B9.0	fEnglish	[1] castrate [2] eunuch
U+95BA.0	fUTF8	閺
U+95BA.0	fCNS11643	2-5B56
U+95BA.0	fBig5	EC4A
U+95BA.0	fGB	5-3D63
U+95BA.0	fR/S	 169.8
U+95BA.0	fTotalStrokes	 16
U+95BA.0	fCangjie	ANAYK
U+95BA.0	fFourCorner	77404
U+95BA.0	fCantonese	man4
U+95BA.0	fMandarin	wen2; min2
U+95BB.0	fUTF8	閻
U+95BB.0	fCNS11643	1-726F
U+95BB.0	fBig5	C046
U+95BB.0	fGB	1-5156
U+95BB.0	fR/S	 169.8
U+95BB.0	fTotalStrokes	 16
U+95BB.0	fCangjie	ANNHX
U+95BB.0	fFourCorner	77777
U+95BB.0	fHakka	  ngiam2;  jam2; niam2; gniam3; ram2
U+95BB.0	fCantonese	jim4
U+95BB.0	fMandarin	yan2; yan3; yan4; e4; xian2; yan1
U+95BB.0	fEnglish	[1] village gate; gate of a lane [2] a Chinese family name
U+95BC.0	fUTF8	閼
U+95BC.0	fCNS11643	2-5B52
U+95BC.0	fBig5	EC46
U+95BC.0	fGB	1-6355
U+95BC.0	fR/S	 169.8
U+95BC.0	fTotalStrokes	 16
U+95BC.0	fCangjie	ANYSY
U+95BC.0	fFourCorner	77233
U+95BC.0	fHakka	  ot7;  at7;  hot7; jen1
U+95BC.0	fCantonese	aat1 (also); jin1; ngaat1 (also); aat3; ngaat3 (also); jyu3
U+95BC.0	fMandarin	e4; yu4; ye4
U+95BC.0	fEnglish	[1] block; obstruct; stop up [2] anything used to block up a flow [3] anger; angry
U+95BC.1	fUTF8	閼
U+95BC.1	fCNS11643	2-5B52
U+95BC.1	fBig5	EC46
U+95BC.1	fGB	1-6355
U+95BC.1	fR/S	 169.8
U+95BC.1	fTotalStrokes	 16
U+95BC.1	fCangjie	ANYSY
U+95BC.1	fFourCorner	77233
U+95BC.1	fMandarin	yan1
U+95BC.1	fEnglish	the formal wife of the chieftain of the Huns in the Han dynasty
U+95BD.0	fUTF8	閽
U+95BD.0	fCNS11643	2-5B5A
U+95BD.0	fBig5	EC4E
U+95BD.0	fGB	1-6354
U+95BD.0	fR/S	 169.8
U+95BD.0	fTotalStrokes	 16
U+95BD.0	fCangjie	ANHPA
U+95BD.0	fFourCorner	77605
U+95BD.0	fHakka	 fun1
U+95BD.0	fCantonese	fan1
U+95BD.0	fMandarin	hun1
U+95BD.0	fEnglish	[1] gatekeeper [2] gate; door; palace gate
U+95BE.0	fUTF8	閾
U+95BE.0	fCNS11643	2-5B54
U+95BE.0	fBig5	EC48
U+95BE.0	fGB	1-6350
U+95BE.0	fR/S	 169.8
U+95BE.0	fTotalStrokes	 16
U+95BE.0	fCangjie	ANIRM
U+95BE.0	fFourCorner	77153
U+95BE.0	fHakka	 wet7; {het8}; vet7
U+95BE.0	fCantonese	wik6; kwik1
U+95BE.0	fMandarin	yu4; xu4; yan1
U+95BE.0	fEnglish	[1] doorsill; threshold [2] separated [3] confined
U+95BF.0	fUTF8	閿
U+95BF.0	fCNS11643	2-5B58
U+95BF.0	fBig5	EC4C
U+95BF.0	fGB	1-6353
U+95BF.0	fR/S	 169.8
U+95BF.0	fTotalStrokes	 16
U+95BF.0	fCangjie	ANBBE
U+95BF.0	fFourCorner	77407
U+95BF.0	fHakka	 wun2; vun2
U+95BF.0	fCantonese	man4
U+95BF.0	fMandarin	wen2; min2
U+95BF.0	fEnglish	Wen xiang; Henan province
U+95C0.0	fUTF8	闀
U+95C0.0	fCNS11643	2-603C
U+95C0.0	fBig5	EEEF
U+95C0.0	fGB	3-3B29
U+95C0.0	fR/S	 169.8
U+95C0.0	fTotalStrokes	 16
U+95C0.0	fCangjie	ANTCU
U+95C0.0	fFourCorner	77712
U+95C0.0	fHakka	 hong5; hong6
U+95C0.0	fCantonese	hung3; hung6
U+95C0.0	fMandarin	xiang4
U+95C1.0	fUTF8	闁
U+95C1.0	fCNS11643	3-5344
U+95C1.0	fGB	E-4371
U+95C1.0	fR/S	  169.8
U+95C1.0	fTotalStrokes	 16
U+95C1.0	fCangjie	ANAN
U+95C1.0	fFourCorner	77220
U+95C1.0	fCantonese	bou1
U+95C1.0	fMandarin	bao1
U+95C2.0	fUTF8	闂
U+95C2.0	fCNS11643	3-5340
U+95C2.0	fGB	E-4372
U+95C2.0	fR/S	  169.8
U+95C2.0	fTotalStrokes	 16
U+95C2.0	fCangjie	ANFQU
U+95C2.0	fFourCorner	77710
U+95C2.0	fCantonese	hung3
U+95C2.0	fMandarin	xiang4
U+95C3.0	fUTF8	闃
U+95C3.0	fCNS11643	2-603E
U+95C3.0	fBig5	EEF1
U+95C3.0	fGB	1-6356
U+95C3.0	fR/S	 169.9
U+95C3.0	fTotalStrokes	 17
U+95C3.0	fCangjie	ANBUK
U+95C3.0	fFourCorner	77430
U+95C3.0	fHakka	  hit7;  ngiet7; kiet7; ket7
U+95C3.0	fCantonese	gwik1; kwik1
U+95C3.0	fMandarin	qu4; kuo4
U+95C3.0	fEnglish	alone; quiet; without people around
U+95C4.0	fUTF8	闄
U+95C4.0	fCNS11643	3-566D
U+95C4.0	fGB	3-3B2B
U+95C4.0	fR/S	  169.9
U+95C4.0	fTotalStrokes	 17
U+95C4.0	fCangjie	ANMWV
U+95C4.0	fFourCorner	77400
U+95C4.0	fCantonese	jiu2
U+95C4.0	fMandarin	yao3
U+95C5.0	fUTF8	闅
U+95C5.0	fCNS11643	2-603F
U+95C5.0	fBig5	EEF2
U+95C5.0	fGB	E-4373
U+95C5.0	fR/S	 169.9
U+95C5.0	fTotalStrokes	 17
U+95C5.0	fCangjie	ANBUE
U+95C5.0	fFourCorner	77407
U+95C5.0	fHakka	 wun2; vun2
U+95C5.0	fCantonese	man4; gwik1
U+95C5.0	fMandarin	wen2; min2
U+95C5.0	fEnglish	[1] look closely at [2] an ancient form of $
U+95C6.0	fUTF8	闆
U+95C6.0	fCNS11643	1-757E
U+95C6.0	fBig5	C1F3
U+95C6.0	fGB	1-7822
U+95C6.0	fR/S	 169.9
U+95C6.0	fTotalStrokes	 17
U+95C6.0	fCangjie	ANRRR
U+95C6.0	fFourCorner	77666
U+95C6.0	fHakka	 ban3
U+95C6.0	fCantonese	baan2
U+95C6.0	fMandarin	ban3
U+95C6.0	fEnglish	 boss; the owner; person in charge
U+95C7.0	fUTF8	闇
U+95C7.0	fCNS11643	2-603B
U+95C7.0	fBig5	EEEE
U+95C7.0	fGB	E-4374
U+95C7.0	fR/S	 169.9
U+95C7.0	fTotalStrokes	 17
U+95C7.0	fCangjie	ANYTA
U+95C7.0	fFourCorner	77601
U+95C7.0	fHakka	  am5
U+95C7.0	fCantonese	am2; ngam2 (also); am1; am3; ngam1 (also); ngam3 (also)
U+95C7.0	fMandarin	an4; yan3; an3; an1; yin1; yin3
U+95C7.0	fEnglish	[1] close; shut the door [2] dark; dismal; obscure; obscurity [3] evening; night [4] lunar or solar eclipses [5] stupid and dull; ignorant and foolish [6] retire [7] private
U+95C8.0	fUTF8	闈
U+95C8.0	fCNS11643	1-757D
U+95C8.0	fBig5	C1F2
U+95C8.0	fGB	1-6347
U+95C8.0	fR/S	 169.9; 169.10
U+95C8.0	fTotalStrokes	 17
U+95C8.0	fCangjie	ANDMQ
U+95C8.0	fFourCorner	77506
U+95C8.0	fHakka	 wui2; wi2; wui3; vui2
U+95C8.0	fCantonese	wai4
U+95C8.0	fMandarin	wei2; hui1
U+95C8.0	fEnglish	[1] side doors of a palace [2] living quarters of the queen and the imperial concubines [3] ladies' living quarters; private quarters [4] a hall where the civil service examinations took place
U+95C9.0	fUTF8	闉
U+95C9.0	fCNS11643	2-603D
U+95C9.0	fBig5	EEF0
U+95C9.0	fGB	3-3B2A
U+95C9.0	fR/S	 169.9
U+95C9.0	fTotalStrokes	 17
U+95C9.0	fCangjie	ANMWG
U+95C9.0	fFourCorner	77104
U+95C9.0	fHakka	 jin1;  jen1; rin1; ren1
U+95C9.0	fCantonese	jan1
U+95C9.0	fMandarin	yin1; yan1
U+95C9.0	fEnglish	[1] curved; bent [2] the gate of the city wall
U+95CA.0	fUTF8	闊
U+95CA.0	fCNS11643	1-757A
U+95CA.0	fBig5	C1EF
U+95CA.0	fGB	1-402B
U+95CA.0	fR/S	 169.9
U+95CA.0	fTotalStrokes	 17
U+95CA.0	fCangjie	ANEHR
U+95CA.0	fFourCorner	77164
U+95CA.0	fHakka	  fat7;  k'wat7; kwat7; gwat7
U+95CA.0	fCantonese	fut3
U+95CA.0	fMandarin	kuo4; yu4
U+95CA.0	fEnglish	[1] broad; ample; wide [2] widely apart; separated [3] rich; wealthy; extravagant; loaded [4] ease
U+95CB.0	fUTF8	闋
U+95CB.0	fCNS11643	1-757B
U+95CB.0	fBig5	C1F0
U+95CB.0	fGB	1-6357
U+95CB.0	fR/S	 169.9
U+95CB.0	fTotalStrokes	 17
U+95CB.0	fCangjie	ANNOK
U+95CB.0	fFourCorner	77431
U+95CB.0	fHakka	 kiet7; kui2; ket7
U+95CB.0	fCantonese	kyut3
U+95CB.0	fMandarin	que4; jue2; gui4; kui2
U+95CB.0	fEnglish	[1] close; shut the door after finishing something [2] be at rest [3] end [4] the expiry of the period of mourning [5] a numerical classifier for songs [6] empty; blank
U+95CC.0	fUTF8	闌
U+95CC.0	fCNS11643	1-757C
U+95CC.0	fBig5	C1F1
U+95CC.0	fGB	1-403B
U+95CC.0	fR/S	 169.9
U+95CC.0	fTotalStrokes	 17
U+95CC.0	fCangjie	ANDWF
U+95CC.0	fFourCorner	77906
U+95CC.0	fHakka	  lan2; {nang2}; lan3
U+95CC.0	fCantonese	laan4
U+95CC.0	fMandarin	lan2; lan4; du1; she2
U+95CC.0	fEnglish	[1] door screen; door curtain [2] railing ; fence [3] block up; cut off [4] the end of (a year, etc.) [5] weakened; withered
U+95CD.0	fUTF8	闍
U+95CD.0	fCNS11643	2-5B53
U+95CD.0	fBig5	EC47
U+95CD.0	fGB	3-3B24
U+95CD.0	fR/S	 169.9
U+95CD.0	fTotalStrokes	 17
U+95CD.0	fCangjie	ANJKA
U+95CD.0	fFourCorner	77604
U+95CD.0	fHakka	  du1; sha2
U+95CD.0	fCantonese	dou1; se4; ze1 (also); se6
U+95CD.0	fMandarin	du1; du2; qu4; zhe1
U+95CD.0	fEnglish	tower over a city wall
U+95CD.1	fUTF8	闍
U+95CD.1	fCNS11643	2-5B53
U+95CD.1	fBig5	EC47
U+95CD.1	fGB	3-3B24
U+95CD.1	fR/S	 169.9
U+95CD.1	fTotalStrokes	 17
U+95CD.1	fCangjie	ANJKA
U+95CD.1	fFourCorner	77604
U+95CD.1	fMandarin	she2
U+95CD.1	fEnglish	correct form of &#x a Buddhist high priest;
U+95CE.0	fUTF8	闎
U+95CE.0	fCNS11643	3-666C
U+95CE.0	fGB	E-4375
U+95CE.0	fR/S	 169.9
U+95CE.0	fTotalStrokes	 17
U+95CE.0	fCangjie	ANHAE
U+95CE.0	fFourCorner	7723
U+95CE.0	fCantonese	cyun4
U+95CF.0	fUTF8	闏
U+95CF.0	fGB	1-7D63
U+95CF.0	fR/S	 169.9
U+95CF.0	fTotalStrokes	 17
U+95CF.0	fCangjie	ANHNI
U+95CF.0	fCantonese	fung1
U+95D0.0	fUTF8	闐
U+95D0.0	fCNS11643	1-7760
U+95D0.0	fBig5	C2F5
U+95D0.0	fGB	1-6359
U+95D0.0	fR/S	 169.10
U+95D0.0	fTotalStrokes	 18
U+95D0.0	fCangjie	ANJBC
U+95D0.0	fFourCorner	77801
U+95D0.0	fHakka	  tien2; ten2
U+95D0.0	fCantonese	tin4
U+95D0.0	fMandarin	tian2; dian4; ji4
U+95D0.0	fEnglish	 fill; fill to the brim; be full of
U+95D1.0	fUTF8	闑
U+95D1.0	fCNS11643	2-643A
U+95D1.0	fBig5	F16E
U+95D1.0	fGB	3-3B30
U+95D1.0	fR/S	 169.10
U+95D1.0	fTotalStrokes	 18
U+95D1.0	fCangjie	ANHUD
U+95D1.0	fFourCorner	77904
U+95D1.0	fHakka	 ngiet7; nget7
U+95D1.0	fCantonese	jit6; nip6 (also); lip6 (also); jip6
U+95D1.0	fMandarin	nie4; gui4
U+95D1.0	fEnglish	the wood bars on both side of the central door
U+95D2.0	fUTF8	闒
U+95D2.0	fCNS11643	2-6438
U+95D2.0	fBig5	F16C
U+95D2.0	fGB	3-3B2F
U+95D2.0	fR/S	 169.10
U+95D2.0	fTotalStrokes	 18
U+95D2.0	fCangjie	ANASM
U+95D2.0	fFourCorner	77127
U+95D2.0	fHakka	  tap7
U+95D2.0	fCantonese	taap3; daap6 (also)
U+95D2.0	fMandarin	ta4; da2; la4
U+95D2.0	fEnglish	[1] upper-story door; upstairs door [2] the sound of bells and drums [3] mean
U+95D3.0	fUTF8	闓
U+95D3.0	fCNS11643	2-6439
U+95D3.0	fBig5	F16D
U+95D3.0	fGB	3-3B21
U+95D3.0	fR/S	 169.10
U+95D3.0	fTotalStrokes	 18
U+95D3.0	fCangjie	ANUMT
U+95D3.0	fFourCorner	77108
U+95D3.0	fHakka	  koi3
U+95D3.0	fCantonese	hoi2; hoi1
U+95D3.0	fMandarin	kai3; kai1
U+95D3.0	fEnglish	[1] open [2] peaceful; harmonious
U+95D4.0	fUTF8	闔
U+95D4.0	fCNS11643	1-775E
U+95D4.0	fBig5	C2F3
U+95D4.0	fGB	1-6358
U+95D4.0	fR/S	 169.10
U+95D4.0	fTotalStrokes	 18
U+95D4.0	fCangjie	ANGIT
U+95D4.0	fFourCorner	77107
U+95D4.0	fHakka	  hap8; kap7
U+95D4.0	fCantonese	hap6
U+95D4.0	fMandarin	he2; que1; ge2; que4
U+95D4.0	fEnglish	[1] leaf of a door [2] close; shut [3] whole; entire; all [4] Why?; Why not?
U+95D5.0	fUTF8	闕
U+95D5.0	fCNS11643	1-7761
U+95D5.0	fBig5	C2F6
U+95D5.0	fGB	1-635A
U+95D5.0	fR/S	 169.10
U+95D5.0	fTotalStrokes	 18
U+95D5.0	fCangjie	ANTUO
U+95D5.0	fFourCorner	77482
U+95D5.0	fHakka	  kiet7; ket7; k'iat7
U+95D5.0	fCantonese	kyut3; gwat6
U+95D5.0	fMandarin	que4; jue2; lan2
U+95D5.0	fEnglish	[1] watch tower outside the palace gate in ancient China [2] palace [3] a Chinese family name
U+95D5.1	fUTF8	闕
U+95D5.1	fCNS11643	1-7761
U+95D5.1	fBig5	C2F6
U+95D5.1	fGB	1-635A
U+95D5.1	fR/S	 169.10
U+95D5.1	fTotalStrokes	 18
U+95D5.1	fCangjie	ANTUO
U+95D5.1	fFourCorner	77482
U+95D5.1	fMandarin	que1
U+95D5.1	fEnglish	[1] faults; errors; mistakes; defects [2] lack; deficient; deficiency
U+95D6.0	fUTF8	闖
U+95D6.0	fCNS11643	1-775F
U+95D6.0	fBig5	C2F4
U+95D6.0	fGB	1-3433
U+95D6.0	fR/S	 169.10
U+95D6.0	fTotalStrokes	 18
U+95D6.0	fCangjie	ANSQF
U+95D6.0	fFourCorner	77327
U+95D6.0	fHakka	 cong3; cim1; {cong3}; chim5; cihm5; {cong6}; {cong5}; cong5; cim5
U+95D6.0	fCantonese	cong2; cong3; cam3
U+95D6.0	fMandarin	chuang3; chen4; chen1; chen3; he2
U+95D6.0	fEnglish	[1] rush in all of a sudden; burst in; charge in; intrude into [2] be trained by experience; hew out one's way [3] cause (disaster, etc.)
U+95D6.1	fUTF8	闖
U+95D6.1	fCNS11643	1-775F
U+95D6.1	fBig5	C2F4
U+95D6.1	fGB	1-3433
U+95D6.1	fR/S	 169.10
U+95D6.1	fTotalStrokes	 18
U+95D6.1	fCangjie	ANSQF
U+95D6.1	fFourCorner	77327
U+95D6.1	fMandarin	chuang4
U+95D6.1	fEnglish	[1] cause (disaster) [2] hit
U+95D7.0	fUTF8	闗
U+95D7.0	fCNS11643	3-594B
U+95D7.0	fGB	E-4376
U+95D7.0	fR/S	 169.10; 169.12
U+95D7.0	fTotalStrokes	 18; 20
U+95D7.0	fCangjie	ANVIF
U+95D7.0	fFourCorner	77333
U+95D7.0	fHakka	 kwan1; wan1; gwan1
U+95D7.0	fCantonese	gwaan1; gwan1
U+95D7.0	fMandarin	guan1
U+95D7.0	fEnglish	 a frontier pass or gate to shut or close custom-house suburbs of a city
U+95D8.0	fUTF8	闘
U+95D8.0	fCNS11643	3-594A
U+95D8.0	fGB	E-4377
U+95D8.0	fR/S	 169.10
U+95D8.0	fTotalStrokes	 18
U+95D8.0	fCangjie	ANMTI
U+95D8.0	fFourCorner	77140
U+95D8.0	fCantonese	dau3
U+95D8.0	fMandarin	dou4; dou3
U+95D8.0	fEnglish	struggle; fight; compete;contend
U+95D9.0	fUTF8	闙
U+95D9.0	fCNS11643	7-4B40
U+95D9.0	fGB	E-4378
U+95D9.0	fR/S	 169.11
U+95D9.0	fTotalStrokes	 19
U+95D9.0	fCangjie	ANHRK
U+95D9.0	fFourCorner	77248
U+95D9.0	fCantonese	kai2
U+95D9.0	fMandarin	qi3
U+95DA.0	fUTF8	闚
U+95DA.0	fCNS11643	2-6764
U+95DA.0	fBig5	F377
U+95DA.0	fGB	E-4379
U+95DA.0	fR/S	 169.11
U+95DA.0	fTotalStrokes	 19
U+95DA.0	fCangjie	ANQOU
U+95DA.0	fFourCorner	77516
U+95DA.0	fHakka	 gui1; kwui1
U+95DA.0	fCantonese	kwai1
U+95DA.0	fMandarin	kui1; kui4; kui3
U+95DA.0	fEnglish	[1] same as $ [2] tempt with material gain
U+95DB.0	fUTF8	闛
U+95DB.0	fCNS11643	2-6765
U+95DB.0	fBig5	F378
U+95DB.0	fGB	3-3B31
U+95DB.0	fR/S	 169.11
U+95DB.0	fTotalStrokes	 19
U+95DB.0	fCangjie	ANFBG
U+95DB.0	fFourCorner	77104
U+95DB.0	fCantonese	tong1; tong4; coeng1
U+95DB.0	fMandarin	tang2; chang1; tang1
U+95DC.0	fUTF8	關
U+95DC.0	fCNS11643	1-7943
U+95DC.0	fBig5	C3F6
U+95DC.0	fGB	1-3958
U+95DC.0	fR/S	 169.11
U+95DC.0	fTotalStrokes	 19
U+95DC.0	fCangjie	ANVIT
U+95DC.0	fFourCorner	77772
U+95DC.0	fHakka	 gan1;  kwan1; gon1; gwan1
U+95DC.0	fCantonese	gwaan1
U+95DC.0	fMandarin	guan1; chuan3; wan1
U+95DC.0	fEnglish	[1] frontier pass; checkpoint [2] close; shut [3] the bar across the door [4] customs house; customs barrier [5] key point; turning point [6] relationship; related; involve; concern [7] negociate; go between [8] draw (money, pay) [9] a Chinese family name
U+95DD.0	fUTF8	闝
U+95DD.0	fCNS11643	3-5B53
U+95DD.0	fGB	5-3D64
U+95DD.0	fR/S	  169.11
U+95DD.0	fTotalStrokes	 19
U+95DD.0	fCangjie	ANBCK
U+95DD.0	fFourCorner	77840
U+95DD.0	fCantonese	piu4
U+95DD.0	fMandarin	piao2
U+95DE.0	fUTF8	闞
U+95DE.0	fCNS11643	2-6A53
U+95DE.0	fBig5	F545
U+95DE.0	fGB	1-635B
U+95DE.0	fR/S	 169.12
U+95DE.0	fTotalStrokes	 20
U+95DE.0	fCangjie	ANMJK
U+95DE.0	fFourCorner	77148
U+95DE.0	fHakka	 kam3; kam5
U+95DE.0	fCantonese	ham3; haam2; haam3; ham6
U+95DE.0	fMandarin	kan4; han4; kan3; xian4
U+95DE.0	fEnglish	[1] steal a glance; peep; spy; watch secretly [2] a Chinese family name
U+95DE.1	fUTF8	闞
U+95DE.1	fCNS11643	2-6A53
U+95DE.1	fBig5	F545
U+95DE.1	fGB	1-635B
U+95DE.1	fR/S	 169.12
U+95DE.1	fTotalStrokes	 20
U+95DE.1	fCangjie	ANMJK
U+95DE.1	fFourCorner	77148
U+95DE.1	fMandarin	han3
U+95DE.1	fEnglish	[1] roaring (tiger); growling [2] brave
U+95DF.0	fUTF8	闟
U+95DF.0	fCNS11643	2-6A55
U+95DF.0	fBig5	F547
U+95DF.0	fGB	3-3B33
U+95DF.0	fR/S	 169.12
U+95DF.0	fTotalStrokes	 20
U+95DF.0	fCangjie	ANORM
U+95DF.0	fFourCorner	88427
U+95DF.0	fHakka	  hip7;  tap7
U+95DF.0	fCantonese	taap3; kap1; hap1; jap1; kaap1
U+95DF.0	fMandarin	xi4; ta4; xi1; se4; sa4
U+95DF.0	fEnglish	 peacefully quietly
U+95E0.0	fUTF8	闠
U+95E0.0	fCNS11643	2-6A54
U+95E0.0	fBig5	F546
U+95E0.0	fGB	3-3B2D
U+95E0.0	fR/S	 169.12
U+95E0.0	fTotalStrokes	 20
U+95E0.0	fCangjie	ANLMC
U+95E0.0	fFourCorner	77806
U+95E0.0	fHakka	 kui1; kui6; kui5; kwi5; k'wui5
U+95E0.0	fCantonese	kui2; gwai6 (also); kwui2 (also); wui6; fui2
U+95E0.0	fMandarin	hui4; kui4
U+95E0.0	fEnglish	 gate of a market
U+95E1.0	fUTF8	闡
U+95E1.0	fCNS11643	1-7A4F
U+95E1.0	fBig5	C4C4
U+95E1.0	fGB	1-327B
U+95E1.0	fR/S	 169.12
U+95E1.0	fTotalStrokes	 20
U+95E1.0	fCangjie	ANRRJ
U+95E1.0	fFourCorner	77506
U+95E1.0	fHakka	 sen3; chan3; can3; shan3; san3; shan5; shen5; shen3
U+95E1.0	fCantonese	zin2; cin2 (also); sin6 (also)
U+95E1.0	fMandarin	chan3; chan1; guan1; wan1
U+95E1.0	fEnglish	[1] explain; clarify; elucidate; expound; elaborate; make clear [2] evident; clear
U+95E2.0	fUTF8	闢
U+95E2.0	fCNS11643	1-7B3C
U+95E2.0	fBig5	C550
U+95E2.0	fGB	1-785B
U+95E2.0	fR/S	 169.13
U+95E2.0	fTotalStrokes	 21
U+95E2.0	fCangjie	ANSRJ
U+95E2.0	fFourCorner	77641
U+95E2.0	fHakka	  pit7; bit7
U+95E2.0	fCantonese	pik1; bei6
U+95E2.0	fMandarin	pi4; bi4; pi1; ta4
U+95E2.0	fEnglish	[1] open up; develop [2] rid; do away with [3] refute
U+95E3.0	fUTF8	闣
U+95E3.0	fCNS11643	2-6C5B
U+95E3.0	fBig5	F66D
U+95E3.0	fGB	3-3A7E
U+95E3.0	fR/S	 169.13
U+95E3.0	fTotalStrokes	 21
U+95E3.0	fCangjie	ANFBW
U+95E3.0	fFourCorner	77606
U+95E3.0	fHakka	 dong5
U+95E3.0	fCantonese	dong3; tong1
U+95E3.0	fMandarin	dang4; tang1
U+95E3.0	fEnglish	[1] block a doorway [2] full [3] the sound of a drum
U+95E4.0	fUTF8	闤
U+95E4.0	fCNS11643	2-6C5A
U+95E4.0	fBig5	F66C
U+95E4.0	fGB	3-3B34
U+95E4.0	fR/S	 169.13
U+95E4.0	fTotalStrokes	 21
U+95E4.0	fCangjie	ANWLV
U+95E4.0	fFourCorner	77732
U+95E4.0	fHakka	  fan2; fan3
U+95E4.0	fCantonese	waan4
U+95E4.0	fMandarin	huan2; wan1
U+95E4.0	fEnglish	 wall around market place
U+95E5.0	fUTF8	闥
U+95E5.0	fCNS11643	2-6C59
U+95E5.0	fBig5	F66B
U+95E5.0	fGB	1-634B
U+95E5.0	fR/S	 169.13
U+95E5.0	fTotalStrokes	 21
U+95E5.0	fCangjie	ANYGQ
U+95E5.0	fFourCorner	77304
U+95E5.0	fHakka	 dat7
U+95E5.0	fCantonese	taat3
U+95E5.0	fMandarin	ta4; chan3
U+95E5.0	fEnglish	[1] door; gate; small door; wicket gate [2] the space between the door and the screen inside the door [3] fast
U+95E6.0	fUTF8	闦
U+95E6.0	fCNS11643	F-6872
U+95E6.0	fGB	E-437A
U+95E6.0	fR/S	 169.13
U+95E6.0	fTotalStrokes	 21
U+95E6.0	fCangjie	ANTWT
U+95E6.0	fFourCorner	7710
U+95E6.0	fCantonese	ngan4; an4 (also)
U+95E6.0	fMandarin	wen2
U+95E7.0	fUTF8	闧
U+95E7.0	fCNS11643	3-666D
U+95E7.0	fGB	E-437B
U+95E7.0	fR/S	 169.14
U+95E7.0	fTotalStrokes	 22
U+95E7.0	fCangjie	ANYGV
U+95E7.0	fFourCorner	7730
U+95E7.0	fCantonese	jyun5
U+95E8.0	fUTF8	门
U+95E8.0	fGB	0-4345
U+95E8.0	fR/S	 169.0
U+95E8.0	fTotalStrokes	 3
U+95E8.0	fCangjie	LS
U+95E8.0	fHakka	 mun2
U+95E8.0	fCantonese	mun4; mun2 (also)
U+95E8.0	fMandarin	men2
U+95E8.0	fEnglish	simplified form of &#x9580;
U+95E8.1	fUTF8	门
U+95E8.1	fGB	0-4345
U+95E8.1	fR/S	 169.0
U+95E8.1	fTotalStrokes	 3
U+95E8.1	fCangjie	LS
U+95E8.1	fMandarin	men5
U+95E9.0	fUTF8	闩
U+95E9.0	fGB	0-6345
U+95E9.0	fR/S	 169.1
U+95E9.0	fTotalStrokes	 4
U+95E9.0	fCangjie	LSM
U+95E9.0	fHakka	 con1
U+95E9.0	fCantonese	saan1
U+95E9.0	fMandarin	shuan1; zao2; zao4; zuo4
U+95E9.0	fEnglish	simplified form of &#x9582;
U+95EA.0	fUTF8	闪
U+95EA.0	fGB	0-4941
U+95EA.0	fR/S	 169.2
U+95EA.0	fTotalStrokes	 5
U+95EA.0	fCangjie	LSO
U+95EA.0	fHakka	 sham3
U+95EA.0	fCantonese	sim2
U+95EA.0	fMandarin	shan3; shan4
U+95EA.0	fEnglish	simplified form of &#x9583;
U+95EB.0	fUTF8	闫
U+95EB.0	fGB	0-6346
U+95EB.0	fR/S	 169.3
U+95EB.0	fTotalStrokes	 6
U+95EB.0	fCangjie	LSMMM
U+95EB.0	fHakka	 jam2
U+95EB.0	fCantonese	jim4
U+95EB.0	fMandarin	yan4; yan2; yan3
U+95EB.0	fEnglish	simplified form of &#x9586;
U+95EC.0	fUTF8	闬
U+95EC.0	fGB	8-7B2C
U+95EC.0	fR/S	 169.3
U+95EC.0	fTotalStrokes	 6
U+95EC.0	fCangjie	LSMJ
U+95EC.0	fCantonese	hon6
U+95EC.0	fMandarin	han4
U+95EC.0	fEnglish	village
U+95ED.0	fUTF8	闭
U+95ED.0	fGB	0-3155
U+95ED.0	fR/S	 169.3
U+95ED.0	fTotalStrokes	 6
U+95ED.0	fCangjie	LSDH
U+95ED.0	fHakka	 bi5
U+95ED.0	fCantonese	bai3
U+95ED.0	fMandarin	bi4; bie1
U+95ED.0	fEnglish	simplified form of &#x9589;
U+95EE.0	fUTF8	问
U+95EE.0	fGB	0-4E4A
U+95EE.0	fR/S	 169.3
U+95EE.0	fTotalStrokes	 6
U+95EE.0	fCangjie	LSR
U+95EE.0	fHakka	 mun5
U+95EE.0	fCantonese	man6
U+95EE.0	fMandarin	wen4
U+95EE.0	fEnglish	simplified form of &#x554F;
U+95EE.1	fUTF8	问
U+95EE.1	fGB	0-4E4A
U+95EE.1	fR/S	 169.3
U+95EE.1	fTotalStrokes	 6
U+95EE.1	fCangjie	LSR
U+95EE.1	fMandarin	wen5
U+95EF.0	fUTF8	闯
U+95EF.0	fGB	0-3433
U+95EF.0	fR/S	 169.3
U+95EF.0	fTotalStrokes	 6
U+95EF.0	fCangjie	LSNVM
U+95EF.0	fHakka	 cim5
U+95EF.0	fCantonese	cong2; cong3
U+95EF.0	fMandarin	chuang3; chen4; chen1; chen3; he2
U+95EF.0	fEnglish	simplified form of &#x95D6;
U+95F0.0	fUTF8	闰
U+95F0.0	fGB	0-4872
U+95F0.0	fR/S	 169.4
U+95F0.0	fTotalStrokes	 7
U+95F0.0	fCangjie	LSMG
U+95F0.0	fHakka	 jun5
U+95F0.0	fCantonese	jeon6
U+95F0.0	fMandarin	run4
U+95F0.0	fEnglish	simplified form of &#x958F;
U+95F1.0	fUTF8	闱
U+95F1.0	fGB	0-6347
U+95F1.0	fR/S	 169.4
U+95F1.0	fTotalStrokes	 7
U+95F1.0	fCangjie	LSQS
U+95F1.0	fHakka	 vui2
U+95F1.0	fCantonese	wai4
U+95F1.0	fMandarin	wei2; hui1
U+95F1.0	fEnglish	simplified form of &#x95C8;
U+95F2.0	fUTF8	闲
U+95F2.0	fGB	0-4F50
U+95F2.0	fR/S	 169.4
U+95F2.0	fTotalStrokes	 7
U+95F2.0	fCangjie	LSD
U+95F2.0	fHakka	 han2
U+95F2.0	fCantonese	haan4; haan2 (also)
U+95F2.0	fMandarin	xian2
U+95F2.0	fEnglish	simplified form of &#x9591;
U+95F3.0	fUTF8	闳
U+95F3.0	fGB	0-6348
U+95F3.0	fR/S	 169.4
U+95F3.0	fTotalStrokes	 7
U+95F3.0	fCangjie	LSKI
U+95F3.0	fHakka	 fen2
U+95F3.0	fCantonese	wang4
U+95F3.0	fMandarin	hong2
U+95F3.0	fEnglish	simplified form of &#x958E;
U+95F4.0	fUTF8	间
U+95F4.0	fGB	0-3C64
U+95F4.0	fR/S	 169.4
U+95F4.0	fTotalStrokes	 7
U+95F4.0	fCangjie	LSA
U+95F4.0	fHakka	 gan1; gan3; gen1; kian1; gien1; gan5
U+95F4.0	fCantonese	gaan1; gaan3
U+95F4.0	fMandarin	jian1; jian4
U+95F4.0	fEnglish	simplified form of &#x9593;
U+95F4.1	fUTF8	间
U+95F4.1	fGB	0-3C64
U+95F4.1	fR/S	 169.4
U+95F4.1	fTotalStrokes	 7
U+95F4.1	fCangjie	LSA
U+95F4.2	fUTF8	间
U+95F4.2	fGB	0-3C64
U+95F4.2	fR/S	 169.4
U+95F4.2	fTotalStrokes	 7
U+95F4.2	fCangjie	LSA
U+95F4.2	fMandarin	jian5
U+95F5.0	fUTF8	闵
U+95F5.0	fCNS11643	F-2539
U+95F5.0	fGB	0-6349
U+95F5.0	fR/S	 169.4
U+95F5.0	fTotalStrokes	 7
U+95F5.0	fCangjie	LSYK
U+95F5.0	fHakka	 men1
U+95F5.0	fCantonese	man5
U+95F5.0	fMandarin	min3; min2
U+95F5.0	fEnglish	simplified form of &#x9594;
U+95F6.0	fUTF8	闶
U+95F6.0	fGB	0-634A
U+95F6.0	fR/S	 169.4
U+95F6.0	fTotalStrokes	 7
U+95F6.0	fCangjie	LSYHN
U+95F6.0	fHakka	 kong5
U+95F6.0	fCantonese	kong3
U+95F6.0	fMandarin	kang4; kang1
U+95F6.0	fEnglish	simplified form of &#x958C;
U+95F7.0	fUTF8	闷
U+95F7.0	fGB	0-4346
U+95F7.0	fR/S	 169.4
U+95F7.0	fTotalStrokes	 7
U+95F7.0	fCangjie	LSP
U+95F7.0	fHakka	 mun3
U+95F7.0	fCantonese	mun6
U+95F7.0	fMandarin	men4; men2; men3
U+95F7.0	fEnglish	simplified form of &#x60B6;
U+95F7.1	fUTF8	闷
U+95F7.1	fGB	0-4346
U+95F7.1	fR/S	 169.4
U+95F7.1	fTotalStrokes	 7
U+95F7.1	fCangjie	LSP
U+95F7.1	fMandarin	men1
U+95F8.0	fUTF8	闸
U+95F8.0	fGB	0-5522
U+95F8.0	fR/S	 169.5
U+95F8.0	fTotalStrokes	 8
U+95F8.0	fCangjie	LSWL
U+95F8.0	fHakka	 ap7; {gap7}
U+95F8.0	fCantonese	zaap6
U+95F8.0	fMandarin	zha2; ya1
U+95F8.0	fEnglish	simplified form of &#x9598;
U+95F9.0	fUTF8	闹
U+95F9.0	fGB	0-4456
U+95F9.0	fR/S	 169.5
U+95F9.0	fTotalStrokes	 8
U+95F9.0	fCangjie	LSYLB
U+95F9.0	fHakka	 nau5
U+95F9.0	fCantonese	naau6; laau6 (also)
U+95F9.0	fMandarin	nao4
U+95F9.0	fEnglish	simplified form of &#x9B27;
U+95F9.1	fUTF8	闹
U+95F9.1	fGB	0-4456
U+95F9.1	fR/S	 169.5
U+95F9.1	fTotalStrokes	 8
U+95F9.1	fCangjie	LSYLB
U+95F9.1	fMandarin	nao5
U+95FA.0	fUTF8	闺
U+95FA.0	fGB	0-396B
U+95FA.0	fR/S	 169.6
U+95FA.0	fTotalStrokes	 9
U+95FA.0	fCangjie	LSGG
U+95FA.0	fHakka	 kwui1
U+95FA.0	fCantonese	gwai1
U+95FA.0	fMandarin	gui1
U+95FA.0	fEnglish	simplified form of &#x95A8;
U+95FB.0	fUTF8	闻
U+95FB.0	fGB	0-4E45
U+95FB.0	fR/S	 169.6
U+95FB.0	fTotalStrokes	 9
U+95FB.0	fCangjie	LSSJ
U+95FB.0	fHakka	 vun2
U+95FB.0	fCantonese	man4; man2 (also); man6
U+95FB.0	fMandarin	wen2; wen4
U+95FB.0	fEnglish	simplified form of &#x805E;
U+95FC.0	fUTF8	闼
U+95FC.0	fGB	0-634B
U+95FC.0	fR/S	 169.6
U+95FC.0	fTotalStrokes	 9
U+95FC.0	fCangjie	XLSYK
U+95FC.0	fHakka	 dat7
U+95FC.0	fCantonese	taat3
U+95FC.0	fMandarin	ta4; chan3
U+95FC.0	fEnglish	simplified form of &#x95E5;
U+95FD.0	fUTF8	闽
U+95FD.0	fGB	0-4376
U+95FD.0	fR/S	 169.6
U+95FD.0	fTotalStrokes	 9
U+95FD.0	fCangjie	LSLMI
U+95FD.0	fHakka	 men2; man2
U+95FD.0	fCantonese	man5; man4
U+95FD.0	fMandarin	min3; man2; min2; wen2
U+95FD.0	fEnglish	simplified form of &#x95A9;
U+95FE.0	fUTF8	闾
U+95FE.0	fGB	0-634C
U+95FE.0	fR/S	 169.6
U+95FE.0	fTotalStrokes	 9
U+95FE.0	fCangjie	LSRR
U+95FE.0	fHakka	 li1; lu2
U+95FE.0	fCantonese	leoi4; leoi5 (also)
U+95FE.0	fMandarin	l2; luu2; hong4; lu2
U+95FE.0	fEnglish	simplified form of &#x95AD;
U+95FF.0	fUTF8	闿
U+95FF.0	fGB	8-7B2D
U+95FF.0	fR/S	 169.10; 169.6
U+95FF.0	fTotalStrokes	 9
U+95FF.0	fCangjie	LSUSU
U+95FF.0	fCantonese	hoi2
U+95FF.0	fMandarin	kai3; kai1
U+95FF.0	fEnglish	open; peaceful
U+9600.0	fUTF8	阀
U+9600.0	fGB	0-3727
U+9600.0	fR/S	 169.6
U+9600.0	fTotalStrokes	 9
U+9600.0	fCangjie	LSOI
U+9600.0	fHakka	 fat8
U+9600.0	fCantonese	fat6
U+9600.0	fMandarin	fa2
U+9600.0	fEnglish	simplified form of &#x95A5;
U+9601.0	fUTF8	阁
U+9601.0	fGB	0-3873
U+9601.0	fR/S	 169.6
U+9601.0	fTotalStrokes	 9
U+9601.0	fCangjie	LSHER
U+9601.0	fHakka	 gok7
U+9601.0	fCantonese	gok3; gok2 (also)
U+9601.0	fMandarin	ge2; gao3
U+9601.0	fEnglish	simplified form of &#x95A3;
U+9602.0	fUTF8	阂
U+9602.0	fGB	0-3A52
U+9602.0	fR/S	 169.6
U+9602.0	fTotalStrokes	 9
U+9602.0	fCangjie	LSYVO
U+9602.0	fHakka	 ngoi5
U+9602.0	fCantonese	hat6; goi1; ngoi6; oi6 (also)
U+9602.0	fMandarin	he2; ai4; gai1; hai4; kai3
U+9602.0	fEnglish	simplified form of &#x95A1;
U+9603.0	fUTF8	阃
U+9603.0	fGB	0-634D
U+9603.0	fR/S	 169.7
U+9603.0	fTotalStrokes	 10
U+9603.0	fCangjie	LSWD
U+9603.0	fHakka	 kwun5
U+9603.0	fCantonese	kwan2
U+9603.0	fMandarin	kun3; kun4
U+9603.0	fEnglish	simplified form of &#x95AB;
U+9604.0	fUTF8	阄
U+9604.0	fGB	0-634E
U+9604.0	fR/S	 169.7
U+9604.0	fTotalStrokes	 10
U+9604.0	fCangjie	LSNWU
U+9604.0	fHakka	 keu1
U+9604.0	fCantonese	gau1
U+9604.0	fMandarin	jiu1
U+9604.0	fEnglish	simplified form of &#x9B2E;
U+9605.0	fUTF8	阅
U+9605.0	fGB	0-5444
U+9605.0	fR/S	 169.7
U+9605.0	fTotalStrokes	 10
U+9605.0	fCangjie	LSCRU
U+9605.0	fCantonese	jyut6
U+9605.0	fMandarin	yue4
U+9605.0	fEnglish	simplified form of &#x95B2;
U+9606.0	fUTF8	阆
U+9606.0	fGB	0-634F
U+9606.0	fR/S	 169.7
U+9606.0	fTotalStrokes	 10
U+9606.0	fCangjie	LSIAV
U+9606.0	fHakka	 long5
U+9606.0	fCantonese	long6; long5; long4
U+9606.0	fMandarin	lang3; lang2; lang4; liang3; tang2
U+9606.0	fEnglish	simplified form of &#x95AC;
U+9607.0	fUTF8	阇
U+9607.0	fGB	8-7B2F
U+9607.0	fR/S	 169.9; 169.8
U+9607.0	fTotalStrokes	 11
U+9607.0	fCangjie	LSJKA
U+9607.0	fCantonese	dou1; se4; ze1 (also); se6
U+9607.0	fMandarin	du1; she2
U+9607.0	fEnglish	a Buddhist high priest
U+9608.0	fUTF8	阈
U+9608.0	fGB	0-6350
U+9608.0	fR/S	 169.8
U+9608.0	fTotalStrokes	 11
U+9608.0	fCangjie	LSIRM
U+9608.0	fHakka	 vet7
U+9608.0	fCantonese	wik6
U+9608.0	fMandarin	yu4; xu4; yan1
U+9608.0	fEnglish	simplified form of &#x95BE;
U+9609.0	fUTF8	阉
U+9609.0	fGB	0-514B
U+9609.0	fR/S	 169.8
U+9609.0	fTotalStrokes	 11
U+9609.0	fCangjie	LSKLU
U+9609.0	fHakka	 jam1
U+9609.0	fCantonese	jim1
U+9609.0	fMandarin	yan1; yan2; yan3
U+9609.0	fEnglish	simplified form of &#x95B9;
U+960A.0	fUTF8	阊
U+960A.0	fGB	0-6351
U+960A.0	fR/S	 169.8
U+960A.0	fTotalStrokes	 11
U+960A.0	fCangjie	LSAA
U+960A.0	fHakka	 chong1
U+960A.0	fCantonese	coeng1
U+960A.0	fMandarin	chang1; tang1
U+960A.0	fEnglish	simplified form of &#x95B6;
U+960B.0	fUTF8	阋
U+960B.0	fGB	0-6352
U+960B.0	fR/S	 169.8
U+960B.0	fTotalStrokes	 11
U+960B.0	fCangjie	LSHXU
U+960B.0	fHakka	 hit7; ngiet7
U+960B.0	fCantonese	jik1
U+960B.0	fMandarin	xi4
U+960B.0	fEnglish	simplified form of &#x9B29;
U+960C.0	fUTF8	阌
U+960C.0	fGB	0-6353
U+960C.0	fR/S	 169.8
U+960C.0	fTotalStrokes	 11
U+960C.0	fCangjie	LSBBE
U+960C.0	fHakka	 vun2
U+960C.0	fCantonese	man4
U+960C.0	fMandarin	wen2; min2
U+960C.0	fEnglish	simplified form of &#x95BF;
U+960D.0	fUTF8	阍
U+960D.0	fGB	0-6354
U+960D.0	fR/S	 169.8
U+960D.0	fTotalStrokes	 11
U+960D.0	fCangjie	LSHPA
U+960D.0	fHakka	 fun1
U+960D.0	fCantonese	fan1
U+960D.0	fMandarin	hun1
U+960D.0	fEnglish	simplified form of &#x95BD;
U+960E.0	fUTF8	阎
U+960E.0	fGB	0-5156
U+960E.0	fR/S	 169.8
U+960E.0	fTotalStrokes	 11
U+960E.0	fCangjie	LSNHX
U+960E.0	fHakka	 jam2
U+960E.0	fCantonese	jim4
U+960E.0	fMandarin	yan2; yan4; yan3; e4; xian2; yan1
U+960E.0	fEnglish	simplified form of &#x95BB;
U+960F.0	fUTF8	阏
U+960F.0	fGB	0-6355
U+960F.0	fR/S	 169.8
U+960F.0	fTotalStrokes	 11
U+960F.0	fCangjie	LSYSY
U+960F.0	fHakka	 ot7; hot7
U+960F.0	fCantonese	aat3; ngaat3 (also); jin1; aat1; jyu3
U+960F.0	fMandarin	e4; yu4; yan1; ye4
U+960F.0	fEnglish	simplified form of &#x95BC;
U+9610.0	fUTF8	阐
U+9610.0	fGB	0-327B
U+9610.0	fR/S	 169.8
U+9610.0	fTotalStrokes	 11
U+9610.0	fCangjie	LSCWJ
U+9610.0	fHakka	 shen5; shen3
U+9610.0	fCantonese	zin2; cin2 (also); sin6 (also)
U+9610.0	fMandarin	chan3; chan1; guan1; wan1
U+9610.0	fEnglish	simplified form of &#x95E1;
U+9611.0	fUTF8	阑
U+9611.0	fGB	0-403B
U+9611.0	fR/S	 169.9
U+9611.0	fTotalStrokes	 12
U+9611.0	fCangjie	LSDWF
U+9611.0	fHakka	 lan2
U+9611.0	fCantonese	laan4
U+9611.0	fMandarin	lan2; du1; lan4; she2
U+9611.0	fEnglish	simplified form of &#x95CC;
U+9612.0	fUTF8	阒
U+9612.0	fGB	0-6356
U+9612.0	fR/S	 169.9
U+9612.0	fTotalStrokes	 12
U+9612.0	fCangjie	LSBUK
U+9612.0	fHakka	 ket7; ngiet7; kiet7; hit7
U+9612.0	fCantonese	gwik1
U+9612.0	fMandarin	qu4; kuo4
U+9612.0	fEnglish	simplified form of &#x95C3;
U+9613.0	fUTF8	阓
U+9613.0	fGB	8-7B30
U+9613.0	fR/S	 169.9
U+9613.0	fTotalStrokes	 12
U+9613.0	fCangjie	LSLMO
U+9613.0	fCantonese	kui2; kwui2 (also); gwai6 (also); fui2
U+9613.0	fMandarin	hui4
U+9613.0	fEnglish	gate of a market
U+9614.0	fUTF8	阔
U+9614.0	fGB	0-402B
U+9614.0	fR/S	 169.9
U+9614.0	fTotalStrokes	 12
U+9614.0	fCangjie	LSEHR
U+9614.0	fHakka	 fat7; k'wat7
U+9614.0	fCantonese	fut3
U+9614.0	fMandarin	kuo4; yu4
U+9614.0	fEnglish	simplified form of &#x95CA;
U+9615.0	fUTF8	阕
U+9615.0	fGB	0-6357
U+9615.0	fR/S	 169.9
U+9615.0	fTotalStrokes	 12
U+9615.0	fCangjie	LSNOK
U+9615.0	fHakka	 ket7
U+9615.0	fCantonese	kyut3
U+9615.0	fMandarin	que4; gui4; jue2
U+9615.0	fEnglish	simplified form of &#x95CB;
U+9616.0	fUTF8	阖
U+9616.0	fGB	0-6358
U+9616.0	fR/S	 169.10
U+9616.0	fTotalStrokes	 13
U+9616.0	fCangjie	LSGIT
U+9616.0	fHakka	 hap8
U+9616.0	fCantonese	hap6
U+9616.0	fMandarin	he2; ge2; que1; que4
U+9616.0	fEnglish	simplified form of &#x95D4;
U+9617.0	fUTF8	阗
U+9617.0	fGB	0-6359
U+9617.0	fR/S	 169.10
U+9617.0	fTotalStrokes	 13
U+9617.0	fCangjie	LSJBC
U+9617.0	fHakka	 tien2
U+9617.0	fCantonese	tin4
U+9617.0	fMandarin	tian2; dian4; ji4
U+9617.0	fEnglish	simplified form of &#x95D0;
U+9618.0	fUTF8	阘
U+9618.0	fGB	8-2C30
U+9618.0	fR/S	 169.10
U+9618.0	fTotalStrokes	 13
U+9618.0	fCangjie	LSASM
U+9618.0	fCantonese	taap3; daap6 (also)
U+9618.0	fMandarin	ta4; da2
U+9618.0	fEnglish	upper-story door or window
U+9619.0	fUTF8	阙
U+9619.0	fGB	0-635A
U+9619.0	fR/S	 169.10
U+9619.0	fTotalStrokes	 13
U+9619.0	fCangjie	LSTUO
U+9619.0	fHakka	 ket7; kiet7
U+9619.0	fCantonese	kyut3; gwat6
U+9619.0	fMandarin	que4; jue2; lan2; que1
U+9619.0	fEnglish	simplified form of &#x95D5;
U+961A.0	fUTF8	阚
U+961A.0	fGB	0-635B
U+961A.0	fR/S	 169.11
U+961A.0	fTotalStrokes	 14
U+961A.0	fCangjie	LSNJK
U+961A.0	fHakka	 kam5
U+961A.0	fCantonese	ham3; haam2; haam3
U+961A.0	fMandarin	kan4; han4; kan3
U+961A.0	fEnglish	simplified form of &#x95DE;
U+961B.0	fUTF8	阛
U+961B.0	fGB	8-7B31
U+961B.0	fR/S	 169.13
U+961B.0	fTotalStrokes	 16
U+961B.0	fCangjie	LSWLV
U+961B.0	fCantonese	waan4
U+961B.0	fMandarin	huan2
U+961B.0	fEnglish	wall around market place
U+961C.0	fUTF8	阜
U+961C.0	fCNS11643	1-4F5D
U+961C.0	fBig5	AAFA
U+961C.0	fGB	0-3837
U+961C.0	fR/S	 170.0
U+961C.0	fTotalStrokes	 8
U+961C.0	fCangjie	HRJ
U+961C.0	fFourCorner	27407
U+961C.0	fHakka	  fu5;  feu3; fu2; fu6; fiu3; feu5; fu3
U+961C.0	fCantonese	fau6
U+961C.0	fMandarin	fu4
U+961C.0	fEnglish	[1] mound; small hill [2] abundant; ample; numerous; flourishing [3] the continent; the mainland [4] Kangxi radical 170
U+961D.0	fUTF8	阝
U+961D.0	fCNS11643	3-2143
U+961D.0	fGB	0-5A62
U+961D.0	fR/S	 170.0
U+961D.0	fTotalStrokes	 3
U+961D.0	fCangjie	NL
U+961D.0	fFourCorner	77220
U+961D.0	fHakka	 fu5
U+961D.0	fMandarin	fu4; yi4
U+961D.0	fEnglish	 place Kangxi radical number 170
U+961E.0	fUTF8	阞
U+961E.0	fCNS11643	2-2169
U+961E.0	fBig5	C9AA
U+961E.0	fGB	3-752A
U+961E.0	fR/S	 170.2
U+961E.0	fTotalStrokes	 5
U+961E.0	fCangjie	NLKS
U+961E.0	fFourCorner	74227
U+961E.0	fHakka	 let8;  lit8
U+961E.0	fCantonese	lak6
U+961E.0	fMandarin	le4
U+961E.0	fEnglish	vein
U+961F.0	fUTF8	队
U+961F.0	fGB	0-3653
U+961F.0	fR/S	 170.2
U+961F.0	fTotalStrokes	 5
U+961F.0	fCangjie	NLO
U+961F.0	fHakka	 dui3; chui5; tui5
U+961F.0	fCantonese	deoi6; deoi2 (also); zeoi6
U+961F.0	fMandarin	dui4; sui4; zhui4
U+961F.0	fEnglish	simplified form of &#x968A;
U+9620.0	fUTF8	阠
U+9620.0	fCNS11643	2-2278
U+9620.0	fBig5	CA58
U+9620.0	fGB	E-437C
U+9620.0	fR/S	 170.3
U+9620.0	fTotalStrokes	 6
U+9620.0	fCangjie	NLNJ
U+9620.0	fFourCorner	72210
U+9620.0	fCantonese	seon3; can2; san1
U+9620.0	fMandarin	xin4; shen1; shen4; xun4
U+9621.0	fUTF8	阡
U+9621.0	fCNS11643	1-486A
U+9621.0	fBig5	A6E9
U+9621.0	fGB	0-5A64
U+9621.0	fR/S	 170.3
U+9621.0	fTotalStrokes	 6
U+9621.0	fCangjie	NLHJ
U+9621.0	fFourCorner	72240
U+9621.0	fHakka	 cien1; cen1
U+9621.0	fCantonese	cin1
U+9621.0	fMandarin	qian1; pi4
U+9621.0	fEnglish	[1] footpaths between fields, running north and south; paths on farms [2] the path leading to a grave
U+9622.0	fUTF8	阢
U+9622.0	fCNS11643	2-2276
U+9622.0	fBig5	CA56
U+9622.0	fGB	0-5A63
U+9622.0	fR/S	 170.3
U+9622.0	fTotalStrokes	 6
U+9622.0	fCangjie	NLMU
U+9622.0	fFourCorner	71210
U+9622.0	fHakka	 ngut7
U+9622.0	fCantonese	ngat6; at6 (also)
U+9622.0	fMandarin	wu4; wei2; wei4
U+9622.0	fEnglish	uneasy; apprehensive
U+9623.0	fUTF8	阣
U+9623.0	fCNS11643	2-2279
U+9623.0	fBig5	CA59
U+9623.0	fGB	5-7464
U+9623.0	fR/S	 170.3
U+9623.0	fTotalStrokes	 6
U+9623.0	fCangjie	NLON
U+9623.0	fFourCorner	78217
U+9623.0	fCantonese	goi3; ngat6
U+9623.0	fMandarin	yi4; gai4
U+9624.0	fUTF8	阤
U+9624.0	fCNS11643	2-2277
U+9624.0	fBig5	CA57
U+9624.0	fGB	3-752B
U+9624.0	fR/S	 170.3
U+9624.0	fTotalStrokes	 6
U+9624.0	fCangjie	NLPD
U+9624.0	fFourCorner	75212
U+9624.0	fHakka	 ti3; chi3; cih3
U+9624.0	fCantonese	zi6; to4; ji5; co4
U+9624.0	fMandarin	tuo2; zhi4; duo4; yi3; shi3
U+9624.0	fEnglish	hill
U+9625.0	fUTF8	阥
U+9625.0	fCNS11643	4-252E
U+9625.0	fGB	E-437D
U+9625.0	fR/S	 170.4
U+9625.0	fTotalStrokes	 7
U+9625.0	fCangjie	NLE
U+9625.0	fFourCorner	72230
U+9625.0	fCantonese	jam1
U+9625.0	fMandarin	yin1; yin4; an1
U+9626.0	fUTF8	阦
U+9626.0	fCNS11643	3-273B
U+9626.0	fGB	E-437E
U+9626.0	fR/S	 170.4
U+9626.0	fTotalStrokes	 7
U+9626.0	fCangjie	NLF
U+9626.0	fFourCorner	79280
U+9626.0	fCantonese	joeng4
U+9626.0	fMandarin	yang2
U+9627.0	fUTF8	阧
U+9627.0	fCNS11643	3-273A
U+9627.0	fGB	5-746A
U+9627.0	fR/S	 170.4
U+9627.0	fTotalStrokes	 7
U+9627.0	fCangjie	NLYJ
U+9627.0	fFourCorner	74200
U+9627.0	fHakka	  deu3
U+9627.0	fCantonese	dau2
U+9627.0	fMandarin	dou3
U+9627.0	fEnglish	 sloping; steep suddenly; abruptly
U+9628.0	fUTF8	阨
U+9628.0	fCNS11643	2-252E
U+9628.0	fBig5	CBAE
U+9628.0	fGB	E-4421
U+9628.0	fR/S	 170.4
U+9628.0	fTotalStrokes	 7
U+9628.0	fCangjie	NLMSU
U+9628.0	fFourCorner	71212
U+9628.0	fHakka	 ak7
U+9628.0	fCantonese	aai3; ak1; ai3 (also); ngaai3 (also); ngai3 (also); ngak1 (also); aak1 (also); ngaak1 (also)
U+9628.0	fMandarin	e4; ai4; qian1
U+9628.0	fEnglish	[1] strategic position [2] precarious position [3] block up; obstruct [4] destitute; poverty-stricken; difficulty
U+9629.0	fUTF8	阩
U+9629.0	fCNS11643	3-273F
U+9629.0	fGB	E-4422
U+9629.0	fR/S	 170.4
U+9629.0	fTotalStrokes	 7
U+9629.0	fCangjie	NLHT
U+9629.0	fFourCorner	74200
U+9629.0	fCantonese	sing1
U+9629.0	fMandarin	sheng1
U+962A.0	fUTF8	阪
U+962A.0	fCNS11643	1-4B62
U+962A.0	fBig5	A8C1
U+962A.0	fGB	0-5A66
U+962A.0	fR/S	 170.4
U+962A.0	fTotalStrokes	 7
U+962A.0	fCangjie	NLHE
U+962A.0	fFourCorner	71247
U+962A.0	fHakka	 fan3;  ban3
U+962A.0	fCantonese	baan2; faan2 (also); baan6
U+962A.0	fMandarin	ban3; fan3
U+962A.0	fEnglish	[1] hillside; slope of a hill [2] hillside farm field
U+962B.0	fUTF8	阫
U+962B.0	fCNS11643	3-273C
U+962B.0	fGB	3-752C
U+962B.0	fR/S	 170.4
U+962B.0	fTotalStrokes	 7
U+962B.0	fCangjie	NLMF
U+962B.0	fFourCorner	71290
U+962B.0	fCantonese	pui4 (also); pei1
U+962B.0	fMandarin	pei2; pei1
U+962C.0	fUTF8	阬
U+962C.0	fCNS11643	1-4B63
U+962C.0	fBig5	A8C2
U+962C.0	fGB	E-4423
U+962C.0	fR/S	 170.4
U+962C.0	fTotalStrokes	 7
U+962C.0	fCangjie	NLYHN
U+962C.0	fFourCorner	70217
U+962C.0	fHakka	  hang1;  kang1; gang1
U+962C.0	fCantonese	gon1 (also); gong1; haang1; kong3
U+962C.0	fMandarin	keng1; gang1; kang4
U+962C.0	fEnglish	 pit; hole; shaft
U+962D.0	fUTF8	阭
U+962D.0	fCNS11643	2-2530
U+962D.0	fBig5	CBB0
U+962D.0	fGB	3-752E
U+962D.0	fR/S	 170.4
U+962D.0	fTotalStrokes	 7
U+962D.0	fCangjie	NLIHU
U+962D.0	fFourCorner	73210
U+962D.0	fCantonese	wan5; jin5
U+962D.0	fMandarin	yun3; yan3
U+962E.0	fUTF8	阮
U+962E.0	fCNS11643	1-4B60
U+962E.0	fBig5	A8BF
U+962E.0	fGB	0-486E
U+962E.0	fR/S	 170.4
U+962E.0	fTotalStrokes	 7
U+962E.0	fCangjie	NLMMU
U+962E.0	fFourCorner	71211
U+962E.0	fHakka	 ngion3;  ngien3; ngion1
U+962E.0	fCantonese	jyun2; jyun5 (also)
U+962E.0	fMandarin	ruan3; yuan2; ai4; juan4; e4; yuan3
U+962E.0	fEnglish	[1] an ancient state in today's Kansu province [2] an ancient musical instrument [3] a Chinese family name
U+962F.0	fUTF8	阯
U+962F.0	fCNS11643	2-252F
U+962F.0	fBig5	CBAF
U+962F.0	fGB	E-4424
U+962F.0	fR/S	 170.4
U+962F.0	fTotalStrokes	 7
U+962F.0	fCangjie	NLYLM
U+962F.0	fFourCorner	71210
U+962F.0	fHakka	 zhi3; zih3; tje3
U+962F.0	fCantonese	zi2
U+962F.0	fMandarin	zhi3; ruan3; yuan3
U+962F.0	fEnglish	 site; address
U+9630.0	fUTF8	阰
U+9630.0	fCNS11643	2-252D
U+9630.0	fBig5	CBAD
U+9630.0	fGB	3-752D
U+9630.0	fR/S	 170.4
U+9630.0	fTotalStrokes	 7
U+9630.0	fCangjie	NLPP
U+9630.0	fFourCorner	71210
U+9630.0	fCantonese	pei4
U+9630.0	fMandarin	pi2; bi4
U+9631.0	fUTF8	阱
U+9631.0	fCNS11643	1-4B61
U+9631.0	fBig5	A8C0
U+9631.0	fGB	0-5A65
U+9631.0	fR/S	 170.4
U+9631.0	fTotalStrokes	 7
U+9631.0	fCangjie	NLTT
U+9631.0	fFourCorner	30558
U+9631.0	fHakka	 ziang3
U+9631.0	fCantonese	zing6; zeng2 (also); zeng6 (also)
U+9631.0	fMandarin	jing3; jing4
U+9631.0	fEnglish	pitfall; trap; snare
U+9632.0	fUTF8	防
U+9632.0	fCNS11643	1-4B5F
U+9632.0	fBig5	A8BE
U+9632.0	fGB	0-3740
U+9632.0	fR/S	 170.4
U+9632.0	fTotalStrokes	 7
U+9632.0	fCangjie	NLYHS
U+9632.0	fFourCorner	70227
U+9632.0	fHakka	  fong2; fong3
U+9632.0	fCantonese	fong4
U+9632.0	fMandarin	fang2
U+9632.0	fEnglish	 defend prevent embankment
U+9633.0	fUTF8	阳
U+9633.0	fCNS11643	3-273D
U+9633.0	fGB	0-5174
U+9633.0	fR/S	  170.4
U+9633.0	fTotalStrokes	 7
U+9633.0	fCangjie	NLA
U+9633.0	fFourCorner	76200
U+9633.0	fHakka	 jong2; rong2
U+9633.0	fCantonese	joeng4; joeng2 (also)
U+9633.0	fMandarin	yang2; chang2
U+9633.0	fEnglish	simplified form of &#x967D;
U+9634.0	fUTF8	阴
U+9634.0	fCNS11643	3-273E
U+9634.0	fGB	0-5275
U+9634.0	fR/S	 170.4
U+9634.0	fTotalStrokes	 7
U+9634.0	fCangjie	NLB
U+9634.0	fFourCorner	77220
U+9634.0	fHakka	 jim1
U+9634.0	fCantonese	jam1; jam3; am1; ngam1 (also)
U+9634.0	fMandarin	yin1; an1; yin4
U+9634.0	fEnglish	simplified form of &#x9670;
U+9635.0	fUTF8	阵
U+9635.0	fGB	0-5573
U+9635.0	fR/S	 170.4
U+9635.0	fTotalStrokes	 7
U+9635.0	fCangjie	NLKQ
U+9635.0	fHakka	 chin5
U+9635.0	fCantonese	zan6; zan2 (also)
U+9635.0	fMandarin	zhen4
U+9635.0	fEnglish	simplified form of &#x9663;
U+9636.0	fUTF8	阶
U+9636.0	fGB	0-3D57
U+9636.0	fR/S	 170.4
U+9636.0	fTotalStrokes	 7
U+9636.0	fCangjie	NLOLL
U+9636.0	fHakka	 gai1
U+9636.0	fCantonese	gaai1
U+9636.0	fMandarin	jie1
U+9636.0	fEnglish	simplified form of &#x968E;
U+9637.0	fUTF8	阷
U+9637.0	fCNS11643	3-2A79
U+9637.0	fGB	3-752F
U+9637.0	fR/S	 170.5
U+9637.0	fTotalStrokes	 8
U+9637.0	fCangjie	NLMYM
U+9637.0	fFourCorner	71210
U+9637.0	fCantonese	cing1
U+9637.0	fMandarin	cheng1
U+9638.0	fUTF8	阸
U+9638.0	fCNS11643	4-2833
U+9638.0	fGB	5-746F
U+9638.0	fR/S	 170.5
U+9638.0	fTotalStrokes	 8
U+9638.0	fCangjie	NLISN
U+9638.0	fFourCorner	73217
U+9638.0	fHakka	  ak7;  ai5
U+9638.0	fCantonese	ak1; aak1 (also); ngaak1 (also); ngak1 (also); aai3; ai3 (also); ngai3 (also); ngaai3 (also)
U+9638.0	fMandarin	e4; ai4
U+9638.0	fEnglish	difficulty
U+9639.0	fUTF8	阹
U+9639.0	fCNS11643	2-2878
U+9639.0	fBig5	CDD8
U+9639.0	fGB	3-7530
U+9639.0	fR/S	 170.5
U+9639.0	fTotalStrokes	 8
U+9639.0	fCangjie	NLGI
U+9639.0	fFourCorner	74231
U+9639.0	fHakka	  ki1
U+9639.0	fCantonese	keoi1
U+9639.0	fMandarin	qu1
U+9639.0	fEnglish	 pen surround
U+963A.0	fUTF8	阺
U+963A.0	fCNS11643	2-287B
U+963A.0	fBig5	CDDB
U+963A.0	fGB	3-7531
U+963A.0	fR/S	 170.5
U+963A.0	fTotalStrokes	 8
U+963A.0	fCangjie	NLHPM
U+963A.0	fFourCorner	72240
U+963A.0	fCantonese	dai2
U+963A.0	fMandarin	di3
U+963B.0	fUTF8	阻
U+963B.0	fCNS11643	1-4F60
U+963B.0	fBig5	AAFD
U+963B.0	fGB	0-5768
U+963B.0	fR/S	 170.5
U+963B.0	fTotalStrokes	 8
U+963B.0	fCangjie	NLBM
U+963B.0	fFourCorner	77210
U+963B.0	fHakka	  zu3;  zo3; zii3; zih3
U+963B.0	fCantonese	zo2
U+963B.0	fMandarin	zu3; zu4; zhu4
U+963B.0	fEnglish	[1] prevent; stop; prohibit; proscribe [2] separate; separated [3] impede; hinder; obstruct; oppose; blockade; detain [4] difficulty; suffer [5] rely on [6] strategic pass
U+963C.0	fUTF8	阼
U+963C.0	fCNS11643	2-287A
U+963C.0	fBig5	CDDA
U+963C.0	fGB	0-5A68
U+963C.0	fR/S	 170.5
U+963C.0	fTotalStrokes	 8
U+963C.0	fCangjie	NLOS
U+963C.0	fFourCorner	78211
U+963C.0	fHakka	 cih5; co5; ze5
U+963C.0	fCantonese	zou6
U+963C.0	fMandarin	zuo4; cu2; zu4
U+963C.0	fEnglish	the steps leading to the eastern door
U+963D.0	fUTF8	阽
U+963D.0	fCNS11643	2-2879
U+963D.0	fBig5	CDD9
U+963D.0	fGB	0-5A67
U+963D.0	fR/S	 170.5
U+963D.0	fTotalStrokes	 8
U+963D.0	fCangjie	NLYR
U+963D.0	fFourCorner	71260
U+963D.0	fHakka	 diam5; jam2; ram2
U+963D.0	fCantonese	dim3; jim4 (also)
U+963D.0	fMandarin	dian4; yan2
U+963D.0	fEnglish	[1] a wall ready to fall [2] dangerous
U+963E.0	fUTF8	阾
U+963E.0	fCNS11643	3-2A7C
U+963E.0	fGB	5-746D
U+963E.0	fR/S	 170.5
U+963E.0	fTotalStrokes	 8
U+963E.0	fCangjie	NLOII
U+963E.0	fFourCorner	78230
U+963E.0	fCantonese	ling5
U+963E.0	fMandarin	ling3
U+963F.0	fUTF8	阿
U+963F.0	fCNS11643	1-4F5F
U+963F.0	fBig5	AAFC
U+963F.0	fGB	0-3022
U+963F.0	fR/S	 170.5
U+963F.0	fTotalStrokes	 8
U+963F.0	fCangjie	NLMNR
U+963F.0	fFourCorner	71220
U+963F.0	fHakka	  o1;  a1; a5; a6; a3
U+963F.0	fCantonese	aa3; aa2; ngaa3 (also); ngo1 (also); ngaa2 (also); o1; aak3; aa1
U+963F.0	fMandarin	e1; a5; a3; a2; e3
U+963F.0	fEnglish	[1] favor; toady; assent; pander to; play up to [2] rely on [3] riverbank [4] corner; edge [5] pillar [6] slender and beautiful [7] discharge (night soil, urine, etc.) [8] a Chinese family name
U+963F.1	fUTF8	阿
U+963F.1	fCNS11643	1-4F5F
U+963F.1	fBig5	AAFC
U+963F.1	fGB	0-3022
U+963F.1	fR/S	 170.5
U+963F.1	fTotalStrokes	 8
U+963F.1	fCangjie	NLMNR
U+963F.1	fFourCorner	71220
U+963F.1	fMandarin	a4; a1
U+963F.1	fEnglish	[1] a prefix to a name or term of address [2] used in transliteration [3]  a prefix to a name or term of address
U+9640.0	fUTF8	陀
U+9640.0	fCNS11643	1-4F5E
U+9640.0	fBig5	AAFB
U+9640.0	fGB	0-4D53
U+9640.0	fR/S	 170.5
U+9640.0	fTotalStrokes	 8
U+9640.0	fCangjie	NLJP
U+9640.0	fFourCorner	73211
U+9640.0	fHakka	  to2; to3
U+9640.0	fCantonese	to4; to2 (also)
U+9640.0	fMandarin	tuo2; duo4; chi2
U+9640.0	fEnglish	 rugged terrain; rough terrain; craggy
U+9641.0	fUTF8	陁
U+9641.0	fCNS11643	3-2A7B
U+9641.0	fGB	5-746C
U+9641.0	fR/S	 170.5
U+9641.0	fTotalStrokes	 8
U+9641.0	fCangjie	NLOPD
U+9641.0	fFourCorner	78210
U+9641.0	fHakka	  to2; ri3; ji3
U+9641.0	fCantonese	to4; ji5
U+9641.0	fMandarin	tuo2; yi3
U+9641.0	fEnglish	 steep bank; declivity
U+9642.0	fUTF8	陂
U+9642.0	fCNS11643	1-4F62
U+9642.0	fBig5	AB40
U+9642.0	fGB	0-5A69
U+9642.0	fR/S	 170.5
U+9642.0	fTotalStrokes	 8
U+9642.0	fCangjie	NLDHE
U+9642.0	fFourCorner	74247
U+9642.0	fHakka	  bi1; bi5; po1
U+9642.0	fCantonese	bei1; po1; pei4; bei3; bo1
U+9642.0	fMandarin	bei1; bi4; pi2; bo1; po2; zhi3
U+9642.0	fEnglish	[1] water pond; reservoir [2] hillside
U+9642.1	fUTF8	陂
U+9642.1	fCNS11643	1-4F62
U+9642.1	fBig5	AB40
U+9642.1	fGB	0-5A69
U+9642.1	fR/S	 170.5
U+9642.1	fTotalStrokes	 8
U+9642.1	fCangjie	NLDHE
U+9642.1	fFourCorner	74247
U+9642.1	fMandarin	po1
U+9642.1	fEnglish	steep and craggy
U+9643.0	fUTF8	陃
U+9643.0	fCNS11643	2-287C
U+9643.0	fBig5	CDDC
U+9643.0	fGB	E-4425
U+9643.0	fR/S	 170.5
U+9643.0	fTotalStrokes	 8
U+9643.0	fCangjie	NLMOB
U+9643.0	fFourCorner	71227
U+9643.0	fCantonese	bing2; bo3
U+9643.0	fMandarin	bing3
U+9644.0	fUTF8	附
U+9644.0	fCNS11643	1-4F61
U+9644.0	fBig5	AAFE
U+9644.0	fGB	0-383D
U+9644.0	fR/S	 170.5
U+9644.0	fTotalStrokes	 8
U+9644.0	fCangjie	NLODI
U+9644.0	fFourCorner	74200
U+9644.0	fHakka	  fu5
U+9644.0	fCantonese	fu6
U+9644.0	fMandarin	fu4; pou3; fu1; bu4
U+9644.0	fEnglish	[1] rely on; be dependent on; attach to; adhere to [2] attach; enclose; send along with; append; enclosed [3] near; close to [4] add to; increase [5] be possessed by; possess (evil spirit)
U+9645.0	fUTF8	际
U+9645.0	fGB	0-3C4A
U+9645.0	fR/S	 170.5
U+9645.0	fTotalStrokes	 8
U+9645.0	fCangjie	NLMMF
U+9645.0	fHakka	 zi5
U+9645.0	fCantonese	zai3
U+9645.0	fMandarin	ji4
U+9645.0	fEnglish	simplified form of &#x969B;
U+9646.0	fUTF8	陆
U+9646.0	fGB	0-423D
U+9646.0	fR/S	 170.5
U+9646.0	fTotalStrokes	 8
U+9646.0	fCangjie	NLQU
U+9646.0	fHakka	 liuk8
U+9646.0	fCantonese	luk6
U+9646.0	fMandarin	lu4; liu4
U+9646.0	fEnglish	simplified form of &#x9678;
U+9647.0	fUTF8	陇
U+9647.0	fGB	0-4224
U+9647.0	fR/S	 170.5
U+9647.0	fTotalStrokes	 8
U+9647.0	fCangjie	NLIKP
U+9647.0	fHakka	 liung1; lung3
U+9647.0	fCantonese	lung5
U+9647.0	fMandarin	long3; xi2
U+9647.0	fEnglish	simplified form of &#x96B4;
U+9648.0	fUTF8	陈
U+9648.0	fGB	0-3342
U+9648.0	fR/S	 170.5
U+9648.0	fTotalStrokes	 8
U+9648.0	fCangjie	NLKD
U+9648.0	fHakka	 chin2
U+9648.0	fCantonese	can4; can2 (also); zan6
U+9648.0	fMandarin	chen2; zhen4
U+9648.0	fEnglish	simplified form of &#x9673;
U+9649.0	fUTF8	陉
U+9649.0	fGB	0-5A6A
U+9649.0	fR/S	 170.5
U+9649.0	fTotalStrokes	 8
U+9649.0	fCangjie	NLNOM
U+9649.0	fHakka	 hin2; gang1; gin1
U+9649.0	fCantonese	jing4; ging3; zing4
U+9649.0	fMandarin	xing2; jing4
U+9649.0	fEnglish	simplified form of &#x9658;
U+964A.0	fUTF8	陊
U+964A.0	fCNS11643	2-2D67
U+964A.0	fBig5	D0C6
U+964A.0	fGB	5-7473
U+964A.0	fR/S	 170.6
U+964A.0	fTotalStrokes	 9
U+964A.0	fCangjie	NLNIN
U+964A.0	fFourCorner	77227
U+964A.0	fHakka	  to5
U+964A.0	fCantonese	do6; to5 (also); to4
U+964A.0	fMandarin	duo4; zhi4
U+964A.0	fEnglish	 fall; come down with a crash
U+964B.0	fUTF8	陋
U+964B.0	fCNS11643	1-536F
U+964B.0	fBig5	ADAE
U+964B.0	fGB	0-422A
U+964B.0	fR/S	 170.6
U+964B.0	fTotalStrokes	 9
U+964B.0	fCangjie	NLMBV
U+964B.0	fFourCorner	71212
U+964B.0	fHakka	 leu5; lieu5; leu6; liu3; leu3
U+964B.0	fCantonese	lau6
U+964B.0	fMandarin	lou4; lu4
U+964B.0	fEnglish	[1] narrow and small [2] ugly [3] vile; low; means; humble [4] ignorant; crude; simple-minded [5] poor (performance, knowledge, etc.); inferior; superficial; shallow [6] stingy; tight-fisted
U+964C.0	fUTF8	陌
U+964C.0	fCNS11643	1-5370
U+964C.0	fBig5	ADAF
U+964C.0	fGB	0-4430
U+964C.0	fR/S	 170.6
U+964C.0	fTotalStrokes	 9
U+964C.0	fCangjie	NLMA
U+964C.0	fFourCorner	71260
U+964C.0	fHakka	 met7; mak7
U+964C.0	fCantonese	mak6; maak6
U+964C.0	fMandarin	mo4; pi2; po1
U+964C.0	fEnglish	[1] foot path between rice fields [2] road; street
U+964D.0	fUTF8	降
U+964D.0	fCNS11643	1-5371
U+964D.0	fBig5	ADB0
U+964D.0	fGB	0-3D35
U+964D.0	fR/S	 170.6
U+964D.0	fTotalStrokes	 9
U+964D.0	fCangjie	NLHEQ
U+964D.0	fFourCorner	77254
U+964D.0	fHakka	  hong2; hong3
U+964D.0	fCantonese	gong3; hong4
U+964D.0	fMandarin	jiang4; hong2; xiang4
U+964D.0	fEnglish	[1] descend [2] fall; drop [3] lower [4] condescend; deign [5] surrender
U+964D.1	fUTF8	降
U+964D.1	fCNS11643	1-5371
U+964D.1	fBig5	ADB0
U+964D.1	fGB	0-3D35
U+964D.1	fR/S	 170.6
U+964D.1	fTotalStrokes	 9
U+964D.1	fCangjie	NLHEQ
U+964D.1	fFourCorner	77254
U+964D.1	fHakka	 gong5
U+964D.1	fMandarin	xiang2
U+964D.1	fEnglish	[1] surrender; submit to [2] bring up terms; conquer; subjugate
U+964E.0	fUTF8	陎
U+964E.0	fCNS11643	2-2D68
U+964E.0	fBig5	D0C7
U+964E.0	fGB	E-4426
U+964E.0	fR/S	 170.6
U+964E.0	fTotalStrokes	 9
U+964E.0	fCangjie	NLHJD
U+964E.0	fFourCorner	75290
U+964E.0	fCantonese	syu4
U+964E.0	fMandarin	shu1
U+964F.0	fUTF8	陏
U+964F.0	fCNS11643	2-2D64
U+964F.0	fBig5	D0C3
U+964F.0	fGB	3-7534
U+964F.0	fR/S	 170.6
U+964F.0	fTotalStrokes	 9
U+964F.0	fCangjie	NLKB
U+964F.0	fFourCorner	74227
U+964F.0	fCantonese	do6; ceoi4
U+964F.0	fMandarin	duo4; mo4; sui2
U+9650.0	fUTF8	限
U+9650.0	fCNS11643	1-536E
U+9650.0	fBig5	ADAD
U+9650.0	fGB	0-4F5E
U+9650.0	fR/S	 170.6
U+9650.0	fTotalStrokes	 9
U+9650.0	fCangjie	NLAV
U+9650.0	fFourCorner	77232
U+9650.0	fHakka	  han5;  han3; han6
U+9650.0	fCantonese	haan6
U+9650.0	fMandarin	xian4; en3; wen3
U+9650.0	fEnglish	[1] boundary; limit; line [2] doorsill; threshold [3] limits; restriction; limit; restrict [4] specify; fix
U+9651.0	fUTF8	陑
U+9651.0	fCNS11643	2-2D65
U+9651.0	fBig5	D0C4
U+9651.0	fGB	3-7535
U+9651.0	fR/S	 170.6
U+9651.0	fTotalStrokes	 9
U+9651.0	fCangjie	NLMBL
U+9651.0	fFourCorner	71227
U+9651.0	fHakka	 ji2; ri2
U+9651.0	fCantonese	ji4
U+9651.0	fMandarin	er2
U+9651.0	fEnglish	the place where the Shong dynasty was founded
U+9652.0	fUTF8	陒
U+9652.0	fCNS11643	4-2B5D
U+9652.0	fGB	5-7472
U+9652.0	fR/S	 170.6
U+9652.0	fTotalStrokes	 9
U+9652.0	fCangjie	NLNMU
U+9652.0	fFourCorner	77210
U+9652.0	fCantonese	gwai2
U+9652.0	fMandarin	gui3
U+9653.0	fUTF8	陓
U+9653.0	fCNS11643	2-2D66
U+9653.0	fBig5	D0C5
U+9653.0	fGB	3-7536
U+9653.0	fR/S	 170.6
U+9653.0	fTotalStrokes	 9
U+9653.0	fCangjie	NLKMS
U+9653.0	fFourCorner	74227
U+9653.0	fCantonese	jyu1; wu1 (also)
U+9653.0	fMandarin	wu1; yu1
U+9654.0	fUTF8	陔
U+9654.0	fCNS11643	2-2D63
U+9654.0	fBig5	D0C2
U+9654.0	fGB	0-5A6B
U+9654.0	fR/S	 170.6
U+9654.0	fTotalStrokes	 9
U+9654.0	fCangjie	NLYVO
U+9654.0	fFourCorner	70282
U+9654.0	fHakka	  goi1
U+9654.0	fCantonese	goi1
U+9654.0	fMandarin	gai1
U+9654.0	fEnglish	[1] step; grade [2] ridges between plots of farm land
U+9655.0	fUTF8	陕
U+9655.0	fCNS11643	4-2B5B
U+9655.0	fGB	0-4942
U+9655.0	fR/S	 170.6
U+9655.0	fTotalStrokes	 9
U+9655.0	fCangjie	NLKT
U+9655.0	fFourCorner	75230
U+9655.0	fHakka	 sham3
U+9655.0	fCantonese	sim2
U+9655.0	fMandarin	shan3; xia2; sheng1
U+9655.0	fEnglish	simplified form of &#x965D;
U+9656.0	fUTF8	陖
U+9656.0	fCNS11643	3-3436
U+9656.0	fGB	5-7475
U+9656.0	fR/S	  170.7
U+9656.0	fTotalStrokes	 10
U+9656.0	fCangjie	NLICE
U+9656.0	fFourCorner	73240
U+9656.0	fCantonese	zeon3
U+9656.0	fMandarin	xun4; jun4
U+9657.0	fUTF8	陗
U+9657.0	fCNS11643	4-303F
U+9657.0	fGB	E-4427
U+9657.0	fR/S	 170.7
U+9657.0	fTotalStrokes	 10
U+9657.0	fCangjie	NLFB
U+9657.0	fFourCorner	79220
U+9657.0	fHakka	  ciau5
U+9657.0	fCantonese	ciu3; ciu5
U+9657.0	fMandarin	qiao4
U+9657.0	fEnglish	 steep hill; precipitous
U+9658.0	fUTF8	陘
U+9658.0	fCNS11643	1-5867
U+9658.0	fBig5	B0A4
U+9658.0	fGB	1-5A6A
U+9658.0	fR/S	 170.7
U+9658.0	fTotalStrokes	 10
U+9658.0	fCangjie	NLMVM
U+9658.0	fFourCorner	71211
U+9658.0	fHakka	 hin2; gang1; gin1
U+9658.0	fCantonese	jing4; ging3; zing4
U+9658.0	fMandarin	xing2; jing4
U+9658.0	fEnglish	 defile; mountain pass; gorge; deep valley
U+9659.0	fUTF8	陙
U+9659.0	fCNS11643	3-3434
U+9659.0	fGB	3-753B
U+9659.0	fR/S	 170.7
U+9659.0	fTotalStrokes	 10
U+9659.0	fCangjie	NLMMV
U+9659.0	fFourCorner	71230
U+9659.0	fCantonese	seon4
U+9659.0	fMandarin	chun2
U+965A.0	fUTF8	陚
U+965A.0	fCNS11643	3-395E
U+965A.0	fGB	3-7540
U+965A.0	fR/S	 170.8
U+965A.0	fTotalStrokes	 11
U+965A.0	fCangjie	NLMPM
U+965A.0	fFourCorner	73240
U+965A.0	fCantonese	fu3
U+965A.0	fMandarin	fu4
U+965B.0	fUTF8	陛
U+965B.0	fCNS11643	1-5864
U+965B.0	fBig5	B0A1
U+965B.0	fGB	0-315D
U+965B.0	fR/S	 170.7
U+965B.0	fTotalStrokes	 10
U+965B.0	fCangjie	NLPPG
U+965B.0	fFourCorner	71214
U+965B.0	fHakka	 pi3; pi6; pi5
U+965B.0	fCantonese	bai6
U+965B.0	fMandarin	bi4
U+965B.0	fEnglish	wide and high step in the palace; steps leading to the throne
U+965C.0	fUTF8	陜
U+965C.0	fCNS11643	2-3348
U+965C.0	fBig5	D445
U+965C.0	fGB	E-4428
U+965C.0	fR/S	 170.7
U+965C.0	fTotalStrokes	 10
U+965C.0	fCangjie	NLKOO
U+965C.0	fFourCorner	74238
U+965C.0	fHakka	  hiap8; kiap8; sam3
U+965C.0	fCantonese	haap6; haap3 (also); gip6
U+965C.0	fMandarin	xia2; shan3; xian4
U+965C.0	fEnglish	 narrow mountain pass
U+965D.0	fUTF8	陝
U+965D.0	fCNS11643	1-5865
U+965D.0	fBig5	B0A2
U+965D.0	fGB	1-4942
U+965D.0	fR/S	 170.7
U+965D.0	fTotalStrokes	 10
U+965D.0	fCangjie	NLKOO
U+965D.0	fFourCorner	74238
U+965D.0	fHakka	 sam3;  sham3
U+965D.0	fCantonese	sim2
U+965D.0	fMandarin	shan3; xia2; sheng1
U+965D.0	fEnglish	[1] short for Shensi province [2] a Chinese family name
U+965E.0	fUTF8	陞
U+965E.0	fCNS11643	1-5868
U+965E.0	fBig5	B0A5
U+965E.0	fGB	8-2F67
U+965E.0	fR/S	 170.7
U+965E.0	fTotalStrokes	 10
U+965E.0	fCangjie	NLHTG
U+965E.0	fFourCorner	74214
U+965E.0	fHakka	  shin1; siin1; sin1
U+965E.0	fCantonese	sing1
U+965E.0	fMandarin	sheng1
U+965E.0	fEnglish	[1] promote; rise; ascend; elevate; hoist [2] a Chinese family name
U+965F.0	fUTF8	陟
U+965F.0	fCNS11643	2-3349
U+965F.0	fBig5	D446
U+965F.0	fGB	0-5A6C
U+965F.0	fR/S	 170.7
U+965F.0	fTotalStrokes	 10
U+965F.0	fCangjie	NLYLH
U+965F.0	fFourCorner	71221
U+965F.0	fHakka	 zhit7; tsiit7; zit7
U+965F.0	fCantonese	zik1
U+965F.0	fMandarin	zhi4; de2; shan3
U+965F.0	fEnglish	[1] mount; ascend [2] advance; elevate; promote
U+9660.0	fUTF8	陠
U+9660.0	fCNS11643	3-3433
U+9660.0	fGB	5-7474
U+9660.0	fR/S	 170.7
U+9660.0	fTotalStrokes	 10
U+9660.0	fCangjie	NLIJB
U+9660.0	fFourCorner	73220
U+9660.0	fCantonese	pou1
U+9660.0	fMandarin	pu1
U+9661.0	fUTF8	陡
U+9661.0	fCNS11643	1-5863
U+9661.0	fBig5	B07E
U+9661.0	fGB	0-3638
U+9661.0	fR/S	 170.7
U+9661.0	fTotalStrokes	 10
U+9661.0	fCangjie	NLGYO
U+9661.0	fFourCorner	74281
U+9661.0	fHakka	  deu3; diu3
U+9661.0	fCantonese	dau2
U+9661.0	fMandarin	dou3
U+9661.0	fEnglish	[1] steep; sloping; precipituous [2] abruptly; suddenly
U+9662.0	fUTF8	院
U+9662.0	fCNS11643	1-5861
U+9662.0	fBig5	B07C
U+9662.0	fGB	0-543A
U+9662.0	fR/S	 170.7
U+9662.0	fTotalStrokes	 10
U+9662.0	fCangjie	NLJMU
U+9662.0	fFourCorner	73211
U+9662.0	fHakka	  jen5;  jan5; ren5; jan6
U+9662.0	fCantonese	jyun2; jyun6 (also)
U+9662.0	fMandarin	yuan4; huan2; wan3; yung4
U+9662.0	fEnglish	[1] courtyard; yard [2] a designation for certain government offices and public places [3] short for the Executive Yuan, Legislative Yan, Examination Yuan, Judicial Yuan, or Control Yuan
U+9663.0	fUTF8	陣
U+9663.0	fCNS11643	1-5862
U+9663.0	fBig5	B07D
U+9663.0	fGB	1-5573
U+9663.0	fR/S	 170.7
U+9663.0	fTotalStrokes	 10
U+9663.0	fCangjie	NLJWJ
U+9663.0	fFourCorner	75206
U+9663.0	fHakka	 chin6; cihn5; cin3; cin5; chin5
U+9663.0	fCantonese	zan6; zan2 (also)
U+9663.0	fMandarin	zhen4
U+9663.0	fEnglish	[1] column or troops; row or file of troops; the army; the rank and file [2] battle; go to war; fight at the front; battlefield; battle array; battle formation [3] anything that occurs in a certain duration of spel of time (rain, wind, business, etc.)
U+9664.0	fUTF8	除
U+9664.0	fCNS11643	1-5866
U+9664.0	fBig5	B0A3
U+9664.0	fGB	0-337D
U+9664.0	fR/S	 170.7
U+9664.0	fTotalStrokes	 10
U+9664.0	fCangjie	NLOMD
U+9664.0	fFourCorner	78294
U+9664.0	fHakka	 cu2;  chu2; chu3; ciu2
U+9664.0	fCantonese	ceoi4; cyu4 (also); cyu1; zyu6
U+9664.0	fMandarin	chu2; shu1; zhu4; yu2
U+9664.0	fEnglish	[1] divide; division; divisible (mathemathics) [2] eliminate; remove; rid of; wipe out [3] be appointed to an official rank or office [4] subtract; deduct [5] aside from; besides; except; unless [6] change; turn (as a new year)
U+9665.0	fUTF8	陥
U+9665.0	fGB	E-4429
U+9665.0	fR/S	 170.7
U+9665.0	fTotalStrokes	 10
U+9665.0	fCangjie	NLNLA
U+9665.0	fCantonese	haam6
U+9665.0	fMandarin	xian4
U+9665.0	fEnglish	 submerge; sink; plunge trap
U+9666.0	fUTF8	陦
U+9666.0	fGB	E-442A
U+9666.0	fR/S	 170.7
U+9666.0	fTotalStrokes	 10
U+9666.0	fCangjie	NLQKI
U+9666.0	fCantonese	dou2
U+9666.0	fMandarin	zhi4
U+9667.0	fUTF8	陧
U+9667.0	fGB	0-5A6D
U+9667.0	fR/S	 170.7
U+9667.0	fTotalStrokes	 10
U+9667.0	fCangjie	NLAG
U+9667.0	fHakka	 ngiet7; ngiap7
U+9667.0	fCantonese	nip6; lip6 (also); nit6 (also); lit6 (also)
U+9667.0	fMandarin	nie4
U+9667.0	fEnglish	simplified form of &#x9689;
U+9668.0	fUTF8	陨
U+9668.0	fGB	0-5449
U+9668.0	fR/S	 170.7
U+9668.0	fTotalStrokes	 10
U+9668.0	fCangjie	NLRBO
U+9668.0	fHakka	 jun3
U+9668.0	fCantonese	wan5; jyun4
U+9668.0	fMandarin	yun3; yuan2; ai4; e4; yun2; yun4
U+9668.0	fEnglish	simplified form of &#x9695;
U+9669.0	fUTF8	险
U+9669.0	fGB	0-4F55
U+9669.0	fR/S	 170.7
U+9669.0	fTotalStrokes	 10
U+9669.0	fCangjie	NLOMM
U+9669.0	fHakka	 hiam3
U+9669.0	fCantonese	him2
U+9669.0	fMandarin	xian3; yan2; shan3; wei3
U+9669.0	fEnglish	simplified form of &#x96AA;
U+966A.0	fUTF8	陪
U+966A.0	fCNS11643	1-5D71
U+966A.0	fBig5	B3AD
U+966A.0	fGB	0-4563
U+966A.0	fR/S	 170.8
U+966A.0	fTotalStrokes	 11
U+966A.0	fCangjie	NLYTR
U+966A.0	fFourCorner	70261
U+966A.0	fHakka	 pui2;  poi2; boi2; pi2; pui5
U+966A.0	fCantonese	pui4
U+966A.0	fMandarin	pei2
U+966A.0	fEnglish	[1] accompany; be with; keep company [2] same as $ [3] assist
U+966B.0	fUTF8	陫
U+966B.0	fCNS11643	2-3A2D
U+966B.0	fBig5	D849
U+966B.0	fGB	5-7478
U+966B.0	fR/S	 170.8
U+966B.0	fTotalStrokes	 11
U+966B.0	fCangjie	NLLMY
U+966B.0	fFourCorner	71211
U+966B.0	fHakka	 pi5; pui5
U+966B.0	fCantonese	fei2; fei6
U+966B.0	fMandarin	pei2; fei4; fei3
U+966C.0	fUTF8	陬
U+966C.0	fCNS11643	1-5D79
U+966C.0	fBig5	B3B5
U+966C.0	fGB	0-5A6E
U+966C.0	fR/S	 170.8
U+966C.0	fTotalStrokes	 11
U+966C.0	fCangjie	NLSJE
U+966C.0	fFourCorner	71247
U+966C.0	fHakka	  zeu1
U+966C.0	fCantonese	zau1
U+966C.0	fMandarin	zou1; bo2; zhe2; chui2
U+966C.0	fEnglish	[1] corner; nook; niche [2] foothills [3] a frontier village where people live together [4] the first month of the lunar year
U+966D.0	fUTF8	陭
U+966D.0	fCNS11643	2-3A2C
U+966D.0	fBig5	D848
U+966D.0	fGB	3-7541
U+966D.0	fR/S	 170.8
U+966D.0	fTotalStrokes	 11
U+966D.0	fCangjie	NLKMR
U+966D.0	fFourCorner	74221
U+966D.0	fHakka	 ji1; ri1; ri2; ji2; {ze1}; {je1}
U+966D.0	fCantonese	kei1; ji2
U+966D.0	fMandarin	yi1; yi4; yi3; qi1; ji4
U+966D.0	fEnglish	 project
U+966E.0	fUTF8	陮
U+966E.0	fCNS11643	3-3961
U+966E.0	fGB	5-7479
U+966E.0	fR/S	 170.8
U+966E.0	fTotalStrokes	 11
U+966E.0	fCangjie	NLOG
U+966E.0	fFourCorner	70210
U+966E.0	fCantonese	deoi6
U+966E.0	fMandarin	dui3; dui4
U+966F.0	fUTF8	陯
U+966F.0	fCNS11643	2-3A2F
U+966F.0	fBig5	D84B
U+966F.0	fGB	E-442B
U+966F.0	fR/S	 170.8
U+966F.0	fTotalStrokes	 11
U+966F.0	fCangjie	NLOMB
U+966F.0	fFourCorner	78227
U+966F.0	fCantonese	leon4; leon6
U+966F.0	fMandarin	lun2; lun4
U+9670.0	fUTF8	陰
U+9670.0	fCNS11643	1-5D75
U+9670.0	fBig5	B3B1
U+9670.0	fGB	1-5275
U+9670.0	fR/S	 170.8
U+9670.0	fTotalStrokes	 11
U+9670.0	fCangjie	NLOII
U+9670.0	fFourCorner	78231
U+9670.0	fHakka	  jim1; rim1; jang3
U+9670.0	fCantonese	jam1; jam3; am1; ngam1 (also)
U+9670.0	fMandarin	yin1; an1
U+9670.0	fEnglish	[1] negative (as opposite to positive, as electricity) [2] female; feminine [3] cloudy; dark [4] shady [5] secret [6] the back side [7] the north side of a mountain [8] the south side of a stream [9] reproductive organs of both sexes [10] Hades; hell [11] cunning and crafty [12] injure another in a clandestine manner [13] time [14] a Chinese family name
U+9670.1	fUTF8	陰
U+9670.1	fCNS11643	1-5D75
U+9670.1	fBig5	B3B1
U+9670.1	fGB	1-5275
U+9670.1	fR/S	 170.8
U+9670.1	fTotalStrokes	 11
U+9670.1	fCangjie	NLOII
U+9670.1	fFourCorner	78231
U+9670.1	fMandarin	yin4
U+9670.1	fEnglish	shaded by trees
U+9671.0	fUTF8	陱
U+9671.0	fCNS11643	2-3A2E
U+9671.0	fBig5	D84A
U+9671.0	fGB	5-747C
U+9671.0	fR/S	 170.8
U+9671.0	fTotalStrokes	 11
U+9671.0	fCangjie	NLPFD
U+9671.0	fFourCorner	77220
U+9671.0	fCantonese	guk1
U+9671.0	fMandarin	ju2; ju1
U+9672.0	fUTF8	陲
U+9672.0	fCNS11643	1-6270
U+9672.0	fBig5	B6AB
U+9672.0	fGB	0-5A6F
U+9672.0	fR/S	 170.8
U+9672.0	fTotalStrokes	 11
U+9672.0	fCangjie	NLHJM
U+9672.0	fFourCorner	72214
U+9672.0	fHakka	 sui2;  shui2
U+9672.0	fCantonese	seoi4
U+9672.0	fMandarin	chui2
U+9672.0	fEnglish	 frontier; border
U+9673.0	fUTF8	陳
U+9673.0	fCNS11643	1-5D73
U+9673.0	fBig5	B3AF
U+9673.0	fGB	1-3342
U+9673.0	fR/S	 170.8
U+9673.0	fTotalStrokes	 11
U+9673.0	fCangjie	NLDW
U+9673.0	fFourCorner	75296
U+9673.0	fHakka	 cin2;  chin2; cihn2; chin3; chin5; cihn5; chin6; cin3
U+9673.0	fCantonese	can4; zan6; can2 (also)
U+9673.0	fMandarin	chen2
U+9673.0	fEnglish	[1] a Chinese family name [2] arrange; exhibit; display; spread out [3] tell; state; narrate; explain [4] old; stale; preserved for a long time [5] the name of a stat in the Spring and Autumn period [6] name of a dynasty (557-589) [7] make public
U+9673.1	fUTF8	陳
U+9673.1	fCNS11643	1-5D73
U+9673.1	fBig5	B3AF
U+9673.1	fGB	1-3342
U+9673.1	fR/S	 170.8
U+9673.1	fTotalStrokes	 11
U+9673.1	fCangjie	NLDW
U+9673.1	fFourCorner	75296
U+9673.1	fMandarin	zhen4
U+9673.1	fEnglish	tactical deployment of troops
U+9674.0	fUTF8	陴
U+9674.0	fCNS11643	1-5D76
U+9674.0	fBig5	B3B2
U+9674.0	fGB	0-5A70
U+9674.0	fR/S	 170.8
U+9674.0	fTotalStrokes	 11
U+9674.0	fCangjie	NLHWJ
U+9674.0	fFourCorner	76140
U+9674.0	fHakka	  bi1
U+9674.0	fCantonese	pei4
U+9674.0	fMandarin	pi2; bei1; bi4; pai2
U+9674.0	fEnglish	a parapet on a city wall
U+9675.0	fUTF8	陵
U+9675.0	fCNS11643	1-5D72
U+9675.0	fBig5	B3AE
U+9675.0	fGB	0-416A
U+9675.0	fR/S	 170.8
U+9675.0	fTotalStrokes	 11
U+9675.0	fCangjie	NLGCE
U+9675.0	fFourCorner	74247
U+9675.0	fHakka	  lin2; lin3
U+9675.0	fCantonese	ling4
U+9675.0	fMandarin	ling2
U+9675.0	fEnglish	[1] high mound [2] mausoleum; tomb of an emperor [3] offend; insult; outrage; abuse [4] usurp [5] clime; scale [6] a Chinese family name
U+9676.0	fUTF8	陶
U+9676.0	fCNS11643	1-5D77
U+9676.0	fBig5	B3B3
U+9676.0	fGB	0-4C55
U+9676.0	fR/S	 170.8
U+9676.0	fTotalStrokes	 11
U+9676.0	fCangjie	NLPOU
U+9676.0	fFourCorner	77220
U+9676.0	fHakka	  tau2; to2;  jau2; jau3; to3; rau2; jeu2
U+9676.0	fCantonese	tou4; jiu4
U+9676.0	fMandarin	tao2; dao4
U+9676.0	fEnglish	[1] make pottery; ceramics; pottery; earthenware [2] happy; joyful [3] move nad influence a person [4] a Chinese family name
U+9676.1	fUTF8	陶
U+9676.1	fCNS11643	1-5D77
U+9676.1	fBig5	B3B3
U+9676.1	fGB	0-4C55
U+9676.1	fR/S	 170.8
U+9676.1	fTotalStrokes	 11
U+9676.1	fCangjie	NLPOU
U+9676.1	fFourCorner	77220
U+9676.1	fMandarin	yao2
U+9676.1	fEnglish	correct form of &#xthe name of a person in ancient China;
U+9677.0	fUTF8	陷
U+9677.0	fCNS11643	1-5D78
U+9677.0	fBig5	B3B4
U+9677.0	fGB	0-4F5D
U+9677.0	fR/S	 170.8
U+9677.0	fTotalStrokes	 11
U+9677.0	fCangjie	NLNHX
U+9677.0	fFourCorner	77277
U+9677.0	fHakka	  ham3;  ham5
U+9677.0	fCantonese	haam6; ham6 (also)
U+9677.0	fMandarin	xian4; lou4; xuan4
U+9677.0	fEnglish	[1] submerge; sink; stick; bog [2] frame (up); harm another with trumped-up charges [3] entrap; beguile [4] crush (enemy position); fall; capture [5] defect; deficiency
U+9678.0	fUTF8	陸
U+9678.0	fCNS11643	1-5D74
U+9678.0	fBig5	B3B0
U+9678.0	fGB	1-423D
U+9678.0	fR/S	 170.8
U+9678.0	fTotalStrokes	 11
U+9678.0	fCangjie	NLGCG
U+9678.0	fFourCorner	74214
U+9678.0	fHakka	  liuk8; liuk7;  luk8
U+9678.0	fCantonese	luk6
U+9678.0	fMandarin	lu4; liu4
U+9678.0	fEnglish	[1] land; continent; shore [2] by way; by land; land transportation [3] six used in accountancy [4] a Chinese family name
U+9679.0	fUTF8	陹
U+9679.0	fCNS11643	3-3965
U+9679.0	fGB	E-442C
U+9679.0	fR/S	 170.8
U+9679.0	fTotalStrokes	 11
U+9679.0	fCangjie	NLHTA
U+9679.0	fFourCorner	74260
U+9679.0	fCantonese	sing1
U+9679.0	fMandarin	sheng1
U+967A.0	fUTF8	険
U+967A.0	fGB	E-442D
U+967A.0	fR/S	 170.8
U+967A.0	fTotalStrokes	 11
U+967A.0	fCangjie	XNLOM
U+967A.0	fCantonese	him2
U+967A.0	fMandarin	xian3
U+967A.0	fEnglish	narrow pass; strategic point
U+967B.0	fUTF8	陻
U+967B.0	fCNS11643	3-3F71
U+967B.0	fGB	E-442E
U+967B.0	fR/S	 170.9
U+967B.0	fTotalStrokes	 12
U+967B.0	fCangjie	NLMWG
U+967B.0	fFourCorner	71210
U+967B.0	fHakka	 jin1; rin1; jen1
U+967B.0	fCantonese	jan1
U+967B.0	fMandarin	yin1
U+967B.0	fEnglish	[1] mound [2] bury [3] dam (stream); block up
U+967C.0	fUTF8	陼
U+967C.0	fCNS11643	2-3A2B
U+967C.0	fBig5	D847
U+967C.0	fGB	5-7476
U+967C.0	fR/S	 170.9
U+967C.0	fTotalStrokes	 12
U+967C.0	fCangjie	NLJKA
U+967C.0	fFourCorner	74260
U+967C.0	fHakka	  du3; zhu3; tswu3
U+967C.0	fCantonese	zyu2
U+967C.0	fMandarin	zhu3; du3; tu2
U+967C.0	fEnglish	islet
U+967D.0	fUTF8	陽
U+967D.0	fCNS11643	1-626C
U+967D.0	fBig5	B6A7
U+967D.0	fGB	1-5174
U+967D.0	fR/S	 170.9
U+967D.0	fTotalStrokes	 12
U+967D.0	fCangjie	NLAMH
U+967D.0	fFourCorner	76227
U+967D.0	fHakka	  jong2; rong2; jong3
U+967D.0	fCantonese	joeng4; joeng2 (also)
U+967D.0	fMandarin	yang2; chang2
U+967D.0	fEnglish	[1] positive (electricity) [2] male; masculine [3] the sun; sunlight; solar [4] the north of a stream [5] the south of a hill [6] bright; brilliant [7] the male genitals [8] pertianing to this world (as opposed to Hades) [9] a Chinese family name
U+967E.0	fUTF8	陾
U+967E.0	fCNS11643	2-4144
U+967E.0	fBig5	DC7D
U+967E.0	fGB	3-7543
U+967E.0	fR/S	 170.9
U+967E.0	fTotalStrokes	 12
U+967E.0	fCangjie	NLMBK
U+967E.0	fFourCorner	71234
U+967E.0	fHakka	 jin2; rin2
U+967E.0	fCantonese	jing4
U+967E.0	fMandarin	reng2; er2; nou3
U+967E.0	fEnglish	in file
U+967F.0	fUTF8	陿
U+967F.0	fCNS11643	3-3F72
U+967F.0	fGB	E-442F
U+967F.0	fR/S	 170.9
U+967F.0	fTotalStrokes	 12
U+967F.0	fCangjie	NLSKO
U+967F.0	fFourCorner	71210
U+967F.0	fCantonese	haap6; haap3 (also); hap6 (also); gip6 (also)
U+967F.0	fMandarin	shan3; xia2
U+9680.0	fUTF8	隀
U+9680.0	fCNS11643	2-4148
U+9680.0	fBig5	DCA3
U+9680.0	fGB	E-4430
U+9680.0	fR/S	 170.9
U+9680.0	fTotalStrokes	 12
U+9680.0	fCangjie	NLHJG
U+9680.0	fFourCorner	72214
U+9680.0	fCantonese	cung4
U+9680.0	fMandarin	chong2
U+9681.0	fUTF8	隁
U+9681.0	fCNS11643	3-3F73
U+9681.0	fGB	5-7521
U+9681.0	fR/S	 170.9
U+9681.0	fTotalStrokes	 12
U+9681.0	fCangjie	NLSAV
U+9681.0	fFourCorner	71210
U+9681.0	fCantonese	jin2
U+9681.0	fMandarin	yan4
U+9682.0	fUTF8	隂
U+9682.0	fCNS11643	3-3F75
U+9682.0	fGB	E-4431
U+9682.0	fR/S	 170.9
U+9682.0	fTotalStrokes	 12
U+9682.0	fCangjie	NLOSI
U+9682.0	fFourCorner	78230
U+9682.0	fHakka	 jim1; rim1
U+9682.0	fCantonese	jam1; jam3; am1; ngam1 (also)
U+9682.0	fMandarin	yin1; an1; yin4
U+9682.0	fEnglish	 shady; secret; dark mysterious cold the negative of female principle in nature
U+9683.0	fUTF8	隃
U+9683.0	fCNS11643	2-4147
U+9683.0	fBig5	DCA2
U+9683.0	fGB	3-7545
U+9683.0	fR/S	 170.9
U+9683.0	fTotalStrokes	 12
U+9683.0	fCangjie	NLOMN
U+9683.0	fFourCorner	78221
U+9683.0	fCantonese	jyu4; jyu6; jiu4
U+9683.0	fMandarin	yu2; yao2; shu1; shu4; xu1
U+9683.0	fEnglish	[1] exceed [2] a county in Shensi province during the Han dynasty, famous for producnig ink-sticks
U+9684.0	fUTF8	隄
U+9684.0	fCNS11643	1-6271
U+9684.0	fBig5	B6AC
U+9684.0	fGB	E-4432
U+9684.0	fR/S	 170.9
U+9684.0	fTotalStrokes	 12
U+9684.0	fCangjie	NLAMO
U+9684.0	fFourCorner	76281
U+9684.0	fHakka	 ti2; doi1
U+9684.0	fCantonese	tai4
U+9684.0	fMandarin	di1; chi2; ti2; shi2
U+9684.0	fEnglish	 dike; embankment; levee
U+9685.0	fUTF8	隅
U+9685.0	fCNS11643	1-626D
U+9685.0	fBig5	B6A8
U+9685.0	fGB	0-5367
U+9685.0	fR/S	 170.9
U+9685.0	fTotalStrokes	 12
U+9685.0	fCangjie	NLWLB
U+9685.0	fFourCorner	76227
U+9685.0	fHakka	 ngi2
U+9685.0	fCantonese	jyu4; jyu6
U+9685.0	fMandarin	yu2
U+9685.0	fEnglish	[1] corner; nook [2] angle [3] remote place; out-of-the way place; recess
U+9686.0	fUTF8	隆
U+9686.0	fCNS11643	1-626E
U+9686.0	fBig5	B6A9
U+9686.0	fGB	0-4221
U+9686.0	fR/S	 170.9; 170.8
U+9686.0	fTotalStrokes	 12
U+9686.0	fCangjie	NLHEM
U+9686.0	fFourCorner	77214
U+9686.0	fHakka	  lung2; lun3
U+9686.0	fCantonese	lung4; naam4
U+9686.0	fMandarin	long2
U+9686.0	fEnglish	[1] prosperous; flourishing; brisk (business, etc.); booming [2] lofty; eminent; glorious [3] abundant; plentiful; abundant; ample; generous [4] the rumble (thunder, vehicles, artillery fire, etc.) [5] a Chinese family name
U+9686.1	fUTF8	隆
U+9686.1	fCNS11643	1-626E
U+9686.1	fBig5	B6A9
U+9686.1	fGB	0-4221
U+9686.1	fR/S	 170.9; 170.8
U+9686.1	fTotalStrokes	 12
U+9686.1	fCangjie	NLHEM
U+9686.1	fFourCorner	77214
U+9686.1	fMandarin	long1
U+9687.0	fUTF8	隇
U+9687.0	fCNS11643	2-4143
U+9687.0	fBig5	DC7C
U+9687.0	fGB	3-7542
U+9687.0	fR/S	 170.9
U+9687.0	fTotalStrokes	 12
U+9687.0	fCangjie	NLIHV
U+9687.0	fFourCorner	73250
U+9687.0	fCantonese	wai1
U+9687.0	fMandarin	wei1
U+9688.0	fUTF8	隈
U+9688.0	fCNS11643	2-4145
U+9688.0	fBig5	DC7E
U+9688.0	fGB	0-5A71
U+9688.0	fR/S	 170.9
U+9688.0	fTotalStrokes	 12
U+9688.0	fCangjie	NLWMV
U+9688.0	fFourCorner	76232
U+9688.0	fHakka	 woi1; voe1; voi1
U+9688.0	fCantonese	wui1
U+9688.0	fMandarin	wei1; wei4
U+9688.0	fEnglish	[1] river bend [2] mountain recess
U+9689.0	fUTF8	隉
U+9689.0	fCNS11643	2-4146
U+9689.0	fBig5	DCA1
U+9689.0	fGB	1-5A6D
U+9689.0	fR/S	 170.9
U+9689.0	fTotalStrokes	 12
U+9689.0	fCangjie	NLHXG
U+9689.0	fFourCorner	77214
U+9689.0	fHakka	  ngiap7;  ngiet7; nget7
U+9689.0	fCantonese	nip6; lip6 (also); nit6 (also); lit6 (also); jit6
U+9689.0	fMandarin	nie4
U+9689.0	fEnglish	correct form of &#xprecarious;dangerous
U+968A.0	fUTF8	隊
U+968A.0	fCNS11643	1-6269
U+968A.0	fBig5	B6A4
U+968A.0	fGB	1-3653
U+968A.0	fR/S	 170.9
U+968A.0	fTotalStrokes	 12
U+968A.0	fCangjie	NLTPO
U+968A.0	fFourCorner	78232
U+968A.0	fHakka	  dui5;  dui3; doi5; ch'ui6; cui5; chui3; zui3; zui5; chui5; tui5
U+968A.0	fCantonese	deoi6; zeoi6; deoi2 (also)
U+968A.0	fMandarin	dui4; sui4; zhui4
U+968A.0	fEnglish	[1] team; group; batch [2] the troops
U+968B.0	fUTF8	隋
U+968B.0	fCNS11643	1-626B
U+968B.0	fBig5	B6A6
U+968B.0	fGB	0-4B65
U+968B.0	fR/S	 170.9
U+968B.0	fTotalStrokes	 12
U+968B.0	fCangjie	NLKMB
U+968B.0	fFourCorner	74227
U+968B.0	fHakka	 sui2; to5; to6; cui2; sui3
U+968B.0	fCantonese	ceoi4; do6
U+968B.0	fMandarin	sui2; duo4; tuo3; sui1; sui4; tuo1
U+968B.0	fEnglish	[1] the Sui dynasty (581-618) [2] a Chinese family name
U+968C.0	fUTF8	隌
U+968C.0	fCNS11643	3-3F70
U+968C.0	fGB	5-7522
U+968C.0	fR/S	 170.9
U+968C.0	fTotalStrokes	 12
U+968C.0	fCangjie	NLYTA
U+968C.0	fFourCorner	70260
U+968C.0	fCantonese	am2; ngam2 (also)
U+968C.0	fMandarin	an3
U+968C.0	fEnglish	[1] the sun obscured by clouds; dark; gloomy [2] invisible; secret
U+968D.0	fUTF8	隍
U+968D.0	fCNS11643	1-626F
U+968D.0	fBig5	B6AA
U+968D.0	fGB	0-5A72
U+968D.0	fR/S	 170.9
U+968D.0	fTotalStrokes	 12
U+968D.0	fCangjie	NLHAG
U+968D.0	fFourCorner	76214
U+968D.0	fHakka	 fong2
U+968D.0	fCantonese	wong4
U+968D.0	fMandarin	huang2; yong4; zou1
U+968D.0	fEnglish	dry ditch; dry moat
U+968E.0	fUTF8	階
U+968E.0	fCNS11643	1-626A
U+968E.0	fBig5	B6A5
U+968E.0	fGB	1-3D57
U+968E.0	fR/S	 170.9
U+968E.0	fTotalStrokes	 12
U+968E.0	fCangjie	NLPPA
U+968E.0	fFourCorner	71261
U+968E.0	fHakka	  gai1; kie1; giai1
U+968E.0	fCantonese	gaai1
U+968E.0	fMandarin	jie1
U+968E.0	fEnglish	[1] way leading to the main hall [2] a flight of stairs; steps [3] rank; degree [4] rely on
U+968F.0	fUTF8	随
U+968F.0	fCNS11643	3-4540
U+968F.0	fGB	0-4B66
U+968F.0	fR/S	 170.10; 170.9
U+968F.0	fTotalStrokes	 13; 12
U+968F.0	fCangjie	NLYKB
U+968F.0	fFourCorner	74230
U+968F.0	fHakka	 sui2; cui2
U+968F.0	fCantonese	ceoi4
U+968F.0	fMandarin	sui2
U+968F.0	fEnglish	simplified form of &#x96A8;
U+9690.0	fUTF8	隐
U+9690.0	fGB	0-527E
U+9690.0	fR/S	 170.9
U+9690.0	fTotalStrokes	 12
U+9690.0	fCangjie	NLNSP
U+9690.0	fHakka	 jun3
U+9690.0	fCantonese	jan2; jan3
U+9690.0	fMandarin	yin3; yin4; sui4; zhui4
U+9690.0	fEnglish	simplified form of &#x96B1;
U+9691.0	fUTF8	隑
U+9691.0	fCNS11643	2-4859
U+9691.0	fBig5	E0D3
U+9691.0	fGB	3-7538
U+9691.0	fR/S	 170.10
U+9691.0	fTotalStrokes	 13
U+9691.0	fCangjie	NLUMT
U+9691.0	fFourCorner	72218
U+9691.0	fCantonese	kei4; goi1
U+9691.0	fMandarin	gai1; gai4; qi2; ai2; ai3; kai4; yi2
U+9692.0	fUTF8	隒
U+9692.0	fCNS11643	2-4857
U+9692.0	fBig5	E0D1
U+9692.0	fGB	3-7547
U+9692.0	fR/S	 170.10
U+9692.0	fTotalStrokes	 13
U+9692.0	fCangjie	NLTXC
U+9692.0	fFourCorner	78237
U+9692.0	fHakka	 hiam3
U+9692.0	fCantonese	jim2 (also); jim5
U+9692.0	fMandarin	yan3; lian3; qian3
U+9692.0	fEnglish	 the appearance of a mountain; as if two pots were standing one upon the other the steep bank of a stream a rough mountain path
U+9693.0	fUTF8	隓
U+9693.0	fCNS11643	2-4858
U+9693.0	fBig5	E0D2
U+9693.0	fGB	5-7524
U+9693.0	fR/S	 170.10
U+9693.0	fTotalStrokes	 13
U+9693.0	fCangjie	NLKMM
U+9693.0	fFourCorner	74211
U+9693.0	fHakka	 fi1; to3
U+9693.0	fCantonese	fai1; to5
U+9693.0	fMandarin	hui1; duo4
U+9694.0	fUTF8	隔
U+9694.0	fCNS11643	1-6752
U+9694.0	fBig5	B96A
U+9694.0	fGB	0-3874
U+9694.0	fR/S	 170.10
U+9694.0	fTotalStrokes	 13
U+9694.0	fCangjie	NLMRB
U+9694.0	fFourCorner	71227
U+9694.0	fHakka	  gak7
U+9694.0	fCantonese	gaak3; gik1
U+9694.0	fMandarin	ge2; ge1; rong3; jie1; ji1; jie2; jie4
U+9694.0	fEnglish	[1] separate; divide; partition [2] blocked; obstruct; be veiled [3] at a distance from; at an interval of
U+9695.0	fUTF8	隕
U+9695.0	fCNS11643	1-6753
U+9695.0	fBig5	B96B
U+9695.0	fGB	1-5449
U+9695.0	fR/S	 170.10
U+9695.0	fTotalStrokes	 13
U+9695.0	fCangjie	NLRBC
U+9695.0	fFourCorner	76286
U+9695.0	fHakka	  jun3; run3; run5; jun5
U+9695.0	fCantonese	wan5; jyun4
U+9695.0	fMandarin	yun3; yuan2; ai4; e4; yun2; yun4
U+9695.0	fEnglish	[1] fall [2] die
U+9696.0	fUTF8	隖
U+9696.0	fCNS11643	3-4542
U+9696.0	fGB	E-4433
U+9696.0	fR/S	 170.10
U+9696.0	fTotalStrokes	 13
U+9696.0	fCangjie	NLHRF
U+9696.0	fFourCorner	77220
U+9696.0	fHakka	 wu3; vu3
U+9696.0	fCantonese	wu2; ou3; ngou3 (also)
U+9696.0	fMandarin	wu4
U+9696.0	fEnglish	entrenchment; bank; low wall
U+9697.0	fUTF8	隗
U+9697.0	fCNS11643	2-485A
U+9697.0	fBig5	E0D4
U+9697.0	fGB	0-5A73
U+9697.0	fR/S	 170.10
U+9697.0	fTotalStrokes	 13
U+9697.0	fCangjie	NLHUI
U+9697.0	fFourCorner	76213
U+9697.0	fHakka	 ngui5
U+9697.0	fCantonese	ngai5; ai5 (also); ngaai5 (also); kwai4 (also); aai5 (also); wai5 (also); ngai4; wai4; ang5
U+9697.0	fMandarin	wei3; kui2; gui1; wei2; yuan2; yun3
U+9697.0	fEnglish	[1] high; lofty [2] a Chinese familly name
U+9698.0	fUTF8	隘
U+9698.0	fCNS11643	1-6751
U+9698.0	fBig5	B969
U+9698.0	fGB	0-302F
U+9698.0	fR/S	 170.10
U+9698.0	fTotalStrokes	 13
U+9698.0	fCangjie	NLTCT
U+9698.0	fFourCorner	78217
U+9698.0	fHakka	 ai5; ai3; rit7; jit7
U+9698.0	fCantonese	aai3; ai3 (also); ngaai3 (also); ngai3 (also); ak1; ngak1
U+9698.0	fMandarin	ai4; e4; huang2
U+9698.0	fEnglish	[1] narrow [2] urgen [3] strategic pass; strategic point [4] destitute
U+9699.0	fUTF8	隙
U+9699.0	fCNS11643	1-6B61
U+9699.0	fBig5	BBD8
U+9699.0	fGB	0-4F36
U+9699.0	fR/S	 170.10
U+9699.0	fTotalStrokes	 13
U+9699.0	fCangjie	NLFHF
U+9699.0	fFourCorner	79296
U+9699.0	fHakka	  hit7
U+9699.0	fCantonese	gwik1; kwik1 (also)
U+9699.0	fMandarin	xi4
U+9699.0	fEnglish	[1] crack; split; fissure; crevice [2] grudge; dislike; dispute; quarrel; complaint [3] opportunity; loophole [4] important passaggeway; important corridor
U+969A.0	fUTF8	隚
U+969A.0	fCNS11643	4-4824
U+969A.0	fGB	E-4434
U+969A.0	fR/S	 170.11
U+969A.0	fTotalStrokes	 14
U+969A.0	fCangjie	NLFBG
U+969A.0	fFourCorner	79210
U+969A.0	fCantonese	tong4
U+969A.0	fMandarin	tang2
U+969B.0	fUTF8	際
U+969B.0	fCNS11643	1-6B63
U+969B.0	fBig5	BBDA
U+969B.0	fGB	1-3C4A
U+969B.0	fR/S	 170.11
U+969B.0	fTotalStrokes	 14
U+969B.0	fCangjie	NLBOF
U+969B.0	fFourCorner	77291
U+969B.0	fHakka	  zi5
U+969B.0	fCantonese	zai3
U+969B.0	fMandarin	ji4
U+969B.0	fEnglish	[1] border; boundary; edge [2] (to occur) at the time or on the occasion of [3] by the side of; beside [4] in the middle; between; among [5] opportunity; fortune; luck
U+969C.0	fUTF8	障
U+969C.0	fCNS11643	1-6B62
U+969C.0	fBig5	BBD9
U+969C.0	fGB	0-554F
U+969C.0	fR/S	 170.11
U+969C.0	fTotalStrokes	 14
U+969C.0	fCangjie	NLYTJ
U+969C.0	fFourCorner	70246
U+969C.0	fHakka	 zong5;  zhong5; zhong1
U+969C.0	fCantonese	zoeng3
U+969C.0	fMandarin	zhang4; zhang1
U+969C.0	fEnglish	[1] separate; screen; barrier; screen [2] dike; embankment [3] defend; guard; shield [4] guarantee [5] hinder; obstruct
U+969D.0	fUTF8	隝
U+969D.0	fCNS11643	3-4A45
U+969D.0	fGB	5-746E
U+969D.0	fR/S	 170.11
U+969D.0	fTotalStrokes	 14
U+969D.0	fCangjie	NLHAF
U+969D.0	fFourCorner	77220
U+969D.0	fCantonese	dou2; wu2
U+969D.0	fMandarin	dao3
U+969D.0	fEnglish	dock, entrenchment, low wall
U+969E.0	fUTF8	隞
U+969E.0	fCNS11643	2-4F23
U+969E.0	fBig5	E4BB
U+969E.0	fGB	3-7546
U+969E.0	fR/S	 170.11
U+969E.0	fTotalStrokes	 14
U+969E.0	fCangjie	NLGSK
U+969E.0	fFourCorner	78240
U+969E.0	fCantonese	ngou4; ou4 (also)
U+969E.0	fMandarin	ao2
U+969F.0	fUTF8	隟
U+969F.0	fCNS11643	3-4A46
U+969F.0	fGB	5-752B
U+969F.0	fR/S	 170.11
U+969F.0	fTotalStrokes	 14
U+969F.0	fCangjie	NLVVD
U+969F.0	fFourCorner	72290
U+969F.0	fCantonese	gwik1; kwik1 (also)
U+969F.0	fMandarin	xi4
U+96A0.0	fUTF8	隠
U+96A0.0	fCNS11643	3-4A44
U+96A0.0	fGB	E-4435
U+96A0.0	fR/S	 170.11
U+96A0.0	fTotalStrokes	 14
U+96A0.0	fCangjie	NLBSP
U+96A0.0	fFourCorner	72230
U+96A0.0	fCantonese	jan2
U+96A0.0	fMandarin	yin3; yin4
U+96A0.0	fEnglish	to hide, conceal, cover; to disappear, lurk; an open secret
U+96A1.0	fUTF8	隡
U+96A1.0	fCNS11643	2-4F24
U+96A1.0	fBig5	E4BC
U+96A1.0	fGB	E-4436
U+96A1.0	fR/S	 170.11
U+96A1.0	fTotalStrokes	 14
U+96A1.0	fCangjie	NLYHM
U+96A1.0	fFourCorner	70215
U+96A1.0	fCantonese	saat3
U+96A1.0	fMandarin	sa4
U+96A2.0	fUTF8	隢
U+96A2.0	fCNS11643	2-5554
U+96A2.0	fBig5	E8AB
U+96A2.0	fGB	E-4437
U+96A2.0	fR/S	 170.12
U+96A2.0	fTotalStrokes	 15
U+96A2.0	fCangjie	NLGGU
U+96A2.0	fFourCorner	74211
U+96A2.0	fCantonese	jiu5
U+96A2.0	fMandarin	rao4
U+96A3.0	fUTF8	隣
U+96A3.0	fCNS11643	3-4F5A
U+96A3.0	fGB	E-4438
U+96A3.0	fR/S	 170.12
U+96A3.0	fTotalStrokes	 15
U+96A3.0	fCangjie	NLFDQ
U+96A3.0	fFourCorner	79250
U+96A3.0	fHakka	 lin2
U+96A3.0	fCantonese	leon4
U+96A3.0	fMandarin	lin2
U+96A3.0	fEnglish	[1] neighbor [2] neighboring; nearby
U+96A4.0	fUTF8	隤
U+96A4.0	fCNS11643	2-5553
U+96A4.0	fBig5	E8AA
U+96A4.0	fGB	3-7544
U+96A4.0	fR/S	 170.12
U+96A4.0	fTotalStrokes	 15
U+96A4.0	fCangjie	NLLMC
U+96A4.0	fFourCorner	75286
U+96A4.0	fHakka	 tui2; kui2; tui3; k'wui2
U+96A4.0	fCantonese	teoi4
U+96A4.0	fMandarin	tui2
U+96A4.0	fEnglish	[1] fall in ruins; collapse [2] tired and jaded (horses)
U+96A5.0	fUTF8	隥
U+96A5.0	fCNS11643	3-4F5B
U+96A5.0	fGB	5-752C
U+96A5.0	fR/S	  170.12
U+96A5.0	fTotalStrokes	 15
U+96A5.0	fCangjie	NLNOT
U+96A5.0	fFourCorner	72210
U+96A5.0	fCantonese	dang3
U+96A5.0	fMandarin	deng4
U+96A6.0	fUTF8	隦
U+96A6.0	fCNS11643	4-5452
U+96A6.0	fGB	E-4439
U+96A6.0	fR/S	 170.13
U+96A6.0	fTotalStrokes	 16
U+96A6.0	fCangjie	NLSRJ
U+96A6.0	fFourCorner	70240
U+96A6.0	fHakka	 bi1
U+96A6.0	fCantonese	pei4
U+96A6.0	fMandarin	pi3; pi2
U+96A7.0	fUTF8	隧
U+96A7.0	fCNS11643	1-7270
U+96A7.0	fBig5	C047
U+96A7.0	fGB	0-4B6D
U+96A7.0	fR/S	 170.13
U+96A7.0	fTotalStrokes	 16
U+96A7.0	fCangjie	NLYTO
U+96A7.0	fFourCorner	78233
U+96A7.0	fHakka	 sui5; sui6
U+96A7.0	fCantonese	seoi6; zeoi6
U+96A7.0	fMandarin	sui4; zhui4; dui4; xian3
U+96A7.0	fEnglish	[1] underground passagge; tunnel [2] go round and round [3] tower on the wall to watch signal fires
U+96A8.0	fUTF8	隨
U+96A8.0	fCNS11643	1-7271
U+96A8.0	fBig5	C048
U+96A8.0	fGB	1-4B66
U+96A8.0	fR/S	 170.13
U+96A8.0	fTotalStrokes	 16
U+96A8.0	fCangjie	NLYKB
U+96A8.0	fFourCorner	74232
U+96A8.0	fHakka	 cui2;  sui2; sui3; to3
U+96A8.0	fCantonese	ceoi4
U+96A8.0	fMandarin	sui2
U+96A8.0	fEnglish	[1] follow; trace; come after [2] listen to; submit to; comply with [3] let (it go, it be, etc.) [4] accompany [5] resemble; look alike [6] a Chinese family name
U+96A9.0	fUTF8	隩
U+96A9.0	fCNS11643	2-5B5B
U+96A9.0	fBig5	EC4F
U+96A9.0	fGB	3-7549
U+96A9.0	fR/S	 170.13
U+96A9.0	fTotalStrokes	 16
U+96A9.0	fCangjie	NLHBK
U+96A9.0	fFourCorner	77234
U+96A9.0	fHakka	 au5;  juk7; ruk7
U+96A9.0	fCantonese	juk1; ou3 (also); ngou3 (also)
U+96A9.0	fMandarin	yu4; ao4
U+96A9.0	fEnglish	[1] bend of a stream; cove [2] warm [3] inhabitable land
U+96AA.0	fUTF8	險
U+96AA.0	fCNS11643	1-7272
U+96AA.0	fBig5	C049
U+96AA.0	fGB	1-4F55
U+96AA.0	fR/S	 170.13
U+96AA.0	fTotalStrokes	 16
U+96AA.0	fCangjie	NLOMO
U+96AA.0	fFourCorner	78286
U+96AA.0	fHakka	  hiam3
U+96AA.0	fCantonese	him2
U+96AA.0	fMandarin	xian3; yan2; shan3; wei3; jian3
U+96AA.0	fEnglish	[1] dangerous; danger [2] obstructed; difficult [3] strategic pass; strategic point [4] cunning; mean; crafy; sinister [5] nearly; almost; within an ace of
U+96AB.0	fUTF8	隫
U+96AB.0	fCNS11643	3-5348
U+96AB.0	fGB	5-747E
U+96AB.0	fR/S	 170.13; 170.12
U+96AB.0	fTotalStrokes	 16; 15
U+96AB.0	fCangjie	NLJTC
U+96AB.0	fFourCorner	74280
U+96AB.0	fCantonese	fan4
U+96AB.0	fMandarin	fen1; fen2
U+96AC.0	fUTF8	隬
U+96AC.0	fCNS11643	2-6043
U+96AC.0	fBig5	EEF6
U+96AC.0	fGB	E-443A
U+96AC.0	fR/S	 170.14
U+96AC.0	fTotalStrokes	 17
U+96AC.0	fCangjie	NLMFB
U+96AC.0	fFourCorner	71227
U+96AC.0	fCantonese	nei5; lei5 (also)
U+96AC.0	fMandarin	ni3
U+96AD.0	fUTF8	隭
U+96AD.0	fCNS11643	3-5670
U+96AD.0	fGB	E-443B
U+96AD.0	fR/S	 170.14
U+96AD.0	fTotalStrokes	 17
U+96AD.0	fCangjie	NLMBB
U+96AD.0	fFourCorner	71220
U+96AD.0	fCantonese	ji4
U+96AD.0	fMandarin	er2
U+96AE.0	fUTF8	隮
U+96AE.0	fCNS11643	2-6041
U+96AE.0	fBig5	EEF4
U+96AE.0	fGB	3-7539
U+96AE.0	fR/S	 170.14
U+96AE.0	fTotalStrokes	 17
U+96AE.0	fCangjie	NLYX
U+96AE.0	fFourCorner	70223
U+96AE.0	fHakka	  zi1; zi5
U+96AE.0	fCantonese	zai1
U+96AE.0	fMandarin	ji1; ji4; zi1
U+96AE.0	fEnglish	[1] rise up; ascend [2] rainbow [3] fall; topple
U+96AF.0	fUTF8	隯
U+96AF.0	fCNS11643	3-566F
U+96AF.0	fGB	E-443C
U+96AF.0	fR/S	 170.14
U+96AF.0	fTotalStrokes	 17
U+96AF.0	fCangjie	NLGNI
U+96AF.0	fFourCorner	74240
U+96AF.0	fCantonese	dou2
U+96AF.0	fMandarin	dao3
U+96B0.0	fUTF8	隰
U+96B0.0	fCNS11643	2-6042
U+96B0.0	fBig5	EEF5
U+96B0.0	fGB	0-5A74
U+96B0.0	fR/S	 170.14
U+96B0.0	fTotalStrokes	 17
U+96B0.0	fCangjie	NLAVF
U+96B0.0	fFourCorner	76233
U+96B0.0	fHakka	 ship7; sip7
U+96B0.0	fCantonese	zaap6
U+96B0.0	fMandarin	xi2; xie4; zhi4
U+96B0.0	fEnglish	[1] low, marshy land [2] newly opened farmland [3] a Chinese family name
U+96B1.0	fUTF8	隱
U+96B1.0	fCNS11643	1-7621
U+96B1.0	fBig5	C1F4
U+96B1.0	fGB	1-527E
U+96B1.0	fR/S	 170.14
U+96B1.0	fTotalStrokes	 17
U+96B1.0	fCangjie	NLBMP
U+96B1.0	fFourCorner	72237
U+96B1.0	fHakka	  jun3; jin3; wun3; run3
U+96B1.0	fCantonese	jan2; jan3
U+96B1.0	fMandarin	yin3; yin4; sui4; zhui4
U+96B1.0	fEnglish	[1] hidden; concealed; mysterious; secret [2] dark; obscure; not evident; not obvious [3] retire; reject public life; live like a hermit [4] painful; grievous [5] riddle [6] destitue; poor [7] examine and study [8] low wall [9] a Chinese family name
U+96B2.0	fUTF8	隲
U+96B2.0	fCNS11643	3-5671
U+96B2.0	fGB	E-443D
U+96B2.0	fR/S	 170.14
U+96B2.0	fTotalStrokes	 17
U+96B2.0	fCangjie	NLFHF
U+96B2.0	fFourCorner	79220
U+96B2.0	fHakka	 zhit7; tsii8t7
U+96B2.0	fCantonese	zat1
U+96B2.0	fMandarin	e2; zhi4; yin3; yin4
U+96B2.0	fEnglish	 stallion promote
U+96B3.0	fUTF8	隳
U+96B3.0	fCNS11643	2-643B
U+96B3.0	fBig5	F16F
U+96B3.0	fGB	0-6344
U+96B3.0	fR/S	 170.15
U+96B3.0	fTotalStrokes	 18
U+96B3.0	fCangjie	NBOP
U+96B3.0	fFourCorner	74338
U+96B3.0	fHakka	 tui3; fi1; fui1; to3
U+96B3.0	fCantonese	fai1; fui1 (also)
U+96B3.0	fMandarin	hui1
U+96B3.0	fEnglish	 destroy; ruin; break
U+96B4.0	fUTF8	隴
U+96B4.0	fCNS11643	1-7944
U+96B4.0	fBig5	C3F7
U+96B4.0	fGB	1-4224
U+96B4.0	fR/S	 170.16
U+96B4.0	fTotalStrokes	 19
U+96B4.0	fCangjie	NLYBP
U+96B4.0	fFourCorner	71211
U+96B4.0	fHakka	  lung3;  liung1; lung1
U+96B4.0	fCantonese	lung5
U+96B4.0	fMandarin	long3; xi2
U+96B4.0	fEnglish	[1] another name of Kansu province [2] mound [3] prosperous [4] a Chinese family name
U+96B5.0	fUTF8	隵
U+96B5.0	fCNS11643	3-5D49
U+96B5.0	fGB	E-443E
U+96B5.0	fR/S	 170.17
U+96B5.0	fTotalStrokes	 20
U+96B5.0	fCangjie	NLYTI
U+96B5.0	fFourCorner	73250
U+96B5.0	fCantonese	hei1
U+96B5.0	fMandarin	xi1
U+96B6.0	fUTF8	隶
U+96B6.0	fCNS11643	3-2A7D
U+96B6.0	fGB	0-4125
U+96B6.0	fR/S	 171.0
U+96B6.0	fTotalStrokes	 8
U+96B6.0	fCangjie	LE
U+96B6.0	fFourCorner	50130
U+96B6.0	fHakka	  ti5; tai6; li5
U+96B6.0	fCantonese	dai6
U+96B6.0	fMandarin	dai4; li4; lie4; yi4; di4
U+96B6.0	fEnglish	simplified form of &#x96B8;
U+96B7.0	fUTF8	隷
U+96B7.0	fCNS11643	3-5349
U+96B7.0	fGB	E-443F
U+96B7.0	fR/S	 171.8
U+96B7.0	fTotalStrokes	 16
U+96B7.0	fCangjie	GFLE
U+96B7.0	fFourCorner	45930
U+96B7.0	fCantonese	dai6; lai6
U+96B7.0	fMandarin	li4
U+96B7.0	fEnglish	 be subservient to servant
U+96B8.0	fUTF8	隸
U+96B8.0	fCNS11643	1-7622
U+96B8.0	fBig5	C1F5
U+96B8.0	fGB	1-4125
U+96B8.0	fR/S	 171.9
U+96B8.0	fTotalStrokes	 17
U+96B8.0	fCangjie	DFLE
U+96B8.0	fFourCorner	45932
U+96B8.0	fHakka	 li5; li6; ti5
U+96B8.0	fCantonese	dai6; lai6; doi6
U+96B8.0	fMandarin	li4; lie4
U+96B8.0	fEnglish	[1] be subordinatee of; be subservient to; inferior; belong to; attach to [2] servant; slave; underling [3] a type of Chinese calligraphy [4] practise; learn [5] a Chinese family name
U+96B9.0	fUTF8	隹
U+96B9.0	fCNS11643	1-4F63
U+96B9.0	fBig5	AB41
U+96B9.0	fGB	0-763F
U+96B9.0	fR/S	 172.0
U+96B9.0	fTotalStrokes	 8
U+96B9.0	fCangjie	OG
U+96B9.0	fFourCorner	20214
U+96B9.0	fHakka	 zui1; chui1; zhui1
U+96B9.0	fCantonese	zeoi1
U+96B9.0	fMandarin	zhui1; cui1; li4; wei2; zhui3; zui1; zui3
U+96B9.0	fEnglish	[1] a general name of short-tailed birds [2] Kangxi radical 172
U+96BA.0	fUTF8	隺
U+96BA.0	fCNS11643	4-3041
U+96BA.0	fGB	3-3268
U+96BA.0	fR/S	 172.2
U+96BA.0	fTotalStrokes	 10
U+96BA.0	fCangjie	OBG
U+96BA.0	fFourCorner	40210
U+96BA.0	fHakka	 kok7
U+96BA.0	fCantonese	hok6
U+96BA.0	fMandarin	hu2; he4; que4
U+96BA.0	fEnglish	a bird flying high ambition
U+96BB.0	fUTF8	隻
U+96BB.0	fCNS11643	1-5869
U+96BB.0	fBig5	B0A6
U+96BB.0	fGB	1-7922
U+96BB.0	fR/S	 172.2
U+96BB.0	fTotalStrokes	 10
U+96BB.0	fCangjie	OGE
U+96BB.0	fFourCorner	20407
U+96BB.0	fHakka	  zak7;  chak7; zhak7
U+96BB.0	fCantonese	zek3
U+96BB.0	fMandarin	zhi1; zhi3
U+96BB.0	fEnglish	[1] numerical classifier for hen, pigeon, bird, ox, goat, hand, foot, etc. [2] single; one of pair; alone [3] odd (number) [4] a general numerical classifier equivalent to Mandarin "ge4"
U+96BC.0	fUTF8	隼
U+96BC.0	fCNS11643	2-334A
U+96BC.0	fBig5	D447
U+96BC.0	fGB	0-7640
U+96BC.0	fR/S	 172.2
U+96BC.0	fTotalStrokes	 10
U+96BC.0	fCangjie	OGJ
U+96BC.0	fFourCorner	20401
U+96BC.0	fHakka	 sun3; zun3; zhun3
U+96BC.0	fCantonese	zeon2; seon2
U+96BC.0	fMandarin	sun3; zhun3
U+96BC.0	fEnglish	[1] aquiline nose [2] hawk; falcon
U+96BD.0	fUTF8	隽
U+96BD.0	fCNS11643	3-3439
U+96BD.0	fGB	0-7641
U+96BD.0	fR/S	 172.2
U+96BD.0	fTotalStrokes	 10
U+96BD.0	fCangjie	OGNHS
U+96BD.0	fFourCorner	20220
U+96BD.0	fHakka	 zun5
U+96BD.0	fCantonese	zeon3; syun5
U+96BD.0	fMandarin	jun4; juan4
U+96BD.0	fEnglish	superior; outstanding; talented
U+96BE.0	fUTF8	难
U+96BE.0	fCNS11643	F-326F
U+96BE.0	fGB	0-4451
U+96BE.0	fR/S	 172.2; 29.8
U+96BE.0	fTotalStrokes	 10
U+96BE.0	fCangjie	EOG
U+96BE.0	fHakka	 nan2; nan5
U+96BE.0	fCantonese	naan4; naan6; laan4 (also); laan6 (also); no4; lo4 (also)
U+96BE.0	fMandarin	nan2; nuo2; nuo3
U+96BE.0	fEnglish	simplified form of &#x96E3;
U+96BE.1	fUTF8	难
U+96BE.1	fCNS11643	F-326F
U+96BE.1	fGB	0-4451
U+96BE.1	fR/S	 172.2; 29.8
U+96BE.1	fTotalStrokes	 10
U+96BE.1	fCangjie	EOG
U+96BE.1	fMandarin	nan4
U+96BE.2	fUTF8	难
U+96BE.2	fCNS11643	F-326F
U+96BE.2	fGB	0-4451
U+96BE.2	fR/S	 172.2; 29.8
U+96BE.2	fTotalStrokes	 10
U+96BE.2	fCangjie	EOG
U+96BE.2	fMandarin	nan5
U+96BF.0	fUTF8	隿
U+96BF.0	fCNS11643	2-3A30
U+96BF.0	fBig5	D84C
U+96BF.0	fGB	E-4440
U+96BF.0	fR/S	 172.3
U+96BF.0	fTotalStrokes	 11
U+96BF.0	fCangjie	IPOG
U+96BF.0	fFourCorner	40014
U+96BF.0	fCantonese	jik6
U+96BF.0	fMandarin	yi4
U+96C0.0	fUTF8	雀
U+96C0.0	fCNS11643	1-5D7A
U+96C0.0	fBig5	B3B6
U+96C0.0	fGB	0-4838
U+96C0.0	fR/S	 172.3
U+96C0.0	fTotalStrokes	 11
U+96C0.0	fCangjie	FOG
U+96C0.0	fFourCorner	90214
U+96C0.0	fHakka	  ziok7
U+96C0.0	fCantonese	zoek3; zoek2 (also)
U+96C0.0	fMandarin	que4; qiao3; qiao1; jue2
U+96C0.0	fEnglish	[1] a general name of small birds, as sparrows, chickadees, etc. [2] freckled
U+96C1.0	fUTF8	雁
U+96C1.0	fCNS11643	1-6272
U+96C1.0	fBig5	B6AD
U+96C1.0	fGB	0-5163
U+96C1.0	fR/S	 172.4
U+96C1.0	fTotalStrokes	 12
U+96C1.0	fCangjie	MOOG
U+96C1.0	fFourCorner	71214
U+96C1.0	fHakka	  ngan5;  ngien5; ngan3; ngian5
U+96C1.0	fCantonese	ngaan6; aan6 (also); ngaan2 (also); aan2 (also)
U+96C1.0	fMandarin	yan4
U+96C1.0	fEnglish	the wild goose
U+96C2.0	fUTF8	雂
U+96C2.0	fCNS11643	2-4149
U+96C2.0	fBig5	DCA4
U+96C2.0	fGB	5-7451
U+96C2.0	fR/S	 172.4
U+96C2.0	fTotalStrokes	 12
U+96C2.0	fCangjie	ONOG
U+96C2.0	fFourCorner	80214
U+96C2.0	fCantonese	kam4
U+96C2.0	fMandarin	qin2; qian2; an2; gan1; han2; kan1
U+96C3.0	fUTF8	雃
U+96C3.0	fCNS11643	2-414B
U+96C3.0	fBig5	DCA6
U+96C3.0	fGB	5-744F
U+96C3.0	fR/S	 172.4
U+96C3.0	fTotalStrokes	 12
U+96C3.0	fCangjie	MTOG
U+96C3.0	fFourCorner	10414
U+96C3.0	fCantonese	hin1
U+96C3.0	fMandarin	ya3; an2; e4; keng1; qian1; yan2
U+96C4.0	fUTF8	雄
U+96C4.0	fCNS11643	1-6274
U+96C4.0	fBig5	B6AF
U+96C4.0	fGB	0-505B
U+96C4.0	fR/S	 172.4
U+96C4.0	fTotalStrokes	 12
U+96C4.0	fCangjie	KIOG
U+96C4.0	fFourCorner	40014
U+96C4.0	fHakka	  hiung2; hiung3; hiak7
U+96C4.0	fCantonese	hung4
U+96C4.0	fMandarin	xiong2
U+96C4.0	fEnglish	[1] male; masculine; virile [2] a person or state having great power and influence [3] heroic; brave; strong; martial; ambitious [4] win; triumph; victory [5] scold others with insulting words [6] a Chinese family name
U+96C5.0	fUTF8	雅
U+96C5.0	fCNS11643	1-6273
U+96C5.0	fBig5	B6AE
U+96C5.0	fGB	0-5145
U+96C5.0	fR/S	 172.4
U+96C5.0	fTotalStrokes	 12
U+96C5.0	fCangjie	MHOG
U+96C5.0	fFourCorner	70214
U+96C5.0	fHakka	  nga3; nga1; {nga1}
U+96C5.0	fCantonese	ngaa5; aa5 (also); aa1; ngaa1
U+96C5.0	fMandarin	ya3; ya1; ya2
U+96C5.0	fEnglish	[1] refined; polished; sophisticated; not common; not vulgar [2] elegant; graceful [3] usually; often; frequently; much [4] the name of an ancient musical instrument [5] friendship; acquaintance [6] wine vessel [7] your (polite) [8] a Chinese family name
U+96C6.0	fUTF8	集
U+96C6.0	fCNS11643	1-6275
U+96C6.0	fBig5	B6B0
U+96C6.0	fGB	0-3C2F
U+96C6.0	fR/S	 172.4
U+96C6.0	fTotalStrokes	 12
U+96C6.0	fCangjie	OGD
U+96C6.0	fFourCorner	20904
U+96C6.0	fHakka	  cip8; sip8; sit7; zip8
U+96C6.0	fCantonese	zaap6; zap6 (also); zaap2 (also)
U+96C6.0	fMandarin	ji2
U+96C6.0	fEnglish	[1] assemble; collect; gather together; concentrate [2] a collection of work by one or more authors; compile; edit [3] achievements [4] fair; periodical market
U+96C7.0	fUTF8	雇
U+96C7.0	fCNS11643	1-6276
U+96C7.0	fBig5	B6B1
U+96C7.0	fGB	0-394D
U+96C7.0	fR/S	 172.4
U+96C7.0	fTotalStrokes	 12
U+96C7.0	fCangjie	HSOG
U+96C7.0	fFourCorner	30214
U+96C7.0	fHakka	  gu5
U+96C7.0	fCantonese	gu3; gwu3 (also)
U+96C7.0	fMandarin	gu4; hu4
U+96C7.0	fEnglish	 employ; hire
U+96C8.0	fUTF8	雈
U+96C8.0	fCNS11643	2-414A
U+96C8.0	fBig5	DCA5
U+96C8.0	fGB	E-4441
U+96C8.0	fR/S	 172.4
U+96C8.0	fTotalStrokes	 12
U+96C8.0	fCangjie	TOG
U+96C8.0	fFourCorner	44214
U+96C8.0	fCantonese	wun4; wan4
U+96C8.0	fMandarin	huan2
U+96C9.0	fUTF8	雉
U+96C9.0	fCNS11643	1-6756
U+96C9.0	fBig5	B96E
U+96C9.0	fGB	0-6F74
U+96C9.0	fR/S	 172.5
U+96C9.0	fTotalStrokes	 13
U+96C9.0	fCangjie	OKOG
U+96C9.0	fFourCorner	80414
U+96C9.0	fHakka	 ci3;  chi1; chi3; cih1; cih6; ci1; cih2; chii1
U+96C9.0	fCantonese	zi6; zi5
U+96C9.0	fMandarin	zhi4; kai3; yi3; si4; juan4; jun4
U+96C9.0	fEnglish	[1] the pheasant [2] a unit of volume measure in ancient China (30' square by 10')
U+96CA.0	fUTF8	雊
U+96CA.0	fCNS11643	1-6757
U+96CA.0	fBig5	B96F
U+96CA.0	fGB	3-7526
U+96CA.0	fR/S	 172.5
U+96CA.0	fTotalStrokes	 13
U+96CA.0	fCangjie	PROG
U+96CA.0	fFourCorner	20614
U+96CA.0	fHakka	 kieu5; geu5
U+96CA.0	fCantonese	gau3
U+96CA.0	fMandarin	gou4; ju4
U+96CA.0	fEnglish	the crow of a male pheasant
U+96CB.0	fUTF8	雋
U+96CB.0	fCNS11643	1-6755
U+96CB.0	fBig5	B96D
U+96CB.0	fGB	E-4442
U+96CB.0	fR/S	 172.5
U+96CB.0	fTotalStrokes	 13
U+96CB.0	fCangjie	OGLMS
U+96CB.0	fFourCorner	20227
U+96CB.0	fHakka	 zun5; zun3
U+96CB.0	fCantonese	zeon3; syun5; cyun5
U+96CB.0	fMandarin	jun4; ju1; zui4
U+96CB.0	fEnglish	[1] good-looking [2] extraordinary; outstanding; talented
U+96CB.1	fUTF8	雋
U+96CB.1	fCNS11643	1-6755
U+96CB.1	fBig5	B96D
U+96CB.1	fGB	E-4442
U+96CB.1	fR/S	 172.5
U+96CB.1	fTotalStrokes	 13
U+96CB.1	fCangjie	OGLMS
U+96CB.1	fFourCorner	20227
U+96CB.1	fMandarin	juan4
U+96CB.1	fEnglish	[1] fat meat [2] meaningful [3] a Chinese family name
U+96CC.0	fUTF8	雌
U+96CC.0	fCNS11643	1-6B64
U+96CC.0	fBig5	BBDB
U+96CC.0	fGB	0-3446
U+96CC.0	fR/S	 172.5
U+96CC.0	fTotalStrokes	 13
U+96CC.0	fCangjie	YMPOG
U+96CC.0	fFourCorner	20114
U+96CC.0	fHakka	 zih1;  cih1; chi1; ci1; cu1; ze1
U+96CC.0	fCantonese	ci1
U+96CC.0	fMandarin	ci2; ci1; qi1
U+96CC.0	fEnglish	[1] female; feminine; gentle; soft (voice, etc.); woman-like [2] weak; retiring [3] the vanquished; defeated [4] scold [5] expose the teeth; show the teeth
U+96CD.0	fUTF8	雍
U+96CD.0	fCNS11643	1-6754
U+96CD.0	fBig5	B96C
U+96CD.0	fGB	0-533A
U+96CD.0	fR/S	 172.5
U+96CD.0	fTotalStrokes	 13
U+96CD.0	fCangjie	YVHG
U+96CD.0	fFourCorner	00710
U+96CD.0	fHakka	 jung1; rung1
U+96CD.0	fCantonese	jung1; jung3
U+96CD.0	fMandarin	yong1; yong3; yong4; zhi4
U+96CD.0	fEnglish	[1] harmony; union; harmonious; peaceful [2] the name of various places or states throughout the dynasties [3] block; obstruct [4] a Chinese family name
U+96CE.0	fUTF8	雎
U+96CE.0	fCNS11643	2-485B
U+96CE.0	fBig5	E0D5
U+96CE.0	fGB	0-7642
U+96CE.0	fR/S	 172.5
U+96CE.0	fTotalStrokes	 13
U+96CE.0	fCangjie	BMOG
U+96CE.0	fFourCorner	70114
U+96CE.0	fHakka	 zih1; zui1; zu1; zii1
U+96CE.0	fCantonese	zeoi1
U+96CE.0	fMandarin	ju1; cui1; qu1; zhui1
U+96CE.0	fEnglish	[1] the osprey; a kind of waterfowl which observes fidelity in sex life [2] hesitate; avoid; hold back
U+96CF.0	fUTF8	雏
U+96CF.0	fCNS11643	4-4157
U+96CF.0	fGB	0-337B
U+96CF.0	fR/S	 172.5
U+96CF.0	fTotalStrokes	 13
U+96CF.0	fCangjie	NSOG
U+96CF.0	fFourCorner	20110
U+96CF.0	fHakka	 cih1; ci1
U+96CF.0	fCantonese	co1; co4 (also)
U+96CF.0	fMandarin	chu2; ju4
U+96CF.0	fEnglish	simplified form of &#x96DB;
U+96D0.0	fUTF8	雐
U+96D0.0	fCNS11643	3-4A47
U+96D0.0	fGB	3-6145
U+96D0.0	fR/S	 141.8; 172.6; 172.8
U+96D0.0	fTotalStrokes	 14
U+96D0.0	fCangjie	YPOG
U+96D0.0	fFourCorner	21210
U+96D0.0	fCantonese	fu1
U+96D0.0	fMandarin	hu1
U+96D1.0	fUTF8	雑
U+96D1.0	fCNS11643	3-4A49
U+96D1.0	fGB	E-4443
U+96D1.0	fR/S	 172.6
U+96D1.0	fTotalStrokes	 14
U+96D1.0	fCangjie	KDOG
U+96D1.0	fFourCorner	40910
U+96D1.0	fCantonese	zaap6
U+96D1.0	fMandarin	za2
U+96D1.0	fEnglish	 mixed; blended mix; mingle
U+96D2.0	fUTF8	雒
U+96D2.0	fCNS11643	1-6B65
U+96D2.0	fBig5	BBDC
U+96D2.0	fGB	0-7643
U+96D2.0	fR/S	 172.6
U+96D2.0	fTotalStrokes	 14
U+96D2.0	fCangjie	HROG
U+96D2.0	fFourCorner	20614
U+96D2.0	fHakka	 lok8; lok7
U+96D2.0	fCantonese	lok3; lok6
U+96D2.0	fMandarin	luo4
U+96D2.0	fEnglish	[1] a black horse with white mane [2] brand [3] the name of a river [4] owl
U+96D3.0	fUTF8	雓
U+96D3.0	fCNS11643	2-5555
U+96D3.0	fBig5	E8AC
U+96D3.0	fGB	E-4444
U+96D3.0	fR/S	 172.7
U+96D3.0	fTotalStrokes	 15
U+96D3.0	fCangjie	ODOG
U+96D3.0	fFourCorner	80914
U+96D3.0	fHakka	 ji2; ri2
U+96D3.0	fCantonese	jyu4; ji4
U+96D3.0	fMandarin	yu2
U+96D3.0	fEnglish	a chicken just out of the shell
U+96D4.0	fUTF8	雔
U+96D4.0	fCNS11643	2-5B5C
U+96D4.0	fBig5	EC50
U+96D4.0	fGB	5-7457
U+96D4.0	fR/S	 172.8
U+96D4.0	fTotalStrokes	 16
U+96D4.0	fCangjie	OGOG
U+96D4.0	fFourCorner	20214
U+96D4.0	fHakka	 shiu2; chiu2; su2; cu2
U+96D4.0	fCantonese	cau4; sau4
U+96D4.0	fMandarin	chou2
U+96D4.0	fEnglish	[1] a pair of birds [2] a kind of silk-worm
U+96D5.0	fUTF8	雕
U+96D5.0	fCNS11643	1-7273
U+96D5.0	fBig5	C04A
U+96D5.0	fGB	0-3571
U+96D5.0	fR/S	 172.8
U+96D5.0	fTotalStrokes	 16
U+96D5.0	fCangjie	BROG
U+96D5.0	fFourCorner	70214
U+96D5.0	fHakka	  diau1; tiau1
U+96D5.0	fCantonese	diu1
U+96D5.0	fMandarin	diao1; huo4
U+96D5.0	fEnglish	[1] engrave; inlay; carve; cut (sculpture) [2] the eagle; the hawk [3] exhaust; weaken
U+96D6.0	fUTF8	雖
U+96D6.0	fCNS11643	1-7623
U+96D6.0	fBig5	C1F6
U+96D6.0	fGB	1-4B64
U+96D6.0	fR/S	 172.9
U+96D6.0	fTotalStrokes	 17
U+96D6.0	fCangjie	RIOG
U+96D6.0	fFourCorner	60114
U+96D6.0	fHakka	  sui1
U+96D6.0	fCantonese	seoi1
U+96D6.0	fMandarin	sui1; sui2
U+96D6.0	fEnglish	[1] although; even if; even though; supposing [2] push away; dismiss [3] only [4] a lizard-like reptile
U+96D7.0	fUTF8	雗
U+96D7.0	fCNS11643	2-643C
U+96D7.0	fBig5	F170
U+96D7.0	fGB	5-733F
U+96D7.0	fR/S	 172.10
U+96D7.0	fTotalStrokes	 18
U+96D7.0	fCangjie	JJOOG
U+96D7.0	fFourCorner	58014
U+96D7.0	fHakka	 hon5; hon6
U+96D7.0	fCantonese	hon6; hon5
U+96D7.0	fMandarin	han4; han2
U+96D7.0	fEnglish	a white pheasant
U+96D8.0	fUTF8	雘
U+96D8.0	fCNS11643	2-6440
U+96D8.0	fBig5	F174
U+96D8.0	fGB	3-6B3D
U+96D8.0	fR/S	 172.10
U+96D8.0	fTotalStrokes	 18
U+96D8.0	fCangjie	BYTOE
U+96D8.0	fFourCorner	74447
U+96D8.0	fHakka	 wok7
U+96D8.0	fCantonese	wok3
U+96D8.0	fMandarin	huo4; hu4; wo4
U+96D8.0	fEnglish	 dye made from red soil ochre
U+96D9.0	fUTF8	雙
U+96D9.0	fCNS11643	1-7764
U+96D9.0	fBig5	C2F9
U+96D9.0	fGB	1-4B2B
U+96D9.0	fR/S	 172.10
U+96D9.0	fTotalStrokes	 18
U+96D9.0	fCangjie	OGE
U+96D9.0	fFourCorner	20407
U+96D9.0	fHakka	  sung1
U+96D9.0	fCantonese	soeng1; sung1 (also)
U+96D9.0	fMandarin	shuang1; shuan1; shuang4
U+96D9.0	fEnglish	[1] set of two; pair; brace; couple; both; persons or things that come in pairs [2] two; both; even (as distinct from odd)
U+96DA.0	fUTF8	雚
U+96DA.0	fCNS11643	2-643D
U+96DA.0	fBig5	F171
U+96DA.0	fGB	5-6A48
U+96DA.0	fR/S	 172.10
U+96DA.0	fTotalStrokes	 18
U+96DA.0	fCangjie	TRRG
U+96DA.0	fFourCorner	44214
U+96DA.0	fHakka	 gon5; gwon5; kwan5
U+96DA.0	fCantonese	gun3; gwun3 (also); fun2; wun4
U+96DA.0	fMandarin	guan4; guan1; huan2
U+96DA.0	fEnglish	 heron small cup
U+96DB.0	fUTF8	雛
U+96DB.0	fCNS11643	1-7765
U+96DB.0	fBig5	C2FA
U+96DB.0	fGB	1-337B
U+96DB.0	fR/S	 172.10
U+96DB.0	fTotalStrokes	 18
U+96DB.0	fCangjie	PUOG
U+96DB.0	fFourCorner	20414
U+96DB.0	fHakka	 cu1;  ci1; cu2; ze1; cih1
U+96DB.0	fCantonese	co1; co4 (also)
U+96DB.0	fMandarin	chu2; ju4; ju2
U+96DB.0	fEnglish	[1] chick [2] fledging; very young bird [3] infant; small kid; toddler
U+96DC.0	fUTF8	雜
U+96DC.0	fCNS11643	1-7763
U+96DC.0	fBig5	C2F8
U+96DC.0	fGB	1-5453
U+96DC.0	fR/S	 172.10
U+96DC.0	fTotalStrokes	 18
U+96DC.0	fCangjie	YDOG
U+96DC.0	fFourCorner	00914
U+96DC.0	fHakka	  cap8
U+96DC.0	fCantonese	zaap6; zaap2 (also)
U+96DC.0	fMandarin	za2
U+96DC.0	fEnglish	[1] mixed; blended; mix; blend; mingle [2] miscellaneous [3] motley; medley [4] petty and numerous
U+96DD.0	fUTF8	雝
U+96DD.0	fCNS11643	2-6441
U+96DD.0	fBig5	F175
U+96DD.0	fGB	E-4445
U+96DD.0	fR/S	 172.10
U+96DD.0	fTotalStrokes	 18
U+96DD.0	fCangjie	VUOG
U+96DD.0	fFourCorner	20714
U+96DD.0	fHakka	  jung1; rung1
U+96DD.0	fCantonese	jung1
U+96DD.0	fMandarin	yong1; yong3
U+96DD.0	fEnglish	peaceful; harmonious
U+96DE.0	fUTF8	雞
U+96DE.0	fCNS11643	1-7766
U+96DE.0	fBig5	C2FB
U+96DE.0	fGB	E-4446
U+96DE.0	fR/S	 172.10
U+96DE.0	fTotalStrokes	 18
U+96DE.0	fCangjie	BKOG
U+96DE.0	fFourCorner	20414
U+96DE.0	fHakka	  gai1; kie1; ke1; ge1
U+96DE.0	fCantonese	gai1
U+96DE.0	fMandarin	ji1
U+96DE.0	fEnglish	 chicken; hen; cock; domestic fowl
U+96DF.0	fUTF8	雟
U+96DF.0	fCNS11643	2-643F
U+96DF.0	fBig5	F173
U+96DF.0	fGB	3-3970
U+96DF.0	fR/S	 172.10
U+96DF.0	fTotalStrokes	 18
U+96DF.0	fCangjie	UOGB
U+96DF.0	fFourCorner	22227
U+96DF.0	fHakka	  hi1; sui1
U+96DF.0	fCantonese	kwai1; sai1; seoi2; seoi5; kwai4
U+96DF.0	fMandarin	xi1; sui3; sui2; gui1; juan4
U+96DF.0	fEnglish	an old town in todays' Szechwan provice
U+96E0.0	fUTF8	雠
U+96E0.0	fGB	0-7645
U+96E0.0	fR/S	 172.10
U+96E0.0	fTotalStrokes	 18
U+96E0.0	fCangjie	OGIVG
U+96E0.0	fHakka	 chiu2
U+96E0.0	fCantonese	cau4; sau4
U+96E0.0	fMandarin	chou2; shou4
U+96E0.0	fEnglish	simplified form of &#x8B8E;
U+96E1.0	fUTF8	雡
U+96E1.0	fCNS11643	2-6766
U+96E1.0	fBig5	F379
U+96E1.0	fGB	5-745D
U+96E1.0	fR/S	 172.11
U+96E1.0	fTotalStrokes	 19
U+96E1.0	fCangjie	SHOG
U+96E1.0	fFourCorner	10214
U+96E1.0	fCantonese	lau6
U+96E1.0	fMandarin	liu4
U+96E2.0	fUTF8	離
U+96E2.0	fCNS11643	1-7762
U+96E2.0	fBig5	C2F7
U+96E2.0	fGB	1-406B
U+96E2.0	fR/S	 172.11
U+96E2.0	fTotalStrokes	 19
U+96E2.0	fCangjie	YBOG
U+96E2.0	fFourCorner	00414
U+96E2.0	fHakka	  li2; li3; li6; li5
U+96E2.0	fCantonese	lei4; lei6
U+96E2.0	fMandarin	li2; li3; chi1; li4; gu3
U+96E2.0	fEnglish	[1] leave; depart; go away; separate; separation [2] defy; go against [3] distant; apart from [4] run into; meet with [5] act in pair [6] bright (light) [7] a Chinese family name
U+96E3.0	fUTF8	難
U+96E3.0	fCNS11643	1-7945
U+96E3.0	fBig5	C3F8
U+96E3.0	fGB	1-4451
U+96E3.0	fR/S	 172.11; 172.10
U+96E3.0	fTotalStrokes	 19
U+96E3.0	fCangjie	TOOG
U+96E3.0	fFourCorner	40514
U+96E3.0	fHakka	  nan2;  lan2; lan3; nan3; lan5; nan6
U+96E3.0	fCantonese	naan4; naan6; no4; laan4 (also); laan6 (also); lo4 (also)
U+96E3.0	fMandarin	nan2; nuo2; nuo3
U+96E3.0	fEnglish	[1] difficult; not easy; arduous; hard [2] unable; not in a position to [3] unpleasant; not good [4] now
U+96E3.1	fUTF8	難
U+96E3.1	fCNS11643	1-7945
U+96E3.1	fBig5	C3F8
U+96E3.1	fGB	1-4451
U+96E3.1	fR/S	 172.11; 172.10
U+96E3.1	fTotalStrokes	 19
U+96E3.1	fCangjie	TOOG
U+96E3.1	fFourCorner	40514
U+96E3.1	fHakka	 nan5
U+96E3.1	fMandarin	nan4
U+96E3.1	fEnglish	[1] disaster; calamity; misfortune [2] rebuke; reprove; reprimand
U+96E3.2	fUTF8	難
U+96E3.2	fCNS11643	1-7945
U+96E3.2	fBig5	C3F8
U+96E3.2	fGB	1-4451
U+96E3.2	fR/S	 172.11; 172.10
U+96E3.2	fTotalStrokes	 19
U+96E3.2	fCangjie	TOOG
U+96E3.2	fFourCorner	40514
U+96E3.2	fMandarin	nan5
U+96E4.0	fUTF8	雤
U+96E4.0	fCNS11643	3-5E70
U+96E4.0	fGB	E-4447
U+96E4.0	fR/S	 172.13
U+96E4.0	fTotalStrokes	 21
U+96E4.0	fCangjie	HBOG
U+96E4.0	fFourCorner	77210
U+96E4.0	fCantonese	hok6
U+96E4.0	fMandarin	xue2
U+96E5.0	fUTF8	雥
U+96E5.0	fCNS11643	2-705B
U+96E5.0	fBig5	F8CD
U+96E5.0	fGB	5-745F
U+96E5.0	fR/S	 172.16
U+96E5.0	fTotalStrokes	 24
U+96E5.0	fCangjie	OGOGG
U+96E5.0	fFourCorner	20214
U+96E5.0	fCantonese	zaap6
U+96E5.0	fMandarin	za2
U+96E6.0	fUTF8	雦
U+96E6.0	fCNS11643	3-6125
U+96E6.0	fGB	E-4448
U+96E6.0	fR/S	 172.16
U+96E6.0	fTotalStrokes	 24
U+96E6.0	fCangjie	OGOGG
U+96E6.0	fFourCorner	20210
U+96E6.0	fCantonese	zaap6
U+96E6.0	fMandarin	ji2
U+96E7.0	fUTF8	雧
U+96E7.0	fCNS11643	3-622F
U+96E7.0	fGB	5-4639
U+96E7.0	fR/S	 172.20
U+96E7.0	fTotalStrokes	 28
U+96E7.0	fCangjie	OGOGD
U+96E7.0	fFourCorner	20900
U+96E7.0	fCantonese	zaap6; zap6 (also)
U+96E7.0	fMandarin	ji2
U+96E8.0	fUTF8	雨
U+96E8.0	fCNS11643	1-4F64
U+96E8.0	fBig5	AB42
U+96E8.0	fGB	0-536A
U+96E8.0	fR/S	 173.0
U+96E8.0	fTotalStrokes	 8
U+96E8.0	fCangjie	MLBY
U+96E8.0	fFourCorner	10227
U+96E8.0	fHakka	  ji3; ri3; i3
U+96E8.0	fCantonese	jyu5; jyu6
U+96E8.0	fMandarin	yu3
U+96E8.0	fEnglish	[1] rain; rainy [2] Kangxi radical 173
U+96E8.1	fUTF8	雨
U+96E8.1	fCNS11643	1-4F64
U+96E8.1	fBig5	AB42
U+96E8.1	fGB	0-536A
U+96E8.1	fR/S	 173.0
U+96E8.1	fTotalStrokes	 8
U+96E8.1	fCangjie	MLBY
U+96E8.1	fFourCorner	10227
U+96E8.1	fMandarin	yu4
U+96E8.1	fEnglish	 rain down; pour down
U+96E9.0	fUTF8	雩
U+96E9.0	fCNS11643	1-5D7C
U+96E9.0	fBig5	B3B8
U+96E9.0	fGB	0-7627
U+96E9.0	fR/S	 173.3
U+96E9.0	fTotalStrokes	 11
U+96E9.0	fCangjie	MBMMS
U+96E9.0	fFourCorner	10727
U+96E9.0	fHakka	 ri1; ji1; i2; ji2
U+96E9.0	fCantonese	jyu4; jyu6
U+96E9.0	fMandarin	yu2; xu1; wu1
U+96E9.0	fEnglish	 offer sacrifice for rain; pray for rain; sacrifice for the rain god
U+96E9.1	fUTF8	雩
U+96E9.1	fCNS11643	1-5D7C
U+96E9.1	fBig5	B3B8
U+96E9.1	fGB	0-7627
U+96E9.1	fR/S	 173.3
U+96E9.1	fTotalStrokes	 11
U+96E9.1	fCangjie	MBMMS
U+96E9.1	fFourCorner	10727
U+96E9.1	fMandarin	yu4
U+96E9.1	fEnglish	 rainbow
U+96EA.0	fUTF8	雪
U+96EA.0	fCNS11643	1-5D7B
U+96EA.0	fBig5	B3B7
U+96EA.0	fGB	0-5129
U+96EA.0	fR/S	 173.3
U+96EA.0	fTotalStrokes	 11
U+96EA.0	fCangjie	MBSM
U+96EA.0	fFourCorner	10177
U+96EA.0	fHakka	 siet7; set7; siat7
U+96EA.0	fCantonese	syut3
U+96EA.0	fEnglish	[1] snow [2] wipe away; wash away; clean
U+96EA.1	fUTF8	雪
U+96EA.1	fCNS11643	1-5D7B
U+96EA.1	fBig5	B3B7
U+96EA.1	fGB	0-5129
U+96EA.1	fR/S	 173.3
U+96EA.1	fTotalStrokes	 11
U+96EA.1	fCangjie	MBSM
U+96EA.1	fFourCorner	10177
U+96EA.1	fMandarin	xue4; xue3
U+96EA.1	fEnglish	[1] snow-white; snowy [2] avenge; wipe out grievances
U+96EB.0	fUTF8	雫
U+96EB.0	fCNS11643	F-392A
U+96EB.0	fGB	E-4449
U+96EB.0	fR/S	 173.3
U+96EB.0	fTotalStrokes	 11
U+96EB.0	fCangjie	MBMY
U+96EB.0	fFourCorner	1023
U+96EB.0	fCantonese	no5; lo5 (also)
U+96EB.0	fMandarin	na3
U+96EB.0	fEnglish	a drop, trickle, dripping
U+96EC.0	fUTF8	雬
U+96EC.0	fCNS11643	3-3F7E
U+96EC.0	fGB	E-444A
U+96EC.0	fR/S	 173.4
U+96EC.0	fTotalStrokes	 12
U+96EC.0	fCangjie	MBD
U+96EC.0	fFourCorner	10900
U+96EC.0	fCantonese	fau2
U+96EC.0	fMandarin	fou3
U+96ED.0	fUTF8	雭
U+96ED.0	fCNS11643	3-3F7C
U+96ED.0	fGB	5-7340
U+96ED.0	fR/S	 173.4
U+96ED.0	fTotalStrokes	 12
U+96ED.0	fCangjie	MBNHE
U+96ED.0	fFourCorner	10240
U+96ED.0	fCantonese	saap3
U+96ED.0	fMandarin	se4
U+96EE.0	fUTF8	雮
U+96EE.0	fCNS11643	3-3F7B
U+96EE.0	fGB	E-444B
U+96EE.0	fR/S	 173.4
U+96EE.0	fTotalStrokes	 12
U+96EE.0	fCangjie	MBHQU
U+96EE.0	fFourCorner	10710
U+96EE.0	fCantonese	muk6
U+96EE.0	fMandarin	mu4
U+96EF.0	fUTF8	雯
U+96EF.0	fCNS11643	1-6277
U+96EF.0	fBig5	B6B2
U+96EF.0	fGB	0-7629
U+96EF.0	fR/S	 173.4
U+96EF.0	fTotalStrokes	 12
U+96EF.0	fCangjie	MBYK
U+96EF.0	fFourCorner	10401
U+96EF.0	fHakka	  wun2; vun2
U+96EF.0	fCantonese	man4; man2 (also)
U+96EF.0	fMandarin	wen2
U+96EF.0	fEnglish	the coloring on the clouds
U+96F0.0	fUTF8	雰
U+96F0.0	fCNS11643	2-414D
U+96F0.0	fBig5	DCA8
U+96F0.0	fGB	E-444C
U+96F0.0	fR/S	 173.4
U+96F0.0	fTotalStrokes	 12
U+96F0.0	fCangjie	MBCSH
U+96F0.0	fFourCorner	10227
U+96F0.0	fHakka	  fun1
U+96F0.0	fCantonese	fan1
U+96F0.0	fMandarin	fen1; fen2
U+96F0.0	fEnglish	 mist; fog
U+96F1.0	fUTF8	雱
U+96F1.0	fCNS11643	2-414C
U+96F1.0	fBig5	DCA7
U+96F1.0	fGB	3-743C
U+96F1.0	fR/S	 173.4
U+96F1.0	fTotalStrokes	 12
U+96F1.0	fCangjie	MBYHS
U+96F1.0	fFourCorner	10227
U+96F1.0	fHakka	  pong2
U+96F1.0	fCantonese	pong1; pong4
U+96F1.0	fMandarin	pang2; fang1; pang1
U+96F1.0	fEnglish	snowing heavily
U+96F2.0	fUTF8	雲
U+96F2.0	fCNS11643	1-6278
U+96F2.0	fBig5	B6B3
U+96F2.0	fGB	1-5446
U+96F2.0	fR/S	 173.4
U+96F2.0	fTotalStrokes	 12
U+96F2.0	fCangjie	MBMMI
U+96F2.0	fFourCorner	10731
U+96F2.0	fHakka	  jun2; run2; jun3
U+96F2.0	fCantonese	wan4; wan2 (also)
U+96F2.0	fMandarin	yun2
U+96F2.0	fEnglish	[1] clouds [2] a cloud of; a large number of [3] short for Yunnan province [4] a Chinese family name
U+96F3.0	fUTF8	雳
U+96F3.0	fGB	0-7628
U+96F3.0	fR/S	 173.4
U+96F3.0	fTotalStrokes	 12
U+96F3.0	fCangjie	MBMKS
U+96F3.0	fHakka	 lit8
U+96F3.0	fCantonese	lik1; lik6 (also)
U+96F3.0	fMandarin	li4; ling2
U+96F3.0	fEnglish	simplified form of &#x9742;
U+96F4.0	fUTF8	雴
U+96F4.0	fCNS11643	3-4546
U+96F4.0	fGB	E-444D
U+96F4.0	fR/S	 173.5
U+96F4.0	fTotalStrokes	 13
U+96F4.0	fCangjie	MBYT
U+96F4.0	fFourCorner	10100
U+96F4.0	fCantonese	cap1
U+96F4.0	fMandarin	li4
U+96F5.0	fUTF8	雵
U+96F5.0	fCNS11643	2-485F
U+96F5.0	fBig5	E0D9
U+96F5.0	fGB	E-444E
U+96F5.0	fR/S	 173.5
U+96F5.0	fTotalStrokes	 13
U+96F5.0	fCangjie	MBLBK
U+96F5.0	fFourCorner	10530
U+96F5.0	fCantonese	joeng1
U+96F5.0	fMandarin	ang3; yang1
U+96F6.0	fUTF8	零
U+96F6.0	fCNS11643	1-675B
U+96F6.0	fBig5	B973
U+96F6.0	fGB	0-4163
U+96F6.0	fR/S	 173.5
U+96F6.0	fTotalStrokes	 13
U+96F6.0	fCangjie	MBOII
U+96F6.0	fFourCorner	10307
U+96F6.0	fHakka	  lang2;  len2; lin2; lang3; lan3; len1
U+96F6.0	fCantonese	ling4; leng4 (also); ling1 (also); ling2 (also); lin4
U+96F6.0	fMandarin	ling2; lian2; ling4
U+96F6.0	fEnglish	[1] zero; nil; nought [2] fragment; fraction; fractional; remainder [3] flow down [4] light rain; drizzle
U+96F7.0	fUTF8	雷
U+96F7.0	fCNS11643	1-6758
U+96F7.0	fBig5	B970
U+96F7.0	fGB	0-4057
U+96F7.0	fR/S	 173.5
U+96F7.0	fTotalStrokes	 13
U+96F7.0	fCangjie	MBW
U+96F7.0	fFourCorner	10603
U+96F7.0	fHakka	  lui2; lui3
U+96F7.0	fCantonese	leoi4
U+96F7.0	fMandarin	lei2; lei4; lei3
U+96F7.0	fEnglish	[1] thunder [2] mine (explosive) [3] a Chinese family name
U+96F8.0	fUTF8	雸
U+96F8.0	fCNS11643	2-485E
U+96F8.0	fBig5	E0D8
U+96F8.0	fGB	E-444F
U+96F8.0	fR/S	 173.5
U+96F8.0	fTotalStrokes	 13
U+96F8.0	fCangjie	MBTM
U+96F8.0	fFourCorner	10774
U+96F8.0	fHakka	  gam1
U+96F8.0	fCantonese	ngam4; am4 (also); gam1
U+96F8.0	fMandarin	an2
U+96F8.0	fEnglish	[1] hoar-frost [2] dew
U+96F9.0	fUTF8	雹
U+96F9.0	fCNS11643	1-675A
U+96F9.0	fBig5	B972
U+96F9.0	fGB	0-3122
U+96F9.0	fR/S	 173.5
U+96F9.0	fTotalStrokes	 13
U+96F9.0	fCangjie	MBPRU
U+96F9.0	fFourCorner	10712
U+96F9.0	fHakka	  pok8; pau1; pau5
U+96F9.0	fCantonese	bok6
U+96F9.0	fMandarin	bao2; bo2; diao1; pu2
U+96F9.0	fEnglish	 hail; hailstone
U+96FA.0	fUTF8	雺
U+96FA.0	fCNS11643	2-485C
U+96FA.0	fBig5	E0D6
U+96FA.0	fGB	5-7344
U+96FA.0	fR/S	 173.5
U+96FA.0	fTotalStrokes	 13
U+96FA.0	fCangjie	MBNIH
U+96FA.0	fFourCorner	10227
U+96FA.0	fHakka	 wu5
U+96FA.0	fCantonese	mung4; mung6 (also); mou6; mau4
U+96FA.0	fMandarin	meng2; mou2; mou4; wu4
U+96FB.0	fUTF8	電
U+96FB.0	fCNS11643	1-6759
U+96FB.0	fBig5	B971
U+96FB.0	fGB	1-3567
U+96FB.0	fR/S	 173.5
U+96FB.0	fTotalStrokes	 13
U+96FB.0	fCangjie	MBWU
U+96FB.0	fFourCorner	10716
U+96FB.0	fHakka	  tien5; ten3; tien6; t'ian6
U+96FB.0	fCantonese	din6
U+96FB.0	fMandarin	dian4
U+96FB.0	fEnglish	[1] electricity; electric; lightning [2] short for cable or telegram
U+96FC.0	fUTF8	雼
U+96FC.0	fCNS11643	3-4547
U+96FC.0	fGB	E-4450
U+96FC.0	fR/S	 173.5
U+96FC.0	fTotalStrokes	 13
U+96FC.0	fCangjie	MBMR
U+96FC.0	fFourCorner	10600
U+96FC.0	fCantonese	dong6
U+96FC.0	fMandarin	dang4
U+96FD.0	fUTF8	雽
U+96FD.0	fCNS11643	2-485D
U+96FD.0	fBig5	E0D7
U+96FD.0	fGB	E-4451
U+96FD.0	fR/S	 173.5
U+96FD.0	fTotalStrokes	 13
U+96FD.0	fCangjie	MBHFD
U+96FD.0	fFourCorner	10409
U+96FD.0	fCantonese	wu6; fu1
U+96FD.0	fMandarin	xing2; yu2; hang2; hu1
U+96FE.0	fUTF8	雾
U+96FE.0	fGB	0-4E6D
U+96FE.0	fR/S	 173.5
U+96FE.0	fTotalStrokes	 13
U+96FE.0	fCangjie	MBHES
U+96FE.0	fHakka	 vu5
U+96FE.0	fCantonese	mou6
U+96FE.0	fMandarin	wu4; meng2; meng4
U+96FE.0	fEnglish	simplified form of &#x9727;
U+96FF.0	fUTF8	雿
U+96FF.0	fCNS11643	2-4F25
U+96FF.0	fBig5	E4BD
U+96FF.0	fGB	5-734A
U+96FF.0	fR/S	 173.6
U+96FF.0	fTotalStrokes	 14
U+96FF.0	fCangjie	MBLMO
U+96FF.0	fFourCorner	10113
U+96FF.0	fCantonese	diu6; zaak6
U+96FF.0	fMandarin	zhao4; zhai2; zhe4
U+9700.0	fUTF8	需
U+9700.0	fCNS11643	1-6B66
U+9700.0	fBig5	BBDD
U+9700.0	fGB	0-5068
U+9700.0	fR/S	 173.6
U+9700.0	fTotalStrokes	 14
U+9700.0	fCangjie	MBMBL
U+9700.0	fFourCorner	10227
U+9700.0	fHakka	  si1
U+9700.0	fCantonese	seoi1
U+9700.0	fMandarin	xu1; nuo4; ruan3; ru2; nuan4
U+9700.0	fEnglish	[1] need; require; must; demand [2] expenses; provisions; needs; necessaries [3] hesitation; delay
U+9701.0	fUTF8	霁
U+9701.0	fGB	0-762B
U+9701.0	fR/S	 173.6
U+9701.0	fTotalStrokes	 14
U+9701.0	fCangjie	MBYKL
U+9701.0	fHakka	 zi5
U+9701.0	fCantonese	zai3
U+9701.0	fMandarin	ji4; ji3; pi1
U+9701.0	fEnglish	simplified form of &#x973D;
U+9702.0	fUTF8	霂
U+9702.0	fCNS11643	2-5558
U+9702.0	fBig5	E8AF
U+9702.0	fGB	3-7443
U+9702.0	fR/S	 173.7
U+9702.0	fTotalStrokes	 15
U+9702.0	fCangjie	MBED
U+9702.0	fFourCorner	10394
U+9702.0	fHakka	  muk7
U+9702.0	fCantonese	muk6
U+9702.0	fMandarin	mu4
U+9702.0	fEnglish	fine rain; drizzle
U+9703.0	fUTF8	霃
U+9703.0	fCNS11643	5-523B
U+9703.0	fGB	5-734E
U+9703.0	fR/S	 173.7
U+9703.0	fTotalStrokes	 15
U+9703.0	fCangjie	MBELU
U+9703.0	fFourCorner	10110
U+9703.0	fHakka	 chim2; cihm2
U+9703.0	fCantonese	cam4
U+9703.0	fMandarin	chen2
U+9703.0	fEnglish	long continued rains
U+9704.0	fUTF8	霄
U+9704.0	fCNS11643	1-6F66
U+9704.0	fBig5	BE5D
U+9704.0	fGB	0-4F76
U+9704.0	fR/S	 173.7
U+9704.0	fTotalStrokes	 15
U+9704.0	fCangjie	MBFB
U+9704.0	fFourCorner	10227
U+9704.0	fHakka	  siau1; seu1
U+9704.0	fCantonese	siu1
U+9704.0	fMandarin	xiao1; xiao4; bao2; bo2
U+9704.0	fEnglish	[1] sky [2] clouds or mist [3] night [4] exhaust; dissolve
U+9705.0	fUTF8	霅
U+9705.0	fCNS11643	2-5556
U+9705.0	fBig5	E8AD
U+9705.0	fGB	3-7442
U+9705.0	fR/S	 173.7
U+9705.0	fTotalStrokes	 15
U+9705.0	fCangjie	MBYMR
U+9705.0	fFourCorner	10601
U+9705.0	fHakka	 tiap8; sap8; zhap8; zhap7
U+9705.0	fCantonese	zip3; saap3; zaap6
U+9705.0	fMandarin	zha2; xia2; sa4; zha4; sha4; yi4; zhe2
U+9705.0	fEnglish	thunder
U+9706.0	fUTF8	霆
U+9706.0	fCNS11643	1-6F67
U+9706.0	fBig5	BE5E
U+9706.0	fGB	0-762A
U+9706.0	fR/S	 173.7
U+9706.0	fTotalStrokes	 15
U+9706.0	fCangjie	MBNKG
U+9706.0	fFourCorner	10401
U+9706.0	fHakka	 tin2; tin3
U+9706.0	fCantonese	ting4
U+9706.0	fMandarin	ting2; ding4; ting3; xiao1
U+9706.0	fEnglish	 sudden peal of thunder; thunderbold
U+9707.0	fUTF8	震
U+9707.0	fCNS11643	1-6F68
U+9707.0	fBig5	BE5F
U+9707.0	fGB	0-5570
U+9707.0	fR/S	 173.7
U+9707.0	fTotalStrokes	 15
U+9707.0	fCangjie	MBMMV
U+9707.0	fFourCorner	10232
U+9707.0	fHakka	 zhin3; zin3; {tin1}; zihn3; zun1; zhin6; zhin5; zihn5; zhin1; zin1; zhun1
U+9707.0	fCantonese	zan3
U+9707.0	fMandarin	zhen4; shen1; zhen1
U+9707.0	fEnglish	[1] shake; quake; tremor; tremble, as an earthquake [2] excite; shock [3] scared; terrified [4] thunder; thunderclap [5] the 4th of the Eight Diagrams
U+9708.0	fUTF8	霈
U+9708.0	fCNS11643	2-5557
U+9708.0	fBig5	E8AE
U+9708.0	fGB	0-762C
U+9708.0	fR/S	 173.7
U+9708.0	fTotalStrokes	 15
U+9708.0	fCangjie	MBEJB
U+9708.0	fFourCorner	10127
U+9708.0	fHakka	  pai5; pui5
U+9708.0	fCantonese	pui3
U+9708.0	fMandarin	pei4; ting2
U+9708.0	fEnglish	[1] rains; torrential rains [2] favors; good graces
U+9709.0	fUTF8	霉
U+9709.0	fCNS11643	1-6F69
U+9709.0	fBig5	BE60
U+9709.0	fGB	0-4339
U+9709.0	fR/S	 173.7
U+9709.0	fTotalStrokes	 15
U+9709.0	fCangjie	MBOWY
U+9709.0	fFourCorner	10507
U+9709.0	fHakka	 moi2; mui2; mi2; moi1; moe2
U+9709.0	fCantonese	mui4; mei4
U+9709.0	fMandarin	mei2
U+9709.0	fEnglish	mildew; mildewed; musty; moldy; mold
U+970A.0	fUTF8	霊
U+970A.0	fCNS11643	3-4F60
U+970A.0	fGB	E-4452
U+970A.0	fR/S	 173.7
U+970A.0	fTotalStrokes	 15
U+970A.0	fCangjie	MBMMC
U+970A.0	fFourCorner	10100
U+970A.0	fCantonese	ling4
U+970A.0	fMandarin	ling2
U+970A.0	fEnglish	 spirit; soul spiritual world
U+970B.0	fUTF8	霋
U+970B.0	fCNS11643	2-5B5D
U+970B.0	fBig5	EC51
U+970B.0	fGB	5-7351
U+970B.0	fR/S	 173.8
U+970B.0	fTotalStrokes	 16
U+970B.0	fCangjie	MBJLV
U+970B.0	fFourCorner	10406
U+970B.0	fHakka	  ci1
U+970B.0	fCantonese	cai1; saap3
U+970B.0	fMandarin	qi1
U+970B.0	fEnglish	clouds driving across the sky
U+970C.0	fUTF8	霌
U+970C.0	fCNS11643	3-5350
U+970C.0	fGB	E-4453
U+970C.0	fR/S	 173.8
U+970C.0	fTotalStrokes	 16
U+970C.0	fCangjie	MBBGR
U+970C.0	fFourCorner	10220
U+970C.0	fCantonese	zau1
U+970C.0	fMandarin	zhou1; chou1
U+970D.0	fUTF8	霍
U+970D.0	fCNS11643	1-7277
U+970D.0	fBig5	C04E
U+970D.0	fGB	0-3B74
U+970D.0	fR/S	 173.8
U+970D.0	fTotalStrokes	 16
U+970D.0	fCangjie	MBOG
U+970D.0	fFourCorner	10214
U+970D.0	fHakka	  kok7
U+970D.0	fCantonese	fok3
U+970D.0	fMandarin	huo4; he4; luo4; suo3; za2
U+970D.0	fEnglish	[1] very rapidly; in a flash; suddenly [2] a Chinese family name
U+970E.0	fUTF8	霎
U+970E.0	fCNS11643	1-7274
U+970E.0	fBig5	C04B
U+970E.0	fGB	0-762E
U+970E.0	fR/S	 173.8
U+970E.0	fTotalStrokes	 16
U+970E.0	fCangjie	MBYTV
U+970E.0	fFourCorner	10404
U+970E.0	fHakka	 sap7; ciap7; siap8
U+970E.0	fCantonese	saap3; sap3
U+970E.0	fMandarin	sha4; ni2; sa4
U+970E.0	fEnglish	[1] instant; in the twinkle of an eye; in a very short time [2] slight rain; drizzle
U+970F.0	fUTF8	霏
U+970F.0	fCNS11643	1-7279
U+970F.0	fBig5	C050
U+970F.0	fGB	0-762D
U+970F.0	fR/S	 173.8
U+970F.0	fTotalStrokes	 16
U+970F.0	fCangjie	MBLMY
U+970F.0	fFourCorner	10111
U+970F.0	fHakka	  fui1; fi1
U+970F.0	fCantonese	fei1
U+970F.0	fMandarin	fei1; zhan1
U+970F.0	fEnglish	the falling of snow and rain
U+9710.0	fUTF8	霐
U+9710.0	fCNS11643	2-5B5F
U+9710.0	fBig5	EC53
U+9710.0	fGB	5-7356
U+9710.0	fR/S	 173.8
U+9710.0	fTotalStrokes	 16
U+9710.0	fCangjie	MBENI
U+9710.0	fFourCorner	10132
U+9710.0	fCantonese	wang4
U+9710.0	fMandarin	weng1; hong2
U+9711.0	fUTF8	霑
U+9711.0	fCNS11643	1-7275
U+9711.0	fBig5	C04C
U+9711.0	fGB	E-4454
U+9711.0	fR/S	 173.8
U+9711.0	fTotalStrokes	 16
U+9711.0	fCangjie	MBEYR
U+9711.0	fFourCorner	10161
U+9711.0	fHakka	  zham1; zam1
U+9711.0	fCantonese	zim1
U+9711.0	fMandarin	zhan1; sha4
U+9711.0	fEnglish	[1] be moistened; soaked; moisten; become wet or damp; soaked [2] receive (benefits, etc.)
U+9712.0	fUTF8	霒
U+9712.0	fCNS11643	2-5B5E
U+9712.0	fBig5	EC52
U+9712.0	fGB	5-352C
U+9712.0	fR/S	 173.8
U+9712.0	fTotalStrokes	 16
U+9712.0	fCangjie	MIOIN
U+9712.0	fFourCorner	18727
U+9712.0	fCantonese	jam1
U+9712.0	fMandarin	yin1; ying1
U+9713.0	fUTF8	霓
U+9713.0	fCNS11643	1-7278
U+9713.0	fBig5	C04F
U+9713.0	fGB	0-445E
U+9713.0	fR/S	 173.8
U+9713.0	fTotalStrokes	 16
U+9713.0	fCangjie	MBHXU
U+9713.0	fFourCorner	10217
U+9713.0	fHakka	 ngi2;  ji2; ri2; i2
U+9713.0	fCantonese	ngai4; ai4 (also)
U+9713.0	fMandarin	ni2; ni4; nie4; pei4; yi4
U+9713.0	fEnglish	 rainbow; colored cloud
U+9714.0	fUTF8	霔
U+9714.0	fCNS11643	3-534D
U+9714.0	fGB	5-7355
U+9714.0	fR/S	 173.8
U+9714.0	fTotalStrokes	 16
U+9714.0	fCangjie	MBEYG
U+9714.0	fFourCorner	10110
U+9714.0	fHakka	 zhu5; zu5; shu5
U+9714.0	fCantonese	zyu3
U+9714.0	fMandarin	zhu4
U+9715.0	fUTF8	霕
U+9715.0	fCNS11643	3-534C
U+9715.0	fGB	E-4455
U+9715.0	fR/S	 173.8
U+9715.0	fTotalStrokes	 16
U+9715.0	fCangjie	MIPU
U+9715.0	fFourCorner	15710
U+9715.0	fCantonese	tyun4
U+9715.0	fMandarin	tun2
U+9716.0	fUTF8	霖
U+9716.0	fCNS11643	1-7276
U+9716.0	fBig5	C04D
U+9716.0	fGB	0-4158
U+9716.0	fR/S	 173.8
U+9716.0	fTotalStrokes	 16
U+9716.0	fCangjie	MBDD
U+9716.0	fFourCorner	10994
U+9716.0	fHakka	 lim2; lim3
U+9716.0	fCantonese	lam4
U+9716.0	fMandarin	lin2; fei1
U+9716.0	fEnglish	 long spell of rain; copious rain falling continuously; continious heavy rain
U+9717.0	fUTF8	霗
U+9717.0	fCNS11643	3-5351
U+9717.0	fGB	E-4456
U+9717.0	fR/S	 173.8
U+9717.0	fTotalStrokes	 16
U+9717.0	fCangjie	MBEOI
U+9717.0	fFourCorner	10130
U+9717.0	fCantonese	ling4
U+9717.0	fMandarin	ling2
U+9718.0	fUTF8	霘
U+9718.0	fCNS11643	2-6046
U+9718.0	fBig5	EEF9
U+9718.0	fGB	E-4457
U+9718.0	fR/S	 173.9
U+9718.0	fTotalStrokes	 17
U+9718.0	fCangjie	MBEBR
U+9718.0	fFourCorner	10127
U+9718.0	fCantonese	dung6
U+9718.0	fMandarin	dong4
U+9719.0	fUTF8	霙
U+9719.0	fCNS11643	2-6048
U+9719.0	fBig5	EEFB
U+9719.0	fGB	3-7444
U+9719.0	fR/S	 173.9
U+9719.0	fTotalStrokes	 17
U+9719.0	fCangjie	MBTLK
U+9719.0	fFourCorner	10530
U+9719.0	fHakka	 jin1; rin1
U+9719.0	fCantonese	jing1
U+9719.0	fMandarin	ying1; ji1; lin2
U+9719.0	fEnglish	sleet
U+971A.0	fUTF8	霚
U+971A.0	fCNS11643	3-5674
U+971A.0	fGB	E-4458
U+971A.0	fR/S	 173.9
U+971A.0	fTotalStrokes	 17
U+971A.0	fCangjie	MBNHK
U+971A.0	fFourCorner	10240
U+971A.0	fCantonese	mou6
U+971A.0	fMandarin	wu4
U+971B.0	fUTF8	霛
U+971B.0	fCNS11643	3-5675
U+971B.0	fGB	5-735E
U+971B.0	fR/S	 173.9
U+971B.0	fTotalStrokes	 17
U+971B.0	fCangjie	MBNNN
U+971B.0	fFourCorner	10220
U+971B.0	fHakka	 lin2
U+971B.0	fCantonese	ling4; leng4 (also)
U+971B.0	fMandarin	ling2
U+971B.0	fEnglish	spirit; ingenious; spiritual; supernatural; intelligent; efficacious
U+971C.0	fUTF8	霜
U+971C.0	fCNS11643	1-7624
U+971C.0	fBig5	C1F7
U+971C.0	fGB	0-4B2A
U+971C.0	fR/S	 173.9
U+971C.0	fTotalStrokes	 17
U+971C.0	fCangjie	MBDBU
U+971C.0	fFourCorner	10963
U+971C.0	fHakka	  song1; siong1
U+971C.0	fCantonese	soeng1
U+971C.0	fMandarin	shuang1; shuan1; shuang4
U+971C.0	fEnglish	[1] frost; hoarfrost [2] white and powdery (like hoarfrost) [3] coolnes; indifference; grave [4] virtuous; pure and clean
U+971D.0	fUTF8	霝
U+971D.0	fCNS11643	2-6047
U+971D.0	fBig5	EEFA
U+971D.0	fGB	5-7359
U+971D.0	fR/S	 173.9
U+971D.0	fTotalStrokes	 17
U+971D.0	fCangjie	MBRRR
U+971D.0	fFourCorner	10662
U+971D.0	fHakka	 lin2
U+971D.0	fCantonese	ling4
U+971D.0	fMandarin	ling2; ling4
U+971D.0	fEnglish	[1] fall (in drops) [2] rain; rainfall
U+971E.0	fUTF8	霞
U+971E.0	fCNS11643	1-7625
U+971E.0	fBig5	C1F8
U+971E.0	fGB	0-4F3C
U+971E.0	fR/S	 173.9
U+971E.0	fTotalStrokes	 17
U+971E.0	fCangjie	MBRSE
U+971E.0	fFourCorner	10247
U+971E.0	fHakka	  ha2
U+971E.0	fCantonese	haa4
U+971E.0	fMandarin	xia2
U+971E.0	fEnglish	rosy clouds; colored, low-hanging clouds
U+971F.0	fUTF8	霟
U+971F.0	fCNS11643	2-6045
U+971F.0	fBig5	EEF8
U+971F.0	fGB	E-4459
U+971F.0	fR/S	 173.9
U+971F.0	fTotalStrokes	 17
U+971F.0	fCangjie	MBETC
U+971F.0	fFourCorner	10181
U+971F.0	fCantonese	hung4
U+971F.0	fMandarin	hong2
U+9720.0	fUTF8	霠
U+9720.0	fCNS11643	2-6044
U+9720.0	fBig5	EEF7
U+9720.0	fGB	5-735C
U+9720.0	fR/S	 173.9
U+9720.0	fTotalStrokes	 17
U+9720.0	fCangjie	MBYTN
U+9720.0	fFourCorner	10127
U+9720.0	fHakka	 jim1; rim1
U+9720.0	fCantonese	jam1
U+9720.0	fMandarin	yin1
U+9720.0	fEnglish	a Chinese family name
U+9721.0	fUTF8	霡
U+9721.0	fCNS11643	4-5E38
U+9721.0	fGB	3-744B
U+9721.0	fR/S	 173.10
U+9721.0	fTotalStrokes	 18
U+9721.0	fCangjie	MBBIE
U+9721.0	fFourCorner	10232
U+9721.0	fHakka	  mak7
U+9721.0	fCantonese	mak6
U+9721.0	fMandarin	mai4; mo4
U+9721.0	fEnglish	 drizzling rain soak (used fig. of favours)
U+9722.0	fUTF8	霢
U+9722.0	fCNS11643	2-6443
U+9722.0	fBig5	F177
U+9722.0	fGB	5-7360
U+9722.0	fR/S	 173.10
U+9722.0	fTotalStrokes	 18
U+9722.0	fCangjie	MBBHV
U+9722.0	fFourCorner	10213
U+9722.0	fHakka	 mak7
U+9722.0	fCantonese	mak6; maak6
U+9722.0	fMandarin	mai4; mo4; mi4
U+9722.0	fEnglish	dust
U+9723.0	fUTF8	霣
U+9723.0	fCNS11643	2-6442
U+9723.0	fBig5	F176
U+9723.0	fGB	3-7441
U+9723.0	fR/S	 173.10
U+9723.0	fTotalStrokes	 18
U+9723.0	fCangjie	MBRBC
U+9723.0	fFourCorner	10806
U+9723.0	fCantonese	wan5
U+9723.0	fMandarin	yun3; kun1; yun4
U+9723.0	fEnglish	fall
U+9724.0	fUTF8	霤
U+9724.0	fCNS11643	1-7767
U+9724.0	fBig5	C2FC
U+9724.0	fGB	3-744D
U+9724.0	fR/S	 173.10
U+9724.0	fTotalStrokes	 18
U+9724.0	fCangjie	MBHHW
U+9724.0	fFourCorner	10602
U+9724.0	fHakka	  liu5
U+9724.0	fCantonese	lau6
U+9724.0	fMandarin	liu4; po2; ying1
U+9724.0	fEnglish	[1] the dripping of water (rain, etc.) from theeaves [2] eaves; rain-water catcher
U+9725.0	fUTF8	霥
U+9725.0	fCNS11643	2-6444
U+9725.0	fBig5	F178
U+9725.0	fGB	E-445A
U+9725.0	fR/S	 173.10
U+9725.0	fTotalStrokes	 18
U+9725.0	fCangjie	MBBMO
U+9725.0	fFourCorner	10232
U+9725.0	fCantonese	mung6; mung4
U+9725.0	fMandarin	meng4
U+9726.0	fUTF8	霦
U+9726.0	fCNS11643	2-676B
U+9726.0	fBig5	F37E
U+9726.0	fGB	3-744E
U+9726.0	fR/S	 173.11
U+9726.0	fTotalStrokes	 19
U+9726.0	fCangjie	MBDDH
U+9726.0	fFourCorner	10922
U+9726.0	fHakka	 bin1
U+9726.0	fCantonese	ban1
U+9726.0	fMandarin	bin1
U+9726.0	fEnglish	the brilliancy of a gem
U+9727.0	fUTF8	霧
U+9727.0	fCNS11643	1-7947
U+9727.0	fBig5	C3FA
U+9727.0	fGB	1-4E6D
U+9727.0	fR/S	 173.11
U+9727.0	fTotalStrokes	 19
U+9727.0	fCangjie	MBNHS
U+9727.0	fFourCorner	10227
U+9727.0	fHakka	 mu5; wu5; wu6; vu5
U+9727.0	fCantonese	mou6
U+9727.0	fMandarin	wu4; meng2; meng4
U+9727.0	fEnglish	fog; mist; vapor
U+9728.0	fUTF8	霨
U+9728.0	fCNS11643	2-676A
U+9728.0	fBig5	F37D
U+9728.0	fGB	5-7366
U+9728.0	fR/S	 173.11
U+9728.0	fTotalStrokes	 19
U+9728.0	fCangjie	MBSFI
U+9728.0	fFourCorner	10242
U+9728.0	fHakka	 wui5; wi5; vui5
U+9728.0	fCantonese	wai3
U+9728.0	fMandarin	wei4
U+9728.0	fEnglish	clouding
U+9729.0	fUTF8	霩
U+9729.0	fCNS11643	2-6767
U+9729.0	fBig5	F37A
U+9729.0	fGB	5-7361
U+9729.0	fR/S	 173.11
U+9729.0	fTotalStrokes	 19
U+9729.0	fCangjie	MBYDL
U+9729.0	fFourCorner	10427
U+9729.0	fHakka	 gok7; gwok7
U+9729.0	fCantonese	gwok3; gok3 (also); kwok3
U+9729.0	fMandarin	kuo4; huo4
U+9729.0	fEnglish	[1] clouds dispersing [2] snow melting
U+972A.0	fUTF8	霪
U+972A.0	fCNS11643	1-7946
U+972A.0	fBig5	C3F9
U+972A.0	fGB	0-762F
U+972A.0	fR/S	 173.11
U+972A.0	fTotalStrokes	 19
U+972A.0	fCangjie	MBEBG
U+972A.0	fFourCorner	10114
U+972A.0	fHakka	  jim2; rim2
U+972A.0	fCantonese	jam4
U+972A.0	fMandarin	yin2; liu4
U+972A.0	fEnglish	 rain for a long time
U+972B.0	fUTF8	霫
U+972B.0	fCNS11643	2-6768
U+972B.0	fBig5	F37B
U+972B.0	fGB	3-744F
U+972B.0	fR/S	 173.11
U+972B.0	fTotalStrokes	 19
U+972B.0	fCangjie	MBSMA
U+972B.0	fFourCorner	10602
U+972B.0	fHakka	  sip8
U+972B.0	fCantonese	zap6 (also); zaap6
U+972B.0	fMandarin	xi2; xi4
U+972B.0	fEnglish	heavy rains
U+972C.0	fUTF8	霬
U+972C.0	fCNS11643	2-6769
U+972C.0	fBig5	F37C
U+972C.0	fGB	E-445B
U+972C.0	fR/S	 173.11
U+972C.0	fTotalStrokes	 19
U+972C.0	fCangjie	MBWTC
U+972C.0	fFourCorner	10801
U+972C.0	fCantonese	jik6
U+972C.0	fMandarin	yi4; ling2
U+972D.0	fUTF8	霭
U+972D.0	fGB	0-7630
U+972D.0	fR/S	 173.11
U+972D.0	fTotalStrokes	 19
U+972D.0	fCangjie	MBIVV
U+972D.0	fHakka	 ai5
U+972D.0	fCantonese	oi2; ngoi2 (also)
U+972D.0	fMandarin	ai3; ai4; e4; mai2
U+972D.0	fEnglish	simplified form of &#x9744;
U+972E.0	fUTF8	霮
U+972E.0	fCNS11643	2-6A56
U+972E.0	fBig5	F548
U+972E.0	fGB	3-7451
U+972E.0	fR/S	 173.12
U+972E.0	fTotalStrokes	 20
U+972E.0	fCangjie	MBETV
U+972E.0	fFourCorner	10113
U+972E.0	fCantonese	daam6
U+972E.0	fMandarin	dan4
U+972F.0	fUTF8	霯
U+972F.0	fCNS11643	2-6A57
U+972F.0	fBig5	F549
U+972F.0	fGB	E-445C
U+972F.0	fR/S	 173.12
U+972F.0	fTotalStrokes	 20
U+972F.0	fCangjie	MBNOT
U+972F.0	fFourCorner	10108
U+972F.0	fHakka	  den5; ten5; ten6; ten1
U+972F.0	fCantonese	tang3; dang6; dang1
U+972F.0	fMandarin	deng4
U+972F.0	fEnglish	heavy rain
U+9730.0	fUTF8	霰
U+9730.0	fCNS11643	1-7A50
U+9730.0	fBig5	C4C5
U+9730.0	fGB	0-7631
U+9730.0	fR/S	 173.12
U+9730.0	fTotalStrokes	 20
U+9730.0	fCangjie	MBTBK
U+9730.0	fFourCorner	10248
U+9730.0	fHakka	 san5
U+9730.0	fCantonese	sin3
U+9730.0	fMandarin	xian4; san3; yin2
U+9730.0	fEnglish	snow and rain; sleet
U+9731.0	fUTF8	霱
U+9731.0	fCNS11643	3-5D4F
U+9731.0	fGB	5-736E
U+9731.0	fR/S	 173.12
U+9731.0	fTotalStrokes	 20
U+9731.0	fCangjie	MBNHB
U+9731.0	fFourCorner	10220
U+9731.0	fHakka	 lut7; jut8; rut8
U+9731.0	fCantonese	wat6; leot6
U+9731.0	fMandarin	yu4
U+9731.0	fEnglish	cloud
U+9732.0	fUTF8	露
U+9732.0	fCNS11643	1-7B3F
U+9732.0	fBig5	C553
U+9732.0	fGB	0-4236
U+9732.0	fR/S	 173.12
U+9732.0	fTotalStrokes	 20
U+9732.0	fCangjie	MBRMR
U+9732.0	fFourCorner	10164
U+9732.0	fHakka	  lu5
U+9732.0	fCantonese	lou6; lau6
U+9732.0	fMandarin	lu4
U+9732.0	fEnglish	[1] dew [2] uncovered; bare; open; exposed; show; reveal; betray [3] a cold soothing and aromatic drink; beverages distilled from flowers, fruits, or leaves [4] a Chinese family name
U+9732.1	fUTF8	露
U+9732.1	fCNS11643	1-7B3F
U+9732.1	fBig5	C553
U+9732.1	fGB	0-4236
U+9732.1	fR/S	 173.12
U+9732.1	fTotalStrokes	 20
U+9732.1	fCangjie	MBRMR
U+9732.1	fFourCorner	10164
U+9732.1	fMandarin	lou4
U+9732.1	fEnglish	 appear; emerge; show
U+9733.0	fUTF8	霳
U+9733.0	fCNS11643	3-5D4C
U+9733.0	fGB	5-7367
U+9733.0	fR/S	 173.12
U+9733.0	fTotalStrokes	 20
U+9733.0	fCangjie	MBNLM
U+9733.0	fFourCorner	10210
U+9733.0	fCantonese	lung4
U+9733.0	fMandarin	long2; long1
U+9734.0	fUTF8	霴
U+9734.0	fCNS11643	3-5D4E
U+9734.0	fGB	E-445D
U+9734.0	fR/S	 173.12
U+9734.0	fTotalStrokes	 20
U+9734.0	fCangjie	MILE
U+9734.0	fFourCorner	15730
U+9734.0	fCantonese	doi6
U+9734.0	fMandarin	dai4
U+9735.0	fUTF8	霵
U+9735.0	fCNS11643	2-6C5C
U+9735.0	fBig5	F66E
U+9735.0	fGB	3-7450
U+9735.0	fR/S	 173.13
U+9735.0	fTotalStrokes	 21
U+9735.0	fCangjie	MBRJI
U+9735.0	fFourCorner	10153
U+9735.0	fHakka	  sip8;  cip8
U+9735.0	fCantonese	zap1; cap1
U+9735.0	fMandarin	ji2
U+9735.0	fEnglish	[1] the noise of driving rain [2] the din of musical instruments
U+9736.0	fUTF8	霶
U+9736.0	fCNS11643	3-5E72
U+9736.0	fGB	5-7370
U+9736.0	fR/S	 173.13
U+9736.0	fTotalStrokes	 21
U+9736.0	fCangjie	MBEYS
U+9736.0	fFourCorner	10120
U+9736.0	fHakka	  pong2
U+9736.0	fCantonese	pong4; pong1
U+9736.0	fMandarin	pang2; pang1
U+9736.0	fEnglish	 rain cats and dogs
U+9737.0	fUTF8	霷
U+9737.0	fCNS11643	3-5E71
U+9737.0	fGB	E-445E
U+9737.0	fR/S	 173.13
U+9737.0	fTotalStrokes	 21
U+9737.0	fCangjie	MBAAH
U+9737.0	fFourCorner	10620
U+9737.0	fCantonese	joeng4
U+9737.0	fMandarin	yang2
U+9738.0	fUTF8	霸
U+9738.0	fCNS11643	1-7B3D
U+9738.0	fBig5	C551
U+9738.0	fGB	0-3054
U+9738.0	fR/S	 173.13
U+9738.0	fTotalStrokes	 21
U+9738.0	fCangjie	MBTJB
U+9738.0	fFourCorner	10527
U+9738.0	fHakka	  ba5
U+9738.0	fCantonese	baa3; paak3
U+9738.0	fMandarin	ba4; po4
U+9738.0	fEnglish	[1] rule by might rather than right; dominate [2] feudal lord; chief; leader; oppressor [3] something in which one is specially talented or gifted; outstanding
U+9739.0	fUTF8	霹
U+9739.0	fCNS11643	1-7B3E
U+9739.0	fBig5	C552
U+9739.0	fGB	0-4579
U+9739.0	fR/S	 173.13
U+9739.0	fTotalStrokes	 21
U+9739.0	fCangjie	MBSRJ
U+9739.0	fFourCorner	10641
U+9739.0	fHakka	 pit7
U+9739.0	fCantonese	pik1
U+9739.0	fMandarin	pi1; pi4; xian4
U+9739.0	fEnglish	 thunders; crashing thunder; sudden peal of thunder
U+973A.0	fUTF8	霺
U+973A.0	fCNS11643	2-6C5D
U+973A.0	fBig5	F66F
U+973A.0	fGB	E-445F
U+973A.0	fR/S	 173.13
U+973A.0	fTotalStrokes	 21
U+973A.0	fCangjie	MBHOK
U+973A.0	fFourCorner	10248
U+973A.0	fCantonese	mei4
U+973A.0	fMandarin	wei2
U+973B.0	fUTF8	霻
U+973B.0	fCNS11643	3-666E
U+973B.0	fGB	E-4460
U+973B.0	fR/S	 173.13
U+973B.0	fTotalStrokes	 21
U+973B.0	fCangjie	MBTWT
U+973B.0	fFourCorner	1010
U+973B.0	fCantonese	fung1
U+973C.0	fUTF8	霼
U+973C.0	fCNS11643	3-5F71
U+973C.0	fGB	3-4C21
U+973C.0	fR/S	 173.14
U+973C.0	fTotalStrokes	 22
U+973C.0	fCangjie	MIOND
U+973C.0	fFourCorner	18710
U+973C.0	fHakka	  hi5
U+973C.0	fCantonese	hei3
U+973C.0	fMandarin	xi3; xi4
U+973C.0	fEnglish	cloudy
U+973D.0	fUTF8	霽
U+973D.0	fCNS11643	1-7B7E
U+973D.0	fBig5	C5B4
U+973D.0	fGB	1-762B
U+973D.0	fR/S	 173.14
U+973D.0	fTotalStrokes	 22
U+973D.0	fCangjie	MBYX
U+973D.0	fFourCorner	10223
U+973D.0	fHakka	 zi5
U+973D.0	fCantonese	zai3; cai4
U+973D.0	fMandarin	ji4; ji3; pi1
U+973D.0	fEnglish	clear up after rain cease be ang
U+973E.0	fUTF8	霾
U+973E.0	fCNS11643	1-7C21
U+973E.0	fBig5	C5B5
U+973E.0	fGB	0-7632
U+973E.0	fR/S	 173.14
U+973E.0	fTotalStrokes	 22
U+973E.0	fCangjie	MBBHG
U+973E.0	fFourCorner	10214
U+973E.0	fHakka	 mai2; li2
U+973E.0	fCantonese	maai4; mai4 (also); lei4 (also)
U+973E.0	fMandarin	mai2; ji4; li2; mai4
U+973E.0	fEnglish	[1] misty; foggy; cloudy [2] dust storm
U+973F.0	fUTF8	霿
U+973F.0	fCNS11643	2-6E40
U+973F.0	fBig5	F771
U+973F.0	fGB	3-7452
U+973F.0	fR/S	 173.14
U+973F.0	fTotalStrokes	 22
U+973F.0	fCangjie	MBNHU
U+973F.0	fFourCorner	10264
U+973F.0	fCantonese	mung6 (also); mung4
U+973F.0	fMandarin	meng4; meng2; mou4; fu4; mou2
U+9740.0	fUTF8	靀
U+9740.0	fCNS11643	3-5F72
U+9740.0	fGB	5-736F
U+9740.0	fR/S	  173.14
U+9740.0	fTotalStrokes	 22
U+9740.0	fCangjie	MBTBO
U+9740.0	fFourCorner	10230
U+9740.0	fHakka	  mung2
U+9740.0	fCantonese	mung4
U+9740.0	fMandarin	meng2
U+9740.0	fEnglish	 drizzling; fine rain mist
U+9741.0	fUTF8	靁
U+9741.0	fCNS11643	3-6059
U+9741.0	fGB	5-7373
U+9741.0	fR/S	 173.15
U+9741.0	fTotalStrokes	 23
U+9741.0	fCangjie	MBWWW
U+9741.0	fFourCorner	10660
U+9741.0	fCantonese	leoi4
U+9741.0	fMandarin	lei2; lei4
U+9741.0	fEnglish	thick; strong
U+9742.0	fUTF8	靂
U+9742.0	fCNS11643	1-7C70
U+9742.0	fBig5	C645
U+9742.0	fGB	1-7628
U+9742.0	fR/S	 173.16
U+9742.0	fTotalStrokes	 24
U+9742.0	fCangjie	MBMHM
U+9742.0	fFourCorner	10211
U+9742.0	fHakka	  lit8; lak7; let8
U+9742.0	fCantonese	lik1; lik6 (also)
U+9742.0	fMandarin	li4; ling2
U+9742.0	fEnglish	correct form of &#x thunderclap;crashing thunder; sudden peal of thunder
U+9743.0	fUTF8	靃
U+9743.0	fCNS11643	2-705D
U+9743.0	fBig5	F8CF
U+9743.0	fGB	3-7454
U+9743.0	fR/S	 173.16
U+9743.0	fTotalStrokes	 24
U+9743.0	fCangjie	MBOGG
U+9743.0	fFourCorner	10214
U+9743.0	fCantonese	fok3; seoi5; seoi2
U+9743.0	fMandarin	sui3; huo4
U+9744.0	fUTF8	靄
U+9744.0	fCNS11643	1-7C72
U+9744.0	fBig5	C647
U+9744.0	fGB	1-7630
U+9744.0	fR/S	 173.16
U+9744.0	fTotalStrokes	 24
U+9744.0	fCangjie	MBYRV
U+9744.0	fFourCorner	10627
U+9744.0	fHakka	  ai5; oi3
U+9744.0	fCantonese	oi2; ngoi2 (also)
U+9744.0	fMandarin	ai3; ai4; e4; mai2
U+9744.0	fEnglish	[1] mild; amiable; peaceful; kind; amicable; friendly [2] the heaven covered with clouds; a cloudy but bright sky; cloudy; haze; mist [3] a Chinese family name
U+9745.0	fUTF8	靅
U+9745.0	fCNS11643	3-6126
U+9745.0	fGB	3-3355
U+9745.0	fR/S	 173.16
U+9745.0	fTotalStrokes	 24
U+9745.0	fCangjie	MILNC
U+9745.0	fFourCorner	15780
U+9745.0	fCantonese	fai3
U+9745.0	fMandarin	fei4
U+9746.0	fUTF8	靆
U+9746.0	fCNS11643	2-705C
U+9746.0	fBig5	F8CE
U+9746.0	fGB	3-3357
U+9746.0	fR/S	 173.16
U+9746.0	fTotalStrokes	 24
U+9746.0	fCangjie	MIYLE
U+9746.0	fFourCorner	15733
U+9746.0	fHakka	 ti5; tai5; tai6
U+9746.0	fCantonese	doi6
U+9746.0	fMandarin	dai4; ai3
U+9746.0	fEnglish	[1] correct form of &#x [2] cloudy [3] unclear;not clear; obscure; indistinct [4] glasses; spectacles
U+9747.0	fUTF8	靇
U+9747.0	fCNS11643	2-705E
U+9747.0	fBig5	F8D0
U+9747.0	fGB	5-7343
U+9747.0	fR/S	 173.16
U+9747.0	fTotalStrokes	 24
U+9747.0	fCangjie	MBYBP
U+9747.0	fFourCorner	10211
U+9747.0	fCantonese	lung4; ling4
U+9747.0	fMandarin	long2
U+9748.0	fUTF8	靈
U+9748.0	fCNS11643	1-7C71
U+9748.0	fBig5	C646
U+9748.0	fGB	1-4169
U+9748.0	fR/S	 173.16
U+9748.0	fTotalStrokes	 24
U+9748.0	fCangjie	MBRRO
U+9748.0	fFourCorner	10108
U+9748.0	fHakka	  lin2; lin3
U+9748.0	fCantonese	ling4; leng4 (also); ling2 (also)
U+9748.0	fMandarin	ling2; dai4; ling4
U+9748.0	fEnglish	[1] spirit; soul [2] fairy; elf [3] anything pertaining to the deceased [4] wonderful; wonder [5] mysterious; supernatural; divine [6] clever; nimble; sharp; with quick reflexes [7] good; excellent; efficcious; effective; work [8] witchcraft [9] a Chinese family name
U+9748.1	fUTF8	靈
U+9748.1	fCNS11643	1-7C71
U+9748.1	fBig5	C646
U+9748.1	fGB	1-4169
U+9748.1	fR/S	 173.16
U+9748.1	fTotalStrokes	 24
U+9748.1	fCangjie	MBRRO
U+9748.1	fFourCorner	10108
U+9748.1	fMandarin	ling5
U+9749.0	fUTF8	靉
U+9749.0	fCNS11643	2-7146
U+9749.0	fBig5	F957
U+9749.0	fGB	3-3356
U+9749.0	fR/S	 173.17
U+9749.0	fTotalStrokes	 25
U+9749.0	fCangjie	MIBBE
U+9749.0	fFourCorner	12747
U+9749.0	fHakka	  oi5
U+9749.0	fCantonese	oi2; ngoi2 (also); oi3; ngoi3
U+9749.0	fMandarin	ai4; ai3; li4; yi3; yi4
U+9749.0	fEnglish	[1] cloudy; cloudy sky [2] unclear; not clear; obscure; indistinct [3] glasses; spectacles
U+974A.0	fUTF8	靊
U+974A.0	fCNS11643	4-6D64
U+974A.0	fGB	E-4461
U+974A.0	fR/S	 173.18
U+974A.0	fTotalStrokes	 26
U+974A.0	fCangjie	MBUJT
U+974A.0	fFourCorner	10100
U+974A.0	fCantonese	fung1
U+974A.0	fMandarin	feng1
U+974B.0	fUTF8	靋
U+974B.0	fCNS11643	2-717B
U+974B.0	fBig5	F9AD
U+974B.0	fGB	E-4462
U+974B.0	fR/S	 173.19
U+974B.0	fTotalStrokes	 27
U+974B.0	fCangjie	MBEMM
U+974B.0	fFourCorner	10111
U+974B.0	fCantonese	lik6
U+974B.0	fMandarin	li4
U+974C.0	fUTF8	靌
U+974C.0	fGB	E-4463
U+974C.0	fR/S	 173.19
U+974C.0	fTotalStrokes	 27
U+974C.0	fCangjie	MBJMC
U+974C.0	fCantonese	bou2
U+974C.0	fMandarin	bao3
U+974D.0	fUTF8	靍
U+974D.0	fCNS11643	3-666F
U+974D.0	fGB	E-4464
U+974D.0	fR/S	 173.19
U+974D.0	fTotalStrokes	 27
U+974D.0	fCangjie	MBOGF
U+974D.0	fFourCorner	1022
U+974D.0	fCantonese	fok3
U+974E.0	fUTF8	靎
U+974E.0	fGB	E-4465
U+974E.0	fR/S	 173.19
U+974E.0	fTotalStrokes	 27
U+974E.0	fCangjie	MBCHF
U+974E.0	fCantonese	kam4
U+974E.0	fMandarin	he4
U+974F.0	fUTF8	靏
U+974F.0	fGB	E-4466
U+974F.0	fR/S	 173.21
U+974F.0	fTotalStrokes	 29
U+974F.0	fCangjie	XMBOG
U+974F.0	fCantonese	hok6
U+974F.0	fMandarin	he4
U+9750.0	fUTF8	靐
U+9750.0	fCNS11643	3-6245
U+9750.0	fGB	E-4467
U+9750.0	fR/S	 173.31
U+9750.0	fTotalStrokes	 39
U+9750.0	fCangjie	MWMWW
U+9750.0	fFourCorner	10660
U+9750.0	fHakka	 pin2
U+9750.0	fCantonese	bang6
U+9750.0	fMandarin	bing4
U+9750.0	fEnglish	the roar of thunder; the noise of cannonading
U+9751.0	fUTF8	靑
U+9751.0	fCNS11643	3-2A7E
U+9751.0	fGB	E-4468
U+9751.0	fR/S	 174.0
U+9751.0	fTotalStrokes	 8
U+9751.0	fCangjie	QMBLM
U+9751.0	fFourCorner	50220
U+9751.0	fHakka	 ciang1; ciang5
U+9751.0	fCantonese	cing1; ceng1 (also)
U+9751.0	fMandarin	qing1
U+9751.0	fEnglish	blue
U+9752.0	fUTF8	青
U+9752.0	fCNS11643	1-4F65
U+9752.0	fBig5	AB43
U+9752.0	fGB	0-4760
U+9752.0	fR/S	 174.0
U+9752.0	fTotalStrokes	 8
U+9752.0	fCangjie	QMB
U+9752.0	fFourCorner	50227
U+9752.0	fHakka	 ciang1; ziang1
U+9752.0	fCantonese	cing1; ceng1 (also)
U+9752.0	fMandarin	qing1; jing1
U+9752.0	fEnglish	[1] blue; green; black [2] green grass [3] not ripe [4] young; youth; youthful [5] the skin of bamboo [6] correct form of &#xthe white of an egg [7] short for Tsinghai province or Tsingtao [8] a Chinese family name [9] Kangxi radical 174
U+9753.0	fUTF8	靓
U+9753.0	fGB	0-7626
U+9753.0	fR/S	 174.4
U+9753.0	fTotalStrokes	 12
U+9753.0	fCangjie	QBBHU
U+9753.0	fHakka	 ciang5; cin5
U+9753.0	fCantonese	leng3; zing6; leng1 (also)
U+9753.0	fMandarin	jing4; liang4
U+9753.0	fEnglish	simplified form of &#x975A;
U+9754.0	fUTF8	靔
U+9754.0	fGB	8-2E7A
U+9754.0	fR/S	 174.4
U+9754.0	fTotalStrokes	 12
U+9754.0	fCangjie	QBOMN
U+9754.0	fCantonese	tin1
U+9754.0	fMandarin	tian1
U+9755.0	fUTF8	靕
U+9755.0	fCNS11643	3-4549
U+9755.0	fGB	E-4469
U+9755.0	fR/S	  174.5
U+9755.0	fTotalStrokes	 13
U+9755.0	fCangjie	MMQMB
U+9755.0	fFourCorner	15120
U+9755.0	fCantonese	zan1
U+9755.0	fMandarin	zhen1
U+9756.0	fUTF8	靖
U+9756.0	fCNS11643	1-675C
U+9756.0	fBig5	B974
U+9756.0	fGB	0-3E38
U+9756.0	fR/S	 174.5
U+9756.0	fTotalStrokes	 13
U+9756.0	fCangjie	YTQMB
U+9756.0	fFourCorner	05127
U+9756.0	fHakka	  cin5;  zin3; cin3
U+9756.0	fCantonese	zing6
U+9756.0	fMandarin	jing4; jing1
U+9756.0	fEnglish	[1] tranquil; still; calm; peaceful; quiet [2] pacify; quell (uprising, etc.); traquilize [3] order [4] praise in public [5] a Chinese family name
U+9757.0	fUTF8	靗
U+9757.0	fCNS11643	4-4835
U+9757.0	fGB	5-7339
U+9757.0	fR/S	 174.6
U+9757.0	fTotalStrokes	 14
U+9757.0	fCangjie	QBFMU
U+9757.0	fFourCorner	59210
U+9757.0	fHakka	 cang2
U+9757.0	fCantonese	cang1
U+9757.0	fMandarin	cheng4; cheng1
U+9758.0	fUTF8	靘
U+9758.0	fCNS11643	2-4F26
U+9758.0	fBig5	E4BE
U+9758.0	fGB	3-7438
U+9758.0	fR/S	 174.6
U+9758.0	fTotalStrokes	 14
U+9758.0	fCangjie	QBNAU
U+9758.0	fFourCorner	57217
U+9758.0	fHakka	  cin5; tien5
U+9758.0	fCantonese	cing3; cing1
U+9758.0	fMandarin	qing4; qing1
U+9758.0	fEnglish	a dark colour
U+9759.0	fUTF8	静
U+9759.0	fCNS11643	3-4A4C
U+9759.0	fGB	0-3E32
U+9759.0	fR/S	 174.6
U+9759.0	fTotalStrokes	 14
U+9759.0	fCangjie	QBNSD
U+9759.0	fFourCorner	57250
U+9759.0	fHakka	 cin5
U+9759.0	fCantonese	zing6
U+9759.0	fMandarin	jing4; ai4
U+9759.0	fEnglish	simplified form of &#x975C;
U+9759.1	fUTF8	静
U+9759.1	fCNS11643	3-4A4C
U+9759.1	fGB	0-3E32
U+9759.1	fR/S	 174.6
U+9759.1	fTotalStrokes	 14
U+9759.1	fCangjie	QBNSD
U+9759.1	fFourCorner	57250
U+9759.1	fMandarin	jing5
U+975A.0	fUTF8	靚
U+975A.0	fCNS11643	2-5559
U+975A.0	fBig5	E8B0
U+975A.0	fGB	1-7626
U+975A.0	fR/S	 174.7
U+975A.0	fTotalStrokes	 15
U+975A.0	fCangjie	QBBUU
U+975A.0	fFourCorner	56210
U+975A.0	fHakka	  ziang1; liang5;  ciang5; cin5; ciang6; cin6
U+975A.0	fCantonese	leng3; zing6; leng1 (also)
U+975A.0	fMandarin	jing4; liang4
U+975A.0	fEnglish	[1] make up face; ornament; doll up [2] still; quiet; tranquil
U+975A.1	fUTF8	靚
U+975A.1	fCNS11643	2-5559
U+975A.1	fBig5	E8B0
U+975A.1	fGB	1-7626
U+975A.1	fR/S	 174.7
U+975A.1	fTotalStrokes	 15
U+975A.1	fCangjie	QBBUU
U+975A.1	fFourCorner	56210
U+975A.1	fEnglish	 beautiful; nice; pretty
U+975B.0	fUTF8	靛
U+975B.0	fCNS11643	1-727A
U+975B.0	fBig5	C051
U+975B.0	fGB	0-3565
U+975B.0	fR/S	 174.8
U+975B.0	fTotalStrokes	 16
U+975B.0	fCangjie	QBJMO
U+975B.0	fFourCorner	53281
U+975B.0	fHakka	 tien5; tien6; ten3; t'ian6
U+975B.0	fCantonese	din6
U+975B.0	fMandarin	dian4
U+975B.0	fEnglish	indigo; any blue dye; indigo color
U+975C.0	fUTF8	靜
U+975C.0	fCNS11643	1-727B
U+975C.0	fBig5	C052
U+975C.0	fGB	1-3E32
U+975C.0	fR/S	 174.8
U+975C.0	fTotalStrokes	 16
U+975C.0	fCangjie	QBBSD
U+975C.0	fFourCorner	52257
U+975C.0	fHakka	 cin3; cin6;  cin5
U+975C.0	fCantonese	zing6
U+975C.0	fMandarin	jing4; ai4
U+975C.0	fEnglish	[1] quiet(ly); still; motionless; gentle; calm; tranquility; silent [2] peaceful; harmonious; serene [3] virtuous; chaste
U+975C.1	fUTF8	靜
U+975C.1	fCNS11643	1-727B
U+975C.1	fBig5	C052
U+975C.1	fGB	1-3E32
U+975C.1	fR/S	 174.8
U+975C.1	fTotalStrokes	 16
U+975C.1	fCangjie	QBBSD
U+975C.1	fFourCorner	52257
U+975D.0	fUTF8	靝
U+975D.0	fCNS11643	3-594F
U+975D.0	fGB	5-7338
U+975D.0	fR/S	 174.10
U+975D.0	fTotalStrokes	 18
U+975D.0	fCangjie	QBOND
U+975D.0	fFourCorner	58210
U+975D.0	fHakka	 tien1
U+975D.0	fCantonese	tin1
U+975D.0	fMandarin	tian1
U+975D.0	fEnglish	the heaven (Taoism)
U+975E.0	fUTF8	非
U+975E.0	fCNS11643	1-4F66
U+975E.0	fBig5	AB44
U+975E.0	fGB	0-3747
U+975E.0	fR/S	 175.0
U+975E.0	fTotalStrokes	 8
U+975E.0	fCangjie	LMSY
U+975E.0	fFourCorner	11111
U+975E.0	fHakka	  fui1; fi1
U+975E.0	fCantonese	fei1; fei2; fai1
U+975E.0	fMandarin	fei1; fei3; fei4
U+975E.0	fEnglish	[1] not; negative; non-; not to be [2] faults; mistakes; evils; wrong [3] object; refute; consider as wrong; censure [4] short for Africa
U+975F.0	fUTF8	靟
U+975F.0	fCNS11643	3-4021
U+975F.0	fGB	3-745D
U+975F.0	fR/S	 175.4
U+975F.0	fTotalStrokes	 12
U+975F.0	fCangjie	LYHQU
U+975F.0	fFourCorner	11710
U+975F.0	fCantonese	fei1
U+975F.0	fMandarin	fei1
U+9760.0	fUTF8	靠
U+9760.0	fCNS11643	1-6F6A
U+9760.0	fBig5	BE61
U+9760.0	fGB	0-3F3F
U+9760.0	fR/S	 175.7
U+9760.0	fTotalStrokes	 15
U+9760.0	fCangjie	HGRLY
U+9760.0	fFourCorner	24111
U+9760.0	fHakka	  kau5; ko5
U+9760.0	fCantonese	kaau3
U+9760.0	fMandarin	kao4; gu4; jing2; ku4
U+9760.0	fEnglish	[1] rely on; depend on [2] lean on [3] near to; bordering on; keep to (the left or the right), as in driving [4] make-believer armor worn by actors (Chinese opera) [5] almost
U+9761.0	fUTF8	靡
U+9761.0	fCNS11643	1-7948
U+9761.0	fBig5	C3FB
U+9761.0	fGB	0-4352
U+9761.0	fR/S	 175.11
U+9761.0	fTotalStrokes	 19
U+9761.0	fCangjie	IDLMY
U+9761.0	fFourCorner	00211
U+9761.0	fHakka	 mi2;  mi3
U+9761.0	fCantonese	mei5; mei4
U+9761.0	fMandarin	mi3; ma2; mo2; mei4; mei2
U+9761.0	fEnglish	[1] divide; disperse; scatter; blown away by the wind [2] lean with pressure [3] not; no; negative [4] tiny; pety; small [5] wonderful; good; excellent
U+9761.1	fUTF8	靡
U+9761.1	fCNS11643	1-7948
U+9761.1	fBig5	C3FB
U+9761.1	fGB	0-4352
U+9761.1	fR/S	 175.11
U+9761.1	fTotalStrokes	 19
U+9761.1	fCangjie	IDLMY
U+9761.1	fFourCorner	00211
U+9761.1	fMandarin	mi2
U+9761.1	fEnglish	[1] waste; extravagant [2] rot
U+9762.0	fUTF8	面
U+9762.0	fCNS11643	1-5372
U+9762.0	fBig5	ADB1
U+9762.0	fGB	0-4366
U+9762.0	fR/S	 176.0
U+9762.0	fTotalStrokes	 9
U+9762.0	fCangjie	MWSL
U+9762.0	fFourCorner	10600
U+9762.0	fHakka	 mien5; men5; mian5
U+9762.0	fCantonese	min6; min2 (also)
U+9762.0	fMandarin	mian4
U+9762.0	fEnglish	[1] face of a person [2] surface; plane; top; face [3] side; direction; aspect [4] extent; range; scale; scope [5] face; confront; look [6] face-to-face; in or to one's face; personally; directly [7] plane (mathematics) [8] indicate something flat
U+9762.1	fUTF8	面
U+9762.1	fCNS11643	1-5372
U+9762.1	fBig5	ADB1
U+9762.1	fGB	0-4366
U+9762.1	fR/S	 176.0
U+9762.1	fTotalStrokes	 9
U+9762.1	fCangjie	MWSL
U+9762.1	fFourCorner	10600
U+9762.1	fMandarin	mian5
U+9763.0	fUTF8	靣
U+9763.0	fCNS11643	3-2B21
U+9763.0	fGB	E-446A
U+9763.0	fR/S	 176.0
U+9763.0	fTotalStrokes	 8; 9
U+9763.0	fCangjie	MWR
U+9763.0	fFourCorner	10600
U+9763.0	fHakka	 mien5
U+9763.0	fCantonese	min6
U+9763.0	fMandarin	mian4
U+9764.0	fUTF8	靤
U+9764.0	fCNS11643	3-4A4E
U+9764.0	fGB	5-7761
U+9764.0	fR/S	 176.5
U+9764.0	fTotalStrokes	 14
U+9764.0	fCangjie	MWPRU
U+9764.0	fFourCorner	17610
U+9764.0	fCantonese	paau3
U+9764.0	fMandarin	pao4; kao4
U+9765.0	fUTF8	靥
U+9765.0	fGB	0-584C
U+9765.0	fR/S	 176.6; 27.13
U+9765.0	fTotalStrokes	 15
U+9765.0	fCangjie	MIKW
U+9765.0	fHakka	 ngiap7; jam3; jap8
U+9765.0	fCantonese	jip3; jim2; jip6
U+9765.0	fMandarin	ye4; tian3; yan3
U+9765.0	fEnglish	simplified form of &#x9768;
U+9766.0	fUTF8	靦
U+9766.0	fCNS11643	1-727C
U+9766.0	fBig5	C053
U+9766.0	fGB	3-795F
U+9766.0	fR/S	 176.7
U+9766.0	fTotalStrokes	 16
U+9766.0	fCangjie	MWBUU
U+9766.0	fFourCorner	16610
U+9766.0	fHakka	  tien3; dien3; ten3; men3
U+9766.0	fCantonese	tin2; min5
U+9766.0	fMandarin	tian3; pao4
U+9766.0	fEnglish	ashamed and embarrassed
U+9766.1	fUTF8	靦
U+9766.1	fCNS11643	1-727C
U+9766.1	fBig5	C053
U+9766.1	fGB	3-795F
U+9766.1	fR/S	 176.7
U+9766.1	fTotalStrokes	 16
U+9766.1	fCangjie	MWBUU
U+9766.1	fFourCorner	16610
U+9766.1	fMandarin	mian3
U+9766.1	fEnglish	timid; shy; bashful
U+9767.0	fUTF8	靧
U+9767.0	fCNS11643	3-5E73
U+9767.0	fGB	3-7965
U+9767.0	fR/S	 176.12
U+9767.0	fTotalStrokes	 21
U+9767.0	fCangjie	MWLMC
U+9767.0	fFourCorner	15680
U+9767.0	fHakka	 fui5; fi5
U+9767.0	fCantonese	fui3
U+9767.0	fMandarin	hui4
U+9767.0	fEnglish	 wash one's face
U+9768.0	fUTF8	靨
U+9768.0	fCNS11643	1-7C4E
U+9768.0	fBig5	C5E2
U+9768.0	fGB	1-584C
U+9768.0	fR/S	 176.14
U+9768.0	fTotalStrokes	 23
U+9768.0	fCangjie	MKMWL
U+9768.0	fFourCorner	71601
U+9768.0	fHakka	 jam3; ap7; jap8; ram3
U+9768.0	fCantonese	jip3; jim2; jip6
U+9768.0	fMandarin	ye4; yan3; tian3
U+9768.0	fEnglish	dimples in the face
U+9769.0	fUTF8	革
U+9769.0	fCNS11643	1-5373
U+9769.0	fBig5	ADB2
U+9769.0	fGB	0-386F
U+9769.0	fR/S	 177.0
U+9769.0	fTotalStrokes	 9
U+9769.0	fCangjie	TLJ
U+9769.0	fFourCorner	44506
U+9769.0	fHakka	 get7;  gak7;  giet7; gap7
U+9769.0	fCantonese	gaak3; gaap3 (also); gik1
U+9769.0	fMandarin	ge2; ji3
U+9769.0	fEnglish	[1] leather; animal hides stripped of hair; skin [2] get rid of; eliminate [3] human skin [4] change; reform; renovate [5] Chinese percussion musical instruments [6] armor [7] soldiers [8] one of the eight Diagrams [9] a Chinese family name [10] Kangxi radical 177
U+9769.1	fUTF8	革
U+9769.1	fCNS11643	1-5373
U+9769.1	fBig5	ADB2
U+9769.1	fGB	0-386F
U+9769.1	fR/S	 177.0
U+9769.1	fTotalStrokes	 9
U+9769.1	fCangjie	TLJ
U+9769.1	fFourCorner	44506
U+9769.1	fMandarin	ji2
U+9769.1	fEnglish	urgent; dangerous
U+976A.0	fUTF8	靪
U+976A.0	fCNS11643	2-3A31
U+976A.0	fBig5	D84D
U+976A.0	fGB	5-7722
U+976A.0	fR/S	 177.2
U+976A.0	fTotalStrokes	 11
U+976A.0	fCangjie	TJMN
U+976A.0	fFourCorner	41420
U+976A.0	fHakka	  den1; dang1
U+976A.0	fCantonese	ding1
U+976A.0	fMandarin	ding1; di4; ding4
U+976A.0	fEnglish	[1] mend the soles of shoes [2] patch
U+976B.0	fUTF8	靫
U+976B.0	fCNS11643	4-3B77
U+976B.0	fGB	3-7923
U+976B.0	fR/S	 177.3
U+976B.0	fTotalStrokes	 12
U+976B.0	fCangjie	TJEI
U+976B.0	fFourCorner	47540
U+976B.0	fHakka	 cai1; ca1; ca5
U+976B.0	fCantonese	caai1; caa1 (also)
U+976B.0	fMandarin	cha1; cha2; le1; chai1
U+976B.0	fEnglish	strong and pliable; resilient
U+976C.0	fUTF8	靬
U+976C.0	fCNS11643	2-414E
U+976C.0	fBig5	DCA9
U+976C.0	fGB	3-787B
U+976C.0	fR/S	 177.3
U+976C.0	fTotalStrokes	 12
U+976C.0	fCangjie	TJMJ
U+976C.0	fFourCorner	41540
U+976C.0	fCantonese	gin1
U+976C.0	fMandarin	jian1; kan1; kan4; qian2
U+976D.0	fUTF8	靭
U+976D.0	fCNS11643	3-4024
U+976D.0	fGB	E-446B
U+976D.0	fR/S	 177.3
U+976D.0	fTotalStrokes	 12
U+976D.0	fCangjie	TJSHI
U+976D.0	fFourCorner	47520
U+976D.0	fHakka	 ngiun3
U+976D.0	fCantonese	jan6; ngan6 (also); an6 (also)
U+976D.0	fMandarin	ren4
U+976D.0	fEnglish	strong and pliable; resilient
U+976E.0	fUTF8	靮
U+976E.0	fCNS11643	2-4150
U+976E.0	fBig5	DCAB
U+976E.0	fGB	3-7922
U+976E.0	fR/S	 177.3
U+976E.0	fTotalStrokes	 12
U+976E.0	fCangjie	TJPI
U+976E.0	fFourCorner	47420
U+976E.0	fHakka	 dit7
U+976E.0	fCantonese	dik1
U+976E.0	fMandarin	di2
U+976E.0	fEnglish	bridle
U+976F.0	fUTF8	靯
U+976F.0	fCNS11643	3-4022
U+976F.0	fGB	3-787D
U+976F.0	fR/S	 177.3
U+976F.0	fTotalStrokes	 12
U+976F.0	fCangjie	TJG
U+976F.0	fFourCorner	44510
U+976F.0	fCantonese	dou6
U+976F.0	fMandarin	du4
U+9770.0	fUTF8	靰
U+9770.0	fCNS11643	2-414F
U+9770.0	fBig5	DCAA
U+9770.0	fGB	3-787E
U+9770.0	fR/S	 177.3
U+9770.0	fTotalStrokes	 12
U+9770.0	fCangjie	TJMU
U+9770.0	fFourCorner	41512
U+9770.0	fCantonese	wu1
U+9770.0	fMandarin	wu4
U+9770.0	fEnglish	leg warmer
U+9771.0	fUTF8	靱
U+9771.0	fGB	E-446C
U+9771.0	fR/S	 177.3
U+9771.0	fTotalStrokes	 12
U+9771.0	fCangjie	TJSK
U+9771.0	fHakka	 ngiun3; niun5; ngiun5
U+9771.0	fCantonese	jan6; ngan6 (also); an6 (also)
U+9771.0	fMandarin	ren4; cha4; chai1
U+9771.0	fEnglish	strong and pliable; resilient
U+9772.0	fUTF8	靲
U+9772.0	fCNS11643	2-4863
U+9772.0	fBig5	E0DD
U+9772.0	fGB	3-7926
U+9772.0	fR/S	 177.4
U+9772.0	fTotalStrokes	 13
U+9772.0	fCangjie	TJOIN
U+9772.0	fFourCorner	48527
U+9772.0	fCantonese	kam4
U+9772.0	fMandarin	qin2; jin4; qian2
U+9773.0	fUTF8	靳
U+9773.0	fCNS11643	2-4860
U+9773.0	fBig5	E0DA
U+9773.0	fGB	0-3D79
U+9773.0	fR/S	 177.4
U+9773.0	fTotalStrokes	 13
U+9773.0	fCangjie	TJHML
U+9773.0	fFourCorner	42521
U+9773.0	fHakka	  giun3
U+9773.0	fCantonese	gan3
U+9773.0	fMandarin	jin4
U+9773.0	fEnglish	[1] ornamental trappings under the neck of a horse [2] stingy [3] a Chinese family name
U+9774.0	fUTF8	靴
U+9774.0	fCNS11643	1-675D
U+9774.0	fBig5	B975
U+9774.0	fGB	0-5125
U+9774.0	fR/S	 177.4
U+9774.0	fTotalStrokes	 13
U+9774.0	fCangjie	TJOP
U+9774.0	fFourCorner	44510
U+9774.0	fHakka	  hio1
U+9774.0	fCantonese	hoe1
U+9774.0	fMandarin	xue1
U+9774.0	fEnglish	boots
U+9775.0	fUTF8	靵
U+9775.0	fCNS11643	6-5A24
U+9775.0	fGB	E-446D
U+9775.0	fR/S	 177.4
U+9775.0	fTotalStrokes	 13
U+9775.0	fCangjie	TJNG
U+9775.0	fFourCorner	47510
U+9775.0	fHakka	 niu3
U+9775.0	fCantonese	nau2; lau2 (also)
U+9775.0	fMandarin	niu3
U+9776.0	fUTF8	靶
U+9776.0	fCNS11643	1-675E
U+9776.0	fBig5	B976
U+9776.0	fGB	0-3050
U+9776.0	fR/S	 177.4
U+9776.0	fTotalStrokes	 13
U+9776.0	fCangjie	TJAU
U+9776.0	fFourCorner	47517
U+9776.0	fHakka	 ba3
U+9776.0	fCantonese	baa2; baa3
U+9776.0	fMandarin	ba3; ba4
U+9776.0	fEnglish	[1] target [2] splashboard of a chariot
U+9777.0	fUTF8	靷
U+9777.0	fCNS11643	2-4861
U+9777.0	fBig5	E0DB
U+9777.0	fGB	3-7929
U+9777.0	fR/S	 177.4
U+9777.0	fTotalStrokes	 13
U+9777.0	fCangjie	TJNL
U+9777.0	fFourCorner	42500
U+9777.0	fHakka	 jin3; rin3
U+9777.0	fCantonese	jan5
U+9777.0	fMandarin	yin3; yin4; zhen4
U+9777.0	fEnglish	the leather belts that connect a cart with the horse, etc.
U+9778.0	fUTF8	靸
U+9778.0	fCNS11643	2-4862
U+9778.0	fBig5	E0DC
U+9778.0	fGB	3-7921
U+9778.0	fR/S	 177.4
U+9778.0	fTotalStrokes	 13
U+9778.0	fCangjie	TJNHE
U+9778.0	fFourCorner	47547
U+9778.0	fHakka	  sap7; cap8
U+9778.0	fCantonese	saap3; taat3; cap1
U+9778.0	fMandarin	sa3
U+9778.0	fEnglish	 child's shoes
U+9778.1	fUTF8	靸
U+9778.1	fCNS11643	2-4862
U+9778.1	fBig5	E0DC
U+9778.1	fGB	3-7921
U+9778.1	fR/S	 177.4
U+9778.1	fTotalStrokes	 13
U+9778.1	fCangjie	TJNHE
U+9778.1	fFourCorner	47547
U+9778.1	fMandarin	ta1
U+9778.1	fEnglish	 wear shoes in a casual way
U+9779.0	fUTF8	靹
U+9779.0	fCNS11643	3-454B
U+9779.0	fGB	E-446E
U+9779.0	fR/S	 177.4
U+9779.0	fTotalStrokes	 13
U+9779.0	fCangjie	TJOB
U+9779.0	fFourCorner	44520
U+9779.0	fCantonese	noi6; loi6 (also)
U+9779.0	fMandarin	na4; ren4
U+977A.0	fUTF8	靺
U+977A.0	fCNS11643	2-4F28
U+977A.0	fBig5	E4C0
U+977A.0	fGB	3-792A
U+977A.0	fR/S	 177.5
U+977A.0	fTotalStrokes	 14
U+977A.0	fCangjie	TJDJ
U+977A.0	fFourCorner	45590
U+977A.0	fHakka	 mat7
U+977A.0	fCantonese	mut6; maat6
U+977A.0	fMandarin	mo4
U+977A.0	fEnglish	[1] stocking [2] the Tunggusic tribe
U+977B.0	fUTF8	靻
U+977B.0	fCNS11643	2-4F2D
U+977B.0	fBig5	E4C5
U+977B.0	fGB	3-792C
U+977B.0	fR/S	 177.5
U+977B.0	fTotalStrokes	 14
U+977B.0	fCangjie	TJBM
U+977B.0	fFourCorner	47510
U+977B.0	fCantonese	zou2
U+977B.0	fMandarin	zu3
U+977C.0	fUTF8	靼
U+977C.0	fCNS11643	1-6B67
U+977C.0	fBig5	BBDE
U+977C.0	fGB	0-7730
U+977C.0	fR/S	 177.5
U+977C.0	fTotalStrokes	 14
U+977C.0	fCangjie	TJAM
U+977C.0	fFourCorner	46510
U+977C.0	fHakka	  dat7; tan3
U+977C.0	fCantonese	daat3
U+977C.0	fMandarin	da2; tan3; yang1; yang3; zhe2
U+977C.0	fEnglish	[1] correct form of &#xthe Tartars [2] soft leather;
U+977D.0	fUTF8	靽
U+977D.0	fCNS11643	2-4F27
U+977D.0	fBig5	E4BF
U+977D.0	fGB	3-792E
U+977D.0	fR/S	 177.5
U+977D.0	fTotalStrokes	 14
U+977D.0	fCangjie	TJFQ
U+977D.0	fFourCorner	49450
U+977D.0	fHakka	  ban5; pan5
U+977D.0	fCantonese	bun3; bun6
U+977D.0	fMandarin	ban4
U+977D.0	fEnglish	[1] ropes [2] traces
U+977E.0	fUTF8	靾
U+977E.0	fCNS11643	2-4F29
U+977E.0	fBig5	E4C1
U+977E.0	fGB	5-7729
U+977E.0	fR/S	 177.5
U+977E.0	fTotalStrokes	 14
U+977E.0	fCangjie	TJPT
U+977E.0	fFourCorner	44517
U+977E.0	fHakka	 siet7; ji5; ri5
U+977E.0	fCantonese	sit3 (also); jai6; ji6
U+977E.0	fMandarin	yi4; xie4; xi4
U+977E.0	fEnglish	 paper saddle, to be burnt for the dead
U+977F.0	fUTF8	靿
U+977F.0	fCNS11643	2-4F30
U+977F.0	fBig5	E4C8
U+977F.0	fGB	3-7932
U+977F.0	fR/S	 177.5
U+977F.0	fTotalStrokes	 14
U+977F.0	fCangjie	TJVIS
U+977F.0	fFourCorner	44527
U+977F.0	fHakka	 jau5; rau5; jeu5
U+977F.0	fCantonese	aau3; ngaau3 (also); aau2
U+977F.0	fMandarin	yao4
U+977F.0	fEnglish	the leg of a boot
U+9780.0	fUTF8	鞀
U+9780.0	fCNS11643	2-4F2B
U+9780.0	fBig5	E4C3
U+9780.0	fGB	5-7731
U+9780.0	fR/S	 177.5
U+9780.0	fTotalStrokes	 14
U+9780.0	fCangjie	TJSHR
U+9780.0	fFourCorner	47562
U+9780.0	fCantonese	tou4
U+9780.0	fMandarin	tao2
U+9780.0	fEnglish	drum
U+9781.0	fUTF8	鞁
U+9781.0	fCNS11643	2-4F2F
U+9781.0	fBig5	E4C7
U+9781.0	fGB	3-7931
U+9781.0	fR/S	 177.5
U+9781.0	fTotalStrokes	 14
U+9781.0	fCangjie	TJDHE
U+9781.0	fFourCorner	44547
U+9781.0	fHakka	 pi5
U+9781.0	fCantonese	bei6; pei5
U+9781.0	fMandarin	tuo2; bei4; bai4; bi4; da2; pi1; pi4
U+9782.0	fUTF8	鞂
U+9782.0	fCNS11643	2-4F2C
U+9782.0	fBig5	E4C4
U+9782.0	fGB	E-446F
U+9782.0	fR/S	 177.5
U+9782.0	fTotalStrokes	 14
U+9782.0	fCangjie	TJHD
U+9782.0	fFourCorner	42594
U+9782.0	fCantonese	gaai1; gaat3
U+9782.0	fMandarin	jia2
U+9783.0	fUTF8	鞃
U+9783.0	fCNS11643	2-4F2A
U+9783.0	fBig5	E4C2
U+9783.0	fGB	3-7930
U+9783.0	fR/S	 177.5
U+9783.0	fTotalStrokes	 14
U+9783.0	fCangjie	TJNI
U+9783.0	fFourCorner	42530
U+9783.0	fHakka	 fen2
U+9783.0	fCantonese	wang4
U+9783.0	fMandarin	hong2; gong1; kong1; qiong1
U+9783.0	fEnglish	board
U+9784.0	fUTF8	鞄
U+9784.0	fCNS11643	2-4F2E
U+9784.0	fBig5	E4C6
U+9784.0	fGB	3-792D
U+9784.0	fR/S	 177.5
U+9784.0	fTotalStrokes	 14
U+9784.0	fCangjie	TJPRU
U+9784.0	fFourCorner	47512
U+9784.0	fHakka	 pau2; pok7; bau1
U+9784.0	fCantonese	pok3; paau4 (also); baau6; paai4; paau6
U+9784.0	fMandarin	pao2; bao4; bao1; bo2; po4; pu2; pu3
U+9784.0	fEnglish	 work hides leather bag
U+9785.0	fUTF8	鞅
U+9785.0	fCNS11643	1-6B68
U+9785.0	fBig5	BBDF
U+9785.0	fGB	0-7731
U+9785.0	fR/S	 177.5
U+9785.0	fTotalStrokes	 14
U+9785.0	fCangjie	TJLBK
U+9785.0	fFourCorner	45530
U+9785.0	fHakka	  jong3; jong1; rong3; rong1
U+9785.0	fCantonese	joeng1; joeng2 (also); joeng3; jeon2
U+9785.0	fMandarin	yang1; yang3; obs; yang4
U+9785.0	fEnglish	leather strap over horse's neck
U+9786.0	fUTF8	鞆
U+9786.0	fGB	E-4470
U+9786.0	fR/S	 177.5
U+9786.0	fTotalStrokes	 14
U+9786.0	fCangjie	TJMOB
U+9786.0	fCantonese	bing2
U+9786.0	fMandarin	mo4
U+9786.0	fEnglish	an archer's arm-protector
U+9787.0	fUTF8	鞇
U+9787.0	fCNS11643	3-4F62
U+9787.0	fGB	3-7935
U+9787.0	fR/S	  177.6
U+9787.0	fTotalStrokes	 15
U+9787.0	fCangjie	TJWK
U+9787.0	fFourCorner	46500
U+9787.0	fHakka	 jin1; rin1
U+9787.0	fCantonese	jan1
U+9787.0	fMandarin	yin1
U+9787.0	fEnglish	[1] mat; mattress [2] underclothes
U+9788.0	fUTF8	鞈
U+9788.0	fCNS11643	2-555C
U+9788.0	fBig5	E8B3
U+9788.0	fGB	3-7937
U+9788.0	fR/S	 177.6
U+9788.0	fTotalStrokes	 15
U+9788.0	fCangjie	TJOMR
U+9788.0	fFourCorner	48561
U+9788.0	fHakka	 giap7; kap7
U+9788.0	fCantonese	gaap3
U+9788.0	fMandarin	jia2; ge2; ta4; sa3
U+9788.0	fEnglish	[1] knee-cap of leather [2] white girdle worn by mourners
U+9789.0	fUTF8	鞉
U+9789.0	fCNS11643	3-4F63
U+9789.0	fGB	5-7737
U+9789.0	fR/S	 177.6
U+9789.0	fTotalStrokes	 15
U+9789.0	fCangjie	TJLMO
U+9789.0	fFourCorner	42510
U+9789.0	fHakka	  tau2; to2
U+9789.0	fCantonese	tou4
U+9789.0	fMandarin	tao2
U+9789.0	fEnglish	drum
U+978A.0	fUTF8	鞊
U+978A.0	fCNS11643	2-555A
U+978A.0	fBig5	E8B1
U+978A.0	fGB	5-7732
U+978A.0	fR/S	 177.6
U+978A.0	fTotalStrokes	 15
U+978A.0	fCangjie	TJGR
U+978A.0	fFourCorner	44561
U+978A.0	fCantonese	kit3
U+978A.0	fMandarin	ji2
U+978B.0	fUTF8	鞋
U+978B.0	fCNS11643	1-6F6C
U+978B.0	fBig5	BE63
U+978B.0	fGB	0-502C
U+978B.0	fR/S	 177.6
U+978B.0	fTotalStrokes	 15
U+978B.0	fCangjie	TJGG
U+978B.0	fFourCorner	44514
U+978B.0	fHakka	  hai2;  he2; hai3
U+978B.0	fCantonese	haai4; haai2 (also)
U+978B.0	fMandarin	xie2; gua1; wa1; xi1
U+978B.0	fEnglish	shoes; footwear in general
U+978C.0	fUTF8	鞌
U+978C.0	fCNS11643	3-4F64
U+978C.0	fGB	E-4471
U+978C.0	fR/S	 177.6; 177.12; 40.12
U+978C.0	fTotalStrokes	 15
U+978C.0	fCangjie	JVTJ
U+978C.0	fFourCorner	30500
U+978C.0	fCantonese	on1; ngon1 (also)
U+978C.0	fMandarin	an1
U+978D.0	fUTF8	鞍
U+978D.0	fCNS11643	1-6F6B
U+978D.0	fBig5	BE62
U+978D.0	fGB	0-3030
U+978D.0	fR/S	 177.6
U+978D.0	fTotalStrokes	 15
U+978D.0	fCangjie	TJJV
U+978D.0	fFourCorner	43544
U+978D.0	fHakka	  on1
U+978D.0	fCantonese	on1; ngon1 (also)
U+978D.0	fMandarin	an1
U+978D.0	fEnglish	 saddle; any saddle-shaped object or terrain
U+978E.0	fUTF8	鞎
U+978E.0	fCNS11643	2-555B
U+978E.0	fBig5	E8B2
U+978E.0	fGB	E-4472
U+978E.0	fR/S	 177.6
U+978E.0	fTotalStrokes	 15
U+978E.0	fCangjie	TJAV
U+978E.0	fFourCorner	47532
U+978E.0	fCantonese	han4
U+978E.0	fMandarin	hen2; gen4; hen4; hun2; hun4; jian1; yin2
U+978F.0	fUTF8	鞏
U+978F.0	fCNS11643	1-6F6D
U+978F.0	fBig5	BE64
U+978F.0	fGB	1-392E
U+978F.0	fR/S	 177.6
U+978F.0	fTotalStrokes	 15
U+978F.0	fCangjie	MNTLJ
U+978F.0	fFourCorner	17506
U+978F.0	fHakka	 giung3;  kiung3
U+978F.0	fCantonese	gung2
U+978F.0	fMandarin	gong3; xie2
U+978F.0	fEnglish	[1] tie with thongs; bind with thongs [2] firm; secure; strong; guard; secure; strengthen [3] a Chinese family name
U+9790.0	fUTF8	鞐
U+9790.0	fGB	E-4473
U+9790.0	fR/S	 177.6
U+9790.0	fTotalStrokes	 15
U+9790.0	fCangjie	TJYMY
U+9790.0	fCantonese	lung6
U+9790.0	fMandarin	gong3
U+9791.0	fUTF8	鞑
U+9791.0	fGB	0-7732
U+9791.0	fR/S	 177.6
U+9791.0	fTotalStrokes	 15
U+9791.0	fCangjie	TJYK
U+9791.0	fHakka	 dat7
U+9791.0	fCantonese	taat3; daat3 (also)
U+9791.0	fMandarin	da2; bei4; ta4
U+9791.0	fEnglish	simplified form of &#x97C3;
U+9792.0	fUTF8	鞒
U+9792.0	fGB	0-7733
U+9792.0	fR/S	 177.6
U+9792.0	fTotalStrokes	 15
U+9792.0	fCangjie	TJHKL
U+9792.0	fHakka	  kiau1
U+9792.0	fCantonese	kiu4
U+9792.0	fMandarin	qiao1; qiao2; jue1
U+9792.0	fEnglish	simplified form of &#x97BD;
U+9793.0	fUTF8	鞓
U+9793.0	fCNS11643	3-5353
U+9793.0	fGB	3-7939
U+9793.0	fR/S	 177.7
U+9793.0	fTotalStrokes	 16
U+9793.0	fCangjie	TJRMG
U+9793.0	fFourCorner	46514
U+9793.0	fCantonese	ting1
U+9793.0	fMandarin	ting1
U+9794.0	fUTF8	鞔
U+9794.0	fCNS11643	2-5B62
U+9794.0	fBig5	EC56
U+9794.0	fGB	0-7734
U+9794.0	fR/S	 177.7
U+9794.0	fTotalStrokes	 16
U+9794.0	fCangjie	TJNAU
U+9794.0	fFourCorner	47526
U+9794.0	fHakka	 man2
U+9794.0	fCantonese	mun4; mun6 (also); mong1
U+9794.0	fMandarin	wan3; man2; men3; men4
U+9794.0	fEnglish	sides or uppers of shoes to stretch a skin on a frame for a drum
U+9795.0	fUTF8	鞕
U+9795.0	fCNS11643	4-545F
U+9795.0	fGB	5-773D
U+9795.0	fR/S	 177.7
U+9795.0	fTotalStrokes	 16
U+9795.0	fCangjie	TJMLK
U+9795.0	fFourCorner	41540
U+9795.0	fHakka	 ngang5; ngang6
U+9795.0	fCantonese	ngaang6; aang6 (also)
U+9795.0	fMandarin	ying4
U+9795.0	fEnglish	[1] hard [2] obstinate
U+9796.0	fUTF8	鞖
U+9796.0	fCNS11643	4-5467
U+9796.0	fGB	E-4474
U+9796.0	fR/S	 177.7
U+9796.0	fTotalStrokes	 16
U+9796.0	fCangjie	TJBV
U+9796.0	fFourCorner	42540
U+9796.0	fCantonese	seoi1
U+9796.0	fMandarin	sui1
U+9797.0	fUTF8	鞗
U+9797.0	fCNS11643	2-5B61
U+9797.0	fBig5	EC55
U+9797.0	fGB	5-3275
U+9797.0	fR/S	 177.7
U+9797.0	fTotalStrokes	 16
U+9797.0	fCangjie	OLOJ
U+9797.0	fFourCorner	27256
U+9797.0	fHakka	  tiau2
U+9797.0	fCantonese	tiu4
U+9797.0	fMandarin	tiao2
U+9797.0	fEnglish	reins
U+9798.0	fUTF8	鞘
U+9798.0	fCNS11643	1-727D
U+9798.0	fBig5	C054
U+9798.0	fGB	0-474A
U+9798.0	fR/S	 177.7
U+9798.0	fTotalStrokes	 16
U+9798.0	fCangjie	TJFB
U+9798.0	fFourCorner	49527
U+9798.0	fHakka	  siau5; siau1; sau1; seu5; ciau5; sau5
U+9798.0	fCantonese	ciu3; saau1; sau1 (also)
U+9798.0	fMandarin	qiao4; shao1; xiao4
U+9798.0	fEnglish	 scabbard; sheath
U+9799.0	fUTF8	鞙
U+9799.0	fCNS11643	2-5B60
U+9799.0	fBig5	EC54
U+9799.0	fGB	3-793A
U+9799.0	fR/S	 177.7
U+9799.0	fTotalStrokes	 16
U+9799.0	fCangjie	TJRB
U+9799.0	fFourCorner	46527
U+9799.0	fHakka	 gien1; gien3; gian1; hien3; gen1; gen3
U+9799.0	fCantonese	gyun1; jyun5
U+9799.0	fMandarin	xuan4; juan1; gui3; xuan1; yuan1
U+9799.0	fEnglish	fine
U+979A.0	fUTF8	鞚
U+979A.0	fCNS11643	2-6049
U+979A.0	fBig5	EEFC
U+979A.0	fGB	3-7942
U+979A.0	fR/S	 177.8
U+979A.0	fTotalStrokes	 17
U+979A.0	fCangjie	TJJCM
U+979A.0	fFourCorner	43511
U+979A.0	fHakka	  kung5; kiong5
U+979A.0	fCantonese	hung3
U+979A.0	fMandarin	kong4
U+979A.0	fEnglish	bridle
U+979B.0	fUTF8	鞛
U+979B.0	fCNS11643	3-567A
U+979B.0	fGB	5-7745
U+979B.0	fR/S	 177.8
U+979B.0	fTotalStrokes	 17
U+979B.0	fCangjie	TJYTR
U+979B.0	fFourCorner	40560
U+979B.0	fCantonese	bung2
U+979B.0	fMandarin	beng3
U+979C.0	fUTF8	鞜
U+979C.0	fCNS11643	2-604B
U+979C.0	fBig5	EEFE
U+979C.0	fGB	3-7946
U+979C.0	fR/S	 177.8
U+979C.0	fTotalStrokes	 17
U+979C.0	fCangjie	TJEA
U+979C.0	fFourCorner	42563
U+979C.0	fHakka	 tap8; tap7
U+979C.0	fCantonese	daap6
U+979C.0	fMandarin	ta4
U+979C.0	fEnglish	shoes, boots
U+979D.0	fUTF8	鞝
U+979D.0	fCNS11643	2-604D
U+979D.0	fBig5	EF41
U+979D.0	fGB	5-7743
U+979D.0	fR/S	 177.8
U+979D.0	fTotalStrokes	 17
U+979D.0	fCangjie	TJFBR
U+979D.0	fFourCorner	49527
U+979D.0	fHakka	 zhong3; zong3; zhong5
U+979D.0	fCantonese	soeng5; zoeng2
U+979D.0	fMandarin	zhang3
U+979D.0	fEnglish	the piece of leather used for soles on Chinese shoes; a patch
U+979E.0	fUTF8	鞞
U+979E.0	fCNS11643	2-604C
U+979E.0	fBig5	EF40
U+979E.0	fGB	3-793F
U+979E.0	fR/S	 177.8
U+979E.0	fTotalStrokes	 17
U+979E.0	fCangjie	TJHWJ
U+979E.0	fFourCorner	46540
U+979E.0	fHakka	 bin3
U+979E.0	fCantonese	bei2 (also); bing2; pei4
U+979E.0	fMandarin	bing3; bi3; pi2; bei1
U+979E.0	fEnglish	sheath
U+979F.0	fUTF8	鞟
U+979F.0	fCNS11643	3-567B
U+979F.0	fGB	3-7940
U+979F.0	fR/S	 177.8
U+979F.0	fTotalStrokes	 17
U+979F.0	fCangjie	TJYRD
U+979F.0	fFourCorner	40540
U+979F.0	fHakka	 gok7; gwok7
U+979F.0	fCantonese	kwok3; kok3 (also)
U+979F.0	fMandarin	kuo4
U+979F.0	fEnglish	skin
U+97A0.0	fUTF8	鞠
U+97A0.0	fCNS11643	1-7626
U+97A0.0	fBig5	C1F9
U+97A0.0	fGB	0-3E4F
U+97A0.0	fR/S	 177.8
U+97A0.0	fTotalStrokes	 17
U+97A0.0	fCangjie	TJPFD
U+97A0.0	fFourCorner	47520
U+97A0.0	fHakka	 kiuk7
U+97A0.0	fCantonese	guk1
U+97A0.0	fMandarin	ju1; qu1; qiong1; ju2; qu2
U+97A0.0	fEnglish	[1] bow; bend (gesture of respect) [2] ball; leather ball [3] rear; raise; nourish [4] young; tender [5] high [6] exhaust [7] admonish; warn; caution [8] make judicial investigation [9] a Chinese family name
U+97A1.0	fUTF8	鞡
U+97A1.0	fCNS11643	2-604A
U+97A1.0	fBig5	EEFD
U+97A1.0	fGB	3-793E
U+97A1.0	fR/S	 177.8
U+97A1.0	fTotalStrokes	 17
U+97A1.0	fCangjie	TJQYT
U+97A1.0	fFourCorner	40518
U+97A1.0	fCantonese	laa1 (also); laai1
U+97A1.0	fMandarin	la5; la1
U+97A1.0	fEnglish	leg warmer
U+97A2.0	fUTF8	鞢
U+97A2.0	fCNS11643	2-644B
U+97A2.0	fBig5	F1A1
U+97A2.0	fGB	3-7947
U+97A2.0	fR/S	 177.9
U+97A2.0	fTotalStrokes	 18
U+97A2.0	fCangjie	TJPTD
U+97A2.0	fFourCorner	44594
U+97A2.0	fCantonese	sip3; zaap6
U+97A2.0	fMandarin	xie4; zha2
U+97A3.0	fUTF8	鞣
U+97A3.0	fCNS11643	1-7768
U+97A3.0	fBig5	C2FD
U+97A3.0	fGB	0-7737
U+97A3.0	fR/S	 177.9
U+97A3.0	fTotalStrokes	 18
U+97A3.0	fCangjie	TJNHD
U+97A3.0	fFourCorner	47594
U+97A3.0	fHakka	 ngiu3; ngu3; ju2
U+97A3.0	fCantonese	jau4
U+97A3.0	fMandarin	rou2; qiu1; rou3; rou4
U+97A3.0	fEnglish	tan; soften
U+97A4.0	fUTF8	鞤
U+97A4.0	fCNS11643	2-6449
U+97A4.0	fBig5	F17D
U+97A4.0	fGB	E-4475
U+97A4.0	fR/S	 177.9
U+97A4.0	fTotalStrokes	 18
U+97A4.0	fCangjie	GITLJ
U+97A4.0	fFourCorner	44506
U+97A4.0	fHakka	  bong1
U+97A4.0	fCantonese	bong1
U+97A4.0	fMandarin	bang1
U+97A4.0	fEnglish	the leather heel-band of a ashoe
U+97A5.0	fUTF8	鞥
U+97A5.0	fCNS11643	2-644C
U+97A5.0	fBig5	F1A2
U+97A5.0	fGB	3-794D
U+97A5.0	fR/S	 177.9
U+97A5.0	fTotalStrokes	 18
U+97A5.0	fCangjie	TJORT
U+97A5.0	fFourCorner	48516
U+97A5.0	fCantonese	ang1; ngang1 (also)
U+97A5.0	fMandarin	yi4; eng1
U+97A6.0	fUTF8	鞦
U+97A6.0	fCNS11643	1-7769
U+97A6.0	fBig5	C2FE
U+97A6.0	fGB	1-7864
U+97A6.0	fR/S	 177.9
U+97A6.0	fTotalStrokes	 18
U+97A6.0	fCangjie	TJHDF
U+97A6.0	fFourCorner	49580
U+97A6.0	fHakka	  ciu1
U+97A6.0	fCantonese	cau1
U+97A6.0	fMandarin	qiu1
U+97A6.0	fEnglish	[1] swing [2] crupper [3] traces
U+97A7.0	fUTF8	鞧
U+97A7.0	fCNS11643	4-5E43
U+97A7.0	fGB	3-7950
U+97A7.0	fR/S	 177.9
U+97A7.0	fTotalStrokes	 18
U+97A7.0	fCangjie	TJTCW
U+97A7.0	fFourCorner	48560
U+97A7.0	fCantonese	cau1
U+97A7.0	fMandarin	qiu1
U+97A7.0	fEnglish	 crupper
U+97A8.0	fUTF8	鞨
U+97A8.0	fCNS11643	2-6447
U+97A8.0	fBig5	F17B
U+97A8.0	fGB	3-794A
U+97A8.0	fR/S	 177.9
U+97A8.0	fTotalStrokes	 18
U+97A8.0	fCangjie	TJAPV
U+97A8.0	fFourCorner	46527
U+97A8.0	fHakka	  hot7
U+97A8.0	fCantonese	hot3; maat3; hot6
U+97A8.0	fMandarin	he2; she2; mo4; ta4
U+97A8.0	fEnglish	correct form of &#xthe Tungusic tribe in ancient China;
U+97A9.0	fUTF8	鞩
U+97A9.0	fCNS11643	3-5950
U+97A9.0	fGB	E-4476
U+97A9.0	fR/S	 177.9
U+97A9.0	fTotalStrokes	 18
U+97A9.0	fCangjie	TJFBN
U+97A9.0	fFourCorner	42500
U+97A9.0	fCantonese	ciu3
U+97A9.0	fMandarin	xiao4
U+97AA.0	fUTF8	鞪
U+97AA.0	fCNS11643	2-644A
U+97AA.0	fBig5	F17E
U+97AA.0	fGB	5-774E
U+97AA.0	fR/S	 177.9
U+97AA.0	fTotalStrokes	 18
U+97AA.0	fCangjie	NKTLJ
U+97AA.0	fFourCorner	17506
U+97AA.0	fCantonese	mau4; muk6; mou6
U+97AA.0	fMandarin	mu4; mou2; mao4
U+97AB.0	fUTF8	鞫
U+97AB.0	fCNS11643	2-6448
U+97AB.0	fBig5	F17C
U+97AB.0	fGB	0-7736
U+97AB.0	fR/S	 177.9
U+97AB.0	fTotalStrokes	 18
U+97AB.0	fCangjie	TJPYR
U+97AB.0	fFourCorner	47520
U+97AB.0	fHakka	  kiuk7
U+97AB.0	fCantonese	guk1
U+97AB.0	fMandarin	ju1; ju2; qu1
U+97AB.0	fEnglish	[1] interrogate; question (criminals) [2] extreme poverty
U+97AC.0	fUTF8	鞬
U+97AC.0	fCNS11643	2-6445
U+97AC.0	fBig5	F179
U+97AC.0	fGB	3-7944
U+97AC.0	fR/S	 177.9
U+97AC.0	fTotalStrokes	 18
U+97AC.0	fCangjie	TJNKQ
U+97AC.0	fFourCorner	45044
U+97AC.0	fHakka	 gien1; gian1; gen1
U+97AC.0	fCantonese	gin1
U+97AC.0	fMandarin	jian1; jian4
U+97AC.0	fEnglish	[1] quiver (for arrows) on a horse [2] store; keep and collect
U+97AD.0	fUTF8	鞭
U+97AD.0	fCNS11643	1-776A
U+97AD.0	fBig5	C340
U+97AD.0	fGB	0-315E
U+97AD.0	fR/S	 177.9
U+97AD.0	fTotalStrokes	 18
U+97AD.0	fCangjie	TJOMK
U+97AD.0	fFourCorner	41546
U+97AD.0	fHakka	 bin1; bien1; ben1; pian1
U+97AD.0	fCantonese	bin1
U+97AD.0	fMandarin	bian1
U+97AD.0	fEnglish	[1] whip; lash; flagellate; flog [2] an ancient weapon in the shape of a whip [3] string of firecrackers
U+97AE.0	fUTF8	鞮
U+97AE.0	fCNS11643	2-6446
U+97AE.0	fBig5	F17A
U+97AE.0	fGB	3-7949
U+97AE.0	fR/S	 177.9
U+97AE.0	fTotalStrokes	 18
U+97AE.0	fCangjie	TJAMO
U+97AE.0	fFourCorner	46481
U+97AE.0	fHakka	  ti2
U+97AE.0	fCantonese	dai1
U+97AE.0	fMandarin	di1; ti2
U+97AE.0	fEnglish	leather shoes
U+97AF.0	fUTF8	鞯
U+97AF.0	fGB	0-7735
U+97AF.0	fR/S	 177.9
U+97AF.0	fTotalStrokes	 18
U+97AF.0	fCangjie	TJTKD
U+97AF.0	fCantonese	zin1
U+97AF.0	fMandarin	jian1
U+97AF.0	fEnglish	simplified form of &#x97C9;
U+97B0.0	fUTF8	鞰
U+97B0.0	fCNS11643	4-617C
U+97B0.0	fGB	1-7D79
U+97B0.0	fR/S	 177.9; 177.10
U+97B0.0	fTotalStrokes	 19; 18
U+97B0.0	fCangjie	TJABT
U+97B0.0	fFourCorner	46510
U+97B0.0	fCantonese	wan3
U+97B1.0	fUTF8	鞱
U+97B1.0	fCNS11643	3-5B5F
U+97B1.0	fGB	5-7755
U+97B1.0	fR/S	 177.10
U+97B1.0	fTotalStrokes	 19
U+97B1.0	fCangjie	TJBHX
U+97B1.0	fFourCorner	42570
U+97B1.0	fCantonese	tou1
U+97B1.0	fMandarin	tao1
U+97B2.0	fUTF8	鞲
U+97B2.0	fCNS11643	3-5B5D
U+97B2.0	fGB	0-7738
U+97B2.0	fR/S	  177.10
U+97B2.0	fTotalStrokes	 19
U+97B2.0	fCangjie	TJTTB
U+97B2.0	fFourCorner	45540
U+97B2.0	fHakka	 kieu1; geu1
U+97B2.0	fCantonese	gau1; kau1 (also)
U+97B2.0	fMandarin	gou1
U+97B2.0	fEnglish	the armlet and glove of a falconer
U+97B3.0	fUTF8	鞳
U+97B3.0	fCNS11643	2-676C
U+97B3.0	fBig5	F3A1
U+97B3.0	fGB	3-7948
U+97B3.0	fR/S	 177.10
U+97B3.0	fTotalStrokes	 19
U+97B3.0	fCangjie	TJTOR
U+97B3.0	fFourCorner	44561
U+97B3.0	fHakka	  tap7
U+97B3.0	fCantonese	daap6 (also); taap3
U+97B3.0	fMandarin	ta4; rou2
U+97B3.0	fEnglish	[1] the sound of bells and drums [2] cuirass; sheath
U+97B4.0	fUTF8	鞴
U+97B4.0	fCNS11643	3-5B5E
U+97B4.0	fGB	0-7739
U+97B4.0	fR/S	 177.10
U+97B4.0	fTotalStrokes	 19
U+97B4.0	fCangjie	TJTHB
U+97B4.0	fFourCorner	44520
U+97B4.0	fCantonese	bei6; bou6
U+97B4.0	fMandarin	bei4; bu4; fu2; bai4
U+97B4.0	fEnglish	correct form of &#x piston;
U+97B5.0	fUTF8	鞵
U+97B5.0	fCNS11643	3-5B60
U+97B5.0	fGB	E-4477
U+97B5.0	fR/S	 177.10
U+97B5.0	fTotalStrokes	 19
U+97B5.0	fCangjie	TJBVK
U+97B5.0	fFourCorner	42530
U+97B5.0	fHakka	 hai2; hi1; he2
U+97B5.0	fCantonese	haai4; hoe1
U+97B5.0	fMandarin	xie2
U+97B5.0	fEnglish	 shoes boots
U+97B6.0	fUTF8	鞶
U+97B6.0	fCNS11643	2-676E
U+97B6.0	fBig5	F3A3
U+97B6.0	fGB	5-7754
U+97B6.0	fR/S	 177.10
U+97B6.0	fTotalStrokes	 19
U+97B6.0	fCangjie	HETLJ
U+97B6.0	fFourCorner	27504
U+97B6.0	fHakka	  pan2
U+97B6.0	fCantonese	pun4
U+97B6.0	fMandarin	pan2
U+97B6.0	fEnglish	 big belt
U+97B7.0	fUTF8	鞷
U+97B7.0	fCNS11643	2-676D
U+97B7.0	fBig5	F3A2
U+97B7.0	fGB	E-4478
U+97B7.0	fR/S	 177.10
U+97B7.0	fTotalStrokes	 19
U+97B7.0	fCangjie	MRBJ
U+97B7.0	fFourCorner	10406
U+97B7.0	fCantonese	lik6; gaak3
U+97B7.0	fMandarin	ge2; he2
U+97B8.0	fUTF8	鞸
U+97B8.0	fCNS11643	3-5D50
U+97B8.0	fGB	5-7733
U+97B8.0	fR/S	  177.11
U+97B8.0	fTotalStrokes	 20
U+97B8.0	fCangjie	TJWTJ
U+97B8.0	fFourCorner	46540
U+97B8.0	fHakka	 bit7
U+97B8.0	fCantonese	bat1; bei2 (also)
U+97B8.0	fMandarin	bi4
U+97B8.0	fEnglish	leather arm guard
U+97B9.0	fUTF8	鞹
U+97B9.0	fCNS11643	2-6A58
U+97B9.0	fBig5	F54A
U+97B9.0	fGB	3-7955
U+97B9.0	fR/S	 177.11
U+97B9.0	fTotalStrokes	 20
U+97B9.0	fCangjie	TJYDL
U+97B9.0	fFourCorner	47527
U+97B9.0	fHakka	 gok7; gwok7
U+97B9.0	fCantonese	kwok3; kok3 (also)
U+97B9.0	fMandarin	kuo4; kui1
U+97B9.0	fEnglish	skin
U+97BA.0	fUTF8	鞺
U+97BA.0	fCNS11643	3-5D51
U+97BA.0	fGB	E-4479
U+97BA.0	fR/S	 177.11
U+97BA.0	fTotalStrokes	 20
U+97BA.0	fCangjie	TJFBG
U+97BA.0	fFourCorner	49510
U+97BA.0	fCantonese	tong1
U+97BA.0	fMandarin	tang1
U+97BA.0	fEnglish	the sound of drums
U+97BB.0	fUTF8	鞻
U+97BB.0	fCNS11643	2-6A59
U+97BB.0	fBig5	F54B
U+97BB.0	fGB	3-794F
U+97BB.0	fR/S	 177.11
U+97BB.0	fTotalStrokes	 20
U+97BB.0	fCangjie	TJLLV
U+97BB.0	fFourCorner	45544
U+97BB.0	fCantonese	lau4
U+97BB.0	fMandarin	lou2; ju4; l3; lou4
U+97BC.0	fUTF8	鞼
U+97BC.0	fCNS11643	4-6771
U+97BC.0	fGB	3-794B
U+97BC.0	fR/S	 177.12
U+97BC.0	fTotalStrokes	 21
U+97BC.0	fCangjie	TJLMC
U+97BC.0	fFourCorner	45580
U+97BC.0	fCantonese	gwai6
U+97BC.0	fMandarin	gui4
U+97BD.0	fUTF8	鞽
U+97BD.0	fCNS11643	3-5E75
U+97BD.0	fGB	1-7733
U+97BD.0	fR/S	  177.12
U+97BD.0	fTotalStrokes	 21
U+97BD.0	fCangjie	TJHKB
U+97BD.0	fFourCorner	42520
U+97BD.0	fHakka	  kiau1
U+97BD.0	fCantonese	kiu4
U+97BD.0	fMandarin	qiao1; qiao2; jue1
U+97BD.0	fEnglish	a small wooden sledge used to cross mud
U+97BE.0	fUTF8	鞾
U+97BE.0	fCNS11643	3-5E74
U+97BE.0	fGB	E-447A
U+97BE.0	fR/S	 177.12; 177.10
U+97BE.0	fTotalStrokes	 21
U+97BE.0	fCangjie	TJTMJ
U+97BE.0	fFourCorner	44550
U+97BE.0	fHakka	  hio1
U+97BE.0	fCantonese	hoe1
U+97BE.0	fMandarin	xue1
U+97BE.0	fEnglish	boots
U+97BF.0	fUTF8	鞿
U+97BF.0	fCNS11643	2-6C5E
U+97BF.0	fBig5	F670
U+97BF.0	fGB	3-787A
U+97BF.0	fR/S	 177.12
U+97BF.0	fTotalStrokes	 21
U+97BF.0	fCangjie	TJVII
U+97BF.0	fFourCorner	44553
U+97BF.0	fHakka	 gi1
U+97BF.0	fCantonese	gei1
U+97BF.0	fMandarin	ji1
U+97BF.0	fEnglish	bit of a bridle check a horse
U+97C0.0	fUTF8	韀
U+97C0.0	fCNS11643	3-5F73
U+97C0.0	fGB	5-775D
U+97C0.0	fR/S	 177.13
U+97C0.0	fTotalStrokes	 22
U+97C0.0	fCangjie	TJIXF
U+97C0.0	fFourCorner	40520
U+97C0.0	fHakka	  zien1
U+97C0.0	fCantonese	zin1
U+97C0.0	fMandarin	jian1
U+97C0.0	fEnglish	 saddle-cloth
U+97C1.0	fUTF8	韁
U+97C1.0	fCNS11643	1-7C23
U+97C1.0	fBig5	C5B7
U+97C1.0	fGB	E-447B
U+97C1.0	fR/S	 177.13
U+97C1.0	fTotalStrokes	 22
U+97C1.0	fCangjie	TJMWM
U+97C1.0	fFourCorner	41516
U+97C1.0	fHakka	 giong1
U+97C1.0	fCantonese	goeng1
U+97C1.0	fMandarin	jiang1; gang1
U+97C1.0	fEnglish	same as $
U+97C2.0	fUTF8	韂
U+97C2.0	fCNS11643	4-695E
U+97C2.0	fGB	3-7959
U+97C2.0	fR/S	  177.13
U+97C2.0	fTotalStrokes	 22; 21
U+97C2.0	fCangjie	TJNCR
U+97C2.0	fFourCorner	47560
U+97C2.0	fHakka	 zham5; zam5; zham1; cam1; can5
U+97C2.0	fCantonese	cim3; zim1
U+97C2.0	fMandarin	chan4
U+97C2.0	fEnglish	a saddle-flap. trappings
U+97C3.0	fUTF8	韃
U+97C3.0	fCNS11643	1-7C22
U+97C3.0	fBig5	C5B6
U+97C3.0	fGB	1-7732
U+97C3.0	fR/S	 177.13
U+97C3.0	fTotalStrokes	 22
U+97C3.0	fCangjie	TJYGQ
U+97C3.0	fFourCorner	44534
U+97C3.0	fHakka	 dat7
U+97C3.0	fCantonese	taat3; daat3 (also)
U+97C3.0	fMandarin	da2; ta4; bei4
U+97C3.0	fEnglish	Tartars
U+97C4.0	fUTF8	韄
U+97C4.0	fCNS11643	2-6F5D
U+97C4.0	fBig5	F84F
U+97C4.0	fGB	3-7958
U+97C4.0	fR/S	 177.14
U+97C4.0	fTotalStrokes	 23
U+97C4.0	fCangjie	TJTOE
U+97C4.0	fFourCorner	44547
U+97C4.0	fCantonese	waak6; wu6 (also)
U+97C4.0	fMandarin	huo4; hu4
U+97C5.0	fUTF8	韅
U+97C5.0	fCNS11643	2-6F5E
U+97C5.0	fBig5	F850
U+97C5.0	fGB	3-795A
U+97C5.0	fR/S	 177.14
U+97C5.0	fTotalStrokes	 23
U+97C5.0	fCangjie	TJAVF
U+97C5.0	fFourCorner	46533
U+97C5.0	fCantonese	hin2
U+97C5.0	fMandarin	xian3; qian4; xian4
U+97C5.0	fEnglish	harness
U+97C6.0	fUTF8	韆
U+97C6.0	fCNS11643	1-7C73
U+97C6.0	fBig5	C648
U+97C6.0	fGB	1-7861
U+97C6.0	fR/S	 177.15
U+97C6.0	fTotalStrokes	 24
U+97C6.0	fCangjie	TJYMU
U+97C6.0	fFourCorner	41531
U+97C6.0	fHakka	  cien1; cen1
U+97C6.0	fCantonese	cin1
U+97C6.0	fMandarin	qian1; da2
U+97C6.0	fEnglish	correct form of &#x swing;
U+97C7.0	fUTF8	韇
U+97C7.0	fCNS11643	2-705F
U+97C7.0	fBig5	F8D1
U+97C7.0	fGB	3-793D
U+97C7.0	fR/S	 177.15
U+97C7.0	fTotalStrokes	 24
U+97C7.0	fCangjie	TJGWC
U+97C7.0	fFourCorner	44586
U+97C7.0	fCantonese	duk6
U+97C7.0	fMandarin	du2
U+97C8.0	fUTF8	韈
U+97C8.0	fCNS11643	3-6128
U+97C8.0	fGB	E-447C
U+97C8.0	fR/S	 177.15
U+97C8.0	fTotalStrokes	 24
U+97C8.0	fCangjie	TJTWI
U+97C8.0	fFourCorner	44550
U+97C8.0	fHakka	  mat7
U+97C8.0	fCantonese	mat6; maat6 (also)
U+97C8.0	fMandarin	wa4
U+97C8.0	fEnglish	socks; stockings
U+97C9.0	fUTF8	韉
U+97C9.0	fCNS11643	1-7D36
U+97C9.0	fBig5	C669
U+97C9.0	fGB	1-7735
U+97C9.0	fR/S	 177.17
U+97C9.0	fTotalStrokes	 26
U+97C9.0	fCangjie	TJTIF
U+97C9.0	fFourCorner	44527
U+97C9.0	fCantonese	zin1
U+97C9.0	fMandarin	jian1
U+97C9.0	fEnglish	 saddle blanket; saddle cloth
U+97CA.0	fUTF8	韊
U+97CA.0	fCNS11643	3-623B
U+97CA.0	fGB	E-447D
U+97CA.0	fR/S	 177.21
U+97CA.0	fTotalStrokes	 30
U+97CA.0	fCangjie	TJTAW
U+97CA.0	fFourCorner	44520
U+97CA.0	fCantonese	laan4
U+97CA.0	fMandarin	lan2
U+97CA.0	fEnglish	a case for a bow
U+97CB.0	fUTF8	韋
U+97CB.0	fCNS11643	1-5374
U+97CB.0	fBig5	ADB3
U+97CB.0	fGB	1-4E24
U+97CB.0	fR/S	 178.0
U+97CB.0	fTotalStrokes	 9
U+97CB.0	fCangjie	DMRQ
U+97CB.0	fFourCorner	40506
U+97CB.0	fHakka	 wui3; wi2; wui2; wi3; vui3
U+97CB.0	fCantonese	wai4; wai5
U+97CB.0	fMandarin	wei2; hui2
U+97CB.0	fEnglish	[1] tanned leather [2] a Chinese family name [3] Kangxi radical 178
U+97CC.0	fUTF8	韌
U+97CC.0	fCNS11643	1-6279
U+97CC.0	fBig5	B6B4
U+97CC.0	fGB	1-484D
U+97CC.0	fR/S	 178.3
U+97CC.0	fTotalStrokes	 12
U+97CC.0	fCangjie	DQSHI
U+97CC.0	fFourCorner	47547
U+97CC.0	fHakka	 ngiun3; ngiun5; ngiun6; niun5
U+97CC.0	fCantonese	jan6; ngan6 (also); an6 (also); ngan1 (also); an1 (also)
U+97CC.0	fMandarin	ren4
U+97CC.0	fEnglish	soft but tough; elastic; strong and pliable; tenacious
U+97CD.0	fUTF8	韍
U+97CD.0	fCNS11643	2-4F32
U+97CD.0	fBig5	E4CA
U+97CD.0	fGB	3-4032
U+97CD.0	fR/S	 178.5
U+97CD.0	fTotalStrokes	 14
U+97CD.0	fCangjie	DQIKK
U+97CD.0	fFourCorner	43547
U+97CD.0	fHakka	 fut7
U+97CD.0	fCantonese	fat1
U+97CD.0	fMandarin	fu2; bei4; fei4
U+97CD.0	fEnglish	a leather garment worn during sacrificial rituals in ancient times
U+97CE.0	fUTF8	韎
U+97CE.0	fCNS11643	2-4F31
U+97CE.0	fBig5	E4C9
U+97CE.0	fGB	3-4031
U+97CE.0	fR/S	 178.5
U+97CE.0	fTotalStrokes	 14
U+97CE.0	fCangjie	DQDJ
U+97CE.0	fFourCorner	45590
U+97CE.0	fHakka	 mui3; mi3; moi5; mi5; mui5
U+97CE.0	fCantonese	mui6; mut6 (also)
U+97CE.0	fMandarin	mei4; ji4; mai4; mo4; wa4
U+97CE.0	fEnglish	red
U+97CF.0	fUTF8	韏
U+97CF.0	fCNS11643	2-555E
U+97CF.0	fBig5	E8B5
U+97CF.0	fGB	3-4039
U+97CF.0	fR/S	 178.6
U+97CF.0	fTotalStrokes	 15
U+97CF.0	fCangjie	FQDMQ
U+97CF.0	fFourCorner	90506
U+97CF.0	fCantonese	hyun3; gyun3
U+97CF.0	fMandarin	juan4; juan1; juan3; quan4; ruan2
U+97D0.0	fUTF8	韐
U+97D0.0	fCNS11643	2-555D
U+97D0.0	fBig5	E8B4
U+97D0.0	fGB	3-4038
U+97D0.0	fR/S	 178.6
U+97D0.0	fTotalStrokes	 15
U+97D0.0	fCangjie	DQOMR
U+97D0.0	fFourCorner	48561
U+97D0.0	fHakka	 kap7; giap7
U+97D0.0	fCantonese	gaap3
U+97D0.0	fMandarin	ge2; jia2; xia2
U+97D0.0	fEnglish	knee-pad made of red-dyed leather
U+97D1.0	fUTF8	韑
U+97D1.0	fCNS11643	4-4E57
U+97D1.0	fGB	E-447E
U+97D1.0	fR/S	 178.6
U+97D1.0	fTotalStrokes	 15
U+97D1.0	fCangjie	FUDMQ
U+97D1.0	fFourCorner	94210
U+97D1.0	fCantonese	wai5
U+97D1.0	fMandarin	wei3
U+97D2.0	fUTF8	韒
U+97D2.0	fCNS11643	3-5355
U+97D2.0	fGB	5-432A
U+97D2.0	fR/S	 178.7
U+97D2.0	fTotalStrokes	 16
U+97D2.0	fCangjie	DQFB
U+97D2.0	fFourCorner	49520
U+97D2.0	fHakka	  siau5; seu5
U+97D2.0	fCantonese	ciu3
U+97D2.0	fMandarin	qiao4
U+97D2.0	fEnglish	 sheath; scabbard
U+97D3.0	fUTF8	韓
U+97D3.0	fCNS11643	1-7627
U+97D3.0	fBig5	C1FA
U+97D3.0	fGB	1-3A2B
U+97D3.0	fR/S	 178.8
U+97D3.0	fTotalStrokes	 17
U+97D3.0	fCangjie	JJDMQ
U+97D3.0	fFourCorner	44456
U+97D3.0	fHakka	  hon2
U+97D3.0	fCantonese	hon4
U+97D3.0	fMandarin	han2
U+97D3.0	fEnglish	[1] fence [2] the name of two feudal states in the late Chou dynasty [3] short for the Republic of Korea [4] a Chinese familly name
U+97D4.0	fUTF8	韔
U+97D4.0	fCNS11643	2-604F
U+97D4.0	fBig5	EF43
U+97D4.0	fGB	3-4030
U+97D4.0	fR/S	 178.8
U+97D4.0	fTotalStrokes	 17
U+97D4.0	fCangjie	DQSMV
U+97D4.0	fFourCorner	41532
U+97D4.0	fHakka	 chong3; cong3
U+97D4.0	fCantonese	coeng3
U+97D4.0	fMandarin	chang4; liang4; zhang4
U+97D4.0	fEnglish	[1] wrapper or case for a bow [2] sheathe a bow; put a bow in its case
U+97D5.0	fUTF8	韕
U+97D5.0	fCNS11643	2-604E
U+97D5.0	fBig5	EF42
U+97D5.0	fGB	E-4521
U+97D5.0	fR/S	 178.8
U+97D5.0	fTotalStrokes	 17
U+97D5.0	fCangjie	DQYRD
U+97D5.0	fFourCorner	40547
U+97D5.0	fCantonese	hoeng2; tou4
U+97D6.0	fUTF8	韖
U+97D6.0	fCNS11643	2-644F
U+97D6.0	fBig5	F1A5
U+97D6.0	fGB	5-4331
U+97D6.0	fR/S	 178.9
U+97D6.0	fTotalStrokes	 18
U+97D6.0	fCangjie	DQNHD
U+97D6.0	fFourCorner	47594
U+97D6.0	fCantonese	jau5
U+97D6.0	fMandarin	rou2; rou3
U+97D6.0	fEnglish	tan; soften
U+97D7.0	fUTF8	韗
U+97D7.0	fCNS11643	2-644D
U+97D7.0	fBig5	F1A3
U+97D7.0	fGB	3-403A
U+97D7.0	fR/S	 178.9
U+97D7.0	fTotalStrokes	 18
U+97D7.0	fCangjie	DQBJJ
U+97D7.0	fFourCorner	47552
U+97D7.0	fHakka	 jun5; run6
U+97D7.0	fCantonese	wan6
U+97D7.0	fMandarin	xun4; yun4; yun2
U+97D7.0	fEnglish	 worker in leather
U+97D8.0	fUTF8	韘
U+97D8.0	fCNS11643	2-6450
U+97D8.0	fBig5	F1A6
U+97D8.0	fGB	3-403C
U+97D8.0	fR/S	 178.9
U+97D8.0	fTotalStrokes	 18
U+97D8.0	fCangjie	DQPTD
U+97D8.0	fFourCorner	44594
U+97D8.0	fHakka	 tiap7;  ngiap7
U+97D8.0	fCantonese	sip3
U+97D8.0	fMandarin	she4; xie4
U+97D8.0	fEnglish	archer's thumb ring
U+97D9.0	fUTF8	韙
U+97D9.0	fCNS11643	2-644E
U+97D9.0	fBig5	F1A4
U+97D9.0	fGB	1-6838
U+97D9.0	fR/S	 178.9
U+97D9.0	fTotalStrokes	 18
U+97D9.0	fCangjie	AODMQ
U+97D9.0	fFourCorner	64805
U+97D9.0	fHakka	 wui3; wi3; vui3
U+97D9.0	fCantonese	wai5
U+97D9.0	fMandarin	wei3
U+97D9.0	fEnglish	right; proper; propriety
U+97DA.0	fUTF8	韚
U+97DA.0	fCNS11643	3-5951
U+97DA.0	fGB	E-4522
U+97DA.0	fR/S	 178.9
U+97DA.0	fTotalStrokes	 18
U+97DA.0	fCangjie	DQTLJ
U+97DA.0	fFourCorner	44550
U+97DA.0	fCantonese	gaak3
U+97DA.0	fMandarin	ge2
U+97DB.0	fUTF8	韛
U+97DB.0	fCNS11643	4-6529
U+97DB.0	fGB	3-403F
U+97DB.0	fR/S	 178.11; 178.10
U+97DB.0	fTotalStrokes	 20
U+97DB.0	fCangjie	DQTHB
U+97DB.0	fFourCorner	44520
U+97DB.0	fHakka	 pai5; pi6; pi5; pai6
U+97DB.0	fCantonese	bai6; baai6 (also)
U+97DB.0	fMandarin	bei4
U+97DB.0	fEnglish	bellows as used by blacksmiths
U+97DC.0	fUTF8	韜
U+97DC.0	fCNS11643	1-7949
U+97DC.0	fBig5	C3FC
U+97DC.0	fGB	1-683A
U+97DC.0	fR/S	 178.10
U+97DC.0	fTotalStrokes	 19
U+97DC.0	fCangjie	DQBHX
U+97DC.0	fFourCorner	42577
U+97DC.0	fHakka	 tau1; to1
U+97DC.0	fCantonese	tou1
U+97DC.0	fMandarin	tao1; tao4
U+97DC.0	fEnglish	[1] sheath or scabbards for blades or swords [2] military strategy; tactics [3] conceal [4] idle
U+97DD.0	fUTF8	韝
U+97DD.0	fCNS11643	2-676F
U+97DD.0	fBig5	F3A4
U+97DD.0	fGB	3-403E
U+97DD.0	fR/S	 178.10
U+97DD.0	fTotalStrokes	 19
U+97DD.0	fCangjie	DQTTB
U+97DD.0	fFourCorner	45557
U+97DD.0	fHakka	 kieu1; geu1
U+97DD.0	fCantonese	gau1; kau1; gau3
U+97DD.0	fMandarin	gou1; gou4; kou1
U+97DD.0	fEnglish	leather arm band
U+97DE.0	fUTF8	韞
U+97DE.0	fCNS11643	2-6770
U+97DE.0	fBig5	F3A5
U+97DE.0	fGB	1-6839
U+97DE.0	fR/S	 178.10
U+97DE.0	fTotalStrokes	 19
U+97DE.0	fCangjie	DQWOT
U+97DE.0	fFourCorner	46517
U+97DE.0	fHakka	  wun3;  wun1; wun5; vun3
U+97DE.0	fCantonese	wan3; wan2
U+97DE.0	fMandarin	yun4; wen1; yun3
U+97DE.0	fEnglish	 hide; conceal (one's talent, etc.)
U+97DF.0	fUTF8	韟
U+97DF.0	fCNS11643	2-6771
U+97DF.0	fBig5	F3A6
U+97DF.0	fGB	5-4333
U+97DF.0	fR/S	 178.10
U+97DF.0	fTotalStrokes	 19
U+97DF.0	fCangjie	DQHAJ
U+97DF.0	fFourCorner	46543
U+97DF.0	fCantonese	gou1
U+97DF.0	fMandarin	gao1
U+97E0.0	fUTF8	韠
U+97E0.0	fCNS11643	4-6527
U+97E0.0	fGB	3-4037
U+97E0.0	fR/S	 178.11
U+97E0.0	fTotalStrokes	 20
U+97E0.0	fCangjie	DQWTJ
U+97E0.0	fFourCorner	46550
U+97E0.0	fHakka	  bit7
U+97E0.0	fCantonese	bat1
U+97E0.0	fMandarin	bi4
U+97E0.0	fEnglish	knee-pad
U+97E1.0	fUTF8	韡
U+97E1.0	fCNS11643	2-6C5F
U+97E1.0	fBig5	F671
U+97E1.0	fGB	5-4326
U+97E1.0	fR/S	 178.12
U+97E1.0	fTotalStrokes	 21
U+97E1.0	fCangjie	DQTMJ
U+97E1.0	fFourCorner	44554
U+97E1.0	fHakka	 wi3; wui3; vui3
U+97E1.0	fCantonese	wai5; wai4
U+97E1.0	fMandarin	wei3
U+97E1.0	fEnglish	gorgeous
U+97E2.0	fUTF8	韢
U+97E2.0	fCNS11643	3-5E76
U+97E2.0	fGB	3-4041
U+97E2.0	fR/S	 178.12
U+97E2.0	fTotalStrokes	 21
U+97E2.0	fCangjie	DQJIP
U+97E2.0	fFourCorner	45530
U+97E2.0	fCantonese	ceoi3
U+97E2.0	fMandarin	hui4
U+97E3.0	fUTF8	韣
U+97E3.0	fCNS11643	2-6E41
U+97E3.0	fBig5	F772
U+97E3.0	fGB	3-4043
U+97E3.0	fR/S	 178.13
U+97E3.0	fTotalStrokes	 22
U+97E3.0	fCangjie	DQWLI
U+97E3.0	fFourCorner	46527
U+97E3.0	fHakka	  tuk8
U+97E3.0	fCantonese	duk6
U+97E3.0	fMandarin	du2; shu3; chu4; zhu3
U+97E3.0	fEnglish	bow-bag
U+97E4.0	fUTF8	韤
U+97E4.0	fCNS11643	3-6129
U+97E4.0	fGB	E-4523
U+97E4.0	fR/S	 178.15
U+97E4.0	fTotalStrokes	 24
U+97E4.0	fCangjie	DQTWI
U+97E4.0	fFourCorner	44550
U+97E4.0	fCantonese	mat6; maat6 (also)
U+97E4.0	fMandarin	wa4
U+97E4.0	fEnglish	socks; stockings
U+97E5.0	fUTF8	韥
U+97E5.0	fCNS11643	2-7060
U+97E5.0	fBig5	F8D2
U+97E5.0	fGB	E-4524
U+97E5.0	fR/S	 178.15
U+97E5.0	fTotalStrokes	 24
U+97E5.0	fCangjie	DQGWC
U+97E5.0	fFourCorner	44586
U+97E5.0	fCantonese	duk6
U+97E5.0	fMandarin	du2
U+97E6.0	fUTF8	韦
U+97E6.0	fGB	0-4E24
U+97E6.0	fR/S	 178.0
U+97E6.0	fTotalStrokes	 4
U+97E6.0	fCangjie	QS
U+97E6.0	fHakka	 vui3
U+97E6.0	fCantonese	wai4; wai5
U+97E6.0	fMandarin	wei2; hui2
U+97E6.0	fEnglish	simplified form of &#x97CB;
U+97E7.0	fUTF8	韧
U+97E7.0	fGB	0-484D
U+97E7.0	fR/S	 178.3
U+97E7.0	fTotalStrokes	 7
U+97E7.0	fCangjie	QSSHI
U+97E7.0	fHakka	 ngiun3; ngiun5
U+97E7.0	fCantonese	jan6; ngan6 (also); an6 (also); ngan1 (also); an1 (also)
U+97E7.0	fMandarin	ren4
U+97E7.0	fEnglish	simplified form of &#x97CC;
U+97E8.0	fUTF8	韨
U+97E8.0	fGB	8-7B62
U+97E8.0	fR/S	 178.5
U+97E8.0	fTotalStrokes	 9
U+97E8.0	fCangjie	QSIKE
U+97E8.0	fCantonese	fat1
U+97E8.0	fMandarin	fu2
U+97E8.0	fEnglish	a leather knee-pad worn during sacrifices
U+97E9.0	fUTF8	韩
U+97E9.0	fGB	0-3A2B
U+97E9.0	fR/S	 178.8
U+97E9.0	fTotalStrokes	 12
U+97E9.0	fCangjie	JJQS
U+97E9.0	fHakka	 hon2
U+97E9.0	fCantonese	hon4
U+97E9.0	fMandarin	han2
U+97E9.0	fEnglish	simplified form of &#x97D3;
U+97EA.0	fUTF8	韪
U+97EA.0	fGB	0-6838
U+97EA.0	fR/S	 178.9
U+97EA.0	fTotalStrokes	 13
U+97EA.0	fCangjie	AOQS
U+97EA.0	fHakka	 vui3
U+97EA.0	fCantonese	wai5
U+97EA.0	fMandarin	wei3
U+97EA.0	fEnglish	simplified form of &#x97D9;
U+97EB.0	fUTF8	韫
U+97EB.0	fGB	0-6839
U+97EB.0	fR/S	 178.9
U+97EB.0	fTotalStrokes	 13
U+97EB.0	fCangjie	QSABT
U+97EB.0	fHakka	 vun3
U+97EB.0	fCantonese	wan3
U+97EB.0	fMandarin	yun4; wen1; yun3
U+97EB.0	fEnglish	simplified form of &#x97DE;
U+97EC.0	fUTF8	韬
U+97EC.0	fGB	0-683A
U+97EC.0	fR/S	 178.10
U+97EC.0	fTotalStrokes	 14
U+97EC.0	fCangjie	QSBHX
U+97EC.0	fHakka	 tau1
U+97EC.0	fCantonese	tou1
U+97EC.0	fMandarin	tao1; tao4
U+97EC.0	fEnglish	simplified form of &#x97DC;
U+97ED.0	fUTF8	韭
U+97ED.0	fCNS11643	1-5375
U+97ED.0	fBig5	ADB4
U+97ED.0	fGB	0-3E42
U+97ED.0	fR/S	 179.0
U+97ED.0	fTotalStrokes	 9
U+97ED.0	fCangjie	LSMM
U+97ED.0	fFourCorner	11101
U+97ED.0	fHakka	  giu3
U+97ED.0	fCantonese	gau2
U+97ED.0	fMandarin	jiu3
U+97ED.0	fEnglish	[1] scallion; leek; Chinese chives [2] Kangxi radical 179
U+97EE.0	fUTF8	韮
U+97EE.0	fCNS11643	3-454F
U+97EE.0	fGB	E-4525
U+97EE.0	fR/S	 179.4
U+97EE.0	fTotalStrokes	 13
U+97EE.0	fCangjie	TLSM
U+97EE.0	fFourCorner	44100
U+97EE.0	fHakka	  giu3; kieu3
U+97EE.0	fCantonese	gau2
U+97EE.0	fMandarin	jiu3
U+97EE.0	fEnglish	scallion; leek
U+97EF.0	fUTF8	韯
U+97EF.0	fCNS11643	3-4F65
U+97EF.0	fGB	5-483A
U+97EF.0	fR/S	 179.6; 179.11; 62.11
U+97EF.0	fTotalStrokes	 15
U+97EF.0	fCangjie	JILSM
U+97EF.0	fFourCorner	43150
U+97EF.0	fHakka	 siam2
U+97EF.0	fCantonese	cim1
U+97EF.0	fMandarin	xian1
U+97EF.0	fEnglish	wild onions or leeks
U+97F0.0	fUTF8	韰
U+97F0.0	fCNS11643	2-5B63
U+97F0.0	fBig5	EC57
U+97F0.0	fGB	3-7969
U+97F0.0	fR/S	 179.7
U+97F0.0	fTotalStrokes	 16
U+97F0.0	fCangjie	YELSM
U+97F0.0	fFourCorner	27101
U+97F0.0	fHakka	 hai5; hai6
U+97F0.0	fCantonese	haai6
U+97F0.0	fMandarin	xie4
U+97F0.0	fEnglish	courageous; bold
U+97F1.0	fUTF8	韱
U+97F1.0	fCNS11643	2-6050
U+97F1.0	fBig5	EF44
U+97F1.0	fGB	5-483D
U+97F1.0	fR/S	 179.8
U+97F1.0	fTotalStrokes	 17
U+97F1.0	fCangjie	OILSM
U+97F1.0	fFourCorner	83150
U+97F1.0	fHakka	 siam1; ciam1; siam2
U+97F1.0	fCantonese	cim1
U+97F1.0	fMandarin	xian1; jian1
U+97F1.0	fEnglish	wild onions or leeks
U+97F2.0	fUTF8	韲
U+97F2.0	fCNS11643	3-5B61
U+97F2.0	fGB	E-4526
U+97F2.0	fR/S	 179.10
U+97F2.0	fTotalStrokes	 19
U+97F2.0	fCangjie	YSOM
U+97F2.0	fFourCorner	00100
U+97F2.0	fHakka	 zi1
U+97F2.0	fCantonese	zai1
U+97F2.0	fMandarin	ji1
U+97F2.0	fEnglish	break or smash into pieces; pulverize
U+97F3.0	fUTF8	音
U+97F3.0	fCNS11643	1-5376
U+97F3.0	fBig5	ADB5
U+97F3.0	fGB	0-5274
U+97F3.0	fR/S	 180.0
U+97F3.0	fTotalStrokes	 9
U+97F3.0	fCangjie	YTA
U+97F3.0	fFourCorner	00601
U+97F3.0	fHakka	  jim1; rim1
U+97F3.0	fCantonese	jam1; jam3
U+97F3.0	fMandarin	yin1; yin4
U+97F3.0	fEnglish	[1] sound; voice [2] tone; pitch; accent; pronunciation; timbre [3] musical note [4] news; information; tidings (usually used in correspondence) [5] Kangxi radical 180
U+97F4.0	fUTF8	韴
U+97F4.0	fCNS11643	4-4163
U+97F4.0	fGB	E-4527
U+97F4.0	fR/S	  180.4
U+97F4.0	fTotalStrokes	 13
U+97F4.0	fCangjie	YAMLB
U+97F4.0	fFourCorner	01620
U+97F4.0	fCantonese	zaap6
U+97F4.0	fMandarin	za2
U+97F5.0	fUTF8	韵
U+97F5.0	fCNS11643	3-4550
U+97F5.0	fGB	0-544F
U+97F5.0	fR/S	 180.4
U+97F5.0	fTotalStrokes	 13
U+97F5.0	fCangjie	YAPIM
U+97F5.0	fFourCorner	07620
U+97F5.0	fHakka	 jun5; jun3; run6
U+97F5.0	fCantonese	wan5 (also); wan6; wan2 (also)
U+97F5.0	fMandarin	yun4
U+97F5.0	fEnglish	[1] rhyme [2] harmony
U+97F6.0	fUTF8	韶
U+97F6.0	fCNS11643	1-6B69
U+97F6.0	fBig5	BBE0
U+97F6.0	fGB	0-4958
U+97F6.0	fR/S	 180.5
U+97F6.0	fTotalStrokes	 14
U+97F6.0	fCangjie	YASHR
U+97F6.0	fFourCorner	07662
U+97F6.0	fHakka	 sau2; shau2; seu2; shau1
U+97F6.0	fCantonese	siu4
U+97F6.0	fMandarin	shao2
U+97F6.0	fEnglish	[1] the music during the time of the sage emperor Shun [2] beautiful; excellent; harmonious
U+97F7.0	fUTF8	韷
U+97F7.0	fCNS11643	4-4846
U+97F7.0	fGB	E-4528
U+97F7.0	fR/S	  180.5
U+97F7.0	fTotalStrokes	 14
U+97F7.0	fCangjie	YAUU
U+97F7.0	fFourCorner	02670
U+97F7.0	fCantonese	lyut3
U+97F7.0	fMandarin	le4
U+97F8.0	fUTF8	韸
U+97F8.0	fCNS11643	2-5B64
U+97F8.0	fBig5	EC58
U+97F8.0	fGB	5-787C
U+97F8.0	fR/S	 180.7
U+97F8.0	fTotalStrokes	 16
U+97F8.0	fCangjie	YAHEJ
U+97F8.0	fFourCorner	07654
U+97F8.0	fHakka	  pung2
U+97F8.0	fCantonese	pung4
U+97F8.0	fMandarin	peng2; feng1
U+97F8.0	fEnglish	the noise of drums
U+97F9.0	fUTF8	韹
U+97F9.0	fCNS11643	1-776B
U+97F9.0	fBig5	C341
U+97F9.0	fGB	5-7921
U+97F9.0	fR/S	 180.9
U+97F9.0	fTotalStrokes	 18
U+97F9.0	fCangjie	YAHAG
U+97F9.0	fFourCorner	06614
U+97F9.0	fCantonese	wong4; waang4; jing1
U+97F9.0	fMandarin	heng2; huang2; ying1
U+97FA.0	fUTF8	韺
U+97FA.0	fCNS11643	2-6451
U+97FA.0	fBig5	F1A7
U+97FA.0	fGB	5-787D
U+97FA.0	fR/S	 180.9
U+97FA.0	fTotalStrokes	 18
U+97FA.0	fCangjie	YATLK
U+97FA.0	fFourCorner	04635
U+97FA.0	fCantonese	jing1
U+97FA.0	fMandarin	ying1
U+97FA.0	fEnglish	music
U+97FB.0	fUTF8	韻
U+97FB.0	fCNS11643	1-794A
U+97FB.0	fBig5	C3FD
U+97FB.0	fGB	E-4529
U+97FB.0	fR/S	 180.10
U+97FB.0	fTotalStrokes	 19
U+97FB.0	fCangjie	YARBC
U+97FB.0	fFourCorner	06686
U+97FB.0	fHakka	 jun5; run6; jun3; jun6
U+97FB.0	fCantonese	wan6; wan5
U+97FB.0	fMandarin	yun4
U+97FB.0	fEnglish	[1] rhymes [2] harmony of sound [3] refined; sophisticated; polished; elegant [4] vowels
U+97FC.0	fUTF8	韼
U+97FC.0	fCNS11643	3-5D53
U+97FC.0	fGB	E-452A
U+97FC.0	fR/S	 180.10
U+97FC.0	fTotalStrokes	 20; 19
U+97FC.0	fCangjie	YAYHJ
U+97FC.0	fFourCorner	07630
U+97FC.0	fCantonese	pung4
U+97FC.0	fMandarin	peng2
U+97FD.0	fUTF8	韽
U+97FD.0	fCNS11643	2-6A5A
U+97FD.0	fBig5	F54C
U+97FD.0	fGB	3-7B32
U+97FD.0	fR/S	 180.11
U+97FD.0	fTotalStrokes	 20
U+97FD.0	fCangjie	OWYTA
U+97FD.0	fFourCorner	80661
U+97FD.0	fCantonese	am1; ngam1 (also); am2
U+97FD.0	fMandarin	yin1; an1
U+97FE.0	fUTF8	韾
U+97FE.0	fCNS11643	2-6A5B
U+97FE.0	fBig5	F54D
U+97FE.0	fGB	E-452B
U+97FE.0	fR/S	 180.11
U+97FE.0	fTotalStrokes	 20
U+97FE.0	fCangjie	GEYTA
U+97FE.0	fFourCorner	47601
U+97FE.0	fCantonese	jam1
U+97FE.0	fMandarin	yin1; an1
U+97FF.0	fUTF8	響
U+97FF.0	fCNS11643	1-7B40
U+97FF.0	fBig5	C554
U+97FF.0	fGB	1-4F6C
U+97FF.0	fR/S	 180.11; 180.13
U+97FF.0	fTotalStrokes	 20
U+97FF.0	fCangjie	VLYTA
U+97FF.0	fFourCorner	27601
U+97FF.0	fHakka	  hiong3
U+97FF.0	fCantonese	hoeng2
U+97FF.0	fMandarin	xiang3
U+97FF.0	fEnglish	[1] report; sound; noise; echo [2] loud or hig (sound) [3] make a sound; sound; ring
U+9800.0	fUTF8	頀
U+9800.0	fCNS11643	2-6F5F
U+9800.0	fBig5	F851
U+9800.0	fGB	5-7923
U+9800.0	fR/S	 180.14
U+9800.0	fTotalStrokes	 23
U+9800.0	fCangjie	YATOE
U+9800.0	fFourCorner	04647
U+9800.0	fCantonese	wu6
U+9800.0	fMandarin	hu4; huo4
U+9800.0	fEnglish	music
U+9801.0	fUTF8	頁
U+9801.0	fCNS11643	1-5377
U+9801.0	fBig5	ADB6
U+9801.0	fGB	1-5233
U+9801.0	fR/S	 181.0
U+9801.0	fTotalStrokes	 9
U+9801.0	fCangjie	MBUC
U+9801.0	fFourCorner	10806
U+9801.0	fHakka	  jap8;  giet7; rap8; get7; sak7
U+9801.0	fCantonese	jip6; sau2; kit3
U+9801.0	fMandarin	ye4; xie2; shou3
U+9801.0	fEnglish	[1] page (in books, etc.); sheet [2] Kangxi radical 181
U+9802.0	fUTF8	頂
U+9802.0	fCNS11643	1-5E21
U+9802.0	fBig5	B3BB
U+9802.0	fGB	1-3625
U+9802.0	fR/S	 181.2
U+9802.0	fTotalStrokes	 11
U+9802.0	fCangjie	MNMBC
U+9802.0	fFourCorner	11286
U+9802.0	fHakka	  dang3;  din1;  den3; din3; dang2
U+9802.0	fCantonese	ding2; deng2
U+9802.0	fMandarin	ding3
U+9802.0	fEnglish	[1] the top of anything [2] the crown of the head [3] topmost; extremely; very [4] carry (weight) on one's head; push the head against; wear on the head [5] gore; butt [6] push up; prop up [7] cope with; stand up to [8] substitute [9] equal; be equivalent to [10] offend intenionally; retort; turn down [11] used as a unit [12] hire [13] measure word
U+9803.0	fUTF8	頃
U+9803.0	fCNS11643	1-5E22
U+9803.0	fBig5	B3BC
U+9803.0	fGB	1-476A
U+9803.0	fR/S	 181.2
U+9803.0	fTotalStrokes	 11
U+9803.0	fCangjie	PMBC
U+9803.0	fFourCorner	21786
U+9803.0	fHakka	  ken3;  kin3
U+9803.0	fCantonese	king2; king1
U+9803.0	fMandarin	qing3; qing1; kui3
U+9803.0	fEnglish	[1] moment; instant; just; just now [2] a measure of land (100 mu, 10,000 square meters) [3] lean toward one side; incline
U+9804.0	fUTF8	頄
U+9804.0	fCNS11643	2-3A32
U+9804.0	fBig5	D84E
U+9804.0	fGB	5-5F6B
U+9804.0	fR/S	 181.2
U+9804.0	fTotalStrokes	 11
U+9804.0	fCangjie	KNMBC
U+9804.0	fFourCorner	41086
U+9804.0	fCantonese	kwai4; kau4 (also); wau4
U+9804.0	fMandarin	qiu2; kui2; pan4; gui1; qun1
U+9804.0	fEnglish	cheekbone
U+9805.0	fUTF8	項
U+9805.0	fCNS11643	1-627A
U+9805.0	fBig5	B6B5
U+9805.0	fGB	1-4F6E
U+9805.0	fR/S	 181.3
U+9805.0	fTotalStrokes	 12
U+9805.0	fCangjie	MMBC
U+9805.0	fFourCorner	11186
U+9805.0	fHakka	  hong5; hong6; hong3
U+9805.0	fCantonese	hong6; hong2 (also)
U+9805.0	fMandarin	xiang4
U+9805.0	fEnglish	[1] the back of the neck; nape [2] back of a cap or gown [3] item; article; matter; kind; class [4] funds; sum of money [5] term (mathematics) [6] a Chinese family name
U+9806.0	fUTF8	順
U+9806.0	fCNS11643	1-627B
U+9806.0	fBig5	B6B6
U+9806.0	fGB	1-4B33
U+9806.0	fR/S	 181.3
U+9806.0	fTotalStrokes	 12
U+9806.0	fCangjie	LLLC
U+9806.0	fFourCorner	21086
U+9806.0	fHakka	 sun5; shun6; sun3; shun5; shun3
U+9806.0	fCantonese	seon6; seon2 (also)
U+9806.0	fMandarin	shun4
U+9806.0	fEnglish	[1] follow; obey; submit to; go along with; cause to surrender; fall in with [2] in the same direction as; with; along [3] agreeable; favorable; comfortable; happy [4] arrang; put in order [5] convenient; smooth [6] take the opportunity to
U+9807.0	fUTF8	頇
U+9807.0	fCNS11643	2-4151
U+9807.0	fBig5	DCAC
U+9807.0	fGB	1-717C
U+9807.0	fR/S	 181.3
U+9807.0	fTotalStrokes	 12
U+9807.0	fCangjie	MJMBC
U+9807.0	fFourCorner	11486
U+9807.0	fHakka	  hon2; hon1
U+9807.0	fCantonese	hon1; hon4
U+9807.0	fMandarin	han1; an4; e4; han2; kan1
U+9807.0	fEnglish	correct form of &#xstupid and incompetent;foolish and slow
U+9808.0	fUTF8	須
U+9808.0	fCNS11643	1-627C
U+9808.0	fBig5	B6B7
U+9808.0	fGB	1-506B
U+9808.0	fR/S	 181.3
U+9808.0	fTotalStrokes	 12
U+9808.0	fCangjie	HHMBC
U+9808.0	fFourCorner	21286
U+9808.0	fHakka	  si1; sii1
U+9808.0	fCantonese	seoi1; sou1
U+9808.0	fMandarin	xu1; ban1
U+9808.0	fEnglish	[1] must; have to [2] necessary; proper [3] probably [4] beard [5] moment; while [6] wait for [7] stop at [8] a Chinese family name
U+9809.0	fUTF8	頉
U+9809.0	fCNS11643	4-4168
U+9809.0	fGB	3-4677
U+9809.0	fR/S	 181.3; 181.4
U+9809.0	fTotalStrokes	 12; 13
U+9809.0	fCangjie	YMMBC
U+9809.0	fFourCorner	21180
U+9809.0	fCantonese	ji4
U+9809.0	fMandarin	yi2
U+9809.0	fEnglish	 cheeks jaw chin rear nourish
U+980A.0	fUTF8	頊
U+980A.0	fCNS11643	1-6762
U+980A.0	fBig5	B97A
U+980A.0	fGB	1-676F
U+980A.0	fR/S	 181.4
U+980A.0	fTotalStrokes	 13
U+980A.0	fCangjie	MGMBC
U+980A.0	fFourCorner	11186
U+980A.0	fHakka	 hiuk7
U+980A.0	fCantonese	juk1; guk1
U+980A.0	fMandarin	xu4; yu4
U+980A.0	fEnglish	 grieved anxious
U+980B.0	fUTF8	頋
U+980B.0	fCNS11643	3-4552
U+980B.0	fGB	E-452C
U+980B.0	fR/S	 181.4
U+980B.0	fTotalStrokes	 13
U+980B.0	fCangjie	MUMBC
U+980B.0	fFourCorner	71280
U+980B.0	fCantonese	gu3; gwu3 (also)
U+980B.0	fMandarin	gu4
U+980B.0	fEnglish	 care for; look after to regard turn the head round to look
U+980C.0	fUTF8	頌
U+980C.0	fCNS11643	1-6764
U+980C.0	fBig5	B97C
U+980C.0	fGB	1-4B4C
U+980C.0	fR/S	 181.4
U+980C.0	fTotalStrokes	 13
U+980C.0	fCangjie	CIMBC
U+980C.0	fFourCorner	81786
U+980C.0	fHakka	 siung5; siung3; sung3
U+980C.0	fCantonese	zung6; jung4
U+980C.0	fMandarin	rong2; song4
U+980C.0	fEnglish	[1] praise; acclaim; accolade; extol; eulogize; laud [2] hymn to something; ode; eulogy; paean; composition in praise of some achievements, etc. [3] a section in the Book of Poetry
U+980D.0	fUTF8	頍
U+980D.0	fCNS11643	2-4865
U+980D.0	fBig5	E0DF
U+980D.0	fGB	3-4367
U+980D.0	fR/S	 181.4
U+980D.0	fTotalStrokes	 13
U+980D.0	fCangjie	JEMBC
U+980D.0	fFourCorner	41486
U+980D.0	fHakka	 kui1; kwui1; kwui3
U+980D.0	fCantonese	kwai2; kwai1
U+980D.0	fMandarin	kui3; wei3; ji4
U+980D.0	fEnglish	 raise one's head
U+980E.0	fUTF8	頎
U+980E.0	fCNS11643	2-4866
U+980E.0	fBig5	E0E0
U+980E.0	fGB	1-717D
U+980E.0	fR/S	 181.4
U+980E.0	fTotalStrokes	 13
U+980E.0	fCangjie	HLMBC
U+980E.0	fFourCorner	71286
U+980E.0	fHakka	 ki2
U+980E.0	fCantonese	kei4
U+980E.0	fMandarin	qi2; ken3; jin4
U+980E.0	fEnglish	tall (physical build)
U+980F.0	fUTF8	頏
U+980F.0	fCNS11643	2-4864
U+980F.0	fBig5	E0DE
U+980F.0	fGB	1-717E
U+980F.0	fR/S	 181.4
U+980F.0	fTotalStrokes	 13
U+980F.0	fCangjie	YNMBC
U+980F.0	fFourCorner	01286
U+980F.0	fHakka	 hong2; kong5; kong1
U+980F.0	fCantonese	hong4; kong3
U+980F.0	fMandarin	gang1; hang2; hang4; kang3; kang4; gang3
U+980F.0	fEnglish	correct form of &#x flying up and down;match; contest; well-matched
U+9810.0	fUTF8	預
U+9810.0	fCNS11643	1-675F
U+9810.0	fBig5	B977
U+9810.0	fGB	1-5424
U+9810.0	fR/S	 181.4
U+9810.0	fTotalStrokes	 13
U+9810.0	fCangjie	NNMBC
U+9810.0	fFourCorner	11286
U+9810.0	fHakka	  ji5; ji3; ri6; i5
U+9810.0	fCantonese	jyu6
U+9810.0	fMandarin	yu4
U+9810.0	fEnglish	[1] beforehand; in advance; previously [2] prepare; arrange; make ready; reserve (funds, troops, etc.) [3] take part in
U+9811.0	fUTF8	頑
U+9811.0	fCNS11643	1-6760
U+9811.0	fBig5	B978
U+9811.0	fGB	1-4D67
U+9811.0	fR/S	 181.4
U+9811.0	fTotalStrokes	 13
U+9811.0	fCangjie	MUMBC
U+9811.0	fFourCorner	11286
U+9811.0	fHakka	  ngan2;  nguan2
U+9811.0	fCantonese	waan4; wan4
U+9811.0	fMandarin	wan2; kun1
U+9811.0	fEnglish	[1] obstinate; stubborn [2] recalcitrant; unruly; defiant [3] dull; stupid; ignorant [4] play [5] naughty or impish
U+9812.0	fUTF8	頒
U+9812.0	fCNS11643	1-6763
U+9812.0	fBig5	B97B
U+9812.0	fGB	1-3064
U+9812.0	fR/S	 181.4
U+9812.0	fTotalStrokes	 13
U+9812.0	fCangjie	CHMBC
U+9812.0	fFourCorner	81286
U+9812.0	fHakka	  ban1
U+9812.0	fCantonese	baan1; paan1 (also); fan4
U+9812.0	fMandarin	ban1; fen2
U+9812.0	fEnglish	[1] confer on; bestow on; grant [2] proclaim; publish official; make public; promulgate [3] distribute; send out
U+9813.0	fUTF8	頓
U+9813.0	fCNS11643	1-6761
U+9813.0	fBig5	B979
U+9813.0	fGB	1-3659
U+9813.0	fR/S	 181.4
U+9813.0	fTotalStrokes	 13
U+9813.0	fCangjie	PUMBC
U+9813.0	fFourCorner	51786
U+9813.0	fHakka	  dun3; dun5; tuk8
U+9813.0	fCantonese	deon6; duk6; deon2 (also); eon6
U+9813.0	fMandarin	dun4; zhun1
U+9813.0	fEnglish	[1] pause; stop; halt [2] bow; kowtow [3] stamp (the foot) [4] arrange; put in order [5] time; turn [6] immediately; promptly [7] be tired; fall apart [8] be broken [9] a Chinese family name
U+9813.1	fUTF8	頓
U+9813.1	fCNS11643	1-6761
U+9813.1	fBig5	B979
U+9813.1	fGB	1-3659
U+9813.1	fR/S	 181.4
U+9813.1	fTotalStrokes	 13
U+9813.1	fCangjie	PUMBC
U+9813.1	fFourCorner	51786
U+9813.1	fMandarin	du2
U+9813.1	fEnglish	correct form of &#xthe name of Tartar chieftain in the early Western Han dynasty;
U+9814.0	fUTF8	頔
U+9814.0	fCNS11643	3-4A54
U+9814.0	fGB	3-6069
U+9814.0	fR/S	  181.5
U+9814.0	fTotalStrokes	 14
U+9814.0	fCangjie	LWMBC
U+9814.0	fFourCorner	51686
U+9814.0	fCantonese	dik6; king2 (also)
U+9814.0	fMandarin	di2
U+9814.0	fEnglish	good and beautiful
U+9815.0	fUTF8	頕
U+9815.0	fCNS11643	3-4A55
U+9815.0	fGB	3-6068
U+9815.0	fR/S	  181.5
U+9815.0	fTotalStrokes	 14
U+9815.0	fCangjie	YRMBC
U+9815.0	fFourCorner	21680
U+9815.0	fCantonese	daam1
U+9815.0	fMandarin	dan1; dian4
U+9815.0	fEnglish	raise the head
U+9816.0	fUTF8	頖
U+9816.0	fCNS11643	2-4F33
U+9816.0	fBig5	E4CB
U+9816.0	fGB	5-5F78
U+9816.0	fR/S	 181.5
U+9816.0	fTotalStrokes	 14
U+9816.0	fCangjie	FQMBC
U+9816.0	fFourCorner	91586
U+9816.0	fHakka	 pan5
U+9816.0	fCantonese	pun3
U+9816.0	fMandarin	pan4; cheng1
U+9816.0	fEnglish	 Zhou dynasty school disperse fall apart
U+9817.0	fUTF8	頗
U+9817.0	fCNS11643	1-6B6A
U+9817.0	fBig5	BBE1
U+9817.0	fGB	1-4644
U+9817.0	fR/S	 181.5
U+9817.0	fTotalStrokes	 14
U+9817.0	fCangjie	DEMBC
U+9817.0	fFourCorner	41286
U+9817.0	fHakka	  po3;  bo1; po5; po1
U+9817.0	fCantonese	po2; po1
U+9817.0	fMandarin	pi2; po3; po4; pi3
U+9817.0	fEnglish	[1] somewhat [2] quite; very; rather; fairly; considerably [3] a Chinese family name
U+9817.1	fUTF8	頗
U+9817.1	fCNS11643	1-6B6A
U+9817.1	fBig5	BBE1
U+9817.1	fGB	1-4644
U+9817.1	fR/S	 181.5
U+9817.1	fTotalStrokes	 14
U+9817.1	fCangjie	DEMBC
U+9817.1	fFourCorner	41286
U+9817.1	fMandarin	po1
U+9817.1	fEnglish	inclined to one side; unjustly
U+9818.0	fUTF8	領
U+9818.0	fCNS11643	1-6B6B
U+9818.0	fBig5	BBE2
U+9818.0	fGB	1-416C
U+9818.0	fR/S	 181.5
U+9818.0	fTotalStrokes	 14
U+9818.0	fCangjie	OIMBC
U+9818.0	fFourCorner	81386
U+9818.0	fHakka	  liang1;  liang3; liang5; jang5
U+9818.0	fCantonese	ling5; leng5 (also); leng2 (also)
U+9818.0	fMandarin	ling3
U+9818.0	fEnglish	[1] neck [2] collar; neckband [3] a piece of clothing [4] lead; guide; head [5] receive; get [6] understand [7] manage; operate
U+9819.0	fUTF8	頙
U+9819.0	fCNS11643	3-4A52
U+9819.0	fGB	5-5F75
U+9819.0	fR/S	 181.5; 181.4
U+9819.0	fTotalStrokes	 14; 13
U+9819.0	fCangjie	XMMMB
U+9819.0	fFourCorner	11180
U+9819.0	fCantonese	caak3
U+9819.0	fMandarin	che4; ce4
U+9819.0	fEnglish	correct
U+981A.0	fUTF8	頚
U+981A.0	fCNS11643	3-4A56
U+981A.0	fGB	E-452D
U+981A.0	fR/S	 181.5
U+981A.0	fTotalStrokes	 14
U+981A.0	fCangjie	EGMBC
U+981A.0	fFourCorner	11180
U+981A.0	fCantonese	geng2
U+981A.0	fMandarin	jing3; geng3
U+981A.0	fEnglish	neck; throat
U+981B.0	fUTF8	頛
U+981B.0	fCNS11643	2-5565
U+981B.0	fBig5	E8BC
U+981B.0	fGB	5-5F35
U+981B.0	fR/S	 181.6
U+981B.0	fTotalStrokes	 15
U+981B.0	fCangjie	QDMBC
U+981B.0	fFourCorner	51986
U+981B.0	fCantonese	leoi5
U+981B.0	fMandarin	lei3; lei4
U+981C.0	fUTF8	頜
U+981C.0	fCNS11643	1-6F70
U+981C.0	fBig5	BE67
U+981C.0	fGB	1-7222
U+981C.0	fR/S	 181.6
U+981C.0	fTotalStrokes	 15
U+981C.0	fCangjie	ORMBC
U+981C.0	fFourCorner	81686
U+981C.0	fHakka	 hap8; ngam3
U+981C.0	fCantonese	gap3 (also); hap6
U+981C.0	fMandarin	han4; ge2; han2; he2; ke4; qin1
U+981C.0	fEnglish	 mouth jaw
U+981D.0	fUTF8	頝
U+981D.0	fCNS11643	2-5560
U+981D.0	fBig5	E8B7
U+981D.0	fGB	5-5F7D
U+981D.0	fR/S	 181.6
U+981D.0	fTotalStrokes	 15
U+981D.0	fCangjie	YKMBC
U+981D.0	fFourCorner	01486
U+981D.0	fCantonese	haau1
U+981D.0	fMandarin	qiao1; qiao3
U+981E.0	fUTF8	頞
U+981E.0	fCNS11643	2-555F
U+981E.0	fBig5	E8B6
U+981E.0	fGB	3-6077
U+981E.0	fR/S	 181.6
U+981E.0	fTotalStrokes	 15
U+981E.0	fCangjie	JVMBC
U+981E.0	fFourCorner	31486
U+981E.0	fHakka	 ot7
U+981E.0	fCantonese	aat3; ngaat3 (also)
U+981E.0	fMandarin	e4; an4
U+981E.0	fEnglish	(nose) bridge
U+981F.0	fUTF8	頟
U+981F.0	fCNS11643	3-4F69
U+981F.0	fGB	E-452E
U+981F.0	fR/S	 181.6
U+981F.0	fTotalStrokes	 15
U+981F.0	fCangjie	HRMBC
U+981F.0	fFourCorner	21680
U+981F.0	fHakka	  ngiak7
U+981F.0	fCantonese	ngaak6; aak6 (also)
U+981F.0	fMandarin	e2
U+981F.0	fEnglish	 the forehead a fixed number
U+9820.0	fUTF8	頠
U+9820.0	fCNS11643	2-5564
U+9820.0	fBig5	E8BB
U+9820.0	fGB	3-6074
U+9820.0	fR/S	 181.6
U+9820.0	fTotalStrokes	 15
U+9820.0	fCangjie	NUMBC
U+9820.0	fFourCorner	21286
U+9820.0	fCantonese	ngai5; ai5 (also)
U+9820.0	fMandarin	wei3
U+9820.0	fEnglish	quiet; tranquil
U+9821.0	fUTF8	頡
U+9821.0	fCNS11643	1-6F6E
U+9821.0	fBig5	BE65
U+9821.0	fGB	1-7221
U+9821.0	fR/S	 181.6
U+9821.0	fTotalStrokes	 15
U+9821.0	fCangjie	GRMBC
U+9821.0	fFourCorner	41686
U+9821.0	fHakka	  giet7; get7
U+9821.0	fCantonese	kit3; git3 (also)
U+9821.0	fMandarin	jia2
U+9821.0	fEnglish	 deduct; omit
U+9821.1	fUTF8	頡
U+9821.1	fCNS11643	1-6F6E
U+9821.1	fBig5	BE65
U+9821.1	fGB	1-7221
U+9821.1	fR/S	 181.6
U+9821.1	fTotalStrokes	 15
U+9821.1	fCangjie	GRMBC
U+9821.1	fFourCorner	41686
U+9821.1	fMandarin	xie2; jie2
U+9821.1	fEnglish	 fly upward
U+9822.0	fUTF8	頢
U+9822.0	fCNS11643	3-4F67
U+9822.0	fGB	3-6343
U+9822.0	fR/S	  181.6
U+9822.0	fTotalStrokes	 15
U+9822.0	fCangjie	XXXMR
U+9822.0	fFourCorner	21680
U+9822.0	fCantonese	kut3; kwut3 (also)
U+9822.0	fMandarin	gua1; huo2
U+9823.0	fUTF8	頣
U+9823.0	fCNS11643	4-4E5A
U+9823.0	fGB	E-452F
U+9823.0	fR/S	 181.6; 181.7
U+9823.0	fTotalStrokes	 15
U+9823.0	fCangjie	SLMBC
U+9823.0	fFourCorner	71780
U+9823.0	fCantonese	can2
U+9823.0	fMandarin	shen3
U+9823.0	fEnglish	 view others with raised eyes
U+9824.0	fUTF8	頤
U+9824.0	fCNS11643	1-7326
U+9824.0	fBig5	C05B
U+9824.0	fGB	1-5243
U+9824.0	fR/S	 181.7; 181.6
U+9824.0	fTotalStrokes	 16
U+9824.0	fCangjie	SLMBC
U+9824.0	fFourCorner	71786
U+9824.0	fHakka	 ji2;  hi1; ri2
U+9824.0	fCantonese	ji4
U+9824.0	fMandarin	yi2
U+9824.0	fEnglish	[1] the cheeks [2] nourish; rear; take care of oneself [3] a Chinese family name
U+9825.0	fUTF8	頥
U+9825.0	fCNS11643	4-5470
U+9825.0	fGB	E-4530
U+9825.0	fR/S	 181.7; 181.8
U+9825.0	fTotalStrokes	 16
U+9825.0	fCangjie	LSLC
U+9825.0	fFourCorner	21080
U+9825.0	fCantonese	ji4
U+9825.0	fMandarin	yi2; shen3
U+9826.0	fUTF8	頦
U+9826.0	fCNS11643	2-5561
U+9826.0	fBig5	E8B8
U+9826.0	fGB	1-7224
U+9826.0	fR/S	 181.6
U+9826.0	fTotalStrokes	 15
U+9826.0	fCangjie	YOMBC
U+9826.0	fFourCorner	01886
U+9826.0	fHakka	 goi1;  hoi2
U+9826.0	fCantonese	hoi4; hoi5 (also); goi2
U+9826.0	fMandarin	hai2; ke2; ke1; gai3; gai1; hai4
U+9826.0	fEnglish	the chin
U+9827.0	fUTF8	頧
U+9827.0	fCNS11643	2-5566
U+9827.0	fBig5	E8BD
U+9827.0	fGB	5-5F7B
U+9827.0	fR/S	 181.6
U+9827.0	fTotalStrokes	 15
U+9827.0	fCangjie	HRMBC
U+9827.0	fFourCorner	21786
U+9827.0	fCantonese	deoi2; deoi1; deoi6
U+9827.0	fMandarin	dui1
U+9828.0	fUTF8	頨
U+9828.0	fCNS11643	2-5563
U+9828.0	fBig5	E8BA
U+9828.0	fGB	5-6B2E
U+9828.0	fR/S	 181.6
U+9828.0	fTotalStrokes	 15
U+9828.0	fCangjie	SMMBC
U+9828.0	fFourCorner	11286
U+9828.0	fCantonese	jyu5; pin3
U+9828.0	fMandarin	pian1; xuan1; yu3; bian4
U+9829.0	fUTF8	頩
U+9829.0	fCNS11643	2-5562
U+9829.0	fBig5	E8B9
U+9829.0	fGB	3-6076
U+9829.0	fR/S	 181.6
U+9829.0	fTotalStrokes	 15
U+9829.0	fCangjie	TTMBC
U+9829.0	fFourCorner	81486
U+9829.0	fCantonese	ping1; ping2
U+9829.0	fMandarin	ping1
U+982A.0	fUTF8	頪
U+982A.0	fCNS11643	3-4F66
U+982A.0	fGB	3-6776
U+982A.0	fR/S	  181.6
U+982A.0	fTotalStrokes	 15
U+982A.0	fCangjie	FDMBC
U+982A.0	fFourCorner	91980
U+982A.0	fCantonese	loi6
U+982A.0	fMandarin	lei4
U+982B.0	fUTF8	頫
U+982B.0	fCNS11643	1-6F6F
U+982B.0	fBig5	BE66
U+982B.0	fGB	E-4531
U+982B.0	fR/S	 181.6
U+982B.0	fTotalStrokes	 15
U+982B.0	fCangjie	LMUOC
U+982B.0	fFourCorner	31286
U+982B.0	fHakka	  tiau5; pu3; fu3; fu5
U+982B.0	fCantonese	fu2; tiu3; fu2tiu3
U+982B.0	fMandarin	fu3; tiao4; wan3; tao1
U+982B.0	fEnglish	 bow one's head; with one's chin down
U+982C.0	fUTF8	頬
U+982C.0	fGB	E-4532
U+982C.0	fR/S	 181.6
U+982C.0	fTotalStrokes	 15
U+982C.0	fCangjie	KTMBC
U+982C.0	fCantonese	gaap3
U+982C.0	fMandarin	jia2
U+982C.0	fEnglish	cheeks; jaw
U+982D.0	fUTF8	頭
U+982D.0	fCNS11643	1-7324
U+982D.0	fBig5	C059
U+982D.0	fGB	1-4D37
U+982D.0	fR/S	 181.7
U+982D.0	fTotalStrokes	 16
U+982D.0	fCangjie	MTMBC
U+982D.0	fFourCorner	11186
U+982D.0	fHakka	  teu2; teu3; tiu2
U+982D.0	fCantonese	tau4; tau2 (also)
U+982D.0	fMandarin	tou2
U+982D.0	fEnglish	[1] the head [2] top; first; the first; beginning [3] chief; boss; leader; head (of a group) [4] the two ends (of anything); side; aspect [5] head (of cattle, etc.) [6] an auxiliary, as a suffix
U+982D.1	fUTF8	頭
U+982D.1	fCNS11643	1-7324
U+982D.1	fBig5	C059
U+982D.1	fGB	1-4D37
U+982D.1	fR/S	 181.7
U+982D.1	fTotalStrokes	 16
U+982D.1	fCangjie	MTMBC
U+982D.1	fFourCorner	11186
U+982D.1	fMandarin	tou5
U+982D.1	fEnglish	[1] a suffix indicating positions or directions [2] as a suffix to certains verbs to indicate the worthiness
U+982E.0	fUTF8	頮
U+982E.0	fCNS11643	3-535C
U+982E.0	fGB	5-6027
U+982E.0	fR/S	 181.7
U+982E.0	fTotalStrokes	 16
U+982E.0	fCangjie	ETMBC
U+982E.0	fFourCorner	11480
U+982E.0	fHakka	 fui5; fi5
U+982E.0	fCantonese	fui3
U+982E.0	fMandarin	hui4
U+982E.0	fEnglish	wash
U+982F.0	fUTF8	頯
U+982F.0	fCNS11643	2-5B66
U+982F.0	fBig5	EC5A
U+982F.0	fGB	3-607B
U+982F.0	fR/S	 181.7
U+982F.0	fTotalStrokes	 16
U+982F.0	fCangjie	BTMBC
U+982F.0	fFourCorner	21486
U+982F.0	fCantonese	kwai4; kau4 (also)
U+982F.0	fMandarin	kui2; qiu2; kuai4; kui3; gui3
U+982F.0	fEnglish	cheekbone
U+9830.0	fUTF8	頰
U+9830.0	fCNS11643	1-727E
U+9830.0	fBig5	C055
U+9830.0	fGB	1-3C55
U+9830.0	fR/S	 181.7
U+9830.0	fTotalStrokes	 16
U+9830.0	fCangjie	KOMBC
U+9830.0	fFourCorner	41086
U+9830.0	fHakka	  giap7;  gap7
U+9830.0	fCantonese	gaap3; haap3 (also)
U+9830.0	fMandarin	jia2
U+9830.0	fEnglish	the cheeks
U+9831.0	fUTF8	頱
U+9831.0	fCNS11643	3-5359
U+9831.0	fGB	E-4533
U+9831.0	fR/S	  181.7
U+9831.0	fTotalStrokes	 16
U+9831.0	fCangjie	BIMBC
U+9831.0	fFourCorner	21380
U+9831.0	fCantonese	lyut3
U+9831.0	fMandarin	le4
U+9832.0	fUTF8	頲
U+9832.0	fCNS11643	2-5B67
U+9832.0	fBig5	EC5B
U+9832.0	fGB	3-6072
U+9832.0	fR/S	 181.7
U+9832.0	fTotalStrokes	 16
U+9832.0	fCangjie	NGMBC
U+9832.0	fFourCorner	11486
U+9832.0	fHakka	 tin3
U+9832.0	fCantonese	ting5; ting2
U+9832.0	fMandarin	ting3; ting1; tian5
U+9832.0	fEnglish	straight
U+9833.0	fUTF8	頳
U+9833.0	fCNS11643	3-535A
U+9833.0	fGB	5-6D70
U+9833.0	fR/S	 181.7
U+9833.0	fTotalStrokes	 16
U+9833.0	fCangjie	GCMBC
U+9833.0	fFourCorner	41380
U+9833.0	fCantonese	cing1
U+9833.0	fMandarin	cheng1
U+9833.0	fEnglish	red
U+9834.0	fUTF8	頴
U+9834.0	fCNS11643	3-5228
U+9834.0	fGB	E-4534
U+9834.0	fR/S	 181.7
U+9834.0	fTotalStrokes	 16
U+9834.0	fCangjie	PFMBC
U+9834.0	fFourCorner	21980
U+9834.0	fHakka	  jin3; rin3
U+9834.0	fCantonese	wing6
U+9834.0	fMandarin	ying3
U+9834.0	fEnglish	 rice tassel sharp point clever
U+9835.0	fUTF8	頵
U+9835.0	fCNS11643	2-5B65
U+9835.0	fBig5	EC59
U+9835.0	fGB	3-607D
U+9835.0	fR/S	 181.7
U+9835.0	fTotalStrokes	 16
U+9835.0	fCangjie	SRMBC
U+9835.0	fFourCorner	11686
U+9835.0	fCantonese	wan1; gwan1 (also); gwang1
U+9835.0	fMandarin	jun1; yun1; jun4
U+9835.0	fEnglish	large-headed; top-heavy
U+9836.0	fUTF8	頶
U+9836.0	fCNS11643	3-5358
U+9836.0	fGB	E-4535
U+9836.0	fR/S	  181.7
U+9836.0	fTotalStrokes	 16
U+9836.0	fCangjie	XXHRM
U+9836.0	fFourCorner	21680
U+9836.0	fCantonese	huk6
U+9836.0	fMandarin	hu2
U+9837.0	fUTF8	頷
U+9837.0	fCNS11643	1-7323
U+9837.0	fBig5	C058
U+9837.0	fGB	1-7225
U+9837.0	fR/S	 181.7
U+9837.0	fTotalStrokes	 16
U+9837.0	fCangjie	ORMBC
U+9837.0	fFourCorner	81686
U+9837.0	fHakka	  ngam3;  ngam1; ngap7;  ham3
U+9837.0	fCantonese	ham5
U+9837.0	fMandarin	han4; an3; han2
U+9837.0	fEnglish	[1] the chin; jaws [2] give nod; slight nod of the head
U+9838.0	fUTF8	頸
U+9838.0	fCNS11643	1-7321
U+9838.0	fBig5	C056
U+9838.0	fGB	1-3E31
U+9838.0	fR/S	 181.7
U+9838.0	fTotalStrokes	 16
U+9838.0	fCangjie	MMMBC
U+9838.0	fFourCorner	11186
U+9838.0	fHakka	  giang3
U+9838.0	fCantonese	geng2
U+9838.0	fMandarin	jing3; geng3; qing2
U+9838.0	fEnglish	the neck; throat
U+9839.0	fUTF8	頹
U+9839.0	fCNS11643	1-7325
U+9839.0	fBig5	C05A
U+9839.0	fGB	E-4536
U+9839.0	fR/S	 181.7
U+9839.0	fTotalStrokes	 16
U+9839.0	fCangjie	HUMBC
U+9839.0	fFourCorner	21286
U+9839.0	fHakka	 tui1
U+9839.0	fCantonese	teoi4
U+9839.0	fMandarin	tui2; tuo4
U+9839.0	fEnglish	[1] crumble; disintegrated; ruined; dilapidated; collapse [2] weakened; withered; emaciated; declining; decadend [3] bald [4] descend; cascade down [5] a Chinese family name
U+983A.0	fUTF8	頺
U+983A.0	fCNS11643	3-535D
U+983A.0	fGB	E-4537
U+983A.0	fR/S	 181.7; 181.8
U+983A.0	fTotalStrokes	 16; 17
U+983A.0	fCangjie	HSMBC
U+983A.0	fFourCorner	21380
U+983A.0	fCantonese	teoi4
U+983A.0	fMandarin	tui2
U+983B.0	fUTF8	頻
U+983B.0	fCNS11643	1-7322
U+983B.0	fBig5	C057
U+983B.0	fGB	1-4635
U+983B.0	fR/S	 181.7; 181.8
U+983B.0	fTotalStrokes	 16
U+983B.0	fCangjie	YHMBC
U+983B.0	fFourCorner	21286
U+983B.0	fHakka	  pin2; pin3
U+983B.0	fCantonese	pan4; zi1; ban1; laai6; teoi4
U+983B.0	fMandarin	pin2; bin1
U+983B.0	fEnglish	[1] incessand; successive; continuous; frequently; again and again; repeatedly [2] urgent; precarious [3] same as $ [4] frequency
U+983C.0	fUTF8	頼
U+983C.0	fCNS11643	3-535B
U+983C.0	fGB	E-4538
U+983C.0	fR/S	 181.7
U+983C.0	fTotalStrokes	 16
U+983C.0	fCangjie	DLMBC
U+983C.0	fFourCorner	51980
U+983C.0	fHakka	 lai5; lai2; lai6
U+983C.0	fCantonese	laai6
U+983C.0	fMandarin	lai4
U+983C.0	fEnglish	 rely; depend on accuse falsely
U+983D.0	fUTF8	頽
U+983D.0	fCNS11643	3-535F
U+983D.0	fGB	1-4D47
U+983D.0	fR/S	 181.7
U+983D.0	fTotalStrokes	 16
U+983D.0	fCangjie	HNMBC
U+983D.0	fFourCorner	21280
U+983D.0	fHakka	  tui1; tui3
U+983D.0	fMandarin	tui2
U+983D.0	fEnglish	ruined
U+983E.0	fUTF8	頾
U+983E.0	fCNS11643	4-5E4F
U+983E.0	fGB	E-4539
U+983E.0	fR/S	  181.9
U+983E.0	fTotalStrokes	 18
U+983E.0	fCangjie	HCYMP
U+983E.0	fFourCorner	21110
U+983E.0	fMandarin	zi1
U+983E.0	fEnglish	beard
U+983F.0	fUTF8	頿
U+983F.0	fCNS11643	3-595B
U+983F.0	fGB	5-6039
U+983F.0	fR/S	 181.8
U+983F.0	fTotalStrokes	 18; 17
U+983F.0	fCangjie	HPMBC
U+983F.0	fFourCorner	21180
U+983F.0	fMandarin	zi1
U+9840.0	fUTF8	顀
U+9840.0	fCNS11643	3-5726
U+9840.0	fGB	3-7528
U+9840.0	fR/S	 181.8
U+9840.0	fTotalStrokes	 17
U+9840.0	fCangjie	OGMBC
U+9840.0	fFourCorner	21280
U+9840.0	fHakka	 ch'ui2; cui2; chui2
U+9840.0	fCantonese	ceoi4
U+9840.0	fMandarin	chui2
U+9840.0	fEnglish	projecting forehead
U+9841.0	fUTF8	顁
U+9841.0	fCNS11643	2-6051
U+9841.0	fBig5	EF45
U+9841.0	fGB	5-6036
U+9841.0	fR/S	 181.8
U+9841.0	fTotalStrokes	 17
U+9841.0	fCangjie	JOMBC
U+9841.0	fFourCorner	31886
U+9841.0	fCantonese	ding3; ding2
U+9841.0	fMandarin	ding4; ding3
U+9842.0	fUTF8	顂
U+9842.0	fCNS11643	4-5970
U+9842.0	fGB	5-6022
U+9842.0	fR/S	  181.8
U+9842.0	fTotalStrokes	 17; 16
U+9842.0	fCangjie	DOMBC
U+9842.0	fFourCorner	41980
U+9842.0	fCantonese	laai6
U+9842.0	fMandarin	lai4; lai2
U+9843.0	fUTF8	顃
U+9843.0	fCNS11643	2-6056
U+9843.0	fBig5	EF4A
U+9843.0	fGB	E-453A
U+9843.0	fR/S	 181.8
U+9843.0	fTotalStrokes	 17
U+9843.0	fCangjie	FFMBC
U+9843.0	fFourCorner	91886
U+9843.0	fCantonese	taam4
U+9843.0	fMandarin	yan2; tan2
U+9844.0	fUTF8	顄
U+9844.0	fCNS11643	2-6052
U+9844.0	fBig5	EF46
U+9844.0	fGB	5-6037
U+9844.0	fR/S	 181.8
U+9844.0	fTotalStrokes	 17
U+9844.0	fCangjie	UEMBC
U+9844.0	fFourCorner	11786
U+9844.0	fHakka	  ngam3;  ham3; ngam1; ngap7
U+9844.0	fCantonese	ham5
U+9844.0	fMandarin	han4; han2
U+9844.0	fEnglish	chin; shake the head
U+9845.0	fUTF8	顅
U+9845.0	fCNS11643	2-6055
U+9845.0	fBig5	EF49
U+9845.0	fGB	3-6121
U+9845.0	fR/S	 181.8
U+9845.0	fTotalStrokes	 17
U+9845.0	fCangjie	HBMBC
U+9845.0	fFourCorner	71286
U+9845.0	fCantonese	haan1
U+9845.0	fMandarin	jian1; qian1
U+9846.0	fUTF8	顆
U+9846.0	fCNS11643	1-7628
U+9846.0	fBig5	C1FB
U+9846.0	fGB	1-3F45
U+9846.0	fR/S	 181.8
U+9846.0	fTotalStrokes	 17
U+9846.0	fCangjie	WDMBC
U+9846.0	fFourCorner	61986
U+9846.0	fHakka	 ko5;  ko3; ko1
U+9846.0	fCantonese	fo2; fo3
U+9846.0	fMandarin	ke1; ke3; kuan3
U+9846.0	fEnglish	 drop; droplet; grain; kernel; pill; numerical classifier for boms, bullets, etc.
U+9847.0	fUTF8	顇
U+9847.0	fCNS11643	3-5728
U+9847.0	fGB	E-453B
U+9847.0	fR/S	  181.8
U+9847.0	fTotalStrokes	 17
U+9847.0	fCangjie	YJMBC
U+9847.0	fFourCorner	01480
U+9847.0	fHakka	 cui5; zut7; cui6; cut8
U+9847.0	fCantonese	seoi6; seoi5 (also)
U+9847.0	fMandarin	cui4; zu2
U+9847.0	fEnglish	correct form of &#xhaggard (from grief or anxiety);
U+9848.0	fUTF8	顈
U+9848.0	fCNS11643	2-5E41
U+9848.0	fBig5	EDD4
U+9848.0	fGB	5-6035
U+9848.0	fR/S	 120.11; 181.8
U+9848.0	fTotalStrokes	 17
U+9848.0	fCangjie	PFMBC
U+9848.0	fFourCorner	21986
U+9848.0	fCantonese	gwing2; king2
U+9848.0	fMandarin	jiong3; jiong1; xian3; xian4
U+9849.0	fUTF8	顉
U+9849.0	fCNS11643	2-6054
U+9849.0	fBig5	EF48
U+9849.0	fGB	5-754B
U+9849.0	fR/S	 181.8
U+9849.0	fTotalStrokes	 17
U+9849.0	fCangjie	CMBC
U+9849.0	fFourCorner	81186
U+9849.0	fCantonese	ham6; jam1 (also)
U+9849.0	fMandarin	qin1; an3; qin3; han4
U+984A.0	fUTF8	顊
U+984A.0	fCNS11643	2-6053
U+984A.0	fBig5	EF47
U+984A.0	fGB	5-7460
U+984A.0	fR/S	 181.8
U+984A.0	fTotalStrokes	 17
U+984A.0	fCangjie	HJMBC
U+984A.0	fFourCorner	21486
U+984A.0	fCantonese	ji4
U+984A.0	fMandarin	yi2
U+984B.0	fUTF8	顋
U+984B.0	fCNS11643	3-5959
U+984B.0	fGB	E-453C
U+984B.0	fR/S	 181.9
U+984B.0	fTotalStrokes	 18
U+984B.0	fCangjie	WPMBC
U+984B.0	fFourCorner	61380
U+984B.0	fHakka	 soi1; sii1
U+984B.0	fCantonese	soi1
U+984B.0	fMandarin	sai1
U+984B.0	fEnglish	 lower part of face jaw gills of a fish
U+984C.0	fUTF8	題
U+984C.0	fCNS11643	1-776E
U+984C.0	fBig5	C344
U+984C.0	fGB	1-4C62
U+984C.0	fR/S	 181.9
U+984C.0	fTotalStrokes	 18
U+984C.0	fCangjie	AOMBC
U+984C.0	fFourCorner	61808
U+984C.0	fHakka	  ti2; ti3
U+984C.0	fCantonese	tai4; tai2 (also)
U+984C.0	fMandarin	ti2; di4
U+984C.0	fEnglish	[1] forehead [2] signal; sign [3] subject; title of a composition or speech; headline; theme [4] commentaries; notes [5] sign; write; inscribe [6] end; top [7] the ornamental woodwork under the eaves of public buildings [8] a Chinese family name
U+984D.0	fUTF8	額
U+984D.0	fCNS11643	1-776C
U+984D.0	fBig5	C342
U+984D.0	fGB	1-366E
U+984D.0	fR/S	 181.9
U+984D.0	fTotalStrokes	 18
U+984D.0	fCangjie	JRMBC
U+984D.0	fFourCorner	31686
U+984D.0	fHakka	 ngiak7; gniak7; ngak7; siung5
U+984D.0	fCantonese	ngaak6; aak6 (also); ngaak2 (also); aak2 (also)
U+984D.0	fMandarin	e2
U+984D.0	fEnglish	[1] the forehead [2] a fixed amount; value; number, etc.; quota [3] horizontal tablet or plaque [4] a Chinese family name
U+984E.0	fUTF8	顎
U+984E.0	fCNS11643	1-776F
U+984E.0	fBig5	C345
U+984E.0	fGB	1-7226
U+984E.0	fR/S	 181.9
U+984E.0	fTotalStrokes	 18
U+984E.0	fCangjie	RSMBC
U+984E.0	fFourCorner	61286
U+984E.0	fHakka	 ngok8
U+984E.0	fCantonese	ngok6; ok6 (also); ngok2 (also); ok2 (also)
U+984E.0	fMandarin	e4
U+984E.0	fEnglish	[1] the jowl; cheek bones; jaw [2] high-cheekboned [3] reverence
U+984F.0	fUTF8	顏
U+984F.0	fCNS11643	1-776D
U+984F.0	fBig5	C343
U+984F.0	fGB	E-453D
U+984F.0	fR/S	 181.9
U+984F.0	fTotalStrokes	 18
U+984F.0	fCangjie	YHMBC
U+984F.0	fFourCorner	01286
U+984F.0	fHakka	  ngan2;  ngien2; gnian3; ngian2
U+984F.0	fCantonese	ngaan4; aan4 (also)
U+984F.0	fMandarin	yan2; yai2; ya2
U+984F.0	fEnglish	[1] face (physically); facial appearance; countenance; features [2] reputation; prestige; face (figuratively)
U+9850.0	fUTF8	顐
U+9850.0	fCNS11643	2-6452
U+9850.0	fBig5	F1A8
U+9850.0	fGB	5-6021
U+9850.0	fR/S	 181.9
U+9850.0	fTotalStrokes	 18
U+9850.0	fCangjie	BJMBC
U+9850.0	fFourCorner	71586
U+9850.0	fCantonese	wan6
U+9850.0	fMandarin	hun2; wen4; hun4; wen2
U+9851.0	fUTF8	顑
U+9851.0	fCNS11643	2-6453
U+9851.0	fBig5	F1A9
U+9851.0	fGB	3-6125
U+9851.0	fR/S	 181.9
U+9851.0	fTotalStrokes	 18
U+9851.0	fCangjie	IRMBC
U+9851.0	fFourCorner	51286
U+9851.0	fCantonese	ham2
U+9851.0	fMandarin	kan3; kan4; han3; yan4
U+9851.0	fEnglish	yellow
U+9852.0	fUTF8	顒
U+9852.0	fCNS11643	2-6454
U+9852.0	fBig5	F1AA
U+9852.0	fGB	3-6126
U+9852.0	fR/S	 181.9
U+9852.0	fTotalStrokes	 18
U+9852.0	fCangjie	WBMBC
U+9852.0	fFourCorner	61286
U+9852.0	fHakka	 jung2; rung2
U+9852.0	fCantonese	jung4
U+9852.0	fMandarin	yong2
U+9852.0	fEnglish	[1] severe [2] large; great [3] large-headed
U+9853.0	fUTF8	顓
U+9853.0	fCNS11643	1-7770
U+9853.0	fBig5	C346
U+9853.0	fGB	1-7227
U+9853.0	fR/S	 181.9
U+9853.0	fTotalStrokes	 18
U+9853.0	fCangjie	UBMBC
U+9853.0	fFourCorner	21286
U+9853.0	fHakka	 zhon1; zon1
U+9853.0	fCantonese	zyun1
U+9853.0	fMandarin	zhuan1
U+9853.0	fEnglish	[1] cautious [2] stupid; dull; ignorant [3] a Chinese family name
U+9854.0	fUTF8	顔
U+9854.0	fCNS11643	3-5956
U+9854.0	fGB	1-5155
U+9854.0	fR/S	 181.9
U+9854.0	fTotalStrokes	 18
U+9854.0	fCangjie	YHMBC
U+9854.0	fFourCorner	01280
U+9854.0	fCantonese	ngaan4; aan4 (also)
U+9854.0	fMandarin	yan2; ya2
U+9854.0	fEnglish	face; facial appearance
U+9855.0	fUTF8	顕
U+9855.0	fCNS11643	3-5958
U+9855.0	fGB	E-453E
U+9855.0	fR/S	 181.9
U+9855.0	fTotalStrokes	 18
U+9855.0	fCangjie	ACMBC
U+9855.0	fFourCorner	61180
U+9855.0	fCantonese	hin2
U+9855.0	fMandarin	xian3
U+9855.0	fEnglish	 manifest; display evident;clear
U+9856.0	fUTF8	顖
U+9856.0	fCNS11643	3-5B65
U+9856.0	fGB	5-603C
U+9856.0	fR/S	 181.10
U+9856.0	fTotalStrokes	 19
U+9856.0	fCangjie	XHPMB
U+9856.0	fFourCorner	21380
U+9856.0	fHakka	 sin5;  sim5; sim3
U+9856.0	fCantonese	seon3; seon2 (also); ngai5; ai5 (also); jyun6; jyun2 (also)
U+9856.0	fMandarin	xin4
U+9856.0	fEnglish	the part of the human head from the top to the forehead
U+9857.0	fUTF8	顗
U+9857.0	fCNS11643	2-6775
U+9857.0	fBig5	F3AA
U+9857.0	fGB	3-6071
U+9857.0	fR/S	 181.10
U+9857.0	fTotalStrokes	 19
U+9857.0	fCangjie	UTMBC
U+9857.0	fFourCorner	21186
U+9857.0	fHakka	  koi3; hi3; ngi3
U+9857.0	fCantonese	ngai5; ai5
U+9857.0	fMandarin	yi3
U+9857.0	fEnglish	quiet; tranquil; solemn
U+9858.0	fUTF8	願
U+9858.0	fCNS11643	1-794C
U+9858.0	fBig5	C440
U+9858.0	fGB	1-5438
U+9858.0	fR/S	 181.10
U+9858.0	fTotalStrokes	 19
U+9858.0	fCangjie	MFMBC
U+9858.0	fFourCorner	71286
U+9858.0	fHakka	  ngien5; nien5; ngien6; gnian6; ngian5
U+9858.0	fCantonese	jyun6
U+9858.0	fMandarin	yuan4; ruan3
U+9858.0	fEnglish	[1] be willing; be desirous of; desire; want; hope; wish [2] anything one wishes or desires; ambition; aspiration [3] vow [4] think
U+9859.0	fUTF8	顙
U+9859.0	fCNS11643	2-6773
U+9859.0	fBig5	F3A8
U+9859.0	fGB	1-722A
U+9859.0	fR/S	 181.10
U+9859.0	fTotalStrokes	 19
U+9859.0	fCangjie	EDMBC
U+9859.0	fFourCorner	11986
U+9859.0	fHakka	  song3; song1
U+9859.0	fCantonese	song2
U+9859.0	fMandarin	sang3; sang1
U+9859.0	fEnglish	[1] the forehead [2] kowtow
U+985A.0	fUTF8	顚
U+985A.0	fCNS11643	4-6230
U+985A.0	fGB	E-453F
U+985A.0	fR/S	 181.10
U+985A.0	fTotalStrokes	 19
U+985A.0	fCangjie	PCMBC
U+985A.0	fFourCorner	21880
U+985A.0	fCantonese	din1
U+985A.0	fMandarin	dian1
U+985A.0	fEnglish	 top; peak; summit upset
U+985B.0	fUTF8	顛
U+985B.0	fCNS11643	1-794D
U+985B.0	fBig5	C441
U+985B.0	fGB	1-355F
U+985B.0	fR/S	 181.10
U+985B.0	fTotalStrokes	 19
U+985B.0	fCangjie	JCMBC
U+985B.0	fFourCorner	21886
U+985B.0	fHakka	 dien1; den1; tian1; din5
U+985B.0	fCantonese	din1
U+985B.0	fMandarin	dian1; tian2; tian4
U+985B.0	fEnglish	[1] top; peak; summit; highesst spot; head [2] fall; topple; upset [3] jolt; bump [4] upside down [5] mad; lunatic
U+985C.0	fUTF8	顜
U+985C.0	fCNS11643	2-6772
U+985C.0	fBig5	F3A7
U+985C.0	fGB	3-612D
U+985C.0	fR/S	 181.10
U+985C.0	fTotalStrokes	 19
U+985C.0	fCangjie	TBMBC
U+985C.0	fFourCorner	51486
U+985C.0	fCantonese	gong2; gok3 (also)
U+985C.0	fMandarin	jiang3; jue2; jiao4
U+985D.0	fUTF8	顝
U+985D.0	fCNS11643	2-6774
U+985D.0	fBig5	F3A9
U+985D.0	fGB	3-7975
U+985D.0	fR/S	 181.10
U+985D.0	fTotalStrokes	 19
U+985D.0	fCangjie	BBMBC
U+985D.0	fFourCorner	71286
U+985D.0	fCantonese	fui1
U+985D.0	fMandarin	ku1; kui1; kun1; kui3; kua3
U+985E.0	fUTF8	類
U+985E.0	fCNS11643	1-794B
U+985E.0	fBig5	C3FE
U+985E.0	fGB	1-4060
U+985E.0	fR/S	 181.10; 181.9
U+985E.0	fTotalStrokes	 19
U+985E.0	fCangjie	FKMBC
U+985E.0	fFourCorner	91486
U+985E.0	fHakka	  lui5; lui6; lui3
U+985E.0	fCantonese	leoi6; leoi2 (also)
U+985E.0	fMandarin	lei4; luu4; li4
U+985E.0	fEnglish	[1] species; class; group; kind; category; race [2] similar; alike [3] good; vitue [4] a kind of wildcat [5] biased; prejudiced [6] a Chinese family name
U+985F.0	fUTF8	顟
U+985F.0	fCNS11643	2-6A5F
U+985F.0	fBig5	F551
U+985F.0	fGB	3-6138
U+985F.0	fR/S	 181.11
U+985F.0	fTotalStrokes	 20
U+985F.0	fCangjie	SHMBC
U+985F.0	fFourCorner	11286
U+985F.0	fCantonese	laau4; liu4
U+985F.0	fMandarin	liao2; liao4
U+9860.0	fUTF8	顠
U+9860.0	fCNS11643	2-6A5C
U+9860.0	fBig5	F54E
U+9860.0	fGB	3-6135
U+9860.0	fR/S	 181.11
U+9860.0	fTotalStrokes	 20
U+9860.0	fCangjie	MFMBC
U+9860.0	fFourCorner	11986
U+9860.0	fCantonese	piu5
U+9860.0	fMandarin	piao3; piao1; piao4
U+9861.0	fUTF8	顡
U+9861.0	fCNS11643	4-652C
U+9861.0	fGB	3-6137
U+9861.0	fR/S	 181.11
U+9861.0	fTotalStrokes	 20
U+9861.0	fCangjie	YOMBC
U+9861.0	fFourCorner	01280
U+9861.0	fCantonese	wui6
U+9861.0	fMandarin	yi4; kui4; wei3
U+9862.0	fUTF8	顢
U+9862.0	fCNS11643	2-6A5D
U+9862.0	fBig5	F54F
U+9862.0	fGB	1-7229
U+9862.0	fR/S	 181.11
U+9862.0	fTotalStrokes	 20
U+9862.0	fCangjie	TBMBC
U+9862.0	fFourCorner	41286
U+9862.0	fHakka	  man2
U+9862.0	fCantonese	mun4; mun1 (also)
U+9862.0	fMandarin	man1; man2
U+9862.0	fEnglish	ignorant; thoughtless; careless; stupid
U+9863.0	fUTF8	顣
U+9863.0	fCNS11643	2-6A5E
U+9863.0	fBig5	F550
U+9863.0	fGB	5-6040
U+9863.0	fR/S	 181.11
U+9863.0	fTotalStrokes	 20
U+9863.0	fCangjie	IFMBC
U+9863.0	fFourCorner	51286
U+9863.0	fHakka	  cuk7
U+9863.0	fCantonese	cuk1; zuk1
U+9863.0	fMandarin	cu4; qi1
U+9863.0	fEnglish	frown
U+9864.0	fUTF8	顤
U+9864.0	fCNS11643	2-6C60
U+9864.0	fBig5	F672
U+9864.0	fGB	3-606F
U+9864.0	fR/S	 181.12
U+9864.0	fTotalStrokes	 21
U+9864.0	fCangjie	GUMBC
U+9864.0	fFourCorner	41286
U+9864.0	fCantonese	jiu4; kiu4
U+9864.0	fMandarin	rao4; yao4; yao2; qiao4; xiao1
U+9865.0	fUTF8	顥
U+9865.0	fCNS11643	1-7B42
U+9865.0	fBig5	C556
U+9865.0	fGB	1-722B
U+9865.0	fR/S	 181.12
U+9865.0	fTotalStrokes	 21
U+9865.0	fCangjie	AFMBC
U+9865.0	fFourCorner	61986
U+9865.0	fHakka	 hau3; hau1
U+9865.0	fCantonese	hou6
U+9865.0	fMandarin	hao4
U+9865.0	fEnglish	[1] luminous; bright [2] white; hoary (hair) [3] very great; very large
U+9866.0	fUTF8	顦
U+9866.0	fCNS11643	3-5E78
U+9866.0	fGB	E-4540
U+9866.0	fR/S	 181.12
U+9866.0	fTotalStrokes	 21
U+9866.0	fCangjie	OFMBC
U+9866.0	fFourCorner	21380
U+9866.0	fHakka	 ziau1
U+9866.0	fCantonese	ciu4
U+9866.0	fMandarin	qiao2
U+9866.0	fEnglish	haggard
U+9867.0	fUTF8	顧
U+9867.0	fCNS11643	1-7B41
U+9867.0	fBig5	C555
U+9867.0	fGB	1-394B
U+9867.0	fR/S	 181.12
U+9867.0	fTotalStrokes	 21
U+9867.0	fCangjie	HGMBC
U+9867.0	fFourCorner	31286
U+9867.0	fHakka	  gu5
U+9867.0	fCantonese	gu3; gwu3 (also)
U+9867.0	fMandarin	gu4; gu3
U+9867.0	fEnglish	[1] gaze; look at [2] turn the head around and look; look back [3] look after; attend to; mind; care for; concern oneself about; regard [4] visit; call on [5] however; but; nevertheless [6] indeed; realy [7] a Chinese family name
U+9868.0	fUTF8	顨
U+9868.0	fCNS11643	3-5E77
U+9868.0	fGB	5-372A
U+9868.0	fR/S	  181.12
U+9868.0	fTotalStrokes	 21
U+9868.0	fCangjie	MCML
U+9868.0	fFourCorner	11448
U+9868.0	fCantonese	seon3
U+9868.0	fMandarin	xun4; sun4
U+9869.0	fUTF8	顩
U+9869.0	fCNS11643	2-6E43
U+9869.0	fBig5	F774
U+9869.0	fGB	3-607A
U+9869.0	fR/S	 181.13
U+9869.0	fTotalStrokes	 22
U+9869.0	fCangjie	OOMBC
U+9869.0	fFourCorner	81886
U+9869.0	fCantonese	jim5
U+9869.0	fMandarin	qian1; yan3; qin1; qian3; zhen4; han4
U+986A.0	fUTF8	顪
U+986A.0	fCNS11643	2-6E42
U+986A.0	fBig5	F773
U+986A.0	fGB	3-6070
U+986A.0	fR/S	 181.13
U+986A.0	fTotalStrokes	 22
U+986A.0	fCangjie	YHMBC
U+986A.0	fFourCorner	21286
U+986A.0	fCantonese	fui3; wai6
U+986A.0	fMandarin	hui1; hui4
U+986B.0	fUTF8	顫
U+986B.0	fCNS11643	1-7C24
U+986B.0	fBig5	C5B8
U+986B.0	fGB	1-327C
U+986B.0	fR/S	 181.13
U+986B.0	fTotalStrokes	 22
U+986B.0	fCangjie	YMMBC
U+986B.0	fFourCorner	01186
U+986B.0	fHakka	 zen5; zhan5; zan5; zhun1; zun1; zhen1; zhen5
U+986B.0	fCantonese	zin3
U+986B.0	fMandarin	zhan4; chan4; shan1; zhan3
U+986B.0	fEnglish	 shake; shiver; tremble; trembling; quiver; vibrate
U+986C.0	fUTF8	顬
U+986C.0	fCNS11643	3-605D
U+986C.0	fGB	1-722C
U+986C.0	fR/S	  181.14
U+986C.0	fTotalStrokes	 23
U+986C.0	fCangjie	MBMBC
U+986C.0	fFourCorner	11280
U+986C.0	fCantonese	jyu4
U+986C.0	fMandarin	ru2
U+986C.0	fEnglish	used in combination
U+986D.0	fUTF8	顭
U+986D.0	fCNS11643	3-605E
U+986D.0	fGB	E-4541
U+986D.0	fR/S	  181.14
U+986D.0	fTotalStrokes	 23
U+986D.0	fCangjie	TNMBC
U+986D.0	fFourCorner	41280
U+986D.0	fCantonese	mang6
U+986D.0	fMandarin	hong1; meng2
U+986E.0	fUTF8	顮
U+986E.0	fCNS11643	3-605C
U+986E.0	fGB	3-6134
U+986E.0	fR/S	  181.14
U+986E.0	fTotalStrokes	 23
U+986E.0	fCangjie	XJCMB
U+986E.0	fFourCorner	31880
U+986E.0	fCantonese	ban1
U+986E.0	fMandarin	bin1; pin2
U+986F.0	fUTF8	顯
U+986F.0	fCNS11643	1-7C4F
U+986F.0	fBig5	C5E3
U+986F.0	fGB	1-4F54
U+986F.0	fR/S	 181.14
U+986F.0	fTotalStrokes	 23
U+986F.0	fCangjie	AFMBC
U+986F.0	fFourCorner	61386
U+986F.0	fHakka	  hien3; hen3; hian3
U+986F.0	fCantonese	hin2
U+986F.0	fMandarin	xian3; xian4
U+986F.0	fEnglish	[1] manifest; evident; clear [2] high-positioned; eminent; prominent [3] well-known; renowned; famed; reputed [4] expose; make known; display; show; manifest [5] a prefix referring to one's forebears
U+9870.0	fUTF8	顰
U+9870.0	fCNS11643	1-7C74
U+9870.0	fBig5	C649
U+9870.0	fGB	1-722D
U+9870.0	fR/S	 181.15
U+9870.0	fTotalStrokes	 24
U+9870.0	fCangjie	YCHWJ
U+9870.0	fFourCorner	21406
U+9870.0	fHakka	 pin2
U+9870.0	fCantonese	pan4
U+9870.0	fMandarin	pin2; pi2
U+9870.0	fEnglish	 frown; knit one's brows
U+9871.0	fUTF8	顱
U+9871.0	fCNS11643	1-7D2D
U+9871.0	fBig5	C660
U+9871.0	fGB	1-422D
U+9871.0	fR/S	 181.16
U+9871.0	fTotalStrokes	 25
U+9871.0	fCangjie	YTMBC
U+9871.0	fFourCorner	21286
U+9871.0	fHakka	 lu2
U+9871.0	fCantonese	lou4
U+9871.0	fMandarin	lu2
U+9871.0	fEnglish	[1] the skull [2] the head [3] the forehead
U+9872.0	fUTF8	顲
U+9872.0	fCNS11643	2-7147
U+9872.0	fBig5	F958
U+9872.0	fGB	3-6140
U+9872.0	fR/S	 181.16
U+9872.0	fTotalStrokes	 25
U+9872.0	fCangjie	FWMBC
U+9872.0	fFourCorner	91686
U+9872.0	fCantonese	lam5
U+9872.0	fMandarin	lan3; lin3; lan4
U+9873.0	fUTF8	顳
U+9873.0	fCNS11643	2-717C
U+9873.0	fBig5	F9AE
U+9873.0	fGB	1-7228
U+9873.0	fR/S	 181.18
U+9873.0	fTotalStrokes	 27
U+9873.0	fCangjie	SJMBC
U+9873.0	fFourCorner	11186
U+9873.0	fCantonese	nip6; lip6 (also); jip6
U+9873.0	fMandarin	nie4; huan4
U+9873.0	fEnglish	the temporal bone; the temple
U+9874.0	fUTF8	顴
U+9874.0	fCNS11643	2-717D
U+9874.0	fBig5	F9AF
U+9874.0	fGB	1-4827
U+9874.0	fR/S	 181.18
U+9874.0	fTotalStrokes	 27
U+9874.0	fCangjie	TGMBC
U+9874.0	fFourCorner	41286
U+9874.0	fHakka	 ken2; kien2; kian2
U+9874.0	fCantonese	kyun4
U+9874.0	fMandarin	quan2
U+9874.0	fEnglish	the cheek bones
U+9875.0	fUTF8	页
U+9875.0	fGB	0-5233
U+9875.0	fR/S	 181.0
U+9875.0	fTotalStrokes	 6
U+9875.0	fCangjie	MBO
U+9875.0	fHakka	 jap8; get7; giet7
U+9875.0	fCantonese	jip6
U+9875.0	fMandarin	ye4; shou3; xie2
U+9875.0	fEnglish	simplified form of &#x9801;
U+9876.0	fUTF8	顶
U+9876.0	fGB	0-3625
U+9876.0	fR/S	 181.2
U+9876.0	fTotalStrokes	 8
U+9876.0	fCangjie	MNMBO
U+9876.0	fHakka	 dang3; den3; din1
U+9876.0	fCantonese	ding2; deng2
U+9876.0	fMandarin	ding3
U+9876.0	fEnglish	simplified form of &#x9802;
U+9877.0	fUTF8	顷
U+9877.0	fGB	0-476A
U+9877.0	fR/S	 181.2
U+9877.0	fTotalStrokes	 8
U+9877.0	fCangjie	PMBO
U+9877.0	fHakka	 kin3; ken3
U+9877.0	fCantonese	king2; king1
U+9877.0	fMandarin	qing3; qing1; kui3
U+9877.0	fEnglish	simplified form of &#x9803;
U+9878.0	fUTF8	顸
U+9878.0	fGB	0-717C
U+9878.0	fR/S	 51.6; 181.3
U+9878.0	fTotalStrokes	 9
U+9878.0	fCangjie	MJMBO
U+9878.0	fHakka	 hon2
U+9878.0	fCantonese	hon1
U+9878.0	fMandarin	han1; an4; e4; han2; kan1
U+9878.0	fEnglish	simplified form of &#x9807;
U+9879.0	fUTF8	项
U+9879.0	fGB	0-4F6E
U+9879.0	fR/S	 181.3
U+9879.0	fTotalStrokes	 9
U+9879.0	fCangjie	MMBO
U+9879.0	fHakka	 hong5
U+9879.0	fCantonese	hong6; hong2 (also)
U+9879.0	fMandarin	xiang4
U+9879.0	fEnglish	simplified form of &#x9805;
U+987A.0	fUTF8	顺
U+987A.0	fGB	0-4B33
U+987A.0	fR/S	 181.3
U+987A.0	fTotalStrokes	 9
U+987A.0	fCangjie	LLLO
U+987A.0	fHakka	 shun5
U+987A.0	fCantonese	seon6; seon2 (also)
U+987A.0	fMandarin	shun4
U+987A.0	fEnglish	simplified form of &#x9806;
U+987B.0	fUTF8	须
U+987B.0	fGB	0-506B
U+987B.0	fR/S	 59.6; 181.3
U+987B.0	fTotalStrokes	 9
U+987B.0	fCangjie	HHMBO
U+987B.0	fHakka	 si1
U+987B.0	fCantonese	seoi1; sou1
U+987B.0	fMandarin	xu1; ban1
U+987B.0	fEnglish	simplified form of &#x9808;
U+987C.0	fUTF8	顼
U+987C.0	fGB	0-676F
U+987C.0	fR/S	 181.4; 96.6
U+987C.0	fTotalStrokes	 13
U+987C.0	fCangjie	MGMBO
U+987C.0	fHakka	 hiuk7
U+987C.0	fCantonese	juk1; guk1
U+987C.0	fMandarin	xu4; yu4
U+987C.0	fEnglish	simplified form of &#x980A;
U+987D.0	fUTF8	顽
U+987D.0	fGB	0-4D67
U+987D.0	fR/S	 181.4
U+987D.0	fTotalStrokes	 10
U+987D.0	fCangjie	XMUMB
U+987D.0	fHakka	 ngan2
U+987D.0	fCantonese	waan4; wan4
U+987D.0	fMandarin	wan2
U+987D.0	fEnglish	simplified form of &#x9811;
U+987E.0	fUTF8	顾
U+987E.0	fGB	0-394B
U+987E.0	fR/S	 181.4
U+987E.0	fTotalStrokes	 10
U+987E.0	fCangjie	MUMBO
U+987E.0	fHakka	 gu5
U+987E.0	fCantonese	gu3; gwu3 (also)
U+987E.0	fMandarin	gu4; gu3
U+987E.0	fEnglish	simplified form of &#x9867;
U+987F.0	fUTF8	顿
U+987F.0	fGB	0-3659
U+987F.0	fR/S	 181.4
U+987F.0	fTotalStrokes	 10
U+987F.0	fCangjie	PUMBO
U+987F.0	fHakka	 dun5; dun3
U+987F.0	fCantonese	deon6; duk6; deon2 (also); eon6
U+987F.0	fMandarin	dun4; du2; zhun1
U+987F.0	fEnglish	simplified form of &#x9813;
U+9880.0	fUTF8	颀
U+9880.0	fGB	0-717D
U+9880.0	fR/S	 69.6; 181.4
U+9880.0	fTotalStrokes	 10
U+9880.0	fCangjie	HLMBO
U+9880.0	fHakka	 ki2
U+9880.0	fCantonese	kei4
U+9880.0	fMandarin	qi2; ken3; jin4
U+9880.0	fEnglish	simplified form of &#x980E;
U+9881.0	fUTF8	颁
U+9881.0	fGB	0-3064
U+9881.0	fR/S	 181.4
U+9881.0	fTotalStrokes	 10
U+9881.0	fCangjie	CHMBO
U+9881.0	fHakka	 ban1
U+9881.0	fCantonese	baan1; paan1 (also)
U+9881.0	fMandarin	ban1; fen2
U+9881.0	fEnglish	simplified form of &#x9812;
U+9882.0	fUTF8	颂
U+9882.0	fGB	0-4B4C
U+9882.0	fR/S	 181.4
U+9882.0	fTotalStrokes	 10
U+9882.0	fCangjie	CIMBO
U+9882.0	fHakka	 siung5
U+9882.0	fCantonese	zung6
U+9882.0	fMandarin	rong2; song4
U+9882.0	fEnglish	simplified form of &#x980C;
U+9883.0	fUTF8	颃
U+9883.0	fGB	0-717E
U+9883.0	fR/S	 181.4
U+9883.0	fTotalStrokes	 10
U+9883.0	fCangjie	YNMBO
U+9883.0	fHakka	 hong2; kong1; kong5
U+9883.0	fCantonese	hong4
U+9883.0	fMandarin	gang1; hang2; kang3; hang4; kang4; gang3
U+9883.0	fEnglish	simplified form of &#x980F;
U+9884.0	fUTF8	预
U+9884.0	fGB	0-5424
U+9884.0	fR/S	 181.4
U+9884.0	fTotalStrokes	 10
U+9884.0	fCangjie	NNMBO
U+9884.0	fHakka	 ji5
U+9884.0	fCantonese	jyu6
U+9884.0	fMandarin	yu4
U+9884.0	fEnglish	simplified form of &#x9810;
U+9885.0	fUTF8	颅
U+9885.0	fGB	0-422D
U+9885.0	fR/S	 181.5
U+9885.0	fTotalStrokes	 11
U+9885.0	fCangjie	YSMBO
U+9885.0	fHakka	 lu2
U+9885.0	fCantonese	lou4
U+9885.0	fMandarin	lu2
U+9885.0	fEnglish	simplified form of &#x9871;
U+9886.0	fUTF8	领
U+9886.0	fGB	0-416C
U+9886.0	fR/S	 181.5
U+9886.0	fTotalStrokes	 11
U+9886.0	fCangjie	OIMBO
U+9886.0	fHakka	 liang1; liang3
U+9886.0	fCantonese	ling5; leng5 (also); leng2 (also)
U+9886.0	fMandarin	ling3
U+9886.0	fEnglish	simplified form of &#x9818;
U+9887.0	fUTF8	颇
U+9887.0	fGB	0-4644
U+9887.0	fR/S	 181.5
U+9887.0	fTotalStrokes	 11
U+9887.0	fCangjie	DEMBO
U+9887.0	fHakka	 po3
U+9887.0	fCantonese	po2; po1
U+9887.0	fMandarin	po3; pi2; pi3; po1; po4
U+9887.0	fEnglish	simplified form of &#x9817;
U+9888.0	fUTF8	颈
U+9888.0	fGB	0-3E31
U+9888.0	fR/S	 181.5
U+9888.0	fTotalStrokes	 11
U+9888.0	fCangjie	NMMBO
U+9888.0	fHakka	 giang3
U+9888.0	fCantonese	geng2
U+9888.0	fMandarin	jing3; geng3; qing2
U+9888.0	fEnglish	simplified form of &#x9838;
U+9889.0	fUTF8	颉
U+9889.0	fGB	0-7221
U+9889.0	fR/S	 181.6
U+9889.0	fTotalStrokes	 12
U+9889.0	fCangjie	GRMBO
U+9889.0	fHakka	 giet7
U+9889.0	fCantonese	kit3; git3 (also)
U+9889.0	fMandarin	jia2; jie2; xie2
U+9889.0	fEnglish	simplified form of &#x9821;
U+988A.0	fUTF8	颊
U+988A.0	fGB	0-3C55
U+988A.0	fR/S	 181.6
U+988A.0	fTotalStrokes	 12
U+988A.0	fCangjie	KTMBO
U+988A.0	fHakka	 giap7
U+988A.0	fCantonese	gaap3; haap3 (also)
U+988A.0	fMandarin	jia2
U+988A.0	fEnglish	simplified form of &#x9830;
U+988B.0	fUTF8	颋
U+988B.0	fGB	8-7D77
U+988B.0	fR/S	 181.6
U+988B.0	fTotalStrokes	 12
U+988B.0	fCangjie	NGMBO
U+988B.0	fCantonese	ting5
U+988B.0	fMandarin	tian5; ting3
U+988B.0	fEnglish	straight
U+988C.0	fUTF8	颌
U+988C.0	fGB	0-7222
U+988C.0	fR/S	 181.6
U+988C.0	fTotalStrokes	 12
U+988C.0	fCangjie	XORMB
U+988C.0	fHakka	 hap8
U+988C.0	fCantonese	gap3 (also); hap6
U+988C.0	fMandarin	han4; ge2; ke4; he2; han2
U+988C.0	fEnglish	simplified form of &#x981C;
U+988D.0	fUTF8	颍
U+988D.0	fGB	0-7223
U+988D.0	fR/S	 85.8; 181.6
U+988D.0	fTotalStrokes	 12
U+988D.0	fCangjie	PEMBO
U+988D.0	fHakka	 jin3
U+988D.0	fCantonese	wing6
U+988D.0	fMandarin	ying3
U+988D.0	fEnglish	simplified form of &#x6F41;
U+988E.0	fUTF8	颎
U+988E.0	fGB	8-7D78
U+988E.0	fR/S	 86.8; 181.6
U+988E.0	fTotalStrokes	 12
U+988E.0	fCangjie	XPFMB
U+988E.0	fCantonese	gwing2; gang2; wing6
U+988E.0	fMandarin	jiong3
U+988E.0	fEnglish	bright
U+988F.0	fUTF8	颏
U+988F.0	fGB	0-7224
U+988F.0	fR/S	 181.6
U+988F.0	fTotalStrokes	 12
U+988F.0	fCangjie	YOMBO
U+988F.0	fHakka	 goi1; hoi2
U+988F.0	fCantonese	hoi4; hoi5 (also)
U+988F.0	fMandarin	hai2; ke2; ke1; gai3; gai1; hai4
U+988F.0	fEnglish	simplified form of &#x9826;
U+9890.0	fUTF8	颐
U+9890.0	fGB	0-5243
U+9890.0	fR/S	 181.7
U+9890.0	fTotalStrokes	 13
U+9890.0	fCangjie	SLMBO
U+9890.0	fHakka	 ji2; hi1
U+9890.0	fCantonese	ji4
U+9890.0	fMandarin	yi2
U+9890.0	fEnglish	simplified form of &#x9824;
U+9891.0	fUTF8	频
U+9891.0	fGB	0-4635
U+9891.0	fR/S	 181.7
U+9891.0	fTotalStrokes	 13
U+9891.0	fCangjie	YHMBO
U+9891.0	fHakka	 pin2
U+9891.0	fCantonese	pan4; ban1
U+9891.0	fMandarin	pin2; bin1
U+9891.0	fEnglish	simplified form of &#x983B;
U+9892.0	fUTF8	颒
U+9892.0	fGB	8-2E6D
U+9892.0	fR/S	 181.7
U+9892.0	fTotalStrokes	 13
U+9892.0	fCangjie	ETMBO
U+9892.0	fCantonese	fui3
U+9892.0	fMandarin	hui4
U+9892.0	fEnglish	wash
U+9893.0	fUTF8	颓
U+9893.0	fGB	0-4D47
U+9893.0	fR/S	 181.7
U+9893.0	fTotalStrokes	 13
U+9893.0	fCangjie	HNMBO
U+9893.0	fHakka	 tui1
U+9893.0	fCantonese	teoi4
U+9893.0	fMandarin	tui2
U+9893.0	fEnglish	simplified form of &#x983D;
U+9894.0	fUTF8	颔
U+9894.0	fGB	0-7225
U+9894.0	fR/S	 181.7
U+9894.0	fTotalStrokes	 13
U+9894.0	fCangjie	ORMBO
U+9894.0	fHakka	 ngam3; ham3
U+9894.0	fCantonese	ham5
U+9894.0	fMandarin	han4; an3; han2
U+9894.0	fEnglish	simplified form of &#x9837;
U+9895.0	fUTF8	颕
U+9895.0	fGB	8-2D77
U+9895.0	fR/S	 181.7
U+9895.0	fTotalStrokes	 13
U+9895.0	fCangjie	PFMBO
U+9895.0	fCantonese	wing6
U+9895.0	fMandarin	ying3
U+9895.0	fEnglish	rice tassel; sharp point; clever
U+9896.0	fUTF8	颖
U+9896.0	fGB	0-5331
U+9896.0	fR/S	 181.7
U+9896.0	fTotalStrokes	 13
U+9896.0	fCangjie	PDMBO
U+9896.0	fHakka	 jin3
U+9896.0	fCantonese	wing6
U+9896.0	fMandarin	ying3
U+9896.0	fEnglish	simplified form of &#x7A4E;
U+9897.0	fUTF8	颗
U+9897.0	fGB	0-3F45
U+9897.0	fR/S	 181.8
U+9897.0	fTotalStrokes	 14
U+9897.0	fCangjie	WDMBO
U+9897.0	fHakka	 ko5; ko3
U+9897.0	fCantonese	fo2
U+9897.0	fMandarin	ke1; ke3; kuan3
U+9897.0	fEnglish	simplified form of &#x9846;
U+9898.0	fUTF8	题
U+9898.0	fGB	0-4C62
U+9898.0	fR/S	 181.9
U+9898.0	fTotalStrokes	 15
U+9898.0	fCangjie	AOMBO
U+9898.0	fHakka	 ti2
U+9898.0	fCantonese	tai4; tai2 (also)
U+9898.0	fMandarin	ti2; di4
U+9898.0	fEnglish	simplified form of &#x984C;
U+9899.0	fUTF8	颙
U+9899.0	fGB	8-7D79
U+9899.0	fR/S	 181.9
U+9899.0	fTotalStrokes	 15
U+9899.0	fCangjie	WBMBO
U+9899.0	fCantonese	jung4
U+9899.0	fMandarin	yong2
U+9899.0	fEnglish	solemn, large, grand, majestic
U+989A.0	fUTF8	颚
U+989A.0	fGB	0-7226
U+989A.0	fR/S	 181.9
U+989A.0	fTotalStrokes	 15
U+989A.0	fCangjie	RSMBO
U+989A.0	fCantonese	ngok6; ok6 (also); ngok2 (also); ok2 (also)
U+989A.0	fMandarin	e4
U+989A.0	fEnglish	simplified form of &#x984E;
U+989B.0	fUTF8	颛
U+989B.0	fGB	0-7227
U+989B.0	fR/S	 181.9
U+989B.0	fTotalStrokes	 15
U+989B.0	fCangjie	UBMBO
U+989B.0	fHakka	 zhon1
U+989B.0	fCantonese	zyun1
U+989B.0	fMandarin	zhuan1
U+989B.0	fEnglish	simplified form of &#x9853;
U+989C.0	fUTF8	颜
U+989C.0	fGB	0-5155
U+989C.0	fR/S	 181.9
U+989C.0	fTotalStrokes	 15
U+989C.0	fCangjie	YHMBO
U+989C.0	fCantonese	ngaan4; aan4 (also)
U+989C.0	fMandarin	yan2
U+989C.0	fEnglish	simplified form of &#x9854;
U+989D.0	fUTF8	额
U+989D.0	fGB	0-366E
U+989D.0	fR/S	 181.9
U+989D.0	fTotalStrokes	 15
U+989D.0	fCangjie	JRMBO
U+989D.0	fHakka	 ngiak7
U+989D.0	fCantonese	ngaak6; aak6 (also); ngaak2 (also); aak2 (also)
U+989D.0	fMandarin	e2
U+989D.0	fEnglish	simplified form of &#x984D;
U+989E.0	fUTF8	颞
U+989E.0	fGB	0-7228
U+989E.0	fR/S	 181.9; 181.10
U+989E.0	fTotalStrokes	 16
U+989E.0	fCangjie	SEMBO
U+989E.0	fCantonese	nip6; lip6 (also)
U+989E.0	fMandarin	nie4; huan4
U+989E.0	fEnglish	simplified form of &#x9873;
U+989F.0	fUTF8	颟
U+989F.0	fGB	0-7229
U+989F.0	fR/S	 181.10
U+989F.0	fTotalStrokes	 16
U+989F.0	fCangjie	TBMBO
U+989F.0	fHakka	 man2
U+989F.0	fCantonese	mun4; mun1 (also)
U+989F.0	fMandarin	man1; man2
U+989F.0	fEnglish	simplified form of &#x9862;
U+98A0.0	fUTF8	颠
U+98A0.0	fGB	0-355F
U+98A0.0	fR/S	 181.10
U+98A0.0	fTotalStrokes	 16
U+98A0.0	fCangjie	JCMBO
U+98A0.0	fHakka	 dien1
U+98A0.0	fCantonese	din1
U+98A0.0	fMandarin	dian1; tian2; tian4
U+98A0.0	fEnglish	simplified form of &#x985B;
U+98A1.0	fUTF8	颡
U+98A1.0	fGB	0-722A
U+98A1.0	fR/S	 181.10
U+98A1.0	fTotalStrokes	 16
U+98A1.0	fCangjie	EDMBO
U+98A1.0	fHakka	 song3
U+98A1.0	fCantonese	song2
U+98A1.0	fMandarin	sang3; sang1
U+98A1.0	fEnglish	simplified form of &#x9859;
U+98A2.0	fUTF8	颢
U+98A2.0	fGB	0-722B
U+98A2.0	fR/S	 181.12
U+98A2.0	fTotalStrokes	 18
U+98A2.0	fCangjie	AFMBO
U+98A2.0	fHakka	 hau3
U+98A2.0	fCantonese	hou6
U+98A2.0	fMandarin	hao4
U+98A2.0	fEnglish	simplified form of &#x9865;
U+98A3.0	fUTF8	颣
U+98A3.0	fGB	8-7D7A
U+98A3.0	fR/S	 120.12; 181.12
U+98A3.0	fTotalStrokes	 18
U+98A3.0	fCangjie	FFMBO
U+98A3.0	fCantonese	leoi6
U+98A3.0	fMandarin	lei4
U+98A4.0	fUTF8	颤
U+98A4.0	fGB	0-327C
U+98A4.0	fR/S	 181.13
U+98A4.0	fTotalStrokes	 19
U+98A4.0	fCangjie	YMMBO
U+98A4.0	fHakka	 zhen1; zhen5
U+98A4.0	fCantonese	zin3
U+98A4.0	fMandarin	zhan4; chan4; shan1; zhan3
U+98A4.0	fEnglish	simplified form of &#x986B;
U+98A5.0	fUTF8	颥
U+98A5.0	fGB	0-722C
U+98A5.0	fR/S	 181.14
U+98A5.0	fTotalStrokes	 20
U+98A5.0	fCangjie	MBMBO
U+98A5.0	fCantonese	jyu4
U+98A5.0	fMandarin	ru2
U+98A5.0	fEnglish	simplified form of &#x986C;
U+98A6.0	fUTF8	颦
U+98A6.0	fGB	0-722D
U+98A6.0	fR/S	 24.19; 181.15
U+98A6.0	fTotalStrokes	 21
U+98A6.0	fCangjie	YOHWJ
U+98A6.0	fHakka	 pin2
U+98A6.0	fCantonese	pan4
U+98A6.0	fMandarin	pin2; pi2
U+98A6.0	fEnglish	simplified form of &#x9870;
U+98A7.0	fUTF8	颧
U+98A7.0	fGB	0-4827
U+98A7.0	fR/S	 181.17
U+98A7.0	fTotalStrokes	 23
U+98A7.0	fCangjie	TGMBO
U+98A7.0	fHakka	 ken2
U+98A7.0	fCantonese	kyun4
U+98A7.0	fMandarin	quan2
U+98A7.0	fEnglish	simplified form of &#x9874;
U+98A8.0	fUTF8	風
U+98A8.0	fCNS11643	1-5378
U+98A8.0	fBig5	ADB7
U+98A8.0	fGB	1-3767
U+98A8.0	fR/S	 182.0
U+98A8.0	fTotalStrokes	 9
U+98A8.0	fCangjie	HNHLI
U+98A8.0	fFourCorner	77210
U+98A8.0	fHakka	  fung1; fung3
U+98A8.0	fCantonese	fung1; fung3
U+98A8.0	fMandarin	feng1; fan1; feng3
U+98A8.0	fEnglish	[1] wind; gust; breeze; gale [2] education; influence [3] custums; pratices; fad; fashion; fashionable [4] scene [5] style; manner; deportment; taste [6] fame; reputation [7] rumor [8] ailments supposedly caused by wind and dampness
U+98A8.1	fUTF8	風
U+98A8.1	fCNS11643	1-5378
U+98A8.1	fBig5	ADB7
U+98A8.1	fGB	1-3767
U+98A8.1	fR/S	 182.0
U+98A8.1	fTotalStrokes	 9
U+98A8.1	fCangjie	HNHLI
U+98A8.1	fFourCorner	77210
U+98A8.1	fMandarin	feng4
U+98A8.1	fEnglish	[1] announce; make known [2] ridicule; satirize [3] blow
U+98A9.0	fUTF8	颩
U+98A9.0	fCNS11643	2-4152
U+98A9.0	fBig5	DCAD
U+98A9.0	fGB	3-4E5E
U+98A9.0	fR/S	 182.3
U+98A9.0	fTotalStrokes	 12
U+98A9.0	fCangjie	HNHHH
U+98A9.0	fFourCorner	72212
U+98A9.0	fCantonese	biu1
U+98A9.0	fMandarin	biao1; guan3; diu1; huan4
U+98AA.0	fUTF8	颪
U+98AA.0	fGB	E-4542
U+98AA.0	fR/S	 182.3
U+98AA.0	fTotalStrokes	 12
U+98AA.0	fCangjie	MYHN
U+98AA.0	fCantonese	haa6
U+98AA.0	fEnglish	wind blowing down from the mountains
U+98AB.0	fUTF8	颫
U+98AB.0	fCNS11643	4-4171
U+98AB.0	fGB	E-4543
U+98AB.0	fR/S	  182.4
U+98AB.0	fTotalStrokes	 13
U+98AB.0	fCangjie	HNQO
U+98AB.0	fFourCorner	75210
U+98AB.0	fCantonese	fu4
U+98AB.0	fMandarin	fu2
U+98AB.0	fEnglish	a storm
U+98AC.0	fUTF8	颬
U+98AC.0	fCNS11643	2-4867
U+98AC.0	fBig5	E0E1
U+98AC.0	fGB	3-4E61
U+98AC.0	fR/S	 182.4
U+98AC.0	fTotalStrokes	 13
U+98AC.0	fCangjie	HNMVH
U+98AC.0	fFourCorner	71214
U+98AC.0	fHakka	 ha1; ha2
U+98AC.0	fCantonese	haa1; haa4
U+98AC.0	fMandarin	xia1
U+98AC.0	fEnglish	 take a breath; pant
U+98AD.0	fUTF8	颭
U+98AD.0	fCNS11643	2-4F34
U+98AD.0	fBig5	E4CC
U+98AD.0	fGB	3-4E66
U+98AD.0	fR/S	 182.5
U+98AD.0	fTotalStrokes	 14
U+98AD.0	fCangjie	HNYR
U+98AD.0	fFourCorner	71216
U+98AD.0	fHakka	 zham5; zam5; zam3
U+98AD.0	fCantonese	zim2
U+98AD.0	fMandarin	zhan3
U+98AD.0	fEnglish	sway in wind
U+98AE.0	fUTF8	颮
U+98AE.0	fCNS11643	2-4F35
U+98AE.0	fBig5	E4CD
U+98AE.0	fGB	1-6C29
U+98AE.0	fR/S	 182.5
U+98AE.0	fTotalStrokes	 14
U+98AE.0	fCangjie	HNPRU
U+98AE.0	fFourCorner	77211
U+98AE.0	fHakka	  piau1; pok8; peu1
U+98AE.0	fCantonese	paau4; biu1; paau1
U+98AE.0	fMandarin	biao1; pao2; pao4; po4; bao2
U+98AE.0	fEnglish	storm whirlwind
U+98AF.0	fUTF8	颯
U+98AF.0	fCNS11643	1-6B6C
U+98AF.0	fBig5	BBE3
U+98AF.0	fGB	1-6C2A
U+98AF.0	fR/S	 182.5
U+98AF.0	fTotalStrokes	 14
U+98AF.0	fCangjie	YTHNI
U+98AF.0	fFourCorner	07110
U+98AF.0	fHakka	 sap7
U+98AF.0	fCantonese	saap3
U+98AF.0	fMandarin	sa4; li4; su4
U+98AF.0	fEnglish	[1] the sound of wind; rustling [2] weakened; failing; declining
U+98B0.0	fUTF8	颰
U+98B0.0	fCNS11643	4-4857
U+98B0.0	fGB	5-5047
U+98B0.0	fR/S	  182.5
U+98B0.0	fTotalStrokes	 14
U+98B0.0	fCangjie	XHNIK
U+98B0.0	fFourCorner	73210
U+98B0.0	fCantonese	bat6
U+98B0.0	fMandarin	ba2; fu2
U+98B1.0	fUTF8	颱
U+98B1.0	fCNS11643	1-6B6D
U+98B1.0	fBig5	BBE4
U+98B1.0	fGB	1-786C
U+98B1.0	fR/S	 182.5
U+98B1.0	fTotalStrokes	 14
U+98B1.0	fCangjie	HNIR
U+98B1.0	fFourCorner	73216
U+98B1.0	fHakka	 toi2
U+98B1.0	fCantonese	toi4
U+98B1.0	fMandarin	tai2
U+98B1.0	fEnglish	 typhoon; hurricane
U+98B2.0	fUTF8	颲
U+98B2.0	fCNS11643	2-5567
U+98B2.0	fBig5	E8BE
U+98B2.0	fGB	3-4E6C
U+98B2.0	fR/S	 182.6
U+98B2.0	fTotalStrokes	 15
U+98B2.0	fCangjie	HNMNN
U+98B2.0	fFourCorner	72210
U+98B2.0	fHakka	  liet8
U+98B2.0	fCantonese	lit6
U+98B2.0	fMandarin	lie4
U+98B2.0	fEnglish	[1] violent gust [2] squall
U+98B3.0	fUTF8	颳
U+98B3.0	fCNS11643	1-6F71
U+98B3.0	fBig5	BE68
U+98B3.0	fGB	1-7837
U+98B3.0	fR/S	 182.6
U+98B3.0	fTotalStrokes	 15
U+98B3.0	fCangjie	HNHJR
U+98B3.0	fFourCorner	72216
U+98B3.0	fHakka	 kwat7; gat7
U+98B3.0	fCantonese	gwaat3
U+98B3.0	fMandarin	gua1
U+98B3.0	fEnglish	 blow; wind blowing
U+98B4.0	fUTF8	颴
U+98B4.0	fGB	E-4544
U+98B4.0	fR/S	 182.7
U+98B4.0	fTotalStrokes	 16
U+98B4.0	fCangjie	HNONO
U+98B4.0	fFourCorner	7821
U+98B4.0	fHakka	 sien2
U+98B4.0	fCantonese	syun4
U+98B4.0	fMandarin	xuan4
U+98B4.0	fEnglish	whirlwind
U+98B5.0	fUTF8	颵
U+98B5.0	fCNS11643	3-5361
U+98B5.0	fGB	3-4E73
U+98B5.0	fR/S	 182.7
U+98B5.0	fTotalStrokes	 16
U+98B5.0	fCangjie	HNFB
U+98B5.0	fFourCorner	79210
U+98B5.0	fHakka	  sau1
U+98B5.0	fCantonese	saau1
U+98B5.0	fMandarin	shao4; shao1; xiao1
U+98B5.0	fEnglish	 dry in the wind
U+98B6.0	fUTF8	颶
U+98B6.0	fCNS11643	1-7629
U+98B6.0	fBig5	C1FC
U+98B6.0	fGB	1-6C2B
U+98B6.0	fR/S	 182.8
U+98B6.0	fTotalStrokes	 17
U+98B6.0	fCangjie	HNBMC
U+98B6.0	fFourCorner	76218
U+98B6.0	fHakka	  ki3; gi5
U+98B6.0	fCantonese	geoi6
U+98B6.0	fMandarin	ju4
U+98B6.0	fEnglish	 cyclone; typhoon; gale; hurricane; strong gusts at sea
U+98B7.0	fUTF8	颷
U+98B7.0	fCNS11643	3-572A
U+98B7.0	fGB	5-5053
U+98B7.0	fR/S	 182.8
U+98B7.0	fTotalStrokes	 17
U+98B7.0	fCangjie	HNFF
U+98B7.0	fFourCorner	79210
U+98B7.0	fCantonese	biu1
U+98B7.0	fMandarin	bi1; biao1
U+98B7.0	fEnglish	whirlwind
U+98B8.0	fUTF8	颸
U+98B8.0	fCNS11643	2-6455
U+98B8.0	fBig5	F1AB
U+98B8.0	fGB	3-4E7D
U+98B8.0	fR/S	 182.9
U+98B8.0	fTotalStrokes	 18
U+98B8.0	fCangjie	HNWP
U+98B8.0	fFourCorner	76213
U+98B8.0	fHakka	 sii1; se1
U+98B8.0	fCantonese	si1; ci1
U+98B8.0	fMandarin	si1; chi1
U+98B8.0	fEnglish	cool breeze of autumn
U+98B9.0	fUTF8	颹
U+98B9.0	fCNS11643	3-595C
U+98B9.0	fGB	3-4E60
U+98B9.0	fR/S	  182.9
U+98B9.0	fTotalStrokes	 18
U+98B9.0	fCangjie	HNDMQ
U+98B9.0	fFourCorner	74210
U+98B9.0	fHakka	 wui3; wi3; vui3
U+98B9.0	fCantonese	wai5
U+98B9.0	fMandarin	wei3
U+98B9.0	fEnglish	 fresh breeze
U+98BA.0	fUTF8	颺
U+98BA.0	fCNS11643	1-7771
U+98BA.0	fBig5	C347
U+98BA.0	fGB	3-4E5F
U+98BA.0	fR/S	 182.9
U+98BA.0	fTotalStrokes	 18
U+98BA.0	fCangjie	HNAMH
U+98BA.0	fFourCorner	76212
U+98BA.0	fHakka	  jong2; ngiong2; rong2
U+98BA.0	fCantonese	joeng4
U+98BA.0	fMandarin	yang2; yang4
U+98BA.0	fEnglish	[1] blown or tossed about by the wind; float [2] fly; blow away [3] spread; scatter
U+98BB.0	fUTF8	颻
U+98BB.0	fCNS11643	2-6778
U+98BB.0	fBig5	F3AD
U+98BB.0	fGB	5-505A
U+98BB.0	fR/S	 182.10
U+98BB.0	fTotalStrokes	 19
U+98BB.0	fCangjie	BUHNI
U+98BB.0	fFourCorner	27710
U+98BB.0	fHakka	 jau2; rau2; jeu2
U+98BB.0	fCantonese	jiu4
U+98BB.0	fMandarin	yao2; yao4
U+98BB.0	fEnglish	floating in air; waving and drifting with the wind
U+98BC.0	fUTF8	颼
U+98BC.0	fCNS11643	1-794E
U+98BC.0	fBig5	C442
U+98BC.0	fGB	1-6C2C
U+98BC.0	fR/S	 182.10
U+98BC.0	fTotalStrokes	 19
U+98BC.0	fCangjie	HNHXE
U+98BC.0	fFourCorner	77214
U+98BC.0	fHakka	  seu1; siu1; sau1
U+98BC.0	fCantonese	sau1; suk1
U+98BC.0	fMandarin	sou1
U+98BC.0	fEnglish	[1] the swishing sound of a fast-flying object, as an arrow [2] blow about by the wind
U+98BD.0	fUTF8	颽
U+98BD.0	fCNS11643	2-6777
U+98BD.0	fBig5	F3AC
U+98BD.0	fGB	5-504C
U+98BD.0	fR/S	 182.10
U+98BD.0	fTotalStrokes	 19
U+98BD.0	fCangjie	UTHNI
U+98BD.0	fFourCorner	27110
U+98BD.0	fHakka	 koi3
U+98BD.0	fCantonese	hoi2
U+98BD.0	fMandarin	kai3
U+98BE.0	fUTF8	颾
U+98BE.0	fCNS11643	2-6779
U+98BE.0	fBig5	F3AE
U+98BE.0	fGB	3-4F22
U+98BE.0	fR/S	 182.10
U+98BE.0	fTotalStrokes	 19
U+98BE.0	fCangjie	HNEII
U+98BE.0	fFourCorner	77213
U+98BE.0	fHakka	  sau1; zau1; sau5
U+98BE.0	fCantonese	sau1; sou1
U+98BE.0	fMandarin	sao1; sou1; sao4
U+98BE.0	fEnglish	blowing of the wind
U+98BF.0	fUTF8	颿
U+98BF.0	fCNS11643	2-6776
U+98BF.0	fBig5	F3AB
U+98BF.0	fGB	E-4545
U+98BF.0	fR/S	 182.10
U+98BF.0	fTotalStrokes	 19
U+98BF.0	fCangjie	SFHNI
U+98BF.0	fFourCorner	77310
U+98BF.0	fHakka	 fam2;  fam5
U+98BF.0	fCantonese	faan4
U+98BF.0	fMandarin	fan2; fan4; fan1
U+98BF.0	fEnglish	sail
U+98C0.0	fUTF8	飀
U+98C0.0	fCNS11643	2-6C63
U+98C0.0	fBig5	F675
U+98C0.0	fGB	3-4F23
U+98C0.0	fR/S	 182.10
U+98C0.0	fTotalStrokes	 19
U+98C0.0	fCangjie	HNHHW
U+98C0.0	fFourCorner	77216
U+98C0.0	fHakka	 liu2
U+98C0.0	fCantonese	lau4
U+98C0.0	fMandarin	liu2; liu4
U+98C0.0	fEnglish	 soughing of wind noise
U+98C1.0	fUTF8	飁
U+98C1.0	fCNS11643	2-6A60
U+98C1.0	fBig5	F552
U+98C1.0	fGB	3-4F26
U+98C1.0	fR/S	 182.11
U+98C1.0	fTotalStrokes	 20
U+98C1.0	fCangjie	SAHNI
U+98C1.0	fFourCorner	17610
U+98C1.0	fCantonese	zaap6
U+98C1.0	fMandarin	xi2
U+98C2.0	fUTF8	飂
U+98C2.0	fCNS11643	2-6A61
U+98C2.0	fBig5	F553
U+98C2.0	fGB	3-4F27
U+98C2.0	fR/S	 182.11
U+98C2.0	fTotalStrokes	 20
U+98C2.0	fCangjie	HNSMH
U+98C2.0	fFourCorner	77212
U+98C2.0	fHakka	  liu2; liu3; liau2
U+98C2.0	fCantonese	lau4; liu4; lau6 (also); luk6; lik6
U+98C2.0	fMandarin	liao2; liu2; liu4; lu4
U+98C2.0	fEnglish	wind in high places
U+98C3.0	fUTF8	飃
U+98C3.0	fCNS11643	3-5D56
U+98C3.0	fGB	E-4546
U+98C3.0	fR/S	 182.11
U+98C3.0	fTotalStrokes	 20
U+98C3.0	fCangjie	HNMWF
U+98C3.0	fFourCorner	71210
U+98C3.0	fHakka	 piau1
U+98C3.0	fCantonese	piu1
U+98C3.0	fMandarin	piao1; piao4
U+98C3.0	fEnglish	 whirlwind; cyclone floating
U+98C4.0	fUTF8	飄
U+98C4.0	fCNS11643	1-7A51
U+98C4.0	fBig5	C4C6
U+98C4.0	fGB	1-462E
U+98C4.0	fR/S	 182.11
U+98C4.0	fTotalStrokes	 20
U+98C4.0	fCangjie	MFHNI
U+98C4.0	fFourCorner	17910
U+98C4.0	fHakka	  piau1; peu1
U+98C4.0	fCantonese	piu1
U+98C4.0	fMandarin	piao1; biao1; piao2
U+98C4.0	fEnglish	[1] blow; float (in the air); waft; move with the wind [2] whirlwind; cyclone; whirling wind [3] float; drift (on the water)
U+98C5.0	fUTF8	飅
U+98C5.0	fGB	E-4547
U+98C5.0	fR/S	 182.12
U+98C5.0	fTotalStrokes	 21
U+98C5.0	fCangjie	HNMLW
U+98C5.0	fFourCorner	7121
U+98C5.0	fHakka	 liu2
U+98C5.0	fCantonese	lau4
U+98C5.0	fMandarin	liu2; liu3
U+98C5.0	fEnglish	siging of the wind
U+98C6.0	fUTF8	飆
U+98C6.0	fCNS11643	2-6C62
U+98C6.0	fBig5	F674
U+98C6.0	fGB	1-6C2D
U+98C6.0	fR/S	 182.12
U+98C6.0	fTotalStrokes	 21
U+98C6.0	fCangjie	IKHNI
U+98C6.0	fFourCorner	47410
U+98C6.0	fHakka	 piau1; biau1; beu1; peu1
U+98C6.0	fCantonese	biu1
U+98C6.0	fMandarin	biao1
U+98C6.0	fEnglish	violent winds; gales
U+98C7.0	fUTF8	飇
U+98C7.0	fCNS11643	3-5E7B
U+98C7.0	fGB	E-4548
U+98C7.0	fR/S	 182.12
U+98C7.0	fTotalStrokes	 21
U+98C7.0	fCangjie	HNIKK
U+98C7.0	fFourCorner	73210
U+98C7.0	fHakka	 piau1
U+98C7.0	fCantonese	biu1
U+98C7.0	fMandarin	biao1
U+98C7.0	fEnglish	whirlwind; stormy gale
U+98C8.0	fUTF8	飈
U+98C8.0	fCNS11643	3-5E79
U+98C8.0	fGB	1-6C2E
U+98C8.0	fR/S	 182.12
U+98C8.0	fTotalStrokes	 21
U+98C8.0	fCangjie	HNFFF
U+98C8.0	fFourCorner	79210
U+98C8.0	fHakka	  piau1; biau1; beu1; peu1
U+98C8.0	fCantonese	biu1
U+98C8.0	fMandarin	biao1; biao3
U+98C8.0	fEnglish	whirlwind
U+98C9.0	fUTF8	飉
U+98C9.0	fCNS11643	2-6C61
U+98C9.0	fBig5	F673
U+98C9.0	fGB	5-505E
U+98C9.0	fR/S	 182.12
U+98C9.0	fTotalStrokes	 21
U+98C9.0	fCangjie	HNKCF
U+98C9.0	fFourCorner	74219
U+98C9.0	fCantonese	liu4
U+98C9.0	fMandarin	liao2
U+98CA.0	fUTF8	飊
U+98CA.0	fCNS11643	3-5E7A
U+98CA.0	fGB	E-4549
U+98CA.0	fR/S	 182.12
U+98CA.0	fTotalStrokes	 21
U+98CA.0	fCangjie	FFHNI
U+98CA.0	fFourCorner	97410
U+98CA.0	fCantonese	biu1
U+98CB.0	fUTF8	飋
U+98CB.0	fCNS11643	2-6E44
U+98CB.0	fBig5	F775
U+98CB.0	fGB	5-505F
U+98CB.0	fR/S	 182.13
U+98CB.0	fTotalStrokes	 22
U+98CB.0	fCangjie	HNMGH
U+98CB.0	fFourCorner	71213
U+98CB.0	fCantonese	sat1
U+98CB.0	fMandarin	se4
U+98CC.0	fUTF8	飌
U+98CC.0	fCNS11643	2-717E
U+98CC.0	fBig5	F9B0
U+98CC.0	fGB	5-5062
U+98CC.0	fR/S	 182.18
U+98CC.0	fTotalStrokes	 27
U+98CC.0	fCangjie	TGHNI
U+98CC.0	fFourCorner	47210
U+98CC.0	fCantonese	fung1; fung3
U+98CC.0	fMandarin	feng1; feng4
U+98CC.0	fEnglish	wind
U+98CD.0	fUTF8	飍
U+98CD.0	fCNS11643	3-6225
U+98CD.0	fGB	3-4E7A
U+98CD.0	fR/S	 182.18
U+98CD.0	fTotalStrokes	 27
U+98CD.0	fCangjie	HNHNN
U+98CD.0	fFourCorner	77210
U+98CD.0	fCantonese	haau1
U+98CD.0	fMandarin	biao1; xiu1
U+98CE.0	fUTF8	风
U+98CE.0	fGB	0-3767
U+98CE.0	fR/S	 182.0
U+98CE.0	fTotalStrokes	 4
U+98CE.0	fCangjie	HNK
U+98CE.0	fHakka	 fung1
U+98CE.0	fCantonese	fung1; fung3
U+98CE.0	fMandarin	feng1; fan1; feng3; feng4
U+98CE.0	fEnglish	simplified form of &#x98A8;
U+98CF.0	fUTF8	飏
U+98CF.0	fGB	8-7C57
U+98CF.0	fR/S	 182.9; 182.3
U+98CF.0	fTotalStrokes	 7
U+98CF.0	fCangjie	HNNSH
U+98CF.0	fCantonese	joeng4
U+98CF.0	fMandarin	yang2
U+98CF.0	fEnglish	soar, fly, float; scatter
U+98D0.0	fUTF8	飐
U+98D0.0	fGB	8-2C5B
U+98D0.0	fR/S	 182.5
U+98D0.0	fTotalStrokes	 9
U+98D0.0	fCangjie	XHNYR
U+98D0.0	fCantonese	zim2
U+98D0.0	fMandarin	zhan3
U+98D0.0	fEnglish	sway in wind
U+98D1.0	fUTF8	飑
U+98D1.0	fGB	0-6C29
U+98D1.0	fR/S	 182.5
U+98D1.0	fTotalStrokes	 9
U+98D1.0	fCangjie	XHNPR
U+98D1.0	fHakka	 piau1
U+98D1.0	fCantonese	paau4; biu1; paau1
U+98D1.0	fMandarin	biao1; pao2; pao4; po4; bao2
U+98D1.0	fEnglish	simplified form of &#x98AE;
U+98D2.0	fUTF8	飒
U+98D2.0	fGB	0-6C2A
U+98D2.0	fR/S	 182.5
U+98D2.0	fTotalStrokes	 9
U+98D2.0	fCangjie	YTHNK
U+98D2.0	fHakka	 sap7
U+98D2.0	fCantonese	saap3
U+98D2.0	fMandarin	sa4; su4; li4
U+98D2.0	fEnglish	simplified form of &#x98AF;
U+98D3.0	fUTF8	飓
U+98D3.0	fGB	0-6C2B
U+98D3.0	fR/S	 182.8
U+98D3.0	fTotalStrokes	 12
U+98D3.0	fCangjie	XHNBM
U+98D3.0	fHakka	 ki3
U+98D3.0	fCantonese	geoi6
U+98D3.0	fMandarin	ju4
U+98D3.0	fEnglish	simplified form of &#x98B6;
U+98D4.0	fUTF8	飔
U+98D4.0	fGB	8-7C58
U+98D4.0	fR/S	 182.9
U+98D4.0	fTotalStrokes	 13
U+98D4.0	fCangjie	XHNWP
U+98D4.0	fCantonese	si1
U+98D4.0	fMandarin	si1
U+98D4.0	fEnglish	cool breeze of autumn
U+98D5.0	fUTF8	飕
U+98D5.0	fGB	0-6C2C
U+98D5.0	fR/S	 182.10
U+98D5.0	fTotalStrokes	 14
U+98D5.0	fCangjie	XHNHX
U+98D5.0	fHakka	 seu1
U+98D5.0	fCantonese	sau1
U+98D5.0	fMandarin	sou1
U+98D5.0	fEnglish	simplified form of &#x98BC;
U+98D6.0	fUTF8	飖
U+98D6.0	fGB	8-2E56
U+98D6.0	fR/S	 182.10; 182.9
U+98D6.0	fTotalStrokes	 13
U+98D6.0	fCangjie	BUHNK
U+98D6.0	fCantonese	jiu4
U+98D6.0	fMandarin	yao2
U+98D6.0	fEnglish	floating in air, drifting with wind
U+98D7.0	fUTF8	飗
U+98D7.0	fGB	8-7C59
U+98D7.0	fR/S	 182.10
U+98D7.0	fTotalStrokes	 14
U+98D7.0	fCangjie	XHNHH
U+98D7.0	fCantonese	lau4
U+98D7.0	fMandarin	liu2
U+98D7.0	fEnglish	soughing of wind; noise
U+98D8.0	fUTF8	飘
U+98D8.0	fGB	0-462E
U+98D8.0	fR/S	 182.11
U+98D8.0	fTotalStrokes	 15
U+98D8.0	fCangjie	MFHNK
U+98D8.0	fHakka	 piau1
U+98D8.0	fCantonese	piu1
U+98D8.0	fMandarin	piao1; biao1; piao2
U+98D8.0	fEnglish	simplified form of &#x98C4;
U+98D9.0	fUTF8	飙
U+98D9.0	fGB	0-6C2D
U+98D9.0	fR/S	 182.12
U+98D9.0	fTotalStrokes	 16
U+98D9.0	fCangjie	IKHNK
U+98D9.0	fCantonese	biu1
U+98D9.0	fMandarin	biao1
U+98D9.0	fEnglish	simplified form of &#x98C6;
U+98DA.0	fUTF8	飚
U+98DA.0	fGB	0-6C2E
U+98DA.0	fR/S	 182.12
U+98DA.0	fTotalStrokes	 16
U+98DA.0	fCangjie	XHNFF
U+98DA.0	fHakka	 piau1
U+98DA.0	fCantonese	biu1
U+98DA.0	fMandarin	biao1
U+98DA.0	fEnglish	simplified form of &#x98C8;
U+98DB.0	fUTF8	飛
U+98DB.0	fCNS11643	1-5379
U+98DB.0	fBig5	ADB8
U+98DB.0	fGB	1-3749
U+98DB.0	fR/S	 183.0
U+98DB.0	fTotalStrokes	 9
U+98DB.0	fCangjie	NOHTO
U+98DB.0	fFourCorner	12413
U+98DB.0	fHakka	  fui1;  bui1; fi1; bi1
U+98DB.0	fCantonese	fei1
U+98DB.0	fMandarin	fei1
U+98DB.0	fEnglish	[1] fly; flit [2] go quickly; rapidly [3] dart; high (as a bridge) [4] hang in the air; in the air [5] Kangxi radical 183
U+98DC.0	fUTF8	飜
U+98DC.0	fCNS11643	3-5E7C
U+98DC.0	fGB	E-454A
U+98DC.0	fR/S	 183.12
U+98DC.0	fTotalStrokes	 21
U+98DC.0	fCangjie	HWNOO
U+98DC.0	fFourCorner	22610
U+98DC.0	fCantonese	faan1
U+98DC.0	fMandarin	fan1
U+98DC.0	fEnglish	 turn over
U+98DD.0	fUTF8	飝
U+98DD.0	fCNS11643	3-6226
U+98DD.0	fGB	E-454B
U+98DD.0	fR/S	  183.18
U+98DD.0	fTotalStrokes	 27
U+98DD.0	fCangjie	NONOO
U+98DD.0	fFourCorner	12410
U+98DD.0	fCantonese	fei1
U+98DD.0	fMandarin	fei1
U+98DE.0	fUTF8	飞
U+98DE.0	fGB	0-3749
U+98DE.0	fR/S	 5.2; 183.0
U+98DE.0	fTotalStrokes	 3
U+98DE.0	fCangjie	NO
U+98DE.0	fHakka	 fui1
U+98DE.0	fCantonese	fei1
U+98DE.0	fMandarin	fei1
U+98DE.0	fEnglish	simplified form of &#x98DB;
U+98DF.0	fUTF8	食
U+98DF.0	fCNS11643	1-537A
U+98DF.0	fBig5	ADB9
U+98DF.0	fGB	0-4A33
U+98DF.0	fR/S	 184.0
U+98DF.0	fTotalStrokes	 9
U+98DF.0	fCangjie	OIAV
U+98DF.0	fFourCorner	80732
U+98DF.0	fHakka	 sit8; shit8; sii5; siit8; sii6; se5
U+98DF.0	fCantonese	sik6; zi6; ji6; sik2 (also)
U+98DF.0	fMandarin	shi2; yi4
U+98DF.0	fEnglish	[1] eat [2] meal; food [3] livelihood; living [4] salary; pay [5] same as $ [6] Kangxi radical 184
U+98DF.1	fUTF8	食
U+98DF.1	fCNS11643	1-537A
U+98DF.1	fBig5	ADB9
U+98DF.1	fGB	0-4A33
U+98DF.1	fR/S	 184.0
U+98DF.1	fTotalStrokes	 9
U+98DF.1	fCangjie	OIAV
U+98DF.1	fFourCorner	80732
U+98DF.1	fMandarin	si4
U+98DF.1	fEnglish	 feed
U+98DF.2	fUTF8	食
U+98DF.2	fCNS11643	1-537A
U+98DF.2	fBig5	ADB9
U+98DF.2	fGB	0-4A33
U+98DF.2	fR/S	 184.0
U+98DF.2	fTotalStrokes	 9
U+98DF.2	fCangjie	OIAV
U+98DF.2	fFourCorner	80732
U+98DF.2	fMandarin	shi5
U+98E0.0	fUTF8	飠
U+98E0.0	fGB	1-623B
U+98E0.0	fR/S	 184.0
U+98E0.0	fTotalStrokes	 9
U+98E0.0	fCangjie	OIAI
U+98E0.0	fMandarin	shi2; si4
U+98E0.0	fEnglish	 eat; food Kangxi radical number 184
U+98E1.0	fUTF8	飡
U+98E1.0	fCNS11643	3-3967
U+98E1.0	fGB	5-3370
U+98E1.0	fR/S	 184.2
U+98E1.0	fTotalStrokes	 11
U+98E1.0	fCangjie	IMOIV
U+98E1.0	fFourCorner	38130
U+98E1.0	fHakka	 con1; zhan1; zan1
U+98E1.0	fCantonese	syun1; caan1
U+98E1.0	fMandarin	can1; sun1
U+98E1.0	fEnglish	 meal; eat
U+98E2.0	fUTF8	飢
U+98E2.0	fCNS11643	1-586A
U+98E2.0	fBig5	B0A7
U+98E2.0	fGB	1-783F
U+98E2.0	fR/S	 184.2
U+98E2.0	fTotalStrokes	 11
U+98E2.0	fCangjie	OIHN
U+98E2.0	fFourCorner	87710
U+98E2.0	fHakka	  gi1
U+98E2.0	fCantonese	gei1
U+98E2.0	fMandarin	ji1
U+98E2.0	fEnglish	[1] hunger; starving; hungry; famine [2] a Chinese family name
U+98E3.0	fUTF8	飣
U+98E3.0	fCNS11643	2-334B
U+98E3.0	fBig5	D448
U+98E3.0	fGB	3-7A49
U+98E3.0	fR/S	 184.2
U+98E3.0	fTotalStrokes	 11
U+98E3.0	fCangjie	OIMN
U+98E3.0	fFourCorner	81720
U+98E3.0	fHakka	  dang5;  den5
U+98E3.0	fCantonese	ding3
U+98E3.0	fMandarin	ding4
U+98E3.0	fEnglish	[1] food displayed for show only [2] flowery language without substance
U+98E4.0	fUTF8	飤
U+98E4.0	fCNS11643	3-6670
U+98E4.0	fGB	E-454C
U+98E4.0	fR/S	 184.2
U+98E4.0	fTotalStrokes	 10; 11
U+98E4.0	fCangjie	OIO
U+98E4.0	fFourCorner	8870
U+98E4.0	fCantonese	zi6
U+98E4.0	fMandarin	si4
U+98E4.0	fEnglish	 feed provisions
U+98E5.0	fUTF8	飥
U+98E5.0	fCNS11643	2-3A33
U+98E5.0	fBig5	D84F
U+98E5.0	fGB	3-7A4A
U+98E5.0	fR/S	 184.3
U+98E5.0	fTotalStrokes	 12
U+98E5.0	fCangjie	OIHP
U+98E5.0	fFourCorner	82714
U+98E5.0	fHakka	  tok7
U+98E5.0	fCantonese	tok3
U+98E5.0	fMandarin	tuo1; tuo2
U+98E5.0	fEnglish	rice cake
U+98E6.0	fUTF8	飦
U+98E6.0	fCNS11643	3-3968
U+98E6.0	fGB	5-784A
U+98E6.0	fR/S	 184.3
U+98E6.0	fTotalStrokes	 11; 12
U+98E6.0	fCangjie	OIMJ
U+98E6.0	fFourCorner	81740
U+98E6.0	fHakka	 gon1; zhen1
U+98E6.0	fCantonese	zin1; zi1 (also)
U+98E6.0	fMandarin	zhan1; jian1; gan1
U+98E6.0	fEnglish	 well-boiled congee or gruel thick; rich
U+98E7.0	fUTF8	飧
U+98E7.0	fCNS11643	1-627D
U+98E7.0	fBig5	B6B8
U+98E7.0	fGB	0-6238
U+98E7.0	fR/S	 184.3
U+98E7.0	fTotalStrokes	 12
U+98E7.0	fCangjie	NIOIV
U+98E7.0	fFourCorner	28232
U+98E7.0	fHakka	 sun1; con1
U+98E7.0	fCantonese	syun1
U+98E7.0	fMandarin	sun1
U+98E7.0	fEnglish	 evening meal; supper cooked food
U+98E8.0	fUTF8	飨
U+98E8.0	fGB	0-774F
U+98E8.0	fR/S	 184.3
U+98E8.0	fTotalStrokes	 12
U+98E8.0	fCangjie	VHOIV
U+98E8.0	fHakka	 hiong3
U+98E8.0	fCantonese	hoeng2
U+98E8.0	fMandarin	xiang3; xiang1
U+98E8.0	fEnglish	simplified form of &#x9957;
U+98E9.0	fUTF8	飩
U+98E9.0	fCNS11643	1-6322
U+98E9.0	fBig5	B6BB
U+98E9.0	fGB	1-623D
U+98E9.0	fR/S	 184.4
U+98E9.0	fTotalStrokes	 13
U+98E9.0	fCangjie	OIPU
U+98E9.0	fFourCorner	85717
U+98E9.0	fHakka	 tun2; tun1
U+98E9.0	fCantonese	tan1; tan4 (also); tyun4
U+98E9.0	fMandarin	tun2; dun4; tun5; zhun4
U+98E9.0	fEnglish	correct form of &#xstuffed dumplings;
U+98EA.0	fUTF8	飪
U+98EA.0	fCNS11643	1-627E
U+98EA.0	fBig5	B6B9
U+98EA.0	fGB	1-623F
U+98EA.0	fR/S	 184.4
U+98EA.0	fTotalStrokes	 13
U+98EA.0	fCangjie	OIHG
U+98EA.0	fFourCorner	81714
U+98EA.0	fHakka	 ngim2; ngim3; ngim5
U+98EA.0	fCantonese	jam6; jam5 (also)
U+98EA.0	fMandarin	ren4; ren3
U+98EA.0	fEnglish	 cook
U+98EB.0	fUTF8	飫
U+98EB.0	fCNS11643	2-4153
U+98EB.0	fBig5	DCAE
U+98EB.0	fGB	1-6240
U+98EB.0	fR/S	 184.4
U+98EB.0	fTotalStrokes	 13
U+98EB.0	fCangjie	OIHK
U+98EB.0	fFourCorner	82734
U+98EB.0	fHakka	 ji5; ji3; ri5; i5
U+98EB.0	fCantonese	jyu3
U+98EB.0	fMandarin	yu4
U+98EB.0	fEnglish	[1] surfeited; glutted [2] grant; confer [3] fest [4] eat or drink to repletion
U+98EC.0	fUTF8	飬
U+98EC.0	fCNS11643	3-455B
U+98EC.0	fGB	E-454D
U+98EC.0	fR/S	 184.4
U+98EC.0	fTotalStrokes	 13
U+98EC.0	fCangjie	FQIAV
U+98EC.0	fFourCorner	90730
U+98EC.0	fCantonese	gyun3
U+98EC.0	fMandarin	yang3; juan4
U+98ED.0	fUTF8	飭
U+98ED.0	fCNS11643	1-6324
U+98ED.0	fBig5	B6BD
U+98ED.0	fGB	1-6241
U+98ED.0	fR/S	 184.4
U+98ED.0	fTotalStrokes	 13
U+98ED.0	fCangjie	OIOKS
U+98ED.0	fFourCorner	88727
U+98ED.0	fHakka	  hit7; shit7; cit7; siit7
U+98ED.0	fCantonese	cik1; sik1
U+98ED.0	fMandarin	chi4; shi4
U+98ED.0	fEnglish	[1] severe [2] reverent; respectful; careful [3] manage; keep in order; make ready [4] command; give command; order; instruct; direct
U+98EE.0	fUTF8	飮
U+98EE.0	fGB	E-454E
U+98EE.0	fR/S	 184.4
U+98EE.0	fTotalStrokes	 13
U+98EE.0	fCangjie	OSNO
U+98EE.0	fHakka	 jim3
U+98EE.0	fCantonese	jam2
U+98EE.0	fMandarin	yin3; yin4
U+98EE.0	fEnglish	 drink swallow kind of drink
U+98EF.0	fUTF8	飯
U+98EF.0	fCNS11643	1-6321
U+98EF.0	fBig5	B6BA
U+98EF.0	fGB	1-3739
U+98EF.0	fR/S	 184.4
U+98EF.0	fTotalStrokes	 13
U+98EF.0	fCangjie	OIHE
U+98EF.0	fFourCorner	81747
U+98EF.0	fHakka	  fan5;  fan3; fan6; fon6; p'on6; p'on5
U+98EF.0	fCantonese	faan6; faan2 (also); faan5
U+98EF.0	fMandarin	fan4; fan3
U+98EF.0	fEnglish	[1] cooked rice; cooked grain for food [2] meal [3] feed [4] profession; means for living
U+98F0.0	fUTF8	飰
U+98F0.0	fCNS11643	3-402B
U+98F0.0	fGB	E-454F
U+98F0.0	fR/S	 184.4
U+98F0.0	fTotalStrokes	 12; 13
U+98F0.0	fCangjie	OIYY
U+98F0.0	fFourCorner	80730
U+98F0.0	fCantonese	faan6
U+98F0.0	fMandarin	fan4
U+98F1.0	fUTF8	飱
U+98F1.0	fCNS11643	3-455C
U+98F1.0	fGB	E-4550
U+98F1.0	fR/S	 184.4; 78.9
U+98F1.0	fTotalStrokes	 13
U+98F1.0	fCangjie	MNOIV
U+98F1.0	fFourCorner	18230
U+98F1.0	fHakka	 con1
U+98F1.0	fCantonese	syun1
U+98F1.0	fMandarin	sun1
U+98F1.0	fEnglish	evening meal
U+98F2.0	fUTF8	飲
U+98F2.0	fCNS11643	1-6323
U+98F2.0	fBig5	B6BC
U+98F2.0	fGB	1-527B
U+98F2.0	fR/S	 184.4
U+98F2.0	fTotalStrokes	 13
U+98F2.0	fCangjie	OINO
U+98F2.0	fFourCorner	87782
U+98F2.0	fHakka	  jim3; lim1; rim3
U+98F2.0	fCantonese	jam3; jam2
U+98F2.0	fMandarin	yin3
U+98F2.0	fEnglish	[1] drink [2] drinks [3] swallow (insult, danger, etc.) [4] be hit (by a bullet, an arrow, etc.)
U+98F2.1	fUTF8	飲
U+98F2.1	fCNS11643	1-6323
U+98F2.1	fBig5	B6BC
U+98F2.1	fGB	1-527B
U+98F2.1	fR/S	 184.4
U+98F2.1	fTotalStrokes	 13
U+98F2.1	fCangjie	OINO
U+98F2.1	fFourCorner	87782
U+98F2.1	fMandarin	yin4
U+98F2.1	fEnglish	 make animals drink
U+98F3.0	fUTF8	飳
U+98F3.0	fCNS11643	3-4554
U+98F3.0	fGB	3-7A55
U+98F3.0	fR/S	 184.5
U+98F3.0	fTotalStrokes	 13; 14
U+98F3.0	fCangjie	OIYG
U+98F3.0	fFourCorner	80710
U+98F3.0	fCantonese	tau2
U+98F3.0	fMandarin	zhu4; tou3
U+98F4.0	fUTF8	飴
U+98F4.0	fCNS11643	1-6766
U+98F4.0	fBig5	B97E
U+98F4.0	fGB	1-6242
U+98F4.0	fR/S	 184.5
U+98F4.0	fTotalStrokes	 14
U+98F4.0	fCangjie	OIIR
U+98F4.0	fFourCorner	83760
U+98F4.0	fHakka	  ji2; ri2; i2
U+98F4.0	fCantonese	ji4; zi6
U+98F4.0	fMandarin	yi2; si4
U+98F4.0	fEnglish	[1] sweet-cakes; syrup; jellylike sugar made from grains [2] delicious; tasty [3] delicacies [4] give a gift; present
U+98F5.0	fUTF8	飵
U+98F5.0	fCNS11643	3-4559
U+98F5.0	fGB	3-7A53
U+98F5.0	fR/S	  184.5
U+98F5.0	fTotalStrokes	 13; 14
U+98F5.0	fCangjie	OIOS
U+98F5.0	fFourCorner	88710
U+98F5.0	fCantonese	zok3 (also); zok6
U+98F5.0	fMandarin	zhai3; zuo4
U+98F5.0	fEnglish	food; profit
U+98F6.0	fUTF8	飶
U+98F6.0	fCNS11643	2-4868
U+98F6.0	fBig5	E0E2
U+98F6.0	fGB	3-7A58
U+98F6.0	fR/S	 184.5
U+98F6.0	fTotalStrokes	 14
U+98F6.0	fCangjie	OIPH
U+98F6.0	fFourCorner	83700
U+98F6.0	fHakka	  bit7
U+98F6.0	fCantonese	bit1 (also); bit6; bei3; bat6
U+98F6.0	fMandarin	bi4; bie2
U+98F6.0	fEnglish	odour
U+98F7.0	fUTF8	飷
U+98F7.0	fCNS11643	3-4557
U+98F7.0	fGB	3-7A51
U+98F7.0	fR/S	 184.5
U+98F7.0	fTotalStrokes	 13; 14
U+98F7.0	fCangjie	OIBM
U+98F7.0	fFourCorner	87710
U+98F7.0	fCantonese	ze2
U+98F7.0	fMandarin	jie3; zha3; zhu1
U+98F8.0	fUTF8	飸
U+98F8.0	fCNS11643	3-4A5D
U+98F8.0	fGB	E-4551
U+98F8.0	fR/S	 184.5
U+98F8.0	fTotalStrokes	 14
U+98F8.0	fCangjie	RSOIV
U+98F8.0	fFourCorner	68230
U+98F8.0	fCantonese	tou1
U+98F8.0	fMandarin	tao1
U+98F9.0	fUTF8	飹
U+98F9.0	fCNS11643	2-4869
U+98F9.0	fBig5	E0E3
U+98F9.0	fGB	E-4552
U+98F9.0	fR/S	 184.5
U+98F9.0	fTotalStrokes	 14
U+98F9.0	fCangjie	OIHHL
U+98F9.0	fFourCorner	87720
U+98F9.0	fCantonese	baau2
U+98F9.0	fMandarin	liu3
U+98FA.0	fUTF8	飺
U+98FA.0	fCNS11643	2-5569
U+98FA.0	fBig5	E8C0
U+98FA.0	fGB	5-7847
U+98FA.0	fR/S	 184.6; 184.5
U+98FA.0	fTotalStrokes	 15
U+98FA.0	fCangjie	YPOIV
U+98FA.0	fFourCorner	21732
U+98FA.0	fCantonese	ci1
U+98FA.0	fMandarin	ci2; zi3
U+98FB.0	fUTF8	飻
U+98FB.0	fCNS11643	4-417B
U+98FB.0	fGB	5-7851
U+98FB.0	fR/S	 184.5
U+98FB.0	fTotalStrokes	 13; 14
U+98FB.0	fCangjie	OIOHH
U+98FB.0	fFourCorner	88720
U+98FB.0	fHakka	 tiet7
U+98FB.0	fCantonese	tit3
U+98FB.0	fMandarin	tie4
U+98FC.0	fUTF8	飼
U+98FC.0	fCNS11643	1-6765
U+98FC.0	fBig5	B97D
U+98FC.0	fGB	1-4B47
U+98FC.0	fR/S	 184.5
U+98FC.0	fTotalStrokes	 14
U+98FC.0	fCangjie	OISMR
U+98FC.0	fFourCorner	87720
U+98FC.0	fHakka	 cih2; sii5; cu2; sii1; cu3; ci6; ci5; sii6; ci3; se5
U+98FC.0	fCantonese	zi6
U+98FC.0	fMandarin	si4
U+98FC.0	fEnglish	 raise (domesticated) animals; feed
U+98FD.0	fUTF8	飽
U+98FD.0	fCNS11643	1-6767
U+98FD.0	fBig5	B9A1
U+98FD.0	fGB	1-3125
U+98FD.0	fR/S	 184.5
U+98FD.0	fTotalStrokes	 14
U+98FD.0	fCangjie	OIPRU
U+98FD.0	fFourCorner	87712
U+98FD.0	fHakka	  bau3
U+98FD.0	fCantonese	baau2
U+98FD.0	fMandarin	bao3
U+98FD.0	fEnglish	[1] eat heartily; eat to the full; surfeited [2] satisfied [3] full; plump [4] fully; to the full
U+98FE.0	fUTF8	飾
U+98FE.0	fCNS11643	1-6768
U+98FE.0	fBig5	B9A2
U+98FE.0	fGB	1-4A4E
U+98FE.0	fR/S	 184.5
U+98FE.0	fTotalStrokes	 14
U+98FE.0	fCangjie	OIOLB
U+98FE.0	fFourCorner	88727
U+98FE.0	fHakka	  shit7;  sit7; hit7; siit7
U+98FE.0	fCantonese	sik1; cik1
U+98FE.0	fMandarin	shi4; chi4
U+98FE.0	fEnglish	[1] decorate; ornament; polish (writing); adorn [2] ornamentation; decorations [3] excuse oneself on a pretext, etc.; fake [4] clothing and dresses [5] whitewash; deceive; cover up [6] play the role of; act the part of
U+98FF.0	fUTF8	飿
U+98FF.0	fCNS11643	3-4558
U+98FF.0	fGB	3-7A59
U+98FF.0	fR/S	 184.5
U+98FF.0	fTotalStrokes	 13; 14
U+98FF.0	fCangjie	OIUU
U+98FF.0	fFourCorner	82770
U+98FF.0	fCantonese	deot1; teot1
U+98FF.0	fMandarin	duo4
U+9900.0	fUTF8	餀
U+9900.0	fCNS11643	2-4F37
U+9900.0	fBig5	E4CF
U+9900.0	fGB	5-7850
U+9900.0	fR/S	 184.6
U+9900.0	fTotalStrokes	 15
U+9900.0	fCangjie	OITK
U+9900.0	fFourCorner	84740
U+9900.0	fHakka	 ngoi5; ngai5; ngoi6; ngai6
U+9900.0	fCantonese	hoi3; ngaai6; ngai6
U+9900.0	fMandarin	hai4; kai4; xi4
U+9900.0	fEnglish	food which has been spoiled by keeping
U+9901.0	fUTF8	餁
U+9901.0	fCNS11643	3-4A5E
U+9901.0	fGB	E-4553
U+9901.0	fR/S	 184.6
U+9901.0	fTotalStrokes	 14; 15
U+9901.0	fCangjie	OIOHG
U+9901.0	fFourCorner	82710
U+9901.0	fCantonese	jam6; jam5 (also)
U+9901.0	fMandarin	ren4
U+9901.0	fEnglish	same as $
U+9902.0	fUTF8	餂
U+9902.0	fCNS11643	2-4F36
U+9902.0	fBig5	E4CE
U+9902.0	fGB	3-7A5C
U+9902.0	fR/S	 184.6
U+9902.0	fTotalStrokes	 15
U+9902.0	fCangjie	OIHJR
U+9902.0	fFourCorner	81764
U+9902.0	fHakka	  tiam3
U+9902.0	fCantonese	tim5; tim2 (also); tim4
U+9902.0	fMandarin	tian3; tian2; tian4
U+9902.0	fEnglish	 obtain by hook; entice or bait with flatteries
U+9903.0	fUTF8	餃
U+9903.0	fCNS11643	1-6B6E
U+9903.0	fBig5	BBE5
U+9903.0	fGB	1-3D48
U+9903.0	fR/S	 184.6
U+9903.0	fTotalStrokes	 15
U+9903.0	fCangjie	OIYCK
U+9903.0	fFourCorner	80748
U+9903.0	fHakka	  gau3;  giau2; giau3
U+9903.0	fCantonese	gaau2
U+9903.0	fMandarin	jiao4; jiao3
U+9903.0	fEnglish	stuffed dumplings
U+9904.0	fUTF8	餄
U+9904.0	fCNS11643	5-4B4C
U+9904.0	fGB	3-7A5D
U+9904.0	fR/S	 184.6
U+9904.0	fTotalStrokes	 14; 15
U+9904.0	fCangjie	OIOMR
U+9904.0	fFourCorner	88760
U+9904.0	fHakka	 giap7
U+9904.0	fCantonese	hap6 (also); gaap3
U+9904.0	fMandarin	jia2; he2
U+9904.0	fEnglish	[1] cake [2] bait
U+9905.0	fUTF8	餅
U+9905.0	fCNS11643	1-6B6F
U+9905.0	fBig5	BBE6
U+9905.0	fGB	1-317D
U+9905.0	fR/S	 184.8; 184.6
U+9905.0	fTotalStrokes	 17
U+9905.0	fCangjie	OITT
U+9905.0	fFourCorner	88741
U+9905.0	fHakka	  biang3
U+9905.0	fCantonese	beng2; bing2 (also)
U+9905.0	fMandarin	bing3
U+9905.0	fEnglish	[1] cakes; biscuits; pastry [2] anything round and flat, as a disc
U+9905.1	fUTF8	餅
U+9905.1	fCNS11643	1-6B6F
U+9905.1	fBig5	BBE6
U+9905.1	fGB	1-317D
U+9905.1	fR/S	 184.8; 184.6
U+9905.1	fTotalStrokes	 17
U+9905.1	fCangjie	OITT
U+9905.1	fFourCorner	88741
U+9905.1	fMandarin	bing5
U+9906.0	fUTF8	餆
U+9906.0	fCNS11643	3-4A5B
U+9906.0	fGB	E-4554
U+9906.0	fR/S	 184.6
U+9906.0	fTotalStrokes	 14; 15
U+9906.0	fCangjie	OILMO
U+9906.0	fFourCorner	82710
U+9906.0	fCantonese	jiu4
U+9906.0	fMandarin	yao2
U+9907.0	fUTF8	餇
U+9907.0	fCNS11643	2-4F38
U+9907.0	fBig5	E4D0
U+9907.0	fGB	E-4555
U+9907.0	fR/S	 184.6
U+9907.0	fTotalStrokes	 15
U+9907.0	fCangjie	OIBMR
U+9907.0	fFourCorner	87720
U+9907.0	fCantonese	tung4
U+9907.0	fMandarin	tong2
U+9907.0	fEnglish	food
U+9908.0	fUTF8	餈
U+9908.0	fCNS11643	2-5568
U+9908.0	fBig5	E8BF
U+9908.0	fGB	E-4556
U+9908.0	fR/S	 184.6
U+9908.0	fTotalStrokes	 15
U+9908.0	fCangjie	IOOIV
U+9908.0	fFourCorner	37732
U+9908.0	fCantonese	ci4
U+9908.0	fMandarin	ci2; zi1
U+9908.0	fEnglish	rice cake
U+9909.0	fUTF8	餉
U+9909.0	fCNS11643	1-6B71
U+9909.0	fBig5	BBE8
U+9909.0	fGB	1-6243
U+9909.0	fR/S	 184.6
U+9909.0	fTotalStrokes	 15
U+9909.0	fCangjie	OIHBR
U+9909.0	fFourCorner	87720
U+9909.0	fHakka	  hiong3; hiong1
U+9909.0	fCantonese	hoeng2
U+9909.0	fMandarin	xiang3; shang1; shang3; shang4
U+9909.0	fEnglish	[1] pay, provisions, rations, etc. for the military or the police [2] entertain with food; feast; present food as a gift
U+990A.0	fUTF8	養
U+990A.0	fCNS11643	1-6F72
U+990A.0	fBig5	BE69
U+990A.0	fGB	1-5178
U+990A.0	fR/S	 184.6
U+990A.0	fTotalStrokes	 15
U+990A.0	fCangjie	TOIAV
U+990A.0	fFourCorner	80732
U+990A.0	fHakka	  jong1; jong3; rong1; rong3; {jong1}
U+990A.0	fCantonese	joeng5; joeng6
U+990A.0	fMandarin	yang3
U+990A.0	fEnglish	[1] grow; raise; rear; bring up; support; breed [2] support; keep (a family, etc.) [3] give birth to [4] nourish; cultivate (one's mind, etc.) [5] educate [6] nurse (a wound or illness) [7] oxygen [8] a Chinese family name
U+990A.1	fUTF8	養
U+990A.1	fCNS11643	1-6F72
U+990A.1	fBig5	BE69
U+990A.1	fGB	1-5178
U+990A.1	fR/S	 184.6
U+990A.1	fTotalStrokes	 15
U+990A.1	fCangjie	TOIAV
U+990A.1	fFourCorner	80732
U+990A.1	fMandarin	yang4
U+990A.1	fEnglish	 support one's parents
U+990B.0	fUTF8	餋
U+990B.0	fCNS11643	4-4E63
U+990B.0	fGB	5-7848
U+990B.0	fR/S	 184.6
U+990B.0	fTotalStrokes	 15
U+990B.0	fCangjie	FQOIV
U+990B.0	fFourCorner	90730
U+990B.0	fCantonese	gyun3
U+990B.0	fMandarin	juan4; yang3
U+990C.0	fUTF8	餌
U+990C.0	fCNS11643	1-6B70
U+990C.0	fBig5	BBE7
U+990C.0	fGB	1-367C
U+990C.0	fR/S	 184.6
U+990C.0	fTotalStrokes	 15
U+990C.0	fCangjie	OISJ
U+990C.0	fFourCorner	81740
U+990C.0	fHakka	  ngi3; gniap7
U+990C.0	fCantonese	nei6; lei6 (also)
U+990C.0	fMandarin	er3; er4
U+990C.0	fEnglish	[1] bait; entice [2] cakes [3] food [4] eat
U+990D.0	fUTF8	餍
U+990D.0	fGB	0-7750
U+990D.0	fR/S	 184.6
U+990D.0	fTotalStrokes	 15
U+990D.0	fCangjie	MIKV
U+990D.0	fHakka	 jam3
U+990D.0	fCantonese	jim3
U+990D.0	fMandarin	yan4; yan2
U+990D.0	fEnglish	simplified form of &#x995C;
U+990E.0	fUTF8	餎
U+990E.0	fCNS11643	3-667A
U+990E.0	fGB	3-7A5E
U+990E.0	fR/S	 184.6
U+990E.0	fTotalStrokes	 15
U+990E.0	fCangjie	OIHER
U+990E.0	fFourCorner	8776
U+990E.0	fCantonese	lok3; lok6
U+990E.0	fMandarin	le5
U+990F.0	fUTF8	餏
U+990F.0	fGB	3-7A60
U+990F.0	fR/S	 184.6
U+990F.0	fTotalStrokes	 15
U+990F.0	fCangjie	OIYHV
U+990F.0	fCantonese	si1
U+990F.0	fMandarin	yi1
U+9910.0	fUTF8	餐
U+9910.0	fCNS11643	1-7327
U+9910.0	fBig5	C05C
U+9910.0	fGB	0-324D
U+9910.0	fR/S	 184.7
U+9910.0	fTotalStrokes	 16
U+9910.0	fCangjie	YEOIV
U+9910.0	fFourCorner	27732
U+9910.0	fHakka	  con1
U+9910.0	fCantonese	caan1; syun1
U+9910.0	fMandarin	can1; sun1; sun4; can4
U+9910.0	fEnglish	[1] eat; dine [2] meal [3] food
U+9911.0	fUTF8	餑
U+9911.0	fCNS11643	2-556A
U+9911.0	fBig5	E8C1
U+9911.0	fGB	1-6244
U+9911.0	fR/S	 184.7
U+9911.0	fTotalStrokes	 16
U+9911.0	fCangjie	OIJBD
U+9911.0	fFourCorner	84747
U+9911.0	fHakka	  put8
U+9911.0	fCantonese	but6
U+9911.0	fMandarin	bo1; bo2
U+9911.0	fEnglish	cakes; fancy baked foods
U+9912.0	fUTF8	餒
U+9912.0	fCNS11643	1-6F74
U+9912.0	fBig5	BE6B
U+9912.0	fGB	1-4459
U+9912.0	fR/S	 184.7
U+9912.0	fTotalStrokes	 16
U+9912.0	fCangjie	OIBV
U+9912.0	fFourCorner	82744
U+9912.0	fHakka	  nui3; ne1; lui3; {nui1}
U+9912.0	fCantonese	neoi5; noi5 (also); leoi5 (also); loi5 (also)
U+9912.0	fMandarin	nei3; nei2
U+9912.0	fEnglish	[1] starve; hungry; famished [2] decay or decompsition of fish [3] lacking in evidence, courage, etc.; dishearthened; disprited
U+9913.0	fUTF8	餓
U+9913.0	fCNS11643	1-6F73
U+9913.0	fBig5	BE6A
U+9913.0	fGB	1-3676
U+9913.0	fR/S	 184.7
U+9913.0	fTotalStrokes	 16
U+9913.0	fCangjie	OIHQI
U+9913.0	fFourCorner	83750
U+9913.0	fHakka	  ngo5; ngo6; ngo3
U+9913.0	fCantonese	ngo6; o6
U+9913.0	fMandarin	e4
U+9913.0	fEnglish	[1] hungry; hunger [2] greedy; covetous [3] starve
U+9914.0	fUTF8	餔
U+9914.0	fCNS11643	2-556B
U+9914.0	fBig5	E8C2
U+9914.0	fGB	3-7A63
U+9914.0	fR/S	 184.7
U+9914.0	fTotalStrokes	 16
U+9914.0	fCangjie	OIIJB
U+9914.0	fFourCorner	83727
U+9914.0	fHakka	  pu3;  bu1; pu2
U+9914.0	fCantonese	bou1; bou3 (also); bou6; pou1
U+9914.0	fMandarin	bu1; bu4; bu3
U+9914.0	fEnglish	[1] eat [2] time for supper; dinner time [3] sunset
U+9915.0	fUTF8	餕
U+9915.0	fCNS11643	2-556E
U+9915.0	fBig5	E8C5
U+9915.0	fGB	3-7A6A
U+9915.0	fR/S	 184.7
U+9915.0	fTotalStrokes	 16
U+9915.0	fCangjie	OIICE
U+9915.0	fFourCorner	83747
U+9915.0	fHakka	  zun5
U+9915.0	fCantonese	zeon3
U+9915.0	fMandarin	jun4; quan2
U+9915.0	fEnglish	leftovers
U+9916.0	fUTF8	餖
U+9916.0	fCNS11643	2-556C
U+9916.0	fBig5	E8C3
U+9916.0	fGB	3-7A65
U+9916.0	fR/S	 184.7
U+9916.0	fTotalStrokes	 16
U+9916.0	fCangjie	OIMRT
U+9916.0	fFourCorner	81718
U+9916.0	fHakka	 teu6;  teu5
U+9916.0	fCantonese	dau6
U+9916.0	fMandarin	dou4
U+9916.0	fEnglish	 food items set out for show only, not to be eaten
U+9917.0	fUTF8	餗
U+9917.0	fCNS11643	2-556D
U+9917.0	fBig5	E8C4
U+9917.0	fGB	3-7A64
U+9917.0	fR/S	 184.7
U+9917.0	fTotalStrokes	 16
U+9917.0	fCangjie	OIDL
U+9917.0	fFourCorner	85796
U+9917.0	fHakka	  suk7
U+9917.0	fCantonese	cuk1; suk1
U+9917.0	fMandarin	su4; chuo4
U+9917.0	fEnglish	food in a tripod or caldron
U+9918.0	fUTF8	餘
U+9918.0	fCNS11643	1-6F75
U+9918.0	fBig5	BE6C
U+9918.0	fGB	1-5360
U+9918.0	fR/S	 184.7
U+9918.0	fTotalStrokes	 16
U+9918.0	fCangjie	OIOMD
U+9918.0	fFourCorner	88794
U+9918.0	fHakka	 ji2; ji3; ri2; i2
U+9918.0	fCantonese	jyu4
U+9918.0	fMandarin	yu2; ye2
U+9918.0	fEnglish	[1] remaining; remainder; rest [2] surplus; excess; overplus [3] balance [4] complement of a number or figure; odd [5] after
U+9919.0	fUTF8	餙
U+9919.0	fCNS11643	3-4F6B
U+9919.0	fGB	E-4557
U+9919.0	fR/S	  184.7
U+9919.0	fTotalStrokes	 15; 16
U+9919.0	fCangjie	OIKKB
U+9919.0	fFourCorner	84720
U+9919.0	fHakka	 hit7;  shit7; siit7
U+9919.0	fCantonese	sik1
U+9919.0	fMandarin	shi4
U+9919.0	fEnglish	 adorn ornaments decieve
U+991A.0	fUTF8	餚
U+991A.0	fCNS11643	1-732C
U+991A.0	fBig5	C061
U+991A.0	fGB	E-4558
U+991A.0	fR/S	 184.8
U+991A.0	fTotalStrokes	 17
U+991A.0	fCangjie	OIKKB
U+991A.0	fFourCorner	84727
U+991A.0	fHakka	 ngau2
U+991A.0	fCantonese	ngaau4; aau4 (also)
U+991A.0	fMandarin	yao2; xiao2; hao2
U+991A.0	fEnglish	dishes and foods
U+991B.0	fUTF8	餛
U+991B.0	fCNS11643	1-732A
U+991B.0	fBig5	C05F
U+991B.0	fGB	1-6246
U+991B.0	fR/S	 184.8
U+991B.0	fTotalStrokes	 17
U+991B.0	fCangjie	OIAPP
U+991B.0	fFourCorner	86711
U+991B.0	fHakka	  fun2; wun2
U+991B.0	fCantonese	wan4
U+991B.0	fMandarin	hun2; kun1
U+991B.0	fEnglish	stuffed dumplings with delicate flour wrapping; ravioli
U+991C.0	fUTF8	餜
U+991C.0	fCNS11643	3-5365
U+991C.0	fGB	3-7A6E
U+991C.0	fR/S	  184.8
U+991C.0	fTotalStrokes	 16; 17
U+991C.0	fCangjie	OIWD
U+991C.0	fFourCorner	86790
U+991C.0	fHakka	 go3; gwo3; kwo3
U+991C.0	fCantonese	gwo2; go2 (also)
U+991C.0	fMandarin	guo3
U+991C.0	fEnglish	cakes; dumplings
U+991D.0	fUTF8	餝
U+991D.0	fCNS11643	3-5366
U+991D.0	fGB	5-785A
U+991D.0	fR/S	 184.7; 184.8
U+991D.0	fTotalStrokes	 16
U+991D.0	fCangjie	OITYS
U+991D.0	fFourCorner	84720
U+991D.0	fCantonese	sik1
U+991D.0	fMandarin	shi4
U+991E.0	fUTF8	餞
U+991E.0	fCNS11643	1-7329
U+991E.0	fBig5	C05E
U+991E.0	fGB	1-3D24
U+991E.0	fR/S	 184.8
U+991E.0	fTotalStrokes	 17
U+991E.0	fCangjie	OIII
U+991E.0	fFourCorner	83753
U+991E.0	fHakka	  zien5; zen5
U+991E.0	fCantonese	zin3; zin6
U+991E.0	fMandarin	jian4
U+991E.0	fEnglish	[1] farewell dinner or luncheon [2] send off; convoy [3] present as a gift [4] preserves; jam
U+991F.0	fUTF8	餟
U+991F.0	fCNS11643	2-5B69
U+991F.0	fBig5	EC5D
U+991F.0	fGB	5-7864
U+991F.0	fR/S	 184.8
U+991F.0	fTotalStrokes	 17
U+991F.0	fCangjie	OIEEE
U+991F.0	fFourCorner	87747
U+991F.0	fHakka	 dot7
U+991F.0	fCantonese	zeoi3; zyut3
U+991F.0	fMandarin	zhui4; zhuo2
U+991F.0	fEnglish	 libation
U+9920.0	fUTF8	餠
U+9920.0	fCNS11643	3-5364
U+9920.0	fGB	E-4559
U+9920.0	fR/S	 184.8
U+9920.0	fTotalStrokes	 16; 17
U+9920.0	fCangjie	OIYJJ
U+9920.0	fFourCorner	83740
U+9920.0	fHakka	  biang3
U+9920.0	fCantonese	beng2; bing2 (also)
U+9920.0	fMandarin	bing3
U+9920.0	fEnglish	rice-cakes; biscuits
U+9921.0	fUTF8	餡
U+9921.0	fCNS11643	1-732B
U+9921.0	fBig5	C060
U+9921.0	fGB	1-4F5A
U+9921.0	fR/S	 184.8
U+9921.0	fTotalStrokes	 17
U+9921.0	fCangjie	OINHX
U+9921.0	fFourCorner	87777
U+9921.0	fHakka	 ham5; ham3; ham6; tau1
U+9921.0	fCantonese	haam2; haam6 (also); haam5
U+9921.0	fMandarin	kan4; xian4
U+9921.0	fEnglish	anything sering as filling or stuffing for dumplings
U+9922.0	fUTF8	餢
U+9922.0	fCNS11643	3-5362
U+9922.0	fGB	E-455A
U+9922.0	fR/S	 184.8
U+9922.0	fTotalStrokes	 16; 17
U+9922.0	fCangjie	OIYTR
U+9922.0	fFourCorner	80760
U+9922.0	fCantonese	bau6
U+9922.0	fMandarin	bu4
U+9923.0	fUTF8	餣
U+9923.0	fCNS11643	3-5363
U+9923.0	fGB	3-7A6D
U+9923.0	fR/S	 184.8
U+9923.0	fTotalStrokes	 16; 17
U+9923.0	fCangjie	OIKLU
U+9923.0	fFourCorner	84710
U+9923.0	fCantonese	jip3
U+9923.0	fMandarin	ye4
U+9924.0	fUTF8	餤
U+9924.0	fCNS11643	2-5B68
U+9924.0	fBig5	EC5C
U+9924.0	fGB	3-7A73
U+9924.0	fR/S	 184.8
U+9924.0	fTotalStrokes	 17
U+9924.0	fCangjie	OIFF
U+9924.0	fFourCorner	89789
U+9924.0	fHakka	  tam2
U+9924.0	fCantonese	daam6; taam4
U+9924.0	fMandarin	tan2; dan4; yan2
U+9924.0	fEnglish	incite
U+9925.0	fUTF8	餥
U+9925.0	fCNS11643	2-6057
U+9925.0	fBig5	EF4B
U+9925.0	fGB	3-745E
U+9925.0	fR/S	 184.8
U+9925.0	fTotalStrokes	 17
U+9925.0	fCangjie	LYOIV
U+9925.0	fFourCorner	11732
U+9925.0	fCantonese	fei1
U+9925.0	fMandarin	fei3; fei1; fei4
U+9926.0	fUTF8	餦
U+9926.0	fCNS11643	4-547E
U+9926.0	fGB	3-7A4D
U+9926.0	fR/S	 184.8
U+9926.0	fTotalStrokes	 16; 17
U+9926.0	fCangjie	OISMV
U+9926.0	fFourCorner	81730
U+9926.0	fCantonese	zoeng1
U+9926.0	fMandarin	zhang1
U+9926.0	fEnglish	candy; pastry
U+9927.0	fUTF8	餧
U+9927.0	fCNS11643	2-5B6A
U+9927.0	fBig5	EC5E
U+9927.0	fGB	E-455B
U+9927.0	fR/S	 184.8
U+9927.0	fTotalStrokes	 17
U+9927.0	fCangjie	OIHDV
U+9927.0	fFourCorner	82744
U+9927.0	fHakka	 wi3; wui3; nui3; vui3; wi5; vui5; wui5
U+9927.0	fCantonese	neoi5; noi5 (also); leoi5 (also); loi5 (also); wai3
U+9927.0	fMandarin	wei4; nei3
U+9927.0	fEnglish	same as $ (to feed)
U+9928.0	fUTF8	館
U+9928.0	fCNS11643	1-7328
U+9928.0	fBig5	C05D
U+9928.0	fGB	1-395D
U+9928.0	fR/S	 184.8
U+9928.0	fTotalStrokes	 17
U+9928.0	fCangjie	OIJRR
U+9928.0	fFourCorner	83777
U+9928.0	fHakka	  gon3; gwon3; kwan3
U+9928.0	fCantonese	gun2; gwun2 (also)
U+9928.0	fMandarin	guan3; guan4
U+9928.0	fEnglish	[1] house; guesthouse; hotel [2] stay; lodge [3] official residence [4] embassy; legation; consulate [5] a place for cultural activities [6] premises [7] school (in former times) [8] a suffix for a library, teahouse, restaurant, theater, etc.
U+9929.0	fUTF8	餩
U+9929.0	fCNS11643	2-5B6B
U+9929.0	fBig5	EC5F
U+9929.0	fGB	3-7A71
U+9929.0	fR/S	 184.8
U+9929.0	fTotalStrokes	 17
U+9929.0	fCangjie	OICYO
U+9929.0	fFourCorner	88782
U+9929.0	fCantonese	ak1; ngak1 (also)
U+9929.0	fMandarin	e4
U+9929.0	fEnglish	hiccough
U+992A.0	fUTF8	餪
U+992A.0	fCNS11643	2-605A
U+992A.0	fBig5	EF4E
U+992A.0	fGB	3-7A76
U+992A.0	fR/S	 184.9
U+992A.0	fTotalStrokes	 18
U+992A.0	fCangjie	OIMBK
U+992A.0	fFourCorner	81734
U+992A.0	fHakka	  non3
U+992A.0	fCantonese	nyun5; lyun5 (also)
U+992A.0	fMandarin	nuan3; nuan4
U+992A.0	fEnglish	feast
U+992B.0	fUTF8	餫
U+992B.0	fCNS11643	2-6058
U+992B.0	fBig5	EF4C
U+992B.0	fGB	3-7A62
U+992B.0	fR/S	 184.9
U+992B.0	fTotalStrokes	 18
U+992B.0	fCangjie	OIBJJ
U+992B.0	fFourCorner	87752
U+992B.0	fHakka	 fun2; run6; jun5
U+992B.0	fCantonese	wan6; wan4
U+992B.0	fMandarin	yun4; hun2
U+992B.0	fEnglish	 dumpling supply
U+992C.0	fUTF8	餬
U+992C.0	fCNS11643	2-6059
U+992C.0	fBig5	EF4D
U+992C.0	fGB	E-455C
U+992C.0	fR/S	 184.9
U+992C.0	fTotalStrokes	 18
U+992C.0	fCangjie	OIJRB
U+992C.0	fFourCorner	87720
U+992C.0	fHakka	 fu2
U+992C.0	fCantonese	wu4
U+992C.0	fMandarin	hu2; hu1; hu4
U+992C.0	fEnglish	[1] porridge; gruel; congee [2] paste
U+992D.0	fUTF8	餭
U+992D.0	fCNS11643	2-605E
U+992D.0	fBig5	EF52
U+992D.0	fGB	3-7A79
U+992D.0	fR/S	 184.9
U+992D.0	fTotalStrokes	 18
U+992D.0	fCangjie	OIHAG
U+992D.0	fFourCorner	86714
U+992D.0	fHakka	 fong2
U+992D.0	fCantonese	wong4
U+992D.0	fMandarin	huang2; wang2
U+992D.0	fEnglish	pastry
U+992E.0	fUTF8	餮
U+992E.0	fCNS11643	1-7775
U+992E.0	fBig5	C34B
U+992E.0	fGB	0-7751
U+992E.0	fR/S	 184.9
U+992E.0	fTotalStrokes	 18
U+992E.0	fCangjie	MHOIV
U+992E.0	fFourCorner	18732
U+992E.0	fHakka	  tiet7; tien5; tet7
U+992E.0	fCantonese	tit3
U+992E.0	fMandarin	tie4; tian3
U+992E.0	fEnglish	[1] a legendary fierce animal [2] a greedy and gluttonous person
U+992F.0	fUTF8	餯
U+992F.0	fCNS11643	2-605D
U+992F.0	fBig5	EF51
U+992F.0	fGB	3-7A7B
U+992F.0	fR/S	 184.9
U+992F.0	fTotalStrokes	 18
U+992F.0	fCangjie	OIVNO
U+992F.0	fFourCorner	87731
U+992F.0	fCantonese	fui3
U+992F.0	fMandarin	hui4
U+9930.0	fUTF8	餰
U+9930.0	fCNS11643	2-6060
U+9930.0	fBig5	EF54
U+9930.0	fGB	5-7867
U+9930.0	fR/S	 184.9
U+9930.0	fTotalStrokes	 18
U+9930.0	fCangjie	OIHON
U+9930.0	fFourCorner	81721
U+9930.0	fCantonese	gin1; zin1
U+9930.0	fMandarin	jian1; zhan1; zhan3
U+9931.0	fUTF8	餱
U+9931.0	fCNS11643	2-605F
U+9931.0	fBig5	EF53
U+9931.0	fGB	E-455D
U+9931.0	fR/S	 184.9
U+9931.0	fTotalStrokes	 18
U+9931.0	fCangjie	OIONK
U+9931.0	fFourCorner	87734
U+9931.0	fHakka	  heu2; hiu2
U+9931.0	fCantonese	hau4
U+9931.0	fMandarin	hou2
U+9931.0	fEnglish	dry provisions
U+9932.0	fUTF8	餲
U+9932.0	fCNS11643	2-605C
U+9932.0	fBig5	EF50
U+9932.0	fGB	3-7A77
U+9932.0	fR/S	 184.9
U+9932.0	fTotalStrokes	 18
U+9932.0	fCangjie	OIAPV
U+9932.0	fFourCorner	86727
U+9932.0	fHakka	 ai5; hot8
U+9932.0	fCantonese	aai3; aat3; ngaat3 (also); ngaai3 (also); hot3
U+9932.0	fMandarin	he2; yi4; e4; ai4; a2
U+9932.0	fEnglish	[1] cooked rice which has turned sour [2] spoiled; sour; mouldy
U+9933.0	fUTF8	餳
U+9933.0	fCNS11643	2-605B
U+9933.0	fBig5	EF4F
U+9933.0	fGB	1-623C
U+9933.0	fR/S	 184.9
U+9933.0	fTotalStrokes	 18
U+9933.0	fCangjie	OIAMH
U+9933.0	fFourCorner	86727
U+9933.0	fHakka	 tong2
U+9933.0	fCantonese	cing4; tong4
U+9933.0	fMandarin	tang2; xing2; qing2
U+9933.0	fEnglish	[1] malt sugar; syrup [2] sticky [3] poor; dull (eyesight)
U+9934.0	fUTF8	餴
U+9934.0	fCNS11643	4-5521
U+9934.0	fGB	3-7A6C
U+9934.0	fR/S	 184.9; 184.8
U+9934.0	fTotalStrokes	 17; 16; 18
U+9934.0	fCangjie	OIKJT
U+9934.0	fFourCorner	84740
U+9934.0	fHakka	 fun1
U+9934.0	fCantonese	ban1; fan1 (also)
U+9934.0	fMandarin	fen1
U+9934.0	fEnglish	 steam rice
U+9935.0	fUTF8	餵
U+9935.0	fCNS11643	1-762A
U+9935.0	fBig5	C1FD
U+9935.0	fGB	E-455E
U+9935.0	fR/S	 184.9
U+9935.0	fTotalStrokes	 18
U+9935.0	fCangjie	OIWMV
U+9935.0	fFourCorner	86732
U+9935.0	fHakka	 wi5; wui5; wui6; vui5
U+9935.0	fCantonese	wai3
U+9935.0	fMandarin	wei4
U+9935.0	fEnglish	 feed; raise
U+9936.0	fUTF8	餶
U+9936.0	fCNS11643	3-5962
U+9936.0	fGB	3-7A78
U+9936.0	fR/S	 184.10
U+9936.0	fTotalStrokes	 18; 19
U+9936.0	fCangjie	OIBBB
U+9936.0	fFourCorner	87720
U+9936.0	fCantonese	gwat1
U+9936.0	fMandarin	gu3
U+9937.0	fUTF8	餷
U+9937.0	fCNS11643	4-5A29
U+9937.0	fGB	1-6247
U+9937.0	fR/S	  184.9
U+9937.0	fTotalStrokes	 17; 16; 18
U+9937.0	fCangjie	OIDAM
U+9937.0	fFourCorner	84710
U+9937.0	fCantonese	caa4; zaa1
U+9937.0	fMandarin	cha1
U+9938.0	fUTF8	餸
U+9938.0	fCNS11643	3-5961
U+9938.0	fGB	E-455F
U+9938.0	fR/S	 184.10
U+9938.0	fTotalStrokes	 18; 19
U+9938.0	fCangjie	OIYTK
U+9938.0	fFourCorner	88730
U+9938.0	fHakka	 sung5
U+9938.0	fCantonese	sung3
U+9938.0	fMandarin	song4
U+9938.0	fEnglish	a side dish of food prepared to accompany rice
U+9939.0	fUTF8	餹
U+9939.0	fCNS11643	3-5966
U+9939.0	fGB	E-4560
U+9939.0	fR/S	 184.10
U+9939.0	fTotalStrokes	 18; 19
U+9939.0	fCangjie	OIILR
U+9939.0	fFourCorner	80760
U+9939.0	fHakka	  tong2
U+9939.0	fCantonese	tong4
U+9939.0	fMandarin	tang2; xing2
U+9939.0	fEnglish	 sugar sugared candy crystallized sweetmeats
U+993A.0	fUTF8	餺
U+993A.0	fCNS11643	2-6458
U+993A.0	fBig5	F1AE
U+993A.0	fGB	3-7A7D
U+993A.0	fR/S	 184.10
U+993A.0	fTotalStrokes	 19
U+993A.0	fCangjie	OIIBI
U+993A.0	fFourCorner	83742
U+993A.0	fHakka	 bok7
U+993A.0	fCantonese	bok3
U+993A.0	fMandarin	bo2
U+993A.0	fEnglish	rice cake
U+993B.0	fUTF8	餻
U+993B.0	fCNS11643	3-5965
U+993B.0	fGB	E-4561
U+993B.0	fR/S	 184.10
U+993B.0	fTotalStrokes	 18; 19
U+993B.0	fCangjie	OITGF
U+993B.0	fFourCorner	88730
U+993B.0	fHakka	  go1; gau1
U+993B.0	fCantonese	gou1
U+993B.0	fMandarin	gao1
U+993B.0	fEnglish	cakes; dumplings
U+993C.0	fUTF8	餼
U+993C.0	fCNS11643	2-6457
U+993C.0	fBig5	F1AD
U+993C.0	fGB	1-623E
U+993C.0	fR/S	 184.10
U+993C.0	fTotalStrokes	 19
U+993C.0	fCangjie	OIOND
U+993C.0	fFourCorner	88717
U+993C.0	fHakka	  hi5; ki5
U+993C.0	fCantonese	hei3
U+993C.0	fMandarin	xi4
U+993C.0	fEnglish	[1] present a gift [2] annimals; sacrficial victim; animals for sacrifices [3] grains; rice [4] animal feed; fodder
U+993D.0	fUTF8	餽
U+993D.0	fCNS11643	1-7774
U+993D.0	fBig5	C34A
U+993D.0	fGB	E-4562
U+993D.0	fR/S	 184.10
U+993D.0	fTotalStrokes	 19
U+993D.0	fCangjie	OIHUI
U+993D.0	fFourCorner	86713
U+993D.0	fHakka	 kui3; kui6; kui5; k'wui5; kwi5
U+993D.0	fCantonese	gwai6
U+993D.0	fMandarin	kui4; gui4
U+993D.0	fEnglish	[1] offer food to a superior [2] present as a gift
U+993E.0	fUTF8	餾
U+993E.0	fCNS11643	1-7772
U+993E.0	fBig5	C348
U+993E.0	fGB	1-4173
U+993E.0	fR/S	 184.10
U+993E.0	fTotalStrokes	 19
U+993E.0	fCangjie	OIHHW
U+993E.0	fFourCorner	87762
U+993E.0	fHakka	  liu5; liu2; liu3
U+993E.0	fCantonese	lau6; lau4 (also)
U+993E.0	fMandarin	liu4; liu2
U+993E.0	fEnglish	[1] steam [2] distilled water
U+993F.0	fUTF8	餿
U+993F.0	fCNS11643	1-7773
U+993F.0	fBig5	C349
U+993F.0	fGB	1-6248
U+993F.0	fR/S	 184.10
U+993F.0	fTotalStrokes	 19
U+993F.0	fCangjie	OIHXE
U+993F.0	fFourCorner	87747
U+993F.0	fHakka	 seu1; siu1; sau1
U+993F.0	fCantonese	sau1; suk1 (also)
U+993F.0	fMandarin	sou1
U+993F.0	fEnglish	[1] spoiled; rotten; decayed; stale; rancid [2] lousy; foul
U+9940.0	fUTF8	饀
U+9940.0	fCNS11643	3-5963
U+9940.0	fGB	5-786C
U+9940.0	fR/S	 184.10
U+9940.0	fTotalStrokes	 18; 19
U+9940.0	fCangjie	OIBHX
U+9940.0	fFourCorner	82777
U+9940.0	fCantonese	tou4
U+9940.0	fMandarin	tao2; tao1
U+9941.0	fUTF8	饁
U+9941.0	fCNS11643	2-6456
U+9941.0	fBig5	F1AC
U+9941.0	fGB	3-7A7C
U+9941.0	fR/S	 184.10
U+9941.0	fTotalStrokes	 19
U+9941.0	fCangjie	OIGIT
U+9941.0	fFourCorner	84717
U+9941.0	fHakka	 jap7; kap7; rap7
U+9941.0	fCantonese	jip3; jip6
U+9941.0	fMandarin	ye4ye4; ye4
U+9941.0	fEnglish	carry meal to workers in field
U+9942.0	fUTF8	饂
U+9942.0	fCNS11643	3-6671
U+9942.0	fGB	E-4563
U+9942.0	fR/S	 184.10
U+9942.0	fTotalStrokes	 19
U+9942.0	fCangjie	OIABT
U+9942.0	fFourCorner	8671
U+9942.0	fCantonese	wan1
U+9942.0	fMandarin	yun2
U+9942.0	fEnglish	Japanese noodles
U+9943.0	fUTF8	饃
U+9943.0	fCNS11643	2-677C
U+9943.0	fBig5	F3B1
U+9943.0	fGB	1-6249
U+9943.0	fR/S	 184.11; 184.10
U+9943.0	fTotalStrokes	 19
U+9943.0	fCangjie	OITAK
U+9943.0	fFourCorner	84734
U+9943.0	fHakka	 mo2; mo1
U+9943.0	fCantonese	mo4
U+9943.0	fMandarin	mo2; mo4
U+9943.0	fEnglish	bread
U+9944.0	fUTF8	饄
U+9944.0	fCNS11643	3-5B69
U+9944.0	fGB	E-4564
U+9944.0	fR/S	 184.11
U+9944.0	fTotalStrokes	 19; 20
U+9944.0	fCangjie	OIFBG
U+9944.0	fFourCorner	89710
U+9944.0	fCantonese	tong4
U+9944.0	fMandarin	tang2
U+9945.0	fUTF8	饅
U+9945.0	fCNS11643	1-794F
U+9945.0	fBig5	C443
U+9945.0	fGB	1-4278
U+9945.0	fR/S	 184.11
U+9945.0	fTotalStrokes	 20
U+9945.0	fCangjie	OIAWE
U+9945.0	fFourCorner	86747
U+9945.0	fHakka	  man2; man3
U+9945.0	fCantonese	maan4 (also); maan6
U+9945.0	fMandarin	man2
U+9945.0	fEnglish	stuffed or unstuffed dumplings; steamed bread; steamed buns
U+9946.0	fUTF8	饆
U+9946.0	fCNS11643	4-6238
U+9946.0	fGB	3-7A5A
U+9946.0	fR/S	 184.11
U+9946.0	fTotalStrokes	 20; 19
U+9946.0	fCangjie	OIWTJ
U+9946.0	fFourCorner	86754
U+9946.0	fHakka	  bit7
U+9946.0	fCantonese	bat1
U+9946.0	fMandarin	bi4
U+9946.0	fEnglish	a dumpling with meat inside
U+9947.0	fUTF8	饇
U+9947.0	fCNS11643	2-677B
U+9947.0	fBig5	F3B0
U+9947.0	fGB	3-7A4C
U+9947.0	fR/S	 184.11
U+9947.0	fTotalStrokes	 20
U+9947.0	fCangjie	OISRR
U+9947.0	fFourCorner	81716
U+9947.0	fHakka	 ji5; ri5
U+9947.0	fCantonese	jyu3
U+9947.0	fMandarin	yu4; ou3
U+9947.0	fEnglish	 eat too much; surfeited confer
U+9948.0	fUTF8	饈
U+9948.0	fCNS11643	2-677A
U+9948.0	fBig5	F3AF
U+9948.0	fGB	1-624A
U+9948.0	fR/S	 184.11
U+9948.0	fTotalStrokes	 20
U+9948.0	fCangjie	OITQG
U+9948.0	fFourCorner	88715
U+9948.0	fHakka	  siu1
U+9948.0	fCantonese	sau1
U+9948.0	fMandarin	xiu1
U+9948.0	fEnglish	[1] meal; eat [2] offer; offer as tribute [3] delicious food; delicacy; dainty
U+9949.0	fUTF8	饉
U+9949.0	fCNS11643	1-7950
U+9949.0	fBig5	C444
U+9949.0	fGB	1-624B
U+9949.0	fR/S	 184.11
U+9949.0	fTotalStrokes	 20
U+9949.0	fCangjie	OITLM
U+9949.0	fFourCorner	84714
U+9949.0	fHakka	 giun3
U+9949.0	fCantonese	gan2; gan6
U+9949.0	fMandarin	jin3; jin4
U+9949.0	fEnglish	correct form of &#xfamine;hunger
U+994A.0	fUTF8	饊
U+994A.0	fCNS11643	4-653A
U+994A.0	fGB	1-624C
U+994A.0	fR/S	 184.12
U+994A.0	fTotalStrokes	 20; 21
U+994A.0	fCangjie	OITBK
U+994A.0	fFourCorner	88740
U+994A.0	fHakka	 san3
U+994A.0	fCantonese	saan2
U+994A.0	fMandarin	san3
U+994A.0	fEnglish	fried round cakes of wheat flour
U+994B.0	fUTF8	饋
U+994B.0	fCNS11643	2-6A66
U+994B.0	fBig5	F558
U+994B.0	fGB	1-4021
U+994B.0	fR/S	 184.12
U+994B.0	fTotalStrokes	 21
U+994B.0	fCangjie	OILMC
U+994B.0	fFourCorner	85786
U+994B.0	fHakka	 kui3; kui6; kui5; k'wui5; kian5
U+994B.0	fCantonese	gwai3 (also); gwai6
U+994B.0	fMandarin	kui4; gui4; tui2; tui1
U+994B.0	fEnglish	[1] offer food to a superior [2] send someone a presen; present as a gift
U+994C.0	fUTF8	饌
U+994C.0	fCNS11643	2-6A65
U+994C.0	fBig5	F557
U+994C.0	fGB	1-624D
U+994C.0	fR/S	 184.12
U+994C.0	fTotalStrokes	 21
U+994C.0	fCangjie	OIRUC
U+994C.0	fFourCorner	87781
U+994C.0	fHakka	  con3;  con5; sien3; sion3
U+994C.0	fCantonese	zaan6; syun2
U+994C.0	fMandarin	zhuan4; xuan3; fan4
U+994C.0	fEnglish	[1] prepare food [2] food and drink; dainties [3] eat and drink
U+994D.0	fUTF8	饍
U+994D.0	fCNS11643	3-5D58
U+994D.0	fGB	E-4565
U+994D.0	fR/S	 184.12
U+994D.0	fTotalStrokes	 20; 21
U+994D.0	fCangjie	OITTR
U+994D.0	fFourCorner	88761
U+994D.0	fHakka	 san5; shan6; shan3; san3; sen3; shen5
U+994D.0	fCantonese	sin6
U+994D.0	fMandarin	shan4
U+994D.0	fEnglish	[1] meal [2] eat; board
U+994E.0	fUTF8	饎
U+994E.0	fCNS11643	2-6A63
U+994E.0	fBig5	F555
U+994E.0	fGB	3-7B23
U+994E.0	fR/S	 184.12
U+994E.0	fTotalStrokes	 21
U+994E.0	fCangjie	OIGRR
U+994E.0	fFourCorner	84761
U+994E.0	fHakka	 zhit7; tsiit7; shan5; tje5
U+994E.0	fCantonese	ci3; ci4
U+994E.0	fMandarin	chi4; xi1
U+994E.0	fEnglish	wine and food eaten with wine
U+994F.0	fUTF8	饏
U+994F.0	fCNS11643	3-5E7E
U+994F.0	fGB	E-4566
U+994F.0	fR/S	 184.12; 184.10
U+994F.0	fTotalStrokes	 21; 19
U+994F.0	fCangjie	MKOIV
U+994F.0	fFourCorner	18730
U+994F.0	fCantonese	daam6
U+994F.0	fMandarin	dan4
U+9950.0	fUTF8	饐
U+9950.0	fCNS11643	2-6A62
U+9950.0	fBig5	F554
U+9950.0	fGB	3-7B24
U+9950.0	fR/S	 184.12
U+9950.0	fTotalStrokes	 21
U+9950.0	fCangjie	OIGBT
U+9950.0	fFourCorner	84718
U+9950.0	fHakka	 rit7; jit7; i5; ji5
U+9950.0	fCantonese	ji3; jik1 (also)
U+9950.0	fMandarin	yi4; ye4
U+9950.0	fEnglish	spoiled; rotten; sour
U+9951.0	fUTF8	饑
U+9951.0	fCNS11643	1-7A53
U+9951.0	fBig5	C4C8
U+9951.0	fGB	1-3C22
U+9951.0	fR/S	 184.12
U+9951.0	fTotalStrokes	 21
U+9951.0	fCangjie	OIVII
U+9951.0	fFourCorner	82753
U+9951.0	fHakka	  gi1
U+9951.0	fCantonese	gei1
U+9951.0	fMandarin	ji1; qi2
U+9951.0	fEnglish	[1] year of famine [2] hunger; hungry
U+9952.0	fUTF8	饒
U+9952.0	fCNS11643	1-7A52
U+9952.0	fBig5	C4C7
U+9952.0	fGB	1-4844
U+9952.0	fR/S	 184.12
U+9952.0	fTotalStrokes	 21
U+9952.0	fCangjie	OIGGU
U+9952.0	fFourCorner	84711
U+9952.0	fHakka	 ngiau2; niau2; gniau3; ngieu2
U+9952.0	fCantonese	jiu4
U+9952.0	fMandarin	rao2; rao4
U+9952.0	fEnglish	[1] bountiful; abundant; plentiful; full of; fertile [2] give something extra as a gift; let somebody have something into bargain [3] forgive; spare; have mercy; let somebody off [4] lenient; liberal [5] even though; in spite of the fact that; whatever
U+9953.0	fUTF8	饓
U+9953.0	fCNS11643	2-6A67
U+9953.0	fBig5	F559
U+9953.0	fGB	E-4567
U+9953.0	fR/S	 184.12
U+9953.0	fTotalStrokes	 21
U+9953.0	fCangjie	OIFBD
U+9953.0	fFourCorner	89794
U+9953.0	fHakka	 cang1; cang6; cang5
U+9953.0	fCantonese	caang1
U+9953.0	fMandarin	cheng1
U+9954.0	fUTF8	饔
U+9954.0	fCNS11643	2-6E45
U+9954.0	fBig5	F776
U+9954.0	fGB	0-7753
U+9954.0	fR/S	 184.13
U+9954.0	fTotalStrokes	 22
U+9954.0	fCangjie	YVGV
U+9954.0	fFourCorner	00732
U+9954.0	fHakka	 jung1; rung1
U+9954.0	fCantonese	jung1
U+9954.0	fMandarin	yong1; yong4
U+9954.0	fEnglish	[1] breakfast [2] cooked food [3] slaughtered animals
U+9955.0	fUTF8	饕
U+9955.0	fCNS11643	1-7C25
U+9955.0	fBig5	C5B9
U+9955.0	fGB	0-7752
U+9955.0	fR/S	 184.13
U+9955.0	fTotalStrokes	 22
U+9955.0	fCangjie	RUOIV
U+9955.0	fFourCorner	61732
U+9955.0	fHakka	 tau1; to1; hau1; hau2
U+9955.0	fCantonese	tou1
U+9955.0	fMandarin	tao1
U+9955.0	fEnglish	[1] the name of a legendary ferocious animal [2] fierce person [3] greedy and gluttonous person
U+9956.0	fUTF8	饖
U+9956.0	fCNS11643	2-6C65
U+9956.0	fBig5	F677
U+9956.0	fGB	5-7855
U+9956.0	fR/S	 184.13
U+9956.0	fTotalStrokes	 22
U+9956.0	fCangjie	OIYMH
U+9956.0	fFourCorner	81753
U+9956.0	fCantonese	wai3
U+9956.0	fMandarin	hui4; yi4
U+9957.0	fUTF8	饗
U+9957.0	fCNS11643	1-7B43
U+9957.0	fBig5	C557
U+9957.0	fGB	1-774F
U+9957.0	fR/S	 184.13
U+9957.0	fTotalStrokes	 22
U+9957.0	fCangjie	VLOIV
U+9957.0	fFourCorner	27732
U+9957.0	fHakka	  hiong3
U+9957.0	fCantonese	hoeng2
U+9957.0	fMandarin	xiang3; xiang1
U+9957.0	fEnglish	[1] dine and wine guests; give a big banquet; banquet or party [2] sacrificial ceremony
U+9958.0	fUTF8	饘
U+9958.0	fCNS11643	2-6C64
U+9958.0	fBig5	F676
U+9958.0	fGB	3-7B29
U+9958.0	fR/S	 184.13
U+9958.0	fTotalStrokes	 22
U+9958.0	fCangjie	OIYWM
U+9958.0	fFourCorner	80717
U+9958.0	fHakka	 zhan1; zan1; zen1; zhen1
U+9958.0	fCantonese	zin1
U+9958.0	fMandarin	zhan1; ji1
U+9958.0	fEnglish	thick congee or porridge
U+9959.0	fUTF8	饙
U+9959.0	fCNS11643	2-6A64
U+9959.0	fBig5	F556
U+9959.0	fGB	5-7865
U+9959.0	fR/S	 184.13
U+9959.0	fTotalStrokes	 22
U+9959.0	fCangjie	OIJTC
U+9959.0	fFourCorner	84786
U+9959.0	fHakka	 fun1
U+9959.0	fCantonese	fan1; pan3
U+9959.0	fMandarin	fen1
U+9959.0	fEnglish	half boiled rice
U+995A.0	fUTF8	饚
U+995A.0	fCNS11643	3-5F78
U+995A.0	fGB	E-4568
U+995A.0	fR/S	 184.14
U+995A.0	fTotalStrokes	 22; 23
U+995A.0	fCangjie	OITGT
U+995A.0	fFourCorner	84710
U+995A.0	fCantonese	hoi3
U+995A.0	fMandarin	hai4
U+995B.0	fUTF8	饛
U+995B.0	fCNS11643	2-6E46
U+995B.0	fBig5	F777
U+995B.0	fGB	3-7B27
U+995B.0	fR/S	 184.14
U+995B.0	fTotalStrokes	 23
U+995B.0	fCangjie	OITBO
U+995B.0	fFourCorner	84732
U+995B.0	fHakka	  mung2
U+995B.0	fCantonese	mung4
U+995B.0	fMandarin	meng2
U+995B.0	fEnglish	dish filled with food
U+995C.0	fUTF8	饜
U+995C.0	fCNS11643	1-7C50
U+995C.0	fBig5	C5E4
U+995C.0	fGB	1-7750
U+995C.0	fR/S	 184.14
U+995C.0	fTotalStrokes	 23
U+995C.0	fCangjie	MKOIV
U+995C.0	fFourCorner	71732
U+995C.0	fHakka	 jam5; {ngien5}; ram5; jam6; jam3
U+995C.0	fCantonese	jim3; jim1
U+995C.0	fMandarin	yan4; yan2
U+995C.0	fEnglish	[1] full-stomached; satiated; ssurfeited; sufficient [2] partake plentifully of
U+995D.0	fUTF8	饝
U+995D.0	fCNS11643	3-612A
U+995D.0	fGB	E-4569
U+995D.0	fR/S	  184.16
U+995D.0	fTotalStrokes	 24; 25
U+995D.0	fCangjie	OIIDR
U+995D.0	fFourCorner	80760
U+995D.0	fHakka	 mo2
U+995D.0	fCantonese	mo4
U+995D.0	fMandarin	mo2
U+995D.0	fEnglish	steamed dumplings
U+995E.0	fUTF8	饞
U+995E.0	fCNS11643	1-7D2E
U+995E.0	fBig5	C661
U+995E.0	fGB	1-3276
U+995E.0	fR/S	 184.17
U+995E.0	fTotalStrokes	 26
U+995E.0	fCangjie	OINRI
U+995E.0	fFourCorner	87713
U+995E.0	fHakka	 cam2
U+995E.0	fCantonese	caam4
U+995E.0	fMandarin	chan2
U+995E.0	fEnglish	gluttonous; greedy; piggish
U+995F.0	fUTF8	饟
U+995F.0	fCNS11643	2-7148
U+995F.0	fBig5	F959
U+995F.0	fGB	E-456A
U+995F.0	fR/S	 184.17
U+995F.0	fTotalStrokes	 26
U+995F.0	fCangjie	OIYRV
U+995F.0	fFourCorner	80732
U+995F.0	fHakka	 siong5;  hiong3
U+995F.0	fCantonese	hoeng2
U+995F.0	fMandarin	xiang3; xiang2; rang4; shang1
U+995F.0	fEnglish	 rations and pay for soldiers dried rice
U+9960.0	fUTF8	饠
U+9960.0	fCNS11643	4-6E24
U+9960.0	fGB	3-7A70
U+9960.0	fR/S	 184.19
U+9960.0	fTotalStrokes	 27; 28
U+9960.0	fCangjie	OIWLG
U+9960.0	fFourCorner	86714
U+9960.0	fHakka	  lo2
U+9960.0	fCantonese	lo4
U+9960.0	fMandarin	luo2
U+9960.0	fEnglish	rice cake
U+9961.0	fUTF8	饡
U+9961.0	fCNS11643	2-7221
U+9961.0	fBig5	F9B1
U+9961.0	fGB	3-7B2B
U+9961.0	fR/S	 184.19
U+9961.0	fTotalStrokes	 28
U+9961.0	fCangjie	OIHUC
U+9961.0	fFourCorner	84786
U+9961.0	fCantonese	zaan3
U+9961.0	fMandarin	zuan4; zan4
U+9962.0	fUTF8	饢
U+9962.0	fGB	1-624E
U+9962.0	fR/S	 184.22; 184.21
U+9962.0	fTotalStrokes	 31
U+9962.0	fCangjie	OIJBV
U+9962.0	fCantonese	nong4; long4 (also); nong5; long5 (also)
U+9962.0	fMandarin	nang3; nang2
U+9962.0	fEnglish	bread (Persian "naan")
U+9963.0	fUTF8	饣
U+9963.0	fGB	0-623B
U+9963.0	fR/S	 184.0
U+9963.0	fTotalStrokes	 3
U+9963.0	fCangjie	NV
U+9963.0	fMandarin	shi2; si4
U+9963.0	fEnglish	simplified form of &#x98E0;
U+9964.0	fUTF8	饤
U+9964.0	fGB	8-2D43
U+9964.0	fR/S	 184.2
U+9964.0	fTotalStrokes	 5
U+9964.0	fCangjie	NVMN
U+9964.0	fCantonese	ding3
U+9964.0	fMandarin	ding4
U+9964.0	fEnglish	display food for show only; sacrifice
U+9965.0	fUTF8	饥
U+9965.0	fGB	0-3C22
U+9965.0	fR/S	 184.2
U+9965.0	fTotalStrokes	 5
U+9965.0	fCangjie	NVHN
U+9965.0	fHakka	 gi1
U+9965.0	fCantonese	gei1
U+9965.0	fMandarin	ji1
U+9965.0	fEnglish	simplified form of &#x9951;
U+9966.0	fUTF8	饦
U+9966.0	fGB	8-2D44
U+9966.0	fR/S	 184.3
U+9966.0	fTotalStrokes	 6
U+9966.0	fCangjie	NVHP
U+9966.0	fCantonese	tok3
U+9966.0	fMandarin	tuo1
U+9966.0	fEnglish	rice cake
U+9967.0	fUTF8	饧
U+9967.0	fGB	0-623C
U+9967.0	fR/S	 184.3
U+9967.0	fTotalStrokes	 6
U+9967.0	fCangjie	NVNSH
U+9967.0	fHakka	 tong2
U+9967.0	fCantonese	cing4; tong4
U+9967.0	fMandarin	xing2; tang2; qing2
U+9967.0	fEnglish	simplified form of &#x9933;
U+9968.0	fUTF8	饨
U+9968.0	fGB	0-623D
U+9968.0	fR/S	 184.4
U+9968.0	fTotalStrokes	 7
U+9968.0	fCangjie	NVPU
U+9968.0	fHakka	 tun2
U+9968.0	fCantonese	tan1; tan4 (also)
U+9968.0	fMandarin	tun2; dun4; tun5
U+9968.0	fEnglish	simplified form of &#x98E9;
U+9969.0	fUTF8	饩
U+9969.0	fGB	0-623E
U+9969.0	fR/S	 184.4
U+9969.0	fTotalStrokes	 7
U+9969.0	fCangjie	NVOMN
U+9969.0	fHakka	 hi5
U+9969.0	fCantonese	hei3
U+9969.0	fMandarin	xi4
U+9969.0	fEnglish	simplified form of &#x993C;
U+996A.0	fUTF8	饪
U+996A.0	fGB	0-623F
U+996A.0	fR/S	 184.4
U+996A.0	fTotalStrokes	 7
U+996A.0	fCangjie	NVHG
U+996A.0	fHakka	 ngim2; ngim5
U+996A.0	fCantonese	jam6; jam5 (also)
U+996A.0	fMandarin	ren4; ren3
U+996A.0	fEnglish	simplified form of &#x98EA;
U+996B.0	fUTF8	饫
U+996B.0	fGB	0-6240
U+996B.0	fR/S	 184.4
U+996B.0	fTotalStrokes	 7
U+996B.0	fCangjie	NVHK
U+996B.0	fHakka	 ji5
U+996B.0	fCantonese	jyu3
U+996B.0	fMandarin	yu4
U+996B.0	fEnglish	simplified form of &#x98EB;
U+996C.0	fUTF8	饬
U+996C.0	fGB	0-6241
U+996C.0	fR/S	 184.4
U+996C.0	fTotalStrokes	 7
U+996C.0	fCangjie	NVOKS
U+996C.0	fHakka	 shit7; hit7
U+996C.0	fCantonese	cik1; sik1
U+996C.0	fMandarin	chi4
U+996C.0	fEnglish	simplified form of &#x98ED;
U+996D.0	fUTF8	饭
U+996D.0	fGB	0-3739
U+996D.0	fR/S	 184.4
U+996D.0	fTotalStrokes	 7
U+996D.0	fCangjie	NVHE
U+996D.0	fHakka	 fan3
U+996D.0	fCantonese	faan6; faan2 (also)
U+996D.0	fMandarin	fan4; fan3
U+996D.0	fEnglish	simplified form of &#x98EF;
U+996E.0	fUTF8	饮
U+996E.0	fGB	0-527B
U+996E.0	fR/S	 184.4
U+996E.0	fTotalStrokes	 7
U+996E.0	fCangjie	NVNO
U+996E.0	fHakka	 jim3
U+996E.0	fCantonese	jam2; jam3
U+996E.0	fMandarin	yin3; yin4
U+996E.0	fEnglish	simplified form of &#x98F2;
U+996F.0	fUTF8	饯
U+996F.0	fGB	0-3D24
U+996F.0	fR/S	 184.5
U+996F.0	fTotalStrokes	 8
U+996F.0	fCangjie	NVIJ
U+996F.0	fHakka	 zien5
U+996F.0	fCantonese	zin3
U+996F.0	fMandarin	jian4
U+996F.0	fEnglish	simplified form of &#x991E;
U+9970.0	fUTF8	饰
U+9970.0	fGB	0-4A4E
U+9970.0	fR/S	 184.5
U+9970.0	fTotalStrokes	 8
U+9970.0	fCangjie	NVOLB
U+9970.0	fHakka	 shit7; hit7
U+9970.0	fCantonese	sik1; cik1
U+9970.0	fMandarin	shi4; chi4
U+9970.0	fEnglish	simplified form of &#x98FE;
U+9971.0	fUTF8	饱
U+9971.0	fGB	0-3125
U+9971.0	fR/S	 184.5
U+9971.0	fTotalStrokes	 8
U+9971.0	fCangjie	NVPRU
U+9971.0	fHakka	 bau3
U+9971.0	fCantonese	baau2
U+9971.0	fMandarin	bao3
U+9971.0	fEnglish	simplified form of &#x98FD;
U+9972.0	fUTF8	饲
U+9972.0	fGB	0-4B47
U+9972.0	fR/S	 184.5
U+9972.0	fTotalStrokes	 8
U+9972.0	fCangjie	NVSMR
U+9972.0	fHakka	 sii5
U+9972.0	fCantonese	zi6
U+9972.0	fMandarin	si4
U+9972.0	fEnglish	simplified form of &#x98FC;
U+9973.0	fUTF8	饳
U+9973.0	fGB	7-2148
U+9973.0	fR/S	 184.5
U+9973.0	fTotalStrokes	 8
U+9973.0	fCangjie	NVUU
U+9973.0	fCantonese	deot1
U+9973.0	fMandarin	duo4
U+9974.0	fUTF8	饴
U+9974.0	fGB	0-6242
U+9974.0	fR/S	 184.5
U+9974.0	fTotalStrokes	 8
U+9974.0	fCangjie	NVIR
U+9974.0	fHakka	 ji2
U+9974.0	fCantonese	ji4; zi6
U+9974.0	fMandarin	yi2; si4
U+9974.0	fEnglish	simplified form of &#x98F4;
U+9975.0	fUTF8	饵
U+9975.0	fGB	0-367C
U+9975.0	fR/S	 184.6
U+9975.0	fTotalStrokes	 9
U+9975.0	fCangjie	NVSJ
U+9975.0	fHakka	 ngi3
U+9975.0	fCantonese	nei6; lei6 (also)
U+9975.0	fMandarin	er3; er4
U+9975.0	fEnglish	simplified form of &#x990C;
U+9976.0	fUTF8	饶
U+9976.0	fGB	0-4844
U+9976.0	fR/S	 184.6
U+9976.0	fTotalStrokes	 9
U+9976.0	fCangjie	NVJPU
U+9976.0	fHakka	 ngiau2
U+9976.0	fCantonese	jiu4
U+9976.0	fMandarin	rao2; rao4
U+9976.0	fEnglish	simplified form of &#x9952;
U+9977.0	fUTF8	饷
U+9977.0	fGB	0-6243
U+9977.0	fR/S	 184.6
U+9977.0	fTotalStrokes	 9
U+9977.0	fCangjie	NVHBR
U+9977.0	fHakka	 hiong3; hiong1
U+9977.0	fCantonese	hoeng2
U+9977.0	fMandarin	xiang3; shang1; shang3; shang4
U+9977.0	fEnglish	simplified form of &#x9909;
U+9978.0	fUTF8	饸
U+9978.0	fGB	8-2D45
U+9978.0	fR/S	 184.6
U+9978.0	fTotalStrokes	 9
U+9978.0	fCangjie	NVOMR
U+9978.0	fCantonese	gaap3; hap6 (also)
U+9978.0	fMandarin	jia2; he2
U+9979.0	fUTF8	饹
U+9979.0	fGB	8-2D46
U+9979.0	fR/S	 184.6
U+9979.0	fTotalStrokes	 9
U+9979.0	fCangjie	NVHER
U+9979.0	fCantonese	lok3
U+9979.0	fMandarin	le5
U+997A.0	fUTF8	饺
U+997A.0	fGB	0-3D48
U+997A.0	fR/S	 184.6
U+997A.0	fTotalStrokes	 9
U+997A.0	fCangjie	NVYCK
U+997A.0	fHakka	 gau3; giau2
U+997A.0	fCantonese	gaau2
U+997A.0	fMandarin	jiao3; jiao4
U+997A.0	fEnglish	simplified form of &#x9903;
U+997B.0	fUTF8	饻
U+997B.0	fGB	8-2D47
U+997B.0	fR/S	 184.6
U+997B.0	fTotalStrokes	 9
U+997B.0	fCangjie	NVYHV
U+997B.0	fCantonese	si1
U+997B.0	fMandarin	xi1; yi1
U+997C.0	fUTF8	饼
U+997C.0	fGB	0-317D
U+997C.0	fR/S	 184.6
U+997C.0	fTotalStrokes	 9
U+997C.0	fCangjie	NVTT
U+997C.0	fHakka	 biang3
U+997C.0	fCantonese	bing2 (also); beng2
U+997C.0	fMandarin	bing3
U+997C.0	fEnglish	simplified form of &#x9905;
U+997C.1	fUTF8	饼
U+997C.1	fGB	0-317D
U+997C.1	fR/S	 184.6
U+997C.1	fTotalStrokes	 9
U+997C.1	fCangjie	NVTT
U+997C.1	fMandarin	bing5
U+997D.0	fUTF8	饽
U+997D.0	fGB	0-6244
U+997D.0	fR/S	 184.7
U+997D.0	fTotalStrokes	 10
U+997D.0	fCangjie	NVJBD
U+997D.0	fHakka	 put8
U+997D.0	fCantonese	but6
U+997D.0	fMandarin	bo1; bo2
U+997D.0	fEnglish	simplified form of &#x9911;
U+997E.0	fUTF8	饾
U+997E.0	fGB	8-2D48
U+997E.0	fR/S	 184.7
U+997E.0	fTotalStrokes	 10
U+997E.0	fCangjie	NVMRT
U+997E.0	fCantonese	dau6
U+997E.0	fMandarin	dou4
U+997E.0	fEnglish	food item set out for show only
U+997F.0	fUTF8	饿
U+997F.0	fGB	0-3676
U+997F.0	fR/S	 184.7
U+997F.0	fTotalStrokes	 10
U+997F.0	fCangjie	NVHQI
U+997F.0	fHakka	 ngo5
U+997F.0	fCantonese	ngo6; o6
U+997F.0	fMandarin	e4
U+997F.0	fEnglish	simplified form of &#x9913;
U+9980.0	fUTF8	馀
U+9980.0	fGB	0-6245
U+9980.0	fR/S	 184.7
U+9980.0	fTotalStrokes	 10
U+9980.0	fCangjie	NVOMD
U+9980.0	fHakka	 ji2
U+9980.0	fCantonese	jyu4; ceoi4
U+9980.0	fMandarin	yu2; tu2; xu3; yu4
U+9980.0	fEnglish	simplified form of &#x4F59;
U+9981.0	fUTF8	馁
U+9981.0	fGB	0-4459
U+9981.0	fR/S	 184.7
U+9981.0	fTotalStrokes	 10
U+9981.0	fCangjie	NVBV
U+9981.0	fHakka	 nui3; {nui1}
U+9981.0	fCantonese	neoi5; leoi5 (also); noi5 (also); loi5 (also)
U+9981.0	fMandarin	nei3; nei2
U+9981.0	fEnglish	simplified form of &#x9912;
U+9982.0	fUTF8	馂
U+9982.0	fGB	8-2D49
U+9982.0	fR/S	 184.7
U+9982.0	fTotalStrokes	 10
U+9982.0	fCangjie	NVICE
U+9982.0	fCantonese	zeon3
U+9982.0	fMandarin	jun4
U+9982.0	fEnglish	leftovers
U+9983.0	fUTF8	馃
U+9983.0	fGB	8-2D4A
U+9983.0	fR/S	 184.8
U+9983.0	fTotalStrokes	 11
U+9983.0	fCangjie	NVWD
U+9983.0	fCantonese	gwo2; go2 (also)
U+9983.0	fMandarin	guo3
U+9983.0	fEnglish	cakes, biscuits, pastry
U+9984.0	fUTF8	馄
U+9984.0	fGB	0-6246
U+9984.0	fR/S	 184.8
U+9984.0	fTotalStrokes	 11
U+9984.0	fCangjie	NVAPP
U+9984.0	fHakka	 fun2
U+9984.0	fCantonese	wan4
U+9984.0	fMandarin	hun2; kun1
U+9984.0	fEnglish	simplified form of &#x991B;
U+9985.0	fUTF8	馅
U+9985.0	fGB	0-4F5A
U+9985.0	fR/S	 184.8
U+9985.0	fTotalStrokes	 11
U+9985.0	fCangjie	NVNHX
U+9985.0	fHakka	 ham5
U+9985.0	fCantonese	haam2; haam6 (also); haam5
U+9985.0	fMandarin	xian4; kan4
U+9985.0	fEnglish	simplified form of &#x9921;
U+9986.0	fUTF8	馆
U+9986.0	fGB	0-395D
U+9986.0	fR/S	 184.8
U+9986.0	fTotalStrokes	 11
U+9986.0	fCangjie	NVJRR
U+9986.0	fHakka	 gwon3; gon3; kwan3
U+9986.0	fCantonese	gun2; gwun2 (also)
U+9986.0	fMandarin	guan3; guan4
U+9986.0	fEnglish	simplified form of &#x9928;
U+9987.0	fUTF8	馇
U+9987.0	fGB	0-6247
U+9987.0	fR/S	 184.9
U+9987.0	fTotalStrokes	 12
U+9987.0	fCangjie	NVDAM
U+9987.0	fCantonese	caa4; zaa1
U+9987.0	fMandarin	cha1
U+9987.0	fEnglish	simplified form of &#x9937;
U+9988.0	fUTF8	馈
U+9988.0	fGB	0-4021
U+9988.0	fR/S	 184.9
U+9988.0	fTotalStrokes	 12
U+9988.0	fCangjie	NVLMO
U+9988.0	fHakka	 k'wui5
U+9988.0	fCantonese	gwai6; gwai3 (also)
U+9988.0	fMandarin	kui4; gui4; tui1
U+9988.0	fEnglish	simplified form of &#x994B;
U+9989.0	fUTF8	馉
U+9989.0	fGB	7-2149
U+9989.0	fR/S	 184.10
U+9989.0	fTotalStrokes	 13
U+9989.0	fCangjie	NVBBB
U+9989.0	fCantonese	gwat1
U+9989.0	fMandarin	gu3
U+998A.0	fUTF8	馊
U+998A.0	fGB	0-6248
U+998A.0	fR/S	 184.9
U+998A.0	fTotalStrokes	 12
U+998A.0	fCangjie	NVHXE
U+998A.0	fHakka	 seu1
U+998A.0	fCantonese	sau1; suk1 (also)
U+998A.0	fMandarin	sou1
U+998A.0	fEnglish	simplified form of &#x993F;
U+998B.0	fUTF8	馋
U+998B.0	fGB	0-3276
U+998B.0	fR/S	 184.9
U+998B.0	fTotalStrokes	 12
U+998B.0	fCangjie	NVNUY
U+998B.0	fHakka	 cam2
U+998B.0	fCantonese	caam4
U+998B.0	fMandarin	chan2
U+998B.0	fEnglish	simplified form of &#x995E;
U+998C.0	fUTF8	馌
U+998C.0	fGB	8-2C3E
U+998C.0	fR/S	 184.10
U+998C.0	fTotalStrokes	 13
U+998C.0	fCangjie	NVGIT
U+998C.0	fCantonese	jip3
U+998C.0	fMandarin	ye4
U+998C.0	fEnglish	carry meal to workers in field
U+998D.0	fUTF8	馍
U+998D.0	fGB	0-6249
U+998D.0	fR/S	 184.10
U+998D.0	fTotalStrokes	 13
U+998D.0	fCangjie	NVTAK
U+998D.0	fCantonese	mo4
U+998D.0	fMandarin	mo2; mo4
U+998D.0	fEnglish	simplified form of &#x9943;
U+998E.0	fUTF8	馎
U+998E.0	fGB	8-2D4B
U+998E.0	fR/S	 184.10
U+998E.0	fTotalStrokes	 13
U+998E.0	fCangjie	NVIBI
U+998E.0	fCantonese	bok3
U+998E.0	fMandarin	bo2
U+998E.0	fEnglish	rice cake
U+998F.0	fUTF8	馏
U+998F.0	fGB	0-4173
U+998F.0	fR/S	 184.10
U+998F.0	fTotalStrokes	 13
U+998F.0	fCangjie	NVHHW
U+998F.0	fHakka	 liu5
U+998F.0	fCantonese	lau6; lau4 (also)
U+998F.0	fMandarin	liu4; liu2
U+998F.0	fEnglish	simplified form of &#x993E;
U+9990.0	fUTF8	馐
U+9990.0	fGB	0-624A
U+9990.0	fR/S	 184.11; 184.10
U+9990.0	fTotalStrokes	 13
U+9990.0	fCangjie	NVTQG
U+9990.0	fHakka	 siu1
U+9990.0	fCantonese	sau1
U+9990.0	fMandarin	xiu1
U+9990.0	fEnglish	simplified form of &#x9948;
U+9991.0	fUTF8	馑
U+9991.0	fGB	0-624B
U+9991.0	fR/S	 184.11
U+9991.0	fTotalStrokes	 14
U+9991.0	fCangjie	NVTLM
U+9991.0	fHakka	 giun3
U+9991.0	fCantonese	gan2
U+9991.0	fMandarin	jin3; jin4
U+9991.0	fEnglish	simplified form of &#x9949;
U+9992.0	fUTF8	馒
U+9992.0	fGB	0-4278
U+9992.0	fR/S	 184.11
U+9992.0	fTotalStrokes	 14
U+9992.0	fCangjie	NVAWE
U+9992.0	fHakka	 man2; man3
U+9992.0	fCantonese	maan4 (also); maan6
U+9992.0	fMandarin	man2
U+9992.0	fEnglish	simplified form of &#x9945;
U+9993.0	fUTF8	馓
U+9993.0	fGB	0-624C
U+9993.0	fR/S	 184.12
U+9993.0	fTotalStrokes	 15
U+9993.0	fCangjie	NVTBK
U+9993.0	fHakka	 san3
U+9993.0	fCantonese	saan2
U+9993.0	fMandarin	san3
U+9993.0	fEnglish	simplified form of &#x994A;
U+9994.0	fUTF8	馔
U+9994.0	fGB	0-624D
U+9994.0	fR/S	 184.12
U+9994.0	fTotalStrokes	 15
U+9994.0	fCangjie	NVRUC
U+9994.0	fHakka	 con5; sion3; con3
U+9994.0	fCantonese	zaan6; syun2
U+9994.0	fMandarin	zhuan4; xuan3; fan4
U+9994.0	fEnglish	simplified form of &#x994C;
U+9995.0	fUTF8	馕
U+9995.0	fGB	0-624E
U+9995.0	fR/S	 184.22
U+9995.0	fTotalStrokes	 25
U+9995.0	fCangjie	NVJBV
U+9995.0	fCantonese	nong4; long4 (also); nong5; long5 (also)
U+9995.0	fMandarin	nang3; nang2
U+9995.0	fEnglish	simplified form of &#x9962;
U+9996.0	fUTF8	首
U+9996.0	fCNS11643	1-537B
U+9996.0	fBig5	ADBA
U+9996.0	fGB	0-4A57
U+9996.0	fR/S	 185.0
U+9996.0	fTotalStrokes	 9
U+9996.0	fCangjie	THBU
U+9996.0	fFourCorner	80601
U+9996.0	fHakka	  shiu3; su3; siu3; sii3; shu3
U+9996.0	fCantonese	sau2; sau3
U+9996.0	fMandarin	shou3; shou4
U+9996.0	fEnglish	[1] the head [2] king; emperor; leader; chief [3] the first; beginning [4] a numerical classifier for poems, songs, etc. [5] Kangxi radical 185
U+9997.0	fUTF8	馗
U+9997.0	fCNS11643	2-3A34
U+9997.0	fBig5	D850
U+9997.0	fGB	0-5838
U+9997.0	fR/S	 185.2
U+9997.0	fTotalStrokes	 11
U+9997.0	fCangjie	KNTHU
U+9997.0	fFourCorner	48016
U+9997.0	fHakka	 kui2; k'wui2
U+9997.0	fCantonese	kwai4
U+9997.0	fMandarin	kui2; qiu2
U+9997.0	fEnglish	[1] path; road [2] correct form of &#xthe name of a Taoist immortal famous for catching evil spirits
U+9998.0	fUTF8	馘
U+9998.0	fCNS11643	2-6061
U+9998.0	fBig5	EF55
U+9998.0	fGB	0-5965
U+9998.0	fR/S	 185.8
U+9998.0	fTotalStrokes	 17
U+9998.0	fCangjie	TUIRM
U+9998.0	fFourCorner	83650
U+9998.0	fHakka	 kwet7; wet7; vet7; fet7
U+9998.0	fCantonese	gwik1; gwok3; gok3 (also)
U+9998.0	fMandarin	guo2; xu4; guo1
U+9998.0	fEnglish	 count the number of enemy troops one killed by the number of left ears cut from the bodies
U+9999.0	fUTF8	香
U+9999.0	fCNS11643	1-537C
U+9999.0	fBig5	ADBB
U+9999.0	fGB	0-4F63
U+9999.0	fR/S	 186.0
U+9999.0	fTotalStrokes	 9
U+9999.0	fCangjie	HDA
U+9999.0	fFourCorner	20609
U+9999.0	fHakka	  hiong1
U+9999.0	fCantonese	hoeng1
U+9999.0	fMandarin	xiang1
U+9999.0	fEnglish	[1] fragrant; sweet smelling; aromatic; balmy [2] tasty; delicious [3] fair; beautiful [4] incense; spice; balm [5] Kangxi radical 186
U+999A.0	fUTF8	馚
U+999A.0	fCNS11643	3-455F
U+999A.0	fGB	5-782E
U+999A.0	fR/S	  186.4
U+999A.0	fTotalStrokes	 13
U+999A.0	fCangjie	HACSH
U+999A.0	fFourCorner	28620
U+999A.0	fHakka	  fun2
U+999A.0	fCantonese	fan4
U+999A.0	fMandarin	fen2
U+999A.0	fEnglish	aromatic; perfumed
U+999B.0	fUTF8	馛
U+999B.0	fCNS11643	5-4B4E
U+999B.0	fGB	3-7A29
U+999B.0	fR/S	 186.5
U+999B.0	fTotalStrokes	 14
U+999B.0	fCangjie	HAIKK
U+999B.0	fFourCorner	23640
U+999B.0	fHakka	 pat8
U+999B.0	fCantonese	bat6
U+999B.0	fMandarin	ba2
U+999B.0	fEnglish	fragrance; fragrant
U+999C.0	fUTF8	馜
U+999C.0	fCNS11643	2-4F3A
U+999C.0	fBig5	E4D2
U+999C.0	fGB	3-7A2B
U+999C.0	fR/S	 186.5
U+999C.0	fTotalStrokes	 14
U+999C.0	fCangjie	HASP
U+999C.0	fFourCorner	27611
U+999C.0	fHakka	 ni3
U+999C.0	fCantonese	nei5; lei5 (also)
U+999C.0	fMandarin	ni3
U+999C.0	fEnglish	fragrant
U+999D.0	fUTF8	馝
U+999D.0	fCNS11643	2-4F39
U+999D.0	fBig5	E4D1
U+999D.0	fGB	3-7A2A
U+999D.0	fR/S	 186.5
U+999D.0	fTotalStrokes	 14
U+999D.0	fCangjie	HAPH
U+999D.0	fFourCorner	23600
U+999D.0	fCantonese	bit6; bat6 (also); bit1
U+999D.0	fMandarin	bi4; bie2
U+999D.0	fEnglish	fragrance
U+999E.0	fUTF8	馞
U+999E.0	fCNS11643	2-5B6C
U+999E.0	fBig5	EC60
U+999E.0	fGB	3-7A2C
U+999E.0	fR/S	 186.7
U+999E.0	fTotalStrokes	 16
U+999E.0	fCangjie	HAJBD
U+999E.0	fFourCorner	24647
U+999E.0	fHakka	 put7; put8
U+999E.0	fCantonese	but6
U+999E.0	fMandarin	bo2
U+999F.0	fUTF8	馟
U+999F.0	fCNS11643	5-5A26
U+999F.0	fGB	E-456B
U+999F.0	fR/S	  186.7
U+999F.0	fTotalStrokes	 16
U+999F.0	fCangjie	HABND
U+999F.0	fFourCorner	22640
U+999F.0	fCantonese	tou4
U+999F.0	fMandarin	tu2
U+999F.0	fEnglish	fragrant
U+99A0.0	fUTF8	馠
U+99A0.0	fCNS11643	3-5368
U+99A0.0	fGB	3-7A2E
U+99A0.0	fR/S	  186.7
U+99A0.0	fTotalStrokes	 16
U+99A0.0	fCangjie	HAOIR
U+99A0.0	fFourCorner	28660
U+99A0.0	fCantonese	ham4
U+99A0.0	fMandarin	han1
U+99A1.0	fUTF8	馡
U+99A1.0	fCNS11643	2-6063
U+99A1.0	fBig5	EF57
U+99A1.0	fGB	5-7830
U+99A1.0	fR/S	 186.8
U+99A1.0	fTotalStrokes	 17
U+99A1.0	fCangjie	HALMY
U+99A1.0	fFourCorner	21611
U+99A1.0	fHakka	 fui1; fi1
U+99A1.0	fCantonese	fei1
U+99A1.0	fMandarin	fei1; hui4
U+99A1.0	fEnglish	scent
U+99A2.0	fUTF8	馢
U+99A2.0	fCNS11643	3-5730
U+99A2.0	fGB	E-456C
U+99A2.0	fR/S	  186.8
U+99A2.0	fTotalStrokes	 17
U+99A2.0	fCangjie	HAII
U+99A2.0	fFourCorner	23650
U+99A2.0	fCantonese	zin1
U+99A2.0	fMandarin	jian1
U+99A3.0	fUTF8	馣
U+99A3.0	fCNS11643	2-6062
U+99A3.0	fBig5	EF56
U+99A3.0	fGB	5-782F
U+99A3.0	fR/S	 186.8
U+99A3.0	fTotalStrokes	 17
U+99A3.0	fCangjie	HAKLU
U+99A3.0	fFourCorner	24616
U+99A3.0	fCantonese	am1; ngam1 (also)
U+99A3.0	fMandarin	an1; an3; yan3
U+99A4.0	fUTF8	馤
U+99A4.0	fCNS11643	3-5967
U+99A4.0	fGB	3-7A30
U+99A4.0	fR/S	  186.9
U+99A4.0	fTotalStrokes	 18
U+99A4.0	fCangjie	HAAPV
U+99A4.0	fFourCorner	26620
U+99A4.0	fCantonese	oi3; ngoi3 (also)
U+99A4.0	fMandarin	ai3; ai4
U+99A5.0	fUTF8	馥
U+99A5.0	fCNS11643	1-7776
U+99A5.0	fBig5	C34C
U+99A5.0	fGB	0-7025
U+99A5.0	fR/S	 186.9
U+99A5.0	fTotalStrokes	 18
U+99A5.0	fCangjie	HAOAE
U+99A5.0	fFourCorner	28647
U+99A5.0	fHakka	  fuk8; fuk7; put7; buk7
U+99A5.0	fCantonese	fuk1; fuk6
U+99A5.0	fMandarin	fu4; bi4
U+99A5.0	fEnglish	fragrance; scent; aroma
U+99A6.0	fUTF8	馦
U+99A6.0	fCNS11643	2-677D
U+99A6.0	fBig5	F3B2
U+99A6.0	fGB	3-7A31
U+99A6.0	fR/S	 186.10
U+99A6.0	fTotalStrokes	 19
U+99A6.0	fCangjie	HATXC
U+99A6.0	fFourCorner	28637
U+99A6.0	fHakka	 hiam1
U+99A6.0	fCantonese	him1
U+99A6.0	fMandarin	xian1
U+99A7.0	fUTF8	馧
U+99A7.0	fCNS11643	2-677E
U+99A7.0	fBig5	F3B3
U+99A7.0	fGB	3-7A2F
U+99A7.0	fR/S	 186.10
U+99A7.0	fTotalStrokes	 19
U+99A7.0	fCangjie	HAWOT
U+99A7.0	fFourCorner	26617
U+99A7.0	fCantonese	wan1; wat1
U+99A7.0	fMandarin	wen1
U+99A8.0	fUTF8	馨
U+99A8.0	fCNS11643	1-7A54
U+99A8.0	fBig5	C4C9
U+99A8.0	fGB	0-5C30
U+99A8.0	fR/S	 186.11
U+99A8.0	fTotalStrokes	 20
U+99A8.0	fCangjie	GEHDA
U+99A8.0	fFourCorner	47609
U+99A8.0	fHakka	 hin1;  him1; kin1
U+99A8.0	fCantonese	hing1
U+99A8.0	fMandarin	xing1; xin1; xin4
U+99A8.0	fEnglish	fragrantce or aroma (especially that which comes from afar); distant fragrance
U+99A9.0	fUTF8	馩
U+99A9.0	fCNS11643	3-5F21
U+99A9.0	fGB	5-7831
U+99A9.0	fR/S	  186.12
U+99A9.0	fTotalStrokes	 21
U+99A9.0	fCangjie	HAJTC
U+99A9.0	fFourCorner	24680
U+99A9.0	fHakka	 fun2
U+99A9.0	fCantonese	fan4
U+99A9.0	fMandarin	fen2
U+99A9.0	fEnglish	aromatic; perfumed
U+99AA.0	fUTF8	馪
U+99AA.0	fCNS11643	3-6060
U+99AA.0	fGB	3-7A32
U+99AA.0	fR/S	 186.14
U+99AA.0	fTotalStrokes	 23
U+99AA.0	fCangjie	HAJMC
U+99AA.0	fFourCorner	23680
U+99AA.0	fCantonese	pan1
U+99AA.0	fMandarin	bin1
U+99AB.0	fUTF8	馫
U+99AB.0	fCNS11643	2-7222
U+99AB.0	fBig5	F9B2
U+99AB.0	fGB	E-456D
U+99AB.0	fR/S	 186.18
U+99AB.0	fTotalStrokes	 27
U+99AB.0	fCangjie	HAHAA
U+99AB.0	fFourCorner	10669
U+99AB.0	fCantonese	hing1
U+99AB.0	fMandarin	xing1
U+99AC.0	fUTF8	馬
U+99AC.0	fCNS11643	1-586B
U+99AC.0	fBig5	B0A8
U+99AC.0	fGB	1-426D
U+99AC.0	fR/S	 187.0
U+99AC.0	fTotalStrokes	 10
U+99AC.0	fCangjie	SQSF
U+99AC.0	fFourCorner	71327
U+99AC.0	fHakka	  ma1; ma3
U+99AC.0	fCantonese	maa5; maa1 (also)
U+99AC.0	fMandarin	ma3
U+99AC.0	fEnglish	[1] horse [2] a Chinese family name [3] Kangxi radical 187
U+99AD.0	fUTF8	馭
U+99AD.0	fCNS11643	1-6326
U+99AD.0	fBig5	B6BF
U+99AD.0	fGB	1-5426
U+99AD.0	fR/S	 187.2
U+99AD.0	fTotalStrokes	 12
U+99AD.0	fCangjie	SFE
U+99AD.0	fFourCorner	77340
U+99AD.0	fHakka	 ngi5; ngi6; ngi3
U+99AD.0	fCantonese	jyu6; to4; do6
U+99AD.0	fMandarin	yu4
U+99AD.0	fEnglish	[1] drive (carriage, chariot, etc.); ride [2] manage; control; rule; govern [3] driver
U+99AE.0	fUTF8	馮
U+99AE.0	fCNS11643	1-6325
U+99AE.0	fBig5	B6BE
U+99AE.0	fGB	1-376B
U+99AE.0	fR/S	 187.2
U+99AE.0	fTotalStrokes	 12
U+99AE.0	fCangjie	IMSQF
U+99AE.0	fFourCorner	31127
U+99AE.0	fHakka	 pung2;  fung2; pin1; pung3
U+99AE.0	fCantonese	fung4; pang4
U+99AE.0	fMandarin	feng2; peng1; bing4; fen4
U+99AE.0	fEnglish	[1] gallop [2] by dint of; on the strength of
U+99AE.1	fUTF8	馮
U+99AE.1	fCNS11643	1-6325
U+99AE.1	fBig5	B6BE
U+99AE.1	fGB	1-376B
U+99AE.1	fR/S	 187.2
U+99AE.1	fTotalStrokes	 12
U+99AE.1	fCangjie	IMSQF
U+99AE.1	fFourCorner	31127
U+99AE.1	fMandarin	ping2
U+99AE.1	fEnglish	[1] gallop [2] by dint of; on the strength of
U+99AF.0	fUTF8	馯
U+99AF.0	fCNS11643	2-486A
U+99AF.0	fBig5	E0E4
U+99AF.0	fGB	3-3D71
U+99AF.0	fR/S	 187.3
U+99AF.0	fTotalStrokes	 13
U+99AF.0	fCangjie	SFMJ
U+99AF.0	fFourCorner	71340
U+99AF.0	fHakka	 hon5; hon6
U+99AF.0	fCantonese	hon4; hon6; gon3
U+99AF.0	fMandarin	han4; han2
U+99AF.0	fEnglish	vicious horse
U+99B0.0	fUTF8	馰
U+99B0.0	fCNS11643	2-486C
U+99B0.0	fBig5	E0E6
U+99B0.0	fGB	5-412A
U+99B0.0	fR/S	 187.3
U+99B0.0	fTotalStrokes	 13
U+99B0.0	fCangjie	SFPI
U+99B0.0	fFourCorner	77320
U+99B0.0	fHakka	  dit7
U+99B0.0	fCantonese	dik1
U+99B0.0	fMandarin	di4
U+99B0.0	fEnglish	a bay horse with a white spot on its forehead
U+99B1.0	fUTF8	馱
U+99B1.0	fCNS11643	1-676A
U+99B1.0	fBig5	B9A4
U+99B1.0	fGB	1-4D54
U+99B1.0	fR/S	 187.2
U+99B1.0	fTotalStrokes	 12
U+99B1.0	fCangjie	SFK
U+99B1.0	fFourCorner	74330
U+99B1.0	fHakka	  to2; tok7
U+99B1.0	fCantonese	to4; do6; to6
U+99B1.0	fMandarin	tuo2; dai4
U+99B1.0	fEnglish	 carry (a load) on the back
U+99B1.1	fUTF8	馱
U+99B1.1	fCNS11643	1-676A
U+99B1.1	fBig5	B9A4
U+99B1.1	fGB	1-4D54
U+99B1.1	fR/S	 187.2
U+99B1.1	fTotalStrokes	 12
U+99B1.1	fCangjie	SFK
U+99B1.1	fFourCorner	74330
U+99B1.1	fMandarin	duo4
U+99B1.1	fEnglish	a load carried by a pack animal
U+99B2.0	fUTF8	馲
U+99B2.0	fCNS11643	2-486B
U+99B2.0	fBig5	E0E5
U+99B2.0	fGB	3-3D73
U+99B2.0	fR/S	 187.3
U+99B2.0	fTotalStrokes	 13
U+99B2.0	fCangjie	SFHP
U+99B2.0	fFourCorner	72314
U+99B2.0	fHakka	 tok7
U+99B2.0	fCantonese	tok3; lok6
U+99B2.0	fMandarin	tuo1; zhe2; luo4; huo1
U+99B2.0	fEnglish	the offspring of an ass and a cow
U+99B3.0	fUTF8	馳
U+99B3.0	fCNS11643	1-6769
U+99B3.0	fBig5	B9A3
U+99B3.0	fGB	1-335B
U+99B3.0	fR/S	 187.3
U+99B3.0	fTotalStrokes	 13
U+99B3.0	fCangjie	SFPD
U+99B3.0	fFourCorner	74312
U+99B3.0	fHakka	 chi1; cih1; chi3; ci2; ci1; chii1
U+99B3.0	fCantonese	ci4
U+99B3.0	fMandarin	chi2; tuo2
U+99B3.0	fEnglish	[1] go quickly or swiftly; hurry; rush; speed; fleet [2] exert; exercise [3] spread; propagate
U+99B4.0	fUTF8	馴
U+99B4.0	fCNS11643	1-676B
U+99B4.0	fBig5	B9A5
U+99B4.0	fGB	1-5131
U+99B4.0	fR/S	 187.3
U+99B4.0	fTotalStrokes	 13
U+99B4.0	fCangjie	SFLLL
U+99B4.0	fFourCorner	72300
U+99B4.0	fHakka	 sun2;  shun2
U+99B4.0	fCantonese	seon4; ceon4 (also)
U+99B4.0	fMandarin	xun2; xun4; yun2
U+99B4.0	fEnglish	[1] tame [2] mild; docile; obedient; well-bred [3] gradual [4] tame; put under control; break (an animal)
U+99B5.0	fUTF8	馵
U+99B5.0	fCNS11643	2-486D
U+99B5.0	fBig5	E0E7
U+99B5.0	fGB	3-3D72
U+99B5.0	fR/S	 187.3
U+99B5.0	fTotalStrokes	 13
U+99B5.0	fCangjie	SFT
U+99B5.0	fFourCorner	71442
U+99B5.0	fHakka	 zhu5; zu5
U+99B5.0	fCantonese	zyu3
U+99B5.0	fMandarin	zhu4
U+99B5.0	fEnglish	a horse with the hind left leg white
U+99B6.0	fUTF8	馶
U+99B6.0	fCNS11643	3-4A63
U+99B6.0	fGB	3-3D75
U+99B6.0	fR/S	 187.4
U+99B6.0	fTotalStrokes	 14
U+99B6.0	fCangjie	SFJE
U+99B6.0	fFourCorner	74340
U+99B6.0	fCantonese	zi1
U+99B6.0	fMandarin	zhi1; shi4
U+99B7.0	fUTF8	馷
U+99B7.0	fCNS11643	3-4A62
U+99B7.0	fGB	E-456E
U+99B7.0	fR/S	  187.4
U+99B7.0	fTotalStrokes	 14
U+99B7.0	fCangjie	SFJB
U+99B7.0	fFourCorner	75320
U+99B7.0	fCantonese	pui3
U+99B7.0	fMandarin	pei4
U+99B8.0	fUTF8	馸
U+99B8.0	fCNS11643	3-4A66
U+99B8.0	fGB	3-3D79
U+99B8.0	fR/S	 187.4
U+99B8.0	fTotalStrokes	 14
U+99B8.0	fCangjie	SFHML
U+99B8.0	fFourCorner	72320
U+99B8.0	fCantonese	jan6
U+99B8.0	fMandarin	xin4; jin4
U+99B9.0	fUTF8	馹
U+99B9.0	fCNS11643	2-4F3C
U+99B9.0	fBig5	E4D4
U+99B9.0	fGB	3-3D77
U+99B9.0	fR/S	 187.4
U+99B9.0	fTotalStrokes	 14
U+99B9.0	fCangjie	SFA
U+99B9.0	fFourCorner	76300
U+99B9.0	fHakka	 jit8; rit8; ngit8
U+99B9.0	fCantonese	jat6; jik6
U+99B9.0	fMandarin	ri4
U+99B9.0	fEnglish	post horse; relay horse
U+99BA.0	fUTF8	馺
U+99BA.0	fCNS11643	2-4F3E
U+99BA.0	fBig5	E4D6
U+99BA.0	fGB	3-3D74
U+99BA.0	fR/S	 187.4
U+99BA.0	fTotalStrokes	 14
U+99BA.0	fCangjie	SFNHE
U+99BA.0	fFourCorner	77347
U+99BA.0	fHakka	  sap7
U+99BA.0	fCantonese	saap3
U+99BA.0	fMandarin	sa4
U+99BA.0	fEnglish	[1] irregular [2] swift
U+99BB.0	fUTF8	馻
U+99BB.0	fCNS11643	2-4F3D
U+99BB.0	fBig5	E4D5
U+99BB.0	fGB	E-456F
U+99BB.0	fR/S	 187.4
U+99BB.0	fTotalStrokes	 14
U+99BB.0	fCangjie	SFIHU
U+99BB.0	fFourCorner	73310
U+99BB.0	fCantonese	wan5; jin5
U+99BB.0	fMandarin	yin3; yan3; xun2
U+99BC.0	fUTF8	馼
U+99BC.0	fCNS11643	3-4A61
U+99BC.0	fGB	E-4570
U+99BC.0	fR/S	 187.4
U+99BC.0	fTotalStrokes	 14
U+99BC.0	fCangjie	SFYK
U+99BC.0	fFourCorner	70340
U+99BC.0	fCantonese	man4
U+99BC.0	fMandarin	wen2
U+99BD.0	fUTF8	馽
U+99BD.0	fCNS11643	2-4F40
U+99BD.0	fBig5	E4D8
U+99BD.0	fGB	5-412C
U+99BD.0	fR/S	 187.4
U+99BD.0	fTotalStrokes	 14
U+99BD.0	fCangjie	SFL
U+99BD.0	fFourCorner	71506
U+99BD.0	fHakka	 chip7; tsiip7
U+99BD.0	fCantonese	zap1
U+99BD.0	fMandarin	zhi2; shu4; da2
U+99BD.0	fEnglish	[1] to fetter a horse [2] shackle
U+99BE.0	fUTF8	馾
U+99BE.0	fCNS11643	3-4A60
U+99BE.0	fGB	3-3D7C
U+99BE.0	fR/S	  187.4
U+99BE.0	fTotalStrokes	 14
U+99BE.0	fCangjie	SFLBU
U+99BE.0	fFourCorner	74310
U+99BE.0	fCantonese	daam3
U+99BE.0	fMandarin	dan4
U+99BF.0	fUTF8	馿
U+99BF.0	fCNS11643	3-4A6C
U+99BF.0	fGB	E-4571
U+99BF.0	fR/S	  187.4
U+99BF.0	fTotalStrokes	 14
U+99BF.0	fCangjie	SFHS
U+99BF.0	fFourCorner	73370
U+99BF.0	fHakka	 lu2
U+99BF.0	fCantonese	lou4
U+99BF.0	fMandarin	luu2; l2
U+99BF.0	fEnglish	 ass donkey
U+99C0.0	fUTF8	駀
U+99C0.0	fCNS11643	3-4A64
U+99C0.0	fGB	E-4572
U+99C0.0	fR/S	  187.4
U+99C0.0	fTotalStrokes	 14
U+99C0.0	fCangjie	SFIKU
U+99C0.0	fFourCorner	73310
U+99C0.0	fCantonese	jau6
U+99C0.0	fMandarin	you2
U+99C1.0	fUTF8	駁
U+99C1.0	fCNS11643	1-6B72
U+99C1.0	fBig5	BBE9
U+99C1.0	fGB	1-3235
U+99C1.0	fR/S	 187.4
U+99C1.0	fTotalStrokes	 14
U+99C1.0	fCangjie	SFKK
U+99C1.0	fFourCorner	74340
U+99C1.0	fHakka	  bok7
U+99C1.0	fCantonese	bok3
U+99C1.0	fMandarin	bo2
U+99C1.0	fEnglish	[1] varicolored; variegated; parti-colored [2] mixed; impure; jumbled [3] rebut; dispute; refute; disprove [4] transport; ship; load and unload [5] connect (connection, wire, luck, bones) [6] bet on; depend on (luck)
U+99C2.0	fUTF8	駂
U+99C2.0	fCNS11643	2-4F3F
U+99C2.0	fBig5	E4D7
U+99C2.0	fGB	3-3D7A
U+99C2.0	fR/S	 187.4
U+99C2.0	fTotalStrokes	 14
U+99C2.0	fCangjie	PJSQF
U+99C2.0	fFourCorner	21427
U+99C2.0	fHakka	 bau3
U+99C2.0	fCantonese	bou2
U+99C2.0	fMandarin	bao3
U+99C3.0	fUTF8	駃
U+99C3.0	fCNS11643	2-4F3B
U+99C3.0	fBig5	E4D3
U+99C3.0	fGB	3-3D7D
U+99C3.0	fR/S	 187.4
U+99C3.0	fTotalStrokes	 14
U+99C3.0	fCangjie	SFDK
U+99C3.0	fFourCorner	75337
U+99C3.0	fHakka	 giet7;  kwai5; get7
U+99C3.0	fCantonese	kyut3; faai3
U+99C3.0	fMandarin	kuai4; jue2
U+99C3.0	fEnglish	 hybrid horse
U+99C4.0	fUTF8	駄
U+99C4.0	fCNS11643	3-4A6A
U+99C4.0	fGB	E-4573
U+99C4.0	fR/S	 187.4
U+99C4.0	fTotalStrokes	 14
U+99C4.0	fCangjie	SFKI
U+99C4.0	fFourCorner	74330
U+99C4.0	fHakka	 to2
U+99C4.0	fCantonese	to4
U+99C4.0	fMandarin	tuo2; duo4
U+99C4.0	fEnglish	 carry on the back; bear
U+99C5.0	fUTF8	駅
U+99C5.0	fGB	E-4574
U+99C5.0	fR/S	 187.4
U+99C5.0	fTotalStrokes	 14
U+99C5.0	fCangjie	SFSO
U+99C5.0	fFourCorner	7733
U+99C5.0	fCantonese	jik6
U+99C5.0	fMandarin	yi4
U+99C5.0	fEnglish	relay station
U+99C6.0	fUTF8	駆
U+99C6.0	fCNS11643	4-4860
U+99C6.0	fGB	E-4575
U+99C6.0	fR/S	 187.4
U+99C6.0	fTotalStrokes	 14
U+99C6.0	fCangjie	SFSK
U+99C6.0	fFourCorner	71310
U+99C6.0	fCantonese	keoi1
U+99C6.0	fMandarin	qu1
U+99C6.0	fEnglish	 spur horse on expel; drive away
U+99C7.0	fUTF8	駇
U+99C7.0	fCNS11643	2-4F41
U+99C7.0	fBig5	E4D9
U+99C7.0	fGB	E-4576
U+99C7.0	fR/S	 187.4
U+99C7.0	fTotalStrokes	 14
U+99C7.0	fCangjie	SFOK
U+99C7.0	fFourCorner	70340
U+99C7.0	fCantonese	man4
U+99C7.0	fMandarin	wen2
U+99C8.0	fUTF8	駈
U+99C8.0	fCNS11643	3-4F73
U+99C8.0	fGB	E-4577
U+99C8.0	fR/S	  187.5
U+99C8.0	fTotalStrokes	 15
U+99C8.0	fCangjie	SFOM
U+99C8.0	fFourCorner	72310
U+99C8.0	fCantonese	keoi1
U+99C8.0	fMandarin	qu1
U+99C8.0	fEnglish	 spur horse on expel; drive away
U+99C9.0	fUTF8	駉
U+99C9.0	fCNS11643	2-5575
U+99C9.0	fBig5	E8CC
U+99C9.0	fGB	3-3E27
U+99C9.0	fR/S	 187.5
U+99C9.0	fTotalStrokes	 15
U+99C9.0	fCangjie	SFBR
U+99C9.0	fFourCorner	77320
U+99C9.0	fHakka	 kwen1
U+99C9.0	fCantonese	gwing1
U+99C9.0	fMandarin	jiong1; jiong3
U+99C9.0	fEnglish	big
U+99CA.0	fUTF8	駊
U+99CA.0	fCNS11643	3-4F6D
U+99CA.0	fGB	3-3E2D
U+99CA.0	fR/S	  187.5
U+99CA.0	fTotalStrokes	 15
U+99CA.0	fCangjie	SFDHE
U+99CA.0	fFourCorner	74340
U+99CA.0	fCantonese	po2
U+99CA.0	fMandarin	bo3; po3
U+99CB.0	fUTF8	駋
U+99CB.0	fCNS11643	2-5578
U+99CB.0	fBig5	E8CF
U+99CB.0	fGB	E-4578
U+99CB.0	fR/S	 187.5
U+99CB.0	fTotalStrokes	 15
U+99CB.0	fCangjie	SFSHR
U+99CB.0	fFourCorner	77362
U+99CB.0	fCantonese	ziu1; ciu1
U+99CB.0	fMandarin	zhao1
U+99CC.0	fUTF8	駌
U+99CC.0	fCNS11643	2-557A
U+99CC.0	fBig5	E8D1
U+99CC.0	fGB	E-4579
U+99CC.0	fR/S	 187.5
U+99CC.0	fTotalStrokes	 15
U+99CC.0	fCangjie	NUSQF
U+99CC.0	fFourCorner	27327
U+99CC.0	fCantonese	jyun1
U+99CC.0	fMandarin	yuan1
U+99CD.0	fUTF8	駍
U+99CD.0	fCNS11643	2-5570
U+99CD.0	fBig5	E8C7
U+99CD.0	fGB	3-3E23
U+99CD.0	fR/S	 187.5
U+99CD.0	fTotalStrokes	 15
U+99CD.0	fCangjie	SFMFJ
U+99CD.0	fFourCorner	71349
U+99CD.0	fCantonese	pei4; paang1
U+99CD.0	fMandarin	peng1; pi1
U+99CE.0	fUTF8	駎
U+99CE.0	fCNS11643	2-5574
U+99CE.0	fBig5	E8CB
U+99CE.0	fGB	E-457A
U+99CE.0	fR/S	 187.5
U+99CE.0	fTotalStrokes	 15
U+99CE.0	fCangjie	SFLW
U+99CE.0	fFourCorner	75360
U+99CE.0	fCantonese	zau6
U+99CE.0	fMandarin	zhou4
U+99CF.0	fUTF8	駏
U+99CF.0	fCNS11643	2-5571
U+99CF.0	fBig5	E8C8
U+99CF.0	fGB	3-3D76
U+99CF.0	fR/S	 187.5
U+99CF.0	fTotalStrokes	 15
U+99CF.0	fCangjie	SFSS
U+99CF.0	fFourCorner	71317
U+99CF.0	fHakka	  ki3
U+99CF.0	fCantonese	geoi6
U+99CF.0	fMandarin	ju4
U+99CF.0	fEnglish	offspring of a stallion and a she-mule
U+99D0.0	fUTF8	駐
U+99D0.0	fCNS11643	1-6F77
U+99D0.0	fBig5	BE6E
U+99D0.0	fGB	1-5724
U+99D0.0	fR/S	 187.5
U+99D0.0	fTotalStrokes	 15
U+99D0.0	fCangjie	SFYG
U+99D0.0	fFourCorner	70314
U+99D0.0	fHakka	 zu5;  zhu5
U+99D0.0	fCantonese	zyu3
U+99D0.0	fMandarin	zhu4
U+99D0.0	fEnglish	 be stationed at stop reside
U+99D1.0	fUTF8	駑
U+99D1.0	fCNS11643	1-6F7A
U+99D1.0	fBig5	BE71
U+99D1.0	fGB	1-6665
U+99D1.0	fR/S	 187.5
U+99D1.0	fTotalStrokes	 15
U+99D1.0	fCangjie	VESQF
U+99D1.0	fFourCorner	47327
U+99D1.0	fHakka	  nu2; lu2
U+99D1.0	fCantonese	nou4; lou4 (also)
U+99D1.0	fMandarin	nu2
U+99D1.0	fEnglish	[1] old, worn-out horse; jade; hack [2] incompetent; stupid; good-for-nothing
U+99D2.0	fUTF8	駒
U+99D2.0	fCNS11643	1-6F7C
U+99D2.0	fBig5	BE73
U+99D2.0	fGB	1-3E54
U+99D2.0	fR/S	 187.5
U+99D2.0	fTotalStrokes	 15
U+99D2.0	fCangjie	SFPR
U+99D2.0	fFourCorner	77320
U+99D2.0	fHakka	 ki1;  gi1
U+99D2.0	fCantonese	keoi1
U+99D2.0	fMandarin	ju1; ju4
U+99D2.0	fEnglish	[1] young and fleet-footed horse; colt; foal [2] the sun (figuratively)
U+99D3.0	fUTF8	駓
U+99D3.0	fCNS11643	2-5572
U+99D3.0	fBig5	E8C9
U+99D3.0	fGB	3-3D7E
U+99D3.0	fR/S	 187.5
U+99D3.0	fTotalStrokes	 15
U+99D3.0	fCangjie	SFMFM
U+99D3.0	fFourCorner	71319
U+99D3.0	fHakka	  pi1
U+99D3.0	fCantonese	pei1; pei3
U+99D3.0	fMandarin	pi1; pi2
U+99D3.0	fEnglish	gallop
U+99D4.0	fUTF8	駔
U+99D4.0	fCNS11643	2-5573
U+99D4.0	fBig5	E8CA
U+99D4.0	fGB	1-6660
U+99D4.0	fR/S	 187.5
U+99D4.0	fTotalStrokes	 15
U+99D4.0	fCangjie	SFBM
U+99D4.0	fFourCorner	77310
U+99D4.0	fHakka	  cong3;  zu3;  zong3; zih3
U+99D4.0	fCantonese	zou2; zong2; cong2
U+99D4.0	fMandarin	zang3; cu1; zu3; chu2
U+99D4.0	fEnglish	[1] strong horse [2] horse broker
U+99D4.1	fUTF8	駔
U+99D4.1	fCNS11643	2-5573
U+99D4.1	fBig5	E8CA
U+99D4.1	fGB	1-6660
U+99D4.1	fR/S	 187.5
U+99D4.1	fTotalStrokes	 15
U+99D4.1	fCangjie	SFBM
U+99D4.1	fFourCorner	77310
U+99D4.1	fMandarin	zu4
U+99D4.1	fEnglish	 swift horse; fine horse; excellent horst
U+99D4.2	fUTF8	駔
U+99D4.2	fCNS11643	2-5573
U+99D4.2	fBig5	E8CA
U+99D4.2	fGB	1-6660
U+99D4.2	fR/S	 187.5
U+99D4.2	fTotalStrokes	 15
U+99D4.2	fCangjie	SFBM
U+99D4.2	fFourCorner	77310
U+99D4.2	fMandarin	cang3
U+99D4.2	fEnglish	[1] rascal; ruffian; mean person
U+99D5.0	fUTF8	駕
U+99D5.0	fCNS11643	1-6F7B
U+99D5.0	fBig5	BE72
U+99D5.0	fGB	1-3C5D
U+99D5.0	fR/S	 187.5
U+99D5.0	fTotalStrokes	 15
U+99D5.0	fCangjie	KRSQF
U+99D5.0	fFourCorner	46327
U+99D5.0	fHakka	  ga5
U+99D5.0	fCantonese	gaa3; gaa2 (also)
U+99D5.0	fMandarin	jia4; jia1
U+99D5.0	fEnglish	[1] ride; drive; pilot [2] excel; surpass [3] yoke; put the horses to the carriage [4] vehicles [5] an honorific epithet [6] control; reign; rule [7] the emperor
U+99D6.0	fUTF8	駖
U+99D6.0	fCNS11643	2-5576
U+99D6.0	fBig5	E8CD
U+99D6.0	fGB	3-3E29
U+99D6.0	fR/S	 187.5
U+99D6.0	fTotalStrokes	 15
U+99D6.0	fCangjie	SFOII
U+99D6.0	fFourCorner	78337
U+99D6.0	fCantonese	ling4
U+99D6.0	fMandarin	ling2; lin2
U+99D7.0	fUTF8	駗
U+99D7.0	fCNS11643	2-5579
U+99D7.0	fBig5	E8D0
U+99D7.0	fGB	3-3E28
U+99D7.0	fR/S	 187.5
U+99D7.0	fTotalStrokes	 15
U+99D7.0	fCangjie	SFOHH
U+99D7.0	fFourCorner	78322
U+99D7.0	fCantonese	can2; leon4
U+99D7.0	fMandarin	zhen1; zhen3; chen4; lin2
U+99D8.0	fUTF8	駘
U+99D8.0	fCNS11643	2-5577
U+99D8.0	fBig5	E8CE
U+99D8.0	fGB	1-6666
U+99D8.0	fR/S	 187.5
U+99D8.0	fTotalStrokes	 15
U+99D8.0	fCangjie	SFIR
U+99D8.0	fFourCorner	73360
U+99D8.0	fHakka	 toi2; koi3
U+99D8.0	fCantonese	toi4; toi5; doi6
U+99D8.0	fMandarin	tai2; dai4; zhai4; tai1
U+99D8.0	fEnglish	[1] worn-out horse; jade [2] tired; exhausted; jaded [3] incompetent
U+99D9.0	fUTF8	駙
U+99D9.0	fCNS11643	1-6F7D
U+99D9.0	fBig5	BE74
U+99D9.0	fGB	1-6662
U+99D9.0	fR/S	 187.5
U+99D9.0	fTotalStrokes	 15
U+99D9.0	fCangjie	SFODI
U+99D9.0	fFourCorner	74300
U+99D9.0	fHakka	  fu5
U+99D9.0	fCantonese	fu6
U+99D9.0	fMandarin	fu4
U+99D9.0	fEnglish	[1] extra horses harnessed by the side of the team [2] swift
U+99DA.0	fUTF8	駚
U+99DA.0	fCNS11643	3-4F6E
U+99DA.0	fGB	3-3E26
U+99DA.0	fR/S	 187.5
U+99DA.0	fTotalStrokes	 15
U+99DA.0	fCangjie	SFLBK
U+99DA.0	fFourCorner	75330
U+99DA.0	fCantonese	joeng2
U+99DA.0	fMandarin	yang3; ang4
U+99DB.0	fUTF8	駛
U+99DB.0	fCNS11643	1-6F79
U+99DB.0	fBig5	BE70
U+99DB.0	fGB	1-4A3B
U+99DB.0	fR/S	 187.5
U+99DB.0	fTotalStrokes	 15
U+99DB.0	fCangjie	SFLK
U+99DB.0	fFourCorner	75306
U+99DB.0	fHakka	  sii3;  su3; se3
U+99DB.0	fCantonese	sai2; si2
U+99DB.0	fMandarin	shi3; shi4
U+99DB.0	fEnglish	[1] sail (vessels); run; drive; pilot (vehicles) [2] fast; quick; fleeting
U+99DC.0	fUTF8	駜
U+99DC.0	fCNS11643	2-556F
U+99DC.0	fBig5	E8C6
U+99DC.0	fGB	3-3E2C
U+99DC.0	fR/S	 187.5
U+99DC.0	fTotalStrokes	 15
U+99DC.0	fCangjie	SFPH
U+99DC.0	fFourCorner	73300
U+99DC.0	fHakka	  bit7
U+99DC.0	fCantonese	bat6; bit1 (also); bei3
U+99DC.0	fMandarin	bi4
U+99DC.0	fEnglish	strong horse
U+99DD.0	fUTF8	駝
U+99DD.0	fCNS11643	1-6F76
U+99DD.0	fBig5	BE6D
U+99DD.0	fGB	1-4D55
U+99DD.0	fR/S	 187.5
U+99DD.0	fTotalStrokes	 15
U+99DD.0	fCangjie	SFJP
U+99DD.0	fFourCorner	73311
U+99DD.0	fHakka	  to2; to3
U+99DD.0	fCantonese	to4; to2 (also)
U+99DD.0	fMandarin	tuo2
U+99DD.0	fEnglish	[1] camel [2] hunchbacked; humpbacked [3] carry on the back [4] pay
U+99DD.1	fUTF8	駝
U+99DD.1	fCNS11643	1-6F76
U+99DD.1	fBig5	BE6D
U+99DD.1	fGB	1-4D55
U+99DD.1	fR/S	 187.5
U+99DD.1	fTotalStrokes	 15
U+99DD.1	fCangjie	SFJP
U+99DD.1	fFourCorner	73311
U+99DD.1	fMandarin	tuo5
U+99DE.0	fUTF8	駞
U+99DE.0	fCNS11643	3-4F72
U+99DE.0	fGB	E-457B
U+99DE.0	fR/S	  187.5
U+99DE.0	fTotalStrokes	 15
U+99DE.0	fCangjie	SFOPD
U+99DE.0	fFourCorner	78310
U+99DE.0	fHakka	  to2
U+99DE.0	fCantonese	to4
U+99DE.0	fMandarin	tuo2
U+99DE.0	fEnglish	camel
U+99DF.0	fUTF8	駟
U+99DF.0	fCNS11643	1-6F78
U+99DF.0	fBig5	BE6F
U+99DF.0	fGB	1-6661
U+99DF.0	fR/S	 187.5
U+99DF.0	fTotalStrokes	 15
U+99DF.0	fCangjie	SFWC
U+99DF.0	fFourCorner	76300
U+99DF.0	fHakka	  si5
U+99DF.0	fCantonese	si3
U+99DF.0	fMandarin	si4; xi1
U+99DF.0	fEnglish	[1] a team of four horses [2] horses [3] four [4] the name of a star
U+99E0.0	fUTF8	駠
U+99E0.0	fCNS11643	3-4F74
U+99E0.0	fGB	E-457C
U+99E0.0	fR/S	 187.5
U+99E0.0	fTotalStrokes	 15
U+99E0.0	fCangjie	SFHHL
U+99E0.0	fFourCorner	77320
U+99E0.0	fHakka	 liu2
U+99E0.0	fCantonese	lau4
U+99E0.0	fMandarin	liu2
U+99E0.0	fEnglish	 angry or startled horse
U+99E1.0	fUTF8	駡
U+99E1.0	fCNS11643	3-5370
U+99E1.0	fGB	1-426E
U+99E1.0	fR/S	 187.6; 30.13
U+99E1.0	fTotalStrokes	 16
U+99E1.0	fCangjie	RRSQF
U+99E1.0	fFourCorner	66320
U+99E1.0	fHakka	  ma5
U+99E1.0	fCantonese	maa6; maa3 (also)
U+99E1.0	fMandarin	ma4
U+99E1.0	fEnglish	accuse; blame; curse; scold
U+99E2.0	fUTF8	駢
U+99E2.0	fCNS11643	1-732E
U+99E2.0	fBig5	C063
U+99E2.0	fGB	1-6669
U+99E2.0	fR/S	 187.6
U+99E2.0	fTotalStrokes	 16
U+99E2.0	fCangjie	SFTT
U+99E2.0	fFourCorner	78341
U+99E2.0	fHakka	  pin2; bin1; pen2; bin5
U+99E2.0	fCantonese	pin4; ping4 (also)
U+99E2.0	fMandarin	pian2
U+99E2.0	fEnglish	[1] a pair of horses [2] stand or lie side by side
U+99E3.0	fUTF8	駣
U+99E3.0	fCNS11643	2-5B72
U+99E3.0	fBig5	EC66
U+99E3.0	fGB	3-3E34
U+99E3.0	fR/S	 187.6
U+99E3.0	fTotalStrokes	 16
U+99E3.0	fCangjie	SFLMO
U+99E3.0	fFourCorner	72313
U+99E3.0	fHakka	  tau2; to2; rau2; jau2; teu2
U+99E3.0	fCantonese	tou4
U+99E3.0	fMandarin	tao2; zhao4; dao4
U+99E3.0	fEnglish	 colt
U+99E4.0	fUTF8	駤
U+99E4.0	fCNS11643	2-5B70
U+99E4.0	fBig5	EC64
U+99E4.0	fGB	3-3E31
U+99E4.0	fR/S	 187.6
U+99E4.0	fTotalStrokes	 16
U+99E4.0	fCangjie	SFMIG
U+99E4.0	fFourCorner	71314
U+99E4.0	fCantonese	zi3
U+99E4.0	fMandarin	zhi4
U+99E5.0	fUTF8	駥
U+99E5.0	fCNS11643	2-5B6F
U+99E5.0	fBig5	EC63
U+99E5.0	fGB	5-412E
U+99E5.0	fR/S	 187.6
U+99E5.0	fTotalStrokes	 16
U+99E5.0	fCangjie	SFIJ
U+99E5.0	fFourCorner	73350
U+99E5.0	fHakka	 jung2; rung2
U+99E5.0	fCantonese	jung4
U+99E5.0	fMandarin	rong2; xue4
U+99E5.0	fEnglish	 stallion eight feet high
U+99E6.0	fUTF8	駦
U+99E6.0	fCNS11643	3-5371
U+99E6.0	fGB	E-457D
U+99E6.0	fR/S	 187.6
U+99E6.0	fTotalStrokes	 16
U+99E6.0	fCangjie	FQSQF
U+99E6.0	fFourCorner	90320
U+99E6.0	fHakka	 ten2; tin1
U+99E6.0	fCantonese	tang4
U+99E6.0	fMandarin	teng2
U+99E6.0	fEnglish	 mount; ascend move; turn out
U+99E7.0	fUTF8	駧
U+99E7.0	fCNS11643	2-5B75
U+99E7.0	fBig5	EC69
U+99E7.0	fGB	5-4133
U+99E7.0	fR/S	 187.6
U+99E7.0	fTotalStrokes	 16
U+99E7.0	fCangjie	SFBMR
U+99E7.0	fFourCorner	77320
U+99E7.0	fCantonese	dung6
U+99E7.0	fMandarin	dong4
U+99E8.0	fUTF8	駨
U+99E8.0	fCNS11643	3-536B
U+99E8.0	fGB	3-3E35
U+99E8.0	fR/S	  187.6
U+99E8.0	fTotalStrokes	 16
U+99E8.0	fCangjie	SFPA
U+99E8.0	fFourCorner	77320
U+99E8.0	fCantonese	seon1
U+99E8.0	fMandarin	xun2
U+99E9.0	fUTF8	駩
U+99E9.0	fCNS11643	2-5B74
U+99E9.0	fBig5	EC68
U+99E9.0	fGB	E-457E
U+99E9.0	fR/S	 187.6
U+99E9.0	fTotalStrokes	 16
U+99E9.0	fCangjie	SFOMG
U+99E9.0	fFourCorner	78314
U+99E9.0	fCantonese	cyun1; cyun4
U+99E9.0	fMandarin	quan2
U+99EA.0	fUTF8	駪
U+99EA.0	fCNS11643	2-5B73
U+99EA.0	fBig5	EC67
U+99EA.0	fGB	5-4137
U+99EA.0	fR/S	 187.6
U+99EA.0	fTotalStrokes	 16
U+99EA.0	fCangjie	SFHGU
U+99EA.0	fFourCorner	74311
U+99EA.0	fHakka	  sin1
U+99EA.0	fCantonese	san1
U+99EA.0	fMandarin	shen1
U+99EA.0	fEnglish	crowd
U+99EB.0	fUTF8	駫
U+99EB.0	fCNS11643	3-536C
U+99EB.0	fGB	E-4621
U+99EB.0	fR/S	  187.6
U+99EB.0	fTotalStrokes	 16
U+99EB.0	fCangjie	SFFMU
U+99EB.0	fFourCorner	79310
U+99EB.0	fCantonese	gwing1
U+99EB.0	fMandarin	jiong1
U+99EC.0	fUTF8	駬
U+99EC.0	fCNS11643	2-5B6E
U+99EC.0	fBig5	EC62
U+99EC.0	fGB	5-4130
U+99EC.0	fR/S	 187.6
U+99EC.0	fTotalStrokes	 16
U+99EC.0	fCangjie	SFSJ
U+99EC.0	fFourCorner	71340
U+99EC.0	fHakka	 ngi3; ngi5
U+99EC.0	fCantonese	ji5
U+99EC.0	fMandarin	er3
U+99EC.0	fEnglish	the name of a legendary swift horse
U+99ED.0	fUTF8	駭
U+99ED.0	fCNS11643	1-732D
U+99ED.0	fBig5	C062
U+99ED.0	fGB	1-3A27
U+99ED.0	fR/S	 187.6
U+99ED.0	fTotalStrokes	 16
U+99ED.0	fCangjie	SFYVO
U+99ED.0	fFourCorner	70382
U+99ED.0	fHakka	 hai2; hai3; hai5
U+99ED.0	fCantonese	haai5; hoi5 (also); hoi4; haai6
U+99ED.0	fMandarin	hai4; xie4
U+99ED.0	fEnglish	[1] terrify; frighten; scare; shock; surprise; startle; amaze [2] marvel; wonder
U+99EE.0	fUTF8	駮
U+99EE.0	fCNS11643	2-5B6D
U+99EE.0	fBig5	EC61
U+99EE.0	fGB	E-4622
U+99EE.0	fR/S	 187.6
U+99EE.0	fTotalStrokes	 16
U+99EE.0	fCangjie	SFYCK
U+99EE.0	fFourCorner	70348
U+99EE.0	fHakka	  bok7
U+99EE.0	fCantonese	bok3
U+99EE.0	fMandarin	bo2
U+99EE.0	fEnglish	[1] a kind of fierce animal [2] mixed; impure [3] refute
U+99EF.0	fUTF8	駯
U+99EF.0	fCNS11643	3-536A
U+99EF.0	fGB	E-4623
U+99EF.0	fR/S	  187.6
U+99EF.0	fTotalStrokes	 16
U+99EF.0	fCangjie	SFHJD
U+99EF.0	fFourCorner	75390
U+99EF.0	fCantonese	zyu1
U+99EF.0	fMandarin	zhu1; ju1
U+99F0.0	fUTF8	駰
U+99F0.0	fCNS11643	2-5B71
U+99F0.0	fBig5	EC65
U+99F0.0	fGB	3-3E32
U+99F0.0	fR/S	 187.6
U+99F0.0	fTotalStrokes	 16
U+99F0.0	fCangjie	SFWK
U+99F0.0	fFourCorner	76300
U+99F0.0	fHakka	 jin1; rin1
U+99F0.0	fCantonese	jan1
U+99F0.0	fMandarin	yin1
U+99F0.0	fEnglish	grey horse
U+99F1.0	fUTF8	駱
U+99F1.0	fCNS11643	1-732F
U+99F1.0	fBig5	C064
U+99F1.0	fGB	1-4266
U+99F1.0	fR/S	 187.6
U+99F1.0	fTotalStrokes	 16
U+99F1.0	fCangjie	SFHER
U+99F1.0	fFourCorner	77364
U+99F1.0	fHakka	 lok7; lok8
U+99F1.0	fCantonese	lok3; lok6 (also)
U+99F1.0	fMandarin	luo4; jia4
U+99F1.0	fEnglish	[1] white horse (or steed) wtih black mane [2] camel [3] a Chinese family name
U+99F2.0	fUTF8	駲
U+99F2.0	fGB	E-4624
U+99F2.0	fR/S	 187.6
U+99F2.0	fTotalStrokes	 16
U+99F2.0	fCangjie	SFILL
U+99F2.0	fCantonese	zau1
U+99F3.0	fUTF8	駳
U+99F3.0	fCNS11643	3-5734
U+99F3.0	fGB	E-4625
U+99F3.0	fR/S	  187.8
U+99F3.0	fTotalStrokes	 18; 17
U+99F3.0	fCangjie	SFNKM
U+99F3.0	fFourCorner	72340
U+99F3.0	fCantonese	daan6
U+99F3.0	fMandarin	dan4
U+99F4.0	fUTF8	駴
U+99F4.0	fCNS11643	2-6066
U+99F4.0	fBig5	EF5A
U+99F4.0	fGB	3-3E39
U+99F4.0	fR/S	 187.7
U+99F4.0	fTotalStrokes	 17
U+99F4.0	fCangjie	SFIT
U+99F4.0	fFourCorner	73350
U+99F4.0	fCantonese	haai5
U+99F4.0	fMandarin	xie4; jie4
U+99F4.0	fEnglish	drum
U+99F5.0	fUTF8	駵
U+99F5.0	fCNS11643	3-5737
U+99F5.0	fGB	5-413A
U+99F5.0	fR/S	 187.7
U+99F5.0	fTotalStrokes	 17
U+99F5.0	fCangjie	SFMLS
U+99F5.0	fFourCorner	71370
U+99F5.0	fCantonese	lau4
U+99F5.0	fMandarin	liu2
U+99F6.0	fUTF8	駶
U+99F6.0	fCNS11643	2-606A
U+99F6.0	fBig5	EF5E
U+99F6.0	fGB	E-4626
U+99F6.0	fR/S	 187.7
U+99F6.0	fTotalStrokes	 17
U+99F6.0	fCangjie	SFSSR
U+99F6.0	fFourCorner	77327
U+99F6.0	fCantonese	guk6; guk1
U+99F6.0	fMandarin	ju2
U+99F7.0	fUTF8	駷
U+99F7.0	fCNS11643	2-6067
U+99F7.0	fBig5	EF5B
U+99F7.0	fGB	3-3E3A
U+99F7.0	fR/S	 187.7
U+99F7.0	fTotalStrokes	 17
U+99F7.0	fCangjie	SFDL
U+99F7.0	fFourCorner	75396
U+99F7.0	fCantonese	sung2
U+99F7.0	fMandarin	song3
U+99F8.0	fUTF8	駸
U+99F8.0	fCNS11643	2-6069
U+99F8.0	fBig5	EF5D
U+99F8.0	fGB	3-3E46
U+99F8.0	fR/S	 187.7
U+99F8.0	fTotalStrokes	 17
U+99F8.0	fCangjie	SFSME
U+99F8.0	fFourCorner	77347
U+99F8.0	fHakka	 cim1
U+99F8.0	fCantonese	cam1
U+99F8.0	fMandarin	qin1
U+99F8.0	fEnglish	galloping; speeding
U+99F9.0	fUTF8	駹
U+99F9.0	fCNS11643	2-6068
U+99F9.0	fBig5	EF5C
U+99F9.0	fGB	3-3E3B
U+99F9.0	fR/S	 187.7
U+99F9.0	fTotalStrokes	 17
U+99F9.0	fCangjie	SFIUH
U+99F9.0	fFourCorner	73312
U+99F9.0	fHakka	  mang2
U+99F9.0	fCantonese	mong4
U+99F9.0	fMandarin	mang2
U+99F9.0	fEnglish	black horse with a white face
U+99FA.0	fUTF8	駺
U+99FA.0	fCNS11643	2-6065
U+99FA.0	fBig5	EF59
U+99FA.0	fGB	3-3E45
U+99FA.0	fR/S	 187.7
U+99FA.0	fTotalStrokes	 17
U+99FA.0	fCangjie	SFIAV
U+99FA.0	fFourCorner	73332
U+99FA.0	fCantonese	long4; loeng4
U+99FA.0	fMandarin	liang2; lang2
U+99FB.0	fUTF8	駻
U+99FB.0	fCNS11643	2-606B
U+99FB.0	fBig5	EF5F
U+99FB.0	fGB	3-3E3D
U+99FB.0	fR/S	 187.7
U+99FB.0	fTotalStrokes	 17
U+99FB.0	fCangjie	SFAMJ
U+99FB.0	fFourCorner	76341
U+99FB.0	fHakka	 hon5; gon5; hon3; hon6
U+99FB.0	fCantonese	hon6; hon2; hon5
U+99FB.0	fMandarin	han4
U+99FB.0	fEnglish	[1] wild (horses); fierce [2] saddle
U+99FC.0	fUTF8	駼
U+99FC.0	fCNS11643	2-606E
U+99FC.0	fBig5	EF62
U+99FC.0	fGB	3-3E41
U+99FC.0	fR/S	 187.7
U+99FC.0	fTotalStrokes	 17
U+99FC.0	fCangjie	SFOMD
U+99FC.0	fFourCorner	78391
U+99FC.0	fHakka	  tu2
U+99FC.0	fCantonese	tou4
U+99FC.0	fMandarin	tu2
U+99FC.0	fEnglish	a kind of wild ass
U+99FD.0	fUTF8	駽
U+99FD.0	fCNS11643	2-606C
U+99FD.0	fBig5	EF60
U+99FD.0	fGB	3-3E3E
U+99FD.0	fR/S	 187.7
U+99FD.0	fTotalStrokes	 17
U+99FD.0	fCangjie	SFRB
U+99FD.0	fFourCorner	76327
U+99FD.0	fHakka	 gien1; gen1
U+99FD.0	fCantonese	hyun1; gyun1; hyun3
U+99FD.0	fMandarin	xuan4; xuan1
U+99FD.0	fEnglish	grey
U+99FE.0	fUTF8	駾
U+99FE.0	fCNS11643	2-606D
U+99FE.0	fBig5	EF61
U+99FE.0	fGB	3-3E44
U+99FE.0	fR/S	 187.7
U+99FE.0	fTotalStrokes	 17
U+99FE.0	fCangjie	SFCRU
U+99FE.0	fFourCorner	78316
U+99FE.0	fHakka	 tui3; tui5
U+99FE.0	fCantonese	teoi3; deoi6
U+99FE.0	fMandarin	tui4
U+99FE.0	fEnglish	run away
U+99FF.0	fUTF8	駿
U+99FF.0	fCNS11643	1-762C
U+99FF.0	fBig5	C240
U+99FF.0	fGB	1-3F25
U+99FF.0	fR/S	 187.7
U+99FF.0	fTotalStrokes	 17
U+99FF.0	fCangjie	SFICE
U+99FF.0	fFourCorner	73347
U+99FF.0	fHakka	 zun5
U+99FF.0	fCantonese	zeon3
U+99FF.0	fMandarin	jun4
U+99FF.0	fEnglish	excellent horse; noble steed
U+9A00.0	fUTF8	騀
U+9A00.0	fCNS11643	3-5733
U+9A00.0	fGB	3-3E40
U+9A00.0	fR/S	  187.7
U+9A00.0	fTotalStrokes	 17
U+9A00.0	fCangjie	SFHQI
U+9A00.0	fFourCorner	73350
U+9A00.0	fCantonese	ngo5; o5
U+9A00.0	fMandarin	e2; e3
U+9A01.0	fUTF8	騁
U+9A01.0	fCNS11643	1-762B
U+9A01.0	fBig5	C1FE
U+9A01.0	fGB	1-3352
U+9A01.0	fR/S	 187.7
U+9A01.0	fTotalStrokes	 17
U+9A01.0	fCangjie	SFLWS
U+9A01.0	fFourCorner	75327
U+9A01.0	fHakka	  pin5; pin3
U+9A01.0	fCantonese	cing2; ping3 (also)
U+9A01.0	fMandarin	cheng3
U+9A01.0	fEnglish	[1] go swiftly; speed [2] exert; unfold; develop [3] give free play to; lend wings to
U+9A02.0	fUTF8	騂
U+9A02.0	fCNS11643	2-6064
U+9A02.0	fBig5	EF58
U+9A02.0	fGB	3-3E43
U+9A02.0	fR/S	 187.7
U+9A02.0	fTotalStrokes	 17
U+9A02.0	fCangjie	SFYTJ
U+9A02.0	fFourCorner	70341
U+9A02.0	fHakka	  sin1
U+9A02.0	fCantonese	sing1
U+9A02.0	fMandarin	xing1; sou1; xin1
U+9A02.0	fEnglish	red
U+9A03.0	fUTF8	騃
U+9A03.0	fCNS11643	2-606F
U+9A03.0	fBig5	EF63
U+9A03.0	fGB	E-4627
U+9A03.0	fR/S	 187.7
U+9A03.0	fTotalStrokes	 17
U+9A03.0	fCangjie	SFIOK
U+9A03.0	fFourCorner	72334
U+9A03.0	fHakka	 ai3
U+9A03.0	fCantonese	zi6; ngoi4; daai1; oi4 (also); ngaai6
U+9A03.0	fMandarin	ai2; si4; dai1
U+9A03.0	fEnglish	stupid; foolish
U+9A04.0	fUTF8	騄
U+9A04.0	fCNS11643	2-645D
U+9A04.0	fBig5	F1B3
U+9A04.0	fGB	5-4141
U+9A04.0	fR/S	 187.8
U+9A04.0	fTotalStrokes	 18
U+9A04.0	fCangjie	SFVNE
U+9A04.0	fFourCorner	77332
U+9A04.0	fHakka	 liuk8
U+9A04.0	fCantonese	luk6
U+9A04.0	fMandarin	lu4
U+9A04.0	fEnglish	the name of a legendary swift horse (said of covering 1,000 li in a day)
U+9A05.0	fUTF8	騅
U+9A05.0	fCNS11643	2-6460
U+9A05.0	fBig5	F1B6
U+9A05.0	fGB	1-666D
U+9A05.0	fR/S	 187.8
U+9A05.0	fTotalStrokes	 18
U+9A05.0	fCangjie	SFOG
U+9A05.0	fFourCorner	70314
U+9A05.0	fHakka	 chui1; zui1; zhui1
U+9A05.0	fCantonese	zeoi1
U+9A05.0	fMandarin	zhui1
U+9A05.0	fEnglish	 piebald horse
U+9A06.0	fUTF8	騆
U+9A06.0	fCNS11643	2-6462
U+9A06.0	fBig5	F1B8
U+9A06.0	fGB	E-4628
U+9A06.0	fR/S	 187.8
U+9A06.0	fTotalStrokes	 18
U+9A06.0	fCangjie	SFBGR
U+9A06.0	fFourCorner	77320
U+9A06.0	fCantonese	zau1; dung6
U+9A06.0	fMandarin	zhou1
U+9A07.0	fUTF8	騇
U+9A07.0	fCNS11643	2-6461
U+9A07.0	fBig5	F1B7
U+9A07.0	fGB	3-3E4C
U+9A07.0	fR/S	 187.8
U+9A07.0	fTotalStrokes	 18
U+9A07.0	fCangjie	SFOMR
U+9A07.0	fFourCorner	78364
U+9A07.0	fHakka	 sha5; sa5
U+9A07.0	fCantonese	se3; se5
U+9A07.0	fMandarin	she4; she3
U+9A07.0	fEnglish	 mare
U+9A08.0	fUTF8	騈
U+9A08.0	fCNS11643	3-596A
U+9A08.0	fGB	E-4629
U+9A08.0	fR/S	 187.8
U+9A08.0	fTotalStrokes	 18
U+9A08.0	fCangjie	SFYJJ
U+9A08.0	fFourCorner	73340
U+9A08.0	fHakka	  pin2
U+9A08.0	fCantonese	pin4; ping4 (also)
U+9A08.0	fMandarin	pian2
U+9A08.0	fEnglish	 team of horses associate
U+9A09.0	fUTF8	騉
U+9A09.0	fCNS11643	2-645B
U+9A09.0	fBig5	F1B1
U+9A09.0	fGB	3-3E4A
U+9A09.0	fR/S	 187.8
U+9A09.0	fTotalStrokes	 18
U+9A09.0	fCangjie	SFAPP
U+9A09.0	fFourCorner	76311
U+9A09.0	fCantonese	kwan1
U+9A09.0	fMandarin	kun1
U+9A09.0	fEnglish	fine horse
U+9A0A.0	fUTF8	騊
U+9A0A.0	fCNS11643	2-645F
U+9A0A.0	fBig5	F1B5
U+9A0A.0	fGB	3-3E4E
U+9A0A.0	fR/S	 187.8
U+9A0A.0	fTotalStrokes	 18
U+9A0A.0	fCangjie	SFPOU
U+9A0A.0	fFourCorner	77320
U+9A0A.0	fHakka	 tau2
U+9A0A.0	fCantonese	tou4
U+9A0A.0	fMandarin	tao2
U+9A0B.0	fUTF8	騋
U+9A0B.0	fCNS11643	2-645A
U+9A0B.0	fBig5	F1B0
U+9A0B.0	fGB	3-3E3C
U+9A0B.0	fR/S	 187.8
U+9A0B.0	fTotalStrokes	 18
U+9A0B.0	fCangjie	SFDOO
U+9A0B.0	fFourCorner	74398
U+9A0B.0	fHakka	 loi2
U+9A0B.0	fCantonese	loi4; loi6
U+9A0B.0	fMandarin	lai2
U+9A0B.0	fEnglish	tall (7 chi) horse
U+9A0C.0	fUTF8	騌
U+9A0C.0	fCNS11643	3-596B
U+9A0C.0	fGB	E-462A
U+9A0C.0	fR/S	 187.8
U+9A0C.0	fTotalStrokes	 18
U+9A0C.0	fCangjie	SFJMF
U+9A0C.0	fFourCorner	73390
U+9A0C.0	fHakka	 zung1
U+9A0C.0	fCantonese	zung1
U+9A0C.0	fMandarin	zong1
U+9A0C.0	fEnglish	 a horse's mane bristles
U+9A0D.0	fUTF8	騍
U+9A0D.0	fCNS11643	2-645C
U+9A0D.0	fBig5	F1B2
U+9A0D.0	fGB	1-666C
U+9A0D.0	fR/S	 187.8
U+9A0D.0	fTotalStrokes	 18
U+9A0D.0	fCangjie	SFWD
U+9A0D.0	fFourCorner	76394
U+9A0D.0	fHakka	  ko5
U+9A0D.0	fCantonese	fo3
U+9A0D.0	fMandarin	ke4
U+9A0D.0	fEnglish	mother horse
U+9A0E.0	fUTF8	騎
U+9A0E.0	fCNS11643	1-7777
U+9A0E.0	fBig5	C34D
U+9A0E.0	fGB	1-466F
U+9A0E.0	fR/S	 187.8
U+9A0E.0	fTotalStrokes	 18
U+9A0E.0	fCangjie	SFKMR
U+9A0E.0	fFourCorner	74321
U+9A0E.0	fHakka	  ki2; ji3; ki3; ri3
U+9A0E.0	fCantonese	ke4; kei4 (also); gei6; kei3
U+9A0E.0	fMandarin	qi2; ji4
U+9A0E.0	fEnglish	 ride (horse, etc.); sit astride on
U+9A0F.0	fUTF8	騏
U+9A0F.0	fCNS11643	2-6459
U+9A0F.0	fBig5	F1AF
U+9A0F.0	fGB	1-666B
U+9A0F.0	fR/S	 187.8
U+9A0F.0	fTotalStrokes	 18
U+9A0F.0	fCangjie	SFTMC
U+9A0F.0	fFourCorner	74381
U+9A0F.0	fHakka	  ki2
U+9A0F.0	fCantonese	kei4
U+9A0F.0	fMandarin	qi2
U+9A0F.0	fEnglish	[1] dark-blue horse [2] fine horse; excellent horse [3] dark blue
U+9A10.0	fUTF8	騐
U+9A10.0	fCNS11643	3-596D
U+9A10.0	fGB	E-462B
U+9A10.0	fR/S	 187.8
U+9A10.0	fTotalStrokes	 18
U+9A10.0	fCangjie	SFOIP
U+9A10.0	fFourCorner	78310
U+9A10.0	fHakka	 ngiam5; ngiam6
U+9A10.0	fCantonese	jim6
U+9A10.0	fMandarin	yan4
U+9A10.0	fEnglish	 test
U+9A11.0	fUTF8	騑
U+9A11.0	fCNS11643	2-645E
U+9A11.0	fBig5	F1B4
U+9A11.0	fGB	3-3E49
U+9A11.0	fR/S	 187.8
U+9A11.0	fTotalStrokes	 18
U+9A11.0	fCangjie	SFLMY
U+9A11.0	fFourCorner	71311
U+9A11.0	fHakka	 fui1; fi1
U+9A11.0	fCantonese	fei1
U+9A11.0	fMandarin	fei1
U+9A11.0	fEnglish	 horse run
U+9A12.0	fUTF8	騒
U+9A12.0	fGB	E-462C
U+9A12.0	fR/S	 187.8
U+9A12.0	fTotalStrokes	 18
U+9A12.0	fCangjie	SFELI
U+9A12.0	fCantonese	sou1
U+9A12.0	fMandarin	sao1; sao3
U+9A12.0	fEnglish	harass; bother; annoy; disturb
U+9A13.0	fUTF8	験
U+9A13.0	fGB	E-462D
U+9A13.0	fR/S	 187.8
U+9A13.0	fTotalStrokes	 18
U+9A13.0	fCangjie	XSFOM
U+9A13.0	fCantonese	jim6
U+9A13.0	fMandarin	yan4; yan3
U+9A13.0	fEnglish	 test; examine; inspect verify
U+9A14.0	fUTF8	騔
U+9A14.0	fCNS11643	2-682D
U+9A14.0	fBig5	F3C0
U+9A14.0	fGB	E-462E
U+9A14.0	fR/S	 187.9
U+9A14.0	fTotalStrokes	 19
U+9A14.0	fCangjie	SFAPV
U+9A14.0	fFourCorner	76327
U+9A14.0	fCantonese	got3
U+9A14.0	fMandarin	jie2; ge3
U+9A15.0	fUTF8	騕
U+9A15.0	fCNS11643	2-6822
U+9A15.0	fBig5	F3B5
U+9A15.0	fGB	5-4144
U+9A15.0	fR/S	 187.9
U+9A15.0	fTotalStrokes	 19
U+9A15.0	fCangjie	SFMWV
U+9A15.0	fFourCorner	71344
U+9A15.0	fHakka	 rau5; jau5; jeu5; jau3
U+9A15.0	fCantonese	jiu5; jiu2
U+9A15.0	fMandarin	yao3
U+9A15.0	fEnglish	the name of a fabulous horse
U+9A16.0	fUTF8	騖
U+9A16.0	fCNS11643	1-7951
U+9A16.0	fBig5	C445
U+9A16.0	fGB	1-6670
U+9A16.0	fR/S	 187.9
U+9A16.0	fTotalStrokes	 19
U+9A16.0	fCangjie	NKSQF
U+9A16.0	fFourCorner	18327
U+9A16.0	fHakka	 wu5; mu5; vu5
U+9A16.0	fCantonese	mou6
U+9A16.0	fMandarin	wu4
U+9A16.0	fEnglish	[1] speed; rush [2] unrestrained; uninhibited
U+9A17.0	fUTF8	騗
U+9A17.0	fCNS11643	3-5B6F
U+9A17.0	fGB	3-3E5D
U+9A17.0	fR/S	 187.9
U+9A17.0	fTotalStrokes	 19
U+9A17.0	fCangjie	HBSQF
U+9A17.0	fFourCorner	31227
U+9A17.0	fCantonese	pin3
U+9A17.0	fMandarin	pian4
U+9A17.0	fEnglish	 swindle cheat
U+9A18.0	fUTF8	騘
U+9A18.0	fCNS11643	3-5B6D
U+9A18.0	fGB	E-462F
U+9A18.0	fR/S	 187.9
U+9A18.0	fTotalStrokes	 19
U+9A18.0	fCangjie	SFPKP
U+9A18.0	fFourCorner	77330
U+9A18.0	fCantonese	cung1
U+9A18.0	fMandarin	cong1
U+9A18.0	fEnglish	a piebald horse
U+9A19.0	fUTF8	騙
U+9A19.0	fCNS11643	1-7952
U+9A19.0	fBig5	C446
U+9A19.0	fGB	1-462D
U+9A19.0	fR/S	 187.9
U+9A19.0	fTotalStrokes	 19
U+9A19.0	fCangjie	SFHSB
U+9A19.0	fFourCorner	73327
U+9A19.0	fHakka	 pien5; pen5
U+9A19.0	fCantonese	pin3
U+9A19.0	fMandarin	pian4
U+9A19.0	fEnglish	swindle; cheat out of; defraud
U+9A1A.0	fUTF8	騚
U+9A1A.0	fCNS11643	2-6821
U+9A1A.0	fBig5	F3B4
U+9A1A.0	fGB	E-4630
U+9A1A.0	fR/S	 187.9
U+9A1A.0	fTotalStrokes	 19
U+9A1A.0	fCangjie	SFTBN
U+9A1A.0	fFourCorner	78321
U+9A1A.0	fCantonese	cin4
U+9A1A.0	fMandarin	qian2
U+9A1B.0	fUTF8	騛
U+9A1B.0	fCNS11643	2-6826
U+9A1B.0	fBig5	F3B9
U+9A1B.0	fGB	E-4631
U+9A1B.0	fR/S	 187.9
U+9A1B.0	fTotalStrokes	 19
U+9A1B.0	fCangjie	SFNOO
U+9A1B.0	fFourCorner	72313
U+9A1B.0	fCantonese	fei1
U+9A1B.0	fMandarin	fei1
U+9A1C.0	fUTF8	騜
U+9A1C.0	fCNS11643	2-682C
U+9A1C.0	fBig5	F3BF
U+9A1C.0	fGB	3-3E5A
U+9A1C.0	fR/S	 187.9
U+9A1C.0	fTotalStrokes	 19
U+9A1C.0	fCangjie	SFHAG
U+9A1C.0	fFourCorner	76314
U+9A1C.0	fCantonese	wong4
U+9A1C.0	fMandarin	huang2
U+9A1D.0	fUTF8	騝
U+9A1D.0	fCNS11643	2-6824
U+9A1D.0	fBig5	F3B7
U+9A1D.0	fGB	5-4140
U+9A1D.0	fR/S	 187.9
U+9A1D.0	fTotalStrokes	 19
U+9A1D.0	fCangjie	SFNKQ
U+9A1D.0	fFourCorner	75344
U+9A1D.0	fCantonese	kin4
U+9A1D.0	fMandarin	jian1; qian2
U+9A1E.0	fUTF8	騞
U+9A1E.0	fCNS11643	2-682B
U+9A1E.0	fBig5	F3BE
U+9A1E.0	fGB	3-3E52
U+9A1E.0	fR/S	 187.9
U+9A1E.0	fTotalStrokes	 19
U+9A1E.0	fCangjie	SFQJR
U+9A1E.0	fFourCorner	75361
U+9A1E.0	fCantonese	waak6; faak3
U+9A1E.0	fMandarin	huo4; mo4; huo1; ti1
U+9A1E.0	fEnglish	the sound of a knife cutting something
U+9A1F.0	fUTF8	騟
U+9A1F.0	fCNS11643	3-5B6C
U+9A1F.0	fGB	3-3E5C
U+9A1F.0	fR/S	 187.9
U+9A1F.0	fTotalStrokes	 19
U+9A1F.0	fCangjie	SFOMN
U+9A1F.0	fFourCorner	78320
U+9A1F.0	fCantonese	jyu4
U+9A1F.0	fMandarin	yu1; yu2; yu4
U+9A20.0	fUTF8	騠
U+9A20.0	fCNS11643	2-6828
U+9A20.0	fBig5	F3BB
U+9A20.0	fGB	3-3E56
U+9A20.0	fR/S	 187.9
U+9A20.0	fTotalStrokes	 19
U+9A20.0	fCangjie	SFAMO
U+9A20.0	fFourCorner	76381
U+9A20.0	fHakka	  ti2
U+9A20.0	fCantonese	tai4
U+9A20.0	fMandarin	ti2
U+9A20.0	fEnglish	a hybrid horse produced by mating a donkey with a stallion
U+9A21.0	fUTF8	騡
U+9A21.0	fCNS11643	3-5B6E
U+9A21.0	fGB	E-4632
U+9A21.0	fR/S	 187.9
U+9A21.0	fTotalStrokes	 19
U+9A21.0	fCangjie	SFHAE
U+9A21.0	fFourCorner	76330
U+9A21.0	fHakka	 liuk8
U+9A21.0	fCantonese	cyun1
U+9A21.0	fMandarin	quan2
U+9A22.0	fUTF8	騢
U+9A22.0	fCNS11643	2-6827
U+9A22.0	fBig5	F3BA
U+9A22.0	fGB	3-3E5E
U+9A22.0	fR/S	 187.9
U+9A22.0	fTotalStrokes	 19
U+9A22.0	fCangjie	SFRSE
U+9A22.0	fFourCorner	77347
U+9A22.0	fHakka	 ha2
U+9A22.0	fCantonese	haa4
U+9A22.0	fMandarin	xia2
U+9A22.0	fEnglish	bay
U+9A23.0	fUTF8	騣
U+9A23.0	fCNS11643	2-682A
U+9A23.0	fBig5	F3BD
U+9A23.0	fGB	E-4633
U+9A23.0	fR/S	 187.9
U+9A23.0	fTotalStrokes	 19
U+9A23.0	fCangjie	SFUCE
U+9A23.0	fFourCorner	76347
U+9A23.0	fHakka	  zung1
U+9A23.0	fCantonese	zung1
U+9A23.0	fMandarin	zong1
U+9A23.0	fEnglish	 mane
U+9A24.0	fUTF8	騤
U+9A24.0	fCNS11643	2-6825
U+9A24.0	fBig5	F3B8
U+9A24.0	fGB	3-3E5F
U+9A24.0	fR/S	 187.9
U+9A24.0	fTotalStrokes	 19
U+9A24.0	fCangjie	SFNOK
U+9A24.0	fFourCorner	72334
U+9A24.0	fHakka	 kui2; kwi2; k'wui2
U+9A24.0	fCantonese	kwai4
U+9A24.0	fMandarin	kui2
U+9A24.0	fEnglish	lively (of a horse); vigorous; strong
U+9A25.0	fUTF8	騥
U+9A25.0	fCNS11643	2-6823
U+9A25.0	fBig5	F3B6
U+9A25.0	fGB	E-4634
U+9A25.0	fR/S	 187.9
U+9A25.0	fTotalStrokes	 19
U+9A25.0	fCangjie	SFNHD
U+9A25.0	fFourCorner	77394
U+9A25.0	fCantonese	jau4
U+9A25.0	fMandarin	rou2
U+9A26.0	fUTF8	騦
U+9A26.0	fCNS11643	3-5B6B
U+9A26.0	fGB	5-4146
U+9A26.0	fR/S	  187.9
U+9A26.0	fTotalStrokes	 19
U+9A26.0	fCangjie	SFWP
U+9A26.0	fFourCorner	76330
U+9A26.0	fCantonese	si1
U+9A26.0	fMandarin	si1
U+9A27.0	fUTF8	騧
U+9A27.0	fCNS11643	2-6829
U+9A27.0	fBig5	F3BC
U+9A27.0	fGB	3-3E3F
U+9A27.0	fR/S	 187.9
U+9A27.0	fTotalStrokes	 19
U+9A27.0	fCangjie	SFBBR
U+9A27.0	fFourCorner	77327
U+9A27.0	fHakka	 wa1; wo1; vo1
U+9A27.0	fCantonese	waa1; gwaa1; wo1
U+9A27.0	fMandarin	gua1; wa1
U+9A27.0	fEnglish	tan horse
U+9A28.0	fUTF8	騨
U+9A28.0	fGB	E-4635
U+9A28.0	fR/S	 187.9
U+9A28.0	fTotalStrokes	 19
U+9A28.0	fCangjie	SFFWJ
U+9A28.0	fCantonese	to4
U+9A28.0	fMandarin	tuo2; tan2
U+9A28.0	fEnglish	dappled
U+9A29.0	fUTF8	騩
U+9A29.0	fCNS11643	2-6A6E
U+9A29.0	fBig5	F560
U+9A29.0	fGB	3-3E5B
U+9A29.0	fR/S	 187.10
U+9A29.0	fTotalStrokes	 20
U+9A29.0	fCangjie	SFHUI
U+9A29.0	fFourCorner	76313
U+9A29.0	fCantonese	gwai1; kwai3 (also); gwai3
U+9A29.0	fMandarin	kui4; gui1; tui2
U+9A2A.0	fUTF8	騪
U+9A2A.0	fCNS11643	2-6A6C
U+9A2A.0	fBig5	F55E
U+9A2A.0	fGB	3-3E59
U+9A2A.0	fR/S	 187.10
U+9A2A.0	fTotalStrokes	 20
U+9A2A.0	fCangjie	SFHXE
U+9A2A.0	fFourCorner	77347
U+9A2A.0	fCantonese	sau1
U+9A2A.0	fMandarin	sou1
U+9A2B.0	fUTF8	騫
U+9A2B.0	fCNS11643	1-7A55
U+9A2B.0	fBig5	C4CA
U+9A2B.0	fGB	1-6539
U+9A2B.0	fR/S	 187.10
U+9A2B.0	fTotalStrokes	 20
U+9A2B.0	fCangjie	JTCF
U+9A2B.0	fFourCorner	30327
U+9A2B.0	fHakka	 kien1; ken1; kian1
U+9A2B.0	fCantonese	hin1
U+9A2B.0	fMandarin	qian1; jian3
U+9A2B.0	fEnglish	[1] raise high; hold high; uplift [2] fly; soar; rise [3] pull up [4] frightened
U+9A2C.0	fUTF8	騬
U+9A2C.0	fCNS11643	2-6A6B
U+9A2C.0	fBig5	F55D
U+9A2C.0	fGB	3-3E64
U+9A2C.0	fR/S	 187.10
U+9A2C.0	fTotalStrokes	 20
U+9A2C.0	fCangjie	SFHDP
U+9A2C.0	fFourCorner	72391
U+9A2C.0	fHakka	 shin5; shin1; siin1; siin5
U+9A2C.0	fCantonese	sing4
U+9A2C.0	fMandarin	cheng2; sheng4
U+9A2C.0	fEnglish	 geld a horse
U+9A2D.0	fUTF8	騭
U+9A2D.0	fCNS11643	2-6A71
U+9A2D.0	fBig5	F563
U+9A2D.0	fGB	1-666F
U+9A2D.0	fR/S	 187.10
U+9A2D.0	fTotalStrokes	 20
U+9A2D.0	fCangjie	NHSQF
U+9A2D.0	fFourCorner	79227
U+9A2D.0	fHakka	 zhit7; tsiit7
U+9A2D.0	fCantonese	zat1; zik1
U+9A2D.0	fMandarin	zhi4; zuan1
U+9A2D.0	fEnglish	[1] stallion [2] go up; rise [3] predestined
U+9A2E.0	fUTF8	騮
U+9A2E.0	fCNS11643	2-6A6F
U+9A2E.0	fBig5	F561
U+9A2E.0	fGB	1-6672
U+9A2E.0	fR/S	 187.10
U+9A2E.0	fTotalStrokes	 20
U+9A2E.0	fCangjie	SFHHW
U+9A2E.0	fFourCorner	77362
U+9A2E.0	fHakka	  liu2; liu1
U+9A2E.0	fCantonese	lau4; lau1 (also)
U+9A2E.0	fMandarin	liu2
U+9A2E.0	fEnglish	a legedary fine horse
U+9A2F.0	fUTF8	騯
U+9A2F.0	fCNS11643	3-5D5A
U+9A2F.0	fGB	5-414B
U+9A2F.0	fR/S	 187.10
U+9A2F.0	fTotalStrokes	 20
U+9A2F.0	fCangjie	SFYBS
U+9A2F.0	fFourCorner	70320
U+9A2F.0	fCantonese	paang4
U+9A2F.0	fMandarin	pang2; peng2; bang3
U+9A2F.0	fEnglish	 loud; noisy; boisterous fight
U+9A30.0	fUTF8	騰
U+9A30.0	fCNS11643	1-7A56
U+9A30.0	fBig5	C4CB
U+9A30.0	fGB	1-4C5A
U+9A30.0	fR/S	 187.10
U+9A30.0	fTotalStrokes	 20
U+9A30.0	fCangjie	BFQF
U+9A30.0	fFourCorner	79227
U+9A30.0	fHakka	  ten2;  tin2; ten3
U+9A30.0	fCantonese	tang4
U+9A30.0	fMandarin	teng2
U+9A30.0	fEnglish	[1] prance; rear; leap; jump [2] go up; rise; fly; soar [3] turn over; surrender; transfer
U+9A30.1	fUTF8	騰
U+9A30.1	fCNS11643	1-7A56
U+9A30.1	fBig5	C4CB
U+9A30.1	fGB	1-4C5A
U+9A30.1	fR/S	 187.10
U+9A30.1	fTotalStrokes	 20
U+9A30.1	fCangjie	BFQF
U+9A30.1	fFourCorner	79227
U+9A30.1	fMandarin	teng5
U+9A31.0	fUTF8	騱
U+9A31.0	fCNS11643	2-6A6A
U+9A31.0	fBig5	F55C
U+9A31.0	fGB	3-3E65
U+9A31.0	fR/S	 187.10
U+9A31.0	fTotalStrokes	 20
U+9A31.0	fCangjie	SFBVK
U+9A31.0	fFourCorner	72334
U+9A31.0	fCantonese	hai4
U+9A31.0	fMandarin	xi1; ji1
U+9A32.0	fUTF8	騲
U+9A32.0	fCNS11643	2-6A68
U+9A32.0	fBig5	F55A
U+9A32.0	fGB	3-3E54
U+9A32.0	fR/S	 187.10
U+9A32.0	fTotalStrokes	 20
U+9A32.0	fCangjie	SFTAJ
U+9A32.0	fFourCorner	74346
U+9A32.0	fHakka	 co3;  cau3
U+9A32.0	fCantonese	cou2
U+9A32.0	fMandarin	cao3
U+9A32.0	fEnglish	the female of certain animals-horses; etc.
U+9A33.0	fUTF8	騳
U+9A33.0	fCNS11643	4-6546
U+9A33.0	fGB	5-412B
U+9A33.0	fR/S	  187.10
U+9A33.0	fTotalStrokes	 20
U+9A33.0	fCangjie	SFSQF
U+9A33.0	fFourCorner	71320
U+9A33.0	fCantonese	duk6; ngau6
U+9A33.0	fMandarin	du2
U+9A33.0	fEnglish	run; race
U+9A34.0	fUTF8	騴
U+9A34.0	fCNS11643	2-6A69
U+9A34.0	fBig5	F55B
U+9A34.0	fGB	E-4636
U+9A34.0	fR/S	 187.10
U+9A34.0	fTotalStrokes	 20
U+9A34.0	fCangjie	SFAJV
U+9A34.0	fFourCorner	76344
U+9A34.0	fCantonese	aan3; ngaan3 (also)
U+9A34.0	fMandarin	yan4
U+9A35.0	fUTF8	騵
U+9A35.0	fCNS11643	1-7A58
U+9A35.0	fBig5	C4CD
U+9A35.0	fGB	3-3E62
U+9A35.0	fR/S	 187.10
U+9A35.0	fTotalStrokes	 20
U+9A35.0	fCangjie	SFMHF
U+9A35.0	fFourCorner	71396
U+9A35.0	fHakka	  ngien2
U+9A35.0	fCantonese	jyun4
U+9A35.0	fMandarin	yuan2
U+9A35.0	fEnglish	bay
U+9A36.0	fUTF8	騶
U+9A36.0	fCNS11643	2-6A6D
U+9A36.0	fBig5	F55F
U+9A36.0	fGB	1-6663
U+9A36.0	fR/S	 187.10
U+9A36.0	fTotalStrokes	 20
U+9A36.0	fCangjie	SFPUU
U+9A36.0	fFourCorner	77327
U+9A36.0	fHakka	  zeu1; cu1
U+9A36.0	fCantonese	zau1
U+9A36.0	fMandarin	zou1; zou4; zhou4; zhu1; qu1
U+9A36.0	fEnglish	[1] an official in charge of driving carriages [2] mounted escort
U+9A37.0	fUTF8	騷
U+9A37.0	fCNS11643	1-7A57
U+9A37.0	fBig5	C4CC
U+9A37.0	fGB	1-4927
U+9A37.0	fR/S	 187.10
U+9A37.0	fTotalStrokes	 20
U+9A37.0	fCangjie	SFEII
U+9A37.0	fFourCorner	77336
U+9A37.0	fHakka	  sau1;  zau3; so1
U+9A37.0	fCantonese	sou1
U+9A37.0	fMandarin	sao1; xiao1; sao3; zao4
U+9A37.0	fEnglish	[1] harass; bother; annoy; disturb; agitate [2] worry; feel concerned [3] ill-smellling; stinking [4] amorous; erotic
U+9A38.0	fUTF8	騸
U+9A38.0	fCNS11643	2-6A70
U+9A38.0	fBig5	F562
U+9A38.0	fGB	1-6673
U+9A38.0	fR/S	 187.10
U+9A38.0	fTotalStrokes	 20
U+9A38.0	fCangjie	SFHSM
U+9A38.0	fFourCorner	73327
U+9A38.0	fHakka	 shan5; san5; sen5; shen5
U+9A38.0	fCantonese	sin3
U+9A38.0	fMandarin	shan4
U+9A38.0	fEnglish	geld; castrate a fowl
U+9A39.0	fUTF8	騹
U+9A39.0	fCNS11643	2-6C66
U+9A39.0	fBig5	F678
U+9A39.0	fGB	5-414E
U+9A39.0	fR/S	 187.11
U+9A39.0	fTotalStrokes	 21
U+9A39.0	fCangjie	SFTLM
U+9A39.0	fFourCorner	74314
U+9A39.0	fCantonese	kei4
U+9A39.0	fMandarin	li2; qi2
U+9A3A.0	fUTF8	騺
U+9A3A.0	fCNS11643	2-6C6C
U+9A3A.0	fBig5	F67E
U+9A3A.0	fGB	3-3E30
U+9A3A.0	fR/S	 187.11
U+9A3A.0	fTotalStrokes	 21
U+9A3A.0	fCangjie	GISQF
U+9A3A.0	fFourCorner	49327
U+9A3A.0	fHakka	 zhi5; zih5
U+9A3A.0	fCantonese	zi3
U+9A3A.0	fMandarin	zhi4
U+9A3A.0	fEnglish	 heavy horse horse unable to move because of twisted leg plodding
U+9A3B.0	fUTF8	騻
U+9A3B.0	fCNS11643	3-5F24
U+9A3B.0	fGB	5-414F
U+9A3B.0	fR/S	 187.11
U+9A3B.0	fTotalStrokes	 21
U+9A3B.0	fCangjie	SFKKK
U+9A3B.0	fFourCorner	74330
U+9A3B.0	fCantonese	soeng1
U+9A3B.0	fMandarin	shuang3; shuang1
U+9A3B.0	fEnglish	horse
U+9A3C.0	fUTF8	騼
U+9A3C.0	fCNS11643	3-5F22
U+9A3C.0	fGB	E-4637
U+9A3C.0	fR/S	  187.11
U+9A3C.0	fTotalStrokes	 21
U+9A3C.0	fCangjie	SFIXP
U+9A3C.0	fFourCorner	70310
U+9A3C.0	fCantonese	luk6
U+9A3C.0	fMandarin	lu4
U+9A3D.0	fUTF8	騽
U+9A3D.0	fCNS11643	2-6C67
U+9A3D.0	fBig5	F679
U+9A3D.0	fGB	3-3E69
U+9A3D.0	fR/S	 187.11
U+9A3D.0	fTotalStrokes	 21
U+9A3D.0	fCangjie	SFSMA
U+9A3D.0	fFourCorner	77362
U+9A3D.0	fCantonese	zaap6; jap6
U+9A3D.0	fMandarin	xi2; yu4
U+9A3E.0	fUTF8	騾
U+9A3E.0	fCNS11643	1-7B47
U+9A3E.0	fBig5	C55B
U+9A3E.0	fGB	1-4262
U+9A3E.0	fR/S	 187.11
U+9A3E.0	fTotalStrokes	 21
U+9A3E.0	fCangjie	SFWVF
U+9A3E.0	fFourCorner	76393
U+9A3E.0	fHakka	 lo2; lo3; lui2
U+9A3E.0	fCantonese	lo4 (also); leoi4; leoi6
U+9A3E.0	fMandarin	luo2
U+9A3E.0	fEnglish	mule
U+9A3F.0	fUTF8	騿
U+9A3F.0	fCNS11643	2-6C6D
U+9A3F.0	fBig5	F6A1
U+9A3F.0	fGB	E-4638
U+9A3F.0	fR/S	 187.11
U+9A3F.0	fTotalStrokes	 21
U+9A3F.0	fCangjie	SFYTJ
U+9A3F.0	fFourCorner	70346
U+9A3F.0	fCantonese	zoeng1
U+9A3F.0	fMandarin	zhang1
U+9A40.0	fUTF8	驀
U+9A40.0	fCNS11643	1-7B46
U+9A40.0	fBig5	C55A
U+9A40.0	fGB	1-5D6B
U+9A40.0	fR/S	 187.11
U+9A40.0	fTotalStrokes	 21
U+9A40.0	fCangjie	TAKF
U+9A40.0	fFourCorner	44327
U+9A40.0	fHakka	 met7; ma5; mok7
U+9A40.0	fCantonese	mak6; maak6
U+9A40.0	fMandarin	mo4; ma4
U+9A40.0	fEnglish	[1] sudden; abrupt; suddenly; quickly; abruptly [2] mount a horse
U+9A41.0	fUTF8	驁
U+9A41.0	fCNS11643	2-6C6B
U+9A41.0	fBig5	F67D
U+9A41.0	fGB	1-6671
U+9A41.0	fR/S	 187.11
U+9A41.0	fTotalStrokes	 21
U+9A41.0	fCangjie	GKSQF
U+9A41.0	fFourCorner	58227
U+9A41.0	fHakka	  ngau2
U+9A41.0	fCantonese	ngou4; ou4 (also); ngou6; ou6
U+9A41.0	fMandarin	ao4; ao2; yao4
U+9A41.0	fEnglish	[1] fine horse [2] untamed horse [3] proud; haughty; arrogant [4] indomitable; stubborn
U+9A42.0	fUTF8	驂
U+9A42.0	fCNS11643	2-6C6A
U+9A42.0	fBig5	F67C
U+9A42.0	fGB	1-666E
U+9A42.0	fR/S	 187.11
U+9A42.0	fTotalStrokes	 21
U+9A42.0	fCangjie	SFIIH
U+9A42.0	fFourCorner	73322
U+9A42.0	fHakka	  cam1
U+9A42.0	fCantonese	caam1
U+9A42.0	fMandarin	can1; can3
U+9A42.0	fEnglish	the two outside horses of a team of three horses
U+9A43.0	fUTF8	驃
U+9A43.0	fCNS11643	1-7B45
U+9A43.0	fBig5	C559
U+9A43.0	fGB	1-6674
U+9A43.0	fR/S	 187.11
U+9A43.0	fTotalStrokes	 21
U+9A43.0	fCangjie	SFMWF
U+9A43.0	fFourCorner	71391
U+9A43.0	fHakka	 piau5; piau1; peu5
U+9A43.0	fCantonese	biu1; piu3; biu6; mak6
U+9A43.0	fMandarin	piao4; biao4; biao1
U+9A43.0	fEnglish	[1] horse with a yellowish white color [2] valiant [3] galloping
U+9A44.0	fUTF8	驄
U+9A44.0	fCNS11643	2-6C69
U+9A44.0	fBig5	F67B
U+9A44.0	fGB	1-6675
U+9A44.0	fR/S	 187.11
U+9A44.0	fTotalStrokes	 21
U+9A44.0	fCangjie	SFHWP
U+9A44.0	fFourCorner	76330
U+9A44.0	fHakka	  cung1
U+9A44.0	fCantonese	cung1
U+9A44.0	fMandarin	cong1
U+9A44.0	fEnglish	a horse with a bluish white color
U+9A45.0	fUTF8	驅
U+9A45.0	fCNS11643	1-7B44
U+9A45.0	fBig5	C558
U+9A45.0	fGB	1-477D
U+9A45.0	fR/S	 187.11
U+9A45.0	fTotalStrokes	 21
U+9A45.0	fCangjie	SFSRR
U+9A45.0	fFourCorner	71316
U+9A45.0	fHakka	  ki1
U+9A45.0	fCantonese	keoi1
U+9A45.0	fMandarin	qu1; qiu1; xu3
U+9A45.0	fEnglish	[1] go before others [2] drive; urge [3] expel; drive away [4] command
U+9A46.0	fUTF8	驆
U+9A46.0	fCNS11643	2-6C68
U+9A46.0	fBig5	F67A
U+9A46.0	fGB	E-4639
U+9A46.0	fR/S	 187.11
U+9A46.0	fTotalStrokes	 21
U+9A46.0	fCangjie	SFWTJ
U+9A46.0	fFourCorner	76344
U+9A46.0	fCantonese	bit1; bat1
U+9A46.0	fMandarin	bi4
U+9A46.0	fEnglish	used in transliterating Buddhist books
U+9A47.0	fUTF8	驇
U+9A47.0	fCNS11643	7-5651
U+9A47.0	fGB	E-463A
U+9A47.0	fR/S	 187.11
U+9A47.0	fTotalStrokes	 21
U+9A47.0	fCangjie	GISQF
U+9A47.0	fFourCorner	45327
U+9A47.0	fCantonese	ngai6; ai6 (also)
U+9A47.0	fMandarin	zhi4
U+9A47.0	fEnglish	 heavy horse horse unable to move because of twisted leg plodding
U+9A48.0	fUTF8	驈
U+9A48.0	fCNS11643	2-6E4C
U+9A48.0	fBig5	F77D
U+9A48.0	fGB	3-3E6E
U+9A48.0	fR/S	 187.12
U+9A48.0	fTotalStrokes	 22
U+9A48.0	fCangjie	SFNHB
U+9A48.0	fFourCorner	77327
U+9A48.0	fHakka	 lut8; rut8; jut8
U+9A48.0	fCantonese	wat6; leot6; jyut6
U+9A48.0	fMandarin	yu4; shu4; xue4
U+9A48.0	fEnglish	black horses
U+9A49.0	fUTF8	驉
U+9A49.0	fCNS11643	2-6E4E
U+9A49.0	fBig5	F7A1
U+9A49.0	fGB	3-3E68
U+9A49.0	fR/S	 187.12
U+9A49.0	fTotalStrokes	 22
U+9A49.0	fCangjie	SFYPM
U+9A49.0	fFourCorner	71317
U+9A49.0	fCantonese	heoi1
U+9A49.0	fMandarin	xu1
U+9A4A.0	fUTF8	驊
U+9A4A.0	fCNS11643	2-6E4D
U+9A4A.0	fBig5	F77E
U+9A4A.0	fGB	1-6668
U+9A4A.0	fR/S	 187.12; 187.10
U+9A4A.0	fTotalStrokes	 22
U+9A4A.0	fCangjie	SFTMJ
U+9A4A.0	fFourCorner	74354
U+9A4A.0	fHakka	 fa2
U+9A4A.0	fCantonese	waa4
U+9A4A.0	fMandarin	hua2
U+9A4A.0	fEnglish	the name of an legendary excellent horse
U+9A4B.0	fUTF8	驋
U+9A4B.0	fCNS11643	4-696A
U+9A4B.0	fGB	3-3E2E
U+9A4B.0	fR/S	  187.12
U+9A4B.0	fTotalStrokes	 22; 21
U+9A4B.0	fCangjie	SFNOE
U+9A4B.0	fFourCorner	72340
U+9A4B.0	fCantonese	but3
U+9A4B.0	fMandarin	bo1
U+9A4C.0	fUTF8	驌
U+9A4C.0	fCNS11643	2-6E4A
U+9A4C.0	fBig5	F77B
U+9A4C.0	fGB	3-3E50
U+9A4C.0	fR/S	 187.12; 187.13
U+9A4C.0	fTotalStrokes	 22
U+9A4C.0	fCangjie	SFLX
U+9A4C.0	fFourCorner	75327
U+9A4C.0	fHakka	 suk7; siuk7
U+9A4C.0	fCantonese	suk1
U+9A4C.0	fMandarin	su4
U+9A4C.0	fEnglish	the name of a famous horse
U+9A4D.0	fUTF8	驍
U+9A4D.0	fCNS11643	1-7C27
U+9A4D.0	fBig5	C5BB
U+9A4D.0	fGB	1-6667
U+9A4D.0	fR/S	 187.12
U+9A4D.0	fTotalStrokes	 22
U+9A4D.0	fCangjie	SFGGU
U+9A4D.0	fFourCorner	74311
U+9A4D.0	fHakka	  hiau1; hieu1
U+9A4D.0	fCantonese	hiu1
U+9A4D.0	fMandarin	xiao1
U+9A4D.0	fEnglish	[1] excellent horse [2] having courage and agility; brave; valiant
U+9A4E.0	fUTF8	驎
U+9A4E.0	fCNS11643	2-6E47
U+9A4E.0	fBig5	F778
U+9A4E.0	fGB	3-3E6C
U+9A4E.0	fR/S	 187.12
U+9A4E.0	fTotalStrokes	 22
U+9A4E.0	fCangjie	SFFDQ
U+9A4E.0	fFourCorner	79359
U+9A4E.0	fHakka	  lin2; lin5; nin2
U+9A4E.0	fCantonese	leon4
U+9A4E.0	fMandarin	lin2; lin4
U+9A4E.0	fEnglish	piebald (horses)
U+9A4F.0	fUTF8	驏
U+9A4F.0	fCNS11643	2-6E4B
U+9A4F.0	fBig5	F77C
U+9A4F.0	fGB	1-6676
U+9A4F.0	fR/S	 187.12
U+9A4F.0	fTotalStrokes	 22
U+9A4F.0	fCangjie	SFSND
U+9A4F.0	fFourCorner	77347
U+9A4F.0	fCantonese	zaan2; can2; cin5
U+9A4F.0	fMandarin	zhan4; chan3
U+9A4F.0	fEnglish	horse without saddle
U+9A50.0	fUTF8	驐
U+9A50.0	fCNS11643	2-6E50
U+9A50.0	fBig5	F7A3
U+9A50.0	fGB	3-3E6B
U+9A50.0	fR/S	 187.12
U+9A50.0	fTotalStrokes	 22
U+9A50.0	fCangjie	SFYDK
U+9A50.0	fFourCorner	78340
U+9A50.0	fHakka	  dun1
U+9A50.0	fCantonese	deon1
U+9A50.0	fMandarin	dun1
U+9A50.0	fEnglish	 castrate an animal
U+9A51.0	fUTF8	驑
U+9A51.0	fCNS11643	3-5F79
U+9A51.0	fGB	E-463B
U+9A51.0	fR/S	 187.12
U+9A51.0	fTotalStrokes	 22
U+9A51.0	fCangjie	SFMLW
U+9A51.0	fFourCorner	71360
U+9A51.0	fCantonese	lau4
U+9A51.0	fMandarin	liu2
U+9A51.0	fEnglish	the name of a legendary fine horse
U+9A52.0	fUTF8	驒
U+9A52.0	fCNS11643	2-6E4F
U+9A52.0	fBig5	F7A2
U+9A52.0	fGB	3-3E4F
U+9A52.0	fR/S	 187.12
U+9A52.0	fTotalStrokes	 22
U+9A52.0	fCangjie	SFRRJ
U+9A52.0	fFourCorner	76356
U+9A52.0	fHakka	  to2
U+9A52.0	fCantonese	to4; taan4 (also); cang4; tim5; giu1; tit3; jim6; leoi4; lo4 (also); zin1; ging1; geng1 (also); jik6; jip6; tok3
U+9A52.0	fMandarin	tuo2; di1; dian1; tan2
U+9A52.0	fEnglish	dappled
U+9A53.0	fUTF8	驓
U+9A53.0	fCNS11643	2-6E48
U+9A53.0	fBig5	F779
U+9A53.0	fGB	E-463C
U+9A53.0	fR/S	 187.12
U+9A53.0	fTotalStrokes	 22
U+9A53.0	fCangjie	SFCWA
U+9A53.0	fFourCorner	78366
U+9A53.0	fCantonese	cang4
U+9A53.0	fMandarin	zeng1; zeng4
U+9A54.0	fUTF8	驔
U+9A54.0	fCNS11643	2-6E49
U+9A54.0	fBig5	F77A
U+9A54.0	fGB	3-3E6A
U+9A54.0	fR/S	 187.12
U+9A54.0	fTotalStrokes	 22
U+9A54.0	fCangjie	SFMWJ
U+9A54.0	fFourCorner	71346
U+9A54.0	fHakka	 tam2
U+9A54.0	fCantonese	tim5; taam4
U+9A54.0	fMandarin	tan2; dian3; dian4
U+9A54.0	fEnglish	black horse
U+9A55.0	fUTF8	驕
U+9A55.0	fCNS11643	1-7C26
U+9A55.0	fBig5	C5BA
U+9A55.0	fGB	1-3D3E
U+9A55.0	fR/S	 187.12
U+9A55.0	fTotalStrokes	 22
U+9A55.0	fCangjie	SFHKB
U+9A55.0	fFourCorner	72327
U+9A55.0	fHakka	  giau1; kieu1
U+9A55.0	fCantonese	giu1
U+9A55.0	fMandarin	jiao1; jiao3; xiao1; ju1; qiao2
U+9A55.0	fEnglish	[1] untamed; intractable; disobedient [2] proud; haughty; overbearingl arrogant [3] severe; harsh; intense
U+9A56.0	fUTF8	驖
U+9A56.0	fCNS11643	2-6F60
U+9A56.0	fBig5	F852
U+9A56.0	fGB	3-3E6F
U+9A56.0	fR/S	 187.13
U+9A56.0	fTotalStrokes	 23
U+9A56.0	fCangjie	SFJIG
U+9A56.0	fFourCorner	73350
U+9A56.0	fHakka	 tiet7
U+9A56.0	fCantonese	tit3
U+9A56.0	fMandarin	tie3
U+9A56.0	fEnglish	black (horse)
U+9A57.0	fUTF8	驗
U+9A57.0	fCNS11643	1-7C53
U+9A57.0	fBig5	C5E7
U+9A57.0	fGB	1-5169
U+9A57.0	fR/S	 187.13
U+9A57.0	fTotalStrokes	 23
U+9A57.0	fCangjie	SFOMO
U+9A57.0	fFourCorner	78336
U+9A57.0	fHakka	 ngiam5; niam5; ngiam3; ngiam6
U+9A57.0	fCantonese	jim6
U+9A57.0	fMandarin	yan4
U+9A57.0	fEnglish	[1] test; examine; analyse [2] produce an effect [3] verify; prove
U+9A58.0	fUTF8	驘
U+9A58.0	fCNS11643	3-6062
U+9A58.0	fGB	E-463D
U+9A58.0	fR/S	 187.13
U+9A58.0	fTotalStrokes	 23
U+9A58.0	fCangjie	YNSQF
U+9A58.0	fFourCorner	00210
U+9A58.0	fHakka	  lo2
U+9A58.0	fCantonese	lo4
U+9A58.0	fMandarin	luo2
U+9A58.0	fEnglish	mule
U+9A59.0	fUTF8	驙
U+9A59.0	fCNS11643	2-6F61
U+9A59.0	fBig5	F853
U+9A59.0	fGB	3-3E71
U+9A59.0	fR/S	 187.13
U+9A59.0	fTotalStrokes	 23
U+9A59.0	fCangjie	SFYWM
U+9A59.0	fFourCorner	70316
U+9A59.0	fHakka	 zhan1; zan1; zen1; zhen1
U+9A59.0	fCantonese	zin1; taan4
U+9A59.0	fMandarin	zhan1; tan2
U+9A59.0	fEnglish	a white horse with a black back; unicorn
U+9A5A.0	fUTF8	驚
U+9A5A.0	fCNS11643	1-7C51
U+9A5A.0	fBig5	C5E5
U+9A5A.0	fGB	1-3E2A
U+9A5A.0	fR/S	 187.13
U+9A5A.0	fTotalStrokes	 23
U+9A5A.0	fCangjie	TKSQF
U+9A5A.0	fFourCorner	48327
U+9A5A.0	fHakka	  giang1
U+9A5A.0	fCantonese	geng1; ging1
U+9A5A.0	fMandarin	jing1
U+9A5A.0	fEnglish	frighten; surprise; startle
U+9A5B.0	fUTF8	驛
U+9A5B.0	fCNS11643	1-7C52
U+9A5B.0	fBig5	C5E6
U+9A5B.0	fGB	1-6664
U+9A5B.0	fR/S	 187.13
U+9A5B.0	fTotalStrokes	 23
U+9A5B.0	fCangjie	SFWLJ
U+9A5B.0	fFourCorner	76341
U+9A5B.0	fHakka	  jit8; rit8
U+9A5B.0	fCantonese	jik6
U+9A5B.0	fMandarin	yi4
U+9A5B.0	fEnglish	 courier station; a station where couriers rested in former times
U+9A5C.0	fUTF8	驜
U+9A5C.0	fCNS11643	3-6061
U+9A5C.0	fGB	3-3E25
U+9A5C.0	fR/S	 187.13
U+9A5C.0	fTotalStrokes	 23
U+9A5C.0	fCangjie	SFTCD
U+9A5C.0	fFourCorner	72390
U+9A5C.0	fMandarin	ye4
U+9A5D.0	fUTF8	驝
U+9A5D.0	fCNS11643	4-6C3D
U+9A5D.0	fGB	5-4152
U+9A5D.0	fR/S	 187.14
U+9A5D.0	fTotalStrokes	 24
U+9A5D.0	fCangjie	SFGBD
U+9A5D.0	fFourCorner	74390
U+9A5D.0	fCantonese	tok6
U+9A5D.0	fMandarin	tuo1; zhe2
U+9A5D.0	fEnglish	a camel; the name of an office
U+9A5E.0	fUTF8	驞
U+9A5E.0	fCNS11643	2-7061
U+9A5E.0	fBig5	F8D3
U+9A5E.0	fGB	E-463E
U+9A5E.0	fR/S	 187.14
U+9A5E.0	fTotalStrokes	 24
U+9A5E.0	fCangjie	SFJMC
U+9A5E.0	fFourCorner	73386
U+9A5E.0	fCantonese	pan1; ban1
U+9A5E.0	fMandarin	bin1
U+9A5F.0	fUTF8	驟
U+9A5F.0	fCNS11643	1-7C75
U+9A5F.0	fBig5	C64A
U+9A5F.0	fGB	1-5668
U+9A5F.0	fR/S	 187.14
U+9A5F.0	fTotalStrokes	 24
U+9A5F.0	fCangjie	SFSEO
U+9A5F.0	fFourCorner	77332
U+9A5F.0	fHakka	 ceu3; ciu3
U+9A5F.0	fCantonese	zaau6; zau6 (also)
U+9A5F.0	fMandarin	zhou4; zou4
U+9A5F.0	fEnglish	[1] gallop [2] swift; sudden; suddenly [3] frequent
U+9A60.0	fUTF8	驠
U+9A60.0	fCNS11643	2-7165
U+9A60.0	fBig5	F976
U+9A60.0	fGB	E-463F
U+9A60.0	fR/S	 187.16
U+9A60.0	fTotalStrokes	 26
U+9A60.0	fCangjie	SFTLF
U+9A60.0	fFourCorner	74331
U+9A60.0	fCantonese	jin3; jin1
U+9A60.0	fMandarin	yan4; yan1
U+9A61.0	fUTF8	驡
U+9A61.0	fCNS11643	3-6172
U+9A61.0	fGB	3-5878
U+9A61.0	fR/S	  187.16
U+9A61.0	fTotalStrokes	 26
U+9A61.0	fCangjie	YPSQF
U+9A61.0	fFourCorner	01320
U+9A61.0	fCantonese	lung4
U+9A61.0	fMandarin	long2; peng2; zang3
U+9A62.0	fUTF8	驢
U+9A62.0	fCNS11643	1-7D37
U+9A62.0	fBig5	C66A
U+9A62.0	fGB	1-423F
U+9A62.0	fR/S	 187.16
U+9A62.0	fTotalStrokes	 26
U+9A62.0	fCangjie	SFYPT
U+9A62.0	fFourCorner	71317
U+9A62.0	fHakka	  lu2; li2; lu3
U+9A62.0	fCantonese	lou4; leoi4 (also); leoi2 (also)
U+9A62.0	fMandarin	l2; luu2; lu2
U+9A62.0	fEnglish	 donkey; ass
U+9A63.0	fUTF8	驣
U+9A63.0	fCNS11643	3-6173
U+9A63.0	fGB	E-4640
U+9A63.0	fR/S	 187.16
U+9A63.0	fTotalStrokes	 26
U+9A63.0	fCangjie	SFFQF
U+9A63.0	fFourCorner	79320
U+9A63.0	fCantonese	tang4
U+9A63.0	fMandarin	teng2
U+9A64.0	fUTF8	驤
U+9A64.0	fCNS11643	2-7223
U+9A64.0	fBig5	F9B3
U+9A64.0	fGB	1-6678
U+9A64.0	fR/S	 187.17
U+9A64.0	fTotalStrokes	 27
U+9A64.0	fCangjie	SFYRV
U+9A64.0	fFourCorner	70332
U+9A64.0	fHakka	 siong1
U+9A64.0	fCantonese	soeng1
U+9A64.0	fMandarin	xiang1
U+9A64.0	fEnglish	[1] horse with the right hind leg white [2] gallop about with a raised head [3] uplift
U+9A65.0	fUTF8	驥
U+9A65.0	fCNS11643	1-7D38
U+9A65.0	fBig5	C66B
U+9A65.0	fGB	1-6677
U+9A65.0	fR/S	 187.17
U+9A65.0	fTotalStrokes	 27
U+9A65.0	fCangjie	SFLPC
U+9A65.0	fFourCorner	71381
U+9A65.0	fHakka	 gi3; ki3
U+9A65.0	fCantonese	kei3; gei3
U+9A65.0	fMandarin	ji4
U+9A65.0	fEnglish	[1] very fast horse [2] man of outstanding ability; great man
U+9A66.0	fUTF8	驦
U+9A66.0	fCNS11643	2-7224
U+9A66.0	fBig5	F9B4
U+9A66.0	fGB	3-3E72
U+9A66.0	fR/S	 187.17
U+9A66.0	fTotalStrokes	 27
U+9A66.0	fCangjie	SFMBU
U+9A66.0	fFourCorner	71362
U+9A66.0	fHakka	 song1
U+9A66.0	fCantonese	soeng1
U+9A66.0	fMandarin	shuang1; shuang3
U+9A66.0	fEnglish	horse
U+9A67.0	fUTF8	驧
U+9A67.0	fCNS11643	2-7225
U+9A67.0	fBig5	F9B5
U+9A67.0	fGB	5-4156
U+9A67.0	fR/S	 187.17
U+9A67.0	fTotalStrokes	 27
U+9A67.0	fCangjie	SFTJD
U+9A67.0	fFourCorner	77320
U+9A67.0	fCantonese	guk6; guk1
U+9A67.0	fMandarin	ju2
U+9A68.0	fUTF8	驨
U+9A68.0	fCNS11643	2-7233
U+9A68.0	fBig5	F9C3
U+9A68.0	fGB	5-4158
U+9A68.0	fR/S	 187.18
U+9A68.0	fTotalStrokes	 28
U+9A68.0	fCangjie	SFUOB
U+9A68.0	fFourCorner	72327
U+9A68.0	fCantonese	kwai4
U+9A68.0	fMandarin	xi1
U+9A69.0	fUTF8	驩
U+9A69.0	fCNS11643	2-7232
U+9A69.0	fBig5	F9C2
U+9A69.0	fGB	E-4641
U+9A69.0	fR/S	 187.18
U+9A69.0	fTotalStrokes	 28
U+9A69.0	fCangjie	SFTRG
U+9A69.0	fFourCorner	74314
U+9A69.0	fHakka	  fon1
U+9A69.0	fCantonese	fun1
U+9A69.0	fMandarin	huan1
U+9A69.0	fEnglish	 have joy; enjoy
U+9A6A.0	fUTF8	驪
U+9A6A.0	fCNS11643	1-7D47
U+9A6A.0	fBig5	C67A
U+9A6A.0	fGB	1-666A
U+9A6A.0	fR/S	 187.19
U+9A6A.0	fTotalStrokes	 29
U+9A6A.0	fCangjie	SFMMP
U+9A6A.0	fFourCorner	71311
U+9A6A.0	fHakka	 li2; li3
U+9A6A.0	fCantonese	lei4
U+9A6A.0	fMandarin	li2; chi2
U+9A6A.0	fEnglish	[1] pure black horse [2] drive a carriage drawn by two horses
U+9A6B.0	fUTF8	驫
U+9A6B.0	fCNS11643	2-723B
U+9A6B.0	fBig5	F9CD
U+9A6B.0	fGB	3-3E37
U+9A6B.0	fR/S	 187.20
U+9A6B.0	fTotalStrokes	 30
U+9A6B.0	fCangjie	SFSFF
U+9A6B.0	fFourCorner	71327
U+9A6B.0	fHakka	 biau1; beu1
U+9A6B.0	fCantonese	biu1
U+9A6B.0	fMandarin	biao1; an4
U+9A6B.0	fEnglish	[1] horses [2] run; high speed
U+9A6C.0	fUTF8	马
U+9A6C.0	fGB	0-426D
U+9A6C.0	fR/S	 187.0
U+9A6C.0	fTotalStrokes	 3
U+9A6C.0	fCangjie	NVSM
U+9A6C.0	fHakka	 ma1; ma3
U+9A6C.0	fCantonese	maa5; maa1 (also)
U+9A6C.0	fMandarin	ma3
U+9A6C.0	fEnglish	simplified form of &#x99AC;
U+9A6D.0	fUTF8	驭
U+9A6D.0	fGB	0-5426
U+9A6D.0	fR/S	 187.2
U+9A6D.0	fTotalStrokes	 5
U+9A6D.0	fCangjie	NME
U+9A6D.0	fHakka	 ngi5
U+9A6D.0	fCantonese	jyu6; to4; do6
U+9A6D.0	fMandarin	yu4
U+9A6D.0	fEnglish	simplified form of &#x99AD;
U+9A6E.0	fUTF8	驮
U+9A6E.0	fGB	0-4D54
U+9A6E.0	fR/S	 187.3
U+9A6E.0	fTotalStrokes	 6
U+9A6E.0	fCangjie	NMK
U+9A6E.0	fHakka	 to2
U+9A6E.0	fCantonese	to4; do6
U+9A6E.0	fMandarin	tuo2; duo4
U+9A6E.0	fEnglish	simplified form of &#x99B1;
U+9A6F.0	fUTF8	驯
U+9A6F.0	fGB	0-5131
U+9A6F.0	fR/S	 187.3
U+9A6F.0	fTotalStrokes	 6
U+9A6F.0	fCangjie	NMLLL
U+9A6F.0	fHakka	 shun2
U+9A6F.0	fCantonese	seon4; ceon4 (also); ci4; jat6; bok3; lou4; leoi4 (also); keoi1; leoi2 (also); zou2; zong2; cong2; sai2; si3; fu6
U+9A6F.0	fMandarin	xun2; xun4; yun2
U+9A6F.0	fEnglish	simplified form of &#x99B4;
U+9A70.0	fUTF8	驰
U+9A70.0	fGB	0-335B
U+9A70.0	fR/S	 187.3
U+9A70.0	fTotalStrokes	 6
U+9A70.0	fCangjie	NMPD
U+9A70.0	fHakka	 chi1
U+9A70.0	fCantonese	ci4
U+9A70.0	fMandarin	chi2; tuo2
U+9A70.0	fEnglish	simplified form of &#x99B3;
U+9A71.0	fUTF8	驱
U+9A71.0	fGB	0-477D
U+9A71.0	fR/S	 187.4
U+9A71.0	fTotalStrokes	 7
U+9A71.0	fCangjie	NMSK
U+9A71.0	fHakka	 ki1
U+9A71.0	fCantonese	keoi1
U+9A71.0	fMandarin	qu1; qiu1; xu3
U+9A71.0	fEnglish	simplified form of &#x9A45;
U+9A72.0	fUTF8	驲
U+9A72.0	fGB	8-7B40
U+9A72.0	fR/S	 187.4
U+9A72.0	fTotalStrokes	 7
U+9A72.0	fCangjie	NMA
U+9A72.0	fCantonese	jat6
U+9A72.0	fMandarin	ri4
U+9A72.0	fEnglish	post horse, relay horse
U+9A73.0	fUTF8	驳
U+9A73.0	fGB	0-3235
U+9A73.0	fR/S	 187.4
U+9A73.0	fTotalStrokes	 7
U+9A73.0	fCangjie	NMKK
U+9A73.0	fHakka	 bok7
U+9A73.0	fCantonese	bok3
U+9A73.0	fMandarin	bo2
U+9A73.0	fEnglish	simplified form of &#x99C1;
U+9A74.0	fUTF8	驴
U+9A74.0	fGB	0-423F
U+9A74.0	fR/S	 187.4
U+9A74.0	fTotalStrokes	 7
U+9A74.0	fCangjie	NMIS
U+9A74.0	fHakka	 lu2; li2
U+9A74.0	fCantonese	leoi4; lou4
U+9A74.0	fMandarin	l2; luu2; lu2
U+9A74.0	fEnglish	simplified form of &#x9A62;
U+9A75.0	fUTF8	驵
U+9A75.0	fGB	0-6660
U+9A75.0	fR/S	 187.5
U+9A75.0	fTotalStrokes	 8
U+9A75.0	fCangjie	NMBM
U+9A75.0	fHakka	 cong3; zih3
U+9A75.0	fCantonese	zong2; zou2; cong2
U+9A75.0	fMandarin	zang3; cu1; zu3; chu2; zu4
U+9A75.0	fEnglish	simplified form of &#x99D4;
U+9A76.0	fUTF8	驶
U+9A76.0	fGB	0-4A3B
U+9A76.0	fR/S	 187.5
U+9A76.0	fTotalStrokes	 8
U+9A76.0	fCangjie	NMLK
U+9A76.0	fHakka	 sii3
U+9A76.0	fCantonese	sai2
U+9A76.0	fMandarin	shi3; shi4
U+9A76.0	fEnglish	simplified form of &#x99DB;
U+9A77.0	fUTF8	驷
U+9A77.0	fGB	0-6661
U+9A77.0	fR/S	 187.5
U+9A77.0	fTotalStrokes	 8
U+9A77.0	fCangjie	NMWC
U+9A77.0	fHakka	 si5
U+9A77.0	fCantonese	si3
U+9A77.0	fMandarin	si4; xi1
U+9A77.0	fEnglish	simplified form of &#x99DF;
U+9A78.0	fUTF8	驸
U+9A78.0	fGB	0-6662
U+9A78.0	fR/S	 187.5
U+9A78.0	fTotalStrokes	 8
U+9A78.0	fCangjie	NMODI
U+9A78.0	fHakka	 fu5
U+9A78.0	fCantonese	fu6
U+9A78.0	fMandarin	fu4
U+9A78.0	fEnglish	simplified form of &#x99D9;
U+9A79.0	fUTF8	驹
U+9A79.0	fGB	0-3E54
U+9A79.0	fR/S	 187.5
U+9A79.0	fTotalStrokes	 8
U+9A79.0	fCangjie	NMPR
U+9A79.0	fHakka	 gi1; ki1
U+9A79.0	fCantonese	keoi1
U+9A79.0	fMandarin	ju1; ju4
U+9A79.0	fEnglish	simplified form of &#x99D2;
U+9A7A.0	fUTF8	驺
U+9A7A.0	fGB	0-6663
U+9A7A.0	fR/S	 187.5
U+9A7A.0	fTotalStrokes	 8
U+9A7A.0	fCangjie	NMNSM
U+9A7A.0	fHakka	 zeu1
U+9A7A.0	fCantonese	zau1
U+9A7A.0	fMandarin	zou1; zou4; zhou4
U+9A7A.0	fEnglish	simplified form of &#x9A36;
U+9A7B.0	fUTF8	驻
U+9A7B.0	fGB	0-5724
U+9A7B.0	fR/S	 187.5
U+9A7B.0	fTotalStrokes	 8
U+9A7B.0	fCangjie	NMYG
U+9A7B.0	fHakka	 zhu5
U+9A7B.0	fCantonese	zyu3
U+9A7B.0	fMandarin	zhu4
U+9A7B.0	fEnglish	simplified form of &#x99D0;
U+9A7C.0	fUTF8	驼
U+9A7C.0	fGB	0-4D55
U+9A7C.0	fR/S	 187.5
U+9A7C.0	fTotalStrokes	 8
U+9A7C.0	fCangjie	NMJP
U+9A7C.0	fHakka	 to2
U+9A7C.0	fCantonese	to4; to2 (also)
U+9A7C.0	fMandarin	tuo2
U+9A7C.0	fEnglish	simplified form of &#x99DD;
U+9A7C.1	fUTF8	驼
U+9A7C.1	fGB	0-4D55
U+9A7C.1	fR/S	 187.5
U+9A7C.1	fTotalStrokes	 8
U+9A7C.1	fCangjie	NMJP
U+9A7D.0	fUTF8	驽
U+9A7D.0	fGB	0-6665
U+9A7D.0	fR/S	 187.5
U+9A7D.0	fTotalStrokes	 8
U+9A7D.0	fCangjie	VENVM
U+9A7D.0	fHakka	 nu2
U+9A7D.0	fCantonese	nou4; lou4 (also)
U+9A7D.0	fMandarin	nu2
U+9A7D.0	fEnglish	simplified form of &#x99D1;
U+9A7E.0	fUTF8	驾
U+9A7E.0	fGB	0-3C5D
U+9A7E.0	fR/S	 187.5
U+9A7E.0	fTotalStrokes	 8
U+9A7E.0	fCangjie	KRNVM
U+9A7E.0	fHakka	 ga5
U+9A7E.0	fCantonese	gaa3; gaa2 (also)
U+9A7E.0	fMandarin	jia4; jia1
U+9A7E.0	fEnglish	simplified form of &#x99D5;
U+9A7F.0	fUTF8	驿
U+9A7F.0	fGB	0-6664
U+9A7F.0	fR/S	 187.5
U+9A7F.0	fTotalStrokes	 8
U+9A7F.0	fCangjie	NMEQ
U+9A7F.0	fHakka	 jit8
U+9A7F.0	fCantonese	jik6
U+9A7F.0	fMandarin	yi4
U+9A7F.0	fEnglish	simplified form of &#x9A5B;
U+9A80.0	fUTF8	骀
U+9A80.0	fGB	0-6666
U+9A80.0	fR/S	 187.5
U+9A80.0	fTotalStrokes	 8
U+9A80.0	fCangjie	NMIR
U+9A80.0	fHakka	 toi2
U+9A80.0	fCantonese	toi4; toi5
U+9A80.0	fMandarin	tai2; dai4
U+9A80.0	fEnglish	simplified form of &#x99D8;
U+9A81.0	fUTF8	骁
U+9A81.0	fGB	0-6667
U+9A81.0	fR/S	 187.6
U+9A81.0	fTotalStrokes	 9
U+9A81.0	fCangjie	NMJPU
U+9A81.0	fHakka	 hiau1
U+9A81.0	fCantonese	hiu1
U+9A81.0	fMandarin	xiao1
U+9A81.0	fEnglish	simplified form of &#x9A4D;
U+9A82.0	fUTF8	骂
U+9A82.0	fGB	0-426E
U+9A82.0	fR/S	 30.6; 187.6
U+9A82.0	fTotalStrokes	 9
U+9A82.0	fCangjie	RRNVM
U+9A82.0	fCantonese	maa6; maa3 (also)
U+9A82.0	fMandarin	ma4
U+9A82.0	fEnglish	simplified form of &#x99E1;
U+9A83.0	fUTF8	骃
U+9A83.0	fGB	8-7B41
U+9A83.0	fR/S	 187.6
U+9A83.0	fTotalStrokes	 9
U+9A83.0	fCangjie	NMWK
U+9A83.0	fCantonese	jan1
U+9A83.0	fMandarin	yin1
U+9A83.0	fEnglish	grey horse
U+9A84.0	fUTF8	骄
U+9A84.0	fGB	0-3D3E
U+9A84.0	fR/S	 187.6
U+9A84.0	fTotalStrokes	 9
U+9A84.0	fCangjie	NMHKL
U+9A84.0	fHakka	 giau1
U+9A84.0	fCantonese	giu1
U+9A84.0	fMandarin	jiao1; jiao3; xiao1
U+9A84.0	fEnglish	simplified form of &#x9A55;
U+9A85.0	fUTF8	骅
U+9A85.0	fGB	0-6668
U+9A85.0	fR/S	 187.6
U+9A85.0	fTotalStrokes	 9
U+9A85.0	fCangjie	NMOPJ
U+9A85.0	fHakka	 fa2
U+9A85.0	fCantonese	waa4
U+9A85.0	fMandarin	hua2
U+9A85.0	fEnglish	simplified form of &#x9A4A;
U+9A86.0	fUTF8	骆
U+9A86.0	fGB	0-4266
U+9A86.0	fR/S	 187.6
U+9A86.0	fTotalStrokes	 9
U+9A86.0	fCangjie	NMHER
U+9A86.0	fHakka	 lok8
U+9A86.0	fCantonese	lok3; lok6 (also)
U+9A86.0	fMandarin	luo4
U+9A86.0	fEnglish	simplified form of &#x99F1;
U+9A87.0	fUTF8	骇
U+9A87.0	fGB	0-3A27
U+9A87.0	fR/S	 187.6
U+9A87.0	fTotalStrokes	 9
U+9A87.0	fCangjie	NMYVO
U+9A87.0	fHakka	 hai2; hai5
U+9A87.0	fCantonese	haai5; hoi5 (also)
U+9A87.0	fMandarin	hai4; xie4
U+9A87.0	fEnglish	simplified form of &#x99ED;
U+9A88.0	fUTF8	骈
U+9A88.0	fGB	0-6669
U+9A88.0	fR/S	 187.6
U+9A88.0	fTotalStrokes	 9
U+9A88.0	fCangjie	NMTT
U+9A88.0	fHakka	 pin2; bin5
U+9A88.0	fCantonese	pin4; ping4 (also)
U+9A88.0	fMandarin	pian2
U+9A88.0	fEnglish	simplified form of &#x99E2;
U+9A89.0	fUTF8	骉
U+9A89.0	fGB	8-7B42
U+9A89.0	fR/S	 187.6
U+9A89.0	fTotalStrokes	 9
U+9A89.0	fCangjie	NMNMM
U+9A89.0	fCantonese	biu1; lei4; cing2; ping3 (also); jim6; sing1; cam1; zeon3; kei4 (also); ke4; kei3; gei6; fo3; zeoi1
U+9A89.0	fMandarin	biao1
U+9A89.0	fEnglish	 running herd of horsed
U+9A8A.0	fUTF8	骊
U+9A8A.0	fGB	0-666A
U+9A8A.0	fR/S	 187.7
U+9A8A.0	fTotalStrokes	 10
U+9A8A.0	fCangjie	NMMBB
U+9A8A.0	fHakka	 li2
U+9A8A.0	fCantonese	lei4
U+9A8A.0	fMandarin	li2
U+9A8A.0	fEnglish	simplified form of &#x9A6A;
U+9A8B.0	fUTF8	骋
U+9A8B.0	fGB	0-3352
U+9A8B.0	fR/S	 187.7
U+9A8B.0	fTotalStrokes	 10
U+9A8B.0	fCangjie	NMLWS
U+9A8B.0	fHakka	 pin5
U+9A8B.0	fCantonese	cing2
U+9A8B.0	fMandarin	cheng3
U+9A8B.0	fEnglish	simplified form of &#x9A01;
U+9A8C.0	fUTF8	验
U+9A8C.0	fGB	0-5169
U+9A8C.0	fR/S	 187.7
U+9A8C.0	fTotalStrokes	 10
U+9A8C.0	fCangjie	NMOMM
U+9A8C.0	fHakka	 ngiam5
U+9A8C.0	fCantonese	jim6
U+9A8C.0	fMandarin	yan4
U+9A8C.0	fEnglish	simplified form of &#x9A57;
U+9A8D.0	fUTF8	骍
U+9A8D.0	fGB	8-7B43
U+9A8D.0	fR/S	 187.7
U+9A8D.0	fTotalStrokes	 10
U+9A8D.0	fCangjie	NMYTJ
U+9A8D.0	fCantonese	sing1
U+9A8D.0	fMandarin	xing1; xin1
U+9A8D.0	fEnglish	red, brown, bay; neat, harmonious
U+9A8E.0	fUTF8	骎
U+9A8E.0	fGB	8-7B44
U+9A8E.0	fR/S	 187.7
U+9A8E.0	fTotalStrokes	 10
U+9A8E.0	fCangjie	NMSME
U+9A8E.0	fCantonese	cam1
U+9A8E.0	fMandarin	qin1
U+9A8E.0	fEnglish	galloping; speeding
U+9A8F.0	fUTF8	骏
U+9A8F.0	fGB	0-3F25
U+9A8F.0	fR/S	 187.7
U+9A8F.0	fTotalStrokes	 10
U+9A8F.0	fCangjie	NMICE
U+9A8F.0	fHakka	 zun5
U+9A8F.0	fCantonese	zeon3
U+9A8F.0	fMandarin	jun4
U+9A8F.0	fEnglish	simplified form of &#x99FF;
U+9A90.0	fUTF8	骐
U+9A90.0	fGB	0-666B
U+9A90.0	fR/S	 187.8
U+9A90.0	fTotalStrokes	 11
U+9A90.0	fCangjie	NMTMC
U+9A90.0	fHakka	 ki2
U+9A90.0	fCantonese	kei4
U+9A90.0	fMandarin	qi2
U+9A90.0	fEnglish	simplified form of &#x9A0F;
U+9A91.0	fUTF8	骑
U+9A91.0	fGB	0-466F
U+9A91.0	fR/S	 187.8
U+9A91.0	fTotalStrokes	 11
U+9A91.0	fCangjie	NMKMR
U+9A91.0	fHakka	 ki2; ji3
U+9A91.0	fCantonese	ke4; kei4
U+9A91.0	fMandarin	qi2; ji4
U+9A91.0	fEnglish	simplified form of &#x9A0E;
U+9A92.0	fUTF8	骒
U+9A92.0	fGB	0-666C
U+9A92.0	fR/S	 187.8
U+9A92.0	fTotalStrokes	 11
U+9A92.0	fCangjie	NMWD
U+9A92.0	fHakka	 ko5
U+9A92.0	fCantonese	fo3
U+9A92.0	fMandarin	ke4
U+9A92.0	fEnglish	simplified form of &#x9A0D;
U+9A93.0	fUTF8	骓
U+9A93.0	fGB	0-666D
U+9A93.0	fR/S	 187.8
U+9A93.0	fTotalStrokes	 11
U+9A93.0	fCangjie	NMOG
U+9A93.0	fHakka	 chui1
U+9A93.0	fCantonese	zeoi1
U+9A93.0	fMandarin	zhui1
U+9A93.0	fEnglish	simplified form of &#x9A05;
U+9A94.0	fUTF8	骔
U+9A94.0	fGB	8-2D68
U+9A94.0	fR/S	 187.8
U+9A94.0	fTotalStrokes	 11
U+9A94.0	fCangjie	NMJMF
U+9A94.0	fCantonese	zung1
U+9A94.0	fMandarin	zong1
U+9A94.0	fEnglish	a horse's mane; bristles
U+9A95.0	fUTF8	骕
U+9A95.0	fGB	8-7B45
U+9A95.0	fR/S	 187.12; 187.8
U+9A95.0	fTotalStrokes	 11
U+9A95.0	fCangjie	NMLX
U+9A95.0	fCantonese	suk1
U+9A95.0	fMandarin	su4
U+9A95.0	fEnglish	name of a famous horse
U+9A96.0	fUTF8	骖
U+9A96.0	fGB	0-666E
U+9A96.0	fR/S	 187.8
U+9A96.0	fTotalStrokes	 11
U+9A96.0	fCangjie	NMIKH
U+9A96.0	fHakka	 cam1
U+9A96.0	fCantonese	caam1
U+9A96.0	fMandarin	can1; can3
U+9A96.0	fEnglish	simplified form of &#x9A42;
U+9A97.0	fUTF8	骗
U+9A97.0	fGB	0-462D
U+9A97.0	fR/S	 187.9
U+9A97.0	fTotalStrokes	 12
U+9A97.0	fCangjie	NMISB
U+9A97.0	fHakka	 pien5
U+9A97.0	fCantonese	pin3
U+9A97.0	fMandarin	pian4
U+9A97.0	fEnglish	simplified form of &#x9A19;
U+9A98.0	fUTF8	骘
U+9A98.0	fGB	0-666F
U+9A98.0	fR/S	 187.9
U+9A98.0	fTotalStrokes	 12
U+9A98.0	fCangjie	NHNVM
U+9A98.0	fHakka	 zhit7
U+9A98.0	fCantonese	zat1
U+9A98.0	fMandarin	zhi4; zuan1
U+9A98.0	fEnglish	simplified form of &#x9A2D;
U+9A99.0	fUTF8	骙
U+9A99.0	fGB	8-2C45
U+9A99.0	fR/S	 187.9
U+9A99.0	fTotalStrokes	 12
U+9A99.0	fCangjie	NMNOK
U+9A99.0	fCantonese	kwai4
U+9A99.0	fMandarin	kui2
U+9A99.0	fEnglish	lively; vigorous
U+9A9A.0	fUTF8	骚
U+9A9A.0	fGB	0-4927
U+9A9A.0	fR/S	 187.9
U+9A9A.0	fTotalStrokes	 12
U+9A9A.0	fCangjie	NMEII
U+9A9A.0	fHakka	 sau1; zau3
U+9A9A.0	fCantonese	sou1
U+9A9A.0	fMandarin	sao1; sao3; xiao1; zao4
U+9A9A.0	fEnglish	simplified form of &#x9A37;
U+9A9B.0	fUTF8	骛
U+9A9B.0	fGB	0-6670
U+9A9B.0	fR/S	 187.9
U+9A9B.0	fTotalStrokes	 12
U+9A9B.0	fCangjie	NKNVM
U+9A9B.0	fHakka	 vu5
U+9A9B.0	fCantonese	mou6
U+9A9B.0	fMandarin	wu4
U+9A9B.0	fEnglish	simplified form of &#x9A16;
U+9A9C.0	fUTF8	骜
U+9A9C.0	fGB	0-6671
U+9A9C.0	fR/S	 187.10
U+9A9C.0	fTotalStrokes	 13
U+9A9C.0	fCangjie	QKNVM
U+9A9C.0	fHakka	 ngau2
U+9A9C.0	fCantonese	ngou4; ou4 (also); ou6
U+9A9C.0	fMandarin	ao2; ao4
U+9A9C.0	fEnglish	simplified form of &#x9A41;
U+9A9D.0	fUTF8	骝
U+9A9D.0	fGB	0-6672
U+9A9D.0	fR/S	 187.10
U+9A9D.0	fTotalStrokes	 13
U+9A9D.0	fCangjie	NMHHW
U+9A9D.0	fHakka	 liu2
U+9A9D.0	fCantonese	lau4; lau1 (also)
U+9A9D.0	fMandarin	liu2
U+9A9D.0	fEnglish	simplified form of &#x9A2E;
U+9A9E.0	fUTF8	骞
U+9A9E.0	fGB	0-6539
U+9A9E.0	fR/S	 187.10
U+9A9E.0	fTotalStrokes	 13
U+9A9E.0	fCangjie	JTCM
U+9A9E.0	fHakka	 ken1
U+9A9E.0	fCantonese	hin1
U+9A9E.0	fMandarin	qian1; jian3
U+9A9E.0	fEnglish	simplified form of &#x9A2B;
U+9A9F.0	fUTF8	骟
U+9A9F.0	fGB	0-6673
U+9A9F.0	fR/S	 187.10
U+9A9F.0	fTotalStrokes	 13
U+9A9F.0	fCangjie	NMISM
U+9A9F.0	fHakka	 shen5
U+9A9F.0	fCantonese	sin3
U+9A9F.0	fMandarin	shan4
U+9A9F.0	fEnglish	simplified form of &#x9A38;
U+9AA0.0	fUTF8	骠
U+9AA0.0	fGB	0-6674
U+9AA0.0	fR/S	 187.11
U+9AA0.0	fTotalStrokes	 14
U+9AA0.0	fCangjie	NMMWF
U+9AA0.0	fHakka	 piau1; piau5
U+9AA0.0	fCantonese	piu3; biu1; mak6
U+9AA0.0	fMandarin	piao4; biao4; biao1
U+9AA0.0	fEnglish	simplified form of &#x9A43;
U+9AA1.0	fUTF8	骡
U+9AA1.0	fGB	0-4262
U+9AA1.0	fR/S	 187.11
U+9AA1.0	fTotalStrokes	 14
U+9AA1.0	fCangjie	NMWVF
U+9AA1.0	fHakka	 lo2
U+9AA1.0	fCantonese	lo4 (also); leoi4; leoi6
U+9AA1.0	fMandarin	luo2
U+9AA1.0	fEnglish	simplified form of &#x9A3E;
U+9AA2.0	fUTF8	骢
U+9AA2.0	fGB	0-6675
U+9AA2.0	fR/S	 187.11
U+9AA2.0	fTotalStrokes	 14
U+9AA2.0	fCangjie	NMHWP
U+9AA2.0	fHakka	 cung1
U+9AA2.0	fCantonese	cung1
U+9AA2.0	fMandarin	cong1
U+9AA2.0	fEnglish	simplified form of &#x9A44;
U+9AA3.0	fUTF8	骣
U+9AA3.0	fGB	0-6676
U+9AA3.0	fR/S	 187.12
U+9AA3.0	fTotalStrokes	 15
U+9AA3.0	fCangjie	NMSND
U+9AA3.0	fCantonese	zaan2; cin5; can2
U+9AA3.0	fMandarin	zhan4; chan3
U+9AA3.0	fEnglish	simplified form of &#x9A4F;
U+9AA4.0	fUTF8	骤
U+9AA4.0	fGB	0-5668
U+9AA4.0	fR/S	 187.14
U+9AA4.0	fTotalStrokes	 17
U+9AA4.0	fCangjie	NMSEO
U+9AA4.0	fHakka	 ceu3
U+9AA4.0	fCantonese	zaau6; zau6 (also)
U+9AA4.0	fMandarin	zou4; zhou4
U+9AA4.0	fEnglish	simplified form of &#x9A5F;
U+9AA5.0	fUTF8	骥
U+9AA5.0	fGB	0-6677
U+9AA5.0	fR/S	 187.16
U+9AA5.0	fTotalStrokes	 19
U+9AA5.0	fCangjie	NMLPC
U+9AA5.0	fHakka	 gi3
U+9AA5.0	fCantonese	kei3
U+9AA5.0	fMandarin	ji4
U+9AA5.0	fEnglish	simplified form of &#x9A65;
U+9AA6.0	fUTF8	骦
U+9AA6.0	fGB	8-7B46
U+9AA6.0	fR/S	 187.17
U+9AA6.0	fTotalStrokes	 20
U+9AA6.0	fCangjie	NMMBU
U+9AA6.0	fCantonese	soeng1
U+9AA6.0	fMandarin	shuang1
U+9AA6.0	fEnglish	horse
U+9AA7.0	fUTF8	骧
U+9AA7.0	fGB	0-6678
U+9AA7.0	fR/S	 187.17
U+9AA7.0	fTotalStrokes	 20
U+9AA7.0	fCangjie	NMYRV
U+9AA7.0	fHakka	 siong1
U+9AA7.0	fCantonese	soeng1
U+9AA7.0	fMandarin	xiang1
U+9AA7.0	fEnglish	simplified form of &#x9A64;
U+9AA8.0	fUTF8	骨
U+9AA8.0	fCNS11643	1-586C
U+9AA8.0	fBig5	B0A9
U+9AA8.0	fGB	0-3947
U+9AA8.0	fR/S	 188.0
U+9AA8.0	fTotalStrokes	 10
U+9AA8.0	fCangjie	BBB
U+9AA8.0	fFourCorner	77227
U+9AA8.0	fHakka	  gut7; gwut7
U+9AA8.0	fCantonese	gwat1
U+9AA8.0	fMandarin	gu3
U+9AA8.0	fEnglish	[1] bone [2] skeleton; frame; framework [3] Kangxi radical 188
U+9AA8.1	fUTF8	骨
U+9AA8.1	fCNS11643	1-586C
U+9AA8.1	fBig5	B0A9
U+9AA8.1	fGB	0-3947
U+9AA8.1	fR/S	 188.0
U+9AA8.1	fTotalStrokes	 10
U+9AA8.1	fCangjie	BBB
U+9AA8.1	fFourCorner	77227
U+9AA8.1	fMandarin	gu2
U+9AA8.1	fEnglish	 bone
U+9AA8.2	fUTF8	骨
U+9AA8.2	fCNS11643	1-586C
U+9AA8.2	fBig5	B0A9
U+9AA8.2	fGB	0-3947
U+9AA8.2	fR/S	 188.0
U+9AA8.2	fTotalStrokes	 10
U+9AA8.2	fCangjie	BBB
U+9AA8.2	fFourCorner	77227
U+9AA8.2	fMandarin	gu1
U+9AA8.2	fEnglish	 bone
U+9AA9.0	fUTF8	骩
U+9AA9.0	fCNS11643	3-402F
U+9AA9.0	fGB	E-4642
U+9AA9.0	fR/S	 188.2
U+9AA9.0	fTotalStrokes	 12
U+9AA9.0	fCangjie	BBKN
U+9AA9.0	fFourCorner	74210
U+9AA9.0	fCantonese	wai2; gei1
U+9AA9.0	fMandarin	wei3
U+9AAA.0	fUTF8	骪
U+9AAA.0	fCNS11643	4-4221
U+9AAA.0	fGB	E-4643
U+9AAA.0	fR/S	  188.3
U+9AAA.0	fTotalStrokes	 13
U+9AAA.0	fCangjie	BBHNI
U+9AAA.0	fFourCorner	77210
U+9AAA.0	fCantonese	wai2
U+9AAA.0	fMandarin	wei3
U+9AAA.0	fEnglish	crooked bones
U+9AAB.0	fUTF8	骫
U+9AAB.0	fCNS11643	2-486F
U+9AAB.0	fBig5	E0E9
U+9AAB.0	fGB	3-796D
U+9AAB.0	fR/S	 188.3
U+9AAB.0	fTotalStrokes	 13
U+9AAB.0	fCangjie	BBKNI
U+9AAB.0	fFourCorner	75217
U+9AAB.0	fHakka	 wui3; wi3; vui3
U+9AAB.0	fCantonese	wai2
U+9AAB.0	fMandarin	wei3
U+9AAB.0	fEnglish	 bent; twist
U+9AAC.0	fUTF8	骬
U+9AAC.0	fCNS11643	4-417D
U+9AAC.0	fGB	3-796C
U+9AAC.0	fR/S	  188.3
U+9AAC.0	fTotalStrokes	 13
U+9AAC.0	fCangjie	BBMD
U+9AAC.0	fFourCorner	71240
U+9AAC.0	fCantonese	jyu4
U+9AAC.0	fMandarin	yu2
U+9AAD.0	fUTF8	骭
U+9AAD.0	fCNS11643	2-486E
U+9AAD.0	fBig5	E0E8
U+9AAD.0	fGB	3-796B
U+9AAD.0	fR/S	 188.3
U+9AAD.0	fTotalStrokes	 13
U+9AAD.0	fCangjie	BBMJ
U+9AAD.0	fFourCorner	71240
U+9AAD.0	fHakka	  gon5
U+9AAD.0	fCantonese	gon3; haan6
U+9AAD.0	fMandarin	gan4; shan3
U+9AAD.0	fEnglish	the shinbone
U+9AAE.0	fUTF8	骮
U+9AAE.0	fCNS11643	4-417E
U+9AAE.0	fGB	5-776C
U+9AAE.0	fR/S	  188.3
U+9AAE.0	fTotalStrokes	 13
U+9AAE.0	fCangjie	BBIP
U+9AAE.0	fFourCorner	73240
U+9AAE.0	fCantonese	jik6
U+9AAE.0	fMandarin	yi4
U+9AAF.0	fUTF8	骯
U+9AAF.0	fCNS11643	1-6B73
U+9AAF.0	fBig5	BBEA
U+9AAF.0	fGB	1-3039
U+9AAF.0	fR/S	 188.4
U+9AAF.0	fTotalStrokes	 14
U+9AAF.0	fCangjie	BBYHN
U+9AAF.0	fFourCorner	70217
U+9AAF.0	fHakka	 kong5; hong5; ang1; hong3; kong3
U+9AAF.0	fCantonese	ong3 (also); hong4; ngong3 (also); ong1; ngong1 (also); kon3; kong3
U+9AAF.0	fMandarin	ang1; kang1; hang2; kang3
U+9AAF.0	fEnglish	[1] fat [2] stiff; straight
U+9AB0.0	fUTF8	骰
U+9AB0.0	fCNS11643	1-6B74
U+9AB0.0	fBig5	BBEB
U+9AB0.0	fGB	0-773B
U+9AB0.0	fR/S	 188.4
U+9AB0.0	fTotalStrokes	 14
U+9AB0.0	fCangjie	BBHNE
U+9AB0.0	fFourCorner	77247
U+9AB0.0	fHakka	  teu2; dau2; tiu2; heu2; set7
U+9AB0.0	fCantonese	sik1; tau4 (also); gu2; gwu2 (also)
U+9AB0.0	fMandarin	gu3; shai3
U+9AB0.0	fEnglish	 dice
U+9AB0.1	fUTF8	骰
U+9AB0.1	fCNS11643	1-6B74
U+9AB0.1	fBig5	BBEB
U+9AB0.1	fGB	0-773B
U+9AB0.1	fR/S	 188.4
U+9AB0.1	fTotalStrokes	 14
U+9AB0.1	fCangjie	BBHNE
U+9AB0.1	fFourCorner	77247
U+9AB0.1	fMandarin	tou2
U+9AB0.1	fEnglish	[literary pronunciation] dice
U+9AB1.0	fUTF8	骱
U+9AB1.0	fCNS11643	2-4F42
U+9AB1.0	fBig5	E4DA
U+9AB1.0	fGB	0-773A
U+9AB1.0	fR/S	 188.4
U+9AB1.0	fTotalStrokes	 14
U+9AB1.0	fCangjie	BBOLL
U+9AB1.0	fFourCorner	78820
U+9AB1.0	fCantonese	gaai3; haai6 (also)
U+9AB1.0	fMandarin	xie4; he2; ga4; jie4; jia2
U+9AB2.0	fUTF8	骲
U+9AB2.0	fCNS11643	4-4E74
U+9AB2.0	fGB	3-7971
U+9AB2.0	fR/S	  188.5
U+9AB2.0	fTotalStrokes	 15
U+9AB2.0	fCangjie	BBPRU
U+9AB2.0	fFourCorner	77210
U+9AB2.0	fHakka	 pok8
U+9AB2.0	fCantonese	baau6; biu6
U+9AB2.0	fMandarin	bao2; bao1; yao3; bao4; pu1; pao2
U+9AB2.0	fEnglish	 arrow-head
U+9AB3.0	fUTF8	骳
U+9AB3.0	fCNS11643	2-557B
U+9AB3.0	fBig5	E8D2
U+9AB3.0	fGB	3-7973
U+9AB3.0	fR/S	 188.5
U+9AB3.0	fTotalStrokes	 15
U+9AB3.0	fCangjie	BBDHE
U+9AB3.0	fFourCorner	74247
U+9AB3.0	fHakka	 pi5; pi6
U+9AB3.0	fCantonese	bei6
U+9AB3.0	fMandarin	bi4; bei4
U+9AB3.0	fEnglish	[1] distorted [2] twisted
U+9AB4.0	fUTF8	骴
U+9AB4.0	fCNS11643	2-5B78
U+9AB4.0	fBig5	EC6C
U+9AB4.0	fGB	3-7978
U+9AB4.0	fR/S	 188.5
U+9AB4.0	fTotalStrokes	 15
U+9AB4.0	fCangjie	BBYMP
U+9AB4.0	fFourCorner	71210
U+9AB4.0	fHakka	 zih1; cih1; cu1
U+9AB4.0	fCantonese	ci1; zi6; ci3
U+9AB4.0	fMandarin	chi1; zi4; ci1; zhai4
U+9AB4.0	fEnglish	a putrid carcase
U+9AB5.0	fUTF8	骵
U+9AB5.0	fCNS11643	3-4F77
U+9AB5.0	fGB	E-4644
U+9AB5.0	fR/S	 188.5
U+9AB5.0	fTotalStrokes	 15
U+9AB5.0	fCangjie	BBDM
U+9AB5.0	fFourCorner	75230
U+9AB5.0	fHakka	 ti3
U+9AB5.0	fCantonese	tai2
U+9AB5.0	fMandarin	ti3; ti1
U+9AB6.0	fUTF8	骶
U+9AB6.0	fCNS11643	3-4F76
U+9AB6.0	fGB	0-773E
U+9AB6.0	fR/S	 188.5
U+9AB6.0	fTotalStrokes	 15
U+9AB6.0	fCangjie	BBHVI
U+9AB6.0	fFourCorner	72240
U+9AB6.0	fHakka	  di3
U+9AB6.0	fCantonese	dai2
U+9AB6.0	fMandarin	di3
U+9AB6.0	fEnglish	coccyx
U+9AB7.0	fUTF8	骷
U+9AB7.0	fCNS11643	1-6F7E
U+9AB7.0	fBig5	BE75
U+9AB7.0	fGB	0-773C
U+9AB7.0	fR/S	 188.5
U+9AB7.0	fTotalStrokes	 15
U+9AB7.0	fCangjie	BBJR
U+9AB7.0	fFourCorner	74260
U+9AB7.0	fHakka	  ku1
U+9AB7.0	fCantonese	fu1
U+9AB7.0	fMandarin	ku1
U+9AB7.0	fEnglish	 human skeleton
U+9AB8.0	fUTF8	骸
U+9AB8.0	fCNS11643	1-7330
U+9AB8.0	fBig5	C065
U+9AB8.0	fGB	0-3A21
U+9AB8.0	fR/S	 188.6
U+9AB8.0	fTotalStrokes	 16
U+9AB8.0	fCangjie	BBYVO
U+9AB8.0	fFourCorner	70282
U+9AB8.0	fHakka	 hai2
U+9AB8.0	fCantonese	haai4; hoi4 (also)
U+9AB8.0	fMandarin	gai1; xie2; hai2
U+9AB8.0	fEnglish	[1] the shinbone [2] skeleton
U+9AB9.0	fUTF8	骹
U+9AB9.0	fCNS11643	2-5B76
U+9AB9.0	fBig5	EC6A
U+9AB9.0	fGB	3-797B
U+9AB9.0	fR/S	 188.6
U+9AB9.0	fTotalStrokes	 16
U+9AB9.0	fCangjie	BBYCK
U+9AB9.0	fFourCorner	70248
U+9AB9.0	fHakka	 gau5
U+9AB9.0	fCantonese	haau1; gaau3
U+9AB9.0	fMandarin	qiao1; xiao1
U+9AB9.0	fEnglish	a joint
U+9ABA.0	fUTF8	骺
U+9ABA.0	fCNS11643	4-5531
U+9ABA.0	fGB	0-773F
U+9ABA.0	fR/S	 188.6
U+9ABA.0	fTotalStrokes	 16
U+9ABA.0	fCangjie	BBHMR
U+9ABA.0	fFourCorner	72260
U+9ABA.0	fCantonese	hau4
U+9ABA.0	fMandarin	gou4; hou2
U+9ABA.0	fEnglish	tip of a long bone
U+9ABB.0	fUTF8	骻
U+9ABB.0	fCNS11643	2-5B79
U+9ABB.0	fBig5	EC6D
U+9ABB.0	fGB	3-7976
U+9ABB.0	fR/S	 188.6
U+9ABB.0	fTotalStrokes	 16
U+9ABB.0	fCangjie	BBKMS
U+9ABB.0	fFourCorner	74227
U+9ABB.0	fHakka	  kwa5
U+9ABB.0	fCantonese	kwaa3; kwaa2 (also)
U+9ABB.0	fMandarin	kua4
U+9ABB.0	fEnglish	the waist bone
U+9ABC.0	fUTF8	骼
U+9ABC.0	fCNS11643	1-7331
U+9ABC.0	fBig5	C066
U+9ABC.0	fGB	0-7740
U+9ABC.0	fR/S	 188.6
U+9ABC.0	fTotalStrokes	 16
U+9ABC.0	fCangjie	BBHER
U+9ABC.0	fFourCorner	77264
U+9ABC.0	fHakka	 gok7; giet7; gak7; get7
U+9ABC.0	fCantonese	gaak3
U+9ABC.0	fMandarin	ge2; ge4
U+9ABC.0	fEnglish	 bone; skeleton
U+9ABD.0	fUTF8	骽
U+9ABD.0	fCNS11643	3-573A
U+9ABD.0	fGB	E-4645
U+9ABD.0	fR/S	  188.7
U+9ABD.0	fTotalStrokes	 17
U+9ABD.0	fCangjie	BBBV
U+9ABD.0	fFourCorner	72240
U+9ABD.0	fHakka	  tui3
U+9ABD.0	fCantonese	teoi2
U+9ABD.0	fMandarin	tui3
U+9ABD.0	fEnglish	leg; thigh
U+9ABE.0	fUTF8	骾
U+9ABE.0	fCNS11643	2-6070
U+9ABE.0	fBig5	EF64
U+9ABE.0	fGB	E-4646
U+9ABE.0	fR/S	 188.7
U+9ABE.0	fTotalStrokes	 17
U+9ABE.0	fCangjie	BBMLK
U+9ABE.0	fFourCorner	71246
U+9ABE.0	fHakka	 gien3; gang3; ngang5; ngang6; gen3
U+9ABE.0	fCantonese	gang2; kang2 (also)
U+9ABE.0	fMandarin	geng3
U+9ABE.0	fEnglish	any object stuck in the throat
U+9ABF.0	fUTF8	骿
U+9ABF.0	fCNS11643	2-5B77
U+9ABF.0	fBig5	EC6B
U+9ABF.0	fGB	3-797C
U+9ABF.0	fR/S	 188.8
U+9ABF.0	fTotalStrokes	 18
U+9ABF.0	fCangjie	BBTT
U+9ABF.0	fFourCorner	78241
U+9ABF.0	fHakka	 pin2
U+9ABF.0	fCantonese	pin4; ping4
U+9ABF.0	fMandarin	pian2
U+9ABF.0	fEnglish	the ribs all in one solid piece
U+9AC0.0	fUTF8	髀
U+9AC0.0	fCNS11643	2-6463
U+9AC0.0	fBig5	F1B9
U+9AC0.0	fGB	0-7742
U+9AC0.0	fR/S	 188.8
U+9AC0.0	fTotalStrokes	 18
U+9AC0.0	fCangjie	BBHWJ
U+9AC0.0	fFourCorner	76240
U+9AC0.0	fHakka	 bi3
U+9AC0.0	fCantonese	bei2
U+9AC0.0	fMandarin	bi4; bi3; mi3
U+9AC0.0	fEnglish	[1] buttocks [2] the hipbone; innominate bone
U+9AC1.0	fUTF8	髁
U+9AC1.0	fCNS11643	1-7778
U+9AC1.0	fBig5	C34E
U+9AC1.0	fGB	0-7741
U+9AC1.0	fR/S	 188.8
U+9AC1.0	fTotalStrokes	 18
U+9AC1.0	fCangjie	BBWD
U+9AC1.0	fFourCorner	76294
U+9AC1.0	fHakka	 ko5; ko1; kwa3
U+9AC1.0	fCantonese	fo1; kwaa3 (also)
U+9AC1.0	fMandarin	ke1; ke4; kua4; kua1
U+9AC1.0	fEnglish	[1] the hipbone; innominate bone [2] kneecap; kneepan
U+9AC2.0	fUTF8	髂
U+9AC2.0	fCNS11643	2-682E
U+9AC2.0	fBig5	F3C1
U+9AC2.0	fGB	0-7744
U+9AC2.0	fR/S	 188.9
U+9AC2.0	fTotalStrokes	 19
U+9AC2.0	fCangjie	BBJHR
U+9AC2.0	fFourCorner	73264
U+9AC2.0	fHakka	  ka5
U+9AC2.0	fCantonese	kaa3
U+9AC2.0	fMandarin	qia4; ka4; ge2
U+9AC2.0	fEnglish	the pelvis
U+9AC3.0	fUTF8	髃
U+9AC3.0	fCNS11643	4-6249
U+9AC3.0	fGB	5-777E
U+9AC3.0	fR/S	  188.9
U+9AC3.0	fTotalStrokes	 19
U+9AC3.0	fCangjie	BBWLB
U+9AC3.0	fFourCorner	76220
U+9AC3.0	fHakka	 ngi2; ngieu5
U+9AC3.0	fCantonese	jyu4
U+9AC3.0	fMandarin	yu2; ou3; wei2
U+9AC3.0	fEnglish	shoulder bone
U+9AC4.0	fUTF8	髄
U+9AC4.0	fCNS11643	3-5D61
U+9AC4.0	fGB	E-4647
U+9AC4.0	fR/S	 188.9
U+9AC4.0	fTotalStrokes	 20; 19
U+9AC4.0	fCangjie	BBYKB
U+9AC4.0	fFourCorner	74230
U+9AC4.0	fCantonese	seoi5
U+9AC4.0	fMandarin	sui3
U+9AC4.0	fEnglish	 bone marrow essences; substance
U+9AC5.0	fUTF8	髅
U+9AC5.0	fGB	0-7743
U+9AC5.0	fR/S	 188.9
U+9AC5.0	fTotalStrokes	 19
U+9AC5.0	fCangjie	BBFDV
U+9AC5.0	fHakka	 leu2
U+9AC5.0	fCantonese	lau4
U+9AC5.0	fMandarin	lou2
U+9AC5.0	fEnglish	simplified form of &#x9ACF;
U+9AC6.0	fUTF8	髆
U+9AC6.0	fCNS11643	2-6A74
U+9AC6.0	fBig5	F566
U+9AC6.0	fGB	5-7824
U+9AC6.0	fR/S	 188.10
U+9AC6.0	fTotalStrokes	 20
U+9AC6.0	fCangjie	BBIBI
U+9AC6.0	fFourCorner	73242
U+9AC6.0	fHakka	  bok7
U+9AC6.0	fCantonese	bok3; bok6 (also); pok3
U+9AC6.0	fMandarin	bo2; bo1
U+9AC6.0	fEnglish	shoulder blade
U+9AC7.0	fUTF8	髇
U+9AC7.0	fCNS11643	2-6A72
U+9AC7.0	fBig5	F564
U+9AC7.0	fGB	E-4648
U+9AC7.0	fR/S	 188.10
U+9AC7.0	fTotalStrokes	 20
U+9AC7.0	fCangjie	BBYRB
U+9AC7.0	fFourCorner	60227
U+9AC7.0	fCantonese	haau1; hou1 (also)
U+9AC7.0	fMandarin	xiao1
U+9AC8.0	fUTF8	髈
U+9AC8.0	fCNS11643	3-5D60
U+9AC8.0	fGB	E-4649
U+9AC8.0	fR/S	  188.10
U+9AC8.0	fTotalStrokes	 20
U+9AC8.0	fCangjie	BBYBS
U+9AC8.0	fFourCorner	70220
U+9AC8.0	fHakka	 pong3
U+9AC8.0	fCantonese	bong2; pong5; pong4; bong3; pong1
U+9AC8.0	fMandarin	pang2; bang4; bang3
U+9AC8.0	fEnglish	 defined as the pelvis; the hip-bone now used for humerus
U+9AC9.0	fUTF8	髉
U+9AC9.0	fCNS11643	3-5D5E
U+9AC9.0	fGB	3-7A25
U+9AC9.0	fR/S	  188.10
U+9AC9.0	fTotalStrokes	 20
U+9AC9.0	fCangjie	BBOBG
U+9AC9.0	fFourCorner	74210
U+9AC9.0	fCantonese	bok1
U+9AC9.0	fMandarin	bo1; jiao4
U+9ACA.0	fUTF8	髊
U+9ACA.0	fCNS11643	2-6A73
U+9ACA.0	fBig5	F565
U+9ACA.0	fGB	5-7823
U+9ACA.0	fR/S	 188.10
U+9ACA.0	fTotalStrokes	 20
U+9ACA.0	fCangjie	BBTQM
U+9ACA.0	fFourCorner	78212
U+9ACA.0	fHakka	 co2; cu2; cih1
U+9ACA.0	fCantonese	ci1; zi6; ci2; co4
U+9ACA.0	fMandarin	ci1; cuo1; zi4
U+9ACA.0	fEnglish	brimstone; or block sulphur
U+9ACB.0	fUTF8	髋
U+9ACB.0	fGB	0-7745
U+9ACB.0	fR/S	 188.10
U+9ACB.0	fTotalStrokes	 20
U+9ACB.0	fCangjie	BBJTU
U+9ACB.0	fHakka	 kwon1; k'wan1
U+9ACB.0	fCantonese	fun1
U+9ACB.0	fMandarin	kuan1; kun1
U+9ACB.0	fEnglish	simplified form of &#x9AD6;
U+9ACC.0	fUTF8	髌
U+9ACC.0	fGB	0-7746
U+9ACC.0	fR/S	 188.10
U+9ACC.0	fTotalStrokes	 20
U+9ACC.0	fCangjie	BBJOC
U+9ACC.0	fHakka	 bin5
U+9ACC.0	fCantonese	ban3
U+9ACC.0	fMandarin	bin4
U+9ACC.0	fEnglish	simplified form of &#x9AD5;
U+9ACD.0	fUTF8	髍
U+9ACD.0	fCNS11643	2-6C6E
U+9ACD.0	fBig5	F6A2
U+9ACD.0	fGB	E-464A
U+9ACD.0	fR/S	 188.11
U+9ACD.0	fTotalStrokes	 21
U+9ACD.0	fCangjie	BBIJC
U+9ACD.0	fFourCorner	70294
U+9ACD.0	fCantonese	mo4
U+9ACD.0	fMandarin	mo2
U+9ACE.0	fUTF8	髎
U+9ACE.0	fCNS11643	3-5F25
U+9ACE.0	fGB	3-7A26
U+9ACE.0	fR/S	  188.11
U+9ACE.0	fTotalStrokes	 21
U+9ACE.0	fCangjie	BBSMH
U+9ACE.0	fFourCorner	77220
U+9ACE.0	fCantonese	liu4
U+9ACE.0	fMandarin	liao2; liao4
U+9ACE.0	fEnglish	hip bone
U+9ACF.0	fUTF8	髏
U+9ACF.0	fCNS11643	1-7B48
U+9ACF.0	fBig5	C55C
U+9ACF.0	fGB	1-7743
U+9ACF.0	fR/S	 188.11
U+9ACF.0	fTotalStrokes	 21
U+9ACF.0	fCangjie	BBLLV
U+9ACF.0	fFourCorner	75244
U+9ACF.0	fHakka	 leu2; {lo2}; liu2
U+9ACF.0	fCantonese	lau4
U+9ACF.0	fMandarin	lou2
U+9ACF.0	fEnglish	pCC@] a human skeleton
U+9AD0.0	fUTF8	髐
U+9AD0.0	fCNS11643	2-6E51
U+9AD0.0	fBig5	F7A4
U+9AD0.0	fGB	3-7977
U+9AD0.0	fR/S	 188.12
U+9AD0.0	fTotalStrokes	 22
U+9AD0.0	fCangjie	BBGGU
U+9AD0.0	fFourCorner	74211
U+9AD0.0	fCantonese	haau1
U+9AD0.0	fMandarin	xiao1; nao2
U+9AD1.0	fUTF8	髑
U+9AD1.0	fCNS11643	1-7C56
U+9AD1.0	fBig5	C5EA
U+9AD1.0	fGB	0-7747
U+9AD1.0	fR/S	 188.13
U+9AD1.0	fTotalStrokes	 23
U+9AD1.0	fCangjie	BBWLI
U+9AD1.0	fFourCorner	76227
U+9AD1.0	fHakka	 tuk8; ch'uk7; cuk7
U+9AD1.0	fCantonese	duk6
U+9AD1.0	fMandarin	du2
U+9AD1.0	fEnglish	the human skull
U+9AD2.0	fUTF8	髒
U+9AD2.0	fCNS11643	1-7C28
U+9AD2.0	fBig5	C5BC
U+9AD2.0	fGB	1-787E
U+9AD2.0	fR/S	 188.13
U+9AD2.0	fTotalStrokes	 23
U+9AD2.0	fCangjie	BBTMT
U+9AD2.0	fFourCorner	74241
U+9AD2.0	fHakka	 zong5; zang1; cong1
U+9AD2.0	fCantonese	zong1; zong3 (also)
U+9AD2.0	fMandarin	zang3; zang1; zang4
U+9AD2.0	fEnglish	dirty; filthy
U+9AD3.0	fUTF8	髓
U+9AD3.0	fCNS11643	1-7C54
U+9AD3.0	fBig5	C5E8
U+9AD3.0	fGB	0-4B68
U+9AD3.0	fR/S	 188.13
U+9AD3.0	fTotalStrokes	 23
U+9AD3.0	fCangjie	BBYKB
U+9AD3.0	fFourCorner	74232
U+9AD3.0	fHakka	  sui3; sioi3;  soi3;  ts'ioi3; cui3; sui2
U+9AD3.0	fCantonese	seoi5; seoi2 (also)
U+9AD3.0	fMandarin	sui3; sui2
U+9AD3.0	fEnglish	 pith; marrow; essence
U+9AD4.0	fUTF8	體
U+9AD4.0	fCNS11643	1-7C55
U+9AD4.0	fBig5	C5E9
U+9AD4.0	fGB	1-4C65
U+9AD4.0	fR/S	 188.13
U+9AD4.0	fTotalStrokes	 23
U+9AD4.0	fCangjie	BBTWT
U+9AD4.0	fFourCorner	75218
U+9AD4.0	fHakka	  ti3
U+9AD4.0	fCantonese	tai2
U+9AD4.0	fMandarin	ti3; ti1
U+9AD4.0	fEnglish	[1] body [2] shape; form [3] entity; unit [4] style; fashion; system [5] substance; essence [6] theory (as opposed to practice)
U+9AD5.0	fUTF8	髕
U+9AD5.0	fCNS11643	2-7062
U+9AD5.0	fBig5	F8D4
U+9AD5.0	fGB	1-7746
U+9AD5.0	fR/S	 188.14
U+9AD5.0	fTotalStrokes	 24
U+9AD5.0	fCangjie	BBJMC
U+9AD5.0	fFourCorner	73286
U+9AD5.0	fHakka	  bin5
U+9AD5.0	fCantonese	ban3; pan5
U+9AD5.0	fMandarin	bin4
U+9AD5.0	fEnglish	the kneecap; kneepan
U+9AD6.0	fUTF8	髖
U+9AD6.0	fCNS11643	1-7D2F
U+9AD6.0	fBig5	C662
U+9AD6.0	fGB	1-7745
U+9AD6.0	fR/S	 188.15
U+9AD6.0	fTotalStrokes	 25
U+9AD6.0	fCangjie	BBJTI
U+9AD6.0	fFourCorner	73216
U+9AD6.0	fHakka	 k'wan1; kon1; kwon1
U+9AD6.0	fCantonese	fun1
U+9AD6.0	fMandarin	kuan1; kun1
U+9AD6.0	fEnglish	 hipbone hip
U+9AD7.0	fUTF8	髗
U+9AD7.0	fCNS11643	3-6174
U+9AD7.0	fGB	E-464B
U+9AD7.0	fR/S	 188.16
U+9AD7.0	fTotalStrokes	 26
U+9AD7.0	fCangjie	BBYPT
U+9AD7.0	fFourCorner	71210
U+9AD7.0	fHakka	 lu2
U+9AD7.0	fCantonese	lou4
U+9AD7.0	fMandarin	lu2
U+9AD7.0	fEnglish	 the skull the forehead
U+9AD8.0	fUTF8	高
U+9AD8.0	fCNS11643	1-586D
U+9AD8.0	fBig5	B0AA
U+9AD8.0	fGB	0-385F
U+9AD8.0	fR/S	 189.0
U+9AD8.0	fTotalStrokes	 10
U+9AD8.0	fCangjie	YRBR
U+9AD8.0	fFourCorner	00227
U+9AD8.0	fHakka	   gau1;  go1
U+9AD8.0	fCantonese	gou1
U+9AD8.0	fMandarin	gao1; gao4
U+9AD8.0	fEnglish	[1] high; tall [2] lofty [3] of a high level or degree; above the average [4] a Chinese family name [5] Kangxi radical 1989
U+9AD9.0	fUTF8	髙
U+9AD9.0	fCNS11643	4-362D
U+9AD9.0	fGB	E-464C
U+9AD9.0	fR/S	 189.0
U+9AD9.0	fTotalStrokes	 11; 10
U+9AD9.0	fCangjie	YSLB
U+9AD9.0	fFourCorner	00220
U+9AD9.0	fCantonese	gou1
U+9AD9.0	fMandarin	gao1; gao4
U+9AD9.0	fEnglish	high
U+9ADA.0	fUTF8	髚
U+9ADA.0	fCNS11643	3-4A6E
U+9ADA.0	fGB	E-464D
U+9ADA.0	fR/S	  189.4
U+9ADA.0	fTotalStrokes	 14
U+9ADA.0	fCangjie	YBYHN
U+9ADA.0	fFourCorner	00210
U+9ADA.0	fCantonese	hiu3
U+9ADA.0	fMandarin	qiao4
U+9ADB.0	fUTF8	髛
U+9ADB.0	fCNS11643	4-4E79
U+9ADB.0	fGB	E-464E
U+9ADB.0	fR/S	  189.5
U+9ADB.0	fTotalStrokes	 15
U+9ADB.0	fCangjie	YBSKN
U+9ADB.0	fFourCorner	07210
U+9ADB.0	fCantonese	hou1
U+9ADB.0	fMandarin	kao1
U+9ADC.0	fUTF8	髜
U+9ADC.0	fCNS11643	2-6464
U+9ADC.0	fBig5	F1BA
U+9ADC.0	fGB	E-464F
U+9ADC.0	fR/S	 189.8
U+9ADC.0	fTotalStrokes	 18
U+9ADC.0	fCangjie	YBAHT
U+9ADC.0	fFourCorner	06240
U+9ADC.0	fCantonese	hiu2; haau1
U+9ADC.0	fMandarin	qiao1
U+9ADD.0	fUTF8	髝
U+9ADD.0	fCNS11643	3-5F7B
U+9ADD.0	fGB	3-7B72
U+9ADD.0	fR/S	  189.12
U+9ADD.0	fTotalStrokes	 22
U+9ADD.0	fCangjie	YBFFS
U+9ADD.0	fFourCorner	09220
U+9ADD.0	fCantonese	lou4
U+9ADD.0	fMandarin	lao4
U+9ADE.0	fUTF8	髞
U+9ADE.0	fCNS11643	3-6063
U+9ADE.0	fGB	3-7B73
U+9ADE.0	fR/S	  189.13
U+9ADE.0	fTotalStrokes	 23
U+9ADE.0	fCangjie	YBRRD
U+9ADE.0	fFourCorner	06290
U+9ADE.0	fHakka	 cau5
U+9ADE.0	fCantonese	cou3
U+9ADE.0	fMandarin	zao4
U+9ADE.0	fEnglish	high; imposing; eminent
U+9ADF.0	fUTF8	髟
U+9ADF.0	fCNS11643	2-334C
U+9ADF.0	fBig5	D449
U+9ADF.0	fGB	0-7754
U+9ADF.0	fR/S	 190.0
U+9ADF.0	fTotalStrokes	 10
U+9ADF.0	fCangjie	SIHHH
U+9ADF.0	fFourCorner	72722
U+9ADF.0	fHakka	  piau1;  biau1; peu1
U+9ADF.0	fCantonese	biu1
U+9ADF.0	fMandarin	biao1; shan1; piao4
U+9ADF.0	fEnglish	Kangxi radical 190
U+9AE0.0	fUTF8	髠
U+9AE0.0	fCNS11643	3-4032
U+9AE0.0	fGB	5-7927
U+9AE0.0	fR/S	 190.2
U+9AE0.0	fTotalStrokes	 12
U+9AE0.0	fCangjie	SHHN
U+9AE0.0	fFourCorner	72210
U+9AE0.0	fHakka	 kun1; kwun1
U+9AE0.0	fCantonese	kwan1
U+9AE0.0	fMandarin	kun1
U+9AE0.0	fEnglish	 shave head as punishment prune
U+9AE1.0	fUTF8	髡
U+9AE1.0	fCNS11643	1-676C
U+9AE1.0	fBig5	B9A6
U+9AE1.0	fGB	0-7755
U+9AE1.0	fR/S	 190.3
U+9AE1.0	fTotalStrokes	 13
U+9AE1.0	fCangjie	SHMU
U+9AE1.0	fFourCorner	72211
U+9AE1.0	fHakka	 kun1
U+9AE1.0	fCantonese	kwan1
U+9AE1.0	fMandarin	kun1; ku4
U+9AE1.0	fEnglish	[1] shave hair as a punishment [2] shear trees
U+9AE2.0	fUTF8	髢
U+9AE2.0	fCNS11643	3-4561
U+9AE2.0	fGB	3-7B34
U+9AE2.0	fR/S	 190.3
U+9AE2.0	fTotalStrokes	 13
U+9AE2.0	fCangjie	SHPD
U+9AE2.0	fFourCorner	72710
U+9AE2.0	fHakka	 ti5; ti6
U+9AE2.0	fCantonese	tai3
U+9AE2.0	fMandarin	ti4; xi2; di2; di4
U+9AE2.0	fEnglish	wig
U+9AE3.0	fUTF8	髣
U+9AE3.0	fCNS11643	2-4F43
U+9AE3.0	fBig5	E4DB
U+9AE3.0	fGB	E-4650
U+9AE3.0	fR/S	 190.4
U+9AE3.0	fTotalStrokes	 14
U+9AE3.0	fCangjie	SHYHS
U+9AE3.0	fFourCorner	72227
U+9AE3.0	fHakka	  fong3
U+9AE3.0	fCantonese	fong2
U+9AE3.0	fMandarin	fang3
U+9AE3.0	fEnglish	similar to; like
U+9AE4.0	fUTF8	髤
U+9AE4.0	fCNS11643	3-4A6F
U+9AE4.0	fGB	3-7B35
U+9AE4.0	fR/S	  190.4
U+9AE4.0	fTotalStrokes	 14
U+9AE4.0	fCangjie	SHD
U+9AE4.0	fFourCorner	72900
U+9AE4.0	fHakka	 hiu1
U+9AE4.0	fCantonese	jau1
U+9AE4.0	fMandarin	xiu1; ci4
U+9AE4.0	fEnglish	red-coloured varnish
U+9AE5.0	fUTF8	髥
U+9AE5.0	fCNS11643	3-4A73
U+9AE5.0	fGB	E-4651
U+9AE5.0	fR/S	  190.4
U+9AE5.0	fTotalStrokes	 14
U+9AE5.0	fCangjie	SHBMM
U+9AE5.0	fFourCorner	72550
U+9AE5.0	fHakka	 ngiam1; jam2
U+9AE5.0	fCantonese	jim4
U+9AE5.0	fMandarin	ran2; ran3
U+9AE5.0	fEnglish	[1] whiskers [2] heavily bearded man
U+9AE6.0	fUTF8	髦
U+9AE6.0	fCNS11643	1-6B75
U+9AE6.0	fBig5	BBEC
U+9AE6.0	fGB	0-7756
U+9AE6.0	fR/S	 190.4
U+9AE6.0	fTotalStrokes	 14
U+9AE6.0	fCangjie	SHHQU
U+9AE6.0	fFourCorner	72714
U+9AE6.0	fHakka	 mau2; mau1; mo2
U+9AE6.0	fCantonese	mou4; mou1 (also)
U+9AE6.0	fMandarin	mao2; mou2; li2
U+9AE6.0	fEnglish	[1] a childrens hair stule with front hair covering the forehead [2] mane [3] man of talent
U+9AE7.0	fUTF8	髧
U+9AE7.0	fCNS11643	2-4F44
U+9AE7.0	fBig5	E4DC
U+9AE7.0	fGB	3-7B38
U+9AE7.0	fR/S	 190.4
U+9AE7.0	fTotalStrokes	 14
U+9AE7.0	fCangjie	SHLBU
U+9AE7.0	fFourCorner	72412
U+9AE7.0	fHakka	  tam3
U+9AE7.0	fCantonese	daam6; dam3 (also); jim4
U+9AE7.0	fMandarin	dan4
U+9AE7.0	fEnglish	long hair
U+9AE8.0	fUTF8	髨
U+9AE8.0	fCNS11643	3-4A71
U+9AE8.0	fGB	E-4652
U+9AE8.0	fR/S	 190.4
U+9AE8.0	fTotalStrokes	 14
U+9AE8.0	fCangjie	SHMMU
U+9AE8.0	fFourCorner	72210
U+9AE8.0	fCantonese	kwan1
U+9AE8.0	fMandarin	kun1
U+9AE9.0	fUTF8	髩
U+9AE9.0	fCNS11643	3-4A70
U+9AE9.0	fGB	5-7929
U+9AE9.0	fR/S	  190.4
U+9AE9.0	fTotalStrokes	 14
U+9AE9.0	fCangjie	SHMLS
U+9AE9.0	fFourCorner	72200
U+9AE9.0	fCantonese	ban3
U+9AE9.0	fMandarin	bin4
U+9AEA.0	fUTF8	髪
U+9AEA.0	fCNS11643	3-4A72
U+9AEA.0	fGB	E-4653
U+9AEA.0	fR/S	 190.4
U+9AEA.0	fTotalStrokes	 14
U+9AEA.0	fCangjie	SHKE
U+9AEA.0	fFourCorner	72440
U+9AEA.0	fCantonese	faat3
U+9AEA.0	fMandarin	fa4; fa3
U+9AEA.0	fEnglish	hair
U+9AEB.0	fUTF8	髫
U+9AEB.0	fCNS11643	2-557D
U+9AEB.0	fBig5	E8D4
U+9AEB.0	fGB	0-7758
U+9AEB.0	fR/S	 190.5
U+9AEB.0	fTotalStrokes	 15
U+9AEB.0	fCangjie	SHSHR
U+9AEB.0	fFourCorner	72602
U+9AEB.0	fHakka	  diau1; tiau1;  tiau2
U+9AEB.0	fCantonese	tiu4
U+9AEB.0	fMandarin	tiao2
U+9AEB.0	fEnglish	[1] a children's hair style with hair hanging down the forehead [2] youngster; child
U+9AEC.0	fUTF8	髬
U+9AEC.0	fCNS11643	2-557C
U+9AEC.0	fBig5	E8D3
U+9AEC.0	fGB	3-7B3B
U+9AEC.0	fR/S	 190.5
U+9AEC.0	fTotalStrokes	 15
U+9AEC.0	fCangjie	SHMFM
U+9AEC.0	fFourCorner	72109
U+9AEC.0	fCantonese	pei1
U+9AEC.0	fMandarin	pi1; peng5
U+9AED.0	fUTF8	髭
U+9AED.0	fCNS11643	1-7333
U+9AED.0	fBig5	C068
U+9AED.0	fGB	0-775A
U+9AED.0	fR/S	 190.5
U+9AED.0	fTotalStrokes	 15
U+9AED.0	fCangjie	SHYMP
U+9AED.0	fFourCorner	72111
U+9AED.0	fHakka	 zih1; zu1; zii1
U+9AED.0	fCantonese	zi1
U+9AED.0	fMandarin	zi1
U+9AED.0	fEnglish	moustache
U+9AEE.0	fUTF8	髮
U+9AEE.0	fCNS11643	1-7021
U+9AEE.0	fBig5	BE76
U+9AEE.0	fGB	1-7831
U+9AEE.0	fR/S	 190.5
U+9AEE.0	fTotalStrokes	 15
U+9AEE.0	fCangjie	SHIKK
U+9AEE.0	fFourCorner	72447
U+9AEE.0	fHakka	  fat7
U+9AEE.0	fCantonese	faat3; fat3
U+9AEE.0	fMandarin	fa4; fa3
U+9AEE.0	fEnglish	[1] hair covering human heads [2] hairbreadth; hair's breadth
U+9AEF.0	fUTF8	髯
U+9AEF.0	fCNS11643	1-7022
U+9AEF.0	fBig5	BE77
U+9AEF.0	fGB	0-7757
U+9AEF.0	fR/S	 190.5
U+9AEF.0	fTotalStrokes	 15
U+9AEF.0	fCangjie	SHGB
U+9AEF.0	fFourCorner	72557
U+9AEF.0	fHakka	 ngiam2; jam2; ngiam1; ram2; jam1
U+9AEF.0	fCantonese	jim4; jim6
U+9AEF.0	fMandarin	ran2; ran3
U+9AEF.0	fEnglish	same as &#x9AE5;
U+9AF0.0	fUTF8	髰
U+9AF0.0	fCNS11643	4-4F21
U+9AF0.0	fGB	E-4654
U+9AF0.0	fR/S	 190.5
U+9AF0.0	fTotalStrokes	 15
U+9AF0.0	fCangjie	SHPT
U+9AF0.0	fFourCorner	72710
U+9AF0.0	fCantonese	tai3
U+9AF0.0	fMandarin	ti4
U+9AF1.0	fUTF8	髱
U+9AF1.0	fCNS11643	2-5622
U+9AF1.0	fBig5	E8D7
U+9AF1.0	fGB	E-4655
U+9AF1.0	fR/S	 190.5
U+9AF1.0	fTotalStrokes	 15
U+9AF1.0	fCangjie	SHPRU
U+9AF1.0	fFourCorner	72712
U+9AF1.0	fCantonese	baau6; paau3
U+9AF1.0	fMandarin	pao4
U+9AF1.0	fEnglish	a topknot, bun, coiled hair-knot
U+9AF2.0	fUTF8	髲
U+9AF2.0	fCNS11643	2-5621
U+9AF2.0	fBig5	E8D6
U+9AF2.0	fGB	3-7B3E
U+9AF2.0	fR/S	 190.5
U+9AF2.0	fTotalStrokes	 15
U+9AF2.0	fCangjie	SHDHE
U+9AF2.0	fFourCorner	72247
U+9AF2.0	fCantonese	bei6
U+9AF2.0	fMandarin	pi1; pi4; bi4
U+9AF2.0	fEnglish	a wig
U+9AF3.0	fUTF8	髳
U+9AF3.0	fCNS11643	2-557E
U+9AF3.0	fBig5	E8D5
U+9AF3.0	fGB	3-7B3F
U+9AF3.0	fR/S	 190.5
U+9AF3.0	fTotalStrokes	 15
U+9AF3.0	fCangjie	SHNIH
U+9AF3.0	fFourCorner	72227
U+9AF3.0	fHakka	 mau2; meu5; meu2; mau3
U+9AF3.0	fCantonese	maau4; mau4 (also)
U+9AF3.0	fMandarin	mao2; mou2; meng2
U+9AF3.0	fEnglish	state
U+9AF4.0	fUTF8	髴
U+9AF4.0	fCNS11643	3-4F79
U+9AF4.0	fGB	E-4656
U+9AF4.0	fR/S	 190.5
U+9AF4.0	fTotalStrokes	 15
U+9AF4.0	fCangjie	SHLLN
U+9AF4.0	fFourCorner	72520
U+9AF4.0	fHakka	  fut7
U+9AF4.0	fCantonese	fai3; fat1
U+9AF4.0	fMandarin	fu2; fo2; fei4
U+9AF4.0	fEnglish	correct form of &#xlike;similar
U+9AF5.0	fUTF8	髵
U+9AF5.0	fCNS11643	3-5375
U+9AF5.0	fGB	3-7B42
U+9AF5.0	fR/S	  190.6
U+9AF5.0	fTotalStrokes	 16
U+9AF5.0	fCangjie	SHMBL
U+9AF5.0	fFourCorner	72220
U+9AF5.0	fHakka	 ji2; ri2
U+9AF5.0	fCantonese	ji4
U+9AF5.0	fMandarin	er2
U+9AF5.0	fEnglish	whiskers;bristles on the jaws of an animal
U+9AF6.0	fUTF8	髶
U+9AF6.0	fCNS11643	2-5B7A
U+9AF6.0	fBig5	EC6E
U+9AF6.0	fGB	3-7B41
U+9AF6.0	fR/S	 190.6
U+9AF6.0	fTotalStrokes	 16
U+9AF6.0	fCangjie	SHSJ
U+9AF6.0	fFourCorner	72407
U+9AF6.0	fHakka	 jung2
U+9AF6.0	fCantonese	jung4; ji6
U+9AF6.0	fMandarin	rong2; er4
U+9AF7.0	fUTF8	髷
U+9AF7.0	fCNS11643	2-5B7D
U+9AF7.0	fBig5	EC71
U+9AF7.0	fGB	E-4657
U+9AF7.0	fR/S	 190.6
U+9AF7.0	fTotalStrokes	 16
U+9AF7.0	fCangjie	SHTW
U+9AF7.0	fFourCorner	72602
U+9AF7.0	fCantonese	kuk1
U+9AF7.0	fMandarin	qu1; qu4
U+9AF7.0	fEnglish	a topknot
U+9AF8.0	fUTF8	髸
U+9AF8.0	fCNS11643	3-5374
U+9AF8.0	fGB	E-4658
U+9AF8.0	fR/S	  190.6
U+9AF8.0	fTotalStrokes	 16
U+9AF8.0	fCangjie	SHTC
U+9AF8.0	fFourCorner	72800
U+9AF8.0	fCantonese	gung1
U+9AF8.0	fMandarin	gong1
U+9AF9.0	fUTF8	髹
U+9AF9.0	fCNS11643	2-5B7C
U+9AF9.0	fBig5	EC70
U+9AF9.0	fGB	0-775B
U+9AF9.0	fR/S	 190.6
U+9AF9.0	fTotalStrokes	 16
U+9AF9.0	fCangjie	SHOD
U+9AF9.0	fFourCorner	72294
U+9AF9.0	fHakka	  hiu1
U+9AF9.0	fCantonese	jau1
U+9AF9.0	fMandarin	xiu1
U+9AF9.0	fEnglish	[1] a kind of dark-red paint [2] paint; lacquer (articles)
U+9AFA.0	fUTF8	髺
U+9AFA.0	fCNS11643	2-5B7B
U+9AFA.0	fBig5	EC6F
U+9AFA.0	fGB	3-7B43
U+9AFA.0	fR/S	 190.6
U+9AFA.0	fTotalStrokes	 16
U+9AFA.0	fCangjie	SHHJR
U+9AFA.0	fFourCorner	72604
U+9AFA.0	fHakka	 kwat7; gwat7
U+9AFA.0	fCantonese	kut3; kwut3 (also); gai3
U+9AFA.0	fMandarin	gua4; huo2; kuo4
U+9AFA.0	fEnglish	the hair dishevelled; as in mourning
U+9AFB.0	fUTF8	髻
U+9AFB.0	fCNS11643	1-7332
U+9AFB.0	fBig5	C067
U+9AFB.0	fGB	0-7759
U+9AFB.0	fR/S	 190.6
U+9AFB.0	fTotalStrokes	 16
U+9AFB.0	fCangjie	SHGR
U+9AFB.0	fFourCorner	72601
U+9AFB.0	fHakka	  gi5; giet7
U+9AFB.0	fCantonese	gai3; gai2 (also)
U+9AFB.0	fMandarin	ji4; ji2; jie2
U+9AFB.0	fEnglish	 hair rolled up in a bun; a coiffure with a topknot
U+9AFC.0	fUTF8	髼
U+9AFC.0	fCNS11643	2-6074
U+9AFC.0	fBig5	EF68
U+9AFC.0	fGB	5-793F
U+9AFC.0	fR/S	 190.7
U+9AFC.0	fTotalStrokes	 17
U+9AFC.0	fCangjie	SHHEJ
U+9AFC.0	fFourCorner	42508
U+9AFC.0	fHakka	 pung2
U+9AFC.0	fCantonese	pung4
U+9AFC.0	fMandarin	peng2
U+9AFC.0	fEnglish	[1] dishevelled [2] unkempt
U+9AFD.0	fUTF8	髽
U+9AFD.0	fCNS11643	2-6072
U+9AFD.0	fBig5	EF66
U+9AFD.0	fGB	3-7B47
U+9AFD.0	fR/S	 190.7
U+9AFD.0	fTotalStrokes	 17
U+9AFD.0	fCangjie	SHOOG
U+9AFD.0	fFourCorner	72104
U+9AFD.0	fHakka	 za1; ca1
U+9AFD.0	fCantonese	zaa1
U+9AFD.0	fMandarin	zhua1
U+9AFD.0	fEnglish	women's headdress in mouring
U+9AFE.0	fUTF8	髾
U+9AFE.0	fCNS11643	2-6071
U+9AFE.0	fBig5	EF65
U+9AFE.0	fGB	3-7B46
U+9AFE.0	fR/S	 190.7
U+9AFE.0	fTotalStrokes	 17
U+9AFE.0	fCangjie	SHFB
U+9AFE.0	fFourCorner	72227
U+9AFE.0	fHakka	  sau1
U+9AFE.0	fCantonese	saau1
U+9AFE.0	fMandarin	shao1; shao3; shao4
U+9AFE.0	fEnglish	 tail of a comet long hair
U+9AFF.0	fUTF8	髿
U+9AFF.0	fCNS11643	4-5A37
U+9AFF.0	fGB	3-7B4A
U+9AFF.0	fR/S	 190.7
U+9AFF.0	fTotalStrokes	 17; 16
U+9AFF.0	fCangjie	SHEFH
U+9AFF.0	fFourCorner	72129
U+9AFF.0	fHakka	  sa1
U+9AFF.0	fCantonese	so1; saa1 (also)
U+9AFF.0	fMandarin	sha1; suo1
U+9AFF.0	fEnglish	long, fine hair
U+9B00.0	fUTF8	鬀
U+9B00.0	fCNS11643	3-573B
U+9B00.0	fGB	E-4659
U+9B00.0	fR/S	  190.7
U+9B00.0	fTotalStrokes	 17
U+9B00.0	fCangjie	SHCNH
U+9B00.0	fFourCorner	72220
U+9B00.0	fHakka	 ti5
U+9B00.0	fCantonese	tai3
U+9B00.0	fMandarin	ti4
U+9B00.0	fEnglish	 shave
U+9B01.0	fUTF8	鬁
U+9B01.0	fCNS11643	2-6073
U+9B01.0	fBig5	EF67
U+9B01.0	fGB	5-793D
U+9B01.0	fR/S	 190.7
U+9B01.0	fTotalStrokes	 17
U+9B01.0	fCangjie	SHHDN
U+9B01.0	fFourCorner	72922
U+9B01.0	fCantonese	lei1; lei6
U+9B01.0	fMandarin	li4
U+9B01.0	fEnglish	[1] favus
U+9B02.0	fUTF8	鬂
U+9B02.0	fCNS11643	3-573C
U+9B02.0	fGB	E-465A
U+9B02.0	fR/S	 190.7
U+9B02.0	fTotalStrokes	 17
U+9B02.0	fCangjie	SHOMC
U+9B02.0	fFourCorner	72800
U+9B02.0	fHakka	  bin5
U+9B02.0	fCantonese	ban3
U+9B02.0	fMandarin	bin4
U+9B02.0	fEnglish	the hair on the temples
U+9B03.0	fUTF8	鬃
U+9B03.0	fCNS11643	1-7779
U+9B03.0	fBig5	C34F
U+9B03.0	fGB	0-5757
U+9B03.0	fR/S	 190.8
U+9B03.0	fTotalStrokes	 18
U+9B03.0	fCangjie	SHJMF
U+9B03.0	fFourCorner	72901
U+9B03.0	fHakka	  zung1
U+9B03.0	fCantonese	zung1; cung4
U+9B03.0	fMandarin	zong1
U+9B03.0	fEnglish	[1] mane [2] the topknot of a lady's headdress [3] bristle
U+9B04.0	fUTF8	鬄
U+9B04.0	fCNS11643	2-6466
U+9B04.0	fBig5	F1BC
U+9B04.0	fGB	3-7B4B
U+9B04.0	fR/S	 190.8
U+9B04.0	fTotalStrokes	 18
U+9B04.0	fCangjie	SHAPH
U+9B04.0	fFourCorner	72712
U+9B04.0	fHakka	  ti5
U+9B04.0	fCantonese	tai3; tik1
U+9B04.0	fMandarin	ti4; xi2; di2; di4
U+9B04.0	fEnglish	[1] dishevelled hair [2] false hair; wig
U+9B05.0	fUTF8	鬅
U+9B05.0	fCNS11643	2-6467
U+9B05.0	fBig5	F1BD
U+9B05.0	fGB	5-7942
U+9B05.0	fR/S	 190.8
U+9B05.0	fTotalStrokes	 18
U+9B05.0	fCangjie	SHBB
U+9B05.0	fFourCorner	72227
U+9B05.0	fHakka	  pen2
U+9B05.0	fCantonese	pang4; pung4; lang6
U+9B05.0	fMandarin	peng2
U+9B05.0	fEnglish	 umkempt hair loose flowing hair
U+9B06.0	fUTF8	鬆
U+9B06.0	fCNS11643	1-777A
U+9B06.0	fBig5	C350
U+9B06.0	fGB	1-4B49
U+9B06.0	fR/S	 190.8
U+9B06.0	fTotalStrokes	 18
U+9B06.0	fCangjie	SHDCI
U+9B06.0	fFourCorner	72932
U+9B06.0	fHakka	  sung1; ngiung6; ngiung5
U+9B06.0	fCantonese	sung1
U+9B06.0	fMandarin	song1; gong1; song4
U+9B06.0	fEnglish	[1] lax; loose; slack [2] loosen [3] relax [4] soft; light
U+9B07.0	fUTF8	鬇
U+9B07.0	fCNS11643	3-596F
U+9B07.0	fGB	3-7B44
U+9B07.0	fR/S	 190.6; 190.8
U+9B07.0	fTotalStrokes	 16; 18
U+9B07.0	fCangjie	SHBSD
U+9B07.0	fFourCorner	72500
U+9B07.0	fCantonese	zang1; zaang1 (also)
U+9B07.0	fMandarin	zheng1
U+9B08.0	fUTF8	鬈
U+9B08.0	fCNS11643	2-6465
U+9B08.0	fBig5	F1BB
U+9B08.0	fGB	0-775C
U+9B08.0	fR/S	 190.8
U+9B08.0	fTotalStrokes	 18
U+9B08.0	fCangjie	SHFQU
U+9B08.0	fFourCorner	72712
U+9B08.0	fHakka	 kien2; kian2; ken2
U+9B08.0	fCantonese	kyun4
U+9B08.0	fMandarin	quan2; juan3
U+9B08.0	fEnglish	[1] fine growth of hair [2] curled hair
U+9B09.0	fUTF8	鬉
U+9B09.0	fCNS11643	3-5B71
U+9B09.0	fGB	E-465B
U+9B09.0	fR/S	 190.9
U+9B09.0	fTotalStrokes	 19
U+9B09.0	fCangjie	SHUCE
U+9B09.0	fFourCorner	72400
U+9B09.0	fCantonese	zung1
U+9B09.0	fMandarin	zong1
U+9B0A.0	fUTF8	鬊
U+9B0A.0	fCNS11643	2-6830
U+9B0A.0	fBig5	F3C3
U+9B0A.0	fGB	3-7B53
U+9B0A.0	fR/S	 190.9
U+9B0A.0	fTotalStrokes	 19
U+9B0A.0	fCangjie	SHQKA
U+9B0A.0	fFourCorner	72603
U+9B0A.0	fCantonese	seon3
U+9B0A.0	fMandarin	shun4; chun3
U+9B0B.0	fUTF8	鬋
U+9B0B.0	fCNS11643	2-682F
U+9B0B.0	fBig5	F3C2
U+9B0B.0	fGB	3-7B57
U+9B0B.0	fR/S	 190.9
U+9B0B.0	fTotalStrokes	 19
U+9B0B.0	fCangjie	SHTBN
U+9B0B.0	fFourCorner	72221
U+9B0B.0	fHakka	  zien5; zien1; zien3
U+9B0B.0	fCantonese	zin1; zin2
U+9B0B.0	fMandarin	jian1; jian3; jian4
U+9B0B.0	fEnglish	forelock; bangs
U+9B0C.0	fUTF8	鬌
U+9B0C.0	fCNS11643	2-6832
U+9B0C.0	fBig5	F3C5
U+9B0C.0	fGB	3-7B54
U+9B0C.0	fR/S	 190.9
U+9B0C.0	fTotalStrokes	 19
U+9B0C.0	fCangjie	SHKMB
U+9B0C.0	fFourCorner	72227
U+9B0C.0	fHakka	  do3; to3
U+9B0C.0	fCantonese	do2; do6 (also); ceoi4
U+9B0C.0	fMandarin	duo3; duo4; tuo3; zhui1; sui1
U+9B0C.0	fEnglish	the tufts of hair left on a child's head when first shaved
U+9B0D.0	fUTF8	鬍
U+9B0D.0	fCNS11643	1-7953
U+9B0D.0	fBig5	C447
U+9B0D.0	fGB	1-783A
U+9B0D.0	fR/S	 190.9
U+9B0D.0	fTotalStrokes	 19
U+9B0D.0	fCangjie	SHJRB
U+9B0D.0	fFourCorner	72627
U+9B0D.0	fHakka	  fu2; fu3
U+9B0D.0	fCantonese	wu4
U+9B0D.0	fMandarin	hu2
U+9B0D.0	fEnglish	 beard
U+9B0E.0	fUTF8	鬎
U+9B0E.0	fCNS11643	2-6831
U+9B0E.0	fBig5	F3C4
U+9B0E.0	fGB	5-7945
U+9B0E.0	fR/S	 190.9
U+9B0E.0	fTotalStrokes	 19
U+9B0E.0	fCangjie	SHDBN
U+9B0E.0	fFourCorner	72922
U+9B0E.0	fCantonese	laat6 (also); laat3
U+9B0E.0	fMandarin	la4
U+9B0E.0	fEnglish	favus
U+9B0F.0	fUTF8	鬏
U+9B0F.0	fCNS11643	4-624F
U+9B0F.0	fGB	0-775D
U+9B0F.0	fR/S	  190.9
U+9B0F.0	fTotalStrokes	 19
U+9B0F.0	fCangjie	SHHDF
U+9B0F.0	fFourCorner	72980
U+9B0F.0	fCantonese	cau1 (also); zau1
U+9B0F.0	fMandarin	jiu1
U+9B0F.0	fEnglish	a coiffure on top of the head
U+9B10.0	fUTF8	鬐
U+9B10.0	fCNS11643	2-6A75
U+9B10.0	fBig5	F567
U+9B10.0	fGB	3-7B5A
U+9B10.0	fR/S	 190.10
U+9B10.0	fTotalStrokes	 20
U+9B10.0	fCangjie	SHJPA
U+9B10.0	fFourCorner	72221
U+9B10.0	fHakka	  ki2
U+9B10.0	fCantonese	kei4
U+9B10.0	fMandarin	qi2
U+9B10.0	fEnglish	[1] mane [2] fins
U+9B11.0	fUTF8	鬑
U+9B11.0	fCNS11643	2-6A77
U+9B11.0	fBig5	F569
U+9B11.0	fGB	3-7B60
U+9B11.0	fR/S	 190.10
U+9B11.0	fTotalStrokes	 20
U+9B11.0	fCangjie	SHTXC
U+9B11.0	fFourCorner	72237
U+9B11.0	fCantonese	lim4
U+9B11.0	fMandarin	lian2
U+9B11.0	fEnglish	hair hanging down the temples
U+9B12.0	fUTF8	鬒
U+9B12.0	fCNS11643	2-6A76
U+9B12.0	fBig5	F568
U+9B12.0	fGB	3-7B5C
U+9B12.0	fR/S	 190.10
U+9B12.0	fTotalStrokes	 20
U+9B12.0	fCangjie	SHJBC
U+9B12.0	fFourCorner	72801
U+9B12.0	fHakka	 zhin3; zihn3
U+9B12.0	fCantonese	can2; zan2 (also)
U+9B12.0	fMandarin	zhen3; zhen4
U+9B12.0	fEnglish	having dark, glossy hair
U+9B13.0	fUTF8	鬓
U+9B13.0	fGB	0-775E
U+9B13.0	fR/S	 190.10
U+9B13.0	fTotalStrokes	 20
U+9B13.0	fCangjie	SHJOC
U+9B13.0	fHakka	 bin5
U+9B13.0	fCantonese	ban3
U+9B13.0	fMandarin	bin4
U+9B13.0	fEnglish	simplified form of &#x9B22;
U+9B14.0	fUTF8	鬔
U+9B14.0	fCNS11643	3-5F27
U+9B14.0	fGB	E-465C
U+9B14.0	fR/S	 190.11
U+9B14.0	fTotalStrokes	 21
U+9B14.0	fCangjie	SHYHJ
U+9B14.0	fFourCorner	72300
U+9B14.0	fCantonese	pung4
U+9B14.0	fMandarin	peng2
U+9B15.0	fUTF8	鬕
U+9B15.0	fCNS11643	2-6C6F
U+9B15.0	fBig5	F6A3
U+9B15.0	fGB	3-7B5B
U+9B15.0	fR/S	 190.11
U+9B15.0	fTotalStrokes	 21
U+9B15.0	fCangjie	SHTAK
U+9B15.0	fFourCorner	72806
U+9B15.0	fCantonese	maa6
U+9B15.0	fMandarin	mo4; mai4
U+9B16.0	fUTF8	鬖
U+9B16.0	fCNS11643	2-6C72
U+9B16.0	fBig5	F6A6
U+9B16.0	fGB	3-7B52
U+9B16.0	fR/S	 190.11
U+9B16.0	fTotalStrokes	 21
U+9B16.0	fCangjie	SHIIH
U+9B16.0	fFourCorner	72202
U+9B16.0	fHakka	 cam1;  sem1; sam1
U+9B16.0	fCantonese	saam1
U+9B16.0	fMandarin	san1; san3; sen1; shan1
U+9B16.0	fEnglish	wild hair
U+9B17.0	fUTF8	鬗
U+9B17.0	fCNS11643	2-6C70
U+9B17.0	fBig5	F6A4
U+9B17.0	fGB	5-794B
U+9B17.0	fR/S	 190.11
U+9B17.0	fTotalStrokes	 21
U+9B17.0	fCangjie	SHTLB
U+9B17.0	fFourCorner	72227
U+9B17.0	fCantonese	mun4; maan6
U+9B17.0	fMandarin	man4; man2
U+9B18.0	fUTF8	鬘
U+9B18.0	fCNS11643	2-6C71
U+9B18.0	fBig5	F6A5
U+9B18.0	fGB	3-7B61
U+9B18.0	fR/S	 190.11
U+9B18.0	fTotalStrokes	 21
U+9B18.0	fCangjie	SHAWE
U+9B18.0	fFourCorner	72407
U+9B18.0	fHakka	 man2; man5
U+9B18.0	fCantonese	maan4
U+9B18.0	fMandarin	man2
U+9B18.0	fEnglish	beautiful hair; fair hair
U+9B19.0	fUTF8	鬙
U+9B19.0	fCNS11643	2-6E52
U+9B19.0	fBig5	F7A5
U+9B19.0	fGB	3-7B62
U+9B19.0	fR/S	 190.12
U+9B19.0	fTotalStrokes	 22
U+9B19.0	fCangjie	SHCWA
U+9B19.0	fFourCorner	72606
U+9B19.0	fHakka	  sen1; zen1
U+9B19.0	fCantonese	zang1; sang1
U+9B19.0	fMandarin	seng1
U+9B19.0	fEnglish	matted hair
U+9B1A.0	fUTF8	鬚
U+9B1A.0	fCNS11643	1-7C29
U+9B1A.0	fBig5	C5BD
U+9B1A.0	fGB	1-7876
U+9B1A.0	fR/S	 190.12
U+9B1A.0	fTotalStrokes	 22
U+9B1A.0	fCangjie	SHHHC
U+9B1A.0	fFourCorner	72286
U+9B1A.0	fHakka	  si1
U+9B1A.0	fCantonese	sou1
U+9B1A.0	fMandarin	xu1
U+9B1A.0	fEnglish	[1] beard; whiskers [2] whiskers (of a cat, etc.) [3] awn
U+9B1B.0	fUTF8	鬛
U+9B1B.0	fCNS11643	4-6974
U+9B1B.0	fGB	E-465D
U+9B1B.0	fR/S	 190.12
U+9B1B.0	fTotalStrokes	 22; 21
U+9B1B.0	fCangjie	SHWKV
U+9B1B.0	fFourCorner	72710
U+9B1B.0	fCantonese	lip6
U+9B1B.0	fMandarin	lie4
U+9B1B.0	fEnglish	variant of &#x9B23;
U+9B1C.0	fUTF8	鬜
U+9B1C.0	fCNS11643	4-6971
U+9B1C.0	fGB	3-7B49
U+9B1C.0	fR/S	  190.12
U+9B1C.0	fTotalStrokes	 22; 21
U+9B1C.0	fCangjie	SHANB
U+9B1C.0	fFourCorner	72220
U+9B1C.0	fCantonese	haan1
U+9B1C.0	fMandarin	qian1; chan2
U+9B1C.0	fEnglish	 go bald
U+9B1D.0	fUTF8	鬝
U+9B1D.0	fCNS11643	3-6672
U+9B1D.0	fGB	E-465E
U+9B1D.0	fR/S	 190.11
U+9B1D.0	fTotalStrokes	 21
U+9B1D.0	fCangjie	SHANA
U+9B1D.0	fFourCorner	7222
U+9B1D.0	fCantonese	haan1
U+9B1D.0	fMandarin	qian1
U+9B1E.0	fUTF8	鬞
U+9B1E.0	fCNS11643	2-6F62
U+9B1E.0	fBig5	F854
U+9B1E.0	fGB	5-7936
U+9B1E.0	fR/S	 190.13
U+9B1E.0	fTotalStrokes	 23
U+9B1E.0	fCangjie	SHTWV
U+9B1E.0	fFourCorner	72232
U+9B1E.0	fCantonese	nong4; long4 (also); nung4
U+9B1E.0	fMandarin	nong2
U+9B1F.0	fUTF8	鬟
U+9B1F.0	fCNS11643	2-6F63
U+9B1F.0	fBig5	F855
U+9B1F.0	fGB	0-775F
U+9B1F.0	fR/S	 190.13
U+9B1F.0	fTotalStrokes	 23
U+9B1F.0	fCangjie	SHWLV
U+9B1F.0	fFourCorner	72732
U+9B1F.0	fHakka	  fan2; wan2
U+9B1F.0	fCantonese	waan4
U+9B1F.0	fMandarin	huan2
U+9B1F.0	fEnglish	[1] dress hair in coiled knot [2] female servant; maid
U+9B20.0	fUTF8	鬠
U+9B20.0	fCNS11643	2-6F64
U+9B20.0	fBig5	F856
U+9B20.0	fGB	5-7932
U+9B20.0	fR/S	 190.13
U+9B20.0	fTotalStrokes	 23
U+9B20.0	fCangjie	SHOMA
U+9B20.0	fFourCorner	72606
U+9B20.0	fCantonese	kut3; kwut3 (also); kui3; zang3
U+9B20.0	fMandarin	kuai4; huo2; kuo4
U+9B20.0	fEnglish	 make a top knot
U+9B21.0	fUTF8	鬡
U+9B21.0	fCNS11643	3-612C
U+9B21.0	fGB	3-7B3D
U+9B21.0	fR/S	  190.14
U+9B21.0	fTotalStrokes	 24
U+9B21.0	fCangjie	SHJPN
U+9B21.0	fFourCorner	72200
U+9B21.0	fHakka	  nen2
U+9B21.0	fCantonese	ning4; ling4 (also)
U+9B21.0	fMandarin	ning2
U+9B21.0	fEnglish	[1] tranquillity [2] peace [3] rest
U+9B22.0	fUTF8	鬢
U+9B22.0	fCNS11643	1-7C76
U+9B22.0	fBig5	C64B
U+9B22.0	fGB	1-775E
U+9B22.0	fR/S	 190.14
U+9B22.0	fTotalStrokes	 24
U+9B22.0	fCangjie	SHJMC
U+9B22.0	fFourCorner	72806
U+9B22.0	fHakka	 bin5; bin1
U+9B22.0	fCantonese	ban3
U+9B22.0	fMandarin	bin4
U+9B22.0	fEnglish	hair on the temples
U+9B23.0	fUTF8	鬣
U+9B23.0	fCNS11643	1-7D30
U+9B23.0	fBig5	C663
U+9B23.0	fGB	0-7760
U+9B23.0	fR/S	 190.15
U+9B23.0	fTotalStrokes	 25
U+9B23.0	fCangjie	SHVVV
U+9B23.0	fFourCorner	72716
U+9B23.0	fHakka	 liap8
U+9B23.0	fCantonese	lip6
U+9B23.0	fMandarin	lie4; she4
U+9B23.0	fEnglish	[1] long beard; long whiskers [2] mane [3] fin
U+9B24.0	fUTF8	鬤
U+9B24.0	fCNS11643	2-7226
U+9B24.0	fBig5	F9B6
U+9B24.0	fGB	3-7B66
U+9B24.0	fR/S	 190.17
U+9B24.0	fTotalStrokes	 27
U+9B24.0	fCangjie	SHYRV
U+9B24.0	fFourCorner	72732
U+9B24.0	fHakka	 jong2; rong2
U+9B24.0	fCantonese	joeng4; joeng5
U+9B24.0	fMandarin	rang2; rang3; ning2
U+9B24.0	fEnglish	unkempt hair
U+9B25.0	fUTF8	鬥
U+9B25.0	fCNS11643	1-586E
U+9B25.0	fBig5	B0AB
U+9B25.0	fGB	1-3637
U+9B25.0	fR/S	 191.0
U+9B25.0	fTotalStrokes	 10
U+9B25.0	fCangjie	LN
U+9B25.0	fFourCorner	77114
U+9B25.0	fHakka	 deu5; diu5
U+9B25.0	fCantonese	dau3; dau2
U+9B25.0	fMandarin	dou4; dou3
U+9B25.0	fEnglish	[1] same as $ [2] Kangxi radical 190
U+9B26.0	fUTF8	鬦
U+9B26.0	fCNS11643	3-4A75
U+9B26.0	fGB	E-465F
U+9B26.0	fR/S	 191.4
U+9B26.0	fTotalStrokes	 14
U+9B26.0	fCangjie	LNYJ
U+9B26.0	fFourCorner	77400
U+9B26.0	fCantonese	dau3
U+9B26.0	fMandarin	dou4; dou3
U+9B26.0	fEnglish	same as $
U+9B27.0	fUTF8	鬧
U+9B27.0	fCNS11643	1-7023
U+9B27.0	fBig5	BE78
U+9B27.0	fGB	1-4456
U+9B27.0	fR/S	 191.5
U+9B27.0	fTotalStrokes	 15
U+9B27.0	fCangjie	LNYLB
U+9B27.0	fFourCorner	77227
U+9B27.0	fHakka	 lau5; nau6;  nau5
U+9B27.0	fCantonese	naau6; laau6 (also)
U+9B27.0	fMandarin	nao4
U+9B27.0	fEnglish	[1] disturb; agitate; trouble [2] have; experience (disasters, sickness, etc.) [3] noisy; uproarious; stormy; clamorous
U+9B27.1	fUTF8	鬧
U+9B27.1	fCNS11643	1-7023
U+9B27.1	fBig5	BE78
U+9B27.1	fGB	1-4456
U+9B27.1	fR/S	 191.5
U+9B27.1	fTotalStrokes	 15
U+9B27.1	fCangjie	LNYLB
U+9B27.1	fFourCorner	77227
U+9B27.1	fMandarin	nao5
U+9B28.0	fUTF8	鬨
U+9B28.0	fCNS11643	1-7334
U+9B28.0	fBig5	C069
U+9B28.0	fGB	E-4660
U+9B28.0	fR/S	 191.6
U+9B28.0	fTotalStrokes	 16
U+9B28.0	fCangjie	LNTC
U+9B28.0	fFourCorner	77801
U+9B28.0	fHakka	  fung5; fung3
U+9B28.0	fCantonese	hung3; hung6 (also); hong6
U+9B28.0	fMandarin	hong4; xiang4; hong1; hong3
U+9B28.0	fEnglish	[1] be boisterous; be uproarious; uproar; clamor; noise [2] dispute; quarrel
U+9B29.0	fUTF8	鬩
U+9B29.0	fCNS11643	2-6468
U+9B29.0	fBig5	F1BE
U+9B29.0	fGB	1-6352
U+9B29.0	fR/S	 191.8
U+9B29.0	fTotalStrokes	 18
U+9B29.0	fCangjie	LNHXU
U+9B29.0	fFourCorner	77217
U+9B29.0	fHakka	 hit7;  ngiet7; nget7
U+9B29.0	fCantonese	jik1; hik1
U+9B29.0	fMandarin	xi4; he4
U+9B29.0	fEnglish	 quarrel; conflict
U+9B2A.0	fUTF8	鬪
U+9B2A.0	fCNS11643	3-5D62
U+9B2A.0	fGB	E-4661
U+9B2A.0	fR/S	 191.10; .10
U+9B2A.0	fTotalStrokes	 20
U+9B2A.0	fCangjie	LNMTI
U+9B2A.0	fFourCorner	77140
U+9B2A.0	fHakka	  deu5; diu5
U+9B2A.0	fCantonese	dau3
U+9B2A.0	fMandarin	dou4; dou3
U+9B2A.0	fEnglish	 struggle
U+9B2B.0	fUTF8	鬫
U+9B2B.0	fCNS11643	2-6E53
U+9B2B.0	fBig5	F7A6
U+9B2B.0	fGB	E-4662
U+9B2B.0	fR/S	 191.12
U+9B2B.0	fTotalStrokes	 22
U+9B2B.0	fCangjie	LNMJK
U+9B2B.0	fFourCorner	77148
U+9B2B.0	fHakka	 kam3; kam5
U+9B2B.0	fCantonese	haam2 (also); haam3; ham3
U+9B2B.0	fMandarin	han3; kan3; kan4
U+9B2B.0	fEnglish	growl of angry animal, savage
U+9B2C.0	fUTF8	鬬
U+9B2C.0	fCNS11643	3-612E
U+9B2C.0	fGB	E-4663
U+9B2C.0	fR/S	 191.14
U+9B2C.0	fTotalStrokes	 24
U+9B2C.0	fCangjie	LNBML
U+9B2C.0	fFourCorner	77120
U+9B2C.0	fCantonese	dau3
U+9B2C.0	fMandarin	dou4
U+9B2C.0	fEnglish	struggle
U+9B2D.0	fUTF8	鬭
U+9B2D.0	fCNS11643	4-6D37
U+9B2D.0	fGB	E-4664
U+9B2D.0	fR/S	 191.14; .14
U+9B2D.0	fTotalStrokes	 24; 25
U+9B2D.0	fCangjie	LNRML
U+9B2D.0	fFourCorner	77120
U+9B2D.0	fHakka	  deu5; diu5
U+9B2D.0	fCantonese	dau3
U+9B2D.0	fMandarin	dou4; dou3
U+9B2D.0	fEnglish	 struggle; fight; contend; conflict; vie; compete; quarrelsome
U+9B2E.0	fUTF8	鬮
U+9B2E.0	fCNS11643	2-7166
U+9B2E.0	fBig5	F9C4
U+9B2E.0	fGB	1-634E
U+9B2E.0	fR/S	 191.17; 191.16
U+9B2E.0	fTotalStrokes	 27
U+9B2E.0	fCangjie	LNNXU
U+9B2E.0	fFourCorner	77117
U+9B2E.0	fHakka	 keu1; k'ieu1; kieu1; kiu1; geu1
U+9B2E.0	fCantonese	gau1; kau1
U+9B2E.0	fMandarin	jiu1
U+9B2E.0	fEnglish	lots (to be drawn)
U+9B2F.0	fUTF8	鬯
U+9B2F.0	fCNS11643	2-334D
U+9B2F.0	fBig5	D44A
U+9B2F.0	fGB	0-5B4B
U+9B2F.0	fR/S	 192.0
U+9B2F.0	fTotalStrokes	 10
U+9B2F.0	fCangjie	UIP
U+9B2F.0	fFourCorner	22711
U+9B2F.0	fHakka	 chong3; cong3; zong3
U+9B2F.0	fCantonese	coeng3
U+9B2F.0	fMandarin	chang4
U+9B2F.0	fEnglish	[1] sacrificial wine, spirits; herb-flavored spirits [2] same as $ [3] Kangxi radical 192
U+9B30.0	fUTF8	鬰
U+9B30.0	fCNS11643	3-6228
U+9B30.0	fGB	E-4665
U+9B30.0	fR/S	 192.17
U+9B30.0	fTotalStrokes	 27
U+9B30.0	fCangjie	DDBUH
U+9B30.0	fFourCorner	44720
U+9B30.0	fCantonese	wat1
U+9B30.0	fMandarin	yu4
U+9B30.0	fEnglish	 luxuriant dense; thick moody
U+9B31.0	fUTF8	鬱
U+9B31.0	fCNS11643	1-7D48
U+9B31.0	fBig5	C67B
U+9B31.0	fGB	1-5374
U+9B31.0	fR/S	 192.19
U+9B31.0	fTotalStrokes	 29
U+9B31.0	fCangjie	DDBUH
U+9B31.0	fFourCorner	44722
U+9B31.0	fHakka	  ngut7; jut7;  wut7; rut7
U+9B31.0	fCantonese	wat1
U+9B31.0	fMandarin	yu4
U+9B31.0	fEnglish	[1] the tulip [2] the plum [3] held in check; pent-up; stagnant [4] luxuriant; dense; thick; lush
U+9B32.0	fUTF8	鬲
U+9B32.0	fCNS11643	1-586F
U+9B32.0	fBig5	B0AC
U+9B32.0	fGB	0-582A
U+9B32.0	fR/S	 193.0
U+9B32.0	fTotalStrokes	 10
U+9B32.0	fCangjie	MRBL
U+9B32.0	fFourCorner	10227
U+9B32.0	fHakka	 lit8; gak7
U+9B32.0	fCantonese	lik6; gaak3; ak1; ngak1 (also)
U+9B32.0	fMandarin	ge2; e1; e4
U+9B32.0	fEnglish	[1] same as $ [2] an ancient state [3] the name of an ancient sage [4] Kangxi radical 193
U+9B32.1	fUTF8	鬲
U+9B32.1	fCNS11643	1-586F
U+9B32.1	fBig5	B0AC
U+9B32.1	fGB	0-582A
U+9B32.1	fR/S	 193.0
U+9B32.1	fTotalStrokes	 10
U+9B32.1	fCangjie	MRBL
U+9B32.1	fFourCorner	10227
U+9B32.1	fMandarin	li4
U+9B32.1	fEnglish	a kind of caldron
U+9B33.0	fUTF8	鬳
U+9B33.0	fCNS11643	2-5B7E
U+9B33.0	fBig5	EC72
U+9B33.0	fGB	3-6147
U+9B33.0	fR/S	 193.6
U+9B33.0	fTotalStrokes	 16
U+9B33.0	fCangjie	YPMRB
U+9B33.0	fFourCorner	21227
U+9B33.0	fCantonese	jin6; gyun3
U+9B33.0	fMandarin	juan4
U+9B34.0	fUTF8	鬴
U+9B34.0	fCNS11643	3-573E
U+9B34.0	fGB	5-7959
U+9B34.0	fR/S	 193.7
U+9B34.0	fTotalStrokes	 17
U+9B34.0	fCangjie	MBIJB
U+9B34.0	fFourCorner	13220
U+9B34.0	fHakka	 fu3;  pu3
U+9B34.0	fCantonese	fu2
U+9B34.0	fMandarin	fu3; li4
U+9B34.0	fEnglish	pot
U+9B35.0	fUTF8	鬵
U+9B35.0	fCNS11643	2-6469
U+9B35.0	fBig5	F1BF
U+9B35.0	fGB	3-7B6B
U+9B35.0	fR/S	 193.8
U+9B35.0	fTotalStrokes	 18
U+9B35.0	fCangjie	MUMRB
U+9B35.0	fFourCorner	71227
U+9B35.0	fHakka	  cim2
U+9B35.0	fCantonese	cam4; cim4 (also)
U+9B35.0	fMandarin	qian2; xun2; xin2
U+9B35.0	fEnglish	big iron pot; caldron
U+9B36.0	fUTF8	鬶
U+9B36.0	fGB	8-2C56
U+9B36.0	fR/S	 193.8
U+9B36.0	fTotalStrokes	 18
U+9B36.0	fCangjie	XQUMR
U+9B36.0	fCantonese	kwai1
U+9B36.0	fMandarin	gui1
U+9B37.0	fUTF8	鬷
U+9B37.0	fCNS11643	2-6833
U+9B37.0	fBig5	F3C6
U+9B37.0	fGB	3-7B6C
U+9B37.0	fR/S	 193.9
U+9B37.0	fTotalStrokes	 19
U+9B37.0	fCangjie	MBUCE
U+9B37.0	fFourCorner	12247
U+9B37.0	fHakka	  zung1
U+9B37.0	fCantonese	zung1
U+9B37.0	fMandarin	zong1; zong3; zong4
U+9B37.0	fEnglish	pot
U+9B38.0	fUTF8	鬸
U+9B38.0	fCNS11643	3-5D63
U+9B38.0	fGB	E-4666
U+9B38.0	fR/S	  193.10
U+9B38.0	fTotalStrokes	 20
U+9B38.0	fCangjie	MBHHW
U+9B38.0	fFourCorner	17260
U+9B38.0	fCantonese	lau6
U+9B38.0	fMandarin	liu4
U+9B39.0	fUTF8	鬹
U+9B39.0	fCNS11643	4-6836
U+9B39.0	fGB	3-7B6A
U+9B39.0	fR/S	 193.11
U+9B39.0	fTotalStrokes	 21
U+9B39.0	fCangjie	QUMRB
U+9B39.0	fFourCorner	56220
U+9B39.0	fCantonese	kwai1
U+9B39.0	fMandarin	gui1; xi1
U+9B3A.0	fUTF8	鬺
U+9B3A.0	fCNS11643	2-6C73
U+9B3A.0	fBig5	F6A7
U+9B3A.0	fGB	3-7B68
U+9B3A.0	fR/S	 193.11
U+9B3A.0	fTotalStrokes	 21
U+9B3A.0	fCangjie	MBOAH
U+9B3A.0	fFourCorner	18227
U+9B3A.0	fCantonese	soeng1
U+9B3A.0	fMandarin	shang1
U+9B3B.0	fUTF8	鬻
U+9B3B.0	fCNS11643	2-6E54
U+9B3B.0	fBig5	F7A7
U+9B3B.0	fGB	0-6577
U+9B3B.0	fR/S	 193.12
U+9B3B.0	fTotalStrokes	 22
U+9B3B.0	fCangjie	NNMRB
U+9B3B.0	fFourCorner	17227
U+9B3B.0	fHakka	 zuk7; chuk7; juk7
U+9B3B.0	fCantonese	juk6; zuk1
U+9B3B.0	fMandarin	yu4; zhu4; mi2; zhou1; zhu3; ju4; ju1
U+9B3B.0	fEnglish	[1] sell [2] young; childish [3] nourish; bring up
U+9B3C.0	fUTF8	鬼
U+9B3C.0	fCNS11643	1-5870
U+9B3C.0	fBig5	B0AD
U+9B3C.0	fGB	0-396D
U+9B3C.0	fR/S	 194.0
U+9B3C.0	fTotalStrokes	 10
U+9B3C.0	fCangjie	HUI
U+9B3C.0	fFourCorner	26213
U+9B3C.0	fHakka	  gui3; gwi3; kwui3
U+9B3C.0	fCantonese	gwai2
U+9B3C.0	fMandarin	gui3
U+9B3C.0	fEnglish	[1] ghost; spirit; devil; demon [2] cunning; crafty; wily; deceithful [3] sinister; dark; evil; dirty trick; dirty work [4] Kangxi radical 194
U+9B3D.0	fUTF8	鬽
U+9B3D.0	fCNS11643	3-4562
U+9B3D.0	fGB	5-7834
U+9B3D.0	fR/S	  194.3
U+9B3D.0	fTotalStrokes	 13
U+9B3D.0	fCangjie	HIHHH
U+9B3D.0	fFourCorner	22210
U+9B3D.0	fHakka	 mi5; mi6; mui5; mui6
U+9B3D.0	fCantonese	mei6; mui6 (also)
U+9B3D.0	fMandarin	mei4
U+9B3D.0	fEnglish	[1] ghost [2] bogey
U+9B3E.0	fUTF8	鬾
U+9B3E.0	fCNS11643	2-4F45
U+9B3E.0	fBig5	E4DD
U+9B3E.0	fGB	3-7A34
U+9B3E.0	fR/S	 194.4
U+9B3E.0	fTotalStrokes	 14
U+9B3E.0	fCangjie	HIJE
U+9B3E.0	fFourCorner	24214
U+9B3E.0	fCantonese	gei6
U+9B3E.0	fMandarin	ji4; qi2
U+9B3F.0	fUTF8	鬿
U+9B3F.0	fCNS11643	2-4F46
U+9B3F.0	fBig5	E4DE
U+9B3F.0	fGB	3-7A35
U+9B3F.0	fR/S	 194.4
U+9B3F.0	fTotalStrokes	 14
U+9B3F.0	fCangjie	HIHML
U+9B3F.0	fFourCorner	22212
U+9B3F.0	fCantonese	kei4
U+9B3F.0	fMandarin	qi2
U+9B40.0	fUTF8	魀
U+9B40.0	fCNS11643	3-4A78
U+9B40.0	fGB	5-7836
U+9B40.0	fR/S	 194.4
U+9B40.0	fTotalStrokes	 14
U+9B40.0	fCangjie	HIOLL
U+9B40.0	fFourCorner	28210
U+9B40.0	fCantonese	gaai3
U+9B40.0	fMandarin	jie4
U+9B41.0	fUTF8	魁
U+9B41.0	fCNS11643	1-6B76
U+9B41.0	fBig5	BBED
U+9B41.0	fGB	0-3F7D
U+9B41.0	fR/S	 194.4
U+9B41.0	fTotalStrokes	 14
U+9B41.0	fCangjie	HIYJ
U+9B41.0	fFourCorner	24210
U+9B41.0	fHakka	 kui2; kui3; foi1; kwi2; k'wui2
U+9B41.0	fCantonese	fui1
U+9B41.0	fMandarin	kui2; kui3; kuai4
U+9B41.0	fEnglish	[1] chief; leader; head [2] tall; big; great
U+9B42.0	fUTF8	魂
U+9B42.0	fCNS11643	1-6B77
U+9B42.0	fBig5	BBEE
U+9B42.0	fGB	0-3B6A
U+9B42.0	fR/S	 194.4
U+9B42.0	fTotalStrokes	 14
U+9B42.0	fCangjie	MIHUI
U+9B42.0	fFourCorner	16713
U+9B42.0	fHakka	  fun2; fun3; jun2
U+9B42.0	fCantonese	wan4
U+9B42.0	fMandarin	hun2
U+9B42.0	fEnglish	 soul; spirit
U+9B43.0	fUTF8	魃
U+9B43.0	fCNS11643	2-5624
U+9B43.0	fBig5	E8D9
U+9B43.0	fGB	0-7749
U+9B43.0	fR/S	 194.5
U+9B43.0	fTotalStrokes	 15
U+9B43.0	fCangjie	HIIKK
U+9B43.0	fFourCorner	23214
U+9B43.0	fHakka	  pat8
U+9B43.0	fCantonese	baat6 (also); bat6
U+9B43.0	fMandarin	ba2
U+9B43.0	fEnglish	drought demon
U+9B44.0	fUTF8	魄
U+9B44.0	fCNS11643	1-7025
U+9B44.0	fBig5	BE7A
U+9B44.0	fGB	0-4647
U+9B44.0	fR/S	 194.5
U+9B44.0	fTotalStrokes	 15
U+9B44.0	fCangjie	HAHUI
U+9B44.0	fFourCorner	26613
U+9B44.0	fHakka	  pak7
U+9B44.0	fCantonese	paak3; tok3; bok6; bok3
U+9B44.0	fMandarin	po4; bo2
U+9B44.0	fEnglish	[1] vigor; animation; life (Taoism) [2] form; shape; body [3] the dark part of the moon
U+9B44.1	fUTF8	魄
U+9B44.1	fCNS11643	1-7025
U+9B44.1	fBig5	BE7A
U+9B44.1	fGB	0-4647
U+9B44.1	fR/S	 194.5
U+9B44.1	fTotalStrokes	 15
U+9B44.1	fCangjie	HAHUI
U+9B44.1	fFourCorner	26613
U+9B44.1	fMandarin	tuo4
U+9B44.1	fEnglish	correct form of &#xdispirited;out of luck
U+9B45.0	fUTF8	魅
U+9B45.0	fCNS11643	1-7024
U+9B45.0	fBig5	BE79
U+9B45.0	fGB	0-7748
U+9B45.0	fR/S	 194.5
U+9B45.0	fTotalStrokes	 15
U+9B45.0	fCangjie	HIJD
U+9B45.0	fFourCorner	25219
U+9B45.0	fHakka	 mui5; mi5; mui3
U+9B45.0	fCantonese	mei6; mui6 (also)
U+9B45.0	fMandarin	mei4
U+9B45.0	fEnglish	[1] mischievouss spirit; goblin; elf [2] charm; mislead
U+9B46.0	fUTF8	魆
U+9B46.0	fCNS11643	2-5623
U+9B46.0	fBig5	E8D8
U+9B46.0	fGB	3-7A37
U+9B46.0	fR/S	 194.5
U+9B46.0	fTotalStrokes	 15
U+9B46.0	fCangjie	HIIV
U+9B46.0	fFourCorner	23115
U+9B46.0	fCantonese	fat1 (also); wat1; jyut6
U+9B46.0	fMandarin	xu1; xu4
U+9B46.0	fEnglish	black
U+9B47.0	fUTF8	魇
U+9B47.0	fGB	0-774A
U+9B47.0	fR/S	 194.6
U+9B47.0	fTotalStrokes	 16
U+9B47.0	fCangjie	MIKI
U+9B47.0	fHakka	 jam3; ap7
U+9B47.0	fCantonese	jim2
U+9B47.0	fMandarin	yan3; yan4; ya1
U+9B47.0	fEnglish	simplified form of &#x9B58;
U+9B48.0	fUTF8	魈
U+9B48.0	fCNS11643	2-6075
U+9B48.0	fBig5	EF69
U+9B48.0	fGB	0-774C
U+9B48.0	fR/S	 194.7
U+9B48.0	fTotalStrokes	 17
U+9B48.0	fCangjie	HIFB
U+9B48.0	fFourCorner	29212
U+9B48.0	fHakka	  siau1; seu1
U+9B48.0	fCantonese	siu1
U+9B48.0	fMandarin	xiao1
U+9B48.0	fEnglish	mischevious; one footed spirit that dwells in the mountains
U+9B49.0	fUTF8	魉
U+9B49.0	fCNS11643	3-573F
U+9B49.0	fGB	0-774B
U+9B49.0	fR/S	 194.7
U+9B49.0	fTotalStrokes	 17
U+9B49.0	fCangjie	HIMOB
U+9B49.0	fFourCorner	21212
U+9B49.0	fHakka	 liong3
U+9B49.0	fCantonese	loeng5
U+9B49.0	fMandarin	liang3
U+9B49.0	fEnglish	simplified form of &#x9B4E;
U+9B4A.0	fUTF8	魊
U+9B4A.0	fCNS11643	2-646A
U+9B4A.0	fBig5	F1C0
U+9B4A.0	fGB	3-7A3E
U+9B4A.0	fR/S	 194.8
U+9B4A.0	fTotalStrokes	 18
U+9B4A.0	fCangjie	HIIRM
U+9B4A.0	fFourCorner	23215
U+9B4A.0	fCantonese	wik6
U+9B4A.0	fMandarin	yu4
U+9B4A.0	fEnglish	a mythical creature which supposedly hurts human beings by casting sand out of its mouth
U+9B4B.0	fUTF8	魋
U+9B4B.0	fCNS11643	2-646C
U+9B4B.0	fBig5	F1C2
U+9B4B.0	fGB	3-7A40
U+9B4B.0	fR/S	 194.8
U+9B4B.0	fTotalStrokes	 18
U+9B4B.0	fCangjie	HIOG
U+9B4B.0	fFourCorner	20211
U+9B4B.0	fHakka	  tui2; kui2; k'wui3
U+9B4B.0	fCantonese	teoi4; zeoi1; ceoi4
U+9B4B.0	fMandarin	tui2; zhui1; chui2
U+9B4B.0	fEnglish	bear
U+9B4C.0	fUTF8	魌
U+9B4C.0	fCNS11643	2-646B
U+9B4C.0	fBig5	F1C1
U+9B4C.0	fGB	3-7A3D
U+9B4C.0	fR/S	 194.8
U+9B4C.0	fTotalStrokes	 18
U+9B4C.0	fCangjie	HITMC
U+9B4C.0	fFourCorner	24218
U+9B4C.0	fHakka	 ki2
U+9B4C.0	fCantonese	hei1; kei1
U+9B4C.0	fMandarin	qi1
U+9B4C.0	fEnglish	ugly demon
U+9B4D.0	fUTF8	魍
U+9B4D.0	fCNS11643	1-777D
U+9B4D.0	fBig5	C353
U+9B4D.0	fGB	0-774D
U+9B4D.0	fR/S	 194.8
U+9B4D.0	fTotalStrokes	 18
U+9B4D.0	fCangjie	HIBTV
U+9B4D.0	fFourCorner	27212
U+9B4D.0	fHakka	  miong3;  wong3; mong3; vong3
U+9B4D.0	fCantonese	mong5
U+9B4D.0	fMandarin	wang3
U+9B4D.0	fEnglish	a kind of monster
U+9B4E.0	fUTF8	魎
U+9B4E.0	fCNS11643	1-777C
U+9B4E.0	fBig5	C352
U+9B4E.0	fGB	1-774B
U+9B4E.0	fR/S	 194.8
U+9B4E.0	fTotalStrokes	 18
U+9B4E.0	fCangjie	HIMLB
U+9B4E.0	fFourCorner	21212
U+9B4E.0	fHakka	 liong3
U+9B4E.0	fCantonese	loeng5
U+9B4E.0	fMandarin	liang3
U+9B4E.0	fEnglish	a kind of monster
U+9B4F.0	fUTF8	魏
U+9B4F.0	fCNS11643	1-777B
U+9B4F.0	fBig5	C351
U+9B4F.0	fGB	0-4E3A
U+9B4F.0	fR/S	 194.8
U+9B4F.0	fTotalStrokes	 18
U+9B4F.0	fCangjie	HVHUI
U+9B4F.0	fFourCorner	26413
U+9B4F.0	fHakka	 ngui3; ngui6; ngui5
U+9B4F.0	fCantonese	ngai6; ai6 (also); ngai4
U+9B4F.0	fMandarin	wei4; wei2; wei1
U+9B4F.0	fEnglish	[1] Wei, a kingdom during the Warring States period [2] lofty; stately; magnificient [3] a Chinese family name
U+9B50.0	fUTF8	魐
U+9B50.0	fCNS11643	3-5D64
U+9B50.0	fGB	5-7841
U+9B50.0	fR/S	 194.10
U+9B50.0	fTotalStrokes	 20
U+9B50.0	fCangjie	HITXC
U+9B50.0	fFourCorner	28210
U+9B50.0	fCantonese	gaam1
U+9B50.0	fMandarin	jian1
U+9B51.0	fUTF8	魑
U+9B51.0	fCNS11643	1-7B4A
U+9B51.0	fBig5	C55E
U+9B51.0	fGB	0-774E
U+9B51.0	fR/S	 194.11
U+9B51.0	fTotalStrokes	 21
U+9B51.0	fCangjie	HIYUB
U+9B51.0	fFourCorner	20212
U+9B51.0	fHakka	  li2; ci1; mi2
U+9B51.0	fCantonese	ci1; lei4
U+9B51.0	fMandarin	chi1; qin2
U+9B51.0	fEnglish	a montain demon resembling a tiger
U+9B52.0	fUTF8	魒
U+9B52.0	fCNS11643	2-6C74
U+9B52.0	fBig5	F6A8
U+9B52.0	fGB	E-4667
U+9B52.0	fR/S	 194.11
U+9B52.0	fTotalStrokes	 21
U+9B52.0	fCangjie	HIMWF
U+9B52.0	fFourCorner	21219
U+9B52.0	fHakka	 piau1; peu1
U+9B52.0	fCantonese	piu1
U+9B52.0	fMandarin	piao1
U+9B52.0	fEnglish	a spirit living in one ot the stars of the Great Bear
U+9B53.0	fUTF8	魓
U+9B53.0	fCNS11643	3-5F28
U+9B53.0	fGB	E-4668
U+9B53.0	fR/S	  194.11
U+9B53.0	fTotalStrokes	 21
U+9B53.0	fCangjie	HIWTJ
U+9B53.0	fFourCorner	26210
U+9B53.0	fCantonese	bat1
U+9B53.0	fMandarin	bi4
U+9B54.0	fUTF8	魔
U+9B54.0	fCNS11643	1-7B49
U+9B54.0	fBig5	C55D
U+9B54.0	fGB	0-4427
U+9B54.0	fR/S	 194.11
U+9B54.0	fTotalStrokes	 21
U+9B54.0	fCangjie	IDHUI
U+9B54.0	fFourCorner	00213
U+9B54.0	fHakka	 mo2; mo1; mo3
U+9B54.0	fCantonese	mo1; mo4
U+9B54.0	fMandarin	mo2
U+9B54.0	fEnglish	 wizzard; witch; demon power
U+9B55.0	fUTF8	魕
U+9B55.0	fCNS11643	2-6E56
U+9B55.0	fBig5	F7A9
U+9B55.0	fGB	3-7A33
U+9B55.0	fR/S	 194.12
U+9B55.0	fTotalStrokes	 22
U+9B55.0	fCangjie	HIVII
U+9B55.0	fFourCorner	22253
U+9B55.0	fHakka	 gi1; ki2; ki5
U+9B55.0	fCantonese	kei4; gei1
U+9B55.0	fMandarin	ji3; qi2
U+9B55.0	fEnglish	[1] a demon which bewilders people [2] devilish
U+9B56.0	fUTF8	魖
U+9B56.0	fCNS11643	2-6E55
U+9B56.0	fBig5	F7A8
U+9B56.0	fGB	3-7A42
U+9B56.0	fR/S	 194.12
U+9B56.0	fTotalStrokes	 22
U+9B56.0	fCangjie	HIYPM
U+9B56.0	fFourCorner	21111
U+9B56.0	fCantonese	heoi1
U+9B56.0	fMandarin	xu1
U+9B57.0	fUTF8	魗
U+9B57.0	fCNS11643	3-612F
U+9B57.0	fGB	5-783D
U+9B57.0	fR/S	  194.14
U+9B57.0	fTotalStrokes	 24
U+9B57.0	fCangjie	GIHUI
U+9B57.0	fFourCorner	46610
U+9B57.0	fCantonese	cau4; cau2 (also)
U+9B57.0	fMandarin	chou3; chou2
U+9B58.0	fUTF8	魘
U+9B58.0	fCNS11643	1-7C77
U+9B58.0	fBig5	C64C
U+9B58.0	fGB	1-774A
U+9B58.0	fR/S	 194.14
U+9B58.0	fTotalStrokes	 24
U+9B58.0	fCangjie	MKHUI
U+9B58.0	fFourCorner	71213
U+9B58.0	fHakka	 ap7; ram5; jam5; jam3
U+9B58.0	fCantonese	jim2; jip3
U+9B58.0	fMandarin	yan3; yan4; ya1
U+9B58.0	fEnglish	 nightmare; bad dreams
U+9B59.0	fUTF8	魙
U+9B59.0	fCNS11643	2-7063
U+9B59.0	fBig5	F8D5
U+9B59.0	fGB	5-7843
U+9B59.0	fR/S	 194.14
U+9B59.0	fTotalStrokes	 24
U+9B59.0	fCangjie	ELHUI
U+9B59.0	fFourCorner	32413
U+9B59.0	fHakka	 zam3; ciam3; zit7; ciam5
U+9B59.0	fCantonese	zim6
U+9B59.0	fMandarin	zhan3; jian4
U+9B59.0	fEnglish	the death of a disembodied spirit
U+9B5A.0	fUTF8	魚
U+9B5A.0	fCNS11643	1-5E23
U+9B5A.0	fBig5	B3BD
U+9B5A.0	fGB	1-5363
U+9B5A.0	fR/S	 195.0
U+9B5A.0	fTotalStrokes	 11
U+9B5A.0	fCangjie	NWF
U+9B5A.0	fFourCorner	27336
U+9B5A.0	fHakka	  ng2; ng3; ngiu2
U+9B5A.0	fCantonese	jyu4; jyu2 (also)
U+9B5A.0	fMandarin	yu2
U+9B5A.0	fEnglish	[1] fish [2] Kangxi radical 195
U+9B5B.0	fUTF8	魛
U+9B5B.0	fCNS11643	2-4870
U+9B5B.0	fBig5	E0EA
U+9B5B.0	fGB	3-773B
U+9B5B.0	fR/S	 195.2
U+9B5B.0	fTotalStrokes	 13
U+9B5B.0	fCangjie	NFSH
U+9B5B.0	fFourCorner	27320
U+9B5B.0	fHakka	  dau1; do1
U+9B5B.0	fCantonese	dou1
U+9B5B.0	fMandarin	dao1
U+9B5B.0	fEnglish	the mullet
U+9B5C.0	fUTF8	魜
U+9B5C.0	fCNS11643	4-4229
U+9B5C.0	fGB	5-7636
U+9B5C.0	fR/S	  195.2
U+9B5C.0	fTotalStrokes	 13
U+9B5C.0	fCangjie	NFO
U+9B5C.0	fFourCorner	28300
U+9B5C.0	fHakka	 ngin2
U+9B5C.0	fCantonese	jan4
U+9B5C.0	fMandarin	ren2
U+9B5C.0	fEnglish	 mermaid manatee
U+9B5D.0	fUTF8	魝
U+9B5D.0	fCNS11643	3-4563
U+9B5D.0	fGB	3-773A
U+9B5D.0	fR/S	  195.2
U+9B5D.0	fTotalStrokes	 13
U+9B5D.0	fCangjie	NFLN
U+9B5D.0	fFourCorner	22300
U+9B5D.0	fHakka	 giet7; gi5; get7
U+9B5D.0	fCantonese	git3
U+9B5D.0	fMandarin	ji4; jie1
U+9B5D.0	fEnglish	[1] split up fish for drying [2] dissect
U+9B5E.0	fUTF8	魞
U+9B5E.0	fGB	E-4669
U+9B5E.0	fR/S	 195.2
U+9B5E.0	fTotalStrokes	 13
U+9B5E.0	fCangjie	XNFO
U+9B5E.0	fCantonese	jap6
U+9B5E.0	fMandarin	ba1
U+9B5E.0	fEnglish	 fish trap
U+9B5F.0	fUTF8	魟
U+9B5F.0	fCNS11643	2-4F49
U+9B5F.0	fBig5	E4E1
U+9B5F.0	fGB	3-773C
U+9B5F.0	fR/S	 195.3
U+9B5F.0	fTotalStrokes	 14
U+9B5F.0	fCangjie	NFM
U+9B5F.0	fFourCorner	21310
U+9B5F.0	fHakka	  gung1
U+9B5F.0	fCantonese	gung1; hung4; hung1
U+9B5F.0	fMandarin	gong1; hong2; hong1; jiang1
U+9B5F.0	fEnglish	rays
U+9B60.0	fUTF8	魠
U+9B60.0	fCNS11643	2-4F47
U+9B60.0	fBig5	E4DF
U+9B60.0	fGB	3-773E
U+9B60.0	fR/S	 195.3
U+9B60.0	fTotalStrokes	 14
U+9B60.0	fCangjie	NFHP
U+9B60.0	fFourCorner	22314
U+9B60.0	fCantonese	tok3
U+9B60.0	fMandarin	tuo1; tuo2
U+9B61.0	fUTF8	魡
U+9B61.0	fCNS11643	2-4F48
U+9B61.0	fBig5	E4E0
U+9B61.0	fGB	5-763B
U+9B61.0	fR/S	 195.3
U+9B61.0	fTotalStrokes	 14
U+9B61.0	fCangjie	NFPI
U+9B61.0	fFourCorner	27320
U+9B61.0	fCantonese	diu3; dik1; dik6
U+9B61.0	fMandarin	diao4; di4
U+9B62.0	fUTF8	魢
U+9B62.0	fCNS11643	4-486B
U+9B62.0	fGB	3-7740
U+9B62.0	fR/S	  195.3
U+9B62.0	fTotalStrokes	 14
U+9B62.0	fCangjie	NFSU
U+9B62.0	fFourCorner	27310
U+9B62.0	fCantonese	gei2
U+9B62.0	fMandarin	ji3
U+9B62.0	fEnglish	fish name
U+9B63.0	fUTF8	魣
U+9B63.0	fCNS11643	3-4F7E
U+9B63.0	fGB	3-7751
U+9B63.0	fR/S	 195.4
U+9B63.0	fTotalStrokes	 15
U+9B63.0	fCangjie	NFNIN
U+9B63.0	fFourCorner	27320
U+9B63.0	fCantonese	jyu4
U+9B63.0	fMandarin	xu4; yu2
U+9B64.0	fUTF8	魤
U+9B64.0	fCNS11643	2-562D
U+9B64.0	fBig5	E8E2
U+9B64.0	fGB	5-7645
U+9B64.0	fR/S	 195.4
U+9B64.0	fTotalStrokes	 15
U+9B64.0	fCangjie	NFOP
U+9B64.0	fFourCorner	21310
U+9B64.0	fCantonese	ngo4; faa3
U+9B64.0	fMandarin	e2
U+9B65.0	fUTF8	魥
U+9B65.0	fCNS11643	4-4F30
U+9B65.0	fGB	5-763C
U+9B65.0	fR/S	  195.4
U+9B65.0	fTotalStrokes	 15
U+9B65.0	fCangjie	NFNHE
U+9B65.0	fFourCorner	27340
U+9B65.0	fCantonese	ngap6; ap6 (also)
U+9B65.0	fMandarin	e4; qie4
U+9B66.0	fUTF8	魦
U+9B66.0	fCNS11643	2-5628
U+9B66.0	fBig5	E8DD
U+9B66.0	fGB	5-7642
U+9B66.0	fR/S	 195.4
U+9B66.0	fTotalStrokes	 15
U+9B66.0	fCangjie	NFFH
U+9B66.0	fFourCorner	29320
U+9B66.0	fHakka	 sa1
U+9B66.0	fCantonese	saa1
U+9B66.0	fMandarin	sha1; shuo1; shao4
U+9B66.0	fEnglish	shark family; including some rays and skates
U+9B67.0	fUTF8	魧
U+9B67.0	fCNS11643	2-5625
U+9B67.0	fBig5	E8DA
U+9B67.0	fGB	3-774E
U+9B67.0	fR/S	 195.4
U+9B67.0	fTotalStrokes	 15
U+9B67.0	fCangjie	NFYHN
U+9B67.0	fFourCorner	20317
U+9B67.0	fCantonese	hong4; gong1
U+9B67.0	fMandarin	hang2; gang1
U+9B68.0	fUTF8	魨
U+9B68.0	fCNS11643	2-562C
U+9B68.0	fBig5	E8E1
U+9B68.0	fGB	3-7745
U+9B68.0	fR/S	 195.4
U+9B68.0	fTotalStrokes	 15
U+9B68.0	fCangjie	NFPU
U+9B68.0	fFourCorner	25317
U+9B68.0	fHakka	  tun2
U+9B68.0	fCantonese	tyun4
U+9B68.0	fMandarin	tun2
U+9B68.0	fEnglish	blowfish; globefish; spheroides vermicular
U+9B69.0	fUTF8	魩
U+9B69.0	fCNS11643	3-4F7C
U+9B69.0	fGB	E-466A
U+9B69.0	fR/S	 195.4
U+9B69.0	fTotalStrokes	 15
U+9B69.0	fCangjie	NFPHH
U+9B69.0	fFourCorner	27320
U+9B69.0	fCantonese	mut6
U+9B69.0	fMandarin	mo4
U+9B6A.0	fUTF8	魪
U+9B6A.0	fCNS11643	4-4F35
U+9B6A.0	fGB	3-774A
U+9B6A.0	fR/S	  195.4
U+9B6A.0	fTotalStrokes	 15
U+9B6A.0	fCangjie	NFOLL
U+9B6A.0	fFourCorner	28320
U+9B6A.0	fHakka	 gai5; kie5; giai5
U+9B6A.0	fCantonese	gaai3
U+9B6A.0	fMandarin	jie4
U+9B6A.0	fEnglish	[1] sole [2] plaice [3] flounder
U+9B6B.0	fUTF8	魫
U+9B6B.0	fCNS11643	3-4F7B
U+9B6B.0	fGB	3-774F
U+9B6B.0	fR/S	  195.4
U+9B6B.0	fTotalStrokes	 15
U+9B6B.0	fCangjie	NFLBU
U+9B6B.0	fFourCorner	24310
U+9B6B.0	fHakka	 chim2; shim3; siim3; cihm2
U+9B6B.0	fCantonese	sam2; cam4
U+9B6B.0	fMandarin	shen3
U+9B6B.0	fEnglish	the young of fish
U+9B6C.0	fUTF8	魬
U+9B6C.0	fCNS11643	2-562E
U+9B6C.0	fBig5	E8E3
U+9B6C.0	fGB	3-7749
U+9B6C.0	fR/S	 195.4
U+9B6C.0	fTotalStrokes	 15
U+9B6C.0	fCangjie	NFHE
U+9B6C.0	fFourCorner	21347
U+9B6C.0	fCantonese	baan2
U+9B6C.0	fMandarin	fan3; ban3
U+9B6C.0	fEnglish	sole or flounder
U+9B6D.0	fUTF8	魭
U+9B6D.0	fCNS11643	4-4F37
U+9B6D.0	fGB	5-763E
U+9B6D.0	fR/S	  195.4
U+9B6D.0	fTotalStrokes	 15
U+9B6D.0	fCangjie	NFMMU
U+9B6D.0	fFourCorner	21310
U+9B6D.0	fCantonese	jyun4
U+9B6D.0	fMandarin	yuan2; wan3; wan2
U+9B6E.0	fUTF8	魮
U+9B6E.0	fCNS11643	4-4F2C
U+9B6E.0	fGB	3-7746
U+9B6E.0	fR/S	  195.4
U+9B6E.0	fTotalStrokes	 15
U+9B6E.0	fCangjie	NFPP
U+9B6E.0	fFourCorner	21310
U+9B6E.0	fCantonese	pei4
U+9B6E.0	fMandarin	bi2; pi2
U+9B6F.0	fUTF8	魯
U+9B6F.0	fCNS11643	1-7027
U+9B6F.0	fBig5	BE7C
U+9B6F.0	fGB	1-4233
U+9B6F.0	fR/S	 195.4
U+9B6F.0	fTotalStrokes	 15
U+9B6F.0	fCangjie	NWFA
U+9B6F.0	fFourCorner	27603
U+9B6F.0	fHakka	  lu1; lu3
U+9B6F.0	fCantonese	lou5
U+9B6F.0	fMandarin	lu3; l3
U+9B6F.0	fEnglish	[1] foolish; stupid; dull [2] vulgar [3] an ancient kingdom in what is today's Shantung [4] another name for Shantung [5] a Chinese family name
U+9B70.0	fUTF8	魰
U+9B70.0	fCNS11643	2-562B
U+9B70.0	fBig5	E8E0
U+9B70.0	fGB	E-466B
U+9B70.0	fR/S	 195.4
U+9B70.0	fTotalStrokes	 15
U+9B70.0	fCangjie	NFYK
U+9B70.0	fFourCorner	20341
U+9B70.0	fHakka	 wun2; wun3; vun2
U+9B70.0	fCantonese	man4
U+9B70.0	fMandarin	wen2
U+9B70.0	fEnglish	the flying-fish
U+9B71.0	fUTF8	魱
U+9B71.0	fCNS11643	2-5627
U+9B71.0	fBig5	E8DC
U+9B71.0	fGB	3-7747
U+9B71.0	fR/S	 195.4
U+9B71.0	fTotalStrokes	 15
U+9B71.0	fCangjie	NFMVM
U+9B71.0	fFourCorner	21317
U+9B71.0	fCantonese	wu4; wu6
U+9B71.0	fMandarin	hu2; hu4
U+9B72.0	fUTF8	魲
U+9B72.0	fCNS11643	3-4F7D
U+9B72.0	fGB	E-466C
U+9B72.0	fR/S	  195.4
U+9B72.0	fTotalStrokes	 15
U+9B72.0	fCangjie	NFHS
U+9B72.0	fFourCorner	23377
U+9B72.0	fCantonese	lou4
U+9B72.0	fMandarin	lu2
U+9B72.0	fEnglish	fish name
U+9B73.0	fUTF8	魳
U+9B73.0	fCNS11643	4-4F2A
U+9B73.0	fGB	5-763F
U+9B73.0	fR/S	  195.4
U+9B73.0	fTotalStrokes	 15
U+9B73.0	fCangjie	NFMLB
U+9B73.0	fFourCorner	21320
U+9B73.0	fCantonese	zaap3
U+9B73.0	fMandarin	za2; shi1
U+9B73.0	fEnglish	a barracuda
U+9B74.0	fUTF8	魴
U+9B74.0	fCNS11643	2-5626
U+9B74.0	fBig5	E8DB
U+9B74.0	fGB	1-7650
U+9B74.0	fR/S	 195.4
U+9B74.0	fTotalStrokes	 15
U+9B74.0	fCangjie	NFYHS
U+9B74.0	fFourCorner	20327
U+9B74.0	fHakka	  fong2
U+9B74.0	fCantonese	fong4
U+9B74.0	fMandarin	fang2
U+9B74.0	fEnglish	the freshwater bream
U+9B75.0	fUTF8	魵
U+9B75.0	fCNS11643	2-562A
U+9B75.0	fBig5	E8DF
U+9B75.0	fGB	3-774D
U+9B75.0	fR/S	 195.4
U+9B75.0	fTotalStrokes	 15
U+9B75.0	fCangjie	NFCSH
U+9B75.0	fFourCorner	28327
U+9B75.0	fCantonese	fan4; fan5 (also)
U+9B75.0	fMandarin	fen2; fen4
U+9B75.0	fEnglish	shrimp
U+9B76.0	fUTF8	魶
U+9B76.0	fCNS11643	2-5629
U+9B76.0	fBig5	E8DE
U+9B76.0	fGB	5-7643
U+9B76.0	fR/S	 195.4
U+9B76.0	fTotalStrokes	 15
U+9B76.0	fCangjie	NFOB
U+9B76.0	fFourCorner	24327
U+9B76.0	fHakka	 nap8
U+9B76.0	fCantonese	naap6; laap6 (also)
U+9B76.0	fMandarin	na4
U+9B76.0	fEnglish	[1] a hairy marine animal [2] be able to climb trees
U+9B77.0	fUTF8	魷
U+9B77.0	fCNS11643	1-7026
U+9B77.0	fBig5	BE7B
U+9B77.0	fGB	1-764F
U+9B77.0	fR/S	 195.4
U+9B77.0	fTotalStrokes	 15
U+9B77.0	fCangjie	NFIKU
U+9B77.0	fFourCorner	23314
U+9B77.0	fHakka	 ju2; riu2; jiu2
U+9B77.0	fCantonese	jau4; jau2 (also)
U+9B77.0	fMandarin	you2
U+9B77.0	fEnglish	the cuttlefish
U+9B78.0	fUTF8	魸
U+9B78.0	fGB	E-466D
U+9B78.0	fR/S	 195.4
U+9B78.0	fTotalStrokes	 15
U+9B78.0	fCangjie	XNFLL
U+9B78.0	fCantonese	pin3
U+9B79.0	fUTF8	魹
U+9B79.0	fCNS11643	F-533E
U+9B79.0	fGB	E-466E
U+9B79.0	fR/S	 195.4
U+9B79.0	fTotalStrokes	 15
U+9B79.0	fCangjie	NFHQU
U+9B79.0	fCantonese	mou4
U+9B7A.0	fUTF8	魺
U+9B7A.0	fCNS11643	2-5C2B
U+9B7A.0	fBig5	EC7D
U+9B7A.0	fGB	3-7756
U+9B7A.0	fR/S	 195.5
U+9B7A.0	fTotalStrokes	 16
U+9B7A.0	fCangjie	NFMNR
U+9B7A.0	fFourCorner	21320
U+9B7A.0	fHakka	 ho1;  ko3
U+9B7A.0	fCantonese	ho4; go2; ho1
U+9B7A.0	fMandarin	he2; he3
U+9B7B.0	fUTF8	魻
U+9B7B.0	fCNS11643	2-5C26
U+9B7B.0	fBig5	EC78
U+9B7B.0	fGB	3-7758
U+9B7B.0	fR/S	 195.5
U+9B7B.0	fTotalStrokes	 16
U+9B7B.0	fCangjie	NFWL
U+9B7B.0	fFourCorner	26350
U+9B7B.0	fCantonese	haap6; haap3
U+9B7B.0	fMandarin	xia2; qia4
U+9B7C.0	fUTF8	魼
U+9B7C.0	fCNS11643	2-5C24
U+9B7C.0	fBig5	EC76
U+9B7C.0	fGB	3-7754
U+9B7C.0	fR/S	 195.5
U+9B7C.0	fTotalStrokes	 16
U+9B7C.0	fCangjie	NFGI
U+9B7C.0	fFourCorner	24331
U+9B7C.0	fHakka	 ki1
U+9B7C.0	fCantonese	keoi1; heoi1
U+9B7C.0	fMandarin	qu1; ta4; xie4
U+9B7C.0	fEnglish	[1] flounder [2] sole [3] the walrus
U+9B7D.0	fUTF8	魽
U+9B7D.0	fCNS11643	2-5C2D
U+9B7D.0	fBig5	ECA1
U+9B7D.0	fGB	5-764A
U+9B7D.0	fR/S	 195.5
U+9B7D.0	fTotalStrokes	 16
U+9B7D.0	fCangjie	NFTM
U+9B7D.0	fFourCorner	24370
U+9B7D.0	fHakka	 ham1; tiam2
U+9B7D.0	fCantonese	haam4; ham1
U+9B7D.0	fMandarin	han1
U+9B7D.0	fEnglish	bivalve shells (with scalloped surfaces)
U+9B7E.0	fUTF8	魾
U+9B7E.0	fCNS11643	2-5C25
U+9B7E.0	fBig5	EC77
U+9B7E.0	fGB	3-7757
U+9B7E.0	fR/S	 195.5
U+9B7E.0	fTotalStrokes	 16
U+9B7E.0	fCangjie	NFMFM
U+9B7E.0	fFourCorner	21319
U+9B7E.0	fCantonese	pei1; pei4
U+9B7E.0	fMandarin	pi2; pi1
U+9B7F.0	fUTF8	魿
U+9B7F.0	fCNS11643	4-5547
U+9B7F.0	fGB	3-775E
U+9B7F.0	fR/S	 195.5
U+9B7F.0	fTotalStrokes	 16
U+9B7F.0	fCangjie	NFOII
U+9B7F.0	fFourCorner	28330
U+9B7F.0	fHakka	 lang2
U+9B7F.0	fCantonese	ling4
U+9B7F.0	fMandarin	ling2
U+9B7F.0	fEnglish	a pond or river fish, with scales
U+9B80.0	fUTF8	鮀
U+9B80.0	fCNS11643	2-5C21
U+9B80.0	fBig5	EC73
U+9B80.0	fGB	3-7762
U+9B80.0	fR/S	 195.5
U+9B80.0	fTotalStrokes	 16
U+9B80.0	fCangjie	NFJP
U+9B80.0	fFourCorner	23311
U+9B80.0	fHakka	 to2
U+9B80.0	fCantonese	to4
U+9B80.0	fMandarin	tuo2
U+9B80.0	fEnglish	snakefish
U+9B81.0	fUTF8	鮁
U+9B81.0	fCNS11643	4-5543
U+9B81.0	fGB	1-7651
U+9B81.0	fR/S	  195.5
U+9B81.0	fTotalStrokes	 16
U+9B81.0	fCangjie	NFIKE
U+9B81.0	fFourCorner	23310
U+9B81.0	fCantonese	bat6; bat3 (also); but3
U+9B81.0	fMandarin	ba2; bo4; bo1; ba4; fei4
U+9B81.0	fEnglish	the bonito
U+9B82.0	fUTF8	鮂
U+9B82.0	fCNS11643	2-5C27
U+9B82.0	fBig5	EC79
U+9B82.0	fGB	3-775B
U+9B82.0	fR/S	 195.5
U+9B82.0	fTotalStrokes	 16
U+9B82.0	fCangjie	NFWO
U+9B82.0	fFourCorner	26300
U+9B82.0	fHakka	 ciu2;  siu2
U+9B82.0	fCantonese	cau4
U+9B82.0	fMandarin	qiu2; zou2
U+9B82.0	fEnglish	a thin long thin fish of the pike family
U+9B83.0	fUTF8	鮃
U+9B83.0	fCNS11643	3-5378
U+9B83.0	fGB	1-7652
U+9B83.0	fR/S	  195.5
U+9B83.0	fTotalStrokes	 16
U+9B83.0	fCangjie	NFMFJ
U+9B83.0	fFourCorner	21340
U+9B83.0	fCantonese	ping4
U+9B83.0	fMandarin	ping2
U+9B83.0	fEnglish	a flatfish, flounder
U+9B84.0	fUTF8	鮄
U+9B84.0	fCNS11643	3-537A
U+9B84.0	fGB	3-7764
U+9B84.0	fR/S	  195.5
U+9B84.0	fTotalStrokes	 16
U+9B84.0	fCangjie	NFLLN
U+9B84.0	fFourCorner	25320
U+9B84.0	fCantonese	fat1; fat6 (also)
U+9B84.0	fMandarin	fu2
U+9B85.0	fUTF8	鮅
U+9B85.0	fCNS11643	2-5C22
U+9B85.0	fBig5	EC74
U+9B85.0	fGB	3-7763
U+9B85.0	fR/S	 195.5
U+9B85.0	fTotalStrokes	 16
U+9B85.0	fCangjie	NFPH
U+9B85.0	fFourCorner	23300
U+9B85.0	fHakka	 bit7
U+9B85.0	fCantonese	bit1
U+9B85.0	fMandarin	bi4; zha3
U+9B85.0	fEnglish	a kind of fish; a bleak (Squaliobarbus curriculus)
U+9B86.0	fUTF8	鮆
U+9B86.0	fCNS11643	2-607E
U+9B86.0	fBig5	EF72
U+9B86.0	fGB	3-776F
U+9B86.0	fR/S	 195.5
U+9B86.0	fTotalStrokes	 16
U+9B86.0	fCangjie	YPNWF
U+9B86.0	fFourCorner	21336
U+9B86.0	fHakka	  ci3; zi3
U+9B86.0	fCantonese	zai1 (also); cai5; ci2 (also)
U+9B86.0	fMandarin	ji4; ci3; ji3; zi4
U+9B86.0	fEnglish	fish
U+9B87.0	fUTF8	鮇
U+9B87.0	fCNS11643	2-5C23
U+9B87.0	fBig5	EC75
U+9B87.0	fGB	3-7752
U+9B87.0	fR/S	 195.5
U+9B87.0	fTotalStrokes	 16
U+9B87.0	fCangjie	NFJD
U+9B87.0	fFourCorner	25390
U+9B87.0	fCantonese	mei6
U+9B87.0	fMandarin	wei4
U+9B88.0	fUTF8	鮈
U+9B88.0	fCNS11643	2-5C2E
U+9B88.0	fBig5	ECA2
U+9B88.0	fGB	3-7760
U+9B88.0	fR/S	 195.5
U+9B88.0	fTotalStrokes	 16
U+9B88.0	fCangjie	NFPR
U+9B88.0	fFourCorner	27320
U+9B88.0	fCantonese	keoi1; geoi1
U+9B88.0	fMandarin	ju1; qu1; gou3
U+9B89.0	fUTF8	鮉
U+9B89.0	fCNS11643	3-5423
U+9B89.0	fGB	E-466F
U+9B89.0	fR/S	 195.5
U+9B89.0	fTotalStrokes	 16
U+9B89.0	fCangjie	NFSHR
U+9B89.0	fFourCorner	27360
U+9B89.0	fCantonese	diu1
U+9B89.0	fMandarin	diao1
U+9B89.0	fEnglish	bream
U+9B8A.0	fUTF8	鮊
U+9B8A.0	fCNS11643	4-5545
U+9B8A.0	fGB	3-775D
U+9B8A.0	fR/S	  195.5
U+9B8A.0	fTotalStrokes	 16
U+9B8A.0	fCangjie	NFHA
U+9B8A.0	fFourCorner	26300
U+9B8A.0	fHakka	 pak8
U+9B8A.0	fCantonese	bat6; baak6
U+9B8A.0	fMandarin	bo2; ba4
U+9B8B.0	fUTF8	鮋
U+9B8B.0	fCNS11643	3-537C
U+9B8B.0	fGB	3-7759
U+9B8B.0	fR/S	 195.5
U+9B8B.0	fTotalStrokes	 16
U+9B8B.0	fCangjie	NFLW
U+9B8B.0	fFourCorner	25360
U+9B8B.0	fCantonese	jau4
U+9B8B.0	fMandarin	you2; chou2
U+9B8C.0	fUTF8	鮌
U+9B8C.0	fCNS11643	3-5421
U+9B8C.0	fGB	5-7650
U+9B8C.0	fR/S	  195.5
U+9B8C.0	fTotalStrokes	 16
U+9B8C.0	fCangjie	NFYVI
U+9B8C.0	fFourCorner	20330
U+9B8C.0	fCantonese	gwan2
U+9B8C.0	fMandarin	gun3
U+9B8D.0	fUTF8	鮍
U+9B8D.0	fCNS11643	5-5A4A
U+9B8D.0	fGB	3-7765
U+9B8D.0	fR/S	  195.5
U+9B8D.0	fTotalStrokes	 16
U+9B8D.0	fCangjie	NFDHE
U+9B8D.0	fFourCorner	24340
U+9B8D.0	fCantonese	pei4
U+9B8D.0	fMandarin	pi1; pi2
U+9B8E.0	fUTF8	鮎
U+9B8E.0	fCNS11643	3-5425
U+9B8E.0	fGB	1-7653
U+9B8E.0	fR/S	 195.5
U+9B8E.0	fTotalStrokes	 16
U+9B8E.0	fCangjie	NFYR
U+9B8E.0	fFourCorner	21360
U+9B8E.0	fHakka	 ngiam2
U+9B8E.0	fCantonese	nim4; lim4 (also)
U+9B8E.0	fMandarin	nian2
U+9B8E.0	fEnglish	Parasilaarus asotus, a kind of slippery long fish
U+9B8F.0	fUTF8	鮏
U+9B8F.0	fCNS11643	3-537D
U+9B8F.0	fGB	5-764F
U+9B8F.0	fR/S	 195.5
U+9B8F.0	fTotalStrokes	 16
U+9B8F.0	fCangjie	NFHQM
U+9B8F.0	fFourCorner	25310
U+9B8F.0	fHakka	 siang1
U+9B8F.0	fCantonese	sing1
U+9B8F.0	fMandarin	xing1; zeng1
U+9B90.0	fUTF8	鮐
U+9B90.0	fCNS11643	2-5C2A
U+9B90.0	fBig5	EC7C
U+9B90.0	fGB	1-7658
U+9B90.0	fR/S	 195.5
U+9B90.0	fTotalStrokes	 16
U+9B90.0	fCangjie	NFIR
U+9B90.0	fFourCorner	23360
U+9B90.0	fHakka	  toi2
U+9B90.0	fCantonese	toi4; toi1
U+9B90.0	fMandarin	tai2; yi2
U+9B90.0	fEnglish	blowfish; globefish; tetraodon
U+9B91.0	fUTF8	鮑
U+9B91.0	fCNS11643	1-7335
U+9B91.0	fBig5	C06A
U+9B91.0	fGB	1-312B
U+9B91.0	fR/S	 195.5
U+9B91.0	fTotalStrokes	 16
U+9B91.0	fCangjie	NFPRU
U+9B91.0	fFourCorner	27312
U+9B91.0	fHakka	 bau1;  pau5; bau5
U+9B91.0	fCantonese	baau1; baau6
U+9B91.0	fMandarin	bao4; bao1; pao1
U+9B91.0	fEnglish	[1] the abalone [2] slated fish [3] a Chinese family name
U+9B92.0	fUTF8	鮒
U+9B92.0	fCNS11643	2-5C29
U+9B92.0	fBig5	EC7B
U+9B92.0	fGB	1-7656
U+9B92.0	fR/S	 195.5
U+9B92.0	fTotalStrokes	 16
U+9B92.0	fCangjie	NFODI
U+9B92.0	fFourCorner	24300
U+9B92.0	fHakka	  fu5
U+9B92.0	fCantonese	fu6
U+9B92.0	fMandarin	fu4; fu1
U+9B92.0	fEnglish	 gold carp; crucian carp; Carassicus auratus
U+9B93.0	fUTF8	鮓
U+9B93.0	fCNS11643	2-5C28
U+9B93.0	fBig5	EC7A
U+9B93.0	fGB	3-775C
U+9B93.0	fR/S	 195.5
U+9B93.0	fTotalStrokes	 16
U+9B93.0	fCangjie	NFOS
U+9B93.0	fFourCorner	28311
U+9B93.0	fHakka	 za5
U+9B93.0	fCantonese	zaa2; zaa3
U+9B93.0	fMandarin	zha3; zha4
U+9B93.0	fEnglish	minced and salted fish; preserve
U+9B94.0	fUTF8	鮔
U+9B94.0	fCNS11643	3-5379
U+9B94.0	fGB	E-4670
U+9B94.0	fR/S	  195.5
U+9B94.0	fTotalStrokes	 16
U+9B94.0	fCangjie	NFSS
U+9B94.0	fFourCorner	21310
U+9B94.0	fCantonese	geoi6
U+9B94.0	fMandarin	ju4
U+9B95.0	fUTF8	鮕
U+9B95.0	fCNS11643	2-5C2C
U+9B95.0	fBig5	EC7E
U+9B95.0	fGB	E-4671
U+9B95.0	fR/S	 195.5
U+9B95.0	fTotalStrokes	 16
U+9B95.0	fCangjie	NFJR
U+9B95.0	fFourCorner	24360
U+9B95.0	fHakka	 gu1
U+9B95.0	fCantonese	gu1; gwu1 (also)
U+9B95.0	fMandarin	gu1; gu3; gu4
U+9B96.0	fUTF8	鮖
U+9B96.0	fCNS11643	F-5938
U+9B96.0	fGB	E-4672
U+9B96.0	fR/S	 195.5
U+9B96.0	fTotalStrokes	 16
U+9B96.0	fCangjie	NFMR
U+9B96.0	fCantonese	sek6
U+9B97.0	fUTF8	鮗
U+9B97.0	fGB	E-4673
U+9B97.0	fR/S	 195.5
U+9B97.0	fTotalStrokes	 16
U+9B97.0	fCangjie	NFHEY
U+9B97.0	fFourCorner	2733
U+9B97.0	fCantonese	dung1
U+9B98.0	fUTF8	鮘
U+9B98.0	fCNS11643	3-6673
U+9B98.0	fGB	E-4674
U+9B98.0	fR/S	 195.5
U+9B98.0	fTotalStrokes	 16
U+9B98.0	fCangjie	XNFOI
U+9B98.0	fFourCorner	2334
U+9B98.0	fCantonese	doi6
U+9B99.0	fUTF8	鮙
U+9B99.0	fCNS11643	3-5744
U+9B99.0	fGB	E-4675
U+9B99.0	fR/S	  195.6
U+9B99.0	fTotalStrokes	 17
U+9B99.0	fCangjie	NFSMM
U+9B99.0	fFourCorner	27320
U+9B99.0	fCantonese	taap3
U+9B99.0	fMandarin	ta4
U+9B9A.0	fUTF8	鮚
U+9B9A.0	fCNS11643	2-6076
U+9B9A.0	fBig5	EF6A
U+9B9A.0	fGB	1-765A
U+9B9A.0	fR/S	 195.6
U+9B9A.0	fTotalStrokes	 17
U+9B9A.0	fCangjie	NFGR
U+9B9A.0	fFourCorner	24361
U+9B9A.0	fCantonese	git3; git6
U+9B9A.0	fMandarin	jie2; ji2; qia4; qie4
U+9B9A.0	fEnglish	 clam oyster
U+9B9B.0	fUTF8	鮛
U+9B9B.0	fCNS11643	2-6079
U+9B9B.0	fBig5	EF6D
U+9B9B.0	fGB	3-776E
U+9B9B.0	fR/S	 195.6
U+9B9B.0	fTotalStrokes	 17
U+9B9B.0	fCangjie	NFYMF
U+9B9B.0	fFourCorner	21394
U+9B9B.0	fCantonese	suk1
U+9B9B.0	fMandarin	shu4; shua1; shu1
U+9B9C.0	fUTF8	鮜
U+9B9C.0	fCNS11643	4-5A49
U+9B9C.0	fGB	3-7778
U+9B9C.0	fR/S	 195.6
U+9B9C.0	fTotalStrokes	 17; 16
U+9B9C.0	fCangjie	NFHMR
U+9B9C.0	fFourCorner	22360
U+9B9C.0	fCantonese	hau6
U+9B9C.0	fMandarin	hou4
U+9B9D.0	fUTF8	鮝
U+9B9D.0	fCNS11643	3-5745
U+9B9D.0	fGB	1-765F
U+9B9D.0	fR/S	 195.6
U+9B9D.0	fTotalStrokes	 17
U+9B9D.0	fCangjie	FQNWF
U+9B9D.0	fFourCorner	90330
U+9B9D.0	fHakka	 siong3
U+9B9D.0	fCantonese	soeng2
U+9B9D.0	fMandarin	xiang3; zhen4
U+9B9D.0	fEnglish	dried fish
U+9B9E.0	fUTF8	鮞
U+9B9E.0	fCNS11643	2-6078
U+9B9E.0	fBig5	EF6C
U+9B9E.0	fGB	1-765C
U+9B9E.0	fR/S	 195.6
U+9B9E.0	fTotalStrokes	 17
U+9B9E.0	fCangjie	NFMBL
U+9B9E.0	fFourCorner	21327
U+9B9E.0	fHakka	 ji2; ri2; i2
U+9B9E.0	fCantonese	ji4
U+9B9E.0	fMandarin	er2; rou4
U+9B9E.0	fEnglish	roe
U+9B9F.0	fUTF8	鮟
U+9B9F.0	fCNS11643	3-5740
U+9B9F.0	fGB	3-7823
U+9B9F.0	fR/S	 195.6
U+9B9F.0	fTotalStrokes	 17
U+9B9F.0	fCangjie	NFJV
U+9B9F.0	fFourCorner	23340
U+9B9F.0	fCantonese	on1; ngon1 (also); jin2
U+9B9F.0	fMandarin	an4; yan3
U+9B9F.0	fEnglish	anglerfish
U+9BA0.0	fUTF8	鮠
U+9BA0.0	fCNS11643	2-6122
U+9BA0.0	fBig5	EF74
U+9BA0.0	fGB	3-777B
U+9BA0.0	fR/S	 195.6
U+9BA0.0	fTotalStrokes	 17
U+9BA0.0	fCangjie	NFNMU
U+9BA0.0	fFourCorner	27312
U+9BA0.0	fHakka	 ngui2
U+9BA0.0	fCantonese	wai4; ngai4
U+9BA0.0	fMandarin	wei2
U+9BA0.0	fEnglish	a kind of shad with a head like a sturgeon
U+9BA1.0	fUTF8	鮡
U+9BA1.0	fCNS11643	2-607B
U+9BA1.0	fBig5	EF6F
U+9BA1.0	fGB	3-777A
U+9BA1.0	fR/S	 195.6
U+9BA1.0	fTotalStrokes	 17
U+9BA1.0	fCangjie	NFLMO
U+9BA1.0	fFourCorner	22313
U+9BA1.0	fCantonese	siu6; ziu6
U+9BA1.0	fMandarin	tiao1; zhao3; zhao4; yao2; dao4
U+9BA2.0	fUTF8	鮢
U+9BA2.0	fCNS11643	2-6121
U+9BA2.0	fBig5	EF73
U+9BA2.0	fGB	E-4676
U+9BA2.0	fR/S	 195.6
U+9BA2.0	fTotalStrokes	 17
U+9BA2.0	fCangjie	NFHJD
U+9BA2.0	fFourCorner	25390
U+9BA2.0	fCantonese	zyu1
U+9BA2.0	fMandarin	zhu1
U+9BA3.0	fUTF8	鮣
U+9BA3.0	fCNS11643	3-5742
U+9BA3.0	fGB	3-775F
U+9BA3.0	fR/S	 195.5; 195.6
U+9BA3.0	fTotalStrokes	 17; 16
U+9BA3.0	fCangjie	NFHPL
U+9BA3.0	fFourCorner	27320
U+9BA3.0	fCantonese	jan3
U+9BA3.0	fMandarin	yin4
U+9BA4.0	fUTF8	鮤
U+9BA4.0	fCNS11643	2-607D
U+9BA4.0	fBig5	EF71
U+9BA4.0	fGB	3-776B
U+9BA4.0	fR/S	 195.6
U+9BA4.0	fTotalStrokes	 17
U+9BA4.0	fCangjie	MNNWF
U+9BA4.0	fFourCorner	12336
U+9BA4.0	fCantonese	lit6
U+9BA4.0	fMandarin	lie4; li4
U+9BA5.0	fUTF8	鮥
U+9BA5.0	fCNS11643	2-607C
U+9BA5.0	fBig5	EF70
U+9BA5.0	fGB	3-777D
U+9BA5.0	fR/S	 195.6
U+9BA5.0	fTotalStrokes	 17
U+9BA5.0	fCangjie	NFHER
U+9BA5.0	fFourCorner	27364
U+9BA5.0	fCantonese	lok6; gaak3; gok3
U+9BA5.0	fMandarin	luo4; hui4; ge4; ge2
U+9BA6.0	fUTF8	鮦
U+9BA6.0	fCNS11643	2-607A
U+9BA6.0	fBig5	EF6E
U+9BA6.0	fGB	3-7771
U+9BA6.0	fR/S	 195.6
U+9BA6.0	fTotalStrokes	 17
U+9BA6.0	fCangjie	NFBMR
U+9BA6.0	fFourCorner	27320
U+9BA6.0	fHakka	  tung2; tung3
U+9BA6.0	fCantonese	zau6; tung4; tung3; dung4
U+9BA6.0	fMandarin	tong2; zhou4; chong3
U+9BA6.0	fEnglish	snakefish
U+9BA7.0	fUTF8	鮧
U+9BA7.0	fCNS11643	4-5A41
U+9BA7.0	fGB	3-776D
U+9BA7.0	fR/S	  195.6
U+9BA7.0	fTotalStrokes	 17; 16
U+9BA7.0	fCangjie	NFKN
U+9BA7.0	fFourCorner	25330
U+9BA7.0	fHakka	 ti2
U+9BA7.0	fCantonese	ji4; tai4
U+9BA7.0	fMandarin	yi2; ti2
U+9BA7.0	fEnglish	the sheat fish
U+9BA8.0	fUTF8	鮨
U+9BA8.0	fCNS11643	2-6077
U+9BA8.0	fBig5	EF6B
U+9BA8.0	fGB	3-777C
U+9BA8.0	fR/S	 195.6
U+9BA8.0	fTotalStrokes	 17
U+9BA8.0	fCangjie	NFPA
U+9BA8.0	fFourCorner	21361
U+9BA8.0	fCantonese	ngai6; kei4; zi1; ai6 (also); zi2
U+9BA8.0	fMandarin	qi2; qi1; yi4; yi3; zhi1
U+9BA8.0	fEnglish	seasoned rice mixed with fish or vegetables
U+9BA9.0	fUTF8	鮩
U+9BA9.0	fCNS11643	4-5A3F
U+9BA9.0	fGB	3-7821
U+9BA9.0	fR/S	  195.6
U+9BA9.0	fTotalStrokes	 17; 16
U+9BA9.0	fCangjie	NFTT
U+9BA9.0	fFourCorner	28340
U+9BA9.0	fCantonese	bing6
U+9BA9.0	fMandarin	bing4
U+9BAA.0	fUTF8	鮪
U+9BAA.0	fCNS11643	1-762F
U+9BAA.0	fBig5	C243
U+9BAA.0	fGB	1-765B
U+9BAA.0	fR/S	 195.6
U+9BAA.0	fTotalStrokes	 17
U+9BAA.0	fCangjie	NFKB
U+9BAA.0	fFourCorner	24327
U+9BAA.0	fHakka	 wi3; wui3; riu3; ju3; vui3
U+9BAA.0	fCantonese	fui2; wai5
U+9BAA.0	fMandarin	wei3; you3
U+9BAA.0	fEnglish	 tuna
U+9BAB.0	fUTF8	鮫
U+9BAB.0	fCNS11643	1-762E
U+9BAB.0	fBig5	C242
U+9BAB.0	fGB	1-765E
U+9BAB.0	fR/S	 195.6
U+9BAB.0	fTotalStrokes	 17
U+9BAB.0	fCangjie	NFYCK
U+9BAB.0	fFourCorner	20348
U+9BAB.0	fHakka	  gau1
U+9BAB.0	fCantonese	gaau1
U+9BAB.0	fMandarin	jiao1; jiao3
U+9BAB.0	fEnglish	 shark
U+9BAC.0	fUTF8	鮬
U+9BAC.0	fCNS11643	4-5A44
U+9BAC.0	fGB	3-776A
U+9BAC.0	fR/S	  195.6
U+9BAC.0	fTotalStrokes	 17; 16
U+9BAC.0	fCangjie	NFKMS
U+9BAC.0	fFourCorner	24320
U+9BAC.0	fCantonese	fu1
U+9BAC.0	fMandarin	gu1; bu4; ku1; qu4; pi1; ku4
U+9BAD.0	fUTF8	鮭
U+9BAD.0	fCNS11643	1-7630
U+9BAD.0	fBig5	C244
U+9BAD.0	fGB	1-7659
U+9BAD.0	fR/S	 195.6
U+9BAD.0	fTotalStrokes	 17
U+9BAD.0	fCangjie	NFGG
U+9BAD.0	fFourCorner	24314
U+9BAD.0	fHakka	 gui1; wa1; kwui1; va1
U+9BAD.0	fCantonese	gwai1; haai4; waa1
U+9BAD.0	fMandarin	gui1; xie2; wa1; hua3; kui2; kui1; hua4
U+9BAD.0	fEnglish	 salmon
U+9BAE.0	fUTF8	鮮
U+9BAE.0	fCNS11643	1-762D
U+9BAE.0	fBig5	C241
U+9BAE.0	fGB	1-4F4A
U+9BAE.0	fR/S	 195.6
U+9BAE.0	fTotalStrokes	 17
U+9BAE.0	fCangjie	NFTQ
U+9BAE.0	fFourCorner	28351
U+9BAE.0	fHakka	 sien1;  sien3;  sen1; sian1; sian3
U+9BAE.0	fCantonese	sin1; sin2
U+9BAE.0	fMandarin	xian1; xian4
U+9BAE.0	fEnglish	[1] fresh; new [2] delicious; tasty [3] bright; attractive [4] fluid [5] clear; lucid
U+9BAE.1	fUTF8	鮮
U+9BAE.1	fCNS11643	1-762D
U+9BAE.1	fBig5	C241
U+9BAE.1	fGB	1-4F4A
U+9BAE.1	fR/S	 195.6
U+9BAE.1	fTotalStrokes	 17
U+9BAE.1	fCangjie	NFTQ
U+9BAE.1	fFourCorner	28351
U+9BAE.1	fMandarin	xian3
U+9BAE.1	fEnglish	rare; few; seldom
U+9BAE.2	fUTF8	鮮
U+9BAE.2	fCNS11643	1-762D
U+9BAE.2	fBig5	C241
U+9BAE.2	fGB	1-4F4A
U+9BAE.2	fR/S	 195.6
U+9BAE.2	fTotalStrokes	 17
U+9BAE.2	fCangjie	NFTQ
U+9BAE.2	fFourCorner	28351
U+9BAE.2	fMandarin	xian5
U+9BAF.0	fUTF8	鮯
U+9BAF.0	fCNS11643	2-6123
U+9BAF.0	fBig5	EF75
U+9BAF.0	fGB	E-4677
U+9BAF.0	fR/S	 195.6
U+9BAF.0	fTotalStrokes	 17
U+9BAF.0	fCangjie	NFOMR
U+9BAF.0	fFourCorner	28361
U+9BAF.0	fCantonese	gap3
U+9BAF.0	fMandarin	ge2
U+9BB0.0	fUTF8	鮰
U+9BB0.0	fCNS11643	4-5A45
U+9BB0.0	fGB	3-7773
U+9BB0.0	fR/S	 195.6
U+9BB0.0	fTotalStrokes	 17; 16
U+9BB0.0	fCangjie	NFWR
U+9BB0.0	fFourCorner	26300
U+9BB0.0	fHakka	 fui2; fi2
U+9BB0.0	fCantonese	wui4
U+9BB0.0	fMandarin	hui2
U+9BB0.0	fEnglish	a small kind of sturgeon; found in the Yangtze
U+9BB1.0	fUTF8	鮱
U+9BB1.0	fCNS11643	F-5D76
U+9BB1.0	fGB	E-4678
U+9BB1.0	fR/S	 195.6
U+9BB1.0	fTotalStrokes	 17
U+9BB1.0	fCangjie	NFJKP
U+9BB1.0	fCantonese	lou5
U+9BB2.0	fUTF8	鮲
U+9BB2.0	fGB	E-4679
U+9BB2.0	fR/S	 195.6
U+9BB2.0	fTotalStrokes	 17
U+9BB2.0	fCangjie	NFOIK
U+9BB2.0	fCantonese	fuk6
U+9BB3.0	fUTF8	鮳
U+9BB3.0	fCNS11643	4-5A43
U+9BB3.0	fGB	3-7768
U+9BB3.0	fR/S	  195.6
U+9BB3.0	fTotalStrokes	 17; 16
U+9BB3.0	fCangjie	NFJKS
U+9BB3.0	fFourCorner	24320
U+9BB3.0	fCantonese	kaau3
U+9BB3.0	fMandarin	kao3; kao4
U+9BB4.0	fUTF8	鮴
U+9BB4.0	fGB	E-467A
U+9BB4.0	fR/S	 195.6
U+9BB4.0	fTotalStrokes	 17
U+9BB4.0	fCangjie	NFOD
U+9BB4.0	fCantonese	jau1
U+9BB5.0	fUTF8	鮵
U+9BB5.0	fCNS11643	2-6472
U+9BB5.0	fBig5	F1C8
U+9BB5.0	fGB	E-467B
U+9BB5.0	fR/S	 195.7
U+9BB5.0	fTotalStrokes	 18
U+9BB5.0	fCangjie	NFCRU
U+9BB5.0	fFourCorner	28316
U+9BB5.0	fCantonese	dyut6
U+9BB5.0	fMandarin	duo2
U+9BB6.0	fUTF8	鮶
U+9BB6.0	fCNS11643	2-6475
U+9BB6.0	fBig5	F1CB
U+9BB6.0	fGB	3-782E
U+9BB6.0	fR/S	 195.7
U+9BB6.0	fTotalStrokes	 18
U+9BB6.0	fCangjie	NFSKR
U+9BB6.0	fFourCorner	27367
U+9BB6.0	fCantonese	gwan1
U+9BB6.0	fMandarin	jun1
U+9BB7.0	fUTF8	鮷
U+9BB7.0	fCNS11643	4-5F26
U+9BB7.0	fGB	5-7660
U+9BB7.0	fR/S	  195.7
U+9BB7.0	fTotalStrokes	 18
U+9BB7.0	fCangjie	NFCNH
U+9BB7.0	fFourCorner	28320
U+9BB7.0	fCantonese	tai4
U+9BB7.0	fMandarin	ti2; di4
U+9BB8.0	fUTF8	鮸
U+9BB8.0	fCNS11643	2-6473
U+9BB8.0	fBig5	F1C9
U+9BB8.0	fGB	3-782D
U+9BB8.0	fR/S	 195.7
U+9BB8.0	fTotalStrokes	 18
U+9BB8.0	fCangjie	NFNAU
U+9BB8.0	fFourCorner	27316
U+9BB8.0	fHakka	 mien3; men3
U+9BB8.0	fCantonese	min5
U+9BB8.0	fMandarin	man3; mian3; wan3
U+9BB8.0	fEnglish	a yellow fish found on the coast of Korea
U+9BB9.0	fUTF8	鮹
U+9BB9.0	fCNS11643	2-6477
U+9BB9.0	fBig5	F1CD
U+9BB9.0	fGB	3-7828
U+9BB9.0	fR/S	 195.7
U+9BB9.0	fTotalStrokes	 18
U+9BB9.0	fCangjie	NFFB
U+9BB9.0	fFourCorner	29327
U+9BB9.0	fHakka	 sau1;  siau1; seu1
U+9BB9.0	fCantonese	saau1; siu1
U+9BB9.0	fMandarin	xiao1; shao1
U+9BB9.0	fEnglish	a whip-like fish, with a forked tail
U+9BBA.0	fUTF8	鮺
U+9BBA.0	fCNS11643	5-6679
U+9BBA.0	fGB	5-664C
U+9BBA.0	fR/S	 195.6; 195.7
U+9BBA.0	fTotalStrokes	 17; 18
U+9BBA.0	fCangjie	TQNWF
U+9BBA.0	fFourCorner	80230
U+9BBA.0	fCantonese	zaa2
U+9BBA.0	fMandarin	za3
U+9BBB.0	fUTF8	鮻
U+9BBB.0	fCNS11643	3-5974
U+9BBB.0	fGB	E-467C
U+9BBB.0	fR/S	  195.7
U+9BBB.0	fTotalStrokes	 18
U+9BBB.0	fCangjie	NFICE
U+9BBB.0	fFourCorner	23340
U+9BBB.0	fCantonese	so1; leng4
U+9BBB.0	fMandarin	sha1; suo1
U+9BBB.0	fEnglish	dace; Canton dace
U+9BBC.0	fUTF8	鮼
U+9BBC.0	fCNS11643	4-5F22
U+9BBC.0	fGB	E-467D
U+9BBC.0	fR/S	  195.7
U+9BBC.0	fTotalStrokes	 18
U+9BBC.0	fCangjie	NFSME
U+9BBC.0	fFourCorner	27340
U+9BBC.0	fCantonese	cam1
U+9BBC.0	fMandarin	qin1
U+9BBD.0	fUTF8	鮽
U+9BBD.0	fCNS11643	2-6478
U+9BBD.0	fBig5	F1CE
U+9BBD.0	fGB	E-467E
U+9BBD.0	fR/S	 195.7
U+9BBD.0	fTotalStrokes	 18
U+9BBD.0	fCangjie	NFOMD
U+9BBD.0	fFourCorner	28394
U+9BBD.0	fCantonese	jyu4
U+9BBD.0	fMandarin	yu2
U+9BBE.0	fUTF8	鮾
U+9BBE.0	fCNS11643	4-5F2A
U+9BBE.0	fGB	E-4721
U+9BBE.0	fR/S	  195.7
U+9BBE.0	fTotalStrokes	 18
U+9BBE.0	fCangjie	NFBV
U+9BBE.0	fFourCorner	22340
U+9BBE.0	fHakka	  nui3
U+9BBE.0	fCantonese	neoi5; leoi5 (also)
U+9BBE.0	fMandarin	nei3
U+9BBE.0	fEnglish	putrid
U+9BBF.0	fUTF8	鮿
U+9BBF.0	fCNS11643	2-6470
U+9BBF.0	fBig5	F1C6
U+9BBF.0	fGB	3-7824
U+9BBF.0	fR/S	 195.7
U+9BBF.0	fTotalStrokes	 18
U+9BBF.0	fCangjie	NFSJU
U+9BBF.0	fFourCorner	22310
U+9BBF.0	fCantonese	zip3
U+9BBF.0	fMandarin	zhe2
U+9BBF.0	fEnglish	dried fish
U+9BC0.0	fUTF8	鯀
U+9BC0.0	fCNS11643	1-7824
U+9BC0.0	fBig5	C358
U+9BC0.0	fGB	1-7667
U+9BC0.0	fR/S	 195.7
U+9BC0.0	fTotalStrokes	 18
U+9BC0.0	fCangjie	NFHVF
U+9BC0.0	fFourCorner	22393
U+9BC0.0	fHakka	 gun3; kun3; gwun3
U+9BC0.0	fCantonese	gwan2; kwan2
U+9BC0.0	fMandarin	gun3
U+9BC0.0	fEnglish	[1] fish name [2] the father of emperor Yuu
U+9BC1.0	fUTF8	鯁
U+9BC1.0	fCNS11643	2-6471
U+9BC1.0	fBig5	F1C7
U+9BC1.0	fGB	1-7661
U+9BC1.0	fR/S	 195.7
U+9BC1.0	fTotalStrokes	 18
U+9BC1.0	fCangjie	NFMLK
U+9BC1.0	fFourCorner	21346
U+9BC1.0	fHakka	  gang3; gien3; gen5
U+9BC1.0	fCantonese	gang2; kang2 (also)
U+9BC1.0	fMandarin	geng3; jing1
U+9BC1.0	fEnglish	[1] a fish bone stuck in the throat [2] honest; upright; straightforward
U+9BC2.0	fUTF8	鯂
U+9BC2.0	fCNS11643	3-5973
U+9BC2.0	fGB	E-4722
U+9BC2.0	fR/S	 195.7; 195.6
U+9BC2.0	fTotalStrokes	 18; 17
U+9BC2.0	fCangjie	NFMCW
U+9BC2.0	fFourCorner	21360
U+9BC2.0	fCantonese	sou1
U+9BC2.0	fMandarin	su1
U+9BC3.0	fUTF8	鯃
U+9BC3.0	fCNS11643	2-646F
U+9BC3.0	fBig5	F1C5
U+9BC3.0	fGB	E-4723
U+9BC3.0	fR/S	 195.7
U+9BC3.0	fTotalStrokes	 18
U+9BC3.0	fCangjie	NFMMR
U+9BC3.0	fFourCorner	21361
U+9BC3.0	fCantonese	ng4
U+9BC3.0	fMandarin	wu2; wu1; yu1; yu3
U+9BC4.0	fUTF8	鯄
U+9BC4.0	fCNS11643	2-6476
U+9BC4.0	fBig5	F1CC
U+9BC4.0	fGB	5-765A
U+9BC4.0	fR/S	 195.7
U+9BC4.0	fTotalStrokes	 18
U+9BC4.0	fCangjie	NFIJE
U+9BC4.0	fFourCorner	23332
U+9BC4.0	fCantonese	kau4
U+9BC4.0	fMandarin	qiu2
U+9BC5.0	fUTF8	鯅
U+9BC5.0	fCNS11643	3-5B74
U+9BC5.0	fGB	3-7777
U+9BC5.0	fR/S	 195.7; 195.8
U+9BC5.0	fTotalStrokes	 19; 18
U+9BC5.0	fCangjie	NFNKM
U+9BC5.0	fFourCorner	22340
U+9BC5.0	fCantonese	saan1
U+9BC5.0	fMandarin	shan1; ting2; chan2; ting3; shan3
U+9BC6.0	fUTF8	鯆
U+9BC6.0	fCNS11643	2-646E
U+9BC6.0	fBig5	F1C4
U+9BC6.0	fGB	3-7826
U+9BC6.0	fR/S	 195.7
U+9BC6.0	fTotalStrokes	 18
U+9BC6.0	fCangjie	NFIJB
U+9BC6.0	fFourCorner	23327
U+9BC6.0	fHakka	  pu2; pu1
U+9BC6.0	fCantonese	pou1; bou1
U+9BC6.0	fMandarin	fu3; bu1; pu2; pu1
U+9BC6.0	fEnglish	the skate or ray
U+9BC7.0	fUTF8	鯇
U+9BC7.0	fCNS11643	2-646D
U+9BC7.0	fBig5	F1C3
U+9BC7.0	fGB	1-7669
U+9BC7.0	fR/S	 195.7
U+9BC7.0	fTotalStrokes	 18
U+9BC7.0	fCangjie	NFJMU
U+9BC7.0	fFourCorner	23311
U+9BC7.0	fHakka	 won3; fan6; fan5; fon5; van3; van1
U+9BC7.0	fCantonese	waan5
U+9BC7.0	fMandarin	wan3; huan3; hun3; huan4; huang4
U+9BC7.0	fEnglish	a grass carp
U+9BC8.0	fUTF8	鯈
U+9BC8.0	fCNS11643	1-7823
U+9BC8.0	fBig5	C357
U+9BC8.0	fGB	3-3233
U+9BC8.0	fR/S	 195.7
U+9BC8.0	fTotalStrokes	 18
U+9BC8.0	fCangjie	OLOF
U+9BC8.0	fFourCorner	27236
U+9BC8.0	fHakka	 tiau2; ju2; riu2
U+9BC8.0	fCantonese	jau4; tiu4
U+9BC8.0	fMandarin	you2; tiao2; chou2
U+9BC8.0	fEnglish	fish
U+9BC9.0	fUTF8	鯉
U+9BC9.0	fCNS11643	1-7821
U+9BC9.0	fBig5	C355
U+9BC9.0	fGB	1-4070
U+9BC9.0	fR/S	 195.7
U+9BC9.0	fTotalStrokes	 18
U+9BC9.0	fCangjie	NFWG
U+9BC9.0	fFourCorner	26314
U+9BC9.0	fHakka	  li1; li3
U+9BC9.0	fCantonese	lei5
U+9BC9.0	fMandarin	li3
U+9BC9.0	fEnglish	[1] carp [2] letters; epistles
U+9BCA.0	fUTF8	鯊
U+9BCA.0	fCNS11643	1-777E
U+9BCA.0	fBig5	C354
U+9BCA.0	fGB	1-7668
U+9BCA.0	fR/S	 195.7
U+9BCA.0	fTotalStrokes	 18
U+9BCA.0	fCangjie	EHNWF
U+9BCA.0	fFourCorner	39336
U+9BCA.0	fHakka	 sa1
U+9BCA.0	fCantonese	saa1
U+9BCA.0	fMandarin	sha1; sa4
U+9BCA.0	fEnglish	the shark
U+9BCB.0	fUTF8	鯋
U+9BCB.0	fCNS11643	3-5976
U+9BCB.0	fGB	E-4724
U+9BCB.0	fR/S	 195.7
U+9BCB.0	fTotalStrokes	 18
U+9BCB.0	fCangjie	NFEFH
U+9BCB.0	fFourCorner	29320
U+9BCB.0	fCantonese	saa1
U+9BCB.0	fMandarin	sha1
U+9BCB.0	fEnglish	shark
U+9BCC.0	fUTF8	鯌
U+9BCC.0	fCNS11643	3-5975
U+9BCC.0	fGB	E-4725
U+9BCC.0	fR/S	  195.7
U+9BCC.0	fTotalStrokes	 18
U+9BCC.0	fCangjie	NFHGR
U+9BCC.0	fFourCorner	24360
U+9BCC.0	fCantonese	hou3
U+9BCC.0	fMandarin	kao4; gao4
U+9BCD.0	fUTF8	鯍
U+9BCD.0	fCNS11643	3-5743
U+9BCD.0	fGB	E-4726
U+9BCD.0	fR/S	 195.7; 195.6
U+9BCD.0	fTotalStrokes	 18; 17
U+9BCD.0	fCangjie	NFYIU
U+9BCD.0	fFourCorner	20310
U+9BCD.0	fCantonese	mang4
U+9BCD.0	fMandarin	meng2
U+9BCE.0	fUTF8	鯎
U+9BCE.0	fGB	E-4727
U+9BCE.0	fR/S	 195.7; 195.6
U+9BCE.0	fTotalStrokes	 18
U+9BCE.0	fCangjie	NFIHS
U+9BCE.0	fCantonese	sing4
U+9BCF.0	fUTF8	鯏
U+9BCF.0	fCNS11643	F-6172
U+9BCF.0	fGB	E-4728
U+9BCF.0	fR/S	 195.7
U+9BCF.0	fTotalStrokes	 18
U+9BCF.0	fCangjie	NFHDN
U+9BCF.0	fCantonese	lei6
U+9BCF.0	fEnglish	 fish name
U+9BD0.0	fUTF8	鯐
U+9BD0.0	fGB	E-4729
U+9BD0.0	fR/S	 195.7
U+9BD0.0	fTotalStrokes	 18
U+9BD0.0	fCangjie	NFGYO
U+9BD0.0	fCantonese	zau2
U+9BD1.0	fUTF8	鯑
U+9BD1.0	fGB	E-472A
U+9BD1.0	fR/S	 195.7
U+9BD1.0	fTotalStrokes	 18
U+9BD1.0	fCangjie	NFKKB
U+9BD1.0	fCantonese	hei1
U+9BD2.0	fUTF8	鯒
U+9BD2.0	fCNS11643	3-5971
U+9BD2.0	fGB	3-782F
U+9BD2.0	fR/S	  195.7
U+9BD2.0	fTotalStrokes	 18
U+9BD2.0	fCangjie	NFNIB
U+9BD2.0	fFourCorner	27320
U+9BD2.0	fCantonese	jung2
U+9BD2.0	fMandarin	yong3
U+9BD2.0	fEnglish	a flathead
U+9BD3.0	fUTF8	鯓
U+9BD3.0	fCNS11643	2-6474
U+9BD3.0	fBig5	F1CA
U+9BD3.0	fGB	E-472B
U+9BD3.0	fR/S	 195.7
U+9BD3.0	fTotalStrokes	 18
U+9BD3.0	fCangjie	NFHXH
U+9BD3.0	fFourCorner	27340
U+9BD3.0	fCantonese	san1
U+9BD3.0	fMandarin	ni2
U+9BD4.0	fUTF8	鯔
U+9BD4.0	fCNS11643	2-683C
U+9BD4.0	fBig5	F3CF
U+9BD4.0	fGB	1-7676
U+9BD4.0	fR/S	 195.8
U+9BD4.0	fTotalStrokes	 19
U+9BD4.0	fCangjie	NFVVW
U+9BD4.0	fFourCorner	22363
U+9BD4.0	fHakka	 zih1
U+9BD4.0	fCantonese	zi1
U+9BD4.0	fMandarin	zi1; chun3
U+9BD4.0	fEnglish	mullet
U+9BD5.0	fUTF8	鯕
U+9BD5.0	fCNS11643	2-6842
U+9BD5.0	fBig5	F3D5
U+9BD5.0	fGB	3-7833
U+9BD5.0	fR/S	 195.8
U+9BD5.0	fTotalStrokes	 19
U+9BD5.0	fCangjie	NFTMC
U+9BD5.0	fFourCorner	24381
U+9BD5.0	fCantonese	kei4
U+9BD5.0	fMandarin	qi2
U+9BD5.0	fEnglish	Coryphaena hippurus
U+9BD6.0	fUTF8	鯖
U+9BD6.0	fCNS11643	1-7956
U+9BD6.0	fBig5	C44A
U+9BD6.0	fGB	1-766B
U+9BD6.0	fR/S	 195.8
U+9BD6.0	fTotalStrokes	 19
U+9BD6.0	fCangjie	NFQMB
U+9BD6.0	fFourCorner	25327
U+9BD6.0	fHakka	 ciang1; zhin1; zihn1; cin1
U+9BD6.0	fCantonese	cing1; zing1; zaang1
U+9BD6.0	fMandarin	zheng1; qing1
U+9BD6.0	fEnglish	the mackerel
U+9BD7.0	fUTF8	鯗
U+9BD7.0	fCNS11643	2-683D
U+9BD7.0	fBig5	F3D0
U+9BD7.0	fGB	5-765F
U+9BD7.0	fR/S	 195.8
U+9BD7.0	fTotalStrokes	 19
U+9BD7.0	fCangjie	TONWF
U+9BD7.0	fFourCorner	90336
U+9BD7.0	fHakka	 siong3
U+9BD7.0	fCantonese	soeng2
U+9BD7.0	fMandarin	xiang3
U+9BD7.0	fEnglish	dried and slated fish
U+9BD8.0	fUTF8	鯘
U+9BD8.0	fCNS11643	5-6B33
U+9BD8.0	fGB	5-7667
U+9BD8.0	fR/S	  195.8
U+9BD8.0	fTotalStrokes	 19
U+9BD8.0	fCangjie	NFHDV
U+9BD8.0	fFourCorner	22340
U+9BD8.0	fHakka	 nui3
U+9BD8.0	fCantonese	neoi5; leoi5 (also)
U+9BD8.0	fMandarin	nei3
U+9BD9.0	fUTF8	鯙
U+9BD9.0	fCNS11643	2-6840
U+9BD9.0	fBig5	F3D3
U+9BD9.0	fGB	E-472C
U+9BD9.0	fR/S	 195.8
U+9BD9.0	fTotalStrokes	 19
U+9BD9.0	fCangjie	NFYRD
U+9BD9.0	fFourCorner	20347
U+9BD9.0	fCantonese	seon4; king4
U+9BD9.0	fMandarin	chun2
U+9BDA.0	fUTF8	鯚
U+9BDA.0	fCNS11643	2-6844
U+9BDA.0	fBig5	F3D7
U+9BDA.0	fGB	3-7839
U+9BDA.0	fR/S	 195.8
U+9BDA.0	fTotalStrokes	 19
U+9BDA.0	fCangjie	NFHDD
U+9BDA.0	fFourCorner	22347
U+9BDA.0	fHakka	 gi5; gui5
U+9BDA.0	fCantonese	gwai3
U+9BDA.0	fMandarin	ji4
U+9BDA.0	fEnglish	a delicate fish found in the Yang-tsze
U+9BDB.0	fUTF8	鯛
U+9BDB.0	fCNS11643	1-7957
U+9BDB.0	fBig5	C44B
U+9BDB.0	fGB	1-7674
U+9BDB.0	fR/S	 195.8
U+9BDB.0	fTotalStrokes	 19
U+9BDB.0	fCangjie	NFBGR
U+9BDB.0	fFourCorner	27320
U+9BDB.0	fHakka	 diau1; tiau1; tiau2
U+9BDB.0	fCantonese	diu1
U+9BDB.0	fMandarin	diao1
U+9BDB.0	fEnglish	Pagrosomus major; porgy; sea bream; scup
U+9BDC.0	fUTF8	鯜
U+9BDC.0	fCNS11643	2-683F
U+9BDC.0	fBig5	F3D2
U+9BDC.0	fGB	E-472D
U+9BDC.0	fR/S	 195.8
U+9BDC.0	fTotalStrokes	 19
U+9BDC.0	fCangjie	NFYTV
U+9BDC.0	fFourCorner	20344
U+9BDC.0	fCantonese	cip3
U+9BDC.0	fMandarin	qie4
U+9BDD.0	fUTF8	鯝
U+9BDD.0	fCNS11643	4-625C
U+9BDD.0	fGB	1-7671
U+9BDD.0	fR/S	  195.8
U+9BDD.0	fTotalStrokes	 19
U+9BDD.0	fCangjie	NFWJR
U+9BDD.0	fFourCorner	26300
U+9BDD.0	fHakka	 gu5
U+9BDD.0	fCantonese	gu3; gwu3 (also)
U+9BDD.0	fMandarin	gu4
U+9BDD.0	fEnglish	fish guts
U+9BDE.0	fUTF8	鯞
U+9BDE.0	fCNS11643	2-6837
U+9BDE.0	fBig5	F3CA
U+9BDE.0	fGB	3-783E
U+9BDE.0	fR/S	 195.8
U+9BDE.0	fTotalStrokes	 19
U+9BDE.0	fCangjie	NFSMB
U+9BDE.0	fFourCorner	27327
U+9BDE.0	fCantonese	zau2
U+9BDE.0	fMandarin	zhou3
U+9BDF.0	fUTF8	鯟
U+9BDF.0	fCNS11643	3-5B75
U+9BDF.0	fGB	E-472E
U+9BDF.0	fR/S	  195.8
U+9BDF.0	fTotalStrokes	 19
U+9BDF.0	fCangjie	NFDW
U+9BDF.0	fFourCorner	25390
U+9BDF.0	fCantonese	dung1
U+9BDF.0	fMandarin	dong1
U+9BE0.0	fUTF8	鯠
U+9BE0.0	fCNS11643	2-6836
U+9BE0.0	fBig5	F3C9
U+9BE0.0	fGB	E-472F
U+9BE0.0	fR/S	 195.8
U+9BE0.0	fTotalStrokes	 19
U+9BE0.0	fCangjie	NFDOO
U+9BE0.0	fFourCorner	24398
U+9BE0.0	fHakka	 loi2
U+9BE0.0	fCantonese	loi4
U+9BE0.0	fMandarin	lai2
U+9BE0.0	fEnglish	 confer tobestow on an inferior to reward
U+9BE1.0	fUTF8	鯡
U+9BE1.0	fCNS11643	2-6843
U+9BE1.0	fBig5	F3D6
U+9BE1.0	fGB	1-766E
U+9BE1.0	fR/S	 195.8
U+9BE1.0	fTotalStrokes	 19
U+9BE1.0	fCangjie	NFLMY
U+9BE1.0	fFourCorner	21311
U+9BE1.0	fCantonese	fei1
U+9BE1.0	fMandarin	fei4; fei1
U+9BE1.0	fEnglish	the herring
U+9BE2.0	fUTF8	鯢
U+9BE2.0	fCNS11643	2-683A
U+9BE2.0	fBig5	F3CD
U+9BE2.0	fGB	1-7672
U+9BE2.0	fR/S	 195.8
U+9BE2.0	fTotalStrokes	 19
U+9BE2.0	fCangjie	NFHXU
U+9BE2.0	fFourCorner	27317
U+9BE2.0	fHakka	  ngi2; ni2
U+9BE2.0	fCantonese	ngai4; ai4 (also)
U+9BE2.0	fMandarin	ni2
U+9BE2.0	fEnglish	 Cryptobranchus japonicus lizard
U+9BE3.0	fUTF8	鯣
U+9BE3.0	fCNS11643	3-5B77
U+9BE3.0	fGB	5-7664
U+9BE3.0	fR/S	  195.8
U+9BE3.0	fTotalStrokes	 19
U+9BE3.0	fCangjie	NFAPH
U+9BE3.0	fFourCorner	26320
U+9BE3.0	fCantonese	jik6
U+9BE3.0	fMandarin	yi4
U+9BE3.0	fEnglish	a cuttlefish
U+9BE4.0	fUTF8	鯤
U+9BE4.0	fCNS11643	2-6838
U+9BE4.0	fBig5	F3CB
U+9BE4.0	fGB	1-766F
U+9BE4.0	fR/S	 195.8
U+9BE4.0	fTotalStrokes	 19
U+9BE4.0	fCangjie	NFAPP
U+9BE4.0	fFourCorner	26311
U+9BE4.0	fHakka	  kun1; kwun1
U+9BE4.0	fCantonese	kwan1; gwan1
U+9BE4.0	fMandarin	kun1; hun2
U+9BE4.0	fEnglish	[1] a kind of legendary fish said to be thousand of miles long [2] spawn; roe
U+9BE5.0	fUTF8	鯥
U+9BE5.0	fCNS11643	2-6841
U+9BE5.0	fBig5	F3D4
U+9BE5.0	fGB	3-7832
U+9BE5.0	fR/S	 195.8
U+9BE5.0	fTotalStrokes	 19
U+9BE5.0	fCangjie	NFGCG
U+9BE5.0	fFourCorner	24314
U+9BE5.0	fHakka	 lin2
U+9BE5.0	fCantonese	luk6
U+9BE5.0	fMandarin	lu4
U+9BE6.0	fUTF8	鯦
U+9BE6.0	fCNS11643	2-6839
U+9BE6.0	fBig5	F3CC
U+9BE6.0	fGB	5-766B
U+9BE6.0	fR/S	 195.8
U+9BE6.0	fTotalStrokes	 19
U+9BE6.0	fCangjie	NFHOR
U+9BE6.0	fFourCorner	28364
U+9BE6.0	fHakka	 giu5
U+9BE6.0	fCantonese	gau6; cau4; oi2
U+9BE6.0	fMandarin	jiu4; qiu2
U+9BE7.0	fUTF8	鯧
U+9BE7.0	fCNS11643	1-7955
U+9BE7.0	fBig5	C449
U+9BE7.0	fGB	1-7670
U+9BE7.0	fR/S	 195.8
U+9BE7.0	fTotalStrokes	 19
U+9BE7.0	fCangjie	NFAA
U+9BE7.0	fFourCorner	26360
U+9BE7.0	fHakka	 cong1; ciong1; chong1
U+9BE7.0	fCantonese	coeng1
U+9BE7.0	fMandarin	chang1
U+9BE7.0	fEnglish	pomfret; stromateoides argenteus
U+9BE8.0	fUTF8	鯨
U+9BE8.0	fCNS11643	1-7954
U+9BE8.0	fBig5	C448
U+9BE8.0	fGB	1-3E28
U+9BE8.0	fR/S	 195.8
U+9BE8.0	fTotalStrokes	 19
U+9BE8.0	fCangjie	NFYRF
U+9BE8.0	fFourCorner	20396
U+9BE8.0	fHakka	 gin1; kiang1; kiang2; kin2; kin1
U+9BE8.0	fCantonese	king4
U+9BE8.0	fMandarin	jing1; qing2; qiang1
U+9BE8.0	fEnglish	the whale
U+9BE9.0	fUTF8	鯩
U+9BE9.0	fCNS11643	3-5B79
U+9BE9.0	fGB	3-774B
U+9BE9.0	fR/S	 195.8
U+9BE9.0	fTotalStrokes	 19
U+9BE9.0	fCangjie	NFOMB
U+9BE9.0	fFourCorner	28320
U+9BE9.0	fCantonese	leon4
U+9BE9.0	fMandarin	lun2
U+9BEA.0	fUTF8	鯪
U+9BEA.0	fCNS11643	2-6834
U+9BEA.0	fBig5	F3C7
U+9BEA.0	fGB	1-766C
U+9BEA.0	fR/S	 195.8
U+9BEA.0	fTotalStrokes	 19
U+9BEA.0	fCangjie	NFGCE
U+9BEA.0	fFourCorner	24347
U+9BEA.0	fHakka	  lin2; lien1; lin3; lang3; lang2; lien2
U+9BEA.0	fCantonese	ling4; leng4 (also)
U+9BEA.0	fMandarin	ling2
U+9BEA.0	fEnglish	[1] a legedary fish [2] dace
U+9BEB.0	fUTF8	鯫
U+9BEB.0	fCNS11643	2-6835
U+9BEB.0	fBig5	F3C8
U+9BEB.0	fGB	1-766D
U+9BEB.0	fR/S	 195.8
U+9BEB.0	fTotalStrokes	 19
U+9BEB.0	fCangjie	NFSJE
U+9BEB.0	fFourCorner	27340
U+9BEB.0	fHakka	  zeu1
U+9BEB.0	fCantonese	zau1; zau6; zip3
U+9BEB.0	fMandarin	zou1; qu1; qian3; chou3; se4
U+9BEB.0	fEnglish	[1] a small fish [2] small
U+9BEC.0	fUTF8	鯬
U+9BEC.0	fCNS11643	2-683E
U+9BEC.0	fBig5	F3D1
U+9BEC.0	fGB	5-7666
U+9BEC.0	fR/S	 195.8
U+9BEC.0	fTotalStrokes	 19
U+9BEC.0	fCangjie	HHNWF
U+9BEC.0	fFourCorner	27336
U+9BEC.0	fCantonese	lei4; lai4
U+9BEC.0	fMandarin	li2
U+9BED.0	fUTF8	鯭
U+9BED.0	fCNS11643	4-6266
U+9BED.0	fGB	1-7C56
U+9BED.0	fR/S	 195.8
U+9BED.0	fTotalStrokes	 19
U+9BED.0	fCangjie	NFNDT
U+9BED.0	fFourCorner	27310
U+9BED.0	fCantonese	maang1
U+9BED.0	fMandarin	meng3
U+9BED.0	fEnglish	泥鯭, a kind of fish that lives in turbid water
U+9BEE.0	fUTF8	鯮
U+9BEE.0	fCNS11643	3-5B7A
U+9BEE.0	fGB	3-783C
U+9BEE.0	fR/S	 195.8
U+9BEE.0	fTotalStrokes	 19
U+9BEE.0	fCangjie	NFJMF
U+9BEE.0	fFourCorner	23390
U+9BEE.0	fCantonese	zung1
U+9BEE.0	fMandarin	zong1
U+9BEF.0	fUTF8	鯯
U+9BEF.0	fCNS11643	4-6268
U+9BEF.0	fGB	E-4730
U+9BEF.0	fR/S	  195.8
U+9BEF.0	fTotalStrokes	 19
U+9BEF.0	fCangjie	NFHBN
U+9BEF.0	fFourCorner	22300
U+9BEF.0	fCantonese	zai3
U+9BEF.0	fMandarin	zhi4; ji4
U+9BF0.0	fUTF8	鯰
U+9BF0.0	fCNS11643	2-683B
U+9BF0.0	fBig5	F3CE
U+9BF0.0	fGB	1-7673
U+9BF0.0	fR/S	 195.8
U+9BF0.0	fTotalStrokes	 19
U+9BF0.0	fCangjie	NFOIP
U+9BF0.0	fFourCorner	28332
U+9BF0.0	fHakka	 ngiam2; ngiam5
U+9BF0.0	fCantonese	nim4; lim4 (also); lim6
U+9BF0.0	fMandarin	nian2; nian3
U+9BF0.0	fEnglish	sheat
U+9BF1.0	fUTF8	鯱
U+9BF1.0	fGB	E-4731
U+9BF1.0	fR/S	 195.8
U+9BF1.0	fTotalStrokes	 19
U+9BF1.0	fCangjie	NFYPU
U+9BF1.0	fFourCorner	2131
U+9BF1.0	fCantonese	fu2
U+9BF1.0	fEnglish	mythical dolphin-like fish
U+9BF2.0	fUTF8	鯲
U+9BF2.0	fGB	E-4732
U+9BF2.0	fR/S	 195.8
U+9BF2.0	fTotalStrokes	 19
U+9BF2.0	fCangjie	NFYSY
U+9BF2.0	fCantonese	jyu1
U+9BF3.0	fUTF8	鯳
U+9BF3.0	fCNS11643	F-646B
U+9BF3.0	fGB	E-4733
U+9BF3.0	fR/S	 195.8
U+9BF3.0	fTotalStrokes	 19
U+9BF3.0	fCangjie	NFIHM
U+9BF3.0	fFourCorner	2034
U+9BF3.0	fCantonese	dai2
U+9BF4.0	fUTF8	鯴
U+9BF4.0	fCNS11643	4-6255
U+9BF4.0	fGB	1-7675
U+9BF4.0	fR/S	  195.8
U+9BF4.0	fTotalStrokes	 19
U+9BF4.0	fCangjie	NFNHI
U+9BF4.0	fFourCorner	27310
U+9BF4.0	fCantonese	sat1
U+9BF4.0	fMandarin	shi1
U+9BF5.0	fUTF8	鯵
U+9BF5.0	fGB	E-4734
U+9BF5.0	fR/S	 195.8
U+9BF5.0	fTotalStrokes	 19
U+9BF5.0	fCangjie	NFIKH
U+9BF5.0	fCantonese	sam1
U+9BF5.0	fMandarin	shen1; sao1
U+9BF6.0	fUTF8	鯶
U+9BF6.0	fCNS11643	3-5D6F
U+9BF6.0	fGB	5-7658
U+9BF6.0	fR/S	  195.9
U+9BF6.0	fTotalStrokes	 20
U+9BF6.0	fCangjie	NFBJJ
U+9BF6.0	fFourCorner	27350
U+9BF6.0	fCantonese	waan5; wan6
U+9BF6.0	fMandarin	hun3; huan4
U+9BF7.0	fUTF8	鯷
U+9BF7.0	fCNS11643	2-6A7A
U+9BF7.0	fBig5	F56C
U+9BF7.0	fGB	3-7849
U+9BF7.0	fR/S	 195.9
U+9BF7.0	fTotalStrokes	 20
U+9BF7.0	fCangjie	NFAMO
U+9BF7.0	fFourCorner	26381
U+9BF7.0	fHakka	  ti2
U+9BF7.0	fCantonese	tai4
U+9BF7.0	fMandarin	shi4; ti2; ti4
U+9BF7.0	fEnglish	anchovy
U+9BF8.0	fUTF8	鯸
U+9BF8.0	fCNS11643	2-6A7D
U+9BF8.0	fBig5	F56F
U+9BF8.0	fGB	3-7851
U+9BF8.0	fR/S	 195.9
U+9BF8.0	fTotalStrokes	 20
U+9BF8.0	fCangjie	NFONK
U+9BF8.0	fFourCorner	27334
U+9BF8.0	fHakka	  heu2; hiu2
U+9BF8.0	fCantonese	hau4
U+9BF8.0	fMandarin	hou2
U+9BF8.0	fEnglish	blowfish
U+9BF9.0	fUTF8	鯹
U+9BF9.0	fCNS11643	3-5D71
U+9BF9.0	fGB	E-4735
U+9BF9.0	fR/S	  195.9
U+9BF9.0	fTotalStrokes	 20
U+9BF9.0	fCangjie	NFAHM
U+9BF9.0	fFourCorner	26310
U+9BF9.0	fHakka	 siang1;  sen1
U+9BF9.0	fCantonese	sing1
U+9BF9.0	fMandarin	xing1; zheng1
U+9BF9.0	fEnglish	putrid fish
U+9BFA.0	fUTF8	鯺
U+9BFA.0	fCNS11643	4-6256
U+9BFA.0	fGB	5-7661
U+9BFA.0	fR/S	 195.9; 195.8
U+9BFA.0	fTotalStrokes	 20; 19
U+9BFA.0	fCangjie	NFJKA
U+9BFA.0	fFourCorner	24360
U+9BFA.0	fCantonese	zyu1
U+9BFA.0	fMandarin	zhu1; chu2
U+9BFA.0	fEnglish	blowfish
U+9BFB.0	fUTF8	鯻
U+9BFB.0	fCNS11643	3-5D67
U+9BFB.0	fGB	3-7845
U+9BFB.0	fR/S	 195.8; 195.9
U+9BFB.0	fTotalStrokes	 20; 19
U+9BFB.0	fCangjie	NFDLN
U+9BFB.0	fFourCorner	22300
U+9BFB.0	fCantonese	laat6
U+9BFB.0	fMandarin	la4
U+9BFC.0	fUTF8	鯼
U+9BFC.0	fCNS11643	3-5D69
U+9BFC.0	fGB	5-7670
U+9BFC.0	fR/S	  195.9
U+9BFC.0	fTotalStrokes	 20
U+9BFC.0	fCangjie	NFUCE
U+9BFC.0	fFourCorner	22340
U+9BFC.0	fHakka	 zung1
U+9BFC.0	fCantonese	zung1
U+9BFC.0	fMandarin	zong1
U+9BFC.0	fEnglish	a large fish with a round body and a long nose
U+9BFD.0	fUTF8	鯽
U+9BFD.0	fCNS11643	1-7822
U+9BFD.0	fBig5	C356
U+9BFD.0	fGB	1-766A
U+9BFD.0	fR/S	 195.9
U+9BFD.0	fTotalStrokes	 20
U+9BFD.0	fCangjie	NFAIL
U+9BFD.0	fFourCorner	27320
U+9BFD.0	fHakka	  zit7; zit8
U+9BFD.0	fCantonese	zik1; zak1 (also)
U+9BFD.0	fMandarin	ji4; ji2; zei2; ze2
U+9BFD.0	fEnglish	Carassius auratus; crucian carp; gold carp
U+9BFE.0	fUTF8	鯾
U+9BFE.0	fCNS11643	3-5D6A
U+9BFE.0	fGB	3-784E
U+9BFE.0	fR/S	  195.9
U+9BFE.0	fTotalStrokes	 20
U+9BFE.0	fCangjie	NFOMK
U+9BFE.0	fFourCorner	21340
U+9BFE.0	fCantonese	bin1; bin2 (also)
U+9BFE.0	fMandarin	bian1
U+9BFF.0	fUTF8	鯿
U+9BFF.0	fCNS11643	3-5D74
U+9BFF.0	fGB	1-767D
U+9BFF.0	fR/S	 195.9
U+9BFF.0	fTotalStrokes	 20
U+9BFF.0	fCangjie	NFHSB
U+9BFF.0	fFourCorner	23320
U+9BFF.0	fHakka	  bien1
U+9BFF.0	fCantonese	bin1; bin2 (also)
U+9BFF.0	fMandarin	bian1
U+9BFF.0	fEnglish	 freshwater bream
U+9C00.0	fUTF8	鰀
U+9C00.0	fCNS11643	3-5D75
U+9C00.0	fGB	E-4736
U+9C00.0	fR/S	 195.9
U+9C00.0	fTotalStrokes	 20
U+9C00.0	fCangjie	NFBME
U+9C00.0	fFourCorner	22340
U+9C00.0	fHakka	 fon5; fon6
U+9C00.0	fCantonese	waan5
U+9C00.0	fMandarin	huan4; wan3
U+9C01.0	fUTF8	鰁
U+9C01.0	fCNS11643	4-655A
U+9C01.0	fGB	3-7850
U+9C01.0	fR/S	  195.9
U+9C01.0	fTotalStrokes	 20
U+9C01.0	fCangjie	NFHAE
U+9C01.0	fFourCorner	26330
U+9C01.0	fCantonese	cyun4
U+9C01.0	fMandarin	quan2
U+9C01.0	fEnglish	a fish name
U+9C02.0	fUTF8	鰂
U+9C02.0	fCNS11643	3-5D6E
U+9C02.0	fGB	3-7774
U+9C02.0	fR/S	 195.9
U+9C02.0	fTotalStrokes	 20
U+9C02.0	fCangjie	NFBCN
U+9C02.0	fFourCorner	22300
U+9C02.0	fHakka	 cet7;  zet7
U+9C02.0	fCantonese	caak6; zak1; caak3
U+9C02.0	fMandarin	ze2; zei2; obs
U+9C02.0	fEnglish	cuttlefish
U+9C03.0	fUTF8	鰃
U+9C03.0	fCNS11643	4-6559
U+9C03.0	fGB	3-784C
U+9C03.0	fR/S	  195.9
U+9C03.0	fTotalStrokes	 20
U+9C03.0	fCangjie	NFWMV
U+9C03.0	fFourCorner	26330
U+9C03.0	fCantonese	wai1; wui1 (also)
U+9C03.0	fMandarin	wei1; wei4
U+9C03.0	fEnglish	a fish name
U+9C04.0	fUTF8	鰄
U+9C04.0	fCNS11643	3-5D68
U+9C04.0	fGB	E-4737
U+9C04.0	fR/S	  195.9
U+9C04.0	fTotalStrokes	 20
U+9C04.0	fCangjie	NFIHV
U+9C04.0	fFourCorner	23350
U+9C04.0	fCantonese	wai1
U+9C04.0	fMandarin	wei1
U+9C05.0	fUTF8	鰅
U+9C05.0	fCNS11643	2-6A7B
U+9C05.0	fBig5	F56D
U+9C05.0	fGB	3-784A
U+9C05.0	fR/S	 195.9
U+9C05.0	fTotalStrokes	 20
U+9C05.0	fCangjie	NFWLB
U+9C05.0	fFourCorner	26327
U+9C05.0	fCantonese	jyu4; jung4 (also)
U+9C05.0	fMandarin	yu2
U+9C06.0	fUTF8	鰆
U+9C06.0	fCNS11643	2-6B23
U+9C06.0	fBig5	F573
U+9C06.0	fGB	3-7843
U+9C06.0	fR/S	 195.9
U+9C06.0	fTotalStrokes	 20
U+9C06.0	fCangjie	NFQKA
U+9C06.0	fFourCorner	25363
U+9C06.0	fHakka	 chun1; cun1
U+9C06.0	fCantonese	ceon1; zeon1
U+9C06.0	fMandarin	chun1; qun1
U+9C06.0	fEnglish	a saltwater fish; mullet; roach
U+9C07.0	fUTF8	鰇
U+9C07.0	fCNS11643	2-6B21
U+9C07.0	fBig5	F571
U+9C07.0	fGB	E-4738
U+9C07.0	fR/S	 195.9
U+9C07.0	fTotalStrokes	 20
U+9C07.0	fCangjie	NFNHD
U+9C07.0	fFourCorner	27394
U+9C07.0	fCantonese	jau4
U+9C07.0	fMandarin	rou2
U+9C08.0	fUTF8	鰈
U+9C08.0	fCNS11643	2-6A79
U+9C08.0	fBig5	F56B
U+9C08.0	fGB	1-7678
U+9C08.0	fR/S	 195.9
U+9C08.0	fTotalStrokes	 20
U+9C08.0	fCangjie	NFPTD
U+9C08.0	fFourCorner	24394
U+9C08.0	fHakka	  tiap8
U+9C08.0	fCantonese	dip6; cip3
U+9C08.0	fMandarin	die2; die1; qie4; qie1; zha2; ta4
U+9C08.0	fEnglish	 sole (fish)
U+9C09.0	fUTF8	鰉
U+9C09.0	fCNS11643	2-6B26
U+9C09.0	fBig5	F576
U+9C09.0	fGB	1-767C
U+9C09.0	fR/S	 195.9
U+9C09.0	fTotalStrokes	 20
U+9C09.0	fCangjie	NFHAG
U+9C09.0	fFourCorner	26314
U+9C09.0	fHakka	 fong2
U+9C09.0	fCantonese	wong4
U+9C09.0	fMandarin	huang2
U+9C09.0	fEnglish	sturgeon
U+9C0A.0	fUTF8	鰊
U+9C0A.0	fCNS11643	3-5D66
U+9C0A.0	fGB	3-7847
U+9C0A.0	fR/S	 195.9
U+9C0A.0	fTotalStrokes	 20
U+9C0A.0	fCangjie	NFDWF
U+9C0A.0	fFourCorner	25390
U+9C0A.0	fCantonese	lin6
U+9C0A.0	fMandarin	lian4
U+9C0A.0	fEnglish	a herring
U+9C0B.0	fUTF8	鰋
U+9C0B.0	fCNS11643	2-6A78
U+9C0B.0	fBig5	F56A
U+9C0B.0	fGB	3-7844
U+9C0B.0	fR/S	 195.9
U+9C0B.0	fTotalStrokes	 20
U+9C0B.0	fCangjie	NFSAV
U+9C0B.0	fFourCorner	21314
U+9C0B.0	fHakka	 jen3; ren3
U+9C0B.0	fCantonese	jin2
U+9C0B.0	fMandarin	yan3
U+9C0B.0	fEnglish	sheat
U+9C0C.0	fUTF8	鰌
U+9C0C.0	fCNS11643	3-5D76
U+9C0C.0	fGB	E-4739
U+9C0C.0	fR/S	 195.9
U+9C0C.0	fTotalStrokes	 20
U+9C0C.0	fCangjie	NFTCW
U+9C0C.0	fFourCorner	28360
U+9C0C.0	fHakka	  ciu1
U+9C0C.0	fCantonese	cau1; jau4
U+9C0C.0	fMandarin	qiu2; qiu1
U+9C0C.0	fEnglish	 loach
U+9C0D.0	fUTF8	鰍
U+9C0D.0	fCNS11643	1-7A5A
U+9C0D.0	fBig5	C4CF
U+9C0D.0	fGB	1-767A
U+9C0D.0	fR/S	 195.9
U+9C0D.0	fTotalStrokes	 20
U+9C0D.0	fCangjie	NFHDF
U+9C0D.0	fFourCorner	29380
U+9C0D.0	fHakka	  ciu1
U+9C0D.0	fCantonese	cau1
U+9C0D.0	fMandarin	qiu1
U+9C0D.0	fEnglish	 loach
U+9C0E.0	fUTF8	鰎
U+9C0E.0	fCNS11643	2-6B22
U+9C0E.0	fBig5	F572
U+9C0E.0	fGB	5-766D
U+9C0E.0	fR/S	 195.9
U+9C0E.0	fTotalStrokes	 20
U+9C0E.0	fCangjie	NFNKQ
U+9C0E.0	fFourCorner	25344
U+9C0E.0	fCantonese	gin2
U+9C0E.0	fMandarin	jian4
U+9C0F.0	fUTF8	鰏
U+9C0F.0	fCNS11643	4-6554
U+9C0F.0	fGB	3-7846
U+9C0F.0	fR/S	  195.9
U+9C0F.0	fTotalStrokes	 20
U+9C0F.0	fCangjie	NFMRW
U+9C0F.0	fFourCorner	21360
U+9C0F.0	fCantonese	bik1
U+9C0F.0	fMandarin	bi4; bi1
U+9C10.0	fUTF8	鰐
U+9C10.0	fCNS11643	3-5D73
U+9C10.0	fGB	1-7679
U+9C10.0	fR/S	 195.9
U+9C10.0	fTotalStrokes	 20
U+9C10.0	fCangjie	NFRRS
U+9C10.0	fFourCorner	26320
U+9C10.0	fHakka	 ngok8
U+9C10.0	fCantonese	ngok6; ok6 (also)
U+9C10.0	fMandarin	e4
U+9C10.0	fEnglish	alligator
U+9C11.0	fUTF8	鰑
U+9C11.0	fCNS11643	4-6557
U+9C11.0	fGB	E-473A
U+9C11.0	fR/S	  195.9
U+9C11.0	fTotalStrokes	 20
U+9C11.0	fCangjie	NFAMH
U+9C11.0	fFourCorner	26320
U+9C11.0	fCantonese	joeng4
U+9C11.0	fMandarin	yang2
U+9C12.0	fUTF8	鰒
U+9C12.0	fCNS11643	2-6A7C
U+9C12.0	fBig5	F56E
U+9C12.0	fGB	1-767B
U+9C12.0	fR/S	 195.9
U+9C12.0	fTotalStrokes	 20
U+9C12.0	fCangjie	NFOAE
U+9C12.0	fFourCorner	28347
U+9C12.0	fHakka	 fuk8; fuk7
U+9C12.0	fCantonese	fuk1; fuk6
U+9C12.0	fMandarin	fu4; bao4
U+9C12.0	fEnglish	 abalone
U+9C13.0	fUTF8	鰓
U+9C13.0	fCNS11643	1-7A59
U+9C13.0	fBig5	C4CE
U+9C13.0	fGB	1-487A
U+9C13.0	fR/S	 195.9
U+9C13.0	fTotalStrokes	 20
U+9C13.0	fCangjie	NFWP
U+9C13.0	fFourCorner	26330
U+9C13.0	fHakka	 soi1; sui1; sii1
U+9C13.0	fCantonese	soi1; saai2; sai2
U+9C13.0	fMandarin	sai1; xi3
U+9C13.0	fEnglish	fish gills
U+9C14.0	fUTF8	鰔
U+9C14.0	fCNS11643	2-6B25
U+9C14.0	fBig5	F575
U+9C14.0	fGB	5-766E
U+9C14.0	fR/S	 195.9
U+9C14.0	fTotalStrokes	 20
U+9C14.0	fCangjie	NFIHR
U+9C14.0	fFourCorner	23350
U+9C14.0	fHakka	 gam3
U+9C14.0	fCantonese	gam2; zam1; gam3
U+9C14.0	fMandarin	jian3; gan3; xian2
U+9C14.0	fEnglish	a kind of shad (fish)
U+9C15.0	fUTF8	鰕
U+9C15.0	fCNS11643	3-5D72
U+9C15.0	fGB	3-7857
U+9C15.0	fR/S	 195.9
U+9C15.0	fTotalStrokes	 20
U+9C15.0	fCangjie	NFRSE
U+9C15.0	fFourCorner	27340
U+9C15.0	fHakka	 ha2
U+9C15.0	fCantonese	haa1
U+9C15.0	fMandarin	xia2; xia1
U+9C15.0	fEnglish	shrimp; prawn
U+9C16.0	fUTF8	鰖
U+9C16.0	fCNS11643	4-655D
U+9C16.0	fGB	E-473B
U+9C16.0	fR/S	  195.9
U+9C16.0	fTotalStrokes	 20
U+9C16.0	fCangjie	NFKMB
U+9C16.0	fFourCorner	24320
U+9C16.0	fCantonese	to2
U+9C16.0	fMandarin	tuo3; wei3
U+9C17.0	fUTF8	鰗
U+9C17.0	fCNS11643	2-6B24
U+9C17.0	fBig5	F574
U+9C17.0	fGB	E-473C
U+9C17.0	fR/S	 195.9
U+9C17.0	fTotalStrokes	 20
U+9C17.0	fCangjie	NFJRB
U+9C17.0	fFourCorner	27320
U+9C17.0	fCantonese	wu4
U+9C17.0	fMandarin	hu2
U+9C18.0	fUTF8	鰘
U+9C18.0	fGB	E-473D
U+9C18.0	fR/S	 195.9
U+9C18.0	fTotalStrokes	 20
U+9C18.0	fCangjie	NFJMG
U+9C18.0	fCantonese	sat1
U+9C19.0	fUTF8	鰙
U+9C19.0	fGB	E-473E
U+9C19.0	fR/S	 195.9; 195.8
U+9C19.0	fTotalStrokes	 20
U+9C19.0	fCangjie	NFTKR
U+9C19.0	fCantonese	joek6
U+9C19.0	fMandarin	ruo4
U+9C1A.0	fUTF8	鰚
U+9C1A.0	fGB	E-473F
U+9C1A.0	fR/S	 195.9
U+9C1A.0	fTotalStrokes	 20
U+9C1A.0	fCangjie	NFJMM
U+9C1A.0	fCantonese	syun1
U+9C1B.0	fUTF8	鰛
U+9C1B.0	fCNS11643	3-5D70
U+9C1B.0	fGB	3-784B
U+9C1B.0	fR/S	 195.9
U+9C1B.0	fTotalStrokes	 20
U+9C1B.0	fCangjie	NFABT
U+9C1B.0	fFourCorner	26310
U+9C1B.0	fCantonese	wan1
U+9C1B.0	fMandarin	wen1
U+9C1B.0	fEnglish	sardine
U+9C1C.0	fUTF8	鰜
U+9C1C.0	fCNS11643	2-6C77
U+9C1C.0	fBig5	F6AB
U+9C1C.0	fGB	3-7861
U+9C1C.0	fR/S	 195.10
U+9C1C.0	fTotalStrokes	 21
U+9C1C.0	fCangjie	NFTXC
U+9C1C.0	fFourCorner	28337
U+9C1C.0	fHakka	  giam1
U+9C1C.0	fCantonese	gim1
U+9C1C.0	fMandarin	jian1; xian2
U+9C1C.0	fEnglish	 sole (fish)
U+9C1D.0	fUTF8	鰝
U+9C1D.0	fCNS11643	2-6C76
U+9C1D.0	fBig5	F6AA
U+9C1D.0	fGB	3-785E
U+9C1D.0	fR/S	 195.10
U+9C1D.0	fTotalStrokes	 21
U+9C1D.0	fCangjie	NFYRB
U+9C1D.0	fFourCorner	20327
U+9C1D.0	fHakka	  hau1; gau3
U+9C1D.0	fCantonese	hou6; hou1
U+9C1D.0	fMandarin	hao4
U+9C1D.0	fEnglish	female whale
U+9C1E.0	fUTF8	鰞
U+9C1E.0	fCNS11643	5-7251
U+9C1E.0	fGB	E-4740
U+9C1E.0	fR/S	  195.10
U+9C1E.0	fTotalStrokes	 21
U+9C1E.0	fCangjie	NFHRF
U+9C1E.0	fFourCorner	22320
U+9C1E.0	fCantonese	wu1
U+9C1E.0	fMandarin	wu1; wu4
U+9C1E.0	fEnglish	squid
U+9C1F.0	fUTF8	鰟
U+9C1F.0	fCNS11643	3-5F2C
U+9C1F.0	fGB	3-785F
U+9C1F.0	fR/S	  195.10
U+9C1F.0	fTotalStrokes	 21
U+9C1F.0	fCangjie	NFYBS
U+9C1F.0	fFourCorner	20320
U+9C1F.0	fCantonese	pong4; fong4
U+9C1F.0	fMandarin	fang2; pang2
U+9C20.0	fUTF8	鰠
U+9C20.0	fCNS11643	5-724B
U+9C20.0	fGB	1-767E
U+9C20.0	fR/S	 195.9; 195.10
U+9C20.0	fTotalStrokes	 21; 20
U+9C20.0	fCangjie	NFEII
U+9C20.0	fFourCorner	27330
U+9C20.0	fHakka	 sau1
U+9C20.0	fCantonese	sou1
U+9C20.0	fMandarin	sao1
U+9C20.0	fEnglish	a kind of bram found in Shensi
U+9C21.0	fUTF8	鰡
U+9C21.0	fCNS11643	2-6C7D
U+9C21.0	fBig5	F6B1
U+9C21.0	fGB	3-785D
U+9C21.0	fR/S	 195.10
U+9C21.0	fTotalStrokes	 21
U+9C21.0	fCangjie	NFHHW
U+9C21.0	fFourCorner	27362
U+9C21.0	fCantonese	lau4
U+9C21.0	fMandarin	liu2; liu1
U+9C22.0	fUTF8	鰢
U+9C22.0	fCNS11643	5-724E
U+9C22.0	fGB	E-4741
U+9C22.0	fR/S	  195.10
U+9C22.0	fTotalStrokes	 21
U+9C22.0	fCangjie	NFSQF
U+9C22.0	fFourCorner	21320
U+9C22.0	fHakka	 ma3; ma1
U+9C22.0	fCantonese	maa5
U+9C22.0	fMandarin	ma3
U+9C22.0	fEnglish	prawns
U+9C23.0	fUTF8	鰣
U+9C23.0	fCNS11643	2-6C79
U+9C23.0	fBig5	F6AD
U+9C23.0	fGB	1-7665
U+9C23.0	fR/S	 195.10
U+9C23.0	fTotalStrokes	 21
U+9C23.0	fCangjie	NFAGI
U+9C23.0	fFourCorner	24341
U+9C23.0	fHakka	 shi2; sii2; si2
U+9C23.0	fCantonese	si4
U+9C23.0	fMandarin	shi2
U+9C23.0	fEnglish	 reeves' shad; hilsa herring
U+9C24.0	fUTF8	鰤
U+9C24.0	fCNS11643	2-6C7C
U+9C24.0	fBig5	F6B0
U+9C24.0	fGB	3-7770
U+9C24.0	fR/S	 195.10
U+9C24.0	fTotalStrokes	 21
U+9C24.0	fCangjie	NFHRB
U+9C24.0	fFourCorner	21327
U+9C24.0	fCantonese	si1
U+9C24.0	fMandarin	shi1
U+9C24.0	fEnglish	a yellowtail
U+9C25.0	fUTF8	鰥
U+9C25.0	fCNS11643	1-7B4C
U+9C25.0	fBig5	C560
U+9C25.0	fGB	1-7724
U+9C25.0	fR/S	 195.10
U+9C25.0	fTotalStrokes	 21
U+9C25.0	fCangjie	NFWLE
U+9C25.0	fFourCorner	26333
U+9C25.0	fHakka	 gan1; gon1; gwan1
U+9C25.0	fCantonese	gwaan1; kwan1
U+9C25.0	fMandarin	guan1; guan4; gun3; kun1; yin2
U+9C25.0	fEnglish	[1] a kind of predatory huge fish [2] widower [3] bachelor
U+9C26.0	fUTF8	鰦
U+9C26.0	fCNS11643	4-683B
U+9C26.0	fGB	3-7855
U+9C26.0	fR/S	  195.10; 195.9
U+9C26.0	fTotalStrokes	 21
U+9C26.0	fCangjie	NFTVI
U+9C26.0	fFourCorner	24330
U+9C26.0	fHakka	 zih1
U+9C26.0	fCantonese	zi1
U+9C26.0	fMandarin	zi1
U+9C26.0	fEnglish	the name of a small fish
U+9C27.0	fUTF8	鰧
U+9C27.0	fCNS11643	4-683E
U+9C27.0	fGB	3-4E24
U+9C27.0	fR/S	  195.10
U+9C27.0	fTotalStrokes	 21
U+9C27.0	fCangjie	BFQF
U+9C27.0	fFourCorner	79230
U+9C27.0	fCantonese	tang4
U+9C27.0	fMandarin	teng2
U+9C28.0	fUTF8	鰨
U+9C28.0	fCNS11643	2-6C7A
U+9C28.0	fBig5	F6AE
U+9C28.0	fGB	1-7723
U+9C28.0	fR/S	 195.10
U+9C28.0	fTotalStrokes	 21
U+9C28.0	fCangjie	NFASM
U+9C28.0	fFourCorner	26327
U+9C28.0	fHakka	 tap7
U+9C28.0	fCantonese	taap3; taat3; dip6
U+9C28.0	fMandarin	ta4; ta3; na4; die2
U+9C28.0	fEnglish	sole
U+9C29.0	fUTF8	鰩
U+9C29.0	fCNS11643	2-6C7B
U+9C29.0	fBig5	F6AF
U+9C29.0	fGB	1-7725
U+9C29.0	fR/S	 195.10
U+9C29.0	fTotalStrokes	 21
U+9C29.0	fCangjie	NFBOU
U+9C29.0	fFourCorner	27372
U+9C29.0	fHakka	 jau2; rau2; jeu2
U+9C29.0	fCantonese	jiu4
U+9C29.0	fMandarin	yao2
U+9C29.0	fEnglish	the nautilus; the ray
U+9C2A.0	fUTF8	鰪
U+9C2A.0	fCNS11643	4-6839
U+9C2A.0	fGB	3-7859
U+9C2A.0	fR/S	  195.10
U+9C2A.0	fTotalStrokes	 21
U+9C2A.0	fCangjie	NFGIT
U+9C2A.0	fFourCorner	24310
U+9C2A.0	fCantonese	ap1; ngap1 (also)
U+9C2A.0	fMandarin	ge2
U+9C2B.0	fUTF8	鰫
U+9C2B.0	fCNS11643	2-6C75
U+9C2B.0	fBig5	F6A9
U+9C2B.0	fGB	5-7675
U+9C2B.0	fR/S	 195.10
U+9C2B.0	fTotalStrokes	 21
U+9C2B.0	fCangjie	NFJCR
U+9C2B.0	fFourCorner	23368
U+9C2B.0	fHakka	 jung2; rung2
U+9C2B.0	fCantonese	jung4; sung4 (also)
U+9C2B.0	fMandarin	rong2; yong2
U+9C2B.0	fEnglish	a kind of tench, common at Canton
U+9C2C.0	fUTF8	鰬
U+9C2C.0	fCNS11643	2-6C78
U+9C2C.0	fBig5	F6AC
U+9C2C.0	fGB	3-785B
U+9C2C.0	fR/S	 195.10
U+9C2C.0	fTotalStrokes	 21
U+9C2C.0	fCangjie	NFYPK
U+9C2C.0	fFourCorner	21344
U+9C2C.0	fCantonese	kin4
U+9C2C.0	fMandarin	qian2
U+9C2D.0	fUTF8	鰭
U+9C2D.0	fCNS11643	1-7B4B
U+9C2D.0	fBig5	C55F
U+9C2D.0	fGB	1-7722
U+9C2D.0	fR/S	 195.10
U+9C2D.0	fTotalStrokes	 21
U+9C2D.0	fCangjie	NFJPA
U+9C2D.0	fFourCorner	24361
U+9C2D.0	fHakka	  ki2
U+9C2D.0	fCantonese	kei4
U+9C2D.0	fMandarin	qi2
U+9C2D.0	fEnglish	 fin
U+9C2E.0	fUTF8	鰮
U+9C2E.0	fCNS11643	3-5F2A
U+9C2E.0	fGB	E-4742
U+9C2E.0	fR/S	  195.10
U+9C2E.0	fTotalStrokes	 21
U+9C2E.0	fCangjie	NFWOT
U+9C2E.0	fFourCorner	26310
U+9C2E.0	fCantonese	wan1
U+9C2E.0	fMandarin	wen1
U+9C2E.0	fEnglish	sardine
U+9C2F.0	fUTF8	鰯
U+9C2F.0	fCNS11643	F-692A
U+9C2F.0	fGB	3-7862
U+9C2F.0	fR/S	 195.10
U+9C2F.0	fTotalStrokes	 21
U+9C2F.0	fCangjie	NFNMM
U+9C2F.0	fFourCorner	2732
U+9C2F.0	fCantonese	joek6
U+9C2F.0	fMandarin	ruo4
U+9C2F.0	fEnglish	 sardine
U+9C30.0	fUTF8	鰰
U+9C30.0	fGB	E-4743
U+9C30.0	fR/S	 195.10
U+9C30.0	fTotalStrokes	 21
U+9C30.0	fCangjie	NFMFL
U+9C30.0	fCantonese	san4
U+9C31.0	fUTF8	鰱
U+9C31.0	fCNS11643	1-7C2B
U+9C31.0	fBig5	C5BF
U+9C31.0	fGB	1-7663
U+9C31.0	fR/S	 195.11
U+9C31.0	fTotalStrokes	 22
U+9C31.0	fCangjie	NFYJJ
U+9C31.0	fFourCorner	25330
U+9C31.0	fHakka	  lien2; lian3; len2
U+9C31.0	fCantonese	lin4
U+9C31.0	fMandarin	lian2
U+9C31.0	fEnglish	 silver carp
U+9C32.0	fUTF8	鰲
U+9C32.0	fCNS11643	2-6E61
U+9C32.0	fBig5	F7B4
U+9C32.0	fGB	1-7721
U+9C32.0	fR/S	 195.11
U+9C32.0	fTotalStrokes	 22
U+9C32.0	fCangjie	GKNWF
U+9C32.0	fFourCorner	48336
U+9C32.0	fHakka	 ngau2; ngau3; ngo2
U+9C32.0	fCantonese	ngou4; ou4 (also)
U+9C32.0	fMandarin	ao2
U+9C32.0	fEnglish	 sea turtle
U+9C33.0	fUTF8	鰳
U+9C33.0	fCNS11643	2-6E5C
U+9C33.0	fBig5	F7AF
U+9C33.0	fGB	1-7726
U+9C33.0	fR/S	 195.11
U+9C33.0	fTotalStrokes	 22
U+9C33.0	fCangjie	NFTJS
U+9C33.0	fFourCorner	24327
U+9C33.0	fCantonese	lak6; laak6 (also)
U+9C33.0	fMandarin	le4
U+9C33.0	fEnglish	 chinese herring shad
U+9C34.0	fUTF8	鰴
U+9C34.0	fCNS11643	2-6E60
U+9C34.0	fBig5	F7B3
U+9C34.0	fGB	3-3A2C
U+9C34.0	fR/S	 195.11
U+9C34.0	fTotalStrokes	 22
U+9C34.0	fCangjie	HOUFK
U+9C34.0	fFourCorner	28240
U+9C34.0	fCantonese	fai1
U+9C34.0	fMandarin	hui1
U+9C35.0	fUTF8	鰵
U+9C35.0	fCNS11643	3-6025
U+9C35.0	fGB	1-772A
U+9C35.0	fR/S	 195.11
U+9C35.0	fTotalStrokes	 22
U+9C35.0	fCangjie	OKNWF
U+9C35.0	fFourCorner	88330
U+9C35.0	fHakka	 men3; men1; mien3
U+9C35.0	fCantonese	man5
U+9C35.0	fMandarin	min3
U+9C35.0	fEnglish	codfish
U+9C36.0	fUTF8	鰶
U+9C36.0	fCNS11643	2-6E63
U+9C36.0	fBig5	F7B6
U+9C36.0	fGB	3-7864
U+9C36.0	fR/S	 195.11
U+9C36.0	fTotalStrokes	 22
U+9C36.0	fCangjie	NFBOF
U+9C36.0	fFourCorner	27391
U+9C36.0	fCantonese	man5; zai3
U+9C36.0	fMandarin	ji4
U+9C37.0	fUTF8	鰷
U+9C37.0	fCNS11643	2-6E5F
U+9C37.0	fBig5	F7B2
U+9C37.0	fGB	1-7666
U+9C37.0	fR/S	 195.11
U+9C37.0	fTotalStrokes	 22
U+9C37.0	fCangjie	NFOLD
U+9C37.0	fFourCorner	27394
U+9C37.0	fHakka	 tiau2
U+9C37.0	fCantonese	tiu4
U+9C37.0	fMandarin	tiao2; xiao3
U+9C37.0	fEnglish	minnow
U+9C38.0	fUTF8	鰸
U+9C38.0	fCNS11643	3-5F7E
U+9C38.0	fGB	3-7744
U+9C38.0	fR/S	  195.11
U+9C38.0	fTotalStrokes	 22
U+9C38.0	fCangjie	NFSRR
U+9C38.0	fFourCorner	21310
U+9C38.0	fCantonese	keoi1
U+9C38.0	fMandarin	qu1
U+9C39.0	fUTF8	鰹
U+9C39.0	fCNS11643	2-6E5B
U+9C39.0	fBig5	F7AE
U+9C39.0	fGB	1-7664
U+9C39.0	fR/S	 195.11
U+9C39.0	fTotalStrokes	 22
U+9C39.0	fCangjie	NFSEG
U+9C39.0	fFourCorner	27314
U+9C39.0	fHakka	 gien1; kian1; gian1; gen1
U+9C39.0	fCantonese	gin1
U+9C39.0	fMandarin	jian1
U+9C39.0	fEnglish	 bonito
U+9C3A.0	fUTF8	鰺
U+9C3A.0	fCNS11643	3-6023
U+9C3A.0	fGB	3-7841
U+9C3A.0	fR/S	 195.11
U+9C3A.0	fTotalStrokes	 22
U+9C3A.0	fCangjie	NFIIH
U+9C3A.0	fFourCorner	23320
U+9C3A.0	fCantonese	sam1
U+9C3A.0	fMandarin	sao1; shen1; can1
U+9C3A.0	fEnglish	a horse mackerel
U+9C3B.0	fUTF8	鰻
U+9C3B.0	fCNS11643	1-7C2D
U+9C3B.0	fBig5	C5C1
U+9C3B.0	fGB	1-7729
U+9C3B.0	fR/S	 195.11
U+9C3B.0	fTotalStrokes	 22
U+9C3B.0	fCangjie	NFAWE
U+9C3B.0	fFourCorner	26347
U+9C3B.0	fHakka	 man2; man1; man3
U+9C3B.0	fCantonese	maan4 (also); maan6
U+9C3B.0	fMandarin	man2; wan4
U+9C3B.0	fEnglish	 eel
U+9C3C.0	fUTF8	鰼
U+9C3C.0	fCNS11643	2-6E5E
U+9C3C.0	fBig5	F7B1
U+9C3C.0	fGB	3-7868
U+9C3C.0	fR/S	 195.11
U+9C3C.0	fTotalStrokes	 22
U+9C3C.0	fCangjie	NFSMA
U+9C3C.0	fFourCorner	27362
U+9C3C.0	fCantonese	zaap6
U+9C3C.0	fMandarin	xi2
U+9C3C.0	fEnglish	loach; Misgurnus anguillicaudatu
U+9C3D.0	fUTF8	鰽
U+9C3D.0	fCNS11643	2-6E62
U+9C3D.0	fBig5	F7B5
U+9C3D.0	fGB	5-7677
U+9C3D.0	fR/S	 195.11
U+9C3D.0	fTotalStrokes	 22
U+9C3D.0	fCangjie	NFTWA
U+9C3D.0	fFourCorner	24366
U+9C3D.0	fHakka	 siu2
U+9C3D.0	fCantonese	cau4; cou4
U+9C3D.0	fMandarin	qiu2
U+9C3D.0	fEnglish	herring
U+9C3E.0	fUTF8	鰾
U+9C3E.0	fCNS11643	1-7C2C
U+9C3E.0	fBig5	C5C0
U+9C3E.0	fGB	1-7727
U+9C3E.0	fR/S	 195.11
U+9C3E.0	fTotalStrokes	 22
U+9C3E.0	fCangjie	NFMWF
U+9C3E.0	fFourCorner	21391
U+9C3E.0	fHakka	  piau5; piau3; peu5
U+9C3E.0	fCantonese	piu5; biu1
U+9C3E.0	fMandarin	biao4; biao3
U+9C3E.0	fEnglish	[1] maw of a fish; air bladder; swimming bladder [2] fish glue
U+9C3F.0	fUTF8	鰿
U+9C3F.0	fCNS11643	2-6E59
U+9C3F.0	fBig5	F7AC
U+9C3F.0	fGB	3-7830
U+9C3F.0	fR/S	 195.11
U+9C3F.0	fTotalStrokes	 22
U+9C3F.0	fCangjie	NFQMC
U+9C3F.0	fFourCorner	25386
U+9C3F.0	fCantonese	zik1
U+9C3F.0	fMandarin	ji1; yi2; ji2; ze2
U+9C40.0	fUTF8	鱀
U+9C40.0	fCNS11643	2-6A7E
U+9C40.0	fBig5	F570
U+9C40.0	fGB	3-7856
U+9C40.0	fR/S	 195.11
U+9C40.0	fTotalStrokes	 22
U+9C40.0	fCangjie	AUNWF
U+9C40.0	fFourCorner	71336
U+9C40.0	fCantonese	kei3; gei6
U+9C40.0	fMandarin	ji4; ji1
U+9C41.0	fUTF8	鱁
U+9C41.0	fCNS11643	2-6E5D
U+9C41.0	fBig5	F7B0
U+9C41.0	fGB	3-785A
U+9C41.0	fR/S	 195.11
U+9C41.0	fTotalStrokes	 22
U+9C41.0	fCangjie	NFYMO
U+9C41.0	fFourCorner	21333
U+9C41.0	fCantonese	zuk6
U+9C41.0	fMandarin	zhu2
U+9C42.0	fUTF8	鱂
U+9C42.0	fCNS11643	4-6A23
U+9C42.0	fGB	3-7852
U+9C42.0	fR/S	 195.11
U+9C42.0	fTotalStrokes	 22; 21
U+9C42.0	fCangjie	NFVMI
U+9C42.0	fFourCorner	27342
U+9C42.0	fCantonese	zoeng1
U+9C42.0	fMandarin	jiang1
U+9C42.0	fEnglish	a fish name
U+9C43.0	fUTF8	鱃
U+9C43.0	fCNS11643	5-752D
U+9C43.0	fGB	3-7860
U+9C43.0	fR/S	  195.11
U+9C43.0	fTotalStrokes	 22
U+9C43.0	fCangjie	NFTQG
U+9C43.0	fFourCorner	28310
U+9C43.0	fCantonese	sau1
U+9C43.0	fMandarin	qiu1
U+9C44.0	fUTF8	鱄
U+9C44.0	fCNS11643	2-6E5A
U+9C44.0	fBig5	F7AD
U+9C44.0	fGB	3-7742
U+9C44.0	fR/S	 195.11
U+9C44.0	fTotalStrokes	 22
U+9C44.0	fCangjie	NFJII
U+9C44.0	fFourCorner	25343
U+9C44.0	fHakka	 zon1; ton2; zhon1
U+9C44.0	fCantonese	zyun1; tyun4
U+9C44.0	fMandarin	zhuan1; tuan2; lian4; shan4; zhuan3
U+9C44.0	fEnglish	anchovy
U+9C45.0	fUTF8	鱅
U+9C45.0	fCNS11643	3-6022
U+9C45.0	fGB	1-772B
U+9C45.0	fR/S	 195.11
U+9C45.0	fTotalStrokes	 22
U+9C45.0	fCangjie	NFILB
U+9C45.0	fFourCorner	20320
U+9C45.0	fHakka	 sung2; jung2; rung2; siung3; hiung2
U+9C45.0	fCantonese	jung4; sung4 (also); zung4
U+9C45.0	fMandarin	yong2; yong1; obs
U+9C45.0	fEnglish	 bighead
U+9C46.0	fUTF8	鱆
U+9C46.0	fCNS11643	2-6E57
U+9C46.0	fBig5	F7AA
U+9C46.0	fGB	5-7679
U+9C46.0	fR/S	 195.11
U+9C46.0	fTotalStrokes	 22
U+9C46.0	fCangjie	NFYTJ
U+9C46.0	fFourCorner	20346
U+9C46.0	fCantonese	zoeng1; zeon1
U+9C46.0	fMandarin	zhang1; zhang4
U+9C46.0	fEnglish	an octopus, devilfish
U+9C47.0	fUTF8	鱇
U+9C47.0	fCNS11643	F-6A3E
U+9C47.0	fGB	3-7865
U+9C47.0	fR/S	 195.11
U+9C47.0	fTotalStrokes	 22
U+9C47.0	fCangjie	NFILE
U+9C47.0	fFourCorner	2033
U+9C47.0	fCantonese	hong1
U+9C47.0	fMandarin	kang1
U+9C47.0	fEnglish	anglerfish
U+9C48.0	fUTF8	鱈
U+9C48.0	fCNS11643	2-6E58
U+9C48.0	fBig5	F7AB
U+9C48.0	fGB	1-7728
U+9C48.0	fR/S	 195.11
U+9C48.0	fTotalStrokes	 22
U+9C48.0	fCangjie	NFMBM
U+9C48.0	fFourCorner	21374
U+9C48.0	fHakka	 set7
U+9C48.0	fCantonese	syut3
U+9C48.0	fMandarin	xue3
U+9C48.0	fEnglish	 codfish; cod
U+9C49.0	fUTF8	鱉
U+9C49.0	fCNS11643	1-7C2A
U+9C49.0	fBig5	C5BE
U+9C49.0	fGB	1-316E
U+9C49.0	fR/S	 195.12
U+9C49.0	fTotalStrokes	 23
U+9C49.0	fCangjie	FKNWF
U+9C49.0	fFourCorner	97336
U+9C49.0	fHakka	  biet7; bet7; piat7
U+9C49.0	fCantonese	bit3
U+9C49.0	fMandarin	bie1
U+9C49.0	fEnglish	turtle
U+9C4A.0	fUTF8	鱊
U+9C4A.0	fCNS11643	2-6F68
U+9C4A.0	fBig5	F85A
U+9C4A.0	fGB	3-786F
U+9C4A.0	fR/S	 195.12
U+9C4A.0	fTotalStrokes	 23
U+9C4A.0	fCangjie	NFNHB
U+9C4A.0	fFourCorner	27327
U+9C4A.0	fHakka	 lut8; rut8; jut8
U+9C4A.0	fCantonese	wat6; leot6
U+9C4A.0	fMandarin	jue2; yu4; shu4
U+9C4B.0	fUTF8	鱋
U+9C4B.0	fCNS11643	2-6F6A
U+9C4B.0	fBig5	F85C
U+9C4B.0	fGB	E-4744
U+9C4B.0	fR/S	 195.12
U+9C4B.0	fTotalStrokes	 23
U+9C4B.0	fCangjie	NFYPM
U+9C4B.0	fFourCorner	21317
U+9C4B.0	fCantonese	keoi1
U+9C4B.0	fMandarin	qu1; ta4; xie4
U+9C4C.0	fUTF8	鱌
U+9C4C.0	fCNS11643	2-6F6D
U+9C4C.0	fBig5	F85F
U+9C4C.0	fGB	E-4745
U+9C4C.0	fR/S	 195.12
U+9C4C.0	fTotalStrokes	 23
U+9C4C.0	fCangjie	NFNAO
U+9C4C.0	fFourCorner	27332
U+9C4C.0	fCantonese	zoeng6
U+9C4C.0	fMandarin	xiang4
U+9C4D.0	fUTF8	鱍
U+9C4D.0	fCNS11643	2-6F69
U+9C4D.0	fBig5	F85B
U+9C4D.0	fGB	3-7766
U+9C4D.0	fR/S	 195.12
U+9C4D.0	fTotalStrokes	 23
U+9C4D.0	fCangjie	NFNOE
U+9C4D.0	fFourCorner	22347
U+9C4D.0	fHakka	  pat7
U+9C4D.0	fCantonese	but6; but3 (also)
U+9C4D.0	fMandarin	bo1; po1
U+9C4D.0	fEnglish	fish moveing its tail in swimming
U+9C4E.0	fUTF8	鱎
U+9C4E.0	fCNS11643	2-6F6E
U+9C4E.0	fBig5	F860
U+9C4E.0	fGB	3-7776
U+9C4E.0	fR/S	 195.12
U+9C4E.0	fTotalStrokes	 23
U+9C4E.0	fCangjie	NFHKB
U+9C4E.0	fFourCorner	22327
U+9C4E.0	fHakka	 giau3; kieu3
U+9C4E.0	fCantonese	giu2
U+9C4E.0	fMandarin	jiao3; jiao1
U+9C4E.0	fEnglish	fish of the genus Culter - peculiar to China
U+9C4F.0	fUTF8	鱏
U+9C4F.0	fCNS11643	3-6065
U+9C4F.0	fGB	3-786E
U+9C4F.0	fR/S	 195.12
U+9C4F.0	fTotalStrokes	 23
U+9C4F.0	fCangjie	NFMWJ
U+9C4F.0	fFourCorner	21340
U+9C4F.0	fHakka	 cim2; tam2
U+9C4F.0	fCantonese	cam4
U+9C4F.0	fMandarin	xun2
U+9C4F.0	fEnglish	the snouted sturgeon
U+9C50.0	fUTF8	鱐
U+9C50.0	fCNS11643	2-6F67
U+9C50.0	fBig5	F859
U+9C50.0	fGB	3-783D
U+9C50.0	fR/S	 195.12
U+9C50.0	fTotalStrokes	 23
U+9C50.0	fCangjie	NFLX
U+9C50.0	fFourCorner	25327
U+9C50.0	fCantonese	suk1; sau1
U+9C50.0	fMandarin	su4; xiu1
U+9C50.0	fEnglish	dried fish
U+9C51.0	fUTF8	鱑
U+9C51.0	fCNS11643	3-6066
U+9C51.0	fGB	5-7676
U+9C51.0	fR/S	 195.12
U+9C51.0	fTotalStrokes	 23
U+9C51.0	fCangjie	NFTLC
U+9C51.0	fFourCorner	24380
U+9C51.0	fHakka	 wong2; vong2
U+9C51.0	fCantonese	wong4
U+9C51.0	fMandarin	huang2
U+9C51.0	fEnglish	the sturgeon
U+9C52.0	fUTF8	鱒
U+9C52.0	fCNS11643	2-6F65
U+9C52.0	fBig5	F857
U+9C52.0	fGB	1-772E
U+9C52.0	fR/S	 195.12
U+9C52.0	fTotalStrokes	 23
U+9C52.0	fCangjie	NFTWI
U+9C52.0	fFourCorner	28346
U+9C52.0	fHakka	 zun3; zun1
U+9C52.0	fCantonese	zyun1; zyun3 (also); zeon1; zyun6; cyun5
U+9C52.0	fMandarin	zun1; zun4; obs; cun2
U+9C52.0	fEnglish	barbel
U+9C53.0	fUTF8	鱓
U+9C53.0	fCNS11643	3-6068
U+9C53.0	fGB	E-4746
U+9C53.0	fR/S	 195.12
U+9C53.0	fTotalStrokes	 23
U+9C53.0	fCangjie	NFRRJ
U+9C53.0	fFourCorner	26350
U+9C53.0	fHakka	 shan1; san1; shan3; sen1; san3; sen3; shen5
U+9C53.0	fCantonese	to4; sin5
U+9C53.0	fMandarin	shan4; tuo2
U+9C53.0	fEnglish	eel
U+9C54.0	fUTF8	鱔
U+9C54.0	fCNS11643	1-7C57
U+9C54.0	fBig5	C5EB
U+9C54.0	fGB	1-772D
U+9C54.0	fR/S	 195.12
U+9C54.0	fTotalStrokes	 23
U+9C54.0	fCangjie	NFTGR
U+9C54.0	fFourCorner	28361
U+9C54.0	fHakka	 shan1; san1; sien1; shan3; sen1; san3; sen3; shen1; shen3
U+9C54.0	fCantonese	sin5; sin2 (also)
U+9C54.0	fMandarin	shan4
U+9C54.0	fEnglish	 eel; moray eel
U+9C55.0	fUTF8	鱕
U+9C55.0	fCNS11643	2-6F6B
U+9C55.0	fBig5	F85D
U+9C55.0	fGB	E-4747
U+9C55.0	fR/S	 195.12
U+9C55.0	fTotalStrokes	 23
U+9C55.0	fCangjie	NFHDW
U+9C55.0	fFourCorner	22369
U+9C55.0	fCantonese	faan1
U+9C55.0	fMandarin	fan1; fan2
U+9C56.0	fUTF8	鱖
U+9C56.0	fCNS11643	1-7C59
U+9C56.0	fBig5	C5ED
U+9C56.0	fGB	1-772C
U+9C56.0	fR/S	 195.12
U+9C56.0	fTotalStrokes	 23
U+9C56.0	fCangjie	NFMTO
U+9C56.0	fFourCorner	21382
U+9C56.0	fHakka	 kiet7; kui3; ket7; gui3; ke3; k'wui3
U+9C56.0	fCantonese	gwai3; kyut3
U+9C56.0	fMandarin	jue2; gui4; gui1; qiu1; wan3
U+9C56.0	fEnglish	 Mandarin fish
U+9C57.0	fUTF8	鱗
U+9C57.0	fCNS11643	1-7C58
U+9C57.0	fBig5	C5EC
U+9C57.0	fGB	1-415B
U+9C57.0	fR/S	 195.12
U+9C57.0	fTotalStrokes	 23
U+9C57.0	fCangjie	NFFDQ
U+9C57.0	fFourCorner	29359
U+9C57.0	fHakka	  lin2;  lin3; lin1
U+9C57.0	fCantonese	leon4
U+9C57.0	fMandarin	lin2
U+9C57.0	fEnglish	 fish scales
U+9C58.0	fUTF8	鱘
U+9C58.0	fCNS11643	2-6F66
U+9C58.0	fBig5	F858
U+9C58.0	fGB	1-7660
U+9C58.0	fR/S	 195.12
U+9C58.0	fTotalStrokes	 23
U+9C58.0	fCangjie	NFSMI
U+9C58.0	fFourCorner	27346
U+9C58.0	fHakka	 cim2
U+9C58.0	fCantonese	cam4
U+9C58.0	fMandarin	xun2; xin2
U+9C58.0	fEnglish	 sturgeon
U+9C59.0	fUTF8	鱙
U+9C59.0	fCNS11643	2-6F6C
U+9C59.0	fBig5	F85E
U+9C59.0	fGB	E-4748
U+9C59.0	fR/S	 195.12
U+9C59.0	fTotalStrokes	 23
U+9C59.0	fCangjie	NFGGU
U+9C59.0	fFourCorner	24311
U+9C59.0	fCantonese	jiu4; miu4
U+9C59.0	fMandarin	miao2
U+9C5A.0	fUTF8	鱚
U+9C5A.0	fGB	3-786C
U+9C5A.0	fR/S	 195.12
U+9C5A.0	fTotalStrokes	 23
U+9C5A.0	fCangjie	NFGRR
U+9C5A.0	fCantonese	hei1
U+9C5A.0	fMandarin	xi3
U+9C5B.0	fUTF8	鱛
U+9C5B.0	fGB	E-4749
U+9C5B.0	fR/S	 195.12
U+9C5B.0	fTotalStrokes	 23
U+9C5B.0	fCangjie	NFCWA
U+9C5B.0	fCantonese	zang1
U+9C5B.0	fEnglish	 fish name
U+9C5C.0	fUTF8	鱜
U+9C5C.0	fGB	E-474A
U+9C5C.0	fR/S	 195.13
U+9C5C.0	fTotalStrokes	 24
U+9C5C.0	fCangjie	NFVHL
U+9C5C.0	fCantonese	hoeng1
U+9C5C.0	fMandarin	xiang1
U+9C5D.0	fUTF8	鱝
U+9C5D.0	fCNS11643	3-6132
U+9C5D.0	fGB	1-7677
U+9C5D.0	fR/S	 195.13; 195.12
U+9C5D.0	fTotalStrokes	 24; 23
U+9C5D.0	fCangjie	NFJTC
U+9C5D.0	fFourCorner	24380
U+9C5D.0	fHakka	 fun5
U+9C5D.0	fCantonese	fan5
U+9C5D.0	fMandarin	fen4
U+9C5D.0	fEnglish	skate (fish)
U+9C5E.0	fUTF8	鱞
U+9C5E.0	fCNS11643	2-7068
U+9C5E.0	fBig5	F8DA
U+9C5E.0	fGB	5-767E
U+9C5E.0	fR/S	 195.13
U+9C5E.0	fTotalStrokes	 24
U+9C5E.0	fCangjie	NFWLV
U+9C5E.0	fFourCorner	26322
U+9C5E.0	fCantonese	gwaan1; gwan1
U+9C5E.0	fMandarin	guan1
U+9C5F.0	fUTF8	鱟
U+9C5F.0	fCNS11643	1-7C78
U+9C5F.0	fBig5	C64D
U+9C5F.0	fGB	1-7657
U+9C5F.0	fR/S	 195.13
U+9C5F.0	fTotalStrokes	 24
U+9C5F.0	fCangjie	HBNWF
U+9C5F.0	fFourCorner	77336
U+9C5F.0	fHakka	 heu5; heu6; hiu3
U+9C5F.0	fCantonese	hau6
U+9C5F.0	fMandarin	hou4; wo4; wu4
U+9C5F.0	fEnglish	 king crab
U+9C60.0	fUTF8	鱠
U+9C60.0	fCNS11643	2-7069
U+9C60.0	fBig5	F8DB
U+9C60.0	fGB	3-7779
U+9C60.0	fR/S	 195.13
U+9C60.0	fTotalStrokes	 24
U+9C60.0	fCangjie	NFOMA
U+9C60.0	fFourCorner	28366
U+9C60.0	fHakka	  kwai5
U+9C60.0	fCantonese	kui2; kwui2 (also); kui3
U+9C60.0	fMandarin	kuai4
U+9C60.0	fEnglish	 minced fish hash
U+9C61.0	fUTF8	鱡
U+9C61.0	fCNS11643	3-6133
U+9C61.0	fGB	5-7673
U+9C61.0	fR/S	 195.13
U+9C61.0	fTotalStrokes	 24
U+9C61.0	fCangjie	NFBCJ
U+9C61.0	fFourCorner	23350
U+9C61.0	fHakka	 cet7
U+9C61.0	fCantonese	caak6
U+9C61.0	fMandarin	zei2
U+9C61.0	fEnglish	the cuttle-fish
U+9C62.0	fUTF8	鱢
U+9C62.0	fCNS11643	2-7067
U+9C62.0	fBig5	F8D9
U+9C62.0	fGB	5-767D
U+9C62.0	fR/S	 195.13
U+9C62.0	fTotalStrokes	 24
U+9C62.0	fCangjie	NFRRD
U+9C62.0	fFourCorner	26394
U+9C62.0	fCantonese	sou1
U+9C62.0	fMandarin	sao1
U+9C63.0	fUTF8	鱣
U+9C63.0	fCNS11643	2-7064
U+9C63.0	fBig5	F8D6
U+9C63.0	fGB	3-7873
U+9C63.0	fR/S	 195.13
U+9C63.0	fTotalStrokes	 24
U+9C63.0	fCangjie	NFYWM
U+9C63.0	fFourCorner	20316
U+9C63.0	fHakka	 zhan1; zan1; zen1; zhen1
U+9C63.0	fCantonese	zin1; sin5
U+9C63.0	fMandarin	zhan1; tan2
U+9C63.0	fEnglish	a kind of sturgeon (Acipenser mikadoi)
U+9C63.1	fUTF8	鱣
U+9C63.1	fCNS11643	2-7064
U+9C63.1	fBig5	F8D6
U+9C63.1	fGB	3-7873
U+9C63.1	fR/S	 195.13
U+9C63.1	fTotalStrokes	 24
U+9C63.1	fCangjie	NFYWM
U+9C63.1	fFourCorner	20316
U+9C63.1	fMandarin	shan4
U+9C63.1	fEnglish	 eel
U+9C64.0	fUTF8	鱤
U+9C64.0	fCNS11643	4-6C4A
U+9C64.0	fGB	3-7872
U+9C64.0	fR/S	 195.13
U+9C64.0	fTotalStrokes	 24
U+9C64.0	fCangjie	NFIRP
U+9C64.0	fFourCorner	23350
U+9C64.0	fHakka	 gam3
U+9C64.0	fCantonese	gam2
U+9C64.0	fMandarin	gan3
U+9C64.0	fEnglish	a kind of shad (fish)
U+9C65.0	fUTF8	鱥
U+9C65.0	fCNS11643	3-6134
U+9C65.0	fGB	3-7772
U+9C65.0	fR/S	 195.13
U+9C65.0	fTotalStrokes	 24
U+9C65.0	fCangjie	NFYMH
U+9C65.0	fFourCorner	21350
U+9C65.0	fCantonese	gwai3
U+9C65.0	fMandarin	gui4
U+9C66.0	fUTF8	鱦
U+9C66.0	fCNS11643	2-7066
U+9C66.0	fBig5	F8D8
U+9C66.0	fGB	5-7665
U+9C66.0	fR/S	 195.13
U+9C66.0	fTotalStrokes	 24
U+9C66.0	fCangjie	NFRXU
U+9C66.0	fFourCorner	27317
U+9C66.0	fCantonese	jing6; maang5; sing4; sing6
U+9C66.0	fMandarin	sheng2
U+9C67.0	fUTF8	鱧
U+9C67.0	fCNS11643	2-7065
U+9C67.0	fBig5	F8D7
U+9C67.0	fGB	1-772F
U+9C67.0	fR/S	 195.13
U+9C67.0	fTotalStrokes	 24
U+9C67.0	fCangjie	NFTWT
U+9C67.0	fFourCorner	25318
U+9C67.0	fHakka	  li1;  li3
U+9C67.0	fCantonese	lai5
U+9C67.0	fMandarin	li3
U+9C67.0	fEnglish	snakehead
U+9C68.0	fUTF8	鱨
U+9C68.0	fCNS11643	2-7149
U+9C68.0	fBig5	F95A
U+9C68.0	fGB	3-7848
U+9C68.0	fR/S	 195.14
U+9C68.0	fTotalStrokes	 25
U+9C68.0	fCangjie	NFFBA
U+9C68.0	fFourCorner	29361
U+9C68.0	fHakka	 shong2; song2
U+9C68.0	fCantonese	soeng4
U+9C68.0	fMandarin	chang2
U+9C68.0	fEnglish	codfish
U+9C69.0	fUTF8	鱩
U+9C69.0	fGB	E-474B
U+9C69.0	fR/S	 195.13
U+9C69.0	fTotalStrokes	 24
U+9C69.0	fCangjie	NFMBW
U+9C69.0	fCantonese	leoi4
U+9C6A.0	fUTF8	鱪
U+9C6A.0	fGB	E-474C
U+9C6A.0	fR/S	 195.13
U+9C6A.0	fTotalStrokes	 24
U+9C6A.0	fCangjie	NFAJA
U+9C6A.0	fCantonese	syu2
U+9C6B.0	fUTF8	鱫
U+9C6B.0	fGB	E-474D
U+9C6B.0	fR/S	 195.13
U+9C6B.0	fTotalStrokes	 24
U+9C6B.0	fCangjie	NFBBE
U+9C6B.0	fCantonese	oi3; ngoi3 (also)
U+9C6B.0	fMandarin	ai4
U+9C6C.0	fUTF8	鱬
U+9C6C.0	fCNS11643	3-6159
U+9C6C.0	fGB	3-7875
U+9C6C.0	fR/S	 195.14
U+9C6C.0	fTotalStrokes	 25
U+9C6C.0	fCangjie	NFMBB
U+9C6C.0	fFourCorner	21320
U+9C6C.0	fCantonese	jyu4
U+9C6C.0	fMandarin	ru2
U+9C6D.0	fUTF8	鱭
U+9C6D.0	fCNS11643	2-714B
U+9C6D.0	fBig5	F95C
U+9C6D.0	fGB	1-765D
U+9C6D.0	fR/S	 195.14
U+9C6D.0	fTotalStrokes	 25
U+9C6D.0	fCangjie	NFYX
U+9C6D.0	fFourCorner	88323
U+9C6D.0	fHakka	 ci3; ci5; zii5
U+9C6D.0	fCantonese	cai5; cai4 (also); ci5
U+9C6D.0	fMandarin	ji4; qi2
U+9C6D.0	fEnglish	anchovy
U+9C6E.0	fUTF8	鱮
U+9C6E.0	fCNS11643	2-714A
U+9C6E.0	fBig5	F95B
U+9C6E.0	fGB	3-773D
U+9C6E.0	fR/S	 195.14
U+9C6E.0	fTotalStrokes	 25
U+9C6E.0	fCangjie	NFHXC
U+9C6E.0	fFourCorner	27381
U+9C6E.0	fHakka	  si5; si3
U+9C6E.0	fCantonese	zeoi6; zoei6
U+9C6E.0	fMandarin	xu4; yu2
U+9C6E.0	fEnglish	bream
U+9C6F.0	fUTF8	鱯
U+9C6F.0	fCNS11643	3-615B
U+9C6F.0	fGB	3-7870
U+9C6F.0	fR/S	 195.14
U+9C6F.0	fTotalStrokes	 25
U+9C6F.0	fCangjie	NFTOE
U+9C6F.0	fFourCorner	24340
U+9C6F.0	fHakka	 fa5; fa6
U+9C6F.0	fCantonese	wu6
U+9C6F.0	fMandarin	huo4; hu1; hua2; hu4
U+9C6F.0	fEnglish	a fish of the shad kind, with a white belly
U+9C70.0	fUTF8	鱰
U+9C70.0	fGB	E-474E
U+9C70.0	fR/S	 195.14
U+9C70.0	fTotalStrokes	 25
U+9C70.0	fCangjie	NFWLA
U+9C70.0	fCantonese	cyu5
U+9C71.0	fUTF8	鱱
U+9C71.0	fCNS11643	2-7169
U+9C71.0	fBig5	F979
U+9C71.0	fGB	E-474F
U+9C71.0	fR/S	 195.15
U+9C71.0	fTotalStrokes	 26
U+9C71.0	fCangjie	NFMTB
U+9C71.0	fFourCorner	21327
U+9C71.0	fCantonese	lai6
U+9C71.0	fMandarin	li4
U+9C72.0	fUTF8	鱲
U+9C72.0	fCNS11643	5-7A53
U+9C72.0	fGB	3-7879
U+9C72.0	fR/S	 195.15
U+9C72.0	fTotalStrokes	 26
U+9C72.0	fCangjie	NFVVV
U+9C72.0	fFourCorner	22310
U+9C72.0	fHakka	 liap8
U+9C72.0	fCantonese	lip6; lap6; laap6
U+9C72.0	fMandarin	lie4
U+9C72.0	fEnglish	fishes with bristly dorsal fins
U+9C73.0	fUTF8	鱳
U+9C73.0	fCNS11643	2-7168
U+9C73.0	fBig5	F978
U+9C73.0	fGB	3-7761
U+9C73.0	fR/S	 195.15
U+9C73.0	fTotalStrokes	 26
U+9C73.0	fCangjie	NFVID
U+9C73.0	fFourCorner	22394
U+9C73.0	fCantonese	lik6; lok6; luk6
U+9C73.0	fMandarin	li4; luo4; lu4
U+9C74.0	fUTF8	鱴
U+9C74.0	fCNS11643	2-7167
U+9C74.0	fBig5	F977
U+9C74.0	fGB	3-7874
U+9C74.0	fR/S	 195.15
U+9C74.0	fTotalStrokes	 26
U+9C74.0	fCangjie	NFTWI
U+9C74.0	fFourCorner	24343
U+9C74.0	fCantonese	mit6
U+9C74.0	fMandarin	mie4
U+9C75.0	fUTF8	鱵
U+9C75.0	fCNS11643	2-716A
U+9C75.0	fBig5	F97A
U+9C75.0	fGB	3-7878
U+9C75.0	fR/S	 195.15
U+9C75.0	fTotalStrokes	 26
U+9C75.0	fCangjie	NFHIR
U+9C75.0	fFourCorner	28353
U+9C75.0	fHakka	 zhim1; zihm1
U+9C75.0	fCantonese	zam1
U+9C75.0	fMandarin	zhen1
U+9C75.0	fEnglish	 saury
U+9C76.0	fUTF8	鱶
U+9C76.0	fCNS11643	3-6175
U+9C76.0	fGB	E-4750
U+9C76.0	fR/S	 195.15
U+9C76.0	fTotalStrokes	 26
U+9C76.0	fCangjie	NFTOV
U+9C76.0	fFourCorner	28330
U+9C76.0	fHakka	 rong1; jong1; siong3
U+9C76.0	fCantonese	soeng2
U+9C76.0	fMandarin	xiang3
U+9C76.0	fEnglish	a shark
U+9C77.0	fUTF8	鱷
U+9C77.0	fCNS11643	1-7D40
U+9C77.0	fBig5	C673
U+9C77.0	fGB	E-4751
U+9C77.0	fR/S	 195.16
U+9C77.0	fTotalStrokes	 27
U+9C77.0	fCangjie	NFMGR
U+9C77.0	fFourCorner	26327
U+9C77.0	fHakka	 ngok8
U+9C77.0	fCantonese	ngok6; ok6 (also)
U+9C77.0	fMandarin	e4
U+9C77.0	fEnglish	 crocodile; alligator
U+9C78.0	fUTF8	鱸
U+9C78.0	fCNS11643	1-7D41
U+9C78.0	fBig5	C674
U+9C78.0	fGB	1-7654
U+9C78.0	fR/S	 195.16
U+9C78.0	fTotalStrokes	 27
U+9C78.0	fCangjie	NFYPT
U+9C78.0	fFourCorner	21317
U+9C78.0	fHakka	  lu2; lu3
U+9C78.0	fCantonese	lou4
U+9C78.0	fMandarin	lu2
U+9C78.0	fEnglish	 perch; bass
U+9C79.0	fUTF8	鱹
U+9C79.0	fCNS11643	2-7238
U+9C79.0	fBig5	F9CA
U+9C79.0	fGB	E-4752
U+9C79.0	fR/S	 195.18
U+9C79.0	fTotalStrokes	 29
U+9C79.0	fCangjie	NFTRG
U+9C79.0	fFourCorner	24314
U+9C79.0	fCantonese	gun3; gwun3 (also)
U+9C79.0	fMandarin	guan4; jiu1
U+9C7A.0	fUTF8	鱺
U+9C7A.0	fCNS11643	2-723C
U+9C7A.0	fBig5	F9CE
U+9C7A.0	fGB	1-7662
U+9C7A.0	fR/S	 195.19
U+9C7A.0	fTotalStrokes	 30
U+9C7A.0	fCangjie	NFMMP
U+9C7A.0	fFourCorner	21311
U+9C7A.0	fHakka	 li2
U+9C7A.0	fCantonese	lei4
U+9C7A.0	fMandarin	li2; li3
U+9C7A.0	fEnglish	eel
U+9C7B.0	fUTF8	鱻
U+9C7B.0	fCNS11643	3-6241
U+9C7B.0	fGB	E-4753
U+9C7B.0	fR/S	 195.22
U+9C7B.0	fTotalStrokes	 33
U+9C7B.0	fCangjie	NFNFF
U+9C7B.0	fFourCorner	27330
U+9C7B.0	fHakka	  sien1; sen1
U+9C7B.0	fCantonese	sin1
U+9C7B.0	fMandarin	xian1; xian3
U+9C7B.0	fEnglish	fresh; tasty
U+9C7C.0	fUTF8	鱼
U+9C7C.0	fGB	0-5363
U+9C7C.0	fR/S	 195.0
U+9C7C.0	fTotalStrokes	 8
U+9C7C.0	fCangjie	NWM
U+9C7C.0	fHakka	 ng2
U+9C7C.0	fCantonese	jyu4; jyu2 (also)
U+9C7C.0	fMandarin	yu2
U+9C7C.0	fEnglish	simplified form of &#x9B5A;
U+9C7D.0	fUTF8	鱽
U+9C7D.0	fGB	7-2136
U+9C7D.0	fR/S	 195.2
U+9C7D.0	fTotalStrokes	 10
U+9C7D.0	fCangjie	NMSH
U+9C7D.0	fCantonese	dou1
U+9C7D.0	fMandarin	dao1
U+9C7D.0	fEnglish	the mullet
U+9C7E.0	fUTF8	鱾
U+9C7E.0	fGB	7-2137
U+9C7E.0	fR/S	 195.3
U+9C7E.0	fTotalStrokes	 11
U+9C7E.0	fCangjie	XNMSU
U+9C7E.0	fCantonese	gei2
U+9C7E.0	fMandarin	ji3
U+9C7E.0	fEnglish	name of fish
U+9C7F.0	fUTF8	鱿
U+9C7F.0	fGB	0-764F
U+9C7F.0	fR/S	 195.4
U+9C7F.0	fTotalStrokes	 12
U+9C7F.0	fCangjie	NMIKU
U+9C7F.0	fHakka	 ju2
U+9C7F.0	fCantonese	jau4; jau2 (also)
U+9C7F.0	fMandarin	you2
U+9C7F.0	fEnglish	simplified form of &#x9B77;
U+9C80.0	fUTF8	鲀
U+9C80.0	fGB	8-2D2F
U+9C80.0	fR/S	 195.4
U+9C80.0	fTotalStrokes	 12
U+9C80.0	fCangjie	NMPU
U+9C80.0	fCantonese	tyun4
U+9C80.0	fMandarin	tun2
U+9C80.0	fEnglish	blowfish, globefish, Spheroides vermicular
U+9C81.0	fUTF8	鲁
U+9C81.0	fGB	0-4233
U+9C81.0	fR/S	 195.4
U+9C81.0	fTotalStrokes	 12
U+9C81.0	fCangjie	NWMA
U+9C81.0	fHakka	 lu3; lu1
U+9C81.0	fCantonese	lou5
U+9C81.0	fMandarin	lu3
U+9C81.0	fEnglish	simplified form of &#x9B6F;
U+9C82.0	fUTF8	鲂
U+9C82.0	fGB	0-7650
U+9C82.0	fR/S	 195.4
U+9C82.0	fTotalStrokes	 12
U+9C82.0	fCangjie	NMYHS
U+9C82.0	fHakka	 fong2
U+9C82.0	fCantonese	fong4
U+9C82.0	fMandarin	fang2
U+9C82.0	fEnglish	simplified form of &#x9B74;
U+9C83.0	fUTF8	鲃
U+9C83.0	fGB	7-2138
U+9C83.0	fR/S	 195.4
U+9C83.0	fTotalStrokes	 12
U+9C83.0	fCangjie	NMAU
U+9C83.0	fCantonese	baa1; bat6
U+9C83.0	fMandarin	ba1; ba4
U+9C83.0	fEnglish	[1] a kind of fish; a bonito [2] same as &#x9B81;
U+9C84.0	fUTF8	鲄
U+9C84.0	fGB	8-2D30
U+9C84.0	fR/S	 195.5
U+9C84.0	fTotalStrokes	 13
U+9C84.0	fCangjie	NMMNR
U+9C84.0	fCantonese	ho4
U+9C84.0	fMandarin	he2; ge3
U+9C85.0	fUTF8	鲅
U+9C85.0	fGB	0-7651
U+9C85.0	fR/S	 195.5
U+9C85.0	fTotalStrokes	 13
U+9C85.0	fCangjie	NMIKE
U+9C85.0	fCantonese	bat6; bat3 (also); but3
U+9C85.0	fMandarin	bo1; bo4; ba2; ba4; fei4
U+9C85.0	fEnglish	simplified form of &#x9B81;
U+9C86.0	fUTF8	鲆
U+9C86.0	fGB	0-7652
U+9C86.0	fR/S	 195.5
U+9C86.0	fTotalStrokes	 13
U+9C86.0	fCangjie	NMMFJ
U+9C86.0	fCantonese	ping4
U+9C86.0	fMandarin	ping2
U+9C86.0	fEnglish	simplified form of &#x9B83;
U+9C87.0	fUTF8	鲇
U+9C87.0	fGB	0-7653
U+9C87.0	fR/S	 195.5
U+9C87.0	fTotalStrokes	 13
U+9C87.0	fCangjie	NMYR
U+9C87.0	fHakka	 ngiam2
U+9C87.0	fCantonese	nim4; lim4 (also)
U+9C87.0	fMandarin	nian2
U+9C87.0	fEnglish	simplified form of &#x9B8E;
U+9C88.0	fUTF8	鲈
U+9C88.0	fGB	0-7654
U+9C88.0	fR/S	 195.5
U+9C88.0	fTotalStrokes	 13
U+9C88.0	fCangjie	NMYS
U+9C88.0	fHakka	 lu2
U+9C88.0	fCantonese	lou4
U+9C88.0	fMandarin	lu2
U+9C88.0	fEnglish	simplified form of &#x9C78;
U+9C89.0	fUTF8	鲉
U+9C89.0	fGB	7-2139
U+9C89.0	fR/S	 195.5
U+9C89.0	fTotalStrokes	 13
U+9C89.0	fCangjie	NMLW
U+9C89.0	fCantonese	jau4
U+9C89.0	fMandarin	you2
U+9C8A.0	fUTF8	鲊
U+9C8A.0	fGB	8-2D31
U+9C8A.0	fR/S	 195.5
U+9C8A.0	fTotalStrokes	 13
U+9C8A.0	fCangjie	NMOS
U+9C8A.0	fCantonese	zaa2; zaa3
U+9C8A.0	fMandarin	zha3; zha4
U+9C8A.0	fEnglish	minced and salted fish; to preserve
U+9C8B.0	fUTF8	鲋
U+9C8B.0	fGB	0-7656
U+9C8B.0	fR/S	 195.5
U+9C8B.0	fTotalStrokes	 13
U+9C8B.0	fCangjie	XNMOD
U+9C8B.0	fHakka	 fu5
U+9C8B.0	fCantonese	fu6
U+9C8B.0	fMandarin	fu4; fu1
U+9C8B.0	fEnglish	simplified form of &#x9B92;
U+9C8C.0	fUTF8	鲌
U+9C8C.0	fGB	7-213A
U+9C8C.0	fR/S	 195.5
U+9C8C.0	fTotalStrokes	 13
U+9C8C.0	fCangjie	NMHA
U+9C8C.0	fCantonese	baak6
U+9C8C.0	fMandarin	bo2; ba4
U+9C8D.0	fUTF8	鲍
U+9C8D.0	fGB	0-312B
U+9C8D.0	fR/S	 195.5
U+9C8D.0	fTotalStrokes	 13
U+9C8D.0	fCangjie	NMPRU
U+9C8D.0	fHakka	 bau1; pau5; bau5
U+9C8D.0	fCantonese	baau6; baau1
U+9C8D.0	fMandarin	bao4; bao1
U+9C8D.0	fEnglish	simplified form of &#x9B91;
U+9C8E.0	fUTF8	鲎
U+9C8E.0	fGB	0-7657
U+9C8E.0	fR/S	 195.5
U+9C8E.0	fTotalStrokes	 13
U+9C8E.0	fCangjie	FBNWM
U+9C8E.0	fHakka	 heu5
U+9C8E.0	fCantonese	hau6
U+9C8E.0	fMandarin	hou4; wo4; wu4
U+9C8E.0	fEnglish	simplified form of &#x9C5F;
U+9C8F.0	fUTF8	鲏
U+9C8F.0	fGB	7-213B
U+9C8F.0	fR/S	 195.5
U+9C8F.0	fTotalStrokes	 13
U+9C8F.0	fCangjie	NMDHE
U+9C8F.0	fCantonese	pei4
U+9C8F.0	fMandarin	pi1; pi2; ju4
U+9C90.0	fUTF8	鲐
U+9C90.0	fGB	0-7658
U+9C90.0	fR/S	 195.5
U+9C90.0	fTotalStrokes	 13
U+9C90.0	fCangjie	XNMIR
U+9C90.0	fHakka	 toi2
U+9C90.0	fCantonese	toi4
U+9C90.0	fMandarin	tai2; yi2
U+9C90.0	fEnglish	simplified form of &#x9B90;
U+9C91.0	fUTF8	鲑
U+9C91.0	fGB	0-7659
U+9C91.0	fR/S	 195.6
U+9C91.0	fTotalStrokes	 14
U+9C91.0	fCangjie	NMGG
U+9C91.0	fHakka	 kwui1; va1
U+9C91.0	fCantonese	gwai1; haai4; waa1
U+9C91.0	fMandarin	gui1; xie2; wa1; hua3; kui1
U+9C91.0	fEnglish	simplified form of &#x9BAD;
U+9C92.0	fUTF8	鲒
U+9C92.0	fGB	0-765A
U+9C92.0	fR/S	 195.6
U+9C92.0	fTotalStrokes	 14
U+9C92.0	fCangjie	NMGR
U+9C92.0	fCantonese	git3
U+9C92.0	fMandarin	jie2; ji2; qie4
U+9C92.0	fEnglish	simplified form of &#x9B9A;
U+9C93.0	fUTF8	鲓
U+9C93.0	fGB	8-2D32
U+9C93.0	fR/S	 195.6
U+9C93.0	fTotalStrokes	 14
U+9C93.0	fCangjie	NMJKS
U+9C93.0	fCantonese	kaau3
U+9C93.0	fMandarin	kao3; kao4
U+9C94.0	fUTF8	鲔
U+9C94.0	fGB	0-765B
U+9C94.0	fR/S	 195.6
U+9C94.0	fTotalStrokes	 14
U+9C94.0	fCangjie	NMKB
U+9C94.0	fHakka	 vui3
U+9C94.0	fCantonese	fui2
U+9C94.0	fMandarin	wei3; you3
U+9C94.0	fEnglish	simplified form of &#x9BAA;
U+9C95.0	fUTF8	鲕
U+9C95.0	fGB	0-765C
U+9C95.0	fR/S	 195.6
U+9C95.0	fTotalStrokes	 14
U+9C95.0	fCangjie	NMMBL
U+9C95.0	fHakka	 ji2
U+9C95.0	fCantonese	ji4
U+9C95.0	fMandarin	er2; rou4
U+9C95.0	fEnglish	simplified form of &#x9B9E;
U+9C96.0	fUTF8	鲖
U+9C96.0	fGB	8-2D33
U+9C96.0	fR/S	 195.6
U+9C96.0	fTotalStrokes	 14
U+9C96.0	fCangjie	NMBMR
U+9C96.0	fCantonese	tung4; zau6; tung3
U+9C96.0	fMandarin	tong2; zhou4
U+9C96.0	fEnglish	snakefish
U+9C97.0	fUTF8	鲗
U+9C97.0	fGB	8-2D34
U+9C97.0	fR/S	 195.6
U+9C97.0	fTotalStrokes	 14
U+9C97.0	fCangjie	NMBON
U+9C97.0	fCantonese	caak6; zak1
U+9C97.0	fMandarin	ze2; zei2; obs
U+9C97.0	fEnglish	cuttlefish
U+9C98.0	fUTF8	鲘
U+9C98.0	fGB	8-2D35
U+9C98.0	fR/S	 195.6
U+9C98.0	fTotalStrokes	 14
U+9C98.0	fCangjie	NMHMR
U+9C98.0	fCantonese	hau6
U+9C98.0	fMandarin	hou4
U+9C99.0	fUTF8	鲙
U+9C99.0	fGB	8-2D36
U+9C99.0	fR/S	 195.6
U+9C99.0	fTotalStrokes	 14
U+9C99.0	fCangjie	NMOMI
U+9C99.0	fCantonese	kui2; kwui2 (also)
U+9C99.0	fMandarin	kuai4
U+9C99.0	fEnglish	minced fish; hash
U+9C9A.0	fUTF8	鲚
U+9C9A.0	fGB	0-765D
U+9C9A.0	fR/S	 195.6
U+9C9A.0	fTotalStrokes	 14
U+9C9A.0	fCangjie	NMYKL
U+9C9A.0	fHakka	 ci3
U+9C9A.0	fCantonese	cai5; cai4 (also)
U+9C9A.0	fMandarin	ji4; qi2
U+9C9A.0	fEnglish	simplified form of &#x9C6D;
U+9C9B.0	fUTF8	鲛
U+9C9B.0	fGB	0-765E
U+9C9B.0	fR/S	 195.6
U+9C9B.0	fTotalStrokes	 14
U+9C9B.0	fCangjie	NMYCK
U+9C9B.0	fHakka	 gau1
U+9C9B.0	fCantonese	gaau1
U+9C9B.0	fMandarin	jiao1; jiao3
U+9C9B.0	fEnglish	simplified form of &#x9BAB;
U+9C9C.0	fUTF8	鲜
U+9C9C.0	fGB	0-4F4A
U+9C9C.0	fR/S	 195.6
U+9C9C.0	fTotalStrokes	 14
U+9C9C.0	fCangjie	NMTQ
U+9C9C.0	fHakka	 sien3; sien1; sen1
U+9C9C.0	fCantonese	sin1; sin2
U+9C9C.0	fMandarin	xian1; xian4; xian3
U+9C9C.0	fEnglish	simplified form of &#x9BAE;
U+9C9C.1	fUTF8	鲜
U+9C9C.1	fGB	0-4F4A
U+9C9C.1	fR/S	 195.6
U+9C9C.1	fTotalStrokes	 14
U+9C9C.1	fCangjie	NMTQ
U+9C9C.1	fMandarin	xian5
U+9C9D.0	fUTF8	鲝
U+9C9D.0	fGB	8-2E71
U+9C9D.0	fR/S	 195.6
U+9C9D.0	fTotalStrokes	 14
U+9C9D.0	fCangjie	TQNWM
U+9C9D.0	fCantonese	zaa2
U+9C9D.0	fMandarin	zha3; za3
U+9C9E.0	fUTF8	鲞
U+9C9E.0	fGB	0-765F
U+9C9E.0	fR/S	 195.6
U+9C9E.0	fTotalStrokes	 14
U+9C9E.0	fCangjie	FQNWM
U+9C9E.0	fHakka	 siong3
U+9C9E.0	fCantonese	soeng2
U+9C9E.0	fMandarin	xiang3
U+9C9E.0	fEnglish	simplified form of &#x9B9D;
U+9C9F.0	fUTF8	鲟
U+9C9F.0	fGB	0-7660
U+9C9F.0	fR/S	 195.6
U+9C9F.0	fTotalStrokes	 14
U+9C9F.0	fCangjie	NMSMI
U+9C9F.0	fHakka	 cim2
U+9C9F.0	fCantonese	cam4
U+9C9F.0	fMandarin	xun2; xin2
U+9C9F.0	fEnglish	simplified form of &#x9C58;
U+9CA0.0	fUTF8	鲠
U+9CA0.0	fGB	0-7661
U+9CA0.0	fR/S	 195.7
U+9CA0.0	fTotalStrokes	 15
U+9CA0.0	fCangjie	NMMLK
U+9CA0.0	fHakka	 gen5
U+9CA0.0	fCantonese	gang2; kang2 (also)
U+9CA0.0	fMandarin	geng3; jing1
U+9CA0.0	fEnglish	simplified form of &#x9BC1;
U+9CA1.0	fUTF8	鲡
U+9CA1.0	fGB	0-7662
U+9CA1.0	fR/S	 195.7
U+9CA1.0	fTotalStrokes	 15
U+9CA1.0	fCangjie	XNMMB
U+9CA1.0	fHakka	 li2
U+9CA1.0	fCantonese	lei4
U+9CA1.0	fMandarin	li2; li3
U+9CA1.0	fEnglish	simplified form of &#x9C7A;
U+9CA2.0	fUTF8	鲢
U+9CA2.0	fGB	0-7663
U+9CA2.0	fR/S	 195.8; 195.7
U+9CA2.0	fTotalStrokes	 15
U+9CA2.0	fCangjie	NMYKQ
U+9CA2.0	fHakka	 lien2
U+9CA2.0	fCantonese	lin4
U+9CA2.0	fMandarin	lian2
U+9CA2.0	fEnglish	simplified form of &#x9C31;
U+9CA3.0	fUTF8	鲣
U+9CA3.0	fGB	0-7664
U+9CA3.0	fR/S	 195.7
U+9CA3.0	fTotalStrokes	 15
U+9CA3.0	fCangjie	NMLEG
U+9CA3.0	fHakka	 gen1
U+9CA3.0	fCantonese	gin1
U+9CA3.0	fMandarin	jian1
U+9CA3.0	fEnglish	simplified form of &#x9C39;
U+9CA4.0	fUTF8	鲤
U+9CA4.0	fGB	0-4070
U+9CA4.0	fR/S	 195.7
U+9CA4.0	fTotalStrokes	 15
U+9CA4.0	fCangjie	NMWG
U+9CA4.0	fHakka	 li1; li3
U+9CA4.0	fCantonese	lei5
U+9CA4.0	fMandarin	li3
U+9CA4.0	fEnglish	simplified form of &#x9BC9;
U+9CA5.0	fUTF8	鲥
U+9CA5.0	fGB	0-7665
U+9CA5.0	fR/S	 195.7
U+9CA5.0	fTotalStrokes	 15
U+9CA5.0	fCangjie	NMADI
U+9CA5.0	fHakka	 shi2
U+9CA5.0	fCantonese	si4
U+9CA5.0	fMandarin	shi2
U+9CA5.0	fEnglish	simplified form of &#x9C23;
U+9CA6.0	fUTF8	鲦
U+9CA6.0	fGB	0-7666
U+9CA6.0	fR/S	 195.7
U+9CA6.0	fTotalStrokes	 15
U+9CA6.0	fCangjie	NMHED
U+9CA6.0	fHakka	 tiau2
U+9CA6.0	fCantonese	tiu4
U+9CA6.0	fMandarin	tiao2; xiao3
U+9CA6.0	fEnglish	simplified form of &#x9C37;
U+9CA7.0	fUTF8	鲧
U+9CA7.0	fGB	0-7667
U+9CA7.0	fR/S	 195.7
U+9CA7.0	fTotalStrokes	 15
U+9CA7.0	fCangjie	NMHVF
U+9CA7.0	fHakka	 gwun3
U+9CA7.0	fCantonese	gwan2
U+9CA7.0	fMandarin	gun3
U+9CA7.0	fEnglish	simplified form of &#x9BC0;
U+9CA8.0	fUTF8	鲨
U+9CA8.0	fGB	0-7668
U+9CA8.0	fR/S	 195.7
U+9CA8.0	fTotalStrokes	 15
U+9CA8.0	fCangjie	EHNWM
U+9CA8.0	fHakka	 sa1
U+9CA8.0	fCantonese	saa1
U+9CA8.0	fMandarin	sha1; sa4
U+9CA8.0	fEnglish	simplified form of &#x9BCA;
U+9CA9.0	fUTF8	鲩
U+9CA9.0	fGB	0-7669
U+9CA9.0	fR/S	 195.7
U+9CA9.0	fTotalStrokes	 15
U+9CA9.0	fCangjie	NMJMU
U+9CA9.0	fHakka	 fon5; van3; van1
U+9CA9.0	fCantonese	waan5
U+9CA9.0	fMandarin	wan3; huan3; hun3; huan4; huang4
U+9CA9.0	fEnglish	simplified form of &#x9BC7;
U+9CAA.0	fUTF8	鲪
U+9CAA.0	fGB	7-213C
U+9CAA.0	fR/S	 195.7
U+9CAA.0	fTotalStrokes	 15
U+9CAA.0	fCangjie	NMSKR
U+9CAA.0	fCantonese	gwan1
U+9CAA.0	fMandarin	jun1
U+9CAB.0	fUTF8	鲫
U+9CAB.0	fGB	0-766A
U+9CAB.0	fR/S	 195.7
U+9CAB.0	fTotalStrokes	 15
U+9CAB.0	fCangjie	NMAIL
U+9CAB.0	fHakka	 zit7
U+9CAB.0	fCantonese	zik1; zak1 (also)
U+9CAB.0	fMandarin	ji4; ji2; ze2
U+9CAB.0	fEnglish	simplified form of &#x9BFD;
U+9CAC.0	fUTF8	鲬
U+9CAC.0	fGB	7-213D
U+9CAC.0	fR/S	 195.7
U+9CAC.0	fTotalStrokes	 15
U+9CAC.0	fCangjie	NMNIB
U+9CAC.0	fCantonese	jung2
U+9CAC.0	fMandarin	yong3
U+9CAC.0	fEnglish	a flathead (fish)
U+9CAD.0	fUTF8	鲭
U+9CAD.0	fGB	0-766B
U+9CAD.0	fR/S	 195.8
U+9CAD.0	fTotalStrokes	 16
U+9CAD.0	fCangjie	NMQMB
U+9CAD.0	fHakka	 zhin1; ciang1
U+9CAD.0	fCantonese	cing1; zing1
U+9CAD.0	fMandarin	zheng1; qing1
U+9CAD.0	fEnglish	simplified form of &#x9BD6;
U+9CAE.0	fUTF8	鲮
U+9CAE.0	fGB	0-766C
U+9CAE.0	fR/S	 195.8
U+9CAE.0	fTotalStrokes	 16
U+9CAE.0	fCangjie	NMGCE
U+9CAE.0	fHakka	 lin2
U+9CAE.0	fCantonese	ling4; leng4 (also)
U+9CAE.0	fMandarin	ling2
U+9CAE.0	fEnglish	simplified form of &#x9BEA;
U+9CAF.0	fUTF8	鲯
U+9CAF.0	fGB	7-213E
U+9CAF.0	fR/S	 195.8
U+9CAF.0	fTotalStrokes	 16
U+9CAF.0	fCangjie	XNMTM
U+9CAF.0	fCantonese	kei4
U+9CAF.0	fMandarin	qi2
U+9CAF.0	fEnglish	Coryphaena hippurus
U+9CB0.0	fUTF8	鲰
U+9CB0.0	fGB	0-766D
U+9CB0.0	fR/S	 195.8
U+9CB0.0	fTotalStrokes	 16
U+9CB0.0	fCangjie	NMSJE
U+9CB0.0	fHakka	 zeu1
U+9CB0.0	fCantonese	zau1
U+9CB0.0	fMandarin	zou1; qu1; qian3; chou3; se4
U+9CB0.0	fEnglish	simplified form of &#x9BEB;
U+9CB1.0	fUTF8	鲱
U+9CB1.0	fGB	0-766E
U+9CB1.0	fR/S	 195.8
U+9CB1.0	fTotalStrokes	 16
U+9CB1.0	fCangjie	NMLMY
U+9CB1.0	fCantonese	fei1
U+9CB1.0	fMandarin	fei1; fei4
U+9CB1.0	fEnglish	simplified form of &#x9BE1;
U+9CB2.0	fUTF8	鲲
U+9CB2.0	fGB	0-766F
U+9CB2.0	fR/S	 195.8
U+9CB2.0	fTotalStrokes	 16
U+9CB2.0	fCangjie	NMAPP
U+9CB2.0	fHakka	 kwun1
U+9CB2.0	fCantonese	kwan1; gwan1
U+9CB2.0	fMandarin	kun1; hun2
U+9CB2.0	fEnglish	simplified form of &#x9BE4;
U+9CB3.0	fUTF8	鲳
U+9CB3.0	fGB	0-7670
U+9CB3.0	fR/S	 195.8
U+9CB3.0	fTotalStrokes	 16
U+9CB3.0	fCangjie	NMAA
U+9CB3.0	fHakka	 cong1
U+9CB3.0	fCantonese	coeng1
U+9CB3.0	fMandarin	chang1
U+9CB3.0	fEnglish	simplified form of &#x9BE7;
U+9CB4.0	fUTF8	鲴
U+9CB4.0	fGB	0-7671
U+9CB4.0	fR/S	 195.8
U+9CB4.0	fTotalStrokes	 16
U+9CB4.0	fCangjie	NMWJR
U+9CB4.0	fHakka	 gu5
U+9CB4.0	fCantonese	gu3; gwu3 (also)
U+9CB4.0	fMandarin	gu4
U+9CB4.0	fEnglish	simplified form of &#x9BDD;
U+9CB5.0	fUTF8	鲵
U+9CB5.0	fGB	0-7672
U+9CB5.0	fR/S	 195.8
U+9CB5.0	fTotalStrokes	 16
U+9CB5.0	fCangjie	NMHXU
U+9CB5.0	fHakka	 ngi2; ni2
U+9CB5.0	fCantonese	ngai4; ai4 (also)
U+9CB5.0	fMandarin	ni2
U+9CB5.0	fEnglish	simplified form of &#x9BE2;
U+9CB6.0	fUTF8	鲶
U+9CB6.0	fGB	0-7673
U+9CB6.0	fR/S	 195.8
U+9CB6.0	fTotalStrokes	 16
U+9CB6.0	fCangjie	NMOIP
U+9CB6.0	fCantonese	nim4; lim4 (also); lim6
U+9CB6.0	fMandarin	nian2; nian3
U+9CB6.0	fEnglish	simplified form of &#x9BF0;
U+9CB7.0	fUTF8	鲷
U+9CB7.0	fGB	0-7674
U+9CB7.0	fR/S	 195.8
U+9CB7.0	fTotalStrokes	 16
U+9CB7.0	fCangjie	NMBGR
U+9CB7.0	fHakka	 diau1; tiau2
U+9CB7.0	fCantonese	diu1
U+9CB7.0	fMandarin	diao1
U+9CB7.0	fEnglish	simplified form of &#x9BDB;
U+9CB8.0	fUTF8	鲸
U+9CB8.0	fGB	0-3E28
U+9CB8.0	fR/S	 195.8
U+9CB8.0	fTotalStrokes	 16
U+9CB8.0	fCangjie	NMYRF
U+9CB8.0	fHakka	 gin1; kin1; kiang2
U+9CB8.0	fCantonese	king4
U+9CB8.0	fMandarin	jing1; qing2; qiang1
U+9CB8.0	fEnglish	simplified form of &#x9BE8;
U+9CB9.0	fUTF8	鲹
U+9CB9.0	fGB	7-213F
U+9CB9.0	fR/S	 195.8
U+9CB9.0	fTotalStrokes	 16
U+9CB9.0	fCangjie	XNMIK
U+9CB9.0	fCantonese	sam1
U+9CB9.0	fMandarin	shen1
U+9CB9.0	fEnglish	a horse mackerel
U+9CBA.0	fUTF8	鲺
U+9CBA.0	fGB	0-7675
U+9CBA.0	fR/S	 195.8
U+9CBA.0	fTotalStrokes	 16
U+9CBA.0	fCangjie	NMNHI
U+9CBA.0	fCantonese	sat1
U+9CBA.0	fMandarin	shi1
U+9CBA.0	fEnglish	simplified form of &#x9BF4;
U+9CBB.0	fUTF8	鲻
U+9CBB.0	fGB	0-7676
U+9CBB.0	fR/S	 195.8
U+9CBB.0	fTotalStrokes	 16
U+9CBB.0	fCangjie	NMVVW
U+9CBB.0	fHakka	 zih1
U+9CBB.0	fCantonese	zi1
U+9CBB.0	fMandarin	zi1; chun3
U+9CBB.0	fEnglish	simplified form of &#x9BD4;
U+9CBC.0	fUTF8	鲼
U+9CBC.0	fGB	0-7677
U+9CBC.0	fR/S	 195.9
U+9CBC.0	fTotalStrokes	 17
U+9CBC.0	fCangjie	NMJTO
U+9CBC.0	fHakka	 fun5
U+9CBC.0	fCantonese	fan5
U+9CBC.0	fMandarin	fen4
U+9CBC.0	fEnglish	simplified form of &#x9C5D;
U+9CBD.0	fUTF8	鲽
U+9CBD.0	fGB	0-7678
U+9CBD.0	fR/S	 195.9
U+9CBD.0	fTotalStrokes	 17
U+9CBD.0	fCangjie	NMPTD
U+9CBD.0	fHakka	 tiap8
U+9CBD.0	fCantonese	dip6; cip3
U+9CBD.0	fMandarin	die2; die1; qie1; ta4
U+9CBD.0	fEnglish	simplified form of &#x9C08;
U+9CBE.0	fUTF8	鲾
U+9CBE.0	fGB	7-2140
U+9CBE.0	fR/S	 195.10
U+9CBE.0	fTotalStrokes	 18
U+9CBE.0	fCangjie	NMMRW
U+9CBE.0	fCantonese	bik1
U+9CBE.0	fMandarin	bi4; bi1
U+9CBF.0	fUTF8	鲿
U+9CBF.0	fGB	8-2D37
U+9CBF.0	fR/S	 195.9
U+9CBF.0	fTotalStrokes	 17
U+9CBF.0	fCangjie	NMFBI
U+9CBF.0	fCantonese	soeng4
U+9CBF.0	fMandarin	chang2
U+9CBF.0	fEnglish	codfish
U+9CC0.0	fUTF8	鳀
U+9CC0.0	fGB	8-2D38
U+9CC0.0	fR/S	 195.9
U+9CC0.0	fTotalStrokes	 17
U+9CC0.0	fCangjie	NMAMO
U+9CC0.0	fCantonese	tai4
U+9CC0.0	fMandarin	shi4; ti2
U+9CC0.0	fEnglish	anchovy
U+9CC1.0	fUTF8	鳁
U+9CC1.0	fGB	8-2D39
U+9CC1.0	fR/S	 195.9
U+9CC1.0	fTotalStrokes	 17
U+9CC1.0	fCangjie	NMABT
U+9CC1.0	fCantonese	wan1
U+9CC1.0	fMandarin	wen1
U+9CC1.0	fEnglish	sardine
U+9CC2.0	fUTF8	鳂
U+9CC2.0	fGB	7-2142
U+9CC2.0	fR/S	 195.9
U+9CC2.0	fTotalStrokes	 17
U+9CC2.0	fCangjie	NMWMV
U+9CC2.0	fCantonese	wai1; wui1 (also)
U+9CC2.0	fMandarin	wei1
U+9CC2.0	fEnglish	name of fish
U+9CC3.0	fUTF8	鳃
U+9CC3.0	fGB	0-487A
U+9CC3.0	fR/S	 195.9
U+9CC3.0	fTotalStrokes	 17
U+9CC3.0	fCangjie	NMWP
U+9CC3.0	fHakka	 soi1
U+9CC3.0	fCantonese	soi1; sai2
U+9CC3.0	fMandarin	sai1; xi3
U+9CC3.0	fEnglish	simplified form of &#x9C13;
U+9CC4.0	fUTF8	鳄
U+9CC4.0	fGB	0-7679
U+9CC4.0	fR/S	 195.9
U+9CC4.0	fTotalStrokes	 17
U+9CC4.0	fCangjie	NMRRS
U+9CC4.0	fHakka	 ngok8
U+9CC4.0	fCantonese	ngok6; ok6 (also)
U+9CC4.0	fMandarin	e4
U+9CC4.0	fEnglish	simplified form of &#x9C10;
U+9CC5.0	fUTF8	鳅
U+9CC5.0	fGB	0-767A
U+9CC5.0	fR/S	 195.9
U+9CC5.0	fTotalStrokes	 17
U+9CC5.0	fCangjie	NMHDF
U+9CC5.0	fHakka	 ciu1
U+9CC5.0	fCantonese	cau1
U+9CC5.0	fMandarin	qiu1
U+9CC5.0	fEnglish	simplified form of &#x9C0D;
U+9CC6.0	fUTF8	鳆
U+9CC6.0	fGB	0-767B
U+9CC6.0	fR/S	 195.9
U+9CC6.0	fTotalStrokes	 17
U+9CC6.0	fCangjie	NMOAE
U+9CC6.0	fHakka	 fuk7
U+9CC6.0	fCantonese	fuk1
U+9CC6.0	fMandarin	fu4; bao4
U+9CC6.0	fEnglish	simplified form of &#x9C12;
U+9CC7.0	fUTF8	鳇
U+9CC7.0	fGB	0-767C
U+9CC7.0	fR/S	 195.9
U+9CC7.0	fTotalStrokes	 17
U+9CC7.0	fCangjie	NMHAG
U+9CC7.0	fHakka	 fong2
U+9CC7.0	fCantonese	wong4
U+9CC7.0	fMandarin	huang2
U+9CC7.0	fEnglish	simplified form of &#x9C09;
U+9CC8.0	fUTF8	鳈
U+9CC8.0	fGB	7-2143
U+9CC8.0	fR/S	 195.9
U+9CC8.0	fTotalStrokes	 17
U+9CC8.0	fCangjie	NMHAE
U+9CC8.0	fCantonese	cyun4
U+9CC8.0	fMandarin	quan2
U+9CC8.0	fEnglish	name of fish
U+9CC9.0	fUTF8	鳉
U+9CC9.0	fGB	7-2141
U+9CC9.0	fR/S	 195.9
U+9CC9.0	fTotalStrokes	 17
U+9CC9.0	fCangjie	XNMLM
U+9CC9.0	fCantonese	zoeng1
U+9CC9.0	fMandarin	jiang1
U+9CC9.0	fEnglish	name of fish
U+9CCA.0	fUTF8	鳊
U+9CCA.0	fGB	0-767D
U+9CCA.0	fR/S	 195.9
U+9CCA.0	fTotalStrokes	 17
U+9CCA.0	fCangjie	XNMIS
U+9CCA.0	fHakka	 bien1
U+9CCA.0	fCantonese	bin1; bin2 (also)
U+9CCA.0	fMandarin	bian1
U+9CCA.0	fEnglish	simplified form of &#x9BFF;
U+9CCB.0	fUTF8	鳋
U+9CCB.0	fGB	0-767E
U+9CCB.0	fR/S	 195.9
U+9CCB.0	fTotalStrokes	 17
U+9CCB.0	fCangjie	XNMEI
U+9CCB.0	fHakka	 sau1
U+9CCB.0	fCantonese	sou1
U+9CCB.0	fMandarin	sao1
U+9CCB.0	fEnglish	simplified form of &#x9C20;
U+9CCC.0	fUTF8	鳌
U+9CCC.0	fGB	0-7721
U+9CCC.0	fR/S	 195.10
U+9CCC.0	fTotalStrokes	 18
U+9CCC.0	fCangjie	QKNWM
U+9CCC.0	fHakka	 ngau2
U+9CCC.0	fCantonese	ngou4; ou4 (also)
U+9CCC.0	fMandarin	ao2
U+9CCC.0	fEnglish	simplified form of &#x9C32;
U+9CCD.0	fUTF8	鳍
U+9CCD.0	fGB	0-7722
U+9CCD.0	fR/S	 195.10
U+9CCD.0	fTotalStrokes	 18
U+9CCD.0	fCangjie	NMJPA
U+9CCD.0	fHakka	 ki2
U+9CCD.0	fCantonese	kei4
U+9CCD.0	fMandarin	qi2
U+9CCD.0	fEnglish	simplified form of &#x9C2D;
U+9CCE.0	fUTF8	鳎
U+9CCE.0	fGB	0-7723
U+9CCE.0	fR/S	 195.10
U+9CCE.0	fTotalStrokes	 18
U+9CCE.0	fCangjie	NMASM
U+9CCE.0	fHakka	 tap7
U+9CCE.0	fCantonese	taap3; dip6; taat3
U+9CCE.0	fMandarin	ta4; ta3; na4; die2
U+9CCE.0	fEnglish	simplified form of &#x9C28;
U+9CCF.0	fUTF8	鳏
U+9CCF.0	fGB	0-7724
U+9CCF.0	fR/S	 195.10
U+9CCF.0	fTotalStrokes	 18
U+9CCF.0	fCangjie	NMWLF
U+9CCF.0	fCantonese	gwaan1; kwan1
U+9CCF.0	fMandarin	guan1; yin2; guan4; gun3; kun1
U+9CCF.0	fEnglish	simplified form of &#x9C25;
U+9CD0.0	fUTF8	鳐
U+9CD0.0	fGB	0-7725
U+9CD0.0	fR/S	 195.10
U+9CD0.0	fTotalStrokes	 18
U+9CD0.0	fCangjie	NMBOU
U+9CD0.0	fHakka	 jau2
U+9CD0.0	fCantonese	jiu4
U+9CD0.0	fMandarin	yao2
U+9CD0.0	fEnglish	simplified form of &#x9C29;
U+9CD1.0	fUTF8	鳑
U+9CD1.0	fGB	7-2144
U+9CD1.0	fR/S	 195.10
U+9CD1.0	fTotalStrokes	 18
U+9CD1.0	fCangjie	NMYBS
U+9CD1.0	fCantonese	pong4; fong4
U+9CD1.0	fMandarin	fang2; pang2
U+9CD2.0	fUTF8	鳒
U+9CD2.0	fGB	8-2D3A
U+9CD2.0	fR/S	 195.10
U+9CD2.0	fTotalStrokes	 18
U+9CD2.0	fCangjie	NMTXC
U+9CD2.0	fCantonese	gim1
U+9CD2.0	fMandarin	jian1
U+9CD2.0	fEnglish	big-mouthed flounder
U+9CD3.0	fUTF8	鳓
U+9CD3.0	fGB	0-7726
U+9CD3.0	fR/S	 195.11
U+9CD3.0	fTotalStrokes	 19
U+9CD3.0	fCangjie	NMTJS
U+9CD3.0	fCantonese	lak6; laak6 (also)
U+9CD3.0	fMandarin	le4
U+9CD3.0	fEnglish	simplified form of &#x9C33;
U+9CD4.0	fUTF8	鳔
U+9CD4.0	fGB	0-7727
U+9CD4.0	fR/S	 195.11
U+9CD4.0	fTotalStrokes	 19
U+9CD4.0	fCangjie	XNMMW
U+9CD4.0	fHakka	 piau5
U+9CD4.0	fCantonese	piu5
U+9CD4.0	fMandarin	biao4; biao3
U+9CD4.0	fEnglish	simplified form of &#x9C3E;
U+9CD5.0	fUTF8	鳕
U+9CD5.0	fGB	0-7728
U+9CD5.0	fR/S	 195.11
U+9CD5.0	fTotalStrokes	 19
U+9CD5.0	fCangjie	NMMBS
U+9CD5.0	fCantonese	syut3
U+9CD5.0	fMandarin	xue3
U+9CD5.0	fEnglish	simplified form of &#x9C48;
U+9CD6.0	fUTF8	鳖
U+9CD6.0	fGB	0-316E
U+9CD6.0	fR/S	 195.11
U+9CD6.0	fTotalStrokes	 19
U+9CD6.0	fCangjie	FKNWM
U+9CD6.0	fHakka	 biet7
U+9CD6.0	fCantonese	bit3
U+9CD6.0	fMandarin	bie1
U+9CD6.0	fEnglish	simplified form of &#x9C49;
U+9CD7.0	fUTF8	鳗
U+9CD7.0	fGB	0-7729
U+9CD7.0	fR/S	 195.11
U+9CD7.0	fTotalStrokes	 19
U+9CD7.0	fCangjie	NMAWE
U+9CD7.0	fHakka	 man2
U+9CD7.0	fCantonese	maan4 (also); maan6
U+9CD7.0	fMandarin	man2; wan4
U+9CD7.0	fEnglish	simplified form of &#x9C3B;
U+9CD8.0	fUTF8	鳘
U+9CD8.0	fGB	0-772A
U+9CD8.0	fR/S	 195.11
U+9CD8.0	fTotalStrokes	 19
U+9CD8.0	fCangjie	OKNWM
U+9CD8.0	fHakka	 men1; mien3
U+9CD8.0	fCantonese	man5
U+9CD8.0	fMandarin	min3
U+9CD8.0	fEnglish	simplified form of &#x9C35;
U+9CD9.0	fUTF8	鳙
U+9CD9.0	fGB	0-772B
U+9CD9.0	fR/S	 195.11
U+9CD9.0	fTotalStrokes	 19
U+9CD9.0	fCangjie	NMILB
U+9CD9.0	fHakka	 jung2; sung2
U+9CD9.0	fCantonese	jung4; sung4 (also)
U+9CD9.0	fMandarin	yong1; yong2; obs
U+9CD9.0	fEnglish	simplified form of &#x9C45;
U+9CDA.0	fUTF8	鳚
U+9CDA.0	fGB	7-2145
U+9CDA.0	fR/S	 195.11
U+9CDA.0	fTotalStrokes	 19
U+9CDA.0	fCangjie	NMSFI
U+9CDA.0	fCantonese	wai3; to5; waai3
U+9CDA.0	fMandarin	wei4
U+9CDA.0	fEnglish	general term of a kind of fish, a snake-like fish
U+9CDB.0	fUTF8	鳛
U+9CDB.0	fGB	8-2D3B
U+9CDB.0	fR/S	 195.11
U+9CDB.0	fTotalStrokes	 19
U+9CDB.0	fCangjie	NMSMA
U+9CDB.0	fCantonese	zaap6
U+9CDB.0	fMandarin	xi2
U+9CDB.0	fEnglish	the weather or dojo loach, Misgurnus anguillicaudatus
U+9CDC.0	fUTF8	鳜
U+9CDC.0	fGB	0-772C
U+9CDC.0	fR/S	 195.12
U+9CDC.0	fTotalStrokes	 20
U+9CDC.0	fCangjie	NMMTO
U+9CDC.0	fHakka	 ket7; kiet7; k'wui3
U+9CDC.0	fCantonese	gwai3; kyut3
U+9CDC.0	fMandarin	jue2; gui4; gui1; qiu1; wan3
U+9CDC.0	fEnglish	simplified form of &#x9C56;
U+9CDD.0	fUTF8	鳝
U+9CDD.0	fGB	0-772D
U+9CDD.0	fR/S	 195.12
U+9CDD.0	fTotalStrokes	 20
U+9CDD.0	fCangjie	NMTTR
U+9CDD.0	fHakka	 shen1; shen3
U+9CDD.0	fCantonese	sin5; sin2 (also)
U+9CDD.0	fMandarin	shan4
U+9CDD.0	fEnglish	simplified form of &#x9C54;
U+9CDE.0	fUTF8	鳞
U+9CDE.0	fGB	0-415B
U+9CDE.0	fR/S	 195.12
U+9CDE.0	fTotalStrokes	 20
U+9CDE.0	fCangjie	NMFDQ
U+9CDE.0	fHakka	 lin2
U+9CDE.0	fCantonese	leon4
U+9CDE.0	fMandarin	lin2
U+9CDE.0	fEnglish	simplified form of &#x9C57;
U+9CDF.0	fUTF8	鳟
U+9CDF.0	fGB	0-772E
U+9CDF.0	fR/S	 195.12
U+9CDF.0	fTotalStrokes	 20
U+9CDF.0	fCangjie	NMTWI
U+9CDF.0	fHakka	 zun3
U+9CDF.0	fCantonese	zyun1; zyun3 (also); cyun5
U+9CDF.0	fMandarin	zun1; zun4; obs; cun2
U+9CDF.0	fEnglish	simplified form of &#x9C52;
U+9CE0.0	fUTF8	鳠
U+9CE0.0	fGB	8-2D3C
U+9CE0.0	fR/S	 195.13
U+9CE0.0	fTotalStrokes	 21
U+9CE0.0	fCangjie	NMTOE
U+9CE0.0	fCantonese	wu6
U+9CE0.0	fMandarin	huo4; hu4
U+9CE1.0	fUTF8	鳡
U+9CE1.0	fGB	7-2146
U+9CE1.0	fR/S	 195.13
U+9CE1.0	fTotalStrokes	 21
U+9CE1.0	fCangjie	NMIRP
U+9CE1.0	fCantonese	gam2
U+9CE1.0	fMandarin	gan3
U+9CE2.0	fUTF8	鳢
U+9CE2.0	fGB	0-772F
U+9CE2.0	fR/S	 195.13
U+9CE2.0	fTotalStrokes	 21
U+9CE2.0	fCangjie	NMTWT
U+9CE2.0	fHakka	 li3; li1
U+9CE2.0	fCantonese	lai5
U+9CE2.0	fMandarin	li3
U+9CE2.0	fEnglish	simplified form of &#x9C67;
U+9CE3.0	fUTF8	鳣
U+9CE3.0	fGB	8-2D3D
U+9CE3.0	fR/S	 195.13
U+9CE3.0	fTotalStrokes	 21
U+9CE3.0	fCangjie	NMYWM
U+9CE3.0	fCantonese	zin1; sin5
U+9CE3.0	fMandarin	zhan1; shan4
U+9CE3.0	fEnglish	sturgeon
U+9CE4.0	fUTF8	鳤
U+9CE4.0	fGB	7-2147
U+9CE4.0	fR/S	 195.14
U+9CE4.0	fTotalStrokes	 22
U+9CE4.0	fCangjie	NMHJR
U+9CE4.0	fCantonese	gun2; gwun2 (also)
U+9CE4.0	fMandarin	guan3
U+9CE5.0	fUTF8	鳥
U+9CE5.0	fCNS11643	1-5E24
U+9CE5.0	fBig5	B3BE
U+9CE5.0	fGB	1-4471
U+9CE5.0	fR/S	 196.0
U+9CE5.0	fTotalStrokes	 11
U+9CE5.0	fCangjie	HAYF
U+9CE5.0	fFourCorner	27327
U+9CE5.0	fHakka	  diau1;  ngiau1; ngiau3;  niau1;  niau3; {ngiau1}; {niau1}
U+9CE5.0	fCantonese	niu5; diu2; liu5 (also); tiu2
U+9CE5.0	fMandarin	diao3; niao3; dao3; jue2; que4
U+9CE5.0	fEnglish	[1] bird [2] Kangxi radical 196
U+9CE6.0	fUTF8	鳦
U+9CE6.0	fCNS11643	2-4154
U+9CE6.0	fBig5	DCAF
U+9CE6.0	fGB	3-5C74
U+9CE6.0	fR/S	 196.1
U+9CE6.0	fTotalStrokes	 12
U+9CE6.0	fCangjie	HFN
U+9CE6.0	fFourCorner	27310
U+9CE6.0	fHakka	 ret7; jet7; jat7
U+9CE6.0	fCantonese	jyut3 (also); jyut6
U+9CE6.0	fMandarin	yi3; ya4; yi4
U+9CE6.0	fEnglish	swallow
U+9CE7.0	fUTF8	鳧
U+9CE7.0	fCNS11643	2-4873
U+9CE7.0	fBig5	E0ED
U+9CE7.0	fGB	5-5B73
U+9CE7.0	fR/S	 196.2
U+9CE7.0	fTotalStrokes	 13
U+9CE7.0	fCangjie	HFHN
U+9CE7.0	fFourCorner	27217
U+9CE7.0	fHakka	 fu2
U+9CE7.0	fCantonese	fu4
U+9CE7.0	fMandarin	fu2
U+9CE7.0	fEnglish	 wild duck
U+9CE8.0	fUTF8	鳨
U+9CE8.0	fCNS11643	3-4564
U+9CE8.0	fGB	3-5C75
U+9CE8.0	fR/S	 196.2
U+9CE8.0	fTotalStrokes	 13
U+9CE8.0	fCangjie	HFKS
U+9CE8.0	fFourCorner	24320
U+9CE8.0	fCantonese	lik6
U+9CE8.0	fMandarin	li4
U+9CE9.0	fUTF8	鳩
U+9CE9.0	fCNS11643	1-676D
U+9CE9.0	fBig5	B9A7
U+9CE9.0	fGB	1-702F
U+9CE9.0	fR/S	 196.2
U+9CE9.0	fTotalStrokes	 13
U+9CE9.0	fCangjie	KNHAF
U+9CE9.0	fFourCorner	47027
U+9CE9.0	fHakka	 kieu1; giu1; gu1; geu1; chim2
U+9CE9.0	fCantonese	gau1; kau1 (also); kau2
U+9CE9.0	fMandarin	jiu1; zhi4; qiu2; qiu1
U+9CE9.0	fEnglish	[1] pigeon; dove [2] collect; assemble
U+9CEA.0	fUTF8	鳪
U+9CEA.0	fCNS11643	2-4871
U+9CEA.0	fBig5	E0EB
U+9CEA.0	fGB	3-3054
U+9CEA.0	fR/S	 196.2
U+9CEA.0	fTotalStrokes	 13
U+9CEA.0	fCangjie	YHAF
U+9CEA.0	fFourCorner	27027
U+9CEA.0	fCantonese	buk1
U+9CEA.0	fMandarin	bu3; bo1
U+9CEB.0	fUTF8	鳫
U+9CEB.0	fCNS11643	3-4566
U+9CEB.0	fGB	E-4754
U+9CEB.0	fR/S	 196.2
U+9CEB.0	fTotalStrokes	 13
U+9CEB.0	fCangjie	MHAF
U+9CEB.0	fFourCorner	71220
U+9CEB.0	fCantonese	ngaan6; aan6 (also)
U+9CEB.0	fMandarin	yan4
U+9CEB.0	fEnglish	wild goose
U+9CEC.0	fUTF8	鳬
U+9CEC.0	fCNS11643	3-2F42
U+9CEC.0	fGB	1-596C
U+9CEC.0	fR/S	 196.2
U+9CEC.0	fTotalStrokes	 9; 13
U+9CEC.0	fCangjie	HAYN
U+9CEC.0	fFourCorner	27210
U+9CEC.0	fCantonese	fu4
U+9CEC.0	fMandarin	fu2
U+9CEC.0	fEnglish	 wild duck; teal swim
U+9CED.0	fUTF8	鳭
U+9CED.0	fCNS11643	2-4872
U+9CED.0	fBig5	E0EC
U+9CED.0	fGB	3-326C
U+9CED.0	fR/S	 196.2; 196.3
U+9CED.0	fTotalStrokes	 13
U+9CED.0	fCangjie	SHHAF
U+9CED.0	fFourCorner	17227
U+9CED.0	fCantonese	dou1; diu1
U+9CED.0	fMandarin	diao1; jiao1; zhao1; chao2
U+9CEE.0	fUTF8	鳮
U+9CEE.0	fCNS11643	3-4568
U+9CEE.0	fGB	E-4755
U+9CEE.0	fR/S	 196.2
U+9CEE.0	fTotalStrokes	 13
U+9CEE.0	fCangjie	EHAF
U+9CEE.0	fFourCorner	17420
U+9CEE.0	fCantonese	gai1
U+9CEE.0	fMandarin	ji1
U+9CEE.0	fEnglish	 chickens domestic fowl
U+9CEF.0	fUTF8	鳯
U+9CEF.0	fCNS11643	3-4567
U+9CEF.0	fGB	E-4756
U+9CEF.0	fR/S	 196.2
U+9CEF.0	fTotalStrokes	 13
U+9CEF.0	fCangjie	HNHAF
U+9CEF.0	fFourCorner	77210
U+9CEF.0	fCantonese	fung6
U+9CEF.0	fMandarin	feng4
U+9CEF.0	fEnglish	 male phoenix symbol of joy
U+9CF0.0	fUTF8	鳰
U+9CF0.0	fGB	E-4757
U+9CF0.0	fR/S	 196.2
U+9CF0.0	fTotalStrokes	 13
U+9CF0.0	fCangjie	OHAF
U+9CF0.0	fFourCorner	8702
U+9CF0.0	fCantonese	jap6
U+9CF1.0	fUTF8	鳱
U+9CF1.0	fCNS11643	2-4F4A
U+9CF1.0	fBig5	E4E2
U+9CF1.0	fGB	5-3548
U+9CF1.0	fR/S	 196.3
U+9CF1.0	fTotalStrokes	 14
U+9CF1.0	fCangjie	MJHAF
U+9CF1.0	fFourCorner	17427
U+9CF1.0	fHakka	 gon1
U+9CF1.0	fCantonese	gon1; hon6
U+9CF1.0	fMandarin	gan1; han4; an4
U+9CF1.0	fEnglish	magpie
U+9CF2.0	fUTF8	鳲
U+9CF2.0	fCNS11643	2-4F4B
U+9CF2.0	fBig5	E4E3
U+9CF2.0	fGB	3-3B51
U+9CF2.0	fR/S	 196.3
U+9CF2.0	fTotalStrokes	 14
U+9CF2.0	fCangjie	SHAF
U+9CF2.0	fFourCorner	77227
U+9CF2.0	fHakka	 chi1; shi1; cih1; si1; sii1; she1
U+9CF2.0	fCantonese	si1
U+9CF2.0	fMandarin	shi1
U+9CF2.0	fEnglish	 cuckoo
U+9CF3.0	fUTF8	鳳
U+9CF3.0	fCNS11643	1-6B7A
U+9CF3.0	fBig5	BBF1
U+9CF3.0	fGB	1-376F
U+9CF3.0	fR/S	 196.3
U+9CF3.0	fTotalStrokes	 14
U+9CF3.0	fCangjie	HNMAF
U+9CF3.0	fFourCorner	77210
U+9CF3.0	fHakka	 fung3; fung6;  fung5
U+9CF3.0	fCantonese	fung6; fung2 (also)
U+9CF3.0	fMandarin	feng4
U+9CF3.0	fEnglish	Feng, a male phoenix
U+9CF4.0	fUTF8	鳴
U+9CF4.0	fCNS11643	1-6B78
U+9CF4.0	fBig5	BBEF
U+9CF4.0	fGB	1-4379
U+9CF4.0	fR/S	 196.3
U+9CF4.0	fTotalStrokes	 14
U+9CF4.0	fCangjie	RHAF
U+9CF4.0	fFourCorner	67027
U+9CF4.0	fHakka	  min2; min3
U+9CF4.0	fCantonese	ming4
U+9CF4.0	fMandarin	ming2; ming4
U+9CF4.0	fEnglish	[1] cry; sing; chirp; warble (birds); crow (cocks) [2] make sounds; sound [3] the notes of birds
U+9CF5.0	fUTF8	鳵
U+9CF5.0	fCNS11643	2-4F4C
U+9CF5.0	fBig5	E4E4
U+9CF5.0	fGB	E-4758
U+9CF5.0	fR/S	 196.3
U+9CF5.0	fTotalStrokes	 14
U+9CF5.0	fCangjie	NJHAF
U+9CF5.0	fFourCorner	17427
U+9CF5.0	fCantonese	bou2
U+9CF5.0	fMandarin	bao3
U+9CF6.0	fUTF8	鳶
U+9CF6.0	fCNS11643	1-6B79
U+9CF6.0	fBig5	BBF0
U+9CF6.0	fGB	1-7030
U+9CF6.0	fR/S	 196.3
U+9CF6.0	fTotalStrokes	 14
U+9CF6.0	fCangjie	IPHAF
U+9CF6.0	fFourCorner	43327
U+9CF6.0	fHakka	 jen1; ren1; jan1
U+9CF6.0	fCantonese	jyun1; jyun4 (also)
U+9CF6.0	fMandarin	yuan1; qian4; yuan2
U+9CF6.0	fEnglish	 kite; hawk
U+9CF7.0	fUTF8	鳷
U+9CF7.0	fCNS11643	2-5633
U+9CF7.0	fBig5	E8E8
U+9CF7.0	fGB	3-4366
U+9CF7.0	fR/S	 196.4
U+9CF7.0	fTotalStrokes	 15
U+9CF7.0	fCangjie	JEHAF
U+9CF7.0	fFourCorner	47427
U+9CF7.0	fHakka	 zhi1; zi1
U+9CF7.0	fCantonese	zi1
U+9CF7.0	fMandarin	zhi1
U+9CF7.0	fEnglish	bird
U+9CF8.0	fUTF8	鳸
U+9CF8.0	fCNS11643	3-5027
U+9CF8.0	fGB	5-5265
U+9CF8.0	fR/S	 196.4
U+9CF8.0	fTotalStrokes	 15
U+9CF8.0	fCangjie	HSHAF
U+9CF8.0	fFourCorner	30220
U+9CF8.0	fHakka	 fu5; fu6
U+9CF8.0	fCantonese	wu6
U+9CF8.0	fMandarin	hu4
U+9CF8.0	fEnglish	general name for finches
U+9CF9.0	fUTF8	鳹
U+9CF9.0	fCNS11643	2-5636
U+9CF9.0	fBig5	E8EB
U+9CF9.0	fGB	3-5D21
U+9CF9.0	fR/S	 196.4
U+9CF9.0	fTotalStrokes	 15
U+9CF9.0	fCangjie	ONHAF
U+9CF9.0	fFourCorner	87227
U+9CF9.0	fCantonese	kam4; kim4; ling4
U+9CF9.0	fMandarin	qin2; qian2
U+9CFA.0	fUTF8	鳺
U+9CFA.0	fCNS11643	2-5630
U+9CFA.0	fBig5	E8E5
U+9CFA.0	fGB	3-5C79
U+9CFA.0	fR/S	 196.4
U+9CFA.0	fTotalStrokes	 15
U+9CFA.0	fCangjie	QOHAF
U+9CFA.0	fFourCorner	57027
U+9CFA.0	fCantonese	fu1
U+9CFA.0	fMandarin	fu1
U+9CFB.0	fUTF8	鳻
U+9CFB.0	fCNS11643	2-5637
U+9CFB.0	fBig5	E8EC
U+9CFB.0	fGB	3-5D22
U+9CFB.0	fR/S	 196.4
U+9CFB.0	fTotalStrokes	 15
U+9CFB.0	fCangjie	CHHAF
U+9CFB.0	fFourCorner	87227
U+9CFB.0	fHakka	 fun1; ban1
U+9CFB.0	fCantonese	baan1; fan4; fan1
U+9CFB.0	fMandarin	fen1; ban1
U+9CFB.0	fEnglish	the wild pigeon
U+9CFC.0	fUTF8	鳼
U+9CFC.0	fCNS11643	2-562F
U+9CFC.0	fBig5	E8E4
U+9CFC.0	fGB	3-4F3F
U+9CFC.0	fR/S	 196.4
U+9CFC.0	fTotalStrokes	 15
U+9CFC.0	fCangjie	YKHAF
U+9CFC.0	fFourCorner	07427
U+9CFC.0	fCantonese	man4
U+9CFC.0	fMandarin	wen2; min2
U+9CFD.0	fUTF8	鳽
U+9CFD.0	fCNS11643	2-5631
U+9CFD.0	fBig5	E8E6
U+9CFD.0	fGB	3-5C78
U+9CFD.0	fR/S	 196.4
U+9CFD.0	fTotalStrokes	 15
U+9CFD.0	fCangjie	MTHAF
U+9CFD.0	fFourCorner	17427
U+9CFD.0	fHakka	 ngiak8; gien1
U+9CFD.0	fCantonese	gin1; hang1; zin1
U+9CFD.0	fMandarin	jian1; keng1; yan2
U+9CFE.0	fUTF8	鳾
U+9CFE.0	fCNS11643	3-502A
U+9CFE.0	fGB	3-5C7A
U+9CFE.0	fR/S	  196.4
U+9CFE.0	fTotalStrokes	 15
U+9CFE.0	fCangjie	MBHAF
U+9CFE.0	fFourCorner	17220
U+9CFE.0	fCantonese	si1
U+9CFE.0	fMandarin	shi1
U+9CFF.0	fUTF8	鳿
U+9CFF.0	fCNS11643	2-5632
U+9CFF.0	fBig5	E8E7
U+9CFF.0	fGB	3-3F2C
U+9CFF.0	fR/S	 196.4
U+9CFF.0	fTotalStrokes	 15
U+9CFF.0	fCangjie	MGHAF
U+9CFF.0	fFourCorner	14127
U+9CFF.0	fHakka	 ngiuk8
U+9CFF.0	fCantonese	juk6
U+9CFF.0	fMandarin	yu4
U+9CFF.0	fEnglish	a water-fowl
U+9D00.0	fUTF8	鴀
U+9D00.0	fCNS11643	2-5635
U+9D00.0	fBig5	E8EA
U+9D00.0	fGB	3-5C7B
U+9D00.0	fR/S	 196.4
U+9D00.0	fTotalStrokes	 15
U+9D00.0	fCangjie	MFHAF
U+9D00.0	fFourCorner	17927
U+9D00.0	fCantonese	fau4; fau2 (also)
U+9D00.0	fMandarin	fou3; fou1; fou2
U+9D01.0	fUTF8	鴁
U+9D01.0	fCNS11643	3-5024
U+9D01.0	fGB	3-5C7C
U+9D01.0	fR/S	 196.4
U+9D01.0	fTotalStrokes	 15
U+9D01.0	fCangjie	HKHAF
U+9D01.0	fFourCorner	27420
U+9D01.0	fCantonese	jiu1
U+9D01.0	fMandarin	yao1; ao3
U+9D01.0	fEnglish	a legendary bird
U+9D02.0	fUTF8	鴂
U+9D02.0	fCNS11643	3-502B
U+9D02.0	fGB	3-5D28
U+9D02.0	fR/S	 196.4
U+9D02.0	fTotalStrokes	 15
U+9D02.0	fCangjie	DKHAF
U+9D02.0	fFourCorner	57020
U+9D02.0	fHakka	  giet7; get7
U+9D02.0	fCantonese	kyut3; gwik1
U+9D02.0	fMandarin	jue4; jue2; gui1
U+9D02.0	fEnglish	 shrike; butcherbird
U+9D03.0	fUTF8	鴃
U+9D03.0	fCNS11643	1-702A
U+9D03.0	fBig5	BEA1
U+9D03.0	fGB	5-5C21
U+9D03.0	fR/S	 196.4
U+9D03.0	fTotalStrokes	 15
U+9D03.0	fCangjie	HFDK
U+9D03.0	fFourCorner	25330
U+9D03.0	fHakka	 giet7; get7
U+9D03.0	fCantonese	kyut3; gwik1
U+9D03.0	fMandarin	jue2; ju2
U+9D03.0	fEnglish	 shrike butcherbird
U+9D04.0	fUTF8	鴄
U+9D04.0	fCNS11643	2-563A
U+9D04.0	fBig5	E8EF
U+9D04.0	fGB	5-5B7C
U+9D04.0	fR/S	 196.4
U+9D04.0	fTotalStrokes	 15
U+9D04.0	fCangjie	SCHAF
U+9D04.0	fFourCorner	77727
U+9D04.0	fHakka	  pit7
U+9D04.0	fCantonese	pat1
U+9D04.0	fMandarin	pi1
U+9D04.0	fEnglish	duck
U+9D05.0	fUTF8	鴅
U+9D05.0	fCNS11643	2-5639
U+9D05.0	fBig5	E8EE
U+9D05.0	fGB	3-5D27
U+9D05.0	fR/S	 196.4
U+9D05.0	fTotalStrokes	 15
U+9D05.0	fCangjie	BYHAF
U+9D05.0	fFourCorner	27427
U+9D05.0	fCantonese	fun1
U+9D05.0	fMandarin	huan1
U+9D06.0	fUTF8	鴆
U+9D06.0	fCNS11643	1-7028
U+9D06.0	fBig5	BE7D
U+9D06.0	fGB	1-7032
U+9D06.0	fR/S	 196.4
U+9D06.0	fTotalStrokes	 15
U+9D06.0	fCangjie	LUHAF
U+9D06.0	fFourCorner	47027
U+9D06.0	fHakka	 dam1; chin3; cim3; cihn3; cim2; chim2
U+9D06.0	fCantonese	zam6
U+9D06.0	fMandarin	zhen4; chen2; tan2
U+9D06.0	fEnglish	[1] a kind of venomous bird [2] poisoned wine
U+9D07.0	fUTF8	鴇
U+9D07.0	fCNS11643	2-5634
U+9D07.0	fBig5	E8E9
U+9D07.0	fGB	1-7031
U+9D07.0	fR/S	 196.4
U+9D07.0	fTotalStrokes	 15
U+9D07.0	fCangjie	PJHAF
U+9D07.0	fFourCorner	47427
U+9D07.0	fHakka	 bau3;  bo3
U+9D07.0	fCantonese	bou2
U+9D07.0	fMandarin	bao3
U+9D07.0	fEnglish	[1] Otis dybowskii, a bird resembling the wild goose [2] prostitute [3] procuress
U+9D08.0	fUTF8	鴈
U+9D08.0	fCNS11643	2-5638
U+9D08.0	fBig5	E8ED
U+9D08.0	fGB	E-4759
U+9D08.0	fR/S	 196.4
U+9D08.0	fTotalStrokes	 15
U+9D08.0	fCangjie	MOHF
U+9D08.0	fFourCorner	71227
U+9D08.0	fHakka	  ngan5; ngan6; ngien5; ngien6; ngian5
U+9D08.0	fCantonese	ngaan6; aan6 (also)
U+9D08.0	fMandarin	yan4
U+9D08.0	fEnglish	[1] same as $ [2] same ase @
U+9D09.0	fUTF8	鴉
U+9D09.0	fCNS11643	1-7029
U+9D09.0	fBig5	BE7E
U+9D09.0	fGB	1-513B
U+9D09.0	fR/S	 196.4
U+9D09.0	fTotalStrokes	 15
U+9D09.0	fCangjie	MHHAF
U+9D09.0	fFourCorner	17227
U+9D09.0	fHakka	  a1
U+9D09.0	fCantonese	aa1; ngaa1 (also)
U+9D09.0	fMandarin	ya1; ya4; ya3
U+9D09.0	fEnglish	 crow; raven
U+9D0A.0	fUTF8	鴊
U+9D0A.0	fCNS11643	3-542D
U+9D0A.0	fGB	3-5D29
U+9D0A.0	fR/S	 196.5
U+9D0A.0	fTotalStrokes	 16
U+9D0A.0	fCangjie	MMHAF
U+9D0A.0	fFourCorner	17120
U+9D0A.0	fCantonese	zing3
U+9D0A.0	fMandarin	zheng4
U+9D0B.0	fUTF8	鴋
U+9D0B.0	fCNS11643	4-4F42
U+9D0B.0	fGB	3-4F42
U+9D0B.0	fR/S	 196.4
U+9D0B.0	fTotalStrokes	 15
U+9D0B.0	fCangjie	YSHAF
U+9D0B.0	fFourCorner	07220
U+9D0B.0	fCantonese	fong1
U+9D0B.0	fMandarin	fang1
U+9D0C.0	fUTF8	鴌
U+9D0C.0	fCNS11643	3-5025
U+9D0C.0	fGB	E-475A
U+9D0C.0	fR/S	 196.4
U+9D0C.0	fTotalStrokes	 15
U+9D0C.0	fCangjie	MKHAF
U+9D0C.0	fFourCorner	20320
U+9D0C.0	fCantonese	fung6
U+9D0C.0	fMandarin	feng4
U+9D0D.0	fUTF8	鴍
U+9D0D.0	fCNS11643	3-5022
U+9D0D.0	fGB	5-5078
U+9D0D.0	fR/S	 196.4
U+9D0D.0	fTotalStrokes	 15
U+9D0D.0	fCangjie	YKHAF
U+9D0D.0	fFourCorner	00320
U+9D0D.0	fCantonese	man4
U+9D0D.0	fMandarin	wen2
U+9D0E.0	fUTF8	鴎
U+9D0E.0	fGB	E-475B
U+9D0E.0	fR/S	 196.4
U+9D0E.0	fTotalStrokes	 15
U+9D0E.0	fCangjie	SKHAF
U+9D0E.0	fCantonese	au1; ngau1 (also)
U+9D0E.0	fMandarin	ou1
U+9D0E.0	fEnglish	seagull; tern
U+9D0F.0	fUTF8	鴏
U+9D0F.0	fCNS11643	3-542A
U+9D0F.0	fGB	E-475C
U+9D0F.0	fR/S	 196.5
U+9D0F.0	fTotalStrokes	 16
U+9D0F.0	fCangjie	HFOIP
U+9D0F.0	fFourCorner	23340
U+9D0F.0	fCantonese	dak6
U+9D0F.0	fMandarin	te4
U+9D10.0	fUTF8	鴐
U+9D10.0	fCNS11643	2-5C38
U+9D10.0	fBig5	ECAC
U+9D10.0	fGB	5-5C28
U+9D10.0	fR/S	 196.5
U+9D10.0	fTotalStrokes	 16
U+9D10.0	fCangjie	KRHAF
U+9D10.0	fFourCorner	46327
U+9D10.0	fCantonese	gaa1
U+9D10.0	fMandarin	jia1
U+9D11.0	fUTF8	鴑
U+9D11.0	fCNS11643	4-5552
U+9D11.0	fGB	E-475D
U+9D11.0	fR/S	 196.5
U+9D11.0	fTotalStrokes	 16
U+9D11.0	fCangjie	VEHAF
U+9D11.0	fFourCorner	47320
U+9D11.0	fCantonese	jyu4
U+9D11.0	fMandarin	nu2; ru2
U+9D12.0	fUTF8	鴒
U+9D12.0	fCNS11643	1-733A
U+9D12.0	fBig5	C06F
U+9D12.0	fGB	3-5D32
U+9D12.0	fR/S	 196.5
U+9D12.0	fTotalStrokes	 16
U+9D12.0	fCangjie	OIHAF
U+9D12.0	fFourCorner	87327
U+9D12.0	fHakka	  lin2
U+9D12.0	fCantonese	ling4
U+9D12.0	fMandarin	ling2
U+9D12.0	fEnglish	 wagtail
U+9D13.0	fUTF8	鴓
U+9D13.0	fCNS11643	3-542C
U+9D13.0	fGB	3-5D38
U+9D13.0	fR/S	 196.5
U+9D13.0	fTotalStrokes	 16
U+9D13.0	fCangjie	PHHAF
U+9D13.0	fFourCorner	37327
U+9D13.0	fCantonese	bat6; mit6 (also); mat6 (also)
U+9D13.0	fMandarin	mie4
U+9D14.0	fUTF8	鴔
U+9D14.0	fCNS11643	2-5C33
U+9D14.0	fBig5	ECA7
U+9D14.0	fGB	3-5D24
U+9D14.0	fR/S	 196.5
U+9D14.0	fTotalStrokes	 16
U+9D14.0	fCangjie	HOHAF
U+9D14.0	fFourCorner	27727
U+9D14.0	fCantonese	fuk6
U+9D14.0	fMandarin	fu2; ji2
U+9D15.0	fUTF8	鴕
U+9D15.0	fCNS11643	1-7336
U+9D15.0	fBig5	C06B
U+9D15.0	fGB	1-4D52
U+9D15.0	fR/S	 196.5
U+9D15.0	fTotalStrokes	 16
U+9D15.0	fCangjie	HFJP
U+9D15.0	fFourCorner	23311
U+9D15.0	fHakka	 to2
U+9D15.0	fCantonese	to4
U+9D15.0	fMandarin	tuo2
U+9D15.0	fEnglish	 ostrich
U+9D16.0	fUTF8	鴖
U+9D16.0	fCNS11643	3-5436
U+9D16.0	fGB	3-5D39
U+9D16.0	fR/S	 196.5
U+9D16.0	fTotalStrokes	 16
U+9D16.0	fCangjie	RPHAF
U+9D16.0	fFourCorner	77727
U+9D16.0	fCantonese	man4
U+9D16.0	fMandarin	wen2
U+9D17.0	fUTF8	鴗
U+9D17.0	fCNS11643	2-5C30
U+9D17.0	fBig5	ECA4
U+9D17.0	fGB	3-5F4D
U+9D17.0	fR/S	 196.5
U+9D17.0	fTotalStrokes	 16
U+9D17.0	fCangjie	YTHAF
U+9D17.0	fFourCorner	07127
U+9D17.0	fHakka	 lip8
U+9D17.0	fCantonese	lap6 (also); laap6
U+9D17.0	fMandarin	li4
U+9D17.0	fEnglish	a small species of king-fisher
U+9D18.0	fUTF8	鴘
U+9D18.0	fCNS11643	2-5C36
U+9D18.0	fBig5	ECAA
U+9D18.0	fGB	3-5D3B
U+9D18.0	fR/S	 196.5
U+9D18.0	fTotalStrokes	 16
U+9D18.0	fCangjie	ITHAF
U+9D18.0	fFourCorner	27427
U+9D18.0	fHakka	 pien3
U+9D18.0	fCantonese	bin2; bin6; bing3
U+9D18.0	fMandarin	bian4
U+9D18.0	fEnglish	 falcon
U+9D19.0	fUTF8	鴙
U+9D19.0	fCNS11643	2-5C39
U+9D19.0	fBig5	ECAD
U+9D19.0	fGB	5-5A6F
U+9D19.0	fR/S	 196.5
U+9D19.0	fTotalStrokes	 16
U+9D19.0	fCangjie	OKHAF
U+9D19.0	fFourCorner	87427
U+9D19.0	fHakka	 chii1; chi1
U+9D19.0	fCantonese	zi6
U+9D19.0	fMandarin	zhi4
U+9D1A.0	fUTF8	鴚
U+9D1A.0	fCNS11643	3-5433
U+9D1A.0	fGB	3-5D2B
U+9D1A.0	fR/S	 196.5
U+9D1A.0	fTotalStrokes	 16
U+9D1A.0	fCangjie	HFMNR
U+9D1A.0	fFourCorner	21320
U+9D1A.0	fCantonese	go1; gaa1 (also)
U+9D1A.0	fMandarin	ge1; jia1
U+9D1B.0	fUTF8	鴛
U+9D1B.0	fCNS11643	1-733B
U+9D1B.0	fBig5	C070
U+9D1B.0	fGB	1-5427
U+9D1B.0	fR/S	 196.5
U+9D1B.0	fTotalStrokes	 16
U+9D1B.0	fCangjie	NUHAF
U+9D1B.0	fFourCorner	27327
U+9D1B.0	fHakka	 jen1; jan1; rong1; jong1
U+9D1B.0	fCantonese	jyun1; jin1 (also)
U+9D1B.0	fMandarin	yuan1; wen1
U+9D1B.0	fEnglish	the male mandarin duck
U+9D1C.0	fUTF8	鴜
U+9D1C.0	fCNS11643	4-5A4E
U+9D1C.0	fGB	E-475E
U+9D1C.0	fR/S	 196.5; 196.6
U+9D1C.0	fTotalStrokes	 16; 17
U+9D1C.0	fCangjie	YPHAF
U+9D1C.0	fFourCorner	21320
U+9D1C.0	fCantonese	ci4
U+9D1C.0	fMandarin	zi1
U+9D1D.0	fUTF8	鴝
U+9D1D.0	fCNS11643	2-5C35
U+9D1D.0	fBig5	ECA9
U+9D1D.0	fGB	1-7036
U+9D1D.0	fR/S	 196.5
U+9D1D.0	fTotalStrokes	 16
U+9D1D.0	fCangjie	PRHAF
U+9D1D.0	fFourCorner	27627
U+9D1D.0	fHakka	 ki1
U+9D1D.0	fCantonese	keoi4
U+9D1D.0	fMandarin	qu2; gou1; gou4
U+9D1D.0	fEnglish	a species of mynah (myna, minah)
U+9D1E.0	fUTF8	鴞
U+9D1E.0	fCNS11643	2-5C32
U+9D1E.0	fBig5	ECA6
U+9D1E.0	fGB	3-5D2F
U+9D1E.0	fR/S	 196.5
U+9D1E.0	fTotalStrokes	 16
U+9D1E.0	fCangjie	RSHAF
U+9D1E.0	fFourCorner	67227
U+9D1E.0	fHakka	  hiau1;  hau1; hieu1
U+9D1E.0	fCantonese	hiu1
U+9D1E.0	fMandarin	xiao1
U+9D1E.0	fEnglish	 owl
U+9D1F.0	fUTF8	鴟
U+9D1F.0	fCNS11643	2-5C3A
U+9D1F.0	fBig5	ECAE
U+9D1F.0	fGB	1-7037
U+9D1F.0	fR/S	 196.5
U+9D1F.0	fTotalStrokes	 16
U+9D1F.0	fCangjie	HMHAF
U+9D1F.0	fFourCorner	77727
U+9D1F.0	fHakka	 chi1; cih1; chii1
U+9D1F.0	fCantonese	ci1
U+9D1F.0	fMandarin	chi1; zhi1; hun1
U+9D1F.0	fEnglish	[1] kite [2] owl [3] wine cups
U+9D20.0	fUTF8	鴠
U+9D20.0	fCNS11643	2-5C31
U+9D20.0	fBig5	ECA5
U+9D20.0	fGB	3-5D2E
U+9D20.0	fR/S	 196.5
U+9D20.0	fTotalStrokes	 16
U+9D20.0	fCangjie	AMHAF
U+9D20.0	fFourCorner	67127
U+9D20.0	fHakka	  dan5
U+9D20.0	fCantonese	daan3; taan3
U+9D20.0	fMandarin	dan4; dan3
U+9D20.0	fEnglish	a kind of nightingale
U+9D21.0	fUTF8	鴡
U+9D21.0	fCNS11643	3-5437
U+9D21.0	fGB	5-5C23
U+9D21.0	fR/S	 196.5
U+9D21.0	fTotalStrokes	 16
U+9D21.0	fCangjie	BMHAF
U+9D21.0	fFourCorner	77120
U+9D21.0	fCantonese	zeoi1
U+9D21.0	fMandarin	ju1
U+9D22.0	fUTF8	鴢
U+9D22.0	fCNS11643	2-5C37
U+9D22.0	fBig5	ECAB
U+9D22.0	fGB	3-5D3C
U+9D22.0	fR/S	 196.5
U+9D22.0	fTotalStrokes	 16
U+9D22.0	fCangjie	VIKSF
U+9D22.0	fFourCorner	27727
U+9D22.0	fCantonese	jiu2; aau2
U+9D22.0	fMandarin	you4; ao1; ao3
U+9D23.0	fUTF8	鴣
U+9D23.0	fCNS11643	1-7337
U+9D23.0	fBig5	C06C
U+9D23.0	fGB	1-7033
U+9D23.0	fR/S	 196.5
U+9D23.0	fTotalStrokes	 16
U+9D23.0	fCangjie	JRHAF
U+9D23.0	fFourCorner	47627
U+9D23.0	fHakka	  gu1
U+9D23.0	fCantonese	gu1; gwu1 (also)
U+9D23.0	fMandarin	gu1
U+9D23.0	fEnglish	a species of pigeon
U+9D24.0	fUTF8	鴤
U+9D24.0	fCNS11643	3-5429
U+9D24.0	fGB	5-5C25
U+9D24.0	fR/S	  196.5
U+9D24.0	fTotalStrokes	 16
U+9D24.0	fCangjie	HFHEY
U+9D24.0	fFourCorner	27330
U+9D24.0	fCantonese	zung1
U+9D24.0	fMandarin	zhong1
U+9D25.0	fUTF8	鴥
U+9D25.0	fCNS11643	2-5C2F
U+9D25.0	fBig5	ECA3
U+9D25.0	fGB	3-5D37
U+9D25.0	fR/S	 196.5
U+9D25.0	fTotalStrokes	 16
U+9D25.0	fCangjie	HFJC
U+9D25.0	fFourCorner	37827
U+9D25.0	fHakka	 lut8
U+9D25.0	fCantonese	wat6; jyut6
U+9D25.0	fMandarin	yu4; xue2; yue1
U+9D25.0	fEnglish	flaying rapidly; flitting
U+9D26.0	fUTF8	鴦
U+9D26.0	fCNS11643	1-7338
U+9D26.0	fBig5	C06D
U+9D26.0	fGB	1-516C
U+9D26.0	fR/S	 196.5
U+9D26.0	fTotalStrokes	 16
U+9D26.0	fCangjie	LKHAF
U+9D26.0	fFourCorner	50327
U+9D26.0	fHakka	  jong1; ren1; jen1
U+9D26.0	fCantonese	joeng1
U+9D26.0	fMandarin	yang1
U+9D26.0	fEnglish	the female mandarin duck
U+9D27.0	fUTF8	鴧
U+9D27.0	fCNS11643	3-5435
U+9D27.0	fGB	E-475F
U+9D27.0	fR/S	 196.5
U+9D27.0	fTotalStrokes	 16
U+9D27.0	fCangjie	JUHAF
U+9D27.0	fFourCorner	37220
U+9D27.0	fCantonese	wat6
U+9D27.0	fMandarin	rong4
U+9D28.0	fUTF8	鴨
U+9D28.0	fCNS11643	1-7339
U+9D28.0	fBig5	C06E
U+9D28.0	fGB	1-513C
U+9D28.0	fR/S	 196.5
U+9D28.0	fTotalStrokes	 16
U+9D28.0	fCangjie	WLHAF
U+9D28.0	fFourCorner	67527
U+9D28.0	fHakka	  ap7; a1
U+9D28.0	fCantonese	aap3; ngaap3 (also); aap2 (also); ngaap2 (also)
U+9D28.0	fMandarin	ya1
U+9D28.0	fEnglish	 duck
U+9D29.0	fUTF8	鴩
U+9D29.0	fCNS11643	2-5C34
U+9D29.0	fBig5	ECA8
U+9D29.0	fGB	E-4760
U+9D29.0	fR/S	 196.5
U+9D29.0	fTotalStrokes	 16
U+9D29.0	fCangjie	HOHAF
U+9D29.0	fFourCorner	27527
U+9D29.0	fCantonese	tit3
U+9D29.0	fMandarin	tie3; zhi4; die2; hu2; yi4
U+9D2A.0	fUTF8	鴪
U+9D2A.0	fCNS11643	3-5432
U+9D2A.0	fGB	E-4761
U+9D2A.0	fR/S	 196.5
U+9D2A.0	fTotalStrokes	 16
U+9D2A.0	fCangjie	JCHAF
U+9D2A.0	fFourCorner	37820
U+9D2A.0	fCantonese	wat6
U+9D2A.0	fMandarin	yu4
U+9D2A.0	fEnglish	swoop
U+9D2B.0	fUTF8	鴫
U+9D2B.0	fGB	E-4762
U+9D2B.0	fR/S	 196.5
U+9D2B.0	fTotalStrokes	 16
U+9D2B.0	fCangjie	WHAF
U+9D2B.0	fCantonese	tin4
U+9D2B.0	fEnglish	 a snipe
U+9D2C.0	fUTF8	鴬
U+9D2C.0	fGB	E-4763
U+9D2C.0	fR/S	 196.5
U+9D2C.0	fTotalStrokes	 16
U+9D2C.0	fCangjie	XFBHA
U+9D2C.0	fCantonese	ang1; ngang1 (also)
U+9D2C.0	fMandarin	ying1
U+9D2C.0	fEnglish	oriole; green finch
U+9D2D.0	fUTF8	鴭
U+9D2D.0	fCNS11643	2-6135
U+9D2D.0	fBig5	EFA9
U+9D2D.0	fGB	E-4764
U+9D2D.0	fR/S	 196.6
U+9D2D.0	fTotalStrokes	 17
U+9D2D.0	fCangjie	HRHAF
U+9D2D.0	fFourCorner	27727
U+9D2D.0	fCantonese	deoi1
U+9D2D.0	fMandarin	zhui1
U+9D2E.0	fUTF8	鴮
U+9D2E.0	fCNS11643	2-6128
U+9D2E.0	fBig5	EF7A
U+9D2E.0	fGB	E-4765
U+9D2E.0	fR/S	 196.6
U+9D2E.0	fTotalStrokes	 17
U+9D2E.0	fCangjie	KSHAF
U+9D2E.0	fFourCorner	47227
U+9D2E.0	fHakka	 wu1; wu5; vu1
U+9D2E.0	fCantonese	wu1; kwaa1
U+9D2E.0	fMandarin	wu1; wu4
U+9D2E.0	fEnglish	a bird which scrapes for fish in marshy ground
U+9D2F.0	fUTF8	鴯
U+9D2F.0	fCNS11643	2-6129
U+9D2F.0	fBig5	EF7B
U+9D2F.0	fGB	1-7039
U+9D2F.0	fR/S	 196.6
U+9D2F.0	fTotalStrokes	 17
U+9D2F.0	fCangjie	MBHAF
U+9D2F.0	fFourCorner	17227
U+9D2F.0	fCantonese	ji4
U+9D2F.0	fMandarin	er2
U+9D2F.0	fEnglish	swallow
U+9D30.0	fUTF8	鴰
U+9D30.0	fCNS11643	2-612C
U+9D30.0	fBig5	EF7E
U+9D30.0	fGB	1-703B
U+9D30.0	fR/S	 196.6
U+9D30.0	fTotalStrokes	 17
U+9D30.0	fCangjie	HRHAF
U+9D30.0	fFourCorner	27627
U+9D30.0	fHakka	 kwat7; gat7
U+9D30.0	fCantonese	kut3; kwut3 (also); gwaat3 (also)
U+9D30.0	fMandarin	gua1
U+9D30.0	fEnglish	the crow
U+9D31.0	fUTF8	鴱
U+9D31.0	fCNS11643	2-612A
U+9D31.0	fBig5	EF7C
U+9D31.0	fGB	3-5D2A
U+9D31.0	fR/S	 196.6
U+9D31.0	fTotalStrokes	 17
U+9D31.0	fCangjie	TKHAF
U+9D31.0	fFourCorner	47427
U+9D31.0	fHakka	 ngai5; ngoi6; ngoi5; ngai6
U+9D31.0	fCantonese	ngaai6; aai6 (also); ngaai3
U+9D31.0	fMandarin	ai4; yi4
U+9D31.0	fEnglish	the hen of the tailor-bird
U+9D32.0	fUTF8	鴲
U+9D32.0	fCNS11643	4-5A55
U+9D32.0	fGB	3-5D45
U+9D32.0	fR/S	  196.6
U+9D32.0	fTotalStrokes	 17; 16
U+9D32.0	fCangjie	PAHAF
U+9D32.0	fFourCorner	27620
U+9D32.0	fCantonese	zi1
U+9D32.0	fMandarin	zhi1
U+9D33.0	fUTF8	鴳
U+9D33.0	fCNS11643	2-6124
U+9D33.0	fBig5	EF76
U+9D33.0	fGB	5-5C34
U+9D33.0	fR/S	 196.6
U+9D33.0	fTotalStrokes	 17
U+9D33.0	fCangjie	JVHAF
U+9D33.0	fFourCorner	37427
U+9D33.0	fHakka	 on5
U+9D33.0	fCantonese	aan3; ngaan3 (also)
U+9D33.0	fMandarin	yan4; an1
U+9D33.0	fEnglish	a species of quail
U+9D34.0	fUTF8	鴴
U+9D34.0	fCNS11643	4-5A56
U+9D34.0	fGB	3-5D44
U+9D34.0	fR/S	  196.6
U+9D34.0	fTotalStrokes	 17; 16
U+9D34.0	fCangjie	HOMNF
U+9D34.0	fFourCorner	27220
U+9D34.0	fCantonese	hang4; hong4 (also)
U+9D34.0	fMandarin	heng2; hang2
U+9D35.0	fUTF8	鴵
U+9D35.0	fCNS11643	3-574D
U+9D35.0	fGB	E-4766
U+9D35.0	fR/S	 196.6
U+9D35.0	fTotalStrokes	 17
U+9D35.0	fCangjie	LMUOF
U+9D35.0	fFourCorner	37120
U+9D35.0	fCantonese	giu1
U+9D35.0	fMandarin	jiao1
U+9D36.0	fUTF8	鴶
U+9D36.0	fCNS11643	2-6127
U+9D36.0	fBig5	EF79
U+9D36.0	fGB	E-4767
U+9D36.0	fR/S	 196.6
U+9D36.0	fTotalStrokes	 17
U+9D36.0	fCangjie	GRHAF
U+9D36.0	fFourCorner	47627
U+9D36.0	fCantonese	gat1; git3
U+9D36.0	fMandarin	ji2; jia4
U+9D37.0	fUTF8	鴷
U+9D37.0	fCNS11643	2-6131
U+9D37.0	fBig5	EFA5
U+9D37.0	fGB	3-5D3F
U+9D37.0	fR/S	 196.6
U+9D37.0	fTotalStrokes	 17
U+9D37.0	fCangjie	MNHAF
U+9D37.0	fFourCorner	12327
U+9D37.0	fHakka	  liet8
U+9D37.0	fCantonese	lit6
U+9D37.0	fMandarin	lie4
U+9D37.0	fEnglish	 woodpecker
U+9D38.0	fUTF8	鴸
U+9D38.0	fCNS11643	2-612B
U+9D38.0	fBig5	EF7D
U+9D38.0	fGB	E-4768
U+9D38.0	fR/S	 196.6
U+9D38.0	fTotalStrokes	 17
U+9D38.0	fCangjie	HDHAF
U+9D38.0	fFourCorner	27927
U+9D38.0	fCantonese	zyu1
U+9D38.0	fMandarin	zhu1
U+9D39.0	fUTF8	鴹
U+9D39.0	fCNS11643	3-5748
U+9D39.0	fGB	5-6644
U+9D39.0	fR/S	 196.6
U+9D39.0	fTotalStrokes	 17
U+9D39.0	fCangjie	TQHAF
U+9D39.0	fFourCorner	87520
U+9D39.0	fCantonese	joeng4
U+9D39.0	fMandarin	ren2
U+9D3A.0	fUTF8	鴺
U+9D3A.0	fCNS11643	4-5A4F
U+9D3A.0	fGB	5-5C2D
U+9D3A.0	fR/S	 196.6
U+9D3A.0	fTotalStrokes	 17; 16
U+9D3A.0	fCangjie	KNHAF
U+9D3A.0	fFourCorner	57020
U+9D3A.0	fCantonese	ji4; tai4
U+9D3A.0	fMandarin	yi2; ti2
U+9D3A.0	fEnglish	the pelican
U+9D3B.0	fUTF8	鴻
U+9D3B.0	fCNS11643	1-7631
U+9D3B.0	fBig5	C245
U+9D3B.0	fGB	1-3A68
U+9D3B.0	fR/S	 196.6
U+9D3B.0	fTotalStrokes	 17
U+9D3B.0	fCangjie	EMHF
U+9D3B.0	fFourCorner	37127
U+9D3B.0	fHakka	  fung2;  pung2; fung3
U+9D3B.0	fCantonese	hung4
U+9D3B.0	fMandarin	hong2; gong3; hong4; hong3
U+9D3B.0	fEnglish	[1] wild swan; wild goose [2] vast; great; huge; large
U+9D3C.0	fUTF8	鴼
U+9D3C.0	fCNS11643	4-5A4D
U+9D3C.0	fGB	5-5C31
U+9D3C.0	fR/S	 196.6
U+9D3C.0	fTotalStrokes	 17; 16
U+9D3C.0	fCangjie	HFHER
U+9D3C.0	fFourCorner	27360
U+9D3C.0	fCantonese	lou6
U+9D3C.0	fMandarin	luo4
U+9D3D.0	fUTF8	鴽
U+9D3D.0	fCNS11643	2-6133
U+9D3D.0	fBig5	EFA7
U+9D3D.0	fGB	3-5D48
U+9D3D.0	fR/S	 196.6
U+9D3D.0	fTotalStrokes	 17
U+9D3D.0	fCangjie	VRHAF
U+9D3D.0	fFourCorner	46327
U+9D3D.0	fHakka	 ji2; ri2; i2
U+9D3D.0	fCantonese	jyu4
U+9D3D.0	fMandarin	ru2
U+9D3D.0	fEnglish	Trunix blakistoni, a kind of quail
U+9D3E.0	fUTF8	鴾
U+9D3E.0	fCNS11643	2-6130
U+9D3E.0	fBig5	EFA4
U+9D3E.0	fGB	E-4769
U+9D3E.0	fR/S	 196.6
U+9D3E.0	fTotalStrokes	 17
U+9D3E.0	fCangjie	IQHAF
U+9D3E.0	fFourCorner	27527
U+9D3E.0	fCantonese	mau4
U+9D3E.0	fMandarin	mou2
U+9D3E.0	fEnglish	a crested ibis
U+9D3F.0	fUTF8	鴿
U+9D3F.0	fCNS11643	1-7632
U+9D3F.0	fBig5	C246
U+9D3F.0	fGB	1-386B
U+9D3F.0	fR/S	 196.6
U+9D3F.0	fTotalStrokes	 17
U+9D3F.0	fCangjie	ORHAF
U+9D3F.0	fFourCorner	87627
U+9D3F.0	fHakka	  gap7
U+9D3F.0	fCantonese	gap3; gaap3 (also); gap2 (also)
U+9D3F.0	fMandarin	ge1
U+9D3F.0	fEnglish	 pigeon; dove
U+9D40.0	fUTF8	鵀
U+9D40.0	fCNS11643	2-6132
U+9D40.0	fBig5	EFA6
U+9D40.0	fGB	3-5D43
U+9D40.0	fR/S	 196.6
U+9D40.0	fTotalStrokes	 17
U+9D40.0	fCangjie	OGHAF
U+9D40.0	fFourCorner	22327
U+9D40.0	fHakka	 ngim5; ngim6
U+9D40.0	fCantonese	jam4; jam6 (also)
U+9D40.0	fMandarin	ren4; ren2
U+9D40.0	fEnglish	hoopoe
U+9D41.0	fUTF8	鵁
U+9D41.0	fCNS11643	2-6125
U+9D41.0	fBig5	EF77
U+9D41.0	fGB	3-5D47
U+9D41.0	fR/S	 196.6
U+9D41.0	fTotalStrokes	 17
U+9D41.0	fCangjie	YKHAF
U+9D41.0	fFourCorner	07427
U+9D41.0	fHakka	  gau1
U+9D41.0	fCantonese	gaau1
U+9D41.0	fMandarin	jiao1; wen2
U+9D41.0	fEnglish	[1] Nycticorax prasinosceles, a kind of water bird [2] a long narrow boat
U+9D42.0	fUTF8	鵂
U+9D42.0	fCNS11643	2-612E
U+9D42.0	fBig5	EFA2
U+9D42.0	fGB	1-703C
U+9D42.0	fR/S	 196.6
U+9D42.0	fTotalStrokes	 17
U+9D42.0	fCangjie	ODHF
U+9D42.0	fFourCorner	27227
U+9D42.0	fHakka	 hiu1
U+9D42.0	fCantonese	jau1
U+9D42.0	fMandarin	xiu1
U+9D42.0	fEnglish	 owl
U+9D43.0	fUTF8	鵃
U+9D43.0	fCNS11643	2-612F
U+9D43.0	fBig5	EFA3
U+9D43.0	fGB	3-655B
U+9D43.0	fR/S	 196.6
U+9D43.0	fTotalStrokes	 17
U+9D43.0	fCangjie	HYHAF
U+9D43.0	fFourCorner	27427
U+9D43.0	fHakka	 zhiu1; zu1
U+9D43.0	fCantonese	zau1; zaau1
U+9D43.0	fMandarin	zhou1; zhao1; diao1
U+9D43.0	fEnglish	correct form of &#xTreron permangna, a kind of pigeon;
U+9D44.0	fUTF8	鵄
U+9D44.0	fCNS11643	3-574A
U+9D44.0	fGB	5-604A
U+9D44.0	fR/S	 196.6
U+9D44.0	fTotalStrokes	 17
U+9D44.0	fCangjie	MGHAF
U+9D44.0	fFourCorner	17120
U+9D44.0	fCantonese	ci1
U+9D44.0	fMandarin	zhi1; chi1
U+9D44.0	fEnglish	 kite; horned owl wine cups
U+9D45.0	fUTF8	鵅
U+9D45.0	fCNS11643	2-612D
U+9D45.0	fBig5	EFA1
U+9D45.0	fGB	3-5D46
U+9D45.0	fR/S	 196.6
U+9D45.0	fTotalStrokes	 17
U+9D45.0	fCangjie	HRHAF
U+9D45.0	fFourCorner	27627
U+9D45.0	fHakka	 lok8
U+9D45.0	fCantonese	lok6; gaak3
U+9D45.0	fMandarin	luo4
U+9D46.0	fUTF8	鵆
U+9D46.0	fGB	E-476A
U+9D46.0	fR/S	 196.6
U+9D46.0	fTotalStrokes	 17
U+9D46.0	fCangjie	HOHFN
U+9D46.0	fCantonese	hang4
U+9D47.0	fUTF8	鵇
U+9D47.0	fCNS11643	F-5E24
U+9D47.0	fGB	E-476B
U+9D47.0	fR/S	 196.6
U+9D47.0	fTotalStrokes	 17
U+9D47.0	fCangjie	OQHAF
U+9D47.0	fCantonese	nin4; lin4 (also)
U+9D48.0	fUTF8	鵈
U+9D48.0	fGB	E-476C
U+9D48.0	fR/S	 196.6
U+9D48.0	fTotalStrokes	 17
U+9D48.0	fCangjie	XSJHA
U+9D48.0	fCantonese	diu1
U+9D49.0	fUTF8	鵉
U+9D49.0	fCNS11643	3-6675
U+9D49.0	fGB	E-476D
U+9D49.0	fR/S	 196.6
U+9D49.0	fTotalStrokes	 17
U+9D49.0	fCangjie	YCHAF
U+9D49.0	fFourCorner	0032
U+9D49.0	fCantonese	lyun4
U+9D49.0	fMandarin	luan2
U+9D4A.0	fUTF8	鵊
U+9D4A.0	fCNS11643	2-647C
U+9D4A.0	fBig5	F1D2
U+9D4A.0	fGB	3-5D40
U+9D4A.0	fR/S	 196.7
U+9D4A.0	fTotalStrokes	 18
U+9D4A.0	fCangjie	KOHAF
U+9D4A.0	fFourCorner	47027
U+9D4A.0	fHakka	 kiap7; hiap8; kiap8
U+9D4A.0	fCantonese	gaap3
U+9D4A.0	fMandarin	jia2
U+9D4A.0	fEnglish	[1] narrow [2] strait
U+9D4B.0	fUTF8	鵋
U+9D4B.0	fCNS11643	2-647E
U+9D4B.0	fBig5	F1D4
U+9D4B.0	fGB	E-476E
U+9D4B.0	fR/S	 196.7
U+9D4B.0	fTotalStrokes	 18
U+9D4B.0	fCangjie	SPHAF
U+9D4B.0	fFourCorner	17327
U+9D4B.0	fCantonese	gei6
U+9D4B.0	fMandarin	ji4
U+9D4C.0	fUTF8	鵌
U+9D4C.0	fCNS11643	2-6523
U+9D4C.0	fBig5	F1D7
U+9D4C.0	fGB	3-5D54
U+9D4C.0	fR/S	 196.7
U+9D4C.0	fTotalStrokes	 18
U+9D4C.0	fCangjie	HFOMD
U+9D4C.0	fFourCorner	28394
U+9D4C.0	fHakka	 tu2
U+9D4C.0	fCantonese	tou4
U+9D4C.0	fMandarin	yu2; tu2
U+9D4C.0	fEnglish	a kind of bird which shares its nest iwth rats
U+9D4D.0	fUTF8	鵍
U+9D4D.0	fCNS11643	3-5979
U+9D4D.0	fGB	3-5D5A
U+9D4D.0	fR/S	 196.7
U+9D4D.0	fTotalStrokes	 18
U+9D4D.0	fCangjie	JUHAF
U+9D4D.0	fFourCorner	37220
U+9D4D.0	fCantonese	fun1
U+9D4D.0	fMandarin	huan1
U+9D4E.0	fUTF8	鵎
U+9D4E.0	fCNS11643	3-597B
U+9D4E.0	fGB	3-5D57
U+9D4E.0	fR/S	 196.7
U+9D4E.0	fTotalStrokes	 18
U+9D4E.0	fCangjie	BVHAF
U+9D4E.0	fFourCorner	27420
U+9D4E.0	fCantonese	to2
U+9D4E.0	fMandarin	tuo3
U+9D4F.0	fUTF8	鵏
U+9D4F.0	fCNS11643	2-647B
U+9D4F.0	fBig5	F1D1
U+9D4F.0	fGB	3-5D4C
U+9D4F.0	fR/S	 196.7
U+9D4F.0	fTotalStrokes	 18
U+9D4F.0	fCangjie	IBHAF
U+9D4F.0	fFourCorner	57227
U+9D4F.0	fCantonese	bou2; bou6
U+9D4F.0	fMandarin	bu1; pu2; bu3; pu4
U+9D50.0	fUTF8	鵐
U+9D50.0	fCNS11643	3-597D
U+9D50.0	fGB	3-5D4A
U+9D50.0	fR/S	 196.7
U+9D50.0	fTotalStrokes	 18
U+9D50.0	fCangjie	MOHAF
U+9D50.0	fFourCorner	17120
U+9D50.0	fCantonese	mou4
U+9D50.0	fMandarin	wu2
U+9D51.0	fUTF8	鵑
U+9D51.0	fCNS11643	1-7825
U+9D51.0	fBig5	C359
U+9D51.0	fGB	1-3E69
U+9D51.0	fR/S	 196.7
U+9D51.0	fTotalStrokes	 18
U+9D51.0	fCangjie	RBHAF
U+9D51.0	fFourCorner	67227
U+9D51.0	fHakka	 gien1; gen1; gian1
U+9D51.0	fCantonese	gyun1
U+9D51.0	fMandarin	juan1
U+9D51.0	fEnglish	[1] correct form of &#x [2] the cuckoo [3] azelea;
U+9D52.0	fUTF8	鵒
U+9D52.0	fCNS11643	2-6525
U+9D52.0	fBig5	F1D9
U+9D52.0	fGB	1-7041
U+9D52.0	fR/S	 196.7
U+9D52.0	fTotalStrokes	 18
U+9D52.0	fCangjie	CRHAF
U+9D52.0	fFourCorner	87627
U+9D52.0	fHakka	 guk7; ruk7; juk7; juk8
U+9D52.0	fCantonese	juk6
U+9D52.0	fMandarin	yu4; ji4
U+9D52.0	fEnglish	correct form of &#x mynah (mina, myna) bird;
U+9D53.0	fUTF8	鵓
U+9D53.0	fCNS11643	2-647A
U+9D53.0	fBig5	F1D0
U+9D53.0	fGB	1-703E
U+9D53.0	fR/S	 196.7
U+9D53.0	fTotalStrokes	 18
U+9D53.0	fCangjie	JDHAF
U+9D53.0	fFourCorner	47427
U+9D53.0	fHakka	  put8; put7
U+9D53.0	fCantonese	but6
U+9D53.0	fMandarin	bo2
U+9D53.0	fEnglish	a species of pigeon
U+9D54.0	fUTF8	鵔
U+9D54.0	fCNS11643	2-6526
U+9D54.0	fBig5	F1DA
U+9D54.0	fGB	3-5D5B
U+9D54.0	fR/S	 196.7
U+9D54.0	fTotalStrokes	 18
U+9D54.0	fCangjie	HFICE
U+9D54.0	fFourCorner	23347
U+9D54.0	fHakka	 zun5
U+9D54.0	fCantonese	zeon3; seon3
U+9D54.0	fMandarin	xun4; jun4
U+9D55.0	fUTF8	鵕
U+9D55.0	fCNS11643	3-597E
U+9D55.0	fGB	5-5C3A
U+9D55.0	fR/S	  196.7
U+9D55.0	fTotalStrokes	 18
U+9D55.0	fCangjie	IEHAF
U+9D55.0	fFourCorner	27420
U+9D55.0	fCantonese	zeon3
U+9D55.0	fMandarin	xun4
U+9D56.0	fUTF8	鵖
U+9D56.0	fCNS11643	2-6522
U+9D56.0	fBig5	F1D6
U+9D56.0	fGB	3-5D53
U+9D56.0	fR/S	 196.7
U+9D56.0	fTotalStrokes	 18
U+9D56.0	fCangjie	HPHAF
U+9D56.0	fFourCorner	27127
U+9D56.0	fCantonese	gip3
U+9D56.0	fMandarin	bi4; jie2; bi1
U+9D57.0	fUTF8	鵗
U+9D57.0	fCNS11643	2-6524
U+9D57.0	fBig5	F1D8
U+9D57.0	fGB	3-5D56
U+9D57.0	fR/S	 196.7
U+9D57.0	fTotalStrokes	 18
U+9D57.0	fCangjie	KBHAF
U+9D57.0	fFourCorner	47427
U+9D57.0	fHakka	 hi1
U+9D57.0	fCantonese	hei1
U+9D57.0	fMandarin	xi1; ji1
U+9D58.0	fUTF8	鵘
U+9D58.0	fCNS11643	2-6528
U+9D58.0	fBig5	F1DC
U+9D58.0	fGB	E-476F
U+9D58.0	fR/S	 196.7
U+9D58.0	fTotalStrokes	 18
U+9D58.0	fCangjie	SRHAF
U+9D58.0	fFourCorner	17627
U+9D58.0	fCantonese	gwan6
U+9D58.0	fMandarin	jun4; jun3
U+9D59.0	fUTF8	鵙
U+9D59.0	fCNS11643	2-6521
U+9D59.0	fBig5	F1D5
U+9D59.0	fGB	5-4A46
U+9D59.0	fR/S	 196.7
U+9D59.0	fTotalStrokes	 18
U+9D59.0	fCangjie	BCHAF
U+9D59.0	fFourCorner	67827
U+9D59.0	fHakka	 giet7; get7
U+9D59.0	fCantonese	gwik1; kyut3
U+9D59.0	fMandarin	ju2; jue2
U+9D59.0	fEnglish	a shrike
U+9D5A.0	fUTF8	鵚
U+9D5A.0	fCNS11643	2-6529
U+9D5A.0	fBig5	F1DD
U+9D5A.0	fGB	E-4770
U+9D5A.0	fR/S	 196.7
U+9D5A.0	fTotalStrokes	 18
U+9D5A.0	fCangjie	HUHAF
U+9D5A.0	fFourCorner	27227
U+9D5A.0	fHakka	  tut7
U+9D5A.0	fCantonese	tuk1
U+9D5A.0	fMandarin	tu2; tu1
U+9D5A.0	fEnglish	[1] a kind of water bird [2] a bird moulting
U+9D5B.0	fUTF8	鵛
U+9D5B.0	fCNS11643	2-647D
U+9D5B.0	fBig5	F1D3
U+9D5B.0	fGB	3-5D3A
U+9D5B.0	fR/S	 196.7
U+9D5B.0	fTotalStrokes	 18
U+9D5B.0	fCangjie	MMHAF
U+9D5B.0	fFourCorner	17127
U+9D5B.0	fCantonese	ging1
U+9D5B.0	fMandarin	jing1
U+9D5C.0	fUTF8	鵜
U+9D5C.0	fCNS11643	2-6479
U+9D5C.0	fBig5	F1CF
U+9D5C.0	fGB	1-7043
U+9D5C.0	fR/S	 196.7
U+9D5C.0	fTotalStrokes	 18
U+9D5C.0	fCangjie	CHHAF
U+9D5C.0	fFourCorner	87227
U+9D5C.0	fHakka	 ti2
U+9D5C.0	fCantonese	tai4
U+9D5C.0	fMandarin	ti2; ti4; ti1
U+9D5C.0	fEnglish	 pelican
U+9D5D.0	fUTF8	鵝
U+9D5D.0	fCNS11643	1-7826
U+9D5D.0	fBig5	C35A
U+9D5D.0	fGB	1-366C
U+9D5D.0	fR/S	 196.7
U+9D5D.0	fTotalStrokes	 18
U+9D5D.0	fCangjie	HIHAF
U+9D5D.0	fFourCorner	27527
U+9D5D.0	fHakka	  ngo2; ngo3
U+9D5D.0	fCantonese	ngo4; ngo2 (also); o4
U+9D5D.0	fMandarin	e2
U+9D5D.0	fEnglish	 goose; gander
U+9D5E.0	fUTF8	鵞
U+9D5E.0	fCNS11643	3-5A22
U+9D5E.0	fGB	E-4771
U+9D5E.0	fR/S	 196.7
U+9D5E.0	fTotalStrokes	 18
U+9D5E.0	fCangjie	HIHAF
U+9D5E.0	fFourCorner	23320
U+9D5E.0	fHakka	 ngo2
U+9D5E.0	fCantonese	ngo4; o4
U+9D5E.0	fMandarin	e2
U+9D5E.0	fEnglish	goose
U+9D5F.0	fUTF8	鵟
U+9D5F.0	fCNS11643	2-6527
U+9D5F.0	fBig5	F1DB
U+9D5F.0	fGB	3-5D59
U+9D5F.0	fR/S	 196.7
U+9D5F.0	fTotalStrokes	 18
U+9D5F.0	fCangjie	KGHAF
U+9D5F.0	fFourCorner	41327
U+9D5F.0	fCantonese	kong4 (also); kwong4
U+9D5F.0	fMandarin	kuang2
U+9D5F.0	fEnglish	buzzard; also called &#x571F;&#x8C79;
U+9D60.0	fUTF8	鵠
U+9D60.0	fCNS11643	1-7827
U+9D60.0	fBig5	C35B
U+9D60.0	fGB	1-7040
U+9D60.0	fR/S	 196.7
U+9D60.0	fTotalStrokes	 18
U+9D60.0	fCangjie	HRHAF
U+9D60.0	fFourCorner	27627
U+9D60.0	fHakka	  guk7; kwuk7
U+9D60.0	fCantonese	huk6; guk1; huk1
U+9D60.0	fMandarin	hu2; ge3; he4
U+9D60.0	fEnglish	[1] swan [2] standing erect; standing quietly
U+9D60.1	fUTF8	鵠
U+9D60.1	fCNS11643	1-7827
U+9D60.1	fBig5	C35B
U+9D60.1	fGB	1-7040
U+9D60.1	fR/S	 196.7
U+9D60.1	fTotalStrokes	 18
U+9D60.1	fCangjie	HRHAF
U+9D60.1	fFourCorner	27627
U+9D60.1	fMandarin	gu3
U+9D60.1	fEnglish	 target
U+9D61.0	fUTF8	鵡
U+9D61.0	fCNS11643	1-7959
U+9D61.0	fBig5	C44D
U+9D61.0	fGB	1-7044
U+9D61.0	fR/S	 196.7
U+9D61.0	fTotalStrokes	 18
U+9D61.0	fCangjie	MMHAF
U+9D61.0	fFourCorner	17127
U+9D61.0	fHakka	  wu3; wu1; vu3
U+9D61.0	fCantonese	mou5
U+9D61.0	fMandarin	wu3
U+9D61.0	fEnglish	correct form of &#x parrot;
U+9D62.0	fUTF8	鵢
U+9D62.0	fCNS11643	4-5F34
U+9D62.0	fGB	E-4772
U+9D62.0	fR/S	  196.7
U+9D62.0	fTotalStrokes	 18
U+9D62.0	fCangjie	HHHAF
U+9D62.0	fFourCorner	27420
U+9D62.0	fCantonese	san1
U+9D62.0	fMandarin	shen1
U+9D63.0	fUTF8	鵣
U+9D63.0	fCNS11643	3-597A
U+9D63.0	fGB	E-4773
U+9D63.0	fR/S	  196.7
U+9D63.0	fTotalStrokes	 18
U+9D63.0	fCangjie	DLHAF
U+9D63.0	fFourCorner	57920
U+9D63.0	fCantonese	laai6
U+9D63.0	fMandarin	lai4; chi4
U+9D64.0	fUTF8	鵤
U+9D64.0	fCNS11643	F-6177
U+9D64.0	fGB	E-4774
U+9D64.0	fR/S	 148.11; 196.7
U+9D64.0	fTotalStrokes	 18
U+9D64.0	fCangjie	XNBHA
U+9D64.0	fCantonese	gok3
U+9D65.0	fUTF8	鵥
U+9D65.0	fGB	E-4775
U+9D65.0	fR/S	 196.7
U+9D65.0	fTotalStrokes	 18
U+9D65.0	fCangjie	FNHAF
U+9D65.0	fCantonese	pun3
U+9D66.0	fUTF8	鵦
U+9D66.0	fCNS11643	3-5B7D
U+9D66.0	fGB	E-4776
U+9D66.0	fR/S	  196.8
U+9D66.0	fTotalStrokes	 19
U+9D66.0	fCangjie	HFVNE
U+9D66.0	fFourCorner	27330
U+9D66.0	fCantonese	luk6
U+9D66.0	fMandarin	lu4
U+9D67.0	fUTF8	鵧
U+9D67.0	fCNS11643	2-6126
U+9D67.0	fBig5	EF78
U+9D67.0	fGB	E-4777
U+9D67.0	fR/S	 196.8
U+9D67.0	fTotalStrokes	 19
U+9D67.0	fCangjie	TTHAF
U+9D67.0	fFourCorner	87427
U+9D67.0	fCantonese	pei4
U+9D67.0	fMandarin	bing4; ping2; bai3; pa2; pai2; pi2
U+9D68.0	fUTF8	鵨
U+9D68.0	fCNS11643	2-685E
U+9D68.0	fBig5	F3F1
U+9D68.0	fGB	E-4778
U+9D68.0	fR/S	 196.8
U+9D68.0	fTotalStrokes	 19
U+9D68.0	fCangjie	ORHAF
U+9D68.0	fFourCorner	87627
U+9D68.0	fCantonese	syu1
U+9D68.0	fMandarin	shu1
U+9D69.0	fUTF8	鵩
U+9D69.0	fCNS11643	2-6855
U+9D69.0	fBig5	F3E8
U+9D69.0	fGB	3-5D65
U+9D69.0	fR/S	 196.8
U+9D69.0	fTotalStrokes	 19
U+9D69.0	fCangjie	BSEF
U+9D69.0	fFourCorner	77227
U+9D69.0	fCantonese	fuk6
U+9D69.0	fMandarin	fu2
U+9D69.0	fEnglish	 vulture; buzzard
U+9D6A.0	fUTF8	鵪
U+9D6A.0	fCNS11643	1-795B
U+9D6A.0	fBig5	C44F
U+9D6A.0	fGB	1-7046
U+9D6A.0	fR/S	 196.8
U+9D6A.0	fTotalStrokes	 19
U+9D6A.0	fCangjie	KUHAF
U+9D6A.0	fFourCorner	47727
U+9D6A.0	fHakka	 em1; am1
U+9D6A.0	fCantonese	am1; ngam1 (also); aan3; ngaan3 (also)
U+9D6A.0	fMandarin	an1; ya1; yan4
U+9D6A.0	fEnglish	 quail
U+9D6B.0	fUTF8	鵫
U+9D6B.0	fCNS11643	2-6851
U+9D6B.0	fBig5	F3E4
U+9D6B.0	fGB	3-5D60
U+9D6B.0	fR/S	 196.8
U+9D6B.0	fTotalStrokes	 19
U+9D6B.0	fCangjie	YJHAF
U+9D6B.0	fFourCorner	27427
U+9D6B.0	fCantonese	zok6; zaau6
U+9D6B.0	fMandarin	zhao4; zhuo2
U+9D6B.0	fEnglish	pheasant
U+9D6C.0	fUTF8	鵬
U+9D6C.0	fCNS11643	1-795C
U+9D6C.0	fBig5	C450
U+9D6C.0	fGB	1-4574
U+9D6C.0	fR/S	 196.8
U+9D6C.0	fTotalStrokes	 19
U+9D6C.0	fCangjie	BBHF
U+9D6C.0	fFourCorner	77227
U+9D6C.0	fHakka	 pen2; pen3
U+9D6C.0	fCantonese	paang4
U+9D6C.0	fMandarin	peng2; feng4; beng1
U+9D6C.0	fEnglish	Peng, a fabulous bird supposed to be of enormous size, comparable to the roc
U+9D6D.0	fUTF8	鵭
U+9D6D.0	fCNS11643	3-5C27
U+9D6D.0	fGB	E-4779
U+9D6D.0	fR/S	 196.8
U+9D6D.0	fTotalStrokes	 19
U+9D6D.0	fCangjie	CHAF
U+9D6D.0	fFourCorner	87120
U+9D6D.0	fCantonese	kam4
U+9D6D.0	fMandarin	qin2
U+9D6D.0	fEnglish	a small bird with black neck
U+9D6E.0	fUTF8	鵮
U+9D6E.0	fCNS11643	3-5C22
U+9D6E.0	fGB	3-5D67
U+9D6E.0	fR/S	 196.8
U+9D6E.0	fTotalStrokes	 19
U+9D6E.0	fCangjie	NXHAF
U+9D6E.0	fFourCorner	17720
U+9D6E.0	fHakka	  kam1
U+9D6E.0	fCantonese	zaam1; cim1
U+9D6E.0	fMandarin	qian1
U+9D6E.0	fEnglish	[1] peck [2] ridicule; deride
U+9D6F.0	fUTF8	鵯
U+9D6F.0	fCNS11643	2-685A
U+9D6F.0	fBig5	F3ED
U+9D6F.0	fGB	1-7047
U+9D6F.0	fR/S	 196.8
U+9D6F.0	fTotalStrokes	 19
U+9D6F.0	fCangjie	HJHAF
U+9D6F.0	fFourCorner	27427
U+9D6F.0	fHakka	 pit7
U+9D6F.0	fCantonese	bei1; pat1
U+9D6F.0	fMandarin	bei1; pi4
U+9D6F.0	fEnglish	bird
U+9D70.0	fUTF8	鵰
U+9D70.0	fCNS11643	2-6854
U+9D70.0	fBig5	F3E7
U+9D70.0	fGB	E-477A
U+9D70.0	fR/S	 196.8
U+9D70.0	fTotalStrokes	 19
U+9D70.0	fCangjie	BRHAF
U+9D70.0	fFourCorner	77227
U+9D70.0	fHakka	  diau1
U+9D70.0	fCantonese	diu1
U+9D70.0	fMandarin	diao1
U+9D70.0	fEnglish	[1] bird of prey; vulture [2] bird
U+9D71.0	fUTF8	鵱
U+9D71.0	fCNS11643	2-684A
U+9D71.0	fBig5	F3DD
U+9D71.0	fGB	E-477B
U+9D71.0	fR/S	 196.8
U+9D71.0	fTotalStrokes	 19
U+9D71.0	fCangjie	GGHAF
U+9D71.0	fFourCorner	47127
U+9D71.0	fCantonese	luk6
U+9D71.0	fMandarin	lu4
U+9D72.0	fUTF8	鵲
U+9D72.0	fCNS11643	1-795A
U+9D72.0	fBig5	C44E
U+9D72.0	fGB	1-4835
U+9D72.0	fR/S	 196.8
U+9D72.0	fTotalStrokes	 19
U+9D72.0	fCangjie	TAHAF
U+9D72.0	fFourCorner	47627
U+9D72.0	fHakka	  siak7
U+9D72.0	fCantonese	zoek3; coek3 (also); zoek2 (also)
U+9D72.0	fMandarin	que4; qiao3; que2
U+9D72.0	fEnglish	 magpie
U+9D73.0	fUTF8	鵳
U+9D73.0	fCNS11643	2-6857
U+9D73.0	fBig5	F3EA
U+9D73.0	fGB	3-5D6C
U+9D73.0	fR/S	 196.8
U+9D73.0	fTotalStrokes	 19
U+9D73.0	fCangjie	HBHAF
U+9D73.0	fFourCorner	77227
U+9D73.0	fCantonese	gin1
U+9D73.0	fMandarin	jian1
U+9D74.0	fUTF8	鵴
U+9D74.0	fCNS11643	2-6852
U+9D74.0	fBig5	F3E5
U+9D74.0	fGB	E-477C
U+9D74.0	fR/S	 196.8
U+9D74.0	fTotalStrokes	 19
U+9D74.0	fCangjie	PDHAF
U+9D74.0	fFourCorner	27927
U+9D74.0	fHakka	  kiuk7
U+9D74.0	fCantonese	guk1
U+9D74.0	fMandarin	ju2
U+9D74.0	fEnglish	common cuckoo
U+9D75.0	fUTF8	鵵
U+9D75.0	fCNS11643	2-6853
U+9D75.0	fBig5	F3E6
U+9D75.0	fGB	3-5D66
U+9D75.0	fR/S	 196.8
U+9D75.0	fTotalStrokes	 19
U+9D75.0	fCangjie	NIHAF
U+9D75.0	fFourCorner	27227
U+9D75.0	fCantonese	tou3
U+9D75.0	fMandarin	tu4; tu2
U+9D76.0	fUTF8	鵶
U+9D76.0	fCNS11643	3-5C25
U+9D76.0	fGB	E-477D
U+9D76.0	fR/S	  196.8
U+9D76.0	fTotalStrokes	 19
U+9D76.0	fCangjie	MMHAF
U+9D76.0	fFourCorner	17120
U+9D76.0	fHakka	 a1
U+9D76.0	fCantonese	aa1; ngaa1 (also)
U+9D76.0	fMandarin	ya1
U+9D77.0	fUTF8	鵷
U+9D77.0	fCNS11643	2-6845
U+9D77.0	fBig5	F3D8
U+9D77.0	fGB	3-5D6B
U+9D77.0	fR/S	 196.8
U+9D77.0	fTotalStrokes	 19
U+9D77.0	fCangjie	JUHAF
U+9D77.0	fFourCorner	37227
U+9D77.0	fHakka	 wan3; jen1; ren1; jan1
U+9D77.0	fCantonese	jyun1
U+9D77.0	fMandarin	yuan1
U+9D77.0	fEnglish	[CC@] a bird resembling the legendary phoenix
U+9D78.0	fUTF8	鵸
U+9D78.0	fCNS11643	2-684C
U+9D78.0	fBig5	F3DF
U+9D78.0	fGB	3-5D5D
U+9D78.0	fR/S	 196.8
U+9D78.0	fTotalStrokes	 19
U+9D78.0	fCangjie	KRHAF
U+9D78.0	fFourCorner	47627
U+9D78.0	fCantonese	kei4
U+9D78.0	fMandarin	qi2; yi1
U+9D79.0	fUTF8	鵹
U+9D79.0	fCNS11643	2-685B
U+9D79.0	fBig5	F3EE
U+9D79.0	fGB	5-5C3E
U+9D79.0	fR/S	 196.8
U+9D79.0	fTotalStrokes	 19
U+9D79.0	fCangjie	HHHAF
U+9D79.0	fFourCorner	27327
U+9D79.0	fCantonese	lei4
U+9D79.0	fMandarin	li2
U+9D7A.0	fUTF8	鵺
U+9D7A.0	fCNS11643	3-5B7E
U+9D7A.0	fGB	E-477E
U+9D7A.0	fR/S	  196.8
U+9D7A.0	fTotalStrokes	 19
U+9D7A.0	fCangjie	YKHAF
U+9D7A.0	fFourCorner	07220
U+9D7A.0	fCantonese	je6
U+9D7A.0	fMandarin	ye4
U+9D7B.0	fUTF8	鵻
U+9D7B.0	fCNS11643	2-6858
U+9D7B.0	fBig5	F3EB
U+9D7B.0	fGB	3-5D64
U+9D7B.0	fR/S	 196.8
U+9D7B.0	fTotalStrokes	 19
U+9D7B.0	fCangjie	HFOG
U+9D7B.0	fFourCorner	20314
U+9D7B.0	fHakka	 chui1; zui1; zhui1
U+9D7B.0	fCantonese	zeoi1; zeon2
U+9D7B.0	fMandarin	zhui1
U+9D7B.0	fEnglish	pigeon
U+9D7C.0	fUTF8	鵼
U+9D7C.0	fCNS11643	5-6B3F
U+9D7C.0	fGB	3-5D6A
U+9D7C.0	fR/S	 196.8
U+9D7C.0	fTotalStrokes	 19
U+9D7C.0	fCangjie	JMHAF
U+9D7C.0	fFourCorner	37120
U+9D7C.0	fCantonese	hung1
U+9D7C.0	fMandarin	kong1
U+9D7C.0	fEnglish	a fabulous night-bird, grotesque monster
U+9D7D.0	fUTF8	鵽
U+9D7D.0	fCNS11643	2-6850
U+9D7D.0	fBig5	F3E3
U+9D7D.0	fGB	3-5D70
U+9D7D.0	fR/S	 196.8
U+9D7D.0	fTotalStrokes	 19
U+9D7D.0	fCangjie	EEEEF
U+9D7D.0	fFourCorner	17427
U+9D7D.0	fHakka	  dot7
U+9D7D.0	fCantonese	zyut3; dyut3
U+9D7D.0	fMandarin	duo4; zhui4; duo2; zhua1
U+9D7D.0	fEnglish	the sand-goose
U+9D7E.0	fUTF8	鵾
U+9D7E.0	fCNS11643	3-5C26
U+9D7E.0	fGB	3-5D61
U+9D7E.0	fR/S	 196.8
U+9D7E.0	fTotalStrokes	 19
U+9D7E.0	fCangjie	APHAF
U+9D7E.0	fFourCorner	67720
U+9D7E.0	fHakka	 kun1; kwun1
U+9D7E.0	fCantonese	kwan1
U+9D7E.0	fMandarin	kun1
U+9D7E.0	fEnglish	a bird resembling the crane
U+9D7F.0	fUTF8	鵿
U+9D7F.0	fCNS11643	2-685C
U+9D7F.0	fBig5	F3EF
U+9D7F.0	fGB	E-4821
U+9D7F.0	fR/S	 196.8
U+9D7F.0	fTotalStrokes	 19
U+9D7F.0	fCangjie	HFAHT
U+9D7F.0	fFourCorner	26340
U+9D7F.0	fCantonese	sing1
U+9D7F.0	fMandarin	sheng1
U+9D80.0	fUTF8	鶀
U+9D80.0	fCNS11643	2-684B
U+9D80.0	fBig5	F3DE
U+9D80.0	fGB	5-5C3C
U+9D80.0	fR/S	 196.8
U+9D80.0	fTotalStrokes	 19
U+9D80.0	fCangjie	HFTMC
U+9D80.0	fFourCorner	24381
U+9D80.0	fHakka	 ki2
U+9D80.0	fCantonese	kei4; hei1
U+9D80.0	fMandarin	qi2
U+9D80.0	fEnglish	[1] a kind of small wild goose [2] horned owl
U+9D81.0	fUTF8	鶁
U+9D81.0	fCNS11643	2-6846
U+9D81.0	fBig5	F3D9
U+9D81.0	fGB	E-4822
U+9D81.0	fR/S	 196.8
U+9D81.0	fTotalStrokes	 19
U+9D81.0	fCangjie	YFHAF
U+9D81.0	fFourCorner	07927
U+9D81.0	fCantonese	ging1
U+9D81.0	fMandarin	jing1
U+9D82.0	fUTF8	鶂
U+9D82.0	fCNS11643	2-6859
U+9D82.0	fBig5	F3EC
U+9D82.0	fGB	5-5C3F
U+9D82.0	fR/S	 196.8
U+9D82.0	fTotalStrokes	 19
U+9D82.0	fCangjie	HUHAF
U+9D82.0	fFourCorner	77227
U+9D82.0	fHakka	 nget7; ngiet7
U+9D82.0	fCantonese	jik6; jik1
U+9D82.0	fMandarin	yi4; ni2
U+9D82.0	fEnglish	hawk
U+9D83.0	fUTF8	鶃
U+9D83.0	fCNS11643	3-5C24
U+9D83.0	fGB	3-5D63
U+9D83.0	fR/S	 196.8
U+9D83.0	fTotalStrokes	 19
U+9D83.0	fCangjie	HFHXU
U+9D83.0	fFourCorner	27310
U+9D83.0	fHakka	 ngiet7; nget7
U+9D83.0	fCantonese	jik6
U+9D83.0	fMandarin	yi4
U+9D83.0	fEnglish	hawk
U+9D84.0	fUTF8	鶄
U+9D84.0	fCNS11643	2-6848
U+9D84.0	fBig5	F3DB
U+9D84.0	fGB	3-7437
U+9D84.0	fR/S	 196.8
U+9D84.0	fTotalStrokes	 19
U+9D84.0	fCangjie	QBHAF
U+9D84.0	fFourCorner	57227
U+9D84.0	fHakka	  zin1
U+9D84.0	fCantonese	zing1
U+9D84.0	fMandarin	jing1
U+9D84.0	fEnglish	Mycticorax prasinosceles, a kind of water bird
U+9D85.0	fUTF8	鶅
U+9D85.0	fCNS11643	2-6856
U+9D85.0	fBig5	F3E9
U+9D85.0	fGB	3-5D71
U+9D85.0	fR/S	 196.8
U+9D85.0	fTotalStrokes	 19
U+9D85.0	fCangjie	VWHAF
U+9D85.0	fFourCorner	27627
U+9D85.0	fCantonese	zi1
U+9D85.0	fMandarin	zi1; zi4
U+9D86.0	fUTF8	鶆
U+9D86.0	fCNS11643	2-684D
U+9D86.0	fBig5	F3E0
U+9D86.0	fGB	3-5D4D
U+9D86.0	fR/S	 196.8
U+9D86.0	fTotalStrokes	 19
U+9D86.0	fCangjie	DOHAF
U+9D86.0	fFourCorner	47927
U+9D86.0	fCantonese	loi4
U+9D86.0	fMandarin	lai2
U+9D87.0	fUTF8	鶇
U+9D87.0	fCNS11643	2-685D
U+9D87.0	fBig5	F3F0
U+9D87.0	fGB	1-7034
U+9D87.0	fR/S	 196.8
U+9D87.0	fTotalStrokes	 19
U+9D87.0	fCangjie	DWHAF
U+9D87.0	fFourCorner	57927
U+9D87.0	fCantonese	dung1
U+9D87.0	fMandarin	dong1
U+9D87.0	fEnglish	thrush
U+9D88.0	fUTF8	鶈
U+9D88.0	fCNS11643	2-6849
U+9D88.0	fBig5	F3DC
U+9D88.0	fGB	3-5D5E
U+9D88.0	fR/S	 196.8
U+9D88.0	fTotalStrokes	 19
U+9D88.0	fCangjie	JVHAF
U+9D88.0	fFourCorner	57427
U+9D88.0	fCantonese	cai1
U+9D88.0	fMandarin	qi1
U+9D89.0	fUTF8	鶉
U+9D89.0	fCNS11643	1-7958
U+9D89.0	fBig5	C44C
U+9D89.0	fGB	1-7048
U+9D89.0	fR/S	 196.8
U+9D89.0	fTotalStrokes	 19
U+9D89.0	fCangjie	YDHAF
U+9D89.0	fFourCorner	07427
U+9D89.0	fHakka	 tun2; sun2;  shun2
U+9D89.0	fCantonese	seon4; ceon1 (also)
U+9D89.0	fMandarin	chun2; dun1; tuan2
U+9D89.0	fEnglish	 quail
U+9D8A.0	fUTF8	鶊
U+9D8A.0	fCNS11643	2-6847
U+9D8A.0	fBig5	F3DA
U+9D8A.0	fGB	3-5D68
U+9D8A.0	fR/S	 196.8
U+9D8A.0	fTotalStrokes	 19
U+9D8A.0	fCangjie	IOHAF
U+9D8A.0	fFourCorner	07227
U+9D8A.0	fHakka	  gang1
U+9D8A.0	fCantonese	gang1
U+9D8A.0	fMandarin	geng1
U+9D8A.0	fEnglish	correct form of &#x oriole;
U+9D8B.0	fUTF8	鶋
U+9D8B.0	fCNS11643	2-684E
U+9D8B.0	fBig5	F3E1
U+9D8B.0	fGB	3-5D6E
U+9D8B.0	fR/S	 196.8
U+9D8B.0	fTotalStrokes	 19
U+9D8B.0	fCangjie	SRHAF
U+9D8B.0	fFourCorner	77227
U+9D8B.0	fHakka	  gi1
U+9D8B.0	fCantonese	geoi1
U+9D8B.0	fMandarin	ju1
U+9D8B.0	fEnglish	a sea-bird with a white breast
U+9D8C.0	fUTF8	鶌
U+9D8C.0	fCNS11643	2-684F
U+9D8C.0	fBig5	F3E2
U+9D8C.0	fGB	3-5D6F
U+9D8C.0	fR/S	 196.8
U+9D8C.0	fTotalStrokes	 19
U+9D8C.0	fCangjie	SUHAF
U+9D8C.0	fFourCorner	77227
U+9D8C.0	fCantonese	gwat6
U+9D8C.0	fMandarin	qu1; ju1
U+9D8D.0	fUTF8	鶍
U+9D8D.0	fGB	E-4823
U+9D8D.0	fR/S	 196.8
U+9D8D.0	fTotalStrokes	 19
U+9D8D.0	fCangjie	AHHAF
U+9D8D.0	fCantonese	ji6
U+9D8E.0	fUTF8	鶎
U+9D8E.0	fGB	E-4824
U+9D8E.0	fR/S	 196.8
U+9D8E.0	fTotalStrokes	 19
U+9D8E.0	fCangjie	JFHAF
U+9D8E.0	fCantonese	zung1
U+9D8F.0	fUTF8	鶏
U+9D8F.0	fGB	E-4825
U+9D8F.0	fR/S	 196.8
U+9D8F.0	fTotalStrokes	 19
U+9D8F.0	fCangjie	BOHAF
U+9D8F.0	fCantonese	gai1
U+9D8F.0	fMandarin	ji1
U+9D8F.0	fEnglish	chicken
U+9D90.0	fUTF8	鶐
U+9D90.0	fCNS11643	2-6B2D
U+9D90.0	fBig5	F57D
U+9D90.0	fGB	E-4826
U+9D90.0	fR/S	 196.9
U+9D90.0	fTotalStrokes	 20
U+9D90.0	fCangjie	YCHAF
U+9D90.0	fFourCorner	37327
U+9D90.0	fCantonese	wat6; seot6
U+9D90.0	fMandarin	shu4
U+9D91.0	fUTF8	鶑
U+9D91.0	fCNS11643	3-5C23
U+9D91.0	fGB	E-4827
U+9D91.0	fR/S	  196.8
U+9D91.0	fTotalStrokes	 19
U+9D91.0	fCangjie	FFHAF
U+9D91.0	fFourCorner	99320
U+9D91.0	fCantonese	ang1; ngang1 (also)
U+9D92.0	fUTF8	鶒
U+9D92.0	fCNS11643	2-6B2B
U+9D92.0	fBig5	F57B
U+9D92.0	fGB	3-5D73
U+9D92.0	fR/S	 196.9
U+9D92.0	fTotalStrokes	 20
U+9D92.0	fCangjie	DLKSF
U+9D92.0	fFourCorner	57927
U+9D92.0	fHakka	 chit7; ts'iit7
U+9D92.0	fCantonese	cik1
U+9D92.0	fMandarin	chi4; chi3; zhe4
U+9D92.0	fEnglish	a kind of water bird
U+9D93.0	fUTF8	鶓
U+9D93.0	fCNS11643	4-6569
U+9D93.0	fGB	1-7045
U+9D93.0	fR/S	  196.9; 196.8
U+9D93.0	fTotalStrokes	 20
U+9D93.0	fCangjie	TWHAF
U+9D93.0	fFourCorner	47620
U+9D93.0	fCantonese	miu4
U+9D93.0	fMandarin	miao2
U+9D94.0	fUTF8	鶔
U+9D94.0	fCNS11643	2-6B30
U+9D94.0	fBig5	F5A2
U+9D94.0	fGB	E-4828
U+9D94.0	fR/S	 196.9
U+9D94.0	fTotalStrokes	 20
U+9D94.0	fCangjie	NDHAF
U+9D94.0	fFourCorner	77927
U+9D94.0	fCantonese	jau4
U+9D94.0	fMandarin	rou2
U+9D95.0	fUTF8	鶕
U+9D95.0	fCNS11643	3-5D79
U+9D95.0	fGB	E-4829
U+9D95.0	fR/S	  196.9
U+9D95.0	fTotalStrokes	 20
U+9D95.0	fCangjie	YAHAF
U+9D95.0	fFourCorner	07620
U+9D95.0	fCantonese	am1; ngam1 (also)
U+9D95.0	fMandarin	an1; ya1
U+9D96.0	fUTF8	鶖
U+9D96.0	fCNS11643	2-6B3C
U+9D96.0	fBig5	F5AE
U+9D96.0	fGB	3-5D79
U+9D96.0	fR/S	 196.9
U+9D96.0	fTotalStrokes	 20
U+9D96.0	fCangjie	HFHAF
U+9D96.0	fFourCorner	29327
U+9D96.0	fHakka	 ciu1
U+9D96.0	fCantonese	cau1
U+9D96.0	fMandarin	qiu1
U+9D96.0	fEnglish	 marabou, a kind of water bird
U+9D97.0	fUTF8	鶗
U+9D97.0	fCNS11643	2-6B33
U+9D97.0	fBig5	F5A5
U+9D97.0	fGB	5-5C42
U+9D97.0	fR/S	 196.9
U+9D97.0	fTotalStrokes	 20
U+9D97.0	fCangjie	AOHAF
U+9D97.0	fFourCorner	67827
U+9D97.0	fHakka	  ti2
U+9D97.0	fCantonese	tai4
U+9D97.0	fMandarin	ti2; chi2; di4
U+9D97.0	fEnglish	 cuckoo
U+9D98.0	fUTF8	鶘
U+9D98.0	fCNS11643	2-6B2C
U+9D98.0	fBig5	F57C
U+9D98.0	fGB	1-7049
U+9D98.0	fR/S	 196.9
U+9D98.0	fTotalStrokes	 20
U+9D98.0	fCangjie	JRBHF
U+9D98.0	fFourCorner	47627
U+9D98.0	fHakka	 fu2
U+9D98.0	fCantonese	wu4
U+9D98.0	fMandarin	hu2
U+9D98.0	fEnglish	 pelican
U+9D99.0	fUTF8	鶙
U+9D99.0	fCNS11643	2-6B28
U+9D99.0	fBig5	F578
U+9D99.0	fGB	3-5D7E
U+9D99.0	fR/S	 196.9
U+9D99.0	fTotalStrokes	 20
U+9D99.0	fCangjie	YBHAF
U+9D99.0	fFourCorner	07227
U+9D99.0	fCantonese	tai4
U+9D99.0	fMandarin	ti2; ti4
U+9D9A.0	fUTF8	鶚
U+9D9A.0	fCNS11643	2-6B35
U+9D9A.0	fBig5	F5A7
U+9D9A.0	fGB	1-704A
U+9D9A.0	fR/S	 196.9
U+9D9A.0	fTotalStrokes	 20
U+9D9A.0	fCangjie	RSHAF
U+9D9A.0	fFourCorner	67227
U+9D9A.0	fHakka	 ngok8
U+9D9A.0	fCantonese	ngok6; ok6 (also)
U+9D9A.0	fMandarin	e4
U+9D9A.0	fEnglish	 osprey; water hawk
U+9D9B.0	fUTF8	鶛
U+9D9B.0	fCNS11643	2-6B2E
U+9D9B.0	fBig5	F57E
U+9D9B.0	fGB	3-5D75
U+9D9B.0	fR/S	 196.9
U+9D9B.0	fTotalStrokes	 20
U+9D9B.0	fCangjie	PAHAF
U+9D9B.0	fFourCorner	27627
U+9D9B.0	fHakka	 gai1; kie1; giai1
U+9D9B.0	fCantonese	gaai1; gaai3
U+9D9B.0	fMandarin	jie1; jie4; zhan4
U+9D9B.0	fEnglish	a cowardly bird
U+9D9C.0	fUTF8	鶜
U+9D9C.0	fCNS11643	2-6B31
U+9D9C.0	fBig5	F5A3
U+9D9C.0	fGB	E-482A
U+9D9C.0	fR/S	 196.9
U+9D9C.0	fTotalStrokes	 20
U+9D9C.0	fCangjie	THHAF
U+9D9C.0	fFourCorner	47227
U+9D9C.0	fCantonese	maau4
U+9D9C.0	fMandarin	mao2; mao1
U+9D9D.0	fUTF8	鶝
U+9D9D.0	fCNS11643	2-6B2A
U+9D9D.0	fBig5	F57A
U+9D9D.0	fGB	3-5D74
U+9D9D.0	fR/S	 196.9
U+9D9D.0	fTotalStrokes	 20
U+9D9D.0	fCangjie	MWHAF
U+9D9D.0	fFourCorner	17627
U+9D9D.0	fCantonese	fuk1; bik1
U+9D9D.0	fMandarin	fu2; bi1
U+9D9E.0	fUTF8	鶞
U+9D9E.0	fCNS11643	2-6B38
U+9D9E.0	fBig5	F5AA
U+9D9E.0	fGB	E-482B
U+9D9E.0	fR/S	 196.9
U+9D9E.0	fTotalStrokes	 20
U+9D9E.0	fCangjie	HUHAF
U+9D9E.0	fFourCorner	77227
U+9D9E.0	fCantonese	ceon1
U+9D9E.0	fMandarin	chun1
U+9D9F.0	fUTF8	鶟
U+9D9F.0	fCNS11643	2-6B27
U+9D9F.0	fBig5	F577
U+9D9F.0	fGB	E-482C
U+9D9F.0	fR/S	 196.9
U+9D9F.0	fTotalStrokes	 20
U+9D9F.0	fCangjie	JKHAF
U+9D9F.0	fFourCorner	37427
U+9D9F.0	fCantonese	dat6
U+9D9F.0	fMandarin	tu2
U+9DA0.0	fUTF8	鶠
U+9DA0.0	fCNS11643	2-6B2F
U+9DA0.0	fBig5	F5A1
U+9DA0.0	fGB	3-5D72
U+9DA0.0	fR/S	 196.9
U+9DA0.0	fTotalStrokes	 20
U+9DA0.0	fCangjie	SVHAF
U+9DA0.0	fFourCorner	77727
U+9DA0.0	fHakka	 jen3; ren3
U+9DA0.0	fCantonese	jin2; aan3; ngaan3 (also)
U+9DA0.0	fMandarin	yan3
U+9DA1.0	fUTF8	鶡
U+9DA1.0	fCNS11643	2-6B34
U+9DA1.0	fBig5	F5A6
U+9DA1.0	fGB	3-5D77
U+9DA1.0	fR/S	 196.9
U+9DA1.0	fTotalStrokes	 20
U+9DA1.0	fCangjie	AVHAF
U+9DA1.0	fFourCorner	67727
U+9DA1.0	fHakka	 hot7
U+9DA1.0	fCantonese	hot3; hot6
U+9DA1.0	fMandarin	he2; jie4; fen1; he4
U+9DA1.0	fEnglish	[1] a species of nightingale [2] a sort of pheasant
U+9DA2.0	fUTF8	鶢
U+9DA2.0	fCNS11643	2-6B36
U+9DA2.0	fBig5	F5A8
U+9DA2.0	fGB	E-482D
U+9DA2.0	fR/S	 196.9
U+9DA2.0	fTotalStrokes	 20
U+9DA2.0	fCangjie	BEHAF
U+9DA2.0	fFourCorner	27427
U+9DA2.0	fHakka	 jen1; ren1
U+9DA2.0	fCantonese	wun4; jyun4
U+9DA2.0	fMandarin	yuan2
U+9DA2.0	fEnglish	a bird which frequents the sea-shore
U+9DA3.0	fUTF8	鶣
U+9DA3.0	fCNS11643	2-6B39
U+9DA3.0	fBig5	F5AB
U+9DA3.0	fGB	5-5C45
U+9DA3.0	fR/S	 196.9
U+9DA3.0	fTotalStrokes	 20
U+9DA3.0	fCangjie	HBHAF
U+9DA3.0	fFourCorner	37227
U+9DA3.0	fCantonese	pin1; bin1
U+9DA3.0	fMandarin	pian1; bin4; bian4; pian4; pin4
U+9DA4.0	fUTF8	鶤
U+9DA4.0	fCNS11643	2-6B29
U+9DA4.0	fBig5	F579
U+9DA4.0	fGB	5-5C35
U+9DA4.0	fR/S	 196.9
U+9DA4.0	fTotalStrokes	 20
U+9DA4.0	fCangjie	BJHAF
U+9DA4.0	fFourCorner	37527
U+9DA4.0	fCantonese	kwan1; gwan1
U+9DA4.0	fMandarin	yun4; kun1
U+9DA5.0	fUTF8	鶥
U+9DA5.0	fCNS11643	3-5D7A
U+9DA5.0	fGB	1-704C
U+9DA5.0	fR/S	 196.9
U+9DA5.0	fTotalStrokes	 20
U+9DA5.0	fCangjie	AUHAF
U+9DA5.0	fFourCorner	77220
U+9DA5.0	fCantonese	mei4
U+9DA5.0	fMandarin	mei2
U+9DA6.0	fUTF8	鶦
U+9DA6.0	fCNS11643	2-6B3D
U+9DA6.0	fBig5	F5AF
U+9DA6.0	fGB	E-482E
U+9DA6.0	fR/S	 196.9
U+9DA6.0	fTotalStrokes	 20
U+9DA6.0	fCangjie	JBHAF
U+9DA6.0	fFourCorner	47327
U+9DA6.0	fHakka	 fu2
U+9DA6.0	fCantonese	wu4
U+9DA6.0	fMandarin	hu2
U+9DA6.0	fEnglish	pellican
U+9DA7.0	fUTF8	鶧
U+9DA7.0	fCNS11643	2-6B3E
U+9DA7.0	fBig5	F5B0
U+9DA7.0	fGB	E-482F
U+9DA7.0	fR/S	 196.9
U+9DA7.0	fTotalStrokes	 20
U+9DA7.0	fCangjie	TKHAF
U+9DA7.0	fFourCorner	47527
U+9DA7.0	fCantonese	jing1
U+9DA7.0	fMandarin	ying1
U+9DA8.0	fUTF8	鶨
U+9DA8.0	fCNS11643	2-6B37
U+9DA8.0	fBig5	F5A9
U+9DA8.0	fGB	E-4830
U+9DA8.0	fR/S	 196.9
U+9DA8.0	fTotalStrokes	 20
U+9DA8.0	fCangjie	VOHAF
U+9DA8.0	fFourCorner	27227
U+9DA8.0	fHakka	 chon5; con5
U+9DA8.0	fCantonese	cyun3; deon6; zi6; syun6
U+9DA8.0	fMandarin	dun4; chuan4; zhi4
U+9DA8.0	fEnglish	a bird known as the stupid bird
U+9DA9.0	fUTF8	鶩
U+9DA9.0	fCNS11643	2-6B3B
U+9DA9.0	fBig5	F5AD
U+9DA9.0	fGB	1-704D
U+9DA9.0	fR/S	 196.9
U+9DA9.0	fTotalStrokes	 20
U+9DA9.0	fCangjie	NKHAF
U+9DA9.0	fFourCorner	18327
U+9DA9.0	fHakka	 muk8; wu5; mu5; vu5
U+9DA9.0	fCantonese	mou6
U+9DA9.0	fMandarin	wu4; mu4
U+9DA9.0	fEnglish	 duck
U+9DAA.0	fUTF8	鶪
U+9DAA.0	fCNS11643	2-6B32
U+9DAA.0	fBig5	F5A4
U+9DAA.0	fGB	3-5D76
U+9DAA.0	fR/S	 196.9
U+9DAA.0	fTotalStrokes	 20
U+9DAA.0	fCangjie	BKHAF
U+9DAA.0	fFourCorner	67427
U+9DAA.0	fHakka	 giet7; get7
U+9DAA.0	fCantonese	gwik1; kyut3
U+9DAA.0	fMandarin	ju2
U+9DAA.0	fEnglish	widowbird
U+9DAB.0	fUTF8	鶫
U+9DAB.0	fGB	E-4831
U+9DAB.0	fR/S	 196.9
U+9DAB.0	fTotalStrokes	 20
U+9DAB.0	fCangjie	DWHAF
U+9DAB.0	fCantonese	gaan2
U+9DAB.0	fEnglish	 thrush
U+9DAC.0	fUTF8	鶬
U+9DAC.0	fCNS11643	2-6D2F
U+9DAC.0	fBig5	F6C1
U+9DAC.0	fGB	3-5D25
U+9DAC.0	fR/S	 196.10
U+9DAC.0	fTotalStrokes	 21
U+9DAC.0	fCangjie	ORHAF
U+9DAC.0	fFourCorner	87627
U+9DAC.0	fHakka	 cong1
U+9DAC.0	fCantonese	cong1; coeng1
U+9DAC.0	fMandarin	cang1; qiang1
U+9DAC.0	fEnglish	 oriole
U+9DAD.0	fUTF8	鶭
U+9DAD.0	fCNS11643	2-6D32
U+9DAD.0	fBig5	F6C4
U+9DAD.0	fGB	E-4832
U+9DAD.0	fR/S	 196.10
U+9DAD.0	fTotalStrokes	 21
U+9DAD.0	fCangjie	VSHAF
U+9DAD.0	fFourCorner	20327
U+9DAD.0	fCantonese	fong2
U+9DAD.0	fMandarin	fang3
U+9DAE.0	fUTF8	鶮
U+9DAE.0	fCNS11643	3-5F35
U+9DAE.0	fGB	5-7969
U+9DAE.0	fR/S	  196.10
U+9DAE.0	fTotalStrokes	 21
U+9DAE.0	fCangjie	YBHAF
U+9DAE.0	fFourCorner	07220
U+9DAE.0	fCantonese	hok6
U+9DAE.0	fMandarin	gu4; he4; hu2
U+9DAF.0	fUTF8	鶯
U+9DAF.0	fCNS11643	1-7B4D
U+9DAF.0	fBig5	C561
U+9DAF.0	fGB	1-5D3A
U+9DAF.0	fR/S	 196.10
U+9DAF.0	fTotalStrokes	 21
U+9DAF.0	fCangjie	FFBHF
U+9DAF.0	fFourCorner	99327
U+9DAF.0	fHakka	 jin1;  jin2; rin2; ang1
U+9DAF.0	fCantonese	ang1; ngang1 (also)
U+9DAF.0	fMandarin	ying1
U+9DAF.0	fEnglish	 greenfinch; Chinese oriole
U+9DB0.0	fUTF8	鶰
U+9DB0.0	fCNS11643	3-5F31
U+9DB0.0	fGB	3-5D51
U+9DB0.0	fR/S	  196.10
U+9DB0.0	fTotalStrokes	 21
U+9DB0.0	fCangjie	RCHAF
U+9DB0.0	fFourCorner	67820
U+9DB0.0	fCantonese	jyun4
U+9DB0.0	fMandarin	yuan2
U+9DB1.0	fUTF8	鶱
U+9DB1.0	fCNS11643	2-6D31
U+9DB1.0	fBig5	F6C3
U+9DB1.0	fGB	3-5E2E
U+9DB1.0	fR/S	 196.10
U+9DB1.0	fTotalStrokes	 21
U+9DB1.0	fCangjie	JTCF
U+9DB1.0	fFourCorner	30327
U+9DB1.0	fHakka	 hien1; kien1
U+9DB1.0	fCantonese	hin1
U+9DB1.0	fMandarin	xuan1; xian1
U+9DB1.0	fEnglish	soar
U+9DB2.0	fUTF8	鶲
U+9DB2.0	fCNS11643	2-6D36
U+9DB2.0	fBig5	F6C8
U+9DB2.0	fGB	3-5E2A
U+9DB2.0	fR/S	 196.10
U+9DB2.0	fTotalStrokes	 21
U+9DB2.0	fCangjie	CMHAF
U+9DB2.0	fFourCorner	87127
U+9DB2.0	fCantonese	jung1
U+9DB2.0	fMandarin	weng1
U+9DB3.0	fUTF8	鶳
U+9DB3.0	fCNS11643	2-6D34
U+9DB3.0	fBig5	F6C6
U+9DB3.0	fGB	E-4833
U+9DB3.0	fR/S	 196.10
U+9DB3.0	fTotalStrokes	 21
U+9DB3.0	fCangjie	HFHRB
U+9DB3.0	fFourCorner	21327
U+9DB3.0	fCantonese	si1
U+9DB3.0	fMandarin	shi1
U+9DB4.0	fUTF8	鶴
U+9DB4.0	fCNS11643	1-7B4E
U+9DB4.0	fBig5	C562
U+9DB4.0	fGB	1-3A57
U+9DB4.0	fR/S	 196.10
U+9DB4.0	fTotalStrokes	 21
U+9DB4.0	fCangjie	OGHAF
U+9DB4.0	fFourCorner	47227
U+9DB4.0	fHakka	  hok8
U+9DB4.0	fCantonese	hok6; hok2 (also)
U+9DB4.0	fMandarin	he4; hao2
U+9DB4.0	fEnglish	 crane
U+9DB5.0	fUTF8	鶵
U+9DB5.0	fCNS11643	2-6D2B
U+9DB5.0	fBig5	F6BD
U+9DB5.0	fGB	5-5C26
U+9DB5.0	fR/S	 196.10
U+9DB5.0	fTotalStrokes	 21
U+9DB5.0	fCangjie	PUHAF
U+9DB5.0	fFourCorner	27427
U+9DB5.0	fHakka	 ci1; cu1; cih1
U+9DB5.0	fCantonese	co1; co4 (also)
U+9DB5.0	fMandarin	chu2
U+9DB5.0	fEnglish	 chick; fledging infant; toddler
U+9DB6.0	fUTF8	鶶
U+9DB6.0	fCNS11643	2-6D21
U+9DB6.0	fBig5	F6B3
U+9DB6.0	fGB	E-4834
U+9DB6.0	fR/S	 196.10
U+9DB6.0	fTotalStrokes	 21
U+9DB6.0	fCangjie	IRHAF
U+9DB6.0	fFourCorner	07227
U+9DB6.0	fCantonese	tong4
U+9DB6.0	fMandarin	tang2
U+9DB7.0	fUTF8	鶷
U+9DB7.0	fCNS11643	2-6C7E
U+9DB7.0	fBig5	F6B2
U+9DB7.0	fGB	E-4835
U+9DB7.0	fR/S	 196.10
U+9DB7.0	fTotalStrokes	 21
U+9DB7.0	fCangjie	JRHAF
U+9DB7.0	fFourCorner	37627
U+9DB7.0	fCantonese	hat6
U+9DB7.0	fMandarin	xia4; he4
U+9DB8.0	fUTF8	鶸
U+9DB8.0	fCNS11643	1-7B50
U+9DB8.0	fBig5	C564
U+9DB8.0	fGB	5-5C4A
U+9DB8.0	fR/S	 196.10
U+9DB8.0	fTotalStrokes	 21
U+9DB8.0	fCangjie	NMNMF
U+9DB8.0	fFourCorner	17227
U+9DB8.0	fCantonese	joek6
U+9DB8.0	fMandarin	ruo4
U+9DB8.0	fEnglish	light yellowish-green
U+9DB9.0	fUTF8	鶹
U+9DB9.0	fCNS11643	2-6D2D
U+9DB9.0	fBig5	F6BF
U+9DB9.0	fGB	3-5E2B
U+9DB9.0	fR/S	 196.10
U+9DB9.0	fTotalStrokes	 21
U+9DB9.0	fCangjie	HWHAF
U+9DB9.0	fFourCorner	77627
U+9DB9.0	fHakka	  liu2
U+9DB9.0	fCantonese	lau4
U+9DB9.0	fMandarin	liu2
U+9DB9.0	fEnglish	correct form of &#xthe owl;
U+9DBA.0	fUTF8	鶺
U+9DBA.0	fCNS11643	2-6D2E
U+9DBA.0	fBig5	F6C0
U+9DBA.0	fGB	3-5E2C
U+9DBA.0	fR/S	 196.10
U+9DBA.0	fTotalStrokes	 21
U+9DBA.0	fCangjie	FBHAF
U+9DBA.0	fFourCorner	17227
U+9DBA.0	fHakka	 zit7
U+9DBA.0	fCantonese	zik3; zek3 (also)
U+9DBA.0	fMandarin	ji2
U+9DBA.0	fEnglish	 wagtail
U+9DBB.0	fUTF8	鶻
U+9DBB.0	fCNS11643	2-6D2A
U+9DBB.0	fBig5	F6BC
U+9DBB.0	fGB	1-773D
U+9DBB.0	fR/S	 196.10
U+9DBB.0	fTotalStrokes	 21
U+9DBB.0	fCangjie	BBHAF
U+9DBB.0	fFourCorner	77227
U+9DBB.0	fHakka	 gut7; hok8; gwut7
U+9DBB.0	fCantonese	gwat1; wat6
U+9DBB.0	fMandarin	gu2; gu3; hua2
U+9DBB.0	fEnglish	Treron permagna, a kind of pigeon
U+9DBB.1	fUTF8	鶻
U+9DBB.1	fCNS11643	2-6D2A
U+9DBB.1	fBig5	F6BC
U+9DBB.1	fGB	1-773D
U+9DBB.1	fR/S	 196.10
U+9DBB.1	fTotalStrokes	 21
U+9DBB.1	fCangjie	BBHAF
U+9DBB.1	fFourCorner	77227
U+9DBB.1	fMandarin	hu2
U+9DBB.1	fEnglish	a kind of bird of prey
U+9DBC.0	fUTF8	鶼
U+9DBC.0	fCNS11643	2-6D22
U+9DBC.0	fBig5	F6B4
U+9DBC.0	fGB	1-704F
U+9DBC.0	fR/S	 196.10
U+9DBC.0	fTotalStrokes	 21
U+9DBC.0	fCangjie	TCHAF
U+9DBC.0	fFourCorner	87327
U+9DBC.0	fHakka	  giam1
U+9DBC.0	fCantonese	gim1
U+9DBC.0	fMandarin	jian1; qian1
U+9DBC.0	fEnglish	a fabulous mythical bird having only one wing so that a pair must unite in order to fly
U+9DBD.0	fUTF8	鶽
U+9DBD.0	fCNS11643	3-5F32
U+9DBD.0	fGB	3-5E27
U+9DBD.0	fR/S	  196.10
U+9DBD.0	fTotalStrokes	 21
U+9DBD.0	fCangjie	OJHAF
U+9DBD.0	fFourCorner	27420
U+9DBD.0	fHakka	 sun3
U+9DBD.0	fCantonese	zeon2
U+9DBD.0	fMandarin	zhun3; sun3
U+9DBD.0	fEnglish	a hawk
U+9DBE.0	fUTF8	鶾
U+9DBE.0	fCNS11643	2-6D27
U+9DBE.0	fBig5	F6B9
U+9DBE.0	fGB	5-733E
U+9DBE.0	fR/S	 196.10
U+9DBE.0	fTotalStrokes	 21
U+9DBE.0	fCangjie	JJOHF
U+9DBE.0	fFourCorner	48427
U+9DBE.0	fCantonese	hon6; hon4
U+9DBE.0	fMandarin	han4; han2
U+9DBF.0	fUTF8	鶿
U+9DBF.0	fCNS11643	2-6B3A
U+9DBF.0	fBig5	F5AC
U+9DBF.0	fGB	E-4836
U+9DBF.0	fR/S	 196.10; 196.9
U+9DBF.0	fTotalStrokes	 21
U+9DBF.0	fCangjie	TVIF
U+9DBF.0	fFourCorner	44327
U+9DBF.0	fHakka	 zih1; cih2; cu2
U+9DBF.0	fCantonese	ci4
U+9DBF.0	fMandarin	zi1; ci2
U+9DBF.0	fEnglish	cormorant
U+9DC0.0	fUTF8	鷀
U+9DC0.0	fCNS11643	4-6843
U+9DC0.0	fGB	1-704B
U+9DC0.0	fR/S	 196.10; 196.9
U+9DC0.0	fTotalStrokes	 21; 20
U+9DC0.0	fCangjie	TIHAF
U+9DC0.0	fFourCorner	47727
U+9DC0.0	fHakka	 cih2; zih1; cih3; cu3; ze2
U+9DC0.0	fCantonese	ci4
U+9DC0.0	fMandarin	zi1; ci2
U+9DC0.0	fEnglish	correct form of &#x cormorant;
U+9DC1.0	fUTF8	鷁
U+9DC1.0	fCNS11643	2-6D23
U+9DC1.0	fBig5	F6B5
U+9DC1.0	fGB	3-5E2D
U+9DC1.0	fR/S	 196.10
U+9DC1.0	fTotalStrokes	 21
U+9DC1.0	fCangjie	TTHAF
U+9DC1.0	fFourCorner	87127
U+9DC1.0	fHakka	 ngiak8; jit7; rit7
U+9DC1.0	fCantonese	jik6
U+9DC1.0	fMandarin	yi4; ni4
U+9DC1.0	fEnglish	a fabulous sea bird
U+9DC2.0	fUTF8	鷂
U+9DC2.0	fCNS11643	1-7B4F
U+9DC2.0	fBig5	C563
U+9DC2.0	fGB	1-704E
U+9DC2.0	fR/S	 196.10
U+9DC2.0	fTotalStrokes	 21
U+9DC2.0	fCangjie	BUHAF
U+9DC2.0	fFourCorner	27727
U+9DC2.0	fHakka	  jau5; jau3; rau6; jau6; jeu5
U+9DC2.0	fCantonese	jiu6; jiu2 (also); jiu4
U+9DC2.0	fMandarin	yao4; yao2
U+9DC2.0	fEnglish	[1] sparrow hawk; hawk [2] a kind of kite [3] kite
U+9DC3.0	fUTF8	鷃
U+9DC3.0	fCNS11643	2-6D29
U+9DC3.0	fBig5	F6BB
U+9DC3.0	fGB	3-5E25
U+9DC3.0	fR/S	 196.10
U+9DC3.0	fTotalStrokes	 21
U+9DC3.0	fCangjie	AVHAF
U+9DC3.0	fFourCorner	67427
U+9DC3.0	fHakka	 on5
U+9DC3.0	fCantonese	aan3; ngaan3 (also)
U+9DC3.0	fMandarin	yan4
U+9DC3.0	fEnglish	 quail
U+9DC4.0	fUTF8	鷄
U+9DC4.0	fCNS11643	3-5F36
U+9DC4.0	fGB	1-3C26
U+9DC4.0	fR/S	 196.10
U+9DC4.0	fTotalStrokes	 21
U+9DC4.0	fCangjie	BKHAF
U+9DC4.0	fFourCorner	27420
U+9DC4.0	fHakka	  gai1; kie1; ke1; ge1
U+9DC4.0	fCantonese	gai1
U+9DC4.0	fMandarin	ji1
U+9DC4.0	fEnglish	[1] chicken; hen; cock; domestic fowl [2] [3] prostitute
U+9DC5.0	fUTF8	鷅
U+9DC5.0	fCNS11643	2-6D28
U+9DC5.0	fBig5	F6BA
U+9DC5.0	fGB	5-5C46
U+9DC5.0	fR/S	 196.10
U+9DC5.0	fTotalStrokes	 21
U+9DC5.0	fCangjie	MDHAF
U+9DC5.0	fFourCorner	17927
U+9DC5.0	fCantonese	leot6
U+9DC5.0	fMandarin	li4; piao3; jian1
U+9DC6.0	fUTF8	鷆
U+9DC6.0	fGB	E-4837
U+9DC6.0	fR/S	 196.10
U+9DC6.0	fTotalStrokes	 21
U+9DC6.0	fCangjie	PCHAF
U+9DC6.0	fCantonese	tin4
U+9DC6.0	fMandarin	tian2; zhen1
U+9DC6.0	fEnglish	bird name
U+9DC7.0	fUTF8	鷇
U+9DC7.0	fCNS11643	2-6D24
U+9DC7.0	fBig5	F6B6
U+9DC7.0	fGB	3-4F38
U+9DC7.0	fR/S	 196.10
U+9DC7.0	fTotalStrokes	 21
U+9DC7.0	fCangjie	GFHNE
U+9DC7.0	fFourCorner	07327
U+9DC7.0	fHakka	 keu5
U+9DC7.0	fCantonese	kau3; gau3 (also)
U+9DC7.0	fMandarin	kou4; gu1; gou4; gu4
U+9DC7.0	fEnglish	 chicks; fledglings
U+9DC8.0	fUTF8	鷈
U+9DC8.0	fCNS11643	2-6D30
U+9DC8.0	fBig5	F6C2
U+9DC8.0	fGB	E-4838
U+9DC8.0	fR/S	 196.10
U+9DC8.0	fTotalStrokes	 21
U+9DC8.0	fCangjie	HFHYU
U+9DC8.0	fFourCorner	22317
U+9DC8.0	fCantonese	tai1; tai4
U+9DC8.0	fMandarin	ti1; si1
U+9DC9.0	fUTF8	鷉
U+9DC9.0	fCNS11643	4-6844
U+9DC9.0	fGB	3-5E29
U+9DC9.0	fR/S	 196.10
U+9DC9.0	fTotalStrokes	 21
U+9DC9.0	fCangjie	XHNHA
U+9DC9.0	fFourCorner	77220
U+9DC9.0	fHakka	 ti2; ti1
U+9DC9.0	fCantonese	tai1
U+9DC9.0	fMandarin	ti1
U+9DC9.0	fEnglish	the brown grebe
U+9DCA.0	fUTF8	鷊
U+9DCA.0	fCNS11643	2-6D25
U+9DCA.0	fBig5	F6B7
U+9DCA.0	fGB	3-7B69
U+9DCA.0	fR/S	 196.10
U+9DCA.0	fTotalStrokes	 21
U+9DCA.0	fCangjie	MBHAF
U+9DCA.0	fFourCorner	17227
U+9DCA.0	fHakka	 ngiak8; ngiak7
U+9DCA.0	fCantonese	jik6
U+9DCA.0	fMandarin	ni4; yi4
U+9DCA.0	fEnglish	pheasant
U+9DCB.0	fUTF8	鷋
U+9DCB.0	fCNS11643	2-6E68
U+9DCB.0	fBig5	F7BB
U+9DCB.0	fGB	3-5E24
U+9DCB.0	fR/S	 196.11
U+9DCB.0	fTotalStrokes	 22
U+9DCB.0	fCangjie	TDHAF
U+9DCB.0	fFourCorner	47927
U+9DCB.0	fCantonese	tou4
U+9DCB.0	fMandarin	tu2
U+9DCC.0	fUTF8	鷌
U+9DCC.0	fCNS11643	2-6D33
U+9DCC.0	fBig5	F6C5
U+9DCC.0	fGB	E-4839
U+9DCC.0	fR/S	 196.10
U+9DCC.0	fTotalStrokes	 21
U+9DCC.0	fCangjie	HFSQF
U+9DCC.0	fFourCorner	21327
U+9DCC.0	fCantonese	maa5
U+9DCC.0	fMandarin	ma3
U+9DCD.0	fUTF8	鷍
U+9DCD.0	fCNS11643	2-6D35
U+9DCD.0	fBig5	F6C7
U+9DCD.0	fGB	E-483A
U+9DCD.0	fR/S	 196.10
U+9DCD.0	fTotalStrokes	 21
U+9DCD.0	fCangjie	HDHAF
U+9DCD.0	fFourCorner	37927
U+9DCD.0	fCantonese	hiu1
U+9DCD.0	fMandarin	jiao1
U+9DCE.0	fUTF8	鷎
U+9DCE.0	fCNS11643	2-6D2C
U+9DCE.0	fBig5	F6BE
U+9DCE.0	fGB	3-5E28
U+9DCE.0	fR/S	 196.10
U+9DCE.0	fTotalStrokes	 21
U+9DCE.0	fCangjie	HMHAF
U+9DCE.0	fFourCorner	27427
U+9DCE.0	fCantonese	gou1
U+9DCE.0	fMandarin	gao1; liu2; gao3
U+9DCF.0	fUTF8	鷏
U+9DCF.0	fCNS11643	2-6D26
U+9DCF.0	fBig5	F6B8
U+9DCF.0	fGB	E-483B
U+9DCF.0	fR/S	 196.10
U+9DCF.0	fTotalStrokes	 21
U+9DCF.0	fCangjie	JCHAF
U+9DCF.0	fFourCorner	47827
U+9DCF.0	fHakka	 tien2
U+9DCF.0	fCantonese	tin4
U+9DCF.0	fMandarin	zhen1; tian2
U+9DCF.0	fEnglish	bird name
U+9DD0.0	fUTF8	鷐
U+9DD0.0	fCNS11643	2-6E69
U+9DD0.0	fBig5	F7BC
U+9DD0.0	fGB	5-5C50
U+9DD0.0	fR/S	 196.11
U+9DD0.0	fTotalStrokes	 22
U+9DD0.0	fCangjie	AVHAF
U+9DD0.0	fFourCorner	67227
U+9DD0.0	fCantonese	san4
U+9DD0.0	fMandarin	chen2
U+9DD1.0	fUTF8	鷑
U+9DD1.0	fCNS11643	2-6E6B
U+9DD1.0	fBig5	F7BE
U+9DD1.0	fGB	E-483C
U+9DD1.0	fR/S	 196.11
U+9DD1.0	fTotalStrokes	 22
U+9DD1.0	fCangjie	HTHAF
U+9DD1.0	fFourCorner	87127
U+9DD1.0	fCantonese	gap6; kap6; kap4; lap1
U+9DD1.0	fMandarin	li4; ji2
U+9DD2.0	fUTF8	鷒
U+9DD2.0	fCNS11643	2-6E65
U+9DD2.0	fBig5	F7B8
U+9DD2.0	fGB	E-483D
U+9DD2.0	fR/S	 196.11
U+9DD2.0	fTotalStrokes	 22
U+9DD2.0	fCangjie	JIHAF
U+9DD2.0	fFourCorner	57327
U+9DD2.0	fCantonese	tyun4
U+9DD2.0	fMandarin	zhuan1; tuan2
U+9DD3.0	fUTF8	鷓
U+9DD3.0	fCNS11643	1-7C2E
U+9DD3.0	fBig5	C5C2
U+9DD3.0	fGB	1-7051
U+9DD3.0	fR/S	 196.11
U+9DD3.0	fTotalStrokes	 22
U+9DD3.0	fCangjie	IFHAF
U+9DD3.0	fFourCorner	07227
U+9DD3.0	fHakka	  zha5; za5
U+9DD3.0	fCantonese	ze3
U+9DD3.0	fMandarin	zhe4
U+9DD3.0	fEnglish	 partridge
U+9DD4.0	fUTF8	鷔
U+9DD4.0	fCNS11643	4-6A25
U+9DD4.0	fGB	3-5E23
U+9DD4.0	fR/S	 196.11
U+9DD4.0	fTotalStrokes	 22; 21
U+9DD4.0	fCangjie	GKHAF
U+9DD4.0	fFourCorner	48320
U+9DD4.0	fHakka	 ngau2; chit7
U+9DD4.0	fCantonese	ngou4; ou4 (also); ngou6
U+9DD4.0	fMandarin	ao2
U+9DD4.0	fEnglish	bird of ill omen, whose presence forebodes ruin to the State. It has a white body and red mouth
U+9DD5.0	fUTF8	鷕
U+9DD5.0	fCNS11643	2-6E72
U+9DD5.0	fBig5	F7C5
U+9DD5.0	fGB	3-5E32
U+9DD5.0	fR/S	 196.11
U+9DD5.0	fTotalStrokes	 22
U+9DD5.0	fCangjie	RGHAF
U+9DD5.0	fFourCorner	60327
U+9DD5.0	fHakka	 miau5; rau5; jeu5; meu5; miau3; jau5
U+9DD5.0	fCantonese	jiu6 (also); jiu5; wai5 (also); jiu4
U+9DD5.0	fMandarin	yao3; wei3
U+9DD5.0	fEnglish	call
U+9DD6.0	fUTF8	鷖
U+9DD6.0	fCNS11643	2-6E70
U+9DD6.0	fBig5	F7C3
U+9DD6.0	fGB	3-5E30
U+9DD6.0	fR/S	 196.11
U+9DD6.0	fTotalStrokes	 22
U+9DD6.0	fCangjie	SEHAF
U+9DD6.0	fFourCorner	77327
U+9DD6.0	fHakka	 ji1; ri1
U+9DD6.0	fCantonese	ji1; ai1; ngai1
U+9DD6.0	fMandarin	yi1; yi4
U+9DD6.0	fEnglish	[1] gull [2] dark blue [3] phoenix
U+9DD7.0	fUTF8	鷗
U+9DD7.0	fCNS11643	1-7C2F
U+9DD7.0	fBig5	C5C3
U+9DD7.0	fGB	1-4538
U+9DD7.0	fR/S	 196.11
U+9DD7.0	fTotalStrokes	 22
U+9DD7.0	fCangjie	SRHAF
U+9DD7.0	fFourCorner	77727
U+9DD7.0	fHakka	  eu1; iu1
U+9DD7.0	fCantonese	au1; ngau1 (also)
U+9DD7.0	fMandarin	ou1; you1
U+9DD7.0	fEnglish	 gull
U+9DD8.0	fUTF8	鷘
U+9DD8.0	fCNS11643	2-6E6F
U+9DD8.0	fBig5	F7C2
U+9DD8.0	fGB	5-5C4C
U+9DD8.0	fR/S	 196.11
U+9DD8.0	fTotalStrokes	 22
U+9DD8.0	fCangjie	DKHAF
U+9DD8.0	fFourCorner	58327
U+9DD8.0	fHakka	 chit7; ts'iit7
U+9DD8.0	fCantonese	cik1
U+9DD8.0	fMandarin	chi4
U+9DD9.0	fUTF8	鷙
U+9DD9.0	fCNS11643	2-6E6E
U+9DD9.0	fBig5	F7C1
U+9DD9.0	fGB	1-703A
U+9DD9.0	fR/S	 196.11
U+9DD9.0	fTotalStrokes	 22
U+9DD9.0	fCangjie	GIHAF
U+9DD9.0	fFourCorner	45327
U+9DD9.0	fHakka	 zhi5; zih5; tje5
U+9DD9.0	fCantonese	zi3
U+9DD9.0	fMandarin	zhi4; zhe2
U+9DD9.0	fEnglish	[1] birds of prey [2] fierce; cruel; violent
U+9DDA.0	fUTF8	鷚
U+9DDA.0	fCNS11643	2-6E67
U+9DDA.0	fBig5	F7BA
U+9DDA.0	fGB	1-7052
U+9DDA.0	fR/S	 196.11
U+9DDA.0	fTotalStrokes	 22
U+9DDA.0	fCangjie	SHHAF
U+9DDA.0	fFourCorner	17227
U+9DDA.0	fHakka	 liu2; liu5
U+9DDA.0	fCantonese	lau6; lau4
U+9DDA.0	fMandarin	liao2; liu4; jiu1; liu2; mou2
U+9DDA.0	fEnglish	pipit (Anthus spp.)
U+9DDB.0	fUTF8	鷛
U+9DDB.0	fCNS11643	2-6E64
U+9DDB.0	fBig5	F7B7
U+9DDB.0	fGB	5-5C52
U+9DDB.0	fR/S	 196.11
U+9DDB.0	fTotalStrokes	 22
U+9DDB.0	fCangjie	IBHAF
U+9DDB.0	fFourCorner	07227
U+9DDB.0	fCantonese	jung4
U+9DDB.0	fMandarin	rong2; yong2
U+9DDC.0	fUTF8	鷜
U+9DDC.0	fCNS11643	2-6E6A
U+9DDC.0	fBig5	F7BD
U+9DDC.0	fGB	3-5E21
U+9DDC.0	fR/S	 196.11
U+9DDC.0	fTotalStrokes	 22
U+9DDC.0	fCangjie	LVHAF
U+9DDC.0	fFourCorner	57427
U+9DDC.0	fCantonese	leoi4; lau4
U+9DDC.0	fMandarin	lou2
U+9DDD.0	fUTF8	鷝
U+9DDD.0	fCNS11643	2-6E73
U+9DDD.0	fBig5	F7C6
U+9DDD.0	fGB	E-483E
U+9DDD.0	fR/S	 196.11
U+9DDD.0	fTotalStrokes	 22
U+9DDD.0	fCangjie	WJHAF
U+9DDD.0	fFourCorner	67427
U+9DDD.0	fHakka	 bit7
U+9DDD.0	fCantonese	bat1
U+9DDD.0	fMandarin	bi4
U+9DDE.0	fUTF8	鷞
U+9DDE.0	fCNS11643	2-6E66
U+9DDE.0	fBig5	F7B9
U+9DDE.0	fGB	5-5C4E
U+9DDE.0	fR/S	 196.11
U+9DDE.0	fTotalStrokes	 22
U+9DDE.0	fCangjie	KKHAF
U+9DDE.0	fFourCorner	47027
U+9DDE.0	fHakka	 song1;  song3
U+9DDE.0	fCantonese	soeng1; song2
U+9DDE.0	fMandarin	shuang1; shuang3
U+9DDE.0	fEnglish	the turquoise kingfisher
U+9DDF.0	fUTF8	鷟
U+9DDF.0	fCNS11643	2-6E6C
U+9DDF.0	fBig5	F7BF
U+9DDF.0	fGB	3-5E33
U+9DDF.0	fR/S	 196.11
U+9DDF.0	fTotalStrokes	 22
U+9DDF.0	fCangjie	YKHAF
U+9DDF.0	fFourCorner	08327
U+9DDF.0	fHakka	 cuk8
U+9DDF.0	fCantonese	zok6; zuk1; zok3
U+9DDF.0	fMandarin	zhuo2; zhuo4
U+9DDF.0	fEnglish	a kind of water bird
U+9DE0.0	fUTF8	鷠
U+9DE0.0	fCNS11643	5-7545
U+9DE0.0	fGB	E-483F
U+9DE0.0	fR/S	 196.11; 195.11
U+9DE0.0	fTotalStrokes	 22
U+9DE0.0	fCangjie	NFHAF
U+9DE0.0	fFourCorner	27320
U+9DE0.0	fCantonese	jyu4
U+9DE0.0	fMandarin	yu2
U+9DE1.0	fUTF8	鷡
U+9DE1.0	fCNS11643	2-6F77
U+9DE1.0	fBig5	F869
U+9DE1.0	fGB	E-4840
U+9DE1.0	fR/S	 196.12
U+9DE1.0	fTotalStrokes	 23
U+9DE1.0	fCangjie	OFHAF
U+9DE1.0	fFourCorner	87327
U+9DE1.0	fCantonese	mou4
U+9DE1.0	fMandarin	wu2
U+9DE2.0	fUTF8	鷢
U+9DE2.0	fCNS11643	2-6F7C
U+9DE2.0	fBig5	F86E
U+9DE2.0	fGB	3-3044
U+9DE2.0	fR/S	 196.12
U+9DE2.0	fTotalStrokes	 23
U+9DE2.0	fCangjie	MOHAF
U+9DE2.0	fFourCorner	71327
U+9DE2.0	fCantonese	kyut3
U+9DE2.0	fMandarin	jue2
U+9DE3.0	fUTF8	鷣
U+9DE3.0	fCNS11643	2-6F72
U+9DE3.0	fBig5	F864
U+9DE3.0	fGB	E-4841
U+9DE3.0	fR/S	 196.12
U+9DE3.0	fTotalStrokes	 23
U+9DE3.0	fCangjie	MJHAF
U+9DE3.0	fFourCorner	17427
U+9DE3.0	fCantonese	jam4; jiu6
U+9DE3.0	fMandarin	yin2; yao4
U+9DE4.0	fUTF8	鷤
U+9DE4.0	fCNS11643	2-6F75
U+9DE4.0	fBig5	F867
U+9DE4.0	fGB	5-5C40
U+9DE4.0	fR/S	 196.12
U+9DE4.0	fTotalStrokes	 23
U+9DE4.0	fCangjie	RJHAF
U+9DE4.0	fFourCorner	67427
U+9DE4.0	fCantonese	tai4; taan4; dai6; taan5
U+9DE4.0	fMandarin	ti2; quan2; tan2; di4
U+9DE5.0	fUTF8	鷥
U+9DE5.0	fCNS11643	1-7C5A
U+9DE5.0	fBig5	C5EE
U+9DE5.0	fGB	1-7038
U+9DE5.0	fR/S	 196.12
U+9DE5.0	fTotalStrokes	 23
U+9DE5.0	fCangjie	VFHAF
U+9DE5.0	fFourCorner	22327
U+9DE5.0	fHakka	 sii1; zih1; su1; se1
U+9DE5.0	fCantonese	si1
U+9DE5.0	fMandarin	si1
U+9DE5.0	fEnglish	the egret
U+9DE6.0	fUTF8	鷦
U+9DE6.0	fCNS11643	2-6F79
U+9DE6.0	fBig5	F86B
U+9DE6.0	fGB	1-7054
U+9DE6.0	fR/S	 196.12
U+9DE6.0	fTotalStrokes	 23
U+9DE6.0	fCangjie	OFHAF
U+9DE6.0	fFourCorner	27327
U+9DE6.0	fHakka	  ziau1; zeu1; ziau2
U+9DE6.0	fCantonese	ziu1
U+9DE6.0	fMandarin	jiao1
U+9DE6.0	fEnglish	wren
U+9DE7.0	fUTF8	鷧
U+9DE7.0	fCNS11643	4-6B46
U+9DE7.0	fGB	E-4842
U+9DE7.0	fR/S	  196.12
U+9DE7.0	fTotalStrokes	 23
U+9DE7.0	fCangjie	GTHAF
U+9DE7.0	fFourCorner	47120
U+9DE7.0	fHakka	 ji5; ri5; rit7; jit7
U+9DE7.0	fCantonese	ji3
U+9DE7.0	fMandarin	yi4
U+9DE7.0	fEnglish	cormorant
U+9DE8.0	fUTF8	鷨
U+9DE8.0	fCNS11643	2-7022
U+9DE8.0	fBig5	F872
U+9DE8.0	fGB	E-4843
U+9DE8.0	fR/S	 196.12
U+9DE8.0	fTotalStrokes	 23
U+9DE8.0	fCangjie	TJHAF
U+9DE8.0	fFourCorner	47527
U+9DE8.0	fCantonese	waa4
U+9DE8.0	fMandarin	hua1; hua2; huo4
U+9DE9.0	fUTF8	鷩
U+9DE9.0	fCNS11643	2-6E6D
U+9DE9.0	fBig5	F7C0
U+9DE9.0	fGB	3-5E31
U+9DE9.0	fR/S	 196.12
U+9DE9.0	fTotalStrokes	 23
U+9DE9.0	fCangjie	FKHAF
U+9DE9.0	fFourCorner	97327
U+9DE9.0	fHakka	  biet7; bet7
U+9DE9.0	fCantonese	bit3 (also); bai3; bai6
U+9DE9.0	fMandarin	bi4; bie1; chang3
U+9DE9.0	fEnglish	a kind of pheasant
U+9DEA.0	fUTF8	鷪
U+9DEA.0	fCNS11643	4-6B4A
U+9DEA.0	fGB	E-4844
U+9DEA.0	fR/S	 196.12
U+9DEA.0	fTotalStrokes	 23
U+9DEA.0	fCangjie	BUBHF
U+9DEA.0	fFourCorner	66320
U+9DEA.0	fCantonese	ang1; ngang1 (also)
U+9DEA.0	fMandarin	ying1
U+9DEB.0	fUTF8	鷫
U+9DEB.0	fCNS11643	2-6F73
U+9DEB.0	fBig5	F865
U+9DEB.0	fGB	3-5D6D
U+9DEB.0	fR/S	 196.12; 196.13
U+9DEB.0	fTotalStrokes	 23
U+9DEB.0	fCangjie	LXHAF
U+9DEB.0	fFourCorner	57227
U+9DEB.0	fHakka	 siuk7; suk7
U+9DEB.0	fCantonese	suk1
U+9DEB.0	fMandarin	su4
U+9DEB.0	fEnglish	a kind of wild swan
U+9DEC.0	fUTF8	鷬
U+9DEC.0	fCNS11643	2-6F7D
U+9DEC.0	fBig5	F86F
U+9DEC.0	fGB	E-4845
U+9DEC.0	fR/S	 196.12
U+9DEC.0	fTotalStrokes	 23
U+9DEC.0	fCangjie	TCHAF
U+9DEC.0	fFourCorner	47827
U+9DEC.0	fCantonese	wong4
U+9DEC.0	fMandarin	huang2
U+9DED.0	fUTF8	鷭
U+9DED.0	fCNS11643	2-7023
U+9DED.0	fBig5	F873
U+9DED.0	fGB	3-5E37
U+9DED.0	fR/S	 196.12
U+9DED.0	fTotalStrokes	 23
U+9DED.0	fCangjie	HWHAF
U+9DED.0	fFourCorner	27627
U+9DED.0	fCantonese	faan4; faan1
U+9DED.0	fMandarin	fan2
U+9DEE.0	fUTF8	鷮
U+9DEE.0	fCNS11643	2-6F78
U+9DEE.0	fBig5	F86A
U+9DEE.0	fGB	3-5D42
U+9DEE.0	fR/S	 196.12
U+9DEE.0	fTotalStrokes	 23
U+9DEE.0	fCangjie	HBHAF
U+9DEE.0	fFourCorner	27227
U+9DEE.0	fHakka	  giau1; kieu1
U+9DEE.0	fCantonese	giu1; kiu1
U+9DEE.0	fMandarin	jiao1; qiao2
U+9DEE.0	fEnglish	a species of long-tailed pheasant
U+9DEF.0	fUTF8	鷯
U+9DEF.0	fCNS11643	2-6F71
U+9DEF.0	fBig5	F863
U+9DEF.0	fGB	1-7053
U+9DEF.0	fR/S	 196.12
U+9DEF.0	fTotalStrokes	 23
U+9DEF.0	fCangjie	KFHAF
U+9DEF.0	fFourCorner	47927
U+9DEF.0	fHakka	  liau2;  liau5
U+9DEF.0	fCantonese	liu4; liu1
U+9DEF.0	fMandarin	liao2; liao4
U+9DEF.0	fEnglish	correct form of &#x wren;
U+9DF0.0	fUTF8	鷰
U+9DF0.0	fCNS11643	2-6F7B
U+9DF0.0	fBig5	F86D
U+9DF0.0	fGB	E-4846
U+9DF0.0	fR/S	 196.12
U+9DF0.0	fTotalStrokes	 23
U+9DF0.0	fCangjie	TLPF
U+9DF0.0	fFourCorner	44327
U+9DF0.0	fCantonese	jin3
U+9DF0.0	fMandarin	yan4; yan1
U+9DF1.0	fUTF8	鷱
U+9DF1.0	fCNS11643	4-6B4B
U+9DF1.0	fGB	E-4847
U+9DF1.0	fR/S	  196.12
U+9DF1.0	fTotalStrokes	 23
U+9DF1.0	fCangjie	HJHAF
U+9DF1.0	fFourCorner	27420
U+9DF1.0	fCantonese	gou1
U+9DF1.0	fMandarin	gao1; kao1
U+9DF2.0	fUTF8	鷲
U+9DF2.0	fCNS11643	2-6F7A
U+9DF2.0	fBig5	F86C
U+9DF2.0	fGB	1-7055
U+9DF2.0	fR/S	 196.12
U+9DF2.0	fTotalStrokes	 23
U+9DF2.0	fCangjie	YUHAF
U+9DF2.0	fFourCorner	03327
U+9DF2.0	fHakka	 ciu5; ciu6; ciu3
U+9DF2.0	fCantonese	zau6
U+9DF2.0	fMandarin	jiu4
U+9DF2.0	fEnglish	 vulture
U+9DF3.0	fUTF8	鷳
U+9DF3.0	fCNS11643	2-7021
U+9DF3.0	fBig5	F871
U+9DF3.0	fGB	1-7042
U+9DF3.0	fR/S	 196.12
U+9DF3.0	fTotalStrokes	 23
U+9DF3.0	fCangjie	ABHAF
U+9DF3.0	fFourCorner	77227
U+9DF3.0	fHakka	 han2
U+9DF3.0	fCantonese	haan4
U+9DF3.0	fMandarin	xian2
U+9DF3.0	fEnglish	silver pheasant; phasianus nycth
U+9DF4.0	fUTF8	鷴
U+9DF4.0	fCNS11643	2-6F7E
U+9DF4.0	fBig5	F870
U+9DF4.0	fGB	E-4848
U+9DF4.0	fR/S	 196.12
U+9DF4.0	fTotalStrokes	 23
U+9DF4.0	fCangjie	ADHAF
U+9DF4.0	fFourCorner	77227
U+9DF4.0	fHakka	  han2
U+9DF4.0	fCantonese	haan4
U+9DF4.0	fMandarin	xian2
U+9DF4.0	fEnglish	silver pheasant badge worn by civil officials of the 5th grade
U+9DF5.0	fUTF8	鷵
U+9DF5.0	fCNS11643	2-6E71
U+9DF5.0	fBig5	F7C4
U+9DF5.0	fGB	E-4849
U+9DF5.0	fR/S	 196.12
U+9DF5.0	fTotalStrokes	 23
U+9DF5.0	fCangjie	SAHAF
U+9DF5.0	fFourCorner	77227
U+9DF5.0	fCantonese	tou4
U+9DF5.0	fMandarin	tu2
U+9DF6.0	fUTF8	鷶
U+9DF6.0	fCNS11643	2-6F76
U+9DF6.0	fBig5	F868
U+9DF6.0	fGB	E-484A
U+9DF6.0	fR/S	 196.12
U+9DF6.0	fTotalStrokes	 23
U+9DF6.0	fCangjie	WCHAF
U+9DF6.0	fFourCorner	67827
U+9DF6.0	fCantonese	maai5
U+9DF6.0	fMandarin	mai3
U+9DF7.0	fUTF8	鷷
U+9DF7.0	fCNS11643	2-6F70
U+9DF7.0	fBig5	F862
U+9DF7.0	fGB	E-484B
U+9DF7.0	fR/S	 196.12
U+9DF7.0	fTotalStrokes	 23
U+9DF7.0	fCangjie	TIHAF
U+9DF7.0	fFourCorner	87327
U+9DF7.0	fCantonese	zeon1
U+9DF7.0	fMandarin	zun1; cun2; zhen1
U+9DF8.0	fUTF8	鷸
U+9DF8.0	fCNS11643	2-6F74
U+9DF8.0	fBig5	F866
U+9DF8.0	fGB	1-7056
U+9DF8.0	fR/S	 196.12
U+9DF8.0	fTotalStrokes	 23
U+9DF8.0	fCangjie	NBHAF
U+9DF8.0	fFourCorner	17227
U+9DF8.0	fHakka	  lut8; jut8; rut8
U+9DF8.0	fCantonese	wat6; neot6 (also); leot6 (also); jyut6
U+9DF8.0	fMandarin	yu4; shu4
U+9DF8.0	fEnglish	 snipe
U+9DF9.0	fUTF8	鷹
U+9DF9.0	fCNS11643	1-7C79
U+9DF9.0	fBig5	C64E
U+9DF9.0	fGB	1-5325
U+9DF9.0	fR/S	 196.13
U+9DF9.0	fTotalStrokes	 24
U+9DF9.0	fCangjie	IOGF
U+9DF9.0	fFourCorner	00227
U+9DF9.0	fHakka	  jin1; en1; rin1; jin5
U+9DF9.0	fCantonese	jing1
U+9DF9.0	fMandarin	ying1
U+9DF9.0	fEnglish	eagle; falcon
U+9DFA.0	fUTF8	鷺
U+9DFA.0	fCNS11643	1-7C7A
U+9DFA.0	fBig5	C64F
U+9DFA.0	fGB	1-7058
U+9DFA.0	fR/S	 196.12
U+9DFA.0	fTotalStrokes	 23
U+9DFA.0	fCangjie	RRHAF
U+9DFA.0	fFourCorner	67327
U+9DFA.0	fHakka	  lu5; lu6
U+9DFA.0	fCantonese	lou6
U+9DFA.0	fMandarin	lu4
U+9DFA.0	fEnglish	 egret (Egretta garzetta)
U+9DFB.0	fUTF8	鷻
U+9DFB.0	fCNS11643	2-6F6F
U+9DFB.0	fBig5	F861
U+9DFB.0	fGB	5-5C58
U+9DFB.0	fR/S	 196.12
U+9DFB.0	fTotalStrokes	 23
U+9DFB.0	fCangjie	HFYDK
U+9DFB.0	fFourCorner	28340
U+9DFB.0	fCantonese	tyun4
U+9DFB.0	fMandarin	tuan2; tui2
U+9DFC.0	fUTF8	鷼
U+9DFC.0	fCNS11643	3-606C
U+9DFC.0	fGB	E-484C
U+9DFC.0	fR/S	  196.12
U+9DFC.0	fTotalStrokes	 23
U+9DFC.0	fCangjie	AAHAF
U+9DFC.0	fFourCorner	77220
U+9DFC.0	fCantonese	haan4
U+9DFC.0	fMandarin	xian2
U+9DFD.0	fUTF8	鷽
U+9DFD.0	fCNS11643	2-7074
U+9DFD.0	fBig5	F8E6
U+9DFD.0	fGB	3-5D35
U+9DFD.0	fR/S	 196.13
U+9DFD.0	fTotalStrokes	 24
U+9DFD.0	fCangjie	HBHAF
U+9DFD.0	fFourCorner	77327
U+9DFD.0	fHakka	 hok8; hok7
U+9DFD.0	fCantonese	hok6
U+9DFD.0	fMandarin	xue2; hua1; wo4; wu4
U+9DFD.0	fEnglish	 tanager
U+9DFE.0	fUTF8	鷾
U+9DFE.0	fCNS11643	2-706B
U+9DFE.0	fBig5	F8DD
U+9DFE.0	fGB	5-5C5B
U+9DFE.0	fR/S	 196.13
U+9DFE.0	fTotalStrokes	 24
U+9DFE.0	fCangjie	YPHAF
U+9DFE.0	fFourCorner	07327
U+9DFE.0	fCantonese	ji3
U+9DFE.0	fMandarin	yi4
U+9DFE.0	fEnglish	a swallow
U+9DFF.0	fUTF8	鷿
U+9DFF.0	fCNS11643	2-7073
U+9DFF.0	fBig5	F8E5
U+9DFF.0	fGB	E-484D
U+9DFF.0	fR/S	 196.13
U+9DFF.0	fTotalStrokes	 24
U+9DFF.0	fCangjie	SJHAF
U+9DFF.0	fFourCorner	70327
U+9DFF.0	fHakka	  pit7
U+9DFF.0	fCantonese	pik1
U+9DFF.0	fMandarin	pi4; bo4
U+9DFF.0	fEnglish	[1] a kind of waterfowl [2] the brown grebe
U+9E00.0	fUTF8	鸀
U+9E00.0	fCNS11643	2-7070
U+9E00.0	fBig5	F8E2
U+9E00.0	fGB	3-5E3F
U+9E00.0	fR/S	 196.13
U+9E00.0	fTotalStrokes	 24
U+9E00.0	fCangjie	WIHAF
U+9E00.0	fFourCorner	67127
U+9E00.0	fHakka	 chuk7; shuk8; zuk7; suk8
U+9E00.0	fCantonese	suk6; zuk1; zok6 (also)
U+9E00.0	fMandarin	shu2; zhu2; zhuo2; du2; shu3
U+9E00.0	fEnglish	 blackbird
U+9E01.0	fUTF8	鸁
U+9E01.0	fCNS11643	2-7071
U+9E01.0	fBig5	F8E3
U+9E01.0	fGB	3-5E41
U+9E01.0	fR/S	 196.13
U+9E01.0	fTotalStrokes	 24
U+9E01.0	fCangjie	YNHAF
U+9E01.0	fFourCorner	00217
U+9E01.0	fHakka	 lo3
U+9E01.0	fCantonese	lo4; lo5
U+9E01.0	fMandarin	luo2; luo3
U+9E01.0	fEnglish	grebe
U+9E02.0	fUTF8	鸂
U+9E02.0	fCNS11643	2-706A
U+9E02.0	fBig5	F8DC
U+9E02.0	fGB	3-5E42
U+9E02.0	fR/S	 196.13
U+9E02.0	fTotalStrokes	 24
U+9E02.0	fCangjie	EBKF
U+9E02.0	fFourCorner	37127
U+9E02.0	fHakka	 hai1; k'ie1
U+9E02.0	fCantonese	kai1
U+9E02.0	fMandarin	qi1; xi1
U+9E02.0	fEnglish	a kind of water bird resembling the mandarin duck
U+9E03.0	fUTF8	鸃
U+9E03.0	fCNS11643	2-706D
U+9E03.0	fBig5	F8DF
U+9E03.0	fGB	3-5C77
U+9E03.0	fR/S	 196.13
U+9E03.0	fTotalStrokes	 24
U+9E03.0	fCangjie	HFTGI
U+9E03.0	fFourCorner	28353
U+9E03.0	fHakka	  ngi2
U+9E03.0	fCantonese	ji4
U+9E03.0	fMandarin	yi2
U+9E03.0	fEnglish	a small kind of pheasant; crow-pheasant
U+9E04.0	fUTF8	鸄
U+9E04.0	fCNS11643	2-7075
U+9E04.0	fBig5	F8E7
U+9E04.0	fGB	E-484E
U+9E04.0	fR/S	 196.13
U+9E04.0	fTotalStrokes	 24
U+9E04.0	fCangjie	HKHAF
U+9E04.0	fFourCorner	28327
U+9E04.0	fCantonese	gik1
U+9E04.0	fMandarin	ji2; ji1; jiao3; jiao4
U+9E05.0	fUTF8	鸅
U+9E05.0	fCNS11643	2-706F
U+9E05.0	fBig5	F8E1
U+9E05.0	fGB	5-5C2A
U+9E05.0	fR/S	 196.13
U+9E05.0	fTotalStrokes	 24
U+9E05.0	fCangjie	WJHAF
U+9E05.0	fFourCorner	67427
U+9E05.0	fHakka	 cet8
U+9E05.0	fCantonese	zaak6; zak6
U+9E05.0	fMandarin	zhe2
U+9E05.0	fEnglish	the white pelican
U+9E06.0	fUTF8	鸆
U+9E06.0	fCNS11643	2-706E
U+9E06.0	fBig5	F8E0
U+9E06.0	fGB	3-5E3C
U+9E06.0	fR/S	 196.13
U+9E06.0	fTotalStrokes	 24
U+9E06.0	fCangjie	YKHAF
U+9E06.0	fFourCorner	27227
U+9E06.0	fCantonese	jyu4
U+9E06.0	fMandarin	yu2
U+9E07.0	fUTF8	鸇
U+9E07.0	fCNS11643	2-706C
U+9E07.0	fBig5	F8DE
U+9E07.0	fGB	3-5E40
U+9E07.0	fR/S	 196.13
U+9E07.0	fTotalStrokes	 24
U+9E07.0	fCangjie	YMHAF
U+9E07.0	fFourCorner	07127
U+9E07.0	fHakka	 zhan1; zan1; zen1; zhen1
U+9E07.0	fCantonese	zin1
U+9E07.0	fMandarin	zhan1; zhen1
U+9E07.0	fEnglish	a kind of bird of prey
U+9E08.0	fUTF8	鸈
U+9E08.0	fCNS11643	3-6135
U+9E08.0	fGB	E-484F
U+9E08.0	fR/S	  196.13
U+9E08.0	fTotalStrokes	 24
U+9E08.0	fCangjie	TDHAF
U+9E08.0	fFourCorner	37920
U+9E08.0	fCantonese	jip6
U+9E08.0	fMandarin	ye4
U+9E09.0	fUTF8	鸉
U+9E09.0	fCNS11643	2-7072
U+9E09.0	fBig5	F8E4
U+9E09.0	fGB	3-5D4B
U+9E09.0	fR/S	 196.13
U+9E09.0	fTotalStrokes	 24
U+9E09.0	fCangjie	DHHAF
U+9E09.0	fFourCorner	46327
U+9E09.0	fCantonese	joeng4
U+9E09.0	fMandarin	yang2; ying2
U+9E0A.0	fUTF8	鸊
U+9E0A.0	fCNS11643	4-6C56
U+9E0A.0	fGB	3-5E43
U+9E0A.0	fR/S	 196.13
U+9E0A.0	fTotalStrokes	 24
U+9E0A.0	fCangjie	SRYJF
U+9E0A.0	fFourCorner	77220
U+9E0A.0	fCantonese	pik1
U+9E0A.0	fMandarin	pi4; bi4
U+9E0A.0	fEnglish	Podiceps ruficolus (a kind of duck)
U+9E0B.0	fUTF8	鸋
U+9E0B.0	fCNS11643	2-714C
U+9E0B.0	fBig5	F95D
U+9E0B.0	fGB	3-5D36
U+9E0B.0	fR/S	 196.14
U+9E0B.0	fTotalStrokes	 25
U+9E0B.0	fCangjie	JNHAF
U+9E0B.0	fFourCorner	37227
U+9E0B.0	fCantonese	ning4; ling4 (also)
U+9E0B.0	fMandarin	ning2; ning4
U+9E0C.0	fUTF8	鸌
U+9E0C.0	fCNS11643	4-6D3A
U+9E0C.0	fGB	1-7057
U+9E0C.0	fR/S	 196.14
U+9E0C.0	fTotalStrokes	 25
U+9E0C.0	fCangjie	HFTOE
U+9E0C.0	fFourCorner	74340
U+9E0C.0	fHakka	 wok7; fu6; fu5; vok7
U+9E0C.0	fCantonese	wok3; wok6 (also)
U+9E0C.0	fMandarin	huo4; hu4
U+9E0C.0	fEnglish	a water-fowl, whose cry brings cry
U+9E0D.0	fUTF8	鸍
U+9E0D.0	fCNS11643	2-714D
U+9E0D.0	fBig5	F95E
U+9E0D.0	fGB	E-4850
U+9E0D.0	fR/S	 196.14
U+9E0D.0	fTotalStrokes	 25
U+9E0D.0	fCangjie	MBHAF
U+9E0D.0	fFourCorner	17227
U+9E0D.0	fCantonese	mei4; si1
U+9E0D.0	fMandarin	mi2; shi1
U+9E0E.0	fUTF8	鸎
U+9E0E.0	fCNS11643	3-615D
U+9E0E.0	fGB	E-4851
U+9E0E.0	fR/S	 196.14
U+9E0E.0	fTotalStrokes	 25
U+9E0E.0	fCangjie	BCHAF
U+9E0E.0	fFourCorner	66320
U+9E0E.0	fHakka	 rin1; jin1; jin2
U+9E0E.0	fCantonese	ang1; ngang1 (also)
U+9E0E.0	fMandarin	ying1
U+9E0E.0	fEnglish	 the mango-bird the Chinese oriole
U+9E0F.0	fUTF8	鸏
U+9E0F.0	fCNS11643	2-714F
U+9E0F.0	fBig5	F960
U+9E0F.0	fGB	3-5E3A
U+9E0F.0	fR/S	 196.14
U+9E0F.0	fTotalStrokes	 25
U+9E0F.0	fCangjie	TOHAF
U+9E0F.0	fFourCorner	47227
U+9E0F.0	fHakka	 mung2
U+9E0F.0	fCantonese	mung4
U+9E0F.0	fMandarin	meng2; meng3; mang2
U+9E0F.0	fEnglish	fledgelings of water-birds
U+9E10.0	fUTF8	鸐
U+9E10.0	fCNS11643	2-714E
U+9E10.0	fBig5	F95F
U+9E10.0	fGB	5-5C5C
U+9E10.0	fR/S	 196.14
U+9E10.0	fTotalStrokes	 25
U+9E10.0	fCangjie	SGHAF
U+9E10.0	fFourCorner	17227
U+9E10.0	fHakka	  tit8
U+9E10.0	fCantonese	dik6; tik6
U+9E10.0	fMandarin	di2; zhuo2
U+9E10.0	fEnglish	the long-tailed, or Reeve's pheasant
U+9E11.0	fUTF8	鸑
U+9E11.0	fCNS11643	2-7151
U+9E11.0	fBig5	F962
U+9E11.0	fGB	3-5D7D
U+9E11.0	fR/S	 196.14
U+9E11.0	fTotalStrokes	 25
U+9E11.0	fCangjie	KKHAF
U+9E11.0	fFourCorner	23327
U+9E11.0	fHakka	  ngok8
U+9E11.0	fCantonese	ngok6; ok6 (also)
U+9E11.0	fMandarin	yue4
U+9E11.0	fEnglish	kind of large waterfowl with red
U+9E12.0	fUTF8	鸒
U+9E12.0	fCNS11643	2-7150
U+9E12.0	fBig5	F961
U+9E12.0	fGB	E-4852
U+9E12.0	fR/S	 196.14
U+9E12.0	fTotalStrokes	 25
U+9E12.0	fCangjie	HCHAF
U+9E12.0	fFourCorner	77327
U+9E12.0	fHakka	 ji2; ri2
U+9E12.0	fCantonese	jyu4 (also); jyu6; jyu5
U+9E12.0	fMandarin	yu2; yu4
U+9E12.0	fEnglish	the eastern jackdaw
U+9E13.0	fUTF8	鸓
U+9E13.0	fCNS11643	2-716C
U+9E13.0	fBig5	F97C
U+9E13.0	fGB	3-5E45
U+9E13.0	fR/S	 196.15
U+9E13.0	fTotalStrokes	 26
U+9E13.0	fCangjie	WWHAF
U+9E13.0	fFourCorner	67627
U+9E13.0	fHakka	 lui3; lui2
U+9E13.0	fCantonese	leoi5; leoi4
U+9E13.0	fMandarin	lei3; lei2
U+9E13.0	fEnglish	general term for flying squirrels, flying foxes etc.
U+9E14.0	fUTF8	鸔
U+9E14.0	fCNS11643	2-716B
U+9E14.0	fBig5	F97B
U+9E14.0	fGB	E-4853
U+9E14.0	fR/S	 196.15
U+9E14.0	fTotalStrokes	 26
U+9E14.0	fCangjie	HFATE
U+9E14.0	fFourCorner	26332
U+9E14.0	fCantonese	buk1; bok3; bou6
U+9E14.0	fMandarin	bao4; bu3; bo1
U+9E15.0	fUTF8	鸕
U+9E15.0	fCNS11643	2-7227
U+9E15.0	fBig5	F9B7
U+9E15.0	fGB	1-7035
U+9E15.0	fR/S	 196.16
U+9E15.0	fTotalStrokes	 27
U+9E15.0	fCangjie	YTHAF
U+9E15.0	fFourCorner	27227
U+9E15.0	fHakka	 lu2; lu3; lu5
U+9E15.0	fCantonese	lou4
U+9E15.0	fMandarin	lu2
U+9E15.0	fEnglish	 cormorant
U+9E16.0	fUTF8	鸖
U+9E16.0	fCNS11643	3-6229
U+9E16.0	fGB	E-4854
U+9E16.0	fR/S	 196.16
U+9E16.0	fTotalStrokes	 27
U+9E16.0	fCangjie	MGHAF
U+9E16.0	fFourCorner	17220
U+9E16.0	fCantonese	hok6
U+9E16.0	fMandarin	he4
U+9E17.0	fUTF8	鸗
U+9E17.0	fCNS11643	2-7228
U+9E17.0	fBig5	F9B8
U+9E17.0	fGB	3-587B
U+9E17.0	fR/S	 196.16
U+9E17.0	fTotalStrokes	 27
U+9E17.0	fCangjie	YPHAF
U+9E17.0	fFourCorner	01327
U+9E17.0	fCantonese	lung4
U+9E17.0	fMandarin	long2
U+9E18.0	fUTF8	鸘
U+9E18.0	fCNS11643	3-6230
U+9E18.0	fGB	3-5E46
U+9E18.0	fR/S	 196.17
U+9E18.0	fTotalStrokes	 28
U+9E18.0	fCangjie	MUHAF
U+9E18.0	fFourCorner	17920
U+9E18.0	fHakka	 song1
U+9E18.0	fCantonese	soeng1
U+9E18.0	fMandarin	shuang1
U+9E18.0	fEnglish	eagle
U+9E19.0	fUTF8	鸙
U+9E19.0	fCNS11643	2-7234
U+9E19.0	fBig5	F9C5
U+9E19.0	fGB	E-4855
U+9E19.0	fR/S	 196.17
U+9E19.0	fTotalStrokes	 28
U+9E19.0	fCangjie	OBHAF
U+9E19.0	fFourCorner	87227
U+9E19.0	fCantonese	joek6
U+9E19.0	fMandarin	yue4
U+9E1A.0	fUTF8	鸚
U+9E1A.0	fCNS11643	1-7D45
U+9E1A.0	fBig5	C678
U+9E1A.0	fGB	1-7050
U+9E1A.0	fR/S	 196.17
U+9E1A.0	fTotalStrokes	 28
U+9E1A.0	fCangjie	BVHAF
U+9E1A.0	fFourCorner	67427
U+9E1A.0	fHakka	 jin1;  en1; rin1; {en1}
U+9E1A.0	fCantonese	jing1
U+9E1A.0	fMandarin	ying1
U+9E1A.0	fEnglish	 parrot
U+9E1B.0	fUTF8	鸛
U+9E1B.0	fCNS11643	1-7D49
U+9E1B.0	fBig5	C67C
U+9E1B.0	fGB	1-7059
U+9E1B.0	fR/S	 196.18
U+9E1B.0	fTotalStrokes	 29
U+9E1B.0	fCangjie	TGHAF
U+9E1B.0	fFourCorner	47227
U+9E1B.0	fHakka	 kwan5; gon5; gwon5
U+9E1B.0	fCantonese	gun3; gwun3 (also)
U+9E1B.0	fMandarin	guan4; huan1; quan2
U+9E1B.0	fEnglish	 stork
U+9E1C.0	fUTF8	鸜
U+9E1C.0	fCNS11643	3-6238
U+9E1C.0	fGB	5-5C5D
U+9E1C.0	fR/S	 196.18; 196.16
U+9E1C.0	fTotalStrokes	 29; 27
U+9E1C.0	fCangjie	BGHAF
U+9E1C.0	fFourCorner	67220
U+9E1C.0	fHakka	 ki1
U+9E1C.0	fCantonese	keoi4
U+9E1C.0	fMandarin	qu2; quan2
U+9E1C.0	fEnglish	a species of mina (minah, mynah)
U+9E1D.0	fUTF8	鸝
U+9E1D.0	fCNS11643	2-723D
U+9E1D.0	fBig5	F9CF
U+9E1D.0	fGB	1-703F
U+9E1D.0	fR/S	 196.19
U+9E1D.0	fTotalStrokes	 30
U+9E1D.0	fCangjie	MPHAF
U+9E1D.0	fFourCorner	17227
U+9E1D.0	fHakka	  li2
U+9E1D.0	fCantonese	lei4
U+9E1D.0	fMandarin	li2
U+9E1D.0	fEnglish	correct form of &#x Chinese oriole;
U+9E1E.0	fUTF8	鸞
U+9E1E.0	fCNS11643	1-7D4A
U+9E1E.0	fBig5	C67D
U+9E1E.0	fGB	1-703D
U+9E1E.0	fR/S	 196.19
U+9E1E.0	fTotalStrokes	 30
U+9E1E.0	fCangjie	VFHAF
U+9E1E.0	fFourCorner	22327
U+9E1E.0	fHakka	 lan2; lan3; lon2
U+9E1E.0	fCantonese	lyun4
U+9E1E.0	fMandarin	luan2
U+9E1E.0	fEnglish	[1] luan, a fabulous bird related to the phoenix [2] the bells at horses' bits
U+9E1F.0	fUTF8	鸟
U+9E1F.0	fGB	0-4471
U+9E1F.0	fR/S	 196.0
U+9E1F.0	fTotalStrokes	 5
U+9E1F.0	fCangjie	PYSM
U+9E1F.0	fHakka	 ngiau3; niau3; {ngiau1}; {niau1}; diau1
U+9E1F.0	fCantonese	niu5; liu5 (also); diu2; tiu2
U+9E1F.0	fMandarin	niao3; diao3; jue2; dao3
U+9E1F.0	fEnglish	simplified form of &#x9CE5;
U+9E20.0	fUTF8	鸠
U+9E20.0	fGB	0-702F
U+9E20.0	fR/S	 196.2
U+9E20.0	fTotalStrokes	 7
U+9E20.0	fCangjie	KNPYM
U+9E20.0	fHakka	 chim2; geu1
U+9E20.0	fCantonese	gau1; kau1 (also); kau2
U+9E20.0	fMandarin	jiu1; zhi4; qiu1
U+9E20.0	fEnglish	simplified form of &#x9CE9;
U+9E21.0	fUTF8	鸡
U+9E21.0	fGB	0-3C26
U+9E21.0	fR/S	 196.2
U+9E21.0	fTotalStrokes	 7
U+9E21.0	fCangjie	EPYM
U+9E21.0	fHakka	 gai1; ge1
U+9E21.0	fCantonese	gai1
U+9E21.0	fMandarin	ji1
U+9E21.0	fEnglish	simplified form of &#x9DC4;
U+9E22.0	fUTF8	鸢
U+9E22.0	fGB	0-7030
U+9E22.0	fR/S	 196.3
U+9E22.0	fTotalStrokes	 8
U+9E22.0	fCangjie	IPPYM
U+9E22.0	fHakka	 jen1
U+9E22.0	fCantonese	jyun1; jyun4 (also)
U+9E22.0	fMandarin	yuan1; qian4; yuan2
U+9E22.0	fEnglish	simplified form of &#x9CF6;
U+9E23.0	fUTF8	鸣
U+9E23.0	fGB	0-4379
U+9E23.0	fR/S	 30.5; 196.3
U+9E23.0	fTotalStrokes	 8
U+9E23.0	fCangjie	RPYM
U+9E23.0	fHakka	 min2
U+9E23.0	fCantonese	ming4
U+9E23.0	fMandarin	ming2; ming4
U+9E23.0	fEnglish	simplified form of &#x9CF4;
U+9E24.0	fUTF8	鸤
U+9E24.0	fGB	8-2C63
U+9E24.0	fR/S	 196.3
U+9E24.0	fTotalStrokes	 8
U+9E24.0	fCangjie	SPYM
U+9E24.0	fCantonese	si1
U+9E24.0	fMandarin	shi1
U+9E24.0	fEnglish	turtledove, cuckoo
U+9E25.0	fUTF8	鸥
U+9E25.0	fGB	0-4538
U+9E25.0	fR/S	 196.4
U+9E25.0	fTotalStrokes	 9
U+9E25.0	fCangjie	SKPYM
U+9E25.0	fHakka	 eu1
U+9E25.0	fCantonese	au1; ngau1 (also)
U+9E25.0	fMandarin	ou1; you1
U+9E25.0	fEnglish	simplified form of &#x9DD7;
U+9E26.0	fUTF8	鸦
U+9E26.0	fGB	0-513B
U+9E26.0	fR/S	 196.4
U+9E26.0	fTotalStrokes	 9
U+9E26.0	fCangjie	MHPYM
U+9E26.0	fHakka	 a1
U+9E26.0	fCantonese	aa1; ngaa1 (also)
U+9E26.0	fMandarin	ya1; ya4
U+9E26.0	fEnglish	simplified form of &#x9D09;
U+9E27.0	fUTF8	鸧
U+9E27.0	fGB	8-7D5C
U+9E27.0	fR/S	 196.4
U+9E27.0	fTotalStrokes	 9
U+9E27.0	fCangjie	OUPYM
U+9E27.0	fCantonese	cong1; coeng1
U+9E27.0	fMandarin	cang1; qiang1
U+9E27.0	fEnglish	oriole
U+9E28.0	fUTF8	鸨
U+9E28.0	fGB	0-7031
U+9E28.0	fR/S	 196.4
U+9E28.0	fTotalStrokes	 9
U+9E28.0	fCangjie	PJPYM
U+9E28.0	fHakka	 bau3; bo3
U+9E28.0	fCantonese	bou2
U+9E28.0	fMandarin	bao3
U+9E28.0	fEnglish	simplified form of &#x9D07;
U+9E29.0	fUTF8	鸩
U+9E29.0	fGB	0-7032
U+9E29.0	fR/S	 196.4
U+9E29.0	fTotalStrokes	 9
U+9E29.0	fCangjie	LUPYM
U+9E29.0	fHakka	 dam1; chin3
U+9E29.0	fCantonese	zam6
U+9E29.0	fMandarin	zhen4; chen2; tan2
U+9E29.0	fEnglish	simplified form of &#x9D06;
U+9E2A.0	fUTF8	鸪
U+9E2A.0	fGB	0-7033
U+9E2A.0	fR/S	 196.5
U+9E2A.0	fTotalStrokes	 10
U+9E2A.0	fCangjie	JRPYM
U+9E2A.0	fHakka	 gu1
U+9E2A.0	fCantonese	gu1; gwu1 (also)
U+9E2A.0	fMandarin	gu1
U+9E2A.0	fEnglish	simplified form of &#x9D23;
U+9E2B.0	fUTF8	鸫
U+9E2B.0	fGB	0-7034
U+9E2B.0	fR/S	 196.5
U+9E2B.0	fTotalStrokes	 10
U+9E2B.0	fCangjie	KDPYM
U+9E2B.0	fCantonese	dung1
U+9E2B.0	fMandarin	dong1
U+9E2B.0	fEnglish	simplified form of &#x9D87;
U+9E2C.0	fUTF8	鸬
U+9E2C.0	fGB	0-7035
U+9E2C.0	fR/S	 196.5
U+9E2C.0	fTotalStrokes	 10
U+9E2C.0	fCangjie	YSPYM
U+9E2C.0	fHakka	 lu2
U+9E2C.0	fCantonese	lou4
U+9E2C.0	fMandarin	lu2
U+9E2C.0	fEnglish	simplified form of &#x9E15;
U+9E2D.0	fUTF8	鸭
U+9E2D.0	fGB	0-513C
U+9E2D.0	fR/S	 196.5
U+9E2D.0	fTotalStrokes	 10
U+9E2D.0	fCangjie	WLPYM
U+9E2D.0	fHakka	 ap7
U+9E2D.0	fCantonese	aap3; ngaap3 (also); aap2 (also); ngaap2 (also)
U+9E2D.0	fMandarin	ya1
U+9E2D.0	fEnglish	simplified form of &#x9D28;
U+9E2E.0	fUTF8	鸮
U+9E2E.0	fGB	8-7D5D
U+9E2E.0	fR/S	 196.5
U+9E2E.0	fTotalStrokes	 10
U+9E2E.0	fCangjie	XRSPY
U+9E2E.0	fCantonese	hiu1
U+9E2E.0	fMandarin	xiao1
U+9E2E.0	fEnglish	owl
U+9E2F.0	fUTF8	鸯
U+9E2F.0	fGB	0-516C
U+9E2F.0	fR/S	 196.5
U+9E2F.0	fTotalStrokes	 10
U+9E2F.0	fCangjie	LKPYM
U+9E2F.0	fHakka	 jong1
U+9E2F.0	fCantonese	joeng1
U+9E2F.0	fMandarin	yang1
U+9E2F.0	fEnglish	simplified form of &#x9D26;
U+9E30.0	fUTF8	鸰
U+9E30.0	fGB	8-7D5E
U+9E30.0	fR/S	 196.5
U+9E30.0	fTotalStrokes	 10
U+9E30.0	fCangjie	OIPYM
U+9E30.0	fCantonese	ling4
U+9E30.0	fMandarin	ling2
U+9E30.0	fEnglish	species of lark; wagtail; Motacilla species (various)
U+9E31.0	fUTF8	鸱
U+9E31.0	fGB	0-7037
U+9E31.0	fR/S	 196.5
U+9E31.0	fTotalStrokes	 10
U+9E31.0	fCangjie	XHIPY
U+9E31.0	fHakka	 chi1
U+9E31.0	fCantonese	ci1
U+9E31.0	fMandarin	zhi1; chi1; hun1
U+9E31.0	fEnglish	simplified form of &#x9D1F;
U+9E32.0	fUTF8	鸲
U+9E32.0	fGB	0-7036
U+9E32.0	fR/S	 196.5
U+9E32.0	fTotalStrokes	 10
U+9E32.0	fCangjie	PRPYM
U+9E32.0	fHakka	 ki1
U+9E32.0	fCantonese	keoi4
U+9E32.0	fMandarin	qu2; gou1
U+9E32.0	fEnglish	simplified form of &#x9D1D;
U+9E33.0	fUTF8	鸳
U+9E33.0	fGB	0-5427
U+9E33.0	fR/S	 196.5
U+9E33.0	fTotalStrokes	 10
U+9E33.0	fCangjie	NUPYM
U+9E33.0	fHakka	 jen1
U+9E33.0	fCantonese	jyun1; jin1 (also)
U+9E33.0	fMandarin	yuan1; wen1
U+9E33.0	fEnglish	simplified form of &#x9D1B;
U+9E34.0	fUTF8	鸴
U+9E34.0	fGB	8-7D5F
U+9E34.0	fR/S	 196.13; 196.5
U+9E34.0	fTotalStrokes	 10
U+9E34.0	fCangjie	XFBPY
U+9E34.0	fCantonese	hok6; ang1; ngang1 (also)
U+9E34.0	fMandarin	xue2
U+9E34.0	fEnglish	oriental bullfinch, weaver bird; Pyrrhula species (various)
U+9E35.0	fUTF8	鸵
U+9E35.0	fGB	0-4D52
U+9E35.0	fR/S	 196.5
U+9E35.0	fTotalStrokes	 10
U+9E35.0	fCangjie	PMJP
U+9E35.0	fHakka	 to2
U+9E35.0	fCantonese	to4
U+9E35.0	fMandarin	tuo2
U+9E35.0	fEnglish	simplified form of &#x9D15;
U+9E36.0	fUTF8	鸶
U+9E36.0	fGB	0-7038
U+9E36.0	fR/S	 196.5
U+9E36.0	fTotalStrokes	 10
U+9E36.0	fCangjie	VVMPM
U+9E36.0	fHakka	 sii1
U+9E36.0	fCantonese	si1
U+9E36.0	fMandarin	si1
U+9E36.0	fEnglish	simplified form of &#x9DE5;
U+9E37.0	fUTF8	鸷
U+9E37.0	fGB	0-703A
U+9E37.0	fR/S	 196.6
U+9E37.0	fTotalStrokes	 11
U+9E37.0	fCangjie	QIPYM
U+9E37.0	fHakka	 zhi5
U+9E37.0	fCantonese	zi3
U+9E37.0	fMandarin	zhi4
U+9E37.0	fEnglish	simplified form of &#x9DD9;
U+9E38.0	fUTF8	鸸
U+9E38.0	fGB	0-7039
U+9E38.0	fR/S	 196.6
U+9E38.0	fTotalStrokes	 11
U+9E38.0	fCangjie	XXMBP
U+9E38.0	fCantonese	ji4
U+9E38.0	fMandarin	er2
U+9E38.0	fEnglish	simplified form of &#x9D2F;
U+9E39.0	fUTF8	鸹
U+9E39.0	fGB	0-703B
U+9E39.0	fR/S	 196.6
U+9E39.0	fTotalStrokes	 11
U+9E39.0	fCangjie	XHRPY
U+9E39.0	fHakka	 kwat7
U+9E39.0	fCantonese	kut3; kwut3 (also); gwaat3 (also)
U+9E39.0	fMandarin	gua1
U+9E39.0	fEnglish	simplified form of &#x9D30;
U+9E3A.0	fUTF8	鸺
U+9E3A.0	fGB	0-703C
U+9E3A.0	fR/S	 196.6
U+9E3A.0	fTotalStrokes	 11
U+9E3A.0	fCangjie	ODPM
U+9E3A.0	fHakka	 hiu1
U+9E3A.0	fCantonese	jau1
U+9E3A.0	fMandarin	xiu1
U+9E3A.0	fEnglish	simplified form of &#x9D42;
U+9E3B.0	fUTF8	鸻
U+9E3B.0	fGB	7-2128
U+9E3B.0	fR/S	 196.6
U+9E3B.0	fTotalStrokes	 11
U+9E3B.0	fCangjie	HOMNM
U+9E3B.0	fCantonese	hang4; hong4 (also)
U+9E3B.0	fMandarin	heng2; hang2
U+9E3C.0	fUTF8	鸼
U+9E3C.0	fGB	7-2129
U+9E3C.0	fR/S	 196.6
U+9E3C.0	fTotalStrokes	 11
U+9E3C.0	fCangjie	HYPYM
U+9E3C.0	fCantonese	zau1
U+9E3C.0	fMandarin	zhou1; zhao1
U+9E3D.0	fUTF8	鸽
U+9E3D.0	fGB	0-386B
U+9E3D.0	fR/S	 196.6
U+9E3D.0	fTotalStrokes	 11
U+9E3D.0	fCangjie	ORPYM
U+9E3D.0	fHakka	 gap7
U+9E3D.0	fCantonese	gap3; gap2 (also); gaap3 (also)
U+9E3D.0	fMandarin	ge1
U+9E3D.0	fEnglish	simplified form of &#x9D3F;
U+9E3E.0	fUTF8	鸾
U+9E3E.0	fGB	0-703D
U+9E3E.0	fR/S	 196.6
U+9E3E.0	fTotalStrokes	 11
U+9E3E.0	fCangjie	YCPYM
U+9E3E.0	fHakka	 lan2
U+9E3E.0	fCantonese	lyun4
U+9E3E.0	fMandarin	luan2
U+9E3E.0	fEnglish	simplified form of &#x9E1E;
U+9E3F.0	fUTF8	鸿
U+9E3F.0	fGB	0-3A68
U+9E3F.0	fR/S	 196.6
U+9E3F.0	fTotalStrokes	 11
U+9E3F.0	fCangjie	EMPM
U+9E3F.0	fHakka	 fung2; pung2
U+9E3F.0	fCantonese	hung4
U+9E3F.0	fMandarin	hong2; gong3; hong3
U+9E3F.0	fEnglish	simplified form of &#x9D3B;
U+9E40.0	fUTF8	鹀
U+9E40.0	fGB	8-2C64
U+9E40.0	fR/S	 196.7
U+9E40.0	fTotalStrokes	 12
U+9E40.0	fCangjie	MOPYM
U+9E40.0	fCantonese	mou4
U+9E40.0	fMandarin	wu2
U+9E41.0	fUTF8	鹁
U+9E41.0	fGB	0-703E
U+9E41.0	fR/S	 196.7
U+9E41.0	fTotalStrokes	 12
U+9E41.0	fCangjie	JDPYM
U+9E41.0	fHakka	 put8
U+9E41.0	fCantonese	but6
U+9E41.0	fMandarin	bo2
U+9E41.0	fEnglish	simplified form of &#x9D53;
U+9E42.0	fUTF8	鹂
U+9E42.0	fGB	0-703F
U+9E42.0	fR/S	 196.7
U+9E42.0	fTotalStrokes	 12
U+9E42.0	fCangjie	MBPYM
U+9E42.0	fHakka	 li2
U+9E42.0	fCantonese	lei4
U+9E42.0	fMandarin	li2
U+9E42.0	fEnglish	simplified form of &#x9E1D;
U+9E43.0	fUTF8	鹃
U+9E43.0	fGB	0-3E69
U+9E43.0	fR/S	 196.7
U+9E43.0	fTotalStrokes	 12
U+9E43.0	fCangjie	RBPYM
U+9E43.0	fHakka	 gen1
U+9E43.0	fCantonese	gyun1
U+9E43.0	fMandarin	juan1
U+9E43.0	fEnglish	simplified form of &#x9D51;
U+9E44.0	fUTF8	鹄
U+9E44.0	fGB	0-7040
U+9E44.0	fR/S	 196.7
U+9E44.0	fTotalStrokes	 12
U+9E44.0	fCangjie	HRPYM
U+9E44.0	fHakka	 kwuk7
U+9E44.0	fCantonese	guk1; huk6; huk1
U+9E44.0	fMandarin	hu2; ge3; gu3
U+9E44.0	fEnglish	simplified form of &#x9D60;
U+9E45.0	fUTF8	鹅
U+9E45.0	fGB	0-366C
U+9E45.0	fR/S	 196.7
U+9E45.0	fTotalStrokes	 12
U+9E45.0	fCangjie	HIPYM
U+9E45.0	fHakka	 ngo2
U+9E45.0	fCantonese	ngo4; ngo2 (also); o4
U+9E45.0	fMandarin	e2
U+9E45.0	fEnglish	simplified form of &#x9D5D;
U+9E46.0	fUTF8	鹆
U+9E46.0	fGB	0-7041
U+9E46.0	fR/S	 196.7
U+9E46.0	fTotalStrokes	 12
U+9E46.0	fCangjie	CRPYM
U+9E46.0	fHakka	 juk8
U+9E46.0	fCantonese	juk6
U+9E46.0	fMandarin	yu4; ji4
U+9E46.0	fEnglish	simplified form of &#x9D52;
U+9E47.0	fUTF8	鹇
U+9E47.0	fGB	0-7042
U+9E47.0	fR/S	 196.7
U+9E47.0	fTotalStrokes	 12
U+9E47.0	fCangjie	LDPYM
U+9E47.0	fCantonese	haan4
U+9E47.0	fMandarin	xian2
U+9E47.0	fEnglish	simplified form of &#x9DF3;
U+9E48.0	fUTF8	鹈
U+9E48.0	fGB	0-7043
U+9E48.0	fR/S	 196.7
U+9E48.0	fTotalStrokes	 12
U+9E48.0	fCangjie	CHPYM
U+9E48.0	fHakka	 ti2
U+9E48.0	fCantonese	tai4
U+9E48.0	fMandarin	ti2; ti4
U+9E48.0	fEnglish	simplified form of &#x9D5C;
U+9E49.0	fUTF8	鹉
U+9E49.0	fGB	0-7044
U+9E49.0	fR/S	 196.8
U+9E49.0	fTotalStrokes	 13
U+9E49.0	fCangjie	MMPYM
U+9E49.0	fHakka	 vu3
U+9E49.0	fCantonese	mou5
U+9E49.0	fMandarin	wu3
U+9E49.0	fEnglish	simplified form of &#x9D61;
U+9E4A.0	fUTF8	鹊
U+9E4A.0	fGB	0-4835
U+9E4A.0	fR/S	 196.8
U+9E4A.0	fTotalStrokes	 13
U+9E4A.0	fCangjie	TAPYM
U+9E4A.0	fHakka	 siak7
U+9E4A.0	fCantonese	zoek3; coek3 (also); zoek2 (also)
U+9E4A.0	fMandarin	que4; qiao3; que2
U+9E4A.0	fEnglish	simplified form of &#x9D72;
U+9E4B.0	fUTF8	鹋
U+9E4B.0	fGB	0-7045
U+9E4B.0	fR/S	 196.9; 196.8
U+9E4B.0	fTotalStrokes	 13
U+9E4B.0	fCangjie	TWPYM
U+9E4B.0	fCantonese	miu4
U+9E4B.0	fMandarin	miao2
U+9E4B.0	fEnglish	simplified form of &#x9D93;
U+9E4C.0	fUTF8	鹌
U+9E4C.0	fGB	0-7046
U+9E4C.0	fR/S	 196.8
U+9E4C.0	fTotalStrokes	 13
U+9E4C.0	fCangjie	KUPYM
U+9E4C.0	fHakka	 am1; em1
U+9E4C.0	fCantonese	am1; ngam1 (also); aan3; ngaan3 (also)
U+9E4C.0	fMandarin	an1; ya1; yan4
U+9E4C.0	fEnglish	simplified form of &#x9D6A;
U+9E4D.0	fUTF8	鹍
U+9E4D.0	fGB	8-7D60
U+9E4D.0	fR/S	 196.8
U+9E4D.0	fTotalStrokes	 13
U+9E4D.0	fCangjie	APPYM
U+9E4D.0	fCantonese	kwan1
U+9E4D.0	fMandarin	kun1
U+9E4D.0	fEnglish	a bird resembling the crane
U+9E4E.0	fUTF8	鹎
U+9E4E.0	fGB	0-7047
U+9E4E.0	fR/S	 196.8
U+9E4E.0	fTotalStrokes	 13
U+9E4E.0	fCangjie	HJPYM
U+9E4E.0	fHakka	 pit7
U+9E4E.0	fCantonese	bei1; pat1
U+9E4E.0	fMandarin	bei1; pi4
U+9E4E.0	fEnglish	simplified form of &#x9D6F;
U+9E4F.0	fUTF8	鹏
U+9E4F.0	fGB	0-4574
U+9E4F.0	fR/S	 196.8
U+9E4F.0	fTotalStrokes	 13
U+9E4F.0	fCangjie	BBPM
U+9E4F.0	fHakka	 pen2
U+9E4F.0	fCantonese	paang4
U+9E4F.0	fMandarin	peng2; beng1; feng4
U+9E4F.0	fEnglish	simplified form of &#x9D6C;
U+9E50.0	fUTF8	鹐
U+9E50.0	fGB	8-2C65
U+9E50.0	fR/S	 196.8
U+9E50.0	fTotalStrokes	 13
U+9E50.0	fCangjie	NXPYM
U+9E50.0	fCantonese	zaam1
U+9E50.0	fMandarin	qian1
U+9E50.0	fEnglish	to peck, as birds
U+9E51.0	fUTF8	鹑
U+9E51.0	fGB	0-7048
U+9E51.0	fR/S	 196.8
U+9E51.0	fTotalStrokes	 13
U+9E51.0	fCangjie	YDPYM
U+9E51.0	fHakka	 tun2; shun2
U+9E51.0	fCantonese	seon4; ceon1 (also)
U+9E51.0	fMandarin	chun2; dun1; tuan2
U+9E51.0	fEnglish	simplified form of &#x9D89;
U+9E52.0	fUTF8	鹒
U+9E52.0	fGB	8-7D61
U+9E52.0	fR/S	 196.8
U+9E52.0	fTotalStrokes	 13
U+9E52.0	fCangjie	IOPYM
U+9E52.0	fCantonese	gang1
U+9E52.0	fMandarin	geng1
U+9E52.0	fEnglish	oriole
U+9E53.0	fUTF8	鹓
U+9E53.0	fGB	8-7D62
U+9E53.0	fR/S	 196.8
U+9E53.0	fTotalStrokes	 13
U+9E53.0	fCangjie	JUPYM
U+9E53.0	fCantonese	jyun1
U+9E53.0	fMandarin	yuan1
U+9E53.0	fEnglish	co-worker
U+9E54.0	fUTF8	鹔
U+9E54.0	fGB	8-7D63
U+9E54.0	fR/S	 196.12; 196.8
U+9E54.0	fTotalStrokes	 13
U+9E54.0	fCangjie	LXPYM
U+9E54.0	fCantonese	suk1
U+9E54.0	fMandarin	su4
U+9E54.0	fEnglish	turquoise kingfisher
U+9E55.0	fUTF8	鹕
U+9E55.0	fGB	0-7049
U+9E55.0	fR/S	 196.9
U+9E55.0	fTotalStrokes	 14
U+9E55.0	fCangjie	JRBPM
U+9E55.0	fHakka	 fu2
U+9E55.0	fCantonese	wu4
U+9E55.0	fMandarin	hu2
U+9E55.0	fEnglish	simplified form of &#x9D98;
U+9E56.0	fUTF8	鹖
U+9E56.0	fGB	8-7D64
U+9E56.0	fR/S	 196.9
U+9E56.0	fTotalStrokes	 14
U+9E56.0	fCangjie	AVPYM
U+9E56.0	fCantonese	hot3
U+9E56.0	fMandarin	he2; jie4
U+9E56.0	fEnglish	crossbill, species of nightingale
U+9E57.0	fUTF8	鹗
U+9E57.0	fGB	0-704A
U+9E57.0	fR/S	 196.9
U+9E57.0	fTotalStrokes	 14
U+9E57.0	fCangjie	RSPYM
U+9E57.0	fHakka	 ngok8
U+9E57.0	fCantonese	ngok6; ok6 (also)
U+9E57.0	fMandarin	e4
U+9E57.0	fEnglish	simplified form of &#x9D9A;
U+9E58.0	fUTF8	鹘
U+9E58.0	fGB	0-773D
U+9E58.0	fR/S	 196.10
U+9E58.0	fTotalStrokes	 15
U+9E58.0	fCangjie	BBPYM
U+9E58.0	fHakka	 gwut7
U+9E58.0	fCantonese	gwat1; wat6
U+9E58.0	fMandarin	gu2; gu3; hu2; hua2
U+9E58.0	fEnglish	simplified form of &#x9DBB;
U+9E59.0	fUTF8	鹙
U+9E59.0	fGB	8-7D65
U+9E59.0	fR/S	 196.9
U+9E59.0	fTotalStrokes	 14
U+9E59.0	fCangjie	HFPYM
U+9E59.0	fCantonese	cau1
U+9E59.0	fMandarin	qiu1
U+9E59.0	fEnglish	large waterfowl with naked head; Garrulus glandarius
U+9E5A.0	fUTF8	鹚
U+9E5A.0	fGB	0-704B
U+9E5A.0	fR/S	 196.9
U+9E5A.0	fTotalStrokes	 14
U+9E5A.0	fCangjie	TIPYM
U+9E5A.0	fHakka	 cih2
U+9E5A.0	fCantonese	ci4
U+9E5A.0	fMandarin	ci2; zi1
U+9E5A.0	fEnglish	simplified form of &#x9DC0;
U+9E5B.0	fUTF8	鹛
U+9E5B.0	fGB	0-704C
U+9E5B.0	fR/S	 196.9
U+9E5B.0	fTotalStrokes	 14
U+9E5B.0	fCangjie	AUPYM
U+9E5B.0	fCantonese	mei4
U+9E5B.0	fMandarin	mei2
U+9E5B.0	fEnglish	simplified form of &#x9DA5;
U+9E5C.0	fUTF8	鹜
U+9E5C.0	fGB	0-704D
U+9E5C.0	fR/S	 196.9
U+9E5C.0	fTotalStrokes	 14
U+9E5C.0	fCangjie	NKPYM
U+9E5C.0	fHakka	 muk8; vu5
U+9E5C.0	fCantonese	mou6
U+9E5C.0	fMandarin	mu4; wu4
U+9E5C.0	fEnglish	simplified form of &#x9DA9;
U+9E5D.0	fUTF8	鹝
U+9E5D.0	fGB	8-2C66
U+9E5D.0	fR/S	 196.10
U+9E5D.0	fTotalStrokes	 15
U+9E5D.0	fCangjie	XMBPY
U+9E5D.0	fCantonese	jik6
U+9E5D.0	fMandarin	ni4; yi4
U+9E5D.0	fEnglish	pheasant
U+9E5E.0	fUTF8	鹞
U+9E5E.0	fGB	0-704E
U+9E5E.0	fR/S	 196.10
U+9E5E.0	fTotalStrokes	 15
U+9E5E.0	fCangjie	BUPYM
U+9E5E.0	fHakka	 jau5
U+9E5E.0	fCantonese	jiu6; jiu2 (also); jiu4
U+9E5E.0	fMandarin	yao4; yao2
U+9E5E.0	fEnglish	simplified form of &#x9DC2;
U+9E5F.0	fUTF8	鹟
U+9E5F.0	fGB	7-212B
U+9E5F.0	fR/S	 196.10
U+9E5F.0	fTotalStrokes	 15
U+9E5F.0	fCangjie	CMPYM
U+9E5F.0	fCantonese	jung1
U+9E5F.0	fMandarin	weng1
U+9E60.0	fUTF8	鹠
U+9E60.0	fGB	8-2C67
U+9E60.0	fR/S	 196.10
U+9E60.0	fTotalStrokes	 15
U+9E60.0	fCangjie	HWPYM
U+9E60.0	fCantonese	lau4
U+9E60.0	fMandarin	liu2
U+9E60.0	fEnglish	the owl
U+9E61.0	fUTF8	鹡
U+9E61.0	fGB	8-7D66
U+9E61.0	fR/S	 196.10
U+9E61.0	fTotalStrokes	 15
U+9E61.0	fCangjie	FBPYM
U+9E61.0	fCantonese	zik3; zek3 (also)
U+9E61.0	fMandarin	ji2
U+9E61.0	fEnglish	wagtail
U+9E62.0	fUTF8	鹢
U+9E62.0	fGB	8-7D67
U+9E62.0	fR/S	 196.10
U+9E62.0	fTotalStrokes	 15
U+9E62.0	fCangjie	TTPYM
U+9E62.0	fCantonese	jik6
U+9E62.0	fMandarin	ni4; yi4
U+9E62.0	fEnglish	fishhawk bow or prow
U+9E63.0	fUTF8	鹣
U+9E63.0	fGB	0-704F
U+9E63.0	fR/S	 196.10
U+9E63.0	fTotalStrokes	 15
U+9E63.0	fCangjie	TCPYM
U+9E63.0	fHakka	 giam1
U+9E63.0	fCantonese	gim1
U+9E63.0	fMandarin	jian1
U+9E63.0	fEnglish	simplified form of &#x9DBC;
U+9E64.0	fUTF8	鹤
U+9E64.0	fGB	0-3A57
U+9E64.0	fR/S	 196.10
U+9E64.0	fTotalStrokes	 15
U+9E64.0	fCangjie	OGPYM
U+9E64.0	fHakka	 hok8
U+9E64.0	fCantonese	hok6; hok2 (also)
U+9E64.0	fMandarin	he4; hao2
U+9E64.0	fEnglish	simplified form of &#x9DB4;
U+9E65.0	fUTF8	鹥
U+9E65.0	fGB	8-7D68
U+9E65.0	fR/S	 196.11
U+9E65.0	fTotalStrokes	 16
U+9E65.0	fCangjie	SEPYM
U+9E65.0	fCantonese	ji1
U+9E65.0	fMandarin	yi1
U+9E65.0	fEnglish	seagull
U+9E66.0	fUTF8	鹦
U+9E66.0	fGB	0-7050
U+9E66.0	fR/S	 196.11
U+9E66.0	fTotalStrokes	 16
U+9E66.0	fCangjie	BVPYM
U+9E66.0	fHakka	 {en1}; jin1
U+9E66.0	fCantonese	jing1
U+9E66.0	fMandarin	ying1
U+9E66.0	fEnglish	simplified form of &#x9E1A;
U+9E67.0	fUTF8	鹧
U+9E67.0	fGB	0-7051
U+9E67.0	fR/S	 196.11
U+9E67.0	fTotalStrokes	 16
U+9E67.0	fCangjie	IFPYM
U+9E67.0	fHakka	 zha5
U+9E67.0	fCantonese	ze3
U+9E67.0	fMandarin	zhe4
U+9E67.0	fEnglish	simplified form of &#x9DD3;
U+9E68.0	fUTF8	鹨
U+9E68.0	fGB	0-7052
U+9E68.0	fR/S	 196.11
U+9E68.0	fTotalStrokes	 16
U+9E68.0	fCangjie	SHPYM
U+9E68.0	fHakka	 liu5
U+9E68.0	fCantonese	lau6
U+9E68.0	fMandarin	liao2; liu4; jiu1; liu2; mou2
U+9E68.0	fEnglish	simplified form of &#x9DDA;
U+9E69.0	fUTF8	鹩
U+9E69.0	fGB	0-7053
U+9E69.0	fR/S	 196.12
U+9E69.0	fTotalStrokes	 17
U+9E69.0	fCangjie	KFPYM
U+9E69.0	fHakka	 liau2
U+9E69.0	fCantonese	liu4; liu1
U+9E69.0	fMandarin	liao2; liao4
U+9E69.0	fEnglish	simplified form of &#x9DEF;
U+9E6A.0	fUTF8	鹪
U+9E6A.0	fGB	0-7054
U+9E6A.0	fR/S	 196.12
U+9E6A.0	fTotalStrokes	 17
U+9E6A.0	fCangjie	OFPYM
U+9E6A.0	fHakka	 ziau1
U+9E6A.0	fCantonese	ziu1
U+9E6A.0	fMandarin	jiao1
U+9E6A.0	fEnglish	simplified form of &#x9DE6;
U+9E6B.0	fUTF8	鹫
U+9E6B.0	fGB	0-7055
U+9E6B.0	fR/S	 196.12
U+9E6B.0	fTotalStrokes	 17
U+9E6B.0	fCangjie	YUPYM
U+9E6B.0	fHakka	 ciu5
U+9E6B.0	fCantonese	zau6
U+9E6B.0	fMandarin	jiu4
U+9E6B.0	fEnglish	simplified form of &#x9DF2;
U+9E6C.0	fUTF8	鹬
U+9E6C.0	fGB	0-7056
U+9E6C.0	fR/S	 196.12
U+9E6C.0	fTotalStrokes	 17
U+9E6C.0	fCangjie	NBPYM
U+9E6C.0	fHakka	 jut8; lut8
U+9E6C.0	fCantonese	wat6; neot6 (also); leot6 (also)
U+9E6C.0	fMandarin	yu4; shu4
U+9E6C.0	fEnglish	simplified form of &#x9DF8;
U+9E6D.0	fUTF8	鹭
U+9E6D.0	fGB	0-7058
U+9E6D.0	fR/S	 196.13
U+9E6D.0	fTotalStrokes	 18
U+9E6D.0	fCangjie	RRPYM
U+9E6D.0	fHakka	 lu5
U+9E6D.0	fCantonese	lou6
U+9E6D.0	fMandarin	lu4
U+9E6D.0	fEnglish	simplified form of &#x9DFA;
U+9E6E.0	fUTF8	鹮
U+9E6E.0	fGB	7-212D
U+9E6E.0	fR/S	 196.13
U+9E6E.0	fTotalStrokes	 18
U+9E6E.0	fCangjie	WVPYM
U+9E6E.0	fCantonese	syun4; waan4 (also)
U+9E6E.0	fMandarin	xuan2; huan2
U+9E6F.0	fUTF8	鹯
U+9E6F.0	fGB	8-7D69
U+9E6F.0	fR/S	 196.13
U+9E6F.0	fTotalStrokes	 18
U+9E6F.0	fCangjie	YMPYM
U+9E6F.0	fCantonese	zin1
U+9E6F.0	fMandarin	zhan1
U+9E6F.0	fEnglish	hawk; Butastur indicus
U+9E70.0	fUTF8	鹰
U+9E70.0	fGB	0-5325
U+9E70.0	fR/S	 196.13
U+9E70.0	fTotalStrokes	 18
U+9E70.0	fCangjie	IOGM
U+9E70.0	fHakka	 jin1
U+9E70.0	fCantonese	jing1
U+9E70.0	fMandarin	ying1
U+9E70.0	fEnglish	simplified form of &#x9DF9;
U+9E71.0	fUTF8	鹱
U+9E71.0	fGB	0-7057
U+9E71.0	fR/S	 196.14
U+9E71.0	fTotalStrokes	 19
U+9E71.0	fCangjie	PMTOE
U+9E71.0	fHakka	 wok7; fu6; fu5; vok7
U+9E71.0	fCantonese	wok3; wok6 (also)
U+9E71.0	fMandarin	huo4; hu4
U+9E71.0	fEnglish	simplified form of &#x9E0C;
U+9E72.0	fUTF8	鹲
U+9E72.0	fGB	7-212C
U+9E72.0	fR/S	 196.14
U+9E72.0	fTotalStrokes	 19
U+9E72.0	fCangjie	TOPYM
U+9E72.0	fCantonese	mung4
U+9E72.0	fMandarin	meng2
U+9E73.0	fUTF8	鹳
U+9E73.0	fGB	0-7059
U+9E73.0	fR/S	 196.17
U+9E73.0	fTotalStrokes	 22
U+9E73.0	fCangjie	TGPYM
U+9E73.0	fHakka	 gwon5
U+9E73.0	fCantonese	gun3; gwun3 (also)
U+9E73.0	fMandarin	guan4; huan1
U+9E73.0	fEnglish	simplified form of &#x9E1B;
U+9E74.0	fUTF8	鹴
U+9E74.0	fGB	8-7D6A
U+9E74.0	fR/S	 196.17
U+9E74.0	fTotalStrokes	 22
U+9E74.0	fCangjie	MUPYM
U+9E74.0	fCantonese	soeng1
U+9E74.0	fMandarin	shuang1
U+9E74.0	fEnglish	eagle
U+9E75.0	fUTF8	鹵
U+9E75.0	fCNS11643	1-5E25
U+9E75.0	fBig5	B3BF
U+9E75.0	fGB	1-7852
U+9E75.0	fR/S	 197.0
U+9E75.0	fTotalStrokes	 11
U+9E75.0	fCangjie	YWII
U+9E75.0	fFourCorner	21600
U+9E75.0	fHakka	  lu1;  lu3
U+9E75.0	fCantonese	lou5
U+9E75.0	fMandarin	lu3; lu2
U+9E75.0	fEnglish	[1] alkaline soil; saline soil [2] natural salt [3] rude; unrefined [4] capture; seize [5] Kangxi radical 197
U+9E76.0	fUTF8	鹶
U+9E76.0	fCNS11643	4-4F48
U+9E76.0	fGB	3-6F45
U+9E76.0	fR/S	  197.4
U+9E76.0	fTotalStrokes	 15
U+9E76.0	fCangjie	YWOIN
U+9E76.0	fFourCorner	28620
U+9E76.0	fCantonese	ging1
U+9E76.0	fMandarin	jin1; jing1
U+9E76.0	fEnglish	 salty and bitter pitiful
U+9E77.0	fUTF8	鹷
U+9E77.0	fCNS11643	5-5A5B
U+9E77.0	fGB	E-4856
U+9E77.0	fR/S	  197.5
U+9E77.0	fTotalStrokes	 16
U+9E77.0	fCangjie	YWOII
U+9E77.0	fFourCorner	28630
U+9E77.0	fCantonese	ling4
U+9E77.0	fMandarin	ling2
U+9E78.0	fUTF8	鹸
U+9E78.0	fGB	E-4857
U+9E78.0	fR/S	 197.8
U+9E78.0	fTotalStrokes	 19
U+9E78.0	fCangjie	YWOMO
U+9E78.0	fCantonese	gaan2
U+9E78.0	fMandarin	jian3
U+9E78.0	fEnglish	alkaline; alkali; lye; salt
U+9E79.0	fUTF8	鹹
U+9E79.0	fCNS11643	1-7A5B
U+9E79.0	fBig5	C4D0
U+9E79.0	fGB	1-7874
U+9E79.0	fR/S	 197.9
U+9E79.0	fTotalStrokes	 20
U+9E79.0	fCangjie	YWIHR
U+9E79.0	fFourCorner	23650
U+9E79.0	fHakka	  ham2; ham3
U+9E79.0	fCantonese	haam4; haam2 (also)
U+9E79.0	fMandarin	xian2; jian3
U+9E79.0	fEnglish	salty; saltish; briny; salted; pickled
U+9E7A.0	fUTF8	鹺
U+9E7A.0	fCNS11643	2-6D37
U+9E7A.0	fBig5	F6C9
U+9E7A.0	fGB	1-753A
U+9E7A.0	fR/S	 197.10
U+9E7A.0	fTotalStrokes	 21
U+9E7A.0	fCangjie	YWTQM
U+9E7A.0	fFourCorner	28610
U+9E7A.0	fHakka	  co1; ca1
U+9E7A.0	fCantonese	co4; co1
U+9E7A.0	fMandarin	cuo2
U+9E7A.0	fEnglish	[1] briny; salty [2] salt
U+9E7B.0	fUTF8	鹻
U+9E7B.0	fCNS11643	3-5F37
U+9E7B.0	fGB	E-4858
U+9E7B.0	fR/S	 197.10
U+9E7B.0	fTotalStrokes	 21
U+9E7B.0	fCangjie	YWTXC
U+9E7B.0	fFourCorner	28630
U+9E7B.0	fHakka	  giam5;  gam3; gan3; giam3
U+9E7B.0	fCantonese	gaan2
U+9E7B.0	fMandarin	jian3
U+9E7B.0	fEnglish	alkali
U+9E7C.0	fUTF8	鹼
U+9E7C.0	fCNS11643	1-7C7B
U+9E7C.0	fBig5	C650
U+9E7C.0	fGB	1-3C6F
U+9E7C.0	fR/S	 197.13
U+9E7C.0	fTotalStrokes	 24
U+9E7C.0	fCangjie	YWOMO
U+9E7C.0	fFourCorner	28686
U+9E7C.0	fHakka	  gan3;  giam3; gien3; gam3; gian3; giam5
U+9E7C.0	fCantonese	gaan2
U+9E7C.0	fMandarin	jian3; qian1; yan4; zuan1
U+9E7C.0	fEnglish	alkaline; alkali; lye
U+9E7D.0	fUTF8	鹽
U+9E7D.0	fCNS11643	1-7C7C
U+9E7D.0	fBig5	C651
U+9E7D.0	fGB	1-514E
U+9E7D.0	fR/S	 197.13
U+9E7D.0	fTotalStrokes	 24
U+9E7D.0	fCangjie	SWBT
U+9E7D.0	fFourCorner	78107
U+9E7D.0	fHakka	  jam2; ram2
U+9E7D.0	fCantonese	jim4; jim1 (also); jim6
U+9E7D.0	fMandarin	yan2
U+9E7D.0	fEnglish	 salt; common salt
U+9E7D.1	fUTF8	鹽
U+9E7D.1	fCNS11643	1-7C7C
U+9E7D.1	fBig5	C651
U+9E7D.1	fGB	1-514E
U+9E7D.1	fR/S	 197.13
U+9E7D.1	fTotalStrokes	 24
U+9E7D.1	fCangjie	SWBT
U+9E7D.1	fFourCorner	78107
U+9E7D.1	fMandarin	yan4
U+9E7D.1	fEnglish	[1] salt [2] envy
U+9E7E.0	fUTF8	鹾
U+9E7E.0	fGB	0-753A
U+9E7E.0	fR/S	 197.9
U+9E7E.0	fTotalStrokes	 17; 16
U+9E7E.0	fCangjie	YWTQM
U+9E7E.0	fHakka	 co1; ca1
U+9E7E.0	fCantonese	co4
U+9E7E.0	fMandarin	cuo2
U+9E7E.0	fEnglish	simplified form of &#x9E7A;
U+9E7F.0	fUTF8	鹿
U+9E7F.0	fCNS11643	1-5E26
U+9E7F.0	fBig5	B3C0
U+9E7F.0	fGB	0-4239
U+9E7F.0	fR/S	 198.0
U+9E7F.0	fTotalStrokes	 11
U+9E7F.0	fCangjie	IXP
U+9E7F.0	fFourCorner	00211
U+9E7F.0	fHakka	  luk8
U+9E7F.0	fCantonese	luk6; luk2 (also)
U+9E7F.0	fMandarin	lu4; l2
U+9E7F.0	fEnglish	[1] deer; stag; doe [2] Kangxi radical 198
U+9E80.0	fUTF8	麀
U+9E80.0	fCNS11643	2-4874
U+9E80.0	fBig5	E0EE
U+9E80.0	fGB	3-7C2A
U+9E80.0	fR/S	 198.2
U+9E80.0	fTotalStrokes	 13
U+9E80.0	fCangjie	IPP
U+9E80.0	fFourCorner	00211
U+9E80.0	fHakka	 ju2; riu2
U+9E80.0	fCantonese	jau1
U+9E80.0	fMandarin	you1
U+9E80.0	fEnglish	 female deer; doe
U+9E81.0	fUTF8	麁
U+9E81.0	fCNS11643	3-4569
U+9E81.0	fGB	5-342D
U+9E81.0	fR/S	 198.2
U+9E81.0	fTotalStrokes	 13
U+9E81.0	fCangjie	NIXP
U+9E81.0	fFourCorner	27210
U+9E81.0	fCantonese	cou1; zoeng1
U+9E81.0	fMandarin	cu1
U+9E81.0	fEnglish	 rough; thick; course rude
U+9E82.0	fUTF8	麂
U+9E82.0	fCNS11643	1-676E
U+9E82.0	fBig5	B9A8
U+9E82.0	fGB	0-7764
U+9E82.0	fR/S	 198.2
U+9E82.0	fTotalStrokes	 13
U+9E82.0	fCangjie	IPHN
U+9E82.0	fFourCorner	00211
U+9E82.0	fHakka	 gi3
U+9E82.0	fCantonese	gei2
U+9E82.0	fMandarin	ji3
U+9E82.0	fEnglish	a species of deer without antlers (Moschus chinensis)
U+9E83.0	fUTF8	麃
U+9E83.0	fCNS11643	2-563B
U+9E83.0	fBig5	E8F0
U+9E83.0	fGB	3-7C2E
U+9E83.0	fR/S	 198.4
U+9E83.0	fTotalStrokes	 15
U+9E83.0	fCangjie	IPF
U+9E83.0	fFourCorner	00231
U+9E83.0	fHakka	 piau1; peu1; biau1
U+9E83.0	fCantonese	biu1; paau4; piu2
U+9E83.0	fMandarin	biao1; piao3
U+9E83.0	fEnglish	[1] cultivate fields; till the land; plow; weed [2] valiant; vigorous
U+9E83.1	fUTF8	麃
U+9E83.1	fCNS11643	2-563B
U+9E83.1	fBig5	E8F0
U+9E83.1	fGB	3-7C2E
U+9E83.1	fR/S	 198.4
U+9E83.1	fTotalStrokes	 15
U+9E83.1	fCangjie	IPF
U+9E83.1	fFourCorner	00231
U+9E83.1	fMandarin	pao2
U+9E83.1	fEnglish	a species of roe
U+9E84.0	fUTF8	麄
U+9E84.0	fCNS11643	3-502C
U+9E84.0	fGB	5-797C
U+9E84.0	fR/S	 198.4
U+9E84.0	fTotalStrokes	 15
U+9E84.0	fCangjie	CSHP
U+9E84.0	fFourCorner	80210
U+9E84.0	fHakka	 cu1; cih1
U+9E84.0	fCantonese	cou1
U+9E84.0	fMandarin	cu1
U+9E84.0	fEnglish	coarse; rough; rude
U+9E85.0	fUTF8	麅
U+9E85.0	fCNS11643	3-5439
U+9E85.0	fGB	3-7C31
U+9E85.0	fR/S	 198.5
U+9E85.0	fTotalStrokes	 16
U+9E85.0	fCangjie	IPPRU
U+9E85.0	fFourCorner	00210
U+9E85.0	fHakka	  pau2
U+9E85.0	fCantonese	paau4; pau4
U+9E85.0	fMandarin	biao1; pao2
U+9E85.0	fEnglish	a species of roe
U+9E86.0	fUTF8	麆
U+9E86.0	fCNS11643	2-5C3C
U+9E86.0	fBig5	ECB0
U+9E86.0	fGB	3-7C2F
U+9E86.0	fR/S	 198.5
U+9E86.0	fTotalStrokes	 16
U+9E86.0	fCangjie	IPBM
U+9E86.0	fFourCorner	00217
U+9E86.0	fHakka	 cu1; cu6; cu5; cih1; ze1
U+9E86.0	fCantonese	cyu4; co1
U+9E86.0	fMandarin	zhu4; zu3; chu2; cu1
U+9E86.0	fEnglish	the faron of an antilope
U+9E87.0	fUTF8	麇
U+9E87.0	fCNS11643	2-5C3D
U+9E87.0	fBig5	ECB1
U+9E87.0	fGB	0-7765
U+9E87.0	fR/S	 198.5
U+9E87.0	fTotalStrokes	 16
U+9E87.0	fCangjie	IPHD
U+9E87.0	fFourCorner	00904
U+9E87.0	fHakka	  kiun2; kun1; giun1; kwun1
U+9E87.0	fCantonese	gwan1; kwan4; kwan2
U+9E87.0	fMandarin	jun1; qun2; kun3
U+9E87.0	fEnglish	 general name for the hornless deer collect to band together
U+9E88.0	fUTF8	麈
U+9E88.0	fCNS11643	2-5C3B
U+9E88.0	fBig5	ECAF
U+9E88.0	fGB	0-7766
U+9E88.0	fR/S	 198.5
U+9E88.0	fTotalStrokes	 16
U+9E88.0	fCangjie	IPYG
U+9E88.0	fFourCorner	00214
U+9E88.0	fHakka	 zhu3; zu3
U+9E88.0	fCantonese	zyu2
U+9E88.0	fMandarin	zhu3
U+9E88.0	fEnglish	[1] species of deer [2] whisk; dust
U+9E89.0	fUTF8	麉
U+9E89.0	fCNS11643	2-6137
U+9E89.0	fBig5	EFAB
U+9E89.0	fGB	E-4859
U+9E89.0	fR/S	 198.6
U+9E89.0	fTotalStrokes	 17
U+9E89.0	fCangjie	MJIXP
U+9E89.0	fFourCorner	11211
U+9E89.0	fCantonese	gin1
U+9E89.0	fMandarin	jian1; qian1
U+9E8A.0	fUTF8	麊
U+9E8A.0	fCNS11643	2-6136
U+9E8A.0	fBig5	EFAA
U+9E8A.0	fGB	5-6726
U+9E8A.0	fR/S	 198.6
U+9E8A.0	fTotalStrokes	 17
U+9E8A.0	fCangjie	FDIXP
U+9E8A.0	fFourCorner	90211
U+9E8A.0	fCantonese	mei4; mai4
U+9E8A.0	fMandarin	mi2; ming2
U+9E8B.0	fUTF8	麋
U+9E8B.0	fCNS11643	1-7633
U+9E8B.0	fBig5	C247
U+9E8B.0	fGB	0-7767
U+9E8B.0	fR/S	 198.6
U+9E8B.0	fTotalStrokes	 17
U+9E8B.0	fCangjie	IPFD
U+9E8B.0	fFourCorner	00601
U+9E8B.0	fHakka	  mi2
U+9E8B.0	fCantonese	mei4; mai4
U+9E8B.0	fMandarin	mi2
U+9E8B.0	fEnglish	 elk; a kind of deer (Alces machlis)
U+9E8C.0	fUTF8	麌
U+9E8C.0	fCNS11643	2-652B
U+9E8C.0	fBig5	F1DF
U+9E8C.0	fGB	3-7C33
U+9E8C.0	fR/S	 198.7
U+9E8C.0	fTotalStrokes	 18
U+9E8C.0	fCangjie	IPRVK
U+9E8C.0	fFourCorner	00430
U+9E8C.0	fHakka	 ngi3; ngi2
U+9E8C.0	fCantonese	jyu5
U+9E8C.0	fMandarin	wu2; yu2; yu3
U+9E8C.0	fEnglish	 stag herd
U+9E8D.0	fUTF8	麍
U+9E8D.0	fCNS11643	2-6138
U+9E8D.0	fBig5	EFAC
U+9E8D.0	fGB	E-485A
U+9E8D.0	fR/S	 198.7
U+9E8D.0	fTotalStrokes	 18
U+9E8D.0	fCangjie	IPYIU
U+9E8D.0	fFourCorner	00213
U+9E8D.0	fCantonese	lau4
U+9E8D.0	fMandarin	liu2
U+9E8E.0	fUTF8	麎
U+9E8E.0	fCNS11643	2-652A
U+9E8E.0	fBig5	F1DE
U+9E8E.0	fGB	E-485B
U+9E8E.0	fR/S	 198.7
U+9E8E.0	fTotalStrokes	 18
U+9E8E.0	fCangjie	IPMMV
U+9E8E.0	fFourCorner	00232
U+9E8E.0	fHakka	 shin2; siin2
U+9E8E.0	fCantonese	san4
U+9E8E.0	fMandarin	chen2; shen4; shi1
U+9E8F.0	fUTF8	麏
U+9E8F.0	fCNS11643	4-5F3A
U+9E8F.0	fGB	E-485C
U+9E8F.0	fR/S	 198.7
U+9E8F.0	fTotalStrokes	 18
U+9E8F.0	fCangjie	IPSKR
U+9E8F.0	fFourCorner	00260
U+9E8F.0	fCantonese	gwan1
U+9E8F.0	fMandarin	jun1; qun2
U+9E90.0	fUTF8	麐
U+9E90.0	fCNS11643	3-5A24
U+9E90.0	fGB	E-485D
U+9E90.0	fR/S	 198.7
U+9E90.0	fTotalStrokes	 18
U+9E90.0	fCangjie	IPYKR
U+9E90.0	fFourCorner	00260
U+9E90.0	fHakka	 lin2
U+9E90.0	fCantonese	leon4
U+9E90.0	fMandarin	lin2; lin4
U+9E90.0	fEnglish	the female of a fabulous animal resembling a deer
U+9E91.0	fUTF8	麑
U+9E91.0	fCNS11643	2-6860
U+9E91.0	fBig5	F3F3
U+9E91.0	fGB	3-7C35
U+9E91.0	fR/S	 198.8
U+9E91.0	fTotalStrokes	 19
U+9E91.0	fCangjie	IPHXU
U+9E91.0	fFourCorner	00217
U+9E91.0	fHakka	 ngi2; mi2
U+9E91.0	fCantonese	ngai4; ai4 (also)
U+9E91.0	fMandarin	ni2; mi2
U+9E91.0	fEnglish	 fawn; young deer
U+9E92.0	fUTF8	麒
U+9E92.0	fCNS11643	1-795D
U+9E92.0	fBig5	C451
U+9E92.0	fGB	0-7768
U+9E92.0	fR/S	 198.8
U+9E92.0	fTotalStrokes	 19
U+9E92.0	fCangjie	IPTMC
U+9E92.0	fFourCorner	04281
U+9E92.0	fHakka	  ki2; ki3
U+9E92.0	fCantonese	kei4
U+9E92.0	fMandarin	qi2
U+9E92.0	fEnglish	the male of a legendary auspicious animal resembling the deer
U+9E93.0	fUTF8	麓
U+9E93.0	fCNS11643	1-795F
U+9E93.0	fBig5	C453
U+9E93.0	fGB	0-4234
U+9E93.0	fR/S	 198.8
U+9E93.0	fTotalStrokes	 19
U+9E93.0	fCangjie	DDIXP
U+9E93.0	fFourCorner	44211
U+9E93.0	fHakka	 luk8
U+9E93.0	fCantonese	luk1; luk6
U+9E93.0	fMandarin	lu4
U+9E93.0	fEnglish	 foot of hill or mountain; foothill
U+9E94.0	fUTF8	麔
U+9E94.0	fCNS11643	2-685F
U+9E94.0	fBig5	F3F2
U+9E94.0	fGB	3-7C36
U+9E94.0	fR/S	 198.8
U+9E94.0	fTotalStrokes	 19
U+9E94.0	fCangjie	IPHOR
U+9E94.0	fFourCorner	00264
U+9E94.0	fHakka	 kiu3; kiu5
U+9E94.0	fCantonese	kau5; gau6
U+9E94.0	fMandarin	jiu4; gao1; jiu3; bao1
U+9E94.0	fEnglish	male of the elk
U+9E95.0	fUTF8	麕
U+9E95.0	fCNS11643	3-5C29
U+9E95.0	fGB	5-7A22
U+9E95.0	fR/S	 198.8
U+9E95.0	fTotalStrokes	 19
U+9E95.0	fCangjie	IPWHD
U+9E95.0	fFourCorner	00260
U+9E95.0	fHakka	 kiun3;  kiun2; kun1; kwun1
U+9E95.0	fCantonese	gwan1; kwan4; kwan2
U+9E95.0	fMandarin	jun1; qun2; kun3
U+9E95.0	fEnglish	muntjac deer;hornless river deer
U+9E96.0	fUTF8	麖
U+9E96.0	fCNS11643	4-6275
U+9E96.0	fGB	3-7C37
U+9E96.0	fR/S	 198.8
U+9E96.0	fTotalStrokes	 19
U+9E96.0	fCangjie	IPYRF
U+9E96.0	fFourCorner	00290
U+9E96.0	fHakka	 giang1;  gin1
U+9E96.0	fCantonese	ging1
U+9E96.0	fMandarin	jing1
U+9E96.0	fEnglish	a deer
U+9E97.0	fUTF8	麗
U+9E97.0	fCNS11643	1-795E
U+9E97.0	fBig5	C452
U+9E97.0	fGB	1-4076
U+9E97.0	fR/S	 198.8
U+9E97.0	fTotalStrokes	 19
U+9E97.0	fCangjie	MMBBP
U+9E97.0	fFourCorner	11211
U+9E97.0	fHakka	  li5; li6; li2; li3
U+9E97.0	fCantonese	lai6; lei4; lai4
U+9E97.0	fMandarin	li4; li2; li3; si1
U+9E97.0	fEnglish	[1] beautiful; magnificent; elegant; fine [2] hang [3] same as $
U+9E98.0	fUTF8	麘
U+9E98.0	fCNS11643	3-5E21
U+9E98.0	fGB	E-485E
U+9E98.0	fR/S	 198.9
U+9E98.0	fTotalStrokes	 20
U+9E98.0	fCangjie	IPHDA
U+9E98.0	fFourCorner	00260
U+9E98.0	fHakka	  hiong1
U+9E98.0	fCantonese	hoeng1
U+9E98.0	fMandarin	xiang1
U+9E98.0	fEnglish	scent
U+9E99.0	fUTF8	麙
U+9E99.0	fCNS11643	2-6B3F
U+9E99.0	fBig5	F5B1
U+9E99.0	fGB	3-7C38
U+9E99.0	fR/S	 198.9
U+9E99.0	fTotalStrokes	 20
U+9E99.0	fCangjie	IPIHR
U+9E99.0	fFourCorner	00253
U+9E99.0	fCantonese	ngaam4; haam4 (also); aam4 (also)
U+9E99.0	fMandarin	yan2; xian2; qian2
U+9E9A.0	fUTF8	麚
U+9E9A.0	fCNS11643	2-6B41
U+9E9A.0	fBig5	F5B3
U+9E9A.0	fGB	E-485F
U+9E9A.0	fR/S	 198.9
U+9E9A.0	fTotalStrokes	 20
U+9E9A.0	fCangjie	IPRSE
U+9E9A.0	fFourCorner	00747
U+9E9A.0	fHakka	 ga1
U+9E9A.0	fCantonese	gaa1
U+9E9A.0	fMandarin	jia1
U+9E9A.0	fEnglish	[1] buck [2] stag
U+9E9B.0	fUTF8	麛
U+9E9B.0	fCNS11643	2-6B40
U+9E9B.0	fBig5	F5B2
U+9E9B.0	fGB	3-7C39
U+9E9B.0	fR/S	 198.9
U+9E9B.0	fTotalStrokes	 20
U+9E9B.0	fCangjie	IPNSJ
U+9E9B.0	fFourCorner	00241
U+9E9B.0	fHakka	 mi2
U+9E9B.0	fCantonese	mai4; ngai4; nei4
U+9E9B.0	fMandarin	mi2
U+9E9B.0	fEnglish	fawn
U+9E9C.0	fUTF8	麜
U+9E9C.0	fCNS11643	2-6D38
U+9E9C.0	fBig5	F6CA
U+9E9C.0	fGB	3-7C3A
U+9E9C.0	fR/S	 198.10
U+9E9C.0	fTotalStrokes	 21
U+9E9C.0	fCangjie	IPMWD
U+9E9C.0	fFourCorner	00331
U+9E9C.0	fCantonese	leot6
U+9E9C.0	fMandarin	li4
U+9E9D.0	fUTF8	麝
U+9E9D.0	fCNS11643	1-7B51
U+9E9D.0	fBig5	C565
U+9E9D.0	fGB	0-776A
U+9E9D.0	fR/S	 198.10
U+9E9D.0	fTotalStrokes	 21
U+9E9D.0	fCangjie	IPHHI
U+9E9D.0	fFourCorner	00241
U+9E9D.0	fHakka	 sa5; sha6; sha5
U+9E9D.0	fCantonese	se6
U+9E9D.0	fMandarin	she4
U+9E9D.0	fEnglish	 musk deer
U+9E9E.0	fUTF8	麞
U+9E9E.0	fCNS11643	3-5F3A
U+9E9E.0	fGB	E-4860
U+9E9E.0	fR/S	 198.11
U+9E9E.0	fTotalStrokes	 22; 21
U+9E9E.0	fCangjie	IPYTJ
U+9E9E.0	fFourCorner	00240
U+9E9E.0	fHakka	 zhong1; zong1
U+9E9E.0	fCantonese	zoeng1
U+9E9E.0	fMandarin	zhang1
U+9E9E.0	fEnglish	Moschus chinloo, a deer-like animal
U+9E9F.0	fUTF8	麟
U+9E9F.0	fCNS11643	1-7C5B
U+9E9F.0	fBig5	C5EF
U+9E9F.0	fGB	0-776B
U+9E9F.0	fR/S	 198.12
U+9E9F.0	fTotalStrokes	 23
U+9E9F.0	fCangjie	IPFDQ
U+9E9F.0	fFourCorner	09259
U+9E9F.0	fHakka	  lin2; lin3
U+9E9F.0	fCantonese	leon4; leon2 (also)
U+9E9F.0	fMandarin	lin2
U+9E9F.0	fEnglish	the female of a fabulous animal resembling the deer
U+9EA0.0	fUTF8	麠
U+9EA0.0	fCNS11643	2-7076
U+9EA0.0	fBig5	F8E8
U+9EA0.0	fGB	E-4861
U+9EA0.0	fR/S	 198.13
U+9EA0.0	fTotalStrokes	 24
U+9EA0.0	fCangjie	IPMWM
U+9EA0.0	fFourCorner	00271
U+9EA0.0	fHakka	 giang1;  gin1
U+9EA0.0	fCantonese	ging1; geng1
U+9EA0.0	fMandarin	jing1
U+9EA0.0	fEnglish	a deer
U+9EA1.0	fUTF8	麡
U+9EA1.0	fCNS11643	2-7152
U+9EA1.0	fBig5	F963
U+9EA1.0	fGB	E-4862
U+9EA1.0	fR/S	 198.14
U+9EA1.0	fTotalStrokes	 25
U+9EA1.0	fCangjie	IPYX
U+9EA1.0	fFourCorner	00223
U+9EA1.0	fCantonese	cai4
U+9EA1.0	fMandarin	ji1; qi2
U+9EA2.0	fUTF8	麢
U+9EA2.0	fCNS11643	3-6231
U+9EA2.0	fGB	E-4863
U+9EA2.0	fR/S	 198.17
U+9EA2.0	fTotalStrokes	 28
U+9EA2.0	fCangjie	IPMBR
U+9EA2.0	fFourCorner	00260
U+9EA2.0	fHakka	 lin2
U+9EA2.0	fCantonese	ling4
U+9EA2.0	fMandarin	ling2
U+9EA3.0	fUTF8	麣
U+9EA3.0	fCNS11643	3-623F
U+9EA3.0	fGB	E-4864
U+9EA3.0	fR/S	 198.20
U+9EA3.0	fTotalStrokes	 31
U+9EA3.0	fCangjie	IPRRK
U+9EA3.0	fFourCorner	06240
U+9EA3.0	fCantonese	ngaam4; aam4 (also)
U+9EA3.0	fMandarin	yan2
U+9EA4.0	fUTF8	麤
U+9EA4.0	fCNS11643	2-7241
U+9EA4.0	fBig5	F9D2
U+9EA4.0	fGB	E-4865
U+9EA4.0	fR/S	 198.22
U+9EA4.0	fTotalStrokes	 33
U+9EA4.0	fCangjie	IPIPP
U+9EA4.0	fFourCorner	00210
U+9EA4.0	fHakka	 cu1; cih1
U+9EA4.0	fCantonese	cou1
U+9EA4.0	fMandarin	cu1; ceng4
U+9EA4.0	fEnglish	rough; coarse
U+9EA5.0	fUTF8	麥
U+9EA5.0	fCNS11643	1-5E27
U+9EA5.0	fBig5	B3C1
U+9EA5.0	fGB	1-4273
U+9EA5.0	fR/S	 199.0
U+9EA5.0	fTotalStrokes	 11
U+9EA5.0	fCangjie	JONI
U+9EA5.0	fFourCorner	40207
U+9EA5.0	fHakka	  mak8
U+9EA5.0	fCantonese	mak6; maak6 (also)
U+9EA5.0	fMandarin	mai4; mo4
U+9EA5.0	fEnglish	[1] wheat; barley; oats [2] a Chinese family naem [3] Kangxi radical 199
U+9EA6.0	fUTF8	麦
U+9EA6.0	fCNS11643	3-2547
U+9EA6.0	fGB	0-4273
U+9EA6.0	fR/S	 199.0; 34.5; 29.5
U+9EA6.0	fTotalStrokes	 7; 11
U+9EA6.0	fCangjie	QME
U+9EA6.0	fFourCorner	50240
U+9EA6.0	fHakka	 mak8
U+9EA6.0	fCantonese	mak6; maak6 (also)
U+9EA6.0	fMandarin	mai4; mo4
U+9EA6.0	fEnglish	simplified form of &#x9EA5;
U+9EA7.0	fUTF8	麧
U+9EA7.0	fCNS11643	2-4F4D
U+9EA7.0	fBig5	E4E5
U+9EA7.0	fGB	5-6C7C
U+9EA7.0	fR/S	 199.3
U+9EA7.0	fTotalStrokes	 14
U+9EA7.0	fCangjie	JEON
U+9EA7.0	fFourCorner	48217
U+9EA7.0	fHakka	 ngiet7;  ngut7
U+9EA7.0	fCantonese	hat6; ngat6
U+9EA7.0	fMandarin	ge1; he2; xie2
U+9EA7.0	fEnglish	wheat in the husk
U+9EA8.0	fUTF8	麨
U+9EA8.0	fCNS11643	4-4F4C
U+9EA8.0	fGB	5-6D21
U+9EA8.0	fR/S	 199.4
U+9EA8.0	fTotalStrokes	 15
U+9EA8.0	fCangjie	JEFH
U+9EA8.0	fFourCorner	49402
U+9EA8.0	fCantonese	ciu2
U+9EA8.0	fMandarin	chao3
U+9EA9.0	fUTF8	麩
U+9EA9.0	fCNS11643	1-702B
U+9EA9.0	fBig5	BEA2
U+9EA9.0	fGB	1-746F
U+9EA9.0	fR/S	 199.4
U+9EA9.0	fTotalStrokes	 15
U+9EA9.0	fCangjie	JEQO
U+9EA9.0	fFourCorner	45243
U+9EA9.0	fHakka	 fu2;  fu1
U+9EA9.0	fCantonese	fu1
U+9EA9.0	fMandarin	fu1
U+9EA9.0	fEnglish	bran
U+9EAA.0	fUTF8	麪
U+9EAA.0	fCNS11643	3-502F
U+9EAA.0	fGB	E-4866
U+9EAA.0	fR/S	 199.4
U+9EAA.0	fTotalStrokes	 15
U+9EAA.0	fCangjie	JEMLS
U+9EAA.0	fFourCorner	41240
U+9EAA.0	fHakka	 men3; mien5
U+9EAA.0	fCantonese	min6
U+9EAA.0	fMandarin	mian3; mian4
U+9EAA.0	fEnglish	flour; noodles; dough
U+9EAB.0	fUTF8	麫
U+9EAB.0	fCNS11643	3-502E
U+9EAB.0	fGB	E-4867
U+9EAB.0	fR/S	 199.4; 199.5
U+9EAB.0	fTotalStrokes	 15; 16
U+9EAB.0	fCangjie	JEMYS
U+9EAB.0	fFourCorner	41240
U+9EAB.0	fCantonese	min6
U+9EAB.0	fMandarin	mian3; mian4
U+9EAB.0	fEnglish	 flour vermicelli dough
U+9EAC.0	fUTF8	麬
U+9EAC.0	fCNS11643	3-543A
U+9EAC.0	fGB	E-4868
U+9EAC.0	fR/S	 199.5
U+9EAC.0	fTotalStrokes	 16
U+9EAC.0	fCangjie	JEDHE
U+9EAC.0	fFourCorner	44404
U+9EAC.0	fCantonese	fu1
U+9EAC.0	fMandarin	fu1
U+9EAD.0	fUTF8	麭
U+9EAD.0	fCNS11643	2-5C3F
U+9EAD.0	fBig5	ECB3
U+9EAD.0	fGB	E-4869
U+9EAD.0	fR/S	 199.5
U+9EAD.0	fTotalStrokes	 16
U+9EAD.0	fCangjie	JEPRU
U+9EAD.0	fFourCorner	47241
U+9EAD.0	fCantonese	paau3
U+9EAD.0	fMandarin	pao4
U+9EAD.0	fEnglish	a sticky rice ball
U+9EAE.0	fUTF8	麮
U+9EAE.0	fCNS11643	2-5C3E
U+9EAE.0	fBig5	ECB2
U+9EAE.0	fGB	3-6D60
U+9EAE.0	fR/S	 199.5
U+9EAE.0	fTotalStrokes	 16
U+9EAE.0	fCangjie	JEGI
U+9EAE.0	fFourCorner	44403
U+9EAE.0	fCantonese	heoi3
U+9EAE.0	fMandarin	qu4; qu3
U+9EAE.0	fEnglish	porridge
U+9EAF.0	fUTF8	麯
U+9EAF.0	fCNS11643	3-5750
U+9EAF.0	fGB	1-7865
U+9EAF.0	fR/S	 199.6
U+9EAF.0	fTotalStrokes	 17
U+9EAF.0	fCangjie	JETW
U+9EAF.0	fFourCorner	45246
U+9EAF.0	fHakka	  kiuk7; giuk8
U+9EAF.0	fCantonese	kuk1; guk1 (also)
U+9EAF.0	fMandarin	qu2; qu1
U+9EAF.0	fEnglish	yeast; leaven
U+9EB0.0	fUTF8	麰
U+9EB0.0	fCNS11643	2-6139
U+9EB0.0	fBig5	EFAD
U+9EB0.0	fGB	3-6D64
U+9EB0.0	fR/S	 199.6
U+9EB0.0	fTotalStrokes	 17
U+9EB0.0	fCangjie	JEIHQ
U+9EB0.0	fFourCorner	43405
U+9EB0.0	fHakka	  meu2
U+9EB0.0	fCantonese	mau4
U+9EB0.0	fMandarin	mou2
U+9EB0.0	fEnglish	barley
U+9EB1.0	fUTF8	麱
U+9EB1.0	fCNS11643	3-5A26
U+9EB1.0	fGB	5-6D26
U+9EB1.0	fR/S	 199.7
U+9EB1.0	fTotalStrokes	 18
U+9EB1.0	fCangjie	JEIJB
U+9EB1.0	fFourCorner	43402
U+9EB1.0	fCantonese	fu1
U+9EB1.0	fMandarin	fu1
U+9EB2.0	fUTF8	麲
U+9EB2.0	fCNS11643	3-5A25
U+9EB2.0	fGB	E-486A
U+9EB2.0	fR/S	 199.7
U+9EB2.0	fTotalStrokes	 18
U+9EB2.0	fCangjie	JEBUU
U+9EB2.0	fFourCorner	46241
U+9EB2.0	fCantonese	haan6
U+9EB2.0	fMandarin	xian4
U+9EB3.0	fUTF8	麳
U+9EB3.0	fCNS11643	4-627E
U+9EB3.0	fGB	3-6D65
U+9EB3.0	fR/S	  199.8
U+9EB3.0	fTotalStrokes	 19
U+9EB3.0	fCangjie	JEDOO
U+9EB3.0	fFourCorner	44290
U+9EB3.0	fCantonese	loi4
U+9EB3.0	fMandarin	lai2; li4; li2
U+9EB4.0	fUTF8	麴
U+9EB4.0	fCNS11643	1-7960
U+9EB4.0	fBig5	C454
U+9EB4.0	fGB	0-7470
U+9EB4.0	fR/S	 199.8
U+9EB4.0	fTotalStrokes	 19
U+9EB4.0	fCangjie	JNPFD
U+9EB4.0	fFourCorner	47242
U+9EB4.0	fHakka	  kiuk7; giuk8
U+9EB4.0	fCantonese	kuk1; guk1 (also)
U+9EB4.0	fMandarin	qu1; qu2; jiao2; qu3
U+9EB4.0	fEnglish	yeast; a ferment for brewing
U+9EB5.0	fUTF8	麵
U+9EB5.0	fCNS11643	1-7A5C
U+9EB5.0	fBig5	C4D1
U+9EB5.0	fGB	1-7859
U+9EB5.0	fR/S	 199.9
U+9EB5.0	fTotalStrokes	 20
U+9EB5.0	fCangjie	JEMWL
U+9EB5.0	fFourCorner	41246
U+9EB5.0	fHakka	  mien5; mien6; men3; mian6
U+9EB5.0	fCantonese	min6
U+9EB5.0	fMandarin	mian4
U+9EB5.0	fEnglish	[1] flour [2] dough [3] noodles
U+9EB6.0	fUTF8	麶
U+9EB6.0	fCNS11643	2-6E74
U+9EB6.0	fBig5	F7C7
U+9EB6.0	fGB	E-486B
U+9EB6.0	fR/S	 199.11
U+9EB6.0	fTotalStrokes	 22
U+9EB6.0	fCangjie	JEYUB
U+9EB6.0	fFourCorner	40427
U+9EB6.0	fCantonese	ci1; nim4
U+9EB6.0	fMandarin	li2; chi1
U+9EB7.0	fUTF8	麷
U+9EB7.0	fCNS11643	2-7239
U+9EB7.0	fBig5	F9CB
U+9EB7.0	fGB	3-6D5B
U+9EB7.0	fR/S	 199.18
U+9EB7.0	fTotalStrokes	 29
U+9EB7.0	fCangjie	JEUJT
U+9EB7.0	fFourCorner	45218
U+9EB7.0	fCantonese	fung1
U+9EB7.0	fMandarin	feng1; peng3
U+9EB8.0	fUTF8	麸
U+9EB8.0	fGB	0-746F
U+9EB8.0	fR/S	 199.4
U+9EB8.0	fTotalStrokes	 11
U+9EB8.0	fCangjie	QEQO
U+9EB8.0	fHakka	 fu1
U+9EB8.0	fCantonese	fu1
U+9EB8.0	fMandarin	fu1
U+9EB8.0	fEnglish	simplified form of &#x9EA9;
U+9EB9.0	fUTF8	麹
U+9EB9.0	fGB	E-486C
U+9EB9.0	fR/S	 199.8
U+9EB9.0	fTotalStrokes	 15
U+9EB9.0	fCangjie	QEPFD
U+9EB9.0	fCantonese	kuk1; guk1 (also)
U+9EB9.0	fMandarin	qu2; qu1
U+9EB9.0	fEnglish	 yeast; leaven a Chinese family name
U+9EBA.0	fUTF8	麺
U+9EBA.0	fGB	E-486D
U+9EBA.0	fR/S	 199.9
U+9EBA.0	fTotalStrokes	 16
U+9EBA.0	fCangjie	QEMWL
U+9EBA.0	fCantonese	min6
U+9EBA.0	fMandarin	mian4
U+9EBA.0	fEnglish	flour; dough; noodles
U+9EBB.0	fUTF8	麻
U+9EBB.0	fCNS11643	1-5E28
U+9EBB.0	fBig5	B3C2
U+9EBB.0	fGB	0-4269
U+9EBB.0	fR/S	 200.0
U+9EBB.0	fTotalStrokes	 11
U+9EBB.0	fCangjie	IJCC
U+9EBB.0	fFourCorner	00294
U+9EBB.0	fHakka	  ma2; ma3
U+9EBB.0	fCantonese	maa4; maa2 (also)
U+9EBB.0	fMandarin	ma2; ma1
U+9EBB.0	fEnglish	[1] hemp; jute; ramie; sisal; flax [2] sesame [3] numb; torpid [4] tingle [5] pockmarked [6] rough [7] pitted; spotty
U+9EBB.1	fUTF8	麻
U+9EBB.1	fCNS11643	1-5E28
U+9EBB.1	fBig5	B3C2
U+9EBB.1	fGB	0-4269
U+9EBB.1	fR/S	 200.0
U+9EBB.1	fTotalStrokes	 11
U+9EBB.1	fCangjie	IJCC
U+9EBB.1	fFourCorner	00294
U+9EBB.1	fMandarin	ma5
U+9EBC.0	fUTF8	麼
U+9EBC.0	fCNS11643	1-6B7B
U+9EBC.0	fBig5	BBF2
U+9EBC.0	fGB	E-486E
U+9EBC.0	fR/S	 200.3
U+9EBC.0	fTotalStrokes	 14
U+9EBC.0	fCangjie	IDVI
U+9EBC.0	fFourCorner	00232
U+9EBC.0	fHakka	  mo2;  mo1; mak7; ma3; man3; mut8; mut7
U+9EBC.0	fCantonese	mo1; maa1; mo5
U+9EBC.0	fMandarin	mo5; mo2; ma1; mo3
U+9EBC.0	fEnglish	[1] tiny [2] a special particle found in dramatic dialogues
U+9EBC.1	fUTF8	麼
U+9EBC.1	fCNS11643	1-6B7B
U+9EBC.1	fBig5	BBF2
U+9EBC.1	fGB	E-486E
U+9EBC.1	fR/S	 200.3
U+9EBC.1	fTotalStrokes	 14
U+9EBC.1	fCangjie	IDVI
U+9EBC.1	fFourCorner	00232
U+9EBC.1	fMandarin	ma5
U+9EBC.1	fEnglish	a interrogative final particle
U+9EBC.2	fUTF8	麼
U+9EBC.2	fCNS11643	1-6B7B
U+9EBC.2	fBig5	BBF2
U+9EBC.2	fGB	E-486E
U+9EBC.2	fR/S	 200.3
U+9EBC.2	fTotalStrokes	 14
U+9EBC.2	fCangjie	IDVI
U+9EBC.2	fFourCorner	00232
U+9EBC.2	fMandarin	ma2
U+9EBC.2	fEnglish	a particle used in combination ( CC)
U+9EBC.3	fUTF8	麼
U+9EBC.3	fCNS11643	1-6B7B
U+9EBC.3	fBig5	BBF2
U+9EBC.3	fGB	E-486E
U+9EBC.3	fR/S	 200.3
U+9EBC.3	fTotalStrokes	 14
U+9EBC.3	fCangjie	IDVI
U+9EBC.3	fFourCorner	00232
U+9EBC.3	fMandarin	me5
U+9EBC.3	fEnglish	a particle used in the interrogative phrase
U+9EBD.0	fUTF8	麽
U+9EBD.0	fCNS11643	3-4B22
U+9EBD.0	fGB	0-7761
U+9EBD.0	fR/S	 200.3
U+9EBD.0	fTotalStrokes	 14
U+9EBD.0	fCangjie	IDHI
U+9EBD.0	fFourCorner	00230
U+9EBD.0	fHakka	  mo1; mo2; mak7; ma3; man3; jau1; mut7
U+9EBD.0	fCantonese	mo1; jiu1; maa1
U+9EBD.0	fMandarin	me0; ma5; me5; yao1; mo5; mo3; mo2
U+9EBD.0	fEnglish	simplified form of &#x4E48;
U+9EBE.0	fUTF8	麾
U+9EBE.0	fCNS11643	1-702C
U+9EBE.0	fBig5	BEA3
U+9EBE.0	fGB	0-7762
U+9EBE.0	fR/S	 200.4
U+9EBE.0	fTotalStrokes	 15
U+9EBE.0	fCangjie	IDHQU
U+9EBE.0	fFourCorner	00214
U+9EBE.0	fHakka	  fui1; fi1
U+9EBE.0	fCantonese	fai1
U+9EBE.0	fMandarin	hui1; hui4
U+9EBE.0	fEnglish	[1] pennant; flag; banner; standard [2] command; lead
U+9EBF.0	fUTF8	麿
U+9EBF.0	fCNS11643	F-617C
U+9EBF.0	fGB	E-486F
U+9EBF.0	fR/S	 200.7
U+9EBF.0	fTotalStrokes	 18
U+9EBF.0	fCangjie	IDRR
U+9EBF.0	fFourCorner	0026
U+9EBF.0	fCantonese	leoi5
U+9EBF.0	fEnglish	I; personal name marker
U+9EC0.0	fUTF8	黀
U+9EC0.0	fCNS11643	2-6861
U+9EC0.0	fBig5	F3F4
U+9EC0.0	fGB	3-7C25
U+9EC0.0	fR/S	 200.8
U+9EC0.0	fTotalStrokes	 19
U+9EC0.0	fCangjie	IDSJE
U+9EC0.0	fFourCorner	00248
U+9EC0.0	fCantonese	zau1
U+9EC0.0	fMandarin	zou1; qu3
U+9EC1.0	fUTF8	黁
U+9EC1.0	fCNS11643	3-5E22
U+9EC1.0	fGB	3-7C27
U+9EC1.0	fR/S	 200.9
U+9EC1.0	fTotalStrokes	 20
U+9EC1.0	fCangjie	IDHDA
U+9EC1.0	fFourCorner	00260
U+9EC1.0	fCantonese	nyun5; lyun5 (also)
U+9EC1.0	fMandarin	nen1; nun2
U+9EC2.0	fUTF8	黂
U+9EC2.0	fCNS11643	2-7024
U+9EC2.0	fBig5	F874
U+9EC2.0	fGB	3-7C26
U+9EC2.0	fR/S	 200.13
U+9EC2.0	fTotalStrokes	 24
U+9EC2.0	fCangjie	IDJTC
U+9EC2.0	fFourCorner	00806
U+9EC2.0	fHakka	 fun2; pun2
U+9EC2.0	fCantonese	fan4; wan4
U+9EC2.0	fMandarin	fen2; fen4
U+9EC2.0	fEnglish	the seeds of flax or hemp
U+9EC3.0	fUTF8	黃
U+9EC3.0	fCNS11643	1-6327
U+9EC3.0	fBig5	B6C0
U+9EC3.0	fGB	E-4870
U+9EC3.0	fR/S	 201.0
U+9EC3.0	fTotalStrokes	 12
U+9EC3.0	fCangjie	TMLC
U+9EC3.0	fFourCorner	44806
U+9EC3.0	fHakka	  wong2; vong2
U+9EC3.0	fCantonese	wong4
U+9EC3.0	fMandarin	huang2
U+9EC3.0	fEnglish	[1] yellow [2] a Chinese family name [3] Kangxi radical 201
U+9EC4.0	fUTF8	黄
U+9EC4.0	fCNS11643	3-396A
U+9EC4.0	fGB	0-3B46
U+9EC4.0	fR/S	 201.0
U+9EC4.0	fTotalStrokes	 11; 12
U+9EC4.0	fCangjie	TLWC
U+9EC4.0	fFourCorner	44800
U+9EC4.0	fHakka	 wong2; wong3
U+9EC4.0	fCantonese	wong4; wong2 (also)
U+9EC4.0	fMandarin	huang2
U+9EC4.0	fEnglish	 yellow a Chinese family name
U+9EC5.0	fUTF8	黅
U+9EC5.0	fCNS11643	3-543C
U+9EC5.0	fGB	3-7B74
U+9EC5.0	fR/S	 201.4
U+9EC5.0	fTotalStrokes	 16
U+9EC5.0	fCangjie	TCOIN
U+9EC5.0	fFourCorner	48820
U+9EC5.0	fCantonese	gam1
U+9EC5.0	fMandarin	jin1
U+9EC6.0	fUTF8	黆
U+9EC6.0	fCNS11643	3-543B
U+9EC6.0	fGB	E-4871
U+9EC6.0	fR/S	 201.4
U+9EC6.0	fTotalStrokes	 16
U+9EC6.0	fCangjie	TCLBU
U+9EC6.0	fFourCorner	44810
U+9EC6.0	fHakka	 gong1; gwong1; kwong1
U+9EC6.0	fCantonese	gwong1; gong1 (also)
U+9EC6.0	fMandarin	guang1
U+9EC6.0	fEnglish	[1] brave [2] valorous
U+9EC7.0	fUTF8	黇
U+9EC7.0	fCNS11643	4-5A5D
U+9EC7.0	fGB	3-7B76
U+9EC7.0	fR/S	 201.5
U+9EC7.0	fTotalStrokes	 16; 17
U+9EC7.0	fCangjie	TCYR
U+9EC7.0	fFourCorner	41860
U+9EC7.0	fCantonese	tim1
U+9EC7.0	fMandarin	tian1
U+9EC8.0	fUTF8	黈
U+9EC8.0	fCNS11643	2-613A
U+9EC8.0	fBig5	EFAE
U+9EC8.0	fGB	E-4872
U+9EC8.0	fR/S	 201.5
U+9EC8.0	fTotalStrokes	 17
U+9EC8.0	fCangjie	TCYG
U+9EC8.0	fFourCorner	40814
U+9EC8.0	fHakka	 teu5
U+9EC8.0	fCantonese	tau2
U+9EC8.0	fMandarin	tou3
U+9EC8.0	fEnglish	[1] yellow [2] augment; increase
U+9EC9.0	fUTF8	黉
U+9EC9.0	fGB	0-5964
U+9EC9.0	fR/S	 201.5
U+9EC9.0	fTotalStrokes	 17
U+9EC9.0	fCangjie	FBTLC
U+9EC9.0	fHakka	 fung2
U+9EC9.0	fCantonese	hung4
U+9EC9.0	fMandarin	hong2; heng2
U+9EC9.0	fEnglish	simplified form of &#x9ECC;
U+9ECA.0	fUTF8	黊
U+9ECA.0	fCNS11643	3-5A28
U+9ECA.0	fGB	3-7B77
U+9ECA.0	fR/S	  201.6
U+9ECA.0	fTotalStrokes	 18
U+9ECA.0	fCangjie	TCGG
U+9ECA.0	fFourCorner	44810
U+9ECA.0	fCantonese	waa6
U+9ECA.0	fMandarin	xi1; hua4
U+9ECB.0	fUTF8	黋
U+9ECB.0	fCNS11643	4-5F3E
U+9ECB.0	fGB	3-7B7A
U+9ECB.0	fR/S	 201.6
U+9ECB.0	fTotalStrokes	 18
U+9ECB.0	fCangjie	FUTLC
U+9ECB.0	fFourCorner	94210
U+9ECB.0	fCantonese	kong3; kwong3 (also)
U+9ECB.0	fMandarin	kuang3; kuang4
U+9ECC.0	fUTF8	黌
U+9ECC.0	fCNS11643	1-7D31
U+9ECC.0	fBig5	C664
U+9ECC.0	fGB	1-5964
U+9ECC.0	fR/S	 201.13
U+9ECC.0	fTotalStrokes	 25
U+9ECC.0	fCangjie	HBTLC
U+9ECC.0	fFourCorner	77806
U+9ECC.0	fHakka	 wang2; fung2
U+9ECC.0	fCantonese	hung4; waang4
U+9ECC.0	fMandarin	hong2; heng2
U+9ECC.0	fEnglish	 school
U+9ECD.0	fUTF8	黍
U+9ECD.0	fCNS11643	1-6328
U+9ECD.0	fBig5	B6C1
U+9ECD.0	fGB	0-4A72
U+9ECD.0	fR/S	 202.0
U+9ECD.0	fTotalStrokes	 12
U+9ECD.0	fCangjie	HDOE
U+9ECD.0	fFourCorner	20132
U+9ECD.0	fHakka	  shu3; su5; su3
U+9ECD.0	fCantonese	syu2
U+9ECD.0	fMandarin	shu3
U+9ECD.0	fEnglish	[1] a variety of millet [2] Kangxi radical 202
U+9ECE.0	fUTF8	黎
U+9ECE.0	fCNS11643	1-702D
U+9ECE.0	fBig5	BEA4
U+9ECE.0	fGB	0-4068
U+9ECE.0	fR/S	 202.3
U+9ECE.0	fTotalStrokes	 15
U+9ECE.0	fCangjie	HHOE
U+9ECE.0	fFourCorner	27132
U+9ECE.0	fHakka	 lai2; li2; li3; lai3; le2
U+9ECE.0	fCantonese	lai4
U+9ECE.0	fMandarin	li2; li4
U+9ECE.0	fEnglish	[1] a Chinese family name [2] numerous; many [3] the aborignes of Hainan [4] black; dark
U+9ECF.0	fUTF8	黏
U+9ECF.0	fCNS11643	1-7634
U+9ECF.0	fBig5	C248
U+9ECF.0	fGB	0-7024
U+9ECF.0	fR/S	 202.5
U+9ECF.0	fTotalStrokes	 17
U+9ECF.0	fCangjie	HEYR
U+9ECF.0	fFourCorner	21160
U+9ECF.0	fHakka	 ngiam2; zham1; zam1
U+9ECF.0	fCantonese	nim1; nim4; lim1 (also); zim1; lim4 (also)
U+9ECF.0	fMandarin	nian2; zhan1; nian1
U+9ECF.0	fEnglish	[1] stick (to) [2] glutinous; sticky; gluey; adhesive; clammy; viscid
U+9ED0.0	fUTF8	黐
U+9ED0.0	fCNS11643	2-7025
U+9ED0.0	fBig5	F875
U+9ED0.0	fGB	3-7C5A
U+9ED0.0	fR/S	 202.10; 202.11
U+9ED0.0	fTotalStrokes	 22
U+9ED0.0	fCangjie	HEYUB
U+9ED0.0	fFourCorner	20127
U+9ED0.0	fHakka	 li2; ci1; ti1
U+9ED0.0	fCantonese	ci1
U+9ED0.0	fMandarin	li2; chi1
U+9ED0.0	fEnglish	 stick sticky
U+9ED1.0	fUTF8	黑
U+9ED1.0	fCNS11643	1-6329
U+9ED1.0	fBig5	B6C2
U+9ED1.0	fGB	0-3A5A
U+9ED1.0	fR/S	 203.0
U+9ED1.0	fTotalStrokes	 12
U+9ED1.0	fCangjie	WGF
U+9ED1.0	fFourCorner	60331
U+9ED1.0	fHakka	  het7; hek7; het8
U+9ED1.0	fCantonese	hak1; haak1 (also)
U+9ED1.0	fMandarin	hei1; he4; hei3
U+9ED1.0	fEnglish	[1] black; dark [2] evil; sinister; gloomy [3] Kangxi radical 203
U+9ED2.0	fUTF8	黒
U+9ED2.0	fCNS11643	3-396B
U+9ED2.0	fGB	E-4873
U+9ED2.0	fR/S	 203.0
U+9ED2.0	fTotalStrokes	 11; 12
U+9ED2.0	fCangjie	WGF
U+9ED2.0	fFourCorner	60330
U+9ED2.0	fHakka	 het7
U+9ED2.0	fCantonese	hak1
U+9ED2.0	fMandarin	hei1; he4
U+9ED2.0	fEnglish	 black dark evil
U+9ED3.0	fUTF8	黓
U+9ED3.0	fCNS11643	2-563C
U+9ED3.0	fBig5	E8F1
U+9ED3.0	fGB	3-7C3E
U+9ED3.0	fR/S	 203.3
U+9ED3.0	fTotalStrokes	 15
U+9ED3.0	fCangjie	WFIP
U+9ED3.0	fFourCorner	63340
U+9ED3.0	fHakka	 jit8; rit8
U+9ED3.0	fCantonese	jik6; jit6
U+9ED3.0	fMandarin	yi4
U+9ED3.0	fEnglish	black
U+9ED4.0	fUTF8	黔
U+9ED4.0	fCNS11643	1-733D
U+9ED4.0	fBig5	C072
U+9ED4.0	fGB	0-472D
U+9ED4.0	fR/S	 203.4
U+9ED4.0	fTotalStrokes	 16
U+9ED4.0	fCangjie	WFOIN
U+9ED4.0	fFourCorner	68327
U+9ED4.0	fHakka	 kiam2; kim2
U+9ED4.0	fCantonese	kim4
U+9ED4.0	fMandarin	qian2; qin2
U+9ED4.0	fEnglish	[1] black [2] another name for Kweichow
U+9ED5.0	fUTF8	黕
U+9ED5.0	fCNS11643	2-5C40
U+9ED5.0	fBig5	ECB4
U+9ED5.0	fGB	3-7C43
U+9ED5.0	fR/S	 203.4
U+9ED5.0	fTotalStrokes	 16
U+9ED5.0	fCangjie	WFLBU
U+9ED5.0	fFourCorner	64312
U+9ED5.0	fCantonese	dam2; daam2
U+9ED5.0	fMandarin	dan1; zhen3; dan3
U+9ED5.0	fEnglish	red
U+9ED6.0	fUTF8	黖
U+9ED6.0	fCNS11643	2-5C41
U+9ED6.0	fBig5	ECB5
U+9ED6.0	fGB	3-7C42
U+9ED6.0	fR/S	 203.4
U+9ED6.0	fTotalStrokes	 16
U+9ED6.0	fCangjie	WFMVU
U+9ED6.0	fFourCorner	61314
U+9ED6.0	fCantonese	hei3
U+9ED6.0	fMandarin	xi4
U+9ED7.0	fUTF8	黗
U+9ED7.0	fCNS11643	3-543D
U+9ED7.0	fGB	3-7C41
U+9ED7.0	fR/S	 203.4
U+9ED7.0	fTotalStrokes	 16
U+9ED7.0	fCangjie	WFPU
U+9ED7.0	fFourCorner	65310
U+9ED7.0	fCantonese	tan1
U+9ED7.0	fMandarin	tuan3
U+9ED8.0	fUTF8	默
U+9ED8.0	fCNS11643	1-733C
U+9ED8.0	fBig5	C071
U+9ED8.0	fGB	0-442C
U+9ED8.0	fR/S	 203.4
U+9ED8.0	fTotalStrokes	 16
U+9ED8.0	fCangjie	WFIK
U+9ED8.0	fFourCorner	63334
U+9ED8.0	fHakka	  met8; miet8
U+9ED8.0	fCantonese	mak6
U+9ED8.0	fMandarin	mo4
U+9ED8.0	fEnglish	[1] speechless; silent [2] quiet; still
U+9ED9.0	fUTF8	黙
U+9ED9.0	fCNS11643	4-5560
U+9ED9.0	fGB	E-4874
U+9ED9.0	fR/S	 203.4; 86.11
U+9ED9.0	fTotalStrokes	 15; 16; 23
U+9ED9.0	fCangjie	WKF
U+9ED9.0	fFourCorner	63330
U+9ED9.0	fCantonese	mak6
U+9ED9.0	fMandarin	mo4
U+9ED9.0	fEnglish	 silent; quiet; still dark
U+9EDA.0	fUTF8	黚
U+9EDA.0	fCNS11643	2-613B
U+9EDA.0	fBig5	EFAF
U+9EDA.0	fGB	3-7C44
U+9EDA.0	fR/S	 203.5
U+9EDA.0	fTotalStrokes	 17
U+9EDA.0	fCangjie	WFTM
U+9EDA.0	fFourCorner	64370
U+9EDA.0	fHakka	 kiam2
U+9EDA.0	fCantonese	kim4
U+9EDA.0	fMandarin	qian2; qin2; gan4
U+9EDB.0	fUTF8	黛
U+9EDB.0	fCNS11643	1-7638
U+9EDB.0	fBig5	C24C
U+9EDB.0	fGB	0-776C
U+9EDB.0	fR/S	 203.5
U+9EDB.0	fTotalStrokes	 17
U+9EDB.0	fCangjie	OPWGF
U+9EDB.0	fFourCorner	23331
U+9EDB.0	fHakka	 toi5; toi6; toi3
U+9EDB.0	fCantonese	doi6
U+9EDB.0	fMandarin	dai4
U+9EDB.0	fEnglish	[1] a bluish-black material used by women to blacken their eyebrows [2] a beauty
U+9EDC.0	fUTF8	黜
U+9EDC.0	fCNS11643	1-7636
U+9EDC.0	fBig5	C24A
U+9EDC.0	fGB	0-776D
U+9EDC.0	fR/S	 203.5
U+9EDC.0	fTotalStrokes	 17
U+9EDC.0	fCangjie	WFUU
U+9EDC.0	fFourCorner	62372
U+9EDC.0	fHakka	  tut7; cut7; zhot7
U+9EDC.0	fCantonese	zeot1; ceot1 (also); zyut3 (also); zyut6 (also)
U+9EDC.0	fMandarin	chu4
U+9EDC.0	fEnglish	[1] reject; dispel [2] dismiss; demote; downgrade; degrade
U+9EDD.0	fUTF8	黝
U+9EDD.0	fCNS11643	1-7637
U+9EDD.0	fBig5	C24B
U+9EDD.0	fGB	0-776E
U+9EDD.0	fR/S	 203.5
U+9EDD.0	fTotalStrokes	 17
U+9EDD.0	fCangjie	WFVIS
U+9EDD.0	fFourCorner	64327
U+9EDD.0	fHakka	 ju3;  ju5; riu3; jiu3; jiu5
U+9EDD.0	fCantonese	jau2
U+9EDD.0	fMandarin	you3; yao4; yi1
U+9EDD.0	fEnglish	bluish black
U+9EDE.0	fUTF8	點
U+9EDE.0	fCNS11643	1-7635
U+9EDE.0	fBig5	C249
U+9EDE.0	fGB	1-3563
U+9EDE.0	fR/S	 203.5
U+9EDE.0	fTotalStrokes	 17
U+9EDE.0	fCangjie	WFYR
U+9EDE.0	fFourCorner	61360
U+9EDE.0	fHakka	  diam3; {diam5}; diam5; dem1; dien3
U+9EDE.0	fCantonese	dim2
U+9EDE.0	fMandarin	dian3; zhan1; dian4; duo4
U+9EDE.0	fEnglish	[1] dot; speck; spot [2] point [3] drop; small amount; a little [4] snacks; refreshments [5] hours; [6] dot; mark [7] instruct; teach [8] check; examine; investigate; review [9] select; pick out [10] light; enlight; ignite [11] nod (the head) [12] touch; poit at
U+9EDF.0	fUTF8	黟
U+9EDF.0	fCNS11643	2-652C
U+9EDF.0	fBig5	F1E0
U+9EDF.0	fGB	0-7770
U+9EDF.0	fR/S	 203.6
U+9EDF.0	fTotalStrokes	 18
U+9EDF.0	fCangjie	WFNIN
U+9EDF.0	fFourCorner	62327
U+9EDF.0	fHakka	 ji1; ri1; i1; ji2
U+9EDF.0	fCantonese	ji1
U+9EDF.0	fMandarin	yi1
U+9EDF.0	fEnglish	[1] dark; black [2] ebony
U+9EE0.0	fUTF8	黠
U+9EE0.0	fCNS11643	1-7828
U+9EE0.0	fBig5	C35C
U+9EE0.0	fGB	0-776F
U+9EE0.0	fR/S	 203.6
U+9EE0.0	fTotalStrokes	 18
U+9EE0.0	fCangjie	WFGR
U+9EE0.0	fFourCorner	64361
U+9EE0.0	fHakka	 giet7;  gat7; get7
U+9EE0.0	fCantonese	kit3; hat6 (also); waat6 (also); hat1
U+9EE0.0	fMandarin	xie2; xia2
U+9EE0.0	fEnglish	[1] smart; clever; shrewd [2] artful; crafty; wily; cunning
U+9EE1.0	fUTF8	黡
U+9EE1.0	fGB	8-7A26
U+9EE1.0	fR/S	 203.6
U+9EE1.0	fTotalStrokes	 18
U+9EE1.0	fCangjie	MIKF
U+9EE1.0	fCantonese	jim2
U+9EE1.0	fMandarin	yan3
U+9EE1.0	fEnglish	mole, scar, blemish
U+9EE2.0	fUTF8	黢
U+9EE2.0	fCNS11643	4-6324
U+9EE2.0	fGB	0-7771
U+9EE2.0	fR/S	  203.7
U+9EE2.0	fTotalStrokes	 19
U+9EE2.0	fCangjie	WFICE
U+9EE2.0	fFourCorner	63340
U+9EE2.0	fCantonese	ceot1 (also); zeot1
U+9EE2.0	fMandarin	cu4; qu1; qu4
U+9EE2.0	fEnglish	black
U+9EE3.0	fUTF8	黣
U+9EE3.0	fCNS11643	3-5C2A
U+9EE3.0	fGB	3-7C47
U+9EE3.0	fR/S	 203.7
U+9EE3.0	fTotalStrokes	 19
U+9EE3.0	fCangjie	WFOWY
U+9EE3.0	fFourCorner	68350
U+9EE3.0	fCantonese	mui5
U+9EE3.0	fMandarin	mei3
U+9EE4.0	fUTF8	黤
U+9EE4.0	fCNS11643	2-6B43
U+9EE4.0	fBig5	F5B5
U+9EE4.0	fGB	5-7A39
U+9EE4.0	fR/S	 203.8
U+9EE4.0	fTotalStrokes	 20
U+9EE4.0	fCangjie	WFKLU
U+9EE4.0	fFourCorner	64316
U+9EE4.0	fCantonese	jim2; am2; ngam2 (also)
U+9EE4.0	fMandarin	yan3; an4
U+9EE4.0	fEnglish	bluish black
U+9EE5.0	fUTF8	黥
U+9EE5.0	fCNS11643	2-6B42
U+9EE5.0	fBig5	F5B4
U+9EE5.0	fGB	0-7774
U+9EE5.0	fR/S	 203.8
U+9EE5.0	fTotalStrokes	 20
U+9EE5.0	fCangjie	WFYRF
U+9EE5.0	fFourCorner	60396
U+9EE5.0	fHakka	  gin1;  kiang2
U+9EE5.0	fCantonese	king4
U+9EE5.0	fMandarin	qing2; jing1
U+9EE5.0	fEnglish	an ancient punishment of tattooing the face as a punishment; branding
U+9EE6.0	fUTF8	黦
U+9EE6.0	fCNS11643	2-6B45
U+9EE6.0	fBig5	F5B7
U+9EE6.0	fGB	3-7C49
U+9EE6.0	fR/S	 203.8
U+9EE6.0	fTotalStrokes	 20
U+9EE6.0	fCangjie	WFJNU
U+9EE6.0	fFourCorner	63312
U+9EE6.0	fHakka	 ngut7; jut7; rut7
U+9EE6.0	fCantonese	wat1; jyut3 (also); jit3
U+9EE6.0	fMandarin	yu4; ye4; yue4
U+9EE6.0	fEnglish	yellowish black
U+9EE7.0	fUTF8	黧
U+9EE7.0	fCNS11643	2-6B44
U+9EE7.0	fBig5	F5B6
U+9EE7.0	fGB	0-7773
U+9EE7.0	fR/S	 203.8
U+9EE7.0	fTotalStrokes	 20
U+9EE7.0	fCangjie	HHWGF
U+9EE7.0	fFourCorner	27331
U+9EE7.0	fHakka	 li2; li3; lai2
U+9EE7.0	fCantonese	lai4
U+9EE7.0	fMandarin	li2; lai2
U+9EE7.0	fEnglish	dark yellow; sallow
U+9EE8.0	fUTF8	黨
U+9EE8.0	fCNS11643	1-7A5D
U+9EE8.0	fBig5	C4D2
U+9EE8.0	fGB	1-3533
U+9EE8.0	fR/S	 203.8
U+9EE8.0	fTotalStrokes	 20
U+9EE8.0	fCangjie	FBRWF
U+9EE8.0	fFourCorner	90331
U+9EE8.0	fHakka	  dong3
U+9EE8.0	fCantonese	dong2; tong2
U+9EE8.0	fMandarin	dang3; zhang3
U+9EE8.0	fEnglish	[1] party; gang; faction; clique; association [2] relative [3] community of 500 families [4] take sides; associate; be a partizan [5] correct form of &#x village;
U+9EE9.0	fUTF8	黩
U+9EE9.0	fGB	0-7772
U+9EE9.0	fR/S	 203.8
U+9EE9.0	fTotalStrokes	 20
U+9EE9.0	fCangjie	WFJNK
U+9EE9.0	fHakka	 tuk8
U+9EE9.0	fCantonese	duk6
U+9EE9.0	fMandarin	du2
U+9EE9.0	fEnglish	simplified form of &#x9EF7;
U+9EEA.0	fUTF8	黪
U+9EEA.0	fGB	0-7775
U+9EEA.0	fR/S	 203.8
U+9EEA.0	fTotalStrokes	 20
U+9EEA.0	fCangjie	WFIKH
U+9EEA.0	fHakka	 cam3
U+9EEA.0	fCantonese	caam2
U+9EEA.0	fMandarin	can3
U+9EEA.0	fEnglish	simplified form of &#x9EF2;
U+9EEB.0	fUTF8	黫
U+9EEB.0	fCNS11643	2-6D39
U+9EEB.0	fBig5	F6CB
U+9EEB.0	fGB	5-7A3F
U+9EEB.0	fR/S	 203.9
U+9EEB.0	fTotalStrokes	 21
U+9EEB.0	fCangjie	WFMWG
U+9EEB.0	fFourCorner	61314
U+9EEB.0	fCantonese	jan1
U+9EEB.0	fMandarin	yin1; yan1
U+9EEC.0	fUTF8	黬
U+9EEC.0	fCNS11643	4-6856
U+9EEC.0	fGB	3-7C4C
U+9EEC.0	fR/S	 203.9
U+9EEC.0	fTotalStrokes	 21
U+9EEC.0	fCangjie	WFIHR
U+9EEC.0	fFourCorner	63350
U+9EEC.0	fCantonese	am2; ngam2 (also); am3 (also); ngam3 (also); ngaam4
U+9EEC.0	fMandarin	an4
U+9EEC.0	fEnglish	blackhead
U+9EED.0	fUTF8	黭
U+9EED.0	fCNS11643	2-6D3B
U+9EED.0	fBig5	F6CD
U+9EED.0	fGB	3-7C4D
U+9EED.0	fR/S	 203.9
U+9EED.0	fTotalStrokes	 21
U+9EED.0	fCangjie	WFORT
U+9EED.0	fFourCorner	68346
U+9EED.0	fHakka	  am5; jam3; ram3
U+9EED.0	fCantonese	am2; jim2; ngam2 (also)
U+9EED.0	fMandarin	yan3; yan1; an4; yin4
U+9EED.0	fEnglish	[1] turned black (as fruit) [2] sudden; quick
U+9EEE.0	fUTF8	黮
U+9EEE.0	fCNS11643	2-6D3A
U+9EEE.0	fBig5	F6CC
U+9EEE.0	fGB	3-7C4B
U+9EEE.0	fR/S	 203.9
U+9EEE.0	fTotalStrokes	 21
U+9EEE.0	fCangjie	WFTMV
U+9EEE.0	fFourCorner	64311
U+9EEE.0	fHakka	 tam3; shim3; siim3; tam5
U+9EEE.0	fCantonese	taam5 (also); sam6; dam2; daam6; taam2; taam3; taan2
U+9EEE.0	fMandarin	tan3; tan2; dan3; dan4
U+9EEE.0	fEnglish	pitch-dark; pitch dark
U+9EEE.1	fUTF8	黮
U+9EEE.1	fCNS11643	2-6D3A
U+9EEE.1	fBig5	F6CC
U+9EEE.1	fGB	3-7C4B
U+9EEE.1	fR/S	 203.9
U+9EEE.1	fTotalStrokes	 21
U+9EEE.1	fCangjie	WFTMV
U+9EEE.1	fFourCorner	64311
U+9EEE.1	fMandarin	shen4
U+9EEE.1	fEnglish	 mulberry
U+9EEF.0	fUTF8	黯
U+9EEF.0	fCNS11643	1-7B52
U+9EEF.0	fBig5	C566
U+9EEF.0	fGB	0-7776
U+9EEF.0	fR/S	 180.12; 203.9
U+9EEF.0	fTotalStrokes	 21
U+9EEF.0	fCangjie	WFYTA
U+9EEF.0	fFourCorner	60361
U+9EEF.0	fHakka	  am5
U+9EEF.0	fCantonese	am2; ngam2 (also); am3
U+9EEF.0	fMandarin	an4; an1
U+9EEF.0	fEnglish	[1] very dark; pitch-dark [2] miserable; dismal
U+9EF0.0	fUTF8	黰
U+9EF0.0	fCNS11643	2-6E75
U+9EF0.0	fBig5	F7C8
U+9EF0.0	fGB	5-7A44
U+9EF0.0	fR/S	 203.10
U+9EF0.0	fTotalStrokes	 22
U+9EF0.0	fCangjie	WFJBC
U+9EF0.0	fFourCorner	64381
U+9EF0.0	fHakka	 zhin3; zihn3
U+9EF0.0	fCantonese	zan2 (also); can2
U+9EF0.0	fMandarin	zhen3
U+9EF0.0	fEnglish	young girl's hair style
U+9EF1.0	fUTF8	黱
U+9EF1.0	fCNS11643	3-6029
U+9EF1.0	fGB	5-4F76
U+9EF1.0	fR/S	  203.10
U+9EF1.0	fTotalStrokes	 22
U+9EF1.0	fCangjie	BFQF
U+9EF1.0	fFourCorner	79230
U+9EF1.0	fCantonese	doi6
U+9EF1.0	fMandarin	dai4
U+9EF2.0	fUTF8	黲
U+9EF2.0	fCNS11643	2-7026
U+9EF2.0	fBig5	F876
U+9EF2.0	fGB	1-7775
U+9EF2.0	fR/S	 203.11
U+9EF2.0	fTotalStrokes	 23
U+9EF2.0	fCangjie	WFIIH
U+9EF2.0	fFourCorner	63322
U+9EF2.0	fHakka	  cam3; ciam3
U+9EF2.0	fCantonese	caam2
U+9EF2.0	fMandarin	can3
U+9EF2.0	fEnglish	light bluish black
U+9EF3.0	fUTF8	黳
U+9EF3.0	fCNS11643	2-7027
U+9EF3.0	fBig5	F877
U+9EF3.0	fGB	3-7C4E
U+9EF3.0	fR/S	 203.11
U+9EF3.0	fTotalStrokes	 23
U+9EF3.0	fCangjie	SEWGF
U+9EF3.0	fFourCorner	77331
U+9EF3.0	fCantonese	ji1
U+9EF3.0	fMandarin	yi1
U+9EF4.0	fUTF8	黴
U+9EF4.0	fCNS11643	1-7C5C
U+9EF4.0	fBig5	C5F0
U+9EF4.0	fGB	1-7854
U+9EF4.0	fR/S	 203.11
U+9EF4.0	fTotalStrokes	 23
U+9EF4.0	fCangjie	HOUFK
U+9EF4.0	fFourCorner	28240
U+9EF4.0	fHakka	  mi2; moi1; mui5; mi5
U+9EF4.0	fCantonese	mei4; mui4 (also)
U+9EF4.0	fMandarin	mei2; mei4
U+9EF4.0	fEnglish	[1] mold; mildew; must [2] germs; bacteria [3] fungi [4] dirty; dingy
U+9EF5.0	fUTF8	黵
U+9EF5.0	fCNS11643	2-7153
U+9EF5.0	fBig5	F964
U+9EF5.0	fGB	3-7C51
U+9EF5.0	fR/S	 203.13
U+9EF5.0	fTotalStrokes	 25
U+9EF5.0	fCangjie	WFNCR
U+9EF5.0	fFourCorner	67361
U+9EF5.0	fCantonese	daam2; zaam2; zin2
U+9EF5.0	fMandarin	dan3
U+9EF5.0	fEnglish	 tattoo the face
U+9EF6.0	fUTF8	黶
U+9EF6.0	fCNS11643	2-716D
U+9EF6.0	fBig5	F97D
U+9EF6.0	fGB	3-3045
U+9EF6.0	fR/S	 203.14
U+9EF6.0	fTotalStrokes	 26
U+9EF6.0	fCangjie	MKWGF
U+9EF6.0	fFourCorner	71331
U+9EF6.0	fHakka	 ram5; jam5; jam3
U+9EF6.0	fCantonese	jim2
U+9EF6.0	fMandarin	yan3
U+9EF6.0	fEnglish	black moles on the skin
U+9EF7.0	fUTF8	黷
U+9EF7.0	fCNS11643	1-7D42
U+9EF7.0	fBig5	C675
U+9EF7.0	fGB	1-7772
U+9EF7.0	fR/S	 203.15
U+9EF7.0	fTotalStrokes	 27
U+9EF7.0	fCangjie	WFGWC
U+9EF7.0	fFourCorner	64386
U+9EF7.0	fHakka	 tuk8
U+9EF7.0	fCantonese	duk6
U+9EF7.0	fMandarin	du2
U+9EF7.0	fEnglish	[1] tarnish [2] be rash about [3] corrupt
U+9EF8.0	fUTF8	黸
U+9EF8.0	fCNS11643	4-6E43
U+9EF8.0	fGB	5-7A32
U+9EF8.0	fR/S	  203.16
U+9EF8.0	fTotalStrokes	 28
U+9EF8.0	fCangjie	WFYPT
U+9EF8.0	fFourCorner	61310
U+9EF8.0	fHakka	 lu2
U+9EF8.0	fCantonese	lou1
U+9EF8.0	fMandarin	lu2
U+9EF9.0	fUTF8	黹
U+9EF9.0	fCNS11643	2-4155
U+9EF9.0	fBig5	DCB0
U+9EF9.0	fGB	0-6D69
U+9EF9.0	fR/S	 204.0
U+9EF9.0	fTotalStrokes	 12
U+9EF9.0	fCangjie	TCFB
U+9EF9.0	fFourCorner	32227
U+9EF9.0	fHakka	 zhi3; zih3; tje3
U+9EF9.0	fCantonese	zi2
U+9EF9.0	fMandarin	zhi3; xian4
U+9EF9.0	fEnglish	[1] embroidery; needlework [2] Kangxi radical 204
U+9EFA.0	fUTF8	黺
U+9EFA.0	fCNS11643	2-5C42
U+9EFA.0	fBig5	ECB6
U+9EFA.0	fGB	E-4875
U+9EFA.0	fR/S	 204.4
U+9EFA.0	fTotalStrokes	 16
U+9EFA.0	fCangjie	TBCSH
U+9EFA.0	fFourCorner	38227
U+9EFA.0	fHakka	  fun3
U+9EFA.0	fCantonese	fan2
U+9EFA.0	fMandarin	fen3
U+9EFA.0	fEnglish	court robes covered with a spotted embroidery
U+9EFB.0	fUTF8	黻
U+9EFB.0	fCNS11643	2-613C
U+9EFB.0	fBig5	EFB0
U+9EFB.0	fGB	0-6D6A
U+9EFB.0	fR/S	 204.5
U+9EFB.0	fTotalStrokes	 17
U+9EFB.0	fCangjie	TBIKK
U+9EFB.0	fFourCorner	33247
U+9EFB.0	fHakka	  fut7
U+9EFB.0	fCantonese	fat1
U+9EFB.0	fMandarin	fu2; fu4
U+9EFB.0	fEnglish	a kind of embroidered design on ancient sacrificial robes; a kind of sacrifical robes
U+9EFC.0	fUTF8	黼
U+9EFC.0	fCNS11643	2-6862
U+9EFC.0	fBig5	F3F5
U+9EFC.0	fGB	0-6D6B
U+9EFC.0	fR/S	 204.7
U+9EFC.0	fTotalStrokes	 19
U+9EFC.0	fCangjie	TBIJB
U+9EFC.0	fFourCorner	33227
U+9EFC.0	fHakka	  pu3
U+9EFC.0	fCantonese	fu2
U+9EFC.0	fMandarin	fu3
U+9EFC.0	fEnglish	a kind of sacrifical robes embroidered with a symbol of distinction
U+9EFD.0	fUTF8	黽
U+9EFD.0	fCNS11643	2-4875
U+9EFD.0	fBig5	E0EF
U+9EFD.0	fGB	1-763C
U+9EFD.0	fR/S	 205.0
U+9EFD.0	fTotalStrokes	 13
U+9EFD.0	fCangjie	RXU
U+9EFD.0	fFourCorner	77717
U+9EFD.0	fHakka	 men3
U+9EFD.0	fCantonese	man5; maang5; min5
U+9EFD.0	fMandarin	min3; mian3; meng3; meng2; mang2; mian2
U+9EFD.0	fEnglish	[1] strive; endeavor [2] Kangxi radical 205
U+9EFE.0	fUTF8	黾
U+9EFE.0	fCNS11643	3-2772
U+9EFE.0	fGB	0-763C
U+9EFE.0	fR/S	 205.0
U+9EFE.0	fTotalStrokes	 8; 13
U+9EFE.0	fCangjie	RLWU
U+9EFE.0	fFourCorner	60710
U+9EFE.0	fHakka	 men3
U+9EFE.0	fCantonese	man5; maang5
U+9EFE.0	fMandarin	min3; mian3; meng3; meng2; mang2; mian2
U+9EFE.0	fEnglish	simplified form of &#x9EFD;
U+9EFF.0	fUTF8	黿
U+9EFF.0	fCNS11643	2-613D
U+9EFF.0	fBig5	EFB1
U+9EFF.0	fGB	1-763D
U+9EFF.0	fR/S	 205.4
U+9EFF.0	fTotalStrokes	 17
U+9EFF.0	fCangjie	MMUU
U+9EFF.0	fFourCorner	10716
U+9EFF.0	fHakka	 ngan2
U+9EFF.0	fCantonese	jyun4
U+9EFF.0	fMandarin	yuan2; wan2
U+9EFF.0	fEnglish	a kind of large turtle
U+9F00.0	fUTF8	鼀
U+9F00.0	fCNS11643	2-652E
U+9F00.0	fBig5	F1E2
U+9F00.0	fGB	5-7446
U+9F00.0	fR/S	 205.5
U+9F00.0	fTotalStrokes	 18
U+9F00.0	fCangjie	GCRXU
U+9F00.0	fFourCorner	40717
U+9F00.0	fCantonese	cuk1
U+9F00.0	fMandarin	cu4; qiu1
U+9F01.0	fUTF8	鼁
U+9F01.0	fCNS11643	2-652D
U+9F01.0	fBig5	F1E1
U+9F01.0	fGB	5-7447
U+9F01.0	fR/S	 205.5
U+9F01.0	fTotalStrokes	 18
U+9F01.0	fCangjie	GIRXU
U+9F01.0	fFourCorner	40717
U+9F01.0	fCantonese	heoi3
U+9F01.0	fMandarin	qu4; qu1
U+9F02.0	fUTF8	鼂
U+9F02.0	fCNS11643	3-5A29
U+9F02.0	fGB	5-7448
U+9F02.0	fR/S	 205.5
U+9F02.0	fTotalStrokes	 18
U+9F02.0	fCangjie	AMRU
U+9F02.0	fFourCorner	60710
U+9F02.0	fHakka	 chau2; cau2
U+9F02.0	fCantonese	ciu4; ziu1
U+9F02.0	fMandarin	chao2; zhao1
U+9F02.0	fEnglish	 a kind of sea turtle a Chinese family name
U+9F03.0	fUTF8	鼃
U+9F03.0	fCNS11643	3-5C2D
U+9F03.0	fGB	E-4876
U+9F03.0	fR/S	 205.6
U+9F03.0	fTotalStrokes	 19
U+9F03.0	fCangjie	GGRXU
U+9F03.0	fFourCorner	40710
U+9F03.0	fHakka	 wa1; va1
U+9F03.0	fCantonese	waa1
U+9F03.0	fMandarin	wa1
U+9F03.0	fEnglish	 frog
U+9F04.0	fUTF8	鼄
U+9F04.0	fCNS11643	3-5C2E
U+9F04.0	fGB	5-744B
U+9F04.0	fR/S	  205.6
U+9F04.0	fTotalStrokes	 19
U+9F04.0	fCangjie	HDRXU
U+9F04.0	fFourCorner	25710
U+9F04.0	fCantonese	zyu1
U+9F04.0	fMandarin	zhu1
U+9F04.0	fEnglish	the spider
U+9F05.0	fUTF8	鼅
U+9F05.0	fCNS11643	3-5F3C
U+9F05.0	fGB	5-744C
U+9F05.0	fR/S	  205.8
U+9F05.0	fTotalStrokes	 21
U+9F05.0	fCangjie	ORRXU
U+9F05.0	fFourCorner	86710
U+9F05.0	fCantonese	zi1
U+9F05.0	fMandarin	zhi1
U+9F05.0	fEnglish	 spider
U+9F06.0	fUTF8	鼆
U+9F06.0	fCNS11643	2-7028
U+9F06.0	fBig5	F878
U+9F06.0	fGB	3-7525
U+9F06.0	fR/S	 205.10
U+9F06.0	fTotalStrokes	 23
U+9F06.0	fCangjie	BCRXU
U+9F06.0	fFourCorner	77817
U+9F06.0	fCantonese	man4
U+9F06.0	fMandarin	mang2; meng3
U+9F07.0	fUTF8	鼇
U+9F07.0	fCNS11643	1-7C7D
U+9F07.0	fBig5	C652
U+9F07.0	fGB	E-4877
U+9F07.0	fR/S	 205.10; 205.11
U+9F07.0	fTotalStrokes	 23
U+9F07.0	fCangjie	GKRXU
U+9F07.0	fFourCorner	48717
U+9F07.0	fHakka	 ngau2
U+9F07.0	fCantonese	ngou4; ou4 (also)
U+9F07.0	fMandarin	ao2
U+9F07.0	fEnglish	a huge sea turtle
U+9F08.0	fUTF8	鼈
U+9F08.0	fCNS11643	3-615F
U+9F08.0	fGB	E-4878
U+9F08.0	fR/S	 205.12; 205.11
U+9F08.0	fTotalStrokes	 25; 24
U+9F08.0	fCangjie	FKRXU
U+9F08.0	fFourCorner	98710
U+9F08.0	fHakka	 bet7; biet7
U+9F08.0	fCantonese	bit3
U+9F08.0	fMandarin	bie1
U+9F08.0	fEnglish	Trionyx sinensis, a kind of fresh-water turtle
U+9F09.0	fUTF8	鼉
U+9F09.0	fCNS11643	2-7154
U+9F09.0	fBig5	F965
U+9F09.0	fGB	1-763E
U+9F09.0	fR/S	 205.12
U+9F09.0	fTotalStrokes	 25
U+9F09.0	fCangjie	RRWMU
U+9F09.0	fFourCorner	66717
U+9F09.0	fHakka	  to2
U+9F09.0	fCantonese	to4
U+9F09.0	fMandarin	tuo2; shan4; tan2
U+9F09.0	fEnglish	a kind of water lizard (whose skin was often used to make drums)
U+9F0A.0	fUTF8	鼊
U+9F0A.0	fCNS11643	2-716E
U+9F0A.0	fBig5	F97E
U+9F0A.0	fGB	E-4879
U+9F0A.0	fR/S	 205.13
U+9F0A.0	fTotalStrokes	 26
U+9F0A.0	fCangjie	SJRXU
U+9F0A.0	fFourCorner	70711
U+9F0A.0	fCantonese	bik1
U+9F0A.0	fMandarin	bi4
U+9F0B.0	fUTF8	鼋
U+9F0B.0	fGB	0-763D
U+9F0B.0	fR/S	 213.4; 205.4
U+9F0B.0	fTotalStrokes	 12
U+9F0B.0	fCangjie	XMMUU
U+9F0B.0	fHakka	 ngan2
U+9F0B.0	fCantonese	jyun4
U+9F0B.0	fMandarin	yuan2; wan2
U+9F0B.0	fEnglish	simplified form of &#x9EFF;
U+9F0C.0	fUTF8	鼌
U+9F0C.0	fGB	8-2E7B
U+9F0C.0	fR/S	 205.5
U+9F0C.0	fTotalStrokes	 13
U+9F0C.0	fCangjie	XAMRU
U+9F0C.0	fCantonese	ciu4; ziu1
U+9F0C.0	fMandarin	chao2; zhao1
U+9F0D.0	fUTF8	鼍
U+9F0D.0	fGB	0-763E
U+9F0D.0	fR/S	 205.12
U+9F0D.0	fTotalStrokes	 20
U+9F0D.0	fCangjie	XRRWM
U+9F0D.0	fHakka	 to2
U+9F0D.0	fCantonese	to4
U+9F0D.0	fMandarin	tuo2; shan4; tan2
U+9F0D.0	fEnglish	simplified form of &#x9F09;
U+9F0E.0	fUTF8	鼎
U+9F0E.0	fCNS11643	1-676F
U+9F0E.0	fBig5	B9A9
U+9F0E.0	fGB	0-3626
U+9F0E.0	fR/S	 206.0
U+9F0E.0	fTotalStrokes	 13
U+9F0E.0	fCangjie	BUVML
U+9F0E.0	fFourCorner	22221
U+9F0E.0	fHakka	  din3; den3
U+9F0E.0	fCantonese	ding2
U+9F0E.0	fMandarin	ding3; zhen1
U+9F0E.0	fEnglish	[1] large, three-legged bronze caldron with two ears; heavy three-legged caldron or sacrificial vessel [2] vigorous; thriving; flourising [3] involving three parts or things; triangular [4] Kangxi radical 206
U+9F0F.0	fUTF8	鼏
U+9F0F.0	fCNS11643	2-563D
U+9F0F.0	fBig5	E8F2
U+9F0F.0	fGB	3-3269
U+9F0F.0	fR/S	 206.2
U+9F0F.0	fTotalStrokes	 15
U+9F0F.0	fCangjie	BBUL
U+9F0F.0	fFourCorner	77227
U+9F0F.0	fHakka	 met7
U+9F0F.0	fCantonese	mik6
U+9F0F.0	fMandarin	mi4
U+9F0F.0	fEnglish	 cover of tripod caldron
U+9F10.0	fUTF8	鼐
U+9F10.0	fCNS11643	2-563E
U+9F10.0	fBig5	E8F3
U+9F10.0	fGB	0-583E
U+9F10.0	fR/S	 206.2
U+9F10.0	fTotalStrokes	 15
U+9F10.0	fCangjie	NSBUL
U+9F10.0	fFourCorner	17227
U+9F10.0	fHakka	 nai5; nai6; lai3
U+9F10.0	fCantonese	naai5; laai5 (also); noi6
U+9F10.0	fMandarin	nai4; nai3; neng4
U+9F10.0	fEnglish	 huge tripod caldron
U+9F11.0	fUTF8	鼑
U+9F11.0	fCNS11643	4-4F56
U+9F11.0	fGB	E-487A
U+9F11.0	fR/S	 206.2
U+9F11.0	fTotalStrokes	 15
U+9F11.0	fCangjie	YUVML
U+9F11.0	fFourCorner	21227
U+9F11.0	fCantonese	ding2
U+9F11.0	fMandarin	ding3; zhen1
U+9F12.0	fUTF8	鼒
U+9F12.0	fCNS11643	2-5C43
U+9F12.0	fBig5	ECB7
U+9F12.0	fGB	3-7C3B
U+9F12.0	fR/S	 206.3
U+9F12.0	fTotalStrokes	 16
U+9F12.0	fCangjie	DHBUL
U+9F12.0	fFourCorner	40227
U+9F12.0	fHakka	 zih1; zii1
U+9F12.0	fCantonese	zi1
U+9F12.0	fMandarin	zi1
U+9F12.0	fEnglish	large tripod caldron with small
U+9F13.0	fUTF8	鼓
U+9F13.0	fCNS11643	1-6770
U+9F13.0	fBig5	B9AA
U+9F13.0	fGB	0-3944
U+9F13.0	fR/S	 207.0
U+9F13.0	fTotalStrokes	 13
U+9F13.0	fCangjie	GTJE
U+9F13.0	fFourCorner	44147
U+9F13.0	fHakka	  gu3; shi6; ku1
U+9F13.0	fCantonese	gu2; gwu2 (also)
U+9F13.0	fMandarin	gu3
U+9F13.0	fEnglish	[1] drum [2] drum; beat a drum [3] vibrate; quiver [4] rouse; stir up; instigate [5] Kangxi radical 207
U+9F14.0	fUTF8	鼔
U+9F14.0	fCNS11643	3-456A
U+9F14.0	fGB	5-4923
U+9F14.0	fR/S	 207.0; 207.9; 66.9
U+9F14.0	fTotalStrokes	 13
U+9F14.0	fCangjie	GTYE
U+9F14.0	fFourCorner	41140
U+9F14.0	fHakka	 gu3
U+9F14.0	fCantonese	gu2; gwu2 (also)
U+9F14.0	fMandarin	gu3
U+9F14.0	fEnglish	 drum drum; from whence comes: rouse swell bulge
U+9F15.0	fUTF8	鼕
U+9F15.0	fCNS11643	1-7829
U+9F15.0	fBig5	C35D
U+9F15.0	fGB	1-782E
U+9F15.0	fR/S	 207.5
U+9F15.0	fTotalStrokes	 18
U+9F15.0	fCangjie	GEHEY
U+9F15.0	fFourCorner	44303
U+9F15.0	fHakka	 tung2; dung2
U+9F15.0	fCantonese	dung1; tung4
U+9F15.0	fMandarin	dong1; tong2
U+9F15.0	fEnglish	the rattle of drums
U+9F16.0	fUTF8	鼖
U+9F16.0	fCNS11643	2-652F
U+9F16.0	fBig5	F1E3
U+9F16.0	fGB	5-7A56
U+9F16.0	fR/S	 207.5
U+9F16.0	fTotalStrokes	 18
U+9F16.0	fCangjie	JTGE
U+9F16.0	fFourCorner	40147
U+9F16.0	fHakka	 fun2
U+9F16.0	fCantonese	fan4; tou4
U+9F16.0	fMandarin	fen2
U+9F16.0	fEnglish	drum
U+9F17.0	fUTF8	鼗
U+9F17.0	fCNS11643	3-5C30
U+9F17.0	fGB	0-583B
U+9F17.0	fR/S	 207.6
U+9F17.0	fTotalStrokes	 19
U+9F17.0	fCangjie	LOGTE
U+9F17.0	fFourCorner	32140
U+9F17.0	fHakka	  tau2; rau2; jeu2; jau2
U+9F17.0	fCantonese	tou4
U+9F17.0	fMandarin	tao2
U+9F17.0	fEnglish	a kind of small hand drum with two swinging knobs (that can strike the face of the drum being twirled)
U+9F18.0	fUTF8	鼘
U+9F18.0	fCNS11643	2-6D3D
U+9F18.0	fBig5	F6CF
U+9F18.0	fGB	E-487B
U+9F18.0	fR/S	 207.8
U+9F18.0	fTotalStrokes	 21
U+9F18.0	fCangjie	GELXL
U+9F18.0	fFourCorner	44224
U+9F18.0	fHakka	 jen1; ren1
U+9F18.0	fCantonese	jyun1
U+9F18.0	fMandarin	yuan1; yin1
U+9F18.0	fEnglish	the roll of drums
U+9F19.0	fUTF8	鼙
U+9F19.0	fCNS11643	1-7B53
U+9F19.0	fBig5	C567
U+9F19.0	fGB	0-5C31
U+9F19.0	fR/S	 207.8
U+9F19.0	fTotalStrokes	 21
U+9F19.0	fCangjie	GEHWJ
U+9F19.0	fFourCorner	44406
U+9F19.0	fHakka	  bi1; pi2
U+9F19.0	fCantonese	pei4
U+9F19.0	fMandarin	pi2; quan2
U+9F19.0	fEnglish	a kind of war drum carried on horseback
U+9F1A.0	fUTF8	鼚
U+9F1A.0	fCNS11643	2-6D3E
U+9F1A.0	fBig5	F6D0
U+9F1A.0	fGB	3-7C5B
U+9F1A.0	fR/S	 207.8
U+9F1A.0	fTotalStrokes	 21
U+9F1A.0	fCangjie	GESMV
U+9F1A.0	fFourCorner	44732
U+9F1A.0	fCantonese	coeng1
U+9F1A.0	fMandarin	chang1
U+9F1A.0	fEnglish	drumming
U+9F1B.0	fUTF8	鼛
U+9F1B.0	fCNS11643	2-6D3C
U+9F1B.0	fBig5	F6CE
U+9F1B.0	fGB	3-7C5F
U+9F1B.0	fR/S	 207.8
U+9F1B.0	fTotalStrokes	 21
U+9F1B.0	fCangjie	GEHOR
U+9F1B.0	fFourCorner	44601
U+9F1B.0	fHakka	  gau1
U+9F1B.0	fCantonese	gou1
U+9F1B.0	fMandarin	gao1
U+9F1B.0	fEnglish	drum
U+9F1C.0	fUTF8	鼜
U+9F1C.0	fCNS11643	2-7029
U+9F1C.0	fBig5	F879
U+9F1C.0	fGB	3-7C61
U+9F1C.0	fR/S	 207.10
U+9F1C.0	fTotalStrokes	 23
U+9F1C.0	fCangjie	GEEII
U+9F1C.0	fFourCorner	44136
U+9F1C.0	fCantonese	cik1
U+9F1C.0	fMandarin	qi4; cao4
U+9F1D.0	fUTF8	鼝
U+9F1D.0	fCNS11643	3-6160
U+9F1D.0	fGB	3-7C62
U+9F1D.0	fR/S	  207.11
U+9F1D.0	fTotalStrokes	 25; 24
U+9F1D.0	fCangjie	ELGTE
U+9F1D.0	fFourCorner	32140
U+9F1D.0	fCantonese	jyun1
U+9F1D.0	fMandarin	yuan1
U+9F1E.0	fUTF8	鼞
U+9F1E.0	fCNS11643	2-7077
U+9F1E.0	fBig5	F8E9
U+9F1E.0	fGB	E-487C
U+9F1E.0	fR/S	 207.11
U+9F1E.0	fTotalStrokes	 24
U+9F1E.0	fCangjie	GEFBG
U+9F1E.0	fFourCorner	44104
U+9F1E.0	fHakka	 tong2; dong1
U+9F1E.0	fCantonese	tong1; dong1; tong4
U+9F1E.0	fMandarin	tang1
U+9F1E.0	fEnglish	noise of drums
U+9F1F.0	fUTF8	鼟
U+9F1F.0	fCNS11643	4-6D48
U+9F1F.0	fGB	3-7C63
U+9F1F.0	fR/S	  207.12
U+9F1F.0	fTotalStrokes	 25
U+9F1F.0	fCangjie	GENOT
U+9F1F.0	fFourCorner	44100
U+9F1F.0	fHakka	 den1; ten1
U+9F1F.0	fCantonese	tang1; dang1 (also)
U+9F1F.0	fMandarin	deng4; teng1; deng1
U+9F1F.0	fEnglish	the roll of drums
U+9F20.0	fUTF8	鼠
U+9F20.0	fCNS11643	1-6771
U+9F20.0	fBig5	B9AB
U+9F20.0	fGB	0-4A73
U+9F20.0	fR/S	 208.0
U+9F20.0	fTotalStrokes	 13
U+9F20.0	fCangjie	HXVYV
U+9F20.0	fFourCorner	77717
U+9F20.0	fHakka	  chu3;  cu3; su3
U+9F20.0	fCantonese	syu2
U+9F20.0	fMandarin	shu3
U+9F20.0	fEnglish	[1] rat; mouse [2] Kangxi radical 208
U+9F21.0	fUTF8	鼡
U+9F21.0	fGB	E-487D
U+9F21.0	fR/S	 208.0; 42.5
U+9F21.0	fTotalStrokes	 13
U+9F21.0	fCangjie	FHNQ
U+9F21.0	fCantonese	lip6
U+9F21.0	fMandarin	shu3
U+9F21.0	fEnglish	rat; mouse
U+9F22.0	fUTF8	鼢
U+9F22.0	fCNS11643	2-6140
U+9F22.0	fBig5	EFB4
U+9F22.0	fGB	0-7777
U+9F22.0	fR/S	 208.4
U+9F22.0	fTotalStrokes	 17
U+9F22.0	fCangjie	HVCSH
U+9F22.0	fFourCorner	78727
U+9F22.0	fHakka	  fun3; fun1
U+9F22.0	fCantonese	fan4; fan5
U+9F22.0	fMandarin	fen2; fen3; fen4
U+9F22.0	fEnglish	a variety of mole
U+9F23.0	fUTF8	鼣
U+9F23.0	fCNS11643	2-613F
U+9F23.0	fBig5	EFB3
U+9F23.0	fGB	E-487E
U+9F23.0	fR/S	 208.4
U+9F23.0	fTotalStrokes	 17
U+9F23.0	fCangjie	HVIK
U+9F23.0	fFourCorner	23713
U+9F23.0	fCantonese	fai6
U+9F23.0	fMandarin	fei4
U+9F24.0	fUTF8	鼤
U+9F24.0	fCNS11643	2-613E
U+9F24.0	fBig5	EFB2
U+9F24.0	fGB	3-7C66
U+9F24.0	fR/S	 208.4
U+9F24.0	fTotalStrokes	 17
U+9F24.0	fCangjie	HVYK
U+9F24.0	fFourCorner	70740
U+9F24.0	fCantonese	man4
U+9F24.0	fMandarin	wen2; wen4
U+9F25.0	fUTF8	鼥
U+9F25.0	fCNS11643	2-6530
U+9F25.0	fBig5	F1E4
U+9F25.0	fGB	3-7C68
U+9F25.0	fR/S	 208.5
U+9F25.0	fTotalStrokes	 18
U+9F25.0	fCangjie	HVIKK
U+9F25.0	fFourCorner	23747
U+9F25.0	fHakka	 pat8
U+9F25.0	fCantonese	bat6
U+9F25.0	fMandarin	ba2; fei4
U+9F25.0	fEnglish	beaver
U+9F26.0	fUTF8	鼦
U+9F26.0	fCNS11643	3-5A2B
U+9F26.0	fGB	5-7A5F
U+9F26.0	fR/S	 208.5
U+9F26.0	fTotalStrokes	 18
U+9F26.0	fCangjie	HVSHR
U+9F26.0	fFourCorner	77710
U+9F26.0	fCantonese	diu1
U+9F26.0	fMandarin	diao1
U+9F27.0	fUTF8	鼧
U+9F27.0	fCNS11643	3-5A2A
U+9F27.0	fGB	3-7C6D
U+9F27.0	fR/S	  208.5
U+9F27.0	fTotalStrokes	 18
U+9F27.0	fCangjie	HVJP
U+9F27.0	fFourCorner	73710
U+9F27.0	fHakka	 to2
U+9F27.0	fCantonese	to4
U+9F27.0	fMandarin	tuo2
U+9F27.0	fEnglish	the marmot
U+9F28.0	fUTF8	鼨
U+9F28.0	fCNS11643	2-6534
U+9F28.0	fBig5	F1E8
U+9F28.0	fGB	3-7C6C
U+9F28.0	fR/S	 208.5
U+9F28.0	fTotalStrokes	 18
U+9F28.0	fCangjie	HVHEY
U+9F28.0	fFourCorner	77713
U+9F28.0	fHakka	 zhung1; zung1
U+9F28.0	fCantonese	zung1
U+9F28.0	fMandarin	zhong1; tong2
U+9F28.0	fEnglish	a rodent marked with spots like a leopard
U+9F29.0	fUTF8	鼩
U+9F29.0	fCNS11643	2-6533
U+9F29.0	fBig5	F1E7
U+9F29.0	fGB	3-7C6B
U+9F29.0	fR/S	 208.5
U+9F29.0	fTotalStrokes	 18
U+9F29.0	fCangjie	HVPR
U+9F29.0	fFourCorner	77712
U+9F29.0	fCantonese	keoi4
U+9F29.0	fMandarin	qu2
U+9F2A.0	fUTF8	鼪
U+9F2A.0	fCNS11643	2-6532
U+9F2A.0	fBig5	F1E6
U+9F2A.0	fGB	3-7C6A
U+9F2A.0	fR/S	 208.5
U+9F2A.0	fTotalStrokes	 18
U+9F2A.0	fCangjie	HVHQM
U+9F2A.0	fFourCorner	75711
U+9F2A.0	fHakka	  sen1; sin5
U+9F2A.0	fCantonese	sang1; saang1
U+9F2A.0	fMandarin	sheng1; xing4
U+9F2A.0	fEnglish	 weasel
U+9F2B.0	fUTF8	鼫
U+9F2B.0	fCNS11643	2-6531
U+9F2B.0	fBig5	F1E5
U+9F2B.0	fGB	3-7C67
U+9F2B.0	fR/S	 208.5
U+9F2B.0	fTotalStrokes	 18
U+9F2B.0	fCangjie	HVMR
U+9F2B.0	fFourCorner	71716
U+9F2B.0	fHakka	 shak8; sak8
U+9F2B.0	fCantonese	sek6
U+9F2B.0	fMandarin	shi2
U+9F2B.0	fEnglish	 marmot squirrel
U+9F2C.0	fUTF8	鼬
U+9F2C.0	fCNS11643	1-782A
U+9F2C.0	fBig5	C35E
U+9F2C.0	fGB	0-7778
U+9F2C.0	fR/S	 208.5
U+9F2C.0	fTotalStrokes	 18
U+9F2C.0	fCangjie	HVLW
U+9F2C.0	fFourCorner	75716
U+9F2C.0	fHakka	 ju5; riu5; jiu5
U+9F2C.0	fCantonese	jau6; jau2; jau4
U+9F2C.0	fMandarin	you4
U+9F2C.0	fEnglish	 weasel; Mustela itatis
U+9F2D.0	fUTF8	鼭
U+9F2D.0	fCNS11643	2-6863
U+9F2D.0	fBig5	F3F6
U+9F2D.0	fGB	E-4921
U+9F2D.0	fR/S	 208.6
U+9F2D.0	fTotalStrokes	 19
U+9F2D.0	fCangjie	HVGDI
U+9F2D.0	fFourCorner	74741
U+9F2D.0	fCantonese	si4
U+9F2D.0	fMandarin	shi2
U+9F2E.0	fUTF8	鼮
U+9F2E.0	fCNS11643	2-6B47
U+9F2E.0	fBig5	F5B9
U+9F2E.0	fGB	3-7C6E
U+9F2E.0	fR/S	 208.7
U+9F2E.0	fTotalStrokes	 20
U+9F2E.0	fCangjie	HVNKG
U+9F2E.0	fFourCorner	22714
U+9F2E.0	fCantonese	ting4
U+9F2E.0	fMandarin	ting2; ding4
U+9F2F.0	fUTF8	鼯
U+9F2F.0	fCNS11643	1-7A5E
U+9F2F.0	fBig5	C4D3
U+9F2F.0	fGB	0-7779
U+9F2F.0	fR/S	 208.7
U+9F2F.0	fTotalStrokes	 20
U+9F2F.0	fCangjie	HVMMR
U+9F2F.0	fFourCorner	71761
U+9F2F.0	fHakka	 ngi2; ngu2; ng2; ng3
U+9F2F.0	fCantonese	ng4
U+9F2F.0	fMandarin	wu2
U+9F2F.0	fEnglish	 flying squirrel
U+9F30.0	fUTF8	鼰
U+9F30.0	fCNS11643	2-6B46
U+9F30.0	fBig5	F5B8
U+9F30.0	fGB	E-4922
U+9F30.0	fR/S	 208.7
U+9F30.0	fTotalStrokes	 20
U+9F30.0	fCangjie	HVBUC
U+9F30.0	fFourCorner	76718
U+9F30.0	fCantonese	gwik1
U+9F30.0	fMandarin	nian4
U+9F31.0	fUTF8	鼱
U+9F31.0	fCNS11643	2-6D3F
U+9F31.0	fBig5	F6D1
U+9F31.0	fGB	3-7C71
U+9F31.0	fR/S	 208.8
U+9F31.0	fTotalStrokes	 21
U+9F31.0	fCangjie	HVQMB
U+9F31.0	fFourCorner	75727
U+9F31.0	fCantonese	zing1; cing1 (also)
U+9F31.0	fMandarin	jing1
U+9F32.0	fUTF8	鼲
U+9F32.0	fCNS11643	2-6E78
U+9F32.0	fBig5	F7CB
U+9F32.0	fGB	3-7C70
U+9F32.0	fR/S	 208.9
U+9F32.0	fTotalStrokes	 22
U+9F32.0	fCangjie	HVBJJ
U+9F32.0	fFourCorner	77752
U+9F32.0	fHakka	 fun2
U+9F32.0	fCantonese	wan4
U+9F32.0	fMandarin	hun2
U+9F32.0	fEnglish	marmot
U+9F33.0	fUTF8	鼳
U+9F33.0	fCNS11643	2-6E77
U+9F33.0	fBig5	F7CA
U+9F33.0	fGB	3-7C73
U+9F33.0	fR/S	 208.9
U+9F33.0	fTotalStrokes	 22
U+9F33.0	fCangjie	HVBUK
U+9F33.0	fFourCorner	76734
U+9F33.0	fCantonese	gwik1
U+9F33.0	fMandarin	ju2
U+9F34.0	fUTF8	鼴
U+9F34.0	fCNS11643	1-7C30
U+9F34.0	fBig5	C5C4
U+9F34.0	fGB	E-4923
U+9F34.0	fR/S	 208.9
U+9F34.0	fTotalStrokes	 22
U+9F34.0	fCangjie	HVSAV
U+9F34.0	fFourCorner	71714
U+9F34.0	fHakka	 jen3; ren3; jan3
U+9F34.0	fCantonese	jin2
U+9F34.0	fMandarin	yan3
U+9F34.0	fEnglish	 mole (burrowing animal)
U+9F35.0	fUTF8	鼵
U+9F35.0	fCNS11643	2-6E76
U+9F35.0	fBig5	F7C9
U+9F35.0	fGB	3-7C74
U+9F35.0	fR/S	 208.9
U+9F35.0	fTotalStrokes	 22
U+9F35.0	fCangjie	HVJCK
U+9F35.0	fFourCorner	73734
U+9F35.0	fHakka	 tut7
U+9F35.0	fCantonese	dat6
U+9F35.0	fMandarin	tu2; tu1
U+9F35.0	fEnglish	a small burrowing animal
U+9F36.0	fUTF8	鼶
U+9F36.0	fCNS11643	2-702C
U+9F36.0	fBig5	F87C
U+9F36.0	fGB	3-7C76
U+9F36.0	fR/S	 208.10
U+9F36.0	fTotalStrokes	 23
U+9F36.0	fCangjie	HVHYU
U+9F36.0	fFourCorner	72716
U+9F36.0	fCantonese	si1; tai4
U+9F36.0	fMandarin	si1; ti2
U+9F37.0	fUTF8	鼷
U+9F37.0	fCNS11643	2-702B
U+9F37.0	fBig5	F87B
U+9F37.0	fGB	0-777B
U+9F37.0	fR/S	 208.10
U+9F37.0	fTotalStrokes	 23
U+9F37.0	fCangjie	HVBVK
U+9F37.0	fFourCorner	72734
U+9F37.0	fHakka	 hi1; hi2
U+9F37.0	fCantonese	hai4
U+9F37.0	fMandarin	xi1; xi2
U+9F37.0	fEnglish	 mouse
U+9F38.0	fUTF8	鼸
U+9F38.0	fCNS11643	2-702A
U+9F38.0	fBig5	F87A
U+9F38.0	fGB	3-7C78
U+9F38.0	fR/S	 208.10
U+9F38.0	fTotalStrokes	 23
U+9F38.0	fCangjie	HVTXC
U+9F38.0	fFourCorner	74713
U+9F38.0	fHakka	 kiam3
U+9F38.0	fCantonese	jim6; him2
U+9F38.0	fMandarin	xian3; qian3
U+9F39.0	fUTF8	鼹
U+9F39.0	fCNS11643	3-606D
U+9F39.0	fGB	0-777A
U+9F39.0	fR/S	 208.10
U+9F39.0	fTotalStrokes	 23
U+9F39.0	fCangjie	HVAJV
U+9F39.0	fFourCorner	76710
U+9F39.0	fCantonese	jin2
U+9F39.0	fMandarin	yan3
U+9F39.0	fEnglish	a kind of insectivorous rodent
U+9F3A.0	fUTF8	鼺
U+9F3A.0	fCNS11643	4-6E44
U+9F3A.0	fGB	E-4924
U+9F3A.0	fR/S	  208.15
U+9F3A.0	fTotalStrokes	 28
U+9F3A.0	fCangjie	HVWWW
U+9F3A.0	fFourCorner	76716
U+9F3A.0	fCantonese	leoi4
U+9F3A.0	fMandarin	lei2
U+9F3B.0	fUTF8	鼻
U+9F3B.0	fCNS11643	1-6B7C
U+9F3B.0	fBig5	BBF3
U+9F3B.0	fGB	0-3147
U+9F3B.0	fR/S	 209.0
U+9F3B.0	fTotalStrokes	 14
U+9F3B.0	fCangjie	HUWML
U+9F3B.0	fFourCorner	26446
U+9F3B.0	fHakka	 pi5; pi6; pi3
U+9F3B.0	fCantonese	bei6
U+9F3B.0	fMandarin	bi2
U+9F3B.0	fEnglish	[1] nose [2] before any others; first [3] Kangxi radical 209
U+9F3C.0	fUTF8	鼼
U+9F3C.0	fCNS11643	4-5563
U+9F3C.0	fGB	5-7A65
U+9F3C.0	fR/S	 209.2
U+9F3C.0	fTotalStrokes	 16
U+9F3C.0	fCangjie	HLVL
U+9F3C.0	fFourCorner	22200
U+9F3C.0	fCantonese	jiu6; hiu3
U+9F3C.0	fMandarin	yao3; yao2; jiu4; qiao4
U+9F3C.0	fEnglish	curving upwards
U+9F3D.0	fUTF8	鼽
U+9F3D.0	fCNS11643	2-5C44
U+9F3D.0	fBig5	ECB8
U+9F3D.0	fGB	0-777C
U+9F3D.0	fR/S	 209.2
U+9F3D.0	fTotalStrokes	 16
U+9F3D.0	fCangjie	HLKN
U+9F3D.0	fFourCorner	24417
U+9F3D.0	fHakka	 kiu2
U+9F3D.0	fCantonese	kau4
U+9F3D.0	fMandarin	qiu2; yan3
U+9F3D.0	fEnglish	clogged nose
U+9F3E.0	fUTF8	鼾
U+9F3E.0	fCNS11643	1-7639
U+9F3E.0	fBig5	C24D
U+9F3E.0	fGB	0-777D
U+9F3E.0	fR/S	 209.3
U+9F3E.0	fTotalStrokes	 17
U+9F3E.0	fCangjie	HLMJ
U+9F3E.0	fFourCorner	21240
U+9F3E.0	fHakka	  hon5; hon2; hon1; pon5; pun5
U+9F3E.0	fCantonese	hon4; hon6
U+9F3E.0	fMandarin	han1; han4
U+9F3E.0	fEnglish	 snore
U+9F3F.0	fUTF8	鼿
U+9F3F.0	fCNS11643	4-5A64
U+9F3F.0	fGB	5-7A66
U+9F3F.0	fR/S	 209.3
U+9F3F.0	fTotalStrokes	 17; 16
U+9F3F.0	fCangjie	HLMU
U+9F3F.0	fFourCorner	21210
U+9F3F.0	fHakka	 ngut7
U+9F3F.0	fCantonese	ngat6; at6 (also)
U+9F3F.0	fMandarin	wu1; hui1
U+9F40.0	fUTF8	齀
U+9F40.0	fCNS11643	2-6864
U+9F40.0	fBig5	F3F7
U+9F40.0	fGB	E-4925
U+9F40.0	fR/S	 209.5
U+9F40.0	fTotalStrokes	 19
U+9F40.0	fCangjie	HLMVN
U+9F40.0	fFourCorner	21417
U+9F40.0	fCantonese	ngat6; at6 (also); fui1
U+9F40.0	fMandarin	wu4; wa3
U+9F41.0	fUTF8	齁
U+9F41.0	fCNS11643	2-6865
U+9F41.0	fBig5	F3F8
U+9F41.0	fGB	3-7C7A
U+9F41.0	fR/S	 209.5
U+9F41.0	fTotalStrokes	 19
U+9F41.0	fCangjie	HLPR
U+9F41.0	fFourCorner	27220
U+9F41.0	fHakka	  ku5; kieu1; geu1; heu1
U+9F41.0	fCantonese	hau1
U+9F41.0	fMandarin	hou2; hou1; ku1
U+9F41.0	fEnglish	[1] snoring; snorting [2] sweet or salty (sickeningly)
U+9F42.0	fUTF8	齂
U+9F42.0	fCNS11643	2-6E79
U+9F42.0	fBig5	F7CC
U+9F42.0	fGB	E-4926
U+9F42.0	fR/S	 209.8
U+9F42.0	fTotalStrokes	 22
U+9F42.0	fCangjie	HLLE
U+9F42.0	fFourCorner	25232
U+9F42.0	fCantonese	haai3; hei3
U+9F42.0	fMandarin	xi4; mi4; hui3; hui4
U+9F43.0	fUTF8	齃
U+9F43.0	fCNS11643	2-702D
U+9F43.0	fBig5	F87D
U+9F43.0	fGB	5-7A6C
U+9F43.0	fR/S	 209.9
U+9F43.0	fTotalStrokes	 23
U+9F43.0	fCangjie	HLAPV
U+9F43.0	fFourCorner	46427
U+9F43.0	fCantonese	aat3; ngaat3 (also)
U+9F43.0	fMandarin	ge2; he2; e4
U+9F44.0	fUTF8	齄
U+9F44.0	fCNS11643	4-6B5C
U+9F44.0	fGB	0-777E
U+9F44.0	fR/S	 209.9
U+9F44.0	fTotalStrokes	 23
U+9F44.0	fCangjie	HLDAM
U+9F44.0	fFourCorner	24210
U+9F44.0	fCantonese	zaa1
U+9F44.0	fMandarin	zha1
U+9F44.0	fEnglish	red sores on nose
U+9F45.0	fUTF8	齅
U+9F45.0	fCNS11643	3-613B
U+9F45.0	fGB	5-7A6D
U+9F45.0	fR/S	 209.10
U+9F45.0	fTotalStrokes	 24
U+9F45.0	fCangjie	HLHUK
U+9F45.0	fFourCorner	26234
U+9F45.0	fHakka	 chiu5; hiu3; cu5; chu5
U+9F45.0	fCantonese	cau3
U+9F45.0	fMandarin	xiu4
U+9F45.0	fEnglish	 smell
U+9F46.0	fUTF8	齆
U+9F46.0	fCNS11643	2-7078
U+9F46.0	fBig5	F8EA
U+9F46.0	fGB	3-7C7B
U+9F46.0	fR/S	 209.10
U+9F46.0	fTotalStrokes	 24
U+9F46.0	fCangjie	HLVVU
U+9F46.0	fFourCorner	22417
U+9F46.0	fHakka	 wung5; vung5
U+9F46.0	fCantonese	ung3; ngung3 (also)
U+9F46.0	fMandarin	weng4
U+9F46.0	fEnglish	stuffed noze
U+9F47.0	fUTF8	齇
U+9F47.0	fCNS11643	2-7155
U+9F47.0	fBig5	F966
U+9F47.0	fGB	5-7A6E
U+9F47.0	fR/S	 209.11
U+9F47.0	fTotalStrokes	 25
U+9F47.0	fCangjie	HLYPM
U+9F47.0	fFourCorner	21217
U+9F47.0	fHakka	 za1
U+9F47.0	fCantonese	zaa1
U+9F47.0	fMandarin	zha1
U+9F47.0	fEnglish	pimples; blotches on the skin
U+9F48.0	fUTF8	齈
U+9F48.0	fCNS11643	2-7229
U+9F48.0	fBig5	F9B9
U+9F48.0	fGB	E-4927
U+9F48.0	fR/S	 209.13
U+9F48.0	fTotalStrokes	 27
U+9F48.0	fCangjie	HLTWV
U+9F48.0	fFourCorner	25432
U+9F48.0	fHakka	  nung2; lung2
U+9F48.0	fCantonese	nung6; lung6 (also); nung4
U+9F48.0	fMandarin	nong2; nong4
U+9F48.0	fEnglish	a kind of nasal ailment characterised by abundance of snivel
U+9F49.0	fUTF8	齉
U+9F49.0	fCNS11643	2-7243
U+9F49.0	fBig5	F9D4
U+9F49.0	fGB	3-7C7C
U+9F49.0	fR/S	 209.22
U+9F49.0	fTotalStrokes	 36
U+9F49.0	fCangjie	HLJBV
U+9F49.0	fFourCorner	20432
U+9F49.0	fHakka	 nong2; nong3
U+9F49.0	fCantonese	nong6; long6 (also); nung6
U+9F49.0	fMandarin	nang4
U+9F49.0	fEnglish	stoppage of the nose to speak with a nasal twang
U+9F4A.0	fUTF8	齊
U+9F4A.0	fCNS11643	1-6B7D
U+9F4A.0	fBig5	BBF4
U+9F4A.0	fGB	1-466B
U+9F4A.0	fR/S	 210.0
U+9F4A.0	fTotalStrokes	 14
U+9F4A.0	fCangjie	YX
U+9F4A.0	fFourCorner	00223
U+9F4A.0	fHakka	 ce2; zai1; ci2; ts'ie2; ce3; ci3; ts'e2
U+9F4A.0	fCantonese	cai4; zai6; zi1; cai2 (also); zaai1
U+9F4A.0	fMandarin	qi2; ji4; jian3; ji1
U+9F4A.0	fEnglish	[1] even; uniform; equal; be on a level [2] the name of an ancient feudal state [3] the name of a dynasty [4] set in order [5] a Chinese family name [6] Kangxi radical 210
U+9F4A.1	fUTF8	齊
U+9F4A.1	fCNS11643	1-6B7D
U+9F4A.1	fBig5	BBF4
U+9F4A.1	fGB	1-466B
U+9F4A.1	fR/S	 210.0
U+9F4A.1	fTotalStrokes	 14
U+9F4A.1	fCangjie	YX
U+9F4A.1	fFourCorner	00223
U+9F4A.1	fMandarin	zhai1
U+9F4A.1	fEnglish	plious; respectful; chaste
U+9F4A.2	fUTF8	齊
U+9F4A.2	fCNS11643	1-6B7D
U+9F4A.2	fBig5	BBF4
U+9F4A.2	fGB	1-466B
U+9F4A.2	fR/S	 210.0
U+9F4A.2	fTotalStrokes	 14
U+9F4A.2	fCangjie	YX
U+9F4A.2	fFourCorner	00223
U+9F4A.2	fMandarin	zi1
U+9F4A.2	fEnglish	[1] the lower edge of a garment [2] same as $
U+9F4B.0	fUTF8	齋
U+9F4B.0	fCNS11643	1-763A
U+9F4B.0	fBig5	C24E
U+9F4B.0	fGB	1-552B
U+9F4B.0	fR/S	 210.3
U+9F4B.0	fTotalStrokes	 17
U+9F4B.0	fCangjie	YXF
U+9F4B.0	fFourCorner	00223
U+9F4B.0	fHakka	  zai1; sai2
U+9F4B.0	fCantonese	zaai1; zai1
U+9F4B.0	fMandarin	zhai1; zi1
U+9F4B.0	fEnglish	[1] plious; respectful; chaste; pure [2] abstain from meat; fast [3] purify oneself [4] provide Buddhis monks with meals [5] room for study; study; school [6] vegetarian diet; vegetarian meal
U+9F4B.1	fUTF8	齋
U+9F4B.1	fCNS11643	1-763A
U+9F4B.1	fBig5	C24E
U+9F4B.1	fGB	1-552B
U+9F4B.1	fR/S	 210.3
U+9F4B.1	fTotalStrokes	 17
U+9F4B.1	fCangjie	YXF
U+9F4B.1	fFourCorner	00223
U+9F4B.1	fEnglish	 human tapeworm
U+9F4C.0	fUTF8	齌
U+9F4C.0	fCNS11643	2-6535
U+9F4C.0	fBig5	F1E9
U+9F4C.0	fGB	3-6623
U+9F4C.0	fR/S	 210.4
U+9F4C.0	fTotalStrokes	 18
U+9F4C.0	fCangjie	YXF
U+9F4C.0	fFourCorner	00223
U+9F4C.0	fCantonese	zai1 (also); zai6; cai1 (also)
U+9F4C.0	fMandarin	ji4; qi1
U+9F4D.0	fUTF8	齍
U+9F4D.0	fCNS11643	2-6866
U+9F4D.0	fBig5	F3F9
U+9F4D.0	fGB	5-6534
U+9F4D.0	fR/S	 210.5
U+9F4D.0	fTotalStrokes	 19
U+9F4D.0	fCangjie	YXBT
U+9F4D.0	fFourCorner	00223
U+9F4D.0	fHakka	 zih1; zi2; di2
U+9F4D.0	fCantonese	zi1
U+9F4D.0	fMandarin	zi1; ji1; ji4
U+9F4D.0	fEnglish	sacrificial dishes
U+9F4E.0	fUTF8	齎
U+9F4E.0	fCNS11643	2-6D40
U+9F4E.0	fBig5	F6D2
U+9F4E.0	fGB	E-4928
U+9F4E.0	fR/S	 210.7
U+9F4E.0	fTotalStrokes	 21
U+9F4E.0	fCangjie	YXBUC
U+9F4E.0	fFourCorner	00223
U+9F4E.0	fHakka	 zi1; zii1
U+9F4E.0	fCantonese	zai1; cai4
U+9F4E.0	fMandarin	ji1; qi2
U+9F4E.0	fEnglish	[1] present; offer [2] harbour; have in one's mind; entertain
U+9F4E.1	fUTF8	齎
U+9F4E.1	fCNS11643	2-6D40
U+9F4E.1	fBig5	F6D2
U+9F4E.1	fGB	E-4928
U+9F4E.1	fR/S	 210.7
U+9F4E.1	fTotalStrokes	 21
U+9F4E.1	fCangjie	YXBUC
U+9F4E.1	fFourCorner	00223
U+9F4E.1	fHakka	 zih1
U+9F4E.1	fCantonese	zi1
U+9F4E.1	fMandarin	zi1; zi4
U+9F4E.1	fEnglish	same as &#x8CC7;
U+9F4F.0	fUTF8	齏
U+9F4F.0	fCNS11643	2-702E
U+9F4F.0	fBig5	F87E
U+9F4F.0	fGB	1-6C34
U+9F4F.0	fR/S	 210.9
U+9F4F.0	fTotalStrokes	 23
U+9F4F.0	fCangjie	YXLSM
U+9F4F.0	fFourCorner	00223
U+9F4F.0	fHakka	  zi1
U+9F4F.0	fCantonese	zai1
U+9F4F.0	fMandarin	ji1
U+9F4F.0	fEnglish	[1] break or smash into pieces; pulverized [2] seasonings in powdered form
U+9F50.0	fUTF8	齐
U+9F50.0	fCNS11643	3-2370
U+9F50.0	fGB	0-466B
U+9F50.0	fR/S	 210.0
U+9F50.0	fTotalStrokes	 6
U+9F50.0	fCangjie	YKLL
U+9F50.0	fFourCorner	00220
U+9F50.0	fHakka	 ce2; zai1; ci2
U+9F50.0	fCantonese	cai4; cai2 (also); zai6; zi1; zaai1
U+9F50.0	fMandarin	qi2; ji4; jian3; zi1; ji1; zhai1
U+9F50.0	fEnglish	simplified form of &#x9F4A;
U+9F51.0	fUTF8	齑
U+9F51.0	fGB	0-6C34
U+9F51.0	fR/S	 210.9
U+9F51.0	fTotalStrokes	 15
U+9F51.0	fCangjie	YKLLL
U+9F51.0	fHakka	 zi1
U+9F51.0	fCantonese	zai1
U+9F51.0	fMandarin	ji1
U+9F51.0	fEnglish	simplified form of &#x9F4F;
U+9F52.0	fUTF8	齒
U+9F52.0	fCNS11643	1-702F
U+9F52.0	fBig5	BEA6
U+9F52.0	fGB	1-335D
U+9F52.0	fR/S	 211.0
U+9F52.0	fTotalStrokes	 15
U+9F52.0	fCangjie	YMUOO
U+9F52.0	fFourCorner	21772
U+9F52.0	fHakka	 ci3; chi3; cih3; chii3
U+9F52.0	fCantonese	ci2
U+9F52.0	fMandarin	chi3
U+9F52.0	fEnglish	[1] teeth [2] age [3] speak of; mention [4] a tooth-like part of anything [5] Kangxi radical 211
U+9F53.0	fUTF8	齓
U+9F53.0	fCNS11643	3-543E
U+9F53.0	fGB	5-737D
U+9F53.0	fR/S	 211.1
U+9F53.0	fTotalStrokes	 16
U+9F53.0	fCangjie	YUU
U+9F53.0	fFourCorner	22710
U+9F53.0	fCantonese	can3
U+9F53.0	fMandarin	chen4
U+9F54.0	fUTF8	齔
U+9F54.0	fCNS11643	2-6141
U+9F54.0	fBig5	EFB5
U+9F54.0	fGB	1-7633
U+9F54.0	fR/S	 211.2
U+9F54.0	fTotalStrokes	 17
U+9F54.0	fCangjie	YUP
U+9F54.0	fFourCorner	21710
U+9F54.0	fHakka	  cen5
U+9F54.0	fCantonese	can3
U+9F54.0	fMandarin	chen4
U+9F54.0	fEnglish	[1] lose baby teeth and get adult teeth; have milk teeth replaced with permanent teeth [2] children
U+9F55.0	fUTF8	齕
U+9F55.0	fCNS11643	2-6536
U+9F55.0	fBig5	F1EA
U+9F55.0	fGB	3-7460
U+9F55.0	fR/S	 211.3
U+9F55.0	fTotalStrokes	 18
U+9F55.0	fCangjie	YUON
U+9F55.0	fFourCorner	28717
U+9F55.0	fHakka	  ngat7;  ngiet7
U+9F55.0	fCantonese	hat6; ngaat6 (also); aat6 (also)
U+9F55.0	fMandarin	he2
U+9F55.0	fEnglish	 gnaw; bite; nibble; munch
U+9F56.0	fUTF8	齖
U+9F56.0	fCNS11643	2-6867
U+9F56.0	fBig5	F3FA
U+9F56.0	fGB	3-7462
U+9F56.0	fR/S	 211.4; 211.5
U+9F56.0	fTotalStrokes	 19
U+9F56.0	fCangjie	YUMVH
U+9F56.0	fFourCorner	21740
U+9F56.0	fHakka	  nga2
U+9F56.0	fCantonese	ngaa4; aa4 (also); ngaa6
U+9F56.0	fMandarin	ya2; ya4
U+9F56.0	fEnglish	uneven teeth
U+9F57.0	fUTF8	齗
U+9F57.0	fCNS11643	2-6868
U+9F57.0	fBig5	F3FB
U+9F57.0	fGB	3-7463
U+9F57.0	fR/S	 211.4
U+9F57.0	fTotalStrokes	 19
U+9F57.0	fCangjie	YUHML
U+9F57.0	fFourCorner	22721
U+9F57.0	fHakka	 ngin1; ngiun2
U+9F57.0	fCantonese	ngan4; an4 (also)
U+9F57.0	fMandarin	ken3; kun3; yin2
U+9F57.0	fEnglish	[1] gums(of the teeth) [2] dispute
U+9F58.0	fUTF8	齘
U+9F58.0	fCNS11643	2-6869
U+9F58.0	fBig5	F3FC
U+9F58.0	fGB	3-7464
U+9F58.0	fR/S	 211.4
U+9F58.0	fTotalStrokes	 19
U+9F58.0	fCangjie	YUOLL
U+9F58.0	fFourCorner	28720
U+9F58.0	fHakka	  hai5
U+9F58.0	fCantonese	haai6
U+9F58.0	fMandarin	xie4
U+9F58.0	fEnglish	 gnash the teeth in anger; scaly as a coat of mail
U+9F59.0	fUTF8	齙
U+9F59.0	fCNS11643	2-6B4C
U+9F59.0	fBig5	F5BE
U+9F59.0	fGB	1-7635
U+9F59.0	fR/S	 211.5
U+9F59.0	fTotalStrokes	 20
U+9F59.0	fCangjie	YUPRU
U+9F59.0	fFourCorner	27712
U+9F59.0	fHakka	  pau2; pau3; bau5
U+9F59.0	fCantonese	baau6; paau4; baau1
U+9F59.0	fMandarin	pao2; bao1
U+9F59.0	fEnglish	projecting teeth
U+9F5A.0	fUTF8	齚
U+9F5A.0	fCNS11643	3-5E24
U+9F5A.0	fGB	3-7469
U+9F5A.0	fR/S	 211.5
U+9F5A.0	fTotalStrokes	 20
U+9F5A.0	fCangjie	YUOS
U+9F5A.0	fFourCorner	28710
U+9F5A.0	fCantonese	zaak3; zaak6 (also)
U+9F5A.0	fMandarin	cuo4; ze2
U+9F5A.0	fEnglish	 chew
U+9F5B.0	fUTF8	齛
U+9F5B.0	fCNS11643	2-6B48
U+9F5B.0	fBig5	F5BA
U+9F5B.0	fGB	5-7425
U+9F5B.0	fR/S	 211.5
U+9F5B.0	fTotalStrokes	 20
U+9F5B.0	fCangjie	YUPT
U+9F5B.0	fFourCorner	24717
U+9F5B.0	fCantonese	sit3
U+9F5B.0	fMandarin	shi4; ye4; xie4
U+9F5C.0	fUTF8	齜
U+9F5C.0	fCNS11643	1-7B54
U+9F5C.0	fBig5	C568
U+9F5C.0	fGB	1-7637
U+9F5C.0	fR/S	 211.5
U+9F5C.0	fTotalStrokes	 20
U+9F5C.0	fCangjie	YUYMP
U+9F5C.0	fFourCorner	21710
U+9F5C.0	fHakka	 zih1; cai2
U+9F5C.0	fCantonese	zi1; caai4; ci1
U+9F5C.0	fMandarin	zi1; cha1; chai2; xia4
U+9F5C.0	fEnglish	[1] t open the mouth and show the teeth [2] uneven teeth; crooked teeth
U+9F5D.0	fUTF8	齝
U+9F5D.0	fCNS11643	2-6B4B
U+9F5D.0	fBig5	F5BD
U+9F5D.0	fGB	3-746C
U+9F5D.0	fR/S	 211.5
U+9F5D.0	fTotalStrokes	 20
U+9F5D.0	fCangjie	YUIR
U+9F5D.0	fFourCorner	13760
U+9F5D.0	fHakka	 chi1; cih1; chii1
U+9F5D.0	fCantonese	ci1; si1
U+9F5D.0	fMandarin	chi1; shi1
U+9F5D.0	fEnglish	chew the cud
U+9F5E.0	fUTF8	齞
U+9F5E.0	fCNS11643	2-6B4A
U+9F5E.0	fBig5	F5BC
U+9F5E.0	fGB	3-7468
U+9F5E.0	fR/S	 211.5
U+9F5E.0	fTotalStrokes	 20
U+9F5E.0	fCangjie	YURC
U+9F5E.0	fFourCorner	26780
U+9F5E.0	fCantonese	jin6
U+9F5E.0	fMandarin	yan4; nian3; nian4; yan3
U+9F5E.0	fEnglish	 display the teeth
U+9F5F.0	fUTF8	齟
U+9F5F.0	fCNS11643	1-7A5F
U+9F5F.0	fBig5	C4D4
U+9F5F.0	fGB	1-7634
U+9F5F.0	fR/S	 211.5
U+9F5F.0	fTotalStrokes	 20
U+9F5F.0	fCangjie	YUBM
U+9F5F.0	fFourCorner	27710
U+9F5F.0	fHakka	 zu3; zii1; zih1
U+9F5F.0	fCantonese	zeoi2; zaa1; zeoi6
U+9F5F.0	fMandarin	ju3; zha1
U+9F5F.0	fEnglish	irregular teeth
U+9F60.0	fUTF8	齠
U+9F60.0	fCNS11643	2-6B49
U+9F60.0	fBig5	F5BB
U+9F60.0	fGB	1-7636
U+9F60.0	fR/S	 211.5
U+9F60.0	fTotalStrokes	 20
U+9F60.0	fCangjie	YUSHR
U+9F60.0	fFourCorner	27762
U+9F60.0	fHakka	 tiau1; tiau2
U+9F60.0	fCantonese	tiu4
U+9F60.0	fMandarin	tiao2
U+9F60.0	fEnglish	 shed the milk teeth; lose baby teeth and get adult teeth
U+9F61.0	fUTF8	齡
U+9F61.0	fCNS11643	1-7A61
U+9F61.0	fBig5	C4D6
U+9F61.0	fGB	1-4164
U+9F61.0	fR/S	 211.5
U+9F61.0	fTotalStrokes	 20
U+9F61.0	fCangjie	YUOII
U+9F61.0	fFourCorner	28737
U+9F61.0	fHakka	 lin2; lin3
U+9F61.0	fCantonese	ling4; ling2 (also)
U+9F61.0	fMandarin	ling2
U+9F61.0	fEnglish	age; years
U+9F62.0	fUTF8	齢
U+9F62.0	fGB	E-4929
U+9F62.0	fR/S	 211.5
U+9F62.0	fTotalStrokes	 20
U+9F62.0	fCangjie	YUOII
U+9F62.0	fHakka	 lin2
U+9F62.0	fCantonese	ling4
U+9F62.0	fMandarin	ling2
U+9F62.0	fEnglish	 age years
U+9F63.0	fUTF8	齣
U+9F63.0	fCNS11643	1-7A60
U+9F63.0	fBig5	C4D5
U+9F63.0	fGB	1-782A
U+9F63.0	fR/S	 211.5
U+9F63.0	fTotalStrokes	 20
U+9F63.0	fCangjie	YUPR
U+9F63.0	fFourCorner	27720
U+9F63.0	fHakka	 cut7; ch'ut7; ch'ak7; cak7; chit7
U+9F63.0	fCantonese	ceot1
U+9F63.0	fMandarin	chu1; chi3
U+9F63.0	fEnglish	[1] chapter (of old-style novels) [2] a numerical classifier for plays
U+9F64.0	fUTF8	齤
U+9F64.0	fCNS11643	2-6D42
U+9F64.0	fBig5	F6D4
U+9F64.0	fGB	3-7471
U+9F64.0	fR/S	 211.6
U+9F64.0	fTotalStrokes	 21
U+9F64.0	fCangjie	FQYMU
U+9F64.0	fFourCorner	90772
U+9F64.0	fCantonese	kyun4
U+9F64.0	fMandarin	quan2
U+9F65.0	fUTF8	齥
U+9F65.0	fCNS11643	2-6D41
U+9F65.0	fBig5	F6D3
U+9F65.0	fGB	E-492A
U+9F65.0	fR/S	 211.6
U+9F65.0	fTotalStrokes	 21
U+9F65.0	fCangjie	YULWP
U+9F65.0	fFourCorner	25706
U+9F65.0	fCantonese	sit3
U+9F65.0	fMandarin	xie4; shi4; ye4
U+9F66.0	fUTF8	齦
U+9F66.0	fCNS11643	1-7B55
U+9F66.0	fBig5	C569
U+9F66.0	fGB	1-7638
U+9F66.0	fR/S	 211.6
U+9F66.0	fTotalStrokes	 21
U+9F66.0	fCangjie	YUAV
U+9F66.0	fFourCorner	27732
U+9F66.0	fHakka	 ken3; ngiun2; k'e5
U+9F66.0	fCantonese	ngan4; hang2; an4 (also); han2
U+9F66.0	fMandarin	ken3; yin2; qian3
U+9F66.0	fEnglish	gums (of the teeth)
U+9F67.0	fUTF8	齧
U+9F67.0	fCNS11643	1-7B56
U+9F67.0	fBig5	C56A
U+9F67.0	fGB	E-492B
U+9F67.0	fR/S	 211.6
U+9F67.0	fTotalStrokes	 21
U+9F67.0	fCangjie	QHYMU
U+9F67.0	fFourCorner	57772
U+9F67.0	fHakka	 ngat7;  ngat8;  ngiet7
U+9F67.0	fCantonese	ngit6; jit6 (also); ngaat6; it6 (also); aat6 (also); jit3
U+9F67.0	fMandarin	nie4
U+9F67.0	fEnglish	 bite; gnaw
U+9F68.0	fUTF8	齨
U+9F68.0	fCNS11643	4-6860
U+9F68.0	fGB	5-742F
U+9F68.0	fR/S	  211.6
U+9F68.0	fTotalStrokes	 21
U+9F68.0	fCangjie	YUHX
U+9F68.0	fFourCorner	27770
U+9F68.0	fCantonese	kau5
U+9F68.0	fMandarin	jiu4
U+9F69.0	fUTF8	齩
U+9F69.0	fCNS11643	3-5F3E
U+9F69.0	fGB	E-492C
U+9F69.0	fR/S	 211.6
U+9F69.0	fTotalStrokes	 21
U+9F69.0	fCangjie	YUYCK
U+9F69.0	fFourCorner	20740
U+9F69.0	fHakka	  ngau1; ngau5; ngau3
U+9F69.0	fCantonese	ngaau5; aau5 (also)
U+9F69.0	fMandarin	yao3; jiao1; yao1
U+9F69.0	fEnglish	 chew; bite
U+9F6A.0	fUTF8	齪
U+9F6A.0	fCNS11643	1-7C32
U+9F6A.0	fBig5	C5C6
U+9F6A.0	fGB	1-763A
U+9F6A.0	fR/S	 211.7
U+9F6A.0	fTotalStrokes	 22
U+9F6A.0	fCangjie	YURYO
U+9F6A.0	fFourCorner	26781
U+9F6A.0	fHakka	  cok7; cuk7; zo1; zau1
U+9F6A.0	fCantonese	cuk1
U+9F6A.0	fMandarin	chuo4; chu4
U+9F6A.0	fEnglish	[1] correct form of &#x [2] narrow;small [3] dirty
U+9F6B.0	fUTF8	齫
U+9F6B.0	fCNS11643	2-6E7A
U+9F6B.0	fBig5	F7CD
U+9F6B.0	fGB	5-7431
U+9F6B.0	fR/S	 211.7
U+9F6B.0	fTotalStrokes	 22
U+9F6B.0	fCangjie	YUWD
U+9F6B.0	fFourCorner	26700
U+9F6B.0	fCantonese	wan5; kwan2
U+9F6B.0	fMandarin	kun3; nen3
U+9F6C.0	fUTF8	齬
U+9F6C.0	fCNS11643	1-7C31
U+9F6C.0	fBig5	C5C5
U+9F6C.0	fGB	1-7639
U+9F6C.0	fR/S	 211.7
U+9F6C.0	fTotalStrokes	 22
U+9F6C.0	fCangjie	YUMMR
U+9F6C.0	fFourCorner	21761
U+9F6C.0	fHakka	 ngi3; ngu2; ni3
U+9F6C.0	fCantonese	jyu5
U+9F6C.0	fMandarin	yu3; nuu1; wu2
U+9F6C.0	fEnglish	[1] correct form of &#x [2] uneven teeth [3] disagree;have discord
U+9F6D.0	fUTF8	齭
U+9F6D.0	fCNS11643	4-6B61
U+9F6D.0	fGB	5-7437
U+9F6D.0	fR/S	  211.8
U+9F6D.0	fTotalStrokes	 23
U+9F6D.0	fCangjie	YUHSL
U+9F6D.0	fFourCorner	22720
U+9F6D.0	fCantonese	co2
U+9F6D.0	fMandarin	chu3
U+9F6D.0	fEnglish	toothache
U+9F6E.0	fUTF8	齮
U+9F6E.0	fCNS11643	2-7031
U+9F6E.0	fBig5	F8A3
U+9F6E.0	fGB	3-7476
U+9F6E.0	fR/S	 211.8
U+9F6E.0	fTotalStrokes	 23
U+9F6E.0	fCangjie	YUKMR
U+9F6E.0	fFourCorner	24721
U+9F6E.0	fHakka	 gi3
U+9F6E.0	fCantonese	ji2; ngai5
U+9F6E.0	fMandarin	yi3; qi3; qi1
U+9F6E.0	fEnglish	bite
U+9F6F.0	fUTF8	齯
U+9F6F.0	fCNS11643	2-7032
U+9F6F.0	fBig5	F8A4
U+9F6F.0	fGB	3-7477
U+9F6F.0	fR/S	 211.8
U+9F6F.0	fTotalStrokes	 23
U+9F6F.0	fCangjie	YUHXU
U+9F6F.0	fFourCorner	27717
U+9F6F.0	fHakka	  ngi2
U+9F6F.0	fCantonese	ngai4; ai4 (also)
U+9F6F.0	fMandarin	ni2
U+9F6F.0	fEnglish	teeth grown in old age
U+9F70.0	fUTF8	齰
U+9F70.0	fCNS11643	2-7030
U+9F70.0	fBig5	F8A2
U+9F70.0	fGB	5-7434
U+9F70.0	fR/S	 211.8
U+9F70.0	fTotalStrokes	 23
U+9F70.0	fCangjie	YUTA
U+9F70.0	fFourCorner	24661
U+9F70.0	fCantonese	zaak3; zaak6 (also); zaa6
U+9F70.0	fMandarin	cuo4; zha4; ze2
U+9F70.0	fEnglish	 bite
U+9F71.0	fUTF8	齱
U+9F71.0	fCNS11643	2-702F
U+9F71.0	fBig5	F8A1
U+9F71.0	fGB	3-7475
U+9F71.0	fR/S	 211.8
U+9F71.0	fTotalStrokes	 23
U+9F71.0	fCangjie	YUSJE
U+9F71.0	fFourCorner	27740
U+9F71.0	fCantonese	cuk1; zau1
U+9F71.0	fMandarin	zou1; zou3; chuo4; cu4
U+9F72.0	fUTF8	齲
U+9F72.0	fCNS11643	1-7D21
U+9F72.0	fBig5	C654
U+9F72.0	fGB	1-4823
U+9F72.0	fR/S	 211.9
U+9F72.0	fTotalStrokes	 24
U+9F72.0	fCangjie	YUHLB
U+9F72.0	fFourCorner	22727
U+9F72.0	fHakka	 gi3
U+9F72.0	fCantonese	geoi2
U+9F72.0	fMandarin	qu3
U+9F72.0	fEnglish	tooth decay
U+9F73.0	fUTF8	齳
U+9F73.0	fCNS11643	4-6C64
U+9F73.0	fGB	3-7472
U+9F73.0	fR/S	  211.9
U+9F73.0	fTotalStrokes	 24
U+9F73.0	fCangjie	YUBJJ
U+9F73.0	fFourCorner	27750
U+9F73.0	fCantonese	wan5
U+9F73.0	fMandarin	nen3; yun3; kun3
U+9F73.0	fEnglish	toothless
U+9F74.0	fUTF8	齴
U+9F74.0	fCNS11643	2-7079
U+9F74.0	fBig5	F8EB
U+9F74.0	fGB	3-747B
U+9F74.0	fR/S	 211.9
U+9F74.0	fTotalStrokes	 24
U+9F74.0	fCangjie	YUYHH
U+9F74.0	fFourCorner	20722
U+9F74.0	fCantonese	jin6; ngaan5; aan5 (also)
U+9F74.0	fMandarin	xian3; yan3
U+9F75.0	fUTF8	齵
U+9F75.0	fCNS11643	2-707A
U+9F75.0	fBig5	F8EC
U+9F75.0	fGB	3-7479
U+9F75.0	fR/S	 211.9
U+9F75.0	fTotalStrokes	 24
U+9F75.0	fCangjie	YUWLB
U+9F75.0	fFourCorner	26727
U+9F75.0	fHakka	 ngau2;  ngieu1;  ngi2
U+9F75.0	fCantonese	jyu4
U+9F75.0	fMandarin	ou2; yu2
U+9F75.0	fEnglish	uneven (teeth)
U+9F76.0	fUTF8	齶
U+9F76.0	fCNS11643	2-707B
U+9F76.0	fBig5	F8ED
U+9F76.0	fGB	E-492D
U+9F76.0	fR/S	 211.9
U+9F76.0	fTotalStrokes	 24
U+9F76.0	fCangjie	YURRS
U+9F76.0	fFourCorner	26727
U+9F76.0	fHakka	  ngok8
U+9F76.0	fCantonese	ngok6; ok6 (also)
U+9F76.0	fMandarin	e4
U+9F76.0	fEnglish	 palate; roof of the mouth
U+9F77.0	fUTF8	齷
U+9F77.0	fCNS11643	1-7C7E
U+9F77.0	fBig5	C653
U+9F77.0	fGB	1-763B
U+9F77.0	fR/S	 211.9
U+9F77.0	fTotalStrokes	 24
U+9F77.0	fCangjie	YUSMG
U+9F77.0	fFourCorner	27714
U+9F77.0	fHakka	 wok7; o1; vok7
U+9F77.0	fCantonese	ak1; ngak1 (also); aak1 (also); ngaak1 (also)
U+9F77.0	fMandarin	wo4
U+9F77.0	fEnglish	[1] [CC@] [2] narrow; small [3] dirty
U+9F78.0	fUTF8	齸
U+9F78.0	fCNS11643	2-7156
U+9F78.0	fBig5	F967
U+9F78.0	fGB	5-7440
U+9F78.0	fR/S	 211.10
U+9F78.0	fTotalStrokes	 25
U+9F78.0	fCangjie	YUTCT
U+9F78.0	fFourCorner	28717
U+9F78.0	fHakka	 jit7; rit7
U+9F78.0	fCantonese	jik1; jat6
U+9F78.0	fMandarin	yi4
U+9F78.0	fEnglish	[1] the first stomach of a deer [2] chew the cud
U+9F79.0	fUTF8	齹
U+9F79.0	fCNS11643	2-7159
U+9F79.0	fBig5	F96A
U+9F79.0	fGB	E-492E
U+9F79.0	fR/S	 211.10
U+9F79.0	fTotalStrokes	 25
U+9F79.0	fCangjie	TMYMU
U+9F79.0	fFourCorner	80772
U+9F79.0	fCantonese	ci1; co4
U+9F79.0	fMandarin	chuo1; chi1
U+9F7A.0	fUTF8	齺
U+9F7A.0	fCNS11643	2-7158
U+9F7A.0	fBig5	F969
U+9F7A.0	fGB	3-746A
U+9F7A.0	fR/S	 211.10
U+9F7A.0	fTotalStrokes	 25
U+9F7A.0	fCangjie	YUPUU
U+9F7A.0	fFourCorner	27727
U+9F7A.0	fCantonese	zau1
U+9F7A.0	fMandarin	zou1
U+9F7B.0	fUTF8	齻
U+9F7B.0	fCNS11643	2-7157
U+9F7B.0	fBig5	F968
U+9F7B.0	fGB	3-747C
U+9F7B.0	fR/S	 211.10
U+9F7B.0	fTotalStrokes	 25
U+9F7B.0	fCangjie	YUJBC
U+9F7B.0	fFourCorner	24781
U+9F7B.0	fHakka	  dien1
U+9F7B.0	fCantonese	din1
U+9F7B.0	fMandarin	dian1
U+9F7B.0	fEnglish	 cut one's wisdom teeth
U+9F7C.0	fUTF8	齼
U+9F7C.0	fCNS11643	3-6233
U+9F7C.0	fGB	3-7521
U+9F7C.0	fR/S	  211.13
U+9F7C.0	fTotalStrokes	 28
U+9F7C.0	fCangjie	YUDDO
U+9F7C.0	fFourCorner	24780
U+9F7C.0	fHakka	 cu3; cih3
U+9F7C.0	fCantonese	co2
U+9F7C.0	fMandarin	chu3
U+9F7C.0	fEnglish	the teeth set on edge by eating something sour
U+9F7D.0	fUTF8	齽
U+9F7D.0	fCNS11643	4-6E45
U+9F7D.0	fGB	3-747E
U+9F7D.0	fR/S	  211.13
U+9F7D.0	fTotalStrokes	 28
U+9F7D.0	fCangjie	YUDDF
U+9F7D.0	fFourCorner	24790
U+9F7D.0	fCantonese	gam6
U+9F7D.0	fMandarin	jin4
U+9F7E.0	fUTF8	齾
U+9F7E.0	fCNS11643	2-7242
U+9F7E.0	fBig5	F9D3
U+9F7E.0	fGB	3-747D
U+9F7E.0	fR/S	 211.20
U+9F7E.0	fTotalStrokes	 35
U+9F7E.0	fCangjie	YKYMU
U+9F7E.0	fFourCorner	23772
U+9F7E.0	fHakka	 ngiet7;  ngat7
U+9F7E.0	fCantonese	ngaa3 (also); ngaat3 (also); aat3; aa3 (also)
U+9F7E.0	fMandarin	ya4
U+9F7E.0	fEnglish	fragments; shreds
U+9F7F.0	fUTF8	齿
U+9F7F.0	fGB	0-335D
U+9F7F.0	fR/S	 211.0
U+9F7F.0	fTotalStrokes	 8
U+9F7F.0	fCangjie	YMUO
U+9F7F.0	fHakka	 chi3
U+9F7F.0	fCantonese	ci2
U+9F7F.0	fMandarin	chi3
U+9F7F.0	fEnglish	simplified form of &#x9F52;
U+9F80.0	fUTF8	龀
U+9F80.0	fGB	0-7633
U+9F80.0	fR/S	 211.2
U+9F80.0	fTotalStrokes	 10
U+9F80.0	fCangjie	YUP
U+9F80.0	fHakka	 cen5
U+9F80.0	fCantonese	can3
U+9F80.0	fMandarin	chen4
U+9F80.0	fEnglish	simplified form of &#x9F54;
U+9F81.0	fUTF8	龁
U+9F81.0	fGB	8-7E7B
U+9F81.0	fR/S	 211.3
U+9F81.0	fTotalStrokes	 11
U+9F81.0	fCangjie	YUON
U+9F81.0	fCantonese	hat6; ngaat6 (also); aat6 (also)
U+9F81.0	fMandarin	he2
U+9F81.0	fEnglish	gnaw, bite, nibble
U+9F82.0	fUTF8	龂
U+9F82.0	fGB	8-7E7C
U+9F82.0	fR/S	 211.4
U+9F82.0	fTotalStrokes	 12
U+9F82.0	fCangjie	YUHML
U+9F82.0	fCantonese	ngan4; an4 (also)
U+9F82.0	fMandarin	ken3; yin2
U+9F82.0	fEnglish	gums (of the teeth); to dispute
U+9F83.0	fUTF8	龃
U+9F83.0	fGB	0-7634
U+9F83.0	fR/S	 211.5
U+9F83.0	fTotalStrokes	 13
U+9F83.0	fCangjie	YUBM
U+9F83.0	fHakka	 zih1
U+9F83.0	fCantonese	zeoi2; zaa1
U+9F83.0	fMandarin	ju3
U+9F83.0	fEnglish	simplified form of &#x9F5F;
U+9F84.0	fUTF8	龄
U+9F84.0	fGB	0-4164
U+9F84.0	fR/S	 211.5
U+9F84.0	fTotalStrokes	 13
U+9F84.0	fCangjie	YUOII
U+9F84.0	fCantonese	ling4; ling2 (also)
U+9F84.0	fMandarin	ling2
U+9F84.0	fEnglish	simplified form of &#x9F61;
U+9F85.0	fUTF8	龅
U+9F85.0	fGB	0-7635
U+9F85.0	fR/S	 211.5
U+9F85.0	fTotalStrokes	 13
U+9F85.0	fCangjie	YUPRU
U+9F85.0	fHakka	 pau2
U+9F85.0	fCantonese	baau6; baau1
U+9F85.0	fMandarin	pao2; bao1
U+9F85.0	fEnglish	simplified form of &#x9F59;
U+9F86.0	fUTF8	龆
U+9F86.0	fGB	0-7636
U+9F86.0	fR/S	 211.5
U+9F86.0	fTotalStrokes	 13
U+9F86.0	fCangjie	YUSHR
U+9F86.0	fHakka	 tiau2; tiau1
U+9F86.0	fCantonese	tiu4
U+9F86.0	fMandarin	tiao2
U+9F86.0	fEnglish	simplified form of &#x9F60;
U+9F87.0	fUTF8	龇
U+9F87.0	fGB	0-7637
U+9F87.0	fR/S	 211.6
U+9F87.0	fTotalStrokes	 14
U+9F87.0	fCangjie	YUYMP
U+9F87.0	fHakka	 zih1
U+9F87.0	fCantonese	zi1; caai4
U+9F87.0	fMandarin	zi1; cha1; xia4
U+9F87.0	fEnglish	simplified form of &#x9F5C;
U+9F88.0	fUTF8	龈
U+9F88.0	fGB	0-7638
U+9F88.0	fR/S	 211.6
U+9F88.0	fTotalStrokes	 14
U+9F88.0	fCangjie	YUAV
U+9F88.0	fHakka	 ken3; ngiun2
U+9F88.0	fCantonese	ngan4; an4 (also); hang2; han2
U+9F88.0	fMandarin	ken3; yin2
U+9F88.0	fEnglish	simplified form of &#x9F66;
U+9F89.0	fUTF8	龉
U+9F89.0	fGB	0-7639
U+9F89.0	fR/S	 211.7
U+9F89.0	fTotalStrokes	 15
U+9F89.0	fCangjie	YUMMR
U+9F89.0	fHakka	 ngi3
U+9F89.0	fCantonese	jyu5
U+9F89.0	fMandarin	yu3; nuu1
U+9F89.0	fEnglish	simplified form of &#x9F6C;
U+9F8A.0	fUTF8	龊
U+9F8A.0	fGB	0-763A
U+9F8A.0	fR/S	 211.7
U+9F8A.0	fTotalStrokes	 15
U+9F8A.0	fCangjie	YURYO
U+9F8A.0	fHakka	 cok7; cuk7; zau1
U+9F8A.0	fCantonese	cuk1
U+9F8A.0	fMandarin	chuo4; chu4
U+9F8A.0	fEnglish	simplified form of &#x9F6A;
U+9F8B.0	fUTF8	龋
U+9F8B.0	fGB	0-4823
U+9F8B.0	fR/S	 211.9
U+9F8B.0	fTotalStrokes	 17
U+9F8B.0	fCangjie	YUHLB
U+9F8B.0	fHakka	 gi3
U+9F8B.0	fCantonese	geoi2
U+9F8B.0	fMandarin	qu3
U+9F8B.0	fEnglish	simplified form of &#x9F72;
U+9F8C.0	fUTF8	龌
U+9F8C.0	fGB	0-763B
U+9F8C.0	fR/S	 211.9
U+9F8C.0	fTotalStrokes	 17
U+9F8C.0	fCangjie	YUSMG
U+9F8C.0	fHakka	 vok7
U+9F8C.0	fCantonese	ak1; aak1 (also); ngak1 (also); ngaak1 (also)
U+9F8C.0	fMandarin	wo4
U+9F8C.0	fEnglish	simplified form of &#x9F77;
U+9F8D.0	fUTF8	龍
U+9F8D.0	fCNS11643	1-733E
U+9F8D.0	fBig5	C073
U+9F8D.0	fGB	1-417A
U+9F8D.0	fR/S	 212.0
U+9F8D.0	fTotalStrokes	 16
U+9F8D.0	fCangjie	YBYSP
U+9F8D.0	fFourCorner	01211
U+9F8D.0	fHakka	  liung2; lung2; liung3
U+9F8D.0	fCantonese	lung4; lung2 (also); lung5
U+9F8D.0	fMandarin	long2; mang2; long3
U+9F8D.0	fEnglish	[1] dragon [2] of the emperor; imperial [3] a huge extinct reptile [4] a Chinese family name [5] Kangxi radical 212
U+9F8E.0	fUTF8	龎
U+9F8E.0	fCNS11643	3-5755
U+9F8E.0	fGB	5-3054
U+9F8E.0	fR/S	 212.2; 27.2
U+9F8E.0	fTotalStrokes	 18
U+9F8E.0	fCangjie	MYBP
U+9F8E.0	fFourCorner	71210
U+9F8E.0	fCantonese	pong4
U+9F8E.0	fMandarin	pang2
U+9F8E.0	fEnglish	 disorderly; messy huge; big
U+9F8F.0	fUTF8	龏
U+9F8F.0	fCNS11643	4-632D
U+9F8F.0	fGB	3-5877
U+9F8F.0	fR/S	  212.3
U+9F8F.0	fTotalStrokes	 19
U+9F8F.0	fCangjie	YPT
U+9F8F.0	fFourCorner	01440
U+9F8F.0	fCantonese	gung1
U+9F8F.0	fMandarin	gong1; gong3
U+9F90.0	fUTF8	龐
U+9F90.0	fCNS11643	1-7832
U+9F90.0	fBig5	C365
U+9F90.0	fGB	1-4553
U+9F90.0	fR/S	 212.3; 53.16
U+9F90.0	fTotalStrokes	 19
U+9F90.0	fCangjie	IYBP
U+9F90.0	fFourCorner	00211
U+9F90.0	fHakka	 pong2; lung2; mang2; pang2
U+9F90.0	fCantonese	pong4; lung4
U+9F90.0	fMandarin	pang2; long2; peng2
U+9F90.0	fEnglish	 disorderly; messy huge; big
U+9F91.0	fUTF8	龑
U+9F91.0	fCNS11643	2-6B4D
U+9F91.0	fBig5	F5BF
U+9F91.0	fGB	3-5879
U+9F91.0	fR/S	 212.4
U+9F91.0	fTotalStrokes	 20
U+9F91.0	fCangjie	YPMK
U+9F91.0	fFourCorner	08432
U+9F91.0	fCantonese	jim2 (also); jim5; jin2
U+9F91.0	fMandarin	yan3
U+9F91.0	fEnglish	high and bright
U+9F92.0	fUTF8	龒
U+9F92.0	fCNS11643	2-6D43
U+9F92.0	fBig5	F6D5
U+9F92.0	fGB	E-492F
U+9F92.0	fR/S	 212.5
U+9F92.0	fTotalStrokes	 21
U+9F92.0	fCangjie	YPMMF
U+9F92.0	fFourCorner	01903
U+9F92.0	fCantonese	lung4
U+9F92.0	fMandarin	long2; mang2
U+9F92.0	fEnglish	dragon
U+9F93.0	fUTF8	龓
U+9F93.0	fCNS11643	3-602B
U+9F93.0	fGB	3-587C
U+9F93.0	fR/S	  212.6
U+9F93.0	fTotalStrokes	 22
U+9F93.0	fCangjie	KBYBP
U+9F93.0	fFourCorner	41210
U+9F93.0	fHakka	 lung2
U+9F93.0	fCantonese	lung5
U+9F93.0	fMandarin	long2; you3
U+9F93.0	fEnglish	a halter
U+9F94.0	fUTF8	龔
U+9F94.0	fCNS11643	1-7C33
U+9F94.0	fBig5	C5C7
U+9F94.0	fGB	1-3928
U+9F94.0	fR/S	 212.6
U+9F94.0	fTotalStrokes	 22
U+9F94.0	fCangjie	YPTC
U+9F94.0	fFourCorner	01801
U+9F94.0	fHakka	 giung1; giung3
U+9F94.0	fCantonese	gung1
U+9F94.0	fMandarin	gong1
U+9F94.0	fEnglish	[1] reverential [2] a Chinese family name
U+9F95.0	fUTF8	龕
U+9F95.0	fCNS11643	2-6E7B
U+9F95.0	fBig5	F7CE
U+9F95.0	fGB	1-6D68
U+9F95.0	fR/S	 212.6
U+9F95.0	fTotalStrokes	 22
U+9F95.0	fCangjie	ORYBP
U+9F95.0	fFourCorner	80211
U+9F95.0	fHakka	  kam1
U+9F95.0	fCantonese	ham1
U+9F95.0	fMandarin	kan1; ke4
U+9F95.0	fEnglish	 niche for an idol
U+9F96.0	fUTF8	龖
U+9F96.0	fCNS11643	4-6E55
U+9F96.0	fGB	3-587A
U+9F96.0	fR/S	  212.16
U+9F96.0	fTotalStrokes	 32
U+9F96.0	fCangjie	YBYPP
U+9F96.0	fFourCorner	01210
U+9F96.0	fCantonese	daap6
U+9F96.0	fMandarin	ta4
U+9F96.0	fEnglish	flight of a dragon
U+9F97.0	fUTF8	龗
U+9F97.0	fCNS11643	3-6242
U+9F97.0	fGB	5-5870
U+9F97.0	fR/S	  212.17
U+9F97.0	fTotalStrokes	 33
U+9F97.0	fCangjie	MBRRP
U+9F97.0	fFourCorner	10210
U+9F97.0	fCantonese	ling4
U+9F97.0	fMandarin	ling2
U+9F98.0	fUTF8	龘
U+9F98.0	fCNS11643	2-7244
U+9F98.0	fBig5	F9D5
U+9F98.0	fGB	E-4930
U+9F98.0	fR/S	 212.32
U+9F98.0	fTotalStrokes	 48
U+9F98.0	fCangjie	YPYBP
U+9F98.0	fFourCorner	01211
U+9F98.0	fCantonese	daap6
U+9F98.0	fMandarin	ta4; mei2; mei4; mo3; mo4
U+9F98.0	fEnglish	the appearance of a dragon walking
U+9F99.0	fUTF8	龙
U+9F99.0	fCNS11643	F-2159
U+9F99.0	fGB	0-417A
U+9F99.0	fR/S	 212.0
U+9F99.0	fTotalStrokes	 5
U+9F99.0	fCangjie	IKP
U+9F99.0	fHakka	 liung2
U+9F99.0	fCantonese	lung4; lung2 (also); lung5
U+9F99.0	fMandarin	long2; mang2; long3
U+9F99.0	fEnglish	simplified form of &#x9F8D;
U+9F9A.0	fUTF8	龚
U+9F9A.0	fGB	0-3928
U+9F9A.0	fR/S	 212.6
U+9F9A.0	fTotalStrokes	 11
U+9F9A.0	fCangjie	IPTC
U+9F9A.0	fHakka	 giung1
U+9F9A.0	fCantonese	gung1
U+9F9A.0	fMandarin	gong1
U+9F9A.0	fEnglish	simplified form of &#x9F94;
U+9F9B.0	fUTF8	龛
U+9F9B.0	fGB	0-6D68
U+9F9B.0	fR/S	 212.6
U+9F9B.0	fTotalStrokes	 11
U+9F9B.0	fCangjie	ORIKP
U+9F9B.0	fHakka	 kam1
U+9F9B.0	fCantonese	ham1
U+9F9B.0	fMandarin	kan1
U+9F9B.0	fEnglish	simplified form of &#x9F95;
U+9F9C.0	fUTF8	龜
U+9F9C.0	fCNS11643	1-733F
U+9F9C.0	fBig5	C074
U+9F9C.0	fGB	1-396A
U+9F9C.0	fR/S	 213.0
U+9F9C.0	fTotalStrokes	 16
U+9F9C.0	fCangjie	NXU
U+9F9C.0	fFourCorner	27117
U+9F9C.0	fHakka	 gui1; gwi1; kwui1
U+9F9C.0	fCantonese	gwai1; gwan1; gau1 (also); kau1
U+9F9C.0	fMandarin	gui1; jiu1
U+9F9C.0	fEnglish	[1] turtle; tortoise [2] Kangxi radical 213
U+9F9C.1	fUTF8	龜
U+9F9C.1	fCNS11643	1-733F
U+9F9C.1	fBig5	C074
U+9F9C.1	fGB	1-396A
U+9F9C.1	fR/S	 213.0
U+9F9C.1	fTotalStrokes	 16
U+9F9C.1	fCangjie	NXU
U+9F9C.1	fFourCorner	27117
U+9F9C.1	fMandarin	jun1
U+9F9C.1	fEnglish	chapped; cracked
U+9F9C.2	fUTF8	龜
U+9F9C.2	fCNS11643	1-733F
U+9F9C.2	fBig5	C074
U+9F9C.2	fGB	1-396A
U+9F9C.2	fR/S	 213.0
U+9F9C.2	fTotalStrokes	 16
U+9F9C.2	fCangjie	NXU
U+9F9C.2	fFourCorner	27117
U+9F9C.2	fMandarin	qiu1
U+9F9C.2	fEnglish	[CC@] a country in Central Asia during the Han dynasty
U+9F9D.0	fUTF8	龝
U+9F9D.0	fCNS11643	3-5E47
U+9F9D.0	fGB	5-5B33
U+9F9D.0	fR/S	 115.16; 213.5; 115.17
U+9F9D.0	fTotalStrokes	 21
U+9F9D.0	fCangjie	HDNXU
U+9F9D.0	fFourCorner	27910
U+9F9D.0	fHakka	  ciu1
U+9F9D.0	fCantonese	cau1
U+9F9D.0	fMandarin	qiu1
U+9F9D.0	fEnglish	 autumn; fall year
U+9F9E.0	fUTF8	龞
U+9F9E.0	fCNS11643	4-6E33
U+9F9E.0	fGB	5-707B
U+9F9E.0	fR/S	 213.12
U+9F9E.0	fTotalStrokes	 28; 27
U+9F9E.0	fCangjie	FKNXU
U+9F9E.0	fFourCorner	98110
U+9F9E.0	fCantonese	bit3
U+9F9E.0	fMandarin	bie1
U+9F9E.0	fEnglish	turtle
U+9F9F.0	fUTF8	龟
U+9F9F.0	fGB	0-396A
U+9F9F.0	fR/S	 213.0
U+9F9F.0	fTotalStrokes	 7
U+9F9F.0	fCangjie	NWU
U+9F9F.0	fHakka	 kwui1
U+9F9F.0	fCantonese	gwai1; gwan1; gau1 (also); kau1
U+9F9F.0	fMandarin	gui1; jiu1; jun1; qiu1
U+9F9F.0	fEnglish	simplified form of &#x9F9C;
U+9FA0.0	fUTF8	龠
U+9FA0.0	fCNS11643	2-6142
U+9FA0.0	fBig5	EFB6
U+9FA0.0	fGB	0-595F
U+9FA0.0	fR/S	 214.0
U+9FA0.0	fTotalStrokes	 17
U+9FA0.0	fCangjie	OMRB
U+9FA0.0	fFourCorner	80227
U+9FA0.0	fHakka	  jok7; rok7
U+9FA0.0	fCantonese	joek6
U+9FA0.0	fMandarin	yue4
U+9FA0.0	fEnglish	[1] a kind of flute [2] a kind of ancient measuring vessel [3] Kangxi radical
U+9FA1.0	fUTF8	龡
U+9FA1.0	fCNS11643	3-5F3F
U+9FA1.0	fGB	5-7A6F
U+9FA1.0	fR/S	  214.4
U+9FA1.0	fTotalStrokes	 21
U+9FA1.0	fCangjie	OBNO
U+9FA1.0	fFourCorner	87280
U+9FA1.0	fHakka	 ch'oi1; coi1
U+9FA1.0	fCantonese	ceoi1; ceoi3
U+9FA1.0	fMandarin	chui1; chui4
U+9FA1.0	fEnglish	 blow; exhale
U+9FA2.0	fUTF8	龢
U+9FA2.0	fCNS11643	2-6E7C
U+9FA2.0	fBig5	F7CF
U+9FA2.0	fGB	E-4931
U+9FA2.0	fR/S	 214.5
U+9FA2.0	fTotalStrokes	 22
U+9FA2.0	fCangjie	OBHD
U+9FA2.0	fFourCorner	82294
U+9FA2.0	fHakka	  fo2; fo5
U+9FA2.0	fCantonese	wo4
U+9FA2.0	fMandarin	he2; he4; huo4; hu2; huo2
U+9FA2.0	fEnglish	[1] harmonious; calm; peaceful ??? [2] call for; make an appeal for ???
U+9FA3.0	fUTF8	龣
U+9FA3.0	fCNS11643	3-6161
U+9FA3.0	fGB	5-7A73
U+9FA3.0	fR/S	  214.8
U+9FA3.0	fTotalStrokes	 25
U+9FA3.0	fCangjie	OBVNE
U+9FA3.0	fFourCorner	87230
U+9FA3.0	fCantonese	gok3
U+9FA3.0	fMandarin	jue2; lu4
U+9FA4.0	fUTF8	龤
U+9FA4.0	fCNS11643	2-716F
U+9FA4.0	fBig5	F9A1
U+9FA4.0	fGB	E-4932
U+9FA4.0	fR/S	 214.9
U+9FA4.0	fTotalStrokes	 26
U+9FA4.0	fCangjie	OBPPA
U+9FA4.0	fFourCorner	81261
U+9FA4.0	fHakka	 hai2
U+9FA4.0	fCantonese	haai4
U+9FA4.0	fMandarin	xie2
U+9FA4.0	fEnglish	 harmonize accord with agree
U+9FA5.0	fUTF8	龥
U+9FA5.0	fCNS11643	3-6177
U+9FA5.0	fGB	5-7A70
U+9FA5.0	fR/S	 214.9
U+9FA5.0	fTotalStrokes	 26
U+9FA5.0	fCangjie	OBMBC
U+9FA5.0	fFourCorner	81280
U+9FA5.0	fCantonese	jyu6; joek6 (also)
U+9FA5.0	fMandarin	yu4
U+9FA5.0	fEnglish	beg, appeal for help
